/*****************************************************************************/
/*                                                                           */
/* Author: Michael C. Nechyba                                                */
/* Last modified: 02/15/2004                                                 */
/*                                                                           */
/* File: "ppm_util.c"                                                        */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "file_io.h"
#include "matrix.h"
#include "gen_math.h"
#include "ppm_util.h"

/****************************** Subroutines **********************************/

/*****************************************************************************/

int read_ppm_file(char *file, PPM *image)
{
  int i, j, k, comments;
  FILE *fp;
  char c1, c2, str[STR_REG], *convert[STR_REG];
  
  fp = fopen(file, "r");
  
  if(fp == NULL) {
    fprintf(stderr, "Could not read %s file.\n", file);
    exit(-1);
  }
  
  fscanf(fp, "%c", &c1);
  fscanf(fp, "%c", &c2);

  if(c1 != 'P' && c2 != '6') {
    fprintf(stderr, "Image file is not in the correct PPM format.\n");
    exit(-1);
  }

  fscanf(fp, "%c", &c1);

  comments = 0;
  while(!comments) {
    fscanf(fp, "%c", &str[0]);
    if(str[0] == '#') {
      fscanf(fp, "%[^\n]\n", &str[1]);
    } else {
      comments = 1;
    }
  }
  
  fscanf(fp, "%[^\n]\n", &str[1]);

  image->xdim = (int) strtol(&str[0], &convert[0], 10);
  image->ydim = (int) strtol(convert[0], (char **) NULL, 10);

  /* read '255' */
  fscanf(fp, "%c", &c1);
  fscanf(fp, "%c", &c1);
  fscanf(fp, "%c", &c1);
  fscanf(fp, "%c", &c1);

  for(i = 0; i < image->ydim; i++) {
    for(j = 0; j < image->xdim; j++) {
      for(k = 0; k < 3; k++) {				
	fread(&image->p[j][i][k], sizeof(unsigned char), 1, fp);
      }
    }
  }
  
  fclose(fp);

  return(1);	
}

/*****************************************************************************/

int write_ppm_file(char *file, PPM *image)
{
  int i, j, k;
  FILE *fp;
  
  fp = fopen(file, "w");
  if(fp == NULL) {
    fprintf(stderr, "Could not open %s for writing.\n", file);
    exit(1);
  }		
  
  fprintf(fp, "P6\n");
  fprintf(fp, "%d %d\n", image->xdim, image->ydim);
  fprintf(fp, "255\n");
  
  
  for(i = 0; i < image->ydim; i++) {
    for(j = 0; j < image->xdim; j++) {
      for(k = 0; k < 3; k++) {				
	fwrite(&image->p[j][i][k], sizeof(char), 1, fp);
      }
    }
  }
  
  fclose(fp);
  
  return(1);	
}

/*****************************************************************************/

double convert_one_pixel(unsigned char *pixel)
{
  return(((double) pixel[0] + (double) pixel[1] + (double) pixel[2])/3.);
}

/*****************************************************************************/

/* 
   Note: For compatibility with Mathematica routines in
   ipcv_definitions.nb, the image rows in the converted real (double)
   array are indexed from (ydim - 1) to 0, instead of 0 to (ydim - 1).
   To change to normal indexing, uncomment the line,

        p = &x[i][0];

   in the function below.
*/

double **convert_ppm_to_double(double **x, PPM *image, int alloc)
{
  int i, j;
  double *p;

  if(alloc <= 0)
    x = allocate_matrix(x, image->ydim, image->xdim);

  for(i = 0; i < image->ydim; i++) {
    p = &x[image->ydim - 1 - i][0];
    /* p = &x[i][0]; */ 
    for(j = 0; j < image->xdim; j++) {
      *p++ = convert_one_pixel(&image->p[j][i][0]);
    }
  }

  return(x);
}

/*****************************************************************************/

/*****************************************************************************/

/* 
   Note: For compatibility with Mathematica routines in
   ipcv_definitions.nb, the image rows in the converted real (double)
   array are indexed from (ydim - 1) to 0, instead of 0 to (ydim - 1).
   To change to normal indexing, uncomment the line,

        p = &x[i][0];

   in the function below.
*/

int convert_double_to_ppm(PPM *image, double **x)
{
  int i, j;
  double *p;
  unsigned char v;

  for(i = 0; i < image->ydim; i++) {
    p = &x[image->ydim - 1 - i][0];
    /* p = &x[i][0]; */ 
    for(j = 0; j < image->xdim; j++) {
      v = (unsigned char) limit_real(*p + 0.5, 0., 255.);

      image->p[j][i][0] = v;
      image->p[j][i][1] = v;
      image->p[j][i][2] = v;
      p++;
    }
  }

  return(1);
}

/*****************************************************************************/

