(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[   5997662,     101435]*)
(*NotebookOutlinePosition[   5998469,     101465]*)
(*  CellTagsIndexPosition[   5998395,     101459]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["DWT-based texture segmentation: example #2", "Title"],

Cell[TextData[{
  StyleBox["Note",
    FontWeight->"Bold",
    FontSlant->"Italic",
    FontColor->RGBColor[0, 0, 1]],
  ": This notebook works in conjunction with the external perl script \"",
  StyleBox["texture.pl",
    FontFamily->"Courier"],
  "\" to be executed as follows:\n.",
  StyleBox["./../src/texture.pl parameters",
    FontFamily->"Courier"],
  "\nFurthermore, this notebook assumes that all definitions in ",
  StyleBox["ipcv_definitions.nb (ver. 1.2 or later) ",
    FontWeight->"Bold",
    FontSlant->"Italic",
    FontColor->RGBColor[1, 0, 0]],
  "have been evaluated first, and that the directory and parameter file name \
have been set in the ",
  ButtonBox["Set directory and define external parameter file",
    ButtonData:>"dir",
    ButtonStyle->"Hyperlink"],
  " "
}], "Text",
  FontSize->14],

Cell[CellGroupData[{

Cell["Local definitions", "Section"],

Cell[CellGroupData[{

Cell["Parameter file reading helper function", "Subsection"],

Cell[BoxData[
    \(\(LineGet[parameters_, token_] := 
        Drop[parameters[\([First[Flatten[Position[parameters, \ token]]]]\)], 
          1];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Visualize centroids", "Subsection"],

Cell[BoxData[
    \(\(VisualizeCentroids[means_, \ 
          opts___] := \[IndentingNewLine]Module[{n, m, 
            g1}, \[IndentingNewLine]n = 
            Sqrt[Length[First[means]]]; \[IndentingNewLine]m = 
            Min[Length[means], 16]; \[IndentingNewLine]g1 = 
            Table[ShowImage[Partition[means[\([i]\)], n], 1, 
                Max[means[\([i]\)]], Min[means[\([i]\)]], 
                DisplayFunction \[Rule] Identity], {i, 1, 
                Length[means]}]; \[IndentingNewLine]Show[
            GraphicsArray[Partition[g1, m]], 
            DisplayFunction \[Rule] $DisplayFunction, 
            opts]\[IndentingNewLine]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Histogram computation", "Subsection"],

Cell[BoxData[
    \(\(ComputeHistograms[cls_, \ 
          nCodes_:  
            4] := \[IndentingNewLine]\(N[
              CategoryCounts[#, Table[i, {i, 1, 2^nCodes}]]/
                Length[#]] &\)\  /@ \ cls;\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Classification functions", "Subsection"],

Cell[BoxData[
    \(\(\(ProbabilitiesOneTestVectorOneHistogram[v_, 
          hist_] := \[IndentingNewLine]Table[\(\((#[\([v[\([i]\)]]\)])\) &\)\ \
 /@ \ hist, {i, 1, Length[v]}];\)\(\ \)\)\)], "Input"],

Cell[BoxData[
    \(\(ProbabilitiesOneTestVectorAllHistograms[v_, 
          hist_] := \[IndentingNewLine]Table[
          ProbabilitiesOneTestVectorOneHistogram[v[\([i]\)], 
            hist[\([i]\)]], {i, 1, \ Length[v]}];\)\)], "Input"],

Cell[BoxData[
    \(\(AnalyzeOneTestVector[v_, \ hist_, \ 
          part_:  77] := \[IndentingNewLine]Map[Partition[#, part] &, 
          ProbabilitiesOneTestVectorAllHistograms[v, 
            hist], {1}];\)\)], "Input"],

Cell[BoxData[
    \(\(ClassifyOnProbabilities[probs_] := \[IndentingNewLine]Map[
          First[First[Position[#, Max[#]]]] &, probs, {2}];\)\)], "Input"],

Cell[BoxData[
    \(\(VisualizeClassification[cls_, \ colors_, \ 
          opts___] := \[IndentingNewLine]Show[
          Graphics[RasterArray[Map[colors[\([#]\)] &, \ cls, \ {2}]]], \ 
          AspectRatio \[Rule] Automatic, opts];\)\)], "Input"],

Cell[BoxData[
    \(\(ClassificationPercentages[cls_, 
          n_] := \[IndentingNewLine]Module[{pct}, \[IndentingNewLine]pct = 
            CategoryCounts[cls // Flatten, Range[n]]; \[IndentingNewLine]100 
               pct/\((Plus\  @@ \ pct)\)\  // \ 
            N\[IndentingNewLine]];\)\)], "Input"],

Cell[BoxData[
    \(\(VisualizeResults[probs_, \ colors_, 
          opts___] := \[IndentingNewLine]Module[{cls, g1, pct, g2, g2b, 
            pr}, \[IndentingNewLine]cls = 
            ClassifyOnProbabilities[probs]; \[IndentingNewLine]g1 = 
            VisualizeClassification[cls, colors, \ 
              DisplayFunction \[Rule] Identity]; \[IndentingNewLine]pct = 
            ClassificationPercentages[cls, 
              Length[\(probs // First\) // First]]; \[IndentingNewLine]\ 
          g2 = BarChart[pct, BarStyle \[Rule] colors, 
              BarEdgeStyle \[Rule] colors, Frame \[Rule] True, \ 
              DisplayFunction \[Rule] Identity]; \[IndentingNewLine]pr = 
            FullOptions[g2, PlotRange]; \[IndentingNewLine]g2b = 
            Show[g2, PlotRange \[Rule] {pr // First, {\(-2\), 102}}, \ 
              GridLines \[Rule] {None, Automatic}, \ 
              FrameTicks \[Rule] None, \ 
              Axes \[Rule] False]; \[IndentingNewLine]{pct, 
            Show[GraphicsArray[{{g1}, {g2b}}], opts, 
              DisplayFunction \[Rule] $DisplayFunction]}\[IndentingNewLine]];\
\)\)], "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Set directory and define external parameter file", "Section",
  CellTags->"dir"],

Cell[BoxData[{
    \(\(SetDirectory["\</Users/nechyba/src/texture/demos/set2\>"];\)\), "\
\[IndentingNewLine]", 
    \(\($parameterFile\  = \ "\<parameters\>";\)\)}], "Input",
  Background->RGBColor[1, 1, 0]]
}, Closed]],

Cell[CellGroupData[{

Cell["Read parameter file and set up global variables", "Section"],

Cell[CellGroupData[{

Cell["Read parameter file", "Subsection"],

Cell[BoxData[
    \(\(parameters = Import[$parameterFile, "\<Table\>"];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Extract values from parameter file", "Subsection"],

Cell[BoxData[{
    \(\(trainImageNames\  = \ \(StringDrop[#, \(-4\)] &\)\  /@ \ 
          LineGet[parameters, "\<TrainingFiles\>"];\)\), \
"\[IndentingNewLine]", 
    \(\(testImageNames\  = \ \(StringDrop[#, \(-4\)] &\)\  /@ \ 
          LineGet[parameters, \ "\<TestFiles\>"];\)\), "\[IndentingNewLine]", \

    \(\(dwtlevels\  = \ 
        LineGet[parameters, \ "\<DWTLevels\>"]\  // \ 
          First;\)\), "\[IndentingNewLine]", 
    \(\(windowSize\  = \ 
        LineGet[parameters, \ "\<WindowSize\>"]\  // \ 
          First;\)\), "\[IndentingNewLine]", 
    \(\(windowOffset\  = \ 
        LineGet[parameters, "\<WindowOffset\>"] // 
          First;\)\), "\[IndentingNewLine]", 
    \(\(keepLLband\  = \ 
        LineGet[parameters, \ "\<KeepLLBand\>"]\  // \ 
          First;\)\), "\[IndentingNewLine]", 
    \(\(vqlevels\  = \ 
        LineGet[parameters, \ "\<VQLevels\>"]\  // \ First;\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Derive additional file names and constants", "Subsection"],

Cell[CellGroupData[{

Cell["Read training and test images", "Subsubsection"],

Cell[BoxData[{
    \(\(trainImages\  = \ \(ReadImage[# <> "\<.ppm\>"] &\)\  /@ \ 
          trainImageNames;\)\), "\[IndentingNewLine]", 
    \(\(testImages\  = \ \(ReadImage[# <> "\<.ppm\>"] &\)\  /@ \ 
          testImageNames;\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Derive additional constants", "Subsubsection"],

Cell[BoxData[{
    \(\(nImages\  = \ Length[testImageNames];\)\), "\[IndentingNewLine]", 
    \(\(nBands\  = \ 
        3*dwtlevels\  + \ keepLLband;\)\), "\[IndentingNewLine]", 
    \(\(rowLength\  = \ \((Last[Dimensions[testImages]] - windowSize)\)/
            windowOffset\  + \ 1;\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Derive various file names", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(histFiles\  = \ 
        Table[\(\((# <> "\<.0\>" <> 
                  ToString[i] <> "\<.labels\>")\) &\)\  /@ \ 
            trainImageNames, \ {i, \ 1, \ 
            nBands}];\)\), "\[IndentingNewLine]", 
    \(\(centroidFiles\  = \ 
        Table["\<band.0\>" <> ToString[i] <> "\<.codes.\>" <> 
            ToString[vqlevels], {i, 1, nBands}];\)\), "\[IndentingNewLine]", 
    \(testFiles\  = \ 
      Table[\(\((# <> "\<.0\>" <> ToString[i] <> "\<.labels\>")\) &\)\  /@ \ 
          testImageNames, \ {i, \ 1, \ nBands}]\)}], "Input"],

Cell[BoxData[
    \({{"texture5_test.01.labels", "texture6_test.01.labels", 
        "texture7_test.01.labels", 
        "texture8_test.01.labels"}, {"texture5_test.02.labels", 
        "texture6_test.02.labels", "texture7_test.02.labels", 
        "texture8_test.02.labels"}, {"texture5_test.03.labels", 
        "texture6_test.03.labels", "texture7_test.03.labels", 
        "texture8_test.03.labels"}, {"texture5_test.04.labels", 
        "texture6_test.04.labels", "texture7_test.04.labels", 
        "texture8_test.04.labels"}, {"texture5_test.05.labels", 
        "texture6_test.05.labels", "texture7_test.05.labels", 
        "texture8_test.05.labels"}, {"texture5_test.06.labels", 
        "texture6_test.06.labels", "texture7_test.06.labels", 
        "texture8_test.06.labels"}}\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Analysis and visualization of texture segmentation", "Section"],

Cell[CellGroupData[{

Cell["Visualize training images", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(gtmp = \(ShowImage[#, 1, 255, 0, 
              DisplayFunction \[Rule] Identity] &\)\  /@ \ 
          trainImages;\)\), "\[IndentingNewLine]", 
    \(\(gtrain = 
        Show[GraphicsArray[Partition[gtmp, 2]], 
          DisplayFunction \[Rule] $DisplayFunction, ImageSize \[Rule] 600, 
          GraphicsSpacing \[Rule] 0.05];\)\)}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .5 
%%ImageSize: 600 300 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 0.0119048 0.47619 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.0119048 0.488386 0.244193 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 0.003125 0 0.003125 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 .5 scale
320 string
320 160 8 [320 0 0 160 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
63A24E543D824845984E526B8D5344723835292D5C406D5725624D2153745735432F2F5E
3E495142385C6729775248655B5A206D713A5C482750644B81677E402F88592970574357
6D4253885A54516949797239B07B349A4C41929E6323BF62368E79603E95422489974F4D
53A443259483548E5B3D6F5077553F76774467865CB371756A4A7D8C5864764A628256B2
3A627C41763F54654760C7322065803C84821C50996F3E87A7393A6178478446687A8758
7F865992ADA581A6C57D5C7FAB3C4C6FA05488C06B29779A3F5E49568E45569B4D7FA159
74A74150C28834874A5084895357CE8469C1A66148AB90AE554D8B785558362C5B916840
5B7741216B6E7C445A6B5149A17F3D4C57393D724D95B97539676C78223B635734899C50
3B55964C2E6487783C855E3C55733FAE7D42987B51BBBC684C4E638B38256698
609A4F4A4485434F852D4F7B8750476439424D468B4C696E2D764E30687D6446442F337A
4B4E492C4577702B85503D4E4A462D826E3F513D1B537145956F3A2B4CA46B2C65353A84
86455499545281784B88781A99533FAD53246C9D6821C02E379B986F35C23C187244132E
3E7449207836317260364C37825E29574C4E876F1F955C7D7D334A5A558EA052455456A3
5F55483463705F812D65B245386F703CA38F3B579A57224F966645695C3D7B5E3A5B9158
A5885878995E4B89A1567673984C5B86994C82C6844688A846553E328247598433768557
8C6A366FBD673484645B5F845B61CF8B5274848854AC90A97A47788B7460704576AC7D4A
588E80306F85844E5F6A6E5AB589405F54423F7A4B8AC3775877787D213B654C37989139
3965964425476F713CA58334525939A486479276337C8C4343595A80362D85A5
5688514A374D2C536D30365D674A553B174F494C552C8C772D651B2F5E87765252292662
4B65451418456523861736324607206164386D731C33563B9B9C401C2F78474F7F3C1F38
553879B65A2B5B4C387F813BAD55257C5B2F74883B4AC2522775863434A64B1C804C242A
3779411D71442B5E284A512E776542604E31598A5E895355724C556B536D766B49625599
5B596A505A75717F516AC44231706F3C9C8D3D558D7B3050A26B4266824380795E704A44
91987390958B6B8996466C729E575990854698D693518AAE5074613A70455C7B39828B63
A8643C9DAB3C2D9863686457316ECC9372A1886142C4A58A944E7C956E557B4973A68848
5F8C7B2676967D4F7B706F4C9B89495B6452427B6386C2705973677E213F79553492933F
31A6A2672345742F2D948B30486C3C9C7C4992705E71A94A49946B6B1F379799
665F525541703B467631647E3F327A732F32406055135A6236872A2D5A51415E75553551
2856614136683E19664B2F1B4F34237338253B49292E462F555C653B3C8525295E523955
5E2F4E8B5C4C6F6E4B7286559179339C4D3B8B7D591D8E45278F73312FA51F15869F292E
337052216F4F33703E4955447B6947626F4868936B8B6B74773F4A7D5A797C6F416F57A2
68576E5A607767A16465C24A39789259AF8C3C598985334E996C46768A477E6F65717A50
768C6E91998D6C8DAE6B7B99BF785B8890426BC78454949D478557387B5BA29847837433
A081569CB150296F49545A4F3A5AB67868A5754941D0C98A6B447B8B6C6B544471996A55
A5A46B1E68722E25918A6C3C738A5675966E4E726A8AD49E5B7E646F2570A85D268A8436
347C99542E556C81359E80304D7739984C30887955788346266A938534268896
6A994C3A407D40496D2829727D473A5E32291E456B3D522F256C3D396B5F4441593A3369
434D564046766A28816D3D32402A2672764151583135624B795D6B4169BD5A3861443C55
694770A35E4C668643918A53737935894461B4915E27BD51278F884D39AA2D1666743F31
306A3E40934B306D3A49664D92774F6D6E4B66926E846D727852647156756C704A6E518C
666B83538D78577A5650B85237748F5AA18B365F927530498E3A68B39E49745E6C748A5D
8691869D9A926A8ABE5B6B8FAB664F849F638CD37955A9A0405B5533614C758F4F785143
A46B517E965F348355486E604B6ECE7661A9764A2F86A3987C44755A597F523E628E3631
64724A1E6E49664A5C635A345E653482A5614464546EBA8147777478293C613725846628
2F69762825627B663CA05F225357317A7338989252C46C4930516B794A39A592
619D4E5046934348926C315D6A44404E2F494A498A4658553174363E75795E545F39437D
454C4814459676358F5F3B35412A2E786240505D34447A3F7B5F4C272D72563E60253396
8B427BAF604C86873B91843C4E863280463481836328C05734AC9E6D3CA93E2376583A28
2F644326695F398535456548884C577A644363824675595E665482794A6E6F6066744D8E
58597E537F796B8D554FC0563D938051A98A3F8FC1823245995341708D417A416685885A
827D68B9C7A36A7AAE7B727EB6656CA8B3588ED552318DB5497E5843724D6A864D806657
A56D2E60915C358A574287885979CD8460B3875F41A896A46E4073515669543F64956E41
6E8F4B2C9A586F4F707E564066773849613549656297C6534D97A78C366088572993601F
365972232B4C714C5DAC7424466D37836D34887459CD8F3C334F65772D358291
5E974D5B477D3D579B6D2F7690463D331439365C632E8A593E801E53927A60636E4F4170
434B643D33706D3090203A2B43194B935F394E4112386A427D888544459B523B5C35243F
5B286FB450578A883287805E397C2C6D4E45AAA05028AE3E3F9598605DBB54307C522D3C
55776C3A7B6638832C4C5541A06C546B683D5985597E5C4E6750607740587E6E5561478D
5085774A8B6E5E8A3C47A04359A37D5094992F6AA47C385AA66A497A90447F6D5C657A56
7C71557A868A627FAD79818C91304B7F89468BB272589897618C373A8E4E6564346C765F
909C5480B55338955B3D48575288D09157A2B6845CBFA8B983468A666E8B1F284D945A48
808B31207052283490893834627B3F7A7F173A66707FD3703A605A4E25486C672D8E5D1F
37547C392F54625D376860214967318B6341A96964D091313E6A9175282B8E9B
6A684F534571324178582966383F6A7C3A3941605C2462654D791C2B676C52808F584C6D
2D5D7C41439B4B21773733244F1921683C376761353C583987A48446498E3A588E524E55
622953A96B627F7B3FB69D6C323D236F7A60B8B4122896622A929E1E2B815B2D83A86324
27616359A567327560687229886D687233316A895F6E445D8357706E446FA3725F50328E
698451266C935FB63226A9733F8B642D919A467DB853253670445B9D802F735C61736551
8F8D718C7C65548CC29194B2B0644A81924994C3686A8D923D63522356456975414C5C46
836C619EB6743C894F42594E396DA8834E807F785DCCCEC26E32777865885A3D60A06A4C
7A78412F7B475D4D647B693750662E6876474F586A50C7604B806A6C2E496764388D6634
3A4F87432F5E687240755C2E83862E7A3325727365CE6B17172D4B6C2D27887C
6F744B52477940436F402F697031264C2C2C1B475B425C2846672C3787393247635E5F77
2A4A50274E955D2681533F1E4D35297F562F44563B3447274960673D559C2C3057355E86
60215194697C8E4F2F6D824C426D22625271CBAC4D1C985028928D3E39A15B368873542E
356E783E71773B71533F80628A647C866434587A5A732838624A7C6D23426C5665783376
5065774E487E5F84543E834D4194734376904992B07D37388256446B7F3D5E4B808D4344
9F9C7586C5663F73B1506A92A15B68A7844B99BB587EABA6436D4C204B5487713C475E53
6E564E74AB6F398A5E426F6F5590BB6A50A37E5534948F9C8A3C6379687F543C54875E41
6493713E945C6F475E75754B6884496B6673648C9486C7955388B893354C642A367D6742
3146802A478F95633E6C4E1D576C348865319B7478D3604331567272372A757C
6A9A5144438846459E7C25646C352C5F3B4846597943694A4A78293C8C6E5A566135466C
435457322C5F4A2D98484B3D4B32317A52354F503B485035646A5F33468A51386845344A
5E2F217E566C94853D9A76573A6F295F544CB1966D23C15C68BAA15E42A266297E73324C
678A873A7D58448B5E3A6C4BA172695F6543827752725B4D614A577E3F3F666D61754A7A
56748157506663904F4C623C5AB9AF8572A94078B37E3E579F5D487593457D5E52657351
81705982957160A0B37681998B2A3F7C763F92AB45555C894E7F404B733F585B3A704D4D
69772B599B7136873E4C867D4B8FB87268CDC06A4CACA4AD873A6F767083673258906E4C
6185272E96826F445A85283A7276446D5C275F91937EC4A8528A98962F48638353B47247
2D38974127588066526D5E27614F1F6B6635A56F49B68060396A7A7F322C8096
6091505B476B4A69B1842E74884A4D5A28403B5C5D3E72455670283F664F4F6D693D6380
5154703A3A5F4E329C3F3A34411E3E7D4C3956382A464A2E706B6B3F4D813D386A50564C
673D24676F72867F389F7B61484338745A4EAFA6283BC7723EA08E3138A56A357F873325
2F615F276A3F20574E3B5F2377676D6737366A7E586831617E5366673E6A8C6866644897
637C5B38757152962D558B4037928A5F68854A84B54E3956795A446964468D675E6B5A66
B8955D9B9D383E769A587EA292564890604FA0B35C8E606864805421553F60512D53686B
534C357CA24E50A2494859413F9BAD544665898062D7DBBD8243908670943327568B6359
8E8E5C28855F344490747841647C486471596D5D8775B5A03C7A8B6E2F72AF8649A18827
3F83A6562F5B7A3B67A9772F5F5926705350C2815ED19557467F7B6B3149ABA0
636F515648743A3978602C662F2F475929322C605C33552E657030418033366C7041645B
335B6C3436631B247D44352D5A332C7C2C3D7B563549312B6F836C444B7B29578E5A6141
4F325AA07A716D4E42897A4D4B28154C5A4AC47C27177651349D9E1A208166367A87383B
407A64276545296E3E3C6D48845E5B5A6F45768D627152467D4D61853C416B65625E4779
507D7E549282588D4A4366503D8B762B548E428CA777485F8D5D3B62853F5C375D684C45
77885DA2967077989C5B8CC1AA634FA1403979A65B9569963F624F2D6B499A7C3E5B4B28
4F866296AA502B6E594B5C392379A45D5FA9824C216B818B5B2A66486EB46B2D5D83322B
475B3C2B64384C303B50533E71543D979B6E85687675D4B86B8C95651F4076925261692D
222F71562A5C863F3E56482B8C7531784C36965F78D68027409EBC6F281E5383
626A494F43954C43A78A29778F3E2F612B3334385D56532347612F3B7264434F5C2F5F6D
354955353A5045317B2D44314B2D2B7B433C4A342E4138265F5365414E8C3B2D68547344
4C262560515D636A348A905B57422063453CA8654324B14A47A7912C2D9D5D2F728B4343
448571326E5F356E46426A51935B4E66824E88936B736852864853864A4E6F6A5C5C4F91
4A777F4F6361608F4F5D663F348A7539357F499DAD6A4C6C9C513D7A9046603866677A66
818368B29F8897A8A53D53848A5671B66047A79C407E8F8E4965642F6027527B3C655E4B
395F4B97956D39925A4C68644A6B895F55B3754236A0A26B4E2C6B3E445E563E698A4E31
53522649934D6D37485A5A578C5B2745524677643C4179875B96AC7D2F4A86714E5C6D3D
293D894C2C7DA05F475F41227E4C2E8E6741AA42339B9256375E5B6A3B276A92
699A5157439C4D49A58B2E79904030682739445A754D644670773B4179775367643B5F7F
495D5B354B5E503B8F3D413D512F2D754F3E5B382E332B327152673852964E41765E7A44
5D4E317D554E697F419F86597D67336C413EB3705221A63D4D9C844B3DA4713F80AB3C49
4F7F49318A753C7249648F548C635E7987517B906E8063687A4C769E4F50775173715690
45948F506565477D507465324A9C6E3F347B2F558C6E42648F325A9C99447828405C6A64
8F794D97A07980BAB27469A1873B5797734DB98B315F557542724F52793B6E7138845F4E
3F69276B915A338A485F91603F70A2705CD4CE6446AFB89E5F45946A6172403A7F906537
516727265D55643E42692B468B6B395957378A7A5768899156B2D395304C843D3447492D
2739772B1E5B7B4755AB8C2D7D47174E63409F453FB2684E3E87816A254E98A0
689C4E4F4D84484774242F767E382C6A273A4B686E3B6D43777333567F6F557462356573
45624522465C543EAB224C3C482340764D41482F25524235675044336AAF46456F356582
784E2E7D49426C7740B5682B4D4F23747D52BC774E24B55B3783763744C8701D586C4D50
81925E2D6B4A38803A39634DAB4649675661B47C3F654E938F35455F407B9E4748504D9A
50625127897C4D772F5FA0443072603C597D3E6A9C5A477FA65C33686F4B845E515D6F5E
B0735EB2873139ACB97763A4A56866955451C08A3F8169874470603880486C5C2E887455
41562E797E5525703F4755382D70A3653E9B918959BFA8B478265E6663815A3569976E3B
4C5E492973736D333C7E5E44856A3F604F255E757561A59E348C7D802E48896E546B3D27
3D75A570255669113A575426754D25686742A96041BA603B3C7A74621A23427C
5E835755406E4C64A96723625C3A3C452C5442563A2B67396F3E1B41546C4F7353244D56
43733F24475D4A369B3841395827235B483B5433232F372B60563E273E6E39456F40436D
5C43347E4747574432926D3C5D372B856C40968826449550369A861D1D7B5E2E5D7B204F
6C8068307A40225E34364F3D875D4C60533965795550385972494D7038496258654B3F77
3C65734A8E5A5262426B7A41397C652835763979905B36487A57337B54286C615A6B425D
AD9266BB846E69A9A86D66AAB16E6A966668D2A44E8476696B89632F5A3E6C783C7E7A4E
4A6254928E6235964E5B5B41386B97594AB77E6941C6A06B583467425D79644174885D48
6D654B2E8790645A5F825C468E6C3F706D395959537FD2AC5E95626A217CCA88636E4419
2B41855E2251534E4058512373582C7A3C39896C3FAE691B42A9AB5C31365087
656D4A5441734145896122603A2B545D24343566481D5930A160273B64494486703E5A41
2855532E4A612F2B7C3C2E2F603134672C48705234332D297E7A632A4A8C36678D5C3D91
53273A8565494C4B459C592B584F217460327C5B3C22A1382F969F3226A5672F679C3B3D
5E7771347354326A2B3753458A54495F754B7C81576460738045528151536B5A5B59507B
497C8E4E7A71688D58737044387A6E404980459DAD804151883C53B99B475C2C69747E63
B28C77C4A68B7F88AC8098CDB46B6C954A5AB57742A0A3843D5752286352808C4A826747
596D4D8F9F5961C061516041498FB56F53C3876643C6BB7F58345E45669F614376761D33
50544B4DA08E4A314876553B903E3571773F68442754888A69916848134D854E507F5123
3444694520545C5C4B5459328F62287052348C6C3CB16B4A3259687935395181
67AB4D63439855599A6D2477832C2551262A214F583E631F6D5F2B486A40397A65355463
3F5852355564323F954742435B2F375C233B41492F2E522F5E484C2F4C852D356B5931A2
572A287647385E6745A0622C74692A754D297D5E432CA74662B8995B32AE6C2D658F453E
638D472F8A703C772F49623E8945658373596D734F8B626E714C768D4B506D54747E558E
3D5B714D3D453B705E797B4649A67B436071385C93743B508F51315E7440742D7982756A
B67065BBA6857EA0905056B5AE6787A3636ECF83356BA0824371563C764483914A80713B
75724680A75B4AA65162A5422D599D8851BF767F2E94BFB27F3B6729387A7F47A39C6341
575F26237E7C652F3C77454E976844626E4F6A4D566D96835B84798336527B21367B7854
3F467036348F98585A52471F6B4E2B835B3181492EB4A6743E736560386F7C88
6DAB4D4743744D49733C3E8D822E2C6E29433A573433732F913A2D646E66529B63387582
5958482761805C45A51846424F1E51893A41403A305463436A514A3C6FA74954802A3188
70332C8445416F7B3C8E55204967317F593B46645231A9352B6A735728A56F4051874141
7C8A793B745E328B3F3569488D4E395F654C79513D6C57897439507D424E6E51525A5688
557478506F5452764C86AC402F7D8447638D4471BC84414E935D3F788941794E5A666369
BE7866C59F8E89B27B676DCE8D4F8BA85C98D97941648D564C956031693B795F45C19250
897619629F5A689742466A583F71B8825BCBB36F3887757E763D7C576881413992925C3D
623D412F83765330457A2D2F81704C6B75499A827874A4977CA298793177AA394D5B554B
525586391D4D5754689C6E317241266F51296E5021878F61347063521A3A5979
627B506340714662A25626666A25283E2E54334D322D582097612842544B56A876245A70
4B6C462C4C713F3AA048483B64252A564447565C36385838776231224686404E8A684880
5542408E5D4B5E55409574395551387D6D627F6D402AAD6829907B3721B87E4663901C49
6C7F7C3C9341278D643C5B329F634A502440868C4C5A387D8E4F674930638C636D5A649F
6473703E9265587F41538C6141908B50879A4684B8804B5B9B5668AC7142806377763252
B09C7ACB9A3A89CFB0698DD881417D843077C56C4B769D4F4F8371244A368F7B3D756638
9E825087AC3551A16052651D2168B96B459F817249BCC9B762366A6978A1673D8B813349
7F6E593777763653858C332F696B4D6E72387061435ABAB9B07D6331195881735F5E4F20
5189A46E264851174B68692F67622887542E89642994933C286A63612F4C4A71
6A7C4B613F69374E8D5E27776133496E2B4635616039420E8A5F2F4E5C1A2F638F395639
366459305A7E16308145382352262E5C1539686C503937256278673A47852C609473477E
4226439C584D5D452F787E3C4F672B474B696077492DAE642B91983737C781407DAB4E34
4F6F5B39A0773A745155774591594B6B69426483607C5D6F654D8B8D4348664E8D605387
458491419256387E647A9E464BA97C30727C519AC57533477B343F677638652A68784C47
9F647AD1AD597AB46C4085C499568AA05D8CCF524A86A6853F636B275234877C3D70663D
857F4A75B46C4F603E525D35284E896A48AD7261278FA89D7A3B6136428B72357B781D2F
4E6758376D7E432D3F77462974302D648049936D263B76998FB0C88A2D4F72535360613C
3D45705E1C4053514D615E328058237A382A6952207E9F253BA9AA7835504B72
71A850633B834F4F91642A673427485A2A26154A5C376A30926032647A745A766037556C
5266322B5B6A31418D3A3F3A4E22365B2337405747442A27636D573A548D215D8B70467F
4C25298D4A4D5F7B34776F416B7335886968717C5D2FAA323E7A7A4C3EB25F3655854F35
526B693785763A7139356E4B882D3D685B415D4F3B785C7B5C3159824C44604554656899
5667894E4E4F3F6D4B87A233337D6D3A82773764B07138568A494C7C8B35503A67623D4C
93705FA498699FC99F6186D5965EA9B66A95D06C62C2C8804A65802A51398B7C4BA97745
A78F568AB0814855557994463258A4744CA4605D338879897835664D6A775E3A83934C3E
4D5B5469A484473E4F723523735747546847BE99715D91963D719279284C6347679B9254
554560361B427448465F3F4B9678267C58215C4927668D53203F63773B5D4F5F
72A7516741904E5287533C80651F2C612C3E2C615B307838A3653F8A8C715A8857418785
59643132898244439332492F4222537A394146403E6259455F4C2F3970993B4A814E3EA2
73352D852D4B7E742D6D4425446F3885312F4E8A5A34AD65238C91393CC288496A98643D
5263793D956F3D81614D73478D5B47675F3D5C7F427C5B78624C778F4943665D69566394
5B788B465144465E4690A92E43996E3884895167B67337669947546C7E5977577C715F6B
AF8E96C99B75A8D1AF658ACE5266A8A64985C0523C6E90694C716A4E8F498656335F6647
A460389BA372325D394264573B4F9F6F59BDC78E50B8B1BC724BB87E7D7350419897553D
444B252B6B794E43456A251F775B54554A236B594E598E855C828378274A63262B558856
52464D312E4F89473F5D3331735C286D62286C713169973D234F7255385F4F6B
64894E463C7949527B412562561F2C3D3C5C34603A356F3099372B555F5A4E8A4F28606A
67672122506F44469E3043383D1E345A37454C5E494252415F4A37284C68334F74333074
56342D923D465A6B25777B3E4B5D367E4F55748B4B33B97825877F1B63AA803F6B97506B
A098773B7C5C4D8E5F526F53AB6B49635270A383456B49927A4F8359598D9063665983B1
647D734C9D6B4D62347ABB4D489B75438A7E595CBA705C9C9F4B5C675B34725E7561477E
BA8FA0C08B3E8BA78C4697C87451BEAC5487C657557CB6814C6A6644663C8084417A8E51
A976407D9F6C1C2D3C4F62432B638662418360814CC9ADB4732B6476726A58286D996642
446354406A7E494547651F459F7553595B427E4A48637C795FA2A17D2B5D82425E567864
4C4C5F4F22456D593D744F3E763321724327646B2B4C8D4932577667337A687D
7A9C4A6E3E774C51A3793C807521345C214A2C623E296E3780664957756F48A164347F85
627A3C30566D243F9555654B452A3F663C56856E4D424F49A1763D2A64672E4E7F644087
4B2E33A5546353502B76763F45597C8C3A5182904440BD8227887C2139B27F3C7388472E
46657234725E36744848543E795143542F37626D416C497B6E426E7D404A70505E465E97
6469714B8D6F4C675F8195554B956759A2974E5AB84E3B5CA24C5D687D39684974614C5A
888DA8C47469BDC595679FC7835DB8AB598FD38A719DBF744C686B2F50337F8044757045
AA7A54AA9D6C1E243757654C35598A71549D786839B690A07D396F84767F66296BA8683F
5D885743697D4B5672794421855D4F55724C7D625461AA8873C6C8661D517D5967717A44
505D76792F4D6F3E408B6144936A2D6C5B1D4C62353C9A3D327AA07C21505776
7C8150703D6B4E74AC703C7E572F576B2749427159366133766658546F3857977F347248
3A6E5433597C1C367E5153314A22365D1A345950404533325550432760672A7F94783F81
422E578C515A4B6331627E3D45472F6B47507E972733A1712584803144C17D387FA6664E
617C8739776645935846745C965F3A6153447E763B6F578D713B6C8A4E56875556786E9C
64628153764E476865A3A936429D804480873F51AF714974A34356757D3F5F40745C4163
8291A6B4887EC4C59F6FC6C68269BC913A7FC55D6B99A34C4D6E57344D348B794A705130
945E51C1AD472B3D3B676B292D569586519F6B793BADB9B589386D776A9F7A4789A53C3B
58683F578F7C4F3A5C62412F9B423D5960459181563868784FB8A1782947753680AF9A3A
3B6388643A6E8647347A3F377E883671641C4A6F50379A4F2851717D3A604766
7B494F6A3C54263B7355346014131D3F251C142937274E255D5C4F46524B46977D2E6779
4A5F452E587E4A558947473C3A1C2C482E4C55613F3C4E5B7D6746245E5F243A6E5C3E7E
4929369244524A6A3D6E73395066457E3750939E4334A45A2982883E61B3753275874555
6A805C328D5D56A45B55816DA6694C69554D8564487A61AB65427F72596C77455F686495
54738B4D8D583E5F4598BC2B46A67D448A714167BA77569E93396888655F8941866C4F6E
957893BF756EBDC35E48BB97456AA57F3F89C9528099AB5E4B605751683A8B744D886142
9A3C2FAA894335261E6877432B3C75724BAE71683B9D7CA181408C5D4F7E5836728D4437
4D513B3E57745F4B59623A49986549575C4365696B55908C69BD927929497D5E59568864
40577B4027455352337F40397582387970214B5E3252B35C345F70793E6E4279
7E69404F3E763E507339356E4F1D3A6229502F776422623C80684B544A585196672F7478
5A5F2B2D727540686F414C5553193866395C7D5E3754524EA6833D2A736D354E7D624598
4F283D86364F616B459568305D548379234287933C3B9B4F20868D3347B0874586A15C49
4972883A8361407C4B42556093493D4B334F9252406D497E5941686A45596D4F504F7186
5E7C694B7F4C4B635977A25341A1745EA28D4354B65A547C8E44576F61478A4965594288
A57A8D9C5053C7BD9151BFA16C5BA26D398DD1679DA6A9665665652A5A499052446F5C44
93524AB7914A483523615B3325406E603E8D8E6E4CC09FAE4B2D6B5C6F9376397F9A5237
505E444064705D465B69215CA76C455778506A63635688866AC0A4782A5A9F6F594F6E22
5299A15A334F5F52349F50396D5D2F857522577F591C7348306361542E8E6492
85864955396F4663985E367A5439563B2D5B295C312264337B353A4F5B4E6B9F64265D45
487C262556622B626E3946376017285A2636504F333E2D3F7E62331F5B442C707854407A
572D4F8B3543495A3D787C3A4D456163415D86753A40A86B21889D5254B585428AA65650
50697F349267457E4F4A6A6D966341524337856A4B815F995C486080555674544D385183
5D7A7D5697555168659FA74F3FA472488B853D5AA263647D8D445B6F66618B476D553776
8B91B2AA5560CBBA8F4EC69F835B996B3E94BE748FAAB25960605C334C4188644C655643
9C5161BF8F5180612F616A3E3C5C7659487D5D5243C5A493693669567595613B92913357
917A473E64764E66817B487AAE5F4D5477526A595A80BC8D6FA8AC5E468EA458514A7538
334C675C275264304EA16E38784424884A1742615119892E3178665D1D556A85
94954C763C6736508E583D7F4D2B4148204F3C84572649297154384A563A518E6E2D515F
495540305A6A1D42543E5155491C28593E4745533F484B4E944A2E20613F2647786F4384
4C273B6D3A5239663A8A70393F549D6433669977374B865B1E88B54E56BD884487A95B4F
507D6841A665427345575D678E5E44584537946A537E6A96724F6485595178554F546389
648284517B3F547463AD97233FA9774F93843579A256717C8C476A92686982467F644B7B
8F91A8AC6372C6B189507BAA7251A2693388B7569F9FA441859854304F408264495E4044
A36E5C9D8F30896A2E59621F2D7BA6664A987C654EC5A7A36E35665977B66C41A6A2222B
525E31517680234784743E85B83859866C445B552052846B66B8B55C285B8D564E4F8642
3B737659275E6E512C92483576512E876A1E495D311D8A1F225D7E5C3B60357B
806950763881515F8F56479B6B233F5F2C3F26604D2860329C5922525F4E6192732B5A68
795E4F366B7937595D3F515148235078374A405149694A4C8C503E2D8468345A816E429E
582E3B652752435A4F84603A455EC45D2858A9A04043734B1D89A24D54A852257DA66166
6881412276633C8135716563925171835937AB376F7E6B8B4F769A8B6054654C7B5E5D78
48A3934A6E3F5588569C9B2562C0894C8A5952B0AA5B6B7D56366E785C4F84265C716673
803C99B38C83CE91383D5D9B58519E834583B26F9BA4B0545558362B4B3E9082565C5548
A06647AA9E4F955D2972782D3D4B6948486E546B3DAFD6B67C286E4A568B4832BDA24937
5A73323C658442495B6F228ABB3B3D53483A5D444359965E43828861548CA435666D7B3C
4DA09565446066403AAB632E664F1D69682548605D197046204B4C5938723D7A
7C954A713A8D52626425497656264149517335754B357B389B39388F87494E925C3FA682
70622228919A586184255D4E400B3A6034463C271B4F5C3F8E440F1E88913963602331AF
7C2E3A721B5B7553407951335E4EA956274586733A496636418F954B60B35C347993384D
4E7570359153428A65635D628436384B3D34A74F5470646E4D3D626E5457623A4944487E
565E7E415F4B4968559FAA4134956D4F976D319D9352757087485871673C9357666F6873
876D79795775C59C733DA5924E58A979447BB75D7F9D8B477B7C41394C37805B56845C51
AD3E3A9A964DB27A255A653F4858A655348852602A7F879149367B717D905F40BAA1445D
93893F4E667D3B69927C30628B485168403757565488B38553908B4728586D3A4B4B7525
2C7C987B36505C3425814D2F5C5F27786624466C7F2A923E20444C4F23727E98
88944A6544994B62602E487C4D2541162A51274D102D582B741C1C5977404382531F5C6C
555E231B5A69405B5C334E2B501C39682F4D3E5E424C4643AA6F3D1D6A6232685B332B7C
64273A7F2A3C3C3D4395702F3E3D8F70546298793B4E685B1B6574385BA1784A7794494F
447282399848397D576B518299594C53503DA157586A6EA068587174535F8D5F5C595998
667562378253515C4283AB5745B0514282763DA19F4A4E657F425A6F64429D6475825C7D
8381A88E4886C0AB7858C27A4267A87051A9BF70A0A4753391853D34644272364C763F47
9775579B8B349075315A604B4658A96B3E7D61714BC6B2AE6621618A88B37E47C6993C39
5B7D3E546E73303D69792F497F41476B5E48644C2B5A856F418C9E79326D8E4D4F559A68
53AE76762950666F2E8B592B668736775D2A597A733BBA3A1E3C52593F60387B
6B7A4D363C733F628F4A61864132544D2354286031225D326E4C2059794D499D6D2F6B59
48814F3577612D646F4549315622407221373F533C4A353A7B6D4B2C84543C80746C419C
5F2E51883C4A4030358364373B5176724A88A46C3D618C63308F824962AC7B54748B4A4A
49676F41B5613B714372576182464D52353DA24F54585B88615F6E5A48647C5A64414D7F
6799864D914F73896090A75145B889498A784B90A4525661715178844339764C8E7A4688
8E83B58B3E7FBB9C638BC3913D529335396EBB71B1C19B4D5E773D274334905D4B7F563D
916763A38B4C9B6B3156532B4F7BAF66396F5D6B47D4C5BD8324644C5683874ACAA96E45
708A425A63865E4D7785423B61484F5D6B49614D5576A2913F8AAD793A738F494C537E5F
51A29E5B1C4C6B773085593198872F7E662A437E925188492459956C5A80427B
698850533A7338538648608F58253D4C233531765334673B76562049582F397369296249
2D5458348566294675404F3E5F1E4162374035563746404C9B814C2A85622A61605F3EA1
412B477732423B483F875B34447C65755189A9623552645A379D97446BB375488D964F61
7677543384563A793F725D5C5A3A6D6953348F36726F6B7F3D5F806F4048544561564F70
4768804A7431496E519FA83054B478458C545D9B95485559543A53835A366F32546C4072
7668B8A4638CBE6E3C56B38C4A76B76D367AA35BA2B58D4175763E313E267C43518A4E42
7B2E4CA17D4FB45B357C683E4761985350A399752DA48FA0492261867193654BD0A25848
748D428091956E5078823D548066576671537B424279958A3A7EB473366F8B433E447C37
51B86744226FA876347D382D707F3370893943848F4AA24426295066568A4078
6665516E3D8D474F84426D92692749642D5629644C4250206A541D4053432E7A64255B60
4A514F389666324C73424F4B5C205763354B444F3A5D4B467F6B4339A8782E6B636940A8
66304B6F2D50504441923E35677044773B718B7D375F6E4F4C99844287C0603166893A44
516B6B3D83405084608856817F4A4A6255438F41626280995A53705F5669845D5A3A6689
67746B4170516C5F43A2B25142B34B499C6048B6733F5C65724562805E567E4B6875446D
635EA162428FCD82568EBB4037789F5D418D734E84BC77397D78416D673F8C3761975844
773841845F4781372956412E4C6CA3372C53616149BC95A851246C7970715B43D3984B4D
795D2F514E765250656E283E7B585761413944494F6D9787446A926F2969844031658D40
4A8574401F4B6C6F3778482E758536727D3C43857654B7532F314C685D7D3F84
66715C4A3A63385769354A6845395D3544642C542C40723A6F3D2D5973452E874D32775D
415518286D4F2871742D4F54481A4568396466352C432D378D65283180543A7946383C98
62365A763041493751A06B375A372C6250ABA153497B994E368940316F897B48728F654F
4A707A3C745B3C694287476B75424A5B3850803B504E718658576C5E3B4F715F57395177
626C6E425A45676D4B7A9C5446A961398E6744C07B4949566E435E5D39556646665D3574
7571BE5A2C72B28458ACBF893E7E915077D58F4D87BC7F458D8937375236A447509A6443
7F5F4F94734974413C754E2B4A648638305F593D33BAA194312D876C5D673C31CA6F4553
76834A5A4D7E4B466883333E7F69546461525149487AA7903C65906C226D7F416287AB79
2B906E6D26557B793D9959376A4F3085813F5E9D3E51AD5D3429486F5C7B6191
5D5E4F68357643538752596E392A483B25383763392845246538172B4D30558C62255032
2D4F35286E43175673363A35461C38511B333B462944302F6A6E40377C2D345C4E523384
333E7475414C452B4F8B663755601D5C54A8704D3058593D378352376D9A73446D825D59
49586A3A784E4A7F51846377743D4787725E85314E67988153526F6F4E577161564C677C
60737D46713E6B7954929A4B50A97045986353C781494F486343638B546754425C653572
8374BF674E9EC15E4BCBC8994AB1841B4DBC9F4F8FC75431B18934404933A34954884346
8B4E628B714EA1443774543E49417F56326553443BB49B8D2C1F696A4C61583BCA5F3762
8B7B4758527A584E89853A4D916D58797A5F54533684847E2D58854F4AAB9A3E66749688
2FBE9C67244E722750915E418A501D7156263D7E7064CD442A405E6F19579F9B
5F915379357747538D52609A62344E672C4C2F6F5046632C785618236C474867621F4C58
425752315F3D1B446B2F3D5A481B354F2C4D4D4C2E4C3C32717E47408F5433595157338F
502E60793A4C3B2E83A15835575A224D4BA5A56D3B795B363784583C7E85503F796C4E5E
46435850883946725077649174426787685682357C65916E50758D6B4F585F677D587969
6BA5774194227F7F589E862D71AA734890213893B15A55623E69A0A65A7362438A733E75
657DCB84689EC03639C1A68162C5943E60C87666A8C36025596B263D5944AE5F5E8F2D35
94926B776035854C386A433164819254426950684CBB809E5C2963635B9C6B43C08A3880
A7924583738927609F945571A7366A87716B5F482551825543738F41156279457E979044
2991916B39618F3E256B362D69772C6D8A3E3C826594D677391D4C6343233E6A
6691567A3372454B8A4E619B6034506739583D835F3D732F7F4E121D6045596E5E1C515B
6253432C663A21545E2D48664B1A4756304F4139354A2626687A4948944A36605B573B90
58345280394A4030898B54426C56285A1637897438815332509E4A336D7D3542B18C545E
4B67183D9561425428885F903F2659794F516E1661548B6827526947485A2525504A7746
3155613D7F254A6465BF9D22559B635F985122597843687D2F31497457844620598F467F
292D886F5E9CB63B6BA4A64A5CAF95486FCB65507CA97E4C628F384252338A5373B14C2C
603248916F3E6D27379D491A225E755C35784C5241C6C1B27B275F3D498E69459BAC4152
6D43325F677A433B5F635B768A433B59276A7660467E7B4E2B4974561A685B274F5D6F56
27AF894D203E6D75337247346B67316E81393C8A5275A57435213C5857294D94
888F58763F89444A7A4D7A98533553585C713B795A508A3D8D524A7779414E684C337454
6156285CA34B2D566A333E632D24725F30504E1D43624328643E3E85914A3660312D4D8A
603D5D654066675B818D2F215354334D51988F503885571328596E326391302967704577
677644386B2E575B2F7045945725444D315C6E2A4A3B926A4048514251665F4947346F78
527654326C426369578A813C559553427B63216A72424B55593D5745418C40294F583666
4D5D933F2D90B0679596A87573BFA25089CC926B6FA8925C736C3F6D58458F31498D724A
8B6B5B8678448D37445D492C395F803D319979612159668B562D5D494D69351F6470485E
8F614468687352526E805C558E6B48845A71583F3D977D5B33547A52238E7A305F7A773D
35A3826223385E55518A5B326766335C833F41AB6A75BE522B22335B45298AA1
5A5661463A603E654F25495E374647213943264E1B32572F7D23283A55385688541B473B
586425215B2F276C4D351C492C173C411D43422C34381D1D78542F3F643937553026315B
4C395B6C2B22392A6D8F4A2D452B35584F968A32477C6B2E2E5F463C7A9B5934847A335C
47655A4A7A3D4660456D40745E294E634F517F3652648A755150875740617261544F7976
50707938432B697986B07B3A5C9A5D4E8F672F4F52435260603F55774C8D31495D59345D
7176A5555289B26D94839C718BBE8452A9B6796770A761839A783A5051338145538E5B58
AD8D6F7B63438A4F466E594654659052245158564EBD91AD561E46475D72552B4D693B6A
986B3D64666739699F8041508F644F9D6E735C3234A0A3622E546F3D43948D365A678B7F
2BB982471D4460542E584D318472355D65233E975572A7552A50686333155786
635F546A356334486A367264232A464C3721256D40323C257A493E4933223B6A551D4F3E
3E4D4326652E27595A292C42261A3C4D34473F4642564C266A6D3E5A752E3A6551413764
2B3768793F374C3383825E54644C514A46A47D3736765838438A432E53864238A7935269
4C6A596887464C7C5171547871324E78595F863E546084554A6680534357646856468D67
5171793E4E2965718FAA813F6AA95C589C6D3855614A4D5C64435B853E894C525B804E60
7373974E4792954F8FA5AC6C9FC16D439EAE698992A789764E513360543D7E5D6FB04C49
8E66748C5C4D782E408D4E234B86A56430594F656BD0C3B2882962465F8C773A4385434F
8879396F748E5F578685405B9A3947A773735B2C377D796230506F46207C81367DA29375
2DC1742517465B55314A3C2A62702B617440338B4B688E6E34182B504D173E76
6A8B5E783B7F4247652B58793C3D48514B5F2B7B664568297E5F41454B3D486E4C245F69
5657412D834739786C2F49524028576F3A51402E566949235E4A3C7A854D2D6770593A7C
61316A6D424C5F449082413D52475258527972423B815A3D4791543A5691393277784079
857549557F455E724A764D7A69436A65496C7142695885523B59774A51664D676745976A
67826C4F782972658ABA7E367E98616C9234417A7C49537340567D653E91406F89634A67
5870A85753B1A6406778883F7FA47D54B0AD59635D8D7A825F58306958306F2F5A976955
893E458E584B6B1A3DAF5730455593542B5E4E64449385AF8026706B638868484A934657
8E302C6E797F5E6376422F6C945B3FA13C767860559C6936456E784D1E7B3D2043648678
2DCC8D721C4C5A6137404C37626F3B7882403178497694673317234B501B438A
808067583B6D3E459042749A4543513A4260306C4B4D772A8D3B535D4F3A49653D2B605A
51561C3C963C3373543C48582F1A3F4D2B5938213E4F33225A272B846133356849376374
51306A5A282F4E2E798457444A2C4F6C5D90852E48907B365B8E483F6594562E7C61305A
4D594A63842E5671496639836C324F45387D7D3B4E3770764A5B6044455E60545B3D6E5C
5F5C55775C3B6A496D9C6F36677648618B5B466060333265523F4A413E94394D5D434A74
4F587E3C3D98964E786C945A98B2895EC4A8727C6C876BA59985275B5928694151734D59
81587BB2624B7E2738B743384A5393292772877956BEA1AB41337F5B5F723624538A415F
8B5B44656F7F60759878374D736547A675786E7F5B9C7F6647557049218E914A4D6E9191
36B39784264E4C713A5F5E41552F1D607C36318258A8A75D36131E4E581B4C87
61535A5B3A63334D84407C7D3C5679543B62286B3135561F873434454D425C6B50234F25
4E58282E952B346C4636224F311D3E3D1E463C35504D291F66492F8E431E3A724A2A4A47
292E51613C2F3E39997E556258365359579B6B243E7064354C7F2A32507E3E33716C4375
4C483A6B763F625D51695872743950855B83823E63647453496B85474B525B515E4E7254
6677719D50387E658DA35F34747B5C7C86485D86743F2F6855444D673F8A3A6E6566457A
646A79464EA5A762776C87528FB28176D6A46F81666F54ADCC57266A64276E41586B445C
81528296445D9E4B3BB94A496267853C274F4F5A4ECAA7AC7331674B6B80573E6A733E8E
B35F3A6E73363D92BE8E3F556D68418471856C6B62D2BE6F4C547E3F41CF97475566897D
2F8789511F4E50455F9E6A485F752748733C377E3968AC5C26121A526D216099
7185687040873E45883E7D652B334B3B3C21266E4C323619794332505D34496A4A245C53
50513128822B424C3C2C3F58221A3B3A364A312D3F5C4A256E622E9777422D6958325754
4E2E305D3831493C98752C445E3E522F50A875354C8C582F4F702E35547C3536808B3B78
4B4F428683406B557187538E65426C693A7F7476967F823269A380454E515A626B526F4A
6F7A6A9D3D388B5F95B06A4AB08C5D88803176A37D4C2572547A82563A8B40756E704A94
7470724B4D9AA5506F779353898D3144C09F6E8A6A6E4B898F4E27694126753F4C80546B
7F648989452961462BC23D2967AE8E31325A4B453CCEA9A56B3581508ABA65407A6E2C8B
AC613A776E6B4E546B6F3A52962D359B68796D3128838B5F4953772513999241566B7D2C
1F7B8A59264C5E2434433E5582792C51503065A75D6B6A502B11164E271F82A1
80846B6B659E4658913B83A245486A554D6731855A3A6B2E904C2C55714757685921655E
504F332C7E354D4E4E32486125193C2A375333334168532B725B2DAE7B3F35705024645B
5B3A3769404C73338E6B3E4A5634514E3E645E3D5183542D54792D3550722F607B783582
4853287173376C4352754A82432D546E43804C3C6B756443365473404D583D617A486837
787D5C9C44297B61ACA363245F795D925E2F4E90553623593067945A3B7D307E68733F86
6A7A844E4BA28E5690AF9B4C8CAE746BDA8C749D747C7574704C3669362786394B7A738A
9F6A6E915C4B7C3A28974B352F40614E336357333DD0CC91593486157D9C5C439E70235E
7D3C2C7D6678515A856E344C64382768526E4F3F5A8F5A4E3C5E7C4225AC873C5678894C
273E714B3968884F374232406B882C5F762F407B51888E40382A1E40541C4B6F
7B6E65454B78425582418DA73E4C372B684F35622F556230944C6E8E704146665B2B7868
51472555A44154563F3F49562E3E7D43355444336E724F25641F40A17A3939754E358365
5A3A605B2F4D4D2B895D424E473154496397592053803D469C914137515D304A51643492
5E4E337262386D6A6B694B76592C4B5933737956896D685C52627138485D69536747595B
65796D7A50427D5C9FA2613B89696EB74C31629B54392C624144554D389645A1A77A3378
68A7B67B51A46825517A3E3E94A76E6AD8863F9CB5B28B7E79384577772B813557896E8F
8E398BB6504D6642288456564A455B4835615F3550CDC7A67F347F3D59644350A575386A
7B3431A392895C587F6D4A787F5B42544A7043207B9E4D4B5A8B7B4C29A16B4D8CB2814C
34486719193961523D484B405040428A8239569C49798E4B1B3049685B1D4B81
787B7A6D498D3D4C4F2467843949543D3D572A7144284D256F2B36595B36426E352F5C47
4E5024266533475F4034374F211E493B32522D18444E291E7337249C5C32346427238162
462B6E5C2856432293734C473F34705B689C5B227DAF4D2A53693D5283814C5F505B287B
61522D744331885A634C499766355C4F3A8985585072804C595F3C4B8270566160465E55
6F6C606F494C71428B9C78468C57469B644084B547404B883C50573654872C555D612B59
4266563E5EB8824C92A9633F7AAA7072D9906A9885828C768730448B6C2D7F3957724C89
7D4C6890464969221B744C5D4F515B364E897B4842B38A9C783599676456466FBE7F3F85
A53234666B5F4C89A551253B86534457296D63447EC06A213E5A6A4043BD712F4466582B
386C5C2B23406F37564B464A4F6523597B365E983F89BE3B21121A5466286A90
6B4C695D495D37628833789F425177554F5A3C904F363F3087443063413C548A57264722
556638367D21437A5B3D2A5232284E2434632B22555A3A1D855925A868312D6E512E7327
222F694A335E21206660403E3A1F4B5668B06F2B4B68612F627622394E7C4554516E3481
45442D746A357457676E537765385B4A27686752666255475F693D2C4759566A6D3E7C5F
927139626B537F4C899C5951A77D55A7452D7EA76C312A67317E733D3D7A3278603A6286
54949136419B7233527468407E923973CDA77AC9AE8665775828387263428D384956328A
915594A74B4C672E195024726C57501C3556896152CE9FA16046A05E686B2B4CAA54338B
A86B327F646039496E682D539B54575A4A64593B74CC764B425D6C311D837E41506C5F40
2C4E644E3C6288493C463F68806E255D523D91A2536F964F341A2354301A467A
747C75644C8333456E397D492D5861484B2E2B794B4445197F433759351F26705A31684B
465B32376E334063623E334A342D58292B5F553851532F298B4F28A64D2F4E7E58439E2E
19249543398440267C4E353F4A335836488266345E8C543B8D7E313C566C385B6C7D3B8A
49642A6E5F2D743F5E735E81584E705D357A66636B826A443F504E39514E374B5947602A
5B70475547427A5EBB9B4440888156A7473B6A816E3928521E3C4E53518B36A0786B3855
2C634C4C50A67C447D9E483F8C9D5264B98364CCDA8F68606423688B444A924362442C7E
7745779D354D71461C581E515671693643686C4047D0958C2E346543717D5071A7663973
96633DA48E8759568D55298293434154489F774983DB604184AE6E3C268D7D538FB88B72
464F5B272D878952514C3B485E6227626A2E627A4FA28D45364C5B7C5A1F4A7F
7E7072495C7E3D518E3883953D4F7660556E3D91503C5932954C44706529346C3A376C47
4F411750782D58644B44525D353B6A4138492C245E4B211D633426A2321A234D344C7740
353F982F2D7B3B36AA542E4B4C486353569C54296F9D512A596A35456A6025385182389B
796B37824D3F98617257539F523764553F8F694B397A7F5054582B476260515E5E3B7054
6359447A5F476D4E94985B50925569AE425F877F733829582C4B5255628B296F75664562
4F8F504059B2704187992C306D8F5971C275408FAAA269675E273F856D2D713C71583975
6E1D9EC34D4D6120235D2568524A411E6193853D5ED0C17F5B3E652852664082B9753F82
96253C76737D575D7F45216C88534F3D2A575F5478D9564543546A30278340334C5F7753
3E527D7D3D6C8D6C594F515A677B276E6C2C778245947A4525192357404190A0
6F6C86604C763E55674292A241517F5F6F6B3B8D4F3E5A27732F265F7134526948266449
5E461B35681F64792B383A5B23244C31436E341A5C583D33811F3CA059293D6C52547622
165D7C3B3991152B5E462E432F34544D67A050218E9054376F772554867830435A61246C
4E58369B6B317F606B6D5995613B6E4C2B7067525175755A5351522D3E61586774416854
64582D8064434E3B5D5E5462A553458F3A64BB924052636E2D4C4C326F82478F70425F6B
48854B1537985A40A89C5B3859964C77AF7166B5B3976B6758303B786F37722C435D3F77
724ABDB44F676F1D195C306351574F104B708C4D4AB77977255074455A5B2950A56C3D7B
955C368170865A53837246728D515546446B53495CD15360555879312B7C4845615E7239
426C98853B86754189A8595C5D6C3C976A32837D47ABA655381D2C512120699A
664D694A4F6740697332685D3D838851975753853C44433C9D4720583F44637D41284127
6B652D3A7A1C404C2F241F472524581F313C302B5745252C794B2192322F5A6E4955721B
163F3F1D38841936662D3C764241644B699C4C1D7D823B337F771E3450662E46817C3693
4F363EAB87377D4A6A7B6DA0603E676C48956E5D6B96794C658861353F5C2A7BA651953D
986E418154505263A5812F58B37452AF3676D55B4D373C49234C4625675B30876641465F
40A165455ABB404D9DB962353B734B5992825AB5AC8E6C6B662F44827534823B4880567E
7853C5AF3C79855723592A9694947129626C744258D09251303C524453414065BE533D6D
8E5B3F896E86585A8D5C3C7489515750476762534BCB6B6658638138389A584A647B694B
7FBCA27835B73F155C664F604F1947AE7933786027C4B26B3821334B5C264689
756D7C49507C394B6D38944D253F64578935396E4B44272A7A452150352E4F6844265F59
5D502C3E893E574C482D3B4F2226583764472D2C58643F2C89592263601F34754F577634
2844472144853743652F1C504C465F2356975D248A8242318C88273A554C2A5171792E9E
493336866D34771B74786F982D59897D4DA54F8C798B6D294C664E36434B255665478424
5C69428E255F7E7B9D962345926653AA2445D15158524B4B2B5650377D55328E7D655F5C
3286806172B55C486F7C5432426F3F3F68916197A99389B0943C45877333843D5287507F
936EC8A033979F5D263D1953536B7F4660626C5056CB9B684E386846524A406AC34E3F71
8D593D8C6A7255A3AA6F378782458A6C4267593C3C8F56776156453C6FBA8E55616E3426
437099613CC863365D4F4A655B60368A72346577297C8B4A39282E454F294D94
777980496390405D402589723243543B8B7860956553484792532B49553E556B46275C54
664A2358A550624F4E4E3F5434638A425B4932298A5D38308D402B694D1F3B6D465F873C
3042712F33682A47562F2E604344533B759A5C2A8D86404EBCA33A3F5C442B445F672B99
464543955B345F27645A72821B5378694395426A5683653D485857374339326D4C417339
6E4C3A852D3A736BA6884B4F956A53912E44857A6F62595635787746817A4AAF82626C59
1B725B5178BD557388A166373E511853B8A54E7FAE8E548D7B2D4B806D42763849875AA3
986AC29C19768D4F2C5B356147475C3C68515C1B59C5CF83572F6C4F6D63476CC64F4EA8
AC623F8C6A1D3079AF73388B782B55603E635C273A7C706F5C5D2D1938796D4E6B5E632E
497D6F4874C86939625545938A753CAA6C3D829F44988D563925334E552B588A
A172793B647C426235277073323C4A226B60547551745155945A45656B37526839386B54
643F105B914753492C4035421A275740603D2617585038348215689A52224042257E9868
3740832C36591B4C5A24315D3D4D4E426E755324907D222462732D3E666A395B6E692A9F
5741428E683A63485A49735A38466E623D896B71527B573D54685C42513A448D504D904A
80603B736F5176779E915257A5626292437AA15B5D5C7245309B993F7E5B2DA69A577754
4A9A5B4E87BE44807D8858375A6C2937776E427EBD95827A5B1F43723F925A3649702E68
924EC194225F6A3B34613487524E4F236A4D542548728F8F6226493ABA9F456EBE2E3A84
A1663CA17E5C4C649D67369A8D4E5251385B62232B8B916668636E272E766D4E87A26C3E
55B0413BA98C7B34554E2D626E603C9F39398C9D469D8A31465B867D3D305F77
7666773B597540617B46908E333976606D59413522734A4D7A2E758A793951651E356959
664A12478B30514836483134273E79385135151F5E3E2032813A2F66512350441959753C
254E8C1D394F19506021375D385753479F7F49278C6A4934998823426E68405F7352348A
774D3E9E5359984C5D507D52345075774089737B5EA17C4A54713B3F5D6251794A579D4F
875A4D9A7C69596BB6954D59A130788E4496AA404B9397402056502F744E3992785D804C
498C50429CCB508887752B407587376288422393BD939488551F4696444056336B6D3585
7549C6932B6D60413C713B7D524F5423674B5D4C8B868B79483B3E2F936B3988C54A476E
7E3144717173695A893329A385494C4646927B5229968A89A4A17A2C2D6D2C4651738B56
6EBA8673B2697F3D5B5E3C5E6E6A429664307E904CAF862D26303D623F306298
7A6E8948677B3D697E479085354482616D614F814051432B7A38395A4D2F444F362A5B4E
634D224F993A4F624648283B3249682E45361C2F675136328C38336B462357671F4B6B45
2F51862D3B571D69823B345D2E6268499077542A8570483D967624577F653C5C716A2D9D
615A378C713A7B3C604379734052805448A376844386824F4E7A653F5B5F406C714D7B45
6127447C655E473D767E4056A34D748143A1984349545E4B3568411F775B4EA5564EA06A
4A865344A9BA558B8D965D2C5C8F40679277336BB78C98B473387FA768384F3069634E9E
642FB383255C413A8580406E514A582E819D805D959F92601A58654EB34B2765AC3F5173
775440766C7B6E538F701B8C924A4D543849635A328C555A8177762E3A7E72515269894C
8FC1895FBA5767459E8B40645E5850BD6C38927A4ACBA25F4A2839655D39789F
7263B54465744886784589843754955B6A5D5A8A51544D2F994F486859456D7840314326
6A691E5C962348612F342A562F446C254C563B3B5F3219427E2B3772253B755E184D6148
2575863E32531870602A3058333E4949917F3C4B9B8F543B9B5C1C4181603B6C7B7D34AA
633539907F49753C84678D7741728455397C609D589273436F825846643A3A6C575B6F50
6F3D526E45737B6E8D863169B45A7D69458F8E52506A6D4C417F7237715B4BB16D4D8966
52AD684AA6AD3891B394523368553569A2693586CA616C8C521E4464376358337340459A
643C9F782C7956294746338F817A5C1A5A534D5299C2AD6A3C466052C46C3C6DA74062AD
9D5744947E54649F94702494884D7C6F465B583F498B6479AD9B4F426C7D6B50738B7149
B2C37856C0804C245A5C4492845D388A6550AF73388F9B6D4755A8AD3D296B77
82547D487253427B613F824A3B68833F55274E7549522236854A58814027445A24385D43
5042157A97454435194B2536254D75463D331A307A4526426B285C891E255A5327627926
22575119344C1F836F1C3D66465C3D34766E3D2775592456AC682A4A8A373F52676953C2
9A402D60646A8D30644A9C7421385E5249862C734E975F293D6149709D673650447F8E3C
503E7B8A464E5E69C59E4151A13A8B75486B6D42569E8B434F897135754352A681549848
567D6145AAA03497AD7C53649E873955852D19AACDA98D984243869E54345250A6564A83
5845C3882958543871562A4D3F3746216F5F45296E6C845C364842279B5D3B89B1363F66
6C254F8A9C6059585F24469E8237402D4FA98943257D5A61BDC06D203568377191926C33
8DA33756888876275F4D255973673D8F314F9D9942B4B5674DA2A79C4F2D6B5A
7F5D9841635E3F76694C8F783145893B6D4A43634572492B64263D57673B5353212C574F
4E45154E7E3C493C1943312D153361464D45202365533F5468134A673A1F6624285B694E
4A977E1D2B5C2886893F2B4D31736758848B2967C092353B9443367FAB5C355E69372C5F
6C583D60354876456E3F746B31516132577D638C34887B3D515C2C325660476245496F46
571D4888606C392E4F5F4D4DAB1C55664589732736796A4B40594A31795A4FC24D5C9F57
4C5B4B63CABC5EB2BE773044657337537D4D368A9C7B887D39195676474B434784747B93
4A2C7961346F2F2C806F355C5647482187956A56919B8E351C453B3B8C3219659C474E77
642C3F6865707B698659247A78414E452D4B735227955A5A807A6B233E725451475C7344
AC95493C60577E3C8061395669634E9F673B776D3ECEC240253D3A554E277891
7D508841705056A53D2D6E6334544E32733C4C3326424A308D4C345F743A4B6C4134554F
757C1B4C8036726836254E552338753D6770373A51311C7D6726524F2449904C29507515
1A4048313153237C60422C52294D4E688D432A2A67503842B44C1741755233604F733295
6B483E576056653E8454866B3B4A7353565C65944A91793F4F724643614D466365655146
50324B8E5F7F3C75927F344B90457C6E33875D3A3E7546394154391C625246C55F346958
3B433E358E944195AE6649488091375989543C94A576868F502656944F5073387F737A9C
6470C4582968542C6D632F54574E4B1F5E504E599E9681492E333F3D6E483466942F4E71
5A364681686688A89966246B755B8E5E4F584C402B98608399735F1D4164715E5A647443
CCA84C467C764339887F468F72375FA15D45A461238EB58D4B95526956398C9F
A5619E437D643F55573F86464DAD9B366D455A8B424C31418647345E4F3D707D3C2F342A
50511A46743E413E2541464F1C3F7935343E4647512C224B5C275A5F2B266443305A6431
2F593F234E5522895D2A39623C675477943C2C317E523440BB52314F98523169566B3495
6340484772666B44874C87583A4B6F5E71535F935876653E495E424760404364616D4A44
4F39539E5F72518783853553A0497665296E6D46417A4D404F594E26766553D57A4B8858
41443E3FA09035688182544D99994166B87335999461A2CF6C32598D4E525D3B856775AE
725FC455337D643873593A637F69642C56445550AAA38D473C473E35685B3B6F932A61A1
7C3F46956A3D635E603B5D9E4E335F5F5C8B6F2E4FAF6E7DB185573A807166685C75602D
AE8D455B8B99762F544E29667A4953882153B37140B3C55F52CEBC9B1F206B76
7B55783A725C496836369046274D76325C225469364C23348845336444315B64383B6E4C
7352206A7F515347344C49432652904F504931496C644B74592F857F3830633E60A77F5C
385C41135E6328895A1F548F447139438B6F43357C4E306FB0503453A22F326E837B3591
71314B717862735F7F4B9350587E7B5B745F54A16179624B77733E486B394F7C74675D64
7540517B3D8F5B8293822673933D5F4B2B849B4040783A3654674520705641C8674C694A
4A414F42A7883B66758B4F4EA44D2C477B84398E9859879954345F6E4762463D743D69AA
6D45C351358D703C77372844526B6E3041373D33959CA165474F371964733D709A283B64
581F4B757E7367784B302E6A693B561E275C7E51389B3047A6AC411E506D375483A78343
B36F2D335C678A3C62693D5C6D3B5B6F3C3670612BC6C639294E57833E256973
8662AD3C74684D7642266F513C733F235B344F29234243447E2235806E40726D1733566A
9C4211538C516740213C4E1912327035584F102448422E803F16556833325D1C325D644D
4E955F324D6326896E3E275F41654C5B7B4B3042955A263C6A403156A72720385F663FAC
8A3B3E585C66593875499C5E3146635189593F905B8C642D42664862886149614D7A5244
562D60914F6A3A62A686314A863F524E263E9055407B6F4486B75E206D564BB36F4F6C4F
524B534DB4884D7297B44C52A3883F51566146B4A17A8AAC3D529F893D332D4979596595
3A4CC653356E493B94643045564D4F25413D5E4A926A8B52495C4337544B409DA14E4D7D
53294A765D726F745D42336C5E3A58413A4873633FA75A47808E6021534C383E346D6C43
C99B433A66585E589E833E5F66307FA1483B883F1F95B87741904758212B7089
85508443794F6DAA6038952B375F81254F2B6B6125352A41793429624F37809935322F2B
6C611F67552458602E2958461D3C5C1C5C564547594E3A81612478552E467A4D427D501C
274E4F3163571D5A744B345B26775A75874C164370612A54A7463758A14231575A343A5D
865B5B572A7784486941A477404F34196C6A5375319F7F3E6A51304C6B655158196F7559
48145B8E5085423F55803955832F7C893741A3384A86673847534228653A38AF68608140
6C867554BF8F314C6B8B4A5B8F744165953C1F87826094AC2F236B8439592E3B6E4B6C9D
5A599E5E378326389F6C32475E523E297284705AA483B52D3055563E592C2E63824268AE
7135476D5A4C88C1684A35736950794B3849746057BF6E55A186602A74525B5840724F33
AF9E51518156553F4A66466D5945637A48397F733A93C8743C83565A30204080
9E5AA645814A42824D3BB16C334F8D2A4B1D4D70395238419459305E32244F6F333A5744
66532562683E5142282A544C1D3F66293B3D2B3C4B1A1791561F7A341B41654548895743
3766381D676B2970622F3E712E4C46796B4425334E4F2264AB2A347C8C452F6163634282
85536E7A4C5966556C347A755764615F7B5552935977694E978052506B676C8E72716C84
7F3A428F5E8F586B7E793382982A554D2863A9563E6C533C6E8D461C64613DA1355D8852
5E525462D3944E536A752644904B316BA27435817D5B58723F2462843E6052307D494D81
595C9E38428852367A42447F8E704022524E6B57BD95AB3F3D5565476D6B4C678E254069
7D3949657258669B523B3274494E835F2D4F5A3F79C582619A76343C8D6E744F50AC7D48
AEA82056A09F6D5B533C45B98C49737749408C6B3BAAC76F3A6B596D41253A7F
AE5AAC499B6950845A45AF6E37539033733D67853A4F2A1F79522B587133556C335A7749
64382F786E394D37363A56252A73743A4C321549744C27663B29A44D293A693467956238
4062341568632B856E1E4A7337773278793F2842583E3993A0403A44912F3C62675A4383
8A454B61476F604A65429C564658666289683D8A70995B386774505B6D52436955754F5E
5930575D4374506F8F691F4A8A3E834C253D804263A05C2F77986C216343457E584B5A44
8B825C4AC17F314F78593D66BA8C3C6C93603B928F665D8758276F553A774B3F7A7D5B91
585176323D7C4F387F38284A4D64482862583F44A1919553585E4224797E4475934D4C71
8C424E736E6981AC61422D7C734E61531D44675F7EC6635FAC9659235F4B2D2F3E8C5F3A
B2B25F3964775A644F57345A6C3E6D711A6AC25D3F86C3494A88948440263368
7A618643794E55833137875E344C3A4B84335D5B3E6A612B7A454E7C702F516720415743
6A5B1E5C5D39723D44314F361648612B534019386661317D301F80592F4F72473D675A33
5E794028674420826F3A34572C6E46736E3227638D4E296C943453739B462A5040475A92
905A5A622C837B614C34856E4F5B363C806B4D663F786447864B3B52726155592B716164
4D1D5B7C4461344774782F4F6F306F532C4E772A3B704E356352342D78533C83496F7935
57433E67D0723B3A6D583273A483396EA56A2D99764C65933E3A847A345524697C6E6295
3F4797525089575CAF653E564959323B94765353B87C75366F7F372D87664A8588554F5A
703366A16B6088B0483257AE82506735235C604D82C9466DAF984B25643C53463B853E39
C9C6794D7D7864875B6C466670476E614035726B4775C027244162844A2A3567
9C5894437C414C7C49418A464B7F7F365A335D421C3D2F40602C28423538627A38553E32
78612C764722474C4324573F1D444C1D5055393D563C359A4D21963E244C6B4F3D574C1D
2A57482A5E52368785483767296958A46F472C47525F25728C263E63823F2C54463F4870
9460586B3F7669547956866D4F516F5C7B5D5196759764427C7551487464506A5E815B66
623D6E8E537138668B691F5B6B36843F2753792E3E6F3C3566502A1D685034454E555F3C
71615D69D35C413B7B613D61A28F4171B58A35B77F507C8C35248F8A37512A35565A827E
4D4C6D41537E463E855D41554A604E2C5068675BC58B76324B582A257B2D3F73884E4F52
62294E69695AA5CC592F38967A4B614F2A3C4B4781C96764A1935522605168454E974638
8CA871589F72287967694E7F5B317D8E3C3C86815781A84630545E893E2E417D
AA609B43995E578D4A41A058314E7D3F64276172293B39307E563C485128465526515838
5D5E3364703553463B4A8B3F1C416B39443A2438605B2F9D37258B622F48774B38665E32
4667461F6A63338C7E4A3973396C44877340294E57622B8A943D4A51AC4D28574F55547C
995C757E41785E5D8261765854575C597F4651927C9E564F6E694F4D7565577459804D6B
5D36758C686E4376886522697838833E397796473C665D49685E3D20704F3953586A7149
81626D81D95757379A8E3A4CAA774884B07436BF51327783382BB490437C393D5A475D7A
4E607220597A4A417D503C524B715C3C5A5C5870C4B5915A8682472B986C458BAA574A69
84365E7F53457BA84D2545934950814623493F2F66C5506CB17144348759694560834D37
8AB25461AD7B386C41254D7F7039716B345FA55B2F848533356868702C354C64
B8659942985D5E82494AAB67416D8B417A347F7D29475E39915B374D7149615C25536445
595331636636503F3B4C9A3D193A6B334C351D34657D379B36257E552F4A7142336B5B32
4E61271768693489653D6C6C416933987A3B22505448258A913C555CB04133636B5D5B87
90536B7E3E8756736D547446645D4949773B3F6F839F4D3F75764A516D466478588B4885
65357E73688E586E886A2C897E378851385E6246396E5F3F8991591D796F505562777148
80748589D45A5358A88D4458936A346A9D793DC25646838F4934BA663F6D2A4970616C97
508188237A8E5048863A416D708B425488743C498B63713399AB454BAC75589E9A385587
5827616C815970A032166AA26034623C276841225DB1396BC27F361F5D4368467E8F4F3A
80AB3E5492633684484E4351743B82771855A6915589983C3686928036302956
C06987458C576074324591553E787F3F7F2F7F7722534B3B905039495B456B5429755F41
504447896A3652383545822F1F486035592F15356E75318E253A8D562E4E614252875A2F
597238255D5B3997723E5761468134887032225B585D1F8B86316A94AC38234F573B71A4
9C7B648136A275705057A2644E5C322F7552484344916E356945456995624F512696655E
4C36987B506D266AAA7E266A523A9A683B835E2B3D7B603D574D441D743753565D7B5E29
6B676B64CF4C38739F863C4A908E406DA85034C97F458C8F1D35A97542762E7096657FA0
3F6C98475F65286CA95539494A6B2D5281815C68A6816E2670762F317A2E67B1A14F3E77
5F284E4B876374A67E2F4E9162326E321D5557314FAE1E3E8186471D5C54462E89A27846
92B27373A98149AD8C4E435676427C85303E7A8A5483B4212D51697E3B373168
AC78864161395B843F4B79334F9356608D2E7371216849498B4A305155426B3B275D374E
7C2F31753B376E2F452F802529684B327449194D69612B8839358937357466342E564A1E
52634836714A336E6C36684448654B93883D2F6C635F1F8F561C566C963C24545446767D
84876F6A317657655A589A554D677547744A4D947594723B54573F5876645869538D595C
6234778C5D6833758F67217C4E317C5256AA5C1B58784E3A6952241D924B5344597B4D2A
6C535C56B14653949F864174B5843760A74D30A750429C884A3BAE817E8B2C365948AD77
637E734177873B528556364951694590C08B5E9DBFB1653670742E3E80313E587A4D4195
4F26505051379ACE7528639A6D4371874561592F5CB16665775C3B1D604D353B767B7243
C5D16276B0692285814F474E443997A537497778549C96472B4E606F354C676C
88728544774A607D3A54A2582F5D5D3A5631643C39763C4B5E45436148447D6521533040
5E452E7A4330472A2C2D6335174F50223C3D2C3F403139853638711F29505F2F2D565C36
736C402373604E896E2F6E5065634C9491313461554F298E6333766480342A57513E6467
7F8683703B76475E616A873F48685C4C632A5685828256394B4F4B667D5B577057734E64
623F828761683CAA974847A9792E66295093633741636B4369631E27934A66596A916138
7663683D8B4B41A28C704083B4803C71BA8039BE5247B6944F42BA5C507227306C415970
609580227076505A815D6393A6904D765F606E9AD5AE563F74642B728846485E89334BB7
6234514D59516B8D622151802B619270424C393088924A6085633620755D894C805E382A
A6B13770C29F3A8E54546570602C7B882A55823A1F8A8F532D546D431F626754
B5748D4DA853687C446EB565376B6942682A6D5B21523D6C783F29462D37596028685544
543D5EA562334D323041633E1C59543E452F1B4B6953376F294073332D5B512D36624634
66622D1A605C3C9B4B288350645549AE853D27544E5538AC7C3772627F2C4F7D7B504C58
7757CB863A7749856E636A3571795A486B206D7F728A54606D654C697C5E928F4F6950B0
6D377E5D81905FA8964F217D73316D533B7C664A42684E4396A63B1B865B69694CA8693E
7A664F5DA4584DA4AD4F35589D5A5991A56337C37A4B9A8F3257B17C536B2427603E6C73
457B7E276E5F3F5D774B4545396741615D526379A390573D825D247E794E4A5B9C393D9E
5831647A825384B13D1E779A5C3B6947426C342D9181456CA96F2033885968418E7C492E
84A83178C592379B5730557676377A6F215582874574A53638748A792C3B465C
B17A55507F476D602D58804C3B583D7393397B593B72366B714D5986613A482E387B5343
47212766472D541F423E561C2F6456333D2F1269684338541668753C286238345F7F5439
7768131E61573F9A4D1B66455D5C2E608852285C48223C996C335C678C31214053485F70
6F5CBC5F368A4E62615F8538406B5C5362405D7D6F8442285652445B7245566E49775458
642D73726E7D50A5AF72338E6946A1633C986A45506C945455524C24702D7A795A6D5B4F
A4873F40B87170C69E8C4785A569375CA43E319A5854B882473B8459626435797B477E6D
4F897C2B77703D7D8D3D2B394C4525676F726D8497A34D2B807E3B7C822F61989B524487
63284442745199BB7224598E603E7757334050257A8D29537467351B535639248D906D4B
BFB6477ABB9339A8525B4C5275316D73274E70825278983D223E5B7F33444E6B
9C9A764D7B3E728C40618A50436C3D3362294818336E307825313E53413E4B342C554561
773F35893A4D725152364A351E4D4F365639226C431C54602957691248816730294A4238
7771451A6F31388D693884495F5A5B8D8D293C866A462D9B4E388BAD80302B4553386E62
685CC752319358683964904E426744495537615057896537573C6AA1915E65503D8F6E61
3B3B847A72631B698E59379245378054388153277A928149565A49479D4C666C5EB26735
544F5874B44D30AB8951447DAA764986B27538A36252BB6E424DAD636E67233A57409E76
368D7C397B6E2B58723D3E4D55692B544F57538DB2B34C375576357C7248495691505497
50254D3E67497E826922547C465C9B7049415B297F995C737C643720637F8E2A8F6E4343
9A8B4881A4833AC28B4C54455D2281792652624A497E7639293F59612763615C
788A735199485D83508C792D4C7D544B55579D5A3B7A39825D283D45304E765235441C38
6832377C2827513E421C5341274C31223F4C39714B32593E1B70641D1F5650272F4C2A2B
52582D1F65464B76432C7D3A3D4B5CA46A1D2250533B35993B24426D5522354C583B7C45
5655BB4B2F7F43715C6088464584686C5738687A6E71513750374C799067656148825B5D
563B75746C4A399D994141AA6136744C3D95532E475A573C564F272789477354506F5149
A050354BA4552EA26674497EA55E3C74A27B3F946151B35D365EAD6F6772284A4B46BC6B
4F8C632982793D697D334E676C793C6658465562687B4239687B3894714E596180334A7F
3F2657477B59868D6B236383513E68585A4345368E895052874A242E818A8C27926E6440
96822792B76432AB944B6F5942267D772758663D5AAB593C2F49595518493F3F
AE8D6C56AE4D6A694F80AC58406F4B6F61327C4C444F2C7C61313B4827375B3D366A6557
5D2846923D394A3539366238264258344B523F606F446941207A702E2A5C502E3A54504C
5F56221C664F4C764C2879454F43479470372854513339974C293684541A263B644A8048
5B4598472F703A7C66627C455B7A5B755425646E787E58475C485CA79A6B757A4B7F5E6D
5F4D7E66846D539F95423A9B5E438434439E532F405456405F62253293307B62637B3F4A
BE5C5A76A74037A984844891AF7A4390A44241B68B68CB8D166EA46A7B67316047399B76
599B652FA3824078792A48555C894C67596368837574483B6869246B5842828390495078
393363576F4C7D7C3A1F7896554871405E513D36896D3C6DB76C252955393040BA9E8559
C09C177CB675359961116A9574287D3B226FA55F63754820416687831D61432B
B194685BAF4E6A695683B6584A6F49476A449B544376377466362F723F3A5C353A757054
7E2F3076403E533546335F3825455D4547563D5E7B4B8451288B71342A644E314A67544C
57633C267C593E6E532B704A54445B7F6F3F2C575F2E52A15F2D33856526333F72438A66
72557B5B3C71365B6A668C4750795E704E31617E7D7E493E5D4E60967C638661437C5071
4D5A815E704F42B19E463EAB5555A74E3AA16048476B593D5D5A2F34923A75736B7B4F49
785F727DB75A2D9D8D3F3687B67949AAC6443AAB3058A48F3F3D8246A47941A58550B774
457D693796642B86833F525E5E7B2F6F8A746F8E708C452F8A82386C5613698F8F3E5B81
262A626B564B9D97291D77854947631958655E3A845B13467E68302A6346301E7E7F3264
DABF3692C37B319B623C4F586A2673601E4F8A636C595C261C275571225F4560
A6986379A14B748D608CBB534C704A4B7A3F77345D813B7C66525A7847485A3D2E5B5476
8B333A81285768303525652E326A4E41484F3064763B87411D766A2D2C6A6B233458494D
6D743E26763B5E8958315A3A575E82A0703F3B70633D3798472D5692631F353D52228280
723E8A4D36914B764E709B4F5C5A40656D4C6A223F8B6539501B638F816F703827945E5E
135D9E689A6131758F6342BE384EA35343AA462E7F896C3A534C2039903C775870B96754
9C534775B7773BA09F3D3186AA6C6BBDC37247A86F58B07A434998349094273B4B43B64F
224C5E41C47C22666F4A9B9F9B8F214C69745F938F8C40244B723A6A6B455756683583C2
872D435B28139BC56B21657A4546664950414D4089452C5B78513068B59D7824897D614E
9A9442A3D18E33B36D486C5B601F8C7B2555845A825E573223335A72225D4273
86AB6484593774836D964D35678B3E5E3C498941514F2E9142414D2A2240623C333E2546
6A32739A233246303212582E236516274B5728422D1F9C50247E441055845F1D38531427
475435478A223A5F4D25592C6E5B8C9887152D667A494B98311A205D4A1A3943622D6843
5F3D79442C7B455D3B54824A6C50364B684A6F3D365F52425B3C415C6F76775E3A6B5966
4250795F9276487F943A7ECE7748813265C34C26415648365D5619388A39763B437E7B5C
9D471B3C7D5D25A18C7F3B96842046A9A5794994725BB05F3A5BAD6B82752A634077CE62
86A04534797F3976512F3D51BDAC4C64557F3E9CD8BE5B536458358D5546628B4014529B
612B45495E3D758C672A73642E8AB76D6A46534AA15142686F491742638F74238892856A
D3B03FABCE8A4CDAB64666596127A77D235483629A684C31282A5155187A6E7B
768C588C7C4164797087783C436B3E6C4F3C763A5650246C583C496C333A4E2A55885250
501A447F40454F204A2B5B404C7E3645414725533F2280391F83480E275B473E778B464F
633616277A4564713C1B4432422F8E99562126416746488C5E2F2887432A5C5E673B7646
4D4574523E7847896B69716EAC886A7A7D4A8C785D66436F6D58537D6A7BA473496F6881
5168603DBBBB5F8C8C2849B2665384214CB998524760395AB09740428B3D927E6692635B
9D58418A8F3B45B493893BA28C5257C2948641787461B1473759AF75A273275E4C3F8594
8F974A388B60347965382916827E39875660367F878E704254451A80855461A575375E98
4232514F614CB2955C34957327355E3F60443F7EAF854B616F69213268855C4093947455
D7AA38C9D874229EAC4B6A583D3DC49726519860B9C269342326572926939582
A5B25BBC9E4B70647AA1B44C4C482D9076506D2C8A933D945B4D6990393954153B62544D
8A3343A03F3F47364729471C357C3C52443A2F69673999221F9172273971273246532F60
7E4B293475365E742A256A3C56347E984024323D4E33499B6432298F351E3F5D853D6548
3532965B3E602A9961615E355F715E706C2B7E6747793144745C52723A41AB8F44615786
49676729808C5F969D4D56C06769A54D3DB175465E6648375570514C6C2862955B8F3B33
947D5A8DA121407F84703A9D9C7469C7A28E5C928D63B0634D6BAC729A59295B5545A15C
7F955F43BB813C7C70322A17536A309D645F67998F6C48419167377E5D4998C27E3F5B81
2F3C676B7052BA9221219481393D5F30795E3D378E483089C08B31306633182160724735
D3A1368FBF6224855E36A4853617816F2653A4467F8B5231232E6A4B17576130
99B75482864A724943779C494A6C387C715B7A2A507046A73A8F977C2F2E623631696D4E
822F4AA04039483E53276635337B404E532848817B3BB01231967B284575533144653E67
945E333376416F8F552350404E3C8ACD4B313E3A472584B063405884491B3846594C9378
54426129646041734D89774E6F61648F874C7E5D436B485C5D396E946C6D9D3148695356
34886F5E7A59439C9B455E9B369096413FC43F2C7D7E55344E592D6F8F3E5C5A68A6393C
674D4D99A5373391827245B8AF7266BDA83C63A38C59B23C18709E538F514FA0524DB148
618B413CB9793D7F7C3E342854602FACAC8C69988A905A2D53603B7A3B1F49616D405A7A
53263A657650BB8D5726658145425B1C415362499F5D20526A7E2C356778432774774940
A7D864C1D5712BB0792C747D5F239F2D1D71B36386A640132C4C7C531A657F68
88C75F875F518D677094795B8B94418865609F446E8944A24241755044627E37353B446C
6E27456F254B53363C20722B27671D567D3F3133183D912B1D87402A7C6D3C3448562994
A75830376B2B67824E2762366E394390824380826B284D88432524733C1C3F4559315E3D
613B5C3C54523D55486F47486E5C5067733C66614F4D3D5B49395664676C8F456E4D5255
3561464E78505A938A325A93477B9D3A43984D346E60453951461F6589386C574D945C51
8253338CA34533517A4037A79F6D6CCBAC7F529C87509537246F9D547E5A215E524FB552
84BB4038B43F2F8D624347527F6C2367555366947B69444173643F684A3E47716C3A4F63
502F406A6658B3844B276279453E62456735433F9D3F3E6F6F862B3C777E3B2A75856667
9BD564C1CA5F2A9284337A77462A96471B447F5076C4401920295D2725808374
81BF56887B568A6C73866A3D54703977296C9044623B55A7644EAA4222286C3131603D4A
6F234C762C2C3D21371F581E2A5129454F3B3F595F4F88301E91571A465123343F4F2365
8745253668376C7945265F346134849E4D1D4D4743384A945A2C2091521C3F3F4334773C
5E3162335A6A486F558C4D3F726D5F75723D5B73576D436D6F556A687A62945E7C595E5F
4372595A70658BA2923A65A45087A24846C1624C5B69573A525A346F9236758B53924858
87715B888C22246F656239C09E7171CBAD7D4C8A7A559343357F9F53654C2C5F5257B064
656C5237923D2B664140567B806E317633395A965C552744645C326479474B8C7B415083
5D3046714E7FC790492E5765417D864B763449489C3E43795F832A449B7A3B2A6B40302C
6BC25EA7B5772D755B4082632B2999661E4A823F76B85223252B5F4C1D506C60
ACCA5F9E8F5293698B8E80435B7C35966071893461583BA4577DBD873A2B722B4687574C
6D2159934839431F4630521C345F3A5A50354D6B6B83802324B06A22505D373C597B4289
7F4925406F3363764127623F803D8A8F3D2A605E3F2E6EA05C2A1C9D47336D7655407B43
5555872C517851884C9A4E5D8C846A6778536B684D9C4E978A546868695F95506A658A78
447B625DA8628EAB932D96954387A24E5DCA65455D816F4C696D387B8E3369765689214F
936457968E23267E57633BC38E6994DAB87D49AF8380B35A38868945644C2F6C497CA358
8E876843834932653D426D8BA46D315934374EA79C702F43543F2E956F3E4A90372768A1
4E2C396037677254582C4E5427446B4F6C474280CA77447460532D6CAF722F2861673F43
ACDE8790C97B35785448AD957E2E8C3D2A8A804370A8632A4D5B805C1F526E28
A6A244A680558443848C964351533FBE79824A358A73448A3598BE8433376D1D6C925043
393A9DA44D48422C622943387186395557308099698F6B144DC37B2651534060A18C4E8E
7C5A34646C36677342476F458D4693C65C3A6D743D2D83A258231C833E1A375E513A844F
413A702D4A4E3E593C8D3237717566605F43494A53843B765B3D5A5F3C37623C7B49554E
33703D3D7D4E74B7802D6F75348D783371B85B39617D3B66A077377D564388754F932B6C
C27A6990894348B08F7E3AC5644160BB954C62C8815FA66842A18836534B3368387C9C70
6F88573CB3573269394580AAB37F4355415A2E756B6742455B3A1C75553C659F5B334382
2D3C47444764614B4B274A66313F664C63352A43A7713E726D7E25448F58252260661B1C
95D969A5D0933169341B748962279728134587685D83771A202655231B486663
A4BD5B7F7B528A35324B7740515E2387516F55225A6343802C488A6A333F7B1831614554
4A184F75393F4323413052183C4527594E27686C67AC6C1D258250265E3D19273051308E
82291B4A65325F7F372F5D466B2C719E582D777D58395896572822934D1A3A4E3264C083
733E6F2C5B624E5140A13344746F5D687052373F577E5E9F5139617665475641915E7D41
32844953783D74CA97467F7239B2A3416DB54740708C5D476C5A338A7A32585469AA4755
635F6D9FA43A2B838A534BB8B15663BC902A3494665AAF2A1B829754554255A1556B7544
688A432E54512E73403D718B8A803E7B816D607A6A91344897703A8751197CAB79364176
113350614F656D3E37469E76333B442F7F683C4EA3403CA1B095283668343C768069191F
96B2297E926D2B7142256574562F8D3A1C5162505D7D6425282C572420455855
ACCA64907076BE5F837F6477B9A83C9D5EA39A42564A81BC58887D56859DB12F383640A5
7A2B7562346D73372A4E7425455523818A51745B38CCA84032A33C5E9C5935505A4F259F
9B671F64480F5791683C5E26774F879A5441B5C5583F6785374B589E771B3B50313C626F
673768167B83564634B55D4B5D3779A88F5343356FAB5C9531307D939546601C90688A2E
2B98914F75152C669A49926B17869B437DC210255299694F7F3A0C807A3B50152BAB7E6D
7D4B1A689C522F876F5124B07D5E6BBD976E46705D77933638A3905637331A5F5E7DA032
52835738582D38A144489CAE9867215E667E5D4A1F75392F4F5E3F77623A5C936E384585
32333852535A5E423121525C283B4B293A363146A2452C675A71243E642E13356B632237
B292549CB15E2F9F79395E5D533D9A592255574F757B65302F305D362871925F
60C861AA54479356AB84513C56783F7130757C4A4E3B4F884C856632283F82253836275D
642C7D6B283147283C2F5A1F3F44213F48335147248D5F4838A52A255B64415665531845
53471B654D153C5C4C3A5E173F4B92A7671C597A7D346A7A201517765F1A3B5042457655
633D6C245C64565235814E596942515E6E54392D5F814EA1552C52544657753C6D46965D
36773D649C6E8AA7613A9F833D86583A96AE63365386597FA86B2766683C615640B04979
AE6E5196763F4D96626933BE78658BCEB15F5AA5324E7A4F408F66404C442F7947A7BD66
51833C2C7C41255D3B3D83C2BA7131564C4F3F7D80864C4061634AAE9D536AA5361E5B93
4F323B77407C9D705C2B5863375F8350572F2E58B762467E6D781D4189601A3C66353B73
BB944DADB0531C4F6037614B1B21845321545D205A958A33403D64171C496260
A2964F8F6E648D669AB485465D6957B34F97855570695D9F618F9F753D4A6725585D4256
2E23846E3C4B3A2A4B3335305A5A3B5A55425E7D5CA5635455AA54346B534A7FA46C4074
6047316563245E6452437538564B8788582F7384663AAF9D58201A85402858734855A25B
2F6873347830616D4A712D59735E9E70395950497A582C875E3C75823C4D673A81546549
377E46426E7595C387428C653D9D7743809C5C38686D1B51846C2D624053765E3FAC223A
5C6169AB782D438C786733B73F3061A18D585C786365AB71416D884E3A40347A4078905E
69A455224349308125265F7180853C594E6B346D54755A456843226A843C5CA365393789
4D3444732D3754644A2E56421A3A644C773F2775BD8C43786B582356A664193F642D203A
B94C86C2B46E2A554B2E76612425A041225353505B875E3893AE7E3E2545742B
B2C25D9D7A82A74C69695072AB58338A4298373875596D861D8A96655A876E1A36653E5C
66228777384C53355E40481C2D4B2B5D462163674C9D58194A8A4239712E284665544A82
7121194D5E236A6020386737652A6FA46231809C38269F99574766A44C152D5328669E80
4E3C492896635A4947A13D436C4D8F936A5C312B8B7A4A93314373816C7342309F586439
4394546B6B3177AF9C478F4D459F7B4B97A44461B28C47666E553C994C3C615573B15C6F
6F7D5ABA874443946D5A3DC28F5672B9923B8FB07473A4563489966A51385BB46F809B38
84C261324E3E34944B4466856A7A34605577689B747942486A4836816A4271A9633A389B
3138545B4B524F3C442F5E6F363666698E68444AB07E41939873213E7B5630689B7F324C
BB6741977060358A5136B396452D9E32418A9460828A5E212D3D582F32587944
98C1598B456FAB5E8D6B354469653F682295704051206D8742944526488890243B2B5393
5827893E38746943446A56233F3F2B817F4362485FC99C355B963D64A2684050683249B0
72501F694731A69A473E503C7D3C7DAA3B46B1C16834A17B1B22357F65192D5141597A61
51403F226754564A3A7342476F4769595F5933256D704B8C68445874697C7E5578575A6B
478A5257696594B991438B6F3B9B734F94844036567B49686A2D269951474E1F52AB6475
67454AA5A33D439855412BAB936473AF93396A885E7B9F1B2DB8955E62292F936A828A3B
4C774B2766295DA95A3D476876622988A9A86AA15B8F315FC47947886A46BEC8873C3C93
17408C685D536948244D9E713136522B87823E5EC13946989A792F58931E28635C282D6D
B07350C49929206D61275E51272D9F4620544D317EA2582E3747551237808158
7EAF53A26974AD5E99837E4B8C7D45A33B997B42624D7197619090572A648A2A3A45396D
502481603243483438453627404430494B3B5A395D4F53297187242E6C553C47672A4385
423821693D1E636A3A4A4F365F3782AF2A2152685630B36C3A33298B58172D53326B926B
533E54347D60555E527D464980709A67735D3C327A795294694F6668626B7C5981584E60
568F52537681AAB07943955C43A8584F87725E4B90874D7675682B793C4743346FA64C85
5F6F4E9A68274A986E7638BD80677EA7A2418572519F91351F98735E5225477752898458
8481512B4D222574393B3D30495119444D7D54A46A8F26325463449F6E2C5BB05833549E
39264B622D5B7C652E2A5445234E5F2F65552A6B9E61398D783C1B4584521B54754B1A42
AC5156C1AC68244E5A4587541E269D39378049384F87582B46645E282A4F5032
BAB851A97184AD5899868451857B3A9D46A573406953709A6087956A4767832240565564
4A258C6930504D2F4058392642453B563A264E46894E542D7C8B44467E553F5B884062AB
473E34635A378978465363457E3F9FBC40316F92513DC57A3C40347A32354E60344BA473
3A705C328A3D6C5B5C7A445C846FA16F4A6B573B834E58895C5F79692760765698476E52
4B8027567D66C8B63E489E5644972D4D84805A5EAA683792B1692C702A654D378C842485
8D6E67A5524A849861622FBD533657719137868A3B73884735B7412F4933496A4CA5B267
84983F3669462A82253E63648E63345E476527728572404A6041276C713F98B652313C86
2A366968444659493C4D805C292A4F4B776C2F54A65A45ADB16D1E438734227D6D3A1D51
A97C71C8AB712E515C365A582E2891552F6552394E8D5425476A744F355F6644
B38D58B968878661BEC179507A6C4AA748946F4170496B965D8C99673E703A305B544A70
26308D5737523941435A2929623F376141195D4E7D533C197377384B6D232F5271316797
4B1F2E604239A1741C4F57467B2068A342326C923D31AC74354D4B6B2E111F4634307970
31483F2F8E45543D59853C35584B9771505431399052507242587A613D515C4C984C3F31
5C722E446346B4C273459036508D4351677D4B73AF723E5C665B3A792E49373C996A3570
546462A85B28489868763DBD8D5F61656F3A979D6C8E903C27AC785035325E8E4B79783B
8B9E3B2F5D29349737304E5554562E75747E5792507A314F84503B7C472F849F6C393F95
35264F44474A545F4D3565672E2F593659794062B0763F80966F1E4B84431B77795B2556
96767BC6AC7330616E3F59461B256C483370583A51775F1E3E6D774F34507959
BE954D976697804267646150772C4C8844802947723F905A1F78594254754F1C3D384278
321F7134294F37271F511D153A302F5656284B316177521D744D2651614A2A4B69235870
2A2E2C5A432A69682D4941375D419F992F42909D5535C35B2E4F4C695312264F4F29685C
3C5041318A565F5D5E7C463C846A945F635C3A3989666A865E5E675D4E4E795D8E57455C
5977486A765EB4AB6E44904E4997415B677C585B7F7C4F738030468B4B5B4846BD864895
5A666DA9733152A6413F2EA783585D6462358A876291862E34A67E4C3A30427A5E8B8C6A
8B904142451C2366363E5E59605A2351526B5E974C7C475465624273524C8EB57332438A
2A2F492D454B6B7E4C365F6B3930493A53722C5EA58751A29B72225894651D8279544398
C18160B1844F3FC4A24357452844AA56358381587D8E6832417169413B94A067
B5A2519453908D519780496D9B66506539A262475B2B84916D7C845B427671225267556D
3E409E58354C432D3246231D505741534C2E5E525B556525805B364B6A4D2950642F6B67
3D39306B583B686D344A51487348AE8229345B715834C44C2F4E59654734475D5A2A6C50
384F2B34874A5C4E637F4740866DA46C6655323E8E767A6B5F6E6B60495F7D5F7858455A
5A763F5A7D75C3A75F409B384A84376666655D649476568DA85E4873445C372C9B7042A6
6F3F669D72254EA5516339A595647B9368348DA071BB954A61927953473840866B8C9396
AD825A61612D266F484971676C5224554C55568D5E622A6282553D786C3B7ABE864E5291
443C47415593927E3C3D53573843534E4461337694744E988D553C92C371208F54042C88
BB7B508A8256226776395548252D9962367E59295CAD832F467B621D45967D53
B59B61BB71928569C4C05B4D6762919947A16D5F813C8F936D7E9A6449814C4D8C6D6A5E
15328F633C4C25563F5F2339845D4D523B327746635F54218052324C6655387C76467775
5C3A368A5F396261454F5C517353B58B403B62845C35CB5B3B5663522360668663287F71
5298343A88437A405B5F445B6B61A05C5951263F8E5983585B62685D3B6C7E648D7C8E40
555E339E9A70BB9B487BB3404C6E388D92835561985066CCD157495F5E963D44A76C4CC9
AD6B6A9B5E5992C4656F41C255588B9D5D2A8EA03FB4B47A85805269953B498B4CC3BC7D
A18B5D937A4C2873615C9C96915F305E45525D897A78375889474AB160377DA94A2A86B4
6F38394A217084853D415E351F6C9F6039792A9EB79450908B351B4788511E8F64242151
9C65365E5B6933717D324E544234875D32694C48609B6822437974462E847558
B6695E9A62985A3657884E52562056823F6530728A4094532F8691593E852D184E57615C
28237F4C3E45272725681D1A42434150332A6E41545E2F359169334D3C2D578A753E6A81
312D5C8365315F633F7467518E218CB8623E60853648C49F394B66502B14295F5B2B8459
2255453E803F89474C4E38506654815629555D37821D636552646E5B2773815E7D38533E
546615496C55AE94292C70405169344B5F78535D98432A54713E55435289594CA0461966
726271B339284077665652C4682E50783D23969B448A7631839C552D4D3A3966246A7E4C
808E245B7A4B22712748838894854966455E2C75A8965E608D2C33826644759542175D7B
372A372C425853632A43562E092F60452E661652957650878D671A467C2E19828A663572
94304C876C623C796232723C22336D313775664A5B715D23457C5B3D34908056
C29B509F639E9E5988AC555174586476398D523B622E9E6B2F6191624D9E723157576676
2C339739425C3C2C1655291B55253F654B2D6A42607B4419604E28564E402B735133765C
251D1F514A2A594619375036592D6A80343C60742235B4592F5781644A1E3F63372F9263
33534C528E3F6350606E313B62528C824D4E2E409365775A416F936F3B61455D9557582E
637D475050368795514294255672405A6D483B5D94706F7872327464447A524FB87E3298
66407AA266314692322B4FC9934C6D845A2DA4956DBE91395E505C4D5A292E4936867347
6D9B38547A1F1C69253A5D4F6F58395E4752424772744B4B774D3975523A769C6F377884
2832564B516E6A5E2F4C5343243B3D2C2B483181B1744A8C8F63214A782C1D7E8E663A80
B95A1C2B6B5A3E9A482C6F583A39752567A47746599443448DA74D2F3A946924
AF8B527F5DAA935E909E6872AC697D564AA058505033A08C376F4245528E5F364A325576
263F9B473A4C3531265A2A255E413A443F225A314A5353206E2E265D513C378A3F287E68
442E3A7246338B68444D584666509C7325427C805735C5342A3F7A563226496C5D229B52
2B61364A824B692558603D41473D527162533A317C67765C59655D6549787C5163616244
697146597E739E90385999374E4C449F91634944795C7FA6943958464F822D308D5B42C7
9758779C634574A2454D70D48665A6B4582FA37F5ACCA96872544C5D541D3A613F897923
5A85485E981C2376454077517B40246865615251918D2C3C9E573D734021478B74458375
40254E5854AE8D51234C79523142372D34513C8889537ACCB46A214B7511177B917052A9
BD6C2242671850C8842B47423965A44B377B4874A28E591F3862553A419D6E34
C592529452997C657A5F3F588872875E3E975A53702C9498346F945A4995635885627E4D
25719F4B3949282D1B57211E5A433F49442152407C786220795A3552372A60A985476765
4A2D3F78372C5364474E2D4C8051A1922753796F4B42B94F353F83503963768A6C1EBD51
498A3C537D4E7C3B63573B44554B56635657413F7D2E6C67555E555D2D6B6A525C797C35
68583E91855B7380226C922C3F322B406E733E3D752A7CA8A23A612B5DA0553A9D744D8A
716887C54D2F83A261528CD63C3D6988582AA49A8FDAAB698853432C64291F5536B3A271
7C7F2C92A6561B5D3171C5A8B25A2E52415C3983B6903E3C74313997572E479E392CA3A3
612F525135677967434E6E423A6F67282F4E36887034334D724E20557B291A7B68283A61
B67A25374A3E2C7D582D60381B387B335D90393B7E756A284B7E4F2B56A37E46
C69452AA59AC8671A8A45156694E8C71519D656D8734A796306990564C712B3A6B725754
1947934937521D542E6B193F774D3B4C432B4E36487A463C886E3557331D356960407765
2D5370795438535F43484F3D785199843D4464662462B85D3B428D4A23243F7D6F24B965
355F434D6F5E913E644B5D75594E584F335834357B5C545E565B536633666E4F825B522D
6B512C5A85527587305B922A374534496A6A2F3783667C629236533D44904D3792897297
876F7FB7842762BC60599CCC785485A1712C98A2B2C5AC637957623E55341F562F938F5B
807E3E458C3D1A53233C635B715F34574B5B1B527C8F5750741D27796B384B9120187649
2932564E5C6153523A4A5A3215587033373946B890626094AC3F3D80853B35A58C664773
BB7D2548564D3D87383F9E602D3A481F4071533E5B6C2E5A9C9C41232D87602B
C17488B756AB606884A1495E765D7C69635D4D997835AE77478789523C985344624A6366
2350A15039522D3A20601717514B35502B1B5C454B55311C635731594749396F5A45785F
44225C6E503B573F356249435C3696A55443636B3341AD3F4664926A2E37597B3847CA78
37603D567C5B7632704B2C5269365164526239338160647352555E78477E6A5894706738
70774D62845E80A33D64A03239573A40665D4B6B9B6A7F59862C6A4A4B9468369F8E9287
6A6694BB812B57AA705DBAB182528DAA5A33B4A280BBA250755A6147582B1D824A897A40
A8A55539953C1F6B605E894F72522B57445757636587276EBF642B69513394A87538805F
124395855B5B683A246B6930226568374833267B6D216C9CB04F1D4663141C8195634F73
B66535615C634372392C652D1A3E4C1E5C6B4D4C6C57292F4F7A5F3D39975B33
BE72509153AF81647B9C4B699C768D625D9F59515C329B46256F8753599A454B473B8867
1F4D892B5376432F1F622D1F50394E754228543F4A6C5C27815B2F604A453C543E4EAB62
4131445B493C704A293C4E5B6D2853813E70AD9A4C4CBD1F3038885B2E3E5659431F7371
2A5D3E706F5F532B8756375D594C8E894B594168B1616650478FA9824375417C7C70633B
857553634B44A39239608D36716F323C6838253579556B4536268D6648844B4DC3958188
602E75A97C406BA65645CAB476538C8F452484975AB8763DB6B66248542534A36B93763E
526A412F8C2B54B0605C964F7D4D3883938A6486759D3B48775E316D571C4B8479398478
3F2C6B7259555E4F164B82583D633A18774E338A7050416C9B4C21517044176F98594478
9B20177B8D68358853396F3A366A6D255C5E4864A083632A4571353747B68A64
C884527760B7847081694F8CA0627F24759C655A6339A68E2D59182D488F3942402A5173
21488C4133552F2B1F5D311F521525575E2E4B3F809366247745489C6D4141573A2E6961
512955720E3685653B47153B69466F843440676A3D569B33423F8C4B234B5C4F4C319550
3472226B825E4D176F59385A372A48623F612840814D57433D534C4E3D8059455088661B
6F786297555C6D7331958B284D3F345A5F5A2F2D684B7D56601E654B3F802A4E908B947E
7E6C759E702F62B74434A5A67C67B4A77433877350CEA041556A4A53571C275D4AA98D64
6B854C2D8D241F604D58A2548444244F4D73617B74A25C696D6032676E4A629F8043896F
532B817F55788D893443704B406859276934389271634A5E80503A81802C1957885B67C5
B78C197A832B37946F4BAA431E4B803F51692E438D95632F4786361538807D4E
B7715E9C4D946D85A572416D8D6C884467905558354797762F624E373D9633465B536274
2145985A4552282438612C214B2C383E3D294A1C3561582476142A61583E4A645F4B8F63
5528465A3F3B5657424A434A674A906D404871733F53952E454394352D5E6B785F55A835
2C89456E4E697C3B782A4E7969424042277A483F77285D594753594B2C705B46544A6B31
77553A62687F817E2469973C54492942766F33377A2B6A96902B6B3A60AA7144876185B4
8E6F62B75F3A88C87A4887772437749489328F793DACA15263691C4E85411E503CB8AA70
7993273E8A4526662E57AE98A46433584F724180A19F566C61583E70755060AB51399C8A
643A98832646777438519A332974792A772637A099695464672426577E4B215C732A326A
772D227B75432875382D7C4C31535A296E8E605C7B5E503553804C373A836B3C
C254628B509D475C6F82529B89498F50938268873F4680654E86883B3B6829716D4E6441
2B6D8D51464B1B3335421D385E2F414B2B335F3D47492F398141345A3D4D7D915F439252
41264F544B596A4959594754573A98934C517A6E235B9B3F464590542646544E28637353
32885178686B663C815036627B3E4250438B4752825064454889927A48845D6E7E687844
8989545D5B74AF822D7B9340797C403A6D53558197677E61772D6A4734876C4582655472
5A3F56AF3E2E5CB7633E7487504A909B6A3DBB9049B07157B8AD3E3A603C1D46299DA565
77891E2C7E4223601D3A65668172414F527413375FA56E80A93931755B388CBE50269460
292A7D7F514E71411A6A904D275057349E49318B753B90B1A14623535A292B5F8C4E3971
A7631A697D552C6C66376D282252502C3D6A597C9477407EB89E463543854E1B
CC775B6458B88070773A428A94536F3A8B7F4364415A81422E573941657D2351413D9365
254B663B77813927334E211C4C2452854D3D5C4467623F2A7C4A30623F2C485C573E984A
2822445E415C632D2B543C704F306D853E79A17D3C508C295B6D885E2455525A434E7758
347D2A6A5B64426999543E6B5550765C4379345974407C4539515F5E386F33584E6C5D2B
6C76505F2C4F866533785F357A5B3B39602D354867438853412F85523C875C749983869A
4A4A60B1814072C34443AEA56A3A7E906033BF874FBA804459704148602E5A823D7E8769
9798512A7B2B4C7D435A77517B402C77827B5270668E2C46614D326040296097672FAF72
51338F85564C546C2D5C7E46436338196A302D7972504D5C774222546A341A4C90503874
A96F2489806A32636839613C335455315058478792863A2D4B764B333B70613C
995B5A6A469072627D73407498567937738046451E5B8564405928233767204A37337065
275356303F5136213B58291E4C29273E4C33442D676D523488316994683B4F4D3A5DBE67
5822584A307788513948286E5C567B7B2F697E7B364B8E1E3B3C91571D5A564D4852623D
39892E6B526035397F4840575E494A5837763C56724A64454A6063573E6D4F77426D5E2F
727154543F7B88512A6E56378D4D3D3E4E393F4C5A43854B591F7832336D3F526A70877C
3F3864A7893466C253405D90643A7C897A42A7884BB69B53799C464444152F6B3876663F
766540277C222C684949675379512F4B46865C8461813D7161512F544F4493B87C43C069
573782875C4C5E69265F88644E68482A7E38267B8060656D77421B5763221F4C92635BB1
B170319867582E6D773D614F3B706E2F50453382C069533F598C47393D637F4D
C36D55915399845D7BB14C739F588C3E758B39564455986F3B738638347D1D39515C8277
285689554E54301E4C5220224A27374E452D46404A7450329041336659344D514137764D
3623474D38676E414540387543697D845054757432449237544796591B5C666E6452603A
3D9036735C5C4E5A9346425C67514E412D6F34606B4559474C5860664276588852686837
84825E52577E9061286F64379A673B4A5E5D464F75567E54742A75402E7A6172837B7C81
695B4D9A633857C86F4B77945C387091684C9F8351BB8B537CA9443F3E1A316134785F3E
A766342E78243364243E555E6B583A5A496B638C5D92527D6957284F374376B17E50C463
463A897E5E929574265C8E44406F4C2D944C2C7D9260936D743B2C85814E21496027558F
965D39A23E13318A685D9F4A29767A3E6A742874BE74595780AD703E62A87F38
CC66558E58A3827FA2B34C70A25C9240828E34603C7AA25F42838F433D791945685D8873
28668B50555335225553252A492F3F4B4D2A52445874523293483C6D5B2F475D563C8033
2720495243786A3151423A7837638B6C41597D5E2D47882B574A7F35294E556D414E761D
6E93386845A45A62864455766F554D4C3B7E435F643779664C596A4E529B538444918935
826D58856A8886502C988C41A0383868A08046516C2993B78D2887413B8F62747D678690
4679568E4D4647CF6251748D2D45B69E784DA2415DC99E72BBBB3D665C2E33643DAE8155
A87F214B7F353771176390A0814A2F654E493C93978C3F736760377F694F70AD2242A572
623EA4723258846B275AA83433886E2AA61C359B9B5B869A791D246B805249899247548C
5A1B4F7A774C2E5C13377364346651174E74507ECF5A1F3C6A876A2131693917
C85F7E804E864B698195455B5D63A33E9485396B3C847041628A79474C432382705E7442
408F8C565F452F3E5F3129506E3C474C27495E395848375B9A4A386B4C4A85875B409339
1D4F7A5E40746C30625A407D385D936D3D5E7B5638848E315347882C19534B4135575541
6D7E2C54276A605B7D296D7B5F53583531813E6349216D6347556D433D7A497529576B3C
94423B715185A0312A8D7C47AE2F2E3C6E485B8B46174B786D2891165EA1657077137693
6A575395374B7FCD94559AA1282D6C794D47BC753B9742328EB2313D70474A6334978A4F
B48B26336B294D78213A5075705336604A49195B6F87446A732B22615B3F6AA9746C817B
7056C18B5E5E5D4828879F4D3D683F3CAA3D3578693B9DC3A9481A6668152459784469B3
7330347A746430545A3B704A3874502F4D6B4475CC77403C5076523A409F6B2C
D455597F69BB85748BAB4776864E75388E6F6C92358A7340408D7C537151174B5274903B
225E3A49816C3518525A222544185B773D2F3F3D9A5C2E33783B4E782E27495E4A43A441
2B1E4D643A877022384F366F332D677D3C5E7E5021497E35515975521D5D57242F816643
82793668417D516D813759545764835C4375327C763E73274275737153733F8048603042
9E7861601E64915C2F80464596573E486C1D335C6E4160543F3FA13E397941888F4C6D6A
3C7192A95B3A44B64149809B57346F783B1C917D4E896443567C36505B226F7B558E7523
6E85403E76195F8B4C4D555A644D4987664148825A771D6B6E402D60403E95A690886946
1A66C2916568691C1B6492623C84283DA8583A985E194C5C74371B6565371D5C6B487DC3
843D3D91735E3E87725389453FAE6731717A3376D08453576A9256423CB37A3E
BF5C8B635DA18B9CA9515FAB9F657250AD543E513995833A556024384F611A40223D695D
2D785634404C2B205E6424284E333A42373131224051343A771B476B4C2751561E4A8750
48336052327D8E554B413885456C917F5290BA782B518822405167561C5A52442B454D47
7C6F246E4D642D5A8C3E547E2E3248544279274D603C6F55404C5F765A81457353627134
78705862529C8D4A317E5C359A5752687C5C2F686E529C9A551C7D5240752B477A679571
3E3E408A723945B85F4A707F42366C705D2AAB6263A36D5593AA4860602C28544A9F745F
82682D3D692D5475385361676B1A387F785750915E80258DA943326B3845AEA177887554
3C4CBD6B52BBA9571E4F8845377E533582513F9E9F4C828283331B6B613E2567503399BC
5A2B51914A27345F2A4A823F206045287E822866C495655C7AB06B3561C07E41
BD516D756388578793734566657E7833855B4B59236F7D355592762E4369256E6E776D5C
4F947E4A434B321D675B203248334D4E3F374B294D55363E923A3668572A53733C4D8E39
283955613853614056382852367A8671293F56581F5A7E293955593B376966644272723A
60782C654698715D8443647F5E5E52654A8B3B695D45927E435179716A864D7F5D958D3A
84576F9158C28B364C926C3C98314D66927126635B2A52776B2A9753607D4A608C75AD7C
4E6158B3954D5DCB6B61818D3C426E7C4433B6764D7D3E58927C2D717236436747AC8B5E
A8812C577943887D24618A9A7C52426644474DA18A921A4662573A7E291C547D56839175
4C83D1753C53624E21629B3238854E3D8F4B66BC9C489AAC9A312F82573947805D3684CE
613239706263335B443F774A3775331B5E643046B87D4773B1B0562028894421
C956887A8CB15C638472659F60567B39885B6B8233856744788C673651351F524F6F562C
3C6D634B4D4131266D2D234851284B561C425126543E306E9B3C386945355D8C4859863A
373C566946655E3F6A49386C3977AB683C696F563F8E81364560673C1B52655B41948364
447436642E7D5C576A357368527686504F7A42823C2D77464577973D5A6E597D3E557359
9A54416141A798452D84585BAD4B394F793F24706D4567604B4291385B784A796B2C7569
4B579DB5593766AC5E82B9A5473360875731B4774F7B5D456E522E4240386D743D9F7445
A86B2A284D36A6663A3B52536431479589542C65798130849A3831744D40968F4F77506B
5C69C77E69AD9B4E246A8437216C41537A2F338B572E7A8A8238195E4B33215B3B3DA0C6
5C26378E303176733741753E38B23F225A702D3FB7945C616688372A62AE783C
C356866D67AD5A678360527F716D78619E385675348B38365E85736B83621D50387B8B4A
3E8849526D603A145F4E1D2B381A5D682D383849824C25427B2F396027365F6E39628033
26516153366C642C4E442781355391623A65614B1F5A5E3145715C421E5E6547238C714E
386434713F67427681345A453056606B446D2D5F49357463364C62617F5B4D8754524563
A46D54532D8D8F4739824E3B8F4D404D672C1454563D7F74482D8243475A386855449A45
2635726052365CB7435D8B95453F8FA35D38BD7958A675518A522D475230366F50947A4C
AD662F27453EAE48434D746365524582714B5172537E296772442B7F5639808673858D93
7D3EC85B398B8B662E61824D3C77533E45473CA18B3E747E7A392C9360412E5033226CC6
4724366D4A2F22544242622C36AD542351691723A68E558081883E1D447D6735
C155977E9ABD5A789A955A9F8A69764383446459429E54435F533D3B505B1E443D5F763F
3C8656463C3B40246659182B4531404A212E2D345B441E547A285F6940274A5A3A7F933E
37394E32358467385A39275B33798855449B8A4B2267442A5A7451352569745323945C37
3B702B5F3B54435D6D2D513E3D5354664D77345426327257384C4F587654767E5A5F6E69
98544F3B29C686323D7D4345903F465C6A3833696D3C8E8C4F388C4059593F6C4949AB47
46599956493D67BA68708AA65469CBBA7437CB5287B26866885C2E45633D4D6A43945C5C
AA473032504DBE413785C4CF7B46487E71405076578432805B41288E5A5178702E6F6B88
803DB5705973705C2C596E2133785B3D75366BC28A45677B7E33367A603C2F41313E7BCE
6726354F5A452B6D64425331218E2A194F5C3B38918F48807A794C3261927B34
BC60B76E8D95518BAF8B619987718054A5425A603A965E4565916942555B1D4D536F6938
3A94694941314723635A1B2F472A444E2B313B31493B225A963535603D234A4A26617B59
3E414E4A4E78613C5B353260277D85622B545E59236C4A3643744928264E644823A64A2B
4A6B2E63496947627A2D51444A4F59585E80355923416D4F35495648795B8D655B776C80
944C6F5635C98C3C428C533C8C3B47596E362F72695067744A388F468D8647744D51CB7B
516996473A4094C35C7491953769C78F4C37C45E5888666179505D526243586A44A45F6F
B74E24565249A12C2C3D596270564B7178484AAAB382379C5D344EA25341628250475189
8045854D6586816F2B585D522F62633456323B9F803C5A5F853B1D6E704129444D418DD0
761F347C77473064463E48252F99342250746D3E8B8B458B766F382188B08231
B252A37896AD5466A47E617346898053924E8C764F9D46628A8E603F5E581F5F53636A32
3EA35F47402C5736712D2C5A66313D5025484432463E28859A3336664121515F306E7C53
3F4A67634F8452365D443C681E72A4582D6F773E2D8C533943733D2B254D7B5326945732
3C6A2B5B356736616128533B3353465A59673150373E6645314969667943736E6F5C737B
6B4B504E3ECC8D415D9D5E3A864C47516020237F634B6A803E387C44617140732932A177
5B739B622151AEBA678E8694548AD18E5344C4707998645D785439776F45538078D5A86B
BC7B26463A458D2A3B4D7079593943605C2F315B657E378C54392E7B3247566E52484E55
5640573D6C9074672C546667337566284F595AC58536474F8C46257E65372544424F8ECB
6A1B338B80503B6A4A3942313982573E417F8238808C41938D86402A98B47931
C657B875B5B9597CBA816D97787F7771873B65666B9746366A885241622F3E90435E5622
4B92503E434259234E5218353B2B41461D36273B49351A63732435602E347667327C7B43
2D4653553D76594164474E65407CA45828726B672471403A4078512B2C4C7F584D8E4C37
476F326749502D696D293B262E5B5B586A5A30593C405336426B7461783985806F503F6A
614C572B37D37A34477E41367E4750475D1F21956B4C6570393F8546767E3F70484BAF61
5C689D6956439EAF5F8D73994894D0914F6BBC6F878F4F5472483444532A40414CAC765A
A95F335254528C46485875846841465A553F407D816B278145444092454C5D863C4E4A4D
555A48446EA87F5E234354472D6A44245B605BC29D4032318445277A63342948445085C9
7024478D83574A726B43545947855C3B4F91743A83763882857F6B4AA7B63F28
AE67C45281A3618C934470995E7E5679A44A6B5E66A76546707C4F587F51244D30775C25
3A913B4B5041491B4C541C3C4A2947461D3B2535542D196B68233551421D5C642F80763F
283D445040784E416C373367337899612E7467522678463C537D3E253A5583464367513F
53674A8049554784832A4B3D2E4D634C7053305449435C39344365527A48577661596D3E
734F553D4AD1813D4C7F4F31895D50436F3F32CA6B574E63395F8F447D5A41815E4BA861
3780C86B5046A2A260899D8155A0CF993964B5317EB147538641405551212C5F51AA594D
A041234C48407D333A6C707A544F485B5A484E92716831744D494987393D568939494E55
41603F366BB8794E255E564D30552D2A3D4C55C89B45422970392585512C3C5D2E3E73BD
4E19415B4D395E755E7D885241924F3678AA7D439B8F3F7B7091745DB9BD8239
9B56B062856368A4B56570754A976C6D725A81527BA16E457A65363A67493E61274F522B
53A22D344633471B513F31645C245A5D2A5935426A451F786426515D3E1D5F492D948C34
26393B3D3A8653455927396C38709C483B987B2F23852E334669221D3F5E95536C9F3C35
806C365F3F7C3B695A2C4F494D59453D7D60275D2251714F3138354291556861525B6B2B
6841534347D370394E875458A151584E792B31CC6D498B8E3641753D8956367B3041B770
3B456D4E3257BABA4D61743E3983C38A407AB8655F7F5A4B6E384E7D552546675DC1664C
AB58216245295A2D356E7A8D4F224576643F4A8B707B36A587554394513687913860773E
233D313556C48A3833846B64306A3F3C3E4055C78838897A632A298F3B1B27581C2F84BA
6B375A6F28224F6B3D4553253C984D24747F3147C19D458E77875935C6BC873A
B95EC4608B945559A45982692B6F657D443D7041775F5C6F9E954443601C437E41614219
558B344A50254926552417364228414D164B3C2F3F1F196E732634563B2B754B184C6954
353E4D3D3360483B6B3F3B622E8BA549275A4F212D884B3F417D2B1B354C60459CC76841
74633C4C336C4075421B4F4054543D2F63522E4E1C3F593F27302F387042584B38676727
5E216B5443D3732D81A245376A1E50677D4944D4572E5A853B44602F79603D81343D7452
4A61A65531397DA76482788B508EC98239A2BE5A58875A9BAA5D2B3C65193B4157B47A4D
AE5D19544D2F422E1F59839B583A434D4A1F4DAD9E50215B553F55A94227667A1E536658
3F5624355495815B294C63292C6638345E4461D0882F5052491E369D58305384513B63BC
3D206660543C63543A42592134792A2BA187494AC1835BD2BD8724179CAC7A37
C560B27DC2A6685AA46DB2C3667C607D8E557A40866640387C8A3E37704E3F784B5D381F
457F295A5B3F46194A481F314729454B23473A2D54431773651745361A33776D2B868435
294C6D683F70401F444E3F75274F9B6430796F41246E404C6B82792F4246644160744F48
63627E60435D3C7E511F4136545B524F6B4F355233536C45364942507646A57D4A625A43
58314E363DD46F2750904A3C843A6F506C277AB5795747774872814471684A935E5A857D
655BB8776C528CA770A09F9A5591B9894CADB2655989475C945C42545B1B3C57599E7E4F
AB6F2D644E2D30443257586D57384F494A2A395B84643D61521E4B86543869883B4F5162
4662366A69938C434BBD79441D60582F811546AC8E4A6B90793829903E184473313160B9
5F223E5E4F36724C1D6D7F4A3B761C1C7B7A5269CD642A7FA08A3F17759D8236
905A8B5274855352554283884C626AB585527047A2925C43825A4D66824842604E914C21
467248847734342D5D3E243433348D7D2147385C854019874E2687533B244A5F2281844A
30474C51429B6837402B45794575AD3E4B93895F2A792732404E471F46426D518296593C
4D365761414841BC7D2658334F927152704F6475414B65354D64584179457F7040714442
7A3654333DD082475FA16668B55E705D753477647C5B48582D6C7F42793A6CA2716A7C78
486AB690675A7C925FB1A58E5185A59546D8AF5E56845F806C364B464D1A43656AA46457
B87E2D574540505C33645B704D398DAA635A538185653D97623457894D3F72863B505637
162223336194662E1F59574C25643C3D9055429351309391713E288C552646642B2F67C3
7B2C5385634B6E4C31424C383E742F2896915B6DCF9F459468604D40A2C37829
AC58C2748F994E539D5D8F9E4B643D6B62584C2F66664C3A8464292B6237395630714C1A
4268233D3F2C2A1C452D1C282E2247522E3C2937594D198453185254271B2D4044AF994E
326248252D58472D23274578416783462461563B2A6F2239436E2B134D4777587C7B4A3F
514A7A5B403D2C695D1C5240475E4B395F47455B3040615A41575C325B4F8F7B49505370
7C3A565235B57F45688F376EA15667587D45786F7F664669387B533A696053954B6C786E
42347B4C3353777E5FA79A7F4C84AA883CDFA557687663A875394C39454E6967649F5E69
C583275A2D1C33392D5B60534A305F494C4F5682876432978A484E98425387973F645A34
29291A1B49926D5928926040244E32264F4343C17C487272745E2A9667224F6332307ACE
84346190665D7563654D4C463F8D2F2A9F9B5A97DBA549BCB27E382381D0AE6A
AD69C68C95956074B15790AF4D834A93775D83458C9A4C35909239488A3A41704F955C1C
4F783E63563D2D2A60391C3442405360313B3F485F611D8B6C1E5C711F172E5820566559
355C45593C4648293147417F4C6889452F695A382F6B1A3D466E22377E867B4C7F7A5777
8E57885A4D53397D5C234E4182774B3F5C385954314A5743546164556641887A70534A74
5354824E34CB8A5EA8A96258A5436F7265307A887862618336804B396265589738687C72
5868A046434C849284AB974C6CA5B08A4DE1AD36999A6AB19C5747443D5CBA97569B453F
A76523682C2C4420326676633F3B7F4A35325B7E87563F71684144845742827F3D535A4F
4B2C283655AF936B3FA16A432B4C4439433C37B68A3F7D7A6E4D259F442A52551B47A1C3
6F3A80A9446487684B505921338A4C2FAC812E7FC36B38A8AC81492D55B5C578
9488C4709D5F7191994B736C52B34B7B5F9E9A43AC9747429C8F3F547D357D8657934628
796E3B6352314A2B43253F784D2C51581C4F3849623232A4691B4F5B31346C6938887D67
3787806B353B3C2E624B3C643792AD634157512A5296443A568733174E484D4A8CA94E48
784BA0647C79428F46397153849153326C397646387C65437F9838627248A06A605E3698
4832563538C786396C945388AE294D896B3A9EAF44549B883A8F5869988F529E478AC59F
6970A3483870999F8EB7AD69528C9B4058B6903A628259BFAD4E7B785C244D7F5B967C73
B66C3AA4533F5724588EA9925A408C4732257AC1B1404E604335567E4E61A0744475825F
5B35362A39AD9B7961B390433B6255394E3C40BC8B367E836C4237A65237728B392E6FBD
2C1A527E49483447183F824D3677172184874B8DD33F2375806330253555A53E
A66D8C71B9896B67794EACAA5AA1417D42757F43BA5E5560735F40626F28545B458E3F20
6957478F59332B313831224428317F5C223F28697E30197D441854422535546631898324
1F5C4E504162363632234A5A356CAD4D628D6C2B276D253C5D7F58196353493550675459
64266A60654A208C631F561549676B4C61234D703F54422337585E5653396C6E6A431C9B
6E415931299594476E8F283F804E5C64722E655C5E415D35278E4D3D633D6AA746667654
3578A28B4A4F8F4243678B6A5C9B938B7FA384447569236F925C355D59297A49639C6776
B4412A815E5B995330566F665D49AF84423A4655855B58754C226080486CAC6822466145
5B7C6E5B61A97F3173C98E4724656B3D7F2468B77F4BB3C5853D1D8F3217425C3A3986CC
6321497B565F36573C2B554C3866291F85915B90DA7A3F9E937B392E6E7C8749
9B8E974E8782726340337B895F69399155704D368E77432F4C2B2F47772E622E245B4629
744A254A4D3326324425226424173A51363C1735574820A724126170282E42311C576643
1D683910357845423E104269416CA1172D3E4B32286E152C376F1C186555615458625460
584585457163357D521F614E72523335713A60633D547F4742596A67774F85686D523BA4
633B6F7A3D90863F72A15353905874598547716C64425B4F28754A457246396E5B867B6C
3C5C8256344C895B7F8B9D6E44BE9E8090B1904067765967814B454B2D4AA74B558F375C
9D522B952B355F3035557F5F34326270576051769B374B957A3B597A19428D7B434F591D
38334935669F7B5245B56B503667274A8F5B38B24F19507172581D924323505B1A398BCF
7A284387555D3B60632E52603C713121857B4294D97740987F5A2E1F9B774755
8C729B6699899577924A8B9A669C3AA04B578540928C4C31997A31508F2E61884D804125
8065435D50322D2641261F4C3D334A5129462E49524621A54C1649582830285E376B5930
1A593C503C4A35382D20474F476C782D44403D192B611A313A6C1B2689725C46615A4E8E
5744912F715E2F834526574260511F37763B6C424B73653D3C4C377D674B774869432A8F
395B86542D9678539F99455A88418B8777465E6A58729657276C3C466E4C5198558F7774
537FB549324A8769848EA3604DA2A97694A88349A16B606E9F60415D4318525A6DA9424E
984B329D353F581A3E56895432357749213449819751507370426296585E9E8C454F5E43
53475D455FB093543EC45C4C43573F34484F38BE93475F707B741D7D42284C5F223C8ACF
6E334274506240655B3558483672241F756D4494CF7941926541421570747449
A084BB62875176908049878E73A3368D48739A4894825B578D6E385552317877417E303A
94584F5C4F3A442C362C41773831474D28483158735A22AA52194F582722225F39615843
1D5747574548373A352F4D6055919A3B4A3F421E4A7F26303C85192A4F56504B6256335F
553C912C8C663583353E6A495F4A2E396E326B463857683A3C504B64503F66665F482880
3A474E2E1D845D397F8A45738925626C49338E863649705327884C728F675E8E61AA9885
6B70B26E3D4FA37A8AACA7497BAA8843A3C97A328A5E5766A564446146296D6E6D986685
BF5C34A83F40662B5482B48640377C441A355781943E6C734F3657955C7CA67D4F65735F
6B496B3D70CB9D523EC65D5D484E41543A4236C59139626C72611C75412849711F49A4BA
613B5A89345D686E572C4B1C39823A25894B3AA4C06D3D816A585F1C4E8CD150
A384B080B485514E744A8B546F812E6842A37D3E883E3B396C5D346185206D83486E351C
674A66803C252E352313284B2B475E4F244B2C5875333DAE471952403A44625E37665F35
2F7C63504750273F403766573561923D473F4219355C2D2D3C7B2D195B60495C8E8A4E65
573B9A5074583E94442E6C4F6C52494F773E876846654741678956594A519B69795D2D9F
4E6755422387793D74782287A64886734D33457551536F2627AF3F4B7440769248A19D7D
538FCC6A2B4A6F3F6BA198505ABD88369E60733087313F9DAB4F7569352D6A5D72865078
BF4137A1493D75253F5065735034814E253F9AB1B544615F482F7D71407AAE4F4C846F4F
525C8B3A578B6B453DC3503C6E7854514F483BC58135617259372A83382E6C78213274BF
34489B823A3D2F4C1C276C54337012269C804394A83A3B846F42602F46368A37
9C8A6E5CAC724B473C5FB48E805F4EA646776F57A775423B5B4053738D236A3232713E28
86263558503A20454F1E3254182D55612B33255F893D1D84243474401D1C37532F6F5E22
1B4934374963343C26297A5F547570365D50411B3D6C203D4D73281F61611F486A6C4163
4D3F71316C30218E49335F1B5259524A5426786142634030476F4654443570686C283DA7
556D612B3CA9925078752D697D3975716C564B5A5C4E632D1C984E4B611573915B936C37
1674A469323B633F6781766164C2A96CCA537C3C8438294C843E3152353F7C62949D517A
C346318E2A5EA0333B538E683F3AA167212C616B9156546C5D2C4B5D3E81AD66465B591E
5677996D7A7D5B1B5FBD6A4A3F683E4E633245A773427D9D6D401B6633254C70343F83CA
4835787D464540542F245B4D377C2322767A4AA2B81E228098773A1D575A6733
8597A059A17769674D48657C835426954D652F47956D3D3C59342D56862B6A7C48622F21
8854444F442E2E43311C36502F343F5A2945314B653D24A84423495231315331368C713A
1C5429215A883E46321A6B7162917A225F6A5629426C1A3D4973332D7A774C585D5D408E
52496C21945F3F762D344F2C5B441A455C35765B497C643C43583E7857447E5B8F442D88
427456221E8B715291874069682D868669654E594F7E9A48258D43405F3D748652956C61
507FB8703A40595185837A64A3CBAC69CF5E5A45AE4540508A4B44521E46864D70861A62
A75B309D2336632B4467B07E26226F843C508680AD2F67AA703753712559A9804D595120
40567B638178722D489963494A571F4E655C36AE3D1D6C898C77216C582A4A8B314682CA
5A2E6B9D5B6260735D2762463DAA3B29995145BABD5B2646576C6639514E2215
8B809E5E74508BA46C4E6C7895872B8E43757957A66D3A44906A416B8232618C5A693038
9E534A5149453B473025506A2F403F532B553041582C39A2582A54302B3D6A4D1F4E6633
33854E2F414D295C3E2F616071A27B375340421A536D1A314366152850694D5D795A345C
4B406D2487603E7E254E613A5C4D195669397A3743775E453B4A295357416A4D5652368B
326B763F308C405199893D6F632263656B6545503E47733C218C2B65825982844EACB580
68A2BB694766685E8F7F623E70BE9B6AD2806936985640548C4171712E2E65579B795C96
B5313BA043345E193E749F80422E694A25528E84A7293D69552B5C5D1F59935B678F8E59
707CB05776BB94534CAF5A53514524483E493AC16B324F707E60307B3F3E697A143F8CA6
3E2C62811D455A6966347D363681293495623799A13C43878F77413F9DA8712A
958FA77BB57880856350654E97752A7648AC7C63985968718E62425B4645817457522021
7F3E45422B32352F201742512F3A42412251283A593B26A74F264A492C3A59632F6B5D33
25504C3A4E541A3D3D3564474384822E604C491E5A6C22445D6D362A6161498ABA72476A
436081438A6457863C395D3D5D4D3A5E663678503476683A3C59355C4A366B6B614E2D95
505A5E3D30786B3F8C913F7578509277786A575157447438218C3937764B7A744581827F
6194B742276C7457B0BC915895C16D5BAE906746AE454F718F3D35632D4076549C9165B9
AB443BA44036632B3E57986046397F632133A7BBA94550564528796C3F5BA047445C6C2D
688FA7626566722A41AB59255F73488869354AC17D5399A6542A1D682A295D66254579AB
284378623A4847512B2B71584CA83C278A6348B9AB4523445342563D548C692E
94957690A77F99A0645A8E7FA687378741786E63983F46417F5F535A5237696A655C3227
993F635F35322F3F301D4A4F2B4557552B612E4F543027A54A2D4B3D2D33545E32805938
245340304E542B4F2A246850709F8435736E61224F621C37476A32306461424A806F4375
463E73486B45457C413C5C346D5F426E514E884E44834A5B606A40762B4F8C6C69303CA8
5B69594569AC6A479070459E8052AF9A4D6165575973A3363A893C3B762F7B9A53A89E59
3CA7B9743D796C4B79697C5099CB825C79643B5B904F386C8346477627548B49A5864185
94544ABE32496B295F90BD99473981681F2F88B1A444847342246145296FA176757A7C36
435E873E7FC19D5549AC654B4B472F75753E3BBB5030575C73651A583523475D2A59A1BA
3A1F3E572C5C8480622B50153A9136217E1B2794A556243F59413757C2CE3720
8397895F8981BAA3478DB37AAB4D5C914B602E5E9B505B57453C4A55345A7F32345E292F
9527394D5134263F3D1D4D52173D535F3253263D5330349E374F7451283152533A9F7839
28573321626E324B2436935A64B86E235B4C4D205A5417244C692F426866594C8A553483
5B49683D89673B78334F5730454B1C695F346D304676502B29432764512D573B7B432973
337C652F3F9A3F57AD73327846347A7E626A4A47474B743F2272245D7E31627C4AA4B76F
386DAC604799865EAE73454094B584788B626B4891563E575D3D7F7F29396038AF94699D
A22958AF4D2D55164252746456328651111296D29F303E675439755E2F447944598C783D
364E844B5F64824B57C9542A706F3A73423062CB8E50656C745E21552F2B446F14376CA4
35253957262B405968276B443D7C3A25876D3DA1B057244264633A3E627E681E
738F94758255638A4B525048AB61587757826E5C96486771785848472B4474583C4C222B
784A48502B443D322327564A39564A292B592536461C3A8C3E345124334E7B421E4C643B
345D5632474E3056472B654F66BE72336753401B755B172051491F34424D554C83534364
494C6B4674653865274E5D3B42512D635536694C3F7C592A3765405F5139677271493279
324D6C404BA5423B8C78388D5C5D9677668A64484559723E2975313B874E6756297E9677
4C88B43338BA87539C7E7A509AB67D75BD78695D89674B6D7E484B7D253F84568D7A5DAB
8D3B48A83E2F572B6366966D52398855353F6DB0A34E41562F30706A3A537F5D6B636C3B
3755725F86789B614FCC5A32514E3A7939193DB782446B7932173A76542D436A264D76AA
3329436340484E585F38854C47803928876A41A4AC4A21404F49575298947C24
8F98708CA4877F74549B9B54A27A763D47875F6D7D3F4C729474646F4455695367733722
7D307461263F32351B1C4133294C5E3B2C5D294B512A37893A35502D1B33565B2A5B692B
33494B34767223353F3FAC4746947D438C894C1B584C21335B4736473F59585693644A5C
33728157722E5C703E59474083623471535F904D4260466B7D77456C2F607C7678315397
436E6C4977BB64419D7C42A160838D7825718E6B496E7333609237426F3E7B9E419C7E74
4A8CAD7275CE945AAD8E814EA0BB456FB9A675657751689264495C7B255A9E5B9E5A4EC9
913F4FA3405F6F30746E946D5153A7A325568AC8973D4957593F6F664552897270666D28
2F546E49897F866040D6683B6B573A6E4A4A6CC9773B738D5B2E285E1F2A4B5F225673A2
2C4E876938475452174975453F71244D7A5F3EA7A63C22434C4C5C567D566A47
8E9761699B7E7469404F6B5E8D4D93774772468E9C6050783339576636644E173F512F26
6F1333554F4F1E5C431D4239226E785131443B856E2D3D80376F6C341F39555B2B687141
353C2E103E573541232873576CAF6C1946635E1D643D153B79482B483E46377F936D4153
3B4B685066383D633A5B451E3E5F417441387F4540744F32465E3D5A3C41716C6523377D
4C68561D66B25E3CA74E4AB57C81906C4A715E473C7C6A273A7C3341602144793C8A7133
4BA2AB648DD37D55CCBA8350AAB34839597C606E6022639A5B4A694A275B9A52AA4A428D
7B414E92255B7133686685703634848D506D86D2943882704B4575613A4D8F7E6D667434
3484985D856C71345ADA4F2F6D524B9D5A5272C47754BCC95D352E7E1C41A6782857769C
1B47734F30434C4E5B36734E446734255F5335A3B2343075907B5D426A484B3B
8AA48C84B1715A7B546E996C84529C7A57582167864B497C5747415C3469764A87873725
733F4F5C4C452C483B1C403F263557552C4E2A45592C3D7E23355F461B3C562813317042
3230242253592C40193A7D4E6AB753216E57411E663111184E47214C3C401D696D643D50
44635B5D77543F36335F43394452235C533E703F406F564A4C7349512859796352274076
3E6970445DB94C3F9C4230A64A8B974A61A76D253192633B4A593353854F5C86438D674A
3A69995A9ADA3A2F858C5C5BB1BC6D7C56826F7571565885544F6C672849934EA57B70BF
964F4F932B404D1F625A6A4C4126826823527ED893284E765641744E3A759C66655B7E21
344F73647774703643BC6333524030684F4462C25B3260756043349C2B26445C314E7F9A
314D935C243F445060405C38568835326D272899A7401F3A45403F355F4A5635
789E7F80A6646096536DA1639E86BC7C4F785B709C5B4D909171526A34688C559B90382C
6C354A5D3D3F322B201D403B385163593654415875343E87463E583919335B5A27366339
333C392E6F63323C31407B5B63A0532376573F21642E151E7E35255E5848377F7960527D
3F6F496E96544941376C3F3A495128624D3B6A4D6D746271654859653A7D82516B3A4A6E
337D673D70BA3A53B44C5EC33F607550589E8A532E945D45724F2D5876556A864BA3826F
6B95AC57AACD865BA1924D5EBFB86599A392497E9E6073A87559747F36569F579F6E70C1
9346539F454B632F5851775B4C297E642B526BD8802B554F3E3676411F587968554C692F
3657725D7B878F6C36B888374A5F437A6C485FC58F5297BC8F594890332F57722F49759F
395F9169364C677F654F6F3C53823B48724C2585A74A2B4C5C3421418B8E6237
7A9D64A77E6D899C4F73AB63A58AC07457965B8E7439678B807F8C64246F854E964B2C4A
6D366155294C2F3B1D345A4E4B827E3A50513B697C2D4B733D434F2B183C5D6731386843
776D331F61794B4D38498F5D8DB9712771644C4293481E2F88491F453A473BA1AB683F4E
388A4E467652605A36854C446E6B286A4F587037534C4B59686E445F305E6B4152347078
2F5E5947ACB5384198452F945594746969745C473398532C766D325141338270417A8C72
9EB89651C7BE8753BAAD725BC3B0396D59644A6E72489DB5565D956D3D72815BAB4E46B1
82215E883C5161256D7EAB71613268583F6C63DD78377067462D5C3428547F6151496036
405D785A87A58D552B7C58224E47447D493563BB8B54AAB98F5C7CA855355D6D323E77A6
304FA773243F45663E8083465C9122357356238497273D7173543B295D724B47
7797557889505A4F6E978A679196C54355703C7C7C3D48703F444E6A224754376760293A
51305F6232371E42221A3E2932535F50393B45875E32786C3D465C322B627F6038566A2D
4B401F2978603C4E2046844168B45F389E8E3C35643215216F281C4F3440298366664A4B
346351446A39684B3372454D4451275E4434603859745E405C734D693A6A744C4D3E7681
475D6F4180AE373F924B41A859BF665179885B3D368744427B54306D78495C7B526E7F54
58A77E41C8A0444AAF9C545FD0A9529751514F6E613568863A4570323D818C64AB472DB7
954566882D7E641B474479684C2E525C3E5C5FDB7141747960335F3A39607C465F84843D
3E64783C706B8474366B63246F65497B4337639F8659B1B5935695B571375A75193E7B8B
3565CA8632384260415C745368A532316546237B9B3B2C4747644829566D422F
7BB67399924E5A884F5190667774BA7262885E829E4B478E8D594D742248784A638F333E
50394C482944251F201E47343F5C694E403E2F55492365392A5E702C1B43664926475730
41422228655E3C411A4184487D9A541F5B563134662F1420781A1A50364233A866634345
3A6C4243713F58332E6A457A4F3B1B5643394D2F615F5D435958575F315C5E4F46217886
4D5A644196983142792D479C3EB45A3C837B5E29448D313F89382A74804F5E7A5B6B8B75
71A9804BCA975860A78F4B67D9AE5995536E5F776B54829347578267396C91709B5460A6
812E5B8B274F5F3245457757452E4F61232F4FD06F3D7077473F746C567397514B5E5C2C
518A7A567852675948936C27506045793F2971AC8F59AC9B3D2E99B36D4F918B34346F79
1B349E772F324A6155587A4C5CAA34326048217EA54136505B6C5026546C6B33
6C9C6B9565769A984E54A26EB687C974567760A09F4853949260598024548440668D354D
533A4C4C33512930311D4B4442606D57413E2A5E542F75462B3D572A1B4C61653956512E
575339407B684448324B7258969B5A286B5D3637763D1925792239755A5135B66B467271
37654D4B76364D4838714F7D543E1F5B41424D2C616A4E4A524A5E65285B685653286E6F
4461553F8E9431468830479033B9564A85745D364C9A4440844430857C54778E5C709D73
8DB29D57CAB9847BAFA24170CBAD5379546F53AC9B588BAB50538862446E8678BD6D6FA7
75346F8035664D223A47765D2C297E91356449CA5531706947306D6F53ABA472515E6326
4560665E79667C46297A5C2B56474884322858986E5EBFB37B48C1982938627C2A3E7494
26337B75343E57695167764345B03434755722748136365A4634352758656326
7BA972A88D4C536647567A8AA96CBD6B889851A55C428880674D55734090845D663E3A72
5D3A4A48365A31372E43814F3D5F68516E482B6265429E5D3B3E5A393E806F652F755A23
6C7A3A41616B5F653A4E7960A5A672296C6250619C591C3188201E49383A38B558563B50
2F6531736F30494055925B65514C3784444A47518E784744522D7F802A5A4A757F33726B
5886663B879039729E4B4D8635B9966D7170612B8BA85C3F825457AD7E5A7F8A82A9A277
89B38C64B4A17A84A5AF6957D8A4537A544D2E7C7F4C8E9B44588F6047753D619A7469A7
86217E8E415A4D255C89C07F472A3958305D44B357457A66523371282E725C5B66608640
4050752E659C8F693FA76C32673E39815A3E70A42C4D7F708D55CC856A34527C24489D98
222669552F73A5774C6C693F5EB5362C791B237A87383A6A3F2232407D5A5A2C
739659B68E4F587F476E626580648B30578A45A2783A568367576F602561797089742563
4D37584630523A3F26224D393A7162315A3831602C38795532465120215B60583A815529
48442242593547622D596D387CA368317E7133427946265C72331E52473D39BE675E5050
2D61354E60264C463276494D7C49306C3C5A4D2A695B525966465D5C2D665C5C4E298444
457B5F4CA46B3B797E436C8042A25D3F7F947242587C344E9C37407E5C6D83456D686563
90B56B4782804C8BC7B75859D27D368D9253418C65519D8635629052437F687992565DB0
6C2D725D3A5A4C212C395D6957364456233C41B27F498E7747568A6A6B8863356A827232
455E6964625F667345B959459E6D46574D40999A637477586263D8B16F3C5A7E2331767B
1D55B8783333513D228F7923409D25579A561F6D7B2552BA7B3B25478360562D
6BA1537C775E583E5E867B6F4E6D9E466A4B4498723A6B4330687667286129476D5B2473
304278644742434E342A51112D6C7B406319458C6D338B252B7980322458371742A27D34
4F43155F89645141145587609CB62D2E5F86413E75101254663F215A5A1520978E5F5353
2E915754332A6F733B7A14367674424E186778367014345679465E3A237B7A57471F9077
595C2156BBA3418D671F5C7B4FA55C22574B533681721C378A57497D1B439579936A331E
419B9754877C235C877C4B89DB9B49504963429A495AAD86494587514D88749A994C6CC4
70347E47418A582F40326365494A4C6C3348267966426F5A283978635E6F5E4C52605D2B
5A8A5E6E6562535161B74D23554A6C832A1A7C7D5B8CBC8A3F5DAB5A425883A34E418376
133F9A7A333B55421C5F82404A95153E9252197C671031676B70462A54361A38
71B15CA55C8C93733E617F786D519F457C7F3D978E426E8562545C702E85755A6A702581
68364955384B393D2E2646302B427E436F332E5C593884311C3F582524553D303C885D27
4C3819325B4C4A3E1B437A63A19B50244D66384494421B6A55291D5C4D4E449A6A525059
306C445560294551347A4E545B61326E2B585A39766232446C4A69632F61655347298A62
5F513744A6782B785E31535C37A76F617F5743399183443B85404F9761488C72967A8764
58B38C628E8D658EA37F2367CB8E495D686C4EA273446D8B45415635628C6FA48D3B539F
65318B56265165423E3A6D701F2D6285476A30575945817A4A3E867277915A5656694C1A
405C73668198432946A15C3673242E4D472C896B214950687B7BAA58453148955C5192A9
203CAC7744779B7A3F5E7B3D59A73D2FA7401F84873A31744C373D70A04C3126
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.511614 0.0119048 0.97619 0.244193 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 0.003125 0 0.003125 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 .5 scale
320 string
320 160 8 [320 0 0 160 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage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P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.255807 0.488386 0.488095 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 0.003125 0 0.003125 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 .5 scale
320 string
320 160 8 [320 0 0 160 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage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P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.511614 0.255807 0.97619 0.488095 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 0.003125 0 0.003125 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 .5 scale
320 string
320 160 8 [320 0 0 160 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage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P
% End of image
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{600, 300},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40009H0001;2000`400?l00000o`00003ooooooooo
oooooe[ooooo003oooooooooooooF_ooool00?oooooooooooomJooooo`00oooooooooooooe[ooooo
003oooooooooooooF_ooool00?oooooooooooomJooooo`00oooooooooooooe[ooooo003ooooooooo
ooooF_ooool00?oooooooooooomJooooo`003_ooool0o`1SHf<0XZ:R04i>CP19BDT0PX:204Q8B015
ADD0V9RH04i>CP1OGel0SHf=05=CD`14A4@0LW9b03Ph>00e=CD0:b/[05aLG010@400KFe]05MGE`0U
9BD0HV9R03Lg=`1CDe<0M7Ad05MGE`0e=CD0@d=302l_;`1NGUh0?Shn04U9B@1ADE40@T9203Ph>01R
HV80:BTY07MgM`1BDU80B4Q806EUI@1KFe/0?Cdm06e]K@1aLG40>SXj05aLG018B4P0?3`l06ATI01;
Bd/0PH6106MWI`1nOWh0@41005aLG01IFET0:BTY071`L01GEeL0@d=3069RHP12@T80De=C08R8R01J
FUX0E5AD055AD@1IFET0NGUi079bLP0i>CT0/;2`07]kN`0d=3@0Lg=c0451@@2BTY80WYjN06=SH`0S
8b<0TI6A03Hf=P2>SXh0NGUi061PH00n?Sh0UIFE03<c<`29RHT0UiNG04m?C`1=CDd0De=C07AdM00U
9BD0U9BD08>3P`1DE5@0SXj>05]KF`1FEUH0D51@07MgM`1EEED0?clo07IfMP1gMgL0EUIF08J6QP1L
G5`0/k>c075aL@1eMGD0FUYJ07emO@2<S8`0F5QH06ATI01fMWH0BTY:079bLP1FEUH0/[:b03Xj>P1R
HV80O7al0451@@1KFe/0E5AD06EUI@17AdL0H61P0<O7a`0Y:BT0IFEU0820P00l?3`0Q8B408:2PP0L
71`0MGEe06m_K`0n?Sh0QhN70:NWY`0i>CT0CTi>07QhN017AdL0Q8B404I6AP1XJ6P0NWYj071`L01o
Ogl0QXJ605UIF@2BTY80[Jf]0:FUY@2DU9@0aLG507emO@1LG5`0Ogmo0:^[Z`14A4@0Kfm_0:2PX01D
E5@0R8R80<30`01[Jf/0D51@09ZJVP0o?cl0GUiN04U9B@1FEUH0SXj>04i>CP2KVi/0CDe=07moO`2Q
XJ40FEUI08j>SP11@D40D51@0<;2`P28R8P0=3@d08N7Q`1=CDd0Q8B408V9R@1CDe<0EeMG0<k>cP1g
MgL0`L710:JVYP1QHF40B4Q80:^[Z`2@T900PX:204e=C@2;Rh/0N7Qh05EEE@1HF5P0=SHf04A4A02A
TI40J6QX0410@01KFe/0MgMg034a<@1[Jf/0KVi^07alO014A4@0FUYJ06][J`1=CDd0XJ6Q07moO`0m
?Cd0C4a<05MGE`0k>c/0LW9b04e=C@2EUID0^KVi07EeM@0i>CT0JVYZ07QhN00R8R80>c/k06=SH`1G
EeL0=3@d09>CT`1@D500>c/k05EEE@2FUYH0C4a<04U9B@005`27QhL0N7Qh03`l?025QHD0GUiN03`l
?01TI6@0?clo0:j^[P1mOGd0@T9209RHV01kNg/0QXJ60;bl_01XJ6P0C4a<04i>CP1SHf<0HV9R02DU
9@1VIVH0V9RH00kooooo00X0Fe]K0:>SX`2o_kl0HV9R06QXJ01YJFT0MWIf0:VYZ@1nOWh0De=C0P0E
5AD06@0@41003Ph>00h>3P0[:b/0@41007IfMP24Q8@0P82002d];@0_;bl0;Rh^02d];@0U9BD07alO
01`L700Q8B40>CTi092@T01>CTh0BDU903<c<`0C4a<08b<S08R8R00T92@00P0E5AD0E@0O7al05QHF
02l_;`1RHV80LW9b02h^;P0T92@09bLW02d];@0i>CT0Jf][0=3@d01ADE40:2PX01`L700M7Ad0;bl_
04a<C02`/;00_Kfm09^KV`0j>SX0;Rh^03`l?00e=CD0=cLg04M7A`19BDT0H61P0:BTY028R8P0=3@d
03hn?P11@D40<c<c02h^;P0b<S80?clo096AT@2_[jl0Hf=S05MGE`1;Bd/0?3`l0<k>cP1VIVH0J6QX
02DU9@0B4Q803Ph>00l?3`0S8b<0G5aL09fMW@2ZZZX0L71`0:FUY@1;Bd/0?clo03Ti>@0h>3P0EEEE
04A4A01aLG40e]KF0<g=c@1kNg/0@D5103Lg=`1gMgL0KFe]06][J`0l?3`0=CDe03lo?`19BDT0N7Qh
05=CD`0i>CT0CTi>0<G5a@1^KVh0QhN70;>c/`2_[jl00P3Jf]X08@27QhL0Ogmo09jNWP2@T900YjNW
08J6QP1GEeL0C4a<059BDP0o?cl06QXJ01dM7@1<C4`0?Cdm03Xj>P1lO7`0IVIV038b<P2IVIT0:BTY
01LG5`0H61P05aLG01DE5@0M7Ad0>c/k06YZJP2CTi<0d=3@0=?Cd`25QHD0Hf=S08J6QP0208B4Q00U
0861P@27QhL0];Bd0;:b/P2ZZZX0G5aL07=cL`2PX:00PH6103dm?@0e=CD09RHV01`L700Q8B40@D51
06QXJ025QHD0Ti>C09bLW029RHT0W9bL05ADE02UYJD0gMgM0=[JfP38b<P0PX:207UiN@1[Jf/0Kfm_
079bLP1lO7`0Ph>308B4Q02e]KD0/k>c05IFEP0202DU9@0h03De=@1NGUh0820P01lO7`0M7Ad0:2PX
04Q8B01]KFd0Fe]K03@d=021PH40A4A403@d=00N7Qh0820P08^;R`2g]kL0b/[:0<c<c03Fe]H0d];B
059BDP0L71`06QXJ02`/;01mOGd0Ph>307=cL`0];Bd0MGEe03@d=00H61P07AdM02TY:@1iNGT0IFEU
08N7Q`29RHT0R8R80:^[Z`2ZZZX0^[Zj0:^[Z`2YZJT0Z:RX09^KV`27QhL0P82005MGE`0^;Rh0<S8b
04m?C`37alL0];Bd0:ZZZP1MGEd2051@D00D06YZJP28R8P0V9RH08j>SP2VYZH0PH6107YjNP2UYJD0
MgMg04A4A019BDT0AdM703lo?`1UIFD0ATI604M7A`1OGel0Q8B404A4A018B4P@ooooo`003_ooool0
n@1PH600VYZJ04m?C`17AdL0QHF504=3@`1?Cdl0QHF502d];@1UIFD0QhN7051@D017AdL0I6AT03Ti
>@12@T80BTY:08^;R`1<C4`0JFUY06i^KP0];Bd0MWIf03lo?`1XJ6P0OGem06ATI016ATH0A4A4034a
<@1jNWX0Bd];04i>CP19BDT0;2`/04E5A@1dM7@0:b/[08F5Q@1@D500?Cdm04i>CP1:BTX0>SXj08:2
PP1^KVh0?clo055AD@0m?Cd0=cLg075aL@15ADD0UIFE06m_K`0j>SX0:b/[07QhN01[Jf/0;2`/06EU
I@0e=CD0>SXj08F5Q@15ADD0E5AD09VIV@1DE5@0DU9B0861P@1RHV80R8R807QhN00J6QX0VIVI05=C
D`0o?cl0P82002@T901/K6`0WIfM06QXJ00Q8B40MgMg03Lg=`2KVi/0V9RH06m_K`0e=CD0`/;202XZ
:P1bLW80A4A401<C4`0^;Rh0?Shn05mOG`0P8200N7Qh03Hf=P0a<C40LW9b061PH011@D40=cLg08:2
PP1NGUh0:BTY05MGE`1<C4`0Jf][06m_K`0O7al0UIFE05aLG01mOGd0F5QH04Y:BP1JFUX0EEEE08j>
SP2PX:00DU9B04e=C@1FEUH0Xj>S05mOG`1EEED0B4Q803@d=01ZJVX0GemO0861P@0];Bd0IFEU0;:b
/P0o?cl0Kfm_071`L00l?3`0Xj>S08n?S`0k>c/0NGUi05MGE`0R8R80Cdm?09JFUP1VIVH0EeMG05aL
G00m?Cd0Ng]k05iNGP0j>SX0Fe]K07EeM@2UYJD0R8R805QHF01hN7P0VIVI05iNGP1ZJVX0XJ6Q05IF
EP1fMWH0Lg=c09RHV01DE5@0QXJ609VIV@1<C4`0PX:20<K6aP24Q8@0IfMW0:RXZ016ATH0EEEE03hn
?P0b<S80PX:2051@D024Q8@0<c<c07IfMP25QHD0EeMG07]kN`0f=SH0Kfm_0;fm_@1WIfL0=3@d08B4
Q01PH600GemO08B4Q01KFe/0HF5Q0<o?c`1_Kfl0M7Ad08B4Q028R8P0E5AD0:b/[02@T900TY:B04M7
A`1hN7P0Rh^;07AdM01PH600L71`05iNGP2/[:`0OGem04Y:BP1HF5P0SXj>05QHF01_Kfl0QHF508B4
Q01>CTh0GemO06YZJP1TI6@0]KFe08V9R@10@400GemO05ADE011@D40NWYj04];B`2:RXX0`l?307Mg
M`1HF5P0N7Qh07emO@0Q8B40>c/k06EUI@1<C4`0=cLg09FEU@0203Ti>@0C06EUI@2FUYH0A4A403Hf
=P1_Kfl0LG5a03`l?02UYJD0Ph>303@d=01FEUH0>CTi0:BTY026QXH0AdM709:BTP1fMWH0F5QH08b<
S00204=3@`0605UIF@1JFUX0Fe]K02d];@25QHD0YJFU3_ooool0302a/K40ZZZZ07IfMP0n?Sh0820P
024Q8@0b<S80T92@09>CT`1:BTX06ATI018B4P803Ph>0P0=3@d01`0m?Cd0F5QH09JFUP2QXJ40BTY:
0451@@0i>CT00P0j>SX0300m?Cd0CTi>06QXJ02][Jd0Kfm_0410@00l?3`0:RXZ01<C4`0];Bd0MWIf
01HF5P805ADE01l07AdM01HF5P0A4A404a<C03De=@1YJFT0DU9B038b<P0Z:RX0;Rh^069RHP2g]kL0
GEeM01XJ6P0L71`07alO02LW9`0k>c/0Ti>C0=GEe@34a<@0EEEE01lO7`0P8200:2PX02DU9@0R8R80
=3@d02PX:01/K6`0M7Ad0080:RXZ02T07alO02XZ:P17AdL0H61P04M7A`0S8b<0=SHf08B4Q01UIFD0
EUIF06IVIP2d];@0Y:BT08R8R01JFUX0:RXZ01<C4`13@d<0GEeM04M7A`1]KFd0Fe]K07emO@2YZJT0
CDe=04U9B@1EEED0MgMg09FEU@1ZJVX0F5QH0:BTY020P800Q8B402@T900L71`0820P07emO@1NGUh0
YJFU07IfMP0206UYJ@0Q09^KV`1RHV80ADE509RHV03EeMD0QXJ60:>SX`37alL0_[jn0<o?c`39bLT0
`/;20<[:bP3Bd]80WinO05iNGP0`<30071`L01/K6`0];Bd0Bd];01lO7`0G5aL07AdM059BDP2MWId0
V9RH07UiN@1:BTX0N7Qh01PH600J6QX05aLG00D05ADE04D0De=C0=;BdP2ATI40_kno059BDP0Q8B40
820P02PX:00g=cL0HF5Q0<?3``2k^k/0K6a/05eMG@0h>3P0TY:B07]kN`0L71`0<S8b03Hf=P0L71`0
8B4Q01dM7@0T92@0>CTi04a<C01dM7@0ZZZZ0=_Kf`2YZJT0][Jf0=kNgP2m_Kd0/k>c0;Rh^01jNWX0
Cdm?03dm?@0d=3@0<30`02TY:@0Z:RX0HF5Q0;fm_@1CDe<0B4Q8028R8P0/;2`0De=C03/k>`12@T80
EeMG04M7A`0b<S80<30`06ATI017AdL0MWIf05aLG01FEUH0?Shn04I6AP2KVi/0Zj^[08>3P`1NGUh0
Jf][0;Vi^@0m?Cd00P0E5AD0800L71`0@T9208n?S`1KFe/0@41006i^KP1QHF4071`L02<S8`19BDT0
O7al07inOP2IVIT0YZJV0:VYZ@33`l<0[Zj^0:FUY@22PX80U9BD09:BTP2TY:@0`L7108j>SP1kNg/0
JFUY09>CT`2n_[h0gmoO0=[JfP31`L40IVIV0P1IFET0501MGEd0EeMG065QH@24Q8@0^KVi0:f][@38
b<P0_kno06QXJ01ADE40Bd];01dM7@0S8b<09RHV069RHP1iNGT0QXJ608:2PP1KFe/07AdM4?ooool0
00kooooo01/0EUIF08R8R01ADE40@D5104e=C@0/;2`0De=C06e]K@0`<300BTY:06MWI`1:BTX0EEEE
03/k>`0G5aL0Cdm?04];B`1EEED0;2`/08b<S01gMgL0;Bd]06EUI@0U9BD0GUiN08N7Q`1fMWH00P1B
DU80>@0X:2P0HV9R04];B`1UIFD0ADE501@D500H61P0EEEE02<S8`26QXH05aLG03Hf=P0b<S80ATI6
01@D501QHF40I6AT03Ph>01]KFd0Lg=c02PX:01FEUH0>c/k09^KV`2LW9`0@41001`L701DE5@0AdM7
04m?C`1oOgl0?3`l01lO7`17AdL0>3Ph07UiN@2f][H0FUYJ02/[:`1KFe/0@T9207moO`21PH40>c/k
0:f][@1EEED09BDU06a/K00_;bl0M7Ad08R8R00k>c/0BTY:08Z:RP0W9bL0MGEe08J6QP0203@d=02n
0:JVYP0d=3@0P82004a<C00T92@0:RXZ03Lg=`1MGEd07AdM075aL@14A4@0:b/[05iNGP0X:2P0CTi>
02h^;P1gMgL0IFEU0492@P1PH600CTi>04E5A@2:RXX0GUiN08V9R@1CDe<0EEEE05mOG`1EEED0Jf][
05=CD`1]KFd0MWIf06][J`1FEUH0EEEE09VIV@1KFe/0FEUI06YZJP1@D500J6QX075aL@1oOgl0DE5A
06YZJP34a<@0>SXj071`L01_Kfl0?3`l09bLW02=SHd0?Cdm075aL@1kNg/0<30`051@D02RXZ80Jf][
05ADE022PX80@d=30820P01iNGT0GUiN071`L017AdL0TI6A09RHV01cLg<0T92@09FEU@2;Rh/0NWYj
09JFUP16ATH0K6a/079bLP2NWYh0F5QH092@T025QHD0ATI609RHV03Fe]H0Ti>C06i^KP2^[Zh0D51@
07AdM01QHF40>SXj071`L01ADE40Ng]k03Ti>@22PX80Rh^;06=SH`26QXH0?3`l09fMW@2[Zj/0?3`l
02d];@2HV9P0IVIV06ATI01GEeL0<C4a06i^KP3<c<`0Ph>30:6QX@28R8P0HF5Q0492@P34a<@0YJFU
08n?S`1>CTh0O7al09FEU@1^KVh0EEEE07]kN`1NGUh0YZJV08R8R018B4P0GemO08b<S01ADE40MWIf
09JFUP1mOGd0Cdm?07]kN`1`L700GUiN09^KV`29RHT0BDU905]KF`1TI6@0BTY:07]kN`1SHf<0QXJ6
0<;2`P1`L700FEUI06e]K@1nOWh08B4Q03lo?`1iNGT0EEEE03@d=02CTi<0?clo034a<@2VYZH0XZ:R
06MWI`0d=3@0M7Ad02l_;`0];Bd0U9BD08^;R`0`<300FUYJ03`l?02LW9`0O7al04U9B@2BTY80L71`
06QXJ02YZJT0BTY:04U9B@2DU9@0Jf][04E5A@0g=cL0UiNG09VIV@kooooo00h0[Jf]0:NWY`1RHV80
Jf][04M7A`1RHV80ZZZZ08B4Q01JFUX0NWYj04M7A`0W9bL05ADE018B4P803@d=00H0?clo06UYJ@10
@400FUYJ018B4P0=3@d300h>3P0A010@400C4a<07alO08b<S01BDU80;Bd]03`l?00Z:RX07AdM06IV
IP1?Cdl051@D01<C4`0A4A406QXJ01dM7@0C4a<00P0A4A40BP0T92@0F5QH07]kN`1VIVH0=3@d05MG
E`22PX80TY:B01dM7@0F5QH08B4Q02LW9`0j>SX0MWIf0=CDe03EeMD0`/;20861P@11@D408b<S01hN
7P0O7al0<c<c03lo?`23Ph<0Ogmo034a<@0g=cL0AdM705EEE@15ADD07alO01LG5`0E5AD05aLG02PX
:01TI6@0EeMG05eMG@2FUYH0Rh^;05aLG00[:b/0QHF508R8R011@D407QhN01dM7@0X:2P0820P051@
D02][Jd0KVi^06QXJ01^KVh0T92@0<S8b031`L40GEeM071`L00W9bL0BDU905MGE`0U9BD06ATI06EU
I@2m_Kd0][Jf09>CT`1NGUh0FUYJ06ATI01aLG40Ti>C0P3>c/h03@2?Shl0RXZ:0:BTY02_[jl0P820
065QH@1KFe/0Ph>30:BTY032`/80<C4a01LG5`0D51@00P0I6AT02`0k>c/0ADE501dM7@0E5AD0>SXj
0;Jf]P2IVIT0QXJ608:2PP0h>3P05QHF00805ADE00<04a<C01DE5@0E5AD00P0E5AD09P1:BTX0Ng]k
07MgM`2][Jd0O7al06i^KP1ADE40=cLg04I6AP0g=cL0PH610;Fe]@1OGel0<S8b04Q8B035aLD0V9RH
01LG5`0M7Ad0=3@d02PX:00V9RH07AdM02HV9P0h>3P0CDe=051@D027QhL0c/k>0<c<c03Zj^X0jNWY
09RHV02[Zj/0MGEe06YZJP1fMWH0KVi^0P1bLW807@28R8P0U9BD0<_;b`3Ng]h0RXZ:04m?C`1FEUH0
IFEU02XZ:P15ADD0MWIf079bLP0V9RH0:2PX02/[:`1aLG40OWin04m?C`1>CTh0O7al08V9R@2ATI40
WIfM04U9B@0o?cl0IfMW05YJFP1<C4`0FEUI00805ADE02T061PH02XZ:P2BTY80E5AD05]KF`17AdL0
`L7108^;R`2:RXX0PH6109BDU01[Jf/0D51@0492@P1kNg/0/;2`04a<C01XJ6P0?Shn04];B`1BDU80
@41003@d=00g=cL0Cdm?07]kN`3Ogml0e=CD0:>SX`2j^[X0f=SH0<C4a02JVYX0BTY:024Q8@0K6a/0
820P02XZ:P1^KVh0J6QX0<;2`P020:JVYP0;05MGE`0O7al07QhN02<S8`0P82009bLW079bLP2n_[h0
d=3@0:n_[`1GEeL04?ooool000kooooo0>80J6QX07alO01?Cdl0ADE507MgM`0n?Sh0B4Q8079bLP0]
;Bd0H61P05iNGP0m?Cd0FUYJ06UYJ@0a<C40;Rh^0451@@1PH600:2PX05IFEP19BDT0;Rh^07YjNP0d
=3@0Hf=S05QHF013@d<0D51@06MWI`0n?Sh0GEeM03Hf=P1BDU80G5aL0451@@0n?Sh0HV9R024Q8@1d
M7@0G5aL03Hf=P0W9bL0B4Q802XZ:P1cLg<0EeMG03<c<`16ATH0DE5A030`<01DE5@0?Cdm06MWI`1M
GEd0J6QX03hn?P1jNWX0@410034a<@1PH600Bd];03/k>`1MGEd0>c/k05mOG`2GUiL0GEeM04a<C01[
Jf/0HF5Q08:2PP28R8P0E5AD08:2PP1iNGT0=3@d06i^KP1>CTh0X:2P08N7Q`1LG5`08R8R07UiN@0W
9bL0Shn?07inOP0o?cl0=3@d0:RXZ00N7Qh0MWIf08Z:RP0d=3@0<30`038b<P1KFe/0<C4a0861P@1=
CDd0<S8b06m_K`0l?3`0E5AD04U9B@27QhL0L71`04];B`1XJ6P0Kfm_05UIF@2CTi<0KFe]08R8R01/
K6`0Lg=c065QH@1GEeL0MgMg05QHF01gMgL0M7Ad071`L01KFe/0E5AD09NGU`1WIfL0HF5Q07UiN@1G
EeL0N7Qh05mOG`2>SXh0GEeM05]KF`2m_Kd0@d=307IfMP2ATI40FUYJ0:RXZ02<S8`0>CTi07EeM@1m
OGd0<S8b04a<C02DU9@0De=C07IfMP2DU9@0B4Q807UiN@1WIfL0JFUY07=cL`1]KFd0OWin08n?S`1j
NWX0UiNG09ZJVP2@T900O7al0;Jf]P1SHf<0Lg=c09BDU02e]KD0HV9R08J6QP2HV9P0De=C07alO03=
cLd0Ogmo07YjNP2OWil0A4A4071`L01FEUH0=SHf06i^KP1`L700U9BD04];B`1nOWh0Hf=S03/k>`2<
S8`0E5AD08f=S@2TY:@0F5QH02l_;`1iNGT0Cdm?06ATI01HF5P0@d=306ATI032`/80KVi^0:NWY`1f
MWH0BTY:03Ph>02[Zj/0][Jf08>3P`14A4@0N7Qh07=cL`1SHf<0MGEe05=CD`1FEUH0U9BD051@D013
@d<0QHF508^;R`0m?Cd0Jf][05iNGP1:BTX0>3Ph0P1gMgL03@1>CTh0JFUY07QhN015ADD0O7al09jN
WP1IFET0Jf][05mOG`1lO7`0alO7092@T01ADE400P1dM7@0300W9bL0EUIF08F5Q@1:BTX09RHV07in
OP0_;bl0<S8b07=cL`28R8P0?Shn04=3@`80M7Ad01D0>CTi09nOW`1`L700:BTY05aLG00e=CD0RHV9
061PH00d=3@0T92@08J6QP1iNGT0N7Qh04Q8B00[:b/0GUiN07moO`1OGel0<30`09NGU`2DU9@03_oo
ool04@26QXH0EeMG06=SH`2`/;00HF5Q05IFEP1QHF40O7al09^KV`1mOGd0I6AT04m?C`14A4@0?Shn
03Lg=`0Y:BT0IVIV0080YjNW00D0S8b<030`<00/;2`09RHV01dM7@0201hN7P1Z02@T900_;bl0P820
079bLP16ATH0GEeM059BDP25QHD0^KVi038b<P0E5AD04A4A018B4P0D51@092@T00l?3`0@41004A4A
018B4P0E5AD05QHF02h^;P1BDU80RHV909^KV`3AdM40MgMg04Q8B01<C4`0De=C09BDU036a/H0bl_;
05mOG`1DE5@0HV9R07QhN01gMgL0H61P04];B`14A4@0JVYZ0;jn_P2:RXX0EEEE038b<P0N7Qh04Q8B
01@D500H61P09bLW02HV9P0P82006QXJ02DU9@0m?Cd0DE5A09>CT`2ATI40YjNW07emO@1_Kfl0G5aL
04A4A01:BTX0Bd];05eMG@1RHV80GEeM06QXJ00X:2P07alO030`<01@D500NGUi09ZJVP20P800M7Ad
03/k>`0X:2P0EUIF07IfMP1CDe<0L71`0;Bd]03;bl/0Xj>S07=cL`1[Jf/0Q8B408V9R@1cLg<0Bd];
05QHF02VYZH0_kno0;Ng]`2][Jd0P82006=SH`1TI6@0Lg=c07UiN@2=SHd0FEUI01dM7@<051@D00T0
71`L065QH@1^KVh0AdM7061PH02UYJD0VYZJ0;>c/`2o_kl00P0U9BD00`0X:2P09bLW02XZ:P0202XZ
:P0e038b<P17AdL0NGUi05QHF02EUID0Xj>S06UYJ@1PH600MgMg092@T01cLg<0DE5A05ADE02g]kL0
VYZJ0:j^[P3AdM40Z:RX08B4Q01hN7P0Bd];03Hf=P0n?Sh0H61P07alO025QHD0PH6107alO026QXH0
YjNW0;jn_P29RHT0jNWY0=[JfP2d];@0][Jf0;Vi^@29RHT0EeMG04U9B@13@d<0ADE504A4A017AdL0
D51@07QhN023Ph<0QhN709nOW`1VIVH08b<S024Q8@1BDU800P0i>CT0B018B4P0FUYJ07]kN`2][Jd0
[Zj^06ATI01VIVH0MGEe0:RXZ02PX:00UiNG06i^KP0m?Cd0:2PX02@T901?Cdl0EUIF01HF5P0J6QX0
7alO09FEU@2DU9@0TI6A0:RXZ03Ng]h0/K6a071`L01:BTX0Cdm?03Lg=`0e=CD0B4Q80:FUY@2YZJT0
Ph>303`l?00O7al071`L01PH600G5aL071`L028R8P0c<c<0TI6A0<71`@2[Zj/0L71`0:BTY02DU9@0
QXJ60820P02ATI40ZJVY0:6QX@22PX80IFEU08Z:RP2QXJ40`<300820P022PX80EUIF03Hf=P0e=CD0
>c/k04];B`1LG5`0Ogmo0:^[Z`2HV9P0OWin069RHQ3ooooo000>ooooo`3o065QH@2MWId0CTi>04];
B`2CTi<0@d=304Q8B02BTY80K6a/04M7A`1ZJVX0A4A40410@01>CTh0;bl_04U9B@1:BTX0RXZ:04I6
AP1HF5P0EEEE034a<@1dM7@0>SXj07EeM@1iNGT0GUiN05ADE01OGel0?Shn07emO@15ADD0C4a<04Q8
B00D51@0ADE508J6QP0e=CD0Shn?05mOG`0k>c/0=CDe0451@@0/;2`0N7Qh069RHP10@400D51@05eM
G@0l?3`0NWYj03lo?`1kNg/0GemO04a<C00W9bL0D51@05IFEP0n?Sh0H61P02DU9@0c<c<0TI6A0492
@P1kNg/0[jn_061PH01<C4`0QXJ6065QH@2ATI40Q8B403`l?01>CTh0QXJ6038b<P1SHf<0=3@d0861
P@23Ph<0Hf=S02PX:02<S8`0=3@d0:b/[02NWYh0KFe]03`l?02YZJT0<C4a07IfMP1HF5P0>SXj02PX
:00_;bl0E5AD02HV9P1YJFT0GemO03Ti>@25QHD0=CDe05EEE@18B4P0R8R804a<C01GEeL0NWYj06AT
I01CDe<0PX:204I6AP1eMGD0FEUI05iNGP1MGEd0PX:207UiN@1:BTX0KVi^06m_K`1PH600KFe]04e=
C@2>SXh0F5QH05UIF@1nOWh0De=C07alO01[Jf/0SHf=05EEE@1?Cdl0`<3004Y:BP2CTi<0P820055A
D@2YZJT0RXZ:03lo?`2XZ:P0PX:2038b<P15ADD0VIVI05=CD`1IFET0SHf=0451@@1jNWX0@D5106IV
IP25QHD0LG5a08:2PP1mOGd0J6QX0;Vi^@37alL0Xj>S079bLP2^[Zh0Ng]k079bLP1nOWh0][Jf06UY
J@2XZ:P0/k>c05QHF02>SXh0eMGE059BDP1OGel0]KFe04U9B@1nOWh0F5QH04=3@`1bLW80G5aL08J6
QP1=CDd0P82006IVIP1GEeL0RHV902h^;P1PH600TI6A05aLG00e=CD0RXZ:04e=C@27QhL0R8R805UI
F@1iNGT0cLg=079bLP2c/k<0QhN705mOG`11@D40Z:RX09JFUP29RHT0@41007=cL`1ADE40EUIF06UY
J@1DE5@0DU9B09FEU@1^KVh0@D5106i^KP2?Shl0?3`l09ZJVP1HF5P0Kfm_04m?C`1`L700OWin04];
B`1VIVH0MgMg03Ph>019BDT0HF5Q03lo?`1UIFD0HV9R09NGU`36a/H0De=C04e=C@2OWil0S8b<03Hf
=P1PH600R8R805MGE`0Y:BT0NWYj01lO7`0f=SH0FEUI079bLP0S8b<0?3`l000G075aL@1<C4`0GEeM
0:b/[01dM7@092@T05YJFP0g=cL0Ph>306e]K@0d=3@0R8R807AdM02CTi<0Shn?03`l?00c<c<0Cdm?
06EUI@1BDU80=CDe08:2PP2ATI403_ooool0901;Bd/0ZJVY0;Jf]P1OGel0NGUi05UIF@1^KVh0TY:B
07inOP25QHD0Kfm_05IFEP1:BTX0@D5104I6AP1OGel0WinO0;Rh^03EeMD0c/k>0861P@1MGEd0EeMG
05UIF@1MGEd0I6AT07AdM028R8P0bl_;0<c<c02e]KD0^k^k0<W9b@3Bd]80a/K603Hf=P8051@D02P0
410@01DE5@17AdL05ADE014A4@0?3`l04A4A01HF5P0E5AD04A4A00h>3P1:BTX0RHV90<g=c@3MgMd0
_kno0:VYZ@2UYJD0clo?0<k>cP39bLT0aLG50;>c/`2?Shl0LW9b06ATI01^KVh0UiNG0;6a/@3Lg=`0
/k>c051@D019BDT0:RXZ01LG5`0A4A40820P01dM7@0K6a/051@D100E5AD0<`1GEeL0PH610:j^[P3;
bl/0cLg=096AT@1XJ6P0@D5102`/;00S8b<05QHF01LG5`0C4a<092@T05IFEP0[:b/06ATI01dM7@0d
=3@0CTi>0820P00i>CT0HF5Q08b<S023Ph<0IfMW034a<@19BDT0Q8B40;Vi^@2o_kl0XZ:R061PH022
PX80Vi^K09nOW`2IVIT0RHV909FEU@2NWYh0YjNW07IfMP0h>3P071`L038b<P11@D409RHV05MGE`1a
LG40OGem024Q8@0201<C4`0<01DE5@1OGel0[:b/0861P@1TI6@0RXZ:0<G5a@3Fe]H0d];B03Xj>P0J
6QX05aLG0P0E5AD201HF5P1N01DE5@1EEED0DU9B04a<C02ZZZX0Bd];096AT@2GUiL0PH6109:BTP2K
Vi/0WinO0;2`/034a<@0`l?30>GUi@2l_;`08B4Q01HF5P0U9BD0NWYj09ZJVP29RHT0a/K607IfMP0h
>3P0;Bd]02LW9`0`<300Bd];09VIV@2e]KD0d];B0<_;b`2^[Zh0VYZJ096AT@1RHV80<30`02XZ:P0]
;Bd0:BTY02LW9`0Y:BT0<S8b02l_;`1<C4`0Shn?0;>c/`1:BTX0?clo020P800g=cL0FUYJ04Y:BP2A
TI40WIfM0820P02EUID0_kno09:BTP22PX80VYZJ0;Bd]01CDe<0BTY:075aL@0a<C40;Rh^034a<@0n
?Sh0NWYj06e]K@0k>c/0K6a/0<k>cP3<c<`0/;2`0>OWi`3QhN40V9RH069RHP1WIfL0?clo05mOG`2>
SXh0_kno0=;BdP3IfMT0WIfM01PH600G5aL05ADE01HF5P805aLG00L06ATI030`<02RXZ80PX:205YJ
FP1/K6`0PH6100806QXJ01H0820P03<c<`1[Jf/0T92@0;Rh^031`L40gmoO0<g=c@2UYJD0ZZZZ05EE
E@2WYjL0OGem07=cL`2<S8`0Ph>30:2PX02FUYH0MWIf0:ZZZP1HF5P0KVi^4?ooool000kooooo0?l0
GUiN09NGU`1=CDd0DE5A07emO@0m?Cd0EeMG09^KV`1]KFd0De=C092@T016ATH0?Cdm03<c<`0D51@0
>CTi04U9B@1SHf<0;Rh^08Z:RP1IFET0?Shn0820P00i>CT0TY:B07YjNP1PH600Hf=S06i^KP18B4P0
L71`04=3@`1;Bd/0I6AT03dm?@0c<c<0Kfm_030`<02@T900820P03Xj>P0[:b/0@d=3038b<P2CTi<0
GemO03Ti>@1>CTh0@D5102DU9@1ZJVX0@T9207emO@28R8P0QHF504A4A01`L700DU9B03/k>`1LG5`0
=CDe02@T901=CDd0:2PX06m_K`2d];@0D51@05MGE`2:RXX0GEeM08N7Q`20P800GUiN03Ti>@1lO7`0
;2`/05iNGP15ADD0ZZZZ0:2PX01@D500:2PX07IfMP0o?cl0UIFE09RHV01PH600GEeM0;^k^`12@T80
O7al059BDP0];Bd0?3`l05EEE@1bLW80>SXj07]kN`1VIVH0>3Ph08>3P`0/;2`0DE5A0451@@2PX:00
K6a/05ADE01[Jf/0J6QX04];B`25QHD0FEUI07inOP1LG5`0CTi>05aLG01PH600MgMg0410@01HF5P0
OWin06i^KP1KFe/0AdM708f=S@1@D500QHF507MgM`1:BTX0OGem05iNGP2:RXX0?3`l04M7A`2PX:00
CTi>0:>SX`1mOGd0D51@09BDU02IVIT0;bl_08N7Q`1lO7`0>3Ph05YJFP2VYZH0JVYZ069RHP2@T900
A4A407moO`1]KFd0G5aL06EUI@1XJ6P0O7al075aL@1EEED0NWYj08J6QP2:RXX0LG5a0:f][@1iNGT0
PH6108b<S02ATI40?Shn07moO`29RHT0ATI608^;R`2b/[80LW9b07QhN02GUiL0HF5Q08b<S00g=cL0
>SXj08j>SP1JFUX0I6AT03@d=01/K6`0MWIf05mOG`2FUYH0E5AD0820P02e]KD0De=C03Ph>02EUID0
C4a<04Q8B01GEeL0DU9B08R8R03@d=00M7Ad0::RXP2f][H0Q8B405aLG02o_kl0Z:RX09jNWP16ATH0
RXZ:06IVIP1^KVh0Rh^;01lO7`0k>c/0U9BD05YJFP18B4P0P82008^;R`0Y:BT0L71`059BDP0X:2P0
=3@d092@T029RHT0=SHf069RHP1kNg/0?clo07YjNP1oOgl0:BTY06IVIP1`L700Ogmo0=?Cd`1`L700
>SXj05eMG@1>CTh09BDU04Q8B01/K6`0IfMW02d];@1fMWH07alO03Lg=`1DE5@0O7al03Ti>@12@T80
01L0HV9R05eMG@0g=cL0J6QX061PH00Q8B40F5QH034a<@2;Rh/0Hf=S0451@@2YZJT0JFUY09ZJVP2A
TI40<C4a03hn?P1ZJVX0TI6A04m?C`0[:b/0SXj>09^KV`0>ooooo`0G03dm?@2ATI40UYJF065QH@1m
OGd0ZJVY0;^k^`1^KVh0;Bd]03@d=01cLg<0WIfM0820P01OGel0E5AD06i^KP2KVi/0Ph>309FEU@3<
c<`0Y:BT03`l?00a<C400P0_;bl2034a<@0D04Q8B02LW9`0UIFE06m_K`2DU9@0E5AD06EUI@2?Shl0
K6a/01LG5`0C4a<03Ph>01hN7P2GUiL09BDU018B4P0?3`l04a<C01HF5P0>3Ph2014A4@0>04I6AP0m
?Cd0SXj>0:VYZ@0b<S80;Bd]03Xj>P2d];@0bl_;0:n_[`1cLg<0GEeM05YJFP1WIfL206UYJ@0;08^;
R`2TY:@0emOG08j>SP0Y:BT0>c/k02LW9`1=CDd0A4A401`L700D51@00`0C4a<0300A4A404a<C01DE
5@0`<300eMGE0;:b/P36a/H0^KVi07moO`1=CDd09bLW01DE5@804a<C06<04Q8B01PH600G5aL0@T92
05MGE`0[:b/06a/K03Hf=P1RHV80Ng]k02d];@1DE5@0FUYJ059BDP1HF5P0M7Ad051@D01?Cdl0QXJ6
09:BTP2FUYH0GEeM02<S8`0o?cl0Shn?0;no_`35aLD0Ph>303hn?P1FEUH0QHF505=CD`0I6AT06QXJ
03/k>`0O7al0>c/k04E5A@1RHV80Ng]k02DU9@0F5QH08R8R0492@P2TY:@0dM7A08N7Q`1dM7@0];Bd
0:f][@2o_kl0NWYj02LW9`0E5AD051@D01HF5P0G5aL05ADE030`<01UIFD0=SHf06QXJ026QXH0:2PX
06YZJP24Q8@0BDU90410@00K6a/0<S8b07=cL`31`L40hN7Q0>[ZjP29RHT0ADE503hn?P12@T80DE5A
07moO`2d];@0Fe]K06ATI00m?Cd0>3Ph03/k>`1:BTX0CTi>069RHP3?cll0eMGE0;bl_01eMGD0MgMg
05QHF01JFUX0=CDe02`/;00`<3000P0Y:BT0<@0[:b/0=CDe04];B`1DE5@0LG5a09RHV01OGel0F5QH
01lO7`0h>3P0IfMW06][J`23Ph<0R8R80::RXP2h^;P0^[Zj0:>SX`1QHF40LG5a08:2PP1:BTX0G5aL
0861P@19BDT0<S8b03De=@1;Bd/0WIfM0<W9b@2JVYX0Fe]K04U9B@2MWId0GemO0:VYZ@3Ng]h0c<c<
0:b/[02OWil0[Jf]09fMW@1bLW80NWYj0<S8b03Jf]X0QhN701HF5P0E5AD00P0F5QH02P0G5aL06QXJ
01/K6`1:BTX0OWin06=SH`1HF5P0TI6A02PX:00F5QH201LG5`0E01DE5@0G5aL06ATI02h^;P26QXH0
e]KF0<G5a@25QHD0T92@0:n_[`30`<00/k>c071`L019BDT0EUIF0:6QX@28R8P0QhN707alO025QHD0
HV9R013ooooo000>ooooo`3T06YZJP1XJ6P0Cdm?04a<C01aLG40<S8b0451@@1hN7P0F5QH04Q8B00h
>3P0?clo06YZJP1lO7`0>SXj03Ti>@1ADE40G5aL02@T901RHV80IFEU04e=C@1iNGT092@T06MWI`1/
K6`0DU9B0820P02?Shl0DU9B06e]K@0];Bd0GEeM07alO011@D40@d=307=cL`0Q8B40MgMg03Lg=`0c
<c<092@T04m?C`0M7Ad0J6QX03`l?00g=cL0IfMW065QH@0i>CT0F5QH03Ti>@27QhL0Y:BT08B4Q016
ATH0K6a/03Xj>P1HF5P0SXj>059BDP1>CTh0G5aL02TY:@1CDe<0ZJVY06][J`1RHV80Ogmo05eMG@2f
][H0WIfM06a/K00b<S80?Cdm02<S8`1eMGD0H61P0;Rh^02d];@04Q8B02PX:01lO7`0:RXZ09:BTP2N
WYh07QhN02/[:`21PH40A4A408>3P`2XZ:P0Hf=S02@T900W9bL0HV9R05UIF@2UYJD0IfMW038b<P1e
MGD0H61P06e]K@0Y:BT0R8R806e]K@1XJ6P0LW9b03<c<`1>CTh0RHV905mOG`1^KVh0A4A405eMG@1]
KFd0L71`06i^KP14A4@0Kfm_0:>SX`1bLW80F5QH038b<P2>SXh0JFUY08B4Q01ADE409RHV0820P01O
Gel0][Jf038b<P0V9RH0ZJVY05UIF@2;Rh/0I6AT02d];@2ATI40VYZJ04I6AP2KVi/0De=C02DU9@0f
=SH0L71`04A4A01lO7`0P82002l_;`1cLg<0G5aL065QH@1cLg<0Fe]K08n?S`2=SHd0LG5a08b<S01l
O7`0IFEU071`L032`/80TI6A09BDU02b/[80/;2`05MGE`21PH40TY:B04U9B@2DU9@0`l?306QXJ01l
O7`0TY:B03dm?@1SHf<0DU9B02<S8`1FEUH0EeMG07EeM@11@D40C4a<05aLG016ATH0N7Qh065QH@2N
WYh0][Jf07AdM00l?3`0RHV904U9B@1IFET0CTi>03Ti>@1]KFd0Z:RX06UYJ@20P800Ogmo07QhN01M
GEd0c<c<0<k>cP2HV9P0<S8b07MgM`1hN7P0IFEU08R8R01JFUX0Cdm?0:2PX01ZJVX0C4a<07YjNP1h
N7P0>3Ph07]kN`17AdL0GEeM04e=C@1TI6@0Ng]k0P1@D500:01VIVH0;Rh^06QXJ01fMWH0Bd];05QH
F01ZJVX0D51@0<O7a`1PH600Bd];07EeM@1/K6`0;Rh^04U9B@1WIfL0I6AT03Ph>01jNWX0=3@d03Xj
>P1?Cdl0QhN704=3@`17AdL0J6QX079bLP10@400MGEe05aLG00^;Rh0QHF502h^;P1jNWX0<c<c02DU
9@1bLW80Lg=c09ZJVP1[Jf/201LG5`0602d];@1;Bd/0CDe=02LW9`28R8P0O7al3_ooool05@15ADD0
JVYZ0451@@1TI6@0[:b/0<;2`P2n_[h0Y:BT07MgM`1GEeL0GEeM09>CT`2`/;00SHf=0;jn_P2?Shl0
RHV905ADE00^;Rh0TY:B05UIF@02018B4P0N00l?3`0@41004A4A00h>3P0F5QH0AdM707]kN`15ADD0
DE5A01LG5`0E5AD0?3`l0;:b/P0Y:BT04A4A00d=3@0d=3@0VYZJ04];B`19BDT0>c/k02PX:00@4100
3Ph>02<S8`0F5QH0MWIf08V9R@1nOWh0QhN70P0d=3@05`1VIVH0Jf][0;Bd]032`/80Vi^K07alO00e
=CD0<30`0492@P17AdL0@D5104Q8B02l_;`0[Jf]02l_;`0j>SX05aLG02h^;P0h>3P03Ph>00l?3`0B
4Q80410@00804a<C00@05ADE01<C4`0J6QX0bl_;0P2`/;00102_[jl0R8R803<c<`0J6QX2014A4@03
00l?3`0@41005QHF00806ATI03<0A4A405mOG`0Y:BT0?clo069RHP1gMgL0IfMW0492@P0^;Rh0820P
01dM7@0X:2P0=3@d02XZ:P1WIfL0T92@03hn?P1WIfL0>3Ph01dM7@0Y:BT0Lg=c09NGU`0m?Cd0;Rh^
04=3@`24Q8@0Fe]K02PX:00L71`0?Shn02DU9@0W9bL0FUYJ03`l?01OGel0MgMg04A4A00j>SX0R8R8
09^KV`1<C4`09RHV02TY:@2;Rh/0`<3007inOP20P800MGEe02TY:@0C4a<00P0E5AD07P0O7al0GEeM
03Hf=P1EEED0WIfM03lo?`0X:2P0A4A405ADE00`<300@d=301/K6`0V9RH0?Cdm06IVIP2RXZ80g=cL
08B4Q012@T80>SXj03Ti>@0c<c<0@T92096AT@0X:2P0Q8B408n?S`1TI6@0@T9202TY:@806ATI00X0
][Jf0<_;b`3GemL0VYZJ0:VYZ@1dM7@0@d=301/K6`0N7Qh0:b/[0P0Q8B40<00S8b<07AdM01LG5`0N
7Qh0C4a<07alO02NWYh0KVi^07inOP2[Zj/0U9BD09jNWP2?Shl0XZ:R07UiN@1KFe/0Lg=c0<?3``1A
DE40<C4a0451@@1FEUH0GemO059BDP0[:b/092@T034a<@2MWId0][Jf0;Vi^@12@T806a/K01`L700^
;Rh0P82009>CT`36a/H0dm?C0<O7a`2f][H0JFUY05EEE@2PX:00`/;208^;R`1_Kfl0YZJV02<S8`80
5ADE02806ATI01hN7P0P8200<S8b08^;R`1[Jf/0J6QX09bLW011@D405ADE01HF5P0G5aL06ATI01LG
5`0F5QH05ADE01LG5`1QHF40/K6a0=;BdP2ZZZX0T92@0;Rh^02l_;`0YjNW04Y:BP18B4P0[:b/0:^[
Z`23Ph<0LW9b06ATI01[Jf/0CTi>4?ooool000kooooo00h0Kfm_07AdM01;Bd/0CDe=07UiN@10@400
@d=306m_K`10@400C4a<071`L00a<C409RHV04a<C080;2`/0;@0<C4a05]KF`12@T80G5aL02PX:016
ATH0IfMW038b<P27QhL0>CTi038b<P17AdL0Hf=S05mOG`1gMgL0:RXZ04Y:BP1@D5009bLW04i>CP1i
NGT09RHV0861P@1CDe<0?clo01hN7P1=CDd0;bl_07moO`1FEUH0;bl_04A4A01FEUH0>3Ph04M7A`0W
9bL0BDU9061PH01WIfL0?Cdm07UiN@0/;2`0<30`05MGE`0e=CD0GUiN07=cL`0Q8B40DE5A09BDU01Y
JFT0O7al08j>SP0o?cl0KFe]08:2PP1<C4`0@T9206e]K@0R8R80FUYJ075aL@3;bl/0[:b/04e=C@0L
71`0M7Ad02PX:02BTY80SHf=03hn?P0i>CT0XJ6Q04U9B@28R8P0Lg=c05ADE00^;Rh0=CDe07=cL`0n
?Sh0LG5a07MgM`0k>c/0LG5a05=CD`1PH600HV9R08Z:RP1TI6@0O7al08J6QP1TI6@0ATI607YjNP1J
FUX0Lg=c02PX:00h>3P0EUIF07alO01]KFd08b<S0492@P1/K6`0EUIF06m_K`0c<c<0MWIf051@D01U
IFD0MgMg04i>CP1SHf<0GemO08B4Q01DE5@0?Shn08>3P`17AdL0U9BD07=cL`13@d<0MWIf092@T019
BDT0XJ6Q07emO@0g=cL0>3Ph08:2PP1FEUH0F5QH07moO`0m?Cd0GUiN04];B`20P800SHf=04A4A02O
Wil0W9bL07EeM@26QXH0aLG506IVIP1IFET0/K6a051@D01ZJVX0TY:B0:6QX@1RHV80YjNW08B4Q01;
Bd/0VIVI0;^k^`1HF5P0UIFE0:JVYP13@d<0KFe]04a<C00P8200Bd];06i^KP1aLG40?3`l04M7A`1N
GUh0De=C069RHP1>CTh0M7Ad0:^[Z`1_Kfl0>CTi08Z:RP1@D50206m_K`1@05EEE@2@T900^k^k05eM
G@2SXj<0OWin05EEE@0d=3@0U9BD08n?S`2CTi<0?3`l06=SH`1iNGT0J6QX07moO`1DE5@0B4Q808N7
Q`1NGUh0@D5106ATI02CTi<0F5QH09BDU01LG5`0Kfm_04M7A`1NGUh0MGEe061PH01XJ6P0Q8B404U9
B@1[Jf/0IVIV06a/K02<S8`0U9BD08J6QP37alL0UIFE05=CD`2PX:00Ti>C03De=@1<C4`0I6AT02XZ
:P0f=SH0LW9b0492@P0a<C40ATI60820P00Z:RX0Jf][09FEU@1SHf<0?Shn06a/K01>CTh07AdM069R
HP0d=3@0R8R806EUI@0a<C40Vi^K07AdM02VYZH0H61P04=3@`0a<C40EUIF079bLP1EEED0:RXZ07Ee
M@1lO7`>ooooo`0I05aLG01JFUX0<S8b02d];@1iNGT0cLg=0=?Cd`2g]kL0MGEe04I6AP0k>c/0DU9B
08j>SP1lO7`0/K6a08F5Q@2CTi<0FUYJ03@d=01aLG40FUYJ01LG5`0E5AD07QhN01HF5P0200h>3P05
01<C4`0M7Ad0Ng]k06EUI@0e=CD00P0A4A40>00K6a/0Vi^K05IFEP0D51@07alO06IVIP0V9RH051@D
01DE5@1@D500Kfm_01/K6`0B4Q807alO04E5A@2k^k/0gMgM0<;2`P3<c<`0S8b<08F5Q@33`l<0G5aL
06IVIP0O7al0:BTY04Y:BP1[Jf/0AdM701`L700Z:RX09BDU02PX:01ZJVX0/K6a06=SH`0o?cl04a<C
01DE5@15ADD05QHF00h>3P0A4A406ATI01LG5`0E5AD051@D01DE5@0J6QX0TY:B0:NWY`25QHD0Lg=c
04Y:BP0S8b<04a<C0`0>3Ph2014A4@0e01PH600I6AT07AdM04m?C`1dM7@0P8200;Jf]P1MGEd0?3`l
02HV9P0I6AT05ADE01HF5P0L71`08R8R034a<@25QHD0H61P05UIF@1;Bd/0CTi>01hN7P0L71`0JFUY
092@T00m?Cd0:RXZ03Hf=P2EUID0WYjN06UYJ@11@D40ADE503Ph>00R8R80KVi^02PX:00[:b/0@D51
07EeM@20P800MWIf07emO@0R8R80<C4a04];B`1VIVH0^;Rh0820P01XJ6P0KVi^09>CT`1VIVH00P0I
6AT0701/K6`0C4a<059BDP1kNg/0Rh^;01hN7P1@D500F5QH05MGE`0g=cL0L71`04E5A@14A4@0>3Ph
05QHF02`/;00^;Rh09RHV00e=CD0<C4a02l_;`0`<300;2`/09fMW@0];Bd0ADE5079bLP1WIfL201DE
5@0=01<C4`0U9BD0`/;20=CDe03Jf]X0BTY:038b<P0m?Cd09RHV03lo?`0L71`06ATI01@D500401DE
5@0h02l_;`1GEeL0Ogmo0<c<c02;Rh/0_Kfm08f=S@1QHF40YjNW06m_K`0h>3P08R8R02DU9@0_;bl0
JFUY092@T00f=SH0;2`/05YJFP1TI6@0B4Q80492@P25QHD0WYjN06QXJ02XZ:P0NWYj02d];@0^;Rh0
:RXZ028R8P23Ph<0TY:B0;^k^`2b/[80TY:B05EEE@1_Kfl0`<3007QhN00`<3006ATI02DU9@1SHf<0
FUYJ02LW9`0k>c/0Fe]K07YjNP2@T900WIfM0<?3``2NWYh0QXJ60;no_`0M7Ad201DE5@805aLG01D0
5ADE01LG5`0F5QH06ATI07MgM`11@D40/[:b0;Jf]P2RXZ80Xj>S0;bl_02XZ:P0/k>c0:^[Z`37alL0
Lg=c04i>CP1UIFD0Kfm_06IVIP1GEeL04?ooool000kooooo01@0JVYZ09ZJVP1ADE40A4A408R8R016
ATH0ADE509jNWP1lO7`0ADE506a/K00e=CD0;2`/05mOG`0k>c/0B4Q8051@D01iNGT0@d=306UYJ@80
BTY:0600N7Qh03<c<`2<S8`0KVi^05YJFP1FEUH0HF5Q03hn?P1/K6`0@d=305ADE01GEeL0<S8b02`/
;01EEED0;Bd]09RHV018B4P0Bd];03dm?@1;Bd/0<S8b07YjNP1BDU80=CDe04m?C`1@D500@T92051@
D00e=CD0I6AT06YZJP1OGel0<c<c06QXJ01ADE40>3Ph06QXJ015ADD0=3@d05ADE00_;bl08B4Q07in
OP1FEUH0K6a/09BDU01QHF40VYZJ07IfMP1GEeL0>SXj06m_K`0Y:BT0FUYJ04a<C02a/K40UYJF06e]
K@0S8b<0Shn?06QXJ02j^[X0XJ6Q05iNGP12@T80XZ:R04Q8B01nOWh0Lg=c038b<P1<C4`0IfMW08V9
R@0j>SX0OGem05QHF014A4@0Rh^;05iNGP1CDe<0Bd];0:6QX@1bLW80JFUY05mOG`1UIFD0Hf=S07Mg
M`1BDU80LW9b05]KF`1=CDd0EUIF05MGE`1nOWh203lo?`1206IVIP1]KFd0Jf][04Y:BP1jNWX0EUIF
07AdM021PH40EeMG05]KF`1SHf<0T92@04m?C`1<C4`0HV9R04];B`2i^KT0[jn_08F5Q@1bLW80ZJVY
0410@02FUYH0OWin03hn?P1GEeL0WinO05eMG@1OGel0Ti>C04E5A@1mOGd0GUiN059BDP1UIFD0HV9R
0861P@1`L700FEUI08:2PP2EUID0LG5a0820P02c/k<0MWIf0861P@2IVIT0Rh^;03De=@1lO7`0MWIf
03lo?`2BTY80Zj^[04E5A@1IFET0RHV904i>CP1oOgl0@41004];B`1cLg<0C4a<05]KF`0j>SX0L71`
0P1=CDd0D01`L700:b/[05UIF@2KVi/0LG5a03Hf=P27QhL0ADE508J6QP1mOGd0Bd];08n?S`2h^;P0
KFe]0<g=c@30`<00JVYZ04a<C02/[:`0Y:BT09ZJVP0j>SX0Kfm_07IfMP1`L700Ph>306MWI`15ADD0
T92@06i^KP1<C4`0HF5Q08F5Q@0[:b/0UYJF08:2PP1_Kfl0A4A405YJFP25QHD0<C4a079bLP1fMWH0
A4A406e]K@1LG5`0@d=3096AT@2CTi<0OWin0<C4a02XZ:P0DU9B096AT@2FUYH0;bl_04Q8B01SHf<0
Ph>305=CD`2CTi<0AdM702d];@0h>3P0UiNG0451@@10@400P82006IVIP1BDU80KFe]05iNGP0W9bL0
F5QH01lO7`1[Jf/0IVIV03De=@2UYJD0Kfm_0P20P800201PH600>CTi06YZJP1jNWX0FEUI02`/;020
P800UYJF3_ooool05`28R8P0JVYZ03Hf=P0];Bd0FUYJ09jNWP3=cLd0OGem01dM7@0E5AD04A4A00l?
3`0@4100GEeM0;:b/P2KVi/0FUYJ07YjNP0n?Sh0HV9R06=SH`0G5aL05ADE00805aLG0280410@034a
<@0S8b<06a/K06a/K02?Shl0<C4a014A4@0>3Ph04a<C059BDP1lO7`092@T07]kN`0`<300410@01<C
4`0@41006QXJ03hn?P1OGel0M7Ad09FEU@28R8P0b/[:0<o?c`2m_Kd0alO70<K6aP3>c/h0Vi^K05YJ
FP0B4Q802PX:0P0=3@d0100@4100?3`l05mOG`0n?Sh202<S8`0803Lg=`2WYjL0R8R805EEE@0C4a<0
51@D02TY:@0e=CD200h>3P0@010@400A4A404Q8B01LG5`0B4Q807alO07emO@2RXZ80QhN706][J`0l
?3`0820P00l?3`0D51@04A4A01<C4`805ADE00X05aLG01dM7@0G5aL0;Bd]08J6QP3<c<`0h^;R0=OG
e`2;Rh/092@T0P0L71`06@0X:2P06QXJ01`L701CDe<0;Bd]01HF5P19BDT0JVYZ05QHF00R8R807AdM
03dm?@1]KFd0;2`/02@T900h>3P0b/[:0<C4a01GEeL0?Shn05aLG00n?Sh0CTi>0;Fe]@1XJ6P00P0_
;bl0;017AdL0U9BD08J6QP0i>CT0?3`l055AD@1aLG40K6a/08J6QP1aLG40DE5A05eMG@1PH600UIFE
0861P@2=SHd0Fe]K05aLG01IFET0SXj>02PX:00M7Ad0ADE50:VYZ@1RHV80>3Ph071`L00];Bd0>3Ph
0410@00h>3P0TI6A0::RXP3:b/X0>3Ph02TY:@0a<C40>CTi03Lg=`25QHD0H61P06=SH`17AdL0BTY:
0P0E5AD0B`0V9RH0UYJF0=3@d02XZ:P0V9RH0:f][@1WIfL0:b/[01`L701>CTh0D51@02d];@0[:b/0
8R8R01`L700M7Ad092@T05MGE`1PH600YjNW08n?S`2o_kl0eMGE0;Ng]`3<c<`0NGUi02/[:`0V9RH0
8b<S02DU9@0^;Rh0?clo05mOG`1^KVh0=3@d069RHP25QHD0XZ:R0;^k^`25QHD0Fe]K05UIF@3@d=00
?Cdm01LG5`0O7al0GEeM07alO01XJ6P0c/k>0;Bd]02XZ:P0QHF5075aL@2d];@0<c<c01TI6@0G5aL0
6a/K01hN7P0X:2P0[jn_0;jn_P2_[jl0W9bL08Z:RP26QXH0T92@0<k>cP2LW9`0Ng]k08n?S`1EEED0
:BTY02<S8`0401DE5@0C01XJ6P1>CTh0E5AD079bLP32`/80W9bL09NGU`2d];@0/[:b0:^[Z`2d];@0
emOG09jNWP0j>SX0@d=305ADE02OWil0XJ6Q0:JVYP0@ooooo`003_ooool06`1RHV80P820055AD@1A
DE40L71`0492@P1ADE40UIFE079bLP1=CDd0G5aL03dm?@1:BTX0FUYJ02TY:@0i>CT0BDU905eMG@0i
>CT0I6AT03Xj>P1NGUh0L71`03Hf=P1cLg<0@D5104=3@`0206e]K@3i059BDP1^KVh0@T9205QHF01^
KVh0=cLg03Ph>01;Bd/0:b/[08f=S@12@T80>3Ph034a<@1>CTh0;bl_07emO@0l?3`0>c/k06UYJ@17
AdL0?3`l03hn?P0];Bd0L71`07MgM`1/K6`0@T9206EUI@0c<c<0B4Q807alO01EEED0G5aL055AD@0h
>3P0?clo08B4Q01eMGD0LW9b07YjNP1BDU80U9BD07]kN`1GEeL0BTY:03Hf=P0W9bL0GEeM04a<C02j
^[X0TI6A02PX:00Y:BT0PH6103Ti>@2OWil0UYJF02HV9P0/;2`0Ti>C04m?C`1mOGd0QhN703Hf=P0`
<300>3Ph06QXJ00W9bL0J6QX0492@P0U9BD0Hf=S04I6AP1ADE40=SHf07inOP1SHf<0I6AT065QH@1C
De<0EeMG08J6QP1MGEd0KFe]0492@P1DE5@0IfMW06ATI01fMWH0?Cdm05IFEP1lO7`0IfMW06=SH`18
B4P0R8R805YJFP1mOGd0KFe]04I6AP1oOgl0EEEE09:BTP0l?3`0C4a<07UiN@11@D40Shn?0820P015
ADD0GUiN08Z:RP16ATH0Vi^K06=SH`11@D40Fe]K08>3P`1LG5`0De=C07EeM@13@d<0MGEe04m?C`1N
GUh0JVYZ05EEE@2HV9P0Shn?05eMG@2OWil0VYZJ05ADE01aLG40Vi^K05YJFP25QHD0/[:b09jNWP1E
EED0VIVI051@D014A4@0SHf=0:f][@1LG5`0O7al07moO`1BDU80LG5a059BDP0W9bL0H61P065QH@1W
IfL0=SHf05MGE`1JFUX0BTY:05eMG@1<C4`0RHV90:JVYP1?Cdl0?Shn08R8R01>CTh0Fe]K03dm?@0a
<C40RXZ:0:VYZ@1FEUH0QhN708J6QP1VIVH0@T920:6QX@2^[Zh0RXZ:03Lg=`1kNg/0IfMW06m_K`2T
Y:@0Cdm?0492@P27QhL0Bd];0492@P1[Jf/0MGEe03/k>`1eMGD0C4a<0410@00j>SX0IVIV069RHP1C
De<0Jf][06QXJ013@d<0OWin08J6QP1_Kfl0Hf=S07moO`1eMGD0aLG50:b/[01DE5@0RXZ:06YZJP0W
9bL0FEUI09>CT`2<S8`0CTi>07emO@0Z:RX0<C4a05UIF@2<S8`0EUIF04E5A@20P800?Cdm05=CD`20
P800H61P02d];@1_Kfl0;2`/07AdM01@D500@d=30:b/[01`L700X:2P08^;R`0o?cl0@d=308n?S`2L
W9`0CDe=03@d=01oOgl0TY:B00kooooo02@0_Kfm06a/K017AdL0G5aL08f=S@1iNGT0YZJV038b<P0E
5AD04a<C00l?3`0>3Ph03@d=018B4P28R8P0GUiN04a<C023Ph<0XZ:R071`L01cLg<09RHV01/K6`0E
5AD04a<C01DE5@0Q8B40GEeM05iNGP21PH40[Jf]06e]K@15ADD0@D5103hn?P1FEUH20;Fe]@0D0:RX
Z022PX80Ph>307QhN01iNGT0MGEe06ATI01=CDd0@d=304a<C01`L700c<c<061PH01ADE40QXJ609bL
W03=cLd0Zj^[04i>CP0?3`l300h>3P0[00l?3`0>3Ph04Q8B02HV9P0i>CT0Bd];06i^KP2GUiL0U9BD
071`L01OGel0CDe=02l_;`0k>c/0;Bd]014A4@0>3Ph03`l?00h>3P0A4A404a<C04i>CP1jNWX0`L71
0<g=c@2VYZH0FEUI03lo?`0Z:RX0;2`/02TY:@0R8R806a/K01<C4`0B4Q8061PH02<S8`1lO7`0`<30
0=7Ad@2OWil0[jn_07emO@0206QXJ00`05]KF`1OGel0GUiN04];B`1DE5@08B4Q01HF5P0L71`0?3`l
06ATI01?Cdl08R8R03/k>`1BDU80;Bd]02/[:`17AdL0De=C08>3P`33`l<0TY:B096AT@2BTY80SXj>
07=cL`1[Jf/0JVYZ071`L023Ph<0UYJF09jNWP1/K6`0FEUI05ADE012@T80?Cdm02d];@0c<c<0D51@
05QHF01IFET0CTi>096AT@36a/H0IfMW06e]K@22PX80<c<c0P0K6a/03@1@D500SHf=08B4Q029RHT0
];Bd03`l?00T92@0>CTi02XZ:P1:BTX0TI6A0<c<c01[Jf/00P15ADD09@1;Bd/0EeMG079bLP1kNg/0
E5AD0451@@1QHF40B4Q806IVIP1SHf<0CDe=06][J`2:RXX0@D5104m?C`1hN7P0Ph>303@d=00i>CT0
CTi>059BDP0Z:RX09RHV02TY:@0c<c<0>CTi06ATI01kNg/0ZJVY0;Bd]02b/[80b<S80:^[Z`2OWil0
Ti>C04e=C@0S8b<00P0T92@06@0R8R80:b/[03/k>`1[Jf/0TI6A0:BTY02IVIT0V9RH08b<S01YJFT0
LG5a0;6a/@2UYJD0M7Ad05aLG02EUID0T92@0;6a/@1_Kfl0a<C40<K6aP3Cdm<0^KVi09JFUP0/;2`0
100G5aL02P0L71`09RHV08F5Q@0g=cL08b<S01lO7`0N7Qh07alO03<c<`25QHD205UIF@0@07EeM@1i
NGT0J6QX04A4A01ADE40EeMG04E5A@13@d<0EeMG06i^KP1@D500MgMg06IVIP2RXZ80JFUY0<G5a@80
N7Qh00T0SXj>0:f][@2`/;00NWYj06YZJP1XJ6P0K6a/09FEU@1SHf<04?ooool000kooooo0340HV9R
06YZJP19BDT0BDU909FEU@1<C4`0@d=30:NWY`2:RXX0D51@08n?S`0n?Sh0;bl_065QH@0[:b/0<c<c
03Hf=P1MGEd0EUIF05=CD`0S8b<0AdM7065QH@0e=CD0LW9b06ATI013@d<0Cdm?05aLG017AdL0KFe]
03De=@19BDT0EEEE03De=@0j>SX0Bd];034a<@1kNg/0;Bd]04A4A00a<C40Bd];02`/;01kNg/0@d=3
03`l?01:BTX0=3@d0080>3Ph02T09RHV05mOG`1CDe<0IFEU0451@@1]KFd0>c/k02d];@1XJ6P0E5AD
07=cL`18B4P09RHV02DU9@1PH600DE5A05eMG@1SHf<0Cdm?08Z:RP2@T900Fe]K05MGE`12@T80820P
05ADE00l?3`0Z:RX06EUI@13@d<092@T07inOP17AdL0YjNW096AT@0/;2`0;Bd]09fMW@16ATH0LW9b
08^;R`0204=3@`2h04A4A01kNg/0<S8b06i^KP1OGel0=CDe06i^KP16ATH0EUIF055AD@2CTi<0Fe]K
04i>CP1VIVH0PX:206][J`2CTi<0Jf][07=cL`1XJ6P0DU9B06MWI`1CDe<0QXJ604Y:BP1>CTh0Kfm_
06YZJP1LG5`0Cdm?096AT@1:BTX0MgMg07moO`1?Cdl0HV9R061PH02?Shl0Cdm?05eMG@1VIVH0>SXj
08Z:RP1eMGD0>CTi03De=@1oOgl0BDU90:FUY@1ZJVX0C4a<06a/K02LW9`0DE5A05aLG02@T900ATI6
061PH00h>3P0IVIV06MWI`1`L700PH6108>3P`1XJ6P0/[:b09nOW`28R8P0X:2P0:FUY@0m?Cd0De=C
08B4Q02:RXX0I6AT0;Jf]P1PH600AdM70:NWY`2LW9`0@41008N7Q`2>SXh0BDU906EUI@1TI6@0;bl_
061PH00m?Cd0Ng]k03`l?01UIFD0GUiN04];B`1<C4`0Bd];09NGU`2EUID0KFe]03Ti>@2BTY80De=C
06QXJ01TI6@0BTY:06][J`29RHT0FUYJ0;>c/`1eMGD0@T9203Hf=P2PX:00XZ:R05eMG@0/;2`0Jf][
03hn?P14A4@0GUiN05IFEP1DE5@0RXZ:04i>CP0a<C40De=C059BDP0h>3P0Ti>C04e=C@1]KFd0=cLg
04Q8B01JFUX0FEUI08b<S01KFe/09bLW04E5A@1BDU80GemO06ATI00l?3`0@D5107UiN@27QhL0Fe]K
0:6QX@1mOGd0;bl_04Y:BP26QXH0LG5a04i>CP1UIFD0?Cdm02TY:@0m?Cd0RHV904a<C01EEED0X:2P
05mOG`17AdL0GemO0451@@0R8R80IFEU02h^;P2>SXh0IfMW0451@@2ZZZX0@T9206MWI`2BTY80EUIF
03Lg=`1NGUh0Fe]K05=CD`0W9bL0JVYZ09:BTPkooooo00/0XZ:R09JFUP2JVYX0[:b/07YjNP2VYZH0
Ti>C01lO7`0F5QH05ADE00h>3P02010@400b01<C4`1RHV80<S8b01PH600o?cl0dM7A0;jn_P24Q8@0
Lg=c06=SH`0M7Ad0<C4a03/k>`1]KFd0P82008V9R@20P800J6QX05IFEP18B4P0BDU90451@@2GUiL0
e=CD0;fm_@29RHT0C4a<04=3@`0b<S80;2`/030`<01SHf<0Jf][08b<S01[Jf/0;2`/08V9R@1:BTX0
_;bl09NGU`1JFUX0[:b/0;^k^`0V9RH04A4A00l?3`0>3Ph03`l?00h>3P0?3`l300h>3P0o010@400G
5aL0EeMG08b<S01LG5`0QHF506i^KP1lO7`0U9BD08Z:RP0a<C406QXJ01LG5`0E5AD05aLG01DE5@1l
O7`0clo?0<30`03YjNT0dm?C0<O7a`35aLD0TI6A05eMG@1GEeL0EEEE051@D01ADE40@T9203Hf=P1a
LG40FEUI034a<@1fMWH0[jn_09:BTP12@T80?clo03`l?00i>CT0?Shn03lo?`1IFET0;Bd]04U9B@0O
7al05aLG01XJ6P1?Cdl0E5AD05QHF01SHf<0MGEe06MWI`1@D500VYZJ03hn?P1YJFT0d];B07emO@1E
EED0MgMg0080?Shn01H0Cdm?05YJFP1nOWh0fMWI0;2`/02UYJD0Zj^[03hn?P15ADD0<c<c02DU9@0J
6QX05ADE01/K6`0V9RH0ATI605aLG01aLG40`/;208n?S`24Q8@0:BTY0P0J6QX03`0l?3`0EeMG06IV
IP1>CTh0>CTi04];B`16ATH0@41003Hf=P0l?3`0De=C08J6QP2ATI40OGem01`L700201XJ6P0<01lO
7`0U9BD0GemO03Hf=P1EEED0OGem09VIV@18B4P0:BTY01LG5`0Z:RX0/k>c0P1=CDd0;01OGel0UiNG
08^;R`0o?cl0;Rh^04Q8B00L71`05aLG01DE5@0H61P06ATI02TY:@1oOgl0XZ:R06e]K@2IVIT0FUYJ
07]kN`1bLW80R8R80;Rh^01XJ6P09RHV01lO7`0M7Ad0820P030`<01JFUX0];Bd06=SH`37alL0^KVi
09bLW01YJFT0OWin096AT@0Q8B409bLW02h^;P14A4@0TY:B0<W9b@27QhL0eMGE0P3MgMd00`32`/80
De=C01HF5P0201LG5`806ATI01L071`L030`<01YJFT05aLG01/K6`0I6AT06a/K02LW9`1LG5`0JFUY
04];B`1ADE40Lg=c061PH010@400DU9B0;Jf]P1iNGT0Jf][02DU9@1PH600@T92034a<@02030`<00>
05aLG01KFe/0OGem071`L01FEUH0?3`l03dm?@22PX80NWYj02@T900S8b<0;Rh^07=cL`0^;Rh@oooo
o`003_ooool0o`1YJFT0VYZJ055AD@1=CDd0W9bL04e=C@19BDT0YJFU08^;R`1DE5@0T92@0410@00`
<300J6QX02LW9`0i>CT0Cdm?07EeM@1=CDd0I6AT04I6AP1`L700MgMg03hn?P1iNGT0MgMg05=CD`1W
IfL0I6AT04e=C@1oOgl0BDU905eMG@1KFe/0=CDe04];B`1GEeL0>c/k08n?S`0m?Cd0@D5103dm?@1A
DE40;Rh^07EeM@1?Cdl0?Shn05]KF`0h>3P0<C4a02/[:`0b<S80LG5a059BDP1WIfL0>3Ph07AdM01>
CTh0@D5107IfMP1NGUh0NWYj055AD@1>CTh0<C4a07emO@1EEED0CTi>06UYJ@1PH600WinO08J6QP1I
FET0OGem06MWI`0c<c<0EeMG03hn?P2c/k<0L71`059BDP0Q8B40LW9b04e=C@2LW9`0Q8B404];B`0m
?Cd0Y:BT05QHF020P800Zj^[03`l?019BDT0Cdm?06ATI00a<C40RXZ:07EeM@0l?3`0LW9b04U9B@1j
NWX0E5AD08b<S01SHf<0GUiN07UiN@27QhL0IVIV092@T01^KVh0P82006=SH`1XJ6P0Hf=S07IfMP2N
WYh0Cdm?051@D01gMgL0DE5A079bLP1FEUH0T92@04E5A@2DU9@0Shn?051@D01UIFD0AdM707emO@1@
D500M7Ad06EUI@0n?Sh0W9bL06i^KP0o?cl0=3@d07]kN`0_;bl0LG5a06i^KP12@T80I6AT08n?S`0b
<S80Ng]k09VIV@14A4@0N7Qh02PX:010@400G5aL06MWI`2?Shl0NGUi04e=C@2GUiL0X:2P07UiN@2M
WId0/[:b07AdM01YJFT0XJ6Q08N7Q`19BDT0UiNG07=cL`1=CDd0^KVi08^;R`0a<C40FUYJ07EeM@12
@T80LW9b04m?C`1BDU80NGUi05EEE@1aLG40>3Ph08B4Q01OGel0CTi>05ADE00W9bL0Jf][096AT@1J
FUX0<c<c08Z:RP1DE5@0TI6A061PH00o?cl0L71`0::RXP1VIVH0e=CD0<k>cP1TI6@0ATI60:n_[`2h
^;P0Ogmo04E5A@2DU9@0JVYZ065QH@1bLW80@41005eMG@2@T900IFEU03Lg=`1ADE40IfMW02LW9`1M
GEd0EEEE06ATI00n?Sh0@T9206UYJ@0i>CT0Rh^;06][J`0i>CT0FEUI05MGE`1QHF40NWYj05MGE`1X
J6P0RHV9096AT@1FEUH0`l?309FEU@0`<300C4a<08B4Q00m?Cd0=3@d04Q8B00];Bd09bLW03Ti>@1g
MgL0:b/[03dm?@005`1kNg/0AdM705EEE@2[Zj/0S8b<02d];@1RHV805aLG04i>CP1SHf<0@41009nO
W`15ADD0NGUi06QXJ01>CTh0?Shn08N7Q`21PH40B4Q804i>CP2HV9P0X:2P00kooooo00X0Rh^;05QH
F01IFET0HV9R0;Fe]@3@d=00VYZJ03Ti>@1FEUH0;bl_0P0b<S80;@0S8b<08R8R06YZJP12@T80IFEU
09FEU@2IVIT0_Kfm0<W9b@0f=SH0<30`0;^k^`2g]kL0SXj>05ADE013@d<0PH6108n?S`20P800<C4a
02XZ:P0a<C40Fe]K05aLG02UYJD0VYZJ08f=S@10@400:2PX01XJ6P0M7Ad0G5aL08n?S`1RHV80:2PX
01TI6@0/;2`0PH610:BTY02][Jd0X:2P08R8R02PX:00[jn_03@d=00200d=3@0300h>3P0=3@d03@d=
00<03Ph>03h04Q8B014A4@0E5AD0:2PX05ADE010@400XZ:R0;Fe]@2WYjL0VYZJ09bLW021PH40EEEE
02`/;00O7al0:BTY04M7A`2DU9@0ZJVY0<71`@3>c/h0PH6106a/K03:b/X0;bl_01DE5@0G5aL04Q8B
00h>3P0@410092@T075aL@22PX808B4Q01HF5P1QHF40TI6A0:BTY01?Cdl0>SXj02h^;P0U9BD0<30`
03`l?01/K6`0De=C02PX:00n?Sh08B4Q01XJ6P0_;bl0WYjN0<71`@2HV9P0YjNW0:VYZ@27QhL0[jn_
05QHF030`<00NGUi01`L7080820P01005aLG01dM7@0N7Qh0820P02<S8`25QHD0^;Rh059BDP31`L40
Rh^;034a<@0P82006QXJ01/K6`0L71`07AdM0P0L71`02`0];Bd0I6AT0=OGe`2k^k/0WYjN051@D00M
7Ad0>3Ph071`L019BDT06QXJ00805ADE01@05aLG04A4A00o?cl0FUYJ06e]K@1FEUH0?3`l0:JVYP2@
T9007QhN01`L700I6AT07QhN01dM7@0e=CD0BDU908F5Q@2LW9`0JVYZ02LW9`805ADE04@0Fe]K0=KF
eP3Fe]H0Vi^K06e]K@1JFUX0RXZ:0:NWY`14A4@0?Shn01hN7P0G5aL05QHF01PH600Q8B40LG5a0:VY
Z@2HV9P0GEeM08B4Q01=CDd0HF5Q0;>c/`1eMGD0ZJVY07alO016ATH07QhN01lO7`0S8b<0<S8b06QX
J02QXJ40IFEU09FEU@21PH40[jn_0:6QX@31`L40BTY:01hN7P0U9BD0:RXZ02h^;P1=CDd0MgMg0<W9
b@2YZJT0h>3P0=[JfP2g]kL0QHF501dM7@0L71`06QXJ01DE5@0L71`06a/K03/k>`1MGEd071`L01/K
6`0M7Ad0<30`04Q8B02?Shl0K6a/055AD@80IFEU01X0I6AT05iNGP1IFET0BDU903@d=00Y:BT0?clo
07emO@0e=CD0=cLg02DU9@0a<C40Ph>307EeM@1KFe/0JFUY06EUI@0h>3P0:RXZ04e=C@25QHD0=cLg
02HV9P1DE5@0ZZZZ03Xj>Q3ooooo000>ooooo`3N06QXJ02LW9`0CTi>04i>CP24Q8@0B4Q804M7A`1d
M7@092@T05=CD`1nOWh0>3Ph02`/;01ZJVX09bLW03Xj>P1JFUX0KVi^03/k>`1]KFd0@d=307MgM`1c
Lg<0ADE507moO`1_Kfl0EEEE07AdM01RHV80CDe=07=cL`15ADD0HV9R04E5A@0R8R80ATI605QHF00n
?Sh0Zj^[028R8P1<C4`0?3`l04Q8B00b<S80MWIf04e=C@11@D40B4Q802l_;`0l?3`0@T9203De=@1W
IfL0D51@04A4A00c<c<0SHf=04I6AP15ADD0Kfm_03De=@1UIFD0OGem04i>CP0^;Rh0OGem04U9B@12
@T80K6a/05aLG02e]KD0J6QX02/[:`1=CDd0Cdm?02<S8`1iNGT0DU9B0;bl_01gMgL0CTi>02@T9028
R8P0=cLg08>3P`1fMWH0=cLg04A4A038b<P0AdM705QHF01/K6`0CDe=051@D021PH40N7Qh02d];@1[
Jf/0BTY:03Ph>020P800>SXj04i>CP1=CDd0Zj^[04I6AP19BDT0IfMW05IFEP2;Rh/0O7al03lo?`1U
IFD0CTi>09>CT`1RHV80ADE505mOG`10@400Ng]k09jNWP17AdL0C4a<04e=C@2JVYX0D51@069RHP1A
DE409bLW08>3P`1=CDd0MgMg02l_;`1OGel0X:2P03Xj>P1bLW80H61P03`l?01IFET0OGem03hn?P23
Ph<0FUYJ04M7A`1oOgl0YZJV05aLG01>CTh0Kfm_04];B`24Q8@0GUiN055AD@1MGEd0IfMW0;2`/01c
Lg<0GUiN0;:b/P27QhL0<C4a07=cL`2i^KT0MgMg06=SH`2TY:@0YJFU06MWI`2EUID0E5AD055AD@30
`<00RXZ:03lo?`1eMGD0QhN704A4A01`L700H61P03Ph>020P800FUYJ05aLG00^;Rh0R8R807AdM01E
EED0C4a<02h^;P1iNGT0OWin05EEE@0U9BD0L71`04=3@`1EEED0>3Ph02d];@1`L700Xj>S059BDP2K
Vi/0TI6A08V9R@1IFET0_kno0:RXZ02FUYH09RHV05iNGP1VIVH0Hf=S0861P@1JFUX0Cdm?09NGU`1^
KVh0>c/k04a<C01NGUh0>CTi0P1cLg<02`1]KFd0<c<c03`l?01nOWh0DE5A08F5Q@1ZJVX0?clo061P
H01?Cdl0@T920080MGEe00X0HF5Q0:FUY@2NWYh0=3@d08F5Q@20P800;Rh^04Q8B029RHT0KVi^0P1D
E5@07@0W9bL0?Cdm07EeM@2UYJD0L71`03hn?P1YJFT04A4A03Xj>P1GEeL0E5AD02HV9P1QHF409BDU
06QXJ01WIfL0@T920:VYZ@1PH600OWin061PH00k>c/0?3`l07YjNP1dM7@0?Shn02<S8`12@T80O7al
00kooooo03/0TY:B04];B`1IFET0KFe]0<K6aP2f][H0]KFe0:JVYP1@D5008B4Q02l_;`0i>CT0Cdm?
04Y:BP2CTi<0Rh^;05YJFP0m?Cd0@T9207emO@1gMgL0SHf=08F5Q@2>SXh0De=C05QHF016ATH0;2`/
024Q8@0`<300WIfM07UiN@1`L700GUiN059BDP1IFET0R8R80<[:bP2VYZH0VIVI08R8R017AdL0A4A4
06EUI@1WIfL0BTY:04m?C`1LG5`0F5QH06IVIP2l_;`0G5aL0:ZZZP2EUID0?3`l07emO@14A4@03@d=
00h>3P0300d=3@0`00h>3P0C4a<03Ph>014A4@0C4a<0410@01dM7@19BDT05ADE020P801DE5@0VIVI
0<[:bP2d];@0Y:BT0;fm_@2DU9@0BTY:0410@013@d<0RXZ:0=?Cd`2DU9@0Rh^;0:>SX`21PH40S8b<
06a/K00W9bL05ADE018B4P0A4A40410@00l?3`0X:2P0MWIf01XJ6P0I6AT0FUYJ07IfMP1eMGD0L71`
06UYJ@1EEED0<c<c02PX:00^;Rh0FEUI0P1MGEd01@0];Bd0@d=302LW9`0T92@0RXZ:0080Xj>S02H0
R8R806i^KP0b<S80Ng]k0;^k^`2OWil0ATI601TI6@0G5aL05ADE01HF5P0E5AD06QXJ01`L700U9BD0
GUiN08:2PP1;Bd/0Vi^K04m?C`0N7Qh071`L01XJ6P0L71`06a/K01`L700K6a/06QXJ01LG5`0i>CT0
X:2P09>CT`2=SHd0RXZ:075aL@1hN7P0EEEE01`L70@05ADE00`08B4Q0451@@1nOWh0DU9B02<S8`0i
>CT0][Jf0=CDe025QHD0HF5Q04a<C00o?cl203De=@0c0:RXZ02e]KD0Lg=c07YjNP0I6AT05ADE030`
<024Q8@0CDe=08:2PP2PX:00MgMg04U9B@1BDU80UIFE0:n_[`1PH600:2PX01LG5`0H61P08B4Q08Z:
RP1mOGd0UIFE09NGU`14A4@0>c/k06a/K024Q8@0_Kfm0;Bd]02MWId0[Jf]06e]K@1;Bd/0GemO04M7
A`1=CDd0PH6108b<S024Q8@0NWYj06i^KP1EEED0TY:B0;2`/00g=cL0820P02<S8`0/;2`0B4Q80080
Lg=c00X0][Jf07YjNP1oOgl0dm?C0;bl_02JVYX0KFe]01/K6`0H61P06ATI0P0M7Ad09P19BDT0EUIF
01dM7@0X:2P0@41004m?C`1_Kfl0Z:RX09NGU`1`L700B4Q801XJ6P0I6AT0CDe=04m?C`1QHF40>c/k
030`<025QHD0>CTi034a<@0T92@0:b/[04I6AP1`L700BTY:05]KF`1jNWX0M7Ad03Ph>00_;bl0>c/k
07=cL`1ZJVX0?Shn0451@@1ADE40Jf][4?ooool000kooooo0480GUiN08>3P`1GEeL0Bd];06i^KP1<
C4`0I6AT0:VYZ@1WIfL0@d=305aLG00j>SX0?3`l04E5A@0/;2`0E5AD04a<C00j>SX0:b/[06MWI`0i
>CT0Kfm_03hn?P0^;Rh0E5AD06a/K01?Cdl0Lg=c05=CD`0i>CT0EUIF04=3@`1cLg<0?clo02@T9017
AdL0E5AD03Hf=P2KVi/0>3Ph0451@@0i>CT0F5QH02DU9@1KFe/0B4Q803/k>`1DE5@0<c<c02TY:@0g
=cL0:b/[061PH01FEUH0?Shn02LW9`1FEUH0>CTi04E5A@1_Kfl0@41004=3@`1UIFD0@d=303@d=01n
OWh204M7A`0B05MGE`0k>c/0TY:B06e]K@0l?3`0GEeM03Lg=`0[:b/0NGUi0410@02FUYH0R8R802HV
9P14A4@0Lg=c03Hf=P2JVYX0QXJ60P0M7Ad08@1kNg/0ATI605eMG@1kNg/0820P04m?C`1/K6`0M7Ad
030`<01jNWX0@410028R8P1NGUh0=3@d04=3@`0m?Cd0QhN705eMG@1<C4`0H61P05=CD`1?Cdl0NGUi
05EEE@1@D500>3Ph05UIF@1NGUh0CDe=071`L00h>3P0BDU9069RHP0205QHF00H03lo?`1gMgL0?3`l
06EUI@1cLg<0BTY:07AdM01BDU80HV9R0492@P1[Jf/0NWYj03dm?@1lO7`0IFEU02PX:00e=CD0MWIf
03Ti>@25QHD0Fe]K03Hf=P18B4P0NWYj0P1GEeL0P@1DE5@0:2PX06a/K01QHF40FUYJ06][J`1@D500
[Jf]09:BTP1VIVH0^k^k08B4Q01^KVh0RHV90:RXZ01]KFd0IVIV0:ZZZP2a/K40K6a/09JFUP1VIVH0
J6QX0=;BdP2TY:@0CTi>07emO@1YJFT0Jf][08V9R@1SHf<0;bl_05YJFP1EEED0N7Qh03`l?01nOWh0
NWYj04i>CP1FEUH0E5AD09:BTP2>SXh0HV9R03De=@2FUYH0EEEE05]KF`11@D40>3Ph06][J`2GUiL0
DU9B0;Ng]`1nOWh0JFUY0451@@36a/H0X:2P069RHP0d=3@0IfMW0492@P1MGEd0NGUi06ATI01KFe/0
R8R805eMG@18B4P0KFe]06EUI@0m?Cd0QhN7092@T01TI6@0FUYJ05mOG`22PX80DE5A08j>SP1/K6`0
?clo071`L01]KFd0BDU905UIF@1CDe<0Ogmo0=;BdP2/[:`0GUiN07alO01ZJVX08B4Q07alO03:b/X0
R8R806=SH`1IFET06ATI02/[:`11@D40QHF505iNGP0j>SX0De=C04i>CP10@400F5QH055AD@0S8b<0
IVIV02`/;01jNWX0?3`l03Ti>@29RHT0K6a/07MgM`1YJFT06a/K0492@P2YZJT0Zj^[04M7A`0f=SH0
D51@08N7Q`0>ooooo`0608F5Q@1<C4`0Bd];06IVIP1^KVh0@4100P20P8000`0_;bl06ATI01@D5002
01<C4`0902@T901SHf<0HV9R04=3@`0^;Rh0:2PX04M7A`0S8b<0Fe]K0080W9bL0280=CDe01lO7`0_
;bl0820P01LG5`0E5AD0NWYj07emO@1?Cdl0=3@d02`/;00X:2P0G5aL0<C4a02d];@0X:2P0;jn_P1G
EeL0:BTY05MGE`22PX80K6a/04e=C@0j>SX08b<S02DU9@2OWil0KVi^0;bl_01?Cdl071`L059BDP1D
E5@03Ph>1@0=3@d0800@41003Ph>00h>3P0@41004A4A04E5A@1SHf<06ATI01DE5@0F5QH0820P05IF
EP2NWYh0]KFe0;2`/02WYjL0V9RH04I6AP0a<C40VIVI0;>c/`23Ph<0Cdm?09VIV@2OWil0OWin06YZ
JP1QHF40GUiN05EEE@0n?Sh09RHV0P0G5aL07P2CTi<0<S8b04A4A01`L700QXJ604E5A@0F5QH061PH
02XZ:P1HF5P0FEUI03Lg=`18B4P0EeMG04Q8B01LG5`0=SHf04A4A01`L700R8R80:FUY@1fMWH0Ogmo
04m?C`0i>CT0CDe=06ATI026QXH0CDe=02@T90@05ADE01T05aLG01XJ6P0S8b<0H61P08n?S`13@d<0
DE5A04m?C`0I6AT061PH01`L700H61P071`L01XJ6P0I6AT05QHF01PH600U9BD0NGUi09:BTP2HV9P0
TY:B0<[:bP1VIVH0:2PX00805ADE07P04a<C01DE5@0E5AD05ADE03De=@18B4P0>CTi02LW9`17AdL0
a/K60;6a/@0m?Cd0@T9205IFEP1WIfL0Ph>309jNWP3Ph>00WinO075aL@1eMGD07QhN05UIF@24Q8@0
>CTi0451@@0m?Cd0<30`051@D01RHV80B4Q804a<C027QhL0[Zj^07]kN`0[:b/08B4Q06i^KP1PH600
F5QH0:2PX02[Zj/0WinO071`L01lO7`0P82008R8R025QHD0O7al08b<S01mOGd0Rh^;0:RXZ02a/K40
Z:RX0:2PX02_[jl0Ti>C069RHP1HF5P0P8200<;2`P1<C4`0ADE504e=C@14A4@0GEeM092@T02:RXX0
GUiN09>CT`0_;bl0?Cdm07inOP35aLD0alO709^KV`1EEED0<C4a03/k>`17AdL0B4Q806ATI01NGUh0
CDe=06m_K`21PH40N7Qh0:n_[`3:b/X0LW9b02@T900G5aL05ADE01LG5`0O7al0EEEE059BDP1TI6@0
EeMG08b<S00m?Cd0;Bd]01lO7`0g=cL0>SXj02DU9@0_;bl0=cLg05eMG@1kNg/0EEEE03@d=00b<S80
Hf=S096AT@1eMGD0:BTY01dM7@0[:b/@ooooo`003_ooool0IP1UIFD0KFe]04Y:BP1;Bd/0Lg=c0451
@@15ADD0RHV9065QH@11@D40>SXj02/[:`1DE5@0GEeM02@T900d=3@0CTi>04Q8B00M7Ad0FEUI030`
<02QXJ40H61P034a<@1TI6@0BDU904A4A026QXH0L71`04a<C011@D40:2PX05EEE@1CDe<0;Rh^04Y:
BP18B4P0:b/[07alO00l?3`0;Rh^02l_;`1PH600<c<c06MWI`0/;2`0B4Q8071`L01BDU80=3@d02d]
;@0Y:BT0OWin07YjNP1SHf<0:RXZ06][J`0f=SH0IfMW08f=S@1LG5`0?Cdm079bLP0W9bL0>SXj08F5
Q@1UIFD0BDU904a<C018B4P0W9bL05UIF@0[:b/0F5QH04m?C`0Q8B40JVYZ038b<P1lO7`0Fe]K03`l
?00R8R80KFe]02l_;`2FUYH0WinO038b<P0V9RH0YJFU04];B`1WIfL0W9bL03/k>`0m?Cd0GUiN07Ad
M00d=3@0Lg=c05ADE00b<S80JVYZ02/[:`80ADE50140RXZ:05ADE019BDT0GemO07EeM@1TI6@0PH61
05MGE`1TI6@0H61P07=cL`1SHf<0DU9B0861P@1ADE40De=C06][J`0205YJFP24051@D01kNg/0BDU9
07alO02>SXh0CTi>07IfMP1XJ6P0SHf=05QHF01cLg<0L71`03hn?P1jNWX0KVi^0410@019BDT0P820
04E5A@2UYJD0P8200451@@1ADE40R8R803`l?026QXH0Vi^K04M7A`1LG5`0;2`/06UYJ@1dM7@0LG5a
0;:b/P2<S8`0MgMg0<C4a02VYZH0Rh^;08B4Q02/[:`0P82009RHV03=cLd0];Bd06a/K02EUID0BTY:
05YJFP2e]KD0MgMg0492@P2RXZ80Q8B403dm?@1GEeL0DU9B02PX:01SHf<0JFUY08b<S01:BTX0PX:2
06MWI`17AdL0Hf=S04e=C@2?Shl0WinO05UIF@1QHF40`<3005UIF@1PH600@D5104U9B@2?Shl0]KFe
065QH@33`l<0QhN706IVIP13@d<0a/K60;^k^`1/K6`0=3@d05iNGP15ADD0IVIV09nOW`1QHF40GEeM
07IfMP0M7Ad0<c<c051@D01DE5@0C4a<0:2PX02>SXh0BTY:034a<@18B4P0MWIf04Q8B02@T900?Shn
03De=@1aLG40MgMg05ADE014A4@09bLW05ADE028R8P0RXZ:06UYJ@1mOGd0B4Q801<C4`1=CDd0QHF5
04i>CP1@D500J6QX02<S8`0d=3@0A4A406UYJ@15ADD0>SXj0P1LG5`05@1;Bd/0E5AD05UIF@0b<S80
NGUi02PX:01`L700DU9B03@d=02<S8`0K6a/07MgM`1[Jf/0BTY:038b<P1IFET0J6QX05MGE`0i>CT0
DE5A0861P@0>ooooo`0j08N7Q`1<C4`0B4Q806ATI00e=CD0OWin09FEU@1LG5`0Kfm_02HV9P0E5AD0
5aLG018B4P0A4A40>c/k03lo?`0i>CT0:2PX03`l?00d=3@04Q8B08>3P`27QhL0Xj>S096AT@11@D40
7alO02TY:@0[:b/0EEEE09fMW@1>CTh061PH01hN7P0L71`08b<S03Lg=`39bLT0a<C40<G5a@3=cLd0
i>CT0=gMg@3Ng]h0S8b<05aLG01ZJVX0N7Qh0820P02?Shl0/[:b06QXJ00d=3@0Cdm?01hN7P12@T80
IfMW014A4@803@d=00<030`<00d=3@0=3@d00P0>3Ph07P0?3`l04Q8B02h^;P0k>c/0;Rh^04e=C@0C
4a<0410@01DE5@0G5aL0ATI609fMW@2RXZ80]kNg0=3@d032`/80ZJVY05ADE00U9BD0H61P0:BTY02@
T900WIfM07alO01aLG40A4A402XZ:P0`<300>3Ph05IFEP80IfMW01l0/[:b0:FUY@2m_Kd0U9BD07Yj
NP1OGel0:BTY01XJ6P0I6AT05ADE01XJ6P1:BTX0BDU904Q8B014A4@0@T92065QH@1KFe/0YZJV08J6
QP30`<00Cdm?05QHF01^KVh0F5QH04E5A@2CTi<0ATI603dm?@16ATH092@T00805ADE00/05QHF01LG
5`0E5AD061PH03@d=024Q8@0@d=304U9B@21PH40820P01PH600201LG5`1[01`L700I6AT06ATI01TI
6@0G5aL06a/K04E5A@34a<@0dM7A0:j^[P2QXJ40>CTi024Q8@0E5AD04a<C01HF5P0E5AD05aLG01DE
5@0l?3`0AdM704E5A@0d=3@0KFe]09:BTP1bLW8071`L01XJ6P0M7Ad0820P02HV9P1EEED0dm?C0<W9
b@2^[Zh0WinO0:6QX@1jNWX0:b/[030`<00];Bd07QhN020P800R8R80:b/[04I6AP18B4P0FEUI05mO
G`2RXZ80XJ6Q0:j^[P28R8P0A4A4065QH@2SXj<0@D5103Ph>01NGUh0OGem08:2PP26QXH0Ogmo09FE
U@2a/K40`<300;Fe]@2g]kL0_kno0<30`03GemL0eMGE0:RXZ021PH40O7al0;Vi^@1bLW80820P01dM
7@0Q8B40<c<c079bLP2m_Kd0CDe=038b<P1lO7`0820P03Lg=`0k>c/0Y:BT0;^k^`1iNGT0Q8B407=c
L`1CDe<0Jf][071`L02RXZ80SXj>07moO`2?Shl0UiNG06][J`1mOGd0[:b/05EEE@0J6QX00`0E5AD0
6@0G5aL0DU9B03lo?`1SHf<0d=3@069RHP12@T80:2PX02DU9@0Z:RX07alO02PX:00W9bL0=CDe05YJ
FP1dM7@0JVYZ08B4Q020P800OGem0;>c/`2_[jl0?Shn03Lg=`0/;2`04?ooool000kooooo0?l0IfMW
0:^[Z`1=CDd0De=C09RHV01EEED0FEUI09ZJVP1]KFd0CTi>08>3P`0/;2`09BDU055AD@0V9RH0:RXZ
03Ph>01HF5P0?Shn06=SH`0O7al0KFe]05mOG`0j>SX0JVYZ0410@00i>CT0NWYj06EUI@15ADD0Hf=S
03lo?`1HF5P0DU9B03De=@1EEED0Bd];03lo?`2EUID0AdM70492@P13@d<0Fe]K03<c<`1LG5`08b<S
03/k>`11@D40BDU902l_;`1BDU80;bl_05iNGP18B4P0C4a<02l_;`1YJFT0;Bd]03De=@1[Jf/0FEUI
034a<@1mOGd0:RXZ02PX:01fMWH0AdM703Ph>01NGUh0EUIF0:2PX01RHV80;2`/07AdM01YJFT0:RXZ
065QH@0Y:BT0OGem05iNGP13@d<0;2`/07MgM`1RHV80^;Rh09VIV@1KFe/0<S8b0:j^[P1=CDd0IFEU
08n?S`15ADD0?Shn06=SH`1ZJVX0;bl_08Z:RP1`L700?3`l07MgM`0_;bl0EUIF03hn?P29RHT0ADE5
06EUI@23Ph<0Lg=c06=SH`1cLg<0Cdm?08^;R`1RHV80KVi^05mOG`1fMWH0SHf=04];B`1@D500KFe]
05ADE01iNGT0EEEE08j>SP0m?Cd0Fe]K075aL@1=CDd0@D5103/k>`1`L700GUiN07UiN@1kNg/0B4Q8
0:JVYP1kNg/0@d=3061PH01aLG40>3Ph07QhN01dM7@0>c/k051@D02?Shl0DE5A04Q8B01dM7@0@410
07AdM00];Bd0NGUi08:2PP1`L700][Jf071`L01UIFD0^k^k0:JVYP25QHD0Shn?092@T01@D500EUIF
0;Fe]@2^[Zh0MgMg0:>SX`1SHf<0KVi^0<o?c`23Ph<0=CDe08J6QP22PX80@d=3075aL@1FEUH0?3`l
07IfMP1TI6@0TI6A04Y:BP20P800LG5a03/k>`1dM7@0ATI60820P02WYjL0Fe]K04Y:BP2VYZH0FUYJ
0:FUY@12@T80;Bd]05UIF@2MWId0KFe]0;no_`1fMWH0Ogmo02h^;P2DU9@0_kno09VIV@0k>c/0IfMW
02TY:@0h>3P0NWYj07moO`1eMGD0W9bL06=SH`11@D40EeMG05mOG`0U9BD0OWin07alO01UIFD0;bl_
03`l?01gMgL0BTY:09NGU`1XJ6P0A4A4069RHP1^KVh0GEeM04e=C@1FEUH0KFe]09JFUP23Ph<0Fe]K
07moO`23Ph<0=SHf059BDP1kNg/08B4Q03Hf=P1jNWX0E5AD03lo?`16ATH0L71`03Hf=P1RHV8001L0
V9RH05QHF01JFUX0DU9B04M7A`0O7al0GEeM02/[:`23Ph<0Fe]K034a<@21PH40BDU9075aL@2VYZH0
M7Ad03hn?P1cLg<0IFEU04a<C01_Kfl0O7al08R8R00>ooooo`070;jn_P1>CTh0ATI605=CD`2CTi<0
OWin04Q8B00204A4A00d04m?C`0I6AT05QHF01DE5@0I6AT0Cdm?04I6AP0^;Rh0?clo04m?C`0M7Ad0
ADE504I6AP0`<300AdM70;Ng]`38b<P0YjNW09^KV`2JVYX0e]KF07alO00K6a/07AdM01TI6@0E5AD0
8B4Q04i>CP29RHT08b<S051@D02[Zj/0f][J0=gMg@3<c<`0UYJF07MgM`1SHf<0ATI603Xj>P0Y:BT0
:2PX08b<S00[:b/0<30`03Xj>P0g=cL0LG5a01@D500>3Ph03@d=00`<30@03@d=04<03Ph>024Q8@1Z
JVX0:b/[018B4P24Q8@0H61P04U9B@0l?3`0De=C0:>SX`2IVIT0Lg=c079bLP2UYJD0dM7A08:2PP0Z
:RX06ATI03Ti>@1dM7@0R8R805aLG01VIVH0LG5a05]KF`19BDT0?3`l030`<00Y:BT0:2PX02d];@2C
Ti<0/;2`09nOW`2VYZH0dm?C02`/;00`<300@T9203De=@0N7Qh05aLG01PH600i>CT0FEUI04M7A`1P
H600IFEU0;no_`2j^[X0c/k>0=gMg@2;Rh/0CTi>07moO`2BTY80OWin0;Ng]`1IFET0K6a/03dm?@0_
;bl0:RXZ01HF5P0E5AD05aLG00805ADE01L06ATI05UIF@1DE5@0KFe]0;Bd]01SHf<0Fe]K03Ti>@0Z
:RX06a/K01XJ6P0G5aL05ADE01HF5P0P8200IFEU0=kNgP3Jf]X0emOG0=KFeP2RXZ80?Shn01PH6004
01DE5@0h01LG5`0O7al0J6QX04i>CP1OGel0/K6a05mOG`1WIfL06QXJ01PH600M7Ad0820P04m?C`2=
SHd0Fe]K08^;R`2HV9P0`l?30;Fe]@0S8b<0:2PX02l_;`0I6AT06a/K01lO7`0Q8B408b<S02TY:@18
B4P0IfMW04];B`1^KVh0`l?30;6a/@2>SXh0RXZ:0<30`02JVYX0GEeM04];B`1NGUh0EUIF0492@P1P
H600SHf=059BDP18B4P0YjNW0;^k^`2=SHd0[Zj^0:BTY02j^[X0_Kfm0;^k^`2`/;020=KFeP0Q03De
=@0S8b<08R8R01dM7@0N7Qh0>3Ph07MgM`1JFUX0?Shn07UiN@0S8b<0ATI605=CD`2BTY80EEEE05YJ
FP2ATI40_;bl0=OGe`2?Shl0D51@0<_;b`1gMgL0=CDe02<S8`0X:2P0=CDe03Ti>@1/K6`0;Rh^01LG
5`0E5AD05aLG00805ADE01P0?Cdm05UIF@13@d<0Xj>S07UiN@0b<S806a/K02XZ:P0U9BD05ADE01/K
6`0I6AT0:2PX05mOG`1kNg/0G5aL0:f][@2k^k/0T92@0:FUY@33`l<0XJ6Q09ZJVP29RHT@ooooo`00
3_ooool02`1XJ6P0Ti>C04m?C`1<C4`0Lg=c04Y:BP1FEUH0Rh^;04U9B@1FEUH0MWIf0080:RXZ0280
EUIF02`/;01<C4`0ADE503<c<`0`<300IVIV02PX:02DU9@0CTi>03lo?`1QHF40FEUI05ADE02RXZ80
KFe]04];B`1iNGT0DU9B069RHP17AdL0:RXZ05MGE`1TI6@0@4100:>SX`0`<300AdM703lo?`1JFUX0
<30`071`L00o?cl0A4A40P1;Bd/0B@0m?Cd0GUiN03hn?P1aLG40FUYJ03hn?P0_;bl0NGUi04E5A@1A
DE40QHF504U9B@0m?Cd0M7Ad03/k>`0f=SH0RHV9055AD@16ATH0IfMW05=CD`2BTY80IFEU02d];@1?
Cdl0G5aL03De=@1aLG40Cdm?06=SH`1YJFT0BDU902h^;P1mOGd0:RXZ07emO@1gMgL0AdM702DU9@2_
[jl0GEeM05YJFP2<S8`0;bl_04E5A@1dM7@0P82003`l?024Q8@0D51@02d];@2<S8`0DU9B04i>CP0m
?Cd0UYJF05UIF@12@T80F5QH04E5A@1SHf<0Kfm_04E5A@1SHf<0B4Q808>3P`1SHf<0G5aL06=SH`0i
>CT0FEUI07emO@1JFUX00P1MGEd0NP2DU9@0GEeM07AdM01dM7@0AdM706m_K`1EEED0Ng]k04M7A`1]
KFd0W9bL04E5A@27QhL0R8R804a<C01eMGD0U9BD04E5A@2KVi/0PX:204I6AP1EEED0UiNG05YJFP1c
Lg<0OGem0492@P1mOGd0FEUI06UYJ@1^KVh0EEEE0;Ng]`2:RXX0L71`0<S8b02MWId0I6AT0:FUY@2F
UYH0J6QX07emO@3Cdm<0QhN706IVIP2FUYH0ATI608R8R03?cll0Lg=c04I6AP21PH40De=C04i>CP2<
S8`0JFUY02/[:`1JFUX0GemO06e]K@11@D40Vi^K07alO014A4@0R8R803De=@1eMGD0YZJV04Q8B01M
GEd0W9bL04m?C`1XJ6P0>c/k030`<01]KFd0^KVi06ATI02e]KD0VYZJ075aL@11@D40XZ:R09nOW`24
Q8@0>SXj07=cL`1PH600L71`096AT@1DE5@0IFEU08Z:RP18B4P0@d=3075aL@1FEUH0@41007emO@1f
MWH0ADE50492@P1UIFD0Ph>3030`<01eMGD0KVi^04e=C@1]KFd0M7Ad06=SH`1bLW80GUiN06MWI`2_
[jl0Z:RX09JFUP27QhL0EEEE02DU9@1XJ6P0UYJF0P1FEUH04`1HF5P0=SHf059BDP1_Kfl0UIFE05AD
E00g=cL0E5AD03Hf=P1JFUX0PX:206a/K00`<300H61P02LW9`1kNg/0De=C02`/;01lO7`00P1JFUX0
2@2ATI40Cdm?02h^;P1]KFd0Hf=S0410@013@d<0DU9B07EeM@0>ooooo`0R0=?Cd`2^[Zh0Vi^K08B4
Q017AdL0=CDe03hn?P0W9bL0;Rh^04M7A`1BDU80>3Ph01`L700G5aL0DU9B04E5A@0^;Rh0@T9203Hf
=P14A4@0?Shn02HV9P0O7al07QhN03hn?P2UYJD0R8R807]kN`22PX80`L7106a/K00L71`07AdM01hN
7P805QHF01P0@d=301lO7`0A4A40410@01PH602XZ:P0M7Ad02@T900E5AD04a<C00h>3P092@T03@d=
00/;2`0=3@d0CDe=07UiN@1GEeL0GUiN07moO`1kNg/0E5AD01HF5P0<30`200h>3P803@d=00@03Ph>
03@d=01cLg<0Hf=S0P0`<300P@0V9RH06QXJ02DU9@0_;bl0FUYJ0:VYZ@1SHf<06a/K024Q8@1EEED0
X:2P0451@@17AdL0>c/k02d];@0L71`0820P03De=@18B4P0IFEU04Q8B00g=cL0AdM704];B`1ADE40
GemO04I6AP2HV9P0];Bd09fMW@1XJ6P0Ng]k03Ti>@0E5AD05aLG01dM7@0Y:BT0<30`02PX:00T92@0
;Bd]05UIF@2i^KT0alO70<W9b@3Lg=`0aLG509fMW@2BTY80SHf=09BDU01gMgL0TI6A0;Zj^P1EEED0
>CTi02TY:@0e=CD0;Rh^03@d=00V9RH07AdM01PH600F5QH06ATI051@D01eMGD0LW9b02d];@0N7Qh0
7alO020P800Y:BT0<30`038b<P0c<c<0=SHf030`<00l?3`0RXZ:09^KV`2WYjL0ZZZZ0<?3``2][Jd0
Q8B407QhN01aLG40FUYJ038b<P0J6QX05ADE01HF5P1SHf<0WinO0<;2`P38b<P0QhN707AdM00/;2`0
<c<c0451@@1[Jf/0EeMG02`/;01=CDd0Xj>S09fMW@31`L40`l?30:2PX01BDU808b<S03<c<`14A4@0
@T9204I6AP0n?Sh0=SHf03Xj>P0e=CD0?Shn05aLG02YZJT0WinO07IfMP2TY:@0ZJVY0;2`/00206i^
KP0C065QH@0V9RH0?clo04E5A@0g=cL0A4A404];B`1RHV80Q8B40:6QX@2f][H0YZJV0;fm_@2HV9P0
KFe]03Lg=`1RHV80ZJVY04U9B@02020P800K01lO7`0Q8B40>SXj07=cL`22PX80OWin05]KF`0d=3@0
EUIF07emO@2`/;00HV9R04Q8B01:BTX0@d=308:2PP1QHF40BDU9092@T00V9RH071`L03<c<`1<C4`0
>SXj01XJ6P13@d<0>c/k00D05ADE01P0;2`/04a<C01<C4`0SXj>09bLW01OGel0?Cdm03Ti>@0`<300
?Shn04a<C01JFUX0L71`0::RXP2][Jd0Z:RX0:n_[`2^[Zh0YZJV09>CT`1UIFD0C4a<06MWI`1`L70@
ooooo`003_ooool0EP1ZJVX0O7al04];B`1@D500JFUY03Lg=`1>CTh0SHf=05iNGP1?Cdl0HF5Q03<c
<`19BDT0KVi^02/[:`16ATH0Bd];061PH00i>CT0@T9200h>3P2:RXX0GemO03lo?`1LG5`06QXJ02l_
;`1SHf<0Shn?04Q8B00i>CT0=SHf06ATI01IFET0<30`05YJFP1:BTX0<30`0861P@15ADD0>3Ph02<S
8`1BDU80:RXZ05aLG00E5AD0>CTi06QXJ01/K6`0ADE503Lg=`0U9BD0HV9R07QhN01WIfL0>SXj06IV
IP0/;2`0H61P09BDU01cLg<0AdM7061PH00V9RH0@d=309bLW01HF5P0CDe=05eMG@0j>SX0N7Qh07in
OP0l?3`0Cdm?06MWI`0[:b/0BDU906UYJ@1PH600MgMg04U9B@0];Bd0RHV902/[:`2ATI40V9RH0P0g
=cL0WP37alL0HF5Q07emO@2[Zj/0CTi>03@d=01?Cdl0IFEU03Ti>@2PX:00MgMg03Xj>P1dM7@0DE5A
06IVIP15ADD0TI6A05UIF@1;Bd/0Jf][06UYJ@1CDe<0Ph>3061PH01lO7`0GEeM06m_K`1IFET0Rh^;
08f=S@13@d<0B4Q806IVIP1>CTh0MgMg05=CD`27QhL0ADE508B4Q02ATI40@D5107AdM00h>3P0OWin
06ATI01jNWX0WYjN04U9B@2YZJT0O7al030`<01bLW80O7al055AD@2`/;00MGEe03<c<`17AdL0Ng]k
03@d=01CDe<0MWIf03Ph>01UIFD0:RXZ06QXJ01hN7P0BTY:09nOW`1TI6@0NWYj0=7Ad@2][Jd0FEUI
09NGU`1/K6`0@41008F5Q@34a<@0VIVI071`L02PX:00GEeM08b<S03?cll0DU9B04Y:BP2FUYH0QHF5
03lo?`1SHf<0Jf][02LW9`1BDU80GUiN07alO00m?Cd0L71`06IVIP0m?Cd0PX:204Y:BP1eMGD0];Bd
06a/K01?Cdl0H61P04Q8B01MGEd0=CDe02PX:01>CTh0RHV905UIF@2][Jd0LW9b065QH@0W9bL0Shn?
0:RXZ02<S8`0>c/k065QH@0f=SH0@T9208^;R`1bLW80F5QH07QhN00M7Ad0;bl_04i>CP1WIfL0B4Q8
06e]K@1nOWh0@d=302d];@0o?cl0MgMg03Ph>01dM7@0<30`02d];@1TI6@0P82006i^KP1]KFd09RHV
03/k>`1fMWH0VIVI08n?S`2l_;`0RXZ:02d];@1?Cdl0LW9b0P1CDe<07P1QHF40?3`l03dm?@15ADD0
L71`05iNGP0^;Rh0De=C055AD@1=CDd0HF5Q05iNGP0b<S80K6a/02<S8`1jNWX0>3Ph02XZ:P1YJFT0
DU9B04m?C`2OWil09BDU03/k>`2YZJT0ZZZZ05MGE`1@D500Bd];079bLPkooooo00D0eMGE0=[JfP3A
dM40FUYJ03Ti>@0203De=@0o02@T900P8200;Bd]03hn?P1OGel0O7al051@D02FUYH0HF5Q04Q8B01C
De<0@D5101hN7P0K6a/09BDU028R8P10@400T92@04a<C00U9BD06a/K01HF5P1oOgl0TY:B065QH@0Q
8B404Q8B01<C4`0H61P0=CDe014A4@0?3`l0410@018B4P21PH40BTY:01PH600E5AD03Ph>00X:2P0<
30`03@d=00T92@0E5AD0410@05eMG@1jNWX0[Jf]08:2PP27QhL0W9bL05iNGP0V9RH03`l?00`<300>
3Ph03`l?04=3@`1DE5@08R8R01TI6@0E5AD05QHF00805ADE03H07alO04e=C@28R8P0VIVI0861P@1N
GUh0IVIV0;Zj^P2c/k<0T92@024Q8@0C4a<04Q8B01/K6`0G5aL05ADE01HF5P0d=3@0Ogmo024Q8@0G
5aL08b<S03Lg=`1YJFT08b<S09bLW03>c/h0VIVI02XZ:P0W9bL0JFUY020P800E5AD05aLG01@D500E
5AD06ATI01`L700h>3P0K6a/0;Fe]@3Ng]h0[Zj^0<g=c@2ZZZX0P82008n?S`2>SXh0V9RH09FEU@34
a<@0R8R80861P@15ADD202/[:`1=034a<@0i>CT0@d=304e=C@0[:b/08R8R05=CD`1bLW80PX:209>C
T`0V9RH06a/K01`L700J6QX06a/K01XJ6P0G5aL071`L01dM7@0l?3`0V9RH06EUI@1_Kfl0][Jf0;>c
/`3Cdm<0[Jf]08R8R01KFe/0AdM706][J`2PX:00[jn_06EUI@0Z:RX0Cdm?08Z:RP3Bd]80_[jn0:ZZ
ZP1WIfL0>c/k0410@01oOgl0AdM704m?C`1BDU80ADE503dm?@2?Shl0clo?09fMW@2^[Zh0^[Zj06YZ
JP1ADE40<S8b01`L700S8b<0=SHf04a<C019BDT0?clo05iNGP2CTi<0VYZJ06EUI@23Ph<0Xj>S0:^[
Z`39bLT0N7Qh03lo?`0_;bl09BDU04I6AP0Y:BT00P0E5AD06P0G5aL0IFEU09nOW`1jNWX0Vi^K09ZJ
VP2@T900MGEe04M7A`0a<C40EUIF06a/K02>SXh07alO028R8P1ADE40CTi>04M7A`1EEED0Vi^K0;6a
/@1KFe/0E5AD0492@P2EUID0I6AT0P0P82003`0n?Sh0HF5Q06QXJ02QXJ40_[jn09:BTP1VIVH0C4a<
0861P@0Z:RX05ADE01XJ6P0N7Qh0GemO01LG5`0401DE5@0H0410@01?Cdl0>SXj0::RXP30`<00b/[:
0<;2`P2SXj<0ZJVY0;Ng]`37alL0c<c<0<g=c@2SXj<0KVi^0;bl_01kNg/0Rh^;09^KV`27QhL0;2`/
02HV9P15ADD0IfMW4?ooool000kooooo00D0LG5a0:RXZ01@D500Cdm?08>3P`0204m?C`1=096AT@1T
I6@0BDU903@d=00W9bL0B4Q805YJFP0Z:RX09RHV030`<01LG5`0=cLg06YZJP0`<300TY:B061PH01;
Bd/0NWYj07AdM01JFUX0MWIf061PH016ATH0K6a/059BDP1VIVH0<S8b02/[:`1KFe/0CTi>0451@@2=
SHd0>SXj03lo?`0j>SX0CTi>02`/;01KFe/08b<S03Lg=`10@400EeMG04I6AP0Z:RX09bLW06=SH`1]
KFd0EeMG03Xj>P1aLG408B4Q05eMG@2;Rh/0L71`04I6AP1VIVH09BDU02TY:@2=SHd0BTY:04e=C@1O
Gel0F5QH07MgM`1_Kfl0@D5106][J`1cLg<0=CDe07UiN@1XJ6P0LG5a07alO01MGEd0;bl_06i^KP0n
?Sh00P1jNWX05`1<C4`0?Shn0;:b/P1;Bd/0EEEE08F5Q@1?Cdl0=CDe059BDP1ZJVX0=cLg08F5Q@1f
MWH0>SXj075aL@0i>CT0DU9B04];B`28R8P0;Bd]03dm?@1XJ6P0Fe]K0080Cdm?0:L0>c/k07QhN01L
G5`0Ng]k04M7A`1IFET0PX:204a<C014A4@0H61P04E5A@1MGEd0J6QX09VIV@1FEUH0IfMW08V9R@1>
CTh0Cdm?03lo?`1]KFd0Bd];08N7Q`2RXZ80<c<c07emO@1]KFd0>SXj08:2PP1gMgL0=cLg08Z:RP1a
LG40>3Ph05IFEP2:RXX0BDU906ATI02;Rh/0=CDe051@D00j>SX0IfMW069RHP15ADD0Ti>C071`L01O
Gel0Y:BT09RHV01YJFT0];Bd09nOW`1QHF40QXJ60=GEe@2FUYH0Q8B40;Jf]P1ZJVX0UIFE0=3@d01/
K6`0HV9R0<G5a@20P800BTY:06EUI@20P800:RXZ055AD@1RHV80O7al04];B`2YZJT0MgMg04E5A@2K
Vi/0EUIF08Z:RP2`/;00PH6104Q8B01EEED0IfMW09BDU016ATH0<S8b05QHF02TY:@0H61P0:BTY01P
H600GEeM03<c<`28R8P0NGUi0861P@0e=CD0IVIV04e=C@1ZJVX0MgMg05iNGP1OGel0Ti>C04a<C00n
?Sh0CDe=05]KF`1OGel0Y:BT08B4Q017AdL0?Shn04m?C`1bLW80;2`/07=cL`1GEeL0AdM705ADE01X
J6P0Ph>309VIV@1aLG40GEeM096AT@2FUYH0?Cdm08:2PP1iNGT0:2PX04a<C01SHf<0AdM706MWI`2G
UiL0E5AD05EEE@15ADD0H61P03Hf=P0_;bl0M7Ad04Q8B016ATH0GemO03lo?`1;Bd/0QhN702HV9P1l
O7`0F5QH024Q8@1LG5`0BDU904M7A`2=SHd0De=C020P800o?cl0Hf=S05UIF@1MGEd0Cdm?05mOG`0>
ooooo`0V0;2`/02n_[h0alO7059BDP0^;Rh0:BTY02HV9P0M7Ad08B4Q028R8P1CDe<0BDU90451@@2F
UYH0cLg=0<c<c01bLW80?3`l02HV9P0K6a/07alO04Q8B01JFUX0E5AD08F5Q@0[:b/07AdM01LG5`0F
5QH0Ph>309:BTP1NGUh0L71`03Lg=`0E5AD0:BTY02/[:`0C4a<200h>3P0601<C4`22PX80>SXj01HF
5P0>3Ph03`l?0P0<30`200d=3@803Ph>00T0FEUI0=SHf02o_kl0YZJV08f=S@28R8P0PX:205aLG00c
<c<00P0A4A401P1cLg<0IVIV01dM7@0G5aL05ADE014A4@804a<C00/05ADE02d];@1@D500LG5a092@
T01bLW80V9RH0;2`/02NWYh0S8b<01dM7@0301<C4`0E01DE5@0G5aL05ADE01LG5`0L71`0WinO0:6Q
X@1eMGD0GEeM05iNGP1nOWh0W9bL0>3Ph03Ogml0/k>c03<c<`0U9BD0HF5Q03@d=00`<3008B4Q0080
5ADE01H05QHF01XJ6P1MGEd0UiNG0:FUY@2ZZZX0U9BD096AT@25QHD0Q8B4071`L01lO7`0MWIf05mO
G`2GUiL0H61P09bLW01MGEd0B4Q804Y:BP0j>SX0<c<c0P0k>c/02P1FEUH0A4A404];B`2GUiL0Z:RX
0:BTY019BDT071`L01PH600I6AT201PH601U01`L700I6AT06a/K04];B`2?Shl0[Jf]07IfMP1hN7P0
/[:b0<S8b01aLG40=cLg05MGE`1lO7`0Lg=c04I6AP1=CDd0TI6A0;Vi^@2:RXX0bl_;0;Vi^@2RXZ80
`<3007UiN@1=CDd0SXj>04U9B@0f=SH0:b/[01dM7@0G5aL07AdM061PH03Cdm<0WIfM0;Jf]P2`/;00
G5aL024Q8@0d=3@0<S8b03De=@0_;bl06a/K01`L700M7Ad0820P07alO02c/k<0LW9b05YJFP2MWId0
EUIF09fMW@1]KFd0>c/k03De=@0d=3@061PH01LG5`0F5QH061PH01LG5`1HF5P0IfMW030`<016ATH0
GEeM04];B`1RHV80=CDe05=CD`1>CTh0E5AD08f=S@1]KFd0<C4a04E5A@17AdL0EEEE04I6AP2XZ:P0
OGem0:6QX@16ATH0EeMG09:BTP1LG5`09RHV01dM7@0L71`0820P034a<@2EUID0g=cL0;bl_039bLT0
eMGE09VIV@0J6QX00P0F5QH07`0I6AT0H61P02DU9@0E5AD05QHF01DE5@0S8b<0LW9b08j>SP2ZZZX0
];Bd0=CDe036a/H0^[Zj08R8R01TI6@0L71`06ATI01CDe<0Ogmo08N7Q`2VYZH0UIFE0:j^[P2f][H0
bLW906UYJ@0X:2P0?Shn065QH@1TI6@04?ooool000kooooo01H0LW9b0:NWY`1ADE40E5AD092@T01>
CTh0DU9B08N7Q`1CDe<0GUiN06EUI@0O7al0;2`/065QH@0/;2`0?Shn04M7A`1KFe/0<30`07QhN00h
>3P0Xj>S0P1UIFD0oP2<S8`0LG5a05YJFP28R8P0EeMG06ATI025QHD0FEUI06ATI00a<C40<S8b08V9
R@1SHf<0@d=309>CT`0b<S80BDU902l_;`12@T80>c/k07YjNP0i>CT0@D5104I6AP10@400D51@05UI
F@15ADD0GemO04a<C00_;bl0>CTi08F5Q@0k>c/0BTY:0861P@1>CTh0?Shn08^;R`0e=CD0;Bd]08F5
Q@0];Bd0Bd];07inOP1ADE40KFe]04A4A00U9BD0A4A406m_K`0h>3P0Fe]K02l_;`1>CTh0RXZ:05YJ
FP0d=3@0RHV902<S8`2<S8`0TI6A03Ti>@0l?3`0`/;206UYJ@1ZJVX0V9RH06ATI00m?Cd0DU9B06i^
KP0m?Cd0UIFE06m_K`0m?Cd0PH61065QH@1PH600AdM708f=S@1KFe/0AdM706MWI`1OGel0CDe=07mo
O`12@T80O7al05]KF`1hN7P0EeMG07MgM`2?Shl0BDU904=3@`1VIVH0GEeM061PH01SHf<0U9BD05]K
F`1hN7P0Rh^;04I6AP1;Bd/0ATI605iNGP16ATH0T92@0:VYZ@0i>CT0VIVI06i^KP0h>3P0Q8B408V9
R@1ADE40Shn?07=cL`0g=cL0IVIV09VIV@17AdL0H61P07inOP1IFET0MgMg05MGE`1lO7`0LG5a06EU
I@2_[jl0SXj>09JFUP39bLT0Vi^K07EeM@2m_Kd0[jn_06EUI@2:RXX0c/k>059BDP27QhL0YZJV04U9
B@25QHD0`<30059BDP0l?3`0Ogmo06UYJ@1<C4`0LG5a06YZJP1>CTh0Shn?06QXJ01FEUH0<c<c05mO
G`1VIVH0AdM708:2PP0h>3P0Vi^K0:>SX`1bLW80<S8b05eMG@0n?Sh0I6AT05MGE`0k>c/0Cdm?09nO
W`1TI6@0_Kfm0<O7a`2>SXh0D51@0;Rh^02a/K40UiNG04];B`2h^;P0OWin07emO@1cLg<0D51@06e]
K@2GUiL0EEEE03dm?@14A4@0Bd];02PX:01[Jf/0NGUi04i>CP13@d<0ADE506YZJP0R8R80MgMg05]K
F`1DE5@0EEEE04Y:BP17AdL0FEUI04i>CP1IFET0SXj>08F5Q@1LG5`0Ph>307QhN00W9bL0BTY:06=S
H`0V9RH0:b/[06m_K`1FEUH0DU9B04I6AP1=CDd0<C4a03lo?`29RHT0AdM703lo?`1MGEd0<c<c034a
<@1XJ6P0:2PX06e]K@1RHV80:2PX06a/K01aLG40CDe=09NGU`0m?Cd08b<S04m?C`1bLW80AdM705mO
G`1?Cdl0Jf][3_ooool02011@D40ZZZZ0<?3``1`L700;bl_02PX:00O7al071`L0P0M7Ad0700V9RH0
BTY:03dm?@22PX80`/;20<C4a026QXH0AdM7055AD@1@D500BDU903Lg=`18B4P0IFEU08R8R00b<S80
=cLg055AD@1SHf<0UIFE0<c<c01VIVH0@T9206QXJ021PH40Rh^;020P800K6a/200l?3`0[01TI6@26
QXH0;Bd]01@D500C4a<03`l?01<C4`0>3Ph02PX:00/;2`0>3Ph06a/K0:2PX02JVYX0RHV906e]K@2U
YJD0QXJ60::RXP0f=SH0?Shn03dm?@1SHf<0_kno03@d=00B4Q804A4A00h>3P0A4A404Q8B01hN7P0a
<C40AdM7051@D00d=3@06a/K04];B`25QHD0Xj>S06EUI@1]KFd0;bl_00l?3`0200h>3P1V01DE5@0H
61P05ADE01LG5`0f=SH0[Zj^07MgM`1JFUX0J6QX08b<S02?Shl0RXZ:0=SHf03Cdm<0/K6a03De=@0T
92@0>CTi01@D500G5aL0:RXZ03Lg=`0T92@07QhN01XJ6P1YJFT0U9BD0:BTY02j^[X0Ti>C07moO`1A
DE40A4A406ATI01LG5`0?Cdm03Lg=`1PH600Fe]K05aLG01CDe<09BDU02h^;P19BDT0@41003Ti>@0h
>3P0?3`l07IfMP2@T900`L710<30`02TY:@0];Bd04I6AP0I6AT071`L01PH600G5aL06ATI01XJ6P0U
9BD0LG5a06ATI01nOWh0GemO04m?C`2j^[X0_;bl09:BTP10@400;bl_03Lg=`1>CTh0Hf=S06MWI`1H
F5P0[Jf]0>3Ph03WinL0PX:20:j^[P2][Jd0D51@08B4Q00_;bl06ATI01PH600I6AT06QXJ01LG5`0[
:b/0@D510:2PX02CTi<0WIfM07QhN01[Jf/0@d=303Hf=P0S8b<07AdM0P0I6AT03@0G5aL07AdM02`/
;01<C4`0/K6a07UiN@1NGUh0K6a/04e=C@1?Cdl0;2`/0451@@0c<c<00P0F5QH301DE5@0K020P8016
ATH0D51@03Ti>@0g=cL0DU9B03hn?P23Ph<0Hf=S04];B`1TI6@0RXZ:0;^k^`3?cll0SXj>09BDU02H
V9P0YjNW0;bl_03=cLd0Kfm_08f=S@1BDU80U9BD07emO@1;Bd/08R8R00807AdM02X09RHV06=SH`2a
/K40V9RH0:f][@2_[jl0bLW909FEU@22PX80>SXj01XJ6P0I6AT0Jf][05iNGP0g=cL0?Cdm069RHP2:
RXX0XJ6Q0::RXP2>SXh0VYZJ0<W9b@3>c/h0Zj^[07inOP0^;Rh0>SXj059BDP1^KVh0RHV908^;R`2/
[:`0c/k>0=3@d03Jf]X0bLW909FEU@0n?Sh0JVYZ07EeM@1GEeL@ooooo`003_ooool07`1TI6@0RHV9
04i>CP11@D40NGUi04U9B@1BDU80Ng]k0451@@14A4@0EUIF01lO7`0/;2`0?Cdm03`l?01LG5`0BTY:
03Xj>P0e=CD0Kfm_030`<02IVIT0=cLg0410@01OGel0FUYJ04i>CP2:RXX0Cdm?04A4A01ZJVX00P1W
IfL0E00Q8B408R8R051@D01JFUX0ATI609jNWP0`<300@d=303Ph>00m?Cd0:BTY05YJFP0g=cL0ADE5
04a<C01NGUh0ATI6059BDP11@D40GemO04Y:BP0g=cL0:2PX05YJFP0c<c<0Cdm?07AdM00c<c<0<30`
06EUI@0d=3@0;Bd]09:BTP0m?Cd0ATI605YJFP18B4P0MgMg07]kN`0n?Sh0Bd];05eMG@0f=SH0IfMW
05EEE@1dM7@0Rh^;04];B`0c<c<0VIVI02DU9@27QhL0Ogmo01/K6`1SHf<0ZZZZ061PH01[Jf/0UiNG
051@D01[Jf/0X:2P08R8R00k>c/0O7al05aLG01=CDd0SXj>05mOG`1QHF40De=C0:^[Z`1[Jf/0BDU9
06=SH`1BDU80RXZ:08>3P`15ADD0Jf][04U9B@2BTY80IFEU08>3P`80FEUI08<0SHf=092@T01SHf<0
H61P08>3P`2a/K40I6AT07emO@1cLg<0C4a<08B4Q01=CDd0HV9R03@d=01jNWX0^k^k04];B`2KVi/0
MGEe04=3@`2:RXX0OWin05UIF@2;Rh/0L71`05aLG02LW9`0WinO04];B`1RHV80Fe]K03@d=01bLW80
GUiN07EeM@1QHF40Hf=S0;Zj^P2?Shl0X:2P0<30`02;Rh/0?Shn09VIV@2<S8`0ATI609NGU`38b<P0
M7Ad08R8R02/[:`0E5AD08N7Q`36a/H0EeMG05EEE@2IVIT0PH6104a<C01ZJVX0IVIV04A4A01VIVH0
GUiN08B4Q011@D40NWYj08j>SP1ADE40T92@0410@01mOGd0WinO06a/K00L71`0;Bd]04I6AP1RHV80
@d=302/[:`1SHf<0QXJ6059BDP23Ph<0H61P0861P@1<C4`0bLW90:f][@2DU9@0:b/[06ATI01fMWH0
LW9b06YZJP1HF5P0Bd];09VIV@1VIVH0@T9204A4A01SHf<0BTY:06YZJP1nOWh0BDU904E5A@17AdL0
IFEU038b<P2OWil0MGEe05=CD`1IFET0Fe]K061PH01:BTX0B4Q806=SH`1lO7`0NGUi05mOG`2RXZ80
OGem02/[:`1MGEd0PX:20492@P1NGUh0IfMW06ATI00204a<C00J05mOG`1?Cdl0=3@d06e]K@1IFET0
?Cdm07AdM01?Cdl0?Shn05EEE@0Q8B40LW9b04=3@`0W9bL0I6AT06][J`0l?3`0SHf=04U9B@0b<S80
EeMG07IfMP1=CDd0NWYj06QXJ01mOGd>ooooo`0703De=@28R8P0dm?C0<30`01JFUX0>3Ph028R8P02
01/K6`0M020P800M7Ad0:b/[06UYJ@2RXZ80Zj^[09^KV`2c/k<0GEeM04Q8B016ATH0@T9202PX:00[
:b/0Vi^K096AT@1gMgL0IFEU04=3@`0^;Rh0:BTY06][J`2XZ:P0HV9R03dm?@1<C4`0UiNG01LG5`0C
4a<00P0=3@d01P0M7Ad0Jf][02HV9P0@41003@d=00l?3`<03@d=01H02`/;01DE5@2:RXX0IfMW01lO
7`0];Bd0GUiN07MgM`2b/[80^KVi07UiN@1OGel0UIFE0;Rh^02ZZZX0UIFE04I6AP0Z:RX0:BTY03Hf
=P11@D40?clo0P0_;bl02P0E5AD04a<C014A4@15ADD0FUYJ0861P@29RHT0Hf=S059BDP0H61P200h>
3P0B014A4@0G5aL05ADE02<S8`27QhL0KVi^03De=@0`<300;Rh^03Hf=P1MGEd0>3Ph08^;R`2o_kl0
SXj>06i^KP0N7Qh0<c<c0P0@4100;00A4A405aLG034a<@1RHV80Hf=S07YjNP14A4@0N7Qh08J6QP1h
N7P0Ogmo01lO7`0H61P09BDU03/k>`1NGUh0@41005MGE`1LG5`0HV9R06ATI01BDU80>c/k02l_;`14
A4@0?clo04Q8B019BDT0UYJF0=?Cd`1mOGd0U9BD09RHV02RXZ80K6a/05UIF@0W9bL06a/K01XJ6P0H
61P07AdM061PH01gMgL0L71`0P1HF5P05P2?Shl0@d=302TY:@1WIfL0>3Ph01`L700c<c<0G5aL04m?
C`1_Kfl0a<C40<?3``3Ph>00hN7Q09>CT`3Fe]H0[:b/03De=@1QHF40>c/k02@T900I6AT201LG5`0<
01TI6@0j>SX09bLW07alO01fMWH0Shn?08Z:RP1SHf<09RHV01TI6@0J6QX071`L0P0J6QX03P0M7Ad0
;2`/051@D01UIFD0=3@d06e]K@1bLW80FEUI05eMG@14A4@0:2PX03@d=00_;bl071`L100E5AD0A`0R
8R80CTi>04A4A013@d<0CDe=07QhN01XJ6P0Fe]K038b<P1eMGD0[Zj^0<C4a02n_[h0c<c<06UYJ@2@
T900LG5a07]kN`1`L700U9BD0<k>cP2YZJT0_;bl0<W9b@1nOWh0=CDe028R8P0S8b<0?Shn06QXJ02Y
ZJT0IfMW04=3@`26QXH0SHf=06ATI01BDU80C4a<06m_K`1MGEd0De=C09:BTP31`L40WIfM07inOP1R
HV80L71`08>3P`1WIfL0JVYZ05IFEP35aLD0][Jf09jNWP2:RXX0KFe]08:2PP1QHF40?clo03Hf=P0d
=3@0@d=309FEU@2YZJT0b<S80<?3``2c/k<0ZZZZ09jNWP18B4P07alO013ooooo000>ooooo`1607Yj
NP2LW9`0BTY:05IFEP1gMgL0C4a<055AD@2SXj<0NGUi05iNGP1eMGD08B4Q03@d=01LG5`08B4Q04Y:
BP17AdL0?Shn02TY:@1^KVh0=cLg0820P01VIVH0D51@07EeM@1_Kfl0B4Q80:6QX@1TI6@0FUYJ08F5
Q@1RHV80NWYj03`l?00`<300EUIF04U9B@0o?cl0UIFE05EEE@1UIFD0Bd];04E5A@0e=CD0IVIV03`l
?01FEUH0QHF506i^KP18B4P0Cdm?04U9B@2QXJ40MWIf03dm?@0Z:RX0IVIV02h^;P1>CTh0Ogmo06AT
I010@400JFUY02h^;P0c<c<0YJFU05ADE01SHf<0De=C03hn?P80MWIf0<h0?clo04E5A@1IFET0O7al
06=SH`1ADE40PX:2092@T014A4@0@4100:2PX00W9bL0R8R807alO00Q8B40>CTi0;:b/P1NGUh0Lg=c
08R8R017AdL0;Rh^04I6AP1/K6`0=3@d079bLP1NGUh0=SHf07AdM018B4P0CTi>03hn?P1iNGT0DE5A
04=3@`1DE5@0;bl_04e=C@1]KFd0@D5106a/K019BDT0Ng]k05QHF01^KVh0OGem0410@01:BTX0L71`
051@D01BDU80GUiN09NGU`1TI6@0JFUY075aL@1;Bd/0OWin04a<C01WIfL0GemO0861P@2EUID0D51@
09FEU@1WIfL0FEUI0::RXP2GUiL0CTi>08V9R@1>CTh0>c/k05aLG02RXZ80C4a<06=SH`1mOGd0>CTi
06QXJ019BDT0M7Ad065QH@1CDe<0R8R808f=S@2XZ:P0a<C407AdM01YJFT0`L7109FEU@1WIfL0WinO
0<O7a`23Ph<0Rh^;0:^[Z`1IFET0Shn?0=?Cd`2:RXX0LG5a0:j^[P1dM7@0C4a<06QXJ01[Jf/0;bl_
051@D01IFET0P82004A4A01eMGD0L71`04E5A@2BTY80E5AD0:ZZZP2MWId0K6a/01hN7P0T92@0AdM7
06EUI@1<C4`0=CDe05UIF@2:RXX0Hf=S09fMW@1hN7P0J6QX03Ti>@2f][H0T92@08n?S`0i>CT0Kfm_
08B4Q01fMWH0Ogmo06IVIP1:BTX0Z:RX06QXJ00o?cl0GEeM08R8R01=CDd0JFUY07emO@1;Bd/0EUIF
079bLP1iNGT0<c<c08F5Q@1MGEd0Cdm?05EEE@1bLW80IFEU069RHP1DE5@0HF5Q0:ZZZP28R8P0Lg=c
0<O7a`1VIVH07AdM055AD@1mOGd0FEUI06MWI`1fMWH0A4A4051@D01MGEd0MWIf07UiN@0n?Sh0Kfm_
03hn?P10@400Rh^;065QH@14A4@0Ogmo02d];@1/K6`0Fe]K01dM7@1<C4`0HV9R03Ti>@2JVYX0?Cdm
038b<P1jNWX0X:2P04m?C`1@D500EeMG07IfMPkooooo02l0=cLg08R8R03Hf=P0a/K60;Fe]@2RXZ80
RXZ:06e]K@1ADE40<c<c038b<P1GEeL0/K6a0<;2`P2FUYH0_;bl0;:b/P1lO7`0Ng]k08V9R@1aLG40
@D5105QHF01RHV80<c<c03hn?P0k>c/0;bl_02TY:@0X:2P06QXJ05ADE028R8P0?Shn01LG5`1HF5P0
>c/k018B4P0A4A403@d=01LG5`1OGel07QhN00`<300820P03`l?00@4100200`<300E01PH601/K6`0
CTi>01/K6`0P82009RHV01LG5`1?Cdl0/k>c0<O7a`38b<P0alO70;fm_@1bLW809RHV04a<C02HV9P0
OGem03Ph>00U9BD061PH00805ADE03407AdM01DE5@0D51@04Q8B02/[:`28R8P0V9RH0:b/[01MGEd0
>3Ph02h^;P0>3Ph0410@00l?3`0M7Ad0@T9209BDU01UIFD09bLW01XJ6P0K6a/05aLG01TI6@0l?3`0
8b<S034a<@2MWId0];Bd07=cL`18B4P0<c<c00h>3P0G5aL07QhN020P800Y:BT0K6a/0<g=c@24Q8@0
Y:BT09>CT`26QXH0EeMG05IFEP0F5QH05ADE03hn?P0T92@0A4A40080EEEE04l0M7Ad05MGE`1iNGT0
GemO02XZ:P0];Bd0@D5105=CD`0a<C40OWin0;Fe]@28R8P0ATI6096AT@2l_;`0Ti>C03De=@1@D500
LG5a03@d=00K6a/06ATI03Xj>P2IVIT0O7al06IVIP1XJ6P0S8b<03Ph>00G5aL05ADE01TI6@1PH600
:BTY038b<P0`<300A4A40:2PX03Bd]80_;bl0<S8b03Ng]h0^KVi0=WIf@36a/H0@41001hN7P0M7Ad0
<30`055AD@0a<C40;Bd]04];B`1MGEd06QXJ04a<C01RHV80YJFU09RHV01UIFD061PH01TI6@0L71`0
7AdM01`L700X:2P0@T9205QHF01=CDd09BDU02PX:00m?Cd0SXj>07MgM`2][Jd0CTi>02/[:`0a<C40
=CDe0080;2`/03`06ATI02HV9P0c<c<0A4A408Z:RP19BDT0B4Q807]kN`23Ph<0OWin06EUI@0d=3@0
=cLg092@T02_[jl0clo?0<g=c@2ZZZX0OWin05QHF01EEED0Jf][0:JVYP3Kfm/0/[:b08b<S02a/K40
YJFU05aLG00/;2`0BDU904m?C`1aLG40Hf=S02<S8`1<C4`0Ng]k0820P01kNg/0EeMG04Y:BP0m?Cd0
ATI60410@02YZJT0]KFe0410@00f=SH0BTY:06UYJ@32`/80YjNW06][J`1YJFT0^KVi04Y:BP0h>3P0
DE5A0861P@0T92@201TI6@808B4Q00P0<C4a07moO`28R8P0/;2`0;no_`2PX:00XJ6Q04M7A`807alO
4?ooool000kooooo0:D0O7al0861P@1@D500EeMG06][J`1>CTh0M7Ad0:b/[01`L700GEeM05MGE`0_
;bl0EeMG06][J`0W9bL0BDU905YJFP1IFET0=SHf065QH@0c<c<0MWIf06IVIP1FEUH0Kfm_03Ph>01G
EeL0UiNG07moO`1CDe<0B4Q803Xj>P1^KVh0E5AD03<c<`1IFET0C4a<03Hf=P1nOWh0DE5A05=CD`0a
<C40BTY:02`/;01MGEd06QXJ03@d=01IFET0D51@04=3@`0c<c<0<S8b05EEE@1@D500@d=302LW9`1T
I6@0:RXZ07moO`2DU9@0N7Qh03lo?`1RHV80;Rh^05MGE`2<S8`0DE5A05YJFP1;Bd/0BTY:069RHP1n
OWh0?Cdm04E5A@17AdL0;bl_05UIF@1@D500OWin09NGU`0W9bL0<c<c08V9R@0U9BD0Q8B40820P00a
<C40A4A40<71`@1KFe/0Ogmo0:JVYP1VIVH0CTi>065QH@22PX80>CTi07MgM`1VIVH0ADE509>CT`1H
F5P0GEeM05aLG02FUYH0GemO03Xj>P1QHF40De=C065QH@1fMWH0>c/k06m_K`1GEeL0SHf=05IFEP1/
K6`0RXZ:04i>CP1FEUH0QhN705EEE@1WIfL0KVi^09bLW01TI6@0HV9R0861P@1CDe<0HV9R04M7A`1X
J6P0IFEU0:>SX`2YZJT0?3`l09fMW@20P800A4A40820P027QhL0?clo0820P01aLG40BDU907AdM02S
Xj<0@d=306IVIP1mOGd0?clo05mOG`10@400M7Ad05aLG01BDU80PX:2096AT@2VYZH0];Bd08R8R01n
OWh0aLG509nOW`1_Kfl00P36a/H0K`22PX80Ti>C096AT@0j>SX0Ogmo0<G5a@1MGEd0Jf][09jNWP1<
C4`0CDe=06i^KP1GEeL0=3@d04e=C@1PH600NGUi04Y:BP1`L700DE5A030`<01iNGT0DE5A0<71`@2]
[Jd0AdM702/[:`0m?Cd0DE5A06][J`0Y:BT0;Bd]05IFEP2EUID0K6a/09nOW`1[Jf/0NGUi03/k>`2]
[Jd0^KVi09nOW`0h>3P0KFe]07MgM`1ZJVX0WinO07YjNP1XJ6P0YJFU03`l?00k>c/0F5QH06QXJ01;
Bd/0Shn?07alO01?Cdl0>SXj05aLG01RHV80>3Ph09^KV`12@T80?Cdm05UIF@1PH600Jf][0861P@1F
EUH0>3Ph06QXJ01hN7P0Cdm?0:f][@1hN7P0:BTY04M7A`1eMGD0=SHf0820P02UYJD0>SXj03/k>`1S
Hf<0R8R806ATI01DE5@0QXJ604M7A`0d=3@0NWYj03lo?`0g=cL0Ph>303Hf=P1aLG40I6AT01`L701:
BTX0Kfm_04A4A02JVYX0Cdm?02PX:01ADE40LG5a05aLG01PH600AdM706IVIP0>ooooo`0K04e=C@2X
Z:P0e]KF07UiN@14A4@0Fe]K08N7Q`2h^;P0c<c<0<G5a@2_[jl0`/;20=?Cd`3EeMD0fMWI0=SHf025
QHD0A4A403Ti>@0c<c<0:b/[03<c<`1FEUH0?Shn01LG5`0C4a<04A4A00@03Ph>0240410@04];B`1^
KVh0@d=302d];@1ADE404Q8B01LG5`0=3@d051@D065QH@0V9RH030`<00T92@0<30`04a<C00h>3P0=
3@d0>CTi04];B`0G5aL061PH02DU9@0G5aL051@D01LG5`2VYZH0cLg=0<_;b`33`l<0]KFe07QhN017
AdL00P0[:b/0101aLG40TI6A04=3@`0F5QH401DE5@0E01LG5`0J6QX0=CDe09ZJVP1mOGd0P82006e]
K@0k>c/0?3`l00l?3`0=3@d04a<C08J6QP2HV9P0CTi>03/k>`0J6QX06ATI01XJ6P0F5QH06ATI0080
<C4a01P07AdM06MWI`3<c<`0KFe]08Z:RP1UIFD0@41003De=@0`<300<C4a03Ti>@1UIFD0PX:20:NW
Y`2o_kl0QXJ608^;R`24Q8@0E5AD01TI6@0G5aL0ADE502d];@0Q8B4204i>CP0L08b<S01DE5@0IfMW
05IFEP0^;Rh05QHF01PH600g=cL0Ng]k05ADE024Q8@0VIVI07UiN@2BTY80cLg=09JFUP1JFUX08b<S
03Lg=`1fMWH0Fe]K02XZ:P26QXH0Q8B409JFUP1fMWH0X:2P03@d=0@05ADE03X0>CTi05EEE@0d=3@0
;Bd]02<S8`1/K6`0KFe]08f=S@3De=@0jNWY0=GEe@1YJFT0IfMW03@d=00K6a/071`L038b<P1hN7P0
De=C03Hf=P0e=CD0E5AD01lO7`0a<C40MWIf07emO@2HV9P0GEeM01PH600L71`07alO02PX:00d=3@0
?Shn04a<C00o?cl0=cLg03<c<`0U9BD0;Bd]07YjNP2NWYh0ZZZZ06m_K`23Ph<0OWin0:n_[`2TY:@0
J6QX055AD@1DE5@0EEEE07inOP2f][H0O7al06EUI@13@d<0K6a/0P1bLW80?`11@D40CDe=09:BTP2X
Z:P0ZZZZ05iNGP2e]KD0a<C40;Zj^P2QXJ40/K6a0<71`@38b<P0^KVi0;no_`2RXZ80QXJ60::RXP2;
Rh/0Jf][07alO01_Kfl09RHV0410@01IFET0<c<c04a<C01jNWX0Vi^K069RHP1NGUh0FEUI06EUI@2i
^KT0JVYZ05QHF01YJFT0Rh^;08V9R@2=SHd0_[jn08J6QP2i^KT0Xj>S030`<00_;bl0ATI602d];@0J
6QX05QHF01PH600E5AD06ATI02PX:01UIFD0S8b<07=cL`2`/;00[jn_096AT@0R8R8071`L01lO7`0@
ooooo`003_ooool0A01mOGd0FEUI04Q8B01=CDd0IFEU038b<P16ATH0Lg=c04M7A`1>CTh0<S8b01PH
600/;2`0DE5A02LW9`0f=SH0>CTi04i>CP0U9BD0F5QH034a<@1_Kfl0HV9R04e=C@1>CTh0DU9B04a<
C02GUiL0LW9b04m?C`1iNGT0DU9B05mOG`0h>3P0;Rh^06EUI@1PH600GemO07alO014A4@0BTY:04U9
B@17AdL09bLW05MGE`0d=3@0E5AD06UYJ@1PH600@T92051@D01EEED0TY:B07EeM@12@T809bLW06QX
J00];Bd0A4A407IfMP1OGel0@T9206a/K00Y:BT0>SXj08b<S00m?Cd0DE5A0P1FEUH0d022PX80KVi^
03De=@1GEeL0GEeM06ATI01EEED0BDU908f=S@2IVIT0@41003Ph>01kNg/09BDU08B4Q02;Rh/0>CTi
05ADE02b/[80GEeM07inOP2DU9@0DE5A04m?C`1JFUX0MWIf03Hf=P28R8P0GemO04];B`2@T900De=C
05aLG01WIfL0WIfM05UIF@15ADD0FUYJ04A4A01]KFd0Fe]K04A4A01dM7@0EEEE09FEU@1ADE40M7Ad
06i^KP1?Cdl0Hf=S079bLP1:BTX0FUYJ06][J`2>SXh0FEUI07QhN01jNWX0C4a<06a/K015ADD0HF5Q
04m?C`28R8P0[jn_0492@P2TY:@0NGUi055AD@2FUYH0Ogmo0492@P2;Rh/0JFUY05EEE@2=SHd0TI6A
03lo?`1^KVh0Hf=S05=CD`2:RXX0ADE507IfMP1SHf<0HV9R09fMW@1iNGT0T92@0:j^[P1SHf<0HF5Q
0<71`@1hN7P0CDe=0;fm_@2LW9`0FEUI08B4Q01fMWH0?3`l08^;R`3=cLd0GEeM08n?S`2UYJD0HV9R
055AD@1SHf<0GUiN03hn?P1QHF40J6QX06=SH`19BDT0O7al05mOG`13@d<0Kfm_03dm?@2a/K40SHf=
04M7A`0o?cl0;Rh^04=3@`1YJFT0>c/k02PX:00n?Sh0LW9b05MGE`2NWYh0P82006][J`14A4@0[jn_
08j>SP27QhL0=cLg07alO01MGEd0GemO08V9R@1WIfL0FEUI09BDU01;Bd/0=cLg04m?C`1HF5P0@410
05iNGP1bLW80GUiN04U9B@1JFUX0IVIV0451@@2PX:00JFUY04M7A`1GEeL0JVYZ05UIF@1VIVH0IfMW
05IFEP2<S8`0RHV906YZJP2][Jd0NGUi02XZ:P1BDU80SXj>06MWI`1IFET0IfMW04=3@`19BDT0N7Qh
08j>SP1=CDd0?3`l05UIF@1BDU80=3@d08n?S`18B4P0>CTi075aL@0d=3@0Ogmo07=cL`0R8R80DE5A
06m_K`0o?cl0Ti>C059BDP0b<S80HF5Q06UYJ@1?Cdl0OWin05=CD`26QXH>ooooo`0H07IfMP1=CDd0
D51@09BDU010@400:b/[02DU9@0X:2P0>3Ph06=SH`21PH40]KFe0=SHf03Fe]H0dm?C0;no_`0n?Sh0
;2`/03Ti>@1=CDd0E5AD05=CD`0i>CT03`l?0P0>3Ph200d=3@803Ph>0P0=3@d04P0>3Ph0=SHf069R
HP1DE5@0@T9205EEE@0O7al03@d=01LG5`1^KVh0<30`00d=3@0<30`03Ph>00l?3`0A4A40>SXj03lo
?`805QHF00h08R8R01TI6@0D51@05QHF04Y:BP2h^;P0Jf][04Y:BP1kNg/0OWin07EeM@1^KVh0IfMW
05iNGP80=SHf01T0FUYJ06i^KP14A4@0:b/[02LW9`0d=3@0=SHf03Ti>@0m?Cd0[:b/0;Zj^P1CDe<0
C4a<06UYJ@19BDT0<S8b020P800d=3@0BTY:03`l?00a<C407AdM01`L700I6AT05aLG00806ATI03T0
9BDU04e=C@0W9bL0Jf][0;Zj^P2YZJT0b<S80;Rh^01RHV80CTi>04M7A`1>CTh0FUYJ05mOG`1lO7`0
^;Rh0<g=c@31`L40Zj^[0861P@1JFUX0?Cdm0410@027QhL0?clo01hN7P0U9BD0CDe=06e]K@1dM7@0
F5QH03Lg=`14A4@09bLW01HF5P0G5aL0=3@d09jNWP26QXH0Shn?04Y:BP0H61P0?Shn09RHV02SXj<0
<c<c020P800^;Rh0IFEU0;jn_P2j^[X0/k>c0:f][@1iNGT0:RXZ02HV9P0E5AD00P0F5QH07@0E5AD0
5QHF038b<P1DE5@0GemO04a<C02/[:`0HV9R04I6AP2_[jl0i^KV0<[:bP22PX80Kfm_06EUI@0n?Sh0
A4A403`l?00U9BD07AdM01/K6`0J6QX08b<S03<c<`0N7Qh0UIFE06QXJ02d];@0PX:20080;bl_00@0
>CTi04Q8B017AdL0?clo0P0R8R80>00e=CD092@T024Q8@0f=SH0TY:B0;bl_02e]KD0SXj>08V9R@2D
U9@0Rh^;09BDU02WYjL0/k>c09^KV`2=SHd0PX:20:JVYP2f][H0UIFE08>3P`1VIVH0CDe=03Lg=`1Q
HF40/[:b0;Vi^@25QHD0=SHf03lo?`11@D40AdM707YjNP2LW9`0YZJV0:ZZZP2l_;`0RHV908^;R`2L
W9`0QhN709fMW@34a<@0`l?307QhN013@d<0KVi^05QHF00X:2P07QhN03hn?P1oOgl0e=CD0=SHf030
`<00Vi^K0P2h^;P03P1GEeL0Jf][07IfMP1KFe/0FUYJ06m_K`2IVIT0]kNg0;Rh^02YZJT0Fe]K065Q
H@1@D500=3@d0P0N7Qh03@0K6a/07QhN02@T901<C4`0QHF509jNWP2?Shl0Q8B40<30`02e]KD0@T92
03Ti>@1;Bd/04?ooool000kooooo05d0QHF508J6QP19BDT0AdM706m_K`16ATH0Hf=S09RHV01NGUh0
F5QH05ADE00i>CT0EUIF03/k>`0];Bd0Fe]K04=3@`0a<C408R8R06ATI00c<c<0Ng]k03De=@15ADD0
Fe]K04i>CP1[Jf/0WinO06ATI012@T80ADE504Q8B01lO7`09RHV02DU9@1FEUH0AdM7069RHP1^KVh0
>CTi04I6AP0g=cL0H61P020P801JFUX09RHV03Hf=P1@D500Cdm?03Ti>@0];Bd0?clo07inOP1RHV80
<c<c01lO7`1@D500;2`/071`L01hN7P0E5AD0410@01YJFT0;Bd]04m?C`2;Rh/0=CDe04=3@`19BDT0
C4a<07QhN01lO7`0>SXj04e=C@15ADD0HF5Q059BDP1MGEd0QXJ607EeM@0j>SX0@41008Z:RP0Q8B40
R8R809fMW@1BDU80E5AD0;Fe]@1TI6@0RXZ:0:JVYP1FEUH00P1@D500DP1dM7@0=3@d09:BTP1WIfL0
ADE507inOP1?Cdl0FUYJ06e]K@2FUYH0Hf=S0451@@1BDU80@d=305iNGP1ZJVX0Bd];0861P@1OGel0
VIVI059BDP1PH600P82005EEE@1FEUH0M7Ad05ADE013@d<0DE5A08>3P`1MGEd0NWYj07emO@1FEUH0
MWIf055AD@1XJ6P0IFEU09nOW`2WYjL0AdM70:BTY01bLW80B4Q808^;R`25QHD0?Cdm07inOP1SHf<0
I6AT07emO@2=SHd0A4A406EUI@1VIVH0HF5Q08^;R`17AdL0KFe]05EEE@1GEeL0Rh^;096AT@2b/[80
ZZZZ05EEE@1PH600`l?308n?S`1>CTh0a/K609nOW`23Ph<0NWYj06][J`0n?Sh0U9BD0;jn_P1dM7@0
Shn?0:j^[P1IFET2061PH01S05aLG00c<c<0C4a<06EUI@1TI6@0C4a<06EUI@1FEUH0@d=307MgM`1Q
HF40_kno08n?S`1ADE40P820065QH@18B4P0JVYZ03hn?P0l?3`0G5aL07IfMP1ADE40OGem05eMG@1B
DU80@d=30<G5a@2TY:@0OWin03Hf=P1YJFT0EUIF07EeM@2EUID0HF5Q06MWI`2ATI40<c<c05MGE`2A
TI40NWYj04=3@`1TI6@0MWIf04i>CP1VIVH0PH6107]kN`1QHF40[Zj^05mOG`1=CDd0E5AD07MgM`1N
GUh0FEUI05YJFP20P800_;bl08f=S@1_Kfl0ZZZZ05iNGP16ATH0SXj>0:BTY01HF5P0DE5A061PH00h
>3P0<c<c04a<C01WIfL0G5aL03dm?@1TI6@0<30`04i>CP2QXJ40KVi^03Ph>01NGUh092@T08R8R01:
BTX05aLG0492@P1QHF40=CDe08V9R@0^;Rh0<C4a07QhN01VIVH0?Cdm05EEE@1ZJVX0QHF500kooooo
01T0@d=303hn?P0f=SH0QHF505=CD`0`<3007AdM01lO7`0M7Ad0@41005iNGP1FEUH0WYjN0<o?c`3E
eMD0SXj>02`/;00m?Cd0>SXj05iNGP0k>c/05aLG01DE5@0@41003Ph>00T03@d=05404A4A07inOP1Z
JVX0CDe=09jNWP0V9RH07AdM09bLW01_Kfl03@d=00h>3P0=3@d05aLG06QXJ00g=cL06QXJ01DE5@0K
6a/06QXJ01LG5`0U9BD0CTi>05IFEP0d=3@02@T900d=3@0`<300Ph>305aLG00[:b/07alO04Q8B01J
FUX0;Rh^01`L700d=3@0C4a<03De=@13@d<0BDU90410@00N7Qh09BDU05QHF01WIfL0UiNG05UIF@0h
>3P0QhN709ZJVP1[Jf/0<S8b01LG5`0C4a<05ADE01HF5P0Q8B409bLW01hN7P0J6QX06a/K02/[:`1<
C4`0N7Qh0820P01XJ6P0`L71092@T02CTi<0EEEE03dm?@1<C4`0?Cdm024Q8@0K6a/07AdM079bLP2?
Shl0a/K608j>SP1GEeL00P0X:2P08P0V9RH09BDU03dm?@0m?Cd07QhN03Lg=`1?Cdl0PX:205YJFP0h
>3P0=SHf05YJFP0M7Ad05aLG05UIF@0h>3P0ADE502LW9`11@D405QHF01TI6@2c/k<0f][J09JFUP0/
;2`0<S8b09^KV`2m_Kd0`l?30=;BdP2k^k/0:b/[038b<P0O7al501DE5@0P01HF5P0H61P0BDU90;jn
_P3GemL0Kfm_0:BTY02ZZZX0e]KF0;Zj^P35aLD0f][J06e]K@0X:2P0:RXZ024Q8@0M7Ad06QXJ01TI
6@0H61P071`L030`<00e=CD0/;2`08n?S`2XZ:P0UIFE0410@01KFe/0EUIF0492@P0O7al201TI6@1H
03/k>`0V9RH0820P03hn?P1IFET0ZZZZ0<;2`P3Ti>@0inOW0<k>cP38b<P0[Zj^055AD@0_;bl08b<S
020P800T92@0<c<c0820P02TY:@0^KVi0<G5a@2EUID0:b/[01dM7@1>CTh0/k>c0:2PX021PH40C4a<
07=cL`1HF5P0GEeM07UiN@1mOGd0P82008n?S`2>SXh0FUYJ0410@00j>SX0Kfm_0::RXP34a<@0[:b/
07=cL`0M7Ad0G5aL02@T900W9bL0:b/[06MWI`3AdM40_Kfm0<O7a`2l_;`0Shn?0:6QX@2RXZ80Lg=c
05IFEP0];Bd06ATI01DE5@0J6QX0CDe=069RHP1aLG40Y:BT08B4Q01XJ6P0GemO04Y:BP0W9bL071`L
02@T901;Bd/0GemO08b<S02e]KD0/[:b0;jn_P32`/80a/K60<O7a`31`L40Jf][02/[:a3ooooo000>
ooooo`3o09BDU02EUID0C4a<05UIF@1WIfL0=SHf051@D02>SXh0F5QH05iNGP1=CDd0:b/[0451@@18
B4P0820P04m?C`1PH600EeMG02HV9P19BDT0:BTY075aL@1DE5@0@D5105IFEP0j>SX0DE5A08j>SP1^
KVh0?clo05mOG`19BDT0EEEE0410@00`<300FUYJ04A4A012@T80E5AD03hn?P1ADE40EEEE04U9B@0R
8R80FEUI03hn?P17AdL0ADE505=CD`14A4@0Bd];04i>CP2DU9@0BTY:02h^;P0P8200D51@02HV9P17
AdL0N7Qh06m_K`13@d<0J6QX02LW9`0k>c/0KFe]03Xj>P1BDU80>CTi051@D02:RXX0L71`03Ti>@0o
?cl0E5AD09fMW@1<C4`0IVIV09VIV@1gMgL0=cLg04];B`1aLG407QhN08R8R02e]KD0CTi>05IFEP2m
_Kd0IVIV08N7Q`2YZJT0Fe]K04m?C`1@D500Lg=c0451@@2VYZH0IFEU0492@P1cLg<0ADE505YJFP1W
IfL0SXj>05iNGP14A4@0F5QH04E5A@1VIVH0JVYZ05=CD`1nOWh0JVYZ09JFUP1QHF40I6AT08F5Q@1I
FET0DE5A07QhN01EEED0DU9B06=SH`29RHT0I6AT08:2PP24Q8@0DE5A05eMG@1DE5@0M7Ad06=SH`2]
[Jd0UiNG034a<@2YZJT0MgMg04m?C`2CTi<0Q8B403De=@2>SXh0EUIF075aL@1lO7`0S8b<04M7A`1n
OWh0J6QX06UYJ@22PX80ATI607moO`1TI6@0Hf=S08n?S`2ATI40Z:RX0:b/[01SHf<0LW9b0;bl_029
RHT0D51@07]kN`2ZZZX0LW9b07YjNP1YJFT0<c<c08R8R02g]kL0EUIF09nOW`2RXZ80@D5108F5Q@2H
V9P0E5AD030`<01?Cdl0HF5Q06ATI019BDT0GUiN0410@014A4@0RHV905aLG02MWId0Shn?030`<029
RHT0JVYZ04A4A01RHV807alO02d];@1kNg/0YZJV05QHF02HV9P0O7al06EUI@1>CTh0aLG50:NWY`29
RHT0=CDe06IVIP1IFET0MgMg0;Jf]P1/K6`0M7Ad0::RXP0R8R80:b/[059BDP1NGUh0@D5107IfMP20
P8008b<S04M7A`24Q8@0M7Ad069RHP2h^;P0>3Ph05UIF@26QXH0K6a/051@D01EEED0820P059BDP24
Q8@0Jf][06IVIP2g]kL0G5aL02PX:01KFe/0SHf=05IFEP1>CTh0Jf][0492@P0k>c/0Lg=c07IfMP1I
FET0@d=3000G06i^KP1ADE40;2`/09:BTP18B4P0=CDe06ATI00^;Rh0QhN706YZJP0N7Qh0BDU905eM
G@0W9bL0RXZ:01lO7`0R8R80GEeM07inOP1<C4`0H61P03De=@1kNg/03_ooool05@1MGEd0?clo038b
<P1RHV80JFUY0492@P0Q8B4071`L01dM7@0];Bd0?Shn04Q8B01jNWX0b/[:0<c<c01PH600>3Ph04=3
@`1^KVh0?Shn01DE5@02014A4@0401<C4`0B4Q805ADE00h>3P803@d=01<020P800d=3@0<30`01PH6
00d=3@0<30`08B4Q07inOP1eMGD0NGUi0:FUY@2HV9P0gMgM0;>c/`0V9RH05ADE03<c<`1gMgL0;2`/
00806ATI02D06QXJ01/K6`0D51@0<30`05QHF00Y:BT0A4A401<C4`0=3@d03Ph>02DU9@0[:b/0Hf=S
02<S8`0J6QX06a/K02h^;P1MGEd0A4A402PX:01:BTX0AdM702h^;P0D51@0410@00h>3P0F5QH06a/K
03<c<`0f=SH0Rh^;06QXJ02g]kL0e]KF0820P00E5AD04A4A00<04a<C0P0E5AD06@0F5QH05aLG01/K
6`0h>3P0Ph>309NGU`1PH600=cLg07]kN`2VYZH0R8R802XZ:P1mOGd0F5QH020P800I6AT06a/K01hN
7P1_Kfl0WIfM0;no_`2<S8`0F5QH02LW9`0L71`00P0U9BD08P0Y:BT0?clo03Ti>@0e=CD0Fe]K07al
O01XJ6P0CTi>03<c<`0g=cL0?clo03/k>`1FEUH0ATI6038b<P0L71`0?Shn01/K6`0^;Rh0HF5Q0<[:
bP3Cdm<0_Kfm0<O7a`3Rh^80clo?0=;BdP3?cll0TY:B03dm?@1LG5`0@D5101lO7`0I6AT301DE5@05
01HF5P0I6AT06a/K07]kN`2;Rh/00P2ZZZX02`2JVYX0Ogmo04e=C@1QHF40QHF507UiN@0Z:RX08R8R
01`L700H61P071`L00806a/K02@06ATI01dM7@0j>SX0WIfM0;Jf]P2WYjL0[jn_069RHP0[:b/0<c<c
055AD@0Z:RX061PH024Q8@1@D500;Bd]04e=C@0e=CD0>SXj065QH@1IFET0f=SH0;Zj^P2:RXX0GEeM
04];B`0Q8B4071`L01dM7@0O7al071`L01dM7@1QHF40IfMW08n?S`2j^[X207alO00?06MWI`1nOWh0
OGem04M7A`1eMGD0Z:RX0;no_`2VYZH0TY:B07EeM@24Q8@0KFe]06IVIP1PH600Lg=c0080BTY:01D0
FEUI08j>SP1SHf<0F5QH09VIV@1@D500EUIF01lO7`0S8b<0HV9R0;:b/P2CTi<0KFe]0861P@1EEED0
=SHf03hn?P1hN7P0=3@d01XJ6P0F5QH00P0E5AD0600I6AT0=cLg05QHF01;Bd/0Ogmo08f=S@1cLg<0
PX:2079bLP17AdL0<S8b04];B`1kNg/0TI6A06UYJ@1SHf<0[Jf]0:FUY@35aLD0`/;20;>c/`2<S8`0
GEeM024Q8A3ooooo000>ooooo`0X0820P01YJFT0D51@05MGE`21PH40DE5A05mOG`2?Shl0EUIF075a
L@1[Jf/08b<S03lo?`1OGel0;2`/03lo?`13@d<0CDe=02PX:01PH600<S8b09bLW01IFET0>SXj05mO
G`1>CTh0HF5Q09:BTP1cLg<0@d=306QXJ01iNGT0GUiN04m?C`0f=SH0Jf][05QHF01IFET0GEeM03lo
?`80DE5A0>`0B4Q803Xj>P1hN7P0=cLg04Y:BP10@400DE5A05UIF@1:BTX0C4a<08b<S01@D500?Shn
02d];@1fMWH0=3@d05YJFP21PH40KVi^0492@P1kNg/0;Rh^03/k>`1UIFD09bLW059BDP13@d<0EEEE
08B4Q01PH600>SXj04E5A@1NGUh0a<C404=3@`1HF5P0ZJVY0:2PX010@400@d=305mOG`0M7Ad0RHV9
0::RXP1=CDd0E5AD0:RXZ00l?3`0OGem0:JVYP1QHF40IVIV06QXJ01QHF408R8R07IfMP1SHf<0?3`l
0861P@0e=CD0Jf][06=SH`2BTY80DE5A075aL@23Ph<0FEUI075aL@0g=cL0Kfm_07inOP1[Jf/0Rh^;
06=SH`2JVYX0Rh^;061PH01DE5@0IFEU04a<C01]KFd0GEeM07QhN018B4P0Xj>S09>CT`1:BTX0EeMG
05EEE@28R8P0EUIF09bLW02KVi/0A4A40<30`029RHT0C4a<08Z:RP1IFET0DU9B0:f][@1KFe/0Jf][
07emO@1FEUH0=SHf07=cL`1LG5`0Cdm?08B4Q00V9RH0G5aL075aL@1]KFd0P82003`l?02IVIT0/k>c
08b<S023Ph<0/;2`03Ph>00m?Cd0GEeM09^KV`1HF5P0N7Qh08>3P`15ADD0Ph>30;:b/P1_Kfl0Vi^K
0:ZZZP1DE5@0EEEE05QHF00f=SH0:b/[04];B`1WIfL0PX:205IFEP1LG5`0EEEE04Q8B023Ph<0AdM7
0:ZZZP2NWYh0Cdm?09FEU@1MGEd0CTi>07QhN00];Bd0?Cdm04];B`1YJFT0B4Q806i^KP1DE5@0Jf][
03dm?@2_[jl0e]KF09VIV@0X:2P0KVi^04Y:BP1FEUH0Rh^;04Q8B01hN7P0XZ:R04U9B@0g=cL0FUYJ
07=cL`0g=cL0IFEU08B4Q012@T80BDU905]KF`1_Kfl0EUIF0;^k^`0k>c/0?Cdm05=CD`18B4P0C4a<
04A4A013@d<0FEUI09JFUP1NGUh0@d=308F5Q@1QHF40E5AD08b<S02TY:@0=CDe06IVIP1dM7@0?3`l
04e=C@2PX:00UIFE06EUI@1BDU80IVIV0410@00j>SX0Zj^[06=SH`0^;Rh0Fe]K01dM7@1YJFT0J6QX
02DU9@18B4P0H61P03/k>`1`L700ATI6020P801;Bd/0C4a<04U9B@1bLW80?Cdm07YjNPkooooo01T0
Lg=c05MGE`0m?Cd0@4100:VYZ@1;Bd/09BDU028R8P1NGUh0;2`/034a<@0n?Sh0WYjN0<G5a@2m_Kd0
Ogmo04Y:BP1VIVH0=SHf01DE5@0@41004a<C014A4@0B4Q804A4A00803`l?00L05ADE01dM7@0>3Ph0
3@d=00`<300:2PX0410@00804a<C02D0KFe]0:RXZ02e]KD0bLW90=OGe`34a<@0b<S80:BTY02IVIT0
ZJVY02l_;`0Q8B40820P01DE5@0H61P06a/K03@d=01ADE4071`L00d=3@1:BTX0820P02HV9P0k>c/0
I6AT03De=@1QHF40:2PX01LG5`0E5AD05aLG024Q8@1/K6`0QXJ605=CD`0I6AT04A4A00803Ph>00h0
410@01HF5P0E5AD0<c<c02l_;`0^;Rh0^[Zj0;jn_P36a/H0K6a/01XJ6P0B4Q804A4A01<C4`805ADE
01H051@D01DE5@0G5aL09BDU06IVIP1`L700MGEe04i>CP0L71`09BDU06YZJP2TY:@09BDU04Y:BP1H
F5P0:RXZ01`L700K6a/08b<S08>3P`3>c/h0WIfM0P0a<C403P1@D5009RHV01`L700P8200:2PX02LW
9`18B4P0;bl_07EeM@2c/k<0X:2P0;Ng]`1CDe<0=SHf0P1lO7`0:@0e=CD0DE5A01dM7@0K6a/0Fe]K
05MGE`1;Bd/0TY:B0=OGe`3De=@0gMgM07EeM@2j^[X0^;Rh0:ZZZP30`<00X:2P04i>CP0O7al07AdM
03Hf=P1CDe<06a/K01dM7@0P82008B4Q01XJ6P0L71`0QHF50:VYZ@36a/H0`<300;Fe]@10@4007alO
01`L7019BDT0NWYj03Ti>@0M7Ad06ATI00806a/K02007AdM01lO7`0_;bl0D51@079bLP2c/k<0d];B
0=3@d022PX805ADE01@D500E5AD06QXJ07]kN`1JFUX0[Jf]0;>c/`20P800<c<c01HF5P1<C4`0<c<c
04i>CP2IVIT08B4Q01`L7018B4P0L71`06][J`0o?cl0:b/[028R8P807alO01T0IVIV09nOW`1VIVH0
OWin06][J`1VIVH0T92@0;Rh^00/;2`0<C4a04E5A@2KVi/0h>3P0=OGe`1jNWX0QhN7069RHP1@D500
BDU9055AD@1JFUX09RHV03/k>`10@400@D510080DU9B00/0Ph>30:FUY@1?Cdl0?clo08^;R`2XZ:P0
Ogmo061PH01^KVh0QHF503Lg=`0201DE5@0402/[:`12@T80;2`/01LG5`805ADE01P05aLG024Q8@15
ADD0>3Ph05UIF@2CTi<0QHF50:JVYP39bLT0][Jf08J6QP1lO7`0H61P04U9B@0k>c/0ADE508Z:RP2K
Vi/0^;Rh08V9R@2SXj<0R8R8079bLP1>CTh@ooooo`003_ooool0WP1lO7`0UIFE04Y:BP1FEUH0SHf=
059BDP1RHV80I6AT02DU9@1PH600EUIF02HV9P11@D40BDU9055AD@1cLg<0EEEE04];B`0e=CD0Ng]k
03Ph>02KVi/0>CTi06ATI027QhL0BDU904i>CP2BTY80G5aL07=cL`22PX80L71`069RHP0R8R80:2PX
096AT@1iNGT0HF5Q08B4Q00U9BD0GEeM04i>CP10@4008b<S061PH00d=3@0ATI603`l?00W9bL0=CDe
05aLG00o?cl0SXj>04A4A00?3`l07QhN08f=S@0i>CT0Hf=S061PH00S8b<0<C4a09JFUP0^;Rh0>SXj
079bLP0K6a/0Fe]K07EeM@1:BTX0NGUi055AD@0c<c<0GUiN04i>CP2YZJT0?clo04E5A@26QXH0Lg=c
03Xj>P19BDT0CTi>0451@@2?Shl0UIFE04];B`1PH600/k>c04Q8B01iNGT0Ti>C03Ph>01=CDd0CTi>
07=cL`0e=CD0TI6A05=CD`12@T80RXZ:06EUI@1PH600HV9R08B4Q00f=SH0>3Ph04];B`0m?Cd0KVi^
04m?C`1DE5@0L71`06ATI01^KVh0ADE5069RHP1^KVh0E5AD05MGE`1RHV80>SXj04M7A`18B4P0OWin
05IFEP1NGUh0OWin0451@@1EEED0BDU906QXJ01EEED0WinO0:ZZZP0k>c/0UIFE06e]K@1?Cdl0UiNG
06e]K@0a<C40V9RH059BDP1eMGD0L71`08N7Q`18B4P0IFEU06MWI`0l?3`0Ti>C05MGE`1VIVH0Kfm_
06i^KP27QhL0KFe]0P1iNGT0MP1GEeL0MGEe0;6a/@1cLg<0?Cdm0:FUY@2BTY80CTi>0861P@1iNGT0
A4A407]kN`2g]kL0GEeM07moO`2DU9@0AdM707]kN`1lO7`0@D5103Ti>@1<C4`0G5aL05]KF`1FEUH0
Q8B405aLG01ADE40MWIf03Xj>P2JVYX0UYJF04e=C@2b/[80NWYj0410@01UIFD0?clo04Q8B01HF5P0
YZJV04E5A@28R8P0DU9B061PH00Z:RX0Ogmo08N7Q`1]KFd0=SHf07]kN`1aLG40OGem092@T01OGel0
OGem0:6QX@14A4@0GEeM09>CT`29RHT0AdM706IVIP1mOGd0>c/k06UYJ@2BTY80O7al04U9B@2;Rh/0
B4Q8055AD@1XJ6P0@41004M7A`1FEUH0E5AD08R8R02c/k<0QHF505=CD`2>SXh0AdM702PX:01HF5P0
KFe]03Xj>P1;Bd/0H61P02DU9@0/;2`0O7al09RHV01kNg/0@d=305aLG00d=3@09BDU0861P@1=CDd0
;bl_05iNGP0W9bL0N7Qh06IVIP0T92@0ATI606a/K01EEED0TY:B03hn?P0P8200A4A404a<C00i>CT0
LW9b07inOP2HV9P>ooooo`0H096AT@2IVIT0MWIf04];B`35aLD0Xj>S09bLW021PH40VYZJ07MgM`1l
O7`0/K6a0:ZZZP2o_kl0_[jn0;no_`1oOgl0?clo01DE5@0D51@04A4A01hN7P0G5aL06ATI0P0A4A40
;00C4a<0GEeM069RHP0G5aL05ADE01<C4`0B4Q805aLG01/K6`0/;2`0Fe]K0;6a/@2i^KT0alO709bL
W02<S8`0Lg=c07moO`3?cll0Xj>S03<c<`0R8R805ADE01hN7P0U9BD0DU9B04a<C00N7Qh0<C4a038b
<P0m?Cd0>c/k02h^;P0E5AD06a/K03/k>`1OGel0:BTY01`L700E5AD06ATI01XJ6P0k>c/0E5AD0P0K
6a/0K`0A4A403Ph>00h>3P0E5AD05aLG01DE5@0S8b<0@D5101XJ6P2IVIT0Cdm?09VIV@26QXH0?3`l
01LG5`0C4a<05ADE01<C4`0N7Qh071`L03@d=01HF5P0L71`0451@@0d=3@0@41003Xj>P0K6a/06QXJ
03Ti>@2VYZH0KVi^05IFEP1LG5`0De=C03Ph>00J6QX09BDU0:VYZ@3Ti>@0Xj>S03dm?@0L71`0>3Ph
04A4A019BDT0>SXj030`<00W9bL0<30`03lo?`2OWil0/k>c09BDU02e]KD0Rh^;05YJFP2<S8`0Ng]k
05UIF@13@d<0:BTY02LW9`1UIFD0UIFE0861P@1FEUH0QXJ60<_;b`2XZ:P0NWYj0:BTY01iNGT0BTY:
07IfMP2@T900NGUi02DU9@0M7Ad07QhN028R8P0V9RH0:2PX01TI6@0L71`09BDU079bLP2[Zj/0^;Rh
0:FUY@2a/K40<c<c024Q8@0M7Ad071`L02TY:@2<S8`0HF5Q02DU9@0_;bl0=SHf03hn?P1DE5@0De=C
05eMG@1IFET0H61P0=cLg02i^KT0a/K608R8R00401DE5@0Q075aL@2RXZ80/k>c0<71`@1[Jf/06ATI
02PX:01IFET08B4Q05=CD`1GEeL05ADE01@D500G5aL0LW9b0:n_[`2=SHd0O7al07UiN@25QHD0Q8B4
08R8R00j>SX0UYJF0;Vi^@2`/;00alO70<g=c@2<S8`07AdM01lO7`16ATH0[jn_0080c/k>01P0d];B
0;Vi^@2>SXh0NGUi065QH@1_Kfl0BDU904A4A00/;2`04a<C01LG5`0c<c<0EEEE08F5Q@2SXj<0eMGE
0;6a/@1_Kfl0IFEU05=CD`11@D40>c/k06EUI@0L71`201DE5@0401dM7@1?Cdl0=CDe01LG5`805ADE
00`05aLG01`L700k>c/0<C4a03@d=01OGel0`<300;:b/P2HV9P0J6QX05EEE@0g=cL303Hf=P090451
@@2:RXX0W9bL06UYJ@1UIFD0MGEe07UiN@25QHD0Ogmo013ooooo000>ooooo`1308R8R02DU9@0BTY:
05EEE@2IVIT0Bd];069RHP1PH600;Rh^069RHP1=CDd09BDU0451@@0F5QH0:RXZ055AD@0j>SX0410@
02d];@1HF5P0:b/[07AdM00L71`0>c/k07MgM`10@400@d=308:2PP1CDe<0?Shn06a/K01EEED0GUiN
02<S8`0K6a/0FUYJ05EEE@1KFe/0G5aL03<c<`1>CTh0:b/[051@D00[:b/0J6QX02l_;`1=CDd0?Shn
05iNGP17AdL0ATI604=3@`2ZZZX0Kfm_03dm?@0M7Ad0IVIV038b<P1XJ6P0Fe]K03<c<`0[:b/0L71`
02LW9`0j>SX0Ogmo02XZ:P0203`l?0070410@02EUID0L71`02l_;`0n?Sh0?Cdm08n?S`02069RHP2^
09RHV01iNGT0>c/k04i>CP1RHV806a/K06EUI@1dM7@0>3Ph05]KF`2QXJ40HF5Q07MgM`2DU9@0BDU9
04m?C`14A4@0NWYj03Ti>@2HV9P0B4Q803Ti>@1mOGd0EeMG05iNGP22PX80VIVI05UIF@1<C4`0De=C
051@D01_Kfl0EeMG05QHF01ZJVX0KVi^0:2PX01PH600LG5a07AdM01CDe<0GemO08f=S@1OGel0Fe]K
05UIF@2HV9P0IVIV07EeM@1RHV80=cLg06][J`1ADE40G5aL0492@P23Ph<0Zj^[04i>CP2`/;00DE5A
0492@P22PX80MWIf03dm?@2PX:00BTY:04i>CP1UIFD0Ogmo0492@P1UIFD0I6AT0492@P2MWId0I6AT
07EeM@22PX80KFe]08>3P`21PH40Z:RX08j>SP18B4P0QXJ60;Jf]P1hN7P0F5QH0<;2`P1jNWX0@T92
08R8R01`L700DE5A0:VYZ@2o_kl0L71`0:2PX02=SHd0<c<c096AT@25QHD0?Cdm03@d=01TI6@0FUYJ
03Hf=P1<C4`0MWIf03lo?`17AdL0QXJ605MGE`2KVi/0Rh^;03@d=02@T900MGEe04I6AP1PH600Bd];
04I6AP1HF5P0ZJVY05EEE@1mOGd0HF5Q075aL@1;Bd/0a/K60;:b/P2:RXX08B4Q065QH@2:RXX0R8R8
0;>c/`1nOWh0QhN709VIV@0l?3`0>CTi05]KF`1mOGd0BDU906i^KP1cLg<0<30`03dm?@1YJFT0NGUi
03`l?01oOgl0@D5104M7A`1[Jf/0GUiN05IFEP1<C4`0:b/[05YJFP25QHD0Kfm_0451@@2EUID0NGUi
038b<P1]KFd0SXj>04e=C@1?Cdl0N7Qh06QXJ01CDe<0[Zj^0P1fMWH0600m?Cd0IVIV06m_K`0^;Rh0
Rh^;05UIF@0[:b/0MgMg03Hf=P1gMgL0GEeM02XZ:P1IFET0NWYj05MGE`2j^[X0>SXj01hN7P0l?3`0
DU9B04a<C01PH600>3Ph07]kN`kooooo04T0F5QH069RHP1nOWh0N7Qh0<o?c`2OWil0N7Qh05EEE@1T
I6@0H61P0451@@0m?Cd0RXZ:0:FUY@31`L40_[jn07inOP0I6AT05ADE01<C4`0@41004a<C01DE5@0B
4Q804A4A01<C4`0L71`0N7Qh0410@00`<300A4A4059BDP1CDe<0EUIF06MWI`1ZJVX0GemO08n?S`2=
SHd0^KVi096AT@1NGUh0G5aL06m_K`2HV9P0/[:b09bLW01MGEd0IFEU05QHF01jNWX0B4Q803lo?`12
@T80;2`/02TY:@0_;bl03`l?014A4@0>3Ph0410@01HF5P19BDT0AdM7028R8P0F5QH05ADE01LG5`0m
?Cd0@41001LG5`0N7Qh06ATI00803Ph>01006QXJ01DE5@0F5QH05aLG04=3@`0a<C40WYjN04E5A@2/
[:`0dM7A08V9R@1_Kfl0CTi>03dm?@0g=cL0ADE50P1TI6@08@11@D40:2PX01TI6@0a<C40=3@d03lo
?`0I6AT05aLG02l_;`2?Shl0UiNG0:f][@2ATI40H61P05IFEP0_;bl0F5QH0<G5a@3Hf=P0c<c<09jN
WP0k>c/0:b/[01DE5@0Q8B40C4a<05MGE`0g=cL0=3@d04=3@`0];Bd0KFe]092@T00205eMG@0T08b<
S030`<00`<3009fMW@1HF5P0EUIF06UYJ@2BTY80OGem04E5A@0N7Qh0@d=309NGU`0e=CD0W9bL0=[J
fP2@T900EeMG096AT@26QXH0LG5a09>CT`0Y:BT06a/K01`L700K6a/071`L01/K6`0S8b<0L71`06QX
J01LG5`0aLG5096AT@2<S8`07AdM0P0L71`01`0M7Ad0;bl_06QXJ032`/80UYJF071`L01KFe/00P1<
C4`01@14A4@0=CDe03Ph>02EUID0dm?C0080`l?30300bLW906=SH`0O7al08B4Q055AD@1LG5`0;Bd]
04E5A@14A4@0GemO01lO7`1YJFT0?clo02`/;01BDU80=CDe01DE5@0D51@06ATI03lo?`0I6AT07AdM
024Q8@0W9bL0>SXj08f=S@1=CDd071`L06m_K`3Jf]X0bl_;09nOW`31`L40Ng]k01dM7@0W9bL0PH61
0:VYZ@1YJFT0ZZZZ0<_;b`2m_Kd0/K6a0:FUY@2a/K40eMGE06e]K@0_;bl201DE5@0A01<C4`0E5AD0
@T9207inOP2KVi/0clo?09nOW`1>CTh0<S8b04A4A00_;bl0=3@d059BDP0E5AD051@D01DE5@0I6AT0
0P1;Bd/0400I6AT05ADE01DE5@0G5aL08B4Q04U9B@0/;2`0<c<c04M7A`3@d=00[jn_06e]K@1XJ6P0
H61P030`<00b<S8203<c<`0903hn?P2QXJ40Xj>S06EUI@2HV9P0HV9R05IFEP2GUiL0W9bL013ooooo
000>ooooo`3o06YZJP21PH40Cdm?0410@01cLg<0?3`l05]KF`2;Rh/0BDU907IfMP1=CDd0;2`/04U9
B@1=CDd08b<S04E5A@1<C4`0@T9202/[:`1RHV80=cLg079bLP1ADE40>CTi06UYJ@0n?Sh0@D5108R8
R01[Jf/0BTY:055AD@0k>c/0Jf][05ADE00e=CD0OWin04Q8B01EEED0LW9b04=3@`1<C4`0>3Ph05]K
F`0a<C40JVYZ02`/;00l?3`0>SXj05EEE@11@D40>c/k04=3@`2;Rh/0MgMg04a<C00[:b/0L71`03<c
<`1aLG40JVYZ06IVIP10@400N7Qh02d];@1<C4`0P82003Lg=`16ATH0?Shn03/k>`25QHD0H61P03Hf
=P10@400IfMW06i^KP1QHF40RHV90:NWY`1WIfL0>CTi05YJFP1/K6`0=3@d09JFUP2=SHd0AdM706MW
I`2`/;00Hf=S0861P@2ATI40CDe=05IFEP1PH600JFUY03Xj>P2MWId0G5aL03/k>`1eMGD0@D5106IV
IP1OGel0KVi^0410@01MGEd0GUiN04A4A01YJFT0@d=306=SH`1TI6@0Hf=S08B4Q01GEeL0MgMg06EU
I@14A4@0EUIF06QXJ01@D500F5QH04i>CP1hN7P0EeMG0861P@23Ph<0C4a<069RHP1NGUh0O7al05UI
F@2HV9P0Z:RX04M7A`2f][H0PH6104M7A`2;Rh/0IVIV05ADE02JVYX0CDe=05IFEP1MGEd0Hf=S04I6
AP1eMGD0Cdm?03Ph>01cLg<0?clo075aL@1cLg<0H61P08:2PP1fMWH0]kNg09RHV01ADE40QXJ60:6Q
X@1@D500LG5a0;^k^`2?Shl0A4A408F5Q@1ADE40>3Ph07AdM02_[jl0IVIV0:ZZZP2XZ:P0AdM706YZ
JP1gMgL0?Shn02`/;011@D40FUYJ051@D01>CTh0QHF5059BDP10@400JFUY05QHF02RXZ80Q8B404i>
CP2XZ:P0Hf=S04i>CP1NGUh0=CDe04];B`1^KVh0Y:BT055AD@29RHT0Ng]k071`L00j>SX0_;bl0:ZZ
ZP2;Rh/08b<S06=SH`1YJFT0I6AT08^;R`1fMWH0S8b<0:JVYP1SHf<0AdM7079bLP2<S8`0F5QH07Yj
NP2>SXh0IVIV04m?C`1hN7P0Q8B404A4A01aLG40EeMG05=CD`1RHV80KVi^05iNGP18B4P0C4a<07Qh
N02LW9`0SXj>03dm?@2KVi/0MWIf03Ph>01aLG40SHf=04I6AP15ADD0IFEU04];B`1ADE40[Jf]08>3
P`1@D500?clo000G08Z:RP1gMgL0<S8b0861P@19BDT0;bl_08B4Q00a<C40MgMg07QhN00b<S80@d=3
0861P@1`L700UIFE04M7A`0U9BD0@D5107=cL`1QHF40QHF50451@@1jNWX03_ooool04P1jNWX0SXj>
06m_K`1?Cdl0MgMg0;no_`2HV9P0HV9R03dm?@13@d<0=SHf02/[:`26QXH0PX:20:n_[`31`L40Zj^[
07MgM`80Fe]K00h0GEeM05iNGP1MGEd0GEeM05UIF@1EEED0Kfm_09:BTP1kNg/0EEEE04M7A`10@400
>CTi03De=@80;bl_00l09bLW02<S8`1SHf<0WIfM0=CDe02SXj<0[Jf]0<K6aP2BTY80[jn_0=CDe02h
^;P0[jn_0;Fe]@1@D5000P0E5AD01@0C4a<05ADE04];B`0P82003`l?00<03Ph>03D04a<C01hN7P2H
V9P0;Rh^01dM7@0F5QH061PH04m?C`0S8b<05ADE01/K6`0S8b<08B4Q01`L700F5QH05ADE01HF5P0E
5AD08b<S04m?C`2IVIT0C4a<096AT@1]KFd0FUYJ05QHF00Z:RX09BDU0451@@1JFUX0LG5a04A4A00P
820061PH02`/;01<C4`0G5aL06a/K00h>3P0Cdm?05iNGP1hN7P0AdM706m_K`1LG5`0MgMg08j>SP2c
/k<0b/[:071`L00o?cl0AdM706e]K@0207QhN00V02h^;P0V9RH0820P02PX:00i>CT0ADE506ATI027
QhL0a/K60:f][@1?Cdl0AdM704Y:BP2k^k/0gmoO0<o?c`35aLD0Q8B407MgM`1?Cdl0Ph>3065QH@1@
D500H61P09RHV01?Cdl0HV9R09>CT`21PH40YJFU0;6a/@2@T900@T9204e=C@1jNWX0H61P03Lg=`0K
6a/201XJ6P0A02HV9P0o?cl0IVIV0;:b/P3De=@0d=3@0;:b/P0h>3P09RHV01hN7P0S8b<0:BTY08R8
R03@d=00`/;205ADE00_;bl00P0R8R803P0S8b<0@d=306YZJP1LG5`0WinO0>?Sh`3Jf]X0^KVi0:j^
[P2RXZ80ZZZZ08b<S01XJ6P0=cLg0P0S8b<03P1iNGT0X:2P08Z:RP1aLG40W9bL0;Bd]01ADE40<S8b
01dM7@17AdL08B4Q01@D500B4Q804a<C0P0E5AD05`0U9BD0:RXZ03De=@1/K6`0EEEE05QHF02BTY80
J6QX0:JVYP24Q8@0V9RH0:FUY@1^KVh0G5aL09fMW@2KVi/0PH6108j>SP2VYZH0VYZJ06m_K`1cLg<0
5aLG00805ADE00d051@D01DE5@0M7Ad0Bd];06i^KP1lO7`0Hf=S02<S8`0T92@0>c/k02PX:012@T80
CTi>00<05ADE01h05QHF038b<P1`L700GemO02@T900R8R80>3Ph05EEE@2=SHd0K6a/06m_K`2:RXX0
Ti>C04e=C@1TI6@0J6QX07=cL`1HF5P0;Rh^02l_;`16ATH0Hf=S08>3P`1fMWH0Ti>C0:6QX@1oOgl0
UiNG0;Vi^@3>c/h@ooooo`003_ooool0901VIVH0IFEU055AD@1FEUH0SHf=04M7A`1?Cdl0Q8B40492
@P20P800JFUY02LW9`19BDT0I6AT02d];@1FEUH0AdM704a<C012@T80D51@020P801ZJVX0E5AD02l_
;`1CDe<0@d=302h^;P1jNWX0I6AT0410@01PH600BTY:055AD@1?Cdl0>3Ph09JFUP80C4a<01d0Lg=c
0492@P1?Cdl0Bd];05aLG00l?3`0Hf=S03De=@1;Bd/0A4A404m?C`1<C4`0Bd];04I6AP1oOgl0Jf][
04=3@`0i>CT0T92@02h^;P1[Jf/0Hf=S06UYJ@10@400QhN7030`<01;Bd/0Kfm_02d];@02051@D00<
04=3@`2BTY80?Shn03De=@1WIfL0L71`04A4A01IFET0LG5a08^;R`1mOGd0=cLg0P1OGel05P1<C4`0
VIVI08B4Q012@T80QhN70<30`019BDT0IVIV08V9R@0j>SX0A4A4055AD@1[Jf/0?Cdm08>3P`10@400
D51@08B4Q01PH600Kfm_0861P@1oOgl204Y:BP04069RHP1EEED0JFUY0451@@80HV9R05X0P82009VI
V@1GEeL0L71`05mOG`1FEUH0JFUY08B4Q01MGEd0BTY:06IVIP29RHT0IfMW07AdM01[Jf/0@D51065Q
H@1/K6`0GemO04=3@`2RXZ80/[:b04Y:BP2c/k<0Bd];04U9B@2LW9`0H61P04Q8B02EUID0?clo05aL
G01UIFD0LW9b04E5A@1aLG40GUiN05IFEP1nOWh0Bd];06QXJ01eMGD0FEUI06=SH`1NGUh0XJ6Q069R
HP12@T80Shn?0:RXZ01FEUH0SXj>0;^k^`10@400=cLg08b<S01MGEd0@D5108f=S@1cLg<0CTi>08B4
Q02JVYX0>CTi07emO@1hN7P0@D5106e]K@1WIfL0IVIV03Lg=`1QHF40UiNG05QHF014A4@0F5QH0451
@@24Q8@0GemO04M7A`21PH40=cLg0410@011@D40;Rh^04a<C01/K6`0Xj>S038b<P1CDe<2065QH@19
04U9B@2l_;`0UIFE07emO@0T92@0K6a/07UiN@1`L700LG5a05]KF`2;Rh/0V9RH04];B`1=CDd0NGUi
05eMG@10@400CTi>07IfMP1BDU80D51@06EUI@1^KVh0<c<c07]kN`1HF5P0EeMG065QH@11@D40?clo
04U9B@1?Cdl0KFe]09NGU`27QhL0A4A407inOP1_Kfl0:BTY06UYJ@24Q8@0@410034a<@1iNGT0@410
04Y:BP25QHD0M7Ad0410@00e=CD0K6a/06m_K`0g=cL0N7Qh04Q8B00^;Rh0OGem03Hf=P1bLW80OGem
03`l?013@d<0QHF506EUI@2g]kL0De=C02l_;`0a<C40C4a<06=SH`1mOGd0?clo08B4Q00>ooooo`0H
05iNGP1:BTX0?clo034a<@1<C4`0Rh^;0=GEe@3De=@0];Bd0;6a/@2SXj<0^;Rh0<W9b@2m_Kd0fMWI
0;no_`0f=SH0TI6A0<;2`P1aLG40>SXj038b<P0[:b/0<C4a0P0b<S801`1eMGD0X:2P06i^KP0c<c<0
6ATI01@D500A4A40100D51@01`0A4A404a<C09VIV@3IfMT0hN7Q0<[:bP2k^k/00P2c/k<02`2a/K40
VYZJ08F5Q@2WYjL0Zj^[03De=@0E5AD0410@03De=@1@D5005QHF00<03Ph>0140410@00l?3`0G5aL0
cLg=0:ZZZP1@D500BTY:03Lg=`1dM7@061PH01DE5@0K6a/07QhN02/[:`13@d<0?Cdm01LG5`0201DE
5@0801LG5`25QHD0MGEe0861P@1[Jf/0@T9204A4A00n?Sh201DE5@1d028R8P0W9bL06a/K01hN7P0j
>SX0=SHf03/k>`2NWYh0Z:RX02l_;`0M7Ad0=cLg03Ph>01MGEd05QHF02h^;P0i>CT0JVYZ075aL@3@
d=00UYJF020P800Q8B409BDU01dM7@0L71`092@T03De=@0Z:RX0:BTY030`<00f=SH0=CDe092@T03P
h>00b/[:0<g=c@2WYjL0MGEe07AdM034a<@0a/K608Z:RP2PX:00TI6A0492@P11@D407alO01TI6@0O
7al0FEUI08R8R01mOGd0VIVI03Xj>P0g=cL0?clo0:j^[P22PX80PH610451@@1?Cdl0CDe=08N7Q`1f
MWH0=3@d04I6AP1YJFT0TI6A0:BTY02g]kL0g=cL0<[:bP1=CDd0Jf][04e=C@1?Cdl0CDe=09>CT`1Q
HF40IVIV0=kNgP3Lg=`0WYjN0861P@1TI6@0K6a/0:RXZ00o?cl0LW9b0<;2`P2IVIT0WYjN0:VYZ@2[
Zj/0^k^k0<71`@2d];@0UiNG09BDU014A4@0BTY:0<30`02QXJ40Rh^;09RHV02a/K40e]KF04i>CP1d
M7@0WIfM06UYJ@0D51@05ADE01<C4`0A4A4201DE5@0W01TI6@0X:2P0Kfm_02/[:`0K6a/0IfMW09>C
T`2:RXX0^KVi0<G5a@1dM7@0G5aL0:6QX@3De=@0`/;209FEU@2OWil0MGEe03`l?00];Bd0;2`/051@
D00e=CD04a<C01DE5@0B4Q805ADE01TI6@0l?3`0MgMg04A4A01QHF40DE5A02XZ:P19BDT0:2PX06e]
K@0b<S805QHF00<05ADE01d07QhN05aLG02LW9`0QXJ6092@T01cLg<0E5AD06=SH`1YJFT0`l?30<G5
a@2GUiL0:b/[02PX:01UIFD0W9bL09NGU`24Q8@0JVYZ05EEE@0c<c<0A4A408^;R`2>SXh0D51@05MG
E`2?Shl0^[Zj08N7Q`0@ooooo`003_ooool0S`1VIVH0LG5a05aLG012@T80Hf=S03Ph>01GEeL0JFUY
03De=@1IFET0ADE503Ti>@1MGEd0=CDe04A4A01TI6@0@41002`/;010@400LW9b03Xj>P1_Kfl0?Cdm
04=3@`1cLg<0ADE502h^;P27QhL0CDe=05EEE@1MGEd0@D5105EEE@0H61P0:2PX06e]K@0l?3`0LG5a
07AdM00];Bd0Cdm?05ADE018B4P0<30`06QXJ00i>CT0I6AT06IVIP0e=CD0>3Ph02d];@0g=cL0SHf=
06EUI@0X:2P0<C4a06YZJP0j>SX0NGUi04I6AP0h>3P0?3`l07emO@0f=SH0FUYJ07IfMP0`<300@D51
04U9B@14A4@0X:2P06][J`0g=cL0FUYJ03Lg=`0/;2`0FEUI0:^[Z`2QXJ40De=C04U9B@1kNg/0M7Ad
03Hf=P29RHT0@410034a<@1_Kfl0RHV9069RHP1bLW80Shn?06EUI@1?Cdl0BTY:07EeM@0l?3`0M7Ad
05]KF`0l?3`0JFUY0492@P1WIfL0Jf][07EeM@12@T80BTY:05]KF`0h>3P0J6QX03/k>`1@D500CTi>
075aL@26QXH0F5QH06a/K01NGUh0>c/k04m?C`1aLG40GemO04Q8B01ADE40MgMg069RHP1/K6`0KVi^
0492@P1@D500IfMW06e]K@1;Bd/0NWYj09bLW01=CDd0ZJVY065QH@0i>CT0SXj>06MWI`14A4@0WYjN
0080BDU90280EUIF06i^KP13@d<0GUiN03Ti>@1EEED0IVIV04I6AP1VIVH0GEeM05EEE@1eMGD0LG5a
0;jn_P1JFUX0;2`/079bLP2KVi/0F5QH0:b/[02o_kl0RHV903hn?P28R8P0D51@07MgM`3EeMD0Shn?
04e=C@27QhL0WYjN04E5A@2=SHd0RHV90P0g=cL0H@1BDU80KFe]04M7A`1@D500VYZJ06ATI013@d<0
Kfm_04m?C`2DU9@0Lg=c04U9B@1dM7@0@D5105UIF@1>CTh0:b/[04Y:BP1TI6@0QXJ603@d=01OGel0
FEUI03dm?@0c<c<0^[Zj0:6QX@1SHf<0;Bd]08N7Q`1/K6`0GEeM06MWI`0l?3`0OWin06m_K`15ADD0
De=C07IfMP23Ph<0DU9B04e=C@1nOWh0Bd];04I6AP1XJ6P0Ph>303Ti>@1oOgl0JFUY05ADE01TI6@0
HF5Q059BDP19BDT0B4Q807YjNP2WYjL0T92@03`l?01kNg/0K6a/028R8P1]KFd0Ogmo0451@@1RHV80
VIVI07UiN@0[:b/0T92@06i^KP1]KFd0?Shn07]kN`1iNGT0?Cdm09VIV@1IFET0=cLg05eMG@0`<300
QHF50861P@0o?cl0GUiN09fMW@18B4P0[Jf]05eMG@0d=3@0:BTY04Q8B01VIVH0Ng]k065QH@2ATI40
3_ooool0:P1CDe<0@d=304=3@`1<C4`0A4A403`l?025QHD0Rh^;04E5A@12@T80MGEe0:^[Z`3AdM40
^KVi0=GEe@2j^[X0:BTY08J6QP2QXJ40=cLg01`L700I6AT05aLG01XJ6P0G5aL06QXJ034a<@14A4@0
;2`/01LG5`0>3Ph04A4A01<C4`0A4A40410@01<C4`0D51@04a<C01LG5`2f][H0^KVi092@T080dm?C
00d0V9RH08V9R@3<c<`0S8b<05YJFP1BDU80[Zj^0;fm_@20P800ADE50820P00[:b/05QHF0080410@
01`05ADE01@D500A4A40HF5Q0=?Cd`3Hf=P0/K6a08F5Q@2][Jd0^KVi03`l?00G5aL04a<C01DE5@0J
6QX0:b/[07]kN`0T92@06ATI01DE5@0L71`0QXJ605UIF@2DU9@0IFEU04I6AP12@T80<C4a0P0E5AD0
AP0Z:RX0<S8b02TY:@0b<S809RHV04E5A@1>CTh0Q8B4075aL@0L71`05ADE02DU9@0];Bd0FEUI01LG
5`0I6AT0:BTY0451@@1cLg<0emOG0820P00O7al0BDU902XZ:P0L71`06ATI01HF5P0E5AD0:RXZ03Hf
=P0`<300<C4a04E5A@2WYjL0c/k>08b<S01BDU80GemO0820P03>c/h0^;Rh0;2`/02CTi<0C4a<09BD
U02YZJT0FUYJ01PH600J6QX06ATI05iNGP1OGel0EeMG07MgM`12@T80:b/[03Xj>P2OWil0UYJF04U9
B@2BTY80OWin06IVIP1fMWH0[jn_0;Rh^02BTY80V9RH0;2`/02_[jl20:2PX0070<W9b@18B4P0:b/[
05YJFP1`L700[jn_07MgM`0202PX:00O0;no_`3GemL0a/K60<G5a@3=cLd0gMgM09JFUP2MWId0^k^k
059BDP0`<300G5aL09JFUP2CTi<0Kfm_0:NWY`2i^KT0dM7A0:BTY02QXJ40ZZZZ0=SHf02NWYh0E5AD
09FEU@2n_[h0YJFU079bLP1fMWH0MGEe030`<00301<C4`1:01@D500A4A405ADE01TI6@15ADD0?Shn
01dM7@1DE5@0GemO02TY:@0R8R80Kfm_0<O7a`2XZ:P0OWin096AT@3>c/h0c<c<08B4Q00[:b/07alO
01dM7@0Q8B408b<S03Xj>P1IFET07alO01DE5@0C4a<05ADE01XJ6P0d=3@0TI6A03hn?P1CDe<0`L71
07EeM@1[Jf/0=CDe08f=S@0W9bL051@D01<C4`0E5AD06QXJ06ATI02RXZ80^KVi08J6QP1dM7@0De=C
02/[:`0i>CT0Vi^K0;fm_@2TY:@0JVYZ07YjNP0k>c/0=3@d0<?3``3Kfm/0d=3@06MWI`1IFET0KFe]
07UiN@0o?cl0:b/[055AD@2=SHd0XZ:R0:^[Z`2RXZ8@ooooo`003_ooool0K01MGEd0GUiN04m?C`1?
Cdl0MWIf04=3@`1CDe<0QhN7059BDP1TI6@0>CTi02XZ:P18B4P0>c/k02DU9@0h>3P0CDe=03Ti>@0X
:2P0ADE502@T901UIFD0>3Ph024Q8@1=CDd0<30`05EEE@2<S8`0HV9R03/k>`0b<S80;Bd]04m?C`0e
=CD0:2PX06i^KP0];Bd0EUIF07=cL`0f=SH0>SXj03De=@16ATH0:RXZ055AD@0K6a/0<c<c03/k>`16
ATH0=cLg030`<00_;bl0JVYZ06i^KP10@400=cLg05EEE@0d=3@0G5aL04i>CP1BDU80<c<c05aLG00n
?Sh0M7Ad07EeM@11@D40C4a<04E5A@0m?Cd0Rh^;06IVIP0g=cL0EEEE061PH00M7Ad0F5QH0:RXZ01`
L700CDe=030`<01HF5P0Bd];03Lg=`23Ph<0DU9B03Lg=`1]KFd0VYZJ05aLG01]KFd0PX:205eMG@1I
FET0BDU9065QH@0j>SX0N7Qh04i>CP1:BTX0Ogmo055AD@1dM7@0MgMg07AdM00m?Cd0AdM708N7Q`80
LW9b00H0<C4a04i>CP1WIfL0V9RH0861P@1CDe<206m_K`1g04i>CP1GEeL0LG5a065QH@1ADE40IfMW
07alO01PH600Lg=c07emO@16ATH0F5QH06][J`1iNGT0E5AD09:BTP2JVYX0CTi>0:VYZ@1`L700ADE5
09RHV01SHf<0De=C0:BTY019BDT0Cdm?04Q8B01SHf<0@d=307MgM`1DE5@0IfMW05ADE012@T80G5aL
06EUI@1DE5@0Ph>307AdM02o_kl0IfMW04i>CP2NWYh0T92@04];B`3;bl/0b<S809VIV@1:BTX0Vi^K
01/K6`1=CDd0_;bl09nOW`1?Cdl0Shn?08j>SP0a<C40/K6a08V9R@0d=3@0@41004U9B@1[Jf/0BDU9
05ADE028R8P0@d=304I6AP1]KFd0HV9R08^;R`1aLG40CTi>0:6QX@14A4@0EUIF05ADE00n?Sh0BDU9
0451@@1oOgl0A4A406EUI@1CDe<0A4A403/k>`2d];@0Vi^K05eMG@0O7al0JFUY06YZJP1<C4`0HF5Q
05QHF023Ph<0GemO03Lg=`1RHV80Rh^;07]kN`1@D500DU9B07YjNP1HF5P0CTi>08V9R@25QHD0A4A4
096AT@1]KFd0F5QH07UiN@1jNWX0FUYJ05=CD`0f=SH00P24Q8@09`1nOWh0;Bd]06m_K`1?Cdl0BTY:
0:^[Z`2JVYX0?Shn06IVIP25QHD0R8R802l_;`2n_[h0W9bL06MWI`0i>CT0LW9b02LW9`1@D500TI6A
05iNGP11@D40KFe]01dM7@1aLG40EUIF02HV9P0m?Cd0OWin06YZJP3=cLd0A4A402XZ:P10@400GUiN
04A4A01GEeL0WinO09^KV`0>ooooo`0G06MWI`15ADD0D51@0451@@0b<S80;Rh^03dm?@1aLG40;bl_
01LG5`0T92@0C4a<0<C4a02=SHd0[Zj^0;>c/`0b<S80NWYj07emO@14A4@06a/K01LG5`0E5AD00P0D
51@01@0E5AD0;Bd]03Lg=`0E5AD04A4A00<05ADE0540410@01DE5@0@410051@D01<C4`0c<c<0_[jn
08b<S02QXJ40e]KF0=;BdP1BDU80@41006UYJ@2SXj<0F5QH03<c<`11@D40LW9b09jNWP3Bd]80^[Zj
02TY:@0D51@04A4A01<C4`0O7al0B4Q80820P028R8P0C4a<04m?C`2DU9@0bLW90<K6aP1`L700I6AT
04a<C00[:b/06ATI01LG5`0K6a/0;2`/07]kN`0L71`05ADE02DU9@2BTY80ADE5096AT@2BTY80I6AT
08:2PP0/;2`06ATI01DE5@0m?Cd0FUYJ0410@01@D500B4Q804];B`15ADD0=CDe03Hf=P0G5aL05ADE
02<S8`0X:2P0GEeM01TI6@0F5QH08b<S03Lg=`1EEED0e]KF0:ZZZP1CDe<0DU9B01lO7`0H61P00`0E
5AD0D@0C4a<0;Bd]05mOG`28R8P0P820075aL@1lO7`0_kno07moO`14A4@0C4a<0<C4a02c/k<0BDU9
05iNGP2h^;P0DE5A05mOG`22PX80FEUI03Lg=`0T92@0MGEe04=3@`1:BTX0JVYZ05iNGP11@D40L71`
08f=S@22PX80BTY:034a<@2@T900][Jf08^;R`2a/K40cLg=0=7Ad@39bLT0Z:RX07IfMP1SHf<0IfMW
05YJFP1oOgl08b<S02@T902SXj<0/K6a051@D00S8b<0;bl_07]kN`1WIfL0Cdm?06i^KP25QHD0e]KF
0=kNgP2_[jl0>c/k01lO7`0/;2`08b<S0820P033`l<0ZJVY06][J`1PH600I6AT0;:b/P3>c/h0hN7Q
0>3Ph03<c<`0cLg=0=SHf02NWYh0Fe]K0:^[Z`020410@00403/k>`0N7Qh061PH01PH60<05aLG0100
6a/K06a/K00e=CD0DE5A06IVIP0];Bd08B4Q01dM7@1@D500f=SH0<O7a`2OWil0W9bL0<W9b@2HV9P0
<C4a0P0M7Ad201lO7`0C02@T900Y:BT0De=C04Y:BP0E5AD04a<C01DE5@0a<C40IFEU0;>c/`1YJFT0
Ti>C0=GEe@3Cdm<0Y:BT09VIV@35aLD08b<S010@400201<C4`0N05IFEP1/K6`0IFEU0;Jf]P1oOgl0
EeMG02d];@0W9bL0?Cdm08N7Q`2NWYh0SXj>05mOG`1PH600Ogmo09BDU03De=@0[Zj^0;:b/P2l_;`0
NGUi03/k>`0M7Ad071`L020P800d=3@0JVYZ09JFUP2YZJT0^k^k4?ooool000kooooo02h0GemO096A
T@1CDe<0EeMG07MgM`17AdL0De=C08f=S@1BDU80OGem069RHP0d=3@0CTi>06MWI`0/;2`0C4a<04m?
C`1@D500ATI606=SH`0/;2`0N7Qh05IFEP0N7Qh0K6a/04M7A`18B4P0IfMW069RHP0f=SH0F5QH0492
@P1GEeL0DU9B034a<@1OGel0;2`/04A4A01[Jf/0;bl_03dm?@1JFUX0B4Q802PX:01?Cdl0;2`/0P1=
CDd07@1<C4`0?Cdm03`l?00b<S80LG5a07inOP17AdL0@410079bLP0c<c<0FEUI055AD@1GEeL0<c<c
071`L00^;Rh0H61P07UiN@0j>SX0C4a<03/k>`1IFET0XJ6Q05QHF00e=CD0EeMG05YJFP0R8R80C4a<
0080YJFU03<0KFe]03/k>`1iNGT0BDU903Lg=`24Q8@0F5QH03`l?01nOWh0QHF504Q8B01iNGT0K6a/
04i>CP1NGUh0ATI604i>CP1@D500R8R803Ti>@16ATH0LW9b051@D01^KVh0TI6A07AdM012@T80IfMW
08N7Q`1XJ6P0K6a/03De=@1lO7`0IFEU096AT@1^KVh0Hf=S08f=S@1[Jf/0Cdm?05QHF01OGel0IfMW
06][J`1iNGT0JFUY06][J`2UYJD0MgMg0451@@1KFe/00P1oOgl0CP1HF5P0WYjN08J6QP1?Cdl0ZZZZ
07=cL`18B4P0T92@024Q8@0h>3P0XZ:R05YJFP1EEED0HV9R03hn?P1YJFT0Xj>S05YJFP1cLg<0HV9R
04=3@`2:RXX0Lg=c05YJFP1UIFD0OGem0<_;b`24Q8@0J6QX09jNWP1kNg/0>CTi0<71`@2VYZH0PH61
069RHP2][Jd0?Shn061PH038b<P0MWIf06IVIP2XZ:P0TY:B02DU9@1IFET0Jf][02HV9P0m?Cd0FEUI
07UiN@1OGel0GUiN08n?S`0];Bd0=CDe09>CT`1[Jf/0MgMg061PH00e=CD0QHF504a<C01ADE40@d=3
034a<@1TI6@0PH6109:BTP1;Bd/0JFUY051@D01XJ6P0C4a<0;^k^`20P800OGem02TY:@80Hf=S02L0
Fe]K09bLW01[Jf/0PX:208Z:RP0h>3P0P8200:NWY`2BTY80I6AT07=cL`29RHT09bLW061PH02OWil0
U9BD06=SH`2WYjL0=SHf06YZJP27QhL0LG5a06EUI@18B4P09BDU055AD@22PX80EEEE04=3@`21PH40
@D5101DE5@1RHV80NGUi04E5A@1nOWh0U9BD04A4A00Y:BT00P2ATI406@1[Jf/0CDe=08n?S`0n?Sh0
9BDU06][J`0f=SH0;Bd]071`L00/;2`0KFe]08Z:RP0n?Sh0?3`l08:2PP1mOGd0e]KF07MgM`0i>CT0
7AdM04a<C01CDe<08b<S03hn?P1ZJVX03_ooool02@1<C4`0@41004A4A00f=SH0:RXZ02HV9P0R8R80
Bd];01/K6`0201DE5@1>02TY:@2@T900TY:B09fMW@3;bl/0J6QX08:2PP1HF5P0Kfm_01/K6`0D51@0
4A4A01DE5@0D51@09bLW05eMG@1XJ6P06QXJ018B4P0>3Ph0410@01@D500E5AD061PH02<S8`0^;Rh0
BTY:0:n_[`3:b/X0c/k>0>3Ph03Rh^80YjNW07IfMP0^;Rh0:2PX07]kN`1hN7P0<S8b03hn?P1SHf<0
XZ:R0=3@d03<c<`0X:2P02TY:@10@400J6QX07alO01YJFT0@D5102DU9@0O7al0=cLg03Ti>@1_Kfl0
`/;20;Zj^P1YJFT0E5AD06QXJ01>CTh0=SHf02l_;`0P8200IFEU07QhN00o?cl0L71`09RHV02BTY80
CDe=04i>CP14A4@0Vi^K05UIF@0k>c/0<c<c0P1kNg/05`0n?Sh0;Rh^02`/;00d=3@0=CDe03lo?`0[
:b/05aLG01HF5P0g=cL0?Shn08F5Q@0V9RH05QHF01PH600h>3P0Ph>30<W9b@2LW9`0DU9B05EEE@0U
9BD05aLG00805ADE03L04a<C01DE5@0E5AD0G5aL0<k>cP2VYZH0S8b<07=cL`1RHV80[Jf]0<G5a@2c
/k<0/[:b0:j^[P19BDT0Cdm?0:^[Z`1@D500QhN70492@P10@400EeMG071`L02h^;P0@D5104e=C@1K
Fe/0[:b/08N7Q`1:BTX0:BTY028R8P19BDT0DU9B07EeM@2OWil0E5AD09fMW@35aLD0d=3@07inOP2?
Shl0S8b<05ADE00^;Rh07AdM06ATI02PX:00UIFE09>CT`2@T900@D51030`<01CDe<0<S8b00<05QHF
00H06QXJ075aL@3Hf=P0@d=302/[:`0];Bd201TI6@0Y03Ti>@2KVi/0WinO06ATI00m?Cd0;bl_04I6
AP34a<@0j^[Z092@T01SHf<0[jn_0<o?c`2>SXh0;Bd]0<W9b@3=cLd0_Kfm0;Ng]`2NWYh0QhN707Ui
N@1cLg<0MGEe08F5Q@2GUiL0S8b<08f=S@1/K6`0>c/k02PX:00R8R808B4Q06MWI`1FEUH0R8R802d]
;@0O7al0BDU909jNWP0j>SX00`0L71`03P0N7Qh0820P02PX:00i>CT0K6a/05=CD`1=CDd0IfMW08Z:
RP2n_[h0bl_;0<30`02o_kl0V9RH0P32`/820=[JfP0R05]KF`0I6AT0;2`/06a/K01SHf<0Cdm?05eM
G@2IVIT0XJ6Q05EEE@0n?Sh0;bl_05eMG@2:RXX0Q8B403`l?00Y:BT0>3Ph04U9B@1kNg/0[jn_04i>
CP21PH40W9bL01`L700I6AT07QhN024Q8@0S8b<0@41006m_K`2LW9`0VIVI08f=SA3ooooo000>oooo
o`0b06IVIP2ATI40EUIF05MGE`1bLW80ADE504];B`2:RXX0CTi>07inOP1PH600=3@d051@D01WIfL0
>CTi05QHF01PH600GemO03dm?@1cLg<0;bl_07moO`1>CTh061PH061PH015ADD0FEUI06i^KP1NGUh0
=cLg05]KF`1RHV80De=C04=3@`0/;2`0IVIV02h^;P1DE5@0GUiN02d];@18B4P0IVIV04];B`0a<C40
EUIF030`<01?Cdl0@D5103Ti>@10@40202HV9P1406QXJ01jNWX0BDU904Q8B01_Kfl0=SHf061PH01K
Fe/0EeMG03/k>`1dM7@0=3@d059BDP20P800>CTi04Y:BP10@400GEeM08^;R`1DE5@0@T9206a/K01F
EUH0:2PX03Ph>00g=cL0RHV907AdM00h>3P0PH6104=3@`1@D500WYjN04Y:BP0c<c<0KFe]07emO@0l
?3`0/K6a08b<S01DE5@0GUiN04];B`10@400?Cdm09FEU@1QHF40@T9205ADE00X:2P0M7Ad092@T00o
?cl09RHV05UIF@1iNGT0Cdm?061PH00F5QH0HF5Q05ADE02;Rh/0J6QX03dm?@1YJFT0AdM704Q8B01J
FUX202DU9@2L04e=C@1gMgL0ATI6034a<@1EEED0HF5Q03dm?@1BDU80BTY:06ATI01UIFD0_kno09fM
W@0l?3`0Vi^K06=SH`1OGel0V9RH055AD@0R8R80JFUY04=3@`1XJ6P0OGem02l_;`0a<C40GemO05MG
E`24Q8@0ATI6020P801IFET0Shn?06=SH`0Y:BT0;Bd]08R8R01_Kfl0GUiN09bLW01iNGT0Jf][0:BT
Y02VYZH0BTY:05aLG02RXZ80B4Q806m_K`3;bl/0IFEU051@D01lO7`0U9BD04a<C01RHV80Shn?03Ph
>012@T80DU9B05mOG`1CDe<0Lg=c0;6a/@1<C4`0;2`/04U9B@18B4P0TI6A06m_K`0n?Sh0KFe]02LW
9`1ZJVX0BDU901XJ6P0R8R80GUiN07EeM@19BDT0N7Qh04a<C01BDU80@D510<K6aP31`L40UiNG02LW
9`1OGel0?Cdm04U9B@2>SXh0JFUY071`L02/[:`0@D51059BDP1]KFd0@d=304U9B@1WIfL0NWYj04=3
@`0k>c/0GemO06=SH`1YJFT0RXZ:04=3@`0k>c/0FEUI02LW9`1`L700H61P04I6AP1nOWh0Ng]k04i>
CP0[:b/0GemO05IFEP0J6QX0J6QX05]KF`0W9bL0Cdm?06IVIP1FEUH09bLW0:n_[`29RHT0CDe=02l_
;`1]KFd0MGEe03<c<`1bLW80AdM703@d=01YJFT0<C4a06i^KP21PH40>CTi03`l?02:RXX0I6AT0:FU
Y@1dM7@0=CDe024Q8@0l?3`0F5QH02TY:@1=CDd0U9BD3_ooool0?01@D500?3`l03Ti>@0^;Rh09RHV
01hN7P0K6a/0Bd];02h^;P0D51@04Q8B01PH601oOgl0c<c<0<W9b@3IfMT0^;Rh0:ZZZP2>SXh0/[:b
08J6QP1;Bd/0ATI60451@@0h>3P0;bl_01dM7@0d=3@0B4Q803Ph>017AdL0EeMG06YZJP1[Jf/0GUiN
06UYJ@1SHf<0UiNG0<g=c@2f][H0Q8B4075aL@1jNWX0VIVI07YjNP1JFUX092@T03lo?`22PX80?Shn
01XJ6P0/;2`0>3Ph06EUI@2TY:@0b/[:0:VYZ@1UIFD0?Cdm02DU9@807QhN03@09BDU02TY:@0k>c/0
S8b<0;Ng]`2^[Zh0D51@08f=S@27QhL0C4a<05UIF@1VIVH0CDe=03dm?@1>CTh0RHV90;:b/P3Kfm/0
bLW90:6QX@15ADD0<c<c03lo?`0n?Sh05ADE01@D500C4a<0=CDe07IfMP1PH600A4A4038b<P0/;2`0
=SHf04E5A@0`<3007alO01HF5P0j>SX0SHf=0=7Ad@2][Jd0Ng]k04a<C01?Cdl0b<S80;Fe]@2_[jl0
E5AD02LW9`0N7Qh06QXJ0`0E5AD0=P0C4a<05QHF06QXJ010@400=3@d07=cL`2;Rh/0Q8B407YjNP3L
g=`0clo?06][J`2GUiL0DE5A05EEE@2h^;P0[Jf]0:NWY`0];Bd09BDU030`<00c<c<0PX:207inOP18
B4P0MgMg07alO01?Cdl08B4Q01/K6`0N7Qh08b<S051@D031`L40ZZZZ0=GEe@3@d=00]kNg0<?3``2A
TI40FEUI03<c<`0Q8B405aLG01DE5@0S8b<0Lg=c0=7Ad@1YJFT0:b/[05=CD`28R8P0O7al01lO7`80
5ADE03<05aLG01HF5P1QHF40cLg=04I6AP0i>CT07AdM020P800n?Sh0=cLg07alO02JVYX0Fe]K02PX
:016ATH0RXZ:0<C4a03Bd]80YJFU09jNWP2:RXX0GEeM08Z:RP1kNg/0a/K60;Vi^@2`/;00UYJF051@
D00^;Rh0:b/[038b<P0c<c<0BTY:08n?S`36a/H0LG5a0492@P0d=3@09bLW02<S8`0Q8B40HF5Q030`
<01CDe<0GUiN01`L700c<c<0U9BD06YZJP0N7Qh00P0L71`2020P800b02LW9`0f=SH0RHV90=7Ad@2<
S8`0LG5a08Z:RP2GUiL0Hf=S08B4Q02RXZ80[:b/0<S8b03Cdm<0WIfM09ZJVP3>c/h0YZJV0;>c/`1V
IVH0Bd];05MGE`1LG5`0M7Ad09:BTP2GUiL0NWYj05IFEP2DU9@0`l?309RHV00Z:RX0;2`/03dm?@0/
;2`0BTY:0;Vi^@2?Shl0a<C409NGU`0J6QX06ATI01dM7@0N7Qh0<30`05IFEP1nOWh0UIFE04i>CP2:
RXX@ooooo`003_ooool04028R8P0Shn?05QHF01KFe/0RHV904A4A01:BTX0NWYj04e=C@29RHT0De=C
03De=@1CDe<0F5QH05aLG01aLG4205YJFP0;051@D02:RXX0?Cdm08f=S@1BDU80HF5Q07UiN@11@D40
CTi>06QXJ01<C4`00P1DE5@05P1QHF40EUIF02PX:01LG5`0Xj>S03`l?01FEUH0JVYZ03<c<`0n?Sh0
Hf=S02d];@1;Bd/0GemO030`<01@D500CTi>01dM7@1CDe<0@d=302PX:01TI6@203hn?P2Y08F5Q@1^
KVh0=SHf061PH00a<C40;Bd]04e=C@1eMGD0?Cdm05eMG@1UIFD0@41006IVIP1WIfL0KVi^08f=S@0_
;bl08B4Q05=CD`1DE5@0<c<c04m?C`2HV9P0Shn?051@D00h>3P0QHF503De=@0X:2P0FEUI06i^KP0b
<S80Hf=S096AT@0];Bd0IfMW071`L015ADD0MgMg06MWI`1MGEd0>3Ph06][J`0^;Rh0EeMG05]KF`0_
;bl0Fe]K09BDU01GEeL09BDU04A4A01=CDd0<C4a06EUI@0Z:RX0BTY:03/k>`2BTY80JVYZ04A4A01A
DE40@T92055AD@1VIVH0GemO04U9B@0n?Sh0Kfm_07QhN01BDU80MWIf05ADE00b<S80EeMG06=SH`1Y
JFT0EeMG08Z:RP21PH40BDU909FEU@1CDe<0@T9207]kN`1SHf<08B4Q06i^KP12@T80Bd];05EEE@1I
FET0?Cdm04i>CP11@D40S8b<0410@00Y:BT0Cdm?05QHF01>CTh0CDe=05eMG@2CTi<0?clo02d];@2@
T900S8b<09FEU@2FUYH0Z:RX07EeM@1cLg<0/K6a051@D029RHT0c<c<09:BTP1[Jf/0Kfm_09fMW@1L
G5`0Lg=c06a/K00o?cl0KFe]05QHF01ZJVX0<C4a04U9B@2=SHd0LW9b04Y:BP1kNg/0Fe]K08J6QP1h
N7P0A4A408f=S@0g=cL0DE5A04U9B@0/;2`0>CTi05mOG`20P800=cLg09VIV@1iNGT0HF5Q024Q8@1I
FET0IVIV075aL@0];Bd0GEeM04U9B@1=CDd0JFUY03De=@12@T80L71`04Q8B01NGUh0Shn?065QH@1F
EUH0J6QX07=cL`02059BDP0d06i^KP20P800FEUI08j>SP1[Jf/0B4Q808B4Q01JFUX0IFEU03lo?`0m
?Cd0UiNG07emO@1KFe/0<c<c06MWI`1BDU808b<S08j>SP1jNWX0<30`05mOG`1iNGT0?Cdm03De=@2S
Xj<0PX:2069RHP0^;Rh0GUiN05EEE@1ADE40RXZ:05]KF`0b<S80IfMW03<c<`1LG5`0Ph>303lo?`11
@D40Zj^[071`L02n_[h0DU9B02/[:`0R8R80<c<c051@D00Y:BT0RXZ:0:6QX@kooooo00T0BDU903Lg
=`0e=CD0:RXZ02PX:00R8R807AdM03@d=01HF5P00P0E5AD07P0];Bd0XZ:R0=OGe`3Jf]X0b<S80<;2
`P1UIFD0MWIf08R8R023Ph<0=cLg01dM7@0O7al08B4Q03Ti>@1TI6@0MWIf08V9R@1jNWX0GEeM051@
D01;Bd/0>CTi02XZ:P0Y:BT0820P04];B`2VYZH0<S8b024Q8@807QhN0340Fe]K09>CT`29RHT0IFEU
0451@@2EUID0ZZZZ05eMG@0o?cl0>c/k02/[:`1jNWX0`<3007=cL`0g=cL0>CTi0410@01<C4`0FUYJ
06ATI01OGel0Jf][06YZJP24Q8@0Xj>S06a/K00e=CD0IFEU09nOW`2DU9@0IVIV05mOG`1bLW80C4a<
05aLG01XJ6P0fMWI0=[JfP1/K6`0>3Ph034a<@19BDT06QXJ018B4P0D51@05ADE01hN7P1ADE40Bd];
00807AdM01H071`L01hN7P0^;Rh09bLW038b<P0O7al0P8200<g=c@2BTY80DE5A07IfMP2][Jd0e=CD
0>3Ph03MgMd0YjNW08Z:RP0[:b/06ATI01`L700J6QX061PH0P0G5aL0=P0Y:BT0LG5a028R8P0O7al0
B4Q804M7A`18B4P0JFUY0<30`03@d=00HF5Q06UYJ@22PX80F5QH0:2PX01MGEd0HF5Q03dm?@0U9BD0
92@T02XZ:P1MGEd0Zj^[07MgM`1iNGT0FUYJ05mOG`0Y:BT061PH01TI6@0U9BD09RHV08V9R@35aLD0
e=CD09nOW`1jNWX0Shn?02/[:`0W9bL0:BTY02<S8`0G5aL06ATI01XJ6P1TI6@0dM7A0:VYZ@1oOgl0
@4100:NWY`1iNGT0;Rh^01LG5`805QHF06P071`L05QHF03<c<`0CDe=028R8P0P8200;Rh^01LG5`0[
:b/0A4A40;:b/P30`<00^k^k06e]K@2UYJD0dM7A0:FUY@2o_kl0CTi>01dM7@0P8200HV9R0;2`/03G
emL0QXJ605iNGP1kNg/0OGem03Ph>00j>SX0>CTi0492@P1LG5`0^;Rh0820P030`<00b<S809BDU01_
Kfl0ADE503/k>`1RHV80;2`/04U9B@1aLG40<S8b04I6AP2BTY80EEEE02h^;P0O7al06a/K020P800X
:2P0;2`/06a/K02IVIT0[jn_07alO01oOgl0N7Qh055AD@2=SHd0Vi^K09bLW02o_kl0WYjN0:BTY02@
T900VIVI0;>c/`3Ph>00UiNG0:>SX`2RXZ80JVYZ055AD@0W9bL0<30`02DU9@1LG5`0Ti>C07EeM@1V
IVH0MGEe05=CD`0m?Cd0=cLg03De=@1OGel0`l?30=CDe03Ng]h0e=CD03`l?00M7Ad06QXJ01lO7`1;
Bd/0Fe]K08B4Q01?Cdl0Lg=c079bLQ3ooooo000>ooooo`0l05mOG`1KFe/0Fe]K04Q8B01RHV80>CTi
05MGE`1MGEd0;Rh^061PH00];Bd0>3Ph04M7A`0g=cL0>3Ph038b<P12@T80;Rh^038b<P1:BTX0:RXZ
07alO00f=SH0>c/k04A4A00];Bd0BDU907UiN@1EEED0=3@d03dm?@1;Bd/0FEUI03@d=00T92@0H61P
02/[:`1SHf<0E5AD02l_;`0T92@0ATI602TY:@0[:b/0AdM702TY:@15ADD0@D5103Ti>@11@D40=CDe
028R8P1aLG40HF5Q03Lg=`1=CDd0DE5A03Ti>@1MGEd0@D510P0d=3@0f01>CTh0>3Ph069RHP1cLg<0
=CDe02d];@13@d<0E5AD08V9R@1DE5@0@D5105EEE@0l?3`0@d=304i>CP2MWId0Q8B403De=@0o?cl0
NGUi04];B`0i>CT0MGEe04E5A@0e=CD0IfMW096AT@12@T80UYJF08N7Q`13@d<0Hf=S04Y:BP1QHF40
FEUI0861P@12@T80BDU906i^KP1;Bd/0GEeM07IfMP1XJ6P0;Rh^04i>CP1^KVh0E5AD06i^KP0j>SX0
De=C069RHP27QhL0IFEU05EEE@24Q8@0EEEE0492@P1LG5`0Jf][06EUI@1@D500Ph>306m_K`1ADE40
LG5a07UiN@0k>c/0>SXj06MWI`1eMGD0Rh^;0:f][@1nOWh0D51@0::RXP1MGEd0De=C09JFUP1ZJVX0
=3@d05IFEP17AdL0D51@05iNGP1RHV80@D5106][J`15ADD0Rh^;03lo?`1>CTh0G5aL06e]K@1@D500
LW9b07EeM@2NWYh0DU9B04e=C@2>SXh0PH6109:BTP2DU9@0Y:BT06m_K`2EUID0WIfM04];B`2TY:@0
/[:b075aL@1hN7P0PH6108j>SP1mOGd0M7Ad06EUI@0g=cL0F5QH05=CD`1LG5`0DE5A065QH@2OWil0
E5AD055AD@2<S8`0LW9b08B4Q01PH600B4Q80861P@0o?cl0HF5Q05ADE00e=CD0D51@07IfMP2KVi/0
@d=305EEE@1DE5@0GUiN05eMG@37alL0ZZZZ092@T00T92@0E5AD04M7A`1NGUh0Ogmo06IVIP0n?Sh0
MgMg03lo?`1MGEd0T92@079bLP1CDe<0KFe]07]kN`1<C4`0H61P09>CT`23Ph<0C4a<09FEU@1?Cdl0
Bd];0::RXP1aLG40J6QX02l_;`0f=SH0Shn?08j>SP1RHV80;bl_065QH@12@T80<S8b08R8R027QhL0
=SHf06a/K02:RXX0NWYj02`/;02m_Kd0Ng]k03Hf=P0`<300GUiN05EEE@0`<300DE5A04E5A@0^;Rh0
LW9b030`<01OGel0KFe]038b<P0i>CT0TI6A05mOG`2KVi/0HV9R02l_;`0d=3@0BTY:04e=C@0F5QH0
Bd];07inOPkooooo03/0HF5Q04];B`0g=cL0:RXZ02<S8`0O7al07AdM028R8P2=SHd0IFEU071`L023
Ph<0TI6A09:BTP2YZJT0bl_;0<k>cP2RXZ80Rh^;07MgM`0o?cl0FEUI09VIV@2LW9`0OWin07]kN`1L
G5`0DU9B069RHP17AdL09RHV01lO7`0J6QX06ATI01PH600J6QX07QhN05iNGP0h>3P07AdM01`L700J
6QX05aLG01hN7P1FEUH0QHF50;jn_P3Bd]80a/K60;fm_@26QXH0LW9b06YZJP1SHf<0Shn?0<c<c01_
Kfl0@D510410@00203hn?P0403lo?`0^;Rh0<c<c04A4A080=3@d01X0:RXZ04Y:BP1<C4`0820P02`/
;00o?cl0GUiN07EeM@23Ph<0XZ:R0;2`/03Fe]H0gMgM0=cLg02HV9P0UIFE09>CT`1]KFd09RHV01dM
7@0G5aL051@D01XJ6P0V9RH0=CDe02TY:@806a/K03D07AdM02<S8`16ATH0N7Qh08j>SP2QXJ40IFEU
03lo?`10@400?Shn05]KF`2ZZZX0alO70=OGe`2HV9P0@41004a<C018B4P0A4A403hn?P0W9bL07QhN
02h^;P1]KFd0@D5104Q8B01UIFD0BTY:02DU9@0V9RH0<C4a04Q8B029RHT0?Cdm04A4A02^[Zh0T92@
07emO@1eMGD0?3`l0410@00i>CT0<C4a03De=@1>CTh0Z:RX09jNWP0i>CT08R8R04];B`1>CTh0:RXZ
01PH600201dM7@0N03Xj>P1OGel0OGem079bLP1QHF40DU9B020P800X:2P0;bl_01lO7`0F5QH05aLG
01XJ6P1jNWX0B4Q802PX:013@d<0`<300;:b/P1GEeL0F5QH05eMG@1ADE40@T920451@@0];Bd0[jn_
02l_;`0J6QX05aLG0P0E5AD02P0a<C40@41005MGE`3Cdm<0ZZZZ07alO01DE5@0FUYJ07]kN`0m?Cd2
01`L701D028R8P11@D40K6a/06UYJ@1UIFD0LW9b08F5Q@2?Shl0MWIf06ATI01JFUX0KVi^0:VYZ@2C
Ti<0VYZJ0:FUY@2>SXh0O7al071`L01cLg<0QhN706][J`0[:b/0A4A4071`L02e]KD0Lg=c0820P025
QHD0EEEE05=CD`1IFET0A4A404a<C02<S8`0e]KF0<C4a02WYjL0Jf][03De=@0X:2P0@d=307inOP2^
[Zh0MWIf08B4Q02g]kL0J6QX05QHF01GEeL0TY:B0;Jf]P2;Rh/0Q8B408J6QP2k^k/0Jf][03lo?`0g
=cL0>CTi03Xj>P1OGel0Rh^;06a/K026QXH0];Bd08^;R`1gMgL0WIfM09nOW`2`/;00RHV905ADE01J
FUX0S8b<06MWI`0j>SX0De=C08J6QP1dM7@0@T9204a<C01nOWh0B4Q84?ooool000kooooo02X0JVYZ
08^;R`1NGUh0FUYJ07moO`12@T80AdM706EUI@0[:b/0JFUY03`l?00m?Cd0B4Q8055AD@1;Bd/0GemO
05=CD`1VIVH0ADE506QXJ00Y:BT0OWin05mOG`13@d<0Bd];03dm?@18B4P0KVi^04a<C012@T80JFUY
05IFEP1GEeL0@D5102d];@23Ph<0@41007QhN01/K6`0;bl_04U9B@1BDU820410@03Z06m_K`0j>SX0
DE5A0410@00^;Rh0H61P04U9B@0S8b<0GUiN04Y:BP0l?3`0NWYj06UYJ@0];Bd0IfMW071`L01IFET0
>SXj06m_K`0a<C40JVYZ06e]K@12@T80C4a<05mOG`1ZJVX0PX:20451@@0m?Cd0DU9B04M7A`1BDU80
EEEE07UiN@1bLW80@T9203/k>`21PH40C4a<04M7A`2ATI40E5AD03Xj>P1FEUH0TI6A03Hf=P1gMgL0
N7Qh0410@01iNGT0QHF505mOG`1EEED0Ogmo04E5A@1NGUh0LW9b04Y:BP1RHV80NWYj06UYJ@13@d<0
JVYZ06EUI@19BDT0Kfm_0492@P1YJFT0F5QH08F5Q@1BDU80BTY:07MgM`1:BTX0DE5A06IVIP1=CDd0
IfMW05IFEP2GUiL0JVYZ06MWI`22PX80K6a/04m?C`1ADE40LW9b06EUI@2:RXX0^[Zj07inOP1JFUX0
V9RH065QH@1/K6`0TY:B03@d=011@D40Ng]k04U9B@1CDe<0Lg=c0410@01FEUH0LG5a03hn?P2ATI40
@41006m_K`29RHT0Hf=S05UIF@1HF5P0L71`0:RXZ01GEeL0De=C0;6a/@1cLg<0IfMW07QhN028R8P0
?clo07moO`2ATI40E5AD0;2`/02][Jd0FEUI06=SH`1MGEd0Q8B408:2PP1OGel0F5QH030`<01YJFT0
F5QH051@D00_;bl0FUYJ09NGU`1YJFT0EEEE06ATI015ADD0SXj>05QHF01;Bd/0Jf][01XJ6P1fMWH0
EeMG030`<015ADD0EEEE09>CT`10@400GUiN04i>CP1TI6@0A4A409>CT`25QHD0V9RH02HV9P1`L700
Jf][06=SH`28R8P0J6QX04U9B@2CTi<0ATI605MGE`2>SXh0<30`04e=C@1iNGT0Ogmo05iNGP1SHf<0
MWIf0492@P1>CTh0U9BD05]KF`0o?cl0XJ6Q03`l?01gMgL0H61P05EEE@2LW9`0JFUY03Hf=P15ADD0
Lg=c04e=C@0N7Qh0Ng]k03dm?@0P8200@d=307EeM@1hN7P0;Bd]0<c<c02=SHd0LW9b03@d=01JFUX0
HF5Q03Lg=`10@400C4a<03Lg=`1YJFT0>c/k07QhN022PX80@410034a<@1hN7P0H61P09BDU01WIfL0
<c<c01LG5`0S8b<0CTi>01/K6`13@d<0RXZ:3_ooool0:@2XZ:P0Shn?07AdM011@D40>SXj02HV9P0P
8200@D510;>c/`1FEUH0HF5Q04m?C`0L71`092@T06QXJ02MWId0aLG50<?3``2l_;`0RXZ:06i^KP2U
YJD0dm?C0<c<c027QhL0?Shn02TY:@0N7Qh07AdM03/k>`1NGUh0BTY:04m?C`0d=3@0<S8b030`<01R
HV80=3@d01lO7`0P82007alO0080820P01L07QhN01`L701RHV80TY:B0=;BdP1DE5@0PH610410@014
A4@0CDe=04=3@`2@T900OGem08N7Q`0b<S80<C4a02d];@0/;2`08B4Q03`l?00i>CT0;2`/01dM7@03
01DE5@0;04a<C01TI6@07alO01TI6@0L71`07QhN02DU9@1>CTh0][Jf0=3@d03<c<`00P3WinL0502M
WId0EeMG08:2PP18B4P0@D5103`l?00b<S80;bl_01lO7`0L71`07alO04e=C@1IFET0:RXZ01lO7`0n
?Sh0VYZJ0=OGe`1OGel0<S8b0P0G5aL0700d=3@0QXJ604Y:BP1nOWh0SHf=096AT@11@D40<30`03Xj
>P0a<C40@D5105UIF@1cLg<0]kNg04Y:BP16ATH0QXJ606e]K@0V9RH0:2PX02DU9@0i>CT0ATI60820
P016ATH0>3Ph06i^KP2j^[X209jNWP0907AdM01OGel0DE5A06MWI`23Ph<0AdM708>3P`2JVYX0>CTi
0080:BTY01<0>c/k04m?C`0U9BD06ATI01`L7016ATH0>3Ph059BDP2MWId0PH6105IFEP0R8R80:b/[
01lO7`0I6AT061PH01TI6@0O7al0Jf][0080:b/[00`0OGem07IfMP1=CDd0JVYZ04Y:BP1cLg<0[Zj^
04I6AP0T92@0:b/[04U9B@0J6QX401DE5@1804M7A`2DU9@0emOG0=[JfP1BDU807AdM01/K6`0S8b<0
EeMG01PH600I6AT06a/K034a<@0F5QH0JFUY0451@@1NGUh0:RXZ03dm?@2TY:@0OWin07EeM@1`L700
RHV90;^k^`2MWId0b<S808n?S`10@4008R8R01dM7@0L71`09BDU05eMG@1?Cdl0U9BD0:6QX@2JVYX0
CTi>07AdM00U9BD0>c/k03Xj>P0N7Qh0T92@0>3Ph02e]KD0_;bl069RHP14A4@06ATI01DE5@0S8b<0
ADE508^;R`0g=cL08B4Q03Hf=P2>SXh0;Rh^03/k>`1>CTh0Bd];07YjNP2^[Zh0Cdm?04M7A`1FEUH0
ADE501DE5@0C4a<05ADE0P0D51@05P0Z:RX0EeMG03Ti>@1GEeL0bLW90=OGe`2VYZH0b/[:09JFUP1^
KVh0EEEE05ADE01=CDd0IFEU0<C4a02m_Kd0De=C02h^;P18B4P0QXJ605MGE`0m?Cd@ooooo`003_oo
ool20820P00k06MWI`1:BTX0KFe]03hn?P15ADD0T92@0492@P27QhL0ADE504=3@`1ADE40>SXj0492
@P1PH600CTi>04];B`1=CDd0MgMg02XZ:P2=SHd0>c/k05QHF01?Cdl0>SXj04U9B@1UIFD0?Cdm04I6
AP1JFUX0DE5A05IFEP0L71`0?3`l09JFUP0h>3P0Lg=c05ADE00l?3`0B4Q805QHF00_;bl0;Bd]04e=
C@0[:b/0FEUI03Ph>00Q8B40AdM703<c<`0R8R80FUYJ02LW9`0[:b/0Q8B404Y:BP0e=CD0J6QX04U9
B@0g=cL00P1SHf<0e`0`<300JVYZ05YJFP0X:2P0;bl_04i>CP1DE5@0Q8B405MGE`14A4@0BTY:02`/
;01?Cdl0IFEU092@T025QHD0;Rh^04Q8B02@T900FEUI05]KF`2>SXh0B4Q803lo?`1UIFD0U9BD0492
@P1lO7`0HF5Q030`<01JFUX0CDe=059BDP1SHf<0Q8B402h^;P1FEUH0LG5a04U9B@1@D500Ph>306a/
K00b<S80Cdm?04E5A@0h>3P0OGem03/k>`1>CTh0=cLg071`L01fMWH0De=C061PH014A4@0ADE505iN
GP1PH600E5AD04a<C01^KVh0G5aL05mOG`1LG5`0EEEE07MgM`1<C4`0JVYZ04U9B@1]KFd0W9bL06m_
K`1?Cdl0MWIf04Q8B01QHF40Rh^;05]KF`16ATH0H61P03<c<`0b<S80IFEU059BDP0o?cl0ATI603hn
?P2DU9@0>CTi04e=C@1MGEd0@d=305mOG`1?Cdl0F5QH07inOP0l?3`0?Cdm09RHV01bLW80N7Qh06a/
K02DU9@0FUYJ09RHV02NWYh0GUiN0<C4a02XZ:P0LW9b07alO01/K6`0NGUi0:FUY@2IVIT0QHF502LW
9`1KFe/0FEUI04U9B@11@D40DE5A07=cL`1=CDd0FEUI06e]K@1kNg/0/[:b069RHP1;Bd/0OWin02LW
9`1hN7P0@d=303Ph>01:BTX0De=C09>CT`0X:2P0LW9b08N7Q`1iNGT0EUIF0;jn_P2QXJ40MWIf03<c
<`1oOgl0Fe]K05mOG`1bLW80=SHf03`l?02:RXX0@D5105mOG`2;Rh/0Fe]K05EEE@1_Kfl0Ogmo061P
H01eMGD0V9RH07QhN012@T80Lg=c06EUI@17AdL0YZJV07EeM@1cLg<0Ogmo05]KF`2LW9`0Ogmo06IV
IP17AdL0Hf=S04U9B@0Q8B40SXj>096AT@1:BTX0CDe=0820P02ATI40=SHf0;>c/`2GUiL0Q8B403Xj
>P1<C4`0LG5a03Xj>P1OGel0GUiN0451@@12@T807AdM061PH01lO7`0=SHf034a<@22PX80P8200:NW
Y`1MGEd0=SHf01<C4`0N7Qh0De=C01/K6`1<C4`0QhN700kooooo01T0K6a/05mOG`1ZJVX0S8b<051@
D011@D40E5AD0:FUY@1`L700KFe]02h^;P0J6QX07AdM059BDP1/K6`0UiNG05=CD`2_[jl0VYZJ08n?
S`2k^k/0f][J0=7Ad@2EUID0Jf][00806a/K01L07AdM01LG5`0X:2P0LG5a03Lg=`0d=3@09bLW02TY
:@0];Bd0GEeM0410@01ADE40EeMG05IFEP18B4P0A4A40492@P1?Cdl0Kfm_065QH@34a<@0MWIf038b
<P0201DE5@805QHF05h0=cLg06e]K@1HF5P0H61P03`l?00];Bd0:BTY02PX:010@400CTi>02h^;P0L
71`06ATI01XJ6P0E5AD07AdM09RHV01LG5`0BDU902XZ:P0K6a/07AdM02HV9P29RHT0dm?C0;:b/P38
b<P0VYZJ0<C4a02k^k/0`/;207=cL`1BDU80D51@08N7Q`2@T900O7al07MgM`1iNGT0Rh^;09:BTP1Y
JFT0GEeM07]kN`2o_kl0dm?C04e=C@0k>c/05QHF024Q8@1NGUh0=3@d03lo?`2MWId0NWYj06ATI01=
CDd0EUIF04e=C@13@d<0>CTi03Ph>01GEeL0b<S8092@T025QHD0SXj>03<c<`0Q8B40<c<c01dM7@0Q
8B40Hf=S08f=S@0n?Sh0;Bd]07]kN`1ZJVX0D51@06ATI00d=3@0AdM706MWI`2`/;00De=C075aL@1l
O7`0X:2P04=3@`0N7Qh0820P024Q8@0k>c/0=cLg0P0L71`02@18B4P0?3`l05EEE@2=SHd0Rh^;079b
LP0[:b/0A4A401hN7P0201XJ6P0J01/K6`0/;2`0PX:204I6AP26QXH0GemO03De=@0N7Qh0<30`05QH
F01YJFT0JVYZ03Lg=`0i>CT0;2`/03<c<`0J6QX05ADE01PH600U9BD0J6QX0:NWY`2TY:@0XJ6Q0:b/
[02ZZZX201TI6@0Q02DU9@1ADE4061PH01/K6`0O7al0:2PX01LG5`1IFET0<30`03Xj>P18B4P0@410
0:>SX`2?Shl0OGem075aL@28R8P0U9BD05IFEP2EUID0]kNg08B4Q01WIfL0MGEe04Q8B01UIFD0[:b/
092@T015ADD0Bd];05UIF@1XJ6P0LG5a00806ATI03H0>SXj05YJFP28R8P0bLW904M7A`1JFUX0D51@
03`l?00H61P09BDU05EEE@2n_[h0Ng]k01hN7P0O7al09RHV04];B`0P820071`L02TY:@1ADE40SXj>
092@T00M7Ad09RHV02LW9`18B4P07AdM01<C4`0D51@04a<C018B4P0F5QH0JFUY07moO`1KFe/0^KVi
0;>c/`1WIfL0UIFE09VIV@11@D40ADE504U9B@1OGel0K6a/09:BTP2l_;`0H61P06i^KP24Q8@0E5AD
024Q8@0N7Qh@ooooo`003_ooool0bP1QHF40De=C05YJFP1;Bd/0Hf=S03<c<`1=CDd0Q8B40410@01m
OGd0?3`l05IFEP1iNGT0E5AD03/k>`1RHV80BTY:034a<@0e=CD0EUIF01lO7`27QhL0=3@d03dm?@1=
CDd0@T9205aLG01[Jf/0D51@03Ti>@0U9BD0CTi>05QHF00X:2P0;Rh^09FEU@0`<300K6a/04I6AP0f
=SH08R8R04m?C`0a<C40;Rh^03dm?@0N7Qh0ATI603`l?00e=CD0Cdm?02TY:@0O7al0IVIV04U9B@0_
;bl0SXj>034a<@0j>SX0LW9b04Y:BP0Z:RX0BTY:03Ph>00^;Rh0DE5A065QH@0l?3`0;bl_03hn?P1Y
JFT0OWin05EEE@1RHV80F5QH03Hf=P1CDe<0F5QH09^KV`1[Jf/092@T03hn?P1`L700CDe=04a<C01o
Ogl0:RXZ038b<P1@D500OWin03Ti>@1aLG40K6a/04=3@`1eMGD0C4a<0451@@1[Jf/0MWIf03lo?`1R
HV80GEeM055AD@1QHF40LW9b07AdM00i>CT0D51@08F5Q@1KFe/0Ph>303hn?P1SHf<0I6AT07AdM01C
De<0FUYJ08F5Q@17AdL0Bd];059BDP1KFe/0DE5A05IFEP1bLW80E5AD06IVIP1gMgL0LG5a09fMW@14
A4@0OWin06EUI@2=SHd0Xj>S05mOG`1DE5@0Ng]k05aLG01lO7`0QXJ604Q8B01MGEd0OGem03lo?`0_
;bl0J6QX05EEE@14A4@0FUYJ03lo?`2:RXX0>SXj06i^KP1UIFD0IVIV061PH01TI6@0JVYZ07UiN@16
ATH0CTi>0:FUY@25QHD0MgMg06a/K027QhL0DU9B08n?S`2JVYX0MWIf0=KFeP2TY:@0Kfm_0861P@1V
IVH0HV9R0:f][@3<c<`0EeMG02HV9P1ZJVX0I6AT04];B`11@D40F5QH06][J`14A4@0G5aL06YZJP22
PX80UYJF04A4A01MGEd0WYjN04];B`1jNWX0BTY:04U9B@1RHV80IfMW08F5Q@0b<S8204m?C`1:05YJ
FP1>CTh0b/[:0:NWY`2@T900<C4a06MWI`1;Bd/0Jf][0820P01GEeL0E5AD07=cL`0n?Sh0SXj>0;>c
/`1OGel0E5AD07=cL`0f=SH0?Cdm09:BTP2n_[h0SXj>04Y:BP1]KFd0J6QX0451@@24Q8@0LG5a07Ui
N@1[Jf/0HV9R0=;BdP2n_[h0Kfm_04a<C01YJFT0?clo0451@@3?cll0UiNG04M7A`1EEED0N7Qh07em
O@0_;bl0QhN708V9R@1ADE40=cLg051@D015ADD0GemO09jNWP1ZJVX0B4Q806YZJP0W9bL0B4Q807=c
L`0l?3`0=cLg07inOP1ADE40[:b/05aLG00V9RH04Q8B01XJ6P1PH6008B4Q061PH02IVIT>ooooo`0I
05=CD`1>CTh0ATI604Q8B024Q8@0/[:b0<O7a`2UYJD0PX:2051@D00G5aL06a/K03@d=01>CTh0PH61
051@D00];Bd0_[jn0;Bd]01?Cdl0>c/k0;Fe]@2TY:@0G5aL020P800201<C4`804A4A00@09BDU02PX
:00G5aL05aLG0`0E5AD0100o?cl0>c/k03Hf=P0i>CT203dm?@0903<c<`0h>3P0BTY:07YjNP1mOGd0
c/k>0;:b/P0R8R804a<C00805ADE00`04a<C034a<@1;Bd/0>3Ph03dm?@0S8b<0:2PX028R8P0M7Ad0
92@T02/[:`0];Bd201dM7@1A01`L700G5aL05aLG04Y:BP0E5AD071`L04I6AP19BDT08b<S04I6AP23
Ph<0g]kN0>GUi@3AdM40M7Ad0451@@1fMWH0c<c<0=CDe02e]KD0GUiN09fMW@1nOWh0J6QX07QhN01j
NWX0RXZ:08:2PP2][Jd0^[Zj0<K6aP3De=@0_;bl0861P@14A4@0:BTY0861P@0e=CD0=SHf04i>CP2<
S8`0<S8b05eMG@0Q8B4071`L02TY:@0^;Rh0=3@d03dm?@1<C4`0RHV909jNWP3GemL0@41002LW9`0Q
8B40:BTY01dM7@0P8200Ogmo08b<S018B4P0:2PX07IfMP1<C4`0FEUI06QXJ00W9bL0<30`03lo?`21
PH40EEEE06QXJ02c/k<0VIVI03Ph>00J6QX06a/K01XJ6P0M7Ad0>c/k0080:BTY00X0<S8b03Hf=P1<
C4`0Vi^K0<71`@2BTY80Cdm?04M7A`0P82006QXJ0P0K6a/01@0i>CT0_[jn09ZJVP1NGUh0=cLg0080
71`L05D07AdM02<S8`1KFe/0RHV906IVIP0i>CT0;Bd]030`<00T92@0ATI607inOP22PX80GUiN05UI
F@1lO7`0MgMg06MWI`2][Jd0ADE501hN7P1CDe<0Fe]K01LG5`0I6AT09RHV028R8P0L71`0AdM7028R
8P0Y:BT0ATI607EeM@1oOgl0@41003Lg=`0^;Rh0;2`/04M7A`1KFe/0HF5Q0;Bd]00];Bd08R8R03@d
=02o_kl0ZZZZ07moO`0`<3008R8R0492@P1CDe<0G5aL06EUI@0I6AT05aLG01hN7P0a<C40RHV909:B
TP0j>SX0?Shn05ADE00g=cL0<S8b06ATI018B4P0JVYZ0410@00M7Ad08b<S020P800g=cL09BDU01TI
6@0M7Ad0;bl_05MGE`20P800<C4a020P800O7al0;bl_04E5A@0E5AD04Q8B00804a<C01H08R8R06IV
IP1LG5`0B4Q809nOW`24Q8@0;bl_06UYJ@1oOgl07alO024Q8@0Y:BT0=3@d03`l?01MGEd0VIVI03Ph
>01[Jf/0TI6A04=3@`0e=CD07alO4?ooool000kooooo0;d0LG5a08F5Q@1XJ6P0F5QH08N7Q`0n?Sh0
ADE508R8R00n?Sh0LG5a02/[:`0c<c<0Bd];03/k>`0l?3`08B4Q04Y:BP1<C4`0<S8b03Hf=P0I6AT0
NGUi04=3@`11@D40GEeM03@d=019BDT0JVYZ04Y:BP10@400De=C051@D01ADE40<C4a02PX:022PX80
=cLg04a<C00l?3`0;2`/03lo?`1HF5P08R8R02/[:`0j>SX0=SHf04Y:BP0a<C40;Bd]04i>CP1:BTX0
9BDU06i^KP1RHV80;Rh^09NGU`1MGEd0;Bd]06UYJ@1HF5P0<S8b05MGE`1ADE40;Rh^030`<01MGEd0
>3Ph034a<@19BDT0JVYZ07EeM@0/;2`0A4A405iNGP0n?Sh0DU9B0410@02XZ:P0MGEe03De=@1<C4`0
S8b<04A4A01?Cdl0L71`02h^;P0e=CD0E5AD07alO00f=SH0P82008^;R`0k>c/0N7Qh04];B`19BDT0
QXJ608>3P`10@400Jf][05EEE@1aLG40KFe]08j>SP1UIFD0@T9206a/K01YJFT0>SXj07YjNP1fMWH0
UYJF07moO`22PX80<S8b08J6QP20P800ADE504i>CP1ADE40FUYJ069RHP1OGel0Kfm_04Y:BP1_Kfl0
NWYj06YZJP2MWId0>c/k08^;R`1OGel0UIFE0;2`/01ZJVX0OGem08b<S01MGEd0R8R80820P00a<C40
MWIf092@T01<C4`09BDU079bLP1DE5@0NWYj06a/K00j>SX0Rh^;0410@01eMGD0KVi^071`L01_Kfl0
M7Ad071`L01bLW80Bd];04e=C@2JVYX0Ng]k06m_K`1gMgL0Ti>C05=CD`29RHT0GemO04A4A030`<00
WinO06i^KP2:RXX0JVYZ05eMG@29RHT0Shn?04i>CP0W9bL0JFUY0451@@1>CTh0?clo04a<C020P800
E5AD06][J`1bLW800P29RHT0E`15ADD0:BTY065QH@16ATH0MgMg03dm?@0Y:BT0IfMW0:j^[P2>SXh0
<S8b05YJFP1;Bd/0ADE503`l?03>c/h0ZJVY08R8R00e=CD0PH61051@D02:RXX0^[Zj06EUI@1MGEd0
KVi^02`/;02;Rh/0[:b/065QH@1IFET0KVi^06][J`1>CTh0E5AD06][J`1_Kfl0ATI609JFUP0];Bd0
=CDe09^KV`1XJ6P0Lg=c034a<@0X:2P0Ph>308^;R`1OGel0BDU906EUI@0U9BD04a<C09VIV@2BTY80
@D5105IFEP1dM7@0;2`/01lO7`1kNg/0RXZ:05UIF@0i>CT0GUiN02@T900d=3@0@d=303hn?P1EEED0
OWin02`/;01ADE40D51@030`<01UIFD0YjNW06ATI01ZJVX0D51@02/[:`0A4A405QHF03/k>`0O7al0
PX:20:6QX@0>ooooo`0c05eMG@1BDU80C4a<04e=C@1[Jf/0alO70=;BdP3Cdm<0`l?3028R8P0I6AT0
;Bd]03@d=01NGUh0QXJ604e=C@1YJFT0J6QX0:2PX029RHT0?3`l0861P@2<S8`0MWIf05UIF@0N7Qh0
6a/K02<S8`0M7Ad0=SHf03De=@0N7Qh06QXJ01DE5@0C4a<05ADE02/[:`0c<c<0:RXZ02PX:00S8b<0
820P02@T900b<S80;Rh^07emO@30`<00g]kN0;Bd]01RHV807QhN00<05ADE00`0?clo04M7A`0e=CD0
6ATI01DE5@0G5aL08B4Q01dM7@0N7Qh0;2`/02h^;P0J6QX201`L700Y01HF5P0H61P0?3`l01LG5`0C
4a<05ADE03/k>`2?Shl0VIVI0;Fe]@3?cll0f=SH0:b/[022PX80=cLg02d];@0f=SH0IfMW051@D014
A4@0BDU902TY:@0M7Ad071`L02HV9P26QXH0OGem05eMG@2ZZZX0hn?S0>CTi03@d=00alO709JFUP2;
Rh/0O7al03lo?`0j>SX0L71`05=CD`0K6a/00P19BDT0;`0O7al06a/K01XJ6P0J6QX09BDU02d];@1F
EUH0?3`l08b<S00l?3`08b<S02@T900S8b<0;Bd]0492@P21PH40KVi^04E5A@0Y:BT0OWin03lo?`18
B4P0HF5Q02TY:@0^;Rh0;bl_04a<C02EUID0O7al0820P034a<@0D51@01lO7`0L71`07QhN01`L700O
7al0>c/k04E5A@0j>SX0>c/k051@D02g]kL0_Kfm0<g=c@29RHT0=SHf00807QhN0P0O7al01P1[Jf/0
Y:BT055AD@0i>CT06a/K01XJ6P807AdM06`07QhN0492@P0m?Cd0L71`06m_K`1EEED0T92@07moO`1]
KFd0@D510410@01OGel0/;2`06ATI00O7al0GemO05QHF02TY:@08b<S04A4A01PH6005QHF01TI6@0m
?Cd05ADE02HV9P0e=CD07AdM020P800d=3@0S8b<05IFEP0[:b/07alO01/K6`0F5QH09RHV05MGE`0e
=CD0/K6a05EEE@0M7Ad0De=C09>CT`0h>3P0IfMW02DU9@0Y:BT0<c<c04=3@`1IFET0Hf=S01TI6@0E
5AD061PH028R8P2CTi<0Ng]k07emO@1gMgL0WYjN0;2`/01lO7`0;Rh^01HF5P15ADD0<30`01hN7P0U
9BD08b<S02PX:00_;bl05aLG01/K6`0V9RH0F5QH069RHP0Z:RX08b<S028R8P0N7Qh0=CDe030`<00E
5AD04a<C01DE5@10@400Bd];08f=S@25QHD0VIVI06][J`10@400H61P07EeM@0M7Ad07alO01hN7P0/
;2`0?Shn03lo?`1dM7@0=CDe055AD@1^KVh0;bl_051@D01JFUX@ooooo`003_ooool09@20P800Q8B4
06][J`1XJ6P0WYjN04I6AP1HF5P0TI6A03/k>`2CTi<0ADE504Q8B01ZJVX0EEEE04e=C@1WIfL0Fe]K
05YJFP0j>SX0Jf][02h^;P2@T900C4a<0451@@1aLG40AdM705MGE`1XJ6P0FEUI04=3@`1NGUh0D51@
04m?C`0c<c<0;2`/07inOP11@D400P1>CTh0200b<S80B4Q8065QH@0U9BD0:b/[02XZ:P0g=cL0De=C
0P0c<c<0B01EEED0De=C02/[:`1bLW80Fe]K02d];@2^[Zh0GEeM03De=@1`L700D51@02@T901TI6@0
Fe]K03Xj>P0g=cL0JFUY0410@01<C4`0Lg=c065QH@1[Jf/0?Shn04Y:BP1FEUH0=3@d055AD@16ATH0
I6AT05iNGP0m?Cd0DE5A08>3P`11@D40E5AD07UiN@0];Bd0=CDe051@D01bLW80B4Q807]kN`1hN7P0
=CDe08:2PP18B4P0?Shn075aL@1cLg<0=cLg06a/K013@d<0DU9B061PH022PX80@d=302d];@1DE5@0
KVi^04=3@`1VIVH0?3`l06][J`1eMGD0I6AT04=3@`15ADD0Lg=c0410@01=CDd0F5QH03dm?@80HF5Q
09/0J6QX03Lg=`1hN7P0OGem05aLG02LW9`0=cLg07]kN`1QHF40[:b/0:>SX`1SHf<0@T9207UiN@1M
GEd0TY:B05iNGP0_;bl0CTi>07=cL`0f=SH08b<S05UIF@0`<300IfMW07MgM`0k>c/0OGem030`<01n
OWh0J6QX07=cL`1SHf<0JVYZ07YjNP24Q8@0CTi>04];B`2RXZ80LW9b092@T02_[jl0Vi^K04a<C02<
S8`0TI6A06][J`3Jf]X0S8b<07AdM02MWId0M7Ad07UiN@1dM7@0L71`04a<C00f=SH0JFUY03Hf=P1G
EeL0>CTi04];B`1jNWX0Lg=c08Z:RP25QHD0KVi^096AT@1LG5`0Bd];07alO00j>SX0H61P04];B`0e
=CD0;bl_0410@01QHF40@D5106=SH`1GEeL0<c<c03dm?@3@d=00c<c<07EeM@0d=3@0QXJ601DE5@1m
OGd0W9bL05aLG01aLG40L71`02<S8`1NGUh0OGem03`l?01EEED0IVIV07QhN01ADE40FUYJ08F5Q@1^
KVh0@41006ATI00h>3P09bLW06QXJ01BDU80GemO03lo?`1JFUX0Shn?05YJFP1>CTh0?3`l06e]K@12
@T809BDU0:b/[027QhL0?3`l05IFEP21PH40C4a<02LW9`0n?Sh0LG5a04];B`1ADE40R8R804m?C`0g
=cL0@T92038b<P10@400NWYj02`/;01OGel0MWIf02l_;`10@400Ng]k06e]K@2>SXh0@41003Ph>00Z
:RX07QhN04Y:BP0L71`0Bd];06m_K`0>ooooo`0j08J6QP2DU9@0Y:BT09>CT`2h^;P0d];B0;jn_P2g
]kL0MWIf079bLP19BDT0=SHf07alO02;Rh/0QXJ6079bLP0];Bd05QHF01/K6`1HF5P0UIFE07YjNP2Y
ZJT0][Jf0=_Kf`34a<@0Ph>305=CD`1;Bd/0JFUY07MgM`1;Bd/08B4Q01`L700I6AT05ADE020P800a
<C40:2PX020P800O7al0;Rh^03`l?01IFET0Vi^K0<?3``2FUYH0WIfM06][J`2>SXh0Kfm_03/k>`0S
8b<05ADE04e=C@1?Cdl0?Cdm01LG5`805ADE00L05aLG01lO7`0Z:RX09RHV03Hf=P0O7al06a/K0080
71`L01T05aLG02TY:@0I6AT04a<C018B4P0H61P0U9BD05UIF@2XZ:P0bl_;0=OGe`2LW9`0=CDe03Xj
>P0d=3@0;2`/02/[:`0X:2P0<30`03Ti>@14A4@07alO02/[:`17AdL0De=C0080EUIF04T0UiNG0=SH
f03Cdm<0Fe]K05EEE@1@D500KVi^0:2PX01mOGd0ZZZZ09ZJVP0R8R806ATI028R8P1jNWX0D51@02<S
8`0G5aL05ADE01dM7@0U9BD0AdM703@d=014A4@0J6QX020P800N7Qh0820P02l_;`11@D40GEeM06i^
KP19BDT0=3@d07alO016ATH0=SHf02DU9@0T92@09RHV02DU9@0`<300PX:204i>CP0a<C40X:2P09NG
U`0g=cL0:b/[01dM7@0L71`06ATI03<c<`1bLW80Bd];04A4A021PH40ZZZZ09JFUP1aLG40X:2P07em
O@1GEeL0>c/k0451@@0f=SH0NWYj08J6QP0j>SX08B4Q01lO7`0I6AT06QXJ008071`L01l0DE5A03/k
>`0/;2`0FUYJ0;bl_01_Kfl0DE5A05YJFP1ZJVX0Rh^;09>CT`2l_;`0IVIV02/[:`1GEeL0BTY:071`
L018B4P0>c/k05YJFP0G5aL07AdM0492@P0E5AD0:RXZ02d];@0L71`09bLW0451@@1SHf<0820P00@0
5ADE02406a/K05IFEP0R8R80K6a/09^KV`1bLW80Rh^;03`l?00e=CD0MWIf03De=@0];Bd0=SHf03/k
>`1NGUh0LW9b028R8P0E5AD06ATI03dm?@2_[jl0QHF50:BTY02QXJ40_Kfm0:f][@1IFET05ADE01HF
5P15ADD092@T01hN7P0Q8B400P0O7al09@0i>CT071`L01XJ6P0W9bL0D51@0:RXZ01LG5`0;Bd]01lO
7`0S8b<07alO03De=@0c<c<051@D03Xj>P1BDU80C4a<09NGU`2IVIT0[jn_079bLP1IFET0S8b<0:j^
[P0k>c/0820P024Q8@0X:2P0;bl_0451@@20P800IfMW06m_K`1LG5`0:BTY02h^;P0l?3`04?ooool0
00kooooo02H0Ng]k06i^KP1UIFD0B4Q807QhN012@T80EEEE08:2PP11@D40VYZJ03hn?P1<C4`0=cLg
02/[:`1XJ6P0Cdm?04a<C00_;bl0EEEE069RHP0`<300U9BD04a<C01nOWh0L71`0451@@16ATH0IVIV
05]KF`1BDU80J6QX055AD@17AdL09BDU05EEE@2TY:@0Bd];05IFEP80?clo01/0BDU905IFEP0^;Rh0
GUiN04=3@`0e=CD0E5AD04A4A00c<c<0L71`04m?C`0U9BD0I6AT01lO7`10@400XJ6Q05YJFP0i>CT0
MGEe04i>CP0e=CD0Ph>3061PH00j>SX0H61P05]KF`0_;bl00P1=CDd0H01JFUX0GEeM0492@P1>CTh0
AdM7034a<@1DE5@0EUIF09NGU`1IFET0820P05=CD`20P800@T9209bLW02ATI40@D5103Lg=`1ADE40
GEeM03dm?@1ADE40I6AT03@d=02BTY80GUiN0451@@1bLW80HV9R03Ph>01]KFd0JVYZ06][J`1JFUX0
MWIf05UIF@0/;2`0Bd];05UIF@0c<c<0MWIf05IFEP29RHT0KFe]06QXJ01LG5`0FUYJ075aL@0h>3P0
B4Q805eMG@1YJFT0De=C05MGE`1IFET0Fe]K06EUI@1iNGT0KFe]07YjNP19BDT0OGem05aLG02OWil0
XZ:R065QH@1RHV80JFUY06i^KP2g]kL0C4a<034a<@1RHV80N7Qh03Ti>@0/;2`0HV9R0451@@14A4@0
DE5A03Ph>02FUYH0ADE50:6QX@2WYjL0NWYj05IFEP1XJ6P0YjNW0;Jf]P1kNg/0DE5A0:BTY017AdL0
DE5A07YjNP80?Shn00d0U9BD08^;R`1ZJVX0f=SH08J6QP0o?cl0W9bL0;Fe]@2OWil0OWin07UiN@0h
>3P0ADE50080MgMg00d0EUIF03De=@1GEeL0RHV906i^KP2?Shl0I6AT08^;R`2f][H0D51@04e=C@1V
IVH0@T9200<0EUIF0500BTY:04E5A@1KFe/0?clo065QH@1OGel0=CDe051@D03=cLd0alO709>CT`0d
=3@0Ogmo03dm?@1IFET0I6AT04=3@`1kNg/0MGEe03Ph>01ZJVX0Ng]k03@d=01ZJVX0TY:B08V9R@1L
G5`0F5QH07moO`1]KFd0HF5Q07moO`1KFe/0@T9205ADE01:BTX0FUYJ020P801kNg/0WYjN04e=C@1;
Bd/0FUYJ08>3P`1<C4`0:BTY0:6QX@1[Jf/0CDe=08b<S02JVYX0C4a<03@d=018B4P0IfMW01TI6@0Y
:BT0HF5Q059BDP0m?Cd0B4Q804];B`10@400B4Q80492@P2:RXX0PX:203Ti>@1FEUH0W9bL065QH@2>
SXh0Bd];01/K6`0`<300BDU9069RHP0M7Ad0Bd];0861P@kooooo02P0Bd];055AD@15ADD0JFUY0=CD
e02d];@0WYjN09RHV01>CTh0JVYZ0;:b/P2i^KT0R8R805UIF@1<C4`0>SXj024Q8@0I6AT05ADE01TI
6@17AdL0_[jn0;no_`38b<P0c/k>08:2PP1jNWX0IfMW061PH01UIFD0J6QX06=SH`1GEeL0C4a<04e=
C@0h>3P09RHV0410@013@d<0?Cdm0`0f=SH03`1FEUH0Vi^K05iNGP2FUYH0CTi>02`/;00];Bd0EEEE
075aL@1ZJVX0I6AT09fMW@1VIVH0De=C01HF5P0201DE5@2>01PH600I6AT08b<S02PX:00_;bl07QhN
01/K6`0J6QX07AdM01DE5@0J6QX092@T01DE5@0D51@0<30`05]KF`11@D40alO70;6a/@39bLT0dM7A
08j>SP1EEED0:BTY02l_;`0c<c<0=3@d02HV9P0Y:BT0BTY:07IfMP1ADE40>c/k04i>CP1GEeL0De=C
07QhN03Hf=P0/K6a02h^;P16ATH0L71`07=cL`1SHf<0U9BD0<K6aP2/[:`0>SXj01PH600E5AD0Kfm_
07=cL`0_;bl0820P01PH600L71`0820P05EEE@1ADE40@D5107YjNP0k>c/08B4Q028R8P0a<C40G5aL
06MWI`1SHf<0>c/k059BDP23Ph<0E5AD01lO7`0J6QX06ATI02HV9P0T92@0;bl_09BDU01/K6`0IVIV
0;Zj^P31`L40HF5Q06MWI`1RHV80E5AD04M7A`1IFET0KVi^08f=S@2QXJ40_kno07emO@2g]kL0VIVI
06e]K@1<C4`0Fe]K06e]K@1lO7`0NWYj0861P@2SXj<0E5AD04A4A00/;2`0;Rh^02@T900M7Ad09BDU
05iNGP0c<c<0:b/[069RHP2i^KT0WIfM06ATI01BDU80@d=303Xj>P0U9BD0BTY:06IVIP2:RXX0Lg=c
059BDP13@d<0IfMW04U9B@1SHf<0:BTY059BDP1YJFT05ADE02h^;P0/;2`07QhN02TY:@1HF5P0C4a<
01LG5`<05ADE02D05aLG01PH601@D5008B4Q05]KF`2j^[X0WYjN04A4A00j>SX0EUIF0<O7a`2b/[80
GemO03/k>`0i>CT0HF5Q09nOW`1mOGd09bLW02HV9P2QXJ40[Jf]0:6QX@2FUYH0YZJV051@D015ADD0
Fe]K04E5A@0F5QH0A4A401dM7@0L71`06a/K01dM7@0N7Qh0=cLg00807AdM00L0;Rh^05=CD`1RHV80
;bl_0451@@0U9BD09RHV00809BDU01T0?Shn03`l?01@D500B4Q805UIF@1RHV80Bd];08F5Q@1fMWH0
_kno08N7Q`0i>CT0I6AT05eMG@0_;bl09bLW02XZ:P0_;bl0Ogmo09jNWP24Q8@0MgMg02h^;P0V9RH0
8B4Q013ooooo000>ooooo`0Q079bLP1TI6@0LW9b05MGE`1eMGD0>SXj05MGE`1/K6`0;2`/0861P@0n
?Sh0CDe=06IVIP19BDT0ATI605UIF@1JFUX0BTY:02l_;`16ATH0:b/[07]kN`0h>3P0BDU904i>CP0i
>CT0Bd];07alO016ATH0?clo03De=@1BDU80Fe]K0080;Rh^05<0LG5a03Ph>01]KFd0CTi>03Ti>@0a
<C40DE5A02XZ:P0h>3P0<30`03<c<`1KFe/0;2`/01dM7@1ADE40<S8b01hN7P1lO7`0B4Q802DU9@2R
XZ80BTY:034a<@1YJFT0?3`l02TY:@1jNWX0?Cdm02d];@1/K6`0De=C02h^;P1JFUX0<S8b04m?C`1Z
JVX0ATI604=3@`0m?Cd0:RXZ05iNGP1QHF40YZJV06EUI@0W9bL0I6AT08b<S012@T80Fe]K071`L00`
<300ATI606UYJ@1oOgl0DU9B055AD@1UIFD0;Rh^07inOP1CDe<0?3`l07AdM01GEeL0<c<c07inOP1I
FET0IFEU05IFEP27QhL0IVIV03Lg=`1LG5`0CDe=034a<@1hN7P0EEEE05]KF`1ZJVX0Jf][04Y:BP1P
H600?Cdm03`l?00206EUI@2905IFEP1VIVH0EEEE06e]K@1JFUX0PH6106m_K`1=CDd0JFUY05EEE@1h
N7P0AdM708Z:RP2LW9`0JFUY07=cL`1ZJVX0CTi>0:6QX@1EEED0=cLg0861P@24Q8@0>CTi03/k>`1h
N7P0=cLg06MWI`1@D500BDU90861P@0_;bl0IfMW05mOG`1>CTh0G5aL04];B`1mOGd0M7Ad03Xj>P1@
D500ZZZZ05eMG@1bLW80Shn?06IVIP10@400O7al07YjNP1cLg<0dm?C09bLW01bLW80/K6a09ZJVP1o
Ogl0MgMg071`L00/;2`0?Shn07moO`1XJ6P0GemO03Ti>@1@D500I6AT03lo?`2:RXX0K6a/07inOP2L
W9`0BDU904];B`1XJ6P0:2PX03hn?P0h>3P0J6QX05iNGP1DE5@0EUIF03Hf=P1`L700PX:205EEE@1:
BTX0`L7109FEU@26QXH0?Shn09fMW@1SHf<0IVIV065QH@0i>CT0RHV906YZJP0i>CT0R8R80:NWY`1?
Cdl0De=C06QXJ01PH600@d=306UYJ@2:RXX0GEeM03Ph>02ATI40E5AD04i>CP1IFET0>SXj06ATI010
@400NGUi0<K6aP1`L700=SHf0410@01TI6@0>CTi030`<02PX:00N7Qh03Ph>01:BTX0Hf=S03Hf=P0b
<S80GEeM061PH00m?Cd0@D5107alO010@4000P19BDT04`13@d<0FEUI06UYJ@0T92@0Fe]K06MWI`0j
>SX0N7Qh09fMW@1SHf<0ZZZZ04E5A@0[:b/05QHF01lO7`1@D5008B4Q05QHF025QHD03_ooool0:@0f
=SH0IfMW07EeM@1XJ6P0a<C40;Rh^02ZZZX0[Jf]08B4Q01gMgL0Q8B406i^KP10@4009RHV01XJ6P0G
5aL061PH01dM7@0/;2`0>3Ph06EUI@2YZJT0[:b/0:j^[P2o_kl0Lg=c04M7A`0c<c<0:RXZ04E5A@17
AdL0?3`l03Hf=P0_;bl0;Bd]0451@@1;Bd/0EEEE03<c<`0G5aL05QHF00805ADE0100:b/[04E5A@1J
FUX08B4Q03Ti>@1SHf<0<c<c028R8P0V9RH0;Bd]03dm?@1LG5`0R8R807QhN00S8b<06ATI0`0E5AD0
300Q8B40J6QX07UiN@14A4@0;bl_03De=@0U9BD06ATI01LG5`0i>CT05QHF03Xj>P80@T9202T0S8b<
0;no_`1/K6`0Vi^K0451@@0R8R80=SHf04Y:BP1@D500DU9B05ADE01@D500FUYJ0861P@2?Shl0MgMg
03@d=00R8R805aLG01HF5P0O7al0:b/[055AD@1eMGD08R8R05mOG`1nOWh0T92@069RHP1TI6@09RHV
03Ph>00n?Sh0:b/[02l_;`2:RXX0MWIf03Xj>P0g=cL0@D5104Q8B00208b<S01j06YZJP1PH600KVi^
04e=C@12@T80Lg=c09^KV`2[Zj/0IVIV05YJFP2FUYH0VYZJ08f=S@15ADD0>CTi03Lg=`0f=SH0B4Q8
08Z:RP2`/;00[jn_0<_;b`2k^k/0AdM702h^;P0_;bl0>CTi03hn?P1FEUH0T92@0;no_`39bLT0_kno
0;jn_P38b<P0dM7A069RHP0m?Cd0A4A40492@P16ATH0CDe=06][J`1nOWh0Vi^K0492@P1CDe<0@d=3
04U9B@16ATH0EUIF07IfMP2FUYH0HF5Q079bLP2MWId0Q8B405UIF@0Y:BT0820P01lO7`0N7Qh07AdM
034a<@1;Bd/0Ogmo0:>SX`29RHT0B4Q806m_K`2ATI40_;bl0::RXP2PX:00YZJV07AdM02IVIT0JVYZ
07alO01iNGT0MGEe038b<P0E5AD051@D01DE5@0G5aL061PH02<S8`1kNg/0<30`03Lg=`21PH40R8R8
06e]K@1iNGT0/;2`06ATI01ADE40Rh^;0:j^[P2LW9`0/K6a096AT@1`L700VIVI0<K6aP3Ph>00XJ6Q
08J6QP2YZJT0LG5a03@d=00];Bd0=cLg04E5A@1JFUX0<c<c01/K6`0M7Ad071`L01lO7`0R8R8203Hf
=P0S02@T901EEED0K6a/02HV9P0D51@0:BTY0451@@0^;Rh09BDU02`/;00c<c<0IVIV03<c<`1KFe/0
PX:203hn?P1LG5`0IfMW06a/K01<C4`0@41004U9B@0Z:RX0:2PX04a<C01TI6@0JFUY06ATI01MGEd0
XZ:R07AdM02:RXX0LW9b07=cL`1`L7004?ooool000kooooo03h0M7Ad07alO01eMGD0F5QH08>3P`0c
<c<0ADE506i^KP0i>CT0Hf=S02d];@1HF5P0HF5Q04Q8B01;Bd/0;Rh^059BDP1;Bd/0A4A404E5A@0I
6AT0Ogmo04=3@`18B4P0=CDe01lO7`0V9RH0L71`05YJFP1=CDd0Bd];04I6AP1KFe/0<S8b03Lg=`1^
KVh0>SXj06=SH`1RHV80?Shn03<c<`1:BTX0=3@d04=3@`0Y:BT0:b/[05mOG`1EEED0>3Ph059BDP0_
;bl0:BTY08^;R`1?Cdl0:2PX0:JVYP0n?Sh0CTi>07inOP1HF5P0@d=309jNWP8092@T05@0UIFE04=3
@`0i>CT0Q8B40410@01ADE40CTi>03De=@0o?cl0BTY:03<c<`1HF5P0?clo08:2PP1VIVH0=3@d05iN
GP2<S8`0B4Q808f=S@1nOWh0<C4a03`l?01FEUH0K6a/04Y:BP1/K6`0OGem03/k>`2:RXX0BDU904M7
A`1^KVh0GemO02d];@1dM7@0?clo05iNGP1YJFT0PH6105QHF01>CTh0L71`05eMG@0e=CD0L71`06=S
H`1[Jf/0PX:206YZJP14A4@0B4Q804i>CP0i>CT0DE5A04i>CP0g=cL0Bd];051@D01PH600:RXZ05]K
F`1`L700AdM705EEE@15ADD0NWYj05iNGP2k^k/0Vi^K04A4A01TI6@0PH6105IFEP2WYjL0AdM703/k
>`1ZJVX0N7Qh03Ti>@0X:2P0DU9B01hN7P0l?3`2055AD@0808^;R`0f=SH0X:2P07QhN01[Jf/0AdM7
02`/;01SHf<204a<C01f051@D02VYZH0H61P07emO@2NWYh0B4Q803lo?`2<S8`0N7Qh06ATI02i^KT0
Ph>306ATI03<c<`0f][J07alO01PH600I6AT02<S8`1XJ6P0Rh^;04A4A01^KVh0@d=3069RHP14A4@0
;2`/07inOP1NGUh0MgMg09fMW@0e=CD0CDe=075aL@16ATH0>SXj01hN7P1ADE40EUIF075aL@1YJFT0
?Cdm06QXJ01/K6`0@41004M7A`3@d=00UIFE05eMG@0d=3@0IFEU04=3@`1aLG40OGem051@D02<S8`0
IVIV03Ti>@1cLg<0UYJF06=SH`1aLG40SXj>08N7Q`1IFET0EUIF08f=S@1EEED0EUIF09>CT`13@d<0
@D5105ADE018B4P0Rh^;04U9B@23Ph<0fm_K061PH011@D40Q8B408j>SP0l?3`09RHV08f=S@1mOGd0
De=C08n?S`2RXZ80LW9b04I6AP1?Cdl0Fe]K02LW9`1JFUX0RHV9059BDP1ADE40C4a<03/k>`18B4P0
H61P02LW9`1RHV80JVYZ02h^;P1RHV80NWYj07UiN@2=SHd0ADE503Hf=P1<C4`0Fe]K06][J`0O7al0
BTY:07moO`kooooo0500G5aL04I6AP0c<c<0:BTY0:BTY01TI6@0IFEU0:FUY@2XZ:P0XZ:R0:f][@1k
Ng/0AdM702<S8`0G5aL04a<C01HF5P0K6a/07alO01/K6`0N7Qh0MGEe05EEE@1[Jf/0TY:B0;>c/`2B
TY80Fe]K04M7A`0I6AT03Ph>018B4P0R8R80A4A403Hf=P0U9BD0:b/[06i^KP1BDU80<S8b01LG5`0H
61P05ADE03Lg=`1TI6@06ATI01lO7`0];Bd0G5aL05mOG`1?Cdl0?Shn034a<@0U9BD071`L034a<@2@
T900MgMg071`L00g=cL06QXJ01LG5`0h>3P0ZZZZ08R8R01VIVH0CDe=03/k>`0g=cL0<30`0410@01g
MgL0PH610:j^[P2`/;00/k>c0=cLg02j^[X0R8R805MGE`804a<C00/04Q8B01DE5@0T92@0@D5103/k
>`2<S8`0RHV903Ti>@11@D40FUYJ02`/;00201DE5@0W01HF5P0H61P06QXJ02XZ:P1PH600EUIF05eM
G@0O7al0EUIF03/k>`0];Bd05ADE01PH600I6AT0:RXZ03hn?P12@T80e=CD0;Jf]P2^[Zh0`<300;Vi
^@2o_kl0_Kfm0<C4a036a/H0b/[:079bLP24Q8@0Vi^K06a/K01^KVh0U9BD07moO`2d];@0WinO04m?
C`0c<c<0:RXZ00809bLW01@0=SHf07=cL`2g]kL0e=CD0<g=c@2GUiL0@T9201dM7@0Q8B40820P0410
@01hN7P0MWIf0<g=c@3IfMT0c/k>08R8R01TI6@0RXZ:02DU9@8071`L02/08B4Q01`L700O7al0:BTY
07moO`2k^k/0S8b<038b<P0k>c/09bLW02DU9@0L71`09BDU07YjNP34a<@0^;Rh0<c<c025QHD0GemO
03/k>`0Y:BT09bLW02@T900[:b/08B4Q04E5A@12@T80FUYJ09JFUP1KFe/0OWin0<O7a`2RXZ80Jf][
09BDU02>SXh0WinO0=GEe@3Bd]80`L710<30`01PH600:BTY00804a<C02`05ADE01HF5P0O7al0L71`
06i^KP24Q8@0:b/[07EeM@2WYjL0Q8B409bLW02TY:@0ADE503lo?`0f=SH0TI6A0=[JfP2VYZH0E5AD
02XZ:P0o?cl0HF5Q0=3@d02b/[80_[jn05eMG@1fMWH0;bl_02DU9@0`<300;bl_03dm?@1UIFD06QXJ
01dM7@0N7Qh07alO01dM7@0d=3@0O7al09NGU`35aLD0_;bl03hn?P805ADE01/061PH04Y:BP0a<C40
8B4Q02DU9@1LG5`0:RXZ05QHF02LW9`0Jf][07emO@1cLg<0@T9201<C4`0B4Q808R8R06][J`0o?cl0
<c<c07IfMP1]KFd0BDU905EEE@2SXj<0XZ:R04Y:BP31`L400P2=SHd@ooooo`003_ooool0_`1nOWh0
L71`079bLP1CDe<0OWin03dm?@1ADE40SXj>03Ph>02<S8`0?Cdm04m?C`1fMWH0H61P05EEE@1^KVh0
IfMW051@D00l?3`0FEUI038b<P2EUID0C4a<05YJFP1UIFD0:BTY03@d=01/K6`0>SXj059BDP17AdL0
Cdm?0451@@0G5aL0D51@07QhN013@d<0I6AT04];B`14A4@0DU9B05eMG@0e=CD0De=C0451@@0h>3P0
BDU902`/;00T92@0EEEE024Q8@0M7Ad0Hf=S03@d=00V9RH0XZ:R02HV9P0S8b<0CDe=03@d=01<C4`0
MgMg03/k>`0o?cl0V9RH02l_;`0];Bd0Ng]k03/k>`1`L700E5AD02h^;P1;Bd/0C4a<04Q8B01SHf<0
EEEE09bLW01DE5@0:BTY06m_K`2MWId0?Shn05UIF@1ZJVX0=CDe04E5A@1ZJVX0H61P02l_;`1ADE40
PX:203Ph>02KVi/0NGUi055AD@22PX80CDe=03lo?`2HV9P0HF5Q079bLP1EEED0WinO059BDP0g=cL0
I6AT05EEE@0o?cl0O7al04];B`0i>CT0NWYj07moO`1@D500EUIF02/[:`17AdL0HV9R061PH01ADE40
GUiN04e=C@1`L700E5AD06=SH`1IFET0A4A407YjNP1CDe<0KFe]04i>CP2DU9@0V9RH05]KF`1aLG40
EEEE06UYJ@2^[Zh0@T9205mOG`27QhL0NGUi03Ph>00Y:BT0F5QH02`/;01;Bd/0E5AD069RHP2;Rh/0
:BTY06m_K`1eMGD0IVIV05ADE01?Cdl0Shn?051@D010@400FEUI0;:b/P1IFET0QhN709VIV@0/;2`0
<30`06e]K@1dM7@0LG5a0<;2`P1eMGD0@41008n?S`2ZZZX0QXJ606MWI`1NGUh09bLW03lo?`25QHD0
KFe]04m?C`0l?3`0LG5a05QHF00i>CT0MGEe04I6AP2NWYh0`l?30080CDe=03/0HF5Q020P8010@400
9BDU06QXJ01BDU80BTY:0451@@10@400Ti>C08F5Q@0m?Cd0GUiN0=3@d031`L40KFe]03hn?P1UIFD0
:2PX059BDP1VIVH0@41009jNWP1eMGD0?clo08:2PP2FUYH09BDU05UIF@1cLg<0OGem05MGE`1MGEd0
Ogmo04E5A@17AdL0R8R805=CD`1?Cdl0?Cdm02XZ:P1KFe/0E5AD07QhN03IfMT0EUIF04E5A@13@d<0
GemO030`<00W9bL0Ph>30410@00c<c<0C4a<06][J`1CDe<0?Shn059BDP0207emO@0H05EEE@2=SHd0
K6a/05UIF@1?Cdl0DE5A05YJFP1aLG409bLW06i^KP1/K6`0;2`/07MgM`22PX80KFe]07YjNP15ADD0
9BDU01TI6@0S8b<0C4a<0451@@2@T900X:2P3_ooool04`17AdL0?Cdm02l_;`0/;2`0YZJV05QHF01B
DU80KVi^07moO`2QXJ40/k>c06i^KP10@400AdM702HV9P0@410051@D01DE5@0C4a<00P0>3Ph03`1M
GEd0@D5105]KF`2QXJ40[jn_08f=S@1/K6`0@D5101DE5@0>3Ph0410@00h>3P0@4100:BTY055AD@02
03hn?P0]06i^KP22PX80BDU901dM7@0S8b<0GUiN04a<C00H61P0<c<c0451@@1UIFD0@41005=CD`14
A4@0DU9B0492@P0a<C408b<S0861P@1>CTh0LG5a08j>SP24Q8@0Ng]k05IFEP1LG5`0HF5Q07IfMP1Q
HF40O7al09VIV@2MWId0V9RH09fMW@1jNWX0I6AT06IVIP1oOgl0eMGE0=3@d039bLT0Ogmo01TI6@0E
5AD04Q8B00804a<C01H05aLG03lo?`1;Bd/0GUiN03Lg=`0c<c<0BDU9059BDP0O7al07AdM01`L700M
7Ad061PH02<S8`1_Kfl0OGem04a<C00I6AT0>CTi0451@@0_;bl05QHF0P0E5AD02P0G5aL08R8R03/k
>`2n_[h0I6AT04E5A@0k>c/0=3@d04Y:BP2<S8`20:FUY@0=0;fm_@3Bd]80QHF506i^KP2=SHd0JVYZ
09bLW038b<P0ZJVY079bLP1JFUX0@T9202d];@0202LW9`0D04M7A`2DU9@0c/k>09^KV`1IFET0T92@
09FEU@1[Jf/0IVIV08N7Q`1eMGD0JVYZ05eMG@26QXH0_Kfm09FEU@0X:2P07alO0410@01?Cdl201PH
600/01XJ6P0K6a/0820P03@d=02GUiL0IFEU06QXJ01nOWh0De=C038b<P0Z:RX08B4Q02`/;020P800
YZJV0;:b/P2YZJT0QHF506m_K`1=CDd0@d=304A4A00o?cl092@T02<S8`15ADD0;Bd]02`/;01gMgL0
RHV909NGU`2][Jd0QHF504Y:BP1]KFd0QHF5092@T02`/;00fMWI0=OGe`2GUiL0Hf=S024Q8@0I6AT2
01DE5@0/01`L701dM7@0H61P01LG5`1RHV80JFUY05=CD`1iNGT0Hf=S0<S8b01aLG4071`L01hN7P1E
EED0ZJVY09NGU`32`/80OWin04E5A@1:BTX0H61P0;bl_03Jf]X0PX:20410@00l?3`0CTi>02LW9`0Z
:RX0>CTi02PX:020P800<C4a01dM7@0L71`07AdM03<c<`1_Kfl0Xj>S09:BTP2;Rh/0QhN706IVIP0H
61P201DE5@0L01dM7@1EEED0<30`02h^;P1BDU80;2`/07alO02d];@0WYjN08:2PP2KVi/06ATI01@D
500B4Q805ADE01TI6@2][Jd0^KVi06e]K@2SXj<0FUYJ07=cL`2ATI40aLG506EUI@1JFUX0EUIF06AT
I13ooooo000>ooooo`1`06m_K`1/K6`0QXJ605IFEP1fMWH0?Shn05EEE@1WIfL0@T9209ZJVP11@D40
DE5A07moO`1OGel0Kfm_06][J`1TI6@0Cdm?03hn?P1JFUX09bLW07=cL`0_;bl0@d=3075aL@0d=3@0
DU9B06UYJ@18B4P0ADE504U9B@1NGUh0ATI601/K6`0e=CD0J6QX0492@P1iNGT0:b/[03Ph>00j>SX0
Fe]K02<S8`0h>3P0<C4a04=3@`1^KVh0=3@d01XJ6P1JFUX0?Cdm03<c<`21PH407alO03`l?02PX:00
@D5103dm?@1/K6`0DU9B05ADE01fMWH071`L05eMG@1lO7`0>c/k03Ti>@2ATI405ADE04E5A@16ATH0
;Rh^04=3@`0_;bl0=3@d05ADE01JFUX0X:2P051@D00Q8B40SXj>092@T016ATH0Kfm_07MgM`0U9BD0
E5AD08J6QP1hN7P0>SXj05YJFP1QHF4092@T06a/K01>CTh0AdM709^KV`1[Jf/0<C4a07moO`1PH600
Jf][06=SH`2EUID0HF5Q03/k>`1^KVh0C4a<02/[:`1/K6`0DU9B055AD@80MGEe06X0FUYJ059BDP1B
DU80;Bd]03hn?P1QHF40F5QH06MWI`1KFe/0J6QX05ADE01TI6@0F5QH02d];@20P800E5AD04i>CP0k
>c/0GEeM05iNGP1DE5@0Q8B405=CD`15ADD0Shn?03Xj>P1TI6@0^k^k06UYJ@1BDU80Hf=S06i^KP0]
;Bd0C4a<03lo?`1_Kfl0PX:204M7A`2?Shl0L71`0492@P1UIFD0B4Q808F5Q@1;Bd/05ADE03Lg=`2H
V9P0CDe=0:RXZ02LW9`0Fe]K03Ph>01IFET0LG5a07MgM`2_[jl0LG5a06IVIP2e]KD0/k>c0861P@1W
IfL0F5QH030`<00k>c/0N7Qh06m_K`1EEED0;2`/04=3@`1MGEd0?clo07MgM`1NGUh0_Kfm0;Bd]01?
Cdl0IfMW06m_K`0M7Ad0>c/k030`<01SHf<0DE5A05MGE`1?Cdl0;Rh^071`L02<S8`0CDe=04Y:BP2g
]kL0NGUi04i>CP1@D500M7Ad04E5A@1JFUX0Fe]K02TY:@1kNg/0K6a/03dm?@1kNg/0UIFE0P1LG5`0
>01`L700QXJ605YJFP1CDe<0Ph>3079bLP1LG5`0SHf=055AD@1EEED0ATI604A4A01OGel0BDU905aL
G03AdM40De=C061PH01EEED0JFUY034a<@0[:b/0O7al04Q8B015ADD0HF5Q06QXJ00i>CT0@T9206a/
K02HV9P0QHF5065QH@1eMGD0@D5108V9R@2XZ:P0FEUI05aLG01UIFD0?3`l09NGU`1ZJVX0<S8b08>3
P`1mOGd0NGUi0:JVYP1EEED0>3Ph01dM7@0/;2`0>CTi020P801YJFT0VYZJ3_ooool08026QXH0Hf=S
04E5A@0i>CT0WinO04M7A`1VIVH0@41007EeM@1PH600U9BD08F5Q@0W9bL06a/K020P800>3Ph03`l?
018B4P0@41003@d=00h>3P1MGEd0=SHf04E5A@1[Jf/0WYjN05UIF@1<C4`0@41001XJ6P0?3`l0410@
0P0>3Ph0400@41006ATI055AD@0m?Cd0BDU9069RHP2JVYX0NGUi07AdM02<S8`09BDU024Q8@0o?cl0
Jf][04Q8B00I6AT201DE5@0N01LG5`0Z:RX0D51@051@D025QHD0@T9204M7A`1bLW80WIfM07YjNP0i
>CT07alO01hN7P0/;2`0EeMG05UIF@14A4@0=SHf02h^;P1jNWX0Jf][0:BTY01VIVH0J6QX07alO037
alL0/k>c08Z:RP0U9BD05QHF100E5AD01P0O7al0M7Ad03Ph>02NWYh0L71`06ATI080RXZ:00`0C4a<
04U9B@1GEeL0EUIF06m_K`1oOgl0EeMG04i>CP0L71`08b<S04Q8B00j>SX301LG5`0a01HF5P0I6AT0
CTi>05IFEP19BDT0FEUI034a<@0K6a/071`L02XZ:P1FEUH0];Bd0:VYZ@2k^k/0@T9205]KF`16ATH0
9BDU05YJFP33`l<0]kNg04I6AP0i>CT0ADE505EEE@1<C4`0@T9206EUI@2n_[h0GUiN03@d=00n?Sh0
G5aL0:^[Z`3Jf]X0W9bL065QH@1BDU80G5aL051@D01iNGT0[jn_08b<S01:BTX07alO02DU9@1TI6@0
8b<S01XJ6P0201`L701H038b<P1kNg/0EUIF059BDP11@D40Bd];069RHP23Ph<0EeMG03Hf=P1GEeL0
PH6109FEU@2QXJ40W9bL05=CD`1IFET0=SHf02h^;P0g=cL0>c/k03dm?@1LG5`0?Shn02/[:`0i>CT0
JFUY030`<02;Rh/0FEUI02TY:@0I6AT0;Bd]05EEE@1NGUh0RXZ:0:>SX`2MWId0_[jn0820P00o?cl0
6QXJ01DE5@0I6AT0FEUI06MWI`0H61P05ADE02HV9P35aLD0VYZJ08^;R`2SXj<0_kno02d];@0S8b<0
JFUY08F5Q@0k>c/06a/K04U9B@2_[jl0YZJV07moO`2^[Zh0f][J0=WIf@20P800DE5A01LG5`0j>SX0
Fe]K030`<012@T80:b/[07IfMP1dM7@0?clo065QH@1XJ6P0M7Ad07alO02JVYX0LG5a05QHF01EEED0
TI6A03@d=0<05ADE00/06a/K071`L01<C4`0FUYJ04=3@`2SXj<0aLG50:FUY@2][Jd09RHV01<C4`02
010@400<01DE5@0I6AT0OGem071`L01=CDd0ATI60<k>cP2j^[X0/k>c0=CDe02OWil0UYJF0P2CTi<@
ooooo`003_ooool0?`1VIVH0CDe=06UYJ@1=CDd0IfMW0410@01YJFT0Lg=c038b<P1SHf<0?Cdm08>3
P`28R8P0DE5A09NGU`1GEeL0K6a/03`l?014A4@0@d=303`l?02MWId0AdM703`l?00o?cl0A4A406=S
H`1mOGd0@D5103De=@0W9bL0Jf][06EUI@0];Bd0>SXj07YjNP0^;Rh0C4a<02l_;`0T92@07alO04M7
A`0U9BD0?Shn01lO7`0a<C40?3`l030`<00[:b/0CTi>02DU9@0/;2`0NGUi04];B`0Q8B40TY:B034a
<@1JFUX0KVi^04U9B@1EEED0LW9b01TI6@0203lo?`1001dM7@0h>3P0Q8B401TI6@1>CTh0;Bd]03`l
?01fMWH0@T920451@@1TI6@0FUYJ09bLW01<C4`07AdM07emO@22PX80=cLg07moO`1gMgL07QhN03@d
=01@D500IVIV03Xj>P21PH40O7al03Hf=P2CTi<0Cdm?03Xj>P2[Zj/0QhN703Lg=`1mOGd0BTY:06YZ
JP1dM7@0X:2P061PH00n?Sh0IfMW06a/K018B4P0PX:205eMG@1[Jf/0UYJF07UiN@1<C4`0MgMg065Q
H@0e=CD0?clo05aLG00Z:RX0Ng]k07alO02EUID0?Cdm09RHV01^KVh0@D510861P@80DU9B04<0Hf=S
0:FUY@21PH40;bl_08J6QP1dM7@0DU9B0:n_[`0f=SH0MWIf0=GEe@1DE5@0=cLg03`l?019BDT08b<S
04a<C00f=SH0IfMW05]KF`0`<300QhN706IVIP11@D40De=C0410@02QXJ40IFEU04E5A@1JFUX0^k^k
04M7A`2MWId0^KVi069RHP0e=CD0>c/k05mOG`1IFET0TY:B08:2PP1JFUX0]KFe0:b/[01mOGd0Jf][
06IVIP0_;bl0A4A408:2PP1eMGD0Fe]K03/k>`18B4P0P82005IFEP1nOWh0IVIV0<G5a@2_[jl0?3`l
07UiN@25QHD0EeMG03hn?P0Z:RX0UYJF0080U9BD02X0LG5a04I6AP1/K6`0M7Ad0492@P1HF5P0d=3@
09:BTP11@D40?3`l059BDP14A4@0De=C0451@@10@400TY:B05=CD`0m?Cd0KFe]08j>SP1KFe/0I6AT
06i^KP26QXH0F5QH05YJFP2=SHd0G5aL05QHF029RHT0DE5A05MGE`1@D500AdM706EUI@1CDe<0Bd];
0<_;b`1[Jf/0IVIV05QHF01bLW8203Ph>00R09ZJVP1HF5P0BTY:06ATI01bLW80Bd];07moO`2l_;`0
XZ:R07QhN01fMWH0?clo01DE5@1LG5`0IVIV04m?C`1PH600=3@d04M7A`2^[Zh0NGUi03<c<`1hN7P0
H61P07IfMP2b/[80Jf][03Ph>00Q8B40<c<c05ADE00V9RH0ATI608V9R@kooooo05<0Y:BT0<K6aP2n
_[h0[:b/0:VYZ@1=CDd0EUIF02/[:`1`L700JVYZ07IfMP1<C4`0GEeM01@D500>3Ph03@d=00l?3`0@
41003Ph>00d=3@0>3Ph0HF5Q034a<@0Q8B40<C4a05ADE02DU9@0I6AT051@D00Z:RX04a<C00l?3`0>
3Ph03@d=00h>3P0F5QH0ADE5051@D01MGEd0JFUY07=cL`37alL0a/K60<W9b@0a<C40@T9205mOG`1c
Lg<0;Bd]02DU9@0O7al06a/K02<S8`0K6a/06ATI04e=C@2LW9`0E5AD06i^KP2GUiL0SHf=08>3P`1Q
HF406QXJ01TI6@0N7Qh08B4Q04e=C@1bLW80Rh^;08f=S@1jNWX0KVi^096AT@2XZ:P0QhN706QXJ01M
GEd0UIFE07=cL`1<C4`06ATI01DE5@0201@D500e01<C4`0U9BD0I6AT03Lg=`1EEED0=CDe03dm?@1R
HV80KVi^020P800L71`092@T02`/;010@400Lg=c05QHF014A4@0<C4a04A4A018B4P0>SXj01XJ6P0H
61P05QHF01XJ6P16ATH0EUIF02XZ:P0Y:BT0<30`04a<C00j>SX0:RXZ01dM7@0e=CD0`l?30<k>cP2j
^[X0Jf][06=SH`0T92@0@T9208V9R@2YZJT0LG5a09VIV@1SHf<0:2PX03`l?01MGEd0PX:20;2`/01>
CTh00P0U9BD0:00S8b<0L71`0<o?c`3De=@0EeMG03`l?00c<c<0?3`l06QXJ01oOgl0ZJVY09:BTP28
R8P0<c<c02@T9011@D40I6AT02@T900K6a/0?Shn07IfMP15ADD08B4Q03<c<`1HF5P0;Rh^02d];@0g
=cL0Kfm_0:>SX`2n_[h0c<c<0=CDe02@T900DU9B04=3@`1LG5`0>SXj01`L700R8R82020P800703@d
=015ADD0<30`0492@P1:BTX0=SHf08Z:RP0201dM7@0>020P800M7Ad071`L03De=@17AdL0@T9209bL
W01nOWh0Xj>S09BDU00U9BD09RHV05IFEP2<S8`201LG5`0;03lo?`1jNWX0RXZ:0>3Ph03Fe]H0hN7Q
0;fm_@0^;Rh0J6QX07QhN00R8R800P0E5AD0:P0I6AT0KFe]0=_Kf`39bLT0WinO07emO@2HV9P0][Jf
09VIV@0Y:BT071`L05IFEP21PH40FUYJ07inOP2k^k/0/K6a07emO@1RHV80CDe=07QhN02CTi<0EeMG
04A4A01>CTh0<c<c01hN7P0k>c/0:2PX01DE5@0M7Ad05aLG02HV9P2DU9@0SHf=08>3P`2[Zj/0_Kfm
0;jn_P1oOgl0Cdm?04M7A`80?clo00h0@d=306QXJ02ZZZX0Fe]K01hN7P0U9BD0dM7A0<[:bP2QXJ40
][Jf092@T00i>CT0;2`/03Hf=Q3ooooo000>ooooo`1U07EeM@1]KFd0O7al04e=C@1lO7`0>CTi04];
B`1]KFd0>3Ph075aL@0U9BD0?clo06ATI01GEeL0RHV903De=@1DE5@0Bd];04A4A00W9bL0:RXZ07Yj
NP15ADD0>CTi03De=@0^;Rh0Cdm?06QXJ014A4@0@d=305UIF@1MGEd0D51@02`/;00n?Sh0RHV904];
B`1<C4`0B4Q802d];@0k>c/0Cdm?028R8P0o?cl0=cLg06ATI017AdL0;Bd]02`/;01NGUh0?clo02`/
;029RHT0FEUI028R8P1SHf<0@41003@d=01eMGD0Cdm?05MGE`1fMWH0;Rh^04A4A017AdL08B4Q04A4
A025QHD0=cLg05ADE00_;bl071`L051@D01<C4`0ATI605mOG`0m?Cd0UiNG05eMG@0T92@0RXZ:08:2
PP0j>SX0S8b<08R8R00W9bL0>SXj05EEE@1<C4`0?Shn075aL@1iNGT0;Rh^09jNWP19BDT0=CDe08J6
QP1]KFd0=3@d07MgM`0K6a/00P1dM7@04P2HV9P0;Bd]05UIF@29RHT0OGem04e=C@1jNWX0S8b<07Ui
N@2;Rh/0KFe]02TY:@1IFET0CTi>03Hf=P13@d<0Bd];02DU9@80EUIF01D0Q8B402@T901LG5`0JFUY
0492@P2>SXh0@T9207inOP1kNg/0WIfM09JFUP0S8b<0K6a/06IVIP1CDe<0ZZZZ02@T9015ADD0dM7A
05EEE@1BDU800P1;Bd/0<`0[:b/0EUIF04A4A01mOGd0EEEE038b<P2>SXh0OGem06EUI@1NGUh0<S8b
08J6QP20P800HF5Q079bLP2e]KD0DU9B06m_K`1lO7`0E5AD038b<P12@T80EeMG03lo?`1XJ6P0TI6A
065QH@2GUiL0ZJVY08j>SP2`/;00U9BD03`l?015ADD0QhN707=cL`1LG5`0?Cdm059BDP27QhL0D51@
07moO`21PH40b<S80:2PX00c<c<0UiNG09nOW`1MGEd0<S8b01TI6@0205=CD`0T06][J`1oOgl0De=C
069RHP1/K6`0D51@05IFEP3;bl/0Vi^K05]KF`0h>3P0J6QX04I6AP1BDU80BTY:0410@02GUiL0CTi>
03lo?`1aLG40SHf=05UIF@1UIFD0JVYZ079bLP1EEED0Xj>S0:ZZZP1_Kfl0GemO08:2PP15ADD0RXZ:
06a/K012@T80H61P0P0l?3`0:@2?Shl0EUIF07MgM`1QHF40CTi>03`l?01_Kfl0^[Zj08j>SP1EEED0
HF5Q055AD@0V9RH0@d=3071`L02IVIT0HF5Q08:2PP1SHf<0=SHf05eMG@1?Cdl0BTY:06EUI@1NGUh0
=SHf08Z:RP1bLW80=3@d06EUI@1gMgL0De=C08^;R`1:BTX0>CTi02PX:00^;Rh0BTY:02TY:@1=CDd0
U9BD00kooooo0240Bd];059BDP1[Jf/0ZZZZ0<K6aP1?Cdl0A4A403`l?00Y:BT0@410059BDP0j>SX0
E5AD04m?C`0E5AD03Ph>00d=3@0>3Ph03@d=010@400P8200X:2P04=3@`0J6QX0820P02@T9015ADD0
K6a/07MgM`1DE5@0:RXZ02/[:`0N7Qh00P0Z:RX0=00l?3`0GemO06IVIP1cLg<0YZJV0<71`@2k^k/0
aLG50:VYZ@2LW9`0HF5Q096AT@2;Rh/0;2`/028R8P0T92@0820P024Q8@0X:2P0@D5107YjNP2[Zj/0
Ti>C07emO@1DE5@0BDU903@d=01SHf<0J6QX0492@P1?Cdl0>SXj03@d=01iNGT0]kNg06UYJ@1BDU80
KVi^0:b/[03<c<`0b/[:03hn?P0I6AT0K6a/061PH01RHV809RHV01DE5@0C4a<04Q8B01DE5@0l?3`2
03lo?`0`03Lg=`0R8R80:BTY02l_;`0g=cL05ADE01HF5P0I6AT06a/K02PX:013@d<0FEUI04=3@`0j
>SX0;bl_07emO@1cLg<0?Shn02/[:`0R8R80@D5105UIF@0m?Cd0:2PX028R8P0L71`07alO04];B`1_
Kfl0NGUi07EeM@2<S8`0WinO0;Bd]02OWil0Bd];05MGE`2IVIT0F5QH03Ph>00`<300ADE5079bLP2M
WId0SXj>08B4Q02MWId0YjNW0`0X:2P05P0c<c<0H61P0<G5a@3IfMT0_;bl0:j^[P2d];@0ZZZZ07mo
O`0L71`0P8200<G5a@33`l<0WIfM05IFEP11@D40HV9R05mOG`18B4P0IfMW03Lg=`0K6a/201TI6@06
0410@013@d<0;2`/02DU9@0j>SX0UiNG0P2k^k/0;@2l_;`0QHF506i^KP1EEED08B4Q03Hf=P0n?Sh0
71`L01dM7@0K6a/092@T0492@P0b<S80EEEE02d];@1QHF40JVYZ01LG5`0E5AD06QXJ02DU9@0_;bl0
8R8R02h^;P0G5aL0De=C0:2PX01OGel0TY:B09>CT`2?Shl0eMGE07MgM`1<C4`0FEUI08:2PP2j^[X0
d=3@0=GEe@3Kfm/0d];B0;Vi^@2GUiL0SXj>01XJ6P0301DE5@0B02`/;01EEED0VYZJ04U9B@0R8R80
9bLW03lo?`2=SHd0HV9R07AdM00b<S80=SHf0<c<c034a<@0Rh^;07alO02l_;`0WYjN0P2IVIT09P2W
YjL0QXJ601LG5`0M7Ad09BDU02d];@0E5AD06ATI051@D00_;bl06a/K01DE5@0J6QX0?3`l0<g=c@2E
UID0NWYj0;^k^`2KVi/0=cLg038b<P0n?Sh0?3`l03Hf=P0j>SX0GemO0;:b/P1TI6@07QhN03lo?`2g
]kL0I6AT02h^;P1GEeL0IfMW028R8P0V9RH0:b/[4?ooool000kooooo0?l0S8b<07IfMP1mOGd0De=C
08J6QP11@D40H61P03/k>`0V9RH0N7Qh038b<P10@400Cdm?02l_;`1kNg/0K6a/071`L01KFe/0I6AT
04e=C@1>CTh0Ti>C05MGE`18B4P0H61P03/k>`1DE5@0JVYZ0410@01:BTX0E5AD06EUI@15ADD06QXJ
05YJFP2KVi/0E5AD04a<C00m?Cd0AdM703Xj>P1;Bd/09bLW05]KF`11@D40GUiN04=3@`0/;2`0820P
06ATI00h>3P0<S8b08R8R00[:b/0BTY:08:2PP0i>CT0?Shn05QHF00f=SH0Kfm_092@T013@d<0@D51
07YjNP0^;Rh0=CDe065QH@0S8b<0DE5A02XZ:P0`<300GemO0410@019BDT0DE5A05UIF@28R8P0F5QH
02LW9`2?Shl0PX:203De=@2?Shl0Rh^;03@d=00o?cl0HF5Q05MGE`11@D40IfMW06QXJ00[:b/0W9bL
04m?C`13@d<0TY:B069RHP0g=cL0HF5Q03Ph>01OGel0Hf=S06i^KP0Z:RX0CDe=07=cL`1VIVH0@410
07=cL`1^KVh0E5AD07moO`1NGUh0?Cdm05MGE`1JFUX0?Cdm04Y:BP0j>SX0>c/k07emO@1;Bd/0PX:2
0492@P1gMgL0EUIF03/k>`1lO7`0BTY:07EeM@1aLG40XZ:R08f=S@1?Cdl0N7Qh06IVIP1KFe/0TY:B
03Ti>@1OGel0Ti>C06UYJ@1OGel0IVIV04i>CP0c<c<0RXZ:06IVIP20P800Jf][03`l?02[Zj/0SXj>
05eMG@1UIFD0<c<c08J6QP1KFe/0D51@0820P02n_[h0I6AT08>3P`2EUID0GemO03Lg=`1<C4`0@410
04E5A@2HV9P0RXZ:04Q8B01oOgl0][Jf07moO`24Q8@0Jf][02HV9P17AdL0NGUi05IFEP1YJFT0=cLg
04U9B@1lO7`0A4A408J6QP1iNGT0`/;209RHV00N7Qh0Jf][07alO015ADD0AdM703De=@1dM7@0CDe=
04];B`1FEUH0CDe=04m?C`1HF5P0820P055AD@2LW9`0[jn_07=cL`0[:b/0Fe]K04E5A@2DU9@0PH61
04I6AP2HV9P0?clo04A4A02FUYH0YjNW06ATI01[Jf/0M7Ad03dm?@0n?Sh0Kfm_0:JVYP1]KFd0IFEU
08>3P`0m?Cd0E5AD05UIF@0k>c/0GemO02DU9@0c<c<0Q8B40861P@1[Jf/0HV9R05MGE`0P8200<c<c
07QhN01]KFd0CTi>07UiN@1dM7@0=SHf04m?C`2GUiL0F5QH0492@P2MWId00100LW9b03Lg=`1LG5`0
DU9B03Ti>@1kNg/0M7Ad03`l?02UYJD0De=C03/k>`27QhL0WYjN071`L02<S8`0A4A40P10@4001@1M
GEd0F5QH02h^;P1LG5`0PH6100kooooo04H0D51@04U9B@15ADD0E5AD0<k>cP2UYJD0DE5A020P800J
6QX05ADE01LG5`0N7Qh0<30`055AD@1NGUh0C4a<04E5A@14A4@0BTY:04i>CP1IFET0J6QX07EeM@1W
IfL0@41002l_;`0/;2`0D51@09>CT`2b/[80U9BD0:^[Z`1YJFT0H61P05]KF`1PH600FEUI06m_K`1S
Hf<0KVi^08V9R@1fMWH0N7Qh096AT@2KVi/0U9BD09nOW`1TI6@0B4Q803/k>`0a<C40ATI605ADE01?
Cdl0BTY:03De=@1=CDd0<C4a04Q8B01nOWh0TI6A09BDU02?Shl0D51@05UIF@2GUiL0N7Qh06m_K`21
PH40M7Ad0P1/K6`04P1`L700PH610;Fe]@2CTi<0DE5A01PH600R8R80HV9R05mOG`1YJFT06QXJ01<C
4`0R8R80?Cdm03Ti>@0^;Rh0QXJ602XZ:P80820P00<0:b/[03Lg=`0F5QH00`0E5AD0O`0I6AT0;Rh^
07MgM`28R8P0RHV9096AT@2l_;`0Vi^K05aLG01MGEd0NGUi06][J`0o?cl0:2PX01lO7`0I6AT05aLG
01PH600O7al0ATI608N7Q`28R8P0Q8B405ADE01YJFT0Vi^K0<S8b02/[:`0EEEE02@T900L71`07AdM
01hN7P0U9BD0XZ:R04a<C018B4P0O7al08f=S@1PH600B4Q805YJFP1_Kfl0Y:BT0;Rh^02l_;`0G5aL
03hn?P11@D40GUiN06ATI01UIFD0U9BD07alO01fMWH0U9BD05YJFP1LG5`0K6a/09FEU@2f][H0?clo
01LG5`0H61P05aLG01XJ6P0N7Qh0>SXj055AD@1;Bd/0J6QX0:b/[02k^k/0^;Rh0;^k^`2a/K40@410
02DU9@0L71`07QhN030`<010@400<c<c020P800b<S80IFEU07UiN@2h^;P0/[:b0;Rh^01>CTh05ADE
01<C4`0E5AD06a/K02`/;00M7Ad0;2`/018B4P0F5QH0?Shn07MgM`2>SXh0][Jf06e]K@1QHF40J6QX
07QhN01gMgL0Ph>30:ZZZP2DU9@0V9RH0:^[Z`1fMWH0V9RH0;6a/@0S8b<051@D01HF5P0H61P092@T
04E5A@1cLg<0V9RH03lo?`1:BTX00P1BDU80<`13@d<0SXj>06UYJ@2SXj<0]kNg09fMW@1MGEd0/K6a
0:6QX@2SXj<0X:2P09RHV025QHD0Ph>307AdM00T92@07alO03@d=01?Cdl0<C4a03De=@1UIFD0EEEE
04e=C@1OGel0CDe=06IVIP3?cll0fMWI0<G5a@2ZZZX0De=C02<S8`0U9BD09RHV024Q8@0P8200;bl_
0451@@1nOWh0FUYJ02HV9P21PH40OWin03/k>`0U9BD0?Shn07inOP0k>c/0ADE505=CD`0@ooooo`00
3_ooool0j01fMWH0IVIV07MgM`1:BTX0MGEe0410@01QHF40Ng]k04I6AP2?Shl0<c<c03Ti>@1fMWH0
H61P06e]K@1IFET0>c/k028R8P1cLg<0BTY:04e=C@1jNWX0;Rh^0820P01iNGT0>CTi055AD@1UIFD0
7QhN04m?C`1IFET0IVIV04Y:BP0B4Q80AdM708^;R`11@D40B4Q803Hf=P18B4P0<C4a03@d=00W9bL0
G5aL03Ph>01ADE40=CDe01DE5@0O7al0CTi>020P800b<S80PH6103Xj>P0_;bl0IVIV03Xj>P1@D500
A4A401TI6@1IFET0MGEe034a<@1>CTh0S8b<01dM7@0i>CT0Cdm?01TI6@1HF5P08B4Q03Lg=`1MGEd0
>3Ph05MGE`1CDe<0Lg=c07moO`19BDT09bLW08b<S01ZJVX0?clo09VIV@28R8P08b<S0492@P1^KVh0
J6QX051@D01cLg<0DU9B03@d=02:RXX0MgMg04I6AP2NWYh0De=C05UIF@2HV9P0C4a<05eMG@1WIfL0
DU9B03@d=01@D500MGEe07MgM`10@400OWin07]kN`1NGUh0XJ6Q07alO01:BTX0Hf=S03/k>`0o?cl0
GEeM069RHP1ADE40NGUi055AD@2MWId0Cdm?08N7Q`1JFUX0CDe=09ZJVP1cLg<0FEUI06][J`2f][H0
UIFE04e=C@1mOGd0<30`07QhN02>SXh0A4A409JFUP2ZZZX0ATI609>CT`2GUiL0@410020P801FEUH0
@41007AdM01>CTh0>CTi09:BTP1hN7P0GEeM06IVIP19BDT0S8b<051@D012@T80W9bL0<_;b`1/K6`0
QhN707EeM@0[:b/0@41007EeM@1OGel0HV9R08R8R012@T808b<S09>CT`2m_Kd0U9BD08R8R01EEED0
7alO04I6AP2FUYH0A4A404];B`0c<c<0Jf][06e]K@0e=CD0QHF505mOG`36a/H0Ti>C02/[:`1]KFd0
H61P0451@@1GEeL0>c/k07emO@1BDU80Cdm?05ADE015ADD0Bd];05eMG@1<C4`0Rh^;08J6QP2;Rh/0
HF5Q03/k>`0n?Sh0;bl_09>CT`1[Jf/0>CTi0:NWY`1:BTX0AdM706i^KP1nOWh0<C4a05]KF`1aLG40
Lg=c06UYJ@1JFUX0RHV903<c<`1VIVH0QHF504U9B@1<C4`204I6AP0/08N7Q`1BDU80:BTY09JFUP2:
RXX0RHV90:BTY02>SXh0;2`/02d];@1]KFd0;2`/04I6AP1ADE40Ogmo05IFEP1^KVh0^[Zj08J6QP1c
Lg<0SXj>07moO`0m?Cd0Fe]K05iNGP0l?3`0GUiN06a/K012@T80UYJF06ATI00`<300OWin092@T01n
OWh0QXJ602d];@0V9RH0<30`03dm?@1ADE40<30`069RHP2HV9P>ooooo`0;051@D01MGEd0Jf][07Qh
N035aLD0eMGE09^KV`0/;2`06ATI01HF5P0E5AD00P0C4a<0<@0I6AT0WIfM0:^[Z`1SHf<0E5AD03/k
>`0N7Qh06QXJ01/K6`0L71`0<c<c07QhN024Q8@0PH6109RHV01`L700dm?C0=KFeP2FUYH0?clo02<S
8`0J6QX05aLG04Q8B015ADD071`L01LG5`0L71`0H61P0;Vi^@2=SHd0/k>c07EeM@1EEED0Ogmo04a<
C01PH600SXj>04Y:BP0L71`05ADE01LG5`0o?cl0=CDe01`L700Y:BT0@D510:6QX@1UIFD00P1EEED0
7P1oOgl0dM7A0<O7a`2?Shl0Hf=S02d];@0L71`06ATI01XJ6P0I6AT0LG5a0;^k^`1mOGd0ADE502LW
9`0V9RH0Kfm_05mOG`1lO7`0GUiN07=cL`1oOgl0V9RH0<?3``3Bd]80K6a/02l_;`0/;2`0:b/[04Q8
B0@05ADE03l061PH02d];@1ZJVX0O7al0:n_[`2k^k/0RXZ:07YjNP2d];@0b/[:09>CT`1kNg/0BDU9
03lo?`11@D40A4A403/k>`0Q8B40<S8b059BDP24Q8@0DU9B03Ti>@0R8R809bLW04=3@`2n_[h0LW9b
02`/;00`<3007AdM01`L700M7Ad07alO05=CD`12@T80=3@d04I6AP1eMGD0NGUi07emO@2@T900QXJ6
06YZJP2MWId0OWin07moO`1UIFD0@T9202DU9@0Z:RX0QXJ604e=C@0];Bd0Bd];07MgM`0d=3@07alO
02d];@19BDT0J6QX06IVIP0Y:BT00P0I6AT06P0K6a/07alO04m?C`1;Bd/09RHV06ATI02XZ:P0][Jf
08B4Q01RHV80Y:BT03De=@0U9BD08B4Q01hN7P0O7al07AdM038b<P2QXJ40UIFE0:6QX@2h^;P0`L71
0<W9b@3?cll0MGEe0`0E5AD0600O7al07QhN01hN7P0Y:BT04A4A01DE5@0G5aL0JVYZ06IVIP2OWil0
QhN7092@T023Ph<0R8R808>3P`27QhL0_[jn07QhN01mOGd0:BTY04Q8B023Ph<0RXZ:01`L70805ADE
03`06ATI02d];@1VIVH0QHF50;>c/`19BDT09bLW024Q8@0M7Ad092@T05ADE03Fe]H0alO70:n_[`22
PX80FEUI075aL@0i>CT07QhN03Lg=`1HF5P0CDe=03@d=00i>CT0Ph>3059BDP1IFET05aLG01DE5@0L
71`0CDe=0<;2`P2TY:@0SHf=0:RXZ02^[Zh0_[jn0=_Kf`3Cdm<0c/k>05]KF`0J6QX06a/K01TI6@0J
6QX05aLG01hN7P0Y:BT0NWYj08V9R@2>SXh0d];B05aLG00b<S80EEEE0;6a/@0c<c<05ADE01LG5`0M
7Ad@ooooo`003_ooool0b01jNWX0KVi^08V9R@1HF5P0Ng]k03dm?@1YJFT0OWin04M7A`2;Rh/0=CDe
04A4A022PX80HF5Q06e]K@1QHF40J6QX0410@01ADE40@d=302/[:`1jNWX0>3Ph04Y:BP1=CDd0;bl_
04A4A01?Cdl0=SHf04=3@`1>CTh0Hf=S04e=C@0R8R80Cdm?09VIV@15ADD0HV9R04I6AP18B4P0:2PX
03/k>`0b<S80FEUI02h^;P15ADD0=SHf01`L700_;bl0G5aL03Hf=P0b<S80S8b<03Ph>00c<c<0Jf][
03De=@1GEeL0IfMW01lO7`1;Bd/0Jf][03Xj>P1ADE40QXJ602d];@0k>c/0EeMG01dM7@1fMWH0>c/k
03@d=01MGEd0;Rh^069RHP1XJ6P0KFe]07MgM`1DE5@0:RXZ08F5Q@1`L700@d=309JFUP1fMWH092@T
05MGE`1oOgl0IFEU04a<C01aLG40JVYZ02d];@2MWId0HF5Q04U9B@2<S8`0LG5a03Xj>P1kNg/0?3`l
061PH01NGUh0Lg=c0410@01BDU80P82005ADE018B4P0SHf=08B4Q013@d<0QXJ608:2PP1?Cdl0I6AT
06EUI@0o?cl0Fe]K05mOG`10@400K6a/05mOG`1kNg/0ADE5065QH@0W9bL0A4A407alO01RHV80AdM7
03dm?@1fMWH0OWin0410@01mOGd0CDe=07AdM021PH40@d=30:6QX@2HV9P0ATI605ADE01NGUh0Bd];
03De=@1XJ6P0<30`07MgM`1KFe/0CTi>0:FUY@1FEUH0CTi>08F5Q@1:BTX0QXJ605=CD`14A4@0ZJVY
0;Zj^P1`L700SHf=09JFUP1MGEd0;2`/05aLG01XJ6P0IfMW09:BTP1gMgL0<c<c06][J`2g]kL0TY:B
0;Bd]01cLg<0>3Ph07moO`2WYjL0J6QX04A4A00`<300JFUY06=SH`1>CTh0WYjN04Y:BP2c/k<0Ph>3
02DU9@1LG5`0@D5103Xj>P23Ph<0@41006i^KP1ADE40BTY:0P1HF5P0C02MWId0P82005eMG@2EUID0
WinO09:BTP0m?Cd0F5QH06EUI@1>CTh0/k>c04];B`0W9bL0RHV903lo?`1ADE40Lg=c07MgM`1DE5@0
Fe]K06a/K01kNg/0KVi^05=CD`2?Shl0L71`05ADE02BTY80BTY:04e=C@1DE5@0>3Ph05IFEP1JFUX0
<S8b08b<S01EEED0FUYJ0861P@1gMgL0;Rh^03Xj>P1nOWh0LW9b055AD@1BDU80NGUi04a<C02?Shl0
`L7108V9R@1OGel0RHV906MWI`15ADD0WYjN08^;R`10@400I6AT05]KF`1@D500_Kfm06a/K00h>3P0
TY:B07YjNP2;Rh/0XZ:R05mOG`1:BTX0:2PX03Ti>@1QHF40>CTi07QhN02OWil>ooooo`160;Ng]`1h
N7P0UIFE07MgM`0i>CT0Kfm_0<G5a@1TI6@09BDU01lO7`0E5AD04a<C01@D500j>SX0IfMW03<c<`0Q
8B40=CDe04a<C01>CTh0@41002LW9`0L71`07AdM02XZ:P1DE5@0TY:B0:2PX034a<@0^k^k0;Ng]`1>
CTh0>c/k03Lg=`0e=CD0<S8b02TY:@1CDe<0EUIF028R8P0E5AD0?Shn09VIV@1XJ6P0CTi>07]kN`2I
VIT0/k>c0;jn_P2e]KD0;Bd]03`l?00j>SX0>CTi05mOG`1aLG40>CTi02DU9@0i>CT0ADE507alO01V
IVH0?Shn01TI6@0L71`0SHf=04U9B@0L71`06QXJ01LG5`806QXJ01L071`L01hN7P1QHF40P82007Yj
NP1WIfL0AdM704E5A@16ATH0Zj^[09jNWP1KFe/0F5QH06YZJP1MGEd0EeMG06e]K@2h^;P0TI6A06e]
K@1XJ6P0JFUY02LW9`0201HF5P0P01LG5`0N7Qh0;bl_04U9B@1FEUH0LG5a03Lg=`1@D500FUYJ07Ee
M@2m_Kd0HF5Q03<c<`13@d<0Hf=S069RHP1_Kfl0EEEE07AdM02FUYH0Xj>S04M7A`1FEUH0;2`/020P
800M7Ad0@d=30<;2`P1LG5`07QhN03<c<`0Y:BT201`L701O01dM7@0d=3@0J6QX04];B`0^;Rh0HF5Q
04I6AP1bLW80@d=304];B`1>CTh0DE5A061PH01JFUX0Lg=c08^;R`0];Bd08b<S04i>CP1CDe<08b<S
03/k>`1OGel0<S8b02/[:`0a<C40FUYJ03lo?`1GEeL0GemO02d];@0Y:BT0;Rh^03De=@1IFET0>CTi
05EEE@1WIfL0[jn_09NGU`1YJFT0?clo096AT@25QHD0:RXZ020P800O7al08B4Q030`<01CDe<0b/[:
08j>SP2[Zj/0b/[:0=SHf021PH40P82006IVIP17AdL051@D01DE5@0J6QX06ATI01dM7@0V9RH04A4A
01<C4`0Y:BT0H61P05YJFP2g]kL0LW9b04m?C`1LG5`0FUYJ07YjNP1?Cdl0S8b<0:f][@0^;Rh071`L
02DU9@2OWil0ZJVY03De=@1LG5`0?3`l03Ph>011@D40EUIF08>3P`3Fe]H0>SXj02DU9@0R8R800P0U
9BD01`0j>SX0eMGE071`L02JVYX0WYjN0<?3``2:RXX00`0E5AD0:00G5aL0<30`05]KF`0f=SH0JFUY
09FEU@23Ph<0:2PX03Ph>013@d<0GEeM0<C4a03Cdm<0c<c<07emO@2IVIT0`/;20<[:bP3IfMT0fm_K
08^;R`0_;bl071`L02DU9@11@D40GemO07IfMP2;Rh/0Zj^[0;^k^`3@d=00e]KF09nOW`1[Jf/0Kfm_
0;6a/@0e=CD05aLG01<C4`0G5aL@ooooo`003_ooool0;`1bLW80Hf=S0;Fe]@1EEED0M7Ad04Q8B026
QXH0N7Qh04E5A@27QhL0=cLg05ADE02EUID0Fe]K06YZJP1MGEd0LW9b055AD@1DE5@0CDe=02l_;`2I
VIT0Cdm?05QHF01IFET0ADE506e]K@1hN7P0@41003Xj>P0V9RH0JVYZ06UYJ@0N7Qh0G5aL09JFUP0f
=SH0HF5Q02l_;`0d=3@0:RXZ05IFEP0_;bl0F5QH02DU9@1<C4`0EUIF0080>c/k04H0BDU901TI6@12
@T80OWin02/[:`0g=cL0LW9b030`<01eMGD0GUiN01PH601=CDd0HF5Q03Lg=`1eMGD0QXJ603hn?P0b
<S80De=C01PH601XJ6P0:RXZ030`<01HF5P0<c<c03hn?P19BDT0KFe]07moO`0l?3`0Bd];09^KV`2?
Shl0B4Q809^KV`1LG5`071`L0451@@21PH40H61P05ADE01kNg/0OGem03@d=02ZZZX0Hf=S03Lg=`2@
T900Ogmo04U9B@1eMGD0?3`l08B4Q01jNWX0MgMg0451@@1bLW80Q8B405EEE@0i>CT0KVi^09fMW@1H
F5P0TY:B07=cL`13@d<0NGUi05QHF016ATH0I6AT0P0j>SX0W@1/K6`0FEUI06m_K`1@D500Kfm_03dm
?@1BDU80KVi^05aLG01kNg/0KVi^08f=S@26QXH0<C4a08n?S`1JFUX0OGem06UYJ@15ADD0Shn?08j>
SP1ADE40JVYZ06e]K@1<C4`0@D5107moO`1EEED0LG5a05]KF`1;Bd/0/K6a06e]K@1=CDd0N7Qh059B
DP2][Jd0J6QX04Y:BP2VYZH0[Jf]06EUI@2c/k<0U9BD059BDP0c<c<0J6QX04E5A@1YJFT0XZ:R06UY
J@0e=CD0QXJ60<[:bP1WIfL0S8b<059BDP0N7Qh0A4A406ATI00g=cL0GUiN03<c<`1cLg<0@41004E5
A@2JVYX0D51@09nOW`1hN7P0;2`/07UiN@1FEUH0:BTY04M7A`0c<c<0Shn?0861P@1jNWX0G5aL03Xj
>P1CDe<0CDe=059BDP2IVIT0`/;20:f][@1CDe<0ATI6061PH01BDU80a<C406a/K00l?3`0RXZ:0410
@01RHV80[Jf]09fMW@1GEeL0K6a/07inOP1DE5@0I6AT09nOW`2DU9@0L71`05aLG028R8P0CDe=07al
O01_Kfl0ATI605YJFP0o?cl0BDU908^;R`1TI6@0NGUi0:f][@1eMGD0@T9206a/K01mOGd0Jf][051@
D01cLg<0OWin04U9B@2b/[80`l?307QhN01FEUH0X:2P04a<C00T92@0FUYJ05aLG014A4@0TY:B075a
L@0h>3P0RXZ:06EUI@1@D500[jn_07=cL`1TI6@0Vi^K06e]K@17AdL0EEEE0:RXZ01eMGD0:BTY06][
J`1gMgL03_ooool08P2_[jl0_;bl07EeM@0f=SH0:RXZ028R8P1YJFT0X:2P04I6AP0L71`05aLG01@D
500H61P0HV9R03Ti>@0E5AD06QXJ01dM7@0I6AT07alO038b<P1VIVH0Hf=S065QH@2:RXX0YjNW0<?3
``38b<P0fMWI0=7Ad@1]KFd092@T01LG5`0J6QX201DE5@0>01HF5P0G5aL0;Rh^06ATI01VIVH0HF5Q
09FEU@0P82006a/K01dM7@1;Bd/0d=3@0<30`020P80201DE5@0F01PH600R8R80QhN706][J`1GEeL0
LG5a03`l?01LG5`0MWIf05eMG@0^;Rh05ADE01XJ6P1_Kfl08R8R01LG5`0H61P05QHF01`L700O7al0
:BTY038b<P80W9bL04D0MWIf02TY:@1LG5`0Hf=S06YZJP2i^KT0V9RH0:j^[P2FUYH0MGEe05]KF`1W
IfL0OWin0:>SX`29RHT0Q8B406e]K@19BDT0De=C059BDP0Y:BT0;bl_04E5A@0i>CT0GUiN079bLP18
B4P071`L01/K6`0N7Qh0@41009:BTP0g=cL0820P01dM7@0L71`07alO02h^;P1LG5`0/[:b0=KFeP1a
LG40F5QH051@D00N7Qh071`L01dM7@1`L700a<C404U9B@0L71`07AdM03@d=00U9BD071`L01dM7@0l
?3`0LW9b03dm?@0Y:BT0HV9R04i>CP1GEeL0=cLg030`<01LG5`09bLW02DU9@11@D400P0k>c/0>`0O
7al09BDU03`l?01NGUh0=3@d03De=@14A4@0<C4a03<c<`1FEUH0RXZ:0861P@13@d<0QhN708>3P`1E
EED0A4A403De=@0_;bl0@D5103Ph>013@d<0VIVI09bLW02=SHd0DE5A07MgM`0b<S80K6a/03@d=00k
>c/0FEUI07MgM`2FUYH0/;2`08Z:RP30`<00clo?0:n_[`1UIFD0?3`l05]KF`1NGUh0B4Q801hN7P0[
:b/08R8R01hN7P0^;Rh03Ph>01XJ6P1EEED0JVYZ06ATI01EEED0=cLg034a<@0c<c<0=CDe0080;bl_
02X0=3@d08:2PP0`<3007QhN02TY:@2n_[h0g]kN0<71`@37alL0`l?30:n_[`2d];@0a/K60=;BdP3H
f=P0QXJ602d];@0U9BD0820P02<S8`0b<S80d];B06EUI@1fMWH0Zj^[0;jn_P1nOWh05ADE01@D500I
6AT05ADE01HF5P0`<300OWin09jNWP1/K6`0RHV9071`L00S8b<0:2PX07YjNP3>c/h20=WIf@030:RX
Z02`/;00c<c<0080dm?C01D0c/k>096AT@1;Bd/0Shn?0:VYZ@2g]kL0ZJVY0:6QX@2][Jd0/K6a0:j^
[P1nOWh0TY:B0<30`02>SXh0>SXj07=cL`1UIFD08b<S01DE5@0F5QH04?ooool000kooooo0?l0PX:2
05ADE01mOGd0GEeM05=CD`12@T80Ng]k065QH@0o?cl0IVIV03/k>`1XJ6P0Ph>303lo?`1EEED09bLW
069RHP19BDT0DU9B028R8P0f=SH0QHF504Y:BP1]KFd0@41002LW9`14A4@0FUYJ02@T901;Bd/0@d=3
051@D012@T805ADE07YjNP2GUiL0ADE503De=@0I6AT0Bd];02DU9@0f=SH09BDU065QH@16ATH0?Cdm
03<c<`0J6QX0<30`061PH00V9RH0@T9206][J`0X:2P0G5aL08V9R@0R8R80FUYJ05=CD`0W9bL0HV9R
07UiN@0T92@0EeMG055AD@0I6AT0=3@d04a<C00O7al0NGUi01`L700m?Cd0IVIV04I6AP1LG5`0?Cdm
05EEE@1^KVh0?Cdm02LW9`1eMGD0FEUI03dm?@2/[:`0J6QX02XZ:P1:BTX0RXZ:03Lg=`19BDT0IfMW
06UYJ@1CDe<0`/;209ZJVP0g=cL0H61P06ATI01ZJVX0SHf=030`<01TI6@0Lg=c07AdM00Q8B40>3Ph
05iNGP1BDU80BDU905UIF@1cLg<0CTi>09NGU`1OGel0:BTY04m?C`19BDT0L71`09fMW@1WIfL0=SHf
051@D01RHV80SXj>03`l?01@D500?Shn07]kN`2:RXX0BTY:05iNGP1YJFT0aLG509jNWP11@D40NGUi
03Xj>P2;Rh/0MGEe04Q8B01[Jf/0KFe]04a<C02NWYh0Rh^;04=3@`1?Cdl0RHV905=CD`1eMGD0@d=3
059BDP2VYZH0PH6105ADE01`L700EUIF07emO@1QHF40ADE50:ZZZP2PX:00IVIV0:f][@1lO7`0De=C
06ATI02NWYh0H61P05EEE@25QHD0;Bd]01TI6@2ZZZX0cLg=09^KV`2HV9P0@T9204=3@`26QXH0WYjN
05ADE013@d<0D51@0:JVYP1FEUH0BTY:08>3P`1?Cdl0`l?308R8R00Y:BT0F5QH05ADE00h>3P0I6AT
02XZ:P1=CDd0?clo03Lg=`16ATH0B4Q805mOG`15ADD0:BTY06i^KP1/K6`0Q8B404U9B@18B4P0@T92
02LW9`2KVi/0GEeM03/k>`2MWId0=SHf03lo?`1VIVH0K6a/02DU9@1]KFd0W9bL061PH01IFET0F5QH
05mOG`0T92@0LW9b08:2PP0g=cL0@41002d];@1?Cdl0VIVI04=3@`0U9BD0OGem05YJFP1QHF40_Kfm
09NGU`0P8200=CDe06QXJ00g=cL0LG5a096AT@1oOgl0<c<c08f=S@2SXj<0=cLg05IFEP28R8P001L0
MWIf02LW9`1OGel0CDe=02DU9@1IFET0KFe]03dm?@2?Shl0<C4a04m?C`2MWId0VIVI07]kN`2e]KD0
IfMW04e=C@2RXZ80YjNW07IfMP0];Bd0Jf][05YJFP0>ooooo`0S0<_;b`1^KVh0>c/k02h^;P0Y:BT0
7QhN028R8P27QhL0PX:202HV9P0K6a/06QXJ02l_;`1BDU80<C4a01DE5@0G5aL08b<S024Q8@0Z:RX0
CTi>08:2PP2=SHd0Xj>S08V9R@22PX80VIVI0::RXP2g]kL0dm?C0:NWY`1FEUH07AdM01XJ6P0B4Q80
0`0E5AD0300G5aL0;Rh^09:BTP2h^;P0Z:RX01TI6@0A4A404Q8B01PH6026QXH0B4Q806i^KP<05ADE
00<06ATI06IVIP0e=CD00P0b<S803`0h>3P0Lg=c07IfMP1[Jf/0:BTY01PH600O7al0L71`01`L700J
6QX06ATI02LW9`0`<300D51@06QXJ00207IfMP0M01TI6@0Q8B40@d=302LW9`1RHV80^KVi0;Fe]@0o
?cl0Cdm?05=CD`1`L700KFe]07AdM01/K6`0Bd];059BDP1VIVH0MWIf06YZJP2/[:`0LG5a08>3P`1W
IfL0@41005QHF01ADE40;Rh^03De=@0R8R800P0O7al03P0_;bl0SHf=06IVIP0l?3`0:BTY02DU9@0S
8b<0820P01dM7@1<C4`0XZ:R06YZJP1cLg<0:BTY0P0M7Ad0<00e=CD0aLG50=;BdP1oOgl09RHV01`L
700M7Ad0;Rh^01lO7`10@400ATI607UiN@1<C4`09RHV07QhN01BDU80CTi>0492@P0o?cl0G5aL01LG
5`0F5QH0:RXZ03Ti>@0`<300;Bd]01lO7`0f=SH0KFe]05ADE00Q8B40MgMg04M7A`0R8R80>3Ph05AD
E01VIVH0GEeM06UYJ@1hN7P0CDe=03Lg=`0f=SH0@41004=3@`1ZJVX0IFEU0;>c/`80b<S800<0]KFe
06m_K`0k>c/00P2_[jl0B02IVIT0IVIV05eMG@2CTi<0Rh^;07emO@0l?3`0=CDe05eMG@1SHf<0CTi>
04=3@`1@D500IFEU05YJFP0a<C40De=C04];B`10@4005aLG04a<C01HF5P0MgMg06QXJ00K6a/05ADE
01<C4`0E5AD04a<C010@400C4a<061PH07UiN@1:BTX0<S8b07QhN03IfMT0[:b/051@D01TI6@0Hf=S
061PH01dM7@0LW9b0:>SX`2OWil0/;2`06m_K`0n?Sh0820P02@T900f=SH0c/k>08Z:RP1mOGd0Y:BT
07]kN`1_Kfl0;Rh^028R8P0_;bl08R8R01lO7`0Q8B40UIFE096AT@1ZJVX0IFEU0:f][@2QXJ40P820
08J6QP80`l?301/0^;Rh07AdM02XZ:P0LW9b03Hf=P1LG5`0YZJV0:f][@1kNg/0LW9b09:BTP21PH40
KVi^065QH@0o?cl0D51@07QhN024Q8@0LG5a07=cL`20P800FUYJ06a/K01aLG40IfMW06a/K01eMGD0
4?ooool000kooooo01X0Ogmo05eMG@2HV9P0DU9B05iNGP0o?cl0MWIf06UYJ@1<C4`0Q8B4034a<@15
ADD0RHV903/k>`1]KFd0BTY:05=CD`15ADD0LW9b04U9B@0[:b/0I6AT02HV9P1:BTX0IfMW03/k>`80
De=C09T08B4Q0492@P1?Cdl0CTi>04E5A@0E5AD0CTi>07inOP13@d<0?3`l01TI6@13@d<0<C4a02d]
;@0E5AD0BTY:04I6AP1=CDd0ADE5020P800S8b<0G5aL03lo?`1DE5@0J6QX01<C4`1:BTX0IfMW02d]
;@1VIVH092@T02PX:01KFe/0JFUY04a<C02GUiL0OWin01dM7@0[:b/0G5aL02PX:028R8P0?clo02/[
:`1=CDd0<C4a07=cL`1WIfL0KVi^08^;R`0Y:BT0IfMW0<30`02BTY80>3Ph09BDU013@d<0=SHf07mo
O`2[Zj/0G5aL04Y:BP1YJFT0=cLg02`/;01OGel0K6a/04];B`1PH600=CDe04Q8B01fMWH0ADE506i^
KP1JFUX0Jf][034a<@1ADE40HF5Q038b<P1GEeL0L71`08b<S00d=3@0R8R807]kN`0m?Cd0EeMG02`/
;00b<S80EUIF061PH017AdL0HV9R04M7A`1_Kfl0ATI605MGE`0M7Ad0B4Q808R8R01VIVH0>CTi02h^
;P1?Cdl0GemO04e=C@1lO7`071`L05EEE@1VIVH0ADE508V9R@1cLg<0;bl_07UiN@1ZJVX0Bd];0410
@01IFET0?Shn07UiN@1JFUX0Cdm?0<;2`P1=CDd0G5aL07]kN`1<C4`0Fe]K04];B`1SHf<0b/[:0;bl
_028R8P0_[jn07MgM`0`<300A4A406EUI@1EEED0De=C07emO@1=CDd0=SHf08Z:RP2LW9`0PX:207em
O@0i>CT06ATI05IFEP1fMWH00`17AdL06P24Q8@0M7Ad07]kN`2CTi<0>c/k07UiN@1QHF40=3@d06m_
K`0_;bl0;2`/07QhN00e=CD0G5aL05IFEP17AdL0B4Q805ADE02EUID0JVYZ05IFEP2ATI40Vi^K08j>
SP0Y:BT0ADE50P0k>c/06P2<S8`0<S8b01TI6@21PH40AdM704i>CP1gMgL0I6AT02`/;01DE5@0IFEU
071`L01kNg/0JFUY08J6QP1IFET0Cdm?07QhN011@D40CTi>04E5A@0];Bd0GemO059BDP0W9bL0UIFE
0P1JFUX09P20P800Lg=c02<S8`0n?Sh0LW9b05ADE01ADE40AdM706QXJ014A4@0[:b/09FEU@19BDT0
?3`l05aLG01nOWh0?3`l0820P01QHF40>CTi05IFEP1VIVH0CTi>09nOW`1WIfL0>c/k07MgM`1]KFd0
QXJ60<;2`P10@4009BDU03dm?@0j>SX0DU9B02LW9`1hN7P0TI6A3_ooool01@2PX:00@D5103De=@0[
:b/092@T00806QXJ04L0=SHf0;6a/@0o?cl051@D01TI6@1LG5`0>c/k01hN7P0E5AD07AdM06e]K@2J
VYX0XZ:R08^;R`1`L700IVIV04A4A00U9BD0:BTY0451@@2LW9`0ADE50:FUY@1aLG40RXZ:075aL@0N
7Qh05ADE01@D500F5QH0;2`/03lo?`1bLW80TY:B0:b/[02ZZZX0>SXj01@D500C4a<05ADE06YZJP0R
8R80MGEe02PX:00^;Rh0=cLg0451@@2ATI40JVYZ038b<P0U9BD0:2PX0820P01hN7P0FUYJ020P800K
6a/0?clo05eMG@0L71`09bLW03Ti>@1>CTh0C4a<03/k>`0^;Rh0GEeM01lO7`0>3Ph05ADE024Q8@02
04M7A`0A0;Bd]02[Zj/0PX:207inOP0i>CT0C4a<059BDP1@D500B4Q805UIF@1TI6@0HV9R05QHF01h
N7P0T92@0:b/[02DU9@00P20P8003@1<C4`08b<S01XJ6P0/;2`0<c<c02l_;`15ADD0CDe=04a<C01b
LW80B4Q805aLG00k>c/00P0c<c<08P0`<300D51@09bLW02e]KD0MWIf01TI6@0V9RH0@d=308B4Q03P
h>00d];B07QhN01IFET0=CDe01`L700Y:BT0RHV904i>CP0f=SH0GEeM096AT@1MGEd0c/k>0;Jf]P2E
UID0De=C08b<S01<C4`05QHF01LG5`0I6AT0;Bd]02`/;00k>c/202/[:`0f06QXJ01RHV80?clo0:f]
[@2g]kL0Hf=S04U9B@0l?3`0=SHf04M7A`1=CDd0Ph>309RHV01WIfL0GEeM05iNGP1XJ6P0LG5a08B4
Q02DU9@0^KVi0<30`036a/H0a<C409nOW`2XZ:P0[:b/0;no_`2i^KT0];Bd0:6QX@1=CDd06a/K01PH
600T92@0EEEE06UYJ@22PX80XZ:R07moO`21PH40Shn?07emO@25QHD0MWIf06=SH`10@400DE5A04i>
CP1aLG40@d=301@D500C4a<051@D100E5AD07@0I6AT0FUYJ0820P021PH40Ti>C0<c<c01[Jf/0=CDe
08V9R@3EeMD0L71`04U9B@1=CDd0T92@09BDU01WIfL0U9BD04a<C00K6a/07AdM03lo?`2^[Zh0M7Ad
0861P@1;Bd/0;Bd]055AD@1fMWH0AdM70080>CTi02L0>SXj04=3@`2ZZZX0X:2P09>CT`29RHT0QhN7
0<K6aP0_;bl0EeMG09bLW02DU9@0<C4a01LG5`0k>c/0>SXj01dM7@0Q8B40K6a/0=7Ad@2o_kl0Z:RX
05=CD`0_;bl09BDU03<c<`19BDT0Fe]K06m_K`1UIFD0<c<c06MWI`1^KVh0MgMg04U9B@1TI6@08b<S
02/[:`0h>3P04?ooool000kooooo0440OGem051@D028R8P0FEUI051@D01FEUH0YJFU03dm?@0];Bd0
JFUY03@d=01DE5@0CTi>038b<P1cLg<0?3`l0410@00V9RH0@T9204Y:BP0`<300SHf=04a<C01:BTX0
M7Ad03Xj>P1;Bd/0K6a/0451@@15ADD0Cdm?07EeM@1lO7`06a/K04a<C020P800E5AD06QXJ00f=SH0
9BDU04i>CP1EEED08b<S05MGE`0m?Cd0IfMW071`L00g=cL0>SXj0451@@0L71`0OGem06MWI`0V9RH0
DU9B04m?C`0g=cL0T92@04a<C00Y:BT0D51@07EeM@0H61P0@41004Q8B002034a<@0;05=CD`0S8b<0
KVi^0492@P0/;2`0DU9B02TY:@1=CDd0CTi>07]kN`13@d<00P0Z:RX0aP1WIfL0D51@03dm?@2d];@0
C4a<01LG5`11@D40MGEe059BDP1:BTX0Cdm?07=cL`0b<S80UIFE06][J`13@d<0EeMG061PH01FEUH0
IFEU03hn?P24Q8@0KFe]06][J`0k>c/0BTY:07=cL`1CDe<0EUIF065QH@2DU9@0BTY:096AT@1iNGT0
?clo065QH@16ATH0@d=3065QH@1=CDd0ATI606=SH`1UIFD0DE5A04I6AP1@D500<S8b04];B`2>SXh0
Kfm_03`l?01eMGD0TY:B07moO`0d=3@0KVi^04E5A@1lO7`0KVi^03<c<`27QhL0GEeM03`l?01eMGD0
ATI603Ti>@11@D40E5AD02/[:`1RHV80DU9B04I6AP35aLD0GemO03@d=01QHF40>c/k04=3@`0n?Sh0
=CDe08j>SP2DU9@0Jf][0:j^[P1VIVH0BDU904Q8B020P800I6AT05UIF@29RHT0E5AD03`l?02DU9@0
YJFU07inOP2?Shl0D51@02HV9P1FEUH0U9BD04m?C`1RHV80>3Ph07moO`1cLg<0NWYj09bLW01ZJVX0
a<C405QHF00Y:BT0J6QX05ADE00/;2`0J6QX02l_;`1DE5@0EeMG04i>CP1;Bd/0?clo051@D01>CTh0
FEUI09jNWP2FUYH0PH6103`l?00c<c<0?clo03dm?@1^KVh0B4Q803@d=01mOGd0;bl_04i>CP1aLG40
FUYJ03Hf=P1TI6@0J6QX06IVIP28R8P0Z:RX09VIV@1VIVH0B4Q807EeM@1KFe/0SXj>05iNGP1?Cdl0
DU9B0410@00[:b/0V9RH061PH023Ph<0VIVI06UYJ@0M7Ad0@D5106ATI01aLG40GUiN05YJFP1/K6`0
@d=30<c<c02XZ:P0C4a<04I6AP1iNGT0@d=303Ti>@28R8P0Ogmo04I6AP2?Shl0EEEE05mOG`2QXJ40
GEeM04E5A@2TY:@0HF5Q05UIF@2e]KD0SHf=04];B`2EUID0DU9B061PH00i>CT0S8b<09nOW`kooooo
04X0];Bd0492@P0h>3P0;2`/024Q8@0L71`06a/K01DE5@1NGUh0MgMg01PH600[:b/0J6QX02d];@0K
6a/0=CDe06ATI02NWYh0];Bd07moO`0e=CD08B4Q07AdM02CTi<0Bd];03lo?`1KFe/0RXZ:01TI6@1E
EED0Vi^K03/k>`1]KFd0Ph>303De=@0T92@0?clo06EUI@1dM7@0C4a<03Ti>@1jNWX0IVIV06EUI@0I
6AT04a<C018B4P1WIfL0:BTY08J6QP2b/[80N7Qh05MGE`0g=cL0?3`l061PH01XJ6P0<S8b024Q8@2/
[:`0JFUY06ATI00N7Qh06ATI04E5A@14A4@09bLW03Xj>P0^;Rh0;2`/02TY:@0/;2`0@D5102PX:0<0
5ADE01405QHF01/K6`1DE5@0XJ6Q075aL@2IVIT0^k^k0820P012@T809BDU01lO7`0K6a/06QXJ01`L
700U9BD0>SXj08b<S00207IfMP1Y0;Bd]030`<00KFe]01hN7P0L71`061PH028R8P1dM7@0D51@04M7
A`16ATH0CDe=09>CT`18B4P0AdM702l_;`0b<S80>c/k0451@@1[Jf/0[Zj^0:n_[`2DU9@0S8b<08>3
P`1dM7@0XZ:R06EUI@0o?cl0Ti>C06YZJP1?Cdl0Bd];01lO7`26QXH0?Cdm02DU9@16ATH0]KFe0:j^
[P2][Jd0clo?0<[:bP30`<00d=3@034a<@0E5AD05QHF01DE5@0N7Qh0?3`l038b<P11@D40:RXZ06i^
KP1?Cdl0V9RH0=SHf02WYjL0T92@01/K6`0J6QX0820P01lO7`0m?Cd0O7al09fMW@1bLW80NWYj07Ad
M01UIFD0J6QX09BDU027QhL0_[jn0;Vi^@2GUiL0eMGE0;Rh^01MGEd0=CDe03Ti>@1?Cdl0Zj^[05AD
E00G5aL05ADE01LG5`0P8200GemO06YZJP2UYJD0_[jn0<71`@28R8P0UiNG09RHV02DU9@0IfMW06AT
I01lO7`0AdM706MWI`1^KVh07AdM00<04a<C02@05ADE01LG5`0G5aL05ADE01HF5P0n?Sh0WYjN06][
J`1DE5@0Vi^K04Q8B00];Bd0ATI60:NWY`1cLg<0FUYJ04U9B@1mOGd0T92@04i>CP1CDe<0WYjN03dm
?@0G5aL0AdM70:^[Z`25QHD0=3@d02h^;P0W9bL0<c<c0<c<c02OWil0O7al0820P02>SXh20<S8b00T
05eMG@0Q8B409BDU02`/;01mOGd0A4A40820P01aLG40Jf][01TI6@0G5aL09BDU03hn?P0N7Qh08B4Q
03Ti>@2/[:`0/k>c0<;2`P1hN7P0Cdm?06MWI`1;Bd/0;bl_028R8P1DE5@0I6AT065QH@1GEeL0OGem
08b<S01`L700GUiN024Q8@0W9bL09RHV4?ooool000kooooo02`0T92@05]KF`2;Rh/0G5aL061PH014
A4@0GemO04M7A`0k>c/0JFUY03Xj>P1mOGd0RHV903@d=01UIFD0=3@d06UYJ@0l?3`0C4a<02XZ:P0k
>c/0QhN704I6AP1;Bd/0BTY:03Lg=`1VIVH0LG5a03Xj>P13@d<0>c/k069RHP1BDU807AdM05QHF01j
NWX0BDU904=3@`0];Bd0AdM704Q8B019BDT08B4Q06MWI`80@T9204d0A4A403`l?018B4P0E5AD03Lg
=`1PH600Fe]K02/[:`1`L700Kfm_02l_;`1TI6@0@D5104Q8B021PH40LW9b03hn?P1KFe/0@41001/K
6`1FEUH0G5aL02DU9@1cLg<09BDU04M7A`1iNGT0@41006a/K017AdL0MgMg05IFEP0h>3P0<c<c07em
O@1@D500ADE50;Jf]P1ADE40<c<c055AD@2MWId0@D5104m?C`1]KFd0Lg=c03De=@2CTi<0JVYZ0492
@P1LG5`0MGEe06ATI01_Kfl0DU9B08>3P`1]KFd0E5AD04U9B@1UIFD0MGEe05eMG@1cLg<0FUYJ09ZJ
VP1MGEd0N7Qh06ATI015ADD0IFEU0410@018B4P0IVIV03dm?@19BDT0L71`06][J`0205ADE01@069R
HP0m?Cd0DU9B08f=S@1XJ6P0EUIF08F5Q@2;Rh/0Q8B402h^;P1oOgl0@d=306][J`1HF5P0:RXZ07Ui
N@24Q8@0@T9207UiN@14A4@0>c/k059BDP1PH600=cLg07=cL`1NGUh0BTY:0<o?c`1aLG40C4a<06EU
I@16ATH0@d=304M7A`11@D40Y:BT08b<S01@D500Ng]k08N7Q`1BDU80CTi>09nOW`1EEED0EeMG09ZJ
VP1lO7`0=cLg09BDU02FUYH0NGUi0;Bd]01PH600<c<c05aLG01nOWh0Bd];05IFEP0l?3`0OGem059B
DP1_Kfl0[:b/065QH@34a<@0De=C03@d=025QHD0JVYZ03Xj>P1OGel0<C4a05ADE01YJFT0JVYZ06UY
J@0m?Cd0?Shn04U9B@12@T820:2PX01709NGU`1<C4`0Bd];03/k>`0W9bL0IVIV06MWI`0l?3`0Q8B4
02TY:@1>CTh0Ph>306YZJP0_;bl0IfMW07AdM01HF5P0IFEU06][J`1FEUH0=SHf06EUI@1LG5`0=cLg
05]KF`0o?cl0@T9207IfMP10@400A4A40:FUY@1?Cdl0HV9R0:b/[01cLg<0;2`/06QXJ01_Kfl0Cdm?
05iNGP1`L700P82003Ph>02a/K40OWin03Ti>@17AdL0NWYj0820P00f=SH0Fe]K05aLG00c<c<0HF5Q
05]KF`1GEeL0O7al02l_;`15ADD0TY:B06UYJ@1jNWX0a/K604a<C00n?Sh0SXj>08Z:RP1OGel08b<S
06YZJP1eMGD03_ooool0502n_[h0Shn?04e=C@0_;bl09RHV01dM7@0L71`05aLG01hN7P2EUID0HV9R
09bLW02BTY80RXZ:0861P@1mOGd0HF5Q03lo?`0i>CT06a/K0P0E5AD0;P0o?cl07AdM00l?3`0>3Ph0
3`l?03hn?P0j>SX07AdM05eMG@1oOgl0;Rh^08n?S`36a/H0W9bL04i>CP0V9RH07alO01XJ6P0G5aL0
7QhN04M7A`0o?cl0DU9B04E5A@0];Bd0Ti>C08Z:RP2;Rh/08R8R02@T900[:b/0@41004];B`1LG5`0
JFUY05]KF`24Q8@0[jn_07moO`1QHF4092@T03Ti>@2NWYh0BTY:02TY:@0O7al2024Q8@0302HV9P0k
>c/0:b/[00805ADE01L05QHF01DE5@0E5AD061PH01hN7P1;Bd/0=cLg09:BTP1>CTh0?clo01`L700N
7Qh07alO01/K6`0I6AT061PH01LG5`0J6QX0;Rh^02DU9@0b<S80K6a/075aL@0203lo?`0:01XJ6P0H
61P0:RXZ02LW9`0M7Ad07QhN01lO7`0V9RH0FEUI06ATI0809bLW06<0:RXZ02d];@0k>c/0C4a<0;Bd
]03:b/X0/;2`08b<S02BTY80Ogmo03/k>`0N7Qh0820P02XZ:P1UIFD0NGUi06MWI`1<C4`0CDe=04i>
CP1cLg<0HF5Q04Q8B022PX80`l?309NGU`2OWil0a<C40:b/[01XJ6P0:2PX01LG5`0E5AD0820P04a<
C01FEUH0MgMg07=cL`2l_;`0Rh^;06QXJ01[Jf/0GEeM06m_K`0d=3@06QXJ01TI6@0O7al0<C4a07al
O021PH40Cdm?07UiN@2HV9P0JVYZ05eMG@1hN7P0UIFE0:f][@2d];@0]kNg0:FUY@2DU9@0AdM70410
@00h>3P0=3@d06QXJ02=SHd0DE5A024Q8@0N7Qh0<30`02TY:@0P8200IfMW08^;R`1NGUh0DE5A069R
HP1VIVH0PX:20:VYZ@2m_Kd0K6a/071`L02>SXh0KFe]01XJ6P0C4a<04A4A018B4P0C4a<05ADE01<C
4`0D51@05QHF04=3@`15ADD00P1WIfL0@@2MWId0<c<c02`/;01XJ6P0TI6A03Ti>@0g=cL0Bd];04U9
B@0M7Ad0@D5104U9B@1=CDd0Kfm_06UYJ@1kNg/0];Bd04Q8B00N7Qh092@T01hN7P12@T80HF5Q02<S
8`0N7Qh07alO02DU9@11@D40Xj>S05]KF`0S8b<0820P02LW9`1WIfL0XJ6Q0820P01VIVH07QhN01PH
600G5aL07AdM03Ph>00R8R8092@T03Hf=P1eMGD0CTi>07QhN01?Cdl0AdM701/K6`0F5QH06ATI01lO
7`0[:b/0?clo06=SH`21PH40J6QX06m_K`2;Rh/0RHV9079bLP1BDU80F5QH013ooooo000>ooooo`3o
08J6QP1RHV80[Jf]05QHF01XJ6P0CDe=07IfMP12@T809RHV061PH00l?3`0Lg=c03lo?`0S8b<0Fe]K
03@d=00l?3`08b<S0492@P13@d<0A4A407inOP0R8R80Fe]K06i^KP10@400LW9b06e]K@0G5aL0ADE5
06YZJP2LW9`0@T92014A4@1CDe<0S8b<05aLG010@4008B4Q03`l?01>CTh06ATI018B4P1ADE40=CDe
05QHF01?Cdl0410@02@T9015ADD0;Rh^0820P00o?cl05QHF05EEE@1XJ6P0<c<c05eMG@0L71`0<S8b
05eMG@1TI6@0CTi>09FEU@1OGel0<S8b04e=C@1SHf<09RHV07alO00n?Sh09bLW05mOG`11@D40IFEU
04a<C01[Jf/0Bd];030`<012@T80UIFE05YJFP0a<C40JVYZ0410@00a<C40EUIF0:NWY`0W9bL0;2`/
05mOG`1VIVH0?clo0:b/[02:RXX0?Cdm05QHF01LG5`0IVIV05UIF@0h>3P0MGEe07=cL`1NGUh0<C4a
04I6AP1SHf<0DE5A08V9R@1<C4`0T92@05]KF`2<S8`0I6AT02d];@1DE5@0B4Q8069RHP28R8P0HF5Q
04U9B@1QHF40I6AT059BDP14A4@0EUIF02d];@1PH600TI6A05eMG@0j>SX0HV9R0:JVYP26QXH0<C4a
06QXJ00o?cl0DU9B04i>CP0V9RH0?Shn092@T01;Bd/0Ng]k06m_K`14A4@0QXJ60;Ng]`0o?cl0KFe]
05IFEP1;Bd/0/k>c06m_K`1?Cdl0GUiN059BDP1;Bd/0De=C04e=C@2d];@0R8R8061PH02GUiL0];Bd
04a<C01BDU80Xj>S06ATI01ADE40EUIF065QH@16ATH0];Bd0:6QX@22PX80[:b/03dm?@1BDU80WinO
08V9R@0m?Cd0<30`04U9B@1iNGT0FEUI06EUI@2EUID0@d=30<K6aP1CDe<0=CDe06i^KP19BDT0>c/k
07alO00`<300ADE505IFEP1=CDd0Cdm?03<c<`0m?Cd0GUiN04Y:BP2BTY80JVYZ08^;R`1>CTh0G5aL
04=3@`0g=cL0E5AD04];B`10@400WinO04i>CP1=CDd0OGem05=CD`0Y:BT0H61P05eMG@1bLW80Kfm_
07AdM01MGEd0@T92051@D01NGUh0>SXj05QHF011@D40>SXj05iNGP1SHf<0?clo0:NWY`1JFUX0AdM7
0820P01gMgL08B4Q05=CD`1<C4`0>3Ph03hn?P0d=3@0KFe]04=3@`39bLT0Vi^K04=3@`0j>SX0GemO
000G05iNGP1HF5P0WYjN08>3P`0n?Sh0GemO04];B`1oOgl0XJ6Q04Q8B00k>c/0R8R803lo?`1JFUX0
^;Rh07MgM`11@D40T92@04M7A`0m?Cd0:b/[071`L029RHT03_ooool06@1EEED0PX:209bLW010@400
8b<S01`L700J6QX05aLG01DE5@1VIVH0/[:b0<;2`P1mOGd0=SHf01`L700F5QH06ATI01dM7@0H61P0
7QhN01PH600F5QH0EEEE00l?3`0=3@d00P0>3Ph02P0D51@0E5AD018B4P0C4a<0KFe]09RHV00j>SX0
WinO065QH@0F5QH201PH600Q02@T900W9bL08b<S028R8P1CDe<0BTY:0451@@2GUiL0Kfm_05ADE02E
UID0Fe]K055AD@0i>CT0;Rh^024Q8@0M7Ad0ADE50<o?c`3Bd]80emOG0<c<c038b<P0/;2`0:f][@2L
W9`0Ti>C05MGE`1=CDd0>SXj028R8P0_;bl0=SHf00805QHF0`0E5AD201LG5`0703dm?@0e=CD06a/K
07inOP18B4P0;2`/01HF5P0201LG5`0501DE5@0F5QH061PH01PH600E5AD00P0G5aL0700Z:RX0?Cdm
06m_K`0k>c/0?clo04Q8B00M7Ad0<c<c01DE5@0J6QX06ATI01dM7@0X:2P09BDU0820P01=CDd0B4Q8
04=3@`0g=cL0;bl_0861P@2d];@0`/;20<c<c02_[jl0XJ6Q02/[:`0K6a/201`L700l01dM7@0N7Qh0
<S8b08j>SP2:RXX0c<c<09VIV@2OWil0YJFU08N7Q`16ATH0IfMW0=7Ad@3Bd]80<c<c02DU9@1>CTh0
De=C038b<P0[:b/0B4Q804E5A@25QHD0_[jn0=SHf03GemL0SHf=07AdM01OGel0L71`02TY:@23Ph<0
?Shn01XJ6P0S8b<0IfMW07emO@1fMWH0KFe]09FEU@2WYjL0I6AT04A4A01iNGT0`L7109nOW`32`/80
]KFe05mOG`1TI6@0C4a<04I6AP0`<300820P01XJ6P19BDT0JVYZ02LW9`15ADD0?Cdm0P0L71`04`1R
HV80:BTY01<C4`0E5AD04a<C01HF5P0L71`0@D5109:BTP1/K6`0VYZJ07emO@21PH40;2`/01<C4`0>
3Ph0410@01<C4`0D51@00P0E5AD0AP0F5QH0CDe=02`/;022PX80[Zj^0<O7a`1oOgl0WinO0<G5a@38
b<P0VYZJ06IVIP21PH4071`L01LG5`0K6a/07AdM05MGE`2EUID0[:b/0>OWi`3>c/h0HV9R02HV9P0i
>CT0>SXj04I6AP0G5aL071`L01LG5`0J6QX07AdM01lO7`1EEED0IVIV038b<P0Z:RX0>c/k092@T00l
?3`0K6a/01LG5`0F5QH06ATI01dM7@0O7al0<c<c02@T900S8b<0<c<c05mOG`0^;Rh0L71`03De=@0N
7Qh04a<C01LG5`0I6AT071`L02HV9P0a<C40B4Q804M7A`0a<C40D51@07UiN@2YZJT0^k^k0:VYZ@2_
[jl@ooooo`003_ooool0g@25QHD0D51@08B4Q01NGUh0Cdm?06e]K@2ZZZX0H61P03Ph>01PH600=cLg
05mOG`21PH409BDU04m?C`0[:b/0IVIV02DU9@0e=CD0:RXZ0451@@1iNGT0=3@d04I6AP1?Cdl0=cLg
0820P02IVIT0=CDe034a<@0[:b/0K6a/065QH@0O7al0IfMW05EEE@0n?Sh0H61P02h^;P0Y:BT0F5QH
04I6AP0M7Ad0C4a<01`L701LG5`0EUIF04E5A@17AdL0E5AD03Xj>P21PH40HF5Q02@T901hN7P0EEEE
03Xj>P1jNWX0CDe=0492@P1mOGd0D51@028R8P1>CTh0Cdm?034a<@1SHf<0EeMG01dM7@1WIfL0Bd];
03@d=01KFe/09RHV07MgM`1JFUX0OWin04a<C00F5QH0@d=3071`L01QHF40?clo0:NWY`16ATH0=cLg
05QHF02QXJ40@T9204A4A01JFUX0=3@d03Xj>P1MGEd0QXJ605]KF`1GEeL0:RXZ07MgM`24Q8@0B4Q8
06UYJ@1cLg<0MgMg0410@01?Cdl0=3@d01TI6@1/K6`0GemO07EeM@0a<C40WinO07moO`0n?Sh0GUiN
030`<01<C4`0Jf][06EUI@1ADE40F5QH04A4A01eMGD0FEUI04Q8B00D51@0Fe]K08j>SP1[Jf/0@T92
03lo?`1EEED0P82003Ti>@1/K6`0;bl_07alO029RHT0=cLg0451@@2SXj<0@D5108J6QP1WIfL0>3Ph
04M7A`1CDe<0=CDe06EUI@0j>SX0>3Ph0:n_[`1XJ6P0H61P065QH@1/K6`0QXJ607EeM@1DE5@0_kno
08n?S`0o?cl0Jf][08^;R`1:BTX0Fe]K08n?S`1KFe/0IFEU09FEU@0l?3`07alO08N7Q`22PX80NWYj
0:b/[00_;bl08b<S06][J`24Q8@0>CTi04A4A00k>c/0KVi^04];B`1/K6`0WIfM05YJFP2NWYh0GUiN
03Lg=`23Ph<09RHV03Ph>026QXH0<S8b04M7A`1NGUh0DU9B03hn?P1>CTh0Q8B4071`L01JFUX0Y:BT
08>3P`2e]KD0;bl_05EEE@1FEUH0?Shn05UIF@0/;2`0;Rh^07=cL`12@T80J6QX0:j^[P1aLG40=CDe
0080FUYJ01d0C4a<08R8R031`L40J6QX04Y:BP1DE5@0JFUY051@D01iNGT0Bd];03Ph>01OGel0H61P
05MGE`2o_kl0KVi^05EEE@2QXJ40Lg=c02XZ:P1dM7@0DU9B05]KF`1HF5P0@410065QH@0c<c<0[jn_
09jNWP02055AD@0H06a/K01EEED0?clo04Y:BP1VIVH0ATI606e]K@1?Cdl0Hf=S07YjNP18B4P0>CTi
07moO`1cLg<0IfMW0<S8b01dM7@0?3`l08>3P`1FEUH0ADE5020P8010@400P8203_ooool06016ATH0
QhN7069RHP2LW9`0;bl_01dM7@0H61P05aLG01HF5P1[Jf/0IfMW08R8R017AdL0820P01PH600M7Ad0
6QXJ01HF5P0P8200EeMG02PX:00T92@0F5QH010@40<03Ph>00T0410@04U9B@0I6AT0410@038b<P2b
/[80Fe]K07alO00j>SX00P0K6a/0;`0E5AD051@D01DE5@0E5AD05QHF020P801EEED0Lg=c09nOW`24
Q8@0MWIf034a<@0X:2P0ADE506EUI@1jNWX0OWin08b<S02c/k<0YZJV0:ZZZP3;bl/0_;bl0:b/[038
b<P0]kNg02/[:`0i>CT0Lg=c05EEE@1NGUh0FEUI06EUI@0L71`05QHF01DE5@0G5aL05ADE01LG5`0E
5AD06ATI05mOG`0T92@061PH08R8R018B4P0:2PX00805aLG00<05QHF01DE5@0F5QH00P0K6a/0H`0J
6QX05aLG01DE5@0/;2`0=SHf06EUI@14A4@0;2`/04e=C@10@400>SXj01DE5@0F5QH06ATI02XZ:P0U
9BD0820P03Hf=P0I6AT08B4Q04=3@`1^KVh0];Bd0<;2`P2o_kl0b/[:07EeM@1]KFd0Q8B404i>CP0M
7Ad05aLG01XJ6P0L71`07AdM020P8010@400_;bl0;>c/`3;bl/0`/;208:2PP0k>c/0AdM709nOW`3N
g]h0Vi^K03hn?P0K6a/05QHF01/K6`1@D500VYZJ08N7Q`2j^[X0a/K60<30`02MWId0PX:209nOW`2=
SHd0T92@0:f][@1RHV80?3`l06UYJ@1EEED0E5AD08Z:RP1bLW80A4A402h^;P1>CTh0I6AT02d];@0M
7Ad0?Shn06a/K010@400NGUi0:>SX`2DU9@0M7Ad04M7A`0R8R807AdM028R8P0J6QX09bLW07=cL`1;
Bd/0E5AD02/[:`0O7al08b<S079bLP0T92@05ADE01LG5`0301DE5@0801TI6@1PH600?Cdm069RHP23
Ph<0XJ6Q06ATI00E5AD201<C4`0O014A4@0E5AD04Q8B01@D500O7al0@T92038b<P2FUYH0c/k>0>;R
hP3WinL0_Kfm0:2PX02d];@0/[:b0;Ng]`24Q8@05QHF01LG5`0K6a/06ATI01HF5P1UIFD0/[:b0=oO
g`3AdM40R8R80451@@0l?3`0ATI60451@@0501DE5@0V01PH600k>c/0F5QH05YJFP0c<c<0Bd];07Ad
M00m?Cd0ADE501DE5@0K6a/071`L01LG5`0L71`0<30`024Q8@0V9RH0=SHf05UIF@0Y:BT0Hf=S03<c
<`0X:2P05QHF01LG5`0L71`07QhN02PX:00d=3@0@d=301dM7@0G5aL08B4Q051@D02KVi/0_Kfm0:FU
Y@2m_Kd@ooooo`003_ooool0FP2NWYh0FUYJ0:JVYP1SHf<0BTY:0492@P22PX80CDe=03/k>`2?Shl0
<c<c04m?C`2=SHd0:RXZ04];B`0M7Ad0GemO03Ti>@1BDU80>3Ph0451@@2DU9@0FEUI04M7A`0b<S80
92@T04m?C`1_Kfl0<c<c04U9B@14A4@0IVIV05=CD`0U9BD0HV9R06QXJ018B4P0@T9202PX:00Z:RX0
E5AD04a<C00M7Ad0De=C02TY:@0k>c/0?Cdm02/[:`0l?3`0<c<c01LG5`2ATI40EUIF01lO7`1jNWX0
=3@d02h^;P1UIFD0ADE504Q8B029RHT0EeMG03dm?@1VIVH0>3Ph01dM7@1WIfL0Jf][02TY:@1YJFT0
;bl_03hn?P1aLG40;Rh^04a<C016ATH0LW9b04A4A00U9BD0<c<c04i>CP1?Cdl0@d=30:^[Z`0Z:RX0
=3@d07alO02<S8`0ADE504Q8B080Hf=S0;X0@T9208:2PP25QHD0HF5Q07YjNP1<C4`0FEUI06IVIP1E
EED0K6a/05MGE`1eMGD0EeMG06ATI01QHF40GemO07]kN`1DE5@0Ti>C05UIF@1gMgL0JFUY04i>CP2<
S8`0DU9B051@D01[Jf/0IfMW06a/K02>SXh0LW9b06a/K024Q8@0Ogmo03Xj>P12@T80Shn?07MgM`1H
F5P0Jf][07inOP1iNGT0<c<c08f=S@0Z:RX0EEEE04e=C@0X:2P0Hf=S0:VYZ@1:BTX0K6a/05=CD`0l
?3`0KVi^08f=S@0a<C40I6AT065QH@0m?Cd0XJ6Q03De=@1MGEd0KFe]05iNGP1BDU80E5AD069RHP3C
dm<0U9BD055AD@1ZJVX0MGEe02HV9P14A4@0T92@03hn?P1[Jf/0XZ:R07AdM00e=CD0PH6107emO@1J
FUX0LW9b03lo?`0T92@0HV9R08B4Q00n?Sh0FEUI030`<01mOGd0BDU904e=C@21PH40Fe]K09jNWP0h
>3P0@T9208R8R01BDU80=SHf05iNGP14A4@0Ogmo08j>SP1`L700@41003Xj>P1>CTh0Jf][05MGE`2m
_Kd0UIFE0:^[Z`0n?Sh0EEEE06EUI@17AdL0KFe]06][J`1<C4`0Ng]k02DU9@10@400JFUY07emO@0i
>CT0EeMG079bLP1HF5P0IVIV09^KV`1BDU80>c/k05=CD`19BDT0CTi>08>3P`1OGel0;Bd]05EEE@0o
?cl0NGUi0<G5a@22PX80HF5Q09ZJVP1EEED0?3`l08f=S@1^KVh0M7Ad04m?C`1@D500UIFE04Q8B02^
[Zh0Z:RX020P801FEUH0X:2P06e]K@1KFe/0De=C03`l?015ADD0^KVi06][J`1cLg<0MgMg04U9B@10
@400S8b<06][J`1cLg<0alO706m_K`0j>SX0Jf][05UIF@1GEeL09BDU03Xj>P1oOgl>ooooo`0G07Mg
M`2j^[X0b/[:0<C4a01HF5P08B4Q01TI6@0G5aL0;Rh^09BDU01XJ6P0Vi^K03Hf=P0G5aL0<30`02HV
9P0J6QX05ADE038b<P1bLW80:RXZ0451@@1DE5@00P0@4100300>3Ph0410@00h>3P0i>CT09RHV010@
400E5AD0Ng]k0:j^[P1bLW80<S8b01PH60<05ADE02005aLG01HF5P0E5AD03`l?01<C4`0K6a/0DE5A
07moO`25QHD09RHV014A4@0B4Q806ATI02DU9@0[:b/0=CDe04Y:BP2IVIT0[jn_09nOW`2d];@0_kno
09>CT`32`/80FUYJ02@T900];Bd0IfMW05QHF018B4P0IFEU01lO7`@05ADE00/05QHF01LG5`1@D500
R8R802`/;00L71`0V9RH06YZJP15ADD071`L01TI6@0201LG5`0B01PH600F5QH05aLG01XJ6P0L71`0
6a/K02d];@0d=3@0G5aL05mOG`0c<c<0>c/k05=CD`1/K6`05ADE01LG5`0R8R808B4Q0P0M7Ad03`0[
:b/05ADE01DE5@0F5QH0:RXZ0=7Ad@1/K6`0WIfM08V9R@2[Zj/0^KVi04U9B@1mOGd09RHV01TI6@02
01XJ6P0401`L700O7al09BDU09NGU`80dM7A02T0bl_;08Z:RP2EUID0Z:RX06a/K01>CTh0S8b<06AT
I017AdL0:BTY02PX:01/K6`0^KVi0=KFeP3;bl/0[Zj^0:2PX01LG5`0Fe]K06IVIP1oOgl0UiNG08V9
R@1lO7`0HV9R07inOP1hN7P0`/;209BDU028R8P08B4Q01lO7`0h>3P0DU9B02HV9P0L71`0BDU903Xj
>P0b<S80Cdm?075aL@0208J6QP0>04Y:BP0M7Ad09RHV02/[:`0g=cL0;Rh^07MgM`2EUID0HV9R03Hf
=P0U9BD09bLW07emO@0K6a/401DE5@0M01PH600G5aL0;Rh^030`<01QHF40YJFU09nOW`2UYJD0Hf=S
01DE5@0D51@04A4A01<C4`0E5AD04a<C02d];@0^;Rh0<c<c07]kN`2a/K40TI6A09NGU`2EUID0RHV9
0;>c/`1@D500=cLg055AD@0j>SX00P0L71`201TI6@0:0492@P2l_;`0];Bd04I6AP0l?3`0DU9B0820
P01PH600Bd];01@D50D05ADE01H0=cLg05ADE01UIFD0?Cdm07EeM@1<C4`0HF5Q01/K6`0G5aL06a/K
01TI6@0H61P06a/K03/k>`0Q8B4092@T03Lg=`1ADE4092@T05]KF`0d=3@0<C4a0P0G5aL03@0L71`0
7alO02LW9`0i>CT0;Bd]01/K6`0I6AT071`L028R8P1MGEd0ZJVY0<W9b@2g]kL04?ooool000kooooo
0?X0[Zj^05YJFP2/[:`0LW9b06UYJ@1@D500Q8B405YJFP15ADD0Shn?03Lg=`1CDe<0T92@03<c<`1c
Lg<0?Cdm07IfMP0j>SX0Cdm?02XZ:P0O7al0NGUi059BDP12@T80LG5a03<c<`1EEED0K6a/03<c<`1Y
JFT0BDU906ATI00h>3P0;bl_07QhN01^KVh0@d=303Lg=`0f=SH0>SXj05IFEP0U9BD0:RXZ07AdM00j
>SX0C4a<038b<P0E5AD0BDU9061PH00W9bL0IVIV03/k>`0Y:BT0Y:BT04e=C@0b<S80JFUY03@d=01W
IfL0UIFE069RHP0l?3`0HV9R03@d=00E5AD0J6QX06=SH`0[:b/0NWYj01hN7P1:BTX0Lg=c03Lg=`1g
MgL0<S8b07UiN@0o?cl0:2PX0492@P1HF5P0?Shn06IVIP2PX:00@41003Xj>P14A4@0TI6A02l_;`1?
Cdl0IfMW05YJFP13@d<0Ph>308Z:RP18B4P0HF5Q04M7A`1_Kfl0H61P04Y:BP1UIFD0Kfm_05IFEP16
ATH0F5QH06IVIP1RHV80RHV905=CD`2:RXX0L71`09VIV@1KFe/0>3Ph06i^KP1@D500Fe]K06e]K@1B
DU80@d=306UYJ@1UIFD0Cdm?05iNGP1IFET0<30`05MGE`1MGEd0G5aL051@D01_Kfl0Shn?06UYJ@0O
7al0JVYZ03hn?P23Ph<0C4a<02DU9@0m?Cd0P82005=CD`2PX:00G5aL02l_;`1gMgL0V9RH04M7A`1S
Hf<0@d=304E5A@1nOWh0F5QH04];B`1?Cdl0Rh^;08:2PP1LG5`0BTY:0<71`@1oOgl0@41007QhN01I
FET0?Cdm06IVIP2j^[X0I6AT06a/K02CTi<0H61P03/k>`2BTY80Shn?069RHP27QhL0F5QH02LW9`1_
Kfl0EEEE03Xj>P1QHF40?clo07YjNP1mOGd0Fe]K096AT@1EEED0MWIf038b<P0m?Cd0O7al04m?C`0h
>3P0G5aL02PX:01:BTX0CDe=06ATI018B4P0ADE505QHF00o?cl0A4A40:6QX@2ATI40UIFE05IFEP1N
GUh0@T9202@T901iNGT0OWin04A4A024Q8@0CDe=04a<C01aLG40S8b<0492@P1QHF40KVi^06UYJ@21
PH40[:b/065QH@12@T80EEEE07=cL`1>CTh0HF5Q05=CD`0M7Ad0EUIF05mOG`1nOWh0a/K606=SH`1O
Gel0[:b/07QhN00S8b<0GemO04];B`0];Bd0;bl_03hn?P1fMWH0>SXj0P2b/[806P1OGel0>CTi06i^
KP1JFUX0I6AT04m?C`1GEeL0=3@d05YJFP1EEED0KFe]075aL@0J6QX0JVYZ0<;2`P1MGEd0Hf=S0<?3
``19BDT0BTY:08R8R02DU9@0HV9R02HV9P0c<c<0J6QX3_ooool04`3@d=00f=SH0>OWi`3Lg=`0a<C4
071`L00d=3@0<S8b0:FUY@38b<P0d=3@0<c<c02<S8`0Cdm?0451@@12@T80=CDe01/K6`0D51@00P0E
5AD03`1^KVh0Cdm?00h>3P0>3Ph0410@014A4@0C4a<0DE5A04i>CP0A4A409BDU05QHF02n_[h0W9bL
02HV9P0301DE5@0401<C4`0E5AD05ADE018B4P80410@01`071`L04];B`24Q8@0GUiN04U9B@0D51@0
410@01<C4`0D51@04a<C01DE5@0L71`0MGEe06i^KP2=SHd0UiNG0<71`@1gMgL0EUIF09ZJVP13@d<0
=CDe02l_;`1?Cdl0ADE502HV9P0/;2`05aLG0P0E5AD0?@0G5aL08B4Q05IFEP1ZJVX0IfMW069RHP1e
MGD0aLG50;2`/02YZJT0O7al071`L00g=cL07QhN01TI6@0L71`06a/K01hN7P0L71`07alO05=CD`11
@D40FEUI07QhN017AdL0@41005YJFP1oOgl05QHF01LG5`0Q8B407alO01hN7P0];Bd0>CTi01DE5@0G
5aL05QHF02@T902@T900LG5a0:6QX@15ADD0Bd];0<k>cP2HV9P0IVIV09>CT`1SHf<0@d=303Lg=`0b
<S80;bl_05aLG020P800bl_;0=GEe@3Ng]h0`/;207UiN@0U9BD00P0E5AD03P0I6AT0FUYJ06IVIP1D
E5@0OGem0:ZZZP2MWId0gmoO08B4Q01oOgl0L71`06IVIP1cLg<0IFEU0P1]KFd04`1gMgL0QHF506a/
K02IVIT0X:2P0:FUY@3=cLd0PH6101`L700P8200:2PX03Ph>00O7al07AdM06UYJ@0g=cL0;2`/030`
<01MGEd00P1CDe<03P1:BTX0?Cdm04a<C01iNGT0DU9B04e=C@2BTY80XZ:R07emO@1dM7@0IfMW07Ui
N@2DU9@06ATI0P0E5AD05P0D51@05ADE01TI6@0N7Qh0;2`/02LW9`1_Kfl0H61P05mOG`1LG5`0CTi>
05YJFP0h>3P06ATI01<C4`0A4A405ADE038b<P0g=cL0JFUY08^;R`1:BTX201LG5`0H01lO7`1eMGD0
_Kfm04a<C00Z:RX0<c<c05QHF00[:b/06ATI01/K6`0N7Qh0E5AD0=;BdP1nOWh0<C4a02<S8`19BDT0
E5AD07alO01VIVH05aLG01DE5@0A4A404Q8B0P0E5AD02P0`<300=SHf065QH@1IFET0PH6105QHF00^
;Rh05ADE01/K6`0I6AT201DE5@0@01XJ6P0o?cl08b<S02LW9`0n?Sh0F5QH02XZ:P1LG5`0=3@d030`
<00G5aL06a/K01`L700W9bL0=CDe03/k>`8071`L00L06QXJ01/K6`0L71`0<S8b08F5Q@2KVi/0clo?
013ooooo000>ooooo`2K07YjNP1QHF40QXJ605iNGP1>CTh0EEEE08>3P`0a<C40=cLg07=cL`0d=3@0
C4a<03Xj>P1;Bd/0Q8B403<c<`1LG5`0?Shn06YZJP1QHF40:b/[07YjNP15ADD0IFEU071`L00_;bl0
DE5A06MWI`0P8200C4a<04=3@`1ZJVX0Fe]K01hN7P1LG5`0GEeM05IFEP0m?Cd0A4A4034a<@1?Cdl0
=SHf01HF5P1EEED0:b/[05=CD`10@4006ATI03Ph>01TI6@0<C4a07emO@0`<3007alO0820P01IFET0
?clo079bLP17AdL0?Cdm06MWI`1JFUX0BDU907UiN@10@400:2PX06MWI`14A4@0820P07UiN@0j>SX0
=3@d05MGE`0/;2`0KVi^04I6AP1aLG40<S8b02LW9`1SHf<0SHf=04i>CP1;Bd/0U9BD03@d=01CDe<0
Lg=c09^KV`16ATH0?Cdm0410@017AdL0FUYJ09:BTP2@T900FUYJ069RHP0/;2`0Ph>307]kN`1QHF40
C4a<05eMG@1^KVh0Cdm?05]KF`0f=SH0?3`l0820P01LG5`0IVIV03lo?`1hN7P0I6AT04M7A`1YJFT0
>c/k059BDP1bLW80HF5Q05EEE@1IFET0CTi>065QH@1TI6@0CDe=01dM7@1KFe/0O7al05=CD`0d=3@0
AdM707AdM01hN7P0;bl_05mOG`0`<300Kfm_05=CD`0/;2`0CTi>07MgM`0c<c<0L71`04i>CP0e=CD0
Hf=S059BDP0a<C40N7Qh05=CD`0l?3`0Ph>304U9B@1_Kfl00P1GEeL0;P13@d<0?Shn06MWI`3@d=00
LW9b03/k>`1]KFd0F5QH038b<P1cLg<0Y:BT05iNGP1^KVh0YJFU06YZJP0];Bd0VIVI07IfMP1IFET0
Ti>C03hn?P0j>SX0Q8B407YjNP0d=3@0?Cdm06UYJ@1lO7`0KVi^069RHP2EUID0@d=309NGU`1BDU80
D51@08V9R@1GEeL0G5aL08Z:RP0n?Sh0EUIF04U9B@1IFET0<S8b06QXJ01fMWH205=CD`190;Rh^01l
O7`0MGEe05=CD`1oOgl0=cLg02d];@27QhL0IVIV04Y:BP27QhL0EEEE04m?C`1JFUX0L71`03<c<`24
Q8@0Jf][061PH028R8P0/;2`04Q8B00b<S80Ph>308:2PP1@D500IfMW03De=@0S8b<0GUiN04e=C@22
PX80bLW904I6AP1]KFd0[jn_079bLP0U9BD0I6AT03`l?01CDe<0ATI603/k>`1RHV80>CTi0<W9b@36
a/H0NGUi04e=C@1kNg/0I6AT08N7Q`1KFe/0K6a/04I6AP1VIVH0G5aL06i^KP1QHF40@41003De=@1b
LW80Jf][05iNGP30`<009bLW02@T9011@D40HV9R06MWI`0Z:RX0=CDe06MWI`0>ooooo`0G0:BTY03D
e=@0i^KV0>[ZjP3WinL0g=cL0<S8b033`l<0h^;R0=;BdP34a<@0[Zj^04I6AP1EEED0Ng]k071`L01T
I6@0D51@03`l?00L71`05ADE09:BTP1]KFd00P0F5QH02`0P8200=CDe06MWI`2>SXh0X:2P05aLG01o
Ogl0B4Q80;Bd]03:b/X07QhN00H05ADE00T04a<C010@400B4Q80@d=304i>CP1KFe/0:2PX04m?C`0b
<S800P0E5AD0D`0A4A404a<C01DE5@0R8R807QhN04Q8B03=cLd0]kNg09jNWP1mOGd0EeMG05YJFP1d
M7@07alO01DE5@0G5aL0?Shn04i>CP0W9bL0;2`/02HV9P0f=SH0ADE504A4A00N7Qh06QXJ08^;R`3A
dM40/;2`0;Rh^02SXj<0RHV905iNGP1TI6@0NGUi06ATI010@400BTY:04Q8B01>CTh0Cdm?03Xj>P11
@D40L71`07IfMP34a<@0/k>c096AT@2HV9P0WYjN04Y:BP0P8200:b/[03Xj>P19BDT0WIfM06=SH`10
@400ATI604m?C`2<S8`0[Jf]0<_;b`3Cdm<0`<300<o?c`3EeMD0`/;20;jn_P2PX:00IFEU08^;R`28
R8P0FUYJ07=cL`2IVIT0VYZJ08j>SP1gMgL0fm_K04U9B@0G5aL05QHF00<05ADE03807alO05YJFP2?
Shl0f=SH0;:b/P35aLD0dM7A05]KF`1QHF40CDe=04I6AP0f=SH08B4Q01`L700K6a/07alO04I6AP2Z
ZZX0^KVi0=7Ad@3<c<`0clo?0;Jf]P1NGUh0F5QH06EUI@1SHf<0J6QX04];B`1mOGd0KFe]02PX:00a
<C40;2`/04m?C`0k>c/0G5aL03<c<`2:RXX0F5QH03De=@1>CTh0QHF50;:b/P2c/k<0ZZZZ0:^[Z`2d
];@0Y:BT024Q8@<05ADE00h06ATI01`L700P8200@41004M7A`1QHF40@T920861P@18B4P0:BTY02`/
;00o?cl0DU9B0410@0805ADE00D0>c/k096AT@2^[Zh0L71`03De=@0201<C4`0o01hN7P2JVYX0aLG5
06][J`0d=3@0=SHf03`l?01kNg/0H61P02TY:@0W9bL0VIVI0;6a/@1eMGD0;bl_03Ph>01LG5`0=cLg
07AdM016ATH0CTi>0451@@0U9BD05ADE01PH600E5AD0;bl_02PX:01@D500Lg=c0:VYZ@1ADE4061PH
01DE5@0F5QH06ATI01/K6`0H61P071`L05IFEP0a<C4092@T0410@01eMGD0>CTi05eMG@0i>CT0;Bd]
01DE5@0K6a/08b<S03@d=00j>SX0=3@d01`L700I6AT071`L01/K6`0M7Ad0;Bd]06e]K@2PX:00dM7A
013ooooo000>ooooo`1i0:>SX`1LG5`0V9RH06MWI`1@D500DU9B08F5Q@1:BTX0@D51079bLP0n?Sh0
IfMW07inOP0k>c/0GemO02d];@1MGEd08b<S03`l?00d=3@0>3Ph06m_K`11@D40?3`l04=3@`0`<300
E5AD06QXJ00_;bl0Cdm?03De=@1[Jf/0H61P030`<01]KFd0G5aL03dm?@19BDT0?clo03Lg=`1aLG40
?clo01dM7@1@D500:b/[04Y:BP18B4P0;bl_03/k>`1DE5@0<S8b09bLW012@T808b<S096AT@1@D500
>SXj075aL@1=CDd0>c/k05mOG`1EEED0<30`05mOG`17AdL09BDU06ATI01KFe/0=CDe08J6QP19BDT0
>3Ph06e]K@0a<C40Jf][04i>CP24Q8@0A4A402/[:`1;Bd/0EEEE065QH@1CDe<0T92@038b<P14A4@0
FUYJ09NGU`16ATH0@41004];B`1:BTX0CTi>07IfMP2GUiL0Hf=S07EeM@10@400MgMg06ATI01IFET0
OWin06e]K@1SHf<0DU9B05ADE01VIVH0Fe]K07emO@1BDU80U9BD07UiN@2KVi/0GEeM04U9B@1bLW80
D51@04];B`1eMGD0IFEU05ADE00206m_K`2K05ADE01YJFT0H61P03Xj>P1bLW80SHf=06MWI`0n?Sh0
KVi^08Z:RP1WIfL08B4Q06YZJP0g=cL0Q8B403lo?`0`<300IFEU08R8R00l?3`0Jf][04e=C@0o?cl0
IfMW05MGE`0Z:RX0K6a/051@D00g=cL0C4a<05=CD`1PH600EUIF07UiN@1RHV80IFEU07EeM@3Fe]H0
FUYJ04=3@`2;Rh/0N7Qh03`l?01GEeL0YZJV06ATI01kNg/0/k>c07moO`0f=SH0^k^k06QXJ01NGUh0
R8R803Lg=`0X:2P0XZ:R08f=S@0m?Cd0CDe=03Ti>@1HF5P0DE5A071`L01lO7`0DU9B071`L00a<C40
EUIF07alO018B4P0@41006a/K00o?cl0E5AD04];B`1YJFT0EEEE04E5A@1RHV80H61P06IVIP35aLD0
X:2P08B4Q01HF5P0KFe]03Ti>@0X:2P0RXZ:04e=C@12@T80S8b<05=CD`1=CDd0GUiN07=cL`0`<300
IFEU05iNGP1@D500T92@0;Zj^P1CDe<0:RXZ06YZJP1RHV80CTi>075aL@1;Bd/09bLW04A4A00k>c/0
M7Ad0<O7a`1LG5`0J6QX0:VYZ@1XJ6P0:b/[07AdM01EEED0JFUY04E5A@1GEeL0K6a/03Ph>02;Rh/0
[Jf]06=SH`1MGEd0Shn?030`<01cLg<0E5AD04M7A`1>CTh0Ogmo04i>CP1gMgL0OGem03Ph>01>CTh0
UYJF06i^KP1SHf<0UiNG03dm?@0c<c<0GUiN06=SH`1IFET0<S8b04M7A`1aLG403_ooool0KP2JVYX0
a/K60<g=c@3WinL0cLg=0;bl_01hN7P0VIVI0=KFeP2`/;00LG5a09VIV@2d];@0MWIf04U9B@14A4@0
D51@071`L01oOgl0YZJV0:BTY02[Zj/0Kfm_04U9B@11@D40?clo03De=@0X:2P08B4Q03`l?015ADD0
PX:207EeM@37alL0eMGE07AdM01`L700GemO05aLG019BDT0@T9203`l?00h>3P0AdM705mOG`1BDU80
Cdm?028R8P0H61P0820P04Y:BP15ADD07alO01DE5@0C4a<061PH02@T900L71`0;Rh^09jNWP24Q8@0
FEUI04a<C015ADD0@T920410@01>CTh0<C4a01PH600P8200?clo07moO`1bLW80G5aL04E5A@0S8b<0
5aLG01dM7@1<C4`0MgMg04Q8B02=SHd0^[Zj0;2`/02DU9@0Q8B407]kN`1aLG40ADE503lo?`0k>c/0
=SHf03@d=00`<300<S8b03<c<`1lO7`0UYJF09fMW@1aLG40Shn?09VIV@2NWYh0];Bd08Z:RP2CTi<0
WIfM0::RXP3<c<`0Ogmo0P1TI6@03`1XJ6P0SXj>0<71`@35aLD0YJFU05iNGP1`L700^k^k0;jn_P12
@T806QXJ01`L700b<S80=cLg0492@P0206i^KP0I0;^k^`2;Rh/0De=C0;Fe]@0W9bL061PH01HF5P0G
5aL05QHF01LG5`0K6a/07AdM06ATI03De=@0h^;R0:NWY`2c/k<0FEUI03Hf=P0S8b<06ATI01lO7`0Y
:BT092@T01hN7P0201`L70070410@01dM7@0[Jf]0:JVYP2?Shl0LG5a03<c<`02030`<00U0410@00Y
:BT06a/K02`/;02NWYh0OGem03Lg=`0Y:BT0=3@d05iNGP2GUiL0VIVI05QHF016ATH0@41004Q8B011
@D40LG5a05QHF01<C4`0@d=303Xj>P1HF5P0EUIF01TI6@0E5AD092@T01hN7P0K6a/09bLW05MGE`1Y
JFT0Bd];05iNGP1dM7@0VIVI0492@P0202XZ:P1:03De=@1<C4`0EUIF06ATI02EUID0][Jf06m_K`10
@4007AdM01hN7P0E5AD0=CDe0:f][@3MgMd0f][J0=OGe`3=cLd0TI6A05mOG`1UIFD0UiNG0;Zj^P34
a<@0`/;209JFUP2GUiL0_Kfm0<o?c`2g]kL0Zj^[01LG5`0H61P09bLW03Ti>@0m?Cd0B4Q804U9B@1a
LG40N7Qh09nOW`2ATI40Cdm?04A4A012@T805aLG01HF5P0I6AT06QXJ02/[:`13@d<0TY:B08N7Q`1b
LW80RHV90:VYZ@1>CTh0MgMg059BDP1@D500=SHf038b<P11@D40CTi>03Ti>@0P820071`L02`/;00V
9RH0<C4a04Q8B01QHF40P82009NGU`2e]KD@ooooo`003_ooool0A`2h^;P0IFEU09VIV@1]KFd0GEeM
05iNGP22PX80BDU904Y:BP29RHT0@D5106e]K@2;Rh/0@D5107YjNP0d=3@0OWin02TY:@17AdL0GUiN
03Ti>@2ATI40Fe]K0492@P1aLG40BDU9065QH@1LG5`09BDU05aLG015ADD0FEUI05=CD`0a<C40Hf=S
06IVIP13@d<0?clo03/k>`1<C4`0VYZJ03dm?@0I6AT0De=C03<c<`1<C4`0=CDe01dM7@0d=3@0LG5a
03Lg=`2KVi/0=SHf02DU9@1nOWh0EEEE03dm?@1aLG40@T9203<c<`1[Jf/0Fe]K0410@01QHF409bLW
01LG5`1XJ6P0JFUY03@d=01gMgL0?Cdm0080K6a/0240@D5106UYJ@0c<c<0RHV903/k>`0R8R80D51@
05ADE018B4P0F5QH096AT@0l?3`0EEEE05aLG02`/;00@D5104];B`1[Jf/0GEeM05]KF`27QhL0T92@
05mOG`1nOWh0?Shn08N7Q`1FEUH0Lg=c06e]K@1TI6@0ATI606ATI01MGEd00P19BDT0ZP1gMgL0?Cdm
06m_K`23Ph<0WinO04e=C@0o?cl0MWIf04Y:BP1ADE40KFe]04I6AP1TI6@0N7Qh079bLP18B4P0QHF5
06EUI@0e=CD0OWin07=cL`1kNg/0F5QH06i^KP28R8P0JVYZ02`/;024Q8@0=cLg08R8R01ADE40>3Ph
05iNGP1RHV80@41006i^KP1OGel0?clo08V9R@2ATI40>c/k07UiN@1_Kfl0D51@05EEE@1RHV80MgMg
05eMG@20P800M7Ad08F5Q@29RHT0e=CD05YJFP1FEUH0Z:RX08f=S@14A4@0F5QH09>CT`1?Cdl0JVYZ
09fMW@1iNGT0?Cdm0<;2`P1FEUH0IFEU08n?S`19BDT0=3@d0;Zj^P1VIVH0?clo04a<C019BDT0L71`
065QH@1/K6`0UiNG06UYJ@28R8P08b<S07YjNP2>SXh0D51@04Q8B01PH600@D5106e]K@1`L700Rh^;
0492@P1^KVh0M7Ad03`l?019BDT0Rh^;06=SH`1aLG40IVIV0:^[Z`15ADD0Bd];0:b/[01eMGD0F5QH
09bLW00h>3P0EEEE08N7Q`1HF5P09bLW06MWI`21PH40FEUI071`L02PX:00<S8b01HF5P26QXH0H61P
03@d=01RHV80?3`l02LW9`1EEED08R8R05eMG@2a/K40>CTi06][J`32`/80Fe]K01lO7`1MGEd0@d=3
06QXJ016ATH0OWin06m_K`0j>SX0P8200:^[Z`0n?Sh0E5AD07]kN`0f=SH0Q8B404Q8B01>CTh0@d=3
055AD@1HF5P0PX:207MgM`0H61P0EEEE0:JVYP2ATI40Kfm_09RHV00l?3`0=SHf08J6QP2BTY80Fe]K
030`<00Y:BT0EUIF3_ooool0<014A4@0^;Rh0=OGe`3QhN40Fe]K06ATI01lO7`0aLG50<o?c`3<c<`0
UYJF05iNGP2EUID0`<30075aL@0k>c/0E5AD05UIF@1OGel0QHF50<;2`P2YZJT0?3`l01HF5P0G5aL0
6a/K01XJ6P0F5QH06QXJ02XZ:P11@D40BTY:0;Jf]P3@d=00TI6A0:VYZ@29RHT0S8b<0:j^[P1dM7@0
P82007=cL`1PH600JVYZ07]kN`38b<P0DE5A01PH60805QHF02005aLG03Lg=`1ZJVX0<C4a02<S8`0m
?Cd0820P03Ph>01EEED0OGem09JFUP0k>c/07AdM01DE5@0/;2`0Jf][05eMG@1oOgl0L71`03<c<`0G
5aL05ADE03dm?@1=CDd0?3`l06UYJ@1FEUH0C4a<06UYJ@16ATH0RXZ:0:f][@80X:2P03H0OGem05eM
G@1:BTX0DE5A05iNGP14A4@0<C4a02XZ:P0^;Rh0=cLg04U9B@1RHV80GemO0=CDe01hN7P0ADE506QX
J02][Jd0_Kfm07UiN@2NWYh0clo?09FEU@39bLT0^KVi04m?C`0l?3`0>3Ph034a<@0];Bd0[jn_07Ee
M@2`/;00LG5a09RHV02d];@0[jn_06m_K`0F5QH06ATI01/K6`0I6AT07AdM038b<P1kNg/0UIFE0:FU
Y@25QHD0WinO04A4A00U9BD07QhN01TI6@0L71`201hN7P1705ADE027QhL0/[:b0;Fe]@1iNGT0PX:2
020P800N7Qh0820P01PH600K6a/06QXJ01TI6@0Z:RX092@T01/K6`0W9bL0@D5109VIV@2QXJ40HF5Q
0;fm_@0h>3P07AdM02TY:@0l?3`06a/K01XJ6P0L71`0=3@d06YZJP2JVYX0DU9B03Lg=`1<C4`0LG5a
09NGU`0g=cL05QHF01<C4`0E5AD05QHF028R8P1HF5P08b<S04=3@`1;Bd/0@41009>CT`1YJFT0=3@d
02PX:00I6AT08b<S069RHP1NGUh0;Rh^01hN7P0V9RH0KFe]0<[:bP1cLg<0M7Ad07QhN01bLW80QXJ6
09ZJVP3=cLd0e=CD0:j^[P1SHf<00P0K6a/05P0U9BD0@d=3071`L019BDT0Cdm?0;:b/P3Kfm/0j^[Z
0>OWi`3>c/h0a<C40=3@d033`l<0d=3@0:b/[02=SHd0@d=308n?S`32`/80GemO07AdM00d=3@201DE
5@0Y01@D500E5AD05ADE01XJ6P1EEED0Ng]k0::RXP1dM7@07alO03Ph>01ADE409RHV01lO7`0_;bl0
CDe=05IFEP18B4P0FUYJ0;fm_@2BTY80UiNG0<K6aP2SXj<0YJFU09VIV@2YZJT0WinO08B4Q02;Rh/0
CDe=02PX:00W9bL0:2PX065QH@1eMGD0HV9R02l_;`0i>CT0LG5a09RHV01kNg/04?ooool000kooooo
0?`0`<3006UYJ@27QhL0JFUY05MGE`1PH600M7Ad038b<P15ADD0Lg=c03hn?P1hN7P0Ogmo03lo?`1o
Ogl0;bl_07]kN`0R8R80De=C04];B`0k>c/0T92@051@D011@D40Fe]K04E5A@1[Jf/0E5AD02TY:@1Z
JVX0@D51051@D014A4@0AdM708V9R@1ZJVX0A4A403Ph>00e=CD0ADE508:2PP0_;bl07alO05ADE00e
=CD0FEUI02l_;`0E5AD0=CDe079bLP0a<C40SXj>02DU9@0j>SX0SHf=05IFEP0n?Sh0HF5Q0492@P1B
DU80QhN705YJFP14A4@0LW9b03Ph>00U9BD0GEeM05]KF`0i>CT0QHF503hn?P1GEeL0HF5Q04I6AP21
PH40=3@d07alO00b<S808R8R05]KF`1HF5P0GEeM05EEE@26QXH0<C4a06YZJP2DU9@0[:b/03Ph>00i
>CT0EeMG03/k>`1aLG40Y:BT09bLW01`L700PH6103Hf=P2RXZ80MGEe071`L01@D500OGem06ATI01>
CTh0G5aL038b<P0_;bl0MGEe04e=C@13@d<0A4A4096AT@1^KVh0=CDe05MGE`15ADD0JFUY09FEU@1R
HV80Cdm?055AD@1NGUh0IFEU05iNGP1<C4`0=SHf09RHV01kNg/0GemO02HV9P1ZJVX0ZZZZ07inOP0V
9RH0GUiN03Xj>P2JVYX0J6QX03/k>`23Ph<0GUiN03@d=01kNg/0H61P03dm?@1GEeL0CDe=034a<@1d
M7@0=cLg05=CD`1FEUH0GEeM07]kN`14A4@0Jf][06MWI`1[Jf/0I6AT0<o?c`1<C4`0EUIF09nOW`26
QXH0?3`l04Y:BP2@T900IfMW06e]K@2XZ:P0D51@03@d=039bLT0Ogmo06UYJ@2?Shl07AdM03De=@2Y
ZJT0MGEe0492@P1BDU80L71`09JFUP1UIFD0Ogmo0:2PX01FEUH0V9RH04M7A`1OGel0IFEU02PX:01/
K6`0Ogmo03Ti>@19BDT0BTY:06][J`0];Bd0JVYZ0861P@1LG5`0J6QX0:JVYP21PH40KVi^04];B`1f
MWH0;bl_034a<@1jNWX0;Rh^06MWI`2YZJT0Cdm?03hn?P1gMgL0GemO02PX:01=CDd0QhN706=SH`1d
M7@0YZJV07inOP0_;bl0L71`069RHP0b<S80KVi^038b<P0M7Ad0EUIF034a<@1?Cdl0[Zj^01hN7P0n
?Sh0PH6106MWI`0M7Ad0G5aL05ADE016ATH0;Rh^08V9R@2=SHd0ATI609:BTP2b/[8207=cL`0H09FE
U@19BDT0[Jf]08b<S01>CTh0@d=305IFEP1LG5`0O7al08F5Q@0`<300?Shn07YjNP2:RXX0K6a/0;Bd
]00Q8B40;Bd]055AD@1YJFT0GEeM03Lg=`0a<C40J6QX3_ooool0=`13@d<0SXj>07YjNP31`L40WYjN
0;^k^`3=cLd0];Bd07=cL`2m_Kd0]kNg07MgM`1NGUh0SHf=0;:b/P1@D5005aLG01<C4`0E5AD09BDU
07moO`27QhL0?3`l03dm?@0T92@06QXJ01LG5`0F5QH05ADE02l_;`0H61P0EUIF0=?Cd`2][Jd0PH61
07MgM`0P82008B4Q02l_;`0c<c<0>CTi03Lg=`0c<c<0<30`02h^;P1gMgL0QhN704i>CP0P82005aLG
01DE5@0I6AT0?Shn09ZJVP1LG5`00P0a<C401P1HF5P0QHF508j>SP1VIVH0?clo02l_;`805ADE00@0
9BDU07MgM`1?Cdl0Fe]K0P0C4a<0B`0A4A405ADE03Xj>P1XJ6P0KFe]02`/;016ATH0Q8B405=CD`1d
M7@0NWYj0:2PX01iNGT0LW9b04Q8B019BDT0EEEE06MWI`1TI6@0DE5A03Xj>P0_;bl0:BTY02PX:01D
E5@0L71`0=cLg022PX80;bl_03Ti>@1RHV80d=3@0:JVYP2CTi<0WYjN0:f][@2BTY80MgMg07YjNP10
@400;bl_024Q8@0^;Rh0Z:RX061PH01SHf<0aLG508f=S@1eMGD0SHf=09FEU@0Y:BT06a/K01hN7P0Y
:BT0@41005ADE02MWId0QHF504E5A@1NGUh0S8b<04A4A01PH600BDU903/k>`0g=cL0820P024Q8@1L
G5`0X:2P0:RXZ02DU9@0MGEe07MgM`0201dM7@0301XJ6P0G5aL06ATI00805ADE00l0820P030`<00N
7Qh0:2PX04i>CP2TY:@0J6QX03De=@2XZ:P0HF5Q01dM7@0/;2`0<S8b02<S8`0M7Ad00P0I6AT02P0[
:b/0M7Ad0::RXP2BTY80[Zj^08J6QP1KFe/08B4Q018B4P0A4A4201DE5@1P024Q8@2PX:00IVIV04a<
C01IFET0IVIV09RHV02BTY80J6QX07inOP1eMGD0QXJ609bLW00e=CD0<C4a03hn?P0];Bd0BTY:08^;
R`1MGEd0@T9204I6AP18B4P0GEeM0:b/[039bLT0h>3P0<S8b01TI6@07AdM03<c<`1bLW80GEeM01/K
6`0H61P09BDU03Xj>P2[Zj/0gmoO0=GEe@2:RXX0G5aL0:RXZ032`/80[Jf]02PX:01VIVH0L71`05YJ
FP2HV9P0B4Q804M7A`1MGEd05aLG01PH600O7al0=SHf04e=C@1FEUH0RHV909FEU@2MWId0MgMg01TI
6@14A4@0EeMG07=cL`1UIFD0Fe]K03lo?`0l?3`0<C4a03Hf=P1FEUH0HF5Q0:^[Z`2j^[X0QhN70:JV
YP2QXJ40^;Rh0;fm_@3>c/h0[:b/03@d=00h>3P0EEEE05UIF@16ATH09bLW024Q8@0W9bL0FEUI08V9
R@1^KVh0R8R84?ooool000kooooo01<0[:b/07QhN026QXH0DE5A03Ti>@1KFe/0Q8B403lo?`1;Bd/0
EUIF04m?C`2CTi<0EUIF061PH02=SHd0;Rh^079bLP0Q8B40J6QX0080BDU904<0Rh^;04Y:BP11@D40
EEEE0492@P1[Jf/0>c/k02LW9`1:BTX0CTi>07alO00_;bl0<C4a07EeM@0k>c/0De=C02l_;`15ADD0
;bl_0820P00U9BD0:BTY05YJFP0b<S80M7Ad04U9B@0I6AT0CDe=06EUI@0[:b/0R8R803Ti>@0e=CD0
RHV903Lg=`1EEED0IVIV03@d=00^;Rh0EUIF04Y:BP0h>3P0Hf=S04Q8B00f=SH0LG5a04Y:BP0c<c<0
KFe]03Hf=P1XJ6P0A4A404Q8B01UIFD0Bd];08j>SP0m?Cd0;bl_06a/K01SHf<0GemO05MGE`1FEUH0
71`L05IFEP1/K6`0UYJF0080?3`l0;40E5AD04I6AP1fMWH0OGem08B4Q01kNg/0JVYZ034a<@1fMWH0
EeMG06EUI@1JFUX0NGUi05EEE@1=CDd0IfMW07EeM@17AdL0M7Ad04a<C02DU9@0MGEe09BDU01bLW80
>c/k05IFEP0o?cl0F5QH07IfMP1TI6@0F5QH06UYJ@1`L700FEUI05aLG01RHV80=3@d07MgM`2<S8`0
Hf=S03<c<`1eMGD0Shn?06MWI`0Q8B40IFEU034a<@1lO7`0DU9B05IFEP2ZZZX0G5aL03Xj>P1hN7P0
CTi>03Xj>P1YJFT0DU9B024Q8@2BTY80Bd];05=CD`14A4@0FEUI07]kN`0l?3`0K6a/05=CD`1LG5`0
EUIF0;6a/@16ATH0M7Ad09nOW`26QXH0@D5107AdM02e]KD0GUiN061PH02WYjL0CDe=030`<02WYjL0
D51@06m_K`28R8P0BTY:03/k>`2^[Zh0PH6107inOP1LG5`0=SHf05UIF@18B4P0[Jf]07MgM`1aLG40
Lg=c0451@@1gMgL0QhN703/k>`1BDU80KVi^03Hf=P19BDT0DE5A06UYJ@15ADD0Z:RX08^;R`1NGUh0
WIfM0;no_`2a/K40IFEU05=CD`1dM7@0;Rh^03hn?P20P800<C4a03hn?P1YJFT0CDe=0451@@2EUID0
Cdm?02HV9P1@D500DE5A03Lg=`2JVYX0c/k>07EeM@0X:2P0Ogmo06e]K@13@d<0LG5a08N7Q`15ADD0
GEeM02l_;`1LG5`0/K6a06IVIP1UIFD0MgMg04a<C00M7Ad0H61P04e=C@0e=CD0>c/k07IfMP1gMgL0
@d=30<G5a@3AdM40HV9R07IfMP2=SHd08R8R08F5Q@21PH40Cdm?04M7A`1>CTh0?clo09NGU`2UYJD0
=cLg04U9B@1gMgL00P1hN7P02@2FUYH0AdM702/[:`1>CTh0H61P059BDP1<C4`0IfMW06a/K00>oooo
o`0H05mOG`1jNWX0NGUi0<g=c@36a/H0W9bL0820P02TY:@0ADE508b<S02XZ:P0BTY:051@D01RHV80
Rh^;0:NWY`0X:2P03Ph>010@400C4a<0BDU906i^KP15ADD08B4Q0P0c<c<02`0b<S80>SXj03<c<`0]
;Bd0BDU90;Vi^@3<c<`0^KVi0:6QX@0e=CD0:BTY00805QHF0P0G5aL09P0C4a<03Ph>010@400C4a<0
8b<S06=SH`1gMgL0F5QH01hN7P0E5AD071`L055AD@1eMGD0UIFE06UYJ@1[Jf/0]kNg0;Jf]P1aLG40
:BTY01TI6@0`<300820P01LG5`0D51@0=cLg02DU9@0d=3@051@D01<C4`0G5aL0>c/k07EeM@1oOgl0
IfMW03lo?`2`/;00[:b/0P2EUID09@2XZ:P0?Cdm03Lg=`1;Bd/0SXj>0;2`/02QXJ40PX:206IVIP1@
D500EeMG04a<C00n?Sh0<C4a07emO@31`L40e]KF0:^[Z`0l?3`0B4Q809fMW@2h^;P0WYjN0<G5a@3I
fMT0IVIV06=SH`1TI6@0D51@05YJFP1YJFT0Jf][07MgM`3@d=00G5aL03`l?02ATI400P1^KVh0801g
MgL0QXJ606i^KP0e=CD0GEeM05iNGP15ADD08B4Q05]KF`1hN7P0?Cdm03`l?021PH405ADE01LG5`15
ADD0H61P05QHF02FUYH0R8R8092@T01<C4`0WinO07=cL`1@D500MWIf01TI6@0J6QX05aLG01HF5P0E
5AD05QHF0P0G5aL06@0U9BD0:2PX03/k>`1dM7@0ZJVY05iNGP0^;Rh0NWYj08b<S015ADD0EUIF01hN
7P0E5AD05aLG01hN7P0O7al09BDU03Ti>@2g]kL0Vi^K09ZJVP2TY:@0Bd];01DE5@0C4a<00`0E5AD0
2@0W9bL0De=C03@d=01SHf<0K6a/08F5Q@2ZZZX0^k^k07=cL`02096AT@0@0=3@d01lO7`0=cLg02h^
;P14A4@0@T92075aL@11@D408B4Q020P800/;2`0?Shn05aLG02ATI40^k^k0<W9b@80ZZZZ03`0IfMW
07AdM010@400:2PX01LG5`0E5AD06QXJ02LW9`11@D40_;bl07QhN015ADD0DE5A09jNWP2IVIT0TI6A
061PH01OGel0Rh^;07QhN00l?3`0EUIF0410@01jNWX0Kfm_07AdM01bLW80EeMG03/k>`0^;Rh0FUYJ
0;Zj^P3Fe]H0Y:BT02l_;`1=CDd0Zj^[09fMW@1=CDd0<S8b028R8P0L71`07alO02HV9P1VIVH0L71`
03Ti>@1NGUh0alO70;^k^`2i^KT0GemO06e]K@2DU9@0XZ:R07IfMP1_Kfl0CDe=02PX:00U9BD202@T
90050410@01^KVh0H61P07=cL`1=CDd04?ooool000kooooo00/0R8R8079bLP25QHD0GUiN04Y:BP1P
H600OGem03Xj>P1DE5@0OGem02l_;`0205eMG@0T03Xj>P1FEUH0<C4a051@D00i>CT0MWIf03`l?01;
Bd/0GUiN04E5A@1BDU80B4Q804A4A01mOGd0IFEU024Q8@12@T80@41005iNGP15ADD0;Rh^07YjNP13
@d<0?3`l02XZ:P0/;2`0;Bd]06=SH`0e=CD05aLG051@D00R8R80?3`l03dm?@0/;2`0?clo0P0i>CT0
?025QHD0=SHf03Ph>01aLG407alO03dm?@1OGel0;bl_02d];@1FEUH0G5aL05EEE@1/K6`0@41002<S
8`1cLg<0H61P04i>CP1lO7`0;bl_06i^KP1@D500IFEU06=SH`1<C4`0Ti>C034a<@0d=3@0HF5Q05EE
E@1?Cdl0G5aL06=SH`0c<c<0MWIf06ATI020P800=3@d0451@@1ADE40?Shn06ATI01WIfL0Ogmo08F5
Q@1`L700>c/k07IfMP17AdL0GUiN065QH@1iNGT0?clo04Q8B01XJ6P0G5aL04a<C01SHf<0@41008F5
Q@80PX:20900EUIF03Ti>@1=CDd0Bd];06IVIP1mOGd0Fe]K05MGE`1`L700IFEU04i>CP1TI6@0HV9R
03lo?`22PX80QhN706EUI@0l?3`0ZZZZ09NGU`18B4P0AdM7096AT@0^;Rh0IVIV02TY:@1@D500Ti>C
06=SH`0l?3`0Hf=S06][J`13@d<0JFUY06=SH`0S8b<0Ti>C04Y:BP1VIVH0FEUI06YZJP2ATI40CDe=
07IfMP1SHf<0J6QX03dm?@2;Rh/0Bd];079bLP2<S8`0L71`0410@023Ph<0];Bd05iNGP1aLG40^[Zj
0820P00i>CT0_[jn059BDP1oOgl0U9BD04m?C`12@T80^[Zj05aLG01@D500CDe=030`<01/K6`0@D51
05UIF@1`L700Ng]k0820P00R8R80L71`07IfMP1@D500FUYJ06m_K`1SHf<0Ti>C0:JVYP2@T900CDe=
06][J`1PH600KVi^09ZJVP3EeMD0[Zj^05IFEP1JFUX0I6AT02/[:`1bLW80R8R804I6AP18B4P0M7Ad
03<c<`1;Bd/0]kNg069RHP0d=3@0Cdm?05UIF@1ADE40Jf][08f=S@1RHV808B4Q06UYJ@0[:b/0HF5Q
09:BTP1`L700@T9204=3@`0`<300R8R809:BTP1:BTX0H61P08F5Q@1=CDd0820P07EeM@1MGEd0RHV9
04a<C020P800Bd];02XZ:P2VYZH0/K6a03Lg=`1`L700/K6a03Xj>P2>SXh205ADE00C06EUI@1`L700
ATI607]kN`28R8P0:RXZ05EEE@22PX80>SXj05EEE@2?Shl0De=C02d];@1DE5@0KFe]034a<@1RHV80
IfMW05ADE00>ooooo`0L04Q8B01hN7P0C4a<0:NWY`20P800JFUY06ATI00_;bl06a/K06e]K@3GemL0
QXJ604A4A00f=SH0@41009ZJVP1hN7P08B4Q01`L700S8b<0?clo07EeM@2IVIT0P82007QhN01dM7@0
LW9b07QhN080QXJ600d0`l?30=[JfP2b/[80]kNg06IVIP0^;Rh0>SXj020P800X:2P0=cLg02HV9P0C
4a<05aLG00<05ADE05L0@d=303`l?01[Jf/0C4a<03Hf=P1SHf<0E5AD061PH02ZZZX0YjNW0:j^[P33
`l<0a<C4065QH@0K6a/05ADE01`L700h>3P0:2PX01XJ6P0b<S808B4Q034a<@0E5AD0>3Ph059BDP0n
?Sh0;Bd]02<S8`2PX:00M7Ad07YjNP2RXZ80UYJF0;^k^`2b/[807alO03<c<`0Q8B40@T9208Z:RP0f
=SH0?Cdm05UIF@1lO7`0N7Qh07inOP2g]kL0_;bl0<G5a@2d];@0GUiN09>CT`2l_;`0ZJVY0=?Cd`2d
];@0aLG50;Ng]`2MWId08b<S02DU9@0d=3@0C4a<04I6AP10@400=CDe04M7A`25QHD0^;Rh055AD@1K
Fe/0Q8B404e=C@1ADE40IfMW09VIV@2EUID0>CTi01lO7`0I6AT05QHF03De=@1]KFd0?Shn0410@01e
MGD00P0E5AD0;00F5QH09RHV05]KF`3Fe]H0[jn_02d];@0_;bl0K6a/06IVIP1LG5`0Z:RX02d];@0J
6QX06ATI01DE5@0F5QH05aLG01DE5@0I6AT071`L05]KF`24Q8@0^[Zj09fMW@2<S8`0JVYZ0861P@39
bLT0XZ:R09nOW`1dM7@07AdM01DE5@0X:2P0?Cdm05ADE02BTY80/K6a05EEE@0i>CT0NWYj07IfMP0I
6AT04A4A0P0E5AD08`0H61P0=3@d01`L700E5AD05aLG07]kN`2LW9`0/K6a0=WIf@34a<@0`l?30=GE
e@3MgMd0Rh^;0861P@0^;Rh0A4A4059BDP1JFUX09bLW01@D500E5AD061PH01lO7`0Y:BT0>CTi06EU
I@2PX:00WIfM0:JVYP1jNWX0:2PX01dM7@0N7Qh06a/K00805ADE01D05aLG02/[:`1jNWX0GemO04a<
C01EEED0Ti>C08:2PP1aLG405aLG01DE5@0I6AT0MGEe09FEU@2=SHd0TI6A0<K6aP2YZJT0Kfm_08j>
SP1WIfL00P1mOGd09@2IVIT0IfMW09>CT`3>c/h0];Bd06m_K`24Q8@0Kfm_02`/;00R8R8071`L01dM
7@0O7al0D51@03Ph>00X:2P0;2`/06][J`1oOgl0^[Zj0;bl_01VIVH0AdM705YJFP1]KFd0S8b<04Q8
B00m?Cd0=cLg03dm?@1<C4`0>CTi07UiN@1]KFd0LG5a04i>CP1=CDd04?ooool000kooooo09L0]KFe
07AdM02=SHd0Ng]k05=CD`1XJ6P0O7al04A4A01^KVh0SHf=03Lg=`1[Jf/0JFUY0492@P1XJ6P0:RXZ
06ATI00Q8B40DU9B03dm?@1/K6`0N7Qh03lo?`0h>3P0;Bd]03Lg=`1IFET0H61P02PX:01OGel0A4A4
05ADE00m?Cd0GUiN0:FUY@1RHV80@410038b<P0`<300@D5106=SH`0n?Sh071`L05MGE`0n?Sh0ADE5
02l_;`0K6a/0Bd];05iNGP0g=cL0Kfm_02TY:@10@400Lg=c03<c<`14A4@0DE5A02d];@0f=SH0HV9R
04I6AP1=CDd0HV9R02d];@0J6QX0H61P05aLG00l?3`0Lg=c02PX:023Ph<0D51@06ATI01EEED0BDU9
09ZJVP0m?Cd09bLW05ADE01>CTh0EEEE079bLP1lO7`0=cLg079bLP1RHV80Ogmo02`/;01VIVH0Ng]k
051@D01<C4`0F5QH07MgM`2ATI40QXJ603Xj>P1gMgL0BDU908F5Q@1^KVh0IfMW03De=@1aLG40NGUi
05YJFP18B4P0Jf][04M7A`1oOgl0LW9b08Z:RP1DE5@0H61P06UYJ@1<C4`0JFUY07alO01NGUh0TY:B
08n?S`1LG5`0D51@0;2`/01]KFd0=cLg07inOP1MGEd0RHV905mOG`2XZ:P0UYJF04m?C`0Q8B40N7Qh
034a<@1]KFd0De=C03/k>`1lO7`0IVIV04I6AP1XJ6P0CTi>04=3@`2FUYH0YZJV02/[:`26QXH0Fe]K
0080JFUY02/0C4a<0:RXZ01DE5@0NWYj06IVIP1?Cdl0GEeM0:BTY01HF5P0NGUi0:f][@1?Cdl0=CDe
05QHF02MWId0FUYJ096AT@2UYJD0Hf=S03Lg=`33`l<0NWYj07=cL`2?Shl0<S8b05MGE`2a/K40O7al
05=CD`18B4P09bLW061PH00n?Sh0K6a/07=cL`1PH600OWin02LW9`1^KVh0GemO03lo?`1MGEd0HF5Q
0080ADE50500>CTi06MWI`11@D40GemO059BDP1SHf<0NGUi0:>SX`2@T900EeMG061PH01MGEd092@T
07inOP1iNGT0CTi>04Y:BP1lO7`0>CTi03dm?@2NWYh0F5QH034a<@1_Kfl0PX:205=CD`24Q8@0/K6a
03dm?@0N7Qh0RHV905aLG00k>c/0JFUY04M7A`12@T80D51@02d];@2ATI40PH6104E5A@1/K6`0ZJVY
04Q8B00c<c<0R8R805UIF@1XJ6P0@D5108j>SP1SHf<0;Rh^08B4Q02XZ:P0<C4a07QhN02/[:`0=cLg
09^KV`1GEeL0<30`05EEE@1fMWH0EeMG07YjNP1_Kfl08B4Q05EEE@22PX80QhN705eMG@2UYJD0=SHf
03Ph>01dM7@0RXZ:05=CD`0k>c/0ATI605aLG0kooooo02L0Fe]K05QHF011@D40K6a/08F5Q@1TI6@0
@41003Lg=`1>CTh0YZJV0=gMg@3AdM40[Jf]061PH00c<c<0CDe=0;:b/P38b<P0^KVi0:VYZ@2KVi/0
YZJV0:n_[`1EEED0@d=30410@00i>CT0=CDe02h^;P0a<C40IfMW0;Rh^02ZZZX0AdM704A4A01EEED0
>CTi07UiN@0_;bl00P0J6QX0@P0K6a/0:BTY03<c<`0V9RH05aLG01dM7@1@D500>SXj07moO`2XZ:P0
MWIf07AdM02MWId0V9RH06a/K01@D500VYZJ0;Zj^P1?Cdl08R8R01HF5P0I6AT092@T03Ph>00T92@0
?3`l0451@@12@T80AdM703/k>`0P82005aLG01HF5P10@400FUYJ07inOP1=CDd0UiNG07moO`1eMGD0
Ogmo02LW9`0c<c<09BDU02HV9P1FEUH071`L01hN7P0V9RH0BTY:0=7Ad@2`/;00Shn?07=cL`2JVYX0
LW9b03@d=00e=CD0NGUi0=gMg@25QHD0Rh^;0<W9b@1OGel0T92@01XJ6P805aLG01P06QXJ03De=@0U
9BD09BDU03<c<`0n?Sh0N7Qh0;^k^`2o_kl0NWYj04=3@`0W9bL0>CTi07emO@0S8b<05QHF01`L700I
6AT06QXJ02@T901KFe/0GemO03lo?`1QHF4201DE5@1401HF5P0E5AD05ADE08>3P`1VIVH07AdM01lO
7`1SHf<0NGUi0;Bd]02UYJD0/k>c09JFUP1FEUH0:b/[01`L700I6AT05ADE01`L700T92@0<S8b04U9
B@2JVYX0Ph>305=CD`1DE5@0];Bd06=SH`1fMWH0SXj>0:JVYP2:RXX0JVYZ024Q8@1@D500YJFU079b
LP0o?cl0PX:204=3@`1kNg/0K6a/061PH00J6QX04A4A01DE5@0H61P0<c<c01DE5@0D51@05ADE04U9
B@1@D500WIfM0=SHf02o_kl0Y:BT0<o?c`3@d=00]KFe07EeM@1;Bd/0De=C07MgM`0c<c<07AdM01DE
5@0F5QH201PH600G01LG5`0J6QX08b<S08b<S03Bd]80Q8B404];B`0J6QX06a/K01hN7P0M7Ad051@D
01HF5P0E5AD07AdM079bLP1eMGD0?Cdm08V9R@1/K6`0IVIV05=CD`0E5AD00P0C4a<0500e=CD0d];B
0<G5a@2e]KD0]kNg07alO01^KVh0Fe]K0492@P0`<3007alO02/[:`1SHf<0/K6a092@T03:b/X0TI6A
0:FUY@24Q8@0:BTY0P0R8R806P0h>3P0E5AD04A4A00l?3`0F5QH08Z:RP2GUiL0YJFU0=_Kf`2DU9@0
WYjN0:^[Z`2b/[80`/;20<30`02QXJ40JFUY06ATI01gMgL0WinO0:n_[`25QHD0Kfm_059BDP1CDe<0
71`L4?ooool000kooooo05P0/K6a07YjNP1EEED0J6QX04M7A`1]KFd0H61P02d];@1HF5P0IVIV03/k
>`1HF5P0?Cdm07=cL`2CTi<0>CTi06YZJP0k>c/0LW9b03Hf=P1[Jf/0LG5a04e=C@1`L700HF5Q03Xj
>P18B4P0;Rh^03Ph>01WIfL0@d=304M7A`0Q8B409bLW06IVIP17AdL0@D5101lO7`12@T80?Shn05IF
EP0L71`0;bl_05eMG@0c<c<0?Cdm02l_;`0B4Q80JFUY05IFEP0h>3P0E5AD01HF5P1XJ6P0MGEe03`l
?015ADD0>3Ph03@d=01OGel0Ogmo05ADE01HF5P0J6QX01<C4`0N7Qh0HF5Q05MGE`0o?cl0M7Ad01/K
6`1VIVH0ADE505eMG@1LG5`0;Rh^07AdM01BDU80:2PX05aLG018B4P08R8R06][J`1/K6`0<c<c05aL
G01WIfL0S8b<0P0a<C40_01CDe<0B4Q805mOG`1`L700Kfm_08b<S01OGel0=SHf08Z:RP1>CTh0HV9R
065QH@1bLW80>3Ph0410@01[Jf/0G5aL05=CD`1RHV80Cdm?07emO@1_Kfl0Q8B40492@P0X:2P0E5AD
04A4A01KFe/0LW9b04E5A@1FEUH0KVi^061PH01DE5@0F5QH06ATI00];Bd0Lg=c079bLP1fMWH0D51@
0:FUY@2_[jl0LW9b03<c<`1lO7`0ATI60:6QX@1SHf<0?3`l09RHV01ZJVX0Bd];06a/K02DU9@0E5AD
05EEE@1BDU80>3Ph071`L00];Bd0NWYj07UiN@1JFUX0KFe]05EEE@2TY:@0QhN703lo?`10@400^;Rh
075aL@2KVi/0WYjN08b<S017AdL0QHF50:FUY@1@D500G5aL0:BTY00n?Sh0<C4a09ZJVP1HF5P0QXJ6
08:2PP17AdL0>c/k08B4Q01IFET0HV9R04e=C@1iNGT0Ng]k04M7A`1nOWh0KFe]06a/K01lO7`0:b/[
07MgM`1`L700?Cdm07emO@1UIFD0:b/[03Ti>@1<C4`0ADE502DU9@1[Jf/0LW9b06e]K@24Q8@0UiNG
0:>SX`1=CDd0EUIF07inOP0k>c/0O7al08:2PP0_;bl0HF5Q09ZJVP1BDU80A4A408N7Q`1SHf<0:2PX
04=3@`1dM7@0DE5A09VIV@2k^k/0LW9b02@T901dM7@0H61P03hn?P1gMgL0EeMG03<c<`18B4P09BDU
07YjNP2=SHd0:BTY05=CD`1dM7@0CTi>01/K6`1CDe<0EUIF03Ti>@0T92@0SHf=07moO`1;Bd/0_kno
0;Jf]P17AdL0NWYj0:NWY`0i>CT0Z:RX059BDP1KFe/0C4a<059BDP1CDe<0KFe]07=cL`0W9bL0CTi>
071`L022PX80IFEU09RHV00m?Cd08R8R03hn?P1KFe/0FEUI04A4A01>CTh0Jf][3_ooool0?@1hN7P0
ADE503dm?@1BDU80d=3@0;6a/@2ZZZX0`l?308B4Q010@400EeMG0:ZZZP2h^;P0WYjN09FEU@0g=cL0
Ti>C07inOP1:BTX0@d=306MWI`2:RXX0];Bd03/k>`0`<300:b/[02LW9`0[:b/07alO01XJ6P1OGel0
WYjN07AdM00K6a/05aLG03Lg=`1GEeL0Hf=S01hN7P0L71`0>SXj051@D00j>SX09bLW03@d=019BDT0
>SXj05mOG`18B4P0UiNG0<C4a02i^KT0Y:BT09VIV@25QHD0MWIf06][J`1`L700Q8B404e=C@0a<C40
0P0H61P04P0G5aL07QhN04e=C@1QHF40F5QH05]KF`0b<S8061PH01LG5`0E5AD0>c/k051@D00g=cL0
MgMg05QHF023Ph<0K6a/051@D080@D5100@0;Bd]030`<00Q8B40De=C0P0L71`05P0Y:BT0W9bL08Z:
RP1JFUX0D51@03hn?P1RHV80H61P02h^;P0e=CD0F5QH0=3@d019BDT0bLW90;Zj^P1HF5P0MgMg01HF
5P0H61P05aLG01PH600I6AT201LG5`0=01PH600I6AT0=SHf04i>CP23Ph<0KFe]02d];@0[:b/0@D51
071`L00K6a/05aLG01/K6`0201LG5`0501/K6`17AdL0EEEE03hn?P1RHV801@0E5AD0?@1TI6@0=CDe
01TI6@0M7Ad0MGEe0;Fe]@1jNWX0GEeM02l_;`0l?3`0HV9R07YjNP1eMGD0N7Qh07AdM01lO7`0IfMW
04e=C@1nOWh0[Zj^09RHV01HF5P0R8R80:NWY`1;Bd/0ADE501hN7P0Y:BT0FEUI0:FUY@2;Rh/0WinO
05eMG@1=CDd0IFEU08N7Q`1hN7P0V9RH0::RXP2DU9@0W9bL0410@00V9RH0>SXj02TY:@0C4a<03`l?
01DE5@0b<S80:RXZ07IfMP36a/H0[:b/08j>SP2h^;P0_[jn0:JVYP2k^k/0YJFU09RHV01KFe/00P0M
7Ad00`0B4Q805ADE01HF5P0201DE5@0Q01LG5`0Q8B40IfMW0;Fe]@2l_;`0JFUY01`L700N7Qh092@T
02l_;`0M7Ad06ATI01XJ6P0X:2P0H61P05iNGP1OGel0H61P04E5A@1WIfL0@d=3018B4P0D51@05aLG
06YZJP2>SXh0MGEe05UIF@1YJFT0:BTY01DE5@0D51@04a<C00805ADE0200@d=307emO@0o?cl0CTi>
05aLG026QXH0JVYZ07alO028R8P0Ng]k04E5A@1WIfL0RXZ:08N7Q`25QHD0PX:207alO028R8P0Y:BT
0=OGe`1iNGT0DU9B08:2PP2KVi/0Xj>S0:ZZZP2SXj<0QHF508b<S02MWId0b/[:0:JVYP80Shn?00<0
@d=301TI6@0I6AT04?ooool000kooooo03X0RXZ:09:BTP1eMGD0KFe]04=3@`1XJ6P0R8R804Q8B01g
MgL0HF5Q04Q8B01eMGD0BDU903lo?`1LG5`0@41005IFEP0g=cL0M7Ad03De=@1mOGd0@D5102d];@15
ADD0>CTi04I6AP1QHF40@d=3034a<@0o?cl0CDe=071`L00i>CT0=SHf08>3P`0a<C40CTi>04Q8B01:
BTX0:BTY04m?C`0k>c/08b<S04M7A`0/;2`0Bd];04=3@`0^;Rh0Kfm_03Lg=`1GEeL0Cdm?028R8P1T
I6@0IfMW01PH601@D500G5aL0P0/;2`0fP1;Bd/0=SHf04a<C01UIFD0>CTi01dM7@1ZJVX0?3`l0492
@P1/K6`0<S8b0861P@12@T80CTi>05=CD`1LG5`0Rh^;02<S8`0_;bl0Jf][05mOG`11@D40IVIV04E5
A@0^;Rh0IfMW08f=S@1[Jf/0:BTY03dm?@1FEUH0>SXj07EeM@1DE5@0GemO08f=S@1?Cdl0<30`08V9
R@1>CTh0KFe]04];B`1gMgL0BTY:04A4A01fMWH0EUIF05]KF`1FEUH0Cdm?06EUI@1SHf<0OGem05]K
F`0g=cL0AdM705]KF`2=SHd0TI6A06=SH`1UIFD0FEUI06IVIP1UIFD0GemO04U9B@0k>c/0OGem07Mg
M`1SHf<0:RXZ08>3P`2DU9@0CDe=03`l?01iNGT0=cLg07YjNP1@D500>c/k08^;R`1CDe<0ATI607If
MP1/K6`0@d=305IFEP1EEED0>3Ph09>CT`1:BTX0KFe]061PH01GEeL0TI6A04i>CP1jNWX0D51@04M7
A`1PH600[:b/055AD@1[Jf/0N7Qh06=SH`17AdL0OWin0:RXZ01GEeL0OGem0:ZZZP1hN7P0?3`l09bL
W01RHV80QHF506IVIP0l?3`0EUIF0:f][@1YJFT0Jf][04Y:BP12@T80DE5A04=3@`2][Jd0LG5a06QX
J01`L700<C4a07moO`1dM7@0=3@d065QH@1HF5P0ATI605YJFP1QHF40LG5a03@d=01IFET0Cdm?05AD
E01hN7P0SHf=09NGU`17AdL0C4a<07UiN@0g=cL0R8R8079bLP1;Bd/0DE5A07=cL`12@T80Cdm?08^;
R`18B4P09RHV04];B`1aLG40DE5A08:2PP28R8P0JVYZ02<S8`1^KVh0C4a<04e=C@22PX80I6AT059B
DP19BDT0<30`08N7Q`2ATI40EUIF06=SH`22PX80@d=302LW9`1bLW80QHF508f=S@0Y:BT0TI6A065Q
H@12@T80V9RH08N7Q`0h>3P0RXZ:096AT@0f=SH0]kNg092@T01<C4`0HV9R04m?C`0j>SX0Ogmo07Qh
N00W9bL0EEEE06ATI014A4@0M7Ad06QXJ00k>c/0;2`/04A4A01IFET0?Shn05IFEP1@D500CTi>3_oo
ool0K`1/K6`0?clo03hn?P20P800FEUI06MWI`1cLg<0BDU902XZ:P0N7Qh06a/K01lO7`10@400Jf][
08f=S@2NWYh0W9bL0492@P0g=cL0?clo04I6AP1[Jf/0LG5a02`/;00W9bL08b<S02DU9@0Z:RX09bLW
03De=@2LW9`0E5AD01lO7`0Q8B4051@D034a<@0l?3`0G5aL07alO01oOgl0DE5A02l_;`0V9RH092@T
02/[:`0_;bl0Bd];0::RXP1nOWh0W9bL0;no_`2NWYh0/;2`069RHP1:BTX0ADE501/K6`0G5aL09bLW
06][J`1EEED0<c<c01XJ6P0O7al0:b/[059BDP1SHf<0IFEU065QH@1>CTh0ADE5055AD@1JFUX0B4Q8
03@d=014A4@0@D51028R8P1KFe/0?clo05MGE`1<C4`0Bd];02@T900a<C408B4Q07AdM01oOgl0Kfm_
071`L013@d<0;bl_02`/;00l?3`0IVIV08b<S02MWId0Ogmo05mOG`1mOGd0XJ6Q07alO028R8P0_kno
06QXJ01/K6`061PH01TI6@0F5QH061PH01XJ6P0201TI6@0:01HF5P0G5aL07QhN04A4A013@d<0UYJF
02h^;P0k>c/0CDe=06ATI0806QXJ01l07AdM030`<015ADD0FUYJ08Z:RP2?Shl0T92@0820P00G5aL0
5ADE01HF5P0E5AD05QHF06EUI@0S8b<08R8R0410@02KVi/0Fe]K069RHP2<S8`0K6a/02h^;P0Y:BT0
>CTi05UIF@1YJFT0NGUi0820P02>SXh0WinO0080ZZZZ02d0J6QX0:b/[02YZJT0<C4a02LW9`1CDe<0
>3Ph01TI6@0J6QX0:RXZ08Z:RP14A4@08R8R03@d=01FEUH0Z:RX0<C4a02JVYX0[:b/09^KV`2^[Zh0
_Kfm0;Rh^02ATI40:b/[02<S8`0R8R80=CDe03`l?00W9bL0GemO06ATI00g=cL0Jf][05IFEP2LW9`0
OGem08V9R@2g]kL0_Kfm059BDP0V9RH071`L01TI6@0G5aL00`0F5QH0B00H61P07alO04Y:BP2HV9P0
d=3@0=CDe02GUiL0QhN7092@T02YZJT0Z:RX09NGU`23Ph<0N7Qh0;Fe]@3<c<`0[Zj^06i^KP11@D40
MgMg03Hf=P0E5AD06a/K04m?C`17AdL08B4Q05=CD`29RHT0HV9R01<C4`0D51@05ADE01@D500F5QH0
<30`06UYJ@0];Bd061PH03Xj>P1OGel0Ng]k09fMW@1<C4`0CTi>06m_K`29RHT0FUYJ020P800G5aL0
5ADE01`L701DE5@0H61P0861P@3IfMT0c<c<0:2PX02DU9@0LG5a03lo?`1UIFD0VIVI06YZJP2<S8`0
WIfM0<C4a02e]KD0alO70:6QX@0N7Qh06a/K01XJ6Q3ooooo000>ooooo`0n0:j^[P2=SHd0K6a/08:2
PP1=CDd0JVYZ06UYJ@1?Cdl0P82008:2PP10@400Kfm_04];B`1_Kfl0HF5Q038b<P1TI6@0A4A404m?
C`0/;2`0O7al065QH@0a<C40@T9202LW9`0g=cL0Fe]K03dm?@0f=SH0J6QX05MGE`1MGEd0:2PX04I6
AP2BTY80?Cdm0492@P0e=CD0>CTi03Hf=P1RHV80>3Ph02HV9P1=CDd0=3@d04];B`1BDU80?clo061P
H01JFUX0JFUY0451@@0P8200NWYj071`L00^;Rh0@d=3051@D00^;Rh0>SXj05ADE01@D50205IFEP1S
028R8P0L71`0IVIV04m?C`1<C4`0HF5Q02PX:01iNGT0ADE504m?C`13@d<0AdM708:2PP0g=cL0:2PX
05ADE01ADE40<c<c06QXJ01<C4`0:BTY03Hf=P24Q8@0E5AD01XJ6P0a<C40I6AT04Y:BP20P800B4Q8
05]KF`1_Kfl0AdM702l_;`1`L700>SXj07alO01VIVH0Kfm_04E5A@1KFe/0NWYj05]KF`1eMGD0E5AD
04E5A@1^KVh0N7Qh07inOP1HF5P0AdM7059BDP1LG5`0YjNW09ZJVP1[Jf/0MGEe07YjNP1UIFD0GUiN
06e]K@1OGel0CDe=07inOP1VIVH0NGUi05=CD`2OWil0UIFE0492@P0j>SX0OGem04=3@`24Q8@0=3@d
04=3@`2NWYh0De=C03Ph>01DE5@0EUIF0410@01OGel0HV9R02`/;02CTi<0<30`07]kN`1RHV80Hf=S
07]kN`15ADD0_[jn05aLG01JFUX0MWIf0:NWY`10@400L71`0080Q8B40740B4Q8096AT@2_[jl0GemO
092@T02TY:@0@T920451@@2f][H0Rh^;09ZJVP2=SHd05QHF06i^KP2TY:@0JVYZ07]kN`1<C4`0H61P
04M7A`0i>CT0Vi^K07IfMP1jNWX0IFEU02l_;`2SXj<0PX:20410@01hN7P0DU9B04Q8B01EEED0G5aL
08V9R@1<C4`0H61P06=SH`1XJ6P0Ph>307EeM@1dM7@0B4Q8059BDP1YJFT092@T06][J`1HF5P0@T92
08:2PP2:RXX0BDU9051@D01hN7P0>CTi03<c<`1MGEd0Kfm_04a<C01mOGd0O7al03Xj>P0O7al0QhN7
05EEE@18B4P0LG5a0410@01NGUh0AdM703Hf=P29RHT0KFe]03`l?01]KFd0]kNg04U9B@0Y:BT0EEEE
03Ti>@0`<300@4100;Zj^P2BTY80FEUI0<30`02LW9`05aLG07alO02FUYH0=CDe09VIV@1QHF404A4A
06YZJP2EUID0CTi>07emO@0k>c/08R8R06m_K`2UYJD0GemO06a/K018B4P0820P0451@@1VIVH0QhN7
051@D01QHF40@d=302/[:`0>ooooo`0=04M7A`15ADD0DU9B092@T01=CDd0[jn_09VIV@0l?3`09bLW
01dM7@0G5aL06QXJ01LG5`0201/K6`0505ADE02ZZZX0^[Zj09RHV02GUiL00P2YZJT0102f][H0U9BD
04e=C@0/;2`2024Q8@0L02h^;P2MWId0GemO01hN7P0F5QH09BDU01dM7@0E5AD06QXJ05eMG@2:RXX0
LW9b05mOG`1dM7@0DU9B04=3@`0c<c<0;bl_04e=C@2DU9@0T92@08F5Q@37alL0W9bL09:BTP0l?3`0
=CDe024Q8@805QHF02<05aLG02l_;`2;Rh/0EeMG06MWI`1ZJVX0IFEU071`L02FUYH0De=C03/k>`0L
71`0A4A404Y:BP16ATH0B4Q805EEE@1PH600<S8b02l_;`1=CDd0IfMW03<c<`1<C4`0B4Q804Y:BP0U
9BD0<S8b04m?C`0V9RH07alO024Q8@0K6a/071`L01PH600201XJ6P0<05aLG02/[:`0Hf=S02TY:@0V
9RH0?clo06=SH`1<C4`0Zj^[08>3P`1_Kfl05aLG0P0E5AD00`0G5aL05QHF01LG5`0201DE5@1i01LG
5`0F5QH0=3@d04];B`28R8P0T92@07]kN`21PH40Jf][03Lg=`1`L700IVIV0492@P0b<S80;bl_055A
D@2CTi<0bl_;08V9R@1ADE409BDU01HF5P0G5aL0=SHf06][J`0j>SX0Ph>30<K6aP2ZZZX0De=C04A4
A02@T900gMgM0=7Ad@2_[jl0S8b<06][J`1fMWH0/k>c07=cL`1=CDd0;2`/01lO7`1TI6@0_kno0<C4
a02c/k<0;2`/01TI6@0T92@0CDe=030`<00G5aL071`L05EEE@1HF5P0?clo061PH01nOWh0^;Rh06=S
H`2EUID0R8R807]kN`1KFe/0LW9b0;no_`2PX:00<c<c034a<@0e=CD0GemO071`L00_;bl0HF5Q01/K
6`0J6QX09bLW04A4A01YJFT0VYZJ04Q8B01bLW80SXj>07inOP14A4@0:b/[01dM7@0J6QX05QHF01DE
5@0F5QH05ADE02/[:`1JFUX0EeMG08F5Q@3EeMD0OGem05]KF`2>SXh0a/K604I6AP0h>3P0<S8b03Lg
=`1gMgL0NWYj09:BTP2UYJD0Y:BT0:>SX`0`<3005ADE04Y:BP0M7Ad06QXJ01`L700b<S80Vi^K07Ad
M00401DE5@0W01hN7P1XJ6P0DE5A01DE5@0K6a/0?3`l02d];@1DE5@0VYZJ0;2`/01/K6`07QhN01dM
7@1YJFT06ATI01HF5P0H61P0ATI60861P@2b/[80YJFU06m_K`3?cll0OWin05MGE`21PH40_;bl07Qh
N01PH60092@T03Lg=`1HF5P0Y:BT0=KFeP2QXJ40L71`024Q8@0I6AT06QXJ013ooooo000>ooooo`35
0;6a/@2DU9@0J6QX08F5Q@1>CTh0JVYZ06UYJ@1FEUH0Ph>308N7Q`1:BTX0Kfm_04U9B@17AdL0JVYZ
04A4A01hN7P0@d=307IfMP0g=cL0M7Ad06IVIP0f=SH0DE5A03lo?`0j>SX0G5aL03De=@0j>SX0Lg=c
05ADE01nOWh0;bl_030`<01fMWH0@41004U9B@0e=CD0ATI603<c<`1OGel0>3Ph02DU9@1ADE40ADE5
04M7A`1FEUH0?Cdm05iNGP1SHf<0Q8B4055AD@0X:2P0Rh^;075aL@0d=3@0AdM704i>CP0a<C40BTY:
06MWI`1DE5@0DU9B06=SH`0l?3`09RHV07alO01IFET0?Shn065QH@0[:b/0L71`04Y:BP1DE5@0A4A4
05]KF`1gMgL0?clo02`/;01GEeL0GemO02h^;P1jNWX0GemO02d];@0c<c<0QHF506EUI@0V9RH0>CTi
079bLP13@d<0RXZ:06IVIP1bLW80J6QX05]KF`0l?3`0LG5a03Hf=P1KFe/0JVYZ07UiN@17AdL0D51@
07UiN@1NGUh0L71`04i>CP19BDT0OWin07emO@1nOWh0BDU903hn?P1FEUH0H61P09JFUP1lO7`0Hf=S
08J6QP1QHF40H61P051@D01aLG40CDe=05YJFP21PH40GUiN061PH012@T80/K6a09jNWP16ATH0?Shn
0820P01EEED0YjNW04i>CP0j>SX0XJ6Q061PH018B4P0Jf][05UIF@0m?Cd0GEeM05YJFP0b<S80TY:B
03Xj>P1eMGD0Lg=c06][J`1kNg/0C4a<07QhN01OGel0LW9b07emO@2g]kL0FUYJ06EUI@2=SHd0?clo
03Hf=P27QhL0][Jf065QH@2ZZZX0a/K604A4A00j>SX0Zj^[030`<01nOWh0Shn?03lo?`0m?Cd0PX:2
04I6AP2TY:@0GEeM0:FUY@25QHD0D51@0;Ng]`1dM7@0HF5Q06UYJ@0g=cL0UYJF06ATI00[:b/0QXJ6
065QH@1BDU800P1NGUh0A01kNg/0;bl_07emO@1dM7@0Kfm_08j>SP1`L700S8b<04E5A@1MGEd0PX:2
03Ph>01/K6`0EUIF01<C4`1YJFT0Shn?03hn?P1KFe/0PH6102HV9P0Z:RX0IfMW05IFEP1;Bd/0WIfM
09NGU`0Y:BT07AdM07inOP19BDT0AdM706=SH`0I6AT0F5QH069RHP0j>SX0Q8B405]KF`0C4a<0ATI6
07inOP1<C4`0:RXZ06=SH`16ATH0<30`01hN7P1nOWh0FEUI06ATI03Jf]X0_kno03Hf=P2BTY80WinO
034a<@2KVi/0HV9R03`l?01?Cdl0F5QH04Q8B01cLg<0H61P01hN7P1?Cdl0RXZ:0P1SHf<02@1LG5`0
9RHV01`L700W9bL0EEEE04Y:BP1OGel0ADE5061PH00>ooooo`0<04U9B@1PH600UYJF09:BTP2TY:@0
[Jf]0;2`/02ATI40Fe]K02HV9P0L71`06QXJ0P0G5aL04@0E5AD07QhN04M7A`2XZ:P0GemO028R8P0K
6a/0>SXj04Y:BP19BDT0K6a/071`L01dM7@0NGUi0;:b/P2g]kL0DE5A008071`L00`08b<S03hn?P0G
5aL071`L04e=C@1?Cdl0D51@06YZJP0c<c<09BDU01dM7@0O7al2059BDP0Z06a/K01lO7`0MGEe05iN
GP1eMGD0LW9b03dm?@0g=cL0?Shn04=3@`0N7Qh05QHF02`/;02IVIT0WYjN07UiN@2BTY80GEeM08B4
Q01iNGT0Ph>3075aL@1QHF40FEUI03hn?P0j>SX0?3`l055AD@1LG5`0<S8b01LG5`0H61P0:BTY04a<
C00];Bd0IFEU059BDP1GEeL0Hf=S03dm?@0K6a/07AdM0P0L71`01`0F5QH06ATI01PH600L71`0EeMG
08F5Q@0];Bd00P0L71`03P0Z:RX0Fe]K04A4A02;Rh/0Shn?07=cL`0Y:BT06ATI01DE5@0I6AT05aLG
01TI6@0F5QH05ADE0P0F5QH0XP0];Bd0IFEU06IVIP3=cLd0VIVI06QXJ02=SHd0SXj>03Xj>P0];Bd0
;2`/02PX:00];Bd0GEeM0=KFeP2>SXh06a/K02HV9P1FEUH0Cdm?05YJFP2BTY80SHf=08n?S`30`<00
^k^k05EEE@2<S8`0?Cdm0:>SX`33`l<0QHF5051@D00^;Rh0:b/[04];B`1<C4`0:b/[01hN7P0M7Ad0
7QhN06][J`36a/H0FEUI04a<C02HV9P0<c<c020P800];Bd0Bd];02@T900h>3P0G5aL09NGU`1?Cdl0
I6AT08>3P`2ZZZX0EeMG0:RXZ00];Bd0<S8b04Q8B01>CTh0MWIf09nOW`1ADE409BDU02l_;`13@d<0
QhN70861P@1WIfL05QHF01XJ6P0Y:BT0<c<c02HV9P1jNWX0Fe]K04A4A01bLW80Ti>C05IFEP0n?Sh0
:2PX02@T900R8R80;2`/01XJ6P0I6AT0=3@d06e]K@17AdL0GEeM0<G5a@2HV9P0?clo03<c<`0b<S80
71`L01DE5@0G5aL08B4Q05iNGP1TI6@0Ti>C096AT@3>c/h0alO704Q8B011@D40=SHf01HF5P0K6a/0
7AdM02`/;02QXJ40WinO01lO7`0G5aL071`L024Q8@15ADD0WYjN03<c<`0G5aL07QhN04E5A@0N7Qh0
MWIf06a/K02_[jl09BDU01PH600J6QX0CDe=03lo?`0E5AD05aLG05YJFP2PX:00TY:B05]KF`2FUYH0
I6AT06][J`2LW9`0ZJVY0:2PX038b<P0[Jf]04m?C`2KVi/0^[Zj09RHV02n_[h0EUIF07alO00i>CT0
7AdM01lO7a3ooooo000>ooooo`1j0:JVYP2HV9P0Hf=S08f=S@1;Bd/0M7Ad08f=S@1PH600S8b<08N7
Q`1<C4`0L71`04Y:BP1;Bd/0NWYj03lo?`1FEUH0GEeM0861P@0k>c/0O7al06IVIP1BDU80JFUY04M7
A`18B4P0FUYJ03dm?@0^;Rh0F5QH07IfMP2;Rh/0<c<c03Xj>P21PH40:2PX061PH00`<300=CDe02DU
9@1UIFD0;Rh^038b<P1LG5`0@D5104Q8B01?Cdl0<30`06ATI01IFET0QhN70451@@0M7Ad0MWIf06YZ
JP0];Bd0Bd];06][J`0S8b<0=3@d05QHF019BDT0GEeM07AdM00n?Sh09RHV07IfMP0k>c/0GUiN075a
L@0a<C40FUYJ03Xj>P1GEeL0GUiN08:2PP28R8P0?clo03/k>`1`L700Hf=S03dm?@1XJ6P0AdM702d]
;@1FEUH0TY:B06=SH`0O7al0>CTi059BDP0R8R80PX:20820P01bLW80I6AT04e=C@0f=SH0TI6A04];
B`1fMWH0CTi>08J6QP1?Cdl0G5aL05YJFP10@400IFEU06e]K@1KFe/08R8R03lo?`2;Rh/0IFEU03Ti
>@0f=SH0Hf=S08n?S`21PH40Kfm_071`L00h>3P305iNGP2I01<C4`1MGEd0WYjN06QXJ01nOWh0<C4a
07EeM@2?Shl0Hf=S0492@P1kNg/0CTi>0:>SX`1CDe<0@d=30:ZZZP16ATH0EeMG08V9R@1/K6`0>SXj
05=CD`1<C4`0;Bd]092@T00l?3`0MgMg05QHF01`L700^KVi05iNGP2LW9`0De=C04M7A`1eMGD0]kNg
07MgM`1^KVh0WinO03dm?@0a<C40QXJ60:ZZZP1/K6`0_Kfm0<?3``1bLW80AdM70:RXZ01_Kfl0Q8B4
07YjNP13@d<0BDU909RHV00d=3@0T92@05iNGP0k>c/0Bd];04=3@`2f][H0Cdm?03Lg=`1NGUh0@D51
0<C4a01lO7`08R8R06IVIP1MGEd0Vi^K09nOW`2KVi/0Shn?024Q8@1KFe/0M7Ad05mOG`2CTi<0Shn?
08b<S010@400>3Ph079bLP0j>SX0JVYZ06][J`15ADD0EeMG05mOG`0e=CD0Ph>30<;2`P27QhL0;Bd]
04m?C`0X:2P04a<C09^KV`35aLD0Jf][024Q8@1`L700ADE504I6AP1VIVH0BDU9051@D017AdL0@410
08V9R@15ADD0;2`/05]KF`1hN7P0@D5106QXJ02e]KD0WIfM07QhN00T92@0RHV906m_K`1>CTh0VYZJ
09BDU012@T80Xj>S0;2`/00c<c<0/k>c06e]K@18B4P0K6a/05]KF`10@400S8b<07]kN`0U9BD0EEEE
08B4Q01JFUX0L71`05MGE`0b<S808b<S03<c<`1JFUX0BTY:05eMG@12@T80Lg=c00kooooo00d0TY:B
0;>c/`2g]kL0U9BD05eMG@2?Shl0Cdm?08R8R02<S8`0De=C01hN7P0K6a/05QHF00805ADE00@05QHF
03Hf=P1LG5`0>3Ph0P0@41005@0V9RH0:b/[01@D500A4A406ATI03Hf=P2XZ:P0N7Qh038b<P1iNGT0
<C4a01hN7P0U9BD0<C4a02XZ:P0J6QX0>CTi04=3@`0`<3008R8R01<C4`0201DE5@0N01LG5`12@T80
LG5a06][J`1GEeL0:2PX02<S8`1nOWh0DU9B02@T900V9RH05ADE028R8P1JFUX0K6a/05UIF@1TI6@0
MgMg08f=S@1SHf<0EeMG03Ph>01DE5@0GemO055AD@1`L700QhN707AdM01jNWX0HF5Q0P0U9BD03P15
ADD0A4A402<S8`0H61P09bLW0492@P1YJFT0Rh^;0:JVYP1FEUH0@T92024Q8@0O7al07AdM0P0K6a/3
01`L700>06=SH`1YJFT07QhN01`L700M7Ad09bLW05eMG@19BDT0Bd];09^KV`1^KVh0DU9B03Xj>P0F
5QH301DE5@0G01HF5P0D51@061PH01HF5P0_;bl0IVIV06m_K`2CTi<0fm_K0;Vi^@2>SXh0FUYJ04E5
A@0l?3`0@41006i^KP2SXj<0]KFe0=3@d01>CTh0=cLg03@d=00f=SH00P20P800IP1KFe/0D51@04U9
B@1NGUh0WinO03<c<`1DE5@0V9RH0:ZZZP0k>c/092@T02HV9P0b<S80BTY:04A4A00O7al06ATI01XJ
6P0O7al0<C4a0:ZZZP2[Zj/0HV9R01/K6`17AdL0MGEe028R8P0Y:BT0Cdm?07inOP0h>3P0CDe=0<71
`@1cLg<0Ph>3079bLP20P800HF5Q09>CT`1BDU806ATI01PH600g=cL0DU9B0;Jf]P22PX80Bd];03`l
?014A4@0S8b<0;bl_01YJFT06QXJ01/K6`0g=cL07AdM01/K6`13@d<0PX:2055AD@1FEUH0RHV909FE
U@1HF5P0BDU9034a<@0_;bl0E5AD030`<011@D40Ph>308B4Q00h>3P0OWin09^KV`1ADE4061PH01DE
5@0H61P05aLG01<C4`0J6QX08B4Q05UIF@1/K6`0WinO0:VYZ@1aLG40V9RH0<S8b021PH406QXJ01TI
6@0M7Ad07QhN05eMG@2i^KT0[jn_02LW9`0Z:RX09RHV05QHF080T92@0P0L71`08`0U9BD0B4Q801hN
7P1]KFd0<S8b07]kN`0_;bl05aLG01XJ6P0S8b<0SHf=01TI6@0c<c<0YJFU08F5Q@17AdL0H61P0:>S
X`0N7Qh06a/K024Q8@1;Bd/0Vi^K0;Vi^@33`l<0b/[:05iNGP0X:2P08B4Q08>3P`2`/;00/K6a0;Jf
]P2;Rh/0TY:B013ooooo000>ooooo`3o08J6QP2[Zj/0I6AT06m_K`0g=cL0M7Ad08>3P`1]KFd0UYJF
0451@@1WIfL0Rh^;03hn?P1NGUh0?3`l04U9B@1UIFD0DE5A04m?C`0^;Rh0TI6A0492@P11@D40?3`l
028R8P10@400HV9R03`l?00c<c<0<S8b04I6AP1ZJVX0<S8b07=cL`2JVYX08b<S03`l?00`<300<S8b
018B4P1HF5P0;Rh^02<S8`0n?Sh09bLW04];B`1GEeL0:2PX0492@P0V9RH0W9bL051@D00T92@0OWin
04A4A00@4100KFe]05mOG`0M7Ad0>3Ph05=CD`0D51@0=cLg05ADE00e=CD0AdM708Z:RP0R8R80>SXj
05IFEP0U9BD0FEUI02`/;01^KVh0Fe]K08b<S02@T9005ADE02d];@1VIVH0NWYj04U9B@1bLW80<C4a
01XJ6P0P8200GEeM04Y:BP0J6QX0?Shn069RHP0];Bd0J6QX04=3@`1OGel0Fe]K04A4A00/;2`0Ng]k
04E5A@1MGEd0>c/k06][J`1:BTX0K6a/051@D00f=SH0Bd];06QXJ01MGEd0?Cdm03Hf=P1OGel0DU9B
0492@P1<C4`0@D5105aLG01_Kfl0MWIf07MgM`1NGUh0De=C05UIF@1VIVH0@T92051@D01iNGT0GemO
08B4Q018B4P0Ogmo09BDU00j>SX0OWin0:>SX`18B4P0PH61038b<P1UIFD0`l?304a<C00d=3@0EUIF
04Q8B00f=SH0GEeM05IFEP0Y:BT0RXZ:03Ti>@1fMWH0>c/k04=3@`1nOWh0K6a/09fMW@17AdL06a/K
03`l?01mOGd0GEeM06=SH`2<S8`0Ogmo03/k>`2FUYH0Q8B403<c<`2YZJT0YJFU07UiN@19BDT0U9BD
079bLP26QXH0GemO03Xj>P1KFe/0[Jf]06][J`22PX80D51@06=SH`10@400MgMg0<k>cP1RHV80Ti>C
04E5A@0d=3@0NGUi07moO`0i>CT0MWIf0410@00m?Cd0DE5A0;fm_@2/[:`0C4a<05eMG@1oOgl0?Shn
09bLW03Hf=P0_[jn05]KF`1LG5`0F5QH03De=@2=SHd0EEEE04I6AP1RHV80IVIV01@D501BDU80Vi^K
065QH@0[:b/0AdM705iNGP0m?Cd0MGEe08b<S01WIfL0:RXZ06a/K00^;Rh0RXZ:0;Ng]`1]KFd0JVYZ
04e=C@1:BTX0XJ6Q055AD@12@T80J6QX06m_K`0`<300@T9206=SH`2?Shl0M7Ad02<S8`28R8P0S8b<
06YZJP3Cdm<0/;2`03lo?`2[Zj/0[:b/000G04a<C03Jf]X0][Jf04I6AP1VIVH0FEUI04A4A02WYjL0
OGem02<S8`1DE5@0Ph>3069RHP21PH40C4a<034a<@0X:2P0:RXZ055AD@0g=cL0NWYj06i^KP1kNg/0
3_ooool0=@3MgMd0eMGE0;Fe]@1bLW80M7Ad05iNGP1^KVh0CTi>065QH@1`L700CDe=038b<P0M7Ad0
6a/K01TI6@0G5aL0=SHf03<c<`1GEeL04A4A00h>3P0_;bl0AdM701XJ6P0B4Q8051@D01LG5`27QhL0
?3`l02HV9P2EUID0KVi^04e=C@0l?3`0<c<c04Y:BP0Y:BT0?clo04i>CP0`<300820P01@D500C4a<0
5QHF034a<@1KFe/0?clo05YJFP0[:b/06ATI01XJ6P1MGEd0QXJ60080:2PX03L06a/K02<S8`0o?cl0
TI6A0;>c/`29RHT0Jf][08f=S@1>CTh0=CDe01hN7P0h>3P0AdM703@d=00f=SH0BDU904a<C01QHF40
E5AD01PH600E5AD06QXJ03Xj>P1<C4`0ATI603<c<`0`<300HV9R0<K6aP2EUID0Q8B404a<C00S8b<0
820P028R8P0U9BD0820P01`L700M7Ad07QhN079bLP1fMWH07alO020P800R8R809BDU03Ph>01IFET0
EUIF0;Rh^01mOGd0FEUI04];B`0h>3P06QXJ00D05ADE02805aLG03`l?02PX:00Ng]k0:^[Z`3Hf=P0
QXJ605=CD`1?Cdl0NGUi0;bl_02HV9P0H61P03<c<`0S8b<0X:2P0;6a/@1hN7P0IVIV051@D02LW9`0
FEUI03Lg=`15ADD0=cLg07UiN@31`L40J6QX069RHP3;bl/0X:2P05iNGP1aLG40L71`0P1]KFd06P1R
HV80I6AT05]KF`1NGUh0D51@09RHV02:RXX0ZZZZ08n?S`0U9BD07alO07YjNP0i>CT0@41006=SH`1:
BTX0DE5A07YjNP2m_Kd0PX:203Xj>P1]KFd0;Bd]01TI6@17AdL0EeMG0P0e=CD08P0h>3P0Bd];0:n_
[`2ATI40?3`l0410@01NGUh0XZ:R0<K6aP1oOgl0;2`/024Q8@0V9RH061PH01hN7P0d=3@0Cdm?06IV
IP1DE5@0GemO09fMW@1kNg/0?clo059BDP11@D40=SHf06IVIP26QXH0RHV906=SH`0m?Cd0WIfM06][
J`1GEeL201DE5@0o01HF5P0E5AD051@D01XJ6P0M7Ad0@41008f=S@1kNg/0NWYj051@D01>CTh0[:b/
06a/K00J6QX06ATI01dM7@0i>CT0IfMW065QH@2o_kl0BDU903De=@0g=cL0Cdm?09nOW`1KFe/0;Bd]
05mOG`27QhL0EUIF02LW9`1PH6007alO06m_K`1RHV806a/K01XJ6P0N7Qh0Lg=c04A4A02VYZH0T92@
04M7A`1OGel0]kNg04I6AP0I6AT06a/K024Q8@0Y:BT0ATI603Ti>@0];Bd0M7Ad01PH600G5aL08R8R
0;^k^`3AdM40_Kfm09bLW01jNWX0KFe]013ooooo000>ooooo`0N07IfMP2<S8`0F5QH08B4Q011@D40
I6AT07UiN@1`L700QhN705YJFP13@d<0Jf][03hn?P1/K6`0Cdm?03`l?01HF5P0EUIF051@D00T92@0
K6a/05QHF00l?3`0Fe]K03<c<`0j>SX0CTi>02XZ:P1EEED0KFe]0P1@D500H`0J6QX0A4A407moO`10
@400BTY:020P801:BTX0:b/[05]KF`10@400C4a<05YJFP15ADD0@D5104M7A`0U9BD0De=C034a<@20
P800>CTi01lO7`23Ph<0B4Q800h>3P11@D40AdM703hn?P1gMgL0Rh^;04I6AP1IFET0=SHf01HF5P0W
9bL0NWYj04E5A@1TI6@0EeMG01/K6`14A4@0<S8b0492@P0_;bl0SXj>07QhN00Q8B409RHV0451@@1W
IfL0ATI606YZJP1NGUh0;bl_02PX:027QhL0@d=302XZ:P1MGEd0IfMW03/k>`1fMWH0ATI604e=C@1M
GEd0DU9B03hn?P1hN7P0AdM708V9R@1[Jf/0KFe]06i^KP2/[:`0R8R806YZJP1jNWX0OGem06][J`1h
N7P0GEeM06IVIP13@d<0Kfm_06=SH`1CDe<0OGem06YZJP1kNg/0Y:BT07=cL`1LG5`0J6QX0861P@1A
DE40J6QX061PH00m?Cd0^k^k05mOG`0208b<S02A02PX:019BDT0S8b<05=CD`24Q8@08B4Q04a<C02i
^KT0V9RH04e=C@1PH600>CTi05YJFP2`/;00UiNG0451@@2;Rh/0?Cdm09:BTP1nOWh0IVIV09:BTP1O
Gel0WIfM05QHF011@D40RXZ:08n?S`0k>c/0OGem09>CT`29RHT0>c/k0::RXP2<S8`0EEEE0<;2`P2D
U9@0QXJ60451@@1hN7P0M7Ad08V9R@17AdL0=cLg05UIF@2_[jl0MGEe0::RXP1=CDd0GUiN04a<C00o
?cl0QHF509BDU02CTi<0BTY:03Ph>02;Rh/0H61P03@d=01iNGT0Cdm?02TY:@0F5QH0PX:207inOP0i
>CT0Kfm_061PH00f=SH0Ogmo08N7Q`2>SXh0L71`04];B`15ADD06QXJ0820P025QHD0E5AD065QH@2=
SHd0=cLg05iNGP2HV9P0@T92038b<P1@D500HF5Q04a<C02b/[80UIFE05aLG00d=3@0Q8B402LW9`0e
=CD0GUiN03lo?`1PH600@T9207inOP2_[jl0QHF504];B`1QHF40Kfm_04E5A@0b<S80J6QX08F5Q@1L
G5`0@41009>CT`24Q8@0EEEE0=OGe`2ZZZX0>3Ph0<W9b@2VYZH08R8R09jNWP2/[:`0Bd];06YZJP1H
F5P0?Cdm0<C4a02GUiL09RHV055AD@2HV9P0H61P0;jn_P1YJFT0=3@d02<S8`0V9RH0EeMG02PX:02C
Ti<0UIFE08:2PP0>ooooo`0:0;Zj^P2l_;`0I6AT04Q8B00i>CT0:RXZ03dm?@1oOgl0F5QH06MWI`80
DE5A02@0?Shn030`<00Y:BT0<c<c06IVIP0X:2P0H61P02LW9`0N7Qh0B4Q807moO`1PH600?3`l02h^
;P0h>3P0XZ:R05MGE`1=CDd0][Jf0<C4a02b/[80YjNW09fMW@1kNg/0D51@05UIF@1hN7P0M7Ad05=C
D`0];Bd06QXJ02XZ:P1KFe/0<C4a051@D00^;Rh201TI6@0I01/K6`0i>CT0clo?069RHP13@d<0Bd];
06m_K`1oOgl0E5AD03Hf=P1UIFD0VYZJ07MgM`0Y:BT05QHF01DE5@0R8R80Cdm?02l_;`0L71`0Bd];
03@d=00o?cl0B4Q802@T900201DE5@0R01TI6@0M7Ad07QhN04];B`1SHf<0Rh^;06ATI02BTY80Q8B4
08n?S`1@D5008B4Q01`L700P82008B4Q01dM7@0L71`0<30`07MgM`26QXH0;bl_02DU9@0/;2`0:2PX
02HV9P1oOgl0[jn_0<[:bP2VYZH0;bl_03@d=01<C4`0<C4a01<C4`<05ADE02L05aLG01dM7@1XJ6P0
Shn?07]kN`2WYjL0h^;R0;jn_P2d];@0e=CD0=gMg@21PH40:BTY01LG5`0E5AD0=cLg08^;R`0Z:RX0
9RHV03dm?@1SHf<0Z:RX0;:b/P1fMWH0KFe]06i^KP38b<P0d=3@09ZJVP2e]KD0dM7A09BDU01gMgL0
EEEE03De=@0U9BD0?Shn02TY:@0J6QX00P0G5aL04P0b<S80^[Zj0492@P1VIVH0f=SH08Z:RP0Z:RX0
=cLg0:JVYP20P800RXZ:06UYJ@0g=cL0:BTY07inOP1WIfL0=CDe06YZJP806ATI06L0:b/[03<c<`1A
DE40H61P03dm?@0n?Sh0PX:20:JVYP21PH40C4a<065QH@2h^;P0b/[:09^KV`1TI6@0>3Ph01/K6`0L
71`07AdM02l_;`0o?cl07AdM05UIF@1TI6@0L71`09nOW`1EEED0Bd];06UYJ@2][Jd0]kNg09nOW`1Z
JVX0KVi^06UYJ@2h^;P0EeMG06][J`0G5aL05ADE01HF5P0E5AD051@D01LG5`0S8b<0F5QH092@T00h
>3P0?Cdm06a/K01;Bd/0QHF509NGU`0L71`07alO03Ti>@0m?Cd0DU9B079bLP2f][H0Fe]K06=SH`1P
H600Zj^[0<o?c`1ADE40:BTY04=3@`2>SXh0JFUY04=3@`0l?3`06QXJ055AD@2JVYX0NWYj05aLG01C
De<0Xj>S0;no_`21PH40GemO075aL@38b<P0dM7A024Q8@0J6QX07AdM01hN7P0Q8B40IVIV028R8P0E
5AD0=CDe0492@P0Q8B40Ogmo09FEU@1]KFd0];Bd0<30`01fMWH0M7Ad013ooooo000>ooooo`0H0:FU
Y@2b/[80Fe]K0:f][@1;Bd/0L71`06ATI01jNWX0XJ6Q0820P01<C4`0B4Q802d];@2@T900MWIf051@
D01=CDd0RXZ:09>CT`0m?Cd0U9BD05]KF`1=CDd0OGem0P0i>CT0I01DE5@05ADE03/k>`1KFe/0CDe=
08Z:RP0c<c<0@d=30:2PX00o?cl0@d=303Hf=P17AdL0:BTY04M7A`0L71`0=CDe05aLG01BDU80A4A4
03Xj>P0_;bl0JFUY051@D02IVIT08R8R01lO7`2ATI40LW9b02LW9`1EEED09bLW038b<P16ATH0De=C
02l_;`1_Kfl0Bd];02TY:@0d=3@0MGEe03Hf=P1NGUh0Cdm?02DU9@1ZJVX0?3`l05IFEP0d=3@0OWin
06a/K00T92@0<S8b03dm?@1>CTh0<c<c079bLP1TI6@0<S8b02TY:@2?Shl0=CDe01hN7P1>CTh0QHF5
03dm?@1UIFD0B4Q803De=@1TI6@0Fe]K03hn?P1PH600:RXZ09VIV@1QHF40H61P03De=@1OGel0LG5a
05iNGP1`L700K6a/05EEE@1WIfL0AdM707UiN@0a<C40A4A406QXJ01BDU80LW9b03Xj>P11@D40Zj^[
08n?S`1CDe<0EeMG08J6QP19BDT206MWI`1R02TY:@26QXH0GemO09JFUP2MWId0CDe=05IFEP2DU9@0
JFUY0:FUY@1=CDd0?Cdm0;6a/@1eMGD0DU9B06IVIP18B4P0=cLg05EEE@1`L700Cdm?06a/K00X:2P0
HV9R09FEU@1KFe/0Shn?03Lg=`2DU9@0OGem05YJFP2=SHd0XJ6Q024Q8@1PH600Q8B4071`L00j>SX0
WIfM09bLW01_Kfl0alO70::RXP2>SXh0G5aL09:BTP2=SHd0RXZ:06=SH`1=CDd0Jf][0:b/[01bLW80
VYZJ0451@@1KFe/0EEEE04E5A@2QXJ40G5aL08Z:RP1OGel0@d=30;^k^`21PH40?3`l07alO01ADE40
:RXZ01LG5`1CDe<0JVYZ030`<021PH40GemO06MWI`2IVIT0Shn?06a/K018B4P0JFUY06MWI`0g=cL0
OWin05eMG@19BDT0V9RH0:2PX00o?cl0Fe]K0861P@0_;bl0?3`l06UYJ@1`L700DU9B0;Zj^P2BTY82
024Q8@0b08^;R`0i>CT0?Cdm05mOG`0`<300NGUi04i>CP0g=cL0SXj>04Q8B00`<300RHV90<30`01N
GUh0<30`06IVIP0c<c<061PH024Q8@1PH600GEeM03De=@3Cdm<0XJ6Q03Hf=P2?Shl0TI6A02@T9025
QHD0GUiN03Hf=P2TY:@0QHF502LW9`21PH40Kfm_02HV9P1CDe<0Y:BT04I6AP25QHD0DU9B034a<@0S
8b<0;Rh^06YZJP0a<C40EeMG065QH@0`<30>ooooo`0j08N7Q`2UYJD0FUYJ02h^;P0V9RH09bLW02`/
;012@T80?3`l05]KF`21PH40K6a/06][J`1XJ6P0L71`04i>CP1@D500IVIV05mOG`1gMgL0CTi>03hn
?P1@D500A4A403Lg=`0d=3@0F5QH071`L02LW9`0][Jf08:2PP1BDU80AdM705mOG`1lO7`0WYjN08b<
S025QHD0MgMg08V9R@2:RXX0FEUI055AD@1cLg<0<c<c04];B`0`<3006ATI01LG5`0O7al06QXJ08j>
SP2QXJ40][Jf0=;BdP2f][H0H61P01/K6`807AdM0140<c<c03`l?013@d<0;Bd]01@D500C4a<05aLG
05ADE00b<S806a/K024Q8@11@D40B4Q804i>CP0i>CT06QXJ01LG5`0201XJ6P0C01`L700O7al0BTY:
0:FUY@1WIfL0J6QX09:BTP2IVIT0EeMG03lo?`0c<c<09bLW01lO7`0M7Ad07alO06MWI`1YJFT0S8b<
03/k>`0203<c<`09030`<010@400S8b<09^KV`33`l<0S8b<08F5Q@1ADE408B4Q0080>SXj01D0;Rh^
01lO7`0S8b<0<S8b05]KF`1aLG40O7al05mOG`2EUID0c<c<09FEU@2=SHd0QhN709fMW@1mOGd0?Shn
04Q8B00m?Cd0DU9B03@d=00F5QH00P0E5AD09`0G5aL0@T9206IVIP0c<c<0SHf=03Ph>01TI6@0ZJVY
0410@015ADD0WYjN03Hf=P0N7Qh05QHF01DE5@0X:2P0<S8b014A4@0@41004a<C014A4@0C4a<0KVi^
0410@01=CDd0/[:b08b<S02FUYH0ADE507alO02EUID0EUIF02TY:@0K6a/06QXJ05iNGP2BTY80Ph>3
05aLG00201DE5@1C01XJ6P0a<C40;Rh^024Q8@0E5AD05QHF03Hf=P2PX:00bLW907inOP1YJFT0VYZJ
06e]K@2TY:@0[Jf]09:BTP0U9BD06ATI01`L700Q8B40;bl_01PH600M7Ad0EUIF07]kN`2;Rh/0YjNW
07MgM`22PX80WYjN0;fm_@2g]kL0SHf=09:BTP31`L40ZJVY05ADE02CTi<08b<S01LG5`0H61P071`L
01TI6@0W9bL0Bd];0:6QX@2=SHd0MgMg01lO7`1VIVH0CDe=08>3P`38b<P0FUYJ05MGE`1bLW80OWin
09FEU@2`/;00b<S8096AT@1ADE40=cLg03Ti>@1CDe<0S8b<03@d=00`<300FUYJ05mOG`1iNGT08b<S
01dM7@1@D500e=CD0<30`02][Jd0/[:b08Z:RP2XZ:P0QXJ60;no_`3<c<`00P2QXJ404P1EEED06ATI
01/K6`0P8200>SXj08J6QP0K6a/05ADE01XJ6P1VIVH0TY:B08^;R`0j>SX0I6AT0<c<c03;bl/0fm_K
0<g=cA3ooooo000>ooooo`1M096AT@2o_kl0FUYJ07alO01>CTh0P82005QHF01JFUX0QXJ606m_K`1[
Jf/0P82003dm?@22PX80Jf][05iNGP1RHV80GemO07emO@15ADD0YJFU03hn?P1XJ6P0MWIf03Xj>P18
B4P0L71`03Lg=`0c<c<0EUIF05eMG@1hN7P0:b/[04Q8B028R8P0<c<c04Q8B00j>SX0B4Q802@T901/
K6`0<30`02d];@1@D500DU9B06QXJ00d=3@0?Cdm05YJFP13@d<0XJ6Q01lO7`0W9bL0Shn?05iNGP0Y
:BT0JFUY04Q8B00c<c<0ATI605iNGP0d=3@0SXj>05]KF`0b<S80=CDe075aL@0f=SH0Jf][06i^KP0U
9BD0FEUI03/k>`1NGUh0>c/k06MWI`2;Rh/0>SXj05mOG`1NGUh0FEUI02LW9`23Ph<0De=C03<c<`0n
?Sh0O7al04=3@`0L71`0@D5105UIF@0o?cl0NGUi0080Fe]K03<0Cdm?03<c<`1LG5`0FEUI03lo?`1T
I6@0Bd];06i^KP1;Bd/0Kfm_05mOG`1JFUX0Ng]k07emO@1KFe/0GemO04U9B@1LG5`0@d=305aLG016
ATH0HV9R07alO01ZJVX0KFe]09JFUP0k>c/0Fe]K05=CD`1FEUH0=CDe07EeM@1KFe/0EUIF06MWI`1?
Cdl0V9RH09>CT`0l?3`0G5aL06][J`26QXH0VYZJ03hn?P11@D40[Zj^04I6AP1CDe<0Kfm_04e=C@0g
=cL00P1@D500PP18B4P0S8b<03/k>`1TI6@0FEUI05]KF`2MWId0BDU907EeM@1@D500@41009>CT`2T
Y:@0?Shn059BDP1nOWh0FEUI03hn?P2`/;00YjNW06e]K@34a<@0ZZZZ05iNGP1KFe/0X:2P08Z:RP1m
OGd0>SXj01hN7P1`L700WYjN05ADE027QhL0AdM707moO`1BDU80CTi>0;>c/`1=CDd0Rh^;0451@@0j
>SX0]kNg05aLG00f=SH0QXJ605QHF00n?Sh0?Cdm06YZJP1VIVH0:BTY08J6QP1`L700J6QX09JFUP23
Ph<0OGem04m?C`1=CDd0HV9R03dm?@1aLG40@d=302l_;`18B4P0Jf][03dm?@1EEED0Kfm_059BDP0[
:b/0De=C06i^KP1DE5@0]kNg08V9R@1ADE409bLW075aL@15ADD0@41005mOG`0a<C40E5AD04a<C014
A4@0WYjN04i>CP0_;bl0HF5Q06e]K@1GEeL0>CTi06m_K`1kNg/0?clo02TY:@1eMGD0Jf][05ADE02Q
XJ40emOG06ATI031`L40W9bL02/[:`2QXJ40Ogmo030`<01gMgL0NWYj03dm?@2KVi/0>SXj01`L701K
Fe/0VIVI05YJFP2JVYX0@41001HF5P0V9RH0>c/k06e]K@0_;bl0Lg=c0861P@1^KVh>ooooo`0409JF
UP2h^;P0PH6102d];@809RHV03<0:b/[03@d=010@400Cdm?0492@P0n?Sh0@D5103dm?@0f=SH0=CDe
04U9B@1TI6@0OWin08R8R01gMgL0FEUI05mOG`13@d<0Bd];055AD@0j>SX0<S8b03Xj>P2_[jl0UiNG
07=cL`2=SHd0PX:2055AD@0O7al07AdM03Ti>@1GEeL0L71`08:2PP1cLg<0WinO0:VYZ@29RHT0LW9b
06m_K`1XJ6P0JVYZ06a/K01iNGT0>3Ph018B4P18B4P0IfMW03Ph>00T92@00P0N7Qh00`0W9bL092@T
03hn?P0205mOG`0J05=CD`0Q8B407AdM05UIF@1GEeL09RHV01DE5@19BDT0MGEe07QhN01IFET0A4A4
024Q8@0M7Ad07alO028R8P0Y:BT0=cLg08^;R`1BDU80F5QH05UIF@2^[Zh0^[Zj08J6QP1lO7`20861
P@0W09RHV02MWId0b<S808b<S02g]kL0Ti>C07MgM`1NGUh0De=C05IFEP24Q8@0WIfM0:n_[`22PX80
O7al07=cL`1SHf<0F5QH06a/K02FUYH0QHF5059BDP22PX80MGEe06YZJP1]KFd0L71`07YjNP22PX80
>CTi03lo?`10@400BTY:05EEE@1TI6@0PX:20:2PX02?Shl061PH00805ADE01d05QHF028R8P10@400
De=C05mOG`2:RXX0J6QX02h^;P2=SHd0D51@03/k>`1`L700=cLg01dM7@0H61P0:BTY03`l?00D51@0
4Q8B010@400D51@04a<C01@D5010@400D51@06e]K@1KFe/06ATI028R8P0205IFEP1K04];B`0k>c/0
9bLW02`/;011@D40I6AT0:>SX`1HF5P0Fe]K02/[:`0C4a<05QHF03Ph>00T92@0:RXZ01DE5@0G5aL0
=cLg09NGU`2IVIT0<C4a02XZ:P0o?cl0XZ:R07YjNP2=SHd0Lg=c05IFEP1:BTX0;bl_02PX:00i>CT0
71`L01XJ6P0O7al0NGUi0;fm_@36a/H0YJFU09jNWP2UYJD0alO70=3@d031`L40c/k>0>;RhP2d];@0
W9bL0<g=c@2DU9@0N7Qh071`L01VIVH0IfMW06YZJP1]KFd0OWin08>3P`1WIfL0H61P03hn?P16ATH0
Ogmo0<[:bP21PH40Hf=S059BDP1LG5`0MWIf0:JVYP3=cLd0?clo038b<P0k>c/0B4Q806=SH`20P800
TI6A0:BTY02CTi<0_Kfm0:ZZZP0X:2P09BDU09bLW03Fe]H0eMGE09jNWP1nOWh0WIfM0<W9b@0209fM
W@0E05UIF@0S8b<0<S8b05UIF@1>CTh09bLW01lO7`0c<c<0Q8B401PH600D51@06QXJ04Y:BP2:RXX0
BDU9075aL@2VYZH0QHF50:b/[02b/[80S8b<013ooooo000>ooooo`090861P@2o_kl0EUIF08:2PP1F
EUH0RXZ:06a/K01cLg<0QXJ60080E5AD0100L71`03Ti>@1gMgL0:BTY06a/K01ZJVX0HV9R03/k>`1E
EED0YjNW06ATI01>CTh0MWIf028R8P0X:2P0K6a/0P0a<C40n@1?Cdl0BTY:06m_K`0S8b<0C4a<07If
MP0/;2`0=CDe024Q8@0g=cL07alO05QHF00N7Qh0:RXZ03dm?@15ADD0Cdm?03/k>`0o?cl0FEUI05MG
E`28R8P0<30`01hN7P2ATI40EeMG01XJ6P1<C4`08b<S03@d=00o?cl0Cdm?02<S8`1fMWH0ADE502DU
9@0f=SH0J6QX03Lg=`1/K6`0GemO02HV9P1OGel0=3@d065QH@0d=3@0Q8B407IfMP0M7Ad0CDe=04M7
A`13@d<0>3Ph06m_K`1JFUX0;2`/020P802ATI40DU9B01`L700o?cl0@d=303@d=01gMgL0?3`l05iN
GP1:BTX0<c<c05YJFP1ZJVX0B4Q806m_K`1EEED0KFe]03lo?`1bLW80KFe]05mOG`1eMGD0LW9b04a<
C01cLg<0EeMG06e]K@13@d<0KFe]069RHP1ZJVX0J6QX07YjNP1RHV80U9BD05iNGP1[Jf/0GUiN05mO
G`13@d<0LW9b05UIF@1JFUX0Jf][08^;R`2RXZ80TY:B03Xj>P1UIFD0NWYj08N7Q`2RXZ80B4Q804I6
AP31`L40HV9R05ADE01YJFT0EeMG03Xj>P1BDU80FUYJ059BDP2BTY80=SHf07EeM@2;Rh/0De=C09:B
TP1@D500QhN7075aL@1KFe/0R8R808b<S00R8R80BTY:06EUI@1RHV80>CTi0<30`02NWYh0LG5a0<_;
b`2][Jd0OGem04a<C02:RXX0NWYj07AdM013@d<0=CDe07moO`2OWil0De=C06EUI@0l?3`0GemO059B
DP1GEeL0/;2`06ATI01YJFT0DU9B03Lg=`2BTY80?Cdm02/[:`1VIVH0@D5105IFEP1kNg/0P82006i^
KP0a<C40EEEE03Ti>@1JFUX0UYJF05aLG01EEED09bLW05ADE01LG5`0<S8b06ATI01iNGT0AdM704];
B`24Q8@0@D51051@D023Ph<0GEeM030`<01LG5`0CTi>07moO`37alL0T92@04U9B@0^;Rh0GUiN0451
@@1mOGd0QXJ604];B`1fMWH0?clo04Q8B02LW9`0?Shn04=3@`1iNGT0GemO05MGE`14A4@0Vi^K07Yj
NP0k>c/0:RXZ06][J`0h>3P0;2`/06][J`32`/80GUiN0:NWY`2FUYH0;Bd]07EeM@1KFe/0@41008:2
PP1SHf<0:RXZ09VIV@1VIVH07QhN04Y:BP22PX80?clo09NGU`1BDU808b<S02DU9@0[:b/0GemO03De
=@1@D500K6a/061PH00>ooooo`0;0;Bd]03EeMD0ZZZZ04I6AP0n?Sh0Cdm?05MGE`1BDU80GEeM02/[
:`0J6QX00P0E5AD03@0F5QH07AdM04m?C`1>CTh0=SHf05QHF01bLW80YJFU0:b/[02LW9`0DU9B02HV
9P0K6a/00P0F5QH09P0e=CD0M7Ad08:2PP2a/K40[Zj^07=cL`1OGel0;Rh^02LW9`0O7al0=SHf07Mg
M`1lO7`0Jf][08n?S`2=SHd0ADE5034a<@0`<300=cLg04E5A@29RHT0De=C010@400A4A40K6a/03@d
=00X:2P0ATI602HV9P0/;2`0:BTY03De=@11@D40<C4a01lO7`12@T80BDU90P0k>c/0E01IFET0HF5Q
04M7A`1kNg/0[jn_0;Bd]02[Zj/0Rh^;05eMG@14A4@0?Cdm0492@P1>CTh0=CDe06QXJ01PH600P820
0<c<c02XZ:P0MWIf05EEE@19BDT0?clo03Lg=`16ATH0Q8B40=3@d02MWId0`L7106a/K01ADE40FEUI
08j>SP2<S8`0Jf][030`<012@T80Cdm?06EUI@1iNGT0RHV909fMW@2c/k<0_kno0;>c/`2f][H0LW9b
07EeM@0Q8B405ADE01<C4`0E5AD0<S8b03Hf=P0_;bl0E5AD05IFEP1CDe<0FEUI07inOP2KVi/0SXj>
01PH600M7Ad0?Cdm06a/K01lO7`0=SHf05MGE`0H61P051@D02HV9P1jNWX0P82008J6QP1JFUX0QhN7
04=3@`0L71`06a/K03Hf=P0G5aL04A4A01DE5@804A4A0P0C4a<0D@0d=3@0GUiN09RHV00/;2`071`L
01PH600[:b/0MWIf0:6QX@1^KVh0MWIf03De=@0T92@05ADE04Y:BP1QHF409BDU06i^KP1BDU804a<C
02TY:@0h>3P09RHV01LG5`0g=cL0;bl_08J6QP1/K6`0De=C05mOG`1aLG40Rh^;0:b/[01KFe/0FUYJ
07moO`18B4P0EEEE07MgM`1fMWH0GUiN034a<@0[:b/0TY:B09jNWP3Fe]H0bl_;0:^[Z`2a/K40Xj>S
09^KV`2CTi<0[Zj^0<?3``34a<@0bl_;0;^k^`2h^;P0GEeM02/[:`0Q8B40820P01dM7@0];Bd092@T
04e=C@2/[:`0XJ6Q06][J`1mOGd0][Jf0=;BdP2TY:@0IfMW05iNGP1^KVh0[Jf]0<g=c@3AdM407AdM
01@D500201DE5@0W01LG5`1JFUX0e]KF0<[:bP3;bl/0`l?30<G5a@2m_Kd0][Jf096AT@2m_Kd0I6AT
028R8P0O7al0:2PX06=SH`0d=3@08R8R04e=C@1JFUX0?3`l02TY:@0m?Cd0M7Ad06][J`0f=SH0JVYZ
01TI6@0E5AD092@T0861P@16ATH0TI6A0;2`/01cLg<0<S8b05QHF02f][H0EeMG013ooooo000>oooo
o`1d0:b/[03:b/X0GemO09NGU`1BDU80Ti>C06UYJ@2;Rh/0SXj>069RHP1KFe/0O7al03De=@2FUYH0
H61P075aL@1OGel0HF5Q05QHF00k>c/0Y:BT05MGE`1mOGd0XZ:R03Xj>P0[:b/0LW9b02/[:`16ATH0
Kfm_04a<C01]KFd08B4Q05UIF@2CTi<0B4Q803hn?P0O7al0ATI6030`<01BDU8071`L03@d=01=CDd0
FUYJ051@D00e=CD0CDe=06][J`1gMgL0P82002<S8`0T92@0/;2`06YZJP0R8R80EeMG03Lg=`0l?3`0
FEUI07]kN`12@T80Q8B404U9B@0U9BD0@41006m_K`0c<c<0Hf=S05aLG00W9bL0HV9R03lo?`20P800
?Cdm08Z:RP1VIVH0:RXZ061PH01NGUh0?clo02h^;P27QhL0G5aL02XZ:P0L71`0WIfM04M7A`0c<c<0
LW9b05EEE@10@400Ng]k04=3@`1EEED0KVi^02`/;01ADE40N7Qh055AD@28R8P0C4a<07AdM01MGEd0
S8b<08B4Q01ZJVX0IfMW07QhN01OGel0J6QX04e=C@2LW9`0CTi>09NGU`1_Kfl206QXJ02P06UYJ@1O
Gel0UIFE051@D01XJ6P0RXZ:07QhN014A4@0Ng]k069RHP1MGEd0QHF508j>SP2[Zj/0Ti>C02d];@2F
UYH0K6a/08N7Q`2RXZ80CTi>05eMG@3:b/X0IFEU055AD@21PH40Kfm_04a<C01YJFT0KFe]05YJFP2>
SXh0<c<c06UYJ@1fMWH0EUIF08V9R@0h>3P0Ti>C06ATI01GEeL0UYJF08j>SP0S8b<0DU9B05MGE`1S
Hf<0>c/k0<?3``2>SXh0Ogmo0=[JfP2h^;P0OGem04U9B@2_[jl0Ph>309ZJVP1JFUX0>3Ph08J6QP29
RHT0ADE506ATI00n?Sh0K6a/04U9B@1lO7`0Xj>S05QHF02;Rh/0J6QX04=3@`23Ph<0BDU9038b<P1U
IFD0@41006e]K@2;Rh/0Y:BT06e]K@0a<C40AdM703Lg=`1>CTh0YjNW09bLW01`L700;bl_04a<C00o
?cl0;Rh^09FEU@1_Kfl0?Shn04Y:BP1TI6@09bLW06QXJ02QXJ40CTi>02`/;01=CDd0=cLg06MWI`1b
LW80E5AD05QHF00/;2`0DE5A02LW9`14A4@0Jf][04m?C`1/K6`0ADE50820P03:b/X0MgMg04A4A01d
M7@0H61P0410@01/K6`0[jn_079bLP0_;bl0:2PX065QH@1CDe<0@d=30:b/[03Ng]h0QhN7092@T02R
XZ80=CDe07QhN01DE5@0B4Q80:f][@2EUID0EUIF08b<S00m?Cd0:RXZ08Z:RP20P800@d=308b<S01S
Hf<0:RXZ04e=C@1KFe/0P82003hn?P1BDU80KVi^02PX:0kooooo00`0h^;R0>GUi@3IfMT0[Jf]0:BT
Y02JVYX0Shn?0:b/[015ADD061PH01DE5@0C4a<201TI6@1V01PH600o?cl0BDU903@d=00N7Qh0MWIf
0<?3``34a<@0HV9R03dm?@0I6AT071`L01LG5`0H61P0JFUY04A4A02>SXh0`<3009fMW@1jNWX0VYZJ
0;bl_01QHF40PH6109BDU00X:2P04Q8B01<C4`0H61P0KFe]069RHP0n?Sh08B4Q01PH600G5aL0:BTY
08j>SP0H61P06ATI07QhN00W9bL0=SHf02HV9P1CDe<0D51@069RHP18B4P071`L04Y:BP19BDT0=SHf
03hn?P1NGUh0@d=301hN7P0X:2P0G5aL09bLW027QhL0UYJF06UYJ@0o?cl0:2PX04E5A@1[Jf/0K6a/
04U9B@0l?3`0F5QH0<S8b02HV9P0R8R807=cL`1[Jf/0BTY:04Q8B01:BTX0GEeM08Z:RP2MWId0XZ:R
0<?3``2b/[80DE5A0451@@0m?Cd0IfMW07=cL`0T92@06ATI024Q8@0`<3009bLW02l_;`0i>CT0Ti>C
0<S8b03EeMD0h>3P0<?3``2ZZZX0:b/[1@0E5AD04@0k>c/06ATI01XJ6P0Y:BT0D51@09>CT`2FUYH0
Rh^;08n?S`1oOgl0JFUY04E5A@0[:b/06QXJ01PH601<C4`07AdM00805ADE00X0:BTY06EUI@25QHD0
Vi^K0;:b/P2;Rh/0:RXZ02d];@0o?cl05ADE0`0A4A406P0C4a<05ADE01DE5@1GEeL0U9BD0:2PX00Q
8B405aLG01TI6@0V9RH0J6QX05eMG@0j>SX0=CDe01LG5`0E5AD04a<C01PH601DE5@0?clo02@T9027
QhL0;bl_01LG5`0k>c/09BDU0P0n?Sh0>01;Bd/0TY:B04I6AP11@D40>SXj04M7A`1DE5@0HF5Q09RH
V01gMgL0D51@08B4Q02;Rh/0XJ6Q04Q8B01RHV80Ti>C0:f][@2OWil0MgMg0;bl_02a/K40RXZ:07Ui
N@1^KVh0YZJV0;Jf]P23Ph<0UIFE09jNWP23Ph<0WYjN0;Bd]01VIVH0<C4a02@T900M7Ad0820P03Xj
>P0O7al0=cLg07=cL`3=cLd0ZZZZ0=?Cd`3;bl/0dM7A0:RXZ02;Rh/0WinO0=7Ad@2l_;`0X:2P09VI
V@1SHf<071`L0`0E5AD02`15ADD0gMgM0=oOg`3Ph>00b<S80:b/[03GemL0aLG50<71`@1gMgL09bLW
00807QhN01T09BDU04M7A`1>CTh09BDU02PX:01;Bd/0MgMg05eMG@0g=cL0?Cdm05MGE`28R8P0UYJF
04a<C00V9RH0?Cdm0:2PX01DE5@0NGUi069RHP0k>c/09bLW03hn?P2NWYh0`l?3013ooooo000>oooo
o`1o0:JVYP2RXZ80A4A409>CT`1EEED0Q8B404=3@`24Q8@0S8b<06e]K@1ADE40De=C03lo?`2n_[h0
NGUi08:2PP10@400RXZ:07=cL`14A4@0RXZ:03De=@2HV9P0XJ6Q03<c<`0g=cL0KFe]01dM7@1/K6`0
LG5a04=3@`0i>CT0>SXj09fMW@2TY:@0CDe=04E5A@0/;2`0HV9R02TY:@13@d<0>3Ph075aL@1PH600
EEEE05MGE`0`<300P82009VIV@1lO7`0Jf][01@D501=CDd0`l?307]kN`0V9RH0DU9B0410@01PH600
XJ6Q08b<S01>CTh0QHF505YJFP0d=3@0I6AT06a/K00f=SH0IfMW05]KF`17AdL0Kfm_04E5A@2=SHd0
ATI609>CT`2ATI40>SXj06e]K@1dM7@0?Cdm02d];@2CTi<0F5QH02<S8`0L71`0Ph>303hn?P0J6QX0
Bd];055AD@0j>SX0Q8B404m?C`11@D40EEEE02d];@1:BTX0CTi>03hn?P1IFET0?3`l061PH00g=cL0
LG5a07EeM@1VIVH0H61P05mOG`16ATH0BTY:05=CD`24Q8@0>c/k07IfMP1<C4`0FUYJ05mOG`0l?3`0
=cLg069RHP0l?3`0HV9R05EEE@1>CTh0<c<c071`L00203dm?@1P06IVIP1dM7@0]kNg0820P00];Bd0
Kfm_05EEE@2=SHd0N7Qh03<c<`1aLG40^;Rh05]KF`1=CDd0OGem03/k>`1VIVH0X:2P07MgM`1JFUX0
EUIF04=3@`28R8P0MGEe04m?C`2CTi<0C4a<0<;2`P1jNWX0JFUY092@T029RHT0@d=307alO02?Shl0
OWin03Xj>P35aLD0I6AT055AD@2k^k/0UIFE04a<C01RHV80b<S80861P@23Ph<0J6QX0492@P2QXJ40
R8R803Hf=P1CDe<0?clo06QXJ00h>3P0O7al09bLW01`L700O7al05MGE`0l?3`0/k>c05MGE`0b<S80
JFUY03lo?`20P800ZZZZ0;>c/`1oOgl0@d=304];B`1JFUX0;Rh^07EeM@1[Jf/0IfMW0492@P1@D500
>SXj01`L701eMGD0EEEE03`l?01UIFD0OGem03<c<`13@d<0PX:202d];@0l?3`0ATI604M7A`1TI6@0
HF5Q0P1;Bd/0<`0W9bL0F5QH034a<@0o?cl0IVIV04a<C01SHf<0<30`04=3@`2WYjL0LG5a03hn?P1b
LW80KFe]059BDP14A4@0Shn?05QHF00U9BD08R8R061PH011@D4071`L09FEU@3IfMT0JFUY0:FUY@2b
/[80<C4a06UYJ@0d=3@06a/K07AdM029RHT0ADE509NGU`0X:2P04a<C04E5A@27QhL0J6QX071`L01g
MgL06QXJ020P800V9RH0EEEE01lO7`18B4P0IVIV06=SH`0>ooooo`0;0=SHf03Lg=`0Y:BT05QHF01E
EED0OWin04a<C011@D40Bd];03`l?00K6a/00P0E5AD201TI6@0902PX:00U9BD08R8R04a<C01ZJVX0
Ng]k0:>SX`1aLG40>SXj00805ADE03804a<C0451@@1JFUX0=SHf06a/K01WIfL0:BTY04m?C`2DU9@0
_[jn09bLW01jNWX0R8R803Hf=P0X:2P06ATI01DE5@0^;Rh0PH6106=SH`10@400<c<c03/k>`0`<300
Jf][05aLG01<C4`0Q8B404A4A00S8b<05aLG01hN7P13@d<0MGEe04=3@`0];Bd0L71`07alO00k>c/0
:RXZ02/[:`0d=3@0@T9203dm?@2>SXh0JVYZ0;Jf]P2d];@0RXZ:03De=@8071`L00P0820P03De=@1C
De<0EUIF09fMW@2m_Kd0H61P04E5A@808b<S00/0>3Ph05QHF01ZJVX0Z:RX0;no_`2d];@0Kfm_09:B
TP2j^[X0P82005eMG@0204Q8B00?02`/;00N7Qh05ADE01DE5@0L71`06ATI02/[:`0^;Rh0PH610<;2
`P3@d=00g=cL0;>c/`1MGEd06ATI00D05ADE00`0<C4a02DU9@0H61P05ADE01TI6@0o?cl0HF5Q096A
T@2KVi/0BDU901XJ6P0G5aL201HF5P0>01DE5@0j>SX0=3@d014A4@0C4a<05ADE02PX:01dM7@0DU9B
0:n_[`2f][H0ZJVY0;fm_@13@d<301<C4`0@01DE5@0@41005ADE02XZ:P13@d<0^KVi0<30`00f=SH0
5ADE01PH600[:b/0IVIV04M7A`0f=SH06QXJ01<C4`805ADE02405aLG04A4A01>CTh061PH02`/;01f
MWH05aLG051@D024Q8@0H61P04M7A`2DU9@0Jf][06IVIP1oOgl0N7Qh061PH00m?Cd0>CTi059BDP1d
M7@0VIVI09fMW@2QXJ40/K6a06EUI@1^KVh0J6QX0=3@d02o_kl0VIVI069RHP0N7Qh00P0E5AD0800Q
8B40AdM702PX:00_;bl0@d=305EEE@2OWil0UIFE0:ZZZP2?Shl0Kfm_0451@@0a<C40>CTi0451@@0`
<300<c<c08B4Q039bLT0h^;R0<S8b034a<@0U9BD08Z:RP2i^KT0[jn_0:j^[P16ATH08B4Q03Lg=`2<
S8`0Fe]K0P0E5AD09`0a<C40YJFU0;:b/P1VIVH0QHF50:JVYP2i^KT0^k^k0<_;b`3@d=00CTi>01TI
6@0O7al09bLW02/[:`0k>c/0BDU902@T900U9BD0<c<c03`l?01`L700MGEe061PH018B4P0Ng]k05aL
G01eMGD0OWin09:BTP2FUYH0SXj>08R8R00j>SX0=cLg02/[:`0l?3`0/k>c08B4Q00@ooooo`003_oo
ool0d02TY:@0_Kfm05]KF`1mOGd0DU9B08Z:RP0e=CD0<S8b04];B`1LG5`0DE5A05iNGP0S8b<0QhN7
055AD@1_Kfl0?3`l05YJFP1SHf<0@d=30820P00/;2`0B4Q807YjNP0c<c<0?clo07]kN`0H61P0<C4a
05=CD`1DE5@0BTY:01PH601?Cdl0MGEe03Ti>@11@D408b<S0451@@0`<300DU9B01PH600l?3`0=SHf
05UIF@1>CTh09bLW06QXJ01/K6`0RXZ:06a/K00M7Ad09BDU08:2PP1@D5009RHV04i>CP0I6AT09bLW
030`<01ADE40<30`08R8R00Y:BT06a/K04Y:BP1UIFD0<S8b05mOG`1KFe/0;bl_05eMG@16ATH0Jf][
02`/;01aLG40Ng]k02d];@1gMgL0OGem05QHF00i>CT0MgMg05MGE`0X:2P08R8R09>CT`1=CDd06QXJ
04A4A00b<S80I6AT0<30`023Ph<0Lg=c05MGE`0/;2`0Fe]K069RHP1>CTh0DE5A0410@01ZJVX0A4A4
07AdM01_Kfl0GEeM06QXJ01`L700ADE503lo?`1GEeL0OWin05iNGP2OWil0ADE5065QH@1fMWH0IFEU
04M7A`1FEUH0@D5107QhN01mOGd0@D51038b<P24Q8@0BDU905=CD`1KFe/0M7Ad0<[:bP2GUiL0ATI6
07moO`1FEUH0/[:b0:>SX`11@D40KFe]0;Fe]@17AdL0F5QH08b<S01MGEd0AdM706a/K01JFUX0GemO
07YjNP0b<S80F5QH05ADE01YJFT0ZZZZ04i>CP1SHf<0GemO06e]K@2OWil0Y:BT03Xj>P1GEeL0RXZ:
05=CD`1;Bd/0^;Rh0;6a/@1MGEd0_;bl092@T00Z:RX0=3@d09BDU01VIVH0QHF502XZ:P0K6a/0PX:2
09NGU`1DE5@0EEEE04a<C02QXJ40EEEE06][J`1eMGD0A4A407UiN@13@d<0;Rh^05ADE01ADE40;Rh^
07=cL`0o?cl0LG5a08^;R`2:RXX0P82003hn?P1nOWh0KFe]061PH01jNWX0JVYZ096AT@0d=3@2071`
L01403Xj>P27QhL0DE5A01TI6@1lO7`0TY:B03Hf=P11@D40MWIf014A4@0c<c<0FEUI04m?C`1UIFD0
KFe]03hn?P0g=cL0ATI608Z:RP0c<c<0>c/k04A4A00_;bl0Ogmo059BDP1>CTh0Xj>S0410@00l?3`0
XJ6Q0;2`/01OGel0=SHf06QXJ00d=3@0?3`l07IfMP20P800@D5101lO7`2FUYH0/[:b02TY:@1nOWh0
P82002/[:`1aLG40@T9202DU9@1UIFD0M7Ad04=3@`2=SHd0>SXj01`L701ADE40HV9R051@D01]KFd0
I6AT02DU9@0X:2P0;2`/05MGE`0R8R80ADE505QHF01EEED>ooooo`0Y0::RXP2[Zj/0ZZZZ03lo?`0_
;bl0Cdm?05UIF@0a<C40:b/[03Xj>P17AdL0D51@04Y:BP12@T80<30`02@T900U9BD08B4Q0410@00]
;Bd0AdM70:JVYP2@T900?clo01DE5@0C4a<051@D06IVIP0l?3`0DE5A071`L00a<C407AdM04m?C`1@
D500Z:RX08B4Q00S8b<05QHF01`L700Z:RX00P0a<C40?@0h>3P0?clo06EUI@1QHF40@T9204];B`1<
C4`0@T9209ZJVP2?Shl0SXj>05EEE@0I6AT06QXJ020P801CDe<0@T9204];B`29RHT0MWIf01/K6`0D
51@03Ph>01@D500C4a<08b<S07YjNP1;Bd/0;bl_05IFEP1`L700G5aL07IfMP1BDU80;Rh^03hn?P0d
=3@0;bl_03lo?`2PX:00[Zj^0:>SX`2=SHd0AdM701XJ6P0G5aL09BDU04A4A02?Shl0[Zj^0820P01E
EED0LG5a0;>c/`2e]KD0H61P03<c<`0_;bl092@T01/K6`0J6QX00`0G5aL02@0K6a/0ADE505QHF01W
IfL0O7al0;bl_01?Cdl0C4a<01HF5P0501DE5@0301/K6`0c<c<0820P00805QHF00D071`L0410@02J
VYX0R8R803<c<`0401DE5@0301LG5`0_;bl0>3Ph00@05ADE00L0JVYZ03Ph>01iNGT0VYZJ0;Ng]`3F
e]H09BDU0080410@0P0E5AD0<P0C4a<05aLG03Ti>@1LG5`0`/;20<K6aP1QHF406a/K01TI6@0`<300
Fe]K059BDP0`<3005QHF01DE5@0D51@04a<C01LG5`0Q8B40GEeM01dM7@0G5aL0Fe]K08N7Q`1^KVh0
U9BD06IVIP2DU9@0Shn?038b<P0R8R80;Bd]03`l?01WIfL0GEeM04Q8B00Z:RX0?Shn08n?S`2MWId0
[:b/0;>c/`0^;Rh0>3Ph08Z:RP3GemL0f=SH08b<S00m?Cd05ADE0P0F5QH00`15ADD0:RXZ03@d=002
01LG5`0a02HV9P2GUiL0YJFU0:j^[P2m_Kd0YJFU092@T01UIFD0IfMW06i^KP1IFET0@T920;^k^`3?
cll0e]KF0<G5a@2b/[80MWIf0:^[Z`2g]kL0ZZZZ07MgM`1JFUX0>3Ph02DU9@1BDU80WYjN08>3P`1[
Jf/0Ti>C05iNGP0g=cL0:RXZ06=SH`2e]KD0dm?C0=GEe@3?cll0`l?302DU9@0S8b<09BDU02PX:00R
8R80=3@d04e=C@0V9RH09BDU034a<@0202h^;P0A07IfMP2NWYh0XZ:R0:^[Z`23Ph<0B4Q804e=C@2Y
ZJT0]KFe0;Jf]P1LG5`0@D5104Y:BP0g=cL0K6a/0:NWY`0M7Ad04?ooool000kooooo0?l0[:b/0<[:
bP1TI6@0P82007IfMP2n_[h0GemO08>3P`1oOgl0KVi^0;Vi^@2XZ:P0?3`l09fMW@1NGUh0Xj>S06i^
KP1FEUH0BTY:0861P@2l_;`0F5QH08R8R01ZJVX0QHF509fMW@2a/K40;bl_03Ph>00k>c/0YJFU07Yj
NP0[:b/0MGEe069RHP0d=3@0L71`03Lg=`0Z:RX0CTi>07AdM00U9BD0ADE503`l?021PH40RXZ:055A
D@1dM7@0Fe]K08:2PP2XZ:P0@410038b<P2SXj<0?3`l05iNGP1kNg/0=CDe051@D01JFUX0Cdm?02DU
9@2MWId0IfMW01lO7`1TI6@0B4Q800l?3`1GEeL0OGem03`l?01NGUh09RHV07MgM`1?Cdl0QhN707Mg
M`11@D40]KFe0<G5a@1HF5P0?clo07IfMP0g=cL0Bd];05QHF02NWYh0MgMg01/K6`16ATH0<C4a03`l
?01RHV80Kfm_06MWI`1@D5005QHF07]kN`23Ph<0EUIF04I6AP0d=3@0RHV904];B`1MGEd0=cLg07Ui
N@2XZ:P0Shn?04];B`0e=CD0Kfm_0:^[Z`1LG5`0UIFE034a<@1mOGd0Ti>C09FEU@16ATH0H61P01`L
701lO7`0RXZ:02h^;P0[:b/0V9RH096AT@1?Cdl0ADE502`/;01VIVH0VYZJ04U9B@2BTY80@D5108J6
QP2KVi/0@d=307emO@32`/80410@03`l?02IVIT0JFUY04m?C`1oOgl0>SXj04I6AP1jNWX0>c/k051@
D00E5AD0:b/[0:^[Z`1fMWH0OGem04];B`0J6QX0J6QX09bLW01BDU80Fe]K06m_K`1ADE4092@T0;2`
/01mOGd0IFEU0;fm_@2GUiL0KVi^04I6AP1`L700GEeM08F5Q@0f=SH0>3Ph0:>SX`2@T900EUIF03Lg
=`0W9bL0GemO05iNGP1mOGd0X:2P038b<P1[Jf/0EeMG03Ph>01HF5P0;Bd]03Ph>02QXJ40ATI609bL
W02^[Zh0V9RH06MWI`0Q8B40HV9R07inOP1MGEd0BTY:01lO7`1eMGD0>CTi03lo?`1NGUh0?clo07Mg
M`1RHV80>SXj05aLG021PH40>3Ph04E5A@25QHD0<S8b03<c<`15ADD0De=C05YJFP1NGUh0@T92034a
<@0Q8B40EeMG02PX:00k>c/0Bd];02TY:@0j>SX0=3@d04I6AP2RXZ80ADE502`/;01WIfL0FUYJ04];
B`0n?Sh0I6AT02h^;P0C4a<0=CDe06][J`13@d<0=cLg0;:b/P2BTY80E5AD09bLW028R8P001L0;bl_
09nOW`1iNGT0>CTi05iNGP1MGEd0B4Q809ZJVP1IFET08R8R05EEE@1GEeL0Cdm?07QhN01UIFD0<30`
02l_;`0`<300GEeM02l_;`1aLG40TY:B05mOG`0>ooooo`0;05]KF`17AdL0ZZZZ05YJFP0e=CD0:RXZ
04];B`1:BTX0@d=3055AD@1BDU800P13@d<06@1LG5`0IfMW065QH@1HF5P0?clo03/k>`0[:b/0ATI6
0<c<c02PX:00CDe=02TY:@0L71`0:b/[0:^[Z`2?Shl0];Bd08R8R00_;bl0@D51034a<@0O7al0MgMg
04=3@`0>3Ph00`0=3@d200h>3P0F01/K6`0I6AT07QhN038b<P1IFET0@d=303Hf=P0n?Sh0C4a<0=3@
d01FEUH0=SHf02XZ:P0J6QX0CTi>02l_;`0W9bL09BDU06QXJ01JFUX04A4A010@40804A4A00`05ADE
01LG5`1oOgl0>SXj020P800c<c<0MgMg02l_;`0Y:BT0NWYj06QXJ00J6QX201DE5@0E03Lg=`2SXj<0
;Rh^01lO7`0g=cL0Ph>308^;R`1OGel0De=C07EeM@2MWId0_Kfm09nOW`2=SHd0TI6A0<S8b02BTY80
=SHf02DU9@0T92@061PH00<05QHF00/06ATI01PH600Z:RX0IFEU0451@@1[Jf/0BTY:0:b/[01BDU80
BTY:01/K6`0601DE5@0303Ti>@0L71`071`L008071`L00@0?Cdm0<?3``1NGUh0@d=3100E5AD00`0J
6QX07AdM04U9B@0201DE5@0;014A4@0C4a<0JVYZ03/k>`1@D500NGUi05mOG`31`L408R8R014A4@0C
4a<00P0A4A40=`0C4a<0<30`02@T902d];@0emOG0=gMg@2c/k<0=CDe02DU9@1UIFD0GemO08F5Q@0S
8b<04a<C010@400B4Q804A4A01<C4`0F5QH0EeMG03`l?00M7Ad07QhN09RHV03=cLd0WinO0;Jf]P32
`/80Fe]K01`L700T92@0?3`l04A4A00d=3@08R8R0410@01VIVH0DE5A03lo?`1nOWh0LG5a09BDU02?
Shl0C4a<06=SH`3=cLd0clo?0820P00^;Rh061PH01LG5`0M7Ad0A4A401PH600J6QX00P0E5AD0A00F
5QH0D51@04i>CP1YJFT0MWIf0;Ng]`1eMGD0DE5A04a<C015ADD0Bd];08N7Q`2>SXh0b<S80;^k^`33
`l<0QhN707=cL`1hN7P0=cLg03De=@16ATH0JFUY06YZJP0h>3P0<C4a06][J`3?cll0_[jn05iNGP0W
9bL0=SHf03hn?P1RHV80a/K60=3@d03Kfm/0e]KF0:b/[00c<c<0=3@d02/[:`0U9BD0820P02TY:@1>
CTh0<S8b03Hf=P1DE5@0?3`l03Lg=`1>CTh0[Jf]0<30`02n_[h0bLW908^;R`2][Jd0emOG0;2`/01o
Ogl0Bd];04m?C`1SHf<0TI6A0<K6aP2n_[h0FUYJ4?ooool000kooooo04T0PH610:n_[`1HF5P0Ogmo
05IFEP2@T900GUiN0:>SX`2LW9`0EUIF05YJFP1aLG40Bd];09:BTP10@400QXJ606UYJ@1OGel0DU9B
05IFEP2DU9@0EeMG08Z:RP1/K6`0<c<c04E5A@1eMGD09BDU04Q8B010@400FUYJ04U9B@0X:2P0PH61
06e]K@0b<S80@41002TY:@14A4@0<C4a04Q8B00X:2P0CDe=03lo?`1=CDd0Cdm?03/k>`1HF5P0HV9R
06e]K@1QHF40CTi>04M7A`2XZ:P0?clo02d];@1PH600ATI606][J`25QHD0H61P02`/;01LG5`0AdM7
02HV9P1UIFD0F5QH01dM7@1=CDd0F5QH03lo?`1ZJVX0:2PX0080Bd];02@0SHf=07alO00V9RH0IVIV
07moO`1bLW80=cLg08f=S@0l?3`06a/K01TI6@1nOWh0D51@024Q8@1FEUH0ADE504e=C@2<S8`0F5QH
04U9B@1RHV80;2`/06YZJP1:BTX0G5aL061PH010@400G5aL05UIF@1^KVh0D51@07QhN01WIfL0E5AD
04i>CP0k>c/206e]K@0a03dm?@2DU9@0AdM706ATI01[Jf/0@D51059BDP1^KVh0>c/k069RHP1nOWh0
De=C03Lg=`1kNg/0@T9205=CD`1lO7`0T92@0;Fe]@1dM7@0?Shn09JFUP1IFET0TY:B06QXJ00o?cl0
Rh^;0:FUY@1PH600Bd];07YjNP0j>SX0J6QX09JFUP1/K6`0AdM705ADE018B4P0K6a/05YJFP10@400
[Zj^04Q8B025QHD0J6QX05eMG@2QXJ40MgMg03Hf=P0206e]K@1b06QXJ00c<c<0^k^k05aLG01QHF40
^;Rh09nOW`1LG5`0Fe]K08n?S`1;Bd/0MgMg061PH011@D40OWin07MgM`17AdL0@d=303Xj>P1jNWX0
A4A4092@T02WYjL0HV9R07UiN@19BDT09bLW061PH015ADD0:b/[06m_K`0a<C40LG5a09ZJVP2MWId0
Ng]k03Lg=`1CDe<0GEeM03Xj>P1eMGD0JVYZ07inOP1CDe<0E5AD05=CD`0f=SH0S8b<096AT@18B4P0
Hf=S07UiN@0/;2`0BDU908j>SP1>CTh0<c<c05]KF`0g=cL0FUYJ07UiN@1ZJVX0De=C02d];@1EEED0
:BTY04e=C@1dM7@0CTi>06MWI`0a<C40IfMW0;Zj^P1gMgL0ADE507]kN`1/K6`0A4A404a<C02HV9P0
HV9R01XJ6P0n?Sh0IFEU030`<01GEeL0^[Zj071`L01ZJVX0^;Rh08Z:RP0S8b<0DU9B05IFEP0c<c<0
K6a/05IFEP0R8R80TY:B04Y:BP0R8R80E5AD05QHF00h>3P0MGEe07AdM00f=SH0JVYZ07IfMP1aLG40
9RHV04M7A`1[Jf/0ATI63_ooool02`2ZZZX0MgMg06e]K@2FUYH0E5AD04A4A01?Cdl0FUYJ05EEE@1H
F5P0H61P0080J6QX00L0NWYj07QhN01kNg/0Q8B409JFUP2@T900YZJV0080^k^k00l0_Kfm0:RXZ02P
X:00H61P059BDP28R8P0W9bL0451@@0m?Cd0DU9B07AdM02FUYH0W9bL0;bl_014A4@00P0?3`l01@0>
3Ph03@d=00h>3P0=3@d04Q8B00<05ADE03T05QHF02/[:`13@d<0FEUI07inOP3?cll0Q8B4059BDP18
B4P0A4A403@d=00/;2`0PH6106IVIP1UIFD0=SHf01<C4`0?3`l0410@01@D500S8b<0Fe]K0492@P1@
D500<C4a04];B`1ZJVX0?3`l02l_;`0/;2`0EeMG01hN7P0E5AD07alO08V9R@1cLg<0?Cdm01PH600L
71`0820P051@D00O7al05aLG01`L701GEeL0RHV905YJFP1VIVH0OWin07emO@27QhL0>SXj02l_;`0/
;2`08R8R02@T900[:b/00P0b<S806P0d=3@0BTY:071`L00j>SX0L71`05EEE@2c/k<0KFe]059BDP0d
=3@06QXJ01PH600F5QH05ADE01LG5`0E5AD0ADE502`/;00V9RH0820P030`<01kNg/0IfMW04i>CP1k
Ng/0820P0`0E5AD0100G5aL05QHF05UIF@0L71`201@D500801HF5P20P800EUIF03lo?`1jNWX0?Cdm
09VIV@15ADD2018B4P804A4A0280:2PX096AT@10@400<C4a04=3@`2OWil0b<S80:NWY`2ATI40VYZJ
08B4Q02;Rh/0GEeM030`<00@41004a<C014A4@0B4Q8092@T06][J`1gMgL0I6AT05MGE`2^[Zh0b<S8
0=KFeP36a/H0][Jf06ATI01bLW80Q8B405IFEP0e=CD09BDU0P0J6QX0500P8200;bl_01/K6`0e=CD0
X:2P0:n_[`2b/[80/K6a0=GEe@3Hf=P0e=CD0<_;b`19BDT0:RXZ02/[:`1CDe<06a/K01@D500F5QH0
5ADE0P0C4a<06P0J6QX05aLG04a<C01<C4`0QXJ607IfMP1cLg<0HV9R05=CD`1TI6@0Xj>S0;Fe]@2`
/;00Q8B407]kN`14A4@0=3@d02XZ:P0L71`06a/K01`L700S8b<0<30`05mOG`1lO7`0Jf][0P2PX:00
9`1lO7`0PX:206m_K`1IFET0Hf=S09FEU@31`L40d];B0<k>cP2a/K40Ti>C07moO`1YJFT0FEUI03@d
=010@400O7al07moO`1iNGT0UIFE0::RXP2XZ:P0ZJVY0;6a/@2QXJ40Rh^;0:6QX@1[Jf/0Fe]K0;6a
/@3<c<`0UiNG09nOW`2TY:@0RXZ:04i>CP0Y:BT0;bl_08N7Q`0@ooooo`003_ooool01`2b/[80`/;2
05eMG@2<S8`0PX:20:NWY`1<C4`00P1YJFT0g01QHF40Zj^[05QHF00c<c<0RXZ:0492@P2HV9P0>3Ph
07EeM@1IFET0KFe]08J6QP0M7Ad0RXZ:07inOP1JFUX0QhN706i^KP0J6QX0=SHf059BDP1LG5`0IVIV
028R8P27QhL0MgMg03Ph>01@D500=CDe05iNGP10@400B4Q801`L700];Bd0>c/k05eMG@16ATH08B4Q
06=SH`1WIfL0MGEe05QHF00I6AT0BTY:08Z:RP12@T80>CTi051@D00X:2P0ATI606EUI@1DE5@0BTY:
07YjNP0Q8B406ATI04e=C@1NGUh08b<S06YZJP10@400>3Ph06MWI`0g=cL0IFEU02XZ:P1_Kfl0Ph>3
034a<@20P800W9bL03Ph>00V9RH0W9bL05MGE`17AdL0IVIV0:BTY01<C4`05ADE0410@00X:2P0IVIV
09jNWP20P800CTi>04=3@`0X:2P0UYJF06=SH`1JFUX0BDU904M7A`1_Kfl0@d=306a/K01=CDd0Shn?
09>CT`1ZJVX0AdM702/[:`2;Rh/0NWYj04Y:BP2CTi<0>SXj07=cL`21PH40K6a/07=cL`12@T80<30`
07alO01TI6@0>CTi04=3@`2DU9@0E5AD06][J`1>CTh0MgMg0:n_[`2LW9`0AdM708n?S`19BDT0WinO
07]kN`1;Bd/0UiNG0:BTY014A4@0RXZ:08b<S017AdL0IVIV06i^KP1EEED0Jf][04a<C00l?3`0HF5Q
05EEE@1cLg<0/K6a06IVIP1_Kfl0OGem05YJFP2j^[X0QhN704A4A01/K6`0KFe]05YJFP0m?Cd0`/;2
08n?S`1TI6@0^KVi09:BTP0k>c/0Shn?0;2`/01dM7@0S8b<05IFEP0d=3@0RHV909JFUP1ZJVX0DE5A
04Y:BP2d];@0Kfm_0820P02KVi/0>3Ph0:>SX`1QHF40<S8b04i>CP0n?Sh0=3@d09BDU018B4P0IVIV
08F5Q@1ZJVX0NWYj03@d=01KFe/0MgMg06QXJ02KVi/0M7Ad07UiN@12@T80FEUI04Q8B00f=SH0PH61
06YZJP12@T80LG5a08J6QP0j>SX0>3Ph09^KV`0a<C40>3Ph05QHF01;Bd/0DU9B04m?C`0l?3`0A4A4
02l_;`1WIfL203Hf=P0_06IVIP1YJFT0SXj>05IFEP1:BTX0/;2`07inOP11@D40Ti>C09RHV01:BTX0
?Shn07]kN`1FEUH0<30`06QXJ02KVi/0FEUI04a<C02k^k/0IfMW0451@@2GUiL0J6QX03De=@2:RXX0
DE5A03Hf=P2c/k<0UYJF03Ti>@2NWYh0<S8b0451@@2:RXX0U9BD061PH026QXH0GUiN024Q8@0];Bd0
?Cdm05QHF00a<C40F5QH07UiN@14A4@03_ooool0:@25QHD0Rh^;0:j^[P2=SHd0CDe=03<c<`0X:2P0
8B4Q03Lg=`0a<C4092@T02LW9`0g=cL0?Shn02h^;P0d=3@0MWIf0<W9b@2>SXh0ATI602`/;00Z:RX0
Ng]k0<;2`P0o?cl05ADE014A4@0>3Ph07alO07UiN@0^;Rh09RHV03Hf=P20P800FUYJ0:^[Z`22PX80
5ADE00h>3P0=3@d03Ph>00803@d=00<03Ph>01@D500G5aL00P0G5aL02`0L71`0@4100:>SX`3Ogml0
eMGE0=WIf@2e]KD0`l?309RHV01jNWX0Jf][0080T92@04P0Ng]k05iNGP1MGEd0HF5Q05aLG01UIFD0
QXJ60;>c/`1CDe<0EUIF06QXJ01[Jf/0Lg=c05QHF01]KFd0HV9R03`l?01ADE408B4Q069RHP3IfMT0
NWYj05EEE@0H61P051@D01<C4`0@41005ADE01XJ6P0I6AT0KVi^0<k>cP1>CTh0[jn_09BDU03<c<`0
b<S809VIV@29RHT0OGem06YZJP1MGEd0EeMG04U9B@0l?3`0;bl_03hn?P1TI6@0YJFU0:^[Z`1ZJVX0
/k>c09FEU@1dM7@0J6QX02XZ:P0N7Qh08b<S02@T900R8R808B4Q06][J`19BDT0De=C03dm?@1XJ6P0
GemO01dM7@0o?cl0I6AT065QH@0E5AD301@D500=01HF5P1BDU80NGUi03lo?`1=CDd0MWIf0<[:bP3S
hn<0YZJV0;jn_P0g=cL0NWYj07QhN002010@400=01HF5P15ADD0J6QX04M7A`1CDe<0GEeM09fMW@36
a/H0c/k>06a/K01JFUX0VIVI04Y:BP0206IVIP0Y065QH@0T92@03`l?01DE5@0W9bL0EUIF0451@@0C
4a<07alO07IfMP2/[:`0b<S80=?Cd`2l_;`0Ogmo06YZJP1iNGT0R8R805iNGP0[:b/07QhN01`L700J
6QX07AdM034a<@0_;bl0ADE506EUI@1VIVH0Zj^[0;Bd]03Lg=`0bl_;0=KFeP2OWil0JFUY051@D02h
^;P0Hf=S01TI6@0G5aL00P0E5AD0A@0D51@07AdM028R8P0I6AT0ADE50410@01TI6@0>SXj07=cL`34
a<@0f][J0=_Kf`2c/k<0MGEe09FEU@1IFET0EEEE05MGE`0i>CT0=cLg020P800T92@0820P01XJ6P0M
7Ad08R8R05iNGP31`L40bLW90:2PX00a<C405ADE02DU9@1PH600LW9b04e=C@19BDT0RXZ:09ZJVP27
QhL0L71`05]KF`34a<@0`L710;Rh^02TY:@0O7al09NGU`2KVi/0UIFE0;:b/P3AdM40bLW904U9B@0Q
8B40:2PX05mOG`38b<P0`l?30;Rh^02n_[h0V9RH05eMG@13@d<0=cLg028R8P0Q8B409BDU06][J`0@
ooooo`003_ooool0f02HV9P0`L7105UIF@1XJ6P0Kfm_0:^[Z`1NGUh0SHf=06][J`0m?Cd0JFUY06EU
I@0o?cl0J6QX028R8P2EUID0F5QH055AD@0P8200KFe]08N7Q`12@T80U9BD03Hf=P18B4P0R8R8092@
T00T92@0>c/k03lo?`2CTi<0F5QH02LW9`29RHT0?Shn03Ph>01_Kfl0@d=304A4A01ZJVX0EUIF02<S
8`0o?cl0=CDe0861P@1oOgl0@d=3069RHP18B4P0U9BD09bLW00e=CD0Fe]K09JFUP0m?Cd0I6AT08F5
Q@10@400D51@06QXJ00b<S80BDU9096AT@1@D5007alO06UYJ@17AdL0<C4a0:JVYP1aLG40?Shn051@
D00l?3`0OGem03`l?01mOGd0Lg=c04I6AP2a/K40`L7106QXJ00d=3@0SXj>01/K6`0R8R80=CDe07mo
O`1UIFD06ATI03lo?`11@D40FEUI07YjNP1QHF40DE5A0410@00R8R80IfMW05ADE01FEUH0BTY:03Xj
>P1KFe/0AdM706m_K`17AdL0JFUY05UIF@1OGel0ATI602DU9@1]KFd0L71`04];B`2<S8`0EUIF05QH
F01dM7@0JFUY07alO01nOWh0EEEE06QXJ01JFUX0Jf][04M7A`2:RXX0DU9B05MGE`1WIfL0U9BD0;Vi
^@2ATI40@d=308^;R`1EEED0Vi^K07=cL`1?Cdl0U9BD08B4Q010@400ATI607]kN`19BDT0J6QX06YZ
JP0];Bd0H61P055AD@17AdL0CTi>01lO7`1BDU80Zj^[06e]K@1WIfL0ADE504Y:BP2UYJD0Xj>S03dm
?@1^KVh0EEEE0451@@0[:b/0Zj^[09>CT`1/K6`0[jn_09>CT`0i>CT0JVYZ08R8R01NGUh0SHf=01/K
6`0];Bd0^;Rh09FEU@1NGUh0HV9R02`/;02CTi<0JVYZ08:2PP2:RXX0>c/k069RHP1;Bd/09bLW06IV
IP0Y:BT0GEeM0:VYZ@1<C4`0AdM706QXJ01fMWH0HV9R02TY:@2IVIT0Z:RX06YZJP2QXJ40Fe]K08n?
S`0a<C40TY:B07UiN@17AdL0R8R806YZJP16ATH0_[jn0:RXZ080?3`l03`0Ti>C01LG5`10@400NWYj
05eMG@1CDe<0JFUY04Q8B00T92@0CDe=08R8R00a<C40=SHf059BDP0[:b/0QhN7061PH01NGUh0`L71
03Ti>@16ATH0V9RH09ZJVP1DE5@0F5QH09>CT`0N7Qh0:2PX06=SH`1LG5`0:b/[06e]K@2`/;00Lg=c
051@D034a<@0HF5Q020P801]KFd0HF5Q02LW9`1NGUh0DE5A02XZ:P2OWil0ATI6020P801DE5@0CDe=
034a<@2@T900F5QH02h^;P0g=cL0AdM705EEE@0U9BD0P8200861P@1HF5P>ooooo`0Z06e]K@1TI6@0
IFEU06m_K`0j>SX0;Rh^02XZ:P0V9RH07alO04m?C`1>CTh0LW9b07emO@17AdL0<S8b028R8P0f=SH0
UIFE04A4A00[:b/0:2PX028R8P1nOWh0YZJV01TI6@0E5AD04a<C00l?3`0@4100@410075aL@0l?3`0
<C4a06UYJ@1EEED0Z:RX06MWI`1;Bd/06ATI00d=3@0<30`03Ph>0`0=3@d09`1OGel0PX:208R8R02O
Wil0^[Zj0:n_[`3=cLd0bl_;0<71`@34a<@0g=cL0:NWY`2?Shl0WYjN0:BTY02VYZH0V9RH08N7Q`1J
FUX0?clo02XZ:P0_;bl0LW9b07UiN@1KFe/0D51@03Xj>P0b<S80>3Ph05EEE@12@T80Hf=S0820P023
Ph<0R8R80;Ng]`1kNg/0DU9B069RHP0201<C4`0_01DE5@0A4A405ADE028R8P1LG5`0^k^k0<k>cP3;
bl/0inOW0=3@d03IfMT0bl_;09BDU012@T80?clo0451@@18B4P0EUIF04Q8B010@400?3`l05EEE@1P
H600Bd];06UYJ@2YZJT0_;bl06YZJP2JVYX0^[Zj07IfMP1UIFD0@D5103@d=00Z:RX0:BTY05IFEP1D
E5@0@d=307QhN02WYjL0=SHf01`L700X:2P0Cdm?096AT@0I6AT00`0E5AD0900K6a/0Shn?08J6QP1V
IVH0D51@07EeM@2;Rh/0_Kfm0<o?c`3Jf]X0Lg=c06YZJP32`/80I6AT05YJFP1kNg/0De=C01`L700C
4a<09RHV09:BTP1VIVH0Cdm?09ZJVP2^[Zh0De=C06YZJP1TI6@0Lg=c06EUI@0n?Sh0GemO024Q8@0W
9bL0B4Q804e=C@80410@06P04a<C02d];@1KFe/0aLG509fMW@2SXj<08b<S01TI6@0^;Rh0C4a<03lo
?`0W9bL071`L01HF5P0J6QX0:BTY04=3@`1?Cdl0CTi>02d];@0W9bL0F5QH09fMW@37alL0HV9R05EE
E@1^KVh0Ph>302TY:@1BDU80GemO01XJ6P0F5QH06a/K01LG5`0E5AD061PH02@T900J6QX0@d=304I6
AP1NGUh09RHV04I6AP2ZZZX0dm?C0<g=c@1GEeL0Fe]K08>3P`1GEeL0?clo06EUI@1fMWH0E5AD03Xj
>P0g=cL0<C4a02<S8`0P820092@T02/[:`1VIVH0/k>c06ATI00U9BD09bLW05=CD`1:BTX0=cLg02h^
;P0e=CD0EUIF08B4Q01mOGd0FEUI05ADE01[Jf/0?clo034a<@0X:2P0;bl_04];B`0P82006ATI02@T
900b<S80FUYJ01`L700M7Ad08R8R02d];@35aLD0_kno09jNWP2ATI40QHF503Hf=P0c<c<0=SHf02TY
:@0T92@0=CDe06QXJ13ooooo000>ooooo`3o07inOP2_[jl0De=C08J6QP1dM7@0[Jf]05iNGP2IVIT0
Ph>306EUI@2<S8`0OGem04E5A@2SXj<0>c/k09VIV@1OGel0HV9R04e=C@1aLG40UiNG065QH@2@T900
M7Ad02XZ:P1TI6@0RXZ:02XZ:P0j>SX0?clo06e]K@1@D50092@T0861P@1PH600<S8b04I6AP0d=3@0
>3Ph04E5A@0f=SH09bLW0410@00j>SX0BDU904];B`0k>c/0FUYJ03Ti>@1FEUH0De=C02TY:@1aLG40
QhN702@T900^;Rh0HF5Q03`l?017AdL0IfMW02XZ:P13@d<0I6AT03Ph>00Q8B40JFUY03dm?@0N7Qh0
Hf=S059BDP1:BTX0Cdm?03Hf=P1OGel0=cLg08:2PP1]KFd08B4Q059BDP1XJ6P0EUIF030`<02@T900
>SXj03<c<`0Y:BT0Rh^;05QHF00G5aL0@410038b<P1[Jf/0TY:B06][J`1CDe<0BDU903@d=01mOGd0
H61P05EEE@1NGUh0DU9B069RHP19BDT0P820071`L02JVYX0IfMW07=cL`1=CDd0<S8b07YjNP1iNGT0
DU9B09BDU01LG5`0IVIV06QXJ01RHV80Jf][07alO01IFET0KFe]04i>CP1PH600EUIF08n?S`1BDU80
De=C07alO02ZZZX0/;2`07UiN@13@d<0UIFE051@D02XZ:P0F5QH04m?C`27QhL0LW9b05iNGP1^KVh0
QhN704e=C@1fMWH0MGEe06QXJ01BDU80?3`l04M7A`13@d<0=3@d06m_K`2VYZH0JFUY05mOG`1_Kfl0
CTi>09ZJVP1XJ6P09bLW075aL@1^KVh0MWIf03Ph>02m_Kd0P82007=cL`2WYjL0XZ:R0451@@25QHD0
LW9b055AD@2HV9P0=CDe01lO7`2HV9P0Lg=c05iNGP1BDU80=SHf07MgM`1BDU80RHV908B4Q01HF5P0
Ph>3055AD@0[:b/0CDe=028R8P0U9BD0M7Ad03Xj>P0m?Cd0<30`04U9B@1ADE406ATI04U9B@1mOGd0
E5AD0:BTY01ZJVX0Shn?02HV9P13@d<0Hf=S04A4A02OWil0KVi^02`/;01KFe/0Q8B403<c<`1DE5@0
WYjN03Ti>@0V9RH0EeMG02d];@1KFe/0O7al06EUI@0^;Rh0:RXZ04e=C@0S8b<0CTi>05mOG`0_;bl0
IFEU0410@01[Jf/0WYjN065QH@0i>CT0SHf=07QhN00/;2`0ADE508B4Q01BDU806a/K05ADE01eMGD0
<c<c0492@P2/[:`0DE5A05IFEP31`L40RXZ:000G02@T901>CTh0FUYJ04E5A@27QhL0E5AD028R8P2M
WId0>CTi03Lg=`20P800BDU903Ph>01[Jf/0F5QH02/[:`16ATH0I6AT05iNGP0Y:BT0Cdm?051@D00b
<S803_ooool0P02KVi/0Hf=S04m?C`1>CTh0JFUY03<c<`0T92@07alO01hN7P0l?3`0HV9R04U9B@1m
OGd0>CTi02PX:00_;bl0@d=307alO01UIFD09BDU030`<01VIVH0/k>c0:6QX@0M7Ad06ATI014A4@0>
3Ph04A4A01@D501KFe/0R8R805]KF`1/K6`0LG5a09nOW`1KFe/0<c<c059BDP0K6a/03Ph>00d=3@0>
3Ph03@d=01<C4`18B4P0A4A405YJFP1BDU80BTY:09RHV01HF5P0A4A40;Fe]@3=cLd0fm_K0;jn_P1I
FET0C4a<05iNGP2LW9`0Vi^K0861P@1nOWh0K6a/05eMG@18B4P0D51@07emO@1`L700:b/[01LG5`0C
4a<03Ph>02DU9@1RHV80:RXZ05ADE02OWil0dm?C09fMW@1>CTh0Jf][0:ZZZP0o?cl0;Rh^02l_;`0d
=3@0?3`l079bLP39bLT0e=CD06e]K@3Bd]80g]kN0=7Ad@3<c<`0Z:RX09jNWP1XJ6P0Jf][05QHF01F
EUH0DE5A03`l?00e=CD0;bl_02DU9@0M7Ad07QhN04I6AP31`L40clo?0492@P2RXZ80LG5a04];B`1B
DU80OGem0:2PX02`/;00R8R8071`L01[Jf/0/;2`0:b/[01[Jf/0>CTi0P0G5aL02@18B4P0Rh^;02TY
:@0B4Q804a<C01@D501RHV80TI6A01TI6@0301DE5@0:01HF5P1>CTh0bl_;0<c<c02:RXX0F5QH07If
MP1<C4`0UIFE04U9B@804a<C03H092@T05EEE@1>CTh0@d=303<c<`0K6a/0?3`l08B4Q02=SHd0UYJF
09BDU01]KFd0DU9B03/k>`1RHV80Fe]K05QHF00K6a/0410@018B4P0C4a<0;Rh^07YjNP38b<P0GEeM
0:b/[01NGUh05ADE01dM7@17AdL0?Shn03lo?`0P82005aLG01lO7`10@400G5aL04Y:BP0e=CD0;Rh^
02PX:010@400Fe]K079bLP0b<S8071`L03Ph>01bLW80ADE503<c<`1XJ6P08R8R01TI6@0J6QX201PH
600V01TI6@11@D40>SXj06QXJ01KFe/0CTi>02HV9P15ADD0HV9R05]KF`2OWil0JVYZ09bLW02a/K40
EeMG030`<01[Jf/0@T9203Xj>P0l?3`0CTi>059BDP1@D500@41003Ti>@0n?Sh0F5QH09:BTP26QXH0
QHF5065QH@0k>c/0:b/[034a<@0/;2`09BDU04I6AP2;Rh/206i^KP0306][J`18B4P061PH00806ATI
00<07AdM03Hf=P0M7Ad00P0J6QX04P0L71`0=cLg01lO7`0L71`0820P03/k>`27QhL0ZJVY0820P029
RHT0Z:RX04];B`10@400;Bd]02@T900U9BD0FEUI04Q8B13ooooo000>ooooo`3o0;Zj^P2h^;P0DE5A
08f=S@24Q8@0[Jf]05QHF02IVIT0QXJ606][J`25QHD0Ng]k03Xj>P2MWId0ATI60:FUY@1JFUX0JFUY
05=CD`1`L700VYZJ061PH027QhL0P82004M7A`1WIfL0Ph>3028R8P10@400EUIF06ATI01:BTX09BDU
08b<S01YJFT0<30`04m?C`0_;bl0@41005QHF00i>CT09RHV0492@P10@400EUIF03Xj>P0V9RH0CTi>
04I6AP1/K6`0E5AD02d];@1lO7`0Rh^;04A4A016ATH0JVYZ03lo?`1KFe/0R8R80410@01RHV80NGUi
03hn?P0d=3@0Hf=S05YJFP0g=cL0RHV905mOG`1CDe<0Hf=S04E5A@1nOWh0?clo09nOW`1nOWh0<C4a
06m_K`2BTY80DE5A03dm?@2PX:00?3`l0410@00d=3@0NWYj038b<P0e=CD0EeMG03@d=01;Bd/0Y:BT
07=cL`0j>SX0IVIV038b<P2:RXX0?Cdm06a/K01KFe/0G5aL05mOG`1LG5`0Q8B406m_K`2QXJ40Kfm_
04Y:BP1QHF40>c/k08>3P`1>CTh0F5QH08V9R@1NGUh0NGUi06UYJ@0W9bL0H61P07IfMP1FEUH0L71`
06i^KP1BDU80Bd];0820P00W9bL0EUIF079bLP38b<P0][Jf03hn?P18B4P0WYjN04e=C@2GUiL0;Bd]
04e=C@24Q8@0P82005YJFP24Q8@0J6QX03Lg=`2BTY80/K6a06UYJ@1>CTh0:RXZ06EUI@1=CDd0=cLg
08b<S024Q8@0EEEE08f=S@1^KVh0IfMW0:FUY@1BDU80BTY:08j>SP1QHF40HV9R02l_;`2m_Kd0De=C
04M7A`1aLG40TI6A03Lg=`26QXH0RXZ:03/k>`1nOWh0AdM703De=@2g]kL0@D5102l_;`19BDT0?Shn
06YZJP1<C4`0YJFU0;:b/P1WIfL0SXj>03lo?`0f=SH0JFUY04I6AP0Z:RX0PX:2038b<P1SHf<0I6AT
08j>SP1SHf<0=3@d05=CD`1UIFD09bLW079bLP25QHD0LW9b0410@01EEED0@D5102LW9`1/K6`0LG5a
03lo?`2HV9P0Q8B4034a<@0l?3`0QXJ602XZ:P0f=SH0JFUY04A4A016ATH0FEUI04U9B@0l?3`0CDe=
06i^KP0Y:BT0:RXZ04m?C`1;Bd/0MgMg04i>CP1DE5@0YZJV05YJFP15ADD0[Jf]0;6a/@16ATH0@d=3
08N7Q`0d=3@08R8R07emO@1]KFd0;2`/055AD@2YZJT0O7al075aL@38b<P0SXj>000G02h^;P1ADE40
G5aL03Hf=P1JFUX0F5QH02/[:`2ATI40EEEE02l_;`1UIFD0DU9B03Ti>@1^KVh0E5AD02DU9@17AdL0
JVYZ07AdM012@T80GemO06IVIP14A4@03_ooool07@17AdL0IfMW075aL@1@D500Fe]K059BDP0Y:BT0
8B4Q01lO7`0X:2P0_[jn09FEU@2/[:`0VIVI07YjNP2;Rh/0<S8b06UYJ@2JVYX0LG5a08b<S02b/[80
bLW90;:b/P2<S8`0H61P03@d=00P820061PH00805ADE03<0EEEE0::RXP2GUiL0/[:b0=?Cd`2FUYH0
9RHV02LW9`1MGEd0;bl_00l?3`0@41004A4A00l?3`0G5aL05ADE01HF5P0K6a/0De=C03Xj>P0A4A40
410@03Ti>@38b<P0^KVi07IfMP1UIFD0T92@0;Rh^01mOGd05aLG01DE5@0[:b/0<C4a06ATI01[Jf/0
NGUi0:^[Z`0l?3`06a/K01TI6@0K6a/05aLG01<C4`1_Kfl0Ng]k06=SH`2PX:00IFEU0;jn_P2ZZZX0
0P2g]kL02P1nOWh0Ph>308b<S02ATI40U9BD07]kN`1QHF40MWIf0=;BdP34a<@204Q8B0050;Fe]@2Z
ZZX0`/;203Lg=`0S8b<0100E5AD0=P0D51@05ADE01<C4`0M7Ad0?clo06UYJ@32`/80alO706ATI01k
Ng/0KFe]05IFEP0l?3`0=cLg03Ti>@1>CTh0MGEe0<c<c020P800M7Ad09NGU`0m?Cd0=SHf01`L700E
5AD0;Rh^05aLG01SHf<05QHF01DE5@0R8R80[Zj^02PX:00D51@04a<C01DE5@0D51@04a<C02/[:`1^
KVh0g]kN0:VYZ@2UYJD0VYZJ0;2`/01jNWX0MgMg06a/K01cLg<0M7Ad06][J`0b<S805ADE01<C4`80
4A4A05/071`L071`L02SXj<0^k^k0;2`/01kNg/0AdM707moO`1iNGT061PH010@400>3Ph05ADE01LG
5`0k>c/0LG5a08j>SP2TY:@0B4Q808B4Q00V9RH061PH03`l?015ADD0Bd];03dm?@0P8200<S8b05EE
E@10@400@T9204a<C00a<C40>c/k03@d=00M7Ad0[:b/03<c<`0G5aL092@T04];B`2LW9`0EeMG09VI
V@1OGel0ADE50410@01ADE40J6QX05mOG`2m_Kd0QhN70;>c/`2TY:@0Kfm_05QHF01bLW80J6QX06i^
KP2NWYh0UIFE07moO`2^[Zh0Z:RX08b<S01KFe/07QhN02LW9`0_;bl07QhN01`L700^;Rh0@T920861
P@2TY:@0_kno09nOW`2BTY80Fe]K03De=@0L71`092@T024Q8@0U9BD0:RXZ065QH@2i^KT0MWIf075a
L@1PH600?clo00806ATI01P06a/K01dM7@0g=cL07QhN01TI6@0J6QX07QhN06a/K01hN7P0JFUY04e=
C@21PH40WIfM07AdM01JFUX0UYJF0=_Kf`2LW9`0GUiN03Ti>@0[:b/0=CDe05EEE@19BDT@ooooo`00
3_ooool0:02c/k<0SHf=05QHF02ATI40QhN708J6QP1QHF40_[jn0<71`@1UIFD0NWYj06a/K01:BTX0
YjNW04Q8B02DU9@0F5QH071`L019BDT0Jf][09JFUP1MGEd0S8b<0820P00n?Sh0L71`03Xj>P0`<300
Fe]K04m?C`1`L7009RHV030`<02=SHd0EeMG03Lg=`16ATH0@D5104=3@`1JFUX202TY:@3/069RHP0k
>c/0HF5Q0451@@0I6AT0GEeM04i>CP1XJ6P0?3`l01TI6@1cLg<0MgMg03Ph>01;Bd/0B4Q802l_;`1B
DU80LG5a034a<@1WIfL0LG5a01lO7`0^;Rh0H61P0492@P0i>CT0XJ6Q04Q8B01?Cdl0EeMG04I6AP1k
Ng/0820P06QXJ01cLg<0<S8b06a/K02BTY80?Cdm034a<@2@T900=CDe04e=C@1;Bd/0Jf][02h^;P0A
4A40<c<c03@d=00`<300NGUi071`L00a<C40A4A402l_;`2>SXh0ADE505ADE00m?Cd0FEUI065QH@0e
=CD0F5QH04];B`2GUiL0LG5a051@D013@d<0>CTi092@T01BDU80D51@079bLP1=CDd0NWYj065QH@0m
?Cd0DE5A05aLG01<C4`0LW9b03lo?`0a<C40G5aL079bLP0^;Rh0A4A405EEE@2d];@0`/;207=cL`15
ADD0T92@04=3@`2=SHd0@d=3055AD@1WIfL0OGem04];B`2ATI40LW9b03hn?P1LG5`0IVIV05]KF`1J
FUX0;Rh^04U9B@0g=cL0?3`l09VIV@1ZJVX0De=C05ADE01TI6@0HV9R0:RXZ01KFe/0:2PX071`L01X
J6P0MWIf03dm?@2m_Kd0SHf=061PH01UIFD0Kfm_03Xj>P2GUiL0WIfM06a/K02?Shl0?3`l02LW9`2/
[:`0N7Qh051@D00e=CD0B4Q808j>SP1;Bd/0NGUi07QhN00k>c/0UIFE03/k>`0_;bl0GEeM02TY:@0d
=3@0UiNG03@d=01>CTh0EEEE05ADE01FEUH0;Rh^07EeM@1nOWh0EeMG09:BTP1@D500NWYj034a<@1Z
JVX0D51@03/k>`1lO7`0AdM702l_;`24Q8@0QXJ603Ti>@0o?cl0UIFE03De=@0V9RH0BTY:04M7A`1:
BTX0E5AD05mOG`1=CDd0=CDe06IVIP0^;Rh0;bl_05UIF@0f=SH0FEUI05eMG@1RHV80/;2`07IfMP0o
?cl0P82009JFUP17AdL0Bd];08B4Q013@d<06a/K07MgM`1iNGT0@41005IFEP2FUYH0MWIf07]kN`36
a/H0T92@030`<01QHF40KVi^03lo?`1IFET0ATI6020P801/K6`0B4Q803<c<`1`L700F5QH03Xj>P1T
I6@0GemO01hN7P0n?Sh0KFe]07MgM`12@T80D51@07UiN@1IFET>ooooo`80=CDe03/0?3`l05IFEP1L
G5`0Hf=S05]KF`10@400<c<c07MgM`3@d=00dM7A0;bl_034a<@0][Jf0:f][@1hN7P0OWin0<G5a@2l
_;`0PX:2051@D01eMGD0SXj>04I6AP18B4P0F5QH06=SH`1QHF40DU9B03Hf=P0Q8B40HF5Q0:j^[P3<
c<`0bl_;0<G5a@2<S8`0>CTi02/[:`1?Cdl0:b/[010@400C4a<06a/K01HF5P0G5aL07AdM02l_;`1L
G5`02@T900X:2P0<30`0=3@d0:>SX`1CDe<0G5aL02<S8`1GEeL0OWin05ADE00201DE5@0f01<C4`0E
5AD0410@01HF5P0S8b<0De=C03hn?P0C4a<06QXJ01DE5@0G5aL0:b/[08F5Q@1JFUX0Lg=c0;Ng]`2;
Rh/0_;bl08>3P`1`L700P82005]KF`1EEED0G5aL05ADE01TI6@0YZJV09RHV02FUYH0L71`0:>SX`0[
:b/0>SXj0=_Kf`2YZJT0WYjN03`l?00^;Rh08b<S02HV9P0/;2`0>CTi03/k>`0m?Cd0@T9205QHF02o
_kl0dM7A0>;RhP2f][H0`/;209RHV00[:b/06a/K0P0L71`09`0M7Ad0820P01lO7`1oOgl0dM7A06AT
I01YJFT0EeMG0492@P0S8b<05QHF01TI6@0m?Cd0_[jn04A4A00M7Ad0J6QX05ADE00D51@04a<C010@
400A4A404a<C01PH602;Rh/0KFe]09jNWP0k>c/0IVIV09nOW`0o?cl09RHV0820P03De=@0]KFe09FE
U@1PH6007alO018B4P0201DE5@0:01@D500A4A40GEeM075aL@2XZ:P0Lg=c08F5Q@2EUID0WYjN04i>
CP804a<C03`05ADE01/K6`0c<c<0>SXj05eMG@0`<300D51@03lo?`19BDT0MWIf01PH600];Bd0I6AT
04m?C`1LG5`0>CTi059BDP0Z:RX0=3@d03`l?00d=3@0;Rh^03Lg=`0R8R80<S8b0<K6aP1fMWH05aLG
01lO7`1<C4`0NWYj0<O7a`1dM7@0=cLg06e]K@2=SHd0QHF507UiN@1TI6@0QhN70=;BdP3Cdm<0`l?3
0<71`@1iNGT0QXJ606IVIP1FEUH0Ti>C08n?S`1nOWh0Z:RX09nOW`2[Zj/0MgMg01hN7P0R8R80:RXZ
03`l?00X:2P201`L700/0410@02g]kL0_kno0<30`028R8P0:b/[01LG5`0I6AT07alO020P800U9BD0
AdM70:j^[P1fMWH0FEUI069RHP1aLG40<C4a02HV9P0G5aL06a/K01dM7@0c<c<0<C4a01lO7`0_;bl0
J6QX0;Fe]@2CTi<0OWin09>CT`33`l<0cLg=075aL@1OGel0d=3@0>OWi`3MgMd0ZZZZ04M7A`0n?Sh0
GEeM04E5A@1CDe<@ooooo`003_ooool0702n_[h0UIFE04e=C@1oOgl0UiNG0820P012@T80IfMW06AT
I01IFET0MgMg02`/;01<C4`0R8R804A4A020P800>3Ph079bLP0o?cl0T92@05YJFP0O7al0N7Qh04i>
CP1DE5@0MGEe04m?C`0L71`203dm?@0>07QhN00b<S807alO075aL@0d=3@0:BTY04=3@`0W9bL07alO
055AD@0M7Ad05ADE03Xj>P0`<30205IFEP1802PX:01;Bd/0<C4a06a/K01BDU807AdM07AdM01=CDd0
9RHV055AD@1FEUH0:RXZ04];B`1YJFT08b<S05QHF01=CDd0;Rh^02`/;01JFUX0@d=302XZ:P1YJFT0
Bd];04U9B@11@D40=cLg05eMG@11@D40WinO06ATI012@T80T92@09fMW@1EEED0=CDe08n?S`0^;Rh0
Cdm?04a<C01YJFT0De=C018B4P0k>c/0Cdm?02TY:@1XJ6P0G5aL03`l?019BDT0<C4a08Z:RP1FEUH0
GemO05eMG@1NGUh0HF5Q03`l?024Q8@0JVYZ09BDU01OGel0Hf=S04];B`0i>CT0RHV906IVIP1ZJVX0
QXJ605iNGP1WIfL0GEeM0P1>CTh0201iNGT0GEeM07=cL`15ADD0G5aL05UIF@1gMgL0B4Q80P1ZJVX0
U02d];@0Zj^[06i^KP14A4@0T92@04a<C02GUiL0@D5105]KF`1WIfL0O7al05QHF01]KFd0O7al04m?
C`1cLg<0P820030`<01YJFT0Bd];05]KF`18B4P0ATI60;fm_@26QXH0Kfm_05YJFP1VIVH0KFe]0:VY
Z@1cLg<0<C4a07alO011@D40?clo02h^;P2WYjL0Ph>305]KF`1TI6@0HV9R03De=@2:RXX0QhN7069R
HP2<S8`0;Rh^03@d=02VYZH0OWin04a<C00j>SX0>CTi07YjNP1NGUh0Rh^;08b<S01ZJVX0SXj>0451
@@12@T80ADE501`L700S8b<0IVIV03Xj>P1NGUh0FEUI061PH01JFUX08b<S059BDP1[Jf/0GUiN09NG
U`1<C4`0O7al04M7A`1MGEd0HV9R0492@P1cLg<0DU9B04a<C02>SXh0U9BD038b<P13@d<0RXZ:02XZ
:P0_;bl0>c/k04E5A@1;Bd/0Jf][07inOP1<C4`0=SHf06EUI@0i>CT0<30`04U9B@0j>SX0De=C04m?
C`1NGUh0YJFU08N7Q`1ADE40XZ:R09^KV`1:BTX0F5QH09BDU01UIFD07AdM08:2PP1iNGT0C4a<09RH
V031`L40PH61061PH02a/K40JVYZ03lo?`34a<@0XZ:R04=3@`1GEeL0ADE503Hf=P2ZZZX0EUIF03De
=@23Ph<0PH6105QHF026QXH0J6QX038b<P11@D40LG5a06UYJ@0n?Sh0U9BD0:2PX01WIfL>ooooo`0H
0451@@0h>3P0=CDe030`<01dM7@0SHf=05UIF@2UYJD0a/K609VIV@1XJ6P0Zj^[0;fm_@2QXJ40O7al
0:VYZ@22PX80`L710;Rh^02BTY80<30`01/K6`11@D40EeMG0P0H61P09@0I6AT07AdM024Q8@0c<c<0
HF5Q08R8R02o_kl0/k>c0:6QX@2EUID0LG5a0:^[Z`1oOgl0IfMW038b<P1RHV80AdM703<c<`0a<C40
=3@d03Xj>P12@T80Xj>S02<S8`0;2`/02@T900`<3016ATH0SHf=04e=C@1[Jf/0RHV90:f][@2XZ:P0
LG5a01dM7@0G5aL00P0>3Ph04`0?3`l05ADE01<C4`0n?Sh0De=C01LG5`0C4a<05ADE034a<@1CDe<0
@D5102HV9P0R8R80KVi^03lo?`2;Rh/0SXj>06m_K`0];Bd00`0L71`201/K6`0>02<S8`1]KFd0_;bl
09^KV`25QHD0Fe]K09bLW03MgMd0`l?30=CDe02][Jd0UiNG08n?S`1IFET205QHF00;04];B`0n?Sh0
<C4a02h^;P12@T80O7al0;6a/@3EeMD0gmoO0<?3``0l?3`00`0L71`04`0M7Ad09RHV059BDP2[Zj/0
`/;20;bl_02a/K40WinO07inOP0X:2P061PH01@D500_;bl0e=CD0<o?c`2YZJT0`/;201TI6@0@4100
0P0>3Ph0IP0E5AD08B4Q05IFEP1XJ6P0UYJF09NGU`0U9BD0EUIF08Z:RP0E5AD08B4Q06m_K`2a/K40
bLW90:>SX`1>CTh0:2PX01TI6@0B4Q804A4A00h>3P0E5AD0Q8B407inOP1lO7`0GEeM034a<@1UIFD0
VYZJ04I6AP0B4Q805ADE01PH600_;bl0E5AD028R8P0k>c/05aLG01TI6@1UIFD0;2`/071`L00e=CD0
8B4Q05=CD`1HF5P0N7Qh09ZJVP15ADD0;bl_030`<00d=3@0;Bd]03Ti>@0a<C40:b/[07IfMP3Lg=`0
cLg=06i^KP0c<c<0@D5107QhN03:b/X0/[:b0:n_[`1bLW80FEUI07IfMP24Q8@0R8R809FEU@3Lg=`0
^;Rh0;2`/03=cLd0XJ6Q08F5Q@1TI6@0L71`04Y:BP1ZJVX0HV9R08N7Q`2[Zj/0O7al0820P00Z:RX0
820P02<S8`0X:2P0?3`l03/k>`0g=cL0EeMG065QH@2>SXh0bLW90:6QX@1DE5@0<30`02l_;`807AdM
0280;bl_08V9R@1FEUH05aLG05MGE`19BDT0PH6104Q8B01=CDd06a/K01lO7`0c<c<0EeMG08F5Q@1C
De<0EUIF04M7A`1cLg<0E5AD069RHP2XZ:P0[jn_08N7Q`2CTi<0X:2P0<k>cP1lO7`0MGEe08B4Q02]
[Jd0XJ6Q09BDU01_Kfl0@d=34?ooool000kooooo0340]KFe09nOW`1IFET0QHF5096AT@29RHT0GEeM
0:j^[P2PX:00F5QH0861P@1TI6@0LG5a07moO`10@400XZ:R05iNGP1^KVh0=3@d08Z:RP2BTY80KFe]
07emO@1hN7P0ATI607alO01OGel0>3Ph06m_K`1UIFD0IVIV02XZ:P0i>CT0UiNG05iNGP0i>CT0@410
0492@P0i>CT0De=C02<S8`0[:b/0JVYZ055AD@1CDe<0A4A4030`<01[Jf/0C4a<0080GEeM08H08b<S
0820P01GEeL0=3@d04a<C01MGEd0<C4a06IVIP1]KFd0>c/k075aL@1NGUh0>SXj03<c<`1kNg/0G5aL
03Xj>P1UIFD0DU9B04e=C@1GEeL0CDe=07=cL`1>CTh0/[:b05iNGP0h>3P0GemO07]kN`1JFUX0=CDe
08j>SP0e=CD0DU9B05iNGP1LG5`0=CDe04Y:BP1UIFD0GemO02TY:@1fMWH0HF5Q04E5A@1BDU80=cLg
08R8R017AdL0Jf][04M7A`1OGel0Fe]K04i>CP1iNGT0IfMW0::RXP1TI6@0H61P0410@00o?cl0SXj>
06QXJ01oOgl0HV9R06=SH`1ZJVX0GemO0492@P1VIVH0OWin069RHP1gMgL0JVYZ04e=C@1HF5P0JVYZ
03Ti>@1lO7`0P8200;no_`2QXJ40E5AD05iNGP2WYjL0A4A407UiN@0h>3P0NWYj07alO01dM7@0FEUI
07emO@1SHf<0GUiN0:f][@2m_Kd0Fe]K05UIF@1ADE40NGUi03Xj>P0h>3P0XJ6Q06i^KP20P800SXj>
05EEE@1XJ6P0W9bL06QXJ00o?cl0Ti>C05]KF`1YJFT0?Cdm0;Bd]01eMGD0LG5a09RHV01SHf<0;bl_
08j>SP2PX:00F5QH0:n_[`1RHV80Lg=c08V9R@1VIVH0GUiN06i^KP10@400RHV905aLG080Z:RX00X0
RXZ:09NGU`1LG5`0NWYj06i^KP0m?Cd09bLW075aL@1DE5@0QhN70P1oOgl0C`1IFET0:RXZ059BDP1D
E5@0FUYJ08^;R`1/K6`0KFe]034a<@1bLW80CTi>04A4A02EUID0IVIV03Ti>@1lO7`0SXj>03`l?01/
K6`0Xj>S05YJFP0j>SX0@d=303/k>`22PX80Rh^;08:2PP0m?Cd0?clo051@D00/;2`0F5QH07UiN@1G
EeL0?clo04i>CP2:RXX0YZJV08B4Q01?Cdl0U9BD08b<S00i>CT0KFe]0:JVYP1QHF407alO08n?S`1L
G5`07QhN06e]K@2/[:`0L71`04=3@`1dM7@0J6QX02/[:`1/K6`0NWYj03Hf=P1BDU80CTi>03<c<`2@
T900H61P03@d=01dM7@0De=C03Ti>@21PH40MWIf02TY:@15ADD0NWYj06][J`0f=SH0SHf=07UiN@1F
EUH03_ooool06P2DU9@0EUIF03De=@13@d<0SXj>0;jn_P2VYZH0TI6A09JFUP1HF5P0De=C06a/K020
P800Zj^[07QhN01DE5@0C4a<09NGU`1nOWh0Q8B405aLG015ADD0MgMg03/k>`0F5QH07QhN0P0F5QH0
8`0E5AD04a<C01@D500a<C40XZ:R07alO01_Kfl0Z:RX05YJFP0l?3`0CDe=05mOG`1TI6@0QHF50;^k
^`1oOgl0QHF508n?S`25QHD0Ph>308^;R`0Z:RX02`/;00h>3P0=3@d0LW9b07YjNP1XJ6P0RHV908>3
P`2LW9`0`l?308n?S`1;Bd/06a/K00<03@d=01<03`l?01<C4`0L71`0KVi^03lo?`0h>3P0?clo03Xj
>P0];Bd0:2PX02XZ:P0g=cL0G5aL05YJFP2;Rh/0X:2P07alO013@d<0;Bd]00808B4Q0`0L71`0300P
8200GEeM08>3P`1DE5@0RHV904M7A`2j^[X0Z:RX0;Bd]01@D5008b<S018B4PH05ADE04D051@D01HF
5P0k>c/0GUiN0;^k^`3=cLd0YJFU08>3P`1_Kfl0Ng]k08>3P`2?Shl0RHV908n?S`1dM7@0UIFE0<c<
c02PX:00WinO09ZJVP2ZZZX0PH6106UYJ@2d];@0aLG50;Zj^P2RXZ80_kno096AT@1fMWH0GUiN04];
B`17AdL0?Shn02XZ:P0a<C40/k>c0;:b/P1]KFd0SXj>0:n_[`1DE5@0MGEe06ATI01>CTh0?Cdm02d]
;@1BDU80LW9b06i^KP1hN7P0M7Ad06EUI@1[Jf/0Z:RX0<W9b@1WIfL0Ng]k08:2PP21PH40W9bL05YJ
FP0O7al09BDU03dm?@1CDe<0<30`02XZ:P0M7Ad00P0D51@08P0Q8B40B4Q802h^;P1SHf<09bLW030`
<02c/k<0X:2P07QhN018B4P0=cLg05ADE01EEED0BTY:03<c<`0k>c/0MWIf08b<S01IFET0F5QH08:2
PP2PX:00P8200:NWY`3Cdm<0aLG507IfMP1EEED0CTi>075aL@1ZJVX0Hf=S04a<C017AdL20::RXP0o
0;Jf]P2h^;P0[Jf]07MgM`0S8b<06QXJ03/k>`2GUiL0Shn?0:RXZ02?Shl0SXj>09BDU00i>CT0;Bd]
02/[:`0V9RH0;Bd]05MGE`0W9bL0?Shn061PH01iNGT0Y:BT075aL@1JFUX0H61P07UiN@1_Kfl0Xj>S
06QXJ00H61P06a/K03Ti>@1fMWH0Ogmo096AT@2<S8`0SXj>04m?C`15ADD0@T9207EeM@0k>c/0<30`
03@d=01eMGD0RXZ:0;Rh^02RXZ80OWin09:BTP30`<00]KFe07alO02RXZ80K6a/05ADE01=CDd0KFe]
09ZJVP2NWYh0U9BD013ooooo000>ooooo`0M0;Jf]P1YJFT0GUiN07inOP2HV9P0FUYJ03Hf=P1GEeL0
R8R8051@D01FEUH0820P05IFEP22PX80?clo06EUI@1ADE40RXZ:0410@02DU9@0De=C02l_;`26QXH0
MGEe03hn?P25QHD0;Bd]01PH601>CTh00P1LG5`02`0X:2P08b<S07moO`1<C4`0?Shn03Hf=P0W9bL0
9BDU06QXJ00M7Ad06QXJ0080@T920>X0D51@03<c<`0Z:RX0KVi^0451@@1IFET0;bl_03De=@2ATI40
JFUY03<c<`1=CDd0=CDe05MGE`2:RXX0MGEe03hn?P1ZJVX0FEUI02d];@1LG5`0Ph>306EUI@0a<C40
GemO055AD@1dM7@0IfMW055AD@2>SXh08B4Q08b<S02=SHd0?Shn061PH025QHD0=SHf04Q8B02b/[80
>CTi04];B`1VIVH0D51@02/[:`0D51@0A4A405]KF`0[:b/0Q8B405UIF@0R8R80CDe=03hn?P20P800
?clo08V9R@17AdL0C4a<04=3@`1@D500IVIV05ADE021PH40EUIF02TY:@1IFET0=cLg08:2PP0M7Ad0
Hf=S06EUI@1KFe/0KVi^05]KF`0W9bL0Lg=c0861P@1NGUh0Fe]K05=CD`0n?Sh0E5AD06IVIP0E5AD0
BDU9065QH@2^[Zh0U9BD02TY:@0/;2`0L71`04U9B@1YJFT0=3@d04];B`1OGel0N7Qh05=CD`1kNg/0
@d=302XZ:P1DE5@0LG5a03hn?P1<C4`0DU9B08V9R@1IFET0C4a<0:2PX016ATH0@410079bLP1RHV80
LG5a0;>c/`0i>CT0:2PX05aLG01VIVH0EUIF059BDP34a<@0J6QX03lo?`1hN7P0?Cdm02<S8`2FUYH0
Vi^K04A4A020P800<C4a08>3P`2LW9`0EEEE02d];@1=CDd0>SXj06IVIP0T92@0JVYZ07inOP1<C4`0
QhN702@T901KFe/0NWYj04];B`0R8R80LG5a03Ph>023Ph<0R8R809BDU025QHD0BDU905IFEP1NGUh0
;2`/07EeM@2XZ:P0UYJF05iNGP1gMgL0;2`/03<c<`22PX80IVIV04A4A01eMGD0K6a/01LG5`1MGEd0
Ng]k03Lg=`0Z:RX0<S8b0492@P1HF5P0De=C06=SH`0Z:RX0@d=30492@P092@T0;bl_061PH015ADD0
;Rh^03hn?P1BDU80UIFE07IfMP1@D500QhN708f=S@11@D40ATI607alO00^;Rh06ATI08:2PP2:RXX0
CTi>079bLP2DU9@0<30`04a<C027QhL0IfMW03`l?01iNGT0HV9R038b<P1bLW80?3`l02/[:`1]KFd0
<C4a03Lg=`1eMGD0IVIV04Y:BP1VIVH0GEeM02<S8`15ADD0O7al05]KF`0i>CT0T92@0820P01FEUH>
ooooo`0;0:f][@2OWil0[Zj^0;^k^`1ZJVX0DE5A04e=C@17AdL0BDU9065QH@0K6a/00P0I6AT0101o
Ogl0];Bd0;:b/P1eMGD208Z:RP0/05iNGP2BTY80JFUY0;no_`1nOWh0CDe=04a<C015ADD0?Shn03<c
<`0X:2P07alO06YZJP1_Kfl0QXJ6051@D02f][H0YZJV07IfMP2CTi<0a<C40:FUY@2d];@0^k^k07If
MP0/;2`04Q8B00h>3P0A4A4051@D06=SH`1CDe<0?clo06a/K02j^[X0PH6108J6QP1ZJVX0HV9R07=c
L`3MgMd0[Zj^034a<@0@410200`<30803@d=0P0>3Ph08015ADD0`l?305MGE`14A4@0>SXj0410@00m
?Cd0A4A406=SH`1PH600?Cdm08:2PP3?cll09RHV01`L700];Bd0?clo04e=C@1?Cdl0@41003@d=00U
9BD0?clo09nOW`22PX80G5aL0:>SX`1/K6`0LG5a09fMW@0M7Ad051@D0P0A4A40<@0>3Ph04a<C01DE
5@0G5aL05QHF02HV9P1<C4`0@d=307emO@2GUiL0Lg=c0<O7a`2GUiL0e=CD0:f][@1nOWh0LW9b04a<
C00_;bl0>c/k03<c<`2?Shl0_[jn06e]K@0k>c/0CTi>08B4Q03QhN40jNWY0:RXZ02n_[h0cLg=065Q
H@1JFUX0GemO09BDU035aLD0e=CD0:^[Z`2ATI40]KFe0=cLg03Kfm/0h>3P0=oOg`3UiND0dm?C05]K
F`0_;bl00P0@4100;00>3Ph04Q8B00h>3P0I6AT0SXj>0<W9b@1mOGd06QXJ01PH601lO7`0alO708Z:
RP2<S8`0c<c<0=CDe02_[jl0WinO069RHP1IFET05QHF01DE5@0];Bd05ADE010@400A4A404Q8B01LG
5`1<C4`0@d=309jNWP2CTi<0Xj>S0:6QX@2=SHd0X:2P09NGU`2XZ:P0JVYZ07EeM@1YJFT0TI6A06YZ
JP0H61P05aLG0P0L71`09`0m?Cd0LG5a05]KF`3Lg=`0i>CT0=KFeP2m_Kd0_[jn03Xj>P0g=cL0DE5A
071`L02BTY80alO707QhN02k^k/0clo?0<K6aP26QXH06ATI01DE5@0o?cl0@d=308:2PP1BDU80CDe=
08Z:RP2m_Kd0GUiN04M7A`15ADD0A4A404e=C@1ZJVX0Bd];07AdM025QHD0UYJF04e=C@0201LG5`0U
06ATI02SXj<0@d=303De=@1GEeL0=cLg01`L700Y:BT0Lg=c0;Vi^@2RXZ80];Bd0:ZZZP1[Jf/0EUIF
065QH@2OWil0[Jf]08J6QP20P800I6AT09bLW02DU9@0[:b/0;jn_P3GemL0YJFU0:^[Z`1XJ6P08b<S
02h^;P13@d<0LG5a0:j^[P1^KVh0@D5103Ph>00@ooooo`003_ooool01032`/80Vi^K051@D021PH42
09jNWP3G05UIF@28R8P0[:b/05=CD`1dM7@0F5QH06ATI01fMWH0>CTi08f=S@17AdL0HV9R02h^;P2N
WYh0Jf][02l_;`1QHF40NWYj04e=C@2NWYh0LW9b034a<@1GEeL0GemO07IfMP0/;2`0<c<c09NGU`0i
>CT0@T9204a<C00/;2`05QHF05EEE@0Y:BT06a/K05EEE@0b<S80IFEU04];B`0];Bd0JVYZ0492@P1^
KVh0A4A401TI6@1PH600CTi>02PX:01FEUH0AdM702/[:`1cLg<0DE5A03<c<`1fMWH0@D5101dM7@0O
7al0DE5A04Y:BP0Z:RX0FEUI030`<00g=cL0D51@03Hf=P1IFET0;Bd]06YZJP1JFUX0?3`l061PH01d
M7@08R8R03De=@27QhL0;bl_05MGE`21PH40I6AT04Y:BP0N7Qh0DE5A03Lg=`0_;bl0TY:B06=SH`0c
<c<0D51@059BDP2>SXh0?clo06=SH`1@D500H61P051@D00k>c/0HV9R059BDP2<S8`0PX:204e=C@0n
?Sh0@41009>CT`1UIFD0MgMg05YJFP1HF5P0Ti>C06m_K`0k>c/0HF5Q04E5A@1MGEd0MWIf05QHF00^
;Rh0Hf=S07emO@17AdL0D51@04=3@`27QhL0UIFE055AD@12@T80U9BD03hn?P1bLW80@41005YJFP1]
KFd0B4Q803/k>`1iNGT0L71`06m_K`1hN7P0LW9b038b<P1/K6`0A4A407YjNP1BDU80Cdm?0;Rh^01n
OWh0IFEU06IVIP10@400NWYj0::RXP1VIVH0<C4a06a/K00b<S80:b/[04m?C`39bLT0Ti>C05eMG@24
Q8@0FUYJ02d];@2TY:@0UIFE06e]K@2XZ:P0>CTi05iNGP1@D500G5aL04e=C@1JFUX0;2`/04U9B@0f
=SH0QXJ607=cL`17AdL0Q8B403Ph>01DE5@0NWYj01lO7`0L71`0JFUY030`<01MGEd0Cdm?06m_K`1H
F5P0>CTi05=CD`1BDU80@T9204M7A`1bLW80M7Ad04];B`1QHF40CDe=03Ti>@1eMGD0DU9B03Xj>P1f
MWH0QXJ603Lg=`1hN7P0Q8B402PX:00b<S800P1ADE40=`1^KVh0JVYZ05iNGP0_;bl0C4a<04];B`0T
92@0>c/k03dm?@0/;2`0:b/[03dm?@21PH40/K6a07AdM01:BTX0S8b<08n?S`12@T80BTY:07QhN00/
;2`07AdM07inOP2>SXh0D51@0820P02i^KT0FUYJ01`L700[:b/0Hf=S03hn?P2JVYX0B4Q802`/;01_
Kfl0F5QH03Xj>P1eMGD09BDU06MWI`2TY:@0MgMg04I6AP1gMgL0@d=304A4A02=SHd0YjNW04e=C@0e
=CD0U9BD06UYJ@0T92@03_ooool0@P24Q8@0d=3@0=oOg`35aLD0VIVI08B4Q01dM7@0G5aL05mOG`1L
G5`0@T92020P800Q8B40KFe]05mOG`0o?cl0<30`05EEE@2IVIT0J6QX09NGU`35aLD0]kNg04m?C`0j
>SX0=SHf03Ph>00m?Cd0BDU905IFEP1`L700]KFe0;:b/P1NGUh0Jf][05UIF@0P82005QHF04M7A`1_
Kfl0LG5a06e]K@1QHF40DU9B030`<00a<C408b<S01<C4`0@410071`L08Z:RP2c/k<0TY:B0=GEe@2:
RXX0AdM704A4A00k>c/0JFUY0=7Ad@2c/k<0RXZ:01XJ6P092@T01`L700`<30<03@d=0100B4Q80:FU
Y@2_[jl0VIVI08>3P`1cLg<0JFUY06EUI@1hN7P0P82009nOW`3=cLd0][Jf02@T900J6QX05QHF0P0G
5aL03`0I6AT08R8R03hn?P1KFe/0TI6A0:FUY@2[Zj/0>CTi09^KV`0g=cL0H61P09RHV00O7al04Q8B
010@4003014A4@0>01XJ6P12@T80DE5A04A4A00T92@0=cLg0:RXZ01aLG40PX:20;Bd]02l_;`0FEUI
05IFEP1fMWH208f=S@0Q05QHF011@D40=SHf06QXJ02`/;00TY:B02d];@0X:2P0DU9B0=oOg`3Vi^H0
f=SH0=?Cd`3<c<`0VYZJ09FEU@2BTY80P82007AdM02@T900d=3@0>GUi@2g]kL0gmoO0=kNgP3Lg=`0
YZJV0<W9b@3:b/X0;Rh^014A4@0>3Ph03`l?0080410@01404A4A01PH602BTY80LW9b01dM7@0C4a<0
51@D01LG5`1cLg<0^;Rh061PH039bLT0f][J0=OGe`3?cll0R8R801LG5`0201DE5@0E05iNGP0E5AD0
4Q8B014A4@0D51@04a<C02DU9@0d=3@0V9RH0:n_[`15ADD0V9RH0<K6aP39bLT0`l?30;Rh^02[Zj/0
`L710:j^[P1hN7P09BDU00805aLG04L06a/K01TI6@0Z:RX0<S8b03lo?`2OWil0FEUI096AT@2k^k/0
XZ:R01PH600E5AD06a/K04=3@`20P800Rh^;09FEU@3Hf=P0inOW0=WIf@2XZ:P0De=C03Xj>P15ADD0
@41001XJ6P1:BTX0E5AD04m?C`2;Rh/0RXZ:06MWI`1[Jf/0I6AT075aL@2BTY80R8R80:^[Z`3;bl/0
XJ6Q05mOG`0O7al07AdM05mOG`1YJFT04A4A01@D500G5aL0DE5A04i>CP0`<300GEeM0:BTY03EeMD0
b<S80:VYZ@2FUYH0XJ6Q0;Ng]`3De=@0fm_K0<S8b031`L40b<S80<?3``1ZJVX0`/;20=?Cd`2`/;00
MWIf09fMW@02028R8P0702`/;00c<c<0A4A408^;R`2KVi/0EEEE03/k>`0@ooooo`003_ooool0NP2_
[jl0Rh^;059BDP1^KVh0ZZZZ09>CT`1NGUh0T92@09jNWP1]KFd0[:b/06UYJ@1mOGd0EUIF04Y:BP2P
X:00E5AD051@D00c<c<0X:2P08b<S00g=cL0Kfm_04A4A01BDU80SXj>05mOG`0f=SH0BTY:04A4A01f
MWH09RHV03lo?`2KVi/0AdM703Xj>P11@D40<C4a02HV9P1JFUX0:RXZ02DU9@1NGUh0?Shn04A4A00o
?cl08R8R05YJFP0a<C40Cdm?05=CD`0P8200KVi^02h^;P0V9RH0GEeM04M7A`0g=cL0RXZ:03lo?`0X
:2P0OWin05IFEP0^;Rh0>SXj079bLP16ATH0<c<c08^;R`1FEUH0CDe=05QHF016ATH0IVIV051@D02L
W9`0C4a<0492@P1lO7`0P82005MGE`0e=CD0OGem02XZ:P0o?cl0NWYj05IFEP0b<S809RHV05]KF`1M
GEd08R8R09^KV`1BDU80:b/[04a<C01:BTX0PX:204];B`1YJFT09BDU05QHF01?Cdl0@D5104M7A`0m
?Cd0DU9B075aL@1RHV80AdM7034a<@1lO7`0IfMW07IfMP1LG5`0GemO05eMG@1UIFD0BDU907QhN01l
O7`0DE5A0P1RHV806014A4@0JFUY075aL@16ATH0FEUI07UiN@2NWYh0T92@03Ph>01IFET0VIVI04=3
@`1<C4`0A4A409nOW`2ATI40Hf=S04U9B@1OGel0G5aL07moO`2VYZH0U9BD03Ti>@80Cdm?0140PX:2
02d];@0`<300SHf=05]KF`25QHD0UiNG05QHF01gMgL0W9bL06=SH`15ADD0Rh^;04E5A@1=CDd0L71`
0=CDe00208J6QP1]0;Bd]01HF5P0;bl_0:>SX`1oOgl0FUYJ0;^k^`1XJ6P0LW9b05ADE01<C4`0GEeM
05ADE00/;2`0HF5Q03lo?`29RHT0NGUi02<S8`1`L700B4Q805iNGP2HV9P071`L02<S8`1fMWH0@d=3
07MgM`1ADE40Ng]k0410@00T92@0IfMW065QH@1BDU80DE5A096AT@2=SHd0;2`/06e]K@1GEeL0?Cdm
07=cL`10@4008B4Q04M7A`20P800ADE508>3P`1eMGD0@41002DU9@1CDe<0E5AD0:j^[P2=SHd0DE5A
02<S8`1<C4`0IVIV034a<@12@T80=cLg02d];@0d=3@0AdM708R8R029RHT0De=C07YjNP3<c<`0];Bd
04I6AP1;Bd/0MGEe014A4@0G5aL0Ng]k096AT@1QHF40ZJVY0;fm_@1/K6`08R8R0492@P10@400D51@
0<S8b024Q8@0:b/[04M7A`12@T80Cdm?0:BTY01;Bd/0=cLg07]kN`18B4P0M7Ad09RHV01IFET07alO
03Ph>01RHV80EEEE03hn?P2MWId0KVi^03@d=00>ooooo`0?04Q8B038b<P0_;bl0410@00Y:BT0:b/[
038b<P1:BTX0LW9b09ZJVP2QXJ40>SXj038b<P1`L700GemO0080:RXZ00X0=3@d04e=C@2@T900c<c<
0=[JfP3?cll0ATI602h^;P0W9bL08B4Q0P0I6AT01`0Q8B40;Bd]065QH@1jNWX0N7Qh0820P00Q8B40
0P0A4A406@0@410071`L02PX:00f=SH0E5AD05]KF`0W9bL0:BTY03Lg=`14A4@0;2`/01hN7P1ADE40
QXJ60:RXZ03Fe]H0SXj>03@d=00_;bl0:RXZ08:2PP24Q8@09bLW04A4A01>CTh00P0<30`020030`<0
3@d=00d=3@0?3`l0?Cdm02TY:@0N7Qh0820P0P0M7Ad03@0Q8B40>3Ph0:FUY@2WYjL0Z:RX0:^[Z`2a
/K40NWYj02d];@0M7Ad06QXJ01/K6`0G5aL00P0E5AD04P0G5aL0<S8b08j>SP2<S8`0Hf=S05aLG00f
=SH0I6AT096AT@0S8b<04A4A00h>3P0E5AD08b<S04I6AP1@D500<S8b02LW9`808b<S01d0PH6108>3
P`0d=3@0EUIF0<71`@2g]kL0>3Ph01LG5`0E5AD07QhN04I6AP1mOGd0LG5a05MGE`24Q8@0U9BD0=3@
d01MGEd08B4Q02@T903AdM40fm_K0>3Ph03;bl/0Kfm_04A4A01=CDd0B4Q804m?C`0204e=C@0I06m_
K`2^[Zh0]KFe0<_;b`1YJFT0Cdm?0<30`03Bd]80[:b/06i^KP0@41003@d=010@400>3Ph0410@01<C
4`16ATH0UiNG03@d=00E5AD0410@01@D500B4Q807AdM0;fm_@020;jn_P0L0=7Ad@3EeMD0`l?302PX
:00D51@05ADE02h^;P2QXJ405QHF01DE5@0D51@04a<C00h>3P0C4a<0=cLg07alO01HF5P0BDU905YJ
FP31`L40LW9b05UIF@2/[:`0fm_K0<S8b019BDT09RHV01HF5P806QXJ00/071`L01PH600];Bd0<S8b
04];B`1jNWX0@D5102DU9@1;Bd/0^k^k028R8P0201DE5@0;01dM7@1SHf<0@41006YZJP2BTY80i>CT
0=gMg@33`l<0/[:b07IfMP0g=cL00P0E5AD0=P0J6QX0ATI605EEE@23Ph<0GUiN02h^;P0V9RH092@T
02`/;01LG5`0?Shn08B4Q02g]kL0^[Zj05mOG`1JFUX0CDe=06UYJ@1SHf<05ADE01<C4`0F5QH09bLW
04];B`1UIFD0MgMg0:JVYP2ATI40UIFE0:6QX@1VIVH0HV9R04M7A`19BDT0FEUI05mOG`2EUID0cLg=
0=?Cd`2f][H0c/k>0;^k^`30`<00`L710;Bd]00];Bd0:b/[03Hf=P0j>SX0DU9B05iNGP0m?Cd0Lg=c
08B4Q13ooooo000>ooooo`0S0<G5a@2BTY80DU9B07=cL`2IVIT0O7al06EUI@1jNWX0GemO04a<C028
R8P0LW9b08N7Q`1NGUh0?Shn09NGU`1GEeL0L71`02`/;02DU9@0V9RH03@d=01_Kfl0MgMg04U9B@2E
UID0Hf=S05QHF025QHD0L71`04e=C@0U9BD0LG5a09nOW`1;Bd/00P0i>CT0]00];Bd06a/K05MGE`0Q
8B407QhN05YJFP11@D40BDU904A4A00Q8B40DU9B0410@01jNWX0HV9R020P801iNGT0FUYJ03De=@1B
DU80<C4a061PH02YZJT0QHF504M7A`1WIfL0F5QH02d];@0o?cl0N7Qh03Lg=`0/;2`0De=C05IFEP1>
CTh0;Bd]04a<C020P800DE5A0:6QX@1MGEd0De=C07UiN@1_Kfl0Bd];0492@P24Q8@0=CDe03lo?`23
Ph<0D51@03Ti>@1SHf<0P82006a/K00N7Qh0_Kfm055AD@19BDT0Hf=S05=CD`1mOGd0CTi>07alO00k
>c/0Hf=S04U9B@14A4@0EEEE04];B`1FEUH0Hf=S05IFEP1<C4`0?clo07emO@0^;Rh0K6a/06MWI`1J
FUX0EEEE05eMG@0];Bd0Jf][06YZJP1BDU80Jf][07alO00e=CD0J6QX05QHF00n?Sh0TI6A071`L01c
Lg<0P820028R8P1/K6`0TY:B03Hf=P0b<S80:b/[0410@01^KVh0Lg=c03hn?P1IFET0:RXZ07alO02X
Z:P0XZ:R03Xj>P16ATH0GEeM0:2PX01EEED0>SXj09fMW@1dM7@0K6a/075aL@1XJ6P0QhN70<G5a@1=
CDd0;bl_09>CT`1QHF40DU9B08b<S03Fe]H0?3`l05=CD`28R8P0F5QH02XZ:P2TY:@0VYZJ08n?S`33
`l<0JFUY08R8R01CDe<0@d=302`/;01TI6@092@T05EEE@0f=SH0/k>c0::RXP1aLG40OWin02`/;02B
TY80YZJV05IFEP0K6a/0GEeM055AD@35aLD0Z:RX0;:b/P1JFUX0;Rh^04Y:BP1LG5`0>CTi08>3P`2f
][H0T92@03hn?P1HF5P0<C4a03Ti>@2GUiL0EeMG02h^;P17AdL0K6a/02`/;080Xj>S03/0HF5Q02l_
;`1BDU80=CDe06MWI`1iNGT0IfMW04=3@`1>CTh0F5QH03Xj>P1_Kfl0IfMW02PX:00_;bl0@T9208R8
R01`L700=3@d03<c<`1=CDd0LW9b03Lg=`1EEED0Ng]k02TY:@0J6QX0Ng]k06QXJ00a<C40HF5Q0;Jf
]P1jNWX09BDU03Lg=`14A4@0;2`/07emO@1HF5P0;Bd]061PH00h>3P0:RXZ07]kN`0c<c<0GEeM092@
T00i>CT0>c/k07YjNP1ZJVX0:2PX04];B`1nOWh0Cdm?0451@@2SXj<0OWin04I6AP0>ooooo`0U04A4
A02NWYh0Ogmo03`l?011@D40ADE504=3@`0n?Sh0>c/k06i^KP2l_;`0XZ:R04Y:BP2FUYH0YjNW0:6Q
X@0_;bl0EeMG08V9R@31`L40_Kfm0=KFeP2l_;`0ZZZZ05eMG@1BDU80HF5Q05eMG@1EEED0D51@05EE
E@1[Jf/0MgMg09jNWP1TI6@071`L010@400200h>3P0700l?3`0>3Ph06ATI01XJ6P1IFET0BTY:03Hf
=P02038b<P0W06IVIP2IVIT0K6a/05QHF00o?cl0aLG50:ZZZP11@D40=3@d06m_K`2b/[809RHV00`<
300?3`l0FUYJ030`<00C4a<02PX:00d=3@0C4a<051@D05]KF`0n?Sh0@41005iNGP1DE5@0BDU9051@
D01BDU80PX:207EeM@0c<c<08B4Q03Xj>P2@T900M7Ad05MGE`0U9BD071`L00805ADE014051@D01DE
5@0G5aL0Lg=c08j>SP1fMWH0DE5A04A4A01mOGd0/K6a02DU9@0D51@0410@03Lg=`11@D40:RXZ024Q
8@0201hN7P0K024Q8@0Z:RX0VIVI0492@P0Y:BT0=SHf08R8R02:RXX0SHf=05QHF00O7al07QhN024Q
8@0M7Ad0G5aL09NGU`1FEUH0HF5Q0<G5a@34a<@0L71`03Ph>031`L40b<S80=WIf@21PH407AdM00L0
5ADE00/0Bd];0=WIf@2NWYh07AdM03`l?01NGUh0TY:B08>3P`2/[:`0<C4a00d=3@0300h>3P0X02DU
9@23Ph<0AdM702<S8`0D51@05ADE018B4P0I6AT0GUiN0:6QX@2@T900[Zj^0;Zj^P3>c/h0WinO01HF
5P0E5AD0<S8b08f=S@17AdL07QhN01DE5@0D51@05ADE014A4@0D51@0B4Q808B4Q01IFET0EEEE08f=
S@1mOGd0U9BD0;^k^`3Fe]H0YZJV09RHV00K6a/04a<C01HF5P806ATI0P0G5aL0;@0l?3`0:b/[07Ad
M01`L700=3@d01TI6@0K6a/0Jf][04Y:BP0E5AD05aLG01/K6`1FEUH08B4Q02TY:@0k>c/0Shn?03hn
?P1hN7P0_[jn08Z:RP0M7Ad05ADE01LG5`0L71`0=CDe04Y:BP2RXZ80MgMg05=CD`0d=3@0;Bd]02`/
;01VIVH0AdM709>CT`25QHD0XZ:R07MgM`1jNWX0WYjN09>CT`1gMgL05ADE01<C4`0201TI6@0:024Q
8@34a<@0f=SH0<71`@2UYJD0PH6103`l?00Q8B40:2PX01TI6@805ADE01<05aLG05aLG02ZZZX0d=3@
0=oOg`2g]kL0HV9R07=cL`2DU9@0_kno08Z:RP0l?3`0>CTi04e=C@2IVIT0ATI603Hf=P0g=cL0FEUI
013ooooo000>ooooo`2;0<K6aP2DU9@0DU9B08:2PP2/[:`0QXJ6075aL@2XZ:P0Y:BT05ADE01YJFT0
CTi>08b<S01aLG40DE5A09fMW@1YJFT0QhN703@d=02WYjL0UYJF030`<01YJFT0Lg=c04a<C01aLG40
:b/[03Xj>P1[Jf/0IFEU05ADE00I6AT0AdM709>CT`19BDT0=cLg03Ph>01DE5@0;Rh^06][J`0I6AT0
?clo07MgM`14A4@0C4a<04=3@`0[:b/0CTi>03Hf=P1QHF40ATI603`l?028R8P0KVi^03De=@1GEeL0
:2PX03De=@1YJFT0H61P0410@01gMgL0BDU905=CD`1`L700NGUi05ADE00h>3P0De=C055AD@18B4P0
Cdm?03dm?@1hN7P0DE5A09VIV@1QHF40A4A406ATI01VIVH092@T069RHP2;Rh/0>c/k0492@P2=SHd0
BTY:02<S8`0T92@0GUiN06m_K`0T92@0^KVi06EUI@0e=CD0DE5A04e=C@1_Kfl0GUiN096AT@0n?Sh0
I6AT05ADE01eMGD0FEUI04i>CP1HF5P0Cdm?03<c<`16ATH0=CDe07]kN`1LG5`0E5AD05iNGP1IFET0
De=C06IVIP0c<c<0IVIV06i^KP1?Cdl0Kfm_059BDP0];Bd0Jf][055AD@0/;2`0FUYJ06a/K01eMGD0
QhN7030`<01KFe/0TY:B034a<@15ADD0=3@d04U9B@0206YZJP2202l_;`1MGEd0IVIV07alO01RHV80
TY:B03Hf=P18B4P0A4A4092@T01=CDd0=cLg09:BTP29RHT0QHF508N7Q`1_Kfl0Ogmo0;Ng]`24Q8@0
9bLW08n?S`1PH600FEUI09bLW03<c<`0N7Qh06e]K@2QXJ40LG5a02`/;02HV9P0XZ:R0;:b/P2i^KT0
Hf=S07UiN@1GEeL0HV9R03hn?P1EEED0:RXZ05IFEP0_;bl0Ti>C08n?S`1KFe/0Ogmo03hn?P15ADD0
S8b<03dm?@0J6QX0De=C030`<01SHf<0Fe]K075aL@1OGel0=3@d055AD@1KFe/06a/K059BDP1lO7`0
Shn?05MGE`1RHV807AdM02LW9`1iNGT0Jf][03Ph>01;Bd/0FEUI01PH601fMWH0BDU902TY:@0b<S80
DU9B05aLG01QHF40De=C059BDP0j>SX0BTY:04I6AP0E5AD0F5QH071`L00c<c<0=cLg0410@02h^;P0
T92@069RHP1PH600U9BD0:b/[00n?Sh0P82008F5Q@0k>c/0=CDe0:FUY@2<S8`0EeMG07=cL`2k^k/0
OGem02DU9@18B4P0DU9B03dm?@27QhL0>3Ph03lo?`2NWYh0H61P03@d=018B4P07alO0410@01aLG40
De=C03hn?P1TI6@0;Rh^05YJFP80W9bL00D0@D5102PX:027QhL0H61P02/[:`0>ooooo`0J07UiN@2n
_[h0^k^k07EeM@0k>c/0?3`l02/[:`0X:2P0<C4a03Ph>01TI6@0c<c<0<[:bP2d];@0JFUY08J6QP2A
TI40R8R808F5Q@2j^[X0cLg=0=GEe@2GUiL0OGem01HF5P0C4a<201@D501<01LG5`1ADE40QXJ603@d
=00h>3P0DU9B05ADE00a<C403Ph>010@400>3Ph03`l?00h>3P0?3`l04Q8B02/[:`2RXZ80FUYJ04M7
A`1;Bd/0PX:209RHV00k>c/09RHV02<S8`2EUID0^k^k06e]K@13@d<0/[:b07]kN`0I6AT03@d=00h>
3P17AdL0eMGE0:n_[`26QXH0Ogmo07]kN`2OWil0W9bL09RHV01TI6@0FEUI055AD@17AdL0<c<c038b
<P11@D40DE5A01hN7P0E5AD09BDU07=cL`0G5aL0GemO08N7Q`0d=3@06a/K01DE5@0F5QH06ATI01PH
601KFe/0_;bl0<k>cP2GUiL0ZJVY0<g=c@3Lg=`0GEeM04e=C@1LG5`0<30`01hN7P806a/K02407alO
02DU9@0Z:RX0Hf=S06MWI`0P820092@T034a<@1WIfL0PX:2071`L02NWYh0UYJF04Y:BP0d=3@0<C4a
03lo?`1oOgl0/K6a09VIV@2n_[h0gmoO0=GEe@31`L40dM7A0;>c/`3AdM40=3@d01lO7`0H61P051@D
01DE5@0F5QH00`0E5AD02`0a<C40_kno05UIF@0M7Ad0DU9B02XZ:P1NGUh0[jn_08V9R@20P8004a<C
00803Ph>01H04Q8B07emO@1HF5P09RHV02TY:@0D51@04a<C02DU9@1]KFd0LG5a034a<@1IFET0WYjN
06YZJP28R8P0V9RH01XJ6P0Y:BT0Ti>C02PX:00Y:BT0?3`l0`0E5AD0HP0D51@05aLG03Hf=P3Cdm<0
/[:b0;jn_P2BTY80QHF507IfMP1FEUH0WYjN03Lg=`0b<S80EeMG01LG5`0F5QH05ADE01@D500E5AD0
:RXZ04A4A00X:2P0UiNG0492@P0I6AT05aLG01HF5P0S8b<0KVi^01`L700_;bl0N7Qh0:JVYP0Y:BT0
6ATI02l_;`10@4006ATI03De=@2RXZ80EUIF01@D500E5AD07AdM03Lg=`1FEUH0KVi^06m_K`1>CTh0
Ph>3059BDP10@400Hf=S0;Fe]@2RXZ80Fe]K055AD@1YJFT071`L02XZ:P1KFe/0YZJV0:>SX`0L71`0
5aLG01hN7P0o?cl0OGem09bLW036a/H0ZJVY0;>c/`36a/H0X:2P07IfMP0/;2`05aLG01HF5P0I6AT0
7alO06e]K@29RHT0d=3@0=gMg@2DU9@0?Cdm06i^KP1fMWH0TI6A09JFUP2l_;`0UIFE09bLW03MgMd0
Hf=S03Xj>P0n?Sh0Bd];4?ooool000kooooo06d0`<3007=cL`1/K6`0OGem0:f][@1aLG40IVIV0820
P02OWil0EeMG08V9R@1ZJVX0QHF506IVIP1PH600OWin06ATI01ZJVX0=3@d0:FUY@1OGel0=SHf07]k
N`1^KVh0Bd];09VIV@1<C4`0B4Q805EEE@1MGEd0IfMW024Q8@1?Cdl0UIFE03hn?P16ATH0CTi>03De
=@0P8200HF5Q028R8P0K6a/0DE5A0492@P1SHf<0=cLg028R8P1HF5P0@T9205IFEP17AdL08R8R079b
LP1IFET0<30`05eMG@18B4P0>c/k069RHP1<C4`0BTY:09:BTP1BDU80:RXZ051@D01UIFD0CDe=03`l
?01TI6@0>SXj04m?C`1<C4`0Cdm?06EUI@0_;bl0MGEe06i^KP1JFUX0R8R808>3P`10@400AdM7079b
LP0k>c/0CTi>08f=S@1SHf<0;Rh^03/k>`1QHF40?Shn03<c<`2OWil0MGEe034a<@1?Cdl0Hf=S07If
MP1MGEd0IFEU02l_;`1lO7`0@T9205QHF01QHF40@D51071`L01gMgL0Cdm?0080CTi>08h0VIVI065Q
H@1UIFD0HV9R061PH024Q8@0OGem04E5A@1jNWX0EUIF06YZJP1lO7`0IFEU03Xj>P1kNg/0MWIf051@
D01SHf<0GEeM09:BTP2KVi/0>c/k069RHP2GUiL0ADE506=SH`0f=SH0?Shn06MWI`1;Bd/0>3Ph06e]
K@1PH600MGEe04m?C`1NGUh0:BTY06YZJP1:BTX0S8b<05YJFP12@T80/K6a09:BTP29RHT0IFEU04Y:
BP25QHD0/[:b07moO`0f=SH0QHF506=SH`1ADE40`/;20;>c/`1lO7`0L71`09fMW@1@D500;2`/09bL
W02MWId0KFe]0:>SX`17AdL0UYJF08R8R01RHV80B4Q805IFEP0Y:BT0Ti>C05]KF`2>SXh0N7Qh03lo
?`23Ph<0Bd];03@d=02ATI40=3@d03Xj>P2>SXh0GemO092@T01?Cdl0N7Qh051@D00b<S80KFe]075a
L@1NGUh0MGEe06e]K@2BTY80<C4a07]kN`1QHF40;Rh^06][J`1DE5@0:2PX071`L027QhL0>CTi08:2
PP1/K6`0:BTY03Ph>01nOWh0FUYJ05QHF01SHf<0ADE501dM7@1KFe/0GEeM030`<01TI6@0DE5A02PX
:01PH600=cLg08>3P`1_Kfl0>CTi05MGE`24Q8@0YZJV03Lg=`1<C4`0JVYZ02`/;00J6QX0N7Qh09NG
U`1DE5@0MWIf0:VYZ@13@d<09RHV0P1^KVh05`0l?3`0OGem04I6AP0c<c<0JVYZ03@d=00n?Sh0GEeM
028R8P1LG5`0IFEU04];B`1HF5P0NWYj04I6AP0];Bd0BTY:07IfMP1:BTX0?Cdm0:NWY`1cLg<0C4a<
00kooooo01d0SXj>0:b/[02ZZZX0E5AD04];B`19BDT0@4100451@@10@400AdM706MWI`1lO7`0NWYj
0:RXZ039bLT0fMWI0861P@1?Cdl0>CTi02HV9P16ATH0BTY:07QhN028R8P0IVIV059BDP0U9BD05ADE
01LG5`0204E5A@0901hN7P0Q8B40@41005MGE`1EEED05ADE00l?3`0B4Q804A4A0080410@040051@D
024Q8@1nOWh0Ph>306m_K`2FUYH0/K6a09jNWP1RHV80?clo02`/;018B4P0a<C40<g=c@2OWil0IfMW
05IFEP1PH600DE5A05=CD`1bLW80TY:B09nOW`2GUiL0Rh^;0:6QX@3:b/X0SXj>065QH@1BDU80Bd];
04U9B@0k>c/09RHV01DE5@0O7al0<C4a03<c<`0/;2`061PH079bLP0M7Ad04A4A02@T901iNGT0VYZJ
07QhN01XJ6P0HV9R06i^KP2UYJD0_Kfm0::RXP2][Jd0aLG50=kNgP3Ti>@0b<S80;bl_02DU9@0GEeM
03hn?P1DE5@0GemO0P1[Jf/0AP1SHf<0RHV903lo?`0g=cL0=SHf04a<C01nOWh0PX:20451@@16ATH0
J6QX02DU9@0K6a/05aLG01@D500I6AT0EeMG0:2PX03GemL0_Kfm0:JVYP30`<00a/K609^KV`25QHD0
:2PX028R8P0M7Ad05ADE01HF5P0D51@05ADE01LG5`0E5AD08R8R08F5Q@0/;2`06a/K04A4A00I6AT0
CDe=08R8R02FUYH0Lg=c07moO`0a<C404A4A04m?C`1:BTX09BDU02@T900W9bL0<C4a03<c<`0b<S80
:2PX02`/;00f=SH0QXJ606UYJ@2BTY80VIVI0;Jf]P1`L700TY:B08:2PP1DE5@0C4a<096AT@0e=CD2
01DE5@1R01@D500E5AD0?3`l09:BTP1mOGd0:RXZ05=CD`1GEeL07alO04U9B@0j>SX05aLG01XJ6P0g
=cL0EUIF02@T900E5AD04a<C01/K6`19BDT0<c<c0451@@36a/H0;2`/01TI6@0G5aL05ADE01HF5P1`
L700ATI603@d=00X:2P0FUYJ034a<@0Q8B40<C4a02`/;00m?Cd0@d=309nOW`26QXH07AdM02`/;014
A4@0Kfm_06IVIP0[:b/05ADE038b<P29RHT0RXZ:07MgM`2PX:00SHf=07alO01OGel0J6QX07QhN00/
;2`0:2PX034a<@1FEUH0cLg=0;Rh^02VYZH0Shn?08f=S@1_Kfl0XJ6Q0:f][@2/[:`0O7al02PX:00K
6a/0:2PX020P800K6a/0820P034a<@1@D500PX:203hn?P12@T80YjNW0;jn_P1VIVH0HV9R08>3P`2F
UYH0De=C0:BTY02e]KD0GUiN079bLP2BTY80RHV908f=S@2RXZ8@ooooo`003_ooool0SP38b<P0Q8B4
059BDP1/K6`0]kNg08B4Q01`L700PH6106UYJ@1^KVh0X:2P069RHP1oOgl092@T07EeM@2LW9`0H61P
06=SH`0i>CT0YZJV08j>SP0];Bd0FEUI02<S8`18B4P0Shn?03Ti>@12@T80@41003hn?P1cLg<08B4Q
04Q8B02<S8`0@D5103<c<`12@T80:b/[01lO7`1MGEd0<C4a01lO7`1BDU807AdM05MGE`1NGUh0;Rh^
04];B`0o?cl0RXZ:06IVIP0T92@0MgMg04E5A@18B4P0W9bL05MGE`11@D40EeMG03Xj>P0^;Rh0JFUY
05UIF@0Y:BT0EEEE079bLP0>3Ph0=SHf08F5Q@1@D500AdM701DE5@0k>c/0JFUY04I6AP1_Kfl0G5aL
0410@01WIfL0JVYZ03dm?@1FEUH0IfMW0492@P0o?cl0S8b<04];B`0S8b<0Bd];05IFEP1<C4`0<C4a
09FEU@1@D500=3@d04Y:BP1[Jf/0PX:205iNGP1=CDd05aLG06m_K`19BDT0FUYJ03Lg=`0Z:RX0B4Q8
069RHP0o?cl0ADE50410@021PH40CDe=05MGE`13@d<0B4Q804a<C01>CTh0?Cdm0820P01IFET0ADE5
06a/K01VIVH06a/K06m_K`1hN7P0HV9R09NGU`1IFET0KFe]07=cL`0a<C40UIFE08^;R`0k>c/0?clo
03@d=01JFUX0GemO05YJFP0_;bl204];B`2607emO@1FEUH0H61P01hN7P1HF5P0?clo0820P00Z:RX0
CTi>092@T02;Rh/0RHV907inOP1/K6`0MGEe09jNWP1`L700;bl_08f=S@14A4@0=3@d0:FUY@2VYZH0
O7al08j>SP2WYjL0M7Ad03<c<`27QhL0Lg=c051@D02g]kL0@D5105EEE@1ZJVX0BTY:05=CD`1GEeL0
8R8R05eMG@1:BTX0ZJVY08f=S@1TI6@0N7Qh04a<C00];Bd0SHf=02@T900O7al0H61P05=CD`2RXZ80
E5AD08B4Q014A4@092@T04i>CP1cLg<0HF5Q07]kN`1dM7@0XZ:R05aLG01[Jf/0H61P038b<P1WIfL0
KVi^04Y:BP1RHV80T92@04=3@`29RHT0Kfm_05=CD`0[:b/0P82005EEE@1hN7P0SHf=08V9R@0d=3@0
@d=305iNGP10@400J6QX05UIF@0W9bL0JFUY03Hf=P2BTY80LG5a06=SH`1:BTX0GUiN0820P015ADD0
PH610820P00/;2`06ATI05MGE`28R8P0HF5Q0<G5a@2g]kL0S8b<01TI6@1jNWX0EeMG03Lg=`2DU9@0
Kfm_04];B`2ZZZX0@d=303De=@20P800?clo055AD@1YJFT0;Rh^04=3@`2ATI40Hf=S02l_;`17AdL0
QXJ603Hf=P0W9bL0P82007emO@1>CTh>ooooo`0A0:b/[02j^[X0[:b/05YJFP0e=CD0<C4a020P800N
7Qh06a/K01hN7P0Z:RX0>3Ph06ATI01FEUH0<c<c09^KV`0Y:BT00P0E5AD02`0@4100<C4a03Ph>00N
7Qh0K6a/07YjNP1HF5P0Ti>C0861P@1:BTX0Jf][0080?3`l00L0820P04Y:BP1^KVh0OGem03De=@0K
6a/06ATI008061PH00L05ADE01lO7`0n?Sh061PH014A4@0J6QX0HV9R0080<c<c00/06ATI02d];@1B
DU80WinO0=KFeP3IfMT0_Kfm0:6QX@22PX80K6a/03`l?00201hN7P0:02`/;01<C4`0T92@08>3P`1U
IFD0U9BD09RHV00d=3@0820P00l?3`<03Ph>07<0410@01@D500T92@0=SHf03/k>`12@T80<S8b08>3
P`0F5QH0410@05ADE02RXZ80NGUi0861P@2HV9P0`<300<g=c@3?cll0b<S80<o?c`3Lg=`0c/k>0=[J
fP3Lg=`0/K6a08^;R`33`l<0f][J08N7Q`0Y:BT07QhN01lO7`0Y:BT0Ng]k06MWI`1ADE40=cLg03@d
=011@D40Hf=S03De=@0O7al0<30`03Ti>@0H61P05aLG01PH6016ATH0Bd];03/k>`1fMWH0=cLg06i^
KP1ZJVX0LG5a07EeM@32`/80JVYZ02XZ:P0Y:BT0820P01PH600E5AD05aLG01HF5P0H61P07alO065Q
H@0J6QX0;bl_03/k>`0H61P0<c<c09bLW02_[jl0Lg=c04=3@`2WYjL0OWin07=cL`10@400AdM703/k
>`0Q8B4071`L01/K6`0H61P05aLG02HV9P1iNGT0YZJV07moO`17AdL0FEUI06e]K@1ZJVX05QHF01dM
7@0_;bl0Bd];07AdM02[Zj/0CDe=01DE5@0C4a<08b<S05=CD`3:b/X0WinO05YJFP1gMgL06ATI01`L
701<C4`05aLG00805ADE00<05aLG02@T901CDe<00P0G5aL0301>CTh0GUiN06a/K02@T900g]kN030`
<00L71`06QXJ01PH600E5AD0CTi>01LG5`805ADE03X0820P06m_K`0W9bL0>SXj059BDP1nOWh0/;2`
0=kNgP3:b/X0EUIF04a<C02:RXX0Cdm?01DE5@0M7Ad0:2PX05mOG`1IFET0/;2`0<;2`P2VYZH0SXj>
061PH00e=CD0?3`l09nOW`2BTY80Xj>S0::RXP2JVYX0];Bd05EEE@1aLG40`L710;Fe]@31`L40c<c<
0=SHf031`L40HV9R02<S8`0J6QX09BDU04Y:BP0o?cl0=SHf02DU9@0L71`0DE5A04=3@`0_;bl0JVYZ
0;Jf]P2?Shl0][Jf08j>SP2MWId0B4Q80P1oOgl01P0R8R809bLW02/[:`0j>SX0QhN70;Zj^Q3ooooo
000>ooooo`0L0;Ng]`1aLG40GUiN07EeM@2DU9@0KFe]08F5Q@2UYJD0LW9b05MGE`2=SHd0K6a/08R8
R014A4@0IfMW092@T01GEeL0=CDe04M7A`2GUiL0MWIf02l_;`1RHV80@d=303dm?@2FUYH0<c<c04I6
AP80Fe]K0?P0M7Ad024Q8@15ADD0V9RH05YJFP15ADD0?Cdm02@T900h>3P0HF5Q02`/;00Q8B40Bd];
038b<P0n?Sh0?Cdm02TY:@1:BTX071`L04];B`1HF5P092@T07IfMP0D51@0:RXZ065QH@1;Bd/0BTY:
06ATI01OGel0Bd];08n?S`1LG5`0:2PX04I6AP1JFUX0?clo03/k>`1FEUH0CDe=04Y:BP13@d<0BTY:
06MWI`1:BTX0T92@05MGE`18B4P0LG5a07=cL`0o?cl0De=C069RHP15ADD0@d=309BDU00e=CD0;Bd]
05iNGP1bLW80GemO05EEE@2XZ:P0=CDe02`/;01WIfL0KVi^04i>CP1YJFT0O7al03/k>`1hN7P0?3`l
07UiN@1YJFT0@T920410@012@T809bLW065QH@0o?cl0MgMg02PX:01MGEd0FEUI04e=C@1IFET0Bd];
02`/;01`L700Fe]K04I6AP1?Cdl0Jf][034a<@1gMgL0EEEE03Xj>P1RHV80M7Ad0861P@1nOWh092@T
06UYJ@2GUiL0B4Q804U9B@0Y:BT0@T9207IfMP1_Kfl0<c<c05UIF@0[:b/0JVYZ09JFUP2@T900:b/[
05=CD`1PH600ZZZZ075aL@14A4@0QhN7065QH@2MWId0SXj>06m_K`1RHV80]kNg05mOG`0j>SX0Z:RX
07YjNP18B4P0QhN707MgM`0T92@0EUIF09BDU029RHT0<S8b08n?S`1iNGT0?Cdm0:NWY`1BDU80Hf=S
06UYJ@0L71`0CTi>08F5Q@0`<300D51@03`l?02h^;P0ZZZZ071`L026QXH09bLW03hn?P2:RXX0ADE5
02HV9P1VIVH0@d=30:j^[P2HV9P0Y:BT06ATI00c<c<0E5AD079bLP11@D40P8200:6QX@2OWil0EUIF
06MWI`1HF5P0?Shn071`L01eMGD0D51@061PH01nOWh0>CTi09bLW02:RXX0I6AT03Xj>P2>SXh09RHV
04I6AP1gMgL0M7Ad03Ph>01ADE40IfMW02TY:@1dM7@0NGUi02XZ:P1gMgL0;bl_0:2PX02IVIT0JFUY
05ADE01TI6@0IfMW02DU9@1GEeL0OWin04];B`0Q8B40G5aL07=cL`0^;Rh0JVYZ07MgM`0];Bd08R8R
07]kN`1LG5`0:2PX07EeM@0h>3P0;Bd]07alO01<C4`0@T9205YJFP0Y:BT0KVi^08j>SP1PH600G5aL
06e]K@1@D500=CDe05=CD`20P800C4a<03Ti>@23Ph<0Jf][03`l?0kooooo00@0CTi>08J6QP24Q8@0
;Bd]0P0M7Ad01@0J6QX06ATI01HF5P0H61P071`L0080EUIF01l0LG5a04M7A`13@d<0@D5101DE5@0C
4a<0410@02TY:@12@T80=3@d030`<03<c<`0WYjN05MGE`1dM7@0WinO08F5Q@18B4P0?Shn028R8P17
AdL0S8b<08:2PP1]KFd06a/K01DE5@0G5aL09BDU04A4A012@T804a<C00803@d=01T04a<C04i>CP0Y
:BT0;Rh^014A4@0D51@09bLW0;no_`3Bd]80e]KF0;Rh^023Ph<0EEEE05QHF01_Kfl0IFEU0492@P18
B4P0@410071`L01MGEd0E5AD0;:b/P1dM7@03`l?00803Ph>0`0=3@d200h>3P0:01@D500c<c<0=3@d
03dm?@0S8b<0^[Zj06MWI`1]KFd0JVYZ02PX:08071`L05`06ATI02@T901mOGd0L71`09fMW@3QhN40
fMWI0;Rh^02YZJT0gMgM05UIF@1QHF40Cdm?0:VYZ@2IVIT0;2`/02<S8`0f=SH0A4A407emO@0f=SH0
7AdM01XJ6P0H61P06ATI051@D00g=cL09BDU01`L700m?Cd04Q8B01HF5P0f=SH0:2PX02`/;01ADE40
6a/K01LG5`0g=cL0;Bd]06][J`2DU9@0b<S809jNWP0k>c/0<c<c01dM7@0N7Qh07AdM01TI6@0E5AD0
5aLG020P801eMGD06ATI05UIF@2:RXX0@41004Q8B02l_;`0[Jf]0;no_`29RHT0[:b/0=3@d02HV9P0
>CTi01/K6`0G5aL06QXJ01LG5`0F5QH0;Bd]05YJFP1lO7`0_Kfm0<30`02FUYH09bLW04=3@`1NGUh0
QhN701`L700F5QH06ATI01dM7@0R8R80N7Qh0;^k^`1IFET0<C4a06QXJ080QhN701H0]KFe07EeM@17
AdL06QXJ028R8P1>CTh05aLG01DE5@0F5QH05ADE01LG5`0c<c<0PH6108R8R03?cll0/[:b0;2`/03H
f=P0h^;R06][J`0X:2P092@T0P0E5AD00`1@D5006ATI01DE5@0201DE5@0D038b<P1LG5`0A4A405]K
F`3De=@0f=SH0=CDe03Shn<0SHf=08^;R`1gMgL06ATI01dM7@13@d<0C4a<01lO7`0E5AD0De=C09fM
W@2KVi/205]KF`0E03/k>`0n?Sh0HV9R01/K6`0Q8B40@T920:^[Z`2DU9@0BTY:096AT@1_Kfl0BDU9
05aLG02NWYh0bLW90::RXP1GEeL0Hf=S03De=@0h>3P0;2`/00806ATI01@05aLG01/K6`1BDU80Cdm?
03Xj>P1BDU80/K6a0410@00j>SX0X:2P0<;2`P1LG5`0UYJF092@T00o?cl0=cLg03`l?01hN7P0Ti>C
0=SHf13ooooo000>ooooo`0T0<;2`P1DE5@0HV9R06i^KP2MWId0AdM705aLG01_Kfl0PX:207MgM`29
RHT0BDU908n?S`1@D500Ti>C08:2PP1hN7P0?clo04I6AP20P800IFEU04i>CP26QXH0HV9R03/k>`1X
J6P0:BTY075aL@1]KFd0FEUI0451@@0[:b/0KFe]08f=S@1ADE40ATI60P0c<c<0E`0e=CD0@T9201dM
7@0h>3P0GUiN03Ph>01;Bd/0:b/[03<c<`1OGel0?Cdm04Q8B00_;bl0>CTi0861P@11@D40=3@d05YJ
FP15ADD0OGem096AT@1OGel0@d=309:BTP1:BTX09RHV04m?C`1DE5@0Bd];05UIF@1ZJVX0DE5A05UI
F@17AdL0E5AD05MGE`0j>SX0V9RH071`L01ADE40NWYj06i^KP0S8b<0Fe]K06e]K@16ATH0ADE5092@
T01DE5@09RHV04I6AP1ADE40:2PX06=SH`1cLg<0De=C038b<P1]KFd0N7Qh06QXJ01[Jf/0IVIV03`l
?021PH40@d=3069RHP1kNg/0?Shn0492@P1@D500@d=306UYJ@1BDU80PX:2051@D01TI6@0ADE506UY
J@2BTY80NWYj04Q8B024Q8@0GEeM06i^KP1cLg<0N7Qh04A4A00208V9R@1n05ADE01MGEd0J6QX0:n_
[`22PX80;Bd]07]kN`2CTi<0GEeM07alO010@400>SXj06e]K@1CDe<0EEEE08b<S01WIfL0OWin065Q
H@1gMgL0;Bd]05UIF@0d=3@0QhN706a/K015ADD0PX:206EUI@1SHf<0FUYJ03lo?`1FEUH0[jn_03hn
?P0^;Rh0RXZ:06=SH`0n?Sh0M7Ad08N7Q`1@D500KFe]09^KV`1ZJVX0?Cdm0;^k^`2@T900BDU9096A
T@1GEeL0^;Rh0:f][@0n?Sh0>SXj061PH00];Bd0ATI602TY:@2MWId0YJFU06EUI@20P8007QhN02`/
;01nOWh0@T9202<S8`1PH600;2`/06EUI@1VIVH0PH61079bLP11@D40DE5A07AdM00C4a<0=cLg05mO
G`2UYJD0KVi^09FEU@0i>CT0<C4a07EeM@1KFe/0>3Ph08b<S027QhL09RHV09BDU01PH600:BTY02XZ
:P1nOWh0DE5A04i>CP1aLG40@D5101XJ6P1ZJVX0Kfm_02LW9`1@D500EeMG03@d=02NWYh0?Cdm08^;
R`1eMGD0>c/k092@T02a/K40XJ6Q03De=@1CDe<0FUYJ02TY:@0[:b/0GemO08b<S014A4@0LG5a0:NW
Y`1SHf<06QXJ0P1YJFT05`0/;2`0K6a/06IVIP0g=cL0KFe]02PX:00j>SX0D51@02`/;00m?Cd0JVYZ
05UIF@1lO7`0QXJ60410@01nOWh0^;Rh09jNWP16ATH0?3`l08F5Q@1>CTh06a/K00kooooo02/0CDe=
09ZJVP2f][H0>c/k01lO7`0M7Ad071`L01XJ6P0G5aL0;2`/038b<P2ATI40RXZ:09bLW01;Bd/0;bl_
04M7A`0E5AD051@D01<C4`0l?3`0S8b<06a/K00f=SH0ZZZZ07]kN`2EUID0VYZJ07MgM`28R8P0EUIF
05MGE`1_Kfl0TY:B0;>c/`31`L40XJ6Q04U9B@1FEUH0J6QX04e=C@0I6AT0410@00803@d=01H04a<C
03lo?`1/K6`0@d=306IVIP1IFET0Kfm_09NGU`3?cll0/;2`0;^k^`1UIFD0;Rh^03/k>`1JFUX0Ogmo
079bLP20P800Rh^;0861P@2UYJD0^[Zj0P36a/H00`1?Cdl04a<C00d=3@0500d=3@0W010@400C4a<0
6ATI038b<P0[:b/0;bl_09jNWP3Ng]h0Ng]k02HV9P0E5AD06ATI01LG5`0E5AD0ATI6071`L00K6a/0
7alO0;Rh^02NWYh0FUYJ05QHF03De=@0WIfM04Y:BP15ADD0AdM707QhN02GUiL0KFe]08N7Q`0D51@0
GEeM02HV9P0I6AT05ADE01TI6@0U9BD0Lg=c0080<c<c03P0820P02PX:01MGEd0?Cdm028R8P0[:b/0
D51@030`<00A4A405ADE01TI6@0[:b/0GUiN0;Rh^023Ph<0^[Zj06QXJ00c<c<09bLW01dM7@0K6a/0
5QHF01DE5@0G5aL08b<S069RHP0P8200LW9b07=cL`20P800ZZZZ0=GEe@3<c<`0^;Rh0;fm_@32`/80
[jn_07AdM00F5QH05aLG01`L700b<S80De=C071`L01OGel0EEEE05eMG@1GEeL0NGUi06][J`0O7al0
<S8b05]KF`2PX:00<C4a01/K6`805aLG02l061PH03@d=02UYJD0aLG508J6QP10@400NWYj034a<@23
Ph<0W9bL04Y:BP0G5aL0BDU906i^KP0R8R807alO02<S8`0[:b/0<c<c059BDP2TY:@0alO70<W9b@2`
/;00S8b<06i^KP3:b/X0W9bL02l_;`0V9RH06ATI01DE5@1EEED061PH01DE5@0D51@04a<C01dM7@1K
Fe/0M7Ad0:NWY`3?cll0Z:RX0>;RhP3Ng]h0cLg=0=GEe@0202`/;00:05ADE00Y:BT05ADE01@D500F
5QH0De=C08f=S@24Q8@0@d=308:2PP80IFEU01<0Lg=c02l_;`0K6a/08b<S06][J`2d];@0Ti>C07Qh
N01NGUh04a<C03@d=01JFUX0Shn?0:VYZ@11@D40KVi^09VIV@1<C4`08B4Q00<071`L01<08R8R05EE
E@1?Cdl0=SHf02l_;`2CTi<0=3@d024Q8@0b<S80];Bd0=CDe02j^[X0[:b/092@T02PX:00^;Rh0;bl
_02_[jl0Vi^K013ooooo000>ooooo`1B0<c<c01gMgL0Fe]K05iNGP2h^;P0P820071`L01gMgL0>SXj
06IVIP2DU9@0De=C06m_K`0j>SX0Rh^;07moO`1DE5@0@D5105YJFP21PH40@T9202h^;P1GEeL0?Cdm
06EUI@1mOGd08b<S055AD@11@D40J6QX06EUI@0U9BD0Bd];06IVIP0k>c/0MgMg05eMG@0W9bL0<c<c
04i>CP0Q8B4071`L04a<C00k>c/0QHF504e=C@0m?Cd0G5aL04A4A01UIFD0?clo02XZ:P1lO7`0BTY:
030`<01RHV80=SHf04Q8B01LG5`0EeMG03hn?P2HV9P0>CTi028R8P14A4@0GUiN0451@@1LG5`0Hf=S
02`/;01DE5@0?3`l071`L01?Cdl0<30`06e]K@1RHV80NGUi0:6QX@1mOGd0?3`l051@D080Fe]K01h0
KFe]08R8R01NGUh092@T05EEE@1FEUH0@d=304i>CP1gMgL0F5QH03@d=01DE5@0JVYZ05]KF`1TI6@0
@T9206UYJ@2IVIT0BDU906][J`1EEED0D51@07IfMP1LG5`0@d=305MGE`1IFET0M7Ad0410@01lO7`2
04E5A@0605mOG`1NGUh0>3Ph06m_K`0c<c<0F5QH0P1MGEd0800[:b/0K6a/07IfMP1@D500GemO03hn
?P26QXH0IFEU03<c<`1hN7P0GemO05QHF01KFe/0>c/k03Ti>@1PH600;Bd]03De=@1HF5P0@d=308R8
R01CDe<0@D5102l_;`1/K6`0?3`l08N7Q`1LG5`0M7Ad09VIV@23Ph<0T92@0P1:BTX0N01PH600/K6a
0861P@10@400Vi^K04A4A013@d<0[Zj^0:FUY@1ZJVX0G5aL092@T01PH600<c<c0;no_`27QhL0Cdm?
09fMW@14A4@0FEUI071`L011@D40B4Q8061PH014A4@0PX:203dm?@1nOWh0QhN706UYJ@2HV9P0DE5A
02XZ:P1kNg/0:b/[04a<C01mOGd0Cdm?07MgM`1ADE40Ng]k0410@00/;2`0OGem07]kN`1BDU80L71`
06IVIP2>SXh0;2`/05ADE01=CDd0<S8b061PH010@400:BTY061PH01oOgl0;bl_0:n_[`1bLW80DE5A
03<c<`2:RXX0EUIF04a<C01DE5@0K6a/02d];@1LG5`0HV9R04=3@`1SHf<0>3Ph01TI6@1ZJVX0;bl_
07UiN@1bLW80D51@04e=C@1LG5`0MgMg038b<P1DE5@0JVYZ03@d=00J6QX0C4a<092@T014A4@0M7Ad
0:VYZ@1_Kfl092@T08V9R@1eMGD0<S8b06=SH`1XJ6P0>CTi065QH@0l?3`0A4A405EEE@0a<C40D51@
05QHF017AdL0QhN708b<S00j>SX0;Bd]04];B`1fMWH0Bd];03Lg=`1`L700HF5Q03`l?0kooooo01T0
Kfm_0=7Ad@3?cll0Q8B405UIF@0f=SH08R8R01dM7@0S8b<0I6AT0:^[Z`3Cdm<0c/k>0=3@d01UIFD0
@d=305mOG`0g=cL0?Shn051@D01=CDd0Vi^K09bLW02<S8`0C4a<00805ADE02d0ADE509NGU`3;bl/0
[jn_06IVIP1>CTh0B4Q807UiN@28R8P0a<C40;Jf]P1FEUH05ADE00`<300A4A405ADE00h>3P0B4Q80
6ATI05]KF`2QXJ40XZ:R08V9R@11@D40=SHf07emO@2SXj<0CTi>03Lg=`17AdL02PX:00d=3@0?3`l0
9RHV024Q8@0c<c<0EUIF06QXJ02g]kL0[jn_03`l?00T92@0@d=3034a<@0>3Ph00`0=3@d200h>3P0`
014A4@0C4a<061PH02PX:00m?Cd0=3@d0;Zj^P2g]kL0;Rh^01LG5`0I6AT06a/K01HF5P0i>CT0ZJVY
03Ph>00S8b<0:2PX07]kN`2BTY80?Cdm03@d=02WYjL0;2`/06=SH`0g=cL0<30`06ATI02OWil0/k>c
08R8R00/;2`0IfMW02XZ:P0/;2`0Hf=S07AdM01cLg<0WYjN08:2PP18B4P0<S8b03@d=011@D405QHF
01lO7`13@d<0AdM70`0C4a<02P0E5AD0<C4a04M7A`1eMGD0?3`l07AdM03Bd]80L71`03Lg=`0Y:BT2
02/[:`0S01`L700M7Ad0=cLg055AD@11@D40=cLg034a<@0i>CT0@d=306][J`3AdM40b/[:0:ZZZP2d
];@0UiNG09fMW@1KFe/0P82007emO@1eMGD0GEeM05ADE012@T80820P01`L700M7Ad0Hf=S096AT@0O
7al08B4Q059BDP27QhL0Ogmo01/K6`0L71`00P0E5AD00`0K6a/0B4Q80=7Ad@020<71`@1=051@D01H
F5P0QHF50<30`01lO7`0Jf][0<S8b03Ng]h0_[jn0;6a/@2d];@0X:2P092@T026QXH0IVIV04];B`1d
M7@0YjNW09FEU@10@400Hf=S0;Bd]01RHV809bLW01`L700M7Ad0FEUI01DE5@0K6a/04a<C01DE5@0`
<300Ph>309:BTP2LW9`0^k^k0=CDe02RXZ80TY:B0<K6aP3Ng]h0F5QH06IVIP0O7al05ADE01LG5`0E
5AD05aLG05=CD`1^KVh0I6AT04M7A`1ZJVX0<c<c04I6AP2GUiL0?Cdm04=3@`1JFUX0QXJ60:n_[`2T
Y:@0YZJV0:BTY02=SHd0V9RH05aLG01nOWh0bLW906EUI@1CDe<0G5aL05YJFP0Q8B406QXJ01LG5`0M
7Ad00P14A4@04@1:BTX0;Rh^02PX:01bLW80HF5Q02@T900];Bd0R8R8061PH01QHF40I6AT01XJ6P0N
7Qh0Q8B405MGE`12@T80BDU9013ooooo000>ooooo`1o09VIV@1KFe/0FUYJ05QHF02@T900LW9b069R
HP1mOGd0Lg=c05YJFP2HV9P0EUIF07UiN@0g=cL0Lg=c0820P016ATH07QhN05]KF`25QHD0I6AT0410
@01IFET09RHV03Lg=`1WIfL0820P04Y:BP0g=cL0DU9B06EUI@0W9bL0De=C05IFEP0`<300?clo04A4
A00Q8B40>c/k05QHF00Y:BT07QhN04a<C00X:2P0?Shn04a<C00c<c<0A4A402d];@1ZJVX0DU9B03@d
=028R8P0<C4a06UYJ@2DU9@0DU9B04m?C`1=CDd0>SXj05eMG@2n_[h0H61P028R8P1HF5P0BTY:030`
<01gMgL0R8R804E5A@18B4P0:2PX06i^KP1LG5`0EUIF07]kN`1EEED0JFUY07inOP1kNg/0=SHf04];
B`1FEUH0>c/k03`l?02ATI40EeMG01dM7@1JFUX0DU9B04Q8B01BDU80HV9R03dm?@0i>CT0G5aL06][
J`1BDU80H61P03De=@0i>CT0Ogmo04A4A01GEeL0GUiN04U9B@1:BTX0F5QH03Lg=`1IFET0EUIF079b
LP1:BTX0I6AT04E5A@1EEED0Hf=S05MGE`0n?Sh0KFe]04m?C`1gMgL0F5QH05iNGP0_;bl0LW9b075a
L@0205ADE00c05eMG@28R8P0DE5A02XZ:P1^KVh0EUIF069RHP1=CDd0?Cdm03hn?P1>CTh0>CTi03lo
?`1CDe<0@d=308F5Q@1;Bd/0FEUI01lO7`1EEED0<c<c06e]K@0o?cl0DU9B06YZJP1`L700PX:203lo
?`0h>3P0I6AT0:NWY`29RHT0=3@d09BDU01CDe<0@41005eMG@2@T900I6AT05]KF`29RHT0NWYj0492
@P2WYjL0R8R804];B`2YZJT0De=C07UiN@2LW9`0ATI60080A4A404<08R8R06][J`0h>3P0MWIf06IV
IP0o?cl0KVi^0410@00W9bL0O7al028R8P0/;2`0J6QX04U9B@1WIfL0De=C07UiN@1ADE40;bl_04U9
B@26QXH0G5aL08B4Q01QHF40PH6103dm?@1YJFT0DE5A02l_;`1DE5@0Cdm?04A4A02CTi<0VYZJ04=3
@`30`<00JFUY05MGE`0g=cL0QHF505aLG01<C4`0GUiN06UYJ@0V9RH0GemO07IfMP1>CTh0J6QX04Q8
B00Z:RX0OWin02l_;`1kNg/0P820061PH01UIFD0KFe]07MgM`0_;bl0EeMG06=SH`0R8R807alO04a<
C02BTY80GemO0080/K6a01/0L71`034a<@2HV9P0H61P02h^;P1]KFd0MgMg03dm?@1QHF40Cdm?05IF
EP1^KVh0;bl_051@D015ADD0<c<c08:2PP2EUID0De=C03lo?`1IFET0S8b<04M7A`0k>c/0Hf=S07mo
O`1=CDd03_ooool0>`2e]KD0c/k>0<[:bP2:RXX0H61P05iNGP1KFe/0FUYJ05iNGP1LG5`0^k^k0=7A
d@2?Shl0^[Zj0<S8b02b/[80^KVi07UiN@1BDU80ATI603Ti>@0c<c<0EEEE0;Zj^P1JFUX0>3Ph02l_
;`0L71`07AdM0:ZZZP29RHT0Lg=c05EEE@1UIFD0T92@0:VYZ@2j^[X0Shn?04M7A`14A4@0=SHf02/[
:`0F5QH03Ph>01DE5@0[:b/0P8200:VYZ@1lO7`0KVi^0451@@1;Bd/0QhN704e=C@0k>c/07AdM055A
D@0B4Q803Ph>00803@d=00`0410@024Q8@0N7Qh07alO055AD@1hN7P0?clo01DE5@0a<C40DU9B014A
4@0@410200h>3P1d00d=3@0>3Ph03@d=010@400G5aL08B4Q03Hf=P1LG5`0aLG50:^[Z`1PH600:2PX
028R8P0W9bL0LW9b0;Ng]`2f][H0?Shn02h^;P12@T80NGUi0<;2`P1oOgl0@D5106YZJP0C4a<051@D
04A4A00/;2`0;Bd]07EeM@1nOWh0E5AD09>CT`32`/80YZJV0:BTY01CDe<0=cLg03Hf=P1_Kfl0E5AD
092@T01jNWX0PX:203De=@0M7Ad0;Bd]05eMG@0C4a<03Ph>014A4@0D51@04a<C02PX:00o?cl0Ng]k
04];B`11@D40/[:b0;jn_P1UIFD0EeMG0861P@2KVi/0@41004M7A`1iNGT0CTi>04];B`0L71`05aLG
02<S8`0l?3`0GUiN0;Ng]`2PX:00W9bL0;:b/P29RHT0]kNg0;Rh^01iNGT0O7al08Z:RP1lO7`0GEeM
04Q8B017AdL0=cLg03<c<`1IFET0VYZJ03Lg=`0Q8B40AdM7061PH02m_Kd0<30`01hN7P0L71`0:b/[
04m?C`17AdL0X:2P0<;2`P3AdM40AdM7075aL@0n?Sh0LG5a0=[JfP36a/H0e=CD0=WIf@1[Jf/204U9
B@1E04I6AP13@d<0AdM706UYJ@2o_kl0i>CT0>?Sh`2ATI409RHV02d];@2n_[h0RHV902h^;P0O7al0
<30`061PH00M7Ad0;Bd]01@D500E5AD07QhN02/[:`2CTi<0b<S80<C4a02n_[h0UIFE0:JVYP3GemL0
/[:b09FEU@1EEED0@d=301/K6`0E5AD051@D01HF5P1JFUX0EEEE069RHP1CDe<0CTi>034a<@1nOWh0
QHF504U9B@1<C4`0AdM704=3@`2=SHd0Q8B4092@T01nOWh0MGEe065QH@2?Shl0LG5a0<K6aP2MWId0
FEUI04U9B@1OGel0EUIF02TY:@0I6AT0>c/k03hn?P1;Bd/0D51@02XZ:P0V9RH0F5QH09ZJVP0`<300
?Cdm0:JVYP1RHV80GEeM06MWI`0/;2`07alO06m_K`0Z:RX0AdM704U9B@0@ooooo`003_ooool0H@33
`l<0KFe]05EEE@1bLW80VIVI08B4Q01MGEd0Ng]k0;6a/@1PH600WinO05QHF02<S8`0?Shn07EeM@2;
Rh/0B4Q804A4A01EEED0V9RH06m_K`0k>c/0Lg=c05mOG`0d=3@0OGem01dM7@0i>CT0DE5A06m_K`1g
MgL0:2PX05IFEP29RHT0EEEE04i>CP12@T807QhN04a<C01BDU80820P028R8P1:BTX0;bl_04i>CP15
ADD0;Bd]04I6AP10@400GemO051@D00b<S80T92@0451@@0c<c<0IVIV04M7A`1=CDd0DE5A0451@@0g
=cL0MWIf0492@P0S8b<0AdM704e=C@0h>3P0IfMW06i^KP13@d<0@41003Ph>01eMGD0@d=306UYJ@1m
OGd0JVYZ05ADE01eMGD0M7Ad038b<P14A4@0IFEU05ADE017AdL0UYJF05UIF@0K6a/0G5aL06YZJP1T
I6@0DU9B061PH00j>SX0?Cdm06=SH`1cLg<00P1LG5`0/`1>CTh0FUYJ09>CT`14A4@0G5aL06MWI`1A
DE40CTi>0451@@0];Bd0DU9B061PH01[Jf/0ADE505UIF@17AdL0DU9B061PH01VIVH0@T9207IfMP1H
F5P0R8R805eMG@1XJ6P0=cLg08B4Q022PX80GUiN059BDP1[Jf/0T92@065QH@0X:2P0Kfm_06ATI01Y
JFT0IfMW03/k>`1:BTX0GUiN05eMG@16ATH0HV9R05IFEP1nOWh0E5AD07AdM00Z:RX0Fe]K02h^;P1j
NWX0HF5Q079bLP23Ph<0Ng]k07moO`1YJFT0Fe]K04e=C@2JVYX0Hf=S03Ph>02@T900Kfm_04];B`1g
MgL0U9BD05aLG01DE5@0TI6A06QXJ01<C4`0WinO08>3P`1ADE40Xj>S05=CD`1lO7`0ZJVY04A4A00o
?cl0?Shn02HV9P1QHF40=3@d07QhN01OGel0?Shn08N7Q`0d=3@0;Rh^07QhN00T92@0<c<c06ATI00a
<C40EEEE05iNGP1[Jf/0F5QH03Xj>P1BDU80Jf][06=SH`2<S8`0GEeM09:BTP1BDU80Lg=c05MGE`0X
:2P0Cdm?03Lg=`13@d<0MWIf09RHV01@D500a<C406=SH`16ATH0>SXj08B4Q01NGUh0TY:B09FEU@1d
M7@09RHV05aLG01YJFT0@41006m_K`1<C4`0;Bd]09BDU00l?3`0OGem09:BTP1PH600Ti>C06e]K@1d
M7@0=3@d08F5Q@21PH40CTi>024Q8@19BDT0H61P03hn?P2?Shl0UYJF05eMG@0i>CT0XZ:R02TY:@0a
<C40RXZ:06QXJ01MGEd0WinO04Y:BP1@D500NWYj03hn?P1ZJVX0M7Ad02PX:01dM7@0VIVI05UIF@1G
EeL0P8200:f][@1`L700D51@0:RXZ01oOgl0>3Ph00kooooo0200QXJ604a<C01FEUH0]KFe08^;R`1X
J6P0CDe=04i>CP2HV9P0QXJ602HV9P1cLg<0`l?30:JVYP39bLT0cLg=0;no_`2XZ:P0GEeM051@D01?
Cdl0Ng]k08j>SP2OWil0Lg=c09>CT`34a<@0cLg=0:b/[03Cdm<0a<C40;6a/@80/;2`01X0]KFe0<71
`@3Bd]80_[jn061PH00K6a/06QXJ02HV9P0F5QH06ATI055AD@24Q8@0WYjN0;Bd]02m_Kd0QhN702l_
;`0=3@d092@T04M7A`0k>c/071`L04U9B@0I6AT0410@00d=3@803Ph>00`061PH02/[:`0W9bL0FEUI
0410@01hN7P0>SXj04I6AP1BDU80;bl_01`L700C4a<3014A4@803Ph>01H03@d=010@400_;bl0RHV9
0=CDe02OWil0PX:208J6QP1lO7`0Zj^[08n?S`1CDe<0Xj>S0:VYZ@2BTY80N7Qh07alO039bLT0b/[:
09ZJVP1mOGd0:b/[0P0O7al06018B4P0ADE505=CD`1>CTh0=3@d02XZ:P2/[:`0c/k>07alO01[Jf/0
KVi^08V9R@2`/;00N7Qh08f=S@2HV9P0/[:b07inOP18B4P0CDe=02PX:00C4a<051@D01<C4`805ADE
00@0820P04];B`1GEeL0EEEE0P1BDU80M@3Lg=`0`L710<71`@3Bd]80];Bd07YjNP2IVIT0SHf=06=S
H`1=CDd06ATI01DE5@0I6AT07QhN055AD@2<S8`0GemO065QH@2YZJT0V9RH0=3@d035aLD0Ti>C071`
L01ADE40<c<c01dM7@0H61P06a/K020P800Y:BT09bLW05iNGP1RHV8092@T03Xj>P1^KVh0VIVI09fM
W@0U9BD0B4Q806YZJP0a<C40=3@d05MGE`37alL0/;2`08Z:RP11@D40;Rh^08R8R03MgMd0hN7Q05aL
G02YZJT0/;2`055AD@1?Cdl0BDU9059BDP1dM7@0][Jf0>GUi@3Ti>@0f][J07IfMP0Q8B408b<S08J6
QP2_[jl0GEeM02PX:00j>SX0K6a/01/K6`0L71`06ATI038b<P0S8b<09bLW09JFUP3Vi^H0aLG506AT
I033`l<0a<C408j>SP1nOWh0Z:RX07moO`1BDU80ADE502d];@0F5QH061PH071`L01?Cdl0MgMg055A
D@11@D40G5aL0820P00[:b/06QXJ028R8P0[:b/0DE5A09FEU@26QXH0W9bL08b<S01KFe/0CDe=0:NW
Y`2][Jd0b<S808F5Q@0207AdM00H059BDP1:BTX0DU9B06=SH`10@4007QhN030`<01ADE40DU9B079b
LP38b<P0aLG509:BTP1/K6`0W9bL09>CT`1DE5@0FUYJ06IVIP1CDe<0MGEe04i>CP1UIFD0M7Ad4?oo
ool000kooooo02d0b/[:06=SH`1ZJVX0KFe]09FEU@1WIfL0M7Ad09:BTP2TY:@0F5QH0820P01PH600
Vi^K03lo?`2;Rh/0RXZ:04m?C`0l?3`0Ogmo08V9R@1@D500DU9B08N7Q`1UIFD0ADE505iNGP0N7Qh0
I6AT06a/K01^KVh0Fe]K03@d=01kNg/0S8b<05=CD`1JFUX0?clo030`<01JFUX0@T9202LW9`0m?Cd0
G5aL03dm?@1<C4`00P0j>SX0<01HF5P0?clo05]KF`15ADD0AdM709NGU`19BDT0>SXj06a/K019BDT0
IVIV079bLP1IFET0?Shn08Z:RP0/;2`0>3Ph069RHP1HF5P0@T9207IfMP1[Jf/0ATI604i>CP0m?Cd0
Ng]k03Ph>01PH600Shn?05IFEP1LG5`0O7al05YJFP0c<c<0IVIV05eMG@1EEED0BDU908B4Q00a<C40
8B4Q055AD@1DE5@0>c/k05=CD`1VIVH0;bl_06i^KP80GUiN00`0=SHf08N7Q`1MGEd0GemO08:2PP1<
C4`0NGUi06MWI`1DE5@0De=C0451@@0f=SH2065QH@2W05MGE`0/;2`0Lg=c06EUI@1ADE40K6a/04U9
B@18B4P0Rh^;04i>CP1mOGd0EUIF07YjNP0i>CT0Rh^;05QHF01:BTX0Ng]k07=cL`2CTi<0@D5102/[
:`2CTi<0Q8B407IfMP0d=3@0<c<c059BDP27QhL0I6AT055AD@1TI6@0820P06m_K`2HV9P0OGem02PX
:01LG5`0CDe=09RHV01TI6@0LW9b07YjNP0m?Cd0R8R805QHF01XJ6P0EEEE09:BTP12@T80BDU909VI
V@1kNg/0De=C08N7Q`2GUiL0:b/[06EUI@2<S8`0Hf=S04Y:BP2_[jl0Fe]K04a<C02@T900DU9B0:FU
Y@2g]kL0=cLg059BDP1VIVH0?Cdm06ATI00i>CT0Xj>S08J6QP1BDU80VYZJ02@T900o?cl0MGEe02l_
;`12@T80MGEe03Hf=P1`L700Rh^;07UiN@1?Cdl0<c<c05QHF019BDT0:b/[07MgM`23Ph<0RXZ:0492
@P1^KVh0ATI602d];@1`L700HV9R04M7A`1]KFd0Ng]k05MGE`2CTi<0MgMg06UYJ@1:BTX0VIVI04Q8
B01KFe/0LG5a05YJFP0X:2P0LG5a07=cL`0h>3P0N7Qh05MGE`0c<c<0Z:RX034a<@2:RXX0PX:204];
B`2BTY80[jn_096AT@0Y:BT0JFUY07AdM00d=3@0=cLg075aL@25QHD0De=C0:2PX01WIfL09RHV0492
@P1jNWX0IfMW02l_;`1HF5P0=cLg03Ti>@1bLW80EeMG059BDP1ADE408b<S04i>CP1`L700BTY:07Yj
NP2LW9`0<30`03`l?01MGEd0Ogmo05iNGP0d=3@0Q8B4059BDP0R8R803_ooool03@2b/[80]KFe096A
T@1MGEd0MWIf07YjNP1nOWh0MGEe06MWI`1gMgL0LW9b07]kN`34a<@00P38b<P0;02UYJD0YjNW0<?3
``27QhL0M7Ad06ATI00h>3P0:RXZ02HV9P1FEUH0Zj^[0=CDe03Rh^80e]KF0:^[Z`38b<P0Xj>S08>3
P`1jNWX0P82008>3P`2GUiL0IFEU06UYJ@20P800PH6108Z:RP2CTi<0P820071`L02?Shl0WYjN075a
L@1YJFT0HF5Q02TY:@0R8R808b<S06a/K013@d<0<c<c02PX:019BDT07QhN100>3Ph05P0J6QX0>SXj
0820P01>CTh0Fe]K0::RXP2FUYH0XZ:R0:2PX01SHf<0DE5A04Q8B01ADE40F5QH05ADE01OGel0IfMW
06YZJP2=SHd0c<c<0=oOg`2SXj<20:NWY`0V06=SH`0k>c/09bLW028R8P0W9bL0BDU905iNGP2RXZ80
Y:BT069RHP0T92@0ADE504i>CP14A4@0HV9R06ATI01HF5P0DE5A05aLG01]KFd0IVIV03/k>`2FUYH0
SXj>06IVIP1TI6@0Jf][0820P025QHD0SHf=092@T02SXj<0/;2`0:2PX02`/;00P820030`<00C4a<2
014A4@804a<C00@0A4A406][J`1:BTX0O7al0P23Ph<05`2CTi<0IfMW0861P@2ATI40NGUi096AT@35
aLD0QhN707UiN@17AdL06QXJ01TI6@0M7Ad09BDU03@d=01lO7`0=CDe02d];@1OGel0Bd];06EUI@0Y
:BT05aLG00805ADE00<051@D01DE5@0D51@00`0E5AD02`0G5aL0:2PX07]kN`0_;bl0>c/k07emO@2Y
ZJT0emOG0<;2`P1iNGT0EUIF0080G5aL03X0B4Q804M7A`3Ph>00[jn_06m_K`1jNWX0^[Zj0<K6aP2[
Zj/0RHV907EeM@2a/K40Cdm?0492@P16ATH0D51@09JFUP20P800Ogmo0;2`/015ADD0RXZ:07=cL`0T
92@0?Shn09RHV037alL0KFe]04i>CP1iNGT07AdM024Q8@10@400MgMg05mOG`2JVYX0Q8B40=GEe@3I
fMT0bLW90<_;b`2=SHd0HV9R09:BTP0Y:BT0:RXZ04Q8B01HF5P0HF5Q03hn?P1?Cdl0SHf=07MgM`2J
VYX0IfMW05QHF01bLW80;2`/0P0L71`09`0N7Qh0820P02PX:01QHF40W9bL07EeM@2RXZ80NGUi051@
D01kNg/0^KVi0861P@16ATH0HV9R08f=S@2OWil0R8R808V9R@2PX:00R8R806e]K@1eMGD0Vi^K09bL
W021PH40Y:BT0=CDe03Ogml0e]KF09FEU@1nOWh0_;bl0:^[Z`2YZJT0]KFe06ATI01NGUh0Fe]K06m_
K`0@ooooo`003_ooool0503De=@0EEEE05UIF@1dM7@0^k^k08F5Q@1dM7@0Rh^;0:^[Z`1OGel0QXJ6
04i>CP1eMGD0>3Ph08j>SP1_Kfl0Ogmo03De=@2:RXX0Lg=c0P10@400O02=SHd0J6QX075aL@1ADE40
5aLG04];B`1BDU80PX:203/k>`0R8R80GUiN03Xj>P19BDT0PH61055AD@0H61P0DU9B05YJFP0R8R80
9BDU04A4A00j>SX0MgMg03dm?@0_;bl0?clo03dm?@1kNg/0;Rh^03<c<`1hN7P0>c/k04i>CP1hN7P0
:b/[04U9B@1NGUh0BTY:04=3@`2TY:@0=SHf01hN7P1=CDd0I6AT03Xj>P27QhL0L71`02d];@1?Cdl0
=SHf06m_K`0c<c<0;Bd]06MWI`1MGEd0GUiN07inOP1@D5008B4Q04U9B@1JFUX0DE5A05UIF@1eMGD0
DU9B01dM7@1MGEd0?Shn02l_;`21PH40IVIV04=3@`22PX80F5QH06QXJ011@D40OGem055AD@1]KFd0
PH6104Q8B01DE5@0EeMG06ATI023Ph<0G5aL04=3@`1DE5@0O7al07IfMP0n?Sh0Lg=c02LW9`1LG5`0
Lg=c075aL@1CDe<0Lg=c03lo?`20P800E5AD030`<012@T80WYjN07QhN01QHF40H61P0451@@2ATI40
G5aL02l_;`20P800ATI606i^KP1GEeL0?Shn04Q8B01/K6`07AdM03<c<`1UIFD0@D51061PH01DE5@2
03lo?`1N071`L00i>CT0NGUi0451@@28R8P0Shn?04a<C01/K6`0?3`l075aL@2BTY80ZJVY05]KF`0j
>SX0OGem0451@@19BDT0P82009^KV`1GEeL0DU9B07QhN00k>c/071`L096AT@1mOGd0CTi>07MgM`13
@d<0EUIF07alO00f=SH0D51@05]KF`19BDT0Ng]k05EEE@2>SXh0MGEe02<S8`1jNWX0@41003hn?P1f
MWH06ATI05mOG`2;Rh/0CDe=05EEE@1JFUX0I6AT04e=C@19BDT0MgMg0451@@18B4P0PX:205YJFP1g
MgL07AdM06e]K@10@400;Bd]061PH010@400?Shn09FEU@2KVi/0R8R806UYJ@16ATH06QXJ06IVIP2Z
ZZX0IFEU06QXJ01YJFT071`L01/K6`1TI6@0NWYj03`l?024Q8@0:2PX03dm?@2XZ:P0BDU909RHV01N
GUh06ATI04a<C01LG5`0M7Ad02TY:@80IFEU00L0=cLg01dM7@1LG5`0Jf][06=SH`33`l<0Q8B40080
?Cdm01T0TI6A06UYJ@0n?Sh0QhN7079bLP1CDe<0RHV904E5A@1gMgL0IfMW034a<@1aLG40NWYj03<c
<`1fMWH0ZZZZ05=CD`1GEeL0JVYZ09:BTP1FEUH0?clo0;>c/`1jNWX0?Shn00kooooo04T0P82004m?
C`1;Bd/0;Bd]02HV9P0S8b<0;bl_06e]K@2GUiL0Rh^;0<?3``3IfMT0UIFE0:RXZ031`L40ZJVY08:2
PP12@T80;Bd]03@d=018B4P0EUIF065QH@0o?cl0@T920:2PX03IfMT0g=cL0>?Sh`2ZZZX0UIFE0=CD
e02g]kL0Xj>S0:FUY@1]KFd0;bl_03dm?@1@D500IVIV0:ZZZP3<c<`0/k>c06m_K`2[Zj/0WYjN07em
O@0W9bL05ADE01LG5`12@T80AdM7071`L02=SHd0EEEE03Xj>P0Z:RX09RHV05IFEP0B4Q8051@D014A
4@0@4100>3Ph05eMG@20P800M7Ad055AD@2g]kL0LG5a051@D01/K6`05aLG00<0410@01h08R8R02l_
;`0i>CT0?clo055AD@2DU9@0bl_;0<S8b02h^;P0Shn?05aLG00l?3`0>CTi020P800C4a<071`L01LG
5`1oOgl0QhN707emO@1KFe/08R8R03De=@0f=SH04A4A01<C4`0G5aL0?3`l08Z:RP2PX:0209>CT`0B
0:FUY@2TY:@0OGem04=3@`0j>SX0L71`04];B`0_;bl0>3Ph05ADE01fMWH0FUYJ06a/K01KFe/0CDe=
02d];@14A4@08B4Q0P0E5AD07@0Q8B40Hf=S06=SH`1?Cdl0BTY:079bLP2b/[80W9bL06IVIP0[:b/0
8R8R03Xj>P14A4@0HF5Q08R8R01gMgL0@D5101dM7@0N7Qh08b<S02/[:`1KFe/0DE5A04i>CP0O7al0
=SHf05]KF`1_Kfl06a/K00<05ADE00<05aLG01@D500D51@00`0E5AD09`0F5QH092@T0861P@2ZZZX0
EEEE09NGU`2JVYX0[Zj^07YjNP2RXZ80QhN706a/K01mOGd0PH610:JVYP2l_;`0XZ:R08f=S@2LW9`0
f][J0;no_`1BDU80ZJVY01LG5`11@D40CTi>01XJ6P0H61P0820P04i>CP0c<c<0WinO04M7A`0J6QX0
9BDU07]kN`2QXJ40Bd];04M7A`020=;BdP170<C4a02JVYX0RHV90:^[Z`2o_kl0Y:BT0=OGe`2o_kl0
T92@069RHP0j>SX0QXJ607EeM@0E5AD0:2PX02`/;01FEUH07AdM01LG5`0I6AT0=cLg08Z:RP2j^[X0
`<300<O7a`2YZJT0EeMG07QhN00Y:BT06ATI01`L700M7Ad071`L01hN7P0P8200@T9207=cL`1aLG40
UYJF07alO015ADD0S8b<071`L021PH40NWYj03/k>`1EEED0^[Zj08R8R02EUID0Shn?0;2`/01bLW80
Kfm_07IfMP2/[:`0`l?30=WIf@3Shn<0i>CT0=KFeP2ZZZX0d=3@0>CTi02d];@0dM7A09BDU01=CDd0
PH6109NGU`2=SHd04?ooool000kooooo02h0_kno05aLG02;Rh/0H61P0:6QX@2;Rh/0W9bL0:VYZ@1A
DE40QHF509nOW`1UIFD0LW9b051@D02][Jd0E5AD04Q8B00i>CT0UIFE08>3P`0j>SX0EEEE061PH00^
;Rh0Cdm?065QH@0J6QX0@410028R8P1CDe<0GEeM02d];@1hN7P0EUIF03@d=010@400?3`l020P801N
GUh0I6AT02@T900X:2P0CTi>03Lg=`12@T80=cLg0P0a<C40K00R8R80BDU903@d=00j>SX0MgMg01/K
6`17AdL0Jf][03Xj>P1ADE40EUIF01hN7P1:BTX0QhN704a<C00c<c<0H61P059BDP0b<S80OGem08j>
SP1@D500@D5103Ph>025QHD0ADE506a/K02ATI40JFUY092@T02j^[X0N7Qh02/[:`1ADE40EEEE0410
@01ADE40IfMW05IFEP0L71`0FUYJ04];B`0[:b/0ADE504e=C@17AdL0O7al04Y:BP1^KVh0CDe=06AT
I00];Bd0FUYJ08b<S019BDT0OWin02h^;P0b<S80B4Q805ADE012@T80D51@04e=C@1PH600?3`l06m_
K`1EEED0ATI605mOG`1fMWH0FUYJ0861P@15ADD0Lg=c05]KF`1aLG40=3@d07QhN01`L700F5QH069R
HP1gMgL0SHf=04Y:BP0a<C40OWin05aLG01XJ6P0EeMG059BDP1XJ6P0O7al05aLG00_;bl0Jf][059B
DP2LW9`0VYZJ05EEE@0L71`0J6QX0410@01eMGD0:b/[04M7A`1jNWX0IfMW08>3P`80?Shn01H0@410
08Z:RP1bLW80>CTi07moO`1OGel0BTY:071`L01oOgl0@T92055AD@1`L700GEeM02XZ:P2[Zj/0HV9R
06=SH`28R8P0EEEE09>CT`2ZZZX0B4Q80P1PH600>00Z:RX0E5AD04Y:BP2OWil0M7Ad05mOG`1eMGD0
;Bd]03dm?@1YJFT0;Bd]05ADE01eMGD0ATI6065QH@1WIfL0Jf][01XJ6P0h>3P0O7al05MGE`1@D500
TI6A05iNGP20P8009BDU09^KV`13@d<0<S8b06][J`0h>3P0ADE50:j^[P2<S8`0R8R807EeM@1DE5@0
?3`l04a<C02DU9@0DU9B0;^k^`2YZJT0EeMG01hN7P1?Cdl0IfMW03Lg=`1nOWh0De=C03De=@22PX80
B4Q809jNWP2OWil0C4a<0P22PX809P23Ph<09bLW06][J`1QHF40?Shn02DU9@1WIfL0D51@06IVIP2l
_;`0FUYJ02/[:`1ADE40TI6A03Ti>@0d=3@0GemO02XZ:P1:BTX0PX:203lo?`10@400ADE502PX:01n
OWh0PX:202PX:01VIVH0[Jf]06EUI@1LG5`0NWYj0;2`/01[Jf/0Bd];0<30`01nOWh0@D513_ooool0
M01[Jf/0CTi>0451@@0[:b/09RHV024Q8@0N7Qh07AdM024Q8@1iNGT0UiNG0:>SX`1RHV80EEEE065Q
H@1[Jf/0J6QX01XJ6P0D51@04A4A00l?3`0@41004a<C03lo?`2`/;00VIVI0:RXZ02ATI40bLW906e]
K@1VIVH0_Kfm06e]K@1[Jf/0PX:20:2PX02TY:@0FUYJ03Xj>P2@T900X:2P08R8R02BTY80O7al092@
T027QhL0LW9b01<C4`0>3Ph03`l?010@400U9BD0PX:209^KV`1hN7P0>c/k03Ph>00K6a/0=CDe02LW
9`0E5AD04a<C024Q8@15ADD0B4Q803lo?`0l?3`0KVi^09FEU@2JVYX0K6a/0451@@0J6QX05ADE01LG
5`0U9BD0@4100451@@0i>CT0?3`l04U9B@2=SHd0f][J09NGU`2?Shl0][Jf08:2PP1NGUh0EUIF055A
D@1RHV80MWIf09VIV@2a/K40ZJVY04a<C01GEeL0;Rh^03Lg=`0m?Cd05ADE01@D500E5AD04a<C02@T
9017AdL0HF5Q09BDU02o_kl0/;2`08V9R@0k>c/0IfMW06QXJ00`<3006a/K0P0E5AD06`0b<S80F5QH
06a/K01WIfL0FUYJ020P800Q8B40DE5A03Ti>@0h>3P0B4Q806e]K@1/K6`0Hf=S06EUI@2JVYX0Xj>S
05QHF010@4007AdM01`L700X:2P0?3`l04Y:BP1jNWX0O7al0492@P0202XZ:P0:02DU9@0l?3`0:2PX
01DE5@12@T80BDU90410@01NGUh0LG5a038b<P805ADE00@05aLG01HF5P0C4a<04Q8B0P0C4a<0600@
41006a/K020P801TI6@0eMGE0;^k^`3De=@0g=cL0:2PX02e]KD0V9RH0::RXP1^KVh0Fe]K05iNGP0o
?cl0a/K60;2`/028R8P0O7al0:VYZ@3Ogml0Bd];07IfMP806ATI00P0IfMW01dM7@0M7Ad0@d=302TY
:@0Q8B40RHV9020P808071`L01d0<c<c0;2`/03Jf]X0T92@09FEU@3Vi^H0c/k>0=KFeP2CTi<0Jf][
08F5Q@21PH40`/;20;Ng]`2OWil0>3Ph01XJ6P1`L700NWYj01PH600F5QH08B4Q034a<@15ADD05aLG
01DE5@0I6AT0KFe]0=?Cd`020;no_`060<71`@2ATI40J6QX01XJ6P0L71`07AdM0P0L71`09P0M7Ad0
8B4Q0451@@1MGEd0L71`07YjNP2=SHd0G5aL06QXJ01GEeL0K6a/09VIV@28R8P0S8b<05mOG`0[:b/0
BTY:0:f][@2n_[h0`l?308:2PP2j^[X0clo?0=SHf02b/[80UiNG0=kNgP3EeMD0Shn?08:2PP2TY:@0
c<c<0>CTi02l_;`0VYZJ08F5Q@1XJ6P0GUiN4?ooool000kooooo01/0_Kfm055AD@1]KFd0K6a/08R8
R01GEeL0QhN709>CT`1cLg<0EUIF06EUI@1nOWh0N7Qh03<c<`25QHD0Fe]K059BDP0S8b<0Kfm_07em
O@0e=CD0EEEE09:BTP1BDU80@d=306UYJ@0U9BD00P1^KVh0?P1bLW80G5aL04m?C`2DU9@0OWin04Y:
BP13@d<0?clo01dM7@1WIfL0Fe]K020P800b<S80B4Q80410@01>CTh0?clo03Lg=`1;Bd/0:BTY055A
D@0f=SH0?Shn09:BTP0j>SX0=SHf06QXJ011@D40De=C07=cL`0l?3`0CDe=08j>SP0a<C40>CTi05EE
E@1QHF40>3Ph05=CD`1QHF40Bd];03Ph>00X:2P0DU9B03Hf=P1jNWX0QXJ604e=C@0o?cl0EUIF05QH
F00O7al0FUYJ05ADE00i>CT0EEEE05UIF@0k>c/0=cLg06UYJ@1UIFD0@T920P1bLW802`0j>SX0H61P
059BDP1UIFD0ATI609RHV01aLG40GEeM08B4Q01DE5@0Ogmo0080GUiN08d0DU9B06EUI@1:BTX0Hf=S
06UYJ@1MGEd0ADE509:BTP1nOWh0BTY:07UiN@1aLG40JVYZ08J6QP1=CDd0Ogmo07UiN@2=SHd0>SXj
08B4Q01GEeL0Kfm_096AT@2=SHd0Rh^;03Hf=P1<C4`0TY:B06a/K01ZJVX0<C4a04e=C@1VIVH0TY:B
075aL@0V9RH0GemO02XZ:P1BDU80MgMg06][J`0Z:RX0MGEe061PH01mOGd0BTY:061PH02<S8`0MGEe
09FEU@1>CTh0HF5Q05QHF02c/k<0UIFE04e=C@2DU9@0Jf][065QH@21PH40SHf=03`l?01HF5P0O7al
04A4A00c<c<0][Jf07IfMP1=CDd0GUiN05QHF02BTY80O7al02d];@1aLG40LW9b03dm?@1WIfL0AdM7
0:b/[02;Rh/0GUiN09FEU@0/;2`0EeMG07UiN@13@d<0R8R807emO@13@d<0RXZ:09ZJVP1lO7`0DU9B
0492@P1EEED0AdM704e=C@2QXJ40RXZ:09:BTP0J6QX0E5AD05MGE`0j>SX0OWin02TY:@0L71`0E5AD
06YZJP23Ph<0TI6A07EeM@1<C4`0Ph>30:>SX`0l?3`0De=C069RHP1>CTh08B4Q069RHP1WIfL0>3Ph
08F5Q@1>CTh0?Cdm08n?S`1IFET0_;bl09bLW018B4P0VYZJ0:b/[02JVYX0<30`08:2PP1GEeL0>CTi
04M7A`20P8000P1MGEd07@3>c/h0HF5Q038b<P0i>CT0L71`06=SH`0c<c<0Fe]K04A4A00o?cl0MgMg
04Y:BP1FEUH0<c<c01/K6`1NGUh0I6AT030`<016ATH0Vi^K04M7A`1cLg<0/K6a0;2`/01FEUH092@T
08V9R@14A4@08B4Q00kooooo01<0UiNG051@D00e=CD0;2`/02DU9@0O7al07AdM020P800L71`0ADE5
0410@014A4@0GemO03Ti>@1;Bd/0KFe]06i^KP0G5aL05ADE00<04A4A0540410@01@D501MGEd0WIfM
065QH@0c<c<0EUIF020P8013@d<0MGEe05mOG`0Q8B403`l?014A4@0^;Rh0MgMg09nOW`2PX:00[Zj^
06UYJ@2NWYh0clo?07EeM@0T92@0CDe=02d];@0@41003`l?014A4@0@41009RHV08b<S01kNg/0H61P
055AD@0K6a/07alO04m?C`0C4a<03Ph>03Ph>014A4@0:BTY01<C4`0G5aL0I6AT08F5Q@2PX:00SHf=
05IFEP1QHF40MGEe07UiN@2:RXX0WinO0:6QX@2KVi/0TI6A08j>SP2;Rh/0^k^k05eMG@1aLG40b/[:
0=SHf02b/[80T92@08N7Q`1eMGD0OGem0:j^[P3GemL0EeMG034a<@1BDU80?Shn03Lg=`15ADD05ADE
01<C4`0D51@00P0E5AD01`0F5QH0B4Q803Xj>P2[Zj/0`L710;>c/`1FEUH00P0Q8B4201DE5@0A01LG
5`0E5AD05ADE02LW9`0j>SX0OGem0:2PX011@D40@T9204M7A`1iNGT0Hf=S061PH01aLG40PH61059B
DP1JFUX00P22PX800`1JFUX0EUIF02d];@0201dM7@1P028R8P0g=cL0Ogmo0:NWY`1/K6`0?clo01lO
7`0N7Qh0>c/k01<C4`0E5AD05aLG07=cL`1/K6`0Hf=S065QH@17AdL07alO01PH600G5aL0410@014A
4@0B4Q803Ph>014A4@0@410061PH020P801dM7@0V9RH04=3@`2d];@0inOW0:j^[P1HF5P06a/K01`L
700I6AT061PH01HF5P0g=cL0cLg=06ATI00c<c<092@T03@d=02OWil0]KFe06ATI00Q8B405ADE061P
H01FEUH0ADE503De=@0O7al09bLW05IFEP0P82007AdM01`L700N7Qh0=cLg09fMW@3Ogml0^;Rh0=oO
g`3MgMd0[jn_03hn?P0c<c<0Fe]K04a<C02`/;00c/k>0:ZZZP0K6a/05aLG075aL@2?Shl05ADE01<C
4`0L71`09bLW04E5A@14A4@06ATI01dM7@0c<c<0WinO0<30`033`l<0b/[:0<k>cP1>CTh06ATI0P0K
6a/201`L700V01hN7P0Q8B40@41006a/K01[Jf/0P82009bLW02YZJT0:2PX02d];@1jNWX0Y:BT0;bl
_01SHf<0BDU903Lg=`1;Bd/0HF5Q09VIV@3Ph>00f=SH0<g=c@2UYJD0PX:208R8R02m_Kd0/;2`0:JV
YP3Ph>00emOG0:6QX@3:b/X0gmoO0<c<c01oOgl0IVIV051@D00i>CT@ooooo`003_ooool0o`39bLT0
EUIF08R8R023Ph<0/K6a05aLG01SHf<0Q8B4079bLP22PX80H61P05IFEP1kNg/0>CTi08R8R01KFe/0
MgMg03<c<`25QHD0IfMW04A4A01hN7P0S8b<04m?C`1ADE40=CDe01lO7`1BDU80Cdm?06=SH`0/;2`0
?3`l06e]K@1SHf<0Bd];04e=C@0i>CT09RHV06e]K@0];Bd08b<S04Q8B01ADE40>SXj05IFEP0L71`0
@T92055AD@0V9RH0BDU9030`<01^KVh0Vi^K03`l?00h>3P0JFUY03dm?@1MGEd0S8b<04Q8B01IFET0
QXJ603Ti>@0l?3`0EUIF06UYJ@16ATH0IFEU05iNGP1EEED0BDU903Ph>01/K6`0>CTi07MgM`2[Zj/0
DU9B06UYJ@1_Kfl0EUIF03lo?`2>SXh0G5aL04E5A@1PH600IfMW03`l?00K6a/0DU9B061PH011@D40
U9BD08>3P`1TI6@0A4A405EEE@1TI6@0;Rh^07emO@1LG5`0EeMG06YZJP1DE5@0J6QX059BDP1fMWH0
QXJ6051@D01?Cdl0GUiN08:2PP0l?3`0;Bd]07MgM`16ATH0GUiN09NGU`0m?Cd0FUYJ06i^KP1IFET0
OGem04Y:BP1cLg<0FEUI09ZJVP1DE5@0@D51065QH@1dM7@0V9RH04E5A@0];Bd0Q8B405QHF024Q8@0
Bd];03Ti>@1?Cdl0NGUi03lo?`0T92@0Kfm_04E5A@1WIfL0H61P04];B`12@T80IFEU05]KF`1hN7P0
BTY:07UiN@1[Jf/0;2`/06m_K`1;Bd/0EeMG09fMW@2e]KD0FEUI03Lg=`29RHT0GUiN08:2PP2i^KT0
YJFU04M7A`1:BTX0QhN705MGE`0a<C40];Bd07MgM`1?Cdl0K6a/04E5A@1^KVh0DU9B02h^;P12@T80
@41005=CD`1dM7@0?Cdm09nOW`1dM7@0ADE508Z:RP0Z:RX0:2PX04e=C@0f=SH0YZJV06IVIP0k>c/0
DU9B05=CD`1TI6@0<C4a04M7A`2?Shl0E5AD02`/;01UIFD0NGUi0861P@0`<300Shn?03Ph>00a<C40
M7Ad04e=C@10@400UYJF06m_K`1gMgL0D51@06][J`1LG5`0JFUY0:>SX`1YJFT0[Jf]09^KV`1>CTh0
92@T06YZJP1NGUh08B4Q06a/K011@D40De=C07YjNP0a<C40Rh^;05MGE`0^;Rh0NWYj08Z:RP22PX80
:BTY05iNGP1;Bd/0<c<c024Q8@1KFe/0>c/k06m_K`36a/H0G5aL02HV9P0g=cL0SXj>034a<@005`1f
MWH0Lg=c03Lg=`11@D40MGEe03hn?P1eMGD0?clo028R8P1JFUX0L71`02d];@0o?cl0YZJV05aLG01Q
HF40IVIV08R8R00g=cL0ATI60:j^[P1hN7P0?3`l00kooooo00D0VYZJ07YjNP0k>c/0:RXZ028R8P02
01lO7`0;01dM7@0L71`0>CTi0410@00o?cl0AdM704A4A01EEED0T92@0861P@0M7Ad00P0E5AD04`0D
51@05ADE01DE5@0F5QH0P82004e=C@0j>SX0<c<c03Xj>P0I6AT0?Shn05aLG00O7al06ATI010@400>
3Ph03`l?014A4@19BDT00P32`/802P3@d=00g]kN0;bl_00J6QX06ATI01`L701EEED0J6QX034a<@0H
61P201<C4`0o08J6QP13@d<0A4A406e]K@0h>3P09RHV06a/K00a<C40;Rh^05=CD`0G5aL03`l?00d=
3@0j>SX0<S8b0410@02i^KT0_;bl0::RXP1dM7@0D51@03`l?00`<3008R8R01/K6`0G5aL04a<C018B
4P0E5AD0KVi^0<71`@35aLD0^;Rh0;Zj^P2GUiL0BDU903Lg=`0Z:RX0E5AD0:2PX03Fe]H0EeMG0410
@01;Bd/0D51@03De=@16ATH04Q8B01<C4`0D51@05ADE01<C4`0E5AD07QhN051@D01VIVH0Ph>305mO
G`0F5QH04Q8B01LG5`0E5AD051@D00<05ADE02/05aLG04];B`19BDT0QHF509FEU@1IFET0OGem0<;2
`P22PX80AdM702l_;`0^;Rh08R8R05]KF`1LG5`08B4Q0492@P11@D40GUiN05=CD`11@D40>SXj03Hf
=P1fMWH0fMWI0;:b/P1:BTX09RHV03lo?`18B4P03Ph>01DE5@0G5aL0Kfm_08J6QP3;bl/0OWin05IF
EP0i>CT05aLG01DE5@0A4A40410@00804A4A04T05ADE01@D500M7Ad0:b/[09VIV@3<c<`0V9RH0;^k
^`3Lg=`0De=C01DE5@0F5QH06ATI01lO7`0J6QX06ATI07]kN`21PH40<C4a02LW9`0M7Ad09bLW061P
H02?Shl0Ph>303`l?00J6QX0FUYJ0451@@0m?Cd0Fe]K05QHF01VIVH0T92@06MWI`1QHF40A4A40410
@016ATH0EeMG09NGU`3Kfm/0h>3P0>WYj@2j^[X0<c<c02PX:00h>3P0RHV90;no_`33`l<0QXJ601PH
600K6a/0QXJ60;6a/@0U9BD05ADE01TI6@0Y:BT09bLW03`l?010@400:2PX055AD@1]KFd0Rh^;0<?3
``3De=@0dM7A02/[:`0J6QX06ATI008071`L02L07AdM020P800O7al0?Shn07QhN01QHF40YZJV0<O7
a`2@T9008R8R02DU9@24Q8@0f=SH0=OGe`3:b/X0Jf][05UIF@1:BTX0B4Q80820P03=cLd0VYZJ09VI
V@2QXJ40Vi^K0;Zj^P3Bd]80PX:208R8R03Cdm<0_kno09RHV029RHT0Z:RX0:VYZ@0U9BD09RHV02<S
8`0R8R804?ooool000kooooo02h0`l?305IFEP26QXH0JVYZ0:f][@1JFUX0IfMW08>3P`1PH600JFUY
075aL@1]KFd0N7Qh065QH@2NWYh0>3Ph06IVIP0d=3@0Rh^;03Ph>00f=SH0GUiN08F5Q@1_Kfl0Ph>3
069RHP0M7Ad0D51@03Ph>023Ph<0BTY:03hn?P28R8P0BDU9059BDP1]KFd0CDe=01@D501OGel0CTi>
01dM7@0[:b/0>3Ph03`l?01XJ6P0;Bd]0P0h>3P0TP19BDT0IfMW02DU9@12@T80Ng]k02l_;`0i>CT0
H61P02l_;`1OGel0KVi^03Ti>@1RHV80P82002d];@1ADE40HF5Q05=CD`0f=SH0K6a/06ATI00m?Cd0
A4A402LW9`21PH40=CDe05=CD`2ATI40CTi>06EUI@1QHF40Bd];01lO7`1JFUX0B4Q804E5A@1aLG40
G5aL0492@P0N7Qh0GUiN05IFEP0S8b<0S8b<075aL@1>CTh0>3Ph04a<C01aLG40?clo06MWI`12@T80
MWIf0861P@17AdL0ADE5030`<01FEUH0H61P06][J`14A4@0CDe=05mOG`19BDT0=CDe07AdM01SHf<0
@D51069RHP1QHF40Ogmo05]KF`1=CDd0QhN705=CD`15ADD0Hf=S0:BTY01]KFd0E5AD05=CD`1MGEd0
Shn?04M7A`0i>CT0PX:204i>CP1UIFD0CDe=0410@01=CDd0IfMW02`/;00D51@0EEEE03dm?@1oOgl0
M7Ad04Q8B00];Bd0Hf=S04M7A`1JFUX0>3Ph06QXJ01EEED0A4A4071`L00V9RH0=CDe079bLP1PH600
DU9B03Hf=P2:RXX0@d=305eMG@2;Rh/0UIFE04E5A@1WIfL0Xj>S05eMG@0h>3P0_Kfm07UiN@1HF5P0
SXj>055AD@2:RXX0DU9B02d];@17AdL0DU9B03<c<`1_Kfl0D51@09BDU01jNWX0C4a<08Z:RP0_;bl0
9bLW04E5A@0n?Sh0[Zj^0P18B4P06`1dM7@0Hf=S06EUI@1BDU80ADE507YjNP1;Bd/0DE5A079bLP1C
De<0OWin02TY:@1]KFd0A4A402/[:`1oOgl0EUIF03Ti>@20P800OGem08F5Q@2=SHd0Ti>C07emO@0n
?Sh0TY:B03Ti>@0208^;R`0P06IVIP0^;Rh0HF5Q06QXJ00l?3`0MgMg05=CD`0n?Sh0ADE50492@P2Q
XJ40Rh^;03hn?P1dM7@0OWin07YjNP0c<c<0Ti>C061PH011@D40;Rh^051@D00c<c<0AdM70<K6aP17
AdL092@T03Hf=P1]KFd0?Cdm028R8P1DE5@20492@P0C069RHP0/;2`0LW9b05ADE00S8b<0DE5A06UY
J@0G5aL08b<S09ZJVP1EEED0P8200861P@28R8P0?Shn034a<@1mOGd0IfMW03De=@0>ooooo`0D075a
L@2;Rh/0EUIF02h^;P0R8R808b<S02@T900P820071`L034a<@19BDT0?Cdm03`l?02BTY80_;bl0=[J
fP2QXJ40<30`01TI6@0F5QH301DE5@0<034a<@21PH40A4A404=3@`11@D409BDU01/K6`1BDU80HF5Q
01/K6`0Y:BT04a<C0`0>3Ph0@@0C4a<0NGUi069RHP1KFe/0`/;208Z:RP0K6a/09bLW01<C4`0e=CD0
@D5101XJ6P0];Bd0<30`02PX:02;Rh/0E5AD03Hf=P17AdL0RHV90:n_[`2=SHd0WinO0:f][@1:BTX0
410@00h>3P0O7al0?clo03<c<`0e=CD0K6a/07YjNP1VIVH07QhN01TI6@0H61P04Q8B01lO7`0N7Qh0
6ATI01LG5`0E5AD06ATI08V9R@3Jf]X0Xj>S069RHP1[Jf/0a<C406IVIP15ADD0>SXj06a/K02][Jd0
X:2P059BDP1;Bd/0Cdm?05YJFP15ADD0@41001DE5@0C4a<05ADE00804a<C00T051@D01DE5@0k>c/0
L71`07IfMP1`L7009bLW01<C4`0G5aL0100E5AD0MP0G5aL05ADE04E5A@2ATI40H61P06=SH`2KVi/0
d];B0<;2`P1=CDd05QHF01LG5`0I6AT0=3@d04Q8B00g=cL05aLG01lO7`14A4@04Q8B02PX:01:BTX0
MGEe09>CT`2`/;00_;bl08B4Q022PX80VYZJ08>3P`1LG5`0BDU904e=C@1LG5`0Rh^;0:BTY02l_;`0
UiNG055AD@11@D405aLG01@D500?3`l04A4A018B4P0D51@05ADE01PH600e=CD0ATI60;6a/@3WinL0
];Bd08b<S02j^[X0IVIV02LW9`0F5QH07alO01TI6@0G5aL09BDU0:BTY00a<C40;Bd]0410@00S8b<0
8B4Q059BDP1CDe<0TI6A071`L00h>3P0Cdm?01hN7P0f=SH07QhN01HF5P0Y:BT0ATI601/K6`0M7Ad0
9RHV03dm?@1`L700WYjN0<W9b@3Hf=P0i>CT0>OWi`3Zj^X0gmoO0=3@d02o_kl0e=CD0=kNgP1oOgl0
IVIV01dM7@1CDe<0`L710;Bd]02TY:@0;2`/01DE5@0N7Qh0<30`034a<@1@D500GemO06m_K`1RHV80
GemO04a<C02>SXh0XZ:R03<c<`0R8R8201`L700?01hN7P0W9bL0=3@d028R8P0j>SX0OWin055AD@2M
WId0[:b/07EeM@16ATH0=CDe09ZJVP2i^KT0f][J0080]kNg00L0Y:BT09^KV`2UYJD0_;bl0;Rh^02U
YJD0Lg=c0080DE5A00h0Rh^;0<[:bP2d];@0a<C40<G5a@1EEED0FUYJ03<c<`1?Cdl0]KFe0451@@0Q
8B408R8R02HV9Q3ooooo000>ooooo`2E0<71`@1EEED0UiNG08b<S02m_Kd0FUYJ07QhN02JVYX0UIFE
07emO@2:RXX0JFUY07IfMP13@d<0Ph>304A4A01OGel0@T9209jNWP1DE5@0@d=305mOG`1CDe<0?3`l
051@D01KFe/07QhN04A4A00m?Cd0Jf][03lo?`0l?3`0QXJ605IFEP16ATH0?3`l03hn?P0T92@0IVIV
05UIF@0H61P0:b/[04E5A@0i>CT0BTY:024Q8@0^;Rh0;Bd]03@d=01@D5007QhN05ADE01jNWX0:2PX
05mOG`1YJFT0=3@d04Y:BP1JFUX0>SXj07moO`2CTi<0>c/k03Ti>@1>CTh0<S8b03De=@24Q8@0IfMW
04U9B@0i>CT09bLW05]KF`0c<c<0NGUi08R8R01=CDd0Vi^K08Z:RP1;Bd/08R8R06MWI`0g=cL0FUYJ
07AdM01ADE40=CDe02DU9@1YJFT0I6AT02<S8`2DU9@0G5aL03Lg=`0k>c/0CTi>05mOG`0k>c/0E5AD
04=3@`1MGEd0KFe]03lo?`0n?Sh0?Cdm05=CD`1DE5@0IVIV04e=C@1FEUH0E5AD02HV9P0b<S80LW9b
05MGE`12@T80Cdm?05QHF01fMWH0E5AD07IfMP1nOWh0GEeM06i^KP1YJFT0V9RH05ADE01?Cdl0>c/k
07QhN026QXH0<S8b03dm?@1mOGd0@d=306][J`0o?cl0ATI605aLG01ZJVX0>3Ph03<c<`1[Jf/0?3`l
08j>SP2<S8`0Cdm?03Ph>01VIVH00P1IFET08`0o?cl0K6a/04U9B@19BDT0NGUi04I6AP1IFET0VIVI
05IFEP19BDT0?Cdm096AT@1XJ6P0L71`08Z:RP2VYZH0E5AD09ZJVP2j^[X0M7Ad03Lg=`3;bl/0DU9B
08N7Q`2=SHd0IVIV08R8R01LG5`0;Rh^04E5A@1SHf<0ADE506YZJP13@d<0U9BD0080G5aL05X0NGUi
030`<00b<S80D51@04e=C@2n_[h0@D5105iNGP34a<@0clo?07]kN`16ATH0B4Q807QhN010@400D51@
07IfMP1GEeL0Q8B4038b<P1^KVh0@D5102PX:02>SXh0FUYJ055AD@1hN7P0Cdm?06m_K`1[Jf/0R8R8
0820P00m?Cd0Ti>C05UIF@1cLg<0L71`05aLG00/;2`0FEUI04Q8B00c<c<0N7Qh05]KF`0m?Cd0MGEe
055AD@32`/80RXZ:04E5A@1WIfL0Ng]k07inOP0e=CD0NWYj061PH00l?3`0;bl_0451@@0a<C40GEeM
0<k>cP1WIfL09RHV03De=@1?Cdl0D51@02/[:`1]KFd0I6AT0492@P1CDe<0<C4a05QHF00Z:RX06ATI
04m?C`1LG5`0>c/k03Ph>02@T900B4Q80820P01jNWX0NGUi04a<C01:BTX0TY:B07]kN`0d=3@>oooo
o`0@0<S8b022PX80Ng]k030`<00M7Ad0820P01lO7`0M7Ad06a/K02h^;P1SHf<0?clo08V9R@35aLD0
Z:RX0<C4a080YjNW00<0KFe]03/k>`0Q8B400P0E5AD01@1JFUX0DU9B05MGE`0m?Cd0BDU900807QhN
00@08B4Q02h^;P0_;bl0=3@d0P0E5AD200h>3P0I014A4@1ZJVX0B4Q802<S8`2MWId0MgMg03Hf=P0j
>SX04A4A03@d=00[:b/04A4A01TI6@0d=3@0FUYJ0<S8b02BTY80MgMg08F5Q@27QhL0MWIf092@T02]
[Jd0][Jf04m?C`0200d=3@0=04U9B@0Y:BT0;bl_04I6AP1ADE40EeMG08^;R`1BDU80Cdm?03lo?`0W
9bL0>c/k04a<C0020451@@0G02h^;P1UIFD0<C4a0:NWY`2LW9`0@T9203dm?@1KFe/0820P02@T900S
8b<06ATI02LW9`13@d<0Xj>S04e=C@1hN7P0OGem05YJFP1@D5006QXJ01DE5@0D51@00P0E5AD02P0D
51@05ADE024Q8@2VYZH0PX:20:BTY01>CTh0820P01`L700G5aL301DE5@0=01XJ6P0O7al0NGUi05=C
D`1IFET09RHV06ATI02=SHd0Shn?02/[:`0G5aL061PH02d];@0204i>CP0>01TI6@0E5AD05QHF0492
@P0];Bd04a<C010@400C4a<0:BTY0:RXZ02m_Kd0dM7A0;bl_01LG5`201/K6`0?01dM7@0T92@0;bl_
030`<01YJFT0^[Zj08^;R`1kNg/0IVIV051@D00L71`03Ph>01<C4`0E5AD0:BTY0080<c<c05`0Cdm?
08:2PP39bLT0Zj^[06IVIP1JFUX0KFe]05]KF`17AdL05aLG01DE5@0G5aL061PH03Lg=`1cLg<06QXJ
024Q8@1BDU80=SHf02HV9P1@D500K6a/07emO@2CTi<0NGUi030`<00d=3@05aLG01DE5@0F5QH0DU9B
024Q8@0M7Ad06a/K01PH600K6a/07QhN059BDP1=CDd0d=3@0>OWi`3Kfm/0c<c<0:b/[037alL0g=cL
0>CTi03@d=00=3@d05EEE@1XJ6P0`/;205UIF@0Q8B40EeMG08R8R00_;bl06a/K038b<P0l?3`0DE5A
0820P01WIfL0HF5Q05iNGP1?Cdl0IfMW0:NWY`2g]kL0S8b<06a/K00i>CT0:RXZ0451@@1_Kfl0EeMG
05mOG`2SXj<0V9RH0<k>cP1VIVH0>3Ph065QH@2KVi/0QhN707IfMP2TY:@0EeMG04E5A@1=CDd0BTY:
05aLG080^KVi0100SXj>09jNWP2ZZZX0[Jf]0<g=c@2ATI40U9BD0;fm_@2^[Zh0>c/k069RHP1DE5@0
@41008j>SP27QhL08b<S0P0U9BD@ooooo`003_ooool0602g]kL0FEUI0:f][@23Ph<0XJ6Q05=CD`1i
NGT0ZZZZ08F5Q@1dM7@0IfMW07emO@20P800E5AD09bLW018B4P0Kfm_04E5A@2JVYX0DU9B05ADE01h
N7P0T92@05=CD`80FUYJ0?`07QhN05IFEP1CDe<0JVYZ03De=@0l?3`0W9bL06ATI018B4P0@D5103lo
?`0];Bd0JVYZ04A4A00T92@0ADE505MGE`0h>3P0Cdm?02PX:00m?Cd0@410038b<P13@d<09BDU071`
L02HV9P0=3@d03Hf=P1SHf<0<C4a04i>CP1EEED0:b/[06QXJ01lO7`0Bd];04I6AP1KFe/0EeMG04m?
C`1nOWh0FUYJ04];B`0m?Cd0=cLg06ATI00S8b<0N7Qh09FEU@15ADD0HV9R06][J`1<C4`0:2PX07al
O014A4@0@d=307AdM013@d<0:RXZ02HV9P1>CTh0GemO02DU9@2MWId0DE5A02l_;`13@d<0C4a<05mO
G`0o?cl0J6QX03lo?`1RHV80KVi^03lo?`10@400?clo055AD@1@D500FEUI05aLG01DE5@0EEEE02d]
;@10@400JVYZ04Y:BP0n?Sh0H61P05MGE`1iNGT0Cdm?0820P01ZJVX0J6QX071`L01nOWh0P82004a<
C01PH600DU9B08>3P`2=SHd0?clo051@D02EUID0FEUI069RHP14A4@0AdM705EEE@1WIfL0:b/[02TY
:@1`L700CTi>06UYJ@1jNWX0A4A403Ph>01VIVH0MgMg07alO014A4@0M7Ad03/k>`12@T80V9RH05IF
EP1^KVh0VIVI05EEE@0^;Rh0BDU90;2`/01RHV80PH6108b<S02EUID0ATI60:>SX`2?Shl0D51@03hn
?P34a<@0IfMW06UYJ@1kNg/0GemO07UiN@1BDU80Bd];06EUI@1YJFT0CDe=07EeM@1NGUh0_Kfm08B4
Q01]KFd0T92@02DU9@1>CTh0ATI604M7A`2GUiL0:b/[03dm?@1UIFD0KVi^06EUI@18B4P0AdM706YZ
JP0l?3`0?Shn08>3P`2<S8`0P82003Lg=`1gMgL0=cLg03hn?P2?Shl0@d=304A4A01LG5`0IFEU04Q8
B01@D500Kfm_06][J`13@d<0FUYJ06UYJ@2;Rh/0Ng]k06][J`0/;2`0EUIF061PH00a<C40K6a/06EU
I@0^;Rh0De=C04U9B@2b/[80Ph>303Ti>@1ADE40EeMG08V9R@0a<C40MWIf06][J`0l?3`09bLW04A4
A018B4P0Jf][0<k>cP1`L7007AdM03@d=024Q8@0I6AT03Hf=P1WIfL0B4Q803`l?015ADD0:b/[065Q
H@16ATH0<30`04U9B@1jNWX0N7Qh03/k>`29RHT0@d=308n?S`22PX80Ng]k03`l?01KFe/0/[:b07in
OP0a<C4>ooooo`0C0:VYZ@2_[jl0WinO061PH00Y:BT08b<S01lO7`0M7Ad071`L03Hf=P2f][H0Q8B4
05mOG`0n?Sh0:RXZ02d];@0k>c/0EeMG05YJFP0206m_K`0>07QhN02:RXX0X:2P04Y:BP1ADE40Bd];
03Hf=P0X:2P071`L014A4@0F5QH0:2PX04M7A`0R8R82010@400f00l?3`0E5AD0QXJ6051@D00L71`0
TI6A0<O7a`2h^;P0L71`05MGE`1EEED0IVIV03/k>`0_;bl0>SXj03Ph>01PH600MWIf08F5Q@2>SXh0
Xj>S09FEU@2CTi<0WIfM07AdM00`<30071`L04e=C@1IFET0=cLg04M7A`26QXH0>SXj04];B`0A4A40
3`l?01<C4`0M7Ad092@T03<c<`1@D500FEUI0:2PX02OWil0;2`/02@T901<C4`0Rh^;08b<S01`L700
ADE5030`<00^;Rh0:RXZ0P0/;2`02@12@T80OWin06e]K@1oOgl0S8b<092@T02UYJD0@d=302DU9@02
01/K6`0N01dM7@0Q8B40>c/k04U9B@3<c<`0W9bL0;Rh^02NWYh0U9BD06ATI01_Kfl0Jf][04];B`16
ATH0CTi>069RHP2WYjL0PH6108f=S@1bLW80YJFU09^KV`1eMGD05aLG01DE5@0R8R80AdM70451@@0L
71`04a<C0P0E5AD03@0M7Ad0?Cdm02XZ:P0B4Q806QXJ02`/;02EUID0^k^k0:BTY02BTY80EeMG03<c
<`0l?3`00P0g=cL0300j>SX0D51@06MWI`2[Zj/0SXj>06a/K01XJ6P0N7Qh07emO@1:BTX0=CDe03`l
?080@D5101<0Cdm?04Y:BP1TI6@0^;Rh09:BTP1mOGd0Q8B4075aL@2[Zj/0EEEE024Q8@0I6AT061PH
01XJ6P1<C4`07QhN01DE5@0F5QH08b<S0080@D510240CDe=08^;R`2TY:@0IVIV09FEU@0V9RH05aLG
01HF5P0L71`0AdM706ATI00M7Ad06a/K01PH600K6a/06QXJ020P800c<c<0CDe=0:JVYP2YZJT0FEUI
09JFUP0e=CD0EeMG0:6QX@2EUID0d];B08F5Q@2VYZH0XJ6Q038b<P0A4A400P0E5AD09P0/;2`0E5AD
06IVIP1UIFD0EUIF04A4A00l?3`0C4a<05iNGP0m?Cd0IFEU08f=S@3=cLd0JFUY07IfMP27QhL0UYJF
0:ZZZP2YZJT0_;bl0:BTY01oOgl0c<c<0<g=c@2c/k<0<c<c01lO7`1@D500UYJF09ZJVP2LW9`0JFUY
04E5A@16ATH0Fe]K07inOP2IVIT0LW9b0P11@D404@0m?Cd0?Shn04i>CP1_Kfl0P82004U9B@1SHf<0
Rh^;028R8P15ADD0IFEU065QH@1GEeL0_Kfm08b<S01aLG40C4a<013ooooo000>ooooo`1Y0<K6aP1G
EeL0^;Rh09FEU@2i^KT0FEUI07alO02j^[X0PH6108:2PP1hN7P0Ogmo07MgM`1aLG40QhN703/k>`1V
IVH0Jf][09NGU`16ATH0=SHf06YZJP28R8P0BTY:069RHP0_;bl0?Shn092@T013@d<0FUYJ028R8P1;
Bd/0TY:B051@D00n?Sh0@d=304i>CP0S8b<0CTi>059BDP0H61P0=CDe03/k>`0f=SH0ATI601dM7@0f
=SH09bLW03/k>`0o?cl06QXJ06=SH`1cLg<092@T03De=@1PH600<C4a07IfMP1WIfL0<S8b07alO01k
Ng/0>3Ph04I6AP1CDe<0EEEE03dm?@1fMWH0FEUI05=CD`17AdL0CTi>06EUI@10@400O7al0:BTY010
@400LW9b06][J`1WIfL092@T075aL@0m?Cd0@41007QhN01ADE40:b/[02`/;01<C4`0K6a/04e=C@2>
SXh0C4a<03Lg=`17AdL0DE5A06MWI`19BDT0D51@02d];@1YJFT0KFe]038b<P0V9RH0;Rh^0080Fe]K
0:/0F5QH06YZJP15ADD0FEUI03`l?010@400De=C03Hf=P1GEeL0M7Ad065QH@1hN7P0>CTi08F5Q@20
P800H61P03lo?`1ZJVX0HF5Q04a<C01GEeL0:b/[08F5Q@1jNWX0=3@d04M7A`1nOWh0@D5105YJFP17
AdL0D51@04M7A`1MGEd07alO024Q8@20P800C4a<06EUI@1`L700>CTi03lo?`1VIVH0MWIf07inOP0o
?cl0L71`04Q8B01;Bd/0R8R805aLG01XJ6P0WIfM06UYJ@1FEUH0@d=30:NWY`1OGel0SHf=07=cL`2I
VIT0B4Q80;:b/P2ATI40Cdm?06][J`2l_;`0Kfm_08N7Q`1_Kfl0E5AD079bLP18B4P0=3@d04A4A01C
De<0=CDe0451@@1<C4`0[:b/07IfMP1JFUX0Q8B403<c<`1BDU80E5AD059BDP2<S8`0ATI6051@D01e
MGD0Q8B406QXJ011@D40ATI605QHF00o?cl0@41007emO@21PH40Jf][02LW9`1SHf<0A4A40410@02B
TY80ADE504a<C01MGEd0HF5Q04i>CP1:BTX0CDe=05EEE@1JFUX0ATI606i^KP2XZ:P0Ogmo05iNGP0S
8b<0@d=304i>CP0];Bd0JVYZ04A4A00T92@0Fe]K05iNGP32`/80WIfM0410@00b<S80<C4a08B4Q00f
=SH0NWYj06=SH`0d=3@0:BTY04Q8B014A4@0Jf][0<W9b@1`L70092@T04M7A`2=SHd0KFe]04Y:BP1b
LW80Jf][04=3@`1DE5@0FEUI06IVIP1LG5`0>c/k04m?C`2ATI40M7Ad03Xj>P1mOGd0>3Ph08:2PP25
QHD0Ogmo06][J`1iNGT0][Jf03lo?`0X:2P03_ooool07@2CTi<0H61P0;>c/`2QXJ40B4Q803<c<`0U
9BD0;2`/04Y:BP1`L700WYjN07UiN@0[:b/09BDU02d];@0V9RH06a/K05=CD`0k>c/09bLW02<S8`0Q
8B40?Cdm0;:b/P1;Bd/0E5AD05UIF@0e=CD08B4Q00805ADE00/07alO02LW9`0];Bd05aLG02/[:`0E
5AD061PH05iNGP3>c/h0XJ6Q04i>CP020<G5a@0704E5A@0B4Q8051@D01TI6@0Y:BT04Q8B00X:2P03
00d=3@0D04M7A`1ADE40TY:B05=CD`21PH40VIVI05ADE013@d<0MgMg07AdM02BTY80`<300:NWY`37
alL0`l?309VIV@0V9RH0410@00h>3P0@410200h>3P0;018B4P0a<C408R8R02`/;01mOGd09bLW02h^
;P0h>3P0>c/k01`L700Q8B400P0>3Ph4014A4@0Z01<C4`14A4@0HF5Q04=3@`2;Rh/0_[jn0<S8b02I
VIT0Bd];04U9B@1IFET0EUIF05ADE01EEED0F5QH0;6a/@24Q8@0X:2P0<;2`P2k^k/0WinO07moO`26
QXH0FEUI0492@P11@D40B4Q80:n_[`2b/[80UIFE07]kN`1MGEd0[Zj^08R8R01QHF408b<S03Ti>@16
ATH06a/K01DE5@0C4a<051@D0P0E5AD0500O7al0C4a<07=cL`14A4@0;bl_07YjNP3WinL0Ti>C0:FU
Y@1gMgL0BDU906IVIP24Q8@0K6a/05ADE00e=CD0Bd];06ATI02WYjL0JVYZ0P1nOWh01@2OWil0UIFE
07emO@1?Cdl06a/K00805ADE028061PH07UiN@3?cll0_kno0<30`03Cdm<0h>3P0>CTi02g]kL0=3@d
01DE5@0G5aL0<S8b03De=@0C4a<05ADE01HF5P0I6AT07QhN06EUI@1`L700N7Qh0:j^[P1[Jf/0Lg=c
034a<@0d=3@0Cdm?04e=C@0[:b/0=3@d02DU9@0L71`06ATI0P0L71`02`11@D40ADE508R8R022PX80
De=C01hN7P1lO7`0<c<c02TY:@1MGEd0WYjN0080h>3P0200D51@05aLG00e=CD04a<C014A4@0C4a<0
5ADE01TI6@0Q8B40<30`061PH02HV9P0[:b/0:JVYP1iNGT0EEEE061PH02MWId0N7Qh0=?Cd`3=cLd0
/k>c092@T022PX80Ogmo06=SH`1OGel0UiNG0;Zj^P3IfMT0d];B09JFUP80R8R800d0a<C40=OGe`2b
/[80V9RH02l_;`0o?cl0NGUi08n?S`12@T80=3@d038b<P0W9bL0;2`/0080<30`00h0?Cdm07AdM014
A4@0HV9R09NGU`0];Bd0?clo06m_K`1[Jf/0Q8B409^KV`0o?cl0LG5a0;Bd]13ooooo000>ooooo`27
0:j^[P1WIfL0a<C406YZJP2SXj<0HF5Q08b<S02CTi<0A4A408F5Q@1NGUh0OWin05IFEP1iNGT0Y:BT
04Y:BP1UIFD0IVIV0:NWY`1UIFD0ATI6071`L01lO7`0E5AD07moO`1ADE4092@T04e=C@0`<300JVYZ
02DU9@0j>SX0TI6A03/k>`1;Bd/0D51@04E5A@0K6a/0C4a<05ADE00L71`0?3`l04Y:BP0h>3P0ATI6
01dM7@0k>c/09BDU03De=@11@D406ATI06][J`1XJ6P08b<S03De=@1ADE40<30`05aLG01TI6@0;bl_
0820P01fMWH0=3@d03dm?@14A4@0D51@0410@01hN7P0CTi>05MGE`0g=cL0<c<c06MWI`0c<c<0N7Qh
09VIV@18B4P0M7Ad06MWI`1BDU809RHV07QhN011@D40De=C07emO@0n?Sh09BDU03Xj>P1EEED0IFEU
04=3@`1WIfL0DE5A03lo?`1CDe<0FEUI0820P019BDT0EEEE04M7A`24Q8@0Ph>303/k>`0m?Cd0;Rh^
04e=C@1SHf<0C4a<071`L012@T80E5AD04U9B@13@d<0G5aL03Ti>@0l?3`0IFEU059BDP1jNWX0B4Q8
05MGE`1fMWH0GEeM06e]K@0n?Sh0Lg=c04m?C`1EEED0?Cdm08j>SP21PH40?Cdm04a<C01oOgl0Cdm?
0080GEeM04P0D51@04=3@`1_Kfl0?clo038b<P2KVi/0EeMG04i>CP1SHf<0>CTi05mOG`1ZJVX0OGem
05YJFP11@D40PH6105iNGP1;Bd/0QHF503Lg=`20P800b<S806][J`1@D500ATI60::RXP1PH600RHV9
09fMW@21PH40EEEE0;Rh^02IVIT0>CTi06ATI02e]KD0<C4a07inOP1lO7`0De=C08J6QP11@D40@410
05EEE@1ADE409bLW05mOG`1ADE40ZZZZ05UIF@1=CDd0LG5a02<S8`1<C4`0B4Q80410@01mOGd0<c<c
05=CD`1`L700NWYj05ADE01?Cdl0B4Q805]KF`18B4P0CTi>09:BTP1aLG40J6QX034a<@1QHF4204U9
B@1308N7Q`0i>CT0?Cdm05IFEP1QHF40BDU904i>CP1EEED0@D51061PH00k>c/0Jf][0;Rh^01iNGT0
CTi>02DU9@1NGUh0DU9B030`<01EEED0;Bd]02XZ:P0m?Cd0DE5A0<S8b02KVi/0ADE50492@P0Y:BT0
L71`02l_;`25QHD0DE5A02`/;00l?3`0GEeM02h^;P1IFET0_Kfm04i>CP0I6AT0@D5105]KF`13@d<0
GUiN07EeM@1NGUh0OGem08R8R01BDU80JVYZ04m?C`0f=SH0N7Qh0:ZZZP1mOGd0@d=309FEU@0o?cl0
Ng]k071`L02ATI40M7Ad08^;R`2m_Kd0PX:203Ti>@0>ooooo`0l075aL@1WIfL0SHf=0<30`02m_Kd0
RHV907inOP1WIfL0BDU901hN7P0O7al0IfMW06i^KP0_;bl09BDU02h^;P0G5aL0=cLg04M7A`0f=SH0
<c<c02h^;P0Y:BT0ZJVY08^;R`1KFe/0Ng]k01hN7P0E5AD04Q8B00h>3P0E5AD09BDU04I6AP0m?Cd0
QhN705mOG`1BDU80E5AD07YjNP2IVIT0^KVi0;no_`27QhL04Q8B014A4@0B4Q806ATI04Q8B00H61P0
3@d=00h>3P0=3@d03`l?034a<@0];Bd0QhN706UYJ@1IFET0GEeM0P0l?3`02`26QXH0DE5A04M7A`15
ADD0XZ:R09VIV@2^[Zh0Ph>305EEE@0l?3`05QHF00803Ph>00H03@d=014A4@10@400;2`/02/[:`1b
LW8203/k>`0404U9B@0[:b/06ATI02@T90803@d=05/03`l?00h>3P0>3Ph03@d=00`<300N7Qh0FUYJ
04=3@`1HF5P0_Kfm0<[:bP2h^;P0O7al06UYJ@1`L700HF5Q07UiN@1QHF40MGEe03dm?@0i>CT0O7al
06a/K01SHf<0O7al09^KV`2ZZZX0YJFU08R8R01WIfL0Cdm?05]KF`21PH40M7Ad0;6a/@15ADD0Zj^[
09>CT`2^[Zh0];Bd06a/K00U9BD03Ph>014A4@0>3Ph0410@018B4P0C4a<05ADE05UIF@1BDU8092@T
02XZ:P21PH40h^;R0=[JfP3=cLd0_[jn0:RXZ023Ph<0DU9B04i>CP1HF5P0@41002h^;P0n?Sh0a<C4
03Lg=`1EEED0`l?30:VYZ@3AdM40;Bd]01HF5P0E5AD05aLG01TI6@0T92@0LW9b0<S8b018B4P0RXZ:
0:RXZ021PH40Vi^K0:6QX@1hN7P061PH01dM7@1]KFd07QhN00805ADE02@05QHF01DE5@0Y:BT0<C4a
08B4Q01/K6`0^KVi09:BTP1nOWh0KVi^05UIF@0j>SX09RHV01dM7@0O7al0ADE5028R8P0J6QX071`L
02HV9P1;Bd/0OWin05mOG`20P800:2PX01hN7P1?Cdl0XJ6Q051@D02n_[h0h>3P0=[JfP3:b/X0820P
04i>CP1?Cdl201DE5@0]01<C4`0E5AD05aLG01dM7@0O7al0820P03De=@1XJ6P0^k^k0;Ng]`2FUYH0
_Kfm0=[JfP2][Jd0dM7A092@T01LG5`0?clo0410@01fMWH0_kno0<g=c@35aLD0][Jf0;^k^`3Ti>@0
h>3P0;Fe]@2?Shl0bl_;0=GEe@39bLT0HV9R02HV9P1GEeL0WinO038b<P0L71`07AdM02/[:`0b<S80
:b/[02l_;`0`<300?3`l0080F5QH00/0JVYZ09FEU@0g=cL0=cLg096AT@2TY:@0^;Rh0;jn_P20P800
R8R80;Jf]P0@ooooo`003_ooool0L`2KVi/0EUIF0;2`/01dM7@0Hf=S06QXJ02TY:@0]KFe06EUI@1c
Lg<0BTY:09NGU`1/K6`0KFe]079bLP1JFUX0JVYZ07]kN`2QXJ40KVi^04E5A@1jNWX0IFEU03Ph>01W
IfL0BDU903hn?P1QHF409bLW055AD@0[:b/0De=C0::RXP0];Bd0=3@d04I6AP0m?Cd06a/K055AD@0o
?cl0<C4a06ATI01LG5`0?clo05eMG@0Z:RX0FEUI03De=@12@T80F5QH01lO7`1hN7P0I6AT02HV9P1A
DE40GEeM02h^;P1OGel0BDU902d];@2DU9@0S8b<02d];@0i>CT0>c/k03dm?@0j>SX0QXJ605=CD`1?
Cdl09bLW03Ti>@1/K6`0>3Ph071`L02LW9`0@T9209RHV01kNg/0;bl_02<S8`25QHD0<C4a04I6AP1Y
JFT08R8R01dM7@0o?cl0GUiN07AdM01/K6`0WinO03`l?00e=CD0P820055AD@1OGel0?clo07alO00k
>c/0JFUY05YJFP0n?Sh0BDU904e=C@1IFET0ADE503dm?@1mOGd0A4A405eMG@0R8R80DE5A075aL@1?
Cdl00P0e=CD0X@12@T80TI6A05EEE@1XJ6P0HF5Q05MGE`1[Jf/0:b/[06QXJ011@D40De=C04=3@`2=
SHd0L71`03Ti>@1>CTh0QhN705ADE01mOGd0DE5A05QHF01>CTh0NGUi02/[:`0a<C40WIfM04U9B@2;
Rh/0SXj>03Hf=P11@D40FEUI08V9R@1FEUH0=SHf07]kN`0`<300@D5109BDU00k>c/0ADE506e]K@1>
CTh0<S8b05MGE`2j^[X0CDe=065QH@1dM7@0?Shn03Ti>@2SXj<0RXZ:0410@01jNWX0^;Rh06EUI@1O
Gel0KFe]04];B`1^KVh0>3Ph04i>CP1mOGd0EEEE03Hf=P1WIfL0GEeM0<71`@1VIVH0C4a<08:2PP0Q
8B40HV9R04E5A@0Y:BT0FUYJ02d];@1BDU80NWYj08f=S@1?Cdl08R8R04E5A@1]KFd0?clo04Y:BP2;
Rh/0L71`07]kN`0f=SH0UYJF05EEE@13@d<0U9BD055AD@0f=SH0QhN706EUI@1PH600MgMg03hn?P0S
8b<0?Cdm03<c<`1FEUH0a<C408Z:RP0h>3P0<c<c08B4Q01XJ6P0<30`06YZJP0o?cl0?3`l03hn?P1;
Bd/0alO708R8R00h>3P0RHV907YjNP1SHf<0:RXZ08n?S`0k>c/06a/K02LW9`1HF5P071`L05YJFP2j
^[X0Jf][03Lg=`1JFUX0Kfm_02DU9@1?Cdl0Jf][03dm?@15ADD0De=C02DU9@1ZJVX0CDe=02@T901d
M7@0Ogmo034a<@17AdL0[jn_04E5A@2>SXh0MgMg08N7Q`1IFET0OWin0;bl_027QhL0>SXj00kooooo
01@0G5aL079bLP1DE5@0];Bd0=kNgP3IfMT0N7Qh02PX:00L71`05QHF01DE5@0K6a/0G5aL061PH00b
<S80:2PX01hN7P0G5aL0ADE502/[:`809bLW00D0:2PX0861P@3>c/h0Xj>S06][J`0301DE5@0@01dM
7@0K6a/07alO03`l?00M7Ad0<30`09fMW@1eMGD0@T9202@T901XJ6P0bl_;0;bl_01cLg<0=CDe03Ti
>@80?clo05L0IFEU059BDP0f=SH09BDU01XJ6P17AdL0<C4a02<S8`1RHV80Zj^[0:n_[`0T92@0:RXZ
05QHF01JFUX0O7al07=cL`2>SXh0UYJF08^;R`20P800Ogmo02XZ:P0e=CD0CDe=03Hf=P0H61P0410@
01<C4`0J6QX0@d=303`l?024Q8@0EEEE04I6AP1GEeL071`L024Q8@0V9RH03Ph>010@400?3`l0410@
00h>3P0=3@d03Ph>02XZ:P1nOWh0Ng]k07EeM@39bLT0dM7A0<;2`P2RXZ80TY:B0;Jf]P1LG5`0H61P
069RHP0j>SX092@T04i>CP25QHD09BDU04A4A01BDU80EeMG08V9R@1JFUX0KVi^06YZJP0^;Rh0AdM7
0:6QX@2ZZZX0d];B08b<S02i^KT0Y:BT0:j^[P3AdM40SXj>01<C4`0@410051@D01DE5@0C4a<00P0D
51@08P0/;2`0=3@d04A4A00T92@0?clo0::RXP3Fe]H0]KFe08F5Q@3>c/h0RXZ:06IVIP27QhL0A4A4
03Ti>@1JFUX0H61P06i^KP0o?cl0SXj>06i^KP2DU9@0UYJF061PH01eMGD08B4Q01`L700K6a/0;2`/
05YJFP2IVIT0c<c<0;6a/@1PH60206QXJ00_051@D01QHF40W9bL03lo?`1nOWh0_kno02<S8`0E5AD0
5QHF01/K6`0E5AD0:b/[02PX:010@400HV9R09bLW010@400CDe=04m?C`0T92@07AdM01LG5`0I6AT0
7AdM08:2PP1NGUh0>SXj0492@P1JFUX0Rh^;05mOG`14A4@0Fe]K01/K6`0L71`0;Rh^0=?Cd`3Kfm/0
YjNW09NGU`1cLg<0UYJF02<S8`11@D40HV9R01HF5P0D51@00P0E5AD07@0C4a<05aLG01dM7@0T92@0
9BDU03`l?028R8P0`L7108R8R02GUiL0`l?307EeM@1:BTX0HV9R08J6QP1DE5@0>3Ph04=3@`1NGUh0
emOG08V9R@14A4@0SHf=0;Zj^P2RXZ80a/K60;:b/P3<c<`0f=SH0080`L7100<0Ogmo0820P00h>3P0
0P0L71`0100Q8B40;Rh^030`<00U9BD202/[:`0>0492@P1IFET0QhN706i^KP27QhL0FEUI030`<01W
IfL0/;2`0:FUY@3EeMD0fMWI0<[:bP3Cdm<@ooooo`003_ooool0<02i^KT0GUiN0<C4a01fMWH0U9BD
05EEE@1IFET0Y:BT05UIF@1fMWH0:b/[06m_K`1UIFD0OGem04A4A00m?Cd0FEUI07MgM`1OGel0G5aL
06m_K`2NWYh0UIFE04A4A01PH60071`L04=3@`1nOWh0@D51059BDP0I6AT0EEEE08^;R`0d=3@0BTY:
051@D00g=cL09RHV05EEE@0T92@05aLG03Hf=P12@T80=CDe04e=C@0F5QH0Bd];03`l?080;bl_03H0
6ATI06i^KP1cLg<09RHV03@d=01FEUH0<c<c07EeM@1;Bd/061PH04a<C01YJFT0ADE503hn?P1=CDd0
?Cdm03<c<`1PH600B4Q805=CD`0o?cl0>c/k069RHP0^;Rh0Rh^;0:FUY@0h>3P0FUYJ04m?C`0Q8B40
;Bd]08R8R015ADD0@D5107emO@0[:b/06a/K03De=@1<C4`0De=C09bLW037alL0J6QX0451@@1dM7@0
D51@04a<C00c<c<0K6a/0410@01eMGD0@T9203De=@10@40205ADE02/03dm?@0_;bl0Hf=S0410@01>
CTh071`L03lo?`1IFET0?clo02`/;00_;bl0>3Ph071`L012@T80F5QH04];B`1@D500IfMW02LW9`1N
GUh08B4Q06][J`1DE5@0Rh^;07=cL`0];Bd0PH610::RXP15ADD0DE5A01hN7P1@D500IfMW07emO@19
BDT0A4A409JFUP0^;Rh0FUYJ08F5Q@0k>c/0A4A404Q8B01iNGT0H61P03dm?@21PH40=3@d03dm?@1S
Hf<0BTY:065QH@2VYZH0EEEE034a<@0i>CT0TY:B06ATI022PX80N7Qh08^;R`1@D500[:b/08:2PP0i
>CT0XZ:R0;jn_P1JFUX0F5QH075aL@2KVi/0ZZZZ05eMG@0[:b/0?3`l06EUI@0Z:RX0@D5105MGE`2d
];@0NWYj04e=C@26QXH06ATI05ADE01=CDd0;bl_0492@P0^;Rh0?3`l08>3P`2KVi/0F5QH03Xj>P13
@d<0C4a<01lO7`1=CDd0[Jf]09jNWP1@D5008B4Q05QHF00o?cl0EEEE0:VYZ@12@T809bLW06IVIP1<
C4`0De=C06IVIP1HF5P0?clo05IFEP0];Bd0E5AD09FEU@21PH40Fe]K02TY:@1<C4`0ATI602`/;01V
IVH0>3Ph03@d=01NGUh0De=C0=3@d028R8P0;bl_051@D01BDU80BDU902XZ:P2MWId0F5QH030`<01C
De<0Q8B4055AD@1?Cdl0_;bl03dm?@0P8200IVIV061PH018B4P0Hf=S05ADE00j>SX0@T9205UIF@0Q
8B40EeMG02XZ:P0[:b/0XJ6Q08N7Q`19BDT0BTY:0::RXP1KFe/0d];B0;fm_@27QhL092@T05YJFP2/
[:`0NWYj03Lg=`kooooo00X0GEeM07QhN01RHV80W9bL0=cLg034a<@0>SXj020P800I6AT05QHF0P0E
5AD0400H61P0DE5A06YZJP0m?Cd09bLW01TI6@0Y:BT0BTY:01LG5`0H61P06ATI05eMG@2?Shl0];Bd
05EEE@0I6AT201DE5@3Q028R8P1:BTX0>c/k06][J`15ADD0?Cdm04U9B@2`/;00^[Zj08n?S`36a/H0
HF5Q0:>SX`2LW9`0>CTi02DU9@0T92@08R8R01DE5@0F5QH0:RXZ03lo?`1YJFT0OGem03Hf=P0G5aL0
CDe=0=oOg`3<c<`0EUIF02PX:00i>CT0BTY:0:ZZZP2g]kL0UYJF051@D00C4a<09BDU05eMG@1iNGT0
;Bd]020P800d=3@0EEEE03Hf=P0F5QH05ADE02<S8`1PH600]KFe09:BTP1lO7`0UiNG02d];@0T92@0
4a<C010@400>3Ph04A4A01<C4`0E5AD03Ph>00l?3`14A4@0QHF50;bl_01TI6@0MgMg09>CT`25QHD0
S8b<03Ti>@2CTi<0YjNW07=cL`1^KVh0RXZ:092@T03:b/X0MGEe01@D500C4a<08B4Q03Xj>P1CDe<0
LW9b03Xj>P1NGUh0SXj>09>CT`2n_[h0clo?0<?3``2IVIT0YZJV08R8R02h^;P0b/[:09fMW@0C4a<0
410@014A4@0C4a<051@D018B4P0U9BD0@41003Ti>@0];Bd0ATI606i^KP1[Jf/0WIfM0861P@2BTY80
Shn?0:n_[`24Q8@0KVi^07IfMP18B4P08B4Q01LG5`0J6QX05ADE02`/;02UYJD0`L710;fm_@1hN7P0
Q8B404e=C@0i>CT0C4a<0861P@2j^[X0WinO0:b/[02WYjL0aLG508n?S`1cLg<0JFUY069RHP25QHD0
gMgM0:FUY@2j^[X0UIFE03<c<`0I6AT07alO01hN7P0M7Ad08B4Q03Xj>P1^KVh0Lg=c02DU9@0g=cL0
:RXZ03@d=00Q8B4061PH01/K6`0_;bl0Cdm?03Ph>00c<c<0@D51079bLP1eMGD0De=C065QH@1JFUX0
71`L01dM7@0c<c<0^k^k06ATI00m?Cd0CTi>07inOP2@T900=cLg02h^;P1SHf<08B4Q01DE5@0C4a<0
51@D01<C4`0H61P07AdM028R8P0U9BD0?Cdm0;Bd]02CTi<0FUYJ08Z:RP1YJFT07AdM02l_;`0Y:BT0
>3Ph092@T02BTY80FUYJ09nOW`39bLT0AdM704Y:BP2HV9P0ADE504=3@`1VIVH0UYJF0=GEe@3Rh^80
/;2`0<;2`P3=cLd0WIfM008071`L01D07AdM02/[:`0];Bd0:BTY02DU9@0W9bL0:BTY04=3@`1^KVh0
IfMW06m_K`1iNGT0=3@d04Y:BP1QHF40TY:B08n?S`3GemL0YjNW07YjNP1`L7004?ooool000kooooo
01`0aLG5061PH02b/[80X:2P0:JVYP1XJ6P0FUYJ0:BTY01]KFd0^k^k06IVIP1lO7`0H61P07emO@2>
SXh0EEEE05eMG@26QXH0IVIV0410@00h>3P0O7al08Z:RP0k>c/0L71`04i>CP0o?cl0N7Qh0P1;Bd/0
j00O7al0ADE507moO`0Y:BT0FUYJ05]KF`13@d<06ATI04Y:BP18B4P07alO034a<@17AdL0=cLg04];
B`0S8b<0AdM703Xj>P0];Bd0C4a<01LG5`1cLg<0IFEU01LG5`15ADD0=SHf02LW9`1gMgL0KFe]02/[
:`26QXH0Q8B402l_;`1<C4`0KFe]06QXJ00o?cl0L71`0410@00b<S80CTi>03lo?`1eMGD09bLW04m?
C`2KVi/0BTY:07UiN@1_Kfl0@D5102@T901^KVh0ATI606][J`22PX80NGUi02l_;`12@T80ATI605=C
D`1PH600M7Ad04m?C`18B4P0Hf=S071`L01PH600@d=305eMG@0l?3`0OWin055AD@0`<300=SHf05AD
E01KFe/0DU9B04m?C`1[Jf/0@T92059BDP0c<c<0De=C06a/K015ADD0@4100492@P1@D500MWIf04I6
AP2UYJD0OGem05IFEP1JFUX0@d=305QHF00a<C40CTi>03Hf=P29RHT0Kfm_02LW9`1@D500T92@04Y:
BP1PH600>SXj06m_K`1@D500K6a/02LW9`1jNWX0UiNG05MGE`17AdL0MgMg04Q8B01bLW80Hf=S075a
L@1XJ6P0BTY:09>CT`1NGUh0FUYJ0861P@1UIFD0Fe]K0;Rh^01gMgL0K6a/059BDP2JVYX0L71`0:2P
X02OWil0VYZJ05EEE@2UYJD0RHV904a<C02][Jd0/[:b06EUI@1IFET0J6QX05aLG02DU9@0G5aL0492
@P1DE5@0Fe]K02`/;01GEeL0FEUI09jNWP1nOWh0Cdm?08f=S@0];Bd0I6AT04i>CP0];Bd0<30`04A4
A015ADD0F5QH06e]K@1GEeL0>3Ph04m?C`1:BTX0:RXZ03Ti>@1KFe/0Q8B406ATI00m?Cd0FUYJ01hN
7P1;Bd/0QXJ605ADE00h>3P0JFUY069RHP1?Cdl0DE5A069RHP16ATH0HV9R051@D01YJFT0Ti>C08b<
S013@d<0Bd];0;fm_@1OGel07AdM061PH01HF5P0;bl_0861P@0^;Rh0[:b/08j>SP1:BTX0Jf][092@
T01iNGT0<C4a092@T00n?Sh061PH04A4A01cLg<0<C4a04U9B@2i^KT0GemO028R8P0n?Sh0GUiN04=3
@`1bLW80C4a<01dM7@1]KFd0Ogmo04Y:BP1IFET201`L700>07]kN`1jNWX0DU9B06UYJ@2IVIT0:RXZ
07moO`2PX:00RXZ:03lo?`16ATH0WIfM08:2PP0f=SH>ooooo`1H05eMG@2:RXX0GEeM0<W9b@3MgMd0
dm?C09VIV@13@d<06a/K01TI6@0L71`08b<S02l_;`0U9BD0BDU906UYJ@0g=cL07QhN01`L701JFUX0
<C4a01PH600E5AD0F5QH03/k>`2c/k<0SHf=02d];@0a<C40?clo06UYJ@2ATI40VYZJ0:b/[02EUID0
E5AD04=3@`0f=SH0GEeM0=GEe@2m_Kd0CTi>06YZJP2KVi/0K6a/02LW9`0J6QX06ATI01HF5P0B4Q80
61PH01DE5@0H61P0GemO03Hf=P0L71`0P8200>CTi03Cdm<0Rh^;05QHF01IFET0VIVI092@T03;bl/0
?Shn02PX:00J6QX05ADE01hN7P1MGEd0RHV903lo?`0O7al0:b/[05QHF01CDe<07alO04a<C02^[Zh0
bl_;0;fm_@3De=@0`/;207emO@0g=cL0410@00h>3P80410@02X0=cLg05QHF00F5QH04a<C01`L702Q
XJ40g=cL0<30`02ATI40SXj>0:2PX02d];@0U9BD0:j^[P32`/80][Jf0;:b/P2?Shl0IfMW09bLW01e
MGD07alO01XJ6P0C4a<05ADE038b<P1TI6@0Ogmo05eMG@1SHf<0YjNW0<;2`P3:b/X0@D5101HF5P0J
6QX0;2`/08J6QP2YZJT0];Bd0451@@0C4a<2014A4@0C01<C4`0D51@0?Cdm02/[:`1;Bd/0EUIF06UY
J@0`<3006ATI04i>CP1WIfL0KFe]0;bl_03Cdm<0_;bl0:n_[`0h>3P08b<S01hN7P0201`L701[01/K
6`0N7Qh0OGem0:f][@2m_Kd0ZJVY07UiN@2IVIT0Jf][06m_K`2?Shl0XZ:R0<[:bP1jNWX07alO0451
@@2IVIT0c<c<0:n_[`2[Zj/0`l?30>3Ph03<c<`0DE5A05UIF@29RHT0KFe]02/[:`0m?Cd0;Rh^02LW
9`0l?3`0O7al06a/K00N7Qh0=3@d024Q8@1<C4`07QhN01LG5`0M7Ad0ADE502LW9`0Q8B409bLW04a<
C01nOWh0FUYJ096AT@2:RXX0KFe]01`L700N7Qh0>SXj09fMW@0Y:BT0A4A4065QH@28R8P0LW9b05MG
E`0P8200H61P03Hf=P0E5AD04a<C01DE5@0F5QH06a/K01lO7`0R8R80:b/[04Y:BP2CTi<0V9RH075a
L@2c/k<0AdM702@T900i>CT0@D5103Ph>00V9RH0Ng]k0<K6aP32`/80VYZJ08R8R02EUID0AdM703/k
>`14A4@0DU9B08N7Q`3EeMD0_;bl07UiN@2/[:`0dm?C08Z:RP0L71`07AdM020P800W9bL09BDU02LW
9`0Y:BT00P0X:2P03P11@D40Jf][065QH@0f=SH0OWin02h^;P2_[jl0Y:BT0;6a/@3=cLd0ZZZZ06EU
I@1YJFT0D51@4?ooool000kooooo0<X0WYjN05UIF@2WYjL0Lg=c08n?S`1ADE40De=C07UiN@1@D500
SXj>04a<C01SHf<0E5AD092@T01dM7@0EEEE04e=C@24Q8@0O7al05ADE00o?cl0Ph>305mOG`12@T80
LW9b0410@00n?Sh0Fe]K03lo?`1WIfL07QhN04A4A01]KFd0=SHf065QH@1KFe/0<30`02DU9@1ADE40
=SHf020P800^;Rh0<C4a04];B`1XJ6P0:2PX0492@P0a<C40BTY:05]KF`0I6AT0QXJ6055AD@0O7al0
KFe]05ADE00Y:BT0?3`l051@D00c<c<0V9RH08n?S`0o?cl0EEEE04Y:BP0k>c/0>3Ph07YjNP1HF5P0
<S8b02TY:@15ADD0NGUi04=3@`1^KVh0V9RH03dm?@1jNWX0L71`04e=C@0Z:RX0M7Ad02h^;P12@T80
GUiN03Ti>@0I6AT0BTY:04E5A@1TI6@0Ogmo08V9R@1BDU80?Shn04m?C`1EEED0GUiN0451@@13@d<0
=cLg09>CT`1]KFd0>c/k03Xj>P1;Bd/0N7Qh05iNGP16ATH0J6QX051@D01XJ6P0>CTi04I6AP1SHf<0
B4Q805=CD`1JFUX0>SXj06YZJP1:BTX0QhN707IfMP1CDe<0C4a<05UIF@1kNg/0>3Ph05EEE@13@d<0
OWin0861P@16ATH0I6AT09RHV01?Cdl0Rh^;05YJFP1/K6`0Fe]K07UiN@0m?Cd0N7Qh07AdM01QHF40
AdM7065QH@0c<c<0M7Ad05ADE01aLG40CDe=061PH02LW9`0GUiN06][J`1gMgL0ADE504m?C`2IVIT0
KVi^04e=C@1GEeL0PH6105mOG`2/[:`0X:2P08N7Q`1?Cdl0UiNG08n?S`11@D40g=cL0:ZZZP1KFe/0
GemO06m_K`2DU9@0LG5a03Ph>01<C4`0@41004U9B@15ADD0IVIV06MWI`2RXZ80HF5Q061PH02PX:00
:RXZ05UIF@0i>CT0;Rh^0492@P1;Bd/0B4Q805iNGP1RHV80C4a<03De=@1fMWH0JFUY0P1EEED0BP22
PX80QXJ606EUI@0h>3P0QhN703hn?P1CDe<0TI6A04Q8B019BDT0OGem06IVIP1JFUX0F5QH03Hf=P0P
82009RHV02<S8`1EEED0Ti>C06YZJP14A4@092@T07IfMP1ADE409BDU05UIF@0g=cL0<S8b071`L018
B4P0ZZZZ06MWI`0l?3`0Ph>308:2PP1cLg<0?3`l096AT@1NGUh092@T04];B`1TI6@0;bl_055AD@39
bLT0P820030`<01JFUX0Rh^;05eMG@1bLW80F5QH04];B`18B4P0C4a<03lo?`1PH600;bl_02TY:@2K
Vi/0UYJF05]KF`22PX80_;bl04M7A`2XZ:P0SHf=06m_K`13@d<0HV9R0<[:bP2CTi<0BTY:3_ooool0
2P2e]KD0b/[:0::RXP2c/k<0LW9b08j>SP2SXj<0XJ6Q06EUI@0_;bl201`L701601/K6`0O7al07alO
04=3@`1bLW80AdM704e=C@2FUYH0[Jf]05iNGP1QHF40Y:BT09BDU03=cLd0dM7A0:f][@1bLW80CTi>
0492@P1ADE40PX:207IfMP2UYJD0ATI6030`<00W9bL0@D510<K6aP2o_kl0N7Qh08J6QP0Z:RX0>3Ph
061PH01ADE408R8R01PH600C4a<05ADE01HF5P0R8R80H61P07MgM`25QHD0SHf=05QHF01>CTh0HF5Q
07]kN`35aLD0V9RH02d];@0X:2P0?Shn03dm?@0b<S80;Rh^01HF5P0B4Q809bLW06=SH`1mOGd0HV9R
04A4A01fMWH0]kNg08b<S02@T900Zj^[0;bl_080_kno01d0Ng]k07EeM@1OGel0D51@0451@@1IFET0
>3Ph02@T900I6AT092@T04];B`10@400EeMG0<30`03IfMT0f][J0=OGe`3Bd]80c/k>0:n_[`2;Rh/0
WYjN03@d=00R8R805aLG038b<P1PH600DE5A02<S8`0201/K6`2=01DE5@0T92@0EeMG0<[:bP2_[jl0
Q8B408J6QP2LW9`0BTY:02d];@0J6QX0<C4a059BDP1FEUH0WinO05=CD`1ADE40=3@d01PH600E5AD0
>SXj04A4A013@d<0Bd];038b<P0K6a/05QHF01DE5@0G5aL0MgMg08V9R@2l_;`0SHf=0:VYZ@1UIFD0
CDe=04M7A`0`<300DE5A04i>CP1EEED0Bd];05EEE@1fMWH0Ogmo0<W9b@35aLD0NWYj03Hf=P24Q8@0
[Zj^0<[:bP31`L40G5aL01`L700I6AT08R8R04Y:BP1dM7@0Z:RX0<G5a@2d];@0UYJF06m_K`0n?Sh0
ADE5051@D01^KVh0PX:208>3P`2:RXX0YZJV0<g=c@2CTi<0;2`/02TY:@0m?Cd0FEUI059BDP0U9BD0
;bl_04E5A@0S8b<0>CTi05eMG@1eMGD0DU9B04E5A@2HV9P0/K6a06YZJP1FEUH0Bd];04i>CP1[Jf/0
CTi>04I6AP1VIVH0Ph>309RHV01[Jf/0820P05ADE01MGEd05QHF01<C4`0F5QH0820P028R8P0Q8B40
8R8R03Lg=`1jNWX0ATI60:VYZ@1_Kfl0<c<c030`<014A4@0HV9R08:2PP1[Jf/0De=C07AdM02;Rh/0
QhN7075aL@2SXj<0@d=303Ph>012@T80BTY:06=SH`2GUiL0@41004=3@`1?Cdl0JVYZ09BDU026QXH0
:2PX0080820P01@092@T02XZ:P0];Bd0;bl_03De=@11@D40NWYj06QXJ01GEeL0Fe]K0<71`@2UYJD0
^[Zj0<[:bP3Fe]H0dM7A06YZJP1kNg/0S8b<08B4Q13ooooo000>ooooo`2o0:f][@1YJFT0a/K6096A
T@2EUID0H61P07AdM02a/K40EeMG0:2PX01=CDd0Ph>304Y:BP2CTi<0MgMg05eMG@1TI6@0S8b<09ZJ
VP1<C4`0=CDe092@T02BTY80@D5108Z:RP0j>SX0@D51071`L01?Cdl0NGUi01`L701?Cdl0N7Qh03hn
?P1SHf<0EUIF03De=@0Z:RX0H61P03Ti>@0L71`0=3@d0492@P1:BTX0H61P034a<@0k>c/0?clo04Q8
B01PH6007AdM08^;R`1/K6`07QhN05aLG01aLG406a/K02h^;P1HF5P0820P05IFEP1UIFD0AdM705aL
G015ADD0FEUI03`l?016ATH0B4Q802d];@17AdL0@D5107moO`1<C4`0J6QX08V9R@0j>SX0JFUY05YJ
FP0h>3P0;bl_06][J`0/;2`0ATI606i^KP0R8R80=cLg07inOP26QXH0I6AT07moO`1jNWX0EeMG07Mg
M`2>SXh0L71`05YJFP1=CDd0De=C03Ti>@1mOGd0G5aL03Ti>@11@D40PX:207MgM`1;Bd/0?clo05aL
G019BDT0E5AD034a<@1:BTX0EeMG04=3@`1KFe/0I6AT05EEE@1VIVH0@D5108R8R01jNWX0HV9R04Y:
BP1dM7@0De=C05ADE022PX80CTi>0820P02:RXX0GUiN0:RXZ02YZJT0HV9R07moO`13@d<0Kfm_079b
LP1UIFD0<30`07YjNP20P800HV9R065QH@23Ph<0=SHf0820P012@T80HV9R06EUI@1HF5P0UiNG03Ph
>01XJ6P0MgMg05QHF01XJ6P0X:2P04I6AP13@d<0C4a<08^;R`24Q8@0Zj^[09NGU`1<C4`0K6a/0:^[
Z`2:RXX0CDe=0>7Qh@2][Jd0=SHf09VIV@22PX80/K6a09bLW01GEeL0AdM704A4A00m?Cd0Rh^;09NG
U`1FEUH0Vi^K04E5A@0o?cl0QXJ602<S8`1XJ6P00P0/;2`0E@14A4@0820P04a<C01fMWH0Hf=S03lo
?`0k>c/0Ogmo0410@00b<S80Fe]K07inOP27QhL0EUIF03lo?`1]KFd0@D5104A4A024Q8@0EeMG0492
@P22PX80GUiN05=CD`1JFUX0Cdm?04];B`0/;2`0;bl_05EEE@2_[jl0Ti>C06][J`0o?cl0XJ6Q05MG
E`0[:b/0C4a<04A4A00i>CT0@d=303Xj>P2f][H0RXZ:03lo?`1mOGd0NWYj06i^KP0i>CT0WinO04A4
A00Z:RX0DU9B05EEE@0K6a/0M7Ad0<?3``1_Kfl0>SXj0820P02YZJT0E5AD08N7Q`1XJ6P0Bd];051@
D01IFET08B4Q05mOG`1<C4`0;bl_0:b/[021PH40;Rh^07moO`2GUiL0>3Ph0:RXZ02/[:`0PH6104U9
B@11@D40]KFe0<G5a@1hN7P03_ooool0J`37alL0c<c<0=?Cd`2[Zj/0M7Ad04E5A@1=CDd0DE5A05iN
GP1oOgl0@T9202`/;00T92@08B4Q01`L700Q8B40PX:20<;2`P2RXZ80/;2`0<W9b@2b/[80Shn?07Ad
M023Ph<0KVi^0<;2`P2NWYh0^k^k08n?S`2KVi/0b<S80=GEe@3IfMT0[:b/05QHF01VIVH0QXJ608Z:
RP2IVIT0XZ:R0451@@1mOGd0C4a<00h>3P0G5aL0>3Ph07YjNP0_;bl05aLG01XJ6P0Q8B40G5aL0<W9
b@2TY:@0Jf][02h^;P0I6AT07AdM024Q8@15ADD0S8b<04];B`1ADE4061PH01DE5@0c<c<0BTY:05=C
D`10@400;Rh^01/K6`0E5AD0<S8b0;:b/P3IfMT0^;Rh0:BTY02][Jd0PH6105UIF@29RHT0c<c<07Yj
NP1UIFD0LW9b05eMG@14A4@0Ogmo08j>SP0a<C40820P04e=C@1EEED06a/K02<S8`1QHF40clo?0>3P
h03?cll0YZJV06UYJ@2ATI40LW9b02<S8`0m?Cd06a/K00<05ADE02h06ATI05QHF01CDe<0>CTi01`L
700F5QH05ADE01LG5`1hN7P0=CDe02LW9`0Z:RX0OGem08J6QP0o?cl071`L059BDP1oOgl0BTY:07in
OP2LW9`0ATI608B4Q021PH40DE5A04E5A@0n?Sh0>CTi04U9B@0J6QX06ATI01DE5@0G5aL05ADE03<c
<`3GemL0][Jf06UYJ@1^KVh0;Rh^024Q8@25QHD0RHV903Hf=P0M7Ad05QHF0P0E5AD0@00V9RH0JVYZ
09JFUP2OWil0gMgM0<;2`P2n_[h0_Kfm09nOW`3Bd]80Ogmo03lo?`0I6AT05aLG01`L7019BDT0J6QX
06IVIP1RHV80^[Zj0:j^[P1mOGd0DE5A05QHF01dM7@0UiNG0:BTY02ATI40UiNG0=3@d032`/80^k^k
0<;2`P3:b/X0`/;20;Vi^@2IVIT0VYZJ06IVIP1JFUX0Jf][06m_K`0h>3P0=CDe075aL@0b<S80820P
05eMG@1ADE40Z:RX05EEE@0H61P0>CTi01dM7@0d=3@0YZJV0<30`02CTi<0G5aL05eMG@2DU9@0<S8b
01/K6`12@T8205eMG@0f0451@@0c<c<0C4a<0:VYZ@1BDU80^;Rh05QHF00@41004a<C01DE5@21PH40
gMgM0<G5a@2XZ:P0E5AD02HV9P0K6a/0JVYZ09FEU@1SHf<0=cLg04];B`20P800][Jf05QHF00M7Ad0
=3@d05=CD`13@d<0Ph>308V9R@1QHF40:BTY01dM7@0e=CD0<c<c03Ph>00j>SX0D51@07emO@2e]KD0
Vi^K069RHP1oOgl0b/[:06i^KP2JVYX0c/k>0<S8b02WYjL0`l?3061PH00f=SH0@4104?ooool000ko
oooo0?l0U9BD08R8R034a<@0QhN705mOG`1aLG40TI6A09VIV@1;Bd/0L71`059BDP2c/k<0Bd];07]k
N`1OGel0WYjN06m_K`2/[:`0UiNG04M7A`12@T80W9bL08n?S`1:BTX0OGem03De=@1mOGd0QXJ605MG
E`1]KFd0:2PX07UiN@1^KVh0>c/k06=SH`1BDU80?Shn02/[:`13@d<09BDU03lo?`1hN7P0CDe=03lo
?`1HF5P071`L04m?C`0h>3P0BDU904Y:BP0b<S80Y:BT06UYJ@0K6a/0Cdm?05]KF`0c<c<0K6a/06UY
J@0h>3P0R8R807emO@1?Cdl0QhN70820P01[Jf/0=CDe03/k>`0l?3`0B4Q804];B`0l?3`0I6AT03Lg
=`2BTY80[Jf]059BDP1GEeL0DE5A02XZ:P1BDU80UYJF03lo?`1FEUH0QhN703<c<`0G5aL0CTi>04Q8
B01<C4`0S8b<0:VYZ@1>CTh0B4Q807QhN01fMWH0I6AT07alO01iNGT0@T9208n?S`16ATH0EEEE05=C
D`24Q8@0TI6A05=CD`0b<S80K6a/05QHF016ATH0>3Ph07alO01UIFD0@d=308b<S00h>3P0HV9R079b
LP18B4P0X:2P06YZJP1OGel0=SHf09RHV018B4P0<S8b05IFEP0e=CD0P82008J6QP0i>CT0K6a/09BD
U01CDe<0Vi^K02TY:@1=CDd0RHV906][J`0j>SX0WYjN07YjNP1DE5@0Vi^K08R8R00j>SX0Shn?065Q
H@2HV9P0Shn?059BDP2NWYh0AdM708Z:RP2b/[80JFUY071`L02SXj<0B4Q803Ph>01`L700W9bL08j>
SP2g]kL0[Jf]06UYJ@1BDU80U9BD0410@01HF5P0][Jf092@T00j>SX0HV9R06i^KP2o_kl0[Jf]04i>
CP1kNg/0N7Qh05aLG00i>CT0Ogmo05]KF`2FUYH0O7al07=cL`2ATI40>SXj0:BTY01CDe<0?clo05MG
E`0T92@0Lg=c0:VYZ@2BTY80FUYJ0410@02<S8`0?Cdm02DU9@1jNWX0`L710;6a/@10@400CTi>059B
DP0e=CD0EUIF07inOP1>CTh0HF5Q0:2PX01LG5`0MGEe08:2PP1OGel0Fe]K03De=@0`<300>CTi0:f]
[@2KVi/0NGUi065QH@2c/k<0JVYZ03/k>`1RHV80EEEE03Ti>@1>CTh0?Shn0;bl_02;Rh/0=SHf07in
OP23Ph<0K6a/03dm?@2VYZH0DU9B03Lg=`1bLW80Rh^;03Ti>@1?Cdl0_Kfm02`/;00J6QX0DU9B07in
OP19BDT001L0=3@d04M7A`0H61P0?clo08:2PP1=CDd0EeMG01LG5`0Q8B40Q8B408N7Q`1;Bd/0SHf=
08V9R@0S8b<0MGEe0820P01SHf<0<30`02d];@1EEED0YJFU03hn?P0>ooooo`060<[:bP3De=@0hN7Q
0;Bd]02VYZH0LG5a0`1?Cdl0O`1HF5P0MgMg08J6QP1WIfL0CTi>03dm?@11@D40WinO0:JVYP2<S8`0
TY:B0<K6aP1KFe/0<c<c01lO7`1gMgL0`<300<K6aP2`/;00YjNW0;Bd]01hN7P0H61P06m_K`2QXJ40
WIfM0:^[Z`1UIFD0A4A4024Q8@0R8R80RHV903/k>`0g=cL0De=C01lO7`0D51@05ADE05YJFP26QXH0
=3@d04=3@`2JVYX0/[:b096AT@1cLg<0E5AD034a<@0Y:BT09RHV03/k>`0S8b<09BDU05IFEP10@400
BTY:01/K6`0T92@0C4a<05EEE@0f=SH07alO030`<00Z:RX061PH05mOG`36a/H0WIfM0;Jf]P38b<P0
`L710;Rh^02<S8`0<C4a05mOG`1cLg<0E5AD06MWI`1hN7P0]kNg0:6QX@2XZ:P0WYjN06MWI`0E5AD0
?3`l075aL@1KFe/0?clo0;2`/01ADE40GemO02l_;`0/;2`0NGUi04E5A@0O7al0;2`/01LG5`0E5AD0
4a<C01DE5@0N7Qh0LG5a05iNGP0R8R807QhN01DE5@0M7Ad0O7al03Lg=`0Q8B408R8R06MWI`0c<c<0
W9bL03<c<`17AdL0[Zj^08R8R01cLg<0e=CD09NGU`2ATI40U9BD05YJFP1ADE400P0k>c/01`0c<c<0
5QHF01HF5P0E5AD05QHF01LG5`0^;Rh00P3IfMT01`24Q8@0QXJ603Ph>00Q8B40GUiN03<c<`0G5aL0
0`0E5AD0=P0G5aL0?clo055AD@1oOgl0OGem0>CTi02g]kL0QXJ605YJFP1?Cdl0P8200861P@22PX80
EEEE02PX:00L71`0<c<c06i^KP1FEUH0@41009:BTP3Jf]X0emOG0<71`@2c/k<0^[Zj0;bl_036a/H0
^;Rh0;:b/P3Kfm/0h>3P0:JVYP1jNWX0K6a/08:2PP2:RXX0LG5a09jNWP33`l<0cLg=08Z:RP1:BTX0
=SHf04i>CP0V9RH06a/K01TI6@0Z:RX0F5QH07YjNP1>CTh05QHF01/K6`807AdM01T0=SHf09JFUP2O
Wil0PH61075aL@2`/;00JVYZ03<c<`0h>3P092@T0451@@1gMgL0YZJV0;bl_02e]KD0X:2P0<O7a`0_
;bl051@D01DE5@0G5aL0@D510<K6aP2GUiL0?Shn00<06QXJ02D0:b/[09VIV@2MWId0NGUi0;2`/02]
[Jd0YjNW04=3@`0K6a/0:BTY061PH01?Cdl0HV9R09>CT`25QHD0BTY:03@d=02=SHd0N7Qh05EEE@1R
HV80WIfM09>CT`1DE5@0AdM706m_K`2UYJD0WIfM08^;R`2i^KT0WIfM07=cL`1HF5P0QXJ603Ti>@0k
>c/0@d=3013ooooo000>ooooo`3o0:JVYP1]KFd0S8b<09FEU@29RHT0Jf][06MWI`1iNGT0CTi>0:^[
Z`1JFUX0XJ6Q0451@@1mOGd0@T9207EeM@1QHF40^[Zj05iNGP1EEED0H61P07=cL`1OGel0DE5A06m_
K`0X:2P0E5AD05]KF`15ADD0IfMW020P801YJFT0EeMG04M7A`2?Shl0FEUI02l_;`0a<C40>3Ph034a
<@0R8R80A4A402PX:01HF5P0G5aL028R8P0o?cl0:2PX06UYJ@1GEeL06ATI07emO@14A4@061PH05AD
E012@T80;Bd]05ADE01VIVH0<C4a08V9R@23Ph<08R8R05aLG01>CTh0D51@0451@@1RHV80=SHf03@d
=00S8b<0BTY:05YJFP0e=CD0K6a/0:f][@1HF5P0SHf=06a/K00[:b/09bLW06e]K@0a<C40GEeM07mo
O`1HF5P06ATI06=SH`1CDe<0?clo051@D01WIfL0E5AD05UIF@1TI6@0B4Q8061PH01UIFD0BTY:020P
802<S8`0Hf=S03/k>`0E5AD0BDU906MWI`1[Jf/0C4a<065QH@0h>3P0L71`03lo?`1DE5@0@T9202<S
8`18B4P0GUiN05IFEP1CDe<0>CTi06a/K01^KVh0EeMG01`L702KVi/0KVi^0451@@1IFET0<C4a05mO
G`2DU9@0AdM706i^KP2?Shl0:2PX061PH01>CTh0G5aL06ATI01bLW80;Rh^06EUI@1MGEd0@D5105eM
G@0e=CD09bLW08j>SP15ADD0Hf=S03dm?@1ZJVX0YjNW04I6AP1VIVH0IFEU03De=@1hN7P0XZ:R08^;
R`1:BTX0Cdm?06UYJ@13@d<0IfMW08^;R`1ZJVX0G5aL09NGU`2;Rh/0Ogmo0:>SX`24Q8@0A4A407Ee
M@16ATH0Kfm_09:BTP1LG5`0=CDe05eMG@1IFET0DU9B04U9B@1SHf<0W9bL06MWI`1fMWH0Ng]k02XZ
:P21PH40GUiN05]KF`2IVIT0De=C04=3@`1_Kfl0IVIV05eMG@19BDT0[jn_06=SH`0j>SX0ATI605EE
E@25QHD0Fe]K05QHF01QHF408R8R061PH020P800B4Q806a/K02/[:`0ADE504I6AP1QHF40ADE505]K
F`1lO7`0IFEU065QH@2YZJT0Ogmo034a<@1cLg<0bLW906][J`0T92@0IFEU06][J`0m?Cd0Ogmo04I6
AP2g]kL0Ogmo04];B`2c/k<0aLG508F5Q@0];Bd0Shn?038b<P0G5aL0@T9205aLG00j>SX0H61P0<c<
c01SHf<08B4Q04U9B@1kNg/0Fe]K000G03Hf=P1GEeL0?3`l02/[:`1EEED0C4a<04m?C`0Y:BT07alO
08F5Q@2ATI40Fe]K092@T02ZZZX0?clo09jNWP2CTi<0Ng]k03Ti>@1>CTh0O7al08N7Q`19BDT03_oo
ool0K03MgMd0d=3@0<71`@2k^k/0^;Rh0;>c/`26QXH0O7al07YjNP1IFET0CTi>06=SH`2=SHd0ZJVY
08b<S02WYjL0b/[:07UiN@19BDT0WYjN0=[JfP1fMWH0?clo079bLP1:BTX0WinO0<W9b@34a<@0a/K6
0<;2`P2VYZH0VIVI08B4Q02FUYH0Z:RX09ZJVP1=CDd0;Bd]02DU9@0N7Qh0BTY:0:>SX`1aLG40B4Q8
06e]K@0W9bL05aLG069RHP3Bd]80[Jf]0=3@d02TY:@0QXJ605UIF@1dM7@0EEEE05mOG`1VIVH0MGEe
01dM7@0G5aL05QHF051@D00U9BD0ADE506IVIP0^;Rh0:2PX061PH01=CDd06QXJ01hN7P0f=SH0DE5A
06=SH`1oOgl0ADE506YZJP28R8P0cLg=051@D00<30`02PX:014A4@0i>CT0KFe]065QH@1EEED0/k>c
0<30`031`L40eMGE08:2PP2ATI40f=SH06=SH`0M7Ad06a/K07YjNP0K6a/0A4A404i>CP0T92@0BTY:
07AdM00Y:BT0;bl_01dM7@@05ADE02X0:RXZ07moO`10@40092@T01dM7@0h>3P0G5aL030`<00N7Qh0
8B4Q06i^KP0F5QH0YJFU0;Jf]P1aLG40C4a<0;Jf]P2JVYX0c<c<0=GEe@1XJ6P0NWYj0;Rh^01eMGD0
@T9207emO@0K6a/05aLG01DE5@0F5QH071`L04a<C02SXj<0inOW0>WYj@2h^;P0/;2`055AD@14A4@0
D51@02XZ:P0I6AT301DE5@0^01`L700T92@0@T92030`<02g]kL0emOG08j>SP0e=CD0?clo03dm?@12
@T80Fe]K03lo?`1OGel0MWIf06IVIP11@D40RXZ:05aLG01=CDd0G5aL0<O7a`36a/H0RXZ:07moO`1T
I6@0E5AD06m_K`1CDe<0Lg=c0=OGe`32`/80Ogmo06=SH`18B4P0BTY:0410@012@T80LW9b0;Fe]@3E
eMD0W9bL04a<C01SHf<0>c/k01XJ6P806ATI01L061PH04=3@`1/K6`0EeMG01DE5@0O7al07AdM01`L
700V9RH0=CDe08B4Q02][Jd0PX:20<71`@3Lg=`0Shn?06a/K01eMGD0PH6109:BTP2HV9P0TI6A079b
LP020:^[Z`0`01LG5`0C4a<04a<C01DE5@1;Bd/0EUIF03<c<`0E5AD05QHF01DE5@0J6QX092@T04Q8
B02XZ:P0LG5a03Hf=P0O7al0=3@d0410@00L71`0;Rh^08F5Q@2OWil0YjNW08b<S026QXH0MWIf04m?
C`2EUID0][Jf07]kN`1nOWh0MgMg05ADE01`L700QhN70;6a/@1nOWh0WIfM0:FUY@2=SHd0Z:RX08f=
S@1`L700PX:204I6AP0Z:RX0?clo4?ooool000kooooo0?l0Vi^K08j>SP2GUiL0Jf][08:2PP1bLW80
Hf=S0410@00c<c<0PX:205mOG`1YJFT0>CTi096AT@1EEED0L71`0492@P2>SXh0MgMg04=3@`0_;bl0
C4a<02/[:`0k>c/0MgMg02h^;P1RHV80;Rh^02@T901ADE40:BTY07AdM01:BTX09BDU04Y:BP1=CDd0
;Bd]038b<P14A4@09BDU028R8P1TI6@092@T02TY:@1ADE40=SHf03`l?00G5aL0=CDe051@D00P8200
YjNW02@T900B4Q80HF5Q071`L00[:b/0@T92034a<@0L71`0EeMG06IVIP0`<300J6QX03Ti>@0@4100
=CDe07QhN015ADD0@410010@4012@T80JFUY0451@@1/K6`0XJ6Q028R8P0n?Sh0Bd];038b<P0X:2P0
KVi^024Q8@0g=cL0Kfm_01`L700H61P0IFEU05EEE@1KFe/0F5QH069RHP1DE5@0H61P05QHF01UIFD0
ADE5075aL@1SHf<0=CDe07emO@1BDU80@41004i>CP1bLW80DU9B03<c<`0e=CD0LG5a04e=C@1SHf<0
?Cdm05ADE01oOgl0AdM704i>CP1ZJVX0IfMW07MgM`1?Cdl0QHF506QXJ01PH600>c/k0:BTY01SHf<0
>c/k06m_K`1jNWX0IfMW08J6QP0o?cl0LW9b0:6QX@1CDe<0LW9b05QHF01dM7@0FEUI08F5Q@17AdL0
LG5a06QXJ012@T80Fe]K04m?C`0X:2P0MGEe04Q8B01bLW80ATI603Ti>@1^KVh0Fe]K08J6QP1dM7@0
?3`l05aLG022PX80EUIF03@d=01<C4`0LW9b07moO`2;Rh/0WIfM06i^KP14A4@0[Zj^0820P02@T900
/K6a092@T010@400IfMW06QXJ01WIfL0PH6104];B`15ADD0Bd];02d];@1iNGT0Bd];05EEE@2?Shl0
=cLg05aLG01hN7P0:b/[09FEU@0[:b/0=CDe05mOG`0`<300ADE507moO`1OGel0=3@d038b<P1RHV80
I6AT061PH01ADE40MWIf09^KV`0g=cL0Bd];08R8R00k>c/0FEUI07YjNP0I6AT0@T9208f=S@1OGel0
Cdm?05UIF@0M7Ad0>3Ph03<c<`0o?cl0IVIV09nOW`1kNg/0DU9B04E5A@2e]KD0GUiN03Hf=P1WIfL0
9bLW04Y:BP2?Shl0BTY:0;:b/P1?Cdl06ATI051@D01aLG40LW9b03/k>`2BTY80@d=302<S8`1@D500
Fe]K01XJ6P1RHV80clo?07YjNP0X:2P0@d=308N7Q`1IFET001L0>c/k061PH01SHf<0;Rh^059BDP1P
H600EeMG034a<@0Q8B40QHF507]kN`12@T80U9BD0:RXZ010@400V9RH07moO`1JFUX0;Rh^05eMG@1g
MgL0AdM705EEE@0>ooooo`80e=CD02@0a/K609FEU@2h^;P0XZ:R06a/K00m?Cd0=SHf03dm?@1/K6`0
S8b<07QhN02][Jd0`l?30=OGe`3Cdm<0d];B0=GEe@2ATI40Ng]k0:j^[P2XZ:P0c<c<0::RXP31`L40
c/k>0;^k^`3@d=00`L710;Bd]029RHT0Kfm_08>3P`2<S8`0Ogmo05=CD`0g=cL202TY:@0N03Xj>P0i
>CT0;bl_04a<C01WIfL0W9bL09^KV`2MWId0JFUY06][J`2WYjL0W9bL08>3P`1fMWH0IVIV04m?C`0a
<C40>c/k04m?C`0d=3@07AdM01DE5@15ADD0@T9203@d=01VIVH0Ph>304A4A012@T80Jf][0P0N7Qh0
900d=3@0GEeM0:>SX`1TI6@0Ogmo07YjNP11@D40SXj>01TI6@0=3@d03Ph>00d=3@0>3Ph06QXJ061P
H027QhL0]KFe0<c<c03Cdm<0b/[:0;:b/P2QXJ40d=3@02@T900@41006ATI05ADE00M7Ad071`L03/k
>`0_;bl0:b/[06UYJ@1BDU80:BTY03Ph>0805QHF02d05ADE01<C4`0F5QH0=SHf08:2PP0`<300;2`/
061PH01=CDd09BDU01lO7`0_;bl0JFUY01PH601?Cdl06QXJ02l_;`11@D40CDe=0<S8b02n_[h0f][J
09nOW`1_Kfl0Ph>308R8R01jNWX0HV9R02XZ:P0M7Ad07alO02PX:01/K6`0^;Rh096AT@2l_;`0hN7Q
0<S8b02k^k/0S8b<07inOP0`<300820P02<S8`0E5AD00P0G5aL0;@0V9RH0<c<c03<c<`0Q8B40UiNG
0=?Cd`2QXJ40>c/k02TY:@0e=CD0?Cdm05MGE`1TI6@0Cdm?03Hf=P0n?Sh0RXZ:09ZJVP1XJ6P0O7al
09bLW03;bl/0DE5A02<S8`0i>CT0ADE505MGE`2MWId0];Bd0:j^[P2ZZZX0a/K608b<S029RHT0WYjN
08V9R@1mOGd0Vi^K09fMW@1OGel0/k>c0<K6aP2=SHd0G5aL01PH600201LG5`0H01DE5@0G5aL05aLG
04A4A01BDU805ADE01/K6`0V9RH07alO028R8P0X:2P0Hf=S0;bl_03:b/X0e]KF0=?Cd`2IVIT0HF5Q
04M7A`13@d<0EUIF07EeM@1iNGT0OGem0P2n_[h0200Z:RX05ADE01LG5`0U9BD0UiNG09>CT`0_;bl0
5aLG0`0K6a/06`0W9bL0D51@05mOG`0I6AT071`L01lO7`0Z:RX0?Shn020P801:BTX0B4Q807MgM`21
PH40Rh^;0:^[Z`2OWil0XZ:R0;bl_024Q8@0MgMg07moO`2GUiL0X:2P0:j^[P2e]KD0]kNg05MGE`02
0;Zj^P08096AT@2k^k/0VYZJ0;Jf]P2m_Kd0KVi^03De=@1dM7@@ooooo`003_ooool0o`2<S8`0LW9b
09^KV`20P800RHV909FEU@1gMgL0TY:B04Y:BP2CTi<0IVIV09bLW00j>SX0X:2P04];B`1GEeL0Hf=S
09:BTP2<S8`0C4a<034a<@2IVIT0NWYj0451@@2?Shl0;Rh^065QH@28R8P0CDe=065QH@0U9BD0P820
06EUI@13@d<0GEeM051@D00`<3009RHV0451@@0V9RH07alO04a<C00m?Cd0?clo055AD@0Y:BT0ATI6
02h^;P19BDT0C4a<024Q8@2UYJD0C4a<01HF5P19BDT0F5QH02`/;00X:2P0GUiN03Lg=`1[Jf/0FEUI
02DU9@1IFET0?3`l051@D00l?3`0BTY:03De=@0c<c<0820P04M7A`1?Cdl0AdM706a/K01hN7P0>CTi
0410@00m?Cd06ATI02/[:`1QHF409RHV03Xj>P1/K6`06a/K02HV9P29RHT0LW9b055AD@1QHF40FUYJ
04i>CP2>SXh0EeMG06][J`0_;bl0LG5a05iNGP0_;bl0Ph>304E5A@0o?cl0@T92061PH01ADE407alO
03Lg=`1fMWH0E5AD0492@P1;Bd/0Lg=c06EUI@0m?Cd0A4A403Lg=`1mOGd0IfMW04];B`1gMgL0B4Q8
05IFEP0Z:RX0Shn?03Ti>@1KFe/0QXJ605ADE01RHV80N7Qh05=CD`2OWil0VIVI04E5A@1aLG40@D51
08^;R`27QhL0MgMg04I6AP1NGUh0HF5Q079bLP2FUYH0EeMG02LW9`1/K6`0@D5106i^KP1<C4`0DE5A
09RHV01EEED0Shn?07IfMP1CDe<0Ogmo0;Fe]@19BDT0<S8b04Y:BP1hN7P0Q8B408j>SP2SXj<0H61P
04e=C@2VYZH0MWIf09BDU02XZ:P0Ph>304U9B@2QXJ40IVIV06i^KP2OWil0H61P0451@@1MGEd0@d=3
03De=@1JFUX0KFe]0:VYZ@12@T80CTi>079bLP0b<S80WIfM03De=@0o?cl0F5QH01XJ6P1:BTX0RHV9
05ADE00b<S80=CDe07MgM`0e=CD0=3@d04U9B@21PH40UiNG055AD@1@D500LW9b0492@P1RHV80UYJF
05QHF01NGUh0WYjN06UYJ@1?Cdl0GUiN04=3@`1CDe<0AdM7055AD@1OGel0/;2`09>CT`1DE5@0?Shn
0<C4a01DE5@0@d=305MGE`0o?cl0=3@d04Q8B014A4@0_[jn09>CT`17AdL0GemO071`L01kNg/0BDU9
07emO@12@T80:2PX04a<C01OGel08R8R06=SH`3?cll0KVi^03<c<`12@T80M7Ad05UIF@005010@400
IFEU05]KF`0e=CD0F5QH04Q8B01DE5@092@T01lO7`1eMGD0KFe]04A4A02DU9@0Y:BT0451@@2BTY80
IFEU0451@@12@T80@d=30P1dM7@00`19BDWoooooooooo`0<ooooo`1C0<G5a@2f][H0V9RH05QHF01M
GEd0/K6a0;:b/P25QHD0KVi^06MWI`1JFUX0EEEE06MWI`2=SHd0d];B0=?Cd`2`/;00[jn_0<[:bP10
@40071`L04];B`2n_[h0GEeM03lo?`18B4P0IVIV09VIV@2j^[X0a/K609FEU@1>CTh0=CDe02PX:00/
;2`0IVIV05eMG@1>CTh0Bd];04Q8B00e=CD0=3@d034a<@0M7Ad08B4Q08:2PP2m_Kd0RHV904m?C`1U
IFD0Bd];07QhN00i>CT0;2`/061PH01?Cdl08R8R01TI6@0k>c/0>SXj034a<@0E5AD0>CTi04e=C@1@
D500C4a<06ATI02[Zj/0Shn?09ZJVP0g=cL0<c<c09:BTP2m_Kd0[:b/075aL@38b<P0@d=30451@@1`
L7004Q8B00l?3`0=3@d00`0>3Ph0A@0B4Q80De=C0=GEe@3Cdm<0[jn_09NGU`2OWil0:2PX06e]K@0U
9BD04A4A01DE5@18B4P07AdM01XJ6P0H61P0A4A4024Q8@11@D40R8R804i>CP0b<S80;Rh^01HF5P0C
4a<05ADE01TI6@0G5aL0Kfm_05ADE00l?3`0SXj>03Xj>P0];Bd0<S8b03Xj>P1UIFD0;bl_01XJ6P0>
3Ph04a<C02LW9`0k>c/0]kNg0;Rh^01CDe<0P8200;Jf]P2?Shl0/K6a06e]K@1bLW80DU9B03lo?`12
@T80Ng]k09NGU`0g=cL06QXJ05ADE01ZJVX0bLW90;jn_P3Kfm/0TY:B02XZ:P0M7Ad0:BTY01lO7`02
01TI6@0T020P800l?3`06a/K01dM7@2?Shl0`/;20<W9b@2=SHd0=cLg024Q8@0[:b/0?Shn06=SH`1U
IFD0I6AT05YJFP1/K6`0NGUi0;fm_@32`/80[:b/0;bl_019BDT0ADE505=CD`1`L700[jn_0=?Cd`2i
^KT0Fe]K09:BTP39bLT0YjNW08n?S`2;Rh/0Ng]k0P27QhL0202c/k<0QXJ60<30`02n_[h0YjNW02PX
:00L71`04a<C0P0E5AD0:00G5aL04Q8B01HF5P1EEED09BDU01TI6@0`<30092@T02HV9P0Y:BT0L71`
0<C4a03Lg=`0alO70;6a/@1VIVH0DU9B05IFEP1EEED0A4A403Lg=`16ATH0[Jf]0=?Cd`2FUYH0TY:B
05=CD`1SHf<0Q8B40::RXP2o_kl0BTY:01HF5P0L71`06a/K01dM7@0T92@0@410055AD@0K6a/201dM
7@0O02LW9`1?Cdl0=SHf03lo?`17AdL0CDe=07QhN01dM7@0UYJF0<G5a@25QHD0_Kfm0;no_`2e]KD0
U9BD07emO@1MGEd0W9bL0<S8b02j^[X0V9RH0:6QX@1[Jf/0V9RH0;Zj^P0g=cL0B4Q804m?C`3:b/X0
Xj>S06i^KP0@ooooo`003_ooool0N02PX:00Q8B40;^k^`1eMGD0DE5A07IfMP2@T900P82004U9B@2;
Rh/0Lg=c0:>SX`0f=SH0SHf=04Q8B01cLg<0LG5a09BDU022PX80Fe]K05MGE`2=SHd0KVi^04M7A`1B
DU80<C4a07QhN01gMgL0@D5105MGE`0j>SX0U9BD05QHF01?Cdl0G5aL04m?C`0o?cl0;2`/03Hf=P0/
;2`0@D5107MgM`0h>3P0?3`l04e=C@0X:2P0B4Q8034a<@1HF5P0IfMW028R8P2ZZZX0DU9B01TI6@1?
Cdl0F5QH02DU9@0R8R80GemO03Ti>@1QHF40F5QH030`<01GEeL0AdM705MGE`15ADD0B4Q803Lg=`0h
>3P0;bl_04e=C@1PH600EEEE096AT@2JVYX0@d=303lo?`12@T807QhN04Y:BP1oOgl0:b/[03`l?025
QHD06ATI02XZ:P1?Cdl0EUIF04i>CP1RHV80EUIF03<c<`1OGel0EEEE06MWI`0/;2`0S8b<06IVIP0e
=CD0Ph>303De=@1DE5@0BDU905mOG`1:BTX0;Rh^03Ti>@1^KVh0Cdm?04I6AP0h>3P0EeMG06QXJ00j
>SX0ATI604];B`1TI6@0D51@03lo?`80IVIV09`0E5AD02PX:020P800>SXj04M7A`1>CTh0;Rh^055A
D@1MGEd0>CTi07moO`2:RXX0ADE507inOP0U9BD0HV9R06a/K019BDT0<c<c08j>SP1NGUh0BDU9071`
L01CDe<09bLW08R8R01OGel0Shn?06MWI`1NGUh0SXj>065QH@2ZZZX0Shn?06][J`1`L700/[:b06i^
KP0m?Cd0Cdm?08n?S`2:RXX0[:b/0:NWY`19BDT0Ng]k09VIV@13@d<0Xj>S0<W9b@1jNWX0<S8b08Z:
RP1KFe/0IVIV0:FUY@1TI6@0A4A4065QH@16ATH0Bd];06i^KP1]KFd0V9RH06IVIP25QHD0SXj>03@d
=02XZ:P0?clo0410@01VIVH0:b/[06][J`2d];@0QXJ60410@00g=cL0O7al02l_;`0e=CD0EeMG0861
P@2DU9@0?Shn06a/K01QHF40=SHf05MGE`2EUID0G5aL07alO02VYZH0IVIV06EUI@1cLg<0GemO06][
J`19BDT0E5AD071`L03;bl/0WIfM059BDP0n?Sh0a/K605eMG@18B4P0CTi>0451@@1DE5@0>SXj03`l
?035aLD0TI6A03Ti>@1RHV80K6a/079bLP0o?cl0MGEe0451@@0X:2P0BDU9075aL@0O7al0MgMg0;Zj
^P1QHF40>c/k05YJFP29RHT0BDU906QXJ01^KVh0EeMG02`/;01;Bd/071`L05iNGP0j>SX09BDU08V9
R@1;Bd/0>SXj0:BTY02GUiL0?Cdm0861P@1ZJVX0F5QH05mOG`0e=CD0S8b<0=7Ad@1@D50>ooooo`0H
0<30`02/[:`0UYJF05MGE`0h>3P0KVi^0=;BdP37alL0UIFE08Z:RP1mOGd0QHF507UiN@1WIfL0RXZ:
0=;BdP2VYZH0>3Ph06UYJ@1oOgl0CTi>03<c<`2KVi/0<S8b0P0L71`0>P0_;bl0@D5106QXJ02o_kl0
Ti>C04a<C00];Bd08R8R01hN7P0];Bd0L71`04M7A`0e=CD0<30`04i>CP1NGUh0?Cdm024Q8@1>CTh0
IFEU07UiN@0T92@0410@02PX:00b<S80G5aL05MGE`0[:b/0CTi>05MGE`0i>CT09BDU04A4A00U9BD0
?clo0451@@0l?3`0Fe]K05ADE01OGel0Jf][07moO`2l_;`0SXj>07emO@2YZJT0_;bl0492@P28R8P0
XJ6Q07inOP0c<c<0BTY:05mOG`0=3@d03Ph>00l?3`0@410201<C4`0J00h>3P0g=cL0Shn?05aLG00h
>3P0@41006a/K00J6QX0>3Ph02`/;00>3Ph04a<C04=3@`0T92@08B4Q010@4011@D407AdM02<S8`1U
IFD0NGUi04I6AP20P800=CDe01`L700G5aL201DE5@0:02`/;026QXH0Hf=S08>3P`0Z:RX0<S8b03dm
?@0o?cl0JFUY02`/;0803Ph>01406a/K01DE5@0E5AD0Ph>30:n_[`10@400>3Ph059BDP2[Zj/0f][J
0=3@d02YZJT0UYJF0<[:bP3Ng]h0UiNG02LW9`0201DE5@0j04=3@`0n?Sh0L71`0<30`03;bl/0b<S8
0492@P0O7al09BDU04M7A`0G5aL07AdM02`/;00_;bl05aLG04E5A@3>c/h0/;2`08j>SP2RXZ80UIFE
06QXJ00n?Sh0?3`l07IfMP2GUiL0VIVI09>CT`30`<00_[jn0;Bd]01TI6@0?clo07AdM00K6a/05ADE
01/K6`19BDT0]KFe07QhN01RHV80@T920:^[Z`2YZJT0aLG507]kN`1SHf<0KFe]04m?C`1<C4`0YJFU
0:NWY`3@d=00bLW907QhN01>CTh06a/K00l?3`805ADE04T06ATI01DE5@0C4a<0AdM705MGE`0J6QX0
:2PX02HV9P0g=cL0HV9R08Z:RP3De=@0h^;R0;>c/`29RHT0LG5a051@D014A4@0C4a<05EEE@1YJFT0
^;Rh0=_Kf`2a/K40EEEE06a/K032`/80Q8B403Hf=P0X:2P0XJ6Q06UYJ@0U9BD06ATI01`L700M7Ad0
:2PX0410@01?Cdl071`L01hN7P0P8200:BTY05YJFP0l?3`0ATI604];B`1HF5P0ADE506=SH`1kNg/0
d];B0=WIf@2e]KD0`<300;bl_02EUID0VIVI09FEU@3<c<`0/[:b0<C4a02o_kl0G5aL06IVIP1oOgl0
^[Zj051@D010@400D51@0;bl_00];Bd08R8R013ooooo000>ooooo`0`0:2PX027QhL0Shn?08n?S`1l
O7`0CTi>04];B`1HF5P0EEEE08V9R@1hN7P0L71`03hn?P27QhL0A4A408f=S@1QHF40V9RH05YJFP0o
?cl0>SXj06ATI01?Cdl0EeMG08V9R@0R8R80K6a/05]KF`0m?Cd0EEEE028R8P1gMgL0>3Ph04i>CP1/
K6`0ATI602`/;00m?Cd0>CTi01TI6@0];Bd0D51@028R8P1:BTX0F5QH02PX:00o?cl0:BTY0P1LG5`0
c`0];Bd0VIVI03Hf=P0W9bL0Hf=S0410@00^;Rh0CDe=05UIF@0c<c<0Jf][05mOG`0Y:BT0Hf=S04a<
C014A4@0B4Q805YJFP0^;Rh0>CTi030`<01`L700Fe]K04E5A@1[Jf/0PH6104I6AP18B4P0@T9201XJ
6P0i>CT0I6AT02h^;P15ADD0MgMg02/[:`0L71`0GUiN065QH@13@d<0O7al07]kN`18B4P0I6AT059B
DP1RHV80@D51071`L014A4@0<30`096AT@17AdL0C4a<03De=@1OGel0EUIF04i>CP1=CDd0IVIV05UI
F@1UIFD0A4A406ATI014A4@0>CTi06YZJP1>CTh0EeMG04M7A`13@d<0QXJ606UYJ@1KFe/0=CDe0:>S
X`1BDU80JVYZ05]KF`0g=cL0I6AT08J6QP17AdL0MWIf07MgM`0X:2P0QHF50451@@1nOWh0LW9b05eM
G@15ADD0B4Q8061PH01ADE40JFUY02XZ:P0R8R80Y:BT04U9B@1[Jf/0:b/[07EeM@2BTY80DU9B09ZJ
VP1`L700=CDe08:2PP2h^;P0JVYZ02l_;`13@d<0E5AD06UYJ@2ATI40QhN705UIF@1OGel0[Jf]055A
D@2d];@0FUYJ07QhN00f=SH0QXJ603De=@1eMGD0V9RH04M7A`1CDe<0GUiN03De=@1EEED0H61P08>3
P`2BTY80DE5A07UiN@23Ph<0=3@d09RHV00j>SX0CTi>08^;R`0/;2`0B4Q807YjNP1^KVh0B4Q803Lg
=`2ATI40?clo03Hf=P1nOWh0SXj>0:>SX`1=CDd0Fe]K05eMG@0^;Rh0I6AT06MWI`0o?cl0OWin0:j^
[P1BDU80L71`06ATI00g=cL0E5AD06YZJP1cLg<0JFUY08B4Q01SHf<0<30`04i>CP30`<00D51@05MG
E`1`L700BDU9051@D01IFET0?clo0;Jf]P1jNWX0?3`l06m_K`28R8P0Hf=S030`<01eMGD0=SHf02XZ
:P1LG5`0M7Ad02/[:`1KFe/0aLG503hn?P0o?cl0RXZ:0820P011@D40>3Ph051@D00202HV9P0C06AT
I01ADE40EUIF01/K6`0T92@0RHV907emO@17AdL0Vi^K06i^KP0_;bl0PX:208B4Q01MGEd0CDe=03/k
>`18B4P0NGUi03De=@0>ooooo`1A0<30`02o_kl0^;Rh04e=C@1GEeL0QXJ60;Ng]`23Ph<0?Shn02LW
9`0P82007QhN01`L700J6QX061PH07alO01XJ6P0IFEU05=CD`10@400Cdm?096AT@2m_Kd0?Shn01TI
6@0M7Ad0?Cdm034a<@0M7Ad0K6a/06EUI@27QhL0R8R807QhN01<C4`0=cLg06a/K02CTi<0OWin07Ui
N@22PX80SXj>07YjNP28R8P0ZZZZ06=SH`1ZJVX061PH01`L700T92@0EEEE05IFEP1oOgl0E5AD0410
@00S8b<0A4A404m?C`1BDU80De=C051@D01MGEd0E5AD06ATI01HF5P0DU9B061PH01EEED0CTi>03Ti
>@0N7Qh0FUYJ0:f][@24Q8@0[:b/05iNGP10@400L71`0861P@10@4004A4A00803Ph>0`0?3`l0G@0C
4a<0AdM7065QH@0h>3P0=CDe04A4A020P800D51@05IFEP1<C4`09BDU02LW9`1XJ6P0FUYJ0492@P0]
;Bd0HV9R06e]K@0i>CT0A4A40:2PX02[Zj/0PH6104a<C00g=cL0<S8b03@d=00a<C40ATI608J6QP2[
Zj/0U9BD09RHV01QHF40HV9R06a/K02TY:@0BTY:03`l?00m?Cd0BDU904Q8B01EEED0_Kfm09RHV00J
6QX061PH02LW9`1IFET0SXj>04Y:BP0h>3P0BTY:0:ZZZP1MGEd0:RXZ02HV9P0W9bL0;bl_07=cL`26
QXH0X:2P07MgM`3;bl/0d=3@08Z:RP1FEUH0=CDe030`<00b<S80=cLg02XZ:P0a<C40E5AD09fMW@37
alL0B4Q805iNGP2LW9`0V9RH07EeM@1hN7P0UYJF07MgM`1CDe<0>c/k03`l?01XJ6P0ZJVY08b<S00`
<300?Shn05EEE@0301DE5@0@02TY:@1YJFT0IVIV05mOG`1:BTX0_Kfm09FEU@1lO7`0SHf=0:2PX01X
J6P0FEUI05MGE`23Ph<0e]KF0:n_[`80O7al04h0NWYj04Q8B00E5AD051@D024Q8@0C4a<0410@014A
4@0/;2`0QXJ606YZJP0];Bd0>3Ph04e=C@2EUID0`l?30;Fe]@3Ogml0alO709VIV@1hN7P0IVIV05aL
G01HF5P0Fe]K0:b/[03Ng]h0][Jf08R8R02ZZZX0X:2P05aLG01dM7@0UIFE07IfMP1dM7@0JVYZ06=S
H`15ADD0:b/[01dM7@0W9bL0DE5A04E5A@0N7Qh092@T03Hf=P15ADD0M7Ad04Q8B01;Bd/0?Cdm02@T
900N7Qh0?Shn0:ZZZP35aLD0e=CD0;>c/`2KVi/0JVYZ05QHF01_Kfl0a<C40=KFeP2`/;00YZJV08J6
QP0/;2`0<c<c04I6AP1PH600T92@04m?C`1HF5P0KVi^024Q8@0Y:BT@ooooo`003_ooool0FP25QHD0
UiNG0:2PX01mOGd0MgMg06UYJ@1WIfL0CDe=04Q8B01aLG40Ph>305ADE00V9RH0UIFE04e=C@1UIFD0
>c/k09FEU@1]KFd0?Cdm03`l?01IFET0=3@d0492@P26QXH0:b/[06YZJP1lO7`0B4Q804U9B@0Q8B40
R8R805ADE014A4@0Cdm?04A4A00^;Rh0@d=3034a<@0L71`0=SHf051@D00_;bl0>SXj05YJFP0Y:BT0
ADE5034a<@1;Bd/0DE5A02@T902XZ:P0A4A402<S8`19BDT0DU9B034a<@1CDe<0<C4a03Hf=P2<S8`0
LG5a02/[:`1DE5@0:BTY024Q8@1JFUX0R8R803hn?P0l?3`06QXJ06][J`1aLG40HV9R096AT@1jNWX0
@D5106YZJP1FEUH0:BTY0492@P1/K6`0;2`/04U9B@1cLg<0<c<c02d];@1jNWX0MgMg059BDP80GEeM
02d0@41008j>SP1BDU80Fe]K024Q8@2DU9@0GemO03lo?`1fMWH0;Bd]0492@P0/;2`0Fe]K04A4A00J
6QX0ADE505aLG01FEUH0Fe]K04U9B@1lO7`0I6AT03`l?01>CTh0?Shn07QhN01GEeL0A4A407inOP1K
Fe/0JVYZ02d];@28R8P0@T9207AdM01FEUH08R8R05EEE@1aLG40DU9B096AT@27QhL0@41006UYJ@0]
;Bd00P26QXH0R`1YJFT0IFEU04i>CP1DE5@0OWin09ZJVP18B4P09BDU08f=S@12@T80GemO03dm?@1d
M7@0QXJ6059BDP2EUID0IfMW051@D01oOgl0^;Rh071`L00j>SX0@41005EEE@25QHD0Ph>307YjNP1T
I6@0Xj>S0;bl_01YJFT0clo?05iNGP1JFUX0ADE50:j^[P13@d<0D51@08Z:RP1;Bd/0A4A4059BDP0N
7Qh0IVIV04e=C@1`L700QXJ601XJ6P1RHV80PH61030`<02MWId08b<S03Hf=P1SHf<0:b/[05IFEP2`
/;00OWin02HV9P0R8R80Kfm_061PH01@D500QXJ60820P02][Jd0;bl_06MWI`2=SHd0=cLg05=CD`1a
LG409BDU05UIF@2YZJT0IfMW05UIF@1ADE40820P0410@01FEUH0Kfm_0861P@1hN7P0LW9b02d];@18
B4P0VIVI05IFEP1:BTX0EeMG01lO7`1>CTh0IFEU04U9B@2^[Zh0?Cdm01dM7@1/K6`0RHV908b<S01<
C4`0K6a/05QHF00Z:RX0BTY:08^;R`0a<C40I6AT0<[:bP1JFUX0;Rh^06][J`2MWId0GemO061PH01c
Lg<0GEeM02LW9`1RHV80ATI607AdM00k>c/0:BTY09VIV@1ADE40ADE50;Zj^P2<S8`09RHV04I6AP1G
EeL0K6a/06IVIP15ADD0CTi>028R8P0E5AD03_ooool02@33`l<0/;2`0:^[Z`1oOgl0OWin08f=S@1o
Ogl0;2`/02HV9P0201dM7@8071`L0P0M7Ad0G`0Z:RX0EUIF03Xj>P1JFUX0Hf=S03Xj>P13@d<0XZ:R
05eMG@0Q8B40:RXZ03Lg=`0G5aL05ADE04m?C`0K6a/0=cLg07AdM01nOWh0>CTi03Hf=P0];Bd0ADE5
06EUI@1ADE40BTY:04A4A01FEUH0SHf=08R8R02][Jd0SHf=06UYJ@0^;Rh0:b/[04U9B@1cLg<0U9BD
092@T01VIVH0?Shn01lO7`0L71`061PH01/K6`0k>c/0?clo04i>CP0[:b/0DE5A075aL@1^KVh0/;2`
0:BTY01aLG40Vi^K0=3@d02e]KD0_Kfm0<W9b@2VYZH0X:2P06a/K02e]KD0O7al065QH@1IFET0?3`l
02LW9`0G5aL061PH02LW9`2^[Zh0MWIf07]kN`1ADE40J6QX075aL@0n?Sh0D51@04m?C`0M7Ad061PH
05EEE@2;Rh/0=CDe038b<P0Z:RX0?Shn0::RXP2WYjL0Rh^;0:JVYP2[Zj/0@T920080820P04<092@T
024Q8@0W9bL0J6QX0=_Kf`3Lg=`0bLW90=_Kf`2:RXX0OGem07EeM@0n?Sh0ATI6038b<P0];Bd0ADE5
0;Rh^03Zj^X0/K6a071`L00D51@0410@01DE5@1WIfL0KVi^06a/K03<c<`0Cdm?01dM7@0R8R809RHV
028R8P0M7Ad0M7Ad08B4Q00f=SH0>c/k07YjNP3Ph>00[Zj^06a/K01kNg/0NWYj061PH00N7Qh0BTY:
07UiN@2FUYH0PH6109bLW012@T80?3`l04e=C@0Q8B40LG5a03<c<`0[:b/0RHV906MWI`1EEED0Fe]K
096AT@36a/H0NGUi0492@P1ADE408B4Q00805ADE00d051@D034a<@1ADE40R8R80:>SX`3AdM40/k>c
059BDP12@T80<C4a06m_K`0K6a/05aLG00805ADE01P0JVYZ0<?3``1lO7`0E5AD05IFEP1`L7006ATI
01LG5`0_;bl08b<S02d];@0F5QH071`L05iNGP39bLT0/k>c0=3@d03Cdm<0aLG50=;BdP2>SXh0e]KF
04U9B@0D51@201<C4`805ADE03H092@T0;6a/@2ZZZX0>3Ph06a/K01nOWh0QhN706m_K`15ADD0>3Ph
06][J`2_[jl0Y:BT06i^KP0o?cl0EeMG04Y:BP0b<S80P82003Ti>@0W9bL0@T9203hn?P12@T80LW9b
04m?C`16ATH07QhN01lO7`0P8200G5aL05eMG@2VYZH0bLW90;Rh^01cLg<0?Shn08R8R02TY:@0e]KF
0<?3``0k>c/0H61P065QH@0];Bd0>CTi03`l?017AdL0N7Qh08n?S`1QHF40=SHf03Ph>01LG5`@oooo
o`003_ooool0SP2;Rh/0P82009jNWP1YJFT0D51@08^;R`2TY:@0K6a/04i>CP1bLW80UIFE08N7Q`0[
:b/0SXj>04=3@`1eMGD0J6QX0:JVYP1]KFd0>SXj04A4A02@T900JVYZ05IFEP22PX80<S8b065QH@2<
S8`0FUYJ04e=C@0h>3P0WYjN05=CD`1:BTX0DE5A04U9B@10@400AdM7030`<00U9BD0D51@06YZJP0_
;bl0@41005=CD`0[:b/0EEEE030`<011@D40@T9203Ti>@2RXZ80F5QH02XZ:P1DE5@0<30`03@d=01Z
JVX0CDe=01lO7`1>CTh0IVIV03<c<`25QHD0CTi>02l_;`11@D40CDe=02TY:@1=CDd0;bl_065QH@1P
H600LG5a0::RXP1kNg/0ADE50410@012@T806QXJ05=CD`1]KFd09RHV04=3@`1VIVH05ADE02PX:01@
D500JFUY05EEE@1iNGT0FUYJ03@d=01LG5`0Bd];05MGE`0T92@0QhN7061PH00n?Sh0OWin02DU9@1H
F5P0>SXj05aLG01=CDd06ATI05IFEP1YJFT0FUYJ03Lg=`13@d<0MgMg05iNGP15ADD0@d=302TY:@1C
De<0EeMG0451@@1ZJVX0CDe=05ADE00f=SH0Rh^;038b<P1[Jf/0MWIf03lo?`1NGUh0@410055AD@2I
VIT0RHV903dm?@1YJFT08R8R06=SH`1UIFD0Jf][06EUI@15ADD204M7A`0S07=cL`0l?3`08B4Q08b<
S018B4P0PX:205UIF@22PX80Q8B404i>CP2/[:`0Vi^K06QXJ02RXZ80^k^k06UYJ@17AdL0IVIV06=S
H`2?Shl0Ogmo069RHP0n?Sh0L71`0:f][@1ZJVX0d];B0820P01YJFT0=SHf09RHV01;Bd/0E5AD08b<
S011@D400P1aLG40H@0^;Rh0BTY:05MGE`2KVi/0NGUi05aLG02FUYH0Lg=c03/k>`2PX:00@d=303@d
=01NGUh06ATI05UIF@2OWil0P8200492@P0^;Rh0JFUY03Ph>01BDU80SXj>08B4Q02WYjL0:BTY03dm
?@1OGel0:b/[05aLG01MGEd07alO05UIF@2CTi<0HF5Q08n?S`2>SXh0FEUI071`L01lO7`0Q8B407If
MP2k^k/0U9BD05=CD`1<C4`0[jn_05MGE`1ADE40ADE502@T9018B4P0?Shn0492@P31`L40Jf][038b
<P1?Cdl0L71`07inOP18B4P0Ng]k03lo?`0n?Sh0JFUY07YjNP0D51@0IVIV0:JVYP0n?Sh0;2`/069R
HP21PH40<C4a05YJFP1YJFT0IVIV03@d=01mOGd0=SHf05aLG00Y:BT0=3@d09FEU@1RHV80=cLg09VI
V@1_Kfl0@d=308N7Q`2?Shl0MgMg0451@@1^KVh0Z:RX075aL@0Z:RX03_ooool02@36a/H0U9BD06m_
K`2b/[80WYjN0:VYZ@15ADD0;Bd]02TY:@02020P800?01dM7@0N7Qh06a/K01/K6`0M7Ad0Bd];0492
@P13@d<0Lg=c05ADE00b<S80O7al07=cL`1gMgL0CDe=00806ATI07005aLG071`L00a<C40?clo06=S
H`14A4@071`L02<S8`0W9bL092@T02l_;`1PH600CTi>0492@P1_Kfl0RXZ:0:f][@32`/80TY:B0;6a
/@2LW9`0LG5a08N7Q`2n_[h0Rh^;08b<S01JFUX0;2`/01dM7@0L71`08B4Q03<c<`0[:b/0<C4a079b
LP0a<C40ADE50=KFeP3;bl/0`L710;Jf]P37alL0Z:RX06MWI`2=SHd0/;2`0;jn_P2FUYH0UIFE0<_;
b`30`<00Fe]K020P800R8R80?3`l05iNGP1QHF40JVYZ0:j^[P30`<00OGem08f=S@2JVYX0ZZZZ05UI
F@15ADD0TI6A07UiN@1QHF40B4Q808f=S@2_[jl0@d=304E5A@0/;2`0;Bd]07IfMP30`<00BTY:055A
D@3>c/h0YJFU065QH@1FEUH0DE5A05=CD`1HF5P0QXJ60=KFeP3<c<`0bl_;0<S8b02?Shl0^KVi07Qh
N02j^[X0S8b<096AT@2OWil0fMWI0=7Ad@3>c/h0Xj>S0:^[Z`1;Bd/04a<C01DE5@1QHF40hn?S0>7Q
h@1aLG40:BTY0P0L71`05`0V9RH09BDU01dM7@1hN7P0@d=301PH600W9bL0Jf][0<c<c03IfMT0TI6A
0:2PX02f][H0K6a/065QH@2XZ:P0K6a/02@T901OGel0Vi^K04M7A`1?Cdl08b<S00806ATI01X0H61P
03Xj>P1MGEd0UIFE096AT@1<C4`0XJ6Q0820P02EUID0P82007IfMP0O7al05QHF01DE5@0G5aL0De=C
06e]K@0k>c/0=SHf07UiN@1gMgL0=SHf04U9B@1@D500GEeM02/[:`805aLG05<092@T09fMW@2SXj<0
XZ:R08Z:RP1PH600FEUI01XJ6P0I6AT071`L02HV9P1ADE40820P02XZ:P24Q8@0^KVi0=GEe@3Hf=P0
Xj>S0;Zj^P3Lg=`0LW9b0<o?c`0[:b/04a<C01@D500C4a<05ADE01LG5`1=CDd0bLW906MWI`0d=3@0
C4a<02/[:`1eMGD0Ogmo05IFEP18B4P0>SXj08>3P`27QhL0Rh^;09JFUP2<S8`0N7Qh08Z:RP2NWYh0
>SXj04];B`16ATH0@T9204m?C`1XJ6P0HF5Q02h^;P0N7Qh08B4Q03dm?@1SHf<0Bd];0:^[Z`3MgMd0
eMGE0=KFeP3De=@0GUiN01/K6`1aLG40/k>c03De=@1:BTX0CTi>02/[:`0f=SH0?clo04=3@`1JFUX0
Vi^K07UiN@11@D40Lg=c07AdM00@ooooo`003_ooool0E`2EUID0Shn?0:NWY`2HV9P0N7Qh0820P025
QHD0Hf=S051@D01JFUX0UiNG07EeM@0Z:RX0MWIf04Q8B02/[:`0L71`09RHV01IFET0J6QX075aL@2>
SXh0HV9R04m?C`16ATH0ADE50861P@1dM7@0EeMG03Ti>@0Q8B40Ogmo03hn?P15ADD0@T9202/[:`0d
=3@0;bl_020P800G5aL0@T92055AD@0_;bl0?Shn0451@@0R8R80DE5A02PX:00j>SX0BTY:02HV9P2W
YjL0Cdm?02HV9P1:BTX0BDU903<c<`1IFET0Hf=S02l_;`1[Jf/0GEeM02`/;01@D500C4a<03Xj>P1>
CTh0E5AD01XJ6P0m?Cd0=CDe06ATI017AdL0@d=308B4Q022PX80AdM704a<C019BDT07QhN05YJFP1/
K6`0<c<c05eMG@1]KFd0=SHf02XZ:P02065QH@0O06YZJP2j^[X0LW9b04M7A`1ZJVX0@d=3075aL@13
@d<0RXZ:06ATI01GEeL0QXJ603`l?01;Bd/0?Cdm05eMG@1=CDd0>SXj05iNGP1VIVH0EeMG051@D00d
=3@0MWIf06QXJ00j>SX0Bd];03De=@1LG5`0BTY:03Hf=P0206][J`0P05QHF00];Bd0UIFE051@D01J
FUX0GUiN03dm?@1DE5@0Jf][03lo?`2<S8`0TI6A03lo?`1gMgL0D51@09:BTP1gMgL0N7Qh06YZJP1G
EeL0E5AD04A4A01dM7@0>3Ph024Q8@2<S8`0>3Ph07IfMP1;Bd/0NWYj07AdM015ADD20861P@1U065Q
H@2DU9@0]kNg0492@P0W9bL0K6a/06IVIP2`/;00_;bl096AT@1HF5P0UIFE09NGU`1KFe/0[Zj^092@
T01WIfL0ATI60:j^[P1:BTX0LG5a08n?S`0m?Cd0=CDe06=SH`0];Bd0Fe]K05ADE02LW9`0TI6A06EU
I@2i^KT0N7Qh03/k>`2TY:@0@41003Hf=P1SHf<0:b/[04];B`2HV9P0H61P04I6AP0i>CT0Ogmo0492
@P0c<c<0YjNW0;^k^`2YZJT0ADE5051@D01>CTh0:2PX07UiN@1/K6`0?clo05]KF`2PX:00ATI605aL
G01/K6`0;Bd]06QXJ02?Shl0QHF506EUI@1VIVH0LW9b02XZ:P11@D40Zj^[03lo?`1OGel0Lg=c04Q8
B028R8P0JFUY0410@031`L40OGem05=CD`2IVIT0YZJV05ADE00T92@0J6QX02XZ:P0Y:BT0GEeM06IV
IP0U9BD0GemO0:^[Z`0X:2P0@d=307QhN01RHV80@D5104M7A`1ADE400P0[:b/04`1aLG40F5QH07Yj
NP0l?3`09bLW08Z:RP1SHf<0B4Q80;Vi^@1hN7P08b<S04A4A01CDe<0@T9205IFEP19BDT0S8b<06UY
J@0^;Rh03_ooool0303Cdm<0aLG50<c<c030`<00_kno07UiN@15ADD0@d=302LW9`0P82008b<S02DU
9@808B4Q02h07QhN01XJ6P17AdL0@d=302d];@0n?Sh0MGEe02TY:@1HF5P0H61P05MGE`16ATH07alO
02LW9`16ATH0DE5A09>CT`2h^;P0^KVi09FEU@1QHF40EUIF0451@@0c<c<09RHV030`<01<C4`0IVIV
092@T02m_Kd0[jn_0<30`03:b/X0];Bd0<K6aP2KVi/0IfMW0;>c/`2NWYh0QHF5071`L01PH600:RXZ
03Hf=P0[:b/06ATI0P0E5AD0I`20P800^KVi0<[:bP3Cdm<0e]KF0<_;b`2b/[80PH61059BDP1;Bd/0
K6a/0<?3``35aLD0]KFe09JFUP1jNWX0MGEe08J6QP1@D50051@D01<C4`0E5AD06ATI02DU9@0_;bl0
NGUi0<K6aP2UYJD0][Jf0=?Cd`2IVIT0][Jf04e=C@0U9BD0=CDe03dm?@1RHV80XJ6Q05mOG`1RHV80
?3`l04U9B@20P800NWYj0:ZZZP1eMGD0a/K60;^k^`1kNg/0E5AD03/k>`0d=3@0:RXZ06a/K03Bd]80
^KVi07alO02LW9`0MGEe08b<S00J6QX0KVi^06ATI02WYjL0e=CD07YjNP0g=cL0Lg=c06=SH`29RHT0
^KVi03Xj>P0G5aL0M7Ad0=_Kf`2OWil0=cLg02<S8`0L71`06ATI01hN7P0M7Ad07alO08Z:RP0g=cL0
6QXJ01`L700c<c<0eMGE0>[ZjP3GemL0dM7A0=GEe@2l_;`0`<30069RHP0K6a/07QhN06=SH`2GUiL0
>CTi04Q8B00F5QH00P0E5AD0I@0N7Qh0HF5Q04Y:BP1ADE40MgMg0<W9b@3Cdm<0@41007YjNP2BTY80
UiNG069RHP0m?Cd08R8R01XJ6P1bLW80=3@d02TY:@0a<C40C4a<05mOG`0];Bd0>c/k07MgM`1=CDd0
BTY:01/K6`0N7Qh0GUiN04E5A@1:BTX0R8R808b<S02`/;00V9RH02`/;00O7al09RHV02XZ:P1;Bd/0
9bLW02h^;P1gMgL0fm_K0=?Cd`38b<P0gMgM0:f][@3:b/X0aLG50<[:bP0P820051@D01<C4`0D51@0
5ADE01`L702<S8`0XJ6Q02`/;00N7Qh0BTY:02h^;P0h>3P0Ng]k06ATI00i>CT0D51@09jNWP1/K6`0
ADE506ATI028R8P0[Jf]0<k>cP3EeMD0XZ:R07YjNP1ADE40EUIF06UYJ@1NGUh0Jf][02`/;00`<300
N7Qh08F5Q@0d=3@0T92@0;Zj^P3Jf]X0b<S80;^k^`2h^;P0OGem02<S8`1BDU80^[Zj03Lg=`18B4P0
@D510080:BTY00P0=SHf04E5A@1ADE40OWin09FEU@21PH40BTY:051@D13ooooo000>ooooo`1R09BD
U02EUID0MWIf09bLW01oOgl0VIVI0:2PX01TI6@0FUYJ08N7Q`2VYZH0QhN703Lg=`27QhL0@D5107Qh
N01YJFT0V9RH03lo?`16ATH0@D5107moO`1OGel0EeMG059BDP0g=cL0JFUY06YZJP1UIFD0AdM702LW
9`2IVIT0?clo06=SH`1OGel0=CDe034a<@0o?cl0<30`01dM7@1:BTX0Cdm?02/[:`1>CTh0EEEE02/[
:`1QHF40;Rh^04m?C`12@T809bLW0:FUY@1:BTX0;Bd]04];B`0m?Cd0<30`05ADE01NGUh0<S8b0820
P01IFET0;Rh^05=CD`10@400<30`04i>CP1DE5@0:b/[03dm?@0T92@0J6QX051@D01`L700WinO08B4
Q01DE5@0KVi^065QH@0R8R80Cdm?069RHP0Z:RX0AdM706YZJP0b<S80<30`06ATI01QHF40ATI60820
P01_Kfl0@d=307EeM@16ATH0FEUI04Q8B01[Jf/204E5A@2407alO011@D40C4a<03@d=01]KFd0GemO
0492@P1^KVh0DE5A06][J`1>CTh0A4A408>3P`1:BTX0Fe]K06EUI@10@400MWIf02/[:`1?Cdl0S8b<
06a/K01=CDd0?3`l0:RXZ01KFe/0JFUY05UIF@15ADD0Rh^;06YZJP17AdL0T92@071`L015ADD0Shn?
059BDP2_[jl0VYZJ04e=C@1QHF40IFEU05QHF01cLg<0Xj>S03Hf=P0j>SX0RHV903`l?01fMWH0;bl_
07]kN`2JVYX0De=C0:RXZ01lO7`0?3`l0:NWY`2i^KT0M7Ad03dm?@1iNGT0G5aL07UiN@1YJFT0O7al
051@D02IVIT0YjNW05aLG01iNGT0I6AT03/k>`1KFe/0T92@04A4A01/K6`0Ph>304I6AP17AdL0MWIf
02LW9`1`L700BDU90:FUY@26QXH0@D5108F5Q@1dM7@0BTY:0;jn_P0b<S80BDU906][J`0Y:BT0N7Qh
0;fm_@2IVIT0AdM703Ti>@21PH40A4A402l_;`28R8P0/K6a0:BTY014A4@0Q8B405]KF`0T92@0HF5Q
04E5A@0Y:BT0Kfm_0:6QX@1fMWH0NWYj07alO00f=SH0@d=305iNGP1SHf<0Ogmo0<71`@2MWId0EEEE
04U9B@2/[:`0F5QH04];B`17AdL0;bl_0P1eMGD0;00m?Cd0^k^k051@D00`<300EeMG05aLG01cLg<0
@41005QHF00e=CD08b<S04M7A`1MGEd0:RXZ07emO@2j^[X0>SXj01lO7`0n?Sh0EeMG04A4A024Q8@0
P820069RHP0[:b/0D51@01DE5@1VIVH0=SHf024Q8@1nOWh06a/K02LW9`2DU9@0OWin02@T900o?cl0
FEUI0451@@0g=cL0SHf=0<k>cP0g=cL0820P3_ooool0302OWil0/K6a0<k>cP3De=@0f=SH08N7Q`0c
<c<0;Rh^028R8P0N7Qh07AdM01`L70806ATI03L05ADE01HF5P0k>c/0;Rh^024Q8@0d=3@0Vi^K09jN
WP2;Rh/0PH6107MgM`25QHD0Ti>C055AD@0P82006a/K01XJ6P1WIfL0^[Zj09JFUP2SXj<0QhN7061P
H01EEED0D51@04E5A@0g=cL0IFEU08^;R`2k^k/0^;Rh0=3@d02l_;`0[:b/0<30`02VYZH0Q8B4096A
T@2RXZ80`l?30;Bd]02JVYX0OWin03Lg=`0U9BD06ATI01/K6`0M7Ad0PX:20;Ng]`1oOgl0GemO06QX
J02>SXh0_[jn0080^KVi00`0a/K60=KFeP3Vi^H0a/K60:j^[P13@d<0<C4a03<c<`11@D40LW9b05aL
G00B4Q8301DE5@0402HV9P23Ph<0]kNg0=OGe`80^KVi04H0clo?09RHV01KFe/0?3`l038b<P0/;2`0
9bLW0410@02XZ:P0IFEU04i>CP22PX80QhN703Ti>@2DU9@0/k>c0<g=c@2BTY80MGEe08R8R02BTY80
TI6A07inOP1=CDd0^k^k06ATI02KVi/0b/[:0<c<c01@D5004Q8B02TY:@1IFET0MGEe04i>CP0H61P0
5ADE01lO7`2PX:00FUYJ0;bl_02a/K40:b/[08^;R`2][Jd0JVYZ03/k>`0Q8B407AdM024Q8@0V9RH0
Bd];07EeM@36a/H0Shn?03Ti>@0W9bL0<C4a0=WIf@3MgMd0d];B0=SHf03Lg=`0inOW0<?3``0Z:RX0
6ATI01hN7P20P800EUIF0P0g=cL0F`0H61P05ADE01DE5@0M7Ad0EUIF06EUI@1BDU80De=C09bLW031
`L40ADE50820P023Ph<0OWin03Ti>@1>CTh0Hf=S08V9R@1`L700:b/[02h^;P0g=cL0KVi^092@T00]
;Bd0<c<c04E5A@16ATH0LG5a05EEE@1cLg<0ADE5028R8P0Y:BT0D51@06e]K@1WIfL0/k>c06UYJ@0^
;Rh0A4A403Lg=`1UIFD0=CDe0410@02SXj<0bl_;09FEU@2][Jd0f=SH0>3Ph02h^;P0i>CT0=3@d01F
EUH051@D01<C4`0E5AD061PH04E5A@30`<00D51@01TI6@0H61P0EeMG01hN7P0J6QX06ATI05YJFP1F
EUH0<S8b06ATI02KVi/0>SXj03Hf=P1RHV80Z:RX0;bl_03Cdm<0b/[:0:JVYP2/[:`0`L710:RXZ01?
Cdl0HF5Q04i>CP2TY:@0Fe]K02DU9@1FEUH00P2c/k<0201QHF40OGem06UYJ@0^;Rh0SXj>0;Zj^P3<
c<`0eMGE0P1HF5P02`0g=cL09bLW02TY:@1@D500Hf=S06EUI@1FEUH0^k^k0=WIf@2g]kL0a/K6013o
oooo000>ooooo`2c08>3P`2GUiL0RHV907AdM021PH40^[Zj0:>SX`17AdL0SHf=09NGU`2[Zj/0CDe=
05aLG02ATI40Bd];061PH016ATH0Vi^K051@D01KFe/0EeMG04E5A@0l?3`0D51@03@d=01JFUX0Ogmo
038b<P0d=3@0A4A402l_;`2EUID09bLW03Ti>@1=CDd0DE5A02d];@0o?cl0?Cdm01dM7@1=CDd0DU9B
01LG5`18B4P0GemO038b<P1CDe<09RHV03dm?@12@T80=3@d09jNWP0g=cL0Cdm?07AdM01ADE40;Bd]
059BDP1CDe<0>SXj09nOW`1hN7P0<C4a05MGE`0c<c<08B4Q069RHP1^KVh0<S8b03Ph>00f=SH0Ti>C
05YJFP1TI6@0^;Rh06i^KP0o?cl0C4a<04e=C@0P8200FUYJ05ADE00N7Qh0C4a<06UYJ@0_;bl0@T92
06QXJ01VIVH0De=C08Z:RP1EEED0=3@d08>3P`1KFe/0FEUI03dm?@29RHT0IfMW03/k>`1hN7P0<c<c
05=CD`0`<300ADE504];B`0L71`0JFUY05mOG`1ADE40<30`04I6AP1fMWH0D51@02/[:`0f=SH09bLW
06ATI01ADE40;Bd]05MGE`0k>c/0GemO02TY:@1cLg<0<c<c07alO01UIFD0;bl_06e]K@0o?cl0EeMG
0:f][@1cLg<0<S8b05mOG`0d=3@0NWYj07inOP1RHV80JVYZ04Y:BP17AdL0Bd];07AdM00o?cl08R8R
079bLP11@D40OWin034a<@1RHV80O7al04Y:BP2TY:@0Ti>C03Ph>01]KFd0[:b/061PH017AdL0VIVI
079bLP2^[Zh0Lg=c04E5A@10@400U9BD09fMW@1hN7P0Rh^;069RHP1[Jf/0B4Q8096AT@1:BTX0EeMG
05eMG@0m?Cd00P1oOgl0H@0Y:BT0CDe=03Ph>02_[jl0U9BD06UYJ@2MWId0IVIV05QHF02_[jl0CDe=
02d];@1EEED05QHF04Y:BP1dM7@0I6AT05IFEP0b<S80QXJ6034a<@0B4Q80UYJF0=;BdP2OWil0<30`
03hn?P1NGUh0>CTi07EeM@1NGUh0;bl_04A4A01iNGT0Cdm?08b<S01hN7P0?Cdm03Hf=P1>CTh0J6QX
05mOG`1TI6@0PX:204];B`1GEeL0bLW903lo?`1`L700Kfm_03Xj>P1cLg<0@T9204U9B@3;bl/0SXj>
051@D01UIFD0K6a/07AdM010@400EEEE02l_;`0[:b/0A4A406m_K`0D51@0DU9B0:BTY00e=CD09BDU
03Ti>@1GEeL0:BTY0410@01IFET0J6QX02LW9`1[Jf/0A4A405eMG@0j>SX09BDU08N7Q`1]KFd0?Cdm
0:6QX@24Q8@092@T0492@P1TI6@0Hf=S03Xj>P1@D500OWin06QXJ00N7Qh03_ooool02P2IVIT0dM7A
0=OGe`2BTY80@D5108>3P`2TY:@0D51@02HV9P0N7Qh201TI6@@05ADE00`0>CTi02h^;P0`<300DE5A
0:ZZZP2DU9@0VIVI0;no_`38b<P0^k^k07IfMP0I6AT201DE5@1:01HF5P0Z:RX0MgMg02/[:`0T92@0
=cLg079bLP2BTY80QXJ606=SH`1EEED0Q8B406a/K032`/80e=CD0;Rh^02TY:@0Shn?0;:b/P2<S8`0
DE5A05aLG01jNWX0][Jf08>3P`2NWYh0U9BD07QhN01]KFd0Hf=S06m_K`1]KFd0]KFe05QHF01CDe<0
C4a<04=3@`0g=cL0U9BD06UYJ@16ATH0BDU905UIF@2d];@0ZJVY07moO`0Q8B405ADE01dM7@0V9RH0
;Rh^04i>CP13@d<061PH01dM7@1<C4`0K6a/05mOG`2VYZH0/;2`0=GEe@3Ph>00e]KF09RHV01TI6@0
D51@059BDP1OGel0LG5a07IfMP1/K6`0Y:BT02LW9`0M7Ad203lo?`0F07IfMP29RHT0Ti>C02<S8`0@
41004a<C01DE5@0Z:RX0GEeM0;Ng]`2_[jl0Jf][0;6a/@3:b/X0h>3P04E5A@0l?3`0=cLg02`/;01<
C4`08R8R01LG5`805ADE04l0EUIF09jNWP24Q8@0c<c<0;6a/@2^[Zh0UiNG0:j^[P20P800I6AT059B
DP11@D40QXJ60;>c/`1ZJVX0=cLg07EeM@2/[:`0MWIf09bLW021PH40NWYj08j>SP2KVi/0bl_;0>3P
h03Ogml0UiNG03Lg=`0T92@0UYJF02/[:`10@400<c<c01PH600G5aL061PH028R8P0N7Qh0F5QH08>3
P`1oOgl0Xj>S0;6a/@2=SHd0QhN709FEU@1XJ6P07QhN02HV9P0k>c/0JVYZ04i>CP0f=SH0ADE506EU
I@2YZJT0g]kN05EEE@0X:2P0<C4a05IFEP2>SXh0R8R804E5A@0/;2`08R8R02PX:00/;2`0O7al0820
P02UYJD0/K6a08Z:RP2BTY80JFUY0:b/[02VYZH0[:b/0080iNGU00<0XJ6Q09^KV`2m_Kd00P3Ogml0
8@3Shn<0bLW90:BTY00P82005ADE01PH600[:b/0ZJVY09VIV@0Q8B405QHF01/K6`1HF5P05ADE01XJ
6P0G5aL06QXJ06ATI012@T80<C4a07=cL`25QHD0?Cdm079bLP2GUiL0a/K60=3@d02g]kL0aLG509bL
W02?Shl0_[jn09JFUP0202<S8`0K04Q8B00N7Qh09BDU07moO`2`/;00QXJ602l_;`1^KVh0FEUI02TY
:@0a<C40BDU907AdM03Cdm<0[Jf]06e]K@0V9RH08B4Q02`/;014A4@0Bd];05ADE01_Kfl0/;2`0=_K
f`3Fe]H0emOG013ooooo000>ooooo`2W07=cL`2?Shl0U9BD07alO01EEED0Hf=S08Z:RP1;Bd/0DU9B
04a<C02[Zj/0HF5Q05QHF01gMgL0EeMG08:2PP1UIFD0UYJF04Q8B01WIfL0LG5a07QhN01HF5P0B4Q8
02/[:`14A4@0M7Ad05QHF00l?3`0=cLg02/[:`1hN7P0BTY:04Q8B01@D500:b/[0451@@0b<S808b<S
02LW9`1FEUH0BTY:03Ti>@1@D500:BTY02/[:`1IFET09BDU03Hf=P0a<C40>SXj08b<S00n?Sh0=3@d
055AD@0T92@0@D5107]kN`12@T807QhN04a<C01TI6@0>3Ph05eMG@1FEUH0<S8b04M7A`1>CTh0<30`
04m?C`0[:b/0IFEU04m?C`1VIVH0_[jn079bLP1=CDd0De=C0410@00K6a/0MGEe05]KF`0L71`0DE5A
04U9B@0O7al0=3@d0492@P1=CDd0DE5A08>3P`1CDe<0@d=306ATI019BDT0G5aL04I6AP1dM7@0IFEU
03Ph>01UIFD09bLW05IFEP0k>c/0@T92055AD@0];Bd0Hf=S05EEE@1@D500C4a<03lo?`1lO7`0FEUI
02XZ:P1>CTh0@41005mOG`1ADE40>CTi06MWI`1bLW80GEeM038b<P1iNGT0<S8b04e=C@1/K6`0@410
07QhN012@T80>c/k08b<S01hN7P0>3Ph07EeM@1MGEd0UYJF07MgM`1VIVH0RXZ:06ATI017AdL0FEUI
079bLP0n?Sh0:BTY07EeM@0f=SH0QhN704i>CP1WIfL0EUIF02TY:@1nOWh0QhN704a<C028R8P0];Bd
03<c<`0h>3P0^[Zj06e]K@2LW9`0OWin07YjNP1@D5000P2JVYX0K@1eMGD0_Kfm07QhN01YJFT0GEeM
08V9R@1IFET0KFe]07inOP18B4P0Bd];07emO@0U9BD0HV9R05IFEP2=SHd0NWYj05eMG@2[Zj/0I6AT
04Q8B02XZ:P0?Shn02l_;`1GEeL0:b/[06EUI@2FUYH0KFe]059BDP0i>CT0R8R804E5A@0o?cl0KFe]
0;2`/02SXj<0CTi>0451@@13@d<0<30`071`L01ZJVX0>SXj05=CD`1oOgl0I6AT06=SH`1/K6`0>c/k
03Lg=`1EEED0JFUY08J6QP1hN7P0Vi^K065QH@1?Cdl0c<c<04I6AP1ADE40CTi>03Xj>P1iNGT0>CTi
02/[:`2g]kL0PX:204A4A01[Jf/0NGUi038b<P0Y:BT0MWIf05ADE00];Bd0@d=306YZJP0V9RH0HV9R
0:ZZZP0c<c<0:BTY04=3@`1SHf<0A4A404i>CP1HF5P0GemO03Ph>025QHD0C4a<06ATI00i>CT0:2PX
08N7Q`1ZJVX0@D510:BTY01kNg/08B4Q0410@01?Cdl0BDU905MGE`1eMGD0U9BD07alO00T92@03_oo
ool0D01UIFD0XJ6Q08V9R@1eMGD0?3`l03dm?@1=CDd0Rh^;07=cL`0V9RH06ATI01HF5P0E5AD05QHF
01XJ6P0g=cL0J6QX08f=S@2OWil0IVIV03Ph>00l?3`0GEeM07EeM@20P800KVi^05ADE00P82006ATI
01HF5P0K6a/0820P06][J`0Y:BT0;Rh^02DU9@0O7al09RHV03Ph>01SHf<0Ph>30:JVYP36a/H0hn?S
0<[:bP1HF5P0CDe=04a<C01[Jf/0KFe]03@d=00a<C40>SXj0;^k^`2f][H0O7al0:6QX@22PX80TY:B
08:2PP1cLg<0Kfm_0:ZZZP1oOgl0C4a<02PX:00S8b<0AdM707AdM00L71`05ADE01<C4`0i>CT0F5QH
0:j^[P1lO7`07QhN01dM7@0H61P08R8R0P0T92@05P1EEED0MWIf07AdM01RHV80AdM7061PH022PX80
]KFe0<_;b`2ATI40Jf][0861P@2JVYX0Ng]k069RHP1TI6@0F5QH05aLG00P8200K6a/051@D00N7Qh2
0451@@0607=cL`2i^KT0?Shn01TI6@0D51@04Q8B0P0A4A403`0@4100BDU90<g=c@2k^k/0/K6a0:^[
Z`39bLT0b/[:02/[:`0D51@05ADE02HV9P14A4@092@T01HF5P0201LG5`0=06=SH`2VYZH0a/K60=cL
g03Jf]X0`l?30;^k^`2ATI40SXj>09>CT`1mOGd0I6AT028R8P0301DE5@0E038b<P1kNg/0YJFU02HV
9P1hN7P0LW9b09RHV039bLT0YJFU08R8R02KVi/0_kno0::RXP2`/;007alO07IfMP1KFe/0Bd];02d]
;@0Q8B40<C4a0080;bl_02<0BTY:09>CT`1gMgL0/;2`0;^k^`3=cLd0c<c<04e=C@0M7Ad09RHV03/k
>`12@T80>c/k07EeM@1XJ6P0S8b<096AT@3<c<`0U9BD038b<P0d=3@0KFe]09^KV`1LG5`0BTY:03<c
<`10@400B4Q802l_;`1fMWH0XJ6Q08R8R01nOWh0`l?30<_;b`020=CDe0090<o?c`3Cdm<0fMWI0<[:
bP1oOgl0UiNG09FEU@2NWYh0gMgM0080jNWY03`0`l?306m_K`1:BTX0CTi>06QXJ03:b/X0@d=303De
=@1=CDd0Ogmo0;Zj^P0K6a/05QHF01DE5@0G5aL06ATI06=SH`0_;bl0>SXj0:JVYP2UYJD0Xj>S0;Bd
]02j^[X0bLW90<[:bP2e]KD0c<c<08B4Q031`L40WIfM08>3P`1iNGT0MGEe0451@@1=CDd0YJFU0;Vi
^@0d=3@0A4A406m_K`1DE5@09bLW02l_;`0j>SX0?Cdm079bLP38b<P0YjNW02TY:@0^;Rh0>CTi04E5
A@1<C4`0Jf][092@T02JVYX0GemO03Xj>P0f=SH@ooooo`003_ooool0J02?Shl0V9RH06UYJ@2>SXh0
Ph>307YjNP1_Kfl0BTY:07EeM@1^KVh0V9RH06ATI025QHD0FUYJ04M7A`1mOGd0JFUY08f=S@1@D500
CTi>07EeM@1TI6@0EeMG06EUI@0m?Cd0GEeM05aLG00e=CD0De=C04];B`0T92@0MWIf028R8P1DE5@0
Fe]K03/k>`0h>3P0BDU902l_;`0M7Ad0@T9203Hf=P0V9RH0I6AT04I6AP0_;bl0DE5A038b<P1XJ6P0
ATI603Xj>P25QHD0>CTi059BDP1NGUh0<C4a02XZ:P1FEUH0Fe]K02/[:`1RHV80KFe]03De=@13@d<0
?Cdm028R8P1JFUX0IFEU02`/;00f=SH0=3@d092@T01?Cdl0FEUI0::RXP1eMGD0C4a<07IfMP1EEED0
71`L05iNGP15ADD0:BTY06YZJP18B4P0<C4a04Q8B00o?cl0D51@05YJFP2CTi<0JFUY04I6AP1HF5P0
=cLg06YZJP1DE5@0K6a/03<c<`1=CDd0JVYZ03`l?01@D500;bl_0P1QHF40B@0k>c/0Lg=c04Y:BP1Z
JVX0BDU90451@@1ZJVX0Bd];04m?C`1WIfL0@D5106=SH`0f=SH0DE5A07MgM`1aLG40CDe=04E5A@2:
RXX0B4Q806][J`1QHF40<c<c09>CT`1QHF40?clo0::RXP1UIFD0ATI608f=S@22PX80Shn?079bLP0h
>3P0LG5a07IfMP1>CTh0MGEe06m_K`0];Bd0CDe=08N7Q`0l?3`0J6QX030`<01PH600S8b<03lo?`2C
Ti<0IVIV04];B`2GUiL0[:b/06][J`21PH40dM7A075aL@2m_Kd0Y:BT08:2PP1?Cdl0YJFU07moO`1D
E5@0RHV9096AT@1[Jf/0JVYZ06a/K01@D500UYJF061PH01:BTX00P1SHf<04@0V9RH0O7al05MGE`2T
Y:@0DU9B04Q8B02[Zj/0Hf=S04m?C`2KVi/0<c<c05eMG@1`L700<S8b06a/K02=SHd0Kfm_0080A4A4
03L0UYJF06YZJP1RHV80R8R80<g=c@2FUYH0>c/k06IVIP1KFe/0@T92079bLP1TI6@0@410051@D02<
S8`0M7Ad06IVIP1aLG40;Rh^038b<P1/K6`0Jf][08N7Q`1fMWH0O7al04Y:BP1=CDd0f=SH04U9B@1/
K6`0EEEE04=3@`26QXH0DU9B05mOG`37alL0MgMg04Q8B02HV9P0Zj^[05aLG00_;bl0KVi^01hN7P0f
=SH0NGUi06a/K00U9BD0IVIV09nOW`0T92@0Bd];07emO@1LG5`0?Cdm0080D51@01D0>CTi0410@01d
M7@0BTY:05MGE`0/;2`0>CTi06e]K@1IFET0>SXj0:FUY@1bLW80:BTY05aLG01^KVh0I6AT05eMG@1P
H600Cdm?05]KF`11@D403_ooool07@0l?3`0CDe=08n?S`1nOWh0@41003Ph>00i>CT0?clo0410@01G
EeL0EUIF03Xj>P0d=3@0<c<c038b<P0Z:RX0;Bd]02/[:`0Z:RX0>c/k04Y:BP1IFET0E5AD08N7Q`1O
Gel0E5AD065QH@1SHf<09RHV00806ATI0<P07alO075aL@12@T8092@T02LW9`0Z:RX09RHV024Q8@0V
9RH0@d=3069RHP1aLG40_kno08f=S@1PH600Bd];0451@@14A4@0J6QX061PH011@D40?Shn08f=S@2T
Y:@0UiNG08N7Q`1`L700PH6106QXJ01VIVH0LG5a06YZJP2MWId0QXJ604m?C`1EEED0NWYj02LW9`0I
6AT08R8R04A4A012@T80;Bd]069RHP3Fe]H0GEeM01lO7`0N7Qh07AdM024Q8@0Y:BT0@T920<71`@23
Ph<0MWIf06a/K01UIFD0R8R806YZJP20P8009BDU02XZ:P0^;Rh0;2`/0410@01LG5`0I6AT07QhN01j
NWX0M7Ad075aL@2f][H0Q8B408Z:RP2;Rh/0/k>c08J6QP0i>CT06a/K01LG5`0E5AD04Q8B01DE5@0G
5aL0JVYZ03<c<`0V9RH0@D5106i^KP2CTi<0Rh^;02`/;00D51@05QHF02/[:`0X:2P0?Shn06ATI019
BDT0?3`l03Ti>@2c/k<0f][J0=gMg@3Ogml0bLW90:^[Z`1`L700JVYZ05mOG`1DE5@0>CTi020P800G
5aL05ADE01PH600Q8B40EeMG05QHF01MGEd0ZZZZ09RHV02h^;P0c<c<06UYJ@1SHf<0<C4a03<c<`1D
E5@0ZJVY08f=S@20P800J6QX06EUI@1ADE40BTY:055AD@16ATH0@41003Xj>P0m?Cd0HF5Q08>3P`3J
f]X0i^KV0=oOg`2EUID0>c/k06i^KP1dM7@0IfMW06e]K@1/K6`0G5aL0451@@0a<C40@D5109fMW@2@
T900Shn?0:VYZ@1LG5`0@d=306QXJ01>CTh0F5QH05]KF`1RHV80IVIV06e]K@1OGel0Hf=S096AT@3:
b/X0fMWI0<c<c02YZJT0[jn_0;2`/02b/[80T92@07IfMP1OGel0E5AD08V9R@3AdM40f=SH0=cLg03E
eMD0XJ6Q08N7Q`2l_;`0QXJ603`l?00o?cl0>c/k04];B`1ZJVX0EeMG038b<P<05QHF0180;bl_07]k
N`1bLW80[Jf]0:2PX01hN7P0QhN70<C4a03IfMT0_;bl08R8R02;Rh/0fm_K0;fm_@2DU9@0EEEE05eM
G@1nOWh20<S8b00H0:^[Z`0k>c/0@410069RHP1iNGT0>CTi02@T900X:2P0>CTi04Q8B01;Bd/0J6QX
0<g=c@2EUID0BDU903Xj>P14A4@0DE5A092@T02HV9P0UYJF02h^;P0b<S80BTY:4?ooool000kooooo
0?l0RXZ:0:BTY02<S8`0Vi^K075aL@1JFUX0Ng]k05ADE01^KVh0Ph>308B4Q01BDU80W9bL07YjNP1G
EeL0F5QH04A4A026QXH0Bd];04U9B@1lO7`0EeMG04M7A`1?Cdl0=3@d06UYJ@1fMWH0BTY:08N7Q`1O
Gel09BDU07=cL`0o?cl0Cdm?05aLG01<C4`0>CTi04Q8B00k>c/071`L0410@00o?cl09RHV04I6AP1E
EED0;2`/04i>CP0Z:RX0ADE504=3@`0m?Cd0OWin02<S8`0e=CD0GemO04I6AP0/;2`0EUIF02PX:00C
4a<0<C4a071`L00j>SX0<30`02@T900R8R80De=C05UIF@0/;2`0;Bd]03Xj>P1mOGd0CTi>06YZJP2g
]kL0De=C04Q8B01GEeL0@D5101hN7P1VIVH0<C4a01DE5@1>CTh0AdM7024Q8@1<C4`0?3`l0410@013
@d<0KFe]06ATI00m?Cd0D51@04A4A01OGel0GEeM07MgM`1DE5@0?clo03Hf=P0c<c<0DE5A03Ti>@14
A4@0DU9B02<S8`1LG5`0De=C05MGE`0o?cl0@41006m_K`1FEUH0BTY:061PH019BDT0DE5A02PX:01I
FET0NGUi06=SH`0m?Cd0@41007IfMP0n?Sh0JFUY071`L014A4@0Rh^;04a<C00o?cl0W9bL0492@P0`
<300N7Qh08^;R`2GUiL0BTY:065QH@2WYjL0KFe]02/[:`2BTY80Hf=S03/k>`1:BTX0FEUI04=3@`25
QHD0Cdm?05aLG026QXH0@d=308f=S@1IFET0>SXj06UYJ@2IVIT0FUYJ09ZJVP3Jf]X0=CDe08F5Q@2<
S8`0G5aL05]KF`2a/K40UIFE07alO01FEUH0PX:206m_K`1eMGD0LG5a05MGE`25QHD0E5AD04m?C`1/
K6`0IfMW02PX:01^KVh0CTi>0:FUY@1kNg/0L71`0;no_`1cLg<0Cdm?09>CT`0[:b/0@41004e=C@0O
7al0GUiN06YZJP1<C4`0@D5102HV9P22PX80ATI6059BDP1nOWh0f=SH09>CT`0X:2P0CTi>06IVIP11
@D40M7Ad04i>CP0j>SX0MGEe09bLW01VIVH0Fe]K07inOP0Q8B40=3@d04m?C`1/K6`0MgMg07AdM01`
L700=SHf04=3@`2l_;`0Bd];059BDP10@400<30`06QXJ01?Cdl0De=C0<;2`P1KFe/0<S8b061PH01e
MGD0H61P03`l?02LW9`0:b/[02HV9P14A4@0G5aL034a<@1WIfL0VYZJ034a<@1=CDd0Ti>C05aLG00b
<S8001<0A4A4051@D01PH600@41005aLG00h>3P0Kfm_03De=@0b<S80KFe]02LW9`0X:2P0VIVI07Ad
M00O7al0>SXj04E5A@10@400?clo0080BTY:00<0EUIF03De=Oooool03Oooool03`0j>SX0?Cdm0:RX
Z02CTi<0De=C03Hf=P0];Bd0:RXZ030`<015ADD0UiNG04Y:BP0V9RH06a/K01DE5@0201@D500X01LG
5`0E5AD05ADE01DE5@0X:2P0IVIV0:NWY`2SXj<0I6AT08B4Q02FUYH0QHF502DU9@0L71`08B4Q0:>S
X`2?Shl0=cLg02HV9P0N7Qh08R8R024Q8@0/;2`0DE5A05UIF@24Q8@0PX:207=cL`0R8R80FUYJ07in
OP1gMgL0LG5a05UIF@1_Kfl0MgMg071`L01QHF40WIfM05eMG@80=SHf02<0C4a<018B4P0@41003`l?
01TI6@0a<C40Xj>S0=gMg@1]KFd04Q8B01@D500Q8B40?Cdm01dM7@0V9RH0J6QX07YjNP1PH600KFe]
02l_;`0J6QX06a/K01LG5`0Q8B40_[jn03dm?@0d=3@0=CDe04A4A01WIfL0IFEU07]kN`0V9RH09BDU
030`<00201TI6@0401`L700];Bd0<c<c03Lg=`80=SHf00L0OWin08f=S@3Cdm<0inOW08F5Q@1KFe/0
CDe=00806ATI010071`L01dM7@0/;2`0A4A40:>SX`0N7Qh092@T03`l?01:BTX0U9BD09nOW`24Q8@0
DE5A03`l?013@d<0D51@0P1KFe/0501NGUh0IfMW04E5A@20P800Zj^[0<k>cP3Fe]H0_Kfm06YZJP0k
>c/0=3@d03Lg=`0m?Cd0Bd];06e]K@1iNGT0H61P03<c<`1QHF408B4Q0P0E5AD06@0`<300P82007Yj
NP3>c/h0RHV90<;2`P2GUiL0P8200:f][@2XZ:P0NWYj08>3P`1nOWh0PX:207AdM01mOGd0Hf=S03@d
=00];Bd0;bl_02<S8`0O7al0=3@d0=7Ad@3Xj>P00P3YjNT0203WinL0`/;208^;R`13@d<0:BTY02DU
9@1WIfL0?clo0P0a<C403@1VIVH0_;bl0:6QX@35aLD0[Zj^06YZJP2?Shl0NWYj07QhN01eMGD0Q8B4
0;>c/`0l?3`00P0L71`02@12@T80UiNG08F5Q@37alL0eMGE0;:b/P1`L700;bl_02@T900201lO7`0;
03Lg=`1TI6@0[Zj^0;no_`3WinL0clo?06m_K`0g=cL0?clo0:BTY00d=3@00P0c<c<0<`1GEeL0A4A4
030`<01EEED0G5aL02PX:00I6AT06a/K04i>CP3Ogml0eMGE0=;BdP38b<P0[:b/09VIV@3AdM40`l?3
0<O7a`3De=@0cLg=04e=C@2?Shl0UiNG06ATI01`L700c<c<0=WIf@33`l<0Ph>3069RHP2YZJT0][Jf
03Xj>P0U9BD09bLW030`<00h>3P0CDe=09JFUP3GemL0dm?C0<C4a01^KVh0EeMG08B4Q035aLD0]KFe
0<_;b`2@T900K6a/04M7A`0@ooooo`003_ooool0o`1hN7P0WYjN07moO`2CTi<0I6AT061PH02FUYH0
De=C06e]K@22PX80WYjN08J6QP2l_;`0O7al04m?C`1hN7P0IVIV09bLW01KFe/0CDe=092@T02ATI40
LG5a05iNGP0d=3@0J6QX08b<S01EEED0Vi^K06ATI00/;2`0K6a/03De=@1:BTX0GEeM03dm?@0i>CT0
:b/[020P800M7Ad0@41003/k>`0h>3P0FUYJ05UIF@0f=SH0E5AD0451@@1HF5P0EEEE03hn?P27QhL0
ATI603hn?P1HF5P0>CTi02HV9P1KFe/0FUYJ02LW9`0f=SH0Hf=S03Hf=P0l?3`0>CTi02h^;P1_Kfl0
Hf=S038b<P0g=cL0@41007]kN`1KFe/0Hf=S0:2PX01CDe<0CDe=05MGE`0o?cl08B4Q06ATI00^;Rh0
6QXJ07inOP0e=CD09BDU05iNGP1HF5P0B4Q805]KF`1iNGT0H61P059BDP1mOGd0?clo05aLG01^KVh0
UYJF05ADE019BDT0@D5103Lg=`1FEUH0>SXj04U9B@1ADE40:2PX069RHP1=CDd0De=C04e=C@1]KFd0
M7Ad069RHP1aLG40EeMG05UIF@1UIFD0>SXj07emO@22PX80DE5A05=CD`1:BTX0KVi^03<c<`1mOGd0
IfMW03dm?@2EUID0>SXj05=CD`2d];@0C4a<05iNGP21PH40H61P07EeM@1@D500F5QH09jNWP2:RXX0
@D5109BDU01MGEd0ADE5079bLP1?Cdl0@d=307IfMP1EEED0JVYZ08J6QP1;Bd/0Xj>S07UiN@1[Jf/0
UIFE0:b/[01GEeL0ZZZZ0<g=c@1aLG40XJ6Q09:BTP1=CDd0GUiN0;no_`2?Shl0VIVI0:>SX`2BTY80
BDU907inOP2NWYh0JVYZ0:RXZ01eMGD0FEUI07AdM01oOgl0=SHf07]kN`1GEeL0WinO06i^KP1`L700
`L7106e]K@1CDe<0WinO04E5A@1;Bd/0Hf=S02l_;`1EEED0MgMg05]KF`1<C4`0:BTY07inOP18B4P0
DU9B06][J`3Hf=P0P82002/[:`1EEED0AdM703Hf=P1fMWH0@D5101lO7`1HF5P0NGUi05mOG`1<C4`0
JFUY02l_;`0f=SH0EeMG06QXJ01kNg/0QhN708n?S`1/K6`0=SHf0;Rh^01PH600BTY:05mOG`13@d<0
NWYj06a/K01DE5@0aLG508n?S`1BDU80UiNG0;bl_02?Shl0DE5A092@T00c<c<0;bl_05MGE`1bLW80
;bl_05mOG`2OWil0>CTi05mOG`2ATI40JFUY0451@@005`1WIfL0Ogmo06EUI@1?Cdl0Kfm_03`l?01[
Jf/0>c/k04Q8B01bLW80C4a<02DU9@25QHD0NGUi02/[:`1<C4`0G5aL03@d=00Q8B40IVIV08j>SP1R
HV80=cLg00kooooo00P0?3`l04];B`3@d=00`l?309fMW@12@T80=SHf02TY:@809BDU00<0E5AD02HV
9P0I6AT00`0E5AD201LG5`@05ADE03X0:2PX0<30`039bLT0Zj^[0;bl_01]KFd0[Jf]059BDP13@d<0
GEeM0;2`/026QXH0PH6104U9B@0S8b<0820P03/k>`1IFET0BDU903/k>`1/K6`0[Zj^08^;R`0];Bd0
7alO07moO`3?cll0XZ:R07UiN@0c<c<07alO01DE5@0Q8B40NWYj034a<@15ADD0>CTi051@D00m?Cd0
5ADE01@D500E5AD0;Rh^06EUI@2/[:`0/[:b02PX:00O7al0AdM705aLG00a<C40;2`/08J6QP0Q8B40
410@05QHF020P800=CDe0P0I6AT0300Q8B40Y:BT04E5A@0U9BD0=CDe079bLP1GEeL0SXj>08n?S`1;
Bd/0;Bd]020P80<05aLG03@06QXJ02<S8`0[:b/0;Bd]02h^;P1ADE40EUIF07QhN033`l<0ZJVY08N7
Q`1hN7P071`L028R8P0h>3P0OGem05aLG02HV9P0^[Zj02/[:`10@400DE5A03Ti>@0l?3`0_[jn0<W9
b@1QHF409bLW02h^;P0m?Cd0@d=303Ti>@0c<c<0CDe=069RHP2LW9`0^[Zj0<g=c@2d];@0^[Zj08n?
S`1JFUX0ADE503lo?`0l?3`0>SXj04A4A00o?cl0GEeM0;6a/@1JFUX05QHF0P0E5AD05`0M7Ad0QhN7
0:b/[02n_[h0cLg=0=cLg03>c/h0^KVi0:^[Z`2YZJT0Fe]K04A4A01WIfL0EUIF03Ti>@0k>c/0IFEU
08B4Q01ADE40<c<c02LW9`0P8200MgMg0080cLg=0280G5aL09NGU`3Ng]h0@T9201dM7@13@d<061PH
028R8P26QXH0G5aL04Y:BP1<C4`0LG5a09jNWP2a/K40MgMg0;Fe]@3AdM40WinO0:ZZZP1SHf<0?Cdm
02<S8`1mOGd0:BTY01dM7@0J6QX0:b/[05EEE@1^KVh0Zj^[0;2`/02OWil05aLG0`0E5AD0;`0G5aL0
5ADE02PX:03:b/X0h^;R0<g=c@2b/[80EeMG020P800c<c<0WIfM065QH@0Y:BT0?clo06QXJ00l?3`0
<30`02h^;P0g=cL0G5aL04a<C00/;2`0YJFU0=KFeP31`L40T92@04a<C00g=cL0Lg=c09BDU035aLD0
_Kfm0;Zj^P3AdM40ADE505]KF`2m_Kd0]kNg0<C4a03De=@0eMGE0<W9b@3?cll0bl_;0=KFeP3:b/X0
F5QH0080=SHf0100?3`l04Q8B02NWYh0WIfM06m_K`2TY:@0d=3@0=OGe`37alL0bl_;0=GEe@2m_Kd0
YJFU03Lg=`18B4P0OWin4?ooool000kooooo08X0NWYj09fMW@1TI6@0Ti>C06e]K@29RHT0W9bL04m?
C`1cLg<0QhN70:FUY@2:RXX0`<3007AdM01GEeL0UYJF07EeM@1dM7@0>CTi06MWI`2;Rh/0P82007mo
O`1hN7P092@T06m_K`25QHD0CTi>09JFUP0l?3`0BTY:06e]K@0f=SH0HF5Q05EEE@0Y:BT0?Shn03/k
>`0M7Ad0=3@d05YJFP1>CTh0Bd];0820P00j>SX0D51@055AD@0k>c/0JFUY05EEE@1;Bd/0Lg=c03dm
?@13@d<0Cdm?02/[:`0Z:RX0GEeM06MWI`0a<C40>3Ph06QXJ01MGEd0KFe]03<c<`0O7al0HF5Q07Ui
N@1;Bd/0@d=304U9B@2?Shl0GEeM08f=S@2i^KT0LG5a04a<C01TI6@0C4a<0492@P2CTi<0B4Q802LW
9`28R8P0BDU901lO7`15ADD0>SXj04M7A`1^KVh0Zj^[06QXJ00o?cl0CTi>03Ph>01/K6`0ATI607If
MP1BDU80H61P05YJFP0f=SH0JFUY04A4A01^KVh0Jf][02PX:01ZJVX0Cdm?06ATI00g=cL0De=C04a<
C01;Bd/0FEUI06][J`14A4@0GemO030`<01NGUh0Jf][0451@@13@d<0L71`07QhN00_;bl0GUiN05UI
F@17AdL0/K6a03Ph>011@D40V9RH04E5A@0_;bl0MGEe09BDU01dM7@206UYJ@2:07AdM01LG5`0?Cdm
09RHV01CDe<0;2`/07IfMP1]KFd0@T920451@@0c<c<0PX:2071`L011@D40NWYj07moO`2NWYh0^;Rh
09JFUP1ADE40alO70;jn_P1]KFd0^[Zj0:f][@1bLW80Fe]K0<?3``1eMGD0KFe]05UIF@1TI6@0BTY:
06i^KP1bLW80Lg=c0;Fe]@1FEUH0GEeM09FEU@1]KFd0?Cdm07YjNP1KFe/0Zj^[04i>CP16ATH0/K6a
059BDP1NGUh0R8R803`l?01ADE40HF5Q02DU9@1fMWH0Zj^[075aL@1QHF40<S8b06QXJ01<C4`0K6a/
06=SH`3MgMd0N7Qh03Lg=`1`L700EeMG02d];@1LG5`0=3@d02PX:01DE5@0Ogmo05UIF@19BDT0H61P
03Hf=P10@400GEeM06UYJ@27QhL0YJFU08f=S@1EEED0:b/[07alO00m?Cd0CTi>04M7A`14A4@0OGem
04U9B@1<C4`0^k^k08^;R`1DE5@0ZZZZ0;Vi^@2?Shl0K6a/0:RXZ01EEED0=CDe05eMG@1]KFd0<S8b
05]KF`2VYZH0<30`04m?C`2WYjL0Lg=c038b<P15ADD0IVIV03hn?P20P800Ph>304I6AP1gMgL08R8R
03De=@1cLg<0EUIF02<S8`24Q8@0GemO03dm?@1aLG40Lg=c05ADE00k>c/0@d=3079bLP1;Bd/0AdM7
3_ooool04`1dM7@0]kNg0=GEe@32`/80KVi^065QH@2?Shl0HF5Q04];B`0k>c/0FEUI02`/;00];Bd0
820P038b<P0P82006QXJ01TI6@0H61P00P0J6QX03`0H61P06a/K0;Zj^P3MgMd0c<c<09:BTP21PH40
TY:B03`l?014A4@0Rh^;030`<00G5aL0;Bd]08B4Q002055AD@0J05iNGP1fMWH0HF5Q05QHF016ATH0
XJ6Q09bLW022PX80A4A406YZJP2^[Zh0;Bd]01DE5@0G5aL06QXJ01TI6@0W9bL0K6a/01`L700E5AD0
9RHV0451@@1JFUX0;Rh^01LG5`0O7al2075aL@0L0:f][@3Hf=P0^k^k0820P023Ph<0SHf=08J6QP2G
UiL0/[:b01DE5@0?3`l04Q8B06EUI@2YZJT0ADE502h^;P0a<C40[jn_07=cL`0j>SX0IfMW08R8R029
RHT0J6QX02PX:01WIfL0DU9B01dM7@806QXJ030061PH01TI6@0N7Qh09bLW02XZ:P0R8R80>CTi09nO
W`1GEeL0Zj^[0<c<c02e]KD0c<c<06a/K01=CDd0>c/k02d];@13@d<0WinO09fMW@1NGUh0V9RH05=C
D`1dM7@0D51@09FEU@26QXH0e=CD08:2PP1WIfL0MGEe06m_K`1XJ6P0Hf=S06e]K@22PX80`<300<c<
c03;bl/0V9RH03@d=02b/[80BTY:034a<@0Y:BT08R8R024Q8@0S8b<201dM7@80DE5A0P0E5AD0>`0G
5aL08B4Q055AD@2UYJD0Shn?0=7Ad@3Xj>P0c<c<0<O7a`2QXJ40^KVi055AD@28R8P0?3`l020P800N
7Qh07alO02LW9`0`<300LW9b07UiN@14A4@0CDe=08^;R`3?cll0VIVI04m?C`1>CTh0b/[:03/k>`15
ADD0TY:B03Xj>P0n?Sh0LG5a0<71`@2f][H0_;bl0:JVYP1<C4`0G5aL0:JVYP3AdM40g=cL0<71`@2?
Shl0?Shn01hN7P0Y:BT0EUIF02LW9`0/;2`08R8R05ADE016ATH0EeMG06QXJ00f=SH0CDe=00<05ADE
01405aLG01/K6`0F5QH0:RXZ0<o?c`2n_[h0Z:RX0;2`/02@T900<C4a02/[:`1GEeL0V9RH03Ph>01@
D500FEUI02XZ:P0203@d=00902l_;`0b<S80=cLg092@T02LW9`0GEeM05mOG`0/;2`0=SHf0080Cdm?
02@0<S8b061PH02/[:`0`/;20=OGe`2][Jd0C4a<0;>c/`3Fe]H0g=cL0=OGe`3;bl/0/K6a0=7Ad@3D
e=@0fm_K0>3Ph031`L40RXZ:03`l?00^;Rh0QHF507QhN00_;bl0GemO071`L02h^;P0aLG50=cLg03K
fm/0`/;20;fm_@2b/[80QhN70;6a/@25QHD@ooooo`003_ooool0FP1gMgL0UiNG05EEE@21PH40D51@
05YJFP1?Cdl0KVi^09NGU`1iNGT0TI6A09JFUP35aLD0@d=305EEE@1`L700G5aL07alO00m?Cd0B4Q8
071`L00o?cl0A4A405aLG00R8R80AdM705ADE00g=cL0IfMW04E5A@0j>SX0DE5A030`<01OGel0HV9R
038b<P0[:b/0@T92028R8P0J6QX0?Shn02TY:@0b<S80FEUI051@D00i>CT0>c/k04E5A@27QhL0B4Q8
07QhN01/K6`0?Cdm04I6AP1LG5`0<S8b04M7A`1oOgl0H61P03Ph>01FEUH0JVYZ03`l?010@4007alO
02TY:@1hN7P0H61P03`l?00g=cL0ATI608B4Q011@D40J6QX0;Bd]01OGel0Jf][08j>SP0l?3`0=CDe
06ATI00b<S806a/K06m_K`0X:2P071`L04m?C`0d=3@0@41005IFEP80IVIV04h0BTY:04];B`0d=3@0
FUYJ04A4A01ZJVX0>CTi06QXJ01;Bd/0<c<c05aLG01=CDd0A4A4055AD@0W9bL0GUiN04A4A01:BTX0
>3Ph05UIF@1dM7@0GUiN0410@01XJ6P0CDe=06UYJ@0j>SX0JVYZ07AdM01<C4`0ATI607IfMP21PH40
AdM705eMG@1_Kfl0@D5109NGU`0g=cL0?clo09:BTP1;Bd/0@D510861P@2o_kl0IVIV055AD@1iNGT0
R8R805]KF`0j>SX0QhN704A4A012@T80Ng]k05ADE01?Cdl0N7Qh04U9B@1LG5`0Ng]k059BDP1^KVh0
JVYZ05QHF02WYjL0OWin0451@@38b<P0X:2P04M7A`2_[jl0W9bL05ADE01OGel0d=3@07inOP2GUiL2
055AD@0/04m?C`1^KVh0HF5Q04m?C`26QXH0>SXj04E5A@1`L700<S8b03dm?@27QhL0I6AT0:^[Z`17
AdL0;Bd]0;Ng]`1]KFd0IVIV08R8R00];Bd0OWin06ATI00K6a/0ATI607UiN@1XJ6P0C4a<02h^;P1B
DU80CDe=05aLG01OGel0fm_K075aL@11@D40M7Ad06e]K@0c<c<0GemO03Xj>P0i>CT0H61P07alO01C
De<208B4Q00d03dm?@0n?Sh0I6AT05YJFP1`L700Jf][08B4Q01dM7@0=SHf06][J`14A4@0Kfm_06EU
I@19BDT0Ng]k04=3@`1=CDd0WinO08J6QP1IFET0/K6a0;Fe]@2CTi<0MWIf0;Fe]@1aLG40=cLg05YJ
FP1eMGD06ATI05eMG@2;Rh/0=CDe06EUI@3:b/X0QXJ603De=@12@T80H61P0451@@1LG5`0M7Ad05=C
D`27QhL0<S8b034a<@1UIFD0ATI602<S8`1kNg/0Jf][02`/;080AdM700H0I6AT04Q8B010@400KFe]
0492@P0_;bl>ooooo`0i08F5Q@1UIFD0YjNW0=3@d01^KVh0Bd];05aLG01^KVh0GemO06ATI023Ph<0
Kfm_0:f][@2f][H0`L71071`L00g=cL0>SXj03`l?00m?Cd0Bd];04e=C@1OGel0emOG0=cLg03QhN40
cLg=0;:b/P2WYjL0S8b<0<G5a@2CTi<0=3@d02/[:`0S8b<09RHV0;2`/02/[:`0Zj^[09^KV`2NWYh0
VIVI05]KF`28R8P0];Bd0<S8b02VYZH0f][J0<O7a`0i>CT08R8R02`/;00b<S80>3Ph03hn?P1dM7@0
92@T00805ADE06X0:BTY04=3@`1^KVh0@d=308V9R@2YZJT0RXZ:0;Bd]03Fe]H0QhN7069RHP22PX80
UiNG096AT@2NWYh0YjNW03Ph>00B4Q804A4A02`/;03Jf]X0e=CD0:n_[`2UYJD0a/K60<C4a02f][H0
clo?0=cLg02j^[X09bLW01lO7`0L71`0CDe=05=CD`0O7al071`L01TI6@0L71`07AdM024Q8@0M7Ad0
8B4Q01lO7`2EUID0cLg=0=_Kf`3Zj^X0hn?S0>KViP3Jf]X0V9RH0861P@2@T900][Jf0=cLg01jNWX0
DE5A07moO`2d];@0c/k>0:JVYP22PX80:RXZ06MWI`1XJ6P09bLW02l_;`0/;2`07QhN01/K6`0G5aL0
92@T07MgM`2:RXX0c/k>09FEU@0e=CD0=cLg0;6a/@0c<c<07AdM01/K6`0M7Ad071`L02DU9@0R8R80
9RHV05]KF`0E5AD06QXJ03dm?@1:BTX0<c<c03`l?03<c<`0emOG0>CTi03AdM40eMGE09FEU@33`l<0
][Jf04e=C@10@4008B4Q0P0N7Qh0K`0P82008R8R02@T900h>3P0W9bL0;Bd]02b/[80IFEU08N7Q`1F
EUH0QXJ60<71`@34a<@0QHF505=CD`1WIfL0_;bl06YZJP1KFe/0Q8B40<G5a@2@T900@T9202/[:`0X
:2P0QHF50:2PX02[Zj/0LG5a01dM7@0N7Qh0ADE502d];@0c<c<0BTY:04a<C01nOWh0G5aL05IFEP1H
F5P0;2`/03<c<`0G5aL05ADE01HF5P0H61P05QHF01PH600T92@0WIfM07UiN@26QXH0[Zj^0:FUY@1I
FET0=3@d04E5A@2UYJD0K6a/06UYJ@13@d<0:b/[03Lg=`12@T80?Shn03Ti>@1?Cdl0Y:BT08R8R02<
S8`0BDU903Lg=`1CDe<0Hf=S04M7A`0k>c/0<S8b03De=@1SHf<0d];B0=oOg`2i^KT0bl_;0;Jf]P3L
g=`0b<S807IfMP2XZ:P0`l?30=3@d03MgMd0fMWI0;Ng]`2?Shl0Vi^K08V9R@2>SXh0?Shn04Y:BP1i
NGT0MWIf09ZJVP3Kfm/0fMWI0=GEe@3Kfm/0f=SH0;6a/@39bLT0eMGE07emO@0@ooooo`003_ooool0
E01kNg/0][Jf07=cL`2FUYH0CTi>05YJFP28R8P0Cdm?055AD@1kNg/0MgMg07AdM02j^[X0LW9b069R
HP28R8P0L71`09jNWP1;Bd/0AdM708j>SP2=SHd0FEUI065QH@0R8R80B4Q807QhN01:BTX0Hf=S065Q
H@0n?Sh0D51@03Ti>@1<C4`0B4Q802TY:@0e=CD07alO020P800N7Qh0AdM703@d=00o?cl0Hf=S04i>
CP10@400?Shn02l_;`1EEED0=SHf06EUI@0i>CT0:RXZ05iNGP1`L700;2`/02l_;`1VIVH0BDU902HV
9P17AdL0EeMG03Ti>@12@T808R8R02PX:01UIFD0GUiN03`l?00^;Rh0@D5108B4Q018B4P0OGem09ZJ
VP1DE5@0?Cdm05IFEP0a<C40=3@d06IVIP0_;bl06QXJ07QhN0806QXJ01H0D51@03Hf=P12@T80KVi^
06IVIP1SHf<0@d=304E5A@0j>SX0EeMG04=3@`1aLG40?clo05QHF00c<c<0;Rh^05QHF01jNWX0Cdm?
03/k>`0K6a/0EUIF0P13@d<0@00_;bl0HF5Q05mOG`1MGEd0@d=305UIF@1GEeL0GemO034a<@1LG5`0
GUiN04m?C`0d=3@0N7Qh08J6QP1=CDd0FUYJ06ATI011@D40UiNG034a<@12@T80NGUi02d];@17AdL0
KFe]0;Bd]01JFUX0?3`l08>3P`1kNg/0GUiN03Lg=`2=SHd0<C4a03lo?`29RHT0>3Ph04m?C`20P800
Cdm?05iNGP1jNWX0Fe]K06][J`20P800LG5a0:VYZ@20P800Bd];0<[:bP2GUiL0G5aL0:NWY`2?Shl0
Bd];06MWI`3IfMT0Q8B409FEU@1CDe<0KVi^05mOG`1gMgL206][J`1V09>CT`17AdL0EeMG08:2PP1W
IfL0>CTi07moO`1`L700Vi^K05ADE01PH600YZJV05QHF01KFe/0Rh^;02LW9`1?Cdl0GemO038b<P15
ADD0MgMg05MGE`15ADD0;Rh^04m?C`12@T80;bl_04m?C`3@d=00Kfm_03dm?@1`L700GemO03lo?`1d
M7@0K6a/05IFEP1cLg<0UiNG04i>CP1NGUh0G5aL02`/;01ADE40RXZ:06QXJ01hN7P0DU9B06MWI`1I
FET0B4Q809>CT`1:BTX0D51@061PH015ADD0NGUi03lo?`1=CDd0[:b/08n?S`1IFET0[:b/09^KV`0m
?Cd0I6AT0;>c/`1]KFd0Cdm?096AT@2;Rh/0=3@d059BDP1iNGT06a/K03@d=02NWYh0MgMg034a<@1:
BTX0HF5Q05EEE@1HF5P0NWYj04a<C023Ph<0=3@d038b<P1PH600B4Q8024Q8@1nOWh0Lg=c03Hf=P1@
D500Fe]K06a/K01@D500?Cdm06a/K01[Jf/0<c<c3_ooool0D@2NWYh0OWin07alO01eMGD0QhN708V9
R@1KFe/0E5AD061PH01HF5P0IVIV03Xj>P16ATH0J6QX09ZJVP2:RXX0HF5Q04U9B@0n?Sh0@D5103lo
?`0Z:RX0JFUY0;no_`3Kfm/0h>3P0=CDe03IfMT0a/K60::RXP2;Rh/0SXj>075aL@1;Bd/0>CTi02h^
;P1kNg/0A4A404];B`1YJFT0S8b<0;Fe]@2YZJT0S8b<0=?Cd`3QhN40dm?C0>?Sh`3Fe]H0UIFE04m?
C`1FEUH0JVYZ0::RXP2PX:00[Zj^04];B`0];Bd08b<S028R8P0Z:RX0GemO07MgM`0b<S80IFEU04];
B`1ZJVX0bl_;06IVIP0R8R80;2`/09NGU`1MGEd0=SHf04E5A@2FUYH07QhN03Ti>@2GUiL0^k^k09VI
V@0208J6QP070;Zj^P2MWId0VYZJ0<O7a`3IfMT0XJ6Q02DU9@0201<C4`1;01LG5`18B4P0DU9B028R
8P0G5aL06QXJ01dM7@0J6QX0820P02<S8`0`<300Vi^K09BDU02o_kl0f][J0<71`@2UYJD0DE5A05iN
GP1MGEd0OGem0;^k^`3WinL0/[:b03Ti>@0d=3@0KFe]0:NWY`32`/80QhN7034a<@15ADD0IVIV01<C
4`0E5AD06QXJ01/K6`0G5aL061PH01hN7P1RHV80Shn?0<o?c`2`/;00Lg=c02HV9P1KFe/0M7Ad028R
8P0L71`07AdM01PH600T92@0:BTY034a<@1TI6@092@T03dm?@0g=cL08b<S020P800/;2`0Zj^[09VI
V@3Lg=`0UIFE08>3P`33`l<0][Jf07inOP0W9bL0AdM703@d=00R8R80820P00807QhN06d092@T02h^
;P1VIVH0dm?C07QhN00[:b/0Vi^K07=cL`2m_Kd0^[Zj0=3@d023Ph<0IVIV06][J`2c/k<0ZJVY0:RX
Z01TI6@0<C4a03Lg=`1UIFD0IfMW059BDP2ATI40^[Zj09bLW01kNg/0;bl_03<c<`1:BTX0:2PX04I6
AP2EUID0a/K605UIF@1bLW80Hf=S05iNGP0k>c/0?Shn01TI6@0F5QH06QXJ01TI6@0L71`06ATI03Xj
>P1PH600B4Q809>CT`2EUID0_[jn09BDU01QHF40Jf][0861P@2k^k/0^KVi05=CD`0j>SX0<c<c02d]
;@0m?Cd0Lg=c0;Vi^@2JVYX0BDU905=CD`1mOGd0KFe]05mOG`1;Bd/0DU9B03Ti>@0m?Cd0<S8b02LW
9`2;Rh/0emOG0:ZZZP0f=SH0>c/k0;^k^`2XZ:P0[Zj^0<o?c`39bLT0clo?0<?3``2>SXh0WinO0:f]
[@20P800HV9R09>CT`2QXJ40];Bd0<_;b`3>c/h0c<c<0<;2`P3Rh^80emOG0<W9b@34a<@0Rh^;0;Ng
]`2o_kl0`l?3013ooooo000>ooooo`0@07AdM02SXj<0Kfm_08b<S01QHF40MgMg07moO`1;Bd/0EEEE
08F5Q@2`/;00NWYj0<?3``1`L700Kfm_08R8R080OWin01@0ADE506i^KP2:RXX0OGem05MGE`1YJFT0
<S8b079bLP24Q8@0Cdm?06IVIP1?Cdl0H61P05QHF00j>SX0Bd];04Y:BP0e=CD0@T9203@d=080<30`
04X0IVIV04Y:BP10@400IVIV05ADE01HF5P0@d=302/[:`1PH600Bd];08Z:RP1BDU80<c<c03hn?P1I
FET0<S8b04Y:BP1XJ6P0IFEU03@d=01VIVH0EUIF04I6AP1WIfL0>SXj0451@@1^KVh0JVYZ059BDP17
AdL0CDe=07IfMP1LG5`0WYjN0:6QX@1VIVH0Bd];061PH013@d<0C4a<08V9R@1;Bd/08b<S0861P@0Q
8B40;2`/05mOG`19BDT0ATI607MgM`1RHV80CTi>05MGE`1QHF40<c<c059BDP1OGel0Lg=c03<c<`1;
Bd/0A4A404M7A`1/K6`0LG5a05=CD`15ADD0:b/[071`L013@d<0B4Q803lo?`1hN7P0LG5a04];B`80
AdM702L0Kfm_07=cL`0Y:BT0Fe]K05UIF@1VIVH0C4a<071`L01]KFd0CTi>07AdM01NGUh0?Cdm08n?
S`0e=CD0G5aL09>CT`0n?Sh0BTY:061PH02i^KT0MWIf05aLG01kNg/0LW9b05mOG`1?Cdl0XJ6Q051@
D010@400Ph>304a<C01_Kfl0OGem05MGE`1kNg/0S8b<06m_K`2=SHd00P2;Rh/0N@2c/k<0UIFE05iN
GP2o_kl0[Jf]0820P02ZZZX0ZJVY05EEE@1TI6@0d];B07inOP1jNWX0E5AD05iNGP11@D40U9BD08f=
S@1`L700Xj>S04Y:BP1FEUH0S8b<065QH@16ATH0JVYZ06e]K@2/[:`0LG5a06a/K02WYjL0EEEE07Mg
M`27QhL0>c/k061PH01=CDd092@T05YJFP2KVi/0KVi^03Xj>P0Z:RX0G5aL05ADE01QHF40AdM70;no
_`1FEUH0>c/k07EeM@1KFe/0<S8b06m_K`1<C4`0@D5108n?S`20P800HV9R05mOG`1eMGD0<c<c04=3
@`1HF5P0FUYJ06m_K`21PH40QXJ605QHF00d=3@0TI6A04Y:BP1OGel0@d=30451@@23Ph<0ATI604a<
C02NWYh0CDe=05IFEP2OWil0TY:B08B4Q02;Rh/0Shn?04Y:BP0f=SH0FUYJ07alO00W9bL0IVIV09JF
UP0T92@0;Bd]079bLP1UIFD0ATI607inOP1`L700Cdm?06YZJP1`L700@D5108>3P`0e=CD0<30`07Mg
M`0i>CT08b<S07MgM`1NGUh0>3Ph069RHP13@d<0:b/[03@d=01@D500H61P05mOG`0Y:BT03_ooool0
1`1@D500B4Q804A4A00o?cl0>SXj0451@@1GEeL00P1SHf<0S023Ph<0YjNW09nOW`20P800DU9B04E5
A@1KFe/0GUiN05aLG01PH600NGUi08V9R@2FUYH0RHV909RHV02j^[X0c/k>09NGU`2>SXh0a<C40861
P@1=CDd0DE5A05ADE01GEeL0Hf=S0;Vi^@2LW9`0JVYZ08F5Q@1TI6@0K6a/04e=C@19BDT0W9bL0=KF
eP3<c<`0/K6a0:JVYP2[Zj/0ZJVY0:FUY@2FUYH0WIfM08b<S01^KVh0=SHf01DE5@0I6AT07AdM02PX
:00c<c<0IFEU05mOG`11@D40BDU905=CD`1IFET0[Jf]07QhN00l?3`0@d=302/[:`0i>CT0IFEU04Q8
B01DE5@0b/[:0:6QX@19BDT092@T01`L700M7Ad092@T0;Rh^01@D500>c/k07YjNP2@T900JVYZ055A
D@0D51@04A4A01DE5@0T92@0];Bd08j>SP1]KFd0GUiN05aLG01IFET0De=C05eMG@1dM7@0G5aL04m?
C`2JVYX0e=CD0;Bd]03Ogml0YZJV05YJFP14A4@0>CTi03`l?01_Kfl0aLG50492@P0`<300@d=3059B
DP20P800L71`03Xj>P14A4@0P820028R8P0B4Q805QHF01XJ6P0H61P0=CDe059BDP2<S8`0/k>c07in
OP1`L700PX:206IVIP0K6a/0>c/k06MWI`1@D500:2PX01hN7P0K6a/08R8R03Xj>P1XJ6P0EeMG0P0N
7Qh0>`0O7al0=CDe05MGE`1RHV80QXJ60451@@1iNGT0ZZZZ09nOW`2?Shl0VIVI071`L01@D500R8R8
06ATI01KFe/0ATI602l_;`0U9BD0<30`07=cL`1iNGT0EUIF06][J`3Hf=P0a/K60;fm_@2ZZZX0d];B
0<;2`P2@T900`L7109VIV@1fMWH0@D5102PX:00M7Ad05aLG01HF5P0T92@0GUiN0::RXP1^KVh0Zj^[
08F5Q@16ATH0Kfm_05QHF01EEED0Fe]K05EEE@1iNGT0HV9R05aLG01VIVH0BDU9038b<P17AdL071`L
00806a/K048061PH01/K6`0L71`0=CDe024Q8@0i>CT0Ti>C09RHV02IVIT0R8R807QhN025QHD0YjNW
0<K6aP3GemL0^;Rh08>3P`1aLG40TI6A09nOW`2TY:@0Vi^K06UYJ@1eMGD0Rh^;09>CT`1/K6`0H61P
059BDP12@T80=SHf03@d=00l?3`0>SXj06e]K@3Fe]H0][Jf08j>SP2JVYX0Rh^;092@T03Lg=`0bl_;
0;:b/P3De=@0KVi^04=3@`1CDe<0G5aL071`L01fMWH0V9RH09bLW02[Zj/0]KFe0;6a/@2UYJD0MWIf
0<K6aP2i^KT0];Bd08b<S02TY:@0WIfM04M7A`1GEeL@ooooo`003_ooool0G01cLg<0UYJF05UIF@2R
XZ80Cdm?05QHF01oOgl0AdM706i^KP1TI6@0P82006ATI02;Rh/0<30`05MGE`2:RXX0M7Ad07QhN00j
>SX0EUIF08>3P`1WIfL0EeMG06QXJ00U9BD0HF5Q07UiN@1`L700RHV904e=C@1SHf<0CDe=03Lg=`1H
F5P0ATI6030`<016ATH0?clo02HV9P0R8R80CDe=03Ti>@0j>SX0JVYZ034a<@1JFUX0>3Ph034a<@1P
H600<S8b07UiN@1EEED0<S8b04I6AP1ADE40820P03hn?P1PH600F5QH03Xj>P21PH40EEEE03Ti>@14
A4@08R8R0492@P1IFET0=CDe04M7A`18B4P0FEUI06e]K@0h>3P0O7al0:>SX`1XJ6P0F5QH075aL@0c
<c<0@T9207UiN@16ATH0@D51079bLP0c<c<07QhN059BDP17AdL0?Cdm07alO01WIfL0GUiN0P1@D500
6P0];Bd0Bd];04i>CP1PH6009RHV04a<C016ATH0<S8b061PH01=CDd0O7al04U9B@0`<300K6a/03`l
?01DE5@0:RXZ06UYJ@1KFe/0DU9B05UIF@1FEUH0GEeM05aLG00];Bd0IVIV0P1LG5`0W00l?3`0Q8B4
04A4A015ADD0Ng]k05mOG`1<C4`0R8R803/k>`1iNGT0OWin04=3@`1/K6`0HF5Q0::RXP1MGEd0?clo
07moO`2DU9@0LW9b04e=C@1lO7`0=3@d04i>CP2LW9`0=cLg05mOG`1LG5`0KFe]08>3P`15ADD0KFe]
06QXJ01TI6@0T92@0;Fe]@1[Jf/0AdM708:2PP20P800K6a/0<O7a`2g]kL0F5QH05UIF@3Bd]80FUYJ
08f=S@2BTY80De=C0451@@2<S8`0IFEU07MgM`26QXH0=CDe069RHP2@T900DU9B04=3@`1dM7@0NGUi
09:BTP1FEUH0GEeM0;2`/01=CDd0LW9b05eMG@0j>SX0FUYJ04a<C00Q8B40<c<c05eMG@1YJFT0EeMG
03Hf=P14A4@0?Cdm03`l?011@D40/[:b07moO`19BDT0SXj>05mOG`1FEUH0RXZ:06YZJP1[Jf/0R8R8
06=SH`1@D500PX:2079bLP0b<S80ADE505iNGP1WIfL0HV9R05mOG`1VIVH0Lg=c04E5A@2i^KT0Cdm?
09jNWP1]KFd0ATI605MGE`1=CDd0KFe]09ZJVP1SHf<0M7Ad07MgM`1HF5P0HV9R09jNWP2a/K40Kfm_
03`l?01JFUX0OWin02<S8`1DE5@0Ng]k01dM7@1EEED0^;Rh07QhN00c<c<0DE5A03dm?@0R8R80Shn?
07UiN@0S8b<0Kfm_02DU9@1GEeL0VYZJ05IFEP0O7al0KFe]051@D01BDU80^[Zj07]kN`0k>c/09BDU
06EUI@1PH600EUIF02d];@kooooo00D0D51@04M7A`0k>c/0:b/[02@T9003024Q8@0K02PX:02DU9@0
/K6a0<C4a02XZ:P0GEeM051@D022PX80Ng]k06MWI`1EEED0Bd];05aLG02a/K40gMgM09RHV02EUID0
VYZJ04E5A@21PH40PX:207moO`0f=SH09RHV01lO7`0/;2`0@D510080Q8B401D0>c/k065QH@1dM7@0
9bLW01dM7@0f=SH0Xj>S04a<C010@400IfMW0:ZZZP2o_kl0]KFe0;:b/P2LW9`0WYjN0:j^[P2SXj<0
M7Ad03lo?`0L71`00P0I6AT02`0J6QX0C4a<05iNGP0N7Qh06a/K0410@01:BTX0U9BD07IfMP1RHV80
5aLG00805ADE04`0:RXZ08R8R02_[jl0a/K603lo?`0J6QX05QHF01LG5`0F5QH0<C4a092@T00[:b/0
9bLW06QXJ01RHV80B4Q804e=C@0l?3`08R8R05eMG@1`L700=CDe024Q8@0X:2P092@T02HV9P0];Bd0
>SXj07=cL`1lO7`0^;Rh0;jn_P3Jf]X0fm_K0:n_[`3Ogml0[jn_02DU9@0I6AT05aLG01<C4`0A4A40
9bLW06e]K@0Q8B4061PH01LG5`0m?Cd0H61P04M7A`0a<C40HV9R06i^KP0L71`0820P02LW9`1@D500
C4a<0410@02FUYH0VYZJ05eMG@1FEUH0JVYZ07IfMP1WIfL092@T01hN7P1?Cdl0Jf][03Ti>@0o?cl0
CDe=0492@P11@D40Hf=S0P0W9bL02@13@d<0E5AD075aL@1JFUX0KVi^0492@P0W9bL0RXZ:0<W9b@02
0:b/[00a096AT@1LG5`0:2PX02PX:013@d<0PH6107alO01OGel0Ogmo08^;R`2][Jd0_Kfm0=7Ad@1S
Hf<0];Bd0;Rh^035aLD0hN7Q0=gMg@3Ogml0WIfM02DU9@0R8R80>CTi04A4A00H61P05aLG01HF5P0I
6AT09RHV07UiN@27QhL0H61P0:6QX@0V9RH092@T03De=@15ADD0>c/k02d];@1@D500FEUI05ADE01U
IFD0<c<c03Ti>@11@D406ATI01`L700201XJ6P0/01`L700O7al0=SHf02DU9@1RHV80VYZJ05YJFP2@
T900Jf][04E5A@1CDe<0R8R808j>SP1IFET0YJFU0:VYZ@3De=@0YjNW0;:b/P2d];@0L71`08B4Q02@
T900IVIV028R8P0Y:BT0;Bd]04E5A@1<C4`0Bd];04Y:BP1;Bd/0HF5Q0<;2`P2QXJ40ADE506YZJP2`
/;00Xj>S08j>SP2o_kl0QhN708>3P`2EUID201LG5`8061PH0140>3Ph08B4Q01iNGT0Hf=S075aL@2?
Shl0_[jn0;:b/P3GemL0Ti>C0<K6aP1OGel0Cdm?06i^KP2CTi<0ADE502h^;P0@ooooo`003_ooool0
EP1[Jf/0XJ6Q05=CD`1jNWX0GUiN05QHF00n?Sh0GUiN08J6QP1eMGD0CTi>06e]K@2NWYh0ATI606YZ
JP1;Bd/0KVi^079bLP0j>SX0Jf][04=3@`0`<300J6QX06m_K`0X:2P0HF5Q02TY:@17AdL0KFe]0410
@01cLg<0<30`0492@P1hN7P0I6AT04M7A`13@d<0CTi>03@d=00Z:RX0DE5A014A4@0];Bd0M7Ad0410
@01SHf<06ATI04E5A@2<S8`0D51@08^;R`0U9BD0:b/[07UiN@20P800<S8b03hn?P0g=cL05aLG0492
@P2RXZ80OGem0492@P13@d<05ADE05mOG`29RHT0I6AT055AD@0[:b/0EEEE08N7Q`1PH600W9bL0;Jf
]P0];Bd0AdM708J6QP11@D40?Shn07EeM@0@4100<c<c06IVIP0o?cl08B4Q0P1JFUX0100E5AD0G5aL
08j>SP1OGel205=CD`2h02h^;P1dM7@0E5AD03<c<`0Z:RX0Kfm_07=cL`0k>c/0AdM703Hf=P1fMWH0
M7Ad0492@P1>CTh061PH071`L00f=SH0L71`01@D500d=3@0EUIF061PH01NGUh0>SXj02<S8`1kNg/0
NWYj05MGE`0c<c<0T92@07MgM`1IFET0G5aL024Q8@1FEUH0[jn_0451@@2=SHd0IfMW01lO7`1LG5`0
IFEU0:FUY@1LG5`08R8R05MGE`1;Bd/0De=C05aLG01bLW8071`L03Lg=`2:RXX0EeMG06=SH`0K6a/0
@d=309FEU@1iNGT0Ti>C06YZJP0Y:BT0@D5109^KV`2GUiL0E5AD08N7Q`1lO7`0@41008N7Q`1lO7`0
Bd];08V9R@3Kfm/0LW9b051@D019BDT0Hf=S0492@P2JVYX0BDU908B4Q026QXH0BDU904E5A@27QhL0
DE5A04e=C@1nOWh0VYZJ09VIV@1<C4`0K6a/0<C4a01BDU80OWin04M7A`11@D40RXZ:05QHF00_;bl0
>CTi06=SH`1UIFD0BDU904Y:BP1<C4`0D51@04Q8B00V9RH0NGUi06IVIP12@T80Kfm_0451@@0i>CT0
N7Qh06=SH`1NGUh0Kfm_05iNGP1?Cdl0H61P05eMG@0[:b/0FUYJ08Z:RP1VIVH0IFEU069RHP1CDe<0
DE5A065QH@2g]kL0>3Ph05EEE@1:BTX0K6a/08>3P`0Z:RX0Bd];07emO@1KFe/0S8b<0;bl_02:RXX0
?clo08B4Q01JFUX0@T9205QHF023Ph<0Xj>S04i>CP1RHV80MWIf01<C4`0o?cl0VYZJ07YjNP0g=cL0
EEEE0492@P0L71`0GemO08:2PP10@400L71`01DE5@0n?Sh0TY:B059BDP0I6AT0O7al03`l?00a<C40
IfMW06][J`1`L700ATI603lo?`0f=SH06QXJ03Ph>0kooooo04h0C4a<03dm?@0e=CD0<C4a02LW9`0V
9RH0820P01dM7@0K6a/0QXJ604Y:BP1iNGT0^[Zj0=CDe02`/;00B4Q804a<C01UIFD0S8b<0<[:bP34
a<@0T92@0:RXZ01nOWh0Vi^K0;fm_@2:RXX0=cLg01LG5`0g=cL0LG5a02/[:`0O7al071`L01TI6@0Y
:BT0S8b<03Ti>@0_;bl0KFe]02d];@1LG5`0V9RH0820P00f=SH0=cLg03Xj>P1oOgl0JVYZ0::RXP32
`/80YZJV092@T028R8P0AdM703Hf=P1WIfL0Kfm_05MGE`0f=SH0:2PX03Xj>P1KFe/07AdM01/K6`0/
;2`0Bd];09JFUP1fMWH0;2`/02LW9`0E5AD05aLG02TY:@31`L40b<S809bLW00i>CT201DE5@0701XJ
6P0^;Rh0QHF50<S8b01RHV80=3@d06YZJP0206UYJ@0704Y:BP2d];@0XJ6Q04i>CP0M7Ad05QHF01@D
500201DE5@0@01/K6`0J6QX05aLG01/K6`0n?Sh0AdM704a<C02WYjL0fm_K08b<S02m_Kd0`l?302d]
;@0I6AT05ADE01XJ6P805ADE0180IFEU059BDP0K6a/05ADE02PX:01EEED0C4a<034a<@16ATH0^;Rh
07MgM`1MGEd0FEUI03Lg=`1?Cdl0EEEE0=;BdP2=SHd205eMG@190:b/[02FUYH0[Zj^06m_K`0m?Cd0
;2`/07MgM`1eMGD0>c/k024Q8@0K6a/09bLW05QHF026QXH0HV9R06YZJP1`L700>3Ph04E5A@0I6AT0
EUIF04e=C@28R8P0NGUi06UYJ@2>SXh0SHf=06e]K@1aLG40>3Ph02@T900k>c/0R8R806ATI02OWil0
[jn_0<K6aP37alL0LG5a04U9B@2GUiL0H61P04a<C02o_kl0bl_;0<o?c`2>SXh0=SHf02d];@0O7al0
C4a<01LG5`0M7Ad0;Bd]02LW9`0m?Cd0O7al0:BTY00i>CT0Ti>C04M7A`11@D40=SHf030`<00j>SX0
<S8b04m?C`1UIFD0JVYZ055AD@0c<c<0<30`0410@00201lO7`8071`L04407AdM03Lg=`12@T80>c/k
08>3P`2LW9`0C4a<05eMG@1RHV80BTY:04i>CP1LG5`0[jn_0:b/[01SHf<0e]KF0<C4a036a/H0f][J
0;Rh^02VYZH0XZ:R06i^KP0c<c<071`L028R8P0X:2P09bLW04I6AP12@T80AdM7055AD@2YZJT0a<C4
079bLP1>CTh0D51@09JFUP3;bl/0U9BD0=?Cd`31`L40`l?307emO@0E5AD05aLG01PH600F5QH06QXJ
05EEE@0m?Cd08b<S02LW9`0g=cL0IfMW075aL@2^[Zh0BTY:08J6QP0g=cL0B4Q807moO`21PH40KFe]
05YJFP0@ooooo`003_ooool0>01aLG40/K6a05aLG021PH40S8b<09>CT`1cLg<0?Shn065QH@1lO7`0
KFe]055AD@2OWil0ADE507alO01oOgl0JVYZ08j>SP12@T80KVi^08F5Q@1RHV80E5AD06IVIP0^;Rh0
QHF507EeM@1JFUX0JVYZ04];B`21PH40J6QX03Hf=P19BDT0EEEE03Ph>012@T80?Cdm02h^;P0V9RH0
ATI6030`<00[:b/0H61P04=3@`1_Kfl0<c<c02h^;P1LG5`0BDU908B4Q00a<C4071`L03lo?`1HF5P0
9BDU0P0m?Cd0g00`<300?3`l08R8R01MGEd0>SXj03Ph>00I6AT0<S8b05]KF`1<C4`0BTY:02d];@13
@d<0NWYj06=SH`2QXJ40Vi^K051@D00i>CT0IVIV03Ph>014A4@0U9BD0492@P13@d<0EEEE02TY:@0M
7Ad0G5aL04e=C@1>CTh0Kfm_06YZJP1BDU80D51@05UIF@0`<300F5QH05EEE@1PH600:BTY04E5A@1A
DE40=3@d06ATI01DE5@0Fe]K065QH@0b<S80KVi^02/[:`1IFET0>CTi07IfMP1RHV80<S8b04A4A01K
Fe/0JFUY06=SH`0_;bl0HF5Q06EUI@1CDe<0>3Ph08Z:RP1RHV80GemO055AD@0g=cL0A4A408n?S`0[
:b/0N7Qh05iNGP0a<C40De=C04Y:BP2WYjL0Kfm_065QH@1oOgl0EeMG04=3@`1UIFD0Ph>304A4A00k
>c/0QHF50410@01cLg<0HF5Q04Q8B02<S8`0LW9b09JFUP1jNWX0MWIf05QHF02c/k<0S8b<069RHP2>
SXh0SHf=07YjNP2SXj<0Ogmo02<S8`1WIfL0bl_;06a/K01MGEd0J6QX06a/K01>CTh0XZ:R07=cL`1I
FET0Rh^;04E5A@11@D40EUIF03De=@1RHV80OWin0:BTY02=SHd0>c/k05=CD`2OWil0Bd];08^;R`1F
EUH09RHV055AD@1UIFD0@T9203`l?01]KFd0L71`01lO7`0];Bd0HV9R06IVIP1ZJVX0<30`05MGE`1I
FET0ADE50861P@1RHV80?Shn08J6QP1bLW80MgMg096AT@1JFUX0EUIF06UYJ@1<C4`06QXJ0410@01L
G5`0KFe]0861P@2HV9P0@d=302TY:@16ATH0XJ6Q04U9B@1cLg<092@T02h^;P1=CDd0AdM705]KF`1[
Jf/08B4Q04U9B@1@D500J6QX07]kN`2CTi<0F5QH04E5A@0a<C40B4Q809FEU@1LG5`0LW9b0:VYZ@0P
8200?3`l0:b/[01gMgL0GUiN09^KV`1jNWX0?clo05iNGP1kNg/0?Cdm0820P00m?Cd0;bl_0:NWY`10
@4007alO08B4Q01QHF40<C4a07AdM01<C4`0=cLg03dm?@28R8P0C4a<034a<@0V9RH>ooooo`0D04I6
AP10@400=CDe03De=@0^;Rh0:BTY020P800O7al0;2`/0;Fe]@28R8P0QHF507moO`33`l<0>3Ph01/K
6`0E5AD05aLG04m?C`2k^k/20:n_[`2Q0:2PX02k^k/0Q8B407emO@1YJFT05ADE014A4@0C4a<09BDU
04I6AP0o?cl0>3Ph02/[:`0K6a/0G5aL08N7Q`0c<c<0OGem05QHF02EUID0Q8B4051@D00L71`07AdM
01lO7`1:BTX0J6QX07EeM@2CTi<0Z:RX0;bl_038b<P0X:2P04m?C`0n?Sh0DE5A06IVIP1JFUX0Fe]K
08B4Q02>SXh0FUYJ03dm?@14A4@0XJ6Q0;jn_P20P8005aLG0451@@0Y:BT07AdM08B4Q02MWId0NGUi
092@T02DU9@0>CTi0451@@15ADD0KFe]05IFEP1EEED0Xj>S07QhN01ZJVX0SHf=05aLG014A4@0c/k>
06IVIP0N7Qh06ATI01DE5@0Q8B405ADE01PH600E5AD05QHF01LG5`0M7Ad0?Cdm06IVIP2=SHd0_;bl
0=WIf@39bLT0b<S80;bl_02VYZH0J6QX04Q8B00];Bd061PH01DE5@0];Bd0Ph>303De=@0G5aL06a/K
06ATI017AdL0Fe]K05MGE`3Bd]80De=C04Q8B015ADD0FEUI08J6QP2LW9`0h>3P0<30`02d];@0MWIf
0:BTY038b<P0RXZ:071`L01UIFD0QHF509JFUP2HV9P0OWin05EEE@1HF5P0De=C06MWI`3=cLd0]KFe
05eMG@0^;Rh0A4A401lO7`0C4a<05aLG05QHF02:RXX0EeMG061PH02:RXX0NGUi08n?S`1[Jf/0M7Ad
059BDP0e=CD0W9bL06i^KP2TY:@0_;bl0<71`@1RHV80Hf=S0861P@1>CTh09BDU02<S8`1JFUX0[Jf]
0;jn_P1lO7`00P0g=cL06P0P8200BTY:03Hf=P0e=CD0De=C04M7A`0m?Cd0>CTi09BDU01KFe/0PX:2
07IfMP1dM7@0VIVI038b<P1:BTX0BDU906ATI01dM7@0OGem0492@P0l?3`0>CTi04i>CP0Q8B408R8R
0P0O7al0;@0R8R80BTY:02h^;P1NGUh0X:2P07emO@1>CTh0D51@02LW9`0_;bl0A4A405iNGP2>SXh0
_kno09ZJVP2d];@0_[jn0<O7a`32`/80YJFU0<O7a`2n_[h0>3Ph01`L700N7Qh08B4Q02PX:00c<c<0
<C4a05mOG`1<C4`0QhN709ZJVP2XZ:P0/K6a04Y:BP1ADE40T92@0<?3``39bLT0WIfM0;bl_03Kfm/0
NGUi01LG5`0201TI6@0301hN7P0`<300FUYJ0080;2`/00d0:BTY03@d=01PH600JVYZ069RHP0k>c/0
GemO02PX:00/;2`0Fe]K04I6AP14A4@0Fe]K013ooooo003oooooooooooooF_ooool00?oooooooooo
oomJooooo`00oooooooooooooe[ooooo003oooooooooooooF_ooool00?oooooooooooomJooooo`00
oooooooooooooe[ooooo000>ooooo`0:06a/K01FEUH0EUIF07AdM014A4@0AdM705UIF@26QXH0FEUI
0410@080Cdm?00l0IFEU08f=S@0a<C40@D5104e=C@1/K6`0[:b/06MWI`0N7Qh0DE5A05]KF`1OGel0
[jn_02TY:@0O7al00P19BDT0H@1lO7`0>SXj02@T901=CDd0Ng]k05QHF02FUYH0Fe]K030`<01FEUH0
FEUI06=SH`23Ph<0Q8B403Ti>@1@D500S8b<071`L023Ph<0?Shn04i>CP2UYJD0BTY:05MGE`1RHV80
6a/K03Lg=`1SHf<0ADE504m?C`0b<S807alO069RHP1UIFD0?Shn05iNGP0X:2P0?3`l09JFUP2ATI40
:b/[0410@00R8R80Fe]K0;bl_01ADE409RHV03Hf=P0c<c<0Jf][0;Ng]`0k>c/0:b/[05IFEP0n?Sh0
TY:B0>3Ph00g=cL071`L07IfMP1MGEd0Zj^[07=cL`0O7al09BDU06MWI`1OGel0R8R8059BDP0O7al0
9RHV05]KF`1BDU80G5aL01HF5P0];Bd0Hf=S06a/K00/;2`0?Shn04I6AP0e=CD0GUiN06ATI00];Bd0
>SXj03@d=01CDe<0Ph>307IfMP0b<S80Fe]K04];B`1DE5@0WYjN05aLG00b<S800P1GEeL0O@0m?Cd0
U9BD02/[:`0/;2`0KFe]04U9B@1IFET0JFUY01/K6`14A4@0HF5Q05eMG@1fMWH0`/;202/[:`18B4P0
G5aL03/k>`2;Rh/0bl_;030`<00l?3`0LG5a05QHF02m_Kd0YJFU03dm?@15ADD0J6QX04a<C02MWId0
N7Qh020P801?Cdl0MWIf07MgM`2BTY80@T9202@T901JFUX0MgMg0861P@28R8P0<c<c0492@P1BDU80
XZ:R08^;R`1]KFd0@41007=cL`23Ph<0?3`l05IFEP1JFUX0=cLg09VIV@34a<@0EUIF05=CD`10@400
G5aL0:>SX`1@D500=3@d055AD@1?Cdl0R8R807emO@0f=SH0EeMG04U9B@1IFET0RHV903@d=01<C4`0
Fe]K04Q8B02XZ:P0PH6103De=@20P800IfMW08b<S01WIfL09BDU02PX:01hN7P0LW9b0:BTY00/;2`0
<30`065QH@1`L700^KVi06MWI`11@D40Cdm?08>3P`1mOGd0HF5Q03Xj>P12@T80De=C092@T01VIVH0
DU9B02PX:010@400BDU907YjNP1:BTX0=cLg04A4A00n?Sh0C4a<075aL@1XJ6P0JVYZ06][J`0Y:BT0
B4Q809>CT`0n?Sh0>c/k0080BTY:01L0J6QX04m?C`0N7Qh0ATI604i>CP0i>CT0H61P04U9B@0I6AT0
=3@d03Xj>P1FEUH0Bd];06YZJP0M7Ad0;bl_0451@@1LG5`0HF5Q075aL@0V9RH0>CTi05ADE00>oooo
o`0N05iNGP1JFUX0C4a<05MGE`1TI6@0E5AD03Hf=P1ADE40E5AD055AD@1MGEd0FEUI04i>CP1=CDd0
MgMg06i^KP1JFUX0GUiN04U9B@0S8b<0CTi>03Ph>00l?3`0E5AD03`l?00k>c/0DE5A04];B`1IFET0
BDU90P10@4001P1@D500C4a<051@D018B4P0GUiN02LW9`80;Rh^00/0=CDe0451@@0i>CT0ATI603De
=@0m?Cd0BTY:02h^;P0U9BD0>SXj04Y:BP0203De=@2n04a<C01QHF40PH6104=3@`1>CTh0<c<c03@d
=01=CDd0>SXj04A4A01GEeL0Lg=c06=SH`0m?Cd0CDe=05]KF`1WIfL0De=C06=SH`15ADD071`L01lO
7`0m?Cd0CTi>03Ph>012@T80EEEE069RHP1BDU80CTi>05=CD`1EEED09bLW04a<C00o?cl0<C4a0451
@@14A4@0@d=303<c<`1=CDd0@d=304];B`0l?3`0EeMG04E5A@10@400ATI602DU9@0h>3P0EUIF05MG
E`1XJ6P0GEeM038b<P1<C4`0?3`l05ADE00a<C40EEEE04A4A00m?Cd0E5AD05=CD`0_;bl0>c/k0492
@P0n?Sh0Kfm_05IFEP2YZJT0Lg=c038b<P1OGel0=CDe06ATI01>CTh0T92@05mOG`0a<C40QHF50451
@@1;Bd/0F5QH01dM7@1<C4`092@T02`/;01;Bd/0BTY:0;no_`1^KVh0Hf=S04e=C@2GUiL0@41005eM
G@1;Bd/08B4Q05EEE@0_;bl0820P05UIF@0h>3P0ATI601/K6`1;Bd/0<30`03<c<`0`<300AdM703Hf
=P0_;bl0J6QX04E5A@2GUiL0FEUI06QXJ01VIVH0>SXj05eMG@0d=3@0IFEU08n?S`0g=cL0DE5A05=C
D`0Z:RX0@T9204];B`1MGEd0ATI609bLW02GUiL0CDe=05ADE00/;2`0De=C01@D5019BDT0Bd];05iN
GP1CDe<0D51@05iNGP2LW9`0EEEE05iNGP0n?Sh08R8R05mOG`0A4A40EUIF02LW9`1HF5P0@D5103Lg
=`1IFET0EEEE05mOG`1KFe/0Bd];05IFEP21PH40JVYZ04a<C01TI6@092@T06=SH`16ATH0IfMW03De
=@1gMgL0D51@03Ti>@18B4P0D51@04U9B@1GEeL0HF5Q03lo?`0_;bl0J6QX02l_;`0Y:BT0=cLg01lO
7`1KFe/0CTi>06QXJ080FEUI00<0De=C03Ti>@1DE5@00P1UIFD0701MGEd0=cLg055AD@1CDe<0=3@d
014A4@2:RXX0`/;207QhN01]KFd0@d=304];B`0/;2`0=SHf04Q8B015ADD0IFEU030`<01ADE40OWin
08Z:RP1lO7`0IFEU05UIF@1BDU80IVIV05MGE`1/K6`@ooooo`003_ooool0O`2c/k<0DE5A04m?C`1`
L700V9RH04Q8B016ATH0OWin08>3P`2RXZ80@D5103Xj>P1SHf<0IFEU065QH@1kNg/0BDU905mOG`1G
EeL0KVi^05YJFP1dM7@0:b/[038b<P1lO7`0>3Ph03Lg=`0d=3@0820P051@D018B4P0QXJ6065QH@0S
8b<0:BTY03lo?`1kNg/0;Rh^06i^KP0S8b<0:b/[08:2PP2][Jd0?clo05=CD`0d=3@0EUIF0;>c/`1j
NWX0>3Ph07]kN`0j>SX0Q8B408n?S`0U9BD08R8R059BDP1KFe/0NWYj05ADE00D51@0=cLg059BDP21
PH40WinO024Q8@0e=CD0EeMG09>CT`1[Jf/0M7Ad01TI6@0Z:RX0LW9b08J6QP1ADE40KFe]01lO7`0c
<c<0IVIV09BDU01RHV80De=C01lO7`11@D40N7Qh08:2PP1LG5`0Lg=c038b<P1dM7@0IfMW04=3@`1=
CDd0QXJ603hn?P1mOGd0O7al02/[:`10@400Cdm?0492@P22PX80@d=302PX:017AdL0HF5Q05IFEP1i
NGT0CTi>02`/;01:BTX0Jf][05IFEP2QXJ40?Cdm04U9B@1TI6@0N7Qh069RHP2QXJ40Q8B404I6AP1U
IFD0QXJ606=SH`2m_Kd00P15ADD0<`1_Kfl0KVi^061PH01UIFD08B4Q03Lg=`1kNg/0A4A4071`L01<
C4`0;Rh^04M7A`2LW9`0LG5a07=cL`0];Bd0<c<c03hn?P21PH40BDU906a/K00`<300;2`/05aLG01k
Ng/0BDU904E5A@0^;Rh0=SHf05eMG@22PX80@41003Xj>P1=CDd0AdM70:2PX01nOWh0<30`05QHF00i
>CT0N7Qh0;Bd]019BDT0<30`05EEE@1@D500Ogmo0;no_`0g=cL0BTY:08j>SP0205UIF@0e07=cL`0Q
8B40C4a<09:BTP1NGUh0P82005IFEP1HF5P0Jf][09ZJVP1_Kfl0`<3003Ti>@1^KVh0TY:B05]KF`1Y
JFT0BDU902/[:`25QHD0De=C07=cL`0m?Cd0BDU904e=C@1cLg<0MGEe092@T00i>CT0UIFE08f=S@1m
OGd0Bd];071`L01DE5@0[:b/06m_K`1>CTh0H61P03hn?P1iNGT0YZJV04i>CP1CDe<0QHF505MGE`1V
IVH0F5QH03<c<`1_Kfl0IfMW05ADE02QXJ400P0j>SX08`1LG5`0Q8B406a/K022PX80JVYZ03`l?013
@d<0De=C0492@P2KVi/0/[:b034a<@0l?3`0O7al065QH@1mOGd0R8R803@d=01QHF40EeMG03Ph>01j
NWX0SXj>02LW9`0o?cl0LG5a03dm?@1EEED0QhN703<c<`0];Bd0CDe=038b<P1>CTh0Jf][0080<c<c
00@0Bd];05ADE019BDT0Q8B43_ooool0@`18B4P0HV9R02/[:`0f=SH0F5QH06][J`1RHV80G5aL05EE
E@0l?3`0Cdm?05mOG`1IFET0IVIV08B4Q027QhL0GemO05aLG01;Bd/05QHF02`/;00K6a/0<c<c04i>
CP1MGEd0DE5A07YjNP1WIfL0HV9R05IFEP1RHV80HF5Q06i^KP1VIVH0O7al075aL@1RHV80=SHf02l_
;`18B4P0>SXj05EEE@0m?Cd0Cdm?03De=@0l?3`0CTi>03Xj>P0b<S80:RXZ03Lg=`0R8R807alO03<c
<`0g=cL0?Cdm02@T900R8R80;Bd]02l_;`0a<C40<c<c04E5A@1:BTX0Fe]K04i>CP0k>c/00P0h>3P0
CP14A4@07QhN02`/;01BDU8092@T02XZ:P1SHf<0D51@04m?C`1:BTX0?Cdm03De=@0l?3`0>3Ph02LW
9`0Z:RX08b<S03@d=00a<C40:2PX03<c<`0T92@0;Bd]028R8P0_;bl0=cLg03Ph>00o?cl0Bd];03<c
<`13@d<0C4a<01lO7`0W9bL0IVIV05=CD`0j>SX0D51@04i>CP1[Jf/0Cdm?0410@01WIfL0TY:B05UI
F@1`L700GemO07UiN@0_;bl0EeMG05ADE01OGel0KVi^06MWI`2SXj<0Ti>C0492@P1XJ6P0F5QH096A
T@1WIfL0M7Ad04i>CP1=CDd0Q8B403<c<`1ADE40P82005IFEP1:BTX0AdM708j>SP1JFUX0J6QX075a
L@1FEUH0De=C03hn?P80CDe=0400G5aL04e=C@0e=CD0GEeM0492@P0@4100KVi^03Ti>@1KFe/0<C4a
0492@P0/;2`0=cLg0410@01EEED0Bd];030`<020P800EEEE08>3P`0/;2`0ATI6038b<P13@d<0KVi^
04=3@`1LG5`0V9RH061PH01;Bd/0B4Q80492@P1ADE40@D5104i>CP0b<S80M7Ad07UiN@1:BTX0ADE5
0492@P1JFUX0ATI605MGE`0i>CT0LW9b04=3@`1DE5@0EEEE0::RXP1^KVh0PH6105YJFP0`<300KVi^
01lO7`1<C4`0Cdm?05ADE01?Cdl0BTY:05UIF@1UIFD0H61P0P1EEED0?`1bLW80U9BD06UYJ@1MGEd0
W9bL04i>CP1RHV80>3Ph04a<C00/;2`0N7Qh03lo?`10@400D51@059BDP1GEeL0@d=3051@D01:BTX0
BDU904e=C@0k>c/0GUiN06i^KP0c<c<0R8R805=CD`1cLg<05ADE02DU9@19BDT0=CDe04A4A01ADE40
C4a<07MgM`1:BTX0KFe]04m?C`0e=CD051@D05]KF`2k^k/0WYjN08^;R`15ADD0Bd];03<c<`1]KFd0
Lg=c03Ti>@13@d<0<c<c05aLG01SHf<0FUYJ0492@P1EEED0IfMW05QHF01EEED0P82006UYJ@0@oooo
o`003_ooool09`1hN7P0M7Ad05YJFP1OGel0PH6107IfMP16ATH0LG5a03dm?@1@D500P8200492@P1`
L700EEEE03Lg=`1NGUh0IFEU02`/;01=CDd0=cLg03hn?P2?Shl0BTY:03Hf=P1KFe/0Hf=S08>3P`1=
CDd0820P03lo?`14A4@0MGEe09bLW00R8R80=SHf04I6AP1aLG40FEUI09RHV00204M7A`3102l_;`1@
D500Lg=c0:b/[00L71`0=CDe04e=C@1aLG40I6AT05YJFP0Z:RX0De=C059BDP0l?3`0@T9205iNGP0m
?Cd0BTY:069RHP0m?Cd0BTY:03`l?00i>CT0SHf=07=cL`0S8b<0@410030`<01:BTX0F5QH02h^;P0X
:2P0NGUi03<c<`0m?Cd0M7Ad01TI6@0V9RH0=SHf04A4A01BDU80PX:201`L700j>SX0BDU905QHF01^
KVh0f=SH03@d=01PH600W9bL06QXJ02=SHd0d];B03Ph>01ZJVX0QHF5075aL@1/K6`0NGUi01lO7`1l
O7`0Bd];05mOG`1nOWh08B4Q03Ti>@1HF5P0IFEU06ATI027QhL05ADE030`<013@d<0F5QH055AD@25
QHD07QhN02LW9`1EEED0[Jf]051@D01[Jf/07QhN03Hf=P1fMWH0TY:B02l_;`1?Cdl09BDU0451@@2:
RXX0>c/k02`/;018B4P0@D5107alO02:RXX0;2`/03@d=01SHf<0Bd];09jNWP1RHV80;bl_04m?C`1G
EeL0EEEE0;>c/`1ADE40:b/[075aL@1_Kfl0GEeM08j>SP0Z:RX0=SHf06YZJP1RHV80A4A409ZJVP1F
EUH0?Cdm06ATI02ATI40Fe]K0;Rh^01>CTh0AdM708>3P`2BTY80F5QH0::RXP0e=CD0<C4a07EeM@1J
FUX0EeMG06MWI`0W9bL0C4a<06=SH`19BDT0LW9b03dm?@0g=cL0GUiN04i>CP1=CDd0G5aL03Ti>@13
@d<0IVIV06ATI01?Cdl0A4A408^;R`2LW9`0HF5Q03Ph>00n?Sh0@D510;>c/`25QHD0>SXj05EEE@1;
Bd/0GUiN0;fm_@0o?cl0>3Ph06MWI`1UIFD0I6AT08f=S@0[:b/0LG5a06MWI`1IFET0Shn?04A4A015
ADD0RXZ:08^;R`24Q8@0_[jn04Q8B01@D500KFe]05]KF`2k^k/0U9BD038b<P0205iNGP0Z07moO`2Y
ZJT0ADE5038b<P15ADD0O7al05]KF`1gMgL0A4A403/k>`1PH600Ti>C061PH01[Jf/0?Shn02d];@1D
E5@0YZJV06YZJP1cLg<0ADE502<S8`1mOGd0^[Zj071`L01=CDd0<C4a03Xj>P1OGel0UiNG04U9B@0^
;Rh0DU9B03lo?`1HF5P0RXZ:030`<00Z:RX0CTi>03dm?@1HF5P0X:2P3_ooool0;01GEeL0RXZ:03hn
?P1=CDd0Bd];04a<C00j>SX0IFEU06ATI011@D40HV9R09NGU`2@T900NGUi05]KF`1BDU80I6AT05QH
F01FEUH0;Rh^03<c<`0a<C40=SHf03hn?P11@D40EUIF05ADE01JFUX0Lg=c04I6AP1MGEd0JFUY071`
L00m?Cd0CTi>065QH@1PH600:BTY034a<@0m?Cd0EEEE05ADE00k>c/0E5AD0P17AdL0301CDe<0ATI6
03Xj>P0l?3`0EEEE02@T900L71`0Bd];0492@P1DE5@0<S8b02h^;P80>CTi03D0ATI6030`<011@D40
EEEE05YJFP1OGel0<30`04=3@`1VIVH0G5aL03hn?P15ADD0Kfm_04Y:BP1@D500Fe]K051@D01UIFD0
L71`04m?C`1@D500Bd];04E5A@0k>c/0ATI6034a<@0b<S80;bl_030`<00m?Cd0?3`l055AD@0`<300
;2`/03Lg=`1>CTh0?Cdm04=3@`1?Cdl0AdM703Ti>@0/;2`0>SXj03lo?`17AdL0>SXj055AD@0^;Rh0
<c<c03Lg=`16ATH0;Rh^03/k>`0203<c<`1J05UIF@1TI6@0A4A404i>CP1PH600HF5Q07MgM`1bLW80
^KVi09BDU01ADE40GUiN03Ti>@1jNWX0GEeM06QXJ01GEeL0@410059BDP14A4@0<c<c08R8R01DE5@0
MWIf06ATI01;Bd/0HV9R07EeM@2WYjL0LG5a055AD@0o?cl0GEeM04M7A`24Q8@0H61P0451@@1ADE40
CTi>01DE5@1[Jf/0Bd];07alO00I6AT0?Shn0451@@0n?Sh0>3Ph04m?C`0l?3`0<S8b092@T01SHf<0
NGUi05=CD`1UIFD0Bd];0410@01=CDd0<S8b04U9B@2SXj<0>3Ph06UYJ@1FEUH0:BTY04Y:BP1<C4`0
Ph>302d];@1CDe<0H61P038b<P0d=3@0820P04A4A00E5AD0DE5A02TY:@1?Cdl0@d=306MWI`0k>c/0
Q8B404I6AP2NWYh0LG5a034a<@2;Rh/0:b/[0P1IFET0A`1[Jf/0DE5A03@d=01EEED0GUiN051@D013
@d<0<c<c04a<C01KFe/0D51@038b<P1WIfL0;Rh^04U9B@1:BTX0Kfm_04M7A`1XJ6P0BTY:04U9B@15
ADD0>SXj03De=@0Z:RX0?clo02XZ:P0o?cl0Fe]K04i>CP18B4P0=CDe04A4A01;Bd/0DE5A05iNGP0l
?3`0FUYJ05iNGP0o?cl0?Cdm065QH@1MGEd0[:b/05eMG@1iNGT0@d=304a<C00`<300GemO08V9R@1a
LG40GemO05iNGP1=CDd09RHV03Ti>@11@D40;Bd]04];B`0/;2`0EUIF06IVIP1OGel0>c/k03lo?`1G
EeL0ADE505aLG018B4P0D51@013ooooo000>ooooo`3[07moO`1/K6`0R8R807=cL`1LG5`0De=C05AD
E024Q8@0E5AD04U9B@1@D500HF5Q09BDU01DE5@0=cLg05ADE01;Bd/0De=C08B4Q018B4P0<c<c04m?
C`15ADD0Jf][08^;R`0Q8B40?clo05EEE@0o?cl0OWin030`<00c<c<0EEEE06=SH`0k>c/0QHF505MG
E`0^;Rh0Jf][05mOG`1CDe<0RHV9075aL@0d=3@0QHF507MgM`1EEED0Rh^;05UIF@1ADE40RHV9038b
<P1]KFd0J6QX02d];@19BDT0Hf=S0451@@1JFUX0A4A403Ph>01LG5`0GemO04=3@`1ZJVX0F5QH04Y:
BP1nOWh0EUIF0492@P1bLW80@d=304i>CP28R8P0B4Q803De=@1BDU80@D5107=cL`20P800:b/[034a
<@1VIVH0@41007YjNP1XJ6P092@T02l_;`1FEUH0EeMG06IVIP17AdL0=cLg04=3@`1TI6@0CTi>07em
O@0h>3P0:BTY04i>CP1SHf<0C4a<051@D00X:2P0>SXj06IVIP1WIfL0>CTi05EEE@1CDe<0C4a<09:B
TP1EEED0?Cdm069RHP0n?Sh0K6a/09bLW01EEED0=3@d07moO`1^KVh0EeMG0;:b/P17AdL0@D5105mO
G`1?Cdl0DU9B09jNWP0o?cl0A4A4051@D018B4P0EEEE07=cL`0_;bl0DU9B06QXJ01CDe<0M7Ad09bL
W00i>CT0Bd];065QH@16ATH0P82009nOW`0i>CT0EeMG08^;R`1@D500Rh^;07emO@0T92@0BDU906MW
I`1;Bd/0Ph>303`l?00R8R80BDU908>3P`1ADE40OWin059BDP0j>SX0Fe]K061PH01SHf<0GEeM03@d
=01ADE40KFe]06IVIP1NGUh0De=C03Ph>01WIfL0YZJV05EEE@1;Bd/0De=C04M7A`1lO7`0Lg=c03dm
?@1>CTh0BTY:075aL@1lO7`0@T9205MGE`1mOGd0BDU9071`L02>SXh0?Cdm06QXJ01HF5P0EEEE06i^
KP0c<c<0DU9B055AD@15ADD0Ph>305mOG`11@D40KFe]06UYJ@27QhL0LW9b02/[:`1:BTX0Hf=S07Ee
M@2XZ:P0DE5A034a<@1RHV80NWYj09jNWP1MGEd0<C4a05EEE@2=SHd0PX:207AdM00i>CT0?clo075a
L@26QXH0DE5A07AdM00203hn?P0407YjNP1oOgl0HF5Q06e]K@80D51@00X0RXZ:05aLG00k>c/0I6AT
0820P016ATH0Y:BT0:VYZ@1;Bd/0F5QH0P1SHf<05`2IVIT0EUIF02/[:`16ATH0IVIV04U9B@1^KVh0
HV9R02`/;015ADD0CTi>04Q8B01>CTh0QXJ602TY:@0e=CD0B4Q803`l?00o?cl0N7Qh02HV9P0e=CD0
DE5A00kooooo05L0EeMG05=CD`0/;2`0DE5A05eMG@1[Jf/0CDe=04U9B@1QHF40@41005UIF@1WIfL0
De=C06IVIP1EEED0@D5105=CD`1JFUX0BTY:02/[:`0k>c/0:2PX030`<012@T80?Shn04I6AP1FEUH0
CDe=04a<C01:BTX0G5aL05ADE01]KFd0D51@05MGE`1BDU80De=C02XZ:P0l?3`0A4A403Hf=P1<C4`0
>CTi0492@P0h>3P0@D5104Y:BP16ATH0BDU90410@016ATH0<C4a030`<012@T80B4Q805UIF@0b<S80
>c/k03Lg=`1@D500BDU903`l?01?Cdl0C4a<06MWI`1QHF40CDe=04];B`1SHf<0FUYJ0492@P1=CDd0
HF5Q02DU9@0[:b/0De=C04a<C018B4P0ADE504M7A`16ATH0BTY:04a<C00c<c<0ADE502h^;P0c<c<0
0P0[:b/0N00i>CT0=3@d03lo?`0[:b/0=cLg03/k>`10@400>CTi04Y:BP0g=cL0@d=303Ti>@0d=3@0
>c/k03Ph>01CDe<0C4a<06ATI00k>c/0De=C04M7A`13@d<0ATI605IFEP0i>CT0A4A404U9B@1FEUH0
<S8b04Q8B00e=CD0@410065QH@1PH600IfMW06a/K00c<c<0Fe]K020P801dM7@0FUYJ07emO@1>CTh0
BDU906ATI00e=CD0B4Q804U9B@0a<C40C4a<03`l?01UIFD0?3`l06IVIP29RHT0G5aL04Y:BP1=CDd0
Kfm_03hn?P1?Cdl0Bd];03Ti>@1XJ6P0<c<c020P801dM7@0@d=305eMG@0i>CT0E5AD04M7A`1PH600
CDe=06ATI00n?Sh0<S8b055AD@1LG5`0Q8B403Ti>@1ADE40@T92030`<01@D500AdM705=CD`1ZJVX0
D51@08J6QP1dM7@0<S8b03hn?P12@T80CDe=04Q8B01SHf<0LG5a05ADE01JFUX0ADE504U9B@0g=cL0
Fe]K04Y:BP1UIFD0Cdm?065QH@1=CDd0Ph>305YJFP1dM7@0KVi^06a/K02XZ:P0PH6106][J`1?Cdl0
Bd];05]KF`80EUIF04<0F5QH05YJFP1BDU80D51@05iNGP1/K6`0FEUI03dm?@1bLW80=3@d04i>CP0k
>c/0FEUI02l_;`1QHF40?clo03Xj>P18B4P0ADE505QHF01EEED0E5AD05aLG01KFe/0G5aL04I6AP14
A4@0Cdm?0410@01JFUX0Jf][0861P@0a<C40@41005IFEP0k>c/0GUiN05UIF@1XJ6P0MGEe05IFEP24
Q8@0Jf][07QhN01<C4`0MGEe0:RXZ02@T900R8R806=SH`1NGUh0?clo061PH01fMWH0@410051@D00g
=cL0EeMG069RHP1QHF40CDe=04m?C`1CDe<0FUYJ05eMG@1lO7`0LW9b013ooooo000>ooooo`2n09jN
WP1ADE40CDe=075aL@2SXj<0B4Q803dm?@1PH600IVIV08R8R01=CDd0<30`04a<C01^KVh0JFUY0:BT
Y01PH600CDe=05iNGP1XJ6P0MGEe0;>c/`0O7al0DE5A08N7Q`1EEED0S8b<06EUI@0Z:RX0ATI609NG
U`1LG5`0U9BD030`<00Z:RX0A4A406UYJ@1aLG40RXZ:01TI6@0[:b/0CDe=08B4Q01XJ6P0RXZ:02PX
:00b<S80P82006IVIP1:BTX0GEeM03Ti>@10@400LG5a03Xj>P0`<300FEUI07YjNP2_[jl0CDe=02TY
:@0i>CT0?Shn06][J`2][Jd0>3Ph028R8P13@d<0HF5Q06e]K@1fMWH07AdM03Hf=P1[Jf/0K6a/08B4
Q01jNWX09BDU04A4A022PX80J6QX065QH@1]KFd09BDU06ATI02MWId0CDe=08:2PP2;Rh/0:RXZ05MG
E`1hN7P0=cLg04U9B@2JVYX0<C4a05QHF02:RXX0?Cdm06a/K01:BTX0<30`05EEE@1>CTh0AdM707Ui
N@0W9bL0>c/k07emO@1NGUh0>c/k04e=C@0];Bd0EeMG07MgM`1:BTX0C4a<05EEE@0e=CD0ADE507Ee
M@2LW9`0>c/k04=3@`1_Kfl0MgMg09bLW01KFe/0;Rh^04e=C@1@D500OGem09>CT`0/;2`0?3`l06AT
I01NGUh0ZZZZ06a/K00Q8B40@d=306=SH`21PH40VYZJ02LW9`0a<C40CDe=06QXJ01lO7`0N7Qh03Xj
>P13@d<0EUIF09>CT`1VIVH0E5AD02h^;P0[:b/0Fe]K092@T00n?Sh0E5AD04M7A`13@d<0IfMW07Ui
N@13@d<0Hf=S0492@P1PH600Vi^K055AD@0g=cL0Bd];059BDP1jNWX0Rh^;02XZ:P0f=SH0DE5A05IF
EP1XJ6P0IfMW030`<01DE5@0AdM704e=C@2XZ:P0IfMW03Hf=P80ATI605H0W9bL0;Rh^00f=SH0?Shn
05MGE`1eMGD0/[:b04m?C`0S8b<0Bd];05IFEP2/[:`0>SXj04A4A01LG5`0F5QH07emO@22PX80820P
05QHF026QXH0CDe=09ZJVP1UIFD0=SHf07YjNP1lO7`0E5AD09NGU`1IFET0EeMG09fMW@1<C4`0PX:2
096AT@13@d<0DU9B08V9R@1:BTX0Hf=S04E5A@0n?Sh0EeMG03hn?P12@T80LW9b02l_;`0m?Cd0Fe]K
08>3P`0n?Sh0M7Ad05QHF00j>SX0EEEE0:2PX01HF5P0E5AD059BDP1RHV80^KVi06UYJ@0c<c<0IFEU
061PH01CDe<0Vi^K07=cL`0_;bl0;Bd]05YJFP15ADD0Xj>S0861P@0_;bl092@T05eMG@11@D40KVi^
06IVIP0N7Qh0<c<c04];B`1=CDd0@D510:6QX@kooooo01X0GEeM07QhN01<C4`0=CDe059BDP19BDT0
=CDe05UIF@1OGel0?Cdm05aLG02ATI40S8b<09^KV`1gMgL0HV9R05]KF`1DE5@0F5QH02`/;00g=cL0
>c/k02l_;`0h>3P0>SXj05UIF@80J6QX01P0H61P05MGE`1ZJVX0FEUI05ADE015ADD0EeMG05QHF01;
Bd/0=3@d02TY:@0c<c<0?Cdm04i>CP0_;bl0ATI603lo?`0l?3`0BDU903Lg=`0c<c<0>3Ph04Q8B00b
<S82030`<00801`L700[:b/0;Bd]034a<@0];Bd0;bl_04A4A00o?cl20451@@3F06MWI`1:BTX071`L
02@T901BDU80CDe=02@T900_;bl0HV9R03`l?01?Cdl0HF5Q055AD@1KFe/0GemO04m?C`1IFET0D51@
04Q8B00d=3@0@41002@T900M7Ad0;bl_02HV9P0a<C40<S8b0451@@0S8b<09RHV03Hf=P0i>CT09RHV
03`l?013@d<0C4a<05QHF010@400CDe=04];B`1/K6`0B4Q804m?C`0i>CT0BTY:05ADE00j>SX0;Bd]
03Ph>00];Bd0=SHf055AD@1@D500ADE504e=C@14A4@0DU9B07inOP1`L700QHF507inOP1DE5@0J6QX
024Q8@1fMWH0EeMG07MgM`0j>SX0:RXZ03Lg=`0L71`0<c<c069RHP10@400Hf=S05QHF00l?3`0IVIV
07UiN@2/[:`0OWin05aLG01FEUH0E5AD04e=C@21PH40M7Ad03`l?013@d<0=CDe01DE5@1VIVH0=SHf
069RHP0J6QX0?3`l03Lg=`1/K6`0GEeM03hn?P0e=CD0JFUY0:^[Z`1aLG40K6a/04=3@`21PH40BDU9
04];B`1PH600<c<c03@d=01PH600=SHf069RHP1QHF406a/K04A4A010@400H61P02/[:`1gMgL0QHF5
05IFEP23Ph<0@d=3065QH@0T92@0HV9R02<S8`1hN7P0D51@06=SH`0];Bd0V9RH05ADE02?Shl0K6a/
0<?3``2QXJ40Ph>306i^KP1BDU80GEeM05UIF@15ADD0IVIV05aLG01KFe/0DE5A03De=@1DE5@0WIfM
06e]K@0l?3`0MGEe03Ti>@1kNg/0CDe=07YjNP1HF5P0QHF505YJFP1LG5`0Bd];03Xj>P1KFe/0<c<c
02d];@12@T80DE5A04A4A00c<c<0GEeM05aLG00_;bl0IfMW04];B`1]KFd0;Rh^03Ph>015ADD06ATI
03Ph>01SHf<0Jf][09RHV01UIFD0I6AT051@D01VIVH0CTi>05IFEP1DE5@0D51@0410@01=CDd0B4Q8
02l_;`0Z:RX0AdM703lo?`15ADD08B4Q03lo?`1aLG40J6QX038b<P10@400HV9R04E5A@1IFET0EEEE
05UIFA3ooooo000>ooooo`3F08J6QP2a/K40Lg=c06a/K01`L700Jf][05]KF`1fMWH0?Shn05QHF022
PX80EeMG07alO00^;Rh0>CTi05mOG`1^KVh0CTi>07moO`0R8R80ATI605QHF01QHF40DU9B04=3@`0j
>SX0Cdm?055AD@0/;2`0?3`l04=3@`19BDT0NGUi02HV9P0U9BD0=cLg04M7A`1cLg<0][Jf04a<C00N
7Qh0CTi>071`L01[Jf/0bl_;02d];@0W9bL0H61P06YZJP1oOgl0N7Qh01TI6@12@T80M7Ad03lo?`1J
FUX0N7Qh02TY:@1[Jf/0NGUi03lo?`2ZZZX0>CTi02d];@1YJFT0J6QX06IVIP1XJ6P08b<S03/k>`1d
M7@0;2`/05mOG`1cLg<06a/K03dm?@2GUiL09BDU04m?C`0g=cL0>CTi03lo?`1mOGd0>CTi05iNGP11
@D40>SXj04i>CP25QHD0?3`l04i>CP16ATH0H61P0861P@3Cdm<0?Cdm04=3@`18B4P0EeMG08b<S01o
Ogl0:b/[05QHF01YJFT0HF5Q0:VYZ@0a<C4071`L04E5A@1kNg/0RXZ:09:BTP0O7al0;Bd]08J6QP1e
MGD0MWIf08V9R@0R8R80<C4a07EeM@2_[jl0I6AT07]kN`0O7al0<c<c06EUI@1mOGd0B4Q805QHF00X
:2P0A4A405IFEP14A4@0<30`06EUI@0T92@0IFEU0861P@18B4P0Cdm?06i^KP0m?Cd0VIVI08^;R`13
@d<0H61P06ATI01?Cdl0QXJ606i^KP1ADE40GemO04U9B@1?Cdl0Ph>30451@@0/;2`0BTY:05UIF@13
@d<0OGem06QXJ00`<300BDU907=cL`1:BTX0T92@03`l?01:BTX0Jf][08^;R`1VIVH0YJFU03hn?P1M
GEd0S8b<05QHF023Ph<0EUIF03<c<`1WIfL0RHV9065QH@2^[Zh0GEeM04I6AP1eMGD0X:2P06IVIP14
A4@0=cLg04I6AP2;Rh/0R8R8079bLP0c<c<0BDU906a/K027QhL0BDU904E5A@0c<c<0IfMW08f=S@18
B4P0L71`03`l?01KFe/0Zj^[04i>CP11@D40Cdm?034a<@80M7Ad03L0=SHf051@D01DE5@0ATI606a/
K01NGUh0AdM705IFEP1HF5P0PH610;jn_P0k>c/0EUIF05YJFP1^KVh0/K6a06][J`0Y:BT0HF5Q05mO
G`21PH40cLg=055AD@0e=CD0@41009JFUP1hN7P0SHf=06UYJ@0_;bl0?clo06a/K01hN7P0[jn_05QH
F014A4@0De=C08>3P`1VIVH0UIFE03Ti>@0b<S80M7Ad09BDU015ADD0ATI6024Q8@0O7al0Cdm?061P
H015ADD0>SXj03Hf=P0a<C40?clo0080Ng]k00D0?Cdm04A4A00k>c/0?Cdm07emO@0>ooooo`1X075a
L@1XJ6P0;Bd]04E5A@1MGEd0EEEE0492@P1KFe/0HF5Q04U9B@1GEeL0DU9B04Q8B01HF5P0W9bL08:2
PP1eMGD0GEeM04e=C@0W9bL0ATI603Hf=P0d=3@0C4a<03Ti>@14A4@0AdM705UIF@1/K6`0ADE505=C
D`1PH600I6AT05=CD`1FEUH0FEUI05EEE@0j>SX0?3`l04Q8B00n?Sh0Cdm?03Xj>P1?Cdl0>c/k04Q8
B01;Bd/0@41003Ti>@0j>SX0FEUI03dm?@0^;Rh0@d=302l_;`1;Bd/0=CDe03Ph>00i>CT0ADE5059B
DP1DE5@0BDU904a<C02LW9`0IVIV03`l?014A4@0JFUY05UIF@0_;bl0A4A405iNGP0X:2P09RHV05mO
G`1EEED0DE5A04Y:BP1<C4`0ADE504=3@`14A4@0=3@d05ADE00g=cL092@T04I6AP11@D40>c/k03hn
?P1:BTX0;Bd]030`<00j>SX0?Shn03Hf=P18B4P0@D5103<c<`0/;2`0<C4a0410@00o?cl204m?C`1C
05iNGP0`<300C4a<05MGE`0i>CT0>c/k051@D00g=cL0BDU90492@P0j>SX0<30`04Y:BP1BDU80?clo
065QH@1IFET0S8b<06QXJ00[:b/0@410018B4P1EEED0De=C08b<S01KFe/0?Cdm0451@@0_;bl0=cLg
038b<P0Z:RX0Fe]K030`<00n?Sh0E5AD07]kN`2n_[h0OGem051@D00n?Sh0J6QX02d];@11@D40?Cdm
03<c<`1FEUH0=3@d03Lg=`1cLg<0<C4a04I6AP0Y:BT0@D51034a<@0k>c/0A4A404];B`15ADD0<c<c
04A4A010@400P82004m?C`1^KVh0ATI6030`<015ADD0C4a<04Y:BP2JVYX0BDU90:>SX`2MWId0820P
05EEE@1GEeL0Ogmo04I6AP1RHV80Lg=c03dm?@12@T800P0d=3@0:00Z:RX0L71`03De=@1<C4`0Fe]K
06m_K`17AdL0MgMg034a<@17AdL0ADE503/k>`0W9bL0PH6105aLG00e=CD0>SXj04i>CP0_;bl0@410
05UIF@1HF5P0?3`l03<c<`1FEUH0ADE504=3@`0k>c/0Ogmo030`<018B4P0>SXj05EEE@0_;bl0G5aL
02h^;P0N7Qh0?Cdm0451@@0a<C420492@P0T03hn?P0i>CT0EEEE0451@@10@400?Shn0410@01=CDd0
Bd];08R8R00_;bl0?3`l06MWI`0m?Cd0KFe]059BDP1:BTX0AdM7061PH01RHV80BDU907EeM@21PH40
KVi^09:BTP2@T900IVIV07YjNP1QHF40?clo04U9B@1EEED0@d=305ADE00Y:BT0@T920P23Ph<01`1<
C4`0CDe=06MWI`1>CTh0C4a<059BDP1KFe/04?ooool000kooooo0?l0Kfm_05ADE02TY:@0Q8B404e=
C@19BDT0EeMG08B4Q00h>3P0>CTi06a/K028R8P0^[Zj04E5A@0R8R80:b/[05mOG`2@T900/K6a02HV
9P0j>SX0Bd];05UIF@1lO7`0FUYJ01HF5P14A4@0Lg=c055AD@1YJFT06ATI02h^;P1bLW80GUiN06=S
H`2n_[h0A4A403`l?01hN7P0S8b<04Y:BP1eMGD0M7Ad02TY:@1bLW80S8b<06ATI02IVIT08R8R03dm
?@1nOWh0@d=306YZJP1?Cdl05aLG03`l?01ADE40?Cdm04e=C@0Z:RX0>c/k05QHF00o?cl0=CDe04m?
C`0[:b/0Fe]K0:>SX`17AdL0;2`/06MWI`0o?cl0Ng]k0:FUY@0J6QX04Q8B05MGE`1TI6@0QHF5071`
L00Q8B40?Shn065QH@1PH600WIfM08N7Q`0K6a/0EEEE08>3P`1QHF40YjNW04E5A@0W9bL0ATI608>3
P`1OGel0L71`02XZ:P10@400FUYJ07inOP1>CTh0IVIV020P800n?Sh0Q8B40861P@0h>3P0Bd];03dm
?@1IFET0L71`04m?C`0_;bl0@41004M7A`1VIVH0VYZJ03Xj>P0Z:RX0B4Q805YJFP1OGel0WIfM034a
<@0X:2P0?3`l05IFEP18B4P0YZJV02HV9P0d=3@0FUYJ04Y:BP1BDU80UYJF01`L700l?3`0IfMW04m?
C`23Ph<0/;2`03Ti>@1MGEd0EeMG06IVIP2][Jd0XJ6Q03De=@1fMWH0PH61096AT@2k^k/0VYZJ02l_
;`1SHf<0T92@07IfMP2TY:@0@d=302/[:`16ATH0GemO06ATI02;Rh/0DU9B04Q8B01[Jf/0DE5A06m_
K`1fMWH0=SHf03dm?@1aLG40F5QH05iNGP1EEED0Cdm?05QHF01nOWh0AdM703/k>`1EEED0G5aL07Yj
NP1GEeL08b<S03Ph>01CDe<0Ogmo06QXJ00V9RH0?clo0410@01CDe<0YjNW0410@00^;Rh0KFe]05iN
GP1WIfL0I6AT02`/;01@D500I6AT05IFEP2HV9P0ATI602/[:`1dM7@0FEUI08F5Q@1]KFd0=CDe04e=
C@1NGUh0IVIV08Z:RP0`<300=CDe05IFEP1MGEd0TY:B0492@P0_;bl0IfMW06i^KP13@d<0I6AT03`l
?00b<S80K6a/0;Vi^@1ZJVX0IfMW02HV9P0l?3`0K6a/07=cL`1>CTh0UIFE055AD@1?Cdl0LG5a096A
T@1<C4`0?Shn05ADE00o?cl0/k>c071`L00[:b/0@4100451@@1?Cdl001L0WYjN07UiN@0S8b<0?3`l
061PH01EEED0JVYZ02PX:00C4a<0<C4a065QH@12@T80FEUI04i>CP0W9bL0=3@d04i>CP0g=cL0?clo
04m?C`0U9BD0>SXj051@D00>ooooo`0J07AdM01TI6@0;bl_04m?C`1]KFd0Fe]K04a<C01KFe/0K6a/
0492@P1JFUX0IVIV07moO`1]KFd0QHF507emO@1MGEd0F5QH04e=C@0U9BD0ATI602PX:00/;2`0>SXj
04I6AP1KFe/304I6AP2G03`l?01=CDd0>c/k04I6AP10@400D51@04M7A`14A4@08R8R01lO7`0a<C40
ATI604];B`0S8b<0;2`/0451@@0/;2`0:b/[03De=@0o?cl0?Shn05ADE00g=cL0:RXZ04A4A00c<c<0
Hf=S034a<@0l?3`0>3Ph05UIF@1>CTh0ADE504a<C018B4P0E5AD04m?C`12@T80ATI604A4A015ADD0
?clo03Xj>P1CDe<0;Bd]03`l?01FEUH0EeMG05YJFP1FEUH0ADE504];B`17AdL0D51@02DU9@0o?cl0
<c<c04I6AP0h>3P0=CDe04I6AP0d=3@0ADE502XZ:P0i>CT0Bd];03Ti>@0g=cL0AdM703Ph>013@d<0
=SHf03Xj>P0o?cl0@D5105mOG`12@T80DE5A03De=@1BDU80De=C05EEE@14A4@0DE5A03<c<`0a<C40
@T9205ADE011@D40De=C04i>CP18B4P0NGUi07emO@1lO7`0KVi^0492@P1MGEd04Q8B061PH01;Bd/0
KVi^03Ph>01JFUX0De=C02TY:@0g=cL0BTY:02DU9@1RHV80<S8b071`L012@T80FUYJ079bLP1ADE40
JVYZ05YJFP0m?Cd0@T9207emO@1/K6`0>3Ph06QXJ00n?Sh0820P07=cL`19BDT0De=C038b<P1FEUH0
?3`l05ADE01IFET0KVi^04=3@`14A4@0KVi^05iNGP1mOGd0ATI606QXJ00`<300?clo05MGE`1JFUX0
CDe=05]KF`11@D40B4Q805eMG@0203<c<`0703hn?P1ADE40Bd];079bLP2>SXh0D51@05YJFP020451
@@1G02TY:@1=CDd0=SHf05QHF01=CDd0E5AD04A4A028R8P0@T9207AdM01@D500@410075aL@1jNWX0
K6a/03`l?016ATH0A4A403lo?`1:BTX0D51@05IFEP15ADD0=3@d04m?C`1VIVH0Cdm?02`/;01MGEd0
7alO05YJFP0e=CD0FUYJ03Lg=`1cLg<0E5AD04i>CP1NGUh0E5AD04i>CP1JFUX0GUiN04=3@`0o?cl0
IfMW03De=@0a<C40AdM704=3@`1bLW80E5AD06i^KP0l?3`0?Cdm0410@00i>CT0:b/[04e=C@1MGEd0
Jf][05]KF`1RHV80Ogmo07]kN`1KFe/0LG5a07QhN02=SHd0KVi^07]kN`1SHf<0?Cdm04a<C01NGUh0
=CDe04E5A@0I6AT0?3`l05EEE@1TI6@0De=C04Q8B01<C4`0DU9B05eMG@1lO7`0GemO013ooooo000>
ooooo`3o0;Fe]@1BDU80DU9B071`L01kNg/0=SHf05=CD`1ADE40BDU906a/K00R8R80<C4a06=SH`1h
N7P0>CTi06m_K`0n?Sh0AdM708n?S`1FEUH0B4Q804m?C`0J6QX0C4a<09jNWP11@D40J6QX03Ti>@0O
7al0LW9b03hn?P0h>3P0G5aL02TY:@0W9bL0Jf][05eMG@0j>SX0EeMG04I6AP0f=SH0M7Ad096AT@0W
9bL0=cLg0451@@2MWId0YJFU04];B`0];Bd0EEEE05ADE02f][H0Ng]k02LW9`16ATH0LW9b08V9R@2Y
ZJT0<C4a02l_;`1?Cdl0NGUi079bLP2=SHd0>3Ph02<S8`14A4@0HV9R05UIF@0_;bl08B4Q0492@P2:
RXX0LG5a05MGE`1>CTh0;2`/07YjNP2OWil0@d=304E5A@1;Bd/0A4A407QhN02ATI40E5AD04I6AP1F
EUH0?Cdm08B4Q02HV9P09bLW03Lg=`1=CDd0B4Q809FEU@1LG5`08b<S04a<C01WIfL0IVIV07inOP0S
8b<0:BTY04e=C@1lO7`0CTi>08>3P`18B4P0BDU906QXJ01?Cdl0IFEU08>3P`0i>CT0BTY:06m_K`1c
Lg<0Fe]K0:VYZ@1IFET0BTY:0820P01`L700Kfm_08J6QP0b<S80=cLg07MgM`1CDe<0OGem08:2PP0Y
:BT0@41007inOP1jNWX0MgMg03dm?@0];Bd0=SHf0:2PX02=SHd0RXZ:04U9B@0[:b/0K6a/0=kNgP1a
LG40MWIf038b<P0d=3@0IfMW0;Rh^01HF5P0ADE502LW9`1IFET0[jn_06e]K@0];Bd0Bd];04Y:BP1N
GUh0];Bd0:VYZ@0e=CD0CDe=061PH02GUiL0YjNW03De=@19BDT0DU9B05iNGP2@T900OGem03Ph>01M
GEd0Lg=c06YZJP2c/k<0EEEE03De=@15ADD0P820055AD@23Ph<0@T92065QH@24Q8@0C4a<06=SH`23
Ph<08B4Q07QhN02BTY80AdM705IFEP0n?Sh0<c<c07QhN00g=cL0L71`03/k>`0m?Cd0Ogmo09nOW`1<
C4`0EEEE038b<P11@D40Kfm_03/k>`17AdL0?clo0492@P1lO7`0J6QX02LW9`1TI6@0Cdm?07IfMP1_
Kfl0;Bd]04M7A`1RHV80?clo09^KV`1IFET092@T03lo?`1^KVh0H61P092@T00f=SH0=cLg04];B`1L
G5`0@D5109RHV01^KVh0>SXj04U9B@23Ph<0P820096AT@2/[:`0>c/k069RHP2VYZH0Shn?09FEU@1@
D500;Bd]000G06i^KP2IVIT0H61P07moO`1fMWH08R8R05UIF@1PH600>c/k05mOG`1NGUh08b<S04M7
A`1UIFD0=3@d05EEE@0i>CT07AdM03/k>`1ZJVX07AdM04];B`11@D403_ooool0L@1JFUX0HV9R04I6
AP13@d<0C4a<04A4A01<C4`0BTY:059BDP17AdL0EEEE07inOP1fMWH0JVYZ07EeM@1IFET0GEeM04e=
C@1:BTX0<c<c04M7A`0m?Cd0@d=303/k>`0c<c<0?3`l04m?C`1XJ6P0QHF504Y:BP17AdL0Cdm?05eM
G@12@T80CDe=05mOG`1BDU80<S8b024Q8@0^;Rh0:BTY04=3@`0k>c/0?3`l02PX:00h>3P0?Cdm04=3
@`0/;2`0=cLg03hn?P0f=SH0;Bd]03<c<`0U9BD0@d=303@d=00o?cl0<30`02`/;00h>3P0CDe=051@
D01>CTh0LG5a06MWI`0h>3P0@41005MGE`1MGEd09RHV034a<@1<C4`0:BTY024Q8@1?Cdl0ATI60492
@P14A4@0?Cdm03hn?P0i>CT0?3`l03<c<`16ATH0;Bd]02HV9P10@400?Shn03Xj>P0b<S80?Shn02`/
;00P8200;Bd]03Ph>00Y:BT0<c<c04a<C010@400BDU9034a<@0k>c/0FUYJ069RHP1:BTX0EEEE04U9
B@1HF5P0NWYj05ADE01ADE40EEEE00<0ADE505P0EEEE02PX:00o?cl0C4a<03De=@1?Cdl0F5QH08J6
QP1TI6@0@41005UIF@0N7Qh0MgMg06UYJ@1/K6`0?Cdm04M7A`1PH600<C4a04E5A@1GEeL0<C4a04=3
@`0R8R80>CTi04A4A01gMgL0a/K606YZJP15ADD0Bd];05EEE@0f=SH0A4A404m?C`0a<C40?Shn02d]
;@0N7Qh0LG5a03@d=01OGel09BDU03dm?@0[:b/0De=C05mOG`1<C4`0ADE503/k>`1eMGD0FUYJ075a
L@13@d<0FUYJ02/[:`0n?Sh0FUYJ059BDP1FEUH0PH61061PH01EEED0D51@02d];@0i>CT0B4Q805AD
E01:BTX0W9bL0::RXP1ADE40DU9B05YJFP1NGUh0@T9205ADE00`<300N7Qh05IFEP1GEeL0BTY:08n?
S`1VIVH0Ogmo05aLG00[:b/0VIVI0P1<C4`02P1fMWH0H61P05UIF@1<C4`0HF5Q05iNGP1MGEd0D51@
04a<C01LG5`2069RHP0l06=SH`1oOgl0E5AD06EUI@1CDe<0JVYZ04=3@`2?Shl0D51@03Ti>@15ADD0
;bl_04Q8B00a<C40<S8b03lo?`14A4@0D51@05IFEP1BDU80CDe=05IFEP1>CTh0F5QH06EUI@0h>3P0
B4Q8055AD@0b<S80=SHf059BDP1TI6@0Lg=c06IVIP1YJFT0Ph>308:2PP20P800D51@079bLP1fMWH0
H61P06e]K@1EEED0GUiN06=SH`1iNGT0CDe=051@D00[:b/0DE5A07IfMP1[Jf/0BDU9051@D01QHF40
BDU905]KF`1?Cdl0D51@4?ooool000kooooo0?l0W9bL079bLP20P800DU9B08Z:RP1>CTh0@410071`
L01@D500RXZ:08F5Q@0h>3P0C4a<04=3@`0n?Sh0Z:RX04U9B@0Y:BT0D51@03De=@1UIFD0UiNG02DU
9@0c<c<0ADE506EUI@38b<P0?clo02TY:@1=CDd0Fe]K08R8R02c/k<0:2PX04E5A@0m?Cd0Fe]K0:VY
Z@3Lg=`08B4Q028R8P1@D500XZ:R092@T02JVYX04a<C04U9B@29RHT0VYZJ06EUI@12@T8071`L04M7
A`1dM7@0@41005UIF@0V9RH0>CTi04E5A@14A4@0;2`/04i>CP0U9BD0LG5a08Z:RP1JFUX0=3@d05]K
F`0V9RH0EUIF06YZJP0L71`0<C4a0451@@1EEED0Lg=c06i^KP0I6AT0<C4a05ADE01/K6`0GemO06IV
IP0I6AT0>3Ph06a/K01LG5`0XJ6Q09NGU`0Q8B40C4a<08V9R@1SHf<0X:2P09:BTP0i>CT0I6AT09^K
V`1aLG40^;Rh07AdM00/;2`0Kfm_06IVIP1]KFd0]kNg02TY:@0j>SX0Hf=S04i>CP1RHV80H61P03dm
?@0h>3P0NGUi055AD@1KFe/0IfMW02PX:012@T80HF5Q06=SH`0o?cl0EEEE02TY:@1/K6`0UiNG05aL
G00g=cL0G5aL03<c<`2GUiL0fm_K04];B`0o?cl0@D5104A4A02/[:`0UIFE02TY:@0g=cL0G5aL059B
DP30`<00@d=302l_;`10@400VIVI07alO02^[Zh0<c<c03Xj>P15ADD0NWYj08F5Q@1nOWh07alO02XZ
:P1;Bd/0PH6104];B`1YJFT0De=C02/[:`1DE5@0[:b/06ATI01kNg/0;Rh^03dm?@1DE5@0GUiN05IF
EP1PH600820P03hn?P1gMgL0<c<c04a<C016ATH09BDU05ADE020P800CDe=06EUI@0n?Sh0=CDe0820
P01bLW80<c<c03dm?@13@d<0MGEe08n?S`1HF5P0B4Q803Ph>01kNg/0][Jf04U9B@0];Bd0B4Q805EE
E@2l_;`0Fe]K024Q8@1:BTX0KVi^07emO@2QXJ40>c/k03Lg=`1BDU80ATI60::RXP1HF5P0ATI6051@
D01LG5`0CDe=0;no_`0n?Sh0R8R80:BTY01;Bd/0U9BD08N7Q`0k>c/0Fe]K08R8R01NGUh0Ogmo04=3
@`0k>c/0HF5Q06m_K`1JFUX0J6QX02HV9P0/;2`0E5AD061PH01;Bd/0H61P04i>CP0R8R80E5AD06AT
I01IFET0QHF503De=@0c<c<0TY:B08:2PP0k>c/001L0GemO0492@P10@400Rh^;071`L00W9bL0>c/k
05=CD`11@D40NGUi075aL@0Z:RX07alO04m?C`1MGEd0WIfM06IVIP0E5AD0;Bd]05IFEP0k>c/0UIFE
08B4Q00>ooooo`0N05mOG`1WIfL0<30`0410@01NGUh0Bd];03/k>`1eMGD0RXZ:04=3@`1MGEd0G5aL
05YJFP1mOGd0MWIf05IFEP1LG5`0?Shn04A4A00h>3P0ATI603De=@0a<C40@T9203`l?014A4@092@T
01hN7P0_;bl0>CTi0P0c<c<0=01;Bd/0?Cdm0492@P0d=3@0?Cdm03/k>`0n?Sh0;2`/03De=@0o?cl0
?Cdm03`l?00b<S80BTY:04e=C@0i>CT0>3Ph04I6AP1NGUh0>3Ph02DU9@0n?Sh08B4Q04Y:BP0P8200
:BTY03Ph>01<C4`0<c<c02HV9P0l?3`0=3@d059BDP1GEeL0Fe]K06=SH`1DE5@0ADE503hn?P0m?Cd0
HV9R02DU9@0/;2`0IFEU04];B`1GEeL0Cdm?051@D01ADE40@D5104Q8B00U9BD203Lg=`0f03/k>`17
AdL0?3`l0492@P0n?Sh0@41002`/;00c<c<0A4A403Lg=`0V9RH0?Shn038b<P12@T80:RXZ03dm?@0e
=CD0>3Ph0492@P1>CTh0DU9B03lo?`0m?Cd0CDe=03`l?016ATH0B4Q803De=@0`<300>3Ph04Y:BP0l
?3`0Bd];05EEE@12@T80L71`05=CD`1jNWX0JFUY04A4A025QHD0@41004Q8B01:BTX0P82004Y:BP19
BDT0Cdm?04M7A`13@d<0Ogmo03/k>`1ADE40<C4a0P1ADE401`1ZJVX0Jf][055AD@1PH600EEEE06YZ
JP0n?Sh00P1NGUh0C`0Z:RX0LW9b05IFEP0Z:RX0KFe]03/k>`0j>SX06QXJ04i>CP0h>3P0A4A4055A
D@1RHV80>CTi03<c<`1XJ6P0DU9B07]kN`13@d<0EEEE00h>3P0`<300EEEE051@D01?Cdl0TY:B04Y:
BP1PH600D51@02`/;015ADD0E5AD07=cL`14A4@0H61P06i^KP17AdL0IVIV03<c<`0e=CD0:2PX06UY
J@0n?Sh0A4A404M7A`1;Bd/0B4Q8092@T01JFUX0Lg=c07alO00k>c/0RHV905eMG@1=CDd0BDU9065Q
H@17AdL0<C4a065QH@1:BTX0EEEE04=3@`0k>c/0De=C08b<S01iNGT0>c/k07IfMP0M7Ad0KVi^04Y:
BP1gMgL0;Rh^06YZJP12@T8071`L02TY:@0V9RH00P12@T80501;Bd/0@T920492@P1IFET0EEEE05aL
G01CDe<0@d=304];B`1GEeL0HV9R03dm?@1UIFD0G5aL02/[:`0d=3@0EEEE06ATI02<S8`0J6QX0P1d
M7@0602;Rh/0L71`03Ph>01dM7@0NGUi06e]K@1aLG40CDe=02PX:00o?cl0FEUI04Y:BP1NGUh0;2`/
0451@@2OWil0Xj>S055AD@1BDU80MWIf051@D01HF5P0EEEE065QHA3ooooo000>ooooo`0V07alO01J
FUX0OGem06a/K01]KFd0GEeM05QHF01oOgl0BTY:05YJFP1RHV80K6a/0:FUY@18B4P0>SXj0492@P1H
F5P0MgMg08n?S`0^;Rh0?Shn04a<C014A4@0Kfm_04=3@`0W9bL0I6AT07AdM012@T80GemO02LW9`0^
;Rh0JVYZ0492@P0n?Sh0PX:203hn?P0Z:RX206MWI`1g04=3@`2>SXh0CDe=04E5A@1`L700=3@d05=C
D`29RHT0@d=307]kN`1GEeL0@T9206ATI011@D40<c<c06YZJP1/K6`0ATI606=SH`0o?cl0?Shn07em
O@15ADD0AdM706YZJP1MGEd0FUYJ08R8R014A4@0?clo065QH@11@D40HV9R07YjNP0X:2P0>CTi04m?
C`16ATH0JVYZ06a/K00I6AT0;Bd]05]KF`18B4P0P82007UiN@0P8200<c<c06UYJ@1:BTX0S8b<04=3
@`0d=3@0De=C06e]K@1@D500NWYj03De=@0/;2`0FUYJ08^;R`1GEeL0OGem03Lg=`0m?Cd0MWIf08V9
R@0l?3`0K6a/04m?C`1MGEd0M7Ad04];B`0k>c/0F5QH04U9B@1jNWX0VIVI04M7A`0n?Sh0K6a/06QX
J01OGel0PH6103Ph>010@400IfMW05mOG`13@d<0Jf][03De=@1BDU80Jf][05ADE01RHV80JFUY038b
<P18B4P0Lg=c06ATI01PH600RXZ:03dm?@1RHV80IfMW04Q8B01UIFD0HV9R03lo?`1jNWX0M7Ad059B
DP2FUYH0F5QH03De=@1hN7P0Lg=c04];B`1`L7000P0i>CT0M@1CDe<0LG5a05=CD`1`L700?3`l04A4
A01gMgL0C4a<05]KF`1XJ6P0CDe=061PH02IVIT0De=C04m?C`1PH600CTi>06QXJ02EUID0>3Ph04Y:
BP1BDU80FEUI09FEU@1YJFT0=SHf059BDP1ADE40Ng]k08R8R00W9bL0@d=306][J`1bLW80Ogmo04U9
B@0k>c/0HV9R05QHF01JFUX0J6QX02DU9@1OGel0EUIF069RHP2HV9P0C4a<03<c<`1/K6`0HV9R08f=
S@1UIFD0;Rh^05IFEP1aLG40Jf][07QhN00T92@0@D5107]kN`1OGel0RXZ:05]KF`0e=CD0Fe]K0861
P@1<C4`0EUIF04M7A`1EEED0MWIf08V9R@1HF5P0LW9b04m?C`1QHF40SXj>06=SH`0k>c/0OGem065Q
H@1ADE40IVIV09>CT`19BDT0B4Q8069RHP1ADE40_;bl07QhN00h>3P0IfMW05MGE`1YJFT0XJ6Q04];
B`11@D40MGEe05QHF016ATH0NWYj04A4A00U9BD0?Shn05YJFP0k>c/0DU9B04Y:BP0V9RH0?Cdm06=S
H`11@D40C4a<05aLG00Y:BT0Bd];05IFEP0>ooooo`0H07YjNP1JFUX0<S8b04e=C@1NGUh0DU9B03dm
?@1<C4`0AdM702d];@1=CDd0LW9b06EUI@1XJ6P0GEeM04i>CP1SHf<0E5AD04];B`0Y:BT0@T9202`/
;00i>CT0AdM70P14A4@01`1ADE40IFEU071`L014A4@0Fe]K06m_K`1]KFd00P1@D500@01PH600FEUI
03Ph>00h>3P0?Cdm03`l?01@D500A4A405EEE@0m?Cd0@T9204Y:BP1=CDd0=CDe03dm?@18B4P0;bl_
020P800e=CD0:BTY04];B`0b<S80<C4a03/k>`10@400@D5102l_;`1?Cdl0EUIF06ATI01SHf<0<S8b
03Ph>01QHF40Bd];03@d=00o?cl0>3Ph02HV9P0`<300C4a<04i>CP1:BTX0ADE503/k>`1:BTX0ATI6
03lo?`0Y:BT0>3Ph03<c<`0i>CT0@D5103`l?00o?cl0?Cdm03lo?`0Z:RX0<S8b0492@P0m?Cd0=SHf
03dm?@0h>3P203hn?P2403Ph>00l?3`0Cdm?055AD@13@d<0DE5A0451@@1CDe<0G5aL03Xj>P14A4@0
@d=3038b<P11@D40<C4a03hn?P0W9bL0=cLg04Q8B00f=SH0DU9B04a<C025QHD0Kfm_04i>CP1lO7`0
8b<S06=SH`1BDU80GEeM03Xj>P1bLW80IFEU02l_;`11@D40C4a<0410@01VIVH0<30`03@d=011@D40
Hf=S0;Fe]@1JFUX0KFe]059BDP1@D500<C4a05QHF01PH6009bLW05YJFP13@d<04a<C06UYJ@0a<C40
IfMW02DU9@1EEED0=CDe03Xj>P19BDT0HV9R03lo?`0^;Rh0GEeM0451@@1MGEd0:2PX069RHP0`<300
;bl_051@D01NGUh0Bd];08V9R@0i>CT0G5aL05QHF00G5aL0>CTi038b<P1ADE40=CDe07EeM@22PX80
BDU904];B`1@D500AdM703Ti>@17AdL0=SHf05iNGP1;Bd/0ADE504Q8B02ATI40AdM706=SH`1NGUh0
@T9205QHF013@d<0DE5A05IFEP1QHF40CTi>03dm?@1IFET0CTi>05UIF@1>CTh0ATI605mOG`1KFe/0
GemO04e=C@1aLG40A4A405=CD`13@d<0HV9R0451@@1oOgl0LW9b0492@P13@d<0BTY:05=CD`1KFe/0
Lg=c0P18B4P0:`1UIFD0?Shn04E5A@1>CTh0Bd];06][J`1MGEd0JVYZ04A4A011@D40HF5Q03Hf=P1=
CDd0H61P05=CD`1ZJVX0EUIF075aL@1]KFd0Ogmo05mOG`1FEUH0WIfM05aLG01hN7P0EeMG05eMG@0h
>3P0C4a<05aLG01;Bd/0IFEU03Xj>P11@D40PH6107QhN01@D500@T92061PH01:BTX0EUIF05eMG@1J
FUX04?ooool000kooooo0380Xj>S065QH@1FEUH0HV9R0;Zj^P1FEUH0@T92055AD@1GEeL0T92@04M7
A`0b<S80C4a<04e=C@1RHV80_kno04I6AP1;Bd/0E5AD04e=C@2DU9@0LG5a01DE5@1>CTh0HV9R05EE
E@2c/k<08B4Q02l_;`1MGEd0GemO04a<C01SHf<06QXJ02PX:00_;bl0CTi>07moO`1YJFT08R8R03/k
>`1nOWh0R8R803/k>`1FEUH06QXJ05UIF@2`/;00HV9R04];B`80=cLg07T0S8b<03/k>`0M7Ad0=CDe
04Y:BP1JFUX0TI6A02l_;`0D51@0<c<c03Lg=`2CTi<0Vi^K04Q8B00X:2P0E5AD05iNGP23Ph<0GemO
020P8010@400ADE505IFEP1YJFT0Shn?02<S8`1SHf<0OGem03Ph>01IFET0HF5Q01/K6`1VIVH0Lg=c
0451@@1@D500JFUY03De=@2>SXh0];Bd04Q8B01KFe/0AdM703Hf=P1nOWh0N7Qh04E5A@1MGEd0?Shn
03<c<`1eMGD0@D5104=3@`1EEED0?Shn04Y:BP1iNGT0IFEU02h^;P0k>c/0ATI607moO`2SXj<0>CTi
03dm?@1lO7`0EEEE07alO03Ng]h0SHf=02h^;P1WIfL0Bd];08>3P`3:b/X0A4A403hn?P1FEUH0FUYJ
09ZJVP2d];@0?clo05=CD`21PH40Bd];07moO`1?Cdl0?Shn05EEE@2BTY80P82008J6QP0];Bd0DE5A
07MgM`2d];@0GemO04e=C@0d=3@0BDU907UiN@2QXJ40EeMG06MWI`16ATH0BDU909jNWP23Ph<0>3Ph
05QHF00n?Sh0D51@0;fm_@1eMGD0?3`l05mOG`0j>SX0G5aL096AT@0Z:RX0?Cdm0080AdM706L0M7Ad
06IVIP0W9bL0Bd];06EUI@1BDU80M7Ad059BDP0m?Cd0IfMW06m_K`21PH40YJFU04E5A@1KFe/0JFUY
03dm?@1UIFD0F5QH01dM7@1@D500PH61051@D028R8P0?Cdm03<c<`1/K6`0DE5A08N7Q`0R8R80=SHf
04i>CP1lO7`0Bd];04U9B@0_;bl0FUYJ07]kN`18B4P0MWIf04];B`1=CDd0NWYj0410@01EEED0L71`
038b<P1NGUh0QXJ605EEE@1QHF40P8200451@@2CTi<0Hf=S0451@@1<C4`0ADE504=3@`1NGUh0EUIF
0410@01EEED0?Cdm03Lg=`1fMWH0N7Qh02`/;016ATH0H61P08F5Q@2<S8`0/k>c03lo?`1ADE40RHV9
07moO`2SXj<0I6AT03hn?P1jNWX0RXZ:05]KF`3Rh^80EeMG01hN7P16ATH0^;Rh061PH02;Rh/0Hf=S
02d];@16ATH0EEEE0410@01?Cdl0;Bd]02l_;`0Z:RX0IfMW03De=@1OGel0Fe]K00kooooo04T0PH61
079bLP0V9RH0A4A405QHF012@T80<S8b065QH@1_Kfl0@41005ADE01CDe<0DE5A07AdM01NGUh0CDe=
06ATI015ADD0?Cdm020P801CDe<0>3Ph034a<@10@400BDU904=3@`1FEUH0GUiN05EEE@0d=3@0DE5A
04m?C`1/K6`0C4a<05=CD`1OGel0C4a<03Lg=`16ATH0CDe=034a<@1<C4`0@41004U9B@0f=SH0CTi>
065QH@11@D40@d=304I6AP1MGEd0@41003@d=01:BTX0:2PX061PH012@T80C4a<04U9B@1BDU80BTY:
03`l?01MGEd0BDU906i^KP1bLW80@D5104E5A@1aLG40GemO04];B`1=CDd0;2`/00806a/K00/0BTY:
05iNGP0o?cl0?3`l059BDP1=CDd0=cLg04=3@`0l?3`0ATI603`l?00205YJFP1L04A4A015ADD0BDU9
03Ph>00U9BD0DU9B055AD@0d=3@0B4Q805YJFP0h>3P0Cdm?0492@P14A4@0@D5104A4A015ADD0@d=3
05=CD`0o?cl0BTY:065QH@1@D500M7Ad061PH00h>3P0?clo038b<P0`<300=SHf0451@@14A4@0<30`
04=3@`12@T80EUIF04M7A`0n?Sh0UYJF04M7A`1mOGd0EUIF0861P@14A4@0EUIF05=CD`0];Bd0E5AD
09>CT`0T92@0EeMG030`<01;Bd/0EeMG065QH@2IVIT0EUIF05iNGP1HF5P0L71`04E5A@1ADE40HF5Q
01`L702EUID0Jf][00/;2`1PH600:b/[04m?C`0H61P0ADE502@T900];Bd0=cLg04i>CP0i>CT06a/K
0492@P1>CTh0C4a<02d];@1TI6@0>CTi03Ph>01ADE40Bd];03/k>`1XJ6P0;Bd]06ATI01/K6`202XZ
:P0B03Lg=`0j>SX0<30`05eMG@1SHf<0?clo059BDP0o?cl0A4A4038b<P19BDT0;2`/06MWI`0m?Cd0
:2PX03@d=02OWil09RHV0P1BDU80800];Bd0M7Ad04M7A`1:BTX0G5aL06MWI`1OGel0>c/k05EEE@1>
CTh0GEeM04=3@`0o?cl0FUYJ08j>SP1fMWH0CDe=08V9R@1<C4`0Lg=c05QHF01]KFd092@T07YjNP2I
VIT0=CDe051@D01ADE40?3`l03dm?@1:BTX0=3@d0P0f=SH0100V9RH0:RXZ03@d=014A4@204i>CP0T
06i^KP0K6a/0ATI60820P00b<S80CDe=04=3@`1KFe/0RXZ:06][J`1?Cdl0BTY:04e=C@1;Bd/0CTi>
09NGU`0k>c/0EUIF04E5A@1@D500=cLg04=3@`18B4P0=3@d06YZJP0a<C40@41008Z:RP1jNWX0BTY:
059BDP1[Jf/0D51@05ADE01ADE40Hf=S4?ooool000kooooo01H0VIVI07inOP2NWYh0CTi>07QhN02O
Wil0MgMg09bLW00l?3`0Kfm_08B4Q01IFET0IfMW02LW9`0`<300GUiN05aLG01@D500DE5A03`l?019
BDT0T92@0P0n?Sh0G00o?cl0EUIF09nOW`1>CTh092@T03dm?@12@T80EUIF08f=S@16ATH06a/K03Lg
=`0i>CT0VYZJ0<G5a@0^;Rh0;2`/05eMG@1UIFD0PH6106YZJP0];Bd0;Rh^05]KF`1/K6`0Ogmo01dM
7@0e=CD0Fe]K09nOW`1LG5`0SXj>02`/;00U9BD0GUiN06QXJ016ATH0KFe]02DU9@0m?Cd0L71`05iN
GP1GEeL0J6QX01TI6@0l?3`0I6AT02XZ:P1BDU80BTY:01lO7`0f=SH0Ogmo02d];@0d=3@0CDe=04M7
A`0n?Sh0EEEE01lO7`16ATH0FEUI05=CD`1MGEd0Lg=c02/[:`1HF5P0KFe]06YZJP2[Zj/0J6QX02l_
;`1FEUH0Hf=S05iNGP2OWil0=3@d020P801QHF40OWin08F5Q@2j^[X092@T03<c<`12@T80KFe]096A
T@1oOgl0<C4a030`<023Ph<0MgMg0P1/K6`05`11@D40@d=307alO03AdM40M7Ad05]KF`0/;2`0?Cdm
07moO`25QHD0?Shn04i>CP0W9bL0DU9B09VIV@1WIfL0?Shn04Q8B012@T80GUiN071`L00c<c<0IfMW
0080F5QH03D0SXj>03Xj>P0i>CT0G5aL04=3@`1ZJVX0QhN702XZ:P0[:b/0EeMG08B4Q01?Cdl0ZZZZ
03`l?012@T80Bd];06a/K01DE5@0P82005ADE00d=3@0H61P0;6a/@2DU9@0]KFe03Ti>@1=CDd0Ogmo
05YJFP21PH40NWYj01dM7@0o?cl0NWYj04M7A`1NGUh0?Shn02l_;`1QHF40IFEU03<c<`1MGEd0820P
03@d=01gMgL0F5QH04Y:BP1BDU808R8R065QH@2UYJD0=cLg03Hf=P0204A4A01@08n?S`1CDe<0>3Ph
04a<C01FEUH0T92@04Y:BP0X:2P0CDe=04];B`1`L700LG5a03Hf=P11@D40GEeM04U9B@28R8P0F5QH
0410@01SHf<0D51@07moO`38b<P0?3`l06MWI`25QHD0E5AD0;>c/`25QHD0Bd];05MGE`1oOgl0MWIf
0<71`@1]KFd0>c/k04m?C`1^KVh0UYJF08>3P`19BDT0@410051@D026QXH0DU9B07]kN`1>CTh0;bl_
05eMG@2_[jl0Rh^;0:FUY@0d=3@0GEeM07=cL`28R8P0JFUY0820P00i>CT0DU9B0:2PX02RXZ80;Rh^
04Q8B00d=3@0=SHf05YJFP1VIVH0;Bd]03Hf=P11@D40?Shn08>3P`1dM7@0820P02PX:01CDe<0?Shn
08V9R@2ATI4>ooooo`0Q06YZJP1CDe<0;bl_05=CD`1VIVH0Bd];03Xj>P1KFe/0A4A402<S8`1>CTh0
M7Ad06UYJ@1ZJVX0H61P05EEE@1GEeL0@T9203hn?P0I6AT0=SHf02HV9P0`<300?3`l03Lg=`1BDU80
JVYZ06IVIP1YJFT0AdM706ATI01EEED0N7Qh0080K6a/08h0KFe]05QHF00U9BD0<c<c03Lg=`0i>CT0
DE5A03lo?`1GEeL0?3`l04Y:BP1@D500BTY:02h^;P0n?Sh0AdM702LW9`0O7al0=3@d01TI6@0n?Sh0
8R8R03<c<`0e=CD0<30`03Ti>@0W9bL0?3`l03lo?`1PH600EUIF04e=C@1QHF40Fe]K04A4A00T92@0
=SHf02h^;P0O7al09BDU05iNGP19BDT0D51@04=3@`0o?cl0Bd];03`l?018B4P092@T03<c<`0Y:BT0
:2PX04=3@`0g=cL0=SHf02/[:`0k>c/09RHV02LW9`13@d<0>3Ph03@d=010@4009bLW03lo?`1FEUH0
C4a<04I6AP1]KFd0GUiN03Hf=P10@400?Cdm04U9B@1ADE40@T92059BDP0l?3`0:BTY03dm?@1=CDd0
F5QH034a<@0m?Cd0H61P05eMG@1JFUX0EEEE06QXJ01eMGD0HF5Q08V9R@0E5AD0EUIF055AD@1MGEd0
>c/k092@T01jNWX09RHV0451@@1BDU80<S8b059BDP0c<c<0?3`l02PX:01WIfL0`/;207IfMP1XJ6P0
C4a<04m?C`0g=cL0B4Q805ADE00T92@0BTY:05]KF`071`L0M7Ad03dm?@1RHV80:BTY05IFEP0N7Qh0
9bLW03Hf=P1GEeL0EEEE030`<00o?cl0@d=305IFEP0R8R80HV9R07YjNP0m?Cd0?Shn05=CD`1EEED0
WYjN0P1>CTh0?@17AdL08R8R02TY:@0a<C40A4A402LW9`1bLW80MgMg034a<@0d=3@0BDU904E5A@0b
<S80?Shn01XJ6P0h>3P0>SXj03dm?@0k>c/0Hf=S02XZ:P1ADE40@d=305MGE`2VYZH0GUiN05=CD`1@
D500KVi^059BDP1;Bd/0HV9R03Lg=`0l?3`0@D5104E5A@1>CTh0Ph>307UiN@1EEED0OGem04Q8B01M
GEd0<S8b04a<C00b<S80AdM706EUI@0i>CT0?clo04];B`1FEUH0<S8b051@D00Y:BT0;bl_05iNGP1<
C4`0>CTi0410@00`<3000P0i>CT01P1=CDd09BDU04E5A@1/K6`092@T030`<080C4a<01`0DU9B05AD
E01^KVh0IVIV08B4Q01dM7@0A4A407YjNP10@400L71`05QHF01GEeL0C4a<05QHF01NGUh0CDe=06AT
I00f=SH0IVIV09:BTP1kNg/0FEUI04];B`1cLg<0Cdm?069RHP1NGUh0GEeM4?ooool000kooooo04@0
GemO079bLP2;Rh/0IFEU051@D01NGUh0C4a<08V9R@0e=CD0@T9206m_K`2;Rh/0/[:b02TY:@0_;bl0
CTi>06ATI02TY:@0WIfM024Q8@1;Bd/0JVYZ05=CD`2>SXh0?Cdm02l_;`1JFUX0SHf=069RHP1_Kfl0
<S8b04A4A01MGEd0EUIF059BDP2=SHd0<30`04e=C@1DE5@0L71`05YJFP27QhL0>CTi03Ph>01kNg/0
?Cdm03hn?P1ZJVX0;bl_05EEE@18B4P0<30`04a<C00^;Rh0>CTi05eMG@0l?3`0>CTi0451@@0m?Cd0
GUiN09VIV@0b<S80=CDe03hn?P0f=SH0KFe]0;>c/`80;Rh^0180IFEU04a<C020P800RXZ:01DE5@0T
92@0G5aL04m?C`27QhL0JFUY020P800_;bl0@41004E5A@2][Jd0CTi>03Ph>01CDe<204I6AP2l08R8
R00h>3P0>CTi04Y:BP22PX80@D51069RHP0Y:BT09BDU04m?C`21PH40;bl_03dm?@0U9BD0>SXj07in
OP1<C4`09bLW03`l?00n?Sh0IfMW0:RXZ00e=CD08B4Q04M7A`1;Bd/0TY:B0;2`/00V9RH0<30`0410
@01fMWH0QXJ60:j^[P0M7Ad0ADE5069RHP1NGUh0O7al07IfMP0];Bd0GUiN06IVIP1SHf<0WIfM061P
H00c<c<0IfMW061PH00k>c/0][Jf08J6QP0l?3`0O7al079bLP1ADE40U9BD075aL@10@400Z:RX08J6
QP1VIVH0MWIf04A4A00];Bd0L71`06UYJ@1GEeL0OGem01hN7P0^;Rh0Ng]k07moO`1<C4`0N7Qh038b
<P1>CTh0K6a/04=3@`1<C4`0EEEE02XZ:P1UIFD0WIfM04e=C@1:BTX0IfMW04U9B@1hN7P0TY:B02PX
:01;Bd/0IVIV07AdM02:RXX0;Rh^02HV9P10@400E5AD096AT@1jNWX0=SHf0410@01@D500H61P07Yj
NP0L71`0@d=304e=C@1MGEd0Fe]K07IfMP0f=SH0MWIf06ATI01HF5P0N7Qh04=3@`19BDT0WinO061P
H022PX80CDe=04U9B@1^KVh0JVYZ05mOG`0g=cL092@T04];B`2MWId0LW9b051@D00g=cL0@D51059B
DP2JVYX0@d=304U9B@0c<c<0ADE505ADE02=SHd0ATI6051@D00Q8B40FEUI0:FUY@1>CTh0:2PX04E5
A@19BDT0@T9209jNWP2UYJD0=3@d03Lg=`1NGUh0Kfm_0:FUY@18B4P0:b/[08R8R01bLW80IFEU0;Bd
]00k>c/0=CDe069RHP2:RXX0MGEe07moO`0/;2`08B4Q04U9B@1dM7@0BDU908>3P`1IFET06QXJ05eM
G@1eMGD0FUYJ05MGE`1`L7007QhN0492@P1jNWX>ooooo`0O079bLP1aLG40B4Q804E5A@1RHV80EUIF
04Y:BP1GEeL0IfMW03Ti>@1RHV80T92@08V9R@28R8P0I6AT051@D01BDU80C4a<059BDP0^;Rh0ADE5
030`<01BDU80>SXj02h^;P13@d<0BDU904Y:BP1YJFT0De=C069RHP0205QHF00?04M7A`1BDU80E5AD
055AD@0`<300:b/[02l_;`1>CTh0F5QH03dm?@19BDT0=CDe04a<C01@D500BTY:0080@T9200d0Cdm?
04A4A014A4@0BTY:02LW9`1<C4`0>CTi04M7A`15ADD0B4Q805=CD`0f=SH0D51@00<0K6a/0180?Cdm
0451@@1jNWX0H61P03<c<`0m?Cd0BTY:03Xj>P0i>CT0@D5104e=C@1IFET0F5QH04i>CP1NGUh0?clo
05QHF00l?3`203hn?P0P03Lg=`1MGEd0=SHf04Y:BP11@D40@41003<c<`0e=CD0EUIF03Xj>P0c<c<0
De=C02l_;`16ATH0Bd];03Ti>@0m?Cd0B4Q804A4A00^;Rh0BDU90492@P1ADE40E5AD04Y:BP28R8P0
HV9R02<S8`14A4@0<S8b01lO7`0[:b/203lo?`1D03hn?P17AdL0HF5Q09>CT`1hN7P0Ph>309>CT`1J
FUX0Rh^;07AdM01PH600?Shn05IFEP1BDU80<C4a04A4A01TI6@0>c/k065QH@0S8b<0LW9b0492@P16
ATH0`L7107UiN@1ADE40BDU9069RHP0k>c/0FEUI06=SH`0/;2`0E5AD065QH@0@4100I6AT05EEE@2;
Rh/0<c<c05QHF00_;bl0=cLg04M7A`1ADE40GUiN02@T900j>SX0C4a<05QHF00L71`0EUIF07EeM@0n
?Sh0BTY:04i>CP0g=cL0IFEU02`/;01CDe<0ATI601@D5016ATH0DE5A04A4A00Y:BT0CDe=07AdM01?
Cdl0;bl_03`l?00n?Sh0=cLg05EEE@0N7Qh0Cdm?04A4A010@400DE5A08^;R`1FEUH0DU9B055AD@0e
=CD0C4a<0P0h>3P01`10@400C4a<04Q8B00V9RH0BTY:03lo?`14A4@00P0b<S80700l?3`0@d=305QH
F00j>SX0LG5a03Ti>@1ADE40<c<c055AD@0X:2P0IFEU0:BTY00k>c/0Bd];03Xj>P0a<C40820P04m?
C`0f=SH0B4Q805aLG01FEUH0<C4a03<c<`0X:2P0DE5A05mOG`2BTY8203Lg=`0Q06ATI00k>c/0EEEE
05=CD`1GEeL0LW9b05eMG@1`L700Cdm?06IVIP1UIFD0Ph>30=[JfP1gMgL0E5AD04E5A@19BDT0<S8b
02TY:@0f=SH0CTi>05eMG@0W9bL0@D5107moO`1OGel0@T9204m?C`1JFUX0@d=305IFEP1LG5`0HF5Q
013ooooo000>ooooo`1N0;Vi^@1LG5`0K6a/09fMW@2UYJD0@41005EEE@1JFUX0IfMW07IfMP0Y:BT0
?Shn075aL@18B4P0?Shn04a<C00j>SX0BDU908j>SP1DE5@0CTi>03/k>`0U9BD0FEUI06IVIP0d=3@0
BTY:01XJ6P1:BTX0IVIV02l_;`0b<S80F5QH01dM7@1EEED0/;2`069RHP0c<c<0CDe=028R8P1MGEd0
_[jn07]kN`0a<C40DU9B04A4A01PH600_;bl03Lg=`19BDT0Bd];05ADE02ZZZX0=CDe02@T901HF5P0
HV9R07alO01_Kfl07alO03`l?01_Kfl0De=C06YZJP1^KVh092@T02l_;`1PH600I6AT04M7A`0L71`0
=CDe04M7A`24Q8@0<c<c02l_;`12@T80?3`l05IFEP1^KVh0:2PX034a<@0l?3`0?Cdm08J6QP1cLg<0
8B4Q030`<00i>CT0@4100::RXP1HF5P07QhN03`l?080FEUI0;H0V9RH030`<00`<300@D5107=cL`13
@d<0QXJ602<S8`1@D500Bd];05]KF`1MGEd0ZZZZ04e=C@0o?cl0ATI608j>SP1ADE40WIfM03Xj>P14
A4@0MGEe0861P@1OGel0S8b<05=CD`0o?cl0Lg=c05=CD`17AdL0SHf=02TY:@0b<S80ZJVY03/k>`1a
LG40H61P02d];@1<C4`0YjNW03`l?01`L700EEEE038b<P1JFUX0Vi^K04A4A01=CDd0?3`l04E5A@1e
MGD0MWIf03Ti>@1JFUX0>CTi07alO032`/80KVi^034a<@1?Cdl0Fe]K08>3P`3Kfm/0?3`l02DU9@11
@D40KVi^06ATI031`L40B4Q803<c<`1;Bd/0GemO09nOW`26QXH0;2`/030`<012@T80Lg=c071`L01K
Fe/0@41005UIF@1YJFT0BTY:04E5A@1<C4`0<C4a08j>SP1oOgl0CDe=07MgM`0f=SH0Bd];09FEU@0k
>c/0DE5A05aLG00T92@0Ng]k06YZJP0o?cl0DE5A03Xj>P1BDU80Ng]k04U9B@1KFe/09bLW05IFEP2d
];@0MgMg03Xj>P1MGEd0?Cdm09^KV`2;Rh/0;Rh^05ADE01SHf<0W9bL04U9B@0U9BD0;Bd]06=SH`1Y
JFT0`/;2055AD@0i>CT0D51@06e]K@1FEUH0Shn?02TY:@11@D40GEeM07EeM@1QHF40TI6A02XZ:P0f
=SH0C4a<06UYJ@1IFET0YjNW07QhN00f=SH0@410059BDP17AdL0Q8B40:6QX@10@400EUIF07IfMP14
A4@0WinO051@D00h>3P0Jf][06ATI013@d<0SHf=05UIF@0U9BD0J6QX06IVIP0/;2`0O7al04M7A`0M
7Ad0A4A407QhN00_;bl0>3Ph038b<P0S8b<0=cLg07AdM00U9BD092@T03Lg=`kooooo05P0JFUY055A
D@0T92@0EEEE059BDP13@d<0;2`/04Y:BP1;Bd/0@d=3055AD@17AdL0A4A405UIF@1mOGd0Cdm?059B
DP0m?Cd0?Shn034a<@15ADD0;2`/02h^;P13@d<0=SHf04U9B@14A4@0BTY:07=cL`17AdL0BDU905YJ
FP17AdL0ADE5055AD@1IFET0JFUY030`<00c<c<0:b/[02h^;P0m?Cd0<S8b03/k>`0X:2P0ADE5055A
D@0f=SH0>c/k0451@@19BDT08b<S01lO7`0i>CT08B4Q04E5A@0X:2P0:BTY0410@01:BTX0B4Q804A4
A01=CDd0E5AD08N7Q`1RHV80D51@06EUI@1fMWH0JFUY04a<C01FEUH092@T01XJ6P0O7al0GUiN04i>
CP14A4@0@d=303lo?`1=CDd0BDU9059BDP0k>c/0CDe=03De=@0j>SX0C4a<0`1=CDd0A`1EEED0=cLg
030`<013@d<0A4A4034a<@1CDe<0Bd];03lo?`0k>c/0@D5104A4A01ADE40B4Q802l_;`1BDU80ADE5
05]KF`1OGel0?3`l08N7Q`1ZJVX0?Cdm04m?C`0g=cL0@d=302/[:`1<C4`0Bd];04E5A@1FEUH0EeMG
07YjNP1SHf<0KVi^07YjNP0Z:RX0HF5Q05QHF02>SXh0AdM706QXJ01YJFT0=SHf04=3@`1IFET09BDU
0492@P0Y:BT0AdM703dm?@1ADE40X:2P05IFEP1MGEd0CTi>08:2PP0d=3@0@41005iNGP0@4100C4a<
06IVIP0@4100FEUI03<c<`16ATH05aLG04Y:BP16ATH0>SXj0080AdM70400CTi>03De=@1CDe<0Fe]K
05ADE00R8R80NGUi09fMW@1<C4`0CTi>059BDP0c<c<0QhN702LW9`1NGUh0E5AD01LG5`0[:b/0<c<c
04e=C@0L71`0IFEU07alO01;Bd/0C4a<03lo?`14A4@0=SHf04A4A00O7al0C4a<05]KF`1PH600B4Q8
08:2PP1?Cdl0De=C04a<C018B4P0FEUI02h^;P12@T80C4a<05iNGP1PH600CDe=05EEE@1=CDd0Bd];
03Ti>@0`<300AdM705eMG@1TI6@0>c/k0:RXZ01ADE40E5AD030`<01UIFD0=cLg04i>CP1iNGT0;Bd]
0P12@T80<01=CDd0?3`l05QHF011@D40>c/k065QH@14A4@0<30`04E5A@0b<S80C4a<04U9B@1FEUH0
:2PX038b<P1LG5`0;bl_03Lg=`11@D40Hf=S08>3P`1BDU80Cdm?05=CD`1iNGT0CTi>04a<C01XJ6P0
8b<S05EEE@0m?Cd0C4a<04E5A@16ATH0F5QH02l_;`1]KFd0ADE504i>CP1UIFD0GemO04Y:BP0i>CT0
B4Q804M7A`1ADE40GEeM07EeMA3ooooo000>ooooo`0Q0:BTY01BDU80GemO05MGE`2TY:@0EeMG04A4
A01<C4`0?Cdm09VIV@0h>3P0=cLg051@D019BDT0P8200;:b/P0n?Sh0FUYJ07AdM01DE5@0/k>c09bL
W00X:2P0>CTi071`L02OWil0`l?303Ti>@0T92@0GEeM05UIF@23Ph<0Xj>S00<0>3Ph05H0Lg=c04a<
C02:RXX0>3Ph03Hf=P1>CTh0R8R806YZJP1MGEd071`L03Hf=P1JFUX0De=C04Y:BP0G5aL0>c/k05MG
E`1YJFT0>CTi051@D00`<300CDe=06][J`0f=SH0;bl_04U9B@15ADD0C4a<06UYJ@0M7Ad0820P04A4
A00i>CT0O7al02`/;00O7al092@T05EEE@1>CTh0F5QH05UIF@0b<S80?Shn07QhN01DE5@0KFe]04e=
C@0e=CD0EUIF0:FUY@1WIfL0NWYj06m_K`0_;bl0Fe]K061PH011@D40F5QH04I6AP0d=3@0F5QH051@
D018B4P0KFe]03Ph>00`<300GemO03dm?@1JFUX0KFe]01hN7P0h>3P0Bd];04Q8B00i>CT0DU9B02XZ
:P0f=SH0S8b<04];B`10@400>c/k03Xj>P18B4P0S8b<06QXJ080ADE509X0H61P06IVIP2j^[X0DE5A
01/K6`1YJFT0FEUI09nOW`2KVi/0=cLg02h^;P1oOgl0OGem0;Jf]P1nOWh0>CTi04Y:BP2JVYX0Ogmo
0:2PX00o?cl0;bl_04m?C`1kNg/0M7Ad08B4Q00c<c<0>SXj07QhN02QXJ40HF5Q071`L00o?cl0Bd];
096AT@2][Jd0Bd];069RHP10@400=CDe08b<S02BTY80D51@05ADE00a<C40FEUI09JFUP1:BTX0ADE5
0410@00_;bl0@D5106QXJ00g=cL0FEUI03lo?`1SHf<0PX:205ADE00O7al0A4A404I6AP1GEeL0[jn_
04M7A`0h>3P0Kfm_04E5A@1nOWh0NWYj02<S8`1OGel0I6AT061PH02IVIT07AdM03lo?`1SHf<0P820
0;Jf]P0m?Cd08B4Q05eMG@2MWId0MGEe06e]K@1=CDd0F5QH07=cL`1:BTX0IFEU04U9B@22PX80_[jn
06MWI`1fMWH0SHf=04];B`1hN7P0Zj^[05QHF026QXH0FEUI05QHF01^KVh0Q8B406MWI`2GUiL0B4Q8
03hn?P1RHV80FUYJ0410@01FEUH0;bl_03lo?`1_Kfl0YjNW03`l?01LG5`0DU9B038b<P1VIVH0alO7
07=cL`1FEUH0=3@d03dm?@2ZZZX0K6a/03lo?`1HF5P0@d=304e=C@2c/k<0GUiN02<S8`15ADD0Cdm?
04U9B@2YZJT0MGEe02LW9`0d=3@0G5aL06IVIP1nOWh0ADE5018B4P0_;bl0BDU906ATI01^KVh0ADE5
3_ooool0301VIVH0EEEE038b<P0h>3P0De=C06a/K01MGEd0Fe]K05ADE00[:b/0B4Q805ADE080GemO
03X0BDU90410@016ATH0?clo055AD@0_;bl0<S8b01lO7`0c<c<0=cLg04M7A`1IFET0Hf=S055AD@1I
FET0EEEE069RHP1?Cdl0KFe]07EeM@2?Shl0I6AT04i>CP0Q8B407alO02PX:014A4@0Cdm?02h^;P15
ADD0=cLg03dm?@0g=cL0Bd];03hn?P0_;bl0<30`02d];@0N7Qh0>c/k028R8P0^;Rh0:RXZ02l_;`0d
=3@09bLW02/[:`0c<c<0:b/[030`<01PH600CDe=02l_;`0V9RH0A4A40410@00e=CD0;bl_0P0O7al0
LP0S8b<0=3@d04m?C`19BDT0ATI604E5A@1:BTX0>SXj0492@P0O7al0:BTY01hN7P0h>3P0<c<c030`
<00d=3@09bLW03Lg=`0N7Qh0=cLg03dm?@0`<300;bl_04=3@`0e=CD0>c/k04A4A01?Cdl0@d=3059B
DP1<C4`0De=C06IVIP12@T80E5AD04e=C@0o?cl0Q8B404e=C@0i>CT0IFEU0451@@1YJFT0>c/k04m?
C`1:BTX0EeMG06=SH`1>CTh0RXZ:07QhN02ZZZX0W9bL028R8P1aLG40I6AT06QXJ00l?3`0KFe]059B
DP15ADD0DE5A06ATI00k>c/0Bd];02/[:`1?Cdl0AdM705EEE@2SXj<0NGUi09JFUP1cLg<0FEUI03`l
?016ATH0De=C02TY:@1bLW80J6QX018B4P1^KVh0;bl_05aLG00Z:RX0ADE503Lg=`1TI6@0De=C059B
DP0l?3`06a/K03hn?P1:BTX0Kfm_02DU9@0k>c/0GUiN0410@017AdL0Cdm?04Y:BP1fMWH0;2`/06IV
IP1kNg/051@D02@T900m?Cd0Ogmo02l_;`1SHf<0JVYZ03hn?P80Bd];01X0@T9203Ph>011@D409bLW
05EEE@13@d<0;bl_03Ph>022PX80DE5A07QhN01/K6`0@41004U9B@0i>CT0ATI604m?C`1FEUH0CTi>
02l_;`1;Bd/0=CDe03lo?`0a<C40<c<c03Ti>@80EEEE0200:BTY04m?C`19BDT0EeMG020P801;Bd/0
=3@d04i>CP2FUYH0=cLg04M7A`10@400CTi>030`<00i>CT0;bl_0492@P1<C4`0=3@d05EEE@1CDe<0
Cdm?06i^KP1HF5P0Ph>305iNGP17AdL0I6AT04e=C@1:BTX0B4Q804I6AP80MWIf01X0E5AD03Lg=`1U
IFD0HV9R06QXJ01/K6`0=3@d065QH@1MGEd0B4Q803/k>`13@d<0FEUI02LW9`1HF5P0ATI6075aL@1M
GEd0GUiN06][J`1@D500DU9B055AD@1EEED0NWYj08>3Pa3ooooo000>ooooo`3I0:FUY@1nOWh0Z:RX
05mOG`24Q8@0Ph>308J6QP2RXZ80?Cdm05iNGP2DU9@0K6a/07inOP0G5aL0;2`/07UiN@1=CDd0FEUI
05mOG`0T92@0=SHf06IVIP12@T80EEEE02LW9`0b<S80MGEe06=SH`0i>CT0G5aL01XJ6P1DE5@0QHF5
02d];@0Z:RX0Bd];0410@01=CDd0O7al02HV9P0c<c<0I6AT04M7A`1cLg<0Z:RX02/[:`0m?Cd0BDU9
0451@@2SXj<0>SXj02LW9`1PH600CDe=06MWI`2`/;00;2`/03hn?P1QHF40?clo069RHP2YZJT0?Cdm
03lo?`1MGEd0ADE5065QH@26QXH0=CDe04];B`1dM7@0;Rh^05MGE`0n?Sh09BDU03`l?01RHV80:BTY
05IFEP17AdL0=SHf0410@01LG5`0=CDe059BDP0V9RH0De=C06YZJP1lO7`09BDU04U9B@13@d<0=cLg
07emO@1SHf<0<30`05eMG@0j>SX0@D5108^;R`1IFET07QhN059BDP10@400K6a/0<30`00/;2`09RHV
05eMG@1hN7P0JVYZ0;:b/P0[:b/09RHV05=CD`1_Kfl0RXZ:07EeM@0N7Qh0<c<c069RHP1nOWh0N7Qh
05IFEP0Y:BT0ADE505mOG`1oOgl0B4Q8055AD@0[:b/0ATI607AdM01UIFD0De=C03Lg=`0m?Cd0EUIF
0:>SX`1LG5`0QHF5061PH010@400TY:B08f=S@0n?Sh0@d=303Ti>@1;Bd/0KVi^04I6AP0k>c/0Fe]K
05=CD`1EEED0Ng]k03Ph>00k>c/0F5QH06MWI`1`L700NWYj05ADE013@d<0FUYJ08^;R`1NGUh0/;2`
03Hf=P1?Cdl0FUYJ04a<C01mOGd0YjNW02DU9@1FEUH0Fe]K04Q8B02/[:`0EeMG01lO7`19BDT0Ng]k
05=CD`1^KVh0:2PX02l_;`1fMWH0IFEU05YJFP17AdL08b<S03hn?P1lO7`0Cdm?04Q8B00[:b/0?Shn
071`L01@D500ATI603hn?P0m?Cd0U9BD06e]K@0m?Cd0CTi>02TY:@1ADE40U9BD03dm?@1JFUX0E5AD
059BDP2RXZ80KFe]03Xj>P02051@D00k07UiN@2LW9`0?Shn0410@01PH600PH610:FUY@2FUYH0?3`l
065QH@1LG5`0N7Qh0=;BdP25QHD0;bl_0861P@20P800Ph>30::RXP0o?cl09RHV04I6AP1lO7`0OGem
0;^k^`1XJ6P07AdM03hn?P2=SHd0XZ:R0;no_`0k>c/0:b/[071`L02l_;`0KVi^07YjNP0n?Sh0GUiN
07EeM@2IVIT0FEUI07emO@0_;bl0;2`/05mOG`2NWYh0E5AD04=3@`1IFET0<30`06MWI`1MGEd0>3Ph
04I6AP18B4P0<30`08f=S@23Ph<03_ooool0D@1XJ6P0JVYZ034a<@17AdL0C4a<0451@@0[:b/0H61P
06YZJP0b<S80C4a<096AT@2GUiL0SHf=06][J`1FEUH0FUYJ05=CD`12@T80>SXj03dm?@0c<c<0ATI6
03Xj>P0/;2`0@D5104i>CP1HF5P0TY:B04M7A`1HF5P0T92@05QHF00o?cl0CTi>06QXJ01[Jf/0=CDe
03hn?P0W9bL0=3@d04A4A013@d<0Bd];02`/;01CDe<0GemO04e=C@1@D500BDU904i>CP19BDT0?Cdm
05UIF@0T92@0@d=304A4A00c<c<0@d=304e=C@19BDT0;Bd]04M7A`19BDT0Kfm_071`L00c<c<0>CTi
07AdM01^KVh0=3@d05=CD`0m?Cd09bLW03<c<`1?Cdl0?Cdm04U9B@1=CDd0@41004];B`0203/k>`33
034a<@0j>SX09BDU028R8P15ADD0>SXj03/k>`0m?Cd0BDU9018B4P0Q8B40>3Ph038b<P0P8200=3@d
03Hf=P0g=cL0<S8b02`/;00j>SX0A4A403Xj>P0m?Cd0CTi>04M7A`14A4@0Cdm?03`l?00o?cl0?Cdm
038b<P11@D40>CTi03De=@0o?cl0>CTi06YZJP17AdL0Jf][06ATI02WYjL0JFUY065QH@1mOGd0GUiN
09RHV021PH40Xj>S05MGE`25QHD0G5aL02<S8`14A4@0I6AT04A4A01JFUX0ATI604i>CP0d=3@0GemO
08f=S@1lO7`0DU9B04A4A01ZJVX0@41006i^KP1hN7P0<S8b04];B`1CDe<05aLG04m?C`0n?Sh0J6QX
02<S8`1CDe<09BDU05IFEP0i>CT0>c/k04A4A01;Bd/0KVi^05]KF`11@D406QXJ04E5A@1PH600=cLg
055AD@1?Cdl0Bd];07]kN`0`<300FUYJ05]KF`0Q8B40:2PX03Ti>@1?Cdl0@D5105iNGP20P800LG5a
069RHP17AdL0EeMG038b<P0i>CT061PH04A4A01FEUH0ADE502h^;P1^KVh0<30`04=3@`0j>SX0>3Ph
05ADE00R8R80=SHf02l_;`1CDe<0DE5A02l_;`12@T80@d=304M7A`0/;2`0;Bd]0492@P1ZJVX0I6AT
01lO7`22PX80CDe=05]KF`0X:2P0LW9b03Ph>00c<c<0HV9R02<S8`0f=SH0<C4a03Lg=`0Y:BT0>c/k
03Hf=P0k>c/0EEEE02h^;P0c<c<0A4A404i>CP0l?3`0AdM708^;R`1IFET0ATI6071`L00h>3P0Bd];
05YJFP1CDe<0Ng]k04U9B@27QhL0BTY:05iNGP1HF5P0=cLg030`<00P8200Q8B405QHF01=CDd0ADE5
04U9B@1DE5@0<S8b07IfMP1=CDd0>CTi08>3P`1OGel0Bd];05EEE@1IFET0Cdm?06YZJP1LG5`0GUiN
013ooooo000>ooooo`0Q0:ZZZP1DE5@0IfMW07inOP20P800@41004a<C023Ph<0IfMW06EUI@0g=cL0
IfMW07moO`1:BTX0Bd];05iNGP0o?cl0Ng]k09RHV00h>3P0ADE504Q8B00/;2`0MGEe06=SH`0o?cl0
DU9B02`/;018B4P0Q8B402h^;P1RHV80I6AT0080B4Q804@0M7Ad0410@012@T80CTi>02d];@1IFET0
TI6A03/k>`11@D40Jf][04e=C@2PX:00V9RH02PX:01EEED0Hf=S07]kN`2MWId0>CTi02@T901QHF40
I6AT06e]K@1MGEd08B4Q02d];@1SHf<0BDU906YZJP1ADE408R8R04M7A`2HV9P0>c/k04m?C`0b<S80
>3Ph05UIF@1hN7P0=cLg04e=C@1?Cdl0=3@d06a/K01[Jf/0>SXj04M7A`1IFET0@d=308f=S@1HF5P0
;bl_04];B`1QHF40BTY:08Z:RP15ADD07QhN04=3@`1TI6@0DE5A0820P00_;bl0:BTY04Y:BP1YJFT0
>SXj07MgM`80;2`/01X0E5AD05eMG@0f=SH0Ng]k04Y:BP0j>SX0Bd];04a<C01CDe<0Ph>303hn?P1:
BTX0Fe]K03De=@0o?cl0LG5a04Q8B01?Cdl0Ogmo038b<P1XJ6P0Kfm_03`l?013@d<0NWYj03De=@80
DU9B0940>SXj069RHP1_Kfl0>CTi05IFEP1IFET0BTY:07IfMP2UYJD0Bd];05YJFP1EEED0DE5A08:2
PP1NGUh0BDU906i^KP1>CTh0FUYJ0:NWY`1SHf<0=3@d04m?C`1ADE40GEeM0;6a/@1>CTh0=cLg059B
DP1RHV80W9bL0;bl_014A4@0?Cdm05QHF01<C4`0N7Qh08f=S@0Z:RX0=CDe05IFEP1>CTh0HV9R04a<
C00_;bl0Cdm?07IfMP19BDT0D51@04U9B@1=CDd0N7Qh04a<C00k>c/0GEeM04Q8B01XJ6P0MWIf038b
<P1=CDd0BTY:03Hf=P1WIfL0A4A4051@D01NGUh0=3@d06=SH`1NGUh0?3`l05aLG014A4@0GemO0:NW
Y`1IFET0CDe=061PH017AdL0N7Qh079bLP0d=3@0CTi>065QH@1nOWh0J6QX03Lg=`19BDT0JFUY06i^
KP2KVi/0G5aL03Ph>01TI6@0OWin061PH01mOGd0=SHf03/k>`1EEED0Ti>C06a/K02@T900>3Ph051@
D01_Kfl0O7al03lo?`24Q8@0GemO04=3@`1_Kfl0Ti>C05]KF`1[Jf/0QXJ605YJFP2RXZ80EeMG04U9
B@1cLg<0EEEE04a<C022PX80Cdm?03Ph>01fMWH0GUiN04a<C01WIfL0H61P038b<P1;Bd/0HF5Q04=3
@`0n?Sh0G5aL02l_;`18B4P0@T9202`/;01ADE40Hf=S02HV9P1EEED0GemO00kooooo0540EUIF051@
D00b<S80=cLg04m?C`1>CTh0ATI605QHF01;Bd/0>3Ph04e=C@1BDU80C4a<05EEE@1MGEd0E5AD05QH
F01=CDd0@T9202TY:@13@d<0?3`l03Ti>@17AdL0@d=3055AD@14A4@0B4Q804a<C015ADD0DE5A05QH
F01MGEd0Cdm?05ADE01KFe/0Cdm?028R8P0b<S80>CTi02h^;P1;Bd/0>c/k04U9B@0g=cL0?clo04=3
@`1ADE40?Cdm04Q8B019BDT0=SHf02HV9P0n?Sh0;2`/04e=C@14A4@0=CDe0410@00b<S80=cLg03De
=@0n?Sh0<c<c06YZJP1QHF40A4A404];B`1@D500Bd];03Ph>017AdL0?clo03Lg=`0k>c/0@T9204Y:
BP0m?Cd0>SXj04=3@`0i>CT00P11@D408`0V9RH0>SXj02HV9P0l?3`0@d=303Xj>P0i>CT0=3@d04U9
B@0K6a/0=cLg03hn?P0`<300:RXZ0410@00j>SX0>3Ph0451@@0n?Sh0?clo04U9B@0e=CD0BTY:05AD
E00];Bd0ATI605EEE@0f=SH0Fe]K02d];@0_;bl0Bd];030`<00b<S80ATI60080>SXj01@0BTY:06i^
KP18B4P0H61P04Q8B02<S8`0M7Ad02d];@1@D500DE5A06EUI@19BDT0L71`04];B`11@D40EEEE07Ui
N@0f=SH0ADE503`l?080GEeM0200HF5Q09bLW01^KVh0QXJ605ADE01UIFD0?Cdm04Y:BP1NGUh08B4Q
05ADE01OGel07AdM06QXJ015ADD0LG5a03Ph>01OGel09bLW051@D00o?cl0GemO061PH00g=cL0@d=3
03dm?@1@D5008R8R05eMG@1RHV80A4A404i>CP80DE5A03h0OWin04i>CP1VIVH0Hf=S02`/;01@D500
DU9B05mOG`1ADE40HF5Q07=cL`1KFe/0IFEU03hn?P0i>CT0A4A404];B`19BDT0D51@04a<C01UIFD0
A4A406UYJ@1=CDd0HV9R04i>CP0g=cL0CTi>02LW9`0h>3P0>c/k055AD@1KFe/0BDU904M7A`0l?3`0
CDe=03`l?00a<C40De=C04i>CP1GEeL0=SHf07QhN01SHf<0C4a<02<S8`1=CDd0BDU903Ph>01bLW80
:b/[03`l?010@400A4A403@d=015ADD0>3Ph04I6AP1GEeL0CDe=04A4A080Bd];02H0FUYJ06QXJ020
P800HV9R03Ti>@1VIVH0BTY:0451@@0e=CD0BTY:07]kN`1DE5@0F5QH05mOG`1eMGD0Hf=S051@D01=
CDd0:b/[06YZJP19BDT0G5aL03Ti>@1GEeL0EUIF038b<P1YJFT0Bd];04Q8B01ZJVX0IVIV04];B`1F
EUH0Fe]K04Q8B01RHV80G5aL06QXJ13ooooo000>ooooo`0509ZJVP1NGUh0LW9b055AD@1OGel00P1C
De<08`1MGEd0>SXj06MWI`1@D500>c/k05YJFP0[:b/0HV9R0:VYZ@1NGUh0>CTi04A4A016ATH0LW9b
0820P00Q8B40=CDe04];B`2e]KD0Ti>C01PH600U9BD0C4a<04I6AP2^[Zh0DU9B00h>3P0k>c/0B4Q8
04a<C01nOWh0Lg=c01PH601=CDd0Q8B40080J6QX03/0?Shn01PH601GEeL0YJFU05IFEP1GEeL0:RXZ
02h^;P1[Jf/0Lg=c05UIF@1iNGT09BDU05=CD`1XJ6P0<S8b05UIF@1WIfL0>CTi07moO`1iNGT0;Bd]
034a<@1>CTh0<30`079bLP14A4@05aLG03lo?`0l?3`08R8R0492@P1VIVH092@T03dm?@0f=SH0=cLg
07MgM`1PH6007AdM04Y:BP1WIfL0H61P0:j^[P34a<@0:RXZ03dm?@1RHV80Cdm?08b<S01lO7`0@d=3
059BDP1ADE40J6QX09:BTP0a<C40:BTY04M7A`02065QH@0707emO@0I6AT0:RXZ03dm?@1ZJVX0>SXj
096AT@0202/[:`0X055AD@1LG5`0GUiN05UIF@0W9bL0@41004m?C`1KFe/0>CTi03dm?@0Q8B40Hf=S
08:2PP0c<c<0;Rh^059BDP0n?Sh0P82007emO@0Q8B40=cLg05EEE@1WIfL0ZZZZ04M7A`0a<C40@d=3
061PH01`L700^[Zj04I6AP0[:b/0@d=307=cL`1RHV80PH6103Ti>@15ADD0I6AT07UiN@80JFUY07X0
9bLW034a<@1HF5P0SXj>06EUI@29RHT0?clo03hn?P1XJ6P0R8R805eMG@1nOWh0;Bd]04e=C@2PX:00
MgMg06e]K@1JFUX09BDU059BDP1dM7@0:b/[055AD@0n?Sh0<30`04Q8B01YJFT0=CDe04m?C`0Y:BT0
@41008:2PP0d=3@0>SXj03@d=01CDe<0RHV908f=S@0Z:RX0?3`l02h^;P1]KFd0Xj>S02/[:`0m?Cd0
HF5Q08V9R@2;Rh/0=3@d02TY:@1`L700GUiN08j>SP1NGUh0>c/k05UIF@2SXj<0G5aL09BDU01KFe/0
FEUI07UiN@1ADE40H61P06e]K@0b<S80Hf=S096AT@1?Cdl0L71`06EUI@0a<C40K6a/05mOG`1VIVH0
TY:B05IFEP0o?cl0IfMW07IfMP19BDT0QXJ603`l?00a<C40>SXj0820P01:BTX0Jf][01hN7P0e=CD0
E5AD09>CT`1jNWX0PX:203De=@0f=SH0MWIf09BDU01<C4`0CDe=05=CD`1<C4`0SXj>06=SH`0S8b<0
>CTi05iNGP1<C4`0YJFU09JFUP0U9BD0>CTi05=CD`1ADE40UYJF06YZJP0D51@0?Cdm06IVIP1?Cdl0
UYJF05MGE`kooooo02<0GUiN06m_K`16ATH0@T9205iNGP1<C4`0>c/k06a/K01XJ6P0A4A404];B`1U
IFD0IfMW065QH@1DE5@0C4a<06ATI01NGUh0Bd];030`<019BDT0<S8b03Lg=`0k>c/0<30`03dm?@1<
C4`0J6QX0861P@1?Cdl0FEUI07inOP1NGUh0EeMG05eMG@02069RHP1V030`<00d=3@0;Bd]03dm?@1?
Cdl0@d=305IFEP0_;bl0Bd];051@D01LG5`0Bd];055AD@1;Bd/0;bl_01dM7@1=CDd0>3Ph03hn?P1;
Bd/0820P04M7A`14A4@0Cdm?04I6AP12@T80BTY:06e]K@1VIVH0;Rh^034a<@1QHF40BTY:020P8018
B4P0I6AT03Lg=`13@d<0C4a<03/k>`1;Bd/0BDU903`l?011@D40=CDe03Lg=`0_;bl0=3@d02TY:@0M
7Ad0BTY:0451@@0c<c<0?Cdm04=3@`0L71`06a/K03Lg=`0a<C407AdM020P800`<300<c<c04I6AP0T
92@0?Cdm05EEE@0Z:RX0@D5105ADE01<C4`0D51@04U9B@0/;2`0>3Ph02`/;00^;Rh0@D5103<c<`0P
8200<C4a03Ti>@1ZJVX0ATI604i>CP1HF5P0PH6103Lg=`1QHF40QXJ60492@P2:RXX0L71`06EUI@1?
Cdl0FUYJ04a<C00S8b<0?3`l05eMG@1BDU80GUiN0P19BDT0>`11@D40KFe]0<71`@1]KFd0EeMG04=3
@`1DE5@0<c<c06][J`1cLg<0@T9203`l?011@D4051@D04m?C`0i>CT0MWIf03Ph>017AdL0;Rh^069R
HP12@T80<30`03<c<`0f=SH0KFe]04a<C01:BTX05ADE05IFEP1SHf<0CDe=04i>CP1JFUX0De=C061P
H017AdL0N7Qh0861P@0M7Ad0@D510410@01ADE40?clo06][J`2;Rh/0BTY:0410@01=CDd0D51@05MG
E`16ATH0?clo04i>CP1GEeL0F5QH05=CD`2JVYX0Jf][0080FUYJ0140B4Q805=CD`1ZJVX0?Shn04Q8
B01OGel0Kfm_04m?C`1MGEd0BDU904a<C01:BTX0Fe]K04m?C`1ADE40GEeM04E5A@0207EeM@0i06YZ
JP0/;2`0DE5A03dm?@1:BTX0Ph>303<c<`1BDU80BDU904E5A@18B4P0CDe=03<c<`17AdL0Hf=S04i>
CP13@d<0@T9203Ph>01:BTX0@D5109bLW01oOgl0Hf=S07IfMP0Z:RX0?Cdm04U9B@14A4@0LG5a05=C
D`1=CDd0NGUi03lo?`1HF5P0DE5A04=3@`0Z:RX0>SXj03Lg=`1:BTX0;Bd]034a<@17AdL0BTY:07Mg
M`0m?Cd0DU9B0;Bd]028R8P0CTi>07inOP2:RXX0B4Q8061PH01TI6@0IfMW013ooooo000>ooooo`0K
07=cL`1[Jf/0TI6A05eMG@1RHV80K6a/09:BTP2h^;P0?Cdm05]KF`1dM7@0NWYj0:j^[P0c<c<0<30`
03Lg=`1`L700RHV907YjNP0Y:BT0C4a<04I6AP1=CDd0OGem02@T900Z:RX0IfMW0080DU9B0?T0HV9R
02XZ:P1SHf<0OWin02l_;`0^;Rh0DU9B020P800k>c/0N7Qh03Ti>@0Y:BT0HF5Q02d];@1HF5P0QHF5
03Ti>@0n?Sh0FEUI03Lg=`2=SHd09BDU028R8P0n?Sh0<30`059BDP37alL0>SXj04];B`24Q8@0ADE5
06UYJ@2=SHd07alO04a<C02HV9P0J6QX07MgM`23Ph<092@T03Hf=P1IFET0DU9B09:BTP0f=SH0;bl_
04Y:BP1<C4`0?clo092@T01<C4`05QHF04E5A@1dM7@0CDe=051@D00d=3@07alO0410@02<S8`0IfMW
06e]K@0U9BD08R8R04U9B@2IVIT0>c/k04Q8B00/;2`0@D5107moO`1ZJVX0:RXZ04E5A@1=CDd0Ogmo
08N7Q`0l?3`0820P04Y:BP1KFe/0Q8B40<C4a00a<C40>3Ph04a<C01?Cdl0VIVI0:BTY00K6a/0;Rh^
05iNGP1/K6`0M7Ad08Z:RP0H61P0ATI609NGU`1^KVh0OWin06i^KP0l?3`0G5aL08f=S@1FEUH0Lg=c
03Lg=`1@D500Jf][08f=S@1@D500Jf][05YJFP0m?Cd0QXJ608R8R019BDT0G5aL03`l?00f=SH0Ogmo
04a<C00a<C40KFe]04=3@`0c<c<0QhN702DU9@0^;Rh0ADE502d];@0_;bl0Rh^;04];B`0j>SX0B4Q8
05ADE01;Bd/0XJ6Q03Ph>010@400Fe]K03dm?@1fMWH0Ng]k02/[:`15ADD0M7Ad05MGE`2b/[80IFEU
028R8P0m?Cd0Jf][07alO02MWId092@T02d];@1UIFD0LG5a07=cL`0e=CD0820P05iNGP2TY:@0D51@
04E5A@0Z:RX0;Bd]06i^KP1YJFT0@4100492@P0l?3`0Rh^;06IVIP1?Cdl0GUiN02LW9`17AdL0Hf=S
02/[:`17AdL0=CDe03dm?@1FEUH0Lg=c0451@@13@d<0DE5A07AdM023Ph<0>c/k0410@017AdL0MGEe
0;:b/P24Q8@0;bl_04M7A`0k>c/0Ng]k0<O7a`1bLW80=CDe04M7A`1=CDd0JVYZ0:FUY@1ZJVX0=CDe
03<c<`1FEUH0SXj>0;no_`0e=CD07alO05EEE@21PH40Ng]k0:n_[`0^;Rh0CDe=06YZJP28R8P0GemO
05iNGP0k>c/0DU9B07moO`2EUID0CDe=07=cL`0L71`0:2PX06IVIP2SXj<0K6a/03dm?@14A4@0<C4a
08b<S021PH40>c/k04A4A00b<S80<30`059BDP1/K6`03_ooool0Q@17AdL0Bd];02TY:@0e=CD0BTY:
04U9B@0f=SH0G5aL06=SH`0d=3@0ATI604A4A01<C4`0MGEe05=CD`1<C4`0EeMG04A4A00n?Sh061PH
03Ti>@0[:b/0=cLg0451@@14A4@0?Shn05MGE`1KFe/0BDU903hn?P1<C4`0EeMG04a<C019BDT0Bd];
04Q8B01=CDd0=cLg038b<P0j>SX0;Rh^04M7A`0l?3`0@41002`/;00l?3`0@T9203hn?P0c<c<0CDe=
05ADE00b<S80=3@d051@D00h>3P0CDe=04M7A`0n?Sh0A4A404I6AP1;Bd/0ATI605MGE`16ATH0Jf][
06YZJP1KFe/0HF5Q075aL@1KFe/0:2PX059BDP1PH600:2PX02@T901>CTh0?3`l05=CD`15ADD0=cLg
04I6AP0o?cl0>SXj02<S8`12@T80:RXZ02HV9P10@400<c<c03Lg=`0b<S80BTY:02@T900i>CT0A4A4
03Xj>P0/;2`0A4A403Ti>@11@D40>CTi03Xj>P0k>c/0;bl_034a<@11@D40ATI6028R8P1:BTX0Jf][
02XZ:P1MGEd0AdM703/k>`1EEED0:RXZ03De=@0j>SX0=CDe02h^;P13@d<0IfMW05eMG@2<S8`0?3`l
06QXJ01KFe/07AdM03`l?01>CTh0P82003Lg=`10@4000P0i>CT0K017AdL0De=C02l_;`0m?Cd0?clo
06EUI@17AdL0BDU908B4Q015ADD0GEeM061PH026QXH0>3Ph05=CD`1/K6`0;bl_059BDP1GEeL08B4Q
06e]K@0m?Cd0H61P02h^;P1`L700:b/[04a<C00c<c<0JFUY04Y:BP0P8200<S8b034a<@1eMGD0;bl_
05MGE`1ADE40BTY:04U9B@1PH600EeMG07]kN`0b<S80EeMG05aLG00c<c<0Jf][05MGE`1[Jf/0@410
05UIF@1cLg<0De=C05mOG`0j>SX0;Bd]03Ti>@1VIVH0HV9R06MWI`1MGEd0Q8B404M7A`1KFe/0A4A4
07UiN@1[Jf/0De=C07QhN016ATH0@d=305MGE`1mOGd0IVIV0492@P1=CDd0A4A404e=C@0/;2`0:RXZ
05=CD`1bLW80K6a/04m?C`2EUID0MGEe06YZJP0i>CT0J6QX059BDP16ATH0LG5a02/[:`0l?3`0?Shn
02`/;00V9RH0F5QH02HV9P15ADD0FUYJ04e=C@0m?Cd0;bl_06ATI02OWil0IfMW055AD@80<C4a0240
Ph>302<S8`13@d<0JVYZ04i>CP1aLG40FUYJ06m_K`2?Shl0DU9B04U9B@1_Kfl0G5aL01dM7@1OGel0
De=C06i^KP17AdL0?Cdm04A4A00^;Rh0F5QH03Ti>@0^;Rh0Cdm?05IFEP19BDT0ATI604Q8B015ADD0
C4a<059BDP1IFET04?ooool000kooooo0<`0XJ6Q04m?C`1FEUH0PH6109ZJVP0m?Cd0D51@07alO01[
Jf/0D51@03@d=018B4P0IVIV04A4A00/;2`0C4a<034a<@18B4P0Ph>30410@014A4@0?Shn02XZ:P2N
WYh0?3`l028R8P0j>SX0:BTY0451@@1bLW808b<S04E5A@1fMWH0@T9206QXJ02OWil09BDU03Xj>P0n
?Sh0Hf=S05mOG`37alL0;Rh^0451@@2?Shl0I6AT0861P@1gMgL09BDU04m?C`1[Jf/0RXZ:05UIF@0W
9bL0>CTi04Q8B01WIfL0PH61071`L00c<c<0;2`/04a<C01=CDd0Vi^K04a<C00[:b/0CDe=07emO@1<
C4`0KVi^02/[:`0Y:BT0G5aL08N7Q`0O7al08b<S03/k>`0U9BD0Jf][08f=S@0M7Ad0;2`/0451@@1L
G5`0GemO038b<P0F5QH0<S8b03lo?`1SHf<0a<C4059BDP0Q8B40Bd];05aLG01iNGT0Q8B4020P800S
8b<0I6AT06e]K@1LG5`0Ph>302h^;P0n?Sh0LW9b05eMG@1>CTh0YjNW04=3@`0c<c<0Cdm?07]kN`16
ATH0Lg=c02h^;P16ATH0Lg=c05ADE00i>CT0Jf][03hn?P0m?Cd0NGUi038b<P0o?cl0?Shn01hN7P0i
>CT0QXJ602TY:@19BDT0=SHf055AD@1fMWH0E5AD02<S8`1PH600BTY:0451@@2=SHd0Ogmo02DU9@1;
Bd/0GemO06=SH`2HV9P0D51@030`<01KFe/0MgMg079bLP2l_;`0=SHf030`<01YJFT0KVi^06a/K02D
U9@0:BTY02l_;`1XJ6P0O7al0:RXZ02CTi<0ADE504];B`1UIFD0OGem0:FUY@22PX80=CDe04=3@`2;
Rh/0OGem0861P@1;Bd/0:2PX0451@@2GUiL0Bd];059BDP14A4@0=cLg06QXJ015ADD0>SXj05EEE@19
BDT0Jf][059BDP0[:b/0?3`l0451@@1>CTh0QhN703lo?`0f=SH0A4A402l_;`1JFUX0Hf=S028R8P0n
?Sh206ATI00m0:FUY@0g=cL0Bd];06MWI`1iNGT0Xj>S04];B`0Z:RX0DE5A06YZJP2KVi/0JVYZ01/K
6`18B4P0PX:207alO02LW9`0=SHf02d];@1?Cdl0VIVI05eMG@1YJFT0=SHf03dm?@1FEUH0[Jf]05eM
G@1EEED08R8R03/k>`1DE5@0F5QH04Q8B01bLW80@4100451@@26QXH0TI6A03Ph>012@T80DU9B04Q8
B026QXH0EeMG03<c<`15ADD0GemO061PH025QHD09bLW02@T9017AdL0?clo034a<@26QXH0<c<c01/K
6`0`<300CTi>05iNGP0205mOG`0701TI6@0j>SX0R8R8059BDP1OGel0D51@03@d=00204U9B@kooooo
08X0KFe]06=SH`0l?3`0?Shn05ADE01<C4`0?Shn05IFEP1ADE40>c/k04e=C@1ADE40F5QH06MWI`20
P800LG5a069RHP1PH600@T9201dM7@0e=CD08b<S038b<P0o?cl0B4Q805=CD`18B4P0GUiN05aLG01>
CTh0GEeM06EUI@1LG5`0GUiN06ATI01RHV80F5QH038b<P0c<c<0>SXj03Ph>01?Cdl0=SHf051@D018
B4P0DE5A04Q8B01LG5`0=CDe04a<C00i>CT0:BTY02/[:`0k>c/0<30`03Ti>@0f=SH0;Bd]0451@@15
ADD0@T9203lo?`0k>c/0>3Ph06EUI@1VIVH0<S8b02TY:@14A4@0AdM705EEE@1_Kfl0P82002PX:00[
:b/0D51@04=3@`15ADD0>SXj0451@@16ATH0<c<c04A4A00J6QX0;bl_020P800i>CT0?Cdm030`<00h
>3P0<C4a04M7A`0L71`0?3`l03Ph>00R8R80=CDe03<c<`0^;Rh0<c<c04a<C00_;bl0BTY:05=CD`0]
;Bd0F5QH05mOG`0n?Sh0AdM704a<C00_;bl0Ogmo02HV9P0Q8B40Cdm?03Xj>P0a<C40@41004Q8B00j
>SX0FUYJ06EUI@17AdL0De=C0410@01iNGT0NWYj034a<@1XJ6P0Jf][06IVIP0];Bd0FEUI04M7A`0m
?Cd0?clo030`<00g=cL20492@P2203Lg=`17AdL0IFEU0=OGe`1QHF40Ng]k04];B`12@T80?Shn05YJ
FP1LG5`0>SXj065QH@1HF5P08B4Q055AD@0k>c/0GUiN03dm?@1RHV80:RXZ05EEE@17AdL0<c<c0451
@@0X:2P0Bd];04e=C@16ATH06a/K055AD@15ADD0Fe]K05=CD`0[:b/0@41005mOG`0k>c/0I6AT07Ad
M00b<S80@41004I6AP1DE5@0IVIV06m_K`2=SHd0FEUI0410@01NGUh0FUYJ04A4A00a<C40=3@d06][
J`1CDe<0D51@069RHP2PX:00ATI602d];@14A4@0@T9204a<C00n?Sh0CDe=05YJFP1FEUH0@D5105=C
D`1HF5P0>SXj04E5A@18B4P0=cLg04i>CP19BDT0Bd];02`/;00i>CT0BTY:05=CD`0B4Q80@41003De
=@0V9RH0U9BD03`l?00g=cL0<c<c0492@P13@d<0DU9B04e=C@1CDe<0GEeM04A4A00c<c<0?Shn03Lg
=`0m?Cd0=cLg06IVIP1?Cdl0=3@d06EUI@0`<300H61P0451@@1JFUX0Z:RX06EUI@1WIfL0Ng]k05aL
G01iNGT0SHf=07inOP1=CDd0LW9b04=3@`10@400GEeM05=CD`1XJ6P0C4a<05eMG@1=CDd0ADE506MW
I`80I6AT00H0Kfm_05QHF01ADE40CTi>079bLP1XJ6P@ooooo`003_ooool0mP2`/;00D51@04i>CP1J
FUX0Z:RX04Y:BP0c<c<0ATI606IVIP20P800<30`02<S8`1EEED0H61P05eMG@2[Zj/0;2`/03/k>`1Y
JFT0@d=306QXJ01FEUH0<C4a0492@P23Ph<0CDe=06a/K00K6a/0:RXZ069RHP1]KFd0OWin0451@@0m
?Cd0A4A404i>CP1HF5P0Fe]K05=CD`0F5QH0CDe=06e]K@1QHF40H61P04I6AP0i>CT0IfMW09:BTP1=
CDd0A4A4020P801XJ6P0XZ:R04m?C`0c<c<0F5QH02TY:@2:RXX0S8b<03<c<`0f=SH0E5AD03/k>`2g
]kL0R8R801TI6@0[:b/0GemO04i>CP27QhL0<C4a02LW9`0^;Rh0Cdm?06][J`14A4@0GemO02DU9@0f
=SH0EUIF04U9B@1DE5@0@D5102TY:@1FEUH0JVYZ04M7A`27QhL0JFUY02XZ:P1QHF40Jf][03Lg=`2K
Vi/0:RXZ03dm?@1fMWH0BDU904M7A`1cLg<0;2`/028R8P1SHf<0:BTY03Lg=`1<C4`0?Shn04Y:BP23
Ph<0H61P04E5A@1JFUX0ADE505]KF`2QXJ40ADE504Q8B01KFe/0D51@05=CD`2`/;00FEUI03dm?@1L
G5`0EeMG08j>SP1@D5008b<S02d];@1XJ6P0GEeM09JFUP0n?Sh09RHV03Ti>@1BDU80IVIV08J6QP15
ADD0;2`/04a<C02ZZZX0HV9R0820P00M7Ad0=cLg04M7A`1IFET0A4A406=SH`0R8R80BDU9071`L01i
NGT0<c<c04];B`15ADD0@d=308>3P`1DE5@092@T05]KF`11@D40Fe]K0:2PX014A4@0;bl_04i>CP0^
;Rh0KVi^09fMW@0a<C40=cLg055AD@0o?cl0Ng]k07MgM`0X:2P0B4Q805UIF@1RHV80RHV905eMG@0]
;Bd0?Cdm04U9B@1^KVh0ZJVY03Lg=`1UIFD0IVIV03Ph>01gMgL0ADE501lO7`1@D500I6AT08R8R01e
MGD09RHV02LW9`1JFUX0BDU905QHF00R8R80ATI605QHF01XJ6P0De=C03dm?@0_;bl0LG5a07QhN00d
=3@0I6AT04Q8B01DE5@0JFUY02/[:`10@400IFEU03lo?`1_Kfl0I6AT038b<P1JFUX0De=C04m?C`2J
VYX0=3@d04=3@`1JFUX0EeMG055AD@2RXZ80;bl_02`/;01QHF40LG5a05YJFP2RXZ80Jf][038b<P18
B4P0EEEE03lo?`80Rh^;01h0A4A405aLG01/K6`0MgMg09fMW@10@400C4a<06QXJ01PH600GEeM09bL
W00o?cl0<30`059BDP27QhL0CTi>08N7Q`1SHf<092@T04M7A`1DE5@0BDU905YJFP0f=SH07QhN06=S
H`1;Bd/09bLW05IFEP0/;2`>ooooo`0S05IFEP1>CTh0>3Ph05mOG`1HF5P0EEEE04Q8B01WIfL0L71`
03Lg=`1>CTh0OGem06i^KP1iNGT0H61P04E5A@1FEUH0HV9R05EEE@0h>3P0@d=303Ph>01BDU80>CTi
02h^;P0n?Sh0Hf=S06UYJ@1SHf<0?Cdm05aLG01]KFd0FUYJ04I6AP1@D5000P1fMWH0?00g=cL0>SXj
03Hf=P0d=3@0?clo03Ph>01@D500ADE504];B`17AdL0F5QH03`l?01>CTh0ATI603<c<`0/;2`0A4A4
03@d=00j>SX0CTi>03dm?@1=CDd0BDU904a<C018B4P0ATI604m?C`25QHD0P8200410@00l?3`0LW9b
04i>CP0T92@0<c<c04M7A`0Q8B40:b/[03/k>`0f=SH0Cdm?04M7A`0l?3`0BTY:03Ph>00j>SX08R8R
030`<00P82007AdM04=3@`0c<c<0>SXj03/k>`1>CTh06ATI03@d=01CDe<0=cLg02DU9@80;bl_02T0
?3`l02l_;`0Z:RX0@41004e=C@0N7Qh0?Cdm0410@00n?Sh0?3`l05=CD`0T92@0HF5Q03Lg=`0P8200
EUIF0451@@0R8R80@D51030`<017AdL0ATI607moO`1ZJVX0I6AT05UIF@1FEUH0K6a/04i>CP23Ph<0
JVYZ071`L016ATH0Ogmo05QHF00[:b/0>c/k03dm?@0l?3`0EEEE02<S8`0204A4A02804E5A@1YJFT0
@D5106i^KP1;Bd/0SHf=02/[:`1PH600G5aL02XZ:P10@400BDU902TY:@1SHf<0CDe=09VIV@0j>SX0
Hf=S03Xj>P1<C4`0Bd];04a<C00l?3`0=3@d07EeM@1JFUX0G5aL02PX:01@D500>c/k03De=@0f=SH0
@D51051@D02JVYX0>3Ph04U9B@1EEED08B4Q04M7A`1<C4`0DE5A03dm?@1lO7`0YJFU05QHF01IFET0
<C4a03Hf=P0^;Rh0Fe]K03dm?@10@400@d=305IFEP0];Bd0H61P030`<01KFe/0E5AD03Ph>01?Cdl0
>SXj03<c<`1BDU80LW9b04e=C@14A4@0FEUI0492@P18B4P0>3Ph034a<@1=CDd0M7Ad079bLP0l?3`0
G5aL06=SH`1]KFd0<C4a05ADE016ATH0Cdm?079bLP1PH600Cdm?06IVIP1TI6@0Fe]K051@D00i>CT0
?Shn06][J`1:BTX0AdM70492@P0_;bl0Bd];03Xj>P1eMGD0EUIF03Ti>@1=CDd09bLW04=3@`1BDU80
>SXj051@D00k>c/0BTY:07YjNP1QHF40DE5A05]KF`17AdL0@41007MgM`1BDU80Bd];02XZ:P0^;Rh0
BDU904=3@`1BDU80>c/k04a<C01fMWH0F5QH04M7A`1ADE40E5AD04=3@`1CDe<0D51@05UIFA3ooooo
000>ooooo`2S08F5Q@1YJFT0VIVI059BDP1FEUH0FEUI06][J`1dM7@0:2PX05eMG@1DE5@0D51@05iN
GP0P8200@d=304Q8B00h>3P0JVYZ0492@P0e=CD0DU9B06][J`0`<300A4A403<c<`24Q8@0OWin03Hf
=P0U9BD0?Cdm03Ti>@2OWil0UiNG01`L700i>CT0EEEE04A4A02LW9`0VYZJ01TI6@0c<c<0L71`05UI
F@1kNg/0E5AD02TY:@0j>SX0MgMg05aLG01fMWH07alO02@T901?Cdl0Kfm_05IFEP2KVi/0=CDe03Xj
>P27QhL0L71`06ATI01^KVh0;2`/05EEE@2HV9P0Ogmo05mOG`2>SXh07alO04I6AP1KFe/0820P04i>
CP11@D40:BTY03Xj>P1:BTX0AdM7071`L00c<c<0<30`061PH01NGUh07alO03Xj>P0k>c/0BDU908N7
Q`21PH408B4Q051@D010@400CTi>09RHV016ATH0:RXZ06EUI@1[Jf/0GemO0;fm_@0h>3P07alO065Q
H@1CDe<0MgMg07UiN@0E5AD09bLW06UYJ@1jNWX0];Bd0:NWY`0Q8B40=cLg06m_K`1bLW80Jf][04U9
B@0W9bL0@T9205iNGP24Q8@0H61P03dm?@0Y:BT0Jf][07alO014A4@0;Rh^0492@P0i>CT0QhN707mo
O`12@T80GEeM03<c<`18B4P0PH6105eMG@0g=cL0Hf=S04m?C`1>CTh0]kNg07alO00X:2P0De=C04Y:
BP1GEeL0XZ:R03@d=00e=CD0KVi^055AD@1<C4`0N7Qh024Q8@0m?Cd0D51@07EeM@1DE5@0UYJF04M7
A`0204Y:BP0:05MGE`1bLW80MWIf02l_;`11@D40DU9B055AD@1WIfL0I6AT01`L7080EUIF06D0@T92
08f=S@1ZJVX08R8R02h^;P1MGEd0Fe]K06=SH`0N7Qh0:RXZ05mOG`1FEUH0C4a<03Ph>00o?cl0CTi>
06IVIP0a<C40@D5101dM7@1SHf<0U9BD03/k>`0];Bd0?Shn05IFEP1kNg/0<c<c02`/;01CDe<0=cLg
09NGU`15ADD08b<S04M7A`1DE5@0AdM709^KV`1NGUh0H61P07=cL`1<C4`0T92@08^;R`0d=3@0Fe]K
08J6QP28R8P0YZJV07moO`11@D40F5QH05UIF@21PH40ZZZZ04M7A`0l?3`0KVi^07AdM01iNGT0QhN7
03@d=00^;Rh0C4a<07MgM`11@D40J6QX030`<00Y:BT0AdM707MgM`1HF5P0LG5a02TY:@19BDT0N7Qh
0861P@10@400C4a<02DU9@18B4P0I6AT05aLG00O7al0EeMG02PX:00k>c/0H61P07UiN@0X:2P0=cLg
03lo?`1eMGD0O7al08B4Q00Q8B40:BTY04];B`1UIFD0SHf=038b<P0>ooooo`1?059BDP1=CDd0:b/[
03Lg=`12@T80ADE5030`<01<C4`0?3`l034a<@0l?3`0BDU904i>CP1SHf<0I6AT04Y:BP19BDT0ATI6
03Xj>P0Q8B40?clo02`/;00o?cl0B4Q803Ti>@14A4@0=cLg04E5A@14A4@0>SXj04E5A@19BDT0>CTi
04A4A01<C4`0Bd];055AD@0a<C40<c<c03dm?@0/;2`0>SXj03De=@0_;bl0:BTY038b<P0c<c<0>SXj
02LW9`0m?Cd0?clo02TY:@0L71`0;2`/03lo?`1KFe/0>SXj034a<@18B4P0H61P04m?C`0c<c<0GUiN
04E5A@1>CTh0EUIF059BDP1DE5@0IFEU05IFEP0l?3`0HF5Q06QXJ00Y:BT0:b/[04U9B@15ADD0AdM7
0492@P020451@@2:03`l?017AdL0<c<c03dm?@0e=CD0AdM704e=C@0m?Cd0@T9203Lg=`1:BTX0;2`/
04A4A01<C4`0<S8b02XZ:P0l?3`0A4A40492@P17AdL0>SXj055AD@1CDe<0?clo05MGE`1DE5@0=3@d
04E5A@1UIFD07alO07=cL`1ZJVX0?clo07=cL`13@d<0<C4a03Ph>00_;bl0E5AD03Ti>@1>CTh0ATI6
08Z:RP15ADD0J6QX06a/K00j>SX0NWYj05mOG`1VIVH0CDe=05MGE`15ADD0?Shn04E5A@1CDe<0<S8b
04I6AP13@d<0CDe=05QHF01KFe/0^;Rh05]KF`29RHT0IfMW05QHF00Y:BT0EeMG04i>CP0];Bd0C4a<
03Ph>00O7al0IfMW04Y:BP1DE5@0<S8b05eMG@0d=3@0Hf=S05EEE@17AdL0@D510410@015ADD0@D51
06MWI`0Z:RX0IFEU02LW9`0g=cL0@41005EEE@1?Cdl0FUYJ03Ti>@1@D500CDe=02HV9P1bLW80Hf=S
07alO01BDU80EUIF0820P017AdL0ADE503Xj>P10@400?clo03De=@0`<300@d=304a<C01RHV80;bl_
06ATI00W9bL0EUIF06EUI@0f=SH0CTi>03/k>`18B4P0HV9R04m?C`1NGUh0C4a<04i>CP1=CDd0FUYJ
03Ph>00[:b/0CTi>03dm?@1>CTh0>3Ph0P1TI6@0>@1KFe/08b<S051@D00l?3`0<S8b08:2PP1=CDd0
HV9R04=3@`0k>c/0@41004Y:BP0V9RH0?Shn05YJFP15ADD0;Bd]02XZ:P0h>3P0FUYJ04a<C01QHF40
?3`l02l_;`1HF5P0=CDe04i>CP16ATH0?Shn08B4Q01NGUh0HV9R07EeM@1^KVh0Cdm?06MWI`0g=cL0
61PH05MGE`1=CDd0E5AD02d];@1:BTX0E5AD03@d=01PH600=SHf03dm?@1oOgl0NWYj051@D01ADE40
E5AD04a<C01;Bd/0DU9B05mOG`0@ooooo`003_ooool0o`2:RXX0E5AD09VIV@1NGUh0Cdm?071`L02@
T900YjNW03Lg=`1:BTX0@41007MgM`1aLG407AdM02l_;`1>CTh0C4a<0:2PX02;Rh/051@D04A4A02=
SHd0AdM707emO@0L71`0Bd];05MGE`16ATH0F5QH071`L00I6AT0EUIF0::RXP1:BTX0FUYJ06=SH`0]
;Bd0EEEE08J6QP19BDT0Jf][07=cL`0I6AT0>CTi06MWI`0k>c/0K6a/03`l?00^;Rh0MWIf04m?C`1F
EUH0CDe=02LW9`0b<S80CDe=03/k>`1fMWH0Fe]K03lo?`1mOGd0I6AT020P800n?Sh0@d=305eMG@1U
IFD0Rh^;02<S8`0i>CT0DU9B05MGE`1lO7`0HV9R02`/;00i>CT0F5QH04a<C02VYZH0B4Q8030`<00^
;Rh0Fe]K05IFEP1`L7007QhN02TY:@0[:b/0Ogmo05MGE`1PH6008B4Q03De=@1LG5`0LG5a05MGE`1E
EED09BDU034a<@1@D500HV9R030`<017AdL0:BTY0492@P1WIfL0@d=303Hf=P1^KVh0<30`0861P@2]
[Jd0>3Ph02XZ:P1:BTX0?3`l09bLW02GUiL09bLW02d];@0h>3P0?Cdm071`L02ATI407alO03@d=01@
D500G5aL05YJFP1]KFd0;2`/05ADE01mOGd0Jf][09:BTP0m?Cd0=SHf07UiN@2MWId0GemO0;jn_P1Y
JFT0=cLg06i^KP2d];@0FEUI08Z:RP0k>c/0Cdm?06e]K@1DE5@0CTi>0861P@0a<C4092@T07QhN01c
Lg<0C4a<075aL@0N7Qh0>SXj061PH01=CDd0EeMG05QHF00U9BD0HV9R0;:b/P1CDe<0MWIf05ADE00g
=cL0P8200:JVYP1BDU80EeMG04A4A00k>c/0UYJF09jNWP0`<300AdM703lo?`1aLG40Zj^[030`<00U
9BD0<C4a05IFEP1fMWH0?Shn01lO7`0g=cL0FUYJ05IFEP1nOWh0;Bd]01TI6@1BDU80BDU9071`L01>
CTh09bLW06EUI@1DE5@0@D5106a/K00U9BD0DU9B04i>CP0o?cl0JVYZ03De=@12@T80I6AT05EEE@1Y
JFT0>c/k02TY:@1?Cdl0P82005ADE01GEeL0;2`/03`l?021PH40Vi^K04];B`1ADE40?3`l05]KF`2E
UID0SXj>03Ph>01JFUX0C4a<05EEE@2PX:00K6a/02d];@0h>3P0Cdm?06e]K@2c/k<0Cdm?020P8010
@400EeMG069RHP2f][H0?clo02HV9P1KFe/0M7Ad07MgM`005`2@T900;Rh^02@T901TI6@0JVYZ04Y:
BP2<S8`071`L02<S8`1GEeL0Hf=S04=3@`1cLg<0=cLg02<S8`11@D40Lg=c05=CD`1BDU80D51@02<S
8`1kNg/0Ogmo00kooooo01/0F5QH05YJFP0l?3`0Bd];055AD@1SHf<0BTY:07AdM01QHF40=cLg04m?
C`1_Kfl0KVi^07EeM@1jNWX0F5QH05iNGP1EEED0DU9B01hN7P0d=3@09bLW03@d=00h>3P0=3@d04I6
AP1WIfL00P1TI6@0601=CDd0JVYZ06m_K`1RHV80De=C04i>CP1QHF40FEUI01lO7`0^;Rh0?Cdm02HV
9P14A4@0@D5104i>CP0Z:RX0D51@03dm?@1LG5`0:b/[04i>CP16ATH0@T9201`L7080?Cdm0980;Bd]
04A4A00e=CD0AdM70410@01<C4`0=cLg03De=@12@T80FEUI05=CD`0l?3`0@D5104e=C@0k>c/0?clo
03<c<`0a<C406QXJ01lO7`1;Bd/09bLW03Ti>@0e=CD09RHV03<c<`0/;2`0:b/[01lO7`0U9BD0:BTY
02PX:00o?cl0<c<c03Xj>P0[:b/0@41003<c<`0];Bd0=SHf03De=@0i>CT0=3@d02l_;`15ADD0EEEE
03Lg=`1DE5@0K6a/0492@P0l?3`0=CDe04];B`1ADE40F5QH03/k>`1FEUH0?clo04M7A`1[Jf/0BDU9
04=3@`0m?Cd0;Bd]03Ti>@1:BTX0GUiN03dm?@1iNGT0HV9R079bLP1kNg/0=cLg092@T01RHV80NWYj
04I6AP1eMGD0E5AD03Hf=P1CDe<0EeMG03lo?`13@d<0;bl_0451@@10@400EeMG0<71`@1CDe<0FUYJ
04M7A`1cLg<0:2PX06IVIP1KFe/0=cLg04I6AP14A4@07QhN06QXJ00X:2P0Cdm?03Ph>01aLG40>c/k
03<c<`0h>3P0GemO0451@@0];Bd0K6a/05EEE@0k>c/07QhN04=3@`0D51@0?Shn04=3@`0R8R80=CDe
061PH00b<S80EeMG05aLG00^;Rh0;Bd]03Lg=`15ADD0D51@07EeM@2^[Zh0BTY:055AD@16ATH0CTi>
030`<00b<S80<30`03dm?@0d=3@0@d=302XZ:P1[Jf/0;Bd]02PX:080@D5100T0F5QH03Xj>P0o?cl0
EEEE05MGE`16ATH0@T92061PH00e=CD00P0k>c/0@00`<300@T9204M7A`1EEED0ADE506IVIP1hN7P0
F5QH01TI6@0^;Rh0=cLg03Hf=P2?Shl0A4A405MGE`0o?cl0DE5A0410@01<C4`09BDU04Y:BP1PH600
<c<c02l_;`14A4@0?clo07inOP1GEeL0De=C05YJFP0j>SX0OGem04I6AP1[Jf/0CDe=04I6AP1ZJVX0
<c<c04U9B@1QHF40CTi>059BDP2UYJD0Fe]K01/K6`1RHV80BDU905]KF`0b<S80J6QX05IFEP0c<c<0
DU9B03lo?`13@d<0HV9R06QXJ01VIVH0F5QH05]KF`19BDT0Bd];04e=C@1<C4`@ooooo`003_ooool0
I02a/K40EUIF07MgM`1gMgL0Ph>304m?C`1FEUH0LG5a04i>CP1fMWH0?Cdm04Q8B01XJ6P0>SXj05=C
D`1dM7@0:b/[06ATI025QHD0DE5A0861P@1TI6@0:BTY04i>CP14A4@0H61P07moO`0T92@0>c/k05eM
G@1@D500I6AT05]KF`0W9bL0D51@06QXJ00_;bl0HV9R04Y:BP0V9RH0I6AT092@T01;Bd/0C4a<0451
@@19BDT0QXJ607inOP0g=cL0FEUI03dm?@1NGUh0Ng]k0492@P0f=SH0CDe=04E5A@1eMGD0JVYZ02d]
;@0h>3P0Jf][04m?C`2=SHd0JFUY038b<P1TI6@0JFUY04a<C02>SXh0<c<c02XZ:P17AdL0C4a<03Hf
=P1^KVh0GemO01dM7@1@D500IFEU04A4A01[Jf/0E5AD03@d=01GEeL0GemO04Y:BP1eMGD0@T9203lo
?`1/K6`0@41004Y:BP2DU9@0=3@d05YJFP25QHD0=3@d04=3@`1NGUh203lo?`0P08j>SP0j>SX0BTY:
04i>CP1:BTX0Hf=S0:>SX`11@D40GemO06e]K@1CDe<0G5aL09BDU00k>c/0De=C06ATI013@d<0J6QX
092@T00h>3P0?clo05UIF@13@d<0NWYj07=cL`0V9RH0B4Q8075aL@1OGel0JVYZ03Xj>P0^;Rh205AD
E02>06a/K02ATI40?3`l04=3@`1LG5`0QhN706ATI02=SHd0ATI603Lg=`1cLg<0K6a/04=3@`1^KVh0
?Cdm05mOG`1hN7P0H61P02l_;`1=CDd0A4A405]KF`24Q8@0BTY:02l_;`1`L700?Shn07=cL`2MWId0
<S8b04=3@`1VIVH0B4Q808n?S`25QHD0=CDe03/k>`1RHV80?Cdm071`L01VIVH09bLW04=3@`1VIVH0
AdM709NGU`1CDe<0?3`l05MGE`1XJ6P0FUYJ07alO00[:b/0JVYZ07QhN01>CTh0OGem04a<C00O7al0
Hf=S08F5Q@1cLg<0GEeM028R8P12@T80GEeM069RHP2:RXX0=SHf04a<C024Q8@0BTY:06=SH`0/;2`0
C4a<08:2PP1OGel0=SHf07IfMP1EEED0RHV907moO`0m?Cd0EEEE07AdM01XJ6P0U9BD07moO`12@T80
DE5A0820P01LG5`0Q8B404E5A@1:BTX0IVIV07QhN01ADE40QHF5034a<@13@d<0Jf][06UYJ@1>CTh0
P82006EUI@0d=3@0IVIV06a/K01=CDd0VIVI07AdM00f=SH0MWIf0820P01hN7P0LW9b04e=C@17AdL0
Kfm_0410@016ATH0O7al02l_;`0/;2`0EEEE05UIF@19BDT0Vi^K05aLG00m?Cd0I6AT05eMG@1>CTh0
JVYZ0451@@0`<300I6AT06EUI@0i>CT0Cdm?03Lg=`kooooo06H0Fe]K06MWI`0l?3`0@T9204Y:BP1=
CDd0@T9206i^KP1gMgL0BDU905IFEP1/K6`0M7Ad04a<C01HF5P0B4Q8055AD@16ATH0ADE503De=@10
@400<c<c04Q8B01=CDd0>CTi055AD@1:BTX0EUIF07=cL`15ADD0CTi>05MGE`19BDT0EeMG05IFEP1?
Cdl0EeMG03Hf=P0c<c<0=SHf0451@@16ATH0@d=304M7A`0`<300@d=304e=C@1?Cdl09RHV04i>CP12
@T80D51@02XZ:P18B4P0?Shn03Hf=P1CDe<0>c/k055AD@1DE5@0EEEE02d];@1<C4`0E5AD07=cL`1d
M7@0?Cdm04I6AP1/K6`0EEEE04=3@`1BDU80IVIV03<c<`0e=CD0GEeM03De=@1>CTh0B4Q80492@P1:
BTX0=SHf04A4A00[:b/0>SXj03De=@0c<c<0EUIF0451@@16ATH0?clo05IFEP0V9RH0;bl_05ADE00i
>CT0;bl_03Xj>P0h>3P0ATI603dm?@0l?3`204U9B@0i03De=@0j>SX0>c/k02/[:`0a<C40D51@02DU
9@14A4@0Cdm?03Lg=`1IFET0@41002@T900b<S80<30`0451@@13@d<0HV9R055AD@2PX:00EEEE05aL
G01_Kfl0>3Ph06MWI`1UIFD0KVi^04M7A`1aLG40FUYJ03hn?P1KFe/0JFUY03`l?01:BTX0=cLg05AD
E01?Cdl0DU9B0:JVYP1PH600IFEU05iNGP1kNg/0<30`04Q8B01CDe<08B4Q05IFEP1BDU80:2PX06][
J`15ADD0I6AT03@d=01MGEd0:b/[0080BTY:02T0ADE503Xj>P0b<S80De=C055AD@1@D500;2`/059B
DP0S8b<0>c/k04A4A011@D40CDe=06MWI`0e=CD0=SHf03/k>`0F5QH0?Shn03`l?01OGel0FEUI061P
H02CTi<0C4a<04e=C@13@d<0@T9203Hf=P13@d<0ATI605ADE015ADD0DU9B01/K6`1aLG40:b/[03hn
?P1NGUh0=3@d059BDP0203dm?@0G05UIF@1MGEd0CDe=03hn?P1?Cdl0@d=3059BDP0l?3`0:2PX03hn
?P14A4@0EUIF02l_;`1LG5`0H61P05]KF`0B4Q80=SHf034a<@0Z:RX0K6a/038b<P1EEED00P15ADD0
;`0j>SX0D51@02TY:@11@D40G5aL0410@00a<C40Cdm?04M7A`1SHf<0CDe=06][J`1FEUH08B4Q06YZ
JP0Y:BT0Kfm_05ADE017AdL0Ng]k05eMG@1[Jf/0IfMW04i>CP13@d<0Jf][0451@@0_;bl0Hf=S051@
D01LG5`0=cLg0492@P1FEUH0B4Q805aLG00g=cL0AdM707moO`1?Cdl0BTY:05]KF`1CDe<0B4Q805UI
F@1FEUH0D51@013ooooo000>ooooo`2H08J6QP1oOgl0c<c<04a<C01lO7`0Ng]k0:ZZZP1JFUX0BDU9
06][J`1BDU80YjNW071`L00i>CT0<C4a061PH01;Bd/0PH610492@P0a<C40E5AD08b<S01FEUH0NGUi
01dM7@1JFUX0LW9b065QH@13@d<0DE5A03hn?P1:BTX0D51@030`<00Y:BT0DU9B03Lg=`1MGEd0MWIf
03`l?00n?Sh0EeMG055AD@1_Kfl0:2PX02LW9`1?Cdl0GEeM059BDP2=SHd092@T02HV9P1BDU80NGUi
065QH@2h^;P0:BTY04Q8B01[Jf/0FEUI06][J`2?Shl07QhN04m?C`1ZJVX0ATI60;Zj^P2BTY806QXJ
03dm?@18B4P0HF5Q06ATI00];Bd0;Rh^03Hf=P1FEUH0>SXj05ADE00T92@07QhN02`/;01RHV80>SXj
0861P@12@T8092@T07UiN@1jNWX0?Cdm06EUI@0K6a/0ADE5096AT@1ZJVX0=3@d059BDP0[:b/0E5AD
09VIV@0V9RH07QhN0410@00l?3`0TI6A0861P@0I6AT0ADE504=3@`1BDU80][Jf08:2PP0I6AT0?3`l
06UYJ@18B4P0MgMg071`L00K6a/0AdM709JFUP1XJ6P0EeMG03Lg=`0T92@0FUYJ07YjNP0n?Sh0NWYj
04M7A`0e=CD0Ogmo05mOG`0h>3P0EUIF02h^;P10@400TY:B079bLP0[:b/0EUIF02h^;P15ADD0PX:2
08B4Q016ATH0K6a/0410@011@D40_;bl06ATI00h>3P205IFEP1l055AD@2FUYH0:b/[03lo?`19BDT0
BTY:07moO`23Ph<0=SHf03Xj>P15ADD0>3Ph0861P@2YZJT0;2`/03@d=01FEUH0G5aL09FEU@2CTi<0
:b/[04I6AP1XJ6P0IfMW07IfMP1=CDd07AdM04Q8B01IFET0IFEU08N7Q`0M7Ad0B4Q806e]K@1LG5`0
KFe]03/k>`0i>CT0@D5108B4Q01ADE40BDU9020P801HF5P0MgMg03dm?@0g=cL0>CTi03De=@2TY:@0
NGUi03dm?@1MGEd08B4Q09jNWP1HF5P0<C4a05UIF@12@T80<c<c09^KV`1@D500<S8b04i>CP1CDe<0
P82008V9R@0a<C40HF5Q0861P@2LW9`0[jn_0:2PX00U9BD0F5QH05eMG@1]KFd0[Zj^03Ti>@10@400
Hf=S055AD@1PH600S8b<03Lg=`0Z:RX0E5AD07EeM@1ZJVX0W9bL02l_;`0/;2`0D51@08F5Q@1SHf<0
F5QH02DU9@0g=cL0HV9R06YZJP1?Cdl0Ng]k02h^;P0k>c/0TY:B08R8R016ATH0GUiN01TI6@0S8b<0
PH6107alO00h>3P0BTY:03/k>`13@d<0PX:206ATI00V9RH0=SHf05=CD`1KFe/0QHF503hn?Pkooooo
0P1CDe<0=`1;Bd/0KFe]069RHP1HF5P0Bd];04I6AP19BDT0>SXj04M7A`1VIVH0LW9b07]kN`2_[jl0
UIFE06EUI@1LG5`0EeMG0410@012@T80ATI605aLG012@T80?Cdm05iNGP1_Kfl0H61P05aLG01GEeL0
HF5Q04a<C01QHF40EEEE059BDP1JFUX0EEEE02TY:@0V9RH0@D51051@D01LG5`0B4Q804Y:BP0f=SH0
?clo02TY:@0h>3P0:2PX03`l?012@T80@D5102<S8`0i>CT0=cLg04i>CP14A4@00P0i>CT0:`18B4P0
?3`l02d];@17AdL0<C4a04Q8B01>CTh0@d=305MGE`11@D40ADE503Lg=`0i>CT0BTY:02LW9`0X:2P0
IVIV0410@01FEUH0AdM704a<C01;Bd/0@41004A4A00^;Rh0;bl_03Ph>00g=cL0Cdm?0451@@0o?cl0
@T9204m?C`0e=CD0>3Ph04U9B@0d=3@0>3Ph038b<P0e=CD0@d=304M7A`0o?cl00P1BDU808@0_;bl0
FUYJ02d];@0l?3`0=SHf05YJFP0U9BD0EEEE06EUI@1/K6`0Ng]k0451@@0e=CD0?Cdm034a<@0g=cL0
EEEE04i>CP0j>SX0AdM705=CD`1dM7@0LW9b03dm?@1LG5`0C4a<07YjNP0n?Sh0Ng]k05IFEP0b<S80
>3Ph03Xj>P0204M7A`0]030`<013@d<0E5AD05UIF@36a/H0Lg=c07IfMP1NGUh0EEEE03/k>`1UIFD0
FEUI030`<01XJ6P0CTi>020P801KFe/092@T05MGE`0`<300GUiN03De=@0i>CT0D51@05aLG011@D40
8b<S04Q8B019BDT0>CTi02d];@1ZJVX0?clo03Xj>P1@D500?3`l0410@01UIFD0?Shn02HV9P0m?Cd0
9BDU0492@P1<C4`0De=C0080MGEe05`0[:b/06MWI`1IFET0KFe]051@D01ADE40>SXj03Lg=`1PH600
GemO05eMG@0V9RH0L71`03/k>`0f=SH0EUIF05aLG01jNWX0G5aL04Q8B01BDU80HV9R061PH018B4P0
I6AT04a<C01?Cdl0@D5104E5A@11@D40@d=3059BDP10@400H61P079bLP1JFUX0>CTi04E5A@14A4@0
B4Q808Z:RP11@D40ADE504e=C@1/K6`0EUIF05iNGP0Y:BT0?Shn04i>CP14A4@0BTY:06EUI@1:BTX0
K6a/055AD@1FEUH0De=C010@4029RHT0DE5A05EEE@1MGEd0Hf=S071`L01?Cdl0/K6a07EeM@27QhL0
L71`0:RXZ01QHF40>3Ph08F5Q@1OGel0MGEe03Hf=P10@400BTY:03`l?01@D500B4Q805ADE01SHf<0
BDU906QXJ01cLg<0Bd];059BDP1ADE40IfMW05ADE13ooooo000>ooooo`1607QhN01FEUH0J6QX06a/
K01DE5@0=3@d051@D02JVYX0ATI606UYJ@0c<c<0De=C0820P00a<C40>SXj03`l?016ATH0Ng]k04i>
CP0W9bL0@D5103Ph>01BDU80TY:B02<S8`0`<300GUiN03Ti>@1cLg<0TI6A01@D5011@D40DU9B0451
@@1cLg<0XZ:R02`/;01KFe/0DE5A04Y:BP1lO7`0ZJVY02d];@1LG5`0LG5a04a<C01[Jf/0?3`l02LW
9`1KFe/0J6QX04a<C01IFET051@D0451@@1dM7@0CTi>08N7Q`1;Bd/0<c<c055AD@29RHT0Bd];06=S
H`0d=3@0@d=306MWI`2UYJD0>3Ph04M7A`80B4Q808`0WYjN0861P@0G5aL0<c<c059BDP1HF5P0U9BD
04Q8B00J6QX092@T03/k>`1TI6@0UiNG03Hf=P0Z:RX0>c/k04e=C@1TI6@0Xj>S02HV9P0l?3`0=3@d
06=SH`1IFET0H61P02l_;`16ATH0Fe]K05ADE012@T80De=C01HF5P0X:2P0HV9R03<c<`1IFET0M7Ad
03De=@1cLg<0TY:B05YJFP1IFET0NWYj03<c<`2OWil0Xj>S02LW9`10@400G5aL03Ph>01cLg<0>CTi
02@T9016ATH0Cdm?03dm?@1oOgl0KFe]01hN7P1JFUX0IFEU05iNGP1PH600:2PX03@d=01UIFD0X:2P
079bLP2VYZH0>CTi02DU9@1ZJVX0H61P04E5A@2YZJT08R8R02l_;`1CDe<0LW9b06MWI`28R8P0:2PX
02h^;P1eMGD0DE5A08n?S`2IVIT07QhN02<S8`25QHD0DU9B06a/K01:BTX0>CTi03/k>`1oOgl0FEUI
061PH00L71`06QXJ03/k>`2KVi/0D51@04A4A00I6AT0<c<c07YjNP2XZ:P0B4Q805ADE00g=cL0LW9b
09FEU@0/;2`0<C4a04e=C@12@T80VIVI05YJFP0T92@0>c/k04U9B@1bLW80MgMg02DU9@0l?3`0I6AT
03Lg=`1OGel0EEEE02<S8`1iNGT0SXj>069RHP20P8009RHV059BDP1RHV80L71`09bLW080>c/k0300
MWIf07MgM`2=SHd0@D5102/[:`1?Cdl0]kNg06=SH`1HF5P08R8R03hn?P1[Jf/0[jn_06a/K01OGel0
:2PX05mOG`1oOgl0LW9b03Lg=`1<C4`0?Cdm05QHF02SXj<0Fe]K03Ti>@18B4P0@41004U9B@21PH40
HV9R02TY:@1JFUX0CTi>06m_K`2IVIT0;Rh^02l_;`1DE5@0GUiN06UYJ@2@T900:b/[01lO7`13@d<0
C4a<05MGE`2>SXh201lO7`0>04A4A01RHV80BDU907alO014A4@0AdM7059BDP1dM7@0ADE506UYJ@0k
>c/0@D5107]kN`1`L70>ooooo`0306ATI01nOWh0BTY:0080?clo07L0K6a/04=3@`1JFUX0JVYZ055A
D@1TI6@0EUIF05eMG@1ADE40EEEE04i>CP1ZJVX0BDU904A4A00Q8B40=cLg034a<@0m?Cd0?3`l03/k
>`11@D40E5AD05YJFP1KFe/0?3`l04Y:BP1;Bd/0A4A404I6AP1EEED0EUIF055AD@19BDT0<c<c04A4
A00W9bL0@41003lo?`17AdL0<S8b03Xj>P0i>CT0BDU904I6AP1CDe<0?Shn04m?C`0h>3P0B4Q803De
=@0Q8B40ATI603dm?@0d=3@0Bd];05=CD`11@D40C4a<04Q8B01NGUh0G5aL02d];@0e=CD0G5aL055A
D@0c<c<0>3Ph06a/K00T92@0:b/[0410@00`<300ATI603dm?@0^;Rh0=3@d03Hf=P0`<3009bLW02l_
;`0O7al0<C4a03Xj>P13@d<0=CDe03Ti>@1=CDd09bLW02d];@0j>SX0?clo02TY:@0g=cL0ATI604Y:
BP1IFET0;bl_051@D01HF5P07QhN04I6AP0Q8B40C4a<02XZ:P1:BTX0?3`l03lo?`0b<S80:BTY0451
@@0n?Sh07QhN04A4A00j>SX0>CTi051@D01hN7P0EEEE07QhN00203Xj>P1H065QH@0h>3P0MWIf05YJ
FP1SHf<0@d=309VIV@1IFET0EEEE06UYJ@1]KFd0A4A406MWI`0X:2P0AdM705YJFP1<C4`0OWin05QH
F02ATI40Cdm?06UYJ@1QHF40LW9b04e=C@0[:b/0EUIF0492@P0T92@0Jf][03Xj>P1lO7`0<c<c03/k
>`0h>3P0ATI605QHF01BDU80=cLg03De=@1aLG40FUYJ04E5A@12@T80SHf=03lo?`1;Bd/0DE5A034a
<@1?Cdl0Kfm_04e=C@11@D40I6AT01HF5P14A4@0ATI605UIF@1<C4`0CTi>08b<S01HF5P0OWin0492
@P0n?Sh0A4A405]KF`1FEUH0BDU904E5A@1@D500>3Ph0:n_[`1gMgL0>3Ph05ADE00Z:RX0B4Q803`l
?00f=SH0B4Q805YJFP1LG5`0=CDe05=CD`16ATH0C4a<03De=@80>3Ph03h0B4Q806=SH`0Z:RX0HF5Q
05EEE@1GEeL051@D05UIF@13@d<0>CTi05]KF`0O7al0CDe=04U9B@0];Bd0@41005QHF00V9RH0;Rh^
051@D00[:b/06ATI02XZ:P0P8200De=C0410@02@T900L71`03dm?@1JFUX0:2PX061PH01LG5`0HV9R
07moO`1KFe/0C4a<04i>CP1CDe<0K6a/09:BTP1GEeL0@D5105MGE`1IFET0CDe=04Y:BP0o?cl0C4a<
05ADE01TI6@0?Shn04E5A@1oOgl0G5aL04Y:BP1FEUH0>c/k0492@P1UIFD0DE5A05EEEA3ooooo000>
ooooo`0m0;no_`1CDe<0OWin07moO`2/[:`0ATI603lo?`1@D500M7Ad09>CT`0c<c<0=CDe05iNGP1E
EED0PX:2079bLP0Y:BT0ATI606ATI028R8P0ZZZZ03<c<`0i>CT0I6AT03hn?P1WIfL0De=C024Q8@0l
?3`0KFe]0492@P14A4@0G5aL02PX:00o?cl0L71`04m?C`1XJ6P0?3`l02TY:@1HF5P0UYJF04i>CP0o
?cl0:b/[075aL@2EUID0>SXj028R8P0o?cl0<30`08Z:RP2LW9`0:BTY02@T901ADE40>c/k0:RXZ01e
MGD05ADE0410@00206YZJP1^0;Zj^P1@D500=SHf04];B`1JFUX0EUIF08Z:RP0O7al0>3Ph05YJFP2F
UYH0Bd];06m_K`1JFUX07QhN05EEE@2PX:00=SHf06IVIP0^;Rh0:RXZ07MgM`2QXJ40:2PX08B4Q00O
7al0=CDe08F5Q@1MGEd0F5QH06ATI00U9BD0EEEE08^;R`14A4@0>CTi05IFEP0k>c/0;bl_08f=S@0S
8b<0=cLg03lo?`0m?Cd0NWYj09VIV@0c<c<0>CTi05mOG`1YJFT0UiNG0:j^[P0l?3`0EUIF08b<S01M
GEd0LG5a09bLW00l?3`0Bd];09ZJVP1bLW80UIFE07inOP0O7al0>c/k07YjNP1NGUh0MGEe02l_;`0^
;Rh0FEUI061PH02NWYh0IFEU01hN7P0i>CT0K6a/08R8R00m?Cd0EEEE02LW9`0k>c/0JVYZ07UiN@0_
;bl0CTi>03hn?P17AdL0Ph>307YjNP0^;Rh0Bd];02XZ:P2>SXh0XJ6Q03`l?00U9BD0BTY:04a<C02M
WId0EUIF01hN7P0X:2P0>CTi05=CD`2OWil0>SXj01/K6`0T92@205mOG`1W0820P00T92@09RHV03hn
?P21PH40EUIF09bLW00o?cl0G5aL071`L01BDU80Jf][08F5Q@0U9BD0Hf=S06QXJ010@400NGUi04I6
AP0Q8B40O7al07QhN00m?Cd0D51@02`/;00`<300JFUY065QH@13@d<09BDU055AD@1dM7@0BTY:04a<
C00P8200@T9208B4Q01`L7007alO06][J`1DE5@0QXJ605iNGP0W9bL0<30`05]KF`26QXH0WYjN04U9
B@0X:2P0J6QX06EUI@1oOgl0WIfM02/[:`0^;Rh0KVi^08>3P`1hN7P0QHF503@d=012@T80Kfm_09BD
U01bLW80]KFe06IVIP0d=3@0GemO07emO@1=CDd0UYJF05]KF`0i>CT0YjNW06m_K`1^KVh0WIfM03lo
?`1:BTX0Rh^;06i^KP10@400Jf][02HV9P0_;bl0Fe]K03Xj>P0Y:BT0GUiN02DU9@0d=3@0FEUI05YJ
FP0l?3`0FEUI03dm?@0Z:RX0GEeM07AdM00R8R80>CTi03De=@0>ooooo`09051@D01:BTX0>CTi05mO
G`1GEeL0CDe=04A4A01CDe<0LG5a0080ATI600H0OWin07UiN@1GEeL0KVi^059BDP1ADE4204=3@`0f
02<S8`12@T80=3@d03lo?`1IFET0@41004E5A@1LG5`0IfMW08R8R019BDT0De=C05YJFP1BDU80ADE5
04A4A01@D500Hf=S03De=@0j>SX0C4a<03Ph>01<C4`0DE5A05EEE@0^;Rh0@d=3051@D01?Cdl0;bl_
0451@@1dM7@0?Cdm01DE5@0k>c/0@T9204i>CP10@400:b/[04m?C`1PH600A4A402TY:@1DE5@0>c/k
07EeM@1YJFT0CDe=04i>CP1@D500>c/k03Xj>P0h>3P0Hf=S0P1;Bd/0;P1FEUH0<C4a04Y:BP1@D500
?Cdm03<c<`0];Bd0:2PX02d];@0n?Sh09RHV02PX:018B4P0E5AD030`<013@d<0E5AD01XJ6P0Q8B40
A4A4030`<00a<C40:b/[0451@@0k>c/0>CTi03dm?@1;Bd/0=cLg03Ti>@1SHf<0:b/[02`/;00a<C40
K6a/020P800o?cl0?Shn03De=@14A4@0<S8b01/K6`0i>CT0=cLg04I6AP0j>SX20492@P0[09ZJVP11
@D40DU9B055AD@0^;Rh0BTY:04];B`1OGel0<c<c03/k>`0m?Cd0<S8b05mOG`1^KVh0KFe]06YZJP0f
=SH0BDU906ATI01KFe/0/K6a05eMG@1SHf<0H61P08B4Q00o?cl0K6a/06UYJ@0k>c/0HV9R055AD@0V
9RH0JFUY01PH6015ADD09RHV04U9B@0T92@0<30`04a<C01YJFT0>CTi02PX:00203De=@1C065QH@0c
<c<0C4a<03<c<`0b<S80=cLg04a<C01QHF40P82003De=@2EUID0Xj>S03De=@1GEeL0FUYJ07UiN@1^
KVh0IfMW09:BTP1FEUH0B4Q805aLG010@400C4a<04M7A`12@T80D51@04];B`1ZJVX07alO06QXJ01C
De<0FUYJ07=cL`0i>CT0C4a<04I6AP1BDU80G5aL069RHP1OGel0EUIF05QHF01CDe<0E5AD0410@00j
>SX0DU9B0492@P1ADE40=3@d06e]K@1TI6@0@T9201PH600m?Cd0<C4a04M7A`1iNGT04Q8B0410@01<
C4`0?3`l03Hf=P1BDU80<30`04I6AP1IFET0=CDe02l_;`18B4P0=3@d07alO01ZJVX0DE5A04M7A`0Z
:RX0MWIf0492@P1ZJVX0LG5a06EUI@2PX:000P1iNGT06@1eMGD0LW9b05EEE@1`L700IfMW051@D01U
IFD0Jf][05EEE@1DE5@0BDU905iNGP18B4P0F5QH04a<C016ATH0De=C05ADE01FEUH0GEeM04U9B@1?
Cdl0O7al05eMG@1PH6004?ooool000kooooo00/0Ng]k09bLW02j^[X0FUYJ09nOW`2;Rh/0LW9b04U9
B@12@T80Ng]k04=3@`0205iNGP2m02LW9`1DE5@0ZJVY055AD@1TI6@071`L03<c<`1oOgl0Ph>302l_
;`1LG5`0D51@09:BTP1dM7@09RHV02TY:@1>CTh0JFUY07MgM`1]KFd061PH02<S8`1MGEd0NWYj08b<
S00c<c<08R8R03@d=01`L700M7Ad09:BTP0`<300<S8b071`L020P800CDe=07alO00/;2`0;Rh^07]k
N`1VIVH0AdM707AdM00N7Qh0?3`l06ATI016ATH0JVYZ05YJFP0Z:RX0@T9205mOG`0Y:BT0Lg=c05aL
G00W9bL0P82003De=@0P8200?Shn03Lg=`0Z:RX0ADE5069RHP0G5aL0<C4a04M7A`0k>c/0KFe]03Ti
>@0W9bL0?3`l07UiN@1_Kfl0[Zj^03Ph>00S8b<0BTY:04=3@`22PX80WinO01LG5`0c<c<0S8b<096A
T@2:RXX0[jn_01/K6`0e=CD0IfMW06a/K028R8P0JFUY01DE5@1BDU80Q8B4069RHP22PX80FEUI02PX
:01@D500IVIV0492@P1PH600>3Ph030`<01DE5@0Kfm_04Q8B011@D408b<S04E5A@1aLG40Shn?05AD
E01QHF40@D5105ADE025QHD0E5AD028R8P0k>c/0ATI609BDU02EUID0;2`/03<c<`1;Bd/0MgMg06e]
K@2ATI40:RXZ04=3@`1;Bd/0LW9b06MWI`23Ph<07QhN04m?C`1MGEd0K6a/05QHF02=SHd05aLG04Q8
B01nOWh0J6QX05ADE02<S8`0:b/[0492@P28R8P0Jf][06e]K@1ZJVX0;2`/05mOG`2LW9`0?Shn04U9
B@0n?Sh0?Cdm07emO@1_Kfl0=cLg05UIF@0l?3`0>3Ph06IVIP1PH600>SXj05mOG`0i>CT0?Shn07al
O01=CDd0FEUI03hn?P1@D500PH6106MWI`0/;2`0Bd];059BDP1`L700N7Qh02@T900R8R80@4100080
Jf][00L07AdM0410@01YJFT0ADE507QhN00M7Ad0<C4a0080JVYZ0440@D5109NGU`0f=SH0MGEe08j>
SP1CDe<0N7Qh07emO@0k>c/0XJ6Q0;Vi^@1CDe<0ZZZZ038b<P18B4P0RXZ:07IfMP1`L700V9RH03@d
=00b<S80VIVI05aLG01EEED0P82002l_;`1JFUX0GUiN06EUI@12@T80G5aL02h^;P10@400S8b<0:6Q
X@1;Bd/0D51@03Hf=P1>CTh0IfMW05EEE@0c<c<0Cdm?06ATI01OGel0LG5a03/k>`0R8R80Cdm?065Q
H@1YJFT0^KVi0451@@0X:2P0>c/k05YJFP1GEeL0[Jf]0492@P10@400B4Q8071`L01QHF40W9bL030`
<00>ooooo`0;05YJFP1RHV80@T920492@P1DE5@0De=C04];B`1TI6@0HV9R04E5A@1FEUH00P16ATH0
VP1bLW80MWIf05MGE`1RHV80B4Q80410@00M7Ad0=cLg02LW9`0k>c/0;2`/038b<P18B4P0EUIF04m?
C`1<C4`0@d=3061PH01;Bd/0E5AD04A4A01=CDd0JFUY06a/K015ADD0:b/[04a<C00`<300CDe=04A4
A01ADE40=cLg05EEE@1@D500G5aL04U9B@18B4P0<C4a03lo?`0Q8B40<c<c04M7A`0];Bd0?3`l030`
<00_;bl08B4Q03/k>`12@T80>SXj034a<@0m?Cd0A4A40451@@1>CTh0BDU904Y:BP1mOGd0MgMg065Q
H@0H61P07AdM05]KF`0m?Cd0BTY:04];B`1<C4`0Bd];034a<@14A4@0<c<c02d];@0k>c/0I6AT04a<
C010@400Bd];0451@@1>CTh0<C4a03hn?P1HF5P0>3Ph03lo?`0j>SX0<C4a0451@@17AdL0?clo03hn
?P15ADD071`L04U9B@0j>SX0F5QH01lO7`0m?Cd061PH03/k>`14A4@0FUYJ05mOG`0[:b/05QHF03/k
>`0Z:RX0;Rh^03`l?01BDU80:RXZ03hn?P0i>CT0IfMW05]KF`14A4@0O7al03Hf=P1LG5`0>3Ph07Mg
M`18B4P0De=C04I6AP1@D500>CTi04U9B@0K6a/0>c/k061PH01MGEd0i>CT06i^KP21PH40E5AD071`
L012@T80Cdm?04Y:BP0h>3P0FEUI05=CD`1EEED0Ogmo02/[:`1HF5P0=cLg071`L00U9BD0>SXj04i>
CP0o?cl0Cdm?02/[:`1>CTh203Xj>P0Z02l_;`1HF5P0@d=304Q8B01CDe<0=CDe04];B`1ZJVX0=cLg
04Q8B01?Cdl0>3Ph04M7A`13@d<0?Cdm06i^KP1ZJVX0];Bd08Z:RP1MGEd0De=C05IFEP14A4@0>SXj
03Lg=`1>CTh0@41004i>CP0l?3`0][Jf05aLG00V9RH0DU9B0410@01YJFT0De=C03Xj>P0n?Sh0FEUI
05mOG`0l?3`0H61P0P1<C4`0@@0_;bl0@d=303Xj>P10@400CTi>03Xj>P1MGEd0GUiN05EEE@0;2`/0
?Cdm038b<P0c<c<0FEUI02h^;P0j>SX0BTY:05=CD`13@d<0FUYJ02/[:`19BDT0CTi>03Xj>P0T92@0
BTY:01dM7@13@d<0ADE506i^KP1bLW80ADE505]KF`21PH40Q8B405mOG`0^;Rh0K6a/04=3@`1kNg/0
S8b<08F5Q@1>CTh0MWIf06e]K@0`<300G5aL05eMG@1GEeL0AdM704Q8B01:BTX0DU9B04I6AP11@D40
CTi>07AdM01FEUH0DU9B09VIV@1UIFD0FUYJ07]kN`1IFET0P820013ooooo000>ooooo`0U07UiN@1j
NWX0/;2`055AD@1OGel0XJ6Q0<_;b`1SHf<0=cLg05eMG@1TI6@0VIVI08>3P`0h>3P0:2PX06UYJ@26
QXH0/[:b02LW9`15ADD0B4Q809RHV01PH600YjNW01/K6`1DE5@0KFe]04];B`1]KFd0IVIV01dM7@1g
MgL0RHV904Q8B01@D500MgMg02`/;00208^;R`0<02l_;`1HF5P0LG5a01lO7`1NGUh0CDe=03De=@1J
FUX0=CDe02d];@1_Kfl0?3`l0P0h>3P0I`0l?3`0L71`07inOP0_;bl0ADE503lo?`1=CDd0OWin09bL
W00W9bL0>CTi04i>CP0k>c/0VIVI0:>SX`0K6a/0>c/k04A4A01NGUh0^k^k05UIF@0H61P0<C4a06=S
H`1YJFT0VIVI02h^;P0I6AT0@d=3059BDP1UIFD0QHF501HF5P0g=cL0Jf][07IfMP1IFET0Cdm?01XJ
6P1=CDd0QHF505iNGP1:BTX0>c/k02d];@1;Bd/0OGem0410@00X:2P0ADE502HV9P1MGEd0PX:201LG
5`0X:2P0@T9204E5A@2JVYX0NWYj01`L700S8b<0DU9B061PH02=SHd0@d=302LW9`1:BTX0JVYZ06=S
H`1ZJVX07alO03Ti>@1ADE40EEEE05aLG01fMWH0De=C034a<@1UIFD0Lg=c05eMG@20P800=cLg06=S
H`2GUiL0P82004A4A02FUYH0=cLg03Xj>P1TI6@0BTY:055AD@1lO7`07QhN0492@P1HF5P0=cLg0492
@P1gMgL09BDU03<c<`1hN7P00P18B4P06`1CDe<09BDU05aLG02n_[h0A4A403dm?@0Z:RX0<C4a05=C
D`2[Zj/0>CTi05aLG00h>3P0AdM70:>SX`2QXJ40:b/[04I6AP1@D500FUYJ09fMW@0g=cL0:b/[03Ph
>01NGUh0Ph>3092@T00203De=@1A04m?C`1GEeL0WIfM034a<@10@400BDU905QHF01@D500Kfm_02@T
9013@d<0GemO03Xj>P12@T80<C4a04M7A`1XJ6P0ATI604M7A`1^KVh0>c/k05iNGP18B4P09bLW04M7
A`0a<C40@41008f=S@1;Bd/0>CTi03Xj>P1CDe<0Lg=c0:j^[P0o?cl0@d=303hn?P1BDU80[Zj^05mO
G`0O7al0IFEU038b<P1]KFd0dm?C0861P@0S8b<0D51@03hn?P1oOgl0cLg=04Y:BP0`<300IFEU05UI
F@1/K6`0/k>c04E5A@0N7Qh0@41006UYJ@1gMgL0];Bd03Hf=P0e=CD0CTi>06YZJP1jNWX0QXJ601lO
7`1BDU80OWin06=SH`0[:b/0JFUY02LW9`15ADD0SXj>0:ZZZP0n?Sh0De=C0080?clo00P0KFe]07]k
N`0Y:BT0?Cdm03Hf=P19BDT0DU9B04m?C`kooooo04H0C4a<059BDP12@T80DU9B04E5A@1;Bd/0=SHf
05EEE@1YJFT0@D5105IFEP1ADE40J6QX05=CD`1eMGD0DE5A05]KF`16ATH0@D5103dm?@1MGEd0?Cdm
05ADE01>CTh0>3Ph051@D01KFe/0FEUI06EUI@1=CDd0GUiN055AD@1?Cdl0DE5A05eMG@1cLg<0Ogmo
02h^;P0f=SH0?clo04E5A@1CDe<0?clo05=CD`13@d<0Cdm?061PH01ZJVX0Cdm?05ADE00e=CD0FEUI
02XZ:P0j>SX0IVIV03@d=017AdL0>SXj04U9B@1<C4`0EeMG04E5A@1<C4`0EEEE05eMG@1QHF40=SHf
030`<01GEeL0CDe=0P0Z:RX0100k>c/0;2`/03@d=01=CDd203`l?00@03Hf=P0T92@0?Shn02XZ:P0d
=3@0>3Ph02`/;00X:2P0:b/[051@D01KFe/0>SXj03dm?@17AdL08R8R020P8080ADE505P0?3`l04I6
AP0j>SX0BDU906=SH`12@T80EEEE05mOG`0U9BD0Bd];02l_;`0m?Cd071`L05aLG00c<c<0BDU905EE
E@0d=3@0BDU902l_;`0[:b/0CDe=01dM7@0d=3@0?Cdm06i^KP0k>c/0P82003De=@0n?Sh0BDU904A4
A00l?3`0AdM7071`L015ADD0/k>c051@D00b<S80De=C065QH@0i>CT0EUIF03dm?@1UIFD0HF5Q06QX
J02=SHd0DE5A05eMG@16ATH0H61P04Q8B01HF5P0EEEE04=3@`1UIFD0GemO03<c<`22PX80Bd];04i>
CP11@D40ADE502`/;00g=cL0@d=304M7A`18B4P0<C4a05YJFP1<C4`0A4A403@d=01OGel0;Bd]030`
<018B4P0@d=306=SH`1mOGd0G5aL07EeM@1^KVh0>c/k06IVIP1^KVh0Hf=S0P1MGEd0D01nOWh0HF5Q
06a/K00n?Sh0=cLg03<c<`1BDU80ATI604=3@`16ATH0DU9B02PX:01YJFT0Fe]K04M7A`1MGEd0:b/[
04e=C@0c<c<0;Rh^03Ph>00j>SX0E5AD03De=@1CDe<0A4A405ADE00^;Rh0=SHf03Xj>P18B4P0KFe]
02h^;P1EEED0BDU904=3@`0/;2`0?Cdm03Ti>@1GEeL0GemO018B4P12@T80>SXj02h^;P0V9RH0BTY:
01DE5@0[:b/0EEEE04M7A`1@D500LW9b04M7A`1BDU80De=C06YZJP1YJFT0=CDe05QHF00a<C40B4Q8
04m?C`16ATH0P82004E5A@1eMGD0KFe]07YjNP1lO7`0XJ6Q08Z:RP1:BTX0PH6106EUI@1KFe/0<C4a
05YJFP0b<S80BDU90P15ADD02P1LG5`0UIFE059BDP1PH600PH61051@D019BDT0H61P071`L01HF5P@
ooooo`003_ooool0[02>SXh0EeMG075aL@1fMWH0NWYj0410@01LG5`0VIVI06e]K@1mOGd0<C4a04A4
A01QHF40F5QH04e=C@1FEUH0=cLg04a<C01@D500C4a<07alO00U9BD0<S8b06a/K017AdL0O7al051@
D00Y:BT0LW9b0861P@0n?Sh0JVYZ079bLP0b<S80?Shn079bLP11@D40C4a<03Hf=P0o?cl0Jf][0861
P@0h>3P0H61P05QHF017AdL0UIFE06a/K00`<300F5QH04A4A01PH600GemO02d];@0k>c/0D51@03hn
?P1eMGD0CDe=02XZ:P1<C4`0GEeM05]KF`1hN7P0@410034a<@15ADD0J6QX05IFEP1^KVh071`L02DU
9@18B4P0Ogmo05EEE@1>CTh0BDU902l_;`1IFET0IFEU04A4A01ADE40=3@d05UIF@1aLG40C4a<061P
H01cLg<0>CTi05aLG01eMGD0:b/[04Y:BP1IFET0>3Ph06i^KP20P800=CDe065QH@1bLW80=SHf051@
D02;Rh/0<C4a05QHF01DE5@0>SXj07QhN02FUYH0@410051@D01/K6`0AdM7071`L01gMgL0;Bd]06=S
H`1VIVH0>c/k05QHF01^KVh09RHV05mOG`1>CTh0>3Ph07IfMP1=CDd0:2PX055AD@1YJFT0=cLg06][
J`0Z:RX0?Cdm05YJFP0f=SH0HF5Q06m_K`0k>c/0=CDe07YjNP1PH600>3Ph05ADE00b<S80EeMG08f=
S@17AdL0BDU905aLG00`<300B4Q8071`L011@D40?Shn05eMG@0m?Cd0Kfm_075aL@0Y:BT0>SXj061P
H01;Bd/0Shn?04e=C@0Z:RX0ADE505mOG`11@D40Q8B404M7A`0];Bd206ATI00a04Y:BP1cLg<0:b/[
034a<@1eMGD0KFe]04Q8B02<S8`0>3Ph05=CD`1fMWH0FEUI05IFEP28R8P0;Bd]071`L01fMWH0@D51
05]KF`0/;2`0<C4a07emO@1gMgL0ATI6055AD@0/;2`0J6QX05ADE015ADD0ATI603lo?`1EEED0UIFE
04e=C@1JFUX0@41005EEE@1VIVH0>c/k020P801NGUh0D51@08Z:RP1<C4`0:RXZ04M7A`1hN7P0IVIV
08n?S`0204A4A00e061PH01dM7@0CTi>08N7Q`0`<300?3`l05IFEP2CTi<0Hf=S07alO00^;Rh0?3`l
05UIF@1>CTh0DE5A08f=S@15ADD0<c<c055AD@1NGUh0CDe=08b<S01/K6`0=3@d08:2PP1kNg/0=cLg
09:BTP0e=CD0@41006a/K014A4@0<c<c05QHF00e=CD0=3@d06a/K00i>CT0<c<c05YJFP1>CTh0FUYJ
079bLP1RHV80=SHf05iNGP15ADD0EeMG06][J`1GEeL0@D5105aLG019BDT03_ooool07`1EEED0GUiN
03hn?P1EEED0Cdm?04E5A@0j>SX0GEeM071`L00i>CT0ATI606YZJP1WIfL0D51@06IVIP1>CTh0GUiN
03`l?00/;2`08R8R03lo?`0X:2P0=CDe03Lg=`0];Bd0@D51051@D01PH600J6QX04=3@`1@D5000P1N
GUh0g017AdL0D51@065QH@1PH600:b/[038b<P0m?Cd0:2PX04I6AP15ADD0CDe=02PX:012@T80EeMG
05eMG@0];Bd0Bd];03lo?`1:BTX0;Bd]03Ph>01DE5@0C4a<04M7A`0d=3@0ATI605EEE@1?Cdl0AdM7
055AD@17AdL0Fe]K05QHF015ADD0C4a<06=SH`1ADE40Cdm?05EEE@1IFET09BDU024Q8@1ADE40<c<c
03/k>`0f=SH0@T920410@00];Bd0>SXj03De=@0k>c/0?3`l04M7A`1EEED0Cdm?04Y:BP11@D40BDU9
02PX:011@D40De=C04A4A01>CTh0Cdm?03dm?@1?Cdl0D51@0451@@1EEED0Fe]K03lo?`1LG5`0Cdm?
06EUI@16ATH0GEeM03/k>`1=CDd0ADE5055AD@1?Cdl0?Shn02XZ:P0m?Cd092@T04E5A@0l?3`0DU9B
03<c<`1JFUX0?Cdm06EUI@1>CTh0EUIF071`L01EEED0PX:205IFEP1kNg/0DE5A03lo?`1HF5P0I6AT
03De=@1<C4`08B4Q03dm?@1IFET0JVYZ0:^[Z`1DE5@0Kfm_05=CD`1ZJVX0Cdm?05IFEP1<C4`0=SHf
06YZJP17AdL0<S8b08V9R@12@T80EEEE0451@@1HF5P0@d=303lo?`1:BTX0GemO05=CD`0c<c<0ADE5
04a<C01KFe/0@T92055AD@0_;bl0@41004];B`0o?cl0Cdm?075aL@1CDe<0IFEU061PH00`<300?Cdm
04U9B@1OGel0IVIV05QHF02MWId0Hf=S055AD@1BDU80BTY:04A4A00o?cl0@d=305MGE`1IFET0IVIV
04E5A@2?Shl0H61P04a<C01VIVH0;Bd]04];B`0d=3@0@41004e=C@17AdL0F5QH04E5A@14A4@0?clo
04i>CP0`<300<C4a03Ph>00e=CD0C4a<02`/;01^KVh0Cdm?04E5A@0B4Q80BDU903@d=01BDU80IFEU
01PH600g=cL0C4a<03hn?P0h>3P0Bd];03`l?01?Cdl0FEUI03Xj>P0g=cL0BTY:04U9B@1]KFd0De=C
079bLP2ATI40D51@07UiN@1CDe<0N7Qh06][J`1RHV80QhN7051@D01ZJVX208J6QP0G06e]K@1aLG40
Q8B4051@D01]KFd0BTY:05aLG00b<S80@41003Xj>P19BDT0ATI603Ti>@0f=SH0N7Qh04a<C01TI6@0
HF5Q05IFEP1NGUh0EeMG06QXJ01RHV804?ooool000kooooo0?l0^;Rh09RHV027QhL0EeMG09VIV@1g
MgL0ZJVY0410@00o?cl0FEUI05eMG@2NWYh0JVYZ02PX:012@T80I6AT051@D01CDe<0:2PX02h^;P1P
H600IVIV03<c<`19BDT09RHV07MgM`29RHT09RHV02/[:`0n?Sh0<30`05]KF`1iNGT0;2`/024Q8@19
BDT0EeMG09VIV@1FEUH051@D04A4A01/K6`0Cdm?0:FUY@0/;2`0=CDe05MGE`2BTY80Bd];08F5Q@0G
5aL0E5AD061PH01:BTX0I6AT09nOW`0N7Qh0QHF50:^[Z`1LG5`0O7al07YjNP0c<c<0J6QX06UYJ@1>
CTh0UIFE04M7A`0h>3P0J6QX05MGE`13@d<0Ogmo02<S8`0/;2`0GEeM04a<C016ATH0GEeM02DU9@0j
>SX0R8R805mOG`14A4@0<c<c01@D501;Bd/0S8b<05MGE`0i>CT0Bd];03hn?P22PX80XJ6Q01lO7`0_
;bl0EUIF03/k>`2NWYh0_Kfm01/K6`0U9BD0De=C03lo?`2JVYX0@D5101LG5`0^;Rh0GEeM06][J`1a
LG40A4A402PX:01RHV80I6AT0492@P2KVi/0De=C02<S8`1QHF40LW9b04];B`1>CTh0;Rh^03Lg=`1J
FUX0TI6A04e=C@12@T80;Rh^04=3@`1]KFd0IVIV02`/;01JFUX0<30`04m?C`1fMWH0>SXj03`l?01G
EeL0>c/k07QhN01lO7`09bLW03De=@1?Cdl0;Bd]07moO`1QHF406ATI02h^;P1<C4`0AdM706MWI`2?
Shl05aLG04A4A024Q8@0>SXj06a/K02/[:`0;2`/05EEE@24Q8@0?clo07IfMP24Q8@0BTY:05]KF`1d
M7@0@d=30:2PX01FEUH0<S8b07EeM@1<C4`0G5aL06QXJ016ATH0D51@059BDP1CDe<0BTY:051@D00^
;Rh0ADE507=cL`0h>3P0EUIF02l_;`0e=CD0L71`05EEE@0I6AT0=cLg03@d=01BDU80Q8B402TY:@0e
=CD0E5AD075aL@1@D5006ATI02d];@1IFET0NWYj08V9R@0S8b<0<S8b06YZJP1@D500AdM708F5Q@0h
>3P0>c/k069RHP1BDU80L71`06QXJ00h>3P0FUYJ0861P@1FEUH0W9bL03Ph>01DE5@0KFe]03Xj>P1G
EeL0SHf=01XJ6P12@T80OWin03dm?@1VIVH0JVYZ04];B`0W9bL0I6AT08^;R`1>CTh0IFEU030`<00j
>SX0DE5A09:BTP1@D500M7Ad02h^;P0k>c/0Fe]K06YZJP14A4@001L0Bd];01`L701FEUH0Ph>303lo
?`0F5QH0DU9B03hn?P18B4P0WinO03lo?`0T92@0DE5A05IFEP1TI6@0/;2`03Ti>@0O7al0D51@04Y:
BP1/K6`0/;2`02`/;00>ooooo`3o04i>CP1eMGD0CDe=07YjNP1MGEd0LG5a03hn?P1NGUh0SHf=05AD
E01bLW80N7Qh06i^KP1SHf<0VYZJ05QHF01MGEd0?clo05mOG`0[:b/0DU9B04Q8B01JFUX0C4a<03Xj
>P1PH600?Cdm06MWI`1bLW80FUYJ059BDP1jNWX0CDe=075aL@1hN7P0H61P04a<C00b<S80:b/[02l_
;`1FEUH0I6AT04=3@`19BDT0>SXj04I6AP1BDU80FEUI02TY:@1PH600?clo05aLG00_;bl0<S8b05iN
GP0_;bl0=3@d020P801=CDd0Bd];05ADE00X:2P0De=C05iNGP1fMWH0EEEE03Hf=P11@D40FUYJ055A
D@1=CDd0De=C07IfMP0m?Cd0<S8b07EeM@0m?Cd0C4a<03Ph>01ADE40DU9B02<S8`17AdL0BDU903Xj
>P0m?Cd0>c/k065QH@15ADD0Hf=S051@D01GEeL0:BTY04m?C`1]KFd0CDe=05eMG@1UIFD0<c<c06QX
J01WIfL0?clo05UIF@1YJFT0>CTi059BDP1>CTh0LG5a059BDP1PH6009BDU051@D01LG5`0=SHf03dm
?@1?Cdl0:RXZ04Q8B00_;bl0>c/k0410@01KFe/0CDe=06YZJP0U9BD0EeMG04I6AP0n?Sh0FEUI05YJ
FP2OWil0C4a<06UYJ@1@D500<C4a05MGE`1IFET0?clo05EEE@0U9BD0Bd];06IVIP1mOGd0alO703Ph
>01/K6`0FEUI05aLG011@D40Ogmo06UYJ@0g=cL0JFUY065QH@0Q8B40GemO05IFEP1aLG40@41004=3
@`11@D40CTi>05IFEP0i>CT0CTi>04A4A01jNWX0K6a/055AD@0c<c<0LG5a02h^;P0a<C40C4a<05MG
E`1EEED0OWin03hn?P1EEED0FEUI034a<@1=CDd0D51@07MgM`1bLW80BDU9071`L018B4P0Cdm?05AD
E00j>SX0>CTi04];B`0d=3@0BTY:05=CD`1PH600>3Ph0:NWY`1PH600?Shn05]KF`0Q8B40Jf][04i>
CP16ATH0Cdm?05iNGP1eMGD0CTi>06QXJ01FEUH0C4a<03`l?01BDU80@d=305mOG`1IFET0De=C06IV
IP1_Kfl0FEUI03Hf=P19BDT0ADE505EEE@1[Jf/0820P03hn?P1HF5P0De=C04m?C`1<C4`0>c/k05AD
E01XJ6P0DU9B06=SH`1gMgL0AdM705=CD`1=CDd0[Jf]08b<S00l?3`0Ph>304Q8B01NGUh0DE5A07em
O@2BTY80@d=309>CT`20P800PH61000G05YJFP1MGEd0UiNG06e]K@2_[jl0JFUY051@D016ATH0KFe]
05mOG`1;Bd/0@d=305=CD`1MGEd0HF5Q03<c<`1@D500GemO03Xj>P13@d<0DE5A05IFEP1IFET04?oo
ool000kooooo03<0HV9R08^;R`2l_;`0G5aL04E5A@13@d<0IVIV08F5Q@0k>c/0Cdm?03@d=01TI6@0
QHF502HV9P1;Bd/0A4A406e]K@2SXj<0<S8b03Ti>@1ADE40DU9B05YJFP2JVYX06QXJ04A4A01_Kfl0
IFEU09fMW@25QHD08b<S03Hf=P1JFUX0QHF506YZJP2YZJT0:RXZ04U9B@1UIFD0=CDe06a/K00/;2`0
7QhN05MGE`1BDU80D51@0861P@0O7al0?clo06a/K00_;bl00P0m?Cd0Z011@D40F5QH075aL@0`<300
@d=303/k>`1:BTX0MgMg08B4Q00S8b<0>3Ph04I6AP18B4P0YZJV07EeM@0M7Ad0@T9203/k>`1OGel0
dM7A065QH@0I6AT0=cLg059BDP1cLg<0OWin02XZ:P0H61P0=SHf05QHF02LW9`0N7Qh01<C4`0h>3P0
BDU90492@P1nOWh0MgMg01@D501:BTX0Lg=c061PH01UIFD0AdM701`L700`<300HV9R04Q8B00R8R80
>3Ph04E5A@1?Cdl0FEUI02d];@11@D40<C4a04Y:BP1oOgl0Kfm_02<S8`0m?Cd0Cdm?05UIF@2PX:00
KVi^01lO7`0l?3`0Fe]K05MGE`1fMWH0;bl_02XZ:P18B4P0E5AD06EUI@2KVi/0=cLg0410@01QHF40
M7Ad05QHF02c/k<0=3@d04m?C`28R8P0FEUI05iNGP2OWil0>c/k04e=C@2HV9P0A4A4061PH01dM7@0
=3@d04Y:BP2YZJT0A4A4051@D01QHF409RHV04=3@`1[Jf/0>SXj059BDP1FEUH0:RXZ04=3@`1lO7`0
EEEE051@D00W9bL092@T04m?C`2ATI40CDe=05YJFP0f=SH0JFUY08^;R`2<S8`0>c/k04A4A00Y:BT0
H61P0;>c/`0g=cL0:BTY05]KF`1:BTX0PH610;:b/P0V9RH0:RXZ06UYJ@1JFUX0[Zj^03Xj>P0f=SH0
@T9206YZJP17AdL0MgMg02DU9@1;Bd/0Hf=S05UIF@1gMgL0@41005IFEP1nOWh0CDe=06=SH`1bLW80
7alO05]KF`1mOGd0AdM706m_K`0h>3P0>c/k06ATI01KFe/0EeMG02XZ:P0b<S80HV9R09ZJVP80DE5A
02D0<C4a051@D01JFUX0Cdm?02l_;`1NGUh0>c/k08J6QP34a<@0G5aL02l_;`1ADE40=cLg09bLW02A
TI40:b/[01lO7`0c<c<0DE5A08N7Q`2JVYX0;Bd]03@d=01VIVH0GUiN05YJFP2PX:00;Rh^02d];@1c
Lg<0PH6106a/K01kNg/08R8R02@T9023Ph<0Q8B40080ATI60100@T9204=3@`1EEED0EEEE03`l?01Z
JVX0Cdm?04=3@`1JFUX0DE5A03Hf=P13@d<0?Cdm04E5A@1]KFd0@d=33_ooool0g018B4P0E5AD03`l
?01/K6`0BDU904=3@`0];Bd0CTi>07=cL`10@400CDe=07QhN02;Rh/0De=C07emO@1EEED0HF5Q03Hf
=P0e=CD0:2PX04Y:BP0^;Rh0=3@d03lo?`0^;Rh0?Shn03Ph>01MGEd0O7al04M7A`19BDT0GemO04M7
A`10@400BDU905YJFP1RHV80;2`/038b<P10@400=CDe0492@P13@d<0@D5102DU9@0j>SX0De=C0410
@00Y:BT0AdM7051@D01:BTX0A4A404Y:BP1LG5`0Bd];04m?C`18B4P0Cdm?04e=C@1HF5P0ADE5055A
D@18B4P0Vi^K05]KF`0e=CD0ADE506ATI01iNGT0Cdm?04E5A@18B4P0<C4a038b<P19BDT0?3`l06][
J`1]KFd0C4a<05=CD`1:BTX0ADE504U9B@17AdL0?Cdm0410@01RHV80=3@d05=CD`17AdL0H61P03De
=@17AdL0HF5Q05QHF01>CTh0Hf=S03dm?@1WIfL0IFEU03Xj>P1NGUh0Kfm_03<c<`1RHV80GEeM09:B
TP1XJ6P0IVIV03Ph>01VIVH0KVi^03/k>`1BDU80ATI601dM7@12@T80:RXZ05aLG019BDT0F5QH0451
@@1oOgl09bLW06ATI011@D40ATI6079bLP1DE5@0YjNW065QH@2<S8`0PX:203dm?@1KFe/0N7Qh04Q8
B01ADE40<30`0451@@1VIVH0GEeM09nOW`1:BTX0Jf][06i^KP1dM7@0GUiN04i>CP19BDT0<c<c05MG
E`19BDT0;bl_09JFUP0m?Cd0B4Q802PX:01HF5P0:b/[04A4A01?Cdl0LW9b061PH00M7Ad0@d=304a<
C01KFe/0>c/k06MWI`0/;2`0B4Q805ADE00O7al0=SHf06MWI`0a<C40:RXZ0410@00T92@0<30`03<c
<`0`<300D51@03Ph>022PX80C4a<04];B`14A4@0AdM703<c<`0d=3@0@D5104E5A@1GEeL0Ogmo03Xj
>P1aLG40Fe]K04i>CP1^KVh0;Rh^04Q8B01CDe<0HF5Q06][J`1;Bd/0IfMW06EUI@1LG5`0Hf=S05aL
G01BDU80CTi>04m?C`16ATH0EUIF0451@@1XJ6P205YJFP0@02@T901fMWH0;Rh^03Ti>@1CDe<0:RXZ
05EEE@1CDe<0ATI606EUI@1dM7@0DE5A04a<C026QXH0BDU903Lg=`80>CTi02H0EUIF04Q8B01?Cdl0
HV9R03dm?@1IFET0;2`/05QHF01JFUX0H61P096AT@1GEeL0AdM706=SH`1PH600@d=308n?S`23Ph<0
ATI605eMG@1>CTh0EEEE03hn?P16ATH0>c/k059BDP1@D500?Shn03Lg=`1hN7P0HV9R05IFEP1nOWh0
F5QH05]KF`1[Jf/0I6AT065QHA3ooooo000>ooooo`2908R8R01@D500LW9b07inOP1ADE40@d=3079b
LP1QHF40FUYJ09fMW@0];Bd0LG5a07emO@1NGUh0V9RH05IFEP11@D40KFe]06a/K018B4P0Q8B402h^
;P0b<S80Kfm_059BDP1gMgL0CTi>02<S8`1QHF40Q8B402`/;01bLW80ADE503De=@12@T80HF5Q02h^
;P17AdL0;2`/06EUI@2IVIT0KVi^02XZ:P1BDU80=CDe07QhN03Fe]H0@T9202/[:`1QHF40@T92096A
T@28R8P04Q8B03Ti>@1kNg/0Hf=S0861P@1CDe<0=SHf059BDP1OGel0PH6109bLW00_;bl0<c<c04M7
A`24Q8@0E5AD06][J`0K6a/0>SXj07inOP27QhL0EeMG05aLG00];Bd0<C4a05=CD`1<C4`092@T04A4
A00[:b/0G5aL08b<S011@D409BDU05EEE@0K6a/0Bd];0:RXZ00Y:BT0=CDe04U9B@14A4@0KFe]065Q
H@0I6AT0;bl_04=3@`1GEeL0Lg=c06EUI@0V9RH0EeMG04i>CP0f=SH0PH6107IfMP0X:2P0Ph>309VI
V@0g=cL0M7Ad07QhN00c<c<0GemO071`L00h>3P0HV9R05=CD`0I6AT0Fe]K05iNGP0e=CD0NGUi03lo
?`0P8200DU9B06ATI00a<C40E5AD024Q8@0];Bd0C4a<03Ti>@18B4P00P16ATH0R`13@d<0S8b<03Ph
>00/;2`0AdM705IFEP1_Kfl0Ogmo02<S8`0j>SX0D51@06EUI@1VIVH0S8b<03Hf=P0W9bL0FEUI05IF
EP27QhL0SXj>01DE5@0f=SH0HV9R05UIF@2JVYX0<C4a01<C4`0m?Cd0Ph>305iNGP1cLg<0<S8b02HV
9P1CDe<0RXZ:04Y:BP1fMWH0<30`024Q8@1bLW80Q8B403hn?P2CTi<0>SXj04A4A01_Kfl0K6a/06IV
IP1mOGd0=SHf08b<S021PH40<30`06QXJ015ADD0>3Ph08Z:RP1/K6`0Fe]K04i>CP0Y:BT0HV9R07]k
N`0n?Sh0C4a<03Ti>@26QXH0YJFU03hn?P15ADD0EeMG06ATI02][Jd0@D5103Hf=P1NGUh0IVIV0;Jf
]P1OGel07QhN0492@P1/K6`0M7Ad09^KV`0e=CD0Bd];05eMG@1QHF40I6AT09bLW00b<S80?Shn05aL
G01eMGD0Cdm?07IfMP0S8b<0CDe=07moO`1NGUh0EEEE06m_K`0h>3P0?clo08n?S`1VIVH0=SHf06=S
H`19BDT0@41009bLW022PX80?Shn05=CD`0W9bL0Cdm?06QXJ00d=3@0>SXj03lo?`0h>3P0A4A407=c
L`0S8b<0=cLg05eMG@19BDT0A4A408>3P`1cLg<0>SXj06UYJ@1FEUH0C4a<0861P@1KFe/09BDU05AD
E011@D403_ooool0``1UIFD0PX:206e]K@1BDU80I6AT08F5Q@1/K6`0G5aL07]kN`1UIFD0KFe]05=C
D`1EEED0CDe=092@T01jNWX0EeMG04I6AP1SHf<0ATI604U9B@0o?cl0HF5Q04A4A00b<S80Ph>304U9
B@1MGEd0IVIV05]KF`1EEED0JFUY05QHF01[Jf/0O7al061PH01=CDd0;Rh^028R8P0Z:RX0?clo04E5
A@0e=CD0=cLg03Ph>00k>c/0@D5103lo?`0j>SX0D51@02l_;`0k>c/0BDU904I6AP1:BTX0?3`l03Ph
>00];Bd0ATI604Q8B017AdL0;2`/04Y:BP16ATH0IVIV04e=C@18B4P0F5QH05aLG00o?cl0D51@0492
@P1gMgL0:RXZ02DU9@1eMGD0@D5104];B`0l?3`0@41003dm?@0a<C40=CDe03`l?00T92@0:2PX02l_
;`18B4P0>3Ph03dm?@10@400@D51030`<00c<c<0A4A404E5A@13@d<0AdM705QHF019BDT0E5AD051@
D01WIfL0FUYJ04i>CP27QhL0Jf][07MgM`1WIfL0OGem03Xj>P1KFe/0K6a/0492@P1HF5P0DU9B02@T
901<C4`0:BTY05=CD`0k>c/0HV9R04];B`24Q8@0:RXZ08F5Q@1CDe<0IFEU07EeM@1<C4`0_Kfm08F5
Q@2;Rh/0NGUi03De=@1FEUH0Lg=c03Ti>@1>CTh0@T9205MGE`1RHV80Ng]k0<G5a@0g=cL0FEUI04I6
AP1OGel0E5AD06IVIP1RHV80D51@05YJFP1DE5@06a/K05iNGP0k>c/0O7al030`<01BDU80<C4a04Q8
B012@T80?clo04M7A`0^;Rh0GUiN05eMG@1BDU80>c/k08:2PP1:BTX0;bl_0451@@0n?Sh0>c/k09:B
TP17AdL0DE5A04];B`0j>SX0AdM704Y:BP1BDU80EeMG04M7A`25QHD0MWIf04Y:BP0o?cl0>CTi02l_
;`0g=cL0>c/k06QXJ00205=CD`0C02XZ:P2GUiL0CDe=02d];@1JFUX09RHV04U9B@1CDe<0:b/[03Ti
>@11@D40Bd];0492@P1HF5P0B4Q805aLG00l?3`0?clo05MGE`0204I6AP0l03Lg=`1VIVH0Jf][04E5
A@0Q8B40=cLg055AD@1FEUH0GemO03@d=01;Bd/0Ng]k05IFEP1:BTX0@D510410@01<C4`0ATI603`l
?00e=CD0C4a<03Ph>01`L700HV9R07MgM`2i^KT0G5aL07inOP1<C4`0JVYZ06MWI`17AdL0HF5Q03Xj
>P1/K6`0I6AT069RHP15ADD0EeMG0:2PX01NGUh0SXj>05ADE012@T80@D5104Y:BP1BDU80ATI604=3
@`1<C4`0BTY:05MGE`0k>c/0Lg=c07EeM@1>CTh0GUiN05eMG@2CTi<0NWYj4?ooool000kooooo0?@0
KVi^075aL@1lO7`0Hf=S06YZJP1PH600TI6A04A4A00n?Sh0VYZJ04a<C018B4P0@D5102h^;P1kNg/0
Q8B403Hf=P18B4P0DE5A0820P02RXZ80:RXZ028R8P19BDT0D51@0<W9b@1oOgl05aLG01`L701@D500
CTi>0<g=c@2=SHd08B4Q06ATI01PH600CTi>09BDU00g=cL06QXJ061PH01IFET0>c/k06IVIP0K6a/0
=cLg08f=S@1mOGd0Bd];05QHF00J6QX0>c/k08^;R`1GEeL0Cdm?06e]K@0S8b<0E5AD06UYJ@0Z:RX0
GUiN03Xj>P0i>CT0[:b/05QHF00h>3P0KVi^03Xj>P0O7al0KVi^01lO7`0/;2`0EEEE0410@016ATH0
Hf=S04=3@`0g=cL0FEUI055AD@1GEeL0UYJF03Ph>01=CDd0HV9R05YJFP1ZJVX0^[Zj02DU9@0b<S80
PH6105aLG01iNGT0Lg=c01<C4`0l?3`0K6a/03/k>`1]KFd0O7al02LW9`0_;bl0EUIF038b<P1DE5@0
=CDe028R8P1DE5@0LW9b02h^;P19BDT0F5QH03Lg=`24Q8@0QhN702@T901DE5@0<c<c055AD@1PH600
O7al02`/;017AdL0K6a/04e=C@25QHD0ADE501lO7`0/;2`0GUiN061PH02@T9006a/K03<c<`13@d<0
IVIV07emO@1jNWX0:b/[05YJFP1GEeL0R8R807AdM01EEED0:b/[05YJFP1IFET0CTi>069RHP0/;2`0
;bl_07EeM@2e]KD0SHf=03Ph>016ATH0>c/k06UYJ@2_[jl0DE5A069RHP1FEUH0>CTi07=cL`2<S8`0
;Bd]04e=C@1ADE40@41006QXJ01ADE40<c<c04m?C`11@D40?3`l07MgM`0h>3P09RHV03Ph>00o?cl0
HV9R08B4Q011@D40ADE504e=C@14A4@0Lg=c0;no_`0b<S80HV9R07IfMP1>CTh0KFe]03Ph>00P8200
KFe]06IVIP1fMWH0Jf][01@D500g=cL0Hf=S05mOG`17AdL0:RXZ034a<@1/K6`0[Zj^08^;R`0W9bL0
>3Ph069RHP1NGUh0CTi>06YZJP0_;bl0BDU906ATI010@400EeMG04E5A@0j>SX0O7al05UIF@14A4@0
M7Ad01dM7@0h>3P0N7Qh03/k>`1;Bd/0EeMG03De=@1SHf<0XJ6Q04E5A@1CDe<0IVIV04I6AP1FEUH0
N7Qh065QH@0U9BD0EeMG0P1YJFT08024Q8@0JFUY030`<01YJFT0Ogmo08f=S@2f][H0<30`03hn?P1=
CDd0BDU90861P@2g]kL0>CTi03Xj>P1JFUX0Bd];0820P02ATI407QhN02XZ:P12@T80FEUI06i^KP2:
RXX0:BTY02d];@10@400E5AD02l_;`0f=SH07alO3_ooool05P15ADD0KVi^05iNGP1CDe<0@41005=C
D`0];Bd0?3`l07UiN@0_;bl0DU9B04m?C`1LG5`0GEeM051@D011@D40BTY:03/k>`15ADD0?clo059B
DP1FEUH204=3@`0;04E5A@1;Bd/0>3Ph061PH01LG5`0CDe=065QH@1UIFD0Jf][04Q8B01IFET00P1S
Hf<0l@1<C4`0:RXZ04U9B@12@T80DE5A04A4A01FEUH0=3@d051@D014A4@0FEUI01`L701=CDd0?Cdm
05MGE`0Y:BT08b<S04I6AP0g=cL0?Cdm03Ph>00n?Sh0AdM704=3@`1=CDd0A4A404];B`31`L40MWIf
02`/;00a<C40IFEU06a/K017AdL0ADE505iNGP16ATH0CTi>06MWI`0n?Sh0EUIF051@D01:BTX0ATI6
03dm?@12@T80@D5103Ph>00c<c<0;2`/04];B`0d=3@0ATI604a<C01FEUH09BDU03`l?015ADD0?Cdm
0410@00k>c/0A4A40492@P1CDe<0=3@d04I6AP1HF5P0;Rh^04Y:BP1GEeL0PH6106=SH`1ADE40D51@
05mOG`1HF5P0JFUY05EEE@18B4P08b<S05aLG00V9RH0EeMG04Q8B02KVi/0EEEE07moO`0V9RH0VIVI
05QHF01MGEd0Ti>C04];B`1^KVh0QhN70;^k^`1QHF40CDe=06UYJ@20P800<30`04M7A`0Y:BT0B4Q8
03Xj>P1PH600[Zj^06IVIP20P800AdM7079bLP1:BTX0IFEU05ADE01<C4`0MgMg06m_K`12@T80PX:2
04E5A@17AdL0@T9205IFEP0k>c/0=CDe04A4A01BDU80JFUY04Q8B00h>3P0C4a<05YJFP15ADD0EUIF
04Q8B00j>SX0AdM706ATI01DE5@0GUiN03Hf=P1JFUX0Cdm?02@T900_;bl0@D51055AD@19BDT0;Rh^
075aL@1NGUh0H61P0410@012@T80<30`04E5A@0f=SH0;2`/02h^;P1;Bd/0<C4a09FEU@1HF5P0>SXj
069RHP0R8R80FEUI04m?C`0d=3@0FEUI03Ph>01RHV80AdM704e=C@13@d<0HV9R03Hf=P0h>3P0=CDe
065QH@1FEUH0ADE504Y:BP1CDe<0IFEU03De=@1?Cdl0?clo05QHF01GEeL0:BTY04Y:BP14A4@0<c<c
03Xj>P1ADE40@D5103/k>`1NGUh0EEEE07=cL`1VIVH0HF5Q0451@@0c<c<0Kfm_096AT@1WIfL0GEeM
05=CD`2XZ:P0S8b<0492@P1ZJVX0;Rh^06e]K@1NGUh0U9BD04Y:BP1eMGD0QHF50451@@24Q8@0F5QH
04i>CP0^;Rh0@410051@D01?Cdl07AdM02`/;00h>3P0IfMW03Ti>@1VIVH0MGEe06ATI01JFUX0H61P
05aLG01cLg<04?ooool000kooooo04D0SXj>0:VYZ@2k^k/0DE5A06][J`26QXH0f=SH05mOG`0k>c/0
K6a/07IfMP2][Jd0AdM702TY:@0W9bL0De=C05iNGP1`L700:2PX03`l?01gMgL0H61P05iNGP1IFET0
=cLg04Y:BP1WIfL0=CDe05]KF`0d=3@0>CTi05YJFP2JVYX0=CDe05MGE`1IFET08b<S07UiN@1]KFd0
:2PX03Xj>P0_;bl0CTi>08N7Q`0c<c<0<C4a0492@P0g=cL0EeMG08F5Q@0I6AT0:RXZ04m?C`1GEeL0
KVi^0:RXZ00P8200=3@d04U9B@1YJFT0/K6a07IfMP0H61P0G5aL06i^KP1BDU80alO705QHF00J6QX0
0P1:BTX0901JFUX0XJ6Q02PX:00^;Rh0DE5A07QhN01YJFT0FUYJ01PH600i>CT0HV9R065QH@0f=SH0
A4A4028R8P0g=cL0GEeM05aLG00c<c<0;Bd]02TY:@1<C4`0Ogmo02DU9@0`<300=cLg04M7A`1iNGT0
MWIf01PH600f=SH0CTi>04];B`2IVIT0?Shn02@T9080CDe=0340GUiN09RHV00d=3@0;2`/0451@@1a
LG40F5QH0::RXP1GEeL09BDU03dm?@1MGEd0EEEE08V9R@0d=3@0<30`04A4A01KFe/0Hf=S09bLW00a
<C40=cLg06UYJ@1QHF40:b/[04Q8B00U9BD0Hf=S07alO00f=SH0=CDe05IFEP0S8b<0CTi>06QXJ00S
8b<0?Cdm04i>CP0o?cl0Kfm_07AdM00P8200;2`/06][J`1cLg<0Ogmo06QXJ00L71`0AdM70080?clo
02L0QhN70:b/[00];Bd0ATI604e=C@11@D40Xj>S08N7Q`0/;2`0C4a<06][J`1]KFd0[Jf]07emO@0O
7al0EeMG075aL@1GEeL0T92@01lO7`0l?3`0L71`05eMG@1;Bd/0FEUI024Q8@1GEeL0K6a/05IFEP2C
Ti<0@d=304];B`2BTY80U9BD04i>CP1=CDd08b<S06UYJ@2EUID00P1DE5@0:`10@400EEEE07YjNP0U
9BD0>CTi0451@@1@D500WYjN04E5A@0b<S80CTi>0410@01IFET0SHf=03@d=01?Cdl0EUIF04e=C@28
R8P0PX:203De=@1cLg<0GEeM06=SH`2b/[80=SHf034a<@2>SXh0E5AD09VIV@2NWYh092@T03hn?P1X
J6P0ATI607]kN`1lO7`07QhN04a<C02@T900R8R806a/K02k^k/00P1=CDd05P1SHf<0Ti>C06YZJP2E
UID0BDU905mOG`2e]KD0SXj>05iNGP0m?Cd0?Shn03Xj>P1UIFD0GemO02XZ:P12@T8092@T04i>CP2A
TI40Fe]K01dM7@0o?cl204=3@`08079bLP1IFET0:RXZ04I6AP1DE5@0Kfm_069RHP0U9BD>ooooo`0h
04Q8B01eMGD0>3Ph04m?C`0o?cl0OGem03Xj>P1;Bd/0Ph>30410@01?Cdl0C4a<04e=C@19BDT0M7Ad
05EEE@1/K6`0<C4a03/k>`0M7Ad0Cdm?038b<P1BDU80FEUI03hn?P1EEED0ADE5079bLP2<S8`0EUIF
05QHF01/K6`0IFEU04e=C@1DE5@0FUYJ071`L01;Bd/0ADE5055AD@17AdL0EUIF05MGE`1UIFD0BDU9
04a<C01=CDd0E5AD02h^;P1ADE40EEEE05YJFP17AdL0BTY:07=cL`0g=cL204m?C`0k05UIF@11@D40
FUYJ055AD@1>CTh0D51@05UIF@1GEeL0@d=303`l?01DE5@0?3`l02TY:@0^;Rh0Cdm?02l_;`0`<300
CDe=03Xj>P1FEUH0EEEE03hn?P0i>CT0@T9203@d=00l?3`0=cLg03@d=00a<C40EEEE024Q8@0Y:BT0
>3Ph04E5A@0R8R80;Rh^0451@@0h>3P0<30`03Ph>01>CTh0?clo04=3@`1IFET0F5QH03Ph>018B4P0
I6AT04a<C012@T80C4a<07EeM@1=CDd0C4a<06QXJ01GEeL0@d=304E5A@0N7Qh00P10@400W`1:BTX0
<S8b04a<C01PH600Ng]k034a<@1=CDd0?Cdm05QHF01<C4`0Hf=S0:ZZZP1LG5`0CTi>051@D00j>SX0
EeMG07emO@19BDT0De=C030`<01MGEd0F5QH07IfMP2RXZ80NWYj04Q8B01NGUh0NGUi05iNGP1]KFd0
D51@01dM7@17AdL0GUiN02XZ:P1mOGd0Kfm_05]KF`1:BTX0EeMG05UIF@1QHF40GEeM04a<C01fMWH0
CTi>05aLG01^KVh0LG5a05=CD`1[Jf/0BDU903`l?01DE5@0@d=303hn?P1/K6`09RHV069RHP1DE5@0
=3@d03Ti>@19BDT0De=C06a/K01<C4`0QHF506QXJ01IFET0FUYJ04I6AP0e=CD0>CTi03lo?`18B4P0
@T9205QHF017AdL0I6AT06i^KP1@D500Jf][01LG5`0d=3@0=SHf03<c<`1>CTh0>CTi05MGE`1DE5@0
C4a<03lo?`18B4P0=CDe03Hf=P17AdL0>SXj0492@P0j>SX0MWIf05QHF00i>CT06QXJ06QXJ00g=cL0
A4A405IFEP0O7al0@d=304M7A`1GEeL0H61P05eMG@16ATH0D51@09BDU01EEED0?Cdm03Lg=`0e=CD0
Ogmo06YZJP2=SHd0Ti>C059BDP1IFET0=3@d05ADE01<C4`0GUiN08Z:RP14A4@0Lg=c04E5A@1SHf<0
GEeM08:2PP21PH40CTi>05MGE`1:BTX0De=C04I6AP1>CTh0Cdm?04];B`16ATH0?clo04];B`24Q8@0
?Shn051@D02?Shl0BTY:04a<C013@d<0IFEU05=CD`0@ooooo`003_ooool0:@1RHV80IfMW09nOW`1/
K6`0DE5A04I6AP1bLW80SXj>05=CD`1UIFD0:b/[05eMG@2>SXh0?3`l065QH@0l?3`0@D5106i^KP0U
9BD0@d=304Q8B015ADD0EEEE06QXJ00N7Qh0H61P065QH@1:BTX0RXZ:03`l?00K6a/0K6a/08J6QP1?
Cdl0[:b/08n?S`0T92@0HV9R07AdM01<C4`0Vi^K0080;Rh^03L0HV9R03<c<`1aLG40LG5a03Ti>@0`
<300L71`05YJFP1`L700>c/k01hN7P10@400B4Q804I6AP1`L700<c<c030`<02EUID0/K6a04i>CP1A
DE40;Bd]02/[:`22PX80Ph>3034a<@1QHF409BDU04Q8B02OWil0F5QH02HV9P0c<c<0>c/k05UIF@2>
SXh0>3Ph01LG5`0c<c<0?Shn08Z:RP2BTY804Q8B03<c<`0g=cL0FUYJ07alO01MGEd061PH05IFEP1N
GUh0BDU908J6QP0m?Cd051@D0080C4a<00X0?clo04Q8B01>CTh0>SXj05]KF`1NGUh0:b/[069RHP1Y
JFT0:b/[0P1LG5`0=00`<300NWYj07UiN@0S8b<0Kfm_061PH00Y:BT0?3`l02<S8`0b<S80EEEE04e=
C@0f=SH0C4a<03Ti>@11@D40MGEe03hn?P0f=SH0CTi>06YZJP1KFe/0OWin02@T900/;2`0EeMG06IV
IP1nOWh0TI6A01XJ6P18B4P0TI6A04Y:BP1EEED0HV9R01/K6`16ATH0O7al04E5A@1]KFd0Rh^;02@T
900U9BD0Kfm_02`/;01YJFT0E5AD024Q8@0X:2P0L71`05ADE01NGUh2020P801`03Lg=`2>SXh0EEEE
05iNGP0S8b<0;2`/03Hf=P1QHF40=SHf05=CD`0];Bd0Ogmo0:^[Z`1ADE40;Bd]05YJFP1<C4`0W9bL
0:RXZ00U9BD0>3Ph04A4A012@T80VYZJ061PH00N7Qh0CTi>069RHP1CDe<0R8R801DE5@18B4P0LW9b
04m?C`1dM7@0CDe=04];B`1JFUX0ADE5061PH01YJFT05QHF04];B`1gMgL0IFEU0:6QX@0];Bd0?Cdm
0:FUY@1UIFD0IfMW02/[:`0_;bl0J6QX09ZJVP1DE5@0CTi>02XZ:P1:BTX0MgMg065QH@0Z:RX0ADE5
04I6AP1TI6@0SHf=04M7A`0Z:RX0HF5Q0451@@1[Jf/0Q8B405]KF`0Y:BT0EeMG055AD@0n?Sh0Q8B4
04Y:BP0U9BD0BDU904=3@`1=CDd0Xj>S05ADE00`<300A4A404I6AP1^KVh0J6QX02DU9@14A4@0AdM7
0492@P14A4@0HF5Q01@D5011@D40O7al08V9R@13@d<0Cdm?03hn?P1OGel0LG5a06QXJ01PH600SHf=
059BDP1NGUh0QhN7051@D0kooooo04`0@D5105YJFP18B4P0IVIV04];B`1fMWH0FEUI061PH02KVi/0
Bd];055AD@1bLW80UIFE06IVIP1TI6@0@D5104m?C`15ADD0@D5103<c<`1IFET0;Rh^04Y:BP1<C4`0
7QhN04a<C00M7Ad0De=C04];B`19BDT0Lg=c08:2PP2<S8`0C4a<05YJFP1jNWX0LG5a02HV9P0b<S80
ATI603`l?01ADE40B4Q8055AD@0m?Cd0EUIF04i>CP1[Jf/0@T9205YJFP0o?cl0BTY:03Ph>00a<C40
JFUY05]KF`1CDe<0CDe=05eMG@1ADE40JFUY05MGE`1QHF40Fe]K0861P@1kNg/0FEUI05aLG01/K6`0
FUYJ03dm?@0j>SX0TI6A03/k>`0^;Rh0QHF50P1GEeL05P19BDT0DU9B059BDP13@d<0Bd];04m?C`0c
<c<0@T9205YJFP1EEED0?Cdm04];B`1GEeL0DE5A03@d=00l?3`0HF5Q04i>CP0b<S80ATI604M7A`1>
CTh20492@P2`04M7A`1<C4`0=CDe03/k>`1XJ6P0CTi>05UIF@1GEeL0CTi>065QH@1BDU80<S8b03`l
?00`<300;2`/055AD@0M7Ad0@d=304a<C02QXJ40B4Q806][J`0e=CD0MgMg04Y:BP1?Cdl0FEUI03Ph
>01OGel0KFe]06IVIP1=CDd0Cdm?06ATI01IFET0D51@055AD@0_;bl0<30`03De=@1CDe<0/;2`05iN
GP1iNGT0DE5A05IFEP1CDe<0TI6A05eMG@0`<300Jf][06=SH`0B4Q80Y:BT04U9B@1TI6@0D51@071`
L01>CTh0Jf][05MGE`1eMGD0JFUY02HV9P1LG5`0Cdm?04U9B@0_;bl0Hf=S04];B`1IFET0IFEU04a<
C01@D500Lg=c04a<C01hN7P0LW9b03@d=00k>c/0A4A406i^KP2<S8`0DU9B09:BTP1cLg<0FEUI04m?
C`13@d<0?Cdm04Q8B01?Cdl0B4Q804a<C01NGUh0;Rh^08:2PP1PH600>c/k07YjNP0i>CT0@41003dm
?@10@400E5AD05IFEP1ZJVX0C4a<04m?C`18B4P0CTi>03<c<`0f=SH0?3`l04M7A`18B4P0>3Ph04e=
C@1[Jf/0EeMG03De=@1?Cdl0EUIF07UiN@1fMWH0Cdm?05aLG01TI6@0JVYZ05mOG`1EEED0?clo051@
D00j>SX08b<S05eMG@1fMWH0?3`l07AdM01MGEd0K6a/0<71`@1CDe<0IfMW03@d=01aLG40N7Qh06EU
I@2NWYh0P8200;2`/01PH600PX:204i>CP1lO7`0TI6A0492@P1XJ6P0Cdm?06][J`11@D40DE5A03/k
>`1oOgl0?Cdm055AD@0a<C40IVIV0451@@19BDT0EeMG04U9B@1JFUX0EEEE06UYJ@1RHV8@ooooo`00
3_ooool0V02<S8`0JFUY09RHV01aLG40Ng]k04Y:BP1SHf<0De=C05]KF`2=SHd0?Cdm05EEE@1WIfL0
CTi>06m_K`1XJ6P0>CTi05YJFP0o?cl0KVi^092@T00W9bL0<S8b05EEE@12@T80TY:B075aL@0P8200
?clo06UYJ@19BDT0Ti>C05MGE`0V9RH0A4A407AdM01DE5@0Q8B4034a<@0/;2`0CDe=05]KF`0n?Sh0
Jf][02XZ:P1BDU80Jf][04i>CP1;Bd/0GEeM04A4A020P800Ng]k03dm?@19BDT0EeMG04M7A`2CTi<0
E5AD03De=@1UIFD0FEUI04M7A`2KVi/0C4a<02TY:@1HF5P0B4Q8030`<025QHD0>3Ph030`<01ADE40
Cdm?0410@01NGUh0A4A403/k>`14A4@0J6QX03lo?`29RHT0ADE503<c<`1KFe/0?Cdm06=SH`1gMgL0
8B4Q03Ph>01RHV80=SHf06IVIP1LG5`08B4Q051@D01cLg<0?Cdm06UYJ@16ATH0:b/[04A4A01aLG40
@410071`L00m?Cd0:b/[05eMG@1cLg<0?clo06MWI`0m?Cd0?clo06UYJ@2ATI40B4Q805]KF`1?Cdl0
A4A407emO@1BDU80:2PX04i>CP0j>SX0EeMG08Z:RP0j>SX0:b/[05MGE`1DE5@0HF5Q08^;R`0[:b/0
;Bd]05QHF01@D500L71`03lo?`0T92@0@D5106][J`16ATH0Fe]K05=CD`0e=CD0BTY:06EUI@0l?3`0
MWIf04i>CP0b<S80AdM70P1RHV80O01<C4`0HF5Q03Lg=`1fMWH0K6a/02`/;016ATH0FUYJ03hn?P2<
S8`0I6AT02`/;01HF5P0BTY:04a<C02/[:`0KFe]03lo?`1PH600@d=303De=@1jNWX0>SXj03`l?01O
Gel0?clo03Hf=P1lO7`0CDe=04m?C`1PH600?Cdm07IfMP20P800<C4a05UIF@1HF5P0B4Q8079bLP0f
=SH09bLW06ATI01;Bd/0OGem04e=C@0R8R80C4a<069RHP1NGUh0C4a<038b<P1cLg<0WIfM06][J`1h
N7P0?3`l05]KF`2=SHd0De=C03Ti>@1mOGd0CTi>07QhN01?Cdl0<c<c07MgM`1JFUX0HV9R07inOP13
@d<0?Cdm06EUI@1RHV80J6QX08:2PP0g=cL0FUYJ079bLP0n?Sh0AdM708^;R`0f=SH0GUiN05YJFP1M
GEd0C4a<096AT@1KFe/0DE5A05MGE`1DE5@0EEEE0:JVYP1LG5`0?Shn04i>CP1CDe<0Ph>309RHV013
@d<0>c/k06e]K@1QHF40NWYj06][J`0a<C40?Shn06EUI@0m?Cd0JFUY07]kN`0_;bl0@41007IfMP1F
EUH0C4a<06e]K@0d=3@0:2PX05iNGP1IFET0EUIF06MWI`0X:2P>ooooo`3K04e=C@1IFET0>3Ph05YJ
FP1<C4`0EUIF03@d=011@D40LW9b04A4A01;Bd/0NGUi0820P01ADE40J6QX04A4A01JFUX0?3`l04U9
B@0f=SH0FEUI03Lg=`1@D500E5AD02l_;`1:BTX06a/K05mOG`1VIVH0E5AD06EUI@2@T900HF5Q05=C
D`1JFUX0KFe]075aL@11@D40ATI6055AD@11@D40F5QH05EEE@1MGEd0@41005=CD`1KFe/0HF5Q03<c
<`1GEeL0B4Q805UIF@0j>SX0<S8b05aLG01>CTh0BDU903lo?`1OGel0HV9R05EEE@10@400FUYJ05]K
F`1[Jf/0Ti>C05QHF01:BTX0Fe]K05QHF01DE5@0CTi>07emO@0g=cL0>CTi075aL@15ADD0IVIV051@
D01DE5@0EeMG0492@P1:BTX0Bd];03<c<`14A4@0ATI605QHF00d=3@0D51@04a<C01=CDd0<30`03Lg
=`1IFET0BTY:04];B`1IFET0B4Q805YJFP1YJFT0EeMG05iNGP1OGel0DU9B061PH01`L700Fe]K06EU
I@1mOGd0FUYJ04m?C`1ZJVX0AdM705ADE00n?Sh08R8R04i>CP0U9BD0EeMG03Xj>P1EEED0Fe]K07em
O@17AdL0L71`05EEE@1OGel0K6a/061PH028R8P0Ogmo06m_K`1WIfL0>SXj069RHP1cLg<0>c/k055A
D@0];Bd0C4a<04=3@`1VIVH0/;2`08j>SP1hN7P0JVYZ05=CD`1MGEd0JFUY05mOG`0e=CD0Hf=S05YJ
FP0Y:BT0Kfm_04E5A@1UIFD0@D5107EeM@0i>CT0F5QH04A4A01?Cdl0Bd];02d];@1BDU80CTi>06EU
I@1;Bd/0FEUI03dm?@0l?3`0Bd];02l_;`1DE5@0L71`01LG5`1OGel0DU9B028R8P11@D40ATI6059B
DP1ZJVX0@410092@T01dM7@0HV9R0451@@1QHF40@41003hn?P0l?3`0EeMG05]KF`1BDU80?3`l09RH
V01_Kfl0?Shn06QXJ00Y:BT0EeMG05aLG00l?3`0EUIF04=3@`1YJFT0CDe=069RHP16ATH0De=C03hn
?P12@T80?Shn05ADE01KFe/0=cLg0080IFEU03T0IVIV04M7A`1PH600C4a<06QXJ01^KVh09RHV051@
D01MGEd0@d=303hn?P1NGUh0=CDe03/k>`1XJ6P0;2`/03Lg=`16ATH0<c<c05eMG@2g]kL0c<c<0;Ng
]`1kNg/0IfMW09jNWP21PH40LG5a05aLG01_Kfl0F5QH07AdM011@D40O7al04E5A@1kNg/0IFEU04M7
A`1iNGT0?Shn055AD@0h>3P0BTY:04a<C01QHF40A4A404m?C`15ADD0QhN703hn?P13@d<0NGUi051@
D01>CTh0FEUI06UYJ@1WIfL04?ooool000kooooo0P1VIVH0>02d];@0DU9B06EUI@1kNg/0U9BD04I6
AP0n?Sh0CDe=06EUI@2a/K40ATI602h^;P1?Cdl0MWIf07emO@1QHF40;Rh^03`l?01ZJVX0FEUI07in
OP1JFUX0:RXZ04=3@`1]KFd0Lg=c09JFUP0m?Cd0;Bd]04M7A`1jNWX0>c/k07emO@1QHF408B4Q05IF
EP1WIfL0=SHf05QHF00U9BD0?Shn07=cL`1FEUH0LW9b05EEE@0L71`0A4A406m_K`0b<S80MGEe02l_
;`0U9BD0I6AT08B4Q00V9RH0De=C0P1;Bd/0eP2HV9P0E5AD01/K6`15ADD0HV9R06ATI02^[Zh0DU9B
01<C4`1=CDd0KVi^06=SH`1kNg/06QXJ01dM7@0W9bL0D51@05=CD`1^KVh07QhN038b<P1LG5`0GemO
06=SH`1HF5P06ATI01lO7`1UIFD0HV9R05aLG00/;2`09bLW04Y:BP1jNWX0?Shn04];B`0X:2P08B4Q
06EUI@26QXH0;Rh^03<c<`10@400>CTi08F5Q@1?Cdl0:BTY04e=C@1QHF40>SXj09:BTP1:BTX05ADE
03Lg=`1?Cdl0C4a<08:2PP14A4@09RHV04Y:BP0l?3`0@T9206e]K@0U9BD08R8R05UIF@1fMWH0?3`l
06ATI00];Bd0De=C0:b/[024Q8@0C4a<05MGE`0Y:BT0R8R80;Zj^P0n?Sh0F5QH079bLP0j>SX0BTY:
09ZJVP15ADD0De=C03lo?`0P8200>SXj06QXJ00m?Cd0Fe]K06UYJ@0U9BD0<30`07alO00S8b<0EUIF
0492@P14A4@0F5QH0:^[Z`0o?cl0CTi>03lo?`0f=SH0MGEe07IfMP0[:b/0EEEE06a/K01GEeL0U9BD
0410@00T92@0@d=305QHF02KVi/0X:2P02/[:`0j>SX0?clo069RHP2KVi/0W9bL01hN7P1GEeL0U9BD
05QHF02>SXh09bLW04];B`1VIVH0FEUI03Ph>00m?Cd0:BTY05IFEP2=SHd0?Cdm05UIF@1JFUX0DU9B
05mOG`0X:2P0Hf=S04a<C011@D40T92@04Q8B010@400KVi^03Ti>@10@400TI6A09fMW@20P800?Shn
04Y:BP1dM7@0EEEE03Ti>@1KFe/0HF5Q06m_K`2VYZH0:BTY03/k>`1KFe/0GEeM08>3P`2/[:`0=cLg
034a<@1IFET0JVYZ08:2PP1nOWh092@T01dM7@1EEED0L71`09RHV02`/;00=cLg01dM7@1QHF40Ogmo
06][J`1WIfL0>c/k06EUI@1aLG40MGEe06=SH`0e=CD071`L0492@P1ZJVX0B4Q80410@018B4P071`L
038b<P1dM7@0CTi>01hN7P14A4@0?3`l03De=@1cLg<0EUIF02TY:@1VIVH203De=@0305eMG@13@d?o
oooo00gooooo0?l0CTi>06=SH`12@T80Fe]K04a<C01GEeL0?clo04i>CP1jNWX0<C4a0451@@1CDe<0
IVIV05MGE`2:RXX0FUYJ06IVIP16ATH0DU9B03Lg=`1DE5@08B4Q03De=@19BDT06ATI04a<C00O7al0
Kfm_04m?C`1EEED0Kfm_08B4Q01cLg<0C4a<05iNGP25QHD0JVYZ02/[:`0e=CD0De=C03lo?`1IFET0
Bd];069RHP12@T80H61P04m?C`1VIVH0?Cdm05UIF@10@400G5aL0492@P0X:2P0J6QX03Xj>P11@D40
<c<c05]KF`16ATH0EEEE03Hf=P1<C4`0Lg=c04i>CP26QXH0?clo0451@@1]KFd0@d=305=CD`1@D500
J6QX02LW9`0T92@0Hf=S03Ph>01DE5@0A4A405EEE@1JFUX0?Shn04m?C`14A4@0;Bd]03lo?`15ADD0
C4a<03dm?@1>CTh0ADE504M7A`0b<S80<c<c05=CD`0n?Sh0De=C059BDP0k>c/0D51@06e]K@1CDe<0
HF5Q0820P01DE5@0EUIF07=cL`28R8P0EEEE05mOG`1<C4`0HF5Q06i^KP1JFUX0GUiN04Q8B00Y:BT0
D51@03<c<`1?Cdl0DE5A06EUI@16ATH0NWYj05EEE@1iNGT0F5QH079bLP1oOgl0D51@079bLP23Ph<0
V9RH07moO`15ADD0FUYJ06MWI`18B4P0E5AD03dm?@16ATH0De=C07]kN`2?Shl0PH6107]kN`1RHV80
J6QX075aL@1iNGT0RXZ:03lo?`1KFe/0FEUI01LG5`22PX80>c/k05MGE`0h>3P0E5AD03`l?01ADE40
@T920451@@1HF5P07alO04A4A01IFET0F5QH06=SH`1DE5@0@41003Lg=`17AdL0;Bd]05eMG@2SXj<0
;2`/055AD@1NGUh08R8R059BDP1=CDd0BDU905]KF`11@D40Shn?06YZJP13@d<0<C4a06=SH`0h>3P0
=SHf03Ti>@1ADE40B4Q803lo?`0U9BD0VIVI07MgM`0Z:RX0ADE5018B4P1DE5@0GUiN02DU9@1<C4`0
;Bd]05MGE`18B4P0DE5A04E5A@13@d<0<c<c03De=@0j>SX0Hf=S06UYJ@0L71`0C4a<04E5A@1:BTX0
8B4Q05=CD`0Q8B40CDe=05iNGP0I6AT0BTY:05eMG@1JFUX0Bd];079bLP0j>SX0?clo06YZJP0j>SX0
>CTi0410@012@T80EUIF0;jn_P1FEUH0G5aL0410@01CDe<0Y:BT08V9R@1<C4`0EeMG05IFEP11@D40
D51@04e=C@1dM7@000P0IfMW0<O7a`2KVi/0F5QH05iNGP15ADD0GEeM03<c<`80>3Ph00d0D51@03/k
>`18B4P0@d=307YjNP0m?Cd0@d=308b<S01=CDd0De=C05EEE@1NGUh0EeMG013ooooo000>ooooo`3@
06=SH`1BDU80SXj>08V9R@1:BTX0B4Q806a/K02<S8`0HF5Q07emO@0d=3@0EeMG09:BTP17AdL0SHf=
02<S8`1;Bd/0S8b<04Y:BP1=CDd0FEUI01TI6@0`<300E5AD028R8P1FEUH09bLW03hn?P1HF5P0@T92
03<c<`0o?cl0<30`04A4A01KFe/0Ogmo01hN7P1FEUH0>c/k069RHP1hN7P0:BTY01DE5@1CDe<0AdM7
09BDU02@T900820P01lO7`1YJFT0B4Q80;Rh^00j>SX0?3`l04e=C@1BDU80C4a<06e]K@0J6QX0<S8b
03Ph>01cLg<0CDe=04m?C`0N7Qh0;2`/05aLG01jNWX0;bl_03@d=00];Bd0CDe=069RHP19BDT0>CTi
05mOG`0c<c<0LG5a08n?S`0`<3009BDU0492@P0];Bd0LW9b0820P00P82006a/K02TY:@1?Cdl0Ng]k
04];B`0Q8B40;Rh^03Ti>@0o?cl0MgMg02DU9@0K6a/0;2`/05ADE013@d<0Lg=c05ADE00X:2P0@D51
06EUI@0k>c/0QHF509>CT`0l?3`0E5AD06QXJ00i>CT0HV9R07]kN`0V9RH0K6a/06m_K`15ADD0L71`
03hn?P0R8R80HV9R05YJFP1;Bd/0HF5Q024Q8@16ATH0Ogmo03Lg=`1;Bd/0F5QH04I6AP1<C4`0RHV9
02TY:@0n?Sh0J6QX0451@@1cLg<0Ph>3024Q8@0a<C40Kfm_05MGE`1/K6`0JFUY01DE5@0W9bL0BDU9
03dm?@1dM7@0ZJVY02TY:@0O7al0CTi>051@D02<S8`0=3@d02LW9`18B4P0MgMg07IfMP2c/k<0@T92
02`/;01;Bd/0NGUi06MWI`2TY:@0=CDe03/k>`1[Jf/0H61P04i>CP1jNWX06a/K04E5A@1/K6`0?Cdm
04U9B@1bLW80;Rh^05IFEP2SXj<0B4Q803Ti>@0k>c/0@D5109BDU01LG5`07AdM04i>CP12@T80E5AD
05eMG@0O7al0=SHf04m?C`1YJFT0JVYZ04Q8B00e=CD0EUIF04U9B@2GUiL0Hf=S01TI6@80EeMG04@0
RXZ:09JFUP0_;bl0:b/[08N7Q`2i^KT0R8R8028R8P1>CTh0S8b<07=cL`1KFe/0FUYJ02/[:`1=CDd0
T92@06YZJP0n?Sh0GUiN0410@01;Bd/0JVYZ05mOG`0l?3`0F5QH03De=@1MGEd0GUiN06ATI00k>c/0
O7al03Lg=`1GEeL0Y:BT05UIF@0a<C40?Shn03lo?`1JFUX0V9RH02DU9@0^;Rh0EUIF03hn?P1cLg<0
MGEe01DE5@0h>3P0I6AT04Y:BP1/K6`0Jf][01dM7@0U9BD0H61P06MWI`19BDT0MgMg04E5A@0];Bd0
Fe]K08N7Q`14A4@0UiNG0492@P1;Bd/0GUiN03`l?0kooooo06d0E5AD06i^KP13@d<0LW9b069RHP1_
Kfl0C4a<04=3@`1dM7@0FEUI04m?C`1;Bd/0K6a/04Y:BP27QhL0K6a/05IFEP14A4@0Cdm?04];B`1e
MGD0?3`l04E5A@1WIfL0<C4a04E5A@0E5AD0De=C06EUI@1>CTh0EUIF06=SH`1;Bd/0D51@05]KF`1^
KVh0O7al0492@P0^;Rh0Cdm?05iNGP1SHf<0@d=305MGE`11@D40BTY:04a<C01DE5@092@T06ATI01>
CTh0D51@05IFEP0n?Sh0Lg=c03lo?`1>CTh0B4Q805eMG@1CDe<0G5aL03lo?`1EEED0Lg=c04M7A`27
QhL0BTY:04U9B@1nOWh0G5aL04a<C00o?cl0[:b/03Hf=P0[:b/0RXZ:04=3@`1PH600@T9205]KF`1B
DU80AdM705EEE@1=CDd0B4Q80492@P16ATH0K6a/051@D01IFET0DU9B05MGE`0l?3`0>c/k06ATI01?
Cdl0CTi>06QXJ014A4@0IFEU05iNGP1@D500F5QH051@D017AdL0DU9B05mOG`1@D500?Cdm0080@410
00h0IVIV04m?C`0c<c<0>c/k04Q8B00M7Ad0@D5102DU9@1>CTh0>c/k05aLG01=CDd0HV9R03Hf=P<0
HV9R02X0P82005IFEP1YJFT0KVi^05EEE@1?Cdl0>SXj06e]K@24Q8@0D51@06YZJP0g=cL0EEEE04=3
@`22PX80V9RH079bLP1^KVh0EUIF07QhN01MGEd0MGEe061PH00o?cl0V9RH06UYJ@0L71`0QXJ60492
@P20P800ATI605UIF@0j>SX0O7al05ADE01;Bd/0J6QX02h^;P1]KFd0IfMW069RHP1TI6@2079bLP0[
03dm?@1KFe/0;Bd]04I6AP1DE5@092@T04a<C016ATH071`L02<S8`0a<C40ADE5055AD@0V9RH0M7Ad
069RHP0c<c<0:2PX03Ph>011@D40=3@d03Hf=P0b<S80ADE504a<C00M7Ad0H61P04Q8B00V9RH0DU9B
00T92@0h>3P0BDU9024Q8@1?Cdl0@D5106][J`18B4P0F5QH0410@013@d<0<C4a03lo?`0204A4A00h
04U9B@0G5aL0C4a<0492@P1;Bd/09RHV04m?C`0m?Cd0M7Ad08^;R`0K6a/0EUIF05ADE00d=3@0>SXj
05ADE00X:2P0<30`07MgM`0h>3P0E5AD05eMG@0h>3P0FEUI08j>SP1@D500=CDe04Q8B01mOGd0_Kfm
07QhN015ADD0C4a<061PH01EEED0K6a/020P801UIFD0?3`l05aLG01DE5@0=cLg07emO@0[:b/0CDe=
03Ph>01HF5P0>CTi065QH@0m?Cd0Hf=S04M7A`1hN7P0<30`04m?C`29RHT3055AD@0308>3P`1HF5So
oooo00oooooo000>ooooo`0A0:2PX01XJ6P0JFUY06][J`2MWId0B4Q806UYJ@1BDU80Kfm_08:2PP0f
=SH0B4Q804M7A`1YJFT0QHF502l_;`0V9RH00P19BDT0@P2BTY80Q8B401hN7P0W9bL0E5AD069RHP37
alL0HV9R01`L700h>3P0GEeM09>CT`2YZJT0<S8b02TY:@0j>SX0EEEE04e=C@1VIVH0820P02XZ:P19
BDT0DU9B02PX:01EEED09BDU05ADE025QHD0?Shn03Lg=`10@4008B4Q06m_K`1[Jf/06ATI02l_;`1H
F5P0BDU90:n_[`1?Cdl0<S8b05EEE@0n?Sh0IVIV08R8R00Q8B406a/K02l_;`1ADE40Cdm?07EeM@0G
5aL0C4a<079bLP1FEUH0DE5A08V9R@0^;Rh0<c<c06YZJP1MGEd0MgMg0::RXP0d=3@0BDU908V9R@80
L71`04@0QHF501dM7@17AdL0KVi^03Ph>021PH40CTi>02h^;P1OGel0O7al03lo?`1aLG40ATI602LW
9`0k>c/0MWIf03Hf=P1@D5008B4Q01TI6@1BDU80PX:203Lg=`20P800?Cdm028R8P1NGUh0Shn?02h^
;P1LG5`0DU9B03hn?P2CTi<0K6a/020P801=CDd0DE5A06=SH`2IVIT0:RXZ038b<P1MGEd0I6AT0:NW
Y`2g]kL07QhN03dm?@1oOgl0G5aL09>CT`0j>SX0=3@d05EEE@22PX80IfMW079bLP0m?Cd0CTi>05iN
GP1]KFd0=SHf04Y:BP0S8b<09bLW05UIF@2IVIT0DU9B02d];@80@T9200<0LG5a0820P00a<C400P11
@D40A00o?cl0/[:b05mOG`0Y:BT0=cLg03Ph>011@D40[Zj^06i^KP10@400F5QH03dm?@1ADE40Xj>S
038b<P0d=3@0N7Qh04m?C`1CDe<0WinO04i>CP1DE5@0A4A4055AD@1lO7`0JVYZ038b<P1=CDd0DU9B
06YZJP2MWId0?Cdm02PX:01]KFd0EUIF08n?S`0l?3`07AdM06EUI@1bLW80GEeM05YJFP0[:b/0AdM7
09>CT`1CDe<0KFe]02l_;`1;Bd/0Shn?04U9B@0h>3P0M7Ad03Ti>@2CTi<0F5QH02d];@1/K6`0B4Q8
06EUI@1oOgl0:RXZ03hn?P1>CTh0JVYZ08:2PP1jNWX0:BTY0P1HF5P0;P0l?3`0IfMW08V9R@0];Bd0
BDU905=CD`1<C4`0J6QX0=SHf01>CTh0>CTi04A4A01FEUH0B4Q80;Vi^@1<C4`0?clo05ADE019BDT0
DE5A07AdM00Z:RX0Cdm?0861P@12@T80DU9B05]KF`0O7al0?3`l05MGE`1>CTh0H61P06MWI`0M7Ad0
7alO0410@01CDe<0=SHf03lo?`0K6a/0=cLg05=CD`1<C4`0=CDe02l_;`0U9BD>ooooo`0P0451@@1G
EeL0@T9204E5A@0i>CT0Fe]K04Q8B01>CTh0Ph>303Ti>@14A4@0DE5A05EEE@18B4P0N7Qh05YJFP25
QHD0CDe=04];B`0P8200Cdm?02XZ:P0e=CD0ATI602h^;P1@D500:b/[04e=C@1^KVh0De=C055AD@1H
F5P204];B`0H04m?C`1BDU80EUIF03hn?P0g=cL0?Shn03`l?01=CDd0DE5A05aLG00o?cl0D51@075a
L@1MGEd0=3@d0451@@1UIFD0G5aL0410@00/;2`0EeMG03Ti>@1<C4`0ADE50P17AdL0Z01?Cdl0AdM7
051@D01@D500F5QH06m_K`1=CDd0CTi>05iNGP1WIfL0De=C05aLG02?Shl0EEEE0451@@1]KFd0@d=3
05UIF@1;Bd/0CDe=04=3@`1ADE40A4A403Xj>P10@400;Rh^03dm?@1;Bd/0>CTi04I6AP0m?Cd0D51@
02@T9015ADD0CDe=04i>CP19BDT0E5AD05MGE`1MGEd0Fe]K06=SH`1WIfL0NWYj06EUI@1SHf<0NGUi
05YJFP1TI6@0De=C04Y:BP1GEeL0MGEe0451@@1@D500A4A402DU9@1HF5P0:2PX04I6AP0f=SH0HV9R
05iNGP1_Kfl0=cLg065QH@1`L700M7Ad08j>SP20P800JFUY09JFUP1XJ6P0Lg=c03Ph>01NGUh0OWin
04Y:BP1>CTh0;bl_03hn?P0j>SX0Ti>C08Z:RP1/K6`0LW9b06UYJ@1^KVh0LW9b096AT@1OGel0820P
04U9B@1NGUh092@T08F5Q@1=CDd0DE5A03Xj>P1LG5`0=cLg05IFEP10@400GUiN05eMG@0M7Ad0>c/k
04a<C016ATH0AdM7059BDP1OGel0>SXj059BDP0d=3@0GUiN07QhN00N7Qh0BTY:059BDP0/;2`0<c<c
03hn?P11@D40GEeM04=3@`2<S8`0M7Ad04i>CP11@D40A4A403Xj>P0^;Rh0>c/k03Hf=P1DE5@0Bd];
03Ph>02?Shl0Lg=c03dm?@1JFUX092@T04E5A@1ADE407alO05UIF@11@D40JVYZ05EEE@1FEUH0A4A4
05=CD`0i>CT0?clo030`<019BDT0Cdm?01XJ6P1>CTh0;bl_05IFEP0[:b/0I6AT030`<01VIVH0MGEe
01LG5`80AdM70300CTi>03Ph>012@T80>SXj04Y:BP1[Jf/0@T9204E5A@0l?3`0@d=306ATI01;Bd/0
TI6A05eMG@1cLg<0`<300;no_`2BTY80C4a<08F5Q@2=SHd0De=C07QhN00/;2`0M7Ad05iNGP26QXH0
EeMG0451@@1QHF40ATI605EEE@0c<c<0>3Ph04i>CP1aLG40?clo03Ph>01?Cdl0IFEU03De=@0h>3P0
MgMg04I6AP1DE5@0CTi>05MGE`1MGEd@ooooo`003_ooool0?@2;Rh/0TY:B0:2PX01VIVH0Kfm_07Ui
N@2KVi/0?clo05IFEP1gMgL0NGUi0;^k^`18B4P0DU9B08J6QP1/K6`0D51@06MWI`0M7Ad0>3Ph06i^
KP10@400?Shn03Ti>@0b<S80EUIF08Z:RP0_;bl0De=C03hn?P1?Cdl0O7al061PH00[:b/0@d=30451
@@1UIFD0Z:RX04a<C00X:2P0AdM705aLG016ATH0L71`01PH600i>CT0IFEU05UIF@1SHf<0LG5a02LW
9`17AdL0GUiN04i>CP1UIFD0Lg=c03De=@1TI6@0GEeM07EeM@1iNGT00P10@4005@1bLW80Jf][05aL
G02DU9@0?Cdm03Ti>@1cLg<0@d=304A4A01?Cdl07QhN01dM7@0k>c/0@D51038b<P13@d<07QhN03lo
?`1XJ6P0<S8b04I6AP0203Lg=`2L05eMG@1`L7008b<S034a<@0^;Rh0@T9208f=S@1dM7@05aLG03`l
?019BDT0JFUY0;bl_013@d<071`L02`/;01BDU80JFUY0:>SX`0];Bd08R8R038b<P1CDe<0HV9R0:FU
Y@0l?3`0;Rh^03Ti>@17AdL0GemO08R8R00[:b/0?clo06IVIP1dM7@0>SXj055AD@0Q8B40=SHf06AT
I01/K6`0;bl_04e=C@0S8b<0BTY:06][J`10@400?Shn06i^KP0X:2P0IFEU08J6QP0Q8B40=SHf05mO
G`0T92@0EEEE06=SH`0X:2P0EeMG04=3@`11@D40OGem02/[:`0E5AD0D51@05UIF@1DE5@0GEeM07Ee
M@0V9RH0IFEU0820P01@D500UIFE06i^KP0U9BD0MGEe0;Fe]@16ATH0S8b<06ATI00^;Rh0LW9b09NG
U`1?Cdl0MGEe01lO7`0W9bL0D51@061PH018B4P0PH6103@d=00c<c<0@d=305MGE`2<S8`0BTY:024Q
8@1`L700Ogmo051@D01[Jf/0;2`/03`l?02GUiL0OGem03Ti>@0l?3`0>3Ph07]kN`1SHf<07alO03dm
?@1@D500JFUY06i^KP0W9bL0CTi>03lo?`16ATH0YJFU02<S8`17AdL0EeMG0492@P1cLg<0cLg=04e=
C@1?Cdl0LW9b09BDU03=cLd0<30`04i>CP1VIVH0HF5Q09^KV`2QXJ40@D5104I6AP1jNWX0Bd];07Yj
NP20P800=CDe02TY:@24Q8@0>CTi06EUI@1WIfL0:b/[03Xj>P19BDT0HF5Q0P1ADE406`0`<300<c<c
05=CD`1YJFT0BDU905aLG00l?3`0IVIV07MgM`14A4@0?Cdm02d];@1BDU80PH6108J6QP0f=SH0;Rh^
04M7A`1UIFD0KFe]0:2PX00h>3P06a/K03hn?P1KFe/0VYZJ06QXJ002024Q8@0504U9B@1=CDd0Q8B4
071`L00C4a<03_ooool07`1?Cdl0M7Ad06IVIP1HF5P0E5AD07IfMP0j>SX0=cLg05aLG00V9RH0;Bd]
05mOG`2KVi/0EEEE06e]K@1?Cdl0DE5A04m?C`1KFe/0?Shn05YJFP0Y:BT0?clo055AD@0J6QX0C4a<
01lO7`1PH600HF5Q04i>CP1^KVh00P1]KFd01019BDT0HF5Q07UiN@1VIVH202/[:`0<04Q8B010@400
EeMG03Xj>P17AdL0=cLg038b<P0c<c<0ATI601dM7@1>CTh0<C4a0P0f=SH04@0[:b/0@41005EEE@1A
DE40>CTi05YJFP1^KVh0D51@04E5A@1QHF40GemO03Ti>@1EEED0BTY:051@D01ADE40@T920080DE5A
00`0FUYJ02XZ:P0T92@0GUiN05EEE@1?Cdl0ADE505]KF`1MGEd0BDU905EEE@17AdL203/k>`0J05mO
G`1NGUh0HV9R05UIF@17AdL0EEEE03Hf=P1RHV80E5AD04=3@`1GEeL0Cdm?03Ph>017AdL0F5QH04i>
CP1CDe<0Jf][04a<C01[Jf/0GUiN06ATI018B4P0BTY:05=CD`1GEeL205mOG`0J05YJFP18B4P09bLW
05eMG@0^;Rh0Fe]K05=CD`1]KFd0NWYj07alO01DE5@0ZJVY07AdM01bLW80NGUi065QH@23Ph<0Rh^;
07=cL`1_Kfl0@41007EeM@2ATI40DE5A06UYJ@0g=cL204M7A`0e07IfMP26QXH0TI6A09NGU`1BDU80
O7al055AD@1KFe/0MGEe04A4A01jNWX0I6AT018B4P1gMgL0=CDe05iNGP19BDT0Fe]K05ADE01_Kfl0
DE5A0451@@1?Cdl0=CDe05IFEP1NGUh0EeMG05ADE01MGEd0LG5a04i>CP1QHF408b<S04A4A01`L700
;bl_06EUI@1aLG409bLW03<c<`0l?3`0Fe]K07=cL`0d=3@0R8R806e]K@16ATH0F5QH04e=C@0c<c<0
=CDe0451@@0i>CT00P1QHF40400I6AT0PH6105UIF@0g=cL0MWIf02PX:012@T80ADE503De=@1`L700
CDe=06][J`1UIFD0EUIF04U9B@1@D50203dm?@0G04A4A017AdL0Bd];024Q8@1_Kfl0FEUI04Q8B00/
;2`0HF5Q03/k>`1hN7P0TI6A01`L701=CDd0DE5A04a<C010@400C4a<03dm?@1HF5P0MWIf034a<@1D
E5@00P1QHF409P1SHf<0GemO06MWI`1<C4`0D51@08b<S01HF5P0;bl_05ADE01BDU80CTi>05MGE`1Z
JVX0HV9R0::RXP1FEUH0T92@079bLP1DE5@0IfMW02PX:019BDT0>CTi04M7A`1;Bd/0G5aL05=CD`1>
CTh0>3Ph04];B`17AdL0De=C06UYJ@1?Cdl0H61P05YJFP29RHT0JVYZ4?ooool000kooooo04T0Hf=S
06a/K02][Jd0F5QH0410@01^KVh0/;2`059BDP0e=CD0Cdm?04Q8B023Ph<0A4A403hn?P1@D500GUiN
07QhN02>SXh09BDU034a<@1@D500>SXj075aL@1OGel0;2`/03lo?`1NGUh0Rh^;0;2`/01=CDd0@D51
05EEE@1aLG40Kfm_08V9R@0i>CT0<c<c07EeM@2EUID0DE5A079bLP0N7Qh0;Bd]08N7Q`12@T80C4a<
02l_;`0i>CT0B4Q8096AT@12@T80GUiN03Xj>P0O7al0CDe=06a/K00_;bl0Fe]K02HV9P15ADD0Fe]K
04];B`0e=CD0?Cdm03De=@18B4P0Ng]k07MgM`0M7Ad0?Shn05=CD`25QHD0UYJF00808b<S0</0;2`/
05]KF`1UIFD0Jf][01/K6`0e=CD0AdM706a/K02CTi<0DU9B00l?3`17AdL0GUiN05UIF@1gMgL0=cLg
02HV9P1VIVH0SHf=04A4A01RHV8071`L01PH601TI6@0P82002<S8`1SHf<0<S8b02l_;`1XJ6P0IFEU
01lO7`1;Bd/0=3@d024Q8@1eMGD0N7Qh02<S8`0j>SX0C4a<03Lg=`26QXH0NWYj034a<@1?Cdl0@410
03De=@26QXH0=CDe03Ti>@1FEUH0@41005=CD`2DU9@0;bl_04A4A01`L700@T9205iNGP2WYjL08B4Q
05ADE022PX80BDU907AdM02=SHd0:BTY04a<C01cLg<0@T9208j>SP1hN7P07AdM04i>CP1ZJVX0=3@d
092@T01VIVH0ADE502PX:01[Jf/0ADE505QHF00k>c/09RHV03/k>`21PH40BDU905mOG`10@400Bd];
059BDP29RHT0?Shn04A4A012@T80E5AD08^;R`1HF5P07QhN04M7A`16ATH0CDe=08V9R@0f=SH0:2PX
03/k>`19BDT0_;bl07YjNP0Y:BT0AdM705ADE01[Jf/0WIfM034a<@0P8200CDe=0861P@1MGEd0Bd];
03Ti>@1?Cdl0MGEe04U9B@2:RXX0;bl_06IVIP1mOGd0@d=306a/K01DE5@06ATI09bLW012@T80DU9B
0410@00G5aL0<30`08V9R@1jNWX0Fe]K028R8P1CDe<0UYJF06ATI01?Cdl0BTY:02h^;P1IFET0Vi^K
04U9B@10@400EeMG05UIF@20P800Ph>302d];@0c<c<0F5QH05EEE@1gMgL0X:2P02h^;P0/;2`0EeMG
055AD@2VYZH0/K6a02PX:00Y:BT0BTY:04];B`29RHT0YJFU02PX:01=CDd0N7Qh06YZJP2@T900@d=3
02DU9@17AdL0L71`06QXJ01lO7`0Kfm_02l_;`10@400Ogmo08b<S013@d<0T92@02`/;011@D40IfMW
05YJFP0o?cl0J6QX02h^;P1FEUH0MGEe03Xj>P0>ooooo`0<06i^KP1EEED0?Shn07moO`1aLG40F5QH
02d];@0o?cl0YJFU065QH@1KFe/0Hf=S0P1KFe/0F01VIVH0I6AT06UYJ@1EEED0ADE504U9B@1XJ6P0
=CDe03@d=01SHf<0?3`l05YJFP0N7Qh0I6AT09RHV01LG5`0M7Ad08j>SP1BDU80HV9R06YZJP1^KVh0
P82004E5A@0d=3@0>CTi05QHF01ADE40?3`l03lo?`0^;Rh0:b/[04m?C`10@4009RHV04m?C`18B4P0
@d=304a<C01:BTX0IFEU0451@@15ADD0=cLg055AD@18B4P0EEEE03Lg=`1DE5@0Ogmo03lo?`1MGEd0
?clo04I6AP1eMGD0D51@0410@016ATH0YjNW03Ti>@0T92@0MgMg03`l?01LG5`0>3Ph04m?C`17AdL0
?Shn04Y:BP12@T80<30`02l_;`0g=cL0EUIF04i>CP16ATH0@410059BDP0W9bL0B4Q8065QH@17AdL0
B4Q805aLG017AdL0FUYJ05EEE@1DE5@205EEE@2S03Xj>P1EEED0I6AT06YZJP0m?Cd0?clo04U9B@1g
MgL0BTY:03dm?@10@400BDU902LW9`18B4P092@T0451@@0d=3@0De=C05MGE`1;Bd/0:b/[04];B`1:
BTX0HF5Q08J6QP1?Cdl0IfMW075aL@1]KFd0ADE503lo?`1bLW80JFUY0492@P1CDe<0=CDe059BDP1?
Cdl0W9bL0:JVYP1fMWH0Fe]K05EEE@1NGUh0H61P08Z:RP2ATI40=CDe05=CD`1XJ6P0;2`/07inOP0l
?3`0NWYj03`l?01EEED0=CDe05UIF@1EEED0@d=304i>CP0X:2P0BDU905QHF00m?Cd0?clo07EeM@1U
IFD0>SXj05EEE@0m?Cd0DU9B06m_K`11@D40?clo051@D00[:b/0@T9204i>CP1:BTX0Cdm?02LW9`2;
Rh/0NWYj04e=C@1?Cdl0FUYJ051@D00_;bl0ATI603Hf=P1MGEd0D51@02/[:`2QXJ40MWIf059BDP2J
VYX09bLW04Y:BP1>CTh09BDU051@D011@D40NWYj06YZJP1OGel0FUYJ055AD@11@D40CDe=0410@01S
Hf<0CTi>01@D501=CDd0AdM705iNGP0c<c<0E5AD02l_;`1<C4`0GEeM014A4@1@D500AdM704a<C014
A4@0CDe=03lo?`19BDT0FEUI03Hf=P14A4@0CDe=065QH@1nOWh0L71`0:ZZZP1ADE40GemO07emO@0n
?Sh07alO051@D01lO7`0IFEU069RHP2:RXX0=CDe069RHP1_Kfl0QHF5075aL@1MGEd0KVi^05IFEP1C
De<0AdM705eMG@1LG5`0EeMG0492@P0205EEE@0905]KF`0o?cl0@41005iNGP1:BTX0DE5A059BDP1F
EUH0De=C013ooooo000>ooooo`0j06IVIP1NGUh0NWYj09nOW`1:BTX0ADE505YJFP1iNGT0Shn?07]k
N`12@T80D51@069RHP1mOGd0]kNg01`L701CDe<0LG5a05MGE`1NGUh0IVIV02XZ:P1@D500HV9R03Lg
=`1IFET0=SHf02l_;`2DU9@0RXZ:06MWI`1>CTh0:b/[02HV9P1bLW80QXJ602@T900h>3P08B4Q05iN
GP1_Kfl0@d=301dM7@16ATH0Fe]K0::RXP1VIVH06a/K02TY:@1@D500KVi^0:^[Z`0l?3`09bLW024Q
8@1ADE40GemO08b<S080:2PX0=X0CTi>06m_K`1lO7`0MGEe028R8P0m?Cd0CDe=06m_K`1UIFD0KVi^
02`/;00c<c<0LW9b05eMG@15ADD0GemO02/[:`13@d<0I6AT03Lg=`16ATH0Ti>C03Ti>@1nOWh0Lg=c
034a<@0d=3@0CDe=030`<01NGUh0ATI601dM7@0`<300@T92051@D01lO7`0;Bd]01/K6`0h>3P0Bd];
03`l?01lO7`0M7Ad010@4017AdL0G5aL055AD@2GUiL0K6a/01hN7P1IFET0MWIf069RHP1oOgl0P820
02`/;01EEED0J6QX03`l?01YJFT0>SXj02HV9P1hN7P0ADE505=CD`1PH600<30`0410@01=CDd0GUiN
05UIF@1JFUX08R8R04=3@`1KFe/0@D5105EEE@1<C4`061PH03/k>`1lO7`0:b/[03<c<`0_;bl0ATI6
07alO01gMgL09BDU03Xj>P0_;bl0A4A40;2`/02SXj<09BDU02d];@19BDT0EEEE0:BTY01WIfL08b<S
02TY:@1ADE40H61P0;2`/01<C4`0=SHf04e=C@1GEeL0Lg=c09:BTP0T92@0DE5A06QXJ01BDU80C4a<
06i^KP0T92@0?Cdm05EEE@1ZJVX0Cdm?075aL@0g=cL0FUYJ09>CT`0^;Rh0Bd];03Lg=`0^;Rh0Lg=c
05QHF00];Bd0EeMG02HV9P1:BTX0Bd];02PX:01DE5@0<C4a02XZ:P2HV9P0F5QH04U9B@17AdL0DU9B
08:2PP1YJFT0<C4a051@D017AdL0YjNW07emO@0K6a/0;Bd]069RHP1nOWh0UIFE01hN7P0j>SX0IFEU
07EeM@1iNGT0BTY:0410@01OGel0GUiN071`L01eMGD0LW9b0492@P27QhL0YZJV04e=C@1MGEd0HF5Q
02PX:01GEeL0QXJ606IVIP15ADD0UiNG02l_;`1:BTX0UYJF07MgM`11@D40JFUY02XZ:P0i>CT0IfMW
03@d=010@400De=C03De=@1LG5`0P82002PX:019BDT0=SHf03<c<`0o?cl0L71`02d];@0o?cl0>CTi
03lo?`1eMGD0^k^k03Hf=P0k>c/0EUIF03Lg=`1OGel0a/K604M7A`15ADD0Jf][03Xj>Pkooooo03L0
AdM705EEE@18B4P0?clo03/k>`1BDU80>3Ph02LW9`1EEED09BDU02XZ:P1PH600Shn?04=3@`15ADD0
@d=304i>CP0o?cl0Cdm?020P8016ATH0@41004I6AP19BDT0@D5104I6AP0W9bL0?Shn05YJFP13@d<0
BTY:05mOG`16ATH0@D5103hn?P1:BTX0Hf=S03<c<`0m?Cd0=3@d02PX:00n?Sh0D51@05aLG00d=3@0
=cLg06MWI`1@D500:b/[03Ti>@1UIFD0EUIF02d];@0K6a/0=SHf0080Bd];01`0CTi>04Q8B01>CTh0
F5QH05IFEP1HF5P0Jf][07moO`1/K6`0Bd];04M7A`1oOgl0DE5A04E5A@14A4@0Xj>S061PH01JFUX0
L71`0492@P1RHV80CDe=05IFEP1HF5P0EeMG05iNGP0m?Cd0@d=30P0l?3`0D@1KFe/0EUIF05IFEP11
@D40GUiN02`/;01CDe<0IfMW05=CD`1@D500G5aL06EUI@1XJ6P0GUiN05mOG`1RHV80L71`05IFEP1J
FUX0OWin05UIF@1PH600BDU90492@P16ATH0BTY:04U9B@13@d<0@T9202@T9015ADD0@T9204e=C@0h
>3P0I6AT06][J`1`L700>c/k07UiN@1OGel0PX:209RHV01QHF40PH6109RHV02h^;P0O7al03lo?`1^
KVh0QHF504A4A01/K6`0AdM7061PH00m?Cd0Ng]k075aL@1hN7P0RXZ:06][J`1aLG40Bd];04Q8B01L
G5`0@41005eMG@1?Cdl09RHV096AT@16ATH0Cdm?051@D01OGel0BTY:05MGE`1EEED0AdM704];B`0K
6a/0C4a<05]KF`020451@@1?05mOG`1RHV80>3Ph04E5A@0`<300CDe=06=SH`0H61P0H61P05QHF00K
6a/0;2`/03/k>`1FEUH0LG5a03dm?@1iNGT0NWYj079bLP1LG5`0ADE504e=C@11@D40Bd];03dm?@1R
HV80J6QX04I6AP1RHV80ADE504=3@`1kNg/0=cLg05]KF`1DE5@0@T9207YjNP1CDe<0NGUi07EeM@1F
EUH0DE5A06IVIP11@D40De=C05QHF01;Bd/0GemO03@d=01TI6@0A4A4051@D016ATH0Jf][04Y:BP1e
MGD0LG5a03De=@1@D500JVYZ05MGE`1@D500EUIF04Q8B013@d<0MGEe03`l?010@400B4Q806][J`2<
S8`0JVYZ09^KV`0h>3P0L71`051@D016ATH0>3Ph04Y:BP0205MGE`0504A4A01GEeL0GemO06a/K015
ADD00`1NGUh04@21PH40<c<c069RHP13@d<0CDe=03`l?01_Kfl0>CTi03dm?@0j>SX0I6AT04I6AP1=
CDd0GEeM04];B`1KFe/0GUiN0080FUYJ4?ooool000kooooo0600P82009JFUP2<S8`0H61P08n?S`1f
MWH0Xj>S03/k>`14A4@0Hf=S06IVIP1[Jf/0?Cdm04a<C01VIVH0BTY:059BDP1?Cdl0>SXj04Y:BP1X
J6P0?3`l04a<C01LG5`0ATI608>3P`1QHF40=CDe04a<C01?Cdl0HV9R08>3P`0a<C40=SHf04m?C`16
ATH0BTY:06i^KP0a<C409bLW0410@01CDe<0?Cdm069RHP0O7al0?Shn05QHF010@400F5QH05MGE`0^
;Rh0@T9205YJFP12@T80JFUY065QH@0Z:RX0G5aL04Y:BP15ADD0K6a/02TY:@0g=cL0RHV905ADE01;
Bd/0Lg=c02LW9`0T92@0FEUI02/[:`19BDT0@d=302@T9018B4P0FEUI03Ph>013@d<0EeMG034a<@1F
EUH0Lg=c03/k>`1MGEd0E5AD03<c<`1cLg<0MWIf02@T9014A4@0ATI603Lg=`20P800QXJ6028R8P18
B4P203hn?P2d08J6QP1XJ6P071`L03lo?`1OGel0HF5Q08J6QP0g=cL06a/K059BDP23Ph<0DE5A061P
H00`<3008b<S04M7A`1XJ6P0De=C07MgM`0^;Rh0@D5106=SH`1bLW80@D5105eMG@0_;bl0@d=307Ui
N@1=CDd0;Rh^05]KF`0^;Rh0IfMW08V9R@0h>3P0ADE505MGE`12@T80MgMg07emO@0n?Sh0C4a<06m_
K`1@D500KVi^04E5A@0U9BD0De=C05MGE`12@T80Hf=S024Q8@0K6a/0EeMG08>3P`1:BTX0EeMG05UI
F@0O7al0DU9B079bLP12@T80Kfm_07EeM@0^;Rh0EUIF061PH00l?3`0Kfm_07QhN00c<c<0Jf][05iN
GP1:BTX0HV9R03@d=00h>3P0HV9R04=3@`1<C4`0GemO03`l?00o?cl0KFe]0451@@1DE5@0B4Q803Ti
>@1]KFd0Lg=c03/k>`0n?Sh0@d=304E5A@2?Shl0G5aL03Lg=`13@d<0@D5108:2PP1HF5P09RHV04e=
C@1CDe<0QHF504I6AP0^;Rh0EEEE04U9B@0l?3`0VIVI02PX:00h>3P0FUYJ0492@P1[Jf/0Q8B403De
=@1RHV80CDe=05eMG@2HV9P0;Rh^04I6AP24Q8@0A4A405MGE`1`L700=cLg05iNGP1fMWH0@T9206=S
H`1YJFT0<S8b05MGE`1kNg/0=CDe055AD@1<C4`0?Shn05]KF`2>SXh0DE5A03lo?`17AdL0C4a<06m_
K`2KVi/0GUiN04Y:BP1GEeL0>c/k06=SH`1iNGT0?Cdm04I6AP1IFET0>CTi07QhN01jNWX0>3Ph030`
<01DE5@0DE5A06YZJP20P800<30`03Hf=P1FEUH0G5aL06UYJ@1WIfL0:2PX02HV9P18B4P0Bd];06e]
K@1HF5P08b<S3_ooool02@1ADE40H61P059BDP1<C4`0@D5105QHF00n?Sh0?3`l079bLP02038b<P0[
055AD@1ZJVX0B4Q8065QH@1=CDd0G5aL04a<C01UIFD0>CTi05=CD`0m?Cd0@T92059BDP0V9RH0Bd];
03<c<`1@D500H61P04i>CP1ZJVX0M7Ad06QXJ01LG5`0KVi^07EeM@1bLW80>3Ph038b<P15ADD0Bd];
05eMG@18B4P0EEEE03Ph>00d=3@0Bd];05eMG@0W9bL0D51@03/k>`14A4@0@d=302/[:`0204a<C00>
03lo?`0`<300EUIF05QHF01?Cdl0=SHf05IFEP1aLG40@D5105eMG@16ATH0BDU906IVIP1DE5@204M7
A`1406UYJ@0c<c<092@T05QHF00o?cl0B4Q803Xj>P1=CDd0@D5103dm?@1:BTX0@410038b<P0^;Rh0
?clo04m?C`12@T80BTY:03Hf=P12@T808B4Q0410@01ADE40=SHf04I6AP1=CDd0A4A4055AD@1MGEd0
EEEE05eMG@1]KFd0BTY:05]KF`1/K6`0IfMW04M7A`1?Cdl0C4a<06QXJ01KFe/0GUiN05EEE@1=CDd0
;Bd]05eMG@0X:2P0D51@04A4A020P800GUiN061PH00d=3@0Kfm_04Q8B01aLG40QXJ606YZJP28R8P0
QhN708Z:RP1YJFT0ATI606IVIP2>SXh0D51@05MGE`0d=3@204m?C`0J09FEU@26QXH0Ph>308V9R@1?
Cdl0L71`05IFEP1ZJVX0R8R805UIF@2@T900MGEe02l_;`1dM7@0CTi>05UIF@1EEED0HV9R05aLG01<
C4`0HV9R05IFEP1QHF40=SHf04M7A`1ADE4205eMG@0805QHF01RHV80B4Q8059BDP0b<S80DE5A06MW
I`0`<302061PH00?02XZ:P0o?cl0AdM705=CD`1UIFD0?Shn09:BTP1kNg/0DU9B04M7A`1`L700HV9R
0492@P1DE5@0=CDe0080Fe]K02@0>SXj092@T01WIfL0@d=306m_K`0T92@0EeMG05UIF@0f=SH0EEEE
04Q8B01^KVh0G5aL06EUI@1ADE40Bd];03hn?P1HF5P0D51@06EUI@1OGel0;Bd]069RHP1JFUX0EEEE
02h^;P1FEUH0>SXj05QHF01eMGD07alO04M7A`1`L700G5aL04Y:BP1UIFD204a<C00>05iNGP0m?Cd0
B4Q804A4A01KFe/0FUYJ04Y:BP1SHf<0DE5A08n?S`1/K6`0Ti>C071`L01CDe<205]KF`0K05iNGP1i
NGT0?Cdm05aLG01EEED0Ogmo06e]K@1KFe/0LG5a03Ti>@13@d<0>CTi04];B`0d=3@0DE5A0410@014
A4@0F5QH075aL@0j>SX0DU9B0;Ng]`1CDe<0HF5Q05YJFP1fMWH0Kfm_013ooooo000>ooooo`2l07Ee
M@1?Cdl0Ogmo07YjNP0j>SX0C4a<05iNGP1TI6@0HV9R06YZJP0`<300N7Qh06=SH`1PH600Lg=c0451
@@1ADE40L71`030`<019BDT0A4A403Xj>P19BDT0GemO02TY:@1ADE40:RXZ05eMG@2MWId0BTY:03Ph
>01OGel0>CTi04Y:BP2QXJ40QhN702PX:00`<300?3`l04m?C`2IVIT0<S8b03@d=01BDU80?Cdm06e]
K@1CDe<05ADE03@d=01PH600FEUI08>3P`0_;bl09BDU04I6AP1_Kfl0Bd];03@d=00X:2P0Bd];06UY
J@1]KFd0@T9204A4A00Z:RX0ADE5075aL@1TI6@092@T04M7A`0i>CT0J6QX07MgM`0O7al08b<S04Q8
B015ADD0De=C06][J`0M7Ad0;Rh^03/k>`13@d<0Shn?06UYJ@0R8R80?clo051@D00o?cl0OGem04M7
A`0L71`0CDe=04];B`0c<c<0SHf=028R8P0d=3@0<c<c05iNGP0l?3`0LG5a03Hf=P0k>c/0@D5106e]
K@13@d<0JVYZ04U9B@0X:2P0MGEe05]KF`0];Bd0B4Q805=CD`0e=CD0Vi^K04Y:BP0m?Cd0E5AD02/[
:`0R8R80KVi^03hn?P0];Bd0?3`l02h^;P1ZJVX0T92@02h^;P0_;bl0?Shn03<c<`2@T900Zj^[01lO
7`0c<c<0ATI604M7A`21PH40Y:BT01`L701=CDd0Cdm?03Hf=P27QhL0J6QX028R8P1ZJVX0N7Qh05]K
F`2a/K40Hf=S0492@P1EEED0QHF50492@P1lO7`0Cdm?030`<01EEED0UYJF05aLG01kNg/0<C4a03/k
>`1WIfL0Rh^;05UIF@1hN7P0:BTY04Y:BP1GEeL0IFEU038b<P16ATH0=SHf05EEE@2:RXX0;bl_03Hf
=P19BDT0@T920::RXP2CTi<09RHV04A4A018B4P206=SH`1H01`L700c<c<0FEUI0492@P18B4P0CTi>
01@D501FEUH0GEeM03Xj>P1jNWX0;2`/05MGE`1VIVH0Bd];079bLP14A4@08b<S09BDU01@D500Ph>3
05mOG`0V9RH0?Cdm08N7Q`1KFe/0@T9202h^;P18B4P0LG5a05aLG01>CTh0>c/k04=3@`1^KVh0KFe]
03Xj>P0d=3@0DE5A051@D01gMgL0OWin03Ti>@0l?3`0@d=304M7A`1aLG40GemO01lO7`0Y:BT0@d=3
05UIF@1fMWH0N7Qh01dM7@0b<S80@41005iNGP1eMGD0@D5102`/;01DE5@0BTY:05]KF`2?Shl0:RXZ
03lo?`1ZJVX0IfMW05YJFP1EEED0HF5Q028R8P0o?cl0IVIV04A4A01FEUH0GUiN02LW9`1<C4`0MWIf
05aLG01:BTX0IVIV02HV9P14A4@0Rh^;04E5A@kooooo02<0DE5A05mOG`1;Bd/0EEEE04M7A`1DE5@0
=SHf03@d=01lO7`0<C4a04I6AP1QHF40TY:B06MWI`1eMGD0BDU90861P@19BDT0JFUY028R8P1NGUh0
>SXj04I6AP1KFe/0A4A40492@P1:BTX0D51@06a/K018B4P0De=C06IVIP1NGUh0?3`l04a<C00205aL
G00702l_;`0d=3@0=3@d038b<P16ATH0B4Q804U9B@0203Hf=P0:04=3@`1<C4`0<c<c03Ti>@1EEED0
Bd];03Xj>P0g=cL0D51@04e=C@80Cdm?0P1CDe<0:@1FEUH0C4a<05=CD`1kNg/0KVi^07]kN`14A4@0
ADE506][J`1JFUX0DU9B0451@@1/K6`0EEEE0451@@1_Kfl0BTY:05ADE013@d<0@D510492@P1EEED0
B4Q804M7A`1>CTh0C4a<030`<01;Bd/0D51@04a<C011@D40DU9B034a<@0o?cl0FEUI04a<C01ADE40
EUIF061PH01QHF40H61P0080GEeM02@0JVYZ059BDP14A4@0Ng]k04M7A`13@d<0BTY:04m?C`13@d<0
BDU904Y:BP1ADE40BDU9020P800o?cl0Bd];059BDP0[:b/0Bd];075aL@1TI6@0;Rh^05ADE00i>CT0
Cdm?05mOG`19BDT0E5AD05EEE@1PH600BDU903<c<`1OGel0MgMg04E5A@1GEeL205UIF@1P02l_;`1T
I6@0E5AD06EUI@1[Jf/0ATI607moO`1SHf<0Rh^;07QhN014A4@0BDU905YJFP0M7Ad0NGUi05UIF@2_
[jl0Fe]K05YJFP1mOGd0Lg=c0861P@1HF5P0Lg=c03lo?`1NGUh0F5QH05IFEP1NGUh0?3`l05IFEP10
@400De=C02LW9`1MGEd0NGUi03dm?@1UIFD0FEUI04=3@`0`<300?Cdm04a<C01VIVH0A4A408:2PP1l
O7`0MgMg03lo?`1@D500DE5A04I6AP1PH6008b<S04m?C`1oOgl0<C4a07QhN01LG5`0>3Ph08B4Q00B
4Q80A4A4055AD@1=CDd0UiNG06e]K@2;Rh/0NGUi06UYJ@1IFET0E5AD04Y:BP1MGEd0@D5103`l?018
B4P051@D05]KF`1=CDd0C4a<02l_;`1OGel0:b/[06=SH`21PH409BDU061PH01ADE40>c/k055AD@1T
I6@0EEEE051@D024Q8@0Bd];0P15ADD09`0o?cl0?Cdm04A4A01dM7@0<c<c03Ti>@14A4@0LG5a02l_
;`17AdL0MGEe07QhN01BDU80MGEe02h^;P1lO7`0@D5107emO@1EEED0NGUi09nOW`0o?cl0@T9204A4
A01FEUH0<C4a04a<C00S8b<0?3`l061PH01kNg/0>SXj05IFEP2QXJ40DU9B05YJFP1<C4`0De=C065Q
H@0@ooooo`003_ooool0=`1]KFd0DE5A05QHF022PX80G5aL04m?C`1UIFD0KFe]092@T027QhL0@T92
04Q8B01;Bd/0Ogmo08V9R@0U9BD0G5aL0861P@19BDT0MWIf06MWI`0P8200?3`l07AdM01OGel0ZZZZ
024Q8@0I6AT0@d=305iNGP1oOgl0Xj>S038b<P0K6a/0>SXj07YjNP1JFUX0DE5A020P800O7al0EeMG
07IfMP15ADD0C4a<02TY:@1ZJVX0S8b<02LW9`14A4@0=3@d02h^;P2ATI40CDe=01/K6`0h>3P00P1;
Bd/0g@1_Kfl08b<S03Ph>01<C4`0N7Qh059BDP20P80061PH0492@P1ADE40L71`05IFEP1oOgl07alO
04=3@`1RHV80EUIF069RHP2RXZ809bLW03Lg=`1:BTX0=SHf05IFEP22PX80820P03<c<`1UIFD0Bd];
05eMG@1WIfL06QXJ03Hf=P1SHf<0;bl_055AD@14A4@09bLW05IFEP18B4P0<c<c05]KF`0`<300D51@
05mOG`1BDU80>c/k04m?C`0[:b/0CDe=0:6QX@1=CDd0<30`04M7A`1:BTX0?clo096AT@1`L700;2`/
051@D01GEeL0GUiN09RHV00h>3P05aLG04U9B@1_Kfl0IFEU07IfMP0J6QX08R8R04e=C@1>CTh0Jf][
06a/K00a<C40>3Ph05YJFP17AdL0MGEe02l_;`0U9BD0>CTi08n?S`1WIfL0@D5102LW9`0N7Qh0EEEE
08>3P`0/;2`0>c/k03Hf=P0k>c/0K6a/08b<S00T92@0;bl_04I6AP1YJFT0YJFU06QXJ00Q8B40>c/k
04];B`1MGEd0`L71055AD@0Q8B40=SHf05QHF01VIVH0]kNg02/[:`1<C4`0BDU904a<C01GEeL0G5aL
01`L701FEUH0I6AT04Q8B01GEeL0PX:203lo?`1GEeL0T92@05UIF@1[Jf/0AdM703Xj>P1/K6`0@T92
06ATI01eMGD0;Bd]03lo?`1JFUX0>SXj08:2PP0g=cL0;Rh^07AdM00_;bl0C4a<03Hf=P0i>CT0Shn?
079bLP0R8R80@D5103Lg=`1=CDd0RXZ:04=3@`12@T80A4A4071`L01PH60092@T03Hf=P1OGel0EUIF
08f=S@1KFe/07alO05QHF01jNWX0EEEE07UiN@25QHD0:b/[05eMG@1eMGD0?Shn06i^KP1_Kfl08B4Q
05MGE`2BTY80C4a<0820P02HV9P0ADE506e]K@2CTi<0BDU906QXJ01ZJVX0;bl_05mOG`2QXJ40>CTi
06QXJ01iNGT0;Bd]06EUI@2BTY80:BTY04U9B@0e=CD0<30`0410@01fMWH08b<S02l_;`1=CDd0;Bd]
05YJFP1OGel0<C4a03Lg=`1=CDd0=CDe079bLP29RHT0=cLg034a<@1BDU80@T9200kooooo0380EeMG
069RHP1ADE40?Shn03Ti>@1EEED0<c<c02`/;01KFe/0A4A4071`L01DE5@0N7Qh05IFEP1cLg<0Cdm?
06ATI01;Bd/0Hf=S02@T901=CDd0=SHf0451@@1IFET06a/K04=3@`0Z:RX0EeMG05iNGP1>CTh0Ogmo
06i^KP1gMgL0HV9R07=cL`1iNGT0JVYZ02d];@0V9RH0:RXZ03Xj>P1UIFD0D51@06QXJ01<C4`0D51@
04U9B@1VIVH0;Rh^05]KF`80@T9200L0ADE502PX:01>CTh0Bd];03`l?00[:b/0De=C0080Bd];01d0
;2`/04Q8B01CDe<0=3@d055AD@1<C4`0BTY:07IfMP13@d<0G5aL04E5A@25QHD0?3`l02<S8`1QHF40
@D51051@D00k>c/0CDe=055AD@0k>c/0BDU904a<C00`<300:RXZ02/[:`1CDe<0DE5A04I6AP0203hn
?P0701TI6@0a<C40Bd];034a<@10@400ADE50410@00204U9B@1J04Q8B01CDe<0JFUY05aLG01CDe<0
KVi^05aLG01JFUX0JFUY06a/K01JFUX0I6AT07UiN@1cLg<0AdM703Lg=`1EEED08b<S055AD@0n?Sh0
Ng]k06MWI`1nOWh0?Cdm08V9R@1OGel0L71`0:BTY01SHf<0Lg=c07]kN`1ZJVX0OWin03Ph>01DE5@0
GemO05ADE01SHf<0=3@d04U9B@0n?Sh0RXZ:09JFUP1gMgL0IVIV0410@01>CTh0@T9204];B`1nOWh0
H61P08:2PP2;Rh/08B4Q08>3P`0i>CT0C4a<04Q8B01IFET0IfMW0410@01BDU80H61P06UYJ@0h>3P0
MWIf075aL@1dM7@0BDU908N7Q`1YJFT0>3Ph051@D00h>3P0HF5Q08>3P`1[Jf/0QhN70820P00j>SX0
?Shn0451@@1VIVH0SHf=04a<C02>SXh0Ng]k04Q8B014A4@0Fe]K0P1ADE40?@1DE5@0A4A404e=C@1>
CTh0D51@07moO`1YJFT0@T9205IFEP0o?cl0Bd];05=CD`12@T80Bd];05ADE01YJFT0D51@05eMG@16
ATH0ADE503hn?P19BDT0I6AT051@D01HF5P0=CDe08:2PP1lO7`0H61P03De=@1aLG40C4a<071`L020
P800<S8b05EEE@28R8P0TI6A07emO@1]KFd0LG5a06IVIP1kNg/0ADE506ATI01>CTh0DU9B079bLP12
@T80AdM703<c<`1>CTh0FEUI07=cL`1QHF40E5AD04I6AP1EEED0Bd];075aL@0O7al00P1EEED05P2T
Y:@0CTi>051@D01cLg<0Cdm?05eMG@0h>3P0BTY:04U9B@1hN7P0@d=304i>CP0e=CD0DE5A051@D01:
BTX0S8b<04Y:BP1CDe<0B4Q8061PH01cLg<@ooooo`003_ooool0o`2NWYh0Xj>S06ATI01@D500L71`
07AdM023Ph<0?3`l061PH02HV9P0BTY:06e]K@14A4@0DU9B06EUI@1SHf<0BDU905ADE00h>3P0Cdm?
055AD@0e=CD0D51@03dm?@12@T80VYZJ069RHP0Q8B40LG5a02h^;P1[Jf/0XZ:R04i>CP0W9bL0CTi>
03Ph>028R8P0KFe]02h^;P0Q8B40?3`l04M7A`1EEED0Kfm_01HF5P0^;Rh0Ph>307=cL`27QhL0G5aL
024Q8@0k>c/0GUiN03Xj>P2>SXh0@410038b<P1RHV80A4A404Y:BP1TI6@06ATI04=3@`1]KFd0@D51
06a/K02:RXX09bLW02h^;P1SHf<0>CTi03`l?00g=cL07QhN03Xj>P1EEED0>3Ph051@D00e=CD0=cLg
04i>CP24Q8@0<C4a05IFEP0h>3P0Cdm?07inOP2DU9@05QHF03Hf=P1OGel0EeMG09BDU01ZJVX05ADE
04Y:BP1IFET0D51@09>CT`0o?cl0:2PX03/k>`1@D500IfMW07AdM00O7al06a/K02d];@1fMWH0De=C
069RHP0K6a/0:b/[03Xj>P1gMgL0GEeM04];B`0S8b<0@d=305UIF@1bLW80ADE5055AD@0f=SH0Fe]K
05YJFP1:BTX0>3Ph07EeM@0j>SX0IFEU06UYJ@0L71`0=SHf04I6AP1DE5@0QXJ6059BDP0S8b<0=CDe
03dm?@1XJ6P0WIfM02`/;00W9bL0;bl_05MGE`13@d<0Rh^;01XJ6P0a<C40?clo04U9B@1DE5@0JFUY
069RHP0N7Qh0@D5106ATI011@D40O7al04e=C@0[:b/0Fe]K07inOP0f=SH0F5QH03@d=00_;bl0GUiN
05IFEP0e=CD0GEeM01@D500f=SH0H61P02PX:012@T80I6AT01lO7`1CDe<0VIVI04i>CP1[Jf/0@D51
03lo?`2NWYh0UIFE03@d=01XJ6P0=cLg08:2PP2j^[X0>c/k03Hf=P1YJFT0A4A40:n_[`0m?Cd0:RXZ
05EEE@1TI6@0Shn?02`/;00V9RH0IVIV08j>SP22PX80MGEe02LW9`14A4@0M7Ad05ADE02FUYH0HF5Q
03Xj>P2CTi<0LG5a03Xj>P1TI6@08B4Q05ADE01gMgL0@D5107alO01RHV80;Bd]04Q8B02<S8`0@d=3
06ATI01@D500?3`l05eMG@1mOGd0@41004Y:BP0g=cL0=3@d04Y:BP2@T900?3`l04U9B@0i>CT08R8R
051@D02g]kL0G5aL0410@01FEUH0CTi>079bLP2JVYX0>3Ph03hn?P005`1WIfL0LG5a09ZJVP14A4@0
92@T03De=@1kNg/0IFEU08N7Q`1LG5`0@D51055AD@1UIFD0EeMG06UYJ@1RHV8071`L02`/;01jNWX0
FEUI05eMG@11@D408B4Q00kooooo02`0HV9R07alO01VIVH0BTY:04I6AP1UIFD0HF5Q05=CD`1_Kfl0
CDe=03hn?P1=CDd0MWIf05QHF01fMWH0Kfm_061PH01@D500N7Qh04m?C`1VIVH0De=C05iNGP1QHF40
<30`05EEE@0a<C40EUIF071`L01NGUh0Kfm_059BDP1MGEd0@41006a/K027QhL0MgMg04=3@`0[:b/0
:2PX04A4A01UIFD0?clo06UYJ@80@T9202L0BDU904a<C00L71`0ADE503lo?`11@D40B4Q804Y:BP1L
G5`0AdM704Q8B013@d<0HF5Q05YJFP1LG5`0>CTi051@D027QhL0@41006m_K`17AdL0A4A407moO`13
@d<0Jf][0492@P1MGEd0?Shn02/[:`1`L700@41006IVIP11@D40@T920410@00f=SH0ATI604A4A00k
>c/00P0_;bl04`1:BTX0BDU905MGE`18B4P0FUYJ02d];@0a<C40GemO04i>CP1=CDd0Cdm?03hn?P1W
IfL0De=C05EEE@1CDe<0EEEE03dm?@0i>CT00P1CDe<204M7A`1Y05]KF`1JFUX0D51@0451@@1?Cdl0
?Cdm03`l?01EEED0>CTi05eMG@0g=cL0RHV908b<S01lO7`0EUIF092@T01LG5`0M7Ad096AT@27QhL0
[:b/07AdM01RHV80H61P059BDP1PH600EeMG03<c<`16ATH0=CDe05EEE@15ADD0MGEe08V9R@1lO7`0
Ph>305ADE01?Cdl0GUiN075aL@1WIfL0CDe=092@T01oOgl0:b/[06a/K01<C4`0UYJF05QHF01YJFT0
I6AT051@D01fMWH0LG5a071`L00d=3@0Ogmo05aLG018B4P0D51@05aLG01^KVh0De=C065QH@0V9RH0
C4a<07EeM@1?Cdl0AdM704E5A@0f=SH0@D5104Y:BP0n?Sh0FUYJ02PX:02_[jl0SXj>05IFEP0k>c/0
PH6107AdM011@D40Hf=S02`/;01WIfL0HF5Q05ADE02?Shl0TI6A05YJFP1mOGd061PH05YJFP1`L700
>SXj069RHP18B4P0RXZ:05aLG01bLW80GUiN05UIF@0j>SX0EUIF0080DU9B0100G5aL02<S8`16ATH0
E5AD05QHF00X:2P0HV9R030`<01VIVH0QHF502DU9@1GEeL0ATI603/k>`0g=cL0C4a<0P0d=3@0:`1J
FUX0;Bd]03De=@0d=3@0>3Ph04A4A00];Bd0D51@03Hf=P15ADD0@41004M7A`0m?Cd0ATI60820P01h
N7P0NGUi0:ZZZP11@D40SXj>06][J`2`/;00Lg=c0:>SX`2XZ:P0@D510492@P1<C4`0I6AT02d];@12
@T809RHV04e=C@18B4P0Bd];03<c<`13@d<0Kfm_0410@01=CDd0Bd];07alO01HF5P04?ooool000ko
oooo0?l0GUiN08N7Q`1dM7@0EEEE069RHP1jNWX0QhN703Hf=P0i>CT0JFUY06i^KP26QXH0=3@d03lo
?`15ADD0Lg=c09BDU01iNGT061PH03<c<`1;Bd/0C4a<0:6QX@1ZJVX0@d=303`l?01_Kfl0L71`0=;B
dP12@T80AdM704I6AP1EEED0EUIF096AT@0X:2P0<S8b06IVIP0l?3`0EeMG05QHF00G5aL07QhN06a/
K00g=cL0GemO02TY:@0V9RH0?Shn05=CD`0l?3`0Cdm?028R8P0W9bL0IVIV071`L00b<S80?3`l0492
@P1DE5@0Lg=c04i>CP0/;2`0EeMG03Ph>01GEeL0YZJV051@D00G5aL0De=C05mOG`2@T900IfMW01XJ
6P0S8b<0D51@05YJFP2OWil0JVYZ01PH600e=CD0Fe]K06][J`3;bl/0<C4a028R8P0f=SH0EUIF03dm
?@1oOgl07alO03De=@1NGUh0M7Ad03@d=01PH60051@D034a<@1KFe/0Hf=S03Xj>P1YJFT0?3`l061P
H01XJ6P0ATI602TY:@1;Bd/0>3Ph06IVIP24Q8@0>SXj01/K6`0o?cl0=CDe06MWI`2VYZH09bLW03/k
>`1FEUH0FEUI05IFEP2GUiL08B4Q03`l?01RHV80ADE505aLG02j^[X0:2PX05eMG@2EUID0De=C06QX
J01FEUH09RHV05aLG01GEeL0=SHf05]KF`1cLg<03Ph>03`l?01[Jf/0820P04];B`0g=cL0820P05UI
F@1:BTX09RHV05eMG@0_;bl0:BTY04Q8B028R8P09RHV04m?C`0a<C40@d=3065QH@28R8P0:BTY05IF
EP13@d<0MWIf0:^[Z`21PH40<S8b05YJFP18B4P0FUYJ0:j^[P0/;2`06a/K04A4A01<C4`0I6AT08b<
S00M7Ad06a/K04M7A`1^KVh0cLg=03hn?P0_;bl0@T9209:BTP1KFe/0AdM702d];@1WIfL0Ph>306m_
K`1@D500@T92038b<P1aLG40H61P04m?C`1KFe/0<30`08V9R@1LG5`0?clo04m?C`0d=3@0GEeM08J6
QP0T92@0@d=30451@@0e=CD0NWYj09BDU00];Bd0BDU905eMG@1RHV80YJFU02l_;`0f=SH0IfMW05=C
D`2SXj<0/k>c02<S8`0b<S80KFe]04];B`2f][H0IFEU024Q8@0a<C40O7al09RHV02SXj<0Bd];030`
<00g=cL0IVIV06EUI@1jNWX0@D51038b<P0h>3P0LG5a07alO029RHT0@41003`l?01;Bd/0N7Qh065Q
H@1JFUX001L0:b/[02d];@1CDe<0Q8B403Ti>@16ATH0CTi>03@d=00i>CT0H61P03Ti>@0c<c<0ADE5
03Ti>@1UIFD0P82002HV9P0U9BD09bLW04];B`1kNg/0L71`01lO7`0>ooooo`0H07emO@1VIVH0DE5A
05ADE01LG5`0EEEE0492@P1<C4`0T92@04e=C@17AdL0FEUI08b<S01ADE40IVIV04m?C`1]KFd0FEUI
065QH@0n?Sh0Bd];0451@@16ATH0F5QH0P19BDT0H`0i>CT0CDe=05mOG`1;Bd/0EeMG06=SH`1HF5P0
B4Q804a<C01ADE40IfMW0492@P16ATH0A4A403hn?P15ADD0AdM705YJFP19BDT0Cdm?055AD@1DE5@0
820P0492@P1IFET0GEeM04Q8B00T92@0Cdm?04U9B@16ATH0BTY:04i>CP1HF5P0DE5A04I6AP18B4P0
KVi^06m_K`2<S8`0Cdm?04M7A`2CTi<0ATI607IfMP11@D40JVYZ04];B`1NGUh0IfMW03`l?01LG5`0
@D5104E5A@0m?Cd0D51@04a<C01?Cdl0>SXj03Ph>00Z:RX0D51@05YJFP1>CTh0=3@d04m?C`0V9RH0
;bl_04Y:BP0d=3@0CDe=059BDP1LG5`0Hf=S04e=C@1LG5`0FEUI06=SH`1PH600@d=306ATI01MGEd0
FUYJ069RHP1_Kfl0KFe]06][J`1;Bd/0E5AD0451@@0i>CT0BTY:03@d=01BDU80@D5106EUI@1PH600
J6QX04i>CP0207AdM02;08b<S02HV9P0F5QH04m?C`1SHf<0JFUY07AdM00k>c/0EUIF06MWI`0l?3`0
CDe=0492@P1RHV80?clo08f=S@2:RXX0N7Qh05QHF01:BTX0Fe]K05QHF026QXH0TI6A051@D01ADE40
G5aL02d];@1XJ6P0>c/k0410@00b<S80ADE5051@D00h>3P0Cdm?05IFEP1GEeL09bLW03lo?`1=CDd0
BDU903hn?P1?Cdl0G5aL038b<P11@D40?Cdm06YZJP29RHT0Cdm?07inOP1kNg/0=SHf03<c<`19BDT0
HF5Q07UiN@0g=cL0G5aL05UIF@1mOGd0GUiN06ATI01ADE40FEUI061PH00i>CT0HF5Q07]kN`19BDT0
cLg=08b<S00i>CT0Shn?030`<01BDU80IVIV06ATI02<S8`0G5aL06i^KP1WIfL0CTi>04Y:BP1dM7@0
ADE504a<C01aLG40Q8B40861P@0g=cL0T92@08V9R@1]KFd0ATI6075aL@1>CTh0DU9B05eMG@0L71`0
@41004];B`1?Cdl0BTY:05=CD`19BDT0Bd];06=SH`0V9RH0@d=305ADE00o?cl0GemO03hn?P1TI6@0
GUiN04M7A`1?Cdl0LG5a051@D01CDe<0EUIF06=SH`1QHF40RXZ:03Lg=`1kNg/0E5AD09:BTP1QHF40
CDe=06EUI@1CDe<0IVIV02l_;`0`<300>3Ph05aLG00204E5A@0:04Y:BP1SHf<0@d=305MGE`2?Shl0
@d=305UIF@17AdL0CDe=06EUIA3ooooo000>ooooo`0705QHF01GEeL0PH6108V9R@0o?cl0EeMG07em
O@0207]kN`1/06=SH`1>CTh0D51@06MWI`2CTi<0YjNW01lO7`16ATH0IVIV05=CD`1YJFT0?clo02LW
9`0n?Sh0BDU9051@D01IFET05ADE03De=@1mOGd0LG5a03@d=015ADD05aLG03lo?`2UYJD0Z:RX02`/
;016ATH0;2`/05eMG@22PX80;2`/01LG5`18B4P0De=C09RHV01OGel05QHF02XZ:P0o?cl0NGUi09:B
TP0M7Ad0;bl_06EUI@1QHF40G5aL0410@00M7Ad0DU9B07AdM01cLg<0G5aL03Ph>00M7Ad0=CDe06AT
I01KFe/0@D510492@P0E5AD0<S8b06MWI`0T92@0;Rh^04m?C`0P8200@d=306ATI00F5QH0<S8b02`/
;00b<S80U9BD06=SH`0H61P0=3@d038b<P0M7Ad0Lg=c03Lg=`0O7al0>c/k03<c<`0h>3P0[:b/02<S
8`0K6a/0;Bd]055AD@1BDU80W9bL05QHF00b<S80BDU904m?C`1WIfL0MgMg030`<00Z:RX0J6QX08:2
PP1;Bd/0RXZ:03Ti>@0];Bd0M7Ad0P1BDU80W`1ADE40;2`/03Hf=P1JFUX0EeMG05EEE@1RHV807AdM
03<c<`1ZJVX0DE5A05QHF01BDU80=3@d0861P@2?Shl0<30`051@D01BDU80=CDe0820P02g]kL08b<S
03Lg=`1JFUX0@D5108n?S`1CDe<09BDU05aLG018B4P0RXZ:0:RXZ01BDU806ATI03Hf=P1YJFT0GUiN
092@T00S8b<0:2PX03lo?`1PH600JFUY0:n_[`0Z:RX0:2PX06MWI`21PH40@T92051@D00O7al0CDe=
07AdM01RHV80@d=304e=C@0];Bd0IVIV08B4Q01<C4`0<S8b05EEE@0l?3`0O7al079bLP0Q8B40GEeM
03`l?01DE5@0HF5Q02XZ:P1?Cdl0FEUI061PH01mOGd0GEeM020P801KFe/0@T9206i^KP1cLg<0;Rh^
04i>CP14A4@0GEeM09fMW@1;Bd/092@T07MgM`0k>c/0Kfm_03Ti>@10@400AdM708N7Q`1UIFD0G5aL
01hN7P1QHF40OWin05UIF@1DE5@0:RXZ0492@P1[Jf/0QXJ604];B`1LG5`0B4Q804];B`23Ph<0QXJ6
03Xj>P0`<300?clo03hn?P1iNGT0MGEe02HV9P0l?3`0GUiN03hn?P2IVIT0Vi^K024Q8@0d=3@0F5QH
03hn?P22PX80IFEU02<S8`15ADD0BDU905=CD`22PX80?Cdm02LW9`1<C4`0DU9B03Hf=P1NGUh0I6AT
03Hf=P13@d<0CTi>04m?C`1JFUX0IFEU02<S8`1?Cdl0LW9b04M7A`1OGel0Lg=c02@T901^KVh0JFUY
034a<@0>ooooo`1?05IFEP1KFe/0F5QH04];B`0_;bl0GemO05ADE00j>SX0K6a/04a<C01XJ6P0I6AT
06e]K@19BDT0OWin07=cL`1PH600B4Q807UiN@0^;Rh0EeMG05=CD`1>CTh0IfMW03hn?P1SHf<0A4A4
05]KF`1IFET0GemO07=cL`1IFET0PH6106UYJ@1gMgL0OWin06EUI@0f=SH07AdM01hN7P12@T80E5AD
0410@01?Cdl0@41005EEE@0k>c/0CTi>01TI6@16ATH0<30`03Hf=P10@400;Bd]04i>CP1FEUH0C4a<
03dm?@1MGEd0MgMg061PH01?Cdl0GUiN0820P00i>CT0EUIF04E5A@11@D40P82007YjNP1SHf<0<S8b
04e=C@1CDe<0<S8b05eMG@18B4P0Jf][03Xj>P020410@00D038b<P0o?cl0AdM703dm?@0W9bL0:b/[
0492@P14A4@0B4Q804];B`1@D5007AdM030`<01LG5`0?Cdm0410@01BDU80A4A404m?C`13@d<305aL
G00S051@D01JFUX0I6AT059BDP1;Bd/0IfMW05QHF01TI6@0Lg=c06YZJP1YJFT0De=C03lo?`1/K6`0
@D51065QH@0j>SX0MgMg09NGU`2SXj<0IVIV08V9R@1WIfL0UiNG09RHV01DE5@0FUYJ05mOG`1^KVh0
Jf][04A4A017AdL0GUiN03/k>`1?Cdl00P19BDT02P0`<300O7al06ATI01^KVh0F5QH04];B`1lO7`0
M7Ad059BDP1NGUh205aLG01M05ADE00R8R80P8200410@01IFET0A4A405mOG`1TI6@0@T9205mOG`1`
L700HV9R024Q8@15ADD0C4a<04Y:BP0h>3P0@T9206EUI@1<C4`0Cdm?02LW9`0e=CD0MWIf0451@@1G
EeL0IFEU030`<00e=CD0@d=304a<C01IFET0820P07AdM01RHV80@D510410@019BDT0CDe=03Ph>015
ADD08B4Q04i>CP1;Bd/0>CTi08Z:RP1/K6`0@D5105eMG@0K6a/0A4A4051@D00^;Rh0Fe]K03dm?@1E
EED0Cdm?05EEE@16ATH0Cdm?02/[:`16ATH0@41004I6AP17AdL05aLG03`l?01RHV80Bd];02<S8`1?
Cdl0>CTi06ATI028R8P07alO04E5A@1GEeL0ATI603Hf=P1@D500B4Q80492@P1TI6@09BDU02l_;`0f
=SH0Cdm?07moO`17AdL0ADE505IFEP16ATH0=cLg0080BTY:01h0B4Q805aLG01PH600SHf=0:f][@0[
:b/0QhN707MgM`2i^KT0IVIV06YZJP1oOgl0<30`03Ti>@16ATH0<30`02XZ:P1=CDd0?Cdm0492@P0i
>CT0F5QH04Q8B019BDT0Jf][04I6AP1@D500BDU905IFEP1QHF4@ooooo`003_ooool04@27QhL0VIVI
07EeM@1QHF40GemO07EeM@1RHV80DE5A07inOP2KVi/0EUIF071`L00g=cL0CDe=08R8R01hN7P0De=C
0080@D5100T0Q8B408R8R00f=SH0I6AT03/k>`1HF5P0U9BD05QHF00j>SX00P16ATH0C`1[Jf/0KFe]
02<S8`0f=SH0>CTi05MGE`1KFe/0CTi>024Q8@0];Bd0@T9204a<C01CDe<0FUYJ02<S8`1>CTh0IFEU
04M7A`1[Jf/0DU9B02h^;P1MGEd0EUIF051@D01dM7@0:2PX04E5A@1XJ6P0=CDe04E5A@1UIFD0CDe=
051@D01BDU80?3`l05=CD`1mOGd0<30`03`l?01FEUH0>c/k05MGE`1LG5`0AdM704Q8B01LG5`0=CDe
04=3@`18B4P0=SHf06ATI01RHV80:RXZ05MGE`11@D40@41007emO@1QHF4061PH03lo?`13@d<0@D51
09NGU`19BDT06ATI04I6AP0n?Sh0FUYJ0861P@0];Bd07AdM03lo?`1PH600EeMG079bLP0U9BD0;bl_
05MGE`1hN7P00P1ADE40>`0V9RH0=SHf06YZJP2BTY80?Shn0410@00R8R80ATI605mOG`11@D40=CDe
05EEE@0d=3@0Lg=c07moO`0_;bl0>CTi04];B`12@T80OGem071`L00Y:BT0?3`l05EEE@15ADD0QhN7
05MGE`0c<c<0@T9206e]K@1@D500IfMW03Lg=`0[:b/0De=C08R8R00o?cl0Kfm_01lO7`0`<300HF5Q
079bLP0f=SH0E5AD05MGE`1@D500OGem07=cL`0m?Cd0O7al05=CD`0n?Sh0QXJ60861P@0n?Sh0Hf=S
04M7A`1FEUH0Ng]k0080BDU906X0MGEe0410@012@T80MgMg02@T901=CDd0G5aL0451@@1RHV80R8R8
0410@01/K6`0HF5Q0451@@2JVYX0QhN704I6AP1JFUX0<30`07QhN01cLg<09RHV04U9B@1QHF40Jf][
06QXJ00];Bd0@T92065QH@1DE5@0OWin01TI6@0Z:RX0FUYJ06i^KP19BDT0GemO03Ph>01bLW80JVYZ
038b<P1JFUX0BDU903hn?P2BTY80Hf=S04U9B@21PH40?Cdm061PH016ATH0>SXj079bLP1OGel0ADE5
079bLP1mOGd0?3`l07alO01fMWH0CDe=06EUI@1ZJVX0=CDe04e=C@1KFe/0CDe=075aL@2QXJ40AdM7
04];B`1GEeL0ADE505EEE@2=SHd0CDe=04M7A`10@400D51@08R8R02:RXX0?clo05iNGP1=CDd0Bd];
07QhN01PH600=cLg02l_;`1[Jf/0ADE5079bLP1_Kfl09bLW038b<P1TI6@0F5QH05EEE@11@D40<c<c
0492@P1cLg<0E5AD06=SH`15ADD0@d=33_ooool02@1YJFT0HF5Q05ADE01GEeL0FEUI069RHP1ADE40
A4A408N7Q`02055AD@0Q05eMG@25QHD0Kfm_06EUI@15ADD0IFEU069RHP1OGel0=cLg05=CD`1;Bd/0
DU9B05UIF@0j>SX0D51@04A4A01<C4`0LW9b04Q8B01NGUh0FEUI05EEE@0i>CT0E5AD05]KF`1MGEd0
@D5102d];@0`<300?Cdm04e=C@13@d<0E5AD0080?clo0340HF5Q05mOG`0R8R80C4a<059BDP1EEED0
D51@03Ti>@1GEeL0BTY:051@D015ADD0D51@04i>CP1@D500Bd];04M7A`1WIfL0E5AD06][J`1CDe<0
@d=3061PH01DE5@0BDU902`/;01=CDd0B4Q803Xj>P1EEED0?Cdm059BDP0h>3P0;Bd]03Ph>014A4@0
?Shn03lo?`0l?3`0;Rh^030`<014A4@0DU9B0492@P17AdL0Bd];02DU9@0`<300B4Q80080?3`l00D0
CTi>05EEE@1<C4`0Bd];05IFEP0205=CD`2605IFEP1MGEd0JVYZ05mOG`1;Bd/0F5QH055AD@1bLW80
Kfm_04m?C`1JFUX0BTY:0492@P1PH600?Shn05EEE@12@T80Lg=c092@T02JVYX0IFEU09jNWP1_Kfl0
RHV90;fm_@1mOGd0Ti>C06a/K01TI6@0NGUi055AD@1LG5`0IVIV03Lg=`1EEED0<S8b04e=C@0g=cL0
KFe]05]KF`1iNGT0I6AT05UIF@0l?3`0JVYZ06][J`1eMGD0?Shn04e=C@1UIFD0=cLg06][J`0l?3`0
HV9R05MGE`1?Cdl0GemO03`l?01KFe/0A4A405]KF`0[:b/0DU9B04i>CP0m?Cd0@D5104M7A`1KFe/0
CTi>05UIF@0];Bd0ATI607MgM`1GEeL0FEUI05eMG@0Y:BT0<30`04A4A01<C4`0EeMG028R8P1aLG40
J6QX04a<C015ADD0CDe=05IFEP0b<S80B4Q801LG5`1>CTh0CDe=030`<01ZJVX0FUYJ03Xj>P1dM7@0
6ATI051@D01KFe/0?clo071`L01HF5P0IVIV05UIF@1HF5P0CDe=05ADE00g=cL0C4a<04i>CP1NGUh0
G5aL01PH601ADE40JFUY04a<C00U9BD0GUiN030`<01<C4`0GemO010@400o?cl0AdM704Q8B00l?3`0
ATI603`l?00n?Sh0R8R803De=@17AdL204E5A@0V05EEE@0m?Cd0HF5Q05QHF01ZJVX0:2PX051@D01<
C4`0CDe=06QXJ020P800IFEU08N7Q`12@T80RXZ:04i>CP20P800MGEe071`L01YJFT0BDU905MGE`0e
=CD0?Cdm03<c<`15ADD0>3Ph04U9B@15ADD0CDe=0492@P1@D500PH6104i>CP1BDU80DE5A061PH01W
IfL@ooooo`003_ooool0=P1DE5@0NGUi07UiN@1OGel0C4a<06i^KP25QHD0>c/k04Y:BP1CDe<0J6QX
096AT@0n?Sh0<c<c0492@P0h>3P0OWin06MWI`0L71`0:BTY04U9B@10@400Ogmo05EEE@0Z:RX0?clo
04e=C@1YJFT0UiNG02l_;`1DE5@0KVi^0410@01nOWh0Y:BT01hN7P0o?cl0CDe=03dm?@1jNWX0MWIf
01<C4`0c<c<0B4Q8059BDP1[Jf/0=3@d03De=@19BDT0DU9B05MGE`1YJFT0:RXZ01/K6`80LW9b07T0
?Shn024Q8@0P82009RHV05EEE@1PH600CTi>02h^;P0V9RH0ATI606i^KP1HF5P07alO03`l?00a<C40
BDU905eMG@0V9RH0:b/[0492@P0k>c/0X:2P05aLG00E5AD09bLW03Xj>P1<C4`0VYZJ02TY:@0D51@0
>SXj061PH01BDU80LG5a03Ph>00e=CD0;bl_061PH01FEUH0J6QX01HF5P12@T80E5AD071`L00c<c<0
MgMg03De=@0n?Sh0M7Ad05YJFP1FEUH0H61P02TY:@1=CDd0KVi^03Ti>@0c<c<0C4a<038b<P16ATH0
QhN702HV9P0];Bd0A4A4030`<010@400Lg=c01@D500V9RH0DU9B04];B`1VIVH0M7Ad01hN7P14A4@0
G5aL05iNGP1mOGd0J6QX02`/;017AdL0Ng]k04m?C`20P800MgMg03<c<`1CDe<0Cdm?02l_;`2EUID0
7alO034a<@1IFET0Bd];0492@P1`L7007alO02DU9@16ATH0H61P03Hf=P1LG5`06ATI04I6AP1YJFT0
L71`04M7A`1`L700=cLg07=cL`1eMGD0IVIV03<c<`1GEeL0BDU907=cL`2[Zj/0MgMg02@T901MGEd0
BTY:0080QhN705`04a<C02@T901SHf<0IfMW09bLW00h>3P06ATI04m?C`2DU9@0GemO04E5A@0^;Rh0
LW9b07alO013@d<0Xj>S04i>CP1;Bd/0Ti>C03lo?`14A4@0=cLg03Xj>P2GUiL0E5AD0492@P1EEED0
;Rh^028R8P1]KFd09BDU0492@P0X:2P0>c/k05aLG02>SXh0=cLg03dm?@0Y:BT0J6QX0<g=c@1LG5`0
BDU902h^;P0b<S80VYZJ0:FUY@0W9bL0AdM706=SH`1OGel0/[:b09jNWP0/;2`0=CDe06MWI`23Ph<0
R8R802l_;`0T92@0>3Ph06MWI`1gMgL0OWin03hn?P0L71`0?Shn07IfMP21PH40Shn?04A4A00X:2P0
=SHf07=cL`1ADE40MgMg06MWI`16ATH0EEEE05eMG@18B4P0G5aL04=3@`18B4P0FEUI09bLW01UIFD0
BTY:0451@@0_;bl0LG5a08n?S`80BTY:00D0O7al038b<P1hN7P0IVIV03De=@0>ooooo`1105EEE@1T
I6@0Cdm?04Y:BP0k>c/0Fe]K03Ti>@0W9bL0OWin030`<00n?Sh0CTi>061PH01@D500L71`05=CD`1^
KVh0Cdm?079bLP0`<300EeMG04Y:BP16ATH0De=C034a<@1CDe<0=SHf04Y:BP1FEUH0AdM706EUI@1Q
HF40KVi^0451@@1YJFT0Ng]k06ATI00a<C40>CTi034a<@0i>CT0D51@03lo?`12@T80AdM703hn?P0f
=SH0E5AD03De=@16ATH0=SHf03Ph>00j>SX0;Bd]03dm?@0o?cl0?3`l03dm?@1;Bd/0EUIF04=3@`15
ADD0EUIF06e]K@10@4000P1OGel02@1IFET0HF5Q03De=@15ADD0DU9B05UIF@0f=SH07alO04m?C`02
04Y:BP0C02`/;00_;bl0?Shn04I6AP17AdL0ADE5030`<00P8200<S8b04];B`1EEED0@T9204E5A@0m
?Cd06QXJ051@D019BDT0=SHf04I6AP0205EEE@26065QH@1LG5`0EeMG04a<C01DE5@0?clo03Ph>01F
EUH0B4Q803Ti>@16ATH0:RXZ0410@019BDT0L71`04M7A`0h>3P0Bd];05=CD`0i>CT0@T9204A4A02;
Rh/0EEEE06i^KP1<C4`0T92@05EEE@2GUiL0][Jf07=cL`2>SXh0JVYZ06UYJ@1hN7P0LG5a06ATI01]
KFd0D51@05eMG@1=CDd0DU9B03hn?P1OGel0BTY:05ADE01?Cdl0EEEE05ADE01_Kfl0F5QH075aL@0j
>SX0CDe=05EEE@0m?Cd0Vi^K03Lg=`1@D500EEEE05eMG@28R8P0=SHf07]kN`1eMGD0MgMg034a<@1@
D500EEEE03lo?`0i>CT0AdM706QXJ011@D40G5aL04m?C`1;Bd/0LW9b05]KF`1MGEd0MgMg03Ti>@0d
=3@0>SXj04];B`19BDT09BDU09^KV`1dM7@0A4A4059BDP0o?cl0AdM703De=@18B4P06QXJ04e=C@1A
DE40=SHf05=CD`1;Bd/0;bl_07moO`0Y:BT0DE5A05YJFP0m?Cd0HF5Q061PH01YJFT0Fe]K05QHF01=
CDd0CTi>03Ti>@16ATH0Cdm?05=CD`1QHF40?clo05MGE`1oOgl0Cdm?01lO7`19BDT0:RXZ05MGE`1U
IFD0:RXZ0492@P15ADD0B4Q80492@P1LG5`204A4A00[05MGE`0f=SH0GEeM04i>CP1TI6@0N7Qh05IF
EP1HF5P0@D5107inOP1BDU80EUIF069RHP1]KFd0MGEe09ZJVP1kNg/0^;Rh04Q8B02=SHd0V9RH0<K6
aP1ZJVX0RXZ:07MgM`0l?3`0C4a<0492@P0i>CT0>SXj071`L01<C4`0CTi>051@D01ADE40@T9203De
=@1mOGd0ATI6055AD@1:BTX0EeMG05iNGP0@ooooo`003_ooool0o`1LG5`0Ph>308f=S@1oOgl0?Cdm
069RHP2KVi/0LG5a0;>c/`1iNGT0EeMG08N7Q`1NGUh0Jf][09NGU`0g=cL0E5AD0:FUY@1[Jf/0Shn?
02l_;`0b<S80EUIF05UIF@1ZJVX0M7Ad01`L7010@400SXj>071`L01@D500:b/[02DU9@1HF5P0KVi^
05eMG@1:BTX0;2`/01lO7`1<C4`0M7Ad0410@00g=cL0BTY:03Xj>P1`L700GEeM02<S8`1SHf<0>CTi
04];B`24Q8@0:b/[02XZ:P14A4@0BDU906ATI01BDU805QHF02l_;`1@D500G5aL0820P00i>CT05aLG
04=3@`1OGel0Cdm?05MGE`17AdL08R8R034a<@1QHF40C4a<05MGE`20P8007QhN034a<@1jNWX0@410
06m_K`1CDe<08B4Q0410@019BDT0:2PX04Y:BP0N7Qh06QXJ04I6AP1>CTh0:2PX05iNGP0[:b/0>3Ph
08^;R`10@400=3@d05=CD`0c<c<0?3`l08:2PP0j>SX0:RXZ05QHF00g=cL0IfMW08F5Q@0d=3@09RHV
06YZJP1?Cdl0PH610<[:bP14A4@0;Bd]05aLG017AdL0Fe]K05iNGP0K6a/08R8R07AdM01CDe<0DU9B
04];B`0G5aL0?clo06][J`1OGel0Xj>S03Ti>@0Q8B40F5QH07EeM@0f=SH0Kfm_02TY:@0W9bL0Bd];
0:2PX00_;bl0BDU903@d=00b<S80Lg=c05=CD`0O7al0EeMG05eMG@1FEUH0Ph>305EEE@0U9BD0DU9B
04a<C02JVYX0^k^k038b<P0I6AT0B4Q8059BDP1kNg/0Y:BT030`<00f=SH0Kfm_06][J`1RHV80CTi>
01<C4`13@d<0N7Qh08^;R`22PX80Ogmo028R8P1IFET0Ph>304M7A`10@400KVi^03Hf=P1BDU80FUYJ
03/k>`1TI6@09bLW06=SH`2=SHd0ADE504I6AP0k>c/0@D51075aL@1KFe/0?Shn04m?C`1=CDd0EUIF
06YZJP0Y:BT0EUIF051@D01kNg/0/K6a03Ph>00O7al0FUYJ04a<C02d];@0BDU902@T900o?cl0TY:B
07YjNP24Q8@0;2`/02@T901bLW80WYjN08f=S@0g=cL0Bd];05IFEP1[Jf/0IVIV075aL@1DE5@0C4a<
07EeM@2:RXX0FUYJ05iNGP1HF5P0CTi>092@T01mOGd0;bl_05=CD`1nOWh0ATI607]kN`2ATI40<S8b
04Y:BP1EEED0<C4a05eMG@1_Kfl092@T059BDP0j>SX0Fe]K07=cL`0107alO0409RHV0P12@T804`1L
G5`0QHF504];B`0`<300@D51061PH01OGel0VIVI09ZJVP0Z:RX0@41005mOG`1?Cdl0L71`075aL@0_
;bl0J6QX061PH00n?Sh03_ooool0H`1YJFT0QhN705iNGP1@D500CDe=06m_K`1SHf<0=SHf069RHP1:
BTX0IFEU06=SH`1mOGd0F5QH07UiN@1]KFd0Hf=S05mOG`1^KVh0DU9B07AdM01=CDd0@d=304i>CP0m
?Cd0DE5A03Hf=P1LG5`0N7Qh05YJFP1jNWX0IfMW06m_K`1SHf<0IFEU07YjNP21PH40ATI604A4A016
ATH0@d=3069RHP1=CDd0FEUI04Q8B01;Bd/0DU9B06ATI00I6AT0GEeM05EEE@1>CTh0DU9B03`l?01H
F5P0EeMG04Q8B00e=CD0EUIF05]KF`1EEED0@410055AD@1fMWH0Bd];06ATI010@400=CDe07UiN@12
@T80De=C02/[:`12@T80?3`l02XZ:P1XJ6P0?Shn06QXJ00j>SX0AdM703dm?@14A4@0?Cdm04A4A010
@400?Shn02`/;01:BTX0ATI604i>CP1@D500ADE502@T900j>SX0G5aL03Ph>00f=SH0CTi>04e=C@02
04U9B@0h04A4A01DE5@0KVi^04a<C01ADE40LG5a08V9R@1GEeL0B4Q804=3@`1VIVH0L71`04I6AP1B
DU80>SXj0451@@1=CDd0=SHf04M7A`0g=cL0CTi>059BDP1lO7`0?3`l0<G5a@1KFe/0H61P06UYJ@18
B4P0FEUI059BDP1LG5`0JVYZ04=3@`1;Bd/0ATI602h^;P12@T80:BTY03De=@0[:b/0E5AD04M7A`1Y
JFT0N7Qh06MWI`0k>c/0DU9B05YJFP1eMGD0>c/k06IVIP1dM7@0?Cdm05aLG01JFUX2055AD@1^04];
B`1oOgl0ADE505MGE`1FEUH0NWYj03Ti>@1ADE40EUIF03Ti>@1ADE40=3@d05UIF@1ADE40JVYZ0451
@@1ADE40H61P06][J`1IFET0G5aL03Xj>P1ADE40IFEU05ADE010@400<S8b05iNGP1UIFD0?3`l03Ti
>@1<C4`0E5AD04M7A`1OGel0;bl_04Y:BP18B4P0>CTi092@T01UIFD0ATI607YjNP0j>SX0L71`07]k
N`13@d<0EEEE06YZJP20P800EeMG06UYJ@1UIFD0Cdm?03Ti>@1JFUX0C4a<051@D01NGUh0:BTY055A
D@1mOGd0D51@030`<01ADE40A4A405aLG01WIfL0:BTY04i>CP1RHV80DU9B055AD@1RHV80DU9B04U9
B@1]KFd0;2`/03Hf=P0e=CD0@d=303hn?P1>CTh0G5aL04a<C017AdL0EeMG08V9R@1aLG40I6AT07Ad
M02XZ:P0[Jf]0<S8b01EEED0P82004Q8B01fMWH0A4A407UiN@1kNg/0>3Ph04A4A01?Cdl0BDU9038b
<P1HF5P0B4Q805IFEP1<C4`205iNGP0703De=@1bLW80C4a<05UIF@10@400Fe]K07IfMP0@ooooo`00
3_ooool0:@2:RXX0WinO06ATI01iNGT0CDe=08^;R`1=CDd0@d=306][J`2EUID0CTi>05QHF00e=CD0
H61P09FEU@19BDT0?clo03Ph>00Y:BT0HF5Q07]kN`0S8b<0AdM703Ti>@1CDe<0SXj>038b<P0Q8B40
EeMG05IFEP2IVIT0A4A401HF5P0d=3@0F5QH06ATI028R8P0QXJ601/K6`1CDe<0Hf=S0080IfMW00T0
K6a/01XJ6P1YJFT0Rh^;05MGE`2_[jl0=cLg02/[:`1LG5`00P1CDe<0:@1hN7P09BDU03Ti>@1OGel0
A4A408B4Q01gMgL05ADE04];B`29RHT0?Shn06IVIP27QhL0=SHf03/k>`1JFUX07QhN051@D00/;2`0
6QXJ03dm?@2;Rh/0<c<c051@D01:BTX0ATI607QhN01fMWH09bLW05=CD`11@D40DE5A08F5Q@0f=SH0
30`<03Hf=P0m?Cd0I6AT07inOP0g=cL03@d=0080CDe=00/0M7Ad065QH@0d=3@0:b/[03`l?01BDU80
GemO03/k>`0A4A40ATI6051@D00204U9B@0302PX:00Y:BT0ADE50080I6AT03L0?clo03Ti>@0Y:BT0
>c/k08R8R01CDe<0;2`/0410@012@T80Z:RX07]kN`0V9RH0:b/[03dm?@1<C4`0/k>c07IfMP0F5QH0
92@T04E5A@1PH600[Zj^05UIF@0Q8B40<C4a06][J`1@D500VYZJ02h^;P0Q8B40A4A407UiN@15ADD0
N7Qh024Q8@19BDT0>SXj05aLG00a<C40Ogmo05MGE`0V9RH0HF5Q079bLP1FEUH0CDe=03Ti>@0l?3`0
Kfm_05iNGP0^;Rh0F5QH03dm?@15ADD0PH610080?clo03L0Fe]K02LW9`0h>3P0TY:B02d];@0^;Rh0
>CTi05]KF`1RHV80VYZJ03Lg=`1BDU80Bd];05ADE02=SHd0=cLg03De=@1[Jf/0E5AD09>CT`0n?Sh0
8B4Q061PH01jNWX0Hf=S069RHP0W9bL0ATI606QXJ01DE5@0EeMG02PX:01>CTh0HF5Q04m?C`10@400
Cdm?02@T901MGEd0Jf][038b<P15ADD0:RXZ03Xj>P1]KFd0C4a<03Ti>@1YJFT0<S8b05=CD`1LG5`0
;bl_061PH012@T80;2`/0080PX:2034092@T055AD@1ZJVX0DE5A09jNWP1ZJVX09BDU04M7A`1VIVH0
Hf=S07AdM02c/k<0=CDe04=3@`1iNGT0ATI608b<S02ATI40<C4a051@D02<S8`0CTi>09JFUP1?Cdl0
=CDe05iNGP1^KVh0I6AT07]kN`0f=SH09RHV04E5A@1nOWh0A4A406e]K@19BDT0;Bd]03De=@1]KFd0
@d=303Xj>P0d=3@0;bl_034a<@1/K6`0?3`l04e=C@0k>c/0:b/[00kooooo01T0P82006EUI@1PH600
Hf=S06][J`1:BTX0<C4a02`/;02>SXh0<S8b03/k>`1:BTX0MgMg05IFEP1ADE40D51@07=cL`1<C4`0
D51@02XZ:P13@d<0>3Ph0492@P1<C4`0?clo0080>c/k00`0BDU906ATI00l?3`0DU9B04Q8B014A4@0
<C4a03lo?`1?Cdl0Jf][03<c<`15ADD203<c<`1I03Ph>014A4@0De=C04=3@`0l?3`0GEeM04i>CP0O
7al0?Cdm06ATI01JFUX0?3`l03<c<`19BDT0ADE505ADE019BDT0EEEE069RHP1OGel0GEeM05aLG02:
RXX0HF5Q0820P01HF5P0AdM708B4Q01bLW80Jf][0492@P1JFUX0Cdm?0492@P1[Jf/0A4A406ATI01E
EED0C4a<04E5A@14A4@0@D5104I6AP0n?Sh0=cLg02PX:00o?cl0?3`l04Q8B017AdL0CTi>03dm?@0n
?Sh0CTi>03lo?`1:BTX0EeMG05EEE@1OGel0GEeM05mOG`1BDU80ATI604a<C017AdL0DE5A04];B`14
A4@0EEEE03hn?P19BDT0Bd];04m?C`17AdL0>SXj03<c<`15ADD0>3Ph04a<C01>CTh0J6QX0492@P1H
F5P0@T9206ATI01<C4`0Shn?09BDU01NGUh00P1UIFD0CP2?Shl0JVYZ0451@@1@D500IVIV03@d=01A
DE40DU9B05=CD`0O7al0Cdm?05mOG`1NGUh0?clo04i>CP0g=cL0E5AD07UiN@2NWYh0BDU90492@P1=
CDd0?3`l07]kN`0h>3P0GemO06=SH`1NGUh0R8R803lo?`1iNGT0IVIV08>3P`0i>CT0FEUI05iNGP14
A4@0@T9205QHF01aLG40>c/k05ADE00k>c/0EUIF08:2PP1LG5`0DU9B05IFEP0f=SH0:b/[03Xj>P1;
Bd/0BTY:01hN7P1hN7P0DE5A03Lg=`0m?Cd0FEUI07moO`0f=SH0Fe]K03hn?P1MGEd0B4Q80410@01Y
JFT0Cdm?03Ti>@1iNGT0:BTY04a<C01OGel0C4a<079bLP1iNGT0L71`06YZJP80De=C0300IfMW0492
@P13@d<0B4Q807inOP1WIfL0;bl_055AD@1hN7P0JFUY0410@01=CDd0<S8b06QXJ01jNWX08R8R04U9
B@1JFUX0Cdm?04];B`1ADE40=3@d0451@@1oOgl0=cLg04E5A@0g=cL0D51@05QHF01PH600Xj>S0451
@@1;Bd/0=CDe04a<C01SHf<0HF5Q06IVIP1iNGT0IfMW08^;R`0W9bL0PX:205UIF@2e]KD0K6a/0:NW
Y`29RHT206MWI`0@03dm?@1EEED0Kfm_08N7Q`1KFe/0FUYJ06ATI01?Cdl0De=C03lo?`1cLg<0CDe=
05mOG`1CDe<0Jf][06m_Ka3ooooo000>ooooo`3c05EEE@2[Zj/0JVYZ05eMG@1aLG40Rh^;04m?C`0c
<c<0A4A4079bLP20P800PX:202`/;00k>c/0H61P06m_K`27QhL0=cLg038b<P1;Bd/0JFUY06EUI@2E
UID0?Shn05IFEP1UIFD0Fe]K04I6AP1<C4`0<30`07QhN01fMWH0B4Q8079bLP23Ph<0:BTY059BDP2J
VYX0EEEE05aLG012@T807alO04Q8B01lO7`0Cdm?06i^KP0a<C40=CDe07alO01_Kfl0EEEE055AD@0i
>CT0IVIV0:2PX013@d<0<C4a030`<012@T80MGEe0:ZZZP0l?3`0:BTY03Ti>@10@400S8b<0;Ng]`0/
;2`0<C4a05YJFP11@D40XZ:R04Y:BP0=3@d0@41004m?C`1DE5@0[Jf]030`<00Q8B40@d=305IFEP1:
BTX0KFe]01TI6@0N7Qh0C4a<06i^KP0_;bl0JVYZ01dM7@0P8200GemO05iNGP0/;2`0<c<c028R8P0c
<c<0IfMW034a<@0K6a/0ADE502XZ:P20P800U9BD01hN7P0T92@0De=C02l_;`1dM7@0D51@00/;2`0e
=CD0IVIV06EUI@1IFET0DE5A02TY:@0Z:RX0Bd];05IFEP1>CTh0HF5Q02HV9P0i>CT0LG5a03`l?01;
Bd/0DU9B02l_;`1VIVH0S8b<03/k>`1DE5@0@d=303@d=01UIFD0KFe]030`<01KFe/0ADE502l_;`1X
J6P0CTi>03Ph>01ADE409RHV04Q8B01cLg<0=3@d03Ti>@1@D500<c<c065QH@1]KFd0EUIF03Lg=`0m
?Cd0<30`059BDP2EUID0I6AT034a<@1DE5@0De=C06=SH`2_[jl0Ng]k03Xj>P1bLW80EEEE07MgM`2i
^KT08B4Q01/K6`1EEED0B4Q807=cL`17AdL05aLG038b<P1gMgL0F5QH08J6QP0O7al0:RXZ05UIF@1K
Fe/0FEUI04=3@`0/;2`0J6QX071`L00/;2`0GUiN03lo?`17AdL0UYJF03Xj>P0f=SH0A4A40410@02E
UID0>CTi034a<@1fMWH0ATI603Ti>@1hN7P07QhN03hn?P1JFUX0Cdm?08B4Q00b<S80@T9205QHF01U
IFD0Hf=S0;no_`0a<C40H61P07]kN`1LG5`0QHF504e=C@0X:2P0>c/k0;6a/@1PH600Rh^;03Hf=P1<
C4`0DE5A09BDU01HF5P0Ng]k02XZ:P0e=CD0A4A405eMG@1CDe<0N7Qh0080:RXZ0240Cdm?0:2PX01g
MgL0Hf=S02LW9`0k>c/0GemO06MWI`0];Bd0@d=303<c<`1ADE40JVYZ071`L00Z:RX0?Cdm03lo?`1:
BTX0VIVI0820P00V9RH0<S8b03hn?P1NGUh0LW9b055AD@0Z:RX0@D510492@P1OGel0MWIf03dm?@0_
;bl03_ooool07P1CDe<0Fe]K05EEE@1:BTX0>3Ph04a<C00d=3@061PH06a/K00b<S80DU9B05EEE@1Y
JFT0CTi>09JFUP1`L700GemO0410@01WIfL0:b/[04Y:BP0h>3P0@41004e=C@0];Bd0ADE503Lg=`1G
EeL0HV9R04M7A`80FUYJ00X0TY:B065QH@1DE5@0Hf=S06YZJP0`<300@D5103Ph>00[:b/0E5AD0P1@
D5002P13@d<0<30`03Ph>01FEUH0;Bd]03`l?016ATH0?3`l03@d=00W9bL203`l?00J03/k>`0S8b<0
B4Q806=SH`12@T80:RXZ055AD@1=CDd0;Rh^04=3@`1GEeL0GUiN05IFEP0g=cL0Cdm?05mOG`1SHf<0
>SXj020P8012@T80AdM703`l?00];Bd0?Cdm03Ti>@0f=SH203/k>`0H02XZ:P0T92@0<30`04m?C`16
ATH0CTi>04Q8B00l?3`09BDU059BDP1OGel0?Shn05ADE01KFe/0>SXj05=CD`1kNg/0HF5Q05MGE`1M
GEd0DU9B05YJFP1RHV80F5QH0P1?Cdl0P`0l?3`0GUiN06][J`1?Cdl0J6QX04A4A00Q8B40G5aL03Xj
>P1DE5@0>CTi08F5Q@1`L700Ph>302d];@2LW9`0SXj>08N7Q`2i^KT0IFEU06m_K`1PH600GEeM075a
L@17AdL0@T9204a<C00d=3@0Bd];01`L700e=CD08R8R05EEE@0b<S80Cdm?04I6AP1?Cdl0CDe=04I6
AP1<C4`0IfMW04A4A01?Cdl0H61P030`<01jNWX0>c/k04=3@`1:BTX0D51@05YJFP0i>CT0EUIF05EE
E@1HF5P0;2`/04a<C01ADE40Cdm?038b<P11@D40EeMG04A4A01LG5`0=SHf051@D01EEED0Fe]K04e=
C@1OGel0>3Ph03Hf=P19BDT0JVYZ04];B`0l?3`0O7al05YJFP0`<300=cLg05eMG@20P800E5AD06e]
K@0W9bL0EeMG05aLG01@D500UiNG07AdM00a<C40NWYj01`L701[Jf/0PX:203Ph>00n?Sh0C4a<05mO
G`15ADD0Fe]K04=3@`0k>c/0;Bd]04Y:BP14A4@0D51@04Q8B00Z:RX0MWIf09^KV`1DE5@0<C4a06=S
H`0m?Cd0G5aL07emO@0[:b/0@41005MGE`1:BTX0@d=304Y:BP1HF5P0ADE5069RHP0Y:BT0A4A404e=
C@1DE5@0EeMG0080D51@01h0A4A405QHF01LG5`0EEEE04a<C01IFET0VYZJ08n?S`1_Kfl0XJ6Q03Lg
=`2SXj<0YJFU0>WYj@1@D500IfMW06QXJ00e=CD0@D5103Ti>@15ADD0<c<c05QHF00k>c/0ATI60410
@01HF5P0NGUi04A4A01gMgL206EUI@03061PH01oOgl0QXJ6013ooooo000>ooooo`0U059BDP1WIfL0
MWIf07inOP0d=3@0Ng]k08J6QP1LG5`0VYZJ06a/K01iNGT0M7Ad03Ph>019BDT0OWin0451@@1GEeL0
JFUY02h^;P1[Jf/0=SHf03De=@1YJFT0Cdm?02`/;01?Cdl0HV9R05IFEP2DU9@0=CDe04U9B@1JFUX0
<c<c07moO`1hN7P08R8R03@d=002051@D00N0:FUY@1nOWh05aLG02DU9@1>CTh0I6AT0<?3``0d=3@0
7AdM030`<01CDe<0TI6A07]kN`0F5QH0@d=30820P01aLG40LW9b03dm?@0n?Sh0Bd];07QhN01_Kfl0
N7Qh02d];@0l?3`0@d=309jNWP2BTY80=CDe0P14A4@0;`1DE5@0MgMg02d];@17AdL0I6AT01XJ6P1I
FET0K6a/01TI6@1BDU80>CTi03lo?`28R8P0ATI602<S8`17AdL08b<S02l_;`2MWId0:b/[02DU9@17
AdL0A4A404E5A@1fMWH07AdM02<S8`0n?Sh0AdM7061PH01dM7@0:2PX038b<P20P800BTY:05MGE`1d
M7@09BDU04a<C02=SHd0?Cdm04];B`1jNWX0:b/[03Ph>01FEUH0:2PX0080Bd];00`0:RXZ02d];@1O
Gel0=CDe05UIF@1;Bd/0C4a<05aLG01YJFT0:BTY03`l?01EEED206IVIP0L04U9B@0Q8B40AdM705aL
G011@D40KVi^06UYJ@0V9RH0CDe=069RHP19BDT0QhN702d];@0D51@0NWYj05]KF`1^KVh0H61P02TY
:@0b<S80CDe=05QHF01JFUX0EeMG01TI6@0e=CD0De=C06e]K@80Bd];02X06a/K03hn?P1ZJVX0Ph>3
04a<C00n?Sh06ATI03`l?029RHT0Lg=c03dm?@13@d<0:BTY06i^KP1cLg<0;Bd]02h^;P19BDT0C4a<
0;6a/@1ZJVX0;2`/04e=C@0f=SH0PH6106i^KP0a<C40De=C055AD@18B4P0YJFU07EeM@0^;Rh0DE5A
03/k>`1OGel0IfMW038b<P1IFET0C4a<04M7A`2FUYH203@d=01606MWI`1CDe<0Ti>C05MGE`0_;bl0
Jf][07YjNP1DE5@0Cdm?02l_;`1CDe<0VYZJ08Z:RP1JFUX0@41003De=@1]KFd0QhN703Lg=`0f=SH0
CTi>04m?C`2<S8`0GUiN02HV9P1MGEd0Fe]K06IVIP27QhL0>c/k02l_;`1>CTh0@41005UIF@26QXH0
A4A402LW9`1;Bd/0@d=305EEE@1mOGd0=cLg02LW9`1CDe<0JVYZ05UIF@2>SXh0<30`02LW9`1ADE40
T92@08n?S`2KVi/0ATI602h^;P1EEED0QhN704m?C`1IFET0Fe]K02PX:01;Bd/0S8b<04m?C`14A4@0
C4a<02XZ:P2ATI40UiNG03@d=0kooooo0?/0FEUI06][J`1_Kfl0EEEE04E5A@1GEeL0C4a<03@d=01a
LG40@41004E5A@11@D40LG5a05aLG01?Cdl0ADE505EEE@1<C4`0GEeM04m?C`1=CDd0A4A4065QH@13
@d<0=3@d04e=C@0n?Sh0CTi>05YJFP1<C4`0Bd];05=CD`1BDU80?clo04a<C018B4P0FUYJ05MGE`1;
Bd/0@41005YJFP1VIVH0EUIF05QHF01>CTh0@D5104];B`1PH6005QHF05aLG01=CDd0DU9B04=3@`0b
<S80DU9B0410@016ATH0;Bd]04i>CP1GEeL0HF5Q03@d=01GEeL0MgMg04I6AP1bLW80D51@03`l?02<
S8`0ATI604a<C00Q8B40>SXj03Hf=P0/;2`0FUYJ03@d=01EEED0BDU90451@@0g=cL0CDe=03`l?01;
Bd/0CTi>03/k>`0U9BD0AdM7051@D00n?Sh0De=C04U9B@0K6a/0;Rh^04];B`1?Cdl0=SHf03hn?P1;
Bd/0B4Q804E5A@17AdL0E5AD06MWI`14A4@0AdM705EEE@2CTi<0GEeM05aLG01OGel0QHF506IVIP1P
H600BDU9059BDP0W9bL0E5AD05=CD`1?Cdl0A4A4069RHP22PX80ZZZZ03@d=02WYjL0V9RH08F5Q@2W
YjL0RXZ:09^KV`1NGUh0JVYZ05]KF`1@D500Ng]k07alO00_;bl0AdM703`l?01BDU80=SHf0451@@1Z
JVX0EeMG04m?C`1JFUX0?Cdm04A4A01mOGd0VYZJ03`l?01/K6`0H61P02/[:`1^KVh0?Cdm065QH@1P
H600ADE506EUI@15ADD0H61P05=CD`1UIFD0;2`/05aLG01BDU80;Rh^04E5A@24Q8@0SXj>059BDP1W
IfL09BDU04A4A01iNGT0GEeM03Ph>01BDU80:b/[03@d=01:BTX0=3@d03Hf=P0W9bL0N7Qh05EEE@0i
>CT09BDU03@d=01>CTh0<30`04a<C00N7Qh0Cdm?04U9B@0[:b/0W9bL07inOP0o?cl0QXJ602PX:00m
?Cd0EeMG059BDP1gMgL0FEUI06MWI`1LG5`0A4A404];B`1?Cdl0<S8b0410@00k>c/0J6QX061PH00D
51@0:b/[051@D01NGUh0=cLg03De=@0Z:RX0?clo05EEE@0K6a/0>c/k04a<C01@D500>SXj03Hf=P0c
<c<0@41007=cL`1<C4`0@D5102TY:@1MGEd0I6AT04E5A@1OGel0CDe=06EUI@1OGel0VIVI075aL@1K
Fe/0FUYJ05QHF00209NGU`0I06YZJP1eMGD0MWIf096AT@1ZJVX0M7Ad08B4Q015ADD0EEEE05QHF01X
J6P0FEUI061PH017AdL0MWIf06EUI@0c<c<0C4a<03Ti>@1WIfL0Cdm?05UIF@1?Cdl0F5QH071`L00@
ooooo`003_ooool0o`1eMGD0PH61065QH@1dM7@0@D5106=SH`1?Cdl0E5AD06m_K`2JVYX0C4a<05UI
F@1HF5P0JFUY092@T00d=3@0A4A406e]K@2<S8`0UiNG03<c<`0Y:BT0?3`l06i^KP20P800Fe]K03<c
<`0];Bd0KVi^05mOG`2NWYh092@T03hn?P11@D40M7Ad06QXJ02?Shl0=3@d0451@@1YJFT0F5QH0492
@P1RHV80=cLg038b<P1<C4`0ADE503@d=01iNGT08b<S0451@@1EEED0;bl_04A4A01=CDd0<C4a059B
DP1CDe<06QXJ02XZ:P16ATH0CTi>05MGE`0J6QX0820P03Ph>01JFUX0Ph>305iNGP1@D5007QhN06AT
I021PH40De=C06a/K02SXj<0:BTY06=SH`1lO7`0>CTi08:2PP1_Kfl0;Bd]05]KF`1SHf<0D51@06IV
IP0U9BD06QXJ07AdM01YJFT0?3`l05MGE`0I6AT0;bl_07UiN@0l?3`0?Cdm04=3@`0[:b/0>c/k0861
P@1BDU80NGUi02d];@0[:b/0JVYZ06][J`10@400@d=303@d=01GEeL0Fe]K09ZJVP1eMGD0>CTi05IF
EP11@D40L71`0:>SX`0U9BD09RHV05MGE`18B4P0XJ6Q07MgM`0M7Ad0De=C065QH@1PH600UiNG03Lg
=`0h>3P0HV9R06MWI`1>CTh0PH61030`<00K6a/0F5QH07AdM00W9bL0H61P034a<@11@D40LG5a05mO
G`0l?3`0AdM703hn?P1DE5@0Ogmo02l_;`0U9BD0CTi>04Q8B01cLg<0WYjN05ADE00K6a/0EEEE05iN
GP1]KFd0Kfm_01dM7@0P8200A4A407MgM`1dM7@0Jf][024Q8@0Y:BT0A4A406=SH`1CDe<0I6AT034a
<@1ZJVX0HF5Q051@D01UIFD0QXJ6038b<P2JVYX0ZJVY04U9B@1TI6@0820P06IVIP26QXH0CDe=075a
L@0b<S809BDU05=CD`1nOWh0BTY:079bLP0Y:BT0=CDe08B4Q018B4P0HV9R02<S8`1CDe<0][Jf07If
MP0`<300A4A404m?C`28R8P0RXZ:03/k>`1QHF40?Shn07IfMP19BDT0:BTY04A4A01aLG40H61P08J6
QP16ATH0CTi>06EUI@2DU9@0T92@08j>SP1HF5P0?Cdm07EeM@2VYZH0JVYZ09:BTP1<C4`0ADE507Mg
M`1jNWX0IVIV0;no_`1YJFT0<S8b07MgM`1dM7@0De=C0:^[Z`1JFUX06a/K05mOG`1iNGT0?3`l071`
L00];Bd0=3@d065QH@005`1CDe<0B4Q805eMG@0H61P0<c<c08b<S01^KVh0;Bd]051@D018B4P0A4A4
092@T02GUiL0;bl_03`l?01@D500MWIf075aL@1IFET092@T03@d=01KFe/0HF5Q00kooooo02T0HV9R
06MWI`1LG5`0IFEU06m_K`1DE5@0>c/k04U9B@2NWYh0De=C04E5A@1IFET0WIfM08:2PP1XJ6P0E5AD
075aL@1UIFD0HV9R0410@01DE5@0>c/k04A4A01LG5`0EeMG04=3@`0W9bL0E5AD08f=S@13@d<0Bd];
065QH@1ZJVX0ATI60451@@1IFET0RHV904E5A@1:BTX0=3@d02h^;P0203lo?`0V05UIF@13@d<0:RXZ
055AD@1@D500;2`/0451@@1<C4`0DU9B0492@P0c<c<0?Cdm03<c<`1>CTh0Cdm?04];B`1HF5P0HF5Q
05IFEP1HF5P0MgMg05IFEP22PX80GEeM04Y:BP1bLW80GUiN05UIF@0i>CT0FEUI05EEE@1FEUH0Bd];
03Hf=P1?Cdl0;Rh^0410@00j>SX204A4A00=04=3@`0^;Rh0=3@d02TY:@0k>c/0;bl_0492@P0k>c/0
@D5103<c<`0j>SX0C4a<03Xj>P020410@01M05UIF@10@400C4a<05YJFP1SHf<0E5AD04E5A@1;Bd/0
BDU903Lg=`0j>SX0?Shn01TI6@11@D40@d=305EEE@1:BTX0:BTY01hN7P19BDT0;bl_05=CD`15ADD0
JVYZ05iNGP1DE5@0=CDe05mOG`1cLg<0TY:B09NGU`1IFET0J6QX06MWI`2][Jd0RXZ:04Q8B011@D40
EUIF02/[:`11@D40:RXZ02/[:`0e=CD0@d=303Lg=`0j>SX0=3@d05YJFP16ATH0@d=303Xj>P1>CTh0
=CDe04Q8B01ADE40CDe=09^KV`0j>SX0>3Ph04];B`1HF5P0NGUi03Ti>@1OGel0IfMW07]kN`0i>CT0
@D5103dm?@1:BTX0Fe]K04I6AP1;Bd/0?Shn04U9B@18B4P0FUYJ05=CD`1@D500E5AD06QXJ01>CTh0
>CTi0492@P1:BTX0GEeM04A4A01OGel0AdM704A4A015ADD0>CTi0080De=C04<0JVYZ02@T901:BTX0
EeMG0492@P1PH600F5QH03Hf=P1BDU80?3`l04U9B@1?Cdl0@T9204U9B@1LG5`0Fe]K051@D01?Cdl0
BTY:05=CD`17AdL0@T92055AD@17AdL0ADE501DE5@0Z:RX0KVi^03`l?00Q8B40IfMW03Ph>015ADD0
FUYJ02TY:@15ADD0>c/k04I6AP0m?Cd0B4Q804i>CP1ADE40P82003@d=021PH40EeMG04I6AP1;Bd/0
E5AD05YJFP0h>3P0EUIF034a<@17AdL0@410051@D01WIfL0PH6104m?C`1fMWH0KFe]08J6QP2ATI40
a/K60:f][@2`/;00OGem0080B4Q80100=3@d04M7A`1;Bd/0EeMG059BDP1HF5P0EUIF05ADE01LG5`0
<C4a04Y:BP1ADE40AdM704=3@`1LG5`0LG5a4?ooool000kooooo0140J6QX0;:b/P1SHf<0Kfm_071`
L02DU9@0J6QX03`l?01SHf<0Jf][09>CT`1nOWh0;bl_03dm?@1>CTh0FUYJ08B4Q00202h^;P0V05iN
GP1VIVH0B4Q806MWI`0b<S80@D5108j>SP1FEUH0F5QH05ADE00`<300HF5Q075aL@0o?cl0EUIF04Y:
BP0N7Qh0BTY:059BDP0/;2`0GemO04U9B@0/;2`0H61P05]KF`0i>CT0HF5Q030`<00l?3`0OGem05=C
D`16ATH0CTi>03Xj>P1OGel0Ph>303De=@18B4P204=3@`3K05YJFP1aLG409BDU03/k>`1KFe/0@410
05YJFP1NGUh08B4Q02d];@15ADD0ATI60861P@0i>CT06a/K03dm?@1>CTh0E5AD09>CT`0g=cL0820P
03lo?`1IFET0=3@d061PH00[:b/0?Cdm05YJFP1DE5@09BDU05=CD`0Y:BT0CTi>07emO@0m?Cd0<30`
04i>CP13@d<0EEEE079bLP0c<c<0<S8b03lo?`0k>c/0QXJ605IFEP0R8R80?3`l05aLG00^;Rh0M7Ad
05mOG`0W9bL0:b/[05ADE01`L700LG5a03lo?`0P8200@T9205iNGP1GEeL0@41006i^KP0I6AT0D51@
07QhN012@T80IFEU08J6QP0X:2P0N7Qh06i^KP10@400Hf=S04i>CP0d=3@0N7Qh05aLG00/;2`0Jf][
06e]K@0_;bl0Hf=S05EEE@0i>CT0G5aL04A4A01?Cdl0M7Ad02l_;`0n?Sh0MgMg04I6AP1ADE40PX:2
04Q8B00i>CT0Hf=S05MGE`1<C4`0NWYj04m?C`0n?Sh0De=C04m?C`1oOgl0RXZ:03dm?@16ATH0I6AT
03@d=01XJ6P0T92@03<c<`0a<C40FEUI03<c<`1hN7P0KVi^02@T9018B4P0PH61065QH@2;Rh/0<S8b
04E5A@1^KVh0GemO069RHP16ATH0CDe=08f=S@1aLG40CDe=061PH01?Cdl0GEeM0:NWY`1NGUh0>c/k
04A4A018B4P0Xj>S03lo?`0e=CD0L71`04a<C01ADE40KVi^02TY:@1CDe<0IVIV04U9B@2:RXX0IVIV
03hn?P1QHF40>c/k075aL@2^[Zh0?clo05QHF01NGUh0CTi>08f=S@1NGUh0<c<c05UIF@1nOWh0GEeM
0::RXP1;Bd/0>c/k05iNGP1aLG40B4Q806ATI00];Bd0C4a<069RHP2@T900EUIF07]kN`0g=cL0?clo
06e]K@2VYZH0J6QX05UIF@0i>CT0De=C06m_K`17AdL0;Bd]04i>CP0m?Cd0Kfm_0:>SX`18B4P0;Rh^
04e=C@1IFET0IFEU06i^KP16ATH0<c<c03hn?P13@d<0DE5A08^;R`13@d<08b<S03Xj>P0d=3@0De=C
08:2PP10@400;2`/00kooooo03L0J6QX06YZJP1fMWH0DU9B04U9B@1HF5P0De=C01hN7P1MGEd0?Shn
059BDP19BDT0HV9R05ADE01cLg<0MGEe051@D010@400BDU90492@P0i>CT0>3Ph051@D015ADD0?Cdm
04m?C`0l?3`0B4Q805MGE`1DE5@0EeMG05ADE01IFET0EUIF055AD@0l?3`0BTY:03dm?@0l?3`0;2`/
04I6AP1>CTh0A4A403`l?017AdL09RHV03dm?@15ADD0820P05EEE@14A4@0CDe=03dm?@0_;bl0@d=3
0080@T9202/0;Bd]04Q8B01;Bd/0ADE503Hf=P0o?cl0De=C03`l?01DE5@0D51@04Q8B01MGEd0?clo
04m?C`0i>CT0C4a<02LW9`0Z:RX0Cdm?03Ph>01:BTX0:RXZ0451@@0g=cL0@d=303Xj>P15ADD0>SXj
034a<@0`<300@D5104a<C011@D40BTY:04I6AP0T92@0=cLg055AD@17AdL0@D5104e=C@1ADE40Cdm?
0080B4Q800<0Bd];05]KF`12@T800P1>CTh0A@1WIfL0EUIF059BDP12@T80IfMW04a<C010@400D51@
03Ti>@0h>3P0BDU903Ti>@1EEED0EeMG08J6QP22PX80QXJ603`l?01WIfL0O7al07YjNP22PX80NGUi
06e]K@19BDT0FUYJ05aLG01FEUH0H61P06=SH`1ADE40H61P02h^;P0j>SX0A4A40451@@1<C4`0Bd];
04Q8B016ATH0=SHf04];B`17AdL0HF5Q04=3@`1>CTh0K6a/03<c<`1IFET0A4A405EEE@1KFe/0=SHf
05IFEP10@400Kfm_04U9B@1BDU80?Cdm051@D01<C4`0>CTi04a<C01EEED0De=C04Q8B01JFUX0<S8b
0451@@0205eMG@100410@01ADE40<30`03dm?@16ATH0>c/k05IFEP1=CDd0HF5Q03lo?`1=CDd0;bl_
04M7A`1YJFT0?3`l05QHF00];Bd0GUiN05EEE@14A4@0RXZ:06m_K`0h>3P0KVi^01LG5`1:BTX0IfMW
0410@01SHf<0BDU906ATI01JFUX0Bd];04E5A@1=CDd0=cLg04];B`19BDT0EeMG04a<C00I6AT0?Shn
059BDP1HF5P0:BTY03/k>`15ADD0D51@05iNGP0V9RH0@D5104M7A`1HF5P0@T9205IFEP1OGel0Cdm?
05iNGP0V9RH0@41003@d=01:BTX0H61P05MGE`80GemO0280LG5a05aLG01ZJVX0LG5a06EUI@1mOGd0
IVIV07YjNP2VYZH0G5aL0861P@2>SXh0];Bd05EEE@1WIfL0JVYZ055AD@1MGEd0GUiN06EUI@14A4@0
Fe]K05YJFP1[Jf/0HV9R05UIF@1ZJVX0FUYJ06MWI`1_Kfl0EEEE05]KF`1hN7P0KVi^4?ooool000ko
oooo0240D51@079bLP21PH40QhN70451@@1PH600Rh^;071`L02QXJ40HF5Q061PH01bLW80BTY:09bL
W01KFe/0<S8b05eMG@1OGel0R8R807AdM00d=3@0>3Ph06YZJP1WIfL0BTY:03Ph>00a<C40C4a<07Ui
N@0e=CD0ATI603Xj>P0];Bd00P28R8P0g00];Bd0EEEE02LW9`0c<c<0LW9b06ATI00R8R80Cdm?038b
<P1=CDd0TY:B051@D00X:2P0C4a<0410@01ZJVX0N7Qh02`/;01;Bd/0ATI603hn?P2QXJ409RHV02LW
9`1:BTX0FUYJ03/k>`23Ph<07alO024Q8@1>CTh0OWin06ATI01jNWX0DE5A02l_;`1ZJVX0IFEU030`
<02o_kl0L71`038b<P1YJFT0PH6103De=@20P800;bl_030`<01PH600;Rh^038b<P1HF5P07alO02d]
;@1`L7009RHV02XZ:P12@T80=3@d038b<P1cLg<07AdM030`<01=CDd0FEUI06MWI`1fMWH08B4Q04M7
A`1`L700E5AD07YjNP1ZJVX06ATI051@D02g]kL0>3Ph05aLG01lO7`07alO059BDP2=SHd0<c<c07Ee
M@1OGel092@T03Lg=`1RHV80?clo0:JVYP0];Bd0<C4a05ADE025QHD0C4a<05aLG00k>c/0DE5A06MW
I`2TY:@0;bl_05IFEP0k>c/0<S8b08F5Q@2^[Zh09bLW03hn?P0m?Cd0De=C0:RXZ01;Bd/0;Rh^071`
L01JFUX0OGem0;Fe]@0/;2`0=SHf04Y:BP1UIFD0LG5a06QXJ00C4a<0>c/k04=3@`1`L700JVYZ03/k
>`0F5QH0>3Ph06UYJ@1fMWH0I6AT06MWI`0Q8B40@41004a<C01LG5`0A4A404];B`0O7al0EUIF07mo
O`0a<C40DU9B05=CD`0f=SH0VIVI07YjNP0g=cL0Fe]K034a<@1LG5`0QHF503<c<`1JFUX0Jf][06m_
K`2>SXh0De=C03lo?`1PH600Bd];08N7Q`21PH40:RXZ05IFEP1^KVh0G5aL0:NWY`12@T80:b/[07]k
N`1EEED0OGem02<S8`0i>CT0Cdm?07]kN`25QHD0@D5102TY:@16ATH0EeMG06EUI@1oOgl0;Rh^0451
@@17AdL0UYJF05eMG@1WIfL0>CTi03dm?@1BDU80IVIV04E5A@1hN7P0DE5A04M7A`1?Cdl0EeMG0410
@01FEUH0?3`l03@d=01SHf<0Cdm?034a<@1>CTh0>SXj03Ti>@1WIfL0<S8b03dm?@1QHF40>c/k04I6
AP2:RXX202`/;00E075aL@1ZJVX0IFEU08V9R@14A4@0;2`/04];B`1DE5@0HF5Q09jNWP15ADD0<S8b
04U9B@23Ph<0FEUI08^;R`1PH600;Bd]05=CD`1=CDd0E5AD00kooooo02L0JVYZ04m?C`0o?cl0Fe]K
06IVIP1:BTX0<C4a03@d=01aLG40AdM703lo?`16ATH0NGUi05IFEP1ZJVX0E5AD06QXJ01?Cdl0F5QH
04I6AP1@D500B4Q804a<C01EEED0De=C05MGE`0m?Cd0Hf=S08J6QP1ADE40Fe]K05=CD`26QXH0Bd];
04e=C@1IFET0PH6104i>CP16ATH00P14A4@05P1RHV80EeMG04Q8B019BDT08B4Q03lo?`1?Cdl04Q8B
03Ph>01LG5`0De=C03dm?@1:BTX0Cdm?03Lg=`16ATH0F5QH0492@P1MGEd0>c/k04M7A`10@40204M7
A`0T05mOG`13@d<0?clo05IFEP1RHV80CDe=03dm?@1KFe/0AdM704Q8B013@d<0;2`/03Lg=`0h>3P0
<C4a02HV9P0h>3P0;Bd]034a<@0b<S80;Rh^02l_;`0Z:RX0?Cdm03/k>`0n?Sh0>c/k030`<00_;bl0
@41003Lg=`17AdL0?Cdm04I6AP0l?3`0B4Q80P1ADE40A`16ATH0ADE5079bLP1GEeL0AdM704i>CP1<
C4`0;2`/04Q8B00h>3P0@D5104U9B@0a<C40;Bd]03hn?P11@D40>c/k04];B`1?Cdl0P82008R8R00_
;bl0Ogmo07=cL`1lO7`0R8R804U9B@1dM7@0L71`08>3P`1eMGD0BDU907IfMP1hN7P0?Shn05aLG00n
?Sh0FUYJ0451@@1?Cdl0IFEU06YZJP0i>CT0@T9202d];@1?Cdl0Kfm_08Z:RP0l?3`0?Shn04a<C01E
EED0K6a/05aLG00m?Cd0A4A402`/;01MGEd0:RXZ04a<C018B4P0H61P04U9B@0o?cl0CDe=03@d=01M
GEd0Cdm?0492@P0h>3P0Cdm?0080G5aL06H0EUIF05UIF@1LG5`0Ogmo055AD@0h>3P0@D5109>CT`1g
MgL0<S8b02TY:@0b<S80PH6103lo?`11@D40DE5A05ADE01lO7`0<30`055AD@1TI6@0IFEU06ATI01<
C4`0@T9207inOP1:BTX0@41004m?C`0n?Sh0Lg=c05ADE01GEeL0JFUY04=3@`0n?Sh0FUYJ02h^;P0d
=3@0B4Q80410@00g=cL0:2PX03hn?P11@D40@T9202HV9P1ZJVX0>3Ph03`l?01ADE40<c<c051@D00`
<300@T92059BDP1?Cdl0De=C03Hf=P1:BTX0=3@d04Q8B01OGel0C4a<04Y:BP19BDT0EUIF05MGE`10
@400<S8b05YJFP1QHF40E5AD07inOP1CDe<0ADE507alO00O7al0HV9R03`l?016ATH0P82009jNWP1e
MGD0KFe]05MGE`0m?Cd0C4a<05ADE01PH600@41004m?C`1OGel0Cdm?055AD@1;Bd/0GUiN05IFEP1K
Fe/0E5AD069RHP2MWId@ooooo`003_ooool0K021PH40ZZZZ04i>CP1eMGD0HV9R0:6QX@1BDU80@410
06a/K01WIfL0LG5a04Y:BP0g=cL0IVIV08R8R00h>3P0J6QX02HV9P1:BTX0Lg=c02h^;P0N7Qh0A4A4
05aLG022PX80DU9B02HV9P0Z:RX0G5aL07moO`2>SXh09RHV02d];@0l?3`0J6QX04M7A`2ZZZX08b<S
01TI6@1DE5@0G5aL04i>CP28R8P0?clo02PX:01_Kfl0M7Ad06][J`1mOGd06QXJ02`/;01RHV80CTi>
05=CD`16ATH061PH04i>CP1YJFT0;Bd]055AD@13@d<0:RXZ06a/K00m?Cd0820P03hn?P0_;bl09RHV
06UYJ@1LG5`06QXJ030`<00e=CD09RHV075aL@1XJ6P08b<S04E5A@1GEeL0AdM70:n_[`2=SHd0:RXZ
06a/K01BDU80FEUI092@T00h>3P04A4A06=SH`1RHV80J6QX06i^KP0B4Q80=3@d05EEE@14A4@0HF5Q
05=CD`0I6AT0DU9B06MWI`1ADE40EeMG02DU9@0b<S80IVIV07emO@80;bl_07T0:RXZ038b<P1IFET0
OGem03Xj>P0f=SH0=cLg02h^;P1lO7`0TI6A034a<@0_;bl0De=C034a<@2g]kL0R8R801lO7`0Y:BT0
FUYJ051@D02[Zj/0CTi>02LW9`1HF5P0QXJ6079bLP28R8P0>3Ph024Q8@10@400S8b<08V9R@1_Kfl0
820P04=3@`1YJFT0IfMW05iNGP14A4@0=cLg04i>CP1aLG40JFUY03Ti>@1QHF40@T9206m_K`21PH40
@T9203<c<`1EEED0<C4a04];B`1iNGT09BDU02/[:`1>CTh0@41005iNGP25QHD09bLW03Hf=P1MGEd0
ADE507]kN`1YJFT09bLW03hn?P1BDU80ADE50:2PX023Ph<0;Rh^04A4A01ZJVX0LG5a09:BTP0R8R80
>c/k07IfMP1LG5`0GemO04U9B@0P8200B4Q804e=C@1WIfL0P82002HV9P1>CTh0NGUi0451@@1UIFD0
9BDU04=3@`1eMGD0LW9b03Xj>P1PH600>SXj08F5Q@1BDU808b<S04I6AP0Y:BT0FEUI07YjNP0T92@0
Cdm?05aLG00c<c<0MgMg04m?C`0j>SX0Fe]K03lo?`1jNWX0UIFE04e=C@1FEUH0EeMG0080CDe=02d0
SXj>07=cL`1CDe<0F5QH03dm?@1YJFT0[Jf]06UYJ@0k>c/0GemO03/k>`1PH600YJFU07MgM`0Q8B40
Ogmo03dm?@1aLG40_Kfm02l_;`0h>3P0TY:B071`L01DE5@0GUiN034a<@0c<c<0BDU905IFEP1GEeL0
JFUY02DU9@12@T80G5aL06UYJ@0g=cL0DU9B01hN7P0h>3P0AdM706EUI@0Y:BT0Cdm?03hn?P11@D40
3_ooool07P1HF5P0I6AT05ADE016ATH0>CTi055AD@1BDU806QXJ06a/K010@400ADE505YJFP1NGUh0
F5QH08R8R01kNg/0BDU903Lg=`1CDe<0=SHf03Xj>P0V9RH0A4A404i>CP0a<C40B4Q804Y:BP1@D500
CTi>04U9B@80ATI600T0EeMG065QH@1IFET0Cdm?04e=C@0i>CT0ADE503dm?@0l?3`00P1@D500>`1J
FUX0BDU902TY:@19BDT0CDe=02TY:@1<C4`0;Bd]03Xj>P0`<300820P03Xj>P1DE5@0BDU902@T901C
De<0MgMg05=CD`0o?cl0Cdm?05aLG00e=CD0DE5A05IFEP1=CDd0NWYj03Ti>@1:BTX0@T92059BDP0Q
8B407alO03dm?@0b<S80C4a<02d];@0h>3P0?Cdm04M7A`0m?Cd0BTY:0451@@1>CTh0>c/k04U9B@1H
F5P0CTi>06=SH`1>CTh0?Shn03Ti>@1RHV80ATI60451@@1;Bd/0C4a<055AD@0j>SX0DE5A0080Bd];
0140>CTi03@d=014A4@0De=C04=3@`1VIVH0>CTi05IFEP17AdL0L71`05IFEP0c<c<0BTY:05UIF@0d
=3@0A4A408V9R@0207QhN00;069RHP0W9bL0DU9B059BDP17AdL0Bd];05eMG@1VIVH0@d=304i>CP1?
Cdl00P13@d<0I01CDe<0ATI604M7A`0^;Rh0:BTY03Ph>01NGUh0E5AD04e=C@14A4@0Bd];04Y:BP1@
D500<30`04E5A@0a<C40CDe=05mOG`15ADD0SXj>04=3@`17AdL0GUiN051@D01dM7@0?3`l071`L01U
IFD0Lg=c0451@@15ADD0CTi>05iNGP1BDU80De=C055AD@1DE5@0JVYZ04A4A00h>3P0FEUI06MWI`0j
>SX0E5AD01dM7@0_;bl0@41003/k>`1/K6`0JFUY038b<P0a<C40G5aL04I6AP13@d<0NGUi0492@P1N
GUh0?Shn07=cL`1VIVH0F5QH06i^KP1[Jf/0BTY:04];B`0h>3P0?Shn05EEE@18B4P0D51@06ATI01K
Fe/0F5QH061PH01KFe/0Bd];04I6AP1EEED0CTi>071`L01<C4`0=3@d05iNGP1XJ6P0DU9B030`<015
ADD0E5AD06=SH`1XJ6P0;2`/05=CD`1DE5@0C4a<04=3@`14A4@0G5aL05QHF01fMWH205IFEP0Q051@
D01:BTX0GemO065QH@28R8P0E5AD05YJFP0m?Cd0CDe=04m?C`1eMGD0HV9R07EeM@1CDe<0PX:204Q8
B01oOgl0BTY:06=SH`1KFe/0Lg=c06ATI00j>SX0CDe=04M7A`1CDe<0?Cdm04M7A`1@D500F5QH05AD
E012@T80C4a<0080CTi>00D0EEEE05MGE`1FEUH0H61P06UYJ@0@ooooo`003_ooool0<@28R8P0[Zj^
05aLG01SHf<0Kfm_0;jn_P1OGel0BDU905iNGP22PX80_Kfm05YJFP0a<C40CDe=08^;R`1:BTX0SHf=
02LW9`0l?3`0MWIf06MWI`1/K6`0JFUY02`/;01RHV80G5aL03/k>`1EEED0@D5102l_;`1<C4`0A4A4
03Lg=`1VIVH0?Shn038b<P18B4P0G5aL03`l?01UIFD0>CTi03De=@1HF5P0?Cdm04M7A`19BDT08b<S
06=SH`2ATI400P0_;bl0<P0X:2P0<30`08N7Q`23Ph<06QXJ038b<P1;Bd/0FUYJ0;Fe]@1mOGd04A4A
04U9B@12@T80GEeM09jNWP1>CTh061PH04];B`1<C4`0GUiN096AT@0G5aL09bLW04E5A@1<C4`0L71`
0;2`/00N7Qh0:b/[04=3@`1MGEd0A4A4051@D00E5AD0:b/[04a<C014A4@061PH034a<@0W9bL0AdM7
05mOG`0e=CD06a/K038b<P0Q8B40BDU9092@T01?Cdl09BDU0P1NGUh0Y`1oOgl09BDU01lO7`0f=SH0
DE5A0410@028R8P07QhN028R8P1EEED0SHf=06][J`26QXH0;2`/01PH601@D500Ti>C06MWI`1SHf<0
H61P01dM7@1GEeL0XZ:R04e=C@1FEUH0GEeM04M7A`1cLg<0KFe]04U9B@1NGUh0?Cdm03De=@1oOgl0
Fe]K02HV9P1RHV80=CDe03Ti>@1bLW80@D51028R8P0n?Sh0;2`/05YJFP1jNWX08R8R034a<@1HF5P0
DE5A05aLG02@T900>c/k03dm?@1:BTX0>CTi07YjNP2KVi/0;Bd]038b<P1UIFD0CDe=092@T029RHT0
6ATI03Hf=P1NGUh0EEEE07moO`2IVIT0820P02XZ:P1MGEd0?Shn06MWI`0];Bd08R8R04U9B@1kNg/0
@d=3061PH00Q8B40Cdm?07AdM01TI6@0EeMG02LW9`15ADD0RXZ:07=cL`0M7Ad0?Shn04Y:BP1hN7P0
`L7104m?C`0i>CT0GEeM061PH01nOWh06ATI02DU9@1NGUh0M7Ad05IFEP1UIFD0=CDe05MGE`18B4P0
@D5107emO@0^;Rh0<c<c05aLG00c<c<0Ogmo07emO@0e=CD0F5QH06=SH`1ADE40KVi^02d];@1;Bd/0
YJFU08B4Q02?Shl0Ph>302DU9@0l?3`0Ogmo07=cL`1OGel0GEeM030`<01:BTX0Jf][08N7Q`1CDe<0
F5QH01dM7@0b<S80IfMW09ZJVP1JFUX0F5QH02TY:@1GEeL0Ph>305EEE@0Z:RX0A4A405MGE`1/K6`0
MGEe03dm?@0Y:BT0?Cdm04Q8B02ZZZX0_[jn02<S8`0V9RH0Fe]K05EEE@1TI6@0SXj>00807QhN00H0
GUiN07QhN01?Cdl0JFUY02XZ:P0f=SH>ooooo`3F05]KF`1]KFd0KFe]04e=C@0o?cl0Hf=S06YZJP11
@D40UYJF05mOG`0e=CD0BTY:0861P@1UIFD0ATI604A4A01HF5P0EeMG055AD@0o?cl0EeMG04Q8B016
ATH0FEUI04i>CP14A4@0=3@d06EUI@2IVIT0CTi>06YZJP2;Rh/0Kfm_05=CD`1=CDd0Fe]K05iNGP0e
=CD0?3`l02h^;P17AdL0CDe=03lo?`0k>c/0BDU902l_;`12@T80FEUI01dM7@1HF5P0?clo04m?C`0o
?cl0>SXj059BDP13@d<0?Cdm03@d=01:BTX0?Shn04e=C@1FEUH0@41004A4A013@d<0CDe=04Y:BP0h
>3P0DE5A04E5A@1>CTh0@D5105ADE00Z:RX0;Bd]03Xj>P1ADE40A4A403@d=010@400?3`l0410@00a
<C40?Cdm04E5A@0g=cL0;Rh^03Lg=`1MGEd0?clo04Q8B01=CDd0<S8b0410@01=CDd0AdM704e=C@1@
D500GUiN05EEE@1MGEd0EEEE05mOG`1`L700?Shn04Y:BP1FEUH0Cdm?04U9B@1=CDd0ADE504i>CP15
ADD0:2PX04M7A`0[:b/0<S8b03@d=00_;bl0>c/k04U9B@0h>3P0P82007IfMP0Z:RX0TI6A08:2PP1N
GUh0M7Ad08R8R01hN7P0CDe=059BDP18B4P0Fe]K05mOG`1TI6@0IFEU07=cL`11@D40<30`04=3@`16
ATH0O7al07inOP1lO7`0Lg=c02/[:`0o?cl0B4Q805=CD`0[:b/0ADE505YJFP0b<S80DE5A03lo?`1E
EED0D51@03Ph>01UIFD0>CTi06e]K@19BDT0FEUI04];B`1SHf<0F5QH04=3@`1ADE40E5AD05]KF`0^
;Rh0@d=302d];@10@400HF5Q04U9B@1BDU80J6QX0492@P1?Cdl0FUYJ05iNGP1HF5P0ADE505YJFP18
B4P0HF5Q02HV9P1OGel0Ph>3051@D01KFe/0<30`065QH@1;Bd/0DE5A0:2PX01cLg<0C4a<06ATI01;
Bd/0GemO09>CT`1MGEd0DE5A06EUI@1]KFd0DE5A05EEE@1DE5@0BTY:04Q8B080AdM703h0CTi>06][
J`0/;2`0A4A403lo?`25QHD0B4Q805iNGP18B4P0MGEe07moO`18B4P0A4A405UIF@1CDe<0EUIF07Ee
M@1WIfL0Kfm_0:JVYP0a<C40:BTY04M7A`1fMWH0G5aL0492@P1<C4`0JVYZ09NGU`1GEeL0R8R805iN
GP12@T80E5AD04a<C01lO7`0Vi^K051@D01IFET0K6a/06i^KP1WIfL0RHV9061PH01>CTh0B4Q804U9
B@0n?Sh0>CTi04Y:BP15ADD0Hf=S06a/K01@D500NWYj07UiN@1;Bd/0OWin05QHF01WIfL0PX:207Ee
MA3ooooo000>ooooo`1A05EEE@2b/[80VYZJ07YjNP1JFUX0GEeM0:RXZ01KFe/0M7Ad05]KF`21PH40
JFUY05MGE`1XJ6P0@41003Lg=`1_Kfl0Bd];05mOG`1GEeL08b<S05EEE@2WYjL0ADE504m?C`17AdL0
<C4a096AT@2NWYh0?3`l05IFEP0^;Rh0?Cdm0;6a/@20P800;Rh^04I6AP11@D40DU9B0:b/[00o?cl0
71`L03Xj>P1BDU80IfMW08J6QP0V9RH06QXJ05EEE@1?Cdl0NWYj02DU9@0N7Qh0>SXj08N7Q`0o?cl0
C4a<01lO7`0U9BD0C4a<07UiN@0o?cl0FUYJ02DU9@0Z:RX0D51@06a/K00Y:BT0Bd];0451@@0l?3`0
G5aL03dm?@0F5QH0GEeM03`l?00k>c/0WIfM04I6AP0O7al0D51@0080B4Q803/0QHF5024Q8@0C4a<0
=SHf0492@P1QHF40KFe]028R8P0];Bd0FEUI04M7A`1@D500H61P01/K6`15ADD0E5AD05UIF@1FEUH0
I6AT024Q8@19BDT0HF5Q03Lg=`1iNGT0IVIV02/[:`1OGel0O7al02XZ:P1WIfL0F5QH028R8P1]KFd0
C4a<01lO7`0[:b/0ADE502`/;01CDe<0KVi^01/K6`0j>SX0BDU906e]K@1mOGd0JFUY02HV9P1dM7@0
E5AD06a/K02^[Zh0LG5a01hN7P1IFET0NWYj05ADE02[Zj/0Lg=c01LG5`0205MGE`2604Y:BP1]KFd0
8B4Q01lO7`1ZJVX0B4Q806m_K`1UIFD0?3`l02LW9`22PX80Lg=c06ATI01VIVH0<c<c034a<@1nOWh0
QXJ604a<C013@d<0>c/k05QHF02LW9`0A4A40492@P17AdL0:RXZ061PH02ZZZX0?3`l02l_;`1;Bd/0
Cdm?09:BTP12@T80>3Ph03`l?00j>SX0EEEE08>3P`0[:b/0;Bd]065QH@1GEeL0UIFE02LW9`0`<300
K6a/09jNWP1PH600GEeM03hn?P1CDe<0S8b<06IVIP1lO7`0?3`l04A4A01QHF40>3Ph04Q8B01WIfL0
92@T03/k>`1iNGT0=SHf05ADE00h>3P0?clo08n?S`1HF5P0@D5103lo?`0`<300IVIV0:2PX00^;Rh0
BTY:04Q8B02;Rh/0WYjN08V9R@0^;Rh0AdM704U9B@2BTY80_kno055AD@0e=CD0GEeM06IVIP28R8P0
UIFE02LW9`0X:2P0>CTi05aLG01/K6`0WIfM02TY:@0O7al0?Shn075aL@1SHf<0QHF501lO7`0`<300
IVIV06][J`1YJFT0OWin02d];@0/;2`0GEeM06i^KP1HF5P0M7Ad03`l?00m?Cd0Ph>306YZJP0m?Cd0
Bd];04E5A@0j>SX0WYjN07moO`0T92@0@D5103Hf=P0`<300MgMg04M7A`0T92@>ooooo`1O08F5Q@1d
M7@0BTY:05iNGP1/K6`0C4a<04I6AP0W9bL0EUIF03Lg=`12@T80FUYJ07=cL`1OGel0GUiN04e=C@1E
EED0@T9205]KF`0g=cL0=3@d03`l?00g=cL0E5AD0492@P1>CTh0:b/[0451@@1FEUH0AdM704A4A00m
?Cd0GEeM05IFEP1BDU80GEeM06a/K00k>c/0H61P04e=C@0b<S80BTY:04E5A@1JFUX0>3Ph02LW9`1V
IVH0DE5A01lO7`0i>CT0ATI604e=C@0M7Ad06a/K038b<P1EEED0ADE503@d=01=CDd0Lg=c05MGE`1Y
JFT0GEeM059BDP0j>SX0ATI605MGE`1XJ6P0P82005IFEP1NGUh0@41006QXJ014A4@0@D5103@d=00e
=CD0BDU904M7A`0k>c/0<30`03dm?@0b<S80;bl_02@T900g=cL0=CDe03Ti>@0d=3@0Bd];04A4A00k
>c/0=cLg04Q8B01<C4`00P0m?Cd0;014A4@0EEEE04U9B@0d=3@0D51@05EEE@18B4P0@T9204Y:BP1E
EED0Shn?06YZJP1eMGD0GemO05MGE`1IFET0>c/k069RHP0l?3`0@T9204Q8B01BDU80De=C08J6QP1>
CTh0RXZ:07UiN@0/;2`0@41004M7A`1ZJVX0TI6A05aLG01VIVH0MWIf09NGU`1NGUh0<c<c06MWI`1]
KFd0=SHf051@D01?Cdl0G5aL0P18B4P205YJFP1a03De=@15ADD0@T9204i>CP26QXH0UYJF03De=@0k
>c/0DU9B04a<C01TI6@0CTi>02h^;P10@400?clo05YJFP0[:b/0@d=3059BDP1SHf<0?clo03@d=00i
>CT0@D5105EEE@1;Bd/0DE5A04e=C@1?Cdl0:RXZ03Xj>P1XJ6P0@410051@D01cLg<07AdM02XZ:P0e
=CD0H61P071`L00e=CD0_[jn04U9B@1bLW80JVYZ03Hf=P17AdL0Cdm?07YjNP1EEED0FUYJ06EUI@17
AdL0I6AT0410@00X:2P0Lg=c01TI6@0S8b<0EeMG01TI6@19BDT0D51@05iNGP1TI6@0EUIF04e=C@1R
HV80?3`l04=3@`1DE5@0D51@04m?C`1MGEd0UiNG06m_K`11@D40CDe=09>CT`1_Kfl0MWIf079bLP0i
>CT0EeMG03`l?013@d<0KFe]06ATI010@400>3Ph05UIF@0m?Cd0<c<c034a<@17AdL0Bd];05]KF`1_
Kfl0<c<c03Lg=`0N7Qh0<C4a069RHP2CTi<0T92@07UiN@1@D500Lg=c03lo?`1YJFT0OGem0:f][@0i
>CT00P1BDU804P1jNWX0GEeM02LW9`0`<300EeMG05IFEP0_;bl0@T9207emO@0e=CD0ADE507UiN@1Y
JFT0DE5A05aLG01BDU80D51@08B4Q13ooooo000>ooooo`2g05YJFP1TI6@0Fe]K07]kN`1:BTX0FEUI
06YZJP1WIfL0^;Rh04Y:BP1dM7@0IfMW05=CD`2EUID0Cdm?02DU9@1HF5P0P820075aL@1[Jf/08B4Q
020P801PH600K6a/06i^KP0c<c<0A4A40451@@1<C4`0BDU906i^KP0S8b<0;2`/02l_;`1DE5@0?Cdm
05IFEP0U9BD0;bl_04I6AP1MGEd0=SHf05aLG00f=SH0BDU906MWI`1<C4`0ATI6069RHP0W9bL0NGUi
04a<C00E5AD0>3Ph03Ti>@0b<S80U9BD02PX:00N7Qh0?3`l04M7A`0g=cL0XZ:R01<C4`0_;bl0=SHf
04U9B@11@D40T92@04=3@`1GEeL0Lg=c0492@P14A4@0UYJF05UIF@0/;2`0Jf][06QXJ01ADE40/;2`
04Q8B019BDT0G5aL0492@P1EEED0EeMG01DE5@0_;bl0DU9B03lo?`1SHf<0JVYZ01/K6`14A4@0GEeM
038b<P18B4P07alO02h^;P1NGUh0F5QH03Ph>01@D5009RHV02@T9022PX80LG5a02@T900j>SX0DU9B
03<c<`2OWil0TY:B02<S8`1<C4`0IFEU03Ti>@21PH40Kfm_01DE5@0h>3P0Cdm?04A4A026QXH0<S8b
01XJ6P1FEUH0IVIV04E5A@28R8P0>3Ph02<S8`1DE5@0T92@05eMG@1TI6@0:BTY034a<@1GEeL0Lg=c
02DU9@0o?cl09RHV06QXJ02OWil0=SHf02@T9011@D40;bl_08F5Q@1dM7@09RHV01hN7P13@d<0GemO
06UYJ@1`L7005ADE04Y:BP1@D500Q8B406i^KP1eMGD06a/K04];B`23Ph<0@d=307inOP1:BTX0:2PX
0451@@2HV9P0PX:205EEE@14A4@0:RXZ04M7A`1YJFT0IFEU07YjNP0[:b/0<30`0080FEUI05d0HV9R
06QXJ00Y:BT0DE5A04U9B@0R8R80Ng]k04I6AP0n?Sh0GUiN04A4A017AdL0@D5104A4A02?Shl0TI6A
02`/;01IFET0Fe]K06IVIP2BTY809BDU02TY:@1RHV80GemO07emO@0S8b<0;Bd]06i^KP1RHV80F5QH
03Ti>@0P8200AdM707inOP14A4@0KVi^02HV9P0_;bl0MWIf09RHV01UIFD0LG5a05aLG01ADE40GEeM
08>3P`1=CDd0_;bl03Ti>@17AdL0IVIV05]KF`1UIFD0MgMg038b<P12@T80Fe]K05aLG01@D500JFUY
03/k>`0^;Rh0Q8B404Y:BP1@D500HF5Q03De=@0d=3@0IVIV034a<@0h>3P0>CTi028R8P1FEUH0Y:BT
03lo?`11@D40B4Q802h^;P1QHF40TI6A03dm?@0S8b<0=SHf05eMG@1;Bd/0Shn?051@D00k>c/0CDe=
05]KF`1?Cdl03_ooool09@1CDe<0GEeM05UIF@13@d<0=3@d04m?C`1;Bd/05aLG07YjNP0l?3`0C4a<
06EUI@2GUiL0LG5a08>3P`1QHF40GemO04a<C01SHf<0;2`/03dm?@0[:b/0<c<c05aLG00g=cL0Bd];
0492@P1GEeL0J6QX04m?C`1LG5`0GUiN0820P01PH600Ph>307IfMP1HF5P00P0Y:BT0;P0P8200<30`
03dm?@1HF5P0@D5104A4A00h>3P0AdM706QXJ00`<300AdM703Hf=P15ADD0:2PX01lO7`1?Cdl0A4A4
03Ti>@0Z:RX0DE5A03dm?@15ADD0DU9B03hn?P1EEED0?Shn05ADE01:BTX09BDU05IFEP0e=CD0FEUI
059BDP1>CTh08R8R01lO7`0g=cL0Bd];04E5A@0Y:BT0@d=303hn?P0g=cL0>c/k0451@@0e=CD20492
@P2804I6AP15ADD0De=C04];B`0i>CT0=3@d05UIF@1PH600ADE505YJFP1EEED0C4a<05=CD`1DE5@0
ATI605=CD`1lO7`0B4Q804I6AP1;Bd/0DE5A05IFEP1aLG40JFUY071`L00f=SH0DE5A04Q8B00e=CD0
>3Ph03`l?00Z:RX0=CDe04E5A@0j>SX0CTi>06=SH`14A4@0Q8B406e]K@1TI6@0OGem06YZJP1EEED0
>SXj059BDP12@T80@D5104A4A01ADE40>SXj04M7A`0Y:BT0;Bd]04=3@`1;Bd/0C4a<07=cL`13@d<0
DE5A059BDP13@d<06a/K03Hf=P0W9bL0=CDe059BDP0l?3`0QXJ6055AD@17AdL0DE5A04A4A01bLW80
820P05MGE`1ADE40OWin05aLG013@d<0AdM7065QH@1UIFD0@d=306e]K@1?Cdl0IfMW02h^;P0k>c/0
P820061PH00[:b/0;2`/02DU9@17AdL0BTY:02DU9@0o?cl0BTY:0;>c/`26QXH0:2PX030`<01<C4`0
P82002LW9`11@D40;bl_07YjNP0X:2P0<c<c0:6QX@1mOGd0=SHf0451@@0`<300?Cdm06][J`0];Bd0
EEEE059BDP1SHf<0CTi>07YjNP18B4P0?clo04M7A`1>CTh0@41009RHV01gMgL0=3@d04E5A@14A4@0
NGUi024Q8@80?Shn0240NGUi07alO00c<c<0@T9205ADE01:BTX0?clo05=CD`1KFe/0DE5A08n?S`11
@D40FUYJ05QHF01DE5@0EEEE03dm?@1GEeL0>CTi05QHF00o?cl0JVYZ03Ti>@0h>3P0?3`l04A4A01i
NGT0QXJ60492@P0o?cl0<30`03Ph>019BDT00P1PH6004P0f=SH0A4A404M7A`13@d<0:2PX03<c<`0`
<300@41003Hf=P1LG5`0M7Ad0451@@1QHF40UIFE05]KF`1_Kfl0Ng]k08^;Ra3ooooo000>ooooo`2G
07YjNP2PX:00H61P06MWI`23Ph<0Rh^;04Q8B013@d<0LG5a06e]K@2<S8`0?clo0410@01PH600QhN7
051@D01]KFd0>3Ph04m?C`2ATI40HF5Q05MGE`1RHV80>SXj07UiN@2KVi/0EEEE07]kN`12@T80@410
07]kN`12@T80@41008N7Q`16ATH0;bl_05YJFP0l?3`0>SXj05QHF00f=SH0AdM706QXJ01:BTX0ADE5
04e=C@0^;Rh0@41006IVIP0g=cL0F5QH051@D00n?Sh0IFEU03dm?@0K6a/0?Shn03`l?00i>CT0PH61
030`<00O7al0Cdm?03dm?@14A4@0PX:202h^;P0G5aL0>c/k059BDP14A4@0FEUI01`L700N7Qh0BDU9
06a/K00o?cl0G5aL020P8019BDT0@T92055AD@13@d<0BDU902`/;01MGEd0PX:204=3@`0U9BD0@d=3
02h^;P1^KVh0OGem02d];@0d=3@0@D510492@P1`L700@T9202XZ:P12@T80D51@04Q8B01dM7@0:b/[
020P8018B4P0D51@04Q8B01JFUX07alO02h^;P0g=cL0GEeM05QHF01jNWX0:RXZ02<S8`0j>SX0JFUY
04e=C@1HF5P0IFEU03dm?@1[Jf/0N7Qh04U9B@1NGUh0LG5a03dm?@21PH40FUYJ059BDP1YJFT0DE5A
051@D021PH40A4A402TY:@1JFUX0LW9b03<c<`1eMGD0=CDe02PX:01?Cdl0@D5104e=C@1EEED0:2PX
0492@P0205UIF@1m0451@@2?Shl0AdM704e=C@1BDU80@41004];B`1fMWH0:RXZ03dm?@20P800Bd];
05mOG`1EEED08B4Q04U9B@1kNg/0?Shn059BDP1bLW80;Bd]051@D01RHV80=SHf06=SH`0V9RH0DE5A
08>3P`24Q8@0ADE503/k>`16ATH0Jf][08Z:RP0j>SX0GUiN0410@00n?Sh0R8R806a/K00f=SH0Cdm?
04Q8B01aLG40IFEU03hn?P1;Bd/0Hf=S05QHF01hN7P09RHV04e=C@1ZJVX0ATI604Y:BP1MGEd0?clo
07AdM01RHV80ATI6079bLP0k>c/0D51@06IVIP10@400N7Qh05eMG@0k>c/0MGEe06QXJ01>CTh0Jf][
0410@01DE5@0Ph>308B4Q025QHD0Lg=c04I6AP1SHf<0V9RH06MWI`1OGel0DU9B03Lg=`1KFe/0V9RH
07inOP1;Bd/0FEUI03Lg=`1NGUh0UYJF07YjNP0o?cl0CTi>03`l?01eMGD0QhN704=3@`0i>CT0FEUI
04];B`1iNGT0MWIf03Xj>P0c<c<0GemO05IFEP25QHD0Ti>C03lo?`0_;bl0E5AD05YJFP1BDU80L71`
034a<@0Z:RX0De=C06][J`19BDT0LW9b02`/;00d=3@03_ooool0M01NGUh0LG5a04e=C@1<C4`0A4A4
04a<C01HF5P0;Rh^06e]K@1ADE40EEEE05]KF`2FUYH0LW9b06ATI01HF5P0FEUI059BDP1CDe<0@D51
04M7A`0i>CT0?3`l05QHF015ADD0DE5A02l_;`1>CTh0GemO04Y:BP1BDU80Hf=S059BDP1SHf<0H61P
05YJFP1OGel0?Cdm0410@00j>SX0ATI605=CD`1VIVH0EUIF04i>CP12@T80CDe=06IVIP0R8R80CTi>
04I6AP17AdL09RHV02XZ:P0g=cL0?3`l0410@00_;bl0EeMG05=CD`1HF5P0EeMG05mOG`1dM7@0@T92
06YZJP1XJ6P0B4Q808>3P`1HF5P0H61P04=3@`1iNGT0Bd];03lo?`17AdL0>CTi05]KF`10@400B4Q8
0492@P11@D40?clo04];B`0m?Cd0FEUI03hn?P0c<c<0AdM7059BDP1LG5`0Cdm?04Y:BP0o?cl0GemO
04E5A@18B4P0GUiN04M7A`1OGel0BDU905MGE`1DE5@0EUIF069RHP1>CTh0F5QH06UYJ@1QHF40H61P
05iNGP1eMGD0Cdm?02d];@18B4P0ATI60P11@D40@@19BDT0DU9B04a<C01DE5@0RHV907inOP0];Bd0
DE5A06ATI01kNg/0UIFE069RHP1UIFD0H61P06YZJP1WIfL0CDe=05=CD`1cLg<0@d=304a<C012@T80
@D510410@01GEeL0A4A406UYJ@15ADD0GUiN03Lg=`19BDT0;bl_04Y:BP0g=cL0@D5106=SH`1=CDd0
BDU903hn?P15ADD0Bd];03<c<`19BDT08R8R04U9B@0W9bL0@T9203De=@15ADD0BDU903Lg=`1NGUh0
Lg=c069RHP0c<c<0ATI604E5A@1;Bd/0G5aL05MGE`1CDe<0HF5Q02DU9@0o?cl0C4a<0080Cdm?05P0
<c<c06e]K@1XJ6P0IVIV03lo?`0g=cL0D51@03lo?`1LG5`0@D5105QHF016ATH0=CDe06MWI`1RHV80
:2PX069RHP0/;2`0>SXj06EUI@0i>CT0FUYJ04a<C01OGel0De=C04=3@`11@D40F5QH02XZ:P0b<S80
@T9204I6AP1ADE40B4Q808n?S`1BDU80BTY:051@D01OGel0B4Q806ATI01XJ6P0?clo04i>CP15ADD0
GEeM04i>CP1PH600IVIV05EEE@25QHD0=SHf04I6AP0_;bl0@D5104=3@`1>CTh0IFEU05ADE01TI6@0
DE5A05iNGP0h>3P0CDe=071`L017AdL0ATI6061PH00[:b/0F5QH04i>CP1aLG40EUIF08^;R`1[Jf/0
CDe=04U9B@0n?Sh0?3`l04=3@`1:BTX0?Cdm05mOG`1YJFT0?3`l05=CD`1ZJVX0GEeM0P1DE5@00`1Y
JFT0IVIV07IfMP0@ooooo`003_ooool0o`1CDe<0PH610820P01cLg<0BTY:06MWI`2KVi/0TY:B0;Bd
]01?Cdl0LW9b06YZJP1?Cdl0N7Qh04];B`0h>3P0L71`05iNGP1eMGD0RHV902XZ:P1FEUH0QXJ604a<
C01]KFd0>3Ph04U9B@2HV9P0Hf=S02TY:@0l?3`0;Rh^04U9B@2>SXh0Ogmo02TY:@0l?3`0D51@065Q
H@2EUID0@41002HV9P1FEUH0ATI607MgM`1bLW805QHF02HV9P13@d<0CTi>0;>c/`0Z:RX0;Bd]03lo
?`1SHf<0A4A4069RHP0S8b<0:b/[05EEE@1IFET0AdM705mOG`0I6AT0=3@d06YZJP1gMgL0=cLg05iN
GP0T92@0ADE5069RHP0j>SX09RHV05eMG@0N7Qh0?clo08F5Q@0i>CT09RHV04i>CP0Z:RX0D51@06MW
I`0Z:RX0;bl_04];B`0o?cl0BDU906m_K`0F5QH061PH04E5A@16ATH0E5AD055AD@0/;2`0AdM705eM
G@1CDe<0QHF507emO@0L71`0Cdm?04U9B@14A4@0];Bd055AD@11@D40CDe=08N7Q`18B4P0OGem059B
DP0k>c/0Fe]K07YjNP18B4P0K6a/05mOG`0G5aL0=SHf06EUI@0];Bd0KFe]038b<P0I6AT0De=C09RH
V00`<300M7Ad05MGE`0n?Sh0Kfm_0861P@0T92@0Fe]K03Xj>P11@D40SXj>06ATI00V9RH0DU9B05eM
G@1_Kfl0UiNG02d];@0i>CT0EeMG03dm?@1eMGD0N7Qh01dM7@0e=CD0H61P06][J`20P800K6a/02LW
9`0_;bl0GemO07=cL`1EEED0>c/k01dM7@1NGUh0Rh^;05UIF@1JFUX0?clo02d];@17AdL0M7Ad065Q
H@0`<300De=C02h^;P1OGel0DE5A03De=@1=CDd0<c<c06EUI@1WIfL07alO055AD@1LG5`0@410092@
T00f=SH0:2PX07MgM`1SHf<0TY:B0;:b/P0n?Sh0<S8b06IVIP1lO7`0WinO05iNGP12@T80JFUY06EU
I@21PH40Y:BT02l_;`1IFET0KVi^04a<C01XJ6P08R8R04=3@`2ATI40LW9b04U9B@0a<C40;bl_05aL
G026QXH0:BTY05=CD`0Z:RX0GEeM071`L01GEeL0=3@d05IFEP19BDT0EUIF06m_K`19BDT0>CTi05eM
G@17AdL0SHf=079bLP0Z:RX0;Rh^0451@@1QHF40O7al0820P00U9BD0820P0410@01LG5`0Cdm?06AT
I00G5aL0Bd];05mOG`1KFe/0D51@08b<S0005`0J6QX0A4A406=SH`1YJFT0De=C08f=S@0U9BD0@d=3
06][J`1EEED0Hf=S0:BTY019BDT0;bl_065QH@1IFET0@d=308F5Q@0V9RH0=SHf04i>CP0g=cL09RHV
00kooooo08D0Lg=c07]kN`1_Kfl0HF5Q05]KF`1=CDd0IfMW02<S8`18B4P0@T92051@D021PH40M7Ad
05=CD`25QHD0UIFE05ADE014A4@0Fe]K04m?C`0i>CT0<30`059BDP1MGEd0@d=304i>CP0k>c/0@410
0451@@1FEUH0@T9203Xj>P1>CTh0F5QH06IVIP1`L700F5QH03hn?P0_;bl08b<S03Hf=P0h>3P0BTY:
05QHF012@T80=3@d05iNGP1CDe<0>3Ph04a<C00g=cL0=SHf02XZ:P1?Cdl0CDe=05ADE010@4008b<S
03lo?`1CDe<0EEEE05UIF@1DE5@0IfMW04i>CP1QHF40C4a<04=3@`1LG5`0>CTi0492@P10@400PX:2
03Xj>P0h>3P0Bd];04=3@`1:BTX0;Rh^03lo?`0k>c/0@d=30492@P11@D40?3`l04I6AP15ADD0:RXZ
034a<@17AdL0Bd];03De=@12@T80@41004];B`1EEED0De=C05YJFP1OGel0De=C04a<C01JFUX0GEeM
05YJFP1CDe<0C4a<05ADE01JFUX0N7Qh065QH@1WIfL0PX:2071`L010@400Bd];05UIF@1ADE40ATI6
04U9B@1NGUh0AdM704a<C02FUYH0QhN702l_;`19BDT0LG5a08N7Q`2MWId0X:2P07moO`1ZJVX0Fe]K
0080>CTi0600E5AD04];B`0Z:RX0>c/k02XZ:P0g=cL0@D5104i>CP1TI6@0GEeM04A4A01FEUH0De=C
04Q8B015ADD0EeMG02`/;00`<300@T9204e=C@1WIfL0DE5A05ADE01HF5P0>c/k059BDP0U9BD0Fe]K
02TY:@10@400;2`/0492@P13@d<0D51@05ADE01>CTh0E5AD04Y:BP1IFET0E5AD061PH027QhL0NGUi
06EUI@2@T9003`l?03Xj>P1:BTX0NWYj06=SH`0k>c/0TY:B07inOP1=CDd0;bl_03hn?P1VIVH0BDU9
05=CD`0l?3`0Fe]K08R8R00l?3`0P82007UiN@1:BTX0NGUi01lO7`0Z:RX0GUiN024Q8@16ATH0BDU9
06UYJ@1PH600C4a<04];B`1MGEd0>SXj03@d=01DE5@0Bd];04E5A@0n?Sh0DE5A02l_;`1jNWX0AdM7
08V9R@14A4@0N7Qh06i^KP0W9bL0ADE503/k>`1PH60203lo?`0406YZJP1=CDd0GemO02XZ:P80=cLg
02L0AdM704];B`1EEED0F5QH03/k>`1VIVH0>CTi06e]K@1CDe<0KFe]04Q8B01FEUH0Bd];08>3P`1X
J6P0LG5a06e]K@2NWYh0AdM70;Bd]01bLW80ATI603Ph>010@400?Cdm02h^;P0n?Sh0?clo05MGE`10
@400@D5105iNGP1CDe<0GEeM08N7Q`1BDU80Kfm_06][J`1iNGT04?ooool000kooooo03T0KVi^071`
L018B4P0MGEe04U9B@1PH600?Cdm05EEE@2SXj<0C4a<04m?C`0m?Cd0?3`l07moO`1FEUH092@T04Y:
BP0a<C40K6a/0:FUY@12@T807alO05=CD`1QHF40SXj>02XZ:P0c<c<0ATI606EUI@22PX80E5AD034a
<@0h>3P0D51@075aL@1NGUh0Jf][02TY:@0j>SX0L71`06IVIP1=CDd0H61P038b<P0o?cl0T92@055A
D@17AdL0EeMG02/[:`1SHf<0KFe]02/[:`1YJFT0@d=302@T9021PH400P0^;Rh0f`1<C4`0:BTY02TY
:@1oOgl061PH020P800n?Sh0<c<c08R8R02EUID09bLW02`/;00o?cl0;bl_04e=C@2/[:`0>3Ph03@d
=015ADD0DE5A05IFEP2<S8`0;Bd]04M7A`22PX80EeMG06m_K`1kNg/07AdM02l_;`2<S8`0LW9b07Ad
M00m?Cd07alO04I6AP1iNGT0Cdm?06MWI`0L71`0<C4a05UIF@2?Shl0@d=306UYJ@0R8R806a/K059B
DP1ZJVX08b<S03hn?P0b<S80=CDe061PH01QHF40:b/[03lo?`1:BTX0>CTi0:2PX01kNg/0:BTY02d]
;@1OGel0H61P09bLW012@T807QhN03Xj>P28R8P0DE5A08J6QP12@T80;2`/04Q8B01/K6`0HV9R08f=
S@0g=cL09BDU07IfMP35aLD0>c/k05iNGP0_;bl0<30`05eMG@1_Kfl0>c/k05IFEP0/;2`0BDU9069R
HP16ATH0?3`l07inOP0a<C40K6a/07AdM00Z:RX0;bl_03hn?P1DE5@0P82004A4A00M7Ad0;Rh^05aL
G026QXH0QhN704m?C`0L71`0Cdm?04E5A@1fMWH0FUYJ06][J`0];Bd0Jf][061PH01MGEd0P82003hn
?P0c<c<0KVi^02`/;01XJ6P0DU9B02HV9P1LG5`0B4Q804Y:BP2NWYh0<S8b02TY:@1_Kfl0Ogmo05mO
G`0];Bd0>SXj07YjNP1lO7`0Bd];06MWI`0Q8B40Hf=S0:NWY`1CDe<0@T9205=CD`0o?cl0UIFE04i>
CP0[:b/0CDe=04i>CP1RHV80BTY:02HV9P0n?Sh0Cdm?04U9B@2UYJD0;Bd]04E5A@1_Kfl0IFEU06UY
J@2QXJ40Cdm?0410@01YJFT0F5QH07AdM02c/k<0BTY:04e=C@1SHf<0?Shn05UIF@1UIFD0;Bd]04A4
A01jNWX0J6QX059BDP21PH40@41005ADE01aLG40<C4a04e=C@1WIfL09BDU03`l?02ATI40K6a/061P
H01SHf<0:BTY03Ph>02?Shl0BTY:03dm?@1FEUH09bLW03dm?@1KFe/0GemO02LW9`14A4@09bLW05UI
F@2WYjL0?clo02h^;P15ADD0=cLg05YJFP0>ooooo`0C06=SH`1NGUh0H61P04A4A00d=3@0DU9B04a<
C00`<300GemO05=CD`1cLg<0Ph>30;Vi^@1eMGD0HF5Q05YJFP1RHV80EEEE06QXJ00203Ph>00S03Hf
=P1:BTX0GEeM03Lg=`1CDe<0C4a<05EEE@1LG5`0ATI604Y:BP1>CTh0Lg=c0820P01`L700P82006MW
I`1EEED0>CTi02TY:@1CDe<0HF5Q06EUI@1?Cdl0DU9B030`<00e=CD0De=C024Q8@16ATH0>SXj04E5
A@0C4a<09RHV03dm?@0j>SX00P0`<300>`13@d<0>c/k03dm?@0/;2`0>c/k04E5A@0_;bl0AdM705EE
E@18B4P0KFe]04Q8B01QHF40H61P08>3P`17AdL0?Cdm03hn?P0k>c/0?3`l03@d=00l?3`0<30`03Ph
>00d=3@0>c/k03De=@16ATH0:RXZ02/[:`1:BTX0>3Ph04a<C011@D40>CTi03Ph>01:BTX0>CTi03Hf
=P11@D40>c/k04A4A01EEED0AdM704m?C`1^KVh0De=C05QHF01JFUX0DE5A06YZJP1EEED0IFEU0820
P01aLG40FEUI059BDP17AdL0EUIF0080ATI600P0HF5Q05QHF01PH600PX:208>3P`1BDU80Cdm?069R
HP80Q8B40700?Shn05iNGP1dM7@0EEEE092@T018B4P0H61P06QXJ00k>c/0B4Q803@d=00c<c<0De=C
05QHF00a<C40BTY:03De=@1LG5`0@d=303dm?@1:BTX0EUIF01XJ6P0m?Cd0Cdm?04A4A00o?cl0AdM7
04Q8B01=CDd0=3@d051@D00I6AT0A4A402LW9`19BDT0:BTY03Xj>P14A4@0<30`05=CD`1OGel0JVYZ
034a<@1<C4`0@41004];B`1bLW80Hf=S04I6AP1IFET0:RXZ03lo?`1CDe<0LG5a07EeM@0S8b<0E5AD
055AD@23Ph<08b<S01/K6`1>CTh0;Rh^065QH@0f=SH0CDe=06ATI00L71`0SXj>07AdM00M7Ad0Cdm?
01XJ6P0Z:RX0GUiN02PX:01ZJVX0F5QH055AD@1CDe<0ADE503Ti>@13@d<0<30`02TY:@0k>c/0DE5A
04A4A01:BTX0TY:B06=SH`0m?Cd0GEeM075aL@1<C4`0EUIF05YJFP0n?Sh0DU9B02XZ:P0g=cL0>CTi
02/[:`17AdL0>c/k05MGE`0f=SH0D51@02`/;01IFET0H61P0P1DE5@08`0e=CD0IFEU03`l?016ATH0
:b/[04i>CP1oOgl0GemO05EEE@2CTi<0@T9204A4A00l?3`0IfMW04=3@`1JFUX0IVIV04=3@`0l?3`0
@T9203Ph>00W9bL0>SXj02PX:016ATH0ADE503De=@15ADD0EUIF04a<C01BDU80BTY:07AdM01QHF40
L71`013ooooo000>ooooo`3o09VIV@2EUID0EEEE05eMG@2:RXX0[:b/03@d=015ADD0NGUi06m_K`2^
[Zh0Bd];04U9B@1_Kfl0V9RH04m?C`1TI6@0A4A404=3@`1QHF40H61P05eMG@1OGel0>SXj04e=C@0f
=SH0Cdm?07AdM00e=CD0A4A40:FUY@0_;bl0=cLg05IFEP0h>3P0;Rh^05=CD`0S8b<0;Bd]04];B`0c
<c<0D51@06QXJ00h>3P0=cLg0451@@1CDe<0L71`06=SH`0W9bL0@T92030`<01FEUH0YZJV05UIF@0D
51@0@T9204I6AP1ZJVX0QXJ602`/;00g=cL0CDe=05EEE@1kNg/0YjNW01lO7`0/;2`0BTY:051@D01W
IfL0DE5A01XJ6P0^;Rh0ATI606m_K`0o?cl0GUiN02h^;P14A4@0EUIF069RHP18B4P0:BTY030`<01:
BTX0Jf][02LW9`0I6AT0@D5103<c<`25QHD0EeMG01LG5`0I6AT0?Cdm04A4A01cLg<0=CDe01HF5P14
A4@0HF5Q07AdM02LW9`09bLW01LG5`0c<c<0K6a/05IFEP1fMWH05aLG03Lg=`1CDe<0Fe]K04A4A01W
IfL07QhN02d];@1WIfL0MgMg03lo?`1HF5P0ATI604Y:BP2ATI40Jf][02`/;01@D5008b<S04E5A@1b
LW80>CTi0451@@15ADD0:2PX04i>CP2WYjL0@41001dM7@1GEeL0UYJF0451@@2DU9@0=CDe04=3@`14
A4@0ADE506MWI`1kNg/06ATI05=CD`1]KFd0Bd];06QXJ032`/80?Cdm051@D02?Shl0IVIV05YJFP1/
K6`0=3@d04i>CP1jNWX0BDU905=CD`0k>c/0:b/[06QXJ02DU9@0ATI606=SH`1FEUH0;2`/04i>CP1i
NGT0=CDe04i>CP0/;2`0E5AD08f=S@22PX80@d=304Y:BP0c<c<0SHf=07MgM`0P8200@T9202/[:`1]
KFd0_[jn04U9B@0W9bL0AdM705YJFP2DU9@0De=C01`L7015ADD0KVi^05aLG01VIVH07AdM0451@@29
RHT0AdM707MgM`1GEeL08B4Q061PH01nOWh0F5QH0:RXZ00T92@0>c/k05UIF@1<C4`0GUiN04I6AP0j
>SX0Fe]K04Y:BP16ATH0I6AT03/k>`0h>3P0Kfm_08f=S@1kNg/0N7Qh03Xj>P0f=SH0N7Qh07EeM@1B
DU80B4Q803De=@29RHT0K6a/06=SH`1FEUH0DU9B024Q8@1<C4`0RHV903/k>`0a<C40FEUI03Xj>P1e
MGD0]KFe034a<@0Y:BT0?clo000G03Lg=`2EUID0^[Zj02TY:@0i>CT0ADE505iNGP1nOWh0PX:201DE
5@0P8200:2PX04Q8B010@400IfMW01hN7P0K6a/0EUIF05=CD`0o?cl0FEUI024Q8@0T92@03_ooool0
401KFe/0KVi^04M7A`0o?cl0Bd];079bLP1ADE406QXJ06i^KP1GEeL0AdM705=CD`1NGUh0BTY:069R
HP1WIfL204Q8B007059BDP0e=CD0;Bd]02XZ:P11@D40CDe=04Y:BP0204a<C00P03lo?`12@T80A4A4
04I6AP1HF5P0CTi>06UYJ@1MGEd0AdM705iNGP19BDT0=cLg03Ph>00n?Sh0Bd];05YJFP1_Kfl0Cdm?
038b<P1MGEd0LW9b02/[:`1CDe<0H61P069RHP0R8R80:2PX04E5A@0m?Cd0>SXj03Hf=P16ATH204A4
A00e04Q8B011@D40Cdm?04U9B@1JFUX0ADE503<c<`1KFe/0KFe]03Ti>@0_;bl0BTY:0410@014A4@0
=CDe03Hf=P12@T80Bd];04E5A@0];Bd0Bd];038b<P17AdL0CDe=04U9B@0X:2P0:BTY04A4A00j>SX0
DU9B04e=C@0n?Sh0>CTi055AD@1DE5@0BDU905=CD`1YJFT0CDe=04=3@`1DE5@0CDe=03`l?010@400
>SXj03Lg=`1^KVh0IVIV07=cL`1FEUH0JFUY04I6AP12@T800P15ADD02@19BDT0<c<c03/k>`17AdL0
@D5103@d=01[Jf/0Kfm_0492@P0206QXJ00:0861P@2@T900LW9b06ATI01PH600D51@09>CT`1<C4`0
HF5Q06QXJ0<0@d=304L0DE5A04];B`13@d<0GEeM06ATI01JFUX0IVIV03`l?01ADE40HV9R07MgM`1@
D500=3@d0410@01JFUX0F5QH04e=C@0l?3`0CTi>0410@01EEED07QhN04a<C00h>3P0LW9b01@D500f
=SH0@D5103@d=01=CDd0?clo05MGE`0i>CT0Bd];04E5A@1?Cdl0LW9b04a<C01:BTX0F5QH01TI6@0d
=3@0?3`l03@d=018B4P0<c<c06i^KP1=CDd0@D5102<S8`0g=cL0N7Qh02LW9`1DE5@0820P04I6AP18
B4P0:BTY05MGE`1HF5P0>c/k08^;R`0d=3@06a/K04i>CP0I6AT0@T9205=CD`1UIFD0DU9B05UIF@02
04e=C@0Q03lo?`18B4P0BDU906m_K`1fMWH0?3`l06=SH`1fMWH0KFe]05YJFP1NGUh0ATI606i^KP1O
Gel09BDU03Ph>00I6AT0@D5102XZ:P0[:b/0HV9R05aLG01dM7@0BDU904M7A`0X:2P0@D5104E5A@1K
Fe/0De=C069RHP1eMGD0=cLg0080DE5A01h0BDU9059BDP12@T80<30`065QH@1MGEd0HF5Q05QHF01e
MGD0=cLg0861P@1`L700EUIF04=3@`16ATH0?3`l03lo?`1>CTh0>3Ph04Y:BP1YJFT0EUIF04a<C029
RHT0Cdm?05QHF01DE5@0I6AT05=CD`1TI6@@ooooo`003_ooool0DP1JFUX0SXj>05mOG`1QHF40AdM7
08N7Q`2:RXX0JVYZ08b<S01;Bd/0K6a/04Y:BP0o?cl0IVIV03dm?@1PH600PX:202l_;`11@D40EeMG
03Xj>P1^KVh0VIVI03@d=01@D500<C4a04U9B@2?Shl0;Bd]02l_;`1GEeL0CTi>065QH@20P8008b<S
028R8P1OGel0E5AD06a/K01iNGT0820P03Ph>01@D500H61P05aLG00a<C40820P04A4A01@D500@D51
06][J`0Q8B4092@T04A4A01^KVh0:b/[0410@00P8200>c/k055AD@16ATH07QhN03Xj>P0H61P0AdM7
0::RXP1;Bd/08B4Q055AD@0];Bd0Jf][07inOP0Z:RX08R8R051@D018B4P0Fe]K09^KV`0W9bL0<c<c
05MGE`19BDT207inOP3001@D500i>CT0MGEe04E5A@1DE5@0N7Qh01`L700W9bL0K6a/03/k>`1RHV80
@41002/[:`1ADE40C4a<028R8P1`L700Jf][02@T901PH600HV9R034a<@1RHV80=CDe024Q8@2CTi<0
D51@01TI6@0o?cl0B4Q802XZ:P24Q8@0;bl_01/K6`0c<c<0IVIV04E5A@1YJFT0KVi^028R8P1/K6`0
M7Ad07]kN`2JVYX0;Rh^034a<@21PH40J6QX075aL@2YZJT0:RXZ04Y:BP1_Kfl0De=C04I6AP1mOGd0
>c/k02HV9P1fMWH0KVi^05IFEP1OGel06a/K02l_;`20P800=SHf05EEE@1YJFT0820P055AD@12@T80
EEEE05]KF`12@T80<C4a05=CD`1FEUH0:RXZ04];B`10@400Bd];0;:b/P1eMGD07QhN04Y:BP13@d<0
GEeM0<G5a@2ZZZX0820P03De=@1OGel0GUiN08R8R00U9BD071`L04U9B@1LG5`0Jf][03De=@0N7Qh0
ATI60:BTY01PH600O7al02TY:@1BDU80EUIF05MGE`19BDT0K6a/03hn?P1PH600I6AT03De=@1KFe/0
ADE504A4A02MWId0>c/k05aLG012@T80<S8b04U9B@1hN7P0A4A404m?C`0];Bd0<S8b0861P@1RHV80
>3Ph03<c<`1jNWX0MWIf05iNGP0Q8B40EEEE055AD@2EUID0XJ6Q02h^;P0e=CD0ATI606a/K02RXZ80
WYjN03@d=00l?3`0=3@d065QH@2RXZ80T92@02TY:@0h>3P0Fe]K061PH01UIFD0KFe]038b<P0V9RH0
GUiN07IfMP13@d<0E5AD03hn?P0h>3P0Cdm?06EUI@12@T80GUiN02LW9`12@T80HF5Q03/k>`18B4P0
IFEU02@T9021PH40XZ:R05=CD`0Y:BT0De=C030`<01ADE40X:2P04M7A`0d=3@0ATI6030`<01EEED0
VYZJ0P0^;Rh>ooooo`1]07IfMP1gMgL0Bd];03dm?@1GEeL0De=C051@D00K6a/0F5QH0451@@1VIVH0
KVi^07UiN@1WIfL0SXj>08b<S01EEED0CDe=06][J`0c<c<0:b/[024Q8@0b<S80G5aL034a<@1ADE40
;Rh^05EEE@18B4P0@D5104i>CP1;Bd/0IFEU08N7Q`1SHf<0LW9b05iNGP13@d<0:2PX024Q8@0h>3P0
C4a<06EUI@0c<c<0B4Q803Ti>@0o?cl0@d=302LW9`14A4@0>3Ph04I6AP0N7Qh0<C4a03hn?P1?Cdl0
?Cdm020P801GEeL0MgMg06ATI018B4P0De=C05QHF00l?3`0EeMG055AD@19BDT0K6a/04A4A01DE5@0
Cdm?06][J`0d=3@0?Shn04e=C@1UIFD0BTY:03De=@1=CDd09bLW03Hf=P0a<C40?Cdm02/[:`0m?Cd0
=cLg03Hf=P16ATH0C4a<055AD@0j>SX0>CTi04I6AP1;Bd/0@d=305aLG01CDe<0?Cdm04Q8B01HF5P0
Cdm?04i>CP1?Cdl0CDe=05iNGP13@d<0=CDe03dm?@0204];B`0M05QHF01>CTh0Ogmo05]KF`0h>3P0
ATI60410@00f=SH0=cLg04M7A`1ZJVX0MgMg06e]K@0k>c/0EeMG05YJFP1kNg/0Xj>S07EeM@1fMWH0
FUYJ04U9B@2CTi<0Bd];071`L01fMWH0B4Q804E5A@0X:2P00P0f=SH0I`1NGUh0Fe]K05iNGP0g=cL0
AdM703lo?`0n?Sh0?3`l06MWI`0Y:BT0?clo04Y:BP1JFUX0J6QX075aL@18B4P0AdM70410@01QHF40
:RXZ04I6AP0[:b/0ADE501/K6`0n?Sh0B4Q804i>CP1IFET0@41007moO`0m?Cd0Cdm?04Y:BP1?Cdl0
M7Ad06EUI@1?Cdl0IVIV02<S8`0f=SH0C4a<05mOG`1bLW80@D5109:BTP20P800Fe]K01lO7`0Z:RX0
B4Q803dm?@1NGUh0@D51055AD@1OGel0=3@d06UYJ@1OGel07AdM04U9B@0`<30061PH05eMG@0h>3P0
B4Q805QHF01UIFD0CDe=0410@00o?cl0@d=302TY:@0l?3`0@T9204Y:BP1<C4`0?Shn09VIV@2k^k/0
IfMW071`L01aLG40@d=306m_K`1MGEd0?3`l051@D00m?Cd0DE5A03dm?@1EEED0H61P069RHP20P800
>CTi051@D00S8b<0B4Q804e=C@1@D500Bd];03`l?00n?Sh00P1BDU805011@D40FEUI061PH01=CDd0
92@T08^;R`0f=SH0@d=304Y:BP1aLG40@d=309FEU@1DE5@0ADE5055AD@1EEED0=cLg02h^;P1:BTX0
ATI60`1;Bd/0200g=cL0IFEU059BDP1GEeL0De=C06QXJ01MGEd0Lg=c4?ooool000kooooo0?l0EEEE
06EUI@2BTY80LW9b04E5A@1_Kfl0P8200:NWY`2d];@0?Cdm06a/K01FEUH0PX:20<30`01UIFD08R8R
079bLP1HF5P0T92@06e]K@0b<S80;bl_07AdM01hN7P0S8b<02LW9`0j>SX0DE5A06IVIP1RHV80>3Ph
03lo?`12@T80EEEE04m?C`0i>CT0@d=303Ph>01@D500OWin04A4A00j>SX0EUIF03`l?01lO7`0LW9b
02<S8`0m?Cd0@41005MGE`2IVIT0<S8b020P800e=CD0?3`l05MGE`1gMgL07QhN03Lg=`1;Bd/0A4A4
05iNGP1TI6@08b<S03<c<`1dM7@0De=C06QXJ02@T900820P04U9B@1KFe/0?Shn059BDP2d];@0<30`
038b<P1CDe<0EeMG04m?C`1PH6006QXJ03Ti>@1nOWh0ATI6030`<013@d<08B4Q01/K6`1ZJVX0>3Ph
03Xj>P11@D404a<C04M7A`1_Kfl051@D03@d=00o?cl0>SXj06e]K@2<S8`09BDU05MGE`1WIfL0>c/k
09bLW01[Jf/09RHV04Q8B02FUYH0F5QH08B4Q01BDU80@41005YJFP1GEeL0?clo061PH00n?Sh0:BTY
04];B`1fMWH0Cdm?06EUI@0U9BD0ATI609JFUP2DU9@0<S8b04e=C@0];Bd0:RXZ07alO01YJFT0<30`
05iNGP18B4P0I6AT096AT@1iNGT09BDU04Y:BP1>CTh0P82008n?S`0K6a/09RHV05eMG@1EEED0OWin
09nOW`0O7al0CDe=05MGE`1>CTh0[:b/092@T00G5aL0AdM707inOP1GEeL0JVYZ05IFEP0M7Ad0CTi>
09nOW`1:BTX0TY:B03Ti>@0V9RH0G5aL0;>c/`1<C4`0L71`05IFEP0[:b/0OGem079bLP12@T80B4Q8
02DU9@12@T80Lg=c02l_;`1ADE40>3Ph038b<P1fMWH0>c/k05MGE`1[Jf/0;2`/04a<C02NWYh0EeMG
04E5A@1QHF40CDe=08F5Q@1YJFT0<c<c04m?C`17AdL0VYZJ08B4Q00X:2P0?clo04m?C`1/K6`0T92@
02XZ:P0];Bd0C4a<075aL@1FEUH09BDU04Q8B01bLW80D51@03Xj>P1dM7@0<C4a06ATI02RXZ80SHf=
0451@@22PX80?Cdm079bLP2^[Zh0FUYJ06MWI`1kNg/0@d=307QhN02[Zj/0D51@02l_;`1FEUH0<c<c
05eMG@30`<00GEeM02d];@1<C4`0@d=305UIF@2GUiL0;Bd]03Ti>@1:BTX0<c<c06MWI`2LW9`000H0
:RXZ02h^;P1>CTh0@d=305UIF@2][Jd202l_;`0?071`L01YJFT0B4Q80;2`/00c<c<0ADE506EUI@16
ATH0>c/k05eMG@0R8R80@D5104e=C@1;Bd/0?clo00kooooo0240GUiN075aL@1>CTh0=CDe04A4A01X
J6P0GemO02PX:01QHF40Hf=S03<c<`1TI6@0ZZZZ08f=S@1^KVh0BDU9069RHP1IFET0EEEE04A4A01<
C4`0AdM70451@@1IFET0BDU905mOG`0o?cl0H61P07YjNP1EEED0CDe=06ATI015ADD00P1?Cdl00`1D
E5@0JFUY04Y:BP0203<c<`0?05MGE`1KFe/0I6AT04];B`1WIfL0>c/k0410@01LG5`0A4A405IFEP0b
<S80>c/k018B4P0d=3@0>c/k0080:BTY01d08b<S03dm?@0`<300>c/k03De=@0];Bd0<C4a03lo?`16
ATH0ADE502l_;`1ADE40ADE503Xj>P0`<300E5AD030`<00l?3`0CDe=01`L700a<C40;Bd]03De=@0X
:2P0;Rh^02d];@0d=3@0;Rh^03hn?P0202TY:@0^03`l?00e=CD0>c/k04A4A00k>c/09RHV05IFEP12
@T80;Bd]04E5A@14A4@0@T9203`l?00f=SH0BDU904i>CP16ATH0BTY:055AD@18B4P0DU9B06=SH`0a
<C40BTY:04a<C00l?3`0A4A403Ti>@0a<C40=CDe0410@018B4P0@D5103Ph>01SHf<0JVYZ03Lg=`1>
CTh0DU9B05ADE01YJFT0P82007=cL`1[Jf/0Fe]K07]kN`80BDU908D0FEUI05MGE`1LG5`0<S8b03/k
>`0o?cl0BDU906IVIP1IFET0DU9B04U9B@0i>CT0>c/k05]KF`1TI6@0;bl_04Y:BP1FEUH0DE5A04Q8
B01CDe<0GemO05UIF@12@T80Jf][02h^;P15ADD09BDU04a<C00H61P0De=C04e=C@10@400FEUI04A4
A01OGel0>c/k04];B`0k>c/0E5AD09ZJVP1gMgL0@d=308F5Q@0f=SH0De=C075aL@1WIfL0KVi^04=3
@`1DE5@0IFEU071`L00m?Cd0:BTY04a<C00d=3@0IfMW03Ph>012@T80DE5A04A4A02NWYh0WinO02/[
:`1LG5`06a/K02DU9@1cLg<09bLW03lo?`1;Bd/0F5QH05=CD`14A4@0?Cdm04i>CP0g=cL0;Rh^0492
@P1EEED0DU9B04I6AP2FUYH0jNWY08N7Q`1JFUX0L71`04E5A@21PH40IVIV03@d=01>CTh0DU9B05=C
D`1ADE40ADE506IVIP1ADE40HF5Q024Q8@0c<c<0;Rh^05=CD`1EEED0ADE505YJFP1MGEd0EeMG05AD
E01/K6`0Jf][075aL@1YJFT0KFe]06EUI@2<S8`0DE5A06QXJ01jNWX0TY:B05YJFP2FUYH0GemO04Q8
B00h>3P0@T9205IFEP1=CDd0EEEE04U9B@1[Jf/0KFe]0080FUYJ00L0L71`05=CD`1kNg/0GemO08N7
Q`1TI6@0IVIV013ooooo000>ooooo`0P09jNWP1dM7@0CTi>06YZJP1LG5`0PH6103/k>`1eMGD0]KFe
06QXJ025QHD0B4Q803/k>`23Ph<0S8b<03`l?01CDe<09bLW04A4A01aLG40=CDe04E5A@1EEED0EeMG
0:2PX00c<c<0;Bd]05aLG00o?cl0GEeM04e=C@0L71`203@d=03d03Lg=`16ATH0GemO02`/;010@400
Ng]k05IFEP1YJFT0Q8B403De=@1LG5`0H61P05EEE@22PX80E5AD03Xj>P1WIfL0D51@03dm?@1aLG40
ATI601dM7@16ATH0<c<c03hn?P1?Cdl08R8R03`l?01XJ6P071`L04M7A`1FEUH0:BTY02`/;021PH40
B4Q803Hf=P10@400>3Ph04A4A02CTi<0MgMg030`<00a<C40>CTi05eMG@2OWil0BDU903Hf=P0o?cl0
H61P06IVIP2CTi<08B4Q038b<P1@D500EeMG092@T01bLW809BDU02h^;P1ADE40?3`l065QH@0O7al0
9bLW04A4A01ZJVX0FUYJ08F5Q@0U9BD0820P05UIF@1UIFD0:2PX04M7A`0Z:RX0@41004Y:BP1FEUH0
7AdM04m?C`0`<300@4100:VYZ@29RHT09BDU03`l?019BDT0GEeM0:ZZZP0];Bd09BDU03Xj>P1MGEd0
H61P08Z:RP0Y:BT0=CDe04Y:BP1fMWH0MgMg09jNWP0[:b/0=cLg06][J`1jNWX0EEEE079bLP0j>SX0
ATI608F5Q@1oOgl0JFUY05aLG00b<S80ADE507MgM`1?Cdl0=3@d05UIF@0[:b/0GemO09jNWP12@T80
ADE505ADE00`<300CDe=06=SH`0S8b<0DE5A05IFEP0M7Ad0O7al04A4A00O7al0@T9204=3@`1;Bd/0
][Jf07moO`0[:b/0@d=304A4A01iNGT0/[:b03De=@0g=cL0Cdm?05ADE02VYZH0JFUY01PH601NGUh0
?Shn07alO01hN7P09BDU028R8P1iNGT0HV9R05YJFP0N7Qh0=CDe051@D01dM7@0HV9R038b<P0O7al0
AdM706ATI019BDT0?3`l03Ti>@0a<C40Jf][051@D00c<c<0Jf][02`/;020P800@410038b<P1HF5P0
B4Q8034a<@1kNg/092@T030`<01_Kfl0LG5a05EEE@35aLD0?Cdm04Q8B021PH40KFe]09:BTP38b<P0
?3`l05ADE01oOgl0O7al07YjNP23Ph<0;bl_03hn?P24Q8@0W9bL071`L022PX807alO03hn?P1kNg/0
FEUI07MgM`1iNGT071`L04U9B@20P800Bd];07]kN`1]KFd0<C4a03hn?P1kNg/0>3Ph03hn?P18B4P0
9bLW02HV9P1MGEd0Lg=c02PX:00e=CD0=SHf05UIF@1LG5`0BDU902h^;P1<C4`08R8R061PH0kooooo
01P0EeMG08>3P`1[Jf/0FUYJ092@T01=CDd0@T9202l_;`1BDU80BDU903Ti>@1PH600Kfm_06EUI@1c
Lg<0GUiN04a<C01ADE40G5aL02d];@0a<C40>SXj02PX:01CDe<204U9B@0803Ph>01;Bd/0De=C04A4
A015ADD0F5QH06MWI`1OGel205=CD`0B059BDP1<C4`0=SHf03Hf=P0d=3@0<C4a0451@@1@D500@410
03Xj>P1QHF40EeMG02TY:@16ATH0DE5A04Q8B00A4A407AdM0P18B4P08@10@400>3Ph04A4A015ADD0
BTY:04I6AP1>CTh0BTY:05QHF01KFe/0DE5A03@d=01ZJVX0GemO04e=C@11@D40MGEe05QHF01EEED0
DU9B03@d=01BDU80Bd];04E5A@0c<c<0CTi>03Lg=`16ATH0B4Q804U9B@0R8R80<S8b04E5A@0204U9
B@1W03Ti>@0n?Sh0>3Ph04I6AP15ADD0De=C059BDP1VIVH0EUIF0451@@1IFET0BTY:03Ph>011@D40
B4Q803`l?01`L700G5aL071`L01:BTX0LG5a055AD@1;Bd/0=CDe038b<P0a<C40=3@d0451@@0g=cL0
>c/k05QHF01OGel0Lg=c04M7A`1CDe<0KVi^06ATI02?Shl0HV9R06MWI`15ADD0AdM707emO@1ZJVX0
JFUY05IFEP0b<S80ADE503@d=00e=CD0;2`/03Hf=P1nOWh0FUYJ03Hf=P1DE5@0@41004E5A@0d=3@0
B4Q802HV9P0/;2`0ADE504Q8B01^KVh0IFEU0451@@1;Bd/0EeMG07IfMP0^;Rh0I6AT03Ti>@1[Jf/0
;2`/03lo?`15ADD0=cLg06ATI00g=cL0F5QH02DU9@0n?Sh0>SXj04U9B@1DE5@0HF5Q04I6AP25QHD0
820P02PX:01<C4`0HV9R08j>SP1DE5@0KFe]07=cL`1dM7@07AdM03Ti>@1?Cdl0@D5107AdM00204];
B`0B06MWI`1EEED0A4A405ADE00Y:BT0HV9R01hN7P0>3Ph0EeMG02/[:`1BDU80J6QX06IVIP15ADD0
FUYJ04a<C00j>SX0?clo0P15ADD0?P1lO7`0KFe]08F5Q@3:b/X0Ogmo0861P@1fMWH0Ogmo04Q8B01A
DE40EUIF03@d=01JFUX0>3Ph05=CD`1MGEd0=CDe0492@P1=CDd0GemO01`L701<C4`0<C4a05MGE`1N
GUh0FEUI04e=C@14A4@0BTY:03<c<`10@400;2`/04A4A01`L700JVYZ02d];@2KVi/0FEUI06=SH`1K
Fe/0GEeM0410@023Ph<0HF5Q05aLG01?Cdl0@D5102`/;01SHf<0GEeM02PX:01CDe<0H61P04I6AP1=
CDd0SXj>05=CD`1JFUX0GUiN06QXJ01MGEd0RXZ:4?ooool000kooooo07d0KFe]0:NWY`1JFUX0IFEU
061PH02YZJT0K6a/07alO01dM7@0FEUI08f=S@1FEUH0Bd];055AD@10@400CTi>092@T00g=cL0Cdm?
03dm?@0j>SX0IfMW09^KV`0o?cl0GUiN04Y:BP1?Cdl0Rh^;04a<C015ADD0D51@03@d=01]KFd0PH61
030`<00_;bl0@d=304];B`1XJ6P0GemO01`L700];Bd0GUiN05ADE024Q8@0>SXj02l_;`1FEUH0EeMG
0410@01UIFD0:b/[04Q8B01XJ6P0E5AD03<c<`19BDT0?Cdm051@D027QhL0?Shn02l_;`1=CDd0<S8b
06i^KP1aLG40>c/k03Ti>@1=CDd0?clo07=cL`1WIfL0<C4a030`<016ATH0A4A405MGE`26QXH08b<S
02`/;01RHV80Hf=S06MWI`0l?3`07alO0492@P1CDe<0Fe]K06m_K`1JFUX07QhN04A4A01LG5`0C4a<
051@D01EEED0>CTi05QHF016ATH09RHV05EEE@1@D500>SXj08N7Q`0e=CD0=cLg05QHF01:BTX0>SXj
07emO@0o?cl0;Bd]051@D01IFET0<C4a07AdM017AdL06a/K05iNGP1lO7`0Cdm?05mOG`16ATH0@D51
06e]K@0205eMG@070:2PX01<C4`0=CDe09:BTP1FEUH0G5aL08B4Q00203Ti>@2>08>3P`17AdL0?Cdm
06][J`0d=3@0:b/[071`L01BDU80HF5Q06EUI@12@T80Bd];06m_K`0l?3`0Ogmo05mOG`0l?3`0F5QH
08B4Q014A4@0FUYJ04A4A00k>c/0Fe]K08b<S011@D40EUIF05]KF`13@d<0IFEU06i^KP0b<S80EEEE
04];B`14A4@0NGUi07MgM`0^;Rh0@41005IFEP1:BTX0Ng]k03Lg=`0^;Rh0FEUI07MgM`1RHV809bLW
028R8P1UIFD0IfMW04a<C01[Jf/0:RXZ04I6AP2>SXh0GUiN04E5A@1lO7`0DU9B08Z:RP1IFET0<C4a
05QHF015ADD0MWIf08J6QP0];Bd0HF5Q059BDP0f=SH0GemO05]KF`19BDT0De=C0410@01EEED0WIfM
05=CD`1:BTX0CDe=051@D01hN7P0B4Q803Ph>01XJ6P0CTi>06EUI@2GUiL0=SHf0492@P1ZJVX0J6QX
06m_K`1hN7P0<C4a03Xj>P19BDT0HF5Q08N7Q`2?Shl0=SHf03lo?`1UIFD0GEeM05iNGP1ADE40:BTY
06=SH`2@T900Fe]K0451@@1IFET0;Bd]07AdM02CTi<0?clo04I6AP1:BTX0?Shn07AdM02BTY80@T92
04];B`1<C4`0@D5107QhN02JVYX0>SXj03De=@16ATH0B4Q804M7A`1eMGD092@T02l_;`1ADE40=3@d
04I6AP1dM7@0BTY:04m?C`kooooo0540GemO06EUI@0n?Sh0>3Ph05ADE01kNg/0GUiN02`/;01SHf<0
DU9B05MGE`1fMWH0RHV907AdM025QHD0PH61061PH01JFUX0IfMW04Q8B00g=cL0>CTi03hn?P1CDe<0
?clo05iNGP11@D40HF5Q05MGE`1;Bd/0EUIF075aL@1JFUX0NWYj07AdM01WIfL0JVYZ055AD@0i>CT0
@4100451@@14A4@0HF5Q03lo?`1=CDd0=3@d03Ti>@1RHV80ADE505=CD`0[:b/0BTY:028R8P0];Bd0
AdM70410@00h>3P06QXJ051@D018B4P0BTY:034a<@15ADD0F5QH03dm?@1<C4`0GemO0451@@1WIfL0
=SHf04I6AP10@400G5aL02/[:`0h>3P0C4a<03dm?@12@T80:b/[04=3@`0m?Cd00P0d=3@02@0i>CT0
9RHV04=3@`0U9BD09bLW02`/;016ATH0A4A403De=@0203dm?@0905=CD`1:BTX0D51@04Y:BP0j>SX0
ATI604M7A`1>CTh0CDe=0080E5AD00`0Cdm?04U9B@1>CTh0EEEE06ATI00m?Cd0G5aL04M7A`1GEeL0
F5QH04a<C018B4P20451@@1l04a<C01FEUH0GemO06UYJ@1bLW80BTY:05mOG`1[Jf/0JFUY08b<S01Y
JFT0JVYZ04I6AP1RHV80K6a/04A4A01LG5`0GEeM05=CD`1OGel0<C4a03dm?@0j>SX0>CTi04U9B@1=
CDd0@41005EEE@1FEUH0EeMG05mOG`1/K6`0820P02h^;P1FEUH0CDe=04U9B@0o?cl0CTi>04e=C@0l
?3`0F5QH02<S8`1:BTX08B4Q04e=C@0O7al0BDU905YJFP0n?Sh0EEEE03/k>`1DE5@0=cLg05UIF@19
BDT0Bd];07IfMP1kNg/0@41007inOP0Y:BT0=cLg055AD@1IFET0NWYj04A4A02<S8`0MGEe071`L00W
9bL0=cLg05YJFP0Y:BT0I6AT02h^;P1<C4`0GEeM03Hf=P1lO7`0MGEe02`/;01IFET0<S8b02<S8`1V
IVH0=3@d03Lg=`1CDe<0F5QH05EEE@17AdL0CTi>05MGE`0i>CT0?Cdm04Y:BP1EEED0EeMG038b<P1^
KVh0dM7A0;Ng]`1WIfL0CTi>04a<C01iNGT0KVi^02XZ:P1DE5@0A4A40492@P1LG5`0IfMW06UYJ@1Z
JVX0O7al04A4A01FEUH0FUYJ06i^KP1`L700H61P05=CD`80Cdm?0240B4Q805MGE`0c<c<0DE5A06MW
I`17AdL0@d=308Z:RP1LG5`0Cdm?06e]K@2:RXX0DE5A0861P@1EEED0D51@04Q8B014A4@0@T9204i>
CP1CDe<0=CDe06a/K01UIFD0De=C04i>CP1VIVH0G5aL0820P01JFUX0NWYj05aLG01gMgL04?ooool0
00kooooo08/0JFUY05YJFP1mOGd0KFe]04M7A`1EEED0ATI609:BTP2_[jl0>SXj055AD@1NGUh0GEeM
07]kN`0/;2`09BDU061PH01NGUh0QXJ605EEE@0D51@0ADE507emO@1dM7@0Shn?01XJ6P0H61P0DU9B
06i^KP1mOGd0=3@d04I6AP1<C4`0I6AT05MGE`1ADE40?Shn038b<P1EEED0U9BD05mOG`1DE5@0C4a<
03<c<`1ADE40GemO04E5A@1NGUh0?Cdm0492@P1bLW80F5QH03<c<`0n?Sh0;Bd]04Q8B01WIfL09BDU
034a<@0g=cL0>SXj065QH@1WIfL061PH0451@@1;Bd/0EeMG05]KF`2FUYH0820P05QHF01jNWX0I6AT
08:2PP2KVi/0:RXZ02d];@1TI6@0LG5a05mOG`21PH406a/K03Lg=`1aLG40Fe]K08n?S`15ADD05QHF
02HV9P1QHF40=3@d07YjNP0n?Sh09bLW02d];@1CDe<09bLW05iNGP0Y:BT09RHV06MWI`1hN7P0:RXZ
04U9B@0o?cl0?Cdm08f=S@1=CDd05aLG03dm?@23Ph<0EeMG092@T01DE5@07AdM03lo?`2SXj<0ADE5
0:VYZ@1BDU8071`L03dm?@19BDT0Bd];05MGE`0I6AT0;Bd]04m?C`1=CDd0Cdm?059BDP0K6a/0;bl_
06a/K021PH40G5aL04A4A00S8b<0C4a<0080Jf][0P0f=SH02@0U9BD0TI6A0;:b/P10@400;Rh^0451
@@1<C4`0X:2P09jNWP0203Hf=P0V05IFEP1ADE40QhN704Y:BP0D51@0<S8b06IVIP1]KFd0Z:RX05QH
F00H61P0BDU906IVIP1GEeL0UiNG02d];@0c<c<0I6AT08f=S@1:BTX0XZ:R05ADE01EEED0CTi>05AD
E00d=3@0E5AD01TI6@1MGEd0Fe]K038b<P1UIFD0EUIF034a<@2ATI40NWYj071`L01MGEd202d];@1D
08J6QP1/K6`0FEUI030`<01=CDd0I6AT04M7A`13@d<0<S8b04U9B@2QXJ40RHV902LW9`0n?Sh0FUYJ
0:n_[`23Ph<07AdM04I6AP1GEeL0GemO08f=S@0c<c<092@T059BDP1QHF40@d=307moO`0/;2`0=cLg
06m_K`1DE5@0IVIV0;:b/P0/;2`0I6AT0:FUY@1iNGT0EEEE06=SH`0b<S80J6QX07]kN`1PH600CTi>
05EEE@0U9BD0IFEU09^KV`0l?3`0=SHf05aLG00];Bd0?Shn09RHV01;Bd/0G5aL06MWI`19BDT0N7Qh
05ADE00V9RH0ATI6059BDP0l?3`0V9RH0:^[Z`0/;2`0;Rh^04Q8B01LG5`0M7Ad0:6QX@0j>SX0@410
04m?C`0m?Cd0FUYJ08>3P`0O7al0=3@d06e]K@15ADD0`L713_ooool08@1ADE40Lg=c04Q8B019BDT0
EEEE07MgM`1eMGD0<c<c065QH@1[Jf/0@D5106EUI@1[Jf/0GEeM07UiN@27QhL0Fe]K06EUI@1RHV80
?clo02TY:@11@D40?Cdm05ADE00o?cl0CDe=03Xj>P1@D500H61P04A4A01:BTX0IfMW06ATI00206YZ
JP0505YJFP1TI6@0IFEU04=3@`12@T800P0m?Cd0100o?cl0CTi>0492@P0g=cL205ADE00l03dm?@19
BDT0@T9204=3@`0G5aL092@T04i>CP0l?3`0A4A40451@@1DE5@0BDU905ADE01:BTX0Bd];061PH01G
EeL0FEUI05]KF`0l?3`0KVi^05=CD`19BDT0>3Ph07EeM@0`<300@T9205EEE@0k>c/0HV9R04Q8B019
BDT0?clo04];B`0k>c/0@D5103hn?P1EEED09BDU02HV9P11@D40BTY:04i>CP1:BTX0?Shn03Ph>01;
Bd/0=3@d051@D01EEED0H61P05iNGP1<C4`0FEUI05YJFP1PH600HF5Q04E5A@1DE5@0ATI60P1JFUX0
8@1@D500KFe]059BDP18B4P0EEEE04i>CP0n?Sh0?Cdm0451@@1<C4`0CDe=04=3@`1DE5@0O7al03De
=@1DE5@0MGEe07inOP2OWil0N7Qh08:2PP1JFUX0SHf=06][J`1?Cdl0IVIV06QXJ013@d<0CTi>030`
<00n?Sh0:b/[03Ph>00205]KF`0406ATI01DE5@0@d=304E5A@80BTY:0400:BTY03/k>`1;Bd/0BTY:
05iNGP1MGEd0@D5105EEE@1DE5@0Ng]k03Ti>@1:BTX0;bl_06ATI00?3`l0?Cdm04M7A`0X:2P0CDe=
03hn?P1WIfL0;Rh^04];B`0g=cL0De=C05mOG`1/K6`0?Cdm07alO014A4@0BDU906QXJ029RHT0S8b<
03Ph>01_Kfl0EeMG05eMG@23Ph<0NGUi04m?C`15ADD0GemO04m?C`1PH600LW9b03@d=01HF5P0HV9R
02DU9@1GEeL0<C4a02DU9@1HF5P0DU9B07inOP1bLW80GemO05MGE`1ADE40C4a<04E5A@19BDT0AdM7
0P1CDe<0?@1LG5`0?clo05=CD`16ATH0RXZ:069RHP1_Kfl0GemO08N7Q`1hN7P0=CDe051@D018B4P0
BTY:05]KF`0_;bl0ATI606i^KP2?Shl0ADE504m?C`17AdL0U9BD08V9R@1QHF40Cdm?03/k>`1:BTX0
Hf=S06MWI`1>CTh0LW9b06EUI@1IFET0Cdm?07=cL`1FEUH0@T9206][J`1mOGd0BTY:08>3P`18B4P0
H61P05eMG@1DE5@0@d=3055AD@1?Cdl0?clo06e]K@1XJ6P0FUYJ05UIF@1cLg<0DE5A06=SH`1HF5P0
JVYZ05]KF`24Q8@04?ooool000kooooo0;H0VYZJ06ATI01?Cdl0PX:209:BTP21PH40F5QH04i>CP2?
Shl0Q8B409fMW@13@d<0Bd];06i^KP1bLW80E5AD06EUI@0k>c/0I6AT09NGU`0o?cl0J6QX06UYJ@0m
?Cd0Hf=S08:2PP0X:2P0G5aL03@d=013@d<0C4a<01HF5P0[:b/0Cdm?03/k>`11@D40BDU9010@400e
=CD0>3Ph04];B`1XJ6P0MWIf02d];@1CDe<0>CTi05QHF01iNGT0Lg=c03Xj>P1EEED0EUIF05iNGP1n
OWh05QHF02<S8`1WIfL0DE5A07]kN`10@400<c<c04Q8B01VIVH0=SHf07IfMP1ADE404A4A03/k>`1;
Bd/0=SHf06MWI`0I6AT09RHV04a<C01HF5P0A4A403dm?@0Y:BT0=SHf04Y:BP23Ph<0BDU904a<C011
@D40ATI60:FUY@2DU9@08b<S02d];@0i>CT0AdM70:VYZ@1HF5P07AdM0451@@1JFUX0ADE509RHV00e
=CD05aLG03Hf=P1ZJVX0GUiN09NGU`0D51@092@T038b<P1XJ6P0ADE5069RHP0N7Qh0=CDe065QH@1`
L700;bl_061PH00T92@0=SHf05eMG@1gMgL09bLW04e=C@0g=cL0MGEe0:j^[P18B4P0;Bd]05MGE`1J
FUX0Kfm_09fMW@0U9BD061PH04e=C@1YJFT0H61P08^;R`0Q8B40>SXj05aLG02@T900IfMW06QXJ00c
<c<0@d=306IVIP1[Jf/0Lg=c06EUI@0e=CD0E5AD05aLG01<C4`0Lg=c07moO`0Y:BT0CDe=08^;R`0j
>SX0EeMG04U9B@0N7Qh0De=C06IVIP11@D40WIfM02h^;P0O7al0PX:206UYJ@0i>CT0KVi^03<c<`18
B4P0OGem05EEE@11@D40FEUI038b<P1fMWH0_Kfm06=SH`80B4Q805h0ADE50:NWY`1YJFT092@T04U9
B@1TI6@0NGUi09RHV00^;Rh0@d=30492@P1QHF40V9RH02@T900/;2`0JFUY03/k>`1VIVH0<30`01lO
7`1FEUH0Ogmo04Q8B01SHf<0:2PX055AD@1mOGd0>CTi079bLP1_Kfl06a/K05QHF01[Jf/0B4Q8061P
H00W9bL0;Rh^069RHP0^;Rh0Fe]K04Y:BP0c<c<0GUiN09ZJVP0f=SH0ADE504i>CP1DE5@0IfMW08n?
S`0Y:BT0De=C05ADE01LG5`0IVIV092@T00f=SH0=cLg071`L01fMWH0OWin09jNWP0U9BD0?Cdm05MG
E`1HF5P0Zj^[09:BTP0W9bL0Hf=S071`L01LG5`0RXZ:05iNGP0H61P0?Shn08:2PP1>CTh0HF5Q03`l
?00P8200ADE508N7Q`1WIfL0?Cdm05iNGP0/;2`0=SHf07IfMP1=CDd0=cLg03hn?P0S8b<0>SXj3_oo
ool0;@1aLG40Q8B406e]K@1:BTX0IVIV05eMG@1KFe/09bLW07QhN01>CTh0GemO06a/K01XJ6P0GemO
079bLP28R8P0BDU904Y:BP1EEED0D51@0410@014A4@0DE5A05aLG019BDT0Fe]K04I6AP1@D500>c/k
05=CD`1KFe/0FEUI061PH01nOWh0XJ6Q07EeM@1VIVH0IFEU03dm?@0j>SX0@T9204U9B@1WIfL0@D51
05MGE`020410@01V05UIF@0m?Cd0DE5A02/[:`10@4005aLG020P8010@400?Cdm03Ph>00O7al0@d=3
04m?C`14A4@0ATI604];B`1FEUH0@D5104m?C`1VIVH0@D5105MGE`0n?Sh0ADE504=3@`1fMWH0AdM7
05IFEP1IFET0ATI603lo?`0X:2P0=SHf03`l?00g=cL0=3@d02`/;00[:b/0ATI602h^;P0W9bL0<c<c
04M7A`15ADD0?clo04=3@`1JFUX0I6AT04A4A018B4P0D51@03hn?P16ATH0Bd];05aLG01TI6@0HV9R
05YJFP1HF5P0EUIF05ADE01cLg<0IVIV05IFEP1iNGT0IVIV051@D01MGEd0I6AT05IFEP0k>c/0Bd];
05=CD`1WIfL0I6AT069RHP2;Rh/0ATI604a<C01UIFD0De=C07AdM01RHV80E5AD04e=C@1:BTX0OWin
04A4A01LG5`0JVYZ04i>CP1KFe/09bLW03<c<`0a<C40Bd];05YJFP1BDU80>CTi03Xj>P1GEeL0E5AD
0P1>CTh0GP0X:2P0?Cdm05QHF01VIVH0DE5A03dm?@1LG5`0IVIV03Hf=P1:BTX0=cLg055AD@0^;Rh0
CDe=020P801:BTX0CTi>04Y:BP1TI6@0>c/k06QXJ00/;2`0@T9203Lg=`12@T80GemO06IVIP0W9bL0
D51@02XZ:P0/;2`0AdM704U9B@1]KFd0@d=308Z:RP1fMWH0MgMg0:2PX02e]KD0SXj>04];B`1`L700
DE5A06QXJ01MGEd0L71`06i^KP1`L700>c/k09:BTP10@400=SHf059BDP0d=3@0=cLg051@D01JFUX0
Cdm?04U9B@0h>3P0C4a<03Xj>P0i>CT0?clo05=CD`1/K6`0:2PX04a<C01GEeL0GUiN04M7A`1?Cdl0
CDe=071`L01TI6@0>SXj04m?C`10@400>c/k03lo?`0l?3`0Lg=c03lo?`1:BTX0B4Q805eMG@0o?cl0
@410051@D00k>c/0De=C04U9B@1FEUH203dm?@0O02LW9`16ATH0NWYj030`<01CDe<0MgMg05YJFP1A
DE40HV9R0861P@1PH600YjNW06m_K`2OWil0EUIF04E5A@0h>3P0IVIV03hn?P17AdL0Jf][06ATI01b
LW80F5QH07=cL`1fMWH0W9bL06IVIP21PH40GemO07inOP0@ooooo`003_ooool0E01TI6@0UYJF04];
B`1`L700EEEE092@T01BDU80F5QH05iNGP17AdL0TI6A03Ph>011@D40BDU901hN7P1iNGT0TY:B03<c
<`0`<300Bd];04i>CP22PX80NGUi02d];@1<C4`0JVYZ06EUI@2_[jl0=CDe03/k>`1PH600Bd];07If
MP2?Shl07AdM03dm?@1OGel0KFe]08Z:RP19BDT08b<S051@D02FUYH0DE5A05mOG`0X:2P0:BTY05]K
F`2@T900IFEU06i^KP0M7Ad0<c<c05UIF@1=CDd0;bl_03Lg=`0f=SH0DE5A06ATI00Q8B408b<S04E5
A@0c<c<0MgMg08V9R@0a<C40:2PX055AD@1@D500/K6a051@D00@41009RHV04m?C`1>CTh0K6a/03hn
?P0I6AT0:2PX04];B`1LG5`0P820059BDP80:b/[0<00RHV904e=C@24Q8@0E5AD024Q8@0Z:RX0G5aL
03Xj>P1LG5`0;Bd]02PX:00W9bL0EeMG02LW9`1:BTX0=3@d03Xj>P1JFUX0<C4a02LW9`19BDT0<c<c
061PH01oOgl0:b/[01hN7P13@d<0HF5Q07MgM`22PX80<C4a02PX:012@T80Ph>306ATI01kNg/0BDU9
03Hf=P1EEED0I6AT09:BTP2NWYh0=3@d03lo?`1_Kfl0MgMg08j>SP27QhL0;bl_0410@01gMgL0@D51
05=CD`1`L7009bLW04A4A020P800<C4a07MgM`0e=CD09BDU04M7A`21PH40<C4a06UYJ@0o?cl0?Cdm
051@D02YZJT0=cLg04i>CP0o?cl0;bl_08F5Q@2@T9009RHV03Lg=`16ATH0@4100;>c/`1NGUh06ATI
0451@@1jNWX0JVYZ0;>c/`10@4009BDU055AD@1jNWX0QXJ605YJFP0];Bd0<30`06QXJ02TY:@0SHf=
0410@00`<300<c<c06QXJ015ADD0@d=3038b<P11@D40IVIV04e=C@13@d<0De=C03`l?01IFET0De=C
02LW9`1@D500:b/[08>3P`1aLG40;2`/0451@@1:BTX0>3Ph092@T01?Cdl08B4Q04U9B@0h>3P0TI6A
0:6QX@0J6QX0AdM704m?C`1NGUh0^;Rh04m?C`0U9BD0J6QX05EEE@2c/k<0PX:2030`<00d=3@0EeMG
06e]K@1aLG40KFe]03@d=013@d<0FEUI05QHF025QHD0K6a/03Ph>01<C4`0MWIf05]KF`1GEeL0A4A4
02PX:01OGel0K6a/06][J`0g=cL0GemO038b<P1JFUX0S8b<03<c<`0g=cL0>CTi030`<025QHD0WIfM
0451@@0o?cl0?Cdm065QH@1iNGT0FUYJ01DE5@0N7Qh0<c<c04];B`1gMgL0R8R8024Q8@0m?Cd0AdM7
03dm?@1]KFd0K6a/01lO7`11@D4>ooooo`0J04A4A01QHF40=CDe03Lg=`1;Bd/0MWIf08J6QP0f=SH0
JVYZ05=CD`0n?Sh0JFUY075aL@1XJ6P0T92@09:BTP15ADD0ATI6075aL@13@d<09RHV03dm?@0];Bd0
CDe=04A4A01QHF4204a<C00T04Y:BP19BDT0EUIF05eMG@1NGUh0EeMG05iNGP1RHV80K6a/059BDP0Z
:RX0;Rh^05ADE01KFe/0CTi>0492@P1;Bd/0B4Q803/k>`1MGEd0D51@04i>CP0Z:RX0A4A401PH600_
;bl0Cdm?03`l?00m?Cd0;bl_04Y:BP15ADD0A4A404=3@`16ATH0I6AT0P1PH600V`1fMWH0=SHf07em
O@13@d<0A4A403`l?01JFUX0<c<c0451@@1=CDd0<c<c04A4A00a<C40?Shn034a<@17AdL0=CDe03@d
=011@D40DU9B02<S8`0];Bd0D51@03Xj>P19BDT0EEEE03/k>`0c<c<0BTY:04m?C`0i>CT0E5AD051@
D01CDe<0@41004i>CP1?Cdl0IfMW04Q8B01PH600C4a<05EEE@1jNWX0WIfM079bLP23Ph<0HF5Q07Ad
M01]KFd0EEEE065QH@1BDU80Bd];04e=C@0i>CT0=cLg069RHP2?Shl0FEUI065QH@1hN7P0JVYZ0:6Q
X@1cLg<0JVYZ06e]K@1TI6@0Ti>C06e]K@1jNWX0MgMg07EeM@1^KVh0=cLg03Xj>P0f=SH0ADE508F5
Q@1EEED0BDU903dm?@0o?cl0GemO04a<C01EEED0:2PX02<S8`0m?Cd0JVYZ05=CD`1GEeL0@d=305IF
EP0d=3@0IfMW03Xj>P0l?3`09BDU04a<C00T92@0BTY:059BDP0e=CD0ATI60492@P1[Jf/0CTi>059B
DP16ATH0DE5A061PH01mOGd08R8R04i>CP0`<300:BTY04U9B@1VIVH0MGEe02/[:`31`L40TY:B06IV
IP1CDe<0IfMW0820P01HF5P0M7Ad03Xj>P1ZJVX0K6a/04Y:BP1UIFD0J6QX01<C4`12@T807alO00d=
3@1ADE409RHV034a<@1DE5@0Jf][05eMG@1JFUX0F5QH06e]K@0k>c/0@41004e=C@20P800PH6101/K
6`1ADE40;2`/0080=CDe03L0D51@04e=C@1UIFD0H61P028R8P1BDU80CDe=04E5A@1:BTX0?clo059B
DP1NGUh0H61P020P8010@4009RHV04=3@`1HF5P0De=C04i>CP14A4@0CDe=03Lg=`18B4P0@D5105]K
F`1cLg<0G5aL09NGU`2`/;00PX:206IVIP2SXj<0WinO04e=C@1PH600?3`l04a<C01DE5@0KVi^04U9
B@0i>CT0>c/k0492@P1TI6@0E5AD04m?C`19BDT0Bd];04I6AP1RHV80GUiN08j>SP1KFe/0IFEU013o
oooo000>ooooo`3g061PH027QhL0O7al07moO`12@T80LG5a05mOG`2j^[X0Ti>C04];B`1cLg<0IVIV
08J6QP1kNg/0?Cdm03Ti>@1fMWH0C4a<05mOG`0_;bl06ATI05IFEP1MGEd0B4Q80:NWY`0g=cL0:RXZ
06MWI`1aLG40De=C02HV9P0S8b<0;Rh^06e]K@11@D409bLW03<c<`0W9bL0PH6107MgM`0L71`0:2PX
03Xj>P1RHV80^KVi05EEE@0/;2`0<c<c04e=C@27QhL0Fe]K01`L700i>CT0?3`l05UIF@1CDe<0KFe]
01@D501=CDd0Ph>3069RHP1aLG40HF5Q02HV9P1CDe<0IVIV04Q8B014A4@0LG5a02DU9@1IFET0IVIV
0451@@1DE5@0D51@01XJ6P19BDT0OGem03Lg=`0n?Sh0?Cdm01`L701;Bd/0J6QX02l_;`0j>SX08b<S
02d];@19BDT0HF5Q01lO7`19BDT0820P034a<@2XZ:P0KFe]02HV9P0d=3@0DE5A0492@P2?Shl0OGem
02`/;01KFe/0CDe=06UYJ@2c/k<0A4A401dM7@1ADE40R8R80451@@21PH40>CTi01dM7@1PH600ATI6
03dm?@2>SXh0820P02`/;01VIVH0Fe]K04M7A`15ADD04a<C03Hf=P1eMGD0IFEU03`l?00a<C407alO
05EEE@2TY:@0HV9R04=3@`0o?cl0=cLg079bLP2FUYH0<C4a024Q8@1=CDd0NGUi0:^[Z`1TI6@092@T
0410@01OGel0I6AT0;>c/`1GEeL092@T030`<01DE5@0G5aL08n?S`0b<S80<c<c04A4A01IFET0>SXj
08B4Q00T92@08b<S069RHP24Q8@0@d=306MWI`0V9RH0?Cdm06QXJ01hN7P0?clo07EeM@0l?3`0NGUi
0:VYZ@1>CTh0?Cdm055AD@0b<S80KVi^04e=C@0W9bL0E5AD03`l?01VIVH0P82002d];@17AdL0C4a<
04Y:BP26QXH0]kNg03`l?01LG5`0FUYJ06YZJP2ATI40>SXj0451@@1aLG40EeMG09^KV`0i>CT0;2`/
04M7A`1MGEd0M7Ad06MWI`0I6AT0@T92071`L00h>3P0HV9R03Lg=`14A4@0LG5a06UYJ@16ATH0F5QH
02/[:`1`L700OGem03Xj>P1>CTh0Fe]K03Xj>P1oOgl0Xj>S04E5A@1;Bd/0De=C0492@P2>SXh0Y:BT
03/k>`0i>CT0EEEE05QHF01jNWX0NGUi01lO7`0Y:BT0@41006ATI00206QXJ00M02HV9P0o?cl0B4Q8
04M7A`24Q8@0E5AD02XZ:P1aLG40QhN703De=@1^KVh0H61P02PX:01_Kfl0OWin03Hf=P1MGEd0ADE5
02PX:00g=cL0UYJF03Xj>P0f=SH0Fe]K01dM7@0o?cl0KFe]03Ti>@14A4@03_ooool0F01BDU80QhN7
07AdM01<C4`0B4Q804];B`17AdL0;bl_07moO`1LG5`0@41006m_K`1aLG40HV9R09JFUP20P800EUIF
04Y:BP1bLW80ATI60451@@16ATH0>SXj04e=C@14A4@0KVi^04Q8B01JFUX0Bd];05QHF01TI6@0Lg=c
075aL@1hN7P0Lg=c06i^KP20P800LW9b05EEE@14A4@0ADE5051@D01ZJVX0DU9B051@D015ADD0=cLg
055AD@19BDT0ADE503Hf=P13@d<09BDU03Hf=P19BDT0;Rh^03/k>`0Y:BT0=3@d04I6AP1<C4`0D51@
055AD@1<C4`0IVIV05eMG@1QHF40?Cdm08J6QP1;Bd/0A4A4030`<01UIFD0BDU905EEE@18B4P0=3@d
04a<C00f=SH0=3@d03Xj>P0m?Cd0=SHf034a<@0c<c<0C4a<020P800j>SX2034a<@1e03/k>`0g=cL0
?Shn03<c<`1IFET0>SXj0451@@1>CTh0IVIV04E5A@1>CTh0HV9R06=SH`1CDe<0EUIF04];B`1@D500
CDe=05QHF01MGEd0N7Qh08b<S01QHF40Jf][06IVIP0n?Sh0EEEE059BDP15ADD0ATI604A4A025QHD0
I6AT07emO@14A4@0LG5a07AdM01QHF40YjNW09BDU023Ph<0JFUY07YjNP1gMgL0Bd];06a/K01^KVh0
DE5A05mOG`12@T80B4Q804I6AP1;Bd/0MGEe05aLG01[Jf/0D51@0492@P1CDe<0Bd];05=CD`0O7al0
=cLg04a<C02BTY80HV9R05]KF`1=CDd0GEeM03dm?@1EEED0>SXj05iNGP0W9bL0Bd];02`/;015ADD0
BDU903Ti>@1@D500?clo05MGE`0b<S80=3@d03Ti>@1?Cdl0Fe]K06e]K@0j>SX0Bd];03Lg=`0d=3@0
GemO05IFEP1VIVH09BDU055AD@19BDT0EUIF08N7Q`2][Jd0][Jf09ZJVP1nOWh0>c/k05YJFP2DU9@0
DU9B08B4Q01nOWh06QXJ061PH00T92@04a<C075aL@0e=CD0FUYJ0080EUIF04D0EEEE05mOG`1:BTX0
@41003Ph>01:BTX0?Shn05aLG01JFUX0;2`/06YZJP1KFe/0>c/k03lo?`1/K6`0CDe=07]kN`1^KVh0
?Shn05UIF@10@400ADE505aLG01>CTh0Fe]K059BDP1IFET0=SHf07UiN@0Y:BT0EEEE05iNGP19BDT0
<C4a04A4A01EEED0B4Q804];B`0d=3@0Cdm?06QXJ00n?Sh0EEEE05QHF01BDU80@d=306a/K02:RXX0
FUYJ0:FUY@16ATH0DE5A0492@P1<C4`0?Cdm04];B`0d=3@0?clo04e=C@1QHF40FUYJ05UIF@1ADE40
E5AD07AdM01>CTh0I6AT04e=C@1ZJVX04?ooool000kooooo04L0OGem04Y:BP1OGel0O7al05mOG`1d
M7@0AdM705]KF`1]KFd0GUiN06EUI@0Y:BT0EEEE09ZJVP1GEeL0=SHf04I6AP1_Kfl0_;bl07YjNP0`
<300Fe]K06ATI01LG5`0^[Zj03Hf=P0W9bL0;bl_04i>CP1QHF40C4a<034a<@1:BTX0CTi>04M7A`1c
Lg<0Fe]K02DU9@0J6QX0Fe]K04I6AP1PH600CTi>028R8P19BDT0?3`l03dm?@1[Jf/0;bl_0492@P1W
IfL09RHV03dm?@0j>SX071`L03hn?P1CDe<08R8R05EEE@1SHf<0:RXZ03Ph>01NGUh08B4Q0492@P0e
=CD0<S8b03Xj>P1YJFT09BDU03dm?@0202h^;P1P065QH@1cLg<07AdM02PX:01dM7@0MGEe06][J`1O
Gel06ATI03Ph>01KFe/0M7Ad0:b/[00l?3`051@D03Ti>@1dM7@0ADE508:2PP0K6a/04a<C030`<01I
FET0DU9B05mOG`0I6AT07alO04Q8B01nOWh0ADE505MGE`0B4Q80:2PX061PH01QHF4092@T05aLG00g
=cL0BTY:07UiN@19BDT0:2PX05ADE00];Bd0GemO0;Ng]`1eMGD04a<C0451@@1<C4`0HV9R0820P00I
6AT0>SXj051@D01LG5`0OGem05MGE`0F5QH0<S8b05QHF01ZJVX0MGEe03De=@0R8R80=3@d075aL@1<
C4`0FUYJ03lo?`0U9BD0HF5Q07inOP1<C4`0Rh^;0451@@0f=SH0Lg=c08R8R00Z:RX0>c/k059BDP15
ADD0N7Qh069RHP0E5AD0EEEE04M7A`0a<C40OGem04a<C00P8200C4a<03Hf=P1aLG40ZZZZ0P0];Bd0
F`1@D500@T9207emO@2j^[X0Fe]K030`<01JFUX0HF5Q08Z:RP2>SXh0:2PX04I6AP18B4P0@d=307Mg
M`0f=SH0=cLg06m_K`1IFET0H61P04Q8B00c<c<0KFe]0;2`/01QHF40J6QX03/k>`1BDU80Jf][04Y:
BP18B4P0<c<c03Hf=P1UIFD0N7Qh02HV9P11@D40<c<c069RHP2OWil0ADE503Hf=P1GEeL0<c<c06i^
KP0f=SH0;Bd]05EEE@1GEeL0GemO08>3P`0Y:BT0Fe]K06UYJ@19BDT0IFEU09jNWP0Z:RX0JFUY09>C
T`1>CTh0TI6A07IfMP0h>3P0EeMG08Z:RP1dM7@0Lg=c07UiN@0T92@0CTi>061PH00m?Cd0@d=305iN
GP0Q8B40>CTi08b<S00a<C40GUiN03Ph>00l?3`0FEUI07QhN00_;bl0C4a<04I6AP1=CDd0OGem08B4
Q00g=cL00P14A4@03P0c<c<0KVi^09>CT`0Y:BT08B4Q05UIF@13@d<0Ng]k0820P00R8R80@41004E5
A@14A4@0I6AT3_ooool0o`1>CTh0N7Qh03lo?`0/;2`0ATI607AdM01QHF40;Bd]08V9R@1GEeL0@D51
06][J`1nOWh0IfMW0820P01]KFd0AdM705=CD`1^KVh0?Shn01lO7`0g=cL0:BTY04];B`19BDT0H61P
03Lg=`1LG5`0KVi^051@D019BDT0D51@06QXJ01ZJVX0Lg=c0820P01mOGd0GemO04Q8B010@400A4A4
04E5A@1?Cdl0@41005ADE011@D40DE5A06EUI@1:BTX0C4a<04I6AP1SHf<0:b/[028R8P1=CDd0@d=3
04E5A@0Y:BT0FUYJ05mOG`1DE5@0DE5A04Y:BP1>CTh0@D5104U9B@1WIfL0De=C05mOG`1=CDd0?3`l
034a<@1>CTh0;2`/03Ph>01NGUh0Bd];055AD@0U9BD0<c<c03`l?00g=cL09bLW03De=@0g=cL0FEUI
01dM7@0l?3`0C4a<03De=@1?Cdl0?Shn04I6AP0k>c/0IFEU04=3@`10@400GEeM04U9B@1EEED0=3@d
0410@017AdL0CDe=05EEE@1VIVH0F5QH04U9B@1bLW80P82008b<S02?Shl0FUYJ03hn?P1/K6`0=cLg
03`l?00^;Rh0B4Q804a<C016ATH0CDe=05mOG`21PH40D51@05=CD`1XJ6P0JFUY08:2PP1GEeL0GemO
06ATI01VIVH0L71`03lo?`1HF5P0JVYZ06i^KP1[Jf/09BDU034a<@1JFUX0H61P096AT@1EEED0ATI6
03Ti>@0o?cl0H61P051@D01OGel0:2PX03Xj>P1=CDd0V9RH05aLG01=CDd0C4a<05IFEP19BDT0IFEU
03De=@1ADE40<30`05UIF@0C4a<0AdM704a<C00f=SH0D51@030`<01lO7`0=SHf03/k>`17AdL0BDU9
04i>CP1TI6@0AdM706a/K00f=SH09bLW03dm?@1bLW80Ogmo038b<P1UIFD0A4A4061PH01WIfL0PH61
0:n_[`27QhL0H61P04E5A@1@D500F5QH0451@@1IFET0GemO01TI6@21PH40:b/[01DE5@1;Bd/0>c/k
03dm?@16ATH0EeMG04e=C@1?Cdl0?clo04Q8B00c<c<0@d=303/k>`1CDe<0K6a/02LW9`0m?Cd0=3@d
03dm?@12@T80@d=304];B`1OGel0DE5A034a<@1KFe/0ADE503Ti>@1:BTX0>3Ph06i^KP1@D500AdM7
01/K6`0n?Sh09RHV06MWI`1TI6@0MgMg04a<C01@D500@T9205aLG01MGEd09RHV0492@P1RHV807alO
07QhN01]KFd0OGem059BDP003@1oOgl0MgMg04e=C@1iNGT0EeMG07QhN01;Bd/0D51@02/[:`15ADD0
61PH03<c<`0o?cl00P1NGUh02017AdL0DE5A07inOP2DU9@0GUiN07]kN`1NGUh0KVi^4?ooool000ko
oooo0:80X:2P061PH01<C4`0H61P07UiN@2FUYH0>3Ph065QH@27QhL0RXZ:08N7Q`0o?cl0=CDe05QH
F01PH600JVYZ06a/K00h>3P0IFEU079bLP0o?cl0N7Qh06EUI@0n?Sh0WinO06a/K01MGEd0M7Ad02LW
9`1>CTh0MGEe02XZ:P1;Bd/0GUiN02l_;`0];Bd0OWin02LW9`1TI6@0;2`/02d];@1bLW80OGem03Lg
=`1ADE407alO04m?C`2MWId0GemO030`<01>CTh0=CDe061PH02KVi/08R8R030`<01NGUh0=cLg071`
L01DE5@0:RXZ02l_;`1@D500ATI60:>SX`1UIFD061PH03/k>`0j>SX0BDU905]KF`0K6a/0:b/[03Xj
>P1BDU80;bl_03`l?00:2PX0;bl_05ADE01RHV80<c<c069RHP0H61P0;2`/0492@P1QHF40:2PX02l_
;`0X:2P0EUIF07UiN@13@d<06ATI034a<@0V9RH0G5aL09:BTP0e=CD04Q8B02h^;P13@d<0HF5Q0820
P00H61P071`L03Hf=P1DE5@0>3Ph09:BTP0K6a/05aLG059BDP22PX80;Rh^0820P00/;2`07alO03`l
?01iNGT0@T9206EUI@0m?Cd0CTi>079bLP1FEUH0GemO06e]K@16ATH0B4Q8092@T01ADE40>c/k06UY
J@0e=CD0F5QH05mOG`0T92@0?Cdm04=3@`1>CTh0O7al06a/K00V9RH0BTY:05YJFP1CDe<0TY:B06YZ
JP0k>c/0Bd];05iNGP15ADD0Ogmo0820P00L71`0AdM707QhN016ATH0_Kfm04i>CP0D51@206ATI01X
05=CD`2IVIT0C4a<01LG5`1KFe/0HV9R05YJFP20P8009RHV02TY:@1=CDd0KFe]06=SH`19BDT06QXJ
03`l?01hN7P0K6a/05IFEP0G5aL0@d=30:>SX`1GEeL0>CTi0451@@1BDU80TI6A08j>SP0a<C40<c<c
06QXJ01VIVH0QXJ602DU9@0m?Cd0LW9b06a/K02;Rh/09BDU03Ph>01ADE40Cdm?05aLG024Q8@0@410
03Hf=P1fMWH0BDU906QXJ01mOGd07QhN04Q8B017AdL0Bd];05MGE`0I6AT0>c/k05UIF@0n?Sh0GEeM
05MGE`0e=CD0HV9R05UIF@0k>c/0GEeM04e=C@0i>CT0G5aL07QhN00i>CT0G5aL0410@01<C4`0F5QH
06MWI`0g=cL0A4A404a<C01EEED0OWin07EeM@0L71`0;bl_05IFEP1SHf<0WYjN071`L00Y:BT0DU9B
06ATI01_Kfl0X:2P02`/;00Y:BT0I6AT08^;R`1LG5`0O7al03Ti>@0S8b<0Bd];09:BTP1aLG4205EE
E@08024Q8@0h>3P0Vi^K06EUI@16ATH0D51@02LW9`0m?Cd>ooooo`1n05]KF`2FUYH0I6AT03Hf=P17
AdL0P82006QXJ011@D40RHV906QXJ01BDU80I6AT071`L01RHV80Jf][07YjNP12@T80BTY:05=CD`1G
EeL0;Bd]03`l?00_;bl0@41004U9B@1;Bd/0A4A405ADE01IFET0G5aL06QXJ01UIFD0F5QH07IfMP2=
SHd0OGem09ZJVP1TI6@0C4a<04i>CP1EEED0F5QH06m_K`1=CDd0F5QH04a<C00h>3P0CDe=03lo?`1F
EUH0;2`/04a<C00H61P0;Rh^05ADE016ATH0?3`l02l_;`1=CDd0>CTi03/k>`1:BTX0@T9204i>CP17
AdL0CTi>06ATI00/;2`0De=C04A4A01=CDd0B4Q806a/K00g=cL0?Shn04];B`0W9bL0AdM702l_;`0]
;Bd0;2`/03lo?`0h>3P0<c<c038b<P19BDT0?Cdm03<c<`1<C4`0?Cdm04I6AP15ADD0?clo04Q8B01L
G5`0ADE504Y:BP1JFUX0BDU9055AD@1BDU80CDe=05QHF01?Cdl0H61P0410@01@D500Bd];06i^KP1V
IVH0N7Qh08n?S`27QhL0ZJVY0?Shn012@T80DE5A03lo?`0l?3`0?Cdm05UIF@1OGel0I6AT07]kN`1K
Fe/0EUIF0P1hN7P0;@2:RXX0I6AT08Z:RP1nOWh0F5QH06a/K01UIFD0JVYZ061PH00d=3@0GemO02`/
;00m?Cd0BTY:03Xj>P1DE5@0F5QH04Q8B01>CTh0FEUI05aLG021PH40MgMg028R8P0l?3`0HV9R08>3
P`29RHT0JFUY05eMG@1CDe<0?3`l065QH@1CDe<0@D51030`<018B4P0<c<c04Q8B01?Cdl0CTi>06AT
I00l?3`0KVi^03dm?@0203Ph>01W04];B`0m?Cd0JFUY02<S8`1EEED0AdM7038b<P1=CDd0BTY:05mO
G`0^;Rh0JVYZ04e=C@12@T80DU9B0492@P1[Jf/0RXZ:06=SH`0/;2`0DE5A05mOG`1;Bd/0Fe]K06EU
I@0S8b<0DE5A030`<00G5aL0DE5A04I6AP0c<c<0C4a<05EEE@1FEUH0B4Q803/k>`1FEUH0@41004A4
A01BDU80G5aL06i^KP16ATH0C4a<04I6AP18B4P0A4A404i>CP0d=3@0JFUY05mOG`0[:b/0Bd];04M7
A`1FEUH0B4Q80451@@1HF5P0HV9R05aLG00e=CD0BDU903<c<`1YJFT0Bd];02`/;00Q8B40<C4a03De
=@16ATH0BDU904I6AP1GEeL0T92@02LW9`1/K6`0DE5A04Y:BP0g=cL0PH6106IVIP1JFUX0UYJF04E5
A@1BDU80D51@07=cL`0i>CT0BTY:02XZ:P1;Bd/0C4a<07emO@1TI6@0NWYj05aLG01ADE40GemO04e=
C@27QhL0EEEE08>3P`0@ooooo`003_ooool0EP1[Jf/0LG5a05=CD`1dM7@0BDU906][J`1UIFD0NWYj
07IfMP1:BTX0L71`05IFEP1OGel0MWIf02HV9P14A4@0KFe]03lo?`1_Kfl0?3`l034a<@14A4@0IVIV
04U9B@1[Jf/09RHV065QH@21PH40B4Q805IFEP1HF5P0=cLg069RHP1_Kfl0=cLg04e=C@1;Bd/0>c/k
07AdM01BDU80:RXZ055AD@13@d<0D51@08N7Q`0[:b/0;Rh^05]KF`18B4P0G5aL04a<C00U9BD0?Cdm
059BDP14A4@0Bd];06=SH`0T92@0ATI605mOG`0o?cl0FEUI06YZJP0^;Rh0DE5A04m?C`0i>CT0Fe]K
05aLG00[:b/0IfMW04a<C00n?Sh0F5QH03dm?@0K6a/0G5aL04e=C@0W9bL0BDU903<c<`0Z:RX0OWin
04A4A00^;Rh0GEeM0P0`<3008`1?Cdl0ATI603<c<`1JFUX0?Shn03Ph>01gMgL0=3@d02h^;P0i>CT0
CDe=03/k>`20P800EUIF02h^;P1>CTh0=3@d04Q8B01HF5P09BDU01dM7@1QHF40>3Ph04Y:BP1eMGD0
>CTi03De=@1RHV80=cLg04A4A02>SXh0?Cdm034a<@1gMgL0HV9R0080@41001H0>SXj04a<C01cLg<0
F5QH0410@018B4P0Fe]K06][J`2PX:00BTY:04E5A@1=CDd0BDU907QhN01[Jf/0=CDe04I6AP1JFUX0
E5AD09VIV@1;Bd/0;Bd]0P1IFET0P@1QHF40][Jf065QH@0d=3@0CDe=069RHP1DE5@0MGEe034a<@0M
7Ad0IfMW07inOP11@D40JVYZ03De=@0c<c<0IfMW06YZJP1DE5@0F5QH03Xj>P1OGel0UYJF04m?C`1=
CDd0C4a<04=3@`1lO7`0MGEe034a<@1OGel0DE5A03dm?@2<S8`0B4Q803@d=01FEUH0ATI604Y:BP1b
LW80:RXZ05]KF`20P800E5AD07=cL`1^KVh0<S8b05mOG`1JFUX0E5AD08b<S00b<S80D51@05aLG01B
DU80TY:B04];B`0U9BD0EeMG071`L01?Cdl0GEeM02`/;010@400Lg=c038b<P1RHV80=3@d05UIF@22
PX80A4A404];B`1IFET0CTi>0861P@1kNg/0<c<c05MGE`1KFe/0AdM707]kN`1_Kfl0?Cdm04I6AP1E
EED0D51@09:BTP1OGel0=cLg04a<C01ADE40ADE506YZJP1aLG40:BTY05aLG01OGel0=SHf05aLG02G
UiL0>SXj06YZJP20P800Bd];05iNGP12@T80?3`l05mOG`1PH600>3Ph05]KF`1ADE40:2PX06e]K@1f
MWH0:BTY055AD@1FEUH0<30`05]KF`1[Jf/07QhN03Hf=P1ADE40?Cdm05=CD`1gMgL08R8R03dm?@0>
ooooo`0o05ADE01eMGD0IfMW04Q8B01BDU80IVIV05eMG@0k>c/0FEUI05QHF01BDU80JVYZ06UYJ@1c
Lg<0M7Ad07inOP16ATH0CTi>075aL@1?Cdl0;bl_04E5A@0b<S80BDU904Q8B01GEeL0@D5105MGE`1O
Gel0DU9B04];B`1eMGD0GemO071`L01eMGD0TY:B09jNWP1QHF40@41004i>CP11@D40A4A405aLG01?
Cdl0DE5A04M7A`1BDU80G5aL03Hf=P1:BTX0?clo05YJFP0W9bL0;Rh^059BDP1CDe<0DE5A02l_;`1:
BTX0B4Q8055AD@1>CTh0C4a<0080DE5A0640E5AD05aLG017AdL0MWIf05IFEP1IFET0@41006UYJ@19
BDT0ADE505aLG00o?cl0CTi>02l_;`0j>SX0>CTi0410@00e=CD0<C4a03Ph>01?Cdl0;Rh^04=3@`15
ADD0?Shn04Y:BP13@d<0CTi>04I6AP1HF5P0AdM704U9B@1FEUH0CTi>04Y:BP18B4P0DU9B05MGE`1D
E5@0D51@04i>CP1ADE40BDU9079bLP1^KVh0M7Ad08j>SP2d];@0c/k>0<G5a@1HF5P0I6AT04a<C013
@d<0B4Q806ATI01eMGD0Q8B408V9R@1VIVH0GEeM07MgM`1]KFd0OWin06e]K@1oOgl0IfMW07=cL`1V
IVH0>3Ph05eMG@1ZJVX0B4Q805QHF00];Bd0@T92055AD@1NGUh0K6a/051@D01LG5`0DE5A03`l?01F
EUH0H61P05YJFP0C4a<0<c<c05ADE01fMWH0L71`05QHF01;Bd/0CTi>04Q8B01RHV80>3Ph0080>c/k
0780DE5A028R8P18B4P0CDe=0451@@1ADE40=SHf061PH00d=3@0CTi>03/k>`16ATH0@41005YJFP0S
8b<0EUIF0451@@0a<C40FEUI05QHF01eMGD0>3Ph05eMG@1ADE40B4Q80492@P1:BTX0L71`03Lg=`1T
I6@0>c/k05MGE`1^KVh0H61P05mOG`1YJFT08B4Q05aLG01CDe<0;Bd]05=CD`1ADE40HF5Q05EEE@1H
F5P0C4a<04i>CP11@D40BTY:03dm?@1<C4`0CDe=05UIF@1WIfL0ATI6069RHP1XJ6P0D51@05UIF@1N
GUh0Bd];06IVIP1NGUh0>3Ph05YJFP0i>CT0A4A405EEE@0n?Sh0HF5Q06UYJ@1>CTh0=3@d055AD@0f
=SH0L71`06ATI01VIVH0>CTi04E5A@0`<300Cdm?055AD@0n?Sh0B4Q808:2PP13@d<0K6a/04Q8B024
Q8@0DE5A09:BTP1VIVH0JFUY09nOW`1EEED0IFEU05QHF01NGUh0A4A404E5A@0i>CT0?3`l0451@@1Q
HF40F5QH05eMG@1FEUH0I6AT06QXJ01ADE40N7Qh05=CD`1`L70@ooooo`003_ooool0o`2a/K40GEeM
04e=C@2ATI40KVi^07=cL`0i>CT0N7Qh09^KV`1]KFd0Ng]k038b<P1<C4`0SHf=08>3P`16ATH0Bd];
05YJFP1`L700Jf][03Ti>@1NGUh0BDU904];B`2NWYh0Cdm?04=3@`1EEED07AdM06QXJ01RHV80<c<c
03De=@0n?Sh0>CTi06][J`1NGUh0:BTY04=3@`1BDU80GUiN08>3P`14A4@07AdM05]KF`0o?cl0JVYZ
0:FUY@11@D409RHV055AD@0l?3`0Ph>308b<S00E5AD0:BTY055AD@17AdL0LG5a04i>CP0@410071`L
05aLG00k>c/0MGEe03Hf=P0L71`092@T06UYJ@0f=SH0CDe=010@4015ADD0UYJF061PH00d=3@0ATI6
03Xj>P0];Bd0NWYj07EeM@0G5aL0>c/k03dm?@14A4@0T92@0410@00H61P0BDU904a<C01JFUX0WinO
018B4P0K6a/0B4Q804i>CP15ADD0Fe]K02DU9@0h>3P0BDU906MWI`1JFUX0820P02/[:`16ATH0IfMW
04=3@`0g=cL0KFe]02LW9`0X:2P0C4a<04m?C`0a<C40B4Q802DU9@18B4P0LG5a08R8R00i>CT0Cdm?
04i>CP2?Shl0E5AD01PH600S8b<0ADE505=CD`1dM7@0@T9202TY:@15ADD0D51@079bLP1nOWh0:RXZ
04A4A01GEeL0GEeM03Hf=P1WIfL0F5QH03lo?`14A4@0DE5A03dm?@2CTi<0D51@02XZ:P1OGel0KFe]
059BDP1HF5P0MgMg02`/;02@T900P82002`/;01FEUH092@T03/k>`1ZJVX0@d=303Ph>01DE5@07QhN
03Ti>@2:RXX0:BTY04A4A01ADE40<c<c09RHV02WYjL0>SXj04M7A`0j>SX0I6AT0<o?c`2IVIT0=CDe
0451@@1HF5P0Ng]k09NGU`0i>CT09BDU07UiN@1gMgL0LW9b0410@01EEED0De=C0:BTY01_Kfl0IFEU
01/K6`0g=cL0R8R8071`L01TI6@08B4Q030`<01_Kfl0WinO059BDP1YJFT0=3@d06QXJ01kNg/0@d=3
04i>CP12@T80820P08b<S00m?Cd0?Shn04a<C00a<C40B4Q80861P@0^;Rh0FUYJ055AD@1GEeL0Rh^;
0:2PX00c<c<0I6AT04];B`1MGEd0MGEe0:VYZ@0a<C40FEUI06EUI@1lO7`0VIVI09^KV`0Y:BT0?Shn
06UYJ@13@d<0SXj>05eMG@0V9RH0=CDe07inOP1cLg<0Q8B4065QH@0N7Qh0DU9B079bLP005`14A4@0
J6QX02PX:00g=cL0DE5A06EUI@13@d<0HV9R02l_;`0];Bd0?Cdm07UiN@10@400?clo05YJFP14A4@0
CTi>06m_K`0W9bL0;Rh^02l_;`0g=cL0FUYJ00kooooo08P0KVi^079bLP1@D500@D5105EEE@1aLG40
EUIF04Q8B01gMgL0EUIF0451@@1QHF40S8b<0:JVYP2KVi/0FEUI04U9B@1:BTX0J6QX04U9B@0a<C40
>3Ph05EEE@1=CDd0>CTi06e]K@1WIfL0I6AT04a<C01/K6`0KFe]08V9R@1QHF40VYZJ0;Vi^@2PX:00
SXj>08J6QP0V9RH0@T92065QH@1[Jf/0Lg=c04e=C@1PH600EUIF051@D01EEED09BDU05eMG@0e=CD0
GEeM01TI6@0[:b/0I6AT05eMG@1EEED0;2`/05aLG00k>c/0@41005ADE01BDU80E5AD04Q8B01DE5@0
I6AT03<c<`1XJ6P0A4A405UIF@0e=CD0J6QX0492@P11@D40NWYj03lo?`1VIVH0<c<c04A4A00h>3P0
A4A402h^;P0_;bl0=3@d05]KF`0];Bd0:BTY0410@00h>3P0?Cdm03Ti>@14A4@0=SHf04];B`11@D40
<c<c04U9B@0l?3`0Bd];0410@019BDT0C4a<059BDP18B4P0@D5104I6AP1?Cdl0JFUY07UiN@1MGEd0
Ogmo09BDU00P8200BTY:065QH@1;Bd/0?clo04U9B@1EEED0FEUI06QXJ02DU9@0R8R805YJFP1SHf<0
MgMg08b<S02e]KD0Kfm_075aL@1dM7@0IFEU06m_K`1:BTX0H61P0P1MGEd0<P1[Jf/0=3@d03hn?P1<
C4`0HV9R08B4Q01TI6@0BDU905IFEP1EEED0B4Q803lo?`1ADE40:RXZ04M7A`1NGUh0CTi>05=CD`1J
FUX0DU9B05iNGP1EEED0O7al05UIF@1?Cdl0?Shn05=CD`0e=CD0DU9B05QHF00k>c/0BDU903<c<`11
@D40>c/k0492@P0];Bd0=CDe05IFEP1_Kfl07QhN04e=C@0e=CD0:b/[05MGE`1PH600TY:B0492@P1B
DU80G5aL0P1CDe<0F016ATH0@41001hN7P1DE5@0ADE508>3P`1HF5P0EUIF06m_K`1mOGd0;2`/05eM
G@0n?Sh0820P061PH01JFUX0Cdm?0410@01<C4`0E5AD04=3@`14A4@0E5AD03lo?`0m?Cd0BDU905eM
G@1kNg/0=SHf05=CD`1dM7@0CTi>04I6AP1ADE40ADE505mOG`1IFET0=3@d05QHF00o?cl0ADE505IF
EP0h>3P0GEeM069RHP1FEUH0<C4a04Y:BP0K6a/0Cdm?04I6AP1<C4`0<c<c05=CD`1JFUX0BTY:04m?
C`0j>SX0@41005MGE`0k>c/0KVi^071`L01VIVH0CTi>0=7Ad@2GUiL0R8R809nOW`1CDe<0DU9B03Lg
=`13@d<0CDe=06UYJ@0Z:RX0?clo03`l?01dM7@0P82007]kN`1EEED0KVi^07=cL`1OGel0XZ:R04U9
B@1mOGd@ooooo`003_ooool0o`1dM7@0JVYZ04U9B@1LG5`0GUiN08n?S`0k>c/0@D5105IFEP1kNg/0
WinO03@d=00a<C40GUiN04a<C028R8P0Q8B402PX:01dM7@0Lg=c079bLP2ATI40Fe]K04=3@`2?Shl0
M7Ad0:FUY@2@T9008R8R03@d=02@T900O7al09:BTP14A4@0:2PX034a<@1LG5`0De=C0:^[Z`0I6AT0
6a/K05ADE01PH600@D5105ADE00O7al0>3Ph07QhN01cLg<0GUiN02DU9@0g=cL0@d=305mOG`0j>SX0
:BTY0451@@0j>SX0UiNG09FEU@0W9bL0;2`/03Ph>01LG5`0/;2`09RHV00R8R809bLW04M7A`1@D500
RHV902`/;00E5AD09bLW05ADE016ATH0J6QX01lO7`0K6a/0:2PX06YZJP0k>c/0JFUY01@D500Z:RX0
F5QH05YJFP0X:2P0=cLg02d];@0f=SH0Fe]K03<c<`0W9bL0BDU9030`<014A4@0Rh^;024Q8@0N7Qh0
CTi>03@d=01DE5@0?3`l014A4@0Z:RX0@T9203hn?P1=CDd0XJ6Q01TI6@0`<300=3@d04=3@`1DE5@0
YZJV01DE5@0/;2`0DE5A05iNGP1:BTX0XZ:R05=CD`1BDU80Fe]K04I6AP1WIfL0Hf=S01dM7@1JFUX0
JVYZ02DU9@1DE5@0LW9b02DU9@1OGel0EeMG024Q8@13@d<0HF5Q01hN7P1MGEd0MWIf034a<@1>CTh0
9bLW02d];@2FUYH0B4Q802/[:`1IFET0Bd];061PH02FUYH0M7Ad02HV9P1NGUh0IfMW05=CD`2QXJ40
FUYJ02XZ:P1OGel0L71`06ATI01KFe/0:RXZ01LG5`1:BTX0ATI607AdM025QHD06ATI034a<@1?Cdl0
F5QH05mOG`0e=CD06ATI05aLG01kNg/0ATI6051@D00F5QH09bLW09JFUP1fMWH0=CDe03`l?00n?Sh0
IfMW05EEE@0P8200ADE5051@D01EEED0K6a/038b<P0i>CT0GemO04E5A@1jNWX0;2`/020P800k>c/0
ATI605ADE02NWYh0ADE503Lg=`1:BTX0De=C09ZJVP1]KFd071`L03dm?@1=CDd0MWIf06QXJ00K6a/0
;Rh^05EEE@1QHF40UiNG0492@P0f=SH0HF5Q07MgM`1EEED0Rh^;04i>CP0W9bL0EEEE079bLP1JFUX0
P82002l_;`0W9bL0GemO06YZJP15ADD0DU9B04A4A00`<300JVYZ07AdM00k>c/0E5AD04M7A`13@d<0
VIVI08B4Q00[:b/0>SXj03Lg=`005`1oOgl0UYJF0492@P0];Bd0C4a<04M7A`1QHF40_Kfm04U9B@0Y
:BT0?3`l04Y:BP1UIFD0M7Ad02d];@0[:b/0A4A403dm?@11@D40HF5Q02LW9`0Q8B40=CDe00kooooo
0?l0D51@07]kN`2:RXX0CDe=03`l?01LG5`0IfMW04=3@`1QHF40JFUY07emO@2:RXX0Lg=c061PH02;
Rh/0]kNg04Q8B01BDU80IfMW05EEE@0Z:RX0ATI603@d=01=CDd0BDU904A4A010@400F5QH05MGE`1>
CTh0DU9B08:2PP1CDe<0CTi>05MGE`1FEUH0QXJ603dm?@0j>SX0ADE503lo?`16ATH0D51@05QHF01>
CTh0Bd];04m?C`1LG5`0CTi>071`L014A4@0HV9R0451@@1WIfL0MGEe05UIF@1YJFT0Fe]K05YJFP16
ATH0GemO06a/K01ZJVX0GUiN08B4Q01`L700EEEE04A4A01dM7@0I6AT071`L016ATH0Ogmo05YJFP19
BDT0N7Qh04A4A01/K6`0=cLg04A4A00j>SX0H61P04Q8B01=CDd0Kfm_06QXJ015ADD0A4A4061PH00l
?3`0ATI6055AD@1:BTX0?Shn051@D01OGel0ATI6059BDP1PH600Bd];05IFEP1/K6`0Fe]K04E5A@1C
De<0AdM705ADE011@D40HF5Q05]KF`1XJ6P0Kfm_05QHF00f=SH0CDe=055AD@1DE5@0@D5103Ph>015
ADD0GemO06UYJ@1`L700O7al06UYJ@1mOGd0S8b<0:f][@35aLD0Lg=c09^KV`1iNGT0WinO065QH@1I
FET0GemO069RHP0n?Sh0De=C02XZ:P1;Bd/0>c/k05EEE@23Ph<0BDU905IFEP1;Bd/0:RXZ04M7A`18
B4P0De=C01/K6`1;Bd/0IfMW06m_K`1YJFT0Cdm?05ADE01JFUX0F5QH061PH00h>3P0?Cdm04Y:BP1A
DE40:RXZ0451@@1:BTX0@d=306=SH`1<C4`0EeMG03Hf=P10@400<c<c03De=@0];Bd0CTi>01XJ6P0`
<300:RXZ01hN7P1=CDd0@d=306MWI`0R8R80NWYj061PH00`<300<c<c02TY:@0/;2`06ATI04m?C`0Z
:RX0CTi>05]KF`1HF5P0UYJF0:RXZ00^;Rh0HV9R03De=@0^;Rh0Hf=S06IVIP2EUID0EEEE05IFEP1<
C4`0Bd];03/k>`0l?3`0>c/k03Ph>00o?cl0CTi>06=SH`0n?Sh0MGEe08:2PP1hN7P0BDU9059BDP0l
?3`0PH6107YjNP0k>c/0De=C02XZ:P0W9bL0EeMG03Ti>@1BDU80J6QX05IFEP15ADD0DU9B03lo?`1L
G5`0ATI606QXJ016ATH0CTi>02@T901=CDd0BTY:05=CD`1<C4`0F5QH028R8P1EEED0?3`l08>3P`1^
KVh001L0L71`04M7A`15ADD0MWIf05aLG028R8P0CTi>04Q8B010@400B4Q803hn?P0i>CT0BTY:05UI
F@1HF5P0O7al04a<C01IFET0H61P06ATI01OGel0G5aL08f=S@0@ooooo`003_ooool0SP1RHV80Ogmo
05mOG`1UIFD0D51@09^KV`1RHV80K6a/05iNGP1>CTh0TI6A06e]K@1YJFT0>SXj03dm?@0`<300N7Qh
03Xj>P1XJ6P06a/K02/[:`1EEED0Kfm_04M7A`1WIfL07AdM05eMG@2=SHd0HV9R05UIF@0h>3P0;bl_
05IFEP1_Kfl0:RXZ02/[:`0f=SH0De=C08Z:RP1OGel0820P04A4A01BDU80EeMG0:BTY00Z:RX0<30`
05ADE01SHf<0MgMg05eMG@0G5aL0Bd];04m?C`1DE5@0NGUi079bLP0H61P09bLW06EUI@1@D500KVi^
04A4A00i>CT0EeMG07UiN@10@400F5QH03dm?@0X:2P0DU9B05MGE`0l?3`0Hf=S01TI6@0Q8B40E5AD
059BDP0/;2`0MWIf02@T900I6AT0NGUi02TY:@0U9BD0@41004a<C01=CDd0MWIf03hn?P0P8200DU9B
03Ti>@0n?Sh0P82002h^;P14A4@0IfMW04e=C@1NGUh0Q8B403/k>`0f=SH0FUYJ02l_;`1IFET0HF5Q
01hN7P0Y:BT0FEUI03Xj>P18B4P0I6AT01LG5`0^;Rh0HV9R03Ph>00/;2`0DU9B02DU9@0X:2P0H61P
06m_K`0c<c<0@T9201XJ6P0c<c<0TY:B06IVIP0_;bl0EUIF04M7A`1RHV80Ng]k02l_;`0a<C40GemO
055AD@1DE5@0?3`l01dM7@13@d<204i>CP1T092@T00Q8B408b<S07AdM01RHV80<c<c05]KF`0_;bl0
9bLW05]KF`2:RXX0>3Ph06e]K@0n?Sh0?Cdm04Y:BP1VIVH0C4a<05UIF@0U9BD0H61P09bLW01JFUX0
=cLg05QHF00b<S80EUIF09ZJVP16ATH0>SXj05]KF`11@D40Zj^[09BDU00j>SX0>3Ph05EEE@15ADD0
UiNG02h^;P0G5aL0Bd];075aL@14A4@0GEeM034a<@1EEED0EUIF06=SH`1`L700J6QX03De=@1`L700
CTi>05UIF@1mOGd07AdM05eMG@17AdL0B4Q805YJFP0Q8B40820P0451@@1NGUh0J6QX03Lg=`0h>3P0
AdM706e]K@0Z:RX0BTY:01/K6`1OGel0[Jf]04i>CP0f=SH0ATI607EeM@2e]KD0Ng]k03@d=01<C4`0
BDU9051@D02_[jl0QHF50492@P1;Bd/0De=C075aL@2h^;P0ATI602@T900l?3`0C4a<05aLG02FUYH0
C4a<028R8P80EeMG0200>CTi0820P01_Kfl0>SXj06=SH`2BTY80EEEE07YjNP0b<S80:2PX07]kN`2<
S8`0:b/[04Y:BP0g=cL092@T06=SH`22PX80@T9204a<C01CDe<06a/K06][J`1hN7P09RHV0410@012
@T809bLW07IfMP2IVIT0:2PX03Lg=`kooooo02`0GUiN069RHP18B4P0B4Q805=CD`1XJ6P0Cdm?03De
=@1`L700DE5A04=3@`1MGEd0Ng]k0820P024Q8@0GemO03`l?00o?cl0GemO04Q8B00M7Ad0=3@d04=3
@`1DE5@0@D5105UIF@1HF5P0CDe=04A4A01ADE40Kfm_05EEE@15ADD0Rh^;0:BTY01oOgl0N7Qh06IV
IP0e=CD0>CTi069RHP1WIfL0GEeM03dm?@80CTi>09L0Bd];05IFEP0b<S80Bd];0492@P1?Cdl06QXJ
02d];@0l?3`0Cdm?04];B`0O7al0?3`l05QHF01SHf<0De=C05eMG@1eMGD0D51@05MGE`1NGUh0@D51
065QH@18B4P0D51@03dm?@1VIVH0Kfm_05EEE@1DE5@0@D5105MGE`0j>SX0=SHf02l_;`0m?Cd0:BTY
02DU9@0V9RH0C4a<03Hf=P0Q8B40?clo02l_;`1>CTh0>3Ph04e=C@0j>SX0Hf=S03`l?016ATH0FEUI
03Ph>017AdL0Cdm?04e=C@18B4P0BTY:05YJFP1bLW80HF5Q04U9B@1hN7P0JFUY075aL@1QHF40U9BD
06QXJ02GUiL0Jf][06UYJ@1OGel0@d=304a<C01bLW80S8b<08B4Q026QXH0IVIV08Z:RP2HV9P0UYJF
08b<S01YJFT0Cdm?065QH@1nOWh0EUIF03/k>`1FEUH0KFe]04=3@`1=CDd0;bl_03dm?@13@d<0HF5Q
07=cL`14A4@0>CTi04=3@`18B4P0GEeM061PH021PH40<S8b02TY:@0h>3P0DU9B08F5Q@1EEED0@d=3
04Y:BP1EEED0H61P05IFEP17AdL0?Cdm04a<C00M7Ad0?Cdm051@D012@T80F5QH038b<P18B4P0AdM7
04U9B@1;Bd/0<30`03Xj>P1=CDd0;Bd]03@d=00b<S805QHF03`l?013@d<0Fe]K02/[:`21PH40Jf][
03Lg=`0l?3`0:BTY05UIF@0c<c<0IFEU0451@@1`L700JVYZ0080GEeM00`0K6a/02DU9@1^KVh0=cLg
02d];@1ADE40AdM705YJFP19BDT0Hf=S055AD@1QHF4204m?C`0e04];B`1=CDd0CTi>065QH@1aLG40
=3@d04E5A@1@D500>3Ph04U9B@1fMWH0BTY:05aLG01:BTX0:BTY05MGE`0m?Cd092@T04=3@`0S8b<0
I6AT06YZJP1>CTh0<C4a04U9B@0[:b/0IVIV04e=C@1aLG40?clo04M7A`0`<300Fe]K0410@00i>CT0
ADE505=CD`0[:b/0IVIV04I6AP1KFe/0>SXj07moO`1HF5P0D51@071`L01?Cdl0DU9B05=CD`1DE5@0
?Shn055AD@0P82000P0k>c/02P1GEeL0H61P05IFEP1GEeL0HF5Q05eMG@1>CTh0IVIV04m?C`1YJFT@
ooooo`003_ooool0f`1[Jf/0CDe=04m?C`1lO7`0Cdm?06IVIP19BDT0Ng]k08f=S@11@D40CTi>055A
D@2SXj<0IVIV02h^;P0T92@0DU9B06ATI032`/80;Bd]02@T900a<C40HV9R06a/K02?Shl0:b/[024Q
8@13@d<0FEUI09ZJVP12@T80A4A404Y:BP1DE5@0GemO07inOP1BDU80ATI606EUI@1_Kfl0EeMG0:RX
Z01@D500<S8b05IFEP1DE5@0K6a/09jNWP0U9BD0>3Ph07=cL`0n?Sh0N7Qh03dm?@0M7Ad0;2`/06YZ
JP0X:2P0BDU902@T900H61P0@T9205mOG`0I6AT0>3Ph051@D00h>3P0K6a/04a<C00H61P0@41003@d
=01RHV80[:b/030`<00G5aL0AdM7059BDP15ADD0_[jn0410@00Z:RX0>c/k059BDP1DE5@0Vi^K018B
4P0[:b/0;Bd]06EUI@1TI6@0QHF501@D500K6a/0=cLg03hn?P1bLW80L71`01DE5@0D51@0FUYJ0492
@P0d=3@07AdM02@T9013@d<0O7al03De=@0J6QX0D51@03De=@1bLW80Ti>C034a<@0K6a/0A4A40492
@P1/K6`0O7al01LG5`0F5QH0>c/k05mOG`1lO7`0Lg=c01PH600W9bL0GEeM06m_K`1ZJVX0?Cdm03<c
<`19BDT0E5AD05YJFP1MGEd0:RXZ03<c<`1CDe<0EUIF01`L700h>3P0?Cdm05YJFP2FUYH0JVYZ02LW
9`14A4@0@T9206][J`2VYZH0ADE501TI6@0g=cL0DU9B04m?C`1mOGd0>c/k02@T901JFUX0@d=30451
@@2ZZZX09bLW02`/;01NGUh0GEeM04m?C`22PX809RHV04e=C@1lO7`0Cdm?07AdM021PH40;Rh^03hn
?P26QXH0D51@05YJFP1SHf<0A4A4055AD@1YJFT0FUYJ06QXJ010@400@T92069RHP0`<300Fe]K03Lg
=`0[:b/0KFe]09fMW@1BDU80FUYJ03<c<`16ATH0S8b<04U9B@13@d<0>CTi07MgM`2MWId0?Shn01TI
6@0f=SH0ADE50:ZZZP1QHF40820P04i>CP1EEED0Bd];06][J`0M7Ad0@d=307MgM`02069RHP0i05UI
F@0i>CT0G5aL08J6QP1LG5`0LG5a07EeM@1<C4`0G5aL06ATI01GEeL0KFe]065QH@0c<c<0F5QH06IV
IP0i>CT0DU9B03lo?`0R8R80IFEU06=SH`17AdL0KFe]0410@00O7al0Fe]K06m_K`0];Bd0Hf=S03Lg
=`0`<300JVYZ05UIF@0S8b<0?Cdm03hn?P1DE5@0JVYZ055AD@0P8200?Shn04Q8B01BDU80S8b<07]k
N`0M7Ad0>CTi08:2PP0m?Cd0QXJ606UYJ@0V9RH0>c/k07=cL`1=CDd0L71`00kooooo01d0D51@07Qh
N01FEUH0<30`03De=@1`L700O7al04Y:BP1MGEd0E5AD051@D01gMgL0Y:BT0::RXP2HV9P0JFUY04i>
CP1MGEd0JFUY04];B`0];Bd0ADE503Lg=`1<C4`0?Cdm05aLG01<C4`0PH6104a<C00204m?C`15075a
L@0k>c/0KFe]05aLG01nOWh0WYjN07UiN@0^;Rh0:BTY03<c<`0j>SX0EUIF04E5A@1ADE40CTi>04M7
A`1GEeL0>CTi065QH@0i>CT0FEUI02PX:01/K6`0Lg=c059BDP1MGEd0FEUI05]KF`15ADD0DU9B079b
LP1UIFD0D51@06YZJP1JFUX0Fe]K055AD@1ZJVX0BDU906YZJP1FEUH0KFe]04I6AP0f=SH0M7Ad05EE
E@1PH600<30`03Lg=`0j>SX0D51@03/k>`0];Bd0>c/k051@D00m?Cd09RHV04Y:BP0i>CT0EEEE03dm
?@1;Bd/0C4a<05MGE`1@D500?clo051@D01MGEd0CDe=0080EEEE04X0F5QH04i>CP1LG5`0?clo04e=
C@11@D40H61P06a/K01NGUh0EeMG05aLG017AdL0Fe]K05YJFP1BDU80>SXj0492@P18B4P0IfMW04a<
C01mOGd0Ogmo07UiN@33`l<0a<C40>GUi@3Jf]X0X:2P08n?S`20P800UiNG06QXJ01DE5@0HV9R06QX
J01<C4`0IFEU034a<@10@400?Shn079bLP1]KFd0ADE504];B`1]KFd0A4A404E5A@1?Cdl0G5aL01TI
6@1ADE40I6AT05QHF01MGEd0BTY:06IVIP1SHf<0FUYJ06MWI`1XJ6P0EEEE03lo?`1<C4`0;bl_03Xj
>P1;Bd/0?Shn05MGE`0j>SX0EUIF04Y:BP1RHV80?Cdm03Xj>P80DE5A04<0?Shn03/k>`0V9RH07QhN
04I6AP1;Bd/0JFUY01hN7P1@D500FUYJ059BDP0P82006a/K03hn?P0K6a/0BTY:03Hf=P1=CDd0BTY:
04m?C`1_Kfl0NGUi01`L701_Kfl0>SXj02l_;`1UIFD0JVYZ06IVIP1ADE40GEeM05aLG012@T80>SXj
05IFEP0n?Sh0<c<c04M7A`1ADE40GemO05MGE`1cLg<0IfMW05]KF`0f=SH0?clo03<c<`1NGUh0G5aL
02/[:`1?Cdl0C4a<05MGE`1>CTh0E5AD051@D01RHV80<C4a03Hf=P1<C4`07QhN07AdM01`L700Q8B4
03<c<`1gMgL0=3@d0080Bd];01l0Ng]k079bLP1oOgl0?clo08n?S`13@d<0I6AT0410@02e]KD0IFEU
055AD@1kNg/0E5AD05UIF@1:BTX0DE5A038b<P1DE5@07AdM051@D01CDe<0Ogmo06YZJP1TI6@0IVIV
07IfMP1/K6`0EeMG07AdM01<C4`0GEeM013ooooo000>ooooo`3o0:b/[01>CTh0CDe=06MWI`1dM7@0
PX:20492@P1SHf<0O7al07EeM@1=CDd0>SXj07IfMP2EUID0BTY:04e=C@1UIFD0?Cdm07MgM`2HV9P0
HF5Q06UYJ@0];Bd0=cLg06m_K`19BDT0HV9R05aLG00S8b<0@d=308:2PP11@D40BTY:04Q8B00L71`0
Hf=S08>3P`0X:2P0>3Ph03De=@0a<C40KVi^05iNGP0V9RH0;Rh^024Q8@1BDU80WinO04I6AP0W9bL0
C4a<04];B`2LW9`0QhN702XZ:P0e=CD0CDe=05IFEP2ATI40>CTi01lO7`0R8R80HV9R05=CD`29RHT0
>3Ph02PX:00i>CT0K6a/05iNGP1MGEd06QXJ02DU9@1CDe<0Hf=S03De=@1PH60092@T02h^;P1OGel0
H61P02LW9`0h>3P061PH038b<P1iNGT0?Cdm02d];@13@d<0=SHf04m?C`2JVYX09bLW01XJ6P0_;bl0
;2`/079bLP2ATI408b<S01dM7@0W9bL0BTY:07EeM@0c<c<051@D02HV9P1@D500DU9B05EEE@1iNGT0
5ADE038b<P2DU9@0Lg=c05YJFP1>CTh08b<S03dm?@1nOWh0IVIV02@T901CDe<08R8R06MWI`26QXH0
;Bd]02l_;`0k>c/09BDU05iNGP1_Kfl07alO03hn?P0j>SX0Bd];096AT@1OGel09BDU05MGE`1EEED0
;Bd]05UIF@12@T80<C4a07inOP1jNWX0JFUY08N7Q`0o?cl0BDU906IVIP1XJ6P0Bd];08V9R@12@T80
:BTY06EUI@1aLG40C4a<08V9R@1<C4`0;Bd]075aL@1NGUh0ADE505eMG@0L71`09BDU06i^KP0a<C40
Bd];04a<C00b<S80>3Ph09RHV018B4P0ADE505=CD`0a<C40Ph>308Z:RP0[:b/0ADE5034a<@1fMWH0
dM7A0820P00/;2`0GUiN04I6AP2ATI40FUYJ02PX:00a<C40M7Ad05QHF01LG5`09bLW05=CD`1IFET0
DU9B06EUI@0K6a/0ADE506][J`1hN7P0@41006i^KP0_;bl0DU9B07moO`0l?3`0BTY:03Lg=`0_;bl0
JVYZ03Lg=`1:BTX0GUiN0451@@12@T80De=C04M7A`1PH600>SXj04E5A@2IVIT0SHf=03lo?`16ATH0
=cLg04i>CP23Ph<0IfMW03De=@1RHV80AdM705MGE`2QXJ40Z:RX02@T9017AdL0EeMG04a<C02l_;`0
IFEU028R8P0j>SX0Ng]k059BDP2f][H0Jf][020P801EEED0Fe]K000G04E5A@1bLW80:BTY02l_;`1F
EUH0EeMG03Xj>P1hN7P0>3Ph02h^;P11@D40QXJ6059BDP0g=cL06ATI030`<01;Bd/0GUiN03Hf=P0d
=3@0=cLg034a<@1IFET03_ooool0201CDe<0PH6109:BTP12@T80?3`l04m?C`14A4@0:2PX0P1/K6`0
7@1?Cdl0GUiN069RHP1ZJVX0XJ6Q06IVIP1BDU80AdM708R8R017AdL07QhN03Ti>@0h>3P0BTY:04=3
@`1ADE40BDU9069RHP1[Jf/0F5QH05EEE@1gMgL0K6a/07QhN01nOWh0Lg=c092@T029RHT0C4a<0080
?Shn03l0BTY:06IVIP1QHF40I6AT05mOG`1XJ6P0LW9b02XZ:P1;Bd/0EEEE05QHF00Q8B40;Bd]051@
D01RHV80F5QH02/[:`1<C4`0Bd];05IFEP1WIfL0DU9B05EEE@1<C4`0CTi>04I6AP0i>CT0GemO061P
H01:BTX0;2`/05ADE01ZJVX0F5QH05iNGP0^;Rh0EEEE04I6AP0_;bl0:2PX04e=C@0c<c<0>c/k05AD
E01BDU80=3@d02LW9`1@D500<S8b0451@@0i>CT0=cLg02h^;P15ADD0?Cdm03lo?`16ATH0?3`l03Xj
>P0h>3P0B4Q804e=C@17AdL00P1@D5005@1GEeL0CTi>06e]K@1QHF40L71`05MGE`1nOWh0Lg=c0:2P
X01BDU80K6a/06YZJP19BDT0DU9B06ATI01XJ6P0Ti>C0;:b/P20P800GemO07emO@020=;BdP0K05EE
E@16ATH0EUIF07]kN`1RHV80ATI604i>CP1FEUH0ATI605iNGP0i>CT0?Shn02XZ:P1SHf<0MWIf06IV
IP1]KFd0FEUI04A4A01ADE40ATI605aLG00S8b<0>3Ph04E5A@1GEeL0NWYj0080BTY:07P0AdM7051@
D01RHV80?Cdm04I6AP1@D500EeMG03Ti>@1@D500F5QH03dm?@1LG5`0C4a<06QXJ00i>CT0@D5104m?
C`0f=SH0:2PX05UIF@0R8R80=CDe05EEE@0c<c<0DU9B02/[:`18B4P05QHF05eMG@1iNGT0>c/k01lO
7`0U9BD0B4Q803<c<`1KFe/0BDU905eMG@1LG5`0EEEE08N7Q`2ATI40=SHf06QXJ01JFUX0ADE506m_
K`1<C4`0PX:205MGE`1OGel0FUYJ05IFEP18B4P0De=C04U9B@18B4P0EUIF05EEE@1]KFd0A4A4059B
DP1QHF40E5AD055AD@1XJ6P0A4A408F5Q@21PH40DU9B05IFEP12@T80B4Q8061PH00g=cL0EeMG04Y:
BP14A4@0BDU904Q8B00_;bl0C4a<03/k>`1SHf<0=CDe0492@P0L71`0B4Q803hn?P1=CDd0?Shn04e=
C@13@d<0L71`03/k>`1gMgL0BDU906m_K`1=CDd0Hf=S0::RXP18B4P0FEUI04a<C01MGEd0GUiN096A
T@1:BTX0BDU903<c<`1OGel0GEeM08N7Q`1/K6`0Hf=S051@D01OGel0GEeM05UIF@1kNg/@ooooo`00
3_ooool0S@1YJFT0M7Ad05mOG`1WIfL0DU9B09FEU@1VIVH0PH61065QH@1HF5P0WinO06ATI021PH40
Hf=S04=3@`1?Cdl0F5QH05IFEP2ATI40@d=305ADE027QhL0Fe]K04I6AP1WIfL0;bl_04U9B@1hN7P0
@41004U9B@14A4@0DU9B06UYJ@1GEeL0;Bd]03Xj>P13@d<0GUiN07AdM00Z:RX08B4Q04a<C01NGUh0
I6AT07UiN@0V9RH0:BTY03`l?01FEUH0LG5a02h^;P0Q8B40CDe=05mOG`1>CTh0EeMG06EUI@0Q8B40
FEUI05EEE@10@400CDe=04a<C00Z:RX0?Shn05YJFP0n?Sh0K6a/030`<00g=cL0JFUY034a<@0`<300
Hf=S02/[:`0X:2P0Fe]K03Hf=P0W9bL0F5QH03@d=00`<300GUiN02LW9`0a<C40FUYJ05IFEP1<C4`0
JVYZ03<c<`0f=SH0EEEE03/k>`1@D500MWIf028R8P0[:b/0BTY:03`l?016ATH0Kfm_01dM7@0d=3@0
Bd];02`/;019BDT0I6AT02DU9@0b<S80HV9R03@d=01@D500I6AT02l_;`1CDe<0LW9b02LW9`0`<300
Bd];03Ti>@15ADD0S8b<055AD@0h>3P0CTi>03<c<`1EEED0K6a/03Lg=`13@d<0Cdm?0492@P1nOWh0
P82002XZ:P16ATH0I6AT04E5A@23Ph<0L71`02@T900204Q8B02706MWI`1^KVh0<30`03<c<`1LG5`0
EeMG069RHP2NWYh0AdM702/[:`26QXH0WinO04Y:BP1iNGT09RHV03<c<`24Q8@0SXj>03Xj>P1KFe/0
@41005UIF@2GUiL0F5QH02l_;`1QHF40CTi>071`L01_Kfl0;Rh^03Ph>011@D40CDe=07emO@1eMGD0
=cLg04=3@`1:BTX0?Shn09nOW`14A4@0:2PX059BDP1PH600CTi>07EeM@0a<C40B4Q805UIF@1?Cdl0
M7Ad05]KF`0`<300FEUI03dm?@1MGEd0J6QX02@T901/K6`0CDe=04i>CP1]KFd0<S8b034a<@1ZJVX0
Fe]K05aLG00e=CD0=SHf071`L01GEeL0>CTi03lo?`0`<300M7Ad09NGU`0m?Cd0Cdm?04U9B@1^KVh0
SHf=05mOG`0k>c/0EeMG05aLG01EEED0QhN703`l?00];Bd0Jf][08F5Q@1OGel0Ogmo04a<C00Z:RX0
CTi>05]KF`1QHF40PH6104E5A@0`<300Bd];06YZJP0h>3P0JFUY051@D00e=CD0J6QX06][J`1:BTX0
QHF5059BDP11@D40NWYj05ADE00d=3@0G5aL03Ti>@0e=CD0M7Ad06IVIP0i>CT0ADE505EEE@0`<300
IfMW0492@P0X:2P0@41004A4A00_;bl0DE5A03lo?`0j>SX0DU9B00kooooo0>D0GUiN07moO`1CDe<0
=CDe05mOG`1cLg<0EEEE03<c<`1QHF40MGEe055AD@1XJ6P0RXZ:09JFUP2:RXX0Jf][05UIF@1JFUX0
MWIf05UIF@0`<300EUIF055AD@1PH600De=C079bLP16ATH0Ph>3065QH@1OGel0De=C08B4Q01SHf<0
TI6A09bLW02RXZ80WIfM08:2PP0g=cL0?clo03dm?@1;Bd/0LG5a04Q8B01>CTh0C4a<04U9B@1PH600
=CDe05YJFP10@400H61P02l_;`0h>3P0GemO04e=C@1ADE40<c<c05aLG01?Cdl0GEeM05MGE`1DE5@0
HV9R04a<C01MGEd0HF5Q04];B`1QHF40?clo065QH@1?Cdl0GUiN04Q8B00a<C40HF5Q04i>CP1CDe<0
=CDe03/k>`0n?Sh0ADE503Lg=`0];Bd0;2`/04Y:BP0i>CT0:RXZ04Q8B00g=cL0D51@03Hf=P13@d<0
?clo05UIF@17AdL0BDU905ADE017AdL0DU9B04e=C@15ADD0CTi>05IFEP1KFe/0FEUI05YJFP1:BTX0
Hf=S061PH01OGel0Cdm?07moO`1DE5@0PX:205EEE@1PH600AdM704U9B@12@T80Jf][05eMG@29RHT0
OGem07moO`2JVYX0TI6A0;Vi^@2[Zj/0PH6109ZJVP1YJFT0RHV904i>CP1?Cdl0Bd];05ADE01<C4`0
EUIF02LW9`0g=cL0>CTi05mOG`1YJFT0DU9B05EEE@1IFET0AdM7051@D01GEeL0IfMW02d];@19BDT0
De=C02l_;`1gMgL0EEEE04e=C@1:BTX0ATI605eMG@0m?Cd0BDU903hn?P16ATH09RHV03De=@14A4@0
=3@d051@D016ATH0IFEU030`<01FEUH0:2PX03Hf=P0n?Sh0J6QX03Ti>@1CDe<0>c/k02DU9@12@T80
Bd];069RHP1<C4`0>SXj06IVIP1GEeL0KFe]03@d=01EEED0=SHf055AD@1=CDd0MGEe055AD@1BDU80
IVIV079bLP0Z:RX0IVIV03dm?@0k>c/0IFEU04m?C`1KFe/0CTi>05]KF`19BDT0FUYJ04I6AP1;Bd/0
BDU903lo?`16ATH0Hf=S079bLP13@d<0CTi>04U9B@1BDU80AdM7055AD@0n?Sh0N7Qh07UiN@0n?Sh0
HV9R0080DU9B0140GEeM03@d=01NGUh0FUYJ03@d=00h>3P0A4A402`/;01cLg<0Bd];05UIF@0^;Rh0
DU9B02<S8`1<C4`0@D5105QHF002055AD@0H05YJFP20P800ADE507moO`1MGEd0ZJVY055AD@1UIFD0
MgMg05eMG@1aLG40FUYJ059BDP0e=CD0DE5A02XZ:P14A4@0<S8b079bLP1WIfL0Lg=c055AD@1PH600
ADE50P1QHF400`1PH600L71`ooooo`0?ooooo`003_ooool0o`2SXj<0EUIF06EUI@1_Kfl0Jf][061P
H00l?3`0Kfm_08B4Q01IFET0?clo04i>CP1YJFT0SHf=03Xj>P0l?3`0>CTi03hn?P1oOgl0CDe=03Ti
>@1EEED0>c/k05MGE`1kNg/0<C4a03Ti>@1EEED0De=C0:^[Z`0b<S80:BTY02`/;019BDT0De=C09nO
W`15ADD05QHF03<c<`0i>CT0H61P0:VYZ@1@D5009BDU04e=C@10@400P820075aL@0M7Ad0Bd];09RH
V00g=cL0N7Qh05=CD`0O7al0@T92092@T011@D40IFEU03/k>`0];Bd0@T92051@D00_;bl0Cdm?01dM
7@11@D40CTi>05mOG`0Z:RX0ATI602h^;P1HF5P0NGUi03@d=00I6AT0>3Ph04A4A01?Cdl0P82005]K
F`0>3Ph0@D5104E5A@1FEUH0WinO02DU9@0O7al0GemO05YJFP1aLG40QHF501hN7P0I6AT0I6AT05=C
D`1[Jf/0:RXZ02TY:@0b<S80QHF506][J`12@T80<C4a01lO7`0b<S80E5AD03Ti>@0L71`0C4a<02PX
:01:BTX0IfMW03Ph>00[:b/0?clo04E5A@1GEeL0H61P02/[:`0M7Ad0>3Ph051@D01WIfL0IVIV01TI
6@0M7Ad0DE5A06ATI01oOgl0;Bd]03Hf=P15ADD0EeMG04M7A`2OWil0<30`02h^;P1DE5@0I6AT04A4
A01jNWX0E5AD03dm?@1UIFD0I6AT04Q8B01]KFd0<c<c051@D01[Jf/0GUiN02@T9017AdL0@d=303lo
?`2XZ:P0=CDe02DU9@1:BTX0ATI604A4A02/[:`0;Rh^01/K6`0k>c/0BTY:05QHF02FUYH09bLW04m?
C`1^KVh0DU9B07inOP23Ph<0<S8b05QHF02`/;00P82009:BTP1LG5`06QXJ051@D01YJFT0L71`08B4
Q01<C4`0;bl_05YJFP1]KFd0/[:b05QHF00R8R80NWYj0;Jf]P1JFUX0K6a/02l_;`0n?Sh0P8200410
@00k>c/07AdM05QHF01cLg<0D51@028R8P1MGEd0BDU906YZJP1LG5`0820P0492@P0i>CT0G5aL08N7
Q`0X:2P0?3`l05aLG01<C4`0YjNW06m_K`0_;bl0IFEU06YZJP1BDU80[Zj^0:2PX00m?Cd0EUIF06UY
J@1OGel0Rh^;03`l?016ATH0HF5Q06IVIP0o?cl0W9bL06YZJP0^;Rh0DE5A06QXJ017AdL0SHf=06AT
I00V9RH0?3`l06UYJ@0k>c/0MGEe02LW9`0];Bd0EEEE07moO`005`10@400IFEU02LW9`12@T80Ti>C
06IVIP0/;2`0EUIF02l_;`0n?Sh0Vi^K09:BTP0_;bl0?Cdm05ADE00h>3P0Ng]k06e]K@0M7Ad0AdM7
06EUI@18B4P0OGem00kooooo0=H0ATI607UiN@1@D500>SXj03`l?01nOWh0JVYZ04I6AP1gMgL0N7Qh
03Hf=P1@D500IVIV06][J`2c/k<0KVi^05UIF@1;Bd/0TY:B04];B`0Y:BT0=3@d03Lg=`19BDT0C4a<
04Q8B014A4@0E5AD08b<S01PH600Cdm?07IfMP1cLg<0D51@061PH01CDe<0P82006ATI011@D40Bd];
0451@@0m?Cd0CDe=069RHP15ADD0?clo06IVIP1[Jf/0<C4a03lo?`1>CTh0Fe]K03Ph>011@D40H61P
05QHF01QHF40@D51065QH@12@T80DE5A05QHF01CDe<0GemO06QXJ022PX80J6QX04M7A`1gMgL0B4Q8
08>3P`12@T80Lg=c079bLP11@D40MGEe03Ph>01LG5`0B4Q802l_;`0a<C40D51@03`l?00k>c/0=cLg
05iNGP15ADD0;Bd]04a<C00j>SX0FUYJ03Lg=`15ADD0A4A406IVIP1DE5@0Cdm?06YZJP1HF5P0H61P
059BDP1CDe<0EEEE04e=C@1LG5`0Bd];051@D01>CTh0EUIF04Y:BP1[Jf/0I6AT06e]K@1ADE40NWYj
04=3@`1=CDd0?Shn04e=C@16ATH0E5AD059BDP2c/k<0_[jn07emO@2HV9P0MgMg08f=S@2OWil0GemO
06MWI`1ADE40L71`04i>CP10@400FUYJ06a/K011@D40EeMG03`l?01KFe/0;bl_075aL@2DU9@0LW9b
04Q8B01?Cdl0>3Ph061PH01GEeL0GemO024Q8@17AdL0F5QH04M7A`2MWId0AdM70410@011@D40IfMW
06ATI00o?cl0>c/k04m?C`1<C4`09RHV03`l?010@400<C4a05]KF`0i>CT0H61P01lO7`10@400<c<c
03Xj>P1;Bd/0QXJ6028R8P1<C4`0@410010@400n?Sh0A4A405QHF00N7Qh0;2`/06e]K@1=CDd0Lg=c
028R8P0j>SX071`L05EEE@0T92@0ADE504m?C`1PH600][Jf08b<S00X:2P0Fe]K04i>CP13@d<0EeMG
04m?C`1VIVH0E5AD06e]K@1HF5P0LG5a055AD@18B4P0C4a<0P1=CDd0?P1kNg/0U9BD061PH01IFET0
BDU906UYJ@13@d<0CDe=04];B`2:RXX0GemO03`l?01/K6`0Fe]K04E5A@1BDU80?clo04A4A01TI6@0
@4100492@P13@d<0;Bd]0451@@0S8b<0DE5A01lO7`1@D500=SHf04U9B@0k>c/0GemO051@D016ATH0
Bd];07UiN@1<C4`0S8b<04I6AP1JFUX0CTi>07QhN022PX80>SXj051@D011@D40DE5A04i>CP1lO7`0
?Cdm0451@@0f=SH0DU9B05EEE@2PX:00GUiN07]kN`1ZJVX0Lg=c06=SH`1?Cdl0GEeM4?ooool000ko
oooo03H0J6QX07MgM`18B4P0GUiN0:RXZ027QhL0<c<c04E5A@1RHV80QXJ604E5A@0e=CD0?Shn07in
OP1eMGD0YJFU051@D010@400CDe=06EUI@1SHf<0WIfM02l_;`0o?cl0^[Zj07MgM`1mOGd0BTY:03`l
?01BDU80IVIV04];B`1dM7@0<S8b01lO7`1dM7@0QHF50451@@1HF5P07QhN02DU9@1DE5@0Ogmo03/k
>`0V9RH09BDU0410@01_Kfl0>3Ph03dm?@0a<C40=cLg05]KF`1XJ6P2024Q8@3N04Q8B01@D500MgMg
0492@P0U9BD0>CTi04e=C@1]KFd0V9RH02/[:`0V9RH0EeMG08:2PP1>CTh0@T9203@d=00Y:BT0LG5a
06][J`0[:b/0Ng]k02DU9@0Y:BT0JFUY07emO@0Y:BT0D51@01LG5`0^;Rh0F5QH07emO@0l?3`0IfMW
034a<@18B4P0M7Ad02d];@0P8200Cdm?03Lg=`23Ph<0D51@01DE5@0V9RH0=cLg02HV9P1hN7P0:RXZ
01@D500U9BD0@d=303lo?`1BDU80Ogmo02<S8`0g=cL0De=C05MGE`1[Jf/0K6a/01DE5@0d=3@0LG5a
079bLP1@D500LW9b02<S8`10@400IFEU06UYJ@0m?Cd0De=C02d];@18B4P0FEUI02DU9@0j>SX0?Shn
020P801SHf<0NGUi03<c<`1LG5`0=3@d02/[:`1OGel0De=C02DU9@0m?Cd0CDe=059BDP2BTY80?Shn
02d];@18B4P0Ph>306e]K@2UYJD0L71`02l_;`1aLG40QhN7065QH@2^[Zh0=SHf03Ti>@1PH600FUYJ
06][J`1HF5P051@D03Lg=`1/K6`0>CTi04M7A`12@T807QhN051@D01LG5`0>3Ph05eMG@0i>CT0AdM7
0;Bd]028R8P0=3@d04A4A00N7Qh0B4Q80;Ng]`1WIfL0<30`03Ph>00m?Cd0Zj^[09>CT`0[:b/0?Shn
05YJFP1eMGD0PX:202<S8`0b<S80AdM706MWI`27QhL08B4Q02PX:01QHF40IFEU05IFEP2g]kL0;bl_
06=SH`1<C4`0@41005MGE`0Q8B409BDU08B4Q01GEeL0Q8B406EUI@0g=cL0CDe=05ADE013@d<0BTY:
02`/;00c<c<0GUiN09>CT`1BDU80K6a/03Lg=`1cLg<0^;Rh09JFUP0g=cL0J6QX03lo?`0a<C40J6QX
08f=S@0X:2P0@T9204=3@`1XJ6P0U9BD09NGU`0a<C40?Shn0451@@1fMWH0TY:B02d];@0i>CT0BTY:
05aLG01/K6`0^k^k04E5A@18B4P0H61P07=cL`1>CTh0VYZJ04Y:BP19BDT0Hf=S06IVIP0m?Cd0BDU9
01lO7`0[:b/0H61P05YJFP13@d<0Bd];028R8P0e=CD0G5aL3_ooool0W@1TI6@0V9RH08N7Q`12@T80
BTY:065QH@15ADD0=cLg05eMG@1_Kfl0?Shn05eMG@1cLg<0O7al0;Vi^@1jNWX0D51@05EEE@2>SXh0
E5AD01lO7`19BDT0D51@04i>CP0l?3`0FUYJ03/k>`1iNGT0GemO05IFEP1@D500QXJ606i^KP2JVYX0
Q8B409FEU@2][Jd0UIFE05IFEP1>CTh0D51@04Q8B028R8P0EEEE059BDP16ATH0Bd];06ATI017AdL0
D51@03`l?01MGEd0?Shn02HV9P14A4@0C4a<04Q8B00Y:BT0@41006EUI@1SHf<0>3Ph04m?C`1CDe<0
=SHf0492@P1MGEd0LW9b06e]K@1MGEd0IfMW04e=C@1SHf<0IFEU04Q8B01>CTh0G5aL04I6AP0X:2P0
;Bd]0451@@0k>c/0?3`l03Lg=`0l?3`0BDU9059BDP0g=cL0AdM70492@P0d=3@0<C4a03lo?`1PH600
ATI603@d=01KFe/0DE5A030`<01=CDd0Ng]k069RHP1EEED0FUYJ07EeM@1CDe<0IVIV04I6AP1WIfL0
@d=30820P01aLG40VIVI071`L027QhL0E5AD06a/K01:BTX0DE5A03/k>`1ZJVX0BTY:0820P01]KFd0
De=C08B4Q01kNg/0^;Rh0<G5a@1cLg<0Xj>S04e=C@2`/;00Cdm?03lo?`1=CDd0GEeM0492@P14A4@0
<C4a04I6AP0];Bd0Cdm?065QH@1@D500C4a<05iNGP1HF5P0ATI604a<C01PH6005ADE03lo?`16ATH0
6QXJ07AdM00j>SX00P16ATH0C@1<C4`0IfMW0451@@18B4P0EeMG05ADE00d=3@0<S8b03Ph>00Y:BT0
@41003De=@14A4@0<S8b05MGE`0W9bL08b<S06YZJP1/K6`0@d=306m_K`0b<S809RHV05ADE00l?3`0
EEEE02/[:`1gMgL0QHF505IFEP1_Kfl0=cLg04i>CP0/;2`0?Shn03dm?@1JFUX0E5AD04e=C@1CDe<0
FUYJ02DU9@1cLg<0DU9B03`l?01ADE40?Cdm04Y:BP0W9bL0ATI603dm?@16ATH0@d=30492@P19BDT0
CTi>0410@01<C4`0EeMG04=3@`1:BTX0?clo069RHP0k>c/0Bd];04e=C@1gMgL0IVIV034a<@16ATH0
B4Q803Ti>@1UIFD0=cLg04i>CP1/K6`0=3@d0080>3Ph00X06ATI05MGE`0Y:BT0CTi>03/k>`1IFET0
9RHV0492@P0m?Cd0FEUI0P1DE5@0701^KVh0VIVI02/[:`1MGEd0CDe=07UiN@0e=CD0Fe]K06i^KP1W
IfL0RXZ:04E5A@16ATH0?Cdm05MGE`0g=cL0ATI604A4A01OGel0F5QH07inOP1GEeL0S8b<05QHF01`
L700Fe]K08^;R`2HV9P@ooooo`003_ooool0o`1LG5`0MgMg08J6QP1bLW80DE5A06EUI@1IFET0Vi^K
06ATI019BDT0J6QX069RHP1lO7`0IfMW034a<@15ADD0LG5a04a<C01:BTX0<c<c02h^;P2?Shl0Lg=c
03/k>`0`<300>c/k05EEE@1[Jf/0=3@d05MGE`1@D500CTi>07alO00_;bl061PH03Lg=`1ZJVX0GEeM
06e]K@0G5aL06a/K03De=@1bLW80MgMg06UYJ@0D51@0>CTi04a<C01DE5@0MGEe02<S8`0];Bd0DU9B
09FEU@1OGel0De=C05aLG012@T80BTY:051@D00^;Rh0C4a<02LW9`0n?Sh0E5AD06IVIP16ATH0Bd];
02<S8`1IFET0EEEE02LW9`0Y:BT0AdM7034a<@0T92@0LW9b02l_;`0N7Qh0@D510492@P11@D40De=C
01PH600P8200<C4a05YJFP1GEeL0TY:B03Lg=`0[:b/0B4Q805ADE01WIfL0UIFE01LG5`1FEUH0GemO
03Ph>01:BTX0Kfm_01@D500e=CD0F5QH02`/;01<C4`0BTY:01TI6@0X:2P0Lg=c03<c<`1=CDd0DE5A
01TI6@11@D40IfMW02<S8`0c<c<0@T92028R8P1?Cdl0Shn?02l_;`1KFe/0BTY:02`/;01VIVH0SHf=
02@T900l?3`0I6AT04];B`2ATI40IfMW018B4P15ADD0J6QX061PH036a/H0?clo01lO7`0h>3P0Fe]K
06i^KP1OGel0;Bd]04a<C01RHV80J6QX059BDP1WIfL09BDU03dm?@1IFET0OGem04M7A`18B4P0<30`
0492@P1VIVH0LW9b01`L7012@T80@D51061PH023Ph<0?Cdm028R8P1IFET0EeMG08>3P`21PH407QhN
04Q8B01@D500GUiN0::RXP1GEeL0820P06UYJ@2YZJT0Bd];08F5Q@0/;2`0:BTY03lo?`1SHf<0BTY:
04i>CP0W9bL0GemO09RHV01OGel0IfMW0451@@1<C4`0Ph>303/k>`1ADE40EEEE01lO7`1dM7@0CDe=
04I6AP1IFET09BDU03De=@25QHD0@T9204e=C@0b<S80=SHf07emO@19BDT071`L03@d=01:BTX0Kfm_
07YjNP0W9bL0?Shn04e=C@1ZJVX0MWIf02l_;`0j>SX0B4Q805eMG@1aLG40WinO03/k>`10@400J6QX
09JFUP1?Cdl0SHf=03De=@0i>CT0FEUI08n?S`1IFET0SHf=02PX:00];Bd0BTY:08V9R@0`<300GemO
04Y:BP18B4P0N7Qh079bLP0f=SH0J6QX03Ph>0005`0Z:RX0LG5a07alO00R8R80IVIV03De=@0_;bl0
GEeM03Ti>@0N7Qh0A4A405UIF@13@d<0Jf][04I6AP0O7al0A4A403Xj>P0h>3P0NWYj03Ph>00M7Ad0
@d=300kooooo0580EeMG09NGU`1FEUH0FUYJ07QhN01`L700F5QH03hn?P1bLW80Jf][05UIF@1_Kfl0
QXJ60:FUY@1iNGT0ATI605ADE01LG5`0I6AT04E5A@0];Bd0DU9B04e=C@1FEUH0Bd];069RHP16ATH0
NGUi05]KF`1OGel0BTY:08R8R01JFUX0P82008j>SP2BTY80ZJVY06a/K011@D40>c/k05YJFP1DE5@0
N7Qh04a<C01IFET0Bd];04i>CP1OGel09RHV05IFEP0i>CT0JFUY04a<C00Y:BT0JFUY05ADE01MGEd0
<30`07alO014A4@0Cdm?05IFEP1@D500EUIF04M7A`1PH600CDe=034a<@1LG5`0@D5105IFEP1;Bd/0
DU9B05=CD`0e=CD0GUiN03<c<`1FEUH0>c/k03Ti>@0n?Sh0@d=30P0b<S80`P0h>3P0BDU903Ph>00Z
:RX0BTY:02h^;P1GEeL0>SXj04U9B@0j>SX0GemO04E5A@17AdL0F5QH04U9B@1JFUX0?clo03/k>`18
B4P0M7Ad055AD@15ADD0DE5A04e=C@1QHF40HV9R09>CT`2[Zj/0WIfM06e]K@1WIfL0FUYJ05QHF017
AdL0C4a<04E5A@1MGEd0J6QX09JFUP2d];@0OGem06YZJP1NGUh0VIVI09RHV01OGel0N7Qh059BDP1Q
HF40=cLg03<c<`1IFET0NWYj04a<C01QHF40=3@d051@D00o?cl0K6a/06UYJ@1_Kfl0@D5105mOG`1=
CDd0De=C051@D01eMGD09bLW07MgM`1TI6@051@D07UiN@0d=3@0IfMW04i>CP1ADE40FEUI0410@01>
CTh0?clo04=3@`0K6a/0F5QH06MWI`0];Bd0AdM703Ti>@1DE5@0;bl_04E5A@0b<S80<30`03dm?@1;
Bd/092@T03hn?P0Y:BT05aLG04i>CP1ADE40EEEE01dM7@1cLg<0SHf=03Xj>P15ADD04Q8B02`/;00D
51@0?clo01LG5`11@D40@4100492@P2BTY80WIfM03hn?P11@D40:RXZ051@D023Ph<0@410051@D010
@400CDe=04i>CP1CDe<0CDe=06UYJ@1?Cdl0De=C055AD@1NGUh0M7Ad051@D01:BTX09RHV07AdM017
AdL0C4a<03@d=02XZ:P0PX:202l_;`1CDe<0IfMW065QH@1VIVH0E5AD07IfMP1KFe/0@D51038b<P0c
<c<06a/K07AdM00_;bl0G5aL02`/;01JFUX0?Shn06m_K`10@400FUYJ05mOG`1DE5@0GUiN06UYJ@19
BDT0L71`04Q8B02;Rh/0>CTi05iNGP1fMWH0BTY:05=CD`15ADD0DU9B05ADE01OGel0?3`l07EeM@1/
K6`0Jf][061PH01kNg/0Cdm?06][J`19BDT0Hf=S06UYJ@1LG5`0F5QH4?ooool000kooooo04H0Ogmo
05YJFP1YJFT0U9BD05ADE01BDU80A4A409ZJVP2KVi/0Bd];04a<C01MGEd0J6QX08F5Q@0^;Rh0<30`
05MGE`1aLG40];Bd038b<P0g=cL0?3`l06MWI`1fMWH0<S8b03Ph>01QHF40GemO05eMG@2>SXh071`L
02TY:@0l?3`0E5AD06QXJ02^[Zh0@d=304Q8B01hN7P0D51@03hn?P22PX80<S8b03lo?`21PH40=CDe
03`l?00];Bd06a/K04];B`0o?cl0GemO079bLP13@d<0?Cdm04e=C@1XJ6P08R8R04I6AP0l?3`0Cdm?
071`L00k>c/0=CDe04E5A@1GEeL0EEEE0:6QX@0i>CT092@T0P1ADE405`1iNGT0_kno034a<@0d=3@0
?3`l06MWI`1DE5@0Rh^;020P800X:2P0FEUI051@D01DE5@0NGUi01`L7011@D40GEeM05YJFP1FEUH0
NWYj01@D5010@400KVi^0080?Cdm0740=SHf01hN7P1IFET0N7Qh02l_;`0a<C40;bl_03Lg=`1_Kfl0
NWYj01LG5`0B4Q80DE5A04m?C`2<S8`0U9BD02HV9P0n?Sh0CDe=04=3@`1oOgl0K6a/02LW9`15ADD0
ATI605QHF01LG5`0>3Ph01lO7`1EEED0MGEe04E5A@16ATH0:RXZ04A4A01?Cdl0KFe]03Ti>@1_Kfl0
5ADE04A4A027QhL0G5aL02LW9`12@T80>CTi06QXJ02DU9@0;Bd]03<c<`19BDT0@T9208N7Q`2YZJT0
@41002PX:017AdL0Hf=S07EeM@2SXj<0:BTY02/[:`1MGEd0Rh^;05UIF@2HV9P08R8R04=3@`1cLg<0
LG5a06][J`1dM7@08R8R04Y:BP2>SXh0@4100492@P0l?3`0;Bd]06YZJP1nOWh0;bl_0451@@0d=3@0
;Bd]06=SH`0n?Sh0<C4a061PH00m?Cd0>3Ph061PH00e=CD0DU9B03Ti>@1/K6`0UiNG05UIF@0m?Cd0
Bd];0451@@2HV9P0Ng]k01lO7`1EEED0Cdm?04i>CP2<S8`0=SHf02h^;P18B4P0EEEE09ZJVP020451
@@0K06e]K@1fMWH0A4A404e=C@0Z:RX0EEEE08B4Q01QHF40NWYj04e=C@1>CTh0Cdm?03hn?P1HF5P0
Rh^;05YJFP0m?Cd0Fe]K05QHF00i>CT0L71`05=CD`0g=cL0Jf][05EEE@0Z:RX0PX:20080@41002D0
J6QX04A4A00T92@0BDU904E5A@13@d<0PH61059BDP0o?cl0Cdm?05MGE`1<C4`0[:b/05aLG00i>CT0
@d=3059BDP1:BTX0U9BD09FEU@0P8200FUYJ08f=S@1CDe<0L71`08R8R00P8200ADE505ADE013@d<0
MGEe03hn?P0M7Ad0?clo06IVIP1?Cdl0NWYj00kooooo02h0PH610:FUY@1OGel0ATI60410@01XJ6P0
HF5Q04E5A@1kNg/0I6AT03@d=016ATH0HF5Q07QhN02EUID0FUYJ06ATI01<C4`0WinO04m?C`0[:b/0
=CDe03hn?P1BDU80E5AD04i>CP16ATH0D51@04a<C01PH600EEEE07AdM024Q8@0FUYJ065QH@1_Kfl0
LW9b06YZJP1<C4`0?Cdm0492@P11@D40E5AD05EEE@18B4P0>CTi0P1>CTh0?P0U9BD0@d=303Ph>01R
HV80DU9B0410@01RHV80C4a<05iNGP0m?Cd0Ogmo04Q8B01@D500GUiN05QHF01RHV80G5aL07IfMP1M
GEd0Bd];07alO01@D500PX:203lo?`1NGUh0MGEe05YJFP1SHf<0?Shn051@D015ADD09RHV03Hf=P1C
De<0ATI604=3@`0b<S80H61P04a<C011@D40D51@0451@@1TI6@0?clo04U9B@14A4@0HF5Q05=CD`1?
Cdl0IfMW06=SH`1[Jf/0G5aL05ADE01=CDd0Cdm?05ADE01ADE40Bd];06EUI@1FEUH0BTY:0P1IFET0
;P1[Jf/0Kfm_07=cL`1:BTX0D51@03`l?015ADD0?3`l0410@01@D500Q8B409RHV01DE5@0Y:BT065Q
H@2?Shl0];Bd08R8R02UYJD0F5QH07alO01@D500EeMG07EeM@1TI6@0?Cdm04=3@`14A4@0CDe=02HV
9P1EEED0S8b<05YJFP1ADE40Bd];04M7A`1>CTh0M7Ad06IVIP0P8200EUIF07emO@0O7al0IfMW03<c
<`0o?cl204A4A01f06m_K`0m?Cd0DU9B03/k>`10@4004Q8B02<S8`0j>SX0;Bd]04E5A@1<C4`0Hf=S
01`L7018B4P0<30`03`l?01WIfL0H61P02@T9017AdL0ATI602HV9P11@D40>c/k04E5A@0e=CD0@d=3
06][J`12@T80E5AD02d];@16ATH0<30`06IVIP0R8R80EUIF04A4A00^;Rh0C4a<055AD@0N7Qh0AdM7
02DU9@0e=CD0@410051@D02BTY80F5QH05MGE`1BDU80D51@04=3@`18B4P0A4A404e=C@0k>c/0HV9R
07]kN`1QHF40VIVI06MWI`1IFET0?Shn04m?C`0k>c/0IFEU071`L01@D500IFEU04I6AP0l?3`0Fe]K
04M7A`1;Bd/0L71`0492@P0/;2`0>CTi0410@01WIfL0AdM7061PH00l?3`0G5aL03dm?@0j>SX0ATI6
05QHF01cLg<0EUIF06m_K`2>SXh0Bd];06QXJ01KFe/0PX:204a<C01OGel0KVi^04Y:BP1KFe/0JFUY
06EUI@1GEeL0DE5A03De=@1ADE40J6QX05YJFP1IFET0Hf=S05IFEP1KFe/0EEEE05MGE`19BDT0Fe]K
06UYJA3ooooo000>ooooo`0C09jNWP1XJ6P0DU9B07IfMP2k^k/0LW9b0451@@1GEeL0Y:BT0:NWY`0k
>c/0B4Q805iNGP2EUID0Ti>C09nOW`1EEED0Bd];05MGE`0204Q8B00d07MgM`0Z:RX0Bd];07=cL`17
AdL0OWin03Ti>@0W9bL0H61P04a<C00`<300ATI601DE5@0b<S80EeMG04E5A@0c<c<0CTi>030`<01M
GEd0PH6104M7A`0b<S80Cdm?038b<P1oOgl0CTi>024Q8@0k>c/0<c<c04E5A@2BTY80RHV902PX:010
@400KVi^04i>CP1nOWh0:BTY02PX:01QHF40NGUi051@D01NGUh0:2PX03Ph>01NGUh0Lg=c03/k>`0g
=cL071`L02HV9P80FEUI0</08b<S04a<C00^;Rh0?3`l07=cL`1=CDd04Q8B04A4A00a<C40F5QH08N7
Q`0I6AT09bLW04m?C`0e=CD0OGem0:NWY`0O7al0<C4a04I6AP18B4P0MGEe04];B`0K6a/0@D5104U9
B@0h>3P0HF5Q01HF5P0f=SH0>3Ph05EEE@0l?3`0=3@d04E5A@0G5aL0DE5A05ADE00n?Sh0;Rh^055A
D@0E5AD0Fe]K0:b/[012@T80:RXZ055AD@0U9BD0O7al09fMW@0V9RH0;Rh^055AD@15ADD0MGEe02<S
8`0W9bL0BTY:07]kN`1KFe/0Ph>302/[:`0Q8B40@d=303dm?@0_;bl0FUYJ03hn?P0g=cL0KFe]05eM
G@1IFET0Q8B402/[:`0f=SH0I6AT06m_K`1BDU80[Jf]03<c<`16ATH0J6QX06ATI00j>SX0FEUI03De
=@0f=SH0JFUY02l_;`0g=cL0Cdm?02XZ:P0f=SH0JVYZ028R8P0d=3@0Hf=S04i>CP2BTY80OWin02/[
:`1CDe<0E5AD05mOG`2m_Kd0HF5Q01hN7P1?Cdl0E5AD061PH02SXj<0ADE503Ti>@1FEUH0GemO07mo
O`0a<C40;2`/07=cL`1ZJVX0J6QX06UYJ@0G5aL0A4A409nOW`1LG5`0@41002DU9@0k>c/0D51@05]K
F`1=CDd0GEeM038b<P26QXH0MgMg03Ti>@16ATH092@T05=CD`1]KFd0;Rh^06=SH`0h>3P0<S8b09BD
U01^KVh0>3Ph04e=C@0d=3@0I6AT0<W9b@2ZZZX0<S8b04U9B@1=CDd0GUiN0;fm_@2GUiL0=SHf04e=
C@1TI6@0BTY:0:>SX`2<S8`0;bl_05=CD`1HF5P0@d=308R8R01FEUH0ADE505EEE@1NGUh0GemO05iN
GP0O7al0@d=305QHF01SHf<0E5AD09^KV`0_;bl0;Rh^05]KF`1cLg<0Cdm?079bLP0a<C40>c/k05EE
E@1nOWh0CDe=04A4A00_;bl0G5aL06QXJ01bLW80:RXZ03Ph>018B4P0Bd];08N7Q`0>ooooo`1M04];
B`1QHF40L71`05UIF@1YJFT0IVIV05ADE011@D40LG5a06a/K00l?3`0FEUI0820P01`L700]kNg069R
HP1JFUX0B4Q80861P@1MGEd0:BTY0410@01LG5`0K6a/0492@P1KFe/0A4A407inOP1:BTX0EUIF04I6
AP1dM7@0LG5a09BDU02:RXX0Z:RX09RHV028R8P0EeMG04M7A`1DE5@0BDU90861P@14A4@0C4a<04A4
A01;Bd/0D51@03De=@1=CDd0@41007=cL`1CDe<09BDU051@D01BDU80F5QH04I6AP1HF5P0FEUI05IF
EP18B4P0CDe=059BDP13@d<0DU9B04m?C`18B4P0G5aL05=CD`1VIVH0BDU905UIF@1KFe/0<c<c055A
D@10@400ATI603Lg=`0W9bL0@D5104A4A00n?Sh0=SHf03De=@1;Bd/0?Cdm034a<@16ATH0?3`l04U9
B@0h>3P0@D510080CTi>06`0=cLg05aLG01CDe<0Bd];055AD@1QHF40EEEE069RHP1JFUX0IfMW05=C
D`1OGel0Cdm?06YZJP1=CDd0VIVI09:BTP2FUYH0MgMg07YjNP1NGUh0IfMW03dm?@1KFe/0@T9206IV
IP1@D500N7Qh08N7Q`0m?Cd0C4a<05YJFP3De=@0a/K605]KF`1UIFD0Fe]K09jNWP0f=SH0>c/k04a<
C01QHF40=3@d05=CD`0R8R80ATI602@T901CDe<0KFe]05QHF00e=CD0CTi>07EeM@1OGel0=3@d05EE
E@0Y:BT0ATI6055AD@0K6a/0K6a/03De=@1@D500BDU905ADE01mOGd0>SXj06=SH`1;Bd/0DE5A02<S
8`0i>CT0D51@03/k>`13@d<0>SXj055AD@0J6QX0?Cdm02<S8`0Y:BT0FUYJ05MGE`1;Bd/0Z:RX04];
B`0T92@0?Shn0820P01oOgl0?clo05UIF@1ZJVX0Kfm_071`L00X:2P0>CTi01dM7@0h>3P09bLW059B
DP1@D500@41005eMG@1aLG40?clo05mOG`1GEeL2030`<01902<S8`1BDU807QhN04];B`16ATH0HF5Q
04];B`1=CDd0D51@05iNGP1EEED0IFEU07emO@1ADE40FUYJ02d];@1VIVH0De=C0:>SX`1GEeL0KVi^
065QH@0^;Rh0D51@05iNGP18B4P0DE5A02l_;`1SHf<0J6QX04Y:BP1VIVH0?Cdm03hn?P1`L700BDU9
09ZJVP15ADD0J6QX05iNGP19BDT0?3`l06ATI01FEUH0@D5106YZJP2DU9@0RHV908N7Q`1^KVh0QXJ6
06IVIP2NWYh0[Zj^05UIF@1OGel0<c<c04I6AP11@D40DU9B034a<@1JFUX0EUIF071`L01EEED0KVi^
05ADE02RXZ80D51@061PH01dM7@0IFEU07YjNP0@ooooo`003_ooool0o`20P800JFUY06IVIP1WIfL0
MgMg09FEU@14A4@0RHV904M7A`1aLG40De=C0492@P1CDe<0@T9202h^;P22PX80G5aL0451@@18B4P0
>3Ph07]kN`2VYZH0<30`02h^;P1FEUH0FEUI09>CT`1ADE40820P05iNGP1RHV80Shn?07moO`0E5AD0
4a<C04Q8B01]KFd0HV9R08^;R`0T92@0:BTY04E5A@19BDT0CTi>02h^;P0`<300@41004A4A00J6QX0
>3Ph01XJ6P0m?Cd0JFUY08N7Q`1KFe/0>SXj03Ti>@0R8R80D51@04e=C@0^;Rh0?3`l03/k>`16ATH0
MgMg02HV9P0R8R80=CDe04Q8B01GEeL0IFEU01hN7P0K6a/0AdM706MWI`15ADD0VIVI01TI6@0^;Rh0
CTi>05=CD`1HF5P0IVIV01<C4`0^;Rh0HF5Q0820P01;Bd/0Vi^K030`<00i>CT0OGem05iNGP0m?Cd0
LG5a02/[:`0j>SX0FEUI034a<@0j>SX0BTY:01hN7P1TI6@0De=C01TI6@16ATH0AdM704e=C@1IFET0
Jf][01<C4`11@D40>SXj04E5A@1YJFT0K6a/024Q8@0g=cL0Jf][05aLG01UIFD0U9BD028R8P10@400
KVi^03Ph>01SHf<0HF5Q010@401JFUX0WIfM0492@P1NGUh0;Rh^03Xj>P1VIVH0CDe=03Hf=P1FEUH0
<C4a01dM7@26QXH0DU9B03Lg=`12@T80:RXZ03Xj>P2[Zj/0Bd];03Ph>01SHf<0@D5103Hf=P2WYjL0
QHF503Xj>P0c<c<0CDe=08V9R@2k^k/0@D5102TY:@1LG5`0C4a<07YjNP22PX80:BTY02XZ:P1cLg<0
C4a<07MgM`1kNg/061PH05ADE02RXZ80EUIF0;Fe]@0n?Sh07alO06e]K@2WYjL0=cLg05=CD`11@D40
?Cdm061PH016ATH0>SXj04=3@`1VIVH0`<3006YZJP0a<C40GemO03hn?P1]KFd0Xj>S03Lg=`0i>CT0
A4A404U9B@1dM7@08b<S02`/;010@400B4Q807alO02a/K40:b/[06MWI`1PH600DE5A09bLW00c<c<0
92@T069RHP1KFe/0/k>c065QH@0O7al0>3Ph07inOP1dM7@0I6AT02DU9@12@T80GUiN06=SH`1RHV80
NGUi02DU9@0n?Sh0GemO08j>SP13@d<0KFe]01lO7`0a<C40ADE505IFEP0h>3P0FUYJ024Q8@0h>3P0
VIVI07]kN`0U9BD0Bd];03hn?P1dM7@0Zj^[06a/K00X:2P0D51@04M7A`005`1aLG40/K6a04A4A00Y
:BT0B4Q8069RHP1CDe<0[Zj^03Ph>00[:b/0=cLg07inOP1IFET0Kfm_024Q8@0M7Ad0>3Ph06m_K`17
AdL0Ng]k01/K6`0N7Qh0Bd];00kooooo00P0L71`0:>SX`1MGEd0CDe=05MGE`1_Kfl0Fe]K02h^;P80
NWYj00/0@41006YZJP1jNWX0NWYj08:2PP1UIFD0Fe]K05ADE01dM7@0BDU902@T90020410@01Q04i>
CP1?Cdl0L71`05YJFP1mOGd0B4Q805IFEP1<C4`0Lg=c08>3P`24Q8@0YZJV08F5Q@2LW9`0JVYZ059B
DP14A4@0G5aL05EEE@1SHf<0CTi>04m?C`0o?cl0BDU905ADE00d=3@0EUIF03/k>`1NGUh0F5QH02DU
9@1MGEd0DE5A05]KF`10@400QhN70410@00h>3P0EUIF04];B`1EEED0@T92061PH01KFe/0?Shn06m_
K`14A4@0LG5a05EEE@1GEeL0HF5Q03hn?P1FEUH0?3`l05YJFP0o?cl07QhN03Ti>@19BDT0?Cdm03`l
?018B4P0Fe]K0410@00/;2`0GUiN03Xj>P1?Cdl0DU9B04];B`0i>CT0De=C05YJFP0m?Cd0DE5A051@
D01MGEd0ATI604M7A`1:BTX0GUiN05QHF01@D500HV9R04I6AP1HF5P0FUYJ09FEU@2a/K40TY:B05IF
EP1/K6`0FUYJ05MGE`0n?Sh0D51@05EEE@0204A4A00J07inOP2XZ:P0IVIV06m_K`1fMWH0OWin06a/
K01OGel0N7Qh03Lg=`1XJ6P0;bl_03dm?@1KFe/0GemO03Lg=`1TI6@0=3@d04i>CP0b<S80H61P07em
O@1aLG40<30`04a<C00j>SX204Y:BP2006m_K`0X:2P0UiNG079bLP0I6AT0EeMG02l_;`1/K6`0D51@
04M7A`1<C4`0>SXj05ADE00o?cl0>c/k01HF5P13@d<0Fe]K03Hf=P15ADD0IfMW07UiN@0i>CT0EEEE
03Hf=P0e=CD0E5AD04];B`0U9BD0BDU902`/;0092@T0<30`03Ph>018B4P030`<05=CD`1LG5`0?Shn
05IFEP0S8b<0@D5101@D501?Cdl06ATI04e=C@15ADD0C4a<05=CD`1JFUX0<c<c05iNGP0L71`08b<S
04E5A@0];Bd0GEeM03Hf=P1OGel0D51@04];B`1GEeL0KVi^0492@P1DE5@0EUIF05=CD`20P800@T92
07IfMP0n?Sh0EEEE03De=@1:BTX0;Rh^06UYJ@1JFUX0@41004Y:BP0l?3`0BDU904a<C00i>CT0MgMg
061PH00l?3`0@41004];B`0X:2P0J6QX03Lg=`16ATH0D51@07=cL`1KFe/0Lg=c05ADE01HF5P0CTi>
03De=@1?Cdl0Jf][01lO7`1=CDd0GUiN07MgM`1KFe/0BDU904Y:BP1LG5`0NGUi0492@P1:BTX0@d=3
06][J`12@T80PX:205eMG@1:BTX0@41005mOG`0d=3@0De=C03/k>`1VIVH0LG5a06a/K02PX:0@oooo
o`003_ooool06P23Ph<0IfMW071`L01fMWH0FUYJ05eMG@1IFET0YjNW061PH01:BTX0Bd];05eMG@2K
Vi/0NWYj03Lg=`14A4@0C4a<08^;R`2<S8`0<30`03Lg=`1;Bd/0HV9R06QXJ00Z:RX0<30`0P1:BTX0
M@0j>SX0RHV903Ph>00e=CD0E5AD05YJFP1?Cdl0PX:203Hf=P1:BTX0LG5a03`l?01OGel0NWYj038b
<P1CDe<0KVi^02l_;`1IFET0EUIF03@d=019BDT08B4Q02LW9`0o?cl0UIFE09JFUP1=CDd0I6AT03Ph
>01GEeL0=SHf0451@@1bLW80@T9202d];@1DE5@0F5QH04U9B@1kNg/0;bl_02HV9P1GEeL0ATI605MG
E`28R8P0;Bd]01XJ6P1<C4`0CTi>05aLG020P8006ATI024Q8@1;Bd/0<c<c06QXJ01NGUh05ADE03Hf
=P1OGel0=CDe03`l?013@d<07alO05UIF@2=SHd0>SXj03Hf=P0h>3P0;Bd]06EUI@1cLg<0<C4a02d]
;@0j>SX0>CTi06e]K@1LG5`09bLW03Hf=P1GEeL0>c/k05eMG@29RHT092@T02l_;`1JFUX0<30`06AT
I01NGUh0:2PX03dm?@1?Cdl0@T9209RHV01WIfL0:b/[05aLG02;Rh/0BTY:04i>CP0o?cl0>3Ph08:2
PP1UIFD0=CDe06QXJ015ADD0D51@07alO00h>3P0:RXZ0451@@1;Bd/0Jf][075aL@02034a<@1=04Y:
BP1<C4`0FUYJ096AT@0i>CT0:RXZ04=3@`1ADE40Hf=S055AD@0Q8B40;2`/05=CD`22PX80ATI6069R
HP0T92@0ADE506QXJ01:BTX0EUIF05QHF00o?cl0GUiN08N7Q`12@T80EUIF05aLG012@T80TY:B07al
O012@T80GEeM04A4A00^;Rh0Lg=c03<c<`1@D500LG5a0492@P1DE5@0K6a/02h^;P1CDe<0HF5Q0410
@02ATI40I6AT0410@01TI6@0@41006][J`1/K6`0;Rh^05eMG@0o?cl0GEeM06YZJP0U9BD0<S8b07=c
L`1MGEd0LW9b030`<00U9BD0FUYJ069RHP12@T80Hf=S02/[:`1OGel0MgMg03Ph>029RHT0?3`l065Q
H@2=SHd00P1@D500=02:RXX0@d=304m?C`1jNWX0AdM70410@01TI6@0CTi>05IFEP24Q8@0EEEE03`l
?01_Kfl0@d=304Q8B01fMWH0:b/[03De=@1XJ6P0IFEU059BDP2<S8`0CDe=04Y:BP1RHV80GEeM04M7
A`24Q8@0CTi>03Xj>P1=CDd0JVYZ04Q8B026QXH0ADE502d];@1IFET0Q8B403`l?01ZJVX0HF5Q02`/
;017AdL0Bd];05IFEP1UIFD08b<S038b<P1OGel0ADE503Lg=`17AdL>ooooo`1E05YJFP2GUiL0IFEU
03`l?017AdL0MGEe06ATI019BDT0Ogmo079bLP16ATH0EEEE05UIF@1bLW80Q8B405eMG@1?Cdl0C4a<
071`L01:BTX0<30`03Xj>P14A4@0EUIF04M7A`1BDU80?Shn05IFEP16ATH0DE5A03hn?P17AdL0Jf][
05mOG`1eMGD0K6a/07moO`1TI6@0DU9B04M7A`16ATH0@d=3065QH@1DE5@0?Shn03Hf=P1>CTh0Bd];
02h^;P0e=CD0>SXj059BDP18B4P0<C4a04a<C01GEeL0IFEU04=3@`1_Kfl0HF5Q059BDP1DE5@0C4a<
059BDP1;Bd/0GUiN061PH01ADE40NGUi069RHP1aLG40?clo05=CD`1OGel0@T9204M7A`1=CDd0Cdm?
03hn?P0U9BD0?Shn04I6AP0i>CT0=CDe02HV9P0204];B`0X03Hf=P1;Bd/0@41005YJFP0j>SX0Cdm?
04I6AP1LG5`0?clo04m?C`1HF5P0Fe]K05QHF017AdL0D51@055AD@18B4P0EeMG04];B`1BDU80F5QH
061PH01FEUH0UIFE07=cL`1hN7P0GemO059BDP1MGEd0F5QH04M7A`1GEeL0@T9206IVIP1HF5P0MWIf
07moO`1MGEd0Jf][08J6QP80]kNg03X0K6a/08:2PP14A4@0P82003hn?P1=CDd0EeMG05YJFP14A4@0
FEUI03Xj>P19BDT0?Shn079bLP1cLg<0Hf=S03/k>`1HF5P0Bd];05IFEP1IFET0KFe]02@T900V9RH0
BDU9028R8P1OGel0<S8b04Y:BP1FEUH0Bd];0861P@0g=cL0EEEE03Ph>010@4007AdM03<c<`19BDT0
<c<c04E5A@0m?Cd0FUYJ01@D500g=cL0<30`03/k>`1PH600D51@04E5A@1[Jf/0@T9201XJ6P0k>c/0
IVIV05IFEP0K6a/0GUiN0P1PH600F@1]KFd0:BTY04M7A`0[:b/0FEUI02LW9`1DE5@0GemO05UIF@1d
M7@0MgMg02PX:01bLW80<c<c03Xj>P13@d<0B4Q806YZJP0e=CD0CDe=04U9B@1CDe<0>SXj04m?C`13
@d<0DE5A04a<C01OGel0Ph>304];B`28R8P0;2`/06a/K014A4@0LG5a03lo?`1kNg/0K6a/055AD@1B
DU80AdM703lo?`1EEED0?Cdm06IVIP1LG5`0ADE504];B`19BDT0@D51069RHP17AdL0Fe]K04=3@`1U
IFD0Bd];04i>CP0h>3P0KFe]05MGE`0d=3@0Cdm?04Q8B00U9BD0LG5a04a<C029RHT0=SHf07IfMP1L
G5`0BTY:06ATI01>CTh0E5AD03hn?P1EEED0@410051@D01CDe<0IfMW05MGE`1iNGT0EEEE06ATI010
@400FUYJ06YZJP1CDe<0HF5Q013ooooo000>ooooo`0:0;Fe]@1ZJVX0Bd];061PH02<S8`0LG5a0410
@01LG5`0JFUY09BDU080<C4a0=80K6a/09ZJVP1XJ6P0[Jf]0451@@14A4@0O7al04e=C@1KFe/0KVi^
02DU9@1VIVH0RHV9059BDP29RHT0<S8b034a<@1TI6@0LG5a06e]K@1KFe/05QHF03/k>`1GEeL0J6QX
05=CD`1=CDd0:2PX04i>CP2HV9P0OWin04Q8B01@D500=cLg06e]K@2=SHd0>CTi04i>CP0c<c<092@T
03Xj>P0[:b/0EEEE04e=C@14A4@0Hf=S0;6a/@0l?3`07alO03<c<`1[Jf/0KVi^0:^[Z`0Q8B40>SXj
06MWI`1SHf<0OWin05IFEP0P8200<S8b06=SH`1mOGd0>3Ph09:BTP0L71`0=3@d05aLG01NGUh0<c<c
04Y:BP0e=CD0FUYJ06a/K00Y:BT0;Rh^08>3P`11@D40Fe]K06i^KP0];Bd0<30`03/k>`1EEED0I6AT
028R8P0H61P08b<S04];B`1cLg<0H61P01TI6@0R8R80>c/k05]KF`1?Cdl0Bd];06a/K00R8R80EUIF
09ZJVP1hN7P0E5AD0492@P0R8R80F5QH07inOP13@d<0?Cdm06QXJ00P8200BDU9096AT@12@T80?3`l
04a<C00I6AT0QHF507EeM@0_;bl0@D5103hn?P18B4P0Jf][02XZ:P0U9BD0C4a<04e=C@1LG5`0TY:B
05=CD`15ADD0GEeM06][J`1LG5`0][Jf03Xj>P0l?3`0HF5Q06UYJ@1UIFD0/K6a04M7A`14A4@0IVIV
05]KF`1HF5P0QXJ601lO7`0`<300GUiN034a<@0c<c<0CTi>02LW9`14A4@0T92@02`/;01ZJVX0?Shn
03Lg=`1MGEd0SHf=034a<@1KFe/0EeMG051@D02QXJ40SXj>020P8013@d<0ADE507IfMP2c/k<0<C4a
03<c<`1MGEd0IfMW0:f][@1RHV808b<S05aLG01ZJVX0Kfm_05ADE00n?Sh0Cdm?065QH@1HF5P0MgMg
01dM7@1KFe/0F5QH04m?C`1JFUX0N7Qh03/k>`2ZZZX0MGEe034a<@1RHV80:b/[02<S8`1nOWh0>SXj
05MGE`0h>3P0<S8b08V9R@1bLW8204M7A`0f034a<@0j>SX0QhN707emO@0m?Cd0G5aL05=CD`1hN7P0
ZZZZ09>CT`0_;bl0Bd];04I6AP1cLg<0X:2P09>CT`0X:2P0GEeM065QH@1LG5`0Xj>S0451@@0k>c/0
BTY:069RHP1oOgl0MWIf028R8P1IFET0JVYZ05QHF020P800WinO02/[:`1JFUX0CTi>04a<C00n?Sh0
De=C024Q8@0n?Sh0DE5A061PH00n?Sh0Cdm?02HV9P0T92@0HF5Q06i^KP0X:2P0=cLg02TY:@0k>c/0
RHV93_ooool0SP1NGUh0XZ:R07=cL`1:BTX0>SXj059BDP10@400=SHf04M7A`1PH600F5QH06EUI@1F
EUH0Fe]K069RHP1ADE40FUYJ04Y:BP1VIVH0D51@03Lg=`0o?cl0CDe=06UYJ@1:BTX0IfMW04A4A01]
KFd0=cLg05]KF`17AdL0Ogmo07QhN02ATI40/[:b0:BTY02OWil0J6QX04I6AP1=CDd0HF5Q04m?C`1m
OGd0EEEE03Hf=P0a<C40C4a<05aLG00_;bl0DU9B02PX:01ADE40KFe]02DU9@1OGel0H61P05ADE00n
?Sh0Ph>304E5A@0_;bl0@41003Hf=P0e=CD0<c<c04];B`1LG5`0>c/k04m?C`0h>3P0EeMG04i>CP1=
CDd0DU9B03Ti>@1LG5`0@d=306=SH`0b<S806a/K038b<P1:BTX0>CTi03/k>`0_;bl0ATI604e=C@0e
=CD0@410030`<01CDe<0>SXj0492@P0Y:BT0BTY:0492@P0d=3@0A4A404Y:BP1?Cdl0?3`l03hn?P1:
BTX0G5aL05EEE@1[Jf/0J6QX04U9B@1SHf<0I6AT09bLW029RHT0I6AT061PH01FEUH0CTi>051@D00j
>SX0E5AD0451@@1ADE40=cLg06MWI`24Q8@0DE5A08J6QP2NWYh0VIVI0:FUY@1gMgL0Q8B405QHF01n
OWh0=3@d04U9B@1KFe/0@D5106e]K@1RHV80:2PX03lo?`17AdL206EUI@0i07inOP1fMWH0JFUY04e=
C@1;Bd/0?Shn06][J`0R8R80BDU904e=C@0G5aL0M7Ad02h^;P1ADE40C4a<0451@@1PH600B4Q804i>
CP15ADD0@T9202HV9P1:BTX0EUIF05EEE@1=CDd0Cdm?06YZJP18B4P0Cdm?03`l?012@T80KVi^08j>
SP0m?Cd0DU9B0451@@0C4a<0;bl_04I6AP1<C4`04a<C05iNGP1PH600DU9B05eMG@0b<S80AdM703Hf
=P1HF5P0;2`/065QH@1?Cdl0HV9R06ATI023Ph<0?Shn0080CTi>04/0=cLg03`l?00];Bd0Fe]K02/[
:`1BDU80A4A405QHF00j>SX0EeMG03hn?P1:BTX0ADE505QHF01`L700<30`09BDU016ATH0GEeM04I6
AP1GEeL0;bl_06MWI`1BDU80>CTi059BDP1CDe<0BDU904Q8B00j>SX0G5aL06=SH`1<C4`0@T9203Ph
>00a<C40EeMG059BDP1cLg<08R8R055AD@0i>CT0>SXj03Hf=P1CDe<0LG5a04U9B@1UIFD0BTY:04I6
AP1PH600IVIV08j>SP1QHF40SHf=05ADE016ATH0FEUI055AD@1EEED0C4a<05eMG@1GEeL0IVIV03Xj
>P1LG5`0BTY:06EUI@16ATH0D51@04a<C01NGUh0HV9R05eMG@28R8P04?ooool000kooooo07X0IFEU
07emO@23Ph<0HF5Q0410@01HF5P0De=C09JFUP1_Kfl0IfMW065QH@1?Cdl0E5AD03Lg=`17AdL0MgMg
04a<C01;Bd/0F5QH03Ph>01XJ6P0HV9R02d];@0m?Cd0A4A406e]K@2OWil0=3@d02@T9019BDT0N7Qh
06e]K@21PH4071`L02DU9@1PH600Cdm?09RHV01nOWh092@T02XZ:P19BDT0K6a/06m_K`0/;2`0>SXj
059BDP1LG5`0ATI607AdM00J6QX06a/K030`<013@d<0EUIF06UYJ@15ADD0>3Ph05mOG`1JFUX0?3`l
071`L00/;2`0BDU907IfMP1lO7`0?3`l06a/K00S8b<0C4a<06=SH`0O7al0ATI605=CD`0i>CT0;Rh^
07MgM`0a<C4092@T038b<P1IFET0ADE505UIF@0J6QX0=3@d04e=C@0f=SH0LW9b0;Ng]`0Z:RX0B4Q8
06UYJ@1>CTh0Lg=c08j>SP0V9RH0Fe]K09VIV@1EEED0Lg=c04i>CP0I6AT0BTY:03Ph>00I6AT0G5aL
03@d=00D51@0;2`/06][J`0H61P0>3Ph04E5A@0`<300>SXj05aLG00F5QH0=CDe065QH@1:BTX0I6AT
08V9R@80;bl_09X0NGUi05YJFP1gMgL0KFe]01/K6`16ATH0LG5a05YJFP29RHT0<c<c03Lg=`1NGUh0
E5AD06EUI@1^KVh06ATI02@T901RHV80IfMW06=SH`1LG5`07QhN04U9B@1`L700I6AT05mOG`1iNGT0
=3@d02h^;P1SHf<0MGEe03`l?012@T80FEUI055AD@2CTi<0FUYJ01dM7@1IFET0?Cdm06ATI01OGel0
92@T034a<@1VIVH0F5QH0;2`/01<C4`071`L0492@P1]KFd0HV9R09NGU`0Y:BT09bLW055AD@24Q8@0
E5AD0820P00S8b<0AdM7075aL@1MGEd0>c/k03De=@12@T80M7Ad06e]K@13@d<0Cdm?034a<@1DE5@0
U9BD02l_;`1?Cdl0?clo030`<02@T900CDe=03Ph>016ATH0De=C09FEU@1kNg/0;bl_06YZJP0m?Cd0
NWYj09bLW00Z:RX09bLW05]KF`24Q8@0VIVI04Q8B00P8200Jf][0820P02ZZZX0Rh^;02h^;P0b<S80
EEEE069RHP1nOWh0IFEU02DU9@0/;2`0LW9b092@T01JFUX0Lg=c03Ph>00l?3`0L71`09>CT`1@D500
M7Ad02HV9P0a<C40RXZ:07emO@0l?3`0KFe]03`l?01RHV80[Zj^06ATI00_;bl0GEeM0492@P1LG5`0
HF5Q06MWI`0a<C40EUIF03Ph>01]KFd0PH61034a<@0^;Rh0ATI604M7A`13@d<0M7Ad03<c<`11@D40
?Cdm059BDP14A4@0JVYZ0451@@0j>SX0BTY:3_ooool0f01NGUh0MgMg04E5A@1>CTh0GUiN05UIF@1R
HV80D51@05UIF@1cLg<0=SHf069RHP1oOgl0SXj>08f=S@1SHf<0J6QX05MGE`1_Kfl0CDe=04=3@`1<
C4`0BTY:05]KF`13@d<0F5QH05IFEP1LG5`0@D51051@D01>CTh0FUYJ07UiN@1XJ6P0IfMW07IfMP2H
V9P0IfMW04];B`1HF5P0CTi>0410@01ADE40J6QX04];B`0k>c/0E5AD055AD@0/;2`0BDU903Ti>@1>
CTh0G5aL04];B`1VIVH0J6QX06EUI@1HF5P0Vi^K04m?C`1JFUX0Fe]K04a<C01;Bd/0@T92069RHP1P
H600ATI607=cL`1BDU80RHV90451@@1IFET0LG5a051@D01PH600B4Q806ATI01?Cdl0;Rh^03Xj>P1<
C4`0>CTi04I6AP0c<c<0De=C05iNGP16ATH0@T9204A4A01SHf<0>c/k055AD@17AdL0G5aL05IFEP1=
CDd0FUYJ04=3@`1OGel0@T9205ADE015ADD0<C4a04=3@`17AdL0C4a<07inOP1`L700@d=305UIF@2E
UID0Ng]k05IFEP1KFe/0BTY:04e=C@0h>3P0FEUI061PH01EEED0Bd];07QhN02@T900@T9206=SH`1/
K6`0[Jf]08b<S026QXH0RHV905YJFP1hN7P0Cdm?05iNGP1UIFD0De=C03Xj>P1UIFD0;Rh^059BDP1H
F5P0Ph>307YjNP1cLg<0B4Q8051@D017AdL0GemO06IVIP1gMgL0@d=307=cL`1OGel05aLG05]KF`0Y
:BT0>3Ph04I6AP0j>SX0MGEe030`<01=CDd0>3Ph04Y:BP0C4a<0:RXZ03lo?`0a<C40@41003Hf=P1<
C4`06QXJ03/k>`0i>CT0EEEE06QXJ02EUID0>SXj04Q8B01EEED08B4Q03Ti>@16ATH0Cdm?028R8P1b
LW80JVYZ05iNGP1VIVH07AdM03Ph>00Q8B40BTY:01TI6@1@D500G5aL06ATI01iNGT0UYJF03Xj>P1K
Fe/08R8R04A4A01EEED0=SHf05eMG@1;Bd/0De=C04U9B@1>CTh0?Cdm04a<C01:BTX0H61P05aLG080
PH610300@T9206a/K016ATH0GEeM03dm?@1=CDd0?Cdm08Z:RP1YJFT0D51@05IFEP19BDT0CDe=05eM
G@0`<300FUYJ07AdM01OGel0IVIV04m?C`0h>3P0HV9R034a<@1]KFd0DU9B07=cL`0l?3`0Kfm_04U9
B@1FEUH0?3`l02<S8`0g=cL0=SHf04a<C01eMGD0FUYJ0820P00m?Cd0Vi^K06QXJ01NGUh0Lg=c03lo
?`16ATH0;2`/04I6AP0a<C420451@@0:05IFEP1EEED0K6a/051@D01;Bd/0BTY:05eMG@1aLG40L71`
06IVIQ3ooooo000>ooooo`0407QhN01JFUX0Jf][07AdM080D51@0?l0HV9R0:f][@1UIFD0@D5105IF
EP20P800_;bl03Hf=P0k>c/0EUIF05iNGP2>SXh0De=C03lo?`12@T80TY:B08F5Q@1]KFd0=cLg03Ti
>@1VIVH0XZ:R05YJFP1lO7`0;bl_051@D01nOWh0GemO03hn?P2SXj<0>SXj03De=@1WIfL0>c/k04I6
AP1EEED0:b/[04e=C@1PH6006ATI0410@00l?3`0;Rh^04m?C`0J6QX07AdM02`/;013@d<0@T9207Qh
N012@T80:2PX04];B`1>CTh0ADE50:2PX00d=3@07alO05IFEP16ATH0HV9R08n?S`0Y:BT0<30`092@
T01GEeL0S8b<0;Fe]@0];Bd05aLG06ATI01HF5P0FEUI05]KF`0:2PX0820P05QHF00V9RH0LG5a05YJ
FP0B4Q80<S8b06QXJ010@400I6AT02XZ:P0a<C40?clo06YZJP0a<C40>CTi01DE5@0c<c<0CDe=07mo
O`0i>CT0?clo02/[:`15ADD0V9RH04];B`0@41008b<S06MWI`1DE5@0VIVI05iNGP0G5aL0B4Q804e=
C@16ATH0U9BD02@T900R8R80<c<c04i>CP1GEeL0Ng]k03Lg=`0R8R80J6QX06IVIP0c<c<0GUiN01DE
5@0m?Cd0EEEE05YJFP0U9BD0ATI602DU9@1_Kfl0WYjN03De=@0K6a/0DU9B0410@02;Rh/0UIFE02XZ
:P0K6a/0<C4a04a<C02KVi/0`/;202PX:012@T80EUIF06][J`20P800O7al034a<@1MGEd0EEEE07al
O01XJ6P0UYJF02h^;P1JFUX0TY:B06QXJ01eMGD0GUiN03De=@1jNWX0T92@04m?C`10@400FUYJ03@d
=02^[Zh0I6AT04A4A01JFUX0=3@d03Ph>01lO7`0BTY:038b<P1SHf<0=3@d08R8R022PX807AdM05mO
G`1>CTh0GEeM0;6a/@1XJ6P0:2PX06e]K@1=CDd0R8R804e=C@0Y:BT0EUIF08f=S@1nOWh0IVIV01XJ
6P10@400IFEU06QXJ02JVYX0:RXZ030`<01=CDd0D51@06EUI@1;Bd/0<c<c07YjNP1cLg<09RHV06MW
I`0`<300@D51079bLP0k>c/0:RXZ04I6AP0M7Ad0DE5A07inOP0];Bd0>SXj05=CD`14A4@0Jf][09BD
U00d=3@0<S8b04=3@`1:BTX0E5AD08J6QP0U9BD0:b/[04Y:BP1SHf<0HV9R0:NWY`15ADD0?Cdm05YJ
FP1^KVh0MWIf09RHV01;Bd/0?Cdm05=CD`1YJFT0@d=305eMG@0`<3000140<C4a065QH@1XJ6P0@T92
07emO@0a<C406ATI05UIF@0d=3@0ADE505=CD`0e=CD07AdM06m_K`1<C4`0:BTY04M7A`0>ooooo`1b
071`L01cLg<0C4a<03Lg=`0Y:BT0AdM704i>CP15ADD0I6AT06EUI@0i>CT0EUIF065QH@1kNg/0SXj>
0:RXZ01TI6@0CTi>09fMW@1JFUX092@T03/k>`14A4@0GEeM03De=@11@D40>3Ph04m?C`12@T80Hf=S
04Q8B01JFUX0LG5a07emO@2JVYX0Ng]k06a/K01lO7`0D51@03hn?P1OGel0BDU906EUI@1DE5@0C4a<
04e=C@1<C4`0A4A403@d=01=CDd0=CDe03lo?`1>CTh0:b/[03lo?`1LG5`0JFUY07EeM@1WIfL0GemO
05aLG01RHV80Fe]K05YJFP1FEUH0GemO05iNGP1EEED0JFUY07=cL`1^KVh0AdM705eMG@1BDU80<S8b
03hn?P14A4@0CDe=03Xj>P0`<300BDU90451@@0o?cl0?Shn04Q8B01:BTX0F5QH03@d=014A4@0?clo
04M7A`0a<C40@41005MGE`1>CTh0@T9205eMG@1DE5@0CTi>051@D01=CDd0Fe]K05UIF@1CDe<0I6AT
04A4A01CDe<0CDe=05eMG@1=CDd0QHF507]kN`1ZJVX0PX:20P1SHf<0601XJ6P0>CTi05mOG`0m?Cd0
KFe]04m?C`1PH600MGEe05ADE01iNGT0_[jn0;Zj^P25QHD0CTi>05UIF@1VIVH0LW9b034a<@0T92@0
B4Q803Xj>P14A4@0IVIV03Hf=P80CTi>00@0NGUi0::RXP1lO7`0FUYJ0P1dM7@07P12@T80=cLg069R
HP19BDT0FUYJ069RHP10@400S8b<0410@012@T80AdM705=CD`22PX80@41005QHF018B4P0EEEE02@T
900g=cL0C4a<03@d=010@400=SHf04m?C`0A4A40?3`l02XZ:P0a<C40;bl_09jNWP80CTi>0680IVIV
024Q8@0d=3@0CTi>04m?C`0E5AD0DE5A05QHF018B4P0EEEE03@d=01UIFD0@D5104m?C`0k>c/0L71`
0410@01BDU80I6AT08R8R00Y:BT0LW9b02TY:@19BDT0EUIF0492@P1=CDd0=SHf04=3@`0l?3`0De=C
03De=@1;Bd/0>CTi04a<C018B4P0Hf=S07AdM011@D40MGEe04Y:BP1ADE40>3Ph05IFEP0S8b<0HV9R
03Ti>@11@D40IVIV051@D00f=SH0=3@d030`<029RHT0EEEE04M7A`1BDU80G5aL03Xj>P1QHF40<c<c
07AdM00P8200DU9B03Lg=`0c<c<0:b/[06EUI@1;Bd/0<S8b03`l?00L71`0;2`/07IfMP1IFET0NWYj
04M7A`1dM7@0IfMW03hn?P1EEED0De=C06QXJ017AdL0Bd];04E5A@0o?cl0>CTi065QH@1?Cdl0Q8B4
05EEE@1KFe/0IFEU06][J`1YJFT0Jf][07IfMQ3ooooo000>ooooo`1W0<W9b@1@D500G5aL08j>SP2F
UYH0B4Q803lo?`1lO7`0Vi^K06m_K`1:BTX0>CTi05=CD`1XJ6P0@41005mOG`0i>CT0DU9B06][J`0f
=SH0AdM706=SH`0H61P0IfMW04Y:BP0c<c<0D51@03lo?`0];Bd0RHV903@d=00[:b/0DU9B0410@01V
IVH0Rh^;0451@@0m?Cd0EeMG04i>CP1kNg/0ZJVY02`/;00h>3P0E5AD04E5A@2QXJ40E5AD01PH6017
AdL0BDU9059BDP25QHD0<30`024Q8@0`<300E5AD06i^KP1VIVH08b<S028R8P1:BTX0FEUI04m?C`1>
CTh08b<S02h^;P1RHV80B4Q803Ph>00X:2P0:BTY0451@@1LG5`0A4A401hN7P0g=cL0@41005eMG@1V
IVH06a/K01TI6@1=CDd0D51@07EeM@1gMgL05ADE018B4P11@D40EEEE0::RXP15ADD05aLG02<S8`1P
H600DE5A0820P00H61P0;Rh^05EEE@0k>c/0C4a<05iNGP0202<S8`0e061PH01dM7@0<30`04A4A01@
D5007alO05mOG`2?Shl0>SXj028R8P0m?Cd0=cLg08Z:RP1hN7P08R8R03@d=01:BTX0A4A407MgM`2;
Rh/0:BTY030`<01DE5@0;Bd]0;6a/@0^;Rh06a/K05EEE@1RHV80HF5Q07]kN`0S8b<0<C4a06m_K`1K
Fe/0Cdm?08f=S@17AdL0=CDe07IfMP19BDT0CDe=06IVIP0U9BD0Cdm?092@T01OGel0?3`l06e]K@0Z
:RX0<S8b08N7Q`15ADD00P17AdL0B`0Y:BT0@41009^KV`1EEED0=cLg03dm?@0g=cL0CTi>07EeM@0^
;Rh0De=C051@D01jNWX0Ti>C07alO00Y:BT0HF5Q075aL@1VIVH0ZJVY03Lg=`13@d<0FUYJ04E5A@1P
H600[jn_03/k>`0n?Sh0/[:b03hn?P1VIVH0<C4a03`l?01KFe/0R8R805UIF@1>CTh0:b/[04i>CP1_
Kfl0?clo0492@P0h>3P0K6a/069RHP0c<c<0>CTi05=CD`0g=cL0][Jf06ATI00/;2`0EeMG03Lg=`1[
Jf/0LG5a02PX:014A4@0De=C04Q8B01jNWX0<S8b04=3@`1RHV80LW9b07emO@1gMgL08b<S03De=@1[
Jf/0DE5A07MgM`28R8P0;2`/04i>CP0208B4Q00;08R8R02^[Zh0>CTi04Q8B01SHf<0FEUI05=CD`1`
L70092@T03/k>`1WIfL00P1MGEd0701CDe<09RHV04];B`1>CTh0Cdm?034a<@1jNWX0>c/k061PH02<
S8`0G5aL020P8017AdL0<S8b05=CD`2k^k/0U9BD024Q8@1:BTX0C4a<065QH@20P800>CTi01HF5P1H
F5P0PH6105mOG`28R8P>ooooo`0J0492@P1RHV80GemO05QHF01IFET0GUiN06i^KP1@D500M7Ad06a/
K01UIFD0KFe]05aLG02FUYH0MgMg05EEE@1SHf<0De=C069RHP1=CDd0@41004=3@`1ADE40H61P0410
@01MGEd205mOG`140492@P1DE5@0E5AD05MGE`1lO7`0GUiN07MgM`24Q8@0RXZ:06EUI@0n?Sh0?Cdm
05aLG01GEeL0Fe]K04];B`1QHF40@41004I6AP1:BTX0;Bd]04Q8B00b<S80?clo065QH@1CDe<0NGUi
06][J`1KFe/0I6AT06e]K@1=CDd0@d=304i>CP0l?3`0@41004=3@`1=CDd0DE5A03lo?`13@d<0<c<c
0410@00k>c/0A4A405MGE`0e=CD0Hf=S03De=@1dM7@0ATI6024Q8@0[:b/0?3`l03<c<`0g=cL0:RXZ
03Ti>@1@D500<c<c04Q8B00f=SH0DU9B038b<P0n?Sh0:2PX04i>CP17AdL204];B`1<03Ph>01:BTX0
BTY:04=3@`1?Cdl0Lg=c06ATI01UIFD0L71`05IFEP1[Jf/0OWin09:BTP29RHT0KVi^07UiN@1fMWH0
De=C059BDP15ADD0BTY:04];B`1=CDd0@41006=SH`2LW9`0De=C06m_K`2LW9`0[Zj^096AT@1;Bd/0
F5QH04];B`1dM7@0>CTi028R8P1PH600>SXj04=3@`1;Bd/0>CTi051@D00e=CD0AdM705mOG`1RHV80
@D5105eMG@15ADD0CTi>0492@P1OGel09RHV08B4Q01/K6`0@T9205YJFP1;Bd/0Cdm?04U9B@0h>3P0
HV9R04];B`1<C4`0>c/k059BDP0i>CT0Bd];05ADE01@D500EEEE059BDP1GEeL0?Cdm061PH080>CTi
00<0?3`l07emO@0h>3P00P1=CDd0H@0O7al0>c/k05]KF`1?Cdl0;Bd]08V9R@20P800Bd];04e=C@0F
5QH0@d=302LW9`11@D405QHF04U9B@1<C4`0GemO08^;R`2ZZZX0>CTi05]KF`0Q8B40<S8b04U9B@0W
9bL0ADE5034a<@1<C4`0Bd];05]KF`18B4P0@d=304M7A`1CDe<0IfMW06EUI@1VIVH0BDU9079bLP1B
DU80K6a/03lo?`1CDe<0ATI6071`L01?Cdl0ATI604Y:BP1=CDd0DE5A04a<C00W9bL0AdM706IVIP12
@T80J6QX03/k>`0^;Rh0EEEE03De=@1ZJVX0>3Ph059BDP0S8b<0Bd];04=3@`1NGUh0G5aL04I6AP1=
CDd05ADE03Hf=P1HF5P0GUiN06a/K01;Bd/0Ph>3065QH@17AdL0F5QH06m_K`1TI6@0;Bd]04i>CP1;
Bd/0De=C03Lg=`1/K6`0L71`07IfMP1iNGT0OGem05IFEP1SHf<0KVi^08N7Q`24Q8@04?ooool000ko
oooo08l0O7al06UYJ@1PH600F5QH0:6QX@1MGEd0=SHf04m?C`1mOGd0QHF503Lg=`0f=SH0FEUI06UY
J@1QHF40[Zj^03Lg=`1<C4`0E5AD05aLG01aLG40WinO02XZ:P17AdL0GEeM07AdM02][Jd0>c/k02l_
;`0i>CT0M7Ad07emO@2=SHd05QHF02TY:@1DE5@0GEeM092@T02;Rh/0<S8b02l_;`1gMgL0J6QX06UY
J@0/;2`07AdM04Y:BP1TI6@0:RXZ03`l?00I6AT0B4Q804I6AP1SHf<09RHV0451@@0V9RH0KVi^0:>S
X`0e=CD05QHF03Xj>P11@D40Jf][07inOP0I6AT0820P04E5A@1CDe<0Kfm_03Ti>@0C4a<0?3`l07=c
L`1iNGT0?Cdm08>3P`0G5aL0=3@d04e=C@1@D500CDe=061PH00H61P0:b/[04Y:BP0`<300B4Q806i^
KP0M7Ad0ADE506QXJ00e=CD0DE5A04U9B@0Y:BT0Kfm_07moO`0W9bL0Kfm_02l_;`0O7al0JFUY04E5
A@0D51@0<S8b059BDP0k>c/0F5QH06ATI00>3Ph0BTY:06e]K@17AdL0HF5Q051@D00S8b<0CTi>079b
LP0e=CD0GUiN06][J`0S8b<0IVIV06QXJ011@D40I6AT05IFEP0X:2P0K6a/06=SH`11@D40Fe]K03@d
=00m?Cd0QhN7030`<00^;Rh0G5aL02DU9@0Y:BT0MGEe051@D00203Lg=`1f055AD@1SHf<0Q8B402/[
:`0d=3@0E5AD05iNGP1[Jf/0b<S804U9B@0Y:BT0EeMG055AD@1nOWh0^KVi02d];@0e=CD0DU9B05aL
G021PH40@D5101lO7`1DE5@0GEeM05=CD`35aLD0DU9B01DE5@1TI6@0Kfm_04E5A@1[Jf/0<S8b02PX
:01dM7@0KVi^02d];@11@D40B4Q804i>CP2>SXh0=3@d02h^;P0n?Sh0HV9R0<C4a01SHf<08B4Q04Y:
BP1<C4`0Jf][07IfMP0W9bL0<c<c05mOG`1YJFT0IVIV01/K6`1PH600MgMg04E5A@2;Rh/0KFe]034a
<@23Ph<0NWYj07AdM02CTi<0>c/k03@d=01ZJVX0IVIV08V9R@0U9BD0=cLg05ADE01<C4`0EeMG06AT
I00T92@0EUIF0:j^[P1`L700FUYJ051@D00/;2`0GEeM07MgM`1EEED0=SHf04=3@`12@T80Fe]K08Z:
RP1fMWH0;Rh^04I6AP1?Cdl0IFEU096AT@1;Bd/0>3Ph05EEE@1?Cdl0WIfM09NGU`0W9bL0ADE506AT
I01PH600Lg=c09:BTP11@D40:RXZ05=CD`2<S8`0Bd];06i^KP80;Bd]00d0GemO0861P@0f=SH0?clo
01HF5P0[:b/0DE5A05]KF`0N7Qh0=3@d02@T9016ATH0D51@00kooooo03T0M7Ad06MWI`19BDT0?clo
03De=@1PH600EeMG04A4A01OGel0NWYj0451@@1EEED0SXj>0;bl_02k^k/0[Jf]06m_K`1GEeL0MgMg
04E5A@0c<c<0BDU903lo?`13@d<0DE5A05=CD`0n?Sh0JFUY05QHF01cLg<0@T920861P@2ZZZX0Shn?
07AdM02;Rh/0RXZ:06i^KP1?Cdl0BDU905YJFP13@d<0LG5a05YJFP1EEED0>c/k05UIF@1NGUh0:2PX
04];B`1>CTh0FEUI05aLG00W9bL0@D5105QHF01OGel00P1ZJVX09`1;Bd/0CDe=075aL@1GEeL0CDe=
075aL@1cLg<0IVIV04U9B@1NGUh0EUIF08V9R@0k>c/0DU9B07QhN01TI6@0GemO03Xj>P1_Kfl0ATI6
02@T900_;bl0C4a<03De=@0n?Sh0B4Q804m?C`1GEeL0?3`l05iNGP0k>c/0E5AD03/k>`10@400=3@d
059BDP1CDe<0BTY:04i>CP0204m?C`2803Ti>@1>CTh0CTi>03`l?01=CDd0>CTi03lo?`1VIVH0Fe]K
04Q8B01JFUX0Ti>C0861P@1/K6`0LG5a05QHF01BDU80=cLg04Y:BP11@D40E5AD03lo?`1lO7`0TY:B
061PH02YZJT0WYjN08n?S`2UYJD0CTi>07EeM@1DE5@0OGem059BDP0[:b/0GEeM04i>CP1?Cdl0IVIV
03Hf=P1=CDd0?Shn06YZJP2<S8`0Kfm_04E5A@1dM7@0D51@05MGE`0n?Sh0IfMW030`<01DE5@0P820
02TY:@1UIFD0;bl_055AD@1>CTh0?3`l05iNGP1=CDd0EeMG03lo?`1:BTX0>SXj051@D01RHV80;Bd]
04U9B@18B4P0DU9B01HF5P0^;Rh0>CTi051@D01SHf<0MGEe0451@@1=CDd0Cdm?01HF5P1@D500BTY:
03Ph>00<30`0>c/k05YJFP1RHV80Hf=S02DU9@12@T80?3`l05=CD`0X:2P0DU9B03hn?P1JFUX0D51@
06i^KP0d=3@0Fe]K01<C4`12@T80E5AD03lo?`1/K6`0D51@06UYJ@1jNWX0Fe]K04i>CP1WIfL0>3Ph
055AD@1ZJVX0F5QH06i^KP1OGel0LW9b08^;R`1@D500?clo04a<C00Z:RX0R8R806ATI011@D40FUYJ
03hn?P1JFUX0F5QH04U9B@1KFe/0IFEU05UIF@80@41002P0;Rh^04i>CP0P8200ZJVY03De=@1EEED0
=SHf04Y:BP0_;bl0MgMg065QH@0[:b/0DU9B04Q8B00_;bl0FEUI05]KF`2:RXX0Bd];069RHP1LG5`0
B4Q805UIF@11@D40DU9B038b<P1?Cdl0;bl_04Y:BP11@D40Fe]K059BDP1KFe/0C4a<05EEE@18B4P0
IFEU05YJFP1`L700LG5a4?ooool000kooooo00@0MGEe05YJFP27QhL0Kfm_0P1PH600^@1EEED0YZJV
06ATI01VIVH0GemO06EUI@28R8P0?Cdm03Hf=P1YJFT0IVIV07MgM`1QHF40;Bd]04e=C@1bLW80KFe]
07QhN00c<c<0=3@d0451@@1CDe<0HV9R07UiN@0L71`0ATI608R8R01HF5P0DU9B07]kN`0Y:BT0Bd];
07IfMP0l?3`0BDU9065QH@0[:b/0DE5A04];B`0Z:RX0IVIV05UIF@0d=3@0JFUY02/[:`0V9RH0A4A4
04i>CP11@D40KVi^04A4A00X:2P0BTY:05QHF010@400OWin03<c<`0i>CT0CTi>0451@@1KFe/0Ph>3
01XJ6P0a<C40FEUI03lo?`1^KVh0F5QH02`/;00g=cL0GEeM034a<@1@D500ADE502<S8`0g=cL0IfMW
028R8P1@D500@T9202HV9P1>CTh0U9BD04E5A@1JFUX0;Rh^02d];@1BDU80NWYj03Lg=`1:BTX0=CDe
038b<P1iNGT0F5QH02@T900n?Sh0?clo055AD@1TI6@0?Cdm01/K6`0j>SX0BDU90451@@2?Shl0De=C
01/K6`0_;bl0BDU904Q8B01[Jf/0<C4a02`/;015ADD0KFe]0492@P1LG5`0?Shn04I6AP1/K6`0Lg=c
02DU9@1ZJVX0<30`051@D020P800B4Q802LW9`1HF5P0?Cdm04e=C@1MGEd0:b/[02PX:01PH600B4Q8
075aL@1ZJVX09RHV02l_;`1GEeL0DE5A05UIF@1FEUH08b<S04U9B@1LG5`0LW9b05iNGP0j>SX092@T
03Lg=`1dM7@0Lg=c03lo?`1YJFT0;2`/06=SH`1aLG40>CTi061PH01>CTh0@T92071`L02DU9@0ATI6
069RHP1WIfL0AdM7096AT@16ATH0;2`/05EEE@1GEeL0Bd];06QXJ00a<C40Bd];04i>CP0l?3`0G5aL
06e]K@0_;bl0H61P0080EUIF04<0V9RH06MWI`0j>SX0K6a/04=3@`1cLg<08b<S02h^;P1cLg<0DE5A
06][J`1RHV80:RXZ05MGE`24Q8@0D51@06YZJP0a<C40@41007inOP1NGUh0De=C03Ti>@1?Cdl0QHF5
06ATI00i>CT0OWin03dm?@1ZJVX0UiNG055AD@11@D40F5QH04E5A@1RHV80Jf][03lo?`1@D500IFEU
04Q8B01WIfL0IVIV02l_;`17AdL0FEUI05QHF01FEUH0Kfm_030`<00h>3P0C4a<05ADE01MGEd0S8b<
04Q8B00c<c<0DE5A051@D01WIfL0QHF503@d=00Y:BT0FUYJ05]KF`1EEED0LW9b0080>CTi0100EUIF
03/k>`1<C4`0JVYZ03Xj>P0m?Cd0HF5Q02l_;`0c<c<0Bd];034a<@17AdL0HF5Q03`l?00m?Cd0FUYJ
3_ooool0801ADE40Ng]k06=SH`1ADE40KVi^07AdM01OGel0ADE506a/K01gMgL0FUYJ071`L01eMGD0
MgMg0:6QX@25QHD0L71`05eMG@1oOgl0EEEE0410@01;Bd/0BTY:04M7A`16ATH0IVIV04E5A@1SHf<0
@T9206IVIP12@T80NWYj0P2BTY806P1mOGd0Ti>C09FEU@1hN7P0DE5A059BDP1YJFT0CDe=07inOP1B
DU80E5AD0410@01?Cdl0Fe]K030`<01IFET0?3`l03lo?`1_Kfl0=CDe059BDP1hN7P0HF5Q04m?C`1_
Kfl0HF5Q0P1GEeL04P1<C4`0D51@0451@@1DE5@0IfMW05MGE`1FEUH0Fe]K06e]K@1;Bd/0EeMG06i^
KP0n?Sh0F5QH04A4A01SHf<0>c/k02PX:080?Cdm0240;Rh^03De=@0^;Rh0>c/k05=CD`0[:b/0>CTi
03Hf=P1;Bd/09BDU03hn?P17AdL0EeMG0492@P19BDT0B4Q803/k>`1;Bd/0@41004A4A01<C4`0BTY:
069RHP1DE5@0G5aL04a<C01JFUX0Kfm_07emO@1`L700IfMW07UiN@22PX800P1HF5P07P12@T80B4Q8
0410@01QHF40BTY:06a/K01eMGD0HV9R08V9R@1jNWX0SXj>09>CT`1IFET0MGEe04m?C`1/K6`0?Cdm
03lo?`1FEUH0Bd];05ADE01IFET0?3`l04I6AP0m?Cd0MWIf09^KV`1ZJVX0F5QH065QH@80FEUI01T0
GemO07]kN`0o?cl0HV9R09JFUP0k>c/0O7al04A4A019BDT0CTi>03@d=01/K6`0<30`05aLG01@D500
CTi>01DE5@10@400DE5A03`l?019BDT0@T92069RHP0K6a/0@T920080?Shn00/0B4Q805eMG@17AdL0
Fe]K055AD@0Q8B40?3`l06ATI01HF5P08b<S04e=C@0205EEE@1I05YJFP0U9BD0@T9202@T9017AdL0
@d=305IFEP1:BTX0EUIF05]KF`1PH600<C4a06MWI`19BDT0@41003dm?@18B4P0F5QH0410@014A4@0
Cdm?05YJFP12@T80BTY:04A4A015ADD0E5AD05YJFP1ZJVX0DE5A07moO`1_Kfl0Cdm?05EEE@1fMWH0
KFe]06a/K01ADE40@41006MWI`1MGEd0Fe]K07alO01OGel0OWin06i^KP1=CDd0C4a<04I6AP0_;bl0
EUIF02/[:`1LG5`0@D5106ATI00k>c/0De=C03@d=01CDe<0IVIV04e=C@1UIFD0@4100492@P1MGEd0
KFe]07emO@13@d<0GemO05UIF@16ATH0FUYJ04M7A`1GEeL0?3`l04];B`0f=SH0B4Q802/[:`1KFe/0
BDU906ATI01RHV80I6AT05IFEP1cLg<0MgMg07IfMP27QhL04?ooool000kooooo00d0_kno04M7A`1D
E5@0Shn?0;Zj^P1;Bd/0AdM706QXJ01oOgl0S8b<02DU9@15ADD0D51@0080JVYZ01L0YJFU02/[:`1A
DE40VIVI05UIF@2SXj<0Q8B4030`<01NGUh0NWYj05UIF@1HF5P07alO02TY:@20P800Hf=S05ADE01C
De<0820P03hn?P1ADE40B4Q80492@P0203Lg=`1:04e=C@2=SHd0BTY:03@d=00U9BD0<S8b08F5Q@2C
Ti<0>SXj05ADE011@D40@41008b<S01;Bd/07QhN04U9B@12@T80MGEe08^;R`0F5QH0:b/[0410@01E
EED0O7al069RHP0K6a/0<c<c04Q8B01MGEd0J6QX02LW9`0Y:BT0?Cdm065QH@0j>SX0<c<c05YJFP0H
61P0?Cdm05QHF00[:b/0:BTY03Xj>P0O7al0MGEe06ATI00G5aL06QXJ03dm?@0e=CD0Ogmo05ADE00F
5QH09bLW03Hf=P1ZJVX0Rh^;030`<00b<S80820P04=3@`1@D500PH6101TI6@0c<c<0BDU9065QH@14
A4@0GUiN04=3@`0W9bL0CTi>08:2PP1VIVH203`l?02R02PX:013@d<0EEEE01DE5@0d=3@0@41003@d
=01UIFD0ADE501`L700g=cL0AdM703@d=02;Rh/0;Bd]02<S8`1RHV80EUIF04U9B@2WYjL071`L03Ph
>014A4@0?Cdm079bLP2n_[h0?Shn03Hf=P1KFe/0D51@065QH@1XJ6P0?Shn059BDP1gMgL0CDe=04m?
C`1`L700<S8b04e=C@1UIFD0?clo04e=C@13@d<09RHV03dm?@1]KFd0;Rh^04A4A00b<S80;bl_061P
H01eMGD0:RXZ04A4A013@d<0IfMW08J6QP2HV9P0<c<c055AD@0j>SX0Hf=S0:JVYP17AdL05QHF06IV
IP11@D40QHF507]kN`0M7Ad0@D5105UIF@1SHf<0/K6a034a<@0M7Ad0H61P09fMW@1RHV80EUIF03/k
>`1LG5`0Fe]K02h^;P0j>SX09bLW06a/K02;Rh/0?Shn04=3@`14A4@0=cLg075aL@15ADD0FEUI05eM
G@0U9BD0IFEU07=cL`1;Bd/0E5AD03Xj>P10@400Ph>304];B`16ATH0>3Ph05=CD`1/K6`0PX:202TY
:@0h>3P0@d=304U9B@1hN7P0/k>c038b<P11@D40E5AD04U9B@2ATI40Shn?0492@P0V9RH0HF5Q051@
D01fMWH0LG5a03hn?P0i>CT0PX:207]kN`1ZJVX0B4Q802l_;`12@T80PX:205UIF@1SHf<0HV9R01TI
6@18B4P0E5AD06e]K@0[:b/0@T9202HV9P1:BTX0N7Qh07YjNP0Z:RX0C4a<03Hf=P1BDU80VIVI05=C
D`0X:2P0DU9B03`l?01NGUh0RXZ:3_ooool06`21PH40Rh^;06m_K`1@D500GEeM06=SH`19BDT0?Shn
04];B`1OGel0C4a<05aLG01lO7`0NGUi07EeM@1oOgl0KVi^04M7A`20P800C4a<04Q8B00o?cl0?Cdm
04i>CP1YJFT0EeMG04i>CP02051@D00Q05mOG`0o?cl0FEUI07moO`1KFe/0Bd];061PH01lO7`0Hf=S
04E5A@18B4P0ADE503lo?`1JFUX0EUIF04m?C`0m?Cd0G5aL05UIF@0i>CT0D51@04E5A@19BDT0HV9R
0451@@1DE5@0JVYZ06a/K01]KFd0Ng]k05ADE01=CDd0GemO0080EeMG04<0GEeM07=cL`27QhL0EEEE
05MGE`1BDU80UYJF04E5A@19BDT0R8R804i>CP1/K6`0@T9208>3P`1^KVh0@D5104A4A01ADE40=SHf
0492@P1=CDd0Bd];05mOG`1:BTX0C4a<03Hf=P1JFUX0=cLg04U9B@1?Cdl0G5aL059BDP1<C4`0DE5A
05EEE@1KFe/0BDU905ADE01@D500@d=305UIF@1EEED0FUYJ06EUI@20P800NWYj08B4Q01[Jf/0HF5Q
06a/K01dM7@0EEEE051@D00o?cl0D51@04I6AP1XJ6P0Bd];07EeM@2OWil0Fe]K04e=C@1PH600LW9b
07AdM021PH40MGEe0080BDU900L0>c/k05QHF017AdL0ATI604E5A@1?Cdl0=cLg0080?3`l00l0G5aL
05YJFP1QHF40IVIV061PH00m?Cd0Fe]K07inOP1RHV80Cdm?069RHP2@T900GUiN09:BTP16ATH00P1N
GUh0BP0e=CD0JVYZ04];B`1;Bd/0A4A404=3@`0`<300@d=305EEE@0l?3`0CDe=055AD@1QHF408B4Q
03Xj>P10@400EEEE069RHP1[Jf/0<30`04a<C01TI6@0820P03hn?P15ADD0BDU902/[:`1eMGD0LW9b
055AD@17AdL08R8R02TY:@0d=3@0DE5A03De=@1:BTX0=SHf04m?C`1IFET0LG5a0410@01<C4`0;bl_
05UIF@1HF5P0@D51061PH00f=SH0B4Q804];B`18B4P0BDU906ATI013@d<0BDU9059BDP0n?Sh0G5aL
06][J`2[Zj/0PH6105YJFP1:BTX0Hf=S03Hf=P21PH40Lg=c0410@01HF5P0Cdm?05ADE01hN7P0DE5A
0P18B4P08@1>CTh0Fe]K055AD@1JFUX0Ogmo05YJFP1`L700<S8b05EEE@15ADD0DU9B02d];@1OGel0
Kfm_03De=@1YJFT0:BTY0492@P1KFe/0G5aL06i^KP1<C4`0KVi^06ATI019BDT0CDe=0410@01NGUh0
<c<c04Q8B00R8R80KVi^04];B`0205IFEP0805eMG@1:BTX0CTi>05]KF`1ZJVX0DE5A05aLG02VYZH@
ooooo`003_ooool0o`2RXZ80HF5Q07alO01<C4`0L71`06i^KP1HF5P0U9BD03dm?@1eMGD0ATI604E5
A@1JFUX09RHV04I6AP2KVi/0D51@04i>CP0m?Cd0<C4a07MgM`2KVi/0:b/[03`l?00m?Cd0RXZ:08^;
R`0[:b/05QHF04];B`1NGUh0UIFE0;6a/@0M7Ad0;Bd]04I6AP1MGEd0X:2P06i^KP0M7Ad0>c/k06][
J`1PH600Hf=S02PX:00/;2`0BDU907]kN`1VIVH0JVYZ020P800Y:BT0BTY:071`L01DE5@0MgMg02@T
900m?Cd0GEeM051@D00R8R80LG5a030`<013@d<0Fe]K02d];@0k>c/0>SXj03<c<`1JFUX0C4a<01dM
7@0Z:RX0>CTi03Ph>01:BTX0Q8B401lO7`0T92@0>c/k04i>CP1EEED0N7Qh01XJ6P0[:b/0EEEE04=3
@`1VIVH0YJFU024Q8@0/;2`0GUiN04=3@`1LG5`0EEEE02HV9P1SHf<0D51@03/k>`1QHF40;Rh^02LW
9`11@D40:b/[02TY:@1@D500;2`/02DU9@1_Kfl0Fe]K028R8P0n?Sh0?Cdm034a<@1NGUh0BDU903Hf
=P1@D500DE5A03Ph>01cLg<0IfMW03Hf=P0g=cL0IfMW06YZJP1gMgL0De=C03De=@14A4@0CTi>03/k
>`1jNWX08R8R034a<@1FEUH0>CTi04m?C`1VIVH08b<S02l_;`2<S8`0FEUI03dm?@0k>c/092@T05aL
G02QXJ40@D5105QHF01EEED0?clo07EeM@2_[jl0<30`03Hf=P1HF5P0@D51092@T02e]KD0?Cdm02<S
8`1/K6`0DE5A08j>SP1ZJVX09RHV05IFEP1nOWh0LG5a09NGU`0b<S80820P071`L02CTi<0MGEe096A
T@0`<30071`L06][J`2<S8`0?Cdm03/k>`0`<300@T9207UiN@10@400@d=302d];@13@d<0LW9b06a/
K00d=3@0?clo038b<P1YJFT0EeMG03De=@1FEUH0?3`l04i>CP1UIFD06QXJ04Y:BP1NGUh0?Cdm07mo
O`1_Kfl0<S8b04M7A`1BDU80NGUi0;6a/@0f=SH0@T9204Y:BP1=CDd0De=C028R8P0d=3@0DE5A05iN
GP22PX80;Rh^02l_;`1JFUX0Lg=c069RHP1GEeL0>SXj034a<@1LG5`0PH6106YZJP1EEED0=cLg03<c
<`1EEED0ZZZZ07QhN00o?cl0Fe]K030`<024Q8@0XZ:R055AD@0k>c/0KFe]05UIF@1fMWH0B4Q8020P
800j>SX0Jf][05IFEP005`1[Jf/0Ti>C03De=@0[:b/0CDe=05IFEP1LG5`0X:2P024Q8@0_;bl0?3`l
08^;R`19BDT0IFEU020P801:BTX0Cdm?05IFEP1<C4`0CTi>02TY:@0f=SH0De=C00kooooo0500Fe]K
06UYJ@1`L700GemO06=SH`1]KFd0D51@0451@@1gMgL0O7al06MWI`22PX80Ng]k06MWI`2GUiL0WinO
07inOP1JFUX0Lg=c05YJFP1HF5P0C4a<03dm?@1CDe<0C4a<06UYJ@13@d<0K6a/0451@@1FEUH0=cLg
06a/K01KFe/0UIFE06=SH`29RHT0WYjN07IfMP1PH600Cdm?079bLP18B4P0LG5a0451@@1ADE40<C4a
04M7A`16ATH0?3`l04M7A`0j>SX0>CTi05QHF00^;Rh0?clo05iNGP16ATH0EeMG055AD@1YJFT0G5aL
059BDP1FEUH0C4a<04m?C`1FEUH0Ph>307]kN`1>CTh0H61P065QH@1UIFD0IfMW05QHF00b<S80E5AD
055AD@1KFe/0Bd];034a<@80ADE501d0>CTi03Ph>00k>c/0ADE505=CD`0/;2`0A4A403@d=01=CDd0
?Cdm0410@01GEeL0De=C04A4A01BDU80@41003Ph>01@D500CDe=05MGE`1EEED0CDe=06i^KP1IFET0
EeMG051@D01WIfL0Ogmo09ZJVP02069RHP1;07moO`1WIfL0GEeM05IFEP17AdL0FUYJ04I6AP1gMgL0
G5aL06a/K01kNg/0GemO08:2PP2DU9@0TY:B09BDU011@D40CTi>04=3@`1<C4`0DU9B02/[:`1FEUH0
Jf][0451@@1CDe<0=3@d0451@@16ATH0H61P08F5Q@1FEUH0?Cdm04m?C`19BDT0BTY:03Lg=`1JFUX0
?clo0451@@1OGel0:2PX08V9R@16ATH0@41004U9B@0c<c<0GUiN03/k>`1JFUX0HV9R05QHF00c<c<0
@D5104];B`14A4@0De=C04a<C01QHF40:RXZ03hn?P15ADD0>SXj04=3@`1<C4`0ATI606EUI@1NGUh0
7QhN04];B`1CDe<0ADE501hN7P1GEeL0Bd];0080EUIF01d06QXJ02l_;`0U9BD0>c/k02l_;`0i>CT0
Cdm?05YJFP1kNg/0RXZ:0492@P1TI6@092@T030`<00o?cl0<30`04U9B@1VIVH0CDe=05IFEP1dM7@0
?3`l04Q8B00i>CT0DE5A03/k>`1^KVh0KFe]03hn?P02055AD@0704i>CP1BDU80S8b<071`L01dM7@0
C4a<04Y:BP0205YJFP0604e=C@1bLW80?clo05iNGP1TI6@0Hf=S0P1nOWh0:00m?Cd0CTi>04];B`1k
Ng/0?3`l061PH00a<C40D51@02TY:@2:RXX0C4a<03lo?`1mOGd0@D5105iNGP1eMGD0HF5Q06][J`17
AdL0YJFU06m_K`1<C4`0ADE5055AD@1IFET0@T9204];B`0];Bd0AdM701dM7@1QHF40>c/k075aL@1J
FUX0Bd];05IFEP1VIVH0Hf=S0861P@1aLG4@ooooo`003_ooool03@1aLG40Kfm_0;>c/`1@D500Cdm?
05QHF028R8P0`L7104U9B@1FEUH0F5QH07EeM@1TI6@00P0U9BD0101eMGD0IfMW07]kN`15ADD204Q8
B00F04U9B@0l?3`0RHV901XJ6P0i>CT0O7al06IVIP0a<C40B4Q803Ti>@0j>SX0OGem06IVIP0m?Cd0
GemO02PX:019BDT0Fe]K03`l?016ATH0IfMW03De=@80BDU90<<0:BTY03dm?@0e=CD0;bl_05=CD`0S
8b<06a/K03<c<`0b<S80>SXj07MgM`0h>3P0A4A404U9B@1`L700BDU90;Bd]00W9bL0820P03De=@18
B4P0Shn?07UiN@0=3@d0;bl_05=CD`1@D500R8R8061PH00J6QX0:RXZ05IFEP10@400FUYJ04Q8B00K
6a/0=SHf04m?C`0N7Qh0DU9B03De=@0Z:RX0=CDe06ATI00d=3@0Fe]K02PX:00f=SH0AdM706ATI00N
7Qh0;Rh^03Lg=`1NGUh0RHV90492@P0T92@0<S8b0451@@1FEUH0SXj>03@d=00Y:BT0@41004Q8B01>
CTh0KFe]03Hf=P0E5AD0De=C04Y:BP1;Bd/0MGEe02PX:00a<C40>c/k06m_K`1:BTX0Lg=c01lO7`0o
?cl0GemO069RHP0P8200F5QH0410@01YJFT0OWin0410@00R8R80ATI604m?C`1[Jf/0Ogmo02TY:@0J
6QX0Cdm?06e]K@2QXJ40HF5Q01lO7`1;Bd/0Cdm?05aLG02JVYX0De=C02DU9@0Y:BT0Jf][07inOP1?
Cdl0=3@d02XZ:P0b<S80HF5Q08^;R`0m?Cd0DE5A02l_;`0i>CT0Bd];03Xj>P0l?3`0=CDe03/k>`1k
Ng/0NWYj02d];@0b<S80B4Q8061PH02TY:@0?clo020P800o?cl0HF5Q06i^KP1ZJVX05ADE03`l?01>
CTh0C4a<09jNWP1?Cdl0BDU905ADE019BDT0GUiN09>CT`0a<C40Cdm?071`L00m?Cd0LG5a02XZ:P0k
>c/0G5aL02HV9P1dM7@0<S8b02/[:`1FEUH0K6a/0492@P19BDT08R8R05=CD`1jNWX0Lg=c05ADE01:
BTX0GemO0:j^[P1MGEd0:2PX03Ti>@0d=3@0QXJ608B4Q00d=3@0;Bd]04=3@`1CDe<0NWYj0451@@0K
6a/0=cLg04m?C`1dM7@0QhN7051@D00^;Rh0A4A405YJFP29RHT00P1PH6008@0O7al0<30`08b<S02L
W9`0IFEU09bLW012@T80OWin09:BTP19BDT0NGUi09jNWP0n?Sh0ATI606i^KP1<C4`0AdM706IVIP10
@400ATI605IFEP12@T80AdM705QHF00Q8B40=CDe06i^KP1EEED0=cLg05QHF00M7Ad08R8R08>3P`02
03Xj>P030451@Ooooooooooo00cooooo0<40DE5A061PH01?Cdl0Bd];05IFEP26QXH0Jf][055AD@1d
M7@0K6a/03Xj>P1MGEd0RXZ:079bLP2_[jl0JVYZ08B4Q01BDU80VIVI059BDP1CDe<0DE5A05=CD`1H
F5P0@D5107]kN`18B4P0MWIf03dm?@1LG5`0@41008Z:RP1ZJVX0TY:B08>3P`2CTi<0R8R807YjNP1G
EeL0B4Q807QhN01?Cdl0Q8B404];B`1WIfL0?clo05EEE@1BDU80<30`05MGE`0f=SH0<S8b06][J`1<
C4`0EUIF079bLP1NGUh0EEEE06IVIP1<C4`0@D5105]KF`0j>SX0@T92055AD@1KFe/0Ng]k0410@016
ATH0CDe=07EeM@0k>c/0ADE506a/K00d=3@0EUIF02`/;01TI6@0Bd];02<S8`0[:b/0@41002/[:`0b
<S80?Cdm03/k>`1>CTh0<C4a03`l?00U9BD0FUYJ02l_;`0n?Sh0:b/[05]KF`1=CDd0>c/k0492@P17
AdL0De=C0492@P17AdL0C4a<05YJFP1HF5P0B4Q805QHF01EEED0JFUY08N7Q`1mOGd0JVYZ07MgM`1^
KVh0RHV905]KF`1@D500B4Q8051@D01?Cdl0Jf][04Y:BP1XJ6P0Ogmo06][J`2FUYH0U9BD08f=S@2L
W9`0C4a<06i^KP1IFET0Jf][03Hf=P0/;2`0?clo06QXJ01=CDd0J6QX0410@00n?Sh0<C4a0492@P1>
CTh0I6AT05YJFP1IFET0GEeM069RHP18B4P0Hf=S0492@P0n?Sh0IfMW02@T901[Jf/0GUiN03hn?P1>
CTh0:BTY07]kN`0Y:BT0GemO03lo?`19BDT0<C4a03Ph>015ADD0<30`04Y:BP17AdL0EeMG02l_;`12
@T80=cLg04=3@`25QHD0S8b<03/k>`16ATH0?Shn01lO7`0l?3`0Ng]k05mOG`0Y:BT0Fe]K05UIF@0o
?cl0=SHf01hN7P1KFe/092@T0080<30`01T0@41002d];@0j>SX0DE5A05EEE@0f=SH0Bd];03/k>`1M
GEd0HF5Q0492@P1;Bd/0@D510492@P11@D40@T92038b<P10@4008R8R03`l?00f=SH0ADE50492@P0V
9RH0KVi^0080CTi>00`0?clo069RHP14A4@0Hf=S05IFEP15ADD0H61P06=SH`1CDe<0N7Qh0492@P1G
EeL205EEE@0R038b<P1:BTX0@T9205ADE00a<C40HV9R03Lg=`1]KFd0>SXj069RHP0f=SH0BDU9061P
H01@D500ZJVY06UYJ@1EEED0UYJF06MWI`1]KFd0?Shn04i>CP19BDT0De=C055AD@17AdL0ADE504];
B`1eMGD0<30`04];B`0i>CT0GemO06a/K080JVYZ00H0LW9b07YjNP20P800Fe]K05MGE`2IVIT@oooo
o`003_ooool0>@20P800C4a<05iNGP22PX80M7Ad03lo?`17AdL0P82008V9R@1CDe<0:2PX06IVIP1i
NGT0?clo02h^;P1@D500<S8b079bLP2WYjL0<S8b03dm?@15ADD0<c<c08N7Q`1NGUh092@T0492@P0h
>3P0IVIV07moO`0I6AT09BDU03Ti>@1:BTX0IVIV0;Zj^P0_;bl0F5QH05=CD`0k>c/0Ph>309nOW`0_
;bl0<S8b06][J`0j>SX0MGEe02@T900X:2P0EUIF06][J`0a<C40ATI601PH600V9RH0BDU9051@D002
04U9B@1F01`L700_;bl0F5QH04m?C`1IFET0?clo01LG5`1@D500VYZJ03Ti>@0j>SX07AdM02<S8`1Q
HF40QXJ601dM7@0J6QX0ATI6069RHP1lO7`0J6QX01<C4`0L71`0EeMG05EEE@2<S8`0HV9R01DE5@0P
8200J6QX04];B`1_Kfl09RHV02<S8`0m?Cd0De=C05mOG`1[Jf/03`l?03Ph>00W9bL0FUYJ03hn?P14
A4@06a/K04U9B@1YJFT0Kfm_0492@P14A4@0<30`05aLG01aLG40PX:202h^;P0g=cL0?Shn04M7A`2C
Ti<09RHV01lO7`0[:b/0AdM705IFEP28R8P07alO02XZ:P14A4@0E5AD05iNGP2<S8`07QhN03Lg=`1U
IFD0MWIf05aLG01cLg<08R8R04e=C@1bLW80IVIV03Hf=P1ZJVX0>3Ph06=SH`2RXZ820492@P1D06AT
I00^;Rh0N7Qh08>3P`0i>CT0ATI605=CD`19BDT0J6QX06=SH`0S8b<0>c/k02h^;P1JFUX0DE5A08Z:
RP0V9RH0E5AD04Y:BP0/;2`0KVi^055AD@0f=SH0<30`06ATI00i>CT0Jf][05eMG@0h>3P0BTY:09^K
V`1MGEd0Shn?03dm?@0U9BD0Hf=S02PX:015ADD0>c/k01LG5`1@D500L71`03Xj>P1TI6@07QhN03`l
?02EUID0KFe]03Ti>@0j>SX0<c<c0820P01FEUH0;bl_0492@P1KFe/0[:b/06YZJP0K6a/0?clo061P
H01gMgL0OWin02<S8`0b<S80CTi>05MGE`1XJ6P0EUIF028R8P1FEUH0N7Qh05=CD`1kNg/0<c<c051@
D023Ph<0I6AT079bLP1KFe/05aLG05UIF@1SHf<0<30`0P1FEUH04@0J6QX0I6AT06UYJ@1=CDd0PH61
07=cL`0S8b<0E5AD0820P00/;2`0@d=304a<C00e=CD0BTY:096AT@1WIfL0A4A40080;bl_01X0Bd];
08R8R017AdL0@D5103Xj>P10@400GUiN08j>SP0e=CD0;Bd]05IFEP0_;bl0KFe]0:6QX@0d=3@04a<C
04];B`15ADD0QhN708b<S00d=3@0>3Ph05eMG@16ATH0G5aL07=cL`kooooo01L0DU9B08^;R`1SHf<0
CTi>05IFEP1YJFT0C4a<03De=@25QHD0PX:20410@01?Cdl0IFEU096AT@1`L700JFUY06i^KP17AdL0
GUiN05eMG@1=CDd0@T9204i>CP02055AD@1D05EEE@1=CDd0ADE505QHF01QHF40D51@05mOG`1hN7P0
FUYJ04e=C@1MGEd0MWIf05ADE015ADD0BTY:04m?C`11@D40FEUI05MGE`1QHF40ADE505aLG01HF5P0
<30`05YJFP1BDU80CTi>07=cL`0b<S80A4A4059BDP1RHV80OGem06e]K@1?Cdl0EEEE07emO@1BDU80
Bd];08N7Q`2CTi<0Vi^K05MGE`1:BTX0Fe]K09FEU@19BDT0DU9B08>3P`1DE5@0D51@0410@01WIfL0
CTi>02XZ:P0e=CD0ATI603Lg=`0d=3@0DU9B04a<C01WIfL0:b/[0410@00/;2`0GUiN03<c<`15ADD0
@d=305QHF01=CDd0B4Q803Xj>P15ADD0D51@055AD@1PH600I6AT04A4A01WIfL0Bd];04e=C@1?Cdl0
EeMG0P1fMWH0I@1SHf<0QXJ605]KF`1RHV80CTi>04a<C015ADD0DU9B04M7A`1TI6@0Fe]K06i^KP2V
YZH0D51@065QH@1[Jf/0L71`09FEU@1[Jf/0NWYj03lo?`1FEUH0=SHf04E5A@17AdL0GUiN04e=C@1;
Bd/0?3`l04A4A00_;bl0B4Q807UiN@1QHF40?Shn04e=C@0[:b/0HV9R05iNGP1SHf<0@D5103dm?@1G
EeL0=CDe07]kN`1FEUH0H61P07QhN011@D40Ogmo0492@P1CDe<0>3Ph04m?C`0?3`l0?clo05MGE`0Y
:BT0E5AD04Y:BP1WIfL05ADE030`<00o?cl0@d=304U9B@1=CDd0>SXj04];B`1=CDd0@T92061PH01A
DE40DU9B03Lg=`20P800M7Ad04E5A@0n?Sh04a<C020P800Q8B40?Cdm03Ph>01GEeL0>CTi04Y:BP2C
Ti<0];Bd04A4A01>CTh0:BTY02l_;`0[:b/0<30`05IFEP12@T80DE5A05=CD`1BDU80?3`l0080A4A4
00l0EEEE03/k>`1YJFT0J6QX03Xj>P1aLG40Cdm?04i>CP0h>3P0LW9b0410@02XZ:P0Hf=S05=CD`1J
FUX00P1:BTX0;`1JFUX0?clo04a<C01MGEd0>CTi05iNGP1ZJVX0Kfm_08F5Q@1<C4`0EUIF01DE5@1B
DU80=CDe04m?C`0Z:RX0PX:206QXJ01?Cdl0PH6105MGE`19BDT0IfMW08Z:RP2[Zj/0GEeM06e]K@1H
F5P0Hf=S069RHP10@400?Cdm03Lg=`1;Bd/0=CDe04a<C00/;2`0H61P05UIF@1iNGT0BDU905]KF`1m
OGd0IVIV059BDP1nOWh0KFe]013ooooo000>ooooo`0O092@T01ADE40CTi>06ATI02NWYh0ADE503Lg
=`1lO7`0MGEe08N7Q`0a<C40@41007MgM`1LG5`0HF5Q07inOP0`<300E5AD09^KV`1GEeL0Lg=c04=3
@`0I6AT0BTY:07inOP1IFET0JVYZ02DU9@0P8200MGEe055AD@0205aLG00P01PH600_;bl0IfMW04Y:
BP1WIfL0:RXZ01LG5`0i>CT0MgMg061PH01>CTh0:RXZ03Xj>P1OGel0IfMW038b<P1=CDd0Cdm?05mO
G`1JFUX08R8R01LG5`0c<c<0?3`l08:2PP1ZJVX07AdM01PH600_;bl0EUIF079bLP1=CDd2020P803C
02LW9`17AdL0LW9b02<S8`0H61P0@D5106ATI015ADD0?Cdm04e=C@0K6a/0BDU905QHF00f=SH0De=C
04Y:BP0^;Rh0B4Q805ADE00F5QH0=3@d04a<C00^;Rh0D51@04A4A00G5aL0?3`l02l_;`1;Bd/0NWYj
02l_;`0/;2`0E5AD03<c<`16ATH0IVIV010@400M7Ad0?Shn05YJFP1IFET0P82004U9B@0K6a/0A4A4
09RHV01BDU80Zj^[03dm?@0T92@0Jf][05]KF`0l?3`0IVIV038b<P0V9RH0UiNG06MWI`0`<300GUiN
03Ph>00H61P0GUiN04=3@`0o?cl0EeMG02DU9@13@d<0GEeM02@T9018B4P0BDU903hn?P1<C4`0I6AT
038b<P13@d<0De=C04E5A@2KVi/0X:2P02h^;P10@400PH6105mOG`2ZZZX0[Jf]03Ti>@1?Cdl0NWYj
06ATI01oOgl0QHF5038b<P0];Bd0Shn?05UIF@1]KFd0>SXj02DU9@19BDT0Ogmo04Y:BP1kNg/0Bd];
02h^;P20P800O7al03Ph>00l?3`0;Bd]0492@P1iNGT0F5QH01dM7@15ADD0AdM7069RHP21PH409RHV
03Hf=P1ADE40J6QX0:b/[015ADD0:2PX06ATI026QXH0IFEU03<c<`0m?Cd0F5QH04m?C`0d=3@0EEEE
02DU9@2JVYX0[Jf]03<c<`1GEeL0>CTi03/k>`1XJ6P0:b/[059BDP0f=SH0>SXj05aLG01MGEd0DU9B
05eMG@0_;bl0BDU905YJFP10@400G5aL02/[:`11@D40JFUY08R8R00m?Cd0C4a<04=3@`1JFUX0MWIf
0:NWY`1CDe<0@T9203Ph>00`<300Vi^K0<71`@1NGUh0:BTY07AdM01QHF40NGUi09VIV@0f=SH0>c/k
06i^KP1lO7`0SHf=02h^;P0V9RH0GUiN071`L01OGel0EEEE04Q8B00H61P0ATI607AdM01@D500?Cdm
04E5A@0a<C40>c/k06IVIP2<S8`0<C4a0451@@0S8b<0ATI60:ZZZP22PX80=cLg04a<C00X:2P0IfMW
09VIV@0>ooooo`80KFe]03d0F5QH059BDP1bLW80LG5a059BDP16ATH0FEUI06a/K01NGUh0LW9b08:2
PP1bLW80UYJF08N7Q`1fMWH0Bd];08B4Q01FEUH0I6AT04i>CP1IFET0F5QH0410@01bLW80A4A406QX
J00o?cl0FUYJ03Hf=P1mOGd0FEUI08F5Q@1EEED0TY:B09>CT`1jNWX0GUiN03hn?P1KFe/0>CTi06YZ
JP0h>3P0DU9B0410@01;Bd/0ADE5024Q8@1?Cdl0?Shn03Ph>01EEED0<c<c03Xj>P1]KFd0GUiN0451
@@1?Cdl0H61P05mOG`1;Bd/0AdM700<0BTY:09H0Jf][06m_K`1?Cdl0KFe]07YjNP1mOGd0MgMg05QH
F00`<300E5AD055AD@1SHf<0?Shn03Ph>01EEED0ADE50451@@0d=3@0>c/k03dm?@1HF5P0;Rh^03`l
?00`<300>c/k02<S8`0_;bl0AdM70451@@0g=cL0ADE5030`<016ATH0E5AD059BDP1IFET0BDU906a/
K01TI6@0GemO06=SH`1@D500L71`0:>SX`2NWYh0EUIF06QXJ01cLg<0I6AT061PH01FEUH0?Cdm05aL
G012@T80LW9b059BDP1WIfL0LG5a04E5A@1WIfL0RXZ:09RHV02XZ:P0GemO061PH01CDe<0H61P03<c
<`0Z:RX0=3@d05IFEP19BDT0J6QX03Xj>P16ATH0@41004a<C01oOgl0IFEU04E5A@1IFET0>CTi05AD
E01=CDd0K6a/03Ph>01>CTh0H61P01HF5P2OWil0@41003lo?`1ADE40@d=308F5Q@0o?cl0H61P04a<
C01@D500:2PX0451@@1<C4`0>SXj061PH00o?cl0CDe=01`L700h>3P0?clo04i>CP1HF5P0IVIV02h^
;P1?Cdl0De=C02TY:@0o?cl0DE5A05IFEP0c<c<0CDe=04Y:BP15ADD0F5QH02DU9@0l?3`092@T03Ph
>00^;Rh0AdM704];B`10@400EEEE06ATI012@T80EUIF01PH600f=SH0ADE504=3@`1GEeL0F5QH05UI
F@1;Bd/0JFUY03Ph>00g=cL0<C4a04i>CP0h>3P205=CD`0Y02TY:@1DE5@0?3`l05MGE`0h>3P0KFe]
024Q8@1TI6@0C4a<03Xj>P1XJ6P0Cdm?03dm?@1WIfL0:b/[055AD@1=CDd0=CDe03@d=00k>c/0<c<c
055AD@0a<C40Q8B401PH601PH600=3@d04Q8B00j>SX0FUYJ05=CD`1nOWh0XZ:R06e]K@1eMGD0RXZ:
05mOG`1JFUX0De=C08F5Q@0c<c<00P1:BTX02P14A4@0?Shn03/k>`1@D5005QHF03@d=00M7Ad0GemO
04M7A`1HF5P204U9B@80EUIF00<0F5QH06][J`22PX804?ooool000kooooo0P26QXH0AP2^[Zh0BDU9
05IFEP1]KFd0N7Qh08n?S`17AdL0J6QX055AD@1JFUX0JVYZ03Lg=`13@d<0G5aL03Hf=P1YJFT0ATI6
04e=C@1MGEd0DU9B02HV9P0j>SX0@D510492@P1_Kfl08R8R01dM7@16ATH0EEEE07emO@1dM7@071`L
02<S8`1:BTX0BDU90:^[Z`0h>3P05ADE01lO7`17AdL0KFe]0861P@0d=3@0?3`l04];B`1ZJVX0?Cdm
079bLP0J6QX0>c/k05EEE@11@D40AdM706YZJP0U9BD0=CDe071`L00g=cL0Bd];05MGE`0[:b/0F5QH
06UYJ@0W9bL0EEEE04M7A`0X:2P0DU9B06a/K00X:2P203Xj>P1G03dm?@16ATH0NGUi01TI6@1@D500
;bl_04I6AP1ADE40IVIV01`L701;Bd/0B4Q8038b<P1LG5`0Lg=c024Q8@12@T80FUYJ03`l?01fMWH0
=CDe02XZ:P15ADD0;Bd]04I6AP1VIVH07QhN02/[:`1`L700D51@03/k>`1QHF409BDU0492@P1aLG40
OGem0410@01<C4`0:b/[03`l?01TI6@0@T92024Q8@12@T80BDU904I6AP22PX80C4a<01HF5P1BDU80
ADE505ADE02RXZ80@D5100h>3P17AdL0D51@06YZJP2:RXX06a/K02h^;P1@D500=CDe06][J`1YJFT0
:BTY038b<P1`L700Ogmo08^;R`1cLg<09RHV03De=@2;Rh/0ATI605mOG`0i>CT07alO04A4A01YJFT0
GemO04Q8B00b<S80<C4a07inOP1kNg/0?3`l0080?clo0180A4A409nOW`1aLG40:RXZ04=3@`1EEED0
O7al09bLW00m?Cd0;bl_05IFEP1?Cdl0LG5a09RHV00L71`0<30`06][J`1EEED206EUI@0701dM7@1L
G5`0P82004m?C`1ADE40>CTi04];B`0207]kN`1F05iNGP1XJ6P0=3@d04];B`1_Kfl0=3@d05aLG00o
?cl0Bd];065QH@0W9bL0Hf=S05mOG`1CDe<0R8R8079bLP0e=CD0ATI603`l?01bLW80GEeM01PH6017
AdL0CDe=079bLP2SXj<08B4Q03hn?P1kNg/0GUiN09bLW00o?cl0:BTY04];B`21PH40BTY:06e]K@0W
9bL0;bl_051@D01eMGD0Fe]K051@D00d=3@0:BTY04A4A01aLG40Lg=c04I6AP1ADE40=3@d04A4A01V
IVH0Lg=c06ATI02BTY80>CTi06YZJP2>SXh0<30`03@d=01^KVh0CTi>09ZJVP23Ph<0>3Ph024Q8@13
@d<0<30`05iNGP1XJ6P07AdM03/k>`13@d<0Kfm_05=CD`22PX806QXJ034a<@1BDU80BDU903hn?P1I
FET0=SHf0410@01=CDd>ooooo`1i04i>CP1GEeL0ADE504e=C@18B4P0TI6A07IfMP19BDT0MgMg07=c
L`19BDT0K6a/08^;R`22PX80/k>c06a/K027QhL0C4a<0:FUY@1TI6@0IVIV04Y:BP1cLg<0GUiN055A
D@1ZJVX0@d=3059BDP1:BTX0HV9R04U9B@28R8P0GemO07alO01mOGd0XJ6Q09NGU`22PX80FEUI03/k
>`1QHF40AdM707YjNP12@T80Jf][0451@@1BDU80ATI6028R8P1NGUh0C4a<0451@@21PH40=SHf04A4
A01gMgL0FUYJ059BDP1LG5`0ADE504=3@`1FEUH0?Shn0451@@1>CTh0GemO09>CT`17AdL0C4a<051@
D01/K6`0<c<c0492@P1QHF40>CTi05IFEP0i>CT0Kfm_04M7A`0o?cl0?3`l04I6AP0_;bl0@41003Xj
>P0n?Sh0DE5A0451@@13@d<0;2`/05eMG@0/;2`0ADE5030`<01LG5`0?clo02l_;`0h>3P0ATI605QH
F00/;2`0DU9B04Y:BP10@400DU9B03Xj>P11@D40J6QX05mOG`1cLg<0LW9b07emO@2<S8`0Jf][08>3
P`1XJ6P0IFEU04e=C@1UIFD0GUiN061PH00207MgM`1M09jNWP1ZJVX0MWIf07YjNP1bLW80QHF504I6
AP19BDT0;Bd]0492@P0a<C40:b/[05QHF01QHF40De=C07IfMP0j>SX0@D5105UIF@1TI6@0IfMW05=C
D`12@T80DU9B061PH01FEUH0BTY:05IFEP1@D500PX:207YjNP0@4100L71`03hn?P1XJ6P0KVi^03De
=@1[Jf/0:RXZ06=SH`1ADE40HV9R03@d=01DE5@0Fe]K04A4A01JFUX0?Cdm04e=C@0Y:BT0@T9202l_
;`14A4@0GEeM05aLG019BDT0Hf=S051@D00>3Ph0?Cdm075aL@1QHF40<30`07moO`1cLg<0@41004Y:
BP0R8R80@d=303@d=00n?Sh0ATI606m_K`0m?Cd0?clo05mOG`1/K6`0>3Ph06i^KP0i>CT0Hf=S06MW
I`0Z:RX0B4Q803De=@1<C4`0Bd];04a<C011@D40=CDe04I6AP1KFe/0@T920080Fe]K00X0;2`/06AT
I00_;bl0GUiN04A4A01iNGT0B4Q80;2`/01fMWH0DE5A0P1BDU804@1EEED0L71`0410@01jNWX0BDU9
03Hf=P0i>CT0De=C06ATI01bLW80=cLg05IFEP0S8b<0GemO03dm?@1GEeL0HV9R0080JFUY01L0H61P
08>3P`1UIFD0C4a<07MgM`1TI6@0IFEU03dm?@1UIFD0;bl_06IVIP1EEED0;Rh^03Hf=P1IFET0LG5a
020P800_;bl0<c<c06YZJP1jNWX0J6QX05EEE@02069RHP0405UIF@1;Bd/0J6QX08F5QA3ooooo000>
ooooo`3o09BDU01GEeL0Ogmo06UYJ@1hN7P0A4A405aLG02NWYh0MgMg06YZJP0i>CT0DU9B08:2PP15
ADD0Bd];06ATI00o?cl0OGem08^;R`0d=3@0Cdm?055AD@0k>c/0MgMg051@D011@D40LG5a05EEE@1=
CDd0GEeM02PX:01JFUX0J6QX03/k>`1[Jf/0M7Ad02/[:`1PH600FEUI03Ph>01_Kfl0HF5Q02/[:`19
BDT0Cdm?04e=C@1eMGD09RHV02`/;01NGUh0?3`l06][J`1]KFd09RHV03dm?@1VIVH0ATI606MWI`10
@400?3`l05YJFP1YJFT0:BTY05EEE@0e=CD0<S8b05IFEP1cLg<0=CDe04A4A019BDT0@D5106UYJ@1g
MgL06QXJ02LW9`1EEED0@41008V9R@1GEeL05aLG03Ti>@1>CTh0>3Ph08j>SP1MGEd04Q8B02l_;`1^
KVh0ATI605mOG`0X:2P0;2`/061PH01dM7@0AdM705EEE@0W9bL0@D51065QH@17AdL09BDU0451@@0[
:b/0GUiN08f=S@1>CTh0<c<c04];B`0k>c/0BTY:07]kN`1<C4`0:2PX04Q8B012@T80@41007inOP0i
>CT09bLW04M7A`1?Cdl0ATI607AdM00];Bd0;Rh^06IVIP1YJFT0BTY:079bLP0U9BD0?Shn0820P01=
CDd0ADE506=SH`0R8R80FUYJ07=cL`1=CDd0>SXj0820P00l?3`0Ng]k08R8R016ATH0BDU906e]K@19
BDT0RHV90861P@0e=CD0BDU905eMG@12@T80IfMW05=CD`0W9bL0=3@d04Y:BP11@D40Hf=S05IFEP0K
6a/0>CTi07IfMP17AdL0EeMG03lo?`0h>3P0De=C05ADE00n?Sh0NWYj051@D00h>3P0J6QX04Q8B01C
De<0QXJ602LW9`0/;2`0Ogmo03hn?P16ATH0AdM703/k>`1KFe/0J6QX03`l?01HF5P0<c<c059BDP2S
Xj<0Jf][04a<C010@400>c/k07YjNP1FEUH0<C4a04e=C@17AdL0Ogmo05mOG`0];Bd0GUiN055AD@1_
Kfl0Hf=S03Ti>@1SHf<0FUYJ03/k>`1lO7`0EUIF04e=C@1ZJVX0GUiN04Y:BP1fMWH0>SXj05]KF`1T
I6@0ATI607AdM01FEUH0;Bd]06m_K`1WIfL0@T9206][J`1/K6`09BDU069RHP1[Jf/0>c/k05MGE`1Y
JFT0?3`l04E5A@1SHf<0=CDe04A4A015ADD0@D5105QHF02LW9`0F5QH05]KF`1CDe<0Bd];05QHF023
Ph<0EUIF000G05ADE01_Kfl0?Cdm05QHF01gMgL0DU9B0492@P1NGUh0?clo04Q8B01bLW80;2`/02d]
;@1RHV80Hf=S065QH@1BDU808R8R03hn?P1XJ6P0=cLg05QHF01MGEd03_ooool0>01HF5P0KFe]05eM
G@1BDU80BTY:07moO`1XJ6P0E5AD09bLW02c/k<0EEEE06][J`2:RXX0UYJF09^KV`1TI6@0Ogmo069R
HP25QHD0De=C06][J`1@D500GemO05eMG@1NGUh0L71`04A4A01FEUH0EEEE06][J`12@T80Jf][05UI
F@1bLW80NGUi07inOP25QHD0Kfm_055AD@10@400F5QH04U9B@1PH600ATI605YJFP12@T80C4a<04=3
@`0V9RH0BTY:051@D018B4P0GUiN03hn?P18B4P0IfMW0P1NGUh0TP1VIVH0IFEU06MWI`1RHV80C4a<
04I6AP1LG5`0Fe]K08Z:RP1JFUX0?3`l069RHP1iNGT0De=C05eMG@1/K6`0CDe=05YJFP15ADD0N7Qh
059BDP0/;2`0>CTi04U9B@0i>CT0;2`/0492@P0o?cl0CDe=02DU9@14A4@0;2`/05UIF@0e=CD0ATI6
03Xj>P1GEeL0AdM703/k>`1>CTh0@d=3065QH@13@d<0GEeM05YJFP1;Bd/0M7Ad04Y:BP1PH600BDU9
055AD@1@D500QHF505EEE@1TI6@0NWYj07IfMP1VIVH0IFEU0492@P1KFe/0@T9205aLG01?Cdl0KVi^
08R8R012@T80IfMW07EeM@22PX80TY:B055AD@1RHV80=SHf075aL@0V9RH0=CDe05UIF@1ZJVX0@d=3
05ADE016ATH0Bd];03/k>`14A4@0S8b<06=SH`19BDT0E5AD04A4A01BDU80EeMG06=SH`0n?Sh0>CTi
05ADE00/;2`0RXZ:05EEE@0o?cl0EeMG03Hf=P23Ph<0;bl_04i>CP0l?3`0D51@02<S8`0i>CT0AdM7
02LW9`1CDe<0AdM705aLG00R8R80<C4a03/k>`0o?cl0EUIF05eMG@12@T80H61P05EEE@0[:b/0B4Q8
06=SH`1MGEd0<c<c06QXJ01CDe<0@d=305UIF@0S8b<0@T9203Ti>@17AdL0<S8b04i>CP10@400?Cdm
07AdM02HV9P0BDU907=cL`0[:b/0@d=30P0n?Sh0B01FEUH0CTi>04E5A@1<C4`0BDU903dm?@1<C4`0
>c/k051@D012@T80FEUI05=CD`0a<C40M7Ad04=3@`1<C4`0=SHf05mOG`0b<S80KFe]06m_K`1IFET0
EUIF05UIF@1UIFD0LW9b04a<C01LG5`0EEEE04];B`17AdL0Bd];03Xj>P1FEUH0?clo06m_K`0j>SX0
Fe]K04E5A@1LG5`0@410061PH01YJFT0Fe]K08Z:RP1TI6@0KFe]08n?S`1lO7`0J6QX059BDP1dM7@0
@T9205UIF@1FEUH0Bd];04E5A@14A4@0BTY:03`l?015ADD0>SXj069RHP1JFUX0Hf=S069RHP1TI6@0
Bd];05UIF@1/K6`0Lg=c08N7Qa3ooooo000>ooooo`3o065QH@1EEED0LG5a04Q8B01ZJVX0CTi>04E5
A@1mOGd0@41008Z:RP0e=CD0=cLg07EeM@0c<c<0J6QX0:ZZZP10@400E5AD05mOG`1JFUX0TI6A07Ad
M00V9RH0?Cdm06e]K@1cLg<0UIFE01lO7`0i>CT0EUIF04U9B@2OWil0UiNG014A4@0e=CD0I6AT04i>
CP2<S8`0F5QH01dM7@19BDT0QhN704A4A01MGEd09bLW030`<01GEeL0LG5a04M7A`1@D500<30`03Hf
=P1UIFD0A4A402DU9@1PH600>3Ph059BDP1QHF407alO030`<01DE5@0@T92096AT@19BDT06a/K03Xj
>P0c<c<0CTi>08N7Q`0e=CD06QXJ02d];@1YJFT0Cdm?05]KF`1LG5`06a/K051@D01nOWh0>SXj05UI
F@1ADE408b<S059BDP1TI6@09bLW04I6AP1mOGd0:b/[05IFEP1DE5@0>CTi04Q8B00m?Cd0CDe=0:f]
[@13@d<0@D5104m?C`0l?3`0=SHf06m_K`0H61P0;Rh^04];B`0i>CT0FUYJ0:j^[P1;Bd/0:RXZ04M7
A`1;Bd/0FUYJ0:FUY@16ATH0=cLg03/k>`1oOgl0F5QH08n?S`1PH6006QXJ05IFEP1SHf<0=cLg07Ui
N@0P82008b<S065QH@1>CTh0@T9206YZJP0c<c<0=CDe09>CT`0_;bl0?3`l059BDP0e=CD0>CTi08F5
Q@1>CTh0DE5A05mOG`0h>3P0Hf=S0:^[Z`0/;2`0@T9207=cL`1:BTX0Ogmo09fMW@0U9BD0@D5108:2
PP1?Cdl0MGEe05mOG`0C4a<0Bd];07IfMP13@d<0WinO03Xj>P0L71`0G5aL05YJFP17AdL0Lg=c02l_
;`0i>CT0UIFE09>CT`1@D500Cdm?02XZ:P0b<S80W9bL05IFEP0O7al0>SXj02TY:@1cLg<0KFe]03Xj
>P0l?3`0;Rh^07UiN@2n_[h0=SHf02d];@19BDT0E5AD07emO@1ADE4071`L04Y:BP1BDU80E5AD061P
H00K6a/0@d=305UIF@17AdL0VIVI02h^;P1DE5@0J6QX04e=C@1TI6@0ATI602TY:@1DE5@0JVYZ07al
O01dM7@0>3Ph05iNGP1MGEd0E5AD07AdM00d=3@0AdM707QhN01jNWX0<S8b051@D00Q8B40@d=305eM
G@2>SXh0GEeM04Y:BP0a<C40E5AD065QH@1dM7@0Cdm?02@T901GEeL0AdM705mOG`27QhL0=SHf04Y:
BP1ADE40F5QH09RHV01`L7009RHV04A4A01MGEd0QHF5000G07moO`1hN7P08B4Q02/[:`14A4@0EUIF
06QXJ00k>c/071`L02l_;`1>CTh0NWYj03@d=01VIVH08b<S04=3@`1ZJVX0@41002h^;P14A4@0:RXZ
03Ph>01fMWH03_ooool0P@25QHD0CDe=04E5A@1]KFd0TI6A08^;R`1hN7P0I6AT05aLG01bLW80CTi>
05mOG`1[Jf/0De=C0;>c/`28R8P0TY:B04a<C01eMGD0AdM707QhN01JFUX0H61P05eMG@1PH600KVi^
04e=C@1fMWH0HV9R07MgM`1:BTX0WIfM05IFEP2CTi<0Ph>30:6QX@2o_kl0RHV9079bLP1<C4`0QhN7
059BDP20P800EUIF07UiN@1KFe/0G5aL04a<C00[:b/0Hf=S055AD@0`<300Jf][03hn?P17AdL0GEeM
04I6AP1PH600GEeM04];B`0n?Sh0Bd];03lo?`0`<300ADE504e=C@1dM7@0B4Q804a<C019BDT0EEEE
03Lg=`0m?Cd0C4a<04E5A@1<C4`0?Cdm05MGE`1QHF40>CTi04Y:BP1GEeL0>SXj03dm?@0f=SH0>3Ph
04Y:BP0o?cl0C4a<03De=@1ADE40<c<c03Xj>P0m?Cd0Bd];04=3@`0[:b/0AdM704Y:BP1@D500Cdm?
04A4A018B4P0J6QX065QH@17AdL0GEeM05aLG00n?Sh0KVi^09jNWP1ZJVX0G5aL08N7Q`2>SXh0Kfm_
06QXJ01;Bd/0E5AD03`l?01?Cdl0Bd];07=cL`2HV9P0DE5A08N7Q`2CTi<0NWYj09nOW`0204e=C@0:
01/K6`0n?Sh06a/K02/[:`1ADE40EeMG04A4A01EEED0?clo03dm?@80EUIF08L0DE5A04i>CP0h>3P0
CDe=02l_;`1<C4`0@41004M7A`0d=3@0?clo04Y:BP0M7Ad0GEeM03hn?P1KFe/0HV9R02`/;01OGel0
@d=3065QH@1=CDd0G5aL024Q8@10@400BDU903`l?01DE5@0Bd];05QHF00f=SH0Bd];038b<P0i>CT0
Hf=S05IFEP0U9BD0>SXj04Q8B00@4100:b/[0410@01=CDd092@T0:6QX@1NGUh0BDU905EEE@0L71`0
FEUI034a<@0o?cl0?3`l05YJFP10@400?Cdm06a/K01mOGd0<S8b04m?C`0M7Ad0LW9b05=CD`0Y:BT0
B4Q8061PH01ADE40EeMG061PH016ATH0DU9B04=3@`1JFUX0?Shn05eMG@17AdL0=3@d06][J`0];Bd0
?3`l04E5A@29RHT0B4Q806UYJ@1JFUX0Hf=S05ADE01^KVh0EeMG071`L013@d<0LG5a079bLP1]KFd0
=cLg0451@@1;Bd/0Fe]K04Y:BP25QHD0A4A405=CD`0e=CD0DE5A051@D021PH40GemO05eMG@27QhL0
EEEE04I6AP1KFe/0GUiN038b<P13@d<0E5AD02`/;02MWId0KVi^0451@@15ADD0DE5A05aLG01JFUX0
EUIF02l_;`1JFUX0F5QH055AD@1gMgL0Jf][04I6AP1MGEd0Kfm_08B4Q02c/k<04?ooool000kooooo
0;40IFEU06][J`38b<P0DE5A06i^KP1`L700PX:209:BTP1GEeL0LW9b065QH@2ATI40][Jf02XZ:P16
ATH0Cdm?04a<C02b/[80<S8b02l_;`0a<C40LW9b04m?C`1`L7008B4Q04a<C02FUYH0KVi^0451@@1M
GEd092@T0820P02OWil0?3`l03<c<`0f=SH0<S8b08J6QP1@D5006ATI02TY:@0k>c/0ATI60:FUY@0`
<300=CDe04];B`0l?3`0G5aL07IfMP0R8R80=CDe02`/;01YJFT0JVYZ08b<S00Z:RX0ATI604Q8B00i
>CT0Jf][05YJFP0c<c<0?Shn0410@01<C4`0PH6106m_K`0B4Q80Bd];04Y:BP1ADE40FUYJ03`l?00P
8200ATI605EEE@1:BTX0HV9R02`/;00h>3P0ADE5051@D00Y:BT0@D5102l_;`0S8b<0>3Ph08Z:RP0[
:b/0>SXj03@d=01EEED0P82003Hf=P0K6a/0B4Q803lo?`1LG5`0PX:201XJ6P1=CDd0HF5Q034a<@1`
L700VYZJ01TI6@0P8200IFEU07YjNP19BDT0MGEe02<S8`0`<300Hf=S05EEE@1:BTX0EEEE02XZ:P0T
92@0EEEE07alO00T92@0>SXj03<c<`0l?3`0P82004Y:BP0G5aL0ATI60492@P1CDe<0NWYj034a<@0Q
8B40JVYZ04e=C@1lO7`0NGUi01PH6016ATH0EEEE04m?C`2PX:00@D51030`<01:BTX0Cdm?06QXJ028
R8P0@D5102XZ:P0c<c<0Hf=S06IVIP1WIfL0?clo03/k>`1=CDd0K6a/04Y:BP0i>CT0C4a<02XZ:P1T
I6@0P820034a<@0g=cL0ATI6061PH02KVi/0>3Ph020P8012@T80AdM705iNGP2VYZH00P0m?Cd205=C
D`1C065QH@1;Bd/07AdM02d];@21PH40HF5Q07YjNP12@T80>3Ph055AD@1[Jf/0De=C05mOG`0Y:BT0
BTY:08>3P`12@T80FUYJ02TY:@0m?Cd0B4Q80451@@1YJFT0=CDe03dm?@22PX80De=C03<c<`0a<C40
BDU90820P01/K6`06a/K04E5A@0n?Sh0JFUY08n?S`0X:2P0<30`05MGE`1?Cdl0YjNW05mOG`0[:b/0
Fe]K05IFEP0l?3`0U9BD034a<@0U9BD0EeMG08b<S01XJ6P0IVIV02HV9P0];Bd0A4A409VIV@2j^[X0
IVIV071`L00/;2`0G5aL08j>SP1>CTh0JFUY05iNGP0`<300EEEE07=cL`14A4@0G5aL07IfMP0V9RH0
HV9R06QXJ00f=SH0=cLg05QHF00R8R80EUIF08V9R@0V9RH00P0e=CD0301HF5P0DU9B06QXJ00O7al0
=SHf03`l?01HF5P0DE5A06][J`0Z:RX0>c/k04a<C0kooooo00<0BTY:05]KF`1OGel00P0k>c/0QP1S
Hf<0FUYJ055AD@1iNGT0TY:B05iNGP1^KVh0T92@07IfMP2QXJ40EUIF09NGU`1[Jf/0RHV904A4A01W
IfL0EUIF06a/K01PH600L71`07emO@12@T80FUYJ055AD@1RHV80B4Q806ATI01oOgl0Lg=c06=SH`1j
NWX0T92@06UYJ@1<C4`0EEEE05YJFP17AdL0FEUI04i>CP1TI6@0A4A406YZJP19BDT0:b/[04Q8B01A
DE40?Cdm06ATI019BDT0DU9B06MWI`1NGUh0IfMW06UYJ@1RHV80GUiN071`L019BDT0E5AD05=CD`1J
FUX0[jn_05mOG`1NGUh0IfMW09RHV014A4@0C4a<06YZJP18B4P0EUIF04A4A01NGUh0?clo02d];@0Y
:BT0?3`l038b<P0Y:BT0C4a<04A4A01=CDd09bLW05MGE`0^;Rh0EUIF03Ti>@0l?3`0>c/k04i>CP18
B4P0>CTi04m?C`1CDe<0GEeM038b<P1>CTh0@T9203Xj>P1QHF40?Shn04E5A@1:BTX0?3`l04Q8B01R
HV80P82007IfMP1PH600G5aL051@D014A4@0?Cdm05eMG@1@D500EeMG04M7A`1lO7`0Ti>C06EUI@1V
IVH0MGEe06UYJ@2>SXh0P82007MgM`0a<C40E5AD02h^;P0[:b/0IfMW05MGE`1NGUh0J6QX0P0l?3`0
R@1;Bd/0A4A4065QH@1NGUh0De=C069RHP10@400GUiN04M7A`1OGel0:BTY03<c<`1/K6`08B4Q07Yj
NP1>CTh0;Rh^05=CD`0k>c/0IFEU0451@@0k>c/0;bl_03`l?00Y:BT0D51@04i>CP0g=cL0E5AD05EE
E@1VIVH09bLW02l_;`12@T80AdM7069RHP1dM7@0>c/k04i>CP1GEeL08R8R04Q8B01<C4`0ADE5024Q
8@1>CTh0;bl_07EeM@20P8007alO038b<P0n?Sh0NWYj04M7A`1TI6@0GEeM03hn?P1hN7P0Ti>C03<c
<`24Q8@08R8R03Lg=`13@d<0ATI605eMG@15ADD0BDU905EEE@0f=SH0>3Ph05EEE@0c<c<0?clo04Y:
BP1BDU80@T9203<c<`2FUYH0GEeM061PH014A4@0L71`01lO7`1WIfL0K6a/08j>SP1XJ6P0GEeM065Q
H@27QhL0FEUI08>3P`1EEED0CDe=04A4A00e=CD0>3Ph051@D00V9RH0JFUY059BDP1nOWh0AdM7079b
LP12@T80Cdm?06YZJP1KFe/0SHf=059BDP1gMgL0NWYj08R8R01XJ6P0Jf][09BDU00d=3@0>c/k03Xj
>P1SHf<0EUIF05ADE01JFUX0AdM705EEE@0l?3`0EUIF05aLG01QHF40FUYJ05IFEP1:BTX0EEEE051@
D01SHf<0QXJ6013ooooo000>ooooo`0L0820P01BDU80IFEU06ATI01KFe/0BDU904a<C02?Shl0H61P
05iNGP0d=3@0?Shn08>3P`1FEUH0ADE5069RHP1>CTh0Rh^;07=cL`0U9BD0?Shn051@D01IFET0NGUi
028R8P0b<S80?Shn069RHP80JFUY0?P0:RXZ03/k>`1VIVH0KVi^07=cL`1DE5@05QHF04m?C`1hN7P0
@d=307]kN`12@T807QhN04Q8B01bLW80GUiN07alO00];Bd0>3Ph04a<C014A4@0L71`05QHF00F5QH0
@T9206e]K@0j>SX0FEUI04I6AP0Q8B40FUYJ09fMW@0c<c<0Fe]K034a<@0f=SH0Lg=c09JFUP0M7Ad0
?Shn059BDP1HF5P0SHf=05QHF00B4Q808B4Q05]KF`1HF5P0_;bl0492@P0K6a/0=CDe07IfMP1]KFd0
ZZZZ02TY:@0J6QX0=cLg061PH015ADD0A4A400`<300c<c<0@D5105IFEP10@400BTY:02TY:@0g=cL0
DU9B04Y:BP0];Bd0ATI602TY:@1DE5@0SXj>0492@P0W9bL0<c<c04U9B@1NGUh0U9BD02d];@0M7Ad0
>3Ph04Q8B01hN7P0ZJVY01dM7@0D51@0BDU9079bLP1SHf<0Kfm_01XJ6P1=CDd0GEeM08b<S00n?Sh0
T92@030`<01GEeL0QXJ6061PH00X:2P0EUIF03`l?01bLW80][Jf055AD@17AdL0QhN703`l?01[Jf/0
VIVI03lo?`17AdL0EeMG03Xj>P2LW9`0Rh^;03<c<`16ATH0Cdm?03Ph>01jNWX0G5aL028R8P16ATH0
GEeM04E5A@2?Shl0GEeM02@T901BDU80IVIV06ATI027QhL0EEEE03Lg=`1KFe/0L71`05eMG@2QXJ40
ATI602/[:`1KFe/0EeMG05EEE@23Ph<0?clo02TY:@1gMgL0:b/[06UYJ@0m?Cd0;Bd]05iNGP1MGEd0
A4A403Xj>P1;Bd/0GemO09FEU@1?Cdl0>c/k04=3@`1EEED0Shn?03Lg=`0c<c<0E5AD06e]K@21PH40
EUIF02TY:@1aLG40GemO07MgM`0^;Rh0:RXZ06MWI`1UIFD0Cdm?08R8R01:BTX0HF5Q09jNWP1bLW80
?Shn071`L00Y:BT0KFe]0861P@17AdL0Ph>303`l?00f=SH0G5aL08^;R`1BDU80Cdm?04I6AP0U9BD0
IFEU06i^KP0`<300DE5A03hn?P0^;Rh0<c<c07]kN`0V9RH0=3@d03`l?00i>CT0L71`09>CT`0^;Rh0
CTi>04a<C01LG5`0PH610<O7a`0i>CT0?Cdm07alO023Ph<0BDU906m_K`0/;2`0>c/k07emO@1CDe<0
P820071`L00L71`0;bl_069RHP0e=CD0IfMW04];B`0P8200<S8b06EUI@0l?3`0>SXj04Q8B0kooooo
06D0F5QH06ATI01]KFd0CDe=06=SH`2EUID0Jf][07IfMP2YZJT0XZ:R04U9B@1:BTX0V9RH0<30`032
`/80Ogmo0861P@19BDT0RHV9059BDP2;Rh/0De=C07=cL`1XJ6P0DU9B08J6QP13@d<0EEEE03lo?`20
P800@T9209bLW01dM7@0Y:BT08f=S@26QXH0`<3008j>SP1]KFd0EEEE09JFUP15ADD0O7al04Q8B01f
MWH0=SHf06e]K@1QHF40Cdm?059BDP1XJ6P0BTY:07inOP0c<c<0@T9207AdM01OGel0EUIF051@D01^
KVh0Fe]K05UIF@1DE5@0IfMW06a/K01]KFd0PH6106QXJ01eMGD0NGUi07MgM`18B4P0Fe]K06YZJP0m
?Cd0H61P03De=@1PH600ADE503Hf=P17AdL0Cdm?03Ti>@0g=cL0>3Ph03/k>`1IFET0<c<c04Q8B00g
=cL0EEEE03Ti>@14A4@0@T92059BDP0m?Cd0?clo03Lg=`0l?3`0CTi>03Ph>00205IFEP0]04=3@`1[
Jf/0CDe=06=SH`1;Bd/0Fe]K06QXJ02VYZH0GemO061PH02DU9@0KFe]069RHP1JFUX0?clo05iNGP0n
?Sh0Kfm_06QXJ028R8P0UIFE05MGE`20P800RHV908N7Q`2KVi/0IfMW06UYJ@0i>CT0JFUY03lo?`0e
=CD0F5QH04a<C00n?Sh0D51@055AD@1CDe<0Fe]K05mOG`2KVi/0MGEe03`l?01:BTX0A4A40080IfMW
01L0IFEU03@d=00a<C40LW9b01TI6@1cLg<0EUIF03De=@1XJ6P0?Shn07UiN@0Z:RX0C4a<028R8P17
AdL0:RXZ03Hf=P0n?Sh0:RXZ059BDP0g=cL0CTi>01`L700302`/;00704M7A`1EEED0?clo05YJFP1O
Gel0=cLg03lo?`0205aLG01M02h^;P1NGUh0AdM703Ti>@1CDe<092@T04M7A`0m?Cd0@d=30451@@18
B4P0@41004A4A01@D500IfMW03Lg=`1CDe<09bLW03<c<`1:BTX0?Cdm04e=C@11@D40DE5A05eMG@1:
BTX0@D5103`l?00Y:BT0Bd];03Ti>@1=CDd0AdM703Ph>01RHV80ATI604a<C014A4@0/K6a03Lg=`29
RHT0OWin07QhN01DE5@0GUiN04=3@`1VIVH0?3`l03Xj>P1NGUh0E5AD07MgM`1fMWH0@T9207AdM00a
<C40P82004E5A@1MGEd0>SXj0451@@0l?3`0M7Ad055AD@1@D500NWYj06UYJ@1ADE40CDe=069RHP0j
>SX0D51@05IFEP10@400BTY:03`l?00k>c/0?Cdm03Xj>P19BDT0:b/[03dm?@0];Bd0EEEE05QHF01F
EUH0IVIV04m?C`1KFe/0JFUY05=CD`24Q8@0S8b<013ooooo000>ooooo`1o0;2`/01?Cdl0Fe]K06m_
K`1kNg/0BDU904a<C01hN7P0UYJF09BDU00_;bl0@T9205iNGP1cLg<0IfMW09BDU00_;bl0DU9B08Z:
RP1?Cdl0U9BD0451@@0b<S80NWYj03Ph>01iNGT0GemO02PX:01CDe<0QXJ602/[:`26QXH0?3`l02<S
8`1^KVh0ZZZZ0451@@0e=CD0;Bd]03<c<`29RHT0Vi^K038b<P0j>SX0<c<c04a<C01/K6`0=CDe02DU
9@11@D40De=C096AT@1RHV806a/K01`L700l?3`0CTi>0:FUY@14A4@0=SHf02@T9019BDT0GUiN09ZJ
VP0h>3P09BDU02/[:`1MGEd0Cdm?06YZJP0H61P0>3Ph04M7A`1BDU80<30`03Ph>00m?Cd09RHV04a<
C01YJFT09BDU03dm?@0b<S80CTi>05mOG`19BDT04a<C03Lg=`0n?Sh0AdM708:2PP0Y:BT06QXJ03Xj
>P1LG5`0LG5a079bLP0[:b/08B4Q04=3@`10@400CTi>06UYJ@0V9RH0>c/k07alO02EUID0IFEU09:B
TP0a<C40F5QH05MGE`1DE5@0;2`/05=CD`0e=CD0>c/k05EEE@1LG5`0<C4a08:2PP0^;Rh0;2`/079b
LP0V9RH071`L04];B`0202h^;P1i09:BTP0S8b<0:BTY05MGE`16ATH0EeMG08:2PP0K6a/0@D510820
P01ADE40LG5a0<o?c`13@d<0CDe=06a/K01;Bd/0Ti>C09^KV`0T92@0J6QX07alO00n?Sh0M7Ad06][
J`0`<300ATI607MgM`0j>SX0EeMG03Hf=P0E5AD0?clo08Z:RP0/;2`0GEeM02TY:@0X:2P0EUIF061P
H00i>CT0EeMG03Ph>019BDT0RXZ:069RHP0_;bl0?3`l05UIF@1hN7P0WinO030`<00];Bd0@41004=3
@`2IVIT0EeMG02h^;P1<C4`0EEEE06=SH`1jNWX09BDU04Y:BP1^KVh0M7Ad06EUI@0F5QH0ADE507Mg
M`1QHF40Cdm?04Y:BP0`<300Q8B406IVIP1>CTh0L71`02h^;P1GEeL0Kfm_02/[:`17AdL0<c<c03Hf
=P1KFe/0H61P04A4A01:BTX0?Cdm06EUI@1[Jf/0>CTi04M7A`0m?Cd0H61P0<O7a`1SHf<07alO03`l
?018B4P0GUiN0;Jf]P1oOgl09bLW03lo?`1KFe/0A4A407QhN01JFUX06a/K03lo?`1JFUX0@d=307If
MP1OGel09BDU05UIF@2HV9P0JVYZ08j>SP0202HV9P0J06i^KP1[Jf/0IVIV075aL@1ZJVX07alO05eM
G@28R8P0EEEE04U9B@0`<3006ATI04m?C`1aLG40:BTY028R8P16ATH0>3Ph07EeM@1cLg<09BDU02HV
9P0i>CT0IVIV07alO01cLg<>ooooo`0a05ADE01TI6@0IfMW059BDP1HF5P0X:2P07moO`1eMGD0Zj^[
0;Fe]@1aLG40Ogmo07MgM`1WIfL0KFe]08B4Q01nOWh0D51@06a/K015ADD0H61P04Y:BP1`L700GUiN
05ADE01ZJVX0EeMG08N7Q`1[Jf/0QHF504a<C02HV9P0OWin08Z:RP1oOgl0/K6a0<71`@2HV9P0KVi^
05EEE@2EUID0EUIF08>3P`1DE5@0OWin04m?C`1gMgL0Fe]K03lo?`0205MGE`2L030`<01eMGD0;Bd]
03Xj>P1ZJVX0CTi>04Q8B01NGUh0HF5Q04];B`1EEED0@D51059BDP18B4P0DE5A09:BTP1LG5`0HV9R
05EEE@1QHF40C4a<04U9B@1WIfL0@41006EUI@12@T80Kfm_04Q8B00Z:RX0?3`l04=3@`0h>3P0?Cdm
04=3@`0/;2`0De=C04Q8B01@D500:RXZ069RHP0h>3P0@d=304E5A@1[Jf/0AdM705ADE01MGEd0?clo
05EEE@0n?Sh0C4a<04I6AP1<C4`0De=C0410@01=CDd0HF5Q06=SH`29RHT0ZJVY07MgM`1^KVh0^KVi
08f=S@1JFUX0H61P04Q8B01DE5@0?3`l069RHP18B4P0Ogmo09nOW`20P800YJFU08N7Q`1=CDd0FEUI
061PH01YJFT0ADE506i^KP17AdL0<S8b05=CD`1]KFd0Bd];04I6AP0e=CD0<c<c04i>CP1NGUh0QHF5
07=cL`1EEED0Hf=S08B4Q01RHV80?Cdm06ATI00j>SX0:BTY06i^KP0/;2`0P82005=CD`14A4@0F5QH
03/k>`1eMGD0<30`04A4A00[:b/0FUYJ028R8P0i>CT0AdM702h^;P1TI6@0>c/k059BDP0L71`0>c/k
024Q8@0_;bl0EEEE06ATI00Z:RX0A4A404e=C@0I6AT0=cLg03`l?01ADE40<c<c069RHP0h>3P0HV9R
06IVIP0N7Qh0Fe]K03Xj>P1;Bd/0>CTi07QhN019BDT0FEUI06MWI`28R8P0ATI605aLG00c<c<0EeMG
06][J`0_;bl0C4a<0P1FEUH09`1OGel0G5aL04A4A011@D40<S8b04E5A@0m?Cd0JFUY0451@@0e=CD0
Jf][02XZ:P17AdL0?Cdm06e]K@0n?Sh0G5aL061PH01`L700E5AD06=SH`1YJFT0PH6104M7A`1BDU80
Bd];051@D010@400>c/k0451@@14A4@0?clo08:2PP1:BTX0JVYZ02/[:`1PH600;2`/059BDP0205IF
EP0L06UYJ@1HF5P0LW9b05ADE01PH600;bl_051@D01SHf<0<30`051@D00g=cL0@T9203hn?P10@400
BTY:03dm?@1@D500?clo069RHP1TI6@0EUIF08B4Q01RHV80Cdm?075aL@1:BTX0Jf][09VIVA3ooooo
000>ooooo`0;07=cL`1MGEd0WYjN04U9B@1GEeL0HV9R071`L01ZJVX0?3`l069RHP1UIFD00P1NGUh0
N00i>CT0ATI607EeM@1?Cdl0G5aL03Hf=P1=CDd0KFe]07moO`0[:b/0@d=305EEE@20P800L71`01DE
5@0Y:BT0Cdm?04Q8B03:b/X0FUYJ01dM7@0];Bd0DE5A069RHP1^KVh0410@02h^;P0f=SH0SXj>06a/
K01ZJVX05QHF04a<C01_Kfl0@T9202d];@1DE5@03Ph>065QH@28R8P0E5AD059BDP1PH600;bl_079b
LP1jNWX0>c/k05aLG01:BTX0;Rh^04m?C`13@d<0<30`04E5A@0a<C40;Bd]04M7A`0g=cL09BDU0451
@@0k>c/0C4a<06YZJP1<C4`08B4Q03Ph>01JFUX0HF5Q08:2PP1:BTX0<C4a04=3@`1UIFD0=SHf09BD
U013@d<0:b/[059BDP1/K6`0F5QH07emO@0F5QH0=SHf05eMG@1HF5P0FEUI061PH00K6a/08R8R05iN
GP0];Bd0=cLg05EEE@0`<300H61P07YjNP0i>CT0:RXZ04e=C@0L71`0Bd];09BDU00];Bd0?Cdm04e=
C@1EEED0Bd];0<;2`P1;Bd/0;2`/071`L011@D40ATI606YZJP0Q8B40820P06IVIP1TI6@0Bd];07Yj
NP80<30`08l0P82003/k>`16ATH0@41001TI6@0U9BD0IFEU04=3@`1VIVH0?clo03<c<`0l?3`0VIVI
05eMG@1ZJVX0ADE503@d=01XJ6P0Ogmo02LW9`14A4@0Cdm?06m_K`26QXH0E5AD01`L700k>c/0BTY:
05]KF`2c/k<0CDe=01`L701GEeL0D51@06EUI@2a/K409BDU028R8P1bLW80HV9R04a<C011@D407AdM
0492@P2MWId0GemO05EEE@14A4@08b<S05]KF`1gMgL0@41004M7A`18B4P0I6AT08b<S00f=SH0?clo
06IVIP0h>3P0U9BD05QHF012@T80H61P02`/;01?Cdl0OGem01lO7`1GEeL0GUiN055AD@2ZZZX0@410
0451@@1]KFd0JVYZ07inOP1;Bd/08B4Q04I6AP1=CDd0JFUY07AdM00N7Qh0=SHf06IVIP1UIFD0T92@
0451@@0`<300?Cdm07alO00l?3`0DU9B024Q8@0Z:RX0<C4a06MWI`1CDe<0BTY:01DE5@0`<300DE5A
06i^KP12@T80>SXj04I6AP0d=3@0JFUY09>CT`0/;2`0@T9205QHF01;Bd/0U9BD061PH00W9bL0F5QH
08>3P`1KFe/0UiNG08f=S@0P8200=cLg06a/K01RHV80K6a/03<c<`0h>3P09bLW055AD@1OGel0>SXj
05IFEP0X:2P0<C4a05UIF@1=CDd0?clo04];B`0K6a/0>c/k06EUI@0>ooooo`1n06YZJP23Ph<0LG5a
06EUI@1OGel0O7al07=cL`1LG5`0Q8B409BDU00o?cl0CTi>08>3P`1bLW80]KFe06YZJP2UYJD0R8R8
09:BTP14A4@0NWYj0861P@1fMWH0HF5Q08:2PP1oOgl0BDU904Y:BP16ATH0J6QX04=3@`1aLG40YJFU
08F5Q@1@D500I6AT0:b/[01YJFT0AdM7075aL@1XJ6P0@d=3055AD@1_Kfl0LG5a05IFEP1^KVh0OGem
05ADE01IFET0B4Q803hn?P1bLW80B4Q804A4A01RHV80GemO06QXJ01VIVH0IfMW04a<C01TI6@0Bd];
055AD@1WIfL0J6QX08B4Q01ZJVX0IfMW06YZJP1cLg<0@T9204A4A01ADE40EeMG05ADE013@d<0FUYJ
05aLG00h>3P0>c/k0451@@0n?Sh09BDU05QHF010@400D51@02PX:01BDU80;Rh^061PH010@400<S8b
03Ti>@1OGel0EEEE04E5A@1IFET0C4a<05mOG`0b<S80F5QH05EEE@15ADD0LW9b0451@@1HF5P0DU9B
05=CD`1IFET0R8R806EUI@1^KVh0P82006m_K`1RHV80F5QH03`l?01ZJVX0EeMG06IVIP17AdL0TY:B
09JFUP15ADD0GUiN0P1gMgL0UP2CTi<0P82008V9R@13@d<0C4a<04i>CP0o?cl0JVYZ04Q8B01?Cdl0
GEeM0451@@0n?Sh0HF5Q04A4A01XJ6P0HF5Q04A4A01=CDd0@41005IFEP19BDT0Fe]K03De=@1=CDd0
P82002DU9@1hN7P0Cdm?05YJFP1[Jf/0=3@d071`L00o?cl0EEEE024Q8@18B4P0=3@d04m?C`1EEED0
8R8R05=CD`11@D40GUiN01LG5`0Y:BT0;2`/02PX:01=CDd0Hf=S02PX:013@d<0EUIF01lO7`0m?Cd0
?Shn04Q8B00X:2P0MGEe02`/;01WIfL0N7Qh01TI6@11@D40@T92061PH00j>SX0ADE504M7A`0j>SX0
P82008N7Q`0a<C40J6QX010@400d=3@0Bd];03hn?P1SHf<0DE5A05]KF`1`L700F5QH04=3@`16ATH0
=CDe04m?C`0l?3`0De=C03Xj>P0];Bd0PH6103Xj>P1EEED0=CDe0820P00N7Qh0SXj>08:2PP21PH40
EeMG04];B`1QHF40QhN703<c<`1MGEd0IVIV05aLG016ATH0@D5103/k>`1DE5@0820P06=SH`0[:b/0
DE5A03Ti>@1ADE40<c<c06i^KP1FEUH0J6QX05UIF@1HF5P0?3`l03Ph>01gMgL0K6a/07AdM015ADD0
B4Q80820P01RHV80@d=303Xj>P1^KVh0JVYZ05iNGP13@d<0?Shn05iNGP1[Jf/0Cdm?04Y:BP1FEUH0
NGUi05eMG@19BDT0QhN70:^[Za3ooooo000>ooooo`1B06i^KP1FEUH0PH61065QH@1HF5P0GUiN07Ui
N@2;Rh/0>SXj04Q8B01KFe/0P82009nOW`0U9BD0@41004e=C@1HF5P0Xj>S02/[:`0a<C40Bd];09>C
T`1MGEd0M7Ad02TY:@1QHF40J6QX02DU9@1LG5`0Bd];02/[:`26QXH0VYZJ034a<@1eMGD0FEUI02TY
:@1bLW80I6AT028R8P0e=CD0:RXZ02<S8`1/K6`0?Cdm03De=@1@D500ADE505IFEP1eMGD06ATI04I6
AP14A4@0@41006ATI01YJFT0820P03hn?P1=CDd0D51@07=cL`13@d<0;Bd]03@d=012@T80EUIF08f=
S@1dM7@09BDU0451@@1KFe/0NGUi0::RXP0c<c<07alO04i>CP1IFET0JFUY06e]K@0O7al07AdM04m?
C`80Fe]K07P0Lg=c01XJ6P0[:b/0B4Q805]KF`0];Bd0@d=301dM7@1BDU80MgMg03hn?P0a<C40=SHf
02d];@2;Rh/0^KVi02/[:`0M7Ad0<c<c0410@01gMgL0P82001/K6`0S8b<0AdM705iNGP21PH40WIfM
03Ph>00o?cl0B4Q8055AD@1GEeL0IVIV028R8P0M7Ad0ADE505MGE`0/;2`0Cdm?038b<P0[:b/0De=C
03Xj>P0U9BD0AdM702l_;`13@d<0WinO04i>CP0R8R80@d=305YJFP1`L700PH6101lO7`0R8R80>SXj
051@D02VYZH0CDe=02`/;01:BTX0DE5A06QXJ02m_Kd0>SXj028R8P1HF5P0M7Ad0451@@2;Rh/0ADE5
02LW9`1ZJVX0U9BD04I6AP17AdL0BDU903Ph>015ADD0O7al03hn?P20P800?Cdm030`<01QHF40Cdm?
0451@@1PH600=SHf05mOG`2@T9009RHV02h^;P1?Cdl0;2`/05YJFP1QHF40<C4a05ADE01:BTX0B4Q8
08^;R`14A4@0Bd];07]kN`17AdL0Bd];09VIV@0f=SH0GemO04e=C@1NGUh0JVYZ02@T900_;bl0J6QX
04U9B@1iNGT204E5A@1605MGE`1<C4`0F5QH0410@015ADD0I6AT05EEE@0c<c<0PX:203Ph>027QhL0
WYjN03`l?00k>c/0?Shn0451@@28R8P0JFUY02`/;01>CTh0<c<c055AD@2=SHd0AdM7038b<P1<C4`0
@D510410@02DU9@0C4a<02LW9`1DE5@0ATI605ADE01MGEd0K6a/03@d=01OGel0NWYj04a<C01hN7P0
KVi^03`l?01gMgL0Xj>S04U9B@2PX:00OWin038b<P1EEED0UYJF05eMG@1CDe<0GUiN024Q8@1?Cdl0
L71`02XZ:P1>CTh0<C4a03Lg=`16ATH0E5AD01`L700`<300=SHf04=3@`1<C4`0K6a/01TI6@80A4A4
3_ooool0@`1ADE40F5QH06i^KP1BDU80JVYZ09jNWP1^KVh0Cdm?0:BTY031`L40Bd];04e=C@24Q8@0
UiNG07emO@2SXj<0K6a/03Ph>018B4P0Cdm?04e=C@0`<300D51@05MGE`18B4P0CTi>03Xj>P13@d<0
;bl_06YZJP0n?Sh0CTi>04Q8B01MGEd0HV9R07=cL`1dM7@0IfMW04e=C@11@D40JVYZ04U9B@1SHf<0
=cLg055AD@12@T80CDe=0410@00i>CT0@41004e=C@0m?Cd0@41002<S8`0W9bL0F5QH05iNGP0i>CT0
>SXj06=SH`1^KVh0?clo0410@01PH600?clo03<c<`1GEeL00P1nOWh0]01ADE40Jf][07alO01YJFT0
CDe=03<c<`17AdL0IFEU05mOG`12@T80;bl_05ADE00m?Cd0@41003/k>`0f=SH0=CDe05EEE@0[:b/0
@T9203Hf=P14A4@09bLW03Lg=`1LG5`0FUYJ03hn?P1?Cdl0@D5103dm?@1<C4`0ATI605YJFP1MGEd0
ADE506e]K@1SHf<0JFUY05]KF`1jNWX0Ng]k09bLW01QHF40K6a/08>3P`1VIVH0FUYJ05]KF`0f=SH0
H61P03hn?P1]KFd0EeMG0:JVYP2d];@0IFEU075aL@2CTi<0];Bd0:6QX@1cLg<0MWIf02TY:@1_Kfl0
RXZ:09ZJVP1NGUh0CTi>05QHF01ADE40E5AD055AD@1UIFD0HV9R092@T01OGel0>3Ph05YJFP1ZJVX0
Hf=S05]KF`1oOgl0Hf=S028R8P1MGEd0Bd];08J6QP1CDe<0<c<c05]KF`16ATH0Lg=c034a<@0n?Sh0
;bl_04m?C`0a<C40?clo04I6AP0d=3@0Fe]K02h^;P13@d<06a/K03lo?`0L71`0<S8b04E5A@1ADE40
:2PX04e=C@1<C4`0:BTY04];B`1?Cdl0EEEE020P801HF5P08B4Q02l_;`1FEUH05aLG04i>CP0i>CT0
C4a<03`l?01HF5P0EEEE0410@01;Bd/0I6AT02XZ:P20P8008B4Q04Y:BP1;Bd/0>CTi04m?C`13@d<0
C4a<065QH@1LG5`0=SHf03<c<`0Y:BT0B4Q803Ph>01SHf<0@41002d];@1]KFd0820P04Y:BP0T92@0
Ogmo038b<P1SHf<0IVIV06e]K@1CDe<0BDU905IFEP1`L700CDe=08B4Q016ATH0Cdm?03`l?014A4@0
ATI604Y:BP0O7al0UiNG03Lg=`1BDU809RHV04a<C016ATH0E5AD061PH080F5QH01/0N7Qh06MWI`1?
Cdl0LG5a05]KF`26QXH0I6AT03Hf=P11@D40@41004m?C`13@d<0BTY:03`l?00_;bl0CDe=059BDP1N
GUh0F5QH05iNGP23Ph<0IFEU04e=C@2;Rh/0EeMG07UiN@22PX804?ooool000kooooo04`0UIFE05QH
F01UIFD0KFe]07AdM012@T80H61P075aL@1HF5P0Shn?03<c<`15ADD0EUIF04m?C`1`L700OGem03Lg
=`1GEeL0IVIV04U9B@2NWYh0@41003De=@1<C4`0@d=307QhN01TI6@0>3Ph04m?C`1IFET0?3`l07em
O@1;Bd/0<S8b05YJFP1_Kfl0=cLg04M7A`1?Cdl0D51@08J6QP1PH600?Cdm04m?C`0m?Cd0CTi>079b
LP0k>c/0>3Ph051@D00d=3@0KFe]06=SH`0I6AT0=CDe04m?C`15ADD0O7al04M7A`0U9BD0?Shn05MG
E`16ATH0KFe]02DU9@0O7al0A4A406a/K014A4@0D51@024Q8@0^;Rh0Hf=S08R8R00];Bd0BDU90P0d
=3@0OP1ZJVX0G5aL038b<P1RHV80<S8b04I6AP1dM7@0?Cdm02XZ:P1=CDd0B4Q803dm?@1bLW80?Cdm
02DU9@1EEED0C4a<059BDP1ADE407QhN04=3@`1BDU80JFUY05EEE@1HF5P07QhN0410@01FEUH0AdM7
04U9B@2;Rh/0<S8b04A4A01kNg/0DU9B04A4A01gMgL0;Rh^04];B`21PH40ADE503Hf=P1SHf<0?Shn
04I6AP2UYJD0A4A403Hf=P1@D500?Cdm04A4A02;Rh/09RHV0410@01UIFD0ADE505IFEP1JFUX09BDU
04A4A01VIVH0C4a<05mOG`1gMgL0:2PX06QXJ02DU9@0Bd];07UiN@0o?cl0?Shn08J6QP1OGel0?3`l
06=SH`14A4@0:BTY07UiN@1?Cdl0?3`l061PH01aLG40=CDe05iNGP1OGel0?Shn06IVIP0k>c/0=cLg
09RHV01MGEd0@T9204U9B@0];Bd0FEUI09nOW`1UIFD0@410061PH00k>c/0C4a<08f=S@0e=CD0<S8b
06MWI`1EEED0VYZJ04I6AP0Q8B40Bd];05mOG`1=CDd0H61P02HV9P0i>CT0Ph>306EUI@1LG5`09BDU
051@D01eMGD0FEUI04Q8B01EEED0AdM707IfMP80DU9B04P0Ng]k03@d=01OGel0Fe]K03Ti>@1^KVh0
Bd];04i>CP1dM7@0?Shn05MGE`1aLG40>c/k05EEE@1^KVh0>SXj06=SH`1^KVh0CDe=08^;R`1;Bd/0
9RHV06YZJP1ADE40EUIF079bLP1<C4`08B4Q04U9B@1_Kfl0@41007]kN`18B4P07QhN03/k>`1RHV80
ADE509BDU00o?cl0<c<c04Y:BP1WIfL0?Cdm06m_K`0S8b<0@d=30820P01bLW80>CTi05QHF01BDU80
<S8b071`L02=SHd0?3`l04A4A00i>CT0>c/k07alO01QHF407QhN03Lg=`1@D500>SXj07UiN@1aLG40
9RHV03hn?P1DE5@0=SHf08:2PP1mOGd>ooooo`2^05YJFP1UIFD0MGEe05IFEP1^KVh0N7Qh075aL@1V
IVH0IFEU06m_K`1BDU80J6QX07UiN@1hN7P0KVi^04i>CP1]KFd0BDU9069RHP12@T80HF5Q05QHF01Q
HF40J6QX06EUI@1]KFd0Cdm?05YJFP0k>c/0De=C04I6AP1RHV80IFEU06MWI`1/K6`0Rh^;09>CT`1j
NWX0Cdm?05ADE01TI6@0BTY:05aLG01=CDd0LG5a04A4A01BDU80DE5A04I6AP1:BTX0DE5A03Ph>01[
Jf/0<30`03dm?@1NGUh0I6AT06UYJ@1OGel0H61P0492@P1ADE40@D5104M7A`1DE5@0EEEE08R8R01W
IfL0EeMG061PH01TI6@0?Shn03Ti>@13@d<0=SHf04Y:BP0d=3@0EEEE04a<C00`<300=CDe038b<P0k
>c/08R8R04];B`0m?Cd0ATI6028R8P17AdL0:RXZ051@D00j>SX0;2`/02h^;P17AdL0@T9203`l?015
ADD0Bd];04m?C`13@d<0CTi>04e=C@16ATH0E5AD03Ph>013@d<0EEEE06MWI`1/K6`0KVi^075aL@1Z
JVX0Lg=c06=SH`1CDe<0D51@03hn?P1CDe<0ATI605MGE`1SHf<0RXZ:0:2PX019BDT0S8b<06][J`1]
KFd0Q8B406IVIP20P800B4Q806=SH`1iNGT0KVi^04];B`1HF5P0Fe]K065QH@14A4@0@d=305MGE`1H
F5P0IVIV06ATI018B4P0GEeM04I6AP1FEUH0>3Ph05UIF@18B4P0?clo04U9B@0J6QX0OWin06IVIP0k
>c/0GEeM03hn?P1jNWX0ATI6055AD@0R8R80Cdm?0410@017AdL0Cdm?030`<01<C4`0@d=305YJFP0^
;Rh0;Bd]0P0c<c<0E017AdL0De=C02d];@11@D40BDU9024Q8@12@T80BTY:04m?C`0L71`0H61P02LW
9`0j>SX0GEeM010@4014A4@0>SXj051@D00g=cL0CDe=05=CD`0n?Sh0IfMW07moO`0_;bl0DU9B01dM
7@14A4@0EeMG0451@@1VIVH0BTY:05IFEP1dM7@0DE5A04A4A016ATH0>CTi04I6AP19BDT0FUYJ04];
B`19BDT0NWYj03Ph>01CDe<09bLW07alO00X:2P0Ph>306][J`1gMgL0Fe]K06IVIP1]KFd0KVi^059B
DP1NGUh0IfMW06i^KP1ADE40@d=3069RHP1MGEd092@T05aLG00e=CD0GUiN03dm?@1GEeL0@D5105MG
E`1DE5@0K6a/07alO01kNg/0C4a<0451@@1JFUX0BDU907AdM01>CTh092@T04i>CP80?Cdm0100;bl_
061PH01DE5@0A4A403`l?012@T80GEeM06m_K`1SHf<0K6a/06EUI@21PH40GEeM05ADE025QHD0VYZJ
4?ooool000kooooo00d0NWYj07QhN02b/[80CDe=06a/K01eMGD0`l?307]kN`10@400HV9R075aL@2/
[:`0Jf][0080>CTi09<0LW9b07=cL`1ZJVX0=CDe03`l?01kNg/0NGUi03`l?01KFe/06a/K05MGE`1X
J6P09RHV05ADE017AdL0;bl_06YZJP1_Kfl09BDU04i>CP0e=CD0CTi>07]kN`0M7Ad07alO03lo?`14
A4@0F5QH065QH@0V9RH0:b/[03hn?P17AdL0F5QH06IVIP0X:2P0<c<c05aLG019BDT0Ph>307]kN`0X
:2P0?Shn06e]K@13@d<0J6QX03@d=00R8R80KFe]06QXJ00l?3`0I6AT038b<P0X:2P0CTi>05QHF011
@D40EEEE02@T900[:b/0?Cdm04I6AP0h>3P0>c/k02LW9`1FEUH0KVi^04I6AP0f=SH0BTY:02d];@1]
KFd0/K6a04];B`0X:2P0A4A40492@P1KFe/0WYjN02@T900a<C40Jf][03hn?P1mOGd0YZJV01/K6`0l
?3`0EeMG02d];@1NGUh0CDe=02`/;00f=SH0E5AD059BDP1NGUh0Ph>303<c<`0Y:BT0CTi>03@d=01<
C4`0DU9B02`/;00l?3`0NGUi05EEE@0U9BD0DU9B04i>CP0l?3`0TY:B03De=@0W9bL0BDU9069RHP1d
M7@0WYjN02l_;`0k>c/0Ng]k05iNGP1bLW80G5aL030`<010@400P82004M7A`1@D5007alO04=3@`1]
KFd0YJFU03Xj>P1EEED0@d=303/k>`1kNg/0Lg=c03<c<`0o?cl0;bl_03Lg=`1jNWX0Jf][038b<P0X
:2P00P1GEeL0A`2XZ:P0=SHf01XJ6P0k>c/0CTi>07emO@2;Rh/0=CDe03<c<`1DE5@0GUiN06YZJP2<
S8`09BDU04Q8B01/K6`0F5QH05aLG019BDT0@41007UiN@1ZJVX0Fe]K08J6QP0o?cl0De=C069RHP0_
;bl0@41008R8R00e=CD0DU9B04E5A@18B4P0?Cdm03<c<`1LG5`0HV9R03@d=01HF5P0<30`07inOP2_
[jl0=CDe04M7A`15ADD0N7Qh0<30`01;Bd/0=3@d05EEE@1FEUH0Q8B407IfMP0g=cL0B4Q806IVIP1P
H600RXZ:03@d=00e=CD0@T9205aLG015ADD0OGem034a<@1;Bd/0BTY:09ZJVP1OGel0DU9B0080=3@d
02T0H61P07emO@0m?Cd0B4Q804I6AP0`<300G5aL07inOP0d=3@0F5QH05aLG00Y:BT0QhN7061PH00Z
:RX0ATI6055AD@15ADD0Q8B407alO00_;bl0:b/[04a<C01DE5@0NGUi03<c<`0K6a/0CDe=05QHF01?
Cdl0EUIF06QXJ00L71`0CDe=079bLP1:BTX0@T9204A4A00I6AT0CDe=07=cL`0>ooooo`1406MWI`1R
HV80IVIV055AD@1HF5P0S8b<07alO01UIFD0Ogmo08j>SP1ADE40CTi>04e=C@13@d<0T92@07emO@1l
O7`0A4A405EEE@1JFUX0Fe]K04=3@`1EEED0GemO04Q8B01;Bd/0@41005ADE00h>3P0GUiN03`l?01;
Bd/0BDU905QHF01`L700QXJ607]kN`1mOGd0Bd];03dm?@1HF5P0D51@05UIF@0`<300CDe=04a<C013
@d<0?Cdm03Lg=`1QHF40BDU903Xj>P1DE5@0;Bd]030`<01MGEd0GUiN0451@@14A4@0Hf=S0820P01M
GEd0CDe=05=CD`19BDT0>c/k065QH@1iNGT206MWI`1`05UIF@1DE5@0Cdm?0451@@1=CDd0?Shn065Q
H@1GEeL0>3Ph0410@01KFe/0;2`/03<c<`0_;bl0=cLg02TY:@1BDU807QhN03Ti>@0/;2`0De=C02l_
;`0c<c<0BTY:05IFEP0i>CT0FEUI03Ti>@0[:b/0ATI605=CD`1ADE40De=C04M7A`1UIFD0GemO05YJ
FP16ATH0KVi^08^;R`2UYJD0H61P05mOG`21PH40FEUI055AD@1DE5@0?Shn05]KF`0n?Sh0IFEU05AD
E01aLG40DU9B03hn?P1QHF40FEUI09ZJVP2ATI40FEUI05eMG@0j>SX0FEUI06MWI`1=CDd0Bd];065Q
H@11@D40?Shn03Ti>@1LG5`0@d=304i>CP2n_[h0H61P0492@P1DE5@0@d=3065QH@1OGel0K6a/051@
D01YJFT0KVi^05]KF`1dM7@0C4a<0410@01^KVh0Bd];0820P01HF5P0De=C02d];@1?Cdl0Bd];03<c
<`17AdL0=CDe04a<C018B4P0E5AD03Lg=`0[:b/0@d=303@d=016ATH0F5QH0410@01ADE40GEeM0492
@P80A4A405h0HV9R05UIF@1TI6@0;bl_05UIF@1/K6`0@T92038b<P1>CTh0>SXj055AD@1IFET0B4Q8
04a<C01LG5`0K6a/05EEE@2DU9@0>SXj04M7A`1ADE40ADE505]KF`0k>c/0G5aL06MWI`0g=cL0>SXj
03dm?@0];Bd0@d=304A4A01KFe/0E5AD0410@029RHT0>SXj04m?C`0^;Rh0OGem0492@P1/K6`0KFe]
092@T01`L700I6AT04a<C02YZJT0BDU9069RHP29RHT0I6AT04Q8B012@T80D51@0410@016ATH0IfMW
04E5A@1KFe/0A4A405aLG011@D40P82004Q8B01BDU80Cdm?071`L01SHf<0N7Qh06YZJP1ADE40FEUI
04I6AP15ADD0PH61069RHP10@400>SXj04I6AP1=CDd0>SXj0410@016ATH0F5QH061PH01LG5`0RXZ:
079bLP1QHF40JVYZ059BDP1[Jf/0Z:RX4?ooool000kooooo00@0HV9R05EEE@1JFUX0GUiN0P1>CTh0
O@29RHT0Y:BT05iNGP1GEeL0De=C05UIF@1TI6@0<c<c03Hf=P0k>c/0FEUI06a/K00_;bl0>3Ph05UI
F@1PH600Fe]K096AT@0O7al0BTY:04];B`1FEUH0Ph>3071`L00d=3@0E5AD05IFEP1cLg<0^;Rh07em
O@0`<300GEeM04Q8B01VIVH0YJFU03De=@0f=SH0CDe=04E5A@1UIFD0GUiN02HV9P0Z:RX0ADE503Hf
=P1MGEd0:2PX034a<@1@D500HF5Q03<c<`1WIfL0:BTY03<c<`1;Bd/0EeMG02LW9`0k>c/0;bl_03Xj
>P1lO7`0KFe]01HF5P0a<C40E5AD069RHP2VYZH0?Cdm02@T9016ATH0CTi>09>CT`1kNg/0:2PX03@d
=01:BTX0?Shn05IFEP1oOgl07alO038b<P0g=cL0KVi^065QH@1NGUh07alO02DU9@1?Cdl0MGEe04a<
C017AdL07AdM04I6AP1^KVh0CDe=034a<@1:BTX0=CDe07YjNP20P800820P02h^;P15ADD0EeMG07Yj
NP2HV9P0:2PX0410@01DE5@0GemO08Z:RP2>SXh0:RXZ028R8P0e=CD0C4a<05iNGP1/K6`061PH02HV
9P0Z:RX0@D51034a<@1KFe/04Q8B0080AdM70940>3Ph02TY:@1ADE409BDU06a/K01jNWX09bLW03dm
?@19BDT0DU9B07AdM00];Bd0:2PX04=3@`22PX80LG5a08V9R@0^;Rh0:2PX03hn?P1eMGD0CTi>08B4
Q00`<300820P03Lg=`1fMWH0Fe]K0820P01SHf<0<c<c06MWI`22PX80=3@d08V9R@0o?cl0A4A407=c
L`10@400?Cdm07AdM00Z:RX0<c<c05aLG015ADD0@d=30<?3``0g=cL0?3`l05mOG`0o?cl0IVIV04m?
C`1IFET0XJ6Q05QHF011@D40@410055AD@1NGUh0alO704Y:BP1>CTh0DU9B07MgM`1jNWX06ATI03/k
>`1FEUH0IVIV09jNWP0Y:BT0@D5106IVIP1CDe<0IVIV02@T900j>SX0Ng]k06i^KP0m?Cd0PX:202HV
9P1JFUX0Xj>S04i>CP1RHV80E5AD03De=@21PH40Q8B403/k>`1iNGT0=CDe024Q8@1UIFD0BTY:02<S
8`1<C4`0AdM7055AD@1NGUh0@T92038b<P1DE5@0BTY:034a<@1jNWX0HV9R03`l?018B4P0@T9205MG
E`2i^KT0T92@03<c<`14A4@0CTi>05QHF02FUYH0SXj>02HV9P11@D40DU9B04E5A@1mOGd0G5aL01LG
5`1:BTX0E5AD03@d=015ADD0:2PX01hN7P0[:b/0EeMG02h^;P12@T80<c<c02@T9015ADD0@d=301XJ
6P0h>3P0=CDe00kooooo03d0ZZZZ09jNWP26QXH0DE5A07=cL`1`L700K6a/071`L017AdL0Fe]K059B
DP1/K6`0FUYJ06e]K@2@T900OWin06i^KP1FEUH0IFEU03Xj>P1MGEd0F5QH059BDP13@d<0Bd];04e=
C@1?Cdl0D51@03De=@1;Bd/0>c/k05MGE`1NGUh0De=C03hn?P1ADE40TI6A0861P@18B4P0EEEE04i>
CP13@d<0?Shn03Ti>@1;Bd/0@T9203Xj>P1>CTh0@D5104M7A`18B4P0>CTi04Y:BP16ATH0E5AD04i>
CP1<C4`0K6a/05EEE@1?Cdl0ATI60080@D5100H0;2`/0410@01?Cdl0GUiN05QHF01>CTh205EEE@80
AdM701/0>c/k03@d=00m?Cd0;Bd]04a<C01DE5@0<C4a02l_;`0[:b/0=SHf02XZ:P1;Bd/0<c<c04=3
@`0/;2`0@41002<S8`18B4P0@41002h^;P0G5aL0@D510410@00S8b<0>SXj04=3@`19BDT00P0k>c/0
O011@D40IFEU04m?C`0c<c<0=SHf04a<C01MGEd0NWYj07=cL`1NGUh0EUIF06YZJP1cLg<0>CTi0492
@P0f=SH0A4A404M7A`19BDT0FUYJ07MgM`1/K6`0Bd];08N7Q`1NGUh0?clo03hn?P18B4P0G5aL0451
@@1jNWX0KFe]02h^;P1<C4`0A4A40492@P19BDT0;Rh^02l_;`1IFET0EUIF04I6AP0m?Cd0ADE505YJ
FP1hN7P0@D5102l_;`1QHF40Cdm?071`L01OGel0<c<c092@T01PH600@41005UIF@1QHF40RHV905QH
F01]KFd0@D5106m_K`15ADD0@D5104a<C01FEUH0De=C03Xj>P1EEED0AdM704E5A@14A4@0=cLg059B
DP1?Cdl0:2PX04m?C`1GEeL05QHF03Ti>@0k>c/0DU9B01hN7P1GEeL0FEUI03Lg=`1aLG405ADE02LW
9`0o?cl0?3`l03Ti>@1MGEd0CTi>065QH@1cLg<0PX:205=CD`1<C4`06a/K06YZJP1aLG40>SXj06UY
J@1GEeL0GEeM06e]K@1CDe<0CDe=0492@P15ADD0FEUI05=CD`2DU9@0Kfm_04Y:BP2ATI408b<S055A
D@0a<C40RHV904=3@`1cLg<207]kN`0b06UYJ@1gMgL0Lg=c08R8R01/K6`0LW9b05]KF`1WIfL0A4A4
0451@@1DE5@0<c<c02h^;P0f=SH0?clo05QHF00X:2P0Fe]K03Hf=P13@d<0EeMG05=CD`1XJ6P0GEeM
06QXJ02HV9P0GEeM06MWI`1NGUh0N7Qh03De=@0c<c<0BTY:04I6AP1EEED0>SXj0451@@0i>CT0E5AD
04m?C`1gMgL0Ogmo05mOG`2][Jd0M7Ad06MWI`2?Shl0GemO08j>SP2/[:`@ooooo`003_ooool05@2D
U9@0D51@05QHF02;Rh/0Ogmo03<c<`1aLG40WIfM05aLG02<S8`0@D5104i>CP1[Jf/0BTY:05EEE@1=
CDd0BTY:08>3P`17AdL0LG5a09bLW00203@d=02U07MgM`0n?Sh0HF5Q04e=C@0L71`0EeMG07inOP16
ATH0De=C02l_;`0T92@0EEEE09jNWP1@D500GemO01/K6`0[:b/0J6QX06MWI`0_;bl0ADE5028R8P1Q
HF40OGem038b<P1:BTX0H61P03Xj>P2?Shl0?clo01HF5P11@D40FEUI06=SH`2g]kL0@d=302XZ:P19
BDT0@T9208J6QP2m_Kd08B4Q03Hf=P0i>CT0ATI605ADE01LG5`0:2PX04a<C01OGel0JFUY03De=@16
ATH0;Bd]02l_;`1PH600GEeM03De=@1fMWH0<c<c071`L02FUYH0<S8b02/[:`15ADD0<30`069RHP2=
SHd08B4Q02l_;`0`<300IfMW079bLP19BDT05ADE03Lg=`1OGel0Kfm_07alO01SHf<06a/K06QXJ01e
MGD0@41005aLG02?Shl09bLW05]KF`1ZJVX0@D5105iNGP2UYJD0;2`/04M7A`1fMWH0?Shn03Hf=P16
ATH0;Rh^04Y:BP2l_;`0D51@028R8P1=CDd0:BTY02l_;`2VYZH09RHV02d];@15ADD0>SXj05]KF`2G
UiL09bLW03dm?@1^KVh0AdM708N7Q`25QHD0<30`061PH01<C4`0B4Q807emO@1:BTX0>CTi071`L01<
C4`092@T065QH@0h>3P06ATI05YJFP1>CTh0=cLg05IFEP0_;bl09bLW06i^KP1fMWH0:RXZ03Lg=`0n
?Sh0ADE5092@T01BDU80=cLg04a<C01:BTX0FEUI0:BTY01EEED0>3Ph059BDP1IFET0NWYj0:NWY`0W
9bL0Cdm?06a/K01XJ6P0[jn_02/[:`0g=cL0JFUY0080Jf][01/0CDe=02PX:00m?Cd0JVYZ07IfMP1K
Fe/09RHV05eMG@1cLg<0?3`l061PH01;Bd/0?Cdm07emO@1UIFD0EUIF05iNGP0^;Rh0GEeM05mOG`0]
;Bd0FEUI04M7A`1`L700];Bd030`<00k>c/00P1ADE40>`2FUYH0QXJ6030`<01IFET0L71`06][J`2i
^KT0J6QX01lO7`1:BTX0EeMG06a/K02^[Zh0GemO02TY:@1@D500FEUI06a/K033`l<0?clo03Ph>011
@D40KFe]06ATI02DU9@0;Bd]02`/;013@d<0Ph>304];B`1LG5`0;Rh^030`<01CDe<0G5aL03@d=01i
NGT0;Bd]0451@@1IFET0IFEU03/k>`12@T80BDU905]KF`1fMWH0FUYJ01<C4`0];Bd0Cdm?04a<C02=
SHd0Ng]k01/K6`0m?Cd0EUIF03Ti>@1eMGD0@d=300kooooo0400ADE505]KF`1iNGT0Cdm?0492@P1?
Cdl0F5QH05UIF@1]KFd0VYZJ04=3@`15ADD0FEUI08R8R01RHV80DE5A06=SH`1@D500EeMG04U9B@1G
EeL0C4a<05=CD`1GEeL0JFUY06ATI00l?3`0BTY:03Xj>P1cLg<0<C4a04a<C01`L700Jf][05ADE01N
GUh0VIVI06IVIP10@400DE5A04a<C00o?cl0EEEE05=CD`1UIFD0@41005ADE01VIVH0E5AD02`/;018
B4P0CDe=05eMG@0R8R80<C4a04i>CP1?Cdl0@T9202<S8`0e=CD0C4a<04i>CP14A4@0?Shn0P1BDU80
;01>CTh0EeMG06ATI01HF5P0BTY:03dm?@0n?Sh0ATI604i>CP13@d<0?Cdm03Ph>00k>c/0=SHf03Ti
>@0`<300;bl_02PX:010@400ADE503Ti>@0U9BD0<S8b02/[:`0k>c/0<c<c02d];@0e=CD0?clo03/k
>`13@d<0>c/k0451@@0m?Cd0?Shn05mOG`1NGUh0;bl_04a<C01?Cdl0B4Q804U9B@1WIfL0QhN70P2=
SHd01`1hN7P0G5aL04a<C00f=SH0@41002@T9012@T800P19BDT0=@1:BTX0BDU905]KF`1@D500FEUI
04I6AP20P800MWIf07EeM@1hN7P0<c<c0410@01<C4`0CTi>03hn?P0c<c<0MWIf07alO01;Bd/0DU9B
04Q8B01:BTX0/k>c071`L01YJFT0DU9B02`/;016ATH0BDU904Y:BP11@D40U9BD0:2PX00k>c/0J6QX
05]KF`1NGUh0P82004Q8B01iNGT0Hf=S07=cL`0O7al0D51@04U9B@1KFe/0JFUY02`/;018B4P0BTY:
06=SH`0K6a/09bLW0080BTY:02T0J6QX071`L00o?cl0D51@05]KF`0`<300ATI603`l?01FEUH0;bl_
06MWI`19BDT0Cdm?05]KF`0N7Qh0C4a<04A4A015ADD0>SXj05QHF01aLG40H61P06e]K@1lO7`0;Rh^
06UYJ@0I6AT0<30`055AD@1cLg<0V9RH05EEE@1RHV80Rh^;038b<P0i>CT0HF5Q03Ph>013@d<0Cdm?
051@D00204e=C@0:0:^[Z`13@d<0OGem01HF5P1YJFT05ADE08F5Q@1iNGT0MGEe05YJFP80IVIV02l0
Lg=c03`l?01;Bd/0Cdm?06QXJ019BDT0ATI608n?S`1LG5`0;Rh^0451@@0i>CT0DU9B04M7A`1jNWX0
@d=3059BDP0i>CT0GUiN06YZJP2ATI40Hf=S07moO`28R8P0N7Qh08n?S`22PX80@d=304];B`1MGEd0
Kfm_04I6AP0m?Cd0?Shn03Lg=`1BDU80I6AT06=SH`1oOgl0LW9b07UiN@23Ph<0[Jf]07MgM`1;Bd/0
Hf=S0;Rh^00@ooooo`003_ooool08@2GUiL0DE5A07emO@1GEeL0];Bd065QH@1JFUX0BDU903<c<`1Y
JFT0>3Ph04e=C@1;Bd/0ADE505EEE@2PX:00>3Ph05IFEP0n?Sh0HV9R0861P@0^;Rh06QXJ051@D01i
NGT0MWIf05EEE@0F5QH09BDU05IFEP1TI6@0[Zj^03lo?`0202TY:@3c06YZJP1cLg<0Ng]k028R8P14
A4@0ATI604Q8B019BDT0EEEE01<C4`0m?Cd0LG5a05QHF01dM7@0MGEe034a<@1bLW80Jf][04U9B@1X
J6P0?Cdm03<c<`1KFe/0C4a<02h^;P1FEUH0>3Ph03dm?@2PX:00EEEE01LG5`1@D500ATI601`L701]
KFd08B4Q02/[:`1:BTX0?Cdm04i>CP2GUiL0>3Ph02LW9`0l?3`0EEEE06=SH`2g]kL0>CTi02`/;01K
Fe/0FUYJ06][J`1gMgL0<c<c02HV9P1WIfL0DE5A05iNGP1iNGT071`L03Xj>P1XJ6P0>CTi05YJFP1]
KFd0410@03Ti>@1cLg<05QHF05eMG@0i>CT0;Rh^05=CD`2TY:@0=3@d04i>CP1GEeL0?clo06e]K@2U
YJD0>CTi05=CD`1:BTX0H61P079bLP2?Shl0=SHf03Ti>@1YJFT0QXJ606e]K@2KVi/0>c/k01`L701@
D500ADE507alO01]KFd09BDU02@T901hN7P0=SHf07inOP0d=3@07QhN03/k>`2CTi<0>CTi04U9B@0b
<S80@D51069RHP21PH40=3@d05ADE00n?Sh0=CDe08:2PP2LW9`0;2`/0492@P1EEED0I6AT092@T00m
?Cd07alO02PX:015ADD0K6a/09:BTP0I6AT09BDU02l_;`1FEUH0G5aL05]KF`0T92@0;Bd]04];B`1N
GUh0>c/k0451@@0R8R80AdM706m_K`13@d<0@D5103lo?`1FEUH0H61P04];B`0];Bd0I6AT04=3@`2Z
ZZX0NGUi01dM7@0c<c<0E5AD03`l?01oOgl092@T03/k>`19BDT0@4100;Fe]@1SHf<0;Bd]069RHP1P
H600Ti>C08j>SP0f=SH0Bd];05MGE`1hN7P0[Zj^0451@@0f=SH0HF5Q05EEE@1XJ6P0:BTY02LW9`0n
?Sh0QXJ6055AD@1_Kfl0:BTY02DU9@1>CTh0N7Qh03Xj>P0m?Cd09bLW02/[:`1HF5P0Xj>S03`l?01B
DU8071`L03Hf=P1bLW80JFUY02DU9@1?Cdl0?Shn04Y:BP25QHD0BDU901lO7`15ADD0@41007EeM@2@
T900:2PX01XJ6P1ADE40NWYj069RHP3Cdm<0?clo030`<01VIVH0CTi>04e=C@1WIfL0?clo02LW9`1B
DU80KVi^04m?C`1GEeL0EUIF01dM7@11@D40DE5A03<c<`0j>SX0@D5103@d=01eMGD0GEeM00kooooo
03D0S8b<08j>SP1HF5P0DU9B08J6QP1/K6`0CDe=04U9B@1IFET0N7Qh05QHF01>CTh0Ph>306][J`2`
/;00V9RH08j>SP1HF5P0JVYZ06e]K@1OGel0FEUI069RHP1fMWH0GemO06ATI01;Bd/0FUYJ0492@P1L
G5`0<S8b05YJFP1EEED0IVIV079bLP2:RXX0T92@07UiN@1BDU80BTY:06a/K016ATH0Fe]K04=3@`1V
IVH0A4A4055AD@17AdL0GEeM0492@P14A4@0>CTi04m?C`02034a<@80G5aL00L0@T9203`l?01QHF40
Lg=c04];B`0n?Sh0GemO0080A4A406H0EUIF0861P@2?Shl0>c/k03Lg=`1ADE40Bd];03Ti>@0U9BD0
>CTi05YJFP1NGUh0?Cdm0410@01CDe<0;bl_03Hf=P0W9bL0<c<c02d];@1:BTX09RHV02d];@0`<300
EEEE03/k>`16ATH0De=C05UIF@0c<c<0EEEE03Ph>00U9BD0=SHf038b<P0i>CT0?3`l04I6AP12@T80
CTi>04A4A01=CDd0ADE507moO`1XJ6P0<C4a03lo?`1LG5`0Lg=c02XZ:P0g=cL0@41003Lg=`0d=3@0
@410071`L01PH600GEeM04];B`2OWil0DU9B0492@P17AdL0@D5105IFEP0g=cL0B4Q805YJFP1OGel0
H61P05UIF@0k>c/0=3@d02l_;`0e=CD0K6a/069RHP1MGEd0@d=303lo?`1ADE40Hf=S05IFEP1jNWX0
UiNG04U9B@10@400C4a<020P802ATI40EUIF038b<P1@D500A4A407=cL`1:BTX0DE5A028R8P1<C4`0
>3Ph02PX:00m?Cd204e=C@1402d];@0l?3`0>c/k03Ph>01<C4`0=cLg051@D018B4P0F5QH0820P01c
Lg<0CTi>05=CD`1fMWH0R8R804a<C01`L700<c<c05YJFP1_Kfl06a/K024Q8@0c<c<0CDe=0410@01>
CTh0B4Q8055AD@1WIfL0M7Ad0410@023Ph<0<c<c04Y:BP1<C4`0ADE5079bLP1YJFT0F5QH05]KF`16
ATH0>CTi03Hf=P0i>CT0ATI604a<C01SHf<0JVYZ069RHP2BTY80=cLg04I6AP0h>3P0QhN7038b<P26
QXH0Ng]k09NGU`1WIfL0GUiN055AD@2CTi<0?clo05IFEP10@400B4Q808:2PP1QHF4204U9B@0V0492
@P1RHV80@T9203Lg=`0S8b<0?Cdm034a<@1`L700Fe]K07AdM01WIfL0Ph>306EUI@2b/[80P82005mO
G`1CDe<0IFEU055AD@1XJ6P0Bd];03Xj>P15ADD0FUYJ04I6AP0c<c<0?Cdm05eMG@1WIfL0RHV906EU
I@22PX80Kfm_05eMG@21PH40IfMW07inOP32`/8@ooooo`003_ooool0JP2=SHd0T92@0;:b/P16ATH0
RXZ:06YZJP2WYjL0MGEe0410@01NGUh0EEEE0:j^[P1JFUX0>CTi03lo?`1>CTh0KFe]08J6QP0K6a/0
HV9R061PH01FEUH0Cdm?06][J`0c<c<0M7Ad07UiN@1@D500U9BD04=3@`1_Kfl0IfMW07=cL`1CDe<0
HV9R01HF5P0f=SH0HV9R03<c<`11@D40GemO03<c<`11@D40IFEU02l_;`0n?Sh0BTY:02h^;P1IFET0
PX:202d];@1OGel0@D510410@02][Jd0LG5a02DU9@1BDU80FUYJ06e]K@1aLG40<C4a02DU9@1@D500
P82004U9B@1[Jf/0C4a<01LG5`12@T80EeMG03dm?@2<S8`0820P02TY:@0b<S80J6QX06m_K`1HF5P0
9bLW01hN7P0m?Cd0CTi>04=3@`1;Bd/04A4A059BDP20P800=SHf02/[:`0g=cL0;Bd]06=SH`2JVYX0
;bl_02XZ:P1;Bd/0=3@d07]kN`2[Zj/08B4Q01<C4`13@d<0?3`l0;2`/01XJ6P202LW9`2Z04i>CP1G
EeL0/;2`05iNGP0N7Qh0=3@d04e=C@18B4P0K6a/08J6QP0G5aL0ATI606m_K`1XJ6P0Bd];05QHF00H
61P0@T9208R8R01PH6007AdM03dm?@0[:b/0M7Ad08Z:RP0T92@09RHV04M7A`1<C4`0b<S805=CD`0H
61P09RHV079bLP1FEUH0LW9b02<S8`0Y:BT0?Shn09NGU`1eMGD0Ti>C02TY:@0o?cl0ADE508j>SP14
A4@0G5aL01<C4`0b<S80GUiN06e]K@0e=CD0DE5A04=3@`0j>SX0Q8B404];B`0Z:RX0ADE5034a<@1C
De<0OWin02@T900U9BD0E5AD03dm?@1QHF40J6QX02d];@1:BTX0J6QX03De=@1WIfL0F5QH030`<01P
H600IFEU06ATI02YZJT0<C4a04m?C`1gMgL0IFEU055AD@1oOgl0<C4a069RHP12@T80HV9R03lo?`0S
8b<0A4A406ATI00o?cl0F5QH02LW9`0/;2`0GemO05MGE`1CDe<0<c<c038b<P1`L700CDe=03Hf=P1H
F5P0?Cdm09BDU01VIVH07alO038b<P1:BTX0Jf][08:2PP0o?cl0=CDe04m?C`1CDe<0@T9207YjNP0U
9BD0=cLg03Ph>01MGEd0J6QX08:2PP0S8b<0:b/[06a/K01^KVh0;2`/07moO`0g=cL0=SHf06a/K01h
N7P0<c<c07AdM010@400=SHf07YjNP1MGEd0:2PX0451@@12@T80?Shn05mOG`1VIVH0;bl_04Y:BP10
@400BTY:08N7Q`18B4P0;Rh^03Lg=`0h>3P09BDU05=CD`1[Jf/0820P02`/;014A4@0@d=305aLG01N
GUh05ADE04I6AP1iNGT>ooooo`0V05aLG01[Jf/0IVIV059BDP1MGEd0Ogmo07alO01dM7@0YJFU0;>c
/`20P800NGUi08f=S@2DU9@0W9bL08V9R@1cLg<0NGUi07MgM`1:BTX0E5AD069RHP1GEeL0EUIF06e]
K@1SHf<0EeMG07YjNP1PH600Jf][059BDP1ZJVX0PH6107MgM`1hN7P0TY:B0:VYZ@2GUiL205=CD`3:
06][J`1RHV80L71`051@D022PX80C4a<05eMG@1HF5P0Ph>305QHF01EEED0?3`l08:2PP0`<300>SXj
06MWI`1SHf<0EUIF069RHP1MGEd0Bd];04i>CP13@d<0@D51051@D019BDT0P82005=CD`1:BTX0E5AD
055AD@10@400Cdm?05MGE`0e=CD0@d=303Ti>@1?Cdl0BTY:030`<00d=3@08b<S02`/;00M7Ad0@410
02l_;`0k>c/071`L04=3@`0Q8B40Cdm?03<c<`0X:2P0?clo04e=C@0l?3`0>3Ph04M7A`0i>CT0B4Q8
04M7A`12@T80ATI605UIF@1;Bd/09BDU038b<P1TI6@0Bd];0492@P13@d<0Cdm?05aLG014A4@0?clo
034a<@0b<S809RHV04A4A00l?3`0BTY:03Lg=`1WIfL0Jf][02h^;P0];Bd0=3@d05]KF`1UIFD0KFe]
08b<S00/;2`0Bd];059BDP17AdL0>SXj030`<01MGEd0PX:205eMG@0e=CD0>3Ph03Ti>@1/K6`0JFUY
06YZJP1TI6@0>c/k0410@00g=cL0Bd];04=3@`26QXH0NWYj01`L702VYZH0IFEU03<c<`1KFe/0?clo
08V9R@1NGUh0F5QH02XZ:P1_Kfl0?clo03hn?P1@D500BDU905EEE@0d=3@0BDU902/[:`0l?3`0:b/[
03dm?@1EEED0G5aL03Hf=P0o?cl0>SXj01lO7`13@d<0?3`l059BDP0m?Cd0JFUY065QH@0a<C40EUIF
02DU9@0_;bl0A4A40410@015ADD0DE5A059BDP1KFe/0IVIV08n?S`0k>c/0Fe]K03Ti>@13@d<0CTi>
055AD@1iNGT0IVIV06e]K@1TI6@0@D5105=CD`1<C4`0Bd];061PH01<C4`0T92@096AT@1CDe<0Ti>C
03dm?@1IFET0=CDe08N7Q`12@T80LW9b05mOG`1_Kfl0M7Ad06IVIP1FEUH0Ph>304i>CP1]KFd0Rh^;
06UYJ@11@D40@d=305UIF@15ADD0E5AD06QXJ080A4A402809BDU03hn?P1<C4`0Hf=S069RHP1]KFd0
C4a<08R8R01DE5@0`l?306ATI01NGUh0EUIF07QhN00V9RH0<c<c04Q8B017AdL0:RXZ02h^;P15ADD0
<30`02<S8`16ATH0J6QX07MgM`1YJFT0[:b/07inOP21PH40U9BD05QHF01cLg<0cLg=4?ooool000ko
oooo05h0KFe]05MGE`1cLg<0OWin06EUI@15ADD0DU9B07AdM01QHF40LW9b038b<P13@d<0D51@04Y:
BP1iNGT0@T9204I6AP1/K6`0CDe=07UiN@1kNg/0>3Ph04Y:BP1_Kfl0BDU907]kN`15ADD0@41006EU
I@1HF5P0@d=3055AD@0d=3@0EEEE0:>SX`1iNGT0B4Q804E5A@10@400IfMW07inOP0^;Rh0=SHf04i>
CP1ADE40IfMW08F5Q@0X:2P0B4Q805aLG00k>c/0SXj>04i>CP0R8R80?clo06e]K@1=CDd0S8b<03dm
?@0R8R80B4Q806IVIP1FEUH0Ph>302`/;00b<S80?3`l04m?C`0h>3P0GEeM02XZ:P15ADD0Kfm_05=C
D`0j>SX0IfMW03Ph>015ADD0G5aL03dm?@0V9RH0FUYJ03lo?`1VIVH0I6AT01hN7P0P8200B4Q804];
B`1DE5@0DE5A01hN7P0`<300CTi>0P1KFe/09@0a<C404a<C03De=@1SHf<0IFEU03lo?`1BDU8071`L
04Q8B01LG5`0;bl_04I6AP1NGUh09bLW07UiN@1fMWH0<c<c03lo?`1CDe<0>3Ph071`L02KVi/0ATI6
02@T9017AdL0<C4a061PH01iNGT09bLW03Lg=`1NGUh0=3@d03Ti>@1dM7@092@T04E5A@1MGEd00P1C
De<0C@1JFUX0=3@d05ADE01ZJVX0A4A405IFEP1KFe/0<S8b06m_K`1[Jf/0BDU907QhN00a<C40A4A4
08N7Q`1CDe<0ATI604i>CP0`<300>CTi07QhN01kNg/0=cLg07YjNP0n?Sh0?clo07IfMP1mOGd0>c/k
05YJFP0m?Cd0AdM707YjNP16ATH0?Shn05eMG@1>CTh0FEUI08N7Q`19BDT0?clo05eMG@0l?3`0Rh^;
08f=S@0e=CD0AdM7055AD@1LG5`0Ogmo038b<P0f=SH0NGUi06m_K`1cLg<0Fe]K03Xj>P1HF5P0UIFE
05mOG`1KFe/0?3`l0820P01gMgL0>CTi051@D01:BTX0J6QX0;no_`1FEUH0BDU905MGE`1:BTX0I6AT
05UIF@10@400GUiN0080D51@0400Y:BT04I6AP1NGUh0Fe]K04E5A@1QHF40JFUY03Lg=`1SHf<0FUYJ
05=CD`2>SXh0@d=303@d=01KFe/0De=C05mOG`1gMgL0?3`l03Hf=P1BDU80JFUY04A4A01^KVh0=CDe
03/k>`1?Cdl0A4A4059BDP1dM7@0?Cdm04U9B@1cLg<0GUiN03lo?`1SHf<0<C4a0451@@1UIFD0Hf=S
03Hf=P1PH600A4A403Ti>@1iNGT0IVIV02DU9@10@400BDU904a<C01XJ6P0E5AD024Q8@0Y:BT0G5aL
05IFEP1fMWH0?3`l01XJ6P0W9bL0De=C04=3@`1bLW80CTi>3_ooool05P29RHT0T92@08f=S@1ZJVX0
Lg=c07MgM`1YJFT0GemO08f=S@2QXJ40CDe=04M7A`1kNg/0HV9R09bLW02<S8`0SHf=05]KF`1hN7P0
HF5Q04E5A@1GEeL206m_K`2Y05UIF@1]KFd0B4Q8051@D01FEUH0MGEe03Xj>P1NGUh0QHF50861P@1i
NGT0QHF50;Zj^P27QhL0AdM705eMG@1XJ6P0?3`l04I6AP1>CTh0IVIV04I6AP1LG5`0FEUI06UYJ@0n
?Sh0EEEE04i>CP1/K6`0@T9204A4A01`L700KVi^05mOG`1GEeL0MGEe071`L01KFe/0DE5A065QH@1J
FUX0DU9B06YZJP1hN7P0OWin05MGE`1QHF40IfMW03/k>`1IFET0C4a<0451@@1<C4`0EeMG04U9B@0h
>3P0C4a<038b<P0h>3P0:RXZ04A4A00i>CT0@D5102`/;012@T80;bl_055AD@0n?Sh0>CTi05QHF01G
EeL0ADE504U9B@18B4P0?Cdm04i>CP16ATH0D51@059BDP12@T80EeMG05UIF@1?Cdl0CTi>04E5A@1B
DU80GEeM04a<C01JFUX0IVIV05UIF@15ADD0Bd];030`<01IFET0@d=305UIF@1>CTh0K6a/0861P@1N
GUh0Ng]k05EEE@1VIVH0P8200451@@1=CDd0:RXZ059BDP1HF5P0>CTi04e=C@1>CTh0@4100451@@1>
CTh0?3`l05QHF01/K6`0O7al06e]K@0m?Cd0ADE5055AD@1MGEd0GUiN07MgM`0f=SH0NWYj07MgM`0_
;bl0V9RH05eMG@0o?cl0Fe]K03hn?P1gMgL0CTi>06MWI`0[:b/0EeMG03@d=00/;2`0ADE504M7A`16
ATH0<S8b04=3@`0a<C40?Shn03`l?00e=CD0GUiN061PH014A4@0NGUi04i>CP0e=CD0DU9B04E5A@1L
G5`0BDU906EUI@12@T80BTY:071`L00L71`0>3Ph0492@P0204E5A@0<05IFEP1HF5P0FUYJ06][J`1n
OWh0Bd];06=SH`0_;bl0AdM705eMG@1KFe/0MgMg0P1ADE40A@1`L700A4A404E5A@1GEeL0ATI6055A
D@1IFET0F5QH05mOG`1DE5@0UYJF03Xj>P1TI6@07AdM06UYJ@0S8b<0RHV908b<S01jNWX0I6AT07al
O01dM7@0T92@06a/K02EUID0JVYZ05eMG@1PH600EeMG06i^KP1HF5P0@D5105MGE`1=CDd0IVIV0451
@@1JFUX0ATI605]KF`1ADE40EEEE06EUI@1^KVh0ATI6096AT@1GEeL0FEUI05ADE02?Shl0;2`/04];
B`1HF5P0EUIF03Hf=P0g=cL0@d=304M7A`0h>3P0GEeM06ATI01nOWh0GemO07AdM01aLG40VYZJ07mo
O`1dM7@0NWYj0;Rh^00@ooooo`003_ooool0QP24Q8@0S8b<0:6QX@1<C4`0NGUi05]KF`1lO7`0=cLg
03`l?01oOgl0GUiN07EeM@0l?3`0<30`04A4A025QHD0FUYJ069RHP0R8R80?Shn06MWI`11@D40<30`
03Ph>00S8b<0IFEU06ATI00M7Ad0=3@d0410@02CTi<0O7al02l_;`0J6QX0@d=304Q8B024Q8@0CTi>
01PH600`<300ATI603Lg=`1LG5`0H61P01TI6@1<C4`0M7Ad0492@P1mOGd0LW9b02HV9P1CDe<0IfMW
055AD@1^KVh0H61P03Xj>P1gMgL0M7Ad04A4A022PX80<C4a03/k>`1KFe/0LW9b038b<P1lO7`0<S8b
02h^;P1MGEd0@d=303`l?01XJ6P09RHV03dm?@1CDe<0@41004Q8B00j>SX0?Cdm03Lg=`2ATI40CTi>
03lo?`1WIfL0ADE506QXJ02=SHd092@T03Xj>P1lO7`0@41005YJFP1mOGd04A4A03`l?01_Kfl0>CTi
05eMG@1FEUH04A4A034a<@1HF5P0?3`l05YJFP0d=3@0:RXZ04U9B@1lO7`0@T9204I6AP1<C4`07AdM
06YZJP2IVIT0;bl_04E5A@17AdL0<30`05eMG@24Q8@0<c<c0410@01WIfL0<C4a07EeM@2NWYh09bLW
01dM7@1hN7P0=cLg07IfMP16ATH09BDU0P1DE5@0@`1:BTX0OWin030`<00Q8B40=SHf05aLG019BDT0
G5aL01PH600l?3`0DE5A03`l?00V9RH0Hf=S028R8P0f=SH0Vi^K05iNGP0M7Ad0>SXj03lo?`1BDU80
ZZZZ04A4A00U9BD0=cLg05iNGP1/K6`0[Zj^02`/;00j>SX0@d=3061PH02UYJD0Ti>C02<S8`16ATH0
HF5Q06YZJP1bLW80E5AD04e=C@1UIFD0OWin05MGE`1WIfL0?Cdm04A4A02>SXh0J6QX03@d=01DE5@0
>3Ph07UiN@1oOgl0;2`/06IVIP1BDU80>SXj07IfMP0d=3@0ADE503lo?`0[:b/0M7Ad055AD@0j>SX0
0P1HF5P0<`1gMgL0Vi^K02TY:@12@T80G5aL08:2PP2o_kl0HF5Q01hN7P1OGel0I6AT09BDU01=CDd0
<C4a04E5A@1OGel0F5QH08F5Q@0O7al0CTi>06=SH`1^KVh0=CDe05UIF@0W9bL0AdM707AdM01VIVH0
<30`07moO`0e=CD0>SXj07AdM01ZJVX0:2PX06][J`10@400AdM708:2PP19BDT0;bl_055AD@0n?Sh0
B4Q80:VYZ@1:BTX0:2PX04i>CP1<C4`0Hf=S0:BTY00202LW9`0D04e=C@1QHF40L71`08J6QP0R8R80
7AdM04M7A`1^KVh0Bd];04E5A@1WIfL09BDU03lo?`1RHV80:2PX04Q8B00^;Rh06a/K03/k>`1bLW8>
ooooo`0>04Q8B01HF5P0LG5a05UIF@1MGEd0M7Ad0:b/[02XZ:P0^KVi0:b/[01FEUH0CDe=08b<S028
R8P20;2`/00n08b<S01BDU80PH6106MWI`1DE5@0EEEE06EUI@1:BTX0@d=306EUI@1NGUh0OGem06IV
IP20P800DU9B079bLP2:RXX0MgMg09nOW`2^[Zh0/;2`09ZJVP1KFe/0GUiN059BDP16ATH0FUYJ05EE
E@1jNWX0@D5106a/K01<C4`0QHF504a<C01UIFD0?Cdm08V9R@0_;bl0=SHf06i^KP1UIFD0@41006YZ
JP1UIFD0EeMG05IFEP13@d<0DU9B06i^KP16ATH0UYJF06a/K01cLg<0M7Ad06=SH`1VIVH0@T9205]K
F`0f=SH0A4A404U9B@1PH60203`l?00Q04];B`0/;2`0<S8b02XZ:P1BDU80>3Ph05EEE@0j>SX0ATI6
03De=@1]KFd0=SHf034a<@14A4@0J6QX04I6AP17AdL0CDe=03`l?01@D500BDU904i>CP1ADE40CTi>
05]KF`11@D40BDU9051@D01;Bd/0?3`l03dm?@1HF5P0N7Qh0080JFUY0:<0EEEE04m?C`13@d<0G5aL
06MWI`1VIVH0Jf][08F5Q@2FUYH0MgMg0:^[Z`1XJ6P0EeMG07MgM`1eMGD0SXj>038b<P1:BTX0P820
05ADE01JFUX0B4Q807EeM@23Ph<0JFUY055AD@1VIVH0EUIF05UIF@2@T900SHf=06IVIP10@400H61P
04Q8B01BDU80>3Ph09nOW`2HV9P07alO07=cL`1>CTh0<C4a05UIF@0h>3P0Cdm?0451@@1_Kfl03Ph>
02h^;P0_;bl0ADE506IVIP0[:b/0=CDe02l_;`15ADD06QXJ03<c<`0_;bl0@d=309:BTP1cLg<0=3@d
03Ti>@14A4@0:BTY04E5A@1DE5@0MWIf05IFEP1eMGD0LG5a03De=@1ADE409RHV04Y:BP1<C4`09BDU
03Hf=P17AdL0GEeM061PH01UIFD0MGEe03Lg=`2=SHd0Bd];06ATI025QHD0C4a<04e=C@1gMgL0J6QX
06MWI`1PH600De=C051@D01BDU80KFe]04a<C01QHF40JFUY07=cL`2i^KT08b<S05MGE`1>CTh0Kfm_
04I6AP1RHV80MWIf06a/K01CDe<0H61P07UiN@3Bd]80Ph>308b<S025QHD0KVi^09bLW01DE5@0JVYZ
05mOG`1QHF40EeMG05=CD`16ATH0;Bd]03lo?`0S8b<0N7Qh05EEE@1GEeL0Lg=c09>CT`1MGEd0ZZZZ
061PH01KFe/0DE5A08f=S@1EEED0T92@06MWI`18B4P0FUYJ06QXJ01@D500B4Q803/k>`16ATH0Lg=c
08j>SP1cLg<0_[jn09^KV`2GUiL0UYJF051@D01RHV80b/[:013ooooo000>ooooo`2105ADE01GEeL0
TY:B069RHP1KFe/0NWYj0<c<c01]KFd0=SHf05=CD`22PX80_;bl04E5A@0f=SH0;bl_04Y:BP23Ph<0
S8b<02XZ:P0i>CT0@41006QXJ02;Rh/0IVIV024Q8@1KFe/0HV9R05eMG@27QhL0A4A4038b<P12@T80
CTi>03Ti>@1fMWH08b<S034a<@1]KFd0;bl_06IVIP1XJ6P0;bl_03Ph>024Q8@0?3`l03Xj>P1TI6@0
8R8R03Ti>@1FEUH0:b/[05aLG00k>c/09bLW05UIF@1]KFd09bLW065QH@0l?3`0>3Ph06i^KP1?Cdl0
;bl_0451@@1PH600?Cdm06i^KP1RHV807QhN0451@@1]KFd0F5QH08Z:RP0J6QX0;Bd]05aLG01SHf<0
PX:207IfMP0N7Qh09BDU05=CD`1DE5@0F5QH03Lg=`0I6AT0=CDe04M7A`19BDT0:b/[03<c<`0Y:BT0
BTY:05IFEP0/;2`09bLW020P800N7Qh0T92@09VIV@0D51@0:2PX04A4A01=CDd0^;Rh0:NWY`0O7al0
BDU905EEE@1:BTX0[jn_09BDU00E5AD0=SHf03Ph>01<C4`0Ph>306a/K00J6QX0:b/[04];B`1KFe/0
De=C05aLG00F5QH0@D5106EUI@1@D500;2`/0080?Cdm08h0Hf=S07=cL`0U9BD0<C4a03dm?@1=CDd0
Ph>3051@D00I6AT0>c/k03lo?`1PH600NGUi01`L700[:b/0LW9b06=SH`1[Jf/0Ng]k01hN7P0Z:RX0
E5AD09^KV`1HF5P0LW9b034a<@0Y:BT0EEEE08R8R00f=SH0C4a<055AD@0o?cl0M7Ad04U9B@0_;bl0
M7Ad02HV9P14A4@0OGem04A4A00U9BD0JFUY02HV9P1<C4`0I6AT02l_;`1EEED0DE5A030`<022PX80
Kfm_03/k>`1=CDd0@d=3059BDP2g]kL0<C4a0492@P1XJ6P0Bd];0:NWY`1CDe<09BDU04M7A`1;Bd/0
QHF506e]K@0H61P0Cdm?061PH02=SHd0S8b<030`<01FEUH0RXZ:05aLG01QHF4061PH0451@@2@T900
PX:205UIF@1QHF409BDU065QH@2JVYX0?Shn03`l?00];Bd0@D5107MgM`1JFUX0<C4a05iNGP0o?cl0
ATI606e]K@0V9RH0<c<c055AD@1:BTX0@d=307AdM00G5aL0;bl_04M7A`1JFUX0De=C06][J`0c<c<0
@d=305aLG01DE5@0H61P06][J`0^;Rh0<C4a05ADE01IFET0FUYJ069RHP19BDT0;2`/06ATI01RHV80
BTY:05=CD`0d=3@07AdM04i>CP0n?Sh0:b/[055AD@0_;bl0;Bd]03lo?`1MGEd08B4Q04i>CP0i>CT0
=CDe0P1;Bd/00`0L71`0;Rh^055AD@0>ooooo`0R05EEE@21PH40YZJV06a/K01=CDd0JVYZ05mOG`1@
D500OGem08b<S02KVi/0YZJV0:FUY@2BTY80WinO05QHF01hN7P0EUIF06ATI01KFe/0@d=303Lg=`1;
Bd/0F5QH04E5A@1;Bd/0ADE505]KF`14A4@0Ng]k05YJFP1OGel0HF5Q069RHP80I6AT04H0UIFE06e]
K@0h>3P0FUYJ03@d=00c<c<0ADE50492@P1VIVH0B4Q804Y:BP13@d<0GemO06=SH`1@D500?Shn07=c
L`18B4P0GEeM07=cL`1RHV80BTY:06i^KP1XJ6P0CTi>05UIF@1<C4`0Cdm?07AdM01WIfL0KVi^06YZ
JP1hN7P0NGUi06a/K01JFUX0=CDe071`L00k>c/0?clo05IFEP1NGUh0DU9B04a<C018B4P0?clo03Lg
=`0/;2`0EEEE034a<@1>CTh0:b/[04=3@`0g=cL0L71`03/k>`0b<S80CDe=05aLG01BDU80D51@05AD
E015ADD0GEeM05=CD`1UIFD0F5QH04E5A@20P800ATI60P1>CTh0CP13@d<0E5AD06a/K011@D40BTY:
06i^KP17AdL0?3`l04M7A`0X:2P0De=C04A4A01CDe<0A4A406ATI01OGel0B4Q805IFEP13@d<0I6AT
08F5Q@2:RXX0^k^k04Q8B01TI6@0TY:B06YZJP1NGUh0IfMW05IFEP0o?cl0@T9202XZ:P1RHV80Hf=S
04i>CP2PX:00@d=304I6AP1?Cdl0L71`075aL@1nOWh0=cLg0410@017AdL07QhN0:^[Z`1=CDd0<c<c
06IVIP0e=CD0L71`03<c<`1>CTh04A4A04];B`0Y:BT0<30`04M7A`0m?Cd0?3`l0451@@1@D5006QXJ
03@d=00l?3`0:b/[04Y:BP1KFe/0@d=306UYJ@1LG5`0?Shn05mOG`0f=SH0BDU903Ph>080Bd];03<0
>CTi05ADE00P8200=SHf04E5A@0i>CT0?clo05]KF`0U9BD0?3`l08N7Q`2KVi/0BDU904A4A019BDT0
EeMG065QH@1@D500FEUI055AD@1:BTX0LG5a038b<P1:BTX0C4a<03`l?01DE5@0CTi>08j>SP1eMGD0
E5AD06QXJ017AdL0FUYJ01HF5P1eMGD061PH06][J`1fMWH0MGEe09RHV02=SHd0JVYZ08Z:RP1=CDd0
Rh^;07alO01aLG40EeMG04a<C01gMgL00P1TI6@09@1aLG40Fe]K05EEE@0/;2`0FEUI034a<@1^KVh0
JVYZ061PH01oOgl0Shn?06m_K`2BTY80T92@08V9R@2FUYH0QhN705eMG@1JFUX0F5QH05UIF@1MGEd0
QhN704U9B@1;Bd/0>3Ph0:VYZ@1^KVh0c/k>05=CD`1aLG40WIfM0=3@d026QXH0SXj>08>3P`2IVIT0
4?ooool000kooooo01H0GUiN06MWI`1YJFT0QhN707emO@18B4P0IVIV07emO@1PH600GEeM03Xj>P1Y
JFT0N7Qh04e=C@1WIfL0<S8b03/k>`1oOgl0@D5104i>CP13@d<06a/K0P1DE5@01P0c<c<0KFe]03De
=@0W9bL0QXJ606MWI`80A4A403T0>CTi04I6AP2PX:00JVYZ04=3@`11@D40@41007UiN@25QHD071`L
03Lg=`1PH600Jf][06IVIP1WIfL09RHV05MGE`1lO7`0ATI609^KV`18B4P05aLG04M7A`1mOGd0FUYJ
096AT@0M7Ad07QhN0410@026QXH0Hf=S08b<S00Q8B40;bl_03/k>`1WIfL0<30`05aLG00L71`0Bd];
05YJFP0f=SH08B4Q03Lg=`0];Bd0JVYZ071`L00G5aL0;2`/03lo?`1?Cdl0QhN704];B`0<30`09RHV
03Xj>P1@D5000P1NGUh0T`0?3`l0=CDe05EEE@1JFUX0IfMW03Ti>@0F5QH0<30`05MGE`1IFET0>SXj
04a<C00O7al0>3Ph06][J`0i>CT0@d=3051@D00[:b/0BTY:07]kN`10@400<30`05UIF@0j>SX0VYZJ
0:JVYP18B4P0;bl_03Ti>@12@T80IfMW09nOW`0K6a/0=cLg04U9B@14A4@0PX:204m?C`0_;bl0EeMG
06m_K`1DE5@0Rh^;04U9B@10@400EEEE05aLG01>CTh0Ogmo0410@00];Bd0I6AT06YZJP0g=cL0RXZ:
03Hf=P0];Bd0Shn?05MGE`0W9bL0B4Q803De=@0c<c<0J6QX0451@@0b<S80DU9B0451@@0/;2`0Hf=S
06m_K`0_;bl0De=C04=3@`1BDU80U9BD0410@00a<C40CDe=07moO`1QHF40`<30034a<@16ATH0EEEE
03/k>`1`L700Jf][01XJ6P0j>SX0IVIV04E5A@1BDU80?clo03Ti>@25QHD0KFe]05YJFP0h>3P0?3`l
07QhN028R8P0@D5104U9B@0];Bd0KVi^09fMW@1;Bd/0@T9202XZ:P1/K6`0VIVI0451@@0c<c<0@D51
03<c<`2;Rh/09BDU02TY:@1@D500F5QH05aLG02@T900;2`/05ADE01SHf<0M7Ad07EeM@1>CTh0=CDe
06EUI@1?Cdl0KVi^0;fm_@0];Bd071`L05QHF00k>c/0N7Qh09FEU@0/;2`06ATI06UYJ@0`<300De=C
092@T00202d];@0E04Y:BP0n?Sh0G5aL079bLP0W9bL09RHV0820P01_Kfl0;bl_05ADE00K6a/0=cLg
0861P@23Ph<0;bl_05QHF00];Bd0Cdm?0:f][@1WIfL06QXJ0080=CDe00l0C4a<0:NWY`1@D5006a/K
03@d=01SHf<0JFUY0;Zj^P1?Cdl08b<S03dm?@18B4P0=SHf04Q8B00^;Rh03_ooool0?P1ADE40OGem
0:RXZ01[Jf/0CTi>07YjNP1mOGd0L71`0;6a/@2GUiL0BTY:04E5A@1FEUH0FEUI08V9R@27QhL0PX:2
03Ph>01cLg<0I6AT04m?C`1=CDd0JVYZ0820P01:BTX0De=C06IVIP2FUYH0ADE505ADE01cLg<0O7al
06IVIP1DE5@0Ng]k0820P02>SXh0RXZ:05MGE`0l?3`0Bd];0492@P1>CTh0A4A405EEE@17AdL0FUYJ
0451@@1>CTh0ATI605QHF013@d<0EUIF03<c<`0_;bl0Fe]K06ATI01dM7@0DU9B069RHP1JFUX0FEUI
0P1BDU805@1GEeL0@41008N7Q`23Ph<0Kfm_06e]K@1JFUX0C4a<02XZ:P1RHV80JVYZ0492@P1>CTh0
BTY:03@d=00c<c<0>3Ph02DU9@0`<3005aLG03lo?`0204E5A@1l02DU9@0m?Cd0;bl_04U9B@12@T80
>3Ph04A4A01EEED0?3`l04E5A@19BDT0?Shn04I6AP18B4P0MWIf06QXJ015ADD0RHV904Y:BP1@D500
EUIF05=CD`1WIfL0O7al04e=C@1HF5P0Ogmo05UIF@13@d<0KVi^038b<P1cLg<0ADE505]KF`14A4@0
TI6A0:NWY`1mOGd0O7al05aLG01mOGd0R8R804e=C@16ATH0HV9R08f=S@1UIFD0@D5104E5A@0k>c/0
KFe]0410@01gMgL0<c<c07QhN01MGEd0=SHf0861P@1>CTh0EeMG05EEE@1FEUH0=3@d05mOG`0k>c/0
H61P07=cL`10@400XJ6Q05IFEP0T92@0EeMG03Xj>P1eMGD0=cLg07=cL`0A4A40Jf][05=CD`1?Cdl0
KFe]03hn?P0_;bl0>c/k059BDP13@d<0DU9B038b<P0J6QX0JVYZ05mOG`0_;bl0FEUI055AD@0Q8B40
?Cdm05=CD`1@D500<C4a055AD@0n?Sh0@d=307AdM00Y:BT0?3`l03De=@1>CTh0?3`l04M7A`1JFUX0
KFe]04Y:BP1IFET0820P079bLP0e=CD0ATI605YJFP1PH600Ti>C06EUI@1HF5P0Lg=c04m?C`80Fe]K
0440Bd];06e]K@1TI6@0HF5Q069RHP1@D500^;Rh038b<P23Ph<0>c/k079bLP16ATH0Shn?08b<S02A
TI40EEEE04U9B@1;Bd/0`L7107QhN02JVYX0QXJ605mOG`22PX80CTi>0:JVYP1RHV80F5QH05=CD`1@
D500?Cdm02`/;01<C4`0BTY:07AdM01TI6@0DE5A03Xj>P1<C4`0NWYj08j>SP1mOGd0I6AT059BDP1?
Cdl0>CTi03dm?@1?Cdl0?clo0451@@0m?Cd0GEeM05=CD`1NGUh0EEEE07alO027QhL0Q8B40<C4a022
PX80X:2P0:VYZ@2d];@0UIFE0;^k^`0@ooooo`003_ooool0<021PH40L71`08F5Q@1FEUH0LW9b0451
@@1RHV80ADE504a<C01]KFd0=SHf0451@@16ATH0@41007alO012@T80;bl_05UIF@1DE5@0NGUi07em
O@0I6AT0<C4a055AD@1dM7@0YjNW030`<00T92@0>c/k059BDP22PX80Jf][01lO7`0N7Qh0C4a<06MW
I`1oOgl0?Shn01`L701CDe<0M7Ad04I6AP1GEeL0FEUI02/[:`1RHV80LG5a04M7A`<0CTi>0;d0F5QH
06QXJ00^;Rh0AdM704=3@`0n?Sh0UIFE05IFEP0R8R80>3Ph02<S8`1ADE40^k^k05aLG00V9RH0EUIF
02l_;`0k>c/0UYJF02DU9@0^;Rh0DE5A04i>CP1DE5@0NGUi03Xj>P0i>CT0Bd];03dm?@25QHD0YjNW
02LW9`1HF5P0QXJ603@d=01DE5@0J6QX02LW9`0c<c<0GemO02d];@0T92@0EUIF024Q8@1EEED0GEeM
01PH600c<c<0=cLg04Y:BP1NGUh0GemO028R8P13@d<0ATI605QHF01oOgl0OWin02<S8`1HF5P0IFEU
05ADE010@400UYJF01dM7@12@T80De=C06MWI`1QHF40GEeM02@T900o?cl0O7al030`<01VIVH0KFe]
02PX:010@400SHf=03<c<`1bLW80E5AD02HV9P1OGel0V9RH02d];@1<C4`0<30`02l_;`1oOgl0V9RH
02d];@11@D40CTi>04e=C@2l_;`0NWYj02@T901>CTh0IVIV05EEE@2][Jd0@d=301hN7P1<C4`0F5QH
04e=C@2DU9@09bLW03Ph>00`<300ADE504a<C02IVIT05aLG03@d=017AdL0EUIF07=cL`1BDU807alO
03Ph>01YJFT0KVi^06MWI`0d=3@0;Rh^08j>SP2c/k<0Cdm?051@D00j>SX0GemO0;Zj^P19BDT0=cLg
069RHP1jNWX0Xj>S04A4A00K6a/0J6QX07AdM01NGUh0MgMg01lO7`0h>3P0HF5Q05]KF`1mOGd09bLW
02l_;`1hN7P0D51@055AD@1FEUH09BDU05ADE00o?cl0EeMG08V9R@0b<S80AdM70861P@1>CTh0OWin
03hn?P0[:b/0IVIV07moO`0b<S80HV9R03lo?`1>CTh0MgMg06IVIP0[:b/0BTY:02TY:@0l?3`0R8R8
05iNGP0_;bl0>SXj02`/;012@T80Rh^;03hn?P0X:2P00P11@D40201^KVh0/k>c03Xj>P0f=SH0?Shn
06a/K01`L700Ti>C0P0X:2P06P1?Cdl0PH6105MGE`1nOWh0:2PX024Q8@18B4P0TI6A03Lg=`1QHF40
820P02XZ:P0k>c/0IFEU02XZ:P0b<S80?3`l02LW9`1@D500GUiN01dM7@0h>3P0@d=306e]K@2UYJD0
GUiN3_ooool03`1WIfL0Q8B40:VYZ@1bLW80EUIF07QhN01oOgl0O7al08R8R02ATI40FUYJ071`L02E
UID0Fe]K0:2PX00208j>SP0404i>CP1QHF40Hf=S04m?C`80C4a<00L0De=C059BDP1?Cdl0>SXj04U9
B@1CDe<0FEUI0080CTi>02h0Jf][06UYJ@1jNWX0UYJF09VIV@21PH40FEUI04E5A@0b<S80Fe]K07Ee
M@1:BTX0Ogmo065QH@1iNGT0ATI606][J`1nOWh0IVIV03Lg=`1hN7P0<c<c03dm?@1PH600@41004];
B`1>CTh0DE5A038b<P0a<C40AdM70492@P1IFET0GEeM07YjNP1EEED0CTi>06=SH`1QHF40IfMW03Ti
>@1?Cdl0@T9204E5A@1BDU80GemO0P1=CDd02P11@D40>c/k03`l?00[:b/0BDU9038b<P14A4@0?clo
0492@P0a<C42055AD@0N03Xj>P12@T80F5QH04a<C012@T80CTi>04U9B@1>CTh0>3Ph05MGE`1<C4`0
Cdm?06QXJ01WIfL0GUiN04a<C01?Cdl0R8R808F5Q@16ATH0E5AD07AdM01IFET0A4A405IFEP1CDe<0
GEeM03/k>`1QHF40QXJ60P2UYJD0T`1eMGD0Ogmo075aL@1CDe<0Kfm_05UIF@1[Jf/0<C4a06MWI`2[
Zj/0HV9R06YZJP1ADE40P8200492@P1oOgl0=cLg061PH021PH40EEEE09ZJVP1JFUX0M7Ad04a<C023
Ph<0DU9B06ATI01WIfL0XJ6Q08j>SP0`<300QXJ6071`L01?Cdl0JFUY04m?C`29RHT0Hf=S0820P00E
5AD0=cLg02`/;00c<c<0Bd];03Xj>P0[:b/0>c/k06IVIP0G5aL08R8R03Ti>@0`<300UiNG07moO`0e
=CD0EEEE0451@@0g=cL0GEeM03Xj>P10@400=CDe08J6QP1<C4`0;2`/05MGE`0H61P0BDU903hn?P0e
=CD0>c/k061PH00l?3`0CDe=069RHP1kNg/0<c<c05YJFP0^;Rh0J6QX09>CT`11@D40CTi>06=SH`1J
FUX0MgMg0410@018B4P0I6AT04m?C`1<C4`0KVi^09nOW`2:RXX0B4Q8069RHP0V9RH0CTi>03Hf=P2>
SXh0@D51071`L01MGEd0DE5A079bLP26QXH0I6AT06YZJP1=CDd0N7Qh07moO`15ADD0E5AD03Ph>01c
Lg<0@d=306MWI`1;Bd/0NGUi05aLG00o?cl0G5aL02XZ:P1<C4`0>CTi04M7A`0h>3P0C4a<06i^KP2F
UYH0DE5A07YjNP2IVIT0Y:BT051@D016ATH0Cdm?06IVIP17AdL0?Shn03lo?`1IFET0<c<c05QHF01c
Lg<0V9RH06e]K@0209FEU@0509:BTP20P800GemO079bLP3Fe]H04?ooool000kooooo0?l0XZ:R0;^k
^`2KVi/0CTi>06IVIP1aLG40SXj>03De=@0m?Cd0I6AT06i^KP1fMWH08R8R03Lg=`0l?3`0@T9205UI
F@14A4@08R8R05=CD`1WIfL0<c<c061PH015ADD0:RXZ09nOW`1oOgl09BDU05UIF@0Z:RX0GemO08F5
Q@13@d<0<S8b04M7A`0I6AT0K6a/069RHP0P8200;Rh^04i>CP0m?Cd0EEEE06m_K`0L71`0?clo05=C
D`0e=CD0UIFE06][J`0f=SH0HV9R07EeM@1?Cdl0WIfM096AT@0X:2P0Jf][05EEE@1LG5`0MGEe02@T
900g=cL0EeMG07inOP0b<S80QHF504a<C00C4a<0Cdm?03hn?P1PH600I6AT01LG5`0g=cL0DE5A07in
OP1;Bd/0>SXj02TY:@1BDU80MWIf04M7A`13@d<0?clo04];B`1iNGT0QhN7020P800h>3P0D51@04Q8
B01RHV80VIVI01`L700Y:BT0KFe]04m?C`2<S8`0P82002TY:@14A4@0JFUY059BDP2[Zj/0KFe]01`L
7019BDT0KFe]04=3@`1LG5`0ADE5038b<P1DE5@0R8R803Xj>P1>CTh0E5AD04I6AP1FEUH0KFe]03@d
=012@T80C4a<03De=@2@T900Ph>3030`<00/;2`0M7Ad04a<C01dM7@0@T9202XZ:P0o?cl0MGEe07al
O01OGel0:b/[020P801>CTh0[:b/06QXJ01UIFD05ADE03dm?@1YJFT0Ng]k05UIF@1MGEd0;Bd]030`
<01BDU80S8b<02d];@1?Cdl0;Bd]03lo?`1^KVh0OWin02PX:01@D500@41007MgM`2MWId0CTi>03<c
<`1CDe<0IfMW08Z:RP2EUID06a/K02TY:@12@T80Bd];09JFUP1]KFd08R8R02/[:`19BDT0E5AD07em
O@1NGUh0@T9205EEE@1FEUH0D51@08n?S`1DE5@0KFe]06a/K00b<S80HV9R04E5A@0M7Ad0Hf=S04E5
A@16ATH0=3@d02`/;02?Shl0DE5A065QH@16ATH0=3@d07MgM`2BTY80@D5104m?C`0_;bl0H61P0;Rh
^01JFUX0;bl_04A4A01RHV80XZ:R04m?C`0S8b<0D51@05UIF@1]KFd0O7al01dM7@0l?3`0J6QX04m?
C`1PH600UYJF01/K6`0g=cL0ADE5051@D01]KFd0R8R802LW9`0e=CD0De=C07AdM01=CDd0QhN702l_
;`1;Bd/0Cdm?06IVIP1KFe/0QhN703<c<`13@d<0Fe]K03dm?@0];Bd0>CTi03Ph>017AdL000l0K6a/
02<S8`0W9bL0=CDe03`l?01DE5@0KVi^01LG5`0];Bd0CTi>05YJFP13@d<0G5aL05=CD`0L71`00P1M
GEd01P0n?Sh0LW9b05YJFP0N7Qh0FUYJ09JFUPkooooo00H0OWin07QhN028R8P0Lg=c07]kN`22PX82
09fMW@0H06MWI`1jNWX0GUiN08B4Q01JFUX0G5aL08R8R02TY:@0Rh^;055AD@1[Jf/0G5aL03hn?P1R
HV80EeMG05EEE@1CDe<0HF5Q051@D01KFe/0K6a/08B4Q01VIVH0EEEE0P1oOgl0;`1cLg<0Kfm_09VI
V@1nOWh0@d=304];B`0H61P09bLW0410@01>CTh0HF5Q0492@P1KFe/0Bd];05iNGP14A4@0D51@04A4
A01LG5`0E5AD05eMG@1PH600FEUI0861P@1SHf<0E5AD04E5A@16ATH0D51@04=3@`1EEED0Bd];05MG
E`1=CDd0I6AT07AdM01jNWX0F5QH03Xj>P1[Jf/0ADE504M7A`18B4P0@D5105aLG01>CTh0=cLg0080
ATI602D0<S8b05YJFP1?Cdl0CTi>03/k>`10@400<S8b04U9B@0h>3P09RHV03Hf=P15ADD0?Cdm038b
<P1=CDd0IFEU04Q8B00_;bl0EEEE04m?C`0m?Cd0FEUI04E5A@1BDU80EEEE05]KF`1HF5P0HV9R04m?
C`1CDe<0N7Qh059BDP0k>c/0C4a<02/[:`19BDT0<S8b0080E5AD09`0O7al08j>SP2ATI40[jn_06YZ
JP1>CTh0QXJ6069RHP21PH40I6AT06][J`2RXZ80FUYJ06IVIP1>CTh0Ph>306m_K`28R8P0>3Ph05]K
F`1CDe<0@T9207QhN022PX80PH6104Y:BP1iNGT0BTY:07YjNP0n?Sh0@D5104a<C00Y:BT0JFUY04a<
C00m?Cd0HF5Q04=3@`1HF5P0ATI606IVIP0U9BD0@d=304];B`0n?Sh0F5QH0451@@1BDU80>c/k05AD
E00Z:RX0;bl_02LW9`0^;Rh0Bd];04i>CP1EEED0N7Qh061PH00R8R80ADE506][J`1cLg<0@41004e=
C@11@D40CTi>08^;R`0f=SH0:BTY0410@01<C4`0?Shn04I6AP14A4@0CDe=05=CD`23Ph<0:b/[07Ad
M00b<S80<c<c04a<C01ZJVX0L71`05QHF016ATH0FEUI04E5A@17AdL0De=C04a<C016ATH0Cdm?05IF
EP1LG5`0O7al0:JVYP0X:2P0J6QX01HF5P1BDU807alO08n?S`1kNg/0Ph>304i>CP19BDT0?3`l06a/
K00o?cl0WinO07EeM@1CDe<0MGEe04A4A01`L700?clo05=CD`19BDT0>CTi02d];@0R8R80@41002/[
:`1=CDd0AdM7069RHP10@400P82007alO02BTY80EUIF07inOP1<C4`0QHF50410@017AdL0ATI604U9
B@0k>c/0ADE503@d=014A4@0;2`/06IVIP1/K6`0Ng]k07moO`2QXJ40LW9b07UiN@29RHT0LG5a05]K
F`2@T90@ooooo`003_ooool0N@1YJFT0Lg=c0:j^[P1XJ6P0Cdm?06IVIP2h^;P0LW9b04a<C01JFUX0
PH6108n?S`1=CDd0<c<c059BDP18B4P0MGEe05]KF`0Q8B40<c<c05MGE`1LG5`0X:2P04];B`0h>3P0
GEeM05iNGP1FEUH0[jn_03Hf=P0Z:RX0HV9R06ATI01kNg/0OWin02@T900W9bL0NGUi04Q8B01gMgL0
Fe]K02PX:00f=SH0GUiN02`/;01aLG40?Shn01LG5`0e=CD0EeMG03De=@1UIFD08B4Q02/[:`1ADE40
RHV904E5A@1`L7008B4Q05iNGP2IVIT0AdM702LW9`19BDT0>CTi034a<@1/K6`0Lg=c018B4P0j>SX0
B4Q808J6QP1dM7@04a<C02@T9018B4P0H61P08b<S02BTY804A4A04e=C@1ZJVX0CDe=05iNGP0h>3P0
5QHF04I6AP1cLg<0C4a<05]KF`1>CTh0:b/[04i>CP1lO7`0H61P04];B`0V9RH09BDU051@D01SHf<0
=3@d030`<00n?Sh0GemO07YjNP1dM7@08b<S02d];@1=CDd0De=C0;Ng]`2BTY806ATI030`<01]KFd0
IVIV0:NWY`2PX:006ATI04=3@`24Q8@00P1`L700@@0n?Sh0@T92055AD@1fMWH0DE5A065QH@13@d<0
F5QH075aL@2DU9@0BTY:05IFEP16ATH0DE5A07QhN01;Bd/07AdM06m_K`1EEED0A4A409RHV01YJFT0
;Bd]05MGE`1KFe/0AdM708R8R00`<3009BDU05]KF`1bLW80=SHf09ZJVP0G5aL09bLW05mOG`1HF5P0
CDe=092@T019BDT0HF5Q06ATI01=CDd0EeMG08^;R`0o?cl0EUIF08n?S`1MGEd0MgMg07emO@0P8200
D51@0:b/[00];Bd0H61P03`l?00W9bL0ADE507UiN@14A4@0EeMG03Ph>015ADD0LG5a0080C4a<0P1H
F5P0EP1mOGd0EEEE02LW9`1:BTX0C4a<08f=S@16ATH08R8R04U9B@1fMWH0VYZJ04E5A@11@D40ATI6
069RHP1[Jf/0KFe]03Lg=`1CDe<0MGEe04Y:BP1EEED0E5AD02TY:@1ZJVX0NGUi03Xj>P1aLG40=SHf
0451@@28R8P0CTi>04Y:BP1WIfL0;Rh^02TY:@1iNGT0@T9204a<C01WIfL09BDU03Lg=`1nOWh08b<S
038b<P1FEUH0=SHf06a/K02=SHd0=3@d04m?C`1DE5@0>3Ph06YZJP32`/80IfMW038b<P1SHf<0BDU9
05UIF@2ATI40FEUI03Lg=`1ZJVX0AdM705eMG@1cLg<092@T03<c<`1IFET0AdM7038b<P1CDe<09RHV
02<S8`13@d<0K6a/02l_;`17AdL09bLW03/k>`1DE5@0H61P02<S8`1;Bd/0>SXj3_ooool0B@27QhL0
P8200820P01_Kfl0JVYZ06IVIP15ADD0AdM706ATI01HF5P0C4a<05ADE01kNg/0Zj^[09jNWP1_Kfl0
TY:B08B4Q01bLW80IFEU05IFEP1BDU80FEUI06=SH`1KFe/0IVIV05YJFP1ADE40DU9B06EUI@1TI6@0
De=C07inOP1GEeL0RXZ:08>3P`20P800Ng]k05aLG01=CDd0J6QX03dm?@19BDT0@d=305aLG015ADD0
HF5Q04I6AP1>CTh0Bd];061PH01@D500IfMW03Ti>@0b<S80GemO069RHP0g=cL0CDe=07moO`1YJFT0
Fe]K04m?C`19BDT0I6AT04];B`1mOGd0O7al09JFUP1dM7@0FEUI06YZJP1=CDd00P1MGEd04@16ATH0
A4A404i>CP11@D40=3@d03De=@0N7Qh0<30`01LG5`0k>c/0:b/[04Y:BP0R8R80A4A4034a<@1]KFd0
ADE50080@d=300L0GEeM04e=C@14A4@0@D5102h^;P0o?cl0=cLg0080HV9R01H0?Shn0861P@1LG5`0
DE5A05MGE`1CDe<0KFe]06a/K015ADD0>CTi06QXJ01JFUX0@D5103Hf=P0[:b/0D51@03Ti>@1DE5@0
EUIF09:BTP2DU9@0EUIF0P1HF5P0U`17AdL0Q8B4069RHP27QhL0EeMG06MWI`2JVYX0WYjN079bLP14
A4@0IVIV06=SH`1WIfL0CTi>05QHF02BTY80Fe]K07emO@1RHV80BDU904A4A01fMWH0DE5A07QhN01;
Bd/0EeMG065QH@0h>3P0Jf][09FEU@17AdL0G5aL04I6AP1GEeL0MgMg06YZJP0K6a/0@d=306ATI012
@T80De=C0492@P14A4@0<S8b04=3@`0k>c/0>3Ph04A4A00`<300De=C05iNGP0/;2`0CDe=04A4A00I
6AT0?clo03@d=01CDe<09RHV07moO`1:BTX07alO05EEE@0H61P0:BTY02l_;`0`<300BTY:04e=C@1J
FUX0GUiN04Y:BP1nOWh0=CDe05EEE@0f=SH0<C4a04M7A`18B4P0IVIV06e]K@1XJ6P0I6AT04A4A01A
DE40FEUI04];B`0k>c/0De=C05EEE@1RHV80ATI606ATI01?Cdl0AdM70451@@2<S8`0<30`051@D01?
Cdl0De=C04A4A01ADE40CTi>09ZJVP1OGel0V9RH06m_K`1;Bd/0HF5Q03Ti>@1TI6@0CTi>075aL@1=
CDd0ATI603Hf=P0R8R80A4A405IFEP1aLG40CDe=051@D013@d<0@41005YJFP2[Zj/0G5aL08R8R01A
DE40Vi^K04a<C014A4@0E5AD05=CD`16ATH0@D5103`l?01CDe<0@T9207emO@1cLg<0Vi^K08f=S@2Y
ZJT0^KVi0;Ng]`2;Rh/0IfMW05=CD`2c/k<04?ooool000kooooo06P0L71`07moO`1_Kfl0MgMg05]K
F`1IFET0Kfm_05ADE01TI6@0JVYZ03`l?01SHf<0DU9B05EEE@1nOWh0C4a<0410@01@D500>c/k06UY
J@1HF5P0:2PX04A4A01@D500GemO09nOW`0h>3P092@T059BDP1CDe<0Hf=S03<c<`0Q8B40?clo06=S
H`0k>c/0HF5Q02h^;P0c<c<0GUiN06=SH`0b<S80HF5Q05=CD`0l?3`0PX:207moO`1IFET0IfMW06EU
I@15ADD0MgMg04A4A00/;2`0OWin059BDP19BDT0VYZJ03Xj>P0T92@0Fe]K06][J`1NGUh0Ph>302l_
;`0[:b/0DE5A05ADE00n?Sh0LG5a024Q8@0i>CT0Bd];03`l?00_;bl0OWin038b<P0k>c/0FUYJ04e=
C@1;Bd/0JVYZ02h^;P19BDT0J6QX03Xj>P18B4P0EUIF034a<@16ATH0CDe=02PX:01?Cdl0EUIF03<c
<`1ZJVX0@d=3020P800m?Cd0BTY:034a<@1@D500HV9R02`/;080EeMG0:40>c/k05YJFP2:RXX0:RXZ
04M7A`1EEED0@D5105=CD`29RHT0;Rh^04e=C@20P800CTi>05YJFP1bLW80<C4a04I6AP1bLW80?clo
069RHP12@T80:b/[04Q8B027QhL0:BTY05EEE@0l?3`0@T9207EeM@1XJ6P0AdM7065QH@0h>3P0;bl_
08f=S@1^KVh0B4Q806=SH`0h>3P0ATI609NGU`2EUID0AdM706IVIP17AdL0=SHf08n?S`1OGel07AdM
05YJFP12@T80De=C08N7Q`0h>3P05aLG04U9B@1BDU80EUIF075aL@0R8R80;Rh^07MgM`1IFET0IFEU
05QHF00[:b/0H61P09JFUP1;Bd/0Cdm?05eMG@0f=SH0NWYj07inOP10@400CDe=059BDP1?Cdl0O7al
03Ti>@0Y:BT0H61P04U9B@1nOWh0KFe]02TY:@1FEUH0JFUY05mOG`1lO7`0<30`06UYJ@1/K6`0Bd];
06MWI`0b<S80FEUI08Z:RP18B4P0JFUY05aLG00R8R80GEeM02l_;`1aLG40PH6103De=@1]KFd0OGem
0410@01]KFd0?3`l0410@021PH40QHF503dm?@1BDU80=SHf06EUI@2TY:@0I6AT03<c<`1QHF40A4A4
04a<C02ATI40?3`l02l_;`1IFET0>CTi04i>CP2>SXh0820P02@T9017AdL0CDe=08F5Q@2?Shl0:RXZ
02`/;013@d<0Kfm_05MGE`1lO7`0:b/[03@d=01FEUH0LG5a04I6AP1fMWH0:BTY02l_;`10@400A4A4
02@T901<C4`08B4Q03De=@1IFET0ADE502TY:@0204Q8B00903lo?`1UIFD0@T9201TI6@0f=SH0C4a<
02`/;01_Kfl0GUiN00kooooo0800EUIF06=SH`1bLW80HV9R04m?C`1ZJVX0MWIf06i^KP2RXZ80S8b<
06a/K01gMgL0OWin05mOG`1oOgl0PX:207alO01RHV80K6a/05aLG017AdL0CDe=04i>CP19BDT0Fe]K
06IVIP15ADD0FEUI05QHF01_Kfl0Hf=S05EEE@1gMgL0P820092@T020P800U9BD09>CT`1;Bd/0Cdm?
02`/;00g=cL0Cdm?04i>CP1iNGT0Fe]K06][J`1>CTh0K6a/05eMG@1YJFT0?3`l08n?S`18B4P0?clo
05eMG@1KFe/0Cdm?05ADE01aLG40EeMG05=CD`17AdL0D51@05MGE`1ADE40MGEe05IFEP1?Cdl0K6a/
06i^KP1WIfL0CDe=07inOP1>CTh0=3@d055AD@15ADD0@d=303`l?011@D40?Cdm03Xj>P0];Bd0EUIF
02l_;`12@T80=SHf04U9B@0f=SH0EEEE04M7A`0_;bl0>3Ph04U9B@17AdL0?3`l04U9B@1?Cdl0E5AD
03Hf=P1IFET0CTi>04e=C@1DE5@0@41004a<C01;Bd/0EEEE06m_K`1eMGD0Bd];04Y:BP1dM7@0GUiN
03Ti>@13@d<0>SXj04E5A@0_;bl0DU9B06EUI@1]KFd0OWin08N7Q`2IVIT0P82004i>CP80GemO09@0
Ng]k04];B`1>CTh0JVYZ065QH@1>CTh0=cLg05IFEP1KFe/0G5aL03Ph>01<C4`0EUIF04];B`1EEED0
FUYJ06UYJ@1:BTX0IVIV05EEE@1^KVh0D51@04e=C@1BDU80<C4a08n?S`20P800=cLg05UIF@1?Cdl0
M7Ad05MGE`1ZJVX08B4Q059BDP1@D500<30`04a<C00o?cl0FUYJ02l_;`10@400:b/[03<c<`0Z:RX0
=CDe069RHP1^KVh0?Cdm055AD@1GEeL0?Shn05aLG016ATH0HF5Q030`<01oOgl0IfMW04=3@`1NGUh0
9RHV03Ti>@14A4@0>SXj0451@@1GEeL0BTY:05aLG01FEUH0KVi^028R8P1SHf<0GUiN03Hf=P1LG5`0
C4a<05EEE@1KFe/0EEEE05iNGP1?Cdl0ATI603/k>`16ATH0E5AD04Y:BP1jNWX0M7Ad04e=C@2@T900
<c<c04];B`0a<C40FUYJ03hn?P1LG5`0De=C065QH@1OGel0HF5Q05ADE02EUID0Jf][09nOW`23Ph<0
MWIf06IVIP1?Cdl0NGUi03lo?`1QHF40?Cdm05iNGP1?Cdl0=cLg04E5A@19BDT0B4Q805MGE`1<C4`0
HF5Q059BDP1QHF40RXZ:06][J`21PH40GUiN07AdM01PH600Hf=S04Y:BP1@D500@D5104m?C`1DE5@0
G5aL02TY:@1^KVh0NGUi0:FUY@2<S8`0Xj>S0:^[Z`2d];@0QHF508R8R021PH40Y:BT4?ooool000ko
oooo00h0Jf][09BDU02`/;00DE5A051@D02>SXh0_kno04Y:BP0o?cl0Fe]K06QXJ02:RXX0<30`04I6
AP80EeMG0300SXj>05=CD`0Z:RX0>c/k05aLG01<C4`0XJ6Q04e=C@0_;bl0HF5Q069RHP1OGel0RXZ:
04I6AP0m?Cd0IFEU02d];@1GEeL0HV9R01dM7@1NGUh0RHV903lo?`1RHV80GUiN01hN7P1FEUH0Kfm_
03<c<`1^KVh0>c/k02HV9P1WIfL0N7Qh02d];@0i>CT0<S8b04M7A`2OWil0IfMW02h^;P1/K6`0Lg=c
065QH@22PX8061PH03Lg=`1HF5P208F5Q@0B09:BTP0R8R80<c<c04A4A01:BTX0UYJF075aL@0Y:BT0
;bl_071`L01iNGT0KFe]05UIF@0V9RH0EEEE06QXJ01CDe<0EeMG0P0c<c<0C`1VIVH0P82003Ph>00e
=CD0A4A403Ti>@2EUID0clo?04a<C00Z:RX0CDe=03hn?P2<S8`0LW9b02@T900m?Cd0A4A405MGE`2J
VYX0EEEE01DE5@0f=SH0De=C04Q8B01dM7@0;bl_02h^;P0d=3@0C4a<04A4A02OWil0J6QX02<S8`0k
>c/0D51@04Q8B01BDU80>CTi024Q8@1ZJVX0Q8B403Ti>@0f=SH0B4Q805YJFP2JVYX0F5QH01dM7@14
A4@0HV9R06ATI01mOGd0:b/[02`/;01ADE40XZ:R07inOP2m_Kd0:BTY02DU9@1?Cdl0SHf=07EeM@1k
Ng/0:2PX02<S8`1;Bd/0Ph>305eMG@1KFe/061PH04U9B@1gMgL0IfMW03<c<`1DE5@0ADE5030`<01]
KFd00P10@400K`1LG5`0@d=308>3P`1gMgL0;Rh^04A4A01/K6`0C4a<08N7Q`1@D500;bl_04I6AP1N
GUh0@41009:BTP1WIfL09RHV04=3@`1_Kfl0E5AD0;>c/`16ATH0IFEU0:NWY`1FEUH0MGEe03Lg=`0R
8R80JFUY04E5A@28R8P0=cLg02/[:`1`L700FUYJ05=CD`19BDT08B4Q05MGE`21PH40;Bd]04Q8B00X
:2P0IVIV0:NWY`1NGUh0@D5104A4A01ADE40Vi^K03`l?00Y:BT0ATI607AdM01oOgl0Lg=c024Q8@0l
?3`0@D5107EeM@1?Cdl0PH6102TY:@0m?Cd0Bd];05=CD`1JFUX0LW9b0410@01MGEd0MgMg03lo?`10
@400EUIF02LW9`1;Bd/0WinO05QHF00n?Sh0I6AT02XZ:P13@d<0MgMg04i>CP0c<c<0Bd];038b<P0k
>c/0I6AT03Ph>00O7al0IFEU03dm?@0n?Sh0LW9b03/k>`0N7Qh0@T9204A4A00g=cL0O7al06a/K00P
8200DU9B05iNGP1GEeL0J6QX05ADE00/;2`0CDe=05QHF00>ooooo`0L09nOW`2;Rh/0PH6107MgM`28
R8P0Ogmo07EeM@1gMgL0IFEU08J6QP1>CTh0LG5a079bLP1^KVh0IfMW08R8R02LW9`0EEEE08B4Q01c
Lg<0E5AD05IFEP1MGEd0AdM705IFEP1HF5P0DU9B05EEE@80D51@04/0G5aL04e=C@1oOgl0J6QX07al
O028R8P0RXZ:08B4Q00e=CD0<C4a02h^;P0W9bL0@T9204];B`1OGel0BDU905YJFP1ADE40Cdm?03Lg
=`1BDU80De=C04i>CP0c<c<0>c/k061PH01XJ6P0CDe=04M7A`1IFET0T92@059BDP0o?cl0M7Ad04=3
@`0a<C40?Cdm06ATI02:RXX0B4Q804=3@`1cLg<0NWYj07MgM`1bLW80IVIV05MGE`1CDe<0HV9R061P
H016ATH0Bd];059BDP0g=cL0ADE503/k>`1<C4`0>SXj03dm?@1ADE40N7Qh055AD@1OGel0Hf=S06AT
I016ATH0BTY:04Q8B011@D40BTY:03hn?P1^KVh0JFUY04A4A022PX800P1BDU80N00e=CD0?3`l05aL
G01NGUh0IFEU05QHF01XJ6P0Fe]K05IFEP0j>SX0;bl_05IFEP1>CTh0D51@04i>CP1lO7`0NGUi04E5
A@1ADE40HV9R04i>CP1WIfL0QhN709^KV`1FEUH0C4a<05MGE`1;Bd/0@41003lo?`1EEED0=SHf0492
@P0o?cl0BTY:06][J`1]KFd0QXJ606m_K`1=CDd0?3`l065QH@28R8P0PX:204e=C@1OGel0H61P03hn
?P2:RXX0KFe]02TY:@1HF5P0<c<c06YZJP1CDe<0GUiN02PX:01`L700CDe=01lO7`0`<300ADE505=C
D`0[:b/0=cLg02TY:@10@400AdM703/k>`15ADD0CTi>034a<@1<C4`0D51@02`/;00h>3P0?Cdm06e]
K@0^;Rh0N7Qh051@D00f=SH0K6a/02<S8`0_;bl0?Cdm03hn?P1>CTh0De=C04e=C@1NGUh0TY:B09^K
V`1NGUh0CTi>05UIF@1TI6@0OWin04E5A@1XJ6P0H61P05IFEP1@D500BDU905IFEP16ATH0CDe=05IF
EP1QHF40FEUI06QXJ01GEeL0SXj>051@D021PH409BDU061PH00T92@0QhN707]kN`80GUiN0340FUYJ
06][J`2NWYh0IfMW06YZJP2;Rh/0JFUY061PH01`L700Kfm_05UIF@23Ph<0Cdm?04E5A@1JFUX0?clo
04];B`0W9bL0I6AT04U9B@18B4P0DU9B04i>CP1oOgl0XZ:R04];B`1/K6`0LG5a07]kN`18B4P0FEUI
051@D01<C4`0HV9R05]KF`19BDT0HF5Q05=CD`2f][H0TI6A0;2`/02/[:`0aLG508:2PP2;Rh/0Ogmo
0:>SX`1gMgL0V9RH013ooooo000>ooooo`3o055AD@1hN7P0[jn_08^;R`18B4P0GEeM0;no_`2CTi<0
J6QX07MgM`1UIFD0ZJVY08F5Q@1HF5P0WinO05UIF@1/K6`0VIVI03lo?`1_Kfl0=3@d03Lg=`1FEUH0
Cdm?03@d=019BDT0<30`03`l?02@T900Bd];059BDP11@D40;Bd]07EeM@1/K6`07QhN04=3@`1=CDd0
@T9208Z:RP1nOWh0<c<c02DU9@1YJFT0EeMG0;6a/@1:BTX09RHV03Xj>P1IFET0TY:B08:2PP0B4Q80
:2PX0492@P1FEUH0H61P04=3@`0N7Qh0;bl_05=CD`1ADE40ADE502PX:00W9bL0OGem09>CT`1WIfL0
:RXZ05EEE@0K6a/0C4a<07YjNP0T92@0;Bd]055AD@1<C4`0MgMg06IVIP0G5aL0>c/k061PH01RHV80
TI6A03`l?00Q8B40EEEE08j>SP1SHf<0Q8B4055AD@0Y:BT0@T9205UIF@1cLg<0SHf=04Y:BP0S8b<0
F5QH0:BTY00k>c/0Cdm?03hn?P17AdL0K6a/0:n_[`12@T80A4A404I6AP1IFET0Zj^[096AT@0K6a/0
@41005=CD`1RHV80Shn?0::RXP0W9bL0>SXj04];B`19BDT0MgMg03`l?00U9BD0?Cdm069RHP13@d<0
IfMW04A4A01?Cdl0IVIV0;2`/01gMgL0QhN7038b<P1OGel0QHF504i>CP0];Bd0KVi^03Ti>@11@D40
VIVI07=cL`0U9BD0Bd];030`<01GEeL0MGEe03Ti>@0g=cL0H61P05MGE`1dM7@0VYZJ03@d=00W9bL0
BDU906ATI01JFUX0P82004I6AP0[:b/0DE5A04U9B@1IFET0Shn?02TY:@1XJ6P0IVIV04a<C021PH40
P82003/k>`1ZJVX0];Bd06ATI02NWYh0>3Ph03hn?P1:BTX0JVYZ04a<C01_Kfl09RHV04E5A@1hN7P0
Jf][07UiN@1FEUH0H61P0;jn_P29RHT0BTY:055AD@0e=CD0VYZJ06IVIP0f=SH0@41004];B`2YZJT0
JFUY02/[:`0m?Cd0CTi>04E5A@2PX:009RHV04i>CP29RHT0MGEe07YjNP1`L700:RXZ059BDP1PH600
IFEU09BDU00[:b/0@d=308Z:RP1GEeL0KFe]07inOP0T92@0;Rh^06QXJ00j>SX0A4A404Q8B00U9BD0
>3Ph06UYJ@0L71`0>3Ph04i>CP1?Cdl0De=C08F5Q@0c<c<09bLW04E5A@1VIVH0I6AT08b<S012@T80
<C4a05ADE015ADD0Jf][08f=S@0n?Sh0;bl_000G04e=C@1TI6@0A4A408Z:RP1?Cdl0;Bd]05UIF@1F
EUH0E5AD08:2PP0^;Rh0=CDe0451@@1JFUX0EEEE069RHP0M7Ad0;2`/04A4A01OGel0>SXj07QhN00Z
:RX03_ooool0501@D500EEEE06EUI@1PH600B4Q805eMG@1cLg<0RXZ:08N7Q`1jNWX0LG5a07moO`1G
EeL0Cdm?06EUI@1CDe<0IfMW05]KF`1QHF40GemO0P1ADE40401@D500EeMG05=CD`1CDe<0EUIF07If
MP1MGEd0EUIF06EUI@1ZJVX0N7Qh06i^KP24Q8@0VYZJ0:BTY02UYJD204M7A`0h06MWI`1BDU80HV9R
05EEE@24Q8@0EeMG06i^KP1BDU80M7Ad04m?C`1`L700@41009:BTP11@D40>c/k065QH@1FEUH0=cLg
03/k>`1IFET0DU9B04M7A`0k>c/0BDU904e=C@0j>SX0JFUY051@D019BDT0GEeM04a<C01NGUh0A4A4
051@D010@400;Bd]05MGE`1?Cdl0;bl_02`/;019BDT0=cLg02LW9`0c<c<0BDU902HV9P12@T80ADE5
04Y:BP0k>c/0GEeM04I6AP0e=CD0<S8b04e=C@17AdL204I6AP0]05=CD`1MGEd0CTi>05MGE`1=CDd0
HV9R06=SH`1JFUX0EUIF03De=@1<C4`0KVi^07=cL`0j>SX0AdM705QHF00m?Cd0>c/k03De=@0g=cL0
;Bd]02PX:00o?cl0?Cdm04M7A`1SHf<0JFUY0861P@2HV9P0MGEe071`L01BDU80Fe]K065QH@1ADE40
S8b<06EUI@1ADE40A4A405]KF`19BDT0N7Qh05MGE`1>CTh0Jf][0080Bd];0P1GEeL0P`1FEUH0K6a/
02l_;`1FEUH0@D5103Ti>@1JFUX0<c<c09FEU@1^KVh0>CTi05eMG@1LG5`0YJFU051@D01ADE408B4Q
0410@011@D409BDU03`l?00[:b/0G5aL04Y:BP1DE5@0>c/k04E5A@0X:2P07QhN05aLG01ADE40BTY:
069RHP1;Bd/0<C4a04];B`1:BTX0MgMg0451@@2c/k<0VYZJ03lo?`1YJFT0;2`/03`l?00m?Cd0>3Ph
051@D01HF5P0DE5A06e]K@1BDU80GemO02LW9`16ATH0D51@03De=@1CDe<0@d=304i>CP1<C4`0MgMg
05ADE01ADE40B4Q803/k>`1KFe/0H61P04];B`1NGUh0IVIV04I6AP2IVIT0?clo04e=C@0n?Sh0Hf=S
0410@01DE5@0EeMG06QXJ01UIFD0FEUI05ADE02>SXh0EEEE08>3P`2:RXX0U9BD08B4Q01GEeL0GEeM
03De=@1GEeL0DU9B05=CD`16ATH0>SXj05=CD`18B4P0K6a/0451@@0n?Sh0TY:B07]kN`1eMGD0Ph>3
05aLG01kNg/0D51@06i^KP1gMgL0QXJ606ATI01XJ6P0H61P07IfMP1VIVH0MGEe03dm?@1]KFd0Ng]k
0;no_`1aLG40OWin0:b/[02CTi<0EeMG07MgM`1CDe<0U9BD013ooooo000>ooooo`3N069RHP1KFe/0
HF5Q07AdM01XJ6P0BDU905mOG`1VIVH0GEeM06][J`0[:b/0C4a<05MGE`12@T80ZZZZ03lo?`0g=cL0
JFUY07=cL`25QHD0Bd];02PX:010@400DU9B07UiN@2SXj<07QhN03hn?P1CDe<0MWIf0;Vi^@0d=3@0
;Bd]0451@@1?Cdl0C4a<07IfMP0L71`0;bl_02`/;01EEED0=SHf05aLG00m?Cd0>CTi05EEE@1PH600
7AdM061PH018B4P0FEUI09bLW01@D500;bl_04A4A00`<300GUiN079bLP0N7Qh0<30`03Hf=P1BDU80
MgMg04A4A00F5QH05aLG03hn?P1hN7P0Fe]K07AdM00];Bd092@T0492@P14A4@0DE5A08J6QP0L71`0
DE5A05]KF`0_;bl0DU9B09bLW00i>CT0BDU905YJFP0E5AD0PX:204a<C00R8R80BDU908J6QP0g=cL0
C4a<05=CD`0Q8B40Cdm?04U9B@0V9RH0<c<c04A4A016ATH0WYjN04];B`0[:b/0A4A408j>SP1JFUX0
Ph>306i^KP0b<S80L71`08j>SP1<C4`0OWin08R8R00a<C40MgMg09^KV`13@d<0GEeM07AdM00W9bL0
GUiN06QXJ00o?cl0OWin05eMG@0W9bL0Kfm_07QhN00U9BD0G5aL034a<@0i>CT0WYjN07emO@0Z:RX0
GemO04Y:BP10@400SHf=061PH00a<C40DU9B07inOP1DE5@0][Jf05QHF013@d<0Ogmo05EEE@1jNWX0
e]KF038b<P0W9bL0Ng]k09^KV`1DE5@0L71`03Lg=`0];Bd0HF5Q05ADE01?Cdl0MgMg01LG5`14A4@0
QXJ6055AD@1HF5P0=3@d034a<@1PH600XJ6Q03Xj>P0m?Cd0<30`04U9B@2DU9@0QXJ602d];@13@d<0
Hf=S07=cL`2YZJT09RHV03Ph>01>CTh0KFe]0:>SX`1gMgL09RHV05MGE`1kNg/0IFEU07MgM`0S8b<0
H61P06QXJ01EEED0LW9b02PX:01OGel0/[:b05IFEP1CDe<0?Shn02@T901NGUh0EeMG05EEE@2DU9@0
@D5105iNGP1hN7P0BTY:0451@@0i>CT0H61P08N7Q`1[Jf/0>c/k0P12@T80=P1cLg<0TI6A03Hf=P0f
=SH0G5aL05iNGP1cLg<0S8b<02HV9P0];Bd0HF5Q04U9B@1kNg/0SXj>01dM7@12@T80AdM704U9B@1S
Hf<0GUiN01`L701EEED0HF5Q05aLG01@D500?3`l03De=@1=CDd0Ph>306i^KP0d=3@0Fe]K01lO7`0]
;Bd0FUYJ06ATI00J6QX0@T9202/[:`0_;bl0J6QX06IVIP0V9RH0@D5103dm?@1LG5`0_kno0410@00K
6a/0<30`03dm?@1KFe/0d];B04];B`kooooo00@0L71`06m_K`1]KFd0GUiN0P1FEUH09P1?Cdl0D51@
07MgM`2MWId0BTY:05iNGP1bLW80OWin0:JVYP22PX80RXZ:07QhN02IVIT0K6a/0410@01JFUX0Fe]K
04e=C@1?Cdl0BTY:0492@P14A4@0<30`069RHP1?Cdl0?clo05mOG`2VYZH0OGem05]KF`2>SXh0T92@
04M7A`0m?Cd0?clo034a<@0k>c/0BTY:0P1HF5P0201=CDd0@T9205QHF01QHF40GUiN0492@P1jNWX0
H61P0P1QHF40=01OGel0HF5Q05UIF@1KFe/0IFEU06YZJP1?Cdl0D51@06QXJ01MGEd0Ogmo05UIF@1C
De<0MgMg071`L01cLg<0CTi>06UYJ@1QHF40ADE505QHF01FEUH0BDU903De=@0n?Sh0>CTi02LW9`0Z
:RX0EUIF03<c<`1KFe/0A4A4055AD@1=CDd0J6QX04m?C`0c<c<0CDe=05eMG@1DE5@0A4A405ADE01c
Lg<0Hf=S04E5A@1_Kfl0FUYJ03Xj>P1QHF40AdM704Q8B015ADD305EEE@0@04Q8B01GEeL0Bd];03De
=@1>CTh0@T9203De=@0`<300ADE505ADE01HF5P0Jf][07QhN01TI6@0SXj>09VIV@80De=C0340D51@
05YJFP1?Cdl0IfMW07emO@1HF5P0EeMG04M7A`1^KVh0FEUI04U9B@1<C4`0GEeM059BDP1TI6@0Jf][
07QhN01cLg<0?clo05IFEP1?Cdl0J6QX04];B`1ZJVX0GEeM018B4P2LW9`0OGem03Hf=P1@D500;Rh^
04m?C`10@400EEEE00`<3016ATH0=SHf02l_;`0o?cl0=3@d055AD@1OGel0Kfm_01hN7P0b<S80;Bd]
02h^;P2ATI40Hf=S0080=SHf01/0FUYJ05MGE`1[Jf/0<c<c04i>CP10@400IVIV05IFEP17AdL0GEeM
02@T900Y:BT0@41003Hf=P11@D40MGEe04A4A017AdL0O7al09JFUP14A4@0=cLg03dm?@1YJFT0PH61
04=3@`1GEeL00P1>CTh0=01HF5P0BDU904Q8B00k>c/0ADE504Y:BP16ATH0VIVI08J6QP0n?Sh0GEeM
03<c<`1JFUX0?clo06UYJ@15ADD0HV9R06IVIP1?Cdl0OGem05aLG01;Bd/0GUiN05UIF@2<S8`0Lg=c
05EEE@1LG5`0LG5a0820P01=CDd0IVIV04M7A`1_Kfl0JFUY0492@P1KFe/0=SHf07=cL`1WIfL0=cLg
04M7A`0_;bl0JVYZ09JFUP2/[:`0YJFU08>3P`25QHD0ADE504U9B@16ATH203hn?P0?05ADE00d=3@0
EUIF030`<02ZZZX0Kfm_0:2PX02BTY80Vi^K0;2`/02j^[X0NGUi07alO01JFUX0YJFU013ooooo000>
ooooo`3o07AdM02UYJD0Vi^K06ATI02=SHd0Shn?0:RXZ00k>c/0Kfm_07YjNP2KVi/0Ogmo03Xj>P17
AdL0L71`09RHV01IFET0<S8b03hn?P1GEeL0MgMg03Lg=`1SHf<0>SXj05=CD`2>SXh0AdM702h^;P1C
De<09bLW06ATI01oOgl09BDU04a<C00e=CD0:b/[07=cL`1DE5@07QhN04E5A@14A4@0EeMG07]kN`1=
CDd0:2PX04e=C@1MGEd0ATI608J6QP1IFET0<C4a04M7A`1ADE40Hf=S07EeM@0N7Qh0DU9B06MWI`1B
DU80OWin07UiN@0J6QX0>c/k08b<S01;Bd/0KVi^06UYJ@0N7Qh0Cdm?069RHP0/;2`0EUIF03Ph>00U
9BD0DE5A05aLG019BDT0>SXj03`l?01:BTX0J6QX08J6QP0[:b/0CDe=05EEE@13@d<0TY:B0:>SX`0L
71`0A4A405UIF@19BDT0Q8B409ZJVP0<30`0;Bd]03dm?@1GEeL0Ph>306a/K00L71`0?Shn0492@P0_
;bl0C4a<02h^;P0_;bl0BDU907emO@0l?3`0I6AT03Hf=P0[:b/0B4Q807IfMP0R8R80A4A404M7A`0n
?Sh0Ogmo09ZJVP0[:b/0?Shn07IfMP1>CTh0UiNG0492@P0S8b<0:BTY075aL@17AdL0XZ:R03<c<`0X
:2P0>c/k09jNWP1>CTh0P82002HV9P0P8200?Shn0:f][@1QHF40KFe]01lO7`17AdL0B4Q807AdM00f
=SH0?Cdm02d];@15ADD0LG5a08N7Q`0/;2`0>SXj04i>CP1ADE40W9bL0410@00R8R80L71`05mOG`1f
MWH0][Jf02/[:`0M7Ad0JVYZ061PH02g]kL0A4A401PH600P8200KFe]05YJFP1hN7P07QhN0451@@0o
?cl0GUiN03Ph>01EEED0@D5105IFEP1HF5P0DU9B0492@P2JVYX0;Bd]07QhN029RHT08b<S04m?C`0a
<C40@d=306IVIP0h>3P0Ng]k03<c<`1KFe/0YZJV03`l?01<C4`0BTY:07inOP2OWil0EEEE01hN7P1>
CTh0Cdm?0820P02DU9@0;Bd]03Lg=`1kNg/0I6AT05ADE00_;bl0DU9B06=SH`26QXH0MGEe03hn?P0g
=cL0MWIf07inOP1VIVH0EUIF075aL@0k>c/0KVi^07alO00Z:RX0?3`l04a<C00c<c<0F5QH08Z:RP0l
?3`0<30`04Y:BP0d=3@0L71`09fMW@0[:b/08b<S06IVIP19BDT0H61P065QH@0S8b<0:b/[06i^KP1[
Jf/0FEUI000G09^KV`0[:b/07AdM04Y:BP1gMgL0ATI607MgM`0H61P08b<S04U9B@1JFUX0;Rh^05EE
E@15ADD061PH04e=C@1^KVh09BDU04m?C`0_;bl07AdM04];B`1UIFD03_ooool0;@23Ph<0MgMg05iN
GP1]KFd0NGUi08N7Q`1XJ6P0EeMG05iNGP1]KFd0C4a<05IFEP1RHV80MgMg08J6QP1VIVH0Lg=c04];
B`1PH600J6QX05]KF`1<C4`0K6a/075aL@1;Bd/0=cLg03hn?P13@d<0DE5A04A4A01SHf<0CDe=06][
J`1PH600OGem06m_K`1nOWh0OGem038b<P0S8b<0?3`l02/[:`17AdL0D51@03hn?P0204M7A`0603Ti
>@0j>SX0;bl_04a<C00m?Cd0ADE50P0X:2P0W@1;Bd/0BTY:03dm?@0e=CD0CDe=05UIF@15ADD0?Shn
06=SH`0n?Sh0=3@d03/k>`1YJFT0Q8B404U9B@1<C4`0KVi^08j>SP1KFe/0CDe=05=CD`1NGUh0LW9b
06ATI01<C4`0@41003Xj>P13@d<0:b/[03Lg=`0i>CT0Cdm?04I6AP10@400BTY:05EEE@0_;bl0?3`l
06][J`1LG5`0?clo04Y:BP0l?3`0?clo04m?C`0m?Cd0IFEU05MGE`16ATH0Jf][07]kN`1^KVh0BTY:
04m?C`1/K6`0PX:2059BDP15ADD0HV9R04Q8B01GEeL0C4a<030`<017AdL0=cLg04I6AP0`<300OGem
0861P@1LG5`0Y:BT08>3P`1OGel0KFe]06i^KP2=SHd0@D5105]KF`1WIfL0HV9R04];B`19BDT0G5aL
038b<P1WIfL0Cdm?04];B`1KFe/0GEeM04a<C01ADE40@d=3065QH@1[Jf/0GEeM06a/K01TI6@0QXJ6
08F5Q@1LG5`0MWIf07moO`0b<S80JVYZ04];B`1XJ6P0BTY:05QHF00k>c/0Fe]K03@d=00U9BD0?clo
03@d=01EEED0?3`l05EEE@0];Bd0:b/[02@T900g=cL0>3Ph03lo?`0m?Cd0C4a<03lo?`0[:b/0ATI6
0410@01KFe/0ATI606UYJ@1VIVH0ADE507inOP0R8R808b<S04I6AP19BDT0H61P04];B`0i>CT0ADE5
03`l?017AdL0;bl_03Ph>01OGel0=cLg055AD@1UIFD0P82007=cL`1LG5`0Kfm_03@d=012@T800P1A
DE40:@1HF5P0D51@04A4A01BDU80AdM708N7Q`1YJFT0R8R802h^;P1>CTh0=CDe07]kN`1bLW80De=C
04A4A019BDT0Bd];075aL@0l?3`0FEUI06UYJ@1GEeL0M7Ad06MWI`21PH40AdM705=CD`10@400DE5A
05=CD`0g=cL0?Shn03<c<`19BDT0=SHf04];B`1SHf<0EUIF09^KV`2UYJD0@D510080?clo00d0IVIV
04E5A@1XJ6P0D51@04Q8B01MGEd0UYJF05EEE@1@D5009BDU06m_K`2;Rh/0WYjN0080];Bd00H0TY:B
08^;R`22PX80LG5a04];B`27QhL@ooooo`003_ooool0o`1>CTh0M7Ad09BDU01GEeL0@41005EEE@2;
Rh/0@T920410@012@T80C4a<07inOP17AdL0>c/k04Q8B016ATH0LG5a03lo?`0W9bL0?Shn04M7A`1K
Fe/0PX:204A4A00^;Rh0JVYZ069RHP1OGel0QhN702@T901IFET0SHf=04Q8B02OWil0JVYZ028R8P10
@400KVi^04M7A`2=SHd0B4Q803<c<`1>CTh0Bd];05EEE@2`/;00<C4a03Hf=P1cLg<0E5AD05mOG`1]
KFd07QhN055AD@2MWId0QHF503Hf=P16ATH0:BTY05EEE@2PX:00C4a<03Xj>P11@D40CDe=06a/K029
RHT0;2`/03Hf=P1<C4`0N7Qh0;Rh^01bLW807alO02LW9`1GEeL0JVYZ0;Zj^P1GEeL04Q8B03Xj>P1^
KVh0JFUY0<O7a`0/;2`09RHV0451@@1RHV80IVIV06EUI@0d=3@0;bl_05YJFP2:RXX0FUYJ05MGE`0a
<C40;bl_071`L01cLg<05ADE03@d=018B4P0Fe]K09bLW00n?Sh0:BTY04Y:BP1CDe<0HV9R0;bl_01<
C4`07alO04M7A`23Ph<0GemO0;jn_P0k>c/0820P051@D01LG5`0@41006YZJP0N7Qh0B4Q805iNGP1F
EUH0:RXZ04I6AP0n?Sh0FEUI06UYJ@1FEUH08b<S05EEE@0l?3`0JFUY09jNWP0g=cL09bLW03Lg=`11
@D40J6QX092@T00Q8B40>SXj06QXJ01HF5P0LG5a08N7Q`0K6a/0A4A405UIF@1TI6@0E5AD09FEU@0b
<S80CDe=04A4A01]KFd0@41009:BTP0f=SH0=3@d05IFEP25QHD0CTi>0861P@0o?cl0Ogmo07IfMP0b
<S80>CTi04];B`0m?Cd0OWin04];B`0Y:BT0EUIF04a<C01:BTX0NWYj07]kN`0f=SH0D51@04A4A01Z
JVX0clo?03@d=01NGUh0MGEe04Y:BP27QhL0DE5A02HV9P1WIfL0Bd];0:f][@0];Bd0=SHf06QXJ01;
Bd/0L71`034a<@0V9RH0JVYZ06i^KP0Q8B40@D5102TY:@1KFe/0Shn?04U9B@1?Cdl0De=C051@D01n
OWh09BDU034a<@1>CTh0=CDe065QH@1/K6`05ADE03`l?01@D500B4Q804e=C@24Q8@07AdM03Xj>P1O
Gel0B4Q807IfMP1FEUH030`<03De=@1iNGT0ATI606a/K029RHT09BDU04U9B@2>SXh0LW9b08>3P`2=
SHd0BDU905eMG@2;Rh/0;Rh^06MWI`2NWYh0B4Q804a<C0005`2KVi/0IfMW03Hf=P1hN7P0>3Ph03Ti
>@1]KFd0@d=302<S8`1QHF40;Bd]03Xj>P1MGEd0Cdm?01hN7P0b<S80@41003lo?`25QHD0FEUI020P
800`<300De=C00kooooo0<T0IVIV06UYJ@2<S8`0Ogmo04a<C01_Kfl0O7al06UYJ@2ZZZX0Y:BT055A
D@1NGUh0EEEE06YZJP2ATI40SXj>092@T01DE5@0I6AT05iNGP1ADE40D51@05aLG01KFe/0Bd];04I6
AP1>CTh0EEEE06QXJ01@D500R8R807UiN@2OWil0MWIf0:2PX02<S8`0_Kfm0<?3``0o?cl0@d=307=c
L`11@D40@T9205iNGP1TI6@0EeMG06MWI`17AdL0DU9B04i>CP1lO7`0=SHf07AdM017AdL0@d=3061P
H01<C4`0DE5A04A4A013@d<0;bl_04=3@`0l?3`0=CDe0492@P11@D40IVIV03Ti>@0O7al0DU9B05]K
F`12@T8071`L03Xj>P12@T80=3@d03dm?@0b<S80<30`02XZ:P10@400>SXj034a<@0[:b/0?clo038b
<P0k>c/0@d=304Y:BP0h>3P0DU9B03Ph>00];Bd07alO03`l?00m?Cd0<c<c03Lg=`0l?3`0Cdm?04i>
CP1?Cdl0D51@05iNGP1ADE40;bl_0410@00m?Cd0GemO069RHP1XJ6P0B4Q805ADE01<C4`0ATI605]K
F`1?Cdl0CTi>059BDP0o?cl0FEUI0492@P1[Jf/0LW9b05EEE@1XJ6P0IVIV055AD@1GEeL0DU9B04m?
C`1DE5@0DU9B075aL@1KFe/0EeMG04U9B@24Q8@0Hf=S06e]K@0o?cl0ATI605mOG`1GEeL0GemO06YZ
JP1WIfL0BDU904I6AP1=CDd0PH6104];B`17AdL0F5QH01LG5`2=SHd0SXj>02/[:`19BDT0KFe]08Z:
RP1SHf<0IVIV02DU9@0c<c<0;Rh^03@d=013@d<0?Shn04Y:BP1DE5@0GUiN03lo?`1=CDd0@41002DU
9@1?Cdl0C4a<03Hf=P1PH600DU9B02d];@17AdL0?Cdm05EEE@0c<c<0Ng]k079bLP0i>CT0E5AD01TI
6@0k>c/0BTY:02h^;P10@400FUYJ055AD@1SHf<0E5AD05mOG`1IFET00P1HF5P08017AdL0Cdm?03Hf
=P1BDU80FUYJ04a<C01IFET0>c/k04E5A@0e=CD0CTi>06ATI013@d<0C4a<05aLG017AdL0SXj>03<c
<`1=CDd0AdM705UIF@17AdL0MGEe05ADE01BDU80JVYZ06a/K01PH600C4a<03Hf=P1/K6`0F5QH0P1b
LW805P1DE5@0F5QH04I6AP1]KFd0?Cdm06m_K`1XJ6P0>SXj04m?C`0N7Qh0De=C05mOG`0n?Sh0PH61
04E5A@18B4P0L71`05QHF01TI6@0EeMG06ATI019BDT205=CD`0A065QH@1ADE40EEEE051@D01eMGD0
BDU907alO01mOGd0W9bL06ATI01cLg<0_;bl0<S8b01ZJVX0[Jf]06m_K`2LW9`04?ooool000kooooo
0?<0FUYJ059BDP1cLg<0R8R804Q8B01EEED0K6a/092@T02NWYh0IVIV04];B`1=CDd0G5aL07UiN@2I
VIT08B4Q04M7A`1SHf<0ADE507AdM00o?cl09bLW03dm?@1ADE40GEeM059BDP0c<c<0@D510861P@1R
HV80CTi>03Ti>@0Y:BT0NWYj09jNWP0`<300=cLg02`/;01ADE40`L7106][J`0E5AD0=CDe03lo?`1L
G5`0VIVI07MgM`0M7Ad0B4Q8061PH01`L700R8R802l_;`0c<c<0H61P09VIV@1WIfL07AdM02/[:`1B
DU80Hf=S08n?S`1hN7P08R8R02HV9P1EEED0BTY:03De=@15ADD0A4A401dM7@1FEUH0J6QX02HV9P0i
>CT0EeMG02PX:01HF5P0OGem01LG5`0`<300D51@03Ti>@1iNGT0FUYJ01LG5`0[:b/0=SHf03Xj>P2<
S8`0J6QX01LG5`0c<c<0Cdm?075aL@2IVIT09BDU03hn?P1HF5P0IFEU04a<C01cLg<0>c/k04E5A@1R
HV80F5QH03Lg=`2JVYX0CDe=0410@01/K6`0KVi^02DU9@1<C4`0GEeM03Ti>@2JVYX0UIFE03De=@1=
CDd0E5AD03<c<`29RHT0MGEe02<S8`1>CTh0FEUI04U9B@23Ph<0JVYZ0410@01VIVH0JVYZ07inOP2f
][H0>3Ph03Xj>P2<S8`0QHF504m?C`2HV9P0<c<c03hn?P1`L700Cdm?03`l?02KVi/0:b/[05UIF@29
RHT0PH6108Z:RP1cLg<0>CTi03/k>`2>SXh0EUIF03Lg=`1OGel0<c<c051@D02<S8`0BDU903`l?00n
?Sh0EEEE05IFEP2j^[X0=cLg04m?C`1]KFd0EeMG08n?S`1TI6@0;Rh^07inOP1`L700BTY:08J6QP1:
BTX0;Rh^069RHP2HV9P0Cdm?07inOP0_;bl0@D5105]KF`1MGEd0V9RH038b<P18B4P0CTi>06QXJ011
@D40B4Q803hn?P2LW9`0Kfm_04A4A01_Kfl0DU9B0:^[Z`2TY:@0M7Ad04a<C01IFET0DE5A09jNWP0H
61P0ADE506YZJP1NGUh0OGem07AdM00e=CD0H61P04A4A01lO7`0];Bd02<S8`1ADE40^KVi04E5A@1g
MgL0L71`034a<@1@D500Jf][03Ti>@1MGEd0JFUY02`/;01ADE40PX:201`L701BDU80;2`/04Y:BP1M
GEd0U9BD03/k>`0n?Sh00`0i>CT0801NGUh0BDU903dm?@18B4P0@d=308R8R02][Jd0B4Q802h^;P1<
C4`0KFe]06UYJ@2HV9P0?3`l03@d=01>CTh0KFe]04Y:BP2g]kL0?Shn02d];@1:BTX0Fe]K059BDP1o
Ogl07AdM03Hf=P0l?3`0F5QH03dm?@1MGEd0820P3_ooool03`1gMgL0PH6107IfMP1RHV80E5AD07Mg
M`2LW9`0PX:20::RXP2SXj<0TY:B09JFUP1eMGD0`<300<O7a`0308f=S@1e09>CT`26QXH0DU9B07al
O01UIFD0NGUi071`L01TI6@0D51@061PH01KFe/0LG5a07]kN`1DE5@0QHF50861P@2DU9@0SXj>0<G5
a@3<c<`0@d=303/k>`18B4P0<S8b05iNGP1RHV80JFUY06QXJ01TI6@0AdM705QHF01MGEd0Ng]k03Lg
=`1jNWX0HV9R05MGE`1^KVh0IVIV05]KF`1CDe<0@T920451@@1bLW80DU9B05QHF01@D500BDU906MW
I`1@D500De=C06UYJ@1BDU80EEEE04Y:BP1LG5`0HV9R0410@01KFe/0EEEE04I6AP0c<c<0@41004M7
A`0e=CD0:RXZ05YJFP12@T80Hf=S04Q8B01BDU80Bd];05]KF`12@T80@d=305EEE@1GEeL0@d=303De
=@0f=SH0<c<c04I6AP0[:b/0C4a<04U9B@0U9BD0Bd];034a<@11@D40;bl_038b<P1?Cdl0AdM703/k
>`13@d<0BDU903lo?`1>CTh0<c<c02@T900k>c/0EUIF04a<C01RHV80UYJF09fMW@2OWil0gMgM0:ZZ
ZP1SHf<0JVYZ06UYJ@28R8P0OWin059BDP2<S8`0R8R80080J6QX08d0Cdm?02h^;P0o?cl0AdM707=c
L`0h>3P0Cdm?06=SH`1WIfL0Kfm_04Y:BP1XJ6P0?clo055AD@1MGEd0P82008Z:RP0@4100U9BD0;:b
/P1;Bd/0K6a/03Xj>P1BDU80E5AD059BDP0H61P0D51@04Q8B00V9RH0>c/k038b<P1ADE40;Rh^04I6
AP0X:2P0;bl_02`/;018B4P0CDe=05QHF00i>CT0BTY:06IVIP11@D40Fe]K03/k>`18B4P0BTY:08b<
S01oOgl0AdM706YZJP0R8R80:BTY04M7A`0l?3`0BDU906QXJ00e=CD0BDU90492@P1HF5P0GEeM034a
<@1<C4`0B4Q8079bLP0e=CD0EEEE071`L010@400M7Ad04i>CP1KFe/0F5QH05]KF`1OGel0EUIF079b
LP1nOWh0:RXZ06UYJ@0i>CT0EUIF03<c<`1eMGD0;2`/065QH@1GEeL0H61P0492@P15ADD0C4a<05UI
F@16ATH0DE5A06a/K01TI6@0TY:B08V9R@2[Zj/0BDU904Y:BP0/;2`0;Bd]03Hf=P0`<300GemO04E5
A@1WIfL0I6AT04M7A`1KFe/07alO08R8R02[Zj/0M7Ad075aL@1RHV80VYZJ04Y:BP1;Bd/0EEEE055A
D@1DE5@0FEUI06IVIP1[Jf/0=cLg0;Jf]P24Q8@0]KFe08n?S`26QXH0RXZ:09>CT`1oOgl0H61P05]K
F`1aLG404?ooool000kooooo00@0RXZ:09RHV01mOGd0IfMW0P1bLW802`29RHT0@D5105iNGP1gMgL0
HF5Q079bLP0f=SH0EeMG07MgM`1>CTh0GemO0080=cLg0=00I6AT05eMG@0h>3P0H61P03dm?@1]KFd0
HF5Q03Lg=`0l?3`0DE5A05IFEP1cLg<0?Shn028R8P19BDT0GemO03hn?P1fMWH0:2PX028R8P1PH600
IVIV03/k>`1`L700Fe]K04A4A01MGEd0C4a<03Ti>@1_Kfl0@T9205=CD`1OGel0@d=305QHF01UIFD0
820P04Q8B01`L700?3`l05MGE`1EEED0;Rh^05]KF`22PX80=SHf05]KF`1ADE40>SXj061PH01SHf<0
;Bd]05UIF@17AdL0?3`l07AdM02UYJD0=CDe04];B`1:BTX0<30`06=SH`2LW9`0<30`05eMG@1GEeL0
A4A408b<S02>SXh08B4Q03dm?@1PH600D51@079bLP1RHV8061PH04Q8B01<C4`0DE5A07IfMP19BDT0
;bl_059BDP16ATH0EUIF07moO`0];Bd0AdM7069RHP1jNWX0ATI607IfMP11@D40BDU9061PH01^KVh0
@41007moO`1TI6@0JFUY07]kN`1nOWh0>CTi04M7A`1VIVH0EUIF09NGU`1GEeL0:RXZ04];B`1[Jf/0
EUIF08b<S00c<c<0>SXj06ATI01OGel0K6a/07emO@0g=cL0?Shn06MWI`23Ph<0E5AD06i^KP1:BTX0
De=C08b<S01/K6`0ATI606][J`0n?Sh0IFEU09>CT`1aLG40;Bd]061PH01KFe/0G5aL08j>SP1;Bd/0
<c<c06YZJP1FEUH0TY:B08B4Q00X:2P0>CTi06YZJP1<C4`0W9bL05eMG@0`<300>c/k06ATI01ADE40
Jf][03Ph>010@400De=C07QhN01?Cdl0GEeM055AD@1HF5P0OWin0410@01?Cdl0GemO03Ph>02_[jl0
Q8B403/k>`1NGUh0=cLg04a<C01JFUX0<c<c06i^KP14A4@0H61P08j>SP1:BTX0GemO07YjNP1lO7`0
TY:B08>3P`15ADD0I6AT0410@02@T900KVi^038b<P1MGEd0OGem05iNGP1PH600:BTY0492@P23Ph<0
Fe]K06a/K01=CDd0D51@06a/K02SXj<0C4a<0P1^KVh0<@13@d<0L71`09NGU`11@D40AdM706ATI011
@D40LG5a06EUI@0];Bd0B4Q805UIF@1>CTh0IfMW06UYJ@0`<300>SXj05EEE@0h>3P0NGUi075aL@0N
7Qh0<c<c075aL@1BDU80Fe]K08R8R00`<30092@T05QHF01@D500=CDe06MWI`0C4a<0820P05MGE`1R
HV80Cdm?06i^KP0e=CD0<c<c05=CD`19BDT0:b/[07EeM@0j>SX0BDU906QXJ00a<C403_ooool0:@1l
O7`0PX:208>3P`2;Rh/0MGEe07MgM`22PX80OWin07EeM@27QhL0N7Qh07inOP1HF5P0GEeM096AT@2@
T900UYJF04a<C01LG5`0HV9R05YJFP1<C4`0CDe=075aL@1FEUH0KFe]05IFEP1DE5@0DE5A05=CD`1V
IVH0CTi>06EUI@1CDe<0NWYj07UiN@2EUID0MgMg04i>CP17AdL0ADE50080ATI607H0De=C05YJFP1Q
HF40DE5A03lo?`15ADD0B4Q805EEE@0l?3`0DU9B04e=C@1ADE40E5AD05=CD`1nOWh0HV9R04U9B@16
ATH0F5QH04a<C01@D500FUYJ05]KF`1UIFD0EUIF059BDP1HF5P0DE5A05IFEP0d=3@0AdM704a<C011
@D40H61P05eMG@1EEED0@T9204A4A00k>c/0@41002/[:`17AdL0@T9205IFEP1CDe<0H61P04=3@`1G
EeL0E5AD051@D012@T80C4a<04e=C@0o?cl0?3`l03lo?`1?Cdl0?clo06ATI01GEeL0B4Q806IVIP16
ATH0FEUI04Q8B016ATH0EUIF061PH00X:2P0?clo04=3@`0`<300ATI603lo?`0l?3`0BDU903De=@1<
C4`0E5AD06MWI`1bLW80EEEE06m_K`1/K6`0BDU905aLG01GEeL0K6a/05YJFP1ADE40HF5Q04m?C`1:
BTX0<30`05ADE010@400EEEE03`l?01<C4`0>CTi055AD@19BDT0BTY:0451@@1CDe<0EUIF061PH01c
Lg<0KFe]055AD@1aLG40A4A40861P@23Ph<08R8R04E5A@1DE5@0K6a/0P1TI6@0H00H61P0@D5103De
=@0g=cL0FEUI030`<01>CTh0<c<c055AD@0/;2`0@D5103De=@0b<S80EEEE065QH@13@d<0B4Q804e=
C@0X:2P0@d=303Ph>01BDU80@d=308f=S@1nOWh0A4A4069RHP0X:2P0<C4a04M7A`0`<300B4Q806MW
I`1ADE40JVYZ055AD@1QHF40GemO04];B`1QHF40BDU906][J`13@d<0H61P05MGE`1;Bd/0E5AD030`
<017AdL0=CDe04A4A01BDU80AdM706YZJP1]KFd0BTY:07YjNP1DE5@0HV9R03`l?01TI6@0B4Q806UY
J@1FEUH0CTi>05IFEP18B4P0@D5106YZJP0g=cL0L71`061PH01FEUH0GemO05iNGP26QXH0E5AD05QH
F01BDU80De=C071`L011@D40ATI603<c<`1WIfL0ADE505]KF`1hN7P0?clo06=SH`1hN7P0Jf][08J6
QP1LG5`0LG5a04];B`80EEEE0140IFEU055AD@1`L700GUiN07emO@1LG5`0KFe]09bLW02@T900R8R8
0:j^[P2SXj<0WYjN08Z:RP1XJ6P0JVYZ0:ZZZP0@ooooo`003_ooool0<01EEED0HF5Q0:f][@2;Rh/0
@d=304e=C@22PX80MgMg08N7Q`1GEeL0Kfm_08V9R@1:BTX0Fe]K07IfMP0S8b<0L71`06QXJ010@400
EEEE038b<P10@400KFe]0492@P0e=CD0?3`l02/[:`1HF5P0[:b/03`l?01KFe/0MGEe059BDP2VYZH0
QHF502PX:00j>SX0MGEe04Y:BP2VYZH0CTi>03Lg=`0N7Qh0ADE508j>SP2c/k<0>SXj04E5A@80ATI6
01d0UiNG07emO@0O7al0C4a<08R8R00k>c/0A4A404Y:BP0f=SH0F5QH061PH00b<S80Bd];03Ti>@0_
;bl0B4Q805iNGP0H61P0<c<c059BDP17AdL0VYZJ06i^KP0P8200?clo04Y:BP1/K6`0Rh^;02d];@02
02HV9P0304a<C01PH600V9RH0080:BTY04l0AdM706e]K@10@400EUIF01<C4`0g=cL0S8b<0820P00o
?cl0GUiN02h^;P1:BTX0RXZ:09ZJVP10@400FUYJ03Hf=P1IFET0LG5a05YJFP0Z:RX0Cdm?03Lg=`1c
Lg<0^KVi05]KF`0M7Ad0A4A403lo?`1DE5@0aLG503dm?@0Q8B40ATI60820P01ZJVX0Shn?0492@P17
AdL0DE5A06m_K`1=CDd0IfMW05EEE@1;Bd/0WIfM07EeM@1NGUh0Ti>C03@d=01SHf<0PX:203De=@11
@D40Jf][0451@@1;Bd/0SXj>03Hf=P0h>3P0GEeM059BDP2?Shl0_;bl05=CD`0W9bL0Hf=S04a<C01:
BTX0`<3003Ph>00j>SX0EUIF055AD@1KFe/0_kno03De=@0`<300MgMg0080LG5a06d0MGEe03/k>`1T
I6@0X:2P04];B`1?Cdl0=SHf02LW9`1ZJVX0Ph>303lo?`1:BTX0<S8b04U9B@1PH600MGEe04A4A01F
EUH0B4Q805MGE`2DU9@0>CTi04U9B@0l?3`0Kfm_0:^[Z`1;Bd/09bLW05eMG@1?Cdl0UiNG034a<@0T
92@0AdM7079bLP2MWId0CDe=03Xj>P1GEeL0KVi^05YJFP1VIVH09RHV05]KF`1aLG40K6a/0;Zj^P29
RHT0BTY:0;Bd]015ADD0HV9R06YZJP0d=3@0BTY:07QhN013@d<0MWIf05ADE00a<C40:b/[06e]K@0U
9BD0Fe]K03hn?P0o?cl0De=C07IfMP0Y:BT0FUYJ03<c<`0l?3`0SHf=0<;2`P0e=CD0A4A405eMG@0o
?cl0SHf=0:^[Z`0^;Rh0Bd];07YjNP15ADD0M7Ad09JFUP0k>c/0A4A404Y:BP1MGEd0EeMG09FEU@0M
7Ad0?Cdm04];B`0^;Rh0Bd];06e]K@0S8b<0:2PX071`L01:BTX0CDe=0451@@0K6a/09RHV05IFEP1H
F5P09RHV0080AdM73_ooool0;`1_Kfl0OWin05QHF01LG5`0Bd];06IVIP1bLW80Fe]K08Z:RP2GUiL0
HF5Q06IVIP1[Jf/0IFEU08b<S01RHV80KFe]05YJFP1hN7P0Q8B405UIF@16ATH0C4a<08F5Q@1nOWh0
KVi^06IVIP1DE5@0XJ6Q04Q8B01HF5P0A4A407IfMP26QXH0NWYj07QhN02?Shl0MgMg03De=@1EEED0
B4Q80492@P0o?cl0IFEU06YZJP22PX80IVIV0080EEEE00T0DU9B08>3P`0b<S80Ng]k0451@@0j>SX0
F5QH065QH@1;Bd/00P1?Cdl0CP12@T80IVIV0451@@19BDT0GUiN03Ph>01RHV80BTY:03/k>`15ADD0
EeMG059BDP0h>3P0>CTi05YJFP1_Kfl0CDe=03<c<`1UIFD0HF5Q02DU9@0[:b/0CTi>01lO7`0o?cl0
BDU90410@00e=CD0?Cdm030`<01BDU80?Cdm04];B`0d=3@0>CTi04];B`0R8R80?3`l061PH00c<c<0
9RHV04a<C01PH6009bLW04i>CP10@400F5QH04E5A@1FEUH0ADE505iNGP0n?Sh0DU9B04M7A`0m?Cd0
A4A404M7A`1;Bd/0CTi>03dm?@1RHV80S8b<08B4Q02^[Zh0UIFE0;Fe]@24Q8@0=3@d065QH@1@D500
K6a/06i^KP1ADE40L71`07UiN@1VIVH0De=C05YJFP80@41008X0>c/k05IFEP10@400G5aL05YJFP1`
L700EeMG05UIF@1QHF40B4Q8051@D016ATH0Hf=S08^;R`0/;2`0SHf=08b<S01>CTh0L71`065QH@1C
De<0EUIF061PH00];Bd0De=C04m?C`0W9bL08B4Q03/k>`1<C4`0>c/k051@D00i>CT0;Bd]03dm?@1<
C4`0K6a/07QhN00j>SX0ATI605ADE00m?Cd0GemO03Lg=`1;Bd/0=SHf04Y:BP1=CDd0=cLg06IVIP0X
:2P071`L04M7A`0a<C40B4Q80451@@0e=CD0EeMG04U9B@2PX:00L71`0410@01BDU80;bl_05=CD`13
@d<0F5QH06MWI`0o?cl0I6AT04=3@`1:BTX0C4a<04Q8B00o?cl0E5AD03lo?`1IFET0<30`06MWI`1G
EeL0L71`01`L7017AdL0;Bd]07]kN`1FEUH0Cdm?0451@@1ADE40J6QX08V9R@0_;bl0FUYJ07EeM@1P
H600S8b<04M7A`16ATH0>SXj0410@00h>3P0;Bd]065QH@0i>CT0HF5Q02TY:@1IFET0EUIF05]KF`1>
CTh0FUYJ08>3P`2ATI40?Shn05IFEP0f=SH0D51@034a<@19BDT0A4A404M7A`1:BTX0GUiN07emO@1f
MWH0?Shn07]kN`2=SHd0VIVI06ATI025QHD0/;2`0<C4a027QhL0MGEe061PH01jNWX@ooooo`003_oo
ool0S`1bLW80KVi^059BDP26QXH0J6QX04U9B@1<C4`0GemO08:2PP21PH40>c/k071`L02<S8`0NGUi
075aL@0m?Cd0ADE506i^KP1dM7@0W9bL02LW9`0c<c<0EUIF06][J`1kNg/0;bl_04E5A@11@D40QXJ6
09VIV@2c/k<09bLW03dm?@1XJ6P0LW9b02l_;`1QHF406a/K02h^;P1ZJVX0RHV90451@@1=CDd0:RXZ
04A4A02EUID0OWin03Ph>01HF5P0@D5108^;R`2ZZZX0@T92034a<@1QHF40I6AT07alO00k>c/09bLW
03<c<`17AdL0EUIF0:j^[P0[:b/0;Rh^04m?C`1:BTX0?clo09JFUP1XJ6P0<C4a065QH@2<S8`0DE5A
07YjNP1gMgL09bLW05IFEP1KFe/09bLW04E5A@1KFe/0=cLg05=CD`0o?cl08b<S06a/K014A4@0>CTi
08N7Q`0g=cL07AdM04A4A01MGEd0GUiN07IfMP0Q8B409RHV05MGE`1FEUH0G5aL08V9R@0_;bl0A4A4
07QhN027QhL0ATI609ZJVP1?Cdl0=cLg0;^k^`2j^[X0>3Ph07YjNP1BDU80=3@d0:FUY@31`L40?3`l
06IVIP0g=cL04a<C0861P@2TY:@0;Bd]07QhN00b<S80?clo05QHF01iNGT0:BTY05EEE@1;Bd/0JVYZ
09^KV`11@D40;bl_07EeM@1WIfL0JVYZ0;>c/`0a<C40>3Ph0080UYJF08D0OWin09>CT`0J6QX0G5aL
09:BTP1NGUh0DE5A0::RXP12@T80?clo06ATI01@D500FEUI09BDU00Y:BT0@4100820P01_Kfl0Fe]K
06=SH`0^;Rh0HV9R07IfMP0h>3P0Kfm_04i>CP1CDe<0V9RH07moO`0[:b/0CTi>06EUI@1SHf<0Zj^[
04E5A@0f=SH0D51@05QHF02HV9P0Ti>C03dm?@1YJFT0RXZ:04E5A@2DU9@0>3Ph028R8P2<S8`0Shn?
0492@P1DE5@0;2`/05mOG`1/K6`0EEEE08B4Q00i>CT0F5QH0861P@0j>SX0C4a<04];B`0d=3@0OWin
03@d=015ADD0=SHf03<c<`1mOGd0W9bL03lo?`15ADD0Jf][09bLW02^[Zh0=CDe04U9B@18B4P0BDU9
0:j^[P2?Shl0=CDe0410@01TI6@0D51@0:NWY`1eMGD09RHV04A4A01KFe/0@d=30:b/[01XJ6P092@T
03dm?@1UIFD0CTi>05QHF01DE5@0:b/[03Xj>P26QXH0Cdm?03lo?`1DE5@09bLW05]KF`2VYZH0G5aL
03/k>`0a<C40820P05IFEP2XZ:P0F5QH01dM7@0h>3P0ATI605QHF02UYJD0ADE5018B4P12@T80GUiN
07EeM@1OGel092@T03`l?00j>SX0>CTi071`L01bLW808B4Q00kooooo04h0XJ6Q08^;R`1OGel0N7Qh
07=cL`22PX80SXj>061PH02SXj<0^;Rh06ATI02KVi/0P82006YZJP2RXZ80Xj>S08>3P`2:RXX0MGEe
05QHF01KFe/0J6QX06ATI01iNGT0M7Ad05mOG`1lO7`0D51@0;Fe]@1CDe<0OGem03dm?@2GUiL0I6AT
096AT@1[Jf/0PH6106ATI00g=cL0Bd];03Ph>00l?3`0>c/k059BDP14A4@0CDe=04U9B@1ADE40De=C
0451@@1bLW80EeMG07emO@16ATH0ADE507QhN01jNWX0IFEU05eMG@1KFe/0DU9B07MgM`1@D500Lg=c
06EUI@12@T80Kfm_06YZJP1`L700GEeM051@D01`L700Hf=S071`L01JFUX0;bl_04A4A01;Bd/203lo
?`80@D5100/0ADE502<S8`19BDT0@d=306EUI@1DE5@0DU9B0492@P1VIVH0C4a<05YJFP0204e=C@0K
04m?C`1LG5`0>SXj02TY:@1HF5P0?Cdm05UIF@14A4@0=CDe04i>CP1DE5@0Lg=c04I6AP10@400P820
07AdM00^;Rh08B4Q04E5A@1OGel0?Cdm04];B`17AdL0C4a<04i>CP1FEUH0MWIf0080T92@0900MgMg
05aLG01dM7@0?3`l06ATI01RHV80LW9b07UiN@1TI6@0OGem06=SH`0o?cl08B4Q05IFEP1aLG40GEeM
03Lg=`0o?cl0MWIf05EEE@1=CDd0E5AD04m?C`1JFUX0H61P04Y:BP1VIVH0KVi^03De=@11@D409bLW
06YZJP27QhL0;2`/02d];@1IFET0KVi^06UYJ@1WIfL06ATI03`l?00g=cL0820P03De=@0R8R80@d=3
02/[:`1:BTX0=cLg03Xj>P15ADD0?3`l03/k>`1?Cdl0BTY:05ADE01gMgL0ATI603lo?`1;Bd/0MgMg
05MGE`1TI6@0EeMG03Lg=`1^KVh0:b/[028R8P15ADD0@41005EEE@1?Cdl0@d=307QhN00g=cL0D51@
03Lg=`1OGel0N7Qh03dm?@1=CDd0@d=305YJFP1LG5`0EeMG038b<P16ATH0C4a<030`<00l?3`0@T92
055AD@1@D500FUYJ0451@@2SXj<0Lg=c051@D00i>CT0Fe]K03/k>`1NGUh0>3Ph06QXJ01EEED0De=C
04M7A`25QHD0De=C06MWI`24Q8@0G5aL051@D01BDU80EUIF05=CD`1WIfL0HF5Q02DU9@1IFET0?Cdm
0820P01BDU80LG5a04m?C`1eMGD0Bd];06a/K01UIFD0Rh^;05MGE`1NGUh0@41005ADE00d=3@0D51@
05aLG01IFET0DU9B06ATI023Ph<0SXj>055AD@1jNWX20:RXZ00807alO02JVYX0OWin092@T02IVIT0
Lg=c06i^KP29RHT@ooooo`003_ooool0o`1gMgL0eMGE05iNGP1LG5`0Rh^;07moO`1kNg/0A4A4065Q
H@1iNGT0IfMW04a<C00];Bd0MWIf061PH00a<C40EeMG02/[:`0o?cl0Hf=S059BDP0Q8B40EeMG0492
@P1HF5P0Cdm?01`L7012@T80Fe]K05mOG`2YZJT0GUiN02d];@18B4P0FEUI05eMG@1mOGd07QhN030`
<01IFET0CTi>06ATI02NWYh0ADE504U9B@1hN7P0MGEe07inOP1/K6`0>3Ph05=CD`2?Shl0Hf=S07mo
O`21PH4092@T04=3@`1[Jf/0@41005ADE014A4@06ATI06QXJ01[Jf/0<C4a05eMG@0/;2`071`L04a<
C01PH600@d=305EEE@0/;2`0@d=3092@T01mOGd07alO04i>CP0m?Cd0B4Q809jNWP2GUiL0<c<c07Ui
N@1JFUX0E5AD0<K6aP2BTY8051@D05IFEP1NGUh0KVi^08f=S@12@T80=cLg05IFEP0h>3P0De=C0;Vi
^@1CDe<0:b/[06QXJ01RHV80D51@06a/K00[:b/0EeMG071`L021PH40;bl_05QHF01SHf<0EEEE08>3
P`1GEeL07AdM04m?C`2=SHd0MGEe09ZJVP16ATH071`L06UYJ@2_[jl0EeMG0;Fe]@13@d<0;bl_05AD
E01^KVh0?clo051@D00Z:RX0<S8b08n?S`1eMGD0HF5Q05MGE`0P8200A4A407AdM01UIFD0C4a<05MG
E`0h>3P0L71`0:^[Z`1YJFT0;2`/04];B`1XJ6P0R8R80;Vi^@1ADE40:b/[06ATI01]KFd0QhN70;fm
_@0^;Rh05QHF05ADE01kNg/0SHf=09FEU@0^;Rh09RHV06i^KP1LG5`0YJFU03lo?`0g=cL0?Cdm07Ad
M01LG5`0MGEe0410@00f=SH0@D5106MWI`1?Cdl0HF5Q03/k>`15ADD0AdM704i>CP1DE5@0UiNG05QH
F03?cll0IVIV034a<@1BDU80DE5A04Q8B01jNWX0;Rh^0451@@1BDU80IVIV09^KV`0j>SX0ATI609>C
T`28R8P0V9RH06a/K00_;bl0F5QH04m?C`22PX80C4a<03Ph>01KFe/0V9RH09>CT`1XJ6P07AdM04];
B`2MWId0Hf=S07UiN@1<C4`0;2`/05IFEP2JVYX0Cdm?04E5A@19BDT0>CTi07UiN@2CTi<07AdM02`/
;00o?cl0EUIF08Z:RP2NWYh0;bl_03De=@1ADE40ATI608R8R01IFET09BDU024Q8@1KFe/0FUYJ07If
MP10@4006ATI034a<@1dM7@0Rh^;09FEU@0104=3@`80>CTi01@0H61P07inOP18B4P0Bd];02PX:00]
;Bd0ADE504];B`0W9bL0BTY:02/[:`13@d<0LG5a03hn?P0P8200ADE5020P800e=CD0K6a/038b<Pko
oooo02L0HV9R075aL@1FEUH0GEeM061PH01]KFd0Fe]K05EEE@27QhL0OGem05eMG@2<S8`0HF5Q08N7
Q`2<S8`0OWin08B4Q01SHf<0L71`05ADE01=CDd0G5aL04m?C`1`L700J6QX05EEE@1bLW80IfMW0;>c
/`1>CTh0OGem04M7A`2NWYh0KFe]09jNWP1[Jf/0WYjN09>CT`0e=CD00P1HF5P0B@1TI6@0=cLg065Q
H@1BDU80Hf=S04m?C`1WIfL0GemO055AD@24Q8@0>3Ph05mOG`0l?3`0?clo0820P01mOGd0C4a<059B
DP1OGel0EEEE04Y:BP0o?cl0DE5A05QHF00m?Cd0IVIV04i>CP10@400EUIF04U9B@1MGEd0?clo04U9
B@1;Bd/0>CTi065QH@1CDe<0EEEE069RHP1@D500BDU904I6AP0j>SX0A4A403Ti>@16ATH0<30`03Ph
>00];Bd0HV9R04A4A01JFUX0>CTi04M7A`18B4P0>CTi02TY:@0a<C40D51@04U9B@1FEUH0F5QH051@
D028R8P0>SXj05MGE`1=CDd0Bd];04m?C`1PH600@T9204m?C`12@T800P14A4@0XP17AdL0EUIF055A
D@15ADD0FEUI0:BTY017AdL0IfMW0:2PX02ZZZX0N7Qh02h^;P1>CTh0Lg=c09BDU00h>3P0;2`/04Y:
BP1oOgl0IFEU028R8P14A4@0<C4a05IFEP0Y:BT0C4a<03/k>`1@D500AdM706QXJ028R8P0H61P05=C
D`1;Bd/0Jf][07EeM@1<C4`0J6QX00/;2`1^KVh0OWin01/K6`0o?cl0@41006ATI01DE5@0Bd];00`<
3019BDT0ADE504A4A01RHV80;Rh^04i>CP0W9bL0BTY:01lO7`0j>SX0;Bd]059BDP12@T80HF5Q0410
@00f=SH0KVi^030`<01>CTh0>SXj0492@P18B4P0MGEe079bLP1ADE40H61P03dm?@0i>CT0CTi>03/k
>`0f=SH0Cdm?03Xj>P0Y:BT0@D5107MgM`1?Cdl0<c<c04U9B@0W9bL0?Shn051@D00k>c/0E5AD04e=
C@11@D4092@T0410@00d=3@0A4A404i>CP13@d<0H61P06=SH`10@400B4Q804=3@`1LG5`0>CTi08B4
Q01MGEd0FEUI03dm?@14A4@0IfMW055AD@1GEeL0QHF504M7A`1oOgl0RHV904I6AP1NGUh0Bd];05]K
F`1SHf<0FEUI04m?C`1=CDd0Hf=S04E5A@1ADE40=3@d07inOP15ADD0Kfm_06a/K01IFET0IfMW08j>
SP14A4@0ATI603hn?P1BDU80?clo06IVIP1QHF40LW9b075aL@2=SHd0VYZJ07alO01mOGd0VYZJ0:6Q
X@2CTi<0W9bL0;Vi^@2NWYh0YJFU08^;R`1KFe/0O7al0;Fe]A3ooooo000>ooooo`3o05MGE`26QXH0
WIfM05MGE`1HF5P0Ng]k09JFUP0h>3P0@T9206MWI`1kNg/0WIfM02XZ:P1<C4`0EUIF04];B`1lO7`0
;2`/02TY:@1`L700Lg=c05mOG`2e]KD0<S8b06YZJP1SHf<0?Shn065QH@1SHf<0<30`071`L01TI6@0
KFe]0:2PX01UIFD05aLG06=SH`1;Bd/0HF5Q0<71`@0o?cl09BDU05]KF`1OGel0EUIF03`l?00c<c<0
A4A407moO`0k>c/0EEEE04I6AP0c<c<0Kfm_08j>SP0a<C40@41002l_;`1NGUh0]KFe05YJFP0];Bd0
@d=305=CD`1cLg<0Zj^[03@d=00K6a/0>c/k055AD@29RHT0fMWI05aLG00Q8B40AdM706][J`17AdL0
VIVI04e=C@0?3`l0Bd];0820P01CDe<0N7Qh03Hf=P0R8R80Ogmo0:NWY`1[Jf/0JFUY02DU9@0a<C40
Ti>C05UIF@0L71`0DE5A03lo?`1KFe/0];Bd0451@@0O7al0?3`l04M7A`1^KVh0Zj^[03@d=00U9BD0
GUiN08Z:RP27QhL0RXZ:01dM7@1GEeL0S8b<0861P@1;Bd/0MgMg01hN7P19BDT0GemO05iNGP0i>CT0
De=C028R8P18B4P0T92@06][J`0O7al0J6QX04I6AP22PX80GemO024Q8@0^;Rh0AdM70492@P2`/;00
P82001dM7@0T92@0FEUI05aLG01kNg/0LW9b01XJ6P0/;2`0MGEe0<30`01eMGD0F5QH020P8012@T80
X:2P0;Rh^01TI6@0CTi>01lO7`1_Kfl0Q8B4065QH@13@d<0Ogmo03Ph>01DE5@0SHf=059BDP1JFUX0
EeMG04U9B@2=SHd0UIFE04A4A01CDe<0:2PX04Y:BP2?Shl0GemO02`/;01:BTX0>3Ph05MGE`2PX:00
IVIV04Y:BP11@D40E5AD08V9R@32`/80@T9207=cL`1XJ6P0KFe]07moO`0m?Cd0;2`/06=SH`1?Cdl0
LW9b02PX:01CDe<0L71`06=SH`1GEeL0<c<c03/k>`1iNGT0Ng]k03/k>`1?Cdl09bLW06EUI@1MGEd0
=3@d05iNGP0b<S80OGem096AT@0R8R80=SHf04E5A@1@D500Ogmo069RHP0_;bl0?Cdm05YJFP1<C4`0
PH6106YZJP0S8b<0Fe]K08n?S`1GEeL0IVIV05mOG`11@D40FEUI0820P00n?Sh0I6AT07=cL`0m?Cd0
HF5Q05mOG`0Y:BT0ATI6069RHP0Q8B40J6QX07alO00];Bd0=3@d04];B`0S8b<0M7Ad000G06YZJP0T
92@0ATI603`l?00b<S80H61P065QH@0T92@0Bd];04U9B@0h>3P0HF5Q07inOP12@T80:b/[03Ti>@1I
FET0?Cdm075aL@0Q8B40>c/k0410@00f=SH03_ooool0=@1TI6@0QXJ60861P@1nOWh0DU9B07]kN`2C
Ti<0HV9R09VIV@2KVi/0QXJ60:BTY022PX80OGem09BDU01QHF40G5aL055AD@1FEUH0?clo055AD@1[
Jf/0FEUI05iNGP2EUID0KVi^06e]K@0m?Cd0Rh^;06MWI`1`L700ADE507QhN021PH40NGUi06i^KP2P
X:00a<C404I6AP19BDT0@41003lo?`1;Bd/0FUYJ05=CD`1[Jf/0CDe=06][J`1RHV80HF5Q08f=S@17
AdL0PH610080@T9203D0OGem06m_K`1PH600M7Ad059BDP1=CDd0Lg=c04a<C01OGel0N7Qh05QHF01i
NGT0Cdm?06EUI@1TI6@0FEUI06e]K@1:BTX0Kfm_07=cL`0f=SH0Kfm_06IVIP1@D500=CDe051@D01I
FET0A4A403lo?`1PH600?Shn06IVIP1;Bd/0=SHf04Y:BP1`L700AdM7061PH014A4@0Bd];04M7A`1=
CDd0@d=303Xj>P1VIVH0A4A406a/K01@D500ADE506UYJ@0n?Sh0H61P05iNGP0206a/K01W06e]K@0h
>3P0De=C05ADE01MGEd0B4Q805ADE00k>c/0EUIF04A4A017AdL0F5QH079bLP1iNGT0G5aL04U9B@1D
E5@0>SXj0410@019BDT0EUIF06UYJ@1>CTh0E5AD05=CD`1;Bd/0<S8b06UYJ@0n?Sh0EeMG03lo?`1V
IVH0CDe=06=SH`1RHV80NGUi03lo?`12@T80GEeM05]KF`1CDe<0I6AT0820P02PX:008B4Q07emO@1l
O7`04a<C07=cL`1NGUh0P820069RHP1NGUh08R8R08F5Q@1:BTX092@T03Lg=`0j>SX0I6AT03Ph>019
BDT0>SXj04M7A`0c<c<0CTi>04Q8B01_Kfl0<c<c04M7A`1cLg<0BDU905MGE`0[:b/0@T9202`/;01H
F5P0FUYJ03Hf=P1iNGT0;bl_024Q8@11@D40;Rh^04Y:BP1bLW80=CDe04A4A00i>CT0Ng]k071`L00c
<c<0F5QH02/[:`1GEeL0D51@02d];@1=CDd0D51@05IFEP0j>SX0BTY:04I6AP0204M7A`0o03/k>`0o
?cl0Cdm?03Xj>P1EEED0=SHf05EEE@0_;bl0=cLg03Ph>01OGel0:b/[05eMG@1QHF40F5QH0410@01f
MWH0<c<c07MgM`2<S8`0R8R809:BTP1^KVh0OWin096AT@13@d<0FEUI03lo?`1^KVh0=CDe05aLG00e
=CD0J6QX03dm?@1NGUh0IFEU06QXJ01QHF40UYJF06e]K@1OGel0Fe]K06YZJP0o?cl0FUYJ05UIF@1E
EED0Shn?08F5Q@1kNg/0Lg=c06QXJ02MWId0YJFU0;Ng]`1XJ6P0Lg=c0:JVYP3UiND0De=C08>3P`1d
M7@0KFe]013ooooo000>ooooo`0Z05mOG`1XJ6P0/[:b09FEU@12@T80H61P08R8R02SXj<0RHV905aL
G01BDU80PH6105ADE01eMGD0BTY:030`<01PH600I6AT03dm?@1dM7@0:2PX03dm?@1JFUX0@T9204U9
B@0a<C40<c<c04U9B@1kNg/0C4a<05mOG`0_;bl0@41008F5Q@1YJFT09RHV03Hf=P0`<300N7Qh0<?3
``1:BTX0:b/[0P17AdL0GP2JVYX0UIFE02XZ:P0/;2`0De=C06QXJ02a/K40C4a<02HV9P0/;2`0LW9b
07EeM@2CTi<0:BTY020P800g=cL0E5AD02PX:028R8P0:2PX03De=@1MGEd0IVIV034a<@1ZJVX0@d=3
02h^;P1LG5`0Jf][02@T901dM7@0E5AD0492@P27QhL0MgMg01lO7`1:BTX0HF5Q0410@01gMgL0?Shn
038b<P12@T80Cdm?051@D02ZZZX0:2PX03`l?01]KFd0QXJ6061PH029RHT06a/K04E5A@1MGEd0NWYj
05UIF@2][Jd0<c<c04A4A01SHf<0GEeM03De=@1dM7@0@d=305QHF02;Rh/0LW9b02PX:018B4P0FUYJ
05aLG02IVIT0EUIF02XZ:P1EEED0GEeM05IFEP2:RXX0<30`01TI6@1RHV80IVIV05MGE`2l_;`0?clo
059BDP1kNg/0@41008:2PP2OWil0>c/k061PH02j^[X2055AD@0F08J6QP0k>c/0?clo07AdM019BDT0
@T9205eMG@0];Bd0IFEU05=CD`0R8R80;Rh^06a/K012@T80D51@0:f][@0];Bd0>CTi07]kN`1CDe<0
IfMW0=?Cd`80>c/k01l0LW9b04U9B@2WYjL0YZJV02l_;`1kNg/0[jn_04Q8B01lO7`0<C4a02LW9`1R
HV80O7al04Q8B01IFET071`L04Q8B019BDT0GUiN06EUI@17AdL09bLW05mOG`1YJFT0Jf][071`L00S
8b<0D51@0:JVYP1eMGD09bLW0080?clo0540YZJV069RHP0U9BD0?Shn06IVIP2=SHd0CTi>02HV9P1C
De<0Jf][05QHF01^KVh0>SXj04m?C`1CDe<0FEUI0=3@d01QHF40;bl_06][J`0j>SX0MGEe08j>SP0S
8b<0D51@05YJFP17AdL0J6QX0861P@0b<S80?clo06m_K`0g=cL0MGEe04A4A00^;Rh0CDe=05QHF00T
92@0@d=3028R8P15ADD0DE5A08V9R@0`<300EUIF05aLG01RHV80JVYZ06ATI00U9BD0A4A405UIF@1?
Cdl0Ti>C08V9R@0Z:RX0>CTi06ATI017AdL0OGem06m_K`0P8200:b/[07alO018B4P0Rh^;065QH@0W
9bL07QhN05MGE`2FUYH0?clo06QXJ00Q8B40@41004];B`0h>3P0>c/k034a<@0[:b/03_ooool0>P23
Ph<0UIFE059BDP1UIFD0IfMW07inOP1_Kfl0Cdm?07inOP2SXj<0JVYZ06m_K`1DE5@0F5QH075aL@1Z
JVX0PH6105=CD`1bLW80N7Qh051@D01LG5`0@D51051@D01GEeL0I6AT06=SH`18B4P0De=C03hn?P1P
H600?3`l05YJFP12@T80Jf][06e]K@2<S8`0NGUi04i>CP1BDU80?Shn04A4A013@d<0CTi>03dm?@1;
Bd/0BTY:059BDP10@400>SXj05ADE01LG5`0Jf][04=3@`16ATH0Kfm_0820P01OGel205]KF`3J06][
J`1UIFD0BTY:07MgM`1GEeL0?Shn065QH@1ZJVX0NGUi069RHP1:BTX0Ng]k06UYJ@1cLg<0LG5a055A
D@1YJFT0IFEU06][J`1<C4`0@d=304U9B@18B4P0<S8b04I6AP11@D40BDU904m?C`17AdL0A4A405UI
F@1:BTX0CDe=061PH01JFUX0A4A405ADE01?Cdl0F5QH06EUI@1<C4`0M7Ad05YJFP1;Bd/0Shn?04=3
@`1BDU80EEEE04a<C01[Jf/0XJ6Q0492@P1KFe/0L71`055AD@1JFUX0KFe]04I6AP1EEED0=cLg06i^
KP1`L700Kfm_0:ZZZP23Ph<0Lg=c079bLP14A4@0I6AT059BDP1XJ6P0K6a/06][J`1gMgL0KVi^05eM
G@19BDT0FUYJ03Xj>P1FEUH09bLW05IFEP15ADD0KVi^04i>CP1PH600FUYJ06MWI`1GEeL0AdM707]k
N`1DE5@09BDU03Ph>00U9BD0OWin08:2PP0`<300KVi^051@D02DU9@0HV9R0492@P0>3Ph0?3`l038b
<P0U9BD0A4A402<S8`0k>c/09bLW03Ti>@19BDT0M7Ad04=3@`0m?Cd0>CTi05=CD`0g=cL0EeMG075a
L@0j>SX0AdM703`l?01CDe<0AdM709NGU`2EUID0AdM7092@T00b<S80DE5A079bLP0i>CT0B4Q8059B
DP0`<300E5AD04m?C`2GUiL0LW9b06m_K`1dM7@0=SHf03Xj>P15ADD0Fe]K07=cL`1LG5`0@T9203Lg
=`1KFe/0AdM703dm?@13@d<0FUYJ04E5A@1?Cdl08B4Q06ATI01RHV80Kfm_02`/;01=CDd0@T9206IV
IP1MGEd0H61P06QXJ01YJFT0CTi>05EEE@0b<S80OWin09bLW01_Kfl0PH61061PH01?Cdl0GemO03`l
?01jNWX0@T9206a/K01IFET0T92@06MWI`2h^;P0H61P069RHP27QhL0MGEe08V9R@21PH40B4Q8065Q
H@11@D40EeMG06=SH`1cLg<0EeMG059BDP1EEED0K6a/06i^KP1UIFD0Ogmo0<G5a@2[Zj/0/K6a08f=
S@2BTY80NGUi07QhN01[Jf/0/K6a0861P@2PX:0@ooooo`003_ooool03@1^KVh0_;bl071`L01fMWH0
I6AT08>3P`1mOGd0CTi>06EUI@1YJFT0SXj>061PH00i>CT00P1kNg/0400c<c<0Bd];03hn?P1VIVH0
QXJ605YJFP0j>SX0C4a<04I6AP1JFUX0CDe=02PX:01FEUH0DE5A04e=C@2:RXX202h^;P3e04a<C01O
Gel0>SXj069RHP0Z:RX0AdM706YZJP1QHF40@d=308>3P`0f=SH0@4100820P01MGEd0I6AT08^;R`0^
;Rh0ADE507UiN@1RHV80EeMG06a/K00K6a/0J6QX05]KF`1FEUH0NWYj03dm?@0T92@0QHF505IFEP1>
CTh0IVIV02/[:`0];Bd0L71`051@D013@d<0Jf][030`<00j>SX0Ogmo08F5Q@0`<300Fe]K04e=C@0m
?Cd0QXJ608R8R00_;bl0B4Q8055AD@1:BTX0MgMg03/k>`0P8200FUYJ07MgM`1DE5@0QhN703Hf=P0Q
8B40JVYZ06a/K015ADD0TY:B03Ti>@0V9RH0JFUY055AD@14A4@0BTY:03`l?018B4P0Ph>3079bLP0i
>CT0JFUY04Q8B01CDe<0RXZ:075aL@15ADD0MGEe04A4A015ADD0R8R8051@D00V9RH0HV9R059BDP1_
Kfl0SXj>04I6AP0O7al0G5aL07QhN01QHF40K6a/02TY:@0_;bl0L71`065QH@1nOWh0IFEU05=CD`1N
GUh0UIFE05YJFP18B4P0GEeM03@d=01_Kfl0][Jf05YJFP1QHF40FEUI02l_;`1YJFT0YZJV04m?C`0l
?3`0GemO05=CD`1aLG40WIfM03Ti>@0b<S80OWin0820P026QXH0FUYJ02l_;`1ADE40JVYZ05ADE026
QXH0BDU904m?C`1EEED0NWYj04Y:BP1bLW80<c<c05=CD`2KVi/0H61P02/[:`19BDT0CTi>06a/K01I
FET0:BTY03lo?`1]KFd0DU9B0861P@18B4P0=SHf06][J`1XJ6P0D51@06ATI00Y:BT0G5aL05iNGP1F
EUH0P82003Ti>@1MGEd0MWIf04i>CP2RXZ80DE5A03@d=01oOgl0?Shn05=CD`0^;Rh0<S8b06=SH`2P
X:00GUiN04M7A`0[:b/0De=C07UiN@1KFe/0I6AT03De=@14A4@0IVIV096AT@0b<S80AdM704Q8B01D
E5@0QhN709NGU`0e=CD0FEUI051@D01UIFD0Ng]k055AD@0V9RH0C4a<05YJFP18B4P0Ng]k06a/K00^
;Rh0Bd];04e=C@0j>SX0KVi^05MGE`0P8200>CTi06YZJP1LG5`0WYjN05]KF`0X:2P0B4Q806e]K@1H
F5P0Fe]K059BDP0W9bL0B4Q80820P017AdL0BTY:05EEE@10@400DU9B071`L00l?3`0=CDe05QHF00c
<c<0MgMg08Z:RP10@4003_ooool05`1LG5`0J6QX07IfMP1[Jf/0Bd];06=SH`24Q8@0H61P05eMG@1h
N7P0R8R80:6QX@2GUiL0LW9b08f=S@1_Kfl0MGEe05aLG01aLG40EeMG05=CD`1UIFD0EEEE0080JVYZ
04P0BTY:06EUI@1?Cdl0OWin05UIF@1lO7`0ADE507YjNP1XJ6P0NWYj07MgM`2:RXX0TY:B05QHF01E
EED0DE5A05UIF@10@400De=C04Q8B01iNGT0F5QH05eMG@1LG5`0KFe]08>3P`11@D40JVYZ03hn?P0i
>CT0K6a/05ADE013@d<0J6QX04E5A@0j>SX0EeMG059BDP1=CDd0F5QH05]KF`1YJFT0?Cdm04M7A`1T
I6@0Hf=S075aL@14A4@0HF5Q069RHP0d=3@0GEeM055AD@1HF5P0=cLg0410@016ATH0?Shn03Ph>00o
?cl0=CDe0410@019BDT0CTi>03`l?01RHV80CDe=05MGE`0a<C40?Shn04a<C00n?Sh204E5A@0k05]K
F`18B4P0Fe]K051@D01;Bd/0IVIV059BDP1EEED0Fe]K06EUI@1cLg<0QHF505aLG01YJFT0N7Qh059B
DP1@D500KFe]04a<C01HF5P0>3Ph06IVIP2@T900MWIf0::RXP2>SXh0Shn?07alO00];Bd0E5AD04Y:
BP19BDT0C4a<04e=C@1XJ6P0Q8B406EUI@19BDT0C4a<03<c<`1@D500<S8b06YZJP0c<c<0L71`05=C
D`1iNGT0ADE5079bLP1LG5`0@D5104Q8B01TI6@0Bd];04m?C`0X:2P0DE5A09VIV@0Q8B400P14A4@0
601?Cdl0DU9B05eMG@0H61P0ATI603/k>`0d=3@0Bd];02l_;`11@D40;Bd]04Y:BP0e=CD0<30`03/k
>`0i>CT0D51@07IfMP11@D40ATI605ADE00n?Sh0AdM702@T9080;bl_01D0Cdm?05]KF`0h>3P0Q8B4
02<S8`0L71`0?Cdm02HV9P0j>SX0EUIF02HV9P0b<S80@d=309BDU01_Kfl0=3@d04Q8B00^;Rh0FUYJ
04I6AP0X:2P00P0o?cl0A@0d=3@0<C4a03/k>`0V9RH0=CDe0410@00g=cL0Cdm?06QXJ00f=SH0Cdm?
055AD@1RHV80;Rh^05ADE015ADD0FEUI04i>CP1RHV80DE5A03`l?01=CDd0HV9R028R8P1XJ6P0PH61
06m_K`20P800EUIF07]kN`2JVYX0HV9R06EUI@1FEUH0KFe]03Ph>01DE5@0Bd];06EUI@19BDT0IFEU
0:JVYP1^KVh0LG5a07QhN01PH600GUiN04I6AP1PH600>3Ph055AD@18B4P0De=C05EEE@1RHV80OWin
06i^KP1bLW80PX:209JFUP2ZZZX0SXj>09BDU01^KVh0TY:B07QhN01fMWH0J6QX07MgM`0@ooooo`00
3_ooool0MP1BDU80TI6A0;Zj^P20P800C4a<06=SH`2BTY80DE5A07AdM011@D40DE5A06=SH`0e=CD0
ADE504I6AP19BDT0J6QX02HV9P0S8b<0BTY:03@d=01iNGT0UIFE02<S8`0T92@0A4A404I6AP24Q8@0
LG5a02d];@1DE5@0;Rh^06][J`2h^;P0LG5a01DE5@1:BTX0<C4a09VIV@2/[:`092@T03@d=01PH600
JFUY092@T00n?Sh0AdM703Lg=`1JFUX0B4Q808B4Q00T92@0:BTY07AdM01eMGD0ADE505eMG@0E5AD0
AdM708^;R`0n?Sh06a/K03dm?@0S8b<0FEUI0:2PX00];Bd05aLG03Ph>01ADE40U9BD0::RXP0S8b<0
7AdM020P801@D500RHV90;bl_00L71`0:b/[06UYJ@1EEED0Hf=S07IfMP0G5aL0;bl_06e]K@2:RXX0
<c<c05aLG00X:2P0IFEU092@T01bLW80:BTY055AD@0U9BD0F5QH096AT@15ADD09bLW04E5A@1ADE40
QXJ6075aL@0O7al0820P051@D01kNg/0RXZ:06UYJ@0U9BD0:BTY055AD@2BTY80M7Ad0;2`/00n?Sh2
03Xj>P2N08f=S@1CDe<0FEUI02d];@1=CDd0DE5A06i^KP19BDT0NWYj02h^;P0o?cl0MGEe03De=@10
@400C4a<02h^;P1MGEd0P82001hN7P0/;2`0BTY:04I6AP1dM7@0GUiN02PX:00Y:BT0Ogmo071`L02O
Wil0D51@03Ti>@19BDT0PH61096AT@1^KVh0JFUY03Ti>@1GEeL0MgMg08F5Q@1@D500XJ6Q030`<01S
Hf<0GEeM03dm?@1IFET0KVi^03`l?01MGEd0NWYj02@T901LG5`0<c<c03Lg=`29RHT0M7Ad01/K6`1A
DE40AdM704];B`2QXJ40FUYJ05=CD`1=CDd0BTY:06MWI`2@T900<c<c06QXJ01_Kfl0K6a/0:JVYP10
@400>CTi05mOG`1ZJVX0U9BD034a<@0i>CT0Fe]K05mOG`1mOGd0=CDe03hn?P1lO7`0Jf][04E5A@1J
FUX0JVYZ06i^KP1GEeL092@T04];B`1SHf<0PX:206e]K@0P8200?clo04m?C`0`<300LW9b04Q8B00W
9bL0F5QH04];B`12@T80SXj>06IVIP0b<S80Cdm?05iNGP16ATH0V9RH05UIF@0d=3@0IVIV07IfMP0k
>c/0QhN70:f][@0c<c<0EeMG05IFEP14A4@0De=C05YJFP0K6a/0A4A40:VYZ@0i>CT0MWIf065QH@0T
92@0DU9B0861P@0W9bL0EEEE03@d=00R8R80>3Ph06][J`0`<300>c/k02<S8`0f=SH0?Shn07YjNP1C
De<0<S8b02XZ:P0d=3@0IfMW092@T00^;Rh0;2`/05QHF014A4@>ooooo`0<08n?S`2IVIT0DU9B06IV
IP1NGUh0Jf][06=SH`0i>CT0Cdm?07AdM01_Kfl0RHV90P2GUiL08@2;Rh/0Ng]k07QhN01IFET0QHF5
06i^KP1EEED0NWYj05QHF01SHf<0IfMW07UiN@20P800BDU906=SH`1=CDd0GUiN0451@@1^KVh0CTi>
0861P@1dM7@0QXJ606i^KP1>CTh0G5aL0492@P10@400?3`l06IVIP1WIfL0Rh^;069RHP0205iNGP0]
04i>CP1KFe/0@T9206][J`1EEED0E5AD06EUI@1HF5P0AdM706EUI@0o?cl0>3Ph051@D01;Bd/0C4a<
05UIF@0n?Sh0IVIV05EEE@2DU9@0GemO0451@@1ZJVX0D51@065QH@20P800@T9206i^KP1IFET0J6QX
0410@00n?Sh0AdM7051@D00Y:BT0?Shn04e=C@15ADD0E5AD05EEE@11@D40Bd];03Ph>01ADE40Cdm?
0080@d=30:D0DE5A04];B`1OGel0De=C03hn?P1WIfL0J6QX04E5A@21PH40J6QX065QH@15ADD0=cLg
06QXJ01LG5`0@D5104m?C`1RHV80Cdm?0410@017AdL0@d=304A4A01IFET0GUiN08b<S02FUYH0WYjN
08R8R01;Bd/0MWIf04];B`1EEED0F5QH07UiN@1fMWH0OGem0820P01[Jf/0EUIF05=CD`1>CTh0=CDe
05=CD`0j>SX0EUIF04=3@`1QHF40N7Qh05eMG@1=CDd0GUiN06=SH`11@D40C4a<061PH01DE5@0J6QX
05MGE`1ADE40S8b<02@T9023Ph<0G5aL059BDP16ATH0GemO030`<01;Bd/0G5aL03`l?01<C4`0>3Ph
05=CD`0i>CT0?Shn03/k>`0Y:BT0=3@d0410@00n?Sh0HF5Q034a<@17AdL0IfMW0451@@0k>c/0>3Ph
05=CD`0d=3@0HF5Q07=cL`0d=3@0Fe]K02PX:00U9BD0@d=302PX:00j>SX0ADE501/K6`0n?Sh0A4A4
05IFEP0i>CT0<S8b04e=C@0G5aL0;bl_03Lg=`19BDT0Fe]K03<c<`0a<C40@41004E5A@0h>3P0=SHf
03De=@1FEUH0>CTi04];B`0];Bd0DU9B03Ph>01OGel0=CDe05ADE00a<C40>c/k02/[:`13@d<0B4Q8
04a<C01/K6`0P82004a<C01fMWH0PX:205aLG01DE5@0<c<c0492@P1WIfL0A4A4071`L014A4@0EUIF
04Q8B01lO7`0CDe=07MgM`26QXH0RXZ:06=SH`17AdL0N7Qh08N7Q`1GEeL0HF5Q04I6AP1<C4`0=3@d
04Q8B017AdL00P1>CTh03`1mOGd0O7al06MWI`1`L700WinO09JFUP2VYZH0XJ6Q0<_;b`1NGUh0IfMW
061PH01VIVH0LW9b08f=S@0@ooooo`003_ooool0o`1?Cdl0IfMW08f=S@2SXj<0AdM704e=C@1]KFd0
KVi^0<c<c01BDU80Cdm?059BDP1=CDd0UiNG06QXJ00[:b/0D51@061PH016ATH0JFUY02d];@19BDT0
Jf][069RHP1?Cdl05ADE03Xj>P1NGUh0EUIF04Q8B01[Jf/07AdM03@d=01]KFd0OGem04];B`1>CTh0
7AdM04Y:BP2GUiL0CTi>03Lg=`1EEED0D51@079bLP16ATH09RHV03`l?01GEeL0MGEe09nOW`0R8R80
5aLG05aLG01gMgL0E5AD08^;R`0R8R80820P06=SH`1^KVh0ADE50:^[Z`0O7al0@d=305UIF@1KFe/0
AdM70:VYZ@12@T80?3`l06e]K@1=CDd0B4Q805IFEP0R8R80=SHf079bLP0n?Sh06ATI061PH00W9bL0
E5AD0820P00O7al05ADE03`l?013@d<0Fe]K07inOP0R8R80>CTi05eMG@1YJFT0F5QH07EeM@0O7al0
Cdm?0:f][@2PX:00?Shn07MgM`0^;Rh0Jf][09ZJVP1=CDd0DU9B051@D00Y:BT0F5QH092@T01WIfL0
A4A406m_K`0j>SX0CTi>0:2PX01GEeL0;bl_04i>CP0c<c<0AdM708j>SP0m?Cd0AdM705YJFP1IFET0
GemO0;^k^`1]KFd0C4a<05MGE`0^;Rh0K6a/096AT@0];Bd0@D5106i^KP0k>c/0ADE507UiN@0G5aL0
ADE507AdM012@T80ADE505UIF@0c<c<0EeMG05UIF@0c<c<0EUIF08>3P`0];Bd0IFEU07]kN`0];Bd0
G5aL06=SH`0d=3@0FEUI0<K6aP13@d<0G5aL05MGE`1HF5P0OWin0:JVYP0d=3@0JFUY0861P@1mOGd0
[jn_05UIF@0k>c/0GEeM07moO`2@T900a/K60410@016ATH0MgMg07YjNP20P800B4Q801dM7@1SHf<0
YZJV04];B`1cLg<09bLW061PH02BTY80QhN704];B`1ZJVX0@T920::RXP1lO7`09bLW069RHP0h>3P0
UYJF06UYJ@0b<S80J6QX051@D013@d<0Ph>303<c<`1VIVH0Lg=c06=SH`2CTi<0BDU905EEE@2ATI40
BDU908f=S@2BTY80<C4a04Y:BP1KFe/0De=C09fMW@1/K6`06ATI04E5A@1YJFT0@d=307moO`19BDT0
=SHf0492@P1XJ6P0IfMW06IVIP0S8b<0;Bd]04=3@`1LG5`0=SHf05IFEP0Y:BT0@D51075aL@1hN7P0
9RHV03De=@19BDT0@d=308V9R@2MWId09bLW03<c<`005`1OGel0@D5108Z:RP1EEED0;2`/02DU9@1I
FET0Q8B407IfMP0Z:RX08R8R03lo?`19BDT0Cdm?07EeM@0f=SH0;2`/01lO7`1ADE40FEUI051@D00I
6AT09bLW00kooooo00l0B4Q806m_K`1_Kfl0Lg=c069RHP1bLW80S8b<051@D01JFUX0UYJF07]kN`2T
Y:@0MgMg05IFEP2>SXh00P1kNg/0?P1IFET0OWin05eMG@1/K6`0Hf=S04m?C`1YJFT0FEUI04Q8B01B
DU80FEUI06YZJP19BDT0EUIF04Y:BP1WIfL0HV9R075aL@26QXH0V9RH0:^[Z`1IFET0Fe]K04U9B@1B
DU80F5QH05iNGP0j>SX0Kfm_06=SH`1QHF40Bd];05UIF@1/K6`0B4Q805EEE@0a<C40?clo06QXJ01c
Lg<0AdM707EeM@1CDe<0EeMG05YJFP15ADD0GemO05ADE016ATH0GEeM04I6AP13@d<0EeMG04M7A`1_
Kfl0L71`05YJFP1FEUH0IFEU05EEE@17AdL0Bd];0P0n?Sh04P0_;bl0=CDe03<c<`0R8R80<C4a0492
@P1EEED0ATI602h^;P17AdL0>SXj04U9B@0k>c/0?clo04I6AP1CDe<0A4A4030`<080DE5A06L0EUIF
04i>CP1NGUh0OWin04i>CP1LG5`0ATI604e=C@2ATI40RHV904E5A@1<C4`0JVYZ06e]K@1IFET0Jf][
05YJFP1WIfL0<S8b05EEE@2YZJT0H61P069RHP1XJ6P0M7Ad07=cL`1XJ6P0QXJ605aLG01JFUX0L71`
06i^KP1aLG40K6a/055AD@0g=cL0KFe]03`l?01RHV80;Rh^06MWI`1FEUH0M7Ad04M7A`1/K6`0AdM7
06][J`1/K6`0EUIF069RHP1:BTX0ATI606YZJP0e=CD0IVIV07IfMP0g=cL0BTY:04i>CP1GEeL0EUIF
05ADE00C4a<0A4A4051@D00i>CT0DU9B034a<@1<C4`0=3@d04e=C@13@d<0<30`04=3@`0n?Sh0=cLg
05aLG00^;Rh0K6a/07alO012@T80=CDe024Q8@0k>c/0;Rh^05mOG`1VIVH0>SXj07alO00X:2P071`L
03Hf=P0O7al0?clo0451@@0];Bd0C4a<03Lg=`1WIfL0CDe=04U9B@1`L7008B4Q0080<c<c02809bLW
04I6AP0j>SX0=SHf03Ti>@1=CDd0=CDe03`l?00d=3@0>3Ph03Xj>P18B4P0<C4a06EUI@0o?cl0F5QH
03/k>`28R8P0DE5A04A4A01?Cdl0E5AD05IFEP1OGel0EeMG08Z:RP1ADE40LW9b05EEE@1OGel0I6AT
02h^;P1cLg<0Ng]k0P1]KFd204a<C00R02d];@1=CDd0@T9206i^KP1SHf<0IVIV07=cL`11@D40IVIV
06QXJ00n?Sh0OWin08V9R@1cLg<0>3Ph04E5A@19BDT0LW9b04A4A01;Bd/0N7Qh0=;BdP25QHD0MgMg
09^KV`1nOWh0H61P07alO01KFe/0I6AT06MWI`1_Kfl0PH6106IVIQ3ooooo000>ooooo`0^08R8R02D
U9@0CTi>05mOG`1ADE40KVi^05=CD`14A4@0EEEE05YJFP1ZJVX0>SXj02PX:01CDe<0LG5a03Hf=P1A
DE40BDU907YjNP2KVi/0@D5103Hf=P1CDe<0BTY:096AT@0Y:BT07AdM04i>CP1[Jf/0HV9R0820P00G
5aL07QhN05]KF`1/K6`0Bd];06IVIP0Z:RX0>c/k09BDU01aLG40FEUI07alO00/;2`0F5QH08B4Q080
KVi^0>H0T92@03Ti>@2DU9@0T92@03Ph>01cLg<0C4a<03@d=01cLg<0AdM70492@P1bLW80>SXj028R
8P1OGel0=cLg034a<@1EEED0?3`l04A4A01/K6`0AdM703hn?P0k>c/0De=C06MWI`1nOWh0=cLg02HV
9P1KFe/0@d=306QXJ02`/;008b<S03Lg=`1SHf<0BDU904E5A@1cLg<08R8R03`l?01@D500>SXj06UY
J@1=CDd06QXJ03Lg=`1BDU80:BTY04Y:BP15ADD0<c<c04A4A01ZJVX0<C4a03Ph>01:BTX0:b/[07Ui
N@2<S8`0:b/[02PX:01NGUh0H61P07IfMP35aLD0BDU903<c<`26QXH0JVYZ07MgM`29RHT0>3Ph02/[
:`1`L700IVIV08n?S`2KVi/0:BTY0451@@1CDe<0NGUi06m_K`1PH600<c<c04M7A`1KFe/0>3Ph05eM
G@1:BTX0>c/k06m_K`2d];@0Cdm?0410@01;Bd/0?3`l0:6QX@23Ph<09RHV04Y:BP1ADE40@41009>C
T`2a/K40<S8b03Ti>@1<C4`0@D5109bLW023Ph<0<C4a0451@@1@D500Hf=S09VIV@0Z:RX09RHV0410
@01YJFT0FUYJ05]KF`0N7Qh0EUIF07QhN01nOWh0?clo05QHF00V9RH0I6AT09RHV01JFUX0>c/k04m?
C`0l?3`0X:2P08n?S`0L71`0?clo06ATI01WIfL0NWYj02h^;P0_;bl0DE5A04i>CP1DE5@0IfMW02l_
;`1OGel0Rh^;07EeM@1jNWX0<S8b04=3@`1oOgl0KFe]0;Jf]P14A4@0?Cdm05iNGP1>CTh0N7Qh02h^
;P14A4@0FUYJ09RHV01<C4`0EeMG0410@01oOgl0Xj>S04e=C@1_Kfl0<C4a0410@01LG5`0Q8B403Ti
>@1DE5@0>c/k055AD@2_[jl0N7Qh03Hf=P1RHV80D51@05QHF02MWId0PX:202@T901@D500A4A4069R
HP2h^;P0Jf][028R8P1cLg<0U9BD05YJFP1fMWH0?3`l02d];@1NGUh0T92@07]kN`1^KVh0;bl_04E5
A@1;Bd/0IFEU03@d=00S8b<0;2`/04e=C@1QHF40CDe=01XJ6P0Z:RX0EUIF03`l?02OWil0L71`01/K
6`0W9bL0>SXj03lo?`2JVYX0?clo01LG5`kooooo07H0GemO05ADE01;Bd/0Bd];04i>CP1?Cdl0Kfm_
06IVIP14A4@0HF5Q09NGU`2IVIT0PX:206a/K027QhL0UIFE08^;R`1BDU80GUiN03Ph>00j>SX0GEeM
059BDP12@T80De=C08:2PP1DE5@0:2PX04E5A@1jNWX0DU9B01TI6@1;Bd/0JFUY05eMG@0a<C40L71`
0;^k^`11@D40?clo04=3@`0g=cL08R8R03@d=017AdL0FEUI04=3@`16ATH0@d=304i>CP13@d<0=CDe
04i>CP19BDT0@D5104a<C016ATH0CDe=05EEE@0e=CD0<S8b04Y:BP1<C4`0De=C04Q8B01IFET0DU9B
0410@01RHV80GEeM06=SH`1TI6@0DU9B055AD@1IFET0?clo05mOG`1FEUH0GUiN04E5A@11@D40CTi>
04Q8B00a<C40?Shn04E5A@11@D40De=C04m?C`0h>3P0EEEE04a<C01IFET0Bd];04E5A@1GEeL0AdM7
04];B`13@d<0DE5A034a<@1BDU80F5QH04U9B@1YJFT0@D51051@D01HF5P0MGEe07]kN`2@T900GUiN
08j>SP1eMGD0Bd];04Q8B01ADE40@D510P0j>SX02P1<C4`0F5QH079bLP1nOWh0L71`0:NWY`27QhL0
@41003@d=01BDU8306EUI@0P05UIF@1DE5@0Hf=S04M7A`1QHF40@T9204];B`0m?Cd0IfMW0492@P1P
H600AdM707MgM`1OGel0H61P03dm?@0Y:BT0?clo05mOG`10@400Bd];01LG5`0m?Cd0OWin028R8P1A
DE40FUYJ06i^KP1QHF40Hf=S01XJ6P19BDT204E5A@0V05UIF@0c<c<0B4Q802PX:01:BTX0EEEE03Lg
=`1@D500DE5A04=3@`1?Cdl0:BTY0492@P1UIFD0Cdm?03Ph>00b<S80>CTi04E5A@1IFET0Jf][03hn
?P1cLg<09RHV01dM7@0c<c<05aLG03dm?@0g=cL06ATI03dm?@13@d<0GUiN05]KF`0/;2`0ADE5024Q
8@0/;2`2038b<P1704M7A`0n?Sh0=CDe04U9B@1>CTh0;Bd]03Ti>@0Y:BT0@41003Lg=`18B4P0?Cdm
04a<C01HF5P0GEeM0410@00m?Cd0Bd];03Lg=`1iNGT0G5aL06YZJP1CDe<0JVYZ07alO01HF5P0Z:RX
06=SH`1HF5P0IVIV04Q8B01:BTX0KFe]04I6AP1PH600C4a<04I6AP0h>3P0Fe]K04a<C01TI6@0LW9b
07emO@23Ph<0PH6109FEU@1gMgL0EEEE07]kN`1;Bd/0LG5a05]KF`1PH600Cdm?0492@P1=CDd0Ogmo
07]kN`1ADE40JVYZ0:2PX026QXH0/[:b0861P@20P800UYJF0:2PX02HV9P0KVi^07=cL`1jNWX04?oo
ool00?oooooooooooomJooooo`00oooooooooooooe[ooooo003oooooooooooooF_ooool00?oooooo
oooooomJooooo`00oooooooooooooe[ooooo003oooooooooooooF_ooool00?oooooooooooomJoooo
o`00\
\>"],
  ImageRangeCache->{{{0, 599}, {299, 0}} -> {-0.0517664, -0.0250054, \
0.00351174, 0.00351174}, {{14.6875, 292.5}, {291.875, 152.938}} -> {-16.9197, \
-8.22585, 1.15187, 1.15187}, {{306.438, 584.25}, {291.875, 152.938}} -> \
{-352.977, -8.22585, 1.15187, 1.15187}, {{14.6875, 292.5}, {146, 7.0625}} -> \
{-16.9197, -176.255, 1.15187, 1.15187}, {{306.438, 584.25}, {146, 7.0625}} -> \
{-352.977, -176.255, 1.15187, 1.15187}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Import external model data and compute histograms", "Subsection"],

Cell[CellGroupData[{

Cell["Load histogram data and compute histograms", "Subsubsection"],

Cell[BoxData[{
    \(\(histData = 
        Map[ReadList[#, Number] &, histFiles, \ {2}]\  + \ 1;\)\), "\n", 
    \(\(hist = \(ComputeHistograms[#, vqlevels] &\)\  /@ \ 
          histData;\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Load VQ codebooks", "Subsubsection"],

Cell[BoxData[
    \(\(centroids\  = \ \(Import[#, \ "\<Table\>"] &\)\  /@ \ 
          centroidFiles;\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Visualize centroids by band", "Subsection"],

Cell["\<\
The graphics below illustrate the centroids by band (HL1, LH1, HH1, \
HL2, LH2, HH2) down the columns.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(VisualizeCentroids[#, ImageSize \[Rule] 500] &\)\  /@ \ 
        centroids;\)\)], "Input"],

Cell[CellGroupData[{

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .12 
%%ImageSize: 500 60 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0595238 0.00285714 0.0595238 [
[ 0 0 0 0 ]
[ 1 .12 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .12 L
0 .12 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.00285714 0.0782313 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
4EEE78531176A82C
BCCB930A0041CE4A
56FD5C2225B3AB29
EDB460041A55E666
7AEE4A0053DFA739
FC8E23163C77DD92
8ABD390977F38F4D
FF640A324E63C6C3
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0836735 0.00285714 0.138095 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
35764B22D76C2C28
49591F22BC511D25
3F7F251BF85C2C18
49520C26DD322620
475E182BFF302D1F
4C39002BDF172D16
4B3B103EE31B412B
421F0435BA153817
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.143537 0.00285714 0.197959 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
2A1B324683B781FF
1F18244891C98EE5
130015469BD07FF4
100813399ADE89DE
0E0010418DD189E8
1D1517418DCE9FE0
240B173A77C58CFE
242B254979958BE7
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.203401 0.00285714 0.257823 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0F0501361F8EEC0E
0E11043C22A1F615
060B004231BEFF07
010D003A2FD4F80D
0413024D44EDF100
02170D4A49EDDA07
102514625FF3C604
02111D4E52CDAA06
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.263265 0.00285714 0.317687 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0724B9493C261426
042AD03827251129
0028EB39472A1014
0A39E92030220B18
073CFF1C3B2D0F09
1A4EE3092E190E0B
1C53E41030281904
275CA20528141006
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.323129 0.00285714 0.377551 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
150114659F7E255D
49016FC28C2512FF
05002C8D8E5F1945
22239CB1710E0BE9
0B055FB1853C2B3E
1558B692450B08BB
15198CA4651C5334
139CC375221C06A2
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.382993 0.00285714 0.437415 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
306C59DA5F672BE2
21603FEF5A5923AD
0A715BE75B5D21AF
107D3DF345600C92
108962FF6C5B15A1
0F8D47EF4F600084
198081EB67601B8C
1E7760D5515E148F
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.442857 0.00285714 0.497279 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
343D33411D9B86A9
3828422530D76ADF
26352B3538C471B9
1A1E4D1433FF56DE
13352F3653C553B2
142C651B55F73AC1
0B3C374C71AC3F97
001A622260D4358D
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.502721 0.00285714 0.557143 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
DA882BDC9A383831
E8701ED989272330
E96A25F48316323D
E65B19F46A0D3737
E44831FB58094745
DD4425FF53004D45
BA3B4CE43314634D
B53E3DD842026249
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.562585 0.00285714 0.617007 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
7A482E27D2D44E3E
784F2F25E0D33730
81511C25F3DF2D39
81611032F9CC1E27
7860143CFFC71F2F
7367004FEAA50C21
6C5C1151E999132E
5E520260C3730214
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.622449 0.00285714 0.676871 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1B24251C1B100E65
15231E160F070D5D
222A2A180E001075
1C1E181A0F000C86
262324190E0111C9
261914110E0810F6
30151F14130F14FF
231613160B0B13B4
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.682313 0.00285714 0.736735 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
075553C3EB334107
095555D3F3084112
006269EDFD1B4812
056269ECF8024818
106D7FFFEF124314
186D7FEBD9074016
1B8195E8C5153C1D
247986C2B00E2C1D
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.742177 0.00285714 0.796599 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
24DA461127230234
27E5340B1B1E0228
28F936141B190329
2CFF2C0213170422
35F72A121317082B
30F721001A110C1E
41DB2D14161F0F33
35D11D041B111427
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.802041 0.00285714 0.856463 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
7A20177DA95024E5
39070D62AF8422BD
511A2688AB451AFF
21001C99C66514C1
39153B8EAD2B1BF1
160232C0B24B19BD
22295597971B1DD5
1F0842C0903D2A9A
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.861905 0.00285714 0.916327 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
DA691C250B0D3156
D76F232606062848
FF73131E100F215B
EE5F1318000E1D52
EA6E1A0C060C1D5E
D976231D06201E6F
C87E2D1E121B2364
B36C2327183C2877
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.921769 0.00285714 0.97619 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
68400E0049897E44
A9630E3D646F569E
622A0603749B6D43
9F3F0766656342DB
4A160519919A5339
83191D746B4F2BF1
310B09489F8A2E4C
720D3787603520FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.0627211 0.0782313 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
4B65D05F1C23943B
1AE0D87512467441
7F7BC744141AD41C
30FBC13C006F8319
9E939B2C0C2BF12A
62FFA41511999819
ABAA7022274CF144
71ED890931BD8441
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0836735 0.0627211 0.138095 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1C08362EDF201D2F
18043328E81D0D18
1000321FF5151214
0E092E1CFC180004
0E063123EE1D0706
0B09302BFF1B040C
17113138D927080A
15102E2AD2270005
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.143537 0.0627211 0.197959 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
181A09051F47C52E
181303090C40DE45
11140A051338E72D
121304061330FF3A
1107000B102EFE35
190D080B1E25FA3E
200206102324E83C
1B030A081D28D040
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.203401 0.0627211 0.257823 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
464C5A60382AC470
3950575E2624CA6A
355153612315F083
2C555E62101EF27B
23504F581115FF86
23495C4C042CF17D
1B494F460E30EA7B
1E334C390037C164
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.263265 0.0627211 0.317687 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
337D92DE44001CB4
1825C1B36B08592A
3CC48CB7180D2076
343EE3A1390BA42D
52F0907E00272C41
5466E37A0A2AD330
6EF9843D09384923
6160D9690051FF3A
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.323129 0.0627211 0.377551 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0E2B38DE3F222425
093143F534231412
08303FF732221B12
09313BFF2E200A0F
054147F235221910
0B3746EF2F231600
17454CD234301D18
172E47C3271E2005
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.382993 0.0627211 0.437415 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
6B43372C755AC0AA
63471B057466B0C1
5C3900005245AFA3
694D12055C56B3C7
622D0D00394DAFAC
704D20155F59BFDD
7A43161B555FC8BB
6B60272A5977AEFF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.442857 0.0627211 0.497279 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
2413583629E84613
1C12572D2EE92F05
240C632E26F92B05
1A0F5C3123F32608
190E59262DFF1A03
1C125B3229E92200
221F4D2532DF2005
2726492D33C71E12
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.502721 0.0627211 0.557143 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
BB373D0715173816
D5283A0D1B1C390C
E6274701061F3A0F
FF0C411013283913
FA10420E09353E24
F800371A11393627
E3022B1715453C36
BF03281B13322D41
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.562585 0.0627211 0.617007 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
B5A7500F506D666E
756D1E0951BAA427
BE7E2234556C67B1
7C5B110188CB8F27
B8530C51556D60F1
6846001DAEC16F36
A4262759626553FF
46430034C5B64D21
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.622449 0.0627211 0.676871 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
48291D0A030C20DD
43241D08020D1BFF
45251A08000C17F9
42301C11060D1BEA
47311A0B0A0F13AD
39291B150307147A
38351916120A1367
342214100F0C0C64
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.682313 0.0627211 0.736735 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
6C04B9E26FB06A4A
7100D5E857A7716F
6C02E3FF57BB6C6E
610AEAFD33AC716C
5B0DF4F834B75A8D
4526EED91CA75A76
441EE7D31E9E5091
2629D39F09814A7A
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.742177 0.0627211 0.796599 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
6CC7C03A2509261E
67DFC5152A003413
89F8C525290F2325
86F7BB0932043524
A0FFB21E34092D31
95E8A4082C0A4735
B0E9981A2D083A44
92B87D001B024534
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.802041 0.0627211 0.856463 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
431A7094615724F7
193317568A3F2F2F
2E0CAA98493B2BFF
053736627C311F17
1B29B093312A2BE4
003B535F75232305
22529E7A18262AA9
023E463C66102709
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.861905 0.0627211 0.916327 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
2B44D13B130F1337
1F40E1340E0B0430
2634EA3E070D0B32
1C38FF3B020F002C
2131F736090D0A25
2033EB35061D0A28
2233D5320F181B38
1D2FBC2B02171436
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.921769 0.0627211 0.97619 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
952E350D2B2B54D0
99122A01322F67E3
9016280D3B345DF5
8C062000482E68FA
800E240B4A3B67FF
76071F06522C6AF2
680C2314563664E8
580B180B4E2B57C5
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{500, 60},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007d0000?2000`400?l00000o`00003ooooooooe
ooooo`00oooooooeooooo`00oooooooeooooo`003oooool3odi>CPCok^k^0omhN7P4oe=CD`?o4A4A
1?mfMWH3ojRXZ0?o;2`/0_ooool4ocDe=@?oMWIf1?m;Bd/3ob8R8PCoemOG0om/K6`4ob`/;0;o:2PX
0oooool3obXZ:PCo6a/K0olb<S84odI6AP?oPh>31?ng]kL3oh61P@Gooooo1?l?3`l3o`D51@Co0@41
0olf=SH3oalO7`CoSXj>0oo/k>`3o`h>3P?ooooo0ol71`L4ob@T90?o^KVi0om9BDT4oc`l?0?o9RHV
1?lD51@2obHV9P?ooooo0olE5AD4o`410@?o51@D1?mUIFD3oinOW`CoOWin0olU9BD3oeeMG@;ooooo
1?l`<303ofa/K0CoFEUI0ooJf]X4oemOG`?oIfMW1?l[:b/2on;RhP?ooooo0old=3@4ocdm?@?o<c<c
1?m1@D43oadM7@CoVi^K0on6QXH3ojVYZ@;ooooo1?oJf]X3ohR8R0Co:b/[0ooLg=`4oiZJVPOo>3Ph
0_la<C43ooooo`?oNWYj1?m8B4P3obh^;PCo9bLW0ooBd]84omCDe0?oCTi>0oln?Sh2ooooo`Co6a/K
0olT92@4obDU9@?o71`L1?lK6a/3oa0@40?o3Ph>0omUIFD3ooooo`?o1`L71?mEEED3oe=CD`Co`l?3
0oo[jn/3oc<c<`Co@D510_l71`L3ooooo`Co92@T0ooJf]X3odI6APCo4A4A0olW9bL4ob<S8`?o0P82
0old=3@3ooooo`?oNWYj0olP8204oaLG5`?oOGem1?nYZJT3oe1@D0Co92@T0_oUiND3ooooo`?of][J
1?mYJFT3oa`L70Co9BDU0ol;2`/4o`d=3@?o<C4a0omFEUH2ooooo`CoJ6QX0om0@404o`h>3P?o0000
1?m9BDT3ohV9R@CoOWin0_m4A4@Aooooo`003oooool3odi>CPCok^k^0omhN7P4oe=CD`?o4A4A1?mf
MWH3ojRXZ0?o;2`/0_ooool4ocDe=@?oMWIf1?m;Bd/3ob8R8PCoemOG0om/K6`4ob`/;0;o:2PX0ooo
ool3obXZ:PCo6a/K0olb<S84odI6AP?oPh>31?ng]kL3oh61P@Gooooo1?l?3`l3o`D51@Co0@410olf
=SH3oalO7`CoSXj>0oo/k>`3o`h>3P?ooooo0ol71`L4ob@T90?o^KVi0om9BDT4oc`l?0?o9RHV1?lD
51@2obHV9P?ooooo0olE5AD4o`410@?o51@D1?mUIFD3oinOW`CoOWin0olU9BD3oeeMG@;ooooo1?l`
<303ofa/K0CoFEUI0ooJf]X4oemOG`?oIfMW1?l[:b/2on;RhP?ooooo0old=3@4ocdm?@?o<c<c1?m1
@D43oadM7@CoVi^K0on6QXH3ojVYZ@;ooooo1?oJf]X3ohR8R0Co:b/[0ooLg=`4oiZJVPOo>3Ph0_la
<C43ooooo`?oNWYj1?m8B4P3obh^;PCo9bLW0ooBd]84omCDe0?oCTi>0oln?Sh2ooooo`Co6a/K0olT
92@4obDU9@?o71`L1?lK6a/3oa0@40?o3Ph>0omUIFD3ooooo`?o1`L71?mEEED3oe=CD`Co`l?30oo[
jn/3oc<c<`Co@D510_l71`L3ooooo`Co92@T0ooJf]X3odI6APCo4A4A0olW9bL4ob<S8`?o0P820old
=3@3ooooo`?oNWYj0olP8204oaLG5`?oOGem1?nYZJT3oe1@D0Co92@T0_oUiND3ooooo`?of][J1?mY
JFT3oa`L70Co9BDU0ol;2`/4o`d=3@?o<C4a0omFEUH2ooooo`CoJ6QX0om0@404o`h>3P?o00001?m9
BDT3ohV9R@CoOWin0_m4A4@Aooooo`003oooool3odi>CPCok^k^0omhN7P4oe=CD`?o4A4A1?mfMWH3
ojRXZ0?o;2`/0_ooool4ocDe=@?oMWIf1?m;Bd/3ob8R8PCoemOG0om/K6`4ob`/;0;o:2PX0oooool3
obXZ:PCo6a/K0olb<S84odI6AP?oPh>31?ng]kL3oh61P@Gooooo1?l?3`l3o`D51@Co0@410olf=SH3
oalO7`CoSXj>0oo/k>`3o`h>3P?ooooo0ol71`L4ob@T90?o^KVi0om9BDT4oc`l?0?o9RHV1?lD51@2
obHV9P?ooooo0olE5AD4o`410@?o51@D1?mUIFD3oinOW`CoOWin0olU9BD3oeeMG@;ooooo1?l`<303
ofa/K0CoFEUI0ooJf]X4oemOG`?oIfMW1?l[:b/2on;RhP?ooooo0old=3@4ocdm?@?o<c<c1?m1@D43
oadM7@CoVi^K0on6QXH3ojVYZ@;ooooo1?oJf]X3ohR8R0Co:b/[0ooLg=`4oiZJVPOo>3Ph0_la<C43
ooooo`?oNWYj1?m8B4P3obh^;PCo9bLW0ooBd]84omCDe0?oCTi>0oln?Sh2ooooo`Co6a/K0olT92@4
obDU9@?o71`L1?lK6a/3oa0@40?o3Ph>0omUIFD3ooooo`?o1`L71?mEEED3oe=CD`Co`l?30oo[jn/3
oc<c<`Co@D510_l71`L3ooooo`Co92@T0ooJf]X3odI6APCo4A4A0olW9bL4ob<S8`?o0P820old=3@3
ooooo`?oNWYj0olP8204oaLG5`?oOGem1?nYZJT3oe1@D0Co92@T0_oUiND3ooooo`?of][J1?mYJFT3
oa`L70Co9BDU0ol;2`/4o`d=3@?o<C4a0omFEUH2ooooo`CoJ6QX0om0@404o`h>3P?o00001?m9BDT3
ohV9R@CoOWin0_m4A4@Aooooo`003oooool3okbl_0Cobl_;0onCTi<4o`X:2P?o00001?m1@D43olk>
cP?oBTY:0_ooool4odU9B@?oFEUI1?lO7al3ob8R8PCo_;bl0omADE44oadM7@;o9BDU0oooool3oalO
7`Co61PH0olT92@4odQ8B0?oTI6A1?o9bLT3ohj>SP;oiNGU0oooool4o`h>3P?o4A4A1?l410@3oc`l
?0?o8R8R1?nQXJ43ooKfmP?o5ADE0oooool3o`@410Co:RXZ0oo@d=03ocPh>0Co9bLW0olU9BD4oa4A
4@;o:BTY0oooool3odU9B@Co0@410om_Kfl4ol;2`P?oS8b<1?lU9BD3oa8B4PGooooo1?lQ8B43of1P
H0Co?clo0oo_knl4oeYJFP?oFEUI1?lS8b<2ojf][@?ooooo0olh>3P4obPX:0?o@T921?lU9BD3oc0`
<0CoemOG0omZJVX3omoOg`;ooooo1?oXj>P3og1`L0Co7QhN0ooIfMT4ohV9R@?o9bLW1?lS8b<2oc0`
<0?ooooo0omhN7P4odm?C`?o;bl_1?lU9BD3on3Ph0Codm?C0olg=cL3oc0`<0;ooooo1?lE5AD3ob<S
8`Co7QhN0olF5QH4o`l?3`?o1`L70ol=3@d3oeeMG@?ooooo0ol92@T7oeEEE@Codm?C0ooclo<3o`P8
20Co@D510_lB4Q83ooooo`Co9bLW0ooUiND3oc@d=0Co2`/;0olK6a/4oahN7P?o0P820olX:2P3oooo
o`?o>CTi0ol71`L4o`d=3@?oHV9R1?n_[jl3ohB4Q0Co8R8R0_nm_Kd3ooooo`?oemOG1?m_Kfl3ob<S
8`Co9RHV1ol61PH3obPX:0?oB4Q80_ooool4ojVYZ@?oHf=S1?l>3Ph3ocdm?@CoI6AT0om_Kfl4oeIF
EP;oWYjN4Oooool000oooooo0onl_;`4ol_;b`?oTi>C1?l:2PX3o`0000Co@D510oo>c/h3odY:BP;o
oooo1?m9BDT3oeUIF@Co7alO0olR8R84okbl_0?oDE5A1?lM7Ad2obDU9@?ooooo0olO7al4oaPH60?o
92@T1?m8B4P3oi6AT@CobLW90on>SXh2onGUi@?ooooo1?l>3Ph3oa4A4@Co10@40oll?3`3ob8R8PCo
XJ6Q0oofm_H3oaDE5@?ooooo0ol410@4obXZ:P?od=3@0olh>3P4obLW9`?o9BDU1?lA4A42obTY:@?o
oooo0om9BDT4o`410@?oKfm_1?o2`/83ohb<S0Co9BDU0olB4Q85ooooo`Co8B4Q0omPH604oclo?`?o
kno_1?mJFUX3oeUIF@Co8b<S0_n][Jd3ooooo`?o>3Ph1?lX:2P3od92@PCo9BDU0ol`<304omOGe`?o
JVYZ0ooOgml2ooooo`Coj>SX0om`L704oahN7P?ofMWI1?n9RHT3obLW9`Co8b<S0_l`<303ooooo`?o
N7Qh1?m?Cdl3obl_;`Co9BDU0ooPh>04om?Cd`?o=cLg0ol`<302ooooo`Co5ADE0olS8b<4oahN7P?o
5QHF1?l?3`l3o`L71`?o3@d=0omMGEd3ooooo`?o2@T91omEEED4om?Cd`?olo?c0ol820P4od51@@;o
4Q8B0oooool4obLW9`?oiNGU0old=3@4o`/;2`?o6a/K1?lN7Qh3o`820P?o:2PX0oooool3ocTi>@?o
1`L71?l=3@d3of9RHPCo[jn_0on4Q8@4ob8R8P;o_Kfm0oooool3omOGe`CoKfm_0olS8b<4obHV9POo
1PH60olX:2P3odQ8B0;ooooo1?nYZJT3of=SH`Co3Ph>0olm?Cd4ofATI0?oKfm_1?mFEUH2oijNWQ7o
oooo000?ooooo`?o_;bl1?o;bl/3oi>CT`Co2PX:0ol00004od51@@?oc/k>0om:BTX2ooooo`CoBDU9
0omIFET4oalO7`?o8R8R1?nl_;`3oe5AD@Co7AdM0_lU9BD3ooooo`?o7alO1?lH61P3ob@T90CoB4Q8
0onATI44olW9b@?oSXj>0_oUiND3ooooo`Co3Ph>0olA4A44o`@410?o?3`l0olR8R84oj6QX@?om_Kf
0olE5AD3ooooo`?o10@41?lZ:RX3om3@d0?o>3Ph1?lW9bL3obDU9@Co4A4A0_lY:BT3ooooo`?oBDU9
1?l10@43ofm_K`Co`/;20on<S8`4obDU9@?o4Q8B1Oooool4ob4Q8@?oH61P1?lo?cl3ono_k`CoFUYJ
0omIFET4ob<S8`;o[Jf]0oooool3ocPh>0Co:2PX0om2@T84obDU9@?o<30`1?oGemL3ofYZJP?ogmoO
0_ooool4onSXj0?oL71`1?lN7Qh3omWIf@CoRHV90olW9bL4ob<S8`;o<30`0oooool3ogQhN0CoCdm?
0ol_;bl4obDU9@?oh>3P1?oCdm<3ocLg=`?o<30`0_ooool4oaDE5@?o8b<S1?lN7Qh3oaHF5PCo3`l?
0ol71`L3o`d=3@?oGEeM0oooool3o`T92@OoEEEE1?oCdm<3oo?cl`?o20P81?m1@D42oa8B4P?ooooo
1?lW9bL3onGUi@?o=3@d1?l;2`/3oa/K6`Co7QhN0ol20P83obPX:0?ooooo0oli>CT3o`L71`Co3@d=
0omRHV84ojn_[`?oQ8B41?lR8R82okfm_@?ooooo0ooGemL4ofm_K`?o8b<S1?lV9RH7o`H61P?o:2PX
0om8B4P2ooooo`CoZJVY0omSHf<4o`h>3P?o?Cdm1?mTI6@3ofm_K`CoEUIF0_nNWYhAooooo`003ooo
ool3oeIFEPCooOgm0omLG5`4ob8R8P?o9BDU1?nc/k<3oj^[Z`?o:BTY0_ooool4oclo?`?oOgmo1?lU
9BD3oa/K6`Con?Sh0omLG5`4ob`/;0;o61PH0oooool3oa<C4`Co00000olE5AD4odI6AP?oVi^K1?o@
d=03ogmoO`;om?Cd0oooool4o`H61P?o2`/;1?l00003od92@P?o<C4a1?nn_[h3ooooo`?o1`L70ooo
ool3o`0000Co:2PX0oo[jn/3ocTi>@CoAdM70olZ:RX4oa0@40;o51@D0oooool3o`D51@Co00000ol/
;2`4ohf=S@?oSXj>1?mOGel3oaTI6@?oADE50_ooool4o`X:2P?oLG5a1?mKFe/3onOWi`CoFe]K0omM
GEd4ob4Q8@;o[jn_0oooool3obHV9PCo=CDe0ol[:b/4ocDe=@?o>3Ph1?o4a<@3og5aL@?o^KVi0_oo
ool4onWYj@?oJVYZ1?lU9BD3ooCdm0CoPh>30olF5QH4oc8b<P;o?Cdm0oooool3oh61P@CoDE5A0olL
71`4obDU9@?olo?c1?oOgml3obd];@?o>CTi0_ooool4ob8R8POo:RXZ0olH61P4o`h>3P?o00000ol@
4103ogEeM@?ooooo0ol00004of9RHP?oJFUY1?o]kNd3oogmo@?o6a/K1?m8B4P2oa8B4P?ooooo1?lX
:2P3ooWin@?o=SHf1?lD51@3oa/K6`Co6ATI0ol30`<3obTY:@?ooooo0omADE43oaXJ6PCo9RHV0on8
R8P4oj^[Z`?oADE51?lJ6QX8ooooo`CoLg=c0olC4a<4oahN7P?o410@1?l?3`l3ob4Q8@?oFe]K0_oo
ool4of9RHP?o:RXZ1?l61PH3o`<30`CoM7Ad0onKVi/4ofe]K@;o@d=34Oooool000oooooo0omFEUH4
oogmo@?oG5aL1?lR8R83obDU9@Co/k>c0on[Zj/3obTY:@;ooooo1?lo?cl3ogmoO`Co9BDU0olK6a/4
ooShn0?oG5aL1?l/;2`2oaPH60?ooooo0olC4a<4o`0000?o5ADE1?m6ATH3oi^KV`Cod=3@0omoOgl2
ooCdm0?ooooo1?l61PH3o`/;2`Co00000om2@T83oc4a<@Co_[jn0oooool3o`L71`?ooooo0ol00004
obPX:0?ojn_[0oli>CT4odM7A`?o:RXZ1?l@4102oa@D50?ooooo0ol51@D4o`0000?o;2`/1?n=SHd3
ohj>SPCoGemO0olI6AT3odE5A@;ooooo1?l:2PX3og5aL@CoFe]K0ooWinL4oe]KF`?oGEeM1?lQ8B42
ojn_[`?ooooo0olV9RH4ocDe=@?o:b/[1?le=CD3ocPh>0Coa<C40omaLG43okVi^@;ooooo1?oYjNT3
ofYZJPCo9BDU0oodm?@4oh>3P`?o5QHF1?lb<S82ocdm?@?ooooo0on1PH44oe5AD@?o71`L1?lU9BD3
oo?cl`CogmoO0ol];Bd3ocTi>@;ooooo1?lR8R87obXZ:P?o61PH1?l>3Ph3o`0000?o410@0omeMGD3
ooooo`?o00001?mRHV83ofUYJ@CokNg]0oomoOd3oa/K6`CoB4Q80_lB4Q83ooooo`Co:2PX0ooinOT3
ocHf=PCo51@D0olK6a/4oaTI6@?o0`<30olY:BT3ooooo`?oDE5A0olJ6QX4obHV9P?oR8R81?n[Zj/3
odE5A@Co6QXJ2?ooool4og=cL`?o4a<C1?lN7Qh3oa0@40Co3`l?0olQ8B43oe]KF`;ooooo1?mRHV83
obXZ:PCo1PH60ol30`<4ogAdM0?oVi^K1?m]KFd2od=3@a7ooooo000?ooooo`?oEUIF1?omoOd3oeaL
G0Co8R8R0olU9BD4ok>c/`?oZj^[0olY:BT2ooooo`Co?clo0omoOgl4obDU9@?o6a/K1?ohn?P3oeaL
G0Co;2`/0_lH61P3ooooo`?o4a<C1?l00003oaDE5@CoATI60onKVi/4om3@d0?oOgmo0_odm?@3oooo
o`Co1PH60ol;2`/4o`0000?o@T920ola<C44okjn_P?ooooo0ol71`L3ooooo`?o00001?lX:2P3on_[
j`?o>CTi1?m7AdL3obXZ:PCo410@0_lD51@3ooooo`?o1@D51?l00003ob`/;0CoSHf=0on>SXh4oemO
G`?o6ATI0om5ADD2ooooo`Co2PX:0omaLG44oe]KF`?oinOW1?mKFe/3oeeMG@Co8B4Q0_n_[jl3oooo
o`?o9RHV1?le=CD3ob/[:`Co=CDe0olh>3P4olC4a0?oLG5a0oni^KT2ooooo`CojNWY0omZJVX4obDU
9@?om?Cd1?n3Ph<3oaHF5PCo<S8b0_lm?Cd3ooooo`?oPH611?mADE43oa`L70Co9BDU0ooclo<4omoO
g`?o;Bd]0oli>CT2ooooo`Co8R8R1olZ:RX3oaPH60Co3Ph>0ol00003oa0@40?oMGEe0oooool3o`00
00CoHV9R0omYJFT4ong]k@?ooOgm0olK6a/4odQ8B0;o4Q8B0oooool4obPX:0?onOWi0olf=SH4oa@D
50?o6a/K1?lI6AT3o`<30`?o:BTY0oooool3oe5AD@?o6QXJ1?lV9RH3ohR8R0CoZj^[0om5ADD4oaXJ
6PSooooo1?mcLg<3oa<C4`Co7QhN0ol@4104o`l?3`?o8B4Q0omKFe/2ooooo`CoHV9R0olZ:RX4o`H6
1P?o0`<31?mdM7@3oi^KV`CoKFe]0_m3@d<Aooooo`003oooool3oeIFEPCooOgm0omLG5`4ob8R8P?o
9BDU1?nc/k<3oj^[Z`?o:BTY0_ooool4oclo?`?oOgmo1?lU9BD3oa/K6`Con?Sh0omLG5`4ob`/;0;o
61PH0oooool3oa<C4`Co00000olE5AD4odI6AP?oVi^K1?o@d=03ogmoO`;om?Cd0oooool4o`H61P?o
2`/;1?l00003od92@P?o<C4a1?nn_[h3ooooo`?o1`L70oooool3o`0000Co:2PX0oo[jn/3ocTi>@Co
AdM70olZ:RX4oa0@40;o51@D0oooool3o`D51@Co00000ol/;2`4ohf=S@?oSXj>1?mOGel3oaTI6@?o
ADE50_ooool4o`X:2P?oLG5a1?mKFe/3onOWi`CoFe]K0omMGEd4ob4Q8@;o[jn_0oooool3obHV9PCo
=CDe0ol[:b/4ocDe=@?o>3Ph1?o4a<@3og5aL@?o^KVi0_ooool4onWYj@?oJVYZ1?lU9BD3ooCdm0Co
Ph>30olF5QH4oc8b<P;o?Cdm0oooool3oh61P@CoDE5A0olL71`4obDU9@?olo?c1?oOgml3obd];@?o
>CTi0_ooool4ob8R8POo:RXZ0olH61P4o`h>3P?o00000ol@4103ogEeM@?ooooo0ol00004of9RHP?o
JFUY1?o]kNd3oogmo@?o6a/K1?m8B4P2oa8B4P?ooooo1?lX:2P3ooWin@?o=SHf1?lD51@3oa/K6`Co
6ATI0ol30`<3obTY:@?ooooo0omADE43oaXJ6PCo9RHV0on8R8P4oj^[Z`?oADE51?lJ6QX8ooooo`Co
Lg=c0olC4a<4oahN7P?o410@1?l?3`l3ob4Q8@?oFe]K0_ooool4of9RHP?o:RXZ1?l61PH3o`<30`Co
M7Ad0onKVi/4ofe]K@;o@d=34Oooool000oooooo0oo]kNd4okBd]0?oH61P1?l410@3oaXJ6PCoEEEE
0ooVi^H3ofIVIP;ooooo1?m9BDT3oe9BDPCo30`<0olV9RH4omgMg@?o<S8b1?lV9RH2ob0P80?ooooo
0ol@4104o`P820?o4a<C1?li>CT3oiZJVPCog]kN0on9RHT2omkNgP?ooooo1?l10@43o`d=3@Co0000
0olj>SX3obl_;`Coe=CD0oohn?P3o`d=3@?ooooo0ol:2PX4ocTi>@?ojNWY0olP8204oc0`<0?o8R8R
1?l;2`/2oaPH60?ooooo0olR8R84ob<S8`?oW9bL1?na/K43og5aL@Co3Ph>0ol;2`/3onWYj@;ooooo
1?l@4103ogemO@Co?Cdm0ooclo<4odE5A@?oH61P1?l<30`2oi:BTP?ooooo0olJ6QX4oahN7P?oCDe=
1?lD51@3oc<c<`Cooooo0omFEUH3omkNgP;ooooo1?oVi^H3oe]KF`Co6ATI0oodm?@4ofYZJP?o3@d=
1_lg=cL3ooooo`?oPH611?mQHF43oa0@40Co<S8b0ooinOT4olc<c0?o7QhN0olW9bL2ooooo`Co71`L
0olN7Qh4oaPH60?o6QXJ1?l?3`l3o`0000?o30`<0on6QXH3ooooo`?o1@D51?mRHV83ofUYJ@Cok>c/
0oohn?P3o`820PCoB4Q80_lH61P3ooooo`Co;2`/0oooool3ob`/;0Co0P820olC4a<4oaLG5`?o10@4
0olR8R83ooooo`?o8B4Q0ol00004oa`L70?oVIVI1?o6a/H3ofEUI@Co51@D0_o1`L43ooooo`?ok^k^
1?mOGel3oa<C4`Co61PH0ol00004o`h>3P?o7AdM0omBDU82ooooo`CoWinO0olo?cl4o`L71`?oIVIV
1?mUIFD3of=SH`Co@T920_oKfm/Aooooo`003oooool3ong]k@Co];Bd0omPH604o`@410?o6QXJ1?mE
EED3onKViP?oIVIV0_ooool4odU9B@?oDU9B1?l<30`3obHV9PCogMgM0olb<S84obHV9P;o820P0ooo
ool3oa0@40Co20P80olC4a<4ocTi>@?oVYZJ1?oNg]h3ohV9R@;og]kN0oooool4o`410@?o3@d=1?l0
0003ocXj>P?o;bl_1?oDe=@3ooShn0?o3@d=0oooool3o`X:2PCo>CTi0ooYjNT3ob0P80Co<30`0olR
8R84o`/;2`;o61PH0oooool3ob8R8PCo8b<S0onLW9`4ok6a/@?oLG5a1?l>3Ph3o`/;2`?ojNWY0_oo
ool4oa0@40?oOGem1?lm?Cd3oo?cl`CoADE50omPH604o``<30;oTY:B0oooool3oaXJ6PCo7QhN0om=
CDd4oa@D50?o<c<c1?ooool3oeIFEP?og]kN0_ooool4onKViP?oFe]K1?lI6AT3ooCdm0CoJVYZ0ol=
3@d6ocLg=`?ooooo0on1PH44of5QH@?o410@1?lb<S83ooWin@Coc<c<0olN7Qh3obLW9`;ooooo1?lL
71`3oahN7PCo61PH0olJ6QX4o`l?3`?o00000ol<30`3ohJ6QP?ooooo0ol51@D4of9RHP?oJFUY1?o/
k>`3ooShn0?o0P821?m8B4P2oaPH60?ooooo1?l/;2`3ooooo`?o;2`/1?l20P83oa<C4`Co5aLG0ol4
10@3ob8R8P?ooooo0olQ8B43o`0000Co71`L0onIVIT4olK6aP?oIFEU1?lD51@2ol71`@?ooooo0oo^
k^h4oemOG`?o4a<C1?lH61P3o`0000Co3Ph>0olM7Ad3oe9BDP;ooooo1?nOWil3oclo?`Co1`L70omV
IVH4ofEUI@?oHf=S1?m2@T82om_Kfa7ooooo000?ooooo`?okNg]1?nd];@3of1PH0Co10@40olJ6QX4
oeEEE@?oi^KV0omVIVH2ooooo`CoBDU90omBDU84o``<30?o9RHV1?oMgMd3oc8b<PCo9RHV0_lP8203
ooooo`?o410@1?l820P3oa<C4`Co>CTi0onJVYX4omkNgP?oRHV90_oNg]h3ooooo`Co0@410ol=3@d4
o`0000?o>SXj0ol_;bl4omCDe0?on?Sh0ol=3@d3ooooo`?o2PX:1?li>CT3onWYj@?o820P1?l`<303
ob8R8PCo2`/;0_lH61P3ooooo`?o8R8R1?lS8b<3oibLW0Co/K6a0omaLG44o`h>3P?o2`/;0ooYjNT2
ooooo`Co410@0ommOGd4ocdm?@?olo?c1?m5ADD3of1PH0Co30`<0_nBTY83ooooo`?o6QXJ1?lN7Qh3
ode=C@Co51@D0olc<c<4ooooo`?oEUIF0ooNg]h2ooooo`Coi^KV0omKFe/4oaTI6@?om?Cd1?mZJVX3
o`d=3@Ko=cLg0oooool3oh61P@CoHF5Q0ol@4104oc8b<P?onOWi1?o<c<`3oahN7P?o9bLW0_ooool4
oa`L70?o7QhN1?lH61P3oaXJ6PCo3`l?0ol00003o``<30?oQXJ60oooool3o`D51@CoHV9R0omYJFT4
onc/k0?on?Sh0ol20P84odQ8B0;o61PH0oooool4ob`/;0?ooooo0ol/;2`4o`820P?o4a<C1?lG5aL3
o`@410?o8R8R0oooool3ob4Q8@?o00001?lL71`3oiVIV@Coa/K60omUIFD4oa@D50;o`L710oooool3
onk^kPCoGemO0olC4a<4oaPH60?o00001?l>3Ph3oadM7@?oDU9B0_ooool4oinOW`?o?clo1?l71`L3
ofIVIPCoIFEU0omSHf<4od92@P;ofm_K4Oooool000oooooo0omjNWX4onk^kP?oBTY:1?l00003oe=C
D`CogmoO0onWYjL3ocTi>@;ooooo1?m7AdL3oeiNGPCo61PH0ol[:b/4ooooo`?o<30`1?l];Bd2oalO
7`?ooooo0ol>3Ph4o`0000?o410@1?m1@D43ohf=S@CodM7A0on9RHT2onSXj0?ooooo1?l410@3oa<C
4`Co0P820om=CDd3odA4A0CokNg]0ooalO43o`0000?ooooo0ol71`L4oc`l?0?ooooo0olL71`4oc/k
>`?o;Bd]1?l?3`l2o`T92@?ooooo0ol;2`/4o`D51@?oGemO1?na/K43ohF5Q@Co?3`l0ol[:b/3ochn
?P;ooooo1?l@4103ohV9R@CoHV9R0oooool4ofa/K0?oFe]K1?lE5AD2oj6QX@?ooooo0olC4a<4ocDe
=@?o;bl_1?lf=SH3oe=CD`CoaLG50omCDe<3ok:b/P;ooooo1?oTi>@3odQ8B0Co<C4a0ookno/4oeQH
F0?o2@T91?m7AdL2odE5A@?ooooo0omhN7P4of1PH0?o51@D1?ll?3`3ooooo`CoalO70olO7al3obl_
;`;ooooo1?lV9RH3ob<S8`Co92@T0olI6AT4o`h>3P?o0@410olA4A43olW9b@?ooooo0ol@4104ofe]
K@?oOgmo1?ooool3ono_k`?o4Q8B1?m3@d<2oa@D50?ooooo1?le=CD3ooOgm`?o:RXZ1?lB4Q83oa<C
4`Co5aLG0ol820P3ob/[:`?ooooo0oli>CT3oaDE5@Co>c/k0on>SXh4ojf][@?o:b/[1?lK6a/2oo7a
l@?ooooo0ooZj^X4ofi^KP?o6QXJ1?l<30`3o`H61PCo30`<0olM7Ad3oeiNGP;ooooo1?m:BTX3oaHF
5PCo1@D50olI6AT4oi6AT@?oVYZJ1?mCDe<2ocTi>A7ooooo000?ooooo`?oNWYj1?o^k^h3odY:BPCo
00000omCDe<4omoOg`?oYjNW0oli>CT2ooooo`CoAdM70omNGUh4oaPH60?o:b/[1?ooool3oc0`<0Co
;Bd]0_lO7al3ooooo`?o3Ph>1?l00003oa0@40Co@D510on=SHd4om7Ad@?oRHV90_oXj>P3ooooo`Co
10@40olC4a<4o`820P?oCDe=0om4A4@4ong]k@?olO7a0ol00003ooooo`?o1`L71?ll?3`3ooooo`?o
71`L1?lk>c/3obd];@Co3`l?0_l92@T3ooooo`?o2`/;1?l51@D3oemOG`Co/K6a0on5QHD4oc`l?0?o
:b/[0oln?Sh2ooooo`Co410@0on9RHT4of9RHP?ooooo1?m/K6`3oe]KF`Co5ADE0_nQXJ43ooooo`?o
4a<C1?le=CD3obl_;`Co=SHf0omCDe<4olG5a@?oDe=C0onb/[82ooooo`Coi>CT0om8B4P4oc4a<@?o
no_k1?mHF5P3o`T92@CoAdM70_m5ADD3ooooo`?oN7Qh1?mPH603oa@D50Co?3`l0oooool4olO7a`?o
7alO0ol_;bl2ooooo`Co9RHV0olS8b<4ob@T90?o6ATI1?l>3Ph3o`410@?o4A4A0oo9bLT3ooooo`?o
410@1?m]KFd3ogmoO`Cooooo0oo_knl3oa8B4PCo@d=30_lD51@3ooooo`Co=CDe0oogmoL3obXZ:PCo
4Q8B0olC4a<4oaLG5`?o20P80ol[:b/3ooooo`?o>CTi0olE5AD4oc/k>`?oSXj>1?n][Jd3ob/[:`Co
6a/K0_oalO43ooooo`?oj^[Z1?m^KVh3oaXJ6PCo30`<0ol61PH4o``<30?o7AdM0omNGUh2ooooo`Co
BTY:0olF5QH4o`D51@?o6ATI1?nATI43oiZJVPCoDe=C0_li>CTAooooo`003oooool3ogYjNPCok^k^
0om:BTX4o`0000?oDe=C1?oOgml3ojNWY`?o>CTi0_ooool4odM7A`?oGUiN1?lH61P3ob/[:`Cooooo
0ol`<304obd];@;o7alO0oooool3o`h>3PCo00000ol@4104od51@@?oSHf=1?oAdM43ohV9R@;oj>SX
0oooool4o`@410?o4a<C1?l20P83ode=C@?oA4A41?o]kNd3oo7al@?o00000oooool3o`L71`Co?3`l
0oooool3oa`L70Co>c/k0ol];Bd4o`l?3`;o2@T90oooool3o`/;2`Co1@D50omOGel4ok6a/@?oQHF5
1?ll?3`3ob/[:`?o?Shn0_ooool4oa0@40?oRHV91?mRHV83ooooo`CoK6a/0omKFe/4oaDE5@;oXJ6Q
0oooool3oa<C4`Co=CDe0ol_;bl4ocHf=P?oDe=C1?o5aLD3oe=CD`?o/[:b0_ooool4onCTi0?oB4Q8
1?la<C43oo_kn`CoF5QH0ol92@T4odM7A`;oADE50oooool3ogQhN0CoH61P0olD51@4oc`l?0?ooooo
1?o7alL3oalO7`?o;bl_0_ooool4obHV9P?o8b<S1?lT92@3oaTI6@Co3Ph>0ol10@43oa4A4@?obLW9
0oooool3oa0@40CoKFe]0omoOgl4ooooo`?okno_0olB4Q84od=3@`;o51@D0oooool4ocDe=@?omoOg
0olZ:RX4oa8B4P?o4a<C1?lG5aL3o`P820?o:b/[0oooool3ocTi>@?o5ADE1?lk>c/3ohj>SPCo[Jf]
0ol[:b/4oa/K6`;olO7a0oooool3on[ZjPCoKVi^0olJ6QX4o``<30?o1PH61?l<30`3oadM7@?oGUiN
0_ooool4odY:BP?o5QHF1?l51@D3oaTI6@CoTI6A0onJVYX4oe=CD`;o>CTi4Oooool000oooooo0ool
o?`4ohj>SP?o8b<S1?lF5QH3oc`l?0CoMgMg0ooMgMd3oi:BTP;ooooo1?m<C4`3ocTi>@Co00000ol[
:b/4omoOg`?o5aLG1?l];Bd2oaHF5P?ooooo0olM7Ad4oaDE5@?o5aLG1?m1@D43ohf=S@Coc/k>0onO
Wil2on3Ph0?ooooo1?l20P83oaLG5`Co3@d=0om:BTX3odU9B@CokNg]0ooJf]X3o`L71`?ooooo0olJ
6QX4odi>CP?ohn?S0ol92@T4obh^;P?o6ATI1?l>3Ph2o`/;2`?ooooo0olE5AD4oeQHF0?o][Jf1?nB
TY83odE5A@Co2`/;0ol820P3ok^k^`;ooooo1?l?3`l3ohf=S@CoAdM70oo_knl4odm?C`?oH61P1?l0
0002ohB4Q0?ooooo0olD51@4ob`/;0?oIFEU1?lK6a/3oeEEE@ComoOg0olj>SX3ol71`@;ooooo1?oM
gMd3odA4A0Co9BDU0oooool4oe=CD`?o00001?m=CDd2odE5A@?ooooo0omcLg<4ofMWI`?o00001?m?
Cdl3on[ZjPCoYJFU0ol<30`3ob4Q8@;ooooo1?lV9RH3oaTI6@Co51@D0olA4A44o`h>3P?o20P80ol@
4103ooKfmP?ooooo0olH61P4ofe]K@?oOgmo1?o[jn/3omWIf@?o1`L71?m0@402oaHF5P?ooooo1?l`
<303ooOgm`?o8B4Q1?l00003oaXJ6PCo4A4A0ol<30`3oahN7P?ooooo0olF5QH3o`820PCo<S8b0oo0
`<04ok:b/P?oBd];1?lI6AT2okfm_@?ooooo0ooIfMT4ogIfMP?o8b<S1?lM7Ad3o`H61PCo820P0olN
7Qh3ofm_K`;ooooo1?n3Ph<3oaTI6@Co7AdM0omdM7@4of][J`?oCdm?1?l[:b/2oo7alA7ooooo000?
ooooo`?oo?cl1?n>SXh3ob<S8`Co5QHF0oll?3`4ogMgM`?ogMgM0onBTY82ooooo`CoC4a<0oli>CT4
o`0000?o:b/[1?oOgml3oaLG5`Co;Bd]0_lF5QH3ooooo`?o7AdM1?lE5AD3oaLG5`Co@D510on=SHd4
olk>cP?oWinO0_oPh>03ooooo`Co0P820olG5aL4o`d=3@?oBTY:0om9BDT4ong]k@?of][J0ol71`L3
ooooo`?o6QXJ1?m>CTh3on?Sh`?o2@T91?l^;Rh3oaTI6@Co3Ph>0_l;2`/3ooooo`?o5ADE1?mHF5P3
okJf]PCoTY:B0om5ADD4o`/;2`?o20P80onk^k/2ooooo`Co3`l?0on=SHd4odM7A`?okno_1?m?Cdl3
of1PH0Co00000_n4Q8@3ooooo`?o51@D1?l/;2`3ofEUI@Co6a/K0omEEED4ooOgm`?o>SXj0oo1`L42
ooooo`CogMgM0om4A4@4obDU9@?ooooo1?mCDe<3o`0000CoCDe=0_m5ADD3ooooo`?oLg=c1?mWIfL3
o`0000CoCdm?0ooZj^X4ojFUY@?o30`<0olQ8B42ooooo`Co9RHV0olI6AT4oa@D50?o4A4A1?l>3Ph3
o`P820?o410@0oofm_H3ooooo`?o61PH1?m]KFd3ogmoO`Cojn_[0ooIfMT3o`L71`Co@4100_lF5QH3
ooooo`Co<30`0oogmoL3ob4Q8@Co00000olJ6QX4oa4A4@?o30`<0olN7Qh3ooooo`?o5QHF0ol20P84
oc8b<P?o`<301?nb/[83od];B`Co6ATI0_nm_Kd3ooooo`?ofMWI1?mfMWH3ob<S8`Co7AdM0ol61PH4
ob0P80?o7QhN0om_Kfl2ooooo`CoPh>30olI6AT4oadM7@?oM7Ad1?m[Jf/3odm?C`Co:b/[0_oalO4A
ooooo`003oooool3ooclo0CoSXj>0olS8b<4oaHF5P?o?3`l1?mgMgL3omgMg@?oTY:B0_ooool4oda<
C0?o>CTi1?l00003ob/[:`CogmoO0olG5aL4obd];@;o5QHF0oooool3oadM7@Co5ADE0olG5aL4od51
@@?oSHf=1?o>c/h3oinOW`;oh>3P0oooool4o`820P?o5aLG1?l=3@d3odY:BP?oBDU91?o]kNd3om[J
fP?o1`L70oooool3oaXJ6PCoCTi>0ooShn<3o`T92@Co;Rh^0olI6AT4o`h>3P;o2`/;0oooool3oaDE
5@CoF5QH0onf][H4oi:BTP?oADE51?l;2`/3o`P820?o^k^k0_ooool4o`l?3`?oSHf=1?m7AdL3ono_
k`CoCdm?0omPH604o`0000;oQ8B40oooool3oa@D50Co;2`/0omUIFD4oa/K6`?oEEEE1?ogmoL3ocXj
>P?o`L710_ooool4omgMg@?oA4A41?lU9BD3ooooo`CoDe=C0ol00004ode=C@;oADE50oooool3og=c
L`CoIfMW0ol00004odm?C`?oj^[Z1?nUYJD3o``<30?o8B4Q0_ooool4obHV9P?o6ATI1?lD51@3oa4A
4@Co3Ph>0ol820P3oa0@40?om_Kf0oooool3oaPH60CoKFe]0omoOgl4on_[j`?ofMWI0ol71`L4od10
@0;o5QHF0oooool4oc0`<0?omoOg0olQ8B44o`0000?o6QXJ1?lA4A43o``<30?o7QhN0oooool3oaHF
5P?o0P821?lb<S83ol30`0Co/[:b0om;Bd/4oaTI6@;o_Kfm0oooool3omWIf@CoMWIf0olS8b<4oadM
7@?o1PH61?lP8203oahN7P?oKfm_0_ooool4oh>3P`?o6ATI1?lM7Ad3ogAdM0CoJf][0om?Cdl4ob/[
:`;olO7a4Oooool000oooooo0on:RXX4okfm_@?o>CTi1?l92@T3ogMgM`Colo?c0on?Shl3ode=C@;o
oooo1?m;Bd/3oc/k>`Co410@0oln?Sh4on?Sh`?o6a/K1?m1@D42ob/[:`?ooooo0olT92@4o`/;2`?o
5aLG1?lj>SX3ogMgM`CoaLG50on<S8`2ooknoP?ooooo1?l@4103obDU9@Co51@D0omRHV83oemOG`Co
lo?c0oo6a/H3o`@410?ooooo0olL71`4oe=CD`?oi>CT0ol@4104oc0`<0?o:2PX1?lI6AT2o`@410?o
oooo0olE5AD4oaTI6@?oS8b<1?nTY:@3ofEUI@Co71`L0omCDe<3oc@d=0;ooooo1?lI6AT3oh20P0Co
PH610oo[jn/4ofMWI`?oH61P1?lK6a/2ohb<S0?ooooo0ol;2`/4oc`l?0?o=cLg1?m<C4`3og5aL@Co
[:b/0olo?cl3oiNGU`;ooooo1?nj^[X3oc/k>`CoC4a<0ooTi>@4oc<c<`?o51@D1?mSHf<2ode=C@?o
oooo0om/K6`4oeaLG0?o4A4A1?mADE43onWYj@CoVIVI0olC4a<3obh^;P;ooooo1?l`<303oaDE5@Co
7alO0olD51@4oa<C4`?o3`l?0olD51@6ooooo`?o6a/K1?n1PH43oiFEU@Coj>SX0oo5aLD3oaDE5@Co
?3`l0_lM7Ad3ooooo`Co@D510ooKfm/3obd];@Co51@D0olF5QH4oalO7`?o3`l?0olc<c<3ooooo`?o
8R8R0olY:BT4oeEEE@OoUiNG0olK6a/4oadM7@;oeMGE0oooool3olS8b0CoOWin0ol];Bd4oahN7P?o
4Q8B1?lK6a/3ob<S8`?oI6AT0_ooool4oc4a<@?o2`/;1?l92@T3odQ8B0CoWinO0on:RXX4obh^;P;o
C4a<4Oooool000oooooo0on:RXX4okfm_@?o>CTi1?l92@T3ogMgM`Colo?c0on?Shl3ode=C@;ooooo
1?m;Bd/3oc/k>`Co410@0oln?Sh4on?Sh`?o6a/K1?m1@D42ob/[:`?ooooo0olT92@4o`/;2`?o5aLG
1?lj>SX3ogMgM`CoaLG50on<S8`2ooknoP?ooooo1?l@4103obDU9@Co51@D0omRHV83oemOG`Colo?c
0oo6a/H3o`@410?ooooo0olL71`4oe=CD`?oi>CT0ol@4104oc0`<0?o:2PX1?lI6AT2o`@410?ooooo
0olE5AD4oaTI6@?oS8b<1?nTY:@3ofEUI@Co71`L0omCDe<3oc@d=0;ooooo1?lI6AT3oh20P0CoPH61
0oo[jn/4ofMWI`?oH61P1?lK6a/2ohb<S0?ooooo0ol;2`/4oc`l?0?o=cLg1?m<C4`3og5aL@Co[:b/
0olo?cl3oiNGU`;ooooo1?nj^[X3oc/k>`CoC4a<0ooTi>@4oc<c<`?o51@D1?mSHf<2ode=C@?ooooo
0om/K6`4oeaLG0?o4A4A1?mADE43onWYj@CoVIVI0olC4a<3obh^;P;ooooo1?l`<303oaDE5@Co7alO
0olD51@4oa<C4`?o3`l?0olD51@6ooooo`?o6a/K1?n1PH43oiFEU@Coj>SX0oo5aLD3oaDE5@Co?3`l
0_lM7Ad3ooooo`Co@D510ooKfm/3obd];@Co51@D0olF5QH4oalO7`?o3`l?0olc<c<3ooooo`?o8R8R
0olY:BT4oeEEE@OoUiNG0olK6a/4oadM7@;oeMGE0oooool3olS8b0CoOWin0ol];Bd4oahN7P?o4Q8B
1?lK6a/3ob<S8`?oI6AT0_ooool4oc4a<@?o2`/;1?l92@T3odQ8B0CoWinO0on:RXX4obh^;P;oC4a<
4Oooool000oooooo0on:RXX4okfm_@?o>CTi1?l92@T3ogMgM`Colo?c0on?Shl3ode=C@;ooooo1?m;
Bd/3oc/k>`Co410@0oln?Sh4on?Sh`?o6a/K1?m1@D42ob/[:`?ooooo0olT92@4o`/;2`?o5aLG1?lj
>SX3ogMgM`CoaLG50on<S8`2ooknoP?ooooo1?l@4103obDU9@Co51@D0omRHV83oemOG`Colo?c0oo6
a/H3o`@410?ooooo0olL71`4oe=CD`?oi>CT0ol@4104oc0`<0?o:2PX1?lI6AT2o`@410?ooooo0olE
5AD4oaTI6@?oS8b<1?nTY:@3ofEUI@Co71`L0omCDe<3oc@d=0;ooooo1?lI6AT3oh20P0CoPH610oo[
jn/4ofMWI`?oH61P1?lK6a/2ohb<S0?ooooo0ol;2`/4oc`l?0?o=cLg1?m<C4`3og5aL@Co[:b/0olo
?cl3oiNGU`;ooooo1?nj^[X3oc/k>`CoC4a<0ooTi>@4oc<c<`?o51@D1?mSHf<2ode=C@?ooooo0om/
K6`4oeaLG0?o4A4A1?mADE43onWYj@CoVIVI0olC4a<3obh^;P;ooooo1?l`<303oaDE5@Co7alO0olD
51@4oa<C4`?o3`l?0olD51@6ooooo`?o6a/K1?n1PH43oiFEU@Coj>SX0oo5aLD3oaDE5@Co?3`l0_lM
7Ad3ooooo`Co@D510ooKfm/3obd];@Co51@D0olF5QH4oalO7`?o3`l?0olc<c<3ooooo`?o8R8R0olY
:BT4oeEEE@OoUiNG0olK6a/4oadM7@;oeMGE0oooool3olS8b0CoOWin0ol];Bd4oahN7P?o4Q8B1?lK
6a/3ob<S8`?oI6AT0_ooool4oc4a<@?o2`/;1?l92@T3odQ8B0CoWinO0on:RXX4obh^;P;oC4a<4Ooo
ool001;ooooo1?mTI6@3o`X:2PCo<S8b0om>CTh4of=SH`?oa/K60oo3`l<2ooooo`Co@T920olO7al4
o`@410?o=CDe1?nj^[X3oaDE5@Co>3Ph0_lG5aL3ooooo`?o92@T1?l[:b/3obDU9@CoBDU90omiNGT4
oiFEU@?oRh^;0_oWinL3ooooo`Co0P820olA4A44oadM7@?oCTi>0omBDU84olg=c@?oZZZZ0ol61PH3
ooooo`?o9bLW1?mLG5`3oj:RXP?o1@D51?lX:2P3oa@D50Co410@0_l61PH3ooooo`?o4a<C1?nLW9`3
ol?3``CoMGEe0olR8R84oa`L70?o1PH60onRXZ82ooooo`Co7QhN0omgMgL4of1PH0?oeMGE1?mADE43
oeiNGPCo51@D0_n?Shl3ooooo`?o00001?lJ6QX3of9RHPCo8R8R0omPH604omCDe0?o=CDe0on=SHd2
ooooo`Co]KFe0oln?Sh4ocdm?@?of=SH1?m2@T83o`820PCoHV9R0_m9BDT3ooooo`?oGUiN1?mBDU83
o`820PCoH61P0oo3`l<4og=cL`?o0P820olD51@2ooooo`Co8b<S0olF5QH4oa<C4`?o5QHF1ol;2`/3
oa<C4`?o];Bd0oooool3ob@T90CoNGUi0on6QXH4ol;2`P?o/;2`0ol>3Ph4ob`/;0;o7AdM0oooool4
ocDe=@?odM7A0olM7Ad4o`@410?o6a/K1?lA4A43oa@D50?o9bLW0oooool3oalO7`?o20P81?m2@T83
ol30`0CoT92@0olm?Cd4obXZ:P;oVYZJ0oooool3ok>c/`CoK6a/0olS8b<4obLW9`?o61PH1?ll?3`3
obPX:0?oMgMg0_ooool4og9bLP?o3@d=1?lg=cL3ohN7Q`CoH61P0ole=CD4ob0P81?ooooo000Boooo
o`CoI6AT0ol:2PX4oc8b<P?oCTi>1?mSHf<3olK6aP?o`l?30_ooool4od92@P?o7alO1?l410@3ocDe
=@Co^[Zj0olE5AD4ocPh>0;o5aLG0oooool3ob@T90Co:b/[0olU9BD4odU9B@?oNGUi1?nEUID3oh^;
R`;oinOW0oooool4o`820P?o4A4A1?lM7Ad3odi>CP?oDU9B1?o=cLd3ojZZZP?o1PH60oooool3obLW
9`CoG5aL0onRXZ83o`D51@Co:2PX0olD51@4oa0@40;o1PH60oooool3oa<C4`CoW9bL0oo3`l<4ogEe
M@?o8R8R1?lL71`3o`H61P?oXZ:R0_ooool4oahN7P?oMgMg1?mPH603omGEe@CoDE5A0omNGUh4oa@D
50;oShn?0oooool3o`0000Co6QXJ0omRHV84ob8R8P?oH61P1?oDe=@3ocDe=@?oSHf=0_ooool4okFe
]@?o?Shn1?lm?Cd3omSHf0Co@T920ol20P84of9RHP;oBDU90oooool3oeiNGPCoDU9B0ol20P84of1P
H0?o`l?31?mcLg<3o`820P?o51@D0_ooool4ob<S8`?o5QHF1?lC4a<3oaHF5POo2`/;0olC4a<3okBd
]0?ooooo0olT92@4ogUiN@?oQXJ61?o2`/83ok2`/0?o3Ph>1?l/;2`2oadM7@?ooooo1?le=CD3om7A
d@?o7AdM1?l410@3oa/K6`Co4A4A0olD51@3obLW9`?ooooo0olO7al3o`P820Co@T920oo0`<04oi2@
T0?o?Cdm1?lZ:RX2oiZJVP?ooooo0onc/k<4ofa/K0?o8b<S1?lW9bL3oaPH60Co?3`l0olX:2P3ogMg
M`;ooooo1?mbLW83o`d=3@Co=cLg0on7QhL4of1PH0?o=CDe1?lP820Cooooo`004_ooool4ofATI0?o
2PX:1?lb<S83odi>CPCoHf=S0oo6a/H3ol?3``;ooooo1?m2@T83oalO7`Co10@40ole=CD4okZj^P?o
5ADE1?lh>3P2oaLG5`?ooooo0olT92@4ob/[:`?o9BDU1?m9BDT3ogUiN@CoUIFE0on;Rh/2onOWi`?o
oooo1?l20P83oa4A4@Co7AdM0om>CTh3oe9BDPCocLg=0onZZZX3o`H61P?ooooo0olW9bL4oeaLG0?o
XZ:R0ol51@D4obPX:0?o51@D1?l@4102o`H61P?ooooo0olC4a<4oibLW0?o`l?31?meMGD3ob8R8PCo
71`L0ol61PH3oj:RXP;ooooo1?lN7Qh3ogMgM`CoH61P0ooEeMD4oe5AD@?oGUiN1?lD51@2ohn?S`?o
oooo0ol00004oaXJ6P?oHV9R1?lR8R83of1PH0Coe=CD0ole=CD3ohf=S@;ooooo1?ne]KD3ochn?PCo
?Cdm0ooHf=P4od92@P?o0P821?mRHV82odU9B@?ooooo0omNGUh4oe9BDP?o0P821?mPH603ol?3``Co
Lg=c0ol20P83oa@D50;ooooo1?lS8b<3oaHF5PCo4a<C0olF5QH7o`/;2`?o4a<C0ond];@3ooooo`?o
92@T1?miNGT3ohJ6QPCo`/;20on`/;03o`h>3PCo;2`/0_lM7Ad3ooooo`Co=CDe0ooAdM43oadM7@Co
10@40olK6a/4oa4A4@?o51@D0olW9bL3ooooo`?o7alO0ol820P4od92@P?o`<301?n@T903ocdm?@Co
:RXZ0_nJVYX3ooooo`?o/k>c1?m/K6`3ob<S8`Co9bLW0olH61P4oc`l?0?o:2PX0omgMgL2ooooo`Co
LW9b0ol=3@d4ocLg=`?oQhN71?mPH603ocDe=@Co820P4oooool001;ooooo1?mTI6@3o`X:2PCo<S8b
0om>CTh4of=SH`?oa/K60oo3`l<2ooooo`Co@T920olO7al4o`@410?o=CDe1?nj^[X3oaDE5@Co>3Ph
0_lG5aL3ooooo`?o92@T1?l[:b/3obDU9@CoBDU90omiNGT4oiFEU@?oRh^;0_oWinL3ooooo`Co0P82
0olA4A44oadM7@?oCTi>0omBDU84olg=c@?oZZZZ0ol61PH3ooooo`?o9bLW1?mLG5`3oj:RXP?o1@D5
1?lX:2P3oa@D50Co410@0_l61PH3ooooo`?o4a<C1?nLW9`3ol?3``CoMGEe0olR8R84oa`L70?o1PH6
0onRXZ82ooooo`Co7QhN0omgMgL4of1PH0?oeMGE1?mADE43oeiNGPCo51@D0_n?Shl3ooooo`?o0000
1?lJ6QX3of9RHPCo8R8R0omPH604omCDe0?o=CDe0on=SHd2ooooo`Co]KFe0oln?Sh4ocdm?@?of=SH
1?m2@T83o`820PCoHV9R0_m9BDT3ooooo`?oGUiN1?mBDU83o`820PCoH61P0oo3`l<4og=cL`?o0P82
0olD51@2ooooo`Co8b<S0olF5QH4oa<C4`?o5QHF1ol;2`/3oa<C4`?o];Bd0oooool3ob@T90CoNGUi
0on6QXH4ol;2`P?o/;2`0ol>3Ph4ob`/;0;o7AdM0oooool4ocDe=@?odM7A0olM7Ad4o`@410?o6a/K
1?lA4A43oa@D50?o9bLW0oooool3oalO7`?o20P81?m2@T83ol30`0CoT92@0olm?Cd4obXZ:P;oVYZJ
0oooool3ok>c/`CoK6a/0olS8b<4obLW9`?o61PH1?ll?3`3obPX:0?oMgMg0_ooool4og9bLP?o3@d=
1?lg=cL3ohN7Q`CoH61P0ole=CD4ob0P81?ooooo003oooooooGooooo003oooooooGooooo003ooooo
ooGooooo000?ooooo`?oBd];1?mUIFD3om3@d0CoGemO0olL71`4ob<S8`?oU9BD0olk>c/2ooooo`Co
71`L0ol820P4ocHf=P?o;Rh^1?oOgml3ob0P80Co7AdM0_l_;bl3ooooo`?o61PH1?lJ6QX3o`T92@Co
1@D50olO7al4odM7A`?oaLG50_l^;Rh3ooooo`CoATI60om<C4`4oeYJFP?oH61P0olh>3P4obXZ:P?o
a<C40om`L703ooooo`?o<c<c1?mmOGd3oi:BTP?og]kN1?m4A4@3o`0000Co71`L0_nd];@3ooooo`?o
3Ph>1?l[:b/3ocPh>0Cog]kN0olo?cl4ob8R8P?o92@T0olU9BD2ooooo`CoJf][0om3@d<4ocLg=`?o
;2`/1?meMGD3oeYJFPCo`<300_nZZZX3ooooo`?o92@T1?lC4a<3oeQHF0Co=SHf0olY:BT4onSXj0?o
ATI60olC4a<2ooooo`Co^k^k0olg=cL4ocdm?@?o1`L71?lE5AD3oaLG5`Co>3Ph0_lF5QH3ooooo`?o
]KFe1?nWYjL3oe1@D0Co3`l?0om@D504ofe]K@?oIVIV0om^KVh2ooooo`CoB4Q80olY:BT4oadM7@?o
2PX:1?l30`<3o``<30?o820P0ooMgMd3ooooo`?oK6a/1?l410@3okVi^@Coh^;R0om_Kfl3ok2`/0Co
JVYZ0_m:BTX3ooooo`CoK6a/0oo7alL3ol30`0Co>SXj0olU9BD4o`T92@?o9RHV0olN7Qh3ooooo`?o
@d=30olJ6QX4og1`L0?oU9BD1?mQHF43oeMGE`Co92@T0_ogmoL3ooooo`?o:b/[1?m4A4@3om7Ad@Co
>c/k0olC4a<4o`l?3`?o4a<C0olg=cL2ooooo`CoUIFE0ol^;Rh4ocDe=@?o3@d=1ol[:b/4oeADE0;o
d=3@4Oooool000oooooo0om;Bd/4ofEUI@?od=3@1?mOGel3oa`L70Co8b<S0onDU9@3oc/k>`;ooooo
1?lL71`3o`P820Co=SHf0ol^;Rh4omoOg`?o820P1?lM7Ad2obl_;`?ooooo0olH61P4oaXJ6P?o2@T9
1?l51@D3oalO7`CoAdM70oo5aLD2obh^;P?ooooo1?m6ATH3oda<C0CoFUYJ0omPH603ocPh>0Co:RXZ
0oo4a<@3og1`L0?ooooo0olc<c<4ogemO@?oTY:B0ooNg]h4odA4A0?o00001?lL71`2okBd]0?ooooo
0ol>3Ph4ob/[:`?o>3Ph1?oNg]h3oclo?`Co8R8R0olT92@3obDU9@;ooooo1?m[Jf/3od=3@`Co=cLg
0ol/;2`4ogEeM@?oFUYJ1?o0`<02ojZZZP?ooooo0olT92@4oa<C4`?oF5QH1?lf=SH3obTY:@Coj>SX
0om6ATH3oa<C4`;ooooo1?nk^k/3ocLg=`Co?Cdm0ol71`L4oaDE5@?o5aLG1?lh>3P2oaHF5P?ooooo
0one]KD4ojNWY`?oD51@1?l?3`l3oe1@D0CoKFe]0omVIVH3ofi^KP;ooooo1?m8B4P3obTY:@Co7AdM
0ol:2PX4o`<30`?o30`<0olP8203omgMg@?ooooo0om/K6`4o`@410?o^KVi1?oRh^83ofm_K`?o/;2`
1?mZJVX2odY:BP?ooooo1?m/K6`3olO7a`?o`<301?lj>SX3obDU9@Co2@T90olV9RH3oahN7P?ooooo
0om3@d<3oaXJ6PCoL71`0onDU9@4of5QH@?oEeMG1?lT92@2ooOgm`?ooooo0ol[:b/4odA4A0?odM7A
1?lk>c/3oa<C4`Co3`l?0olC4a<3ocLg=`;ooooo1?nEUID3obh^;PCo=CDe0ol=3@d7ob/[:`CoE5AD
0_o@d=0Aooooo`003oooool3od];B`CoIFEU0oo@d=04oemOG`?o71`L1?lS8b<3oiBDU0?o>c/k0_oo
ool4oa`L70?o20P81?lf=SH3obh^;PCogmoO0olP8204oadM7@;o;bl_0oooool3oaPH60Co6QXJ0ol9
2@T4o`D51@?o7alO1?m7AdL3olG5a@;o;Rh^0oooool4odI6AP?oC4a<1?mJFUX3of1PH0?o>3Ph1?lZ
:RX3olC4a0?oL71`0oooool3oc<c<`CoOGem0onBTY83omkNgPCoA4A40ol00004oa`L70;o];Bd0ooo
ool3o`h>3PCo:b/[0olh>3P4omkNgP?o?clo1?lR8R83ob@T90?o9BDU0_ooool4of][J`?o@d=31?lg
=cL3ob`/;0CoMGEe0omJFUX4ol30`0;oZZZZ0oooool3ob@T90Co4a<C0omHF5P4ocHf=P?o:BTY1?oX
j>P3odI6AP?o4a<C0_ooool4ok^k^`?o=cLg1?lm?Cd3o`L71`Co5ADE0olG5aL4ocPh>0;o5QHF0ooo
ool3okFe]@CoYjNW0om@D504o`l?3`?oD51@1?m]KFd3ofIVIP?oKVi^0_ooool4odQ8B0?o:BTY1?lM
7Ad3o`X:2PCo0`<30ol<30`3ob0P80?ogMgM0oooool3ofa/K0Co10@40oni^KT4on;RhP?oKfm_0on`
/;04ofYZJP;oBTY:0oooool4ofa/K0?oalO70oo0`<04ocXj>P?o9BDU1?l92@T3obHV9P?o7QhN0ooo
ool3od=3@`?o6QXJ1?m`L703oiBDU0CoHF5Q0omGEeL4ob@T90;omoOg0oooool3ob/[:`CoA4A40ooA
dM44oc/k>`?o4a<C1?l?3`l3oa<C4`?o=cLg0_ooool4oiFEU@?o;Rh^1?le=CD3o`d=3@Oo:b/[1?mD
E5@2om3@d17ooooo000?ooooo`?oBd];1?mUIFD3om3@d0CoGemO0olL71`4ob<S8`?oU9BD0olk>c/2
ooooo`Co71`L0ol820P4ocHf=P?o;Rh^1?oOgml3ob0P80Co7AdM0_l_;bl3ooooo`?o61PH1?lJ6QX3
o`T92@Co1@D50olO7al4odM7A`?oaLG50_l^;Rh3ooooo`CoATI60om<C4`4oeYJFP?oH61P0olh>3P4
obXZ:P?oa<C40om`L703ooooo`?o<c<c1?mmOGd3oi:BTP?og]kN1?m4A4@3o`0000Co71`L0_nd];@3
ooooo`?o3Ph>1?l[:b/3ocPh>0Cog]kN0olo?cl4ob8R8P?o92@T0olU9BD2ooooo`CoJf][0om3@d<4
ocLg=`?o;2`/1?meMGD3oeYJFPCo`<300_nZZZX3ooooo`?o92@T1?lC4a<3oeQHF0Co=SHf0olY:BT4
onSXj0?oATI60olC4a<2ooooo`Co^k^k0olg=cL4ocdm?@?o1`L71?lE5AD3oaLG5`Co>3Ph0_lF5QH3
ooooo`?o]KFe1?nWYjL3oe1@D0Co3`l?0om@D504ofe]K@?oIVIV0om^KVh2ooooo`CoB4Q80olY:BT4
oadM7@?o2PX:1?l30`<3o``<30?o820P0ooMgMd3ooooo`?oK6a/1?l410@3okVi^@Coh^;R0om_Kfl3
ok2`/0CoJVYZ0_m:BTX3ooooo`CoK6a/0oo7alL3ol30`0Co>SXj0olU9BD4o`T92@?o9RHV0olN7Qh3
ooooo`?o@d=30olJ6QX4og1`L0?oU9BD1?mQHF43oeMGE`Co92@T0_ogmoL3ooooo`?o:b/[1?m4A4@3
om7Ad@Co>c/k0olC4a<4o`l?3`?o4a<C0olg=cL2ooooo`CoUIFE0ol^;Rh4ocDe=@?o3@d=1ol[:b/4
oeADE0;od=3@4Oooool000oooooo0olJ6QX4on3Ph0?of=SH1?meMGD3oa8B4PCoATI60omdM7@3od51
@@;ooooo1?lH61P3o`@410Co<c<c0olX:2P4onSXj0?o7AdM1?l=3@d2oaPH60?ooooo0olH61P4oa<C
4`?o0`<31?l92@T3o``<30Co@4100ooNg]h2odE5A@?ooooo1?li>CT3oe1@D0CoEeMG0omNGUh3obHV
9PCo92@T0oo:b/X3ofYZJP?ooooo0olH61P4obDU9@?o`L710onc/k<4of][J`?o20P81?mIFET2obXZ
:P?ooooo0ol92@T4oc4a<@?o@d=31?oemOD3oc@d=0Co8b<S0olD51@3oa8B4P;ooooo1?mSHf<3odM7
A`Co6a/K0ol51@D4ogAdM0?oIVIV1?n`/;02ol71`@?ooooo0olL71`4oa8B4P?oEeMG1?l];Bd3obh^
;PCojNWY0ol_;bl3o`D51@;ooooo1?oEeMD3obPX:0Co>SXj0ol=3@d4oa/K6`?o71`L1?li>CT2o``<
30?ooooo0omeMGD4ofe]K@?o7QhN1?l92@T3oe5AD@Co^[Zj0onTY:@3obLW9`;ooooo1?m3@d<3ob@T
90Co7AdM0ol820P4o`820P?o3@d=0olK6a/6ooooo`?oLG5a1?l00003omGEe@Coj>SX0omGEeL3ojNW
Y`CoLG5a0_m_Kfl3ooooo`CoIfMW0ooOgml3olG5a@Co5ADE0olZ:RX4o`0000?o=3@d0olC4a<3oooo
o`?o6ATI0olc<c<4oaLG5`?oEUIF1?n:RXX3oclo?`Ko;bl_0oooool3oalO7`Co@4100ooQhN44oc@d
=0?o3Ph>1?l;2`/3o`@410?o<30`0_ooool4oiVIV@?o4Q8B1?lZ:RX3o`410@Co<S8b0ol_;bl4ofMW
I`;ohn?S4Oooool000oooooo0olJ6QX4on3Ph0?of=SH1?meMGD3oa8B4PCoATI60omdM7@3od51@@;o
oooo1?lH61P3o`@410Co<c<c0olX:2P4onSXj0?o7AdM1?l=3@d2oaPH60?ooooo0olH61P4oa<C4`?o
0`<31?l92@T3o``<30Co@4100ooNg]h2odE5A@?ooooo1?li>CT3oe1@D0CoEeMG0omNGUh3obHV9PCo
92@T0oo:b/X3ofYZJP?ooooo0olH61P4obDU9@?o`L710onc/k<4of][J`?o20P81?mIFET2obXZ:P?o
oooo0ol92@T4oc4a<@?o@d=31?oemOD3oc@d=0Co8b<S0olD51@3oa8B4P;ooooo1?mSHf<3odM7A`Co
6a/K0ol51@D4ogAdM0?oIVIV1?n`/;02ol71`@?ooooo0olL71`4oa8B4P?oEeMG1?l];Bd3obh^;PCo
jNWY0ol_;bl3o`D51@;ooooo1?oEeMD3obPX:0Co>SXj0ol=3@d4oa/K6`?o71`L1?li>CT2o``<30?o
oooo0omeMGD4ofe]K@?o7QhN1?l92@T3oe5AD@Co^[Zj0onTY:@3obLW9`;ooooo1?m3@d<3ob@T90Co
7AdM0ol820P4o`820P?o3@d=0olK6a/6ooooo`?oLG5a1?l00003omGEe@Coj>SX0omGEeL3ojNWY`Co
LG5a0_m_Kfl3ooooo`CoIfMW0ooOgml3olG5a@Co5ADE0olZ:RX4o`0000?o=3@d0olC4a<3ooooo`?o
6ATI0olc<c<4oaLG5`?oEUIF1?n:RXX3oclo?`Ko;bl_0oooool3oalO7`Co@4100ooQhN44oc@d=0?o
3Ph>1?l;2`/3o`@410?o<30`0_ooool4oiVIV@?o4Q8B1?lZ:RX3o`410@Co<S8b0ol_;bl4ofMWI`;o
hn?S4Oooool000oooooo0olJ6QX4on3Ph0?of=SH1?meMGD3oa8B4PCoATI60omdM7@3od51@@;ooooo
1?lH61P3o`@410Co<c<c0olX:2P4onSXj0?o7AdM1?l=3@d2oaPH60?ooooo0olH61P4oa<C4`?o0`<3
1?l92@T3o``<30Co@4100ooNg]h2odE5A@?ooooo1?li>CT3oe1@D0CoEeMG0omNGUh3obHV9PCo92@T
0oo:b/X3ofYZJP?ooooo0olH61P4obDU9@?o`L710onc/k<4of][J`?o20P81?mIFET2obXZ:P?ooooo
0ol92@T4oc4a<@?o@d=31?oemOD3oc@d=0Co8b<S0olD51@3oa8B4P;ooooo1?mSHf<3odM7A`Co6a/K
0ol51@D4ogAdM0?oIVIV1?n`/;02ol71`@?ooooo0olL71`4oa8B4P?oEeMG1?l];Bd3obh^;PCojNWY
0ol_;bl3o`D51@;ooooo1?oEeMD3obPX:0Co>SXj0ol=3@d4oa/K6`?o71`L1?li>CT2o``<30?ooooo
0omeMGD4ofe]K@?o7QhN1?l92@T3oe5AD@Co^[Zj0onTY:@3obLW9`;ooooo1?m3@d<3ob@T90Co7AdM
0ol820P4o`820P?o3@d=0olK6a/6ooooo`?oLG5a1?l00003omGEe@Coj>SX0omGEeL3ojNWY`CoLG5a
0_m_Kfl3ooooo`CoIfMW0ooOgml3olG5a@Co5ADE0olZ:RX4o`0000?o=3@d0olC4a<3ooooo`?o6ATI
0olc<c<4oaLG5`?oEUIF1?n:RXX3oclo?`Ko;bl_0oooool3oalO7`Co@4100ooQhN44oc@d=0?o3Ph>
1?l;2`/3o`@410?o<30`0_ooool4oiVIV@?o4Q8B1?lZ:RX3o`410@Co<S8b0ol_;bl4ofMWI`;ohn?S
4Oooool000oooooo0omoOgl4og]kN`?oalO71?m4A4@3oa@D50Co6QXJ0ooDe=@3oa`L70;ooooo1?l@
4103o`0000Co<S8b0olO7al4ooGem@?o5ADE1?lB4Q82oa@D50?ooooo0olA4A44oa@D50?o2PX:1?l5
1@D3oa<C4`Co>3Ph0ooWinL2obd];@?ooooo1?le=CD3oe5AD@CoDe=C0omQHF43ob<S8`Co5ADE0oo`
l?03oh>3P`?ooooo0oll?3`4olC4a0?oS8b<0ong]kL4oaPH60?o3@d=1?lP8202ogIfMP?ooooo0ol8
20P4oc0`<0?o?clo1?ogmoL3oc8b<PCo8R8R0olK6a/3oa8B4P;ooooo1?mLG5`3ocTi>@Oo00001?mB
DU83odE5A@Co[jn_0_nSXj<3ooooo`?o92@T1?l<30`3of=SH`Co;Rh^0olV9RH4ooWin@?o:b/[0ol5
1@D2ooooo`Coi^KV0olW9bL4odM7A`?o0@411?l61PH3oalO7`Co>SXj0_l?3`l3ooooo`?o_[jn1?mn
OWh3ob8R8PCo=3@d0omEEED4ofa/K0?oIfMW0ona/K42ooooo`CoADE50olU9BD4oaXJ6P?o20P81?l0
0003o``<30?o5aLG0ooinOT3ooooo`?oK6a/1?l20P83on?Sh`Cooooo0omGEeL3ok^k^`CoK6a/0_m^
KVh3ooooo`CoRHV90oohn?P3olG5a@Co9BDU0olY:BT4o`l?3`?o8b<S0olU9BD3ooooo`?o;Rh^0ol<
30`4ojZZZP?oV9RH1?m9BDT3oc/k>`Co:b/[1Oooool3obHV9PCo=3@d0ooZj^X4ochn?P?o1`L71?l=
3@d3o`/;2`?o<S8b0_ooool4oi2@T0?o5QHF1?lX:2P3o`d=3@Co>c/k0old=3@4oeeMG@;omOGe4Ooo
ool000oooooo0omoOgl4og]kN`?oalO71?m4A4@3oa@D50Co6QXJ0ooDe=@3oa`L70;ooooo1?l@4103
o`0000Co<S8b0olO7al4ooGem@?o5ADE1?lB4Q82oa@D50?ooooo0olA4A44oa@D50?o2PX:1?l51@D3
oa<C4`Co>3Ph0ooWinL2obd];@?ooooo1?le=CD3oe5AD@CoDe=C0omQHF43ob<S8`Co5ADE0oo`l?03
oh>3P`?ooooo0oll?3`4olC4a0?oS8b<0ong]kL4oaPH60?o3@d=1?lP8202ogIfMP?ooooo0ol820P4
oc0`<0?o?clo1?ogmoL3oc8b<PCo8R8R0olK6a/3oa8B4P;ooooo1?mLG5`3ocTi>@Oo00001?mBDU83
odE5A@Co[jn_0_nSXj<3ooooo`?o92@T1?l<30`3of=SH`Co;Rh^0olV9RH4ooWin@?o:b/[0ol51@D2
ooooo`Coi^KV0olW9bL4odM7A`?o0@411?l61PH3oalO7`Co>SXj0_l?3`l3ooooo`?o_[jn1?mnOWh3
ob8R8PCo=3@d0omEEED4ofa/K0?oIfMW0ona/K42ooooo`CoADE50olU9BD4oaXJ6P?o20P81?l00003
o``<30?o5aLG0ooinOT3ooooo`?oK6a/1?l20P83on?Sh`Cooooo0omGEeL3ok^k^`CoK6a/0_m^KVh3
ooooo`CoRHV90oohn?P3olG5a@Co9BDU0olY:BT4o`l?3`?o8b<S0olU9BD3ooooo`?o;Rh^0ol<30`4
ojZZZP?oV9RH1?m9BDT3oc/k>`Co:b/[1Oooool3obHV9PCo=3@d0ooZj^X4ochn?P?o1`L71?l=3@d3
o`/;2`?o<S8b0_ooool4oi2@T0?o5QHF1?lX:2P3o`d=3@Co>c/k0old=3@4oeeMG@;omOGe4Oooool0
00oooooo0omoOgl4og]kN`?oalO71?m4A4@3oa@D50Co6QXJ0ooDe=@3oa`L70;ooooo1?l@4103o`00
00Co<S8b0olO7al4ooGem@?o5ADE1?lB4Q82oa@D50?ooooo0olA4A44oa@D50?o2PX:1?l51@D3oa<C
4`Co>3Ph0ooWinL2obd];@?ooooo1?le=CD3oe5AD@CoDe=C0omQHF43ob<S8`Co5ADE0oo`l?03oh>3
P`?ooooo0oll?3`4olC4a0?oS8b<0ong]kL4oaPH60?o3@d=1?lP8202ogIfMP?ooooo0ol820P4oc0`
<0?o?clo1?ogmoL3oc8b<PCo8R8R0olK6a/3oa8B4P;ooooo1?mLG5`3ocTi>@Oo00001?mBDU83odE5
A@Co[jn_0_nSXj<3ooooo`?o92@T1?l<30`3of=SH`Co;Rh^0olV9RH4ooWin@?o:b/[0ol51@D2oooo
o`Coi^KV0olW9bL4odM7A`?o0@411?l61PH3oalO7`Co>SXj0_l?3`l3ooooo`?o_[jn1?mnOWh3ob8R
8PCo=3@d0omEEED4ofa/K0?oIfMW0ona/K42ooooo`CoADE50olU9BD4oaXJ6P?o20P81?l00003o``<
30?o5aLG0ooinOT3ooooo`?oK6a/1?l20P83on?Sh`Cooooo0omGEeL3ok^k^`CoK6a/0_m^KVh3oooo
o`CoRHV90oohn?P3olG5a@Co9BDU0olY:BT4o`l?3`?o8b<S0olU9BD3ooooo`?o;Rh^0ol<30`4ojZZ
ZP?oV9RH1?m9BDT3oc/k>`Co:b/[1Oooool3obHV9PCo=3@d0ooZj^X4ochn?P?o1`L71?l=3@d3o`/;
2`?o<S8b0_ooool4oi2@T0?o5QHF1?lX:2P3o`d=3@Co>c/k0old=3@4oeeMG@;omOGe4Oooool000oo
oooo0ol`<304oo_kn`?o`L711?ll?3`3o`0000CoKfm_0on3Ph<3oaTI6@;ooooo1?l>3Ph3o`T92@Co
;Rh^0olL71`4ooclo0?o61PH1?l00002o`@410?ooooo0olB4Q84oa<C4`?o10@41?l61PH3oa<C4`Co
<30`0oooool2ocXj>P?ooooo1?l/;2`3oeEEE@CoGUiN0omRHV83oa0@40Co7QhN0oobl_83og]kN`?o
oooo0old=3@4ochn?P?ohn?S0onQXJ44ocTi>@?o2`/;1?nTY:@2obd];@?ooooo0ol92@T4oc4a<@?o
>c/k1?ooool3obh^;PCo820P0ol:2PX3o`l?3`;ooooo1?mYJFT3ode=C@Co4Q8B0ol51@D4oeaLG0?o
EUIF1?nc/k<2olO7a`?ooooo0olJ6QX4o`l?3`?oG5aL1?la<C43ob<S8`Colo?c0olV9RH3o`P820Ko
oooo0ol<30`4od51@@?o410@1?lC4a<3obPX:0Co>CTi0_lC4a<3ooooo`?oO7al1?mKFe/3oa4A4@Co
0@410on8R8P4ol_;b`?oShn?0olW9bL2ooooo`Co@T920ol`<304oa`L70?o4A4A1?l61PH3o`d=3@?o
6a/K0ooZj^X3ooooo`?oHF5Q1?l:2PX3on[ZjPCooOgm0olc<c<3ojb/[0CoLG5a0_m/K6`3ooooo`Co
QXJ60oogmoL3ok^k^`Co2@T90olb<S84o`@410?o=CDe0olT92@3ooooo`?o1@D50olg=cL4ocHf=P?o
HV9R1?mlO7`3oc4a<@Co7alO0_lG5aL3ooooo`?o71`L1?lh>3P3ooooo`Co>c/k0ol20P84o`l?3`?o
00000ol/;2`2ooooo`CoS8b<0ol61PH4ob0P80?o00001?m8B4P3obh^;PCoJ6QX0_ojn_XAooooo`00
3oooool3oc0`<0Cono_k0oo1`L44oc`l?0?o00001?m_Kfl3oh>3P`?o6ATI0_ooool4o`h>3P?o2@T9
1?l^;Rh3oa`L70Coo?cl0olH61P4o`0000;o10@40oooool3oa8B4PCo4a<C0ol410@4o`H61P?o4a<C
1?l`<303ooooo`;o>SXj0oooool4ob`/;0?oEEEE1?mNGUh3of9RHP?o410@1?lN7Qh3oo;blP?oNg]k
0oooool3oc@d=0Co?Shn0ooShn<3oj6QX@Co>CTi0ol;2`/4ojBTY0;o;Bd]0oooool3o`T92@Co<C4a
0olk>c/4ooooo`?o;Rh^1?lP8203o`X:2P?o3`l?0_ooool4ofUYJ@?oCDe=1?lB4Q83o`D51@CoG5aL
0omFEUH4ok>c/`;oalO70oooool3oaXJ6PCo3`l?0omLG5`4oc4a<@?o8b<S1?oclo<3obHV9P?o20P8
1_ooool3o``<30Co@D510ol@4104oa<C4`?o:2PX1?li>CT2oa<C4`?ooooo0omlO7`4oe]KF`?o4A4A
1?l10@43ohR8R0Cobl_;0on?Shl3obLW9`;ooooo1?m2@T83oc0`<0Co71`L0olA4A44o`H61P?o3@d=
0olK6a/3on[ZjP?ooooo0omQHF44o`X:2P?oj^[Z1?omoOd3oc<c<`?o[:b/1?maLG42ofa/K0?ooooo
1?n6QXH3ooOgm`?o^k^k1?l92@T3oc8b<PCo10@40ole=CD3ob@T90?ooooo0ol51@D3ocLg=`Co=SHf
0omRHV84ogalO0?o<C4a1?lO7al2oaLG5`?ooooo0olL71`4ocPh>0?ooooo1?lk>c/3o`820PCo3`l?
0ol00003ob`/;0;ooooo1?n<S8`3o`H61PCo820P0ol00004odQ8B0?o;Rh^1?mXJ6P2oo[jnQ7ooooo
000?ooooo`?o<30`1?okno/3ol71`@Co?3`l0ol00004ofm_K`?oPh>30olI6AT2ooooo`Co3Ph>0ol9
2@T4obh^;P?o71`L1?olo?`3oaPH60Co00000_l410@3ooooo`?o4Q8B1?lC4a<3o`@410Co1PH60olC
4a<4oc0`<0?ooooo0_lj>SX3ooooo`Co;2`/0omEEED4oeiNGP?oHV9R0ol@4104oahN7P?ol_;b0omk
Ng/3ooooo`?o=3@d1?ln?Sh3on?Sh`?oXJ6Q1?li>CT3o`/;2`CoY:BT0_l];Bd3ooooo`?o2@T91?la
<C43oc/k>`Cooooo0ol^;Rh4ob0P80?o2PX:0ol?3`l2ooooo`CoJFUY0om=CDd4oa8B4P?o1@D51?mL
G5`3oeIFEPCo/k>c0_o7alL3ooooo`?o6QXJ1?l?3`l3oeaLG0Co<C4a0olS8b<4oo?cl`?o9RHV0ol8
20P6ooooo`?o30`<1?m1@D43oa0@40Co4a<C0olX:2P4ocTi>@;o4a<C0oooool3ogalO0CoFe]K0olA
4A44o`410@?oR8R81?o;bl/3ohn?S`?o9bLW0_ooool4od92@P?o<30`1?lL71`3oa4A4@Co1PH60ol=
3@d3oa/K6`?oj^[Z0oooool3of5QH@Co2PX:0ooZj^X4oogmo@?o<c<c0on/[:`4og5aL@;oK6a/0ooo
ool4ohJ6QP?omoOg0onk^k/4o`T92@?o<S8b1?l410@3ocDe=@?o92@T0oooool3o`D51@?o=cLg1?lf
=SH3of9RHPCoO7al0ola<C44oalO7`;o5aLG0oooool3oa`L70Co>3Ph0oooool4oc/k>`?o0P821?l?
3`l3o`0000?o;2`/0_ooool4ohb<S0?o1PH61?lP8203o`0000CoB4Q80ol^;Rh4ofQXJ0;on_[j4Ooo
ool000oooooo0onNWYh4oi>CT`?oVi^K1?l/;2`3o``<30Co:b/[0ooalO43obXZ:P;ooooo1?l>3Ph3
o`H61PCo<C4a0olS8b<4onk^kP?o7AdM1?l71`L2o`H61P?ooooo0olA4A44o`L71`?o00001?l;2`/3
oa0@40Co;Rh^0oono_h2ocDe=@?ooooo1?lS8b<3oe1@D0CoCdm?0omHF5P3oa4A4@Co5ADE0oooool3
ohJ6QP?ooooo0omBDU84oo3`l0?oT92@0omnOWh4o`0000?o9bLW1?l/;2`2od51@@?ooooo0ol51@D4
od51@@?oAdM71?obl_83ocDe=@Co8R8R0olI6AT3oa0@40;ooooo1?mRHV83obd];@Co3@d=0ol00004
ocTi>@?oCDe=1?n_[jl2ojb/[0?ooooo0olI6AT4o`h>3P?oFEUI1?lV9RH3obd];@Cooooo0olJ6QX3
o`<30`;ooooo1?ojn_X3oa0@40Co@T920ol>3Ph4o`T92@?o=CDe1?ln?Sh2ob@T90?ooooo0onh^;P4
oe=CD`?o30`<1?mADE43oeEEE@CoKFe]0omPH603oo7al@;ooooo1?m7AdL3oc4a<@Co6QXJ0ol;2`/4
o`X:2P?o3`l?0olC4a<3ojf][@?ooooo0omKFe/4o`d=3@?om?Cd1?ohn?P3oc@d=0?o]kNg1?mJFUX2
ohf=S@?ooooo1?nPX:03ooooo`?o/[:b1?lN7Qh3oc@d=0Co2@T90ol];Bd3oc4a<@?ooooo0olK6a/3
obTY:@Co/;2`0onCTi<4oc4a<@?o:RXZ1?l[:b/2onCTi0?ooooo0olQ8B44oc4a<@?omoOg1?lf=SH3
o`T92@Co3@d=0ol:2PX3obDU9@;ooooo1?n0P803o`h>3PCo92@T0ol;2`/4odY:BP?o>c/k1?mWIfLC
ooooo`003oooool3oijNWPCoTi>C0onKVi/4ob`/;0?o30`<1?l[:b/3oo7al@?o:RXZ0_ooool4o`h>
3P?o1PH61?la<C43ob<S8`Cok^k^0olM7Ad4o`L71`;o1PH60oooool3oa4A4@Co1`L70ol00004o`/;
2`?o410@1?l^;Rh3ooknoP;o=CDe0oooool4ob<S8`?oD51@1?m?Cdl3oeQHF0?o4A4A1?lE5AD3oooo
o`?oQXJ60oooool3oe9BDPCol?3`0on@T903oginOPCo00000olW9bL4ob`/;0;o@D510oooool3o`D5
1@Co@D510om7AdL4oo;blP?o=CDe1?lR8R83oaTI6@?o410@0_ooool4of9RHP?o;Bd]1?l=3@d3o`00
00Co>CTi0om=CDd4ojn_[`;o[:b/0oooool3oaTI6@Co3Ph>0omIFET4obHV9P?o;Bd]1?ooool3oaXJ
6P?o0`<30_ooool4oo[jnP?o410@1?m2@T83o`h>3PCo2@T90ole=CD4ochn?P;o92@T0oooool3okRh
^0CoDe=C0ol<30`4oe5AD@?oEEEE1?m]KFd3of1PH0?olO7a0_ooool4odM7A`?o<C4a1?lJ6QX3o`/;
2`Co2PX:0ol?3`l3oa<C4`?o[Jf]0oooool3oe]KF`Co3@d=0oodm?@4ooShn0?o=3@d0ong]kL4oeYJ
FP;oSHf=0oooool4oj2PX0?ooooo0onb/[84oahN7P?o=3@d1?l92@T3obd];@?o<C4a0oooool3oa/K
6`?o:BTY1?n`/;03oi>CT`Co<C4a0olZ:RX4ob/[:`;oi>CT0oooool3ob4Q8@Co<C4a0oogmoL4ocHf
=P?o2@T91?l=3@d3o`X:2P?o9BDU0_ooool4oh20P0?o3Ph>1?lT92@3o`/;2`CoBTY:0olk>c/4ofMW
Ia?ooooo000?ooooo`?oWYjN1?nCTi<3oi^KV`Co;2`/0ol<30`4ob/[:`?olO7a0olZ:RX2ooooo`Co
3Ph>0ol61PH4oc4a<@?o8b<S1?o^k^h3oadM7@Co1`L70_l61PH3ooooo`?o4A4A1?l71`L3o`0000Co
2`/;0ol@4104obh^;P?oo_kn0_le=CD3ooooo`Co8b<S0om@D504odm?C`?oF5QH0olA4A44oaDE5@?o
oooo0on6QXH3ooooo`?oDU9B1?o`l?03oi2@T0?oOWin1?l00003obLW9`Co;2`/0_m1@D43ooooo`?o
1@D51?m1@D43odM7A`Col_;b0ole=CD4ob8R8P?o6ATI0ol@4102ooooo`CoHV9R0ol];Bd4o`d=3@?o
00001?li>CT3ode=C@Co[jn_0_n/[:`3ooooo`?o6ATI1?l>3Ph3oeUIF@Co9RHV0ol];Bd4ooooo`?o
6QXJ0ol30`<2ooooo`Con_[j0ol@4104od92@P?o3Ph>1?l92@T3ocDe=@Co?Shn0_lT92@3ooooo`?o
^;Rh1?mCDe<3o``<30CoDE5A0omEEED4ofe]K@?oH61P0ooalO42ooooo`CoAdM70ola<C44oaXJ6P?o
2`/;1?l:2PX3o`l?3`?o4a<C0on][Jd3ooooo`?oFe]K1?l=3@d3ooCdm0Con?Sh0old=3@3okNg]`Co
FUYJ0_n=SHd3ooooo`CoX:2P0oooool3ok:b/PCo7QhN0old=3@4o`T92@?o;Bd]0ola<C43ooooo`?o
6a/K0olY:BT4ok2`/0?oTi>C1?la<C43obXZ:PCo:b/[0_oTi>@3ooooo`?o8B4Q1?la<C43ooOgm`Co
=SHf0ol92@T4o`d=3@?o2PX:0olU9BD2ooooo`CoP8200ol>3Ph4ob@T90?o2`/;1?m:BTX3oc/k>`Co
IfMW4oooool000oooooo0omRHV84ooooo`?oY:BT1?lE5AD3oa4A4@CoVIVI0onHV9P3oaTI6@;ooooo
1?l;2`/3o`T92@Co<30`0ol[:b/4ooooo`?o6a/K1?l410@2o``<30?ooooo0olI6AT4o`d=3@?o20P8
1?l;2`/3oahN7PCo9BDU0oojn_X2ochn?P?ooooo1?lS8b<3odU9B@CoG5aL0om<C4`3o`@410Co;2`/
0ooalO43ogemO@?ooooo0omDE5@4ofIVIP?ohn?S0omjNWX4o`X:2P?o:RXZ1?oCdm<2oc0`<0?ooooo
0ol;2`/4ocLg=`?oATI61?o_knl3obl_;`Co8b<S0olF5QH3o`0000;ooooo1?m`L703ode=C@Co820P
0olE5AD4oemOG`?oFEUI1?no_kl2omgMg@?ooooo0olL71`4oa8B4P?oFe]K1?lb<S83obTY:@CojNWY
0olR8R83o`0000;ooooo1?ohn?P3o`0000Co=cLg0olJ6QX4oa4A4@?o>CTi1?lf=SH2obLW9`?ooooo
0omXJ6P4odI6AP?o00001?lM7Ad3ojj^[PCo`L710om_Kfl3ocHf=P;ooooo1?li>CT3obTY:@Co6a/K
0olE5AD4o`<30`?o1`L70olD51@3ogYjNP?ooooo0om5ADD4obHV9P?ok^k^1?oIfMT3oa`L70?oYjNW
1?mJFUX2ogIfMP?ooooo1?nEUID3onSXj0?oY:BT1?l820P3ob`/;0Co2PX:0om7AdL3ocDe=@?ooooo
0ol00003oc/k>`CoDe=C0omOGel4ogEeM@Oo8b<S0_l51@D3ooooo`?o820P1?lc<c<3on_[j`Co=CDe
0ol61PH4oadM7@?o2PX:0olX:2P2ooooo`CoMWIf0ol71`L4oalO7`?o1PH61?mBDU83ob`/;0CoJVYZ
0_obl_8Aooooo`003oooool3of9RHPCooooo0onTY:@4oaDE5@?o4A4A1?nIVIT3oiRHV0?o6ATI0_oo
ool4o`/;2`?o2@T91?l`<303ob/[:`Cooooo0olK6a/4o`@410;o30`<0oooool3oaTI6@Co3@d=0ol8
20P4o`/;2`?o7QhN1?lU9BD3oo[jnP;o?Shn0oooool4ob<S8`?oBDU91?mLG5`3oda<C0?o10@41?l/
;2`3oo7al@?oOGem0oooool3oeADE0CoIVIV0ooShn<3ogYjNPCo2PX:0olZ:RX4om?Cd`;o<30`0ooo
ool3o`/;2`Co=cLg0om6ATH4ono_k`?o;bl_1?lS8b<3oaHF5P?o00000_ooool4og1`L0?oCDe=1?lP
8203oaDE5@CoGemO0omIFET4okno_`;ogMgM0oooool3oa`L70Co4Q8B0omKFe/4oc8b<P?o:BTY1?oY
jNT3ob8R8P?o00000_ooool4ooShn0?o00001?lg=cL3oaXJ6PCo4A4A0oli>CT4ocHf=P;o9bLW0ooo
ool3ofQXJ0CoATI60ol00004oadM7@?o[Zj^1?o1`L43ofm_K`?o=SHf0_ooool4ocTi>@?o:BTY1?lK
6a/3oaDE5@Co0`<30ol71`L3oa@D50?oNWYj0oooool3odE5A@Co9RHV0oo^k^h4omWIf@?o71`L0onW
YjL4oeYJFP;oMWIf0oooool4oiFEU@?oj>SX0onTY:@4o`P820?o;2`/1?l:2PX3odM7A`?o=CDe0ooo
ool3o`0000?o>c/k1?mCDe<3oemOG`CoMGEe1olS8b<2o`D51@?ooooo0olP8204oc<c<`?ojn_[1?le
=CD3o`H61PCo7AdM0ol:2PX3obPX:0;ooooo1?mfMWH3o`L71`Co7alO0ol61PH4oe9BDP?o;2`/1?mZ
JVX2oo;blQ7ooooo000?ooooo`?oHV9R1?ooool3ojBTY0Co5ADE0olA4A44oiVIV@?oV9RH0olI6AT2
ooooo`Co2`/;0ol92@T4oc0`<0?o:b/[1?ooool3oa/K6`Co10@40_l<30`3ooooo`?o6ATI1?l=3@d3
o`P820Co2`/;0olN7Qh4obDU9@?on_[j0_ln?Sh3ooooo`Co8b<S0om9BDT4oeaLG0?oC4a<0ol410@4
ob`/;0?olO7a0ommOGd3ooooo`?oE5AD1?mVIVH3on?Sh`?oNWYj1?l:2PX3obXZ:PCodm?C0_l`<303
ooooo`?o2`/;1?lg=cL3odI6APCokno_0ol_;bl4ob<S8`?o5QHF0ol00002ooooo`CoL71`0om=CDd4
ob0P80?o5ADE1?mOGel3oeUIF@Co_kno0_oMgMd3ooooo`?o71`L1?lB4Q83oe]KF`Co<S8b0olY:BT4
onWYj@?o8R8R0ol00002ooooo`Con?Sh0ol00004ocLg=`?o6QXJ1?lA4A43ocTi>@Co=SHf0_lW9bL3
ooooo`?oJ6QX1?m6ATH3o`0000Co7AdM0on^[Zh4ol71`@?oKfm_0olf=SH2ooooo`Co>CTi0olY:BT4
oa/K6`?o5ADE1?l30`<3o`L71`?o51@D0omjNWX3ooooo`?oADE51?lV9RH3onk^kPCofMWI0olL71`3
ojNWY`CoFUYJ0_mfMWH3ooooo`CoUIFE0ooXj>P3ojBTY0Co20P80ol/;2`4o`X:2P?oAdM70ole=CD3
ooooo`?o00000olk>c/4oe=CD`?oGemO1?meMGD7ob<S8`;o1@D50oooool3ob0P80Co<c<c0oo[jn/4
ocDe=@?o1PH61?lM7Ad3o`X:2P?o:2PX0_ooool4ogIfMP?o1`L71?lO7al3o`H61PCoDU9B0ol/;2`4
ofYZJP;ol_;b4Oooool000oooooo0omRHV84ooooo`?oY:BT1?lE5AD3oa4A4@CoVIVI0onHV9P3oaTI
6@;ooooo1?l;2`/3o`T92@Co<30`0ol[:b/4ooooo`?o6a/K1?l410@2o``<30?ooooo0olI6AT4o`d=
3@?o20P81?l;2`/3oahN7PCo9BDU0oojn_X2ochn?P?ooooo1?lS8b<3odU9B@CoG5aL0om<C4`3o`@4
10Co;2`/0ooalO43ogemO@?ooooo0omDE5@4ofIVIP?ohn?S0omjNWX4o`X:2P?o:RXZ1?oCdm<2oc0`
<0?ooooo0ol;2`/4ocLg=`?oATI61?o_knl3obl_;`Co8b<S0olF5QH3o`0000;ooooo1?m`L703ode=
C@Co820P0olE5AD4oemOG`?oFEUI1?no_kl2omgMg@?ooooo0olL71`4oa8B4P?oFe]K1?lb<S83obTY
:@CojNWY0olR8R83o`0000;ooooo1?ohn?P3o`0000Co=cLg0olJ6QX4oa4A4@?o>CTi1?lf=SH2obLW
9`?ooooo0omXJ6P4odI6AP?o00001?lM7Ad3ojj^[PCo`L710om_Kfl3ocHf=P;ooooo1?li>CT3obTY
:@Co6a/K0olE5AD4o`<30`?o1`L70olD51@3ogYjNP?ooooo0om5ADD4obHV9P?ok^k^1?oIfMT3oa`L
70?oYjNW1?mJFUX2ogIfMP?ooooo1?nEUID3onSXj0?oY:BT1?l820P3ob`/;0Co2PX:0om7AdL3ocDe
=@?ooooo0ol00003oc/k>`CoDe=C0omOGel4ogEeM@Oo8b<S0_l51@D3ooooo`?o820P1?lc<c<3on_[
j`Co=CDe0ol61PH4oadM7@?o2PX:0olX:2P2ooooo`CoMWIf0ol71`L4oalO7`?o1PH61?mBDU83ob`/
;0CoJVYZ0_obl_8Aooooo`003oooool3oj^[Z`CoZZZZ0om`L704ob8R8P?o9bLW1?m<C4`3oo7al@?o
A4A40_ooool4oaLG5`?o4A4A1?la<C43ocPh>0CofMWI0olW9bL4o`P820;o2PX:0oooool3ob0P80Co
0P820ol61PH4oa0@40?o8b<S1?lT92@3onSXj0;o?3`l0oooool4oa/K6`?oBDU91?m?Cdl3odI6AP?o
3Ph>1?l`<303on[ZjP?oNg]k0oooool3ofi^KPConOWi0on4Q8@3ocdm?@Co2@T90olh>3P4odU9B@;o
8b<S0oooool3oaLG5`CoADE50om<C4`4om;BdP?o=3@d1?l`<303oadM7@?o61PH0_ooool4ogYjNP?o
@d=31?lF5QH3oa/K6`CoEEEE0omOGel4olS8b0;o^k^k0oooool3ob8R8PCo7alO0om=CDd4obDU9@?o
<S8b1?oOgml3ob0P80?o1@D50_ooool4on?Sh`?o0P821?l[:b/3oaLG5`Co5ADE0om5ADD4oc`l?0;o
=SHf0oooool3ojBTY0Co9RHV0olW9bL4oeUIF@?oHV9R1?mUIFD3oe=CD`Gooooo1?lh>3P3ocDe=@Co
6ATI0olF5QH4oa8B4P?o2PX:0olC4a<3ofMWI`?ooooo0om4A4@4oahN7P?oinOW1?oCdm<3oahN7P?o
WYjN1?m@D502oi6AT@?ooooo1?n`/;03onWYj@?oV9RH1?lJ6QX3obd];@Co20P80olj>SX3odA4A0?o
oooo0olR8R83oe9BDPCoWYjN0omjNWX4oaPH60?o9RHV1?lZ:RX2ojVYZ@?ooooo0olR8R84oc<c<`?o
eMGE1?lb<S83o`l?3`Co61PH0olK6a/3ocPh>0;ooooo1?mXJ6P3o``<30Co8b<S0olD51@4oeIFEP?o
=SHf1?mTI6@2onSXj17ooooo000?ooooo`?oZj^[1?nZZZX3og1`L0Co8R8R0olW9bL4oda<C0?olO7a
0om4A4@2ooooo`Co5aLG0olA4A44oc4a<@?o>3Ph1?oIfMT3obLW9`Co20P80_l:2PX3ooooo`?o820P
1?l20P83o`H61PCo410@0olS8b<4ob@T90?oj>SX0_ll?3`3ooooo`Co6a/K0om9BDT4odm?C`?oATI6
0ol>3Ph4oc0`<0?oj^[Z0omkNg/3ooooo`?oKVi^1?oinOT3ohB4Q0?o?Cdm1?l92@T3ocPh>0CoBDU9
0_lS8b<3ooooo`?o5aLG1?m5ADD3oda<C0Cod];B0old=3@4oc0`<0?o7AdM0olH61P2ooooo`CoNWYj
0om3@d<4oaHF5P?o6a/K1?mEEED3oemOG`Cob<S80_nk^k/3ooooo`?o8R8R1?lO7al3ode=C@Co9BDU
0olb<S84omoOg`?o820P0ol51@D2ooooo`Cohn?S0ol20P84ob/[:`?o5aLG1?lE5AD3odE5A@Co?3`l
0_lf=SH3ooooo`?oY:BT1?lV9RH3obLW9`CoFEUI0omRHV84ofEUI@?oDe=C1Oooool4ocPh>0?o=CDe
1?lI6AT3oaHF5PCo4Q8B0ol:2PX3oa<C4`?oIfMW0oooool3odA4A0Co7QhN0ooWinL4om?Cd`?o7QhN
0onNWYh4oe1@D0;oTI6A0oooool4ok2`/0?ojNWY0onHV9P4oaXJ6P?o;Bd]1?l820P3ocXj>P?oA4A4
0oooool3ob8R8P?oDU9B1?nNWYh3ogYjNPCo61PH0olV9RH4obXZ:P;oZJVY0oooool3ob8R8PCo<c<c
0ooEeMD4oc8b<P?o3`l?1?lH61P3oa/K6`?o>3Ph0_ooool4ofQXJ0?o30`<1?lS8b<3oa@D50CoEUIF
0olf=SH4ofATI0;oj>SX4Oooool000oooooo0on[Zj/4ojZZZP?oL71`1?lR8R83obLW9`CoC4a<0ooa
lO43odA4A0;ooooo1?lG5aL3oa4A4@Co<C4a0olh>3P4omWIf@?o9bLW1?l820P2o`X:2P?ooooo0olP
8204o`820P?o1PH61?l@4103ob<S8`Co92@T0ooXj>P2oc`l?0?ooooo1?lK6a/3odU9B@CoCdm?0om6
ATH3o`h>3PCo<30`0ooZj^X3og]kN`?ooooo0om^KVh4ooWin@?oQ8B40olm?Cd4o`T92@?o>3Ph1?m9
BDT2ob<S8`?ooooo0olG5aL4odE5A@?oC4a<1?oBd]83oc@d=0Co<30`0olM7Ad3oaPH60;ooooo1?mj
NWX3od=3@`Co5QHF0olK6a/4oeEEE@?oGemO1?o8b<P2ok^k^`?ooooo0olR8R84oalO7`?oCDe=1?lU
9BD3oc8b<PCogmoO0olP8203o`D51@;ooooo1?oShn<3o`820PCo:b/[0olG5aL4oaDE5@?oADE51?ll
?3`2ocHf=P?ooooo0onTY:@4obHV9P?o9bLW1?mIFET3of9RHPCoIFEU0omCDe<5ooooo`Co>3Ph0ole
=CD4oaTI6@?o5QHF1?lB4Q83o`X:2P?o4a<C0omWIfL3ooooo`?oA4A41?lN7Qh3onOWi`Codm?C0olN
7Qh3oijNWPCoD51@0_nATI43ooooo`Co/;2`0ooYjNT3oiRHV0Co6QXJ0ol];Bd4o`P820?o>SXj0om4
A4@3ooooo`?o8R8R0omBDU84oijNWP?oNWYj1?lH61P3obHV9PCo:RXZ0_nYZJT3ooooo`?o8R8R1?lc
<c<3omGEe@Co<S8b0ol?3`l4oaPH60?o6a/K0olh>3P2ooooo`CoJ6QX0ol<30`4ob<S8`?o51@D1?mF
EUH3ocHf=PCoI6AT0_oXj>PAooooo`003oooool3og5aL@CokNg]0on9RHT4o`T92@?o<C4a1?nm_Kd3
ohB4Q0?o@D510_ooool4oaDE5@?o410@1?l^;Rh3obXZ:PCod];B0olW9bL4o`0000;o1@D50oooool3
oa/K6`Co0`<30ol:2PX4o`P820?o7AdM1?lX:2P3om3@d0;o@4100oooool4oahN7P?o<c<c1?m<C4`3
ocTi>@?o00001?lg=cL3ol71`@?oI6AT0oooool3of5QH@CoH61P0ooIfMT3ofUYJ@Co00000omADE44
ooooo`;o>SXj0oooool3oaLG5`Co;Rh^0om7AdL4ol?3``?o9bLW1?lN7Qh3ob0P80?o1@D50_ooool4
of][J`?oH61P1?lW9bL3obXZ:PCoFEUI0omgMgL4ojj^[PGooooo0olW9bL4obHV9P?oBDU91?l];Bd3
oc<c<`CoalO70olN7Qh3oa8B4P;ooooo1?no_kl3o`<30`Co:2PX0olK6a/4oa<C4`?o<S8b1?l];Bd2
od51@@?ooooo0om6ATH4od=3@`?o00001?ld=3@3olG5a@Co][Jf0om=CDd3ob4Q8@;ooooo1?ld=3@3
ob8R8PCo51@D0ol@4104o`l?3`Ko30`<0omTI6@3ooooo`?o9RHV1?lY:BT3om?Cd`CoWinO0ol92@T3
oh61P@CoBTY:0_mjNWX3ooooo`CoTY:B0onh^;P3ogemO@Co00000olK6a/4o`820P?oADE50old=3@3
ooooo`?o0P820oln?Sh4odI6AP?o?3`l1?mVIVH3oa0@40Co9bLW0_l92@T3ooooo`?o7AdM1?l_;bl3
okbl_0Co:b/[0ol20P84oaLG5`?o51@D0olf=SH2ooooo`CoF5QH0ol;2`/4oaPH60?o2`/;1?m>CTh3
ob/[:`CoEeMG0_o5aLDAooooo`003oooool3og5aL@CokNg]0on9RHT4o`T92@?o<C4a1?nm_Kd3ohB4
Q0?o@D510_ooool4oaDE5@?o410@1?l^;Rh3obXZ:PCod];B0olW9bL4o`0000;o1@D50oooool3oa/K
6`Co0`<30ol:2PX4o`P820?o7AdM1?lX:2P3om3@d0;o@4100oooool4oahN7P?o<c<c1?m<C4`3ocTi
>@?o00001?lg=cL3ol71`@?oI6AT0oooool3of5QH@CoH61P0ooIfMT3ofUYJ@Co00000omADE44oooo
o`;o>SXj0oooool3oaLG5`Co;Rh^0om7AdL4ol?3``?o9bLW1?lN7Qh3ob0P80?o1@D50_ooool4of][
J`?oH61P1?lW9bL3obXZ:PCoFEUI0omgMgL4ojj^[PGooooo0olW9bL4obHV9P?oBDU91?l];Bd3oc<c
<`CoalO70olN7Qh3oa8B4P;ooooo1?no_kl3o`<30`Co:2PX0olK6a/4oa<C4`?o<S8b1?l];Bd2od51
@@?ooooo0om6ATH4od=3@`?o00001?ld=3@3olG5a@Co][Jf0om=CDd3ob4Q8@;ooooo1?ld=3@3ob8R
8PCo51@D0ol@4104o`l?3`Ko30`<0omTI6@3ooooo`?o9RHV1?lY:BT3om?Cd`CoWinO0ol92@T3oh61
P@CoBTY:0_mjNWX3ooooo`CoTY:B0onh^;P3ogemO@Co00000olK6a/4o`820P?oADE50old=3@3oooo
o`?o0P820oln?Sh4odI6AP?o?3`l1?mVIVH3oa0@40Co9bLW0_l92@T3ooooo`?o7AdM1?l_;bl3okbl
_0Co:b/[0ol20P84oaLG5`?o51@D0olf=SH2ooooo`CoF5QH0ol;2`/4oaPH60?o2`/;1?m>CTh3ob/[
:`CoEeMG0_o5aLDAooooo`003oooool3og5aL@CokNg]0on9RHT4o`T92@?o<C4a1?nm_Kd3ohB4Q0?o
@D510_ooool4oaDE5@?o410@1?l^;Rh3obXZ:PCod];B0olW9bL4o`0000;o1@D50oooool3oa/K6`Co
0`<30ol:2PX4o`P820?o7AdM1?lX:2P3om3@d0;o@4100oooool4oahN7P?o<c<c1?m<C4`3ocTi>@?o
00001?lg=cL3ol71`@?oI6AT0oooool3of5QH@CoH61P0ooIfMT3ofUYJ@Co00000omADE44ooooo`;o
>SXj0oooool3oaLG5`Co;Rh^0om7AdL4ol?3``?o9bLW1?lN7Qh3ob0P80?o1@D50_ooool4of][J`?o
H61P1?lW9bL3obXZ:PCoFEUI0omgMgL4ojj^[PGooooo0olW9bL4obHV9P?oBDU91?l];Bd3oc<c<`Co
alO70olN7Qh3oa8B4P;ooooo1?no_kl3o`<30`Co:2PX0olK6a/4oa<C4`?o<S8b1?l];Bd2od51@@?o
oooo0om6ATH4od=3@`?o00001?ld=3@3olG5a@Co][Jf0om=CDd3ob4Q8@;ooooo1?ld=3@3ob8R8PCo
51@D0ol@4104o`l?3`Ko30`<0omTI6@3ooooo`?o9RHV1?lY:BT3om?Cd`CoWinO0ol92@T3oh61P@Co
BTY:0_mjNWX3ooooo`CoTY:B0onh^;P3ogemO@Co00000olK6a/4o`820P?oADE50old=3@3ooooo`?o
0P820oln?Sh4odI6AP?o?3`l1?mVIVH3oa0@40Co9bLW0_l92@T3ooooo`?o7AdM1?l_;bl3okbl_0Co
:b/[0ol20P84oaLG5`?o51@D0olf=SH2ooooo`CoF5QH0ol;2`/4oaPH60?o2`/;1?m>CTh3ob/[:`Co
EeMG0_o5aLDAooooo`00oooooooeooooo`00oooooooeooooo`00\
\>"],
  ImageRangeCache->{{{0, 499}, {59, 0}} -> {-0.525373, -0.04801, 0.0341698, \
0.0341698}, {{15.375, 42.125}, {57.5625, 30.8125}} -> {-4.59822, -0.429951, \
0.299068, 0.299068}, {{44.75, 71.5625}, {57.5625, 30.8125}} -> {-13.3927, \
-0.429951, 0.299068, 0.299068}, {{74.1875, 100.938}, {57.5625, 30.8125}} -> \
{-22.1872, -0.429951, 0.299068, 0.299068}, {{103.625, 130.375}, {57.5625, \
30.8125}} -> {-30.991, -0.429951, 0.299068, 0.299068}, {{133.062, 159.812}, \
{57.5625, 30.8125}} -> {-39.7948, -0.429951, 0.299068, 0.299068}, {{162.5, \
189.25}, {57.5625, 30.8125}} -> {-48.5987, -0.429951, 0.299068, 0.299068}, \
{{191.938, 218.688}, {57.5625, 30.8125}} -> {-57.4025, -0.429951, 0.299068, \
0.299068}, {{221.375, 248.125}, {57.5625, 30.8125}} -> {-66.2063, -0.429951, \
0.299068, 0.299068}, {{250.812, 277.562}, {57.5625, 30.8125}} -> {-75.0101, \
-0.429951, 0.299068, 0.299068}, {{280.25, 307}, {57.5625, 30.8125}} -> \
{-83.814, -0.429951, 0.299068, 0.299068}, {{309.688, 336.438}, {57.5625, \
30.8125}} -> {-92.6178, -0.429951, 0.299068, 0.299068}, {{339.125, 365.875}, \
{57.5625, 30.8125}} -> {-101.422, -0.429951, 0.299068, 0.299068}, {{368.562, \
395.312}, {57.5625, 30.8125}} -> {-110.225, -0.429951, 0.299068, 0.299068}, \
{{398, 424.75}, {57.5625, 30.8125}} -> {-119.029, -0.429951, 0.299068, \
0.299068}, {{427.375, 454.188}, {57.5625, 30.8125}} -> {-127.824, -0.429951, \
0.299068, 0.299068}, {{456.812, 483.625}, {57.5625, 30.8125}} -> {-136.628, \
-0.429951, 0.299068, 0.299068}, {{15.375, 42.125}, {28.125, 1.375}} -> \
{-4.59822, -9.23378, 0.299068, 0.299068}, {{44.75, 71.5625}, {28.125, 1.375}} -> \
{-13.3927, -9.23378, 0.299068, 0.299068}, {{74.1875, 100.938}, {28.125, \
1.375}} -> {-22.1872, -9.23378, 0.299068, 0.299068}, {{103.625, 130.375}, \
{28.125, 1.375}} -> {-30.991, -9.23378, 0.299068, 0.299068}, {{133.062, \
159.812}, {28.125, 1.375}} -> {-39.7948, -9.23378, 0.299068, 0.299068}, \
{{162.5, 189.25}, {28.125, 1.375}} -> {-48.5987, -9.23378, 0.299068, \
0.299068}, {{191.938, 218.688}, {28.125, 1.375}} -> {-57.4025, -9.23378, \
0.299068, 0.299068}, {{221.375, 248.125}, {28.125, 1.375}} -> {-66.2063, \
-9.23378, 0.299068, 0.299068}, {{250.812, 277.562}, {28.125, 1.375}} -> \
{-75.0101, -9.23378, 0.299068, 0.299068}, {{280.25, 307}, {28.125, 1.375}} -> \
{-83.814, -9.23378, 0.299068, 0.299068}, {{309.688, 336.438}, {28.125, \
1.375}} -> {-92.6178, -9.23378, 0.299068, 0.299068}, {{339.125, 365.875}, \
{28.125, 1.375}} -> {-101.422, -9.23378, 0.299068, 0.299068}, {{368.562, \
395.312}, {28.125, 1.375}} -> {-110.225, -9.23378, 0.299068, 0.299068}, \
{{398, 424.75}, {28.125, 1.375}} -> {-119.029, -9.23378, 0.299068, 0.299068}, \
{{427.375, 454.188}, {28.125, 1.375}} -> {-127.824, -9.23378, 0.299068, \
0.299068}, {{456.812, 483.625}, {28.125, 1.375}} -> {-136.628, -9.23378, \
0.299068, 0.299068}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .12 
%%ImageSize: 500 60 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0595238 0.00285714 0.0595238 [
[ 0 0 0 0 ]
[ 1 .12 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .12 L
0 .12 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.00285714 0.0782313 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
56083D72094C7326
3019590E0E6D1400
212D6B2F30402819
25A549046E48101D
1E698D396E3B4F03
16FF5821E41C2841
24D25D759C15B518
34670E2540373A13
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0836735 0.00285714 0.138095 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
121305101F262F20
0A0703090C090812
0109040D00100E0D
0406000306001511
06110304100D0C13
24261A040A140E16
21191B1518100B19
323646382B65FFBB
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.143537 0.00285714 0.197959 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
3E4D394622283046
3B401E1B111D1626
25240D0B01080218
33271B00030C0015
333C34242C272325
69584D3D3D363434
4975524C575C6257
F1FFB8D4AD8D85AF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.203401 0.00285714 0.257823 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
121665001F8D102D
1B1F41204A22185B
279826154D3C221D
47504F8339183E19
30FF1F3C93353A39
7CA22BEF421EC621
6595228B76646E1B
5F043C8C0E356544
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.263265 0.00285714 0.317687 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1423142235353C4A
22201731202BBC5A
1D1822152C286C2A
00090D0C1D251D1E
17140408321E3E2C
1E14212C2C33FF56
2D261F1132478038
241D171B303B3444
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.323129 0.00285714 0.377551 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
4BC35193BB85C94E
59202CC13039E32F
211B3712236A271D
1D0C412500304018
6F60502B28280615
5165EF4C5E90380B
63FD9753FF4A3369
694B58846A62A529
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.382993 0.00285714 0.437415 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
2153774F401C1126
4E206D8E03121406
2B5F851A2E2C0019
42337C7B18272023
0A677E1734170603
6E59C6A1192A2330
2D49921929221119
557BC1FF425A3E44
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.442857 0.00285714 0.497279 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
4445263C263D3E08
D3232D9215275611
B21DB3580BB91A22
AA1CB75C4F9B3E89
CF3EBA5478DE2E6A
5A1FFF1F61D516A3
4F3F922DA37A18AC
0E0D192A2F00442B
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.502721 0.00285714 0.557143 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
2522030300021622
23160D010F080E19
31191B2127262D3C
644C6861536C6167
718090C2A2D0A19F
8A9FD7FFF5F2E9C3
848BB2AD959B9F89
CBC89FBBA5C7AEA1
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.562585 0.00285714 0.617007 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
2F30231713120E01
0B0A090109090E0C
050C000F0F160F16
0B09060B0612160A
0D1112080804080A
0A0A120214070400
031716280F190F09
DCFF7E44483A3636
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.622449 0.00285714 0.676871 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
22070B070C000718
2D3B3E4E594E5759
759CC9FFFCDBD295
61637898D1DCCEB6
7B6F71686068756C
734F4C2F2B2B2F36
383A101F09030817
AD7C796E7360557A
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.682313 0.00285714 0.736735 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
14FF1339A02E452D
7B8E27FB4313C515
6FDA1F9592667316
6FB20FFD4F48FC15
AD4818E72E7DBC13
446E406D3A9F8419
80001CB10857AF31
32114E4407232720
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.742177 0.00285714 0.796599 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
181B643A698F3F1A
18005A596E401628
2515641B71893010
15124F365F5C2921
14109A6A56A64013
1C496B40FFC32134
2514798088823232
1213543B474F1F2D
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.802041 0.00285714 0.856463 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
AA1EEF5019FF1E2E
9C1BC66B6CA72FD7
AE2EA84C5CCE257D
1E14D80F58CA16AD
425A4A27A4460DA3
0D2F770C42774E42
2D4B231D7C001665
830D0F6B2F175A32
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.861905 0.00285714 0.916327 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
B512212317212010
35110D1903151217
82182A240627180E
3F1C16140719000E
FF1E2D481B2D230C
421C09180A170D0D
91392B3C0F1F1C06
B23B2A2E24162027
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.921769 0.00285714 0.97619 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
BE8030FF288BC228
9F724ECA2AE8B21D
7F2058A700BBB337
580B4F590AA44E30
300E4E40157F3C2C
1640381229471224
165E3F182F40180A
646F1D793D384F14
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.0627211 0.0782313 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
77736530743E093B
2948EB124A8F3A27
38A48A09FF3A188D
2D918D1A99587B3C
1A639427BB425E73
0FA02714CB18705F
4027456549259E22
0B1941240D364500
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0836735 0.0627211 0.138095 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0A16241E2D30220A
1E1312040E0C0602
23190C1712090114
130F0F06171B1B15
131F0E201D1E1C10
200F0E261612200D
0F0B1700020C1010
4F40415CFFF7893F
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.143537 0.0627211 0.197959 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
71EA8C513F1C0C13
FFC5714C1C120E21
9DBD775932210A1C
F4AD615E3F1B191C
4D9E5C44282B0F2B
CF7D404A2722002B
8A5F51271B2A0413
6474522A1F330A2F
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.203401 0.0627211 0.257823 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
143D84224B3B2715
16FF2C04C51A113E
2D7B70706F219F01
27DF2D4EEB074B47
1CBD1091A504E30D
54401CA65F11BA07
1E7F074D8C368911
462B03090C24000E
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.263265 0.0627211 0.317687 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1F21261E3F25793B
121100131D193E27
2823111F3A2D4030
35242E443F3BD556
302F22224A3E8237
0E15021522272D22
20221A19392A4638
31293534403BFF57
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.323129 0.0627211 0.377551 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
3D465B5857656854
ABC2FFE1D5A3B069
606B7D67644E4125
160A0C0D210F201A
0900010201181924
0C090F1F2B394142
33422C4B4F565857
82918D8F9FADC7B3
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.382993 0.0627211 0.437415 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1F8F2F306F859B59
60C7783C9AA3B94C
0D4A330870AE7040
7FC87C689E9BC877
0063513A53979649
6BBB423ACD7FC66E
086B2C3B10A69468
57D04A4474FFC0AA
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.442857 0.0627211 0.497279 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
241B3751232B8729
4242002C54163549
3E4522391014670B
C81A1E7D10143116
7C29C0310C951013
D1236488594E5073
A419DC4E3EFF2C34
425511209C113B57
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.502721 0.0627211 0.557143 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
195C1618952B118A
115E3A1B3C3F5329
653B105B3E103444
48983A3D44494312
FF4718FB0C288336
BD6C8E9232D1281B
C7495ED5227B8D86
0215AA0040AF0B4D
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.562585 0.0627211 0.617007 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
231B313621110F18
0B0F11110C061412
05161212081B151B
08001216171A0C18
0D150A150F0C0C0B
0912070910060D06
0C01001014170A0C
3854E5FF823E403F
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.622449 0.0627211 0.676871 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
333A26342E2F322D
321C26322F343D31
4D583E4725393329
C4FFEFC9A74C5526
5041544C47412D2A
0A000C0411062421
29160F150A131322
A76F80636B597C68
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.682313 0.0627211 0.736735 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
3ADE7C8AF51ADE2C
35D53880FF0FD146
38AF1291B203FB22
45440896561BE608
304820544D3A8E0D
69101A64001F7D04
842A6A38046E2306
119F5B2497202B1C
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.742177 0.0627211 0.796599 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
5A2A4A966D6F4F2B
17467531EFFF2C23
2015325C303B2214
181A3F38564B2C1C
1005081C340D1D12
1C00222E07121D14
15072F3B2A433513
3B3C6F60CAEC4135
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.802041 0.0627211 0.856463 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
8926DE5D8FDF1BFF
7125A62F91C818B5
0C2D8B0F6F891B9F
0A38300095231696
04422D1D3C3B3131
4B4D00213C01143A
6A7A196B192F4502
9A13A543169E153D
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.861905 0.0627211 0.916327 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
32171A5E2F534854
0F5E2A1D4B472256
617736FF9B48C7BB
28522324664D093A
356027B0755470BB
003825231E32001E
4A6D28BDB33C6E7E
226D343B983E177D
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.921769 0.0627211 0.97619 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
E1280DB306135621
7C6A755F31B22109
CD3A44E925507176
D32C889E13FF2927
5A30A1841EAF49A6
9408488B36963C7B
070E960D0CBA146B
04552500492C060C
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{500, 60},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007d0000?2000`400?l00000o`00003ooooooooe
ooooo`00oooooooeooooo`00oooooooeooooo`003oooool3oeIFEPCo20P80olm?Cd4og9bLP?o2@T9
1?m<C4`3og=cL`?o9RHV0_ooool4oa8B4P?o4a<C1?l51@D3oa0@40Co7alO0olV9RH4obl_;`;o820P
0oooool3ochn?PCoCDe=0oli>CT4odI6AP?o8R8R1?lX:2P3oc0`<0;oATI60oooool4oa8B4P?o5QHF
1?mUIFD3o`0000?o7alO1?n=SHd3oa0@40?o;Bd]0oooool3oa@D50Co8b<S0olD51@3ob8R8POo=CDe
1?ll?3`2odY:BP?ooooo0om;Bd/4ol?3``?oDE5A1?nCTi<3ok^k^`CoQHF50oo9bLT3odi>CP;ooooo
1?lQ8B43oe=CD`CoMgMg0om?Cdl4od10@0?o71`L1?lA4A42obHV9P?ooooo0om4A4@4odE5A@?o9RHV
1?ll?3`3obHV9PCo?Cdm0oln?Sh3o`P820;ooooo1?lU9BD3ob8R8POo0`<31?l00003o`820PCo5QHF
0_lR8R83ooooo`?o;bl_1?l`<303ob<S8`Co5aLG0olC4a<4oa8B4P?o3Ph>0ol10@42ooooo`Co8R8R
0ol71`L4o`/;2`?o1`L71?l<30`3o`0000?o1`L70olH61P3ooooo`?o51@D1?ooool3oa<C4`Co>CTi
0onPX:03obh^;PCoADE50_l];Bd3ooooo`Co61PH0olK6a/3ofATI0Co>SXj0omYJFT4ohn?S`?o?clo
0olJ6QX3ooooo`?oZZZZ0olN7Qh4ono_k`?oD51@1?lI6AT3ooooo`Co7QhN0_l^;Rh3ooooo`?o]KFe
1?lB4Q83ob4Q8@Co8b<S0olG5aL4ob4Q8@?o820P0ol@4102ooooo`Co_[jn0on0P804oc0`<0?ooooo
1?lX:2P3oh^;R`Co`/;20_lX:2PAooooo`003oooool3oeIFEPCo20P80olm?Cd4og9bLP?o2@T91?m<
C4`3og=cL`?o9RHV0_ooool4oa8B4P?o4a<C1?l51@D3oa0@40Co7alO0olV9RH4obl_;`;o820P0ooo
ool3ochn?PCoCDe=0oli>CT4odI6AP?o8R8R1?lX:2P3oc0`<0;oATI60oooool4oa8B4P?o5QHF1?mU
IFD3o`0000?o7alO1?n=SHd3oa0@40?o;Bd]0oooool3oa@D50Co8b<S0olD51@3ob8R8POo=CDe1?ll
?3`2odY:BP?ooooo0om;Bd/4ol?3``?oDE5A1?nCTi<3ok^k^`CoQHF50oo9bLT3odi>CP;ooooo1?lQ
8B43oe=CD`CoMgMg0om?Cdl4od10@0?o71`L1?lA4A42obHV9P?ooooo0om4A4@4odE5A@?o9RHV1?ll
?3`3obHV9PCo?Cdm0oln?Sh3o`P820;ooooo1?lU9BD3ob8R8POo0`<31?l00003o`820PCo5QHF0_lR
8R83ooooo`?o;bl_1?l`<303ob<S8`Co5aLG0olC4a<4oa8B4P?o3Ph>0ol10@42ooooo`Co8R8R0ol7
1`L4o`/;2`?o1`L71?l<30`3o`0000?o1`L70olH61P3ooooo`?o51@D1?ooool3oa<C4`Co>CTi0onP
X:03obh^;PCoADE50_l];Bd3ooooo`Co61PH0olK6a/3ofATI0Co>SXj0omYJFT4ohn?S`?o?clo0olJ
6QX3ooooo`?oZZZZ0olN7Qh4ono_k`?oD51@1?lI6AT3ooooo`Co7QhN0_l^;Rh3ooooo`?o]KFe1?lB
4Q83ob4Q8@Co8b<S0olG5aL4ob4Q8@?o820P0ol@4102ooooo`Co_[jn0on0P804oc0`<0?ooooo1?lX
:2P3oh^;R`Co`/;20_lX:2PAooooo`003oooool3oeIFEPCo20P80olm?Cd4og9bLP?o2@T91?m<C4`3
og=cL`?o9RHV0_ooool4oa8B4P?o4a<C1?l51@D3oa0@40Co7alO0olV9RH4obl_;`;o820P0oooool3
ochn?PCoCDe=0oli>CT4odI6AP?o8R8R1?lX:2P3oc0`<0;oATI60oooool4oa8B4P?o5QHF1?mUIFD3
o`0000?o7alO1?n=SHd3oa0@40?o;Bd]0oooool3oa@D50Co8b<S0olD51@3ob8R8POo=CDe1?ll?3`2
odY:BP?ooooo0om;Bd/4ol?3``?oDE5A1?nCTi<3ok^k^`CoQHF50oo9bLT3odi>CP;ooooo1?lQ8B43
oe=CD`CoMgMg0om?Cdl4od10@0?o71`L1?lA4A42obHV9P?ooooo0om4A4@4odE5A@?o9RHV1?ll?3`3
obHV9PCo?Cdm0oln?Sh3o`P820;ooooo1?lU9BD3ob8R8POo0`<31?l00003o`820PCo5QHF0_lR8R83
ooooo`?o;bl_1?l`<303ob<S8`Co5aLG0olC4a<4oa8B4P?o3Ph>0ol10@42ooooo`Co8R8R0ol71`L4
o`/;2`?o1`L71?l<30`3o`0000?o1`L70olH61P3ooooo`?o51@D1?ooool3oa<C4`Co>CTi0onPX:03
obh^;PCoADE50_l];Bd3ooooo`Co61PH0olK6a/3ofATI0Co>SXj0omYJFT4ohn?S`?o?clo0olJ6QX3
ooooo`?oZZZZ0olN7Qh4ono_k`?oD51@1?lI6AT3ooooo`Co7QhN0_l^;Rh3ooooo`?o]KFe1?lB4Q83
ob4Q8@Co8b<S0olG5aL4ob4Q8@?o820P0ol@4102ooooo`Co_[jn0on0P804oc0`<0?ooooo1?lX:2P3
oh^;R`Co`/;20_lX:2PAooooo`003oooool3oc0`<0Co6ATI0omIFET7o`h>3PCoKFe]0olD51@3o`00
00;ooooo1?l:2PX3o`L71`Co0`<30ol92@T4o``<30?o2@T91?l820P2oa8B4P?ooooo0olk>c/4od10
@0?o7QhN1?lK6a/3oa4A4@Co7AdM0olF5QH2obHV9P?ooooo1?lK6a/3oalO7`Co@D510olP8203odY:
BPCo8R8R0olH61P3oe]KF`?ooooo0olR8R84ob0P80?o5aLG0ola<C44ob0P80?o:b/[1?nl_;`2oeYJ
FP?ooooo0omIFET4ob0P80?o;2`/1?o1`L43oc0`<0Co>CTi0ooShn<3obl_;`;ooooo1?m>CTh3ob0P
80CoKFe]0on>SXh4o`<30`?o4Q8B1?lD51@2o`H61P?ooooo0ooCdm<4ob<S8`?o;Bd]1?nBTY83oaDE
5@Co9bLW0omFEUH3oa4A4@;ooooo1?lS8b<3oaHF5PCo3@d=0ol10@44o`l?3`?o20P81?l>3Ph2oaTI
6@?ooooo0ol;2`/4o`X:2P?o2@T91?l10@47o`T92@?o3Ph>0ol<30`2ooooo`Co;Bd]0olk>c/4ochn
?P?oCTi>1?mIFET3odi>CP?oEeMG0omIFET3ooooo`?oNg]k1?n>SXh3obLW9`Cono_k0om3@d<3oa<C
4`CoaLG50_lE5AD3ooooo`Co61PH0ol00003oeYJFPCoFEUI0om^KVh4od10@0?o5QHF0olX:2P3oooo
o`?oW9bL0olK6a/4olK6aP?oJf][1?m/K6`3ojNWY`Co;bl_0_oGemL3ooooo`?o=CDe1?lA4A43o`d=
3@Co6ATI0ol30`<4oaDE5@?o4Q8B0olG5aL2ooooo`CoWinO0ombLW84odi>CP?ob/[:1?lZ:RX3onSX
j0Co/[:b0_lM7AdAooooo`003oooool3oc0`<0Co6ATI0omIFET7o`h>3PCoKFe]0olD51@3o`0000;o
oooo1?l:2PX3o`L71`Co0`<30ol92@T4o``<30?o2@T91?l820P2oa8B4P?ooooo0olk>c/4od10@0?o
7QhN1?lK6a/3oa4A4@Co7AdM0olF5QH2obHV9P?ooooo1?lK6a/3oalO7`Co@D510olP8203odY:BPCo
8R8R0olH61P3oe]KF`?ooooo0olR8R84ob0P80?o5aLG0ola<C44ob0P80?o:b/[1?nl_;`2oeYJFP?o
oooo0omIFET4ob0P80?o;2`/1?o1`L43oc0`<0Co>CTi0ooShn<3obl_;`;ooooo1?m>CTh3ob0P80Co
KFe]0on>SXh4o`<30`?o4Q8B1?lD51@2o`H61P?ooooo0ooCdm<4ob<S8`?o;Bd]1?nBTY83oaDE5@Co
9bLW0omFEUH3oa4A4@;ooooo1?lS8b<3oaHF5PCo3@d=0ol10@44o`l?3`?o20P81?l>3Ph2oaTI6@?o
oooo0ol;2`/4o`X:2P?o2@T91?l10@47o`T92@?o3Ph>0ol<30`2ooooo`Co;Bd]0olk>c/4ochn?P?o
CTi>1?mIFET3odi>CP?oEeMG0omIFET3ooooo`?oNg]k1?n>SXh3obLW9`Cono_k0om3@d<3oa<C4`Co
aLG50_lE5AD3ooooo`Co61PH0ol00003oeYJFPCoFEUI0om^KVh4od10@0?o5QHF0olX:2P3ooooo`?o
W9bL0olK6a/4olK6aP?oJf][1?m/K6`3ojNWY`Co;bl_0_oGemL3ooooo`?o=CDe1?lA4A43o`d=3@Co
6ATI0ol30`<4oaDE5@?o4Q8B0olG5aL2ooooo`CoWinO0ombLW84odi>CP?ob/[:1?lZ:RX3onSXj0Co
/[:b0_lM7AdAooooo`003oooool3oc0`<0Co6ATI0omIFET7o`h>3PCoKFe]0olD51@3o`0000;ooooo
1?l:2PX3o`L71`Co0`<30ol92@T4o``<30?o2@T91?l820P2oa8B4P?ooooo0olk>c/4od10@0?o7QhN
1?lK6a/3oa4A4@Co7AdM0olF5QH2obHV9P?ooooo1?lK6a/3oalO7`Co@D510olP8203odY:BPCo8R8R
0olH61P3oe]KF`?ooooo0olR8R84ob0P80?o5aLG0ola<C44ob0P80?o:b/[1?nl_;`2oeYJFP?ooooo
0omIFET4ob0P80?o;2`/1?o1`L43oc0`<0Co>CTi0ooShn<3obl_;`;ooooo1?m>CTh3ob0P80CoKFe]
0on>SXh4o`<30`?o4Q8B1?lD51@2o`H61P?ooooo0ooCdm<4ob<S8`?o;Bd]1?nBTY83oaDE5@Co9bLW
0omFEUH3oa4A4@;ooooo1?lS8b<3oaHF5PCo3@d=0ol10@44o`l?3`?o20P81?l>3Ph2oaTI6@?ooooo
0ol;2`/4o`X:2P?o2@T91?l10@47o`T92@?o3Ph>0ol<30`2ooooo`Co;Bd]0olk>c/4ochn?P?oCTi>
1?mIFET3odi>CP?oEeMG0omIFET3ooooo`?oNg]k1?n>SXh3obLW9`Cono_k0om3@d<3oa<C4`CoaLG5
0_lE5AD3ooooo`Co61PH0ol00003oeYJFPCoFEUI0om^KVh4od10@0?o5QHF0olX:2P3ooooo`?oW9bL
0olK6a/4olK6aP?oJf][1?m/K6`3ojNWY`Co;bl_0_oGemL3ooooo`?o=CDe1?lA4A43o`d=3@Co6ATI
0ol30`<4oaDE5@?o4Q8B0olG5aL2ooooo`CoWinO0ombLW84odi>CP?ob/[:1?lZ:RX3onSXj0Co/[:b
0_lM7AdAooooo`003oooool3ob4Q8@Co;Bd]0om[Jf/4obl_;`?o<30`1?m0@403obPX:0?o6ATI0_oo
ool4o`410@?o2@T91?l410@3o`d=3@Co00000ol@4104o`h>3P;o3@d=0oooool3obDU9@Co92@T0ol=
3@d4o`/;2`?o0@411?l820P3o`820P;o61PH0oooool4obLW9`?oV9RH1?lV9RH3oaDE5@?oCDe=1?ll
?3`3ob8R8P?o7AdM0oooool3oadM7@Co61PH0olR8R83oaDE5@Co;2`/0olX:2P4ofa/K0;o:RXZ0ooo
ool3ob4Q8@Co6a/K0olg=cL4oa8B4P?o8b<S1?mZJVX3obLW9`?o7AdM0_ooool4ob/[:`?oGemO1?n5
QHD3oaXJ6PCo;Rh^0ol/;2`4o`0000;o6ATI0oooool3ok:b/PCo7AdM0onc/k<4oeQHF0?o2`/;1?ni
^KT3oaXJ6P?o8R8R0_ooool4oc4a<@?o6ATI1?lK6a/3ob4Q8@Co9bLW0olV9RH4obd];@;o?3`l0ooo
ool3o`D51@Co30`<0ol00007o`l?3`Co5QHF0ol?3`l3oaHF5P;ooooo1?meMGD3oibLW0CobLW90ooo
ool4ooclo0?ofm_K0ooBd]83oiFEU@?ooooo0om_Kfl4om[JfP?o7alO1?nEUID3oi:BTP?oIVIV1?mc
Lg<2oaHF5P?ooooo1?lU9BD3oaDE5@?oI6AT1?lK6a/3og5aL@CoRHV90ol`<303oa0@40?ooooo0on^
[Zh3obh^;PCoZ:RX0om<C4`4oeaLG0?oc/k>1?lU9BD2ogemO@?ooooo0on2PX84oaPH60?o:RXZ1?lT
92@3o`H61PCo9bLW0olH61P3o`h>3P;ooooo1?moOgl3ob0P80CoF5QH0onWYjL4o`0000?o^k^k1?nc
/k<2ocLg=a7ooooo000?ooooo`?o8B4Q1?l];Bd3of][J`Co;bl_0ol`<304od10@0?o:2PX0olI6AT2
ooooo`Co0@410ol92@T4o`@410?o3@d=1?l00003oa0@40Co3Ph>0_l=3@d3ooooo`?o9BDU1?lT92@3
o`d=3@Co2`/;0ol10@44o`P820?o0P820_lH61P3ooooo`Co9bLW0onHV9P4obHV9P?o5ADE0om=CDd4
oc`l?0?o8R8R0olM7Ad3ooooo`?o7AdM1?lH61P3ob8R8P?o5ADE1?l/;2`3obPX:0CoK6a/0_lZ:RX3
ooooo`?o8B4Q1?lK6a/3ocLg=`Co4Q8B0olS8b<4ofYZJP?o9bLW0olM7Ad2ooooo`Co:b/[0omOGel4
ohF5Q@?o6QXJ1?l^;Rh3ob`/;0Co00000_lI6AT3ooooo`?o/[:b1?lM7Ad3ok>c/`CoF5QH0ol;2`/4
okVi^@?o6QXJ0olR8R82ooooo`Co<C4a0olI6AT4oa/K6`?o8B4Q1?lW9bL3obHV9PCo;Bd]0_ll?3`3
ooooo`?o1@D51?l<30`3o`0000Oo3`l?1?lF5QH3o`l?3`?o5QHF0_ooool4ogEeM@?oW9bL1?o9bLT3
ooooo`Coo?cl0ooKfm/3om;BdP?oUIFE0oooool3ofm_K`Cof][J0olO7al4oiFEU@?oTY:B0omVIVH4
og=cL`;o5QHF0oooool4obDU9@?o5ADE0omTI6@4oa/K6`?oLG5a1?n9RHT3oc0`<0?o410@0oooool3
ojj^[P?o;Rh^1?nXZ:P3oda<C0CoG5aL0oo>c/h4obDU9@;oOGem0oooool3oh:2PPCo61PH0olZ:RX4
ob@T90?o1PH61?lW9bL3oaPH60?o3Ph>0_ooool4ogmoO`?o820P1?mHF5P3ojNWY`Co00000onk^k/4
ok>c/`;o=cLg4Oooool000oooooo0olQ8B44obd];@?oJf][1?l_;bl3oc0`<0Co@4100olX:2P3oaTI
6@;ooooo1?l10@43o`T92@Co10@40ol=3@d4o`0000?o410@1?l>3Ph2o`d=3@?ooooo0olU9BD4ob@T
90?o3@d=1?l;2`/3o`410@Co20P80ol20P82oaPH60?ooooo1?lW9bL3oiRHV0Co9RHV0olE5AD3ode=
C@Co?3`l0olR8R83oadM7@?ooooo0olM7Ad4oaPH60?o8R8R0olE5AD4ob`/;0?o:2PX1?m/K6`2obXZ
:P?ooooo0olQ8B44oa/K6`?o=cLg1?lB4Q83ob<S8`CoJVYZ0olW9bL3oadM7@;ooooo1?l[:b/3oemO
G`CoQHF50olJ6QX4obh^;P?o;2`/1?l00002oaTI6@?ooooo0onb/[84oadM7@?o/k>c1?mHF5P3o`/;
2`Co^KVi0olJ6QX3ob8R8P;ooooo1?la<C43oaTI6@Co6a/K0olQ8B44obLW9`?o9RHV1?l];Bd2oc`l
?0?ooooo0ol51@D4o``<30?o00001ol?3`l4oaHF5P?o3`l?0olF5QH2ooooo`CoMGEe0onLW9`4olW9
b@?ooooo1?olo?`3om_Kf`?od];B0onEUID3ooooo`?oKfm_1?oJf]X3oalO7`CoUIFE0onBTY83ofIV
IPCoLg=c0_lF5QH3ooooo`Co9BDU0olE5AD3ofATI0Co6a/K0omaLG44ohV9R@?o<30`0ol@4103oooo
o`?o[Zj^0ol^;Rh4ojRXZ0?oC4a<1?mLG5`3olk>cPCo9BDU0_mmOGd3ooooo`?oPX:21?lH61P3obXZ
:PCo92@T0ol61PH4obLW9`?o61PH0ol>3Ph2ooooo`CoOgmo0olP8204oeQHF0?oYjNW1?l00003ok^k
^`Co/k>c0_lg=cLAooooo`003oooool3ob4Q8@Co;Bd]0om[Jf/4obl_;`?o<30`1?m0@403obPX:0?o
6ATI0_ooool4o`410@?o2@T91?l410@3o`d=3@Co00000ol@4104o`h>3P;o3@d=0oooool3obDU9@Co
92@T0ol=3@d4o`/;2`?o0@411?l820P3o`820P;o61PH0oooool4obLW9`?oV9RH1?lV9RH3oaDE5@?o
CDe=1?ll?3`3ob8R8P?o7AdM0oooool3oadM7@Co61PH0olR8R83oaDE5@Co;2`/0olX:2P4ofa/K0;o
:RXZ0oooool3ob4Q8@Co6a/K0olg=cL4oa8B4P?o8b<S1?mZJVX3obLW9`?o7AdM0_ooool4ob/[:`?o
GemO1?n5QHD3oaXJ6PCo;Rh^0ol/;2`4o`0000;o6ATI0oooool3ok:b/PCo7AdM0onc/k<4oeQHF0?o
2`/;1?ni^KT3oaXJ6P?o8R8R0_ooool4oc4a<@?o6ATI1?lK6a/3ob4Q8@Co9bLW0olV9RH4obd];@;o
?3`l0oooool3o`D51@Co30`<0ol00007o`l?3`Co5QHF0ol?3`l3oaHF5P;ooooo1?meMGD3oibLW0Co
bLW90oooool4ooclo0?ofm_K0ooBd]83oiFEU@?ooooo0om_Kfl4om[JfP?o7alO1?nEUID3oi:BTP?o
IVIV1?mcLg<2oaHF5P?ooooo1?lU9BD3oaDE5@?oI6AT1?lK6a/3og5aL@CoRHV90ol`<303oa0@40?o
oooo0on^[Zh3obh^;PCoZ:RX0om<C4`4oeaLG0?oc/k>1?lU9BD2ogemO@?ooooo0on2PX84oaPH60?o
:RXZ1?lT92@3o`H61PCo9bLW0olH61P3o`h>3P;ooooo1?moOgl3ob0P80CoF5QH0onWYjL4o`0000?o
^k^k1?nc/k<2ocLg=a7ooooo000?ooooo`?o9BDU1?nUYJD3odU9B@Co10@40om^KVh4odQ8B0?o410@
0olM7Ad2ooooo`Co10@40ol61PH4o`0000?o0`<31?l61PH3o`0000Co5ADE0_lA4A43ooooo`?o<c<c
1?lW9bL3oa/K6`Co00000ol30`<4o``<30?o00000_lE5AD3ooooo`CoAdM70om@D504odm?C`?oPh>3
0oli>CT4oaPH60?o?Shn0olI6AT3ooooo`?o00001?l92@T3o`d=3@?o30`<1?lM7Ad3obDU9@Co7AdM
0_lN7Qh3ooooo`?o7AdM1?l<30`3od51@@Co9BDU0ol00004oc0`<0?o@4100olH61P2ooooo`Co@T92
0olc<c<4ogalO0?oNg]k1?lH61P3obLW9`Co820P0_lS8b<3ooooo`?oZZZZ1?lL71`3okNg]`CoG5aL
0om?Cdl4oi^KV`?o?Shn0on9RHT2ooooo`CoI6AT0om<C4`4ofQXJ0?oHF5Q1?mCDe<3ofa/K0CoHF5Q
0_mWIfL3ooooo`?o2`/;1?l92@T3o`H61PCo2`/;0ol61PH4oa8B4P?o5QHF0ol:2PX2ooooo`CoHF5Q
0omSHf<4ogQhN0?oV9RH1?oAdM43omcLg0?oc/k>0onf][H3ooooo`?oKfm_1?nb/[83o`l?3`CooOgm
0om?Cdl3odQ8B0Coo?cl0_lE5AD3ooooo`Co5ADE0olB4Q83odm?C`Co=SHf0omOGel4oeaLG0?o:BTY
0olQ8B43ooooo`?o7QhN0olD51@4omSHf0?o3`l?1?mHF5P3ol[:bPCo5QHF0_n][Jd3ooooo`?o?clo
1?lL71`3oaHF5PCo51@D0ol71`L4oaTI6@?o00000ol>3Ph2ooooo`CoF5QH0ol;2`/4odm?C`?oFEUI
1?l:2PX3ojBTY0CoCTi>0_l`<30Aooooo`003oooool3obDU9@CoYJFU0om9BDT4o`@410?oKVi^1?m8
B4P3oa0@40?o7AdM0_ooool4o`@410?o1PH61?l00003o`<30`Co1PH60ol00004oaDE5@;o4A4A0ooo
ool3oc<c<`Co9bLW0olK6a/4o`0000?o0`<31?l<30`3o`0000;o5ADE0oooool4odM7A`?oD51@1?m?
Cdl3oh>3P`?o>CTi1?lH61P3ochn?P?o6ATI0oooool3o`0000Co2@T90ol=3@d3o``<30Co7AdM0olU
9BD4oadM7@;o7QhN0oooool3oadM7@Co30`<0om1@D44obDU9@?o00001?l`<303od10@0?o61PH0_oo
ool4od92@P?o<c<c1?mlO7`3og]kN`Co61PH0olW9bL4ob0P80;o8b<S0oooool3ojZZZPCo71`L0ong
]kL4oeaLG0?oCdm?1?nKVi/3ochn?P?oRHV90_ooool4ofATI0?oC4a<1?mXJ6P3of5QH@CoDe=C0om/
K6`4of5QH@;oIfMW0oooool3o`/;2`Co2@T90ol61PH4o`/;2`?o1PH61?lB4Q83oaHF5P?o2PX:0_oo
ool4of5QH@?oHf=S1?mhN7P3oiRHV0CodM7A0ooLg=`3olk>cP?o][Jf0oooool3ofm_K`Co/[:b0ol?
3`l4oogmo@?oCdm?0om8B4P4ooclo0;o5ADE0oooool4oaDE5@?o4Q8B0om?Cdl4ocHf=P?oGemO1?mL
G5`3obTY:@?o8B4Q0oooool3oahN7P?o51@D1?oHf=P3o`l?3`CoF5QH0oo:b/X4oaHF5P;o[Jf]0ooo
ool3oclo?`Co71`L0olF5QH4oa@D50?o1`L71?lI6AT3o`0000?o3Ph>0_ooool4oeQHF0?o2`/;1?m?
Cdl3oeUIF@Co2PX:0onTY:@4odi>CP;o<30`4Oooool000oooooo0olU9BD4ojFUY@?oBDU91?l410@3
ofi^KPCoB4Q80ol@4103oadM7@;ooooo1?l410@3o`H61PCo00000ol30`<4o`H61P?o00001?lE5AD2
oa4A4@?ooooo0olc<c<4obLW9`?o6a/K1?l00003o`<30`Co30`<0ol00002oaDE5@?ooooo1?m7AdL3
oe1@D0CoCdm?0on3Ph<3ocTi>@Co61PH0oln?Sh3oaTI6@?ooooo0ol00004o`T92@?o3@d=0ol<30`4
oadM7@?o9BDU1?lM7Ad2oahN7P?ooooo0olM7Ad4o``<30?o@D511?lU9BD3o`0000Co<30`0om0@403
oaPH60;ooooo1?m2@T83oc<c<`CoO7al0omkNg/4oaPH60?o9bLW1?lP8202ob<S8`?ooooo0onZZZX4
oa`L70?o]kNg1?mLG5`3odm?C`CoVi^K0oln?Sh3ohV9R@;ooooo1?mTI6@3oda<C0CoJ6QX0omQHF44
oe=CD`?oK6a/1?mQHF42ofMWI`?ooooo0ol;2`/4o`T92@?o1PH61?l;2`/3o`H61PCo4Q8B0olF5QH3
o`X:2P;ooooo1?mQHF43of=SH`CoN7Qh0onHV9P4om7Ad@?og=cL0oo>c/h3okJf]P?ooooo0om_Kfl4
ok:b/P?o3`l?1?omoOd3odm?C`?oB4Q81?olo?`2oaDE5@?ooooo1?lE5AD3oa8B4P?oCdm?1?lf=SH3
oemOG`CoG5aL0olY:BT3ob4Q8@?ooooo0olN7Qh3oa@D50Cof=SH0ol?3`l4oeQHF0?ob/[:1?lF5QH2
ojf][@?ooooo0olo?cl4oa`L70?o5QHF1?lD51@3o`L71`Co6ATI0ol00003o`h>3P;ooooo1?mHF5P3
o`/;2`CoCdm?0omIFET4o`X:2P?oY:BT1?m>CTh2oc0`<17ooooo000?ooooo`?o7QhN1?mYJFT3ohf=
S@Co>CTi0om^KVh4oc/k>`?oCdm?0ol30`<2ooooo`Co1PH60olA4A44o`<30`?o10@41?l@4103o`d=
3@Co30`<0_lC4a<3ooooo`?o<c<c1?ll?3`3oc@d=0Co92@T0ol/;2`4obLW9`?o8b<S0_lU9BD3oooo
o`Co<30`0oooool4oalO7`?o?3`l0onCTi<4ocDe=@?o>SXj0oli>CT3ooooo`?o5aLG1?lD51@3o`@4
10?o20P81?lb<S83oahN7PCo?Shn0_l/;2`3ooooo`?oKfm_1?mPH603oe1@D0Co:b/[1olX:2P3o`H6
1P?o5ADE0_ooool4o`X:2P?oIfMW1?mnOWh3oaLG5`Co=3@d0olG5aL4o`H61P;o0`<30oooool3olo?
c`Co?Shn0onj^[X4oeADE0?oN7Qh1?oNg]h3obh^;P?oJVYZ0_ooool4og5aL@?oP8201?n@T903ol;2
`PCoXZ:R0oo@d=04oj6QX@;oWinO0oooool3o`d=3@Co4A4A0olB4Q87o`P820Co10@40ol820P3o`X:
2P;ooooo1?mkNg/3ofm_K`CoLG5a0omXJ6P4of1PH0?oJ6QX0omeMGD3ofa/K0?ooooo0on][Jd4odQ8
B0?o61PH1?oWinL3obh^;P?oOGem1?nl_;`2oa<C4`?ooooo1?lD51@3oa0@40?oVYZJ1?mZJVX3oeIF
EPCoYZJV0om0@403oa<C4`?ooooo0om2@T83oeYJFPCoBTY:0olW9bL4ojBTY0?oATI61?l=3@d2oj>S
X`Kooooo1?lN7Qh3obd];@CoB4Q80olK6a/4obd];@?o8b<S0ol<30`2ooooo`Co<30`0ol>3Ph4odi>
CP?o@4101?lE5AD3ogmoO`Co?3`l0_l/;2`Aooooo`003oooool3oahN7PCoJFUY0on=SHd4ocTi>@?o
KVi^1?lk>c/3odm?C`?o0`<30_ooool4o`H61P?o4A4A1?l30`<3o`@410Co410@0ol=3@d4o``<30;o
4a<C0oooool3oc<c<`Co?3`l0old=3@4ob@T90?o;2`/1?lW9bL3ob<S8`;o9BDU0oooool4oc0`<0?o
oooo1?lO7al3oc`l?0?oTi>C1?le=CD3ocXj>P?o>CTi0oooool3oaLG5`Co51@D0ol410@3o`P820Co
<S8b0olN7Qh4ochn?P;o;2`/0oooool3ofm_K`CoH61P0om@D504ob/[:`Oo:2PX0ol61PH3oaDE5@;o
oooo1?l:2PX3ofMWI`CoOWin0olG5aL4oc@d=0?o5aLG1?l61PH2o`<30`?ooooo0oo?cll4ochn?P?o
^[Zj1?mDE5@3ogQhN0Cog]kN0ol^;Rh3ofYZJP;ooooo1?maLG43oh20P0CoT92@0oo2`/84oj:RXP?o
d=3@1?nQXJ42oinOW`?ooooo0ol=3@d4oa4A4@?o4Q8B1ol820P4o`@410?o20P80ol:2PX2ooooo`Co
Ng]k0om_Kfl4og5aL@?oJ6QX1?mPH603ofQXJ0?oMGEe0om/K6`3ooooo`?o[Jf]1?m8B4P3oaPH60Co
inOW0ol^;Rh3ogemO@Co_;bl0_lC4a<3ooooo`Co51@D0ol@4103oiZJVPCoJVYZ0omFEUH4ojJVYP?o
@4100olC4a<3ooooo`?o@T920omJFUX4odY:BP?o9bLW1?nTY:@3odI6APCo3@d=0_nSXj<6ooooo`Co
7QhN0ol];Bd4odQ8B0?o6a/K1?l];Bd3ob<S8`?o30`<0_ooool4oc0`<0?o3Ph>1?m>CTh3od10@0Co
5ADE0omoOgl4oc`l?0;o;2`/4Oooool000oooooo0olN7Qh4ofUYJ@?oSHf=1?li>CT3ofi^KPCo>c/k
0om?Cdl3o`<30`;ooooo1?l61PH3oa4A4@Co0`<30ol410@4oa0@40?o3@d=1?l<30`2oa<C4`?ooooo
0olc<c<4oc`l?0?o=3@d1?lT92@3ob`/;0Co9bLW0olS8b<2obDU9@?ooooo1?l`<303ooooo`Co7alO
0oll?3`3oi>CT`Co=CDe0olj>SX3ocTi>@?ooooo0olG5aL4oa@D50?o10@40ol820P4oc8b<P?o7QhN
1?ln?Sh2ob`/;0?ooooo0om_Kfl4of1PH0?oD51@1?l[:b/7obPX:0?o1PH60olE5AD2ooooo`Co2PX:
0omWIfL4oginOP?o5aLG1?ld=3@3oaLG5`Co1PH60_l30`<3ooooo`?oclo?1?ln?Sh3okZj^PCoE5AD
0omhN7P4omkNgP?o;Rh^0omZJVX2ooooo`CoLG5a0on0P804oi2@T0?o`/;21?nRXZ83om3@d0CoXJ6Q
0_nOWil3ooooo`?o3@d=1?lA4A43oa8B4POo20P81?l410@3o`P820?o2PX:0_ooool4og]kN`?oKfm_
1?maLG43ofQXJ0CoH61P0omXJ6P3ogEeM@?oK6a/0oooool3ojf][@CoB4Q80olH61P4onOWi`?o;Rh^
0ommOGd4okbl_0;o4a<C0oooool4oa@D50?o410@0onJVYX4ofYZJP?oEUIF1?nVYZH3od10@0?o4a<C
0oooool3od92@P?oFUYJ1?m:BTX3obLW9`CoY:BT0om6ATH4o`d=3@;oXj>S1_ooool4oahN7P?o;Bd]
1?m8B4P3oa/K6`Co;Bd]0olS8b<3o``<30;ooooo1?l`<303o`h>3PCoCTi>0om0@404oaDE5@?oOgmo
1?ll?3`2ob`/;17ooooo000?ooooo`?o5QHF1?ooool3oeQHF0Co8B4Q0ooTi>@4oa`L70?o:2PX0om1
@D42ooooo`Co92@T0olV9RH4oaXJ6P?o10@41?l:2PX3oa@D50Co3Ph>0_lF5QH3ooooo`?oJFUY1?mH
F5P3ode=C@Oo?Cdm1?lf=SH5oc@d=0?ooooo1?mlO7`3oj:RXPCo:b/[0oo_knl3od92@PCo7QhN0oo6
a/H3ob4Q8@?ooooo0olN7Qh4oa@D50?o8B4Q1ol/;2`3oc<c<`Cooooo0_mFEUH3ooooo`?oDE5A1?mU
IFD3ono_k`CoC4a<0omNGUh4oi2@T0?o>3Ph0ol;2`/2ooooo`CoKVi^0omIFET4olK6aP?oXJ6Q1?lI
6AT3obXZ:PCo8b<S0_l`<303ooooo`?oFUYJ1?lO7al3ooooo`Co7alO0omQHF44omGEe@?o5QHF0onS
Xj<2ooooo`CoRXZ:0onOWil4omOGe`?ooooo1?oemOD3oo;blPCojNWY0_o3`l<3ooooo`Oo2PX:0olB
4Q84o`820P?o51@D1?l71`L3o`@410?o00000_ooool4og=cL`?oCdm?1?m<C4`3obl_;`Oo:b/[0ol_
;bl3ocHf=P?ooooo0om4A4@4ofi^KP?o@4101?m]KFd3ocXj>P?oWinO1?n4Q8@2oaTI6@?ooooo1?lL
71`3odU9B@?oJf][1?m0@403ooooo`Co`l?30olQ8B43oc@d=0?ooooo0ol=3@d3obl_;`CoMgMg0ol<
30`4od92@P?oMgMg1?m>CTh2od92@P?ooooo0om2@T84oa`L70?o2@T91?lH61P3o`X:2PCo5aLG1_l=
3@d2ooooo`Co5QHF0om0@404ocPh>0?o4Q8B1?lY:BT3odM7A`Co4Q8B0_lT92@Aooooo`003oooool3
oaHF5PCooooo0omHF5P4ob4Q8@?oi>CT1?lL71`3obPX:0?o@D510_ooool4ob@T90?o9RHV1?lJ6QX3
o`@410Co2PX:0olD51@4o`h>3P;o5QHF0oooool3ofUYJ@CoF5QH0om=CDd7ocdm?@Co=SHf1Old=3@3
ooooo`CoO7al0onRXZ84ob/[:`?okno_0om2@T84oahN7P?oa/K60olQ8B43ooooo`?o7QhN1?lD51@3
ob4Q8@Oo;2`/0olc<c<4ooooo`;oEUIF0oooool3oe5AD@CoIFEU0oo_knl4oda<C0?oGUiN1?n@T903
ocPh>0?o2`/;0_ooool4ofi^KP?oFEUI1?o6a/H3oj6QX@Co6ATI0olZ:RX4ob<S8`;o<30`0oooool3
oeYJFPCo7alO0oooool4oalO7`?oHF5Q1?oEeMD3oaHF5P?oXj>S0_ooool4ohZ:RP?oWinO1?oGemL3
ooooo`ComOGe0oobl_84onWYj@;o`l?30oooool7o`X:2P?o4Q8B1?l20P83oa@D50Co1`L70ol410@3
o`0000;ooooo1?mcLg<3odm?C`CoC4a<0ol_;bl7ob/[:`?o;bl_0olf=SH3ooooo`?oA4A41?m^KVh3
od10@0CoKFe]0olj>SX3oinOW`CoQ8B40_lI6AT3ooooo`Co71`L0om9BDT3of][J`Co@4100oooool4
ol?3``?o8B4Q0old=3@3ooooo`?o3@d=0ol_;bl4ogMgM`?o30`<1?m2@T83ogMgM`CoCTi>0_m2@T83
ooooo`?o@T921?lL71`3o`T92@Co61PH0ol:2PX4oaLG5`Ko3@d=0_ooool4oaHF5P?o@4101?lh>3P3
oa8B4PCo:BTY0om7AdL4oa8B4P;o92@T4Oooool000oooooo0olF5QH4ooooo`?oF5QH1?lQ8B43onCT
i0Co71`L0olX:2P3od51@@;ooooo1?lT92@3obHV9PCo6QXJ0ol410@4o`X:2P?o51@D1?l>3Ph2oaHF
5P?ooooo0omYJFT4oeQHF0?oCDe=1olm?Cd4ocHf=PGo=3@d0oooool4ogalO0?oXZ:R1?l[:b/3ono_
k`?o@T921?lN7Qh3olK6aP?o8B4Q0oooool3oahN7PCo51@D0olQ8B47ob`/;0?o<c<c1?ooool2oeIF
EP?ooooo0omADE44ofEUI@?okno_1?m<C4`3oeiNGPCoT92@0olh>3P3o`/;2`;ooooo1?m^KVh3oeUI
F@Coa/K60onQXJ44oaTI6@?o:RXZ1?lS8b<2oc0`<0?ooooo0omJFUX4oalO7`?ooooo1?lO7al3of5Q
H@CoeMGE0olF5QH3oj>SX`;ooooo1?n:RXX3oinOW`CoemOG0oooool4ooGem@?ol_;b1?oYjNT2ol?3
``?ooooo1ol:2PX3oa8B4PCo0P820olD51@4o`L71`?o10@40ol00002ooooo`CoLg=c0om?Cdl4oda<
C0?o;bl_1ol[:b/3obl_;`?o=SHf0oooool3odA4A0CoKVi^0om0@404ofe]K@?o>SXj0onOWil4ohB4
Q0;o6ATI0oooool4oa`L70?oBDU90om[Jf/4od10@0?ooooo1?o3`l<3ob4Q8@?o=3@d0oooool3o`d=
3@?o;bl_1?mgMgL3o``<30Co@T920omgMgL4odi>CP;o@T920oooool3od92@PCo71`L0ol92@T4oaPH
60?o2PX:1?lG5aL6o`d=3@;ooooo1?lF5QH3od10@0Co>3Ph0olB4Q84obTY:@?oAdM71?lB4Q82ob@T
917ooooo000?ooooo`?o92@T1?oBd]83oeeMG@CoMGEe0onLW9`4oaDE5@?o]KFe0olH61P2ooooo`Co
8B4Q0olI6AT4oa/K6`?o5ADE1?lH61P3oa0@40Co2`/;0_lI6AT3ooooo`?oBDU91?meMGD3oe9BDPCo
C4a<0omGEeL4oeaLG0?oHV9R0_mGEeL3ooooo`CoIFEU0onEUID4ob8R8P?oRh^;0omfMWH4ofATI0?o
KVi^0olK6a/3ooooo`?o;Bd]1?lV9RH3oalO7`?o4A4A1?lb<S83odM7A`CoP8200_lh>3P3ooooo`?o
Hf=S1?omoOd3oiNGU`CoDe=C0oooool4odY:BP?o<c<c0omYJFT2ooooo`Co;Bd]0om9BDT4oi:BTP?o
6ATI1?lY:BT3ob8R8PCo4A4A0_lI6AT3ooooo`?oCdm?1?lo?cl3oi:BTPCo;Bd]0onSXj<4ogYjNP?o
61PH0on/[:`2ooooo`CoQ8B40on;Rh/4ok:b/P?o[Jf]1?nEUID3oi^KV`CoWinO0_n9RHT3ooooo`?o
0`<31?lG5aL3oaHF5PCo:2PX0ol?3`l4oaTI6@?o3`l?0ol92@T2ooooo`Co>3Ph0olj>SX4oa0@40?o
7alO1?l92@T3o`<30`?o20P80olG5aL3ooooo`?oP8201?l00003oa`L70Co/K6a0ol820P3oeMGE`Co
[jn_0_la<C43ooooo`Co9BDU0olD51@3ogUiN@CoP8200on8R8P4oh:2PPKo<S8b0oooool3obd];@?o
Bd];1?lS8b<3oadM7@CoO7al0ol00004oaHF5P;oIFEU0oooool3oi6AT@Co>CTi0ol[:b/4oc`l?0?o
3`l?1?lO7al3oa`L70?o1PH60_ooool4oaHF5P?oGUiN1?lo?cl3oaPH60Co;bl_0om0@404oaPH60;o
2PX:4Oooool000oooooo0olT92@4om;BdP?oGEeM1?meMGD3oibLW0Co5ADE0one]KD3oaPH60;ooooo
1?lQ8B43oaTI6@Co6a/K0olE5AD4oaPH60?o410@1?l;2`/2oaTI6@?ooooo0om9BDT4ogEeM@?oDU9B
1?m<C4`3oeMGE`CoG5aL0omRHV82oeMGE`?ooooo1?mUIFD3oiFEU@Co8R8R0on;Rh/3ogIfMPCoI6AT
0om^KVh3oa/K6`?ooooo0ol];Bd4obHV9P?o7alO0olA4A44oc8b<P?oAdM71?n0P802ocPh>0?ooooo
0omSHf<4oogmo@?oUiNG1?mCDe<3ooooo`CoBTY:0olc<c<3ofUYJ@;ooooo1?l];Bd3odU9B@CoTY:B
0olI6AT4obTY:@?o8R8R1?lA4A42oaTI6@?ooooo0om?Cdl4oclo?`?oTY:B1?l];Bd3oj>SX`CoNWYj
0olH61P3ojb/[0;ooooo1?n4Q8@3oh^;R`Co/[:b0on][Jd4oiFEU@?oVi^K1?nOWil2ohV9R@?ooooo
0ol30`<4oaLG5`?o5QHF1?lX:2P3o`l?3`Co6ATI0ol?3`l3o`T92@;ooooo1?lh>3P3ocXj>PCo410@
0olO7al4o`T92@?o0`<30ol820P3oaLG5`?ooooo0on0P804o`0000?o71`L1?na/K43o`P820?oEeMG
1?n_[jl2oc4a<@?ooooo1?lU9BD3oa@D50?oNGUi1?n0P803ohR8R0CoPX:21_lb<S83ooooo`?o;Bd]
0om;Bd/4ob<S8`?o7AdM1?mlO7`3o`0000Co5QHF0_mUIFD3ooooo`?oTI6A1?li>CT3ob/[:`Co?3`l
0ol?3`l4oalO7`?o71`L0ol61PH2ooooo`Co5QHF0omNGUh4oclo?`?o61PH1?l_;bl3od10@0Co61PH
0_l:2PXAooooo`003oooool3ob@T90Cod];B0omMGEd4ogEeM@?oW9bL1?lE5AD3okFe]@?o61PH0_oo
ool4ob4Q8@?o6ATI1?lK6a/3oaDE5@Co61PH0ol@4104o`/;2`;o6ATI0oooool3odU9B@CoMGEe0omB
DU84oda<C0?oEeMG1?mLG5`3of9RHP;oEeMG0oooool4ofEUI@?oUIFE1?lR8R83oh^;R`?oMWIf1?mT
I6@3ofi^KP?o6a/K0oooool3obd];@Co9RHV0olO7al3oa4A4@Co<S8b0om7AdL4oh20P0;o>3Ph0ooo
ool3of=SH`CooOgm0onGUiL4oe=CD`?ooooo1?m:BTX3oc<c<`?oJFUY0_ooool4obd];@?oBDU91?nB
TY83oaTI6@Co:BTY0olR8R84oa4A4@;o6ATI0oooool3odm?C`Co?clo0onBTY84obd];@?oXj>S1?mj
NWX3oaPH60?o[:b/0_ooool4ohB4Q0?oRh^;1?nb/[83ojf][@CoUIFE0onKVi/4oinOW`;oRHV90ooo
ool3o`<30`Co5aLG0olF5QH4obPX:0?o3`l?1?lI6AT3o`l?3`?o2@T90_ooool4ocPh>0?o>SXj1?l@
4103oalO7`Co2@T90ol30`<3o`P820?o5aLG0oooool3oh20P0Co00000olL71`4ok6a/@?o20P80omG
EeL4ojn_[`;o<C4a0oooool4obDU9@?o51@D0omiNGT4oh20P0?oR8R81?n2PX86oc8b<P?ooooo0ol]
;Bd3od];B`Co8b<S0olM7Ad4ogalO0?o00001?lF5QH2ofEUI@?ooooo0onATI44ocTi>@?o:b/[1?ll
?3`3o`l?3`Co7alO0olL71`3o`H61P;ooooo1?lF5QH3oeiNGPCo?clo0olH61P4obl_;`?o@4101?lH
61P2o`X:2Q7ooooo000?ooooo`?o=3@d1?mWIfL3o`h>3PCo9BDU0om0@404ocLg=`?o>SXj0olC4a<2
ooooo`Co<S8b0olf=SH4odI6AP?o>3Ph1?l[:b/3ofEUI@Cooooo0_nk^k/3ooooo`?olO7a1?ooool3
okRh^0Coe=CD0on][Jd4ohf=S@?oQHF50_n_[jl3ooooo`CoGemO0ol410@4oc`l?0?oS8b<0ol>3Ph4
ocDe=@?oIFEU0om4A4@3ooooo`?o92@T1?lM7Ad3oaLG5`?o6a/K1?l`<303oc/k>`Co=3@d0_m4A4@3
ooooo`?oJFUY1?m;Bd/3oeQHF0CoQ8B40omZJVX4of9RHP?oYJFU0olY:BT2ooooo`CoEEEE0omkNg/4
ol71`@?ooooo1?m2@T83oeYJFPCo?Shn0_m4A4@3ooooo`?o3Ph>1?l=3@d3oaTI6@Co:RXZ0ol_;bl4
o`0000?oA4A40ol[:b/2ooooo`Cobl_;0oo8b<P4oinOW`?o^k^k1?nUYJD3olO7a`Co[Zj^0_nQXJ43
ooooo`?og=cL1?ooool3oginOPCoA4A40om8B4P4ocXj>PKo=SHf0_ooool4ojf][@?oO7al1?miNGT3
ofi^KPCoLg=c0omPH603oeEEE@?oNWYj0oooool3oc8b<PCo4A4A0om>CTh4odA4A0?o1`L70olS8b<4
obLW9`;o820P0oooool4oa8B4P?o4a<C0omDE5@4oc/k>`?oAdM71?m?Cdl3oalO7`?o;Bd]0oooool3
oh>3P`?o3@d=1?l?3`l3of][J`Co;bl_0olG5aL4oeYJFP;o<S8b0oooool3ok:b/PCo>c/k0olZ:RX4
obh^;P?o92@T1?lF5QH3ob0P80?o9bLW0_ooool4ofATI0?oKfm_1?lM7Ad3ogUiN@Co?Cdm0olh>3P4
odm?C`;o51@D4Oooool000oooooo0old=3@4ofMWI`?o3Ph>1?lU9BD3od10@0Co=cLg0olj>SX3oa<C
4`;ooooo1?lb<S83ocHf=PCoATI60olh>3P4ob/[:`?oIFEU1?ooool2ok^k^`?ooooo0ooalO44oooo
o`?o^;Rh1?oDe=@3ojf][@CoSHf=0on5QHD2ojn_[`?ooooo1?mOGel3o`@410Co?3`l0on<S8`3o`h>
3PCo=CDe0omUIFD3odA4A0?ooooo0olT92@4oadM7@?o5aLG0olK6a/4oc0`<0?o>c/k1?ld=3@2odA4
A0?ooooo0omYJFT4od];B`?oF5QH1?n4Q8@3ofYZJPCoHV9R0onUYJD3obTY:@;ooooo1?mEEED3og]k
N`Co`L710oooool4od92@P?oFUYJ1?ln?Sh2odA4A0?ooooo0ol>3Ph4o`d=3@?o6ATI1?lZ:RX3obl_
;`Co00000om4A4@3ob/[:`;ooooo1?o;bl/3olS8b0CoWinO0onk^k/4ojFUY@?oalO71?n^[Zh2oj6Q
X@?ooooo0ooLg=`4ooooo`?oOWin1?m4A4@3odQ8B0Co>SXj1_lf=SH2ooooo`Co[Jf]0omlO7`4ogUi
N@?oKVi^1?mcLg<3of1PH0?oEEEE0omjNWX3ooooo`?o<S8b1?lA4A43odi>CPCoA4A40ol71`L3ob<S
8`Co9bLW0_lP8203ooooo`Co4Q8B0olC4a<3oeADE0Co>c/k0om7AdL4odm?C`?o7alO0ol];Bd3oooo
o`?oPh>30ol=3@d4o`l?3`?oJf][1?l_;bl3oaLG5`CoFUYJ0_lb<S83ooooo`?o/[:b1?lk>c/3obXZ
:PCo;Rh^0olT92@4oaHF5P?o820P0olW9bL2ooooo`CoI6AT0om_Kfl4oadM7@?oNGUi1?lm?Cd3ocPh
>0CoCdm?0_lD51@Aooooo`003oooool3oc@d=0CoIfMW0ol>3Ph4obDU9@?o@4101?lg=cL3ocXj>P?o
4a<C0_ooool4oc8b<P?o=SHf1?m6ATH3ocPh>0Co:b/[0omUIFD4ooooo`;o^k^k0oooool3oo7al@Co
oooo0onh^;P4omCDe0?o[Jf]1?n=SHd3ohF5Q@;o[jn_0oooool4oemOG`?o10@41?ll?3`3ohb<S0?o
3Ph>1?le=CD3ofEUI@?oA4A40oooool3ob@T90Co7AdM0olG5aL3oa/K6`Co<30`0olk>c/4oc@d=0;o
A4A40oooool3ofUYJ@CoBd];0omHF5P4ohB4Q0?oJVYZ1?mRHV83ojFUY@?o:BTY0_ooool4oeEEE@?o
Ng]k1?o1`L43ooooo`Co@T920omJFUX4ochn?P;oA4A40oooool3o`h>3PCo3@d=0olI6AT4obXZ:P?o
;bl_1?l00003odA4A0?o:b/[0_ooool4ol_;b`?ob<S81?nOWil3ok^k^`CoYJFU0oo7alL4ojj^[P;o
XJ6Q0oooool3omcLg0Cooooo0omnOWh4odA4A0?oB4Q81?lj>SX6ocHf=P;ooooo1?n][Jd3ogalO0Co
NGUi0om^KVh4og=cL`?oH61P0omEEED3ogYjNP?ooooo0olb<S84oa4A4@?oCTi>1?m4A4@3o`L71`?o
8b<S1?lW9bL2ob0P80?ooooo1?lB4Q83oa<C4`?oE5AD1?lk>c/3odM7A`CoCdm?0olO7al3obd];@?o
oooo0on3Ph<3o`d=3@Co3`l?0om[Jf/4obl_;`?o5aLG1?mJFUX2oc8b<P?ooooo0onb/[84oc/k>`?o
:RXZ1?l^;Rh3ob@T90Co5QHF0olP8203obLW9`;ooooo1?mTI6@3ofm_K`Co7AdM0omiNGT4ocdm?@?o
>3Ph1?m?Cdl2oa@D517ooooo000?ooooo`?o=3@d1?mWIfL3o`h>3PCo9BDU0om0@404ocLg=`?o>SXj
0olC4a<2ooooo`Co<S8b0olf=SH4odI6AP?o>3Ph1?l[:b/3ofEUI@Cooooo0_nk^k/3ooooo`?olO7a
1?ooool3okRh^0Coe=CD0on][Jd4ohf=S@?oQHF50_n_[jl3ooooo`CoGemO0ol410@4oc`l?0?oS8b<
0ol>3Ph4ocDe=@?oIFEU0om4A4@3ooooo`?o92@T1?lM7Ad3oaLG5`?o6a/K1?l`<303oc/k>`Co=3@d
0_m4A4@3ooooo`?oJFUY1?m;Bd/3oeQHF0CoQ8B40omZJVX4of9RHP?oYJFU0olY:BT2ooooo`CoEEEE
0omkNg/4ol71`@?ooooo1?m2@T83oeYJFPCo?Shn0_m4A4@3ooooo`?o3Ph>1?l=3@d3oaTI6@Co:RXZ
0ol_;bl4o`0000?oA4A40ol[:b/2ooooo`Cobl_;0oo8b<P4oinOW`?o^k^k1?nUYJD3olO7a`Co[Zj^
0_nQXJ43ooooo`?og=cL1?ooool3oginOPCoA4A40om8B4P4ocXj>PKo=SHf0_ooool4ojf][@?oO7al
1?miNGT3ofi^KPCoLg=c0omPH603oeEEE@?oNWYj0oooool3oc8b<PCo4A4A0om>CTh4odA4A0?o1`L7
0olS8b<4obLW9`;o820P0oooool4oa8B4P?o4a<C0omDE5@4oc/k>`?oAdM71?m?Cdl3oalO7`?o;Bd]
0oooool3oh>3P`?o3@d=1?l?3`l3of][J`Co;bl_0olG5aL4oeYJFP;o<S8b0oooool3ok:b/PCo>c/k
0olZ:RX4obh^;P?o92@T1?lF5QH3ob0P80?o9bLW0_ooool4ofATI0?oKfm_1?lM7Ad3ogUiN@Co?Cdm
0olh>3P4odm?C`;o51@D4Oooool00?oooooomOooool00?oooooomOooool00?oooooomOooool000oo
oooo0omgMgL4og=cL`?oIFEU1?l`<303ogAdM0Co?Shn0ol92@T3oc/k>`;ooooo1?l:2PX3oaHF5PCo
92@T0olN7Qh4obd];@?o<30`1?lR8R82o`X:2P?ooooo0omaLG44on[ZjP?oS8b<1?mADE43oclo?`Co
71`L0ol<30`2oa<C4`?ooooo1?lD51@3ocdm?@CoQ8B40olR8R83od];B`Co>c/k0olW9bL3oaDE5@?o
oooo0olO7al4ob4Q8@?o9RHV0olN7Qh4oclo?`?o9BDU1?miNGT2oc/k>`?ooooo0olm?Cd4odI6AP?o
Fe]K1?mHF5P3oeMGE`CoIFEU0omXJ6P3oeADE0;ooooo1?lO7al3ohn?S`Co;bl_0ol`<304ofm_K`?o
QHF51?nKVi/2oeUIF@?ooooo0olT92@4oa/K6`?o=cLg1?mADE43ob<S8`Co:b/[0on7QhL3obTY:@;o
oooo1?lI6AT3oeaLG0Co5QHF0olH61P4oiFEU@?o:b/[1?lA4A42ohZ:RP?ooooo0olS8b<4oa/K6`?o
<C4a1?lf=SH3ob4Q8@Co4A4A0ol?3`l3oaPH60;ooooo1?lc<c<3ocXj>PCo9RHV0old=3@4obh^;P?o
;bl_0olb<S83obd];@?ooooo0olj>SX4omkNgP?oO7al1?n:RXX3ooGem@?o6QXJ1?oNg]h2ob`/;0?o
oooo1?mJFUX3obXZ:P?oBTY:1?nFUYH3ofe]K@CoKfm_0om?Cdl3ob/[:`?ooooo0on9RHT3obHV9PCo
g]kN0omMGEd4ohn?S`?ogmoO1?lK6a/5ooooo`?o<S8b1?lG5aL3oaXJ6PCoGUiN0ol_;bl4oe=CD`?o
B4Q80omDE5@2ooooo`CohN7Q0olX:2P4o`d=3@?o/k>c1?l61PH3oa<C4`CoEUIF0_lQ8B4Aooooo`00
3oooool3ogMgM`CoLg=c0omUIFD4oc0`<0?oM7Ad1?ln?Sh3o`T92@?o>c/k0_ooool4o`X:2P?o5QHF
1?lT92@3oahN7PCo;Bd]0ol`<304ob8R8P;o2PX:0oooool3og5aL@Coj^[Z0on<S8`4oe5AD@?o?clo
1?lL71`3o``<30;o4a<C0oooool4oa@D50?o?Cdm1?n4Q8@3ob8R8P?oBd];1?lk>c/3obLW9`?o5ADE
0oooool3oalO7`Co8B4Q0olV9RH3oahN7PCo?clo0olU9BD4ogUiN@;o>c/k0oooool3ocdm?@CoATI6
0omKFe/4oeQHF0?oEeMG1?mUIFD3ofQXJ0?oE5AD0_ooool4oalO7`?oShn?1?l_;bl3oc0`<0CoKfm_
0on5QHD4oi^KV`;oFEUI0oooool3ob@T90Co6a/K0olg=cL4oe5AD@?o8b<S1?l[:b/3ohN7Q`?o:BTY
0_ooool4oaTI6@?oG5aL1?lF5QH3oaPH60CoUIFE0ol[:b/4oa4A4@;oRXZ:0oooool3ob<S8`Co6a/K
0ola<C44ocHf=P?o8B4Q1?lA4A43o`l?3`?o61PH0_ooool4oc<c<`?o>SXj1?lV9RH3oc@d=0Co;Rh^
0ol_;bl3oc8b<P?o;Bd]0oooool3ocXj>PCog]kN0omlO7`4ohZ:RP?omOGe0olJ6QX4omkNgP;o;2`/
0oooool4oeYJFP?o:RXZ0om:BTX4oiJFUP?oKFe]1?m_Kfl3odm?C`?o:b/[0oooool3ohV9R@?o9RHV
1?oNg]h3oeeMG@CoShn?0ooOgml4oa/K6`Gooooo0olb<S84oaLG5`?o6QXJ1?mNGUh3obl_;`CoDe=C
0om8B4P3oeADE0;ooooo1?oQhN43obPX:0Co3@d=0onc/k<4o`H61P?o4a<C1?mFEUH2ob4Q8A7ooooo
000?ooooo`?oMgMg1?mcLg<3ofEUI@Co<30`0omdM7@4ochn?P?o2@T90olk>c/2ooooo`Co2PX:0olF
5QH4ob@T90?o7QhN1?l];Bd3oc0`<0Co8R8R0_l:2PX3ooooo`?oLG5a1?oZj^X3ohb<S0CoDE5A0olo
?cl4oa`L70?o30`<0_lC4a<3ooooo`Co51@D0olm?Cd4ohB4Q0?o8R8R0om;Bd/4oc/k>`?o9bLW0olE
5AD3ooooo`?o7alO1?lQ8B43obHV9P?o7QhN1?lo?cl3obDU9@CoNGUi0_lk>c/3ooooo`?o?Cdm1?m6
ATH3oe]KF`CoF5QH0omGEeL4ofEUI@?oJ6QX0omDE5@2ooooo`Co7alO0on?Shl4obl_;`?o<30`1?m_
Kfl3ohF5Q@CoVi^K0_mIFET3ooooo`?o92@T1?lK6a/3ocLg=`CoDE5A0olS8b<4ob/[:`?oQhN70olY
:BT2ooooo`Co6ATI0omLG5`4oaHF5P?o61PH1?nEUID3ob/[:`Co4A4A0_n:RXX3ooooo`?o8b<S1?lK
6a/3oc4a<@Co=SHf0olQ8B44oa4A4@?o3`l?0olH61P2ooooo`Co<c<c0olj>SX4obHV9P?o=3@d1?l^
;Rh3obl_;`?o<S8b0ol];Bd3ooooo`?o>SXj1?oNg]h3ogalO0CoRXZ:0ooemOD3oaXJ6PCog]kN0_l/
;2`3ooooo`CoFUYJ0olZ:RX3odY:BPCoUYJF0om]KFd4ofm_K`?oCdm?0ol[:b/3ooooo`?oRHV90olV
9RH4omkNgP?oGEeM1?n?Shl3omoOg`Co6a/K1Oooool3oc8b<PCo5aLG0olJ6QX4oeiNGP?o;bl_1?mC
De<3odQ8B0?oE5AD0_ooool4on7Qh@?o:2PX1?l=3@d3ok>c/`Co1PH60olC4a<4oeIFEP;o8B4Q4Ooo
ool000oooooo0omgMgL4og=cL`?oIFEU1?l`<303ogAdM0Co?Shn0ol92@T3oc/k>`;ooooo1?l:2PX3
oaHF5PCo92@T0olN7Qh4obd];@?o<30`1?lR8R82o`X:2P?ooooo0omaLG44on[ZjP?oS8b<1?mADE43
oclo?`Co71`L0ol<30`2oa<C4`?ooooo1?lD51@3ocdm?@CoQ8B40olR8R83od];B`Co>c/k0olW9bL3
oaDE5@?ooooo0olO7al4ob4Q8@?o9RHV0olN7Qh4oclo?`?o9BDU1?miNGT2oc/k>`?ooooo0olm?Cd4
odI6AP?oFe]K1?mHF5P3oeMGE`CoIFEU0omXJ6P3oeADE0;ooooo1?lO7al3ohn?S`Co;bl_0ol`<304
ofm_K`?oQHF51?nKVi/2oeUIF@?ooooo0olT92@4oa/K6`?o=cLg1?mADE43ob<S8`Co:b/[0on7QhL3
obTY:@;ooooo1?lI6AT3oeaLG0Co5QHF0olH61P4oiFEU@?o:b/[1?lA4A42ohZ:RP?ooooo0olS8b<4
oa/K6`?o<C4a1?lf=SH3ob4Q8@Co4A4A0ol?3`l3oaPH60;ooooo1?lc<c<3ocXj>PCo9RHV0old=3@4
obh^;P?o;bl_0olb<S83obd];@?ooooo0olj>SX4omkNgP?oO7al1?n:RXX3ooGem@?o6QXJ1?oNg]h2
ob`/;0?ooooo1?mJFUX3obXZ:P?oBTY:1?nFUYH3ofe]K@CoKfm_0om?Cdl3ob/[:`?ooooo0on9RHT3
obHV9PCog]kN0omMGEd4ohn?S`?ogmoO1?lK6a/5ooooo`?o<S8b1?lG5aL3oaXJ6PCoGUiN0ol_;bl4
oe=CD`?oB4Q80omDE5@2ooooo`CohN7Q0olX:2P4o`d=3@?o/k>c1?l61PH3oa<C4`CoEUIF0_lQ8B4A
ooooo`003oooool3obTY:@CoB4Q80oo[jn/4oa8B4P?oBTY:1?n?Shl3ocXj>P?o9bLW0_ooool4oahN
7P?o4a<C1?lB4Q83o`@410Co3Ph>0ol<30`4o`H61P;o0P821_ooool4olG5a@?oLG5a1?m<C4`3oa`L
70Co4Q8B0ol>3Ph2ob4Q8@?ooooo1?lF5QH3ooooo`Co;2`/0ol410@3olG5a@Co6QXJ0olA4A43ochn
?P?ooooo0olB4Q84oa4A4@?o00000olC4a<4oadM7@?o6ATI1?ln?Sh2obLW9`?ooooo0on[Zj/4ol;2
`P?ooooo1?oQhN43omGEe@CoXj>S0on`/;03ofUYJ@;ooooo1?mPH603olO7a`CoN7Qh0oll?3`4oiZJ
VP?oXj>S1?ni^KT2oda<C0?ooooo1om2@T83o`0000Co;2`/0omDE5@4oaHF5P?o=CDe0om9BDT2oooo
o`Co4A4A0omNGUh4ocXj>P?o6a/K1?ll?3`3oclo?`CoDe=C0_lY:BT3ooooo`?o2`/;1?l?3`l7oa4A
4@?o30`<1?l61PH3oa@D50?o4Q8B0_ooool4oc8b<P?o71`L1?lV9RH3oc8b<PCo;bl_0old=3@3ocdm
?@?o<C4a0oooool3ocDe=@CoeMGE0olh>3P4oh20P0?ooooo0ol?3`l4om7Ad@;oATI60oooool4oaLG
5`?oATI60omeMGD4oc4a<@?okno_1?ooool3ob`/;0?o8b<S0oooool3og5aL@?o9BDU1?nVYZH3obl_
;`CoTI6A0oo8b<P4oaPH60;o]KFe0oooool3o`l?3`CoGUiN0olZ:RX4oadM7@?oBd];1?m7AdL3ob8R
8P?oEUIF0_ooool4ogalO0?oJVYZ1?meMGD3oemOG`Co<C4a0onb/[84ob4Q8@;o2@T94Oooool000oo
oooo0olY:BT4odQ8B0?ojn_[1?lB4Q83odY:BPCoShn?0olj>SX3obLW9`;ooooo1?lN7Qh3oa<C4`Co
4Q8B0ol410@4o`h>3P?o30`<1?l61PH2o`820PKooooo1?o5aLD3og5aL@CoC4a<0olL71`4oa8B4P?o
3Ph>0_lQ8B43ooooo`Co5QHF0oooool4ob`/;0?o10@40oo5aLD4oaXJ6P?o4A4A0oln?Sh3ooooo`?o
4Q8B1?lA4A43o`0000?o4a<C1?lM7Ad3oaTI6@Co?Shn0_lW9bL3ooooo`?oZj^[1?o2`/83ooooo`Co
hN7Q0ooEeMD4oj>SX`?o/;2`0omYJFT2ooooo`CoH61P0oo7alL4ogQhN0?o?3`l1?nJVYX3oj>SX`Co
^KVi0_m<C4`3ooooo`Oo@T920ol00004ob`/;0?oE5AD1?lF5QH3ocDe=@?oBDU90_ooool4oa4A4@?o
GUiN1?lj>SX3oa/K6`Co?3`l0olo?cl4oe=CD`;o:BTY0oooool3o`/;2`Co3`l?1olA4A43o``<30Co
1PH60olD51@3oa8B4P;ooooo1?lb<S83oa`L70Co9RHV0olb<S84obl_;`?o=3@d0olm?Cd3oc4a<@?o
oooo0ole=CD4omGEe@?o>3Ph1?n0P803ooooo`?o3`l?1?oAdM42odI6AP?ooooo1?lG5aL3odI6AP?o
MGEe1?la<C43ono_k`Cooooo0ol/;2`3ob<S8`?ooooo0omaLG43obDU9@CoYZJV0ol_;bl4oi6AT@?o
b<S81?lH61P2okFe]@?ooooo0ol?3`l4oeiNGP?o:RXZ1?lM7Ad3od];B`CoAdM70olR8R83oeIFEP;o
oooo1?mlO7`3ofYZJPCoMGEe0omOGel4oc4a<@?o/[:b1?lQ8B42o`T92A7ooooo000?ooooo`?o:BTY
1?m8B4P3on_[j`Co4Q8B0om:BTX4ohn?S`?o>SXj0olW9bL2ooooo`Co7QhN0olC4a<4oa8B4P?o10@4
1?l>3Ph3o``<30Co1PH60_l20P86ooooo`CoaLG50omaLG44oda<C0?o71`L1?lB4Q83o`h>3P;o8B4Q
0oooool4oaHF5P?ooooo1?l/;2`3o`@410?oaLG51?lJ6QX3oa4A4@?o?Shn0oooool3oa8B4PCo4A4A
0ol00003oa<C4`Co7AdM0olI6AT4ochn?P;o9bLW0oooool3oj^[Z`Co`/;20oooool4on7Qh@?oeMGE
1?nSXj<3ok2`/0?oJFUY0_ooool4of1PH0?oalO71?mhN7P3oc`l?0CoVYZJ0onSXj<4okVi^@;oC4a<
0oooool7od92@P?o00001?l/;2`3oeADE0Co5QHF0ole=CD3odU9B@;ooooo1?lA4A43oeiNGPCo>SXj
0olK6a/4oc`l?0?o?clo1?mCDe<2obTY:@?ooooo0ol;2`/4o`l?3`Oo4A4A0ol<30`4o`H61P?o51@D
0olB4Q82ooooo`Co<S8b0olL71`4obHV9P?o<S8b1?l_;bl3oc@d=0?o?Cdm0ola<C43ooooo`?o=CDe
1?oEeMD3ocPh>0CoP8200oooool3o`l?3`CodM7A0_m6ATH3ooooo`Co5aLG0om6ATH3ogEeM@Co<C4a
0oo_knl4ooooo`?o;2`/0olS8b<3ooooo`?oLG5a0olU9BD4ojJVYP?o;bl_1?nATI43olS8b0Co61PH
0_ne]KD3ooooo`?o3`l?1?mNGUh3obXZ:PCo7AdM0om;Bd/4odM7A`?o8R8R0omFEUH2ooooo`CoO7al
0omZJVX4ogEeM@?oGemO1?la<C43ok:b/PCo8B4Q0_l92@TAooooo`003oooool3ocPh>0CoY:BT0on:
RXX4o`T92@?ooooo1?lj>SX3oaPH60?oSHf=0_ooool4ob<S8`?o6ATI1?l<30`3oaLG5`Co4Q8B0ol9
2@T4o`410@;o51@D0oooool3oifMW@Co_Kfm0omgMgL4oeUIF@?o<S8b1?lQ8B43o`X:2P;o71`L0ooo
ool4obd];@?oNg]k1om`L703ofm_K`Co8B4Q0onOWil3o`410@?ooooo0olX:2P4ob<S8`?o4A4A0olO
7al4ocXj>P?o;Bd]1?m0@402oc0`<0?ooooo0omPH604of][J`?oOGem1?mWIfL3ofATI0CoCTi>0om1
@D43obDU9@;ooooo1?l=3@d3odY:BPCo<c<c0ol820P4og1`L0?o[Zj^1?m`L702od10@0?ooooo0oln
?Sh4odE5A@?o8R8R1?li>CT3oa0@40Co51@D0omWIfL3o`/;2`;ooooo1?mUIFD3oc/k>`Co410@0omK
Fe/4ochn?P?o410@1?ld=3@2odA4A0?ooooo0ol51@D4oaHF5POo4Q8B0ol820P4oa/K6`?o5ADE0olK
6a/2ooooo`CoCDe=0omHF5P4ochn?P?oAdM71?lU9BD3ocTi>@?o<c<c0olY:BT3ooooo`?o>3Ph1?n_
[jl3oa8B4PCoTI6A0onb/[83o`<30`Cono_k0_lR8R83ooooo`Co820P0olE5AD3oc8b<PCoG5aL0ol`
<304oc/k>`?o8R8R0olD51@3ooooo`?o30`<0ol];Bd4oh^;R`?o3`l?1?m_Kfl3ohV9R@Co6a/K0_nO
Wil3ooooo`?oHF5Q1?mgMgL3ocHf=PCooooo0onKVi/4odQ8B0?oalO70onk^k/2ooooo`CocLg=0olj
>SX4odA4A0?ojNWY1?lU9BD3oe1@D0CoLG5a0_mfMWHAooooo`003oooool3ocPh>0CoY:BT0on:RXX4
o`T92@?ooooo1?lj>SX3oaPH60?oSHf=0_ooool4ob<S8`?o6ATI1?l<30`3oaLG5`Co4Q8B0ol92@T4
o`410@;o51@D0oooool3oifMW@Co_Kfm0omgMgL4oeUIF@?o<S8b1?lQ8B43o`X:2P;o71`L0oooool4
obd];@?oNg]k1om`L703ofm_K`Co8B4Q0onOWil3o`410@?ooooo0olX:2P4ob<S8`?o4A4A0olO7al4
ocXj>P?o;Bd]1?m0@402oc0`<0?ooooo0omPH604of][J`?oOGem1?mWIfL3ofATI0CoCTi>0om1@D43
obDU9@;ooooo1?l=3@d3odY:BPCo<c<c0ol820P4og1`L0?o[Zj^1?m`L702od10@0?ooooo0oln?Sh4
odE5A@?o8R8R1?li>CT3oa0@40Co51@D0omWIfL3o`/;2`;ooooo1?mUIFD3oc/k>`Co410@0omKFe/4
ochn?P?o410@1?ld=3@2odA4A0?ooooo0ol51@D4oaHF5POo4Q8B0ol820P4oa/K6`?o5ADE0olK6a/2
ooooo`CoCDe=0omHF5P4ochn?P?oAdM71?lU9BD3ocTi>@?o<c<c0olY:BT3ooooo`?o>3Ph1?n_[jl3
oa8B4PCoTI6A0onb/[83o`<30`Cono_k0_lR8R83ooooo`Co820P0olE5AD3oc8b<PCoG5aL0ol`<304
oc/k>`?o8R8R0olD51@3ooooo`?o30`<0ol];Bd4oh^;R`?o3`l?1?m_Kfl3ohV9R@Co6a/K0_nOWil3
ooooo`?oHF5Q1?mgMgL3ocHf=PCooooo0onKVi/4odQ8B0?oalO70onk^k/2ooooo`CocLg=0olj>SX4
odA4A0?ojNWY1?lU9BD3oe1@D0CoLG5a0_mfMWHAooooo`003oooool3ocPh>0CoY:BT0on:RXX4o`T9
2@?ooooo1?lj>SX3oaPH60?oSHf=0_ooool4ob<S8`?o6ATI1?l<30`3oaLG5`Co4Q8B0ol92@T4o`41
0@;o51@D0oooool3oifMW@Co_Kfm0omgMgL4oeUIF@?o<S8b1?lQ8B43o`X:2P;o71`L0oooool4obd]
;@?oNg]k1om`L703ofm_K`Co8B4Q0onOWil3o`410@?ooooo0olX:2P4ob<S8`?o4A4A0olO7al4ocXj
>P?o;Bd]1?m0@402oc0`<0?ooooo0omPH604of][J`?oOGem1?mWIfL3ofATI0CoCTi>0om1@D43obDU
9@;ooooo1?l=3@d3odY:BPCo<c<c0ol820P4og1`L0?o[Zj^1?m`L702od10@0?ooooo0oln?Sh4odE5
A@?o8R8R1?li>CT3oa0@40Co51@D0omWIfL3o`/;2`;ooooo1?mUIFD3oc/k>`Co410@0omKFe/4ochn
?P?o410@1?ld=3@2odA4A0?ooooo0ol51@D4oaHF5POo4Q8B0ol820P4oa/K6`?o5ADE0olK6a/2oooo
o`CoCDe=0omHF5P4ochn?P?oAdM71?lU9BD3ocTi>@?o<c<c0olY:BT3ooooo`?o>3Ph1?n_[jl3oa8B
4PCoTI6A0onb/[83o`<30`Cono_k0_lR8R83ooooo`Co820P0olE5AD3oc8b<PCoG5aL0ol`<304oc/k
>`?o8R8R0olD51@3ooooo`?o30`<0ol];Bd4oh^;R`?o3`l?1?m_Kfl3ohV9R@Co6a/K0_nOWil3oooo
o`?oHF5Q1?mgMgL3ocHf=PCooooo0onKVi/4odQ8B0?oalO70onk^k/2ooooo`CocLg=0olj>SX4odA4
A0?ojNWY1?lU9BD3oe1@D0CoLG5a0_mfMWHAooooo`003oooool3obd];@CoTI6A0on=SHd4oaXJ6P?o
VIVI1?mHF5P3og]kN`?o?3`l0_ooool4oa<C4`Oo3`l?0ol61PH4oaLG5`Oo6a/K0_lE5AD3ooooo`?o
m?Cd1?n][Jd3of5QH@CoGUiN0olo?cl4oa/K6`?o6ATI0_lL71`3ooooo`Co9bLW0ooOgml4obd];@?o
CTi>0oo[jn/4o`L71`?oBd];0om7AdL3ooooo`?o=CDe1?lT92@3obh^;P?oA4A41?lo?cl3oc/k>`Co
eMGE0_mFEUH3ooooo`?o5QHF1?l:2PX3o``<30Co3@d=0olQ8B44o`l?3`?o820P0olJ6QX2ooooo`Co
Ogmo0oo8b<P4ogalO0?oJ6QX1?nNWYh3oi^KV`Cob<S80_mgMgL3ooooo`?ob<S81?lJ6QX3oahN7PCo
OGem0ol@4104oa@D50?o<C4a0olF5QH2ooooo`CoB4Q80onHV9P4ocXj>P?o?Cdm1?m4A4@3odU9B@Co
@d=30_lB4Q83ooooo`?o20P81?l00003oa8B4PCo5QHF0olG5aL4oaXJ6P?o30`<0olH61P2ooooo`Co
a<C40oooool4ono_k`?obLW91?nWYjL3oda<C0?oEEEE0olV9RH3ooooo`?oADE51?m4A4@3o`P820Co
UYJF0omFEUH3oa/K6`Coi^KV0_l820P3ooooo`Co61PH0olJ6QX3oclo?`Co>3Ph0omFEUH4od];B`?o
;2`/0olL71`3ooooo`?o2PX:0olh>3P4oc0`<0?o00001?nEUID3ob<S8`Co5QHF0_nFUYH3ooooo`?o
:2PX1?mBDU83ob<S8`Co92@T0omVIVH4ode=C@?o2@T90olj>SX2ooooo`Codm?C0ol/;2`4ohR8R0?o
WYjN1?lC4a<3ooooo`Co:BTY0_lW9bLAooooo`003oooool3obd];@CoTI6A0on=SHd4oaXJ6P?oVIVI
1?mHF5P3og]kN`?o?3`l0_ooool4oa<C4`Oo3`l?0ol61PH4oaLG5`Oo6a/K0_lE5AD3ooooo`?om?Cd
1?n][Jd3of5QH@CoGUiN0olo?cl4oa/K6`?o6ATI0_lL71`3ooooo`Co9bLW0ooOgml4obd];@?oCTi>
0oo[jn/4o`L71`?oBd];0om7AdL3ooooo`?o=CDe1?lT92@3obh^;P?oA4A41?lo?cl3oc/k>`CoeMGE
0_mFEUH3ooooo`?o5QHF1?l:2PX3o``<30Co3@d=0olQ8B44o`l?3`?o820P0olJ6QX2ooooo`CoOgmo
0oo8b<P4ogalO0?oJ6QX1?nNWYh3oi^KV`Cob<S80_mgMgL3ooooo`?ob<S81?lJ6QX3oahN7PCoOGem
0ol@4104oa@D50?o<C4a0olF5QH2ooooo`CoB4Q80onHV9P4ocXj>P?o?Cdm1?m4A4@3odU9B@Co@d=3
0_lB4Q83ooooo`?o20P81?l00003oa8B4PCo5QHF0olG5aL4oaXJ6P?o30`<0olH61P2ooooo`Coa<C4
0oooool4ono_k`?obLW91?nWYjL3oda<C0?oEEEE0olV9RH3ooooo`?oADE51?m4A4@3o`P820CoUYJF
0omFEUH3oa/K6`Coi^KV0_l820P3ooooo`Co61PH0olJ6QX3oclo?`Co>3Ph0omFEUH4od];B`?o;2`/
0olL71`3ooooo`?o2PX:0olh>3P4oc0`<0?o00001?nEUID3ob<S8`Co5QHF0_nFUYH3ooooo`?o:2PX
1?mBDU83ob<S8`Co92@T0omVIVH4ode=C@?o2@T90olj>SX2ooooo`Codm?C0ol/;2`4ohR8R0?oWYjN
1?lC4a<3ooooo`Co:BTY0_lW9bLAooooo`003oooool3obd];@CoTI6A0on=SHd4oaXJ6P?oVIVI1?mH
F5P3og]kN`?o?3`l0_ooool4oa<C4`Oo3`l?0ol61PH4oaLG5`Oo6a/K0_lE5AD3ooooo`?om?Cd1?n]
[Jd3of5QH@CoGUiN0olo?cl4oa/K6`?o6ATI0_lL71`3ooooo`Co9bLW0ooOgml4obd];@?oCTi>0oo[
jn/4o`L71`?oBd];0om7AdL3ooooo`?o=CDe1?lT92@3obh^;P?oA4A41?lo?cl3oc/k>`CoeMGE0_mF
EUH3ooooo`?o5QHF1?l:2PX3o``<30Co3@d=0olQ8B44o`l?3`?o820P0olJ6QX2ooooo`CoOgmo0oo8
b<P4ogalO0?oJ6QX1?nNWYh3oi^KV`Cob<S80_mgMgL3ooooo`?ob<S81?lJ6QX3oahN7PCoOGem0ol@
4104oa@D50?o<C4a0olF5QH2ooooo`CoB4Q80onHV9P4ocXj>P?o?Cdm1?m4A4@3odU9B@Co@d=30_lB
4Q83ooooo`?o20P81?l00003oa8B4PCo5QHF0olG5aL4oaXJ6P?o30`<0olH61P2ooooo`Coa<C40ooo
ool4ono_k`?obLW91?nWYjL3oda<C0?oEEEE0olV9RH3ooooo`?oADE51?m4A4@3o`P820CoUYJF0omF
EUH3oa/K6`Coi^KV0_l820P3ooooo`Co61PH0olJ6QX3oclo?`Co>3Ph0omFEUH4od];B`?o;2`/0olL
71`3ooooo`?o2PX:0olh>3P4oc0`<0?o00001?nEUID3ob<S8`Co5QHF0_nFUYH3ooooo`?o:2PX1?mB
DU83ob<S8`Co92@T0omVIVH4ode=C@?o2@T90olj>SX2ooooo`Codm?C0ol/;2`4ohR8R0?oWYjN1?lC
4a<3ooooo`Co:BTY0_lW9bLAooooo`003oooool3oaXJ6PCoHf=S0onDU9@4obLW9`?o^k^k1?m2@T83
oeiNGP?oLg=c0_ooool4oa<C4`?o7alO1?l>3Ph3ob0P80Co7AdM0olN7Qh4oa`L70;o410@0oooool3
ode=C@CoWYjN0omLG5`4odA4A0?o:2PX1?l[:b/3o`l?3`;o:b/[0oooool4oa`L70?o_Kfm1?l@4103
oi6AT@?oYJFU1?l410@3on?Sh`?o3@d=0oooool3oc0`<0Co;bl_1_lR8R84odY:BP?o?Shn1?n2PX82
ocLg=`?ooooo0ol92@T4o`0000?o0@411?l20P83o`410@Co61PH0olI6AT3ob@T90;ooooo1?l00003
of=SH`CoDE5A0olj>SX4oe=CD`?oUiNG1?nFUYH2odU9B@?ooooo0omlO7`4obTY:@?o`<301?la<C43
o``<30CoUIFE0ol@4103oa<C4`Kooooo0om7AdL4oaPH60?ono_k1?l<30`3obPX:0CoPh>30_lf=SH3
ooooo`?o3@d=1?lE5AD3o`X:2PCo5ADE0ol?3`l7o``<30?o2`/;0_ooool4oe1@D0?o@D511?mDE5@3
oda<C0CoAdM70om1@D43obd];@?o:RXZ0oooool3oc0`<0CoB4Q80olP8204oeADE0?oCDe=0olj>SX4
ohj>SP;o3@d=0oooool4oa0@40?o1@D50ol820P4oa`L70?o=3@d1?l=3@d3oadM7@?o4Q8B0oooool3
o`@410?o@T921?l];Bd3oadM7@Co?3`l0olk>c/6oc4a<@?ooooo0ole=CD4of1PH0?o9bLW1?n`/;03
ogEeM@CoE5AD0om`L703ok^k^`;ooooo1?mJFUX3oc0`<0CoXJ6Q0on4Q8@4oahN7P?o[jn_1?m9BDT2
ojJVYQ7ooooo000?ooooo`?o6QXJ1?mSHf<3oiBDU0Co9bLW0onk^k/4od92@P?oGUiN0omcLg<2oooo
o`Co4a<C0olO7al4o`h>3P?o820P1?lM7Ad3oahN7PCo71`L0_l@4103ooooo`?oCDe=1?nNWYh3oeaL
G0CoA4A40olX:2P4ob/[:`?o3`l?0_l[:b/3ooooo`Co71`L0onm_Kd4oa0@40?oTI6A0onUYJD4o`@4
10?ohn?S0ol=3@d3ooooo`?o<30`1?l_;bl6ob8R8PCoBTY:0oln?Sh4oh:2PP;o=cLg0oooool3o`T9
2@Co00000ol10@44o`820P?o0@411?lH61P3oaTI6@?o92@T0_ooool4o`0000?oHf=S1?mADE43ocXj
>PCoDe=C0onGUiL4oiJFUP;oBDU90oooool3ogalO0Co:BTY0oo0`<04oc4a<@?o30`<1?nEUID3oa0@
40?o4a<C1_ooool3odM7A`Co61PH0ookno/4o``<30?o:2PX1?n3Ph<2ocHf=P?ooooo0ol=3@d4oaDE
5@?o2PX:1?lE5AD3o`l?3`Oo30`<0ol;2`/2ooooo`CoD51@0om1@D44oeADE0?oC4a<1?m7AdL3od51
@@?o;Bd]0olZ:RX3ooooo`?o<30`1?m8B4P3ob0P80CoE5AD0om=CDd3ocXj>PCoSXj>0_l=3@d3oooo
o`Co410@0ol51@D3o`P820Co71`L0old=3@4o`d=3@?o7AdM0olB4Q83ooooo`?o10@40om2@T84obd]
;@?o7AdM1?ll?3`3oc/k>`Ko<C4a0oooool3ocDe=@CoH61P0olW9bL4ok2`/0?oMGEe1?mDE5@3og1`
L0?o^k^k0_ooool4oeYJFP?o<30`1?nQXJ43ohB4Q0Co7QhN0on_[jl4odU9B@;oYZJV4Oooool000oo
oooo0olJ6QX4of=SH`?oU9BD1?lW9bL3ok^k^`Co@T920omNGUh3og=cL`;ooooo1?lC4a<3oalO7`Co
3Ph>0olP8204oadM7@?o7QhN1?lL71`2oa0@40?ooooo0om=CDd4oijNWP?oG5aL1?m4A4@3obPX:0Co
:b/[0ol?3`l2ob/[:`?ooooo1?lL71`3okfm_@Co410@0onATI43ojFUY@Co10@40ooShn<3o`d=3@?o
oooo0ol`<304obl_;`Ko8R8R1?m:BTX3ochn?PCoPX:20_lg=cL3ooooo`?o2@T91?l00003o`410@Co
0P820ol10@44oaPH60?o6ATI0olT92@2ooooo`Co00000omSHf<4oe5AD@?o>SXj1?mCDe<3oiNGU`Co
UYJF0_m9BDT3ooooo`?oO7al1?lY:BT3ol30`0Co<C4a0ol<30`4oiFEU@?o410@0olC4a<6ooooo`?o
AdM71?lH61P3oo_kn`Co30`<0olX:2P4oh>3P`;o=SHf0oooool3o`d=3@Co5ADE0ol:2PX4oaDE5@?o
3`l?1ol<30`3o`/;2`;ooooo1?m@D503od51@@CoE5AD0om<C4`4odM7A`?o@D510ol];Bd3obXZ:P?o
oooo0ol`<304odQ8B0?o820P1?mDE5@3ode=C@?o>SXj1?n>SXh2o`d=3@?ooooo1?l@4103o`D51@?o
20P81?lL71`3oc@d=0Co3@d=0olM7Ad3oa8B4P?ooooo0ol410@3od92@PCo;Bd]0olM7Ad4oc`l?0?o
>c/k1_la<C43ooooo`?o=CDe1?mPH603obLW9`Co/;2`0omeMGD4oeADE0?oL71`0onk^k/2ooooo`Co
FUYJ0ol`<304oj6QX@?oQ8B41?lN7Qh3ojn_[`CoBDU90_nVYZHAooooo`003oooool3o`l?3`CoX:2P
0olW9bL4oa@D50?obl_;1?lH61P3og1`L0?oGemO0_ooool4ob0P80?o3`l?1?l>3Ph3obHV9PCo5QHF
0olB4Q84ob0P80;o3@d=0oooool3olo?c`CoOGem0om0@404odY:BP?o9bLW1?lR8R83o`0000;o:b/[
0oooool4oeADE0?o@4101?lL71`3ojJVYP?oGemO1?lA4A43okZj^P?o1`L70oooool3o`h>3PCo5ADE
0ol20P83oaDE5@Co8R8R0olW9bL4obd];@;o8R8R0oooool3o``<30Co2@T90ol?3`l4oalO7`?o:b/[
1?li>CT3od51@@?o@T920_ooool4of][J`?o^k^k1?m2@T83ocXj>PCocLg=0omoOgl4olK6aP;oKVi^
0oooool3om7Ad@Co8b<S0omTI6@4ohR8R0?oFEUI1?m>CTh3oe1@D0?oLg=c0_ooool4okfm_@?oK6a/
1?n>SXh3oi:BTPCo<S8b0ooAdM44obPX:0;o6a/K0oooool3o`T92@Co4Q8B0ol71`L4o`T92@?o410@
1?l61PH3o`d=3@?o1PH60_ooool4o`X:2P?o00001?l<30`3o`@410Co4A4A0ol61PH3ob@T90?o8B4Q
0oooool3ofUYJ@Co410@0olJ6QX4ofATI0?o00000olO7al4ogemO@;o10@40oooool4oa`L70?o0000
0olR8R84obh^;P?o1`L71?lB4Q83oadM7@?o51@D0oooool3od];B`?oCDe=1?l00003ob4Q8@Co?3`l
0ol10@44oa@D50;o>SXj0oooool3o`0000Co>3Ph0olU9BD4ob<S8`?o7QhN1?lb<S83o`0000?o7QhN
0_ooool4oiBDU0?o20P81?m8B4P3oh^;R`Co=SHf0onFUYH4oc`l?0;oNg]k4Oooool000oooooo0ol?
3`l4oj2PX0?o9bLW1?lD51@3ol_;b`Co61PH0om`L703oemOG`;ooooo1?lP8203o`l?3`Co3Ph>0olV
9RH4oaHF5P?o4Q8B1?lP8202o`d=3@?ooooo0oo?cll4ogemO@?o@4101?m:BTX3obLW9`Co8R8R0ol0
0002ob/[:`?ooooo1?mDE5@3od10@0Co71`L0onVYZH3oemOG`Co4A4A0onj^[X3o`L71`?ooooo0ol>
3Ph4oaDE5@?o0P820olE5AD4ob8R8P?o9bLW1?l];Bd2ob8R8P?ooooo0ol<30`4o`T92@?o3`l?1?lO
7al3ob/[:`Co>CTi0om1@D43od92@P;ooooo1?m[Jf/3ok^k^`Co@T920olj>SX4olg=c@?oOgmo1?o6
a/H2ofi^KP?ooooo0ooAdM44ob<S8`?oI6AT1?n8R8P3oeUIF@CoCTi>0om@D503og=cL`;ooooo1?nm
_Kd3ofa/K0CoSXj>0onBTY84oc8b<P?odM7A1?lX:2P2oa/K6`?ooooo0ol92@T4oa8B4P?o1`L71?l9
2@T3oa0@40Co1PH60ol=3@d3o`H61P;ooooo1?l:2PX3o`0000Co30`<0ol410@4oa4A4@?o1PH60olT
92@3ob4Q8@?ooooo0omYJFT4oa0@40?o6QXJ1?mTI6@3o`0000?o7alO1?mmOGd2o`@410?ooooo1?lL
71`3o`0000?o8R8R1?l^;Rh3o`L71`Co4Q8B0olM7Ad3oa@D50?ooooo0om;Bd/3ode=C@Co00000olQ
8B44oc`l?0?o0@411?lD51@2ocXj>P?ooooo0ol00004ocPh>0?o9BDU1?lS8b<3oahN7PCo<S8b0ol0
0003oahN7P;ooooo1?nDU9@3o`P820CoB4Q80on;Rh/4ocHf=P?oUYJF1?ll?3`2og]kNa7ooooo000?
ooooo`?o3`l?1?nPX:03obLW9`Co51@D0oo;bl/4oaPH60?oL71`0omOGel2ooooo`Co820P0ol?3`l4
o`h>3P?o9RHV1?lF5QH3oa8B4PCo820P0_l=3@d3ooooo`?oclo?1?mmOGd3od10@0CoBTY:0olW9bL4
ob8R8P?o00000_l[:b/3ooooo`CoE5AD0om0@404oa`L70?oYZJV0omOGel4oa4A4@?o^[Zj0ol71`L3
ooooo`?o3Ph>1?lE5AD3o`820P?o5ADE1?lR8R83obLW9`Co;Bd]0_lR8R83ooooo`?o30`<1?l92@T3
o`l?3`Co7alO0ol[:b/4ocTi>@?o@D510om2@T82ooooo`CoJf][0onk^k/4od92@P?o>SXj1?o=cLd3
ogmoO`Coa/K60_m^KVh3ooooo`?odM7A1?lS8b<3ofATI0CoR8R80omIFET4odi>CP?oD51@0omcLg<2
ooooo`Co_Kfm0om/K6`4ohj>SP?oTY:B1?lb<S83om7Ad@Co:2PX0_lK6a/3ooooo`?o2@T91?lB4Q83
o`L71`Co2@T90ol@4104o`H61P?o3@d=0ol61PH2ooooo`Co2PX:0ol00004o``<30?o10@41?lA4A43
o`H61P?o92@T0olQ8B43ooooo`?oJFUY1?l@4103oaXJ6PCoI6AT0ol00003oalO7`CoOGem0_l410@3
ooooo`Co71`L0ol00003ob8R8PCo;Rh^0ol71`L4oa8B4P?o7AdM0olD51@3ooooo`?oBd];0om=CDd4
o`0000?o8B4Q1?ll?3`3o`410@Co51@D0_lj>SX3ooooo`?o00001?lh>3P3obDU9@Co8b<S0olN7Qh4
oc8b<P?o00000olN7Qh2ooooo`CoU9BD0ol820P4odQ8B0?oRh^;1?lf=SH3oiJFUPCo?3`l0_mkNg/A
ooooo`003oooool3o`l?3`CoX:2P0olW9bL4oa@D50?obl_;1?lH61P3og1`L0?oGemO0_ooool4ob0P
80?o3`l?1?l>3Ph3obHV9PCo5QHF0olB4Q84ob0P80;o3@d=0oooool3olo?c`CoOGem0om0@404odY:
BP?o9bLW1?lR8R83o`0000;o:b/[0oooool4oeADE0?o@4101?lL71`3ojJVYP?oGemO1?lA4A43okZj
^P?o1`L70oooool3o`h>3PCo5ADE0ol20P83oaDE5@Co8R8R0olW9bL4obd];@;o8R8R0oooool3o``<
30Co2@T90ol?3`l4oalO7`?o:b/[1?li>CT3od51@@?o@T920_ooool4of][J`?o^k^k1?m2@T83ocXj
>PCocLg=0omoOgl4olK6aP;oKVi^0oooool3om7Ad@Co8b<S0omTI6@4ohR8R0?oFEUI1?m>CTh3oe1@
D0?oLg=c0_ooool4okfm_@?oK6a/1?n>SXh3oi:BTPCo<S8b0ooAdM44obPX:0;o6a/K0oooool3o`T9
2@Co4Q8B0ol71`L4o`T92@?o410@1?l61PH3o`d=3@?o1PH60_ooool4o`X:2P?o00001?l<30`3o`@4
10Co4A4A0ol61PH3ob@T90?o8B4Q0oooool3ofUYJ@Co410@0olJ6QX4ofATI0?o00000olO7al4ogem
O@;o10@40oooool4oa`L70?o00000olR8R84obh^;P?o1`L71?lB4Q83oadM7@?o51@D0oooool3od];
B`?oCDe=1?l00003ob4Q8@Co?3`l0ol10@44oa@D50;o>SXj0oooool3o`0000Co>3Ph0olU9BD4ob<S
8`?o7QhN1?lb<S83o`0000?o7QhN0_ooool4oiBDU0?o20P81?m8B4P3oh^;R`Co=SHf0onFUYH4oc`l
?0;oNg]k4Oooool000oooooo0om0@404obLW9`?oADE51?mUIFD3odU9B@Co9BDU0onNWYh3ob8R8P;o
oooo1?l?3`l3o`/;2`Co5aLG0ol00004o`820P?o30`<1_l@4103ooooo`?oRXZ:1?mOGel3oe5AD@Co
9bLW0olK6a/4obXZ:P?o10@40_lC4a<3ooooo`Co7QhN0omoOgl4o`L71`?oCDe=0on<S8`4ocHf=P?o
RHV90olA4A43ooooo`?o820P1?lR8R83oaXJ6P?o6ATI1?li>CT3obXZ:PCoATI60_lh>3P3ooooo`?o
<c<c1?m2@T83ob`/;0CoBd];0om?Cdl4oeIFEP?oF5QH0omGEeL2ooooo`Co20P80om[Jf/4ob`/;0?o
>c/k1?l@4103ojJVYPCoU9BD0_mXJ6P3ooooo`?oY:BT1?lI6AT3omcLg0CoCTi>0oln?Sh4ooooo`?o
;2`/0old=3@2ooooo`CoalO70om9BDT4oeiNGP?oeMGE1?lR8R83og]kN`CoSHf=0_n6QXH3ooooo`?o
30`<1?l10@43o`0000Co410@0olD51@4oaLG5`?o2PX:0ol<30`2ooooo`Co:BTY0olF5QH4o`l?3`?o
5ADE1?l:2PX6oa<C4`?o8R8R0oooool3ohB4Q0Co:RXZ0omZJVX4ocPh>0?o10@40om^KVh4ob<S8`;o
1PH60oooool4oaDE5@?o1`L70ol_;bl4oc/k>`?o:RXZ1?m3@d<3ocDe=@?o4a<C0oooool3ofYZJP?o
NWYj1?lI6AT3of][J`Co6ATI0ol_;bl4odE5A@;o0P820oooool3odY:BPCoKFe]0olX:2P4okfm_@?o
/k>c1?ll?3`3ofi^KP?oOWin0_ooool4o`L71`?o3Ph>1?nFUYH3o`d=3@Co30`<0onj^[X4oa@D50;o
Jf][4Oooool000oooooo0om0@404obLW9`?oADE51?mUIFD3odU9B@Co9BDU0onNWYh3ob8R8P;ooooo
1?l?3`l3o`/;2`Co5aLG0ol00004o`820P?o30`<1_l@4103ooooo`?oRXZ:1?mOGel3oe5AD@Co9bLW
0olK6a/4obXZ:P?o10@40_lC4a<3ooooo`Co7QhN0omoOgl4o`L71`?oCDe=0on<S8`4ocHf=P?oRHV9
0olA4A43ooooo`?o820P1?lR8R83oaXJ6P?o6ATI1?li>CT3obXZ:PCoATI60_lh>3P3ooooo`?o<c<c
1?m2@T83ob`/;0CoBd];0om?Cdl4oeIFEP?oF5QH0omGEeL2ooooo`Co20P80om[Jf/4ob`/;0?o>c/k
1?l@4103ojJVYPCoU9BD0_mXJ6P3ooooo`?oY:BT1?lI6AT3omcLg0CoCTi>0oln?Sh4ooooo`?o;2`/
0old=3@2ooooo`CoalO70om9BDT4oeiNGP?oeMGE1?lR8R83og]kN`CoSHf=0_n6QXH3ooooo`?o30`<
1?l10@43o`0000Co410@0olD51@4oaLG5`?o2PX:0ol<30`2ooooo`Co:BTY0olF5QH4o`l?3`?o5ADE
1?l:2PX6oa<C4`?o8R8R0oooool3ohB4Q0Co:RXZ0omZJVX4ocPh>0?o10@40om^KVh4ob<S8`;o1PH6
0oooool4oaDE5@?o1`L70ol_;bl4oc/k>`?o:RXZ1?m3@d<3ocDe=@?o4a<C0oooool3ofYZJP?oNWYj
1?lI6AT3of][J`Co6ATI0ol_;bl4odE5A@;o0P820oooool3odY:BPCoKFe]0olX:2P4okfm_@?o/k>c
1?ll?3`3ofi^KP?oOWin0_ooool4o`L71`?o3Ph>1?nFUYH3o`d=3@Co30`<0onj^[X4oa@D50;oJf][
4Oooool000oooooo0om0@404obLW9`?oADE51?mUIFD3odU9B@Co9BDU0onNWYh3ob8R8P;ooooo1?l?
3`l3o`/;2`Co5aLG0ol00004o`820P?o30`<1_l@4103ooooo`?oRXZ:1?mOGel3oe5AD@Co9bLW0olK
6a/4obXZ:P?o10@40_lC4a<3ooooo`Co7QhN0omoOgl4o`L71`?oCDe=0on<S8`4ocHf=P?oRHV90olA
4A43ooooo`?o820P1?lR8R83oaXJ6P?o6ATI1?li>CT3obXZ:PCoATI60_lh>3P3ooooo`?o<c<c1?m2
@T83ob`/;0CoBd];0om?Cdl4oeIFEP?oF5QH0omGEeL2ooooo`Co20P80om[Jf/4ob`/;0?o>c/k1?l@
4103ojJVYPCoU9BD0_mXJ6P3ooooo`?oY:BT1?lI6AT3omcLg0CoCTi>0oln?Sh4ooooo`?o;2`/0old
=3@2ooooo`CoalO70om9BDT4oeiNGP?oeMGE1?lR8R83og]kN`CoSHf=0_n6QXH3ooooo`?o30`<1?l1
0@43o`0000Co410@0olD51@4oaLG5`?o2PX:0ol<30`2ooooo`Co:BTY0olF5QH4o`l?3`?o5ADE1?l:
2PX6oa<C4`?o8R8R0oooool3ohB4Q0Co:RXZ0omZJVX4ocPh>0?o10@40om^KVh4ob<S8`;o1PH60ooo
ool4oaDE5@?o1`L70ol_;bl4oc/k>`?o:RXZ1?m3@d<3ocDe=@?o4a<C0oooool3ofYZJP?oNWYj1?lI
6AT3of][J`Co6ATI0ol_;bl4odE5A@;o0P820oooool3odY:BPCoKFe]0olX:2P4okfm_@?o/k>c1?ll
?3`3ofi^KP?oOWin0_ooool4o`L71`?o3Ph>1?nFUYH3o`d=3@Co30`<0onj^[X4oa@D50;oJf][4Ooo
ool000oooooo0ol;2`/4oaTI6@?o@D511?lT92@3o`d=3@Co=SHf0om5ADD3o`0000;ooooo1?m?Cdl3
od10@0Co@D510omLG5`4ooooo`?omoOg1?n9RHT2oclo?`?ooooo0omTI6@4ogAdM0?oDU9B1?lZ:RX3
oalO7`Co<c<c0ol:2PX2obl_;`?ooooo1?m6ATH3ob/[:`Co0`<30ol92@T3o``<30Co92@T0ol00003
o`h>3P?ooooo0ola<C44obTY:@?o=CDe0old=3@4od10@0?o>c/k1?ooool2oeMGE`?ooooo0on2PX84
oi6AT@?oSHf=1?n?Shl3oinOW`Co[Jf]0oo7alL3ok>c/`;ooooo1?mGEeL3om3@d0CoBTY:0om4A4@4
ogAdM0?ooooo1?o0`<02ojZZZP?ooooo0om2@T84oeEEE@?o4A4A1?lP8203oibLW0Co4A4A0olk>c/3
oeMGE`;ooooo1?l20P83oaDE5@CoZZZZ0ol00004od10@0?o[jn_1?l;2`/2ode=C@?ooooo0olh>3P4
oeADE0?oiNGU1?ooool3oh:2PPCo?Shn0om0@403oclo?`;ooooo1?nWYjL3ofm_K`CoP8200omSHf<4
of][J`?oFEUI0omlO7`3ofQXJ0?ooooo0olA4A44oinOW`?oFe]K1?lT92@3oiNGU`?o820P1?l[:b/2
oa`L70?ooooo1?lk>c/3oc`l?0?oKfm_1?mPH603ol[:bPCok>c/0om1@D43ocDe=@?ooooo0onJVYX3
oa<C4`CoYJFU0om3@d<4oaHF5P?oWYjN1?lE5AD2ocdm?@?ooooo0olR8R84ofe]K@?o=3@d1?lk>c/3
oiRHV0Co?Shn0olG5aL3ogemO@;ooooo1?l410@3oeEEE@Co9BDU0ol00004odU9B@?o;2`/1?l61PH2
o``<317ooooo000?ooooo`?o2`/;1?lI6AT3od51@@Co92@T0ol=3@d4ocHf=P?oADE50ol00002oooo
o`CoCdm?0om0@404od51@@?oG5aL1?ooool3ooOgm`CoRHV90_lo?cl3ooooo`?oI6AT1?mdM7@3oe9B
DPCo:RXZ0olO7al4oc<c<`?o2PX:0_l_;bl3ooooo`CoATI60ol[:b/4o`<30`?o2@T90ol<30`4ob@T
90?o00000ol>3Ph3ooooo`?o<C4a1?lY:BT3ocDe=@?o=3@d1?m0@403oc/k>`Cooooo0_mGEeL3oooo
o`?oPX:21?nATI43ohf=S@CoShn?0onOWil4ojf][@?oalO70onc/k<2ooooo`CoEeMG0oo@d=04odY:
BP?oA4A41?mdM7@3ooooo`Co`<300_nZZZX3ooooo`?o@T921?mEEED3oa4A4@Co820P0onLW9`4oa4A
4@?o>c/k0omGEeL2ooooo`Co0P820olE5AD4ojZZZP?o00001?m0@403ojn_[`Co2`/;0_m=CDd3oooo
o`?o>3Ph1?mDE5@3onGUi@Cooooo0on2PX84ochn?P?o@4100olo?cl2ooooo`CoYjNW0om_Kfl4oh20
P0?oHf=S1?m[Jf/3oeUIF@?oO7al0omXJ6P3ooooo`?o4A4A1?nOWil3oe]KF`Co92@T0onGUiL3ob0P
80Co:b/[0_lL71`3ooooo`Co>c/k0oll?3`3ofm_K`CoH61P0oo:b/X4onc/k0?o@D510ole=CD3oooo
o`?oVYZJ0olC4a<4ojFUY@?o@d=31?lF5QH3oijNWPCo5ADE0_lm?Cd3ooooo`?o8R8R1?m]KFd3oc@d
=0Co>c/k0onHV9P4ochn?P?o5aLG0ommOGd2ooooo`Co10@40omEEED4obDU9@?o00001?m9BDT3ob`/
;0Co1PH60_l<30`Aooooo`003oooool3o`/;2`Co6ATI0om1@D44ob@T90?o3@d=1?lf=SH3odE5A@?o
00000_ooool4odm?C`?o@4101?m1@D43oeaLG0Cooooo0oogmoL4ohV9R@;o?clo0oooool3ofATI0Co
M7Ad0omBDU84obXZ:P?o7alO1?lc<c<3o`X:2P;o;bl_0oooool4odI6AP?o:b/[1?l30`<3o`T92@?o
30`<1?lT92@3o`0000?o3Ph>0oooool3oc4a<@Co:BTY0ole=CD3oc@d=0Co@4100olk>c/4ooooo`;o
EeMG0oooool3oh:2PPCoTI6A0on=SHd4ohn?S`?oWinO1?n][Jd3olO7a`?o/k>c0_ooool4oeMGE`?o
d=3@1?m:BTX3odA4A0CoM7Ad0oooool4ol30`0;oZZZZ0oooool3od92@PCoEEEE0olA4A44ob0P80?o
W9bL1?lA4A43oc/k>`?oEeMG0_ooool4o`820P?o5ADE1?nZZZX3o`0000Co@4100on_[jl4o`/;2`;o
CDe=0oooool3ocPh>0CoE5AD0ooUiND4ooooo`?oPX:21?ln?Sh3od10@0?o?clo0_ooool4ojNWY`?o
Kfm_1?n0P803of=SH`CoJf][0omIFET3ogalO0?oJ6QX0oooool3oa4A4@CoWinO0omKFe/4ob@T90?o
UiNG0olP8204ob/[:`;o71`L0oooool4oc/k>`?o?3`l0om_Kfl4of1PH0?ob/[:1?o/k>`3od51@@?o
=CDe0oooool3oiZJVP?o4a<C1?nUYJD3od=3@`Co5QHF0onNWYh4oaDE5@;o?Cdm0oooool3ob8R8PCo
KFe]0old=3@4oc/k>`?oV9RH1?ln?Sh3oaLG5`?oOGem0_ooool4o`@410?oEEEE1?lU9BD3o`0000Co
BDU90ol/;2`4o`H61P;o30`<4Oooool00?oooooomOooool00?oooooomOooool00001\
\>"],
  ImageRangeCache->{{{0, 499}, {59, 0}} -> {-0.525373, -0.04801, 0.0341698, \
0.0341698}, {{15.375, 42.125}, {57.5625, 30.8125}} -> {-4.59822, -0.429951, \
0.299068, 0.299068}, {{44.75, 71.5625}, {57.5625, 30.8125}} -> {-13.3927, \
-0.429951, 0.299068, 0.299068}, {{74.1875, 100.938}, {57.5625, 30.8125}} -> \
{-22.1872, -0.429951, 0.299068, 0.299068}, {{103.625, 130.375}, {57.5625, \
30.8125}} -> {-30.991, -0.429951, 0.299068, 0.299068}, {{133.062, 159.812}, \
{57.5625, 30.8125}} -> {-39.7948, -0.429951, 0.299068, 0.299068}, {{162.5, \
189.25}, {57.5625, 30.8125}} -> {-48.5987, -0.429951, 0.299068, 0.299068}, \
{{191.938, 218.688}, {57.5625, 30.8125}} -> {-57.4025, -0.429951, 0.299068, \
0.299068}, {{221.375, 248.125}, {57.5625, 30.8125}} -> {-66.2063, -0.429951, \
0.299068, 0.299068}, {{250.812, 277.562}, {57.5625, 30.8125}} -> {-75.0101, \
-0.429951, 0.299068, 0.299068}, {{280.25, 307}, {57.5625, 30.8125}} -> \
{-83.814, -0.429951, 0.299068, 0.299068}, {{309.688, 336.438}, {57.5625, \
30.8125}} -> {-92.6178, -0.429951, 0.299068, 0.299068}, {{339.125, 365.875}, \
{57.5625, 30.8125}} -> {-101.422, -0.429951, 0.299068, 0.299068}, {{368.562, \
395.312}, {57.5625, 30.8125}} -> {-110.225, -0.429951, 0.299068, 0.299068}, \
{{398, 424.75}, {57.5625, 30.8125}} -> {-119.029, -0.429951, 0.299068, \
0.299068}, {{427.375, 454.188}, {57.5625, 30.8125}} -> {-127.824, -0.429951, \
0.299068, 0.299068}, {{456.812, 483.625}, {57.5625, 30.8125}} -> {-136.628, \
-0.429951, 0.299068, 0.299068}, {{15.375, 42.125}, {28.125, 1.375}} -> \
{-4.59822, -9.23378, 0.299068, 0.299068}, {{44.75, 71.5625}, {28.125, 1.375}} -> \
{-13.3927, -9.23378, 0.299068, 0.299068}, {{74.1875, 100.938}, {28.125, \
1.375}} -> {-22.1872, -9.23378, 0.299068, 0.299068}, {{103.625, 130.375}, \
{28.125, 1.375}} -> {-30.991, -9.23378, 0.299068, 0.299068}, {{133.062, \
159.812}, {28.125, 1.375}} -> {-39.7948, -9.23378, 0.299068, 0.299068}, \
{{162.5, 189.25}, {28.125, 1.375}} -> {-48.5987, -9.23378, 0.299068, \
0.299068}, {{191.938, 218.688}, {28.125, 1.375}} -> {-57.4025, -9.23378, \
0.299068, 0.299068}, {{221.375, 248.125}, {28.125, 1.375}} -> {-66.2063, \
-9.23378, 0.299068, 0.299068}, {{250.812, 277.562}, {28.125, 1.375}} -> \
{-75.0101, -9.23378, 0.299068, 0.299068}, {{280.25, 307}, {28.125, 1.375}} -> \
{-83.814, -9.23378, 0.299068, 0.299068}, {{309.688, 336.438}, {28.125, \
1.375}} -> {-92.6178, -9.23378, 0.299068, 0.299068}, {{339.125, 365.875}, \
{28.125, 1.375}} -> {-101.422, -9.23378, 0.299068, 0.299068}, {{368.562, \
395.312}, {28.125, 1.375}} -> {-110.225, -9.23378, 0.299068, 0.299068}, \
{{398, 424.75}, {28.125, 1.375}} -> {-119.029, -9.23378, 0.299068, 0.299068}, \
{{427.375, 454.188}, {28.125, 1.375}} -> {-127.824, -9.23378, 0.299068, \
0.299068}, {{456.812, 483.625}, {28.125, 1.375}} -> {-136.628, -9.23378, \
0.299068, 0.299068}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .12 
%%ImageSize: 500 60 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0595238 0.00285714 0.0595238 [
[ 0 0 0 0 ]
[ 1 .12 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .12 L
0 .12 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.00285714 0.0782313 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
093530615135134B
163B2058432B1735
00232469490B0F1A
12182C564B0D1D2B
10284F6E56062635
112C29C6AD1F322D
1F304CFF9C31255F
2724343E3C183725
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0836735 0.00285714 0.138095 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1E07110B17050A51
2616141121050B48
2320150824110C31
1E3128264E231235
0619292E806E422E
000F2A3A9DFF6827
131A1D1D739B3C54
1A171C2A35372138
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.143537 0.00285714 0.197959 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0F0902020B040B27
020B030008090210
090805070F03101D
0C140D0C0707051D
091311090E0A0C16
08120C0E0F10021D
0A0D0B0F130A1617
461B220F1B1F0FFF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.203401 0.00285714 0.257823 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
76F62A662FB49661
6CFF095D177CD052
34F208685F75D251
2AAD176A5E50EE30
1F6C044A6925B316
33650C6E3818A440
3259006A37146E72
916D224E29916446
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.263265 0.00285714 0.317687 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
A9AAAF4F3E2F3B6B
829AB75647002E59
BDC6C83E2E3A306B
B3C3BF55630F3E5D
FFBEBF563D2E429C
92A8A62A38242E64
C496A533372A5073
CBDEA6524837475D
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.323129 0.00285714 0.377551 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
371C12810D405B0F
40250B5A21344D31
6F120A2D03261E0E
8714091805391859
D35415250050214D
FFAD20241F6D1776
ABEF1F4B2672457D
4C4F2C6F2A584E37
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.382993 0.00285714 0.437415 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0E04062724213128
0400092F56213F4A
0C091141766E634A
231B1055CAFF7A41
252106213F654E56
21261E1D2B21425F
2118211721243D3D
1A0D1A291E404C68
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.442857 0.00285714 0.497279 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1B3B3E3157294EFF
24203C28401C50A8
102024154D24405D
05291D0C2D143240
00351F153103293E
1444130A260E2C2B
1F5836243F0F3156
244135323A3349DD
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.502721 0.00285714 0.557143 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
473E1F1B0D181B6F
543D23191C1924AB
2D1B181A0E181A77
23150007080C0E3E
1E141512140B1B43
492E210F19242CAD
60593724242E30FF
452A301F2F293073
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.562585 0.00285714 0.617007 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
A31E3A0BAD410F34
FF124D18B0650B40
C3055F2B96CF1820
DA0C41328EBA1C58
8F0E504150F40358
58002E32429C0B71
2E00612110810463
530A392E362B128C
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.622449 0.00285714 0.676871 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
2F2962000A0D3D34
4585FF271F174339
2054D93B25294030
15183334322B1B40
0720351539340D3E
070D0A082422043D
08150B091937093E
1D1352180F1B255A
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.682313 0.00285714 0.736735 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0250001B77123524
096E0A06481D2A2F
05740A1334002F24
0482120C280A2E5A
0E77210B25074174
0E634D1B3E103FD6
095258205E154FFF
163E2C2E571C5F66
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.742177 0.00285714 0.796599 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0F050D19170B060C
0308040D0901060E
0E0509180E020B17
0A00080E05011519
120C0C100607101A
05010B0802040E21
0F0B100C07080F16
642F3BFF3945695C
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.802041 0.00285714 0.856463 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
3300371805590D5A
2C0A4B1B174D115C
22037F1C0A460129
4405845B163B0936
3B1FA198194D0514
472F80FF26362120
5D365DEB116A251C
5313585A1B6E2045
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.861905 0.00285714 0.916327 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
2E7948262504102F
132E141E351A161F
0724160B270E0E26
0E2807071300181F
01100514160A1422
04030A0506000422
221109010D0F1225
55FF693A210E212E
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.921769 0.00285714 0.97619 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
122F004B10112B80
1A1F0271110620A0
103C10663B1132FF
0934115B631741DA
062F284C7E184BE7
04342C478B0F4DB3
042932236D0E4E8A
0A1C172F28072F2C
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.0627211 0.0782313 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
101A1B3270C29551
00232D3E47FFAD1D
20171C396DF6A293
050D2B3230D49746
101B225588FF9F7A
1111222004E88C1F
06161B0F41F59226
2A181B1F57A56A3C
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0836735 0.0627211 0.138095 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
2F134D0F3D321FCB
0D0B3D07342A20C6
4F132D036711008E
8015371F6B301E78
B604401EA55B0F37
FF0F41238B702645
D6044D2F81A41036
572376235243387E
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.143537 0.0627211 0.197959 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1904260B1904103F
050A0F091504073E
12291F081B021C49
143620001D081E2C
39526217291D3547
4E85FF2C21183449
404BC93F32133538
3B2630132C151E4A
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.203401 0.0627211 0.257823 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
842A0B4100350D2C
9A5C102E1364146C
85A71A340D582F60
97C30B3E136A4A77
5FFF09472F6B6670
379C134F304D9744
249102284C428527
33441C2D2036451B
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.263265 0.0627211 0.317687 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1B280F3823333D8D
3C06191B3F2C4783
1023033C334B407D
27001B2C4029488A
082C14383D403F88
340B202E56225272
151B19303B233A70
985C6243798BC4FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.323129 0.0627211 0.377551 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
541D6CD016331D1D
361C56FF292F1423
3E1D2A990E3F322C
3B12075F06362F25
3916103B002D3821
4125173E0A3C2838
461A17380930172F
54137365192B1A32
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.382993 0.0627211 0.437415 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
2A180F1E2964875C
14090B021E323D32
32130404072D4836
0909020408162645
18070D0007161E20
1B0609090A0C281D
0B0809021F172224
2616292D2EBEFF55
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.442857 0.0627211 0.497279 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
15361B74560B29FF
0840124A8F1941D5
0A2F2A477B0A4ACC
133A212985093990
062A1D1152064849
0F3D0C073B01274B
004011112605383B
21280C6434051C5B
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.502721 0.0627211 0.557143 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
3C27272426191871
5F40252014121C72
8066434D35111DB2
8E6D4B4E291928CD
6F624644241927FF
4C43233317001C69
4037172121142A4A
573D363238202E78
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.562585 0.0627211 0.617007 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
6C146B2559DD175F
750C4F3365FF1B7C
5122532C2DD311B3
2504431B16740774
23034D17043E0082
1A05601B093D114E
3108632F1148103D
701E7122596D1B43
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.622449 0.0627211 0.676871 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
12373B0D1D013831
0F2E4A0519013C1B
37285E0923004447
3762FF1820115124
35369B122512454A
1B0F66071D1D2141
190F5A0720151341
0A2C160E1F231931
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.682313 0.0627211 0.736735 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
104E062A11256124
163110370E125427
1422003F08183D49
221F0D6500102477
222F0F79171D26F0
1B251062351931D0
213B1F68402136FF
1F4B11322D3F494A
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.742177 0.0627211 0.796599 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
112334141B19223B
0E0F18100B09121E
0D2131141E17192B
001F15100E1A0921
0419181010180E18
061E0C0B130F0423
0E231F1416180A1F
6BCAFF39EDA6607E
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.802041 0.0627211 0.856463 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
410686431155063E
4B24846212420D26
7F2C5FEF0B431425
434996BF18411E0C
77353EFF0E45442F
443D2C660A39360F
66260E880020803B
2A3729411A1A262E
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.861905 0.0627211 0.916327 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1011253E89401A37
0A031A2C40462E2A
0100131E382B2A36
06001B211A1A1A38
0808111C1B262823
0511111625171932
08011C0D1A181216
14243066FF3B2731
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.921769 0.0627211 0.97619 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.125 0 0.125 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
8 string
8 8 8 [8 0 0 8 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0D7959092815539F
0863590A40254DE3
0C51690A382C43D6
06226E0A333641FF
03285E0B3C3B25D5
0A0F490A333B28B9
1015370041251187
0D663D0F3B272551
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{500, 60},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007d0000?2000`400?l00000o`00003ooooooooe
ooooo`00oooooooeooooo`00oooooooeooooo`003oooool3o`T92@Co=CDe0ol`<304of5QH@?oDE5A
1?le=CD3oa<C4`?oBd];0_ooool4oahN7P?o1`L71?lA4A43o`/;2`Co5aLG0ol51@D4o`X:2P;oDE5A
0oooool3o`l?3`Co2@T91ol20P83o`/;2`Co10@40ol;2`/2obLW9`?ooooo1?mfMWH3ooKfmPCo:RXZ
0omVIVH3obl_;`Co];Bd0onFUYH3of5QH@?ooooo0onYZJT4ojZZZP?o[jn_0om?Cdl4ochn?P?o;bl_
1?lk>c/2of][J`?ooooo0olg=cL4oa`L70?o4Q8B1?n1PH43o`d=3@Co@4100omKFe/3o`l?3`;ooooo
1?l>3Ph3o`@410Co1PH60olW9bL4ob@T90?o8B4Q1?la<C42obPX:0?ooooo0olK6a/4oc/k>`?o?Shn
1?la<C43oeMGE`Co:BTY0om>CTh5ooooo`CoAdM70oln?Sh4oalO7`?o6a/K1?l=3@d3oaPH60Co6a/K
0_m_Kfl3ooooo`?oXj>S1?lN7Qh3ocXj>PCo2`/;0on][Jd4od51@@?o3`l?0old=3@2ooooo`Co;bl_
0olY:BT4of9RHP?o00001?l:2PX3o`d=3@?o?Cdm0old=3@3ooooo`?o0P821?m@D503o`0000Co6a/K
0omgMgL3oa8B4PCo=CDe0_lT92@3ooooo`Co3`l?0ol51@D3o`d=3@Co6ATI0olG5aL4o`/;2`?o1PH6
0ol<30`3ooooo`?o<c<c0ol00004ocLg=`?o61PH1?l51@D3oeUIF@Co3@d=0_mJFUX3ooooo`?o;Rh^
1?miNGT3odQ8B0Co9RHV0olU9BD4o`@410?o410@0ol_;bl2ooooo`Co4Q8B0ol_;bl4o`0000?oBd];
1?l@4103oa4A4@Co:b/[0_n0P80Aooooo`003oooool3o`T92@Co=CDe0ol`<304of5QH@?oDE5A1?le
=CD3oa<C4`?oBd];0_ooool4oahN7P?o1`L71?lA4A43o`/;2`Co5aLG0ol51@D4o`X:2P;oDE5A0ooo
ool3o`l?3`Co2@T91ol20P83o`/;2`Co10@40ol;2`/2obLW9`?ooooo1?mfMWH3ooKfmPCo:RXZ0omV
IVH3obl_;`Co];Bd0onFUYH3of5QH@?ooooo0onYZJT4ojZZZP?o[jn_0om?Cdl4ochn?P?o;bl_1?lk
>c/2of][J`?ooooo0olg=cL4oa`L70?o4Q8B1?n1PH43o`d=3@Co@4100omKFe/3o`l?3`;ooooo1?l>
3Ph3o`@410Co1PH60olW9bL4ob@T90?o8B4Q1?la<C42obPX:0?ooooo0olK6a/4oc/k>`?o?Shn1?la
<C43oeMGE`Co:BTY0om>CTh5ooooo`CoAdM70oln?Sh4oalO7`?o6a/K1?l=3@d3oaPH60Co6a/K0_m_
Kfl3ooooo`?oXj>S1?lN7Qh3ocXj>PCo2`/;0on][Jd4od51@@?o3`l?0old=3@2ooooo`Co;bl_0olY
:BT4of9RHP?o00001?l:2PX3o`d=3@?o?Cdm0old=3@3ooooo`?o0P821?m@D503o`0000Co6a/K0omg
MgL3oa8B4PCo=CDe0_lT92@3ooooo`Co3`l?0ol51@D3o`d=3@Co6ATI0olG5aL4o`/;2`?o1PH60ol<
30`3ooooo`?o<c<c0ol00004ocLg=`?o61PH1?l51@D3oeUIF@Co3@d=0_mJFUX3ooooo`?o;Rh^1?mi
NGT3odQ8B0Co9RHV0olU9BD4o`@410?o410@0ol_;bl2ooooo`Co4Q8B0ol_;bl4o`0000?oBd];1?l@
4103oa4A4@Co:b/[0_n0P80Aooooo`003oooool3o`T92@Co=CDe0ol`<304of5QH@?oDE5A1?le=CD3
oa<C4`?oBd];0_ooool4oahN7P?o1`L71?lA4A43o`/;2`Co5aLG0ol51@D4o`X:2P;oDE5A0oooool3
o`l?3`Co2@T91ol20P83o`/;2`Co10@40ol;2`/2obLW9`?ooooo1?mfMWH3ooKfmPCo:RXZ0omVIVH3
obl_;`Co];Bd0onFUYH3of5QH@?ooooo0onYZJT4ojZZZP?o[jn_0om?Cdl4ochn?P?o;bl_1?lk>c/2
of][J`?ooooo0olg=cL4oa`L70?o4Q8B1?n1PH43o`d=3@Co@4100omKFe/3o`l?3`;ooooo1?l>3Ph3
o`@410Co1PH60olW9bL4ob@T90?o8B4Q1?la<C42obPX:0?ooooo0olK6a/4oc/k>`?o?Shn1?la<C43
oeMGE`Co:BTY0om>CTh5ooooo`CoAdM70oln?Sh4oalO7`?o6a/K1?l=3@d3oaPH60Co6a/K0_m_Kfl3
ooooo`?oXj>S1?lN7Qh3ocXj>PCo2`/;0on][Jd4od51@@?o3`l?0old=3@2ooooo`Co;bl_0olY:BT4
of9RHP?o00001?l:2PX3o`d=3@?o?Cdm0old=3@3ooooo`?o0P821?m@D503o`0000Co6a/K0omgMgL3
oa8B4PCo=CDe0_lT92@3ooooo`Co3`l?0ol51@D3o`d=3@Co6ATI0olG5aL4o`/;2`?o1PH60ol<30`3
ooooo`?o<c<c0ol00004ocLg=`?o61PH1?l51@D3oeUIF@Co3@d=0_mJFUX3ooooo`?o;Rh^1?miNGT3
odQ8B0Co9RHV0olU9BD4o`@410?o410@0ol_;bl2ooooo`Co4Q8B0ol_;bl4o`0000?oBd];1?l@4103
oa4A4@Co:b/[0_n0P80Aooooo`003oooool3oaHF5PCo>c/k0olP8204oeQHF0?o@d=31?l[:b/3oaLG
5`?o=CDe0_ooool4obHV9P?o5QHF1?lD51@3oa4A4@Co8B4Q0ol51@D4o`/;2`;oB4Q80oooool3o`82
0PCo2`/;0ol30`<4o`0000?o20P81?l92@T3o`820P;o410@0oooool4ofa/K0?ooooo1?l92@T3oeeM
G@?o5aLG1?mlO7`3om3@d0?oDU9B0oooool3oh:2PPCoVYZJ0ong]kL3oeIFEPCoAdM70ol00004obh^
;P;oFEUI0oooool3od10@0Co9BDU0ol;2`/4oeYJFP?o8B4Q1?ld=3@3ode=C@?o<C4a0_ooool4o`@4
10?o00001?l92@T3obl_;`CoEUIF0olQ8B44oclo?`;oBTY:0oooool3ob@T90Co820P0oll?3`4obPX
:0?o@4101?lL71`3oe1@D0?oZ:RX0_ooool4oeADE0?o?Cdm1?lS8b<3oaTI6@Co71`L0olI6AT4ob@T
90;oZj^[1_ooool4oa8B4P?oCDe=1?lH61P3ok2`/0CoIFEU0ol;2`/3od10@0;ooooo1?m5ADD3ohF5
Q@Cooooo0olW9bL4oalO7`?o5aLG0om3@d<3ocTi>@?ooooo0ol92@T4ofi^KP?o2PX:1?l61PH3odQ8
B0?o7AdM1?lZ:RX2obl_;`?ooooo1?l30`<3o`P820?o10@41?l=3@d3o`T92@Co0@410ol61PH3o`h>
3P?ooooo0ol/;2`3o`X:2PCoBd];0olK6a/4oaLG5`?oCDe=1?lA4A42oeaLG0?ooooo0olC4a<4obh^
;P?o51@D1?lN7Qh3ocDe=@Co6QXJ0olF5QH3oalO7`;ooooo1?lJ6QX3oalO7`Co0P820omaLG44oa4A
4@?o1PH61?lP8202oj2PX17ooooo000?ooooo`?o5QHF1?lk>c/3ob0P80CoF5QH0om3@d<4ob/[:`?o
5aLG0ole=CD2ooooo`Co9RHV0olF5QH4oa@D50?o4A4A1?lQ8B43o`D51@Co2`/;0_m8B4P3ooooo`?o
0P821?l;2`/3o`<30`Co00000ol820P4o`T92@?o0P820_l@4103ooooo`CoK6a/0oooool4o`T92@?o
GEeM0olG5aL4ogalO0?od=3@0omBDU83ooooo`?oPX:21?nJVYX3okNg]`?oEUIF1?m7AdL3o`0000Co
;Rh^0_mIFET3ooooo`?o@4101?lU9BD3o`/;2`CoFUYJ0olQ8B44oc@d=0?oCDe=0ola<C42ooooo`Co
10@40ol00004o`T92@?o;bl_1?mFEUH3ob4Q8@Co?clo0_m:BTX3ooooo`?o92@T1?lP8203oc`l?0Co
:2PX0om0@404oa`L70?oD51@0onXZ:P2ooooo`CoE5AD0olm?Cd4ob<S8`?o6ATI1?lL71`3oaTI6@Co
92@T0_n[Zj/6ooooo`Co4Q8B0om=CDd4oaPH60?o/;2`1?mUIFD3o`/;2`?o@4100_ooool4odE5A@?o
QHF51?ooool3obLW9`Co7alO0olG5aL3od=3@`?o>CTi0oooool3o`T92@CoKVi^0ol:2PX4o`H61P?o
B4Q80olM7Ad4obXZ:P;o;bl_0oooool4o`<30`?o20P80ol410@4o`d=3@?o2@T91?l10@43o`H61P?o
3Ph>0oooool3ob`/;0?o2PX:1?m;Bd/3oa/K6`Co5aLG0om=CDd4oa4A4@;oG5aL0oooool3oa<C4`Co
;Rh^0olD51@4oahN7P?o=CDe1?lJ6QX3oaHF5P?o7alO0_ooool4oaXJ6P?o7alO1?l20P83og5aL@Co
4A4A0ol61PH4ob0P80;oX:2P4Oooool000oooooo0olF5QH4oc/k>`?o820P1?mHF5P3od=3@`Co:b/[
0olG5aL3ocDe=@;ooooo1?lV9RH3oaHF5PCo51@D0olA4A44ob4Q8@?o1@D51?l;2`/2odQ8B0?ooooo
0ol20P84o`/;2`?o0`<31?l00003o`P820Co2@T90ol20P82oa0@40?ooooo1?m/K6`3ooooo`Co2@T9
0omMGEd3oaLG5`CoO7al0oo@d=03oe9BDP?ooooo0on2PX84oiZJVP?o]kNg0omFEUH4odM7A`?o0000
1?l^;Rh2oeUIF@?ooooo0om0@404obDU9@?o2`/;1?mJFUX3ob4Q8@Co=3@d0om=CDd3oc4a<@;ooooo
1?l410@3o`0000Co2@T90ol_;bl4oeIFEP?o8B4Q1?lo?cl2odY:BP?ooooo0olT92@4ob0P80?o?3`l
1?lX:2P3od10@0Co71`L0om@D503ojRXZ0;ooooo1?mDE5@3ocdm?@Co8b<S0olI6AT4oa`L70?o6ATI
1?lT92@2oj^[Z`Kooooo1?lB4Q83ode=C@Co61PH0on`/;04ofEUI@?o2`/;0om0@402ooooo`CoADE5
0on5QHD4ooooo`?o9bLW1?lO7al3oaLG5`?o@d=30oli>CT3ooooo`?o2@T91?m^KVh3o`X:2PCo1PH6
0om8B4P3oadM7@Co:RXZ0_l_;bl3ooooo`Co0`<30ol820P3o`@410Co3@d=0ol92@T4o`410@?o1PH6
0ol>3Ph3ooooo`?o;2`/0ol:2PX4od];B`?o6a/K1?lG5aL3ode=C@Co4A4A0_mLG5`3ooooo`?o4a<C
1?l^;Rh3oa@D50Co7QhN0ole=CD4oaXJ6P?o5QHF0olO7al2ooooo`Co6QXJ0olO7al4o`820P?oLG5a
1?lA4A43o`H61PCo820P0_nPX:0Aooooo`003oooool3o`0000Co8b<S0olT92@4ofUYJ@?oBDU91?l;
2`/3o`l?3`?o6QXJ0_ooool4ob<S8`?o820P1?lE5AD3o`P820Co92@T0olA4A44o``<30;o<C4a0ooo
ool3o`T92@Co20P80ol51@D4o`L71`?o3`l?1?l30`<3oa0@40;o7AdM0oooool4oc@d=0?ol_;b1?l8
20P3ofQXJ0?oGemO1?meMGD3om;BdP?oDE5A0oooool3okfm_@Coa/K60oo8b<P3ochn?PCo;Rh^0olj
>SX4oc0`<0;oJf][0oooool3ofm_K`Co4Q8B0ol:2PX4obd];@?o0`<31?lV9RH3oahN7P?o3Ph>0_oo
ool4o``<30?o2@T91?lA4A43od51@@CoMWIf0om^KVh4of=SH`;oBTY:0oooool3oa0@40Co820P0olT
92@4oaDE5@?oCDe=1?lT92@3od10@0?oGEeM0_ooool4obd];@?o6a/K1?lH61P3oaXJ6PCo3Ph>0olH
61P4oaXJ6P;oMgMg0oooool3ol?3``Co1@D50omOGel4ob/[:`?oUYJF1?o?cll3oaPH60?o820P0_oo
ool4ob0P80?oE5AD1?oIfMT3oc/k>`Co9BDU0olY:BT3od10@0?o<30`0oooool3o`D51@CoM7Ad0ol:
2PX4oa<C4`?o=3@d0ol00004obl_;`;o92@T0oooool4o`h>3P?o1@D50ol92@T4oaPH60?o3Ph>1?l2
0P83o`/;2`?o5aLG0oooool3ob8R8P?o0`<31?moOgl3oa`L70Co2PX:0om6ATH4o`410@;o:BTY0ooo
ool3o`L71`Co92@T0olF5QH4o`/;2`?o9bLW1ol>3Ph3obHV9P;ooooo1?l@4103oc`l?0Co410@0omV
IVH4oc/k>`?o4A4A1?lb<S8Cooooo`003oooool3o`0000Co8b<S0olT92@4ofUYJ@?oBDU91?l;2`/3
o`l?3`?o6QXJ0_ooool4ob<S8`?o820P1?lE5AD3o`P820Co92@T0olA4A44o``<30;o<C4a0oooool3
o`T92@Co20P80ol51@D4o`L71`?o3`l?1?l30`<3oa0@40;o7AdM0oooool4oc@d=0?ol_;b1?l820P3
ofQXJ0?oGemO1?meMGD3om;BdP?oDE5A0oooool3okfm_@Coa/K60oo8b<P3ochn?PCo;Rh^0olj>SX4
oc0`<0;oJf][0oooool3ofm_K`Co4Q8B0ol:2PX4obd];@?o0`<31?lV9RH3oahN7P?o3Ph>0_ooool4
o``<30?o2@T91?lA4A43od51@@CoMWIf0om^KVh4of=SH`;oBTY:0oooool3oa0@40Co820P0olT92@4
oaDE5@?oCDe=1?lT92@3od10@0?oGEeM0_ooool4obd];@?o6a/K1?lH61P3oaXJ6PCo3Ph>0olH61P4
oaXJ6P;oMgMg0oooool3ol?3``Co1@D50omOGel4ob/[:`?oUYJF1?o?cll3oaPH60?o820P0_ooool4
ob0P80?oE5AD1?oIfMT3oc/k>`Co9BDU0olY:BT3od10@0?o<30`0oooool3o`D51@CoM7Ad0ol:2PX4
oa<C4`?o=3@d0ol00004obl_;`;o92@T0oooool4o`h>3P?o1@D50ol92@T4oaPH60?o3Ph>1?l20P83
o`/;2`?o5aLG0oooool3ob8R8P?o0`<31?moOgl3oa`L70Co2PX:0om6ATH4o`410@;o:BTY0oooool3
o`L71`Co92@T0olF5QH4o`/;2`?o9bLW1ol>3Ph3obHV9P;ooooo1?l@4103oc`l?0Co410@0omVIVH4
oc/k>`?o4A4A1?lb<S8Cooooo`003oooool3o`0000Co8b<S0olT92@4ofUYJ@?oBDU91?l;2`/3o`l?
3`?o6QXJ0_ooool4ob<S8`?o820P1?lE5AD3o`P820Co92@T0olA4A44o``<30;o<C4a0oooool3o`T9
2@Co20P80ol51@D4o`L71`?o3`l?1?l30`<3oa0@40;o7AdM0oooool4oc@d=0?ol_;b1?l820P3ofQX
J0?oGemO1?meMGD3om;BdP?oDE5A0oooool3okfm_@Coa/K60oo8b<P3ochn?PCo;Rh^0olj>SX4oc0`
<0;oJf][0oooool3ofm_K`Co4Q8B0ol:2PX4obd];@?o0`<31?lV9RH3oahN7P?o3Ph>0_ooool4o``<
30?o2@T91?lA4A43od51@@CoMWIf0om^KVh4of=SH`;oBTY:0oooool3oa0@40Co820P0olT92@4oaDE
5@?oCDe=1?lT92@3od10@0?oGEeM0_ooool4obd];@?o6a/K1?lH61P3oaXJ6PCo3Ph>0olH61P4oaXJ
6P;oMgMg0oooool3ol?3``Co1@D50omOGel4ob/[:`?oUYJF1?o?cll3oaPH60?o820P0_ooool4ob0P
80?oE5AD1?oIfMT3oc/k>`Co9BDU0olY:BT3od10@0?o<30`0oooool3o`D51@CoM7Ad0ol:2PX4oa<C
4`?o=3@d0ol00004obl_;`;o92@T0oooool4o`h>3P?o1@D50ol92@T4oaPH60?o3Ph>1?l20P83o`/;
2`?o5aLG0oooool3ob8R8P?o0`<31?moOgl3oa`L70Co2PX:0om6ATH4o`410@;o:BTY0oooool3o`L7
1`Co92@T0olF5QH4o`/;2`?o9bLW1ol>3Ph3obHV9P;ooooo1?l@4103oc`l?0Co410@0omVIVH4oc/k
>`?o4A4A1?lb<S8Cooooo`003oooool3o`0000Co8b<S0olT92@4ofUYJ@?oBDU91?l;2`/3o`l?3`?o
6QXJ0_ooool4ob<S8`?o820P1?lE5AD3o`P820Co92@T0olA4A44o``<30;o<C4a0oooool3o`T92@Co
20P80ol51@D4o`L71`?o3`l?1?l30`<3oa0@40;o7AdM0oooool4oc@d=0?ol_;b1?l820P3ofQXJ0?o
GemO1?meMGD3om;BdP?oDE5A0oooool3okfm_@Coa/K60oo8b<P3ochn?PCo;Rh^0olj>SX4oc0`<0;o
Jf][0oooool3ofm_K`Co4Q8B0ol:2PX4obd];@?o0`<31?lV9RH3oahN7P?o3Ph>0_ooool4o``<30?o
2@T91?lA4A43od51@@CoMWIf0om^KVh4of=SH`;oBTY:0oooool3oa0@40Co820P0olT92@4oaDE5@?o
CDe=1?lT92@3od10@0?oGEeM0_ooool4obd];@?o6a/K1?lH61P3oaXJ6PCo3Ph>0olH61P4oaXJ6P;o
MgMg0oooool3ol?3``Co1@D50omOGel4ob/[:`?oUYJF1?o?cll3oaPH60?o820P0_ooool4ob0P80?o
E5AD1?oIfMT3oc/k>`Co9BDU0olY:BT3od10@0?o<30`0oooool3o`D51@CoM7Ad0ol:2PX4oa<C4`?o
=3@d0ol00004obl_;`;o92@T0oooool4o`h>3P?o1@D50ol92@T4oaPH60?o3Ph>1?l20P83o`/;2`?o
5aLG0oooool3ob8R8P?o0`<31?moOgl3oa`L70Co2PX:0om6ATH4o`410@;o:BTY0oooool3o`L71`Co
92@T0olF5QH4o`/;2`?o9bLW1ol>3Ph3obHV9P;ooooo1?l@4103oc`l?0Co410@0omVIVH4oc/k>`?o
4A4A1?lb<S8Cooooo`003oooool3oa8B4PCo61PH0ol/;2`4oeIFEP?oBd];1?l=3@d3oadM7@?o:b/[
0_ooool4oahN7P?o<C4a1?lX:2P3obHV9PCoCTi>0olS8b<4oa8B4P;o=CDe0oooool3o``<30Co51@D
0ol=3@d4o``<30Oo1`L70ol51@D2oadM7@?ooooo1?lZ:RX3ojf][@Co5aLG0omZJVX3oeiNGPCoD51@
0oo^k^h3oc0`<0?ooooo0onc/k<4ol?3``?o_kno0omEEED4of=SH`?o3`l?1?ln?Sh2oeeMG@?ooooo
0on7QhL4oa@D50?o2@T91?lH61P3o`D51@Co>CTi0olH61P3oeUIF@;ooooo1?lS8b<3oa/K6`Co410@
0omEEED4ol[:bP?ooooo1?mjNWX2od51@@?ooooo0ol51@D4obTY:@?o7AdM1?l<30`3obd];@Co51@D
0olb<S83od10@0;ooooo1?lS8b<3oaDE5@Co00000ol71`L4o`P820?o30`<1?l>3Ph2ochn?P?ooooo
0ooJf]X4o``<30?o@D511?lb<S83ohj>SPCo^[Zj0olL71`3oeQHF0;ooooo1?lE5AD3oaPH60Co<c<c
0old=3@4oc8b<P?o:b/[0olK6a/3od10@0?ooooo0ol410@4oh:2PP?o4Q8B1?l<30`3obPX:0?o2PX:
1?l^;Rh2oeYJFP?ooooo1?l:2PX3o`0000?o20P81?l>3Ph3o`D51@Co0@410olE5AD3oaTI6@?ooooo
0om4A4@3o`D51@CoQ8B40omKFe/4oaHF5P?o>c/k1?l92@T2ocHf=P?ooooo0ol>3Ph4obPX:0Oo1`L7
0olC4a<4o`0000?o61PH0olO7al2ooooo`Co2@T90old=3@4oa4A4@?oFe]K1?mSHf<3oaLG5`Co@D51
0_oJf]XAooooo`003oooool3oa8B4PCo61PH0ol/;2`4oeIFEP?oBd];1?l=3@d3oadM7@?o:b/[0_oo
ool4oahN7P?o<C4a1?lX:2P3obHV9PCoCTi>0olS8b<4oa8B4P;o=CDe0oooool3o``<30Co51@D0ol=
3@d4o``<30Oo1`L70ol51@D2oadM7@?ooooo1?lZ:RX3ojf][@Co5aLG0omZJVX3oeiNGPCoD51@0oo^
k^h3oc0`<0?ooooo0onc/k<4ol?3``?o_kno0omEEED4of=SH`?o3`l?1?ln?Sh2oeeMG@?ooooo0on7
QhL4oa@D50?o2@T91?lH61P3o`D51@Co>CTi0olH61P3oeUIF@;ooooo1?lS8b<3oa/K6`Co410@0omE
EED4ol[:bP?ooooo1?mjNWX2od51@@?ooooo0ol51@D4obTY:@?o7AdM1?l<30`3obd];@Co51@D0olb
<S83od10@0;ooooo1?lS8b<3oaDE5@Co00000ol71`L4o`P820?o30`<1?l>3Ph2ochn?P?ooooo0ooJ
f]X4o``<30?o@D511?lb<S83ohj>SPCo^[Zj0olL71`3oeQHF0;ooooo1?lE5AD3oaPH60Co<c<c0old
=3@4oc8b<P?o:b/[0olK6a/3od10@0?ooooo0ol410@4oh:2PP?o4Q8B1?l<30`3obPX:0?o2PX:1?l^
;Rh2oeYJFP?ooooo1?l:2PX3o`0000?o20P81?l>3Ph3o`D51@Co0@410olE5AD3oaTI6@?ooooo0om4
A4@3o`D51@CoQ8B40omKFe/4oaHF5P?o>c/k1?l92@T2ocHf=P?ooooo0ol>3Ph4obPX:0Oo1`L70olC
4a<4o`0000?o61PH0olO7al2ooooo`Co2@T90old=3@4oa4A4@?oFe]K1?mSHf<3oaLG5`Co@D510_oJ
f]XAooooo`003oooool3oa8B4PCo61PH0ol/;2`4oeIFEP?oBd];1?l=3@d3oadM7@?o:b/[0_ooool4
oahN7P?o<C4a1?lX:2P3obHV9PCoCTi>0olS8b<4oa8B4P;o=CDe0oooool3o``<30Co51@D0ol=3@d4
o``<30Oo1`L70ol51@D2oadM7@?ooooo1?lZ:RX3ojf][@Co5aLG0omZJVX3oeiNGPCoD51@0oo^k^h3
oc0`<0?ooooo0onc/k<4ol?3``?o_kno0omEEED4of=SH`?o3`l?1?ln?Sh2oeeMG@?ooooo0on7QhL4
oa@D50?o2@T91?lH61P3o`D51@Co>CTi0olH61P3oeUIF@;ooooo1?lS8b<3oa/K6`Co410@0omEEED4
ol[:bP?ooooo1?mjNWX2od51@@?ooooo0ol51@D4obTY:@?o7AdM1?l<30`3obd];@Co51@D0olb<S83
od10@0;ooooo1?lS8b<3oaDE5@Co00000ol71`L4o`P820?o30`<1?l>3Ph2ochn?P?ooooo0ooJf]X4
o``<30?o@D511?lb<S83ohj>SPCo^[Zj0olL71`3oeQHF0;ooooo1?lE5AD3oaPH60Co<c<c0old=3@4
oc8b<P?o:b/[0olK6a/3od10@0?ooooo0ol410@4oh:2PP?o4Q8B1?l<30`3obPX:0?o2PX:1?l^;Rh2
oeYJFP?ooooo1?l:2PX3o`0000?o20P81?l>3Ph3o`D51@Co0@410olE5AD3oaTI6@?ooooo0om4A4@3
o`D51@CoQ8B40omKFe/4oaHF5P?o>c/k1?l92@T2ocHf=P?ooooo0ol>3Ph4obPX:0Oo1`L70olC4a<4
o`0000?o61PH0olO7al2ooooo`Co2@T90old=3@4oa4A4@?oFe]K1?mSHf<3oaLG5`Co@D510_oJf]XA
ooooo`003oooool3oa0@40Co:2PX0om?Cdl4ofi^KP?oEUIF1?l61PH3obHV9P?o=CDe0_ooool4o`H6
1P?o6ATI1?lY:BT3obh^;PCoP8200om^KVh4od92@P;o;Rh^0oooool3o`T92@Co4a<C0olA4A44o`T9
2@?o3Ph>1?l:2PX3o``<30;o5QHF0oooool4oalO7`?oK6a/1?l410@3odY:BP?oJFUY1?lU9BD3ok>c
/`?o5QHF1_ooool4okjn_P?o_kno0omFEUH4ocdm?@?o;Rh^1?m2@T82oibLW0?ooooo0ooCdm<4oeAD
E0?o5ADE1?lU9BD3o`0000CoD51@0olQ8B43ode=C@;ooooo1?lU9BD3ob4Q8@Co1PH60olQ8B44oclo
?`?oIFEU1?m>CTh2oeIFEP?ooooo0ol00004ocDe=@?o7alO1?lE5AD3oc4a<@Co0`<30olY:BT3ochn
?P;ooooo1?lN7Qh3oa@D50Co5ADE0olB4Q84oa@D50?o2`/;1?lK6a/2od=3@`?ooooo0on?Shl4o`h>
3P?oD51@1?m1@D43oe1@D0Com?Cd0ol30`<3oeQHF0;ooooo1?l71`L3ob0P80Co=CDe0olE5AD4ocTi
>@?o=3@d0ol=3@d3ochn?P?ooooo0ol>3Ph4ogMgM`?o8B4Q1?l;2`/3obDU9@?o1`L71?m1@D42ogAd
M0?ooooo1?lB4Q86o``<30Co410@0ol61PH4o`L71`?o410@0olJ6QX3ooooo`?o>c/k0olO7al4oj6Q
X@?oV9RH1?lI6AT3ode=C@Co1@D50_lD51@3ooooo`?o0@411?l@4103o`D51@Co51@D0olF5QH4o`X:
2P?o51@D0olR8R82ooooo`Co1PH60ol_;bl4obPX:0?oC4a<1?mnOWh3oaPH60CoBd];0_oWinLAoooo
o`003oooool3oa0@40Co:2PX0om?Cdl4ofi^KP?oEUIF1?l61PH3obHV9P?o=CDe0_ooool4o`H61P?o
6ATI1?lY:BT3obh^;PCoP8200om^KVh4od92@P;o;Rh^0oooool3o`T92@Co4a<C0olA4A44o`T92@?o
3Ph>1?l:2PX3o``<30;o5QHF0oooool4oalO7`?oK6a/1?l410@3odY:BP?oJFUY1?lU9BD3ok>c/`?o
5QHF1_ooool4okjn_P?o_kno0omFEUH4ocdm?@?o;Rh^1?m2@T82oibLW0?ooooo0ooCdm<4oeADE0?o
5ADE1?lU9BD3o`0000CoD51@0olQ8B43ode=C@;ooooo1?lU9BD3ob4Q8@Co1PH60olQ8B44oclo?`?o
IFEU1?m>CTh2oeIFEP?ooooo0ol00004ocDe=@?o7alO1?lE5AD3oc4a<@Co0`<30olY:BT3ochn?P;o
oooo1?lN7Qh3oa@D50Co5ADE0olB4Q84oa@D50?o2`/;1?lK6a/2od=3@`?ooooo0on?Shl4o`h>3P?o
D51@1?m1@D43oe1@D0Com?Cd0ol30`<3oeQHF0;ooooo1?l71`L3ob0P80Co=CDe0olE5AD4ocTi>@?o
=3@d0ol=3@d3ochn?P?ooooo0ol>3Ph4ogMgM`?o8B4Q1?l;2`/3obDU9@?o1`L71?m1@D42ogAdM0?o
oooo1?lB4Q86o``<30Co410@0ol61PH4o`L71`?o410@0olJ6QX3ooooo`?o>c/k0olO7al4oj6QX@?o
V9RH1?lI6AT3ode=C@Co1@D50_lD51@3ooooo`?o0@411?l@4103o`D51@Co51@D0olF5QH4o`X:2P?o
51@D0olR8R82ooooo`Co1PH60ol_;bl4obPX:0?oC4a<1?mnOWh3oaPH60CoBd];0_oWinLAooooo`00
3oooool3oa0@40Co:2PX0om?Cdl4ofi^KP?oEUIF1?l61PH3obHV9P?o=CDe0_ooool4o`H61P?o6ATI
1?lY:BT3obh^;PCoP8200om^KVh4od92@P;o;Rh^0oooool3o`T92@Co4a<C0olA4A44o`T92@?o3Ph>
1?l:2PX3o``<30;o5QHF0oooool4oalO7`?oK6a/1?l410@3odY:BP?oJFUY1?lU9BD3ok>c/`?o5QHF
1_ooool4okjn_P?o_kno0omFEUH4ocdm?@?o;Rh^1?m2@T82oibLW0?ooooo0ooCdm<4oeADE0?o5ADE
1?lU9BD3o`0000CoD51@0olQ8B43ode=C@;ooooo1?lU9BD3ob4Q8@Co1PH60olQ8B44oclo?`?oIFEU
1?m>CTh2oeIFEP?ooooo0ol00004ocDe=@?o7alO1?lE5AD3oc4a<@Co0`<30olY:BT3ochn?P;ooooo
1?lN7Qh3oa@D50Co5ADE0olB4Q84oa@D50?o2`/;1?lK6a/2od=3@`?ooooo0on?Shl4o`h>3P?oD51@
1?m1@D43oe1@D0Com?Cd0ol30`<3oeQHF0;ooooo1?l71`L3ob0P80Co=CDe0olE5AD4ocTi>@?o=3@d
0ol=3@d3ochn?P?ooooo0ol>3Ph4ogMgM`?o8B4Q1?l;2`/3obDU9@?o1`L71?m1@D42ogAdM0?ooooo
1?lB4Q86o``<30Co410@0ol61PH4o`L71`?o410@0olJ6QX3ooooo`?o>c/k0olO7al4oj6QX@?oV9RH
1?lI6AT3ode=C@Co1@D50_lD51@3ooooo`?o0@411?l@4103o`D51@Co51@D0olF5QH4o`X:2P?o51@D
0olR8R82ooooo`Co1PH60ol_;bl4obPX:0?oC4a<1?mnOWh3oaPH60CoBd];0_oWinLAooooo`003ooo
ool3oa4A4@Co;2`/0olY:BT4olK6aP?o[Jf]1?lO7al3oc8b<P?o;Bd]0_ooool4o`0000?o3`l?1?lZ
:RX3ocXj>PCoWIfM0oooool4ofQXJ0;o9bLW0oooool3o`P820Co4Q8B0ol<30`4o`h>3P?o3`l?1?l@
4103o`820P;o7AdM0oooool4oc<c<`?oIFEU1?l<30`3ofi^KP?o>3Ph1?lH61P3ojBTY0?o@4100ooo
ool3oi:BTPCoZ:RX0onVYZH3obXZ:PCo>3Ph0olT92@4obh^;P;oI6AT1_ooool4ojf][@?o820P1?lT
92@3oalO7`CoKFe]0olG5aL3ogIfMP;ooooo1?lQ8B43obHV9PCo7QhN0olM7Ad4ob/[:`?o8B4Q1?m2
@T82oemOG`?ooooo0olD51@4odA4A0?o4a<C1?l:2PX3obHV9PCo3Ph>0ol/;2`3ob/[:`;ooooo1?m9
BDT3obh^;PCo8B4Q0ol?3`l4oaTI6@?o92@T1?l/;2`2ojf][@?ooooo0omHF5P4o`0000?o;Rh^1?lb
<S83od92@PCoW9bL0ol;2`/3og5aL@;ooooo1?l71`L3o`d=3@Co2PX:0ol820P4ob@T90?o8R8R0ol4
10@3ocdm?@?ooooo0ol>3Ph4of=SH`?oCDe=1?lK6a/3ochn?P?o410@1?lo?cl2omKFeP?ooooo1?l5
1@D3o`410@?o2`/;1?l820P3o`820PCo10@40ol>3Ph3ob4Q8@?ooooo0om7AdL3obl_;`CoP8200ooo
ool4obHV9P?o=SHf1?lQ8B42ob0P80?ooooo0ol410@4o`<30`?o2PX:1?l51@D3o`H61PCo00000ol4
10@3ob8R8P;ooooo1?l410@3oc@d=0Co;2`/0om7AdL4oh^;R`?o3`l?1?m=CDd2ok>c/a7ooooo000?
ooooo`?o4A4A1?l/;2`3obTY:@Coa/K60on][Jd4oalO7`?o<S8b0ol];Bd2ooooo`Co00000ol?3`l4
obXZ:P?o>SXj1?nMWId3ooooo`CoJ6QX0_lW9bL3ooooo`?o20P81?lB4Q83o``<30Co3Ph>0ol?3`l4
oa0@40?o0P820_lM7Ad3ooooo`Co<c<c0omUIFD4o``<30?oKVi^0olh>3P4oaPH60?oY:BT0om0@403
ooooo`?oTY:B1?nXZ:P3ojJVYP?o:RXZ1?lh>3P3ob@T90Co;Rh^0_mTI6@6ooooo`Co[Jf]0olP8204
ob@T90?o7alO1?m]KFd3oaLG5`?oMWIf0_ooool4ob4Q8@?o9RHV1?lN7Qh3oadM7@Co:b/[0olQ8B44
od92@P;oGemO0oooool3oa@D50CoA4A40olC4a<4o`X:2P?o9RHV1?l>3Ph3ob`/;0?o:b/[0_ooool4
odU9B@?o;Rh^1?lQ8B43o`l?3`Co6ATI0olT92@4ob`/;0;o[Jf]0oooool3oeQHF0Co00000ol^;Rh4
oc8b<P?o@T921?nLW9`3o`/;2`?oLG5a0_ooool4o`L71`?o3@d=1?l:2PX3o`P820Co92@T0olR8R83
o`@410?o?Cdm0oooool3o`h>3PCoHf=S0om=CDd4oa/K6`?o?Shn0ol@4104oclo?`;oe]KF0oooool4
o`D51@?o0@410ol;2`/4o`P820?o0P821?l410@3o`h>3P?o8B4Q0oooool3odM7A`?o;bl_1?n0P803
ooooo`Co9RHV0olf=SH4ob4Q8@;o820P0oooool3o`@410Co0`<30ol:2PX4o`D51@?o1PH61?l00003
o`@410?o8R8R0_ooool4o`@410?o=3@d1?l/;2`3odM7A`CoRh^;0ol?3`l4ode=C@;o/k>c4Oooool0
00oooooo0olA4A44ob`/;0?o:BTY1?o6a/H3ojf][@Co7alO0olb<S83obd];@;ooooo1?l00003o`l?
3`Co:RXZ0olj>SX4oifMW@?ooooo1?mXJ6P2obLW9`?ooooo0ol820P4oa8B4P?o30`<1?l>3Ph3o`l?
3`Co410@0ol20P82oadM7@?ooooo1?lc<c<3ofEUI@Co30`<0om^KVh3ocPh>0Co61PH0onTY:@3od10
@0?ooooo0onBTY84ojRXZ0?oYZJV0olZ:RX4ocPh>0?o92@T1?l^;Rh2ofATI0Kooooo1?n][Jd3ob0P
80Co92@T0olO7al4ofe]K@?o5aLG0omfMWH2ooooo`Co8B4Q0olV9RH4oahN7P?o7AdM1?l[:b/3ob4Q
8@Co@T920_mOGel3ooooo`?o51@D1?m4A4@3oa<C4`Co2PX:0olV9RH4o`h>3P?o;2`/0ol[:b/2oooo
o`CoBDU90ol^;Rh4ob4Q8@?o3`l?1?lI6AT3ob@T90Co;2`/0_n][Jd3ooooo`?oF5QH1?l00003obh^
;PCo<S8b0om2@T84oibLW0?o2`/;0omaLG42ooooo`Co1`L70ol=3@d4o`X:2P?o20P81?lT92@3ob8R
8P?o10@40olm?Cd3ooooo`?o3Ph>1?mSHf<3ode=C@Co6a/K0oln?Sh3oa0@40Co?clo0_oFe]H3oooo
o`Co1@D50ol10@43o`/;2`Co20P80ol20P84o`@410?o3Ph>0olQ8B43ooooo`?oAdM70ol_;bl4oh20
P0?ooooo1?lV9RH3ocHf=PCo8B4Q0_lP8203ooooo`?o10@41?l30`<3o`X:2PCo1@D50ol61PH4o`00
00?o10@40olR8R82ooooo`Co10@40old=3@4ob`/;0?oAdM71?n;Rh/3o`l?3`CoCDe=0_nc/k<Aoooo
o`003oooool3oalO7`Co<30`0om<C4`4ooooo`?oW9bL1?la<C43obDU9@?oGemO0_ooool4oa<C4`?o
6QXJ1olM7Ad4og=cL`?oVi^K1?ll?3`2oeADE0?ooooo0ol:2PX4o`d=3@?o2`/;1?l?3`l3oa<C4`Co
2PX:0olF5QH2oaLG5`?ooooo1?lb<S83oeUIF@Co00000omZJVX3ocLg=`Co51@D0om^KVh3og9bLP?o
oooo0oo4a<@4oiJFUP?oYJFU0olc<c<4ocLg=`?o:RXZ1?m@D502og=cL`?ooooo0on[Zj/4ono_k`?o
7alO1?m;Bd/3obHV9PCoLW9b0om5ADD3ogemO@;ooooo1?lQ8B43oaPH60Co8B4Q0olG5aL4ob4Q8@?o
92@T1_lm?Cd3ooooo`?o7alO1?mHF5P3ocHf=PCo92@T0olo?cl4o`l?3`?o<C4a0omFEUH2ooooo`Co
H61P0omIFET4ocLg=`Oo92@T0ol^;Rh4oc0`<0Gooooo0ol^;Rh4o`0000?oHF5Q1?lQ8B43oa0@40Co
PH610ol410@3of=SH`;ooooo1?l820P3oaDE5@Co2`/;0ol92@T4oaTI6@?o=cLg0ol92@T3ochn?P?o
oooo0ol92@T4oe9BDP?oF5QH1?lP8203oeiNGP?o5ADE1?m?Cdl5ooooo`Co3`l?0ol;2`/3oa0@40Co
30`<0ol71`L4o`P820?o3`l?0olF5QH3ooooo`?oGEeM0olf=SH4oeeMG@?ojn_[1?lA4A43ofYZJPCo
9BDU0_lL71`3ooooo`?o8R8R1?lA4A43o`T92@Co0@410ol=3@d4o`l?3`?o4Q8B0olU9BD2ooooo`Co
10@40olY:BT4oc8b<P?o8b<S1?m]KFd3o`h>3PCoCTi>0_n:RXXAooooo`003oooool3oalO7`Co<30`
0om<C4`4ooooo`?oW9bL1?la<C43obDU9@?oGemO0_ooool4oa<C4`?o6QXJ1olM7Ad4og=cL`?oVi^K
1?ll?3`2oeADE0?ooooo0ol:2PX4o`d=3@?o2`/;1?l?3`l3oa<C4`Co2PX:0olF5QH2oaLG5`?ooooo
1?lb<S83oeUIF@Co00000omZJVX3ocLg=`Co51@D0om^KVh3og9bLP?ooooo0oo4a<@4oiJFUP?oYJFU
0olc<c<4ocLg=`?o:RXZ1?m@D502og=cL`?ooooo0on[Zj/4ono_k`?o7alO1?m;Bd/3obHV9PCoLW9b
0om5ADD3ogemO@;ooooo1?lQ8B43oaPH60Co8B4Q0olG5aL4ob4Q8@?o92@T1_lm?Cd3ooooo`?o7alO
1?mHF5P3ocHf=PCo92@T0olo?cl4o`l?3`?o<C4a0omFEUH2ooooo`CoH61P0omIFET4ocLg=`Oo92@T
0ol^;Rh4oc0`<0Gooooo0ol^;Rh4o`0000?oHF5Q1?lQ8B43oa0@40CoPH610ol410@3of=SH`;ooooo
1?l820P3oaDE5@Co2`/;0ol92@T4oaTI6@?o=cLg0ol92@T3ochn?P?ooooo0ol92@T4oe9BDP?oF5QH
1?lP8203oeiNGP?o5ADE1?m?Cdl5ooooo`Co3`l?0ol;2`/3oa0@40Co30`<0ol71`L4o`P820?o3`l?
0olF5QH3ooooo`?oGEeM0olf=SH4oeeMG@?ojn_[1?lA4A43ofYZJPCo9BDU0_lL71`3ooooo`?o8R8R
1?lA4A43o`T92@Co0@410ol=3@d4o`l?3`?o4Q8B0olU9BD2ooooo`Co10@40olY:BT4oc8b<P?o8b<S
1?m]KFd3o`h>3PCoCTi>0_n:RXXAooooo`003oooool3oalO7`Co<30`0om<C4`4ooooo`?oW9bL1?la
<C43obDU9@?oGemO0_ooool4oa<C4`?o6QXJ1olM7Ad4og=cL`?oVi^K1?ll?3`2oeADE0?ooooo0ol:
2PX4o`d=3@?o2`/;1?l?3`l3oa<C4`Co2PX:0olF5QH2oaLG5`?ooooo1?lb<S83oeUIF@Co00000omZ
JVX3ocLg=`Co51@D0om^KVh3og9bLP?ooooo0oo4a<@4oiJFUP?oYJFU0olc<c<4ocLg=`?o:RXZ1?m@
D502og=cL`?ooooo0on[Zj/4ono_k`?o7alO1?m;Bd/3obHV9PCoLW9b0om5ADD3ogemO@;ooooo1?lQ
8B43oaPH60Co8B4Q0olG5aL4ob4Q8@?o92@T1_lm?Cd3ooooo`?o7alO1?mHF5P3ocHf=PCo92@T0olo
?cl4o`l?3`?o<C4a0omFEUH2ooooo`CoH61P0omIFET4ocLg=`Oo92@T0ol^;Rh4oc0`<0Gooooo0ol^
;Rh4o`0000?oHF5Q1?lQ8B43oa0@40CoPH610ol410@3of=SH`;ooooo1?l820P3oaDE5@Co2`/;0ol9
2@T4oaTI6@?o=cLg0ol92@T3ochn?P?ooooo0ol92@T4oe9BDP?oF5QH1?lP8203oeiNGP?o5ADE1?m?
Cdl5ooooo`Co3`l?0ol;2`/3oa0@40Co30`<0ol71`L4o`P820?o3`l?0olF5QH3ooooo`?oGEeM0olf
=SH4oeeMG@?ojn_[1?lA4A43ofYZJPCo9BDU0_lL71`3ooooo`?o8R8R1?lA4A43o`T92@Co0@410ol=
3@d4o`l?3`?o4Q8B0olU9BD2ooooo`Co10@40olY:BT4oc8b<P?o8b<S1?m]KFd3o`h>3PCoCTi>0_n:
RXXAooooo`003oooool3obLW9`Co92@T0old=3@4ochn?P?o?3`l1?lH61P3ocLg=`?o9BDU0_ooool4
oaXJ6P?o5aLG1?lL71`3obXZ:PCo=CDe0olg=cL4ob4Q8@;o>3Ph0oooool3odI6APCo6a/K0olR8R84
o`l?3`?o6a/K1?lO7al3o`l?3`Gooooo1?nATI43ofe]K@Co8R8R0om>CTh3obTY:@CoTI6A0omTI6@3
odI6AP?ooooo0oo;bl/4omkNgP?oYZJV0omBDU84odQ8B0?o=cLg1?m7AdL2oeeMG@?ooooo0om<C4`4
odm?C`?o;2`/1?m_Kfl3obXZ:PCoF5QH0om>CTh3ocLg=`;ooooo1?lJ6QX3o`d=3@Co6QXJ0olY:BT4
oahN7P?o@4101?m<C4`2ofQXJ0?ooooo0olT92@4od51@@?o=CDe1?lb<S83ocXj>PCo<c<c0om9BDT3
omgMg@;ooooo1?m5ADD3obXZ:PCo<30`0olO7al4obl_;`?o:BTY1?l`<302og=cL`?ooooo0omCDe<4
o`X:2P?o>CTi1?l^;Rh3ocHf=PCo:b/[0olB4Q83ohb<S0;ooooo1?lM7Ad3oa<C4`CoDU9B0olH61P4
o`l?3`?o6a/K0olU9BD3oeYJFP?ooooo0olF5QH4ochn?P?o;2`/1?l^;Rh3oeMGE`?o71`L1?mOGel2
ofIVIP?ooooo1?mTI6@3obl_;`?o>c/k1?ooool3ocTi>@CoADE50omYJFT3oeaLG0?ooooo0omCDe<3
oa<C4`CoF5QH0omJFUX4oa/K6`?oKVi^1?lP8202odE5A@?ooooo0omEEED4ooooo`?oJFUY1?lj>SX3
ob4Q8@Co3Ph>0olQ8B43obh^;P;ooooo1?l:2PX3oa`L70Co5aLG0ol_;bl4obPX:0?o1`L71?l_;bl2
ob`/;17ooooo000?ooooo`?o9bLW1?lT92@3oc@d=0Co?Shn0oll?3`4oaPH60?o=cLg0olU9BD2oooo
o`Co6QXJ0olG5aL4oa`L70?o:RXZ1?le=CD3ocLg=`Co8B4Q0_lh>3P3ooooo`?oATI61?lK6a/3ob8R
8PCo3`l?0olK6a/4oalO7`?o3`l?1Oooool4oi6AT@?oKFe]1?lR8R83odi>CP?o:BTY1?nATI43ofAT
I0?oATI60oooool3ol_;b`Cog]kN0onVYZH3oe9BDPCoB4Q80olg=cL4odM7A`;oGEeM0oooool3oda<
C0CoCdm?0ol/;2`4ofm_K`?o:RXZ1?mHF5P3odi>CP?o=cLg0_ooool4oaXJ6P?o3@d=1?lJ6QX3obTY
:@Co7QhN0om0@404oda<C0;oJ6QX0oooool3ob@T90Co@D510ole=CD4oc8b<P?o>SXj1?lc<c<3odU9
B@?ogMgM0_ooool4odE5A@?o:RXZ1?l`<303oalO7`Co;bl_0olY:BT4oc0`<0;oLg=c0oooool3oe=C
D`Co2PX:0oli>CT4obh^;P?o=SHf1?l[:b/3oa8B4P?oS8b<0_ooool4oadM7@?o4a<C1?mBDU83oaPH
60Co3`l?0olK6a/3obDU9@?oFUYJ0oooool3oaHF5PCo?Shn0ol/;2`4obh^;P?oEeMG0olL71`4oemO
G`;oIVIV0oooool4ofATI0?o;bl_0olk>c/4ooooo`?o>CTi1?m5ADD3ofUYJ@?oG5aL0oooool3oe=C
D`?o4a<C1?mHF5P3oeYJFPCo6a/K0om^KVh4ob0P80;oADE50oooool3oeEEE@Cooooo0omYJFT4ocXj
>P?o8B4Q1?l>3Ph3ob4Q8@?o;Rh^0_ooool4o`X:2P?o71`L1?lG5aL3obl_;`Co:2PX0ol71`L4obl_
;`;o;2`/4Oooool000oooooo0olW9bL4ob@T90?o=3@d1?ln?Sh3oc`l?0Co61PH0olg=cL3obDU9@;o
oooo1?lJ6QX3oaLG5`Co71`L0olZ:RX4ocDe=@?o=cLg1?lQ8B42ocPh>0?ooooo0om6ATH4oa/K6`?o
8R8R1?l?3`l3oa/K6`Co7alO0ol?3`l5ooooo`CoTI6A0om]KFd4ob8R8P?oCTi>0olY:BT4oi6AT@?o
I6AT0om6ATH3ooooo`?obl_;1?oNg]h3ojJVYP?oDU9B1?m8B4P3ocLg=`CoAdM70_mMGEd3ooooo`?o
C4a<1?m?Cdl3ob`/;0CoKfm_0olZ:RX4oeQHF0?oCTi>0olg=cL2ooooo`Co6QXJ0ol=3@d4oaXJ6P?o
:BTY1?lN7Qh3od10@0CoC4a<0_mXJ6P3ooooo`?o92@T1?m1@D43ocDe=@Co<S8b0olj>SX4oc<c<`?o
BDU90ooMgMd2ooooo`CoADE50olZ:RX4oc0`<0?o7alO1?l_;bl3obTY:@Co<30`0_mcLg<3ooooo`?o
De=C1?l:2PX3ocTi>@Co;Rh^0olf=SH4ob/[:`?o4Q8B0on<S8`2ooooo`Co7AdM0olC4a<4oe9BDP?o
61PH1?l?3`l3oa/K6`?o9BDU0omJFUX3ooooo`?o5QHF1?ln?Sh3ob`/;0Co;Rh^0omGEeL3oa`L70Co
GemO0_mVIVH3ooooo`CoI6AT0ol_;bl3oc/k>`Cooooo0oli>CT4odE5A@?oJFUY0omLG5`3ooooo`?o
De=C0olC4a<4oeQHF0?oFUYJ1?lK6a/3ofi^KPCo820P0_m5ADD3ooooo`?oEEEE1?ooool3ofUYJ@Co
>SXj0olQ8B44o`h>3P?o8B4Q0ol^;Rh2ooooo`Co2PX:0olL71`4oaLG5`?o;bl_1?lX:2P3o`L71`Co
;bl_0_l/;2`Aooooo`003oooool3obLW9`Co92@T0old=3@4ochn?P?o?3`l1?lH61P3ocLg=`?o9BDU
0_ooool4oaXJ6P?o5aLG1?lL71`3obXZ:PCo=CDe0olg=cL4ob4Q8@;o>3Ph0oooool3odI6APCo6a/K
0olR8R84o`l?3`?o6a/K1?lO7al3o`l?3`Gooooo1?nATI43ofe]K@Co8R8R0om>CTh3obTY:@CoTI6A
0omTI6@3odI6AP?ooooo0oo;bl/4omkNgP?oYZJV0omBDU84odQ8B0?o=cLg1?m7AdL2oeeMG@?ooooo
0om<C4`4odm?C`?o;2`/1?m_Kfl3obXZ:PCoF5QH0om>CTh3ocLg=`;ooooo1?lJ6QX3o`d=3@Co6QXJ
0olY:BT4oahN7P?o@4101?m<C4`2ofQXJ0?ooooo0olT92@4od51@@?o=CDe1?lb<S83ocXj>PCo<c<c
0om9BDT3omgMg@;ooooo1?m5ADD3obXZ:PCo<30`0olO7al4obl_;`?o:BTY1?l`<302og=cL`?ooooo
0omCDe<4o`X:2P?o>CTi1?l^;Rh3ocHf=PCo:b/[0olB4Q83ohb<S0;ooooo1?lM7Ad3oa<C4`CoDU9B
0olH61P4o`l?3`?o6a/K0olU9BD3oeYJFP?ooooo0olF5QH4ochn?P?o;2`/1?l^;Rh3oeMGE`?o71`L
1?mOGel2ofIVIP?ooooo1?mTI6@3obl_;`?o>c/k1?ooool3ocTi>@CoADE50omYJFT3oeaLG0?ooooo
0omCDe<3oa<C4`CoF5QH0omJFUX4oa/K6`?oKVi^1?lP8202odE5A@?ooooo0omEEED4ooooo`?oJFUY
1?lj>SX3ob4Q8@Co3Ph>0olQ8B43obh^;P;ooooo1?l:2PX3oa`L70Co5aLG0ol_;bl4obPX:0?o1`L7
1?l_;bl2ob`/;17ooooo003oooooooGooooo003oooooooGooooo003oooooooGooooo000?ooooo`?o
410@1?lJ6QX3oa/K6`Co<S8b0om`L704ol;2`P?oUIFE0omADE42ooooo`Co;bl_0olC4a<4ode=C@?o
3`l?1?lm?Cd3oc8b<PCo7alO0_o;bl/3ooooo`?o6ATI1?l410@3obHV9PCo2`/;0olI6AT4o`@410?o
410@0_lo?cl3ooooo`CoQ8B40olZ:RX4o`/;2`?o@D510ol00004ocDe=@?o3@d=0ol/;2`3ooooo`?o
6a/K1?lX:2P3o`l?3`?o>3Ph1?lS8b<3oc<c<`Co?Cdm0_n=SHd3ooooo`?oE5AD1?lM7Ad3ofa/K0Co
d=3@0olF5QH4oc<c<`Ko7AdM0_ooool4obXZ:P?o61PH1?l?3`l3oahN7PCo:BTY0omTI6@4ohN7Q`;o
G5aL0oooool3oaDE5@Co=SHf0olK6a/4ogAdM0?oEUIF1?l;2`/3obTY:@Gooooo1?ll?3`7obLW9`?o
92@T1?lV9RH3oaTI6@Co61PH0_maLG43ooooo`?oK6a/1?lD51@3of][J`Co9BDU0omIFET4omgMg@?o
5aLG0omOGel2ooooo`Co4Q8B0olg=cL4oc/k>`?o3@d=1?lM7Ad3o`410@?o>3Ph0ola<C43ooooo`?o
410@1?m>CTh3o`H61PCo:RXZ0olA4A43obDU9@CoHF5Q0_lT92@3ooooo`Co4A4A0olS8b<3oc@d=0Co
51@D0olK6a/4oaTI6@?o8R8R0olk>c/3ooooo`?o@D510ol61PH4ohJ6QP?o@d=31?lA4A43oeEEE@Co
1PH60_ln?Sh3ooooo`?o410@1?lA4A43obDU9@Co?Shn0on9RHT4od10@0?o6QXJ0olg=cL2ooooo`Co
3@d=0omiNGT4oeUIF@?o2@T91?lX:2P3oaDE5@CoDe=C0_nOWilAooooo`003oooool3oa0@40Co6QXJ
0olK6a/4oc8b<P?oL71`1?o2`/83oiFEU@?oDE5A0_ooool4obl_;`?o4a<C1?m=CDd3o`l?3`Co?Cdm
0olb<S84oalO7`;obl_;0oooool3oaTI6@Co10@40olV9RH4o`/;2`?o6ATI1?l410@3oa0@40;o?clo
0oooool4ohB4Q0?o:RXZ1?l;2`/3od51@@?o00001?le=CD3o`d=3@?o;2`/0oooool3oa/K6`Co:2PX
0ol?3`l3ocPh>0Co8b<S0olc<c<4ocdm?@;oSHf=0oooool3oeADE0Co7AdM0om/K6`4om3@d0?o5QHF
1?lc<c<6oadM7@;ooooo1?lZ:RX3oaPH60Co3`l?0olN7Qh4obTY:@?oI6AT1?n7QhL2oeaLG0?ooooo
0olE5AD4ocHf=P?o6a/K1?mdM7@3oeIFEPCo2`/;0olY:BT5ooooo`Co?3`l1olW9bL3ob@T90Co9RHV
0olI6AT4oaPH60;oLG5a0oooool3ofa/K0Co51@D0om[Jf/4obDU9@?oFEUI1?oMgMd3oaLG5`?oGemO
0_ooool4oa8B4P?o=cLg1?lk>c/3o`d=3@Co7AdM0ol10@43ocPh>0?o<C4a0oooool3oa0@40CoCTi>
0ol61PH4obXZ:P?o4A4A0olU9BD4of5QH@;o92@T0oooool4oa4A4@?o8b<S0old=3@4oa@D50?o6a/K
1?lI6AT3ob8R8P?o>c/k0oooool3od51@@?o1PH61?n6QXH3od=3@`Co4A4A0omEEED4o`H61P;o?Shn
0oooool3oa0@40Co4A4A0olU9BD4ochn?P?oRHV91?m0@403oaXJ6P?o=cLg0_ooool4o`d=3@?oNGUi
1?mIFET3o`T92@Co:2PX0olE5AD4oe=CD`;oWinO4Oooool000oooooo0ol@4104oaXJ6P?o6a/K1?lb
<S83og1`L0Co`/;20onEUID3oe5AD@;ooooo1?l_;bl3oa<C4`CoCDe=0ol?3`l4ocdm?@?o<S8b1?lO
7al2ol_;b`?ooooo0olI6AT4o`@410?o9RHV1?l;2`/3oaTI6@Co10@40ol@4102oclo?`?ooooo1?n4
Q8@3obXZ:PCo2`/;0om1@D43o`0000Co=CDe0ol=3@d3ob`/;0?ooooo0olK6a/4obPX:0?o3`l?0olh
>3P4ob<S8`?o<c<c1?lm?Cd2ohf=S@?ooooo0omDE5@4oadM7@?oK6a/1?o@d=03oaHF5PCo<c<c1_lM
7Ad2ooooo`Co:RXZ0olH61P4o`l?3`?o7QhN1?lY:BT3ofATI0CoQhN70_mLG5`3ooooo`?o5ADE1?lf
=SH3oa/K6`CoM7Ad0omFEUH4o`/;2`?o:BTY1Oooool4oc`l?0Oo9bLW0olT92@4obHV9P?o6ATI1?lH
61P2og5aL@?ooooo0om/K6`4oa@D50?oJf][1?lU9BD3oeUIF@CogMgM0olG5aL3oemOG`;ooooo1?lB
4Q83ocLg=`Co>c/k0ol=3@d4oadM7@?o0@410olh>3P3oc4a<@?ooooo0ol@4104odi>CP?o1PH61?lZ
:RX3oa4A4@?o9BDU1?mQHF42ob@T90?ooooo1?lA4A43ob<S8`?o=3@d1?lD51@3oa/K6`Co6ATI0olR
8R83oc/k>`?ooooo0om1@D43o`H61PCoQXJ60om3@d<4oa4A4@?oEEEE1?l61PH2ochn?P?ooooo0ol@
4104oa4A4@?o9BDU1?ln?Sh3ohV9R@Co@4100olJ6QX3ocLg=`;ooooo1?l=3@d3ogUiN@CoFEUI0ol9
2@T4obPX:0?o5ADE1?mCDe<2oinOWa7ooooo000?ooooo`?o410@1?lJ6QX3oa/K6`Co<S8b0om`L704
ol;2`P?oUIFE0omADE42ooooo`Co;bl_0olC4a<4ode=C@?o3`l?1?lm?Cd3oc8b<PCo7alO0_o;bl/3
ooooo`?o6ATI1?l410@3obHV9PCo2`/;0olI6AT4o`@410?o410@0_lo?cl3ooooo`CoQ8B40olZ:RX4
o`/;2`?o@D510ol00004ocDe=@?o3@d=0ol/;2`3ooooo`?o6a/K1?lX:2P3o`l?3`?o>3Ph1?lS8b<3
oc<c<`Co?Cdm0_n=SHd3ooooo`?oE5AD1?lM7Ad3ofa/K0Cod=3@0olF5QH4oc<c<`Ko7AdM0_ooool4
obXZ:P?o61PH1?l?3`l3oahN7PCo:BTY0omTI6@4ohN7Q`;oG5aL0oooool3oaDE5@Co=SHf0olK6a/4
ogAdM0?oEUIF1?l;2`/3obTY:@Gooooo1?ll?3`7obLW9`?o92@T1?lV9RH3oaTI6@Co61PH0_maLG43
ooooo`?oK6a/1?lD51@3of][J`Co9BDU0omIFET4omgMg@?o5aLG0omOGel2ooooo`Co4Q8B0olg=cL4
oc/k>`?o3@d=1?lM7Ad3o`410@?o>3Ph0ola<C43ooooo`?o410@1?m>CTh3o`H61PCo:RXZ0olA4A43
obDU9@CoHF5Q0_lT92@3ooooo`Co4A4A0olS8b<3oc@d=0Co51@D0olK6a/4oaTI6@?o8R8R0olk>c/3
ooooo`?o@D510ol61PH4ohJ6QP?o@d=31?lA4A43oeEEE@Co1PH60_ln?Sh3ooooo`?o410@1?lA4A43
obDU9@Co?Shn0on9RHT4od10@0?o6QXJ0olg=cL2ooooo`Co3@d=0omiNGT4oeUIF@?o2@T91?lX:2P3
oaDE5@CoDe=C0_nOWilAooooo`003oooool3o`0000Co8b<S0ol];Bd4ochn?P?oAdM71?ooool3ojf]
[@?o7AdM0_ooool4o`d=3@?o2`/;1?lm?Cd3o`L71`Co=3@d0olZ:RX4ob0P80;oa/K60oooool3o`D5
1@Co2PX:0ol?3`l4o`T92@?o5ADE1?l410@3o`L71`;o?Shn0oooool4oiZJVP?oG5aL1?l@4103obh^
;P?o4a<C1?mTI6@3oa@D50?oK6a/0oooool3oc`l?0Co1PH60olI6AT3oa/K6`Co?clo0ol/;2`4odM7
A`;oPh>30oooool3ocHf=PCo71`L0omFEUH4ooooo`?o:BTY1?l_;bl3oa@D50?o8b<S0_ooool4oa@D
50?o2@T91?l;2`/3o`820PCo7QhN0olb<S84ocdm?@;o<S8b0oooool3o`P820Co@4100olB4Q84odY:
BP?oShn?1?lI6AT3od51@@?oeMGE0_ooool4oemOG`?o@4101?lU9BD3ob0P80Co51@D0olB4Q84oa`L
70;oLW9b0oooool3ogEeM@Co30`<0om?Cdl4oc<c<`?oIFEU1?ooool3oa/K6`?oO7al0_ooool4o`l?
3`?o;Rh^1?m:BTX3o`D51@Co6ATI0ol10@43oc`l?0?o6a/K0oooool3oaHF5PCo<C4a0ol@4104ocLg
=`?o3Ph>0olB4Q84oeADE0;o9bLW0oooool4o`h>3P?o3`l?0olH61P4oa0@40?o2`/;1?l92@T3oa8B
4P?o7QhN0oooool3od];B`?o92@T1?n4Q8@3of9RHPCo4Q8B0om2@T84o`d=3@;o9RHV0oooool3o`X:
2PCo0`<30olJ6QX4ob`/;0?o@4101?m6ATH3obh^;P?o:RXZ0_ooool4o`P820?oHf=S1?mIFET3o`X:
2PCo@4100olU9BD4ode=C@;ohn?S4Oooool000oooooo0ol00004ob<S8`?o;Bd]1?ln?Sh3odM7A`Co
oooo0on][Jd3oadM7@;ooooo1?l=3@d3o`/;2`Co?Cdm0ol71`L4oc@d=0?o:RXZ1?lP8202olK6aP?o
oooo0ol51@D4o`X:2P?o3`l?1?l92@T3oaDE5@Co10@40ol71`L2ochn?P?ooooo1?nJVYX3oeaLG0Co
410@0ol^;Rh3oa<C4`CoI6AT0olD51@3ofa/K0?ooooo0oll?3`4o`H61P?o6ATI0olK6a/4oclo?`?o
;2`/1?m7AdL2oh>3P`?ooooo0olf=SH4oa`L70?oEUIF1?ooool3obTY:@Co;bl_0olD51@3ob<S8`;o
oooo1?lD51@3o`T92@Co2`/;0ol20P84oahN7P?o<S8b1?lm?Cd2oc8b<P?ooooo0ol820P4od10@0?o
4Q8B1?m:BTX3ohn?S`Co6ATI0om1@D43omGEe@;ooooo1?mOGel3od10@0Co9BDU0olP8204oa@D50?o
4Q8B1?lL71`2og9bLP?ooooo0omeMGD4o``<30?oCdm?1?lc<c<3ofEUI@Cooooo0olK6a/3ogalO0;o
oooo1?l?3`l3obh^;PCoBTY:0ol51@D4oaTI6@?o0@410oll?3`3oa/K6`?ooooo0olF5QH4oc4a<@?o
410@1?lg=cL3o`h>3P?o4Q8B1?mDE5@2obLW9`?ooooo1?l>3Ph3o`l?3`?o61PH1?l@4103o`/;2`Co
2@T90olB4Q83oahN7P?ooooo0om;Bd/3ob@T90CoQ8B40omRHV84oa8B4P?o@T921?l=3@d2obHV9P?o
oooo0ol:2PX4o`<30`?o6QXJ1?l/;2`3od10@0CoATI60ol^;Rh3obXZ:P;ooooo1?l820P3of=SH`Co
FEUI0ol:2PX4od10@0?o9BDU1?m=CDd2on?Sha7ooooo000?ooooo`?o00001?lS8b<3obd];@Co?Shn
0om7AdL4ooooo`?o[Jf]0olM7Ad2ooooo`Co3@d=0ol;2`/4ocdm?@?o1`L71?ld=3@3obXZ:PCo820P
0_o6a/H3ooooo`?o1@D51?l:2PX3o`l?3`Co2@T90olE5AD4o`@410?o1`L70_ln?Sh3ooooo`CoVYZJ
0omLG5`4oa0@40?o;Rh^0olC4a<4ofATI0?o51@D0om/K6`3ooooo`?o?3`l1?l61PH3oaTI6@?o6a/K
1?lo?cl3ob`/;0CoAdM70_n3Ph<3ooooo`?o=SHf1?lL71`3oeIFEPCooooo0olY:BT4obl_;`?o51@D
0olS8b<2ooooo`Co51@D0ol92@T4o`/;2`?o0P821?lN7Qh3oc8b<PCo?Cdm0_lb<S83ooooo`?o20P8
1?m0@403oa8B4PCoBTY:0on?Shl4oaTI6@?o@D510ooEeMD2ooooo`CoGemO0om0@404obDU9@?o820P
1?lD51@3oa8B4PCo71`L0_mbLW83ooooo`?oMGEe1?l<30`3odm?C`Co<c<c0omUIFD4ooooo`?o6a/K
0omlO7`2ooooo`Co3`l?0ol^;Rh4odY:BP?o1@D51?lI6AT3o`410@?o?3`l0olK6a/3ooooo`?o5QHF
1?la<C43oa0@40Co=cLg0ol>3Ph3oa8B4PCoE5AD0_lW9bL3ooooo`Co3Ph>0ol?3`l3oaPH60Co410@
0ol;2`/4o`T92@?o4Q8B0olN7Qh3ooooo`?oBd];0olT92@4ohB4Q0?oHV9R1?lB4Q83od92@PCo3@d=
0_lV9RH3ooooo`?o2PX:1?l30`<3oaXJ6PCo;2`/0om0@404odI6AP?o;Rh^0olZ:RX2ooooo`Co20P8
0omSHf<4oeUIF@?o2PX:1?m0@403obDU9@CoCDe=0_oShn<Aooooo`003oooool3ob0P80Co5aLG0olL
71`4ocTi>@?oKFe]1?ofm_H3oj:RXP?oTi>C0_ooool4odm?C`?o4a<C1?l];Bd3o`<30`CoIfMW0olA
4A44o`0000;oSXj>0oooool3oa8B4PCo:BTY0olO7al4o`P820?o6a/K1?l20P83oa`L70;oBDU90ooo
ool4ohF5Q@?oYjNW1?lJ6QX3oc@d=0?o3@d=1?mHF5P3obl_;`?oH61P0oooool3oa0@40Co8b<S0ol3
0`<3oc`l?0Co<c<c0om;Bd/4od10@0;oOGem0oooool3ochn?PCo7AdM0olZ:RX4oiVIV@?o3Ph>1?lo
?cl3oc8b<P?o;2`/0_ooool4oc8b<P?o4a<C1ol410@4o`L71`?o;Bd]1?m8B4P2ocHf=P?ooooo0ol:
2PX4obl_;`?o:RXZ1?m7AdL3og]kN`Co2PX:0om:BTX3olc<c0;ooooo1?n0P803ofIVIPCo@d=30om=
CDd4ocDe=@?o4A4A1?lM7Ad2ok:b/P?ooooo0omADE44ob8R8P?oDe=C1?l/;2`3obd];@Codm?C0olA
4A43ok>c/`;ooooo1?lg=cL3obPX:0CoGUiN0ol92@T4ob<S8`?o00000om4A4@3odM7A`?ooooo0olD
51@4ob8R8P?o00001?lo?cl3o`P820?o61PH1?lm?Cd2odU9B@?ooooo1?l=3@d3ob4Q8@?o<C4a1?lD
51@3oahN7PCo5aLG0olI6AT3ob/[:`?ooooo0omoOgl3ob`/;0CoGemO0oo_knl4o`/;2`?o@d=31?lD
51@2obDU9@?ooooo0ol10@44o`0000?o4a<C1?lN7Qh3ocPh>0Co:b/[0olZ:RX3ocHf=P;ooooo1?l<
30`3oe5AD@CoJFUY0ol:2PX4ocPh>0?o;2`/1?m3@d<2omKFeQ7ooooo000?ooooo`?o820P1?lG5aL3
oa`L70Co>CTi0om]KFd4ooKfmP?oXZ:R0onCTi<2ooooo`CoCdm?0olC4a<4obd];@?o0`<31?mWIfL3
oa4A4@Co00000_n>SXh3ooooo`?o4Q8B1?lY:BT3oalO7`Co20P80olK6a/4o`820P?o71`L0_m9BDT3
ooooo`CoQHF50onWYjL4oaXJ6P?o=3@d0ol=3@d4oeQHF0?o;bl_0omPH603ooooo`?o410@1?lS8b<3
o`<30`?o?3`l1?lc<c<3od];B`Co@4100_mmOGd3ooooo`?o?Shn1?lM7Ad3obXZ:PCoVIVI0ol>3Ph4
oclo?`?o<S8b0ol/;2`2ooooo`Co<S8b0olC4a<7o`@410Co1`L70ol];Bd4odQ8B0;o=SHf0oooool3
o`X:2PCo;bl_0olZ:RX4odM7A`?oNg]k1?l:2PX3odY:BP?oc<c<0_ooool4oh20P0?oIVIV1?m3@d<3
ode=C@Co=CDe0olA4A44oadM7@;o/[:b0oooool3oe5AD@Co8R8R0omCDe<4ob`/;0?o;Bd]1?oCdm<3
oa4A4@?o/k>c0_ooool4ocLg=`?o:2PX1?mNGUh3o`T92@Co8b<S0ol00003odA4A0?oAdM70oooool3
oa@D50Co8R8R0ol00004oclo?`?o20P80olH61P4ocdm?@;oBDU90oooool4o`d=3@?o8B4Q0ola<C44
oa@D50?o7QhN1?lG5aL3oaTI6@?o:b/[0oooool3ogmoO`?o;2`/1?mOGel3ono_k`Co2`/;0om3@d<4
oa@D50;o9BDU0oooool3o`410@Co00000olC4a<4oahN7P?o>3Ph1?l[:b/3obXZ:P?o=SHf0_ooool4
o``<30?oDE5A1?mYJFT3o`X:2PCo>3Ph0ol/;2`4od=3@`;oe]KF4Oooool000oooooo0olP8204oaLG
5`?o71`L1?li>CT3ofe]K@Com_Kf0onRXZ83oi>CT`;ooooo1?m?Cdl3oa<C4`Co;Bd]0ol30`<4ofMW
I`?o4A4A1?l00002ohj>SP?ooooo0olB4Q84obTY:@?o7alO1?l820P3oa/K6`Co0P820olL71`2odU9
B@?ooooo1?n5QHD3ojNWY`Co6QXJ0old=3@3o`d=3@CoF5QH0ol_;bl3of1PH0?ooooo0ol@4104ob<S
8`?o0`<30oll?3`4oc<c<`?oBd];1?m0@402ogemO@?ooooo0oln?Sh4oadM7@?o:RXZ1?nIVIT3o`h>
3PCo?clo0olb<S83ob`/;0;ooooo1?lb<S83oa<C4`Oo10@41?l71`L3obd];@CoB4Q80_lf=SH3oooo
o`?o2PX:1?l_;bl3obXZ:PCoAdM70omkNg/4o`X:2P?oBTY:0oo<c<`2ooooo`CoP8200omVIVH4od=3
@`?oCDe=1?le=CD3oa4A4@Co7AdM0_nb/[83ooooo`?oDE5A1?lR8R83oe=CD`Co;2`/0ol];Bd4om?C
d`?o4A4A0onc/k<2ooooo`Co=cLg0olX:2P4oeiNGP?o2@T91?lS8b<3o`0000?oA4A40om7AdL3oooo
o`?o51@D1?lR8R83o`0000Co?clo0ol820P3oaPH60Co?Cdm0_m9BDT3ooooo`Co3@d=0olQ8B43oc4a
<@Co51@D0olN7Qh4oaLG5`?o6ATI0ol[:b/3ooooo`?oOgmo0ol/;2`4oemOG`?okno_1?l;2`/3od=3
@`Co51@D0_lU9BD3ooooo`?o0@411?l00003oa<C4`Co7QhN0olh>3P4ob/[:`?o:RXZ0olf=SH2oooo
o`Co30`<0omADE44ofUYJ@?o2PX:1?lh>3P3ob`/;0Co@d=30_oFe]HAooooo`003oooool3o`D51@Co
3@d=0ol[:b/4oc8b<P?o<30`1?oDe=@3oiNGU`?oATI60_ooool4oh20P0?o5ADE1?lg=cL3oalO7`Co
Jf][0ol`<304oahN7P;oN7Qh0oooool3oa@D50Co=SHf0olP8204o`0000?o7AdM1?l820P3oahN7P;o
;2`/0oooool4oiNGU`?o`l?31?l;2`/3ochn?P?o4a<C1?mZJVX3odY:BP?oMgMg0oooool3obLW9`Co
00000olK6a/3ob`/;0Co@4100olY:BT4odQ8B0;oRXZ:0oooool3oc/k>`Co4Q8B0ol71`L4oemOG`?o
1PH61?lf=SH3obl_;`?o9BDU0_ooool7o`T92@Co0P820ol410@4o`P820?o5QHF1?lV9RH2odE5A@?o
oooo0olC4a<4ocXj>P?o8B4Q1?lY:BT3ohF5Q@Co2@T90oli>CT3oi2@T0;ooooo1?n>SXh3ofe]K@Co
Bd];0om>CTh4obTY:@?o6ATI1?lX:2P2olg=c@?ooooo0olU9BD4o`@410?o@d=31?lK6a/3oaHF5PCo
M7Ad0ol71`L3ogAdM0;ooooo1?lg=cL3of9RHPCooooo0olH61P4ob0P80?o4A4A0omADE43ob@T90?o
oooo0olR8R84oalO7`?o3@d=1?mUIFD3o`0000?o410@1?lT92@2ogMgM`?ooooo1?l00003oalO7`?o
5ADE1?l@4103o`h>3PCo6QXJ0ol92@T3ob4Q8@?ooooo0om3@d<3odU9B@CoUYJF0ono_kl4oaPH60?o
@D511?lN7Qh2o``<30?ooooo0ol61PH4o`0000?o6a/K1?lQ8B4:oaXJ6P?o>3Ph0_ooool4o`H61P?o
8R8R1?m^KVh3o`X:2PCo<c<c0olf=SH4od51@A?ooooo000?ooooo`?o1@D51?l=3@d3ob/[:`Co<S8b
0ol`<304omCDe0?oUiNG0om6ATH2ooooo`CoP8200olE5AD4ocLg=`?o7alO1?m[Jf/3oc0`<0Co7QhN
0_mhN7P3ooooo`?o51@D1?lf=SH3ob0P80Co00000olM7Ad4o`P820?o7QhN0_l/;2`3ooooo`CoUiNG
0oo3`l<4o`/;2`?o?Shn0olC4a<4ofYZJP?oBTY:0omgMgL3ooooo`?o9bLW1?l00003oa/K6`?o;2`/
1?m0@403obTY:@CoB4Q80_n:RXX3ooooo`?o>c/k1?lB4Q83o`L71`CoGemO0ol61PH4ocHf=P?o;bl_
0olU9BD2ooooo`Oo2@T91?l20P83o`@410Co20P80olF5QH4obHV9P;oADE50oooool3oa<C4`Co>SXj
0olQ8B44obTY:@?oQHF51?l92@T3ocTi>@?oT92@0_ooool4ohj>SP?oKFe]1?m;Bd/3odi>CPCo:BTY
0olI6AT4obPX:0;ocLg=0oooool3obDU9@Co10@40om3@d<4oa/K6`?o5QHF1?mdM7@3o`L71`?oM7Ad
0_ooool4ocLg=`?oHV9R1?ooool3oaPH60Co820P0olA4A43oe5AD@?o92@T0oooool3ob8R8PCo7alO
0ol=3@d4ofEUI@?o00000ol@4104ob@T90;oMgMg0oooool4o`0000?o7alO0olE5AD4oa0@40?o3Ph>
1?lJ6QX3o`T92@?o8B4Q0oooool3od=3@`?oBDU91?nFUYH3okno_`Co61PH0om1@D44oahN7P;o30`<
0oooool3o`H61PCo00000olK6a/4ob4Q8@[o6QXJ0olh>3P2ooooo`Co1PH60olR8R84ofi^KP?o2PX:
1?lc<c<3ocHf=PCo@D514oooool000oooooo0ol51@D4o`d=3@?o:b/[1?lb<S83oc0`<0Coe=CD0onG
UiL3odI6AP;ooooo1?n0P803oaDE5@Co=cLg0olO7al4of][J`?o<30`1?lN7Qh2ogQhN0?ooooo0olD
51@4ocHf=P?o820P1?l00003oadM7@Co20P80olN7Qh2ob`/;0?ooooo1?nGUiL3ol?3``Co2`/;0oln
?Sh3oa<C4`CoJVYZ0om:BTX3ogMgM`?ooooo0olW9bL4o`0000?o6a/K0ol/;2`4od10@0?o:BTY1?m8
B4P2ohZ:RP?ooooo0olk>c/4oa8B4P?o1`L71?mOGel3o`H61PCo=SHf0ol_;bl3obDU9@;ooooo1ol9
2@T4o`820P?o10@41?l820P3oaHF5PCo9RHV0_m5ADD3ooooo`?o4a<C1?lj>SX3ob4Q8@Co:BTY0on5
QHD4o`T92@?o>CTi0on@T902ooooo`CoSXj>0om]KFd4od];B`?oCTi>1?lY:BT3oaTI6@Co:2PX0_o=
cLd3ooooo`?o9BDU1?l410@3od=3@`Co6a/K0olF5QH4ogAdM0?o1`L70omdM7@2ooooo`Co=cLg0omR
HV84ooooo`?o61PH1?lP8203oa4A4@?oDE5A0olT92@3ooooo`?o8R8R1?lO7al3o`d=3@CoIFEU0ol0
0003oa0@40Co92@T0_mgMgL3ooooo`Co00000olO7al3oaDE5@Co410@0ol>3Ph4oaXJ6P?o2@T90olQ
8B43ooooo`?o@d=30om9BDT4oiJFUP?o_kno1?lH61P3od51@@Co7QhN0_l<30`3ooooo`?o1PH61?l0
0003oa/K6`Co8B4Q2_lJ6QX3ocPh>0;ooooo1?l61PH3ob8R8PCoKVi^0ol:2PX4oc<c<`?o=SHf1?m1
@D4Cooooo`003oooool3oa0@40Co6a/K0olR8R84oeEEE@?oR8R81?ooool3oinOW`?oNWYj0_ooool4
okJf]P?o10@41?m0@403oahN7PCoYJFU0omKFe/4o`l?3`;o=cLg0oooool3ocTi>@CoDU9B0omRHV84
oaLG5`?o:BTY1?lM7Ad3ocDe=@;oAdM70oooool4oemOG`?ooooo1?l92@T3odM7A`?o;bl_1?m[Jf/3
ofIVIP?oL71`0oooool3o`P820Co;2`/0olD51@3ocPh>0Co?Cdm0om0@404oclo?`;oR8R80oooool3
ocTi>@Co5QHF0ol@4104oc/k>`?o00001?l];Bd3ocPh>0?o8B4Q0_ooool4oaPH60?o1`L71?l=3@d3
o`0000Co1`L70olF5QH4oahN7P;o820P0oooool3o`H61PCo:RXZ0olM7Ad4oa4A4@?oDU9B1?l61PH3
odQ8B0?oBDU90_ooool4ofm_K`?oHV9R1?m6ATH3odA4A0Co92@T0olI6AT4obLW9`Gooooo0olS8b<4
o`<30`?oCDe=1?lG5aL3o`@410Co?Shn0ol00003oh:2PP;ooooo1?le=CD3ocHf=PCoVi^K0olB4Q84
obDU9@?o4Q8B0om5ADD3odY:BP?ooooo0olR8R84obl_;`?o3`l?1?miNGT3oaLG5`?o7AdM1?lV9RH2
oo3`l0?ooooo1?l410@3oaTI6@?o61PH1ol@4104oaPH60?o3Ph>0olH61P3ooooo`?oMgMg0ole=CD4
ochn?P?ooooo1?l>3Ph3odE5A@CoA4A40_l_;bl3ooooo`Oo20P80olA4A44oa`L70?o6a/K1?lV9RH3
obPX:0?o8b<S0_ooool4o`<30`?o:2PX1?mNGUh3o`/;2`Co?3`l0olk>c/4obDU9@;oeMGE4Oooool0
00oooooo0ol@4104oa/K6`?o8R8R1?mEEED3ohR8R0Cooooo0onOWil3ogYjNP;ooooo1?nf][H3o`@4
10Co@4100olN7Qh4ojFUY@?oFe]K1?l?3`l2ocLg=`?ooooo0oli>CT4oe9BDP?oHV9R1?lG5aL3obTY
:@Co7AdM0ole=CD2odM7A`?ooooo1?mOGel3ooooo`Co2@T90om7AdL3obl_;`CoJf][0omVIVH3og1`
L0?ooooo0ol820P4ob`/;0?o51@D0olh>3P4ocdm?@?o@4101?lo?cl2ohR8R0?ooooo0oli>CT4oaHF
5P?o410@1?lk>c/3o`0000Co;Bd]0olh>3P3ob4Q8@;ooooo1?lH61P3o`L71`Co3@d=0ol00004o`L7
1`?o5QHF1?lN7Qh2ob0P80?ooooo0ol61PH4obXZ:P?o7AdM1?lA4A43oe9BDPCo1PH60om8B4P3odU9
B@;ooooo1?m_Kfl3of9RHPCoATI60om4A4@4ob@T90?o6ATI1?lW9bL5ooooo`?o8b<S1?l30`<3ode=
C@Co5aLG0ol410@4ochn?P?o00000on2PX82ooooo`Co=CDe0olf=SH4oi^KV`?o4Q8B1?lU9BD3oa8B
4P?oADE50om:BTX3ooooo`?o8R8R1?l_;bl3o`l?3`CoNGUi0olG5aL3oadM7@Co9RHV0_o`l?03oooo
o`Co10@40olI6AT3oaPH60Oo410@1?lH61P3o`h>3P?o61PH0oooool3ogMgM`?o=CDe1?ln?Sh3oooo
o`Co3Ph>0om5ADD4odA4A0;o;bl_0oooool7o`P820?o4A4A1?lL71`3oa/K6`Co9RHV0olX:2P3ob<S
8`;ooooo1?l30`<3obPX:0CoGUiN0ol;2`/4oc`l?0?o>c/k1?lU9BD2omGEeA7ooooo000?ooooo`?o
410@1?lK6a/3ob8R8PCoEEEE0on8R8P4ooooo`?oWinO0omjNWX2ooooo`Co][Jf0ol410@4od10@0?o
7QhN1?nUYJD3oe]KF`Co3`l?0_lg=cL3ooooo`?o>CTi1?mBDU83of9RHPCo5aLG0olY:BT4oadM7@?o
=CDe0_m7AdL3ooooo`CoGemO0oooool4o`T92@?oAdM70ol_;bl4of][J`?oIVIV0om`L703ooooo`?o
20P81?l/;2`3oa@D50?o>3Ph1?lm?Cd3od10@0Co?clo0_n8R8P3ooooo`?o>CTi1?lF5QH3oa0@40Co
>c/k0ol00004obd];@?o>3Ph0olQ8B42ooooo`Co61PH0ol71`L4o`d=3@?o00001?l71`L3oaHF5PCo
7QhN0_lP8203ooooo`?o1PH61?lZ:RX3oadM7@Co4A4A0omBDU84o`H61P?oB4Q80om9BDT2ooooo`Co
Kfm_0omRHV84odI6AP?oA4A41?lT92@3oaTI6@Co9bLW1Oooool3ob<S8`Co0`<30om=CDd4oaLG5`?o
10@41?ln?Sh3o`0000?oPX:20_ooool4ocDe=@?o=SHf1?nKVi/3oa8B4PCo9BDU0olB4Q83odE5A@?o
BTY:0oooool3ob8R8PCo;bl_0ol?3`l4ogUiN@?o5aLG0olM7Ad4obHV9P;ol?3`0oooool4o`@410?o
6ATI0olH61P7oa0@40Co61PH0ol>3Ph3oaPH60?ooooo0omgMgL3ocDe=@Co?Shn0oooool4o`h>3P?o
ADE51?m4A4@2obl_;`?ooooo1ol820P3oa4A4@Co71`L0olK6a/4obHV9P?o:2PX0olS8b<2ooooo`Co
0`<30olX:2P4oeiNGP?o2`/;1?ll?3`3oc/k>`Co9BDU0_oEeMDAooooo`003oooool7oa4A4@?o8R8R
1?lP8203o`@410Coj>SX0on<S8`3oalO7`Kooooo0ol?3`l4od51@@?o8b<S1?n;Rh/3og1`L0Co9RHV
0_m5ADD3ooooo`?oCTi>1?n5QHD3ooooo`Co;2`/0olQ8B44oaPH60?o=3@d0_m9BDT3ooooo`Co=cLg
0onLW9`4oa<C4`?oCdm?0ol`<304ode=C@?oUiNG0om4A4@3ooooo`?o=3@d1?l;2`/3ob0P80?o;Rh^
1?mFEUH3ob8R8PCoDU9B0_mbLW83ooooo`?o@D511?lU9BD3oaLG5`Co?Shn0ol:2PX4oc`l?0?o:2PX
0olh>3P2ooooo`Co6a/K0ol61PH7o`T92@Co2PX:0ol<30`4obPX:0;o7AdM0oooool3o`l?3`Co?Cdm
0ol<30`4o`L71`?o>c/k1?l10@43obLW9`?oBd];0_ooool4oda<C0?o@d=31?lS8b<3oc<c<`Co5aLG
0ol00004oa`L70;oJFUY0oooool3oaXJ6PCo1@D50omPH604oa/K6`?o2@T91?lm?Cd3oa4A4@?oCTi>
0_ooool4oa/K6`?o3`l?1?mVIVH3o`L71`Oo7AdM0olQ8B43od51@@?ooooo0olK6a/4obDU9@?o410@
1?mRHV83ocDe=@?o6ATI1?la<C42om3@d0?ooooo1?l61PH3oahN7P?o30`<1?l;2`/3oa<C4`Co3`l?
0ol410@3ob<S8`?ooooo0om4A4@3ocdm?@Co;2`/0omVIVH4o`X:2P?o>CTi1?lf=SH2o`l?3`?ooooo
0ol51@D7oa4A4@Co5QHF0olU9BD4oaLG5`?o6ATI0olb<S82ooooo`Co2PX:0ol?3`l4odU9B@?o2PX:
1?lc<c<3oc/k>`Co:2PX0_ni^KTAooooo`003oooool7oa4A4@?o8R8R1?lP8203o`@410Coj>SX0on<
S8`3oalO7`Kooooo0ol?3`l4od51@@?o8b<S1?n;Rh/3og1`L0Co9RHV0_m5ADD3ooooo`?oCTi>1?n5
QHD3ooooo`Co;2`/0olQ8B44oaPH60?o=3@d0_m9BDT3ooooo`Co=cLg0onLW9`4oa<C4`?oCdm?0ol`
<304ode=C@?oUiNG0om4A4@3ooooo`?o=3@d1?l;2`/3ob0P80?o;Rh^1?mFEUH3ob8R8PCoDU9B0_mb
LW83ooooo`?o@D511?lU9BD3oaLG5`Co?Shn0ol:2PX4oc`l?0?o:2PX0olh>3P2ooooo`Co6a/K0ol6
1PH7o`T92@Co2PX:0ol<30`4obPX:0;o7AdM0oooool3o`l?3`Co?Cdm0ol<30`4o`L71`?o>c/k1?l1
0@43obLW9`?oBd];0_ooool4oda<C0?o@d=31?lS8b<3oc<c<`Co5aLG0ol00004oa`L70;oJFUY0ooo
ool3oaXJ6PCo1@D50omPH604oa/K6`?o2@T91?lm?Cd3oa4A4@?oCTi>0_ooool4oa/K6`?o3`l?1?mV
IVH3o`L71`Oo7AdM0olQ8B43od51@@?ooooo0olK6a/4obDU9@?o410@1?mRHV83ocDe=@?o6ATI1?la
<C42om3@d0?ooooo1?l61PH3oahN7P?o30`<1?l;2`/3oa<C4`Co3`l?0ol410@3ob<S8`?ooooo0om4
A4@3ocdm?@Co;2`/0omVIVH4o`X:2P?o>CTi1?lf=SH2o`l?3`?ooooo0ol51@D7oa4A4@Co5QHF0olU
9BD4oaLG5`?o6ATI0olb<S82ooooo`Co2PX:0ol?3`l4odU9B@?o2PX:1?lc<c<3oc/k>`Co:2PX0_ni
^KTAooooo`003oooool7oa4A4@?o8R8R1?lP8203o`@410Coj>SX0on<S8`3oalO7`Kooooo0ol?3`l4
od51@@?o8b<S1?n;Rh/3og1`L0Co9RHV0_m5ADD3ooooo`?oCTi>1?n5QHD3ooooo`Co;2`/0olQ8B44
oaPH60?o=3@d0_m9BDT3ooooo`Co=cLg0onLW9`4oa<C4`?oCdm?0ol`<304ode=C@?oUiNG0om4A4@3
ooooo`?o=3@d1?l;2`/3ob0P80?o;Rh^1?mFEUH3ob8R8PCoDU9B0_mbLW83ooooo`?o@D511?lU9BD3
oaLG5`Co?Shn0ol:2PX4oc`l?0?o:2PX0olh>3P2ooooo`Co6a/K0ol61PH7o`T92@Co2PX:0ol<30`4
obPX:0;o7AdM0oooool3o`l?3`Co?Cdm0ol<30`4o`L71`?o>c/k1?l10@43obLW9`?oBd];0_ooool4
oda<C0?o@d=31?lS8b<3oc<c<`Co5aLG0ol00004oa`L70;oJFUY0oooool3oaXJ6PCo1@D50omPH604
oa/K6`?o2@T91?lm?Cd3oa4A4@?oCTi>0_ooool4oa/K6`?o3`l?1?mVIVH3o`L71`Oo7AdM0olQ8B43
od51@@?ooooo0olK6a/4obDU9@?o410@1?mRHV83ocDe=@?o6ATI1?la<C42om3@d0?ooooo1?l61PH3
oahN7P?o30`<1?l;2`/3oa<C4`Co3`l?0ol410@3ob<S8`?ooooo0om4A4@3ocdm?@Co;2`/0omVIVH4
o`X:2P?o>CTi1?lf=SH2o`l?3`?ooooo0ol51@D7oa4A4@Co5QHF0olU9BD4oaLG5`?o6ATI0olb<S82
ooooo`Co2PX:0ol?3`l4odU9B@?o2PX:1?lc<c<3oc/k>`Co:2PX0_ni^KTAooooo`003oooool7oa4A
4@?o8R8R1?lP8203o`@410Coj>SX0on<S8`3oalO7`Kooooo0ol?3`l4od51@@?o8b<S1?n;Rh/3og1`
L0Co9RHV0_m5ADD3ooooo`?oCTi>1?n5QHD3ooooo`Co;2`/0olQ8B44oaPH60?o=3@d0_m9BDT3oooo
o`Co=cLg0onLW9`4oa<C4`?oCdm?0ol`<304ode=C@?oUiNG0om4A4@3ooooo`?o=3@d1?l;2`/3ob0P
80?o;Rh^1?mFEUH3ob8R8PCoDU9B0_mbLW83ooooo`?o@D511?lU9BD3oaLG5`Co?Shn0ol:2PX4oc`l
?0?o:2PX0olh>3P2ooooo`Co6a/K0ol61PH7o`T92@Co2PX:0ol<30`4obPX:0;o7AdM0oooool3o`l?
3`Co?Cdm0ol<30`4o`L71`?o>c/k1?l10@43obLW9`?oBd];0_ooool4oda<C0?o@d=31?lS8b<3oc<c
<`Co5aLG0ol00004oa`L70;oJFUY0oooool3oaXJ6PCo1@D50omPH604oa/K6`?o2@T91?lm?Cd3oa4A
4@?oCTi>0_ooool4oa/K6`?o3`l?1?mVIVH3o`L71`Oo7AdM0olQ8B43od51@@?ooooo0olK6a/4obDU
9@?o410@1?mRHV83ocDe=@?o6ATI1?la<C42om3@d0?ooooo1?l61PH3oahN7P?o30`<1?l;2`/3oa<C
4`Co3`l?0ol410@3ob<S8`?ooooo0om4A4@3ocdm?@Co;2`/0omVIVH4o`X:2P?o>CTi1?lf=SH2o`l?
3`?ooooo0ol51@D7oa4A4@Co5QHF0olU9BD4oaLG5`?o6ATI0olb<S82ooooo`Co2PX:0ol?3`l4odU9
B@?o2PX:1?lc<c<3oc/k>`Co:2PX0_ni^KTAooooo`003oooool3o`H61PCo5QHF0olK6a/4o`l?3`?o
@D511?oemOD3oi:BTP?o9RHV0_ooool4omKFeP?o10@41?m=CDd3obl_;`CoPH610onTY:@4oa0@40;o
=SHf0oooool3od10@0CoBd];0oo9bLT4oclo?`?o<S8b1?lC4a<3ocDe=@;o>3Ph0oooool4ob@T90?o
TI6A1?l20P83obPX:0?oC4a<1?m2@T83ohF5Q@?o9bLW0oooool3oaDE5@Co6a/K0olI6AT3oc0`<0Co
>c/k0olS8b<4ocXj>P;oL71`0oooool3odI6APCo6QXJ0olG5aL4ocPh>0?o2@T91?l`<303oaLG5`?o
;bl_0_ooool4o`/;2`?o20P81?l92@T3o`820PCo7alO0olG5aL4ob8R8P;o92@T0oooool3o`0000Co
@4101olA4A43obHV9PCo1@D50olh>3P3oc/k>`;ooooo1?m0@403ocLg=`Co5aLG1olQ8B43oa@D50Co
:RXZ0_m:BTX3ooooo`?o<C4a1?l820P3of=SH`Co;bl_0olA4A44odQ8B0?o410@0olm?Cd2ooooo`Co
6ATI0ol?3`l4oeYJFP?o1`L71?lP8203oaDE5@?o4a<C0om1@D43ooooo`?o8B4Q1?lk>c/3oalO7`Co
J6QX0om0@403ob4Q8@Co=SHf1Oooool4o`h>3P?o8b<S0olO7al4oa@D50?o5QHF1?lH61P3o`X:2P?o
7alO0oooool3ofIVIP?o9RHV1?l>3Ph3ohR8R0Co00000olP8204oh20P0;o>c/k0oooool3o`P820Co
0@410olL71`4o`d=3@?o6QXJ1?lH61P3oa8B4P?o5QHF0_ooool4oa0@40?o5ADE1?lg=cL3o`0000Co
@D510olU9BD4oa4A4@;oQhN74Oooool000oooooo0ol61PH4oaHF5P?o6a/K1?l?3`l3od51@@ComOGe
0onBTY83obHV9P;ooooo1?oFe]H3o`@410CoCDe=0ol_;bl4oh61P@?oY:BT1?l@4102ocHf=P?ooooo
0om0@404od];B`?obLW91?lo?cl3oc8b<PCo4a<C0ole=CD2ocPh>0?ooooo1?lT92@3oi6AT@Co0P82
0olX:2P3oda<C0Co@T920on5QHD3obLW9`?ooooo0olE5AD4oa/K6`?o6ATI0ol`<304oc/k>`?o8b<S
1?lj>SX2og1`L0?ooooo0om6ATH4oaXJ6P?o5aLG1?lh>3P3o`T92@Co<30`0olG5aL3obl_;`;ooooo
1?l;2`/3o`P820Co2@T90ol20P84oalO7`?o5aLG1?lR8R82ob@T90?ooooo0ol00004od10@0Oo4A4A
0olV9RH4o`D51@?o>3Ph0olk>c/2ooooo`Co@4100olg=cL4oaLG5`Oo8B4Q0olD51@4obXZ:P;oBTY:
0oooool3oc4a<@Co20P80omSHf<4obl_;`?o4A4A1?m8B4P3oa0@40?o?Cdm0_ooool4oaTI6@?o3`l?
1?mJFUX3o`L71`Co820P0olE5AD3oa<C4`?o@D510oooool3ob4Q8@Co>c/k0olO7al4ofQXJ0?o@410
0olQ8B44ocHf=PGooooo1?l>3Ph3ob<S8`?o7alO1?lD51@3oaHF5PCo61PH0ol:2PX3oalO7`?ooooo
0omVIVH3obHV9PCo3Ph>0on8R8P4o`0000?o820P1?n0P802oc/k>`?ooooo0ol820P4o`410@?o71`L
1?l=3@d3oaXJ6PCo61PH0olB4Q83oaHF5P;ooooo1?l@4103oaDE5@Co=cLg0ol00004od51@@?o9BDU
1?lA4A42ohN7Qa7ooooo000?ooooo`?o1PH61?lF5QH3oa/K6`Co3`l?0om1@D44ooGem@?oTY:B0olV
9RH2ooooo`Coe]KF0ol410@4ode=C@?o;bl_1?n1PH43ojBTY0Co410@0_lf=SH3ooooo`?o@4101?m;
Bd/3olW9b@Co?clo0olb<S84oa<C4`?o=CDe0_lh>3P3ooooo`Co92@T0onATI44o`820P?o:2PX0om<
C4`4od92@P?oQHF50olW9bL3ooooo`?o5ADE1?lK6a/3oaTI6@?o<30`1?lk>c/3ob<S8`Co>SXj0_m`
L703ooooo`?oATI61?lJ6QX3oaLG5`Co>3Ph0ol92@T4oc0`<0?o5aLG0ol_;bl2ooooo`Co2`/;0ol8
20P4o`T92@?o0P821?lO7al3oaLG5`Co8R8R0_lT92@3ooooo`?o00001?m0@407oa4A4@?o9RHV1?l5
1@D3ocPh>0?o>c/k0_ooool4od10@0?o=cLg1?lG5aL7ob4Q8@?o51@D1?lZ:RX2odY:BP?ooooo0ola
<C44o`P820?oHf=S1?l_;bl3oa4A4@CoB4Q80ol@4103ocdm?@;ooooo1?lI6AT3o`l?3`CoFUYJ0ol7
1`L4ob0P80?o5ADE0olC4a<3od51@@?ooooo0olQ8B44oc/k>`?o7alO1?mXJ6P3od10@0?o8B4Q1?lf
=SH5ooooo`Co3Ph>0olS8b<3oalO7`Co51@D0olF5QH4oaPH60?o2PX:0olO7al3ooooo`?oIVIV0olV
9RH4o`h>3P?oR8R81?l00003ob0P80CoP8200_lk>c/3ooooo`?o20P81?l10@43oa`L70Co3@d=0olJ
6QX4oaPH60?o4Q8B0olF5QH2ooooo`Co410@0olE5AD4ocLg=`?o00001?m1@D43obDU9@Co4A4A0_n7
QhLAooooo`003oooool3obXZ:PCo61PH0olK6a/4oalO7`?oEeMG1?nUYJD3ofYZJP?o?3`l0_ooool4
oeMGE`?o8b<S1?mfMWH3ob<S8`CoDU9B0om3@d<4ocPh>0;oOWin0oooool3oc/k>`Co9RHV0ol`<304
oa<C4`?o;2`/1?lE5AD3oahN7P;oBTY:0oooool4oc<c<`?oA4A41?lL71`3obd];@?o820P1?lf=SH3
odE5A@?o6a/K0oooool3oiRHV0CoG5aL0omRHV83od=3@`CoNGUi0on;Rh/4olC4a0Gooooo0omDE5@4
oa<C4`?oLg=c1?mUIFD3oaTI6@Co:b/[0olJ6QX3oc8b<P;ooooo1?lV9RH3oaHF5PCo:BTY0ol];Bd4
obh^;P?o_[jn1?ooool2oeEEE@?ooooo0olQ8B44obPX:0?o30`<1?mTI6@3oc@d=0Co1@D50olL71`3
oe]KF`;ooooo1?mGEeL3ocdm?@Co=SHf0olb<S84ocPh>0?o820P1?l^;Rh2ogQhN0?ooooo0om`L704
oahN7P?oLG5a1?lR8R83oeUIF@CoKFe]0olK6a/3od=3@`;ooooo1?l:2PX3ob`/;0Co5QHF0ol>3Ph4
oalO7`?o8b<S0olI6AT3oc4a<@?ooooo0olO7al4od];B`?o4A4A1?lb<S83obd];@?o?clo1?m9BDT2
odY:BP?ooooo1?m[Jf/3ol[:bP?ooooo1?li>CT3ong]k@CoYZJV0omPH603oginOP?ooooo0olZ:RX3
ocLg=`Co:BTY0om1@D47oaXJ6PCo9RHV0_l^;Rh3ooooo`?o51@D1?lT92@3oc0`<0CoIVIV0oooool4
oc/k>`?o9bLW0ola<C42ooooo`Co3@d=0omVIVH4ocdm?@?o3`l?1?lk>c/3obLW9`Co9BDU0_mADE4A
ooooo`003oooool3obXZ:PCo61PH0olK6a/4oalO7`?oEeMG1?nUYJD3ofYZJP?o?3`l0_ooool4oeMG
E`?o8b<S1?mfMWH3ob<S8`CoDU9B0om3@d<4ocPh>0;oOWin0oooool3oc/k>`Co9RHV0ol`<304oa<C
4`?o;2`/1?lE5AD3oahN7P;oBTY:0oooool4oc<c<`?oA4A41?lL71`3obd];@?o820P1?lf=SH3odE5
A@?o6a/K0oooool3oiRHV0CoG5aL0omRHV83od=3@`CoNGUi0on;Rh/4olC4a0Gooooo0omDE5@4oa<C
4`?oLg=c1?mUIFD3oaTI6@Co:b/[0olJ6QX3oc8b<P;ooooo1?lV9RH3oaHF5PCo:BTY0ol];Bd4obh^
;P?o_[jn1?ooool2oeEEE@?ooooo0olQ8B44obPX:0?o30`<1?mTI6@3oc@d=0Co1@D50olL71`3oe]K
F`;ooooo1?mGEeL3ocdm?@Co=SHf0olb<S84ocPh>0?o820P1?l^;Rh2ogQhN0?ooooo0om`L704oahN
7P?oLG5a1?lR8R83oeUIF@CoKFe]0olK6a/3od=3@`;ooooo1?l:2PX3ob`/;0Co5QHF0ol>3Ph4oalO
7`?o8b<S0olI6AT3oc4a<@?ooooo0olO7al4od];B`?o4A4A1?lb<S83obd];@?o?clo1?m9BDT2odY:
BP?ooooo1?m[Jf/3ol[:bP?ooooo1?li>CT3ong]k@CoYZJV0omPH603oginOP?ooooo0olZ:RX3ocLg
=`Co:BTY0om1@D47oaXJ6PCo9RHV0_l^;Rh3ooooo`?o51@D1?lT92@3oc0`<0CoIVIV0oooool4oc/k
>`?o9bLW0ola<C42ooooo`Co3@d=0omVIVH4ocdm?@?o3`l?1?lk>c/3obLW9`Co9BDU0_mADE4Aoooo
o`003oooool3obXZ:PCo61PH0olK6a/4oalO7`?oEeMG1?nUYJD3ofYZJP?o?3`l0_ooool4oeMGE`?o
8b<S1?mfMWH3ob<S8`CoDU9B0om3@d<4ocPh>0;oOWin0oooool3oc/k>`Co9RHV0ol`<304oa<C4`?o
;2`/1?lE5AD3oahN7P;oBTY:0oooool4oc<c<`?oA4A41?lL71`3obd];@?o820P1?lf=SH3odE5A@?o
6a/K0oooool3oiRHV0CoG5aL0omRHV83od=3@`CoNGUi0on;Rh/4olC4a0Gooooo0omDE5@4oa<C4`?o
Lg=c1?mUIFD3oaTI6@Co:b/[0olJ6QX3oc8b<P;ooooo1?lV9RH3oaHF5PCo:BTY0ol];Bd4obh^;P?o
_[jn1?ooool2oeEEE@?ooooo0olQ8B44obPX:0?o30`<1?mTI6@3oc@d=0Co1@D50olL71`3oe]KF`;o
oooo1?mGEeL3ocdm?@Co=SHf0olb<S84ocPh>0?o820P1?l^;Rh2ogQhN0?ooooo0om`L704oahN7P?o
LG5a1?lR8R83oeUIF@CoKFe]0olK6a/3od=3@`;ooooo1?l:2PX3ob`/;0Co5QHF0ol>3Ph4oalO7`?o
8b<S0olI6AT3oc4a<@?ooooo0olO7al4od];B`?o4A4A1?lb<S83obd];@?o?clo1?m9BDT2odY:BP?o
oooo1?m[Jf/3ol[:bP?ooooo1?li>CT3ong]k@CoYZJV0omPH603oginOP?ooooo0olZ:RX3ocLg=`Co
:BTY0om1@D47oaXJ6PCo9RHV0_l^;Rh3ooooo`?o51@D1?lT92@3oc0`<0CoIVIV0oooool4oc/k>`?o
9bLW0ola<C42ooooo`Co3@d=0omVIVH4ocdm?@?o3`l?1?lk>c/3obLW9`Co9BDU0_mADE4Aooooo`00
oooooooeooooo`00oooooooeooooo`00\
\>"],
  ImageRangeCache->{{{0, 499}, {59, 0}} -> {-0.525373, -0.04801, 0.0341698, \
0.0341698}, {{15.375, 42.125}, {57.5625, 30.8125}} -> {-4.59822, -0.429951, \
0.299068, 0.299068}, {{44.75, 71.5625}, {57.5625, 30.8125}} -> {-13.3927, \
-0.429951, 0.299068, 0.299068}, {{74.1875, 100.938}, {57.5625, 30.8125}} -> \
{-22.1872, -0.429951, 0.299068, 0.299068}, {{103.625, 130.375}, {57.5625, \
30.8125}} -> {-30.991, -0.429951, 0.299068, 0.299068}, {{133.062, 159.812}, \
{57.5625, 30.8125}} -> {-39.7948, -0.429951, 0.299068, 0.299068}, {{162.5, \
189.25}, {57.5625, 30.8125}} -> {-48.5987, -0.429951, 0.299068, 0.299068}, \
{{191.938, 218.688}, {57.5625, 30.8125}} -> {-57.4025, -0.429951, 0.299068, \
0.299068}, {{221.375, 248.125}, {57.5625, 30.8125}} -> {-66.2063, -0.429951, \
0.299068, 0.299068}, {{250.812, 277.562}, {57.5625, 30.8125}} -> {-75.0101, \
-0.429951, 0.299068, 0.299068}, {{280.25, 307}, {57.5625, 30.8125}} -> \
{-83.814, -0.429951, 0.299068, 0.299068}, {{309.688, 336.438}, {57.5625, \
30.8125}} -> {-92.6178, -0.429951, 0.299068, 0.299068}, {{339.125, 365.875}, \
{57.5625, 30.8125}} -> {-101.422, -0.429951, 0.299068, 0.299068}, {{368.562, \
395.312}, {57.5625, 30.8125}} -> {-110.225, -0.429951, 0.299068, 0.299068}, \
{{398, 424.75}, {57.5625, 30.8125}} -> {-119.029, -0.429951, 0.299068, \
0.299068}, {{427.375, 454.188}, {57.5625, 30.8125}} -> {-127.824, -0.429951, \
0.299068, 0.299068}, {{456.812, 483.625}, {57.5625, 30.8125}} -> {-136.628, \
-0.429951, 0.299068, 0.299068}, {{15.375, 42.125}, {28.125, 1.375}} -> \
{-4.59822, -9.23378, 0.299068, 0.299068}, {{44.75, 71.5625}, {28.125, 1.375}} -> \
{-13.3927, -9.23378, 0.299068, 0.299068}, {{74.1875, 100.938}, {28.125, \
1.375}} -> {-22.1872, -9.23378, 0.299068, 0.299068}, {{103.625, 130.375}, \
{28.125, 1.375}} -> {-30.991, -9.23378, 0.299068, 0.299068}, {{133.062, \
159.812}, {28.125, 1.375}} -> {-39.7948, -9.23378, 0.299068, 0.299068}, \
{{162.5, 189.25}, {28.125, 1.375}} -> {-48.5987, -9.23378, 0.299068, \
0.299068}, {{191.938, 218.688}, {28.125, 1.375}} -> {-57.4025, -9.23378, \
0.299068, 0.299068}, {{221.375, 248.125}, {28.125, 1.375}} -> {-66.2063, \
-9.23378, 0.299068, 0.299068}, {{250.812, 277.562}, {28.125, 1.375}} -> \
{-75.0101, -9.23378, 0.299068, 0.299068}, {{280.25, 307}, {28.125, 1.375}} -> \
{-83.814, -9.23378, 0.299068, 0.299068}, {{309.688, 336.438}, {28.125, \
1.375}} -> {-92.6178, -9.23378, 0.299068, 0.299068}, {{339.125, 365.875}, \
{28.125, 1.375}} -> {-101.422, -9.23378, 0.299068, 0.299068}, {{368.562, \
395.312}, {28.125, 1.375}} -> {-110.225, -9.23378, 0.299068, 0.299068}, \
{{398, 424.75}, {28.125, 1.375}} -> {-119.029, -9.23378, 0.299068, 0.299068}, \
{{427.375, 454.188}, {28.125, 1.375}} -> {-127.824, -9.23378, 0.299068, \
0.299068}, {{456.812, 483.625}, {28.125, 1.375}} -> {-136.628, -9.23378, \
0.299068, 0.299068}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .12 
%%ImageSize: 500 60 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0595238 0.00285714 0.0595238 [
[ 0 0 0 0 ]
[ 1 .12 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .12 L
0 .12 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.00285714 0.0782313 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
9E192A08
7C053D32
45003DAC
08182AFF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0836735 0.00285714 0.138095 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
06061D2F
06010928
14464000
541D08FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.143537 0.00285714 0.197959 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
53FF251B
62FA0903
80D90500
78BA170E
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.203401 0.00285714 0.257823 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0A121136
1D221750
2F0200FF
100114F9
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.263265 0.00285714 0.317687 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
FF361E4D
2C22102E
00313722
123E1021
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.323129 0.00285714 0.377551 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
03107F9D
001EFF9F
0022D8A6
091E5B5E
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.382993 0.00285714 0.437415 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0F3C2217
31FF5B12
1B03112B
02010003
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.442857 0.00285714 0.497279 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
D23CB190
FF1ABD8A
F721AF80
FD007566
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.502721 0.00285714 0.557143 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
3D6AEF14
354FEF00
3F47FF09
4439EE1A
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.562585 0.00285714 0.617007 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
15101B27
0912293F
0025875E
0C23FF4C
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.622449 0.00285714 0.676871 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
06A7D427
0FD5DA0B
23FFB200
53C49A07
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.682313 0.00285714 0.736735 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1E0107FF
15080A1B
0815061C
0C000911
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.742177 0.00285714 0.796599 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
F018A61D
FF03C122
F500E834
CB18F11F
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.802041 0.00285714 0.856463 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1E06A807
1700E100
1D01FF15
100CD318
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.861905 0.00285714 0.916327 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
A2CBBE08
DAEBC500
D7FFCD01
91D8B50C
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.921769 0.00285714 0.97619 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
D659061C
F1450009
FF400721
FD340A1A
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.0627211 0.0782313 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
110D1EFC
0B0519FF
0D061ADB
1A0011AF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0836735 0.0627211 0.138095 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
46101EE6
3B0015F6
3B071CFF
380423D0
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.143537 0.0627211 0.197959 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
08DC3CBE
00FF2EA9
27CD158C
1F611F72
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.203401 0.0627211 0.257823 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1A0B081E
1A0E014D
290F13FF
19181000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.263265 0.0627211 0.317687 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
A0150448
FF1900B2
790E035B
36091D3F
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.323129 0.0627211 0.377551 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
000AFF4B
0A095C46
01071335
0A001419
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.382993 0.0627211 0.437415 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
21FF5129
1B331A7F
0017213D
0612234F
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.442857 0.0627211 0.497279 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
FF750AD8
E96904FF
BE5B00DE
713D06C8
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.502721 0.0627211 0.557143 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
FF9553ED
DD9C00CB
C8D809E2
A2CE17FE
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.562585 0.0627211 0.617007 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
251AE36F
051DFF65
0034ED4C
0F38D23D
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.622449 0.0627211 0.676871 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1F2A4D67
103E3565
008B3552
14FF174D
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.682313 0.0627211 0.736735 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
08020699
17001AFF
0E122B59
0A20143B
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.742177 0.0627211 0.796599 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
B94E0324
DD2E0015
F2160418
FF170C1C
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.802041 0.0627211 0.856463 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
00A2DD61
06D6FF6B
07E0F760
1BADC64F
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.861905 0.0627211 0.916327 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1FB80F00
22DF0E0A
1DFF070C
20C70115
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.921769 0.0627211 0.97619 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0A22272C
34382D3F
FF691355
65340047
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{500, 60},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007d0000?2000`400?l00000o`00003ooooooooe
ooooo`00oooooooeooooo`00oooooooeooooo`003oooool7oijNWPOo6ATI1olZ:RX6o`P820;ooooo
3_l61PH7oadM7@Ko;bl_0oooool7oe=CD`Oooooo1olU9BD5oa/K6`?ooooo1ol:2PX7oa8B4POo4A4A
1_lf=SH:ooooo`Ko=SHf1olN7Qh6ode=C@?ooooo1ol30`<7oa0@40OoOgmo1_nMWId2ooooo`Oo3`l?
1oll?3`7ob8R8PKo5aLG0oooool7om;BdPOo?3`l1ona/K46oi2@T0;ooooo1olm?Cd7ofYZJPOokno_
1_lD51@3ooooo`Oo5ADE1ol@4107oa/K6`Ko9bLW0_ooool7o`H61POoYjNW1ooDe=@6obLW9`?ooooo
1olN7Qh7o`410@Ko1`L72Oooool7oo3`l0Oo61PH1onVYZH6oadM7@?ooooo1_lN7Qh7o`H61POoZ:RX
1_l71`L3ooooo`OoXZ:R1oo;bl/7okjn_PKo20P80_ooool7omKFePOoFEUI1ol61PH6oa`L717ooooo
000?ooooo`OoWYjN1olI6AT7obXZ:PKo20P80_ooool>o`H61POo7AdM1_l_;bl3ooooo`OoDe=C1ooo
ool7obDU9@Go6a/K0oooool7o`X:2POo4Q8B1olA4A46ocHf=P[ooooo1_lf=SH7oahN7PKoCDe=0ooo
ool7o`<30`Oo410@1omoOgl6oifMW@;ooooo1ol?3`l7oc`l?0Oo8R8R1_lG5aL3ooooo`Ood];B1oll
?3`7ok6a/@KoT92@0_ooool7ocdm?@OoJVYZ1oo_knl6oa@D50?ooooo1olE5AD7oa0@40Oo6a/K1_lW
9bL2ooooo`Oo1PH61onWYjL7omCDe0Ko9bLW0oooool7oahN7POo0@411_l71`L9ooooo`Ool?3`1olH
61P7ojJVYPKo7AdM0oooool6oahN7POo1PH61onXZ:P6o`L71`?ooooo1onRXZ87ol_;b`Oo_[jn1_l8
20P2ooooo`Ooe]KF1omIFET7o`H61PKo71`L4Oooool000oooooo1onNWYh7oaTI6@Oo:RXZ1_l820P2
ooooo`ko1PH61olM7Ad6obl_;`?ooooo1omCDe<7ooooo`Oo9BDU1OlK6a/3ooooo`Oo2PX:1olB4Q87
oa4A4@Ko=SHf2_ooool6ocHf=POo7QhN1_m=CDd3ooooo`Oo0`<31ol@4107ogmoO`KoWIfM0_ooool7
o`l?3`Oo?3`l1olR8R86oaLG5`?ooooo1ooBd]87oc`l?0Oo/K6a1_n@T902ooooo`Oo?Cdm1omZJVX7
ono_k`Ko51@D0oooool7oaDE5@Oo410@1olK6a/6obLW9`;ooooo1ol61PH7ojNWY`Ooe=CD1_lW9bL3
ooooo`Oo7QhN1ol10@46o`L71`Wooooo1oo`l?07oaPH60OoYZJV1_lM7Ad3ooooo`Ko7QhN1ol61PH7
ojRXZ0Ko1`L70oooool7oj:RXPOobl_;1onn_[h6o`P820;ooooo1ooFe]H7oeUIF@Oo1PH61_lL71`A
ooooo`003oooool7oijNWPOo6ATI1olZ:RX6o`P820;ooooo3_l61PH7oadM7@Ko;bl_0oooool7oe=C
D`Oooooo1olU9BD5oa/K6`?ooooo1ol:2PX7oa8B4POo4A4A1_lf=SH:ooooo`Ko=SHf1olN7Qh6ode=
C@?ooooo1ol30`<7oa0@40OoOgmo1_nMWId2ooooo`Oo3`l?1oll?3`7ob8R8PKo5aLG0oooool7om;B
dPOo?3`l1ona/K46oi2@T0;ooooo1olm?Cd7ofYZJPOokno_1_lD51@3ooooo`Oo5ADE1ol@4107oa/K
6`Ko9bLW0_ooool7o`H61POoYjNW1ooDe=@6obLW9`?ooooo1olN7Qh7o`410@Ko1`L72Oooool7oo3`
l0Oo61PH1onVYZH6oadM7@?ooooo1_lN7Qh7o`H61POoZ:RX1_l71`L3ooooo`OoXZ:R1oo;bl/7okjn
_PKo20P80_ooool7omKFePOoFEUI1ol61PH6oa`L717ooooo000?ooooo`OoWYjN1olI6AT7obXZ:PKo
20P80_ooool>o`H61POo7AdM1_l_;bl3ooooo`OoDe=C1oooool7obDU9@Go6a/K0oooool7o`X:2POo
4Q8B1olA4A46ocHf=P[ooooo1_lf=SH7oahN7PKoCDe=0oooool7o`<30`Oo410@1omoOgl6oifMW@;o
oooo1ol?3`l7oc`l?0Oo8R8R1_lG5aL3ooooo`Ood];B1oll?3`7ok6a/@KoT92@0_ooool7ocdm?@Oo
JVYZ1oo_knl6oa@D50?ooooo1olE5AD7oa0@40Oo6a/K1_lW9bL2ooooo`Oo1PH61onWYjL7omCDe0Ko
9bLW0oooool7oahN7POo0@411_l71`L9ooooo`Ool?3`1olH61P7ojJVYPKo7AdM0oooool6oahN7POo
1PH61onXZ:P6o`L71`?ooooo1onRXZ87ol_;b`Oo_[jn1_l820P2ooooo`Ooe]KF1omIFET7o`H61PKo
71`L4Oooool000oooooo1onNWYh7oaTI6@Oo:RXZ1_l820P2ooooo`ko1PH61olM7Ad6obl_;`?ooooo
1omCDe<7ooooo`Oo9BDU1OlK6a/3ooooo`Oo2PX:1olB4Q87oa4A4@Ko=SHf2_ooool6ocHf=POo7QhN
1_m=CDd3ooooo`Oo0`<31ol@4107ogmoO`KoWIfM0_ooool7o`l?3`Oo?3`l1olR8R86oaLG5`?ooooo
1ooBd]87oc`l?0Oo/K6a1_n@T902ooooo`Oo?Cdm1omZJVX7ono_k`Ko51@D0oooool7oaDE5@Oo410@
1olK6a/6obLW9`;ooooo1ol61PH7ojNWY`Ooe=CD1_lW9bL3ooooo`Oo7QhN1ol10@46o`L71`Wooooo
1oo`l?07oaPH60OoYZJV1_lM7Ad3ooooo`Ko7QhN1ol61PH7ojRXZ0Ko1`L70oooool7oj:RXPOobl_;
1onn_[h6o`P820;ooooo1ooFe]H7oeUIF@Oo1PH61_lL71`Aooooo`003oooool7ogalO0Oo1@D51olm
?Cd6oc8b<P;ooooo1ol61PH7o`410@Oo2@T91_lX:2P3ooooo`OoHV9R1oojn_X7o`T92@Go0`<30ooo
ool7oadM7@Oo8R8R1olG5aL6oe1@D0?ooooo1ol/;2`6ob8R8POo410@1_l^;Rh3ooooo`Oo00001olN
7Qh7ooooo`KoWinO0_ooool7oc4a<@Oooooo1omKFe/6oa8B4P[ooooo1olJ6QX7okfm_@KoRXZ:0_oo
ool7ocDe=@OoCdm?1oo_knl6o`0000?ooooo1ol92@T7oa8B4POo:BTY1_lo?cl2ooooo`Oo3`l?1ooE
eMD7om[JfPKo2`/;0oooool7oaDE5@Oo20P81_l:2PX6oa/K6`[ooooo1ol30`<7ol71`@Ko8R8R0ooo
ool6oaLG5`Oo00001ooQhN46o`0000?ooooo1ooJf]X7on_[j`OoaLG51_l00002ooooo`OolO7a1om5
ADD7o`0000Ko2@T94Oooool000oooooo1omlO7`7o`D51@Oo?Cdm1_lb<S82ooooo`Oo1PH61ol10@47
o`T92@Ko:2PX0oooool7of9RHPOon_[j1ol92@T5o`<30`?ooooo1olM7Ad7ob8R8POo5aLG1_m@D503
ooooo`Oo;2`/1_lR8R87oa0@40Ko;Rh^0oooool7o`0000Oo7QhN1oooool6oinOW`;ooooo1ola<C47
ooooo`OoFe]K1_lB4Q8:ooooo`Oo6QXJ1onm_Kd6ohZ:RP;ooooo1ole=CD7odm?C`Ookno_1_l00003
ooooo`Oo2@T91olB4Q87obTY:@Ko?clo0_ooool7o`l?3`OoeMGE1ooJf]X6o`/;2`?ooooo1olE5AD7
o`P820Ko2PX:1_lK6a/:ooooo`Oo0`<31oo1`L46ob8R8P?ooooo1_lG5aL7o`0000OohN7Q1_l00003
ooooo`Oof][J1oo[jn/7olG5a@Ko00000_ooool7oo7al@OoADE51ol00006o`T92A7ooooo000?oooo
o`OoO7al1ol51@D7ocdm?@Ko<S8b0_ooool7o`H61POo0@411ol92@T6obPX:0?ooooo1omRHV87oo[j
nPOo2@T91Ol30`<3ooooo`Oo7AdM1olR8R87oaLG5`KoD51@0oooool7ob`/;0Ko8R8R1ol@4106obh^
;P?ooooo1ol00007oahN7POooooo1_nOWil2ooooo`Oo<C4a1oooool7oe]KF`Ko4Q8B2_ooool7oaXJ
6POo_Kfm1_n:RXX2ooooo`Oo=CDe1om?Cdl7ono_k`Ko00000oooool7o`T92@Oo4Q8B1olY:BT6oclo
?`;ooooo1ol?3`l7omGEe@Oof][J1_l;2`/3ooooo`Oo5ADE1ol820P6o`X:2PKo6a/K2_ooool7o`<3
0`Oo`L711_lR8R83ooooo`Ko5aLG1ol00007on7Qh@Ko00000oooool7om[JfPOojn_[1oo5aLD6o`00
00;ooooo1ooalO47odE5A@Oo00001_l92@TAooooo`003oooool7ogalO0Oo1@D51olm?Cd6oc8b<P;o
oooo1ol61PH7o`410@Oo2@T91_lX:2P3ooooo`OoHV9R1oojn_X7o`T92@Go0`<30oooool7oadM7@Oo
8R8R1olG5aL6oe1@D0?ooooo1ol/;2`6ob8R8POo410@1_l^;Rh3ooooo`Oo00001olN7Qh7ooooo`Ko
WinO0_ooool7oc4a<@Oooooo1omKFe/6oa8B4P[ooooo1olJ6QX7okfm_@KoRXZ:0_ooool7ocDe=@Oo
Cdm?1oo_knl6o`0000?ooooo1ol92@T7oa8B4POo:BTY1_lo?cl2ooooo`Oo3`l?1ooEeMD7om[JfPKo
2`/;0oooool7oaDE5@Oo20P81_l:2PX6oa/K6`[ooooo1ol30`<7ol71`@Ko8R8R0oooool6oaLG5`Oo
00001ooQhN46o`0000?ooooo1ooJf]X7on_[j`OoaLG51_l00002ooooo`OolO7a1om5ADD7o`0000Ko
2@T94Oooool000oooooo1omlO7`7o`D51@Oo?Cdm1_lb<S82ooooo`Oo1PH61ol10@47o`T92@Ko:2PX
0oooool7of9RHPOon_[j1ol92@T5o`<30`?ooooo1olM7Ad7ob8R8POo5aLG1_m@D503ooooo`Oo;2`/
1_lR8R87oa0@40Ko;Rh^0oooool7o`0000Oo7QhN1oooool6oinOW`;ooooo1ola<C47ooooo`OoFe]K
1_lB4Q8:ooooo`Oo6QXJ1onm_Kd6ohZ:RP;ooooo1ole=CD7odm?C`Ookno_1_l00003ooooo`Oo2@T9
1olB4Q87obTY:@Ko?clo0_ooool7o`l?3`OoeMGE1ooJf]X6o`/;2`?ooooo1olE5AD7o`P820Ko2PX:
1_lK6a/:ooooo`Oo0`<31oo1`L46ob8R8P?ooooo1_lG5aL7o`0000OohN7Q1_l00003ooooo`Oof][J
1oo[jn/7olG5a@Ko00000_ooool7oo7al@OoADE51ol00006o`T92A7ooooo000?ooooo`OoO7al1ol5
1@D7ocdm?@Ko<S8b0_ooool7o`H61POo0@411ol92@T6obPX:0?ooooo1omRHV87oo[jnPOo2@T91Ol3
0`<3ooooo`Oo7AdM1olR8R87oaLG5`KoD51@0oooool7ob`/;0Ko8R8R1ol@4106obh^;P?ooooo1ol0
0007oahN7POooooo1_nOWil2ooooo`Oo<C4a1oooool7oe]KF`Ko4Q8B2_ooool7oaXJ6POo_Kfm1_n:
RXX2ooooo`Oo=CDe1om?Cdl7ono_k`Ko00000oooool7o`T92@Oo4Q8B1olY:BT6oclo?`;ooooo1ol?
3`l7omGEe@Oof][J1_l;2`/3ooooo`Oo5ADE1ol820P6o`X:2PKo6a/K2_ooool7o`<30`Oo`L711_lR
8R83ooooo`Ko5aLG1ol00007on7Qh@Ko00000oooool7om[JfPOojn_[1oo5aLD6o`0000;ooooo1ooa
lO47odE5A@Oo00001_l92@TAooooo`003oooool7ogalO0Oo1@D51olm?Cd6oc8b<P;ooooo1ol61PH7
o`410@Oo2@T91_lX:2P3ooooo`OoHV9R1oojn_X7o`T92@Go0`<30oooool7oadM7@Oo8R8R1olG5aL6
oe1@D0?ooooo1ol/;2`6ob8R8POo410@1_l^;Rh3ooooo`Oo00001olN7Qh7ooooo`KoWinO0_ooool7
oc4a<@Oooooo1omKFe/6oa8B4P[ooooo1olJ6QX7okfm_@KoRXZ:0_ooool7ocDe=@OoCdm?1oo_knl6
o`0000?ooooo1ol92@T7oa8B4POo:BTY1_lo?cl2ooooo`Oo3`l?1ooEeMD7om[JfPKo2`/;0oooool7
oaDE5@Oo20P81_l:2PX6oa/K6`[ooooo1ol30`<7ol71`@Ko8R8R0oooool6oaLG5`Oo00001ooQhN46
o`0000?ooooo1ooJf]X7on_[j`OoaLG51_l00002ooooo`OolO7a1om5ADD7o`0000Ko2@T94Oooool0
00oooooo1om5ADD7o`0000Oo?Cdm1_n/[:`2ooooo`Oo51@D1om6ATH7od10@0Ko00000oooool7oh20
P0OofMWI1ol51@D5o`0000?ooooo1ol_;bl7o`820POo00002Oooool7o`0000Ko<C4a1olg=cL6ob8R
8P?ooooo1ol00007ob8R8POof=SH1_nVYZH2ooooo`Oo6a/K1ol30`<7oa4A4@Ko:b/[0oooool7ooOg
m`Oo8B4Q1on_[jl6oh20P0;ooooo1olo?cl7odM7A`Oooooo1_l92@T3ooooo`Oo00001olU9BD7ohN7
Q`KoGUiN0_ooool7ob<S8`Oooooo1onb/[86o`0000?ooooo1ol820P7oaDE5@Ko1PH61_lL71`3oooo
o`OomOGe1ol00007onSXj0Ko=3@d0oooool6oadM7@Oo0@411oooool6oaDE5@?ooooo1ooGemL7oooo
o`OocLg=1_l10@49ooooo`Oo@4101ol71`L6ob4Q8A7ooooo000?ooooo`OoADE51ol00007ocdm?@Ko
[:b/0_ooool7oa@D50OoATI61om0@406o`0000?ooooo1on0P807omWIf@Oo1@D51Ol00003ooooo`Oo
;bl_1ol20P87o`0000Wooooo1ol00006oc4a<@Oo=cLg1_lR8R83ooooo`Oo00001olR8R87omSHf0Ko
YZJV0_ooool7oa/K6`Oo0`<31olA4A46ob/[:`?ooooo1oogmoL7ob4Q8@Oo[jn_1_n0P802ooooo`Oo
?clo1om7AdL7ooooo`Ko2@T90oooool7o`0000Oo9BDU1on7QhL6oeiNGP;ooooo1olS8b<7ooooo`Oo
/[:b1_l00003ooooo`Oo20P81olE5AD6o`H61PKo71`L0oooool7ooGem@Oo00001ooXj>P6oc@d=0?o
oooo1_lM7Ad7o`410@Oooooo1_lE5AD3ooooo`OoemOG1oooool7olg=c@Ko0@412Oooool7od10@0Oo
1`L71_lQ8B4Aooooo`003oooool7odE5A@Oo00001olm?Cd6ojb/[0;ooooo1olD51@7odI6APOo@410
1_l00003ooooo`OoP8201ooIfMT7o`D51@Go00000oooool7obl_;`Oo0P821ol00009ooooo`Oo0000
1_la<C47ocLg=`Ko8R8R0oooool7o`0000Oo8R8R1ooHf=P6ojJVYP;ooooo1olK6a/7o`<30`Oo4A4A
1_l[:b/3ooooo`OomoOg1olQ8B47ojn_[`KoP8200_ooool7oclo?`OoAdM71oooool6o`T92@?ooooo
1ol00007obDU9@OoQhN71_mNGUh2ooooo`Oo8b<S1oooool7ok:b/PKo00000oooool7o`P820Oo5ADE
1_l61PH6oa`L70?ooooo1ooemOD7o`0000Ooj>SX1_ld=3@3ooooo`Ko7AdM1ol10@47ooooo`Ko5ADE
0oooool7omOGe`Oooooo1oo=cLd6o`410@Wooooo1om0@407o`L71`Ko8B4Q4Oooool000oooooo1om5
ADD7o`0000Oo?Cdm1_n/[:`2ooooo`Oo51@D1om6ATH7od10@0Ko00000oooool7oh20P0OofMWI1ol5
1@D5o`0000?ooooo1ol_;bl7o`820POo00002Oooool7o`0000Ko<C4a1olg=cL6ob8R8P?ooooo1ol0
0007ob8R8POof=SH1_nVYZH2ooooo`Oo6a/K1ol30`<7oa4A4@Ko:b/[0oooool7ooOgm`Oo8B4Q1on_
[jl6oh20P0;ooooo1olo?cl7odM7A`Oooooo1_l92@T3ooooo`Oo00001olU9BD7ohN7Q`KoGUiN0_oo
ool7ob<S8`Oooooo1onb/[86o`0000?ooooo1ol820P7oaDE5@Ko1PH61_lL71`3ooooo`OomOGe1ol0
0007onSXj0Ko=3@d0oooool6oadM7@Oo0@411oooool6oaDE5@?ooooo1ooGemL7ooooo`OocLg=1_l1
0@49ooooo`Oo@4101ol71`L6ob4Q8A7ooooo000?ooooo`OoADE51ol00007ocdm?@Ko[:b/0_ooool7
oa@D50OoATI61om0@406o`0000?ooooo1on0P807omWIf@Oo1@D51Ol00003ooooo`Oo;bl_1ol20P87
o`0000Wooooo1ol00006oc4a<@Oo=cLg1_lR8R83ooooo`Oo00001olR8R87omSHf0KoYZJV0_ooool7
oa/K6`Oo0`<31olA4A46ob/[:`?ooooo1oogmoL7ob4Q8@Oo[jn_1_n0P802ooooo`Oo?clo1om7AdL7
ooooo`Ko2@T90oooool7o`0000Oo9BDU1on7QhL6oeiNGP;ooooo1olS8b<7ooooo`Oo/[:b1_l00003
ooooo`Oo20P81olE5AD6o`H61PKo71`L0oooool7ooGem@Oo00001ooXj>P6oc@d=0?ooooo1_lM7Ad7
o`410@Oooooo1_lE5AD3ooooo`OoemOG1oooool7olg=c@Ko0@412Oooool7od10@0Oo1`L71_lQ8B4A
ooooo`003oooool7odE5A@Oo00001olm?Cd6ojb/[0;ooooo1olD51@7odI6APOo@4101_l00003oooo
o`OoP8201ooIfMT7o`D51@Go00000oooool7obl_;`Oo0P821ol00009ooooo`Oo00001_la<C47ocLg
=`Ko8R8R0oooool7o`0000Oo8R8R1ooHf=P6ojJVYP;ooooo1olK6a/7o`<30`Oo4A4A1_l[:b/3oooo
o`OomoOg1olQ8B47ojn_[`KoP8200_ooool7oclo?`OoAdM71oooool6o`T92@?ooooo1ol00007obDU
9@OoQhN71_mNGUh2ooooo`Oo8b<S1oooool7ok:b/PKo00000oooool7o`P820Oo5ADE1_l61PH6oa`L
70?ooooo1ooemOD7o`0000Ooj>SX1_ld=3@3ooooo`Ko7AdM1ol10@47ooooo`Ko5ADE0oooool7omOG
e`Oooooo1oo=cLd6o`410@Wooooo1om0@407o`L71`Ko8B4Q4Oooool000oooooo1ol820P7oaPH60Oo
:RXZ2?ooool7oeADE0Oo7AdM1ol820P9ooooo`OoN7Qh1onj^[X7oaLG5`Go3Ph>0oooool7oa0@40Oo
0@411olD51@6ooWin@?ooooo1olB4Q86ochn?POo410@1_lQ8B43ooooo`Oo2@T91olN7Qh7oe]KF`Ko
GUiN0_ooool7o`820POo0@411ol00006o`<30`?ooooo1oomoOd7o`0000OoMGEe1_mVIVH2ooooo`Oo
A4A41oli>CT7onk^kPKo6QXJ0oooool7o``<30Oo8b<S1oooool6oda<C0;ooooo1omCDe<7olC4a0Oo
VYZJ1_l71`L3ooooo`Oo30`<1ol00006o`T92@Ko4A4A0oooool7ol_;b`Oo61PH1ooalO46oalO7`?o
oooo1_l@4107o``<30Oodm?C1_lH61P3ooooo`OoTI6A1ooHf=P7okFe]@Ko30`<0_ooool7oogmo@Oo
=3@d1ol:2PX6oaXJ6Q7ooooo000?ooooo`Oo20P81olH61P7obXZ:PSooooo1omDE5@7oadM7@Oo20P8
2Oooool7ogQhN0Oo^[Zj1olG5aL5o`h>3P?ooooo1ol@4107o`410@Oo51@D1_oinOT3ooooo`Oo4Q8B
1_ln?Sh7oa0@40Ko8B4Q0oooool7o`T92@Oo7QhN1omKFe/6oeiNGP;ooooo1ol20P87o`410@Oo0000
1_l30`<3ooooo`OooOgm1ol00007ogEeM@KoIVIV0_ooool7odA4A0Oo>CTi1oo^k^h6oaXJ6P?ooooo
1ol<30`7ob<S8`Oooooo1_m<C4`2ooooo`OoDe=C1oo4a<@7oiZJVPKo1`L70oooool7o``<30Oo0000
1_l92@T6oa4A4@?ooooo1oo;bl/7oaPH60OolO7a1_lO7al3ooooo`Ko410@1ol<30`7om?Cd`Ko61PH
0oooool7oi6AT@Oof=SH1one]KD6o``<30;ooooo1oomoOd7oc@d=0Oo2PX:1_lJ6QXAooooo`003ooo
ool7o`P820Oo61PH1olZ:RX8ooooo`OoE5AD1olM7Ad7o`P820Wooooo1omhN7P7okZj^POo5aLG1Ol>
3Ph3ooooo`Oo410@1ol10@47oa@D50KonOWi0oooool7oa8B4PKo?Shn1ol@4106ob4Q8@?ooooo1ol9
2@T7oahN7POoFe]K1_mNGUh2ooooo`Oo0P821ol10@47o`0000Ko0`<30oooool7oogmo@Oo00001ome
MGD6ofIVIP;ooooo1om4A4@7ocTi>@Ook^k^1_lJ6QX3ooooo`Oo30`<1olS8b<7ooooo`KoC4a<0_oo
ool7oe=CD`Ooa<C41onJVYX6o`L71`?ooooo1ol<30`7o`0000Ko2@T91_lA4A43ooooo`Oobl_;1olH
61P7oo7al@Ko7alO0oooool6oa0@40Oo30`<1ooCdm<6oaPH60?ooooo1onATI47omSHf0Oo]KFe1_l<
30`2ooooo`OooOgm1old=3@7o`X:2PKo6QXJ4Oooool000oooooo1ol820P7oaPH60Oo:RXZ2?ooool7
oeADE0Oo7AdM1ol820P9ooooo`OoN7Qh1onj^[X7oaLG5`Go3Ph>0oooool7oa0@40Oo0@411olD51@6
ooWin@?ooooo1olB4Q86ochn?POo410@1_lQ8B43ooooo`Oo2@T91olN7Qh7oe]KF`KoGUiN0_ooool7
o`820POo0@411ol00006o`<30`?ooooo1oomoOd7o`0000OoMGEe1_mVIVH2ooooo`OoA4A41oli>CT7
onk^kPKo6QXJ0oooool7o``<30Oo8b<S1oooool6oda<C0;ooooo1omCDe<7olC4a0OoVYZJ1_l71`L3
ooooo`Oo30`<1ol00006o`T92@Ko4A4A0oooool7ol_;b`Oo61PH1ooalO46oalO7`?ooooo1_l@4107
o``<30Oodm?C1_lH61P3ooooo`OoTI6A1ooHf=P7okFe]@Ko30`<0_ooool7oogmo@Oo=3@d1ol:2PX6
oaXJ6Q7ooooo000?ooooo`Oo20P81olH61P7obXZ:PSooooo1omDE5@7oadM7@Oo20P82Oooool7ogQh
N0Oo^[Zj1olG5aL5o`h>3P?ooooo1ol@4107o`410@Oo51@D1_oinOT3ooooo`Oo4Q8B1_ln?Sh7oa0@
40Ko8B4Q0oooool7o`T92@Oo7QhN1omKFe/6oeiNGP;ooooo1ol20P87o`410@Oo00001_l30`<3oooo
o`OooOgm1ol00007ogEeM@KoIVIV0_ooool7odA4A0Oo>CTi1oo^k^h6oaXJ6P?ooooo1ol<30`7ob<S
8`Oooooo1_m<C4`2ooooo`OoDe=C1oo4a<@7oiZJVPKo1`L70oooool7o``<30Oo00001_l92@T6oa4A
4@?ooooo1oo;bl/7oaPH60OolO7a1_lO7al3ooooo`Ko410@1ol<30`7om?Cd`Ko61PH0oooool7oi6A
T@Oof=SH1one]KD6o``<30;ooooo1oomoOd7oc@d=0Oo2PX:1_lJ6QXAooooo`003oooool7o`P820Oo
61PH1olZ:RX8ooooo`OoE5AD1olM7Ad7o`P820Wooooo1omhN7P7okZj^POo5aLG1Ol>3Ph3ooooo`Oo
410@1ol10@47oa@D50KonOWi0oooool7oa8B4PKo?Shn1ol@4106ob4Q8@?ooooo1ol92@T7oahN7POo
Fe]K1_mNGUh2ooooo`Oo0P821ol10@47o`0000Ko0`<30oooool7oogmo@Oo00001omeMGD6ofIVIP;o
oooo1om4A4@7ocTi>@Ook^k^1_lJ6QX3ooooo`Oo30`<1olS8b<7ooooo`KoC4a<0_ooool7oe=CD`Oo
a<C41onJVYX6o`L71`?ooooo1ol<30`7o`0000Ko2@T91_lA4A43ooooo`Oobl_;1olH61P7oo7al@Ko
7alO0oooool6oa0@40Oo30`<1ooCdm<6oaPH60?ooooo1onATI47omSHf0Oo]KFe1_l<30`2ooooo`Oo
oOgm1old=3@7o`X:2PKo6QXJ4Oooool000oooooo1ol820P7oaPH60Oo:RXZ2?ooool7oeADE0Oo7AdM
1ol820P9ooooo`OoN7Qh1onj^[X7oaLG5`Go3Ph>0oooool7oa0@40Oo0@411olD51@6ooWin@?ooooo
1olB4Q86ochn?POo410@1_lQ8B43ooooo`Oo2@T91olN7Qh7oe]KF`KoGUiN0_ooool7o`820POo0@41
1ol00006o`<30`?ooooo1oomoOd7o`0000OoMGEe1_mVIVH2ooooo`OoA4A41oli>CT7onk^kPKo6QXJ
0oooool7o``<30Oo8b<S1oooool6oda<C0;ooooo1omCDe<7olC4a0OoVYZJ1_l71`L3ooooo`Oo30`<
1ol00006o`T92@Ko4A4A0oooool7ol_;b`Oo61PH1ooalO46oalO7`?ooooo1_l@4107o``<30Oodm?C
1_lH61P3ooooo`OoTI6A1ooHf=P7okFe]@Ko30`<0_ooool7oogmo@Oo=3@d1ol:2PX6oaXJ6Q7ooooo
003oooooooGooooo003oooooooGooooo003oooooooGooooo000?ooooo`Oo4A4A1ol=3@d7oahN7PKo
o?cl0_ooool7odI6APOo410@1olN7Qh6onKViP?ooooo1ol820P7omcLg0Oo?3`l1Onn_[h3ooooo`Oo
6QXJ1ol;2`/7o`P820Ko7QhN0oooool7oj2PX0Ko5ADE1ol410@6odQ8B0?ooooo1ol00007o`X:2POo
oooo1_m;Bd/2ooooo`Oo8B4Q1oooool7oe5AD@Ko:BTY2_ooool7ogEeM@Oo2PX:1_oHf=P9ooooo`Oo
UIFE1omCDe<6ong]k@?ooooo1olU9BD7oaXJ6POohn?S1_m_Kfl2ooooo`Oo7alO1olZ:RX7ode=C@Ko
IfMW0oooool7o`P820Oo0P821_l61PH6oiVIV@?ooooo1oni^KT7odi>CPOo0`<31_lT92@3ooooo`Ko
00001onRXZ87omgMg@KoHF5Q0oooool7oalO7`Oo^;Rh1ol?3`l6o`0000;ooooo1ol:2PX7ob8R8POo
9bLW1_l/;2`Aooooo`003oooool7oa4A4@Oo3@d=1olN7Qh6ooclo0;ooooo1om6ATH7oa0@40Oo7QhN
1_oVi^H3ooooo`Oo20P81ooLg=`7oc`l?0Go_[jn0oooool7oaXJ6POo2`/;1ol820P6oahN7P?ooooo
1onPX:06oaDE5@Oo10@41_m8B4P3ooooo`Oo00001ol:2PX7ooooo`KoBd];0_ooool7ob4Q8@Oooooo
1omADE46obTY:@[ooooo1omeMGD7o`X:2PKof=SH2Oooool7oiFEU@OoDe=C1_o]kNd3ooooo`Oo9BDU
1olJ6QX7on?Sh`KoKfm_0_ooool7oalO7`Oo:RXZ1om=CDd6ofMWI`?ooooo1ol820P7o`820PKo1PH6
1_nIVIT3ooooo`Oo^KVi1om>CTh7o`<30`Ko92@T0oooool6o`0000OoXZ:R1ooMgMd6of5QH@?ooooo
1olO7al7okRh^0Oo3`l?1_l00002ooooo`Oo2PX:1olR8R87obLW9`Ko;2`/4Oooool000oooooo1olA
4A47o`d=3@Oo7QhN1_olo?`2ooooo`OoATI61ol@4107oahN7PKoi^KV0oooool7o`P820Oog=cL1oll
?3`5okjn_P?ooooo1olJ6QX7o`/;2`Oo20P81_lN7Qh3ooooo`OoX:2P1_lE5AD7o`@410KoB4Q80ooo
ool7o`0000Oo2PX:1oooool6od];B`;ooooo1olQ8B47ooooo`OoDE5A1_lY:BT:ooooo`OoMGEe1ol:
2PX6omSHf0Wooooo1onEUID7oe=CD`KokNg]0oooool7obDU9@Oo6QXJ1ooShn<6ofm_K`;ooooo1olO
7al7obXZ:POoCDe=1_mWIfL3ooooo`Oo20P81ol20P86o`H61PKoVIVI0oooool7okVi^@OoCTi>1ol3
0`<6ob@T90?ooooo1_l00007oj:RXPOogMgM1_mQHF43ooooo`Oo7alO1onh^;P7o`l?3`Ko00000_oo
ool7o`X:2POo8R8R1olW9bL6ob`/;17ooooo000?ooooo`Oo4A4A1ol=3@d7oahN7PKoo?cl0_ooool7
odI6APOo410@1olN7Qh6onKViP?ooooo1ol820P7omcLg0Oo?3`l1Onn_[h3ooooo`Oo6QXJ1ol;2`/7
o`P820Ko7QhN0oooool7oj2PX0Ko5ADE1ol410@6odQ8B0?ooooo1ol00007o`X:2POooooo1_m;Bd/2
ooooo`Oo8B4Q1oooool7oe5AD@Ko:BTY2_ooool7ogEeM@Oo2PX:1_oHf=P9ooooo`OoUIFE1omCDe<6
ong]k@?ooooo1olU9BD7oaXJ6POohn?S1_m_Kfl2ooooo`Oo7alO1olZ:RX7ode=C@KoIfMW0oooool7
o`P820Oo0P821_l61PH6oiVIV@?ooooo1oni^KT7odi>CPOo0`<31_lT92@3ooooo`Ko00001onRXZ87
omgMg@KoHF5Q0oooool7oalO7`Oo^;Rh1ol?3`l6o`0000;ooooo1ol:2PX7ob8R8POo9bLW1_l/;2`A
ooooo`003oooool7oa4A4@Oo3@d=1olN7Qh6ooclo0;ooooo1om6ATH7oa0@40Oo7QhN1_oVi^H3oooo
o`Oo20P81ooLg=`7oc`l?0Go_[jn0oooool7oaXJ6POo2`/;1ol820P6oahN7P?ooooo1onPX:06oaDE
5@Oo10@41_m8B4P3ooooo`Oo00001ol:2PX7ooooo`KoBd];0_ooool7ob4Q8@Oooooo1omADE46obTY
:@[ooooo1omeMGD7o`X:2PKof=SH2Oooool7oiFEU@OoDe=C1_o]kNd3ooooo`Oo9BDU1olJ6QX7on?S
h`KoKfm_0_ooool7oalO7`Oo:RXZ1om=CDd6ofMWI`?ooooo1ol820P7o`820PKo1PH61_nIVIT3oooo
o`Oo^KVi1om>CTh7o`<30`Ko92@T0oooool6o`0000OoXZ:R1ooMgMd6of5QH@?ooooo1olO7al7okRh
^0Oo3`l?1_l00002ooooo`Oo2PX:1olR8R87obLW9`Ko;2`/4Oooool000oooooo1olA4A47o`d=3@Oo
7QhN1_olo?`2ooooo`OoATI61ol@4107oahN7PKoi^KV0oooool7o`P820Oog=cL1oll?3`5okjn_P?o
oooo1olJ6QX7o`/;2`Oo20P81_lN7Qh3ooooo`OoX:2P1_lE5AD7o`@410KoB4Q80oooool7o`0000Oo
2PX:1oooool6od];B`;ooooo1olQ8B47ooooo`OoDE5A1_lY:BT:ooooo`OoMGEe1ol:2PX6omSHf0Wo
oooo1onEUID7oe=CD`KokNg]0oooool7obDU9@Oo6QXJ1ooShn<6ofm_K`;ooooo1olO7al7obXZ:POo
CDe=1_mWIfL3ooooo`Oo20P81ol20P86o`H61PKoVIVI0oooool7okVi^@OoCTi>1ol30`<6ob@T90?o
oooo1_l00007oj:RXPOogMgM1_mQHF43ooooo`Oo7alO1onh^;P7o`l?3`Ko00000_ooool7o`X:2POo
8R8R1olW9bL6ob`/;17ooooo000?ooooo`Oo4A4A1ol=3@d7oahN7PKoo?cl0_ooool7odI6APOo410@
1olN7Qh6onKViP?ooooo1ol820P7omcLg0Oo?3`l1Onn_[h3ooooo`Oo6QXJ1ol;2`/7o`P820Ko7QhN
0oooool7oj2PX0Ko5ADE1ol410@6odQ8B0?ooooo1ol00007o`X:2POooooo1_m;Bd/2ooooo`Oo8B4Q
1oooool7oe5AD@Ko:BTY2_ooool7ogEeM@Oo2PX:1_oHf=P9ooooo`OoUIFE1omCDe<6ong]k@?ooooo
1olU9BD7oaXJ6POohn?S1_m_Kfl2ooooo`Oo7alO1olZ:RX7ode=C@KoIfMW0oooool7o`P820Oo0P82
1_l61PH6oiVIV@?ooooo1oni^KT7odi>CPOo0`<31_lT92@3ooooo`Ko00001onRXZ87omgMg@KoHF5Q
0oooool7oalO7`Oo^;Rh1ol?3`l6o`0000;ooooo1ol:2PX7ob8R8POo9bLW1_l/;2`Aooooo`003ooo
ool7o`/;2`Oo1@D51olI6AT8ooooo`Oo>c/k1ol00007oaDE5@Kom_Kf0oooool7o`0000Oooooo1ol^
;Rh5ojVYZ@?ooooo1olJ6QX7o`h>3POo0@411_m=CDd:ooooo`Ko6ATI1ol00006ok:b/P?ooooo1ol:
2PX7o`T92@OoG5aL1_m6ATH2ooooo`Oo6a/K1olc<c<7oaXJ6PKoOgmo0oooool7onWYj@OoJFUY1ol4
10@8ooooo`OogMgM1onLW9`7o`0000Kobl_;0oooool7o`D51@Oo7AdM1oooool6ofEUI@;ooooo1ol@
4107ochn?POo=CDe1_mUIFD3ooooo`Oo5aLG1ol00006oaXJ6PWooooo1ooMgMd7obh^;POo00001_lE
5AD3ooooo`Ko1PH61ooFe]H7ooooo`KoJf][0oooool7ob8R8POogmoO1ol>3Ph6o`X:2P;ooooo1old
=3@7ocPh>0Oo;Bd]1_lo?clAooooo`003oooool7o`/;2`Oo1@D51olI6AT8ooooo`Oo>c/k1ol00007
oaDE5@Kom_Kf0oooool7o`0000Oooooo1ol^;Rh5ojVYZ@?ooooo1olJ6QX7o`h>3POo0@411_m=CDd:
ooooo`Ko6ATI1ol00006ok:b/P?ooooo1ol:2PX7o`T92@OoG5aL1_m6ATH2ooooo`Oo6a/K1olc<c<7
oaXJ6PKoOgmo0oooool7onWYj@OoJFUY1ol410@8ooooo`OogMgM1onLW9`7o`0000Kobl_;0oooool7
o`D51@Oo7AdM1oooool6ofEUI@;ooooo1ol@4107ochn?POo=CDe1_mUIFD3ooooo`Oo5aLG1ol00006
oaXJ6PWooooo1ooMgMd7obh^;POo00001_lE5AD3ooooo`Ko1PH61ooFe]H7ooooo`KoJf][0oooool7
ob8R8POogmoO1ol>3Ph6o`X:2P;ooooo1old=3@7ocPh>0Oo;Bd]1_lo?clAooooo`003oooool7o`/;
2`Oo1@D51olI6AT8ooooo`Oo>c/k1ol00007oaDE5@Kom_Kf0oooool7o`0000Oooooo1ol^;Rh5ojVY
Z@?ooooo1olJ6QX7o`h>3POo0@411_m=CDd:ooooo`Ko6ATI1ol00006ok:b/P?ooooo1ol:2PX7o`T9
2@OoG5aL1_m6ATH2ooooo`Oo6a/K1olc<c<7oaXJ6PKoOgmo0oooool7onWYj@OoJFUY1ol410@8oooo
o`OogMgM1onLW9`7o`0000Kobl_;0oooool7o`D51@Oo7AdM1oooool6ofEUI@;ooooo1ol@4107ochn
?POo=CDe1_mUIFD3ooooo`Oo5aLG1ol00006oaXJ6PWooooo1ooMgMd7obh^;POo00001_lE5AD3oooo
o`Ko1PH61ooFe]H7ooooo`KoJf][0oooool7ob8R8POogmoO1ol>3Ph6o`X:2P;ooooo1old=3@7ocPh
>0Oo;Bd]1_lo?clAooooo`003oooool7o`/;2`Oo1@D51olI6AT8ooooo`Oo>c/k1ol00007oaDE5@Ko
m_Kf0oooool7o`0000Oooooo1ol^;Rh5ojVYZ@?ooooo1olJ6QX7o`h>3POo0@411_m=CDd:ooooo`Ko
6ATI1ol00006ok:b/P?ooooo1ol:2PX7o`T92@OoG5aL1_m6ATH2ooooo`Oo6a/K1olc<c<7oaXJ6PKo
Ogmo0oooool7onWYj@OoJFUY1ol410@8ooooo`OogMgM1onLW9`7o`0000Kobl_;0oooool7o`D51@Oo
7AdM1oooool6ofEUI@;ooooo1ol@4107ochn?POo=CDe1_mUIFD3ooooo`Oo5aLG1ol00006oaXJ6PWo
oooo1ooMgMd7obh^;POo00001_lE5AD3ooooo`Ko1PH61ooFe]H7ooooo`KoJf][0oooool7ob8R8POo
gmoO1ol>3Ph6o`X:2P;ooooo1old=3@7ocPh>0Oo;Bd]1_lo?clAooooo`003oooool7o`/;2`Oo1@D5
1olI6AT8ooooo`Oo>c/k1ol00007oaDE5@Kom_Kf0oooool7o`0000Oooooo1ol^;Rh5ojVYZ@?ooooo
1olJ6QX7o`h>3POo0@411_m=CDd:ooooo`Ko6ATI1ol00006ok:b/P?ooooo1ol:2PX7o`T92@OoG5aL
1_m6ATH2ooooo`Oo6a/K1olc<c<7oaXJ6PKoOgmo0oooool7onWYj@OoJFUY1ol410@8ooooo`OogMgM
1onLW9`7o`0000Kobl_;0oooool7o`D51@Oo7AdM1oooool6ofEUI@;ooooo1ol@4107ochn?POo=CDe
1_mUIFD3ooooo`Oo5aLG1ol00006oaXJ6PWooooo1ooMgMd7obh^;POo00001_lE5AD3ooooo`Ko1PH6
1ooFe]H7ooooo`KoJf][0oooool7ob8R8POogmoO1ol>3Ph6o`X:2P;ooooo1old=3@7ocPh>0Oo;Bd]
1_lo?clAooooo`003oooool7o`/;2`Oo1@D51olI6AT8ooooo`Oo>c/k1ol00007oaDE5@Kom_Kf0ooo
ool7o`0000Oooooo1ol^;Rh5ojVYZ@?ooooo1olJ6QX7o`h>3POo0@411_m=CDd:ooooo`Ko6ATI1ol0
0006ok:b/P?ooooo1ol:2PX7o`T92@OoG5aL1_m6ATH2ooooo`Oo6a/K1olc<c<7oaXJ6PKoOgmo0ooo
ool7onWYj@OoJFUY1ol410@8ooooo`OogMgM1onLW9`7o`0000Kobl_;0oooool7o`D51@Oo7AdM1ooo
ool6ofEUI@;ooooo1ol@4107ochn?POo=CDe1_mUIFD3ooooo`Oo5aLG1ol00006oaXJ6PWooooo1ooM
gMd7obh^;POo00001_lE5AD3ooooo`Ko1PH61ooFe]H7ooooo`KoJf][0oooool7ob8R8POogmoO1ol>
3Ph6o`X:2P;ooooo1old=3@7ocPh>0Oo;Bd]1_lo?clAooooo`003oooool7o`d=3@Oo1PH61olJ6QX6
om_Kf`;ooooo1olk>c/7o`L71`Oo71`L2Oooool7obLW9`OocLg=1olE5AD5ohb<S0?ooooo1olY:BT7
o`l?3`Oo4a<C2Oooool7ogUiN@Ko3Ph>1ol30`<6oe]KF`?ooooo1ol10@47o`L71`Oo4a<C1_le=CD2
ooooo`Oo00001olG5aL7ob4Q8@Ko?Cdm0oooool7okjn_POoFe]K1ol00006omkNgP;ooooo1oo8b<P7
omSHf0Oo2@T91_oRh^83ooooo`Oo00001old=3@7ong]k@KoC4a<0_ooool7o`0000OoRh^;1ole=CD6
oe9BDP?ooooo1ol>3Ph7oa8B4PKo:b/[1_mIFET3ooooo`Ool_;b1olF5QH7o`@410Ko61PH0oooool6
o`L71`Ooh>3P1oogmoL6of1PH0?ooooo1olM7Ad7ooooo`Oo1`L71_l<30`9ooooo`OoJFUY1olC4a<6
oeEEEA7ooooo000?ooooo`Oo3@d=1ol61PH7oaXJ6PKofm_K0_ooool7oc/k>`Oo1`L71olL71`9oooo
o`Oo9bLW1oo=cLd7oaDE5@GoS8b<0oooool7obTY:@Oo3`l?1olC4a<9ooooo`OoNGUi1_l>3Ph7o`<3
0`KoFe]K0oooool7o`410@Oo1`L71olC4a<6ocDe=@;ooooo1ol00007oaLG5`Oo8B4Q1_lm?Cd3oooo
o`Oo_[jn1omKFe/7o`0000Kog]kN0_ooool7olS8b0Oof=SH1ol92@T6on;RhP?ooooo1ol00007oc@d
=0OokNg]1_m<C4`2ooooo`Oo00001on;Rh/7ocDe=@KoDU9B0oooool7o`h>3POo4Q8B1_l[:b/6oeUI
F@?ooooo1oobl_87oaHF5POo10@41_lH61P3ooooo`Ko1`L71ooPh>07ooOgm`KoH61P0oooool7oadM
7@Oooooo1ol71`L6o``<30Wooooo1omYJFT7oa<C4`KoEEEE4Oooool000oooooo1ol=3@d7o`H61POo
6QXJ1_oKfm/2ooooo`Oo>c/k1ol71`L7oa`L70Wooooo1olW9bL7olg=c@Oo5ADE1On<S8`3ooooo`Oo
:BTY1ol?3`l7oa<C4`Wooooo1omiNGT6o`h>3POo0`<31_mKFe/3ooooo`Oo0@411ol71`L7oa<C4`Ko
=CDe0_ooool7o`0000Oo5aLG1olQ8B46ocdm?@?ooooo1onn_[h7oe]KF`Oo00001_oNg]h2ooooo`Oo
b<S81ooHf=P7o`T92@Koh^;R0oooool7o`0000Oo=3@d1oo]kNd6oda<C0;ooooo1ol00007oh^;R`Oo
=CDe1_mBDU83ooooo`Oo3Ph>1olB4Q86ob/[:`KoFEUI0oooool7oo;blPOo5QHF1ol410@6oaPH60?o
oooo1_l71`L7on3Ph0OomoOg1_mPH603ooooo`Oo7AdM1oooool7o`L71`Ko30`<2Oooool7ofUYJ@Oo
4a<C1_mEEEDAooooo`003oooool7o`d=3@Oo1PH61olJ6QX6om_Kf`;ooooo1olk>c/7o`L71`Oo71`L
2Oooool7obLW9`OocLg=1olE5AD5ohb<S0?ooooo1olY:BT7o`l?3`Oo4a<C2Oooool7ogUiN@Ko3Ph>
1ol30`<6oe]KF`?ooooo1ol10@47o`L71`Oo4a<C1_le=CD2ooooo`Oo00001olG5aL7ob4Q8@Ko?Cdm
0oooool7okjn_POoFe]K1ol00006omkNgP;ooooo1oo8b<P7omSHf0Oo2@T91_oRh^83ooooo`Oo0000
1old=3@7ong]k@KoC4a<0_ooool7o`0000OoRh^;1ole=CD6oe9BDP?ooooo1ol>3Ph7oa8B4PKo:b/[
1_mIFET3ooooo`Ool_;b1olF5QH7o`@410Ko61PH0oooool6o`L71`Ooh>3P1oogmoL6of1PH0?ooooo
1olM7Ad7ooooo`Oo1`L71_l<30`9ooooo`OoJFUY1olC4a<6oeEEEA7ooooo000?ooooo`Oo3@d=1ol6
1PH7oaXJ6PKofm_K0_ooool7oc/k>`Oo1`L71olL71`9ooooo`Oo9bLW1oo=cLd7oaDE5@GoS8b<0ooo
ool7obTY:@Oo3`l?1olC4a<9ooooo`OoNGUi1_l>3Ph7o`<30`KoFe]K0oooool7o`410@Oo1`L71olC
4a<6ocDe=@;ooooo1ol00007oaLG5`Oo8B4Q1_lm?Cd3ooooo`Oo_[jn1omKFe/7o`0000Kog]kN0_oo
ool7olS8b0Oof=SH1ol92@T6on;RhP?ooooo1ol00007oc@d=0OokNg]1_m<C4`2ooooo`Oo00001on;
Rh/7ocDe=@KoDU9B0oooool7o`h>3POo4Q8B1_l[:b/6oeUIF@?ooooo1oobl_87oaHF5POo10@41_lH
61P3ooooo`Ko1`L71ooPh>07ooOgm`KoH61P0oooool7oadM7@Oooooo1ol71`L6o``<30Wooooo1omY
JFT7oa<C4`KoEEEE4Oooool000oooooo1ol=3@d7o`H61POo6QXJ1_oKfm/2ooooo`Oo>c/k1ol71`L7
oa`L70Wooooo1olW9bL7olg=c@Oo5ADE1On<S8`3ooooo`Oo:BTY1ol?3`l7oa<C4`Wooooo1omiNGT6
o`h>3POo0`<31_mKFe/3ooooo`Oo0@411ol71`L7oa<C4`Ko=CDe0_ooool7o`0000Oo5aLG1olQ8B46
ocdm?@?ooooo1onn_[h7oe]KF`Oo00001_oNg]h2ooooo`Oob<S81ooHf=P7o`T92@Koh^;R0oooool7
o`0000Oo=3@d1oo]kNd6oda<C0;ooooo1ol00007oh^;R`Oo=CDe1_mBDU83ooooo`Oo3Ph>1olB4Q86
ob/[:`KoFEUI0oooool7oo;blPOo5QHF1ol410@6oaPH60?ooooo1_l71`L7on3Ph0OomoOg1_mPH603
ooooo`Oo7AdM1oooool7o`L71`Ko30`<2Oooool7ofUYJ@Oo4a<C1_mEEEDAooooo`003oooool7o`d=
3@Oo1PH61olJ6QX6om_Kf`;ooooo1olk>c/7o`L71`Oo71`L2Oooool7obLW9`OocLg=1olE5AD5ohb<
S0?ooooo1olY:BT7o`l?3`Oo4a<C2Oooool7ogUiN@Ko3Ph>1ol30`<6oe]KF`?ooooo1ol10@47o`L7
1`Oo4a<C1_le=CD2ooooo`Oo00001olG5aL7ob4Q8@Ko?Cdm0oooool7okjn_POoFe]K1ol00006omkN
gP;ooooo1oo8b<P7omSHf0Oo2@T91_oRh^83ooooo`Oo00001old=3@7ong]k@KoC4a<0_ooool7o`00
00OoRh^;1ole=CD6oe9BDP?ooooo1ol>3Ph7oa8B4PKo:b/[1_mIFET3ooooo`Ool_;b1olF5QH7o`@4
10Ko61PH0oooool6o`L71`Ooh>3P1oogmoL6of1PH0?ooooo1olM7Ad7ooooo`Oo1`L71_l<30`9oooo
o`OoJFUY1olC4a<6oeEEEA7ooooo000?ooooo`Oo6QXJ1ol00007oa4A4@Ko[jn_0_ooool7ocPh>0Oo
10@41olS8b<6om3@d0?ooooo1olO7al7of5QH@Oo7alO1OmbLW83ooooo`Oo6ATI1olH61P7oa0@40Ko
00000oooool7ocHf=PKo2@T91olM7Ad6oclo?`?ooooo1ol:2PX7o`0000Oo51@D1_lI6AT2ooooo`Oo
1PH61olB4Q87ob<S8`KoCdm?0oooool7og5aL@Oo?Cdm1ol61PH6olS8b0;ooooo1onRXZ87olk>cPOo
5aLG1_ono_h3ooooo`Oo3`l?1olh>3P7om;BdPKo?Cdm0_ooool7oa@D50Oooooo1olG5aL6ode=C@?o
oooo1ol:2PX7ob0P80Ko51@D1_lk>c/:ooooo`Oo5aLG1ol<30`6oa`L70?ooooo1_lK6a/7ojf][@Oo
a/K61_m?Cdl3ooooo`Oo820P1oo7alL7o`410@Ko5ADE0_ooool7ofEUI@Oo=3@d1ol00006odM7Aa7o
oooo000?ooooo`Oo6QXJ1ol00007oa4A4@Ko[jn_0_ooool7ocPh>0Oo10@41olS8b<6om3@d0?ooooo
1olO7al7of5QH@Oo7alO1OmbLW83ooooo`Oo6ATI1olH61P7oa0@40Ko00000oooool7ocHf=PKo2@T9
1olM7Ad6oclo?`?ooooo1ol:2PX7o`0000Oo51@D1_lI6AT2ooooo`Oo1PH61olB4Q87ob<S8`KoCdm?
0oooool7og5aL@Oo?Cdm1ol61PH6olS8b0;ooooo1onRXZ87olk>cPOo5aLG1_ono_h3ooooo`Oo3`l?
1olh>3P7om;BdPKo?Cdm0_ooool7oa@D50Oooooo1olG5aL6ode=C@?ooooo1ol:2PX7ob0P80Ko51@D
1_lk>c/:ooooo`Oo5aLG1ol<30`6oa`L70?ooooo1_lK6a/7ojf][@Ooa/K61_m?Cdl3ooooo`Oo820P
1oo7alL7o`410@Ko5ADE0_ooool7ofEUI@Oo=3@d1ol00006odM7Aa7ooooo000?ooooo`Oo6QXJ1ol0
0007oa4A4@Ko[jn_0_ooool7ocPh>0Oo10@41olS8b<6om3@d0?ooooo1olO7al7of5QH@Oo7alO1Omb
LW83ooooo`Oo6ATI1olH61P7oa0@40Ko00000oooool7ocHf=PKo2@T91olM7Ad6oclo?`?ooooo1ol:
2PX7o`0000Oo51@D1_lI6AT2ooooo`Oo1PH61olB4Q87ob<S8`KoCdm?0oooool7og5aL@Oo?Cdm1ol6
1PH6olS8b0;ooooo1onRXZ87olk>cPOo5aLG1_ono_h3ooooo`Oo3`l?1olh>3P7om;BdPKo?Cdm0_oo
ool7oa@D50Oooooo1olG5aL6ode=C@?ooooo1ol:2PX7ob0P80Ko51@D1_lk>c/:ooooo`Oo5aLG1ol<
30`6oa`L70?ooooo1_lK6a/7ojf][@Ooa/K61_m?Cdl3ooooo`Oo820P1oo7alL7o`410@Ko5ADE0_oo
ool7ofEUI@Oo=3@d1ol00006odM7Aa7ooooo000?ooooo`Oo6QXJ1ol00007oa4A4@Ko[jn_0_ooool7
ocPh>0Oo10@41olS8b<6om3@d0?ooooo1olO7al7of5QH@Oo7alO1OmbLW83ooooo`Oo6ATI1olH61P7
oa0@40Ko00000oooool7ocHf=PKo2@T91olM7Ad6oclo?`?ooooo1ol:2PX7o`0000Oo51@D1_lI6AT2
ooooo`Oo1PH61olB4Q87ob<S8`KoCdm?0oooool7og5aL@Oo?Cdm1ol61PH6olS8b0;ooooo1onRXZ87
olk>cPOo5aLG1_ono_h3ooooo`Oo3`l?1olh>3P7om;BdPKo?Cdm0_ooool7oa@D50Oooooo1olG5aL6
ode=C@?ooooo1ol:2PX7ob0P80Ko51@D1_lk>c/:ooooo`Oo5aLG1ol<30`6oa`L70?ooooo1_lK6a/7
ojf][@Ooa/K61_m?Cdl3ooooo`Oo820P1oo7alL7o`410@Ko5ADE0_ooool7ofEUI@Oo=3@d1ol00006
odM7Aa7ooooo000?ooooo`Oo6QXJ1ol00007oa4A4@Ko[jn_0_ooool7ocPh>0Oo10@41olS8b<6om3@
d0?ooooo1olO7al7of5QH@Oo7alO1OmbLW83ooooo`Oo6ATI1olH61P7oa0@40Ko00000oooool7ocHf
=PKo2@T91olM7Ad6oclo?`?ooooo1ol:2PX7o`0000Oo51@D1_lI6AT2ooooo`Oo1PH61olB4Q87ob<S
8`KoCdm?0oooool7og5aL@Oo?Cdm1ol61PH6olS8b0;ooooo1onRXZ87olk>cPOo5aLG1_ono_h3oooo
o`Oo3`l?1olh>3P7om;BdPKo?Cdm0_ooool7oa@D50Oooooo1olG5aL6ode=C@?ooooo1ol:2PX7ob0P
80Ko51@D1_lk>c/:ooooo`Oo5aLG1ol<30`6oa`L70?ooooo1_lK6a/7ojf][@Ooa/K61_m?Cdl3oooo
o`Oo820P1oo7alL7o`410@Ko5ADE0_ooool7ofEUI@Oo=3@d1ol00006odM7Aa7ooooo000?ooooo`Oo
6QXJ1ol00007oa4A4@Ko[jn_0_ooool7ocPh>0Oo10@41olS8b<6om3@d0?ooooo1olO7al7of5QH@Oo
7alO1OmbLW83ooooo`Oo6ATI1olH61P7oa0@40Ko00000oooool7ocHf=PKo2@T91olM7Ad6oclo?`?o
oooo1ol:2PX7o`0000Oo51@D1_lI6AT2ooooo`Oo1PH61olB4Q87ob<S8`KoCdm?0oooool7og5aL@Oo
?Cdm1ol61PH6olS8b0;ooooo1onRXZ87olk>cPOo5aLG1_ono_h3ooooo`Oo3`l?1olh>3P7om;BdPKo
?Cdm0_ooool7oa@D50Oooooo1olG5aL6ode=C@?ooooo1ol:2PX7ob0P80Ko51@D1_lk>c/:ooooo`Oo
5aLG1ol<30`6oa`L70?ooooo1_lK6a/7ojf][@Ooa/K61_m?Cdl3ooooo`Oo820P1oo7alL7o`410@Ko
5ADE0_ooool7ofEUI@Oo=3@d1ol00006odM7Aa7ooooo003oooooooGooooo003oooooooGooooo0000
\
\>"],
  ImageRangeCache->{{{0, 499}, {59, 0}} -> {-0.525373, -0.04801, 0.0341698, \
0.0341698}, {{15.375, 42.125}, {57.5625, 30.8125}} -> {-2.29911, -0.214975, \
0.149534, 0.149534}, {{44.75, 71.5625}, {57.5625, 30.8125}} -> {-6.69635, \
-0.214975, 0.149534, 0.149534}, {{74.1875, 100.938}, {57.5625, 30.8125}} -> \
{-11.0936, -0.214975, 0.149534, 0.149534}, {{103.625, 130.375}, {57.5625, \
30.8125}} -> {-15.4955, -0.214975, 0.149534, 0.149534}, {{133.062, 159.812}, \
{57.5625, 30.8125}} -> {-19.8974, -0.214975, 0.149534, 0.149534}, {{162.5, \
189.25}, {57.5625, 30.8125}} -> {-24.2993, -0.214975, 0.149534, 0.149534}, \
{{191.938, 218.688}, {57.5625, 30.8125}} -> {-28.7012, -0.214975, 0.149534, \
0.149534}, {{221.375, 248.125}, {57.5625, 30.8125}} -> {-33.1032, -0.214975, \
0.149534, 0.149534}, {{250.812, 277.562}, {57.5625, 30.8125}} -> {-37.5051, \
-0.214975, 0.149534, 0.149534}, {{280.25, 307}, {57.5625, 30.8125}} -> \
{-41.907, -0.214975, 0.149534, 0.149534}, {{309.688, 336.438}, {57.5625, \
30.8125}} -> {-46.3089, -0.214975, 0.149534, 0.149534}, {{339.125, 365.875}, \
{57.5625, 30.8125}} -> {-50.7108, -0.214975, 0.149534, 0.149534}, {{368.562, \
395.312}, {57.5625, 30.8125}} -> {-55.1127, -0.214975, 0.149534, 0.149534}, \
{{398, 424.75}, {57.5625, 30.8125}} -> {-59.5146, -0.214975, 0.149534, \
0.149534}, {{427.375, 454.188}, {57.5625, 30.8125}} -> {-63.9119, -0.214975, \
0.149534, 0.149534}, {{456.812, 483.625}, {57.5625, 30.8125}} -> {-68.3138, \
-0.214975, 0.149534, 0.149534}, {{15.375, 42.125}, {28.125, 1.375}} -> \
{-2.29911, -4.61689, 0.149534, 0.149534}, {{44.75, 71.5625}, {28.125, 1.375}} -> \
{-6.69635, -4.61689, 0.149534, 0.149534}, {{74.1875, 100.938}, {28.125, \
1.375}} -> {-11.0936, -4.61689, 0.149534, 0.149534}, {{103.625, 130.375}, \
{28.125, 1.375}} -> {-15.4955, -4.61689, 0.149534, 0.149534}, {{133.062, \
159.812}, {28.125, 1.375}} -> {-19.8974, -4.61689, 0.149534, 0.149534}, \
{{162.5, 189.25}, {28.125, 1.375}} -> {-24.2993, -4.61689, 0.149534, \
0.149534}, {{191.938, 218.688}, {28.125, 1.375}} -> {-28.7012, -4.61689, \
0.149534, 0.149534}, {{221.375, 248.125}, {28.125, 1.375}} -> {-33.1032, \
-4.61689, 0.149534, 0.149534}, {{250.812, 277.562}, {28.125, 1.375}} -> \
{-37.5051, -4.61689, 0.149534, 0.149534}, {{280.25, 307}, {28.125, 1.375}} -> \
{-41.907, -4.61689, 0.149534, 0.149534}, {{309.688, 336.438}, {28.125, \
1.375}} -> {-46.3089, -4.61689, 0.149534, 0.149534}, {{339.125, 365.875}, \
{28.125, 1.375}} -> {-50.7108, -4.61689, 0.149534, 0.149534}, {{368.562, \
395.312}, {28.125, 1.375}} -> {-55.1127, -4.61689, 0.149534, 0.149534}, \
{{398, 424.75}, {28.125, 1.375}} -> {-59.5146, -4.61689, 0.149534, 0.149534}, \
{{427.375, 454.188}, {28.125, 1.375}} -> {-63.9119, -4.61689, 0.149534, \
0.149534}, {{456.812, 483.625}, {28.125, 1.375}} -> {-68.3138, -4.61689, \
0.149534, 0.149534}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .12 
%%ImageSize: 500 60 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0595238 0.00285714 0.0595238 [
[ 0 0 0 0 ]
[ 1 .12 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .12 L
0 .12 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.00285714 0.0782313 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
FF040102
50002526
0D1F2A27
2F0A3B2B
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0836735 0.00285714 0.138095 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
35464B3A
3E3A5E17
2B63FF3A
0E040800
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.143537 0.00285714 0.197959 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
2430092C
0A050600
1B113104
68FF1FB1
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.203401 0.00285714 0.257823 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
267975F4
0046A5FF
021672BC
4459356C
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.263265 0.00285714 0.317687 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
20151518
FF131A17
4512021A
162C0005
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.323129 0.00285714 0.377551 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
6032050A
5493264C
46FF68CF
29000F0F
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.382993 0.00285714 0.437415 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
22371C4E
3B360C02
286B000F
BD1DFFEF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.442857 0.00285714 0.497279 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
08281507
146BFF28
2614370B
141C0016
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.502721 0.00285714 0.557143 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1639DA76
0830BB33
00008A29
3808FF4A
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.562585 0.00285714 0.617007 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
10001527
12131216
0A18011D
13151DFF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.622449 0.00285714 0.676871 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0D002300
18215219
05362A1D
FF0B311E
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.682313 0.00285714 0.736735 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
02113518
03091D05
0200090A
0C0FFF17
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.742177 0.00285714 0.796599 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
07000E08
1434300A
2696FF6F
35482A1C
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.802041 0.00285714 0.856463 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
4E08091E
A9200909
FF330009
0D081540
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.861905 0.00285714 0.916327 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
42263100
3B301E0F
60170E04
3F8DFF29
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.921769 0.00285714 0.97619 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
281D2130
6B102500
634CA301
FF1EFA76
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.0627211 0.0782313 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
F4FF1C0E
32450C0C
101D3700
5A581C2F
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0836735 0.0627211 0.138095 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1A460E16
0A2F0513
00280009
24FF0E1B
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.143537 0.0627211 0.197959 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
134A1C0D
281E2E1E
0F132329
46FF4D00
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.203401 0.0627211 0.257823 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
33FF422C
0D570020
07141A36
3B202A3D
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.263265 0.0627211 0.317687 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
2D282213
B8FF240B
2C410A00
2F351A39
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.323129 0.0627211 0.377551 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
11221746
09353DC9
00215AFF
1A160823
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.382993 0.0627211 0.437415 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0C042329
00131F07
0E201919
032721FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.442857 0.0627211 0.497279 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
000F2228
0A3043FF
0924064E
080A0C0D
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.502721 0.0627211 0.557143 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
19A1FF4D
001E2111
20160706
2A4D1A0E
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.562585 0.0627211 0.617007 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
6B786046
99916D87
F3A3008D
E7FFAC00
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.622449 0.0627211 0.676871 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
6D21172F
152A003C
17112A2E
FF691F22
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.682313 0.0627211 0.736735 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
042664D0
12002B4F
35171923
10506CFF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.742177 0.0627211 0.796599 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0000021A
1E200D10
FFC1150A
3E212310
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.802041 0.0627211 0.856463 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
6222130A
21171117
16130017
FF0E0C0D
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.861905 0.0627211 0.916327 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
00103632
12191236
10000720
0143FF1C
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.921769 0.0627211 0.97619 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
8202EB3F
211D4600
BF19142D
58FF282B
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{500, 60},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007d0000?2000`400?l00000o`00003ooooooooe
ooooo`00oooooooeooooo`00oooooooeooooo`005_ooool7o`@410Oo0@411_l20P82ooooo`Oo=CDe
1om6ATH7od];B`Ko>SXj0oooool7ob@T90Oo<30`1ol92@T5ob`/;0?ooooo1olV9RH7ogUiN@OoMGEe
1_odm?@3ooooo`Oo820P3OlE5AD6oaPH60?ooooo1omPH607oc8b<POo1@D51_l:2PX2ooooo`Oo8R8R
1olg=cL7oa`L70KoCTi>0oooool7o`P820Oo:2PX1olE5AD6o`L71`;ooooo1olF5QH7ocTi>@Oof][J
1_mfMWH3ooooo`Oo410@1ol00007oaDE5@Ko9bLW0_ooool7o`d=3@Oo00001olS8b<6o`0000?ooooo
1ol20P87oa4A4@Ko=CDe1_lH61P3ooooo`Oo1`L71ol00007o`h>3PKo20P80oooool6odi>CPOo20P8
1ol92@T6oahN7P?ooooo1om2@T87obHV9POo<C4a1_l00002ooooo`Oo:2PX1olM7Ad7ob4Q8@Ko<30`
4Oooool001Kooooo1ol410@7o`410@Ko0P820_ooool7ocDe=@OoATI61om;Bd/6ocXj>P?ooooo1olT
92@7oc0`<0Oo2@T91Ol/;2`3ooooo`Oo9RHV1omiNGT7ogEeM@Kom?Cd0oooool7ob0P80go5ADE1_lH
61P3ooooo`OoH61P1olb<S87o`D51@Ko2PX:0_ooool7ob8R8POo=cLg1olL71`6odi>CP?ooooo1ol8
20P7obPX:0Oo5ADE1_l71`L2ooooo`Oo5QHF1oli>CT7om[JfPKoMWIf0oooool7oa0@40Oo00001olE
5AD6obLW9`;ooooo1ol=3@d7o`0000Oo8b<S1_l00003ooooo`Oo0P821olA4A46ocDe=@Ko61PH0ooo
ool7o`L71`Oo00001ol>3Ph6o`P820?ooooo1_m>CTh7o`P820Oo2@T91_lN7Qh3ooooo`Oo@T921olV
9RH7oc4a<@Ko00000_ooool7obPX:0Oo7AdM1olQ8B46oc0`<17ooooo000Fooooo`Oo10@41ol10@46
o`820P;ooooo1ole=CD7odI6APOoBd];1_lj>SX3ooooo`Oo92@T1ol`<307o`T92@Go;2`/0oooool7
obHV9POoNGUi1omeMGD6ooCdm0?ooooo1olP820=oaDE5@Ko61PH0oooool7of1PH0Oo<S8b1ol51@D6
o`X:2P;ooooo1olR8R87ocLg=`Oo71`L1_m>CTh3ooooo`Oo20P81olX:2P7oaDE5@Ko1`L70_ooool7
oaHF5POo>CTi1ooJf]X6ogIfMP?ooooo1ol@4107o`0000Oo5ADE1_lW9bL2ooooo`Oo3@d=1ol00007
ob<S8`Ko00000oooool7o`820POo4A4A1_le=CD6oaPH60?ooooo1ol71`L7o`0000Oo3Ph>1_l820P3
ooooo`KoCTi>1ol820P7o`T92@Ko7QhN0oooool7od92@POo9RHV1ola<C46o`0000;ooooo1olX:2P7
oadM7@Oo8B4Q1_l`<30Aooooo`005_ooool7o`@410Oo0@411_l20P82ooooo`Oo=CDe1om6ATH7od];
B`Ko>SXj0oooool7ob@T90Oo<30`1ol92@T5ob`/;0?ooooo1olV9RH7ogUiN@OoMGEe1_odm?@3oooo
o`Oo820P3OlE5AD6oaPH60?ooooo1omPH607oc8b<POo1@D51_l:2PX2ooooo`Oo8R8R1olg=cL7oa`L
70KoCTi>0oooool7o`P820Oo:2PX1olE5AD6o`L71`;ooooo1olF5QH7ocTi>@Oof][J1_mfMWH3oooo
o`Oo410@1ol00007oaDE5@Ko9bLW0_ooool7o`d=3@Oo00001olS8b<6o`0000?ooooo1ol20P87oa4A
4@Ko=CDe1_lH61P3ooooo`Oo1`L71ol00007o`h>3PKo20P80oooool6odi>CPOo20P81ol92@T6oahN
7P?ooooo1om2@T87obHV9POo<C4a1_l00002ooooo`Oo:2PX1olM7Ad7ob4Q8@Ko<30`4Oooool001Ko
oooo1ol410@7o`410@Ko0P820_ooool7ocDe=@OoATI61om;Bd/6ocXj>P?ooooo1olT92@7oc0`<0Oo
2@T91Ol/;2`3ooooo`Oo9RHV1omiNGT7ogEeM@Kom?Cd0oooool7ob0P80go5ADE1_lH61P3ooooo`Oo
H61P1olb<S87o`D51@Ko2PX:0_ooool7ob8R8POo=cLg1olL71`6odi>CP?ooooo1ol820P7obPX:0Oo
5ADE1_l71`L2ooooo`Oo5QHF1oli>CT7om[JfPKoMWIf0oooool7oa0@40Oo00001olE5AD6obLW9`;o
oooo1ol=3@d7o`0000Oo8b<S1_l00003ooooo`Oo0P821olA4A46ocDe=@Ko61PH0oooool7o`L71`Oo
00001ol>3Ph6o`P820?ooooo1_m>CTh7o`P820Oo2@T91_lN7Qh3ooooo`Oo@T921olV9RH7oc4a<@Ko
00000_ooool7obPX:0Oo7AdM1olQ8B46oc0`<17ooooo000Fooooo`Oo10@41ol10@46o`820P;ooooo
1ole=CD7odI6APOoBd];1_lj>SX3ooooo`Oo92@T1ol`<307o`T92@Go;2`/0oooool7obHV9POoNGUi
1omeMGD6ooCdm0?ooooo1olP820=oaDE5@Ko61PH0oooool7of1PH0Oo<S8b1ol51@D6o`X:2P;ooooo
1olR8R87ocLg=`Oo71`L1_m>CTh3ooooo`Oo20P81olX:2P7oaDE5@Ko1`L70_ooool7oaHF5POo>CTi
1ooJf]X6ogIfMP?ooooo1ol@4107o`0000Oo5ADE1_lW9bL2ooooo`Oo3@d=1ol00007ob<S8`Ko0000
0oooool7o`820POo4A4A1_le=CD6oaPH60?ooooo1ol71`L7o`0000Oo3Ph>1_l820P3ooooo`KoCTi>
1ol820P7o`T92@Ko7QhN0oooool7od92@POo9RHV1ola<C46o`0000;ooooo1olX:2P7oadM7@Oo8B4Q
1_l`<30Aooooo`003oooool7oe1@D0Oo00001olU9BD6obHV9P;ooooo1oln?Sh7ocXj>POoGUiN1_lG
5aL3ooooo`Oo2PX:1ol51@D7o`H61PGo00000oooool7o`0000OoATI61onUYJD@ooooo`Ko4a<C1olJ
6QX6oaLG5`?ooooo1omDE5@7oi>CT`Oo9RHV1_m<C4`2ooooo`Oo>c/k1olf=SH7o``<30Ko0P820ooo
ool7oa@D50OoJf][1oooool6obPX:0;ooooo1ol820P7oc0`<0Oo^k^k1_lc<c<3ooooo`Oo4Q8B1olC
4a<7oa8B4PKo5QHF0_ooool7oaPH60Oo8B4Q1omBDU86oaTI6@?ooooo1ol30`<7o`T92@Ko7AdM1_l5
1@D3ooooo`Oo51@D1old=3@7oc0`<0Ko2PX:0oooool6ojVYZ@Oo820P3Ol92@T3ooooo`Oo>c/k1ol`
<307oahN7PKo3`l?0_ooool7of][J`Oo410@1olU9BD6o`00017ooooo000?ooooo`OoD51@1ol00007
obDU9@Ko9RHV0_ooool7ochn?POo>SXj1omNGUh6oaLG5`?ooooo1ol:2PX7o`D51@Oo1PH61Ol00003
ooooo`Oo00001om6ATH7ojFUYA3ooooo1_lC4a<7oaXJ6PKo5aLG0oooool7oeADE0OoTi>C1olV9RH6
oda<C0;ooooo1olk>c/7ocHf=POo30`<1_l20P83ooooo`Oo51@D1om[Jf/7ooooo`Ko:2PX0_ooool7
o`P820Oo<30`1onk^k/6oc<c<`?ooooo1olB4Q87oa<C4`Oo4Q8B1_lF5QH2ooooo`Oo61PH1olQ8B47
oe9BDPKo6ATI0oooool7o`<30`Oo2@T91_lM7Ad6o`D51@?ooooo1olD51@7oc@d=0Oo<30`1_l:2PX3
ooooo`KoZJVY1olP820=o`T92@?ooooo1olk>c/7oc0`<0Oo7QhN1_l?3`l2ooooo`OoJf][1ol@4107
obDU9@Ko00004Oooool000oooooo1om@D507o`0000Oo9BDU1_lV9RH2ooooo`Oo?Shn1olj>SX7oeiN
GPKo5aLG0oooool7o`X:2POo1@D51ol61PH5o`0000?ooooo1ol00007odI6APOoYJFU4?ooool6oa<C
4`Oo6QXJ1_lG5aL3ooooo`OoE5AD1onCTi<7obHV9PKoC4a<0_ooool7oc/k>`Oo=SHf1ol<30`6o`82
0P?ooooo1olD51@7of][J`Oooooo1_lX:2P2ooooo`Oo20P81ol`<307ok^k^`Ko<c<c0oooool7oa8B
4POo4a<C1olB4Q86oaHF5P;ooooo1olH61P7ob4Q8@OoDU9B1_lI6AT3ooooo`Oo0`<31ol92@T6oadM
7@Ko1@D50oooool7oa@D50Oo=3@d1ol`<306o`X:2P?ooooo1_nYZJT7ob0P80go2@T90oooool7oc/k
>`Oo<30`1olN7Qh6o`l?3`;ooooo1om[Jf/7oa0@40Oo9BDU1_l0000Aooooo`003oooool7oe1@D0Oo
00001olU9BD6obHV9P;ooooo1oln?Sh7ocXj>POoGUiN1_lG5aL3ooooo`Oo2PX:1ol51@D7o`H61PGo
00000oooool7o`0000OoATI61onUYJD@ooooo`Ko4a<C1olJ6QX6oaLG5`?ooooo1omDE5@7oi>CT`Oo
9RHV1_m<C4`2ooooo`Oo>c/k1olf=SH7o``<30Ko0P820oooool7oa@D50OoJf][1oooool6obPX:0;o
oooo1ol820P7oc0`<0Oo^k^k1_lc<c<3ooooo`Oo4Q8B1olC4a<7oa8B4PKo5QHF0_ooool7oaPH60Oo
8B4Q1omBDU86oaTI6@?ooooo1ol30`<7o`T92@Ko7AdM1_l51@D3ooooo`Oo51@D1old=3@7oc0`<0Ko
2PX:0oooool6ojVYZ@Oo820P3Ol92@T3ooooo`Oo>c/k1ol`<307oahN7PKo3`l?0_ooool7of][J`Oo
410@1olU9BD6o`00017ooooo000?ooooo`OoD51@1ol00007obDU9@Ko9RHV0_ooool7ochn?POo>SXj
1omNGUh6oaLG5`?ooooo1ol:2PX7o`D51@Oo1PH61Ol00003ooooo`Oo00001om6ATH7ojFUYA3ooooo
1_lC4a<7oaXJ6PKo5aLG0oooool7oeADE0OoTi>C1olV9RH6oda<C0;ooooo1olk>c/7ocHf=POo30`<
1_l20P83ooooo`Oo51@D1om[Jf/7ooooo`Ko:2PX0_ooool7o`P820Oo<30`1onk^k/6oc<c<`?ooooo
1olB4Q87oa<C4`Oo4Q8B1_lF5QH2ooooo`Oo61PH1olQ8B47oe9BDPKo6ATI0oooool7o`<30`Oo2@T9
1_lM7Ad6o`D51@?ooooo1olD51@7oc@d=0Oo<30`1_l:2PX3ooooo`KoZJVY1olP820=o`T92@?ooooo
1olk>c/7oc0`<0Oo7QhN1_l?3`l2ooooo`OoJf][1ol@4107obDU9@Ko00004Oooool000oooooo1om@
D507o`0000Oo9BDU1_lV9RH2ooooo`Oo?Shn1olj>SX7oeiNGPKo5aLG0oooool7o`X:2POo1@D51ol6
1PH5o`0000?ooooo1ol00007odI6APOoYJFU4?ooool6oa<C4`Oo6QXJ1_lG5aL3ooooo`OoE5AD1onC
Ti<7obHV9PKoC4a<0_ooool7oc/k>`Oo=SHf1ol<30`6o`820P?ooooo1olD51@7of][J`Oooooo1_lX
:2P2ooooo`Oo20P81ol`<307ok^k^`Ko<c<c0oooool7oa8B4POo4a<C1olB4Q86oaHF5P;ooooo1olH
61P7ob4Q8@OoDU9B1_lI6AT3ooooo`Oo0`<31ol92@T6oadM7@Ko1@D50oooool7oa@D50Oo=3@d1ol`
<306o`X:2P?ooooo1_nYZJT7ob0P80go2@T90oooool7oc/k>`Oo<30`1olN7Qh6o`l?3`;ooooo1om[
Jf/7oa0@40Oo9BDU1_l0000Aooooo`003oooool7oe1@D0Oo00001olU9BD6obHV9P;ooooo1oln?Sh7
ocXj>POoGUiN1_lG5aL3ooooo`Oo2PX:1ol51@D7o`H61PGo00000oooool7o`0000OoATI61onUYJD@
ooooo`Ko4a<C1olJ6QX6oaLG5`?ooooo1omDE5@7oi>CT`Oo9RHV1_m<C4`2ooooo`Oo>c/k1olf=SH7
o``<30Ko0P820oooool7oa@D50OoJf][1oooool6obPX:0;ooooo1ol820P7oc0`<0Oo^k^k1_lc<c<3
ooooo`Oo4Q8B1olC4a<7oa8B4PKo5QHF0_ooool7oaPH60Oo8B4Q1omBDU86oaTI6@?ooooo1ol30`<7
o`T92@Ko7AdM1_l51@D3ooooo`Oo51@D1old=3@7oc0`<0Ko2PX:0oooool6ojVYZ@Oo820P3Ol92@T3
ooooo`Oo>c/k1ol`<307oahN7PKo3`l?0_ooool7of][J`Oo410@1olU9BD6o`00017ooooo000?oooo
o`Oo3@d=1olO7al7obXZ:PKo9bLW0_ooool7ob/[:`OoHf=S1oooool6ocXj>P?ooooo1olK6a/7oa4A
4@Oo<C4a1Ol410@3ooooo`Oo0P821olF5QH7og9bLPKo_;bl0oooool7odE5A@Ko4Q8B1ol20P86oaXJ
6P?ooooo1om6ATH7ooooo`OoJ6QX1_o?cll2ooooo`Oo:2PX1om[Jf/7o`0000Ko3`l?0oooool7obHV
9POo51@D1olg=cL6o`/;2`;ooooo3_l00007ohZ:RPKo:BTY0oooool7o`X:2POo61PH1ol10@46oadM
7@;ooooo1ol51@D7ocHf=POo:RXZ1_lM7Ad3ooooo`Oo0P821ol00006o`T92@Ko2PX:0oooool7obHV
9POoUYJF1oooool6ofm_K`Wooooo1olc<c<7o`0000Ko2@T90oooool7of1PH0Oo5aLG1ol>3Ph6o`@4
10;ooooo1omSHf<7oda<C0OoXj>S1_l10@4Aooooo`003oooool7o`d=3@Oo7alO1olZ:RX6obLW9`;o
oooo1ol[:b/7of=SH`Oooooo1_lj>SX3ooooo`Oo6a/K1olA4A47oc4a<@Go10@40oooool7o`820POo
5QHF1ombLW86okbl_0?ooooo1om5ADD6oa8B4POo0P821_lJ6QX3ooooo`OoATI61oooool7ofQXJ0Ko
clo?0_ooool7obPX:0OoJf][1ol00006o`l?3`?ooooo1olV9RH7oa@D50Oo=cLg1_l;2`/2ooooo`ko
00001on:RXX6obTY:@?ooooo1ol:2PX7oaPH60Oo0@411_lM7Ad2ooooo`Oo1@D51olf=SH7obXZ:PKo
7AdM0oooool7o`820POo00001_l92@T6o`X:2P?ooooo1olV9RH7oiJFUPOooooo1_m_Kfl9ooooo`Oo
<c<c1ol00006o`T92@?ooooo1omPH607oaLG5`Oo3Ph>1_l410@2ooooo`OoHf=S1om<C4`7oj>SX`Ko
0@414Oooool000oooooo1ol=3@d7oalO7`Oo:RXZ1_lW9bL2ooooo`Oo:b/[1omSHf<7ooooo`Ko>SXj
0oooool7oa/K6`Oo4A4A1ola<C45o`@410?ooooo1ol20P87oaHF5POoLW9b1_nl_;`3ooooo`OoADE5
1_lB4Q87o`820PKo6QXJ0oooool7odI6APOooooo1omXJ6P6olo?c`;ooooo1olX:2P7of][J`Oo0000
1_l?3`l3ooooo`Oo9RHV1olD51@7ocLg=`Ko2`/;0_ooool>o`0000OoRXZ:1_lY:BT3ooooo`Oo2PX:
1olH61P7o`410@Ko7AdM0_ooool7o`D51@Oo=SHf1olZ:RX6oadM7@?ooooo1ol20P87o`0000Ko2@T9
1_l:2PX3ooooo`Oo9RHV1onFUYH7ooooo`KoKfm_2Oooool7oc<c<`Oo00001_l92@T3ooooo`OoH61P
1olG5aL7o`h>3PKo10@40_ooool7of=SH`OoC4a<1onSXj<6o`410A7ooooo000?ooooo`Oo3@d=1olO
7al7obXZ:PKo9bLW0_ooool7ob/[:`OoHf=S1oooool6ocXj>P?ooooo1olK6a/7oa4A4@Oo<C4a1Ol4
10@3ooooo`Oo0P821olF5QH7og9bLPKo_;bl0oooool7odE5A@Ko4Q8B1ol20P86oaXJ6P?ooooo1om6
ATH7ooooo`OoJ6QX1_o?cll2ooooo`Oo:2PX1om[Jf/7o`0000Ko3`l?0oooool7obHV9POo51@D1olg
=cL6o`/;2`;ooooo3_l00007ohZ:RPKo:BTY0oooool7o`X:2POo61PH1ol10@46oadM7@;ooooo1ol5
1@D7ocHf=POo:RXZ1_lM7Ad3ooooo`Oo0P821ol00006o`T92@Ko2PX:0oooool7obHV9POoUYJF1ooo
ool6ofm_K`Wooooo1olc<c<7o`0000Ko2@T90oooool7of1PH0Oo5aLG1ol>3Ph6o`@410;ooooo1omS
Hf<7oda<C0OoXj>S1_l10@4Aooooo`003oooool7o`d=3@Oo7alO1olZ:RX6obLW9`;ooooo1ol[:b/7
of=SH`Oooooo1_lj>SX3ooooo`Oo6a/K1olA4A47oc4a<@Go10@40oooool7o`820POo5QHF1ombLW86
okbl_0?ooooo1om5ADD6oa8B4POo0P821_lJ6QX3ooooo`OoATI61oooool7ofQXJ0Koclo?0_ooool7
obPX:0OoJf][1ol00006o`l?3`?ooooo1olV9RH7oa@D50Oo=cLg1_l;2`/2ooooo`ko00001on:RXX6
obTY:@?ooooo1ol:2PX7oaPH60Oo0@411_lM7Ad2ooooo`Oo1@D51olf=SH7obXZ:PKo7AdM0oooool7
o`820POo00001_l92@T6o`X:2P?ooooo1olV9RH7oiJFUPOooooo1_m_Kfl9ooooo`Oo<c<c1ol00006
o`T92@?ooooo1omPH607oaLG5`Oo3Ph>1_l410@2ooooo`OoHf=S1om<C4`7oj>SX`Ko0@414Oooool0
00oooooo1ol=3@d7oalO7`Oo:RXZ1_lW9bL2ooooo`Oo:b/[1omSHf<7ooooo`Ko>SXj0oooool7oa/K
6`Oo4A4A1ola<C45o`@410?ooooo1ol20P87oaHF5POoLW9b1_nl_;`3ooooo`OoADE51_lB4Q87o`82
0PKo6QXJ0oooool7odI6APOooooo1omXJ6P6olo?c`;ooooo1olX:2P7of][J`Oo00001_l?3`l3oooo
o`Oo9RHV1olD51@7ocLg=`Ko2`/;0_ooool>o`0000OoRXZ:1_lY:BT3ooooo`Oo2PX:1olH61P7o`41
0@Ko7AdM0_ooool7o`D51@Oo=SHf1olZ:RX6oadM7@?ooooo1ol20P87o`0000Ko2@T91_l:2PX3oooo
o`Oo9RHV1onFUYH7ooooo`KoKfm_2Oooool7oc<c<`Oo00001_l92@T3ooooo`OoH61P1olG5aL7o`h>
3PKo10@40_ooool7of=SH`OoC4a<1onSXj<6o`410A7ooooo000?ooooo`Oo;bl_1ol:2PX7oc/k>`Ko
:b/[0_ooool7o`h>3POo10@41ol820P6o`0000?ooooo1omXJ6P7ooooo`Oo7alO1Ona/K43ooooo`Oo
A4A41omIFET7ocDe=@KoK6a/0oooool7oaHF5PKo;2`/1ol00006o`D51@?ooooo1olY:BT7o`0000go
3`l?0_ooool7okfm_@Oo7AdM1oooool6ono_k`?ooooo1olD51@7oa`L70Oo00001_lF5QH2ooooo`Oo
>3Ph1ol820P7ooooo`KoBTY:0oooool7oa<C4`Oo5ADE1olM7Ad?ooooo`Oo2`/;1ola<C46oahN7P?o
oooo1ol<30`7o`l?3`Kooooo1_lG5aL3ooooo`Oo=CDe1om8B4P7obXZ:PKo71`L0oooool6o`d=3@Oo
20P81olE5AD6od10@0?ooooo1olo?cl7ohf=S@Oooooo1_lY:BT9ooooo`Oo7QhN1oojn_X6ogIfMQ7o
oooo000?ooooo`Oo;bl_1ol:2PX7oc/k>`Ko:b/[0_ooool7o`h>3POo10@41ol820P6o`0000?ooooo
1omXJ6P7ooooo`Oo7alO1Ona/K43ooooo`OoA4A41omIFET7ocDe=@KoK6a/0oooool7oaHF5PKo;2`/
1ol00006o`D51@?ooooo1olY:BT7o`0000go3`l?0_ooool7okfm_@Oo7AdM1oooool6ono_k`?ooooo
1olD51@7oa`L70Oo00001_lF5QH2ooooo`Oo>3Ph1ol820P7ooooo`KoBTY:0oooool7oa<C4`Oo5ADE
1olM7Ad?ooooo`Oo2`/;1ola<C46oahN7P?ooooo1ol<30`7o`l?3`Kooooo1_lG5aL3ooooo`Oo=CDe
1om8B4P7obXZ:PKo71`L0oooool6o`d=3@Oo20P81olE5AD6od10@0?ooooo1olo?cl7ohf=S@Oooooo
1_lY:BT9ooooo`Oo7QhN1oojn_X6ogIfMQ7ooooo000?ooooo`Oo;bl_1ol:2PX7oc/k>`Ko:b/[0_oo
ool7o`h>3POo10@41ol820P6o`0000?ooooo1omXJ6P7ooooo`Oo7alO1Ona/K43ooooo`OoA4A41omI
FET7ocDe=@KoK6a/0oooool7oaHF5PKo;2`/1ol00006o`D51@?ooooo1olY:BT7o`0000go3`l?0_oo
ool7okfm_@Oo7AdM1oooool6ono_k`?ooooo1olD51@7oa`L70Oo00001_lF5QH2ooooo`Oo>3Ph1ol8
20P7ooooo`KoBTY:0oooool7oa<C4`Oo5ADE1olM7Ad?ooooo`Oo2`/;1ola<C46oahN7P?ooooo1ol<
30`7o`l?3`Kooooo1_lG5aL3ooooo`Oo=CDe1om8B4P7obXZ:PKo71`L0oooool6o`d=3@Oo20P81olE
5AD6od10@0?ooooo1olo?cl7ohf=S@Oooooo1_lY:BT9ooooo`Oo7QhN1oojn_X6ogIfMQ7ooooo000?
ooooo`Oo;bl_1ol:2PX7oc/k>`Ko:b/[0_ooool7o`h>3POo10@41ol820P6o`0000?ooooo1omXJ6P7
ooooo`Oo7alO1Ona/K43ooooo`OoA4A41omIFET7ocDe=@KoK6a/0oooool7oaHF5PKo;2`/1ol00006
o`D51@?ooooo1olY:BT7o`0000go3`l?0_ooool7okfm_@Oo7AdM1oooool6ono_k`?ooooo1olD51@7
oa`L70Oo00001_lF5QH2ooooo`Oo>3Ph1ol820P7ooooo`KoBTY:0oooool7oa<C4`Oo5ADE1olM7Ad?
ooooo`Oo2`/;1ola<C46oahN7P?ooooo1ol<30`7o`l?3`Kooooo1_lG5aL3ooooo`Oo=CDe1om8B4P7
obXZ:PKo71`L0oooool6o`d=3@Oo20P81olE5AD6od10@0?ooooo1olo?cl7ohf=S@Oooooo1_lY:BT9
ooooo`Oo7QhN1oojn_X6ogIfMQ7ooooo000?ooooo`Oo;bl_1ol:2PX7oc/k>`Ko:b/[0_ooool7o`h>
3POo10@41ol820P6o`0000?ooooo1omXJ6P7ooooo`Oo7alO1Ona/K43ooooo`OoA4A41omIFET7ocDe
=@KoK6a/0oooool7oaHF5PKo;2`/1ol00006o`D51@?ooooo1olY:BT7o`0000go3`l?0_ooool7okfm
_@Oo7AdM1oooool6ono_k`?ooooo1olD51@7oa`L70Oo00001_lF5QH2ooooo`Oo>3Ph1ol820P7oooo
o`KoBTY:0oooool7oa<C4`Oo5ADE1olM7Ad?ooooo`Oo2`/;1ola<C46oahN7P?ooooo1ol<30`7o`l?
3`Kooooo1_lG5aL3ooooo`Oo=CDe1om8B4P7obXZ:PKo71`L0oooool6o`d=3@Oo20P81olE5AD6od10
@0?ooooo1olo?cl7ohf=S@Oooooo1_lY:BT9ooooo`Oo7QhN1oojn_X6ogIfMQ7ooooo000?ooooo`Oo
;bl_1ol:2PX7oc/k>`Ko:b/[0_ooool7o`h>3POo10@41ol820P6o`0000?ooooo1omXJ6P7ooooo`Oo
7alO1Ona/K43ooooo`OoA4A41omIFET7ocDe=@KoK6a/0oooool7oaHF5PKo;2`/1ol00006o`D51@?o
oooo1olY:BT7o`0000go3`l?0_ooool7okfm_@Oo7AdM1oooool6ono_k`?ooooo1olD51@7oa`L70Oo
00001_lF5QH2ooooo`Oo>3Ph1ol820P7ooooo`KoBTY:0oooool7oa<C4`Oo5ADE1olM7Ad?ooooo`Oo
2`/;1ola<C46oahN7P?ooooo1ol<30`7o`l?3`Kooooo1_lG5aL3ooooo`Oo=CDe1om8B4P7obXZ:PKo
71`L0oooool6o`d=3@Oo20P81olE5AD6od10@0?ooooo1olo?cl7ohf=S@Oooooo1_lY:BT9ooooo`Oo
7QhN1oojn_X6ogIfMQ7ooooo000?ooooo`Oo;bl_1ol:2PX7oc/k>`Ko:b/[0_ooool7o`h>3POo10@4
1ol820P6o`0000?ooooo1omXJ6P7ooooo`Oo7alO1Ona/K43ooooo`OoA4A41omIFET7ocDe=@KoK6a/
0oooool7oaHF5PKo;2`/1ol00006o`D51@?ooooo1olY:BT7o`0000go3`l?0_ooool7okfm_@Oo7AdM
1oooool6ono_k`?ooooo1olD51@7oa`L70Oo00001_lF5QH2ooooo`Oo>3Ph1ol820P7ooooo`KoBTY:
0oooool7oa<C4`Oo5ADE1olM7Ad?ooooo`Oo2`/;1ola<C46oahN7P?ooooo1ol<30`7o`l?3`Kooooo
1_lG5aL3ooooo`Oo=CDe1om8B4P7obXZ:PKo71`L0oooool6o`d=3@Oo20P81olE5AD6od10@0?ooooo
1olo?cl7ohf=S@Oooooo1_lY:BT9ooooo`Oo7QhN1oojn_X6ogIfMQ7ooooo003oooooooGooooo003o
ooooooGooooo003oooooooGooooo000?ooooo`Oom?Cd1oooool7oa`L70Ko3Ph>0_ooool7oaXJ6POo
ATI61ol>3Ph6oaHF5P?ooooo1olC4a<7odY:BPOo71`L1Ol=3@d3ooooo`Oo<c<c1oooool7od92@PKo
;2`/0oooool7obd];@Ko:2PX1olR8R86oa<C4`?ooooo1olA4A47ob8R8POo5aLG1_m6ATH2ooooo`Oo
30`<1ol410@7ob<S8`Ko:BTY0oooool7o`0000Oo3`l?1olR8R86obPX:0;ooooo1olI6AT7oj6QX@Oo
oooo1_m=CDd3ooooo`OoJf][1omhN7P7of1PH0KoATI60_ooool7ofe]K@Oo8B4Q1olG5aL6obl_;`?o
oooo1ol410@7obHV9PKoI6AT1_o@d=03ooooo`ko00001ol20P86oaXJ6P?ooooo1_mRHV87ob8R8POo
4a<C1_l:2PX3ooooo`Oo00001ol@4107ocHf=PKo<S8b0_ooool7oh:2PPOo0P821oo[jn/6oclo?a7o
oooo000?ooooo`Oom?Cd1oooool7oa`L70Ko3Ph>0_ooool7oaXJ6POoATI61ol>3Ph6oaHF5P?ooooo
1olC4a<7odY:BPOo71`L1Ol=3@d3ooooo`Oo<c<c1oooool7od92@PKo;2`/0oooool7obd];@Ko:2PX
1olR8R86oa<C4`?ooooo1olA4A47ob8R8POo5aLG1_m6ATH2ooooo`Oo30`<1ol410@7ob<S8`Ko:BTY
0oooool7o`0000Oo3`l?1olR8R86obPX:0;ooooo1olI6AT7oj6QX@Oooooo1_m=CDd3ooooo`OoJf][
1omhN7P7of1PH0KoATI60_ooool7ofe]K@Oo8B4Q1olG5aL6obl_;`?ooooo1ol410@7obHV9PKoI6AT
1_o@d=03ooooo`ko00001ol20P86oaXJ6P?ooooo1_mRHV87ob8R8POo4a<C1_l:2PX3ooooo`Oo0000
1ol@4107ocHf=PKo<S8b0_ooool7oh:2PPOo0P821oo[jn/6oclo?a7ooooo000?ooooo`Oom?Cd1ooo
ool7oa`L70Ko3Ph>0_ooool7oaXJ6POoATI61ol>3Ph6oaHF5P?ooooo1olC4a<7odY:BPOo71`L1Ol=
3@d3ooooo`Oo<c<c1oooool7od92@PKo;2`/0oooool7obd];@Ko:2PX1olR8R86oa<C4`?ooooo1olA
4A47ob8R8POo5aLG1_m6ATH2ooooo`Oo30`<1ol410@7ob<S8`Ko:BTY0oooool7o`0000Oo3`l?1olR
8R86obPX:0;ooooo1olI6AT7oj6QX@Oooooo1_m=CDd3ooooo`OoJf][1omhN7P7of1PH0KoATI60_oo
ool7ofe]K@Oo8B4Q1olG5aL6obl_;`?ooooo1ol410@7obHV9PKoI6AT1_o@d=03ooooo`ko00001ol2
0P86oaXJ6P?ooooo1_mRHV87ob8R8POo4a<C1_l:2PX3ooooo`Oo00001ol@4107ocHf=PKo<S8b0_oo
ool7oh:2PPOo0P821oo[jn/6oclo?a7ooooo000?ooooo`Oom?Cd1oooool7oa`L70Ko3Ph>0_ooool7
oaXJ6POoATI61ol>3Ph6oaHF5P?ooooo1olC4a<7odY:BPOo71`L1Ol=3@d3ooooo`Oo<c<c1oooool7
od92@PKo;2`/0oooool7obd];@Ko:2PX1olR8R86oa<C4`?ooooo1olA4A47ob8R8POo5aLG1_m6ATH2
ooooo`Oo30`<1ol410@7ob<S8`Ko:BTY0oooool7o`0000Oo3`l?1olR8R86obPX:0;ooooo1olI6AT7
oj6QX@Oooooo1_m=CDd3ooooo`OoJf][1omhN7P7of1PH0KoATI60_ooool7ofe]K@Oo8B4Q1olG5aL6
obl_;`?ooooo1ol410@7obHV9PKoI6AT1_o@d=03ooooo`ko00001ol20P86oaXJ6P?ooooo1_mRHV87
ob8R8POo4a<C1_l:2PX3ooooo`Oo00001ol@4107ocHf=PKo<S8b0_ooool7oh:2PPOo0P821oo[jn/6
oclo?a7ooooo000?ooooo`Oom?Cd1oooool7oa`L70Ko3Ph>0_ooool7oaXJ6POoATI61ol>3Ph6oaHF
5P?ooooo1olC4a<7odY:BPOo71`L1Ol=3@d3ooooo`Oo<c<c1oooool7od92@PKo;2`/0oooool7obd]
;@Ko:2PX1olR8R86oa<C4`?ooooo1olA4A47ob8R8POo5aLG1_m6ATH2ooooo`Oo30`<1ol410@7ob<S
8`Ko:BTY0oooool7o`0000Oo3`l?1olR8R86obPX:0;ooooo1olI6AT7oj6QX@Oooooo1_m=CDd3oooo
o`OoJf][1omhN7P7of1PH0KoATI60_ooool7ofe]K@Oo8B4Q1olG5aL6obl_;`?ooooo1ol410@7obHV
9PKoI6AT1_o@d=03ooooo`ko00001ol20P86oaXJ6P?ooooo1_mRHV87ob8R8POo4a<C1_l:2PX3oooo
o`Oo00001ol@4107ocHf=PKo<S8b0_ooool7oh:2PPOo0P821oo[jn/6oclo?a7ooooo000?ooooo`Oo
m?Cd1oooool7oa`L70Ko3Ph>0_ooool7oaXJ6POoATI61ol>3Ph6oaHF5P?ooooo1olC4a<7odY:BPOo
71`L1Ol=3@d3ooooo`Oo<c<c1oooool7od92@PKo;2`/0oooool7obd];@Ko:2PX1olR8R86oa<C4`?o
oooo1olA4A47ob8R8POo5aLG1_m6ATH2ooooo`Oo30`<1ol410@7ob<S8`Ko:BTY0oooool7o`0000Oo
3`l?1olR8R86obPX:0;ooooo1olI6AT7oj6QX@Oooooo1_m=CDd3ooooo`OoJf][1omhN7P7of1PH0Ko
ATI60_ooool7ofe]K@Oo8B4Q1olG5aL6obl_;`?ooooo1ol410@7obHV9PKoI6AT1_o@d=03ooooo`ko
00001ol20P86oaXJ6P?ooooo1_mRHV87ob8R8POo4a<C1_l:2PX3ooooo`Oo00001ol@4107ocHf=PKo
<S8b0_ooool7oh:2PPOo0P821oo[jn/6oclo?a7ooooo000?ooooo`Oom?Cd1oooool7oa`L70Ko3Ph>
0_ooool7oaXJ6POoATI61ol>3Ph6oaHF5P?ooooo1olC4a<7odY:BPOo71`L1Ol=3@d3ooooo`Oo<c<c
1oooool7od92@PKo;2`/0oooool7obd];@Ko:2PX1olR8R86oa<C4`?ooooo1olA4A47ob8R8POo5aLG
1_m6ATH2ooooo`Oo30`<1ol410@7ob<S8`Ko:BTY0oooool7o`0000Oo3`l?1olR8R86obPX:0;ooooo
1olI6AT7oj6QX@Oooooo1_m=CDd3ooooo`OoJf][1omhN7P7of1PH0KoATI60_ooool7ofe]K@Oo8B4Q
1olG5aL6obl_;`?ooooo1ol410@7obHV9PKoI6AT1_o@d=03ooooo`ko00001ol20P86oaXJ6P?ooooo
1_mRHV87ob8R8POo4a<C1_l:2PX3ooooo`Oo00001ol@4107ocHf=PKo<S8b0_ooool7oh:2PPOo0P82
1oo[jn/6oclo?a7ooooo000?ooooo`Oo<S8b1om5ADD=o``<30;ooooo1ol:2PX7obl_;`Oo1@D51_lC
4a<3ooooo`Oo:2PX1olN7Qh7obh^;PGo7QhN0oooool7o`d=3@OoEeMG1ol00006ob0P80?ooooo1onh
^;P6ooooo`Oo92@T1_l;2`/3ooooo`Oo2@T91ole=CD7ocdm?@KobLW90_ooool7o`0000Oo4a<C1olO
7al6o`L71`?ooooo1ol:2PX7oc0`<0Oo@d=32?ooool7o`0000Oo7QhN1olQ8B46oa4A4@?ooooo1onI
VIT7oi6AT@OoKFe]1_n7QhL2ooooo`Oo5ADE1olZ:RX7o`0000Ko?3`l0oooool7oa8B4POo00001_l[
:b/6odm?C`?ooooo1olN7Qh7ob0P80Oo3@d=1_l@4103ooooo`Ko8B4Q1olG5aL7oa4A4@Ko5aLG0ooo
ool7oa8B4POo6ATI1olB4Q86ocHf=P;ooooo1olQ8B47oadM7@OoATI61_l0000Aooooo`003oooool7
oc8b<POoADE53Ol<30`2ooooo`Oo2PX:1ol_;bl7o`D51@Ko4a<C0oooool7obPX:0Oo7QhN1ol^;Rh5
oahN7P?ooooo1ol=3@d7oeMGE`Oo00001_lP8203ooooo`Oo^;Rh1_ooool7ob@T90Ko2`/;0oooool7
o`T92@Oo=CDe1olm?Cd6olW9b@;ooooo1ol00007oa<C4`Oo7alO1_l71`L3ooooo`Oo2PX:1ol`<307
od=3@`Sooooo1ol00007oahN7POo8B4Q1_lA4A43ooooo`OoVIVI1onATI47ofe]K@KoQhN70_ooool7
oaDE5@Oo:RXZ1ol00006oc`l?0?ooooo1olB4Q87o`0000Ko:b/[1_m?Cdl3ooooo`Oo7QhN1olP8207
o`d=3@Ko410@0oooool6ob4Q8@Oo5aLG1olA4A46oaLG5`?ooooo1olB4Q87oaTI6@Oo4Q8B1_lf=SH2
ooooo`Oo8B4Q1olM7Ad7odI6APKo00004Oooool000oooooo1olb<S87odE5A@go30`<0_ooool7o`X:
2POo;bl_1ol51@D6oa<C4`?ooooo1olX:2P7oahN7POo;Rh^1OlN7Qh3ooooo`Oo3@d=1omGEeL7o`00
00Ko820P0oooool7okRh^0Kooooo1olT92@6o`/;2`?ooooo1ol92@T7ocDe=@Oo?Cdm1_o9bLT2oooo
o`Oo00001olC4a<7oalO7`Ko1`L70oooool7o`X:2POo<30`1om3@d<8ooooo`Oo00001olN7Qh7ob4Q
8@Ko4A4A0oooool7oiVIV@OoTI6A1om]KFd6ohN7Q`;ooooo1olE5AD7obXZ:POo00001_ll?3`3oooo
o`Oo4Q8B1ol00006ob/[:`KoCdm?0oooool7oahN7POo820P1ol=3@d6oa0@40?ooooo1_lQ8B47oaLG
5`Oo4A4A1_lG5aL3ooooo`Oo4Q8B1olI6AT7oa8B4PKo=SHf0_ooool7ob4Q8@Oo7AdM1om6ATH6o`00
017ooooo000?ooooo`Oo<S8b1om5ADD=o``<30;ooooo1ol:2PX7obl_;`Oo1@D51_lC4a<3ooooo`Oo
:2PX1olN7Qh7obh^;PGo7QhN0oooool7o`d=3@OoEeMG1ol00006ob0P80?ooooo1onh^;P6ooooo`Oo
92@T1_l;2`/3ooooo`Oo2@T91ole=CD7ocdm?@KobLW90_ooool7o`0000Oo4a<C1olO7al6o`L71`?o
oooo1ol:2PX7oc0`<0Oo@d=32?ooool7o`0000Oo7QhN1olQ8B46oa4A4@?ooooo1onIVIT7oi6AT@Oo
KFe]1_n7QhL2ooooo`Oo5ADE1olZ:RX7o`0000Ko?3`l0oooool7oa8B4POo00001_l[:b/6odm?C`?o
oooo1olN7Qh7ob0P80Oo3@d=1_l@4103ooooo`Ko8B4Q1olG5aL7oa4A4@Ko5aLG0oooool7oa8B4POo
6ATI1olB4Q86ocHf=P;ooooo1olQ8B47oadM7@OoATI61_l0000Aooooo`003oooool7oc8b<POoADE5
3Ol<30`2ooooo`Oo2PX:1ol_;bl7o`D51@Ko4a<C0oooool7obPX:0Oo7QhN1ol^;Rh5oahN7P?ooooo
1ol=3@d7oeMGE`Oo00001_lP8203ooooo`Oo^;Rh1_ooool7ob@T90Ko2`/;0oooool7o`T92@Oo=CDe
1olm?Cd6olW9b@;ooooo1ol00007oa<C4`Oo7alO1_l71`L3ooooo`Oo2PX:1ol`<307od=3@`Sooooo
1ol00007oahN7POo8B4Q1_lA4A43ooooo`OoVIVI1onATI47ofe]K@KoQhN70_ooool7oaDE5@Oo:RXZ
1ol00006oc`l?0?ooooo1olB4Q87o`0000Ko:b/[1_m?Cdl3ooooo`Oo7QhN1olP8207o`d=3@Ko410@
0oooool6ob4Q8@Oo5aLG1olA4A46oaLG5`?ooooo1olB4Q87oaTI6@Oo4Q8B1_lf=SH2ooooo`Oo8B4Q
1olM7Ad7odI6APKo00004Oooool000oooooo1olb<S87odE5A@go30`<0_ooool7o`X:2POo;bl_1ol5
1@D6oa<C4`?ooooo1olX:2P7oahN7POo;Rh^1OlN7Qh3ooooo`Oo3@d=1omGEeL7o`0000Ko820P0ooo
ool7okRh^0Kooooo1olT92@6o`/;2`?ooooo1ol92@T7ocDe=@Oo?Cdm1_o9bLT2ooooo`Oo00001olC
4a<7oalO7`Ko1`L70oooool7o`X:2POo<30`1om3@d<8ooooo`Oo00001olN7Qh7ob4Q8@Ko4A4A0ooo
ool7oiVIV@OoTI6A1om]KFd6ohN7Q`;ooooo1olE5AD7obXZ:POo00001_ll?3`3ooooo`Oo4Q8B1ol0
0006ob/[:`KoCdm?0oooool7oahN7POo820P1ol=3@d6oa0@40?ooooo1_lQ8B47oaLG5`Oo4A4A1_lG
5aL3ooooo`Oo4Q8B1olI6AT7oa8B4PKo=SHf0_ooool7ob4Q8@Oo7AdM1om6ATH6o`00017ooooo000?
ooooo`Oo410@1olM7Ad7ocLg=`Ko00000_ooool7o`0000Oo:2PX1ol00006o`T92@?ooooo1ol?3`l7
oa<C4`Oo8b<S1OlY:BT3ooooo`Oo1`L71olD51@7oaXJ6PKo=SHf0oooool7ob`/;0Ko@D511ol:2PX6
o`0000?ooooo1ol00007ob4Q8@OoFUYJ2?ooool7o`h>3POo820P3OlI6AT3ooooo`Oo2@T91olT92@7
o`H61PKoCTi>0_ooool7ob0P80Oo5QHF1ol71`L6o`H61P?ooooo1ooclo<7oj>SX`Oo00001_n=SHd2
ooooo`Oo5aLG1olA4A47obXZ:PKo;Rh^0oooool7ocDe=@Oo5aLG1_lI6AT6ob<S8`[ooooo1oo1`L47
oaDE5@Ko2PX:0oooool6oaHF5POo4a<C1ol00006oaLG5`?ooooo1ol@4107o`0000Oo1`L71_lP8202
ooooo`Oo_kno1olI6AT7oa@D50Ko;Bd]4Oooool000oooooo1ol@4107oadM7@Oo=cLg1_l00002oooo
o`Oo00001olX:2P7o`0000Ko2@T90oooool7o`l?3`Oo4a<C1olS8b<5obTY:@?ooooo1ol71`L7oa@D
50Oo6QXJ1_lf=SH3ooooo`Oo;2`/1_m1@D47o`X:2PKo00000oooool7o`0000Oo8B4Q1omJFUX8oooo
o`Oo3Ph>1olP820=oaTI6@?ooooo1ol92@T7ob@T90Oo1PH61_m>CTh2ooooo`Oo820P1olF5QH7o`L7
1`Ko1PH60oooool7oo?cl`OoXj>S1ol00006ohf=S@;ooooo1olG5aL7oa4A4@Oo:RXZ1_l^;Rh3oooo
o`Oo=CDe1olG5aL6oaTI6@Ko8b<S2_ooool7ol71`@Oo5ADE1_l:2PX3ooooo`Ko5QHF1olC4a<7o`00
00Ko5aLG0oooool7oa0@40Oo00001ol71`L6ob0P80;ooooo1ono_kl7oaTI6@Oo51@D1_l];BdAoooo
o`003oooool7oa0@40Oo7AdM1olg=cL6o`0000;ooooo1ol00007obPX:0Oo00001_l92@T3ooooo`Oo
3`l?1olC4a<7ob<S8`Go:BTY0oooool7o`L71`Oo51@D1olJ6QX6ocHf=P?ooooo1ol/;2`6od51@@Oo
2PX:1_l00003ooooo`Oo00001olQ8B47oeYJFPSooooo1ol>3Ph7ob0P80go6ATI0oooool7o`T92@Oo
92@T1ol61PH6odi>CP;ooooo1olP8207oaHF5POo1`L71_l61PH3ooooo`Oolo?c1onSXj<7o`0000Ko
SHf=0_ooool7oaLG5`Oo4A4A1olZ:RX6obh^;P?ooooo1ole=CD7oaLG5`Ko6ATI1_lS8b<:ooooo`Oo
`L711olE5AD6o`X:2P?ooooo1_lF5QH7oa<C4`Oo00001_lG5aL3ooooo`Oo410@1ol00007o`L71`Ko
820P0_ooool7okno_`Oo6ATI1olD51@6obd];A7ooooo000?ooooo`Oo410@1olM7Ad7ocLg=`Ko0000
0_ooool7o`0000Oo:2PX1ol00006o`T92@?ooooo1ol?3`l7oa<C4`Oo8b<S1OlY:BT3ooooo`Oo1`L7
1olD51@7oaXJ6PKo=SHf0oooool7ob`/;0Ko@D511ol:2PX6o`0000?ooooo1ol00007ob4Q8@OoFUYJ
2?ooool7o`h>3POo820P3OlI6AT3ooooo`Oo2@T91olT92@7o`H61PKoCTi>0_ooool7ob0P80Oo5QHF
1ol71`L6o`H61P?ooooo1ooclo<7oj>SX`Oo00001_n=SHd2ooooo`Oo5aLG1olA4A47obXZ:PKo;Rh^
0oooool7ocDe=@Oo5aLG1_lI6AT6ob<S8`[ooooo1oo1`L47oaDE5@Ko2PX:0oooool6oaHF5POo4a<C
1ol00006oaLG5`?ooooo1ol@4107o`0000Oo1`L71_lP8202ooooo`Oo_kno1olI6AT7oa@D50Ko;Bd]
4Oooool000oooooo1ol@4107oadM7@Oo=cLg1_l00002ooooo`Oo00001olX:2P7o`0000Ko2@T90ooo
ool7o`l?3`Oo4a<C1olS8b<5obTY:@?ooooo1ol71`L7oa@D50Oo6QXJ1_lf=SH3ooooo`Oo;2`/1_m1
@D47o`X:2PKo00000oooool7o`0000Oo8B4Q1omJFUX8ooooo`Oo3Ph>1olP820=oaTI6@?ooooo1ol9
2@T7ob@T90Oo1PH61_m>CTh2ooooo`Oo820P1olF5QH7o`L71`Ko1PH60oooool7oo?cl`OoXj>S1ol0
0006ohf=S@;ooooo1olG5aL7oa4A4@Oo:RXZ1_l^;Rh3ooooo`Oo=CDe1olG5aL6oaTI6@Ko8b<S2_oo
ool7ol71`@Oo5ADE1_l:2PX3ooooo`Ko5QHF1olC4a<7o`0000Ko5aLG0oooool7oa0@40Oo00001ol7
1`L6ob0P80;ooooo1ono_kl7oaTI6@Oo51@D1_l];BdAooooo`003oooool7oa0@40Oo7AdM1olg=cL6
o`0000;ooooo1ol00007obPX:0Oo00001_l92@T3ooooo`Oo3`l?1olC4a<7ob<S8`Go:BTY0oooool7
o`L71`Oo51@D1olJ6QX6ocHf=P?ooooo1ol/;2`6od51@@Oo2PX:1_l00003ooooo`Oo00001olQ8B47
oeYJFPSooooo1ol>3Ph7ob0P80go6ATI0oooool7o`T92@Oo92@T1ol61PH6odi>CP;ooooo1olP8207
oaHF5POo1`L71_l61PH3ooooo`Oolo?c1onSXj<7o`0000KoSHf=0_ooool7oaLG5`Oo4A4A1olZ:RX6
obh^;P?ooooo1ole=CD7oaLG5`Ko6ATI1_lS8b<:ooooo`Oo`L711olE5AD6o`X:2P?ooooo1_lF5QH7
oa<C4`Oo00001_lG5aL3ooooo`Oo410@1ol00007o`L71`Ko820P0_ooool7okno_`Oo6ATI1olD51@6
obd];A7ooooo000?ooooo`Oo410@1olM7Ad7ocLg=`Ko00000_ooool7o`0000Oo:2PX1ol00006o`T9
2@?ooooo1ol?3`l7oa<C4`Oo8b<S1OlY:BT3ooooo`Oo1`L71olD51@7oaXJ6PKo=SHf0oooool7ob`/
;0Ko@D511ol:2PX6o`0000?ooooo1ol00007ob4Q8@OoFUYJ2?ooool7o`h>3POo820P3OlI6AT3oooo
o`Oo2@T91olT92@7o`H61PKoCTi>0_ooool7ob0P80Oo5QHF1ol71`L6o`H61P?ooooo1ooclo<7oj>S
X`Oo00001_n=SHd2ooooo`Oo5aLG1olA4A47obXZ:PKo;Rh^0oooool7ocDe=@Oo5aLG1_lI6AT6ob<S
8`[ooooo1oo1`L47oaDE5@Ko2PX:0oooool6oaHF5POo4a<C1ol00006oaLG5`?ooooo1ol@4107o`00
00Oo1`L71_lP8202ooooo`Oo_kno1olI6AT7oa@D50Ko;Bd]4Oooool000oooooo1omJFUX7oeQHF0Oo
71`L1_l_;bl2ooooo`Oo92@T1oooool7o`h>3PKo6a/K0oooool7odI6APOooooo1om=CDd5o`0000?o
oooo1olk>c/7ob0P80Oo:RXZ1_lm?Cd3ooooo`Oo;bl_1_le=CD7oaXJ6PKo>CTi0oooool7oaXJ6POo
5QHF1ol820P6ob<S8`;ooooo1ol30`<7obLW9`Oo8B4Q2Oooool7o`P820Oo2PX:1ol<30`6o`d=3@;o
oooo1olZ:RX7ode=C@Oo6QXJ1_l>3Ph3ooooo`OoinOW1oooool7ojb/[0Ko00002Oooool7ofUYJ@Oo
7alO1_lR8R83ooooo`Oo410@1om@D506ofa/K0Wooooo1oln?Sh7ob4Q8@Oo8b<S1_l@4109ooooo`Oo
3Ph>1ol<30`6o`d=3@?ooooo1ol10@47od=3@`Oooooo1_lL71`2ooooo`OoF5QH1oooool7obPX:0Ko
:b/[4Oooool000oooooo1omJFUX7oeQHF0Oo71`L1_l_;bl2ooooo`Oo92@T1oooool7o`h>3PKo6a/K
0oooool7odI6APOooooo1om=CDd5o`0000?ooooo1olk>c/7ob0P80Oo:RXZ1_lm?Cd3ooooo`Oo;bl_
1_le=CD7oaXJ6PKo>CTi0oooool7oaXJ6POo5QHF1ol820P6ob<S8`;ooooo1ol30`<7obLW9`Oo8B4Q
2Oooool7o`P820Oo2PX:1ol<30`6o`d=3@;ooooo1olZ:RX7ode=C@Oo6QXJ1_l>3Ph3ooooo`OoinOW
1oooool7ojb/[0Ko00002Oooool7ofUYJ@Oo7alO1_lR8R83ooooo`Oo410@1om@D506ofa/K0Wooooo
1oln?Sh7ob4Q8@Oo8b<S1_l@4109ooooo`Oo3Ph>1ol<30`6o`d=3@?ooooo1ol10@47od=3@`Oooooo
1_lL71`2ooooo`OoF5QH1oooool7obPX:0Ko:b/[4Oooool000oooooo1omJFUX7oeQHF0Oo71`L1_l_
;bl2ooooo`Oo92@T1oooool7o`h>3PKo6a/K0oooool7odI6APOooooo1om=CDd5o`0000?ooooo1olk
>c/7ob0P80Oo:RXZ1_lm?Cd3ooooo`Oo;bl_1_le=CD7oaXJ6PKo>CTi0oooool7oaXJ6POo5QHF1ol8
20P6ob<S8`;ooooo1ol30`<7obLW9`Oo8B4Q2Oooool7o`P820Oo2PX:1ol<30`6o`d=3@;ooooo1olZ
:RX7ode=C@Oo6QXJ1_l>3Ph3ooooo`OoinOW1oooool7ojb/[0Ko00002Oooool7ofUYJ@Oo7alO1_lR
8R83ooooo`Oo410@1om@D506ofa/K0Wooooo1oln?Sh7ob4Q8@Oo8b<S1_l@4109ooooo`Oo3Ph>1ol<
30`6o`d=3@?ooooo1ol10@47od=3@`Oooooo1_lL71`2ooooo`OoF5QH1oooool7obPX:0Ko:b/[4Ooo
ool000oooooo1omJFUX7oeQHF0Oo71`L1_l_;bl2ooooo`Oo92@T1oooool7o`h>3PKo6a/K0oooool7
odI6APOooooo1om=CDd5o`0000?ooooo1olk>c/7ob0P80Oo:RXZ1_lm?Cd3ooooo`Oo;bl_1_le=CD7
oaXJ6PKo>CTi0oooool7oaXJ6POo5QHF1ol820P6ob<S8`;ooooo1ol30`<7obLW9`Oo8B4Q2Oooool7
o`P820Oo2PX:1ol<30`6o`d=3@;ooooo1olZ:RX7ode=C@Oo6QXJ1_l>3Ph3ooooo`OoinOW1oooool7
ojb/[0Ko00002Oooool7ofUYJ@Oo7alO1_lR8R83ooooo`Oo410@1om@D506ofa/K0Wooooo1oln?Sh7
ob4Q8@Oo8b<S1_l@4109ooooo`Oo3Ph>1ol<30`6o`d=3@?ooooo1ol10@47od=3@`Oooooo1_lL71`2
ooooo`OoF5QH1oooool7obPX:0Ko:b/[4Oooool000oooooo1omJFUX7oeQHF0Oo71`L1_l_;bl2oooo
o`Oo92@T1oooool7o`h>3PKo6a/K0oooool7odI6APOooooo1om=CDd5o`0000?ooooo1olk>c/7ob0P
80Oo:RXZ1_lm?Cd3ooooo`Oo;bl_1_le=CD7oaXJ6PKo>CTi0oooool7oaXJ6POo5QHF1ol820P6ob<S
8`;ooooo1ol30`<7obLW9`Oo8B4Q2Oooool7o`P820Oo2PX:1ol<30`6o`d=3@;ooooo1olZ:RX7ode=
C@Oo6QXJ1_l>3Ph3ooooo`OoinOW1oooool7ojb/[0Ko00002Oooool7ofUYJ@Oo7alO1_lR8R83oooo
o`Oo410@1om@D506ofa/K0Wooooo1oln?Sh7ob4Q8@Oo8b<S1_l@4109ooooo`Oo3Ph>1ol<30`6o`d=
3@?ooooo1ol10@47od=3@`Oooooo1_lL71`2ooooo`OoF5QH1oooool7obPX:0Ko:b/[4Oooool000oo
oooo1omJFUX7oeQHF0Oo71`L1_l_;bl2ooooo`Oo92@T1oooool7o`h>3PKo6a/K0oooool7odI6APOo
oooo1om=CDd5o`0000?ooooo1olk>c/7ob0P80Oo:RXZ1_lm?Cd3ooooo`Oo;bl_1_le=CD7oaXJ6PKo
>CTi0oooool7oaXJ6POo5QHF1ol820P6ob<S8`;ooooo1ol30`<7obLW9`Oo8B4Q2Oooool7o`P820Oo
2PX:1ol<30`6o`d=3@;ooooo1olZ:RX7ode=C@Oo6QXJ1_l>3Ph3ooooo`OoinOW1oooool7ojb/[0Ko
00002Oooool7ofUYJ@Oo7alO1_lR8R83ooooo`Oo410@1om@D506ofa/K0Wooooo1oln?Sh7ob4Q8@Oo
8b<S1_l@4109ooooo`Oo3Ph>1ol<30`6o`d=3@?ooooo1ol10@47od=3@`Oooooo1_lL71`2ooooo`Oo
F5QH1oooool7obPX:0Ko:b/[4Oooool00?oooooomOooool00?oooooomOooool00001\
\>"],
  ImageRangeCache->{{{0, 499}, {59, 0}} -> {-0.525373, -0.04801, 0.0341698, \
0.0341698}, {{15.375, 42.125}, {57.5625, 30.8125}} -> {-2.29911, -0.214975, \
0.149534, 0.149534}, {{44.75, 71.5625}, {57.5625, 30.8125}} -> {-6.69635, \
-0.214975, 0.149534, 0.149534}, {{74.1875, 100.938}, {57.5625, 30.8125}} -> \
{-11.0936, -0.214975, 0.149534, 0.149534}, {{103.625, 130.375}, {57.5625, \
30.8125}} -> {-15.4955, -0.214975, 0.149534, 0.149534}, {{133.062, 159.812}, \
{57.5625, 30.8125}} -> {-19.8974, -0.214975, 0.149534, 0.149534}, {{162.5, \
189.25}, {57.5625, 30.8125}} -> {-24.2993, -0.214975, 0.149534, 0.149534}, \
{{191.938, 218.688}, {57.5625, 30.8125}} -> {-28.7012, -0.214975, 0.149534, \
0.149534}, {{221.375, 248.125}, {57.5625, 30.8125}} -> {-33.1032, -0.214975, \
0.149534, 0.149534}, {{250.812, 277.562}, {57.5625, 30.8125}} -> {-37.5051, \
-0.214975, 0.149534, 0.149534}, {{280.25, 307}, {57.5625, 30.8125}} -> \
{-41.907, -0.214975, 0.149534, 0.149534}, {{309.688, 336.438}, {57.5625, \
30.8125}} -> {-46.3089, -0.214975, 0.149534, 0.149534}, {{339.125, 365.875}, \
{57.5625, 30.8125}} -> {-50.7108, -0.214975, 0.149534, 0.149534}, {{368.562, \
395.312}, {57.5625, 30.8125}} -> {-55.1127, -0.214975, 0.149534, 0.149534}, \
{{398, 424.75}, {57.5625, 30.8125}} -> {-59.5146, -0.214975, 0.149534, \
0.149534}, {{427.375, 454.188}, {57.5625, 30.8125}} -> {-63.9119, -0.214975, \
0.149534, 0.149534}, {{456.812, 483.625}, {57.5625, 30.8125}} -> {-68.3138, \
-0.214975, 0.149534, 0.149534}, {{15.375, 42.125}, {28.125, 1.375}} -> \
{-2.29911, -4.61689, 0.149534, 0.149534}, {{44.75, 71.5625}, {28.125, 1.375}} -> \
{-6.69635, -4.61689, 0.149534, 0.149534}, {{74.1875, 100.938}, {28.125, \
1.375}} -> {-11.0936, -4.61689, 0.149534, 0.149534}, {{103.625, 130.375}, \
{28.125, 1.375}} -> {-15.4955, -4.61689, 0.149534, 0.149534}, {{133.062, \
159.812}, {28.125, 1.375}} -> {-19.8974, -4.61689, 0.149534, 0.149534}, \
{{162.5, 189.25}, {28.125, 1.375}} -> {-24.2993, -4.61689, 0.149534, \
0.149534}, {{191.938, 218.688}, {28.125, 1.375}} -> {-28.7012, -4.61689, \
0.149534, 0.149534}, {{221.375, 248.125}, {28.125, 1.375}} -> {-33.1032, \
-4.61689, 0.149534, 0.149534}, {{250.812, 277.562}, {28.125, 1.375}} -> \
{-37.5051, -4.61689, 0.149534, 0.149534}, {{280.25, 307}, {28.125, 1.375}} -> \
{-41.907, -4.61689, 0.149534, 0.149534}, {{309.688, 336.438}, {28.125, \
1.375}} -> {-46.3089, -4.61689, 0.149534, 0.149534}, {{339.125, 365.875}, \
{28.125, 1.375}} -> {-50.7108, -4.61689, 0.149534, 0.149534}, {{368.562, \
395.312}, {28.125, 1.375}} -> {-55.1127, -4.61689, 0.149534, 0.149534}, \
{{398, 424.75}, {28.125, 1.375}} -> {-59.5146, -4.61689, 0.149534, 0.149534}, \
{{427.375, 454.188}, {28.125, 1.375}} -> {-63.9119, -4.61689, 0.149534, \
0.149534}, {{456.812, 483.625}, {28.125, 1.375}} -> {-68.3138, -4.61689, \
0.149534, 0.149534}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .12 
%%ImageSize: 500 60 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0595238 0.00285714 0.0595238 [
[ 0 0 0 0 ]
[ 1 .12 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .12 L
0 .12 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.00285714 0.0782313 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
20479E2A
3460CF46
3E3A812E
1800FF00
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0836735 0.00285714 0.138095 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
5D3A1412
2C400400
253A2426
FF3D1631
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.143537 0.00285714 0.197959 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
23092B30
1B2442A5
1400091E
FF3F083C
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.203401 0.00285714 0.257823 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
FF28223D
711E0300
603C000A
5E092B7E
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.263265 0.00285714 0.317687 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
291910FF
141B0A61
23001332
1D1C0C3C
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.323129 0.00285714 0.377551 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
C37A0016
C8C1630A
83FFCE14
983F0A39
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.382993 0.00285714 0.437415 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
2B0F958A
4F00FF39
70105F20
0D28576C
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.442857 0.00285714 0.497279 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0001082F
17171929
0A0B141F
4B1120FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.502721 0.00285714 0.557143 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
2C180D48
1F13003A
24341552
130D1CFF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.562585 0.00285714 0.617007 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
34DBA81C
0060A230
36156776
A1FF870B
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.622449 0.00285714 0.676871 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
4B5B150C
063E224E
184F43FF
82341800
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.682313 0.00285714 0.736735 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
57003449
A2580A2A
FFF1301F
3D0F804B
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.742177 0.00285714 0.796599 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
AE410C00
FF881C2B
42500728
750A4304
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.802041 0.00285714 0.856463 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0B20190D
00194627
0C24FF1F
1B18391C
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.861905 0.00285714 0.916327 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
220C1926
1C0E0600
09150C17
1D05FF24
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.921769 0.00285714 0.97619 0.0572789 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1F221910
013E160C
00FF3B3A
12391B34
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.0627211 0.0782313 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
153E0A04
102E0014
0521140F
10FF261E
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0836735 0.0627211 0.138095 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
D1FF5502
3AB97E16
00889231
B99E0314
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.143537 0.0627211 0.197959 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
13000C10
23000DFF
040A0330
05091927
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.203401 0.0627211 0.257823 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
4B002562
FF290B4D
73321517
07262550
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.263265 0.0627211 0.317687 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
3D1214FF
03141034
1102001C
1307161F
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.323129 0.0627211 0.377551 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
904E0035
B9FF1315
7CA2130A
5E011F44
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.382993 0.0627211 0.437415 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
00FFF19B
2547C2C0
5B0ABF5D
15A5605A
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.442857 0.0627211 0.497279 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
24856648
2724001B
18090635
053B42FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.502721 0.0627211 0.557143 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
11611F84
5464005F
B6BD98EB
62FFCA1A
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.562585 0.0627211 0.617007 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
00210E24
0E080C1A
100C081E
0CFF3A38
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.622449 0.0627211 0.676871 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
0E1B224A
3E0E4F5B
630056FF
16332539
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.682313 0.0627211 0.736735 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
00252C2E
3E001158
FF300258
08592323
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.742177 0.0627211 0.796599 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
400F090E
1F00070B
39140701
FF231F0D
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.802041 0.0627211 0.856463 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
1D801C34
04FF9A81
0089B699
34710A15
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.861905 0.0627211 0.916327 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
161CFF36
5200373A
8E210E25
0560DE44
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.921769 0.0627211 0.97619 0.117143 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0 0.25 0 0.25 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 1 scale
4 string
4 4 8 [4 0 0 4 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
00051618
0A111517
1117021F
262123FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{500, 60},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007d0000?2000`400?l00000o`00003ooooooooe
ooooo`00oooooooeooooo`00oooooooeooooo`003oooool7ob0P80OoAdM71onNWYh6obXZ:P;ooooo
1omMGEd7ocXj>POo51@D1_lB4Q83ooooo`Oo8b<S1ol92@T7ob/[:`Go<30`2_ooool7obPX:0Oo8R8R
1_lm?Cd3ooooo`Oo:BTY1_lI6AT7oa0@40Wooooo1oo3`l<7ogYjNPOo00001_lF5QH2ooooo`Oo:b/[
1ol?3`l7oiFEU@KoRXZ:0oooool7o`0000Oo0@411ol820P6obl_;`;ooooo1ol/;2`7oaPH60Oo3@d=
1_m8B4P3ooooo`Oo=3@d1ooKfm/7ojRXZ0Ko71`L0_ooool7od];B`OoFe]K1olE5AD6o``<30?ooooo
1omGEeL7o`0000Ko=3@d1_m9BDT3ooooo`Oo[Zj^1om1@D47o``<30Ko00000oooool6o`/;2`Oo820P
1olI6AT6o`d=3@?ooooo1olR8R87o``<30Oo6ATI1_lV9RH2ooooo`Oo7alO1olR8R87oaTI6@Ko410@
4Oooool000oooooo1olP8207odM7A`OoWYjN1_lZ:RX2ooooo`OoGEeM1olj>SX7oa@D50Ko4Q8B0ooo
ool7ob<S8`Oo2@T91ol[:b/5oc0`<0[ooooo1olX:2P7ob8R8PKo?Cdm0oooool7obTY:@Ko6ATI1ol@
4109ooooo`Oo`l?31omjNWX7o`0000Ko5QHF0_ooool7ob/[:`Oo3`l?1onEUID6ohZ:RP?ooooo1ol0
0007o`410@Oo20P81_l_;bl2ooooo`Oo;2`/1olH61P7o`d=3@KoB4Q80oooool7oc@d=0Oofm_K1onX
Z:P6oa`L70;ooooo1om;Bd/7oe]KF`Oo5ADE1_l<30`3ooooo`OoEeMG1ol00006oc@d=0KoBDU90ooo
ool7ojj^[POo@D511ol<30`6o`0000?ooooo1_l;2`/7ob0P80Oo6ATI1_l=3@d3ooooo`Oo8R8R1ol<
30`7oaTI6@Ko9RHV0_ooool7oalO7`Oo8R8R1olI6AT6oa0@417ooooo000?ooooo`Oo820P1om7AdL7
oijNWPKo:RXZ0_ooool7oeeMG@Oo>SXj1olD51@6oa8B4P?ooooo1olS8b<7o`T92@Oo:b/[1Ol`<30:
ooooo`Oo:2PX1olR8R86ocdm?@?ooooo1olY:BT6oaTI6@Oo410@2Oooool7ol?3``OoNWYj1ol00006
oaHF5P;ooooo1ol[:b/7o`l?3`OoUIFE1_n:RXX3ooooo`Oo00001ol10@47o`P820Ko;bl_0_ooool7
ob`/;0Oo61PH1ol=3@d6odQ8B0?ooooo1old=3@7om_Kf`OoZ:RX1_lL71`2ooooo`OoBd];1omKFe/7
oaDE5@Ko30`<0oooool7oeMGE`Oo00001_ld=3@6odU9B@?ooooo1on^[Zh7od51@@Oo30`<1_l00003
ooooo`Ko2`/;1olP8207oaTI6@Ko3@d=0oooool7ob8R8POo30`<1olI6AT6obHV9P;ooooo1olO7al7
ob8R8POo6ATI1_l@410Aooooo`003oooool7ob0P80OoAdM71onNWYh6obXZ:P;ooooo1omMGEd7ocXj
>POo51@D1_lB4Q83ooooo`Oo8b<S1ol92@T7ob/[:`Go<30`2_ooool7obPX:0Oo8R8R1_lm?Cd3oooo
o`Oo:BTY1_lI6AT7oa0@40Wooooo1oo3`l<7ogYjNPOo00001_lF5QH2ooooo`Oo:b/[1ol?3`l7oiFE
U@KoRXZ:0oooool7o`0000Oo0@411ol820P6obl_;`;ooooo1ol/;2`7oaPH60Oo3@d=1_m8B4P3oooo
o`Oo=3@d1ooKfm/7ojRXZ0Ko71`L0_ooool7od];B`OoFe]K1olE5AD6o``<30?ooooo1omGEeL7o`00
00Ko=3@d1_m9BDT3ooooo`Oo[Zj^1om1@D47o``<30Ko00000oooool6o`/;2`Oo820P1olI6AT6o`d=
3@?ooooo1olR8R87o``<30Oo6ATI1_lV9RH2ooooo`Oo7alO1olR8R87oaTI6@Ko410@4Oooool000oo
oooo1olP8207odM7A`OoWYjN1_lZ:RX2ooooo`OoGEeM1olj>SX7oa@D50Ko4Q8B0oooool7ob<S8`Oo
2@T91ol[:b/5oc0`<0[ooooo1olX:2P7ob8R8PKo?Cdm0oooool7obTY:@Ko6ATI1ol@4109ooooo`Oo
`l?31omjNWX7o`0000Ko5QHF0_ooool7ob/[:`Oo3`l?1onEUID6ohZ:RP?ooooo1ol00007o`410@Oo
20P81_l_;bl2ooooo`Oo;2`/1olH61P7o`d=3@KoB4Q80oooool7oc@d=0Oofm_K1onXZ:P6oa`L70;o
oooo1om;Bd/7oe]KF`Oo5ADE1_l<30`3ooooo`OoEeMG1ol00006oc@d=0KoBDU90oooool7ojj^[POo
@D511ol<30`6o`0000?ooooo1_l;2`/7ob0P80Oo6ATI1_l=3@d3ooooo`Oo8R8R1ol<30`7oaTI6@Ko
9RHV0_ooool7oalO7`Oo8R8R1olI6AT6oa0@417ooooo000?ooooo`Oo820P1om7AdL7oijNWPKo:RXZ
0_ooool7oeeMG@Oo>SXj1olD51@6oa8B4P?ooooo1olS8b<7o`T92@Oo:b/[1Ol`<30:ooooo`Oo:2PX
1olR8R86ocdm?@?ooooo1olY:BT6oaTI6@Oo410@2Oooool7ol?3``OoNWYj1ol00006oaHF5P;ooooo
1ol[:b/7o`l?3`OoUIFE1_n:RXX3ooooo`Oo00001ol10@47o`P820Ko;bl_0_ooool7ob`/;0Oo61PH
1ol=3@d6odQ8B0?ooooo1old=3@7om_Kf`OoZ:RX1_lL71`2ooooo`OoBd];1omKFe/7oaDE5@Ko30`<
0oooool7oeMGE`Oo00001_ld=3@6odU9B@?ooooo1on^[Zh7od51@@Oo30`<1_l00003ooooo`Ko2`/;
1olP8207oaTI6@Ko3@d=0oooool7ob8R8POo30`<1olI6AT6obHV9P;ooooo1olO7al7ob8R8POo6ATI
1_l@410Aooooo`003oooool7oc@d=0OoH61P1oo?cll6odI6AP;ooooo1ol/;2`7od10@0Oo10@41_l0
0003ooooo`Oo6a/K1olT92@7od92@PGoYJFU0oooool7og5aL@Oo7QhN1ol30`<6o`0000?ooooo1olD
51@6oa/K6`Oo2PX:1_mQHF43ooooo`Oob<S81oo1`L47of=SH`Ko2PX:0_ooool7odm?C`Oo00001ooo
ool6ocTi>@?ooooo3_lG5aL7oaTI6@Ko:BTY0_ooool7oalO7`Oo4a<C1ol00006ocXj>P?ooooo1ol0
0007of1PH0OoXZ:R1_l`<302ooooo`Oo1PH61oln?Sh7ob8R8PKoCTi>0oooool7oj:RXPOoF5QH1_l:
2PX6obXZ:P[ooooo1on8R8P7oa`L70Ko:b/[0oooool6o`0000Oo6ATI1om6ATH6obLW9`?ooooo1olL
71`7o`h>3POo1PH61_l00002ooooo`Oo0@411oln?Sh7oaHF5PKo30`<4Oooool000oooooo1old=3@7
of1PH0Ooclo?1_m6ATH2ooooo`Oo;2`/1om0@407o`@410Ko00000oooool7oa/K6`Oo92@T1om2@T85
ojFUY@?ooooo1omaLG47oahN7POo0`<31_l00003ooooo`Oo51@D1_lK6a/7o`X:2PKoHF5Q0oooool7
olS8b0Oo`L711omSHf<6o`X:2P;ooooo1om?Cdl7o`0000Oooooo1_li>CT3ooooo`ko5aLG1olI6AT6
obTY:@;ooooo1olO7al7oa<C4`Oo00001_lj>SX3ooooo`Oo00001omPH607oj:RXPKo<30`0_ooool7
o`H61POo?Shn1olR8R86odi>CP?ooooo1onRXZ87oeQHF0Ko2PX:1_lZ:RX:ooooo`OoR8R81olL71`6
ob/[:`?ooooo1_l00007oaTI6@OoATI61_lW9bL3ooooo`Oo71`L1ol>3Ph7o`H61PKo00000_ooool7
o`410@Oo?Shn1olF5QH6o``<317ooooo000?ooooo`Oo=3@d1omPH607olo?c`KoATI60_ooool7ob`/
;0Oo@4101ol410@6o`0000?ooooo1olK6a/7ob@T90Oo@T921OnUYJD3ooooo`OoLG5a1olN7Qh7o`<3
0`Ko00000oooool7oa@D50Ko6a/K1ol:2PX6of5QH@?ooooo1oo8b<P7ol71`@OoHf=S1_l:2PX2oooo
o`OoCdm?1ol00007ooooo`Ko>CTi0oooool>oaLG5`Oo6ATI1_lY:BT2ooooo`Oo7alO1olC4a<7o`00
00Ko>SXj0oooool7o`0000OoH61P1onRXZ86oc0`<0;ooooo1ol61PH7ochn?POo8R8R1_m>CTh3oooo
o`OoXZ:R1omHF5P6o`X:2PKo:RXZ2_ooool7ohR8R0Oo71`L1_l[:b/3ooooo`Ko00001olI6AT7odI6
APKo9bLW0oooool7oa`L70Oo3Ph>1ol61PH6o`0000;ooooo1ol10@47ochn?POo5QHF1_l<30`Aoooo
o`003oooool7oc@d=0OoH61P1oo?cll6odI6AP;ooooo1ol/;2`7od10@0Oo10@41_l00003ooooo`Oo
6a/K1olT92@7od92@PGoYJFU0oooool7og5aL@Oo7QhN1ol30`<6o`0000?ooooo1olD51@6oa/K6`Oo
2PX:1_mQHF43ooooo`Oob<S81oo1`L47of=SH`Ko2PX:0_ooool7odm?C`Oo00001oooool6ocTi>@?o
oooo3_lG5aL7oaTI6@Ko:BTY0_ooool7oalO7`Oo4a<C1ol00006ocXj>P?ooooo1ol00007of1PH0Oo
XZ:R1_l`<302ooooo`Oo1PH61oln?Sh7ob8R8PKoCTi>0oooool7oj:RXPOoF5QH1_l:2PX6obXZ:P[o
oooo1on8R8P7oa`L70Ko:b/[0oooool6o`0000Oo6ATI1om6ATH6obLW9`?ooooo1olL71`7o`h>3POo
1PH61_l00002ooooo`Oo0@411oln?Sh7oaHF5PKo30`<4Oooool000oooooo1old=3@7of1PH0Ooclo?
1_m6ATH2ooooo`Oo;2`/1om0@407o`@410Ko00000oooool7oa/K6`Oo92@T1om2@T85ojFUY@?ooooo
1omaLG47oahN7POo0`<31_l00003ooooo`Oo51@D1_lK6a/7o`X:2PKoHF5Q0oooool7olS8b0Oo`L71
1omSHf<6o`X:2P;ooooo1om?Cdl7o`0000Oooooo1_li>CT3ooooo`ko5aLG1olI6AT6obTY:@;ooooo
1olO7al7oa<C4`Oo00001_lj>SX3ooooo`Oo00001omPH607oj:RXPKo<30`0_ooool7o`H61POo?Shn
1olR8R86odi>CP?ooooo1onRXZ87oeQHF0Ko2PX:1_lZ:RX:ooooo`OoR8R81olL71`6ob/[:`?ooooo
1_l00007oaTI6@OoATI61_lW9bL3ooooo`Oo71`L1ol>3Ph7o`H61PKo00000_ooool7o`410@Oo?Shn
1olF5QH6o``<317ooooo000?ooooo`Oo=3@d1omPH607olo?c`KoATI60_ooool7ob`/;0Oo@4101ol4
10@6o`0000?ooooo1olK6a/7ob@T90Oo@T921OnUYJD3ooooo`OoLG5a1olN7Qh7o`<30`Ko00000ooo
ool7oa@D50Ko6a/K1ol:2PX6of5QH@?ooooo1oo8b<P7ol71`@OoHf=S1_l:2PX2ooooo`OoCdm?1ol0
0007ooooo`Ko>CTi0oooool>oaLG5`Oo6ATI1_lY:BT2ooooo`Oo7alO1olC4a<7o`0000Ko>SXj0ooo
ool7o`0000OoH61P1onRXZ86oc0`<0;ooooo1ol61PH7ochn?POo8R8R1_m>CTh3ooooo`OoXZ:R1omH
F5P6o`X:2PKo:RXZ2_ooool7ohR8R0Oo71`L1_l[:b/3ooooo`Ko00001olI6AT7odI6APKo9bLW0ooo
ool7oa`L70Oo3Ph>1ol61PH6o`0000;ooooo1ol10@47ochn?POo5QHF1_l<30`Aooooo`003oooool7
oc@d=0OoH61P1oo?cll6odI6AP;ooooo1ol/;2`7od10@0Oo10@41_l00003ooooo`Oo6a/K1olT92@7
od92@PGoYJFU0oooool7og5aL@Oo7QhN1ol30`<6o`0000?ooooo1olD51@6oa/K6`Oo2PX:1_mQHF43
ooooo`Oob<S81oo1`L47of=SH`Ko2PX:0_ooool7odm?C`Oo00001oooool6ocTi>@?ooooo3_lG5aL7
oaTI6@Ko:BTY0_ooool7oalO7`Oo4a<C1ol00006ocXj>P?ooooo1ol00007of1PH0OoXZ:R1_l`<302
ooooo`Oo1PH61oln?Sh7ob8R8PKoCTi>0oooool7oj:RXPOoF5QH1_l:2PX6obXZ:P[ooooo1on8R8P7
oa`L70Ko:b/[0oooool6o`0000Oo6ATI1om6ATH6obLW9`?ooooo1olL71`7o`h>3POo1PH61_l00002
ooooo`Oo0@411oln?Sh7oaHF5PKo30`<4Oooool000oooooo1oln?Sh7ocXj>POoPH611_l^;Rh2oooo
o`Oo9BDU1olj>SX7ob@T90Ko9RHV0oooool7oa@D50Oo00001ol92@T5oahN7P?ooooo1omPH607oc`l
?0Oo00001_l:2PX3ooooo`Oo8b<S1_l00007oa<C4`Ko<S8b0oooool7oh>3P`Oooooo1oo>c/h6oa@D
50;ooooo1om`L707oa0@40OoGemO1_lP8203ooooo`Oo2PX:1ol;2`/7oa@D50Ko7alO0_ooool7ob@T
90Oo=3@d1olE5AD6oe9BDP?ooooo1olf=SH7oaDE5@OoIfMW1_mfMWH2ooooo`Oo61PH1om?Cdl7od=3
@a3ooooo1ooalO46oc0`<0Ko7alO0oooool7od92@POoD51@1ol71`L6obPX:0?ooooo1_l<30`7ob@T
90Oooooo1_lO7al3ooooo`Oo2@T91olE5AD7o``<30Ko5aLG0_ooool7o`0000Oooooo1olk>c/6ocXj
>Q7ooooo000?ooooo`Oo?Shn1olj>SX7oh61P@Ko;Rh^0_ooool7obDU9@Oo>SXj1olT92@6obHV9P?o
oooo1olD51@7o`0000Oo2@T91OlN7Qh3ooooo`OoH61P1oll?3`7o`0000Ko2PX:0oooool7ob<S8`Ko
00001olC4a<6oc8b<P?ooooo1on3Ph<7ooooo`Ooc/k>1_lD51@2ooooo`OoL71`1ol@4107oemOG`Ko
820P0oooool7o`X:2POo2`/;1olD51@6oalO7`;ooooo1olT92@7oc@d=0Oo5ADE1_mBDU83ooooo`Oo
=SHf1olE5AD7ofMWI`KoMWIf0_ooool7oaPH60OoCdm?1om3@d<@ooooo`OolO7a1_l`<306oalO7`?o
oooo1om2@T87oe1@D0Oo1`L71_lX:2P3ooooo`Ko30`<1olT92@7ooooo`Ko7alO0oooool7o`T92@Oo
5ADE1ol<30`6oaLG5`;ooooo1ol00007ooooo`Oo>c/k1_lj>SXAooooo`003oooool7ochn?POo>SXj
1on1PH46obh^;P;ooooo1olU9BD7ocXj>POo92@T1_lV9RH3ooooo`Oo51@D1ol00007o`T92@Go7QhN
0oooool7of1PH0Oo?3`l1ol00006o`X:2P?ooooo1olS8b<6o`0000Oo4a<C1_lb<S83ooooo`OoPh>3
1oooool7olk>cPKo51@D0_ooool7og1`L0Oo410@1omOGel6ob0P80?ooooo1ol:2PX7o`/;2`Oo51@D
1_lO7al2ooooo`Oo92@T1old=3@7oaDE5@KoDU9B0oooool7ocHf=POo5ADE1omWIfL6ogIfMP;ooooo
1olH61P7odm?C`Oo@d=34?ooool7oo7al@Ko<30`1_lO7al3ooooo`Oo@T921om@D507o`L71`Ko:2PX
0oooool6o``<30Oo92@T1oooool6oalO7`?ooooo1ol92@T7oaDE5@Oo30`<1_lG5aL2ooooo`Oo0000
1oooool7oc/k>`Ko>SXj4Oooool000oooooo1oln?Sh7ocXj>POoPH611_l^;Rh2ooooo`Oo9BDU1olj
>SX7ob@T90Ko9RHV0oooool7oa@D50Oo00001ol92@T5oahN7P?ooooo1omPH607oc`l?0Oo00001_l:
2PX3ooooo`Oo8b<S1_l00007oa<C4`Ko<S8b0oooool7oh>3P`Oooooo1oo>c/h6oa@D50;ooooo1om`
L707oa0@40OoGemO1_lP8203ooooo`Oo2PX:1ol;2`/7oa@D50Ko7alO0_ooool7ob@T90Oo=3@d1olE
5AD6oe9BDP?ooooo1olf=SH7oaDE5@OoIfMW1_mfMWH2ooooo`Oo61PH1om?Cdl7od=3@a3ooooo1ooa
lO46oc0`<0Ko7alO0oooool7od92@POoD51@1ol71`L6obPX:0?ooooo1_l<30`7ob@T90Oooooo1_lO
7al3ooooo`Oo2@T91olE5AD7o``<30Ko5aLG0_ooool7o`0000Oooooo1olk>c/6ocXj>Q7ooooo000?
ooooo`Oo?Shn1olj>SX7oh61P@Ko;Rh^0_ooool7obDU9@Oo>SXj1olT92@6obHV9P?ooooo1olD51@7
o`0000Oo2@T91OlN7Qh3ooooo`OoH61P1oll?3`7o`0000Ko2PX:0oooool7ob<S8`Ko00001olC4a<6
oc8b<P?ooooo1on3Ph<7ooooo`Ooc/k>1_lD51@2ooooo`OoL71`1ol@4107oemOG`Ko820P0oooool7
o`X:2POo2`/;1olD51@6oalO7`;ooooo1olT92@7oc@d=0Oo5ADE1_mBDU83ooooo`Oo=SHf1olE5AD7
ofMWI`KoMWIf0_ooool7oaPH60OoCdm?1om3@d<@ooooo`OolO7a1_l`<306oalO7`?ooooo1om2@T87
oe1@D0Oo1`L71_lX:2P3ooooo`Ko30`<1olT92@7ooooo`Ko7alO0oooool7o`T92@Oo5ADE1ol<30`6
oaLG5`;ooooo1ol00007ooooo`Oo>c/k1_lj>SXAooooo`003oooool7ochn?POo>SXj1on1PH46obh^
;P;ooooo1olU9BD7ocXj>POo92@T1_lV9RH3ooooo`Oo51@D1ol00007o`T92@Go7QhN0oooool7of1P
H0Oo?3`l1ol00006o`X:2P?ooooo1olS8b<6o`0000Oo4a<C1_lb<S83ooooo`OoPh>31oooool7olk>
cPKo51@D0_ooool7og1`L0Oo410@1omOGel6ob0P80?ooooo1ol:2PX7o`/;2`Oo51@D1_lO7al2oooo
o`Oo92@T1old=3@7oaDE5@KoDU9B0oooool7ocHf=POo5ADE1omWIfL6ogIfMP;ooooo1olH61P7odm?
C`Oo@d=34?ooool7oo7al@Ko<30`1_lO7al3ooooo`Oo@T921om@D507o`L71`Ko:2PX0oooool6o``<
30Oo92@T1oooool6oalO7`?ooooo1ol92@T7oaDE5@Oo30`<1_lG5aL2ooooo`Oo00001oooool7oc/k
>`Ko>SXj4Oooool000oooooo1olH61P7o`0000Oooooo1_l00009ooooo`Oo?Cdm1olF5QH6oc4a<@[o
oooo1olo?cl7o`P820Go?3`l0oooool7oeiNGPOo2@T91ol[:b/6oginOP?ooooo1olM7Ad6oa`L70Oo
30`<1_ll?3`3ooooo`OoV9RH1olo?cl7o`X:2PKo>CTi0_ooool7o`d=3@Oo:2PX1omGEeL6ofa/K0?o
oooo1om;Bd/7oa4A4@Oo820P2?ooool7oa<C4`Oo3@d=1olL71`9ooooo`OoXJ6Q1oooool7ohN7Q`Ko
2`/;0_ooool7oh:2PPOo=3@d1olH61P6o`0000?ooooo1olm?Cd7o`l?3`KoP8201_m;Bd/3ooooo`Oo
MGEe1ol:2PX7od=3@`Ko10@40oooool6oa/K6`Oo61PH1oli>CT6oa`L70?ooooo1olM7Ad7o`D51@Oo
oooo1_lT92@2ooooo`Oo4Q8B1oli>CT7oa/K6`Ko=3@d4Oooool000oooooo1olH61P7o`0000Oooooo
1_l00009ooooo`Oo?Cdm1olF5QH6oc4a<@[ooooo1olo?cl7o`P820Go?3`l0oooool7oeiNGPOo2@T9
1ol[:b/6oginOP?ooooo1olM7Ad6oa`L70Oo30`<1_ll?3`3ooooo`OoV9RH1olo?cl7o`X:2PKo>CTi
0_ooool7o`d=3@Oo:2PX1omGEeL6ofa/K0?ooooo1om;Bd/7oa4A4@Oo820P2?ooool7oa<C4`Oo3@d=
1olL71`9ooooo`OoXJ6Q1oooool7ohN7Q`Ko2`/;0_ooool7oh:2PPOo=3@d1olH61P6o`0000?ooooo
1olm?Cd7o`l?3`KoP8201_m;Bd/3ooooo`OoMGEe1ol:2PX7od=3@`Ko10@40oooool6oa/K6`Oo61PH
1oli>CT6oa`L70?ooooo1olM7Ad7o`D51@Oooooo1_lT92@2ooooo`Oo4Q8B1oli>CT7oa/K6`Ko=3@d
4Oooool000oooooo1olH61P7o`0000Oooooo1_l00009ooooo`Oo?Cdm1olF5QH6oc4a<@[ooooo1olo
?cl7o`P820Go?3`l0oooool7oeiNGPOo2@T91ol[:b/6oginOP?ooooo1olM7Ad6oa`L70Oo30`<1_ll
?3`3ooooo`OoV9RH1olo?cl7o`X:2PKo>CTi0_ooool7o`d=3@Oo:2PX1omGEeL6ofa/K0?ooooo1om;
Bd/7oa4A4@Oo820P2?ooool7oa<C4`Oo3@d=1olL71`9ooooo`OoXJ6Q1oooool7ohN7Q`Ko2`/;0_oo
ool7oh:2PPOo=3@d1olH61P6o`0000?ooooo1olm?Cd7o`l?3`KoP8201_m;Bd/3ooooo`OoMGEe1ol:
2PX7od=3@`Ko10@40oooool6oa/K6`Oo61PH1oli>CT6oa`L70?ooooo1olM7Ad7o`D51@Oooooo1_lT
92@2ooooo`Oo4Q8B1oli>CT7oa/K6`Ko=3@d4Oooool000oooooo1olH61P7o`0000Oooooo1_l00009
ooooo`Oo?Cdm1olF5QH6oc4a<@[ooooo1olo?cl7o`P820Go?3`l0oooool7oeiNGPOo2@T91ol[:b/6
oginOP?ooooo1olM7Ad6oa`L70Oo30`<1_ll?3`3ooooo`OoV9RH1olo?cl7o`X:2PKo>CTi0_ooool7
o`d=3@Oo:2PX1omGEeL6ofa/K0?ooooo1om;Bd/7oa4A4@Oo820P2?ooool7oa<C4`Oo3@d=1olL71`9
ooooo`OoXJ6Q1oooool7ohN7Q`Ko2`/;0_ooool7oh:2PPOo=3@d1olH61P6o`0000?ooooo1olm?Cd7
o`l?3`KoP8201_m;Bd/3ooooo`OoMGEe1ol:2PX7od=3@`Ko10@40oooool6oa/K6`Oo61PH1oli>CT6
oa`L70?ooooo1olM7Ad7o`D51@Oooooo1_lT92@2ooooo`Oo4Q8B1oli>CT7oa/K6`Ko=3@d4Oooool0
00oooooo1olH61P7o`0000Oooooo1_l00009ooooo`Oo?Cdm1olF5QH6oc4a<@[ooooo1olo?cl7o`P8
20Go?3`l0oooool7oeiNGPOo2@T91ol[:b/6oginOP?ooooo1olM7Ad6oa`L70Oo30`<1_ll?3`3oooo
o`OoV9RH1olo?cl7o`X:2PKo>CTi0_ooool7o`d=3@Oo:2PX1omGEeL6ofa/K0?ooooo1om;Bd/7oa4A
4@Oo820P2?ooool7oa<C4`Oo3@d=1olL71`9ooooo`OoXJ6Q1oooool7ohN7Q`Ko2`/;0_ooool7oh:2
PPOo=3@d1olH61P6o`0000?ooooo1olm?Cd7o`l?3`KoP8201_m;Bd/3ooooo`OoMGEe1ol:2PX7od=3
@`Ko10@40oooool6oa/K6`Oo61PH1oli>CT6oa`L70?ooooo1olM7Ad7o`D51@Oooooo1_lT92@2oooo
o`Oo4Q8B1oli>CT7oa/K6`Ko=3@d4Oooool000oooooo1olH61P7o`0000Oooooo1_l00009ooooo`Oo
?Cdm1olF5QH6oc4a<@[ooooo1olo?cl7o`P820Go?3`l0oooool7oeiNGPOo2@T91ol[:b/6oginOP?o
oooo1olM7Ad6oa`L70Oo30`<1_ll?3`3ooooo`OoV9RH1olo?cl7o`X:2PKo>CTi0_ooool7o`d=3@Oo
:2PX1omGEeL6ofa/K0?ooooo1om;Bd/7oa4A4@Oo820P2?ooool7oa<C4`Oo3@d=1olL71`9ooooo`Oo
XJ6Q1oooool7ohN7Q`Ko2`/;0_ooool7oh:2PPOo=3@d1olH61P6o`0000?ooooo1olm?Cd7o`l?3`Ko
P8201_m;Bd/3ooooo`OoMGEe1ol:2PX7od=3@`Ko10@40oooool6oa/K6`Oo61PH1oli>CT6oa`L70?o
oooo1olM7Ad7o`D51@Oooooo1_lT92@2ooooo`Oo4Q8B1oli>CT7oa/K6`Ko=3@d4Oooool000oooooo
1olH61P7o`0000Oooooo1_l00009ooooo`Oo?Cdm1olF5QH6oc4a<@[ooooo1olo?cl7o`P820Go?3`l
0oooool7oeiNGPOo2@T91ol[:b/6oginOP?ooooo1olM7Ad6oa`L70Oo30`<1_ll?3`3ooooo`OoV9RH
1olo?cl7o`X:2PKo>CTi0_ooool7o`d=3@Oo:2PX1omGEeL6ofa/K0?ooooo1om;Bd/7oa4A4@Oo820P
2?ooool7oa<C4`Oo3@d=1olL71`9ooooo`OoXJ6Q1oooool7ohN7Q`Ko2`/;0_ooool7oh:2PPOo=3@d
1olH61P6o`0000?ooooo1olm?Cd7o`l?3`KoP8201_m;Bd/3ooooo`OoMGEe1ol:2PX7od=3@`Ko10@4
0oooool6oa/K6`Oo61PH1oli>CT6oa`L70?ooooo1olM7Ad7o`D51@Oooooo1_lT92@2ooooo`Oo4Q8B
1oli>CT7oa/K6`Ko=3@d4Oooool00?oooooomOooool00?oooooomOooool00?oooooomOooool000oo
oooo1olE5AD7ochn?POo2PX:1_l410@2ooooo`OodM7A1oooool7oeEEE@Ko0P820oooool7oa<C4`Oo
00001ol<30`5oa0@40?ooooo1om;Bd/7o`0000Oo9BDU1_mRHV83ooooo`Oo?Cdm1_lB4Q87oa@D50Wo
oooo1on@T907odi>CPOo00001_le=CD2ooooo`Oo00001oooool7oo7al@KoVi^K0oooool7ob@T90Oo
QHF51omVIVH6odQ8B0;ooooo1olA4A47of5QH@Oo7alO1_n4Q8@3ooooo`Oo00001olQ8B47o`h>3PKo
92@T0_ooool7o`h>3POo6a/K1olR8R86odY:BP?ooooo1ol00007obDU9@Ko;2`/1_l^;Rh3ooooo`Oo
@4101ol?3`l7o`T92@Ko3Ph>0oooool6oadM7@OoP8201olL71`6oc@d=0?ooooo1olF5QH7oa`L70Oo
oooo1_lf=SH2ooooo`Oo00001ol51@D7oaHF5PKo61PH4Oooool000oooooo1olE5AD7ochn?POo2PX:
1_l410@2ooooo`OodM7A1oooool7oeEEE@Ko0P820oooool7oa<C4`Oo00001ol<30`5oa0@40?ooooo
1om;Bd/7o`0000Oo9BDU1_mRHV83ooooo`Oo?Cdm1_lB4Q87oa@D50Wooooo1on@T907odi>CPOo0000
1_le=CD2ooooo`Oo00001oooool7oo7al@KoVi^K0oooool7ob@T90OoQHF51omVIVH6odQ8B0;ooooo
1olA4A47of5QH@Oo7alO1_n4Q8@3ooooo`Oo00001olQ8B47o`h>3PKo92@T0_ooool7o`h>3POo6a/K
1olR8R86odY:BP?ooooo1ol00007obDU9@Ko;2`/1_l^;Rh3ooooo`Oo@4101ol?3`l7o`T92@Ko3Ph>
0oooool6oadM7@OoP8201olL71`6oc@d=0?ooooo1olF5QH7oa`L70Oooooo1_lf=SH2ooooo`Oo0000
1ol51@D7oaHF5PKo61PH4Oooool000oooooo1olE5AD7ochn?POo2PX:1_l410@2ooooo`OodM7A1ooo
ool7oeEEE@Ko0P820oooool7oa<C4`Oo00001ol<30`5oa0@40?ooooo1om;Bd/7o`0000Oo9BDU1_mR
HV83ooooo`Oo?Cdm1_lB4Q87oa@D50Wooooo1on@T907odi>CPOo00001_le=CD2ooooo`Oo00001ooo
ool7oo7al@KoVi^K0oooool7ob@T90OoQHF51omVIVH6odQ8B0;ooooo1olA4A47of5QH@Oo7alO1_n4
Q8@3ooooo`Oo00001olQ8B47o`h>3PKo92@T0_ooool7o`h>3POo6a/K1olR8R86odY:BP?ooooo1ol0
0007obDU9@Ko;2`/1_l^;Rh3ooooo`Oo@4101ol?3`l7o`T92@Ko3Ph>0oooool6oadM7@OoP8201olL
71`6oc@d=0?ooooo1olF5QH7oa`L70Oooooo1_lf=SH2ooooo`Oo00001ol51@D7oaHF5PKo61PH4Ooo
ool000oooooo1olE5AD7ochn?POo2PX:1_l410@2ooooo`OodM7A1oooool7oeEEE@Ko0P820oooool7
oa<C4`Oo00001ol<30`5oa0@40?ooooo1om;Bd/7o`0000Oo9BDU1_mRHV83ooooo`Oo?Cdm1_lB4Q87
oa@D50Wooooo1on@T907odi>CPOo00001_le=CD2ooooo`Oo00001oooool7oo7al@KoVi^K0oooool7
ob@T90OoQHF51omVIVH6odQ8B0;ooooo1olA4A47of5QH@Oo7alO1_n4Q8@3ooooo`Oo00001olQ8B47
o`h>3PKo92@T0_ooool7o`h>3POo6a/K1olR8R86odY:BP?ooooo1ol00007obDU9@Ko;2`/1_l^;Rh3
ooooo`Oo@4101ol?3`l7o`T92@Ko3Ph>0oooool6oadM7@OoP8201olL71`6oc@d=0?ooooo1olF5QH7
oa`L70Oooooo1_lf=SH2ooooo`Oo00001ol51@D7oaHF5PKo61PH4Oooool000oooooo1olE5AD7ochn
?POo2PX:1_l410@2ooooo`OodM7A1oooool7oeEEE@Ko0P820oooool7oa<C4`Oo00001ol<30`5oa0@
40?ooooo1om;Bd/7o`0000Oo9BDU1_mRHV83ooooo`Oo?Cdm1_lB4Q87oa@D50Wooooo1on@T907odi>
CPOo00001_le=CD2ooooo`Oo00001oooool7oo7al@KoVi^K0oooool7ob@T90OoQHF51omVIVH6odQ8
B0;ooooo1olA4A47of5QH@Oo7alO1_n4Q8@3ooooo`Oo00001olQ8B47o`h>3PKo92@T0_ooool7o`h>
3POo6a/K1olR8R86odY:BP?ooooo1ol00007obDU9@Ko;2`/1_l^;Rh3ooooo`Oo@4101ol?3`l7o`T9
2@Ko3Ph>0oooool6oadM7@OoP8201olL71`6oc@d=0?ooooo1olF5QH7oa`L70Oooooo1_lf=SH2oooo
o`Oo00001ol51@D7oaHF5PKo61PH4Oooool000oooooo1olE5AD7ochn?POo2PX:1_l410@2ooooo`Oo
dM7A1oooool7oeEEE@Ko0P820oooool7oa<C4`Oo00001ol<30`5oa0@40?ooooo1om;Bd/7o`0000Oo
9BDU1_mRHV83ooooo`Oo?Cdm1_lB4Q87oa@D50Wooooo1on@T907odi>CPOo00001_le=CD2ooooo`Oo
00001oooool7oo7al@KoVi^K0oooool7ob@T90OoQHF51omVIVH6odQ8B0;ooooo1olA4A47of5QH@Oo
7alO1_n4Q8@3ooooo`Oo00001olQ8B47o`h>3PKo92@T0_ooool7o`h>3POo6a/K1olR8R86odY:BP?o
oooo1ol00007obDU9@Ko;2`/1_l^;Rh3ooooo`Oo@4101ol?3`l7o`T92@Ko3Ph>0oooool6oadM7@Oo
P8201olL71`6oc@d=0?ooooo1olF5QH7oa`L70Oooooo1_lf=SH2ooooo`Oo00001ol51@D7oaHF5PKo
61PH4Oooool000oooooo1olE5AD7ochn?POo2PX:1_l410@2ooooo`OodM7A1oooool7oeEEE@Ko0P82
0oooool7oa<C4`Oo00001ol<30`5oa0@40?ooooo1om;Bd/7o`0000Oo9BDU1_mRHV83ooooo`Oo?Cdm
1_lB4Q87oa@D50Wooooo1on@T907odi>CPOo00001_le=CD2ooooo`Oo00001oooool7oo7al@KoVi^K
0oooool7ob@T90OoQHF51omVIVH6odQ8B0;ooooo1olA4A47of5QH@Oo7alO1_n4Q8@3ooooo`Oo0000
1olQ8B47o`h>3PKo92@T0_ooool7o`h>3POo6a/K1olR8R86odY:BP?ooooo1ol00007obDU9@Ko;2`/
1_l^;Rh3ooooo`Oo@4101ol?3`l7o`T92@Ko3Ph>0oooool6oadM7@OoP8201olL71`6oc@d=0?ooooo
1olF5QH7oa`L70Oooooo1_lf=SH2ooooo`Oo00001ol51@D7oaHF5PKo61PH4Oooool000oooooo1ol@
4107obh^;POo00001_lD51@2ooooo`Oo>SXj1oni^KT7oginOPKo5QHF0oooool7ob<S8`Oo00001ol=
3@d?ooooo`Oo:BTY1ol;2`/6ode=C@?ooooo1ol30`<6oa@D50Oo410@1_ld=3@3ooooo`Oo^KVi1ooo
ool7oa<C4`Ko5ADE0_ooool7obDU9@OoAdM71oo2`/86ol30`0?ooooo1olW9bL7ob@T90Oo00001_lK
6a/2ooooo`OoE5AD1omTI6@7o`0000KoGemO0oooool7o`h>3POo20P81ol<30`6oaXJ6P;ooooo1oln
?Sh7o`h>3POoCdm?1_mKFe/3ooooo`Oo?Shn1ol00006oa4A4@KoF5QH0oooool7oalO7`Oo00001ol7
1`L6o`/;2`?ooooo1_l410@7ooooo`OoVYZJ1_n1PH43ooooo`OoDU9B1ol00007ocLg=`Ko>SXj0_oo
ool7o`X:2POo4A4A1olE5AD6oaLG5a7ooooo000?ooooo`Oo410@1ol^;Rh7o`0000Ko51@D0_ooool7
ocXj>POo^KVi1omnOWh6oaHF5P?ooooo1olS8b<7o`0000Oo3@d=3oooool7obTY:@Oo2`/;1_m=CDd3
ooooo`Oo0`<31_lD51@7oa0@40Ko=3@d0oooool7okVi^@Oooooo1olC4a<6oaDE5@;ooooo1olU9BD7
odM7A`Oo`/;21_o0`<03ooooo`Oo9bLW1olT92@7o`0000Ko6a/K0_ooool7oeADE0OoI6AT1ol00006
oemOG`?ooooo1ol>3Ph7o`P820Oo30`<1_lJ6QX2ooooo`Oo?Shn1ol>3Ph7odm?C`KoFe]K0oooool7
ochn?POo00001_lA4A46oeQHF0?ooooo1olO7al7o`0000Oo1`L71_l;2`/3ooooo`Ko10@41oooool7
oiZJVPKoPH610oooool7oe9BDPOo00001olg=cL6ocXj>P;ooooo1ol:2PX7oa4A4@Oo5ADE1_lG5aLA
ooooo`003oooool7oa0@40Oo;Rh^1ol00006oa@D50;ooooo1olj>SX7okVi^@OoOWin1_lF5QH3oooo
o`Oo8b<S1ol00007o`d=3@oooooo1olY:BT7o`/;2`KoCDe=0oooool7o`<30`Ko51@D1ol@4106oc@d
=0?ooooo1oni^KT7ooooo`Oo4a<C1_lE5AD2ooooo`Oo9BDU1om7AdL7ol;2`PKo`<300oooool7obLW
9`Oo92@T1ol00006oa/K6`;ooooo1omDE5@7ofATI0Oo00001_mOGel3ooooo`Oo3Ph>1ol820P7o``<
30Ko6QXJ0_ooool7ochn?POo3Ph>1om?Cdl6oe]KF`?ooooo1oln?Sh7o`0000Ko4A4A1_mHF5P3oooo
o`Oo7alO1ol00007o`L71`Ko2`/;0oooool6o`@410Oooooo1onJVYX6oh61P@?ooooo1omBDU87o`00
00Oo=cLg1_lj>SX2ooooo`Oo2PX:1olA4A47oaDE5@Ko5aLG4Oooool000oooooo1ol@4107obh^;POo
00001_lD51@2ooooo`Oo>SXj1oni^KT7oginOPKo5QHF0oooool7ob<S8`Oo00001ol=3@d?ooooo`Oo
:BTY1ol;2`/6ode=C@?ooooo1ol30`<6oa@D50Oo410@1_ld=3@3ooooo`Oo^KVi1oooool7oa<C4`Ko
5ADE0_ooool7obDU9@OoAdM71oo2`/86ol30`0?ooooo1olW9bL7ob@T90Oo00001_lK6a/2ooooo`Oo
E5AD1omTI6@7o`0000KoGemO0oooool7o`h>3POo20P81ol<30`6oaXJ6P;ooooo1oln?Sh7o`h>3POo
Cdm?1_mKFe/3ooooo`Oo?Shn1ol00006oa4A4@KoF5QH0oooool7oalO7`Oo00001ol71`L6o`/;2`?o
oooo1_l410@7ooooo`OoVYZJ1_n1PH43ooooo`OoDU9B1ol00007ocLg=`Ko>SXj0_ooool7o`X:2POo
4A4A1olE5AD6oaLG5a7ooooo000?ooooo`Oo410@1ol^;Rh7o`0000Ko51@D0_ooool7ocXj>POo^KVi
1omnOWh6oaHF5P?ooooo1olS8b<7o`0000Oo3@d=3oooool7obTY:@Oo2`/;1_m=CDd3ooooo`Oo0`<3
1_lD51@7oa0@40Ko=3@d0oooool7okVi^@Oooooo1olC4a<6oaDE5@;ooooo1olU9BD7odM7A`Oo`/;2
1_o0`<03ooooo`Oo9bLW1olT92@7o`0000Ko6a/K0_ooool7oeADE0OoI6AT1ol00006oemOG`?ooooo
1ol>3Ph7o`P820Oo30`<1_lJ6QX2ooooo`Oo?Shn1ol>3Ph7odm?C`KoFe]K0oooool7ochn?POo0000
1_lA4A46oeQHF0?ooooo1olO7al7o`0000Oo1`L71_l;2`/3ooooo`Ko10@41oooool7oiZJVPKoPH61
0oooool7oe9BDPOo00001olg=cL6ocXj>P;ooooo1ol:2PX7oa4A4@Oo5ADE1_lG5aLAooooo`003ooo
ool7oa0@40Oo;Rh^1ol00006oa@D50;ooooo1olj>SX7okVi^@OoOWin1_lF5QH3ooooo`Oo8b<S1ol0
0007o`d=3@oooooo1olY:BT7o`/;2`KoCDe=0oooool7o`<30`Ko51@D1ol@4106oc@d=0?ooooo1oni
^KT7ooooo`Oo4a<C1_lE5AD2ooooo`Oo9BDU1om7AdL7ol;2`PKo`<300oooool7obLW9`Oo92@T1ol0
0006oa/K6`;ooooo1omDE5@7ofATI0Oo00001_mOGel3ooooo`Oo3Ph>1ol820P7o``<30Ko6QXJ0_oo
ool7ochn?POo3Ph>1om?Cdl6oe]KF`?ooooo1oln?Sh7o`0000Ko4A4A1_mHF5P3ooooo`Oo7alO1ol0
0007o`L71`Ko2`/;0oooool6o`@410Oooooo1onJVYX6oh61P@?ooooo1omBDU87o`0000Oo=cLg1_lj
>SX2ooooo`Oo2PX:1olA4A47oaDE5@Ko5aLG4Oooool000oooooo1ol51@D7ob4Q8@Oo51@D1_l?3`l2
ooooo`Oo00001on8R8P7oi:BTPKo<C4a0oooool7o`@410Oo2PX:1ol30`<5oc0`<0?ooooo1omcLg<7
oc8b<POo5ADE1_lG5aL3ooooo`Oo4A4A1_l20P87o`0000Ko71`L0oooool7ogalO0OoXZ:R1olC4a<6
o`X:2P;ooooo1omKFe/7o`X:2POo_kno1_mMGEd3ooooo`Oo61PH1ol92@T7o`H61PKo=CDe0_ooool7
okJf]POo_Kfm1onHV9P6on_[j`?ooooo1ol@4107o``<30Oo20P81_lN7Qh2ooooo`OoHf=S1ol00007
oeIFEQ3ooooo1ol`<306o`820PKoF5QH0oooool7ocTi>@Oo51@D1ol71`L6o`410@?ooooo1_l00007
ohV9R@Oo][Jf1_nIVIT3ooooo`OoSXj>1olQ8B47o`h>3PKo9BDU0_ooool7oa4A4@Oo5aLG1ol20P86
oalO7a7ooooo000?ooooo`Oo1@D51olQ8B47oa@D50Ko3`l?0_ooool7o`0000OoR8R81onBTY86oc4a
<@?ooooo1ol410@7o`X:2POo0`<31Ol`<303ooooo`OoLg=c1olb<S87oaDE5@Ko5aLG0oooool7oa4A
4@Ko0P821ol00006oa`L70?ooooo1omlO7`7oj:RXPOo4a<C1_l:2PX2ooooo`OoFe]K1ol:2PX7okno
_`KoGEeM0oooool7oaPH60Oo2@T91ol61PH6ocDe=@;ooooo1onf][H7okfm_@OoV9RH1_o[jn/3oooo
o`Oo410@1ol<30`7o`P820Ko7QhN0_ooool7of=SH`Oo00001omFEUH@ooooo`Oo<30`1_l20P86oeQH
F0?ooooo1oli>CT7oa@D50Oo1`L71_l10@43ooooo`Ko00001on9RHT7okJf]PKoVIVI0oooool7ohj>
SPOo8B4Q1ol>3Ph6obDU9@;ooooo1olA4A47oaLG5`Oo0P821_lO7alAooooo`003oooool7o`D51@Oo
8B4Q1olD51@6o`l?3`;ooooo1ol00007ohR8R0OoTY:B1_la<C43ooooo`Oo10@41ol:2PX7o`<30`Go
<30`0oooool7og=cL`Oo<S8b1olE5AD6oaLG5`?ooooo1olA4A46o`820POo00001_lL71`3ooooo`Oo
O7al1onRXZ87oa<C4`Ko2PX:0_ooool7oe]KF`Oo2PX:1ono_kl6oeeMG@?ooooo1olH61P7o`T92@Oo
1PH61_le=CD2ooooo`Oo][Jf1onm_Kd7oiRHV0Kojn_[0oooool7oa0@40Oo30`<1ol820P6oahN7P;o
oooo1omSHf<7o`0000OoEUIF4?ooool7oc0`<0Ko0P821_mHF5P3ooooo`Oo>CTi1olD51@7o`L71`Ko
0@410oooool6o`0000OoRHV91onf][H6oiVIV@?ooooo1on>SXh7ob4Q8@Oo3Ph>1_lU9BD2ooooo`Oo
4A4A1olG5aL7o`820PKo7alO4Oooool000oooooo1ol51@D7ob4Q8@Oo51@D1_l?3`l2ooooo`Oo0000
1on8R8P7oi:BTPKo<C4a0oooool7o`@410Oo2PX:1ol30`<5oc0`<0?ooooo1omcLg<7oc8b<POo5ADE
1_lG5aL3ooooo`Oo4A4A1_l20P87o`0000Ko71`L0oooool7ogalO0OoXZ:R1olC4a<6o`X:2P;ooooo
1omKFe/7o`X:2POo_kno1_mMGEd3ooooo`Oo61PH1ol92@T7o`H61PKo=CDe0_ooool7okJf]POo_Kfm
1onHV9P6on_[j`?ooooo1ol@4107o``<30Oo20P81_lN7Qh2ooooo`OoHf=S1ol00007oeIFEQ3ooooo
1ol`<306o`820PKoF5QH0oooool7ocTi>@Oo51@D1ol71`L6o`410@?ooooo1_l00007ohV9R@Oo][Jf
1_nIVIT3ooooo`OoSXj>1olQ8B47o`h>3PKo9BDU0_ooool7oa4A4@Oo5aLG1ol20P86oalO7a7ooooo
000?ooooo`Oo1@D51olQ8B47oa@D50Ko3`l?0_ooool7o`0000OoR8R81onBTY86oc4a<@?ooooo1ol4
10@7o`X:2POo0`<31Ol`<303ooooo`OoLg=c1olb<S87oaDE5@Ko5aLG0oooool7oa4A4@Ko0P821ol0
0006oa`L70?ooooo1omlO7`7oj:RXPOo4a<C1_l:2PX2ooooo`OoFe]K1ol:2PX7okno_`KoGEeM0ooo
ool7oaPH60Oo2@T91ol61PH6ocDe=@;ooooo1onf][H7okfm_@OoV9RH1_o[jn/3ooooo`Oo410@1ol<
30`7o`P820Ko7QhN0_ooool7of=SH`Oo00001omFEUH@ooooo`Oo<30`1_l20P86oeQHF0?ooooo1oli
>CT7oa@D50Oo1`L71_l10@43ooooo`Ko00001on9RHT7okJf]PKoVIVI0oooool7ohj>SPOo8B4Q1ol>
3Ph6obDU9@;ooooo1olA4A47oaLG5`Oo0P821_lO7alAooooo`003oooool7o`D51@Oo8B4Q1olD51@6
o`l?3`;ooooo1ol00007ohR8R0OoTY:B1_la<C43ooooo`Oo10@41ol:2PX7o`<30`Go<30`0oooool7
og=cL`Oo<S8b1olE5AD6oaLG5`?ooooo1olA4A46o`820POo00001_lL71`3ooooo`OoO7al1onRXZ87
oa<C4`Ko2PX:0_ooool7oe]KF`Oo2PX:1ono_kl6oeeMG@?ooooo1olH61P7o`T92@Oo1PH61_le=CD2
ooooo`Oo][Jf1onm_Kd7oiRHV0Kojn_[0oooool7oa0@40Oo30`<1ol820P6oahN7P;ooooo1omSHf<7
o`0000OoEUIF4?ooool7oc0`<0Ko0P821_mHF5P3ooooo`Oo>CTi1olD51@7o`L71`Ko0@410oooool6
o`0000OoRHV91onf][H6oiVIV@?ooooo1on>SXh7ob4Q8@Oo3Ph>1_lU9BD2ooooo`Oo4A4A1olG5aL7
o`820PKo7alO4Oooool000oooooo1ol51@D7ob4Q8@Oo51@D1_l?3`l2ooooo`Oo00001on8R8P7oi:B
TPKo<C4a0oooool7o`@410Oo2PX:1ol30`<5oc0`<0?ooooo1omcLg<7oc8b<POo5ADE1_lG5aL3oooo
o`Oo4A4A1_l20P87o`0000Ko71`L0oooool7ogalO0OoXZ:R1olC4a<6o`X:2P;ooooo1omKFe/7o`X:
2POo_kno1_mMGEd3ooooo`Oo61PH1ol92@T7o`H61PKo=CDe0_ooool7okJf]POo_Kfm1onHV9P6on_[
j`?ooooo1ol@4107o``<30Oo20P81_lN7Qh2ooooo`OoHf=S1ol00007oeIFEQ3ooooo1ol`<306o`82
0PKoF5QH0oooool7ocTi>@Oo51@D1ol71`L6o`410@?ooooo1_l00007ohV9R@Oo][Jf1_nIVIT3oooo
o`OoSXj>1olQ8B47o`h>3PKo9BDU0_ooool7oa4A4@Oo5aLG1ol20P86oalO7a7ooooo000?ooooo`Oo
410@1oooool7obHV9PKo7QhN0_ooool7okVi^@OoWYjN1ol30`<6oa@D50?ooooo1ol51@D7o`T92@Oo
6ATI1OlW9bL3ooooo`Oo1`L71olV9RH7obDU9@KoD51@0oooool7oa<C4`Ko1`L71olF5QH6oalO7`?o
oooo1omNGUh7o`410@Oo7alO1_m4A4@2ooooo`Oo5ADE1onUYJD7of1PH0KoFUYJ0oooool7o`D51@Oo
>c/k1om2@T88ooooo`OoHV9R1oooool7ol[:bPKo6QXJ0oooool7o``<30Oooooo1olj>SX6ocPh>0;o
oooo1olF5QH7oc<c<`Oo9BDU1_li>CT3ooooo`Oo20P81omIFET<ob<S8`[ooooo1olS8b<7oalO7`Ko
3@d=0oooool6oc@d=0OoLG5a1ol:2PX6oaDE5@?ooooo1ol51@D7of1PH0Oog]kN1_m4A4@2ooooo`Oo
9RHV1olQ8B47ob<S8aOooooo000?ooooo`Oo410@1oooool7obHV9PKo7QhN0_ooool7okVi^@OoWYjN
1ol30`<6oa@D50?ooooo1ol51@D7o`T92@Oo6ATI1OlW9bL3ooooo`Oo1`L71olV9RH7obDU9@KoD51@
0oooool7oa<C4`Ko1`L71olF5QH6oalO7`?ooooo1omNGUh7o`410@Oo7alO1_m4A4@2ooooo`Oo5ADE
1onUYJD7of1PH0KoFUYJ0oooool7o`D51@Oo>c/k1om2@T88ooooo`OoHV9R1oooool7ol[:bPKo6QXJ
0oooool7o``<30Oooooo1olj>SX6ocPh>0;ooooo1olF5QH7oc<c<`Oo9BDU1_li>CT3ooooo`Oo20P8
1omIFET<ob<S8`[ooooo1olS8b<7oalO7`Ko3@d=0oooool6oc@d=0OoLG5a1ol:2PX6oaDE5@?ooooo
1ol51@D7of1PH0Oog]kN1_m4A4@2ooooo`Oo9RHV1olQ8B47ob<S8aOooooo000?ooooo`Oo410@1ooo
ool7obHV9PKo7QhN0_ooool7okVi^@OoWYjN1ol30`<6oa@D50?ooooo1ol51@D7o`T92@Oo6ATI1OlW
9bL3ooooo`Oo1`L71olV9RH7obDU9@KoD51@0oooool7oa<C4`Ko1`L71olF5QH6oalO7`?ooooo1omN
GUh7o`410@Oo7alO1_m4A4@2ooooo`Oo5ADE1onUYJD7of1PH0KoFUYJ0oooool7o`D51@Oo>c/k1om2
@T88ooooo`OoHV9R1oooool7ol[:bPKo6QXJ0oooool7o``<30Oooooo1olj>SX6ocPh>0;ooooo1olF
5QH7oc<c<`Oo9BDU1_li>CT3ooooo`Oo20P81omIFET<ob<S8`[ooooo1olS8b<7oalO7`Ko3@d=0ooo
ool6oc@d=0OoLG5a1ol:2PX6oaDE5@?ooooo1ol51@D7of1PH0Oog]kN1_m4A4@2ooooo`Oo9RHV1olQ
8B47ob<S8aOooooo000?ooooo`Oo410@1oooool7obHV9PKo7QhN0_ooool7okVi^@OoWYjN1ol30`<6
oa@D50?ooooo1ol51@D7o`T92@Oo6ATI1OlW9bL3ooooo`Oo1`L71olV9RH7obDU9@KoD51@0oooool7
oa<C4`Ko1`L71olF5QH6oalO7`?ooooo1omNGUh7o`410@Oo7alO1_m4A4@2ooooo`Oo5ADE1onUYJD7
of1PH0KoFUYJ0oooool7o`D51@Oo>c/k1om2@T88ooooo`OoHV9R1oooool7ol[:bPKo6QXJ0oooool7
o``<30Oooooo1olj>SX6ocPh>0;ooooo1olF5QH7oc<c<`Oo9BDU1_li>CT3ooooo`Oo20P81omIFET<
ob<S8`[ooooo1olS8b<7oalO7`Ko3@d=0oooool6oc@d=0OoLG5a1ol:2PX6oaDE5@?ooooo1ol51@D7
of1PH0Oog]kN1_m4A4@2ooooo`Oo9RHV1olQ8B47ob<S8aOooooo000?ooooo`Oo410@1oooool7obHV
9PKo7QhN0_ooool7okVi^@OoWYjN1ol30`<6oa@D50?ooooo1ol51@D7o`T92@Oo6ATI1OlW9bL3oooo
o`Oo1`L71olV9RH7obDU9@KoD51@0oooool7oa<C4`Ko1`L71olF5QH6oalO7`?ooooo1omNGUh7o`41
0@Oo7alO1_m4A4@2ooooo`Oo5ADE1onUYJD7of1PH0KoFUYJ0oooool7o`D51@Oo>c/k1om2@T88oooo
o`OoHV9R1oooool7ol[:bPKo6QXJ0oooool7o``<30Oooooo1olj>SX6ocPh>0;ooooo1olF5QH7oc<c
<`Oo9BDU1_li>CT3ooooo`Oo20P81omIFET<ob<S8`[ooooo1olS8b<7oalO7`Ko3@d=0oooool6oc@d
=0OoLG5a1ol:2PX6oaDE5@?ooooo1ol51@D7of1PH0Oog]kN1_m4A4@2ooooo`Oo9RHV1olQ8B47ob<S
8aOooooo000?ooooo`Oo410@1oooool7obHV9PKo7QhN0_ooool7okVi^@OoWYjN1ol30`<6oa@D50?o
oooo1ol51@D7o`T92@Oo6ATI1OlW9bL3ooooo`Oo1`L71olV9RH7obDU9@KoD51@0oooool7oa<C4`Ko
1`L71olF5QH6oalO7`?ooooo1omNGUh7o`410@Oo7alO1_m4A4@2ooooo`Oo5ADE1onUYJD7of1PH0Ko
FUYJ0oooool7o`D51@Oo>c/k1om2@T88ooooo`OoHV9R1oooool7ol[:bPKo6QXJ0oooool7o``<30Oo
oooo1olj>SX6ocPh>0;ooooo1olF5QH7oc<c<`Oo9BDU1_li>CT3ooooo`Oo20P81omIFET<ob<S8`[o
oooo1olS8b<7oalO7`Ko3@d=0oooool6oc@d=0OoLG5a1ol:2PX6oaDE5@?ooooo1ol51@D7of1PH0Oo
g]kN1_m4A4@2ooooo`Oo9RHV1olQ8B47ob<S8aOooooo003oooooooGooooo003oooooooGooooo0000
\
\>"],
  ImageRangeCache->{{{0, 499}, {59, 0}} -> {-0.525373, -0.04801, 0.0341698, \
0.0341698}, {{15.375, 42.125}, {57.5625, 30.8125}} -> {-2.29911, -0.214975, \
0.149534, 0.149534}, {{44.75, 71.5625}, {57.5625, 30.8125}} -> {-6.69635, \
-0.214975, 0.149534, 0.149534}, {{74.1875, 100.938}, {57.5625, 30.8125}} -> \
{-11.0936, -0.214975, 0.149534, 0.149534}, {{103.625, 130.375}, {57.5625, \
30.8125}} -> {-15.4955, -0.214975, 0.149534, 0.149534}, {{133.062, 159.812}, \
{57.5625, 30.8125}} -> {-19.8974, -0.214975, 0.149534, 0.149534}, {{162.5, \
189.25}, {57.5625, 30.8125}} -> {-24.2993, -0.214975, 0.149534, 0.149534}, \
{{191.938, 218.688}, {57.5625, 30.8125}} -> {-28.7012, -0.214975, 0.149534, \
0.149534}, {{221.375, 248.125}, {57.5625, 30.8125}} -> {-33.1032, -0.214975, \
0.149534, 0.149534}, {{250.812, 277.562}, {57.5625, 30.8125}} -> {-37.5051, \
-0.214975, 0.149534, 0.149534}, {{280.25, 307}, {57.5625, 30.8125}} -> \
{-41.907, -0.214975, 0.149534, 0.149534}, {{309.688, 336.438}, {57.5625, \
30.8125}} -> {-46.3089, -0.214975, 0.149534, 0.149534}, {{339.125, 365.875}, \
{57.5625, 30.8125}} -> {-50.7108, -0.214975, 0.149534, 0.149534}, {{368.562, \
395.312}, {57.5625, 30.8125}} -> {-55.1127, -0.214975, 0.149534, 0.149534}, \
{{398, 424.75}, {57.5625, 30.8125}} -> {-59.5146, -0.214975, 0.149534, \
0.149534}, {{427.375, 454.188}, {57.5625, 30.8125}} -> {-63.9119, -0.214975, \
0.149534, 0.149534}, {{456.812, 483.625}, {57.5625, 30.8125}} -> {-68.3138, \
-0.214975, 0.149534, 0.149534}, {{15.375, 42.125}, {28.125, 1.375}} -> \
{-2.29911, -4.61689, 0.149534, 0.149534}, {{44.75, 71.5625}, {28.125, 1.375}} -> \
{-6.69635, -4.61689, 0.149534, 0.149534}, {{74.1875, 100.938}, {28.125, \
1.375}} -> {-11.0936, -4.61689, 0.149534, 0.149534}, {{103.625, 130.375}, \
{28.125, 1.375}} -> {-15.4955, -4.61689, 0.149534, 0.149534}, {{133.062, \
159.812}, {28.125, 1.375}} -> {-19.8974, -4.61689, 0.149534, 0.149534}, \
{{162.5, 189.25}, {28.125, 1.375}} -> {-24.2993, -4.61689, 0.149534, \
0.149534}, {{191.938, 218.688}, {28.125, 1.375}} -> {-28.7012, -4.61689, \
0.149534, 0.149534}, {{221.375, 248.125}, {28.125, 1.375}} -> {-33.1032, \
-4.61689, 0.149534, 0.149534}, {{250.812, 277.562}, {28.125, 1.375}} -> \
{-37.5051, -4.61689, 0.149534, 0.149534}, {{280.25, 307}, {28.125, 1.375}} -> \
{-41.907, -4.61689, 0.149534, 0.149534}, {{309.688, 336.438}, {28.125, \
1.375}} -> {-46.3089, -4.61689, 0.149534, 0.149534}, {{339.125, 365.875}, \
{28.125, 1.375}} -> {-50.7108, -4.61689, 0.149534, 0.149534}, {{368.562, \
395.312}, {28.125, 1.375}} -> {-55.1127, -4.61689, 0.149534, 0.149534}, \
{{398, 424.75}, {28.125, 1.375}} -> {-59.5146, -4.61689, 0.149534, 0.149534}, \
{{427.375, 454.188}, {28.125, 1.375}} -> {-63.9119, -4.61689, 0.149534, \
0.149534}, {{456.812, 483.625}, {28.125, 1.375}} -> {-68.3138, -4.61689, \
0.149534, 0.149534}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Visualize histograms", "Subsection"],

Cell["\<\
The graphics below illustrate the histogram over the VQ codebooks \
by band (HL1, LH1, HH1, HL2, LH2, HH2) down the columns.\
\>", "Text"],

Cell[BoxData[
    \(\(colors = {Red, Green, Blue, Orange};\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(BarChart[Sequence\  @@ \ #, BarStyle \[Rule] colors, 
            BarEdgeStyle \[Rule] colors, Frame \[Rule] True, 
            ImageSize \[Rule] 500] &\)\  /@ \ hist;\)\)], "Input"],

Cell[CellGroupData[{

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
%%ImageSize: 500 309.017 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0217391 0.0310559 0.0147151 2.33556 [
[.01863 -0.0125 -3 -9 ]
[.01863 -0.0125 3 0 ]
[.04969 -0.0125 -3 -9 ]
[.04969 -0.0125 3 0 ]
[.08075 -0.0125 -3 -9 ]
[.08075 -0.0125 3 0 ]
[.1118 -0.0125 -3 -9 ]
[.1118 -0.0125 3 0 ]
[.14286 -0.0125 -3 -9 ]
[.14286 -0.0125 3 0 ]
[.17391 -0.0125 -3 -9 ]
[.17391 -0.0125 3 0 ]
[.20497 -0.0125 -3 -9 ]
[.20497 -0.0125 3 0 ]
[.23602 -0.0125 -3 -9 ]
[.23602 -0.0125 3 0 ]
[.26708 -0.0125 -3 -9 ]
[.26708 -0.0125 3 0 ]
[.29814 -0.0125 -6 -9 ]
[.29814 -0.0125 6 0 ]
[.32919 -0.0125 -6 -9 ]
[.32919 -0.0125 6 0 ]
[.36025 -0.0125 -6 -9 ]
[.36025 -0.0125 6 0 ]
[.3913 -0.0125 -6 -9 ]
[.3913 -0.0125 6 0 ]
[.42236 -0.0125 -6 -9 ]
[.42236 -0.0125 6 0 ]
[.45342 -0.0125 -6 -9 ]
[.45342 -0.0125 6 0 ]
[.48447 -0.0125 -6 -9 ]
[.48447 -0.0125 6 0 ]
[.51553 -0.0125 -6 -9 ]
[.51553 -0.0125 6 0 ]
[.54658 -0.0125 -6 -9 ]
[.54658 -0.0125 6 0 ]
[.57764 -0.0125 -6 -9 ]
[.57764 -0.0125 6 0 ]
[.6087 -0.0125 -6 -9 ]
[.6087 -0.0125 6 0 ]
[.63975 -0.0125 -6 -9 ]
[.63975 -0.0125 6 0 ]
[.67081 -0.0125 -6 -9 ]
[.67081 -0.0125 6 0 ]
[.70186 -0.0125 -6 -9 ]
[.70186 -0.0125 6 0 ]
[.73292 -0.0125 -6 -9 ]
[.73292 -0.0125 6 0 ]
[.76398 -0.0125 -6 -9 ]
[.76398 -0.0125 6 0 ]
[.79503 -0.0125 -6 -9 ]
[.79503 -0.0125 6 0 ]
[.82609 -0.0125 -6 -9 ]
[.82609 -0.0125 6 0 ]
[.85714 -0.0125 -6 -9 ]
[.85714 -0.0125 6 0 ]
[.8882 -0.0125 -6 -9 ]
[.8882 -0.0125 6 0 ]
[.91925 -0.0125 -6 -9 ]
[.91925 -0.0125 6 0 ]
[.95031 -0.0125 -6 -9 ]
[.95031 -0.0125 6 0 ]
[.98137 -0.0125 -6 -9 ]
[.98137 -0.0125 6 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .01472 -6 -4.5 ]
[-0.0125 .01472 0 4.5 ]
[-0.0125 .13149 -24 -4.5 ]
[-0.0125 .13149 0 4.5 ]
[-0.0125 .24827 -18 -4.5 ]
[-0.0125 .24827 0 4.5 ]
[-0.0125 .36505 -24 -4.5 ]
[-0.0125 .36505 0 4.5 ]
[-0.0125 .48183 -18 -4.5 ]
[-0.0125 .48183 0 4.5 ]
[-0.0125 .59861 -24 -4.5 ]
[-0.0125 .59861 0 4.5 ]
[ 0 0 -0.125 0 ]
[.01863 .63053 -3 0 ]
[.01863 .63053 3 9 ]
[.04969 .63053 -3 0 ]
[.04969 .63053 3 9 ]
[.08075 .63053 -3 0 ]
[.08075 .63053 3 9 ]
[.1118 .63053 -3 0 ]
[.1118 .63053 3 9 ]
[.14286 .63053 -3 0 ]
[.14286 .63053 3 9 ]
[.17391 .63053 -3 0 ]
[.17391 .63053 3 9 ]
[.20497 .63053 -3 0 ]
[.20497 .63053 3 9 ]
[.23602 .63053 -3 0 ]
[.23602 .63053 3 9 ]
[.26708 .63053 -3 0 ]
[.26708 .63053 3 9 ]
[.29814 .63053 -6 0 ]
[.29814 .63053 6 9 ]
[.32919 .63053 -6 0 ]
[.32919 .63053 6 9 ]
[.36025 .63053 -6 0 ]
[.36025 .63053 6 9 ]
[.3913 .63053 -6 0 ]
[.3913 .63053 6 9 ]
[.42236 .63053 -6 0 ]
[.42236 .63053 6 9 ]
[.45342 .63053 -6 0 ]
[.45342 .63053 6 9 ]
[.48447 .63053 -6 0 ]
[.48447 .63053 6 9 ]
[.51553 .63053 -6 0 ]
[.51553 .63053 6 9 ]
[.54658 .63053 -6 0 ]
[.54658 .63053 6 9 ]
[.57764 .63053 -6 0 ]
[.57764 .63053 6 9 ]
[.6087 .63053 -6 0 ]
[.6087 .63053 6 9 ]
[.63975 .63053 -6 0 ]
[.63975 .63053 6 9 ]
[.67081 .63053 -6 0 ]
[.67081 .63053 6 9 ]
[.70186 .63053 -6 0 ]
[.70186 .63053 6 9 ]
[.73292 .63053 -6 0 ]
[.73292 .63053 6 9 ]
[.76398 .63053 -6 0 ]
[.76398 .63053 6 9 ]
[.79503 .63053 -6 0 ]
[.79503 .63053 6 9 ]
[.82609 .63053 -6 0 ]
[.82609 .63053 6 9 ]
[.85714 .63053 -6 0 ]
[.85714 .63053 6 9 ]
[.8882 .63053 -6 0 ]
[.8882 .63053 6 9 ]
[.91925 .63053 -6 0 ]
[.91925 .63053 6 9 ]
[.95031 .63053 -6 0 ]
[.95031 .63053 6 9 ]
[.98137 .63053 -6 0 ]
[.98137 .63053 6 9 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.01863 0 m
.01863 .00625 L
s
[(1)] .01863 -0.0125 0 1 Mshowa
.04969 0 m
.04969 .00625 L
s
[(2)] .04969 -0.0125 0 1 Mshowa
.08075 0 m
.08075 .00625 L
s
[(3)] .08075 -0.0125 0 1 Mshowa
.1118 0 m
.1118 .00625 L
s
[(4)] .1118 -0.0125 0 1 Mshowa
.14286 0 m
.14286 .00625 L
s
[(5)] .14286 -0.0125 0 1 Mshowa
.17391 0 m
.17391 .00625 L
s
[(6)] .17391 -0.0125 0 1 Mshowa
.20497 0 m
.20497 .00625 L
s
[(7)] .20497 -0.0125 0 1 Mshowa
.23602 0 m
.23602 .00625 L
s
[(8)] .23602 -0.0125 0 1 Mshowa
.26708 0 m
.26708 .00625 L
s
[(9)] .26708 -0.0125 0 1 Mshowa
.29814 0 m
.29814 .00625 L
s
[(10)] .29814 -0.0125 0 1 Mshowa
.32919 0 m
.32919 .00625 L
s
[(11)] .32919 -0.0125 0 1 Mshowa
.36025 0 m
.36025 .00625 L
s
[(12)] .36025 -0.0125 0 1 Mshowa
.3913 0 m
.3913 .00625 L
s
[(13)] .3913 -0.0125 0 1 Mshowa
.42236 0 m
.42236 .00625 L
s
[(14)] .42236 -0.0125 0 1 Mshowa
.45342 0 m
.45342 .00625 L
s
[(15)] .45342 -0.0125 0 1 Mshowa
.48447 0 m
.48447 .00625 L
s
[(16)] .48447 -0.0125 0 1 Mshowa
.51553 0 m
.51553 .00625 L
s
[(17)] .51553 -0.0125 0 1 Mshowa
.54658 0 m
.54658 .00625 L
s
[(18)] .54658 -0.0125 0 1 Mshowa
.57764 0 m
.57764 .00625 L
s
[(19)] .57764 -0.0125 0 1 Mshowa
.6087 0 m
.6087 .00625 L
s
[(20)] .6087 -0.0125 0 1 Mshowa
.63975 0 m
.63975 .00625 L
s
[(21)] .63975 -0.0125 0 1 Mshowa
.67081 0 m
.67081 .00625 L
s
[(22)] .67081 -0.0125 0 1 Mshowa
.70186 0 m
.70186 .00625 L
s
[(23)] .70186 -0.0125 0 1 Mshowa
.73292 0 m
.73292 .00625 L
s
[(24)] .73292 -0.0125 0 1 Mshowa
.76398 0 m
.76398 .00625 L
s
[(25)] .76398 -0.0125 0 1 Mshowa
.79503 0 m
.79503 .00625 L
s
[(26)] .79503 -0.0125 0 1 Mshowa
.82609 0 m
.82609 .00625 L
s
[(27)] .82609 -0.0125 0 1 Mshowa
.85714 0 m
.85714 .00625 L
s
[(28)] .85714 -0.0125 0 1 Mshowa
.8882 0 m
.8882 .00625 L
s
[(29)] .8882 -0.0125 0 1 Mshowa
.91925 0 m
.91925 .00625 L
s
[(30)] .91925 -0.0125 0 1 Mshowa
.95031 0 m
.95031 .00625 L
s
[(31)] .95031 -0.0125 0 1 Mshowa
.98137 0 m
.98137 .00625 L
s
[(32)] .98137 -0.0125 0 1 Mshowa
0 0 m
1 0 L
s
0 .01472 m
.00625 .01472 L
s
[(0)] -0.0125 .01472 1 0 Mshowa
0 .13149 m
.00625 .13149 L
s
[(0.05)] -0.0125 .13149 1 0 Mshowa
0 .24827 m
.00625 .24827 L
s
[(0.1)] -0.0125 .24827 1 0 Mshowa
0 .36505 m
.00625 .36505 L
s
[(0.15)] -0.0125 .36505 1 0 Mshowa
0 .48183 m
.00625 .48183 L
s
[(0.2)] -0.0125 .48183 1 0 Mshowa
0 .59861 m
.00625 .59861 L
s
[(0.25)] -0.0125 .59861 1 0 Mshowa
.125 Mabswid
0 .03807 m
.00375 .03807 L
s
0 .06143 m
.00375 .06143 L
s
0 .08478 m
.00375 .08478 L
s
0 .10814 m
.00375 .10814 L
s
0 .15485 m
.00375 .15485 L
s
0 .1782 m
.00375 .1782 L
s
0 .20156 m
.00375 .20156 L
s
0 .22492 m
.00375 .22492 L
s
0 .27163 m
.00375 .27163 L
s
0 .29498 m
.00375 .29498 L
s
0 .31834 m
.00375 .31834 L
s
0 .34169 m
.00375 .34169 L
s
0 .3884 m
.00375 .3884 L
s
0 .41176 m
.00375 .41176 L
s
0 .43512 m
.00375 .43512 L
s
0 .45847 m
.00375 .45847 L
s
0 .50518 m
.00375 .50518 L
s
0 .52854 m
.00375 .52854 L
s
0 .55189 m
.00375 .55189 L
s
0 .57525 m
.00375 .57525 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.01863 .61178 m
.01863 .61803 L
s
[(1)] .01863 .63053 0 -1 Mshowa
.04969 .61178 m
.04969 .61803 L
s
[(2)] .04969 .63053 0 -1 Mshowa
.08075 .61178 m
.08075 .61803 L
s
[(3)] .08075 .63053 0 -1 Mshowa
.1118 .61178 m
.1118 .61803 L
s
[(4)] .1118 .63053 0 -1 Mshowa
.14286 .61178 m
.14286 .61803 L
s
[(5)] .14286 .63053 0 -1 Mshowa
.17391 .61178 m
.17391 .61803 L
s
[(6)] .17391 .63053 0 -1 Mshowa
.20497 .61178 m
.20497 .61803 L
s
[(7)] .20497 .63053 0 -1 Mshowa
.23602 .61178 m
.23602 .61803 L
s
[(8)] .23602 .63053 0 -1 Mshowa
.26708 .61178 m
.26708 .61803 L
s
[(9)] .26708 .63053 0 -1 Mshowa
.29814 .61178 m
.29814 .61803 L
s
[(10)] .29814 .63053 0 -1 Mshowa
.32919 .61178 m
.32919 .61803 L
s
[(11)] .32919 .63053 0 -1 Mshowa
.36025 .61178 m
.36025 .61803 L
s
[(12)] .36025 .63053 0 -1 Mshowa
.3913 .61178 m
.3913 .61803 L
s
[(13)] .3913 .63053 0 -1 Mshowa
.42236 .61178 m
.42236 .61803 L
s
[(14)] .42236 .63053 0 -1 Mshowa
.45342 .61178 m
.45342 .61803 L
s
[(15)] .45342 .63053 0 -1 Mshowa
.48447 .61178 m
.48447 .61803 L
s
[(16)] .48447 .63053 0 -1 Mshowa
.51553 .61178 m
.51553 .61803 L
s
[(17)] .51553 .63053 0 -1 Mshowa
.54658 .61178 m
.54658 .61803 L
s
[(18)] .54658 .63053 0 -1 Mshowa
.57764 .61178 m
.57764 .61803 L
s
[(19)] .57764 .63053 0 -1 Mshowa
.6087 .61178 m
.6087 .61803 L
s
[(20)] .6087 .63053 0 -1 Mshowa
.63975 .61178 m
.63975 .61803 L
s
[(21)] .63975 .63053 0 -1 Mshowa
.67081 .61178 m
.67081 .61803 L
s
[(22)] .67081 .63053 0 -1 Mshowa
.70186 .61178 m
.70186 .61803 L
s
[(23)] .70186 .63053 0 -1 Mshowa
.73292 .61178 m
.73292 .61803 L
s
[(24)] .73292 .63053 0 -1 Mshowa
.76398 .61178 m
.76398 .61803 L
s
[(25)] .76398 .63053 0 -1 Mshowa
.79503 .61178 m
.79503 .61803 L
s
[(26)] .79503 .63053 0 -1 Mshowa
.82609 .61178 m
.82609 .61803 L
s
[(27)] .82609 .63053 0 -1 Mshowa
.85714 .61178 m
.85714 .61803 L
s
[(28)] .85714 .63053 0 -1 Mshowa
.8882 .61178 m
.8882 .61803 L
s
[(29)] .8882 .63053 0 -1 Mshowa
.91925 .61178 m
.91925 .61803 L
s
[(30)] .91925 .63053 0 -1 Mshowa
.95031 .61178 m
.95031 .61803 L
s
[(31)] .95031 .63053 0 -1 Mshowa
.98137 .61178 m
.98137 .61803 L
s
[(32)] .98137 .63053 0 -1 Mshowa
0 .61803 m
1 .61803 L
s
.99375 .01472 m
1 .01472 L
s
.99375 .13149 m
1 .13149 L
s
.99375 .24827 m
1 .24827 L
s
.99375 .36505 m
1 .36505 L
s
.99375 .48183 m
1 .48183 L
s
.99375 .59861 m
1 .59861 L
s
.125 Mabswid
.99625 .03807 m
1 .03807 L
s
.99625 .06143 m
1 .06143 L
s
.99625 .08478 m
1 .08478 L
s
.99625 .10814 m
1 .10814 L
s
.99625 .15485 m
1 .15485 L
s
.99625 .1782 m
1 .1782 L
s
.99625 .20156 m
1 .20156 L
s
.99625 .22492 m
1 .22492 L
s
.99625 .27163 m
1 .27163 L
s
.99625 .29498 m
1 .29498 L
s
.99625 .31834 m
1 .31834 L
s
.99625 .34169 m
1 .34169 L
s
.99625 .3884 m
1 .3884 L
s
.99625 .41176 m
1 .41176 L
s
.99625 .43512 m
1 .43512 L
s
.99625 .45847 m
1 .45847 L
s
.99625 .50518 m
1 .50518 L
s
.99625 .52854 m
1 .52854 L
s
.99625 .55189 m
1 .55189 L
s
.99625 .57525 m
1 .57525 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .01472 m
1 .01472 L
s
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.00621 .01472 m
.00621 .26475 L
.01242 .26475 L
.01242 .01472 L
F
.5 Mabswid
.00621 .01472 m
.00621 .26475 L
.01242 .26475 L
.01242 .01472 L
.00621 .01472 L
s
.03727 .01472 m
.03727 .02373 L
.04348 .02373 L
.04348 .01472 L
F
.03727 .01472 m
.03727 .02373 L
.04348 .02373 L
.04348 .01472 L
.03727 .01472 L
s
.06832 .01472 m
.06832 .01635 L
.07453 .01635 L
.07453 .01472 L
F
.06832 .01472 m
.06832 .01635 L
.07453 .01635 L
.07453 .01472 L
.06832 .01472 L
s
.09938 .01472 m
.09938 .04341 L
.10559 .04341 L
.10559 .01472 L
F
.09938 .01472 m
.09938 .04341 L
.10559 .04341 L
.10559 .01472 L
.09938 .01472 L
s
.13043 .01472 m
.13043 .25573 L
.13665 .25573 L
.13665 .01472 L
F
.13043 .01472 m
.13043 .25573 L
.13665 .25573 L
.13665 .01472 L
.13043 .01472 L
s
.16149 .01472 m
.16149 .04341 L
.1677 .04341 L
.1677 .01472 L
F
.16149 .01472 m
.16149 .04341 L
.1677 .04341 L
.1677 .01472 L
.16149 .01472 L
s
.19255 .01472 m
.19255 .01472 L
.19876 .01472 L
.19876 .01472 L
F
.19255 .01472 m
.19255 .01472 L
.19876 .01472 L
.19876 .01472 L
.19255 .01472 L
s
.2236 .01472 m
.2236 .04341 L
.22981 .04341 L
.22981 .01472 L
F
.2236 .01472 m
.2236 .04341 L
.22981 .04341 L
.22981 .01472 L
.2236 .01472 L
s
.25466 .01472 m
.25466 .03193 L
.26087 .03193 L
.26087 .01472 L
F
.25466 .01472 m
.25466 .03193 L
.26087 .03193 L
.26087 .01472 L
.25466 .01472 L
s
.28571 .01472 m
.28571 .28196 L
.29193 .28196 L
.29193 .01472 L
F
.28571 .01472 m
.28571 .28196 L
.29193 .28196 L
.29193 .01472 L
.28571 .01472 L
s
.31677 .01472 m
.31677 .01717 L
.32298 .01717 L
.32298 .01472 L
F
.31677 .01472 m
.31677 .01717 L
.32298 .01717 L
.32298 .01472 L
.31677 .01472 L
s
.34783 .01472 m
.34783 .01472 L
.35404 .01472 L
.35404 .01472 L
F
.34783 .01472 m
.34783 .01472 L
.35404 .01472 L
.35404 .01472 L
.34783 .01472 L
s
.37888 .01472 m
.37888 .01472 L
.38509 .01472 L
.38509 .01472 L
F
.37888 .01472 m
.37888 .01472 L
.38509 .01472 L
.38509 .01472 L
.37888 .01472 L
s
.40994 .01472 m
.40994 .2172 L
.41615 .2172 L
.41615 .01472 L
F
.40994 .01472 m
.40994 .2172 L
.41615 .2172 L
.41615 .01472 L
.40994 .01472 L
s
.44099 .01472 m
.44099 .03193 L
.4472 .03193 L
.4472 .01472 L
F
.44099 .01472 m
.44099 .03193 L
.4472 .03193 L
.4472 .01472 L
.44099 .01472 L
s
.47205 .01472 m
.47205 .01881 L
.47826 .01881 L
.47826 .01472 L
F
.47205 .01472 m
.47205 .01881 L
.47826 .01881 L
.47826 .01472 L
.47205 .01472 L
s
.50311 .01472 m
.50311 .29672 L
.50932 .29672 L
.50932 .01472 L
F
.50311 .01472 m
.50311 .29672 L
.50932 .29672 L
.50932 .01472 L
.50311 .01472 L
s
.53416 .01472 m
.53416 .07948 L
.54037 .07948 L
.54037 .01472 L
F
.53416 .01472 m
.53416 .07948 L
.54037 .07948 L
.54037 .01472 L
.53416 .01472 L
s
.56522 .01472 m
.56522 .02455 L
.57143 .02455 L
.57143 .01472 L
F
.56522 .01472 m
.56522 .02455 L
.57143 .02455 L
.57143 .01472 L
.56522 .01472 L
s
.59627 .01472 m
.59627 .01472 L
.60248 .01472 L
.60248 .01472 L
F
.59627 .01472 m
.59627 .01472 L
.60248 .01472 L
.60248 .01472 L
.59627 .01472 L
s
.62733 .01472 m
.62733 .05243 L
.63354 .05243 L
.63354 .01472 L
F
.62733 .01472 m
.62733 .05243 L
.63354 .05243 L
.63354 .01472 L
.62733 .01472 L
s
.65839 .01472 m
.65839 .29426 L
.6646 .29426 L
.6646 .01472 L
F
.65839 .01472 m
.65839 .29426 L
.6646 .29426 L
.6646 .01472 L
.65839 .01472 L
s
.68944 .01472 m
.68944 .02127 L
.69565 .02127 L
.69565 .01472 L
F
.68944 .01472 m
.68944 .02127 L
.69565 .02127 L
.69565 .01472 L
.68944 .01472 L
s
.7205 .01472 m
.7205 .08604 L
.72671 .08604 L
.72671 .01472 L
F
.7205 .01472 m
.7205 .08604 L
.72671 .08604 L
.72671 .01472 L
.7205 .01472 L
s
.75155 .01472 m
.75155 .06964 L
.75776 .06964 L
.75776 .01472 L
F
.75155 .01472 m
.75155 .06964 L
.75776 .06964 L
.75776 .01472 L
.75155 .01472 L
s
.78261 .01472 m
.78261 .01881 L
.78882 .01881 L
.78882 .01472 L
F
.78261 .01472 m
.78261 .01881 L
.78882 .01881 L
.78882 .01472 L
.78261 .01472 L
s
.81366 .01472 m
.81366 .01472 L
.81988 .01472 L
.81988 .01472 L
F
.81366 .01472 m
.81366 .01472 L
.81988 .01472 L
.81988 .01472 L
.81366 .01472 L
s
.84472 .01472 m
.84472 .01472 L
.85093 .01472 L
.85093 .01472 L
F
.84472 .01472 m
.84472 .01472 L
.85093 .01472 L
.85093 .01472 L
.84472 .01472 L
s
.87578 .01472 m
.87578 .03685 L
.88199 .03685 L
.88199 .01472 L
F
.87578 .01472 m
.87578 .03685 L
.88199 .03685 L
.88199 .01472 L
.87578 .01472 L
s
.90683 .01472 m
.90683 .15408 L
.91304 .15408 L
.91304 .01472 L
F
.90683 .01472 m
.90683 .15408 L
.91304 .15408 L
.91304 .01472 L
.90683 .01472 L
s
.93789 .01472 m
.93789 .03357 L
.9441 .03357 L
.9441 .01472 L
F
.93789 .01472 m
.93789 .03357 L
.9441 .03357 L
.9441 .01472 L
.93789 .01472 L
s
.96894 .01472 m
.96894 .26065 L
.97516 .26065 L
.97516 .01472 L
F
.96894 .01472 m
.96894 .26065 L
.97516 .26065 L
.97516 .01472 L
.96894 .01472 L
s
0 1 0 r
.01242 .01472 m
.01242 .03357 L
.01863 .03357 L
.01863 .01472 L
F
.01242 .01472 m
.01242 .03357 L
.01863 .03357 L
.01863 .01472 L
.01242 .01472 L
s
.04348 .01472 m
.04348 .2254 L
.04969 .2254 L
.04969 .01472 L
F
.04348 .01472 m
.04348 .2254 L
.04969 .2254 L
.04969 .01472 L
.04348 .01472 L
s
.07453 .01472 m
.07453 .20818 L
.08075 .20818 L
.08075 .01472 L
F
.07453 .01472 m
.07453 .20818 L
.08075 .20818 L
.08075 .01472 L
.07453 .01472 L
s
.10559 .01472 m
.10559 .02865 L
.1118 .02865 L
.1118 .01472 L
F
.10559 .01472 m
.10559 .02865 L
.1118 .02865 L
.1118 .01472 L
.10559 .01472 L
s
.13665 .01472 m
.13665 .01799 L
.14286 .01799 L
.14286 .01472 L
F
.13665 .01472 m
.13665 .01799 L
.14286 .01799 L
.14286 .01472 L
.13665 .01472 L
s
.1677 .01472 m
.1677 .10079 L
.17391 .10079 L
.17391 .01472 L
F
.1677 .01472 m
.1677 .10079 L
.17391 .10079 L
.17391 .01472 L
.1677 .01472 L
s
.19876 .01472 m
.19876 .60332 L
.20497 .60332 L
.20497 .01472 L
F
.19876 .01472 m
.19876 .60332 L
.20497 .60332 L
.20497 .01472 L
.19876 .01472 L
s
.22981 .01472 m
.22981 .11965 L
.23602 .11965 L
.23602 .01472 L
F
.22981 .01472 m
.22981 .11965 L
.23602 .11965 L
.23602 .01472 L
.22981 .01472 L
s
.26087 .01472 m
.26087 .05488 L
.26708 .05488 L
.26708 .01472 L
F
.26087 .01472 m
.26087 .05488 L
.26708 .05488 L
.26708 .01472 L
.26087 .01472 L
s
.29193 .01472 m
.29193 .01963 L
.29814 .01963 L
.29814 .01472 L
F
.29193 .01472 m
.29193 .01963 L
.29814 .01963 L
.29814 .01472 L
.29193 .01472 L
s
.32298 .01472 m
.32298 .08112 L
.32919 .08112 L
.32919 .01472 L
F
.32298 .01472 m
.32298 .08112 L
.32919 .08112 L
.32919 .01472 L
.32298 .01472 L
s
.35404 .01472 m
.35404 .01472 L
.36025 .01472 L
.36025 .01472 L
F
.35404 .01472 m
.35404 .01472 L
.36025 .01472 L
.36025 .01472 L
.35404 .01472 L
s
.38509 .01472 m
.38509 .01717 L
.3913 .01717 L
.3913 .01472 L
F
.38509 .01472 m
.38509 .01717 L
.3913 .01717 L
.3913 .01472 L
.38509 .01472 L
s
.41615 .01472 m
.41615 .01472 L
.42236 .01472 L
.42236 .01472 L
F
.41615 .01472 m
.41615 .01472 L
.42236 .01472 L
.42236 .01472 L
.41615 .01472 L
s
.4472 .01472 m
.4472 .23278 L
.45342 .23278 L
.45342 .01472 L
F
.4472 .01472 m
.4472 .23278 L
.45342 .23278 L
.45342 .01472 L
.4472 .01472 L
s
.47826 .01472 m
.47826 .03111 L
.48447 .03111 L
.48447 .01472 L
F
.47826 .01472 m
.47826 .03111 L
.48447 .03111 L
.48447 .01472 L
.47826 .01472 L
s
.50932 .01472 m
.50932 .03193 L
.51553 .03193 L
.51553 .01472 L
F
.50932 .01472 m
.50932 .03193 L
.51553 .03193 L
.51553 .01472 L
.50932 .01472 L
s
.54037 .01472 m
.54037 .05079 L
.54658 .05079 L
.54658 .01472 L
F
.54037 .01472 m
.54037 .05079 L
.54658 .05079 L
.54658 .01472 L
.54037 .01472 L
s
.57143 .01472 m
.57143 .06554 L
.57764 .06554 L
.57764 .01472 L
F
.57143 .01472 m
.57143 .06554 L
.57764 .06554 L
.57764 .01472 L
.57143 .01472 L
s
.60248 .01472 m
.60248 .02619 L
.6087 .02619 L
.6087 .01472 L
F
.60248 .01472 m
.60248 .02619 L
.6087 .02619 L
.6087 .01472 L
.60248 .01472 L
s
.63354 .01472 m
.63354 .04997 L
.63975 .04997 L
.63975 .01472 L
F
.63354 .01472 m
.63354 .04997 L
.63975 .04997 L
.63975 .01472 L
.63354 .01472 L
s
.6646 .01472 m
.6646 .01717 L
.67081 .01717 L
.67081 .01472 L
F
.6646 .01472 m
.6646 .01717 L
.67081 .01717 L
.67081 .01472 L
.6646 .01472 L
s
.69565 .01472 m
.69565 .22212 L
.70186 .22212 L
.70186 .01472 L
F
.69565 .01472 m
.69565 .22212 L
.70186 .22212 L
.70186 .01472 L
.69565 .01472 L
s
.72671 .01472 m
.72671 .03275 L
.73292 .03275 L
.73292 .01472 L
F
.72671 .01472 m
.72671 .03275 L
.73292 .03275 L
.73292 .01472 L
.72671 .01472 L
s
.75776 .01472 m
.75776 .03521 L
.76398 .03521 L
.76398 .01472 L
F
.75776 .01472 m
.75776 .03521 L
.76398 .03521 L
.76398 .01472 L
.75776 .01472 L
s
.78882 .01472 m
.78882 .01553 L
.79503 .01553 L
.79503 .01472 L
F
.78882 .01472 m
.78882 .01553 L
.79503 .01553 L
.79503 .01472 L
.78882 .01472 L
s
.81988 .01472 m
.81988 .07948 L
.82609 .07948 L
.82609 .01472 L
F
.81988 .01472 m
.81988 .07948 L
.82609 .07948 L
.82609 .01472 L
.81988 .01472 L
s
.85093 .01472 m
.85093 .01881 L
.85714 .01881 L
.85714 .01472 L
F
.85093 .01472 m
.85093 .01881 L
.85714 .01881 L
.85714 .01472 L
.85093 .01472 L
s
.88199 .01472 m
.88199 .08686 L
.8882 .08686 L
.8882 .01472 L
F
.88199 .01472 m
.88199 .08686 L
.8882 .08686 L
.8882 .01472 L
.88199 .01472 L
s
.91304 .01472 m
.91304 .03931 L
.91925 .03931 L
.91925 .01472 L
F
.91304 .01472 m
.91304 .03931 L
.91925 .03931 L
.91925 .01472 L
.91304 .01472 L
s
.9441 .01472 m
.9441 .21064 L
.95031 .21064 L
.95031 .01472 L
F
.9441 .01472 m
.9441 .21064 L
.95031 .21064 L
.95031 .01472 L
.9441 .01472 L
s
.97516 .01472 m
.97516 .02045 L
.98137 .02045 L
.98137 .01472 L
F
.97516 .01472 m
.97516 .02045 L
.98137 .02045 L
.98137 .01472 L
.97516 .01472 L
s
0 0 1 r
.01863 .01472 m
.01863 .07456 L
.02484 .07456 L
.02484 .01472 L
F
.01863 .01472 m
.01863 .07456 L
.02484 .07456 L
.02484 .01472 L
.01863 .01472 L
s
.04969 .01472 m
.04969 .05324 L
.0559 .05324 L
.0559 .01472 L
F
.04969 .01472 m
.04969 .05324 L
.0559 .05324 L
.0559 .01472 L
.04969 .01472 L
s
.08075 .01472 m
.08075 .0803 L
.08696 .0803 L
.08696 .01472 L
F
.08075 .01472 m
.08075 .0803 L
.08696 .0803 L
.08696 .01472 L
.08075 .01472 L
s
.1118 .01472 m
.1118 .14342 L
.11801 .14342 L
.11801 .01472 L
F
.1118 .01472 m
.1118 .14342 L
.11801 .14342 L
.11801 .01472 L
.1118 .01472 L
s
.14286 .01472 m
.14286 .03603 L
.14907 .03603 L
.14907 .01472 L
F
.14286 .01472 m
.14286 .03603 L
.14907 .03603 L
.14907 .01472 L
.14286 .01472 L
s
.17391 .01472 m
.17391 .13112 L
.18012 .13112 L
.18012 .01472 L
F
.17391 .01472 m
.17391 .13112 L
.18012 .13112 L
.18012 .01472 L
.17391 .01472 L
s
.20497 .01472 m
.20497 .01881 L
.21118 .01881 L
.21118 .01472 L
F
.20497 .01472 m
.20497 .01881 L
.21118 .01881 L
.21118 .01472 L
.20497 .01472 L
s
.23602 .01472 m
.23602 .11063 L
.24224 .11063 L
.24224 .01472 L
F
.23602 .01472 m
.23602 .11063 L
.24224 .11063 L
.24224 .01472 L
.23602 .01472 L
s
.26708 .01472 m
.26708 .17293 L
.27329 .17293 L
.27329 .01472 L
F
.26708 .01472 m
.26708 .17293 L
.27329 .17293 L
.27329 .01472 L
.26708 .01472 L
s
.29814 .01472 m
.29814 .03029 L
.30435 .03029 L
.30435 .01472 L
F
.29814 .01472 m
.29814 .03029 L
.30435 .03029 L
.30435 .01472 L
.29814 .01472 L
s
.32919 .01472 m
.32919 .05161 L
.3354 .05161 L
.3354 .01472 L
F
.32919 .01472 m
.32919 .05161 L
.3354 .05161 L
.3354 .01472 L
.32919 .01472 L
s
.36025 .01472 m
.36025 .10243 L
.36646 .10243 L
.36646 .01472 L
F
.36025 .01472 m
.36025 .10243 L
.36646 .10243 L
.36646 .01472 L
.36025 .01472 L
s
.3913 .01472 m
.3913 .12785 L
.39752 .12785 L
.39752 .01472 L
F
.3913 .01472 m
.3913 .12785 L
.39752 .12785 L
.39752 .01472 L
.3913 .01472 L
s
.42236 .01472 m
.42236 .02209 L
.42857 .02209 L
.42857 .01472 L
F
.42236 .01472 m
.42236 .02209 L
.42857 .02209 L
.42857 .01472 L
.42236 .01472 L
s
.45342 .01472 m
.45342 .04505 L
.45963 .04505 L
.45963 .01472 L
F
.45342 .01472 m
.45342 .04505 L
.45963 .04505 L
.45963 .01472 L
.45342 .01472 L
s
.48447 .01472 m
.48447 .11473 L
.49068 .11473 L
.49068 .01472 L
F
.48447 .01472 m
.48447 .11473 L
.49068 .11473 L
.49068 .01472 L
.48447 .01472 L
s
.51553 .01472 m
.51553 .05816 L
.52174 .05816 L
.52174 .01472 L
F
.51553 .01472 m
.51553 .05816 L
.52174 .05816 L
.52174 .01472 L
.51553 .01472 L
s
.54658 .01472 m
.54658 .12866 L
.5528 .12866 L
.5528 .01472 L
F
.54658 .01472 m
.54658 .12866 L
.5528 .12866 L
.5528 .01472 L
.54658 .01472 L
s
.57764 .01472 m
.57764 .03111 L
.58385 .03111 L
.58385 .01472 L
F
.57764 .01472 m
.57764 .03111 L
.58385 .03111 L
.58385 .01472 L
.57764 .01472 L
s
.6087 .01472 m
.6087 .1426 L
.61491 .1426 L
.61491 .01472 L
F
.6087 .01472 m
.6087 .1426 L
.61491 .1426 L
.61491 .01472 L
.6087 .01472 L
s
.63975 .01472 m
.63975 .17867 L
.64596 .17867 L
.64596 .01472 L
F
.63975 .01472 m
.63975 .17867 L
.64596 .17867 L
.64596 .01472 L
.63975 .01472 L
s
.67081 .01472 m
.67081 .03931 L
.67702 .03931 L
.67702 .01472 L
F
.67081 .01472 m
.67081 .03931 L
.67702 .03931 L
.67702 .01472 L
.67081 .01472 L
s
.70186 .01472 m
.70186 .03521 L
.70807 .03521 L
.70807 .01472 L
F
.70186 .01472 m
.70186 .03521 L
.70807 .03521 L
.70807 .01472 L
.70186 .01472 L
s
.73292 .01472 m
.73292 .11637 L
.73913 .11637 L
.73913 .01472 L
F
.73292 .01472 m
.73292 .11637 L
.73913 .11637 L
.73913 .01472 L
.73292 .01472 L
s
.76398 .01472 m
.76398 .13194 L
.77019 .13194 L
.77019 .01472 L
F
.76398 .01472 m
.76398 .13194 L
.77019 .13194 L
.77019 .01472 L
.76398 .01472 L
s
.79503 .01472 m
.79503 .13768 L
.80124 .13768 L
.80124 .01472 L
F
.79503 .01472 m
.79503 .13768 L
.80124 .13768 L
.80124 .01472 L
.79503 .01472 L
s
.82609 .01472 m
.82609 .04751 L
.8323 .04751 L
.8323 .01472 L
F
.82609 .01472 m
.82609 .04751 L
.8323 .04751 L
.8323 .01472 L
.82609 .01472 L
s
.85714 .01472 m
.85714 .10899 L
.86335 .10899 L
.86335 .01472 L
F
.85714 .01472 m
.85714 .10899 L
.86335 .10899 L
.86335 .01472 L
.85714 .01472 L
s
.8882 .01472 m
.8882 .15326 L
.89441 .15326 L
.89441 .01472 L
F
.8882 .01472 m
.8882 .15326 L
.89441 .15326 L
.89441 .01472 L
.8882 .01472 L
s
.91925 .01472 m
.91925 .09505 L
.92547 .09505 L
.92547 .01472 L
F
.91925 .01472 m
.91925 .09505 L
.92547 .09505 L
.92547 .01472 L
.91925 .01472 L
s
.95031 .01472 m
.95031 .04915 L
.95652 .04915 L
.95652 .01472 L
F
.95031 .01472 m
.95031 .04915 L
.95652 .04915 L
.95652 .01472 L
.95031 .01472 L
s
.98137 .01472 m
.98137 .03767 L
.98758 .03767 L
.98758 .01472 L
F
.98137 .01472 m
.98137 .03767 L
.98758 .03767 L
.98758 .01472 L
.98137 .01472 L
s
1 .5 0 r
.02484 .01472 m
.02484 .02455 L
.03106 .02455 L
.03106 .01472 L
F
.02484 .01472 m
.02484 .02455 L
.03106 .02455 L
.03106 .01472 L
.02484 .01472 L
s
.0559 .01472 m
.0559 .11309 L
.06211 .11309 L
.06211 .01472 L
F
.0559 .01472 m
.0559 .11309 L
.06211 .11309 L
.06211 .01472 L
.0559 .01472 L
s
.08696 .01472 m
.08696 .16064 L
.09317 .16064 L
.09317 .01472 L
F
.08696 .01472 m
.08696 .16064 L
.09317 .16064 L
.09317 .01472 L
.08696 .01472 L
s
.11801 .01472 m
.11801 .01963 L
.12422 .01963 L
.12422 .01472 L
F
.11801 .01472 m
.11801 .01963 L
.12422 .01963 L
.12422 .01472 L
.11801 .01472 L
s
.14907 .01472 m
.14907 .02373 L
.15528 .02373 L
.15528 .01472 L
F
.14907 .01472 m
.14907 .02373 L
.15528 .02373 L
.15528 .01472 L
.14907 .01472 L
s
.18012 .01472 m
.18012 .04587 L
.18634 .04587 L
.18634 .01472 L
F
.18012 .01472 m
.18012 .04587 L
.18634 .04587 L
.18634 .01472 L
.18012 .01472 L
s
.21118 .01472 m
.21118 .16474 L
.21739 .16474 L
.21739 .01472 L
F
.21118 .01472 m
.21118 .16474 L
.21739 .16474 L
.21739 .01472 L
.21118 .01472 L
s
.24224 .01472 m
.24224 .03603 L
.24845 .03603 L
.24845 .01472 L
F
.24224 .01472 m
.24224 .03603 L
.24845 .03603 L
.24845 .01472 L
.24224 .01472 L
s
.27329 .01472 m
.27329 .02209 L
.2795 .02209 L
.2795 .01472 L
F
.27329 .01472 m
.27329 .02209 L
.2795 .02209 L
.2795 .01472 L
.27329 .01472 L
s
.30435 .01472 m
.30435 .02619 L
.31056 .02619 L
.31056 .01472 L
F
.30435 .01472 m
.30435 .02619 L
.31056 .02619 L
.31056 .01472 L
.30435 .01472 L
s
.3354 .01472 m
.3354 .37132 L
.34161 .37132 L
.34161 .01472 L
F
.3354 .01472 m
.3354 .37132 L
.34161 .37132 L
.34161 .01472 L
.3354 .01472 L
s
.36646 .01472 m
.36646 .01472 L
.37267 .01472 L
.37267 .01472 L
F
.36646 .01472 m
.36646 .01472 L
.37267 .01472 L
.37267 .01472 L
.36646 .01472 L
s
.39752 .01472 m
.39752 .01472 L
.40373 .01472 L
.40373 .01472 L
F
.39752 .01472 m
.39752 .01472 L
.40373 .01472 L
.40373 .01472 L
.39752 .01472 L
s
.42857 .01472 m
.42857 .03439 L
.43478 .03439 L
.43478 .01472 L
F
.42857 .01472 m
.42857 .03439 L
.43478 .03439 L
.43478 .01472 L
.42857 .01472 L
s
.45963 .01472 m
.45963 .18277 L
.46584 .18277 L
.46584 .01472 L
F
.45963 .01472 m
.45963 .18277 L
.46584 .18277 L
.46584 .01472 L
.45963 .01472 L
s
.49068 .01472 m
.49068 .01717 L
.49689 .01717 L
.49689 .01472 L
F
.49068 .01472 m
.49068 .01717 L
.49689 .01717 L
.49689 .01472 L
.49068 .01472 L
s
.52174 .01472 m
.52174 .02373 L
.52795 .02373 L
.52795 .01472 L
F
.52174 .01472 m
.52174 .02373 L
.52795 .02373 L
.52795 .01472 L
.52174 .01472 L
s
.5528 .01472 m
.5528 .02455 L
.55901 .02455 L
.55901 .01472 L
F
.5528 .01472 m
.5528 .02455 L
.55901 .02455 L
.55901 .01472 L
.5528 .01472 L
s
.58385 .01472 m
.58385 .36066 L
.59006 .36066 L
.59006 .01472 L
F
.58385 .01472 m
.58385 .36066 L
.59006 .36066 L
.59006 .01472 L
.58385 .01472 L
s
.61491 .01472 m
.61491 .01472 L
.62112 .01472 L
.62112 .01472 L
F
.61491 .01472 m
.61491 .01472 L
.62112 .01472 L
.62112 .01472 L
.61491 .01472 L
s
.64596 .01472 m
.64596 .02455 L
.65217 .02455 L
.65217 .01472 L
F
.64596 .01472 m
.64596 .02455 L
.65217 .02455 L
.65217 .01472 L
.64596 .01472 L
s
.67702 .01472 m
.67702 .04505 L
.68323 .04505 L
.68323 .01472 L
F
.67702 .01472 m
.67702 .04505 L
.68323 .04505 L
.68323 .01472 L
.67702 .01472 L
s
.70807 .01472 m
.70807 .28442 L
.71429 .28442 L
.71429 .01472 L
F
.70807 .01472 m
.70807 .28442 L
.71429 .28442 L
.71429 .01472 L
.70807 .01472 L
s
.73913 .01472 m
.73913 .04833 L
.74534 .04833 L
.74534 .01472 L
F
.73913 .01472 m
.73913 .04833 L
.74534 .04833 L
.74534 .01472 L
.73913 .01472 L
s
.77019 .01472 m
.77019 .02291 L
.7764 .02291 L
.7764 .01472 L
F
.77019 .01472 m
.77019 .02291 L
.7764 .02291 L
.7764 .01472 L
.77019 .01472 L
s
.80124 .01472 m
.80124 .01472 L
.80745 .01472 L
.80745 .01472 L
F
.80124 .01472 m
.80124 .01472 L
.80745 .01472 L
.80745 .01472 L
.80124 .01472 L
s
.8323 .01472 m
.8323 .31148 L
.83851 .31148 L
.83851 .01472 L
F
.8323 .01472 m
.8323 .31148 L
.83851 .31148 L
.83851 .01472 L
.8323 .01472 L
s
.86335 .01472 m
.86335 .01472 L
.86957 .01472 L
.86957 .01472 L
F
.86335 .01472 m
.86335 .01472 L
.86957 .01472 L
.86957 .01472 L
.86335 .01472 L
s
.89441 .01472 m
.89441 .0557 L
.90062 .0557 L
.90062 .01472 L
F
.89441 .01472 m
.89441 .0557 L
.90062 .0557 L
.90062 .01472 L
.89441 .01472 L
s
.92547 .01472 m
.92547 .04505 L
.93168 .04505 L
.93168 .01472 L
F
.92547 .01472 m
.92547 .04505 L
.93168 .04505 L
.93168 .01472 L
.92547 .01472 L
s
.95652 .01472 m
.95652 .22212 L
.96273 .22212 L
.96273 .01472 L
F
.95652 .01472 m
.95652 .22212 L
.96273 .22212 L
.96273 .01472 L
.95652 .01472 L
s
.98758 .01472 m
.98758 .02209 L
.99379 .02209 L
.99379 .01472 L
F
.98758 .01472 m
.98758 .02209 L
.99379 .02209 L
.99379 .01472 L
.98758 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{500, 309},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007d0001=B000`400?l00000o`00003ooooooooe
ooooo`00;Oooool3o`0000[ooooo1Ol0000:ooooo`?o00003?ooool3o`0000[ooooo0ol0000;oooo
o`?o00003?ooool00ol0003oooooooooo`0:ooooo`?o00002oooool2o`0000Wooooo0ol00004oooo
o`;o00001Oooool3o`0000?ooooo0ol00005ooooo`?o00000_ooool5o`0000Cooooo0ol00003oooo
o`?o00001Oooool3o`0000Cooooo0ol00004ooooo`?o00000oooool3o`0000Gooooo0ol00003oooo
o`?o00001Oooool3o`0000Cooooo00?o0000ooooooooool01?ooool3o`0000?ooooo0ol00005oooo
o`?o00000oooool2o`0000Gooooo1Ol00003ooooo`;o00001?ooool5o`0000;ooooo0ol00004oooo
o`Go000000?oooooo`000?l000000ol00003ooooo`Go00000_ooool3o`0000Cooooo1Ol00003oooo
o`?o00000oooool5o`0000;ooooo0ol00004ooooo`Go00000_ooool3o`0000Cooooo1Ol00003oooo
o`03o`000?oooooooooo00?ooooo1Ol00002ooooo`?o00001?ooool5o`0000;ooooo0_l00006oooo
o`?o00001?ooool2o`0000Gooooo0ol00003ooooo`?o00001Oooool3o`0000;ooooo1Ol0000>oooo
o`00;_ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000Wooooo00Go0000oooo
ooooooooooooo`00000<ooooo`03o`000?oooooooooo00Sooooo00Go0000ooooooooooooooooo`00
0009ooooo`05o`000?ooooooooooooooool000002oooool00ol0003oooooooooo`09ooooo`05o`00
0?ooooooooooooooool000003?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00;o
oooo00Co0000oooooooooooo00001Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`05o`000?oooooo
ooooooooool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01?ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool0
0oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`03oooo
o`04o`000?ooooooooooo`0000;ooooo00Co0000oooooooooooo00001?ooool01?l0003ooooooooo
ool00003ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooo00000_ooool01?l0003o
ooooooooool00004ooooo`06o`000?ooooooooooo`000?oooooo00000oooool00ol0003ooooooooo
o`02ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool00oooool01_l0003ooooo
ooooool0003oooooo`0000?ooooo00?o0000ooooooooool00_ooool01_l0003oooooooooool0003o
ooooo`0000?ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00003ooooo`03o`00
0?oooooooooo00Cooooo00Ko0000oooooooooooo0000ooooool00003ooooo`03o`000?oooooooooo
00;ooooo00Co0000oooooooooooo00001?ooool00ol0003oooooooooo`02ooooo`05o`000?oooooo
ooooooooool000000_ooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000oooool00ol0003oooooooooo`03ooooo`05o`000?ooooooooooooooool000000_ooool01?l0
003oooooooooool0000>ooooo`00;_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo
00gooooo00?o0000ooooooooool01oooool5o`0000gooooo00?o0000ooooooooool01oooool01Ol0
003ooooooooooooooooo000000cooooo00?o0000ooooooooool02?ooool01Ol0003ooooooooooooo
oooo000000gooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`02ooooo`04o`000?oo
ooooooooo`0000Gooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00?ooooo00Co0000oooooooooooooooo1Ol00005ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00Go0000oooo
ooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00001Ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00ooo
ool01?l0003oooooooooooooool5o`0000Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`03ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooooooooooo0000
00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`05ooooo`05o`000?oooooo
ooooooooool000002Oooool01Ol0003ooooooooooooooooo0000013ooooo000^ooooo`03o`000?oo
oooooooo00cooooo00?o0000ooooooooool02_ooool2o`0000[ooooo00Co0000oooooooooooo0000
3_ooool00ol0003oooooooooo`07ooooo`Co00003Oooool00ol0003oooooooooo`09ooooo`?o0000
2oooool4o`0000Sooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`03
o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool00oooool2o`0000Kooooo00Go0000oooooooo
ooooooooo`000002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`03ooooo`04o`000?ooooooooooooooo`Co00001_ooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool3o`0000Kooooo00?o0000
ooooooooool00_ooool4o`0000Kooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`06ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`02ooooo`;o00001ooo
ool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Co00001oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00Gooooo00Co0000oooooooooooooooo0ol00007ooooo`04
o`000?ooooooooooooooo`Co00001Oooool2o`0000?ooooo00Co0000oooooooooooo00001Oooool2
o`0000Cooooo00?o0000ooooooooool01Oooool2o`0000Gooooo00?o0000ooooooooool03Oooool0
02kooooo00?o0000ooooooooool02Oooool01Ol0003ooooooooooooooooo000000gooooo00?o0000
ooooooooool02?ooool00ol0003oooooo`00000:ooooo`Co00002_ooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00Go0000ooooooooooooooooo`000009ooooo`05o`000?oo
ooooooooooooool000002?ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Go
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`05o`000?oooooooooooooo
ool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Kooooo00Co0000oooo
oooooooooooo1?l00006ooooo`05o`000?ooooooooooooooool000002Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooool00002ooooo`05o`000?ooooooooooooooool000000ooo
ool01Ol0003ooooooooooooooooo000000Gooooo00Oo0000ooooooooooooooooo`000?oooooo0000
00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00Go0000
ooooooooooooooooo`000003ooooo`06o`000?ooooooooooo`000?oooooo00001?ooool01_l0003o
oooooooooooooooo0000ooooo`Co00001?ooool01ol0003ooooooooooooooooo0000ooooool00000
1oooool01Ol0003ooooooooooooooooo000000Gooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo
000000?ooooo00?o0000ooooooooool01Oooool01?l0003oooooooooool00002ooooo`03o`000?oo
oooooooo00Gooooo00Go0000ooooooooooooooooo`000009ooooo`03o`000?oooooo000000?ooooo
00?o0000ooooooooool03?ooool002gooooo0_l0000;ooooo`05o`000?ooooooooooooooool00000
2Oooool01Ol0003ooooooooooooooooo000000_ooooo0_l0000:ooooo`03o`000?oooooooooo00co
oooo00?o0000ooooooooool02_ooool01Ol0003ooooooooooooooooo000000Wooooo00Go0000oooo
ooooooooooooo`000009ooooo`05o`000?ooooooooooooooool000001oooool2o`0000Cooooo00Co
0000oooooooooooo00001?ooool2o`0000Cooooo0_l00006ooooo`;o00000oooool01Ol0003ooooo
oooooooooooo000000Cooooo0_l00003ooooo`05o`000?ooooooooooooooool000001?ooool2o`00
00Gooooo0_l00005ooooo`;o00000oooool00ol0003oooooooooo`06ooooo`;o00001?ooool00ol0
003oooooooooo`05ooooo`;o00000oooool01Ol0003ooooooooooooooooo000000Cooooo0_l00003
ooooo`05o`000?ooooooooooooooool000001?ooool2o`0000?ooooo00Go0000oooooooooooooooo
o`000003ooooo`05o`000?ooooooooooooooool000000_ooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool2o`0000Gooooo00Oo0000ooooooooooooooooo`00
0?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo000000?o
oooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooooooo`;o00001?ooool0
1ol0003ooooooooooooooooo0000ooooool000001oooool01Ol0003ooooooooooooooooo000000;o
oooo00?o0000ooooooooool01?ooool01ol0003ooooooooooooooooo0000ooooool000000oooool0
1Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooool000000oooool01Ol0003ooooooooo
oooooooo000000?ooooo00?o0000ooooool000000oooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000_ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000
ooooooooool00_l00005ooooo`07o`000?ooooooooooooooool0003oooooo`000003ooooo`03o`00
0?oooooooooo00cooooo000^ooooo`03o`000?oooooooooo00[ooooo0ol0000;ooooo`?o00003Ooo
ool00ol0003oooooooooo`08ooooo`Go00002oooool2o`0000[ooooo1Ol0000:ooooo`?o00002ooo
ool3o`0000Wooooo00?o0000ooooooooool00oooool2o`0000Kooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;ooooo0ol00006ooooo`03o`000?oo
oooooooo00;ooooo0ol00006ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?oo
ool01?l0003oooooooooooooool5o`0000Gooooo00?o0000ooooooooool00oooool2o`0000Kooooo
00Co0000oooooooooooooooo1Ol00005ooooo`03o`000?oooooooooo00;ooooo0ol00006ooooo`03
o`000?oooooooooo00;ooooo0ol00005ooooo`?o00001?ooool2o`0000Gooooo0ol00004ooooo`03
o`000?oooooooooo00Cooooo0ol00003ooooo`?o00001Oooool3o`0000?ooooo0ol00005ooooo`?o
00001Oooool00ol0003oooooooooo`03ooooo`?o00000_ooool5o`0000Cooooo0ol00004ooooo`;o
00001Oooool3o`0000;ooooo1Ol00004ooooo`?o00000oooool3o`0000Gooooo0ol00003ooooo`?o
00001Oooool3o`0000Cooooo0_l00005ooooo`?o00001?ooool00ol0003oooooooooo`04ooooo`?o
00000oooool3o`0000oooooo003oooooooGooooo003oooooooGooooo003oooooooGooooo003ooooo
ooGooooo003oooooooGooooo003oooooooGooooo003oooooooGooooo000Voooooooo0000aOl0000:
ooooo`009_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`08ooooo`00
9_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`08ooooo`007Oooool2o`0000Oooooo00?o0000ooooooooool0oooo
ooo1ooooo`03o`000?oooooooooo00Sooooo000Looooo`04o`000?ooooooooooo`0000Kooooo00?o
0000ooooooooool0ooooooo1ooooo`03o`000?oooooooooo00Sooooo000Looooo`04o`000?oooooo
ooooo`0000Kooooo00?o0000ooooooooool0ooooooo1ooooo`03o`000?oooooooooo00Sooooo000L
ooooo`04o`000?ooooooooooo`0000Kooooo0ol00003ool000?o0?l00_l00?l4oomo00;o00000ooo
0003o`3o00;o003o1?ooO`02o`0000?oo`000ol0o`02o`00o`Coogl00_l00003ool000?o0?l00_l0
0?l4oomo00;o00000ooo0003o`3o00;o003o1?ooO`02o`0000?oo`000ol0o`02o`00o`Coogl00_l0
0003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0
o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0ooo
O`02o`0000?oo`000ol0o`03o`00o`Coogl000Co0000ool00?oo003oo`000ol0o`03o`00o`Coogl0
00Co0000ool00?oo003oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o
00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl00_l00003ool000?o
0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`Co
ogl000Co0000ool00?oo003oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo
00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl00_l00003ool0
00?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o1?ooO`001?l0003oo`00ool00?oo0003
o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`Coogl000Co0000ool00?oo003oo`000ol0
o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0ooo
O`02o`0000?oo`000ol0o`03o`00o`?oogl00ol0000:ooooo`007?ooool01?l0003oooooooooool0
0006ooooo`03o`000?oooooooooo00?oo`000ol0o`02o`00o`Coogl00_ooool3ool000?o0?l00_l0
0?l4oomo00;ooooo0ooo0003o`3o00;o003o1?ooO`02ooooo`?oo`000ol0o`02o`00o`Coogl00_oo
ool3ool000?o0?l00_l00?l4oomo00;ooooo0ooo0003o`3o00;o003o1?ooO`05ooooo`?o0?l00ol0
0?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl00_oo
ool3ool000?o0?l00ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`08ooooo`Co003o1ooo
ool3o`3o00Co003o1?ooool4ool000;ooooo0ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0ooo
O`02ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo0ooo
0003o`3o00?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl01Oooool3o`3o00Co003o1?oo
ool3ool000?o0?l00ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`?oo`000ol0
o`03o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0ooo
O`02ooooo`Coo`000_ooool4o`00o`Oooooo0ol0o`03o`00o`?oogl01Oooool3o`3o00Co003o1?oo
ool3ool000?o0?l00ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`?oo`000ol0
o`03o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo00?o0000ooooooooool02?oo
ool001cooooo00Co0000oooooooooooo00001_ooool00ol0003oooooooooo`03ool000?o0?l00_l0
0?l4oomo00;ooooo0ooo0003o`3o00;o003o1?ooO`05ooooo`?o0?l00_l00?l4oomo00;ooooo0ooo
0003o`3o00;o003o1?ooO`02ooooo`Coo`000_ooool2o`00o`Coogl00_ooool3ool000?o0?l00_l0
0?l4oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo0ooo
0003o`3o00?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl01Oooool3o`3o00?o003o0ooo
O`08ooooo`Co003o2_ooool4o`00o`Cooooo1?oo0002ooooo`?o003o0oooO`02ooooo`?oo`000ol0
o`03o`00o`?oogl00_ooool3ool000?o0?l01?l00?l4ooooo`?oo`000ol0o`03o`00o`?oogl00_oo
ool3ool000?o0?l00ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`05ooooo`?o0?l01?l0
0?l4ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool4ool000;ooooo0ol00?l3oomo00;ooooo0ooo
0003o`3o00?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool3ool000?o0?l00ol0
0?l3oomo00;ooooo1?oo0002ooooo`Co003o1oooool3o`3o00?o003o0oooO`05ooooo`?o0?l01?l0
0?l4ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo0ooo
0003o`3o00?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool00ol0003ooooooooo
o`08ooooo`007Oooool2o`0000Oooooo00?o0000ooooooooool00ooo0003o`3o00;o003o1?ooO`02
ooooo`?oo`000ol0o`02o`00o`Coogl01Oooool3o`3o00;o003o1?ooO`02ooooo`?oo`000ol0o`03
o`00o`Gooooo1?oo0002ooooo`;o003o1?ooO`02ooooo`?oo`000ol0o`02o`00o`Coogl01Oooool4
o`3o00;ooooo0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3
oomo00;ooooo1?oo0002ooooo`?o003o0oooO`05ooooo`?o0?l00ol00?l3oomo00Sooooo1?l00?l:
ooooo`Co003o1?ooool4ool000;ooooo0ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`05
ooooo`?o0?l01?l00?l4ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3
oomo00;ooooo0ooo0003o`3o00?o003o0oooO`05ooooo`?o0?l01?l00?l4ooooo`?oo`000ol0o`03
o`00o`?oogl00_ooool4ool000;ooooo0ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02
ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00Sooooo1?l00?l7
ooooo`?o0?l00ol00?l3oomo00Sooooo1?l00?l4ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool3
ool000?o0?l00ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`Coo`000_ooool3
o`00o`?oogl00_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ool0
00?o0?l00_l00?l4oomo00;ooooo0ooo0003o`3o00;o003o1?ooO`05ooooo`?o0?l00_l00?l4oomo
00;ooooo0ooo0003o`3o00?o003o1Oooool4ool000;ooooo0_l00?l4oomo00;ooooo0ooo0003o`3o
00;o003o1?ooO`05ooooo`Co0?l00_ooool3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02oooo
o`?oo`000ol0o`04o`00o`Cooooo1?oo0002ooooo`?o003o0oooO`05ooooo`?o0?l00ol00?l3oomo
00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000Gooooo0oooO`02ooooo`?oo`000ol0o`03o`00
o`?oogl01Oooool3o`3o00Co003o1?ooool3ool000?o0?l00ol00?l3oomo00;ooooo0ooo0003o`3o
00?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl01Oooool3o`3o00Co003o1?ooool3ool0
00?o0?l00ol00?l3oomo00;ooooo1?oo0002ooooo`?o003o0oooO`05ooooo`?o0?l00ol00?l3oomo
00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`?oo`000ol0o`04o`00o`[ooooo1?l00?l7oooo
o`?o0?l00ol00?l3oomo00Sooooo1?l00?l4ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool3ool0
00?o0?l00ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`Coo`000_ooool4o`00
o`Cooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00ooo0003o`3o00?o
003o2?ooool3o`3o00;o003o1?ooO`05ooooo`?o0?l00_l00?l4oomo00;ooooo0ooo0003o`3o00?o
003o1Oooool4ool000;ooooo0ol00?l5ooooo`?oo`000ol0o`02o`00o`Coogl01Oooool4o`3o00;o
oooo0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool3ool000?o0?l01?l00?l4ooooo`Co
o`000_ooool3o`00o`?oogl01Oooool3o`3o00?o003o0oooO`08ooooo`Co003o2_ooool4o`00o`Co
oooo1?oo0005ooooo`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00Gooooo0ol0o`04o`00o`Co
oooo0ooo0003o`3o00Co003o1?ooool3ool000?o0?l01?l00?l7ooooo`?o0?l00ol00?l3oomo00Go
oooo0ol0o`04o`00o`Cooooo0ooo0003o`3o00Co003o1?ooool4ool000;ooooo0ol00?l3oomo00Go
oooo0ol0o`03o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo0ooo0003o`3o00Co
003o2_ooool4o`00o`Oooooo0ol0o`03o`00o`?oogl02?ooool4o`00o`Cooooo0ooo0003o`3o00?o
003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;o
oooo1?oo0002ooooo`Co003o1?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooo
ooooo`03ool000?o0?l00ol00?l8ooooo`?o0?l00_l00?l4oomo00Gooooo0ol0o`02o`00o`Coogl0
0_ooool3ool000?o0?l00ol00?l5ooooo`Coo`000_ooool3o`00o`Gooooo0ooo0003o`3o00;o003o
1?ooO`05ooooo`Co0?l00_ooool3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`?oo`00
0ol0o`04o`00o`Cooooo1?oo0002ooooo`Co003o1oooool3o`3o00?o003o0oooO`08ooooo`Co003o
2_ooool4o`00o`Cooooo1?oo0005ooooo`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00Gooooo
0ol0o`04o`00o`Cooooo0ooo0003o`3o00Co003o1?ooool3ool000?o0?l01?l00?l7ooooo`?o0?l0
0ol00?l3oomo00Sooooo1?l00?l4ooooo`?oo`000ol0o`04o`00o`Cooooo1?oo0002ooooo`?o003o
0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`?oo`00
0ol0o`04o`00o`[ooooo1?l00?l7ooooo`?o0?l00ol00?l3oomo00Sooooo1?l00?l4ooooo`?oo`00
0ol0o`03o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o
0oooO`02ooooo`Coo`000_ooool4o`00o`Cooooo00?o0000ooooooooool02?ooool002Kooooo00?o
0000ooooooooool00ooo0003o`3o00?o003o2?ooool3o`3o00;o003o1?ooO`05ooooo`?o0?l00_l0
0?l4oomo00;ooooo1?oo0002ooooo`?o003o1Oooool4ool000;ooooo0ol00?l5ooooo`?oo`000ol0
o`02o`00o`Coogl01Oooool4o`3o00;ooooo0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl00_oo
ool3ool000?o0?l01?l00?l4ooooo`Coo`000_ooool4o`00o`Oooooo0ol0o`03o`00o`?oogl02?oo
ool4o`00o`[ooooo1?l00?l4ooooo`Coo`001Oooool3oomo00;ooooo0ooo0003o`3o00?o003o0ooo
O`05ooooo`?o0?l01?l00?l4ooooo`?oo`000ol0o`04o`00o`Cooooo0ooo0003o`3o00Co003o1ooo
ool3o`3o00?o003o0oooO`08ooooo`Co003o1?ooool3ool000?o0?l01?l00?l4ooooo`Coo`000_oo
ool3o`00o`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl00_oo
ool3ool000?o0?l01?l00?l:ooooo`Co003o1oooool3o`3o00?o003o0oooO`08ooooo`Co003o1?oo
ool3ool000?o0?l00ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`?oo`000ol0
o`03o`00o`?oogl00_ooool4ool000;ooooo1?l00?l4ooooo`03o`000?oooooooooo00Sooooo000V
ooooo`03o`000?oooooooooo00?oo`000ol0o`03o`00o`Sooooo0ol0o`02o`00o`Coogl01Oooool3
o`3o00;o003o1?ooO`02ooooo`Coo`000_ooool3o`00o`Gooooo1?oo0002ooooo`?o003o1Oooool3
ool000?o0?l00_l00?l4oomo00Gooooo1?l0o`02ooooo`?oogl00_ooool3ool000?o0?l00ol00?l3
oomo00;ooooo0ooo0003o`3o00Co003o1?ooool4ool000gooooo0ol0o`03o`00o`?oogl02?ooool4
o`00o`[ooooo1?l00?l4ooooo`Coo`001Oooool3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`08
ooooo`Co003o1?ooool3ool000?o0?l01?l00?l4ooooo`?oo`000ol0o`04o`00o`Oooooo1?l0o`02
ooooo`?oogl02?ooool4o`00o`Cooooo0ooo0003o`3o00Co003o1?ooool4ool000;ooooo0ol00?l3
oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo0ooo0003
o`3o00Co003o2_ooool4o`00o`Oooooo0ol0o`03o`00o`?oogl02?ooool4o`00o`Cooooo0ooo0003
o`3o00?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3
oomo00;ooooo1?oo0002ooooo`Co003o1?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0
003oooooooooo`04ool000;ooooo0ol00?l8ooooo`?o0?l00_l00?l4oomo00Gooooo0ol0o`02o`00
o`Coogl00_ooool4ool000;ooooo0ol00?l5ooooo`Coo`000_ooool3o`00o`Gooooo0ooo0003o`3o
00;o003o1?ooO`05ooooo`Co0?l00_ooool3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`05oooo
o`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00?o003o0oooO`08ooooo`Co003o2_ooool4o`00
o`Cooooo1?oo0005ooooo`?oogl01Oooool3o`3o00?o003o0oooO`08ooooo`Co003o1?ooool4ool0
00;ooooo1?l00?l4ooooo`?oo`000ol0o`04o`00o`Oooooo1?l0o`02ooooo`?oogl02?ooool4o`00
o`Cooooo0ooo0003o`3o00Co003o1?ooool4ool000;ooooo0ol00?l3oomo00Gooooo0ol0o`03o`00
o`?oogl00_ooool4ool000;ooooo0ol00?l3oomo00;ooooo0ooo0003o`3o00Co003o2_ooool4o`00
o`Oooooo0ol0o`03o`00o`?oogl02?ooool4o`00o`Cooooo0ooo0003o`3o00?o003o0oooO`02oooo
o`?oo`000ol0o`03o`00o`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`000_ooool4o`00
o`Cooooo00?o0000ooooooooool02?ooool002Kooooo0ol00004ool000;ooooo0ol00?l8ooooo`?o
0?l00_l00?l4oomo00Gooooo0ol0o`02o`00o`Coogl00_ooool4ool000;ooooo0ol00?l5ooooo`Co
o`002_ooool3ool000?o0?l00_l00?l4oomo00Gooooo1?l0o`02ooooo`?oogl00_ooool3ool000?o
0?l01?l00?l7ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00?o003o0oooO`08ooooo`Co
003o2_ooool4o`00o`Cooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00Sooooo1?l00?l4ooooo`Co
o`000_ooool4o`00o`Cooooo0ooo0003o`3o00Co003o1oooool4o`3o00;ooooo0oooO`08ooooo`Co
003o1?ooool3ool000?o0?l01?l00?l4ooooo`Coo`000_ooool3o`00o`?oogl01Oooool4o`3o00;o
oooo0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl00_ooool4ool000;ooooo1?l00?l:ooooo`Co
003o1oooool3o`3o00?o003o0oooO`08ooooo`Co003o1?ooool3ool000?o0?l00ol00?l3oomo00;o
oooo0ooo0003o`3o00?o003o0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0002ooooo`Co
003o0oooool2o`0000[ooooo000Vooooo`03o`000?oooooooooo00Coo`000_ooool3o`00o`Sooooo
0ol0o`02o`00o`Coogl01Oooool3o`3o00;o003o1?ooO`02ooooo`Coo`000_ooool3o`00o`Gooooo
1?oo000:ooooo`?oo`000ol0o`02o`00o`Coogl01Oooool4o`3o00;ooooo0oooO`02ooooo`?oo`00
0ol0o`04o`00o`Oooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00Sooooo
1?l00?l:ooooo`Co003o1?ooool4ool000gooooo0ol0o`03o`00o`?oogl02?ooool4o`00o`Cooooo
1?oo0002ooooo`Co003o1?ooool3ool000?o0?l01?l00?l7ooooo`Co0?l00_ooool3oomo00Sooooo
1?l00?l4ooooo`?oo`000ol0o`04o`00o`Cooooo1?oo0002ooooo`?o003o0oooO`05ooooo`Co0?l0
0_ooool3oomo00;ooooo1?oo0002ooooo`?o003o0oooO`02ooooo`Coo`000_ooool4o`00o`[ooooo
1?l00?l7ooooo`?o0?l00ol00?l3oomo00Sooooo1?l00?l7ooooo`?o0?l00ol00?l3oomo00;ooooo
0ooo0003o`3o00?o003o0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo000:ooooo`03o`00
0?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00Coo`000_ooool3o`00o`Sooooo0ol0
o`02o`00o`Coogl01Oooool3o`3o00;o003o1?ooO`02ooooo`Coo`000_ooool3o`00o`Gooooo1?oo
000:ooooo`?oo`000ol0o`02o`00o`Coogl01Oooool4o`3o00;ooooo0oooO`02ooooo`?oo`000ol0
o`04o`00o`Oooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00Sooooo1?l0
0?l:ooooo`Co003o1?ooool4ool000gooooo0ol0o`03o`00o`?oogl02?ooool4o`00o`Cooooo1?oo
0002ooooo`Co003o1?ooool3ool000?o0?l01?l00?l7ooooo`Co0?l00_ooool3oomo00Sooooo1?l0
0?l4ooooo`?oo`000ol0o`04o`00o`Cooooo1?oo0005ooooo`?oogl01Oooool4o`3o00;ooooo0ooo
O`02ooooo`Coo`000_ooool3o`00o`?oogl00_ooool4ool000;ooooo1?l00?l:ooooo`Co003o1ooo
ool3o`3o00?o003o0oooO`08ooooo`Co003o1oooool3o`3o00?o003o0oooO`02ooooo`Coo`000_oo
ool3o`00o`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`002_ooool00ol0003ooooooooo
o`08ooooo`009_ooool00ol0003oooooooooo`04ool000;ooooo0ol00?l8ooooo`?o0?l00_l00?l4
oomo00Gooooo0ol0o`02o`00o`Coogl00_ooool4ool000;ooooo0ol00?l5ooooo`Coo`002_ooool3
ool000?o0?l00_l00?l4oomo00Gooooo1?l0o`02ooooo`?oogl00_ooool3ool000?o0?l01?l00?l7
ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00?o003o0oooO`08ooooo`Co003o2_ooool4
o`00o`Cooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00Sooooo1?l00?l4ooooo`Coo`000_ooool4
o`00o`Cooooo0ooo0003o`3o00Co003o1oooool4o`3o00;ooooo0oooO`08ooooo`Co003o1?ooool3
ool000?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00Gooooo1?l0o`02ooooo`?oogl00_ooool4
ool000;ooooo0ol00?l3oomo00;ooooo1?oo0002ooooo`Co003o2_ooool4o`00o`Oooooo0ol0o`03
o`00o`?oogl02?ooool4o`00o`Oooooo0ol0o`03o`00o`?oogl00_ooool4ool000;ooooo0ol00?l3
oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool0
02Kooooo00?o0000ooooooooool01?oo0002ooooo`?o003o2?ooool3o`3o00;o003o1?ooO`05oooo
o`?o0?l00_l00?l4oomo00Sooooo0ol00?l5ooooo`Coo`003Oooool3o`3o00;o003o1?ooO`05oooo
o`Co0?l00_ooool3oomo00Gooooo0ol0o`04o`00o`Oooooo0ol0o`04o`00o`Cooooo1?oo000=oooo
o`?o0?l00ol00?l3oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02oooo
o`?oogl02?ooool4o`00o`Cooooo1?oo0002ooooo`Co003o1?ooool3ool000?o0?l01?l00?l7oooo
o`Co0?l00_ooool3oomo00Sooooo1?l00?l4ooooo`?oo`000ol0o`04o`00o`Cooooo1?oo000=oooo
o`Co0?l00_ooool3oomo00;ooooo1?oo0002ooooo`?o003o0oooO`02ooooo`Coo`000_ooool4o`00
o`[ooooo1?l00?l7ooooo`?o0?l00ol00?l3oomo00Sooooo1?l00?l7ooooo`?o0?l00ol00?l3oomo
00;ooooo1?oo0002ooooo`Co003o1oooool3o`3o00?o003o0oooO`02ooooo`Coo`002_ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool000;ooooo0ol00?l8ooooo`?o
0?l00_l00?l4oomo00Gooooo0ol0o`02o`00o`Coogl02?ooool3o`00o`Gooooo1?oo000=ooooo`?o
0?l00ol00?l8ooooo`Co0?l00_ooool3oomo00Gooooo0ol0o`04o`00o`Oooooo0ol0o`04o`00o`Co
oooo1?oo000=ooooo`?o0?l00ol00?l3oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000go
oooo1?l0o`02ooooo`?oogl02?ooool4o`00o`Cooooo1?oo0002ooooo`Co003o1?ooool3ool000?o
0?l01?l00?l7ooooo`Co0?l00_ooool3oomo00Sooooo1?l00?l4ooooo`?oo`000ol0o`04o`00o`Co
oooo1?oo000=ooooo`Co0?l00_ooool3oomo00;ooooo1?oo0002ooooo`?o003o0oooO`02ooooo`Co
o`000_ooool4o`00o`[ooooo1?l00?l7ooooo`?o0?l00ol00?l3oomo00Sooooo1?l00?l7ooooo`?o
0?l00ol00?l3oomo00;ooooo1?oo0002ooooo`Co003o1oooool3o`3o00?o003o0oooO`02ooooo`Co
o`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool000;ooooo
0ol00?l8ooooo`?o0?l00_l00?l4oomo00Gooooo0ol0o`02o`00o`Coogl02?ooool3o`00o`Gooooo
1?oo000=ooooo`?o0?l00ol00?l8ooooo`Co0?l00_ooool3oomo00Gooooo0ol0o`04o`00o`Oooooo
0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00Sooooo1?l00?l:ooooo`Co003o
1?ooool4ool000gooooo1?l0o`02ooooo`?oogl02?ooool4o`00o`Cooooo1?oo0002ooooo`Co003o
1?ooool3ool000?o0?l01?l00?l7ooooo`Co0?l00_ooool3oomo00Sooooo1?l00?l4ooooo`?oo`00
0ol0o`04o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo00;ooooo1?oo0002ooooo`Co003o
1?ooool4ool000;ooooo1?l00?l:ooooo`Co003o1oooool4o`3o00;ooooo0oooO`08ooooo`Co003o
1oooool3o`3o00?o003o0oooO`02ooooo`Coo`000_ooool4o`00o`Oooooo1?l0o`02ooooo`?oogl0
0_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo
0002ooooo`?o003o2?ooool3o`3o00;o003o1?ooO`05ooooo`?o0?l00_l00?l4oomo00Sooooo0ol0
0?l5ooooo`Coo`003Oooool3o`3o00?o003o2?ooool4o`3o00;ooooo0oooO`05ooooo`?o0?l01?l0
0?l7ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`08ooooo`Co003o2_oo
ool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo00Sooooo1?l00?l4ooooo`Coo`000_oo
ool4o`00o`Cooooo1?oo0002ooooo`Co003o1oooool4o`3o00;ooooo0oooO`08ooooo`Co003o1?oo
ool4ool000;ooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`02ooooo`Coo`000_oo
ool4o`00o`Cooooo1?oo0002ooooo`Co003o2_ooool4o`00o`Oooooo1?l0o`02ooooo`?oogl02?oo
ool4o`00o`Oooooo0ol0o`03o`00o`?oogl00_ooool4ool000;ooooo1?l00?l7ooooo`Co0?l00_oo
ool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooo
oooo00Coo`000_ooool3o`00o`Sooooo1?l0o`000oooooooogl0oomo0002oomo00Gooooo0ol0o`02
o`00o`Coogl02?ooool3o`00o`Gooooo1?oo000=ooooo`?o0?l00ol00?l8ooooo`Co0?l00_ooool3
oomo00Gooooo0ol0o`04o`00o`Oooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3
oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl02?ooool4
o`00o`Cooooo1?oo0002ooooo`Co003o1?ooool4ool000;ooooo1?l00?l7ooooo`Co0?l00_ooool3
oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl00_ooool4
ool000;ooooo1?l00?l4ooooo`Coo`000_ooool4o`00o`[ooooo1?l00?l7ooooo`Co0?l00_ooool3
oomo00Sooooo1?l00?l7ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0002ooooo`Co003o1oooool4
o`3o00;ooooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0
003oooooooooo`04ool000;ooooo0ol00?l8ooooo`Co0?l000?ooooooomo0?ooO`000_ooO`05oooo
o`?o0?l00_l00?l4oomo00Sooooo0ol00?l5ooooo`Coo`003Oooool3o`3o00?o003o2?ooool4o`3o
00;ooooo0oooO`05ooooo`?o0?l01?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02oooo
o`?oogl02?ooool4o`00o`[ooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`08oooo
o`Co003o1?ooool4ool000;ooooo1?l00?l4ooooo`Coo`000_ooool4o`00o`Oooooo1?l0o`02oooo
o`?oogl02?ooool4o`00o`[ooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`02oooo
o`Coo`000_ooool4o`00o`Cooooo1?oo0002ooooo`Co003o2_ooool4o`00o`Oooooo1?l0o`02oooo
o`?oogl02?ooool4o`00o`Oooooo0ol0o`04o`00o`Cooooo1?oo0002ooooo`Co003o1oooool4o`3o
00;ooooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003o
ooooooooo`04ool000;ooooo0ol00?l8ooooo`Co0?l000?ooooooomo0?ooO`000_ooO`05ooooo`?o
0?l00_l00?l4oomo00Sooooo0ol00?l5ooooo`Coo`003Oooool3o`3o00?o003o2?ooool4o`3o00;o
oooo0oooO`05ooooo`?o0?l01?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?o
ogl02?ooool4o`00o`[ooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`08ooooo`Co
003o1?ooool4ool000[ooooo1?oo0002ooooo`Co003o1oooool4o`3o00;ooooo0oooO`08ooooo`Co
003o2_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo00;ooooo1?oo0002ooooo`Co
003o1?ooool4ool000;ooooo1?l00?l:ooooo`Co003o1oooool4o`3o00;ooooo0oooO`08ooooo`Co
003o1oooool3o`3o00Co003o1?ooool4ool000;ooooo1?l00?l7ooooo`Co0?l00_ooool3oomo00;o
oooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`?o00001?oo0002ooooo`?o003o
2?ooool4o`3o0003ooooooooO`3oogl000;oogl01Oooool3o`3o00;o003o1?ooO`08ooooo`?o003o
1Oooool4ool000gooooo0ol0o`03o`00o`Sooooo1?l0o`02ooooo`?oogl01Oooool3o`3o00Co003o
2_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo00Sooooo1?l00?l:ooooo`Co003o
1?ooool4ool000gooooo1?l0o`02ooooo`?oogl02?ooool4o`00o`Cooooo1?oo000:ooooo`Coo`00
0_ooool4o`00o`Oooooo1?l0o`02ooooo`?oogl02?ooool4o`00o`[ooooo1?l00?l4ooooo`Coo`00
3Oooool4o`3o00;ooooo0oooO`02ooooo`Coo`000_ooool4o`00o`Cooooo1?oo0002ooooo`Co003o
2_ooool4o`00o`Oooooo1?l0o`02ooooo`?oogl02?ooool4o`00o`Oooooo0ol0o`04o`00o`Cooooo
1?oo0002ooooo`Co003o1oooool4o`3o00;ooooo0oooO`02ooooo`Coo`002Oooool2o`0000[ooooo
000Vooooo`03o`000?oooooooooo00Coo`000_ooool3o`00o`Sooooo1?l0o`000oooooooogl0oomo
0002oomo00Gooooo0ol0o`02o`00o`Coogl02?ooool3o`00o`Gooooo1?oo000=ooooo`?o0?l00ol0
0?l8ooooo`Co0?l00_ooool3oomo00Gooooo0ol0o`04o`00o`[ooooo1?l00?l4ooooo`Coo`003Ooo
ool4o`3o00;ooooo0oooO`08ooooo`Co003o2_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_oo
ool3oomo00Sooooo1?l00?l4ooooo`Coo`002_ooool4ool000;ooooo1?l00?l7ooooo`Co0?l00_oo
ool3oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl00_oo
ool4ool000;ooooo1?l00?l4ooooo`Coo`000_ooool4o`00o`[ooooo1?l00?l7ooooo`Co0?l00_oo
ool3oomo00Sooooo1?l00?l7ooooo`?o0?l01?l00?l4ooooo`Coo`000_ooool4o`00o`Oooooo1?l0
o`02ooooo`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000
ooooooooool01?oo0002ooooo`?o003o2?ooool4o`3o0003ooooooooO`3oogl000;oogl01Oooool3
o`3o00;o003o1?ooO`08ooooo`?o003o1Oooool4ool000gooooo0ol0o`03o`00o`Sooooo1?l0o`02
ooooo`?oogl01Oooool3o`3o00Co003o2_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3
oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl02?ooool4
o`00o`Cooooo1?oo000:ooooo`Coo`000_ooool4o`00o`Oooooo1?l0o`02ooooo`?oogl02?ooool4
o`00o`[ooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`02ooooo`Coo`000_ooool4
o`00o`Cooooo1?oo0002ooooo`Co003o2_ooool4o`00o`Oooooo1?l0o`02ooooo`?oogl02?ooool4
o`00o`Oooooo0ol0o`04o`00o`Cooooo1?oo0002ooooo`Co003o1oooool4o`3o00;ooooo0oooO`02
ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool0
00;ooooo0ol00?l8ooooo`Co0?l000?ooooooomo0?ooO`000_ooO`05ooooo`?o0?l00_l00?l4oomo
00Sooooo0ol00?l5ooooo`Coo`003Oooool3o`3o00?o003o2?ooool4o`3o00;ooooo0oooO`05oooo
o`?o0?l01?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl02?ooool4o`00
o`[ooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`08ooooo`Co003o1?ooool4ool0
00[ooooo1?oo0002ooooo`Co003o3Oooool3oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool0
00gooooo1?l0o`02ooooo`?oogl00_ooool4ool000;ooooo1?l00?l4ooooo`Coo`000_ooool4o`00
o`[ooooo1?l00?l7ooooo`Co0?l00_ooool3oomo00Sooooo1?l00?l7ooooo`?o0?l01?l00?l4oooo
o`Coo`000_ooool4o`00o`Oooooo1?l0o`02ooooo`?oogl00_ooool4ool000[ooooo00?o0000oooo
ooooool02?ooool002Kooooo00?o0000ooooooooool01?oo0002ooooo`?o003o2?ooool4o`3o0003
ooooooooO`3oogl000;oogl01Oooool3o`3o00;o003o1?ooO`08ooooo`?o003o1Oooool4ool000go
oooo0ol0o`03o`00o`Sooooo1?l0o`02ooooo`?oogl01Oooool3o`3o00Co003o2_ooool4o`00o`Co
oooo1?oo000=ooooo`Co0?l00_ooool3oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000go
oooo1?l0o`02ooooo`?oogl02?ooool4o`00o`Cooooo1?oo000:ooooo`Coo`000_ooool4o`00o`go
oooo0oooO`08ooooo`Co003o2_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo00;o
oooo1?oo0002ooooo`Co003o1?ooool4ool000;ooooo1?l00?l:ooooo`Co003o1oooool4o`3o00;o
oooo0oooO`08ooooo`Co003o1oooool3o`3o00Co003o1?ooool4ool000;ooooo1?l00?l7ooooo`Co
0?l00_ooool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`00
0?oooooooooo00Coo`000_ooool3o`00o`Sooooo1?l0o`000oooooooogl0oomo0002oomo00Gooooo
0ol0o`02o`00o`Coogl02?ooool3o`00o`Gooooo1?oo000=ooooo`?o0?l00ol00?l8ooooo`Co0?l0
0_ooool3oomo00Gooooo0ol0o`04o`00o`[ooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo
0oooO`08ooooo`Co003o2_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo00Sooooo
1?l00?l4ooooo`Coo`002_ooool4ool000;ooooo1?l00?l=ooooo`?oogl02?ooool4o`00o`[ooooo
1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`02ooooo`Coo`000_ooool4o`00o`[ooooo
1?l00?l:ooooo`Co003o1oooool4o`3o00;ooooo0oooO`08ooooo`Co003o1oooool3o`3o00Co003o
1?ooool4ool000;ooooo1?l00?l7ooooo`Co0?l00_ooool3oomo00;ooooo1?oo000:ooooo`03o`00
0?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00Coo`000_ooool3o`00o`Sooooo1?l0
o`000oooooooogl0oomo0002oomo00Gooooo0ol0o`02o`00o`Coogl02?ooool3o`00o`Gooooo1?oo
000=ooooo`?o0?l00ol00?l8ooooo`Co0?l00_ooool3oomo00Gooooo0ol0o`04o`00o`[ooooo1?l0
0?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`08ooooo`Co003o2_ooool4o`00o`Cooooo1?oo
000=ooooo`Co0?l00_ooool3oomo00Sooooo1?l00?l4ooooo`Coo`002_ooool4ool000;ooooo1?l0
0?l=ooooo`?oogl02?ooool4o`00o`[ooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0ooo
O`02ooooo`Coo`000_ooool4o`00o`[ooooo1?l00?l:ooooo`Co003o1oooool4o`3o00;ooooo0ooo
O`08ooooo`Co003o1oooool3o`3o00Co003o1?ooool4ool000;ooooo1?l00?l7ooooo`Co0?l00_oo
ool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooo
oooo00Coo`003Oooool4o`3o0003ooooooooO`3oogl000;oogl01Oooool3o`3o00;o003o1?ooO`08
ooooo`?o003o1Oooool4ool000gooooo0ol0o`03o`00o`Sooooo1?l0o`02ooooo`?oogl01Oooool3
o`3o00Co003o2_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo00Sooooo1?l00?l:
ooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl02?ooool4o`00o`Cooooo1?oo000:
ooooo`Coo`000_ooool4o`00o`gooooo0oooO`08ooooo`Co003o2_ooool4o`00o`Cooooo1?oo000=
ooooo`Co0?l00_ooool3oomo00;ooooo1?oo0002ooooo`Co003o2_ooool4o`00o`[ooooo1?l00?l7
ooooo`Co0?l00_ooool3oomo00Sooooo1?l00?l7ooooo`?o0?l01?l00?l4ooooo`Coo`000_ooool4
o`00o`Oooooo1?l0o`02ooooo`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool0
02Kooooo00?o0000ooooooooool01?oo000=ooooo`Co0?l000?ooooooomo0?ooO`000_ooO`05oooo
o`?o0?l00_l00?l4oomo00Sooooo0ol00?l5ooooo`Coo`003Oooool3o`3o00?o003o2?ooool4o`3o
00;ooooo0oooO`05ooooo`?o0?l01?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02oooo
o`?oogl02?ooool4o`00o`[ooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`08oooo
o`Co003o1?ooool4ool000[ooooo1?oo0002ooooo`Co003o3Oooool3oomo00Sooooo1?l00?l:oooo
o`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl00_ooool4ool000;ooooo1?l00?l:oooo
o`Co003o2_ooool4o`00o`Oooooo1?l0o`02ooooo`?oogl02?ooool4o`00o`Oooooo0ol0o`04o`00
o`Cooooo1?oo0002ooooo`Co003o1oooool4o`3o00;ooooo0oooO`02ooooo`Coo`002_ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool000gooooo1?l0o`000ooooooo
ogl0oomo0002oomo00Gooooo1?l0o`000oooooooogl0oomo0002oomo00Sooooo0ol00?l5ooooo`Co
o`003Oooool3o`3o00?o003o2?ooool4o`3o00;ooooo0oooO`05ooooo`?o0?l01?l00?l:ooooo`Co
003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl02?ooool4o`00o`[ooooo1?l00?l4ooooo`Co
o`003Oooool4o`3o00;ooooo0oooO`08ooooo`Co003o1?ooool4ool0013ooooo1?l00?l=ooooo`?o
ogl02?ooool4o`00o`[ooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`02ooooo`Co
o`000_ooool4o`00o`[ooooo1?l00?l:ooooo`Co003o3Oooool3oomo00Sooooo1?l00?l7ooooo`?o
0?l01?l00?l4ooooo`Coo`000_ooool4o`00o`Oooooo1?l0o`02ooooo`?oogl00_ooool4ool000[o
oooo00?o0000ooooooooool02?ooool002Kooooo0ol00004ool000gooooo1?l0o`000oooooooogl0
oomo0002oomo00Gooooo1?l0o`000oooooooogl0oomo0002oomo00Sooooo0ol00?l5ooooo`Coo`00
3Oooool3o`3o00?o003o2?ooool4o`3o00;ooooo0oooO`05ooooo`?o0?l01?l00?l:ooooo`Co003o
1?ooool4ool001?ooooo0oooO`08ooooo`Co003o2_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l0
0_ooool3oomo00Sooooo1?l00?l4ooooo`Coo`004?ooool4o`00o`gooooo0oooO`08ooooo`Co003o
2_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo00;ooooo1?oo0002ooooo`Co003o
2_ooool4o`00o`[ooooo1?l00?l=ooooo`?oogl02?ooool4o`00o`Oooooo0ol0o`04o`00o`Cooooo
1?oo0002ooooo`Co003o1oooool4o`3o00;ooooo0oooO`02ooooo`Coo`002Oooool2o`0000[ooooo
000Vooooo`03o`000?oooooooooo00Coo`003Oooool4o`3o0003ooooooooO`3oogl000;oogl01Ooo
ool4o`3o0003ooooooooO`3oogl000;oogl02?ooool3o`00o`Gooooo1?oo000=ooooo`?o0?l00ol0
0?l8ooooo`Co0?l00_ooool3oomo00Gooooo0ol0o`04o`00o`[ooooo1?l00?l4ooooo`Coo`004ooo
ool3oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl02?oo
ool4o`00o`Cooooo1?oo000@ooooo`Co003o3Oooool3oomo00Sooooo1?l00?l:ooooo`Co003o1?oo
ool4ool000gooooo1?l0o`02ooooo`?oogl00_ooool4ool000;ooooo1?l00?l:ooooo`Co003o2_oo
ool4o`00o`gooooo0oooO`08ooooo`Co003o1oooool3o`3o00Co003o1?ooool4ool000;ooooo1?l0
0?l7ooooo`Co0?l00_ooool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000V
ooooo`03o`000?oooooooooo00Coo`003Oooool4o`3o0003ooooooooO`3oogl000;oogl01Oooool4
o`3o0003ooooooooO`3oogl000;oogl02?ooool3o`00o`Gooooo1?oo000=ooooo`?o0?l00ol00?l8
ooooo`Co0?l00_ooool3oomo00Gooooo0ol0o`04o`00o`[ooooo1?l00?l4ooooo`Coo`004oooool3
oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl02?ooool4
o`00o`Cooooo1?oo000@ooooo`Co003o3Oooool3oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4
ool000gooooo1?l0o`02ooooo`?oogl02?ooool4o`00o`[ooooo1?l00?l:ooooo`Co003o3Oooool3
oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000;ooooo1?l00?l7ooooo`Co0?l00_ooool3
oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo
00Coo`003Oooool4o`3o0003ooooooooO`3oogl000;oogl01Oooool4o`3o0003ooooooooO`3oogl0
00;oogl02?ooool3o`00o`Gooooo1?oo000=ooooo`?o0?l00ol00?l8ooooo`Co0?l00_ooool3oomo
00Gooooo0ol0o`04o`00o`[ooooo1?l00?l4ooooo`Coo`004oooool3oomo00Sooooo1?l00?l:oooo
o`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl02?ooool4o`00o`Cooooo1?oo000@oooo
o`Co003o3Oooool3oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02oooo
o`?oogl02?ooool4o`00o`[ooooo1?l00?l:ooooo`Co003o3Oooool3oomo00Sooooo1?l00?l:oooo
o`Co003o1?ooool4ool000;ooooo1?l00?l7ooooo`Co0?l00_ooool3oomo00;ooooo1?oo000:oooo
o`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00Coo`003Oooool4o`3o0003
ooooooooO`3oogl000;oogl01Oooool4o`3o0003ooooooooO`3oogl000;oogl02?ooool3o`00o`Go
oooo1?oo000=ooooo`?o0?l00ol00?l8ooooo`Co0?l00_ooool3oomo00Gooooo0ol0o`04o`00o`[o
oooo1?l00?l4ooooo`Coo`004oooool3oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000go
oooo1?l0o`02ooooo`?oogl02?ooool4o`00o`Cooooo1?oo000@ooooo`Co003o3Oooool3oomo00So
oooo1?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl02?ooool4o`00o`[o
oooo1?l00?l:ooooo`Co003o3Oooool3oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000;o
oooo1?l00?l7ooooo`Co0?l00_ooool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00So
oooo000Vooooo`03o`000?oooooooooo00Coo`003Oooool4o`3o0003ooooooooO`3oogl000;oogl0
1Oooool4o`3o0003ooooooooO`3oogl000;oogl02?ooool3o`00o`Gooooo1?oo000=ooooo`?o0?l0
0ol00?l8ooooo`Co0?l00_ooool3oomo00Gooooo0ol0o`04o`00o`[ooooo1?l00?l4ooooo`Coo`00
4oooool3oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl0
2?ooool4o`00o`Cooooo1?oo000@ooooo`Co003o3Oooool3oomo00Sooooo1?l00?l:ooooo`Co003o
1?ooool4ool000gooooo1?l0o`02ooooo`?oogl02?ooool4o`00o`[ooooo1?l00?l:ooooo`Co003o
3Oooool3oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000;ooooo1?l00?l7ooooo`Co0?l0
0_ooool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oo
oooooooo00Coo`003Oooool4o`3o0003ooooooooO`3oogl000;oogl01Oooool4o`3o0003oooooooo
O`3oogl000;oogl02?ooool3o`00o`Gooooo1?oo000=ooooo`?o0?l00ol00?l8ooooo`Co0?l00_oo
ool3oomo00Gooooo0ol0o`04o`00o`[ooooo1?l00?l4ooooo`Coo`004oooool3oomo00Sooooo1?l0
0?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl02?ooool4o`00o`Cooooo1?oo
000@ooooo`Co003o3Oooool3oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0
o`02ooooo`?oogl02?ooool4o`00o`[ooooo1?l00?l:ooooo`Co003o3Oooool3oomo00Sooooo1?l0
0?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl00_ooool4ool000[ooooo00?o
0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000=ooooo`Co0?l000?ooooo
oomo0?ooO`000_ooO`05ooooo`Co0?l000?ooooooomo0?ooO`000_ooO`08ooooo`?o003o1Oooool4
ool000gooooo0ol0o`03o`00o`Sooooo1?l0o`02ooooo`?oogl01Oooool3o`3o00Co003o2_ooool4
o`00o`Cooooo1?oo000Cooooo`?oogl02?ooool4o`00o`[ooooo1?l00?l4ooooo`Coo`003Oooool4
o`3o00;ooooo0oooO`08ooooo`Co003o1?ooool4ool0013ooooo1?l00?l=ooooo`?oogl02?ooool4
o`00o`[ooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`08ooooo`Co003o2_ooool4
o`00o`[ooooo1?l00?l=ooooo`?oogl02?ooool4o`00o`[ooooo1?l00?l4ooooo`Coo`003Oooool4
o`3o00;ooooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0
003oooooooooo`04ool000gooooo1?l0o`000oooooooogl0oomo0002oomo00Gooooo1?l0o`000ooo
ooooogl0oomo0002oomo00Sooooo0ol00?l5ooooo`Coo`003Oooool3o`3o00?o003o2?ooool4o`3o
00;ooooo0oooO`05ooooo`?o0?l01?l00?l:ooooo`Co003o1?ooool4ool001?ooooo0oooO`08oooo
o`Co003o2_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo00Sooooo1?l00?l4oooo
o`Coo`004?ooool4o`00o`gooooo0oooO`08ooooo`Co003o2_ooool4o`00o`Cooooo1?oo000=oooo
o`Co0?l00_ooool3oomo00Sooooo1?l00?l:ooooo`Co003o2_ooool4o`00o`gooooo0oooO`08oooo
o`Co003o2_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo00;ooooo1?oo000:oooo
o`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00Coo`003Oooool4o`3o0003
ooooooooO`3oogl000;oogl01Oooool4o`3o0003ooooooooO`3oogl000;oogl02?ooool3o`00o`Go
oooo1?oo000@ooooo`?o003o2?ooool4o`3o00;ooooo0oooO`05ooooo`?o0?l01?l00?l:ooooo`Co
003o1?ooool4ool001?ooooo0oooO`0Fooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?o
ogl02?ooool4o`00o`Cooooo1?oo000@ooooo`Co003o3Oooool3oomo00Sooooo1?l00?l:ooooo`Co
003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl02?ooool4o`00o`[ooooo1?l00?l:ooooo`Co
003o3Oooool3oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?o
ogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0
1?oo000=ooooo`Co0?l000?ooooooomo0?ooO`000_ooO`05ooooo`Co0?l000?ooooooomo0?ooO`00
0_ooO`08ooooo`?o003o1Oooool4ool0013ooooo0ol00?l8ooooo`Co0?l00_ooool3oomo00Gooooo
0ol0o`04o`00o`[ooooo1?l00?l4ooooo`Coo`004oooool3oomo01Kooooo1?l00?l4ooooo`Coo`00
3Oooool4o`3o00;ooooo0oooO`08ooooo`Co003o1?ooool4ool0013ooooo1?l00?l=ooooo`?oogl0
2?ooool4o`00o`[ooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`08ooooo`Co003o
2_ooool4o`00o`[ooooo1?l00?l=ooooo`?oogl02?ooool4o`00o`[ooooo1?l00?l4ooooo`Coo`00
3Oooool4o`3o00;ooooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_oo
ool3o`0000Coo`003Oooool4o`3o0003ooooooooO`3oogl000;oogl01Oooool4o`3o0003oooooooo
O`3oogl000;oogl02?ooool3o`00o`Gooooo1?oo000@ooooo`?o003o2?ooool4o`3o00;ooooo0ooo
O`05ooooo`?o0?l01?l00?l:ooooo`Co003o1?ooool4ool001?ooooo0oooO`0Fooooo`Co003o1?oo
ool4ool000gooooo1?l0o`02ooooo`?oogl02?ooool4o`00o`Cooooo1?oo000@ooooo`Co003o3Ooo
ool3oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl02?oo
ool4o`00o`[ooooo1?l00?l:ooooo`Co003o3Oooool3oomo00Sooooo1?l00?l:ooooo`Co003o1?oo
ool4ool000gooooo1?l0o`02ooooo`?oogl00_ooool4ool000Wooooo0_l0000:ooooo`009_ooool0
0ol0003oooooooooo`04ool000gooooo1?l0o`000oooooooogl0oomo0002oomo00Gooooo1?l0o`00
0oooooooogl0oomo0002oomo00Sooooo0ol00?l5ooooo`Coo`004?ooool3o`00o`Sooooo1?l0o`02
ooooo`?oogl01Oooool3o`3o00Co003o2_ooool4o`00o`Cooooo1?oo000Cooooo`?oogl05_ooool4
o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo00Sooooo1?l00?l4ooooo`Coo`004?ooool4
o`00o`gooooo0oooO`08ooooo`Co003o2_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3
oomo00Sooooo1?l00?l:ooooo`Co003o2_ooool4o`00o`gooooo0oooO`0Fooooo`Co003o1?ooool4
ool000gooooo1?l0o`02ooooo`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool0
02Kooooo00?o0000ooooooooool01?oo000=ooooo`Co0?l000?ooooooomo0?ooO`000_ooO`05oooo
o`Co0?l000?ooooooomo0?ooO`000_ooO`08ooooo`?o003o1Oooool4ool0013ooooo0ol00?l8oooo
o`Co0?l00_ooool3oomo00Gooooo1?l0o`0=ooooo`Co003o1?ooool4ool001?ooooo0oooO`0Foooo
o`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl02?ooool4o`00o`Cooooo1?oo000@oooo
o`Co003o3Oooool3oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02oooo
o`?oogl02?ooool4o`00o`[ooooo1?l00?l:ooooo`Co003o3Oooool3oomo01Kooooo1?l00?l4oooo
o`Coo`003Oooool4o`3o00;ooooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08oooo
o`009_ooool00ol0003oooooooooo`04ool000gooooo1?l0o`0:ooooo`Co0?l000?ooooooomo0?oo
O`000_ooO`08ooooo`?o003o1Oooool4ool0013ooooo0ol00?l8ooooo`Co0?l00_ooool3oomo00Go
oooo1?l0o`0=ooooo`Co003o1?ooool4ool001?ooooo0oooO`0Fooooo`Co003o1?ooool4ool000go
oooo1?l0o`02ooooo`?oogl02?ooool4o`00o`Cooooo1?oo000@ooooo`Co003o3Oooool3oomo00So
oooo1?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl02?ooool4o`00o`[o
oooo1?l00?l:ooooo`Co003o3Oooool3oomo01Kooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;o
oooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooo
ooooo`04ool000gooooo1?l0o`0:ooooo`Co0?l000?ooooooomo0?ooO`000_ooO`08ooooo`?o003o
1Oooool4ool0013ooooo0ol00?l8ooooo`Co0?l00_ooool3oomo00Gooooo1?l0o`0=ooooo`Co003o
1?ooool4ool001?ooooo0oooO`0Fooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl0
4?ooool4ool0013ooooo1?l00?l=ooooo`?oogl02?ooool4o`00o`[ooooo1?l00?l4ooooo`Coo`00
3Oooool4o`3o00;ooooo0oooO`08ooooo`Co003o2_ooool4o`00o`[ooooo1?l00?l=ooooo`?oogl0
5_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo00;ooooo1?oo000:ooooo`03o`00
0?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00Coo`003Oooool4o`3o00[ooooo1?l0
o`000oooooooogl0oomo0002oomo00Sooooo0ol00?l5ooooo`Coo`004?ooool3o`00o`Sooooo1?l0
o`02ooooo`?oogl01Oooool4o`3o00gooooo1?l00?l4ooooo`Coo`004oooool3oomo01Kooooo1?l0
0?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0@ooooo`Coo`004?ooool4o`00o`gooooo0ooo
O`08ooooo`Co003o2_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo01Kooooo1?l0
0?l:ooooo`Co003o3Oooool3oomo01Kooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0ooo
O`02ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04
ool000gooooo1?l0o`0:ooooo`Co0?l000?ooooooomo0?ooO`000_ooO`08ooooo`?o003o1Oooool4
ool0013ooooo0ol00?l8ooooo`Co0?l00_ooool3oomo01Kooooo1?l00?l4ooooo`Coo`004oooool3
oomo01Kooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0@ooooo`Coo`004?ooool4
o`00o`gooooo0oooO`08ooooo`Co003o2_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3
oomo01Kooooo1?l00?l:ooooo`Co003o3Oooool3oomo01Kooooo1?l00?l4ooooo`Coo`003Oooool4
o`3o00;ooooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0
003oooooooooo`04ool000gooooo1?l0o`0:ooooo`Co0?l000?ooooooomo0?ooO`000_ooO`08oooo
o`?o003o1Oooool4ool0013ooooo0ol00?l8ooooo`Co0?l00_ooool3oomo01Kooooo1?l00?l4oooo
o`Coo`004oooool3oomo01Kooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0@oooo
o`Coo`004?ooool4o`00o`gooooo0oooO`08ooooo`Co003o2_ooool4o`00o`Cooooo1?oo000=oooo
o`Co0?l00_ooool3oomo01Kooooo1?l00?l:ooooo`Co003o3Oooool3oomo01Kooooo1?l00?l4oooo
o`Coo`003Oooool4o`3o00;ooooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08oooo
o`002oooool2o`0000Cooooo00?o0000ooooooooool00oooool2o`0000?ooooo0ol00007ooooo`03
o`000?oooooooooo00Coo`003Oooool4o`3o00[ooooo1?l0o`000oooooooogl0oomo0002oomo00So
oooo0ol00?l5ooooo`Coo`004?ooool3o`00o`Sooooo1?l0o`02ooooo`?oogl05_ooool4o`00o`Co
oooo1?oo000Cooooo`?oogl05_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo013o
oooo1?oo000@ooooo`Co003o3Oooool3oomo00Sooooo1?l00?l:ooooo`Co003o1?ooool4ool000go
oooo1?l0o`02ooooo`?oogl05_ooool4o`00o`[ooooo1?l00?l=ooooo`?oogl05_ooool4o`00o`Co
oooo1?oo000=ooooo`Co0?l00_ooool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00So
oooo000:ooooo`04o`000?ooooooooooo`0000Sooooo00Ko0000oooooooooooo0000ooooool00003
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?oo000=ooooo`Co0?l02_ooool4
o`3o0003ooooooooO`3oogl000;oogl02?ooool3o`00o`Gooooo1?oo000@ooooo`?o003o2?ooool4
o`3o00;ooooo0oooO`0Fooooo`Co003o1?ooool4ool001?ooooo0oooO`0Fooooo`Co003o1?ooool4
ool000gooooo1?l0o`02ooooo`?oogl04?ooool4ool0013ooooo1?l00?l=ooooo`?oogl02?ooool4
o`00o`[ooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0Fooooo`Co003o2_ooool4
o`00o`gooooo0oooO`0Fooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl00_ooool4
ool000[ooooo00?o0000ooooooooool02?ooool000[ooooo00Co0000oooooooooooo00002?ooool0
1?l0003oooooooooool00005ooooo`03o`000?oooooooooo00Cooooo0ol00004ool000gooooo1?l0
o`0:ooooo`Co0?l000?ooooooomo0?ooO`000_ooO`08ooooo`?o003o1Oooool4ool0013ooooo0ol0
0?l8ooooo`Co0?l00_ooool3oomo01Kooooo1?l00?l4ooooo`Coo`004oooool3oomo01kooooo1?oo
000=ooooo`Co0?l00_ooool3oomo013ooooo1?oo000Qooooo`?oogl02?ooool4o`00o`[ooooo1?l0
0?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0Fooooo`Co003o2_ooool4o`00o`gooooo0ooo
O`0Fooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl00_ooool4ool000Oooooo1?l0
000:ooooo`002_ooool01?l0003oooooooooool00008ooooo`04o`000?ooooooooooo`0000Gooooo
00?o0000ooooooooool01?ooool00ol0003oooooooooo`04ool000gooooo1?l0o`0:ooooo`Co0?l0
00?ooooooomo0?ooO`000_ooO`08ooooo`?o003o1Oooool4ool001_ooooo1?l0o`02ooooo`?oogl0
5_ooool4o`00o`Cooooo1?oo000Cooooo`?oogl07_ooool4ool000gooooo1?l0o`02ooooo`?oogl0
4?ooool4ool0027ooooo0oooO`08ooooo`Co003o2_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l0
0_ooool3oomo01Kooooo1?l00?l:ooooo`Co003o3Oooool3oomo01Kooooo1?l00?l4ooooo`Coo`00
3Oooool4o`3o00;ooooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`002_oo
ool01?l0003oooooooooool00008ooooo`05o`000?ooooooooooo`000?ooool01?l00007ooooo`03
o`000?oooooooooo00Coo`003Oooool4o`3o00[ooooo1?l0o`000oooooooogl0oomo0002oomo00So
oooo0ol00?l5ooooo`Coo`006oooool4o`3o00;ooooo0oooO`0Fooooo`Co003o1?ooool4ool001?o
oooo0oooO`0Nooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0@ooooo`Coo`008Oooool3oomo00So
oooo1?l00?l:ooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl09?ooool4o`00o`go
oooo0oooO`0Fooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl00_ooool4ool000[o
oooo00?o0000ooooooooool02?ooool000[ooooo00Co0000oooooooooooo00002?ooool01_l0003o
ooooooooool0003oooooo`0000[ooooo00?o0000ooooooooool01?oo000=ooooo`Co0?l02_ooool4
o`3o0003ooooooooO`3oogl000;oogl02?ooool3o`00o`Gooooo1?oo000Kooooo`Co0?l00_ooool3
oomo01Kooooo1?l00?l4ooooo`Coo`004oooool3oomo01kooooo1?oo000=ooooo`Co0?l00_ooool3
oomo013ooooo1?oo000Qooooo`?oogl02?ooool4o`00o`[ooooo1?l00?l4ooooo`Coo`003Oooool4
o`3o00;ooooo0oooO`0Tooooo`Co003o3Oooool3oomo01Kooooo1?l00?l4ooooo`Coo`003Oooool4
o`3o00;ooooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`002oooool2o`00
00[ooooo0_l00002ooooo`Go00001_ooool00ol0003oooooooooo`04ool000gooooo1?l0o`0:oooo
o`Co0?l000?ooooooomo0?ooO`000_ooO`08ooooo`?o003o1Oooool4ool001_ooooo1?l0o`02oooo
o`?oogl05_ooool4o`00o`Cooooo1?oo000Cooooo`?oogl07_ooool4ool000gooooo1?l0o`02oooo
o`?oogl04?ooool4ool0027ooooo0oooO`08ooooo`Co003o2_ooool4o`00o`Cooooo1?oo000=oooo
o`Co0?l00_ooool3oomo03Gooooo0oooO`0Fooooo`Co003o1?ooool4ool000gooooo1?l0o`02oooo
o`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooooooo
ool01?oo000=ooooo`Co0?l02_ooool4o`3o0003ooooooooO`3oogl000;oogl02?ooool3o`00o`Go
oooo1?oo000Kooooo`Co0?l00_ooool3oomo01Kooooo1?l00?l4ooooo`Coo`004oooool3oomo01ko
oooo1?oo000=ooooo`Co0?l00_ooool3oomo013ooooo1?oo000Qooooo`?oogl02?ooool4o`00o`[o
oooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0eooooo`?oogl05_ooool4o`00o`Co
oooo1?oo000=ooooo`Co0?l00_ooool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00So
oooo000Vooooo`03o`000?oooooooooo00Coo`003Oooool4o`3o00[ooooo1?l0o`000oooooooogl0
oomo0002oomo00Sooooo0ol00?l5ooooo`Coo`006oooool4o`3o00;ooooo0oooO`0Fooooo`Co003o
1?ooool4ool001?ooooo0oooO`0Nooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0@ooooo`Coo`00
8Oooool3oomo01Kooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0eooooo`?oogl0
5_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo00;ooooo1?oo000:ooooo`03o`00
0?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00Coo`003Oooool4o`3o00[ooooo1?l0
o`000oooooooogl0oomo0002oomo013ooooo1?oo000Kooooo`Co0?l00_ooool3oomo01Kooooo1?l0
0?l4ooooo`Coo`004oooool3oomo01kooooo1?oo000=ooooo`Co0?l00_ooool3oomo013ooooo1?oo
000Qooooo`?oogl05_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo03Gooooo0ooo
O`0Fooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl00_ooool4ool000[ooooo00?o
0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000=ooooo`Co0?l02_ooool4
o`3o0003ooooooooO`3oogl000;oogl04?ooool4ool001_ooooo1?l0o`02ooooo`?oogl05_ooool4
o`00o`Cooooo1?oo000Cooooo`?oogl07_ooool4ool000gooooo1?l0o`02ooooo`?oogl04?ooool4
ool0027ooooo0oooO`0Fooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl0=Oooool3
oomo01Kooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`02ooooo`Coo`002_ooool0
0ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool000gooooo1?l0o`0:oooo
o`Co0?l000?ooooooomo0?ooO`000_ooO`0@ooooo`Coo`006oooool4o`3o00;ooooo0oooO`0Foooo
o`Co003o1?ooool4ool001?ooooo0oooO`0Nooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0@oooo
o`Coo`008Oooool3oomo01Kooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0eoooo
o`?oogl05_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo00;ooooo1?oo000:oooo
o`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00Coo`003Oooool4o`3o00[o
oooo1?l0o`000oooooooogl0oomo0002oomo013ooooo1?oo000Kooooo`Co0?l00_ooool3oomo01Ko
oooo1?l00?l4ooooo`Coo`004oooool3oomo01kooooo1?oo000=ooooo`Co0?l00_ooool3oomo013o
oooo1?oo000Qooooo`?oogl05_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l00_ooool3oomo03Go
oooo0oooO`0Fooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl00_ooool4ool000[o
oooo00?o0000ooooooooool02?ooool002Kooooo0ol00004ool000gooooo1?l0o`0:ooooo`Co0?l0
00?ooooooomo0?ooO`000_ooO`0@ooooo`Coo`006oooool4o`3o00;ooooo0oooO`0Fooooo`Co003o
1?ooool4ool001?ooooo0oooO`0Nooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0@ooooo`Coo`00
8Oooool3oomo01Kooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0eooooo`?oogl0
7_ooool4ool000gooooo1?l0o`02ooooo`?oogl00_ooool4ool000Wooooo0_l0000:ooooo`009_oo
ool00ol0003oooooooooo`04ool000gooooo1?l0o`0:ooooo`Co0?l000?ooooooomo0?ooO`000_oo
O`0@ooooo`Coo`006oooool4o`3o00;ooooo0oooO`0Fooooo`Co003o1?ooool4ool001?ooooo0ooo
O`0Nooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0@ooooo`Coo`008Oooool3oomo01Kooooo1?l0
0?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0eooooo`?oogl0;oooool4o`3o00;ooooo0ooo
O`02ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04
ool000gooooo1?l0o`0:ooooo`Co0?l000?ooooooomo0?ooO`000_ooO`0@ooooo`Coo`006oooool4
o`3o00;ooooo0oooO`0Fooooo`Co003o1?ooool4ool001?ooooo0oooO`0Nooooo`Coo`003Oooool4
o`3o00;ooooo0oooO`0@ooooo`Coo`008Oooool3oomo01Kooooo1?l00?l4ooooo`Coo`003Oooool4
o`3o00;ooooo0oooO`0eooooo`?oogl0;oooool4o`3o00;ooooo0oooO`02ooooo`Coo`002_ooool0
0ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool000gooooo1?l0o`0:oooo
o`Co0?l000?ooooooomo0?ooO`000_ooO`0@ooooo`Coo`006oooool4o`3o00;ooooo0oooO`0Foooo
o`Co003o1?ooool4ool001?ooooo0oooO`0Nooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0@oooo
o`Coo`008Oooool3oomo01Kooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0eoooo
o`?oogl0;oooool4o`3o00;ooooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08oooo
o`009_ooool00ol0003oooooooooo`04ool000gooooo1?l0o`0:ooooo`Co0?l05Oooool4ool001_o
oooo1?l0o`02ooooo`?oogl05_ooool4o`00o`Cooooo1?oo000Cooooo`?oogl07_ooool4ool000go
oooo1?l0o`02ooooo`?oogl04?ooool4ool0027ooooo0oooO`0Fooooo`Co003o1?ooool4ool000go
oooo1?l0o`02ooooo`?oogl0=Oooool3oomo02oooooo1?l0o`02ooooo`?oogl00_ooool4ool000[o
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000=ooooo`Co0?l0
2_ooool4o`3o01Gooooo1?oo000Kooooo`Co0?l06oooool4o`00o`Cooooo1?oo000Cooooo`?oogl0
7_ooool4ool000gooooo1?l0o`02ooooo`?oogl04?ooool4ool0027ooooo0oooO`0Fooooo`Co003o
1?ooool4ool000gooooo1?l0o`02ooooo`?oogl0=Oooool3oomo02oooooo1?l0o`02ooooo`?oogl0
0_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo
000=ooooo`Co0?l02_ooool4o`3o01Gooooo1?oo000Kooooo`Co0?l06oooool4o`00o`Cooooo1?oo
000Cooooo`?oogl07_ooool4ool000gooooo1?l0o`02ooooo`?oogl04?ooool4ool0027ooooo0ooo
O`0Fooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl0=Oooool3oomo02oooooo1?l0
o`02ooooo`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000
ooooooooool01?oo000=ooooo`Co0?l02_ooool4o`3o01Gooooo1?oo000Kooooo`Co0?l06oooool4
o`00o`Cooooo1?oo000Cooooo`?oogl07_ooool4ool000gooooo1?l0o`02ooooo`?oogl04?ooool4
ool0027ooooo0oooO`0Fooooo`Co003o1?ooool4ool000gooooo1?l0o`02ooooo`?oogl0=Oooool3
oomo02oooooo1?l0o`02ooooo`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool0
02Kooooo00?o0000ooooooooool01?oo000=ooooo`Co0?l02_ooool4o`3o01Gooooo1?oo000Koooo
o`Co0?l06oooool4o`00o`Cooooo1?oo000Cooooo`?oogl07_ooool4ool000gooooo1?l0o`02oooo
o`?oogl04?ooool4ool0027ooooo0oooO`0Fooooo`Co003o1?ooool4ool000gooooo1?l0o`02oooo
o`?oogl0=Oooool3oomo02oooooo1?l0o`02ooooo`?oogl00_ooool4ool000[ooooo00?o0000oooo
ooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000=ooooo`Co0?l02_ooool4o`3o01Go
oooo1?oo000Kooooo`Co0?l08oooool4ool001?ooooo0oooO`0Nooooo`Coo`003Oooool4o`3o00;o
oooo0oooO`0@ooooo`Coo`008Oooool3oomo01Kooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;o
oooo0oooO`0eooooo`?oogl0;oooool4o`3o00;ooooo0oooO`02ooooo`Coo`002_ooool00ol0003o
ooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool000gooooo1?l0o`0:ooooo`Co0?l0
5Oooool4ool001_ooooo1?l0o`0Sooooo`Coo`004oooool3oomo01kooooo1?oo000=ooooo`Co0?l0
0_ooool3oomo013ooooo1?oo000Qooooo`?oogl05_ooool4o`00o`Cooooo1?oo000=ooooo`Co0?l0
0_ooool3oomo03Gooooo0oooO`0_ooooo`Co0?l00_ooool3oomo00;ooooo1?oo000:ooooo`03o`00
0?oooooooooo00Sooooo000Vooooo`?o00001?oo000=ooooo`Co0?l02_ooool4o`3o01Gooooo1?oo
000Kooooo`Co0?l08oooool4ool001?ooooo0oooO`0Nooooo`Coo`003Oooool4o`3o00;ooooo0ooo
O`0@ooooo`Coo`008Oooool3oomo01Kooooo1?l00?l4ooooo`Coo`003Oooool4o`3o00;ooooo0ooo
O`0eooooo`?oogl0;oooool4o`3o00;ooooo0oooO`02ooooo`Coo`002Oooool2o`0000[ooooo000V
ooooo`03o`000?oooooooooo00Coo`003Oooool4o`3o00[ooooo1?l0o`0Eooooo`Coo`006oooool4
o`3o02?ooooo1?oo000Cooooo`?oogl07_ooool4ool000gooooo1?l0o`02ooooo`?oogl04?ooool4
ool0027ooooo0oooO`0Nooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0eooooo`?oogl0;oooool4
o`3o00;ooooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0
003oooooooooo`04ool000gooooo1?l0o`0:ooooo`Co0?l05Oooool4ool001_ooooo1?l0o`0Soooo
o`Coo`004oooool3oomo01kooooo1?oo000=ooooo`Co0?l00_ooool3oomo013ooooo1?oo000Qoooo
o`?oogl07_ooool4ool000gooooo1?l0o`02ooooo`?oogl0=Oooool3oomo02oooooo1?l0o`02oooo
o`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooooooo
ool01?oo000=ooooo`Co0?l02_ooool4o`3o01Gooooo1?oo000Kooooo`Co0?l08oooool4ool001?o
oooo0oooO`0Nooooo`Coo`003Oooool4o`3o01Gooooo1?oo000Qooooo`?oogl07_ooool4ool000go
oooo1?l0o`02ooooo`?oogl0=Oooool3oomo02oooooo1?l0o`02ooooo`?oogl00_ooool4ool000[o
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000=ooooo`Co0?l0
2_ooool4o`3o01Gooooo1?oo000Kooooo`Co0?l08oooool4ool001?ooooo0oooO`0Nooooo`Coo`00
3Oooool4o`3o01Gooooo1?oo000Qooooo`?oogl07_ooool4ool000gooooo1?l0o`02ooooo`?oogl0
=Oooool3oomo02oooooo1?l0o`02ooooo`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool0
2?ooool002Kooooo00?o0000ooooooooool01?oo000=ooooo`Co0?l02_ooool4o`3o01Gooooo1?oo
000Kooooo`Co0?l08oooool4ool001?ooooo0oooO`0Nooooo`Coo`003Oooool4o`3o01Gooooo1?oo
000Qooooo`?oogl07_ooool4ool000gooooo1?l0o`02ooooo`?oogl0=Oooool3oomo02oooooo1?l0
o`02ooooo`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000
ooooooooool01?oo000=ooooo`Co0?l02_ooool4o`3o01Gooooo1?oo000Kooooo`Co0?l08oooool4
ool001?ooooo0oooO`0Nooooo`Coo`003Oooool4o`3o01Gooooo1?oo000Qooooo`?oogl07_ooool4
ool000gooooo1?l0o`02ooooo`?oogl0=Oooool3oomo02oooooo1?l0o`02ooooo`?oogl00_ooool4
ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000=oooo
o`Co0?l02_ooool4o`3o01Gooooo1?oo000Kooooo`Co0?l08oooool4ool001?ooooo0oooO`0Noooo
o`Coo`003Oooool4o`3o01Gooooo1?oo000Qooooo`?oogl07_ooool4ool000gooooo1?l0o`02oooo
o`?oogl0=Oooool3oomo02oooooo1?l0o`02ooooo`?oogl00_ooool4ool000[ooooo00?o0000oooo
ooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000=ooooo`Co0?l02_ooool4o`3o01Go
oooo1?oo000Kooooo`Co0?l08oooool4ool001?ooooo0oooO`0Nooooo`Coo`003Oooool4o`3o01Go
oooo1?oo000Qooooo`?oogl07_ooool4ool000gooooo1?l0o`02ooooo`?oogl0=Oooool3oomo02oo
oooo1?l0o`02ooooo`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo
00?o0000ooooooooool01?oo000=ooooo`Co0?l02_ooool4o`3o01Gooooo1?oo000Kooooo`Co0?l0
8oooool4ool001?ooooo0oooO`0Nooooo`Coo`003Oooool4o`3o01Gooooo1?oo000Qooooo`?oogl0
7_ooool4ool000gooooo1?l0o`02ooooo`?oogl0=Oooool3oomo02oooooo1?l0o`02ooooo`?oogl0
0_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo0ol00004ool000gooooo1?l0
o`0:ooooo`Co0?l05Oooool4ool001_ooooo1?l0o`0Sooooo`Coo`004oooool3oomo01kooooo1?oo
000=ooooo`Co0?l05Oooool4ool0027ooooo0oooO`0Nooooo`Coo`003Oooool4o`3o00;ooooo0ooo
O`0eooooo`?oogl0;oooool4o`3o00;ooooo0oooO`02ooooo`Coo`002Oooool2o`0000[ooooo000V
ooooo`03o`000?oooooooooo00Coo`003Oooool4o`3o00[ooooo1?l0o`0Eooooo`Coo`006oooool4
o`3o02?ooooo1?oo000Cooooo`?oogl07_ooool4ool000gooooo1?l0o`0Eooooo`Coo`008Oooool3
oomo01kooooo1?oo000=ooooo`Co0?l00_ooool3oomo03Gooooo0oooO`0_ooooo`Co0?l00_ooool3
oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo
00Coo`003Oooool4o`3o00[ooooo1?l0o`0Eooooo`Coo`006oooool4o`3o02?ooooo1?oo000Coooo
o`?oogl07_ooool4ool000gooooo1?l0o`0Eooooo`Coo`008Oooool3oomo01kooooo1?oo000=oooo
o`Co0?l00_ooool3oomo03Gooooo0oooO`0_ooooo`Co0?l00_ooool3oomo00;ooooo1?oo000:oooo
o`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00Coo`003Oooool4o`3o00[o
oooo1?l0o`0Eooooo`Coo`006oooool4o`3o02?ooooo1?oo000Cooooo`?oogl07_ooool4ool000go
oooo1?l0o`0Eooooo`Coo`008Oooool3oomo01kooooo1?oo000=ooooo`Co0?l00_ooool3oomo03Go
oooo0oooO`0_ooooo`Co0?l00_ooool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00So
oooo000Vooooo`03o`000?oooooooooo00Coo`003Oooool4o`3o02?ooooo1?oo000Kooooo`Co0?l0
8oooool4ool001?ooooo0oooO`0Nooooo`Coo`003Oooool4o`3o01Gooooo1?oo000Qooooo`?oogl0
7_ooool4ool000gooooo1?l0o`02ooooo`?oogl0=Oooool3oomo02oooooo1?l0o`02ooooo`?oogl0
0_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo
000=ooooo`Co0?l08oooool4ool001_ooooo1?l0o`0Sooooo`Coo`004oooool3oomo01kooooo1?oo
000=ooooo`Co0?l05Oooool4ool0027ooooo0oooO`0Nooooo`Coo`003Oooool4o`3o00;ooooo0ooo
O`0eooooo`?oogl0=Oooool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000V
ooooo`03o`000?oooooooooo00Coo`003Oooool4o`3o02?ooooo1?oo000Kooooo`Co0?l08oooool4
ool001?ooooo0oooO`0Nooooo`Coo`003Oooool4o`3o01Gooooo1?oo000Qooooo`?oogl07_ooool4
ool000gooooo1?l0o`02ooooo`?oogl0=Oooool3oomo03Gooooo0oooO`02ooooo`Coo`002_ooool0
0ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool000gooooo1?l0o`0Soooo
o`Coo`006oooool4o`3o02?ooooo1?oo000Cooooo`?oogl07_ooool4ool000gooooo1?l0o`0Eoooo
o`Coo`008Oooool3oomo01kooooo1?oo000=ooooo`Co0?l00_ooool3oomo03Gooooo0oooO`0eoooo
o`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooooooo
ool01?oo000=ooooo`Co0?l08oooool4ool001_ooooo1?l0o`0Sooooo`Coo`004oooool3oomo02oo
oooo1?l0o`0Eooooo`Coo`008Oooool3oomo01kooooo1?oo000=ooooo`Co0?l00_ooool3oomo03Go
oooo0oooO`0eooooo`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo
00?o0000ooooooooool01?oo000=ooooo`Co0?l08oooool4ool001_ooooo1?l0o`0Sooooo`Coo`00
4oooool3oomo02oooooo1?l0o`0Eooooo`Coo`008Oooool3oomo01kooooo1?oo000=ooooo`Co0?l0
0_ooool3oomo03Gooooo0oooO`0eooooo`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool0
2?ooool002Kooooo00?o0000ooooooooool01?oo000=ooooo`Co0?l08oooool4ool001_ooooo1?l0
o`0Sooooo`Coo`004oooool3oomo02oooooo1?l0o`0Eooooo`Coo`008Oooool3oomo01kooooo1?oo
000Cooooo`?oogl0=Oooool3oomo03[ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000V
ooooo`?o00001?oo000=ooooo`Co0?l08oooool4ool001_ooooo1?l0o`0Sooooo`Coo`004oooool3
oomo02oooooo1?l0o`0Eooooo`Coo`008Oooool3oomo01kooooo1?oo000Cooooo`?oogl0=Oooool3
oomo03[ooooo1?oo0009ooooo`;o00002_ooool002Kooooo00?o0000ooooooooool01?oo000doooo
o`Coo`006oooool4o`3o02?ooooo1?oo000Cooooo`?oogl0;oooool4o`3o01Gooooo1?oo000Qoooo
o`?oogl07_ooool4ool001?ooooo0oooO`0eooooo`?oogl0>_ooool4ool000[ooooo00?o0000oooo
ooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000dooooo`Coo`006oooool4o`3o02?o
oooo1?oo000Cooooo`?oogl0;oooool4o`3o01Gooooo1?oo000Qooooo`?oogl07_ooool4ool001?o
oooo0oooO`0eooooo`?oogl0>_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo
00?o0000ooooooooool01?oo000dooooo`Coo`006oooool4o`3o02?ooooo1?oo000Cooooo`?oogl0
;oooool4o`3o01Gooooo1?oo000Qooooo`?oogl07_ooool4ool001?ooooo0oooO`0eooooo`?oogl0
>_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo
000dooooo`Coo`006oooool4o`3o02?ooooo1?oo000Cooooo`?oogl0B?ooool4ool0027ooooo0ooo
O`0Nooooo`Coo`004oooool3oomo03Gooooo0oooO`0jooooo`Coo`002_ooool00ol0003ooooooooo
o`08ooooo`009_ooool00ol0003oooooooooo`04ool003Cooooo1?oo000Kooooo`Co0?l08oooool4
ool001?ooooo0oooO`18ooooo`Coo`008Oooool3oomo01kooooo1?oo000Cooooo`?oogl0=Oooool3
oomo03[ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo
00Coo`00=?ooool4ool001_ooooo1?l0o`0Sooooo`Coo`004oooool3oomo04Sooooo1?oo000Qoooo
o`?oogl07_ooool4ool001?ooooo0oooO`0eooooo`?oogl0>_ooool4ool000[ooooo00?o0000oooo
ooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000dooooo`Coo`006oooool4o`3o02?o
oooo1?oo000Cooooo`?oogl0B?ooool4ool0027ooooo0oooO`0Nooooo`Coo`004oooool3oomo03Go
oooo0oooO`0jooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`004Oooool2o`0000Cooooo
00?o0000ooooooooool00_ooool3o`0000Oooooo00?o0000ooooooooool01?oo000dooooo`Coo`00
6oooool4o`3o02?ooooo1?oo000Cooooo`?oogl0B?ooool4ool0027ooooo0oooO`0Nooooo`Coo`00
4oooool3oomo03Gooooo0oooO`0jooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`004?oo
ool01?l0003oooooooooool00009ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0
1?oo000dooooo`Coo`006oooool4o`3o02?ooooo1?oo000Cooooo`?oogl0B?ooool4ool0027ooooo
0oooO`0Nooooo`Coo`004oooool3oomo03Gooooo0oooO`0jooooo`Coo`002_ooool00ol0003ooooo
ooooo`08ooooo`004?ooool01?l0003oooooooooool00009ooooo`03o`000?oooooooooo00Kooooo
0ol00004ool003Cooooo1?oo000Kooooo`Co0?l08oooool4ool001?ooooo0oooO`18ooooo`Coo`00
8Oooool3oomo01kooooo1?oo000Cooooo`?oogl0=Oooool3oomo03[ooooo1?oo0007ooooo`Co0000
2_ooool0013ooooo00Co0000oooooooooooo00002Oooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo00Coo`00=?ooool4ool001_ooooo1?l0o`0Sooooo`Coo`004oooool3oomo04Sooooo
1?oo000Qooooo`?oogl07_ooool4ool001?ooooo0oooO`0eooooo`?oogl0>_ooool4ool000[ooooo
00?o0000ooooooooool02?ooool0013ooooo00Co0000oooooooooooo00002Oooool00ol0003ooooo
ooooo`06ooooo`03o`000?oooooooooo00Coo`00=?ooool4ool001_ooooo1?l0o`0Sooooo`Coo`00
4oooool3oomo04Sooooo1?oo000Qooooo`?oogl07_ooool4ool001?ooooo0oooO`0eooooo`?oogl0
>_ooool4ool000[ooooo00?o0000ooooooooool02?ooool0013ooooo00Co0000oooooooooooo0000
2?ooool2o`0000Sooooo00?o0000ooooooooool01?oo000dooooo`Coo`006oooool4o`3o02?ooooo
1?oo000Cooooo`?oogl0B?ooool4ool0027ooooo0oooO`0Nooooo`Coo`004oooool3oomo03Gooooo
0oooO`0jooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`004Oooool2o`0000[ooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`04ool005?ooooo1?l0o`0Sooooo`Coo`004ooo
ool3oomo04Sooooo1?oo000Qooooo`?oogl07_ooool4ool001?ooooo0oooO`0eooooo`?oogl0>_oo
ool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo001C
ooooo`Co0?l08oooool4ool001?ooooo0oooO`18ooooo`Coo`008Oooool3oomo01kooooo1?oo000C
ooooo`?oogl0=Oooool3oomo03[ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Voooo
o`03o`000?oooooooooo00Coo`00Doooool4o`3o02?ooooo1?oo000Cooooo`?oogl0B?ooool4ool0
027ooooo0oooO`0Nooooo`Coo`004oooool3oomo03Gooooo0oooO`0jooooo`Coo`002_ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool005?ooooo1?l0o`0Sooooo`Co
o`004oooool3oomo04Sooooo1?oo000Qooooo`?oogl07_ooool4ool001?ooooo0oooO`0eooooo`?o
ogl0B?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool005?ooooo
1?l0o`0Sooooo`Coo`004oooool3oomo04Sooooo1?oo000Qooooo`?oogl07_ooool4ool001?ooooo
0oooO`0eooooo`?oogl0B?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooooooo
o`1Gooooo`Co0?l08oooool4ool001?ooooo0oooO`18ooooo`Coo`008Oooool3oomo01kooooo1?oo
000Cooooo`?oogl0=Oooool3oomo04Sooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000
ooooooooool0Eoooool4o`3o02?ooooo1?oo000Cooooo`?oogl0B?ooool4ool0027ooooo0oooO`0N
ooooo`Coo`004oooool3oomo03Gooooo0oooO`18ooooo`03o`000?oooooooooo00Sooooo000Voooo
o`?o0000Eoooool4o`3o02?ooooo1?oo000Cooooo`?oogl0B?ooool4ool0027ooooo0oooO`0Noooo
o`Coo`004oooool3oomo03Gooooo0oooO`17ooooo`;o00002_ooool002Kooooo00?o0000oooooooo
ool0Eoooool4o`3o02?ooooo1?oo000Cooooo`?oogl0B?ooool4ool0027ooooo0oooO`0Nooooo`Co
o`004oooool3oomo03Gooooo0oooO`18ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`00
0?oooooooooo05Oooooo1?l0o`0Sooooo`Coo`004oooool3oomo04Sooooo1?oo000Qooooo`?oogl0
7_ooool4ool001?ooooo0oooO`0eooooo`?oogl0B?ooool00ol0003oooooooooo`08ooooo`009_oo
ool00ol0003oooooooooo`1Gooooo`Co0?l08oooool4ool001?ooooo0oooO`18ooooo`Coo`008Ooo
ool3oomo01kooooo1?oo000Cooooo`?oogl0=Oooool3oomo04Sooooo00?o0000ooooooooool02?oo
ool002Kooooo00?o0000ooooooooool0Eoooool4o`3o02?ooooo1?oo000Cooooo`?oogl0B?ooool4
ool0027ooooo0oooO`0Nooooo`Coo`004oooool3oomo03Gooooo0oooO`18ooooo`03o`000?oooooo
oooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`0jooooo`?oogl0B?ooool4ool0
027ooooo0oooO`0Nooooo`Coo`004oooool3oomo03Gooooo0oooO`18ooooo`03o`000?oooooooooo
00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`0jooooo`?oogl0B?ooool4ool0027o
oooo0oooO`0Nooooo`Coo`00Boooool3oomo04Sooooo00?o0000ooooooooool02?ooool002Kooooo
00?o0000ooooooooool0Eoooool4o`3o03[ooooo0oooO`18ooooo`Coo`008Oooool3oomo01kooooo
1?oo001;ooooo`?oogl0B?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooooooo
o`1Gooooo`Co0?l0>_ooool3oomo04Sooooo1?oo000Qooooo`?oogl07_ooool4ool004_ooooo0ooo
O`18ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`0j
ooooo`?oogl0B?ooool4ool0027ooooo0oooO`0Nooooo`Coo`00Boooool3oomo04Sooooo00?o0000
ooooooooool02?ooool002Kooooo0ol0001Gooooo`Co0?l0>_ooool3oomo04Sooooo1?oo000Qoooo
o`?oogl07_ooool4ool004_ooooo0oooO`17ooooo`;o00002_ooool002Kooooo00?o0000oooooooo
ool0Eoooool4o`3o03[ooooo0oooO`18ooooo`Coo`008Oooool3oomo06gooooo0oooO`18ooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`0jooooo`?oogl0
KOooool3oomo06gooooo0oooO`18ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oo
oooooooo05Oooooo1?l0o`0jooooo`?oogl0KOooool3oomo06gooooo0oooO`18ooooo`03o`000?oo
oooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`0jooooo`?oogl0KOooool3
oomo06gooooo0oooO`18ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo
05Oooooo1?l0o`0jooooo`?oogl0KOooool3oomo06gooooo0oooO`18ooooo`03o`000?oooooooooo
00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`0jooooo`?oogl0KOooool3oomo06go
oooo0oooO`18ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo
1?l0o`0jooooo`?oogl0KOooool3oomo06gooooo0oooO`18ooooo`03o`000?oooooooooo00Sooooo
000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`0jooooo`?oogl0KOooool3oomo06gooooo0ooo
O`18ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`0j
ooooo`?oogl0KOooool3oomo0;Sooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooo
ooooool0Eoooool4o`3o03[ooooo0oooO`1]ooooo`?oogl0^?ooool00ol0003oooooooooo`08oooo
o`009_ooool3o`0005Oooooo1?l0o`0jooooo`?oogl0KOooool3oomo0;Oooooo0_l0000:ooooo`00
9_ooool00ol0003oooooooooo`1Gooooo`Co0?l0>_ooool3oomo06gooooo0oooO`2hooooo`03o`00
0?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`0jooooo`?oogl0KOoo
ool3oomo0;Sooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4
o`3o03[ooooo0oooO`1]ooooo`?oogl0^?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0
003oooooooooo`1Gooooo`Co0?l0>_ooool3oomo06gooooo0oooO`2hooooo`03o`000?oooooooooo
00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`0jooooo`?oogl0KOooool3oomo0;So
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o03[ooooo
0oooO`1]ooooo`?oogl0^?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooooooo
o`1Gooooo`Co0?l0>_ooool3oomo06gooooo0oooO`2hooooo`03o`000?oooooooooo00Sooooo000V
ooooo`03o`000?oooooooooo05Oooooo1?l0o`0jooooo`?oogl0KOooool3oomo0;Sooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o03[ooooo0oooO`1]oooo
o`?oogl0^?ooool00ol0003oooooooooo`08ooooo`009_ooool3o`0005Oooooo1?l0o`0jooooo`?o
ogl0KOooool3oomo0;Oooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0
>_ooool3oomo06gooooo0oooO`2hooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oo
oooooooo05Oooooo1?l0o`0jooooo`?oogl0KOooool3oomo0;Sooooo00?o0000ooooooooool02?oo
ool002Kooooo00?o0000ooooooooool0Eoooool4o`3o03[ooooo0oooO`1]ooooo`?oogl0^?ooool0
0ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0>_ooool3oomo
06gooooo0oooO`2hooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oo
oooo1?l0o`0jooooo`?oogl0KOooool3oomo0;Sooooo00?o0000ooooooooool02?ooool002Kooooo
00?o0000ooooooooool0Eoooool4o`3o03[ooooo0oooO`1]ooooo`?oogl0^?ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0>_ooool3oomo06gooooo0ooo
O`2hooooo`03o`000?oooooooooo00Sooooo000;ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`?o00000oooool3o`0000Oooooo00?o0000ooooooooool0Eoooool4o`3o03[ooooo0oooO`1]
ooooo`?oogl0^?ooool00ol0003oooooooooo`08ooooo`002_ooool01?l0003oooooooooool00009
ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo05Oooooo1?l0o`0jooooo`?oogl0KOooool3oomo0;Sooooo00?o0000ooooooooool02?oo
ool000[ooooo00Co0000oooooooooooo00002Oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool0Eoooool4o`3o03[ooooo0oooO`3oooooobWooooo00?o
0000ooooooooool02?ooool000[ooooo00Co0000oooooooooooo00002Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00Cooooo1?l0001Fooooo`Co0?l0>_ooool3oomo0?oooooo9_oo
ool4o`0000[ooooo000:ooooo`04o`000?ooooooooooo`0000Wooooo00Co0000oooooooooooooooo
1?l00007ooooo`03o`000?oooooooooo05Oooooo1?l0o`0jooooo`?oogl0oooooolYooooo`03o`00
0?oooooooooo00Sooooo000:ooooo`04o`000?ooooooooooo`0000Sooooo0_l00003ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool0Eoooool4o`3o03[ooooo0oooO`3oooooobWooooo
00?o0000ooooooooool02?ooool000_ooooo0_l0000:ooooo`04o`000?ooooooooooooooo`Go0000
1_ooool00ol0003oooooooooo`1Gooooo`Co0?l0>_ooool3oomo0?oooooo:Oooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03o`000?oo
oooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo
00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_oo
ool00ol0003oooooooooo`08ooooo`009_ooool3o`0005Oooooo1?l0o`3oooooofGooooo0_l0000:
ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03o`000?oooooooooo
00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000oooooooo
ool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03o`000?oo
oooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo
00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_oo
ool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomV
ooooo`03o`000?oooooooooo00Sooooo000Vooooo`?o0000Eoooool4o`3o0?ooooooIOooool2o`00
00[ooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000oooooooo
ool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03o`000?oo
oooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo
00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_oo
ool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomV
ooooo`03o`000?oooooooooo00Sooooo000Vooooo`?o0000Eoooool4o`3o0?ooooooIOooool2o`00
00[ooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000oooooooo
ool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03o`000?oo
oooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo
00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_oo
ool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomV
ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3ooooo
ofKooooo00?o0000ooooooooool02?ooool002Kooooo0ol0001Gooooo`Co0?l0oooooomUooooo`;o
00002_ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03o`000?oo
oooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo
00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_oo
ool00ol0003oooooooooo`08ooooo`004Oooool2o`0000Cooooo00Co0000oooooooooooooooo1Ol0
0006ooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000ooooooooool02?oo
ool0013ooooo00Co0000oooooooooooo00002?ooool01?l0003oooooooooool00006ooooo`03o`00
0?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000ooooooooool02?ooool0013ooooo00Co
0000oooooooooooo00002Oooool00ol0003oooooooooo`06ooooo`Co0000E_ooool4o`3o0?oooooo
Hoooool4o`0000[ooooo000@ooooo`04o`000?ooooooooooo`0000[ooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03o`000?oooooooooo00Sooooo
000@ooooo`04o`000?ooooooooooo`0000Oooooo00Go0000ooooooooooooooooo`000006ooooo`03
o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000ooooooooool02?ooool0013ooooo
00Co0000oooooooooooo00001oooool01Ol0003ooooooooooooooooo000000Kooooo00?o0000oooo
ooooool0Eoooool4o`3o0?ooooooI_ooool00ol0003oooooooooo`08ooooo`004Oooool2o`0000Wo
oooo0ol00007ooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000oooooooo
ool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03o`000?oo
oooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo
00?o0000ooooooooool02?ooool002Kooooo0ol0001Gooooo`Co0?l0oooooomUooooo`;o00002_oo
ool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0003oooooooooo`08
ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03o`000?oooooooooo
00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000oooooooo
ool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03o`000?oo
oooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo
00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_oo
ool00ol0003oooooooooo`08ooooo`009_ooool3o`0005Oooooo1?l0o`3oooooofGooooo0_l0000:
ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03o`000?oooooooooo
00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000oooooooo
ool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03o`000?oo
oooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo
00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_oo
ool00ol0003oooooooooo`08ooooo`009_ooool3o`0005Oooooo1?l0o`3oooooofGooooo0_l0000:
ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03o`000?oooooooooo
00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000oooooooo
ool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03o`000?oo
oooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo
00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_oo
ool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomV
ooooo`03o`000?oooooooooo00Sooooo000Vooooo`?o0000Eoooool4o`3o0?ooooooIOooool2o`00
00[ooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000oooooooo
ool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03o`000?oo
oooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo
00?o0000ooooooooool02?ooool000_ooooo0_l00004ooooo`04o`000?ooooooooooooooo`Go0000
0_ooool3o`0000Oooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0003ooooo
ooooo`08ooooo`002_ooool01?l0003oooooooooool00008ooooo`06o`000?ooooooooooo`000?oo
oooo00000oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo05Oooooo1?l0o`3ooooo
ofKooooo00?o0000ooooooooool02?ooool000[ooooo00Co0000oooooooooooo00002Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo1?l0001Fooooo`Co0?l0oooooomSoooo
o`Co00002_ooool000[ooooo00Co0000oooooooooooo00002_ooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0Eoooool4o`3o0?ooooooI_ooool00ol0
003oooooooooo`08ooooo`002_ooool01?l0003oooooooooool00007ooooo`06o`000?oooooooooo
ooooool0003ooooo1?l00007ooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKooooo00?o
0000ooooooooool02?ooool000[ooooo00Co0000oooooooooooo00001oooool01ol0003ooooooooo
oooooooo0000ooooool000002_ooool00ol0003oooooooooo`3ooooool7ooooo00?o0000oooooooo
ool02?ooool000_ooooo0_l00009ooooo`?o00000_ooool5o`0000Kooooo00?o0000ooooooooool0
ooooooo1ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo0?oooooo`Ooo
ool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooo
oooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooo
oooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooo
oooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooo
oooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooo
oooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooo
oooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooo
oooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooo
oooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooo
oooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooo
oooo00_ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`08ooooo`009_ooool00ol0
003oooooooooo`06ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0
003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0
003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0
003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0
003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0
003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0
003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0
003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0
003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0
003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0
003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_o
oooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`08ooooo`009_ooooooo`000<Go0000
2_ooool00?oooooomOooool00?oooooomOooool00?oooooomOooool00?oooooomOooool00?oooooo
mOooool00?oooooomOooool00?oooooomOooool002gooooo0ol0000:ooooo`Go00002_ooool3o`00
00cooooo0ol0000:ooooo`?o00002oooool3o`0000cooooo00?o0000ooooooooool02_ooool3o`00
00_ooooo0_l00009ooooo`?o00001?ooool2o`0000Gooooo0ol00003ooooo`?o00001Oooool3o`00
00;ooooo1Ol00004ooooo`?o00000oooool3o`0000Gooooo0ol00004ooooo`?o00001?ooool3o`00
00?ooooo0ol00005ooooo`?o00000oooool3o`0000Gooooo0ol00004ooooo`03o`000?oooooooooo
00Cooooo0ol00003ooooo`?o00001Oooool3o`0000?ooooo0_l00005ooooo`Go00000oooool2o`00
00Cooooo1Ol00002ooooo`?o00001?ooool5o`000003ooooool0003o000000?o00000oooool5o`00
00;ooooo0ol00004ooooo`Go00000oooool3o`0000?ooooo1Ol00002ooooo`?o00001?ooool5o`00
00;ooooo0ol00004ooooo`Go00000oooool00ol0003oooooooooo`03ooooo`Go00000_ooool3o`00
00Cooooo1Ol00002ooooo`;o00001_ooool3o`0000Cooooo0_l00005ooooo`?o00000oooool3o`00
00Gooooo0ol00002ooooo`Go00003_ooool002kooooo00?o0000ooooooooool02_ooool01?l0003o
ooooooooool00009ooooo`05o`000?ooooooooooooooool000003?ooool00ol0003oooooooooo`08
ooooo`05o`000?ooooooooooooooool000002Oooool01Ol0003ooooooooooooooooo000000_ooooo
00?o0000ooooooooool02Oooool01Ol0003ooooooooooooooooo000000cooooo00?o0000oooooooo
ool01oooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;ooooo00Co0000oooo
oooooooo00001Oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00Go0000oooooooooooooooo
o`000003ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03
o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`05
o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool00oooool01?l0003oooooooooool00002ooooo`04o`000?oooooo
ooooo`0000Cooooo00Co0000oooooooooooo00000oooool00ol0003oooooooooo`04ooooo`04o`00
0?ooooooooooo`0000;ooooo00Co0000oooooooooooo00001?ooool01_l0003oooooooooool0003o
ooooo`0000?ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00004ooooo`03o`00
0?oooooooooo00?ooooo00Ko0000oooooooooooo0000ooooool00003ooooo`03o`000?oooooooooo
00;ooooo00Ko0000oooooooooooo0000ooooool00003ooooo`03o`000?oooooooooo00;ooooo00Co
0000oooooooooooo00000oooool00ol0003oooooooooo`04ooooo`06o`000?ooooooooooo`000?oo
oooo00000oooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000
ooooooooool00_ooool01Ol0003ooooooooooooooooo000000;ooooo00Co0000oooooooooooo0000
0oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool00oooool01Ol0003o
oooooooooooooooo000000;ooooo00Co0000oooooooooooo00003_ooool002kooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo00Oooooo1Ol0000=oooo
o`03o`000?oooooooooo00Oooooo00Go0000ooooooooooooooooo`00000<ooooo`03o`000?oooooo
oooo00Sooooo00Go0000ooooooooooooooooo`00000=ooooo`03o`000?oooooooooo00Kooooo00?o
0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`03o`000?oooooooooo00?ooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`03ooooo`04o`000?ooooooooooooooo`Go
00001Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00?ooooo00Go0000oooooooo
ooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?ooool00ol0
003oooooooooo`04ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02
ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool00oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00?ooooo00Co0000oooooooooooooooo1Ol00005ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000?o
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Co
oooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`05ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000
ooooooooool01Oooool01Ol0003ooooooooooooooooo000000Wooooo00Go0000oooooooooooooooo
o`00000@ooooo`00;_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00[ooooo0_l0
000:ooooo`04o`000?ooooooooooo`0000kooooo00?o0000ooooooooool01oooool4o`0000gooooo
00?o0000ooooooooool02Oooool3o`0000_ooooo1?l00008ooooo`03o`000?oooooooooo00;ooooo
00Co0000oooooooooooo00001Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00?o
oooo0_l00006ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00oooool01?l0003oooooooooooooool4
o`0000Kooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooo
oooo00;ooooo0ol00006ooooo`03o`000?oooooooooo00;ooooo1?l00006ooooo`05o`000?oooooo
ooooooooool000000_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00;ooooo00?o
0000ooooooooool01_ooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool00_ooool2o`0000Oooooo00Co0000oooooooooooo00000_ooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`04o`0000Oooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`04o`00
0?ooooooooooooooo`?o00001oooool01?l0003oooooooooooooool4o`0000Gooooo0_l00003oooo
o`04o`000?ooooooooooo`0000Gooooo0_l00004ooooo`03o`000?oooooooooo00Gooooo0_l00005
ooooo`03o`000?oooooooooo00gooooo000^ooooo`03o`000?oooooooooo00Wooooo00Go0000oooo
ooooooooooooo`00000=ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooool000002_ooool4
o`0000[ooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`07ooooo`05o`000?oooooo
ooooooooool000002Oooool01Ol0003ooooooooooooooooo000000Sooooo00?o0000ooooooooool0
0_ooool01?l0003oooooooooool00005ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool01Oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool00oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00_ooool00ol0
003oooooo`000006ooooo`04o`000?ooooooooooooooo`Co00001_ooool01Ol0003ooooooooooooo
oooo000000Wooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`03ooooo`05o`000?oo
ooooooooooooool000000oooool00ol0003oooooooooo`03ooooo`05o`000?ooooooooooooooool0
00000oooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000_ooool0
1Ol0003ooooooooooooooooo000000?ooooo00Go0000ooooooooooooooooo`000005ooooo`07o`00
0?ooooooooooooooool0003oooooo`000003ooooo`05o`000?ooooooooooooooool000000oooool0
0ol0003oooooooooo`03ooooo`05o`000?ooooooooooooooool000000oooool01_l0003ooooooooo
ool0003oooooo`0000Cooooo00Ko0000ooooooooooooooooo`000?ooool4o`0000Cooooo00Oo0000
ooooooooooooooooo`000?oooooo000000Oooooo00Go0000ooooooooooooooooo`000005ooooo`05
o`000?ooooooooooooooool000000oooool00ol0003oooooo`000003ooooo`05o`000?oooooooooo
ooooool000000oooool00ol0003oooooo`000003ooooo`03o`000?oooooooooo00Gooooo00Co0000
oooooooooooo00000_ooool00ol0003oooooooooo`05ooooo`05o`000?ooooooooooooooool00000
2Oooool00ol0003oooooo`000003ooooo`03o`000?oooooooooo00cooooo000]ooooo`;o00002ooo
ool01Ol0003ooooooooooooooooo000000Wooooo00Go0000ooooooooooooooooo`00000;ooooo`;o
00002_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00[ooooo00Go0000oooooooo
ooooooooo`000009ooooo`05o`000?ooooooooooooooool000002Oooool01Ol0003ooooooooooooo
oooo000000Oooooo0_l00004ooooo`04o`000?ooooooooooo`0000Cooooo0_l00004ooooo`;o0000
1_ooool2o`0000?ooooo00Go0000ooooooooooooooooo`000004ooooo`;o00000oooool01Ol0003o
oooooooooooooooo000000Cooooo0_l00005ooooo`;o00001Oooool2o`0000?ooooo00?o0000oooo
ooooool01_ooool2o`0000Cooooo00?o0000ooooooooool01Oooool2o`0000?ooooo00Go0000oooo
ooooooooooooo`000004ooooo`;o00000oooool01Ol0003ooooooooooooooooo000000Cooooo0_l0
0003ooooo`05o`000?ooooooooooooooool000000oooool01Ol0003ooooooooooooooooo000000;o
oooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000;ooooo0_l00005
ooooo`07o`000?ooooooooooooooool0003oooooo`000003ooooo`05o`000?ooooooooooooooool0
00000oooool00ol0003oooooo`000003ooooo`05o`000?ooooooooooooooool000000oooool01?l0
003oooooooooooooool2o`0000Cooooo00Oo0000ooooooooooooooooo`000?oooooo000000Oooooo
00Go0000ooooooooooooooooo`000002ooooo`03o`000?oooooooooo00Cooooo00Oo0000oooooooo
ooooooooo`000?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oo
oooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo000000?ooooo
00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`0000;ooooo00Go0000oooo
ooooooooooooo`000003ooooo`03o`000?oooooooooo00;o00001Oooool01ol0003ooooooooooooo
oooo0000ooooool000000oooool00ol0003oooooooooo`0<ooooo`00;_ooool00ol0003ooooooooo
o`0:ooooo`?o00002oooool3o`0000gooooo00?o0000ooooooooool02?ooool5o`0000_ooooo0_l0
000:ooooo`Go00002_ooool3o`0000_ooooo0ol00009ooooo`03o`000?oooooooooo00?ooooo0_l0
0006ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`02ooooo`?o00001_ooool00ol0003oooooooooo`02ooooo`?o00001_ooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooooooo1Ol00005ooooo`03o`00
0?oooooooooo00?ooooo0_l00006ooooo`04o`000?ooooooooooooooo`Go00001Oooool00ol0003o
ooooooooo`02ooooo`?o00001_ooool00ol0003oooooooooo`02ooooo`?o00001Oooool3o`0000Co
oooo0_l00005ooooo`?o00001?ooool00ol0003oooooooooo`04ooooo`?o00000oooool3o`0000Go
oooo0ol00003ooooo`?o00001Oooool3o`0000Gooooo00?o0000ooooooooool00oooool3o`0000;o
oooo1Ol00004ooooo`?o00001?ooool2o`0000Gooooo0ol00002ooooo`Go00001?ooool3o`0000?o
oooo0ol00005ooooo`?o00000oooool3o`0000Gooooo0ol00004ooooo`;o00001Oooool3o`0000Co
oooo00?o0000ooooooooool01?ooool3o`0000?ooooo0ol0000?ooooo`00\
\>"],
  ImageRangeCache->{{{0, 499}, {308, 0}} -> {-2.06768, -0.0201984, 0.0713992, \
0.000949394}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
%%ImageSize: 500 309.017 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0217391 0.0310559 0.0147151 2.4374 [
[.01863 -0.0125 -3 -9 ]
[.01863 -0.0125 3 0 ]
[.04969 -0.0125 -3 -9 ]
[.04969 -0.0125 3 0 ]
[.08075 -0.0125 -3 -9 ]
[.08075 -0.0125 3 0 ]
[.1118 -0.0125 -3 -9 ]
[.1118 -0.0125 3 0 ]
[.14286 -0.0125 -3 -9 ]
[.14286 -0.0125 3 0 ]
[.17391 -0.0125 -3 -9 ]
[.17391 -0.0125 3 0 ]
[.20497 -0.0125 -3 -9 ]
[.20497 -0.0125 3 0 ]
[.23602 -0.0125 -3 -9 ]
[.23602 -0.0125 3 0 ]
[.26708 -0.0125 -3 -9 ]
[.26708 -0.0125 3 0 ]
[.29814 -0.0125 -6 -9 ]
[.29814 -0.0125 6 0 ]
[.32919 -0.0125 -6 -9 ]
[.32919 -0.0125 6 0 ]
[.36025 -0.0125 -6 -9 ]
[.36025 -0.0125 6 0 ]
[.3913 -0.0125 -6 -9 ]
[.3913 -0.0125 6 0 ]
[.42236 -0.0125 -6 -9 ]
[.42236 -0.0125 6 0 ]
[.45342 -0.0125 -6 -9 ]
[.45342 -0.0125 6 0 ]
[.48447 -0.0125 -6 -9 ]
[.48447 -0.0125 6 0 ]
[.51553 -0.0125 -6 -9 ]
[.51553 -0.0125 6 0 ]
[.54658 -0.0125 -6 -9 ]
[.54658 -0.0125 6 0 ]
[.57764 -0.0125 -6 -9 ]
[.57764 -0.0125 6 0 ]
[.6087 -0.0125 -6 -9 ]
[.6087 -0.0125 6 0 ]
[.63975 -0.0125 -6 -9 ]
[.63975 -0.0125 6 0 ]
[.67081 -0.0125 -6 -9 ]
[.67081 -0.0125 6 0 ]
[.70186 -0.0125 -6 -9 ]
[.70186 -0.0125 6 0 ]
[.73292 -0.0125 -6 -9 ]
[.73292 -0.0125 6 0 ]
[.76398 -0.0125 -6 -9 ]
[.76398 -0.0125 6 0 ]
[.79503 -0.0125 -6 -9 ]
[.79503 -0.0125 6 0 ]
[.82609 -0.0125 -6 -9 ]
[.82609 -0.0125 6 0 ]
[.85714 -0.0125 -6 -9 ]
[.85714 -0.0125 6 0 ]
[.8882 -0.0125 -6 -9 ]
[.8882 -0.0125 6 0 ]
[.91925 -0.0125 -6 -9 ]
[.91925 -0.0125 6 0 ]
[.95031 -0.0125 -6 -9 ]
[.95031 -0.0125 6 0 ]
[.98137 -0.0125 -6 -9 ]
[.98137 -0.0125 6 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .01472 -6 -4.5 ]
[-0.0125 .01472 0 4.5 ]
[-0.0125 .13659 -24 -4.5 ]
[-0.0125 .13659 0 4.5 ]
[-0.0125 .25846 -18 -4.5 ]
[-0.0125 .25846 0 4.5 ]
[-0.0125 .38033 -24 -4.5 ]
[-0.0125 .38033 0 4.5 ]
[-0.0125 .5022 -18 -4.5 ]
[-0.0125 .5022 0 4.5 ]
[ 0 0 -0.125 0 ]
[.01863 .63053 -3 0 ]
[.01863 .63053 3 9 ]
[.04969 .63053 -3 0 ]
[.04969 .63053 3 9 ]
[.08075 .63053 -3 0 ]
[.08075 .63053 3 9 ]
[.1118 .63053 -3 0 ]
[.1118 .63053 3 9 ]
[.14286 .63053 -3 0 ]
[.14286 .63053 3 9 ]
[.17391 .63053 -3 0 ]
[.17391 .63053 3 9 ]
[.20497 .63053 -3 0 ]
[.20497 .63053 3 9 ]
[.23602 .63053 -3 0 ]
[.23602 .63053 3 9 ]
[.26708 .63053 -3 0 ]
[.26708 .63053 3 9 ]
[.29814 .63053 -6 0 ]
[.29814 .63053 6 9 ]
[.32919 .63053 -6 0 ]
[.32919 .63053 6 9 ]
[.36025 .63053 -6 0 ]
[.36025 .63053 6 9 ]
[.3913 .63053 -6 0 ]
[.3913 .63053 6 9 ]
[.42236 .63053 -6 0 ]
[.42236 .63053 6 9 ]
[.45342 .63053 -6 0 ]
[.45342 .63053 6 9 ]
[.48447 .63053 -6 0 ]
[.48447 .63053 6 9 ]
[.51553 .63053 -6 0 ]
[.51553 .63053 6 9 ]
[.54658 .63053 -6 0 ]
[.54658 .63053 6 9 ]
[.57764 .63053 -6 0 ]
[.57764 .63053 6 9 ]
[.6087 .63053 -6 0 ]
[.6087 .63053 6 9 ]
[.63975 .63053 -6 0 ]
[.63975 .63053 6 9 ]
[.67081 .63053 -6 0 ]
[.67081 .63053 6 9 ]
[.70186 .63053 -6 0 ]
[.70186 .63053 6 9 ]
[.73292 .63053 -6 0 ]
[.73292 .63053 6 9 ]
[.76398 .63053 -6 0 ]
[.76398 .63053 6 9 ]
[.79503 .63053 -6 0 ]
[.79503 .63053 6 9 ]
[.82609 .63053 -6 0 ]
[.82609 .63053 6 9 ]
[.85714 .63053 -6 0 ]
[.85714 .63053 6 9 ]
[.8882 .63053 -6 0 ]
[.8882 .63053 6 9 ]
[.91925 .63053 -6 0 ]
[.91925 .63053 6 9 ]
[.95031 .63053 -6 0 ]
[.95031 .63053 6 9 ]
[.98137 .63053 -6 0 ]
[.98137 .63053 6 9 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.01863 0 m
.01863 .00625 L
s
[(1)] .01863 -0.0125 0 1 Mshowa
.04969 0 m
.04969 .00625 L
s
[(2)] .04969 -0.0125 0 1 Mshowa
.08075 0 m
.08075 .00625 L
s
[(3)] .08075 -0.0125 0 1 Mshowa
.1118 0 m
.1118 .00625 L
s
[(4)] .1118 -0.0125 0 1 Mshowa
.14286 0 m
.14286 .00625 L
s
[(5)] .14286 -0.0125 0 1 Mshowa
.17391 0 m
.17391 .00625 L
s
[(6)] .17391 -0.0125 0 1 Mshowa
.20497 0 m
.20497 .00625 L
s
[(7)] .20497 -0.0125 0 1 Mshowa
.23602 0 m
.23602 .00625 L
s
[(8)] .23602 -0.0125 0 1 Mshowa
.26708 0 m
.26708 .00625 L
s
[(9)] .26708 -0.0125 0 1 Mshowa
.29814 0 m
.29814 .00625 L
s
[(10)] .29814 -0.0125 0 1 Mshowa
.32919 0 m
.32919 .00625 L
s
[(11)] .32919 -0.0125 0 1 Mshowa
.36025 0 m
.36025 .00625 L
s
[(12)] .36025 -0.0125 0 1 Mshowa
.3913 0 m
.3913 .00625 L
s
[(13)] .3913 -0.0125 0 1 Mshowa
.42236 0 m
.42236 .00625 L
s
[(14)] .42236 -0.0125 0 1 Mshowa
.45342 0 m
.45342 .00625 L
s
[(15)] .45342 -0.0125 0 1 Mshowa
.48447 0 m
.48447 .00625 L
s
[(16)] .48447 -0.0125 0 1 Mshowa
.51553 0 m
.51553 .00625 L
s
[(17)] .51553 -0.0125 0 1 Mshowa
.54658 0 m
.54658 .00625 L
s
[(18)] .54658 -0.0125 0 1 Mshowa
.57764 0 m
.57764 .00625 L
s
[(19)] .57764 -0.0125 0 1 Mshowa
.6087 0 m
.6087 .00625 L
s
[(20)] .6087 -0.0125 0 1 Mshowa
.63975 0 m
.63975 .00625 L
s
[(21)] .63975 -0.0125 0 1 Mshowa
.67081 0 m
.67081 .00625 L
s
[(22)] .67081 -0.0125 0 1 Mshowa
.70186 0 m
.70186 .00625 L
s
[(23)] .70186 -0.0125 0 1 Mshowa
.73292 0 m
.73292 .00625 L
s
[(24)] .73292 -0.0125 0 1 Mshowa
.76398 0 m
.76398 .00625 L
s
[(25)] .76398 -0.0125 0 1 Mshowa
.79503 0 m
.79503 .00625 L
s
[(26)] .79503 -0.0125 0 1 Mshowa
.82609 0 m
.82609 .00625 L
s
[(27)] .82609 -0.0125 0 1 Mshowa
.85714 0 m
.85714 .00625 L
s
[(28)] .85714 -0.0125 0 1 Mshowa
.8882 0 m
.8882 .00625 L
s
[(29)] .8882 -0.0125 0 1 Mshowa
.91925 0 m
.91925 .00625 L
s
[(30)] .91925 -0.0125 0 1 Mshowa
.95031 0 m
.95031 .00625 L
s
[(31)] .95031 -0.0125 0 1 Mshowa
.98137 0 m
.98137 .00625 L
s
[(32)] .98137 -0.0125 0 1 Mshowa
0 0 m
1 0 L
s
0 .01472 m
.00625 .01472 L
s
[(0)] -0.0125 .01472 1 0 Mshowa
0 .13659 m
.00625 .13659 L
s
[(0.05)] -0.0125 .13659 1 0 Mshowa
0 .25846 m
.00625 .25846 L
s
[(0.1)] -0.0125 .25846 1 0 Mshowa
0 .38033 m
.00625 .38033 L
s
[(0.15)] -0.0125 .38033 1 0 Mshowa
0 .5022 m
.00625 .5022 L
s
[(0.2)] -0.0125 .5022 1 0 Mshowa
.125 Mabswid
0 .03909 m
.00375 .03909 L
s
0 .06346 m
.00375 .06346 L
s
0 .08784 m
.00375 .08784 L
s
0 .11221 m
.00375 .11221 L
s
0 .16096 m
.00375 .16096 L
s
0 .18533 m
.00375 .18533 L
s
0 .20971 m
.00375 .20971 L
s
0 .23408 m
.00375 .23408 L
s
0 .28283 m
.00375 .28283 L
s
0 .3072 m
.00375 .3072 L
s
0 .33158 m
.00375 .33158 L
s
0 .35595 m
.00375 .35595 L
s
0 .4047 m
.00375 .4047 L
s
0 .42907 m
.00375 .42907 L
s
0 .45345 m
.00375 .45345 L
s
0 .47782 m
.00375 .47782 L
s
0 .52657 m
.00375 .52657 L
s
0 .55094 m
.00375 .55094 L
s
0 .57532 m
.00375 .57532 L
s
0 .59969 m
.00375 .59969 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.01863 .61178 m
.01863 .61803 L
s
[(1)] .01863 .63053 0 -1 Mshowa
.04969 .61178 m
.04969 .61803 L
s
[(2)] .04969 .63053 0 -1 Mshowa
.08075 .61178 m
.08075 .61803 L
s
[(3)] .08075 .63053 0 -1 Mshowa
.1118 .61178 m
.1118 .61803 L
s
[(4)] .1118 .63053 0 -1 Mshowa
.14286 .61178 m
.14286 .61803 L
s
[(5)] .14286 .63053 0 -1 Mshowa
.17391 .61178 m
.17391 .61803 L
s
[(6)] .17391 .63053 0 -1 Mshowa
.20497 .61178 m
.20497 .61803 L
s
[(7)] .20497 .63053 0 -1 Mshowa
.23602 .61178 m
.23602 .61803 L
s
[(8)] .23602 .63053 0 -1 Mshowa
.26708 .61178 m
.26708 .61803 L
s
[(9)] .26708 .63053 0 -1 Mshowa
.29814 .61178 m
.29814 .61803 L
s
[(10)] .29814 .63053 0 -1 Mshowa
.32919 .61178 m
.32919 .61803 L
s
[(11)] .32919 .63053 0 -1 Mshowa
.36025 .61178 m
.36025 .61803 L
s
[(12)] .36025 .63053 0 -1 Mshowa
.3913 .61178 m
.3913 .61803 L
s
[(13)] .3913 .63053 0 -1 Mshowa
.42236 .61178 m
.42236 .61803 L
s
[(14)] .42236 .63053 0 -1 Mshowa
.45342 .61178 m
.45342 .61803 L
s
[(15)] .45342 .63053 0 -1 Mshowa
.48447 .61178 m
.48447 .61803 L
s
[(16)] .48447 .63053 0 -1 Mshowa
.51553 .61178 m
.51553 .61803 L
s
[(17)] .51553 .63053 0 -1 Mshowa
.54658 .61178 m
.54658 .61803 L
s
[(18)] .54658 .63053 0 -1 Mshowa
.57764 .61178 m
.57764 .61803 L
s
[(19)] .57764 .63053 0 -1 Mshowa
.6087 .61178 m
.6087 .61803 L
s
[(20)] .6087 .63053 0 -1 Mshowa
.63975 .61178 m
.63975 .61803 L
s
[(21)] .63975 .63053 0 -1 Mshowa
.67081 .61178 m
.67081 .61803 L
s
[(22)] .67081 .63053 0 -1 Mshowa
.70186 .61178 m
.70186 .61803 L
s
[(23)] .70186 .63053 0 -1 Mshowa
.73292 .61178 m
.73292 .61803 L
s
[(24)] .73292 .63053 0 -1 Mshowa
.76398 .61178 m
.76398 .61803 L
s
[(25)] .76398 .63053 0 -1 Mshowa
.79503 .61178 m
.79503 .61803 L
s
[(26)] .79503 .63053 0 -1 Mshowa
.82609 .61178 m
.82609 .61803 L
s
[(27)] .82609 .63053 0 -1 Mshowa
.85714 .61178 m
.85714 .61803 L
s
[(28)] .85714 .63053 0 -1 Mshowa
.8882 .61178 m
.8882 .61803 L
s
[(29)] .8882 .63053 0 -1 Mshowa
.91925 .61178 m
.91925 .61803 L
s
[(30)] .91925 .63053 0 -1 Mshowa
.95031 .61178 m
.95031 .61803 L
s
[(31)] .95031 .63053 0 -1 Mshowa
.98137 .61178 m
.98137 .61803 L
s
[(32)] .98137 .63053 0 -1 Mshowa
0 .61803 m
1 .61803 L
s
.99375 .01472 m
1 .01472 L
s
.99375 .13659 m
1 .13659 L
s
.99375 .25846 m
1 .25846 L
s
.99375 .38033 m
1 .38033 L
s
.99375 .5022 m
1 .5022 L
s
.125 Mabswid
.99625 .03909 m
1 .03909 L
s
.99625 .06346 m
1 .06346 L
s
.99625 .08784 m
1 .08784 L
s
.99625 .11221 m
1 .11221 L
s
.99625 .16096 m
1 .16096 L
s
.99625 .18533 m
1 .18533 L
s
.99625 .20971 m
1 .20971 L
s
.99625 .23408 m
1 .23408 L
s
.99625 .28283 m
1 .28283 L
s
.99625 .3072 m
1 .3072 L
s
.99625 .33158 m
1 .33158 L
s
.99625 .35595 m
1 .35595 L
s
.99625 .4047 m
1 .4047 L
s
.99625 .42907 m
1 .42907 L
s
.99625 .45345 m
1 .45345 L
s
.99625 .47782 m
1 .47782 L
s
.99625 .52657 m
1 .52657 L
s
.99625 .55094 m
1 .55094 L
s
.99625 .57532 m
1 .57532 L
s
.99625 .59969 m
1 .59969 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .01472 m
1 .01472 L
s
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.00621 .01472 m
.00621 .18325 L
.01242 .18325 L
.01242 .01472 L
F
.5 Mabswid
.00621 .01472 m
.00621 .18325 L
.01242 .18325 L
.01242 .01472 L
.00621 .01472 L
s
.03727 .01472 m
.03727 .01643 L
.04348 .01643 L
.04348 .01472 L
F
.03727 .01472 m
.03727 .01643 L
.04348 .01643 L
.04348 .01472 L
.03727 .01472 L
s
.06832 .01472 m
.06832 .02241 L
.07453 .02241 L
.07453 .01472 L
F
.06832 .01472 m
.06832 .02241 L
.07453 .02241 L
.07453 .01472 L
.06832 .01472 L
s
.09938 .01472 m
.09938 .19352 L
.10559 .19352 L
.10559 .01472 L
F
.09938 .01472 m
.09938 .19352 L
.10559 .19352 L
.10559 .01472 L
.09938 .01472 L
s
.13043 .01472 m
.13043 .01472 L
.13665 .01472 L
.13665 .01472 L
F
.13043 .01472 m
.13043 .01472 L
.13665 .01472 L
.13665 .01472 L
.13043 .01472 L
s
.16149 .01472 m
.16149 .01557 L
.1677 .01557 L
.1677 .01472 L
F
.16149 .01472 m
.16149 .01557 L
.1677 .01557 L
.1677 .01472 L
.16149 .01472 L
s
.19255 .01472 m
.19255 .01472 L
.19876 .01472 L
.19876 .01472 L
F
.19255 .01472 m
.19255 .01472 L
.19876 .01472 L
.19876 .01472 L
.19255 .01472 L
s
.2236 .01472 m
.2236 .16187 L
.22981 .16187 L
.22981 .01472 L
F
.2236 .01472 m
.2236 .16187 L
.22981 .16187 L
.22981 .01472 L
.2236 .01472 L
s
.25466 .01472 m
.25466 .17555 L
.26087 .17555 L
.26087 .01472 L
F
.25466 .01472 m
.25466 .17555 L
.26087 .17555 L
.26087 .01472 L
.25466 .01472 L
s
.28571 .01472 m
.28571 .01472 L
.29193 .01472 L
.29193 .01472 L
F
.28571 .01472 m
.28571 .01472 L
.29193 .01472 L
.29193 .01472 L
.28571 .01472 L
s
.31677 .01472 m
.31677 .01472 L
.32298 .01472 L
.32298 .01472 L
F
.31677 .01472 m
.31677 .01472 L
.32298 .01472 L
.32298 .01472 L
.31677 .01472 L
s
.34783 .01472 m
.34783 .24999 L
.35404 .24999 L
.35404 .01472 L
F
.34783 .01472 m
.34783 .24999 L
.35404 .24999 L
.35404 .01472 L
.34783 .01472 L
s
.37888 .01472 m
.37888 .02156 L
.38509 .02156 L
.38509 .01472 L
F
.37888 .01472 m
.37888 .02156 L
.38509 .02156 L
.38509 .01472 L
.37888 .01472 L
s
.40994 .01472 m
.40994 .20464 L
.41615 .20464 L
.41615 .01472 L
F
.40994 .01472 m
.40994 .20464 L
.41615 .20464 L
.41615 .01472 L
.40994 .01472 L
s
.44099 .01472 m
.44099 .01472 L
.4472 .01472 L
.4472 .01472 L
F
.44099 .01472 m
.44099 .01472 L
.4472 .01472 L
.4472 .01472 L
.44099 .01472 L
s
.47205 .01472 m
.47205 .16015 L
.47826 .16015 L
.47826 .01472 L
F
.47205 .01472 m
.47205 .16015 L
.47826 .16015 L
.47826 .01472 L
.47205 .01472 L
s
.50311 .01472 m
.50311 .18924 L
.50932 .18924 L
.50932 .01472 L
F
.50311 .01472 m
.50311 .18924 L
.50932 .18924 L
.50932 .01472 L
.50311 .01472 L
s
.53416 .01472 m
.53416 .01557 L
.54037 .01557 L
.54037 .01472 L
F
.53416 .01472 m
.53416 .01557 L
.54037 .01557 L
.54037 .01472 L
.53416 .01472 L
s
.56522 .01472 m
.56522 .01814 L
.57143 .01814 L
.57143 .01472 L
F
.56522 .01472 m
.56522 .01814 L
.57143 .01814 L
.57143 .01472 L
.56522 .01472 L
s
.59627 .01472 m
.59627 .16614 L
.60248 .16614 L
.60248 .01472 L
F
.59627 .01472 m
.59627 .16614 L
.60248 .16614 L
.60248 .01472 L
.59627 .01472 L
s
.62733 .01472 m
.62733 .01557 L
.63354 .01557 L
.63354 .01472 L
F
.62733 .01472 m
.62733 .01557 L
.63354 .01557 L
.63354 .01472 L
.62733 .01472 L
s
.65839 .01472 m
.65839 .15673 L
.6646 .15673 L
.6646 .01472 L
F
.65839 .01472 m
.65839 .15673 L
.6646 .15673 L
.6646 .01472 L
.65839 .01472 L
s
.68944 .01472 m
.68944 .01472 L
.69565 .01472 L
.69565 .01472 L
F
.68944 .01472 m
.68944 .01472 L
.69565 .01472 L
.69565 .01472 L
.68944 .01472 L
s
.7205 .01472 m
.7205 .19523 L
.72671 .19523 L
.72671 .01472 L
F
.7205 .01472 m
.7205 .19523 L
.72671 .19523 L
.72671 .01472 L
.7205 .01472 L
s
.75155 .01472 m
.75155 .01985 L
.75776 .01985 L
.75776 .01472 L
F
.75155 .01472 m
.75155 .01985 L
.75776 .01985 L
.75776 .01472 L
.75155 .01472 L
s
.78261 .01472 m
.78261 .01557 L
.78882 .01557 L
.78882 .01472 L
F
.78261 .01472 m
.78261 .01557 L
.78882 .01557 L
.78882 .01472 L
.78261 .01472 L
s
.81366 .01472 m
.81366 .01814 L
.81988 .01814 L
.81988 .01472 L
F
.81366 .01472 m
.81366 .01814 L
.81988 .01814 L
.81988 .01472 L
.81366 .01472 L
s
.84472 .01472 m
.84472 .18582 L
.85093 .18582 L
.85093 .01472 L
F
.84472 .01472 m
.84472 .18582 L
.85093 .18582 L
.85093 .01472 L
.84472 .01472 L
s
.87578 .01472 m
.87578 .02327 L
.88199 .02327 L
.88199 .01472 L
F
.87578 .01472 m
.87578 .02327 L
.88199 .02327 L
.88199 .01472 L
.87578 .01472 L
s
.90683 .01472 m
.90683 .16871 L
.91304 .16871 L
.91304 .01472 L
F
.90683 .01472 m
.90683 .16871 L
.91304 .16871 L
.91304 .01472 L
.90683 .01472 L
s
.93789 .01472 m
.93789 .01472 L
.9441 .01472 L
.9441 .01472 L
F
.93789 .01472 m
.93789 .01472 L
.9441 .01472 L
.9441 .01472 L
.93789 .01472 L
s
.96894 .01472 m
.96894 .21234 L
.97516 .21234 L
.97516 .01472 L
F
.96894 .01472 m
.96894 .21234 L
.97516 .21234 L
.97516 .01472 L
.96894 .01472 L
s
0 1 0 r
.01242 .01472 m
.01242 .01643 L
.01863 .01643 L
.01863 .01472 L
F
.01242 .01472 m
.01242 .01643 L
.01863 .01643 L
.01863 .01472 L
.01242 .01472 L
s
.04348 .01472 m
.04348 .01814 L
.04969 .01814 L
.04969 .01472 L
F
.04348 .01472 m
.04348 .01814 L
.04969 .01814 L
.04969 .01472 L
.04348 .01472 L
s
.07453 .01472 m
.07453 .28677 L
.08075 .28677 L
.08075 .01472 L
F
.07453 .01472 m
.07453 .28677 L
.08075 .28677 L
.08075 .01472 L
.07453 .01472 L
s
.10559 .01472 m
.10559 .01557 L
.1118 .01557 L
.1118 .01472 L
F
.10559 .01472 m
.10559 .01557 L
.1118 .01557 L
.1118 .01472 L
.10559 .01472 L
s
.13665 .01472 m
.13665 .1978 L
.14286 .1978 L
.14286 .01472 L
F
.13665 .01472 m
.13665 .1978 L
.14286 .1978 L
.14286 .01472 L
.13665 .01472 L
s
.1677 .01472 m
.1677 .01728 L
.17391 .01728 L
.17391 .01472 L
F
.1677 .01472 m
.1677 .01728 L
.17391 .01728 L
.17391 .01472 L
.1677 .01472 L
s
.19876 .01472 m
.19876 .60332 L
.20497 .60332 L
.20497 .01472 L
F
.19876 .01472 m
.19876 .60332 L
.20497 .60332 L
.20497 .01472 L
.19876 .01472 L
s
.22981 .01472 m
.22981 .01557 L
.23602 .01557 L
.23602 .01472 L
F
.22981 .01472 m
.22981 .01557 L
.23602 .01557 L
.23602 .01472 L
.22981 .01472 L
s
.26087 .01472 m
.26087 .01643 L
.26708 .01643 L
.26708 .01472 L
F
.26087 .01472 m
.26087 .01643 L
.26708 .01643 L
.26708 .01472 L
.26087 .01472 L
s
.29193 .01472 m
.29193 .01899 L
.29814 .01899 L
.29814 .01472 L
F
.29193 .01472 m
.29193 .01899 L
.29814 .01899 L
.29814 .01472 L
.29193 .01472 L
s
.32298 .01472 m
.32298 .02413 L
.32919 .02413 L
.32919 .01472 L
F
.32298 .01472 m
.32298 .02413 L
.32919 .02413 L
.32919 .01472 L
.32298 .01472 L
s
.35404 .01472 m
.35404 .01985 L
.36025 .01985 L
.36025 .01472 L
F
.35404 .01472 m
.35404 .01985 L
.36025 .01985 L
.36025 .01472 L
.35404 .01472 L
s
.38509 .01472 m
.38509 .12337 L
.3913 .12337 L
.3913 .01472 L
F
.38509 .01472 m
.38509 .12337 L
.3913 .12337 L
.3913 .01472 L
.38509 .01472 L
s
.41615 .01472 m
.41615 .01985 L
.42236 .01985 L
.42236 .01472 L
F
.41615 .01472 m
.41615 .01985 L
.42236 .01985 L
.42236 .01472 L
.41615 .01472 L
s
.4472 .01472 m
.4472 .24229 L
.45342 .24229 L
.45342 .01472 L
F
.4472 .01472 m
.4472 .24229 L
.45342 .24229 L
.45342 .01472 L
.4472 .01472 L
s
.47826 .01472 m
.47826 .01472 L
.48447 .01472 L
.48447 .01472 L
F
.47826 .01472 m
.47826 .01472 L
.48447 .01472 L
.48447 .01472 L
.47826 .01472 L
s
.50932 .01472 m
.50932 .01899 L
.51553 .01899 L
.51553 .01472 L
F
.50932 .01472 m
.50932 .01899 L
.51553 .01899 L
.51553 .01472 L
.50932 .01472 L
s
.54037 .01472 m
.54037 .01814 L
.54658 .01814 L
.54658 .01472 L
F
.54037 .01472 m
.54037 .01814 L
.54658 .01814 L
.54658 .01472 L
.54037 .01472 L
s
.57143 .01472 m
.57143 .05407 L
.57764 .05407 L
.57764 .01472 L
F
.57143 .01472 m
.57143 .05407 L
.57764 .05407 L
.57764 .01472 L
.57143 .01472 L
s
.60248 .01472 m
.60248 .01899 L
.6087 .01899 L
.6087 .01472 L
F
.60248 .01472 m
.60248 .01899 L
.6087 .01899 L
.6087 .01472 L
.60248 .01472 L
s
.63354 .01472 m
.63354 .20978 L
.63975 .20978 L
.63975 .01472 L
F
.63354 .01472 m
.63354 .20978 L
.63975 .20978 L
.63975 .01472 L
.63354 .01472 L
s
.6646 .01472 m
.6646 .02669 L
.67081 .02669 L
.67081 .01472 L
F
.6646 .01472 m
.6646 .02669 L
.67081 .02669 L
.67081 .01472 L
.6646 .01472 L
s
.69565 .01472 m
.69565 .27565 L
.70186 .27565 L
.70186 .01472 L
F
.69565 .01472 m
.69565 .27565 L
.70186 .27565 L
.70186 .01472 L
.69565 .01472 L
s
.72671 .01472 m
.72671 .01643 L
.73292 .01643 L
.73292 .01472 L
F
.72671 .01472 m
.72671 .01643 L
.73292 .01643 L
.73292 .01472 L
.72671 .01472 L
s
.75776 .01472 m
.75776 .02156 L
.76398 .02156 L
.76398 .01472 L
F
.75776 .01472 m
.75776 .02156 L
.76398 .02156 L
.76398 .01472 L
.75776 .01472 L
s
.78882 .01472 m
.78882 .01899 L
.79503 .01899 L
.79503 .01472 L
F
.78882 .01472 m
.78882 .01899 L
.79503 .01899 L
.79503 .01472 L
.78882 .01472 L
s
.81988 .01472 m
.81988 .03354 L
.82609 .03354 L
.82609 .01472 L
F
.81988 .01472 m
.81988 .03354 L
.82609 .03354 L
.82609 .01472 L
.81988 .01472 L
s
.85093 .01472 m
.85093 .01472 L
.85714 .01472 L
.85714 .01472 L
F
.85093 .01472 m
.85093 .01472 L
.85714 .01472 L
.85714 .01472 L
.85093 .01472 L
s
.88199 .01472 m
.88199 .25769 L
.8882 .25769 L
.8882 .01472 L
F
.88199 .01472 m
.88199 .25769 L
.8882 .25769 L
.8882 .01472 L
.88199 .01472 L
s
.91304 .01472 m
.91304 .01557 L
.91925 .01557 L
.91925 .01472 L
F
.91304 .01472 m
.91304 .01557 L
.91925 .01557 L
.91925 .01472 L
.91304 .01472 L
s
.9441 .01472 m
.9441 .2363 L
.95031 .2363 L
.95031 .01472 L
F
.9441 .01472 m
.9441 .2363 L
.95031 .2363 L
.95031 .01472 L
.9441 .01472 L
s
.97516 .01472 m
.97516 .0207 L
.98137 .0207 L
.98137 .01472 L
F
.97516 .01472 m
.97516 .0207 L
.98137 .0207 L
.98137 .01472 L
.97516 .01472 L
s
0 0 1 r
.01863 .01472 m
.01863 .01472 L
.02484 .01472 L
.02484 .01472 L
F
.01863 .01472 m
.01863 .01472 L
.02484 .01472 L
.02484 .01472 L
.01863 .01472 L
s
.04969 .01472 m
.04969 .01472 L
.0559 .01472 L
.0559 .01472 L
F
.04969 .01472 m
.04969 .01472 L
.0559 .01472 L
.0559 .01472 L
.04969 .01472 L
s
.08075 .01472 m
.08075 .15673 L
.08696 .15673 L
.08696 .01472 L
F
.08075 .01472 m
.08075 .15673 L
.08696 .15673 L
.08696 .01472 L
.08075 .01472 L
s
.1118 .01472 m
.1118 .01472 L
.11801 .01472 L
.11801 .01472 L
F
.1118 .01472 m
.1118 .01472 L
.11801 .01472 L
.11801 .01472 L
.1118 .01472 L
s
.14286 .01472 m
.14286 .2209 L
.14907 .2209 L
.14907 .01472 L
F
.14286 .01472 m
.14286 .2209 L
.14907 .2209 L
.14907 .01472 L
.14286 .01472 L
s
.17391 .01472 m
.17391 .01472 L
.18012 .01472 L
.18012 .01472 L
F
.17391 .01472 m
.17391 .01472 L
.18012 .01472 L
.18012 .01472 L
.17391 .01472 L
s
.20497 .01472 m
.20497 .55284 L
.21118 .55284 L
.21118 .01472 L
F
.20497 .01472 m
.20497 .55284 L
.21118 .55284 L
.21118 .01472 L
.20497 .01472 L
s
.23602 .01472 m
.23602 .01472 L
.24224 .01472 L
.24224 .01472 L
F
.23602 .01472 m
.23602 .01472 L
.24224 .01472 L
.24224 .01472 L
.23602 .01472 L
s
.26708 .01472 m
.26708 .01472 L
.27329 .01472 L
.27329 .01472 L
F
.26708 .01472 m
.26708 .01472 L
.27329 .01472 L
.27329 .01472 L
.26708 .01472 L
s
.29814 .01472 m
.29814 .01472 L
.30435 .01472 L
.30435 .01472 L
F
.29814 .01472 m
.29814 .01472 L
.30435 .01472 L
.30435 .01472 L
.29814 .01472 L
s
.32919 .01472 m
.32919 .01472 L
.3354 .01472 L
.3354 .01472 L
F
.32919 .01472 m
.32919 .01472 L
.3354 .01472 L
.3354 .01472 L
.32919 .01472 L
s
.36025 .01472 m
.36025 .01472 L
.36646 .01472 L
.36646 .01472 L
F
.36025 .01472 m
.36025 .01472 L
.36646 .01472 L
.36646 .01472 L
.36025 .01472 L
s
.3913 .01472 m
.3913 .15759 L
.39752 .15759 L
.39752 .01472 L
F
.3913 .01472 m
.3913 .15759 L
.39752 .15759 L
.39752 .01472 L
.3913 .01472 L
s
.42236 .01472 m
.42236 .01472 L
.42857 .01472 L
.42857 .01472 L
F
.42236 .01472 m
.42236 .01472 L
.42857 .01472 L
.42857 .01472 L
.42236 .01472 L
s
.45342 .01472 m
.45342 .38858 L
.45963 .38858 L
.45963 .01472 L
F
.45342 .01472 m
.45342 .38858 L
.45963 .38858 L
.45963 .01472 L
.45342 .01472 L
s
.48447 .01472 m
.48447 .01472 L
.49068 .01472 L
.49068 .01472 L
F
.48447 .01472 m
.48447 .01472 L
.49068 .01472 L
.49068 .01472 L
.48447 .01472 L
s
.51553 .01472 m
.51553 .01472 L
.52174 .01472 L
.52174 .01472 L
F
.51553 .01472 m
.51553 .01472 L
.52174 .01472 L
.52174 .01472 L
.51553 .01472 L
s
.54658 .01472 m
.54658 .01557 L
.5528 .01557 L
.5528 .01472 L
F
.54658 .01472 m
.54658 .01557 L
.5528 .01557 L
.5528 .01472 L
.54658 .01472 L
s
.57764 .01472 m
.57764 .05835 L
.58385 .05835 L
.58385 .01472 L
F
.57764 .01472 m
.57764 .05835 L
.58385 .05835 L
.58385 .01472 L
.57764 .01472 L
s
.6087 .01472 m
.6087 .01472 L
.61491 .01472 L
.61491 .01472 L
F
.6087 .01472 m
.6087 .01472 L
.61491 .01472 L
.61491 .01472 L
.6087 .01472 L
s
.63975 .01472 m
.63975 .17384 L
.64596 .17384 L
.64596 .01472 L
F
.63975 .01472 m
.63975 .17384 L
.64596 .17384 L
.64596 .01472 L
.63975 .01472 L
s
.67081 .01472 m
.67081 .01557 L
.67702 .01557 L
.67702 .01472 L
F
.67081 .01472 m
.67081 .01557 L
.67702 .01557 L
.67702 .01472 L
.67081 .01472 L
s
.70186 .01472 m
.70186 .42109 L
.70807 .42109 L
.70807 .01472 L
F
.70186 .01472 m
.70186 .42109 L
.70807 .42109 L
.70807 .01472 L
.70186 .01472 L
s
.73292 .01472 m
.73292 .01472 L
.73913 .01472 L
.73913 .01472 L
F
.73292 .01472 m
.73292 .01472 L
.73913 .01472 L
.73913 .01472 L
.73292 .01472 L
s
.76398 .01472 m
.76398 .01472 L
.77019 .01472 L
.77019 .01472 L
F
.76398 .01472 m
.76398 .01472 L
.77019 .01472 L
.77019 .01472 L
.76398 .01472 L
s
.79503 .01472 m
.79503 .01472 L
.80124 .01472 L
.80124 .01472 L
F
.79503 .01472 m
.79503 .01472 L
.80124 .01472 L
.80124 .01472 L
.79503 .01472 L
s
.82609 .01472 m
.82609 .01557 L
.8323 .01557 L
.8323 .01472 L
F
.82609 .01472 m
.82609 .01557 L
.8323 .01557 L
.8323 .01472 L
.82609 .01472 L
s
.85714 .01472 m
.85714 .01472 L
.86335 .01472 L
.86335 .01472 L
F
.85714 .01472 m
.85714 .01472 L
.86335 .01472 L
.86335 .01472 L
.85714 .01472 L
s
.8882 .01472 m
.8882 .17128 L
.89441 .17128 L
.89441 .01472 L
F
.8882 .01472 m
.8882 .17128 L
.89441 .17128 L
.89441 .01472 L
.8882 .01472 L
s
.91925 .01472 m
.91925 .01472 L
.92547 .01472 L
.92547 .01472 L
F
.91925 .01472 m
.91925 .01472 L
.92547 .01472 L
.92547 .01472 L
.91925 .01472 L
s
.95031 .01472 m
.95031 .27993 L
.95652 .27993 L
.95652 .01472 L
F
.95031 .01472 m
.95031 .27993 L
.95652 .27993 L
.95652 .01472 L
.95031 .01472 L
s
.98137 .01472 m
.98137 .01557 L
.98758 .01557 L
.98758 .01472 L
F
.98137 .01472 m
.98137 .01557 L
.98758 .01557 L
.98758 .01472 L
.98137 .01472 L
s
1 .5 0 r
.02484 .01472 m
.02484 .03867 L
.03106 .03867 L
.03106 .01472 L
F
.02484 .01472 m
.02484 .03867 L
.03106 .03867 L
.03106 .01472 L
.02484 .01472 L
s
.0559 .01472 m
.0559 .20208 L
.06211 .20208 L
.06211 .01472 L
F
.0559 .01472 m
.0559 .20208 L
.06211 .20208 L
.06211 .01472 L
.0559 .01472 L
s
.08696 .01472 m
.08696 .06262 L
.09317 .06262 L
.09317 .01472 L
F
.08696 .01472 m
.08696 .06262 L
.09317 .06262 L
.09317 .01472 L
.08696 .01472 L
s
.11801 .01472 m
.11801 .02926 L
.12422 .02926 L
.12422 .01472 L
F
.11801 .01472 m
.11801 .02926 L
.12422 .02926 L
.12422 .01472 L
.11801 .01472 L
s
.14907 .01472 m
.14907 .03867 L
.15528 .03867 L
.15528 .01472 L
F
.14907 .01472 m
.14907 .03867 L
.15528 .03867 L
.15528 .01472 L
.14907 .01472 L
s
.18012 .01472 m
.18012 .21405 L
.18634 .21405 L
.18634 .01472 L
F
.18012 .01472 m
.18012 .21405 L
.18634 .21405 L
.18634 .01472 L
.18012 .01472 L
s
.21118 .01472 m
.21118 .01557 L
.21739 .01557 L
.21739 .01472 L
F
.21118 .01472 m
.21118 .01557 L
.21739 .01557 L
.21739 .01472 L
.21118 .01472 L
s
.24224 .01472 m
.24224 .0438 L
.24845 .0438 L
.24845 .01472 L
F
.24224 .01472 m
.24224 .0438 L
.24845 .0438 L
.24845 .01472 L
.24224 .01472 L
s
.27329 .01472 m
.27329 .04723 L
.2795 .04723 L
.2795 .01472 L
F
.27329 .01472 m
.27329 .04723 L
.2795 .04723 L
.2795 .01472 L
.27329 .01472 L
s
.30435 .01472 m
.30435 .21063 L
.31056 .21063 L
.31056 .01472 L
F
.30435 .01472 m
.30435 .21063 L
.31056 .21063 L
.31056 .01472 L
.30435 .01472 L
s
.3354 .01472 m
.3354 .17898 L
.34161 .17898 L
.34161 .01472 L
F
.3354 .01472 m
.3354 .17898 L
.34161 .17898 L
.34161 .01472 L
.3354 .01472 L
s
.36646 .01472 m
.36646 .05407 L
.37267 .05407 L
.37267 .01472 L
F
.36646 .01472 m
.36646 .05407 L
.37267 .05407 L
.37267 .01472 L
.36646 .01472 L
s
.39752 .01472 m
.39752 .06776 L
.40373 .06776 L
.40373 .01472 L
F
.39752 .01472 m
.39752 .06776 L
.40373 .06776 L
.40373 .01472 L
.39752 .01472 L
s
.42857 .01472 m
.42857 .07546 L
.43478 .07546 L
.43478 .01472 L
F
.42857 .01472 m
.42857 .07546 L
.43478 .07546 L
.43478 .01472 L
.42857 .01472 L
s
.45963 .01472 m
.45963 .0207 L
.46584 .0207 L
.46584 .01472 L
F
.45963 .01472 m
.45963 .0207 L
.46584 .0207 L
.46584 .01472 L
.45963 .01472 L
s
.49068 .01472 m
.49068 .02755 L
.49689 .02755 L
.49689 .01472 L
F
.49068 .01472 m
.49068 .02755 L
.49689 .02755 L
.49689 .01472 L
.49068 .01472 L
s
.52174 .01472 m
.52174 .03867 L
.52795 .03867 L
.52795 .01472 L
F
.52174 .01472 m
.52174 .03867 L
.52795 .03867 L
.52795 .01472 L
.52174 .01472 L
s
.5528 .01472 m
.5528 .20208 L
.55901 .20208 L
.55901 .01472 L
F
.5528 .01472 m
.5528 .20208 L
.55901 .20208 L
.55901 .01472 L
.5528 .01472 L
s
.58385 .01472 m
.58385 .18325 L
.59006 .18325 L
.59006 .01472 L
F
.58385 .01472 m
.58385 .18325 L
.59006 .18325 L
.59006 .01472 L
.58385 .01472 L
s
.61491 .01472 m
.61491 .05236 L
.62112 .05236 L
.62112 .01472 L
F
.61491 .01472 m
.61491 .05236 L
.62112 .05236 L
.62112 .01472 L
.61491 .01472 L
s
.64596 .01472 m
.64596 .03439 L
.65217 .03439 L
.65217 .01472 L
F
.64596 .01472 m
.64596 .03439 L
.65217 .03439 L
.65217 .01472 L
.64596 .01472 L
s
.67702 .01472 m
.67702 .11396 L
.68323 .11396 L
.68323 .01472 L
F
.67702 .01472 m
.67702 .11396 L
.68323 .11396 L
.68323 .01472 L
.67702 .01472 L
s
.70807 .01472 m
.70807 .01643 L
.71429 .01643 L
.71429 .01472 L
F
.70807 .01472 m
.70807 .01643 L
.71429 .01643 L
.71429 .01472 L
.70807 .01472 L
s
.73913 .01472 m
.73913 .04038 L
.74534 .04038 L
.74534 .01472 L
F
.73913 .01472 m
.73913 .04038 L
.74534 .04038 L
.74534 .01472 L
.73913 .01472 L
s
.77019 .01472 m
.77019 .2825 L
.7764 .2825 L
.7764 .01472 L
F
.77019 .01472 m
.77019 .2825 L
.7764 .2825 L
.7764 .01472 L
.77019 .01472 L
s
.80124 .01472 m
.80124 .20293 L
.80745 .20293 L
.80745 .01472 L
F
.80124 .01472 m
.80124 .20293 L
.80745 .20293 L
.80745 .01472 L
.80124 .01472 L
s
.8323 .01472 m
.8323 .23202 L
.83851 .23202 L
.83851 .01472 L
F
.8323 .01472 m
.8323 .23202 L
.83851 .23202 L
.83851 .01472 L
.8323 .01472 L
s
.86335 .01472 m
.86335 .02926 L
.86957 .02926 L
.86957 .01472 L
F
.86335 .01472 m
.86335 .02926 L
.86957 .02926 L
.86957 .01472 L
.86335 .01472 L
s
.89441 .01472 m
.89441 .03354 L
.90062 .03354 L
.90062 .01472 L
F
.89441 .01472 m
.89441 .03354 L
.90062 .03354 L
.90062 .01472 L
.89441 .01472 L
s
.92547 .01472 m
.92547 .03183 L
.93168 .03183 L
.93168 .01472 L
F
.92547 .01472 m
.92547 .03183 L
.93168 .03183 L
.93168 .01472 L
.92547 .01472 L
s
.95652 .01472 m
.95652 .01814 L
.96273 .01814 L
.96273 .01472 L
F
.95652 .01472 m
.95652 .01814 L
.96273 .01814 L
.96273 .01472 L
.95652 .01472 L
s
.98758 .01472 m
.98758 .06947 L
.99379 .06947 L
.99379 .01472 L
F
.98758 .01472 m
.98758 .06947 L
.99379 .06947 L
.99379 .01472 L
.98758 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{500, 309},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007d0001=B000`400?l00000o`00003ooooooooe
ooooo`00;Oooool3o`0000[ooooo1Ol0000:ooooo`?o00003?ooool3o`0000[ooooo0ol0000;oooo
o`?o00003?ooool00ol0003oooooooooo`0:ooooo`?o00002oooool2o`0000Wooooo0ol00004oooo
o`;o00001Oooool3o`0000?ooooo0ol00005ooooo`?o00000_ooool5o`0000Cooooo0ol00003oooo
o`?o00001Oooool3o`0000Cooooo0ol00004ooooo`?o00000oooool3o`0000Gooooo0ol00003oooo
o`?o00001Oooool3o`0000Cooooo00?o0000ooooooooool01?ooool3o`0000?ooooo0ol00005oooo
o`?o00000oooool2o`0000Gooooo1Ol00003ooooo`;o00001?ooool5o`0000;ooooo0ol00004oooo
o`Go000000?oooooo`000?l000000ol00003ooooo`Go00000_ooool3o`0000Cooooo1Ol00003oooo
o`?o00000oooool5o`0000;ooooo0ol00004ooooo`Go00000_ooool3o`0000Cooooo1Ol00003oooo
o`03o`000?oooooooooo00?ooooo1Ol00002ooooo`?o00001?ooool5o`0000;ooooo0_l00006oooo
o`?o00001?ooool2o`0000Gooooo0ol00003ooooo`?o00001Oooool3o`0000;ooooo1Ol0000>oooo
o`00;_ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000Wooooo00Go0000oooo
ooooooooooooo`00000<ooooo`03o`000?oooooooooo00Sooooo00Go0000ooooooooooooooooo`00
0009ooooo`05o`000?ooooooooooooooool000002oooool00ol0003oooooooooo`09ooooo`05o`00
0?ooooooooooooooool000003?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00;o
oooo00Co0000oooooooooooo00001Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`05o`000?oooooo
ooooooooool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01?ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool0
0oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`03oooo
o`04o`000?ooooooooooo`0000;ooooo00Co0000oooooooooooo00001?ooool01?l0003ooooooooo
ool00003ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooo00000_ooool01?l0003o
ooooooooool00004ooooo`06o`000?ooooooooooo`000?oooooo00000oooool00ol0003ooooooooo
o`02ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool00oooool01_l0003ooooo
ooooool0003oooooo`0000?ooooo00?o0000ooooooooool00_ooool01_l0003oooooooooool0003o
ooooo`0000?ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00003ooooo`03o`00
0?oooooooooo00Cooooo00Ko0000oooooooooooo0000ooooool00003ooooo`03o`000?oooooooooo
00;ooooo00Co0000oooooooooooo00001?ooool00ol0003oooooooooo`02ooooo`05o`000?oooooo
ooooooooool000000_ooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000oooool00ol0003oooooooooo`03ooooo`05o`000?ooooooooooooooool000000_ooool01?l0
003oooooooooool0000>ooooo`00;_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo
00gooooo00?o0000ooooooooool01oooool5o`0000gooooo00?o0000ooooooooool01oooool01Ol0
003ooooooooooooooooo000000cooooo00?o0000ooooooooool02?ooool01Ol0003ooooooooooooo
oooo000000gooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`02ooooo`04o`000?oo
ooooooooo`0000Gooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00?ooooo00Co0000oooooooooooooooo1Ol00005ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00Go0000oooo
ooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00001Ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00ooo
ool01?l0003oooooooooooooool5o`0000Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`03ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooooooooooo0000
00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`05ooooo`05o`000?oooooo
ooooooooool000002Oooool01Ol0003ooooooooooooooooo0000013ooooo000^ooooo`03o`000?oo
oooooooo00cooooo00?o0000ooooooooool02_ooool2o`0000[ooooo00Co0000oooooooooooo0000
3_ooool00ol0003oooooooooo`07ooooo`Co00003Oooool00ol0003oooooooooo`09ooooo`?o0000
2oooool4o`0000Sooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`03
o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool00oooool2o`0000Kooooo00Go0000oooooooo
ooooooooo`000002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`03ooooo`04o`000?ooooooooooooooo`Co00001_ooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool3o`0000Kooooo00?o0000
ooooooooool00_ooool4o`0000Kooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`06ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`02ooooo`;o00001ooo
ool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Co00001oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00Gooooo00Co0000oooooooooooooooo0ol00007ooooo`04
o`000?ooooooooooooooo`Co00001Oooool2o`0000?ooooo00Co0000oooooooooooo00001Oooool2
o`0000Cooooo00?o0000ooooooooool01Oooool2o`0000Gooooo00?o0000ooooooooool03Oooool0
02kooooo00?o0000ooooooooool02Oooool01Ol0003ooooooooooooooooo000000gooooo00?o0000
ooooooooool02?ooool00ol0003oooooo`00000:ooooo`Co00002_ooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00Go0000ooooooooooooooooo`000009ooooo`05o`000?oo
ooooooooooooool000002?ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Go
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`05o`000?oooooooooooooo
ool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Kooooo00Co0000oooo
oooooooooooo1?l00006ooooo`05o`000?ooooooooooooooool000002Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooool00002ooooo`05o`000?ooooooooooooooool000000ooo
ool01Ol0003ooooooooooooooooo000000Gooooo00Oo0000ooooooooooooooooo`000?oooooo0000
00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00Go0000
ooooooooooooooooo`000003ooooo`06o`000?ooooooooooo`000?oooooo00001?ooool01_l0003o
oooooooooooooooo0000ooooo`Co00001?ooool01ol0003ooooooooooooooooo0000ooooool00000
1oooool01Ol0003ooooooooooooooooo000000Gooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo
000000?ooooo00?o0000ooooooooool01Oooool01?l0003oooooooooool00002ooooo`03o`000?oo
oooooooo00Gooooo00Go0000ooooooooooooooooo`000009ooooo`03o`000?oooooo000000?ooooo
00?o0000ooooooooool03?ooool002gooooo0_l0000;ooooo`05o`000?ooooooooooooooool00000
2Oooool01Ol0003ooooooooooooooooo000000_ooooo0_l0000:ooooo`03o`000?oooooooooo00co
oooo00?o0000ooooooooool02_ooool01Ol0003ooooooooooooooooo000000Wooooo00Go0000oooo
ooooooooooooo`000009ooooo`05o`000?ooooooooooooooool000001oooool2o`0000Cooooo00Co
0000oooooooooooo00001?ooool2o`0000Cooooo0_l00006ooooo`;o00000oooool01Ol0003ooooo
oooooooooooo000000Cooooo0_l00003ooooo`05o`000?ooooooooooooooool000001?ooool2o`00
00Gooooo0_l00005ooooo`;o00000oooool00ol0003oooooooooo`06ooooo`;o00001?ooool00ol0
003oooooooooo`05ooooo`;o00000oooool01Ol0003ooooooooooooooooo000000Cooooo0_l00003
ooooo`05o`000?ooooooooooooooool000001?ooool2o`0000?ooooo00Go0000oooooooooooooooo
o`000003ooooo`05o`000?ooooooooooooooool000000_ooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool2o`0000Gooooo00Oo0000ooooooooooooooooo`00
0?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo000000?o
oooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooooooo`;o00001?ooool0
1ol0003ooooooooooooooooo0000ooooool000001oooool01Ol0003ooooooooooooooooo000000;o
oooo00?o0000ooooooooool01?ooool01ol0003ooooooooooooooooo0000ooooool000000oooool0
1Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooool000000oooool01Ol0003ooooooooo
oooooooo000000?ooooo00?o0000ooooool000000oooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000_ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000
ooooooooool00_l00005ooooo`07o`000?ooooooooooooooool0003oooooo`000003ooooo`03o`00
0?oooooooooo00cooooo000^ooooo`03o`000?oooooooooo00[ooooo0ol0000;ooooo`?o00003Ooo
ool00ol0003oooooooooo`08ooooo`Go00002oooool2o`0000[ooooo1Ol0000:ooooo`?o00002ooo
ool3o`0000Wooooo00?o0000ooooooooool00oooool2o`0000Kooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;ooooo0ol00006ooooo`03o`000?oo
oooooooo00;ooooo0ol00006ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?oo
ool01?l0003oooooooooooooool5o`0000Gooooo00?o0000ooooooooool00oooool2o`0000Kooooo
00Co0000oooooooooooooooo1Ol00005ooooo`03o`000?oooooooooo00;ooooo0ol00006ooooo`03
o`000?oooooooooo00;ooooo0ol00005ooooo`?o00001?ooool2o`0000Gooooo0ol00004ooooo`03
o`000?oooooooooo00Cooooo0ol00003ooooo`?o00001Oooool3o`0000?ooooo0ol00005ooooo`?o
00001Oooool00ol0003oooooooooo`03ooooo`?o00000_ooool5o`0000Cooooo0ol00004ooooo`;o
00001Oooool3o`0000;ooooo1Ol00004ooooo`?o00000oooool3o`0000Gooooo0ol00003ooooo`?o
00001Oooool3o`0000Cooooo0_l00005ooooo`?o00001?ooool00ol0003oooooooooo`04ooooo`?o
00000oooool3o`0000oooooo003oooooooGooooo003oooooooGooooo003oooooooGooooo003ooooo
ooGooooo003oooooooGooooo003oooooooGooooo003oooooooGooooo000Voooooooo0000aOl0000:
ooooo`009_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`08ooooo`00
9_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`08ooooo`007Oooool2o`0000Oooooo00?o0000ooooooooool0oooo
ooo1ooooo`03o`000?oooooooooo00Sooooo000Looooo`04o`000?ooooooooooo`0000Kooooo00?o
0000ooooooooool0ooooooo1ooooo`03o`000?oooooooooo00Sooooo000Looooo`04o`000?oooooo
ooooo`0000Kooooo00?o0000ooooooooool0ooooooo1ooooo`03o`000?oooooooooo00Sooooo000L
ooooo`04o`000?ooooooooooo`0000Kooooo0ol00003ool000?o0?l00_l00?l4oomo00;o00000ooo
0003o`3o00;o003o1?ooO`02o`0000?oo`000ol0o`02o`00o`Coogl00_l00003ool000?o0?l00_l0
0?l4oomo00;o00000ooo0003o`3o00;o003o1?ooO`02o`0000?oo`000ol0o`02o`00o`Coogl00_l0
0003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0
o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0ooo
O`02o`0000?oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo
0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol0
0?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl00_l0
0003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0
o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0ooo
O`02o`0000?oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo
0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol0
0?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl00_l0
0003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`03o`0000[ooooo000L
ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool00ooo0004o`3o0003oooooooo
O`3oogl000;oogl00_ooool3ool000Co0?l000?ooooooomo0?ooO`000_ooO`02ooooo`?oo`000ol0
o`02o`00o`Coogl00_ooool4ool000Cooooo1?ooO`05ooooo`?o0?l00_l00?l4oomo00Gooooo1?l0
o`000oooooooogl0oomo0002oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo0005ooooo`?oogl00_oo
ool3ool000Co0?l00_ooool3oomo00Gooooo1?l0o`02ooooo`?oogl01Oooool4o`3o00;ooooo0ooo
O`02ooooo`?oo`001?l0o`02ooooo`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo0ooo
0004o`3o00;ooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl00_oo
ool3ool000Co0?l00_ooool3oomo00Gooooo1?l0o`02ooooo`?oogl00_ooool3ool000?o0?l00ol0
0?l3oomo00;ooooo0ooo0004o`3o00;ooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo0ooo
0004o`3o00;ooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo0ooo0004o`3o00;ooooo0ooo
O`02ooooo`?oo`001?l0o`02ooooo`?oogl01Oooool4o`3o00;ooooo0oooO`02ooooo`?oo`001?l0
o`02ooooo`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl00_oo
ool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo0ooo0004o`3o00;ooooo0ooo
O`02ooooo`03o`000?oooooooooo00Sooooo000Looooo`04o`000?ooooooooooo`0000Kooooo00?o
0000ooooooooool01?oo0004ooooo`Coogl02_ooool4oomo00;ooooo0ooo0003o`3o00;o003o1?oo
O`02ooooo`Coo`001?ooool4oomo00Gooooo0ol0o`02o`00o`Coogl02_ooool4oomo00Gooooo0ol0
o`04o`00o`Cooooo1?oo0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`Co0?l00_oo
ool3oomo00Gooooo1?l0o`02ooooo`?oogl00_ooool3ool000Co0?l00_ooool3oomo00;ooooo0ooo
0003o`3o00?o003o0oooO`02ooooo`?oo`001?l0o`02ooooo`?oogl01Oooool3o`3o00?o003o0ooo
O`02ooooo`Coo`001Oooool3oomo00;ooooo0ooo0004o`3o00;ooooo0oooO`0;ooooo`?oogl01Ooo
ool3o`3o00?o003o0oooO`02ooooo`?oo`001?l0o`02ooooo`?oogl01Oooool3o`3o00?o003o0ooo
O`02ooooo`?oo`001?l0o`02ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool000Gooooo0ooo
O`02ooooo`?oo`001?l0o`02ooooo`?oogl01Oooool4o`3o00;ooooo0oooO`05ooooo`Co0?l00_oo
ool3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo1?oo
0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool3ool000Co0?l00_ooool3oomo00;ooooo00?o
0000ooooooooool02?ooool001gooooo0_l00007ooooo`03o`000?oooooooooo00Coo`001?ooool4
oomo00[ooooo1?ooO`02ooooo`?oo`000ol0o`02o`00o`Coogl00_ooool4ool000Cooooo1?ooO`05
ooooo`?o0?l00_l00?l4oomo00[ooooo1?ooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3
oomo00;ooooo1?oo0005ooooo`?oogl02oooool3oomo00Gooooo1?l0o`02ooooo`?oogl00_ooool4
ool000Gooooo0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`05
ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl02oooool3
oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3
oomo00;ooooo0ooo0004o`3o00;ooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3
oomo00Gooooo1?l0o`02ooooo`?oogl02oooool3oomo00Gooooo1?l0o`02ooooo`?oogl00_ooool4
ool000Gooooo0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`05
ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00;ooooo00?o0000ooooooooool02?ooool0
02Kooooo00?o0000ooooooooool01?oo0004ooooo`Coogl02_ooool4oomo00Gooooo0ol0o`02o`00
o`Coogl00_ooool4ool000Cooooo1?ooO`05ooooo`?o0?l00_l00?l4oomo00[ooooo1?ooO`05oooo
o`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl02oooool3oomo
00Gooooo1?l0o`02ooooo`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3oomo
00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`02oooo
o`Coo`001Oooool3oomo00_ooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005oooo
o`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`?oo`001?l0o`02ooooo`?oogl01Oooool3o`3o
00Co003o1?ooool4ool000Gooooo0oooO`0;ooooo`?oogl02oooool3oomo00Gooooo1?l0o`02oooo
o`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool4ool0
00Gooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00;ooooo00?o0000oooo
ooooool02?ooool002Kooooo00?o0000ooooooooool01?oo0004ooooo`Coogl02_ooool4oomo00Go
oooo0ol0o`02o`00o`Coogl00_ooool4ool000Cooooo1?ooO`05ooooo`?o0?l00_l00?l4oomo00[o
oooo1?ooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?o
ogl02oooool3oomo00_ooooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?o
ogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00;o
oooo1?oo0005ooooo`?oogl02oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Go
oooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo0ooo0004o`3o00;ooooo0oooO`05ooooo`?o
0?l01?l00?l4ooooo`Coo`001Oooool3oomo00_ooooo0oooO`0;ooooo`?oogl01Oooool4o`3o00;o
oooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Go
oooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00;ooooo00?o0000oooooooo
ool02?ooool002Kooooo00?o0000ooooooooool01?oo0004ooooo`Coogl02_ooool4oomo00Gooooo
0ol0o`02o`00o`Coogl00_ooool4ool000Cooooo1?ooO`05ooooo`?o0?l00_l00?l4oomo00[ooooo
1?ooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl0
2oooool3oomo00_ooooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl0
0_ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00;ooooo
1?oo0005ooooo`?oogl02oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo
0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o
1?ooool4ool000Gooooo0oooO`0;ooooo`?oogl02oooool3oomo00Gooooo1?l0o`02ooooo`?oogl0
0_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl0
1Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`02ooooo`03o`000?oooooooooo00Sooooo
000Vooooo`03o`000?oooooooooo00Coo`001?ooool4oomo00[ooooo1?ooO`05ooooo`?o0?l00_l0
0?l4oomo00;ooooo1?oo000=ooooo`?o0?l00_l00?l4oomo00[ooooo1?ooO`05ooooo`?o0?l01?l0
0?l4ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl02oooool3oomo00_ooooo0ooo
O`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0ooo
O`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool000Gooooo0oooO`0;ooooo`?oogl01Ooo
ool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_oo
ool4ool000Gooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00_ooooo0ooo
O`0;ooooo`?oogl01Oooool4o`3o00;ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0ooo
O`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo0005ooooo`?oogl00_oo
ool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool000Cooooo1?ooO`0:
ooooo`Coogl01Oooool3o`3o00;o003o1?ooO`02ooooo`Coo`003Oooool3o`3o00;o003o1?ooO`0:
ooooo`Coogl01Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3
oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3
oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo0005
ooooo`?oogl02oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`05
ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4
ool000Gooooo0oooO`0;ooooo`?oogl02oooool3oomo00Gooooo1?l0o`02ooooo`?oogl00_ooool4
ool000gooooo0ol0o`03o`00o`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo0005
ooooo`?oogl00_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool0
00Cooooo1?ooO`0:ooooo`Coogl01Oooool3o`3o00;o003o1?ooO`02ooooo`Coo`003Oooool3o`3o
00;o003o1?ooO`0:ooooo`Coogl01Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`02oooo
o`Coo`001Oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000Gooooo0oooO`05oooo
o`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool0
00[ooooo1?oo0005ooooo`?oogl02oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool0
00Gooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o
00Co003o1?ooool4ool000Gooooo0oooO`0;ooooo`?oogl02oooool3oomo00_ooooo0oooO`02oooo
o`Coo`003Oooool3o`3o00Co003o1?ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo0005oooo
o`?oogl00_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool000Co
oooo1?ooO`0:ooooo`Coogl01Oooool3o`3o00;o003o1?ooO`02ooooo`Coo`003Oooool3o`3o00;o
003o1?ooO`0:ooooo`Coogl01Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`02ooooo`Co
o`001Oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o
0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool000[o
oooo1?oo0005ooooo`?oogl02oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Go
oooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`04o`00o`Co
oooo1?oo0005ooooo`?oogl02oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000go
oooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00;o
oooo00?o0000ooooooooool02?ooool002Kooooo0ol00004ool000Cooooo1?ooO`0:ooooo`Coogl0
1Oooool3o`3o00;o003o1?ooO`02ooooo`Coo`003Oooool3o`3o00;o003o1?ooO`0:ooooo`Coogl0
1Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00_ooooo
0oooO`0;ooooo`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo
1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo0005ooooo`?oogl0
2oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l0
1?l00?l4ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo0005ooooo`?oogl0
2oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo
1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo0003ooooool0003o000000[ooooo
000Vooooo`03o`000?oooooooooo00Coo`004_ooool4oomo00Gooooo0ol0o`02o`00o`Coogl00_oo
ool4ool000gooooo0ol0o`03o`00o`gooooo1?ooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`001Ooo
ool3oomo00;ooooo1?oo0005ooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`Coo`001Ooo
ool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l0
0?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo
0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l0
0?l4ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0
o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00;ooooo00?o
0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000Booooo`Coogl01Oooool3
o`3o00;o003o1?ooO`02ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04
o`00o`Cooooo1?oo0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`0;ooooo`?oogl02oooool3
oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3
oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00Gooooo0ol0o`03
o`00o`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3
oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl02oooool3oomo00_ooooo0oooO`02
ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo0005
ooooo`?oogl00_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool0
01;ooooo1?ooO`05ooooo`?o0?l00_l00?l4oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l=oooo
o`Coogl01Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo0005ooooo`?oogl02oooool3oomo
00_ooooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool0
00Gooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`05oooo
o`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool0
00Gooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;oooo
o`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4oooo
o`Coo`001Oooool3oomo00;ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooooooo
ool01?oo000Booooo`Coogl01Oooool3o`3o00;o003o1?ooO`02ooooo`Coo`003Oooool3o`3o00?o
003o3Oooool4oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_o
oooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Go
oooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`05ooooo`?o
0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool000Go
oooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;ooooo`?o
ogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Co
o`001Oooool3oomo00;ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0
1?oo000Booooo`Coogl01Oooool3o`3o00;o003o1?ooO`02ooooo`Coo`003Oooool3o`3o00?o003o
3Oooool4oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo
0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo
0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`05ooooo`?o0?l0
0ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool000Gooooo
0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;ooooo`?oogl0
0_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`00
1Oooool3oomo00;ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo
000Booooo`Coogl01Oooool3o`3o00;o003o1?ooO`02ooooo`Coo`003Oooool3o`3o00?o003o3Ooo
ool4oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0ooo
O`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0ooo
O`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`05ooooo`?o0?l00ol0
0?l3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool000Gooooo0ooo
O`05ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_oo
ool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`001Ooo
ool3oomo00;ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000B
ooooo`Coogl01Oooool3o`3o00;o003o1?ooO`02ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4
oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02
ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`05
ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`05ooooo`?o0?l00ol00?l3
oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`04
o`00o`Cooooo1?oo000Cooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3
o`3o00Co003o1?ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo0005ooooo`?oogl00_ooool0
0ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool001;ooooo1?ooO`05oooo
o`?o0?l00_l00?l4oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l=ooooo`Coogl01Oooool3o`3o
00Co003o1?ooool4ool000[ooooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000=oooo
o`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool0
00[ooooo1?oo000Cooooo`?oogl02?ooool3o`00o`?oogl00_ooool4ool000gooooo0ol0o`04o`00
o`Cooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`0;oooo
o`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o
00Co003o1?ooool4ool000Gooooo0oooO`02ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03
o`000?oooooooooo00Coo`004_ooool4oomo00Gooooo0ol0o`02o`00o`Coogl00_ooool4ool000go
oooo0ol0o`03o`00o`gooooo1?ooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?o
oooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Go
oooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?o
ogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co
003o1?ooool4ool001?ooooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o
0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`02ooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00Coo`004_ooool4oomo00Gooooo
0ol0o`02o`00o`Coogl00_ooool4ool000gooooo0ol0o`03o`00o`gooooo1?ooO`05ooooo`?o0?l0
1?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo
0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`00
2_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo
1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`0;ooooo`?oogl0
2oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o
1?ooool4ool000Gooooo0oooO`02ooooo`03o`000?oooooooooo00Sooooo000Vooooo`?o00001?oo
000Booooo`Coogl01Oooool3o`3o00?o003o1Oooool4ool000gooooo0ol0o`03o`00o`gooooo1?oo
O`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_oo
ool4ool000gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l0
0?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0
o`04o`00o`Cooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool001?ooooo0ooo
O`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Ooo
ool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`000ooooooo0000o`00000:ooooo`009_ooool0
0ol0003oooooooooo`04ool001;ooooo1?ooO`05ooooo`?o0?l00ol00?l5ooooo`Coo`003Oooool3
o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3
oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3
oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02
ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l00?l4
ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04
o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00;ooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000Booooo`Coogl01Oooool3o`3o
00?o003o1Oooool4ool000gooooo0ol0o`03o`00o`gooooo1?ooO`05ooooo`?o0?l01?l00?l4oooo
o`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`03o`00
o`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool0
01?ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo0005oooo
o`?oogl01Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`0;ooooo`?oogl02oooool3oomo
00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool0
00Gooooo0oooO`02ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00Co
o`004_ooool4oomo00Gooooo0ol0o`03o`00o`Gooooo1?oo000=ooooo`?o0?l00ol00?l=ooooo`Co
ogl01Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000Cooooo`?oogl02oooool3oomo00;o
oooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`04o`00o`Co
oooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co
003o1?ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00_o
oooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o
0?l01?l00?l4ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooo
ooooo`04ool001;ooooo1?ooO`05ooooo`?o0?l00ol00?l5ooooo`Coo`003Oooool3o`3o00?o003o
3Oooool4oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo
0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`05ooooo`?o0?l0
1?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo
0ol0o`04o`00o`Cooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool001?ooooo
0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`00
3Oooool3o`3o00Co003o1?ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o
0000ooooooooool01?oo000Booooo`Coogl01Oooool3o`3o00?o003o1Oooool4ool000gooooo0ol0
o`03o`00o`gooooo1?ooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0ooo
O`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo0005ooooo`?oogl01Ooo
ool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo
000=ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo
000Cooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?oo
ool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000V
ooooo`03o`000?oooooooooo00Coo`004_ooool4oomo00Gooooo0ol0o`03o`00o`Gooooo1?oo000=
ooooo`?o0?l00ol00?l=ooooo`Coogl01Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000C
ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3
o`3o00Co003o1?ooool4ool000[ooooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000=
ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000C
ooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4
ool000gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Voooo
o`03o`000?oooooooooo00Coo`004_ooool4oomo00Gooooo0ol0o`03o`00o`Gooooo1?oo000=oooo
o`?o0?l00ol00?l=ooooo`Coogl01Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000Coooo
o`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o
00Co003o1?ooool4ool000[ooooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000=oooo
o`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000Coooo
o`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool0
00gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03
o`000?oooooooooo00Coo`004_ooool4oomo00Gooooo0ol0o`03o`00o`Gooooo1?oo000=ooooo`?o
0?l00ol00?l=ooooo`Coogl01Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000Cooooo`?o
ogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co
003o1?ooool4ool000[ooooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o
0?l01?l00?l4ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?o
ogl02oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000go
oooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`00
0?oooooooooo00Coo`004_ooool4oomo00Gooooo0ol0o`03o`00o`Gooooo1?oo000=ooooo`?o0?l0
0ol00?l=ooooo`Coogl01Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000Cooooo`?oogl0
2oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o
1?ooool4ool000[ooooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l0
1?l00?l4ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl0
2oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000gooooo
0ol0o`04o`00o`Cooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oo
oooooooo00Coo`004_ooool4oomo00Gooooo0ol0o`03o`00o`Gooooo1?oo000=ooooo`?o0?l00ol0
0?l=ooooo`Coogl01Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000Cooooo`?oogl02ooo
ool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?oo
ool4ool000[ooooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l0
0?l4ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl02ooo
ool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000gooooo0ol0
o`04o`00o`Cooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`?o00001?oo000B
ooooo`Coogl01Oooool3o`3o00?o003o1Oooool4ool000gooooo0ol0o`03o`00o`gooooo1?ooO`05
ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4
ool000gooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4
ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo0005
ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`0;ooooo`?oogl02oooool3
oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4
ool000Wooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`04ool001;ooooo1?ooO`05oooo
o`?o0?l00ol00?l5ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00
o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o
00Co003o1?ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo
00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`05oooo
o`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool0
00gooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool001;ooooo1?ooO`05ooooo`?o
0?l00ol00?l5ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00o`Co
oooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co
003o1?ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_o
oooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`05ooooo`?o
0?l01?l00?l4ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000go
oooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool00ol0003o
ooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool001;ooooo1?ooO`05ooooo`?o0?l0
0ol00?l5ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00o`Cooooo
1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o
1?ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo
0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`05ooooo`?o0?l0
1?l00?l4ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo
0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool001;ooooo1?ooO`05ooooo`?o0?l00ol0
0?l5ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo
000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?oo
ool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0ooo
O`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l0
0?l4ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0
o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool00ol0003ooooooooo
o`08ooooo`009_ooool00ol0003oooooooooo`04ool001;ooooo1?ooO`05ooooo`?o0?l00ol00?l5
ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:
ooooo`Coo`004oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4
ool000gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02
ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l00?l4
ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04
o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool00ol0003oooooooooo`08
ooooo`009_ooool00ol0003oooooooooo`04ool001;ooooo1?ooO`05ooooo`?o0?l00ol00?l5oooo
o`Coo`003Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:oooo
o`Coo`004oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool0
00gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02oooo
o`Coo`003Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l00?l4oooo
o`Coo`004oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00
o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool00ol0003oooooooooo`08oooo
o`009_ooool00ol0003oooooooooo`04ool001;ooooo1?ooO`05ooooo`?o0?l00ol00?l5ooooo`Co
o`003Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Co
o`004oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000go
oooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02ooooo`Co
o`003Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Co
o`004oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Co
oooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`00
9_ooool00ol0003oooooooooo`04ool001;ooooo1?ooO`05ooooo`?o0?l00ol00?l5ooooo`Coo`00
3Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`00
4oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000gooooo
0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02ooooo`Coo`00
3Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`00
4oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo
1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_oo
ool00ol0003oooooooooo`04ool001;ooooo1?ooO`05ooooo`?o0?l00ol00?l5ooooo`Coo`003Ooo
ool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004ooo
ool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000gooooo0ol0
o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Ooo
ool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`004ooo
ool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo
000=ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool0
0ol0003oooooooooo`04ool001;ooooo1?ooO`05ooooo`?o0?l00ol00?l5ooooo`Coo`003Oooool3
o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3
oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000gooooo0ol0o`04
o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3
o`3o00Co003o1?ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3
oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000=
ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool3o`00
00Coo`004_ooool4oomo00Gooooo0ol0o`03o`00o`Gooooo1?oo000=ooooo`?o0?l00ol00?l=oooo
o`Coogl01Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000Cooooo`?oogl02oooool3oomo
00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool0
00[ooooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4oooo
o`Coo`001Oooool3oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl02oooool3oomo
00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000gooooo0ol0o`04o`00
o`Cooooo1?oo0009ooooo`;o00002_ooool002Kooooo00?o0000ooooooooool01?oo000Booooo`Co
ogl01Oooool3o`3o00?o003o1Oooool4ool000gooooo0ol0o`03o`00o`gooooo1?ooO`05ooooo`?o
0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4ool000go
oooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?o
oooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo0005ooooo`?o
ogl01Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`0;ooooo`?oogl02oooool3oomo00;o
oooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000[o
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000Booooo`Coogl0
1Oooool3o`3o00?o003o1Oooool4ool000gooooo0ol0o`03o`00o`gooooo1?ooO`05ooooo`?o0?l0
1?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo
0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo
0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l0
1?l00?l4ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo
0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool001;ooooo1?ooO`05ooooo`?o0?l00ol0
0?l5ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo
000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?oo
ool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0ooo
O`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo
000Cooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?oo
ool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000V
ooooo`03o`000?oooooooooo00Coo`004_ooool4oomo00Gooooo0ol0o`03o`00o`Gooooo1?oo000=
ooooo`?o0?l00ol00?l=ooooo`Coogl01Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000C
ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3
o`3o00Co003o1?ooool4ool000[ooooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000=
ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`0;
ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3
o`3o00Co003o1?ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooo
ooooool01?oo000Booooo`Coogl01Oooool3o`3o00?o003o1Oooool4ool000gooooo0ol0o`03o`00
o`gooooo1?ooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;oooo
o`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4oooo
o`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00
o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;oooo
o`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4oooo
o`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool001;o
oooo1?ooO`05ooooo`?o0?l00ol00?l5ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4oomo00Go
oooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02ooooo`Co
o`004?ooool4o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?o
oooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o
0?l01?l00?l4ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000go
oooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool00ol0003o
ooooooooo`08ooooo`009_ooool00ol0003oooooooooo`04ool001;ooooo1?ooO`05ooooo`?o0?l0
0ol00?l5ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00o`Cooooo
1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02ooooo`Coo`004?ooool4o`00o`Cooooo
1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl0
0_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`00
4oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo
1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`002ooo
ool2o`0000Cooooo00?o0000ooooooooool00oooool2o`0000?ooooo0ol00007ooooo`03o`000?oo
oooooooo00Coo`004_ooool4oomo00Gooooo0ol0o`03o`00o`Gooooo1?oo000=ooooo`?o0?l00ol0
0?l=ooooo`Coogl01Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000Cooooo`?oogl02ooo
ool3oomo00;ooooo1?oo000@ooooo`Co003o1?ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo
000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?oo
ool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl02oooool3oomo00_ooooo0ooo
O`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo
000:ooooo`03o`000?oooooooooo00Sooooo000:ooooo`04o`000?ooooooooooo`0000Sooooo00Ko
0000oooooooooooo0000ooooool00003ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01?oo000Booooo`Coogl01Oooool3o`3o00?o003o1Oooool4ool000gooooo0ol0o`03o`00o`go
oooo1?ooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?o
ogl00_ooool4ool0013ooooo1?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000[o
oooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Co
o`003Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`0;ooooo`?oogl02oooool3oomo00;o
oooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000[o
oooo00?o0000ooooooooool02?ooool000[ooooo00Co0000oooooooooooo00002?ooool01?l0003o
ooooooooool00005ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?oo000Boooo
o`Coogl01Oooool3o`3o00?o003o1Oooool4ool000gooooo0ol0o`03o`00o`gooooo1?ooO`05oooo
o`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4ool0
013ooooo1?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000Coooo
o`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o
00Co003o1?ooool4ool001?ooooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo000=oooo
o`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000[ooooo00?o0000oooo
ooooool02?ooool000[ooooo00Co0000oooooooooooo00002?ooool01?l0003oooooooooool00005
ooooo`03o`000?oooooooooo00Cooooo0ol00004ool001;ooooo1?ooO`05ooooo`?o0?l00ol00?l5
ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:
ooooo`Coo`004oooool3oomo00_ooooo0oooO`02ooooo`Coo`004?ooool4o`00o`Cooooo1?oo000=
ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4
ool000gooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3
oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000=
ooooo`?o0?l01?l00?l4ooooo`Coo`001oooool4o`0000[ooooo000:ooooo`04o`000?oooooooooo
o`0000Sooooo00Go0000oooooooooooo0000ooooo`04o`0000Oooooo00?o0000ooooooooool01?oo
000Booooo`Coogl01Oooool3o`3o00?o003o1Oooool4ool000gooooo0ol0o`03o`00o`gooooo1?oo
O`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_oo
ool4ool0013ooooo1?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo
000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Ooo
ool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo
000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000[ooooo00?o
0000ooooooooool02?ooool000[ooooo00Co0000oooooooooooo00002?ooool01_l0003ooooooooo
ool0003oooooo`0000[ooooo00?o0000ooooooooool01?oo000Booooo`Coogl01Oooool3o`3o00?o
003o1Oooool4ool000gooooo0ol0o`03o`00o`gooooo1?ooO`05ooooo`?o0?l01?l00?l4ooooo`Co
o`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4ool0013ooooo1?l00?l4ooooo`Co
o`003Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000Cooooo`?oogl02oooool3oomo00;o
oooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool001?o
oooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Co
o`003Oooool3o`3o00Co003o1?ooool4ool000[ooooo00?o0000ooooooooool02?ooool000_ooooo
0_l0000:ooooo`;o00000_ooool5o`0000Kooooo00?o0000ooooooooool01?oo000Booooo`Coogl0
1Oooool3o`3o00?o003o1Oooool4ool000gooooo0ol0o`03o`00o`gooooo1?ooO`05ooooo`?o0?l0
1?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4ool0013ooooo
1?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000Cooooo`?oogl0
2oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o
1?ooool4ool001?ooooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l0
1?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000[ooooo00?o0000ooooooooool0
2?ooool002Kooooo00?o0000ooooooooool01?oo000Booooo`Coogl01Oooool3o`3o00?o003o1Ooo
ool4ool000gooooo0ol0o`03o`00o`gooooo1?ooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_oo
ool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4ool0013ooooo1?l00?l4ooooo`Coo`003Ooo
ool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo
000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool001?ooooo0ooo
O`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Ooo
ool3o`3o00Co003o1?ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000
ooooooooool01?oo000Booooo`Coogl01Oooool3o`3o00?o003o1Oooool4ool000gooooo0ol0o`03
o`00o`gooooo1?ooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;
ooooo`?oogl00_ooool4ool0013ooooo1?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4
ool000[ooooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4
ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`0;ooooo`?oogl02oooool3
oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4
ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000Boooo
o`Coogl01Oooool3o`3o00?o003o1Oooool4ool000gooooo0ol0o`03o`00o`gooooo1?ooO`05oooo
o`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4ool0
013ooooo1?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000Coooo
o`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o
00Co003o1?ooool4ool001?ooooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo000=oooo
o`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000[ooooo00?o0000oooo
ooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000Booooo`Coogl01Oooool3o`3o00?o
003o1Oooool4ool000gooooo0ol0o`03o`00o`gooooo1?ooO`05ooooo`?o0?l01?l00?l4ooooo`Co
o`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4ool0013ooooo1?l00?l4ooooo`Co
o`003Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000Cooooo`?oogl02oooool3oomo00;o
oooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool001?o
oooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Co
o`003Oooool3o`3o00Co003o1?ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo
00?o0000ooooooooool01?oo000Booooo`Coogl01Oooool3o`3o00?o003o1Oooool4ool000gooooo
0ol0o`03o`00o`gooooo1?ooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo
0oooO`0;ooooo`?oogl00_ooool4ool0013ooooo1?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o
1?ooool4ool000[ooooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l0
1?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`0;ooooo`?oogl0
2oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o
1?ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo
000Booooo`Coogl01Oooool3o`3o00?o003o1Oooool4ool000gooooo0ol0o`03o`00o`gooooo1?oo
O`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_oo
ool4ool0013ooooo1?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo
000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Ooo
ool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo
000=ooooo`?o0?l01?l00?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000[ooooo00?o
0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000Booooo`Coogl01Oooool4
o`3o00Oooooo1?oo000=ooooo`?o0?l00ol00?l=ooooo`Coogl01Oooool3o`3o00Co003o1?ooool4
ool000[ooooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000Hooooo`Coo`003Oooool3
o`3o00Co003o1?ooool4ool000[ooooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000=
ooooo`?o0?l01?l00?lEooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;
ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4
ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool3o`0000Coo`004_ooool4oomo
00Gooooo1?l0o`07ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00
o`Cooooo1?oo000:ooooo`Coo`004oooool3oomo00_ooooo0oooO`02ooooo`Coo`006?ooool4ool0
00gooooo0ol0o`04o`00oa;ooooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000=oooo
o`?o0?l01?l00?lEooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;oooo
o`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000=ooooo`?o0?l01?l00?l4oooo
o`Coo`002Oooool2o`0000[ooooo000Vooooo`03o`000?oooooooooo00Coo`004_ooool4oomo00Go
oooo1?l0o`07ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00oa;o
oooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000Hooooo`Coo`003Oooool3o`3o00Co
003o4_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00oaGo
oooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`Co
o`003Oooool3o`3o00Co003o1?ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00Coo`004_ooool4oomo00Gooooo
1?l0o`07ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00oa;ooooo
1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000Hooooo`Coo`003Oooool3o`3o00Co003o
4_ooool4ool001?ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00oaGooooo
0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`Coo`00
3Oooool3o`3o00Co003o1?ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`03o`00
0?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00Coo`004_ooool4oomo00Gooooo1?l0
o`07ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00oa;ooooo1?oo
000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000Hooooo`Coo`003Oooool3o`3o00Co003o4_oo
ool4ool001?ooooo0oooO`0;ooooo`?oogl04oooool3o`3o00Co003o5Oooool3o`3o00Co003o1?oo
ool4ool001?ooooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l0
0?l4ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000[ooooo00?o0000ooooooooool02?oo
ool002Kooooo00?o0000ooooooooool01?oo000Booooo`Coogl01Oooool4o`3o00Oooooo1?oo000=
ooooo`?o0?l00ol00?l=ooooo`Coogl01Oooool3o`3o00Co003o4_ooool4ool001?ooooo0oooO`0;
ooooo`?oogl00_ooool4ool001Sooooo1?oo000=ooooo`?o0?l01?l00?lBooooo`Coo`004oooool3
oomo00_ooooo0oooO`0Cooooo`?o0?l01?l00?lEooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3
oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00oaGooooo0ol0o`04
o`00o`Cooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo
00Coo`004_ooool4oomo00Gooooo1?l0o`07ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4oomo
00Gooooo0ol0o`04o`00oa;ooooo1?oo000Cooooo`?oogl02oooool3oomo00;ooooo1?oo000Hoooo
o`Coo`003Oooool3o`3o00Co003o4_ooool4ool001?ooooo0oooO`0;ooooo`?oogl04oooool3o`3o
00Co003o5Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`0;ooooo`?oogl02oooool3oomo
00;ooooo1?oo000=ooooo`Co0?l06?ooool3o`3o00Co003o1?ooool4ool000[ooooo00?o0000oooo
ooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000Booooo`Coogl01Oooool4o`3o00Oo
oooo1?oo000=ooooo`?o0?l00ol00?l=ooooo`Coogl01Oooool3o`3o00Co003o4_ooool4ool001?o
oooo0oooO`0;ooooo`?oogl00_ooool4ool001Sooooo1?oo000=ooooo`?o0?l01?l00?lBooooo`Co
o`004oooool3oomo00_ooooo0oooO`0Cooooo`?o0?l01?l00?lEooooo`?o0?l01?l00?l4ooooo`Co
o`004oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo1?l0o`0Hooooo`?o
0?l01?l00?l4ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooo
ooooo`04ool001;ooooo1?ooO`05ooooo`Co0?l01oooool4ool000gooooo0ol0o`03o`00o`gooooo
1?ooO`05ooooo`?o0?l01?l00?lYooooo`?oogl02oooool3oomo00;ooooo1?oo000Hooooo`Coo`00
3Oooool3o`3o00Co003o4_ooool4ool001?ooooo0oooO`0;ooooo`?oogl04oooool4o`3o01Sooooo
0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`Coo`00
3Oooool4o`3o01Sooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo
000Vooooo`03o`000?oooooooooo00Coo`004_ooool4oomo00Gooooo1?l0o`07ooooo`Coo`003Ooo
ool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00obWooooo0oooO`0;ooooo`?oogl00_oo
ool4ool001Sooooo1?oo000=ooooo`?o0?l01?l00?lBooooo`Coo`004oooool3oomo00_ooooo0ooo
O`0Cooooo`Co0?l06?ooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`0;ooooo`?oogl02ooo
ool3oomo00;ooooo1?oo000=ooooo`Co0?l06?ooool3o`3o00Co003o1?ooool4ool000[ooooo00?o
0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool01?oo000Booooo`Coogl01Oooool4
o`3o00Oooooo1?oo000=ooooo`?o0?l00ol00?l=ooooo`Coogl01Oooool3o`3o00Co003o:Oooool3
oomo013ooooo1?oo000Hooooo`Coo`003Oooool3o`3o00Co003o4_ooool4ool001?ooooo0oooO`0;
ooooo`?oogl04oooool4o`3o01Sooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl02oooool3
oomo00_ooooo0oooO`02ooooo`Coo`003Oooool4o`3o01Sooooo0ol0o`04o`00o`Cooooo1?oo000:
ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00Coo`004_ooool4oomo
00Gooooo1?l0o`07ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00
obWooooo0oooO`0@ooooo`Coo`006?ooool4ool000gooooo0ol0o`04o`00oa;ooooo1?oo000Coooo
o`?oogl02oooool3oomo01?ooooo1?l0o`0Hooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo
00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo1?l0o`0Hooooo`?o0?l01?l00?l4oooo
o`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool3o`0001Kooooo1?ooO`05ooooo`Co
0?l01oooool4ool000gooooo0ol0o`03o`00o`gooooo1?ooO`05ooooo`?o0?l01?l00?lYooooo`?o
ogl04?ooool4ool001Sooooo1?oo000=ooooo`?o0?l01?l00?lBooooo`Coo`004oooool3oomo027o
oooo1?l0o`0Hooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;ooooo`?o
ogl00_ooool4ool000gooooo1?l0o`0Hooooo`?o0?l01?l00?l4ooooo`Coo`002Oooool2o`0000[o
oooo000Vooooo`03o`000?oooooooooo01Kooooo1?ooO`05ooooo`Co0?l01oooool4ool000gooooo
0ol0o`03o`00o`gooooo1?ooO`05ooooo`?o0?l01?l00?lYooooo`?oogl04?ooool4ool001Sooooo
1?oo000=ooooo`?o0?l01?l00?lBooooo`Coo`004oooool3oomo027ooooo1?l0o`0Hooooo`?o0?l0
1?l00?l4ooooo`Coo`004oooool3oomo00_ooooo0oooO`0;ooooo`?oogl04oooool4o`3o01Sooooo
0ol0o`04o`00o`Cooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oo
oooooooo01Kooooo1?ooO`05ooooo`Co0?l01oooool4ool000gooooo0ol0o`03o`00o`gooooo1?oo
O`05ooooo`?o0?l01?l00?lYooooo`?oogl04?ooool4ool001Sooooo1?oo000=ooooo`?o0?l01?l0
0?lYooooo`?oogl08Oooool4o`3o01Sooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl02ooo
ool3oomo00_ooooo0oooO`0Cooooo`Co0?l06?ooool3o`3o00Co003o1?ooool4ool000[ooooo00?o
0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool05_ooool4oomo00Gooooo1?l0o`07
ooooo`Coo`003Oooool3o`3o00?o003o3Oooool4oomo00Gooooo0ol0o`04o`00obWooooo0oooO`0@
ooooo`Coo`006?ooool4ool000gooooo0ol0o`04o`00obWooooo0oooO`0Qooooo`Co0?l06?ooool3
o`3o00Co003o1?ooool4ool001?ooooo0oooO`0;ooooo`?oogl02oooool3oomo01?ooooo1?l0o`0H
ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0
003oooooooooo`0Fooooo`Coogl01Oooool4o`3o01Sooooo0ol0o`03o`00o`gooooo1?ooO`05oooo
o`?o0?l01?l00?lYooooo`?oogl04?ooool4ool001Sooooo1?oo000=ooooo`?o0?l01?l00?lYoooo
o`?oogl08Oooool4o`3o01Sooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl02oooool3oomo
00_ooooo0oooO`0Cooooo`Co0?l06?ooool3o`3o00Co003o1?ooool4ool000[ooooo00?o0000oooo
ooooool02?ooool002Kooooo00?o0000ooooooooool05_ooool4oomo00Gooooo1?l0o`0Hooooo`?o
0?l00ol00?l=ooooo`Coogl01Oooool3o`3o00Co003o:Oooool3oomo013ooooo1?oo000Hooooo`Co
o`003Oooool3o`3o00Co003o:Oooool3oomo027ooooo1?l0o`0Hooooo`?o0?l01?l00?lKooooo`?o
ogl02oooool3oomo00_ooooo0oooO`0Cooooo`Co0?l06?ooool3o`3o00Co003o1?ooool4ool000[o
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool05_ooool4oomo00Gooooo
1?l0o`0Kooooo`?o003o3Oooool4oomo00Gooooo0ol0o`04o`00obWooooo0oooO`0@ooooo`Coo`00
6?ooool4ool000gooooo0ol0o`04o`00obWooooo0oooO`0Qooooo`Co0?l06?ooool3o`3o00Co003o
6oooool3oomo00_ooooo0oooO`0;ooooo`?oogl04oooool4o`3o01Sooooo0ol0o`04o`00o`Cooooo
1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo01Kooooo1?oo
O`05ooooo`Co0?l06oooool3o`00o`gooooo1?ooO`05ooooo`?o0?l01?l00?lYooooo`?oogl04?oo
ool4ool001Sooooo1?oo000=ooooo`?o0?l01?l00?lYooooo`?oogl08Oooool4o`3o01Sooooo0ol0
o`04o`00oa_ooooo0oooO`0;ooooo`?oogl02oooool3oomo01?ooooo1?l0o`0Hooooo`?o0?l01?l0
0?l4ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`0O
ooooo`Co0?l06oooool3o`00o`gooooo1?ooO`05ooooo`?o0?l01?l00?lYooooo`?oogl04?ooool4
ool001Sooooo1?oo000=ooooo`?o0?l01?l00?m=ooooo`Co0?l06?ooool3o`3o00Co003o6oooool3
oomo00_ooooo0oooO`0;ooooo`?oogl04oooool4o`3o01Sooooo0ol0o`04o`00o`Cooooo1?oo000:
ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo01oooooo1?l0o`0Koooo
o`?o003o3Oooool4oomo00Gooooo0ol0o`04o`00obWooooo0oooO`0@ooooo`Coo`00:Oooool3o`3o
00Co003oCOooool4o`3o01Sooooo0ol0o`04o`00oa_ooooo0oooO`0Iooooo`?oogl04oooool4o`3o
01Sooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03
o`000?oooooooooo01oooooo1?l0o`0Kooooo`?o003o3Oooool4oomo00Gooooo0ol0o`04o`00obWo
oooo0oooO`0@ooooo`Coo`00:Oooool3o`3o00Co003oCOooool4o`3o01Sooooo0ol0o`04o`00oa_o
oooo0oooO`0Iooooo`?oogl04oooool4o`3o01Sooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`03
o`000?oooooooooo00Sooooo000Vooooo`?o00007oooool4o`3o01_ooooo0ol00?l=ooooo`Coogl0
1Oooool3o`3o00Co003o:Oooool3oomo013ooooo1?oo000Yooooo`?o0?l01?l00?m=ooooo`Co0?l0
6?ooool3o`3o00Co003o6oooool3oomo01Wooooo0oooO`0Cooooo`Co0?l06?ooool3o`3o00Co003o
1?ooool4ool000Wooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`0Oooooo`Co0?l06ooo
ool3o`00o`gooooo1?ooO`05ooooo`?o0?l01?l00?llooooo`Coo`00:Oooool3o`3o00Co003oJOoo
ool3o`3o00Co003o6oooool3oomo01Wooooo0oooO`0Cooooo`Co0?l06?ooool3o`3o00Co003o1?oo
ool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool07oooool4
o`3o01_ooooo0ol00?l=ooooo`Coogl01Oooool3o`3o00Co003o??ooool4ool002Wooooo0ol0o`04
o`00ofWooooo0ol0o`04o`00oa_ooooo0oooO`0Iooooo`?oogl04oooool4o`3o01Sooooo0ol0o`04
o`00oa;ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool07oooool4o`3o
01_ooooo0ol00?lFooooo`?o0?l01?l00?llooooo`Coo`00:Oooool3o`3o00Co003oJOooool3o`3o
00Co003o6oooool3oomo01Wooooo0oooO`0Cooooo`Co0?l06?ooool3o`3o00Co003o4_ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`0Oooooo`Co0?l06oooool3o`00oaKo
oooo0ol0o`04o`00occooooo1?oo000Yooooo`?o0?l01?l00?mYooooo`?o0?l01?l00?lKooooo`?o
ogl06Oooool3oomo01?ooooo1?l0o`0Hooooo`?o0?l01?l00?lBooooo`03o`000?oooooooooo00So
oooo000Vooooo`03o`000?oooooooooo01oooooo1?l0o`0Kooooo`?o003o5_ooool3o`3o00Co003o
??ooool4ool002Wooooo0ol0o`04o`00ofWooooo0ol0o`04o`00oa_ooooo0oooO`0Iooooo`?oogl0
4oooool4o`3o01Sooooo0ol0o`04o`00oa;ooooo00?o0000ooooooooool02?ooool002Kooooo00?o
0000ooooooooool07oooool4o`3o03Cooooo0ol0o`04o`00occooooo1?oo000Yooooo`?o0?l01?l0
0?mYooooo`?o0?l01?l00?lKooooo`?oogl06Oooool3oomo01?ooooo1?l0o`0Hooooo`?o0?l01?l0
0?lBooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo01oooooo1?l0o`0d
ooooo`?o0?l01?l00?llooooo`Coo`00:Oooool3o`3o00Co003oJOooool3o`3o00Co003o6oooool3
oomo01Wooooo0oooO`0Cooooo`Co0?l06?ooool3o`3o00Co003o4_ooool00ol0003oooooooooo`08
ooooo`009_ooool00ol0003oooooooooo`0Oooooo`Co0?l0=?ooool3o`3o00Co003o??ooool4ool0
02Wooooo0ol0o`04o`00ofWooooo0ol0o`04o`00oa_ooooo0oooO`0Iooooo`?oogl04oooool4o`3o
01Sooooo0ol0o`04o`00oa;ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooooooo
ool07oooool4o`3o03Cooooo0ol0o`04o`00occooooo1?oo000Yooooo`?o0?l01?l00?mYooooo`?o
0?l01?l00?lKooooo`?oogl06Oooool3oomo01?ooooo1?l0o`0Hooooo`?o0?l01?l00?lBooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo01oooooo1?l0o`0dooooo`?o0?l0
1?l00?llooooo`Coo`00:Oooool3o`3o00Co003oJOooool3o`3o00Co003o6oooool3oomo01Wooooo
0oooO`0Cooooo`Co0?l06?ooool3o`3o00Co003o4_ooool00ol0003oooooooooo`08ooooo`009_oo
ool3o`0001oooooo1?l0o`0dooooo`?o0?l01?l00?llooooo`Coo`00:Oooool3o`3o00Co003oJOoo
ool3o`3o00Co003o6oooool3oomo02oooooo1?l0o`0Hooooo`?o0?l01?l00?lAooooo`;o00002_oo
ool002Kooooo00?o0000ooooooooool07oooool4o`3o03Cooooo0ol0o`04o`00occooooo1?oo000Y
ooooo`?o0?l01?l00?mYooooo`?o0?l01?l00?lKooooo`?oogl0;oooool4o`3o01Sooooo0ol0o`04
o`00oa;ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool07oooool4o`3o
03Cooooo0ol0o`04o`00occooooo1?oo000Yooooo`?o0?l01?l00?mYooooo`?o0?l01?l00?lKoooo
o`?oogl0;oooool4o`3o01_ooooo1?l00?lBooooo`03o`000?oooooooooo00Sooooo000Vooooo`03
o`000?oooooooooo01oooooo1?l0o`0dooooo`?o0?l01?l00?llooooo`Coo`00:Oooool3o`3o00Co
003oJOooool3o`3o00Co003o6oooool3oomo02oooooo1?l0o`0Kooooo`Co003o4_ooool00ol0003o
ooooooooo`08ooooo`009_ooool00ol0003oooooooooo`0Oooooo`Co0?l0=?ooool3o`3o00Co003o
??ooool4ool002cooooo1?l00?mYooooo`?o0?l01?l00?lKooooo`?oogl0;oooool4o`3o01_ooooo
1?l00?lBooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo01oooooo1?l0
o`0dooooo`?o0?l01?l00?llooooo`Coo`00;?ooool4o`00ofWooooo0ol0o`04o`00oa_ooooo0ooo
O`0_ooooo`Co0?l06oooool4o`00oa;ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000
ooooooooool07oooool4o`3o03Cooooo0ol0o`04o`00occooooo1?oo000/ooooo`Co003oJOooool3
o`3o00Co003o6oooool3oomo02oooooo1?l0o`0Kooooo`Co003o4_ooool00ol0003oooooooooo`08
ooooo`009_ooool00ol0003oooooooooo`0Oooooo`Co0?l0=?ooool3o`3o00Co003o??ooool4ool0
02cooooo1?l00?mYooooo`?o0?l01?l00?lKooooo`?oogl0;oooool4o`3o01_ooooo1?l00?lBoooo
o`03o`000?oooooooooo00Sooooo000Aooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`?o
00001oooool00ol0003oooooooooo`0Oooooo`Co0?l0=?ooool3o`3o00Co003oK?ooool4o`00ofWo
oooo0ol0o`04o`00oa_ooooo0oooO`0_ooooo`Co0?l06oooool4o`00oa;ooooo00?o0000oooooooo
ool02?ooool0013ooooo00Co0000oooooooooooo00002Oooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo01oooooo1?l0o`0dooooo`?o0?l01?l00?m/ooooo`Co003oJOooool3o`3o00Co
003o6oooool3oomo02oooooo1?l0o`0Kooooo`Co003o4_ooool00ol0003oooooooooo`08ooooo`00
4?ooool01?l0003oooooooooool00009ooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool07oooool4o`3o03Cooooo0ol0o`04o`00ofcooooo1?l00?mYooooo`?o0?l01?l00?lKooooo`?o
ogl0;oooool4o`3o01_ooooo1?l00?lBooooo`03o`000?oooooooooo00Sooooo000@ooooo`04o`00
0?ooooooooooo`0000Wooooo00?o0000ooooooooool01_ooool4o`0001kooooo1?l0o`0dooooo`?o
0?l01?l00?m/ooooo`Co003oJOooool3o`3o00Co003o6oooool3oomo04kooooo1?l00?l?ooooo`Co
00002_ooool0013ooooo00Co0000oooooooooooo00002Oooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo01oooooo1?l0o`0dooooo`?o0?l01?l00?m/ooooo`Co003oJOooool3o`3o00Co
003o6oooool3oomo04kooooo1?l00?lBooooo`03o`000?oooooooooo00Sooooo000@ooooo`04o`00
0?ooooooooooo`0000Sooooo0_l00008ooooo`03o`000?oooooooooo01oooooo1?l0o`0dooooo`?o
0?l01?l00?m/ooooo`Co003oJOooool3o`3o00Co003o6oooool3oomo04kooooo1?l00?lBooooo`03
o`000?oooooooooo00Sooooo000Aooooo`;o00002_ooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo01oooooo1?l0o`0dooooo`?o0?l01?l00?m/ooooo`Co003oJOooool3o`3o00Co003o
6oooool3oomo04kooooo1?l00?lBooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oo
oooooooo01oooooo1?l0o`0dooooo`?o0?l01?l00?m/ooooo`Co003oJOooool3o`3o00Co003o6ooo
ool3oomo04kooooo1?l00?lBooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooo
oooo01oooooo1?l0o`0dooooo`?o0?l01?l00?m/ooooo`Co003oJOooool3o`3o00Co003o6oooool3
oomo04kooooo1?l00?lBooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo
01oooooo1?l0o`0dooooo`?o0?l01?l00?m/ooooo`Co003oJOooool3o`3o00Co003o6oooool3oomo
04kooooo1?l00?lBooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo01oo
oooo1?l0o`0dooooo`?o0?l01?l00?m/ooooo`Co003oJOooool3o`3o00Co003o6oooool3oomo04ko
oooo1?l00?lBooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo01oooooo
1?l0o`0dooooo`?o0?l01?l00?m/ooooo`Co003oK?ooool4o`00oa_ooooo0oooO`1>ooooo`Co003o
4_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`0Oooooo`Co0?l0=?oo
ool3o`3o00Co003oK?ooool4o`00ofcooooo1?l00?lKooooo`?oogl0C_ooool4o`00oa;ooooo00?o
0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool07oooool4o`3o03Cooooo0ol0o`04
o`00ofcooooo1?l00?m/ooooo`Co003o6oooool3oomo06Cooooo00?o0000ooooooooool02?ooool0
02Kooooo0ol0000Oooooo`Co0?l0=?ooool3o`3o00Co003oK?ooool4o`00ofcooooo1?l00?lKoooo
o`?oogl0Hoooool2o`0000[ooooo000Vooooo`03o`000?oooooooooo01oooooo1?l0o`0dooooo`?o
0?l01?l00?m/ooooo`Co003oK?ooool4o`00oh;ooooo00?o0000ooooooooool02?ooool002Kooooo
00?o0000ooooooooool0Eoooool3o`3o00Co003oK?ooool4o`00ofcooooo1?l00?n2ooooo`03o`00
0?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00ofcooooo1?l0
0?m/ooooo`Co003oP_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1G
ooooo`?o0?l01?l00?m/ooooo`Co003oK?ooool4o`00oh;ooooo00?o0000ooooooooool02?ooool0
02Kooooo00?o0000ooooooooool0Eoooool3o`3o00Co003oK?ooool4o`00ofcooooo1?l00?n2oooo
o`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00ofco
oooo1?l00?m/ooooo`Co003oP_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooo
ooooo`1Gooooo`?o0?l01?l00?m/ooooo`Co003oK?ooool4o`00oh;ooooo00?o0000ooooooooool0
2?ooool002Kooooo00?o0000ooooooooool0Eoooool3o`3o00Co003oK?ooool4o`00ofcooooo1?l0
0?n2ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04
o`00ofcooooo1?l00?m/ooooo`Co003oP_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0
003oooooooooo`1Gooooo`?o0?l01?l00?m/ooooo`Co003oK?ooool4o`00oh;ooooo00?o0000oooo
ooooool02?ooool002Kooooo0ol0001Gooooo`?o0?l01?l00?m/ooooo`Co003oK?ooool4o`00oh7o
oooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?m/ooooo`Co003o
K?ooool4o`00oh;ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eooo
ool3o`3o00Co003oK?ooool4o`00ofcooooo1?l00?n2ooooo`03o`000?oooooooooo00Sooooo000V
ooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00ofcooooo1?l00?m/ooooo`Co003oP_ooool0
0ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?m/oooo
o`Co003oK?ooool4o`00oh;ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooooooo
ool0Eoooool3o`3o00Co003oK?ooool4o`00ofcooooo1?l00?n2ooooo`03o`000?oooooooooo00So
oooo000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00ofcooooo1?l00?m/ooooo`Co003o
P_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l0
0?m/ooooo`Co003oK?ooool4o`00oh;ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000
ooooooooool0Eoooool3o`3o00Co003oK?ooool4o`00ofcooooo1?l00?n2ooooo`03o`000?oooooo
oooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00ofcooooo1?l00?m/oooo
o`Co003oP_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o
0?l01?l00?m/ooooo`Co003oK?ooool4o`00oh;ooooo00?o0000ooooooooool02?ooool002Kooooo
0ol0001Gooooo`?o0?l01?l00?m/ooooo`Co003oK?ooool4o`00oh7ooooo0_l0000:ooooo`009_oo
ool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?m/ooooo`Co003oK?ooool4o`00oh;ooooo00?o
0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool3o`3o00Co003oK?ooool4
o`00ofcooooo1?l00?n2ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo
05Oooooo0ol0o`04o`00ofcooooo1?l00?m/ooooo`Co003oP_ooool00ol0003oooooooooo`08oooo
o`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?m/ooooo`Co003oK?ooool4o`00oh;o
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool3o`3o00Co003o
K?ooool4o`00ofcooooo1?l00?n2ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oo
oooooooo05Oooooo0ol0o`04o`00ofcooooo1?l00?m/ooooo`Co003oP_ooool00ol0003ooooooooo
o`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?m/ooooo`Co003oK?ooool4
o`00oh;ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool3o`3o
00Co003oK?ooool4o`00ofcooooo1?l00?n2ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03
o`000?oooooooooo05Oooooo0ol0o`04o`00ofcooooo1?l00?m/ooooo`Co003oP_ooool00ol0003o
ooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?m/ooooo`Co003o
K?ooool4o`00oh;ooooo00?o0000ooooooooool02?ooool002Kooooo0ol0001Gooooo`?o0?l01?l0
0?m/ooooo`Co003oK?ooool4o`00oh7ooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`1G
ooooo`?o0?l01?l00?m/ooooo`Co003oK?ooool4o`00oh;ooooo00?o0000ooooooooool02?ooool0
02Kooooo00?o0000ooooooooool0Eoooool3o`3o00Co003oK?ooool4o`00ofcooooo1?l00?n2oooo
o`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00ofco
oooo1?l00?m/ooooo`Co003oP_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooo
ooooo`1Gooooo`?o0?l01?l00?m/ooooo`Co003oK?ooool4o`00oh;ooooo00?o0000ooooooooool0
2?ooool002Kooooo00?o0000ooooooooool0Eoooool3o`3o00Co003oK?ooool4o`00ofcooooo1?l0
0?n2ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04
o`00ofcooooo1?l00?m/ooooo`Co003oP_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0
003oooooooooo`1Gooooo`?o0?l01?l00?m/ooooo`Co003oK?ooool4o`00oh;ooooo00?o0000oooo
ooooool02?ooool000_ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0ol00003ooooo`?o
00001oooool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?m/ooooo`Co003oK?ooool4o`00oh;o
oooo00?o0000ooooooooool02?ooool000[ooooo00Co0000oooooooooooo00002Oooool01Ol0003o
oooooooooooooooo000000?ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`1Goooo
o`?o0?l01?l00?m/ooooo`Co003oK?ooool4o`00oh;ooooo00?o0000ooooooooool02?ooool000[o
oooo00Co0000oooooooooooo00002Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool0Eoooool3o`3o00Co003oK?ooool4o`00ofcooooo1?l00?n2oooo
o`03o`000?oooooooooo00Sooooo000:ooooo`04o`000?ooooooooooo`0000Wooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`04ooooo`Co0000E_ooool3o`3o00Co003oK?ooool4o`00
ofcooooo1?l00?moooooo`Co00002_ooool000[ooooo00Co0000oooooooooooo00002Oooool01?l0
003oooooooooooooool4o`0000Oooooo00?o0000ooooooooool0Eoooool3o`3o00Co003oK?ooool4
o`00ofcooooo1?l00?n2ooooo`03o`000?oooooooooo00Sooooo000:ooooo`04o`000?oooooooooo
o`0000Sooooo0_l00003ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0Eoooool3
o`3o00Co003oK?ooool4o`00ofcooooo1?l00?n2ooooo`03o`000?oooooooooo00Sooooo000;oooo
o`;o00002_ooool01?l0003oooooooooooooool5o`0000Kooooo00?o0000ooooooooool0Eoooool3
o`3o00Co003oK?ooool4o`00ofcooooo1?l00?n2ooooo`03o`000?oooooooooo00Sooooo000Voooo
o`03o`000?oooooooooo05Oooooo0ol0o`04o`00omcooooo1?l00?n2ooooo`03o`000?oooooooooo
00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00omcooooo1?l00?n2ooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00omcooooo
1?l00?n2ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo0ol0
o`04o`00omcooooo1?l00?n2ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooo
oooo05Oooooo0ol0o`04o`00omcooooo1?l00?n2ooooo`03o`000?oooooooooo00Sooooo000Voooo
o`03o`000?oooooooooo05Oooooo0ol0o`04o`00omcooooo1?l00?n2ooooo`03o`000?oooooooooo
00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00omcooooo1?l00?n2ooooo`03
o`000?oooooooooo00Sooooo000Vooooo`?o0000Eoooool3o`3o00Co003og?ooool4o`00oh7ooooo
0_l0000:ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?oLooooo`Co003oP_oo
ool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?oL
ooooo`Co003oP_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Goooo
o`?o0?l01?l00?oLooooo`Co003oP_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003o
ooooooooo`1Gooooo`?o0?l01?l00?oLooooo`Co003oP_ooool00ol0003oooooooooo`08ooooo`00
9_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?oLooooo`Co003oP_ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?oLooooo`Co003oP_oo
ool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?oL
ooooo`Co003oP_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Goooo
o`?o0?l01?l00?ooooooof?ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooooooo
ool0Eoooool3o`3o00Co003ooooooomSooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`00
0?oooooooooo05Oooooo0ol0o`04o`00ooooooooHoooool00ol0003oooooooooo`08ooooo`009_oo
ool3o`0005Oooooo0ol0o`04o`00ooooooooH_ooool2o`0000[ooooo000Vooooo`03o`000?oooooo
oooo05Oooooo0ol0o`04o`00ooooooooHoooool00ol0003oooooooooo`08ooooo`009_ooool00ol0
003oooooooooo`1Gooooo`?o0?l01?l00?ooooooof?ooooo00?o0000ooooooooool02?ooool002Ko
oooo00?o0000ooooooooool0Eoooool3o`3o00Co003ooooooomSooooo`03o`000?oooooooooo00So
oooo000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00ooooooooHoooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?ooooooof?ooooo00?o
0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool3o`3o00Co003ooooooomS
ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00
ooooooooHoooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o
0?l01?l00?ooooooof?ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0
Eoooool3o`3o00Co003ooooooomSooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oo
oooooooo05Oooooo0ol0o`04o`00ooooooooHoooool00ol0003oooooooooo`08ooooo`009_ooool3
o`0005Oooooo0ol0o`04o`00ooooooooH_ooool2o`0000[ooooo000Vooooo`03o`000?oooooooooo
05Oooooo0ol0o`04o`00ooooooooHoooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003o
ooooooooo`1Gooooo`?o0?l01?l00?ooooooof?ooooo00?o0000ooooooooool02?ooool002Kooooo
00?o0000ooooooooool0Eoooool3o`3o00Co003ooooooomSooooo`03o`000?oooooooooo00Sooooo
000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00ooooooooHoooool00ol0003ooooooooo
o`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?ooooooof?ooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool3o`3o00Co003ooooooomSoooo
o`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00oooo
ooooHoooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l0
1?l00?ooooooof?ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eooo
ool3o`3o00Co003ooooooomSooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooo
oooo05Oooooo0ol0o`04o`00ooooooooHoooool00ol0003oooooooooo`08ooooo`009_ooool3o`00
05Oooooo0ol0o`04o`00ooooooooH_ooool2o`0000[ooooo000Vooooo`03o`000?oooooooooo05Oo
oooo0ol0o`04o`00ooooooooHoooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooo
ooooo`1Gooooo`?o0?l01?l00?ooooooof?ooooo00?o0000ooooooooool02?ooool002Kooooo00?o
0000ooooooooool0Eoooool3o`3o00Co003ooooooomSooooo`03o`000?oooooooooo00Sooooo000V
ooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00ooooooooHoooool00ol0003oooooooooo`08
ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?ooooooof?ooooo00?o0000oooo
ooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool3o`3o00Co003ooooooomSooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00oooooooo
Hoooool00ol0003oooooooooo`08ooooo`004Oooool2o`0000Cooooo00Co0000oooooooooooooooo
1Ol00006ooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00ooooooooHoooool00ol0003ooooo
ooooo`08ooooo`004?ooool01?l0003oooooooooool00008ooooo`04o`000?ooooooooooo`0000Ko
oooo00?o0000ooooooooool0Eoooool3o`3o00Co003ooooooomSooooo`03o`000?oooooooooo00So
oooo000@ooooo`04o`000?ooooooooooo`0000Wooooo00?o0000ooooooooool01_ooool00ol0003o
ooooooooo`1Gooooo`?o0?l01?l00?ooooooof?ooooo00?o0000ooooooooool02?ooool0013ooooo
00Co0000oooooooooooo00002_ooool00ol0003oooooooooo`05ooooo`Co0000E_ooool3o`3o00Co
003ooooooomPooooo`Co00002_ooool0013ooooo00Co0000oooooooooooo00001oooool01Ol0003o
oooooooooooooooo000000Kooooo00?o0000ooooooooool0Eoooool3o`3o00Co003ooooooomSoooo
o`03o`000?oooooooooo00Sooooo000@ooooo`04o`000?ooooooooooo`0000Oooooo00Go0000oooo
ooooooooooooo`000006ooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00ooooooooHoooool0
0ol0003oooooooooo`08ooooo`004Oooool2o`0000Wooooo0ol00007ooooo`03o`000?oooooooooo
05Oooooo0ol0o`04o`00ooooooooHoooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003o
ooooooooo`1Gooooo`?o0?l01?l00?ooooooof?ooooo00?o0000ooooooooool02?ooool002Kooooo
00?o0000ooooooooool0Eoooool3o`3o00Co003ooooooomSooooo`03o`000?oooooooooo00Sooooo
000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00ooooooooHoooool00ol0003ooooooooo
o`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?ooooooof?ooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool3o`3o00Co003ooooooomSoooo
o`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00oooo
ooooHoooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l0
1?l00?ooooooof?ooooo00?o0000ooooooooool02?ooool002Kooooo0ol0001Gooooo`?o0?l01?l0
0?ooooooof;ooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?oo
ooooof?ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool3o`3o
00Co003ooooooomSooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oo
oooo0ol0o`04o`00ooooooooHoooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooo
ooooo`1Gooooo`?o0?l01?l00?ooooooof?ooooo00?o0000ooooooooool02?ooool002Kooooo00?o
0000ooooooooool0Eoooool3o`3o00Co003ooooooomSooooo`03o`000?oooooooooo00Sooooo000V
ooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00ooooooooHoooool00ol0003oooooooooo`08
ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l00?ooooooof?ooooo00?o0000oooo
ooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool3o`3o00Co003ooooooomSooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo0ol0o`04o`00oooooooo
Hoooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`?o0?l01?l0
0?ooooooof?ooooo00?o0000ooooooooool02?ooool002Kooooo0ol0001Gooooo`?o0?l01?l00?oo
ooooof;ooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooooomVoooo
o`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKo
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?oooooo
I_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooo
oomVooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3o
ooooofKooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o
0?ooooooI_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co
0?l0oooooomVooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo
1?l0o`3oooooofKooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eooo
ool4o`3o0?ooooooI_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1G
ooooo`Co0?l0oooooomVooooo`03o`000?oooooooooo00Sooooo000Vooooo`?o0000Eoooool4o`3o
0?ooooooIOooool2o`0000[ooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3oooooofKo
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?oooooo
I_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co0?l0oooo
oomVooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo1?l0o`3o
ooooofKooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o
0?ooooooI_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1Gooooo`Co
0?l0oooooomVooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo05Oooooo
1?l0o`3oooooofKooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Eooo
ool4o`3o0?ooooooI_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1G
ooooo`Co0?l0oooooomVooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo
05Oooooo1?l0o`3oooooofKooooo00?o0000ooooooooool02?ooool002Kooooo0ol0001Gooooo`Co
0?l0oooooomUooooo`;o00002_ooool002Kooooo00?o0000ooooooooool0Eoooool4o`3o0?oooooo
I_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`3ooooool7ooooo00?o
0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0ooooooo1ooooo`03o`000?oooooo
oooo00Sooooo000Vooooo`03o`000?oooooooooo0?oooooo`Oooool00ol0003oooooooooo`08oooo
o`009_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`06ooooo`03o`00
0?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`00
0?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`00
0?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`00
0?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`00
0?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`00
0?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`00
0?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`00
0?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`00
0?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`00
0?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`00
0?oooooooooo00_ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`08ooooo`009_oo
ool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02ooo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02ooo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02ooo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02ooo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02ooo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02ooo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02ooo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02ooo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02ooo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02ooo
ool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`08ooooo`009_ooooooo`000<Go00002_ooool00?oooooomOooool00?oo
oooomOooool00?oooooomOooool00?oooooomOooool00?oooooomOooool00?oooooomOooool00?oo
oooomOooool002gooooo0ol0000:ooooo`Go00002_ooool3o`0000cooooo0ol0000:ooooo`?o0000
2oooool3o`0000cooooo00?o0000ooooooooool02_ooool3o`0000_ooooo0_l00009ooooo`?o0000
1?ooool2o`0000Gooooo0ol00003ooooo`?o00001Oooool3o`0000;ooooo1Ol00004ooooo`?o0000
0oooool3o`0000Gooooo0ol00004ooooo`?o00001?ooool3o`0000?ooooo0ol00005ooooo`?o0000
0oooool3o`0000Gooooo0ol00004ooooo`03o`000?oooooooooo00Cooooo0ol00003ooooo`?o0000
1Oooool3o`0000?ooooo0_l00005ooooo`Go00000oooool2o`0000Cooooo1Ol00002ooooo`?o0000
1?ooool5o`000003ooooool0003o000000?o00000oooool5o`0000;ooooo0ol00004ooooo`Go0000
0oooool3o`0000?ooooo1Ol00002ooooo`?o00001?ooool5o`0000;ooooo0ol00004ooooo`Go0000
0oooool00ol0003oooooooooo`03ooooo`Go00000_ooool3o`0000Cooooo1Ol00002ooooo`;o0000
1_ooool3o`0000Cooooo0_l00005ooooo`?o00000oooool3o`0000Gooooo0ol00002ooooo`Go0000
3_ooool002kooooo00?o0000ooooooooool02_ooool01?l0003oooooooooool00009ooooo`05o`00
0?ooooooooooooooool000003?ooool00ol0003oooooooooo`08ooooo`05o`000?oooooooooooooo
ool000002Oooool01Ol0003ooooooooooooooooo000000_ooooo00?o0000ooooooooool02Oooool0
1Ol0003ooooooooooooooooo000000cooooo00?o0000ooooooooool01oooool00ol0003ooooooooo
o`02ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool00oooool00ol0003ooooo
ooooo`05ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00001Oooool01Ol0003o
oooooooooooooooo000000?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Cooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo
oooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00?o
0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`05o`000?ooooooooooooooool00000
0oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0
0oooool01?l0003oooooooooool00002ooooo`04o`000?ooooooooooo`0000Cooooo00Co0000oooo
oooooooo00000oooool00ol0003oooooooooo`04ooooo`04o`000?ooooooooooo`0000;ooooo00Co
0000oooooooooooo00001?ooool01_l0003oooooooooool0003oooooo`0000?ooooo00?o0000oooo
ooooool00_ooool01?l0003oooooooooool00004ooooo`03o`000?oooooooooo00?ooooo00Ko0000
oooooooooooo0000ooooool00003ooooo`03o`000?oooooooooo00;ooooo00Ko0000oooooooooooo
0000ooooool00003ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000oooool0
0ol0003oooooooooo`04ooooo`06o`000?ooooooooooo`000?oooooo00000oooool00ol0003ooooo
ooooo`02ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool00_ooool01Ol0003o
oooooooooooooooo000000;ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooo
oooo000000?ooooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000;ooooo
00Co0000oooooooooooo00003_ooool002kooooo00?o0000ooooooooool02oooool00ol0003ooooo
ooooo`0=ooooo`03o`000?oooooooooo00Oooooo1Ol0000=ooooo`03o`000?oooooooooo00Oooooo
00Go0000ooooooooooooooooo`00000<ooooo`03o`000?oooooooooo00Sooooo00Go0000oooooooo
ooooooooo`00000=ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool00_ooool01?l0
003oooooooooool00005ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`03ooooo`04o`000?ooooooooooooooo`Go00001Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04ooooo`05o`00
0?ooooooooooooooool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`00
00Gooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00?ooooo00Co0000oooooooooooooooo1Ol00005ooooo`03o`000?oooooooooo00Gooooo00?o0000
ooooooooool00oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00Go0000oooooooooooooooo
o`000003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`05ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool01Oooool01Ol0003o
oooooooooooooooo000000Wooooo00Go0000ooooooooooooooooo`00000@ooooo`00;_ooool00ol0
003oooooooooo`0<ooooo`03o`000?oooooooooo00[ooooo0_l0000:ooooo`04o`000?oooooooooo
o`0000kooooo00?o0000ooooooooool01oooool4o`0000gooooo00?o0000ooooooooool02Oooool3
o`0000_ooooo1?l00008ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00001Ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00?ooooo0_l00006ooooo`05o`000?oo
ooooooooooooool000000_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool00oooool01?l0003oooooooooooooool4o`0000Kooooo00?o0000oooooooo
ool01?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00;ooooo0ol00006ooooo`03
o`000?oooooooooo00;ooooo1?l00006ooooo`05o`000?ooooooooooooooool000000_ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooooooool01_ooool00ol0
003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00_ooool2o`00
00Oooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00Cooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04o`0000Oooooo00?o0000
ooooooooool00oooool00ol0003oooooooooo`05ooooo`04o`000?ooooooooooooooo`?o00001ooo
ool01?l0003oooooooooooooool4o`0000Gooooo0_l00003ooooo`04o`000?ooooooooooo`0000Go
oooo0_l00004ooooo`03o`000?oooooooooo00Gooooo0_l00005ooooo`03o`000?oooooooooo00go
oooo000^ooooo`03o`000?oooooooooo00Wooooo00Go0000ooooooooooooooooo`00000=ooooo`03
o`000?oooooooooo00Sooooo00?o0000ooooool000002_ooool4o`0000[ooooo00?o0000oooooooo
ool03oooool00ol0003oooooooooo`07ooooo`05o`000?ooooooooooooooool000002Oooool01Ol0
003ooooooooooooooooo000000Sooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool0
0005ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool01Ol0003ooooooooo
oooooooo000000?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool00_ooool00ol0003oooooo`000006ooooo`04o`00
0?ooooooooooooooo`Co00001_ooool01Ol0003ooooooooooooooooo000000Wooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`03ooooo`05o`000?ooooooooooooooool000000oooool0
0ol0003oooooooooo`03ooooo`05o`000?ooooooooooooooool000000oooool01Ol0003ooooooooo
oooooooo000000?ooooo00Co0000oooooooooooo00000_ooool01Ol0003ooooooooooooooooo0000
00?ooooo00Go0000ooooooooooooooooo`000005ooooo`07o`000?ooooooooooooooool0003ooooo
o`000003ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03ooooo`05
o`000?ooooooooooooooool000000oooool01_l0003oooooooooool0003oooooo`0000Cooooo00Ko
0000ooooooooooooooooo`000?ooool4o`0000Cooooo00Oo0000ooooooooooooooooo`000?oooooo
000000Oooooo00Go0000ooooooooooooooooo`000005ooooo`05o`000?ooooooooooooooool00000
0oooool00ol0003oooooo`000003ooooo`05o`000?ooooooooooooooool000000oooool00ol0003o
ooooo`000003ooooo`03o`000?oooooooooo00Gooooo00Co0000oooooooooooo00000_ooool00ol0
003oooooooooo`05ooooo`05o`000?ooooooooooooooool000002Oooool00ol0003oooooo`000003
ooooo`03o`000?oooooooooo00cooooo000]ooooo`;o00002oooool01Ol0003ooooooooooooooooo
000000Wooooo00Go0000ooooooooooooooooo`00000;ooooo`;o00002_ooool00ol0003ooooooooo
o`0<ooooo`03o`000?oooooooooo00[ooooo00Go0000ooooooooooooooooo`000009ooooo`05o`00
0?ooooooooooooooool000002Oooool01Ol0003ooooooooooooooooo000000Oooooo0_l00004oooo
o`04o`000?ooooooooooo`0000Cooooo0_l00004ooooo`;o00001_ooool2o`0000?ooooo00Go0000
ooooooooooooooooo`000004ooooo`;o00000oooool01Ol0003ooooooooooooooooo000000Cooooo
0_l00005ooooo`;o00001Oooool2o`0000?ooooo00?o0000ooooooooool01_ooool2o`0000Cooooo
00?o0000ooooooooool01Oooool2o`0000?ooooo00Go0000ooooooooooooooooo`000004ooooo`;o
00000oooool01Ol0003ooooooooooooooooo000000Cooooo0_l00003ooooo`05o`000?oooooooooo
ooooool000000oooool01Ol0003ooooooooooooooooo000000;ooooo00Co0000oooooooooooo0000
0oooool01Ol0003ooooooooooooooooo000000;ooooo0_l00005ooooo`07o`000?oooooooooooooo
ool0003oooooo`000003ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooo`00
0003ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooooooool2o`0000Co
oooo00Oo0000ooooooooooooooooo`000?oooooo000000Oooooo00Go0000ooooooooooooooooo`00
0002ooooo`03o`000?oooooooooo00Cooooo00Oo0000ooooooooooooooooo`000?oooooo000000?o
oooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo000000?ooooo00Go0000oooo
ooooooooooooo`000003ooooo`03o`000?oooooo000000?ooooo00Go0000ooooooooooooooooo`00
0003ooooo`04o`000?ooooooooooo`0000;ooooo00Go0000ooooooooooooooooo`000003ooooo`03
o`000?oooooooooo00;o00001Oooool01ol0003ooooooooooooooooo0000ooooool000000oooool0
0ol0003oooooooooo`0<ooooo`00;_ooool00ol0003oooooooooo`0:ooooo`?o00002oooool3o`00
00gooooo00?o0000ooooooooool02?ooool5o`0000_ooooo0_l0000:ooooo`Go00002_ooool3o`00
00_ooooo0ol00009ooooo`03o`000?oooooooooo00?ooooo0_l00006ooooo`03o`000?oooooooooo
00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`02ooooo`?o00001_ooool00ol0
003oooooooooo`02ooooo`?o00001_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Cooooo00Co0000oooooooooooooooo1Ol00005ooooo`03o`000?oooooooooo00?ooooo0_l00006
ooooo`04o`000?ooooooooooooooo`Go00001Oooool00ol0003oooooooooo`02ooooo`?o00001_oo
ool00ol0003oooooooooo`02ooooo`?o00001Oooool3o`0000Cooooo0_l00005ooooo`?o00001?oo
ool00ol0003oooooooooo`04ooooo`?o00000oooool3o`0000Gooooo0ol00003ooooo`?o00001Ooo
ool3o`0000Gooooo00?o0000ooooooooool00oooool3o`0000;ooooo1Ol00004ooooo`?o00001?oo
ool2o`0000Gooooo0ol00002ooooo`Go00001?ooool3o`0000?ooooo0ol00005ooooo`?o00000ooo
ool3o`0000Gooooo0ol00004ooooo`;o00001Oooool3o`0000Cooooo00?o0000ooooooooool01?oo
ool3o`0000?ooooo0ol0000?ooooo`00\
\>"],
  ImageRangeCache->{{{0, 499}, {308, 0}} -> {-2.06768, -0.0193545, 0.0713992, \
0.000909726}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
%%ImageSize: 500 309.017 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0217391 0.0310559 0.0147151 2.29402 [
[.01863 -0.0125 -3 -9 ]
[.01863 -0.0125 3 0 ]
[.04969 -0.0125 -3 -9 ]
[.04969 -0.0125 3 0 ]
[.08075 -0.0125 -3 -9 ]
[.08075 -0.0125 3 0 ]
[.1118 -0.0125 -3 -9 ]
[.1118 -0.0125 3 0 ]
[.14286 -0.0125 -3 -9 ]
[.14286 -0.0125 3 0 ]
[.17391 -0.0125 -3 -9 ]
[.17391 -0.0125 3 0 ]
[.20497 -0.0125 -3 -9 ]
[.20497 -0.0125 3 0 ]
[.23602 -0.0125 -3 -9 ]
[.23602 -0.0125 3 0 ]
[.26708 -0.0125 -3 -9 ]
[.26708 -0.0125 3 0 ]
[.29814 -0.0125 -6 -9 ]
[.29814 -0.0125 6 0 ]
[.32919 -0.0125 -6 -9 ]
[.32919 -0.0125 6 0 ]
[.36025 -0.0125 -6 -9 ]
[.36025 -0.0125 6 0 ]
[.3913 -0.0125 -6 -9 ]
[.3913 -0.0125 6 0 ]
[.42236 -0.0125 -6 -9 ]
[.42236 -0.0125 6 0 ]
[.45342 -0.0125 -6 -9 ]
[.45342 -0.0125 6 0 ]
[.48447 -0.0125 -6 -9 ]
[.48447 -0.0125 6 0 ]
[.51553 -0.0125 -6 -9 ]
[.51553 -0.0125 6 0 ]
[.54658 -0.0125 -6 -9 ]
[.54658 -0.0125 6 0 ]
[.57764 -0.0125 -6 -9 ]
[.57764 -0.0125 6 0 ]
[.6087 -0.0125 -6 -9 ]
[.6087 -0.0125 6 0 ]
[.63975 -0.0125 -6 -9 ]
[.63975 -0.0125 6 0 ]
[.67081 -0.0125 -6 -9 ]
[.67081 -0.0125 6 0 ]
[.70186 -0.0125 -6 -9 ]
[.70186 -0.0125 6 0 ]
[.73292 -0.0125 -6 -9 ]
[.73292 -0.0125 6 0 ]
[.76398 -0.0125 -6 -9 ]
[.76398 -0.0125 6 0 ]
[.79503 -0.0125 -6 -9 ]
[.79503 -0.0125 6 0 ]
[.82609 -0.0125 -6 -9 ]
[.82609 -0.0125 6 0 ]
[.85714 -0.0125 -6 -9 ]
[.85714 -0.0125 6 0 ]
[.8882 -0.0125 -6 -9 ]
[.8882 -0.0125 6 0 ]
[.91925 -0.0125 -6 -9 ]
[.91925 -0.0125 6 0 ]
[.95031 -0.0125 -6 -9 ]
[.95031 -0.0125 6 0 ]
[.98137 -0.0125 -6 -9 ]
[.98137 -0.0125 6 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .01472 -6 -4.5 ]
[-0.0125 .01472 0 4.5 ]
[-0.0125 .12942 -24 -4.5 ]
[-0.0125 .12942 0 4.5 ]
[-0.0125 .24412 -18 -4.5 ]
[-0.0125 .24412 0 4.5 ]
[-0.0125 .35882 -24 -4.5 ]
[-0.0125 .35882 0 4.5 ]
[-0.0125 .47352 -18 -4.5 ]
[-0.0125 .47352 0 4.5 ]
[-0.0125 .58822 -24 -4.5 ]
[-0.0125 .58822 0 4.5 ]
[ 0 0 -0.125 0 ]
[.01863 .63053 -3 0 ]
[.01863 .63053 3 9 ]
[.04969 .63053 -3 0 ]
[.04969 .63053 3 9 ]
[.08075 .63053 -3 0 ]
[.08075 .63053 3 9 ]
[.1118 .63053 -3 0 ]
[.1118 .63053 3 9 ]
[.14286 .63053 -3 0 ]
[.14286 .63053 3 9 ]
[.17391 .63053 -3 0 ]
[.17391 .63053 3 9 ]
[.20497 .63053 -3 0 ]
[.20497 .63053 3 9 ]
[.23602 .63053 -3 0 ]
[.23602 .63053 3 9 ]
[.26708 .63053 -3 0 ]
[.26708 .63053 3 9 ]
[.29814 .63053 -6 0 ]
[.29814 .63053 6 9 ]
[.32919 .63053 -6 0 ]
[.32919 .63053 6 9 ]
[.36025 .63053 -6 0 ]
[.36025 .63053 6 9 ]
[.3913 .63053 -6 0 ]
[.3913 .63053 6 9 ]
[.42236 .63053 -6 0 ]
[.42236 .63053 6 9 ]
[.45342 .63053 -6 0 ]
[.45342 .63053 6 9 ]
[.48447 .63053 -6 0 ]
[.48447 .63053 6 9 ]
[.51553 .63053 -6 0 ]
[.51553 .63053 6 9 ]
[.54658 .63053 -6 0 ]
[.54658 .63053 6 9 ]
[.57764 .63053 -6 0 ]
[.57764 .63053 6 9 ]
[.6087 .63053 -6 0 ]
[.6087 .63053 6 9 ]
[.63975 .63053 -6 0 ]
[.63975 .63053 6 9 ]
[.67081 .63053 -6 0 ]
[.67081 .63053 6 9 ]
[.70186 .63053 -6 0 ]
[.70186 .63053 6 9 ]
[.73292 .63053 -6 0 ]
[.73292 .63053 6 9 ]
[.76398 .63053 -6 0 ]
[.76398 .63053 6 9 ]
[.79503 .63053 -6 0 ]
[.79503 .63053 6 9 ]
[.82609 .63053 -6 0 ]
[.82609 .63053 6 9 ]
[.85714 .63053 -6 0 ]
[.85714 .63053 6 9 ]
[.8882 .63053 -6 0 ]
[.8882 .63053 6 9 ]
[.91925 .63053 -6 0 ]
[.91925 .63053 6 9 ]
[.95031 .63053 -6 0 ]
[.95031 .63053 6 9 ]
[.98137 .63053 -6 0 ]
[.98137 .63053 6 9 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.01863 0 m
.01863 .00625 L
s
[(1)] .01863 -0.0125 0 1 Mshowa
.04969 0 m
.04969 .00625 L
s
[(2)] .04969 -0.0125 0 1 Mshowa
.08075 0 m
.08075 .00625 L
s
[(3)] .08075 -0.0125 0 1 Mshowa
.1118 0 m
.1118 .00625 L
s
[(4)] .1118 -0.0125 0 1 Mshowa
.14286 0 m
.14286 .00625 L
s
[(5)] .14286 -0.0125 0 1 Mshowa
.17391 0 m
.17391 .00625 L
s
[(6)] .17391 -0.0125 0 1 Mshowa
.20497 0 m
.20497 .00625 L
s
[(7)] .20497 -0.0125 0 1 Mshowa
.23602 0 m
.23602 .00625 L
s
[(8)] .23602 -0.0125 0 1 Mshowa
.26708 0 m
.26708 .00625 L
s
[(9)] .26708 -0.0125 0 1 Mshowa
.29814 0 m
.29814 .00625 L
s
[(10)] .29814 -0.0125 0 1 Mshowa
.32919 0 m
.32919 .00625 L
s
[(11)] .32919 -0.0125 0 1 Mshowa
.36025 0 m
.36025 .00625 L
s
[(12)] .36025 -0.0125 0 1 Mshowa
.3913 0 m
.3913 .00625 L
s
[(13)] .3913 -0.0125 0 1 Mshowa
.42236 0 m
.42236 .00625 L
s
[(14)] .42236 -0.0125 0 1 Mshowa
.45342 0 m
.45342 .00625 L
s
[(15)] .45342 -0.0125 0 1 Mshowa
.48447 0 m
.48447 .00625 L
s
[(16)] .48447 -0.0125 0 1 Mshowa
.51553 0 m
.51553 .00625 L
s
[(17)] .51553 -0.0125 0 1 Mshowa
.54658 0 m
.54658 .00625 L
s
[(18)] .54658 -0.0125 0 1 Mshowa
.57764 0 m
.57764 .00625 L
s
[(19)] .57764 -0.0125 0 1 Mshowa
.6087 0 m
.6087 .00625 L
s
[(20)] .6087 -0.0125 0 1 Mshowa
.63975 0 m
.63975 .00625 L
s
[(21)] .63975 -0.0125 0 1 Mshowa
.67081 0 m
.67081 .00625 L
s
[(22)] .67081 -0.0125 0 1 Mshowa
.70186 0 m
.70186 .00625 L
s
[(23)] .70186 -0.0125 0 1 Mshowa
.73292 0 m
.73292 .00625 L
s
[(24)] .73292 -0.0125 0 1 Mshowa
.76398 0 m
.76398 .00625 L
s
[(25)] .76398 -0.0125 0 1 Mshowa
.79503 0 m
.79503 .00625 L
s
[(26)] .79503 -0.0125 0 1 Mshowa
.82609 0 m
.82609 .00625 L
s
[(27)] .82609 -0.0125 0 1 Mshowa
.85714 0 m
.85714 .00625 L
s
[(28)] .85714 -0.0125 0 1 Mshowa
.8882 0 m
.8882 .00625 L
s
[(29)] .8882 -0.0125 0 1 Mshowa
.91925 0 m
.91925 .00625 L
s
[(30)] .91925 -0.0125 0 1 Mshowa
.95031 0 m
.95031 .00625 L
s
[(31)] .95031 -0.0125 0 1 Mshowa
.98137 0 m
.98137 .00625 L
s
[(32)] .98137 -0.0125 0 1 Mshowa
0 0 m
1 0 L
s
0 .01472 m
.00625 .01472 L
s
[(0)] -0.0125 .01472 1 0 Mshowa
0 .12942 m
.00625 .12942 L
s
[(0.05)] -0.0125 .12942 1 0 Mshowa
0 .24412 m
.00625 .24412 L
s
[(0.1)] -0.0125 .24412 1 0 Mshowa
0 .35882 m
.00625 .35882 L
s
[(0.15)] -0.0125 .35882 1 0 Mshowa
0 .47352 m
.00625 .47352 L
s
[(0.2)] -0.0125 .47352 1 0 Mshowa
0 .58822 m
.00625 .58822 L
s
[(0.25)] -0.0125 .58822 1 0 Mshowa
.125 Mabswid
0 .03766 m
.00375 .03766 L
s
0 .0606 m
.00375 .0606 L
s
0 .08354 m
.00375 .08354 L
s
0 .10648 m
.00375 .10648 L
s
0 .15236 m
.00375 .15236 L
s
0 .1753 m
.00375 .1753 L
s
0 .19824 m
.00375 .19824 L
s
0 .22118 m
.00375 .22118 L
s
0 .26706 m
.00375 .26706 L
s
0 .29 m
.00375 .29 L
s
0 .31294 m
.00375 .31294 L
s
0 .33588 m
.00375 .33588 L
s
0 .38176 m
.00375 .38176 L
s
0 .4047 m
.00375 .4047 L
s
0 .42764 m
.00375 .42764 L
s
0 .45058 m
.00375 .45058 L
s
0 .49646 m
.00375 .49646 L
s
0 .5194 m
.00375 .5194 L
s
0 .54234 m
.00375 .54234 L
s
0 .56528 m
.00375 .56528 L
s
0 .61116 m
.00375 .61116 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.01863 .61178 m
.01863 .61803 L
s
[(1)] .01863 .63053 0 -1 Mshowa
.04969 .61178 m
.04969 .61803 L
s
[(2)] .04969 .63053 0 -1 Mshowa
.08075 .61178 m
.08075 .61803 L
s
[(3)] .08075 .63053 0 -1 Mshowa
.1118 .61178 m
.1118 .61803 L
s
[(4)] .1118 .63053 0 -1 Mshowa
.14286 .61178 m
.14286 .61803 L
s
[(5)] .14286 .63053 0 -1 Mshowa
.17391 .61178 m
.17391 .61803 L
s
[(6)] .17391 .63053 0 -1 Mshowa
.20497 .61178 m
.20497 .61803 L
s
[(7)] .20497 .63053 0 -1 Mshowa
.23602 .61178 m
.23602 .61803 L
s
[(8)] .23602 .63053 0 -1 Mshowa
.26708 .61178 m
.26708 .61803 L
s
[(9)] .26708 .63053 0 -1 Mshowa
.29814 .61178 m
.29814 .61803 L
s
[(10)] .29814 .63053 0 -1 Mshowa
.32919 .61178 m
.32919 .61803 L
s
[(11)] .32919 .63053 0 -1 Mshowa
.36025 .61178 m
.36025 .61803 L
s
[(12)] .36025 .63053 0 -1 Mshowa
.3913 .61178 m
.3913 .61803 L
s
[(13)] .3913 .63053 0 -1 Mshowa
.42236 .61178 m
.42236 .61803 L
s
[(14)] .42236 .63053 0 -1 Mshowa
.45342 .61178 m
.45342 .61803 L
s
[(15)] .45342 .63053 0 -1 Mshowa
.48447 .61178 m
.48447 .61803 L
s
[(16)] .48447 .63053 0 -1 Mshowa
.51553 .61178 m
.51553 .61803 L
s
[(17)] .51553 .63053 0 -1 Mshowa
.54658 .61178 m
.54658 .61803 L
s
[(18)] .54658 .63053 0 -1 Mshowa
.57764 .61178 m
.57764 .61803 L
s
[(19)] .57764 .63053 0 -1 Mshowa
.6087 .61178 m
.6087 .61803 L
s
[(20)] .6087 .63053 0 -1 Mshowa
.63975 .61178 m
.63975 .61803 L
s
[(21)] .63975 .63053 0 -1 Mshowa
.67081 .61178 m
.67081 .61803 L
s
[(22)] .67081 .63053 0 -1 Mshowa
.70186 .61178 m
.70186 .61803 L
s
[(23)] .70186 .63053 0 -1 Mshowa
.73292 .61178 m
.73292 .61803 L
s
[(24)] .73292 .63053 0 -1 Mshowa
.76398 .61178 m
.76398 .61803 L
s
[(25)] .76398 .63053 0 -1 Mshowa
.79503 .61178 m
.79503 .61803 L
s
[(26)] .79503 .63053 0 -1 Mshowa
.82609 .61178 m
.82609 .61803 L
s
[(27)] .82609 .63053 0 -1 Mshowa
.85714 .61178 m
.85714 .61803 L
s
[(28)] .85714 .63053 0 -1 Mshowa
.8882 .61178 m
.8882 .61803 L
s
[(29)] .8882 .63053 0 -1 Mshowa
.91925 .61178 m
.91925 .61803 L
s
[(30)] .91925 .63053 0 -1 Mshowa
.95031 .61178 m
.95031 .61803 L
s
[(31)] .95031 .63053 0 -1 Mshowa
.98137 .61178 m
.98137 .61803 L
s
[(32)] .98137 .63053 0 -1 Mshowa
0 .61803 m
1 .61803 L
s
.99375 .01472 m
1 .01472 L
s
.99375 .12942 m
1 .12942 L
s
.99375 .24412 m
1 .24412 L
s
.99375 .35882 m
1 .35882 L
s
.99375 .47352 m
1 .47352 L
s
.99375 .58822 m
1 .58822 L
s
.125 Mabswid
.99625 .03766 m
1 .03766 L
s
.99625 .0606 m
1 .0606 L
s
.99625 .08354 m
1 .08354 L
s
.99625 .10648 m
1 .10648 L
s
.99625 .15236 m
1 .15236 L
s
.99625 .1753 m
1 .1753 L
s
.99625 .19824 m
1 .19824 L
s
.99625 .22118 m
1 .22118 L
s
.99625 .26706 m
1 .26706 L
s
.99625 .29 m
1 .29 L
s
.99625 .31294 m
1 .31294 L
s
.99625 .33588 m
1 .33588 L
s
.99625 .38176 m
1 .38176 L
s
.99625 .4047 m
1 .4047 L
s
.99625 .42764 m
1 .42764 L
s
.99625 .45058 m
1 .45058 L
s
.99625 .49646 m
1 .49646 L
s
.99625 .5194 m
1 .5194 L
s
.99625 .54234 m
1 .54234 L
s
.99625 .56528 m
1 .56528 L
s
.99625 .61116 m
1 .61116 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .01472 m
1 .01472 L
s
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.00621 .01472 m
.00621 .01633 L
.01242 .01633 L
.01242 .01472 L
F
.5 Mabswid
.00621 .01472 m
.00621 .01633 L
.01242 .01633 L
.01242 .01472 L
.00621 .01472 L
s
.03727 .01472 m
.03727 .14677 L
.04348 .14677 L
.04348 .01472 L
F
.03727 .01472 m
.03727 .14677 L
.04348 .14677 L
.04348 .01472 L
.03727 .01472 L
s
.06832 .01472 m
.06832 .0743 L
.07453 .0743 L
.07453 .01472 L
F
.06832 .01472 m
.06832 .0743 L
.07453 .0743 L
.07453 .01472 L
.06832 .01472 L
s
.09938 .01472 m
.09938 .14113 L
.10559 .14113 L
.10559 .01472 L
F
.09938 .01472 m
.09938 .14113 L
.10559 .14113 L
.10559 .01472 L
.09938 .01472 L
s
.13043 .01472 m
.13043 .01472 L
.13665 .01472 L
.13665 .01472 L
F
.13043 .01472 m
.13043 .01472 L
.13665 .01472 L
.13665 .01472 L
.13043 .01472 L
s
.16149 .01472 m
.16149 .1363 L
.1677 .1363 L
.1677 .01472 L
F
.16149 .01472 m
.16149 .1363 L
.1677 .1363 L
.1677 .01472 L
.16149 .01472 L
s
.19255 .01472 m
.19255 .0284 L
.19876 .0284 L
.19876 .01472 L
F
.19255 .01472 m
.19255 .0284 L
.19876 .0284 L
.19876 .01472 L
.19255 .01472 L
s
.2236 .01472 m
.2236 .12905 L
.22981 .12905 L
.22981 .01472 L
F
.2236 .01472 m
.2236 .12905 L
.22981 .12905 L
.22981 .01472 L
.2236 .01472 L
s
.25466 .01472 m
.25466 .0276 L
.26087 .0276 L
.26087 .01472 L
F
.25466 .01472 m
.25466 .0276 L
.26087 .0276 L
.26087 .01472 L
.25466 .01472 L
s
.28571 .01472 m
.28571 .15563 L
.29193 .15563 L
.29193 .01472 L
F
.28571 .01472 m
.28571 .15563 L
.29193 .15563 L
.29193 .01472 L
.28571 .01472 L
s
.31677 .01472 m
.31677 .10007 L
.32298 .10007 L
.32298 .01472 L
F
.31677 .01472 m
.31677 .10007 L
.32298 .10007 L
.32298 .01472 L
.31677 .01472 L
s
.34783 .01472 m
.34783 .14757 L
.35404 .14757 L
.35404 .01472 L
F
.34783 .01472 m
.34783 .14757 L
.35404 .14757 L
.35404 .01472 L
.34783 .01472 L
s
.37888 .01472 m
.37888 .01472 L
.38509 .01472 L
.38509 .01472 L
F
.37888 .01472 m
.37888 .01472 L
.38509 .01472 L
.38509 .01472 L
.37888 .01472 L
s
.40994 .01472 m
.40994 .12422 L
.41615 .12422 L
.41615 .01472 L
F
.40994 .01472 m
.40994 .12422 L
.41615 .12422 L
.41615 .01472 L
.40994 .01472 L
s
.44099 .01472 m
.44099 .04612 L
.4472 .04612 L
.4472 .01472 L
F
.44099 .01472 m
.44099 .04612 L
.4472 .04612 L
.4472 .01472 L
.44099 .01472 L
s
.47205 .01472 m
.47205 .14516 L
.47826 .14516 L
.47826 .01472 L
F
.47205 .01472 m
.47205 .14516 L
.47826 .14516 L
.47826 .01472 L
.47205 .01472 L
s
.50311 .01472 m
.50311 .02035 L
.50932 .02035 L
.50932 .01472 L
F
.50311 .01472 m
.50311 .02035 L
.50932 .02035 L
.50932 .01472 L
.50311 .01472 L
s
.53416 .01472 m
.53416 .06142 L
.54037 .06142 L
.54037 .01472 L
F
.53416 .01472 m
.53416 .06142 L
.54037 .06142 L
.54037 .01472 L
.53416 .01472 L
s
.56522 .01472 m
.56522 .02679 L
.57143 .02679 L
.57143 .01472 L
F
.56522 .01472 m
.56522 .02679 L
.57143 .02679 L
.57143 .01472 L
.56522 .01472 L
s
.59627 .01472 m
.59627 .17978 L
.60248 .17978 L
.60248 .01472 L
F
.59627 .01472 m
.59627 .17978 L
.60248 .17978 L
.60248 .01472 L
.59627 .01472 L
s
.62733 .01472 m
.62733 .01472 L
.63354 .01472 L
.63354 .01472 L
F
.62733 .01472 m
.62733 .01472 L
.63354 .01472 L
.63354 .01472 L
.62733 .01472 L
s
.65839 .01472 m
.65839 .13066 L
.6646 .13066 L
.6646 .01472 L
F
.65839 .01472 m
.65839 .13066 L
.6646 .13066 L
.6646 .01472 L
.65839 .01472 L
s
.68944 .01472 m
.68944 .03243 L
.69565 .03243 L
.69565 .01472 L
F
.68944 .01472 m
.68944 .03243 L
.69565 .03243 L
.69565 .01472 L
.68944 .01472 L
s
.7205 .01472 m
.7205 .13147 L
.72671 .13147 L
.72671 .01472 L
F
.7205 .01472 m
.7205 .13147 L
.72671 .13147 L
.72671 .01472 L
.7205 .01472 L
s
.75155 .01472 m
.75155 .01794 L
.75776 .01794 L
.75776 .01472 L
F
.75155 .01472 m
.75155 .01794 L
.75776 .01794 L
.75776 .01472 L
.75155 .01472 L
s
.78261 .01472 m
.78261 .14838 L
.78882 .14838 L
.78882 .01472 L
F
.78261 .01472 m
.78261 .14838 L
.78882 .14838 L
.78882 .01472 L
.78261 .01472 L
s
.81366 .01472 m
.81366 .07027 L
.81988 .07027 L
.81988 .01472 L
F
.81366 .01472 m
.81366 .07027 L
.81988 .07027 L
.81988 .01472 L
.81366 .01472 L
s
.84472 .01472 m
.84472 .14113 L
.85093 .14113 L
.85093 .01472 L
F
.84472 .01472 m
.84472 .14113 L
.85093 .14113 L
.85093 .01472 L
.84472 .01472 L
s
.87578 .01472 m
.87578 .01472 L
.88199 .01472 L
.88199 .01472 L
F
.87578 .01472 m
.87578 .01472 L
.88199 .01472 L
.88199 .01472 L
.87578 .01472 L
s
.90683 .01472 m
.90683 .13791 L
.91304 .13791 L
.91304 .01472 L
F
.90683 .01472 m
.90683 .13791 L
.91304 .13791 L
.91304 .01472 L
.90683 .01472 L
s
.93789 .01472 m
.93789 .05336 L
.9441 .05336 L
.9441 .01472 L
F
.93789 .01472 m
.93789 .05336 L
.9441 .05336 L
.9441 .01472 L
.93789 .01472 L
s
.96894 .01472 m
.96894 .1355 L
.97516 .1355 L
.97516 .01472 L
F
.96894 .01472 m
.96894 .1355 L
.97516 .1355 L
.97516 .01472 L
.96894 .01472 L
s
0 1 0 r
.01242 .01472 m
.01242 .39236 L
.01863 .39236 L
.01863 .01472 L
F
.01242 .01472 m
.01242 .39236 L
.01863 .39236 L
.01863 .01472 L
.01242 .01472 L
s
.04348 .01472 m
.04348 .01472 L
.04969 .01472 L
.04969 .01472 L
F
.04348 .01472 m
.04348 .01472 L
.04969 .01472 L
.04969 .01472 L
.04348 .01472 L
s
.07453 .01472 m
.07453 .03082 L
.08075 .03082 L
.08075 .01472 L
F
.07453 .01472 m
.07453 .03082 L
.08075 .03082 L
.08075 .01472 L
.07453 .01472 L
s
.10559 .01472 m
.10559 .01552 L
.1118 .01552 L
.1118 .01472 L
F
.10559 .01472 m
.10559 .01552 L
.1118 .01552 L
.1118 .01472 L
.10559 .01472 L
s
.13665 .01472 m
.13665 .60332 L
.14286 .60332 L
.14286 .01472 L
F
.13665 .01472 m
.13665 .60332 L
.14286 .60332 L
.14286 .01472 L
.13665 .01472 L
s
.1677 .01472 m
.1677 .02116 L
.17391 .02116 L
.17391 .01472 L
F
.1677 .01472 m
.1677 .02116 L
.17391 .02116 L
.17391 .01472 L
.1677 .01472 L
s
.19876 .01472 m
.19876 .05256 L
.20497 .05256 L
.20497 .01472 L
F
.19876 .01472 m
.19876 .05256 L
.20497 .05256 L
.20497 .01472 L
.19876 .01472 L
s
.22981 .01472 m
.22981 .01472 L
.23602 .01472 L
.23602 .01472 L
F
.22981 .01472 m
.22981 .01472 L
.23602 .01472 L
.23602 .01472 L
.22981 .01472 L
s
.26087 .01472 m
.26087 .12342 L
.26708 .12342 L
.26708 .01472 L
F
.26087 .01472 m
.26087 .12342 L
.26708 .12342 L
.26708 .01472 L
.26087 .01472 L
s
.29193 .01472 m
.29193 .01713 L
.29814 .01713 L
.29814 .01472 L
F
.29193 .01472 m
.29193 .01713 L
.29814 .01713 L
.29814 .01472 L
.29193 .01472 L
s
.32298 .01472 m
.32298 .01955 L
.32919 .01955 L
.32919 .01472 L
F
.32298 .01472 m
.32298 .01955 L
.32919 .01955 L
.32919 .01472 L
.32298 .01472 L
s
.35404 .01472 m
.35404 .01472 L
.36025 .01472 L
.36025 .01472 L
F
.35404 .01472 m
.35404 .01472 L
.36025 .01472 L
.36025 .01472 L
.35404 .01472 L
s
.38509 .01472 m
.38509 .12422 L
.3913 .12422 L
.3913 .01472 L
F
.38509 .01472 m
.38509 .12422 L
.3913 .12422 L
.3913 .01472 L
.38509 .01472 L
s
.41615 .01472 m
.41615 .01955 L
.42236 .01955 L
.42236 .01472 L
F
.41615 .01472 m
.41615 .01955 L
.42236 .01955 L
.42236 .01472 L
.41615 .01472 L
s
.4472 .01472 m
.4472 .03726 L
.45342 .03726 L
.45342 .01472 L
F
.4472 .01472 m
.4472 .03726 L
.45342 .03726 L
.45342 .01472 L
.4472 .01472 L
s
.47826 .01472 m
.47826 .01472 L
.48447 .01472 L
.48447 .01472 L
F
.47826 .01472 m
.47826 .01472 L
.48447 .01472 L
.48447 .01472 L
.47826 .01472 L
s
.50932 .01472 m
.50932 .23293 L
.51553 .23293 L
.51553 .01472 L
F
.50932 .01472 m
.50932 .23293 L
.51553 .23293 L
.51553 .01472 L
.50932 .01472 L
s
.54037 .01472 m
.54037 .02518 L
.54658 .02518 L
.54658 .01472 L
F
.54037 .01472 m
.54037 .02518 L
.54658 .02518 L
.54658 .01472 L
.54037 .01472 L
s
.57143 .01472 m
.57143 .03646 L
.57764 .03646 L
.57764 .01472 L
F
.57143 .01472 m
.57143 .03646 L
.57764 .03646 L
.57764 .01472 L
.57143 .01472 L
s
.60248 .01472 m
.60248 .01552 L
.6087 .01552 L
.6087 .01472 L
F
.60248 .01472 m
.60248 .01552 L
.6087 .01552 L
.6087 .01472 L
.60248 .01472 L
s
.63354 .01472 m
.63354 .42134 L
.63975 .42134 L
.63975 .01472 L
F
.63354 .01472 m
.63354 .42134 L
.63975 .42134 L
.63975 .01472 L
.63354 .01472 L
s
.6646 .01472 m
.6646 .01713 L
.67081 .01713 L
.67081 .01472 L
F
.6646 .01472 m
.6646 .01713 L
.67081 .01713 L
.67081 .01472 L
.6646 .01472 L
s
.69565 .01472 m
.69565 .0429 L
.70186 .0429 L
.70186 .01472 L
F
.69565 .01472 m
.69565 .0429 L
.70186 .0429 L
.70186 .01472 L
.69565 .01472 L
s
.72671 .01472 m
.72671 .01552 L
.73292 .01552 L
.73292 .01472 L
F
.72671 .01472 m
.72671 .01552 L
.73292 .01552 L
.73292 .01472 L
.72671 .01472 L
s
.75776 .01472 m
.75776 .20877 L
.76398 .20877 L
.76398 .01472 L
F
.75776 .01472 m
.75776 .20877 L
.76398 .20877 L
.76398 .01472 L
.75776 .01472 L
s
.78882 .01472 m
.78882 .01552 L
.79503 .01552 L
.79503 .01472 L
F
.78882 .01472 m
.78882 .01552 L
.79503 .01552 L
.79503 .01472 L
.78882 .01472 L
s
.81988 .01472 m
.81988 .0276 L
.82609 .0276 L
.82609 .01472 L
F
.81988 .01472 m
.81988 .0276 L
.82609 .0276 L
.82609 .01472 L
.81988 .01472 L
s
.85093 .01472 m
.85093 .01552 L
.85714 .01552 L
.85714 .01472 L
F
.85093 .01472 m
.85093 .01552 L
.85714 .01552 L
.85714 .01472 L
.85093 .01472 L
s
.88199 .01472 m
.88199 .1057 L
.8882 .1057 L
.8882 .01472 L
F
.88199 .01472 m
.88199 .1057 L
.8882 .1057 L
.8882 .01472 L
.88199 .01472 L
s
.91304 .01472 m
.91304 .01713 L
.91925 .01713 L
.91925 .01472 L
F
.91304 .01472 m
.91304 .01713 L
.91925 .01713 L
.91925 .01472 L
.91304 .01472 L
s
.9441 .01472 m
.9441 .03726 L
.95031 .03726 L
.95031 .01472 L
F
.9441 .01472 m
.9441 .03726 L
.95031 .03726 L
.95031 .01472 L
.9441 .01472 L
s
.97516 .01472 m
.97516 .01472 L
.98137 .01472 L
.98137 .01472 L
F
.97516 .01472 m
.97516 .01472 L
.98137 .01472 L
.98137 .01472 L
.97516 .01472 L
s
0 0 1 r
.01863 .01472 m
.01863 .21843 L
.02484 .21843 L
.02484 .01472 L
F
.01863 .01472 m
.01863 .21843 L
.02484 .21843 L
.02484 .01472 L
.01863 .01472 L
s
.04969 .01472 m
.04969 .01472 L
.0559 .01472 L
.0559 .01472 L
F
.04969 .01472 m
.04969 .01472 L
.0559 .01472 L
.0559 .01472 L
.04969 .01472 L
s
.08075 .01472 m
.08075 .01713 L
.08696 .01713 L
.08696 .01472 L
F
.08075 .01472 m
.08075 .01713 L
.08696 .01713 L
.08696 .01472 L
.08075 .01472 L
s
.1118 .01472 m
.1118 .01472 L
.11801 .01472 L
.11801 .01472 L
F
.1118 .01472 m
.1118 .01472 L
.11801 .01472 L
.11801 .01472 L
.1118 .01472 L
s
.14286 .01472 m
.14286 .57353 L
.14907 .57353 L
.14907 .01472 L
F
.14286 .01472 m
.14286 .57353 L
.14907 .57353 L
.14907 .01472 L
.14286 .01472 L
s
.17391 .01472 m
.17391 .01472 L
.18012 .01472 L
.18012 .01472 L
F
.17391 .01472 m
.17391 .01472 L
.18012 .01472 L
.18012 .01472 L
.17391 .01472 L
s
.20497 .01472 m
.20497 .06705 L
.21118 .06705 L
.21118 .01472 L
F
.20497 .01472 m
.20497 .06705 L
.21118 .06705 L
.21118 .01472 L
.20497 .01472 L
s
.23602 .01472 m
.23602 .01472 L
.24224 .01472 L
.24224 .01472 L
F
.23602 .01472 m
.23602 .01472 L
.24224 .01472 L
.24224 .01472 L
.23602 .01472 L
s
.26708 .01472 m
.26708 .06303 L
.27329 .06303 L
.27329 .01472 L
F
.26708 .01472 m
.26708 .06303 L
.27329 .06303 L
.27329 .01472 L
.26708 .01472 L
s
.29814 .01472 m
.29814 .01552 L
.30435 .01552 L
.30435 .01472 L
F
.29814 .01472 m
.29814 .01552 L
.30435 .01552 L
.30435 .01472 L
.29814 .01472 L
s
.32919 .01472 m
.32919 .01472 L
.3354 .01472 L
.3354 .01472 L
F
.32919 .01472 m
.32919 .01472 L
.3354 .01472 L
.3354 .01472 L
.32919 .01472 L
s
.36025 .01472 m
.36025 .01472 L
.36646 .01472 L
.36646 .01472 L
F
.36025 .01472 m
.36025 .01472 L
.36646 .01472 L
.36646 .01472 L
.36025 .01472 L
s
.3913 .01472 m
.3913 .45194 L
.39752 .45194 L
.39752 .01472 L
F
.3913 .01472 m
.3913 .45194 L
.39752 .45194 L
.39752 .01472 L
.3913 .01472 L
s
.42236 .01472 m
.42236 .01472 L
.42857 .01472 L
.42857 .01472 L
F
.42236 .01472 m
.42236 .01472 L
.42857 .01472 L
.42857 .01472 L
.42236 .01472 L
s
.45342 .01472 m
.45342 .02518 L
.45963 .02518 L
.45963 .01472 L
F
.45342 .01472 m
.45342 .02518 L
.45963 .02518 L
.45963 .01472 L
.45342 .01472 L
s
.48447 .01472 m
.48447 .01472 L
.49068 .01472 L
.49068 .01472 L
F
.48447 .01472 m
.48447 .01472 L
.49068 .01472 L
.49068 .01472 L
.48447 .01472 L
s
.51553 .01472 m
.51553 .11214 L
.52174 .11214 L
.52174 .01472 L
F
.51553 .01472 m
.51553 .11214 L
.52174 .11214 L
.52174 .01472 L
.51553 .01472 L
s
.54658 .01472 m
.54658 .01552 L
.5528 .01552 L
.5528 .01472 L
F
.54658 .01472 m
.54658 .01552 L
.5528 .01552 L
.5528 .01472 L
.54658 .01472 L
s
.57764 .01472 m
.57764 .0582 L
.58385 .0582 L
.58385 .01472 L
F
.57764 .01472 m
.57764 .0582 L
.58385 .0582 L
.58385 .01472 L
.57764 .01472 L
s
.6087 .01472 m
.6087 .01472 L
.61491 .01472 L
.61491 .01472 L
F
.6087 .01472 m
.6087 .01472 L
.61491 .01472 L
.61491 .01472 L
.6087 .01472 L
s
.63975 .01472 m
.63975 .31667 L
.64596 .31667 L
.64596 .01472 L
F
.63975 .01472 m
.63975 .31667 L
.64596 .31667 L
.64596 .01472 L
.63975 .01472 L
s
.67081 .01472 m
.67081 .01472 L
.67702 .01472 L
.67702 .01472 L
F
.67081 .01472 m
.67081 .01472 L
.67702 .01472 L
.67702 .01472 L
.67081 .01472 L
s
.70186 .01472 m
.70186 .01955 L
.70807 .01955 L
.70807 .01472 L
F
.70186 .01472 m
.70186 .01955 L
.70807 .01955 L
.70807 .01472 L
.70186 .01472 L
s
.73292 .01472 m
.73292 .01552 L
.73913 .01552 L
.73913 .01472 L
F
.73292 .01472 m
.73292 .01552 L
.73913 .01552 L
.73913 .01472 L
.73292 .01472 L
s
.76398 .01472 m
.76398 .13147 L
.77019 .13147 L
.77019 .01472 L
F
.76398 .01472 m
.76398 .13147 L
.77019 .13147 L
.77019 .01472 L
.76398 .01472 L
s
.79503 .01472 m
.79503 .01472 L
.80124 .01472 L
.80124 .01472 L
F
.79503 .01472 m
.79503 .01472 L
.80124 .01472 L
.80124 .01472 L
.79503 .01472 L
s
.82609 .01472 m
.82609 .01472 L
.8323 .01472 L
.8323 .01472 L
F
.82609 .01472 m
.82609 .01472 L
.8323 .01472 L
.8323 .01472 L
.82609 .01472 L
s
.85714 .01472 m
.85714 .01472 L
.86335 .01472 L
.86335 .01472 L
F
.85714 .01472 m
.85714 .01472 L
.86335 .01472 L
.86335 .01472 L
.85714 .01472 L
s
.8882 .01472 m
.8882 .40443 L
.89441 .40443 L
.89441 .01472 L
F
.8882 .01472 m
.8882 .40443 L
.89441 .40443 L
.89441 .01472 L
.8882 .01472 L
s
.91925 .01472 m
.91925 .01472 L
.92547 .01472 L
.92547 .01472 L
F
.91925 .01472 m
.91925 .01472 L
.92547 .01472 L
.92547 .01472 L
.91925 .01472 L
s
.95031 .01472 m
.95031 .03887 L
.95652 .03887 L
.95652 .01472 L
F
.95031 .01472 m
.95031 .03887 L
.95652 .03887 L
.95652 .01472 L
.95031 .01472 L
s
.98137 .01472 m
.98137 .01472 L
.98758 .01472 L
.98758 .01472 L
F
.98137 .01472 m
.98137 .01472 L
.98758 .01472 L
.98758 .01472 L
.98137 .01472 L
s
1 .5 0 r
.02484 .01472 m
.02484 .08718 L
.03106 .08718 L
.03106 .01472 L
F
.02484 .01472 m
.02484 .08718 L
.03106 .08718 L
.03106 .01472 L
.02484 .01472 L
s
.0559 .01472 m
.0559 .03646 L
.06211 .03646 L
.06211 .01472 L
F
.0559 .01472 m
.0559 .03646 L
.06211 .03646 L
.06211 .01472 L
.0559 .01472 L
s
.08696 .01472 m
.08696 .12422 L
.09317 .12422 L
.09317 .01472 L
F
.08696 .01472 m
.08696 .12422 L
.09317 .12422 L
.09317 .01472 L
.08696 .01472 L
s
.11801 .01472 m
.11801 .04531 L
.12422 .04531 L
.12422 .01472 L
F
.11801 .01472 m
.11801 .04531 L
.12422 .04531 L
.12422 .01472 L
.11801 .01472 L
s
.14907 .01472 m
.14907 .0276 L
.15528 .0276 L
.15528 .01472 L
F
.14907 .01472 m
.14907 .0276 L
.15528 .0276 L
.15528 .01472 L
.14907 .01472 L
s
.18012 .01472 m
.18012 .07833 L
.18634 .07833 L
.18634 .01472 L
F
.18012 .01472 m
.18012 .07833 L
.18634 .07833 L
.18634 .01472 L
.18012 .01472 L
s
.21118 .01472 m
.21118 .16851 L
.21739 .16851 L
.21739 .01472 L
F
.21118 .01472 m
.21118 .16851 L
.21739 .16851 L
.21739 .01472 L
.21118 .01472 L
s
.24224 .01472 m
.24224 .02035 L
.24845 .02035 L
.24845 .01472 L
F
.24224 .01472 m
.24224 .02035 L
.24845 .02035 L
.24845 .01472 L
.24224 .01472 L
s
.27329 .01472 m
.27329 .17978 L
.2795 .17978 L
.2795 .01472 L
F
.27329 .01472 m
.27329 .17978 L
.2795 .17978 L
.2795 .01472 L
.27329 .01472 L
s
.30435 .01472 m
.30435 .05659 L
.31056 .05659 L
.31056 .01472 L
F
.30435 .01472 m
.30435 .05659 L
.31056 .05659 L
.31056 .01472 L
.30435 .01472 L
s
.3354 .01472 m
.3354 .0743 L
.34161 .0743 L
.34161 .01472 L
F
.3354 .01472 m
.3354 .0743 L
.34161 .0743 L
.34161 .01472 L
.3354 .01472 L
s
.36646 .01472 m
.36646 .03968 L
.37267 .03968 L
.37267 .01472 L
F
.36646 .01472 m
.36646 .03968 L
.37267 .03968 L
.37267 .01472 L
.36646 .01472 L
s
.39752 .01472 m
.39752 .04853 L
.40373 .04853 L
.40373 .01472 L
F
.39752 .01472 m
.39752 .04853 L
.40373 .04853 L
.40373 .01472 L
.39752 .01472 L
s
.42857 .01472 m
.42857 .05014 L
.43478 .05014 L
.43478 .01472 L
F
.42857 .01472 m
.42857 .05014 L
.43478 .05014 L
.43478 .01472 L
.42857 .01472 L
s
.45963 .01472 m
.45963 .1524 L
.46584 .1524 L
.46584 .01472 L
F
.45963 .01472 m
.45963 .1524 L
.46584 .1524 L
.46584 .01472 L
.45963 .01472 L
s
.49068 .01472 m
.49068 .02196 L
.49689 .02196 L
.49689 .01472 L
F
.49068 .01472 m
.49068 .02196 L
.49689 .02196 L
.49689 .01472 L
.49068 .01472 L
s
.52174 .01472 m
.52174 .12825 L
.52795 .12825 L
.52795 .01472 L
F
.52174 .01472 m
.52174 .12825 L
.52795 .12825 L
.52795 .01472 L
.52174 .01472 L
s
.5528 .01472 m
.5528 .14918 L
.55901 .14918 L
.55901 .01472 L
F
.5528 .01472 m
.5528 .14918 L
.55901 .14918 L
.55901 .01472 L
.5528 .01472 L
s
.58385 .01472 m
.58385 .18783 L
.59006 .18783 L
.59006 .01472 L
F
.58385 .01472 m
.58385 .18783 L
.59006 .18783 L
.59006 .01472 L
.58385 .01472 L
s
.61491 .01472 m
.61491 .0437 L
.62112 .0437 L
.62112 .01472 L
F
.61491 .01472 m
.61491 .0437 L
.62112 .0437 L
.62112 .01472 L
.61491 .01472 L
s
.64596 .01472 m
.64596 .05498 L
.65217 .05498 L
.65217 .01472 L
F
.64596 .01472 m
.64596 .05498 L
.65217 .05498 L
.65217 .01472 L
.64596 .01472 L
s
.67702 .01472 m
.67702 .09363 L
.68323 .09363 L
.68323 .01472 L
F
.67702 .01472 m
.67702 .09363 L
.68323 .09363 L
.68323 .01472 L
.67702 .01472 L
s
.70807 .01472 m
.70807 .17898 L
.71429 .17898 L
.71429 .01472 L
F
.70807 .01472 m
.70807 .17898 L
.71429 .17898 L
.71429 .01472 L
.70807 .01472 L
s
.73913 .01472 m
.73913 .06303 L
.74534 .06303 L
.74534 .01472 L
F
.73913 .01472 m
.73913 .06303 L
.74534 .06303 L
.74534 .01472 L
.73913 .01472 L
s
.77019 .01472 m
.77019 .14838 L
.7764 .14838 L
.7764 .01472 L
F
.77019 .01472 m
.77019 .14838 L
.7764 .14838 L
.7764 .01472 L
.77019 .01472 L
s
.80124 .01472 m
.80124 .02599 L
.80745 .02599 L
.80745 .01472 L
F
.80124 .01472 m
.80124 .02599 L
.80745 .02599 L
.80745 .01472 L
.80124 .01472 L
s
.8323 .01472 m
.8323 .12744 L
.83851 .12744 L
.83851 .01472 L
F
.8323 .01472 m
.8323 .12744 L
.83851 .12744 L
.83851 .01472 L
.8323 .01472 L
s
.86335 .01472 m
.86335 .03968 L
.86957 .03968 L
.86957 .01472 L
F
.86335 .01472 m
.86335 .03968 L
.86957 .03968 L
.86957 .01472 L
.86335 .01472 L
s
.89441 .01472 m
.89441 .05659 L
.90062 .05659 L
.90062 .01472 L
F
.89441 .01472 m
.89441 .05659 L
.90062 .05659 L
.90062 .01472 L
.89441 .01472 L
s
.92547 .01472 m
.92547 .06866 L
.93168 .06866 L
.93168 .01472 L
F
.92547 .01472 m
.92547 .06866 L
.93168 .06866 L
.93168 .01472 L
.92547 .01472 L
s
.95652 .01472 m
.95652 .1669 L
.96273 .1669 L
.96273 .01472 L
F
.95652 .01472 m
.95652 .1669 L
.96273 .1669 L
.96273 .01472 L
.95652 .01472 L
s
.98758 .01472 m
.98758 .02035 L
.99379 .02035 L
.99379 .01472 L
F
.98758 .01472 m
.98758 .02035 L
.99379 .02035 L
.99379 .01472 L
.98758 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{500, 309},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007d0001=B000`400?l00000o`00003ooooooooe
ooooo`00;Oooool3o`0000[ooooo1Ol0000:ooooo`?o00003?ooool3o`0000[ooooo0ol0000;oooo
o`?o00003?ooool00ol0003oooooooooo`0:ooooo`?o00002oooool2o`0000Wooooo0ol00004oooo
o`;o00001Oooool3o`0000?ooooo0ol00005ooooo`?o00000_ooool5o`0000Cooooo0ol00003oooo
o`?o00001Oooool3o`0000Cooooo0ol00004ooooo`?o00000oooool3o`0000Gooooo0ol00003oooo
o`?o00001Oooool3o`0000Cooooo00?o0000ooooooooool01?ooool3o`0000?ooooo0ol00005oooo
o`?o00000oooool2o`0000Gooooo1Ol00003ooooo`;o00001?ooool5o`0000;ooooo0ol00004oooo
o`Go000000?oooooo`000?l000000ol00003ooooo`Go00000_ooool3o`0000Cooooo1Ol00003oooo
o`?o00000oooool5o`0000;ooooo0ol00004ooooo`Go00000_ooool3o`0000Cooooo1Ol00003oooo
o`03o`000?oooooooooo00?ooooo1Ol00002ooooo`?o00001?ooool5o`0000;ooooo0_l00006oooo
o`?o00001?ooool2o`0000Gooooo0ol00003ooooo`?o00001Oooool3o`0000;ooooo1Ol0000>oooo
o`00;_ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000Wooooo00Go0000oooo
ooooooooooooo`00000<ooooo`03o`000?oooooooooo00Sooooo00Go0000ooooooooooooooooo`00
0009ooooo`05o`000?ooooooooooooooool000002oooool00ol0003oooooooooo`09ooooo`05o`00
0?ooooooooooooooool000003?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00;o
oooo00Co0000oooooooooooo00001Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`05o`000?oooooo
ooooooooool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01?ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool0
0oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`03oooo
o`04o`000?ooooooooooo`0000;ooooo00Co0000oooooooooooo00001?ooool01?l0003ooooooooo
ool00003ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooo00000_ooool01?l0003o
ooooooooool00004ooooo`06o`000?ooooooooooo`000?oooooo00000oooool00ol0003ooooooooo
o`02ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool00oooool01_l0003ooooo
ooooool0003oooooo`0000?ooooo00?o0000ooooooooool00_ooool01_l0003oooooooooool0003o
ooooo`0000?ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00003ooooo`03o`00
0?oooooooooo00Cooooo00Ko0000oooooooooooo0000ooooool00003ooooo`03o`000?oooooooooo
00;ooooo00Co0000oooooooooooo00001?ooool00ol0003oooooooooo`02ooooo`05o`000?oooooo
ooooooooool000000_ooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000oooool00ol0003oooooooooo`03ooooo`05o`000?ooooooooooooooool000000_ooool01?l0
003oooooooooool0000>ooooo`00;_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo
00gooooo00?o0000ooooooooool01oooool5o`0000gooooo00?o0000ooooooooool01oooool01Ol0
003ooooooooooooooooo000000cooooo00?o0000ooooooooool02?ooool01Ol0003ooooooooooooo
oooo000000gooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`02ooooo`04o`000?oo
ooooooooo`0000Gooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00?ooooo00Co0000oooooooooooooooo1Ol00005ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00Go0000oooo
ooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00001Ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00ooo
ool01?l0003oooooooooooooool5o`0000Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`03ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooooooooooo0000
00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`05ooooo`05o`000?oooooo
ooooooooool000002Oooool01Ol0003ooooooooooooooooo0000013ooooo000^ooooo`03o`000?oo
oooooooo00cooooo00?o0000ooooooooool02_ooool2o`0000[ooooo00Co0000oooooooooooo0000
3_ooool00ol0003oooooooooo`07ooooo`Co00003Oooool00ol0003oooooooooo`09ooooo`?o0000
2oooool4o`0000Sooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`03
o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool00oooool2o`0000Kooooo00Go0000oooooooo
ooooooooo`000002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`03ooooo`04o`000?ooooooooooooooo`Co00001_ooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool3o`0000Kooooo00?o0000
ooooooooool00_ooool4o`0000Kooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`06ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`02ooooo`;o00001ooo
ool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Co00001oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00Gooooo00Co0000oooooooooooooooo0ol00007ooooo`04
o`000?ooooooooooooooo`Co00001Oooool2o`0000?ooooo00Co0000oooooooooooo00001Oooool2
o`0000Cooooo00?o0000ooooooooool01Oooool2o`0000Gooooo00?o0000ooooooooool03Oooool0
02kooooo00?o0000ooooooooool02Oooool01Ol0003ooooooooooooooooo000000gooooo00?o0000
ooooooooool02?ooool00ol0003oooooo`00000:ooooo`Co00002_ooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00Go0000ooooooooooooooooo`000009ooooo`05o`000?oo
ooooooooooooool000002?ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Go
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`05o`000?oooooooooooooo
ool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Kooooo00Co0000oooo
oooooooooooo1?l00006ooooo`05o`000?ooooooooooooooool000002Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooool00002ooooo`05o`000?ooooooooooooooool000000ooo
ool01Ol0003ooooooooooooooooo000000Gooooo00Oo0000ooooooooooooooooo`000?oooooo0000
00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00Go0000
ooooooooooooooooo`000003ooooo`06o`000?ooooooooooo`000?oooooo00001?ooool01_l0003o
oooooooooooooooo0000ooooo`Co00001?ooool01ol0003ooooooooooooooooo0000ooooool00000
1oooool01Ol0003ooooooooooooooooo000000Gooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo
000000?ooooo00?o0000ooooooooool01Oooool01?l0003oooooooooool00002ooooo`03o`000?oo
oooooooo00Gooooo00Go0000ooooooooooooooooo`000009ooooo`03o`000?oooooo000000?ooooo
00?o0000ooooooooool03?ooool002gooooo0_l0000;ooooo`05o`000?ooooooooooooooool00000
2Oooool01Ol0003ooooooooooooooooo000000_ooooo0_l0000:ooooo`03o`000?oooooooooo00co
oooo00?o0000ooooooooool02_ooool01Ol0003ooooooooooooooooo000000Wooooo00Go0000oooo
ooooooooooooo`000009ooooo`05o`000?ooooooooooooooool000001oooool2o`0000Cooooo00Co
0000oooooooooooo00001?ooool2o`0000Cooooo0_l00006ooooo`;o00000oooool01Ol0003ooooo
oooooooooooo000000Cooooo0_l00003ooooo`05o`000?ooooooooooooooool000001?ooool2o`00
00Gooooo0_l00005ooooo`;o00000oooool00ol0003oooooooooo`06ooooo`;o00001?ooool00ol0
003oooooooooo`05ooooo`;o00000oooool01Ol0003ooooooooooooooooo000000Cooooo0_l00003
ooooo`05o`000?ooooooooooooooool000001?ooool2o`0000?ooooo00Go0000oooooooooooooooo
o`000003ooooo`05o`000?ooooooooooooooool000000_ooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool2o`0000Gooooo00Oo0000ooooooooooooooooo`00
0?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo000000?o
oooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooooooo`;o00001?ooool0
1ol0003ooooooooooooooooo0000ooooool000001oooool01Ol0003ooooooooooooooooo000000;o
oooo00?o0000ooooooooool01?ooool01ol0003ooooooooooooooooo0000ooooool000000oooool0
1Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooool000000oooool01Ol0003ooooooooo
oooooooo000000?ooooo00?o0000ooooool000000oooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000_ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000
ooooooooool00_l00005ooooo`07o`000?ooooooooooooooool0003oooooo`000003ooooo`03o`00
0?oooooooooo00cooooo000^ooooo`03o`000?oooooooooo00[ooooo0ol0000;ooooo`?o00003Ooo
ool00ol0003oooooooooo`08ooooo`Go00002oooool2o`0000[ooooo1Ol0000:ooooo`?o00002ooo
ool3o`0000Wooooo00?o0000ooooooooool00oooool2o`0000Kooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;ooooo0ol00006ooooo`03o`000?oo
oooooooo00;ooooo0ol00006ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?oo
ool01?l0003oooooooooooooool5o`0000Gooooo00?o0000ooooooooool00oooool2o`0000Kooooo
00Co0000oooooooooooooooo1Ol00005ooooo`03o`000?oooooooooo00;ooooo0ol00006ooooo`03
o`000?oooooooooo00;ooooo0ol00005ooooo`?o00001?ooool2o`0000Gooooo0ol00004ooooo`03
o`000?oooooooooo00Cooooo0ol00003ooooo`?o00001Oooool3o`0000?ooooo0ol00005ooooo`?o
00001Oooool00ol0003oooooooooo`03ooooo`?o00000_ooool5o`0000Cooooo0ol00004ooooo`;o
00001Oooool3o`0000;ooooo1Ol00004ooooo`?o00000oooool3o`0000Gooooo0ol00003ooooo`?o
00001Oooool3o`0000Cooooo0_l00005ooooo`?o00001?ooool00ol0003oooooooooo`04ooooo`?o
00000oooool3o`0000oooooo003oooooooGooooo003oooooooGooooo003oooooooGooooo003ooooo
ooGooooo003oooooooGooooo003oooooooGooooo003oooooooGooooo000Voooooooo0000aOl0000:
ooooo`009_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`08ooooo`00
9_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`08ooooo`007Oooool2o`0000Oooooo00?o0000ooooooooool0oooo
ooo1ooooo`03o`000?oooooooooo00Sooooo000Looooo`04o`000?ooooooooooo`0000Kooooo00?o
0000ooooooooool0ooooooo1ooooo`03o`000?oooooooooo00Sooooo000Looooo`04o`000?oooooo
ooooo`0000Kooooo00?o0000ooooooooool0ooooooo1ooooo`03o`000?oooooooooo00Sooooo000L
ooooo`04o`000?ooooooooooo`0000Kooooo0ol00003ool000?o0?l00_l00?l4oomo00;o00000ooo
0003o`3o00;o003o1?ooO`02o`0000?oo`000ol0o`02o`00o`Coogl00_l00003ool000?o0?l00_l0
0?l4oomo00;o00000ooo0003o`3o00;o003o1?ooO`02o`0000?oo`000ol0o`02o`00o`Coogl00_l0
0003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0
o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0ooo
O`02o`0000?oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo
0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol0
0?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl00_l0
0003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0
o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0ooo
O`02o`0000?oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo
0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol0
0?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl00_l0
0003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`03o`0000[ooooo000L
ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool00ooo0003o`3o00;o003o1?oo
O`02ooooo`Coo`001?ooool4oomo00;ooooo0ooo0003o`3o00;o003o1?ooO`02ooooo`Coo`001?oo
ool4oomo00Gooooo0ol0o`02o`00o`Coogl00_ooool3ool000Co0?l000?ooooooomo0?ooO`000_oo
O`02ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`?oo`000ol0
o`03o`00o`?oogl00_ooool3ool000Co0?l00_ooool3oomo00;ooooo0ooo0004o`3o00;ooooo0ooo
O`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool3ool000Co0?l00_oo
ool3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo0ooo
0003o`3o00?o003o0oooO`02ooooo`?oo`001?l0o`02ooooo`?oogl00_ooool3ool000?o0?l00ol0
0?l3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`?oo`001?l0
o`02ooooo`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl00_oo
ool3ool000?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool3ool000Co0?l00_oo
ool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`?oo`001?l0
o`02ooooo`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl00_oo
ool00ol0003oooooooooo`08ooooo`007?ooool01?l0003oooooooooool00006ooooo`03o`000?oo
oooooooo00?ooooo0ol0o`02o`00o`Coogl00_ooool4ool000Cooooo1?ooO`02ooooo`?oo`001?l0
o`000oooooooogl0oomo0002oomo00;ooooo1?oo0004ooooo`Coogl01Oooool3o`3o00;o003o1?oo
O`02ooooo`?oo`001?l0o`000oooooooogl0oomo0002oomo00;ooooo0ooo0003o`3o00?o003o0ooo
O`02ooooo`Coo`001Oooool3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`Coo`001Ooo
ool3oomo00;ooooo0ooo0004o`3o00;ooooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0
o`03o`00o`?oogl00_ooool3ool000Co0?l00_ooool3oomo00;ooooo0ooo0003o`3o00?o003o0ooo
O`02ooooo`Coo`001Oooool3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`?oo`001?l0
o`02ooooo`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl01Ooo
ool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo0ooo0003o`3o00?o003o0ooo
O`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0ooo
O`02ooooo`?oo`001?l0o`02ooooo`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol0
0?l3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo1?oo
0005ooooo`?oogl00_ooool00ol0003oooooooooo`08ooooo`007Oooool2o`0000Oooooo00?o0000
ooooooooool00oooool3o`3o00;o003o1?ooO`02ooooo`Coo`001?ooool4oomo00;ooooo0ooo0004
o`3o0003ooooooooO`3oogl000;oogl00_ooool4ool000Cooooo1?ooO`05ooooo`?o0?l00_l00?l4
oomo00;ooooo0ooo0004o`3o0003ooooooooO`3oogl000;oogl00_ooool3ool000?o0?l00ol00?l3
oomo00;ooooo1?oo000:ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`02
ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00?o003o0oooO`02
ooooo`Coo`001Oooool3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3
oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool3ool000Co0?l00_ooool3oomo00;ooooo0ooo0003
o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4
ool000Gooooo0oooO`02ooooo`?oo`001?l0o`02ooooo`?oogl00_ooool4ool000Gooooo0oooO`05
ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool3ool000Co0?l00_ooool3
oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3
oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08
ooooo`009_ooool00ol0003oooooooooo`03ooooo`?o0?l00_l00?l4oomo00;ooooo1?oo0004oooo
o`Coogl00_ooool3ool000Co0?l000?ooooooomo0?ooO`000_ooO`02ooooo`Coo`001?ooool4oomo
00Gooooo0ol0o`02o`00o`Coogl00_ooool4ool000Cooooo1?ooO`02ooooo`?oo`000ol0o`03o`00
o`?oogl00_ooool4ool000[ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo
00;ooooo1?oo0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3oomo
00;ooooo1?oo0005ooooo`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo1?oo000=oooo
o`?o0?l00ol00?l3oomo00;ooooo0ooo0004o`3o00;ooooo0oooO`02ooooo`?oo`000ol0o`03o`00
o`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005oooo
o`?oogl00_ooool3ool000Co0?l00_ooool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o
00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo0ooo0004o`3o00;ooooo0oooO`02oooo
o`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`02oooo
o`?oo`000ol0o`03o`00o`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Ko
oooo00?o0000ooooooooool00oooool3o`3o00;o003o1?ooO`02ooooo`Coo`001?ooool4oomo00;o
oooo0ooo0004o`3o0003ooooooooO`3oogl000;oogl00_ooool4ool000Cooooo1?ooO`05ooooo`?o
0?l00_l00?l4oomo00;ooooo1?oo0004ooooo`Coogl00_ooool3ool000?o0?l00ol00?l3oomo00;o
oooo1?oo000:ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`Co
o`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`Co
o`001Oooool3oomo00;ooooo0ooo0004o`3o00;ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o
003o0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`Co
o`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`?o
o`001?l0o`02ooooo`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;o
oooo1?oo0005ooooo`?oogl00_ooool3ool000Co0?l00_ooool3oomo00;ooooo1?oo0005ooooo`?o
ogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo0ooo0003o`3o00?o
003o0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooo
ooooo`03ooooo`?o0?l00_l00?l4oomo00;ooooo1?oo0004ooooo`Coogl00_ooool3ool000Co0?l0
00?ooooooomo0?ooO`000_ooO`02ooooo`Coo`001?ooool4oomo00Gooooo0ol0o`02o`00o`Coogl0
0_ooool4ool000Cooooo1?ooO`02ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool4ool000[ooooo
0ooo0003o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl0
0_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl0
0_ooool3ool000Co0?l00_ooool3oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00;ooooo
1?oo0005ooooo`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo
0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`?oo`001?l0o`02ooooo`?oogl0
0_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo000:ooooo`?oo`00
1?l0o`02ooooo`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo
1?oo0005ooooo`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo1?oo000:ooooo`03o`00
0?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo0ol0o`02o`00o`Coogl00_oo
ool4ool000Cooooo1?ooO`02ooooo`?oo`001?l0o`000oooooooogl0oomo0002oomo00;ooooo1?oo
0004ooooo`Coogl01Oooool3o`3o00?o003o1Oooool4ool000Cooooo1?ooO`05ooooo`?o0?l00ol0
0?l3oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl00_oo
ool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_oo
ool4ool000Gooooo0oooO`02ooooo`?oo`001?l0o`02ooooo`?oogl00_ooool4ool000gooooo0ol0
o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo
0005ooooo`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo0ooo
0004o`3o00;ooooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_oo
ool4ool000[ooooo1?oo0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol0
0?l3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo1?oo
000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo0ol0o`02
o`00o`Coogl00_ooool4ool000Cooooo1?ooO`02ooooo`Coo`001?ooool4oomo00;ooooo1?oo0004
ooooo`Coogl01Oooool3o`3o00?o003o1Oooool4ool000Cooooo1?ooO`05ooooo`?o0?l00ol00?l3
oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool4
ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4
ool000Gooooo0oooO`02ooooo`?oo`001?l0o`02ooooo`?oogl00_ooool4ool000gooooo0ol0o`03
o`00o`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005
ooooo`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo0ooo0004
o`3o00;ooooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4
ool000[ooooo1?oo0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3
oomo00;ooooo1?oo0005ooooo`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo1?oo000:
ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo0ol0o`02o`00
o`Coogl00_ooool4ool000Cooooo1?ooO`02ooooo`Coo`001?ooool4oomo00;ooooo1?oo0004oooo
o`Coogl01Oooool3o`3o00?o003o1Oooool4ool000Cooooo1?ooO`05ooooo`?o0?l00ol00?l3oomo
00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool4ool0
00Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool0
00Gooooo0oooO`02ooooo`?oo`001?l0o`02ooooo`?oogl00_ooool4ool000gooooo0ol0o`03o`00
o`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005oooo
o`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo1?l0o`02oooo
o`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo000:oooo
o`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00?o003o0oooO`02oooo
o`Coo`001Oooool3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`Coo`002_ooool00ol0
003oooooooooo`08ooooo`009_ooool3o`0000?ooooo0ol0o`02o`00o`Coogl00_ooool4ool000Co
oooo1?ooO`02ooooo`Coo`001?ooool4oomo00;ooooo1?oo0004ooooo`Coogl01Oooool3o`3o00?o
003o1Oooool4ool000Cooooo1?ooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo000=ooooo`?o
0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`Co
o`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`?o
o`001?l0o`02ooooo`?oogl00_ooool4ool000gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Go
oooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00?o
003o0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo1?l0o`02ooooo`?oogl00_ooool4ool000Go
oooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo000:ooooo`Coo`001Oooool3oomo00;o
oooo1?oo0005ooooo`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00;o
oooo0ooo0003o`3o00?o003o0oooO`02ooooo`Coo`002Oooool2o`0000[ooooo000Vooooo`03o`00
0?oooooooooo00?ooooo0ol0o`02o`00o`Coogl00_ooool4ool000[ooooo1?oo0004ooooo`Coogl0
0_ooool4ool000Cooooo1?ooO`05ooooo`?o0?l00ol00?l5ooooo`Coo`001?ooool4oomo00Gooooo
0ol0o`03o`00o`?oogl00_ooool4ool000gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo
0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00?o003o
0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool4ool000gooooo
0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`08ooooo`?o003o0oooO`02ooooo`Coo`00
1Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`Co0?l0
0_ooool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`00
2_ooool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl0
0_ooool4ool000Gooooo0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl00_ooool4ool000[ooooo
00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool3o`3o00;o003o1?oo
O`02ooooo`Coo`002_ooool4ool000Cooooo1?ooO`02ooooo`Coo`001?ooool4oomo00Gooooo0ol0
o`03o`00o`Gooooo1?oo0004ooooo`Coogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`003Ooo
ool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl00_oo
ool4ool000gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`Coo`001Ooo
ool3oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl02?oo
ool3o`00o`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo
0005ooooo`?oogl01Oooool4o`3o00;ooooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0
o`03o`00o`?oogl00_ooool4ool000[ooooo1?oo0005ooooo`?oogl00_ooool4ool000gooooo0ol0
o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo
000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo0ol0o`02
o`00o`Coogl00_ooool4ool000[ooooo1?oo0004ooooo`Coogl00_ooool4ool000Cooooo1?ooO`05
ooooo`?o0?l00ol00?l5ooooo`Coo`001?ooool4oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4
ool000gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3
oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool4
ool000Gooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3
oomo00Sooooo0ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00?o003o0oooO`02
ooooo`Coo`001Oooool3oomo00_ooooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03
o`00o`?oogl00_ooool4ool000[ooooo1?oo0005ooooo`?oogl00_ooool4ool000gooooo0ol0o`03
o`00o`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo000:
ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo0ol0o`02o`00
o`Coogl00_ooool4ool000[ooooo1?oo0004ooooo`Coogl00_ooool4ool000Cooooo1?ooO`05oooo
o`?o0?l00ol00?l5ooooo`Coo`001?ooool4oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool0
00gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo
00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool4ool0
00Gooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo
00Sooooo0ol00?l3oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005oooo
o`?oogl02oooool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00?o003o0oooO`02oooo
o`Coo`002_ooool4ool000Gooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`02oooo
o`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool4ool000[ooooo00?o0000oooo
ooooool02?ooool002Kooooo00?o0000ooooooooool00oooool3o`3o00;o003o1?ooO`02ooooo`Co
o`002_ooool4ool000Cooooo1?ooO`02ooooo`Coo`003Oooool3o`3o00?o003o1Oooool4ool000Co
oooo1?ooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00;o
oooo1?oo0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o
003o0oooO`02ooooo`Coo`001Oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o
003o0oooO`02ooooo`Coo`001Oooool3oomo00Sooooo0ol00?l3oomo00;ooooo1?oo000=ooooo`?o
0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl02oooool3oomo00;ooooo1?oo0005ooooo`?o
ogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`002_ooool4ool000Gooooo0oooO`02ooooo`Co
o`003Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?o
ogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0
0oooool3o`3o00;o003o1?ooO`02ooooo`Coo`002_ooool4ool000Cooooo1?ooO`02ooooo`Coo`00
3Oooool3o`3o00?o003o1Oooool4ool000Cooooo1?ooO`05ooooo`?o0?l00ol00?l3oomo00;ooooo
1?oo000=ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool4ool000Gooooo
0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`0;ooooo`?oogl0
0_ooool4ool000gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`08ooooo`?o003o
0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00_ooooo
0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000[ooooo
1?oo0005ooooo`?oogl00_ooool4ool000gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo
0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo
000Vooooo`03o`000?oooooooooo00?ooooo0ol0o`02o`00o`Coogl00_ooool4ool000[ooooo1?oo
0004ooooo`Coogl00_ooool4ool000gooooo0ol0o`03o`00o`Gooooo1?oo0004ooooo`Coogl01Ooo
ool3o`3o00?o003o0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Ooo
ool3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo
000Cooooo`?oogl00_ooool4ool000gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0ooo
O`08ooooo`?o003o0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Ooo
ool3oomo00_ooooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_oo
ool4ool000[ooooo1?oo0005ooooo`?oogl00_ooool4ool000gooooo0ol0o`03o`00o`?oogl00_oo
ool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo000:ooooo`03o`000?oo
oooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo0ol0o`02o`00o`Coogl00_ooool4
ool000[ooooo1?oo0004ooooo`Coogl00_ooool4ool000gooooo0ol0o`03o`00o`Gooooo1?oo0004
ooooo`Coogl02?ooool3o`00o`?oogl00_ooool4ool000gooooo0ol0o`03o`00o`?oogl00_ooool4
ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4
ooooo`Coo`004oooool3oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005
ooooo`?oogl02?ooool3o`00o`?oogl00_ooool4ool000gooooo0ol0o`03o`00o`?oogl00_ooool4
ool000Gooooo0oooO`0;ooooo`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol00?l3
oomo00;ooooo1?oo000:ooooo`Coo`001Oooool3oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3
oomo00;ooooo1?oo0005ooooo`?oogl02oooool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooo
oooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo0ol0o`02o`00o`Coogl00_ooool4ool0
00[ooooo1?oo0004ooooo`Coogl00_ooool4ool000gooooo0ol0o`03o`00o`Gooooo1?oo0004oooo
o`Coogl02?ooool3o`00o`?oogl00_ooool4ool000gooooo0ol0o`03o`00o`?oogl00_ooool4ool0
00Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4oooo
o`Coo`004oooool3oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005oooo
o`?oogl02?ooool3o`00o`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo0005oooo
o`?oogl02oooool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00?o003o0oooO`02oooo
o`Coo`002_ooool4ool000Gooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`02oooo
o`Coo`001Oooool3oomo00_ooooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08oooo
o`009_ooool3o`0000?ooooo0ol0o`02o`00o`Coogl00_ooool4ool000[ooooo1?oo0004ooooo`Co
ogl00_ooool4ool000gooooo0ol0o`03o`00o`Gooooo1?oo0004ooooo`Coogl02?ooool3o`00o`?o
ogl00_ooool4ool000gooooo0ol0o`03o`00o`?oogl00_ooool4ool000[ooooo1?oo0005ooooo`?o
ogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl00_ooool4ool000go
oooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`0;ooooo`?oogl00_ooool4ool000go
oooo0ol0o`04o`00o`Cooooo1?oo0005ooooo`?oogl02oooool3oomo00;ooooo1?oo0005ooooo`?o
ogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`002_ooool4ool000Gooooo0oooO`02ooooo`Co
o`003Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`0;ooooo`?oogl00_ooool4ool000Wo
oooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`03ooooo`?o0?l00_l00?l4oomo00;ooooo
1?oo000:ooooo`Coo`001?ooool4oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l5ooooo`Coo`00
1?ooool4oomo00Sooooo0ol00?l3oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00;ooooo
1?oo000:ooooo`Coo`001Oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`00
4oooool3oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl0
2oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00_ooooo
0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000[ooooo
1?oo0005ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo0005ooooo`?oogl0
2oooool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oo
oooooooo00?ooooo0ol0o`02o`00o`Coogl00_ooool4ool000[ooooo1?oo0004ooooo`Coogl00_oo
ool4ool000gooooo0ol0o`03o`00o`Gooooo1?oo0004ooooo`Coogl02?ooool3o`00o`?oogl00_oo
ool4ool000gooooo0ol0o`03o`00o`?oogl00_ooool4ool000[ooooo1?oo0005ooooo`?oogl00_oo
ool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl00_ooool4ool000gooooo0ol0
o`03o`00o`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?oo
ool4ool000Gooooo0oooO`0;ooooo`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l00ol0
0?l3oomo00;ooooo1?oo000:ooooo`Coo`001Oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l0
0?l4ooooo`Coo`001Oooool3oomo00_ooooo0oooO`02ooooo`Coo`002_ooool00ol0003ooooooooo
o`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`?o0?l00_l00?l4oomo00;ooooo1?oo000:
ooooo`Coo`001?ooool4oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l5ooooo`Coo`001?ooool4
oomo00Sooooo0ol00?l3oomo00;ooooo1?oo000=ooooo`Co0?l00_ooool3oomo00;ooooo1?oo000:
ooooo`Coo`001Oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3
oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4
ool000gooooo0ol0o`04o`00o`Cooooo1?oo0005ooooo`?oogl02oooool3oomo00;ooooo1?oo000=
ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo000:ooooo`Coo`001Oooool3oomo00;ooooo1?oo000=
ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00_ooooo0oooO`02ooooo`Coo`002_ooool0
0ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`?o0?l00_l00?l4oomo
00;ooooo1?oo000:ooooo`Coo`001?ooool4oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l5oooo
o`Coo`001?ooool4oomo00_ooooo0oooO`02ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`02oooo
o`Coo`002_ooool4ool000Gooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool0
01?ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`0;ooooo`?oogl02oooool3oomo
00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00_ooooo0oooO`02oooo
o`Coo`003Oooool3o`3o00?o003o0oooO`02ooooo`Coo`002_ooool4ool000Gooooo0oooO`02oooo
o`Coo`003Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`0;ooooo`?oogl00_ooool4ool0
00[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool3o`3o00;o
003o1?ooO`02ooooo`Coo`002_ooool4ool000Cooooo1?ooO`02ooooo`Coo`003Oooool3o`3o00?o
003o1Oooool4ool000Cooooo1?ooO`0;ooooo`?oogl00_ooool4ool000gooooo1?l0o`02ooooo`?o
ogl00_ooool4ool000[ooooo1?oo0005ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Co
oooo1?oo000Cooooo`?oogl00_ooool4ool000gooooo0ol0o`03o`00o`?oogl02oooool3oomo00_o
oooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`0;ooooo`?o
ogl00_ooool4ool000gooooo0ol0o`03o`00o`?oogl00_ooool4ool000[ooooo1?oo0005ooooo`?o
ogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl00_ooool4ool000[o
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool3o`3o00;o003o
1?ooO`02ooooo`Coo`002_ooool4ool000Cooooo1?ooO`02ooooo`Coo`003Oooool3o`3o00?o003o
1Oooool4ool000Cooooo1?ooO`0;ooooo`?oogl00_ooool4ool000gooooo1?l0o`02ooooo`?oogl0
0_ooool4ool000[ooooo1?oo0005ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo
1?oo000Cooooo`?oogl00_ooool4ool000gooooo0ol0o`03o`00o`?oogl02oooool3oomo00_ooooo
0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`0;ooooo`?oogl0
0_ooool4ool000gooooo0ol0o`03o`00o`?oogl00_ooool4ool001?ooooo0oooO`02ooooo`Coo`00
3Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`02ooooo`Coo`002_ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`?o0?l00_l00?l4oomo00;ooooo1?oo
000:ooooo`Coo`001?ooool4oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l5ooooo`Coo`001?oo
ool4oomo00_ooooo0oooO`02ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`02ooooo`Coo`002_oo
ool4ool000Gooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool001?ooooo0ooo
O`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo
000=ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Ooo
ool3o`3o00?o003o0oooO`02ooooo`Coo`004oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l0
0?l4ooooo`Coo`004oooool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000V
ooooo`03o`000?oooooooooo00?ooooo0ol0o`02o`00o`Coogl00_ooool4ool001;ooooo1?ooO`02
ooooo`Coo`003Oooool3o`3o00?o003o1Oooool4ool000Cooooo1?ooO`0;ooooo`?oogl00_ooool4
ool000gooooo1?l0o`02ooooo`?oogl00_ooool4ool000[ooooo1?oo000:ooooo`Coo`003Oooool3
o`3o00Co003o1?ooool4ool001?ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`0;
ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3
oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`02ooooo`Coo`004oooool3
oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00;ooooo1?oo000:
ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo0ol0o`02o`00
o`Coogl00_ooool4ool001;ooooo1?ooO`02ooooo`Coo`003Oooool3o`3o00?o003o1Oooool4ool0
01?ooooo0oooO`02ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`02ooooo`Coo`002_ooool4ool0
00[ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00;ooooo1?oo000=oooo
o`?o0?l00ol00?l3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00
o`Cooooo1?oo0005ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo
00;ooooo1?oo000Cooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000Coooo
o`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooooooo
ool00oooool3o`3o00;o003o1?ooO`02ooooo`Coo`004_ooool4oomo00;ooooo1?oo000=ooooo`?o
0?l00ol00?l5ooooo`Coo`004oooool3oomo00;ooooo1?oo000=ooooo`Co0?l00_ooool3oomo00;o
oooo1?oo000:ooooo`Coo`002_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?o
ogl00_ooool4ool000gooooo0ol0o`03o`00o`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`Co
o`003Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`0;ooooo`?oogl00_ooool4ool000go
oooo0ol0o`03o`00o`?oogl00_ooool4ool001?ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co
003o1?ooool4ool001?ooooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`00
9_ooool3o`0000?ooooo0ol0o`02o`00o`Coogl00_ooool4ool001;ooooo1?ooO`02ooooo`Coo`00
3Oooool3o`3o00?o003o1Oooool4ool001?ooooo0oooO`02ooooo`Coo`003Oooool4o`3o00;ooooo
0oooO`02ooooo`Coo`002_ooool4ool000[ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`00
4oooool3oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00_ooooo0oooO`0;ooooo`?oogl0
0_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo0005ooooo`?oogl02oooool3oomo00;ooooo
1?oo000=ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo000Cooooo`?oogl00_ooool4ool000gooooo
0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl00_ooool4ool000Wooooo0_l0000:ooooo`009_oo
ool00ol0003oooooooooo`03ooooo`?o0?l00_l00?l4oomo00;ooooo1?oo000Booooo`Coogl00_oo
ool4ool000gooooo0ol0o`03o`00o`Gooooo1?oo000Cooooo`?oogl00_ooool4ool000gooooo1?l0
o`02ooooo`?oogl00_ooool4ool000[ooooo1?oo000:ooooo`Coo`003Oooool3o`3o00Co003o1?oo
ool4ool001?ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`0;ooooo`?oogl02ooo
ool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00_ooooo0ooo
O`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`02ooooo`Coo`004oooool3oomo00;ooooo1?oo
000=ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00;ooooo1?oo000:ooooo`03o`000?oo
oooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo0ol0o`03o`00o`Gooooo1?oo000B
ooooo`Coogl00_ooool4ool000gooooo0ol0o`03o`00o`Gooooo1?oo000Cooooo`?oogl00_ooool4
ool000gooooo1?l0o`02ooooo`?oogl00_ooool4ool000[ooooo1?oo000:ooooo`Coo`003Oooool3
o`3o00Co003o1?ooool4ool001?ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`0;
ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3
oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`02ooooo`Coo`004oooool3
oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00;ooooo1?oo000:
ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo0ol0o`03o`00
o`Gooooo1?oo000Booooo`Coogl00_ooool4ool000gooooo0ol0o`03o`00o`Gooooo1?oo000Coooo
o`?oogl00_ooool4ool000gooooo1?l0o`02ooooo`?oogl00_ooool4ool000[ooooo1?oo000:oooo
o`Coo`003Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`02ooooo`Coo`003Oooool3o`3o
00?o003o0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4oooo
o`Coo`001Oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`02oooo
o`Coo`004oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo
00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?o
oooo0ol0o`03o`00o`Gooooo1?oo000Booooo`Coogl00_ooool4ool000gooooo0ol0o`03o`00o`Go
oooo1?oo000Cooooo`?oogl00_ooool4ool000gooooo1?l0o`02ooooo`?oogl00_ooool4ool000[o
oooo1?oo000:ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`02ooooo`Co
o`003Oooool3o`3o00?o003o0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o
0?l01?l00?l4ooooo`Coo`001Oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o
003o0oooO`02ooooo`Coo`004oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Co
o`004oooool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`00
0?oooooooooo00?ooooo0ol0o`03o`00o`Gooooo1?oo000Booooo`Coogl00_ooool4ool000gooooo
0ol0o`03o`00o`Gooooo1?oo000Cooooo`?oogl00_ooool4ool000gooooo1?l0o`02ooooo`?oogl0
0_ooool4ool000[ooooo1?oo000:ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool001?ooooo
0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo
1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00;ooooo1?oo000=ooooo`?o0?l0
0ol00?l3oomo00;ooooo1?oo000Cooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo
1?oo000Cooooo`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o
0000ooooooooool00oooool3o`3o00?o003o1Oooool4ool001;ooooo1?ooO`02ooooo`Coo`003Ooo
ool3o`3o00?o003o1Oooool4ool001?ooooo0oooO`02ooooo`Coo`003Oooool4o`3o00;ooooo0ooo
O`02ooooo`Coo`002_ooool4ool000[ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`004ooo
ool3oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00_ooooo0oooO`0;ooooo`?oogl00_oo
ool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl00_ooool4ool000gooooo0ol0
o`03o`00o`?oogl00_ooool4ool001?ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?oo
ool4ool001?ooooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool0
0ol0003oooooooooo`03ooooo`?o0?l00ol00?l5ooooo`Coo`004_ooool4oomo00;ooooo1?oo000=
ooooo`?o0?l00ol00?l5ooooo`Coo`004oooool3oomo00;ooooo1?oo000=ooooo`Co0?l00_ooool3
oomo00;ooooo1?oo000:ooooo`Coo`002_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000C
ooooo`?oogl00_ooool4ool000gooooo0ol0o`03o`00o`?oogl02oooool3oomo00_ooooo0oooO`02
ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`02ooooo`Coo`003Oooool3
o`3o00?o003o0oooO`02ooooo`Coo`004oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4
ooooo`Coo`004oooool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000Voooo
o`03o`000?oooooooooo00?ooooo0ol0o`03o`00o`Gooooo1?oo000Booooo`Coogl00_ooool4ool0
00gooooo0ol0o`03o`00o`Gooooo1?oo000Cooooo`?oogl00_ooool4ool000gooooo1?l0o`02oooo
o`?oogl00_ooool4ool001Sooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo
00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4ool0
00gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl00_ooool4ool000gooooo0ol0o`03o`00
o`?oogl00_ooool4ool001?ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool0
01?ooooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003o
ooooooooo`03ooooo`?o0?l00ol00?l5ooooo`Coo`004_ooool4oomo00;ooooo1?oo000=ooooo`?o
0?l00ol00?l5ooooo`Coo`004oooool3oomo00;ooooo1?oo000=ooooo`Co0?l00_ooool3oomo00;o
oooo1?oo000Hooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`02ooooo`Co
o`003Oooool3o`3o00?o003o0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o
0?l01?l00?l4ooooo`Coo`004oooool3oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00;o
oooo1?oo000Cooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?o
ogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo0ol00003ooooo`?o0?l0
0ol00?l5ooooo`Coo`004_ooool4oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l5ooooo`Coo`00
4oooool3oomo00;ooooo1?oo000=ooooo`Co0?l00_ooool3oomo00;ooooo1?oo000Hooooo`Coo`00
3Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o
0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`00
4oooool3oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo000Cooooo`?oogl0
0_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl00_ooool4ool000Wooooo
0_l0000:ooooo`009_ooool00ol0003oooooooooo`03ooooo`?o0?l00ol00?l5ooooo`Coo`004_oo
ool4oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l5ooooo`Coo`004oooool3oomo00;ooooo1?oo
000=ooooo`Co0?l00_ooool3oomo00;ooooo1?oo000Hooooo`Coo`003Oooool3o`3o00Co003o1?oo
ool4ool001?ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`0;ooooo`?oogl02ooo
ool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00;ooooo1?oo
000=ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo000Cooooo`?oogl00_ooool4ool0013ooooo1?l0
0?l4ooooo`Coo`004oooool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000V
ooooo`03o`000?oooooooooo00?ooooo0ol0o`03o`00o`Gooooo1?oo000Booooo`Coogl00_ooool4
ool000gooooo0ol0o`03o`00o`Gooooo1?oo000Cooooo`?oogl00_ooool4ool000gooooo1?l0o`02
ooooo`?oogl00_ooool4ool001Sooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3
oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool4
ool000gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl00_ooool4ool000gooooo0ol0o`03
o`00o`?oogl00_ooool4ool001?ooooo0oooO`02ooooo`Coo`004?ooool4o`00o`Cooooo1?oo000C
ooooo`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooo
ooooool00oooool3o`3o00?o003o1Oooool4ool001;ooooo1?ooO`02ooooo`Coo`003Oooool3o`3o
00?o003o1Oooool4ool001?ooooo0oooO`02ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`02oooo
o`Coo`006?ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl00_ooool4ool0
00gooooo0ol0o`03o`00o`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o
00Co003o1?ooool4ool001?ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`02oooo
o`Coo`004oooool3oomo00;ooooo1?oo000@ooooo`Co003o1?ooool4ool001?ooooo0oooO`02oooo
o`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`?o
0?l00ol00?l5ooooo`Coo`004_ooool4oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l5ooooo`Co
o`004oooool3oomo00;ooooo1?oo000=ooooo`Co0?l00_ooool3oomo00;ooooo1?oo000Hooooo`Co
o`003Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`02ooooo`Coo`003Oooool4o`3o00;o
oooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Co
o`004oooool3oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo000Cooooo`?o
ogl00_ooool4ool0013ooooo1?l00?l4ooooo`Coo`004oooool3oomo00;ooooo1?oo000:ooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo0ol0o`03o`00o`Gooooo
1?oo000Booooo`Coogl00_ooool4ool000gooooo0ol0o`03o`00o`Gooooo1?oo000Cooooo`?oogl0
0_ooool4ool000gooooo1?l0o`02ooooo`?oogl00_ooool4ool001Sooooo1?oo000=ooooo`?o0?l0
1?l00?l4ooooo`Coo`004oooool3oomo00;ooooo1?oo000=ooooo`Co0?l00_ooool3oomo00_ooooo
0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl0
0_ooool4ool000gooooo0ol0o`03o`00o`?oogl00_ooool4ool001?ooooo0oooO`02ooooo`Coo`00
4?ooool4o`00o`Cooooo1?oo000Cooooo`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool0
2?ooool002Kooooo00?o0000ooooooooool00oooool3o`3o00?o003o1Oooool4ool001;ooooo1?oo
O`02ooooo`Coo`003Oooool3o`3o00?o003o1Oooool4ool001?ooooo0oooO`02ooooo`Coo`003Ooo
ool4o`3o00;ooooo0oooO`02ooooo`Coo`006?ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo
000Cooooo`?oogl00_ooool4ool000gooooo1?l0o`02ooooo`?oogl02oooool3oomo00_ooooo0ooo
O`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`02ooooo`Coo`003Ooo
ool3o`3o00?o003o0oooO`02ooooo`Coo`004oooool3oomo00;ooooo1?oo000@ooooo`Co003o1?oo
ool4ool001?ooooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`009_ooool0
0ol0003oooooooooo`03ooooo`?o0?l00ol00?l5ooooo`Coo`004_ooool4oomo00;ooooo1?oo000=
ooooo`?o0?l00ol00?l5ooooo`Coo`004oooool3oomo00;ooooo1?oo000=ooooo`Co0?l00_ooool3
oomo00;ooooo1?oo000Hooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool001?ooooo0oooO`02
ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo000=
ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00;ooooo1?oo000=ooooo`?o0?l00ol00?l3
oomo00;ooooo1?oo000Cooooo`?oogl00_ooool4ool0013ooooo1?l00?l4ooooo`Coo`004oooool3
oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000;ooooo`;o00001?ooool00ol0
003oooooooooo`03ooooo`;o00000oooool3o`0000Oooooo00?o0000ooooooooool00oooool3o`3o
00?o003o1Oooool4ool001;ooooo1?ooO`02ooooo`Coo`003Oooool3o`3o00?o003o1Oooool4ool0
01?ooooo0oooO`02ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`02ooooo`Coo`006?ooool4ool0
00gooooo0ol0o`04o`00o`Cooooo1?oo000Cooooo`?oogl00_ooool4ool000gooooo1?l0o`02oooo
o`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool0
01?ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00?o003o0oooO`02ooooo`Coo`004oooool3oomo
00;ooooo1?oo000@ooooo`Co003o1?ooool4ool001?ooooo0oooO`02ooooo`Coo`002_ooool00ol0
003oooooooooo`08ooooo`002_ooool01?l0003oooooooooool00008ooooo`06o`000?oooooooooo
o`000?oooooo00000oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00?ooooo0ol0
o`03o`00o`Gooooo1?oo000Hooooo`Coo`003Oooool3o`3o00?o003o1Oooool4ool001?ooooo0ooo
O`02ooooo`Coo`004oooool3oomo00;ooooo1?oo000Hooooo`Coo`004?ooool4o`00oa_ooooo0ooo
O`02ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo
000=ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00;ooooo1?oo000=ooooo`?o0?l00ol0
0?l3oomo00;ooooo1?oo000Cooooo`?oogl00_ooool4ool0013ooooo1?l00?l4ooooo`Coo`004ooo
ool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo000:ooooo`04o`000?oooooo
ooooo`0000Sooooo00Co0000oooooooooooo00001Oooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00?ooooo0ol0o`03o`00o`Gooooo1?oo000Hooooo`Coo`003Oooool3o`3o00?o003o
1Oooool4ool001?ooooo0oooO`02ooooo`Coo`004oooool3oomo00;ooooo1?oo000Hooooo`Coo`00
4?ooool4o`00oa_ooooo0oooO`02ooooo`Coo`003Oooool4o`3o00;ooooo0oooO`0;ooooo`?oogl0
2oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00;ooooo
1?oo000=ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo000Cooooo`?oogl00_ooool4ool0013ooooo
1?l00?l4ooooo`Coo`004oooool3oomo00;ooooo1?oo000:ooooo`03o`000?oooooooooo00Sooooo
000:ooooo`04o`000?ooooooooooo`0000Sooooo00Co0000oooooooooooo00001Oooool00ol0003o
ooooooooo`04ooooo`Co00000_ooool3o`3o00?o003o1Oooool4ool001Sooooo1?oo000=ooooo`?o
0?l00ol00?l5ooooo`Coo`004oooool3oomo01Wooooo0oooO`02ooooo`Coo`006?ooool4ool0013o
oooo1?l00?lKooooo`?oogl00_ooool4ool000gooooo1?l0o`0@ooooo`?oogl02oooool3oomo00;o
oooo1?oo000=ooooo`?o0?l01?l00?l4ooooo`Coo`004oooool3oomo00;ooooo1?oo000=ooooo`?o
0?l00ol00?l3oomo00;ooooo1?oo000Hooooo`Coo`004?ooool4o`00o`Cooooo1?oo000Cooooo`?o
ogl00_ooool4ool000Oooooo1?l0000:ooooo`002_ooool01?l0003oooooooooool00008ooooo`05
o`000?ooooooooooo`000?ooool01?l00007ooooo`03o`000?oooooooooo00?ooooo0ol0o`03o`00
o`Gooooo1?oo000Hooooo`Coo`003Oooool3o`3o00?o003o1Oooool4ool001?ooooo0oooO`0Ioooo
o`?oogl00_ooool4ool001Sooooo1?oo000@ooooo`Co003o6oooool3oomo00;ooooo1?oo000=oooo
o`Co0?l04?ooool3oomo00_ooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o6oooool3oomo
01?ooooo1?l0o`02ooooo`?oogl00_ooool4ool001Sooooo1?oo000@ooooo`Co003o1?ooool4ool0
01?ooooo0oooO`02ooooo`Coo`002_ooool00ol0003oooooooooo`08ooooo`002_ooool01?l0003o
ooooooooool00008ooooo`06o`000?ooooooooooo`000?oooooo00002_ooool00ol0003ooooooooo
o`03ooooo`?o0?l00ol00?l5ooooo`Coo`006?ooool4ool000gooooo0ol0o`03o`00o`Gooooo1?oo
000Cooooo`?oogl06Oooool3oomo00;ooooo1?oo000Hooooo`Coo`004?ooool4o`00oa_ooooo0ooo
O`02ooooo`Coo`003Oooool4o`3o013ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0
o`04o`00oa_ooooo0oooO`0Cooooo`Co0?l00_ooool3oomo00;ooooo1?oo000Hooooo`Coo`004?oo
ool4o`00o`Cooooo1?oo000Cooooo`?oogl00_ooool4ool000[ooooo00?o0000ooooooooool02?oo
ool000_ooooo0_l0000:ooooo`;o00000_ooool5o`0000Kooooo00?o0000ooooooooool00oooool3
o`3o00?o003o1Oooool4ool001Sooooo1?oo000=ooooo`?o0?l00ol00?l5ooooo`Coo`004oooool3
oomo01Wooooo0oooO`02ooooo`Coo`006?ooool4ool0013ooooo1?l00?lKooooo`?oogl00_ooool4
ool000gooooo1?l0o`0@ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?lK
ooooo`?oogl04oooool4o`3o00;ooooo0oooO`02ooooo`Coo`006?ooool4ool0013ooooo1?l00?l4
ooooo`Coo`004oooool3oomo013ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooo
ooooool00oooool3o`3o00?o003o1Oooool4ool001Sooooo1?oo000=ooooo`?o0?l00ol00?lLoooo
o`?oogl06Oooool3oomo00;ooooo1?oo000Hooooo`Coo`004?ooool4o`00oa_ooooo0oooO`02oooo
o`Coo`003Oooool4o`3o013ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00
oa_ooooo0oooO`0Cooooo`Co0?l00_ooool3oomo00;ooooo1?oo000Hooooo`Coo`004?ooool4o`00
oa_ooooo0oooO`0@ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?o
oooo0ol0o`03o`00o`Gooooo1?oo000Hooooo`Coo`003Oooool3o`3o00?o003o7?ooool3oomo01Wo
oooo0oooO`02ooooo`Coo`006?ooool4ool0013ooooo1?l00?lKooooo`?oogl00_ooool4ool000go
oooo1?l0o`0@ooooo`?oogl02oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?lKooooo`?o
ogl04oooool4o`3o00;ooooo0oooO`02ooooo`Coo`006?ooool4ool0013ooooo1?l00?lKooooo`?o
ogl04?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`?o0?l0
0ol00?l5ooooo`Coo`00:Oooool3o`3o00?o003o7?ooool3oomo01Wooooo0oooO`02ooooo`Coo`00
6?ooool4ool0013ooooo1?l00?lKooooo`?oogl00_ooool4ool000gooooo1?l0o`0@ooooo`?oogl0
2oooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?lKooooo`?oogl04oooool4o`3o00;ooooo
0oooO`02ooooo`Coo`00;?ooool4o`00oa_ooooo0oooO`0@ooooo`03o`000?oooooooooo00Sooooo
000Vooooo`03o`000?oooooooooo00?ooooo0ol0o`03o`00o`Gooooo1?oo000Yooooo`?o0?l00ol0
0?lLooooo`?oogl06Oooool3oomo00;ooooo1?oo000Hooooo`Coo`004?ooool4o`00oa_ooooo0ooo
O`02ooooo`Coo`003Oooool4o`3o013ooooo0oooO`0;ooooo`?oogl00_ooool4ool000gooooo0ol0
o`04o`00oa_ooooo0oooO`0Cooooo`Co0?l00_ooool3oomo00;ooooo1?oo000/ooooo`Co003o6ooo
ool3oomo013ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool3
o`3o00?o003o<_ooool3o`3o00?o003o7?ooool3oomo01Wooooo0oooO`02ooooo`Coo`006?ooool4
ool0013ooooo1?l00?lKooooo`?oogl04oooool4o`3o013ooooo0oooO`0;ooooo`?oogl00_ooool4
ool000gooooo0ol0o`04o`00oa_ooooo0oooO`0Cooooo`Co0?l00_ooool3oomo00;ooooo1?oo000/
ooooo`Co003o6oooool3oomo013ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooo
ooooool00oooool3o`3o00?o003o<_ooool3o`3o00?o003o7?ooool3oomo01Wooooo0oooO`02oooo
o`Coo`00;?ooool4o`00oa_ooooo0oooO`0Cooooo`Co0?l07_ooool3oomo00;ooooo1?oo000=oooo
o`?o0?l01?l00?lKooooo`?oogl04oooool4o`3o03Oooooo1?l00?lKooooo`?oogl04?ooool00ol0
003oooooooooo`08ooooo`009_ooool3o`0000?ooooo0ol0o`03o`00oc;ooooo0ol0o`03o`00oaco
oooo0oooO`0Iooooo`?oogl00_ooool4ool002cooooo1?l00?lKooooo`?oogl04oooool4o`3o01ko
oooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o6oooool3oomo01?ooooo1?l0o`0gooooo`Co
003o6oooool3oomo00oooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`03ooooo`?o0?l0
0ol00?lbooooo`?o0?l00ol00?lLooooo`?oogl06Oooool3oomo00;ooooo1?oo000/ooooo`Co003o
<Oooool4o`3o01kooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o6oooool3oomo01?ooooo
1?l0o`0gooooo`Co003o6oooool3oomo013ooooo00?o0000ooooooooool02?ooool002Kooooo00?o
0000ooooooooool00oooool3o`3o00?o003o<_ooool3o`3o00?o003o7?ooool3oomo01Wooooo0ooo
O`0booooo`Co003o<Oooool4o`3o01kooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o6ooo
ool3oomo01?ooooo1?l0o`0gooooo`Co003o6oooool3oomo013ooooo00?o0000ooooooooool02?oo
ool002Kooooo00?o0000ooooooooool00oooool3o`3o00?o003o<_ooool3o`3o00?o003o7?ooool3
oomo01Wooooo0oooO`0booooo`Co003o<Oooool4o`3o01kooooo0oooO`02ooooo`Coo`003Oooool3
o`3o00Co003o6oooool3oomo01?ooooo1?l0o`0gooooo`Co003o6oooool3oomo013ooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool3o`3o00?o003o<_ooool3o`3o
00?o003o7?ooool3oomo01Wooooo0oooO`0booooo`Co003o<Oooool4o`3o01kooooo0oooO`02oooo
o`Coo`003Oooool3o`3o00Co003o6oooool3oomo01?ooooo1?l0o`0gooooo`Co003o6oooool3oomo
013ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool3o`3o00?o
003o<_ooool3o`3o00?o003o7?ooool3oomo01Wooooo0oooO`0booooo`Co003o<Oooool4o`3o01ko
oooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o6oooool3oomo01?ooooo1?l0o`0gooooo`Co
003o6oooool3oomo013ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0
0oooool3o`3o00?o003o<_ooool3o`3o00?o003o7?ooool3oomo01Wooooo0oooO`0booooo`Co003o
<Oooool4o`3o01kooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o6oooool3oomo01?ooooo
1?l0o`0gooooo`Co003o6oooool3oomo013ooooo00?o0000ooooooooool02?ooool002Kooooo00?o
0000ooooooooool00oooool3o`3o00?o003o<_ooool3o`3o00?o003o7?ooool3oomo01Wooooo0ooo
O`0booooo`Co003o<Oooool4o`3o01kooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o6ooo
ool3oomo01?ooooo1?l0o`0gooooo`Co003o;_ooool00ol0003oooooooooo`08ooooo`009_ooool0
0ol0003oooooooooo`03ooooo`?o0?l00ol00?lbooooo`?o0?l00ol00?lhooooo`?oogl0<_ooool4
o`00oc7ooooo1?l0o`0Nooooo`?oogl00_ooool4ool000gooooo0ol0o`04o`00oa_ooooo0oooO`0C
ooooo`Co0?l0=oooool4o`00obkooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooo
ooooool00oooool3o`3o00?o003o<_ooool3o`3o00?o003o>?ooool3oomo03;ooooo1?l00?laoooo
o`Co0?l07_ooool3oomo00;ooooo1?oo000=ooooo`?o0?l01?l00?lKooooo`?oogl04oooool4o`3o
03Oooooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo000Vooooo`?o00000oooool3o`3o00?o
003o<_ooool3o`3o00?o003o>?ooool3oomo03;ooooo1?l00?laooooo`Co0?l07_ooool3oomo00;o
oooo1?oo000=ooooo`?o0?l01?l00?lKooooo`?oogl04oooool4o`3o03Oooooo1?l00?l]ooooo`;o
00002_ooool002Kooooo00?o0000ooooooooool00oooool3o`3o00?o003o<_ooool3o`3o00?o003o
>?ooool3oomo03;ooooo1?l00?laooooo`Co0?l07_ooool3oomo00;ooooo1?oo000=ooooo`?o0?l0
1?l00?lKooooo`?oogl04oooool4o`3o03Oooooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo
000Vooooo`03o`000?oooooooooo00?ooooo0ol0o`03o`00oc;ooooo0ol0o`03o`00ocSooooo0ooo
O`0booooo`Co003o<Oooool4o`3o01kooooo0oooO`02ooooo`Coo`003Oooool3o`3o00Co003o6ooo
ool3oomo01?ooooo1?l0o`0gooooo`Co003o;_ooool00ol0003oooooooooo`08ooooo`009_ooool0
0ol0003oooooooooo`03ooooo`?o0?l00ol00?lbooooo`?o0?l00ol00?m]ooooo`Co003o<Oooool4
o`3o01kooooo0oooO`0Cooooo`?o0?l01?l00?laooooo`Co0?l0=oooool4o`00obkooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool3o`3o00?o003o<_ooool3o`3o
00?o003oKOooool4o`00oc7ooooo1?l0o`0Nooooo`?oogl04oooool3o`3o00Co003o<Oooool4o`3o
03Oooooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?o
oooo0ol0o`03o`00oc;ooooo0ol0o`03o`00ofgooooo1?l00?laooooo`Co0?l07_ooool3oomo01?o
oooo0ol0o`04o`00oc7ooooo1?l0o`0gooooo`Co003o;_ooool00ol0003oooooooooo`08ooooo`00
9_ooool00ol0003oooooooooo`03ooooo`?o0?l00ol00?lbooooo`?o0?l00ol00?m]ooooo`Co003o
<Oooool4o`3o01kooooo0oooO`0Cooooo`?o0?l01?l00?laooooo`Co0?l0=oooool4o`00obkooooo
00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool3o`3o00?o003o<_oo
ool3o`3o00?o003oKOooool4o`00oc7ooooo1?l0o`0dooooo`?o0?l01?l00?laooooo`Co0?l0=ooo
ool4o`00obkooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool3
o`3o00?o003o<_ooool3o`3o00?o003oKOooool4o`00oc7ooooo1?l0o`0dooooo`?o0?l01?l00?la
ooooo`Co0?l0=oooool4o`00obkooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooo
ooooool00oooool3o`3o00?o003o<_ooool3o`3o00?o003oKOooool4o`00oc7ooooo1?l0o`0doooo
o`?o0?l01?l00?laooooo`Co0?l0=oooool4o`00obkooooo00?o0000ooooooooool02?ooool002Ko
oooo00?o0000ooooooooool00oooool3o`3o00?o003o<_ooool3o`3o00?o003oKOooool4o`00oc7o
oooo1?l0o`0dooooo`?o0?l01?l00?laooooo`Co0?l0=oooool4o`00obkooooo00?o0000oooooooo
ool02?ooool002Kooooo0ol00003ooooo`?o0?l00ol00?lbooooo`?o0?l00ol00?m]ooooo`Co003o
<Oooool4o`3o03Cooooo0ol0o`04o`00oc7ooooo1?l0o`0gooooo`Co003o;Oooool2o`0000[ooooo
000Vooooo`03o`000?oooooooooo00?ooooo0ol0o`03o`00oc;ooooo0ol0o`03o`00ofgooooo1?l0
0?laooooo`Co0?l0=?ooool3o`3o00Co003o<Oooool4o`3o03Oooooo1?l00?l^ooooo`03o`000?oo
oooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo0ol0o`03o`00oc;ooooo0ol0o`03
o`00ofgooooo1?l00?laooooo`Co0?l0=?ooool3o`3o00Co003o<Oooool4o`3o03Oooooo1?l00?l^
ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo0ol0o`03o`00
oc;ooooo0ol0o`03o`00ofgooooo1?l00?laooooo`Co0?l0=?ooool3o`3o00Co003o<Oooool4o`3o
03Oooooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?o
oooo0ol0o`03o`00oc;ooooo0ol0o`03o`00ofgooooo1?l00?laooooo`Co0?l0=?ooool3o`3o00Co
003o<Oooool4o`3o03Oooooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`00
0?oooooooooo00?ooooo0ol0o`03o`00oc;ooooo0ol0o`03o`00ofgooooo1?l00?laooooo`Co0?l0
=?ooool3o`3o00Co003oK?ooool4o`00obkooooo00?o0000ooooooooool02?ooool002Kooooo00?o
0000ooooooooool00oooool3o`3o00?o003o<_ooool3o`3o00?o003oKOooool4o`00oc7ooooo1?l0
o`0dooooo`?o0?l01?l00?m/ooooo`Co003o;_ooool00ol0003oooooooooo`08ooooo`009_ooool0
0ol0003oooooooooo`03ooooo`?o0?l00ol00?lbooooo`?o0?l00ol00?m]ooooo`Co003o<Oooool4
o`3o03Cooooo0ol0o`04o`00ofcooooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo000Voooo
o`03o`000?oooooooooo00?ooooo0ol0o`03o`00oc;ooooo0ol0o`03o`00ofgooooo1?l00?laoooo
o`Co0?l0=?ooool3o`3o00Co003oK?ooool4o`00obkooooo00?o0000ooooooooool02?ooool002Ko
oooo00?o0000ooooooooool00oooool3o`3o00?o003o<_ooool3o`3o00?o003oKOooool4o`00oc7o
oooo1?l0o`0dooooo`?o0?l01?l00?m/ooooo`Co003o;_ooool00ol0003oooooooooo`08ooooo`00
9_ooool3o`0000?ooooo1?l0o`0dooooo`?o0?l00ol00?m]ooooo`Co003o<Oooool4o`3o03Cooooo
0ol0o`04o`00ofcooooo1?l00?l]ooooo`;o00002_ooool002Kooooo00?o0000ooooooooool00ooo
ool4o`3o03Cooooo0ol0o`03o`00ofgooooo1?l00?laooooo`Co0?l0=?ooool3o`3o00Co003oK?oo
ool4o`00obkooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool4
o`3o03Cooooo0ol0o`03o`00ofgooooo1?l00?laooooo`Co0?l0=?ooool3o`3o00Co003oK?ooool4
o`00obkooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool4o`3o
03Cooooo0ol0o`03o`00ofgooooo1?l00?laooooo`Co0?l0=?ooool3o`3o00Co003oK?ooool4o`00
obkooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool4o`3o03Co
oooo0ol0o`03o`00ofgooooo1?l00?laooooo`Co0?l0=?ooool3o`3o00Co003oK?ooool4o`00obko
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool4o`3o03Cooooo
0ol0o`03o`00ofgooooo1?l00?laooooo`Co0?l0=?ooool3o`3o00Co003oK?ooool4o`00obkooooo
00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool4o`3o03Cooooo0ol0
o`03o`00ofgooooo1?l00?mYooooo`?o0?l01?l00?m/ooooo`Co003o;_ooool00ol0003ooooooooo
o`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0=?ooool3o`3o00?o003oKOooool4
o`00ofWooooo0ol0o`04o`00ofcooooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo000Aoooo
o`;o00001?ooool00ol0003oooooooooo`02ooooo`?o00001oooool00ol0003oooooooooo`03oooo
o`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00ofWooooo0ol0o`04o`00ofcooooo1?l00?l^oooo
o`03o`000?oooooooooo00Sooooo000@ooooo`04o`000?ooooooooooo`0000Wooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`03ooooo`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00
ofWooooo0ol0o`04o`00ofcooooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo000@ooooo`04
o`000?ooooooooooo`0000Wooooo00?o0000ooooooooool01_ooool4o`0000;ooooo1?l0o`0doooo
o`?o0?l00ol00?m]ooooo`Co003oJOooool3o`3o00Co003oK?ooool4o`00ob_ooooo1?l0000:oooo
o`004?ooool01?l0003oooooooooool00009ooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool00oooool4o`3o03Cooooo0ol0o`03o`00ofgooooo1?l00?mYooooo`?o0?l01?l00?m/oooo
o`Co003o;_ooool00ol0003oooooooooo`08ooooo`004?ooool01?l0003oooooooooool00009oooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool00oooool4o`3o03Cooooo0ol0o`03o`00
ofgooooo1?l00?mYooooo`?o0?l01?l00?m/ooooo`Co003o;_ooool00ol0003oooooooooo`08oooo
o`004?ooool01?l0003oooooooooool00008ooooo`;o00002?ooool00ol0003oooooooooo`03oooo
o`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00ofWooooo0ol0o`04o`00ofcooooo1?l00?l^oooo
o`03o`000?oooooooooo00Sooooo000Aooooo`;o00002_ooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo00?ooooo1?l0o`0dooooo`?o0?l00ol00?m]ooooo`Co003oJOooool3o`3o00Co
003oK?ooool4o`00obkooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0
0oooool4o`3o03Cooooo0ol0o`03o`00ofgooooo1?l00?mYooooo`?o0?l01?l00?m/ooooo`Co003o
;_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0=?oo
ool3o`3o00?o003oKOooool4o`00ofWooooo0ol0o`04o`00ofcooooo1?l00?l^ooooo`03o`000?oo
oooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo1?l0o`0dooooo`?o0?l00ol00?m]
ooooo`Co003oJOooool3o`3o00Co003oK?ooool4o`00obkooooo00?o0000ooooooooool02?ooool0
02Kooooo00?o0000ooooooooool00oooool4o`3o03Cooooo0ol0o`03o`00ofgooooo1?l00?mYoooo
o`?o0?l01?l00?m/ooooo`Co003o;_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003o
ooooooooo`03ooooo`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00ofWooooo0ol0o`04o`00ofco
oooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo
1?l0o`0dooooo`?o0?l00ol00?m]ooooo`Co003oJOooool3o`3o00Co003oK?ooool4o`00obkooooo
00?o0000ooooooooool02?ooool002Kooooo0ol00003ooooo`Co0?l0=?ooool3o`3o00?o003oKOoo
ool4o`00ofWooooo0ol0o`04o`00ofcooooo1?l00?l]ooooo`;o00002_ooool002Kooooo00?o0000
ooooooooool00oooool4o`3o03Cooooo0ol0o`03o`00ofgooooo1?l00?mYooooo`?o0?l01?l00?m/
ooooo`Co003o;_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03oooo
o`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00ofWooooo0ol0o`04o`00ofcooooo1?l00?l^oooo
o`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo1?l0o`0dooooo`?o
0?l00ol00?m]ooooo`Co003oJOooool3o`3o00Co003oK?ooool4o`00obkooooo00?o0000oooooooo
ool02?ooool002Kooooo00?o0000ooooooooool00oooool4o`3o03Cooooo0ol0o`03o`00ofgooooo
1?l00?mYooooo`?o0?l01?l00?m/ooooo`Co003o;_ooool00ol0003oooooooooo`08ooooo`009_oo
ool00ol0003oooooooooo`03ooooo`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00ofWooooo0ol0
o`04o`00ofcooooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooo
oooo00?ooooo1?l0o`0dooooo`?o0?l00ol00?m]ooooo`Co003oJOooool3o`3o00Co003oK?ooool4
o`00obkooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool4o`3o
03Cooooo0ol0o`03o`00ofgooooo1?l00?mYooooo`?o0?l01?l00?m/ooooo`Co003o;_ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0=?ooool3o`3o00?o
003oKOooool4o`00ofWooooo0ol0o`04o`00ofcooooo1?l00?l^ooooo`03o`000?oooooooooo00So
oooo000Vooooo`03o`000?oooooooooo00?ooooo1?l0o`0dooooo`?o0?l00ol00?m]ooooo`Co003o
JOooool3o`3o00Co003oK?ooool4o`00obkooooo00?o0000ooooooooool02?ooool002Kooooo0ol0
0003ooooo`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00ofWooooo0ol0o`04o`00ofcooooo1?l0
0?l]ooooo`;o00002_ooool002Kooooo00?o0000ooooooooool00oooool4o`3o03Cooooo0ol0o`03
o`00ofgooooo1?l00?mYooooo`?o0?l01?l00?m/ooooo`Co003o;_ooool00ol0003oooooooooo`08
ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00
ofWooooo0ol0o`04o`00ofcooooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03
o`000?oooooooooo00?ooooo1?l0o`0dooooo`?o0?l00ol00?m]ooooo`Co003oJOooool3o`3o00Co
003oK?ooool4o`00obkooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0
0oooool4o`3o03Cooooo0ol0o`03o`00ofgooooo1?l00?mYooooo`?o0?l01?l00?m/ooooo`Co003o
;_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0=?oo
ool3o`3o00?o003oKOooool4o`00ofWooooo0ol0o`04o`00ofcooooo1?l00?l^ooooo`03o`000?oo
oooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo1?l0o`0dooooo`?o0?l00ol00?m]
ooooo`Co003oJOooool3o`3o00Co003oK?ooool4o`00obkooooo00?o0000ooooooooool02?ooool0
02Kooooo00?o0000ooooooooool00oooool4o`3o03Cooooo0ol0o`03o`00ofgooooo1?l00?mYoooo
o`?o0?l01?l00?m/ooooo`Co003o;_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003o
ooooooooo`03ooooo`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00ofWooooo0ol0o`04o`00ofco
oooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo
1?l0o`0dooooo`?o0?l00ol00?m]ooooo`Co003oJOooool3o`3o00Co003oK?ooool4o`00obkooooo
00?o0000ooooooooool02?ooool002Kooooo0ol00003ooooo`Co0?l0=?ooool3o`3o00?o003oKOoo
ool4o`00ofWooooo0ol0o`04o`00ofcooooo1?l00?l]ooooo`;o00002_ooool002Kooooo00?o0000
ooooooooool00oooool4o`3o03Cooooo0ol0o`03o`00ofgooooo1?l00?mYooooo`?o0?l01?l00?m/
ooooo`Co003o;_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03oooo
o`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00ofWooooo0ol0o`04o`00ofcooooo1?l00?l^oooo
o`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo1?l0o`0dooooo`?o
0?l00ol00?m]ooooo`Co003oJOooool4o`3o06oooooo1?l00?l^ooooo`03o`000?oooooooooo00So
oooo000Vooooo`03o`000?oooooooooo00?ooooo1?l0o`0dooooo`?o0?l00ol00?m]ooooo`Co003o
JOooool4o`3o06oooooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oo
oooooooo00?ooooo1?l0o`0dooooo`?o0?l00ol00?m]ooooo`Co003oJOooool4o`3o06oooooo1?l0
0?l^ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo1?l0o`0d
ooooo`?o0?l00ol00?m]ooooo`Co003oJOooool4o`3o06oooooo1?l00?l^ooooo`03o`000?oooooo
oooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo1?l0o`0dooooo`?o0?l00ol00?m]oooo
o`Co003oJOooool4o`3o06oooooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03
o`000?oooooooooo00?ooooo1?l0o`0dooooo`?o0?l00ol00?m]ooooo`Co003oJOooool4o`3o06oo
oooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo
1?l0o`0dooooo`?o0?l00ol00?m]ooooo`Co003oJOooool4o`3o06oooooo1?l00?l^ooooo`03o`00
0?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo1?l0o`0dooooo`?o0?l00ol0
0?m]ooooo`Co003oJOooool4o`3o06oooooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo000V
ooooo`?o00000oooool4o`3o03Cooooo0ol0o`03o`00ofgooooo1?l00?mYooooo`Co0?l0Koooool4
o`00obgooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0=?ooool3o`3o
00?o003oKOooool4o`00ofWooooo1?l0o`1_ooooo`Co003o;_ooool00ol0003oooooooooo`08oooo
o`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00ofWo
oooo1?l0o`1_ooooo`Co003o;_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooo
ooooo`03ooooo`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00ofWooooo1?l0o`1_ooooo`Co003o
;_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0=?oo
ool3o`3o00?o003oKOooool4o`00ofWooooo1?l0o`1_ooooo`Co003o;_ooool00ol0003ooooooooo
o`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0=?ooool3o`3o00?o003oKOooool4
o`00ofWooooo1?l0o`1_ooooo`Co003o;_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0
003oooooooooo`03ooooo`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00ofWooooo1?l0o`1_oooo
o`Co003o;_ooool00ol0003oooooooooo`08ooooo`002oooool2o`0000Cooooo00?o0000oooooooo
ool00_ooool3o`0000?ooooo0ol00007ooooo`03o`000?oooooooooo00?ooooo1?l0o`0dooooo`?o
0?l00ol00?m]ooooo`Co003oJOooool4o`3o06oooooo1?l00?l^ooooo`03o`000?oooooooooo00So
oooo000:ooooo`04o`000?ooooooooooo`0000Wooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00oooool4o`3o03Cooooo0ol0o`03o`00
ofgooooo1?l00?mYooooo`Co0?l0Koooool4o`00obkooooo00?o0000ooooooooool02?ooool000[o
oooo00Co0000oooooooooooo00002Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool00oooool4o`3o03Cooooo0ol0o`03o`00ofgooooo1?l00?mYoooo
o`Co0?l0Koooool4o`00obkooooo00?o0000ooooooooool02?ooool000[ooooo00Co0000oooooooo
oooo00002Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo1?l00002oooo
o`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00ofWooooo1?l0o`1_ooooo`Co003o:oooool4o`00
00[ooooo000:ooooo`04o`000?ooooooooooo`0000Wooooo00Co0000oooooooooooooooo1?l00007
ooooo`03o`000?oooooooooo00?ooooo1?l0o`0dooooo`?o0?l00ol00?m]ooooo`Co003oJOooool4
o`3o06oooooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo000:ooooo`04o`000?oooooooooo
o`0000Sooooo0_l00003ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool00oooool4
o`3o03Cooooo0ol0o`03o`00ofgooooo1?l00?mYooooo`Co0?l0Koooool4o`00obkooooo00?o0000
ooooooooool02?ooool000_ooooo0_l0000:ooooo`04o`000?ooooooooooooooo`Go00001_ooool0
0ol0003oooooooooo`03ooooo`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00ofWooooo1?l0o`1_
ooooo`Co003o;_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03oooo
o`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00ofWooooo1?l0o`1_ooooo`Co003o;_ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0=?ooool3o`3o00?o
003oKOooool4o`00ofWooooo1?l0o`1_ooooo`Co003o;_ooool00ol0003oooooooooo`08ooooo`00
9_ooool00ol0003oooooooooo`03ooooo`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00ofWooooo
1?l0o`1_ooooo`Co003o;_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooooooo
o`03ooooo`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00ofWooooo1?l0o`1_ooooo`Co003o;_oo
ool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0=?ooool3
o`3o00?o003oKOooool4o`00ofWooooo1?l0o`1_ooooo`Co003o;_ooool00ol0003oooooooooo`08
ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0=?ooool3o`3o00?o003oKOooool4o`00
ofWooooo1?l0o`1_ooooo`Co003o;_ooool00ol0003oooooooooo`08ooooo`009_ooool3o`0000?o
oooo1?l0o`0dooooo`?o0?l00ol00?m]ooooo`Co003oJOooool4o`3o06oooooo1?l00?l]ooooo`;o
00002_ooool002Kooooo00?o0000ooooooooool00oooool4o`3o03Cooooo0ol0o`03o`00ofgooooo
1?l00?mYooooo`Co0?l0Koooool4o`00obkooooo00?o0000ooooooooool02?ooool002Kooooo00?o
0000ooooooooool00oooool4o`3o03Cooooo0ol0o`03o`00ofgooooo1?l00?mYooooo`Co0?l0Kooo
ool4o`00obkooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool4
o`3o03Cooooo0ol0o`03o`00ofgooooo1?l00?mYooooo`Co0?l0Koooool4o`00obkooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool4o`3o03Cooooo0ol0o`03o`00
ofgooooo1?l00?mYooooo`Co0?l0Koooool4o`00obkooooo00?o0000ooooooooool02?ooool002Ko
oooo00?o0000ooooooooool00oooool4o`3o03Cooooo0ol0o`03o`00ofgooooo1?l00?mYooooo`Co
0?l0Koooool4o`00obkooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0
>oooool3o`3o00?o003oKOooool4o`00ofWooooo1?l0o`1_ooooo`Co003o;_ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`0kooooo`?o0?l00ol00?m]ooooo`Co003oJOoo
ool4o`3o06oooooo1?l00?l^ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooo
oooo03_ooooo0ol0o`03o`00ofgooooo1?l00?mYooooo`Co0?l0Koooool4o`00obkooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool0>oooool3o`3o00?o003oKOooool4o`00
ofWooooo1?l0o`1_ooooo`Co003o;_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003o
ooooooooo`0kooooo`?o0?l00ol00?m]ooooo`Co003oJOooool4o`3o06oooooo1?l00?l^ooooo`03
o`000?oooooooooo00Sooooo000Vooooo`?o0000>oooool3o`3o00?o003oKOooool4o`00ofWooooo
1?l0o`1_ooooo`Co003o;Oooool2o`0000[ooooo000Vooooo`03o`000?oooooooooo03_ooooo0ol0
o`03o`00ofgooooo1?l00?mYooooo`Co0?l0XOooool00ol0003oooooooooo`08ooooo`009_ooool0
0ol0003oooooooooo`0kooooo`?o0?l00ol00?m]ooooo`Co003oJOooool4o`3o0:7ooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool0>oooool3o`3o00?o003oKOooool4o`00
ofWooooo1?l0o`2Qooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo03_o
oooo0ol0o`03o`00ofgooooo1?l00?mYooooo`Co0?l0XOooool00ol0003oooooooooo`08ooooo`00
9_ooool00ol0003oooooooooo`0kooooo`?o0?l00ol00?m]ooooo`Co003oJOooool4o`3o0:7ooooo
00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0>oooool3o`3o00?o003oKOoo
ool4o`00ofWooooo1?l0o`2Qooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooo
oooo03_ooooo0ol0o`03o`00ofgooooo1?l00?mYooooo`Co0?l0XOooool00ol0003oooooooooo`08
ooooo`009_ooool00ol0003oooooooooo`0kooooo`?o0?l00ol00?m]ooooo`Co003oooooool?oooo
o`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo03_ooooo0ol0o`03o`00ofgo
oooo1?l00?ooooooo`oooooo00?o0000ooooooooool02?ooool002Kooooo0ol0000kooooo`?o0?l0
0ol00?m]ooooo`Co003oooooool>ooooo`;o00002_ooool002Kooooo00?o0000ooooooooool0>ooo
ool3o`3o00?o003oKOooool4o`00oooooooo3oooool00ol0003oooooooooo`08ooooo`009_ooool0
0ol0003oooooooooo`0kooooo`?o0?l00ol00?m]ooooo`Co003oooooool?ooooo`03o`000?oooooo
oooo00Sooooo000Vooooo`03o`000?oooooooooo03_ooooo0ol0o`03o`00ofgooooo1?l00?oooooo
o`oooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0>oooool3o`3o00?o
003oKOooool4o`00oooooooo3oooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooo
ooooo`0kooooo`?o0?l00ol00?m]ooooo`Co003oooooool?ooooo`03o`000?oooooooooo00Sooooo
000Vooooo`03o`000?oooooooooo03_ooooo0ol0o`03o`00ofgooooo1?l00?ooooooo`oooooo00?o
0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0>oooool3o`3o00?o003oKOooool4
o`00oooooooo3oooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`0koooo
o`?o0?l00ol00?m]ooooo`Co003oooooool?ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03
o`000?oooooooooo03_ooooo0ol0o`03o`00ofgooooo1?l00?ooooooo`oooooo00?o0000oooooooo
ool02?ooool002Kooooo0ol0000kooooo`?o0?l00ol00?m]ooooo`Co003oooooool>ooooo`;o0000
2_ooool002Kooooo00?o0000ooooooooool0>oooool3o`3o00?o003oKOooool4o`00oooooooo3ooo
ool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`0kooooo`?o0?l00ol00?oo
oooooh3ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0>oooool3o`3o
00?o003ooooooon0ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo03_o
oooo0ol0o`03o`00ooooooooP?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooo
ooooo`0kooooo`?o0?l00ol00?oooooooh3ooooo00?o0000ooooooooool02?ooool002Kooooo00?o
0000ooooooooool0>oooool3o`3o00?o003ooooooon0ooooo`03o`000?oooooooooo00Sooooo000V
ooooo`03o`000?oooooooooo03_ooooo0ol0o`03o`00ooooooooP?ooool00ol0003oooooooooo`08
ooooo`004Oooool2o`0000Cooooo00Co0000oooooooooooooooo1Ol00006ooooo`03o`000?oooooo
oooo03_ooooo0ol0o`03o`00ooooooooP?ooool00ol0003oooooooooo`08ooooo`004?ooool01?l0
003oooooooooool00008ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool0>ooo
ool3o`3o00?o003ooooooon0ooooo`03o`000?oooooooooo00Sooooo000@ooooo`04o`000?oooooo
ooooo`0000Wooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0kooooo`?o0?l00ol0
0?oooooooh3ooooo00?o0000ooooooooool02?ooool0013ooooo00Co0000oooooooooooo00002_oo
ool00ol0003oooooooooo`05ooooo`Co0000>_ooool3o`3o00?o003ooooooommooooo`Co00002_oo
ool0013ooooo00Co0000oooooooooooo00001oooool01Ol0003ooooooooooooooooo000000Kooooo
00?o0000ooooooooool0>oooool3o`3o00?o003ooooooon0ooooo`03o`000?oooooooooo00Sooooo
000@ooooo`04o`000?ooooooooooo`0000Oooooo00Go0000ooooooooooooooooo`000006ooooo`03
o`000?oooooooooo03_ooooo0ol0o`03o`00ooooooooP?ooool00ol0003oooooooooo`08ooooo`00
4Oooool2o`0000Wooooo0ol00007ooooo`03o`000?oooooooooo03_ooooo0ol0o`03o`00oooooooo
P?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`0kooooo`?o0?l00ol0
0?oooooooh3ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0>oooool3
o`3o00?o003ooooooon0ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo
03_ooooo0ol0o`03o`00ooooooooP?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003o
ooooooooo`0kooooo`?o0?l00ol00?oooooooh3ooooo00?o0000ooooooooool02?ooool002Kooooo
00?o0000ooooooooool0>oooool3o`3o00?o003ooooooon0ooooo`03o`000?oooooooooo00Sooooo
000Vooooo`03o`000?oooooooooo03_ooooo0ol0o`03o`00ooooooooP?ooool00ol0003ooooooooo
o`08ooooo`009_ooool3o`0003_ooooo0ol0o`03o`00ooooooooOoooool2o`0000[ooooo000Voooo
o`03o`000?oooooooooo03_ooooo0ol0o`03o`00ooooooooP?ooool00ol0003oooooooooo`08oooo
o`009_ooool00ol0003oooooooooo`0kooooo`?o0?l00ol00?oooooooh3ooooo00?o0000oooooooo
ool02?ooool002Kooooo00?o0000ooooooooool0>oooool3o`3o00?o003ooooooon0ooooo`03o`00
0?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo03_ooooo0ol0o`03o`00ooooooooP?oo
ool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`0kooooo`?o0?l00ol00?oo
oooooh3ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0>oooool3o`3o
00?o003ooooooon0ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo03_o
oooo0ol0o`03o`00ooooooooP?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooo
ooooo`0kooooo`?o0?l00ol00?oooooooh3ooooo00?o0000ooooooooool02?ooool002Kooooo00?o
0000ooooooooool0>oooool3o`3o00?o003ooooooon0ooooo`03o`000?oooooooooo00Sooooo000V
ooooo`?o0000>oooool3o`3o00?o003ooooooomoooooo`;o00002_ooool002Kooooo00?o0000oooo
ooooool0>oooool3o`3o00?o003ooooooon0ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03
o`000?oooooooooo03_ooooo0ol0o`03o`00ooooooooP?ooool00ol0003oooooooooo`08ooooo`00
9_ooool00ol0003oooooooooo`0kooooo`?o0?l00ol00?oooooooh3ooooo00?o0000ooooooooool0
2?ooool002Kooooo00?o0000ooooooooool0>oooool3o`3o00?o003ooooooon0ooooo`03o`000?oo
oooooooo00Sooooo000Vooooo`03o`000?oooooooooo03_ooooo0ol0o`03o`00ooooooooP?ooool0
0ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`0kooooo`?o0?l00ol00?oooooo
oh3ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0>oooool3o`3o00?o
003ooooooon0ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo03_ooooo
0ol0o`03o`00ooooooooP?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooooooo
o`0kooooo`?o0?l00ol00?oooooooh3ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000
ooooooooool0>oooool3o`3o00?o003ooooooon0ooooo`03o`000?oooooooooo00Sooooo000Voooo
o`?o0000>oooool3o`3o00?o003ooooooomoooooo`;o00002_ooool002Kooooo00?o0000oooooooo
ool0>oooool3o`3o00?o003ooooooon0ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`00
0?oooooooooo03_ooooo0ol0o`03o`00ooooooooP?ooool00ol0003oooooooooo`08ooooo`009_oo
ool00ol0003oooooooooo`0kooooo`?o0?l00ol00?oooooooh3ooooo00?o0000ooooooooool02?oo
ool002Kooooo00?o0000ooooooooool0>oooool3o`3o00?o003ooooooon0ooooo`03o`000?oooooo
oooo00Sooooo000Vooooo`03o`000?oooooooooo03_ooooo0ol0o`03o`00ooooooooP?ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`0kooooo`?o0?l00ol00?oooooooh3o
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0>oooool3o`3o00?o003o
oooooon0ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo03_ooooo0ol0
o`03o`00ooooooooP?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`0k
ooooo`?o0?l00ol00?oooooooh3ooooo00?o0000ooooooooool02?ooool002Kooooo0ol0000koooo
o`?o0?l00ol00?ooooooogoooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`0kooooo`?o
0?l00ol00?oooooooh3ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0
>oooool3o`3o00?o003ooooooon0ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oo
oooooooo03_ooooo0ol0o`03o`00ooooooooP?ooool00ol0003oooooooooo`08ooooo`009_ooool0
0ol0003oooooooooo`0kooooo`?o0?l00ol00?oooooooh3ooooo00?o0000ooooooooool02?ooool0
02Kooooo00?o0000ooooooooool0>oooool4o`3o0?ooooooP_ooool00ol0003oooooooooo`08oooo
o`009_ooool00ol0003oooooooooo`0kooooo`Co0?l0oooooon2ooooo`03o`000?oooooooooo00So
oooo000Vooooo`03o`000?oooooooooo03_ooooo1?l0o`3ooooooh;ooooo00?o0000ooooooooool0
2?ooool000_ooooo0_l00004ooooo`04o`000?ooooooooooooooo`Go00000_ooool3o`0000Oooooo
00?o0000ooooooooool0>oooool4o`3o0?ooooooP_ooool00ol0003oooooooooo`08ooooo`002_oo
ool01?l0003oooooooooool00008ooooo`06o`000?ooooooooooo`000?oooooo00000oooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo03_ooooo1?l0o`3ooooooh;ooooo00?o0000oooo
ooooool02?ooool000[ooooo00Co0000oooooooooooo00002Oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00Cooooo1?l0000jooooo`Co0?l0oooooomoooooo`Co00002_ooool000[o
oooo00Co0000oooooooooooo00002_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool0>oooool4o`3o0?ooooooP_ooool00ol0003oooooooooo`08oooo
o`002_ooool01?l0003oooooooooool00007ooooo`06o`000?ooooooooooooooool0003ooooo1?l0
0007ooooo`03o`000?oooooooooo03_ooooo1?l0o`3ooooooh;ooooo00?o0000ooooooooool02?oo
ool000[ooooo00Co0000oooooooooooo00001oooool01ol0003ooooooooooooooooo0000ooooool0
00002_ooool00ol0003oooooooooo`0kooooo`Co0?l0oooooon2ooooo`03o`000?oooooooooo00So
oooo000;ooooo`;o00002Oooool3o`0000;ooooo1Ol00006ooooo`03o`000?oooooooooo03_ooooo
1?l0o`3ooooooh;ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0>ooo
ool4o`3o0?ooooooP_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`0k
ooooo`Co0?l0oooooon2ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo
03_ooooo1?l0o`3ooooooh;ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooooooo
ool0ooooooo1ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo0?oooooo
`Oooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`3ooooool7ooooo00?o
0000ooooooooool02?ooool002Kooooo0ol00006ooooo`03o`000?oooooooooo00_ooooo00?o0000
ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000
ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000
ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000
ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000
ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000
ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000
ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000
ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000
ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000
ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000
ooooooooool01?ooool2o`0000[ooooo000Vooooo`03o`000?oooooooooo00Kooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03
o`000?oooooooooo00Kooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`05ooooo`03
o`000?oooooooooo00Sooooo000Voooooooo0000aOl0000:ooooo`00oooooooeooooo`00oooooooe
ooooo`00oooooooeooooo`00oooooooeooooo`00oooooooeooooo`00oooooooeooooo`00oooooooe
ooooo`00;Oooool3o`0000[ooooo1Ol0000:ooooo`?o00003?ooool3o`0000[ooooo0ol0000;oooo
o`?o00003?ooool00ol0003oooooooooo`0:ooooo`?o00002oooool2o`0000Wooooo0ol00004oooo
o`;o00001Oooool3o`0000?ooooo0ol00005ooooo`?o00000_ooool5o`0000Cooooo0ol00003oooo
o`?o00001Oooool3o`0000Cooooo0ol00004ooooo`?o00000oooool3o`0000Gooooo0ol00003oooo
o`?o00001Oooool3o`0000Cooooo00?o0000ooooooooool01?ooool3o`0000?ooooo0ol00005oooo
o`?o00000oooool2o`0000Gooooo1Ol00003ooooo`;o00001?ooool5o`0000;ooooo0ol00004oooo
o`Go000000?oooooo`000?l000000ol00003ooooo`Go00000_ooool3o`0000Cooooo1Ol00003oooo
o`?o00000oooool5o`0000;ooooo0ol00004ooooo`Go00000_ooool3o`0000Cooooo1Ol00003oooo
o`03o`000?oooooooooo00?ooooo1Ol00002ooooo`?o00001?ooool5o`0000;ooooo0_l00006oooo
o`?o00001?ooool2o`0000Gooooo0ol00003ooooo`?o00001Oooool3o`0000;ooooo1Ol0000>oooo
o`00;_ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000Wooooo00Go0000oooo
ooooooooooooo`00000<ooooo`03o`000?oooooooooo00Sooooo00Go0000ooooooooooooooooo`00
0009ooooo`05o`000?ooooooooooooooool000002oooool00ol0003oooooooooo`09ooooo`05o`00
0?ooooooooooooooool000003?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00;o
oooo00Co0000oooooooooooo00001Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`05o`000?oooooo
ooooooooool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01?ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool0
0oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`03oooo
o`04o`000?ooooooooooo`0000;ooooo00Co0000oooooooooooo00001?ooool01?l0003ooooooooo
ool00003ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooo00000_ooool01?l0003o
ooooooooool00004ooooo`06o`000?ooooooooooo`000?oooooo00000oooool00ol0003ooooooooo
o`02ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool00oooool01_l0003ooooo
ooooool0003oooooo`0000?ooooo00?o0000ooooooooool00_ooool01_l0003oooooooooool0003o
ooooo`0000?ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00003ooooo`03o`00
0?oooooooooo00Cooooo00Ko0000oooooooooooo0000ooooool00003ooooo`03o`000?oooooooooo
00;ooooo00Co0000oooooooooooo00001?ooool00ol0003oooooooooo`02ooooo`05o`000?oooooo
ooooooooool000000_ooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000oooool00ol0003oooooooooo`03ooooo`05o`000?ooooooooooooooool000000_ooool01?l0
003oooooooooool0000>ooooo`00;_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo
00gooooo00?o0000ooooooooool01oooool5o`0000gooooo00?o0000ooooooooool01oooool01Ol0
003ooooooooooooooooo000000cooooo00?o0000ooooooooool02?ooool01Ol0003ooooooooooooo
oooo000000gooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`02ooooo`04o`000?oo
ooooooooo`0000Gooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00?ooooo00Co0000oooooooooooooooo1Ol00005ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00Go0000oooo
ooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00001Ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00ooo
ool01?l0003oooooooooooooool5o`0000Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`03ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooooooooooo0000
00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`05ooooo`05o`000?oooooo
ooooooooool000002Oooool01Ol0003ooooooooooooooooo0000013ooooo000^ooooo`03o`000?oo
oooooooo00cooooo00?o0000ooooooooool02_ooool2o`0000[ooooo00Co0000oooooooooooo0000
3_ooool00ol0003oooooooooo`07ooooo`Co00003Oooool00ol0003oooooooooo`09ooooo`?o0000
2oooool4o`0000Sooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`03
o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool00oooool2o`0000Kooooo00Go0000oooooooo
ooooooooo`000002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`03ooooo`04o`000?ooooooooooooooo`Co00001_ooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool3o`0000Kooooo00?o0000
ooooooooool00_ooool4o`0000Kooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`06ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`02ooooo`;o00001ooo
ool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Co00001oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00Gooooo00Co0000oooooooooooooooo0ol00007ooooo`04
o`000?ooooooooooooooo`Co00001Oooool2o`0000?ooooo00Co0000oooooooooooo00001Oooool2
o`0000Cooooo00?o0000ooooooooool01Oooool2o`0000Gooooo00?o0000ooooooooool03Oooool0
02kooooo00?o0000ooooooooool02Oooool01Ol0003ooooooooooooooooo000000gooooo00?o0000
ooooooooool02?ooool00ol0003oooooo`00000:ooooo`Co00002_ooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00Go0000ooooooooooooooooo`000009ooooo`05o`000?oo
ooooooooooooool000002?ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Go
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`05o`000?oooooooooooooo
ool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Kooooo00Co0000oooo
oooooooooooo1?l00006ooooo`05o`000?ooooooooooooooool000002Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooool00002ooooo`05o`000?ooooooooooooooool000000ooo
ool01Ol0003ooooooooooooooooo000000Gooooo00Oo0000ooooooooooooooooo`000?oooooo0000
00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00Go0000
ooooooooooooooooo`000003ooooo`06o`000?ooooooooooo`000?oooooo00001?ooool01_l0003o
oooooooooooooooo0000ooooo`Co00001?ooool01ol0003ooooooooooooooooo0000ooooool00000
1oooool01Ol0003ooooooooooooooooo000000Gooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo
000000?ooooo00?o0000ooooooooool01Oooool01?l0003oooooooooool00002ooooo`03o`000?oo
oooooooo00Gooooo00Go0000ooooooooooooooooo`000009ooooo`03o`000?oooooo000000?ooooo
00?o0000ooooooooool03?ooool002gooooo0_l0000;ooooo`05o`000?ooooooooooooooool00000
2Oooool01Ol0003ooooooooooooooooo000000_ooooo0_l0000:ooooo`03o`000?oooooooooo00co
oooo00?o0000ooooooooool02_ooool01Ol0003ooooooooooooooooo000000Wooooo00Go0000oooo
ooooooooooooo`000009ooooo`05o`000?ooooooooooooooool000001oooool2o`0000Cooooo00Co
0000oooooooooooo00001?ooool2o`0000Cooooo0_l00006ooooo`;o00000oooool01Ol0003ooooo
oooooooooooo000000Cooooo0_l00003ooooo`05o`000?ooooooooooooooool000001?ooool2o`00
00Gooooo0_l00005ooooo`;o00000oooool00ol0003oooooooooo`06ooooo`;o00001?ooool00ol0
003oooooooooo`05ooooo`;o00000oooool01Ol0003ooooooooooooooooo000000Cooooo0_l00003
ooooo`05o`000?ooooooooooooooool000001?ooool2o`0000?ooooo00Go0000oooooooooooooooo
o`000003ooooo`05o`000?ooooooooooooooool000000_ooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool2o`0000Gooooo00Oo0000ooooooooooooooooo`00
0?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo000000?o
oooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooooooo`;o00001?ooool0
1ol0003ooooooooooooooooo0000ooooool000001oooool01Ol0003ooooooooooooooooo000000;o
oooo00?o0000ooooooooool01?ooool01ol0003ooooooooooooooooo0000ooooool000000oooool0
1Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooool000000oooool01Ol0003ooooooooo
oooooooo000000?ooooo00?o0000ooooool000000oooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000_ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000
ooooooooool00_l00005ooooo`07o`000?ooooooooooooooool0003oooooo`000003ooooo`03o`00
0?oooooooooo00cooooo000^ooooo`03o`000?oooooooooo00[ooooo0ol0000;ooooo`?o00003Ooo
ool00ol0003oooooooooo`08ooooo`Go00002oooool2o`0000[ooooo1Ol0000:ooooo`?o00002ooo
ool3o`0000Wooooo00?o0000ooooooooool00oooool2o`0000Kooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;ooooo0ol00006ooooo`03o`000?oo
oooooooo00;ooooo0ol00006ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?oo
ool01?l0003oooooooooooooool5o`0000Gooooo00?o0000ooooooooool00oooool2o`0000Kooooo
00Co0000oooooooooooooooo1Ol00005ooooo`03o`000?oooooooooo00;ooooo0ol00006ooooo`03
o`000?oooooooooo00;ooooo0ol00005ooooo`?o00001?ooool2o`0000Gooooo0ol00004ooooo`03
o`000?oooooooooo00Cooooo0ol00003ooooo`?o00001Oooool3o`0000?ooooo0ol00005ooooo`?o
00001Oooool00ol0003oooooooooo`03ooooo`?o00000_ooool5o`0000Cooooo0ol00004ooooo`;o
00001Oooool3o`0000;ooooo1Ol00004ooooo`?o00000oooool3o`0000Gooooo0ol00003ooooo`?o
00001Oooool3o`0000Cooooo0_l00005ooooo`?o00001?ooool00ol0003oooooooooo`04ooooo`?o
00000oooool3o`0000oooooo0000\
\>"],
  ImageRangeCache->{{{0, 499}, {308, 0}} -> {-2.06768, -0.0205642, 0.0713992, \
0.000966586}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
%%ImageSize: 500 309.017 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0217391 0.0310559 0.0147151 1.61554 [
[.01863 -0.0125 -3 -9 ]
[.01863 -0.0125 3 0 ]
[.04969 -0.0125 -3 -9 ]
[.04969 -0.0125 3 0 ]
[.08075 -0.0125 -3 -9 ]
[.08075 -0.0125 3 0 ]
[.1118 -0.0125 -3 -9 ]
[.1118 -0.0125 3 0 ]
[.14286 -0.0125 -3 -9 ]
[.14286 -0.0125 3 0 ]
[.17391 -0.0125 -3 -9 ]
[.17391 -0.0125 3 0 ]
[.20497 -0.0125 -3 -9 ]
[.20497 -0.0125 3 0 ]
[.23602 -0.0125 -3 -9 ]
[.23602 -0.0125 3 0 ]
[.26708 -0.0125 -3 -9 ]
[.26708 -0.0125 3 0 ]
[.29814 -0.0125 -6 -9 ]
[.29814 -0.0125 6 0 ]
[.32919 -0.0125 -6 -9 ]
[.32919 -0.0125 6 0 ]
[.36025 -0.0125 -6 -9 ]
[.36025 -0.0125 6 0 ]
[.3913 -0.0125 -6 -9 ]
[.3913 -0.0125 6 0 ]
[.42236 -0.0125 -6 -9 ]
[.42236 -0.0125 6 0 ]
[.45342 -0.0125 -6 -9 ]
[.45342 -0.0125 6 0 ]
[.48447 -0.0125 -6 -9 ]
[.48447 -0.0125 6 0 ]
[.51553 -0.0125 -6 -9 ]
[.51553 -0.0125 6 0 ]
[.54658 -0.0125 -6 -9 ]
[.54658 -0.0125 6 0 ]
[.57764 -0.0125 -6 -9 ]
[.57764 -0.0125 6 0 ]
[.6087 -0.0125 -6 -9 ]
[.6087 -0.0125 6 0 ]
[.63975 -0.0125 -6 -9 ]
[.63975 -0.0125 6 0 ]
[.67081 -0.0125 -6 -9 ]
[.67081 -0.0125 6 0 ]
[.70186 -0.0125 -6 -9 ]
[.70186 -0.0125 6 0 ]
[.73292 -0.0125 -6 -9 ]
[.73292 -0.0125 6 0 ]
[.76398 -0.0125 -6 -9 ]
[.76398 -0.0125 6 0 ]
[.79503 -0.0125 -6 -9 ]
[.79503 -0.0125 6 0 ]
[.82609 -0.0125 -6 -9 ]
[.82609 -0.0125 6 0 ]
[.85714 -0.0125 -6 -9 ]
[.85714 -0.0125 6 0 ]
[.8882 -0.0125 -6 -9 ]
[.8882 -0.0125 6 0 ]
[.91925 -0.0125 -6 -9 ]
[.91925 -0.0125 6 0 ]
[.95031 -0.0125 -6 -9 ]
[.95031 -0.0125 6 0 ]
[.98137 -0.0125 -6 -9 ]
[.98137 -0.0125 6 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .01472 -6 -4.5 ]
[-0.0125 .01472 0 4.5 ]
[-0.0125 .09549 -24 -4.5 ]
[-0.0125 .09549 0 4.5 ]
[-0.0125 .17627 -18 -4.5 ]
[-0.0125 .17627 0 4.5 ]
[-0.0125 .25705 -24 -4.5 ]
[-0.0125 .25705 0 4.5 ]
[-0.0125 .33782 -18 -4.5 ]
[-0.0125 .33782 0 4.5 ]
[-0.0125 .4186 -24 -4.5 ]
[-0.0125 .4186 0 4.5 ]
[-0.0125 .49938 -18 -4.5 ]
[-0.0125 .49938 0 4.5 ]
[-0.0125 .58015 -24 -4.5 ]
[-0.0125 .58015 0 4.5 ]
[ 0 0 -0.125 0 ]
[.01863 .63053 -3 0 ]
[.01863 .63053 3 9 ]
[.04969 .63053 -3 0 ]
[.04969 .63053 3 9 ]
[.08075 .63053 -3 0 ]
[.08075 .63053 3 9 ]
[.1118 .63053 -3 0 ]
[.1118 .63053 3 9 ]
[.14286 .63053 -3 0 ]
[.14286 .63053 3 9 ]
[.17391 .63053 -3 0 ]
[.17391 .63053 3 9 ]
[.20497 .63053 -3 0 ]
[.20497 .63053 3 9 ]
[.23602 .63053 -3 0 ]
[.23602 .63053 3 9 ]
[.26708 .63053 -3 0 ]
[.26708 .63053 3 9 ]
[.29814 .63053 -6 0 ]
[.29814 .63053 6 9 ]
[.32919 .63053 -6 0 ]
[.32919 .63053 6 9 ]
[.36025 .63053 -6 0 ]
[.36025 .63053 6 9 ]
[.3913 .63053 -6 0 ]
[.3913 .63053 6 9 ]
[.42236 .63053 -6 0 ]
[.42236 .63053 6 9 ]
[.45342 .63053 -6 0 ]
[.45342 .63053 6 9 ]
[.48447 .63053 -6 0 ]
[.48447 .63053 6 9 ]
[.51553 .63053 -6 0 ]
[.51553 .63053 6 9 ]
[.54658 .63053 -6 0 ]
[.54658 .63053 6 9 ]
[.57764 .63053 -6 0 ]
[.57764 .63053 6 9 ]
[.6087 .63053 -6 0 ]
[.6087 .63053 6 9 ]
[.63975 .63053 -6 0 ]
[.63975 .63053 6 9 ]
[.67081 .63053 -6 0 ]
[.67081 .63053 6 9 ]
[.70186 .63053 -6 0 ]
[.70186 .63053 6 9 ]
[.73292 .63053 -6 0 ]
[.73292 .63053 6 9 ]
[.76398 .63053 -6 0 ]
[.76398 .63053 6 9 ]
[.79503 .63053 -6 0 ]
[.79503 .63053 6 9 ]
[.82609 .63053 -6 0 ]
[.82609 .63053 6 9 ]
[.85714 .63053 -6 0 ]
[.85714 .63053 6 9 ]
[.8882 .63053 -6 0 ]
[.8882 .63053 6 9 ]
[.91925 .63053 -6 0 ]
[.91925 .63053 6 9 ]
[.95031 .63053 -6 0 ]
[.95031 .63053 6 9 ]
[.98137 .63053 -6 0 ]
[.98137 .63053 6 9 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.01863 0 m
.01863 .00625 L
s
[(1)] .01863 -0.0125 0 1 Mshowa
.04969 0 m
.04969 .00625 L
s
[(2)] .04969 -0.0125 0 1 Mshowa
.08075 0 m
.08075 .00625 L
s
[(3)] .08075 -0.0125 0 1 Mshowa
.1118 0 m
.1118 .00625 L
s
[(4)] .1118 -0.0125 0 1 Mshowa
.14286 0 m
.14286 .00625 L
s
[(5)] .14286 -0.0125 0 1 Mshowa
.17391 0 m
.17391 .00625 L
s
[(6)] .17391 -0.0125 0 1 Mshowa
.20497 0 m
.20497 .00625 L
s
[(7)] .20497 -0.0125 0 1 Mshowa
.23602 0 m
.23602 .00625 L
s
[(8)] .23602 -0.0125 0 1 Mshowa
.26708 0 m
.26708 .00625 L
s
[(9)] .26708 -0.0125 0 1 Mshowa
.29814 0 m
.29814 .00625 L
s
[(10)] .29814 -0.0125 0 1 Mshowa
.32919 0 m
.32919 .00625 L
s
[(11)] .32919 -0.0125 0 1 Mshowa
.36025 0 m
.36025 .00625 L
s
[(12)] .36025 -0.0125 0 1 Mshowa
.3913 0 m
.3913 .00625 L
s
[(13)] .3913 -0.0125 0 1 Mshowa
.42236 0 m
.42236 .00625 L
s
[(14)] .42236 -0.0125 0 1 Mshowa
.45342 0 m
.45342 .00625 L
s
[(15)] .45342 -0.0125 0 1 Mshowa
.48447 0 m
.48447 .00625 L
s
[(16)] .48447 -0.0125 0 1 Mshowa
.51553 0 m
.51553 .00625 L
s
[(17)] .51553 -0.0125 0 1 Mshowa
.54658 0 m
.54658 .00625 L
s
[(18)] .54658 -0.0125 0 1 Mshowa
.57764 0 m
.57764 .00625 L
s
[(19)] .57764 -0.0125 0 1 Mshowa
.6087 0 m
.6087 .00625 L
s
[(20)] .6087 -0.0125 0 1 Mshowa
.63975 0 m
.63975 .00625 L
s
[(21)] .63975 -0.0125 0 1 Mshowa
.67081 0 m
.67081 .00625 L
s
[(22)] .67081 -0.0125 0 1 Mshowa
.70186 0 m
.70186 .00625 L
s
[(23)] .70186 -0.0125 0 1 Mshowa
.73292 0 m
.73292 .00625 L
s
[(24)] .73292 -0.0125 0 1 Mshowa
.76398 0 m
.76398 .00625 L
s
[(25)] .76398 -0.0125 0 1 Mshowa
.79503 0 m
.79503 .00625 L
s
[(26)] .79503 -0.0125 0 1 Mshowa
.82609 0 m
.82609 .00625 L
s
[(27)] .82609 -0.0125 0 1 Mshowa
.85714 0 m
.85714 .00625 L
s
[(28)] .85714 -0.0125 0 1 Mshowa
.8882 0 m
.8882 .00625 L
s
[(29)] .8882 -0.0125 0 1 Mshowa
.91925 0 m
.91925 .00625 L
s
[(30)] .91925 -0.0125 0 1 Mshowa
.95031 0 m
.95031 .00625 L
s
[(31)] .95031 -0.0125 0 1 Mshowa
.98137 0 m
.98137 .00625 L
s
[(32)] .98137 -0.0125 0 1 Mshowa
0 0 m
1 0 L
s
0 .01472 m
.00625 .01472 L
s
[(0)] -0.0125 .01472 1 0 Mshowa
0 .09549 m
.00625 .09549 L
s
[(0.05)] -0.0125 .09549 1 0 Mshowa
0 .17627 m
.00625 .17627 L
s
[(0.1)] -0.0125 .17627 1 0 Mshowa
0 .25705 m
.00625 .25705 L
s
[(0.15)] -0.0125 .25705 1 0 Mshowa
0 .33782 m
.00625 .33782 L
s
[(0.2)] -0.0125 .33782 1 0 Mshowa
0 .4186 m
.00625 .4186 L
s
[(0.25)] -0.0125 .4186 1 0 Mshowa
0 .49938 m
.00625 .49938 L
s
[(0.3)] -0.0125 .49938 1 0 Mshowa
0 .58015 m
.00625 .58015 L
s
[(0.35)] -0.0125 .58015 1 0 Mshowa
.125 Mabswid
0 .03087 m
.00375 .03087 L
s
0 .04703 m
.00375 .04703 L
s
0 .06318 m
.00375 .06318 L
s
0 .07934 m
.00375 .07934 L
s
0 .11165 m
.00375 .11165 L
s
0 .1278 m
.00375 .1278 L
s
0 .14396 m
.00375 .14396 L
s
0 .16011 m
.00375 .16011 L
s
0 .19242 m
.00375 .19242 L
s
0 .20858 m
.00375 .20858 L
s
0 .22474 m
.00375 .22474 L
s
0 .24089 m
.00375 .24089 L
s
0 .2732 m
.00375 .2732 L
s
0 .28936 m
.00375 .28936 L
s
0 .30551 m
.00375 .30551 L
s
0 .32167 m
.00375 .32167 L
s
0 .35398 m
.00375 .35398 L
s
0 .37013 m
.00375 .37013 L
s
0 .38629 m
.00375 .38629 L
s
0 .40245 m
.00375 .40245 L
s
0 .43476 m
.00375 .43476 L
s
0 .45091 m
.00375 .45091 L
s
0 .46707 m
.00375 .46707 L
s
0 .48322 m
.00375 .48322 L
s
0 .51553 m
.00375 .51553 L
s
0 .53169 m
.00375 .53169 L
s
0 .54784 m
.00375 .54784 L
s
0 .564 m
.00375 .564 L
s
0 .59631 m
.00375 .59631 L
s
0 .61247 m
.00375 .61247 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.01863 .61178 m
.01863 .61803 L
s
[(1)] .01863 .63053 0 -1 Mshowa
.04969 .61178 m
.04969 .61803 L
s
[(2)] .04969 .63053 0 -1 Mshowa
.08075 .61178 m
.08075 .61803 L
s
[(3)] .08075 .63053 0 -1 Mshowa
.1118 .61178 m
.1118 .61803 L
s
[(4)] .1118 .63053 0 -1 Mshowa
.14286 .61178 m
.14286 .61803 L
s
[(5)] .14286 .63053 0 -1 Mshowa
.17391 .61178 m
.17391 .61803 L
s
[(6)] .17391 .63053 0 -1 Mshowa
.20497 .61178 m
.20497 .61803 L
s
[(7)] .20497 .63053 0 -1 Mshowa
.23602 .61178 m
.23602 .61803 L
s
[(8)] .23602 .63053 0 -1 Mshowa
.26708 .61178 m
.26708 .61803 L
s
[(9)] .26708 .63053 0 -1 Mshowa
.29814 .61178 m
.29814 .61803 L
s
[(10)] .29814 .63053 0 -1 Mshowa
.32919 .61178 m
.32919 .61803 L
s
[(11)] .32919 .63053 0 -1 Mshowa
.36025 .61178 m
.36025 .61803 L
s
[(12)] .36025 .63053 0 -1 Mshowa
.3913 .61178 m
.3913 .61803 L
s
[(13)] .3913 .63053 0 -1 Mshowa
.42236 .61178 m
.42236 .61803 L
s
[(14)] .42236 .63053 0 -1 Mshowa
.45342 .61178 m
.45342 .61803 L
s
[(15)] .45342 .63053 0 -1 Mshowa
.48447 .61178 m
.48447 .61803 L
s
[(16)] .48447 .63053 0 -1 Mshowa
.51553 .61178 m
.51553 .61803 L
s
[(17)] .51553 .63053 0 -1 Mshowa
.54658 .61178 m
.54658 .61803 L
s
[(18)] .54658 .63053 0 -1 Mshowa
.57764 .61178 m
.57764 .61803 L
s
[(19)] .57764 .63053 0 -1 Mshowa
.6087 .61178 m
.6087 .61803 L
s
[(20)] .6087 .63053 0 -1 Mshowa
.63975 .61178 m
.63975 .61803 L
s
[(21)] .63975 .63053 0 -1 Mshowa
.67081 .61178 m
.67081 .61803 L
s
[(22)] .67081 .63053 0 -1 Mshowa
.70186 .61178 m
.70186 .61803 L
s
[(23)] .70186 .63053 0 -1 Mshowa
.73292 .61178 m
.73292 .61803 L
s
[(24)] .73292 .63053 0 -1 Mshowa
.76398 .61178 m
.76398 .61803 L
s
[(25)] .76398 .63053 0 -1 Mshowa
.79503 .61178 m
.79503 .61803 L
s
[(26)] .79503 .63053 0 -1 Mshowa
.82609 .61178 m
.82609 .61803 L
s
[(27)] .82609 .63053 0 -1 Mshowa
.85714 .61178 m
.85714 .61803 L
s
[(28)] .85714 .63053 0 -1 Mshowa
.8882 .61178 m
.8882 .61803 L
s
[(29)] .8882 .63053 0 -1 Mshowa
.91925 .61178 m
.91925 .61803 L
s
[(30)] .91925 .63053 0 -1 Mshowa
.95031 .61178 m
.95031 .61803 L
s
[(31)] .95031 .63053 0 -1 Mshowa
.98137 .61178 m
.98137 .61803 L
s
[(32)] .98137 .63053 0 -1 Mshowa
0 .61803 m
1 .61803 L
s
.99375 .01472 m
1 .01472 L
s
.99375 .09549 m
1 .09549 L
s
.99375 .17627 m
1 .17627 L
s
.99375 .25705 m
1 .25705 L
s
.99375 .33782 m
1 .33782 L
s
.99375 .4186 m
1 .4186 L
s
.99375 .49938 m
1 .49938 L
s
.99375 .58015 m
1 .58015 L
s
.125 Mabswid
.99625 .03087 m
1 .03087 L
s
.99625 .04703 m
1 .04703 L
s
.99625 .06318 m
1 .06318 L
s
.99625 .07934 m
1 .07934 L
s
.99625 .11165 m
1 .11165 L
s
.99625 .1278 m
1 .1278 L
s
.99625 .14396 m
1 .14396 L
s
.99625 .16011 m
1 .16011 L
s
.99625 .19242 m
1 .19242 L
s
.99625 .20858 m
1 .20858 L
s
.99625 .22474 m
1 .22474 L
s
.99625 .24089 m
1 .24089 L
s
.99625 .2732 m
1 .2732 L
s
.99625 .28936 m
1 .28936 L
s
.99625 .30551 m
1 .30551 L
s
.99625 .32167 m
1 .32167 L
s
.99625 .35398 m
1 .35398 L
s
.99625 .37013 m
1 .37013 L
s
.99625 .38629 m
1 .38629 L
s
.99625 .40245 m
1 .40245 L
s
.99625 .43476 m
1 .43476 L
s
.99625 .45091 m
1 .45091 L
s
.99625 .46707 m
1 .46707 L
s
.99625 .48322 m
1 .48322 L
s
.99625 .51553 m
1 .51553 L
s
.99625 .53169 m
1 .53169 L
s
.99625 .54784 m
1 .54784 L
s
.99625 .564 m
1 .564 L
s
.99625 .59631 m
1 .59631 L
s
.99625 .61247 m
1 .61247 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .01472 m
1 .01472 L
s
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.00621 .01472 m
.00621 .07312 L
.01242 .07312 L
.01242 .01472 L
F
.5 Mabswid
.00621 .01472 m
.00621 .07312 L
.01242 .07312 L
.01242 .01472 L
.00621 .01472 L
s
.03727 .01472 m
.03727 .03343 L
.04348 .03343 L
.04348 .01472 L
F
.03727 .01472 m
.03727 .03343 L
.04348 .03343 L
.04348 .01472 L
.03727 .01472 L
s
.06832 .01472 m
.06832 .1457 L
.07453 .1457 L
.07453 .01472 L
F
.06832 .01472 m
.06832 .1457 L
.07453 .1457 L
.07453 .01472 L
.06832 .01472 L
s
.09938 .01472 m
.09938 .01472 L
.10559 .01472 L
.10559 .01472 L
F
.09938 .01472 m
.09938 .01472 L
.10559 .01472 L
.10559 .01472 L
.09938 .01472 L
s
.13043 .01472 m
.13043 .04704 L
.13665 .04704 L
.13665 .01472 L
F
.13043 .01472 m
.13043 .04704 L
.13665 .04704 L
.13665 .01472 L
.13043 .01472 L
s
.16149 .01472 m
.16149 .01982 L
.1677 .01982 L
.1677 .01472 L
F
.16149 .01472 m
.16149 .01982 L
.1677 .01982 L
.1677 .01472 L
.16149 .01472 L
s
.19255 .01472 m
.19255 .01755 L
.19876 .01755 L
.19876 .01472 L
F
.19255 .01472 m
.19255 .01755 L
.19876 .01755 L
.19876 .01472 L
.19255 .01472 L
s
.2236 .01472 m
.2236 .01472 L
.22981 .01472 L
.22981 .01472 L
F
.2236 .01472 m
.2236 .01472 L
.22981 .01472 L
.22981 .01472 L
.2236 .01472 L
s
.25466 .01472 m
.25466 .03399 L
.26087 .03399 L
.26087 .01472 L
F
.25466 .01472 m
.25466 .03399 L
.26087 .03399 L
.26087 .01472 L
.25466 .01472 L
s
.28571 .01472 m
.28571 .14854 L
.29193 .14854 L
.29193 .01472 L
F
.28571 .01472 m
.28571 .14854 L
.29193 .14854 L
.29193 .01472 L
.28571 .01472 L
s
.31677 .01472 m
.31677 .14854 L
.32298 .14854 L
.32298 .01472 L
F
.31677 .01472 m
.31677 .14854 L
.32298 .14854 L
.32298 .01472 L
.31677 .01472 L
s
.34783 .01472 m
.34783 .01472 L
.35404 .01472 L
.35404 .01472 L
F
.34783 .01472 m
.34783 .01472 L
.35404 .01472 L
.35404 .01472 L
.34783 .01472 L
s
.37888 .01472 m
.37888 .22679 L
.38509 .22679 L
.38509 .01472 L
F
.37888 .01472 m
.37888 .22679 L
.38509 .22679 L
.38509 .01472 L
.37888 .01472 L
s
.40994 .01472 m
.40994 .01472 L
.41615 .01472 L
.41615 .01472 L
F
.40994 .01472 m
.40994 .01472 L
.41615 .01472 L
.41615 .01472 L
.40994 .01472 L
s
.44099 .01472 m
.44099 .01585 L
.4472 .01585 L
.4472 .01472 L
F
.44099 .01472 m
.44099 .01585 L
.4472 .01585 L
.4472 .01472 L
.44099 .01472 L
s
.47205 .01472 m
.47205 .02039 L
.47826 .02039 L
.47826 .01472 L
F
.47205 .01472 m
.47205 .02039 L
.47826 .02039 L
.47826 .01472 L
.47205 .01472 L
s
.50311 .01472 m
.50311 .16555 L
.50932 .16555 L
.50932 .01472 L
F
.50311 .01472 m
.50311 .16555 L
.50932 .16555 L
.50932 .01472 L
.50311 .01472 L
s
.53416 .01472 m
.53416 .01528 L
.54037 .01528 L
.54037 .01472 L
F
.53416 .01472 m
.53416 .01528 L
.54037 .01528 L
.54037 .01472 L
.53416 .01472 L
s
.56522 .01472 m
.56522 .22226 L
.57143 .22226 L
.57143 .01472 L
F
.56522 .01472 m
.56522 .22226 L
.57143 .22226 L
.57143 .01472 L
.56522 .01472 L
s
.59627 .01472 m
.59627 .01528 L
.60248 .01528 L
.60248 .01472 L
F
.59627 .01472 m
.59627 .01528 L
.60248 .01528 L
.60248 .01472 L
.59627 .01472 L
s
.62733 .01472 m
.62733 .01868 L
.63354 .01868 L
.63354 .01472 L
F
.62733 .01472 m
.62733 .01868 L
.63354 .01868 L
.63354 .01472 L
.62733 .01472 L
s
.65839 .01472 m
.65839 .05781 L
.6646 .05781 L
.6646 .01472 L
F
.65839 .01472 m
.65839 .05781 L
.6646 .05781 L
.6646 .01472 L
.65839 .01472 L
s
.68944 .01472 m
.68944 .01755 L
.69565 .01755 L
.69565 .01472 L
F
.68944 .01472 m
.68944 .01755 L
.69565 .01755 L
.69565 .01472 L
.68944 .01472 L
s
.7205 .01472 m
.7205 .01472 L
.72671 .01472 L
.72671 .01472 L
F
.7205 .01472 m
.7205 .01472 L
.72671 .01472 L
.72671 .01472 L
.7205 .01472 L
s
.75155 .01472 m
.75155 .09807 L
.75776 .09807 L
.75776 .01472 L
F
.75155 .01472 m
.75155 .09807 L
.75776 .09807 L
.75776 .01472 L
.75155 .01472 L
s
.78261 .01472 m
.78261 .07482 L
.78882 .07482 L
.78882 .01472 L
F
.78261 .01472 m
.78261 .07482 L
.78882 .07482 L
.78882 .01472 L
.78261 .01472 L
s
.81366 .01472 m
.81366 .13947 L
.81988 .13947 L
.81988 .01472 L
F
.81366 .01472 m
.81366 .13947 L
.81988 .13947 L
.81988 .01472 L
.81366 .01472 L
s
.84472 .01472 m
.84472 .01528 L
.85093 .01528 L
.85093 .01472 L
F
.84472 .01472 m
.84472 .01528 L
.85093 .01528 L
.85093 .01472 L
.84472 .01472 L
s
.87578 .01472 m
.87578 .1871 L
.88199 .1871 L
.88199 .01472 L
F
.87578 .01472 m
.87578 .1871 L
.88199 .1871 L
.88199 .01472 L
.87578 .01472 L
s
.90683 .01472 m
.90683 .01585 L
.91304 .01585 L
.91304 .01472 L
F
.90683 .01472 m
.90683 .01585 L
.91304 .01585 L
.91304 .01472 L
.90683 .01472 L
s
.93789 .01472 m
.93789 .01472 L
.9441 .01472 L
.9441 .01472 L
F
.93789 .01472 m
.93789 .01472 L
.9441 .01472 L
.9441 .01472 L
.93789 .01472 L
s
.96894 .01472 m
.96894 .02436 L
.97516 .02436 L
.97516 .01472 L
F
.96894 .01472 m
.96894 .02436 L
.97516 .02436 L
.97516 .01472 L
.96894 .01472 L
s
0 1 0 r
.01242 .01472 m
.01242 .25401 L
.01863 .25401 L
.01863 .01472 L
F
.01242 .01472 m
.01242 .25401 L
.01863 .25401 L
.01863 .01472 L
.01242 .01472 L
s
.04348 .01472 m
.04348 .06235 L
.04969 .06235 L
.04969 .01472 L
F
.04348 .01472 m
.04348 .06235 L
.04969 .06235 L
.04969 .01472 L
.04348 .01472 L
s
.07453 .01472 m
.07453 .05895 L
.08075 .05895 L
.08075 .01472 L
F
.07453 .01472 m
.07453 .05895 L
.08075 .05895 L
.08075 .01472 L
.07453 .01472 L
s
.10559 .01472 m
.10559 .01472 L
.1118 .01472 L
.1118 .01472 L
F
.10559 .01472 m
.10559 .01472 L
.1118 .01472 L
.1118 .01472 L
.10559 .01472 L
s
.13665 .01472 m
.13665 .02322 L
.14286 .02322 L
.14286 .01472 L
F
.13665 .01472 m
.13665 .02322 L
.14286 .02322 L
.14286 .01472 L
.13665 .01472 L
s
.1677 .01472 m
.1677 .01585 L
.17391 .01585 L
.17391 .01472 L
F
.1677 .01472 m
.1677 .01585 L
.17391 .01585 L
.17391 .01472 L
.1677 .01472 L
s
.19876 .01472 m
.19876 .01472 L
.20497 .01472 L
.20497 .01472 L
F
.19876 .01472 m
.19876 .01472 L
.20497 .01472 L
.20497 .01472 L
.19876 .01472 L
s
.22981 .01472 m
.22981 .01472 L
.23602 .01472 L
.23602 .01472 L
F
.22981 .01472 m
.22981 .01472 L
.23602 .01472 L
.23602 .01472 L
.22981 .01472 L
s
.26087 .01472 m
.26087 .60332 L
.26708 .60332 L
.26708 .01472 L
F
.26087 .01472 m
.26087 .60332 L
.26708 .60332 L
.26708 .01472 L
.26087 .01472 L
s
.29193 .01472 m
.29193 .10544 L
.29814 .10544 L
.29814 .01472 L
F
.29193 .01472 m
.29193 .10544 L
.29814 .10544 L
.29814 .01472 L
.29193 .01472 L
s
.32298 .01472 m
.32298 .03456 L
.32919 .03456 L
.32919 .01472 L
F
.32298 .01472 m
.32298 .03456 L
.32919 .03456 L
.32919 .01472 L
.32298 .01472 L
s
.35404 .01472 m
.35404 .01472 L
.36025 .01472 L
.36025 .01472 L
F
.35404 .01472 m
.35404 .01472 L
.36025 .01472 L
.36025 .01472 L
.35404 .01472 L
s
.38509 .01472 m
.38509 .1457 L
.3913 .1457 L
.3913 .01472 L
F
.38509 .01472 m
.38509 .1457 L
.3913 .1457 L
.3913 .01472 L
.38509 .01472 L
s
.41615 .01472 m
.41615 .01472 L
.42236 .01472 L
.42236 .01472 L
F
.41615 .01472 m
.41615 .01472 L
.42236 .01472 L
.42236 .01472 L
.41615 .01472 L
s
.4472 .01472 m
.4472 .02152 L
.45342 .02152 L
.45342 .01472 L
F
.4472 .01472 m
.4472 .02152 L
.45342 .02152 L
.45342 .01472 L
.4472 .01472 L
s
.47826 .01472 m
.47826 .01585 L
.48447 .01585 L
.48447 .01472 L
F
.47826 .01472 m
.47826 .01585 L
.48447 .01585 L
.48447 .01472 L
.47826 .01472 L
s
.50932 .01472 m
.50932 .05157 L
.51553 .05157 L
.51553 .01472 L
F
.50932 .01472 m
.50932 .05157 L
.51553 .05157 L
.51553 .01472 L
.50932 .01472 L
s
.54037 .01472 m
.54037 .01472 L
.54658 .01472 L
.54658 .01472 L
F
.54037 .01472 m
.54037 .01472 L
.54658 .01472 L
.54658 .01472 L
.54037 .01472 L
s
.57143 .01472 m
.57143 .12246 L
.57764 .12246 L
.57764 .01472 L
F
.57143 .01472 m
.57143 .12246 L
.57764 .12246 L
.57764 .01472 L
.57143 .01472 L
s
.60248 .01472 m
.60248 .01528 L
.6087 .01528 L
.6087 .01472 L
F
.60248 .01472 m
.60248 .01528 L
.6087 .01528 L
.6087 .01472 L
.60248 .01472 L
s
.63354 .01472 m
.63354 .01528 L
.63975 .01528 L
.63975 .01472 L
F
.63354 .01472 m
.63354 .01528 L
.63975 .01528 L
.63975 .01472 L
.63354 .01472 L
s
.6646 .01472 m
.6646 .02209 L
.67081 .02209 L
.67081 .01472 L
F
.6646 .01472 m
.6646 .02209 L
.67081 .02209 L
.67081 .01472 L
.6646 .01472 L
s
.69565 .01472 m
.69565 .01472 L
.70186 .01472 L
.70186 .01472 L
F
.69565 .01472 m
.69565 .01472 L
.70186 .01472 L
.70186 .01472 L
.69565 .01472 L
s
.72671 .01472 m
.72671 .01472 L
.73292 .01472 L
.73292 .01472 L
F
.72671 .01472 m
.72671 .01472 L
.73292 .01472 L
.73292 .01472 L
.72671 .01472 L
s
.75776 .01472 m
.75776 .2404 L
.76398 .2404 L
.76398 .01472 L
F
.75776 .01472 m
.75776 .2404 L
.76398 .2404 L
.76398 .01472 L
.75776 .01472 L
s
.78882 .01472 m
.78882 .01698 L
.79503 .01698 L
.79503 .01472 L
F
.78882 .01472 m
.78882 .01698 L
.79503 .01698 L
.79503 .01472 L
.78882 .01472 L
s
.81988 .01472 m
.81988 .04307 L
.82609 .04307 L
.82609 .01472 L
F
.81988 .01472 m
.81988 .04307 L
.82609 .04307 L
.82609 .01472 L
.81988 .01472 L
s
.85093 .01472 m
.85093 .01528 L
.85714 .01528 L
.85714 .01472 L
F
.85093 .01472 m
.85093 .01528 L
.85714 .01528 L
.85714 .01472 L
.85093 .01472 L
s
.88199 .01472 m
.88199 .02549 L
.8882 .02549 L
.8882 .01472 L
F
.88199 .01472 m
.88199 .02549 L
.8882 .02549 L
.8882 .01472 L
.88199 .01472 L
s
.91304 .01472 m
.91304 .02095 L
.91925 .02095 L
.91925 .01472 L
F
.91304 .01472 m
.91304 .02095 L
.91925 .02095 L
.91925 .01472 L
.91304 .01472 L
s
.9441 .01472 m
.9441 .01472 L
.95031 .01472 L
.95031 .01472 L
F
.9441 .01472 m
.9441 .01472 L
.95031 .01472 L
.95031 .01472 L
.9441 .01472 L
s
.97516 .01472 m
.97516 .02436 L
.98137 .02436 L
.98137 .01472 L
F
.97516 .01472 m
.97516 .02436 L
.98137 .02436 L
.98137 .01472 L
.97516 .01472 L
s
0 0 1 r
.01863 .01472 m
.01863 .07709 L
.02484 .07709 L
.02484 .01472 L
F
.01863 .01472 m
.01863 .07709 L
.02484 .07709 L
.02484 .01472 L
.01863 .01472 L
s
.04969 .01472 m
.04969 .09354 L
.0559 .09354 L
.0559 .01472 L
F
.04969 .01472 m
.04969 .09354 L
.0559 .09354 L
.0559 .01472 L
.04969 .01472 L
s
.08075 .01472 m
.08075 .0958 L
.08696 .0958 L
.08696 .01472 L
F
.08075 .01472 m
.08075 .0958 L
.08696 .0958 L
.08696 .01472 L
.08075 .01472 L
s
.1118 .01472 m
.1118 .01642 L
.11801 .01642 L
.11801 .01472 L
F
.1118 .01472 m
.1118 .01642 L
.11801 .01642 L
.11801 .01472 L
.1118 .01472 L
s
.14286 .01472 m
.14286 .05838 L
.14907 .05838 L
.14907 .01472 L
F
.14286 .01472 m
.14286 .05838 L
.14907 .05838 L
.14907 .01472 L
.14286 .01472 L
s
.17391 .01472 m
.17391 .04477 L
.18012 .04477 L
.18012 .01472 L
F
.17391 .01472 m
.17391 .04477 L
.18012 .04477 L
.18012 .01472 L
.17391 .01472 L
s
.20497 .01472 m
.20497 .02946 L
.21118 .02946 L
.21118 .01472 L
F
.20497 .01472 m
.20497 .02946 L
.21118 .02946 L
.21118 .01472 L
.20497 .01472 L
s
.23602 .01472 m
.23602 .05214 L
.24224 .05214 L
.24224 .01472 L
F
.23602 .01472 m
.23602 .05214 L
.24224 .05214 L
.24224 .01472 L
.23602 .01472 L
s
.26708 .01472 m
.26708 .06518 L
.27329 .06518 L
.27329 .01472 L
F
.26708 .01472 m
.26708 .06518 L
.27329 .06518 L
.27329 .01472 L
.26708 .01472 L
s
.29814 .01472 m
.29814 .0873 L
.30435 .0873 L
.30435 .01472 L
F
.29814 .01472 m
.29814 .0873 L
.30435 .0873 L
.30435 .01472 L
.29814 .01472 L
s
.32919 .01472 m
.32919 .08957 L
.3354 .08957 L
.3354 .01472 L
F
.32919 .01472 m
.32919 .08957 L
.3354 .08957 L
.3354 .01472 L
.32919 .01472 L
s
.36025 .01472 m
.36025 .0391 L
.36646 .0391 L
.36646 .01472 L
F
.36025 .01472 m
.36025 .0391 L
.36646 .0391 L
.36646 .01472 L
.36025 .01472 L
s
.3913 .01472 m
.3913 .10204 L
.39752 .10204 L
.39752 .01472 L
F
.3913 .01472 m
.3913 .10204 L
.39752 .10204 L
.39752 .01472 L
.3913 .01472 L
s
.42236 .01472 m
.42236 .07426 L
.42857 .07426 L
.42857 .01472 L
F
.42236 .01472 m
.42236 .07426 L
.42857 .07426 L
.42857 .01472 L
.42236 .01472 L
s
.45342 .01472 m
.45342 .08049 L
.45963 .08049 L
.45963 .01472 L
F
.45342 .01472 m
.45342 .08049 L
.45963 .08049 L
.45963 .01472 L
.45342 .01472 L
s
.48447 .01472 m
.48447 .0425 L
.49068 .0425 L
.49068 .01472 L
F
.48447 .01472 m
.48447 .0425 L
.49068 .0425 L
.49068 .01472 L
.48447 .01472 L
s
.51553 .01472 m
.51553 .07085 L
.52174 .07085 L
.52174 .01472 L
F
.51553 .01472 m
.51553 .07085 L
.52174 .07085 L
.52174 .01472 L
.51553 .01472 L
s
.54658 .01472 m
.54658 .02265 L
.5528 .02265 L
.5528 .01472 L
F
.54658 .01472 m
.54658 .02265 L
.5528 .02265 L
.5528 .01472 L
.54658 .01472 L
s
.57764 .01472 m
.57764 .10544 L
.58385 .10544 L
.58385 .01472 L
F
.57764 .01472 m
.57764 .10544 L
.58385 .10544 L
.58385 .01472 L
.57764 .01472 L
s
.6087 .01472 m
.6087 .04193 L
.61491 .04193 L
.61491 .01472 L
F
.6087 .01472 m
.6087 .04193 L
.61491 .04193 L
.61491 .01472 L
.6087 .01472 L
s
.63975 .01472 m
.63975 .02889 L
.64596 .02889 L
.64596 .01472 L
F
.63975 .01472 m
.63975 .02889 L
.64596 .02889 L
.64596 .01472 L
.63975 .01472 L
s
.67081 .01472 m
.67081 .06235 L
.67702 .06235 L
.67702 .01472 L
F
.67081 .01472 m
.67081 .06235 L
.67702 .06235 L
.67702 .01472 L
.67081 .01472 L
s
.70186 .01472 m
.70186 .02946 L
.70807 .02946 L
.70807 .01472 L
F
.70186 .01472 m
.70186 .02946 L
.70807 .02946 L
.70807 .01472 L
.70186 .01472 L
s
.73292 .01472 m
.73292 .06745 L
.73913 .06745 L
.73913 .01472 L
F
.73292 .01472 m
.73292 .06745 L
.73913 .06745 L
.73913 .01472 L
.73292 .01472 L
s
.76398 .01472 m
.76398 .10885 L
.77019 .10885 L
.77019 .01472 L
F
.76398 .01472 m
.76398 .10885 L
.77019 .10885 L
.77019 .01472 L
.76398 .01472 L
s
.79503 .01472 m
.79503 .05838 L
.80124 .05838 L
.80124 .01472 L
F
.79503 .01472 m
.79503 .05838 L
.80124 .05838 L
.80124 .01472 L
.79503 .01472 L
s
.82609 .01472 m
.82609 .07539 L
.8323 .07539 L
.8323 .01472 L
F
.82609 .01472 m
.82609 .07539 L
.8323 .07539 L
.8323 .01472 L
.82609 .01472 L
s
.85714 .01472 m
.85714 .02492 L
.86335 .02492 L
.86335 .01472 L
F
.85714 .01472 m
.85714 .02492 L
.86335 .02492 L
.86335 .01472 L
.85714 .01472 L
s
.8882 .01472 m
.8882 .09751 L
.89441 .09751 L
.89441 .01472 L
F
.8882 .01472 m
.8882 .09751 L
.89441 .09751 L
.89441 .01472 L
.8882 .01472 L
s
.91925 .01472 m
.91925 .09637 L
.92547 .09637 L
.92547 .01472 L
F
.91925 .01472 m
.91925 .09637 L
.92547 .09637 L
.92547 .01472 L
.91925 .01472 L
s
.95031 .01472 m
.95031 .06859 L
.95652 .06859 L
.95652 .01472 L
F
.95031 .01472 m
.95031 .06859 L
.95652 .06859 L
.95652 .01472 L
.95031 .01472 L
s
.98137 .01472 m
.98137 .07936 L
.98758 .07936 L
.98758 .01472 L
F
.98137 .01472 m
.98137 .07936 L
.98758 .07936 L
.98758 .01472 L
.98137 .01472 L
s
1 .5 0 r
.02484 .01472 m
.02484 .03286 L
.03106 .03286 L
.03106 .01472 L
F
.02484 .01472 m
.02484 .03286 L
.03106 .03286 L
.03106 .01472 L
.02484 .01472 L
s
.0559 .01472 m
.0559 .06972 L
.06211 .06972 L
.06211 .01472 L
F
.0559 .01472 m
.0559 .06972 L
.06211 .06972 L
.06211 .01472 L
.0559 .01472 L
s
.08696 .01472 m
.08696 .05157 L
.09317 .05157 L
.09317 .01472 L
F
.08696 .01472 m
.08696 .05157 L
.09317 .05157 L
.09317 .01472 L
.08696 .01472 L
s
.11801 .01472 m
.11801 .09354 L
.12422 .09354 L
.12422 .01472 L
F
.11801 .01472 m
.11801 .09354 L
.12422 .09354 L
.12422 .01472 L
.11801 .01472 L
s
.14907 .01472 m
.14907 .10544 L
.15528 .10544 L
.15528 .01472 L
F
.14907 .01472 m
.14907 .10544 L
.15528 .10544 L
.15528 .01472 L
.14907 .01472 L
s
.18012 .01472 m
.18012 .0839 L
.18634 .0839 L
.18634 .01472 L
F
.18012 .01472 m
.18012 .0839 L
.18634 .0839 L
.18634 .01472 L
.18012 .01472 L
s
.21118 .01472 m
.21118 .10941 L
.21739 .10941 L
.21739 .01472 L
F
.21118 .01472 m
.21118 .10941 L
.21739 .10941 L
.21739 .01472 L
.21118 .01472 L
s
.24224 .01472 m
.24224 .05271 L
.24845 .05271 L
.24845 .01472 L
F
.24224 .01472 m
.24224 .05271 L
.24845 .05271 L
.24845 .01472 L
.24224 .01472 L
s
.27329 .01472 m
.27329 .01698 L
.2795 .01698 L
.2795 .01472 L
F
.27329 .01472 m
.27329 .01698 L
.2795 .01698 L
.2795 .01472 L
.27329 .01472 L
s
.30435 .01472 m
.30435 .03796 L
.31056 .03796 L
.31056 .01472 L
F
.30435 .01472 m
.30435 .03796 L
.31056 .03796 L
.31056 .01472 L
.30435 .01472 L
s
.3354 .01472 m
.3354 .07142 L
.34161 .07142 L
.34161 .01472 L
F
.3354 .01472 m
.3354 .07142 L
.34161 .07142 L
.34161 .01472 L
.3354 .01472 L
s
.36646 .01472 m
.36646 .10204 L
.37267 .10204 L
.37267 .01472 L
F
.36646 .01472 m
.36646 .10204 L
.37267 .10204 L
.37267 .01472 L
.36646 .01472 L
s
.39752 .01472 m
.39752 .03229 L
.40373 .03229 L
.40373 .01472 L
F
.39752 .01472 m
.39752 .03229 L
.40373 .03229 L
.40373 .01472 L
.39752 .01472 L
s
.42857 .01472 m
.42857 .03456 L
.43478 .03456 L
.43478 .01472 L
F
.42857 .01472 m
.42857 .03456 L
.43478 .03456 L
.43478 .01472 L
.42857 .01472 L
s
.45963 .01472 m
.45963 .05554 L
.46584 .05554 L
.46584 .01472 L
F
.45963 .01472 m
.45963 .05554 L
.46584 .05554 L
.46584 .01472 L
.45963 .01472 L
s
.49068 .01472 m
.49068 .10374 L
.49689 .10374 L
.49689 .01472 L
F
.49068 .01472 m
.49068 .10374 L
.49689 .10374 L
.49689 .01472 L
.49068 .01472 L
s
.52174 .01472 m
.52174 .05951 L
.52795 .05951 L
.52795 .01472 L
F
.52174 .01472 m
.52174 .05951 L
.52795 .05951 L
.52795 .01472 L
.52174 .01472 L
s
.5528 .01472 m
.5528 .09694 L
.55901 .09694 L
.55901 .01472 L
F
.5528 .01472 m
.5528 .09694 L
.55901 .09694 L
.55901 .01472 L
.5528 .01472 L
s
.58385 .01472 m
.58385 .02832 L
.59006 .02832 L
.59006 .01472 L
F
.58385 .01472 m
.58385 .02832 L
.59006 .02832 L
.59006 .01472 L
.58385 .01472 L
s
.61491 .01472 m
.61491 .07823 L
.62112 .07823 L
.62112 .01472 L
F
.61491 .01472 m
.61491 .07823 L
.62112 .07823 L
.62112 .01472 L
.61491 .01472 L
s
.64596 .01472 m
.64596 .0941 L
.65217 .0941 L
.65217 .01472 L
F
.64596 .01472 m
.64596 .0941 L
.65217 .0941 L
.65217 .01472 L
.64596 .01472 L
s
.67702 .01472 m
.67702 .08503 L
.68323 .08503 L
.68323 .01472 L
F
.67702 .01472 m
.67702 .08503 L
.68323 .08503 L
.68323 .01472 L
.67702 .01472 L
s
.70807 .01472 m
.70807 .0958 L
.71429 .0958 L
.71429 .01472 L
F
.70807 .01472 m
.70807 .0958 L
.71429 .0958 L
.71429 .01472 L
.70807 .01472 L
s
.73913 .01472 m
.73913 .03116 L
.74534 .03116 L
.74534 .01472 L
F
.73913 .01472 m
.73913 .03116 L
.74534 .03116 L
.74534 .01472 L
.73913 .01472 L
s
.77019 .01472 m
.77019 .02436 L
.7764 .02436 L
.7764 .01472 L
F
.77019 .01472 m
.77019 .02436 L
.7764 .02436 L
.7764 .01472 L
.77019 .01472 L
s
.80124 .01472 m
.80124 .09524 L
.80745 .09524 L
.80745 .01472 L
F
.80124 .01472 m
.80124 .09524 L
.80745 .09524 L
.80745 .01472 L
.80124 .01472 L
s
.8323 .01472 m
.8323 .05044 L
.83851 .05044 L
.83851 .01472 L
F
.8323 .01472 m
.8323 .05044 L
.83851 .05044 L
.83851 .01472 L
.8323 .01472 L
s
.86335 .01472 m
.86335 .13266 L
.86957 .13266 L
.86957 .01472 L
F
.86335 .01472 m
.86335 .13266 L
.86957 .13266 L
.86957 .01472 L
.86335 .01472 L
s
.89441 .01472 m
.89441 .04363 L
.90062 .04363 L
.90062 .01472 L
F
.89441 .01472 m
.89441 .04363 L
.90062 .04363 L
.90062 .01472 L
.89441 .01472 L
s
.92547 .01472 m
.92547 .0425 L
.93168 .0425 L
.93168 .01472 L
F
.92547 .01472 m
.92547 .0425 L
.93168 .0425 L
.93168 .01472 L
.92547 .01472 L
s
.95652 .01472 m
.95652 .02776 L
.96273 .02776 L
.96273 .01472 L
F
.95652 .01472 m
.95652 .02776 L
.96273 .02776 L
.96273 .01472 L
.95652 .01472 L
s
.98758 .01472 m
.98758 .04704 L
.99379 .04704 L
.99379 .01472 L
F
.98758 .01472 m
.98758 .04704 L
.99379 .04704 L
.99379 .01472 L
.98758 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{500, 309},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007d0001=B000`400?l00000o`00003ooooooooe
ooooo`00;Oooool3o`0000[ooooo1Ol0000:ooooo`?o00003?ooool3o`0000[ooooo0ol0000;oooo
o`?o00003?ooool00ol0003oooooooooo`0:ooooo`?o00002oooool2o`0000Wooooo0ol00004oooo
o`;o00001Oooool3o`0000?ooooo0ol00005ooooo`?o00000_ooool5o`0000Cooooo0ol00003oooo
o`?o00001Oooool3o`0000Cooooo0ol00004ooooo`?o00000oooool3o`0000Gooooo0ol00003oooo
o`?o00001Oooool3o`0000Cooooo00?o0000ooooooooool01?ooool3o`0000?ooooo0ol00005oooo
o`?o00000oooool2o`0000Gooooo1Ol00003ooooo`;o00001?ooool5o`0000;ooooo0ol00004oooo
o`Go000000?oooooo`000?l000000ol00003ooooo`Go00000_ooool3o`0000Cooooo1Ol00003oooo
o`?o00000oooool5o`0000;ooooo0ol00004ooooo`Go00000_ooool3o`0000Cooooo1Ol00003oooo
o`03o`000?oooooooooo00?ooooo1Ol00002ooooo`?o00001?ooool5o`0000;ooooo0_l00006oooo
o`?o00001?ooool2o`0000Gooooo0ol00003ooooo`?o00001Oooool3o`0000;ooooo1Ol0000>oooo
o`00;_ooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000Wooooo00Go0000oooo
ooooooooooooo`00000<ooooo`03o`000?oooooooooo00Sooooo00Go0000ooooooooooooooooo`00
0009ooooo`05o`000?ooooooooooooooool000002oooool00ol0003oooooooooo`09ooooo`05o`00
0?ooooooooooooooool000003?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00;o
oooo00Co0000oooooooooooo00001Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`05o`000?oooooo
ooooooooool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01?ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool0
0oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`03oooo
o`04o`000?ooooooooooo`0000;ooooo00Co0000oooooooooooo00001?ooool01?l0003ooooooooo
ool00003ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooo00000_ooool01?l0003o
ooooooooool00004ooooo`06o`000?ooooooooooo`000?oooooo00000oooool00ol0003ooooooooo
o`02ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool00oooool01_l0003ooooo
ooooool0003oooooo`0000?ooooo00?o0000ooooooooool00_ooool01_l0003oooooooooool0003o
ooooo`0000?ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00003ooooo`03o`00
0?oooooooooo00Cooooo00Ko0000oooooooooooo0000ooooool00003ooooo`03o`000?oooooooooo
00;ooooo00Co0000oooooooooooo00001?ooool00ol0003oooooooooo`02ooooo`05o`000?oooooo
ooooooooool000000_ooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000oooool00ol0003oooooooooo`03ooooo`05o`000?ooooooooooooooool000000_ooool01?l0
003oooooooooool0000>ooooo`00;_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo
00gooooo00?o0000ooooooooool01oooool5o`0000gooooo00?o0000ooooooooool01oooool01Ol0
003ooooooooooooooooo000000cooooo00?o0000ooooooooool02?ooool01Ol0003ooooooooooooo
oooo000000gooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`02ooooo`04o`000?oo
ooooooooo`0000Gooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00?ooooo00Co0000oooooooooooooooo1Ol00005ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00Go0000oooo
ooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00001Ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00ooo
ool01?l0003oooooooooooooool5o`0000Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`03ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooooooooooo0000
00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`05ooooo`05o`000?oooooo
ooooooooool000002Oooool01Ol0003ooooooooooooooooo0000013ooooo000^ooooo`03o`000?oo
oooooooo00cooooo00?o0000ooooooooool02_ooool2o`0000[ooooo00Co0000oooooooooooo0000
3_ooool00ol0003oooooooooo`07ooooo`Co00003Oooool00ol0003oooooooooo`09ooooo`?o0000
2oooool4o`0000Sooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`03
o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool00oooool2o`0000Kooooo00Go0000oooooooo
ooooooooo`000002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`03ooooo`04o`000?ooooooooooooooo`Co00001_ooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool3o`0000Kooooo00?o0000
ooooooooool00_ooool4o`0000Kooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`06ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`02ooooo`;o00001ooo
ool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Co00001oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00Gooooo00Co0000oooooooooooooooo0ol00007ooooo`04
o`000?ooooooooooooooo`Co00001Oooool2o`0000?ooooo00Co0000oooooooooooo00001Oooool2
o`0000Cooooo00?o0000ooooooooool01Oooool2o`0000Gooooo00?o0000ooooooooool03Oooool0
02kooooo00?o0000ooooooooool02Oooool01Ol0003ooooooooooooooooo000000gooooo00?o0000
ooooooooool02?ooool00ol0003oooooo`00000:ooooo`Co00002_ooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00Go0000ooooooooooooooooo`000009ooooo`05o`000?oo
ooooooooooooool000002?ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Go
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`05o`000?oooooooooooooo
ool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Kooooo00Co0000oooo
oooooooooooo1?l00006ooooo`05o`000?ooooooooooooooool000002Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooool00002ooooo`05o`000?ooooooooooooooool000000ooo
ool01Ol0003ooooooooooooooooo000000Gooooo00Oo0000ooooooooooooooooo`000?oooooo0000
00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00Go0000
ooooooooooooooooo`000003ooooo`06o`000?ooooooooooo`000?oooooo00001?ooool01_l0003o
oooooooooooooooo0000ooooo`Co00001?ooool01ol0003ooooooooooooooooo0000ooooool00000
1oooool01Ol0003ooooooooooooooooo000000Gooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo
000000?ooooo00?o0000ooooooooool01Oooool01?l0003oooooooooool00002ooooo`03o`000?oo
oooooooo00Gooooo00Go0000ooooooooooooooooo`000009ooooo`03o`000?oooooo000000?ooooo
00?o0000ooooooooool03?ooool002gooooo0_l0000;ooooo`05o`000?ooooooooooooooool00000
2Oooool01Ol0003ooooooooooooooooo000000_ooooo0_l0000:ooooo`03o`000?oooooooooo00co
oooo00?o0000ooooooooool02_ooool01Ol0003ooooooooooooooooo000000Wooooo00Go0000oooo
ooooooooooooo`000009ooooo`05o`000?ooooooooooooooool000001oooool2o`0000Cooooo00Co
0000oooooooooooo00001?ooool2o`0000Cooooo0_l00006ooooo`;o00000oooool01Ol0003ooooo
oooooooooooo000000Cooooo0_l00003ooooo`05o`000?ooooooooooooooool000001?ooool2o`00
00Gooooo0_l00005ooooo`;o00000oooool00ol0003oooooooooo`06ooooo`;o00001?ooool00ol0
003oooooooooo`05ooooo`;o00000oooool01Ol0003ooooooooooooooooo000000Cooooo0_l00003
ooooo`05o`000?ooooooooooooooool000001?ooool2o`0000?ooooo00Go0000oooooooooooooooo
o`000003ooooo`05o`000?ooooooooooooooool000000_ooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool2o`0000Gooooo00Oo0000ooooooooooooooooo`00
0?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo000000?o
oooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooooooo`;o00001?ooool0
1ol0003ooooooooooooooooo0000ooooool000001oooool01Ol0003ooooooooooooooooo000000;o
oooo00?o0000ooooooooool01?ooool01ol0003ooooooooooooooooo0000ooooool000000oooool0
1Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooool000000oooool01Ol0003ooooooooo
oooooooo000000?ooooo00?o0000ooooool000000oooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000_ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000
ooooooooool00_l00005ooooo`07o`000?ooooooooooooooool0003oooooo`000003ooooo`03o`00
0?oooooooooo00cooooo000^ooooo`03o`000?oooooooooo00[ooooo0ol0000;ooooo`?o00003Ooo
ool00ol0003oooooooooo`08ooooo`Go00002oooool2o`0000[ooooo1Ol0000:ooooo`?o00002ooo
ool3o`0000Wooooo00?o0000ooooooooool00oooool2o`0000Kooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;ooooo0ol00006ooooo`03o`000?oo
oooooooo00;ooooo0ol00006ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?oo
ool01?l0003oooooooooooooool5o`0000Gooooo00?o0000ooooooooool00oooool2o`0000Kooooo
00Co0000oooooooooooooooo1Ol00005ooooo`03o`000?oooooooooo00;ooooo0ol00006ooooo`03
o`000?oooooooooo00;ooooo0ol00005ooooo`?o00001?ooool2o`0000Gooooo0ol00004ooooo`03
o`000?oooooooooo00Cooooo0ol00003ooooo`?o00001Oooool3o`0000?ooooo0ol00005ooooo`?o
00001Oooool00ol0003oooooooooo`03ooooo`?o00000_ooool5o`0000Cooooo0ol00004ooooo`;o
00001Oooool3o`0000;ooooo1Ol00004ooooo`?o00000oooool3o`0000Gooooo0ol00003ooooo`?o
00001Oooool3o`0000Cooooo0_l00005ooooo`?o00001?ooool00ol0003oooooooooo`04ooooo`?o
00000oooool3o`0000oooooo003oooooooGooooo003oooooooGooooo003oooooooGooooo003ooooo
ooGooooo003oooooooGooooo003oooooooGooooo003oooooooGooooo000Voooooooo0000aOl0000:
ooooo`009_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`08ooooo`00
9_ooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`08ooooo`007Oooool2o`0000Oooooo00?o0000ooooooooool0oooo
ooo1ooooo`03o`000?oooooooooo00Sooooo000Looooo`04o`000?ooooooooooo`0000Kooooo00?o
0000ooooooooool0ooooooo1ooooo`03o`000?oooooooooo00Sooooo000Looooo`04o`000?oooooo
ooooo`0000Kooooo00?o0000ooooooooool0ooooooo1ooooo`03o`000?oooooooooo00Sooooo000L
ooooo`04o`000?ooooooooooo`0000Kooooo0ol00003ool000?o0?l00_l00?l4oomo00;o00000ooo
0003o`3o00;o003o1?ooO`02o`0000?oo`000ol0o`02o`00o`Coogl00_l00003ool000?o0?l00_l0
0?l4oomo00;o00000ooo0003o`3o00;o003o1?ooO`02o`0000?oo`000ol0o`02o`00o`Coogl00_l0
0003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0
o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0ooo
O`02o`0000?oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo
0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol0
0?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl00_l0
0003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0
o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0ooo
O`02o`0000?oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo
0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol0
0?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl00_l0
0003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`03o`0000[ooooo000L
ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool00ooo0003o`3o00;o003o1?oo
O`02ooooo`?oo`000ol0o`02o`00o`Coogl00_ooool3ool000?o0?l00_l00?l4oomo00Sooooo0_l0
0?l4oomo00;ooooo0ooo0003o`3o00;o003o1?ooO`02ooooo`Coo`000_ooool2o`00o`Coogl00_oo
ool4ool000;ooooo0ol00?l3oomo00Sooooo0ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0ooo
O`02ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00Sooooo0ol0
0?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`08ooooo`?o003o0oooO`05ooooo`?o0?l00ol0
0?l3oomo00;ooooo1?oo0002ooooo`?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl02?oo
ool3o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00Sooooo0ol00?l3oomo00;ooooo1?oo
0002ooooo`?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool4ool000;ooooo0ol0
0?l3oomo00Sooooo0ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`?oo`000ol0
o`03o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00Sooooo0ol00?l3oomo00;ooooo0ooo
0003o`3o00?o003o0oooO`05ooooo`?o0?l00ol00?l3oomo00Sooooo0ol00?l3oomo00;ooooo0ooo
0003o`3o00?o003o0oooO`02ooooo`03o`000?oooooooooo00Sooooo000Looooo`04o`000?oooooo
ooooo`0000Kooooo00?o0000ooooooooool00ooo0003o`3o00;o003o1?ooO`02ooooo`?oo`000ol0
o`02o`00o`Coogl00_ooool3ool000?o0?l00_l00?l4oomo00[ooooo1?ooO`02ooooo`?oo`000ol0
o`02o`00o`Coogl00_ooool4ool000;ooooo0_l00?l4oomo00Sooooo0ol00?l3oomo00Sooooo0ol0
0?l3oomo00;ooooo0ooo0003o`3o00Co003o1?ooool3ool000?o0?l00ol00?l3oomo00;ooooo0ooo
0003o`3o00?o003o0oooO`08ooooo`?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl02?oo
ool3o`00o`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl00_oo
ool3ool000?o0?l00ol00?l3oomo00Sooooo0ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0ooo
O`08ooooo`?o003o0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl00_ooool3ool000?o0?l00ol0
0?l3oomo00Sooooo0ol00?l3oomo00Sooooo0ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0ooo
O`02ooooo`Coo`000_ooool3o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00Sooooo0ol0
0?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`05ooooo`?o0?l00ol00?l3oomo00Sooooo0ol0
0?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`03o`000?oooooooooo00Sooooo000M
ooooo`;o00001oooool00ol0003oooooooooo`03ool000?o0?l00_l00?l4oomo00;ooooo0ooo0003
o`3o00;o003o1?ooO`02ooooo`?oo`000ol0o`02o`00o`Coogl02_ooool4oomo00;ooooo0ooo0003
o`3o00;o003o1?ooO`08ooooo`;o003o1?ooO`08ooooo`?o003o0oooO`08ooooo`?o003o0oooO`02
ooooo`?oo`000ol0o`04o`00o`Cooooo0ooo0003o`3o00?o003o0oooO`02ooooo`?oo`000ol0o`03
o`00o`?oogl02?ooool3o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00Sooooo0ol00?l3
oomo00Gooooo0ol0o`03o`00o`?oogl02?ooool3o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3
oomo00Sooooo0ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`08ooooo`?o003o0oooO`08
ooooo`?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl02?ooool3o`00o`?oogl02?ooool3
o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo1?oo0002ooooo`?o003o0oooO`02
ooooo`?oo`000ol0o`03o`00o`?oogl02?ooool3o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3
oomo00Gooooo0ol0o`03o`00o`?oogl02?ooool3o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3
oomo00;ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00ooo0003o`3o
00;o003o1?ooO`02ooooo`?oo`000ol0o`02o`00o`Coogl00_ooool3ool000?o0?l00_l00?l4oomo
00[ooooo1?ooO`02ooooo`?oo`000ol0o`02o`00o`Coogl02?ooool2o`00o`Coogl02?ooool3o`00
o`?oogl02?ooool3o`00o`?oogl00_ooool3ool000?o0?l01?l00?l4ooooo`?oo`000ol0o`03o`00
o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00Sooooo0ol00?l3oomo00;ooooo0ooo0003o`3o
00?o003o0oooO`08ooooo`?o003o0oooO`08ooooo`?o003o0oooO`08ooooo`?o003o0oooO`02oooo
o`?oo`000ol0o`03o`00o`?oogl02oooool3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`08oooo
o`?o003o0oooO`08ooooo`?o003o0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl02?ooool3o`00
o`?oogl02?ooool3o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00;ooooo1?oo0002oooo
o`?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl02?ooool3o`00o`?oogl00_ooool3ool0
00?o0?l00ol00?l3oomo00Sooooo0ol00?l3oomo00Sooooo0ol00?l3oomo00;ooooo0ooo0003o`3o
00?o003o0oooO`02ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?o
o`000ol0o`02o`00o`Coogl00_ooool3ool000?o0?l00_l00?l4oomo00;ooooo0ooo0003o`3o00;o
003o1?ooO`0:ooooo`Coogl00_ooool4ool000;ooooo0_l00?l4oomo00Sooooo0_l00?l4oomo00So
oooo0ol00?l3oomo00Sooooo0ol00?l3oomo00;ooooo0ooo0003o`3o00Co003o1?ooool3ool000?o
0?l00ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`08ooooo`?o003o0oooO`02ooooo`?o
o`000ol0o`03o`00o`?oogl02?ooool3o`00o`?oogl02?ooool3o`00o`?oogl02?ooool3o`00o`?o
ogl00_ooool3ool000?o0?l00ol00?l3oomo00_ooooo0oooO`02ooooo`?oo`000ol0o`03o`00o`?o
ogl02?ooool3o`00o`?oogl02?ooool3o`00o`?oogl00_ooool4ool000;ooooo0ol00?l3oomo00So
oooo0ol00?l3oomo00Sooooo0ol00?l3oomo00;ooooo0ooo0003o`3o00Co003o1?ooool4ool000;o
oooo0ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`0;ooooo`?oogl00_ooool3ool000?o
0?l00ol00?l3oomo00Sooooo0ol00?l3oomo00Sooooo0ol00?l3oomo00Sooooo0ol00?l3oomo00;o
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00ooo0003o`3o00;o003o
1?ooO`02ooooo`?oo`000ol0o`02o`00o`Coogl00_ooool3ool000?o0?l00_l00?l4oomo00[ooooo
1?ooO`02ooooo`Coo`000_ooool2o`00o`Coogl02?ooool2o`00o`Coogl02?ooool3o`00o`?oogl0
2?ooool3o`00o`?oogl00_ooool3ool000?o0?l01?l00?l4ooooo`?oo`000ol0o`03o`00o`?oogl0
0_ooool3ool000?o0?l00ol00?l3oomo00Sooooo0ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o
0oooO`08ooooo`?o003o0oooO`08ooooo`?o003o0oooO`08ooooo`?o003o0oooO`02ooooo`?oo`00
0ol0o`03o`00o`?oogl02oooool3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`08ooooo`?o003o
0oooO`08ooooo`?o003o0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl02?ooool3o`00o`?oogl0
2?ooool3o`00o`?oogl00_ooool3ool000?o0?l01?l00?l4ooooo`Coo`000_ooool3o`00o`?oogl0
0_ooool3ool000?o0?l00ol00?l3oomo00_ooooo0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl0
2?ooool3o`00o`?oogl02?ooool3o`00o`?oogl02?ooool3o`00o`?oogl00_ooool00ol0003ooooo
ooooo`08ooooo`009_ooool3o`0000?oo`000ol0o`02o`00o`Coogl00_ooool3ool000?o0?l00_l0
0?l4oomo00;ooooo0ooo0003o`3o00;o003o1?ooO`0:ooooo`Coogl00_ooool4ool000;ooooo0_l0
0?l4oomo00Sooooo0_l00?l4oomo00_ooooo0oooO`08ooooo`?o003o0oooO`02ooooo`?oo`000ol0
o`04o`00o`Cooooo0ooo0003o`3o00?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl02?oo
ool3o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00Sooooo0ol00?l3oomo00Sooooo0ol0
0?l3oomo00Sooooo0ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`0;ooooo`?oogl00_oo
ool3ool000?o0?l01?l00?l:ooooo`?o003o0oooO`0;ooooo`?oogl00_ooool4ool000;ooooo0ol0
0?l3oomo00_ooooo0oooO`08ooooo`?o003o0oooO`02ooooo`?oo`000ol0o`04o`00o`Cooooo1?oo
0002ooooo`?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl02oooool3oomo00;ooooo1?oo
0002ooooo`?o003o0oooO`08ooooo`?o003o0oooO`08ooooo`Co003o2_ooool3o`00o`?oogl000?o
ooooo`000?l000002_ooool002Kooooo00?o0000ooooooooool00ooo0003o`3o00;o003o1?ooO`02
ooooo`?oo`000ol0o`02o`00o`Coogl00_ooool3ool000?o0?l00_l00?l4oomo00[ooooo1?ooO`02
ooooo`Coo`000_ooool2o`00o`Coogl02?ooool2o`00o`Coogl02oooool3oomo00Sooooo0ol00?l3
oomo00;ooooo0ooo0003o`3o00Co003o1?ooool3ool000?o0?l00ol00?l3oomo00;ooooo0ooo0003
o`3o00?o003o0oooO`08ooooo`?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl02?ooool3
o`00o`?oogl02?ooool3o`00o`?oogl02?ooool3o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3
oomo00_ooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`[ooooo0ol00?l3oomo00_ooooo0oooO`02
ooooo`Coo`000_ooool3o`00o`?oogl02oooool3oomo00Sooooo1?l00?l4ooooo`?oo`000ol0o`04
o`00o`Cooooo1?oo0002ooooo`?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl02oooool3
oomo00;ooooo1?oo0002ooooo`?o003o0oooO`08ooooo`?o003o0oooO`08ooooo`Co003o2_ooool3
o`00o`?oogl00_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ool0
00?o0?l00ol00?l8ooooo`?o0?l00_l00?l4oomo00;ooooo0ooo0003o`3o00;o003o1?ooO`0:oooo
o`Coogl00_ooool4ool000;ooooo0_l00?l4oomo00Sooooo0_l00?l4oomo00_ooooo0oooO`08oooo
o`?o003o0oooO`05ooooo`?o0?l01?l00?l4ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool3ool0
00?o0?l00ol00?l3oomo00Sooooo0ol00?l3oomo00;ooooo0ooo0003o`3o00Co003o2_ooool3o`00
o`?oogl02?ooool3o`00o`?oogl02?ooool3o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo
00_ooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`[ooooo0ol00?l3oomo00_ooooo0oooO`02oooo
o`Coo`000_ooool3o`00o`?oogl02oooool3oomo00Sooooo1?l00?l4ooooo`?oo`000ol0o`04o`00
o`Cooooo1?oo0002ooooo`?o003o0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl02oooool3oomo
00;ooooo1?oo0002ooooo`?o003o0oooO`08ooooo`?o003o0oooO`08ooooo`Co003o2_ooool3o`00
o`?oogl00_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ool000?o
0?l00ol00?l8ooooo`?o0?l00_l00?l4oomo00;ooooo0ooo0003o`3o00;o003o1?ooO`0:ooooo`Co
ogl00_ooool4ool000;ooooo0_l00?l4oomo00Sooooo0_l00?l4oomo00_ooooo0oooO`08ooooo`?o
003o0oooO`05ooooo`?o0?l01?l00?l4ooooo`?oo`000ol0o`03o`00o`?oogl00_ooool4ool000;o
oooo0ol00?l3oomo00Sooooo0ol00?l3oomo00;ooooo0ooo0003o`3o00Co003o2_ooool4o`00o`[o
oooo0ol00?l3oomo00Sooooo0ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`0;ooooo`?o
ogl00_ooool3ool000?o0?l01?l00?l:ooooo`?o003o0oooO`0;ooooo`?oogl00_ooool4ool000;o
oooo0ol00?l3oomo00_ooooo0oooO`08ooooo`Co003o1?ooool3ool000?o0?l01?l00?l4ooooo`Co
o`000_ooool3o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00_ooooo0oooO`02ooooo`Co
o`000_ooool3o`00o`?oogl02?ooool3o`00o`?oogl02?ooool4o`00o`[ooooo0ol00?l3oomo00;o
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00ooo0003o`3o00?o003o
2?ooool3o`3o00;o003o1?ooO`02ooooo`?oo`000ol0o`02o`00o`Coogl02_ooool4oomo00;ooooo
1?oo0002ooooo`;o003o1?ooO`08ooooo`;o003o1?ooO`0;ooooo`?oogl02?ooool3o`00o`?oogl0
1Oooool3o`3o00Co003o1?ooool3ool000?o0?l01?l00?l4ooooo`Coo`000_ooool3o`00o`?oogl0
2?ooool3o`00o`?oogl00_ooool3ool000?o0?l01?l00?l:ooooo`Co003o2_ooool3o`00o`?oogl0
2?ooool3o`00o`?oogl00_ooool3ool000?o0?l00ol00?l3oomo00_ooooo0oooO`02ooooo`?oo`00
0ol0o`04o`00o`[ooooo0ol00?l3oomo00_ooooo0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl0
2oooool3oomo00Sooooo1?l00?l4ooooo`?oo`000ol0o`04o`00o`Cooooo1?oo0002ooooo`?o003o
0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl02oooool3oomo00;ooooo1?oo0002ooooo`?o003o
0oooO`08ooooo`?o003o0oooO`08ooooo`Co003o2_ooool3o`00o`?oogl00_ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`03ool000?o0?l00ol00?l8ooooo`?o0?l00_l0
0?l4oomo00;ooooo0ooo0003o`3o00;o003o1?ooO`0:ooooo`Coogl00_ooool4ool000;ooooo0_l0
0?l4oomo00Sooooo0_l00?l4oomo00_ooooo0oooO`08ooooo`?o003o0oooO`05ooooo`?o0?l01?l0
0?l4ooooo`?oo`000ol0o`04o`00o`Cooooo1?oo0002ooooo`?o003o0oooO`0;ooooo`?oogl00_oo
ool3ool000?o0?l01?l00?l:ooooo`Co003o2_ooool3o`00o`?oogl02?ooool3o`00o`?oogl00_oo
ool3ool000?o0?l00ol00?l3oomo00_ooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`[ooooo0ol0
0?l3oomo00_ooooo0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl02oooool3oomo00Sooooo1?l0
0?l4ooooo`?oo`000ol0o`04o`00o`Cooooo1?oo0002ooooo`?o003o0oooO`02ooooo`?oo`000ol0
o`03o`00o`?oogl02oooool3oomo00;ooooo1?oo0002ooooo`?o003o0oooO`08ooooo`?o003o0ooo
O`08ooooo`Co003o2_ooool3o`00o`?oogl00_ooool00ol0003oooooooooo`08ooooo`009_ooool0
0ol0003oooooooooo`03ool000?o0?l00ol00?l8ooooo`?o0?l00_l00?l4oomo00;ooooo0ooo0003
o`3o00;o003o1?ooO`0:ooooo`Coogl00_ooool4ool000;ooooo0_l00?l4oomo00Sooooo0_l00?l4
oomo00_ooooo0oooO`08ooooo`?o003o0oooO`05ooooo`?o0?l01?l00?l4ooooo`?oo`000ol0o`04
o`00o`Cooooo1?oo0002ooooo`?o003o0oooO`0;ooooo`?oogl00_ooool3ool000?o0?l01?l00?l:
ooooo`Co003o2_ooool3o`00o`?oogl02oooool3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`0;
ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=ooooo`?oogl02oooool3oomo00;ooooo1?oo0002
ooooo`?o003o0oooO`0;ooooo`?oogl02?ooool4o`00o`Cooooo0ooo0003o`3o00Co003o1?ooool4
ool000;ooooo0ol00?l3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`0;ooooo`?oogl00_ooool4
ool000;ooooo0ol00?l3oomo00Sooooo1?l00?l:ooooo`Co003o2_ooool3o`00o`?oogl00_ooool0
0ol0003oooooooooo`08ooooo`009_ooool3o`0000?oo`000ol0o`03o`00o`Sooooo0ol0o`02o`00
o`Coogl00_ooool3ool000?o0?l00_l00?l4oomo00[ooooo1?ooO`02ooooo`Coo`000_ooool2o`00
o`Coogl02_ooool4oomo00_ooooo0oooO`08ooooo`?o003o0oooO`05ooooo`?o0?l01?l00?l4oooo
o`?oo`000ol0o`04o`00o`Cooooo1?oo0002ooooo`?o003o0oooO`0;ooooo`?oogl00_ooool3ool0
00?o0?l01?l00?l:ooooo`Co003o2_ooool3o`00o`?oogl02oooool3oomo00;ooooo0ooo0003o`3o
00?o003o0oooO`0;ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=ooooo`?oogl02oooool3oomo
00;ooooo1?oo0002ooooo`?o003o0oooO`0;ooooo`?oogl02?ooool4o`00o`Cooooo0ooo0003o`3o
00Co003o1?ooool4ool000;ooooo0ol00?l3oomo00;ooooo1?oo0002ooooo`?o003o0oooO`0;oooo
o`?oogl00_ooool4ool000;ooooo1?l00?l:ooooo`Co003o2_ooool4o`00o`[ooooo0ol00?l3oomo
0003ooooool0003o000000[ooooo000Vooooo`03o`000?oooooooooo00?oo`000ol0o`03o`00o`So
oooo0ol0o`02o`00o`Coogl00_ooool3ool000?o0?l00_l00?l4oomo00[ooooo1?ooO`08ooooo`;o
003o1?ooO`0:ooooo`Coogl02oooool3oomo00Sooooo0ol00?l3oomo00Gooooo0ol0o`04o`00o`Co
oooo0ooo0003o`3o00Co003o1?ooool4ool000;ooooo0ol00?l3oomo00_ooooo0oooO`02ooooo`?o
o`000ol0o`04o`00o`[ooooo1?l00?l:ooooo`?o003o0oooO`0;ooooo`?oogl00_ooool3ool000?o
0?l00ol00?l3oomo00_ooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`0;ooooo`?o
ogl00_ooool4ool000;ooooo0ol00?l3oomo00_ooooo0oooO`08ooooo`Co003o1?ooool3ool000?o
0?l01?l00?l4ooooo`Coo`000_ooool3o`00o`?oogl00_ooool4ool000;ooooo0ol00?l3oomo00_o
oooo0oooO`02ooooo`Coo`000_ooool4o`00o`[ooooo1?l00?l:ooooo`Co003o2_ooool4o`00o`Co
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00ooo0003o`3o00?o003o
2?ooool3o`3o00;o003o1?ooO`02ooooo`?oo`000ol0o`02o`00o`Coogl02_ooool4oomo00Sooooo
0_l00?l4oomo00[ooooo1?ooO`0;ooooo`?oogl02?ooool3o`00o`?oogl01Oooool3o`3o00Co003o
1?ooool3ool000?o0?l01?l00?l4ooooo`Coo`000_ooool3o`00o`?oogl02oooool3oomo00;ooooo
0ooo0003o`3o00Co003o2_ooool4o`00o`[ooooo0ol00?l3oomo00_ooooo0oooO`02ooooo`?oo`00
0ol0o`03o`00o`?oogl02oooool3oomo00;ooooo0ooo0003o`3o00Co003o3Oooool3oomo00_ooooo
0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl02oooool3oomo00Sooooo1?l00?l4ooooo`?oo`00
0ol0o`04o`00o`Cooooo1?oo0002ooooo`?o003o0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl0
2oooool3oomo00;ooooo1?oo0002ooooo`Co003o2_ooool4o`00o`[ooooo1?l00?l:ooooo`Co003o
1?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ool000?o0?l00ol0
0?l8ooooo`?o0?l00_l00?l4oomo00;ooooo0ooo0003o`3o00?o003o3Oooool4oomo00Sooooo0_l0
0?l4oomo00[ooooo1?ooO`0;ooooo`?oogl02?ooool3o`00o`?oogl01Oooool3o`3o00Co003o1?oo
ool3ool000?o0?l01?l00?l4ooooo`Coo`000_ooool3o`00o`?oogl02oooool3oomo00;ooooo0ooo
0003o`3o00Co003o2_ooool4o`00o`[ooooo0ol00?l3oomo00_ooooo0oooO`02ooooo`Coo`000_oo
ool3o`00o`?oogl02oooool3oomo00;ooooo0ooo0003o`3o00Co003o3Oooool3oomo00_ooooo0ooo
O`02ooooo`Coo`000_ooool3o`00o`?oogl02oooool3oomo00Sooooo1?l00?l4ooooo`?oo`000ol0
o`04o`00o`Cooooo1?oo0002ooooo`?o003o0oooO`02ooooo`Coo`000_ooool4o`00o`gooooo0ooo
O`02ooooo`Coo`000_ooool4o`00o`[ooooo1?l00?l:ooooo`Co003o2_ooool4o`00o`Cooooo00?o
0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00ooo0003o`3o00?o003o2?ooool3
o`3o00;o003o1?ooO`02ooooo`?oo`000ol0o`03o`00o`gooooo1?ooO`08ooooo`;o003o1?ooO`0:
ooooo`Coogl02oooool3oomo01?ooooo0ol0o`04o`00o`Cooooo0ooo0003o`3o00Co003o1?ooool4
ool000;ooooo0ol00?l3oomo00_ooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`[ooooo1?l00?l:
ooooo`?o003o0oooO`0;ooooo`?oogl00_ooool4ool000;ooooo0ol00?l3oomo00_ooooo0oooO`02
ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`0;ooooo`?oogl00_ooool4ool000;ooooo0ol00?l3
oomo00_ooooo0oooO`08ooooo`Co003o1?ooool3ool000?o0?l01?l00?l4ooooo`Coo`000_ooool3
o`00o`?oogl00_ooool4ool000;ooooo1?l00?l=ooooo`?oogl00_ooool4ool000;ooooo1?l00?l:
ooooo`Co003o2_ooool4o`00o`[ooooo1?l00?l4ooooo`03o`000?oooooooooo00Sooooo000Voooo
o`03o`000?oooooooooo00?oo`000ol0o`03o`00o`Sooooo0ol0o`02o`00o`Coogl00_ooool3ool0
00?o0?l00ol00?l=ooooo`Coogl02?ooool2o`00o`Coogl02_ooool4oomo00_ooooo0oooO`0Coooo
o`?o0?l01?l00?l4ooooo`?oo`000ol0o`04o`00o`Cooooo1?oo0002ooooo`?o003o0oooO`0;oooo
o`?oogl00_ooool3ool000?o0?l01?l00?l:ooooo`Co003o2_ooool4o`00o`gooooo0oooO`02oooo
o`Coo`000_ooool3o`00o`?oogl02oooool3oomo00;ooooo0ooo0003o`3o00Co003o3Oooool3oomo
00_ooooo0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl02oooool3oomo00Sooooo1?l00?l4oooo
o`?oo`000ol0o`04o`00o`Cooooo1?oo0002ooooo`?o003o0oooO`02ooooo`Coo`000_ooool4o`00
o`gooooo0oooO`02ooooo`Coo`000_ooool4o`00o`[ooooo1?l00?l:ooooo`Co003o2_ooool4o`00
o`Cooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00ooo0003o`3o00?o
003o2?ooool3o`3o00;o003o1?ooO`02ooooo`?oo`000ol0o`03o`00o`gooooo1?ooO`0:ooooo`Co
ogl02_ooool4oomo00_ooooo0oooO`0Cooooo`?o0?l01?l00?l4ooooo`?oo`000ol0o`04o`00o`Co
oooo1?oo0002ooooo`?o003o0oooO`0;ooooo`?oogl00_ooool3ool000?o0?l01?l00?l:ooooo`Co
003o2_ooool4o`00o`gooooo0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl02oooool3oomo00;o
oooo0ooo0003o`3o00Co003o3Oooool3oomo00_ooooo0oooO`08ooooo`?o003o0oooO`0;ooooo`?o
ogl02?ooool4o`00o`Cooooo0ooo0003o`3o00Co003o1?ooool4ool000Gooooo0oooO`02ooooo`Co
o`000_ooool4o`00o`gooooo0oooO`02ooooo`Coo`000_ooool4o`00o`[ooooo1?l00?l:ooooo`Co
003o2_ooool4o`00o`Cooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0
0ooo0003o`3o00?o003o2?ooool3o`3o00;o003o1?ooO`02ooooo`Coo`000_ooool3o`00o`gooooo
1?ooO`0:ooooo`Coogl02_ooool4oomo00_ooooo0oooO`0Cooooo`?o0?l01?l00?l4ooooo`?oo`00
0ol0o`04o`00o`Cooooo1?oo0002ooooo`?o003o0oooO`0;ooooo`?oogl00_ooool3ool000?o0?l0
1?l00?l:ooooo`Co003o2_ooool4o`00o`gooooo0oooO`02ooooo`Coo`000_ooool4o`00o`gooooo
0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`0;ooooo`?oogl02?ooool3o`00o`?oogl0
2oooool3oomo00Sooooo1?l00?l4ooooo`?oo`000ol0o`04o`00o`Cooooo1?oo0005ooooo`?oogl0
0_ooool4ool000;ooooo1?l00?l=ooooo`?oogl00_ooool4ool000;ooooo1?l00?l:ooooo`Co003o
2_ooool4o`00o`[ooooo1?l00?l4ooooo`03o`000?oooooooooo00Sooooo000Vooooo`?o00000ooo
0003o`3o00?o003o2oooool2o`00o`Coogl00_ooool4ool000;ooooo0ol00?l=ooooo`Coogl02_oo
ool4oomo00[ooooo1?ooO`0;ooooo`?oogl04oooool3o`3o00Co003o1?ooool3ool000?o0?l01?l0
0?l4ooooo`Coo`000_ooool3o`00o`?oogl02oooool3oomo00;ooooo0ooo0003o`3o00Co003o2_oo
ool4o`00o`[ooooo1?l00?l=ooooo`?oogl00_ooool4ool000;ooooo1?l00?l=ooooo`?oogl00_oo
ool3ool000?o0?l01?l00?l=ooooo`?oogl02oooool3oomo00_ooooo0oooO`0;ooooo`?oogl02?oo
ool4o`00o`Cooooo0ooo0003o`3o00Co003o1?ooool4ool000Gooooo0oooO`02ooooo`Coo`000_oo
ool4o`00o`gooooo0oooO`02ooooo`Coo`000_ooool4o`00o`[ooooo1?l00?l:ooooo`Co003o2_oo
ool4o`00o`?ooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`03ool000?o0?l00ol00?l;
ooooo`;o003o1?ooO`02ooooo`Coo`000_ooool3o`00o`gooooo1?ooO`0:ooooo`Coogl02_ooool4
oomo00_ooooo0oooO`0Cooooo`?o0?l01?l00?l4ooooo`?oo`000ol0o`04o`00o`Cooooo1?oo0002
ooooo`?o003o0oooO`0;ooooo`?oogl00_ooool3ool000?o0?l01?l00?l:ooooo`Co003o2_ooool4
o`00o`gooooo0oooO`02ooooo`Coo`000_ooool4o`00o`gooooo0oooO`02ooooo`?oo`000ol0o`04
o`00o`gooooo0oooO`0;ooooo`?oogl02oooool3oomo00_ooooo0oooO`08ooooo`Co003o1?ooool3
ool000?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00;ooooo1?oo0002ooooo`Co003o3Oooool3
oomo00;ooooo1?oo0002ooooo`Co003o2_ooool4o`00o`[ooooo1?l00?l:ooooo`Co003o1?ooool0
0ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ool000?o0?l00ol00?l;oooo
o`;o003o1?ooO`02ooooo`Coo`000_ooool3o`00o`gooooo1?ooO`0:ooooo`Coogl02_ooool4oomo
00_ooooo0oooO`0Cooooo`Co0?l01oooool3ool000?o0?l01?l00?l4ooooo`Coo`000_ooool3o`00
o`?oogl02oooool3oomo00;ooooo0ooo0003o`3o00Co003o2_ooool4o`00o`[ooooo1?l00?l=oooo
o`?oogl00_ooool4ool000;ooooo1?l00?l=ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=oooo
o`?oogl02oooool3oomo00_ooooo0oooO`0;ooooo`?oogl02?ooool4o`00o`Cooooo0ooo0003o`3o
00Co003o1?ooool4ool000Gooooo0oooO`02ooooo`Coo`000_ooool4o`00o`gooooo0oooO`02oooo
o`Coo`000_ooool4o`00o`[ooooo1?l00?l:ooooo`Co003o2_ooool4o`00o`Cooooo00?o0000oooo
ooooool02?ooool002Kooooo00?o0000ooooooooool00ooo0003o`3o00?o003o2oooool2o`00o`Co
ogl00_ooool4ool000;ooooo0ol00?l=ooooo`Coogl02_ooool4oomo00[ooooo1?ooO`0;ooooo`?o
ogl04oooool4o`3o00Oooooo0ooo0003o`3o00Co003o1?ooool4ool000;ooooo0ol00?l3oomo00_o
oooo0oooO`02ooooo`?oo`000ol0o`04o`00o`[ooooo1?l00?l:ooooo`Co003o3Oooool3oomo00;o
oooo1?oo0002ooooo`Co003o3Oooool3oomo00;ooooo0ooo0003o`3o00Co003o3Oooool3oomo00_o
oooo0oooO`0;ooooo`?oogl02oooool3oomo013ooooo0ooo0003o`3o00Co003o1?ooool4ool000Go
oooo0oooO`02ooooo`Coo`000_ooool4o`00o`gooooo0oooO`02ooooo`Coo`000_ooool4o`00o`[o
oooo1?l00?lHooooo`Co003o1?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooo
ooooo`03ool000?o0?l00ol00?l;ooooo`?o003o1Oooool4ool000;ooooo0ol00?l=ooooo`Coogl0
2_ooool4oomo00[ooooo1?ooO`0;ooooo`?oogl04oooool4o`3o00Oooooo0ooo0003o`3o00Co003o
1?ooool4ool000;ooooo1?l00?l=ooooo`?oogl00_ooool3ool000?o0?l01?l00?l:ooooo`Co003o
2_ooool4o`00o`gooooo0oooO`02ooooo`Coo`004oooool3oomo00;ooooo0ooo0003o`3o00Co003o
3Oooool3oomo00_ooooo0oooO`0;ooooo`?oogl02oooool3oomo013ooooo0ooo0003o`3o00Co003o
1?ooool4ool000Gooooo0oooO`02ooooo`Coo`000_ooool4o`00o`gooooo0oooO`02ooooo`Coo`00
0_ooool4o`00o`[ooooo1?l00?lHooooo`Co003o1?ooool00ol0003oooooooooo`08ooooo`009_oo
ool00ol0003oooooooooo`03ooooo`?o0?l00ol00?l;ooooo`?o003o1Oooool4ool000;ooooo0ol0
0?l=ooooo`Coogl02_ooool4oomo00[ooooo1?ooO`0;ooooo`?oogl04oooool4o`3o00Oooooo0ooo
0003o`3o00Co003o1?ooool4ool000;ooooo1?l00?l=ooooo`?oogl00_ooool3ool000?o0?l01?l0
0?l:ooooo`Co003o2_ooool4o`00o`gooooo0oooO`02ooooo`Coo`004oooool3oomo00;ooooo0ooo
0003o`3o00Co003o3Oooool3oomo00_ooooo0oooO`0;ooooo`?oogl02oooool3oomo013ooooo0ooo
0003o`3o00Co003o1?ooool4ool000Gooooo0oooO`02ooooo`Coo`000_ooool4o`00o`gooooo0ooo
O`02ooooo`Coo`000_ooool4o`00o`[ooooo1?l00?lHooooo`Co003o1?ooool00ol0003ooooooooo
o`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`?o0?l00ol00?l;ooooo`?o003o1Oooool4
ool000;ooooo0ol00?l=ooooo`Coogl02_ooool4oomo00[ooooo1?ooO`0;ooooo`?oogl04oooool4
o`3o00Oooooo0ooo0003o`3o00Co003o1?ooool4ool000;ooooo1?l00?l=ooooo`?oogl00_ooool3
ool000?o0?l01?l00?lHooooo`Co003o3Oooool3oomo00;ooooo1?oo000Cooooo`?oogl00_ooool3
ool000?o0?l01?l00?l=ooooo`?oogl02oooool3oomo00_ooooo0oooO`0;ooooo`?oogl04?ooool3
ool000?o0?l01?l00?l=ooooo`?oogl00_ooool4ool001?ooooo0oooO`02ooooo`Coo`000_ooool4
o`00o`[ooooo1?l00?lHooooo`Co003o1?ooool00ol0003oooooooooo`08ooooo`009_ooool3o`00
00?ooooo1?l0o`0=ooooo`?o003o1Oooool4ool000;ooooo0ol00?l=ooooo`Coogl02_ooool4oomo
00[ooooo1?ooO`0;ooooo`?oogl04oooool4o`3o00Oooooo0ooo0003o`3o00Co003o1?ooool4ool0
00;ooooo1?l00?l=ooooo`?oogl00_ooool3ool000?o0?l01?l00?lHooooo`Co003o3Oooool3oomo
00;ooooo1?oo000Cooooo`?oogl00_ooool3ool000?o0?l01?l00?lKooooo`?oogl02oooool3oomo
00_ooooo0oooO`0@ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`Coo`004oooool3oomo
00;ooooo1?oo0002ooooo`Co003o2_ooool4o`00oaSooooo1?l00?l3ooooo`;o00002_ooool002Ko
oooo00?o0000ooooooooool00oooool4o`3o00gooooo0ol00?l5ooooo`Coo`000_ooool3o`00o`go
oooo1?ooO`0:ooooo`Coogl02_ooool4oomo00_ooooo0oooO`0Cooooo`Co0?l01oooool3ool000?o
0?l01?l00?l4ooooo`Coo`000_ooool4o`00o`gooooo0oooO`02ooooo`?oo`000ol0o`04o`00obWo
oooo0oooO`02ooooo`Coo`004oooool3oomo00;ooooo0ooo0003o`3o00Co003o6oooool3oomo00_o
oooo0oooO`0;ooooo`?oogl04?ooool3ool000?o0?l01?l00?l=ooooo`?oogl00_ooool4ool001?o
oooo0oooO`02ooooo`Coo`000_ooool4o`00o`[ooooo1?l00?lPooooo`03o`000?oooooooooo00So
oooo000Vooooo`03o`000?oooooooooo00?ooooo1?l0o`0=ooooo`?o003o1Oooool4ool000;ooooo
0ol00?l=ooooo`Coogl02_ooool4oomo00[ooooo1?ooO`0;ooooo`?oogl04oooool4o`3o00Oooooo
0ooo0003o`3o00Co003o1?ooool4ool000;ooooo1?l00?l=ooooo`?oogl00_ooool3ool000?o0?l0
1?l00?lYooooo`?oogl00_ooool4ool001?ooooo0oooO`02ooooo`?oo`000ol0o`04o`00oa_ooooo
0oooO`0;ooooo`?oogl02oooool3oomo013ooooo0ooo0003o`3o00Co003o3Oooool3oomo00;ooooo
1?oo000Cooooo`?oogl00_ooool4ool000;ooooo1?l00?l:ooooo`Co003o8?ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l03Oooool3o`00o`Gooooo1?oo
0002ooooo`?o003o3Oooool4oomo00[ooooo1?ooO`0Iooooo`?oogl04oooool4o`3o00Oooooo0ooo
0003o`3o00Co003o1?ooool4ool000;ooooo1?l00?l=ooooo`?oogl00_ooool3ool000?o0?l01?l0
0?lYooooo`?oogl00_ooool4ool001?ooooo0oooO`02ooooo`?oo`000ol0o`04o`00oa_ooooo0ooo
O`0Iooooo`?oogl04?ooool3ool000?o0?l01?l00?l=ooooo`?oogl00_ooool4ool001?ooooo0ooo
O`02ooooo`Coo`000_ooool4o`00o`[ooooo1?l00?lPooooo`03o`000?oooooooooo00Sooooo000V
ooooo`03o`000?oooooooooo00?ooooo1?l0o`0=ooooo`?o003o1Oooool4ool000;ooooo0ol00?l=
ooooo`Coogl02_ooool4oomo01Wooooo0oooO`0Cooooo`Co0?l01oooool3ool000?o0?l01?l00?l4
ooooo`Coo`000_ooool4o`00o`gooooo0oooO`02ooooo`?oo`000ol0o`04o`00obWooooo0oooO`02
ooooo`Coo`004oooool3oomo00;ooooo0ooo0003o`3o00Co003o6oooool3oomo01Wooooo0oooO`0@
ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`Coo`004oooool3oomo00;ooooo1?oo0002
ooooo`Co003o2_ooool4o`00ob3ooooo00?o0000ooooooooool02?ooool000_ooooo0_l00004oooo
o`03o`000?oooooooooo00?ooooo0_l00003ooooo`?o00001oooool00ol0003oooooooooo`03oooo
o`Co0?l03Oooool3o`00o`Gooooo1?oo0002ooooo`?o003o3Oooool4oomo00[ooooo1?ooO`0Ioooo
o`?oogl04oooool4o`3o00Oooooo0ooo0004o`3o00Oooooo1?oo0002ooooo`Co003o3Oooool3oomo
00;ooooo0ooo0003o`3o00Co003o:Oooool3oomo00;ooooo1?oo000Cooooo`?oogl00_ooool3ool0
00?o0?l01?l00?lKooooo`?oogl06Oooool3oomo013ooooo0ooo0003o`3o00Co003o3Oooool3oomo
00;ooooo1?oo000Cooooo`?oogl00_ooool4ool000;ooooo1?l00?l:ooooo`Co003o8?ooool00ol0
003oooooooooo`08ooooo`002_ooool01?l0003oooooooooool00008ooooo`06o`000?oooooooooo
o`000?oooooo00000oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00?ooooo1?l0
o`0=ooooo`?o003o1Oooool4ool000;ooooo0ol00?l=ooooo`Coogl02_ooool4oomo01Wooooo0ooo
O`0Cooooo`Co0?l01oooool3ool000Co0?l01oooool4ool001?ooooo0oooO`02ooooo`?oo`000ol0
o`04o`00obWooooo0oooO`02ooooo`Coo`004oooool3oomo00;ooooo0ooo0003o`3o00Co003o6ooo
ool3oomo01Wooooo0oooO`0@ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`Coo`004ooo
ool3oomo00;ooooo1?oo0002ooooo`Co003o2_ooool4o`00ob3ooooo00?o0000ooooooooool02?oo
ool000[ooooo00Co0000oooooooooooo00002?ooool01?l0003oooooooooool00005ooooo`03o`00
0?oooooooooo00Cooooo1?l00002ooooo`Co0?l05Oooool4ool000;ooooo0ol00?lKooooo`Coogl0
6Oooool3oomo01?ooooo1?l0o`07ooooo`?oo`001?l0o`07ooooo`Coo`004oooool3oomo00;ooooo
0ooo0003o`3o00Co003o:Oooool3oomo00;ooooo1?oo000Cooooo`?oogl00_ooool3ool000?o0?l0
1?l00?lKooooo`?oogl06Oooool3oomo013ooooo0ooo0003o`3o00Co003o3Oooool3oomo00;ooooo
1?oo000Cooooo`?oogl00_ooool4ool000;ooooo1?l00?l:ooooo`Co003o7Oooool4o`0000[ooooo
000:ooooo`04o`000?ooooooooooo`0000Sooooo00Co0000oooooooooooo00001Oooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00?ooooo1?l0o`0Eooooo`Coo`008?ooool4oomo01Wo
oooo0oooO`0Cooooo`Co0?l01oooool3ool000Co0?l01oooool4ool001?ooooo0oooO`02ooooo`?o
o`000ol0o`04o`00obWooooo0oooO`02ooooo`Coo`004oooool3oomo00;ooooo0ooo0003o`3o00Co
003oB_ooool3ool000?o0?l01?l00?lBooooo`Coo`004oooool3oomo00;ooooo1?oo0002ooooo`Co
003o2_ooool4o`00ob3ooooo00?o0000ooooooooool02?ooool000[ooooo00Co0000oooooooooooo
00002?ooool01Ol0003oooooooooool0003ooooo00Co00001oooool00ol0003oooooooooo`03oooo
o`Co0?l05Oooool4ool0023ooooo1?ooO`0Iooooo`?oogl04oooool4o`3o00Oooooo0ooo0004o`3o
00Oooooo1?oo000Cooooo`?oogl00_ooool3ool000?o0?l01?l00?lYooooo`?oogl00_ooool4ool0
01Sooooo0ooo0003o`3o00Co003oCOooool3o`3o00Co003o4_ooool4ool001?ooooo0oooO`02oooo
o`Coo`00=?ooool00ol0003oooooooooo`08ooooo`002_ooool01?l0003oooooooooool00008oooo
o`06o`000?ooooooooooo`000?oooooo00002_ooool00ol0003oooooooooo`03ooooo`Co0?l05Ooo
ool4ool0023ooooo1?ooO`0Iooooo`?oogl04oooool4o`3o00Oooooo0ooo0004o`3o00Oooooo1?oo
000Cooooo`?oogl00_ooool3ool000?o0?l01?l00?lYooooo`?oogl00_ooool4ool001Sooooo0ooo
0003o`3o00Co003oCOooool3o`3o00Co003o4_ooool4ool001?ooooo0oooO`02ooooo`Coo`00=?oo
ool00ol0003oooooooooo`08ooooo`002oooool2o`0000[ooooo0_l00002ooooo`Go00001_ooool0
0ol0003oooooooooo`03ooooo`Co0?l05Oooool4ool0023ooooo1?ooO`0Iooooo`?oogl04oooool4
o`3o00Oooooo0ooo0004o`3o00Oooooo1?oo000Hooooo`?oo`001?l0o`0/ooooo`?oogl00_ooool4
ool001Sooooo0ooo0003o`3o00Co003oCOooool3o`3o00Co003o4_ooool4ool001?ooooo0oooO`02
ooooo`Coo`00=?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03oooo
o`Co0?l05Oooool4ool0023ooooo1?ooO`0Iooooo`?oogl04oooool4o`3o00Oooooo0ooo0004o`3o
00Oooooo1?oo000Hooooo`?oo`001?l0o`0aooooo`Coo`006?ooool3ool000?o0?l01?l00?m=oooo
o`?o0?l01?l00?lBooooo`Coo`004oooool3oomo00;ooooo1?oo000dooooo`03o`000?oooooooooo
00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo1?l0o`0Eooooo`Coo`00?Oooool3oomo01?o
oooo1?l0o`07ooooo`Coo`002_ooool4ool001Sooooo0ooo0004o`3o037ooooo1?oo000Hooooo`?o
o`001?l0o`1@ooooo`?o0?l01?l00?lBooooo`Coo`004oooool3oomo00;ooooo1?oo000dooooo`03
o`000?oooooooooo00Sooooo000Vooooo`?o00000oooool4o`3o01Gooooo1?oo001Cooooo`Co0?l0
1oooool4ool000[ooooo1?oo000Hooooo`?oo`001?l0o`0aooooo`Coo`006?ooool3ool000Co0?l0
D?ooool4o`3o01Gooooo1?oo000Cooooo`?oogl00_ooool4ool003?ooooo0_l0000:ooooo`009_oo
ool00ol0003oooooooooo`03ooooo`Co0?l05Oooool4ool005?ooooo1?l0o`07ooooo`Coo`002_oo
ool4ool001Sooooo0ooo0004o`3o037ooooo1?oo000Hooooo`?oo`001?l0o`1@ooooo`Co0?l05Ooo
ool4ool001?ooooo0oooO`02ooooo`Coo`00=?ooool00ol0003oooooooooo`08ooooo`009_ooool0
0ol0003oooooooooo`03ooooo`Co0?l05Oooool4ool005?ooooo1?l0o`07ooooo`Coo`002_ooool4
ool001Sooooo0ooo0004o`3o037ooooo1?oo000Hooooo`?oo`001?l0o`1@ooooo`Co0?l05Oooool4
ool001?ooooo0oooO`02ooooo`Coo`00=?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0
003oooooooooo`03ooooo`Co0?l05Oooool4ool005?ooooo1?l0o`07ooooo`Coo`002_ooool4ool0
01Sooooo0ooo0004o`3o037ooooo1?oo000Hooooo`?oo`001?l0o`1@ooooo`Co0?l05Oooool4ool0
01?ooooo0oooO`02ooooo`Coo`00=?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003o
ooooooooo`03ooooo`Co0?l05Oooool4ool005?ooooo1?l0o`07ooooo`Coo`002_ooool4ool001So
oooo0ooo0004o`3o037ooooo1?oo000Hooooo`?oo`001?l0o`1@ooooo`Co0?l05Oooool4ool001?o
oooo0oooO`02ooooo`Coo`00=?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooo
ooooo`03ooooo`Co0?l05Oooool4ool005?ooooo1?l0o`07ooooo`Coo`002_ooool4ool001Sooooo
0ooo0004o`3o037ooooo1?oo000Hooooo`?oo`001?l0o`1@ooooo`Co0?l05Oooool4ool001?ooooo
0oooO`02ooooo`Coo`00=?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooooooo
o`03ooooo`Co0?l05Oooool4ool005?ooooo1?l0o`07ooooo`Coo`002_ooool4ool001Sooooo0ooo
0004o`3o037ooooo1?oo000Hooooo`Coo`00Doooool4o`3o01Gooooo1?oo000Cooooo`?oogl00_oo
ool4ool003Cooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool4
o`3o01Gooooo1?oo001Cooooo`Co0?l01oooool4ool000[ooooo1?oo000Hooooo`?oo`001?l0o`0a
ooooo`Coo`006?ooool4ool005?ooooo1?l0o`0Eooooo`Coo`004oooool3oomo00;ooooo1?oo000d
ooooo`03o`000?oooooooooo00Sooooo000Vooooo`?o00000oooool4o`3o01Gooooo1?oo001Coooo
o`Co0?l01oooool4ool000[ooooo1?oo000Hooooo`?oo`001?l0o`0aooooo`Coo`006?ooool4ool0
05?ooooo1?l0o`0Eooooo`Coo`004oooool3oomo00;ooooo1?oo000cooooo`;o00002_ooool002Ko
oooo00?o0000ooooooooool00oooool4o`3o01Gooooo1?oo001Cooooo`Co0?l01oooool4ool000[o
oooo1?oo000Hooooo`?oo`001?l0o`0aooooo`Coo`006?ooool4ool005?ooooo1?l0o`0Eooooo`Co
o`004oooool3oomo00;ooooo1?oo000dooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`00
0?oooooooooo00?ooooo1?l0o`0Eooooo`Coo`00Doooool4o`3o00Oooooo1?oo000:ooooo`Coo`00
6?ooool3ool000Co0?l0<Oooool4ool001Sooooo1?oo001Cooooo`Co0?l05Oooool4ool001?ooooo
0oooO`02ooooo`Coo`00=?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooooooo
o`03ooooo`Co0?l05Oooool4ool005?ooooo1?l0o`07ooooo`Coo`002_ooool4ool001Sooooo0ooo
0004o`3o037ooooo1?oo000Hooooo`Coo`00Doooool4o`3o01Gooooo1?oo000Hooooo`Coo`00=?oo
ool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l05Oooool4
ool005?ooooo1?l0o`07ooooo`Coo`002_ooool4ool001Sooooo0ooo0004o`3o037ooooo1?oo000H
ooooo`Coo`00Doooool4o`3o01Gooooo1?oo000Hooooo`Coo`00=?ooool00ol0003oooooooooo`08
ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l05Oooool4ool005?ooooo1?l0o`07oooo
o`Coo`002_ooool4ool001Sooooo0ooo0004o`3o037ooooo1?oo000Hooooo`Coo`00Doooool4o`3o
01Gooooo1?oo000Hooooo`Coo`00=?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003o
ooooooooo`03ooooo`Co0?l05Oooool4ool005?ooooo1?l0o`07ooooo`Coo`002_ooool4ool001So
oooo0ooo0004o`3o037ooooo1?oo000Hooooo`Coo`00Doooool4o`3o037ooooo1?oo000dooooo`03
o`000?oooooooooo00Sooooo000Vooooo`?o00000oooool4o`3o01Gooooo1?oo001Cooooo`Co0?l0
1oooool4ool000[ooooo1?oo000Hooooo`?oo`001?l0o`0aooooo`Coo`006?ooool4ool005?ooooo
1?l0o`0aooooo`Coo`00<oooool2o`0000[ooooo000Vooooo`03o`000?oooooooooo00?ooooo1?l0
o`0Eooooo`Coo`00Doooool4o`3o00Oooooo1?oo000:ooooo`Coo`006?ooool3ool000Co0?l0<Ooo
ool4ool001Sooooo1?oo001Cooooo`Co0?l0<Oooool4ool003Cooooo00?o0000ooooooooool02?oo
ool002Kooooo00?o0000ooooooooool00oooool4o`3o06cooooo1?l0o`07ooooo`Coo`002_ooool4
ool001Sooooo1?oo000dooooo`Coo`006?ooool4ool005?ooooo1?l0o`0aooooo`Coo`00=?ooool0
0ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0K?ooool4o`3o
037ooooo1?oo000dooooo`Coo`006?ooool4ool005?ooooo1?l0o`0aooooo`Coo`00=?ooool00ol0
003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0K?ooool4o`3o037o
oooo1?oo000dooooo`Coo`006?ooool4ool005?ooooo1?l0o`0aooooo`Coo`00=?ooool00ol0003o
ooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0K?ooool4o`3o037ooooo
1?oo000dooooo`Coo`006?ooool4ool005?ooooo1?l0o`0aooooo`Coo`00=?ooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0K?ooool4o`3o037ooooo1?oo
000dooooo`Coo`006?ooool4ool005?ooooo1?l0o`0aooooo`Coo`00=?ooool00ol0003ooooooooo
o`08ooooo`009_ooool3o`0000?ooooo1?l0o`1/ooooo`Co0?l0<Oooool4ool003Cooooo1?oo000H
ooooo`Coo`00Doooool4o`3o037ooooo1?oo000cooooo`;o00002_ooool002Kooooo00?o0000oooo
ooooool00oooool4o`3o06cooooo1?l0o`0aooooo`Coo`00=?ooool4ool001Sooooo1?oo001Coooo
o`Co0?l0<Oooool4ool003Cooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooooooo
ool00oooool4o`3o06cooooo1?l0o`0aooooo`Coo`00=?ooool4ool001Sooooo1?oo001Cooooo`Co
0?l0<Oooool4ool003Cooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0
0oooool4o`3o06cooooo1?l0o`0aooooo`Coo`00=?ooool4ool001Sooooo1?oo001Cooooo`Co0?l0
<Oooool4ool003Cooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00ooo
ool4o`3o06cooooo1?l0o`0aooooo`Coo`00D?ooool4ool005?ooooo1?l0o`0aooooo`Coo`00=?oo
ool00ol0003oooooooooo`08ooooo`004Oooool2o`0000Cooooo00?o0000ooooooooool00_ooool3
o`0000Oooooo00?o0000ooooooooool00oooool4o`3o06cooooo1?l0o`0aooooo`Coo`00D?ooool4
ool005?ooooo1?l0o`0aooooo`Coo`00=?ooool00ol0003oooooooooo`08ooooo`004?ooool01?l0
003oooooooooool00009ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool00oooool4
o`3o06cooooo1?l0o`0aooooo`Coo`00D?ooool4ool005?ooooo1?l0o`0aooooo`Coo`00=?ooool0
0ol0003oooooooooo`08ooooo`004?ooool01?l0003oooooooooool00009ooooo`03o`000?oooooo
oooo00Kooooo00?o0000ooooooooool00oooool4o`3o06cooooo1?l0o`0aooooo`Coo`00D?ooool4
ool005?ooooo1?l0o`0aooooo`Coo`00=?ooool00ol0003oooooooooo`08ooooo`004?ooool01?l0
003oooooooooool00009ooooo`03o`000?oooooooooo00Kooooo1?l00002ooooo`Co0?l0K?ooool4
o`3o037ooooo1?oo001@ooooo`Coo`00Doooool4o`3o037ooooo1?oo000aooooo`Co00002_ooool0
013ooooo00Co0000oooooooooooo00002Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooo
oooo00?ooooo1?l0o`1/ooooo`Co0?l0<Oooool4ool0053ooooo1?oo001Cooooo`Co0?l0<Oooool4
ool003Cooooo00?o0000ooooooooool02?ooool0013ooooo00Co0000oooooooooooo00002?ooool2
o`0000Sooooo00?o0000ooooooooool00oooool4o`3o06cooooo1?l0o`0aooooo`Coo`00D?ooool4
ool005?ooooo1?l0o`0aooooo`Coo`00=?ooool00ol0003oooooooooo`08ooooo`004Oooool2o`00
00[ooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`03ooooo`Co0?l0K?ooool4o`3o
037ooooo1?oo001@ooooo`Coo`00Doooool4o`3o037ooooo1?oo000dooooo`03o`000?oooooooooo
00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo1?l0o`1/ooooo`Co0?l0<Oooool4ool0053o
oooo1?oo001Cooooo`Co0?l0<Oooool4ool003Cooooo00?o0000ooooooooool02?ooool002Kooooo
00?o0000ooooooooool00oooool4o`3o06cooooo1?l0o`0aooooo`Coo`00D?ooool4ool005?ooooo
1?l0o`0aooooo`Coo`00=?ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooooooo
o`03ooooo`Co0?l0K?ooool4o`3o037ooooo1?oo001@ooooo`Coo`00Doooool4o`3o06Wooooo00?o
0000ooooooooool02?ooool002Kooooo0ol00003ooooo`Co0?l0K?ooool4o`3o037ooooo1?oo001@
ooooo`Coo`00Doooool4o`3o06Sooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`03oooo
o`Co0?l0K?ooool4o`3o037ooooo1?oo001@ooooo`Coo`00Doooool4o`3o06Wooooo00?o0000oooo
ooooool02?ooool002Kooooo00?o0000ooooooooool00oooool4o`3o06cooooo1?l0o`0aooooo`Co
o`00D?ooool4ool005?ooooo1?l0o`1Yooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`00
0?oooooooooo00?ooooo1?l0o`1/ooooo`Co0?l0<Oooool4ool0053ooooo1?oo001Cooooo`Co0?l0
JOooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0K?oo
ool4o`3o037ooooo1?oo001@ooooo`Coo`00Doooool4o`3o06Wooooo00?o0000ooooooooool02?oo
ool002Kooooo00?o0000ooooooooool00oooool4o`3o06cooooo1?l0o`0aooooo`Coo`00D?ooool4
ool005?ooooo1?l0o`1Yooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo
00?ooooo1?l0o`1/ooooo`Co0?l0<Oooool4ool0053ooooo1?oo001Cooooo`Co0?l0JOooool00ol0
003oooooooooo`08ooooo`009_ooool3o`0000?ooooo1?l0o`1/ooooo`Co0?l0<Oooool4ool0053o
oooo1?oo001Cooooo`Co0?l0J?ooool2o`0000[ooooo000Vooooo`03o`000?oooooooooo00?ooooo
1?l0o`1/ooooo`Co0?l0<Oooool4ool0053ooooo1?oo001Cooooo`Co0?l0JOooool00ol0003ooooo
ooooo`08ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0K?ooool4o`3o037ooooo1?oo
001@ooooo`Coo`00Doooool4o`3o06Wooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000
ooooooooool00oooool4o`3o06cooooo1?l0o`0aooooo`Coo`00D?ooool4ool005?ooooo1?l0o`1Y
ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo1?l0o`1/oooo
o`Co0?l0<Oooool4ool0053ooooo1?oo001Cooooo`Co0?l0JOooool00ol0003oooooooooo`08oooo
o`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0K?ooool4o`3o037ooooo1?oo001@ooooo`Co
o`00Doooool4o`3o06Wooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0
0oooool4o`3o06cooooo1?l0o`0aooooo`Coo`00D?ooool4ool005?ooooo1?l0o`1Yooooo`03o`00
0?oooooooooo00Sooooo000Vooooo`?o00000oooool4o`3o06cooooo1?l0o`0aooooo`Coo`00Yooo
ool4o`3o06Sooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0K?ooool4
o`3o037ooooo1?oo002Wooooo`Co0?l0JOooool00ol0003oooooooooo`08ooooo`009_ooool00ol0
003oooooooooo`03ooooo`Co0?l0K?ooool4o`3o0=cooooo1?l0o`1Yooooo`03o`000?oooooooooo
00Sooooo000Vooooo`03o`000?oooooooooo00?ooooo1?l0o`1/ooooo`Co0?l0g?ooool4o`3o06Wo
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool4o`3o06cooooo
1?l0o`3Looooo`Co0?l0JOooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooooooo
o`03ooooo`Co0?l0K?ooool4o`3o0=cooooo1?l0o`1Yooooo`03o`000?oooooooooo00Sooooo000V
ooooo`03o`000?oooooooooo00?ooooo1?l0o`1/ooooo`Co0?l0g?ooool4o`3o06Wooooo00?o0000
ooooooooool02?ooool002Kooooo00?o0000ooooooooool00oooool4o`3o06cooooo1?l0o`3Loooo
o`Co0?l0JOooool00ol0003oooooooooo`08ooooo`009_ooool3o`0000?ooooo1?l0o`1/ooooo`Co
0?l0g?ooool4o`3o06Sooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0
K?ooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooooooo
o`03ooooo`Co0?l0K?ooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool0
0ol0003oooooooooo`03ooooo`Co0?l0K?ooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08
ooooo`009_ooool00ol0003oooooooooo`03ooooo`Co0?l0K?ooool4o`3o0?ooooooB_ooool00ol0
003oooooooooo`08ooooo`002oooool2o`0000Cooooo00?o0000ooooooooool00_ooool3o`0000?o
oooo0ol00007ooooo`03o`000?oooooooooo00?ooooo1?l0o`1/ooooo`Co0?l0oooooom:ooooo`03
o`000?oooooooooo00Sooooo000:ooooo`04o`000?ooooooooooo`0000Wooooo00Go0000oooooooo
ooooooooo`000003ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00oooool4o`3o
06cooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool000[ooooo00Co0000oooooooo
oooo00002Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo1?l0001boooo
o`Co0?l0oooooom7ooooo`Co00002_ooool000[ooooo00Co0000oooooooooooo00002Oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0Loooool4o`3o
0?ooooooB_ooool00ol0003oooooooooo`08ooooo`002_ooool01?l0003oooooooooool00009oooo
o`04o`000?ooooooooooooooo`Co00001oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:
ooooo`03o`000?oooooooooo00Sooooo000:ooooo`04o`000?ooooooooooo`0000Sooooo0_l00003
ooooo`03o`000?oooooooooo00Sooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool0
0ol0003oooooooooo`08ooooo`002oooool2o`0000[ooooo00Co0000oooooooooooooooo1Ol00006
ooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool0
02Kooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08oooo
o`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00So
oooo000Vooooo`?o0000Loooool4o`3o0?ooooooBOooool2o`0000[ooooo000Vooooo`03o`000?oo
oooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000
ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0
003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03
o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool002Kooooo
00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_oo
ool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00Sooooo000V
ooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool0
02Kooooo0ol0001cooooo`Co0?l0oooooom9ooooo`;o00002_ooool002Kooooo00?o0000oooooooo
ool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooo
ooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oo
oooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000
ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0
003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03
o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool002Kooooo
0ol0001cooooo`Co0?l0oooooom9ooooo`;o00002_ooool002Kooooo00?o0000ooooooooool0Looo
ool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1c
ooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo
07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooooooo
ool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooo
ooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oo
oooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool002Kooooo0ol0001c
ooooo`Co0?l0oooooom9ooooo`;o00002_ooool002Kooooo00?o0000ooooooooool0Loooool4o`3o
0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co
0?l0oooooom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo
1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Looo
ool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`004Oooool2o`0000Cooooo00Co0000
oooooooooooooooo1Ol00006ooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o
0000ooooooooool02?ooool0013ooooo00Co0000oooooooooooo00002?ooool01?l0003ooooooooo
ool00006ooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool0
2?ooool0013ooooo00Co0000oooooooooooo00002Oooool00ol0003oooooooooo`06ooooo`Co0000
L_ooool4o`3o0?ooooooAoooool4o`0000[ooooo000@ooooo`04o`000?ooooooooooo`0000[ooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`00
0?oooooooooo00Sooooo000@ooooo`04o`000?ooooooooooo`0000Oooooo00Go0000oooooooooooo
ooooo`000006ooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000oooooooo
ool02?ooool0013ooooo00Co0000oooooooooooo00001oooool01Ol0003ooooooooooooooooo0000
00Kooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08oooo
o`004Oooool2o`0000Wooooo0ol00007ooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[o
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Loooool4o`3o0?oooooo
B_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooo
oom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0o`3o
oooood[ooooo00?o0000ooooooooool02?ooool002Kooooo0ol0001cooooo`Co0?l0oooooom9oooo
o`;o00002_ooool002Kooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003o
ooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`00
0?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o
0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool0
0ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:oooo
o`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[o
oooo00?o0000ooooooooool02?ooool002Kooooo0ol0001cooooo`Co0?l0oooooom9ooooo`;o0000
2_ooool002Kooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003ooooooooo
o`08ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooo
oooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000oooo
ooooool02?ooool002Kooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003o
ooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`00
0?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o
0000ooooooooool02?ooool002Kooooo0ol0001cooooo`Co0?l0oooooom9ooooo`;o00002_ooool0
02Kooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08oooo
o`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00So
oooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool0
2?ooool002Kooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003ooooooooo
o`08ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooo
oooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000oooo
ooooool02?ooool002Kooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003o
ooooooooo`08ooooo`009_ooool3o`0007?ooooo1?l0o`3oooooodWooooo0_l0000:ooooo`009_oo
ool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00Sooooo000V
ooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool0
02Kooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08oooo
o`002oooool2o`0000Cooooo00Co0000oooooooooooooooo1Ol00002ooooo`?o00001oooool00ol0
003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00Sooooo000:ooooo`04
o`000?ooooooooooo`0000Sooooo00Ko0000oooooooooooo0000ooooool00003ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003ooooooooo
o`08ooooo`002_ooool01?l0003oooooooooool00009ooooo`03o`000?oooooooooo00Gooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oo
oooooooo00Sooooo000:ooooo`04o`000?ooooooooooo`0000[ooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`04ooooo`Co0000L_ooool4o`3o0?ooooooAoooool4o`0000[ooooo000:
ooooo`04o`000?ooooooooooo`0000Oooooo00Ko0000ooooooooooooooooo`000?ooool4o`0000Oo
oooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`00
2_ooool01?l0003oooooooooool00007ooooo`07o`000?ooooooooooooooool0003oooooo`00000:
ooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool0
00_ooooo0_l00009ooooo`?o00000_ooool5o`0000Kooooo00?o0000ooooooooool0Loooool4o`3o
0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co
0?l0oooooom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo
1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Looo
ool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool3o`0007?ooooo1?l0o`3o
ooooodWooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:oooo
o`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[o
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Loooool4o`3o0?oooooo
B_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooo
oom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0o`3o
oooood[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Loooool4o`3o
0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool3o`0007?ooooo1?l0o`3oooooodWo
oooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`00
0?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o
0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool0
0ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:oooo
o`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[o
oooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Loooool4o`3o0?oooooo
B_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooo
oom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`?o0000Loooool4o`3o0?ooooooBOooool2
o`0000[ooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000oooo
ooooool02?ooool002Kooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003o
ooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`00
0?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o
0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool0
0ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:oooo
o`03o`000?oooooooooo00Sooooo000Vooooo`?o0000Loooool4o`3o0?ooooooBOooool2o`0000[o
oooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool0
2?ooool002Kooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003ooooooooo
o`08ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooo
oooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000oooo
ooooool02?ooool0017ooooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0ol00007ooooo`03
o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool0013ooooo
00Co0000oooooooooooo00001oooool01Ol0003ooooooooooooooooo000000Kooooo00?o0000oooo
ooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`004?ooool01?l0003o
ooooooooool0000;ooooo`03o`000?oooooooooo00Cooooo1?l0001booooo`Co0?l0oooooom7oooo
o`Co00002_ooool0013ooooo00Co0000oooooooooooo00002Oooool2o`0000Oooooo00?o0000oooo
ooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`004?ooool01?l0003o
ooooooooool0000;ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0Loooool4o`3o
0?ooooooB_ooool00ol0003oooooooooo`08ooooo`004?ooool01?l0003oooooooooool00007oooo
o`05o`000?ooooooooooooooool000001_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:
ooooo`03o`000?oooooooooo00Sooooo000Aooooo`;o00002Oooool3o`0000Oooooo00?o0000oooo
ooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003o
ooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`00
0?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o
0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool3
o`0007?ooooo1?l0o`3oooooodWooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`1coooo
o`Co0?l0oooooom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?o
oooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0
Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooooooo
o`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooo
oooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooo
ooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool3o`0007?o
oooo1?l0o`3oooooodWooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0
oooooom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0
o`3ooooood[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Loooool4
o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1coooo
o`Co0?l0oooooom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?o
oooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0
Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool3o`0007?ooooo1?l0
o`3oooooodWooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:
ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooo
od[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Loooool4o`3o0?oo
ooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0
oooooom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0
o`3ooooood[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Loooool4
o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool3o`0007?ooooo1?l0o`3ooooo
odWooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo
00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_oo
ool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:
ooooo`03o`000?oooooooooo00Sooooo000;ooooo`;o00001?ooool00ol0003oooooooooo`02oooo
o`?o00000oooool3o`0000Oooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0
003oooooooooo`08ooooo`002_ooool01?l0003oooooooooool00007ooooo`07o`000?oooooooooo
ooooool0003oooooo`000003ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0Looo
ool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`002_ooool01?l0003oooooooooool0
000;ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?ooool00ol0003ooooooooo
o`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00Sooooo000:ooooo`04o`000?oooooo
ooooo`0000Wooooo0_l00006ooooo`03o`000?oooooooooo00Cooooo1?l0001booooo`Co0?l0oooo
oom7ooooo`Co00002_ooool000[ooooo00Co0000oooooooooooo00002oooool00ol0003oooooo`00
0003o`0000Oooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003ooooooooo
o`08ooooo`002_ooool01?l0003oooooooooool00007ooooo`07o`000?ooooooooooooooool0003o
ooooo`00000:ooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000oooooooo
ool02?ooool000_ooooo0_l00009ooooo`?o00000_ooool5o`0000Kooooo00?o0000ooooooooool0
Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003ooooooooo
o`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooo
oooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooo
ooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool3o`0007?o
oooo1?l0o`3oooooodWooooo0_l0000:ooooo`009_ooool00ol0003oooooooooo`1cooooo`Co0?l0
oooooom:ooooo`03o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo07?ooooo1?l0
o`3ooooood[ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0Loooool4
o`3o0?ooooooB_ooool00ol0003oooooooooo`08ooooo`009_ooool00ol0003oooooooooo`3ooooo
ol7ooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000ooooooooool0ooooooo1ooooo`03
o`000?oooooooooo00Sooooo000Vooooo`03o`000?oooooooooo0?oooooo`Oooool00ol0003ooooo
ooooo`08ooooo`009_ooool3o`0000Kooooo00?o0000ooooooooool02oooool00ol0003ooooooooo
o`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003ooooooooo
o`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003ooooooooo
o`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003ooooooooo
o`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003ooooooooo
o`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003ooooooooo
o`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003ooooooooo
o`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003ooooooooo
o`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003ooooooooo
o`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003ooooooooo
o`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003ooooooooo
o`04ooooo`;o00002_ooool002Kooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool02?ooool002Kooooo00?o0000oooo
ooooool01_ooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool02?ooool002Koooooool00035o`0000[ooooo003oooooooGooooo003oooooooGooooo003o
ooooooGooooo003oooooooGooooo003oooooooGooooo003oooooooGooooo003oooooooGooooo000]
ooooo`?o00002_ooool5o`0000[ooooo0ol0000<ooooo`?o00002_ooool3o`0000_ooooo0ol0000<
ooooo`03o`000?oooooooooo00[ooooo0ol0000;ooooo`;o00002Oooool3o`0000Cooooo0_l00005
ooooo`?o00000oooool3o`0000Gooooo0ol00002ooooo`Go00001?ooool3o`0000?ooooo0ol00005
ooooo`?o00001?ooool3o`0000Cooooo0ol00003ooooo`?o00001Oooool3o`0000?ooooo0ol00005
ooooo`?o00001?ooool00ol0003oooooooooo`04ooooo`?o00000oooool3o`0000Gooooo0ol00003
ooooo`;o00001Oooool5o`0000?ooooo0_l00004ooooo`Go00000_ooool3o`0000Cooooo1Ol00000
0ooooooo0000o`000003o`0000?ooooo1Ol00002ooooo`?o00001?ooool5o`0000?ooooo0ol00003
ooooo`Go00000_ooool3o`0000Cooooo1Ol00002ooooo`?o00001?ooool5o`0000?ooooo00?o0000
ooooooooool00oooool5o`0000;ooooo0ol00004ooooo`Go00000_ooool2o`0000Kooooo0ol00004
ooooo`;o00001Oooool3o`0000?ooooo0ol00005ooooo`?o00000_ooool5o`0000kooooo000^oooo
o`03o`000?oooooooooo00[ooooo00Co0000oooooooooooo00002Oooool01Ol0003ooooooooooooo
oooo000000cooooo00?o0000ooooooooool02?ooool01Ol0003ooooooooooooooooo000000Wooooo
00Go0000ooooooooooooooooo`00000;ooooo`03o`000?oooooooooo00Wooooo00Go0000oooooooo
ooooooooo`00000<ooooo`03o`000?oooooooooo00Oooooo00?o0000ooooooooool00_ooool01?l0
003oooooooooool00005ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Gooooo00Go0000oooooooooooooooo
o`000003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?ooool00ol0003ooooo
ooooo`04ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03ooooo`05
o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
00?ooooo00?o0000ooooooooool01Oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000
ooooooooool00oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00?ooooo00Co0000
oooooooooooo00000_ooool01?l0003oooooooooool00004ooooo`04o`000?ooooooooooo`0000?o
oooo00?o0000ooooooooool01?ooool01?l0003oooooooooool00002ooooo`04o`000?oooooooooo
o`0000Cooooo00Ko0000oooooooooooo0000ooooool00003ooooo`03o`000?oooooooooo00;ooooo
00Co0000oooooooooooo00001?ooool00ol0003oooooooooo`03ooooo`06o`000?ooooooooooo`00
0?oooooo00000oooool00ol0003oooooooooo`02ooooo`06o`000?ooooooooooo`000?oooooo0000
0oooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000?ooooo00?o0000oooooooo
ool01?ooool01_l0003oooooooooool0003oooooo`0000?ooooo00?o0000ooooooooool00_ooool0
1?l0003oooooooooool00004ooooo`03o`000?oooooooooo00;ooooo00Go0000oooooooooooooooo
o`000002ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000002ooooo`04o`000?oooooo
ooooo`0000kooooo000^ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool03Oooool0
0ol0003oooooooooo`07ooooo`Go00003Oooool00ol0003oooooooooo`07ooooo`05o`000?oooooo
ooooooooool000003?ooool00ol0003oooooooooo`08ooooo`05o`000?ooooooooooooooool00000
3Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo
00001Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool00oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool00oooool01?l0003oooooooooooooool5o`0000Gooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`03ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooooooo
oooo000000?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`03ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`03ooooo`04o`00
0?ooooooooooooooo`Go00001Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00?o
oooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`04ooooo`05o`000?ooooooooooooooool000000oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool01Oooool0
1?l0003oooooooooool00002ooooo`03o`000?oooooooooo00Gooooo00Go0000oooooooooooooooo
o`000009ooooo`05o`000?ooooooooooooooool000004?ooool002kooooo00?o0000ooooooooool0
3?ooool00ol0003oooooooooo`0:ooooo`;o00002_ooool01?l0003oooooooooool0000>ooooo`03
o`000?oooooooooo00Oooooo1?l0000=ooooo`03o`000?oooooooooo00Wooooo0ol0000;ooooo`Co
00002?ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`03ooooo`;o00001_ooool01Ol0003ooooooooooooooooo
000000;ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo00?ooooo00Co0000oooooooooooooooo1?l00006ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01?ooool00ol0003oooooooooo`02ooooo`?o00001_ooool00ol0003ooooooooo
o`02ooooo`Co00001_ooool01Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`02ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool0
0oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;ooooo0_l00007ooooo`04o`00
0?ooooooooooo`0000;ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01?l00007ooooo`03o`000?oooooooooo00?o
oooo00?o0000ooooooooool01Oooool01?l0003oooooooooooooool3o`0000Oooooo00Co0000oooo
oooooooooooo1?l00005ooooo`;o00000oooool01?l0003oooooooooool00005ooooo`;o00001?oo
ool00ol0003oooooooooo`05ooooo`;o00001Oooool00ol0003oooooooooo`0=ooooo`00;_ooool0
0ol0003oooooooooo`09ooooo`05o`000?ooooooooooooooool000003Oooool00ol0003ooooooooo
o`08ooooo`03o`000?oooooo000000[ooooo1?l0000:ooooo`03o`000?oooooooooo00oooooo00?o
0000ooooooooool01oooool01Ol0003ooooooooooooooooo000000Wooooo00Go0000oooooooooooo
ooooo`000008ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00001Oooool00ol0
003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00Go0000ooooooooooooooooo`000003
ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`03
ooooo`03o`000?oooooooooo00;ooooo00?o0000ooooool000001_ooool01?l0003ooooooooooooo
ool4o`0000Kooooo00Go0000ooooooooooooooooo`000009ooooo`03o`000?oooooooooo00Gooooo
00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000oooooooo
ool00oooool01Ol0003ooooooooooooooooo000000?ooooo00Go0000ooooooooooooooooo`000003
ooooo`04o`000?ooooooooooo`0000;ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`00
0?ooooooooooooooool000001Oooool01ol0003ooooooooooooooooo0000ooooool000000oooool0
1Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool00oooool01Ol0003ooooooooo
oooooooo000000?ooooo00Ko0000oooooooooooo0000ooooool00004ooooo`06o`000?oooooooooo
ooooool0003ooooo1?l00004ooooo`07o`000?ooooooooooooooool0003oooooo`000007ooooo`05
o`000?ooooooooooooooool000001Oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000
ooooool000000oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooool000000ooo
ool00ol0003oooooooooo`05ooooo`04o`000?ooooooooooo`0000;ooooo00?o0000ooooooooool0
1Oooool01Ol0003ooooooooooooooooo000000Wooooo00?o0000ooooool000000oooool00ol0003o
ooooooooo`0<ooooo`00;Oooool2o`0000_ooooo00Go0000ooooooooooooooooo`000009ooooo`05
o`000?ooooooooooooooool000002oooool2o`0000[ooooo00?o0000ooooooooool03?ooool00ol0
003oooooooooo`0:ooooo`05o`000?ooooooooooooooool000002Oooool01Ol0003ooooooooooooo
oooo000000Wooooo00Go0000ooooooooooooooooo`000007ooooo`;o00001?ooool01?l0003ooooo
ooooool00004ooooo`;o00001?ooool2o`0000Kooooo0_l00003ooooo`05o`000?oooooooooooooo
ool000001?ooool2o`0000?ooooo00Go0000ooooooooooooooooo`000004ooooo`;o00001Oooool2
o`0000Gooooo0_l00003ooooo`03o`000?oooooooooo00Kooooo0_l00004ooooo`03o`000?oooooo
oooo00Gooooo0_l00003ooooo`05o`000?ooooooooooooooool000001?ooool2o`0000?ooooo00Go
0000ooooooooooooooooo`000004ooooo`;o00000oooool01Ol0003ooooooooooooooooo000000?o
oooo00Go0000ooooooooooooooooo`000002ooooo`04o`000?ooooooooooo`0000?ooooo00Go0000
ooooooooooooooooo`000002ooooo`;o00001Oooool01ol0003ooooooooooooooooo0000ooooool0
00000oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooool000000oooool01Ol0
003ooooooooooooooooo000000?ooooo00Co0000oooooooooooooooo0_l00004ooooo`07o`000?oo
ooooooooooooool0003oooooo`000007ooooo`05o`000?ooooooooooooooool000000_ooool00ol0
003oooooooooo`04ooooo`07o`000?ooooooooooooooool0003oooooo`000003ooooo`05o`000?oo
ooooooooooooool000000oooool00ol0003oooooo`000003ooooo`05o`000?ooooooooooooooool0
00000oooool00ol0003oooooo`000003ooooo`05o`000?ooooooooooooooool000000oooool01?l0
003oooooooooool00002ooooo`05o`000?ooooooooooooooool000000oooool00ol0003ooooooooo
o`02o`0000Gooooo00Oo0000ooooooooooooooooo`000?oooooo000000?ooooo00?o0000oooooooo
ool03?ooool002kooooo00?o0000ooooooooool02_ooool3o`0000_ooooo0ol0000=ooooo`03o`00
0?oooooooooo00Sooooo1Ol0000;ooooo`;o00002_ooool5o`0000[ooooo0ol0000;ooooo`?o0000
2Oooool00ol0003oooooooooo`03ooooo`;o00001_ooool00ol0003oooooooooo`03ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool00_ooool3o`0000Kooooo00?o0000ooooooooool0
0_ooool3o`0000Kooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04ooooo`04o`00
0?ooooooooooooooo`Go00001Oooool00ol0003oooooooooo`03ooooo`;o00001_ooool01?l0003o
ooooooooooooool5o`0000Gooooo00?o0000ooooooooool00_ooool3o`0000Kooooo00?o0000oooo
ooooool00_ooool3o`0000Gooooo0ol00004ooooo`;o00001Oooool3o`0000Cooooo00?o0000oooo
ooooool01?ooool3o`0000?ooooo0ol00005ooooo`?o00000oooool3o`0000Gooooo0ol00005oooo
o`03o`000?oooooooooo00?ooooo0ol00002ooooo`Go00001?ooool3o`0000Cooooo0_l00005oooo
o`?o00000_ooool5o`0000Cooooo0ol00003ooooo`?o00001Oooool3o`0000?ooooo0ol00005oooo
o`?o00001?ooool2o`0000Gooooo0ol00004ooooo`03o`000?oooooooooo00Cooooo0ol00003oooo
o`?o00003oooool00001\
\>"],
  ImageRangeCache->{{{0, 499}, {308, 0}} -> {-2.06768, -0.0292005, 0.0713992, \
0.00137252}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
%%ImageSize: 500 309.017 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0217391 0.0310559 0.0147151 1.09104 [
[.01863 -0.0125 -3 -9 ]
[.01863 -0.0125 3 0 ]
[.04969 -0.0125 -3 -9 ]
[.04969 -0.0125 3 0 ]
[.08075 -0.0125 -3 -9 ]
[.08075 -0.0125 3 0 ]
[.1118 -0.0125 -3 -9 ]
[.1118 -0.0125 3 0 ]
[.14286 -0.0125 -3 -9 ]
[.14286 -0.0125 3 0 ]
[.17391 -0.0125 -3 -9 ]
[.17391 -0.0125 3 0 ]
[.20497 -0.0125 -3 -9 ]
[.20497 -0.0125 3 0 ]
[.23602 -0.0125 -3 -9 ]
[.23602 -0.0125 3 0 ]
[.26708 -0.0125 -3 -9 ]
[.26708 -0.0125 3 0 ]
[.29814 -0.0125 -6 -9 ]
[.29814 -0.0125 6 0 ]
[.32919 -0.0125 -6 -9 ]
[.32919 -0.0125 6 0 ]
[.36025 -0.0125 -6 -9 ]
[.36025 -0.0125 6 0 ]
[.3913 -0.0125 -6 -9 ]
[.3913 -0.0125 6 0 ]
[.42236 -0.0125 -6 -9 ]
[.42236 -0.0125 6 0 ]
[.45342 -0.0125 -6 -9 ]
[.45342 -0.0125 6 0 ]
[.48447 -0.0125 -6 -9 ]
[.48447 -0.0125 6 0 ]
[.51553 -0.0125 -6 -9 ]
[.51553 -0.0125 6 0 ]
[.54658 -0.0125 -6 -9 ]
[.54658 -0.0125 6 0 ]
[.57764 -0.0125 -6 -9 ]
[.57764 -0.0125 6 0 ]
[.6087 -0.0125 -6 -9 ]
[.6087 -0.0125 6 0 ]
[.63975 -0.0125 -6 -9 ]
[.63975 -0.0125 6 0 ]
[.67081 -0.0125 -6 -9 ]
[.67081 -0.0125 6 0 ]
[.70186 -0.0125 -6 -9 ]
[.70186 -0.0125 6 0 ]
[.73292 -0.0125 -6 -9 ]
[.73292 -0.0125 6 0 ]
[.76398 -0.0125 -6 -9 ]
[.76398 -0.0125 6 0 ]
[.79503 -0.0125 -6 -9 ]
[.79503 -0.0125 6 0 ]
[.82609 -0.0125 -6 -9 ]
[.82609 -0.0125 6 0 ]
[.85714 -0.0125 -6 -9 ]
[.85714 -0.0125 6 0 ]
[.8882 -0.0125 -6 -9 ]
[.8882 -0.0125 6 0 ]
[.91925 -0.0125 -6 -9 ]
[.91925 -0.0125 6 0 ]
[.95031 -0.0125 -6 -9 ]
[.95031 -0.0125 6 0 ]
[.98137 -0.0125 -6 -9 ]
[.98137 -0.0125 6 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .01472 -6 -4.5 ]
[-0.0125 .01472 0 4.5 ]
[-0.0125 .12382 -18 -4.5 ]
[-0.0125 .12382 0 4.5 ]
[-0.0125 .23292 -18 -4.5 ]
[-0.0125 .23292 0 4.5 ]
[-0.0125 .34203 -18 -4.5 ]
[-0.0125 .34203 0 4.5 ]
[-0.0125 .45113 -18 -4.5 ]
[-0.0125 .45113 0 4.5 ]
[-0.0125 .56024 -18 -4.5 ]
[-0.0125 .56024 0 4.5 ]
[ 0 0 -0.125 0 ]
[.01863 .63053 -3 0 ]
[.01863 .63053 3 9 ]
[.04969 .63053 -3 0 ]
[.04969 .63053 3 9 ]
[.08075 .63053 -3 0 ]
[.08075 .63053 3 9 ]
[.1118 .63053 -3 0 ]
[.1118 .63053 3 9 ]
[.14286 .63053 -3 0 ]
[.14286 .63053 3 9 ]
[.17391 .63053 -3 0 ]
[.17391 .63053 3 9 ]
[.20497 .63053 -3 0 ]
[.20497 .63053 3 9 ]
[.23602 .63053 -3 0 ]
[.23602 .63053 3 9 ]
[.26708 .63053 -3 0 ]
[.26708 .63053 3 9 ]
[.29814 .63053 -6 0 ]
[.29814 .63053 6 9 ]
[.32919 .63053 -6 0 ]
[.32919 .63053 6 9 ]
[.36025 .63053 -6 0 ]
[.36025 .63053 6 9 ]
[.3913 .63053 -6 0 ]
[.3913 .63053 6 9 ]
[.42236 .63053 -6 0 ]
[.42236 .63053 6 9 ]
[.45342 .63053 -6 0 ]
[.45342 .63053 6 9 ]
[.48447 .63053 -6 0 ]
[.48447 .63053 6 9 ]
[.51553 .63053 -6 0 ]
[.51553 .63053 6 9 ]
[.54658 .63053 -6 0 ]
[.54658 .63053 6 9 ]
[.57764 .63053 -6 0 ]
[.57764 .63053 6 9 ]
[.6087 .63053 -6 0 ]
[.6087 .63053 6 9 ]
[.63975 .63053 -6 0 ]
[.63975 .63053 6 9 ]
[.67081 .63053 -6 0 ]
[.67081 .63053 6 9 ]
[.70186 .63053 -6 0 ]
[.70186 .63053 6 9 ]
[.73292 .63053 -6 0 ]
[.73292 .63053 6 9 ]
[.76398 .63053 -6 0 ]
[.76398 .63053 6 9 ]
[.79503 .63053 -6 0 ]
[.79503 .63053 6 9 ]
[.82609 .63053 -6 0 ]
[.82609 .63053 6 9 ]
[.85714 .63053 -6 0 ]
[.85714 .63053 6 9 ]
[.8882 .63053 -6 0 ]
[.8882 .63053 6 9 ]
[.91925 .63053 -6 0 ]
[.91925 .63053 6 9 ]
[.95031 .63053 -6 0 ]
[.95031 .63053 6 9 ]
[.98137 .63053 -6 0 ]
[.98137 .63053 6 9 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.01863 0 m
.01863 .00625 L
s
[(1)] .01863 -0.0125 0 1 Mshowa
.04969 0 m
.04969 .00625 L
s
[(2)] .04969 -0.0125 0 1 Mshowa
.08075 0 m
.08075 .00625 L
s
[(3)] .08075 -0.0125 0 1 Mshowa
.1118 0 m
.1118 .00625 L
s
[(4)] .1118 -0.0125 0 1 Mshowa
.14286 0 m
.14286 .00625 L
s
[(5)] .14286 -0.0125 0 1 Mshowa
.17391 0 m
.17391 .00625 L
s
[(6)] .17391 -0.0125 0 1 Mshowa
.20497 0 m
.20497 .00625 L
s
[(7)] .20497 -0.0125 0 1 Mshowa
.23602 0 m
.23602 .00625 L
s
[(8)] .23602 -0.0125 0 1 Mshowa
.26708 0 m
.26708 .00625 L
s
[(9)] .26708 -0.0125 0 1 Mshowa
.29814 0 m
.29814 .00625 L
s
[(10)] .29814 -0.0125 0 1 Mshowa
.32919 0 m
.32919 .00625 L
s
[(11)] .32919 -0.0125 0 1 Mshowa
.36025 0 m
.36025 .00625 L
s
[(12)] .36025 -0.0125 0 1 Mshowa
.3913 0 m
.3913 .00625 L
s
[(13)] .3913 -0.0125 0 1 Mshowa
.42236 0 m
.42236 .00625 L
s
[(14)] .42236 -0.0125 0 1 Mshowa
.45342 0 m
.45342 .00625 L
s
[(15)] .45342 -0.0125 0 1 Mshowa
.48447 0 m
.48447 .00625 L
s
[(16)] .48447 -0.0125 0 1 Mshowa
.51553 0 m
.51553 .00625 L
s
[(17)] .51553 -0.0125 0 1 Mshowa
.54658 0 m
.54658 .00625 L
s
[(18)] .54658 -0.0125 0 1 Mshowa
.57764 0 m
.57764 .00625 L
s
[(19)] .57764 -0.0125 0 1 Mshowa
.6087 0 m
.6087 .00625 L
s
[(20)] .6087 -0.0125 0 1 Mshowa
.63975 0 m
.63975 .00625 L
s
[(21)] .63975 -0.0125 0 1 Mshowa
.67081 0 m
.67081 .00625 L
s
[(22)] .67081 -0.0125 0 1 Mshowa
.70186 0 m
.70186 .00625 L
s
[(23)] .70186 -0.0125 0 1 Mshowa
.73292 0 m
.73292 .00625 L
s
[(24)] .73292 -0.0125 0 1 Mshowa
.76398 0 m
.76398 .00625 L
s
[(25)] .76398 -0.0125 0 1 Mshowa
.79503 0 m
.79503 .00625 L
s
[(26)] .79503 -0.0125 0 1 Mshowa
.82609 0 m
.82609 .00625 L
s
[(27)] .82609 -0.0125 0 1 Mshowa
.85714 0 m
.85714 .00625 L
s
[(28)] .85714 -0.0125 0 1 Mshowa
.8882 0 m
.8882 .00625 L
s
[(29)] .8882 -0.0125 0 1 Mshowa
.91925 0 m
.91925 .00625 L
s
[(30)] .91925 -0.0125 0 1 Mshowa
.95031 0 m
.95031 .00625 L
s
[(31)] .95031 -0.0125 0 1 Mshowa
.98137 0 m
.98137 .00625 L
s
[(32)] .98137 -0.0125 0 1 Mshowa
0 0 m
1 0 L
s
0 .01472 m
.00625 .01472 L
s
[(0)] -0.0125 .01472 1 0 Mshowa
0 .12382 m
.00625 .12382 L
s
[(0.1)] -0.0125 .12382 1 0 Mshowa
0 .23292 m
.00625 .23292 L
s
[(0.2)] -0.0125 .23292 1 0 Mshowa
0 .34203 m
.00625 .34203 L
s
[(0.3)] -0.0125 .34203 1 0 Mshowa
0 .45113 m
.00625 .45113 L
s
[(0.4)] -0.0125 .45113 1 0 Mshowa
0 .56024 m
.00625 .56024 L
s
[(0.5)] -0.0125 .56024 1 0 Mshowa
.125 Mabswid
0 .03654 m
.00375 .03654 L
s
0 .05836 m
.00375 .05836 L
s
0 .08018 m
.00375 .08018 L
s
0 .102 m
.00375 .102 L
s
0 .14564 m
.00375 .14564 L
s
0 .16746 m
.00375 .16746 L
s
0 .18928 m
.00375 .18928 L
s
0 .2111 m
.00375 .2111 L
s
0 .25474 m
.00375 .25474 L
s
0 .27657 m
.00375 .27657 L
s
0 .29839 m
.00375 .29839 L
s
0 .32021 m
.00375 .32021 L
s
0 .36385 m
.00375 .36385 L
s
0 .38567 m
.00375 .38567 L
s
0 .40749 m
.00375 .40749 L
s
0 .42931 m
.00375 .42931 L
s
0 .47295 m
.00375 .47295 L
s
0 .49477 m
.00375 .49477 L
s
0 .51659 m
.00375 .51659 L
s
0 .53842 m
.00375 .53842 L
s
0 .58206 m
.00375 .58206 L
s
0 .60388 m
.00375 .60388 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.01863 .61178 m
.01863 .61803 L
s
[(1)] .01863 .63053 0 -1 Mshowa
.04969 .61178 m
.04969 .61803 L
s
[(2)] .04969 .63053 0 -1 Mshowa
.08075 .61178 m
.08075 .61803 L
s
[(3)] .08075 .63053 0 -1 Mshowa
.1118 .61178 m
.1118 .61803 L
s
[(4)] .1118 .63053 0 -1 Mshowa
.14286 .61178 m
.14286 .61803 L
s
[(5)] .14286 .63053 0 -1 Mshowa
.17391 .61178 m
.17391 .61803 L
s
[(6)] .17391 .63053 0 -1 Mshowa
.20497 .61178 m
.20497 .61803 L
s
[(7)] .20497 .63053 0 -1 Mshowa
.23602 .61178 m
.23602 .61803 L
s
[(8)] .23602 .63053 0 -1 Mshowa
.26708 .61178 m
.26708 .61803 L
s
[(9)] .26708 .63053 0 -1 Mshowa
.29814 .61178 m
.29814 .61803 L
s
[(10)] .29814 .63053 0 -1 Mshowa
.32919 .61178 m
.32919 .61803 L
s
[(11)] .32919 .63053 0 -1 Mshowa
.36025 .61178 m
.36025 .61803 L
s
[(12)] .36025 .63053 0 -1 Mshowa
.3913 .61178 m
.3913 .61803 L
s
[(13)] .3913 .63053 0 -1 Mshowa
.42236 .61178 m
.42236 .61803 L
s
[(14)] .42236 .63053 0 -1 Mshowa
.45342 .61178 m
.45342 .61803 L
s
[(15)] .45342 .63053 0 -1 Mshowa
.48447 .61178 m
.48447 .61803 L
s
[(16)] .48447 .63053 0 -1 Mshowa
.51553 .61178 m
.51553 .61803 L
s
[(17)] .51553 .63053 0 -1 Mshowa
.54658 .61178 m
.54658 .61803 L
s
[(18)] .54658 .63053 0 -1 Mshowa
.57764 .61178 m
.57764 .61803 L
s
[(19)] .57764 .63053 0 -1 Mshowa
.6087 .61178 m
.6087 .61803 L
s
[(20)] .6087 .63053 0 -1 Mshowa
.63975 .61178 m
.63975 .61803 L
s
[(21)] .63975 .63053 0 -1 Mshowa
.67081 .61178 m
.67081 .61803 L
s
[(22)] .67081 .63053 0 -1 Mshowa
.70186 .61178 m
.70186 .61803 L
s
[(23)] .70186 .63053 0 -1 Mshowa
.73292 .61178 m
.73292 .61803 L
s
[(24)] .73292 .63053 0 -1 Mshowa
.76398 .61178 m
.76398 .61803 L
s
[(25)] .76398 .63053 0 -1 Mshowa
.79503 .61178 m
.79503 .61803 L
s
[(26)] .79503 .63053 0 -1 Mshowa
.82609 .61178 m
.82609 .61803 L
s
[(27)] .82609 .63053 0 -1 Mshowa
.85714 .61178 m
.85714 .61803 L
s
[(28)] .85714 .63053 0 -1 Mshowa
.8882 .61178 m
.8882 .61803 L
s
[(29)] .8882 .63053 0 -1 Mshowa
.91925 .61178 m
.91925 .61803 L
s
[(30)] .91925 .63053 0 -1 Mshowa
.95031 .61178 m
.95031 .61803 L
s
[(31)] .95031 .63053 0 -1 Mshowa
.98137 .61178 m
.98137 .61803 L
s
[(32)] .98137 .63053 0 -1 Mshowa
0 .61803 m
1 .61803 L
s
.99375 .01472 m
1 .01472 L
s
.99375 .12382 m
1 .12382 L
s
.99375 .23292 m
1 .23292 L
s
.99375 .34203 m
1 .34203 L
s
.99375 .45113 m
1 .45113 L
s
.99375 .56024 m
1 .56024 L
s
.125 Mabswid
.99625 .03654 m
1 .03654 L
s
.99625 .05836 m
1 .05836 L
s
.99625 .08018 m
1 .08018 L
s
.99625 .102 m
1 .102 L
s
.99625 .14564 m
1 .14564 L
s
.99625 .16746 m
1 .16746 L
s
.99625 .18928 m
1 .18928 L
s
.99625 .2111 m
1 .2111 L
s
.99625 .25474 m
1 .25474 L
s
.99625 .27657 m
1 .27657 L
s
.99625 .29839 m
1 .29839 L
s
.99625 .32021 m
1 .32021 L
s
.99625 .36385 m
1 .36385 L
s
.99625 .38567 m
1 .38567 L
s
.99625 .40749 m
1 .40749 L
s
.99625 .42931 m
1 .42931 L
s
.99625 .47295 m
1 .47295 L
s
.99625 .49477 m
1 .49477 L
s
.99625 .51659 m
1 .51659 L
s
.99625 .53842 m
1 .53842 L
s
.99625 .58206 m
1 .58206 L
s
.99625 .60388 m
1 .60388 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .01472 m
1 .01472 L
s
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.00621 .01472 m
.00621 .01472 L
.01242 .01472 L
.01242 .01472 L
F
.5 Mabswid
.00621 .01472 m
.00621 .01472 L
.01242 .01472 L
.01242 .01472 L
.00621 .01472 L
s
.03727 .01472 m
.03727 .08288 L
.04348 .08288 L
.04348 .01472 L
F
.03727 .01472 m
.03727 .08288 L
.04348 .08288 L
.04348 .01472 L
.03727 .01472 L
s
.06832 .01472 m
.06832 .01472 L
.07453 .01472 L
.07453 .01472 L
F
.06832 .01472 m
.06832 .01472 L
.07453 .01472 L
.07453 .01472 L
.06832 .01472 L
s
.09938 .01472 m
.09938 .01586 L
.10559 .01586 L
.10559 .01472 L
F
.09938 .01472 m
.09938 .01586 L
.10559 .01586 L
.10559 .01472 L
.09938 .01472 L
s
.13043 .01472 m
.13043 .01472 L
.13665 .01472 L
.13665 .01472 L
F
.13043 .01472 m
.13043 .01472 L
.13665 .01472 L
.13665 .01472 L
.13043 .01472 L
s
.16149 .01472 m
.16149 .18283 L
.1677 .18283 L
.1677 .01472 L
F
.16149 .01472 m
.16149 .18283 L
.1677 .18283 L
.1677 .01472 L
.16149 .01472 L
s
.19255 .01472 m
.19255 .01472 L
.19876 .01472 L
.19876 .01472 L
F
.19255 .01472 m
.19255 .01472 L
.19876 .01472 L
.19876 .01472 L
.19255 .01472 L
s
.2236 .01472 m
.2236 .01472 L
.22981 .01472 L
.22981 .01472 L
F
.2236 .01472 m
.2236 .01472 L
.22981 .01472 L
.22981 .01472 L
.2236 .01472 L
s
.25466 .01472 m
.25466 .01472 L
.26087 .01472 L
.26087 .01472 L
F
.25466 .01472 m
.25466 .01472 L
.26087 .01472 L
.26087 .01472 L
.25466 .01472 L
s
.28571 .01472 m
.28571 .0419 L
.29193 .0419 L
.29193 .01472 L
F
.28571 .01472 m
.28571 .0419 L
.29193 .0419 L
.29193 .01472 L
.28571 .01472 L
s
.31677 .01472 m
.31677 .01472 L
.32298 .01472 L
.32298 .01472 L
F
.31677 .01472 m
.31677 .01472 L
.32298 .01472 L
.32298 .01472 L
.31677 .01472 L
s
.34783 .01472 m
.34783 .17402 L
.35404 .17402 L
.35404 .01472 L
F
.34783 .01472 m
.34783 .17402 L
.35404 .17402 L
.35404 .01472 L
.34783 .01472 L
s
.37888 .01472 m
.37888 .01472 L
.38509 .01472 L
.38509 .01472 L
F
.37888 .01472 m
.37888 .01472 L
.38509 .01472 L
.38509 .01472 L
.37888 .01472 L
s
.40994 .01472 m
.40994 .10433 L
.41615 .10433 L
.41615 .01472 L
F
.40994 .01472 m
.40994 .10433 L
.41615 .10433 L
.41615 .01472 L
.40994 .01472 L
s
.44099 .01472 m
.44099 .01472 L
.4472 .01472 L
.4472 .01472 L
F
.44099 .01472 m
.44099 .01472 L
.4472 .01472 L
.4472 .01472 L
.44099 .01472 L
s
.47205 .01472 m
.47205 .01548 L
.47826 .01548 L
.47826 .01472 L
F
.47205 .01472 m
.47205 .01548 L
.47826 .01548 L
.47826 .01472 L
.47205 .01472 L
s
.50311 .01472 m
.50311 .01472 L
.50932 .01472 L
.50932 .01472 L
F
.50311 .01472 m
.50311 .01472 L
.50932 .01472 L
.50932 .01472 L
.50311 .01472 L
s
.53416 .01472 m
.53416 .09092 L
.54037 .09092 L
.54037 .01472 L
F
.53416 .01472 m
.53416 .09092 L
.54037 .09092 L
.54037 .01472 L
.53416 .01472 L
s
.56522 .01472 m
.56522 .01472 L
.57143 .01472 L
.57143 .01472 L
F
.56522 .01472 m
.56522 .01472 L
.57143 .01472 L
.57143 .01472 L
.56522 .01472 L
s
.59627 .01472 m
.59627 .1162 L
.60248 .1162 L
.60248 .01472 L
F
.59627 .01472 m
.59627 .1162 L
.60248 .1162 L
.60248 .01472 L
.59627 .01472 L
s
.62733 .01472 m
.62733 .01472 L
.63354 .01472 L
.63354 .01472 L
F
.62733 .01472 m
.62733 .01472 L
.63354 .01472 L
.63354 .01472 L
.62733 .01472 L
s
.65839 .01472 m
.65839 .13496 L
.6646 .13496 L
.6646 .01472 L
F
.65839 .01472 m
.65839 .13496 L
.6646 .13496 L
.6646 .01472 L
.65839 .01472 L
s
.68944 .01472 m
.68944 .01472 L
.69565 .01472 L
.69565 .01472 L
F
.68944 .01472 m
.68944 .01472 L
.69565 .01472 L
.69565 .01472 L
.68944 .01472 L
s
.7205 .01472 m
.7205 .01472 L
.72671 .01472 L
.72671 .01472 L
F
.7205 .01472 m
.7205 .01472 L
.72671 .01472 L
.72671 .01472 L
.7205 .01472 L
s
.75155 .01472 m
.75155 .01472 L
.75776 .01472 L
.75776 .01472 L
F
.75155 .01472 m
.75155 .01472 L
.75776 .01472 L
.75776 .01472 L
.75155 .01472 L
s
.78261 .01472 m
.78261 .08058 L
.78882 .08058 L
.78882 .01472 L
F
.78261 .01472 m
.78261 .08058 L
.78882 .08058 L
.78882 .01472 L
.78261 .01472 L
s
.81366 .01472 m
.81366 .01472 L
.81988 .01472 L
.81988 .01472 L
F
.81366 .01472 m
.81366 .01472 L
.81988 .01472 L
.81988 .01472 L
.81366 .01472 L
s
.84472 .01472 m
.84472 .0779 L
.85093 .0779 L
.85093 .01472 L
F
.84472 .01472 m
.84472 .0779 L
.85093 .0779 L
.85093 .01472 L
.84472 .01472 L
s
.87578 .01472 m
.87578 .01472 L
.88199 .01472 L
.88199 .01472 L
F
.87578 .01472 m
.87578 .01472 L
.88199 .01472 L
.88199 .01472 L
.87578 .01472 L
s
.90683 .01472 m
.90683 .16445 L
.91304 .16445 L
.91304 .01472 L
F
.90683 .01472 m
.90683 .16445 L
.91304 .16445 L
.91304 .01472 L
.90683 .01472 L
s
.93789 .01472 m
.93789 .01472 L
.9441 .01472 L
.9441 .01472 L
F
.93789 .01472 m
.93789 .01472 L
.9441 .01472 L
.9441 .01472 L
.93789 .01472 L
s
.96894 .01472 m
.96894 .01472 L
.97516 .01472 L
.97516 .01472 L
F
.96894 .01472 m
.96894 .01472 L
.97516 .01472 L
.97516 .01472 L
.96894 .01472 L
s
0 1 0 r
.01242 .01472 m
.01242 .01472 L
.01863 .01472 L
.01863 .01472 L
F
.01242 .01472 m
.01242 .01472 L
.01863 .01472 L
.01863 .01472 L
.01242 .01472 L
s
.04348 .01472 m
.04348 .0802 L
.04969 .0802 L
.04969 .01472 L
F
.04348 .01472 m
.04348 .0802 L
.04969 .0802 L
.04969 .01472 L
.04348 .01472 L
s
.07453 .01472 m
.07453 .01472 L
.08075 .01472 L
.08075 .01472 L
F
.07453 .01472 m
.07453 .01472 L
.08075 .01472 L
.08075 .01472 L
.07453 .01472 L
s
.10559 .01472 m
.10559 .0151 L
.1118 .0151 L
.1118 .01472 L
F
.10559 .01472 m
.10559 .0151 L
.1118 .0151 L
.1118 .01472 L
.10559 .01472 L
s
.13665 .01472 m
.13665 .01472 L
.14286 .01472 L
.14286 .01472 L
F
.13665 .01472 m
.13665 .01472 L
.14286 .01472 L
.14286 .01472 L
.13665 .01472 L
s
.1677 .01472 m
.1677 .03425 L
.17391 .03425 L
.17391 .01472 L
F
.1677 .01472 m
.1677 .03425 L
.17391 .03425 L
.17391 .01472 L
.1677 .01472 L
s
.19876 .01472 m
.19876 .01472 L
.20497 .01472 L
.20497 .01472 L
F
.19876 .01472 m
.19876 .01472 L
.20497 .01472 L
.20497 .01472 L
.19876 .01472 L
s
.22981 .01472 m
.22981 .01472 L
.23602 .01472 L
.23602 .01472 L
F
.22981 .01472 m
.22981 .01472 L
.23602 .01472 L
.23602 .01472 L
.22981 .01472 L
s
.26087 .01472 m
.26087 .01472 L
.26708 .01472 L
.26708 .01472 L
F
.26087 .01472 m
.26087 .01472 L
.26708 .01472 L
.26708 .01472 L
.26087 .01472 L
s
.29193 .01472 m
.29193 .60332 L
.29814 .60332 L
.29814 .01472 L
F
.29193 .01472 m
.29193 .60332 L
.29814 .60332 L
.29814 .01472 L
.29193 .01472 L
s
.32298 .01472 m
.32298 .01472 L
.32919 .01472 L
.32919 .01472 L
F
.32298 .01472 m
.32298 .01472 L
.32919 .01472 L
.32919 .01472 L
.32298 .01472 L
s
.35404 .01472 m
.35404 .02505 L
.36025 .02505 L
.36025 .01472 L
F
.35404 .01472 m
.35404 .02505 L
.36025 .02505 L
.36025 .01472 L
.35404 .01472 L
s
.38509 .01472 m
.38509 .01472 L
.3913 .01472 L
.3913 .01472 L
F
.38509 .01472 m
.38509 .01472 L
.3913 .01472 L
.3913 .01472 L
.38509 .01472 L
s
.41615 .01472 m
.41615 .08326 L
.42236 .08326 L
.42236 .01472 L
F
.41615 .01472 m
.41615 .08326 L
.42236 .08326 L
.42236 .01472 L
.41615 .01472 L
s
.4472 .01472 m
.4472 .01472 L
.45342 .01472 L
.45342 .01472 L
F
.4472 .01472 m
.4472 .01472 L
.45342 .01472 L
.45342 .01472 L
.4472 .01472 L
s
.47826 .01472 m
.47826 .0151 L
.48447 .0151 L
.48447 .01472 L
F
.47826 .01472 m
.47826 .0151 L
.48447 .0151 L
.48447 .01472 L
.47826 .01472 L
s
.50932 .01472 m
.50932 .01472 L
.51553 .01472 L
.51553 .01472 L
F
.50932 .01472 m
.50932 .01472 L
.51553 .01472 L
.51553 .01472 L
.50932 .01472 L
s
.54037 .01472 m
.54037 .12233 L
.54658 .12233 L
.54658 .01472 L
F
.54037 .01472 m
.54037 .12233 L
.54658 .12233 L
.54658 .01472 L
.54037 .01472 L
s
.57143 .01472 m
.57143 .01472 L
.57764 .01472 L
.57764 .01472 L
F
.57143 .01472 m
.57143 .01472 L
.57764 .01472 L
.57764 .01472 L
.57143 .01472 L
s
.60248 .01472 m
.60248 .01625 L
.6087 .01625 L
.6087 .01472 L
F
.60248 .01472 m
.60248 .01625 L
.6087 .01625 L
.6087 .01472 L
.60248 .01472 L
s
.63354 .01472 m
.63354 .01472 L
.63975 .01472 L
.63975 .01472 L
F
.63354 .01472 m
.63354 .01472 L
.63975 .01472 L
.63975 .01472 L
.63354 .01472 L
s
.6646 .01472 m
.6646 .02123 L
.67081 .02123 L
.67081 .01472 L
F
.6646 .01472 m
.6646 .02123 L
.67081 .02123 L
.67081 .01472 L
.6646 .01472 L
s
.69565 .01472 m
.69565 .01472 L
.70186 .01472 L
.70186 .01472 L
F
.69565 .01472 m
.69565 .01472 L
.70186 .01472 L
.70186 .01472 L
.69565 .01472 L
s
.72671 .01472 m
.72671 .01472 L
.73292 .01472 L
.73292 .01472 L
F
.72671 .01472 m
.72671 .01472 L
.73292 .01472 L
.73292 .01472 L
.72671 .01472 L
s
.75776 .01472 m
.75776 .01472 L
.76398 .01472 L
.76398 .01472 L
F
.75776 .01472 m
.75776 .01472 L
.76398 .01472 L
.76398 .01472 L
.75776 .01472 L
s
.78882 .01472 m
.78882 .12347 L
.79503 .12347 L
.79503 .01472 L
F
.78882 .01472 m
.78882 .12347 L
.79503 .12347 L
.79503 .01472 L
.78882 .01472 L
s
.81988 .01472 m
.81988 .01472 L
.82609 .01472 L
.82609 .01472 L
F
.81988 .01472 m
.81988 .01472 L
.82609 .01472 L
.82609 .01472 L
.81988 .01472 L
s
.85093 .01472 m
.85093 .09246 L
.85714 .09246 L
.85714 .01472 L
F
.85093 .01472 m
.85093 .09246 L
.85714 .09246 L
.85714 .01472 L
.85093 .01472 L
s
.88199 .01472 m
.88199 .01472 L
.8882 .01472 L
.8882 .01472 L
F
.88199 .01472 m
.88199 .01472 L
.8882 .01472 L
.8882 .01472 L
.88199 .01472 L
s
.91304 .01472 m
.91304 .05033 L
.91925 .05033 L
.91925 .01472 L
F
.91304 .01472 m
.91304 .05033 L
.91925 .05033 L
.91925 .01472 L
.91304 .01472 L
s
.9441 .01472 m
.9441 .01472 L
.95031 .01472 L
.95031 .01472 L
F
.9441 .01472 m
.9441 .01472 L
.95031 .01472 L
.95031 .01472 L
.9441 .01472 L
s
.97516 .01472 m
.97516 .01472 L
.98137 .01472 L
.98137 .01472 L
F
.97516 .01472 m
.97516 .01472 L
.98137 .01472 L
.98137 .01472 L
.97516 .01472 L
s
0 0 1 r
.01863 .01472 m
.01863 .01472 L
.02484 .01472 L
.02484 .01472 L
F
.01863 .01472 m
.01863 .01472 L
.02484 .01472 L
.02484 .01472 L
.01863 .01472 L
s
.04969 .01472 m
.04969 .12922 L
.0559 .12922 L
.0559 .01472 L
F
.04969 .01472 m
.04969 .12922 L
.0559 .12922 L
.0559 .01472 L
.04969 .01472 L
s
.08075 .01472 m
.08075 .01472 L
.08696 .01472 L
.08696 .01472 L
F
.08075 .01472 m
.08075 .01472 L
.08696 .01472 L
.08696 .01472 L
.08075 .01472 L
s
.1118 .01472 m
.1118 .01472 L
.11801 .01472 L
.11801 .01472 L
F
.1118 .01472 m
.1118 .01472 L
.11801 .01472 L
.11801 .01472 L
.1118 .01472 L
s
.14286 .01472 m
.14286 .01472 L
.14907 .01472 L
.14907 .01472 L
F
.14286 .01472 m
.14286 .01472 L
.14907 .01472 L
.14907 .01472 L
.14286 .01472 L
s
.17391 .01472 m
.17391 .0622 L
.18012 .0622 L
.18012 .01472 L
F
.17391 .01472 m
.17391 .0622 L
.18012 .0622 L
.18012 .01472 L
.17391 .01472 L
s
.20497 .01472 m
.20497 .01472 L
.21118 .01472 L
.21118 .01472 L
F
.20497 .01472 m
.20497 .01472 L
.21118 .01472 L
.21118 .01472 L
.20497 .01472 L
s
.23602 .01472 m
.23602 .01472 L
.24224 .01472 L
.24224 .01472 L
F
.23602 .01472 m
.23602 .01472 L
.24224 .01472 L
.24224 .01472 L
.23602 .01472 L
s
.26708 .01472 m
.26708 .01472 L
.27329 .01472 L
.27329 .01472 L
F
.26708 .01472 m
.26708 .01472 L
.27329 .01472 L
.27329 .01472 L
.26708 .01472 L
s
.29814 .01472 m
.29814 .32376 L
.30435 .32376 L
.30435 .01472 L
F
.29814 .01472 m
.29814 .32376 L
.30435 .32376 L
.30435 .01472 L
.29814 .01472 L
s
.32919 .01472 m
.32919 .01472 L
.3354 .01472 L
.3354 .01472 L
F
.32919 .01472 m
.32919 .01472 L
.3354 .01472 L
.3354 .01472 L
.32919 .01472 L
s
.36025 .01472 m
.36025 .04459 L
.36646 .04459 L
.36646 .01472 L
F
.36025 .01472 m
.36025 .04459 L
.36646 .04459 L
.36646 .01472 L
.36025 .01472 L
s
.3913 .01472 m
.3913 .01472 L
.39752 .01472 L
.39752 .01472 L
F
.3913 .01472 m
.3913 .01472 L
.39752 .01472 L
.39752 .01472 L
.3913 .01472 L
s
.42236 .01472 m
.42236 .1162 L
.42857 .1162 L
.42857 .01472 L
F
.42236 .01472 m
.42236 .1162 L
.42857 .1162 L
.42857 .01472 L
.42236 .01472 L
s
.45342 .01472 m
.45342 .01472 L
.45963 .01472 L
.45963 .01472 L
F
.45342 .01472 m
.45342 .01472 L
.45963 .01472 L
.45963 .01472 L
.45342 .01472 L
s
.48447 .01472 m
.48447 .0151 L
.49068 .0151 L
.49068 .01472 L
F
.48447 .01472 m
.48447 .0151 L
.49068 .0151 L
.49068 .01472 L
.48447 .01472 L
s
.51553 .01472 m
.51553 .01472 L
.52174 .01472 L
.52174 .01472 L
F
.51553 .01472 m
.51553 .01472 L
.52174 .01472 L
.52174 .01472 L
.51553 .01472 L
s
.54658 .01472 m
.54658 .15679 L
.5528 .15679 L
.5528 .01472 L
F
.54658 .01472 m
.54658 .15679 L
.5528 .15679 L
.5528 .01472 L
.54658 .01472 L
s
.57764 .01472 m
.57764 .01472 L
.58385 .01472 L
.58385 .01472 L
F
.57764 .01472 m
.57764 .01472 L
.58385 .01472 L
.58385 .01472 L
.57764 .01472 L
s
.6087 .01472 m
.6087 .0174 L
.61491 .0174 L
.61491 .01472 L
F
.6087 .01472 m
.6087 .0174 L
.61491 .0174 L
.61491 .01472 L
.6087 .01472 L
s
.63975 .01472 m
.63975 .01472 L
.64596 .01472 L
.64596 .01472 L
F
.63975 .01472 m
.63975 .01472 L
.64596 .01472 L
.64596 .01472 L
.63975 .01472 L
s
.67081 .01472 m
.67081 .02774 L
.67702 .02774 L
.67702 .01472 L
F
.67081 .01472 m
.67081 .02774 L
.67702 .02774 L
.67702 .01472 L
.67081 .01472 L
s
.70186 .01472 m
.70186 .01472 L
.70807 .01472 L
.70807 .01472 L
F
.70186 .01472 m
.70186 .01472 L
.70807 .01472 L
.70807 .01472 L
.70186 .01472 L
s
.73292 .01472 m
.73292 .01472 L
.73913 .01472 L
.73913 .01472 L
F
.73292 .01472 m
.73292 .01472 L
.73913 .01472 L
.73913 .01472 L
.73292 .01472 L
s
.76398 .01472 m
.76398 .01472 L
.77019 .01472 L
.77019 .01472 L
F
.76398 .01472 m
.76398 .01472 L
.77019 .01472 L
.77019 .01472 L
.76398 .01472 L
s
.79503 .01472 m
.79503 .14875 L
.80124 .14875 L
.80124 .01472 L
F
.79503 .01472 m
.79503 .14875 L
.80124 .14875 L
.80124 .01472 L
.79503 .01472 L
s
.82609 .01472 m
.82609 .01472 L
.8323 .01472 L
.8323 .01472 L
F
.82609 .01472 m
.82609 .01472 L
.8323 .01472 L
.8323 .01472 L
.82609 .01472 L
s
.85714 .01472 m
.85714 .13918 L
.86335 .13918 L
.86335 .01472 L
F
.85714 .01472 m
.85714 .13918 L
.86335 .13918 L
.86335 .01472 L
.85714 .01472 L
s
.8882 .01472 m
.8882 .01472 L
.89441 .01472 L
.89441 .01472 L
F
.8882 .01472 m
.8882 .01472 L
.89441 .01472 L
.89441 .01472 L
.8882 .01472 L
s
.91925 .01472 m
.91925 .08671 L
.92547 .08671 L
.92547 .01472 L
F
.91925 .01472 m
.91925 .08671 L
.92547 .08671 L
.92547 .01472 L
.91925 .01472 L
s
.95031 .01472 m
.95031 .01472 L
.95652 .01472 L
.95652 .01472 L
F
.95031 .01472 m
.95031 .01472 L
.95652 .01472 L
.95652 .01472 L
.95031 .01472 L
s
.98137 .01472 m
.98137 .01472 L
.98758 .01472 L
.98758 .01472 L
F
.98137 .01472 m
.98137 .01472 L
.98758 .01472 L
.98758 .01472 L
.98137 .01472 L
s
1 .5 0 r
.02484 .01472 m
.02484 .04497 L
.03106 .04497 L
.03106 .01472 L
F
.02484 .01472 m
.02484 .04497 L
.03106 .04497 L
.03106 .01472 L
.02484 .01472 L
s
.0559 .01472 m
.0559 .01625 L
.06211 .01625 L
.06211 .01472 L
F
.0559 .01472 m
.0559 .01625 L
.06211 .01625 L
.06211 .01472 L
.0559 .01472 L
s
.08696 .01472 m
.08696 .07943 L
.09317 .07943 L
.09317 .01472 L
F
.08696 .01472 m
.08696 .07943 L
.09317 .07943 L
.09317 .01472 L
.08696 .01472 L
s
.11801 .01472 m
.11801 .06603 L
.12422 .06603 L
.12422 .01472 L
F
.11801 .01472 m
.11801 .06603 L
.12422 .06603 L
.12422 .01472 L
.11801 .01472 L
s
.14907 .01472 m
.14907 .05761 L
.15528 .05761 L
.15528 .01472 L
F
.14907 .01472 m
.14907 .05761 L
.15528 .05761 L
.15528 .01472 L
.14907 .01472 L
s
.18012 .01472 m
.18012 .0151 L
.18634 .0151 L
.18634 .01472 L
F
.18012 .01472 m
.18012 .0151 L
.18634 .0151 L
.18634 .01472 L
.18012 .01472 L
s
.21118 .01472 m
.21118 .0825 L
.21739 .0825 L
.21739 .01472 L
F
.21118 .01472 m
.21118 .0825 L
.21739 .0825 L
.21739 .01472 L
.21118 .01472 L
s
.24224 .01472 m
.24224 .06258 L
.24845 .06258 L
.24845 .01472 L
F
.24224 .01472 m
.24224 .06258 L
.24845 .06258 L
.24845 .01472 L
.24224 .01472 L
s
.27329 .01472 m
.27329 .05454 L
.2795 .05454 L
.2795 .01472 L
F
.27329 .01472 m
.27329 .05454 L
.2795 .05454 L
.2795 .01472 L
.27329 .01472 L
s
.30435 .01472 m
.30435 .01472 L
.31056 .01472 L
.31056 .01472 L
F
.30435 .01472 m
.30435 .01472 L
.31056 .01472 L
.31056 .01472 L
.30435 .01472 L
s
.3354 .01472 m
.3354 .06067 L
.34161 .06067 L
.34161 .01472 L
F
.3354 .01472 m
.3354 .06067 L
.34161 .06067 L
.34161 .01472 L
.3354 .01472 L
s
.36646 .01472 m
.36646 .02237 L
.37267 .02237 L
.37267 .01472 L
F
.36646 .01472 m
.36646 .02237 L
.37267 .02237 L
.37267 .01472 L
.36646 .01472 L
s
.39752 .01472 m
.39752 .0645 L
.40373 .0645 L
.40373 .01472 L
F
.39752 .01472 m
.39752 .0645 L
.40373 .0645 L
.40373 .01472 L
.39752 .01472 L
s
.42857 .01472 m
.42857 .01663 L
.43478 .01663 L
.43478 .01472 L
F
.42857 .01472 m
.42857 .01663 L
.43478 .01663 L
.43478 .01472 L
.42857 .01472 L
s
.45963 .01472 m
.45963 .07714 L
.46584 .07714 L
.46584 .01472 L
F
.45963 .01472 m
.45963 .07714 L
.46584 .07714 L
.46584 .01472 L
.45963 .01472 L
s
.49068 .01472 m
.49068 .07714 L
.49689 .07714 L
.49689 .01472 L
F
.49068 .01472 m
.49068 .07714 L
.49689 .07714 L
.49689 .01472 L
.49068 .01472 L
s
.52174 .01472 m
.52174 .0645 L
.52795 .0645 L
.52795 .01472 L
F
.52174 .01472 m
.52174 .0645 L
.52795 .0645 L
.52795 .01472 L
.52174 .01472 L
s
.5528 .01472 m
.5528 .01472 L
.55901 .01472 L
.55901 .01472 L
F
.5528 .01472 m
.5528 .01472 L
.55901 .01472 L
.55901 .01472 L
.5528 .01472 L
s
.58385 .01472 m
.58385 .0511 L
.59006 .0511 L
.59006 .01472 L
F
.58385 .01472 m
.58385 .0511 L
.59006 .0511 L
.59006 .01472 L
.58385 .01472 L
s
.61491 .01472 m
.61491 .03693 L
.62112 .03693 L
.62112 .01472 L
F
.61491 .01472 m
.61491 .03693 L
.62112 .03693 L
.62112 .01472 L
.61491 .01472 L
s
.64596 .01472 m
.64596 .06948 L
.65217 .06948 L
.65217 .01472 L
F
.64596 .01472 m
.64596 .06948 L
.65217 .06948 L
.65217 .01472 L
.64596 .01472 L
s
.67702 .01472 m
.67702 .03233 L
.68323 .03233 L
.68323 .01472 L
F
.67702 .01472 m
.67702 .03233 L
.68323 .03233 L
.68323 .01472 L
.67702 .01472 L
s
.70807 .01472 m
.70807 .05186 L
.71429 .05186 L
.71429 .01472 L
F
.70807 .01472 m
.70807 .05186 L
.71429 .05186 L
.71429 .01472 L
.70807 .01472 L
s
.73913 .01472 m
.73913 .07254 L
.74534 .07254 L
.74534 .01472 L
F
.73913 .01472 m
.73913 .07254 L
.74534 .07254 L
.74534 .01472 L
.73913 .01472 L
s
.77019 .01472 m
.77019 .04842 L
.7764 .04842 L
.7764 .01472 L
F
.77019 .01472 m
.77019 .04842 L
.7764 .04842 L
.7764 .01472 L
.77019 .01472 L
s
.80124 .01472 m
.80124 .01472 L
.80745 .01472 L
.80745 .01472 L
F
.80124 .01472 m
.80124 .01472 L
.80745 .01472 L
.80745 .01472 L
.80124 .01472 L
s
.8323 .01472 m
.8323 .06067 L
.83851 .06067 L
.83851 .01472 L
F
.8323 .01472 m
.8323 .06067 L
.83851 .06067 L
.83851 .01472 L
.8323 .01472 L
s
.86335 .01472 m
.86335 .01586 L
.86957 .01586 L
.86957 .01472 L
F
.86335 .01472 m
.86335 .01586 L
.86957 .01586 L
.86957 .01472 L
.86335 .01472 L
s
.89441 .01472 m
.89441 .06948 L
.90062 .06948 L
.90062 .01472 L
F
.89441 .01472 m
.89441 .06948 L
.90062 .06948 L
.90062 .01472 L
.89441 .01472 L
s
.92547 .01472 m
.92547 .01778 L
.93168 .01778 L
.93168 .01472 L
F
.92547 .01472 m
.92547 .01778 L
.93168 .01778 L
.93168 .01472 L
.92547 .01472 L
s
.95652 .01472 m
.95652 .0511 L
.96273 .0511 L
.96273 .01472 L
F
.95652 .01472 m
.95652 .0511 L
.96273 .0511 L
.96273 .01472 L
.95652 .01472 L
s
.98758 .01472 m
.98758 .07829 L
.99379 .07829 L
.99379 .01472 L
F
.98758 .01472 m
.98758 .07829 L
.99379 .07829 L
.99379 .01472 L
.98758 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{500, 309},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007d0001=B000`400?l00000o`00003ooooooooe
ooooo`00:_ooool3o`0000[ooooo1Ol0000:ooooo`?o00003?ooool3o`0000[ooooo0ol0000;oooo
o`?o00003?ooool00ol0003oooooooooo`0:ooooo`?o00002oooool2o`0000Wooooo0ol00004oooo
o`;o00001Oooool3o`0000?ooooo0ol00005ooooo`?o00000_ooool5o`0000Cooooo0ol00003oooo
o`?o00001Oooool3o`0000Cooooo0ol00004ooooo`?o00000oooool3o`0000Gooooo0ol00003oooo
o`?o00001Oooool3o`0000Cooooo00?o0000ooooooooool01?ooool3o`0000?ooooo0ol00005oooo
o`?o00000oooool2o`0000Gooooo1Ol00003ooooo`;o00001?ooool5o`0000;ooooo0ol00004oooo
o`Go000000?oooooo`000?l000000ol00003ooooo`Go00000_ooool3o`0000Cooooo1Ol00003oooo
o`?o00000oooool5o`0000;ooooo0ol00004ooooo`Go00000_ooool3o`0000Cooooo1Ol00003oooo
o`03o`000?oooooooooo00?ooooo1Ol00002ooooo`?o00001?ooool5o`0000;ooooo0_l00006oooo
o`?o00001?ooool2o`0000Gooooo0ol00003ooooo`?o00001Oooool3o`0000;ooooo1Ol0000Aoooo
o`00:oooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000Wooooo00Go0000oooo
ooooooooooooo`00000<ooooo`03o`000?oooooooooo00Sooooo00Go0000ooooooooooooooooo`00
0009ooooo`05o`000?ooooooooooooooool000002oooool00ol0003oooooooooo`09ooooo`05o`00
0?ooooooooooooooool000003?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00;o
oooo00Co0000oooooooooooo00001Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`05o`000?oooooo
ooooooooool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01?ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool0
0oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`03oooo
o`04o`000?ooooooooooo`0000;ooooo00Co0000oooooooooooo00001?ooool01?l0003ooooooooo
ool00003ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooo00000_ooool01?l0003o
ooooooooool00004ooooo`06o`000?ooooooooooo`000?oooooo00000oooool00ol0003ooooooooo
o`02ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool00oooool01_l0003ooooo
ooooool0003oooooo`0000?ooooo00?o0000ooooooooool00_ooool01_l0003oooooooooool0003o
ooooo`0000?ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00003ooooo`03o`00
0?oooooooooo00Cooooo00Ko0000oooooooooooo0000ooooool00003ooooo`03o`000?oooooooooo
00;ooooo00Co0000oooooooooooo00001?ooool00ol0003oooooooooo`02ooooo`05o`000?oooooo
ooooooooool000000_ooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000oooool00ol0003oooooooooo`03ooooo`05o`000?ooooooooooooooool000000_ooool01?l0
003oooooooooool0000Aooooo`00:oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo
00gooooo00?o0000ooooooooool01oooool5o`0000gooooo00?o0000ooooooooool01oooool01Ol0
003ooooooooooooooooo000000cooooo00?o0000ooooooooool02?ooool01Ol0003ooooooooooooo
oooo000000gooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`02ooooo`04o`000?oo
ooooooooo`0000Gooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00?ooooo00Co0000oooooooooooooooo1Ol00005ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00Go0000oooo
ooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00001Ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00ooo
ool01?l0003oooooooooooooool5o`0000Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`03ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooooooooooo0000
00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`05ooooo`05o`000?oooooo
ooooooooool000002Oooool01Ol0003ooooooooooooooooo000001?ooooo000[ooooo`03o`000?oo
oooooooo00cooooo00?o0000ooooooooool02_ooool2o`0000[ooooo00Co0000oooooooooooo0000
3_ooool00ol0003oooooooooo`07ooooo`Co00003Oooool00ol0003oooooooooo`09ooooo`?o0000
2oooool4o`0000Sooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`03
o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool00oooool2o`0000Kooooo00Go0000oooooooo
ooooooooo`000002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`03ooooo`04o`000?ooooooooooooooo`Co00001_ooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool3o`0000Kooooo00?o0000
ooooooooool00_ooool4o`0000Kooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`06ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`02ooooo`;o00001ooo
ool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Co00001oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00Gooooo00Co0000oooooooooooooooo0ol00007ooooo`04
o`000?ooooooooooooooo`Co00001Oooool2o`0000?ooooo00Co0000oooooooooooo00001Oooool2
o`0000Cooooo00?o0000ooooooooool01Oooool2o`0000Gooooo00?o0000ooooooooool04?ooool0
02_ooooo00?o0000ooooooooool02Oooool01Ol0003ooooooooooooooooo000000gooooo00?o0000
ooooooooool02?ooool00ol0003oooooo`00000:ooooo`Co00002_ooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00Go0000ooooooooooooooooo`000009ooooo`05o`000?oo
ooooooooooooool000002?ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Go
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`05o`000?oooooooooooooo
ool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Kooooo00Co0000oooo
oooooooooooo1?l00006ooooo`05o`000?ooooooooooooooool000002Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooool00002ooooo`05o`000?ooooooooooooooool000000ooo
ool01Ol0003ooooooooooooooooo000000Gooooo00Oo0000ooooooooooooooooo`000?oooooo0000
00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00Go0000
ooooooooooooooooo`000003ooooo`06o`000?ooooooooooo`000?oooooo00001?ooool01_l0003o
oooooooooooooooo0000ooooo`Co00001?ooool01ol0003ooooooooooooooooo0000ooooool00000
1oooool01Ol0003ooooooooooooooooo000000Gooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo
000000?ooooo00?o0000ooooooooool01Oooool01?l0003oooooooooool00002ooooo`03o`000?oo
oooooooo00Gooooo00Go0000ooooooooooooooooo`000009ooooo`03o`000?oooooo000000?ooooo
00?o0000ooooooooool03oooool002[ooooo0_l0000;ooooo`05o`000?ooooooooooooooool00000
2Oooool01Ol0003ooooooooooooooooo000000_ooooo0_l0000:ooooo`03o`000?oooooooooo00co
oooo00?o0000ooooooooool02_ooool01Ol0003ooooooooooooooooo000000Wooooo00Go0000oooo
ooooooooooooo`000009ooooo`05o`000?ooooooooooooooool000001oooool2o`0000Cooooo00Co
0000oooooooooooo00001?ooool2o`0000Cooooo0_l00006ooooo`;o00000oooool01Ol0003ooooo
oooooooooooo000000Cooooo0_l00003ooooo`05o`000?ooooooooooooooool000001?ooool2o`00
00Gooooo0_l00005ooooo`;o00000oooool00ol0003oooooooooo`06ooooo`;o00001?ooool00ol0
003oooooooooo`05ooooo`;o00000oooool01Ol0003ooooooooooooooooo000000Cooooo0_l00003
ooooo`05o`000?ooooooooooooooool000001?ooool2o`0000?ooooo00Go0000oooooooooooooooo
o`000003ooooo`05o`000?ooooooooooooooool000000_ooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool2o`0000Gooooo00Oo0000ooooooooooooooooo`00
0?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo000000?o
oooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooooooo`;o00001?ooool0
1ol0003ooooooooooooooooo0000ooooool000001oooool01Ol0003ooooooooooooooooo000000;o
oooo00?o0000ooooooooool01?ooool01ol0003ooooooooooooooooo0000ooooool000000oooool0
1Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooool000000oooool01Ol0003ooooooooo
oooooooo000000?ooooo00?o0000ooooool000000oooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000_ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000
ooooooooool00_l00005ooooo`07o`000?ooooooooooooooool0003oooooo`000003ooooo`03o`00
0?oooooooooo00oooooo000[ooooo`03o`000?oooooooooo00[ooooo0ol0000;ooooo`?o00003Ooo
ool00ol0003oooooooooo`08ooooo`Go00002oooool2o`0000[ooooo1Ol0000:ooooo`?o00002ooo
ool3o`0000Wooooo00?o0000ooooooooool00oooool2o`0000Kooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;ooooo0ol00006ooooo`03o`000?oo
oooooooo00;ooooo0ol00006ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?oo
ool01?l0003oooooooooooooool5o`0000Gooooo00?o0000ooooooooool00oooool2o`0000Kooooo
00Co0000oooooooooooooooo1Ol00005ooooo`03o`000?oooooooooo00;ooooo0ol00006ooooo`03
o`000?oooooooooo00;ooooo0ol00005ooooo`?o00001?ooool2o`0000Gooooo0ol00004ooooo`03
o`000?oooooooooo00Cooooo0ol00003ooooo`?o00001Oooool3o`0000?ooooo0ol00005ooooo`?o
00001Oooool00ol0003oooooooooo`03ooooo`?o00000_ooool5o`0000Cooooo0ol00004ooooo`;o
00001Oooool3o`0000;ooooo1Ol00004ooooo`?o00000oooool3o`0000Gooooo0ol00003ooooo`?o
00001Oooool3o`0000Cooooo0_l00005ooooo`?o00001?ooool00ol0003oooooooooo`04ooooo`?o
00000oooool3o`0001;ooooo003oooooooGooooo003oooooooGooooo003oooooooGooooo003ooooo
ooGooooo003oooooooGooooo003oooooooGooooo003oooooooGooooo000Soooooooo0000aOl0000=
ooooo`008oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0;ooooo`00
8oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`0;ooooo`006_ooool2o`0000Oooooo00?o0000ooooooooool0oooo
ooo1ooooo`03o`000?oooooooooo00_ooooo000Iooooo`04o`000?ooooooooooo`0000Kooooo00?o
0000ooooooooool0ooooooo1ooooo`03o`000?oooooooooo00_ooooo000Iooooo`04o`000?oooooo
ooooo`0000Kooooo00?o0000ooooooooool0ooooooo1ooooo`03o`000?oooooooooo00_ooooo000I
ooooo`04o`000?ooooooooooo`0000Kooooo0ol00003ool000?o0?l00_l00?l4oomo00;o00000ooo
0003o`3o00;o003o1?ooO`02o`0000?oo`000ol0o`02o`00o`Coogl00_l00003ool000?o0?l00_l0
0?l4oomo00;o00000ooo0003o`3o00;o003o1?ooO`02o`0000?oo`000ol0o`02o`00o`Coogl00_l0
0003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0
o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l4oomo0004o`000?oo003oo`00ool000?o0?l0
0ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl0
0_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`00
0ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o
1?ooO`001?l0003oo`00ool00?oo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?o
ogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?o
o`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o
003o0oooO`02o`0000?oo`000ol0o`03o`00o`Coogl000Co0000ool00?oo003oo`000ol0o`03o`00
o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`00
00?oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o
00?o003o0oooO`03o`0000gooooo000Iooooo`04o`000?ooooooooooo`0000Kooooo00?o0000oooo
ooooool02?ooool4oomo00;ooooo0ooo0003o`3o00?o003o3Oooool4oomo00[ooooo1?ooO`0:oooo
o`Coogl00_ooool3ool000?o0?l00ol00?l>ooooo`?oogl02oooool3oomo00_ooooo0oooO`02oooo
o`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl02oooool3oomo
00;ooooo0ooo0003o`3o00?o003o0oooO`0;ooooo`?oogl02oooool3oomo00_ooooo0oooO`02oooo
o`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl02oooool3oomo
00;ooooo0ooo0003o`3o00?o003o0oooO`0;ooooo`?oogl02oooool3oomo00_ooooo0oooO`02oooo
o`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02oooo
o`?oo`000ol0o`03o`00o`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`03o`000?oooooooooo
00_ooooo000Iooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool02?ooool4oomo
00;ooooo0ooo0003o`3o00?o003o3Oooool4oomo00[ooooo1?ooO`0:ooooo`Coogl00_ooool3ool0
00?o0?l00ol00?l>ooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`?oo`000ol0o`04o`00
o`gooooo0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl02oooool3oomo00;ooooo0ooo0003o`3o
00Co003o3Oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=oooo
o`?oogl00_ooool4ool000Gooooo0oooO`0;ooooo`?oogl00_ooool3ool000?o0?l00ol00?l3oomo
00_ooooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo0ooo0003o`3o00Co003o3Oooool3oomo
00;ooooo0ooo0003o`3o00Co003o3Oooool3oomo00;ooooo0ooo0003o`3o00Co003o3Oooool3oomo
00_ooooo0oooO`02ooooo`03o`000?oooooooooo00_ooooo000Jooooo`;o00001oooool00ol0003o
ooooooooo`08ooooo`Coogl00_ooool3ool000?o0?l00ol00?l=ooooo`Coogl02_ooool4oomo00[o
oooo1?ooO`02ooooo`?oo`000ol0o`03o`00o`kooooo0oooO`0;ooooo`?oogl02oooool3oomo00;o
oooo0ooo0003o`3o00Co003o3Oooool3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`0;ooooo`?o
ogl00_ooool3ool000?o0?l01?l00?l=ooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`?o
o`000ol0o`04o`00o`gooooo0oooO`02ooooo`Coo`001Oooool3oomo00_ooooo0oooO`02ooooo`?o
o`000ol0o`03o`00o`?oogl02oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool3ool000?o
0?l01?l00?l=ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=ooooo`?oogl00_ooool3ool000?o
0?l01?l00?l=ooooo`?oogl02oooool3oomo00;ooooo00?o0000ooooooooool02oooool002?ooooo
00?o0000ooooooooool02?ooool4oomo00;ooooo0ooo0003o`3o00?o003o3Oooool4oomo00[ooooo
1?ooO`0:ooooo`Coogl00_ooool3ool000?o0?l00ol00?l>ooooo`?oogl02oooool3oomo00_ooooo
0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo
0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo
0ooo0003o`3o00Co003o3Oooool3oomo00;ooooo1?oo0005ooooo`?oogl02oooool3oomo00;ooooo
1?oo0002ooooo`?o003o0oooO`0;ooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`?oo`00
0ol0o`04o`00o`gooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`?oo`00
0ol0o`04o`00o`gooooo0oooO`0;ooooo`?oogl00_ooool00ol0003oooooooooo`0;ooooo`008ooo
ool00ol0003oooooooooo`08ooooo`Coogl00_ooool3ool000?o0?l00ol00?l=ooooo`Coogl02_oo
ool4oomo00[ooooo1?ooO`02ooooo`?oo`000ol0o`03o`00o`kooooo0oooO`0;ooooo`?oogl02ooo
ool3oomo00;ooooo0ooo0003o`3o00Co003o3Oooool3oomo00;ooooo1?oo0002ooooo`Co003o3Ooo
ool3oomo00;ooooo0ooo0003o`3o00Co003o3Oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_oo
ool3ool000?o0?l01?l00?l=ooooo`?oogl00_ooool4ool000Gooooo0oooO`0;ooooo`?oogl00_oo
ool4ool000;ooooo0ol00?l3oomo00_ooooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo0ooo
0003o`3o00Co003o3Oooool3oomo00;ooooo0ooo0003o`3o00Co003o3Oooool3oomo00;ooooo0ooo
0003o`3o00Co003o3Oooool3oomo00_ooooo0oooO`02ooooo`03o`000?oooooooooo00_ooooo000S
ooooo`03o`000?oooooooooo00Sooooo1?ooO`02ooooo`?oo`000ol0o`03o`00o`gooooo1?ooO`0:
ooooo`Coogl02_ooool4oomo00;ooooo0ooo0003o`3o00?o003o3_ooool3oomo00_ooooo0oooO`0;
ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=ooooo`?oogl00_ooool4ool000;ooooo1?l00?l=
ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=ooooo`?oogl02oooool3oomo00_ooooo0oooO`02
ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`Coo`001Oooool3oomo00_ooooo0oooO`02
ooooo`Coo`000_ooool3o`00o`?oogl02oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool3
ool000?o0?l01?l00?l=ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=ooooo`?oogl00_ooool3
ool000?o0?l01?l00?l=ooooo`?oogl02oooool3oomo00;ooooo00?o0000ooooooooool02oooool0
02?ooooo00?o0000ooooooooool02?ooool4oomo00;ooooo0ooo0003o`3o00?o003o3Oooool4oomo
00[ooooo1?ooO`0:ooooo`Coogl00_ooool3ool000?o0?l00ol00?l>ooooo`?oogl02oooool3oomo
00_ooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`Coo`000_ooool4o`00
o`gooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`0;ooooo`?oogl02oooool3oomo
00;ooooo0ooo0003o`3o00Co003o3Oooool3oomo00;ooooo1?oo0005ooooo`?oogl02oooool3oomo
00;ooooo1?oo0005ooooo`?oogl02oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool3ool0
00?o0?l01?l00?l=ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=ooooo`?oogl00_ooool3ool0
00?o0?l01?l00?l=ooooo`?oogl02oooool3oomo00;ooooo00?o0000ooooooooool02oooool002?o
oooo00?o0000ooooooooool02?ooool4oomo00;ooooo0ooo0003o`3o00?o003o3Oooool4oomo00[o
oooo1?ooO`0:ooooo`Coogl00_ooool3ool000?o0?l00ol00?l>ooooo`?oogl02oooool3oomo00_o
oooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`Coo`000_ooool4o`00o`go
oooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`0;ooooo`?oogl02oooool3oomo00;o
oooo0ooo0003o`3o00Co003o3Oooool3oomo00;ooooo1?oo0005ooooo`?oogl02oooool3oomo00;o
oooo1?oo0005ooooo`?oogl02oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool3ool000?o
0?l01?l00?l=ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=ooooo`?oogl00_ooool3ool000?o
0?l01?l00?l=ooooo`?oogl02oooool3oomo00;ooooo00?o0000ooooooooool02oooool002?ooooo
00?o0000ooooooooool02?ooool4oomo00;ooooo0ooo0003o`3o00?o003o3Oooool4oomo00[ooooo
1?ooO`0:ooooo`Coogl00_ooool3ool000?o0?l00ol00?l>ooooo`?oogl02oooool3oomo00_ooooo
0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`Coo`000_ooool4o`00o`gooooo
0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo
0ooo0003o`3o00Co003o3Oooool3oomo00;ooooo1?oo0005ooooo`?oogl02oooool3oomo00;ooooo
1?oo000Cooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo
0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo
0oooO`0;ooooo`?oogl00_ooool00ol0003oooooooooo`0;ooooo`008oooool3o`0000Sooooo1?oo
O`02ooooo`?oo`000ol0o`03o`00o`gooooo1?ooO`0:ooooo`Coogl02_ooool4oomo00;ooooo1?oo
0002ooooo`?o003o3_ooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool3ool000?o0?l01?l0
0?l=ooooo`?oogl00_ooool4ool000;ooooo1?l00?l=ooooo`?oogl00_ooool3ool000?o0?l01?l0
0?l=ooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0ooo
O`02ooooo`Coo`001Oooool3oomo00_ooooo0oooO`02ooooo`Coo`004oooool3oomo00_ooooo0ooo
O`0;ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=ooooo`?oogl00_ooool3ool000?o0?l01?l0
0?l=ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=ooooo`?oogl02oooool3oomo0003ooooool0
003o000000gooooo000Sooooo`03o`000?oooooooooo00Sooooo1?ooO`02ooooo`?oo`000ol0o`03
o`00o`gooooo1?ooO`0:ooooo`Coogl02_ooool4oomo00;ooooo1?oo0002ooooo`?o003o3_ooool3
oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=ooooo`?oogl00_ooool4
ool000;ooooo1?l00?l=ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=ooooo`?oogl02oooool3
oomo00_ooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`Coo`004oooool3
oomo00;ooooo1?oo000Cooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`?oo`000ol0o`04
o`00o`gooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`?oo`000ol0o`04
o`00o`gooooo0oooO`0;ooooo`?oogl00_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0
003oooooooooo`08ooooo`Coogl00_ooool3ool000?o0?l00ol00?l=ooooo`Coogl02_ooool4oomo
00[ooooo1?ooO`02ooooo`Coo`000_ooool3o`00o`kooooo0oooO`0;ooooo`?oogl02oooool3oomo
00;ooooo0ooo0003o`3o00Co003o3Oooool3oomo00;ooooo1?oo0002ooooo`Co003o3Oooool3oomo
00;ooooo0ooo0003o`3o00Co003o3Oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool3ool0
00?o0?l01?l00?l=ooooo`?oogl00_ooool4ool001?ooooo0oooO`02ooooo`Coo`004oooool3oomo
00_ooooo0oooO`0;ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=ooooo`?oogl00_ooool3ool0
00?o0?l01?l00?l=ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=ooooo`?oogl02oooool3oomo
00;ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool02?ooool4oomo00;o
oooo0ooo0003o`3o00?o003o3Oooool4oomo00[ooooo1?ooO`0:ooooo`Coogl00_ooool4ool000;o
oooo0ol00?l>ooooo`?oogl02oooool3oomo00_ooooo0oooO`05ooooo`?o0?l01?l00?l=ooooo`?o
ogl00_ooool4ool000;ooooo1?l00?l=ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=ooooo`?o
ogl02oooool3oomo00_ooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`Co
o`004oooool3oomo00;ooooo1?oo000Cooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`?o
o`000ol0o`04o`00o`gooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`?o
o`000ol0o`04o`00o`gooooo0oooO`0;ooooo`?oogl00_ooool00ol0003oooooooooo`0;ooooo`00
8oooool00ol0003oooooooooo`0>ooooo`?oo`000ol0o`03o`00o`gooooo1?ooO`0:ooooo`Coogl0
2_ooool4oomo00;ooooo1?oo0002ooooo`?o003o3_ooool3oomo00_ooooo0oooO`0;ooooo`?oogl0
1Oooool3o`3o00Co003o3Oooool3oomo00;ooooo1?oo000Cooooo`?oogl00_ooool3ool000?o0?l0
1?l00?l=ooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo
0oooO`02ooooo`Coo`004oooool3oomo00;ooooo1?oo000Cooooo`?oogl02oooool3oomo00_ooooo
0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo
0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`0;ooooo`?oogl00_ooool00ol0003ooooo
ooooo`0;ooooo`008oooool00ol0003oooooooooo`0>ooooo`?oo`000ol0o`03o`00o`gooooo1?oo
O`0:ooooo`Coogl02_ooool4oomo00;ooooo1?oo0002ooooo`?o003o3_ooool3oomo00_ooooo0ooo
O`0;ooooo`?oogl01Oooool3o`3o00Co003o3Oooool3oomo00;ooooo1?oo000Cooooo`?oogl00_oo
ool3ool000?o0?l01?l00?l=ooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`?oo`000ol0
o`04o`00o`gooooo0oooO`02ooooo`Coo`004oooool3oomo00;ooooo1?oo000Cooooo`?oogl02ooo
ool3oomo00_ooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`?oo`000ol0
o`04o`00o`gooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`0;ooooo`?oogl00_oo
ool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`0>ooooo`?oo`000ol0o`03
o`00o`gooooo1?ooO`0:ooooo`Coogl02_ooool4oomo00;ooooo1?oo0002ooooo`?o003o3_ooool3
oomo00_ooooo0oooO`0;ooooo`?oogl01Oooool3o`3o00Co003o3Oooool3oomo00;ooooo1?oo000C
ooooo`?oogl00_ooool3ool000?o0?l01?l00?l=ooooo`?oogl02oooool3oomo00_ooooo0oooO`02
ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`Coo`004oooool3oomo00;ooooo1?oo000C
ooooo`?oogl02oooool3oomo013ooooo0ooo0003o`3o00Co003o3Oooool3oomo00;ooooo0ooo0003
o`3o00Co003o3Oooool3oomo00;ooooo0ooo0003o`3o00Co003o3Oooool3oomo00_ooooo0oooO`02
ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo00kooooo0ooo0003o`3o
00?o003o3Oooool4oomo00[ooooo1?ooO`0:ooooo`Coogl00_ooool4ool000;ooooo0ol00?l>oooo
o`?oogl02oooool3oomo00_ooooo0oooO`05ooooo`?o0?l01?l00?l=ooooo`?oogl00_ooool4ool0
01?ooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`0;ooooo`?oogl02oooool3oomo
00;ooooo0ooo0003o`3o00Co003o4_ooool4ool001?ooooo0oooO`02ooooo`Coo`004oooool3oomo
00_ooooo0oooO`0@ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`?oo`000ol0o`04o`00
o`gooooo0oooO`02ooooo`Coo`000_ooool4o`00oa_ooooo0oooO`02ooooo`03o`000?oooooooooo
00_ooooo000Sooooo`03o`000?oooooooooo00kooooo0ooo0003o`3o00?o003o3Oooool4oomo00[o
oooo1?ooO`0:ooooo`Coogl00_ooool4ool000;ooooo0ol00?l>ooooo`?oogl02oooool3oomo00_o
oooo0oooO`05ooooo`?o0?l01?l00?l=ooooo`?oogl00_ooool4ool001?ooooo0oooO`02ooooo`?o
o`000ol0o`04o`00o`gooooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo0ooo0003o`3o00Co
003o4_ooool4ool001?ooooo0oooO`02ooooo`Coo`008Oooool3oomo013ooooo0ooo0003o`3o00Co
003o3Oooool3oomo00;ooooo0ooo0003o`3o00Co003o3Oooool3oomo00;ooooo1?oo0002ooooo`Co
003o6oooool3oomo00;ooooo00?o0000ooooooooool02oooool002?ooooo0ol0000>ooooo`?oo`00
0ol0o`03o`00o`gooooo1?ooO`0:ooooo`Coogl02_ooool4oomo00;ooooo1?oo0002ooooo`?o003o
3_ooool3oomo00_ooooo0oooO`0Cooooo`?o0?l01?l00?l=ooooo`?oogl00_ooool4ool001?ooooo
0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo
0ooo0003o`3o00Co003o4_ooool4ool001?ooooo0oooO`02ooooo`Coo`008Oooool3oomo013ooooo
0ooo0003o`3o00Co003o3Oooool3oomo00;ooooo0ooo0003o`3o00Co003o3Oooool3oomo00;ooooo
1?oo0002ooooo`Co003o6oooool3oomo0003ooooool0003o000000gooooo000Sooooo`03o`000?oo
oooooooo00kooooo0ooo0003o`3o00?o003o3Oooool4oomo00[ooooo1?ooO`0@ooooo`Coo`000_oo
ool3o`00o`kooooo0oooO`0;ooooo`?oogl04oooool3o`3o00Co003o3Oooool3oomo00;ooooo1?oo
000Cooooo`?oogl00_ooool3ool000?o0?l01?l00?l=ooooo`?oogl02oooool3oomo00_ooooo0ooo
O`02ooooo`?oo`000ol0o`04o`00oa;ooooo1?oo000Cooooo`?oogl00_ooool4ool0027ooooo0ooo
O`0@ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0ooo
O`02ooooo`Coo`000_ooool4o`00oa_ooooo0oooO`02ooooo`03o`000?oooooooooo00_ooooo000S
ooooo`03o`000?oooooooooo00kooooo0ooo0003o`3o00?o003o3Oooool4oomo00[ooooo1?ooO`0@
ooooo`Coo`000_ooool3o`00o`kooooo0oooO`0;ooooo`?oogl04oooool3o`3o00Co003o4_ooool4
ool001?ooooo0oooO`02ooooo`?oo`000ol0o`04o`00o`gooooo0oooO`0;ooooo`?oogl02oooool3
oomo00;ooooo0ooo0003o`3o00Co003o4_ooool4ool001?ooooo0oooO`02ooooo`Coo`008Oooool3
oomo013ooooo0ooo0003o`3o00Co003o4_ooool3ool000?o0?l01?l00?l=ooooo`?oogl00_ooool4
ool000;ooooo1?l00?lKooooo`?oogl00_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0
003oooooooooo`0>ooooo`?oo`000ol0o`03o`00o`gooooo1?ooO`0:ooooo`Coogl04?ooool4ool0
01?ooooo0oooO`0Qooooo`?o0?l01?l00?lBooooo`Coo`004oooool3oomo00;ooooo0ooo0003o`3o
00Co003o3Oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool3ool000?o0?l01?l00?lBoooo
o`Coo`004oooool3oomo00;ooooo1?oo000Qooooo`?oogl04?ooool3ool000?o0?l01?l00?lBoooo
o`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`Coo`000_ooool4o`00oa_ooooo0oooO`02oooo
o`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo00kooooo0ooo0003o`3o00?o
003o3Oooool4oomo00[ooooo1?ooO`0@ooooo`Coo`004oooool3oomo027ooooo0ol0o`04o`00oa;o
oooo1?oo000Hooooo`?oo`000ol0o`04o`00o`gooooo0oooO`0;ooooo`?oogl04?ooool3ool000?o
0?l01?l00?lBooooo`Coo`004oooool3oomo00;ooooo1?oo000Qooooo`?oogl04?ooool3ool000?o
0?l01?l00?lBooooo`?oo`000ol0o`04o`00o`gooooo0oooO`02ooooo`Coo`000_ooool4o`00oa_o
oooo0oooO`02ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo00kooooo
0ooo0003o`3o00?o003o3Oooool4oomo01kooooo1?oo000Cooooo`?oogl08Oooool3o`3o00Co003o
4_ooool4ool001Sooooo0ooo0003o`3o00Co003o3Oooool3oomo00_ooooo0oooO`0@ooooo`?oo`00
0ol0o`04o`00oa;ooooo1?oo000Cooooo`?oogl00_ooool4ool0027ooooo0oooO`0@ooooo`?oo`00
0ol0o`04o`00oa;ooooo0ooo0003o`3o00Co003o3Oooool3oomo00;ooooo1?oo0002ooooo`Co003o
6oooool3oomo00;ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool03_oo
ool3ool000?o0?l00ol00?l=ooooo`Coogl07_ooool4ool001?ooooo0oooO`0Qooooo`?o0?l01?l0
0?lBooooo`Coo`006?ooool3ool000?o0?l01?l00?l=ooooo`?oogl02oooool3oomo013ooooo0ooo
0003o`3o00Co003o4_ooool4ool001Sooooo1?oo000Qooooo`?oogl04?ooool3ool000?o0?l01?l0
0?lBooooo`?oo`000ol0o`04o`00oa;ooooo1?oo0002ooooo`Co003o6oooool3oomo00;ooooo00?o
0000ooooooooool02oooool002?ooooo00?o0000ooooooooool03_ooool3ool000?o0?l00ol00?l=
ooooo`Coogl07_ooool4ool001?ooooo0oooO`0Qooooo`?o0?l01?l00?lBooooo`Coo`006?ooool3
ool000?o0?l01?l00?l=ooooo`?oogl02oooool3oomo013ooooo0ooo0003o`3o00Co003o4_ooool4
ool001Sooooo1?oo000Qooooo`?oogl04?ooool3ool000?o0?l01?l00?lBooooo`?oo`000ol0o`04
o`00oa;ooooo1?oo0002ooooo`Co003o6oooool3oomo00;ooooo00?o0000ooooooooool02oooool0
02?ooooo00?o0000ooooooooool03_ooool3ool000?o0?l00ol00?l=ooooo`Coogl07_ooool4ool0
01?ooooo0oooO`0Qooooo`?o0?l01?l00?lBooooo`Coo`006?ooool3ool000?o0?l01?l00?l=oooo
o`?oogl02oooool3oomo013ooooo0ooo0003o`3o00Co003o4_ooool4ool001Sooooo1?oo000doooo
o`?oo`000ol0o`04o`00oa;ooooo0ooo0003o`3o00Co003o4_ooool4ool000;ooooo1?l00?lKoooo
o`?oogl00_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`0>ooooo`?o
o`000ol0o`03o`00o`gooooo1?ooO`0Nooooo`Coo`004oooool3oomo027ooooo0ol0o`04o`00oa;o
oooo1?oo000Hooooo`?oo`000ol0o`04o`00o`gooooo0oooO`0;ooooo`?oogl04?ooool3ool000?o
0?l01?l00?lBooooo`Coo`006?ooool4ool003Cooooo0ooo0003o`3o00Co003o4_ooool3ool000?o
0?l01?l00?lBooooo`Coo`000_ooool4o`00oa_ooooo0oooO`02ooooo`03o`000?oooooooooo00_o
oooo000Sooooo`?o00003_ooool3ool000?o0?l00ol00?l=ooooo`Coogl07_ooool4ool001?ooooo
0oooO`0Qooooo`?o0?l01?l00?lBooooo`Coo`006?ooool3ool000?o0?l01?l00?l^ooooo`?oo`00
0ol0o`04o`00oa;ooooo1?oo000Hooooo`Coo`00=?ooool3ool000?o0?l01?l00?lEooooo`?o0?l0
1?l00?lBooooo`Coo`000_ooool4o`00oaoooooo0_l0000=ooooo`008oooool00ol0003ooooooooo
o`0>ooooo`Coo`000_ooool3o`00oboooooo1?oo000Cooooo`?oogl08Oooool3o`3o00Co003o4_oo
ool4ool001Sooooo0ooo0003o`3o00Co003o;_ooool3ool000?o0?l01?l00?lBooooo`Coo`006?oo
ool4ool003Oooooo0ol0o`04o`00oaGooooo0ol0o`04o`00oa;ooooo1?oo0002ooooo`Co003o8?oo
ool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`0>ooooo`Coo`000_ooool3
o`00oboooooo1?oo000gooooo`?o0?l01?l00?lBooooo`Coo`006?ooool3ool000?o0?l01?l00?l^
ooooo`?oo`000ol0o`04o`00oa;ooooo1?oo000Hooooo`Coo`00=oooool3o`3o00Co003o5Oooool3
o`3o00Co003o4_ooool4ool000;ooooo1?l00?lPooooo`03o`000?oooooooooo00_ooooo000Soooo
o`03o`000?oooooooooo01Cooooo0ol00?l_ooooo`Coo`00=oooool3o`3o00Co003o4_ooool4ool0
01Sooooo1?oo0002ooooo`Co003o;_ooool3ool000?o0?l01?l00?lBooooo`Coo`006?ooool4ool0
03Oooooo0ol0o`04o`00oaGooooo0ol0o`04o`00oa;ooooo1?oo0002ooooo`Co003o8?ooool00ol0
003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`0Dooooo`?o003o;oooool4ool003Oo
oooo0ol0o`04o`00oa;ooooo1?oo000Hooooo`Coo`000_ooool4o`00obkooooo0ooo0003o`3o00Co
003o4_ooool4ool001Sooooo1?oo000gooooo`?o0?l01?l00?lEooooo`?o0?l01?l00?lBooooo`Co
o`009_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`0Dooooo`?o003o
;oooool4ool003Oooooo0ol0o`04o`00oa;ooooo1?oo000Hooooo`Coo`000_ooool4o`00obkooooo
0ooo0003o`3o00Co003o4_ooool4ool001Sooooo1?oo000gooooo`?o0?l01?l00?lEooooo`?o0?l0
1?l00?lBooooo`Coo`009_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003ooooooooo
o`0Dooooo`?o003o;oooool4ool003Oooooo0ol0o`04o`00oa;ooooo1?oo000Hooooo`Coo`000_oo
ool4o`00oc7ooooo0ol0o`04o`00oa;ooooo1?oo000Hooooo`Coo`00=oooool3o`3o00Co003o5Ooo
ool3o`3o00Co003o4_ooool4ool002Kooooo00?o0000ooooooooool02oooool002?ooooo00?o0000
ooooooooool05?ooool3o`00oboooooo1?oo000gooooo`?o0?l01?l00?lBooooo`Coo`006?ooool4
ool000;ooooo1?l00?laooooo`?o0?l01?l00?lBooooo`Coo`006?ooool4ool003Oooooo0ol0o`04
o`00oaSooooo1?l00?lBooooo`Coo`009_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0
003oooooooooo`0Dooooo`?o003o;oooool4ool003Oooooo0ol0o`04o`00oa;ooooo1?oo000Hoooo
o`Coo`000_ooool4o`00oc7ooooo0ol0o`04o`00oa;ooooo1?oo000Hooooo`Coo`00=oooool3o`3o
00Co003o6?ooool4o`00oa;ooooo1?oo000Vooooo`03o`000?oooooooooo00_ooooo000Sooooo`03
o`000?oooooooooo01Cooooo0ol00?l_ooooo`Coo`00=oooool3o`3o00Co003o4_ooool4ool001So
oooo1?oo0002ooooo`Co003o<Oooool3o`3o00Co003o4_ooool4ool001Sooooo1?oo000gooooo`?o
0?l01?l00?lHooooo`Co003o4_ooool4ool002Kooooo00?o0000ooooooooool02oooool002?ooooo
0ol0000Dooooo`?o003o;oooool4ool003Oooooo0ol0o`04o`00oa;ooooo1?oo000Hooooo`Coo`00
0_ooool4o`00oc7ooooo0ol0o`04o`00oa;ooooo1?oo000Hooooo`Coo`00=oooool3o`3o00Co003o
6?ooool4o`00oa;ooooo1?oo000Uooooo`;o00003Oooool002?ooooo00?o0000ooooooooool05?oo
ool3o`00oboooooo1?oo000gooooo`?o0?l01?l00?lBooooo`Coo`006?ooool4ool000;ooooo1?l0
0?laooooo`?o0?l01?l00?lBooooo`Coo`006?ooool4ool003Oooooo0ol0o`04o`00oaSooooo1?l0
0?lBooooo`Coo`009_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`0D
ooooo`?o003o;oooool4ool003Oooooo0ol0o`04o`00oa;ooooo1?oo000Nooooo`Co003o<Oooool3
o`3o00Co003o4_ooool4ool001Sooooo1?oo000gooooo`?o0?l01?l00?lHooooo`Co003o4_ooool4
ool002Kooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool05?ooool3o`00
oboooooo1?oo000gooooo`?o0?l01?l00?lBooooo`Coo`007_ooool4o`00oc7ooooo0ol0o`04o`00
oa;ooooo1?oo000Hooooo`Coo`00=oooool3o`3o00Co003o6?ooool4o`00oa;ooooo1?oo000Voooo
o`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo01Cooooo0ol00?l_ooooo`Co
o`00=oooool3o`3o00Co003o4_ooool4ool001kooooo1?l00?laooooo`?o0?l01?l00?lBooooo`Co
o`006?ooool4ool003Oooooo0ol0o`04o`00oaSooooo1?l00?lBooooo`Coo`009_ooool00ol0003o
ooooooooo`0;ooooo`008oooool00ol0003oooooooooo`0Dooooo`?o003o;oooool4ool003Oooooo
0ol0o`04o`00oa;ooooo1?oo000Nooooo`Co003o<Oooool3o`3o00Co003o4_ooool4ool001Sooooo
1?oo000gooooo`?o0?l01?l00?lHooooo`Co003o4_ooool4ool002Kooooo00?o0000ooooooooool0
2oooool002?ooooo00?o0000ooooooooool05?ooool3o`00oboooooo1?oo000gooooo`?o0?l01?l0
0?lBooooo`Coo`007_ooool4o`00oc7ooooo0ol0o`04o`00oa;ooooo1?oo000Hooooo`Coo`00=ooo
ool3o`3o00Co003o6?ooool4o`00oa;ooooo1?oo000Vooooo`03o`000?oooooooooo00_ooooo000>
ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`?o00001oooool00ol0003oooooooooo`0D
ooooo`?o003o;oooool4ool003Oooooo0ol0o`04o`00oa;ooooo1?oo000Nooooo`Co003o<Oooool3
o`3o00Co003o4_ooool4ool001Sooooo1?oo000gooooo`?o0?l01?l00?lHooooo`Co003o4_ooool4
ool002Kooooo00?o0000ooooooooool02oooool000gooooo00Co0000oooooooooooo00002Oooool0
0ol0003oooooooooo`06ooooo`03o`000?oooooooooo01Cooooo0ol00?l_ooooo`Coo`00=oooool3
o`3o00Co003o4_ooool4ool005?ooooo0ol0o`04o`00obkooooo1?oo000gooooo`?o0?l01?l00?lH
ooooo`Co003o4_ooool4ool002Kooooo00?o0000ooooooooool02oooool000gooooo00Co0000oooo
oooooooo00002Oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo01Cooooo0ol00?l_
ooooo`Coo`00=oooool3o`3o00Co003o4_ooool4ool005?ooooo0ol0o`04o`00obkooooo1?oo000g
ooooo`?o0?l01?l00?lHooooo`Co003o4_ooool4ool002Kooooo00?o0000ooooooooool02oooool0
00gooooo00Co0000oooooooooooo00002Oooool00ol0003oooooooooo`06ooooo`Co00004oooool3
o`00oboooooo1?oo000gooooo`?o0?l01?l00?lBooooo`Coo`00E_ooool4o`00obkooooo1?oo000g
ooooo`?o0?l01?l00?lHooooo`Co003o4_ooool4ool002?ooooo1?l0000=ooooo`003Oooool01?l0
003oooooooooool00009ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool05?ooool3
o`00oboooooo1?oo000gooooo`?o0?l01?l00?lBooooo`Coo`00E_ooool4o`00obkooooo1?oo000j
ooooo`Co003o6?ooool4o`00oa;ooooo1?oo000Vooooo`03o`000?oooooooooo00_ooooo000=oooo
o`04o`000?ooooooooooo`0000Sooooo0_l00008ooooo`03o`000?oooooooooo01Cooooo0ol00?l_
ooooo`Coo`00=oooool3o`3o00Co003o4_ooool4ool005Kooooo1?l00?l^ooooo`Coo`00>_ooool4
o`00oaSooooo1?l00?lBooooo`Coo`009_ooool00ol0003oooooooooo`0;ooooo`003_ooool2o`00
00[ooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`16ooooo`Coo`00=oooool3o`3o
00Co003o4_ooool4ool005Kooooo1?l00?l^ooooo`Coo`00>_ooool4o`00oaSooooo1?l00?lBoooo
o`Coo`009_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`16ooooo`Co
o`00=oooool3o`3o00Co003o4_ooool4ool005Kooooo1?l00?l^ooooo`Coo`00>_ooool4o`00oaSo
oooo1?l00?lBooooo`Coo`009_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003ooooo
ooooo`16ooooo`Coo`00=oooool3o`3o00Co003o4_ooool4ool005Kooooo1?l00?l^ooooo`Coo`00
>_ooool4o`00oaSooooo1?l00?lBooooo`Coo`009_ooool00ol0003oooooooooo`0;ooooo`008ooo
ool00ol0003oooooooooo`16ooooo`Coo`00=oooool3o`3o00Co003o4_ooool4ool005Kooooo1?l0
0?m/ooooo`Co003o6?ooool4o`00oa;ooooo1?oo000Vooooo`03o`000?oooooooooo00_ooooo000S
ooooo`03o`000?oooooooooo04Kooooo1?oo000gooooo`?o0?l01?l00?lBooooo`Coo`00E_ooool4
o`00ofcooooo1?l00?lHooooo`Co003o4_ooool4ool002Kooooo00?o0000ooooooooool02oooool0
02?ooooo00?o0000ooooooooool0A_ooool4ool003Oooooo0ol0o`04o`00oa;ooooo1?oo001Foooo
o`Co003oK?ooool4o`00obkooooo1?oo000Vooooo`03o`000?oooooooooo00_ooooo000Sooooo`03
o`000?oooooooooo04Kooooo1?oo000gooooo`?o0?l01?l00?lBooooo`Coo`00E_ooool4o`00ofco
oooo1?l00?l^ooooo`Coo`009_ooool00ol0003oooooooooo`0;ooooo`008oooool3o`0004Kooooo
1?oo000gooooo`?o0?l01?l00?lBooooo`Coo`00E_ooool4o`00ofcooooo1?l00?l^ooooo`Coo`00
9Oooool2o`0000gooooo000Sooooo`03o`000?oooooooooo04Kooooo1?oo000gooooo`?o0?l01?l0
0?lBooooo`Coo`00E_ooool4o`00ofcooooo1?l00?l^ooooo`Coo`009_ooool00ol0003ooooooooo
o`0;ooooo`008oooool00ol0003oooooooooo`16ooooo`Coo`00=oooool3o`3o00Co003o4_ooool4
ool005Kooooo1?l00?nNooooo`Coo`009_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0
003oooooooooo`16ooooo`Coo`00=oooool3o`3o00Co003o4_ooool4ool005Kooooo1?l00?nNoooo
o`Coo`009_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`16ooooo`Co
o`00=oooool3o`3o00Co003o4_ooool4ool005Kooooo1?l00?nNooooo`Coo`009_ooool00ol0003o
ooooooooo`0;ooooo`008oooool00ol0003oooooooooo`16ooooo`Coo`00=oooool3o`3o00Co003o
4_ooool4ool005Kooooo1?l00?nNooooo`Coo`009_ooool00ol0003oooooooooo`0;ooooo`008ooo
ool00ol0003oooooooooo`16ooooo`Coo`00=oooool3o`3o00Co003o4_ooool4ool00?Sooooo1?oo
000Vooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo04Kooooo1?oo000g
ooooo`?o0?l01?l00?lBooooo`Coo`00n?ooool4ool002Kooooo00?o0000ooooooooool02oooool0
02?ooooo00?o0000ooooooooool0A_ooool4ool003Oooooo0ol0o`04o`00oa;ooooo1?oo003hoooo
o`Coo`009_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`16ooooo`Co
o`00=oooool3o`3o00Co003o4_ooool4ool00?oooooo8oooool00ol0003oooooooooo`0;ooooo`00
8oooool3o`0004Kooooo1?oo000gooooo`?o0?l01?l00?lBooooo`Coo`00oooooolRooooo`;o0000
3Oooool002?ooooo00?o0000ooooooooool0A_ooool4ool003Oooooo0ol0o`04o`00oa;ooooo1?oo
003oooooob?ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0A_ooool4
ool003Oooooo0ol0o`04o`00oa;ooooo1?oo003oooooob?ooooo00?o0000ooooooooool02oooool0
02?ooooo00?o0000ooooooooool0A_ooool4ool003Oooooo0ol0o`04o`00oa;ooooo1?oo003ooooo
ob?ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0A_ooool4ool003Oo
oooo0ol0o`04o`00oooooooo>Oooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003ooooo
ooooo`16ooooo`Coo`00=oooool3o`3o00Co003oooooooliooooo`03o`000?oooooooooo00_ooooo
000Sooooo`03o`000?oooooooooo04Kooooo1?oo000gooooo`?o0?l01?l00?ooooooocWooooo00?o
0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0A_ooool4ool003Oooooo0ol0o`04
o`00oooooooo>Oooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21oooo
o`?o0?l01?l00?ooooooocWooooo00?o0000ooooooooool02oooool002?ooooo0ol00021ooooo`?o
0?l01?l00?ooooooocSooooo0_l0000=ooooo`008oooool00ol0003oooooooooo`21ooooo`?o0?l0
1?l00?ooooooocWooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POoo
ool3o`3o00Co003oooooooliooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooo
oooo087ooooo0ol0o`04o`00oooooooo>Oooool00ol0003oooooooooo`0;ooooo`008oooool00ol0
003oooooooooo`21ooooo`?o0?l01?l00?ooooooocWooooo00?o0000ooooooooool02oooool002?o
oooo00?o0000ooooooooool0POooool3o`3o00Co003oooooooliooooo`03o`000?oooooooooo00_o
oooo000Sooooo`03o`000?oooooooooo087ooooo0ol0o`04o`00oooooooo>Oooool00ol0003ooooo
ooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`?o0?l01?l00?ooooooocWooooo00?o
0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool3o`3o00Co003oooooooli
ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo0ol0o`04o`00
oooooooo>Oooool00ol0003oooooooooo`0;ooooo`008oooool3o`00087ooooo0ol0o`04o`00oooo
oooo>?ooool2o`0000gooooo000Sooooo`03o`000?oooooooooo087ooooo0ol0o`04o`00oooooooo
>Oooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`?o0?l01?l0
0?ooooooocWooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool3
o`3o00Co003oooooooliooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo
087ooooo0ol0o`04o`00oooooooo>Oooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003o
ooooooooo`21ooooo`?o0?l01?l00?ooooooocWooooo00?o0000ooooooooool02oooool002?ooooo
00?o0000ooooooooool0POooool3o`3o00Co003oooooooliooooo`03o`000?oooooooooo00_ooooo
000Sooooo`03o`000?oooooooooo087ooooo0ol0o`04o`00oooooooo>Oooool00ol0003ooooooooo
o`0;ooooo`003_ooool2o`0000Cooooo00Co0000oooooooooooooooo1Ol00006ooooo`03o`000?oo
oooooooo087ooooo0ol0o`04o`00oooooooo>Oooool00ol0003oooooooooo`0;ooooo`003Oooool0
1?l0003oooooooooool00008ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool0
POooool3o`3o00Co003oooooooliooooo`03o`000?oooooooooo00_ooooo000=ooooo`04o`000?oo
ooooooooo`0000Wooooo00?o0000ooooooooool01_ooool4o`00083ooooo0ol0o`04o`00oooooooo
=_ooool4o`0000gooooo000=ooooo`04o`000?ooooooooooo`0000[ooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`21ooooo`?o0?l01?l00?ooooooocWooooo00?o0000ooooooooool0
2oooool000gooooo00Co0000oooooooooooo00001oooool01Ol0003ooooooooooooooooo000000Ko
oooo00?o0000ooooooooool0POooool3o`3o00Co003oooooooliooooo`03o`000?oooooooooo00_o
oooo000=ooooo`04o`000?ooooooooooo`0000Oooooo00Go0000ooooooooooooooooo`000006oooo
o`03o`000?oooooooooo087ooooo0ol0o`04o`00oooooooo>Oooool00ol0003oooooooooo`0;oooo
o`003_ooool2o`0000Wooooo0ol00007ooooo`03o`000?oooooooooo087ooooo0ol0o`04o`00oooo
oooo>Oooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`?o0?l0
1?l00?ooooooocWooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POoo
ool3o`3o00Co003oooooooliooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooo
oooo087ooooo0ol0o`04o`00oooooooo>Oooool00ol0003oooooooooo`0;ooooo`008oooool00ol0
003oooooooooo`21ooooo`?o0?l01?l00?ooooooocWooooo00?o0000ooooooooool02oooool002?o
oooo00?o0000ooooooooool0POooool3o`3o00Co003oooooooliooooo`03o`000?oooooooooo00_o
oooo000Sooooo`?o0000POooool3o`3o00Co003ooooooolhooooo`;o00003Oooool002?ooooo00?o
0000ooooooooool0POooool3o`3o00Co003oooooooliooooo`03o`000?oooooooooo00_ooooo000S
ooooo`03o`000?oooooooooo087ooooo0ol0o`04o`00oooooooo>Oooool00ol0003oooooooooo`0;
ooooo`008oooool00ol0003oooooooooo`21ooooo`?o0?l01?l00?ooooooocWooooo00?o0000oooo
ooooool02oooool002?ooooo00?o0000ooooooooool0POooool3o`3o00Co003oooooooliooooo`03
o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo0ol0o`04o`00oooooooo
>Oooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`?o0?l01?l0
0?ooooooocWooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool3
o`3o00Co003oooooooliooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo
087ooooo0ol0o`04o`00oooooooo>Oooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003o
ooooooooo`21ooooo`?o0?l01?l00?ooooooocWooooo00?o0000ooooooooool02oooool002?ooooo
0ol00021ooooo`?o0?l01?l00?ooooooocSooooo0_l0000=ooooo`008oooool00ol0003ooooooooo
o`21ooooo`?o0?l01?l00?ooooooocWooooo00?o0000ooooooooool02oooool002?ooooo00?o0000
ooooooooool0POooool3o`3o00Co003oooooooliooooo`03o`000?oooooooooo00_ooooo000Soooo
o`03o`000?oooooooooo087ooooo0ol0o`04o`00oooooooo>Oooool00ol0003oooooooooo`0;oooo
o`008oooool00ol0003oooooooooo`21ooooo`?o0?l01?l00?ooooooocWooooo00?o0000oooooooo
ool02oooool002?ooooo00?o0000ooooooooool0POooool3o`3o00Co003oooooooliooooo`03o`00
0?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo0ol0o`04o`00oooooooo>Ooo
ool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`?o0?l01?l00?oo
ooooocWooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool3o`3o
00Co003oooooooliooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo087o
oooo0ol0o`04o`00oooooooo>Oooool00ol0003oooooooooo`0;ooooo`008oooool3o`00087ooooo
0ol0o`04o`00oooooooo>?ooool2o`0000gooooo000Sooooo`03o`000?oooooooooo087ooooo0ol0
o`04o`00oooooooo>Oooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21
ooooo`?o0?l01?l00?ooooooocWooooo00?o0000ooooooooool02oooool002?ooooo00?o0000oooo
ooooool0POooool3o`3o00Co003oooooooliooooo`03o`000?oooooooooo00_ooooo000Sooooo`03
o`000?oooooooooo087ooooo0ol0o`04o`00oooooooo>Oooool00ol0003oooooooooo`0;ooooo`00
8oooool00ol0003oooooooooo`21ooooo`?o0?l01?l00?ooooooocWooooo00?o0000ooooooooool0
2oooool002?ooooo00?o0000ooooooooool0POooool3o`3o00Co003oooooooliooooo`03o`000?oo
oooooooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo0ol0o`04o`00oooooooo>Oooool0
0ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`?o0?l01?l00?oooooo
ocWooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool3o`3o00Co
003oooooooliooooo`03o`000?oooooooooo00_ooooo000Sooooo`?o0000POooool3o`3o00Co003o
oooooolhooooo`;o00003Oooool002?ooooo00?o0000ooooooooool0POooool3o`3o00Co003ooooo
ooliooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3o
oooooccooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o
0?oooooo??ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`Co
0?l0oooooollooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo
1?l0o`3ooooooccooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POoo
ool4o`3o0?oooooo??ooool00ol0003oooooooooo`0;ooooo`003_ooool2o`0000Cooooo00?o0000
ooooooooool00_ooool3o`0000Oooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??ooool0
0ol0003oooooooooo`0;ooooo`003Oooool01?l0003oooooooooool00007ooooo`05o`000?oooooo
ooooooooool000001_ooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oo
oooooooo00_ooooo000=ooooo`04o`000?ooooooooooo`0000_ooooo00?o0000ooooooooool01?oo
ool4o`00083ooooo1?l0o`3oooooocWooooo1?l0000=ooooo`003Oooool01?l0003oooooooooool0
0009ooooo`;o00001oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oo
oooooooo00_ooooo000=ooooo`04o`000?ooooooooooo`0000_ooooo00?o0000ooooooooool01?oo
ool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oooooooooo00_ooooo000=
ooooo`04o`000?ooooooooooo`0000Oooooo00Go0000ooooooooooooooooo`000006ooooo`03o`00
0?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o0000ooooooooool02oooool000kooooo0_l0
0009ooooo`?o00001oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oo
oooooooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o0000
ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??ooool00ol0
003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03
o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo
00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??oo
ool00ol0003oooooooooo`0;ooooo`008oooool3o`00087ooooo1?l0o`3ooooooc_ooooo0_l0000=
ooooo`008oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oooooooooo
00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o0000oooooooo
ool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??ooool00ol0003ooooo
ooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oo
oooooooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o0000
ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??ooool00ol0
003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03
o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo
00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??oo
ool00ol0003oooooooooo`0;ooooo`008oooool3o`00087ooooo1?l0o`3ooooooc_ooooo0_l0000=
ooooo`008oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oooooooooo
00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o0000oooooooo
ool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??ooool00ol0003ooooo
ooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oo
oooooooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o0000
ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??ooool00ol0
003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03
o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo
00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??oo
ool00ol0003oooooooooo`0;ooooo`008oooool3o`00087ooooo1?l0o`3ooooooc_ooooo0_l0000=
ooooo`008oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oooooooooo
00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o0000oooooooo
ool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??ooool00ol0003ooooo
ooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oo
oooooooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o0000
ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??ooool00ol0
003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03
o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo
00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??oo
ool00ol0003oooooooooo`0;ooooo`008oooool3o`00087ooooo1?l0o`3ooooooc_ooooo0_l0000=
ooooo`008oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oooooooooo
00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o0000oooooooo
ool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??ooool00ol0003ooooo
ooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oo
oooooooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o0000
ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??ooool00ol0
003oooooooooo`0;ooooo`003_ooool2o`0000Cooooo00?o0000ooooooooool00oooool3o`0000Ko
oooo00?o0000ooooooooool0POooool4o`3o0?oooooo??ooool00ol0003oooooooooo`0;ooooo`00
3Oooool01?l0003oooooooooool0000:ooooo`03o`000?oooooooooo00Gooooo00?o0000oooooooo
ool0POooool4o`3o0?oooooo??ooool00ol0003oooooooooo`0;ooooo`003Oooool01?l0003ooooo
ooooool00007ooooo`Go00001_ooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03
o`000?oooooooooo00_ooooo000=ooooo`04o`000?ooooooooooo`0000Oooooo00Co0000oooooooo
oooo00001oooool4o`00083ooooo1?l0o`3oooooocWooooo1?l0000=ooooo`003Oooool01?l0003o
ooooooooool00008ooooo`03o`000?oooooo000000Oooooo00?o0000ooooooooool0POooool4o`3o
0?oooooo??ooool00ol0003oooooooooo`0;ooooo`003Oooool01?l0003oooooooooool00009oooo
o`;o00001oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oooooooooo
00_ooooo000>ooooo`;o00002oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo087o
oooo1?l0o`3ooooooccooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0
POooool4o`3o0?oooooo??ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003ooooooooo
o`21ooooo`Co0?l0oooooollooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooo
oooo087ooooo1?l0o`3ooooooccooooo00?o0000ooooooooool02oooool002?ooooo00?o0000oooo
ooooool0POooool4o`3o0?oooooo??ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003o
ooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oooooooooo00_ooooo000Sooooo`?o0000
POooool4o`3o0?oooooo>oooool2o`0000gooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0
o`3ooooooccooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool4
o`3o0?oooooo??ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21oooo
o`Co0?l0oooooollooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo087o
oooo1?l0o`3ooooooccooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0
POooool4o`3o0?oooooo??ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003ooooooooo
o`21ooooo`Co0?l0oooooollooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooo
oooo087ooooo1?l0o`3ooooooccooooo00?o0000ooooooooool02oooool002?ooooo00?o0000oooo
ooooool0POooool4o`3o0?oooooo??ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003o
ooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oooooooooo00_ooooo000Sooooo`?o0000
POooool4o`3o0?oooooo>oooool2o`0000gooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0
o`3ooooooccooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool4
o`3o0?oooooo??ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21oooo
o`Co0?l0oooooollooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo087o
oooo1?l0o`3ooooooccooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0
POooool4o`3o0?oooooo??ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003ooooooooo
o`21ooooo`Co0?l0oooooollooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooo
oooo087ooooo1?l0o`3ooooooccooooo00?o0000ooooooooool02oooool002?ooooo00?o0000oooo
ooooool0POooool4o`3o0?oooooo??ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003o
ooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oooooooooo00_ooooo000Sooooo`?o0000
POooool4o`3o0?oooooo>oooool2o`0000gooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0
o`3ooooooccooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool4
o`3o0?oooooo??ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21oooo
o`Co0?l0oooooollooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo087o
oooo1?l0o`3ooooooccooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0
POooool4o`3o0?oooooo??ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003ooooooooo
o`21ooooo`Co0?l0oooooollooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooo
oooo087ooooo1?l0o`3ooooooccooooo00?o0000ooooooooool02oooool002?ooooo00?o0000oooo
ooooool0POooool4o`3o0?oooooo??ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003o
ooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oooooooooo00_ooooo000Sooooo`?o0000
POooool4o`3o0?oooooo>oooool2o`0000gooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0
o`3ooooooccooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool4
o`3o0?oooooo??ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21oooo
o`Co0?l0oooooollooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo087o
oooo1?l0o`3ooooooccooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0
POooool4o`3o0?oooooo??ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003ooooooooo
o`21ooooo`Co0?l0oooooollooooo`03o`000?oooooooooo00_ooooo000>ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`?o00001oooool00ol0003oooooooooo`21ooooo`Co0?l0ooooooll
ooooo`03o`000?oooooooooo00_ooooo000=ooooo`04o`000?ooooooooooo`0000Oooooo00Go0000
ooooooooooooooooo`000006ooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o
0000ooooooooool02oooool000gooooo00Co0000oooooooooooo00002oooool00ol0003ooooooooo
o`04ooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o0000ooooooooool02ooo
ool000gooooo00Co0000oooooooooooo00002oooool00ol0003oooooooooo`04ooooo`Co0000P?oo
ool4o`3o0?oooooo>Oooool4o`0000gooooo000=ooooo`04o`000?ooooooooooo`0000Oooooo1?l0
0007ooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o0000ooooooooool02ooo
ool000gooooo00Co0000oooooooooooo00001oooool00ol0003oooooooooo`08ooooo`03o`000?oo
oooooooo087ooooo1?l0o`3ooooooccooooo00?o0000ooooooooool02oooool000kooooo0_l00008
ooooo`Go00001_ooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oooooo
oooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o0000oooo
ooooool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??ooool00ol0003o
ooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03o`00
0?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o
0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??ooool0
0ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooolloooo
o`03o`000?oooooooooo00_ooooo000Sooooo`?o0000POooool4o`3o0?oooooo>oooool2o`0000go
oooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o0000ooooooooool0
2oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??ooool00ol0003ooooooooo
o`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03o`000?oooooo
oooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o0000oooo
ooooool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??ooool00ol0003o
ooooooooo`0;ooooo`008oooool00ol0003oooooooooo`21ooooo`Co0?l0oooooollooooo`03o`00
0?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo087ooooo1?l0o`3ooooooccooooo00?o
0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0POooool4o`3o0?oooooo??ooool0
0ol0003oooooooooo`0;ooooo`008oooool3o`00087ooooo1?l0o`3ooooooc_ooooo0_l0000=oooo
o`008oooool00ol0003oooooooooo`3ooooool7ooooo00?o0000ooooooooool02oooool002?ooooo
00?o0000ooooooooool0ooooooo1ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oo
oooooooo0?oooooo`Oooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`06
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0;
ooooo`008oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0;ooooo`00
8oooooooo`000<Go00003Oooool00?oooooomOooool00?oooooomOooool00?oooooomOooool00?oo
oooomOooool00?oooooomOooool00?oooooomOooool00?oooooomOooool002[ooooo0ol0000:oooo
o`Go00002_ooool3o`0000cooooo0ol0000:ooooo`?o00002oooool3o`0000cooooo00?o0000oooo
ooooool02_ooool3o`0000_ooooo0_l00009ooooo`?o00001?ooool2o`0000Gooooo0ol00003oooo
o`?o00001Oooool3o`0000;ooooo1Ol00004ooooo`?o00000oooool3o`0000Gooooo0ol00004oooo
o`?o00001?ooool3o`0000?ooooo0ol00005ooooo`?o00000oooool3o`0000Gooooo0ol00004oooo
o`03o`000?oooooooooo00Cooooo0ol00003ooooo`?o00001Oooool3o`0000?ooooo0_l00005oooo
o`Go00000oooool2o`0000Cooooo1Ol00002ooooo`?o00001?ooool5o`000003ooooool0003o0000
00?o00000oooool5o`0000;ooooo0ol00004ooooo`Go00000oooool3o`0000?ooooo1Ol00002oooo
o`?o00001?ooool5o`0000;ooooo0ol00004ooooo`Go00000oooool00ol0003oooooooooo`03oooo
o`Go00000_ooool3o`0000Cooooo1Ol00002ooooo`;o00001_ooool3o`0000Cooooo0_l00005oooo
o`?o00000oooool3o`0000Gooooo0ol00002ooooo`Go00004Oooool002_ooooo00?o0000oooooooo
ool02_ooool01?l0003oooooooooool00009ooooo`05o`000?ooooooooooooooool000003?ooool0
0ol0003oooooooooo`08ooooo`05o`000?ooooooooooooooool000002Oooool01Ol0003ooooooooo
oooooooo000000_ooooo00?o0000ooooooooool02Oooool01Ol0003ooooooooooooooooo000000co
oooo00?o0000ooooooooool01oooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`00
00Gooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00;ooooo00Co0000oooooooooooo00001Oooool01Ol0003ooooooooooooooooo000000?ooooo00?o
0000ooooooooool00oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00Go
0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00Go0000oooooooooooo
ooooo`000003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`05ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00oooool01?l0003oooooooooool00002
ooooo`04o`000?ooooooooooo`0000Cooooo00Co0000oooooooooooo00000oooool00ol0003ooooo
ooooo`04ooooo`04o`000?ooooooooooo`0000;ooooo00Co0000oooooooooooo00001?ooool01_l0
003oooooooooool0003oooooo`0000?ooooo00?o0000ooooooooool00_ooool01?l0003ooooooooo
ool00004ooooo`03o`000?oooooooooo00?ooooo00Ko0000oooooooooooo0000ooooool00003oooo
o`03o`000?oooooooooo00;ooooo00Ko0000oooooooooooo0000ooooool00003ooooo`03o`000?oo
oooooooo00;ooooo00Co0000oooooooooooo00000oooool00ol0003oooooooooo`04ooooo`06o`00
0?ooooooooooo`000?oooooo00000oooool00ol0003oooooooooo`02ooooo`04o`000?oooooooooo
o`0000Cooooo00?o0000ooooooooool00_ooool01Ol0003ooooooooooooooooo000000;ooooo00Co
0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000oooooooo
ool00oooool01Ol0003ooooooooooooooooo000000;ooooo00Co0000oooooooooooo00004Oooool0
02_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0=ooooo`03o`000?oooooooooo
00Oooooo1Ol0000=ooooo`03o`000?oooooooooo00Oooooo00Go0000ooooooooooooooooo`00000<
ooooo`03o`000?oooooooooo00Sooooo00Go0000ooooooooooooooooo`00000=ooooo`03o`000?oo
oooooooo00Kooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`03ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`03ooooo`04o`00
0?ooooooooooooooo`Go00001Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00?o
oooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`04ooooo`05o`000?ooooooooooooooool000000oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00oooool0
0ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00?ooooo00Co0000oooooooooooooooo
1Ol00005ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00oooool01Ol0003ooooo
oooooooooooo000000?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Cooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo
00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`04o`000?oooooooooo
o`0000;ooooo00?o0000ooooooooool01Oooool01Ol0003ooooooooooooooooo000000Wooooo00Go
0000ooooooooooooooooo`00000Cooooo`00:oooool00ol0003oooooooooo`0<ooooo`03o`000?oo
oooooooo00[ooooo0_l0000:ooooo`04o`000?ooooooooooo`0000kooooo00?o0000ooooooooool0
1oooool4o`0000gooooo00?o0000ooooooooool02Oooool3o`0000_ooooo1?l00008ooooo`03o`00
0?oooooooooo00;ooooo00Co0000oooooooooooo00001Oooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00?ooooo0_l00006ooooo`05o`000?ooooooooooooooool000000_ooool00ol0
003oooooooooo`04ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00oooool01?l0
003oooooooooooooool4o`0000Kooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00;ooooo0ol00006ooooo`03o`000?oooooooooo00;ooooo1?l00006
ooooo`05o`000?ooooooooooooooool000000_ooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo00;ooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`03ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool00_ooool2o`0000Oooooo00Co0000oooooooooooo0000
0_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`04o`0000Oooooo00?o0000ooooooooool00oooool00ol0003ooooo
ooooo`05ooooo`04o`000?ooooooooooooooo`?o00001oooool01?l0003oooooooooooooool4o`00
00Gooooo0_l00003ooooo`04o`000?ooooooooooo`0000Gooooo0_l00004ooooo`03o`000?oooooo
oooo00Gooooo0_l00005ooooo`03o`000?oooooooooo013ooooo000[ooooo`03o`000?oooooooooo
00Wooooo00Go0000ooooooooooooooooo`00000=ooooo`03o`000?oooooooooo00Sooooo00?o0000
ooooool000002_ooool4o`0000[ooooo00?o0000ooooooooool03oooool00ol0003oooooooooo`07
ooooo`05o`000?ooooooooooooooool000002Oooool01Ol0003ooooooooooooooooo000000Sooooo
00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`03o`000?oooooooooo00?o
oooo00?o0000ooooooooool01Oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool00_ooool00ol0003oooooo`000006ooooo`04o`000?ooooooooooooooo`Co00001_ooool0
1Ol0003ooooooooooooooooo000000Wooooo00?o0000ooooooooool01Oooool00ol0003ooooooooo
o`03ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03ooooo`05o`00
0?ooooooooooooooool000000oooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooo
oooooooo00000_ooool01Ol0003ooooooooooooooooo000000?ooooo00Go0000oooooooooooooooo
o`000005ooooo`07o`000?ooooooooooooooool0003oooooo`000003ooooo`05o`000?oooooooooo
ooooool000000oooool00ol0003oooooooooo`03ooooo`05o`000?ooooooooooooooool000000ooo
ool01_l0003oooooooooool0003oooooo`0000Cooooo00Ko0000ooooooooooooooooo`000?ooool4
o`0000Cooooo00Oo0000ooooooooooooooooo`000?oooooo000000Oooooo00Go0000oooooooooooo
ooooo`000005ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooo`000003oooo
o`05o`000?ooooooooooooooool000000oooool00ol0003oooooo`000003ooooo`03o`000?oooooo
oooo00Gooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`05ooooo`05o`000?oo
ooooooooooooool000002Oooool00ol0003oooooo`000003ooooo`03o`000?oooooooooo00oooooo
000Zooooo`;o00002oooool01Ol0003ooooooooooooooooo000000Wooooo00Go0000oooooooooooo
ooooo`00000;ooooo`;o00002_ooool00ol0003oooooooooo`0<ooooo`03o`000?oooooooooo00[o
oooo00Go0000ooooooooooooooooo`000009ooooo`05o`000?ooooooooooooooool000002Oooool0
1Ol0003ooooooooooooooooo000000Oooooo0_l00004ooooo`04o`000?ooooooooooo`0000Cooooo
0_l00004ooooo`;o00001_ooool2o`0000?ooooo00Go0000ooooooooooooooooo`000004ooooo`;o
00000oooool01Ol0003ooooooooooooooooo000000Cooooo0_l00005ooooo`;o00001Oooool2o`00
00?ooooo00?o0000ooooooooool01_ooool2o`0000Cooooo00?o0000ooooooooool01Oooool2o`00
00?ooooo00Go0000ooooooooooooooooo`000004ooooo`;o00000oooool01Ol0003ooooooooooooo
oooo000000Cooooo0_l00003ooooo`05o`000?ooooooooooooooool000000oooool01Ol0003ooooo
oooooooooooo000000;ooooo00Co0000oooooooooooo00000oooool01Ol0003ooooooooooooooooo
000000;ooooo0_l00005ooooo`07o`000?ooooooooooooooool0003oooooo`000003ooooo`05o`00
0?ooooooooooooooool000000oooool00ol0003oooooo`000003ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooooooool2o`0000Cooooo00Oo0000ooooooooooooooooo`00
0?oooooo000000Oooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oooooooooo00Co
oooo00Oo0000ooooooooooooooooo`000?oooooo000000?ooooo00Go0000ooooooooooooooooo`00
0003ooooo`03o`000?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooo`00
00;ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00;o00001Oooool0
1ol0003ooooooooooooooooo0000ooooool000000oooool00ol0003oooooooooo`0?ooooo`00:ooo
ool00ol0003oooooooooo`0:ooooo`?o00002oooool3o`0000gooooo00?o0000ooooooooool02?oo
ool5o`0000_ooooo0_l0000:ooooo`Go00002_ooool3o`0000_ooooo0ol00009ooooo`03o`000?oo
oooooooo00?ooooo0_l00006ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`02ooooo`?o00001_ooool00ol0003oooooooooo`02ooooo`?o00001_oo
ool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooooooo
1Ol00005ooooo`03o`000?oooooooooo00?ooooo0_l00006ooooo`04o`000?ooooooooooooooo`Go
00001Oooool00ol0003oooooooooo`02ooooo`?o00001_ooool00ol0003oooooooooo`02ooooo`?o
00001Oooool3o`0000Cooooo0_l00005ooooo`?o00001?ooool00ol0003oooooooooo`04ooooo`?o
00000oooool3o`0000Gooooo0ol00003ooooo`?o00001Oooool3o`0000Gooooo00?o0000oooooooo
ool00oooool3o`0000;ooooo1Ol00004ooooo`?o00001?ooool2o`0000Gooooo0ol00002ooooo`Go
00001?ooool3o`0000?ooooo0ol00005ooooo`?o00000oooool3o`0000Gooooo0ol00004ooooo`;o
00001Oooool3o`0000Cooooo00?o0000ooooooooool01?ooool3o`0000?ooooo0ol0000Booooo`00
\
\>"],
  ImageRangeCache->{{{0, 499}, {308, 0}} -> {-1.85349, -0.0432382, 0.0713992, \
0.00203234}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
%%ImageSize: 500 309.017 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0217391 0.0310559 0.0147151 0.904007 [
[.01863 -0.0125 -3 -9 ]
[.01863 -0.0125 3 0 ]
[.04969 -0.0125 -3 -9 ]
[.04969 -0.0125 3 0 ]
[.08075 -0.0125 -3 -9 ]
[.08075 -0.0125 3 0 ]
[.1118 -0.0125 -3 -9 ]
[.1118 -0.0125 3 0 ]
[.14286 -0.0125 -3 -9 ]
[.14286 -0.0125 3 0 ]
[.17391 -0.0125 -3 -9 ]
[.17391 -0.0125 3 0 ]
[.20497 -0.0125 -3 -9 ]
[.20497 -0.0125 3 0 ]
[.23602 -0.0125 -3 -9 ]
[.23602 -0.0125 3 0 ]
[.26708 -0.0125 -3 -9 ]
[.26708 -0.0125 3 0 ]
[.29814 -0.0125 -6 -9 ]
[.29814 -0.0125 6 0 ]
[.32919 -0.0125 -6 -9 ]
[.32919 -0.0125 6 0 ]
[.36025 -0.0125 -6 -9 ]
[.36025 -0.0125 6 0 ]
[.3913 -0.0125 -6 -9 ]
[.3913 -0.0125 6 0 ]
[.42236 -0.0125 -6 -9 ]
[.42236 -0.0125 6 0 ]
[.45342 -0.0125 -6 -9 ]
[.45342 -0.0125 6 0 ]
[.48447 -0.0125 -6 -9 ]
[.48447 -0.0125 6 0 ]
[.51553 -0.0125 -6 -9 ]
[.51553 -0.0125 6 0 ]
[.54658 -0.0125 -6 -9 ]
[.54658 -0.0125 6 0 ]
[.57764 -0.0125 -6 -9 ]
[.57764 -0.0125 6 0 ]
[.6087 -0.0125 -6 -9 ]
[.6087 -0.0125 6 0 ]
[.63975 -0.0125 -6 -9 ]
[.63975 -0.0125 6 0 ]
[.67081 -0.0125 -6 -9 ]
[.67081 -0.0125 6 0 ]
[.70186 -0.0125 -6 -9 ]
[.70186 -0.0125 6 0 ]
[.73292 -0.0125 -6 -9 ]
[.73292 -0.0125 6 0 ]
[.76398 -0.0125 -6 -9 ]
[.76398 -0.0125 6 0 ]
[.79503 -0.0125 -6 -9 ]
[.79503 -0.0125 6 0 ]
[.82609 -0.0125 -6 -9 ]
[.82609 -0.0125 6 0 ]
[.85714 -0.0125 -6 -9 ]
[.85714 -0.0125 6 0 ]
[.8882 -0.0125 -6 -9 ]
[.8882 -0.0125 6 0 ]
[.91925 -0.0125 -6 -9 ]
[.91925 -0.0125 6 0 ]
[.95031 -0.0125 -6 -9 ]
[.95031 -0.0125 6 0 ]
[.98137 -0.0125 -6 -9 ]
[.98137 -0.0125 6 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .01472 -6 -4.5 ]
[-0.0125 .01472 0 4.5 ]
[-0.0125 .10512 -18 -4.5 ]
[-0.0125 .10512 0 4.5 ]
[-0.0125 .19552 -18 -4.5 ]
[-0.0125 .19552 0 4.5 ]
[-0.0125 .28592 -18 -4.5 ]
[-0.0125 .28592 0 4.5 ]
[-0.0125 .37632 -18 -4.5 ]
[-0.0125 .37632 0 4.5 ]
[-0.0125 .46672 -18 -4.5 ]
[-0.0125 .46672 0 4.5 ]
[-0.0125 .55712 -18 -4.5 ]
[-0.0125 .55712 0 4.5 ]
[ 0 0 -0.125 0 ]
[.01863 .63053 -3 0 ]
[.01863 .63053 3 9 ]
[.04969 .63053 -3 0 ]
[.04969 .63053 3 9 ]
[.08075 .63053 -3 0 ]
[.08075 .63053 3 9 ]
[.1118 .63053 -3 0 ]
[.1118 .63053 3 9 ]
[.14286 .63053 -3 0 ]
[.14286 .63053 3 9 ]
[.17391 .63053 -3 0 ]
[.17391 .63053 3 9 ]
[.20497 .63053 -3 0 ]
[.20497 .63053 3 9 ]
[.23602 .63053 -3 0 ]
[.23602 .63053 3 9 ]
[.26708 .63053 -3 0 ]
[.26708 .63053 3 9 ]
[.29814 .63053 -6 0 ]
[.29814 .63053 6 9 ]
[.32919 .63053 -6 0 ]
[.32919 .63053 6 9 ]
[.36025 .63053 -6 0 ]
[.36025 .63053 6 9 ]
[.3913 .63053 -6 0 ]
[.3913 .63053 6 9 ]
[.42236 .63053 -6 0 ]
[.42236 .63053 6 9 ]
[.45342 .63053 -6 0 ]
[.45342 .63053 6 9 ]
[.48447 .63053 -6 0 ]
[.48447 .63053 6 9 ]
[.51553 .63053 -6 0 ]
[.51553 .63053 6 9 ]
[.54658 .63053 -6 0 ]
[.54658 .63053 6 9 ]
[.57764 .63053 -6 0 ]
[.57764 .63053 6 9 ]
[.6087 .63053 -6 0 ]
[.6087 .63053 6 9 ]
[.63975 .63053 -6 0 ]
[.63975 .63053 6 9 ]
[.67081 .63053 -6 0 ]
[.67081 .63053 6 9 ]
[.70186 .63053 -6 0 ]
[.70186 .63053 6 9 ]
[.73292 .63053 -6 0 ]
[.73292 .63053 6 9 ]
[.76398 .63053 -6 0 ]
[.76398 .63053 6 9 ]
[.79503 .63053 -6 0 ]
[.79503 .63053 6 9 ]
[.82609 .63053 -6 0 ]
[.82609 .63053 6 9 ]
[.85714 .63053 -6 0 ]
[.85714 .63053 6 9 ]
[.8882 .63053 -6 0 ]
[.8882 .63053 6 9 ]
[.91925 .63053 -6 0 ]
[.91925 .63053 6 9 ]
[.95031 .63053 -6 0 ]
[.95031 .63053 6 9 ]
[.98137 .63053 -6 0 ]
[.98137 .63053 6 9 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.01863 0 m
.01863 .00625 L
s
[(1)] .01863 -0.0125 0 1 Mshowa
.04969 0 m
.04969 .00625 L
s
[(2)] .04969 -0.0125 0 1 Mshowa
.08075 0 m
.08075 .00625 L
s
[(3)] .08075 -0.0125 0 1 Mshowa
.1118 0 m
.1118 .00625 L
s
[(4)] .1118 -0.0125 0 1 Mshowa
.14286 0 m
.14286 .00625 L
s
[(5)] .14286 -0.0125 0 1 Mshowa
.17391 0 m
.17391 .00625 L
s
[(6)] .17391 -0.0125 0 1 Mshowa
.20497 0 m
.20497 .00625 L
s
[(7)] .20497 -0.0125 0 1 Mshowa
.23602 0 m
.23602 .00625 L
s
[(8)] .23602 -0.0125 0 1 Mshowa
.26708 0 m
.26708 .00625 L
s
[(9)] .26708 -0.0125 0 1 Mshowa
.29814 0 m
.29814 .00625 L
s
[(10)] .29814 -0.0125 0 1 Mshowa
.32919 0 m
.32919 .00625 L
s
[(11)] .32919 -0.0125 0 1 Mshowa
.36025 0 m
.36025 .00625 L
s
[(12)] .36025 -0.0125 0 1 Mshowa
.3913 0 m
.3913 .00625 L
s
[(13)] .3913 -0.0125 0 1 Mshowa
.42236 0 m
.42236 .00625 L
s
[(14)] .42236 -0.0125 0 1 Mshowa
.45342 0 m
.45342 .00625 L
s
[(15)] .45342 -0.0125 0 1 Mshowa
.48447 0 m
.48447 .00625 L
s
[(16)] .48447 -0.0125 0 1 Mshowa
.51553 0 m
.51553 .00625 L
s
[(17)] .51553 -0.0125 0 1 Mshowa
.54658 0 m
.54658 .00625 L
s
[(18)] .54658 -0.0125 0 1 Mshowa
.57764 0 m
.57764 .00625 L
s
[(19)] .57764 -0.0125 0 1 Mshowa
.6087 0 m
.6087 .00625 L
s
[(20)] .6087 -0.0125 0 1 Mshowa
.63975 0 m
.63975 .00625 L
s
[(21)] .63975 -0.0125 0 1 Mshowa
.67081 0 m
.67081 .00625 L
s
[(22)] .67081 -0.0125 0 1 Mshowa
.70186 0 m
.70186 .00625 L
s
[(23)] .70186 -0.0125 0 1 Mshowa
.73292 0 m
.73292 .00625 L
s
[(24)] .73292 -0.0125 0 1 Mshowa
.76398 0 m
.76398 .00625 L
s
[(25)] .76398 -0.0125 0 1 Mshowa
.79503 0 m
.79503 .00625 L
s
[(26)] .79503 -0.0125 0 1 Mshowa
.82609 0 m
.82609 .00625 L
s
[(27)] .82609 -0.0125 0 1 Mshowa
.85714 0 m
.85714 .00625 L
s
[(28)] .85714 -0.0125 0 1 Mshowa
.8882 0 m
.8882 .00625 L
s
[(29)] .8882 -0.0125 0 1 Mshowa
.91925 0 m
.91925 .00625 L
s
[(30)] .91925 -0.0125 0 1 Mshowa
.95031 0 m
.95031 .00625 L
s
[(31)] .95031 -0.0125 0 1 Mshowa
.98137 0 m
.98137 .00625 L
s
[(32)] .98137 -0.0125 0 1 Mshowa
0 0 m
1 0 L
s
0 .01472 m
.00625 .01472 L
s
[(0)] -0.0125 .01472 1 0 Mshowa
0 .10512 m
.00625 .10512 L
s
[(0.1)] -0.0125 .10512 1 0 Mshowa
0 .19552 m
.00625 .19552 L
s
[(0.2)] -0.0125 .19552 1 0 Mshowa
0 .28592 m
.00625 .28592 L
s
[(0.3)] -0.0125 .28592 1 0 Mshowa
0 .37632 m
.00625 .37632 L
s
[(0.4)] -0.0125 .37632 1 0 Mshowa
0 .46672 m
.00625 .46672 L
s
[(0.5)] -0.0125 .46672 1 0 Mshowa
0 .55712 m
.00625 .55712 L
s
[(0.6)] -0.0125 .55712 1 0 Mshowa
.125 Mabswid
0 .0328 m
.00375 .0328 L
s
0 .05088 m
.00375 .05088 L
s
0 .06896 m
.00375 .06896 L
s
0 .08704 m
.00375 .08704 L
s
0 .1232 m
.00375 .1232 L
s
0 .14128 m
.00375 .14128 L
s
0 .15936 m
.00375 .15936 L
s
0 .17744 m
.00375 .17744 L
s
0 .2136 m
.00375 .2136 L
s
0 .23168 m
.00375 .23168 L
s
0 .24976 m
.00375 .24976 L
s
0 .26784 m
.00375 .26784 L
s
0 .304 m
.00375 .304 L
s
0 .32208 m
.00375 .32208 L
s
0 .34016 m
.00375 .34016 L
s
0 .35824 m
.00375 .35824 L
s
0 .3944 m
.00375 .3944 L
s
0 .41248 m
.00375 .41248 L
s
0 .43056 m
.00375 .43056 L
s
0 .44864 m
.00375 .44864 L
s
0 .4848 m
.00375 .4848 L
s
0 .50288 m
.00375 .50288 L
s
0 .52096 m
.00375 .52096 L
s
0 .53904 m
.00375 .53904 L
s
0 .5752 m
.00375 .5752 L
s
0 .59328 m
.00375 .59328 L
s
0 .61136 m
.00375 .61136 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.01863 .61178 m
.01863 .61803 L
s
[(1)] .01863 .63053 0 -1 Mshowa
.04969 .61178 m
.04969 .61803 L
s
[(2)] .04969 .63053 0 -1 Mshowa
.08075 .61178 m
.08075 .61803 L
s
[(3)] .08075 .63053 0 -1 Mshowa
.1118 .61178 m
.1118 .61803 L
s
[(4)] .1118 .63053 0 -1 Mshowa
.14286 .61178 m
.14286 .61803 L
s
[(5)] .14286 .63053 0 -1 Mshowa
.17391 .61178 m
.17391 .61803 L
s
[(6)] .17391 .63053 0 -1 Mshowa
.20497 .61178 m
.20497 .61803 L
s
[(7)] .20497 .63053 0 -1 Mshowa
.23602 .61178 m
.23602 .61803 L
s
[(8)] .23602 .63053 0 -1 Mshowa
.26708 .61178 m
.26708 .61803 L
s
[(9)] .26708 .63053 0 -1 Mshowa
.29814 .61178 m
.29814 .61803 L
s
[(10)] .29814 .63053 0 -1 Mshowa
.32919 .61178 m
.32919 .61803 L
s
[(11)] .32919 .63053 0 -1 Mshowa
.36025 .61178 m
.36025 .61803 L
s
[(12)] .36025 .63053 0 -1 Mshowa
.3913 .61178 m
.3913 .61803 L
s
[(13)] .3913 .63053 0 -1 Mshowa
.42236 .61178 m
.42236 .61803 L
s
[(14)] .42236 .63053 0 -1 Mshowa
.45342 .61178 m
.45342 .61803 L
s
[(15)] .45342 .63053 0 -1 Mshowa
.48447 .61178 m
.48447 .61803 L
s
[(16)] .48447 .63053 0 -1 Mshowa
.51553 .61178 m
.51553 .61803 L
s
[(17)] .51553 .63053 0 -1 Mshowa
.54658 .61178 m
.54658 .61803 L
s
[(18)] .54658 .63053 0 -1 Mshowa
.57764 .61178 m
.57764 .61803 L
s
[(19)] .57764 .63053 0 -1 Mshowa
.6087 .61178 m
.6087 .61803 L
s
[(20)] .6087 .63053 0 -1 Mshowa
.63975 .61178 m
.63975 .61803 L
s
[(21)] .63975 .63053 0 -1 Mshowa
.67081 .61178 m
.67081 .61803 L
s
[(22)] .67081 .63053 0 -1 Mshowa
.70186 .61178 m
.70186 .61803 L
s
[(23)] .70186 .63053 0 -1 Mshowa
.73292 .61178 m
.73292 .61803 L
s
[(24)] .73292 .63053 0 -1 Mshowa
.76398 .61178 m
.76398 .61803 L
s
[(25)] .76398 .63053 0 -1 Mshowa
.79503 .61178 m
.79503 .61803 L
s
[(26)] .79503 .63053 0 -1 Mshowa
.82609 .61178 m
.82609 .61803 L
s
[(27)] .82609 .63053 0 -1 Mshowa
.85714 .61178 m
.85714 .61803 L
s
[(28)] .85714 .63053 0 -1 Mshowa
.8882 .61178 m
.8882 .61803 L
s
[(29)] .8882 .63053 0 -1 Mshowa
.91925 .61178 m
.91925 .61803 L
s
[(30)] .91925 .63053 0 -1 Mshowa
.95031 .61178 m
.95031 .61803 L
s
[(31)] .95031 .63053 0 -1 Mshowa
.98137 .61178 m
.98137 .61803 L
s
[(32)] .98137 .63053 0 -1 Mshowa
0 .61803 m
1 .61803 L
s
.99375 .01472 m
1 .01472 L
s
.99375 .10512 m
1 .10512 L
s
.99375 .19552 m
1 .19552 L
s
.99375 .28592 m
1 .28592 L
s
.99375 .37632 m
1 .37632 L
s
.99375 .46672 m
1 .46672 L
s
.99375 .55712 m
1 .55712 L
s
.125 Mabswid
.99625 .0328 m
1 .0328 L
s
.99625 .05088 m
1 .05088 L
s
.99625 .06896 m
1 .06896 L
s
.99625 .08704 m
1 .08704 L
s
.99625 .1232 m
1 .1232 L
s
.99625 .14128 m
1 .14128 L
s
.99625 .15936 m
1 .15936 L
s
.99625 .17744 m
1 .17744 L
s
.99625 .2136 m
1 .2136 L
s
.99625 .23168 m
1 .23168 L
s
.99625 .24976 m
1 .24976 L
s
.99625 .26784 m
1 .26784 L
s
.99625 .304 m
1 .304 L
s
.99625 .32208 m
1 .32208 L
s
.99625 .34016 m
1 .34016 L
s
.99625 .35824 m
1 .35824 L
s
.99625 .3944 m
1 .3944 L
s
.99625 .41248 m
1 .41248 L
s
.99625 .43056 m
1 .43056 L
s
.99625 .44864 m
1 .44864 L
s
.99625 .4848 m
1 .4848 L
s
.99625 .50288 m
1 .50288 L
s
.99625 .52096 m
1 .52096 L
s
.99625 .53904 m
1 .53904 L
s
.99625 .5752 m
1 .5752 L
s
.99625 .59328 m
1 .59328 L
s
.99625 .61136 m
1 .61136 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .01472 m
1 .01472 L
s
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.00621 .01472 m
.00621 .0163 L
.01242 .0163 L
.01242 .01472 L
F
.5 Mabswid
.00621 .01472 m
.00621 .0163 L
.01242 .0163 L
.01242 .01472 L
.00621 .01472 L
s
.03727 .01472 m
.03727 .09563 L
.04348 .09563 L
.04348 .01472 L
F
.03727 .01472 m
.03727 .09563 L
.04348 .09563 L
.04348 .01472 L
.03727 .01472 L
s
.06832 .01472 m
.06832 .02043 L
.07453 .02043 L
.07453 .01472 L
F
.06832 .01472 m
.06832 .02043 L
.07453 .02043 L
.07453 .01472 L
.06832 .01472 L
s
.09938 .01472 m
.09938 .04771 L
.10559 .04771 L
.10559 .01472 L
F
.09938 .01472 m
.09938 .04771 L
.10559 .04771 L
.10559 .01472 L
.09938 .01472 L
s
.13043 .01472 m
.13043 .01884 L
.13665 .01884 L
.13665 .01472 L
F
.13043 .01472 m
.13043 .01884 L
.13665 .01884 L
.13665 .01472 L
.13043 .01472 L
s
.16149 .01472 m
.16149 .06326 L
.1677 .06326 L
.1677 .01472 L
F
.16149 .01472 m
.16149 .06326 L
.1677 .06326 L
.1677 .01472 L
.16149 .01472 L
s
.19255 .01472 m
.19255 .06263 L
.19876 .06263 L
.19876 .01472 L
F
.19255 .01472 m
.19255 .06263 L
.19876 .06263 L
.19876 .01472 L
.19255 .01472 L
s
.2236 .01472 m
.2236 .02392 L
.22981 .02392 L
.22981 .01472 L
F
.2236 .01472 m
.2236 .02392 L
.22981 .02392 L
.22981 .01472 L
.2236 .01472 L
s
.25466 .01472 m
.25466 .01472 L
.26087 .01472 L
.26087 .01472 L
F
.25466 .01472 m
.25466 .01472 L
.26087 .01472 L
.26087 .01472 L
.25466 .01472 L
s
.28571 .01472 m
.28571 .02963 L
.29193 .02963 L
.29193 .01472 L
F
.28571 .01472 m
.28571 .02963 L
.29193 .02963 L
.29193 .01472 L
.28571 .01472 L
s
.31677 .01472 m
.31677 .04073 L
.32298 .04073 L
.32298 .01472 L
F
.31677 .01472 m
.31677 .04073 L
.32298 .04073 L
.32298 .01472 L
.31677 .01472 L
s
.34783 .01472 m
.34783 .06421 L
.35404 .06421 L
.35404 .01472 L
F
.34783 .01472 m
.34783 .06421 L
.35404 .06421 L
.35404 .01472 L
.34783 .01472 L
s
.37888 .01472 m
.37888 .01567 L
.38509 .01567 L
.38509 .01472 L
F
.37888 .01472 m
.37888 .01567 L
.38509 .01567 L
.38509 .01472 L
.37888 .01472 L
s
.40994 .01472 m
.40994 .06263 L
.41615 .06263 L
.41615 .01472 L
F
.40994 .01472 m
.40994 .06263 L
.41615 .06263 L
.41615 .01472 L
.40994 .01472 L
s
.44099 .01472 m
.44099 .07405 L
.4472 .07405 L
.4472 .01472 L
F
.44099 .01472 m
.44099 .07405 L
.4472 .07405 L
.4472 .01472 L
.44099 .01472 L
s
.47205 .01472 m
.47205 .02328 L
.47826 .02328 L
.47826 .01472 L
F
.47205 .01472 m
.47205 .02328 L
.47826 .02328 L
.47826 .01472 L
.47205 .01472 L
s
.50311 .01472 m
.50311 .01598 L
.50932 .01598 L
.50932 .01472 L
F
.50311 .01472 m
.50311 .01598 L
.50932 .01598 L
.50932 .01472 L
.50311 .01472 L
s
.53416 .01472 m
.53416 .04708 L
.54037 .04708 L
.54037 .01472 L
F
.53416 .01472 m
.53416 .04708 L
.54037 .04708 L
.54037 .01472 L
.53416 .01472 L
s
.56522 .01472 m
.56522 .01789 L
.57143 .01789 L
.57143 .01472 L
F
.56522 .01472 m
.56522 .01789 L
.57143 .01789 L
.57143 .01472 L
.56522 .01472 L
s
.59627 .01472 m
.59627 .05438 L
.60248 .05438 L
.60248 .01472 L
F
.59627 .01472 m
.59627 .05438 L
.60248 .05438 L
.60248 .01472 L
.59627 .01472 L
s
.62733 .01472 m
.62733 .02772 L
.63354 .02772 L
.63354 .01472 L
F
.62733 .01472 m
.62733 .02772 L
.63354 .02772 L
.63354 .01472 L
.62733 .01472 L
s
.65839 .01472 m
.65839 .06834 L
.6646 .06834 L
.6646 .01472 L
F
.65839 .01472 m
.65839 .06834 L
.6646 .06834 L
.6646 .01472 L
.65839 .01472 L
s
.68944 .01472 m
.68944 .05216 L
.69565 .05216 L
.69565 .01472 L
F
.68944 .01472 m
.68944 .05216 L
.69565 .05216 L
.69565 .01472 L
.68944 .01472 L
s
.7205 .01472 m
.7205 .01598 L
.72671 .01598 L
.72671 .01472 L
F
.7205 .01472 m
.7205 .01598 L
.72671 .01598 L
.72671 .01472 L
.7205 .01472 L
s
.75155 .01472 m
.75155 .01535 L
.75776 .01535 L
.75776 .01472 L
F
.75155 .01472 m
.75155 .01535 L
.75776 .01535 L
.75776 .01472 L
.75155 .01472 L
s
.78261 .01472 m
.78261 .08421 L
.78882 .08421 L
.78882 .01472 L
F
.78261 .01472 m
.78261 .08421 L
.78882 .08421 L
.78882 .01472 L
.78261 .01472 L
s
.81366 .01472 m
.81366 .0639 L
.81988 .0639 L
.81988 .01472 L
F
.81366 .01472 m
.81366 .0639 L
.81988 .0639 L
.81988 .01472 L
.81366 .01472 L
s
.84472 .01472 m
.84472 .09721 L
.85093 .09721 L
.85093 .01472 L
F
.84472 .01472 m
.84472 .09721 L
.85093 .09721 L
.85093 .01472 L
.84472 .01472 L
s
.87578 .01472 m
.87578 .03597 L
.88199 .03597 L
.88199 .01472 L
F
.87578 .01472 m
.87578 .03597 L
.88199 .03597 L
.88199 .01472 L
.87578 .01472 L
s
.90683 .01472 m
.90683 .03502 L
.91304 .03502 L
.91304 .01472 L
F
.90683 .01472 m
.90683 .03502 L
.91304 .03502 L
.91304 .01472 L
.90683 .01472 L
s
.93789 .01472 m
.93789 .03153 L
.9441 .03153 L
.9441 .01472 L
F
.93789 .01472 m
.93789 .03153 L
.9441 .03153 L
.9441 .01472 L
.93789 .01472 L
s
.96894 .01472 m
.96894 .03851 L
.97516 .03851 L
.97516 .01472 L
F
.96894 .01472 m
.96894 .03851 L
.97516 .03851 L
.97516 .01472 L
.96894 .01472 L
s
0 1 0 r
.01242 .01472 m
.01242 .05565 L
.01863 .05565 L
.01863 .01472 L
F
.01242 .01472 m
.01242 .05565 L
.01863 .05565 L
.01863 .01472 L
.01242 .01472 L
s
.04348 .01472 m
.04348 .01503 L
.04969 .01503 L
.04969 .01472 L
F
.04348 .01472 m
.04348 .01503 L
.04969 .01503 L
.04969 .01472 L
.04348 .01472 L
s
.07453 .01472 m
.07453 .01503 L
.08075 .01503 L
.08075 .01472 L
F
.07453 .01472 m
.07453 .01503 L
.08075 .01503 L
.08075 .01472 L
.07453 .01472 L
s
.10559 .01472 m
.10559 .01472 L
.1118 .01472 L
.1118 .01472 L
F
.10559 .01472 m
.10559 .01472 L
.1118 .01472 L
.1118 .01472 L
.10559 .01472 L
s
.13665 .01472 m
.13665 .01472 L
.14286 .01472 L
.14286 .01472 L
F
.13665 .01472 m
.13665 .01472 L
.14286 .01472 L
.14286 .01472 L
.13665 .01472 L
s
.1677 .01472 m
.1677 .01472 L
.17391 .01472 L
.17391 .01472 L
F
.1677 .01472 m
.1677 .01472 L
.17391 .01472 L
.17391 .01472 L
.1677 .01472 L
s
.19876 .01472 m
.19876 .01472 L
.20497 .01472 L
.20497 .01472 L
F
.19876 .01472 m
.19876 .01472 L
.20497 .01472 L
.20497 .01472 L
.19876 .01472 L
s
.22981 .01472 m
.22981 .01472 L
.23602 .01472 L
.23602 .01472 L
F
.22981 .01472 m
.22981 .01472 L
.23602 .01472 L
.23602 .01472 L
.22981 .01472 L
s
.26087 .01472 m
.26087 .60332 L
.26708 .60332 L
.26708 .01472 L
F
.26087 .01472 m
.26087 .60332 L
.26708 .60332 L
.26708 .01472 L
.26087 .01472 L
s
.29193 .01472 m
.29193 .01472 L
.29814 .01472 L
.29814 .01472 L
F
.29193 .01472 m
.29193 .01472 L
.29814 .01472 L
.29814 .01472 L
.29193 .01472 L
s
.32298 .01472 m
.32298 .01472 L
.32919 .01472 L
.32919 .01472 L
F
.32298 .01472 m
.32298 .01472 L
.32919 .01472 L
.32919 .01472 L
.32298 .01472 L
s
.35404 .01472 m
.35404 .01503 L
.36025 .01503 L
.36025 .01472 L
F
.35404 .01472 m
.35404 .01503 L
.36025 .01503 L
.36025 .01472 L
.35404 .01472 L
s
.38509 .01472 m
.38509 .08294 L
.3913 .08294 L
.3913 .01472 L
F
.38509 .01472 m
.38509 .08294 L
.3913 .08294 L
.3913 .01472 L
.38509 .01472 L
s
.41615 .01472 m
.41615 .01472 L
.42236 .01472 L
.42236 .01472 L
F
.41615 .01472 m
.41615 .01472 L
.42236 .01472 L
.42236 .01472 L
.41615 .01472 L
s
.4472 .01472 m
.4472 .01503 L
.45342 .01503 L
.45342 .01472 L
F
.4472 .01472 m
.4472 .01503 L
.45342 .01503 L
.45342 .01472 L
.4472 .01472 L
s
.47826 .01472 m
.47826 .01598 L
.48447 .01598 L
.48447 .01472 L
F
.47826 .01472 m
.47826 .01598 L
.48447 .01598 L
.48447 .01472 L
.47826 .01472 L
s
.50932 .01472 m
.50932 .09372 L
.51553 .09372 L
.51553 .01472 L
F
.50932 .01472 m
.50932 .09372 L
.51553 .09372 L
.51553 .01472 L
.50932 .01472 L
s
.54037 .01472 m
.54037 .01503 L
.54658 .01503 L
.54658 .01472 L
F
.54037 .01472 m
.54037 .01503 L
.54658 .01503 L
.54658 .01472 L
.54037 .01472 L
s
.57143 .01472 m
.57143 .01472 L
.57764 .01472 L
.57764 .01472 L
F
.57143 .01472 m
.57143 .01472 L
.57764 .01472 L
.57764 .01472 L
.57143 .01472 L
s
.60248 .01472 m
.60248 .01472 L
.6087 .01472 L
.6087 .01472 L
F
.60248 .01472 m
.60248 .01472 L
.6087 .01472 L
.6087 .01472 L
.60248 .01472 L
s
.63354 .01472 m
.63354 .01662 L
.63975 .01662 L
.63975 .01472 L
F
.63354 .01472 m
.63354 .01662 L
.63975 .01662 L
.63975 .01472 L
.63354 .01472 L
s
.6646 .01472 m
.6646 .01472 L
.67081 .01472 L
.67081 .01472 L
F
.6646 .01472 m
.6646 .01472 L
.67081 .01472 L
.67081 .01472 L
.6646 .01472 L
s
.69565 .01472 m
.69565 .01472 L
.70186 .01472 L
.70186 .01472 L
F
.69565 .01472 m
.69565 .01472 L
.70186 .01472 L
.70186 .01472 L
.69565 .01472 L
s
.72671 .01472 m
.72671 .01503 L
.73292 .01503 L
.73292 .01472 L
F
.72671 .01472 m
.72671 .01503 L
.73292 .01503 L
.73292 .01472 L
.72671 .01472 L
s
.75776 .01472 m
.75776 .13148 L
.76398 .13148 L
.76398 .01472 L
F
.75776 .01472 m
.75776 .13148 L
.76398 .13148 L
.76398 .01472 L
.75776 .01472 L
s
.78882 .01472 m
.78882 .01503 L
.79503 .01503 L
.79503 .01472 L
F
.78882 .01472 m
.78882 .01503 L
.79503 .01503 L
.79503 .01472 L
.78882 .01472 L
s
.81988 .01472 m
.81988 .01503 L
.82609 .01503 L
.82609 .01472 L
F
.81988 .01472 m
.81988 .01503 L
.82609 .01503 L
.82609 .01472 L
.81988 .01472 L
s
.85093 .01472 m
.85093 .01472 L
.85714 .01472 L
.85714 .01472 L
F
.85093 .01472 m
.85093 .01472 L
.85714 .01472 L
.85714 .01472 L
.85093 .01472 L
s
.88199 .01472 m
.88199 .01884 L
.8882 .01884 L
.8882 .01472 L
F
.88199 .01472 m
.88199 .01884 L
.8882 .01884 L
.8882 .01472 L
.88199 .01472 L
s
.91304 .01472 m
.91304 .01503 L
.91925 .01503 L
.91925 .01472 L
F
.91304 .01472 m
.91304 .01503 L
.91925 .01503 L
.91925 .01472 L
.91304 .01472 L
s
.9441 .01472 m
.9441 .01472 L
.95031 .01472 L
.95031 .01472 L
F
.9441 .01472 m
.9441 .01472 L
.95031 .01472 L
.95031 .01472 L
.9441 .01472 L
s
.97516 .01472 m
.97516 .01503 L
.98137 .01503 L
.98137 .01472 L
F
.97516 .01472 m
.97516 .01503 L
.98137 .01503 L
.98137 .01472 L
.97516 .01472 L
s
0 0 1 r
.01863 .01472 m
.01863 .17337 L
.02484 .17337 L
.02484 .01472 L
F
.01863 .01472 m
.01863 .17337 L
.02484 .17337 L
.02484 .01472 L
.01863 .01472 L
s
.04969 .01472 m
.04969 .01503 L
.0559 .01503 L
.0559 .01472 L
F
.04969 .01472 m
.04969 .01503 L
.0559 .01503 L
.0559 .01472 L
.04969 .01472 L
s
.08075 .01472 m
.08075 .01535 L
.08696 .01535 L
.08696 .01472 L
F
.08075 .01472 m
.08075 .01535 L
.08696 .01535 L
.08696 .01472 L
.08075 .01472 L
s
.1118 .01472 m
.1118 .01472 L
.11801 .01472 L
.11801 .01472 L
F
.1118 .01472 m
.1118 .01472 L
.11801 .01472 L
.11801 .01472 L
.1118 .01472 L
s
.14286 .01472 m
.14286 .01472 L
.14907 .01472 L
.14907 .01472 L
F
.14286 .01472 m
.14286 .01472 L
.14907 .01472 L
.14907 .01472 L
.14286 .01472 L
s
.17391 .01472 m
.17391 .01503 L
.18012 .01503 L
.18012 .01472 L
F
.17391 .01472 m
.17391 .01503 L
.18012 .01503 L
.18012 .01472 L
.17391 .01472 L
s
.20497 .01472 m
.20497 .01472 L
.21118 .01472 L
.21118 .01472 L
F
.20497 .01472 m
.20497 .01472 L
.21118 .01472 L
.21118 .01472 L
.20497 .01472 L
s
.23602 .01472 m
.23602 .01472 L
.24224 .01472 L
.24224 .01472 L
F
.23602 .01472 m
.23602 .01472 L
.24224 .01472 L
.24224 .01472 L
.23602 .01472 L
s
.26708 .01472 m
.26708 .12546 L
.27329 .12546 L
.27329 .01472 L
F
.26708 .01472 m
.26708 .12546 L
.27329 .12546 L
.27329 .01472 L
.26708 .01472 L
s
.29814 .01472 m
.29814 .01472 L
.30435 .01472 L
.30435 .01472 L
F
.29814 .01472 m
.29814 .01472 L
.30435 .01472 L
.30435 .01472 L
.29814 .01472 L
s
.32919 .01472 m
.32919 .01535 L
.3354 .01535 L
.3354 .01472 L
F
.32919 .01472 m
.32919 .01535 L
.3354 .01535 L
.3354 .01472 L
.32919 .01472 L
s
.36025 .01472 m
.36025 .01567 L
.36646 .01567 L
.36646 .01472 L
F
.36025 .01472 m
.36025 .01567 L
.36646 .01567 L
.36646 .01472 L
.36025 .01472 L
s
.3913 .01472 m
.3913 .16861 L
.39752 .16861 L
.39752 .01472 L
F
.3913 .01472 m
.3913 .16861 L
.39752 .16861 L
.39752 .01472 L
.3913 .01472 L
s
.42236 .01472 m
.42236 .01503 L
.42857 .01503 L
.42857 .01472 L
F
.42236 .01472 m
.42236 .01503 L
.42857 .01503 L
.42857 .01472 L
.42236 .01472 L
s
.45342 .01472 m
.45342 .01662 L
.45963 .01662 L
.45963 .01472 L
F
.45342 .01472 m
.45342 .01662 L
.45963 .01662 L
.45963 .01472 L
.45342 .01472 L
s
.48447 .01472 m
.48447 .05184 L
.49068 .05184 L
.49068 .01472 L
F
.48447 .01472 m
.48447 .05184 L
.49068 .05184 L
.49068 .01472 L
.48447 .01472 L
s
.51553 .01472 m
.51553 .21652 L
.52174 .21652 L
.52174 .01472 L
F
.51553 .01472 m
.51553 .21652 L
.52174 .21652 L
.52174 .01472 L
.51553 .01472 L
s
.54658 .01472 m
.54658 .01472 L
.5528 .01472 L
.5528 .01472 L
F
.54658 .01472 m
.54658 .01472 L
.5528 .01472 L
.5528 .01472 L
.54658 .01472 L
s
.57764 .01472 m
.57764 .01535 L
.58385 .01535 L
.58385 .01472 L
F
.57764 .01472 m
.57764 .01535 L
.58385 .01535 L
.58385 .01472 L
.57764 .01472 L
s
.6087 .01472 m
.6087 .01567 L
.61491 .01567 L
.61491 .01472 L
F
.6087 .01472 m
.6087 .01567 L
.61491 .01567 L
.61491 .01472 L
.6087 .01472 L
s
.63975 .01472 m
.63975 .03756 L
.64596 .03756 L
.64596 .01472 L
F
.63975 .01472 m
.63975 .03756 L
.64596 .03756 L
.64596 .01472 L
.63975 .01472 L
s
.67081 .01472 m
.67081 .01472 L
.67702 .01472 L
.67702 .01472 L
F
.67081 .01472 m
.67081 .01472 L
.67702 .01472 L
.67702 .01472 L
.67081 .01472 L
s
.70186 .01472 m
.70186 .01535 L
.70807 .01535 L
.70807 .01472 L
F
.70186 .01472 m
.70186 .01535 L
.70807 .01535 L
.70807 .01472 L
.70186 .01472 L
s
.73292 .01472 m
.73292 .01472 L
.73913 .01472 L
.73913 .01472 L
F
.73292 .01472 m
.73292 .01472 L
.73913 .01472 L
.73913 .01472 L
.73292 .01472 L
s
.76398 .01472 m
.76398 .16417 L
.77019 .16417 L
.77019 .01472 L
F
.76398 .01472 m
.76398 .16417 L
.77019 .16417 L
.77019 .01472 L
.76398 .01472 L
s
.79503 .01472 m
.79503 .01503 L
.80124 .01503 L
.80124 .01472 L
F
.79503 .01472 m
.79503 .01503 L
.80124 .01503 L
.80124 .01472 L
.79503 .01472 L
s
.82609 .01472 m
.82609 .01694 L
.8323 .01694 L
.8323 .01472 L
F
.82609 .01472 m
.82609 .01694 L
.8323 .01694 L
.8323 .01472 L
.82609 .01472 L
s
.85714 .01472 m
.85714 .01567 L
.86335 .01567 L
.86335 .01472 L
F
.85714 .01472 m
.85714 .01567 L
.86335 .01567 L
.86335 .01472 L
.85714 .01472 L
s
.8882 .01472 m
.8882 .06548 L
.89441 .06548 L
.89441 .01472 L
F
.8882 .01472 m
.8882 .06548 L
.89441 .06548 L
.89441 .01472 L
.8882 .01472 L
s
.91925 .01472 m
.91925 .01852 L
.92547 .01852 L
.92547 .01472 L
F
.91925 .01472 m
.91925 .01852 L
.92547 .01852 L
.92547 .01472 L
.91925 .01472 L
s
.95031 .01472 m
.95031 .01567 L
.95652 .01567 L
.95652 .01472 L
F
.95031 .01472 m
.95031 .01567 L
.95652 .01567 L
.95652 .01472 L
.95031 .01472 L
s
.98137 .01472 m
.98137 .01789 L
.98758 .01789 L
.98758 .01472 L
F
.98137 .01472 m
.98137 .01789 L
.98758 .01789 L
.98758 .01472 L
.98137 .01472 L
s
1 .5 0 r
.02484 .01472 m
.02484 .02392 L
.03106 .02392 L
.03106 .01472 L
F
.02484 .01472 m
.02484 .02392 L
.03106 .02392 L
.03106 .01472 L
.02484 .01472 L
s
.0559 .01472 m
.0559 .02519 L
.06211 .02519 L
.06211 .01472 L
F
.0559 .01472 m
.0559 .02519 L
.06211 .02519 L
.06211 .01472 L
.0559 .01472 L
s
.08696 .01472 m
.08696 .0639 L
.09317 .0639 L
.09317 .01472 L
F
.08696 .01472 m
.08696 .0639 L
.09317 .0639 L
.09317 .01472 L
.08696 .01472 L
s
.11801 .01472 m
.11801 .0401 L
.12422 .0401 L
.12422 .01472 L
F
.11801 .01472 m
.11801 .0401 L
.12422 .0401 L
.12422 .01472 L
.11801 .01472 L
s
.14907 .01472 m
.14907 .05311 L
.15528 .05311 L
.15528 .01472 L
F
.14907 .01472 m
.14907 .05311 L
.15528 .05311 L
.15528 .01472 L
.14907 .01472 L
s
.18012 .01472 m
.18012 .03185 L
.18634 .03185 L
.18634 .01472 L
F
.18012 .01472 m
.18012 .03185 L
.18634 .03185 L
.18634 .01472 L
.18012 .01472 L
s
.21118 .01472 m
.21118 .03597 L
.21739 .03597 L
.21739 .01472 L
F
.21118 .01472 m
.21118 .03597 L
.21739 .03597 L
.21739 .01472 L
.21118 .01472 L
s
.24224 .01472 m
.24224 .0493 L
.24845 .0493 L
.24845 .01472 L
F
.24224 .01472 m
.24224 .0493 L
.24845 .0493 L
.24845 .01472 L
.24224 .01472 L
s
.27329 .01472 m
.27329 .01472 L
.2795 .01472 L
.2795 .01472 L
F
.27329 .01472 m
.27329 .01472 L
.2795 .01472 L
.2795 .01472 L
.27329 .01472 L
s
.30435 .01472 m
.30435 .05977 L
.31056 .05977 L
.31056 .01472 L
F
.30435 .01472 m
.30435 .05977 L
.31056 .05977 L
.31056 .01472 L
.30435 .01472 L
s
.3354 .01472 m
.3354 .0493 L
.34161 .0493 L
.34161 .01472 L
F
.3354 .01472 m
.3354 .0493 L
.34161 .0493 L
.34161 .01472 L
.3354 .01472 L
s
.36646 .01472 m
.36646 .04962 L
.37267 .04962 L
.37267 .01472 L
F
.36646 .01472 m
.36646 .04962 L
.37267 .04962 L
.37267 .01472 L
.36646 .01472 L
s
.39752 .01472 m
.39752 .02011 L
.40373 .02011 L
.40373 .01472 L
F
.39752 .01472 m
.39752 .02011 L
.40373 .02011 L
.40373 .01472 L
.39752 .01472 L
s
.42857 .01472 m
.42857 .04073 L
.43478 .04073 L
.43478 .01472 L
F
.42857 .01472 m
.42857 .04073 L
.43478 .04073 L
.43478 .01472 L
.42857 .01472 L
s
.45963 .01472 m
.45963 .03883 L
.46584 .03883 L
.46584 .01472 L
F
.45963 .01472 m
.45963 .03883 L
.46584 .03883 L
.46584 .01472 L
.45963 .01472 L
s
.49068 .01472 m
.49068 .05692 L
.49689 .05692 L
.49689 .01472 L
F
.49068 .01472 m
.49068 .05692 L
.49689 .05692 L
.49689 .01472 L
.49068 .01472 L
s
.52174 .01472 m
.52174 .03026 L
.52795 .03026 L
.52795 .01472 L
F
.52174 .01472 m
.52174 .03026 L
.52795 .03026 L
.52795 .01472 L
.52174 .01472 L
s
.5528 .01472 m
.5528 .04581 L
.55901 .04581 L
.55901 .01472 L
F
.5528 .01472 m
.5528 .04581 L
.55901 .04581 L
.55901 .01472 L
.5528 .01472 L
s
.58385 .01472 m
.58385 .04391 L
.59006 .04391 L
.59006 .01472 L
F
.58385 .01472 m
.58385 .04391 L
.59006 .04391 L
.59006 .01472 L
.58385 .01472 L
s
.61491 .01472 m
.61491 .04073 L
.62112 .04073 L
.62112 .01472 L
F
.61491 .01472 m
.61491 .04073 L
.62112 .04073 L
.62112 .01472 L
.61491 .01472 L
s
.64596 .01472 m
.64596 .06453 L
.65217 .06453 L
.65217 .01472 L
F
.64596 .01472 m
.64596 .06453 L
.65217 .06453 L
.65217 .01472 L
.64596 .01472 L
s
.67702 .01472 m
.67702 .02233 L
.68323 .02233 L
.68323 .01472 L
F
.67702 .01472 m
.67702 .02233 L
.68323 .02233 L
.68323 .01472 L
.67702 .01472 L
s
.70807 .01472 m
.70807 .04359 L
.71429 .04359 L
.71429 .01472 L
F
.70807 .01472 m
.70807 .04359 L
.71429 .04359 L
.71429 .01472 L
.70807 .01472 L
s
.73913 .01472 m
.73913 .06295 L
.74534 .06295 L
.74534 .01472 L
F
.73913 .01472 m
.73913 .06295 L
.74534 .06295 L
.74534 .01472 L
.73913 .01472 L
s
.77019 .01472 m
.77019 .01598 L
.7764 .01598 L
.7764 .01472 L
F
.77019 .01472 m
.77019 .01598 L
.7764 .01598 L
.7764 .01472 L
.77019 .01472 L
s
.80124 .01472 m
.80124 .03375 L
.80745 .03375 L
.80745 .01472 L
F
.80124 .01472 m
.80124 .03375 L
.80745 .03375 L
.80745 .01472 L
.80124 .01472 L
s
.8323 .01472 m
.8323 .0493 L
.83851 .0493 L
.83851 .01472 L
F
.8323 .01472 m
.8323 .0493 L
.83851 .0493 L
.83851 .01472 L
.8323 .01472 L
s
.86335 .01472 m
.86335 .03026 L
.86957 .03026 L
.86957 .01472 L
F
.86335 .01472 m
.86335 .03026 L
.86957 .03026 L
.86957 .01472 L
.86335 .01472 L
s
.89441 .01472 m
.89441 .05311 L
.90062 .05311 L
.90062 .01472 L
F
.89441 .01472 m
.89441 .05311 L
.90062 .05311 L
.90062 .01472 L
.89441 .01472 L
s
.92547 .01472 m
.92547 .05882 L
.93168 .05882 L
.93168 .01472 L
F
.92547 .01472 m
.92547 .05882 L
.93168 .05882 L
.93168 .01472 L
.92547 .01472 L
s
.95652 .01472 m
.95652 .06453 L
.96273 .06453 L
.96273 .01472 L
F
.95652 .01472 m
.95652 .06453 L
.96273 .06453 L
.96273 .01472 L
.95652 .01472 L
s
.98758 .01472 m
.98758 .06168 L
.99379 .06168 L
.99379 .01472 L
F
.98758 .01472 m
.98758 .06168 L
.99379 .06168 L
.99379 .01472 L
.98758 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{500, 309},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007d0001=B000`400?l00000o`00003ooooooooe
ooooo`00:_ooool3o`0000[ooooo1Ol0000:ooooo`?o00003?ooool3o`0000[ooooo0ol0000;oooo
o`?o00003?ooool00ol0003oooooooooo`0:ooooo`?o00002oooool2o`0000Wooooo0ol00004oooo
o`;o00001Oooool3o`0000?ooooo0ol00005ooooo`?o00000_ooool5o`0000Cooooo0ol00003oooo
o`?o00001Oooool3o`0000Cooooo0ol00004ooooo`?o00000oooool3o`0000Gooooo0ol00003oooo
o`?o00001Oooool3o`0000Cooooo00?o0000ooooooooool01?ooool3o`0000?ooooo0ol00005oooo
o`?o00000oooool2o`0000Gooooo1Ol00003ooooo`;o00001?ooool5o`0000;ooooo0ol00004oooo
o`Go000000?oooooo`000?l000000ol00003ooooo`Go00000_ooool3o`0000Cooooo1Ol00003oooo
o`?o00000oooool5o`0000;ooooo0ol00004ooooo`Go00000_ooool3o`0000Cooooo1Ol00003oooo
o`03o`000?oooooooooo00?ooooo1Ol00002ooooo`?o00001?ooool5o`0000;ooooo0_l00006oooo
o`?o00001?ooool2o`0000Gooooo0ol00003ooooo`?o00001Oooool3o`0000;ooooo1Ol0000Aoooo
o`00:oooool00ol0003oooooooooo`0:ooooo`04o`000?ooooooooooo`0000Wooooo00Go0000oooo
ooooooooooooo`00000<ooooo`03o`000?oooooooooo00Sooooo00Go0000ooooooooooooooooo`00
0009ooooo`05o`000?ooooooooooooooool000002oooool00ol0003oooooooooo`09ooooo`05o`00
0?ooooooooooooooool000003?ooool00ol0003oooooooooo`07ooooo`03o`000?oooooooooo00;o
oooo00Co0000oooooooooooo00001Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo
00Gooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`05o`000?oooooo
ooooooooool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool01?ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool0
0oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`03oooo
o`04o`000?ooooooooooo`0000;ooooo00Co0000oooooooooooo00001?ooool01?l0003ooooooooo
ool00003ooooo`03o`000?oooooooooo00Cooooo00Co0000oooooooooooo00000_ooool01?l0003o
ooooooooool00004ooooo`06o`000?ooooooooooo`000?oooooo00000oooool00ol0003ooooooooo
o`02ooooo`04o`000?ooooooooooo`0000Cooooo00?o0000ooooooooool00oooool01_l0003ooooo
ooooool0003oooooo`0000?ooooo00?o0000ooooooooool00_ooool01_l0003oooooooooool0003o
ooooo`0000?ooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00003ooooo`03o`00
0?oooooooooo00Cooooo00Ko0000oooooooooooo0000ooooool00003ooooo`03o`000?oooooooooo
00;ooooo00Co0000oooooooooooo00001?ooool00ol0003oooooooooo`02ooooo`05o`000?oooooo
ooooooooool000000_ooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool0
00000oooool00ol0003oooooooooo`03ooooo`05o`000?ooooooooooooooool000000_ooool01?l0
003oooooooooool0000Aooooo`00:oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo
00gooooo00?o0000ooooooooool01oooool5o`0000gooooo00?o0000ooooooooool01oooool01Ol0
003ooooooooooooooooo000000cooooo00?o0000ooooooooool02?ooool01Ol0003ooooooooooooo
oooo000000gooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`02ooooo`04o`000?oo
ooooooooo`0000Gooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`05ooooo`03o`00
0?oooooooooo00?ooooo00Co0000oooooooooooooooo1Ol00005ooooo`03o`000?oooooooooo00Go
oooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00Go0000oooo
ooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool0
0ol0003oooooooooo`03ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00001Ooo
ool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00ooo
ool01?l0003oooooooooooooool5o`0000Gooooo00?o0000ooooooooool01Oooool00ol0003ooooo
ooooo`03ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooooooooooo0000
00?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo
00Gooooo00Co0000oooooooooooo00000_ooool00ol0003oooooooooo`05ooooo`05o`000?oooooo
ooooooooool000002Oooool01Ol0003ooooooooooooooooo000001?ooooo000[ooooo`03o`000?oo
oooooooo00cooooo00?o0000ooooooooool02_ooool2o`0000[ooooo00Co0000oooooooooooo0000
3_ooool00ol0003oooooooooo`07ooooo`Co00003Oooool00ol0003oooooooooo`09ooooo`?o0000
2oooool4o`0000Sooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00005ooooo`03
o`000?oooooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`04ooooo`03
o`000?oooooooooo00Cooooo00?o0000ooooooooool00oooool2o`0000Kooooo00Go0000oooooooo
ooooooooo`000002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01Oooool00ol0
003oooooooooo`03ooooo`04o`000?ooooooooooooooo`Co00001_ooool00ol0003oooooooooo`04
ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_ooool3o`0000Kooooo00?o0000
ooooooooool00_ooool4o`0000Kooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool00_ooool00ol0003oooooooooo`06ooooo`03o`000?oo
oooooooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`02ooooo`;o00001ooo
ool01?l0003oooooooooool00002ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool0
1?ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Co00001oooool00ol0003ooooo
ooooo`03ooooo`03o`000?oooooooooo00Gooooo00Co0000oooooooooooooooo0ol00007ooooo`04
o`000?ooooooooooooooo`Co00001Oooool2o`0000?ooooo00Co0000oooooooooooo00001Oooool2
o`0000Cooooo00?o0000ooooooooool01Oooool2o`0000Gooooo00?o0000ooooooooool04?ooool0
02_ooooo00?o0000ooooooooool02Oooool01Ol0003ooooooooooooooooo000000gooooo00?o0000
ooooooooool02?ooool00ol0003oooooo`00000:ooooo`Co00002_ooool00ol0003oooooooooo`0?
ooooo`03o`000?oooooooooo00Oooooo00Go0000ooooooooooooooooo`000009ooooo`05o`000?oo
ooooooooooooool000002?ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Go
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`05o`000?oooooooooooooo
ool000000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000oooo
ooooool00oooool00ol0003oooooooooo`02ooooo`03o`000?oooooo000000Kooooo00Co0000oooo
oooooooooooo1?l00006ooooo`05o`000?ooooooooooooooool000002Oooool00ol0003ooooooooo
o`05ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`00
0?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?oooooooooooooo
ool000000oooool01?l0003oooooooooool00002ooooo`05o`000?ooooooooooooooool000000ooo
ool01Ol0003ooooooooooooooooo000000Gooooo00Oo0000ooooooooooooooooo`000?oooooo0000
00?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00Go0000
ooooooooooooooooo`000003ooooo`06o`000?ooooooooooo`000?oooooo00001?ooool01_l0003o
oooooooooooooooo0000ooooo`Co00001?ooool01ol0003ooooooooooooooooo0000ooooool00000
1oooool01Ol0003ooooooooooooooooo000000Gooooo00Go0000ooooooooooooooooo`000003oooo
o`03o`000?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo
000000?ooooo00?o0000ooooooooool01Oooool01?l0003oooooooooool00002ooooo`03o`000?oo
oooooooo00Gooooo00Go0000ooooooooooooooooo`000009ooooo`03o`000?oooooo000000?ooooo
00?o0000ooooooooool03oooool002[ooooo0_l0000;ooooo`05o`000?ooooooooooooooool00000
2Oooool01Ol0003ooooooooooooooooo000000_ooooo0_l0000:ooooo`03o`000?oooooooooo00co
oooo00?o0000ooooooooool02_ooool01Ol0003ooooooooooooooooo000000Wooooo00Go0000oooo
ooooooooooooo`000009ooooo`05o`000?ooooooooooooooool000001oooool2o`0000Cooooo00Co
0000oooooooooooo00001?ooool2o`0000Cooooo0_l00006ooooo`;o00000oooool01Ol0003ooooo
oooooooooooo000000Cooooo0_l00003ooooo`05o`000?ooooooooooooooool000001?ooool2o`00
00Gooooo0_l00005ooooo`;o00000oooool00ol0003oooooooooo`06ooooo`;o00001?ooool00ol0
003oooooooooo`05ooooo`;o00000oooool01Ol0003ooooooooooooooooo000000Cooooo0_l00003
ooooo`05o`000?ooooooooooooooool000001?ooool2o`0000?ooooo00Go0000oooooooooooooooo
o`000003ooooo`05o`000?ooooooooooooooool000000_ooool01?l0003oooooooooool00003oooo
o`05o`000?ooooooooooooooool000000_ooool2o`0000Gooooo00Oo0000ooooooooooooooooo`00
0?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo000000?o
oooo00Go0000ooooooooooooooooo`000003ooooo`04o`000?ooooooooooooooo`;o00001?ooool0
1ol0003ooooooooooooooooo0000ooooool000001oooool01Ol0003ooooooooooooooooo000000;o
oooo00?o0000ooooooooool01?ooool01ol0003ooooooooooooooooo0000ooooool000000oooool0
1Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooool000000oooool01Ol0003ooooooooo
oooooooo000000?ooooo00?o0000ooooool000000oooool01Ol0003ooooooooooooooooo000000?o
oooo00Co0000oooooooooooo00000_ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000
ooooooooool00_l00005ooooo`07o`000?ooooooooooooooool0003oooooo`000003ooooo`03o`00
0?oooooooooo00oooooo000[ooooo`03o`000?oooooooooo00[ooooo0ol0000;ooooo`?o00003Ooo
ool00ol0003oooooooooo`08ooooo`Go00002oooool2o`0000[ooooo1Ol0000:ooooo`?o00002ooo
ool3o`0000Wooooo00?o0000ooooooooool00oooool2o`0000Kooooo00?o0000ooooooooool00ooo
ool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00;ooooo0ol00006ooooo`03o`000?oo
oooooooo00;ooooo0ol00006ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?oo
ool01?l0003oooooooooooooool5o`0000Gooooo00?o0000ooooooooool00oooool2o`0000Kooooo
00Co0000oooooooooooooooo1Ol00005ooooo`03o`000?oooooooooo00;ooooo0ol00006ooooo`03
o`000?oooooooooo00;ooooo0ol00005ooooo`?o00001?ooool2o`0000Gooooo0ol00004ooooo`03
o`000?oooooooooo00Cooooo0ol00003ooooo`?o00001Oooool3o`0000?ooooo0ol00005ooooo`?o
00001Oooool00ol0003oooooooooo`03ooooo`?o00000_ooool5o`0000Cooooo0ol00004ooooo`;o
00001Oooool3o`0000;ooooo1Ol00004ooooo`?o00000oooool3o`0000Gooooo0ol00003ooooo`?o
00001Oooool3o`0000Cooooo0_l00005ooooo`?o00001?ooool00ol0003oooooooooo`04ooooo`?o
00000oooool3o`0001;ooooo003oooooooGooooo003oooooooGooooo003oooooooGooooo003ooooo
ooGooooo003oooooooGooooo003oooooooGooooo003oooooooGooooo000Soooooooo0000aOl0000=
ooooo`008oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooo
ooooool01Oooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`06ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;ooooo`03
o`000?oooooooooo00_ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`0;ooooo`00
8oooool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
2oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`0;ooooo`006_ooool2o`0000Oooooo00?o0000ooooooooool0oooo
ooo1ooooo`03o`000?oooooooooo00_ooooo000Iooooo`04o`000?ooooooooooo`0000Kooooo00?o
0000ooooooooool0ooooooo1ooooo`03o`000?oooooooooo00_ooooo000Iooooo`04o`000?oooooo
ooooo`0000Kooooo00?o0000ooooooooool0ooooooo1ooooo`03o`000?oooooooooo00_ooooo000I
ooooo`04o`000?ooooooooooo`0000Kooooo0ol00003ool000?o0?l00_l00?l4oomo00;o00000ooo
0003o`3o00;o003o1?ooO`02o`0000?oo`000ol0o`02o`00o`Coogl00_l00003ool000?o0?l00_l0
0?l4oomo00;o00000ooo0003o`3o00;o003o1?ooO`02o`0000?oo`000ol0o`02o`00o`Coogl00_l0
0003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0
o`03o`00o`Coogl000Co0000ool00?oo003oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l0
0ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl0
0_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`00
0ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o
0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o0000
0ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l0
0ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl0
0_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o0oooO`02o`0000?oo`00
0ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00;o00000ooo0003o`3o00?o003o
0oooO`02o`0000?oo`000ol0o`03o`00o`?oogl00_l00003ool000?o0?l00ol00?l3oomo00?o0000
3Oooool001Wooooo00Co0000oooooooooooo00001_ooool00ol0003oooooooooo`03ooooo`?o0?l0
0_l00?l4oomo00;ooooo1?oo0004ooooo`Coogl00_ooool4ool000Cooooo1?ooO`02ooooo`Coo`00
1?ooool4oomo00;ooooo1?oo0004ooooo`Coogl00_ooool4ool000Cooooo1?ooO`02ooooo`Coo`00
1Oooool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool000Gooooo
0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00?o003o
0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0002ooooo`?o003o0oooO`02ooooo`Coo`00
0_ooool3o`00o`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo
1?oo0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`?oo`000ol0o`03o`00o`?oogl0
0_ooool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00_ooooo0oooO`05ooooo`?o0?l0
1?l00?l4ooooo`Coo`001Oooool3oomo00;ooooo1?oo0002ooooo`?o003o0oooO`02ooooo`Coo`00
1Oooool3oomo00;ooooo0ooo0003o`3o00?o003o0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl0
0_ooool4ool000Gooooo0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl00_ooool00ol0003ooooo
ooooo`0;ooooo`006Oooool01?l0003oooooooooool00006ooooo`03o`000?oooooooooo00?ooooo
0ol0o`02o`00o`Coogl00_ooool4ool000Cooooo1?ooO`02ooooo`Coo`001?ooool4oomo00;ooooo
1?oo0004ooooo`Coogl00_ooool4ool000Cooooo1?ooO`02ooooo`Coo`001?ooool4oomo00;ooooo
1?oo0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`00
1Oooool3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o0?l0
0ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`Coo`00
0_ooool3o`00o`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00_ooooo
0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0002ooooo`?o003o0oooO`02ooooo`Coo`00
1Oooool3oomo00;ooooo1?oo0005ooooo`?oogl02oooool3oomo00Gooooo0ol0o`04o`00o`Cooooo
1?oo0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo
0ooo0003o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl0
0_ooool4ool000Gooooo0oooO`02ooooo`03o`000?oooooooooo00_ooooo000Jooooo`;o00001ooo
ool00ol0003oooooooooo`03ooooo`?o0?l00_l00?l4oomo00;ooooo1?oo0004ooooo`Coogl02_oo
ool4oomo00;ooooo1?oo0004ooooo`Coogl02_ooool4oomo00;ooooo1?oo0004ooooo`Coogl00_oo
ool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo
0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0
o`04o`00o`Cooooo1?oo0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`Coo`000_oo
ool3o`00o`?oogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00_ooooo0ooo
O`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0002ooooo`?o003o0oooO`02ooooo`Coo`001Ooo
ool3oomo00;ooooo1?oo0005ooooo`?oogl02oooool3oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo
0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo
0002ooooo`?o003o0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl00_oo
ool4ool000Gooooo0oooO`02ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooo
oooo00?ooooo0ol0o`02o`00o`Coogl00_ooool4ool000Cooooo1?ooO`0:ooooo`Coogl00_ooool4
ool000Cooooo1?ooO`0:ooooo`Coogl00_ooool4ool000Cooooo1?ooO`02ooooo`Coo`001Oooool3
oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`02
ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4
ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0002ooooo`?o003o0oooO`05
ooooo`?o0?l00ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl02oooool3oomo00;ooooo1?oo0005
ooooo`?oogl00_ooool4ool000;ooooo0ol00?l3oomo00;ooooo1?oo000:ooooo`Coo`001Oooool3
oomo00_ooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005
ooooo`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl00_ooool4
ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool0
0ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`03ooooo`?o0?l00ol00?l5oooo
o`Coo`004_ooool4oomo00;ooooo1?oo0004ooooo`Coogl02_ooool4oomo00;ooooo1?oo0004oooo
o`Coogl00_ooool4ool000Gooooo0oooO`0;ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool0
00Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o
00Co003o1?ooool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00Sooooo0ol00?l3oomo
00Gooooo0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`0;ooooo`?oogl00_ooool4ool0
00Gooooo0oooO`02ooooo`Coo`000_ooool3o`00o`?oogl00_ooool4ool000[ooooo1?oo0005oooo
o`?oogl02oooool3oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo0005ooooo`?oogl00_ooool4ool0
00Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0002ooooo`?o003o0oooO`02oooo
o`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`02oooo
o`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo00?ooooo0ol0o`03o`00o`Go
oooo1?oo000Booooo`Coogl00_ooool4ool000Cooooo1?ooO`0:ooooo`Coogl00_ooool4ool000Co
oooo1?ooO`02ooooo`Coo`001Oooool3oomo00_ooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Co
o`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`05ooooo`?o
0?l01?l00?l4ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl02?ooool3o`00o`?o
ogl01Oooool3o`3o00?o003o0oooO`02ooooo`Coo`001Oooool3oomo00_ooooo0oooO`02ooooo`Co
o`001Oooool3oomo00;ooooo1?oo0002ooooo`?o003o0oooO`02ooooo`Coo`002_ooool4ool000Go
oooo0oooO`0;ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`02ooooo`Co
o`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool4ool000;ooooo0ol00?l3oomo00;o
oooo1?oo0005ooooo`?oogl00_ooool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00;o
oooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool00oooool3o`3o00?o003o
1Oooool4ool001;ooooo1?ooO`02ooooo`Coo`001?ooool4oomo00[ooooo1?ooO`02ooooo`Coo`00
1?ooool4oomo00;ooooo1?oo0005ooooo`?oogl02oooool3oomo00Gooooo0ol0o`04o`00o`gooooo
0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o
1?ooool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00Sooooo0ol00?l3oomo00Gooooo
0ol0o`03o`00o`?oogl00_ooool4ool000Gooooo0oooO`0;ooooo`?oogl00_ooool4ool000Gooooo
0oooO`08ooooo`?o003o0oooO`02ooooo`Coo`002_ooool4ool000Gooooo0oooO`0;ooooo`?oogl0
1Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo
1?oo0005ooooo`?oogl00_ooool4ool000;ooooo0ol00?l3oomo00;ooooo1?oo0005ooooo`?oogl0
0_ooool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo00?o0000ooooooooool0
2oooool002?ooooo0ol00003ooooo`?o0?l00ol00?l5ooooo`Coo`004_ooool4oomo00;ooooo1?oo
0004ooooo`Coogl02_ooool4oomo00;ooooo1?oo0004ooooo`Coogl00_ooool4ool000Gooooo0ooo
O`0;ooooo`?oogl01Oooool3o`3o00Co003o3Oooool3oomo00;ooooo1?oo0005ooooo`?oogl00_oo
ool4ool000Gooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00;ooooo1?oo
0005ooooo`?oogl02?ooool3o`00o`?oogl01Oooool3o`3o00Co003o1?ooool4ool000Gooooo0ooo
O`0;ooooo`?oogl00_ooool4ool000Gooooo0oooO`08ooooo`?o003o0oooO`02ooooo`Coo`002_oo
ool4ool000Gooooo0oooO`0;ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool000Gooooo0ooo
O`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo000:ooooo`Coo`000_ooool3o`00o`?oogl00_oo
ool4ool000Gooooo0oooO`0;ooooo`?oogl00_ooool4ool000Gooooo0oooO`000ooooooo0000o`00
000=ooooo`008oooool00ol0003oooooooooo`03ooooo`?o0?l00ol00?l5ooooo`Coo`004_ooool4
oomo00;ooooo1?oo0004ooooo`Coogl02_ooool4oomo00;ooooo1?oo000:ooooo`Coo`001Oooool3
oomo00_ooooo0oooO`05ooooo`?o0?l01?l00?l=ooooo`?oogl00_ooool4ool000Gooooo0oooO`02
ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo0005ooooo`?oogl00_ooool4
ool000Gooooo0oooO`08ooooo`?o003o0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3
oomo00_ooooo0oooO`02ooooo`Coo`001Oooool3oomo00Sooooo0ol00?l3oomo00;ooooo1?oo000:
ooooo`Coo`001Oooool3oomo00_ooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4
ool000Gooooo0oooO`02ooooo`Coo`002_ooool4ool000;ooooo0ol00?l3oomo00;ooooo1?oo0005
ooooo`?oogl02oooool3oomo00;ooooo1?oo0005ooooo`?oogl00_ooool00ol0003oooooooooo`0;
ooooo`008oooool00ol0003oooooooooo`03ooooo`?o0?l00ol00?l5ooooo`Coo`004_ooool4oomo
00;ooooo1?oo0004ooooo`Coogl02_ooool4oomo00;ooooo1?oo000:ooooo`Coo`004oooool3oomo
00Gooooo0ol0o`04o`00o`gooooo0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005oooo
o`?oogl01Oooool3o`3o00Co003o1?ooool4ool000Gooooo0oooO`02ooooo`Coo`001Oooool3oomo
00Sooooo0ol00?l3oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo0005ooooo`?oogl02oooool3oomo
00;ooooo1?oo0005ooooo`?oogl02?ooool3o`00o`?oogl00_ooool4ool000[ooooo1?oo0005oooo
o`?oogl02oooool3oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`001Oooool3oomo
00;ooooo1?oo000@ooooo`?o003o0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo1?oo0005oooo
o`?oogl00_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`03ooooo`?o
0?l00ol00?l5ooooo`Coo`004_ooool4oomo00;ooooo1?oo0004ooooo`Coogl02_ooool4oomo00;o
oooo1?oo000:ooooo`Coo`004oooool3oomo00Gooooo0ol0o`04o`00o`gooooo0oooO`02ooooo`Co
o`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool000Go
oooo0oooO`02ooooo`Coo`001Oooool3oomo00Sooooo0ol00?l3oomo00Gooooo0ol0o`04o`00o`Co
oooo1?oo0005ooooo`?oogl02oooool3oomo00;ooooo1?oo0005ooooo`?oogl02oooool3oomo00;o
oooo1?oo000:ooooo`Coo`001Oooool3oomo00_ooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Co
o`002_ooool4ool000Gooooo0oooO`02ooooo`Coo`004?ooool3o`00o`?oogl02oooool3oomo00_o
oooo0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo00?o0000ooooooooool02oooool002?ooooo
00?o0000ooooooooool00oooool3o`3o00?o003o1Oooool4ool001;ooooo1?ooO`02ooooo`Coo`00
4_ooool4oomo00;ooooo1?oo000:ooooo`Coo`004oooool3oomo00Gooooo0ol0o`04o`00o`gooooo
0oooO`02ooooo`Coo`001Oooool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o
1?ooool4ool000Gooooo0oooO`02ooooo`Coo`004?ooool3o`00o`?oogl01Oooool3o`3o00Co003o
1?ooool4ool000Gooooo0oooO`0;ooooo`?oogl00_ooool4ool000Gooooo0oooO`0;ooooo`?oogl0
0_ooool4ool000[ooooo1?oo0005ooooo`?oogl02oooool3oomo00Gooooo0ol0o`04o`00o`Cooooo
1?oo000:ooooo`Coo`001Oooool3oomo00;ooooo1?oo000@ooooo`?o003o0oooO`0;ooooo`?oogl0
2oooool3oomo00_ooooo0oooO`02ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oo
oooooooo00?ooooo0ol0o`03o`00o`Gooooo1?oo000Booooo`Coogl00_ooool4ool001;ooooo1?oo
O`02ooooo`Coo`002_ooool4ool001?ooooo0oooO`05ooooo`?o0?l01?l00?l=ooooo`?oogl02ooo
ool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo
000@ooooo`?o003o0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`001Oooool3oomo00_ooooo0ooo
O`02ooooo`Coo`004oooool3oomo00;ooooo1?oo000:ooooo`Coo`001Oooool3oomo00_ooooo0ooo
O`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool000Gooooo0oooO`02ooooo`Coo`004?oo
ool3o`00o`?oogl02oooool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool00ol0003ooooooooo
o`0;ooooo`008oooool00ol0003oooooooooo`03ooooo`?o0?l00ol00?l5ooooo`Coo`004_ooool4
oomo00;ooooo1?oo000Booooo`Coogl00_ooool4ool000[ooooo1?oo000Cooooo`?oogl01Oooool3
o`3o00Co003o3Oooool3oomo00_ooooo0oooO`02ooooo`Coo`001Oooool3oomo00Gooooo0ol0o`04
o`00o`Cooooo1?oo000:ooooo`Coo`004?ooool3o`00o`?oogl01Oooool3o`3o00Co003o1?ooool4
ool000Gooooo0oooO`0@ooooo`Coo`004oooool3oomo00;ooooo1?oo000:ooooo`Coo`004oooool3
oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`001Oooool3oomo00;ooooo1?oo000@
ooooo`?o003o0oooO`0;ooooo`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`03o`000?oooooo
oooo00_ooooo000Sooooo`03o`000?oooooooooo00?ooooo0ol0o`03o`00o`Gooooo1?oo000Boooo
o`Coogl00_ooool4ool001;ooooo1?ooO`02ooooo`Coo`002_ooool4ool001?ooooo0oooO`05oooo
o`?o0?l01?l00?l=ooooo`?oogl02oooool3oomo00;ooooo1?oo0005ooooo`?oogl01Oooool3o`3o
00Co003o1?ooool4ool000[ooooo1?oo000@ooooo`?o003o0oooO`05ooooo`?o0?l01?l00?lPoooo
o`Coo`004oooool3oomo00;ooooo1?oo000:ooooo`Coo`004oooool3oomo00Gooooo0ol0o`04o`00
o`Cooooo1?oo000:ooooo`Coo`001Oooool3oomo00;ooooo1?oo000@ooooo`?o003o0oooO`0;oooo
o`?oogl02oooool3oomo00_ooooo0oooO`02ooooo`03o`000?oooooooooo00_ooooo000Sooooo`?o
00000oooool3o`3o00?o003o1Oooool4ool001;ooooo1?ooO`0Hooooo`Coogl00_ooool4ool000[o
oooo1?oo000Kooooo`?o0?l01?l00?l=ooooo`?oogl04?ooool4ool000Gooooo0oooO`05ooooo`?o
0?l01?l00?l4ooooo`Coo`002_ooool4ool0013ooooo0ol00?l3oomo00Gooooo0ol0o`04o`00ob3o
oooo1?oo000Cooooo`?oogl00_ooool4ool000[ooooo1?oo000Cooooo`?oogl01Oooool3o`3o00Co
003o1?ooool4ool000[ooooo1?oo000:ooooo`Coo`004?ooool3o`00o`?oogl02oooool3oomo00_o
oooo0oooO`0;ooooo`?oogl000?oooooo`000?l000003Oooool002?ooooo00?o0000ooooooooool0
0oooool3o`3o00?o003o1Oooool4ool001;ooooo1?ooO`0Hooooo`Coogl00_ooool4ool000[ooooo
1?oo000Kooooo`?o0?l01?l00?l=ooooo`?oogl04?ooool4ool000gooooo0ol0o`04o`00o`Cooooo
1?oo000:ooooo`Coo`004?ooool3o`00o`?oogl01Oooool3o`3o00Co003o8?ooool4ool001?ooooo
0oooO`02ooooo`Coo`002_ooool4ool001?ooooo0oooO`05ooooo`?o0?l01?l00?l4ooooo`Coo`00
2_ooool4ool000[ooooo1?oo000@ooooo`?o003o0oooO`0;ooooo`?oogl02oooool3oomo00_ooooo
0oooO`02ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo00?ooooo0ol0
o`03o`00o`Gooooo1?oo000Booooo`Coogl07_ooool4ool000[ooooo1?oo000Kooooo`?o0?l01?l0
0?l=ooooo`?oogl04?ooool4ool000gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`004ooo
ool3oomo00Gooooo0ol0o`04o`00ob3ooooo1?oo000Cooooo`?oogl00_ooool4ool0027ooooo0ooo
O`05ooooo`?o0?l01?l00?l4ooooo`Coo`002_ooool4ool000[ooooo1?oo000@ooooo`Co003o3Ooo
ool3oomo00_ooooo0oooO`0;ooooo`?oogl00_ooool00ol0003oooooooooo`0;ooooo`008oooool0
0ol0003oooooooooo`06ooooo`?o003o1Oooool4ool001;ooooo1?ooO`0Nooooo`Coo`002_ooool4
ool001_ooooo0ol0o`04o`00o`gooooo0oooO`0@ooooo`Coo`003Oooool3o`3o00Co003o1?ooool4
ool000[ooooo1?oo000Cooooo`?oogl01Oooool3o`3o00Co003o=oooool3oomo00;ooooo1?oo000Q
ooooo`?oogl01Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000:ooooo`Coo`004?ooool4
o`00o`gooooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo00?o0000ooooooooool02oooool0
02?ooooo00?o0000ooooooooool01_ooool3o`00o`Gooooo1?oo000Booooo`Coogl07_ooool4ool0
00[ooooo1?oo000Kooooo`?o0?l01?l00?l=ooooo`?oogl04?ooool4ool000gooooo0ol0o`04o`00
o`Cooooo1?oo000:ooooo`Coo`006oooool3o`3o00Co003o=oooool3oomo00;ooooo1?oo000Qoooo
o`?oogl01Oooool3o`3o00Co003o1?ooool4ool000[ooooo1?oo000:ooooo`Coo`004?ooool4o`00
o`gooooo0oooO`0;ooooo`?oogl02oooool3oomo00;ooooo00?o0000ooooooooool02oooool002?o
oooo00?o0000ooooooooool01_ooool3o`00o`Gooooo1?oo000Booooo`Coogl07_ooool4ool000[o
oooo1?oo000Kooooo`?o0?l01?l00?lPooooo`Coo`003Oooool3o`3o00Co003o1?ooool4ool000[o
oooo1?oo000Kooooo`?o0?l01?l00?lgooooo`?oogl00_ooool4ool0027ooooo0oooO`05ooooo`?o
0?l01?l00?l4ooooo`Coo`002_ooool4ool000[ooooo1?oo000@ooooo`Co003o6oooool3oomo00_o
oooo0oooO`02ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo00Kooooo
0ol00?l5ooooo`Coo`004_ooool4oomo01kooooo1?oo000Yooooo`?o0?l01?l00?lPooooo`Coo`00
3Oooool3o`3o00Co003o4_ooool4ool001_ooooo0ol0o`04o`00ocOooooo0oooO`02ooooo`Coo`00
8Oooool3oomo00Gooooo0ol0o`04o`00o`Cooooo1?oo000:ooooo`Coo`002_ooool4ool0013ooooo
1?l00?lKooooo`?oogl04?ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003ooooooooo
o`06ooooo`?o003o1Oooool4ool0067ooooo0ol0o`04o`00oc7ooooo0ol0o`04o`00oa;ooooo1?oo
000Kooooo`?o0?l01?l00?llooooo`Coo`00:Oooool3o`3o00Co003o1?ooool4ool001Sooooo1?oo
000@ooooo`Co003o;_ooool00ol0003oooooooooo`0;ooooo`008oooool3o`0000Kooooo0ol00?l5
ooooo`Coo`00HOooool3o`3o00Co003o<Oooool3o`3o00Co003o4_ooool4ool001_ooooo0ol0o`04
o`00occooooo1?oo000Yooooo`?o0?l01?l00?l4ooooo`Coo`006?ooool4ool0047ooooo0_l0000=
ooooo`008oooool00ol0003oooooooooo`06ooooo`?o003o1Oooool4ool0067ooooo0ol0o`04o`00
oc7ooooo0ol0o`04o`00oa;ooooo1?oo000Kooooo`?o0?l01?l00?mYooooo`?o0?l01?l00?l4oooo
o`Coo`006?ooool4ool004;ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000oooooooo
ool01_ooool3o`00o`Gooooo1?oo001Qooooo`?o0?l01?l00?laooooo`?o0?l01?l00?lBooooo`Co
o`006oooool3o`3o00Co003oJOooool3o`3o00Co003o1?ooool4ool001Sooooo1?oo0012ooooo`03
o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo00Kooooo0ol00?l5ooooo`Coo`00
HOooool3o`3o00Co003o<Oooool3o`3o00Co003o4_ooool4ool001_ooooo0ol0o`04o`00ofWooooo
0ol0o`04o`00o`Cooooo1?oo000Hooooo`Coo`00@_ooool00ol0003oooooooooo`0;ooooo`008ooo
ool00ol0003oooooooooo`06ooooo`?o003o1Oooool4ool0067ooooo0ol0o`04o`00oc7ooooo0ol0
o`04o`00oc7ooooo0ol0o`04o`00ofWooooo0ol0o`04o`00o`Cooooo1?oo000Hooooo`Coo`00@_oo
ool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`06ooooo`?o003o1Oooool4
ool0067ooooo0ol0o`04o`00oc7ooooo0ol0o`04o`00oc7ooooo0ol0o`04o`00ofWooooo0ol0o`04
o`00o`Cooooo1?oo000Hooooo`Coo`00@_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0
003oooooooooo`06ooooo`?o003o1Oooool4ool0067ooooo0ol0o`04o`00oc7ooooo0ol0o`04o`00
oc7ooooo0ol0o`04o`00ofWooooo0ol0o`04o`00o`Cooooo1?oo000Hooooo`Coo`00@_ooool00ol0
003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`06ooooo`?o003o1Oooool4ool0067o
oooo0ol0o`04o`00oc7ooooo0ol0o`04o`00oc7ooooo0ol0o`04o`00ofWooooo0ol0o`04o`00o`Co
oooo1?oo000Hooooo`Coo`00@_ooool00ol0003oooooooooo`0;ooooo`008oooool3o`0000Kooooo
0ol00?l5ooooo`Coo`00HOooool3o`3o00Co003o=?ooool4o`00oc7ooooo0ol0o`04o`00ofWooooo
0ol0o`04o`00ob3ooooo1?oo0011ooooo`;o00003Oooool002?ooooo00?o0000ooooooooool01_oo
ool3o`00o`Gooooo1?oo001Qooooo`?o0?l01?l00?ldooooo`Co003o<Oooool3o`3o00Co003oJOoo
ool3o`3o00Co003o8?ooool4ool004;ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000
ooooooooool01_ooool3o`00o`Gooooo1?oo001Qooooo`?o0?l01?l00?ldooooo`Co003o<Oooool3
o`3o00Co003oJOooool3o`3o00Co003o8?ooool4ool004;ooooo00?o0000ooooooooool02oooool0
02?ooooo00?o0000ooooooooool01_ooool3o`00o`Gooooo1?oo001Qooooo`?o0?l01?l00?ldoooo
o`Co003o<Oooool3o`3o00Co003oJOooool3o`3o00Co003o8?ooool4ool004;ooooo00?o0000oooo
ooooool02oooool002?ooooo00?o0000ooooooooool01_ooool3o`00o`Gooooo1?oo001Qooooo`?o
0?l01?l00?ldooooo`Co003o=?ooool4o`00ofWooooo0ol0o`04o`00ob3ooooo1?oo0012ooooo`03
o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo00Kooooo0ol00?mZooooo`?o0?l0
1?l00?ldooooo`Co003o=?ooool4o`00ofWooooo0ol0o`04o`00ob3ooooo1?oo0012ooooo`03o`00
0?oooooooooo00_ooooo000>ooooo`;o00001?ooool00ol0003oooooooooo`02ooooo`?o00001ooo
ool00ol0003oooooooooo`06ooooo`?o003oJ_ooool3o`3o00Co003o=?ooool4o`00ocCooooo1?l0
0?mYooooo`?o0?l01?l00?mVooooo`03o`000?oooooooooo00_ooooo000=ooooo`04o`000?oooooo
ooooo`0000Wooooo00?o0000ooooooooool01_ooool00ol0003oooooooooo`06ooooo`?o003oJ_oo
ool3o`3o00Co003o=?ooool4o`00ocCooooo1?l00?mYooooo`?o0?l01?l00?mVooooo`03o`000?oo
oooooooo00_ooooo000=ooooo`04o`000?ooooooooooo`0000Wooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`06ooooo`?o003oJ_ooool3o`3o00Co003o=?ooool4o`00ocCooooo1?l0
0?mYooooo`?o0?l01?l00?mVooooo`03o`000?oooooooooo00_ooooo000=ooooo`04o`000?oooooo
ooooo`0000Wooooo00?o0000ooooooooool01_ooool4o`0000Gooooo0ol00?mZooooo`?o0?l01?l0
0?ldooooo`Co003o=?ooool4o`00ofWooooo0ol0o`04o`00of?ooooo1?l0000=ooooo`003Oooool0
1?l0003oooooooooool00009ooooo`03o`000?oooooooooo00Kooooo00?o0000ooooooooool01_oo
ool3o`00of[ooooo0ol0o`04o`00ocCooooo1?l00?ldooooo`Co003oJOooool3o`3o00Co003oI_oo
ool00ol0003oooooooooo`0;ooooo`003Oooool01?l0003oooooooooool00008ooooo`;o00002?oo
ool00ol0003oooooooooo`06ooooo`?o003oJ_ooool3o`3o00Co003o=?ooool4o`00ocCooooo1?l0
0?mYooooo`?o0?l01?l00?mVooooo`03o`000?oooooooooo00_ooooo000>ooooo`;o00002_ooool0
0ol0003oooooooooo`06ooooo`03o`000?oooooooooo00Kooooo0ol00?mZooooo`?o0?l01?l00?ld
ooooo`Co003o=?ooool4o`00ofWooooo0ol0o`04o`00ofKooooo00?o0000ooooooooool02oooool0
02?ooooo00?o0000ooooooooool01_ooool3o`00of[ooooo0ol0o`04o`00ocCooooo1?l00?ldoooo
o`Co003oJOooool3o`3o00Co003oI_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003o
ooooooooo`06ooooo`?o003oJ_ooool3o`3o00Co003o=?ooool4o`00ocCooooo1?l00?mYooooo`?o
0?l01?l00?mVooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo00Kooooo
0ol00?mZooooo`?o0?l01?l00?ldooooo`Co003o=?ooool4o`00ofWooooo0ol0o`04o`00ofKooooo
00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool01_ooool3o`00of[ooooo0ol0
o`04o`00ocCooooo1?l00?ldooooo`Co003oJOooool3o`3o00Co003oI_ooool00ol0003ooooooooo
o`0;ooooo`008oooool3o`0000Kooooo0ol00?mZooooo`?o0?l01?l00?ldooooo`Co003o=?ooool4
o`00ofWooooo0ol0o`04o`00ofGooooo0_l0000=ooooo`008oooool00ol0003oooooooooo`06oooo
o`?o003oJ_ooool3o`3o00Co003o=?ooool4o`00ocCooooo1?l00?mYooooo`?o0?l01?l00?mVoooo
o`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo00Kooooo0ol00?mZooooo`Co
0?l0=oooool4o`00ocCooooo1?l00?mYooooo`?o0?l01?l00?mVooooo`03o`000?oooooooooo00_o
oooo000Sooooo`03o`000?oooooooooo00Kooooo0ol00?mZooooo`Co0?l0=oooool4o`00ocCooooo
1?l00?mYooooo`?o0?l01?l00?mVooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oo
oooooooo00Kooooo0ol00?mZooooo`Co0?l0=oooool4o`00ocCooooo1?l00?m/ooooo`Co003oI_oo
ool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`06ooooo`?o003oJ_ooool4
o`3o03Oooooo1?l00?ldooooo`Co003oK?ooool4o`00ofKooooo00?o0000ooooooooool02oooool0
02?ooooo00?o0000ooooooooool01_ooool3o`00of[ooooo1?l0o`0gooooo`Co003o=?ooool4o`00
ofcooooo1?l00?mVooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo00Ko
oooo0ol00?mZooooo`Co0?l0=oooool4o`00ocCooooo1?l00?m/ooooo`Co003oI_ooool00ol0003o
ooooooooo`0;ooooo`008oooool3o`0000Kooooo0ol00?mZooooo`Co0?l0=oooool4o`00ocCooooo
1?l00?m/ooooo`Co003oIOooool2o`0000gooooo000Sooooo`03o`000?oooooooooo00Kooooo0ol0
0?mZooooo`Co0?l0=oooool4o`00ocCooooo1?l00?m/ooooo`Co003oI_ooool00ol0003ooooooooo
o`0;ooooo`008oooool00ol0003oooooooooo`06ooooo`?o003oJ_ooool4o`3o03Oooooo1?l00?ld
ooooo`Co003oK?ooool4o`00ofKooooo00?o0000ooooooooool02oooool002?ooooo00?o0000oooo
ooooool01_ooool3o`00of[ooooo1?l0o`0gooooo`Co003o=?ooool4o`00ofcooooo1?l00?mVoooo
o`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo00Kooooo0ol00?mZooooo`Co
0?l0=oooool4o`00ocCooooo1?l00?m/ooooo`Co003oI_ooool00ol0003oooooooooo`0;ooooo`00
8oooool00ol0003oooooooooo`06ooooo`?o003oJ_ooool4o`3o03Oooooo1?l00?ldooooo`Co003o
K?ooool4o`00ofKooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool01_oo
ool3o`00of[ooooo1?l0o`0gooooo`Co003o=?ooool4o`00ofcooooo1?l00?mVooooo`03o`000?oo
oooooooo00_ooooo000Sooooo`03o`000?oooooooooo00Kooooo0ol00?mZooooo`Co0?l0=oooool4
o`00ocCooooo1?l00?m/ooooo`Co003oI_ooool00ol0003oooooooooo`0;ooooo`008oooool3o`00
00Kooooo0ol00?mZooooo`Co0?l0=oooool4o`00ocCooooo1?l00?m/ooooo`Co003oIOooool2o`00
00gooooo000Sooooo`03o`000?oooooooooo00Kooooo0ol00?mZooooo`Co0?l0=oooool4o`00ocCo
oooo1?l00?m/ooooo`Co003oI_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003ooooo
ooooo`06ooooo`?o003oJ_ooool4o`3o03Oooooo1?l00?ldooooo`Co003oK?ooool4o`00ofKooooo
00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool01_ooool3o`00of[ooooo1?l0
o`0gooooo`Co003o=?ooool4o`00omKooooo00?o0000ooooooooool02oooool002?ooooo00?o0000
ooooooooool01_ooool3o`00of[ooooo1?l0o`0gooooo`Co003o=?ooool4o`00omKooooo00?o0000
ooooooooool02oooool002?ooooo00?o0000ooooooooool01_ooool3o`00of[ooooo1?l0o`1_oooo
o`Co003oe_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`06ooooo`?o
003oJ_ooool4o`3o06oooooo1?l00?oFooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`00
0?oooooooooo07?ooooo1?l0o`1_ooooo`Co003oe_ooool00ol0003oooooooooo`0;ooooo`008ooo
ool3o`0007?ooooo1?l0o`1_ooooo`Co003oeOooool2o`0000gooooo000Sooooo`03o`000?oooooo
oooo07?ooooo1?l0o`1_ooooo`Co003oe_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0
003oooooooooo`1cooooo`Co0?l0Koooool4o`00omKooooo00?o0000ooooooooool02oooool002?o
oooo00?o0000ooooooooool0Loooool4o`3o06oooooo1?l00?oFooooo`03o`000?oooooooooo00_o
oooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`1_ooooo`Co003oe_ooool00ol0003ooooo
ooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0Koooool4o`00omKooooo00?o
0000ooooooooool02oooool000kooooo0_l00004ooooo`04o`000?ooooooooooooooo`Go00001_oo
ool00ol0003oooooooooo`1cooooo`Co0?l0Koooool4o`00omKooooo00?o0000ooooooooool02ooo
ool000gooooo00Co0000oooooooooooo00002?ooool01?l0003oooooooooool00006ooooo`03o`00
0?oooooooooo07?ooooo1?l0o`1_ooooo`Co003oe_ooool00ol0003oooooooooo`0;ooooo`003Ooo
ool01?l0003oooooooooool00009ooooo`03o`000?oooooooooo00Kooooo1?l0001booooo`Co0?l0
Koooool4o`00om?ooooo1?l0000=ooooo`003Oooool01?l0003oooooooooool0000:ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool0Loooool4o`3o06oooooo1?l00?oFooooo`03o`00
0?oooooooooo00_ooooo000=ooooo`04o`000?ooooooooooo`0000Oooooo00Go0000oooooooooooo
ooooo`000006ooooo`03o`000?oooooooooo07?ooooo1?l0o`1_ooooo`Co003oe_ooool00ol0003o
ooooooooo`0;ooooo`003Oooool01?l0003oooooooooool00007ooooo`05o`000?oooooooooooooo
ool000001_ooool00ol0003oooooooooo`1cooooo`Co0?l0Koooool4o`00omKooooo00?o0000oooo
ooooool02oooool000kooooo0_l00009ooooo`?o00001oooool00ol0003oooooooooo`1cooooo`Co
0?l0Koooool4o`00omKooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0
Loooool4o`3o06oooooo1?l00?oFooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oo
oooooooo07?ooooo1?l0o`1_ooooo`Co003oe_ooool00ol0003oooooooooo`0;ooooo`008oooool0
0ol0003oooooooooo`1cooooo`Co0?l0Koooool4o`00omKooooo00?o0000ooooooooool02oooool0
02?ooooo0ol0001cooooo`Co0?l0Koooool4o`00omGooooo0_l0000=ooooo`008oooool00ol0003o
ooooooooo`1cooooo`Co0?l0Koooool4o`00omKooooo00?o0000ooooooooool02oooool002?ooooo
00?o0000ooooooooool0Loooool4o`3o06oooooo1?l00?oFooooo`03o`000?oooooooooo00_ooooo
000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02ooo
ool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;
ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo
00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000oooooooo
ool02oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003ooooo
ooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oo
oooooooo00_ooooo000Sooooo`?o0000Loooool4o`3o0?ooooooBOooool2o`0000gooooo000Soooo
o`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?o
oooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`00
8oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo
000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02ooo
ool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;
ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo
00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000oooooooo
ool02oooool002?ooooo0ol0001cooooo`Co0?l0oooooom9ooooo`;o00003Oooool002?ooooo00?o
0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool0
0ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Soooo
o`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?o
oooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`00
8oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo
000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02ooo
ool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;
ooooo`008oooool3o`0007?ooooo1?l0o`3oooooodWooooo0_l0000=ooooo`008oooool00ol0003o
ooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`00
0?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?ooooo00?o
0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool0
0ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Soooo
o`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool000ko
oooo0_l00004ooooo`03o`000?oooooooooo00;ooooo0ol00007ooooo`03o`000?oooooooooo07?o
oooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool000gooooo00Co0000oooooooooooo
00001oooool01Ol0003ooooooooooooooooo000000Kooooo00?o0000ooooooooool0Loooool4o`3o
0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`003Oooool01?l0003oooooooooool0000;oooo
o`03o`000?oooooooooo00Cooooo1?l0001booooo`Co0?l0oooooom7ooooo`Co00003Oooool000go
oooo00Co0000oooooooooooo00002Oooool2o`0000Oooooo00?o0000ooooooooool0Loooool4o`3o
0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`003Oooool01?l0003oooooooooool0000;oooo
o`03o`000?oooooooooo00Cooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0
003oooooooooo`0;ooooo`003Oooool01?l0003oooooooooool00007ooooo`05o`000?oooooooooo
ooooool000001_ooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooo
oooo00_ooooo000>ooooo`;o00002Oooool3o`0000Oooooo00?o0000ooooooooool0Loooool4o`3o
0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co
0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo
1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0Looo
ool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool3o`0007?ooooo1?l0o`3o
ooooodWooooo0_l0000=ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:oooo
o`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[o
oooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?oooooo
B_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooo
oom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3o
oooood[ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o
0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co
0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`?o0000Loooool4o`3o0?oooooo
BOooool2o`0000gooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o
0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool0
0ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:oooo
o`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[o
oooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?oooooo
B_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooo
oom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3o
oooood[ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o
0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool3o`0007?ooooo1?l0o`3oooooodWo
oooo0_l0000=ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`00
0?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o
0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool0
0ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:oooo
o`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[o
oooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?oooooo
B_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooo
oom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`?o0000Loooool4o`3o0?ooooooBOooool2
o`0000gooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000oooo
ooooool02oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003o
ooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`00
0?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o
0000ooooooooool02oooool000kooooo0_l00004ooooo`03o`000?oooooooooo00?ooooo0ol00006
ooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool0
00gooooo00Co0000oooooooooooo00002_ooool00ol0003oooooooooo`05ooooo`03o`000?oooooo
oooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool000gooooo00Co0000oooo
oooooooo00001oooool5o`0000Kooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool0
0ol0003oooooooooo`0;ooooo`003Oooool01?l0003oooooooooool00007ooooo`04o`000?oooooo
ooooo`0000Oooooo1?l0001booooo`Co0?l0oooooom7ooooo`Co00003Oooool000gooooo00Co0000
oooooooooooo00002?ooool00ol0003oooooo`000007ooooo`03o`000?oooooooooo07?ooooo1?l0
o`3ooooood[ooooo00?o0000ooooooooool02oooool000gooooo00Co0000oooooooooooo00002Ooo
ool2o`0000Oooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003ooooooooo
o`0;ooooo`003_ooool2o`0000_ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`1c
ooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo
07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000oooooooo
ool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003ooooo
ooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oo
oooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?ooooo0ol0001c
ooooo`Co0?l0oooooom9ooooo`;o00003Oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o
0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co
0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo
1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0Looo
ool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1c
ooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo
07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000oooooooo
ool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool3o`0007?ooooo
1?l0o`3oooooodWooooo0_l0000=ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooo
oom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3o
oooood[ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o
0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co
0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo
1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0Looo
ool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1c
ooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`?o0000Loooool4o`3o
0?ooooooBOooool2o`0000gooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[o
oooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?oooooo
B_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooo
oom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3o
oooood[ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o
0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co
0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo
1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?ooooo0ol0001cooooo`Co0?l0oooo
oom9ooooo`;o00003Oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool0
0ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:oooo
o`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[o
oooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?oooooo
B_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooo
oom:ooooo`03o`000?oooooooooo00_ooooo000>ooooo`;o00001?ooool00ol0003oooooooooo`02
ooooo`?o00001oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooo
oooo00_ooooo000=ooooo`04o`000?ooooooooooo`0000Oooooo00Go0000ooooooooooooooooo`00
0006ooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02ooo
ool000gooooo00Co0000oooooooooooo00002oooool00ol0003oooooooooo`04ooooo`03o`000?oo
oooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool000gooooo00Co0000
oooooooooooo00002oooool00ol0003oooooooooo`04ooooo`Co0000L_ooool4o`3o0?ooooooAooo
ool4o`0000gooooo000=ooooo`04o`000?ooooooooooo`0000Oooooo1?l00007ooooo`03o`000?oo
oooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool000gooooo00Co0000
oooooooooooo00001oooool00ol0003oooooooooo`08ooooo`03o`000?oooooooooo07?ooooo1?l0
o`3ooooood[ooooo00?o0000ooooooooool02oooool000kooooo0_l00008ooooo`Go00001_ooool0
0ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Soooo
o`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?o
oooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`00
8oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo
000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02ooo
ool002?ooooo0ol0001cooooo`Co0?l0oooooom9ooooo`;o00003Oooool002?ooooo00?o0000oooo
ooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003o
ooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`00
0?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?ooooo00?o
0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool0
0ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Soooo
o`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?o
oooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`00
8oooool3o`0007?ooooo1?l0o`3oooooodWooooo0_l0000=ooooo`008oooool00ol0003ooooooooo
o`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooo
oooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000oooo
ooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003o
ooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`00
0?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?ooooo00?o
0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool0
0ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Soooo
o`?o0000Loooool4o`3o0?ooooooBOooool2o`0000gooooo000Sooooo`03o`000?oooooooooo07?o
oooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0
Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003ooooooooo
o`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooo
oooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000oooo
ooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003o
ooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`00
0?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?ooooo0ol0
001cooooo`Co0?l0oooooom9ooooo`;o00003Oooool002?ooooo00?o0000ooooooooool0Loooool4
o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1coooo
o`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo07?o
oooo1?l0o`3ooooood[ooooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0
Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`008oooool00ol0003ooooooooo
o`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooooooo00_ooooo000>ooooo`;o00001?ooool0
0ol0003oooooooooo`02ooooo`?o00001oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:
ooooo`03o`000?oooooooooo00_ooooo000=ooooo`04o`000?ooooooooooo`0000Oooooo00Go0000
ooooooooooooooooo`000006ooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o
0000ooooooooool02oooool000gooooo00Co0000oooooooooooo00001oooool01Ol0003ooooooooo
oooooooo000000Kooooo1?l0001booooo`Co0?l0oooooom7ooooo`Co00003Oooool000gooooo00Co
0000oooooooooooo00001oooool4o`0000Oooooo00?o0000ooooooooool0Loooool4o`3o0?oooooo
B_ooool00ol0003oooooooooo`0;ooooo`003Oooool01?l0003oooooooooool00007ooooo`03o`00
0?oooooooooo00Sooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003ooooo
ooooo`0;ooooo`003Oooool01?l0003oooooooooool00008ooooo`03o`000?oooooooooo00Oooooo
00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003oooooooooo`0;ooooo`003_oo
ool2o`0000[ooooo0_l00007ooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o
0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool0
0ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:oooo
o`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[o
oooo00?o0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?oooooo
B_ooool00ol0003oooooooooo`0;ooooo`008oooool3o`0007?ooooo1?l0o`3oooooodWooooo0_l0
000=ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooo
oooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000oooo
ooooool02oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003o
ooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`00
0?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o
0000ooooooooool02oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool0
0ol0003oooooooooo`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:oooo
o`03o`000?oooooooooo00_ooooo000Sooooo`?o0000Loooool4o`3o0?ooooooBOooool2o`0000go
oooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000ooooooooool0
2oooool002?ooooo00?o0000ooooooooool0Loooool4o`3o0?ooooooB_ooool00ol0003ooooooooo
o`0;ooooo`008oooool00ol0003oooooooooo`1cooooo`Co0?l0oooooom:ooooo`03o`000?oooooo
oooo00_ooooo000Sooooo`03o`000?oooooooooo07?ooooo1?l0o`3ooooood[ooooo00?o0000oooo
ooooool02oooool002?ooooo00?o0000ooooooooool0ooooooo1ooooo`03o`000?oooooooooo00_o
oooo000Sooooo`03o`000?oooooooooo0?oooooo`Oooool00ol0003oooooooooo`0;ooooo`008ooo
ool00ol0003oooooooooo`3ooooool7ooooo00?o0000ooooooooool02oooool002?ooooo0ol00006
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;
ooooo`03o`000?oooooooooo00_ooooo00?o0000ooooooooool01?ooool2o`0000gooooo000Soooo
o`03o`000?oooooooooo00Kooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;oooo
o`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;oooo
o`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;oooo
o`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;oooo
o`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;oooo
o`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;oooo
o`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;oooo
o`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;oooo
o`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;oooo
o`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`0;oooo
o`03o`000?oooooooooo00_ooooo00?o0000ooooooooool02oooool00ol0003oooooooooo`05oooo
o`03o`000?oooooooooo00_ooooo000Sooooo`03o`000?oooooooooo00Kooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`0;ooooo`03o`000?oooooooooo00_ooooo00?o0000oooooooo
ool02oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00_ooooo000Soooooooo0000
aOl0000=ooooo`00oooooooeooooo`00oooooooeooooo`00oooooooeooooo`00oooooooeooooo`00
oooooooeooooo`00oooooooeooooo`00oooooooeooooo`00:_ooool3o`0000[ooooo1Ol0000:oooo
o`?o00003?ooool3o`0000[ooooo0ol0000;ooooo`?o00003?ooool00ol0003oooooooooo`0:oooo
o`?o00002oooool2o`0000Wooooo0ol00004ooooo`;o00001Oooool3o`0000?ooooo0ol00005oooo
o`?o00000_ooool5o`0000Cooooo0ol00003ooooo`?o00001Oooool3o`0000Cooooo0ol00004oooo
o`?o00000oooool3o`0000Gooooo0ol00003ooooo`?o00001Oooool3o`0000Cooooo00?o0000oooo
ooooool01?ooool3o`0000?ooooo0ol00005ooooo`?o00000oooool2o`0000Gooooo1Ol00003oooo
o`;o00001?ooool5o`0000;ooooo0ol00004ooooo`Go000000?oooooo`000?l000000ol00003oooo
o`Go00000_ooool3o`0000Cooooo1Ol00003ooooo`?o00000oooool5o`0000;ooooo0ol00004oooo
o`Go00000_ooool3o`0000Cooooo1Ol00003ooooo`03o`000?oooooooooo00?ooooo1Ol00002oooo
o`?o00001?ooool5o`0000;ooooo0_l00006ooooo`?o00001?ooool2o`0000Gooooo0ol00003oooo
o`?o00001Oooool3o`0000;ooooo1Ol0000Aooooo`00:oooool00ol0003oooooooooo`0:ooooo`04
o`000?ooooooooooo`0000Wooooo00Go0000ooooooooooooooooo`00000<ooooo`03o`000?oooooo
oooo00Sooooo00Go0000ooooooooooooooooo`000009ooooo`05o`000?ooooooooooooooool00000
2oooool00ol0003oooooooooo`09ooooo`05o`000?ooooooooooooooool000003?ooool00ol0003o
ooooooooo`07ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00001Oooool00ol0
003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00_ooool01?l0
003oooooooooool00005ooooo`05o`000?ooooooooooooooool000000oooool00ol0003ooooooooo
o`03ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool01?ooool01Ol0003ooooooooo
oooooooo000000?ooooo00?o0000ooooooooool00oooool01Ol0003ooooooooooooooooo000000?o
oooo00?o0000ooooooooool00oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Go
oooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00?o0000oooo
ooooool01?ooool00ol0003oooooooooo`03ooooo`04o`000?ooooooooooo`0000;ooooo00Co0000
oooooooooooo00001?ooool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo00Cooooo
00Co0000oooooooooooo00000_ooool01?l0003oooooooooool00004ooooo`06o`000?oooooooooo
o`000?oooooo00000oooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Cooooo
00?o0000ooooooooool00oooool01_l0003oooooooooool0003oooooo`0000?ooooo00?o0000oooo
ooooool00_ooool01_l0003oooooooooool0003oooooo`0000?ooooo00?o0000ooooooooool00_oo
ool01?l0003oooooooooool00003ooooo`03o`000?oooooooooo00Cooooo00Ko0000oooooooooooo
0000ooooool00003ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00001?ooool0
0ol0003oooooooooo`02ooooo`05o`000?ooooooooooooooool000000_ooool01?l0003ooooooooo
ool00003ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03ooooo`05
o`000?ooooooooooooooool000000_ooool01?l0003oooooooooool0000Aooooo`00:oooool00ol0
003oooooooooo`0;ooooo`03o`000?oooooooooo00gooooo00?o0000ooooooooool01oooool5o`00
00gooooo00?o0000ooooooooool01oooool01Ol0003ooooooooooooooooo000000cooooo00?o0000
ooooooooool02?ooool01Ol0003ooooooooooooooooo000000gooooo00?o0000ooooooooool01_oo
ool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool0
0oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool0
1Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00?ooooo00Co0000oooooooooooo
oooo1Ol00005ooooo`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00oooool01Ol0003o
oooooooooooooooo000000?ooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`04oooo
o`03o`000?oooooooooo00Cooooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo
oooo00?ooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`03ooooo`03o`000?oooooo
oooo00;ooooo00Co0000oooooooooooo00001Oooool00ol0003oooooooooo`03ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00Gooooo00?o0000ooooooooool00oooool01?l0003oooooooooooooool5o`0000Gooooo
00?o0000ooooooooool01Oooool00ol0003oooooooooo`03ooooo`05o`000?ooooooooooooooool0
00000oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool01?ooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooooooool00oooool00ol0
003oooooooooo`05ooooo`03o`000?oooooooooo00Gooooo00Co0000oooooooooooo00000_ooool0
0ol0003oooooooooo`05ooooo`05o`000?ooooooooooooooool000002Oooool01Ol0003ooooooooo
oooooooo000001?ooooo000[ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool02_oo
ool2o`0000[ooooo00Co0000oooooooooooo00003_ooool00ol0003oooooooooo`07ooooo`Co0000
3Oooool00ol0003oooooooooo`09ooooo`?o00002oooool4o`0000Sooooo00?o0000ooooooooool0
0_ooool01?l0003oooooooooool00005ooooo`03o`000?oooooooooo00?ooooo00?o0000oooooooo
ool01Oooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00?o0000oooooooo
ool00oooool2o`0000Kooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oooooooooo
00Cooooo00?o0000ooooooooool01Oooool00ol0003oooooooooo`03ooooo`04o`000?oooooooooo
ooooo`Co00001_ooool00ol0003oooooooooo`04ooooo`03o`000?oooooooooo00Cooooo00?o0000
ooooooooool00_ooool3o`0000Kooooo00?o0000ooooooooool00_ooool4o`0000Kooooo00Go0000
ooooooooooooooooo`000002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool00_oo
ool00ol0003oooooooooo`06ooooo`03o`000?oooooooooo00?ooooo00?o0000ooooooooool01Ooo
ool00ol0003oooooooooo`02ooooo`;o00001oooool01?l0003oooooooooool00002ooooo`03o`00
0?oooooooooo00Gooooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`04ooooo`03o`00
0?oooooooooo00Co00001oooool00ol0003oooooooooo`03ooooo`03o`000?oooooooooo00Gooooo
00Co0000oooooooooooooooo0ol00007ooooo`04o`000?ooooooooooooooo`Co00001Oooool2o`00
00?ooooo00Co0000oooooooooooo00001Oooool2o`0000Cooooo00?o0000ooooooooool01Oooool2
o`0000Gooooo00?o0000ooooooooool04?ooool002_ooooo00?o0000ooooooooool02Oooool01Ol0
003ooooooooooooooooo000000gooooo00?o0000ooooooooool02?ooool00ol0003oooooo`00000:
ooooo`Co00002_ooool00ol0003oooooooooo`0?ooooo`03o`000?oooooooooo00Oooooo00Go0000
ooooooooooooooooo`000009ooooo`05o`000?ooooooooooooooool000002?ooool00ol0003ooooo
ooooo`02ooooo`04o`000?ooooooooooo`0000Gooooo00?o0000ooooooooool00oooool00ol0003o
ooooooooo`05ooooo`05o`000?ooooooooooooooool000000oooool00ol0003oooooooooo`03oooo
o`03o`000?oooooooooo00Gooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`02oooo
o`03o`000?oooooo000000Kooooo00Co0000oooooooooooooooo1?l00006ooooo`05o`000?oooooo
ooooooooool000002Oooool00ol0003oooooooooo`05ooooo`03o`000?oooooooooo00?ooooo00Go
0000ooooooooooooooooo`000003ooooo`03o`000?oooooooooo00?ooooo00Go0000oooooooooooo
ooooo`000003ooooo`05o`000?ooooooooooooooool000000oooool01?l0003oooooooooool00002
ooooo`05o`000?ooooooooooooooool000000oooool01Ol0003ooooooooooooooooo000000Gooooo
00Oo0000ooooooooooooooooo`000?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003
ooooo`03o`000?oooooooooo00?ooooo00Go0000ooooooooooooooooo`000003ooooo`06o`000?oo
ooooooooo`000?oooooo00001?ooool01_l0003ooooooooooooooooo0000ooooo`Co00001?ooool0
1ol0003ooooooooooooooooo0000ooooool000001oooool01Ol0003ooooooooooooooooo000000Go
oooo00Go0000ooooooooooooooooo`000003ooooo`03o`000?oooooo000000?ooooo00Go0000oooo
ooooooooooooo`000003ooooo`03o`000?oooooo000000?ooooo00?o0000ooooooooool01Oooool0
1?l0003oooooooooool00002ooooo`03o`000?oooooooooo00Gooooo00Go0000oooooooooooooooo
o`000009ooooo`03o`000?oooooo000000?ooooo00?o0000ooooooooool03oooool002[ooooo0_l0
000;ooooo`05o`000?ooooooooooooooool000002Oooool01Ol0003ooooooooooooooooo000000_o
oooo0_l0000:ooooo`03o`000?oooooooooo00cooooo00?o0000ooooooooool02_ooool01Ol0003o
oooooooooooooooo000000Wooooo00Go0000ooooooooooooooooo`000009ooooo`05o`000?oooooo
ooooooooool000001oooool2o`0000Cooooo00Co0000oooooooooooo00001?ooool2o`0000Cooooo
0_l00006ooooo`;o00000oooool01Ol0003ooooooooooooooooo000000Cooooo0_l00003ooooo`05
o`000?ooooooooooooooool000001?ooool2o`0000Gooooo0_l00005ooooo`;o00000oooool00ol0
003oooooooooo`06ooooo`;o00001?ooool00ol0003oooooooooo`05ooooo`;o00000oooool01Ol0
003ooooooooooooooooo000000Cooooo0_l00003ooooo`05o`000?ooooooooooooooool000001?oo
ool2o`0000?ooooo00Go0000ooooooooooooooooo`000003ooooo`05o`000?ooooooooooooooool0
00000_ooool01?l0003oooooooooool00003ooooo`05o`000?ooooooooooooooool000000_ooool2
o`0000Gooooo00Oo0000ooooooooooooooooo`000?oooooo000000?ooooo00Go0000oooooooooooo
ooooo`000003ooooo`03o`000?oooooo000000?ooooo00Go0000ooooooooooooooooo`000003oooo
o`04o`000?ooooooooooooooo`;o00001?ooool01ol0003ooooooooooooooooo0000ooooool00000
1oooool01Ol0003ooooooooooooooooo000000;ooooo00?o0000ooooooooool01?ooool01ol0003o
oooooooooooooooo0000ooooool000000oooool01Ol0003ooooooooooooooooo000000?ooooo00?o
0000ooooool000000oooool01Ol0003ooooooooooooooooo000000?ooooo00?o0000ooooool00000
0oooool01Ol0003ooooooooooooooooo000000?ooooo00Co0000oooooooooooo00000_ooool01Ol0
003ooooooooooooooooo000000?ooooo00?o0000ooooooooool00_l00005ooooo`07o`000?oooooo
ooooooooool0003oooooo`000003ooooo`03o`000?oooooooooo00oooooo000[ooooo`03o`000?oo
oooooooo00[ooooo0ol0000;ooooo`?o00003Oooool00ol0003oooooooooo`08ooooo`Go00002ooo
ool2o`0000[ooooo1Ol0000:ooooo`?o00002oooool3o`0000Wooooo00?o0000ooooooooool00ooo
ool2o`0000Kooooo00?o0000ooooooooool00oooool00ol0003oooooooooo`05ooooo`03o`000?oo
oooooooo00;ooooo0ol00006ooooo`03o`000?oooooooooo00;ooooo0ol00006ooooo`03o`000?oo
oooooooo00Cooooo00?o0000ooooooooool01?ooool01?l0003oooooooooooooool5o`0000Gooooo
00?o0000ooooooooool00oooool2o`0000Kooooo00Co0000oooooooooooooooo1Ol00005ooooo`03
o`000?oooooooooo00;ooooo0ol00006ooooo`03o`000?oooooooooo00;ooooo0ol00005ooooo`?o
00001?ooool2o`0000Gooooo0ol00004ooooo`03o`000?oooooooooo00Cooooo0ol00003ooooo`?o
00001Oooool3o`0000?ooooo0ol00005ooooo`?o00001Oooool00ol0003oooooooooo`03ooooo`?o
00000_ooool5o`0000Cooooo0ol00004ooooo`;o00001Oooool3o`0000;ooooo1Ol00004ooooo`?o
00000oooool3o`0000Gooooo0ol00003ooooo`?o00001Oooool3o`0000Cooooo0_l00005ooooo`?o
00001?ooool00ol0003oooooooooo`04ooooo`?o00000oooool3o`0001;ooooo0000\
\>"],
  ImageRangeCache->{{{0, 499}, {308, 0}} -> {-1.85349, -0.0521839, 0.0713992, \
0.00245282}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Visualize test images", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(gtmp = \(ShowImage[#, 1, 255, 0, 
              DisplayFunction \[Rule] Identity] &\)\  /@ \ 
          testImages;\)\), "\[IndentingNewLine]", 
    \(\(gtest = 
        Show[GraphicsArray[Partition[gtmp, 2]], 
          DisplayFunction \[Rule] $DisplayFunction, ImageSize \[Rule] 600, 
          GraphicsSpacing \[Rule] 0.05];\)\)}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .5 
%%ImageSize: 600 300 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 0.0119048 0.47619 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.0119048 0.488386 0.244193 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 0.003125 0 0.003125 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 .5 scale
320 string
320 160 8 [320 0 0 160 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
B793CD6F504F4A31642F664A84521F564C28401F175859354E22154250686D6A192C553D
371D503856562725693F3C371A3B59582538755E2247404A403B4F1F5B7A53333E440B2F
712948272C782F3C3F553A452C4E2B7773885B454E523B6F3737286D4C5E997F673D703D
675B674E2B4A60314E8457465C395D493856554350534F43804989515A6D3C673653867C
4CAC6325486E655A836F3C776324698457446F5747A1764B7A2F3686826762704540668C
695F264F78383E9E616AC19A5A94865555635F377B65678E5B465F414FA2676995415FCD
683C5D3B2D7A796F441F8F5B3A5597A176374D6A42706F3B23986959B48C4F2B54683C38
3C6A497C9B6D42456F3B66524267685A3E9BA64C3B33715B597EA49A5A4776794B8F985C
572E8F51387B5D273F6444666A762A5BAC4D2E55649B854F796263376B593C96
B187B5643F504B236238766455682D5B3D232916195F563E4A31234A5B5C60963F284138
35454636446141246D40354A4B5459682D3563662C5F635B404A5F334C846B37487D2336
812F41492D883E3844694C49405F2D708D865B4B494C3F794B3C2C5D3E43A0AD6F4D6B30
795E51651C3E6A3E5084563776477772355D583C677B774277388888534232745F465D44
45BB8641497C2B336877409472476278603B62753C6465617A523F846E6567575D755598
79724086A0733BA34B3CBAA25B8D98384261694A71303E69624F483935725C89B6253CB9
7F3D724721417A644D309A6152385C7559377A9E5A6A7D402C7D70478D67422456707C59
49703C81BA9257437B275D67525E66563E6A7F58452C6A6A5D5B668963446F5A32847E74
552D78843A605B2836703D6B66853B66A171352D2F807856A99E8639715A309A
A27B8D74455E6E2C813D7D68346B35593B1B291E205E4F42473B2B495866568D412E3227
2A503E2D3981522160473E525460565125346D5D23725832384D5633438E322D4F70282D
5C274458319332395C9055434F382150716F4E59542A2D51453C33616534758F66528B41
5940417F36455D204A8486365D2963914364412544547B4366315F845164384433404D54
35897E474981714E66752A898C53708248588AA1487C323A677D4E8B5C43708286685CB3
85652D5D737B40B2955AC9A63B7D955F4A5A7F4C8A76506664525E512D71507B9B4D46BF
7F4189812D338B5B4F2B7F62613E716B794A488A6263714D51888C4B9974352F737D9054
447053589C855E3C8749566062595957456383634E2A6E5D6C755980754068894F6B9887
4B2D6C85416871333C743D65646F2E79A4873F2D37842E3C6D8082376C2F1C7E
BD8A6E7C4963762D7C416F63205F3D553514282A275F4D30505432475056578E3E383324
3350422029826820541C2C47405558512727583A1969846334475B4147916E3B3F773329
5F2F51592D89212842734256583D25738E7439667E593A563A291D3B724C91A256408D4F
614035733545512B315982386036557D4551553D484D6E4A82565C785093454B313C6B80
529C6F30578989527A7B3E8EA23E83A461375E99496D705B668662A99F8C73619A5957BF
AD9C3B6E5C462F7E8C61C1916A748E634A544C54A17F5A5E4D3F51552B683972A86D459C
8432637639278C5A503A7C554E3A675C774B4785666A6A452A6A9152886E40234C5E7D57
40686163916C5A3195545A55444D5D4A487872413C26764C6D81588780396987427C8E7A
3744848F4E616B21307547657B642B60984D31384A8977396A7D562A7959216B
934559674770391F63404A1711553849380926322647481D3141284E281F317743374730
283E453520715819606039403843687341275A3D19618C80454D451E4086653A415C2C36
834B5D4C28745F405563273A6360347B8D662350614D355243311E326035899C60478E58
5E4F3B703154624D37527F4C794F4D6630626D445257502B68644B783F763E673D314D6E
4E9A8B4741745D3D61735278B15B796D7E406C8745848F71707450918E8E6D6D71453FAF
88904A7F5E8D3A7B7551C0866A75A1634B4C8539A18A525C82563D5E2869526DAD4D3F7E
8E3360583B1F8F543540604345407068684E4279726C7A5636678F61786746294B52825F
42817A599C776436924459492B4D7753477269532937907E89765D873636808F537F974A
1B325B72386E7F3B4B99385758714884A1753B222F757350868D8F5757472873
A27469723C4D5A2E4826544F1646394A5C1B393120315A4D453A1743553F4D9A33336762
2A13305522804915545E353E2D29607B4E29475A2849817D5057545F346B6B37554E211D
62434E47236D4D386568774055713893AB885B585E3E3F647253262D5B2376A0784E9228
5965396A233A56553F5F67306B5252823D435F2A49657B496943467D4B5A3062513F564F
3485A56D4864775F6869554FBF7D867A9539648A5F809F7573918796948A6E889C8A49CC
6D7F3D7C659449748A46A1796758A6664F588C2F848F4A6D7D534166227D7766A6644775
9A3D6168342E8A9C6F5468A847213F3B6F494F77345E7469375C7E4C839A5C37484F803A
51778D44B689212B7755616C2F2D40563465867C3D205964485A8D9B4C254A733B819B76
3E3D548750546F2A3B8A4C66566B4A999E7D46282D652C5E80708C5E74441966
7A5CAF9E4F50703C61483F5F1C321E30551C3B270F1A4B413E34152D4D405FA641224163
30392F4022838728515F4357663B496D5D3055804D4B8A3F324D586B326A592A6C7A401F
682D3256396E3322607879407236195F9C804660834327383E432539753469897044A35C
444D2C735F44513D334B8E4D6B573D7B65406A47435282616C714588546A5455533C5580
5684A374385E9568706C5E47CE90737A9F4165AA757A886774896C86A48174B490865DC8
7A4F4582908E4D707429828A6047B0495C7E882B67463873875040542B70895C9E5E1E5E
8B4869712C1A73763033595C473F5646915E40726F5960464C7866264260633D5F66914E
375F6838958A5B37725045503F41584F2C635B5247296D767150868A6E418EA971819568
2A51809D6A70993122695A4F9C7D316B756448282C675C34603D7B66686A3C5D
779FD0B85A604E2B6E6C565E224E50304F093846231B572A3D5D3135432B5495662E453B
2A60412F105B702857312746693C5955151C445F3837645341464A47287D73295C5A5320
624B30654357733F4B786D417C62337DB583353B565F364143371921634A79A272379E68
385F41777D41497B444E986B5F79457F6B456A54494B7165607643886166584867455C85
6B72A07F3764A8696B6F4E43AF715B87874F8FC77C657E4D7B7D626E903A42727E7B48AE
8A74465C63794A70934197894A5DA84B323E643A70864C5B84475862235C6B4CA7723646
772F5470561D7A883B515C7252556B4B9C6F45677C6E656D3760906466665A2B3D4E9175
375A894498957D45707646544E466B73555A5F62303E85978D4F9D9B562B4A945683BC93
4C31417E404A7E522E674132997D4A8C9A513A47516F5B6A8D3E7A5D3F6C4556
5872D3B9585758294E4D413214414129581F31400E136D3E263819385A2A4A846628546F
34472D502574621C475833393D296C795F233755442C7872574F4368226B833663452C1E
59653A674060854E507A4B2C8166396CA885654B5959433F45482B2C6B5D6F92A3469968
34604B748B43488C46518F6F6887467A605675605048614367873C7C2C665A3A55294E7A
5D5CAD5D38637F625A873F388F5D51977936537F745A9056636F5963B88F67B5A37146A1
AB7F4E9076754459864CA2A07C63B56B45416035619055618C4B3B662F4F5D318682413D
7D5C5484711F61B05E546D7548557F6692513E6A6274986A326FA97C7A756234405C9B83
3D6C98408D9D663E8F7846614D2344556B598F804D2F5D764A3D8496893750AC706AA088
76433F937444876A38566343B1A35E8A95655425305548509B4983803D59364B
5442A1BC814D6B52433B366E28414E2153282F4F19154C34202F1D264D32648C4B183969
303B33432665733F436E362D5237546F84362C504524745957514680325F9D444951691B
4972386335378F4158766F476A593965B3846C47483C3D507062342D6A465493A3537D49
33664A7D4F34587A44556A31626F437540315F57444C634848773B79535858375233496C
5E47AB804153804D4B8C634E986845767E596A8563528B6A75755248AA9163B88F6862A2
9D78558C7F9B625D9E5B96BD6753C36A584775445E96565D9F664460373D7D5185844B36
83634980683071C96858817770283B3E71434880523D5E83326194446AA67C613D47834C
335887438AAD5F29534F3A665F4748564A3D7067613A62765E578C8E8F3D46B48E709D93
85483C7B6A498A7246496E469B67616C9269552B335C623C967A838337513451
534454C7AF586B5F3D8740813444722A4E362661261550411D2C2C2E3C3469735A203364
373C3F412A5A7E4339783C3161495D4C7B3026552B1959464B56507C364E8C424090831C
4079456C413596295D7D7B556B60283E706F5441483A1D3D455235336135456587447B63
3147347665313C5C3F65856764693F7D83497741374683774C793A5A64544C3C6C425D5C
45398C7E455A9A6D4D87494C9A7F5783683377919D5A965F7D90754CB16F51A8A99060B2
9A5F2E70869A5850852950907B47CC74344862425C7A397F8E60544423539E627DA1394E
85674CA2251940A659417F75805653589075416D685A765441869B5A41555D2A395A8476
4246765B72A3833A64834457553F45535D44735D3F3B536D626491828F5A39BC8C68997E
74465D8884547A7641677E4D8764515CA98E6332366B41235C8A94642E4E1B3C
5B4548C0B95D656935853E773C577B314C4E21602F15433A1D302929403C61787D33365C
45514243274EAB57316D38566E446058512E476444186D3122313F623744851F1F56671C
38451F494F3077173A59593F4E58232D877C4E3E586E3B42473C1B3B636E608CB4414F6E
3F54295182474442274B75806C763F5E8E4569583F45798E5488496D727270466D48597D
7440976E336B9879487364377B835781735B456395578F5A4E616C50BF6B4C85928659A0
976A4B6864803C619C526BA57C34AE6A392E515566914E4464435C602C31783B92943620
6733365A512A37C432337B3D4254636C7E643E5C585C916E294F8E7E4D6A581D21426588
4B4671707DB47A2D776F375E4332587654456A5A3E416EA367678E8C812B42A47F4D8B70
2B224064654385371C48693880A0403D7A5F512E3359292D3E746C713983473E
554E3EA1A9635848336A3040152F56354C47195C40173C3319312A2B4219335B713B3032
1242373C0D286244225D252C463E585D39231D45541D54685E3A3B552D449E541E49601F
34854154652898614B64795F4B6D473799872F2D3C3C43475249303347524781AF6E5162
2F66534C8D50547E4F55747D7B9E595E75487F6843465C3C498A49604F4B69474D37395F
7241A87D3546A98447797049999252827C70425570508D8A5A9C5D3E8598788C904D3D85
A68051677082513F8A5B81A3643D986D38324B4B5A8D6849774051593E32A06D7E893124
5F493059433744C75A458D466330507D82743A5553377F7C39717F2F35828856224B615F
315F836D72BC601886753C5A5B1F2D43573B5E6246293B5064587AB2753022886D57868A
6F3C3F71552C916E3D54691C558E684B7F713A3A52664E4044949B512E703D39
51433D806B474F4E2B6E2E59432D532C484613352F133436192729245633435F7A3D3551
4534274A3D37973219723F473F294563784628503B14365B5545454738327B662349671C
206A374F58298B70545B772D3799612F8EA37742453B2C4F6F7755343F4C3A7296844A6B
2163634054283F6645536E3D4D79555B71326456455B74724B8A4F628761674771503A64
5C3CAEA54A458F65468F744467964884717835508A688B8E4F827C4D8EA4838586556077
9F6F53636EA16E408F826DB18642878F4D61737147716A5AA0735A57414CB88B7BAD4148
867528602C1E219169438A476E433F526C6B3A4D705763743E758F6339667B492643537C
4A357C7553C47B415B743B5E5B5E4B5059305D967C4F3B68462B449A8353268B6F2D6E8F
74363E634E2B785D473D444463826E457E905E292A553E5735A1A08031553A39
54664D8D7C3D587237491E4E573C5D1F2B532A453407264122332A173C394A7689212639
533729322A35B7691D732E5571523244704E4A676024323B3842575755337C461B42692A
1E6D3154832F88665F6890723C972C1F708E533D715D32263E4649373F5B43608561487F
455340435F4B48755857769654804B5A8C4C7C4638435885487F464988564B34674E3D64
5E398570493B7F57315F6F434DB768898483375EAC70918A508F6B4B8D89677D9A77757F
A76B43558B936E54AE5234758D3C82982A41585F45643A3372724F6E343289816AB34034
967B2B7C563C209C6634927C7A7355717E6F335C88688276437696885085833723415D93
67357247389BA14C72731B455B5D434F56284E63574F3C6459673D99864D257A92604E8F
8C3D3F5B723E83755B4D4B6470836E49929C73333159575F3DBCA87B365D5240
5191607C7831496338683D594735764036542039551E274F0D263E1E411420527F462C35
1D373E2F1018665D1E67242C435334514219263E5C272D39141B333F4B2B8C6D26325E2F
21702E3166307E493633675D3F9C5C2EA2BE573050503D345545272C3C73547597743983
5B51554F4943484E3546678B5278704E8F616A49464C5573518F6F5C895C91555953448F
8D4E90754855A68C43787A2A56815D879F4D506C93697F793075804B8C9C3B48746F6D81
BC7E503B5F9027398AA84DACA63A5F965D44454056987E42826548876D40A86C5BBC6643
816A27686F441E968B1B48527D6D4F7676792D387B658C58307FA28758898255173F6495
683D527838899D5A6B7B3F52554B465656506165615F32637A7A45987F652D79A9745BA3
85455980753B7D6D354481737CA8691D6B7D79373B60652D2B70695B31673923
527B667C8E434A2E1C433A3D1E144135374B0F1934151F40131E2B1B4140395084553140
3B313F3D402077521C6E3D4F40292E616544233E5B2D284D49304049231D6A7B303B461C
1C7047325F2D8188534A763228A4623397B2674C5D4B313A5E472B3132724B6A9982447C
595969575448555A5541697349877552722D595A414C583F2E6162406A3B6B4B513D2348
654190862E255C57317792404FA93742768A3A4C73377AA45584733D76A09D5B7D7E3E5D
B2A47F497D96723A8DB85081C861558F5E4638673C5C7F3C7561466B6B399244429B6040
9C5627534B3F1D907A276569673E4C9D88763F3B81538E7A525E86654496A851272C4A7E
6D4667914E68AB59808D345957413D7C64545A6B74564F89897943918F2D297E986864A5
6D2E3F48582D8C9E5E406E2B669F7A5489825A2B547E6E5A2E797F561B70553A
546138438148536D363D26436D33591B2053272B4A0D0E422521311E414C3D57822F273F
4B3025273D1F825A296340434B62314A6B53243E47141C584D2F364B60296E8B3B47503E
1F72532B6A3F5D81554C615C459C763886BF6B515940333D8E8240333273316A929B4D7B
234D76505E1C2F4E523E5D5C3B5C6E54815B4D5438587B8151773E3D7C5B6D37504C4158
5F1F6D96652E5F6D485C7D506BC07E54679C43517260528B66868871629C965482726458
BA9E8D447C8C874078C0557ACC5C4C8B734B42854C4D853B699A546A653E917954B47E4C
A49037525F3D27909B4D7B6A703A2A564872433C7345515F5D5699714A5B55773F2F4969
32334B7D5989A43D506B4066704532495355509D8648294A5B4235B2AC5C2C5A82357097
9357404450194A64423F5845659F704DB8A25A29283A4454417F7355445C3C2F
50986E4D8C3743624342604B9036505F345F4A314C2A1243291F322F2E65436585342131
4B2A492C351E7F69345B4D3C4C78365576672B4B6D311750322B3A6E742D6564234C6450
1D683D2D7352516B45647C6B4C8F7917337F692C5F6B3D1F45454B2F4B8D35497E88327B
5D4A5330775F4353443C758E61723A458B6D4B452C3B4D563C6A57387E5D56475A704A57
76657C885F3661786A699F6B67C59A585FA948538872468F6E82968071A1A5607F68705B
CA8E6644ABB4783E6DA14A7AC35D4496514C6B774C41704D999F54596247AB864CA26F2D
838F364A4F301A80913396A18E704367558829548558695F6C5AAE9258706D89484D7599
8E3152643C92A57C637D234A736F51645E33336676692D5D7247276E814C284A9F6C4E6C
745B546A7B3B5270504D7E695AAA5C194D7C823A3B466B2825598F7246584131
5193654E964F3A54403F7345672D5376357147385D4E154729224B373156417FA556323D
462F5D2A3219677B376333335F76376E7B2322446941185F292221425C286C803B2F3D2B
1E6D562547314A82302F514D3B809E5057BF862D3B413E3C664D2B222D745969ACA94172
684F52476862434840304F7D5F63704488774058604C515B465F7E43916B6B5C568D5B71
8576798C74377A935F568E4659B0885666A25D8CBC7D3C8561AEB38165967B5CC183705B
C9A04A40789F863D6DA6656FCA673EA17C3D3F5B513B623B5F8B4E5C76438F8051B86748
73A33F5E775B1982B71D4955626E558499974D25656A7B652F428E997C76958D202A4172
7A426B874B58A45E74A6424E523F2E7A7B6B4268642C3770836A316689292D7299714F7E
6629343E5F4D477531256E5251BB8557829D80374D637156384E5A414049391B
516534305F48394619234A3A2D13264326751D0E39321546100D24212D330F34695A2A4E
3F1D46283A13453E194C4827312C2241654B21305B1B195961552E4B4B21518C5537434B
194F642C4E1E38833E3F594F235E9E5F40C78B555945423F6C6436293585585FA6BB506D
6C4A695A72744357783D5C85606F8E4C8171559179524E5E44759143824284624670345E
7865628D3B2E5E7766449142449C784B619245516E6C36745B627E69699B70537E91684F
CDC89D45A1B6713688BB7F7BD1702E94A6504D4E2F639761748B3481884D7B652D6E7C49
6DA9272D593F3E83BB4F676C7947444EA2A95E286F3C666C6E337E87273967765A2D4785
3B243F825861AF493D5B625159513A444B70456A6C5E234066773B6881543E3683685795
A26D45375C3344A6563F904F268E97615C9E7D2F262F575E444E5F562543512D
4F7F5D49744D375D463132285538232A124429193A170B2621172111162C29658B461929
512E3A1940265854494A5930336936545B7338395E621E4262593143774C4D9364304A66
1A5776284A3A2F9B3B374F5E60419F7540C99B6854435A43648F622E3779514C9AB85E5C
6E4479516543324B703B5F69384D744B805D3B84604F4E614C5E8A41895D755545954D54
685A60B56A36526C4143A16448A76F4473966E5A89663899837B79314E929D5FBCA03331
8CA65D4A95BD943057893355BC9C235B6F334163372558468AB1373E633887A060897C2C
5F935C375E4C174FAD4E6C80665E51434397683462705E6283388FAA934C8A87512F4C83
7B4947904E79BF8C5F574B4B806F645A5D613D729F76274266532B79A87C4539831E4373
8D7B4839565039725C3A9085569B745D2D847F3B2634767B634D545060454533
5E5F69527B59304860236D3E814B2346254E331C371F1331291A241F2542356385441C2F
4D2A52292E1F46674C445228315F3D56546D3B48625F204145432F51754F49804B3A6E6E
194F6C316462259453385483733DA35331969A70564F50264C7363302F6C5C4B8BB15644
8653564D535B3C4158386C81765D6136809045782A3E6D8F6E623D186E76692B2D957954
5315369E94435F7814255A603A97874540693A7AA7892464416B9E60477545347C888554
97C0663C778D9A4C65AA7C6ACEB22D4CA64E3D774725595154915F3E674650A16E999A5E
509E683D97781C4DCC3D678F897B5D5D4DA43C385D766669494489A1914E93A62333697F
8F4B447F344F9888586150394E585D585A623344736D2B3C66422562705C4A2E8F7C5757
8D7639354B603B86753B929E699468592C50854B292F788668455C4F603B4439
79738465958D3C4C5F227940794D2B643553462048471C2A3C172F3F2E4F2F618851262F
38205A2C3C16359956404E2450653A4957492B56735B213743251A27594A508459233E61
1C4B532144411D751E202D5F59339F5232A3A6604F91793D4A8A501A2563824FA6D44839
9D5E562B4085424741185480704E32195D8A427621263D625A565F2B5C69686B2F864755
7B663F8C4A255487623542612463846E47775B447B853552694E6B705471996EB18A9C5E
90C7834681608D577CC47A64AEB7457AB6563D70502E786F5F9256597D48419348938D58
4D912F275740183EC84D456D7B69566B518B7736435E6163712C50A99459978B52243A59
8C5A406F7758B9AB4C588A636F51675A557A535E887832376760387C74654F44A7905B5B
7766467788703E7F7554A8A96C9F5D525384885533347B8970487173703E4B31
967C7442656336582C154B2E561B2055295B4C0E3349151E47101A302C55293556632F32
3F26542C4D2528674235604F99882B3E7B6E39334B59223271682E31571632768C345448
11558D44533A186D4A2B356A2C269C804E9CC22228587C5845965E1B1B3A744598D0662B
86577843366D3A676C3A46663E4F714254652F766935444E213B8E505C56477D3D77563D
6A7D4BB74F3E4976635A5D863D6E7D3A61947F46868940729B636F6160809C6FB3845544
88B97F387D4E8B464495685667B4335187543B533B3D85695E983643603E346D3F756A31
407E513251321A33C93D53877D514A65557F5F3845635067833A55B69A64928242293F5E
7B693F697B4DB59B4E8586586D49457D948356657D6E28548F7C48746A712D236B785661
875C223342593488682A588141987F4E314B79503C40844E344C838F5E387730
C59384689158213D522F2F1741441D3A0B365D2D3D5A030A553425230E4D5C6D7B800C23
594C3C1360452F7027203962BFAC141A417351355331091A4D63323A522D295878355B67
16295F3D60461951673E458430108178486DBD693D535E373F8D955C3836522453A08933
78305A583B522343775A4C6648426B53558C4185713E657E61437D495A714F7231776340
5E4549A56746546B472D3D6F325CA471414A7A4188762F5173737A6B40508269A3707A6A
82CE8443624E8D6E43A67C5F82CE515FA9693F48612D7C795B8E6648654926787A946F32
4185653454432831CD514B8E7E6338557B887F3D3E614E94943C50A58055AB9E46323F55
87475D887B48B285328372507B5F2040495641788C7029205555215A7B5C5025804E395D
7D752E304B512386754665942F7262715776793127437D644A2C519336325F30
A5D6A8609BA542386B3D5942586D284C443E4E3031442315401F29351E4369596E7A381D
31455035573A26875C1F31394183542E40665944836938194E372B4D636736615D124582
283852173957255D2E272F506731803C1F3FA46D3F5B58323B6F525037497D4D9CBC7432
8D6261523E75424C704E5F7D6F4D491C528C46823C284054543C7D5257704B76336A7949
547848BD6648556E734B618C2B57B39A5956993E8985475D66526C8658558D678C73789F
CED8946367588279409F8166A9D67158A474464A6C36837A48907150595E248690947753
55A6893C6343283FA77B4D8F82884133536D553E517B3F7D533058A5702F5655523E5773
9845295E3E2B9C9C3C5F4F27595954525E6A294B63833D225E7249514F6F6D2779684447
706E2937455326674B3654A2638059646680743F2B3D5E59643D6C95743E5125
B9C98E5493A7493B5631574F4A5E2E644C3D4928204B2B16501F2F4D2C515F5392905526
3633513F63291B866D183320475B6D2A50511A284460481A4836272B455C35798A45337C
293B80543C66335751262B5B623F707F4B54CA803B618E5C4A7E59262F6288579EBC901F
75635B3627705158584D3B4B7D4D4E4F417D4D5F4E393D504D308771839B578958748E4A
538651C286544E636C4C74A33649AF9B654E99398F925254763A77945D4E93708C9A9768
A5C4964363B6B18C3E9C552F86C17E53AC434984712F73647DB67256654F46969592A13D
386874356F4A23189D653F6F727C673E6E637B3D3674499389473A83956D7E6B46293345
916C3A667F3E8EAD5670826B5D5761585C634E596B934D226377515F50685E318E885952
786333676E783D6D5C497EA863836F5657847D4B2944524F2E2468897239552A
AFC27D374F753B3E3F1F353D3F391546403C480D0F442D164F11122F2B4E361C3D625024
3F3432316130155D3E0E350A326D861A435A5D324666221957633E3E432B17567E502967
1725674F3C48164B744037663B1D588B663ECA76283E51564371503B283F7F568FB68D38
7466535B4B78644F6B8048568F59566C477D63656C4E474F6936837872AA608D675B7145
54764BA98B404362524564944254B8A064537A4BC0B66A437B3D91A1624A863B52908A60
89C99E3E6A99924B52A778575DCC3E368F7036333D3998653E6E5455945523715278A85D
466E7E3F5B4D3519987B4D6471856C43A0718F47347350808C5D41939A727C7359383449
997C536D80488BC1647E9E7D645D59648E88634B5D8A513F97854F695177321F6B745657
6F5825444E6E3F70622641862A75A277496D7319204A5B716C2E4CA04A338E3B
C8AA7052858929304F3F381E275E2D471E223C2016330B0F3E26202D153F5D506C772F18
3E533217574C1E7B43104B241741AE454157673B4466411139473A343B64245E85632387
3B2F8269364C2E3471433E63502F3A9C673DB8A7554D4D582F6454633A458B6886AEA14B
6E6B47615275584F6B894C5C86577572447B567494575254764489844C8040A56D517C3A
6F83529C9D2F5285804F5892232A666B4F555D3488A3703FA02A536E51629F6D5E9A3B1B
6DDBBFA5AEC79F794666908681D28E3A698D413A611F6287457F814A5569256C9189A27B
4D72A453625D4439A2A9586879AB532F6B6584434066328383623B8B8537879A84583543
852F567C834882AF3778896C577A453F515B475B8D9B3F1F50674961658F7C33848C5F60
98713339405F285A7A5E4AAB3D677A8268867E3718303F76712C57B238297E5A
82A57A4D79AD6032575032472B6F33453320411D191B17134630243328414C4B6D683A19
305929325C4A1B6F43214C77302C95525651633B42665A173639342F3D6A2D5679682880
3E2E7F72344C442D763746776E4C389F39369AB85C4C4B4D34849B6B394A774F76ACB862
60464A7150793B5F7A7B4B795D255D634C8F31426D3A5F7A571F603F4E743055374C8232
47492C80944939586A3B82AB753B9D5C3D4F7F4C7A96432B7F52727B493C738448A97C5C
4DA4BEB3C9D8D0A344669D8989D8A23B4F8C5C667D264E8145967B544E6134767A699676
378D9E575B4E191C758A2D56608962356E4E7B283A78546D591A399CA05C636F6B3C465F
956A3A5283377CC2706972504A68684E6B5C3B3E727D71347E814A484A807347959A3537
5B6C544D57704C5F705F59BA707D7A6E47716E581B2B2E7B322E7AA17C3A6B50
9F82734D69B77234585E28513578344F492648251F21281A424E1B373743504A63714122
2D4F253B50541B694F233F76503F82525852563D4059511A343D203867773551842F1B6E
45207745396D572E71253C62633736A6362170932F4769551C3D4C54325F843E57709837
53653839236B44344D3B2F5D6F5655463776703A622641516B345B6E3A634655754EA958
4E695B8978534B6E6B45589D713BB08C5D4F884AC2CB7F45734D88B071437A4434737664
59696A5C857994844162803A48B08940476D2F4156213D5D539581584C5E2F6787528A77
2E5C9330637A4B1A77A44359507944377C6C7E3E3559367E7960346C9A699BC471232C47
7680496981364F9B6575A2734A5E41467E6B6542777E3C274672695350703F2B587E543B
635F1E3E516C4B60652430745E756D7B4E659B7C2E28276571314C9573367360
C48071507BBC7135504E385531733F556D2A5F472D3D411A3F4318403F43643B5E96572F
2E3C2B4D3747136D853235671A3288485959453F697D5F243553231E354A22589E721E66
3727765E25332B2773293359661A2799503BAAAB23334F472C5651371F41695D7FA9A74A
436C424B58644B3F5786586083715D7A487C7F3B7640454D644764803B7E5B7D8E4BAB65
60706682634F52ACA15B59BC4420918F604A8D47618E663F6D466A8D6F416C8E4C9B7773
5446557AB9D2CA8E4971BEA261D4883B73A04E44742C3A893F7F9A5E5A7A36657335809C
54679C394D652F2F76A35F5D538C3E245F456D2E2B64546054663A6C832A49695C3D2542
7B353D6B815E5AB440375A6F45644945635B714777686D35476F706454557B3F778D533E
6B5A22313E4625694E3F3C8359737087587EA1742F1F2862833460AC8E437078
B44F6F3C3E603B30541D1D2B1F4F1C3B58256E381043411A3D430C1E2A3E711529535825
3E471827244C1840341720572C273F1F4B5E4C272C4E39162B67552D3E56313B8D872F72
611D6C8A35452D1C6E2F41535F4334B371459CAD3B3E4844315D5D49314D7D72609AB968
3C874D64625F7144768F635E816D788C427452377151444858324A84346B34546438772B
3D695A607C3E296B944F4F9A693A988B45548D716B9F4A3B7587709A5E4F8D933F986848
3076D2ABD0E4DC934752AB8F77D083354E82526B672452924C9495436D80568765366F92
2E518F6550614C1958B25C5F5C7B4E3C7C57743828757D7B5E7F476CA35F7E9476532741
6E72413E68415EAF874F6455465F505B885B7C4A75816332457A8B6A5C6E8A46739B4241
92884035424E415D6264468E5758A9A45E809266311D2373823461A8873E968A
C34E55624F9C562641572D231C4F463143103C52285A301525451B1C2E225C5C60778B39
253B233A1B51374B4A412066693A714F54546461385C60271E62643245625A4285722D71
6A1D60914464582275423956846B39AA7C4980C46C403A473752A18C564D7F5F4F95C685
35693165564B5A2C5177574F6F41497242537932837B35416D685988396559655439735D
475C39539F61377D73224598662F8D903B406C735F95673D5C8C67886D3F6E874CA27776
3E3ABFE1D4E4E1AF575D883546AA984F5B7C34345B34345A41749C4A45522C4A7C4F6A9E
5747B07842624D1D4DC3585DA4A1644B8A58774D326884586E6E4D509C886895A2702E48
5E75454F773E5DC194506165507A716D8254634C84AA5B313A725F5877AD954562AF442A
5B64602F49542E5BA8874B90523BA39E5F91A1652B436494823C5B9F5318707B
CC57556E53A78E344D772F4A3B615F2C5730463D1E4A301E1F36191E3335536668688D4C
2D372F3B214535466056274363387A5C524B4D5E377263201C564725617C5838766F2F52
701D49632047592671364969917B389A512248A7623637433127414E40366E5C4273B476
2A675B5B6E4A5F3C58886B4268845378525F923F783C335483625E6A3C6B58667A4B7F55
4958613D814A327F775645AA6C2A8FA876577C6F63B47E4073855C92854C788F5F938688
3A3F83E0DFE5E3AD5D559D867EBE7D525F8F5B417247316B3B5F9D485056365783566099
4243A35B407F591E3FCC5F2C5A76714D7B5B954C4478714F73424268A5925D969A38447F
9A915159772F51AE91485B5D3155665D7F5F5D29648C8944357C4A42497290605A995833
72595A2D5142433B7F874E8574488E795F99AD6D461B276F7742549E7E467274
B55653594E927E354846334D315C4A3F68303F3D1642331B183B132D4A3D5F464F837F4A
2C43264726311D326C621D34312F67514D5448393D8165371D492D18385F663878935F4F
831A3E99485568286138273C68402281755B61CD432B4C615C3D6F4841438C8E5CA8C988
2F796355364A6D385A6740334C6B485C3A467E397166323C605C5E79546B4965974C8555
60797C4193624197AB6F42B54E377D916A5E733A538C924D58513A958547824733799B71
494F78E4E6EAD9AB5950AC9588AE7A4B7EB96B42744F335F4D6696534B5A3D606E436798
393E9A45356E3E1933C95244685B4C31826990432D5D60497D372E3E725A47877E1F2246
5479475B602C3FBB8B4759844C56404578946B4A618A6434549476675F75722F63926437
6A61424E837F574E7F885389935291795791AF83581D2D6E4B2F6BAC8639703F
734F5158427978343D47283A2D493E25493A4154152B3B21172F171D38394F422F455C37
2C4430382B3225234B4019304C31644B3C5B65582853593B234F54304745392955795B3E
6F17439253545A29705E3E4B8840267C88626DCC7344433E4537845F312A4F8452ABBA8E
305647666640552D658D7430474F3984654A6220627B3D40522549744F723D3681416840
37596D3B9460255080633DA75C2B5E7E8A69698941779945478148768E49625E5C7F7B4B
3B2B6BDED8CE748E6070BEA395CB97387CB96E4A6851458E504EAD6A608B49578B2C60A7
6D449667366041202DD37A4D75536C4D9797A768285A4B4B92634447775C4996784D2F4A
588B528E784A51C0673E9380435A4731354D614C67764120225F68635D6278555293542F
634F321F47414F3F77463667853992703A5D8C612E304E8C69322D786B31907D
56585179557C8532456B4A4138496B325D2C36602E364E1B19492B243A193B6D62677B2F
1F3B58391B333B325F421A1C5B426729325B6B783662651E1A35452E5148763C5785593C
8D2F32823B5B5C2B5F48375C7E4E297470373585774B424328244D675D2B4B862E738297
3C57244B664548273E70712E4A623169654B6D3755763F3C566A576F537D7657833E538B
5E5A813D9955425B8B5C3FB38A32816E5C498A90446E864071924F6E964C6972795D6964
63373EB39BA25D752E4684A365BBA14F519973515F50488E5243A257457356519C5254B9
6D418C99456A4E3349CD9C4C66506E2D3F4D686925704034757447467F393E828064324B
5C5F2A4D67694ABC71395A7646523F583D6D8C6267725D49246080776C87A769529C4524
848E4E245A45483DA58B546678309C9A6E89AB7B2319295474352E895315616C
724E53765B78832F33553C2E3742712D5F43316F352D583516452B213A273A796262916D
2A38663F39324531736A3E224A2C78672F5174633377784D203021183D445D305B742322
6930317F2C255535574F19305B47287DA96E4BA388385E624C365E4746203E8E567D98AC
376F6E5F58517A65466F5D346F934E704C539357555D354D907A5F742F5D885863264A8A
5E56543DCC8F50586E3326846A27796E7C565088415E8B3B58974F676F47989988507E96
984A8BA894878AB172528C6E419FAC6C579D6160896032745C40A75C3C6C67519F7E5CB2
6951AFAA3E7E451F20BD904B7F639974675796823F7D89456E715662A288436F725B4278
A0A056567D6956BF9461738A5D6474733A3A6D557EAB83592452572A6F9AB4693EC24E19
4255662F63456644717F5B6C8D3898717490BD74531D384471423E9F5E23685D
7B4F4E594D819544385237475F47642F6C4D28562325492C19401A21472F3D5C3E739B7C
2A342F325031291340773E2529124B4928547131223C435B2D3C563E4138311B6C9A7727
803D2B7E702E723953732E3F75381657857B53A8862F41665E41714F4524519C67A69FC5
315A7157523E615E41795632536D4E5D43357050467C32345075535A66517A5475614F83
555A511F928555566A3F3096652C86AC895D609B5FA0BB585184596E8E5565907D46A9A3
685D7D92343961885F5278804A96936A6CA7742A565B296143659F6B42723B5D927752B1
602D638D3AA38C3B20C8AC32666E8F6E826E93401F4C894A8F471E3661823F829C321F46
67926E5A7532378791676E68283152694130544D4D5B61572151686770549A644F9E7247
5458763A827C8B5389865B90A349AA623667A98C6B4567676C3E339373417550
5A475863374E603840401D294A4132124051326A20154E2E1A541E10312944551F497365
2D4D513E3E39562451613943743C6E65306C917C504C4C481C31423B4141643649718027
802F2D997B2D6331608F49405F5C355E84674FBA986234474C4A7247522443826A7FADBC
5C60664F6352824D416A8449607361597A4D796546854B404C6B60776C4F6F4193654C78
507C8A46C2644E6D94643E875C1B5B676E5465953B4F805C4A632970875E769C3E3A8F99
696087A16B4377A048608BAC4EA09A4144889A45702D3878603994414D9B843B92343B91
BC4C728B175A924324C5AF4D5E4F654B6891986B29516B3C897B5A448B7E499BA3884140
5B91497F837A569F76538D9A5F4A4343495B6F64506A604A397B94786E548C553AA9864F
54563C20605E7D4B9D793F6B9D46BF897C70A56C351E2C3C5C4124887A327687
663F446D466D6F1D2C41382B293D5422393C2A6E361D3D1F1745462235172654447E874F
17295B39342243234F5C21193D375A1E23486B7250394E41202B3B303136643C4D7A8A2D
6C4C18556F2C522A4B8A433E61663D4F7B4944ACAF6042433F405150502A4679515C99BD
6058525B784270396181734D626B405C6A5073384F7B4B4B4C3627476639782E4E523F71
324D6745BA91393F585C47909355839E6F58ADBD5E68A44353AF69715F3B85A0883E7D94
1F2646A38B52769A46355287558E925A47898E43614D1D436D3DA162305D733B90453975
763F619D406C673648BBA25D6568663239586C562F4A5B42705B4A45875F39798D6C4343
5B974F555F735AAA76425B74594C513F363A62644D7E7D5B1C3E707B66529B994ABF8555
777C69364F697F41918E5C7A892978576981BE8176203A3D856329816A3E6875
786D47744A677D40314C431D493C733339361D70521B512615404F243543295C576A794C
203269374326491F4C5E2A1D5049503523415D7151335E4A2D3038323E4A6A42467B7338
88731D54802769574084394271532C3A6D3140BE9E5A49493923488069394A723F334D72
54545A3C583F734F3C4956427C964955624A83633E81334F8A7C485F4942936A592B3E94
6063402386A07655733419548F5571C44D374E835E7EA142346D676C5C44538388516584
787090C1894A80987848638A4C87B372568A9A3E6665224B783A9683456C79338F745688
82337EB0426873282BAC84567A94795A3D6C6C5F3A5E6C507C6D575C9B8C4B7682584A4A
63A66E536D644FAEA0836A685B5D667B4848573C4B958D7B214260637894C58E48C26E2D
504E693C37715D2F657D6272A152A0546E8AC7987C2A3840867A3D797659696A
937E4B87506490663E534D275B458941405A2E746420534323314924493B3455546A803A
1C39653A532B4C19468C482B412B51472F4B6D4D2B35676A4531441D2C3B58393A734B23
565D1D43581A4552376D192F4A4D403A99543CA594434E82993F364747356D9D888AB1C2
3F4E8B57591D4C89535B401F4671725A3C21557D48842F2A4B684B5F64397064645C3799
605660599297745C82552C56893760CB8E5A78A45FA7C28A545F796778815F85835F528C
8F84B1BC7F3F8FAD7C4B82A03C849B6A508D9D4D7F6F28427C3FA783466F613B937D528D
8122559828767E3329A7714B4D5152433880645B2E3D57447553365586734071833D3059
79875E4E5C593B6B62695F693F3D435A445258593C5C706F1E3D61474149797B49A3683C
5A537F4E2F67503186705967A9599A594F7FAF9C82373838673B4C927F5D5B5F
5A545278313B574A3B551C1A3B3B57162E4E2E6635114944262F3C11202E2D59211E504B
214B3C17422A4C192B4736315A414236245F816657214B522A4978732E334A27478E923F
704A1C4B7C324D413A8C623D635C12274F7254C3C751254253403C54422533547E54B5C9
6F37725C684E5D7351558E7062737F4B8C485590499776474B704D6F8D4C795F7083469B
566583649081565BAA8D4868913542C37C3F80A05461856963A75E476E64719E644E71A4
8A8BC6C3882957974F32647F527373584999873B6B574375573C86683E74713482473969
8A4B488D2948652D2BB4716869625F474D835D632E425F4C74504D4B8072336B743A3F3D
518E604C605E43868A8049816D57484D4F5655604A657984233F657E5451838E5994794C
625C604026675835775546619C529E5760509093563033377D7137546A344650
6A413D714E435D232B4B5329232C6935243117584A1B45271730542C2614125747408041
093159313D0F4635446725173C5D521E0B2141636627633F131E41442C423D271D4A743E
646B2228693A4F591F705542656B4E215558328FB081485C4A3C405C59564C5E9154BEC5
7D426A51765E67544E6E916C69816D53904F506D3D91694D5A67384E7A4992545D653E98
514467528CB25F3F707D2658896540C8893F617F68737E4F3B826F4D612E3F76664B36A7
4A457E9C75446B968346658B5C77716F409E9C4055502538633383974E5C933771785671
9C564787544E773D27B3698C6B6D5B6559924F7C3F36726773665E55897E40666B5E5E3C
5383725067683D8F8A6742A07A644C505379736242647769395F9C7A544C758353918F59
6158633E3B88833D765226478255876D684A667450374A4C8060385E75333F65
6971487F574B7757243A5C29272453502B41112F5B2B5D311012312228371B4E4B40893B
13272A1E3524322E456E42263F633F523B3E52736F2E6F3422284C472F4A4F5C30678A48
3F91422A6F415D6E2A7F5D3A6869532054695285B7804A5A593A457896584E507830A0A7
7841714A54536D6C4551665366A270417B495C8A3F884442637C505A5C31747E57392E85
6D4957335397794E807C2D5F95543BB3956F688283826F6F497665546E86618472613994
9177BCBF87425B81864C68896B607579469AA3586371334F7A397CA15256832F6154506B
8C4F45946848703F31BCA993798267793F927D9E5438705778805F54A28F5E8A817C5E3B
4A835E466D754E8B7F63449D79585046365260543F70896C30345C72544E72603257864E
61654F231C5B63286A4A345677507E92654F6B6B291D2C427560395284364284
70914D995B4B7D7A35487634464680622B522D3F77345245362A392D2F3E25675E3F8F5C
293F432D37413445328962293A6440755D425A6D5B318362462F574A2C4E535E306D7B29
288C4D235E32566D2D824931657433276986705CCA7E3A759B5639656151577EA24DC0A8
7D3177664E2B4AA259505B313C66844E5122438B41855B3D48686459775276975175476D
704C66794E945A4A876F2D6598423295916F6C7B8C8E9D8D4A877B496B8D5D758577439C
8A6DAFC2834B59A08F5063825A81AF9A468A9D56707A3A4A743A79954D596C3C796E516B
893347855E406F2A1F6B537874A572522753427A3E325B3A46564F61A6742B525E4B4135
4E796B405878458D967E45825D4F54635658566B415B6575383C63514167857548387E2D
55655B43205F6D1D5D6A464F6E2F7A776754827A4B272C3A7C7833447F2F3376
6E8751B9694F817537594F2F474276512B5420336D194944302F321D2B422B6D572B8974
373F4A2B3A472F351D71682445482F58394B6F70523B735447376E5E414A45372A737945
289042317F623C692473733454843A204171754DB36A3D4755514B725C3136499157C4AD
9C4A726B4F5F5F88554C71814B6E7E476C594C8644647E454A5C68545C44728B4C6F4278
703C677C449451487A5F355B8B3E2D90935D71855572979147687A5A918454787C6350A0
9071A7CC7C3E63948A4A628B534E7E7F37709750616A353A6C4E8E88455B712E52825567
7E4B356A653A7545236F518E4D877B6045654A8A353F6E6556664E6AB87D3A6F5D473746
5A847949596E26567D6B3A787750494A565E58784453738745397C7D6443666642277861
606D6C342B7B925254643346805C838556609B94772A2B39623230688949306C
5A55487B543C5F402034432124206B441D4D1D2760213A34152639251C2F163F5129666C
112B3B272A21203B2171531921452D5A403A42686436844D3D1F465942534E5827577B47
296D461A6365357532677E3E557D642A477463408F7051514E544379615247559C5DC5B6
9D51705D5A6D5A6251596B79477463456C67506B444D774A4C505B607149727C46833D73
674974823C986447827C4E508E5830798E7F5F6A8977938A3D4F703D616C4E7E5D48569E
855991C5844855818D4C68937C5D907A437ABA6E54563532633D668F404E72424F8B5E68
9A78418770398A682D7E66A85D7F67883A545A94652E7868708F4C458F8A4B85867C613F
3A66754F91944957906942919052463B2F626D71474C774F2F3D817E57445B643B1E686D
82A36F2B19456A45485C3B456C567EA4705592944F424D477A731E32A35A2F8E
5D854C8E6F466F89333C6C392A3255572753291F5533374A2B2128271F4335474937616D
1F2B2A1B1F391B37225B712A324B2F67754F557B7D48396250254F4642574B643457734A
3449572070762F873D5D7D3757805B28466A5133737A5C524B543E82A0564B5A8B50AABD
A44F7051596754714A52646D40686D4572604C693F6B844B524F4E445C46538C48885058
853E5A7E3F99733B526A4D37797D3E729A837D7F8763A5893752814B718A5889848E4DAA
785B82C98E565B777538649F845EAB813356A06F4C6D3948865767873B678640497B355B
7F6F489E944261442E59699158766F7F2D2D346E612C675241524F42977335495361574B
3D6A7134456E4A53AC6A2F6C70597659434755674564A18253235F734B647D99543B6B5A
4E5A664B23467B304D7A4C497939577E71617F8F4031447C8B6A1E36BA492D87
56954C9F704972963C3D693A2B47626E32523C245A34343C31312E2A2035304B5039536F
292B2F2B22461F4D295479322B4D325465455172724124774722454D40515765325B794C
37385F1E7081307B44438346669D6A3246614D31548859524840335F7557455E7738839B
9D4A65665A6B4B77564D698536607C4C6A7043695D438E4B474C82505F3B58984D6F3859
9544558144957650558C593D8A773F6D883E5358755BC9A4405C8A4E7D62468987884EBC
5D3C55B98E536C918A556BA768618F865999CB7165823E2870516890494860315A8A595B
8C45266075396D5020564A9A45848370473A507B4E4D76664E5A54439F814C6567603E4C
4978894958743B5F977E4F80643D4A5F634E58632C3A708363265E7B60415082594B6480
53686C4832466B573A7362456F666E95796C7D87813C27346761233BC0843778
61A24B9754466D7C3A394F323E4F57723947411C5F472C482F2D3F2B2027254657395487
24232F26295021522C57AA48244E2E5867465B6B533C3C7367343E5C2B3F666A32618836
39445E1E615F34814E377730428A6E343E5C845943836153455445676758455E776497B0
A14C567B6066526F534C5D8C3A588C586770406D654190514959764A5742588B4A724F3C
6B42615F3683734847932B2D6373446F9C806B655B316F7F606A893D727153706D6957C4
806270CAA03346596A265086884F8B6C33517D6F4791333D863F72905B4C6032388B5E5C
955E325F78439369226152AA4762616B4337628154315262476C32306877455264651E39
6170864B4C77563BAA9E4E78915A5A52644D5C5C394A6F876F255B867E4D5095615A6D87
5674623C363E81453E73593A7156759C726BBDAD94412E466D5B2F68C7753077
5685479D6C47748835394D2D243C3E53213F46206335305B2F27502E173C354951203063
3C1F302D204A1A42162C5A35204433566140576C6D6B315267303455443E496637697C41
302F4F1C5E90337B5D2E88534A78763E4B6F81513C88625748594C63616E4C6D8A6D91C0
B442578053642F7B48526F5A3A5A785681593F805B4584453B4751466853578334946730
60376985448760354C7F4B32655B2B4092506D817F4FA08D368BAD3F4B5F4196653F3FAF
4F15329F99615A608F4B4789884B947C437DA9844F8F2F206A594B874B75985643853A5A
927A415E892E5462256355A75C6C6E8257476D8181465D7A4F8E6324739943486761584A
3F5C8E5351816B519F6C41787F546E45554B785B3F476D826E2C6882765852845B66878A
4E6D742E345BA3543A672B2A746A6B9A7A468395693D5985873214388C7B2A83
6393466E685AA79F222D5B4E2F413C73473C492161451C5B2F20524F265322365D3A376E
2C19453A2B38124B313364291D4450583D2D576C7C803C59601D415F4C49444523657746
3A2C2F155183306C4F3091564A657434345774494085515D596044798D6A2E4B74506BAE
A154475732604F86423A56613D4F6E2C696F418C31327D594C5C4B354B494E9B5B755B46
8F504D78458C97522B6C3B267C9B5D3CA72B4A5275459B7835508F4D59702752607E4ACB
6B4262B49B7058698E56448B8E53A19D64788A90538A51185F5747823A38583635604948
76612C529A41644E2A6283AA63838893322947407C414D3F316C692145A42F4179786247
495F8834618D7044A7732F5C7F5084671D30545F39477268282F576D55534A85273A6963
4A82672B31337F3C3E805C2A7E405EA09974AFA155312343736A1F3EB45D1893
5C884D754536505C322643363C20286B443F3C0C5B3C1564372A5C462054324155404075
281826322D461F423F37784B213D4B3F4931456082784962763B34544B4B666C36557134
3C534E173E781E3F542474394F84684236755F2F3F745963514B26434E55394A6D595495
974E5080565D41917642626E3E5E97655B5F3CA67B2D672C3F5E8347533D3A7A60694E2B
5F53525D34486D5A3082441B4D8B433479726C52994F968F5A5DA65D58856D83768C4CBD
A7906DC3A0535685945D477756407E99696F76634DB2631D636747915B4D5732387E604B
78632159903F6D48183B4E9D354F6C81534E9B75AC537E554B775E2E4997522E4A5A2B32
6580885A4C718541BA6A50457E619562603D5757205695765C284C472E617CB96970803A
355F71644C3C902139845D2D7D504D849564BDAA894C2F4E6F7D2F43AE722A6B
547E4B8B7F499689413349291B322B5B16203C185A2F16481D29693D1C3C2B355F251F43
331B31371542243E221E455E5C593A2655364B6E5A362736573B34572420344D31648E46
223B401A50862C28392A80572F54721F2E6D834C498B294672713E5550462147827C47A8
BB4B317A53502D6A693C5B502131806E553E216E7A2F6F5636406D444C54469E72707C42
6369557B665E624D3F8258315C97423F9B896652884C9EBC8749A24C62716B7A6F473593
9A7969C2A23932627239458A8962789D5D648A803D68511F524F5C9369495A26206B7242
787134469A2D627C355166A65B69666A3752BA8BBA414A8456A558234F715F3962614445
444E72386678553193AB5E44672F486D763F655C182E5C6D5D274C4F213E429363728571
3556543C3A4AA0302C5651398C5D6892895799945D3E6BABA1683467AD833377
525D325B7646A08E18204D4127252A6E4F2A39144E43214529175C5235511E27655C3151
3819534A38441D504421413F2A4967324D23486B646D4339751527605831403C0F2B625B
2345370E2F6737353A155155315C6E3423476F4A59914147525643606A47282648753A90
BC6D41755389688F593C6760564272635670489A8F408175424157385A64499F427B8E44
592F4278605468411A646F455689222179785B71743467796E3B8E445067676E5C64438D
AA905EBBA4794E808D384378937078B4395892954E8C632E666C38906F626F5E2B72563F
898F5949A6355471294E70A2635976845971D673B0494C8A518771334B7F69385D6A5454
6B8DA0434B596B30858648536B46526047347C5E27485F5E422E565427514FA4514C7C65
42676B45343472393E7C3D2576453C888B68B4B7654023425658283D955F1C6E
566B4E577F3A8A982C213D46261A1B715C263A083461282A4911364B3C5D35254B442A3F
390C2138373811345023523016215B29491A34456E7653315429203B50294C502F3A6352
294F631A307730374226675D2D706E532F5A6436579A6D5967472D5F8B694439485D2C7B
8F79425B176C6F8C581954745D4474354D854681552763683E46442E395D398B4C5B6E3B
756045696751A356336E562B46A06823836C4670885C73944D4596725F714B6B91835780
A94552C2AF805673936C396C94749ABE7F4C7C723D805D185A6D3B9D413C64662F78482F
6F74274FAB4D516B2A415395466F98A46363C56E9D5A769F5F835F3A6CA9883A6F6B272D
424763584A4D723FB7C64C4B725660655E355A572F5C9F811E2D595D305C82AD66577F53
314E52423C2F88262B60573193533D69764DAFAD77593B537478363F997A3179
58786E58993D85B8703651612B473B7B5A22412D4575291B241350483F49253C62552730
3129373938342A333E296F683B475D2F5A4C4654795D482C6D502A3C301D516A43415D35
215278252B70161E512E63431B617E5435767B3754A85B457A5E354C613B3B3345663DB1
A0783C835E7561848B424F7061426B5C415C495E93426D71464F7260564E248887665131
79744E662340A8793D787715388C6B327E8748587464629E5F378D7462724E4171885F7A
B1483DAFBA7E52A2AB823D648B5571AA906099805456842A535C4CAC8A6D7F4323755C4A
A38A413CB14164753D3C4DA34A3E4D424D4BB275983E4B684B8D5625274D6C396E5C3441
43487F6654516B2851865F537E3F596169415757332247574727405B172F3B8668508178
3F50646054336C3A2D6265358770537B8655BBC89067405A7379363E91863B76
4E5C5A506F285985522F4729234434712B26513F31571723351B3E4B1B4346416A3F1C2D
675937412B3D532F2E173E5D3242340E4C414062733F41175049283E49293E4845427262
212C5D23287C311F3F2760772C5D78212056735364C3272F697E445A933B2A216B8B487B
AE83206568695F56834A5372304A888357745869A95271453C55705D51462B638A5A6A4A
84684D758149A253417F835137997530446A6699A26562B6813C9A49526D795183574350
90542D7C916E418A94705093B07567A5733B5F865F5666357278378F4D526F792E6E682F
5D7C4E3AAC4B3B64333A4AA36C4C61535766B583994458815C81654A28517C467E61364A
41478C685154794870AA4B47885F434859465D4C403B565F3121375B28494B93794C877F
4A50684E48346E513A5F553599824D7E823666A38F5F415A74482E66AD933A64
795C45517D494792322D574E283E306A35163737234D22102A173E5436353F24634C1F22
6844494B3532292E3244645A3147613E532B4C817F7B681C653421584F3654451D2C6E6D
2F325B11247A4F2569153773325273522C476A5E68CC774A5B4B514BA1402C18366D3971
B0A0416E6D7081708448585E362C5C74566A2C569B51745B3745583C555446728C54804A
93765A85614FB87D3572822B2A807638406244526A5B43984A4AA1645A51574C8F784F4C
B06862B9B87E4C8FA861294A98665CA39B4A6B987553682F3155308F965C6473364F6A48
71925D3A9F5F4C74483746B574556F667E4EA0799E58529E7985685D2E5786498D67414F
3C40736E5A5B70457AB9534E7D6D4D525C3D5C523437585B35243773284552877748A287
48516545385BAF65316033206F66477E5F5F8CA6815C41596B6E32347A7A3962
6E5D6A3E6F5D4CB3391D2E46342513625C22402016543C103A0F1D54512F5F133B5D3931
7A3028404A352B314D5077631627553D4F252A4D66684821663619414C314E4A39334E5E
2D2A6523196449263C263C78335A7C6F3A618E5363C389637E565A52BD5D492B56854676
9380377B5D5B7260A04348645451946A415D435D9D3F6453447484624A4C2151894B6031
5C76536233357F5E44708E34256C7F403B70736F7B7B47A962407D5B57567C4E8A7F5D4A
979E81C4BF955388929A3D56AE7055979F4D619D7B4F6D393B5B3081A0666079374C6C4C
659257339E62456736354CB86754705E70579EA9B76B499A777B846C345486598B745759
4036667C565F6D3E74C3745499715D53664791945338576831304E8C554A6471512F687B
445C793B1E2E70402C8D45367E593C9A857089A76C414D959B82373F897C3671
6C5B8E46868246B78940455737403A61612B4D2A18643D17412C205741264525576A4438
624C33425639372047368F8E3A3750294E4D446A7B422D236D4F2B3F3E2752605C416662
232C814125744B3035344B7B304C6C45203F4E584997503D84814840A244341D2E63528B
9C91284D785E7842825B44573B345374495561448258645A3B3D4D5E4A555165944B724D
647B5F795646A055476191362C63823E39797C7A768C49A67C4985755D698B5E9F827446
979276BAC48C508EAA9B4154AF696DC0AF4E5A8D7758863932673C7FA4676D703452634A
5897513FB17D3A7835275BAD6A509C9E83344858855963A93C5F876E395B8038817F555C
403D5F40417C763B6EC94B487C6B45545F3750515834777B211D2B6D4B407C9B8B488B7A
294E705D4D3A742F1D6E6740775B2B5D5E70919EA14A25475A7035427B7C395E
645185457F4C38A4843C48452C45546449235245264E2F152F241A4C351A3D2753693A3E
735B3558462D492D5D1C536A3A3E5841543D46978D5638236C402A5B5444494D493B7365
3A1E722D164762303D2730673B445E6D35465C7C4AC88B536A456B4998615E3133776480
9DA64E5A8E667852A76D3F58733E5C8B4A59804998676588575054544A47486188436D50
6C87555D4E4C9A634B3E9A423172924352827E6A717855C6A84C967756A9A2679F74424A
8A8D6AB4D15528678D9A3C49A44A3770A04F5A86383A67382A424086AD67806B28557959
50AF422D8B6F399E6C273EBB482E4C4C684A4967943C408E7F51832D4580814D5673402B
43688A694658592277BB76535F55326C7551604F41204A503A292D693621436E874A8685
4B4F6D55443EAF492D6650396B68436E5E5D6192A3682E57677A445082965664
72508D47928B3F7F7B365C6E443C436D582F4645273A49202F221A4C3A1E4C1949655043
9236254D54352D1741255A742D294751554B3D59605A45277746162E4344504F4F2E516A
4F2062541A515C31342E326C3E4366743C45557C49C0A4557C426C3A878F69353269616A
A09C5E4B8C598454B170445373415D89465A814495595E86524E4745516247589241915D
5F854172754D8E7E32459F794B68965136657962626F46889641775E34727E578D75653B
858C71AFCA9E668E9E934375C389212B8B578BB87F4F5D4E497F3C6BAB5F9F9D66518146
5AA571397C74257061333CBB8F6362657D6B466E90603E9F9C5F94643A4E825861725354
2E244B583D5D5A4C5AB86556597A394058426750503349602E1D25634B417970803D7C96
58597E5F303495633A673D31727F43645347224C79673A5D5E481C5188805761
684A793A707C44874121324E3A27303C592C39491F605527361D19624F244F2D3F595245
793C284B5D461C1E461D4F7C453F4F664266537B6D6D3522724524405150464C63406374
541D605F1D4C6F373E2E317A3C45758144424E644BBEA25682475E3787CA72423A634A39
91A9714478438F46875853736E415C6E34777F4081325A7E4A453F473753634384417A50
3B79474E58548782432FB274516D99714A788857749B5B8678376C7B4DA58466A29B803F
84602270B5925E8E8C7A29409B822745664B47A27D6059384D7A3B649938597357496D2B
45906A4068943B6A754035AE86646D5B9F8B517D9872409898599B7D42658E6667705853
37244A6B434F544069C25C4C53894C465C3F617A653C506E2C2545905F4189775A224A78
446288641C2F49875E633C313F4C3C6A5A3841737E602E575C6F44385A986258
7549944279945A7B8E4C3D584D2E595276403469373957233A371F6556304F4F5662423F
8941274D6247323F5130489D623D54593F70436C6355292B7C60353C6252478982497575
4C217070244B572B594F2F6F2F4A72864A3E4D392A808D5384484A2943A4704C4C69522F
7D946E44856080467B834C607A49788D495771488E78617F51517C73555A3D499C5D8637
398150583F2E7781583CB43B25588F714174473B50894E8067358484548B64367D7C8341
828E7C8DCA87638CB79E67429F8E405DA26857AA835263592E6E3851B56F7C6F48476949
579A64386B9133696F3D54B581545F567D2A5D91AD93528A885AAC8A4366843551676868
38204169407B67426EC06055878F4B476351534862324D721F131F536249847791425F82
434279401D2F83536F8F454764684A71594D2A495A462A5F7476453959A95C5C
6E499144807956989055374535446547672E3D8354655A0C3753387C3B0D3D51495F1920
82612A4661281E3D4C10236067314C232A5E3471A130281C457C3D57783122405D3C6388
4317546A283F5A2B4758407154395E8E525D87994CB9A03C8D8C8A433DD16C243B547546
B1BB65357958872F527A5C5B392E4E6F4D4D39295B684E6A332E425B4C5E3F4582607C60
478E4A525657717F42318E5C4F59765D38537A6A66A0676F8C48698C5B7C8F82877D583E
6D876F84BD89424870854E3C8C573A78A06C56A34E4F725E275D4B6DB76F8669435F7558
53944C2C798D36685B1E366E7D665C557B3E334C678C3D7C53528B873E667B5752498177
351D3B5F29445B535DBB764354744C4B5A6D6B56572E4B6C35191E57735F858DA84B558B
3D6694602929633D53A1876177893560856A425F94171741696C403650A81954
6C425D4395632E766A383A412D2E5044632D25512A5B5A18292D25706C36353F3A5D5C27
AE6926576C58343F544B406A4727545E3E651A3E6E66623B5A8D172B5F5D3B4B4F253E79
7B2753622B628041425C3C746D4B597D2A324E7B45AE912E4B44654835CC81503B406D49
9DBF83477E69845A7D9B5E4F7B4B547A57485D406370497F5C3D4953466F4D3D8E5D8571
4F9049686E5E638E312E896B586A795C4C718F716E865C9CB550707D33788776867A5A3E
6284806DC0998452A0A86F337197585AA76E55A0733140562251373DA2707A6B542F554E
57A465443D9237608668245A88725C5F92644C746F8D356D874A7C873E5D77664E4A797A
3B1B358650515E6E319167576B804B436C88675B674157753C1F21546A386895A645528F
1D304A5859296525213F7177789833475F57417C8E55243D4E713C3F99BA423C
6A4664314D725DBD60172C3A2E393A2E53402E4B10234527332E18313E322D2E162B4820
B38910203847252135463F7D401B2E542F672E455E6149304E84312F5A4C39484B31256F
5C243C682B337B3F4347327D78537A7B583A5456399FA45E6A4B5A4940B88F6C47416E39
86B49350733C81597D83578689545167435B623C5D4C4F734C3A47482E4B6341823F577A
437F553D566255AD522860695B4E89502D386F59766338707C385A82385873777A998F50
67844D4FA69A8442B5934B3B79A7645F9B564EB99844516820586542B88172776B345A48
45A8575B3F993A59755925557A5B5B86A673477B629A3B6574437D8B4264866545505256
40212D7B4851474C3597644F7B7743697A855C64641B3B60553D26727034514E7C52527E
63427F634B2E9658524746425E7E5256A48261649D1D13426E773A2744873940
82467F525F8B76CF7551283F5A2143347B5B2C4F44397231343538645744254945586E27
90912D334C501A2C3A583E9260353C6137714C646E65422E5288342D5853446A714C297B
57264B8B3F35902342683B824D3A55715D3855482358A65F634E4C252D939361453B773C
6A889549745373616B8348426D4C50644D3C503D5761417A573344524B4A644E877F6889
547F6D54565D48AE4D3C49764A3F916B37408C6287836A83883052955E5D73506AA98D51
5A87294C7C917D42C0A5421B40915F58A26E3976704353383F776637AB52548C6C3A653F
379E6F5F3A8426708C561E65861C3B5D85643F8E63A4297D77437C794985A87A3B636044
48545A7F53474C312A747F6A79711236495044706E38364F451523576E587B546F244373
5554A990341E573A2D705F5757812321687B6178AA38131931654A3357B71822
654382575B7A43A595602E4248285E3C7A23225D5037610E284C4369590B17614F62420B
679C5039572F204F332D1D7175353945175B43687E4D2418337E482B5A4A3A4947512A6E
632E3479412F9046344527788A4661766E3765724489C458768A817F52A78D7C4A429C65
A7B3A8479169745B64AF624F704963896F4D654C6C914F875B495A70554F593F98905468
508B6E504C33509D70495F921F2B6B84443F9F2240565080AF3156985E607436406B8A52
6B8D8C8EA4A0413995AC8127447B396AA592467B7153746E1435533A9B5130446A327444
2A677957619C2D3A6936287F8150455A6335256F779B2C4796408A76282E6A6037827143
3222305E396488511F776137667D375264383752412045764D241C435B2268A69E5B4973
1729597075337A2D24445D6569734D3A6E723C7D96572A24426E613258B76941
6D43685C537532645A51273A1F19382E571B1241452F5C1A1D2C306B5D3A1A4A44716329
45A43B3F6151284A2F60316854284760385824508170633E38B6231E5E5C528A42271B66
77413184211E87603D461B547240668947285E634078CF353946646540BDBD413A357159
9FBBB13353696C4A467657535138375762444B2B3D633D722F202E40465246246D71486B
406D6C4F6564397561496296702F6F6732498C5C5E4E2F699A543A933A5A7E81408B4A2D
47736D64B1B27D42AEBB733358945D40626E40957927345A1D45314C8A844D5B532F7478
43A38E382B614165963D185E8E2E4782A074316F568C1C8CB0616B501F48A86F21318457
424C5E912F384A462B95945C585C275B6B6C435A54151B547A5E1F46540E3457845D5669
4431734D2C30935251636A776A65644B93723784B0452724416B462F89BF7042
6A4770425A9B6276816C2A4E643B382458492F4B3625633E3833215372571C3F133C5835
3DAD19214659252817553567561D1D433D533F475561523D3DA12E1E3D51558E504F1C79
743E3472411F7968394B2B4874466771622E56663F69D374605E5B6A4294B5764D418163
7FBDB1566C60775A5F8254767E6D486E5C595B5B5362317864393C4F2B5B62426B423785
53835C3C5E6D469488273B8E6C3782804433884D6978506265583998544956573FA08A55
599B1E1F6AA99650C2CC201B53866742856E185384433A4838616E388A5B2DA19030703E
1177A2742A6211387C5D1869961D2F505D7D4B7657833D42945A705728275E7740449B47
1B1F3F675F3D503B1D64806E4B721C464464405F4221305A521F204955335D5C745D677C
79456851211F55515E6F6F5A497F62489E623571A15A341D386E432CA3B0793D
5B486C4B3862516572291A244C3A301A385A313E37175A38362B113244541C4142535F4B
369233353F5925492261446560332845445A5B606E605A4E4A9B442945675B684D612F8E
7E51328F57217B6633592F4E80497A9B793D64712341A97392868A34253E84AF826D9F4C
508AA04C7C4655525D8E3144634E598D443A4D485C7E295C4E31547950445B29748D506D
318A964E5C32368F90504CAA33296F7946327B41415450675D341C6E6D54783E1E556F51
49977B8AAAB0955FBECA8739387F7A4A8780426D8C6236571933623978682D526922573E
296F8B5F2C704540633416678D3F5271557B4377747C4B3E8F4D7178603B8A7434469077
4F273C7F663A4D5E31748176486C40684A613068513A35686629266B8452586C733A3A63
7042593B2329805459556E2618677645A371546AA66039214370611C4F91632E
554366644389516D8D7B333B553D4C3E445A323F3F2169293D3523415B44205E5C625D4B
36A046353E603464335C375B98492F3E245058527B50353132855B2B4E502F6D6D6A47AE
9633287D602B97622C583557632A417B553B7DA5594ED2483F495B6D5A3581D5977D6E70
84D0CE404C7065452F8663655D1B315B7249511932785375461E244A5F515B1F5B8A526F
3F5676475F3F2368774B538C3C2046803F2B5B8B626980755A653A877D56697D3C888264
448B7EADBAB2915AB9D6913F3A7F5154918D4D6C77622E63234554336D8E576565245D4F
4B8AAB6D3E925B497A301A6A994C506E446B34445F7841317C347E6D4F397A6929397E4C
36283F814B2E4B88487F937B50AF65764E5625767956284D6E26265A7E6949847A40475E
5A3D704E3435666848508C642B544A2C7880687EC57452161D52612990C45823
584C544A4086393641472950412137393C2012203A1D5F1A1C2F295DA7421B4D45636D25
1A575438615C22502E4E1F304C302A5447564065A87667493B98501C5B60434F45322C95
914F1F783822626C2C53192E6540587B5C2548545563D36B56585F6A55276DBBE0E27C93
77CBCB6F568F5A63569668596B763E768C4D515947773D5B6D4A3C54594D4A2E56945B87
63606A406B7E4C7F7C3A53736343549E472D527D68667E8265854A85844D695341817C61
4C918D84C1CA572F85C69740356F393975874D627259355C233F623D7E925A73702D6D73
5D8EAD4246A067477A4127A1C17E516E426D313E7B934F30742E72714E36797636618F46
3F25347630356E79477587503A81607D716C2E4C5338224E725221486632438F8D665786
872F415462425D7852376F763D656B41867B23296F7A52191F54291A7DA47B3B
534C5B4335554B555A3416314F41361C31472C2A320F45382A310A1A4259244A1D294A4C
1A644516374D3E46183D3C526921142A56443F2D545D5943329A571E344B3D583F532D80
7A5C21705D1E678232442E3C8A304756743F617E615FD5887170517F502653BCD9E38294
55C4CF6F4F96535856A5665569763E70884E52534970345A6A3E41525B57553F5A944D7B
694A8754838C4C8496485B858954549A493E7F856B664E86B9893F778749996144786729
2D71836DB8C47C4DA5D191532C6F8B486B9052768C33254E21355246859D577875255061
5689A56E36825F4C9645196EA757496F6B7F4A27577E3F40854E6B5D523D7C6F276D9D74
513247845035425E266D8A693D733C5243653F765F3821375C3F2A719855506D65162357
8846517A37299DB1774276492B5D9C4B9F7F51485E876916234265234286714D
515151783A725C557D6B3138474329402B62372A4929755122251C204761222B4A3F5A5F
184753233A563E3B4046555060473236603C6F5287736F3F3298602D463B3C5B3D7345C0
9078274C6E245C8D38454239791E447979455A754B44D48E6971534E2C22369DD0E3BA91
38A2CD7D4492517D53A969647D7C42698E4F6C6147783E62794B444D58416D694D9F4583
65398B3965814A83B2412A69804061985023734E557C5344617D446F7B296A6C44779671
3D9471448FBC9357ADD46B364582804D7B9B2B52755B4358363E5E32499F5D86853D4D6E
4777956A459D48418657269ACE69505D51744D437AA04924734690AA3E2D596E3475AF79
382127653A3C848D3F64805747AC7B6546631B5372592744692022588F7C4D7B76393D57
74364A7E1815507A6A39852610377F40988B7F4E5374501F325B7C3151AC8165
515C4E823B776D4C7D8B4747554A254A356A3E2C4A2E5B4513211E1E556427344B435956
1A366D2F4C57452C45495B49544C3C3A6B376C5C7F7E7243338F503245573249767C3DC9
99582A3F732A487134484E37781B4AA589485C66382CB97972744A8E6337256EB0DCC68A
386688643C843B764C994B3A6C733D588A426965427B4F457E5945516840475D41A76B82
583A8861585A3E6F9F554B506E2F427A5C35A17C4755666F648B2269974D726A27537367
52A7984B8AC4612981C9A35B2D56525A899268425E261026192D4D5798B0384B553E7D82
5548A15A4AB7843F7F1E1575A286655F5872190F1C827A2870163F665835606F2249966C
59262561451C2D61405A8C37297C76605759152F4A602F455A21173763584C897B90555D
6C356A944936709F574EA076414A6F309CA8785242903F1B2E46652B83BD892A
52515B9041736B457D944B485B51314F336735284F295D3917391F1D47572449573B4B51
183D7A314C575C354B474E4275693237493C755B74894421257066313E5727436A7537BA
B32C1939612D437C25515936732B2E6A69305E767D3ECD6E43775F8F534A3D78B9D28097
53A1C17E39905B77518E7C4656793A5997596C59407E66417335445C7B50553C3B86618C
54325E4881805A5CBC514F6E865B538D3B33898E546D6648B7B84B46803A778A4B6D6524
1D5589699AD0662B6AC3A466395B2B324E676B41586F2A3E3222482146986C51482A3481
6E49C55D1F888B39833C1872676062535A6059292B6F672D6B6F72886D3655846A62B68A
772A2F5D865A4A7E374D9066466769495C653A635250304A79432E437C4A53BDAE93526D
6E24378F3624818A492B6C64425167336C83784E46937A1C193356264B7D8357
50403F7D4379382A6265393E532035572E4C09124628543D1340371E44561F52582F4A46
0F46803D4C67462F38432C1C4E4C1F36432C4034698252372285421738575553554E208B
934F2B3A3F1C36843C493B2D593D3969744B42568242CC94476D3C8A61443567B2803067
4398C6802E88607E326C7956774B26467D5083362A54464C8D553D4F5238797547A22171
823863183D806756BE492D56736557A04123A07758917E403E7C663A85213F6853567977
406C926A94D29C6A96DEA3746C9F8B5D6A85627A867E404D3724643A56AA7E564C564A9D
754880624091763EA55D2195697964609C8D5B40428A7446998D848C6636598C6D5BB182
7B2E3D658758507E4A60B1905C6F7554957D586A684A21417764333F8E3F347173805386
905A448C3D378984432C79392D678A4C85B3743361A782334E6D8A4644878A55
4C55486A376C4B29584A19345A3327211B572D1B2E11404F1741151D404E2F5034113252
1E20641A2F4A45231E2F503E5D462123534D6A3A4C645729207C4B2A24575B65A5802F67
B659346D56332F7439414F30572E3A81885E40575D41C1A84C7B352A1A212D6F713E3460
3377B7A53779348B4E52492B546B30456F20455D3E5D4D2E675A47594D324C5A3DA96376
734370566A686262CB6C3A518436599E732B81544C616E575E885651936A5C76515CA287
417AA14159BA843BA9DF9A3F35597267999D57404F5743453B4A8D4053A6747D8950427C
2C2D64623B8C4C3E9B511F9E5D7A2F3956744B4F5A8B6127616491AF4B2B6189644CBF65
2B315B7D7855547D2342897C529B9F3D615C2555786B37385E3E2A65C3896B9EA350355F
874D7EA480648E4F5A3CBD83504370297A9576405CA45D111D3F76373B736825
4E494F8A4465613E87852F304541586428522B2C563B5166154D47213F4417586440324A
29429D373748422B3D2B41437D80272B312D633F504F5B2F1B4357382643192B617A403F
C7311C4C724A2A702A327E38631818456E58395A43288C9657842F2A235481C4753F305A
255F948C35776179514780454F753D518359574E407376407D3E4C627F53644A2C8B7A84
57386D5F7C6C36389E6A3B578B514485742FA27269778C688EA249338C65597E443B6C78
3B7BB48192D27A4E81C68D6D315D6D4C759B7E4C5243355E3924544665A35C445032698B
3E35495C3D956B43974535B7AD976B4C876E323142684E2C694155845A2C4F8E6A4DAB7A
5E2E3F50632659793D44BA6739588653A18C5D4F4E7731509B60332A664A388697875E87
8639458D8A5EBD37402B7478484A77526E8C7F457782821F23277752336F8868
4D523E79426C4D25354A38303F2623582F502A0E383D43570B344E254148352A48463C4B
15155C37384A3F1B342040354355262D505076474A4D715A35425439213D5F3C6879472C
B05F3146604E286A5E3D5E335F5C2E5D6C6651879847B9A4609E45371F3057AA8936496B
5D8DBB9834837187454B844A565C293E6A63594C33466C3C79683A4969515A7146A18788
93468B5E7C748D41B473325BA37E57925D3DA37C5179895471ACA4399F6F6E8061477B3C
28597B7594D17E618CD8806A5FA2B08477AE6E72947A48374E305B355296905D6E4A3684
6150686740A28049BA4A2F874C7D3948938C5B3E48784E308B716371453680BA43539C63
433E6C7D824B4A771947A8A0576371385D6D6A4B4C76332D4D5C3C2B706F5A8F77927A9E
81733FA78C68A2375831808B5448876C8B9D9251867D86282527795E2D77A388
526C4487468685484563592F5B633F71346B5D1A3E46416F2548562A4450533959493454
361F81352D4F553B3E225E5C6758283864556F42527D7E673E3D682A3B575C3B5E552A46
9B6E3B4E674C338C743D593A78433A5A582B2C4B7347BEA748747B80672761916C324C64
5F7ACAA437725D9A6040804D5F7D573E71646459515764368B814645522D52643EA04972
9B3C9046629D8B43A66B358CBA835AA6662E775E529FBA7554828E44AC41506E7047A678
48619E3B3FC19E5489DF725125478D6F6D9F5C3F4C5D493433527F4F3E98466E8E613396
2323606A347F5426795323A54C6F50416D485F3F537F662C4F72747556203C6A603D9A6A
3A1E383E694B4A733732A987565947388A78794E486A453E596E48256B7572B9789177B3
A4783CAC894E753B633D7D894E4C9173878B7841ACA28D29242A6423409A9D8A
4B5B4376364D6045474C212C695145501F5C4119221035541F3F311D4C5D4C2F3E142C61
361E9B3A1A3F563A3A2F5A5365531D305A4B653136585D49404C74461C2D52386E5F361F
5466383259371B4C6A3459324F3C1F6767663950562A8CA258684B6D5A428E956B324157
3B49AFA34268446774424F326A805840733D4A684F546B34716F4359785D615139A68F72
8F4C974E4E79745AC09E3A4C9F3F519C7B31765E3452605351755C41A97C5D742F237687
3D626E6F8FD3855586D58F88354F906E60A68A5C5366512C54267F584196896956553276
323F52553676623E894523A54C674C4084475233547B5D2E4759656A582F507960349372
4E274F47754B4569673CBA8566564B48A8736E3F4B624F375947332265706DA5745B2769
A3742FB7642866A2883D707B4E5B9D7281834D178D8F7D24272F5E3E1E58766F
536544A24A6D7F4262765326414E4C6639593D1C302E325621402F1E374E233266423457
22289E6D2E4061365E2D5A4C7EA33C346246876941705536262562503160351E57706529
4E81462B757224577A327F4B5F462262766B3F789B479BA65286A28D452B6A9D6D4E8C98
6479C59E336D7C626347834D577A3C5D8E71464A245585496B39283D6D5C534D22838F5B
7035736451763530787F384F9B6B477F87357966626B8C7544638B63A76164565B3B9569
3D596C58AAD475569ADE697B374D93766EB892615567532B5B31876849A29D708173337C
51594E733C7F7A44875C33A993A6774E864953336183552D3C4D527248335B7153247962
2B2A4F48612736625E45B27B5254694BAC6C5E35748763374C4A2C419CA0729E7872577F
7F412BA76117376A6E3675761E2F7D677FA67B4B9CA26832454E6E541D5BA639
534E416D40767A2735484A2F4B282B5432661D0B2E3D357A143B42324C7F3D1E49364370
36156C6829585622573251273E5D2D377D3B67464E8C756649335425182E4C354D553532
678D3122474729707D345F35625D29556F3D2340694185953A4B42696A33539C4B2C434F
5D7FC7AB4F63614C7145824A50755A416C723F4945486A4A556E3C3D5E58475149A19654
8159935B4C8473429F903449977F5372784FA7A46A62826A3B698F759B52595C67438D57
4065A682A2D3871970B0B19F3A4D692E7AC1A26252685252592F7F6F68B4A06D947C3A8D
5C5C557831969D42725F397D82AC8A53914B4D141B51933D346147B07537697E54255B5F
29294F425626417E5D42AB5B3D91B34E9E7D331848765A3D62642125616448869E9B7C85
89333697784935504F248294504A7019486F8F50A0CC8C201D24603A2C93B958
4B5943773346663B43361B1C334D35391B4A4B191E181A5C2739320F36554E1B4020235E
4B177054142D52303B235A4B51702D22533E5F4636616B644A343E3A1F2C31343C4C4620
2C69381C36501C487C3D553F486B2F4A576C3C487B437192517453737B44789B88505564
6A73C7A1566977436C4AA15B51746D61776F444942507247576C423C5152535C418A8C4C
8B5D8D6F52817F4992963A5BB3A44E7E613371914F5F85465BB2A8649164738D874C8657
2953AB9182D4AD567CD065834456815553939E61525A17253E1C6D4564ABB054A23E3C99
6C5A5471208A94406D6C2C7E3C7A5F52BD88763A4662833F355C35707A336A7D2E298E83
663F514353122E52873EB37E29668C4B859D6D2F4761191C3B59511F564F35534F77477B
B46F1B716A40469C71335B66385DA4694970661774B282272C25562715559C6B
576742AB515792424348712F4F65303D425E4E1F202129561E351D17324B461D3B2F355E
4A18615D23384C2A3F28674C4477413957326C5E52776D6B50343D4223302F3B32544D2A
2D6E2D29566221427431544546562F3B53643C47704280AA708A586E312A509477415B5C
5459AFBA6B5888527946875E596765596565526C4F5066357696424A4F47446951697A39
985E6C733392A5508A8B1B3790885085935278893A80A66C26547D48834556978E4B8E7C
4E47B5433EC7AB5782D1561E31749F7262A3836298804C2E5E1C345B3B849756BE53319C
7458748B43738C3893933A9450895329585F673E5B8487214D8256915523637555315087
462C59646F55404A33276E7B5D66782859696127657C622B4B51373187AE72794A634845
846E2073991F2FA193456270183277694E78564337B4621F342D577A1942AF7B
5D6043AF5A4E82438EA07F384D5B3954465A4C1F322B325B1E3D31234152482A633C3441
3D2163753236532D3C33654C4DA4623F5938877657807E645853626A282F4C453B8E6F36
267D2020417126396C2F78603E6C2950898342465E1C598E5E7D8C85774C58A56C4D8D87
6D5076935E4A7641854573317D8C6F4B57564177554A6C4C605F45576B63544721608B4A
6529667B3A653F2E7496533A92372A5675709DA12D4C53632C61674292926078321E467F
604AB03A65CF9F4687CE74752B3F75646EB08A56494B3F434527576F41A7473EB07C3BA4
31143B6F4076852752673395A1A7845C8A4D4D234A6992412B7934907B34466D2317447B
5A2C302C4C3661897040886B3880B3626E7457123466682D494E2A1843696B884E83854E
AC633485923B1D66633E7B603647716C5CA56A53319A7D214F5C79721C3DBF61
5853387A504A74322E576D2A48321A57414F13093B442D7D14345E3C3C5D3A184759403B
242673903E3158204635441D2A665731572F66644F80681A2E49627128485725204B5729
37874924374D264A7E2347533B6C2A36575F2F59885559AE24324567703D59BC5625323C
5F7CCDC836397A514C26654C475C39446D6750761D346860655B1D28405253592E427437
813F4E7730755C3E779A504F9E8645577058668F656D735E378298524E7975996D4C4884
655EBE769FDE942570C08D9038558E2E5CA5A388836D38546A1A234769BC794997573BAB
58576C7C1F789D4D67601D5D4F676653764A5D273B4A6E2D296424565A3C5D6E46252360
563C34344E0F265A7F4E907E24517664615C733A4F6466324B5B6825465E6D7A7A9A7042
B3481F45774A297F5C288291594A6B3A4472685D2E8D711E20254F661C3CB865
4D6B3D8B425999454A407C304B6E3F4C334B35191F1D165B2A2D400D306B512A39292D55
421A395E20406B3535205B403373251F514A71322C73715F3F334A1A14344D4C2750420E
1E4D442945291026663E4B361A5242324E4F24285F5A54B244545D5E48326EB68C53503F
5476BCAF4A367751672C56455A6F4730405245873D3F6F45586E4942463741643B4C6937
8C5A4C8F356D8350849A213B77A34A62864E5670487E7F571D48755E90685C9F9C584478
2347A88677E1A22D5E9274805F89B9756B87997FC0832D376F2525474F8391598860309E
7967888E21568E3C5D7A34775A6E4E4D906965355A5B7C28315A39623A37597365352765
62483834656D4A48783E97786564726C577F8C5E595C512C587C7A213F603B4B458E8D47
B6742B4997552D7C624A7C793F4B697A469761552A976D272D2F62832838B088
5C6C408447354E395447311F36583F39132243281F1B0F3241324B1019314A262F1E1930
51214C750E12393D2D123F5340803017353A60472A4E5E5B474F5B44282D4D5632503B1E
1A5D3823555D252C693452572866402C5366473E623B45AB73555B69311B4F9B834E5742
693E8DB1683F7D47A34F83224A7A7A40534E1B89755079314184554C4943355E56467C4E
8470469B505173457E9F47245B6E5C99A0656B6F4594B77F356071707E79455E89534F8B
613F963B35CFAB4C7BAC59363A69976F6C919A3DB1793A426535386C30818F666A6D2D92
64508B8A474B8E2E7A7F3E78626B61325A65704865619E1F49834C6F4121527C663D2670
3E2539597270524A531A94A965597E3A5C9B8F5B50546030486A79283D77886D4A838B4C
C3802E4CAB582A9972557F80594B796E4D9F4D2F409179362B3564673535A691
6E89459A6B4071395F7B5B2B2D4A3448553E72342549294A40335E4027495E31354A5037
4B14476E3126482D37314E3F3E90733439355B76436B7A201D4D746734325B462E6C713A
1F613A1C6482342D612B4B681D6E2E2B6D7B4635592A2B895F54667F816055A773527B6F
664954866B45823B7D48833D33556B4A7474447B6A55896A407F546E84754E6544579960
7A4D4D985E625A387DA1722B594D35567775656B496D6062314C63495F975E72552F368A
7B4AA98368D2B65E91A89796384FA38563848F55B27F41494F2F566A308F5232657D3B9A
603C6690604DA4486561376B706874595A58925B7766A4492D82536745354B7769442662
5243252C5C71514862275C825C557737436E795F5C548747505E66315A88775B538B4637
8F6E3D53AB592CA2925070834E42896F44A65644287C783959696E3B212F9382
606437615F395A211F435F3133330D3D5E384B1F375B33452D34876B28495E0C3679513B
4F185C97482D45152A4243121D596A36393746713A75752D1D2A3F4D2C3F7D2E17466748
257C4F15365B373F8B3E3D6227673E17396B41457E743A9D49384C5D7B5B66B95B363B3F
6B82A3C46A3A88536730625C4A684030495F508C483171694567363E4451475842477E5B
6E563E7E566361475C9F7527477A49698A6E657371948D913A4F5C7241A45D5D9E6D5995
8B4F9F9877ADAA6594B1A2A1415D9F805B849F5791513744703531652B723B4D427F3A8E
9070A7B16C53B85B747B4064857381636C55A064735BA14C2A7B6462473D527D66381F69
614A2B3566865747693061B688507E64507B7E556B75834555653924386A604A6F98763C
A457325890521C71563E838E5A3A885343B58F632C8179331D2B54672F167B5E
5379416C67438752436E7B3A2E59382645314D1D144927412C1E654C213A693825533F43
5B2A2D62332C5C3C3832514A2F5B411C3E524E592154585A41425C31182951593B464C18
144E491B314317236246404B214F59284A6A2A254C743B9F654D5A526A7082AD87674C46
8697CBC788418A5B68608B674D5D8E4955766891844980794C5D63534E66484F4A4D7569
66523F93455F6E5765B08123407E4C608A7183766091A399454A758452AE69619D70648E
5F54A7966F95AC4B5C86998A3F5694405C9597599D5E4340783A37632F75515C3D663689
634AA08A546BBB527478316996A18058795D943A6A72A25F25845C746F49537A6E48256E
72632D35678464696D2E66A8725AB052479C852E464B5C33637B7A3437824C4155897446
AE7C2E3876602B9C711B5F81613E9048317D714B4EB57F48212B6868434C8B82
636D44634F2F5947535B5A29376D4C2C2F235432182B0B2E4E255D1D112F544425391123
5631316A1E153C54361A3A5D3C66462A274E436C3D625D714C43605D3B2D57725D444734
1B5D43192D4027306F40435C285C511F4063543054713A9962555F6257344B8E76565347
808DD2D18D47845C716291794862955452645398874A7876554F6051515A425938527E63
7E5C3E9B5981856468B07138587F4A587E5894856A8BCC8D547EA79049855B8AA4725D8E
3D407E715A80A56E6A766058485D926A51886B529C6D2A3C7A39385C40864D4C364E3798
644693A4263F93406A6A2746725852436F628D405B558F432572353D5036557347314876
735334406B40384966476BB7533983563C8884616E867A3344593E305695894657844C2D
8172273B725E359D9B5D687A6D4C8D5F39826E41297263461F2C5E5236247C80
7B924F507335655C6A537F4636595B2542255B3618301E3240264D342D28564B26403934
513434715F2A416E3B3B407144564F3A2D4D3C80457D758350435D68452A55736747504F
1F595C27304F302F6E3D4B55335252273F625530564C2E74815666668B7760B776575A4A
56427AA18247814D7658704C5F8F88495259297D834A80503F53624B524F35535F53A83A
655C479032486B5A61C473233E753B6B814F767A4584CD83353658475AB652508B6D7EA6
81519A4F31619E814C716C4342688A6B60A475488649333050343E4E32546C4A40672764
5937A28F574EA9486963314486534139868278467452AB23458D53495820547B56452851
77202F496278575268194C788050A27040795B5663517B3D4B5D5C323170855279627D4E
9A83345A9380367B5C586C6D5F367E6944959E673585723E3B4D6B5637268881
8081555480376A647771A1674A606922472A583F193B39414E205F483D304C4D295C4E30
4736348E73283D6D343E39704961953E3559449A447F7F653F506F6E522952503B60925A
1F5458234867322C6437526234464C2644675A2E6B7C348E803A7592876F65CF6A466571
765CCDDB8C4996576A40556A465763477289648B75478B844C474947696E534E483E9066
65593C8375715A324BA398293C75484E8A407274506ACF86573D6B7A578E59717F455DA3
6B4E9E83884BAD8048808789495399867CC3A843786F453B563569963C707762556B3D78
744AB1AE7953C1536D7E4F4AAB6D744D665F86468761AE462155475A73394B594D253D79
6D422733556752848A2646744C3B90814C79728579538B425C8B743B2F603722455D734C
895219306A733896574F86AC91499A7C3A7E8B6736855D2F16205872352D8E65
687A3F4760356439394A714E3F5C393355264A281F4C3744591F6F4736456C361B473C32
481C1F5A562A45461F3835392038553340532C6131849150393342543B3969675F3B5345
23716E282B4E373A794D4659363D5C2F3E6E442A5474398AA1445A5C6C6A6DD585633F39
809BBFD59B38756875654979515D79485165578B883F78854F4C6E4447705E5063458F6A
5D844484856B585E46BD8A32486737467E449F854D42C77F5D65889659804B7C8C7759AE
8563B699753EB96D4E708F884A649E444D947B4A8F814D566934325938736D3064614181
6E2877723B499F504E703A6088878D65706A8D427B79A26B26624B415A4A4C59715C2D46
865D2837545C444C6C3D41757455994A308C777884576D204465512F2C55695760427258
795A332E6A7C3C8E563D587E7240935A2C6769463B8449491926688343299089
68655D476D2E5761585D8F5945855E232D25593614260D2B6220572F1926475020351925
564D1C5A4222476A3B243968463C622B284637774461647052566E693A2445706E3F633D
173F5A242E4D20275D5B534F203E5B343D5359256782468AB66B6B6A643355AA845B4A3C
8C93C6CE90437F7484734B8250638B5F5C6D80879F4A797B4F566B464F6C61707440925A
7568428139586F5A46BE5C2B3E6D364B803872814A37B1844E3C75844C8164AA9C7853A3
5748846D4540A2945487A96C417E9E8273C375338A5C312D50373E62388472405C3E2A4F
51279C9B593D9141566C3040864571615B6D85637B5C9E6B3255574A695F414F716B3843
9E662B354C896360836B4AB19D6B965434837D70745A895B415C4C33265D88727953845A
8E883C346B673594565E6C90753D84473B67786458B2664616246A4D314E9E90
8A72714E983940565093A0372D4D632C2F1C5549172C1D2D611D58281D2A495A24332E2B
465719476B2A286A33433679553D834B33574478516F65503F3646604F23444F39509056
23415728557534215D4955734E316939485F7B2D69572F6F95536D74826D63D16A386E80
984794BA893A723F7E734A66508689515869576F924A765C45515D456C743B4D5D47AA5A
5D42398A6F56524862C28B343470315A8A43837E3F48A48F6E37567549895665815B5CAF
8C539A62334A949165898F7A49628F7F73C571378C57303B503A7695345E764577533D3F
501FAFA96236844273763A3D8A497A4E6E8399647F5C9149468947486749597F67633445
893433576C7B53515D3F37A6875A9082647578846D638B59477C724331768D698B8D5338
8E8A3D3C72743BA4B066638E6C37877D4B68AC67417E524C272D5D5136267581
7968473F70373E5239466C4F2E51454265284D411D412821541A60352F2D41442756362B
3E371B617F311B662D5237604A50AA552B3F216250656D6A63404C734A1E416B7147656B
264268362C5B4B205F50395B5631662E40756C2C7C6B387E9D335D6E655F69D86C4E3038
7850BDC38C429B6A5B665A904B586A46657C5F6977387A864E544230526351534E357E67
5D4434776B65584F3A9E8732357B3D3D6F3F7E6B5F4463656339583D44836A7875313699
7E509D876A59AE574686997F4A71925866B5843E886E333E4F3730663765782B5A614750
69297F995F4198484466454F9E8F9D594F5E77687E6A91432A45453B5C3432436F552C49
721A1E3040673147593C37B9864785967579774B508285524674562F2D4F66498999845C
99773A46736B357A915265A75C277A8D4A447E8693BB6B3D1D2D5F593E2A8472
696F7D3F793D3B7C8279A6762C4D45223F22431B1A413E2D5F144B40224E6A5E233C3B25
485916336B2C164E253D2A44322F5B3B31533A643F75A55D5D484E52301D67797B47704E
1E558146314A2B2D756B4B5A403C7C343A614D288580386AAD3E5F544D6565D76F654842
776FCEC96F235B5C4D4C3C5C475D73414A564D4C94436C8D4F5466464D634B556646A67A
676752858C6A5F6B45AA6836347037467F406E58613E58766E36526444736775735740A2
684B9A8F6D609E8F838C9D855766969163C49B3E748F3F55683D3474416A7D3F55583339
683EBCBE583B745748773D2D67377E6C5E7E95607799A86A3642473C72463651823C2F5F
6426284249591D61814D299BAD376E82476D735C545A7C374F827C4628623B3A73A3492A
7F5D414E786C1C66826B4681461F71915C51B48162936155383E706335349587
70556E3A5B2D294F6B6D9459387480474B34656725350E1E662543402225315528492229
60621B3060231D63382D366F553E6B1F1C3933622E36526052424F661B17415A54387345
1D3B5F272645261A454B4159421D64283850562363893552C3676E595D566BCF8C6C4D47
7A61C1B3483579625A70597748598F6563726F46A04F6E8D4F56824F53755F587B4F9379
80624E818C67525A41AB5A3639753E456B388794724C387F703B585D498873858C763E9B
704E8A8D6677B38F848EAA8F5974A99060C687447D9C3A2E503A2D6E3C5C6E42525E433A
6E4593B4834788584E995430753B7548598196666B6C7E4D355B452E56363669936E454D
5C1728545B622F3F5D492485B241617046616D4F56838F533F60562B3150514A6FAA8554
8C5F33375D71367282765498542C5881312A84997DC7763F192A5E5728235E6E
995D923A735F2F703D5B7C351E384F372E154B5C233134256E2B384845383D6A2B3B3339
53641B28592015492F2421554E396F4E2F403A5F68526776683F596D592F3F776B405D3F
264162252745271D47533A5767277F3338596C326482383FC06784736D5D58C663615748
8149ABCA713C7A574C735C7945738A675C77714C994C667948616A4F69834A5B64379D50
624C418270556F5652BB81343D7030507842697B5842419152456A524B975F687A58469C
8B55917F4D5F8C7A617DB66B365F867859BB7331627F33324D3B49924A6476365A504435
583597A768408C543E6F5D2F763C7B6B4470977877809275453D5D4866622B4E8D815155
6D42252E43682B4F6C623184C6494D89686D6F6149557B5A446B8A551D433F2257A88151
8D6846385B89405B894F3573542B5D7A5234AA916FCB6E5D1D215B5130277C74
8F5493387675378E426B966F33476538433451562629222566253343343D405B2E3B4552
52561929523D17503C3C375F5A48906B3C4848577450645D3D345F705229455D3D3C6B5F
3737763E356257204462406E7A28742B35737B3467843631A44A6E8066668AD459466475
915FB5B981347C604655507B3448614561865E376F3E688A4E4C4E355B67495361356874
7A494B6D8154564738885F35306C2F406E5175797B49226D733453564380777881603689
7F538A75667FB971515BB4674958977B51BD8D4B7D9C432F4B403070475A7B2F405D4733
59469EA06242835C406F523E9567875A3766725D648E8C83473554285C6E33518A4E3B4A
676F2E354868243C4F673F589D423E704867746F5C6A835D5D8F9655223E392B41BD8260
7C6D3E4B7B98434C8C683F90713857715442988668CA6F5E2127533E312E7187
7D546E3A614E3D9433488359354F4D27593147392338201C63203041294348531B443E3C
634E163E653F1E42263A2E4B2E32554A3450443A5F42805B50365362261F506651354353
2E4A79311C3A4D275C74324B542D763C2F59632962814C25895B6768556692DD89624947
7059CEC67531707C455753773B546B60487071368F4D5785544D5C484C6C4B4D64436E7A
7B6D72758B5D555E3C8C73362D80404771646E7B7B52244B5E355764408B7A758D793888
7D558F7D7784C28A684ACA7F6259958B52DBA84F7CA75A35714A2A7A556B71313D553941
504C9DB04E3B8C753F803D246B2D5C593F7974626287776836586F3F6C543550868A6457
63553A3D417F4C4452824457B3683D8F677A916A656E733A37558D5A213B373636C09268
6E89563D5A7B373D8E7C42915B33528B8B546F7C5BCDAA6E272C5D5D5028767E
984C7F406A7B4BA447547C5C2F3B472A442544401B2B231C7034253B1936445D2F402B21
47491E26502D223C3C221B4F4C415E34283C4F3956406550593652472C2537614C295051
312C602E1F3A331C355728504F1A5F3231547634588453217D6F6D53564190D68F64544C
807ACFC36C3C737D525B5885455676664E767B36865A5C76554E59474F806850774E6B71
8279737977586A543E946C32438942526E4F79AC75482F4353305C5B3D91665A7F6F3691
5A559A777184C67E4B48D79868609B6141736C4195B04D1F3E3C376B41696F31446B2536
4B3A8B8F2D2D8156428D4527722D5A4152928A766F95817A3E376F477059395F7E79545E
586340586586513D42631B5FA86B3B865563707964736E633A5A8D541F3E5D483BB89562
5C915941628A55338E7B4C87653B629C9D5BB5785BC2B173282A6E593A398A82
B74E973C65A46EA5475D8D87403C4F323E384260292C261E65301D3F32343666313E3B3E
434F2220413E2539492823595F495C4B38445E43694A725D68335D6848363C6E3E2F6C53
282A6D363250291C355A255057216625315C772D518E5F2A75815E72715A88D581516C86
9A5BC6C98B447364617F657F46617A5D4D7E5E3D8E595F89485364424D764E5179448E4E
5F655A7A7053513548A97F3A2B7028656E62556B7148385A513D656C429A80677E643E93
7F578B5966A9C97B3B45B8A66282C26B4297833C7989432043434F824856822250773929
45217C884C3A7A49578B4D21772F5529427A666B5F9C877D3E36664B6C66414E727F5E52
595F2B31406F3C2A3B5E28528B54406A416B79786A717E6B394E88531F3E593837AC7D56
758D553E5E7F4A37B190507F5B2A78B38C66C4633AB0915E2A2F633311377B7C
9B4C77375E8688B43C6F9C8A3B3F56354647416431272F1F66321A463B3E3C60133F4F3E
354E1F2D564B2D35472F2E4F4D5571673E434742873E84704C25465B3F27456967374B51
21307B511B374E2A31592461662066382958722E498F4922566A505F547185D989523E4F
7F54ADBD9840726D4A5F5D955751593757998C457D40678D594F57336A895A4A60319F5D
5958457B8C585342348E773D2D843A3E6D6359737A5B55696E2A564A388178676B713081
83517C686271AB774D2B9FA24560A07A479E8F649696642441502D7252508A2E45813F2E
4B4C7D8C423D74693E6840236E325640496A4F5A5CAD748030364F346968564A8290734E
4B65403642775729427A345E947C516B776D7D855E828C7437478C52214E5C3B34947717
4E996C3D596E332679794A76511455928174C58045AF9D5E2733544237296F88
974B713F575769A943515D4D3564632045443D4023304B2484471035233F617039273425
4F4E241A56473B4A372E2D594A47514C4766653D6230727167364054342B376765324B64
38207457203046274C693E50461D6A5D2D516B2854886D2661745F674C5B5AD68D613E50
6658C7D4AA38786B4B4F498266565549436B7B3E7442507B554D5338496556405F3E6C5A
546A58677E5A5C513E947C38317F4553727956665C414C565E2A54543A80715C6D6E3D8C
875582727278B86F543EACB56562A48D56BEC16FA29A7925415A2B726450803C4A74342E
4F5A698C6D4B7B7D3F675337878495795284586452CEAC9A41354F40797F604D6E8F7044
46654C34426D54314B882D677F7C527059608A6C639C967131416C2827666C342F727A57
67A869396F836530AC7B4A68697777AE947CD5A24896AB5750536C75582D7C99
B94C89435B7771BD495F626B2F4167312F2949582D1F24166D49153D1D1E2F5F3B232B18
2D59331C4F3B273841221D4C534D5A353747653F553865676E3C3C4F4E42385E572A4451
291C6D45202D48232F4F294F4D205958275560323C76692A647A6F6C4F5547CC9D6A3C59
7E64A0D197415D714C5B56806C534F4B4C7F8D42795B567C62424F3D527C653F5E4D7560
4F7465667C63582F3079672C2F7A3A457B7562966A49475B45569D7F3B8181547B843567
625797906476C46741377FA66466A06E25668B529F8C5C326664255C48958F374E612C59
6750437C5D418C672F79453B77627F634E90754D1943659D6B3E4F186189554E636D434F
7F8D51374465303D70822D777B4F47855D5690634FABA070253F672B1E3E46213478805E
6AAE352A578F6C33B05D3C7C79958CB45D88D099409BAD4527406B7751398380
A74B8241568B71C64F574E854C3B533428313446251F261A5E411229232B2F643E202F2E
2A4C381D4059362950253440614C6044344B773F4A446F6C743F304E54473462693C4D4F
231C7751272C3F23395B3A515420525625535E343A7B6C3C769B7176654D3CC2C5893C60
884B8ADCBA5F58645397598E616A5D364269904EA26D6092534A7956568544437B466E45
61865C576B5180433688763F53994F4C854B3855734E4C5551213E673A8A63358F85327C
2528547C7466C286793955C3746AA18B45A0A86192AC741F3C3E1F50584C723B4E71352B
4D3C3B8A754A7066348D5027723A60353E888E78486C5E87395777314B5C46538963342B
51784149587F31213C66438F9B4C2B698259AF69386270452F527C4A1D425632265E9378
6CB981415688602BBD6F376065858AB6696DD56850929F4D29356546294DA087
B04E6A3B63826AC8576E5B8A57374B352C463A59302444296F531E31363838623E234534
304136224C6037285B2A3F3D655068632F4B7B3C6A4B676A7F4E2C50554B2F66795E6163
371C80702B34533C3F5A2D6C7021445D2F707D3A3B77262A85995F8266281E7C7A6A3655
6D32519D9A4F544534684E87263E5E4839606A488F605A8B342D5B4B5973362C593E633B
385953547651644F328885342E66464F98745D8B785E5363733048553D8A834A59742C6B
61406564776BC18E7B3C46BA757FBEA449A0AC507F8D6F294D5B2B66685981365586532B
593D337B714D7A6125524E2A643D5A5E4A946F7451A171715130584681684F3C59826935
5048273344705C2A4B702C757E6F36696B45897D5E8C7F68273C7C5A21494F2C28599448
419F60404F813C289E7E4381707395B76B60D06C4679A549526974472A338186
A852592B435B539B2D556B734B3138203B573951213F69396D561C415F3F416717255A43
2E38242E5F622B40313136372E275D7326405A2C6A3D5069543146745C49275F34225F7A
3D1B743E3D5E64473E5821666F253C452047683547785028577B5EA06B61408887642E52
63528ECCA63455664E5A458D4B4852412D5C9554967653835438574D6F94613C603C7558
4E66446A84626B423884804739904E42786454A06143657E7D374C4D428C89545F86316D
78435D5C5B6AB2886C3A3EB6395698803C99B07981998B2A4D692265644C8E314A6C562B
5E5C42897647647C376D5E2565445A5E4F816F8856AF8085632E564C656E683651957B3A
515D452B4277432960793F6C8D7A3E76785E965D5E967E7639436B5033686529377B9D6D
51A72A356BA67032AC692C8691427DB44387C78C437D9742253B595F37379A72
9651703F446552A65F4E58714B353A191B40314C1916421E65651833452D315A3517382E
2248471C3B3D1B4B49222929413550561C37683E6A473D736933283A4A3A35675C355E4D
241A61422E2A462F43622D4C55273E5C336560243E6E60436D803D81655D3E8DBD734168
705A96B5CB3F3F6F4D533F6559446A392F778E4E844C519964496338486B714264477E5C
4C7F595482687E4E2A795D2934945942877B5DB466554B5A5B33414931908A5264613179
6F425D75565BBC61414883CC796097833DA1B9787A9C9B334F741D5A704C893F536C5737
756E44767A336F803270692A69717B70485D5D773D726A785A3154317285633850814A4B
7285532E477E64304A81376E9451418056518E574F9F886F34607B451B453A0D235E9F65
5ABD4325396F4A57D2A43B44666388A1313B974D409F975C26365D4D417BB677
AE567942405C3E66565A697E464364442C54335428233919615B1A3538273D5C491C3321
2F5E4A23403E154A4E262731613E5F491B457042632F388C784A2C495E4F194A60457752
1B19695436324124466F32454B19325A2F6C5D37246677546BA263995B4E2E66956D2440
5F618CB4CB5642734B794F82654674402E588E57866343855E5E724E5C7B653F5E43784E
5594574F85467F553A747E2C469D5C428A5A43A37462455F5334445239708B4E6D6F356B
4832414B6268C5895331499F77608C9952C1D0746898A0417A701F4468629F3B516A542E
5C60586D7C4C67862E9671356863776A47A9806F4A9065923D4C795F505E2D456F8F6636
5C5C163D6A8D6439436622749778376C4C4E8F8D6B9487331B37685C1C4B4D27265A8E49
46B373353E7466349F834F6F706EA1B3574295462F718E502633544C21386D63
A34C6646487A437379506B783F3057402539254A232336156A681F403F1C2B47371B301D
204A372142551E4D5D253F265E39694D1F2359435C553465653C3B6E6D621B4E60538A65
3615696339366555387030444827315F2663724D206267537CA25EA0617A5659A35F314B
6E4661A8C65B486B427B4B9E403C7657376A803B7F503E834C46594A6798514875426E4B
4A835B4D9A727952418D7E40338D5737806F46C169614273564155633C6F834783853C74
795246684872C39B5F414C9F6276BF9E3393AB5D78CAAA365161375C51457430628B7330
773A26599353658B21758E3C5E737052264D7967417356763D296F725A665D3047836A48
726D35213B725F3748652A4F84682E7946367D7F5E9E8778323F756D1F556B362B678D61
58C375373A69732F8BAA576E6B4B7E8A4A4087633B8B91432337605E3D40BA83
B25D66383F70446A72508E9A613768512D6231612E26502758711C51503030602E284A2E
294F431F43572056652142256532848D2E3462365A64336772512D514B4E21555A3E5368
5617737A3E536D584692363B5135365329668258367E58317A90569895854C3BA83C3B77
856C65A0D13751A153581D6C75447434356F87598B272D9178536221518D8F4E6A24757A
4F746A599F677C341862765934863F42A08B3FC2765A718B5E393D4642838F4A615F3693
8B6061863A3E8C734A264C8D4D3B638A45A7C45A3F58873F605127586A41931E335F6A2E
645C5F639A5A5887315F5D39759894594292587A43A06473502E6363598467384576313B
5A7A4F294077554D8481366A99894373615B78715EA78D6E33476E5A237475392A599974
5CC09037366D76465ABB636D799489A2523A7A683F95914127386671363BC093
8A45744A404A39685F36455236328A463C44317D31344E2F99851F4E432C6681451A3120
4B591F255C532161681E322F541C6058255E7C4B52453483785731535C1C306675586A4E
34226E772E32481B40705E443D142E6D395C681F27386439608C405F45745531A959252A
404B6A94CE472F554251365951396051254973517A622F6F6557625741867D4C5D3E656B
4D635C5A976E7F5038735D3640A952357A6F57CB7B4030485737414B2346824867663188
793955616255BE8050373B878A48648A3390C17F5471814086652839613B7D30536B6F2A
51497272985954A64966642D496D595C429554732E60528654395A3645524F31417A5D37
5E954E2B46754D285776336A7D625490523F6B705CBEB07E3A4B633D1A4C604D29657C50
54BE783E4A8F86438B905F437C9879A14B62957A3A949A3C274A715D3D3E8398
935A65583F843C487A45494B301B364427361C5535243E11575D216E3F1926443E1B2C11
1D53481D4046133756211E1B54326D511C22443F40411F3E5855314D665017396E56704B
480F4D5F373672502F592B334032265827515547164E7B37528D6F9656765A39C188373E
56446B8ECA543863465C37653E35695328548561726931786577985C4F8F6C465E3C6E63
51735766955980563B82762D23765A388153347A755D3A5E434A5F7142437E3584854B84
74677C414759C1863A244B9B8E475F913E97C1895569884442502338734D96414B905F34
704D786C81314495405A6931578455432B887D85459E5B9B57365F8456665A3446876D3E
5D743E2640777142628B295772552F614D3B665A36927C7A344070601D4E563D3381754F
4BBE71312A697B3AAD74483582876484243B6D6C3C9B86462433535B3335B882
A86E6554489441498E46456B6B2B465125612B6238274F3354591D565E3F464F37182C21
2B564D1E3D5936375F2338205B446562403949433D76385363622C49606E213B7759753C
4E11577F4955735F365A253146302D5B284456591953733A5B886EA16067391D8A693139
623D5091CA62364E3368376339286A5F2C547637725F358449415B57458E46477438603F
557B5E629F576B42388E7043298C4B3B806938B47B624864422F3B55463588437B77516C
656BA33A4E54B299583C2C867D4D7FA74F95C76A578C8253594B26304D41724A359F4A21
5F67735172503D8B416788385688493E2370547A3C9A5C9E4442757F626941416C937344
5B79423556886A4A5F783253838E406F624C525A58B38D813B5671491C495128226B8970
45C291403D787938AB635B427F8B587F503C704431977A512330624D203EC698
8C645D3C3E723B4A7256828C6A29405229682B5E413257334C5221535B3A4A4731284B20
2D544F224A693E3363244328574872923A304A2C4F7F3C5D4D343960675C273A5C415864
62164363233F675C31572F3D633D2A5B31627C5D194E6040738F5E996E796221896B3E54
6440686ABA4E345443573A6660325D5E31608A55754F408967516D53489768555D305556
50624B3B83656C352C7E724D2D8F59347F8240AA814D528258463F5644367F428676516D
757CBB5D4667B4945F3F34846F3E4D884A8AC48E5E686F605C4A2D386C3F703029854F25
5F516358B76D39993C4C683F538E455832805A83459B599745275D6A626F46285680563B
4E7A3B23396569455F79256295843F6C4E2B686852C7B17C253D616A29546C462352813C
4AAB9441426771227F544F46724F70AE67407B4C38948E6E2E4274672C2FA28B
90755D60417D3D545B3343595522432E2A612856233761424E4B144354365659291E3C1E
3C5E1A256F6D3E395F274620371C4E663D2C4632346D2E64674F2B314C493446684B5434
4A1537703D367E632A49292A48392E59253357511F665A2C577D558D6F71671F84602B37
5B476D86BA57305A424F2B4F622D4E642F3F7F5E63684B875F4F6F59356C66656755613F
526D5E43956F724C40896E352D855A37806F3DB385654058493C3E544D3D904B856D517D
766ABE6D49579E9155373681904B52935881C998626A717062463258843F73332E8C6734
67453440995A3A7D4D57663F4E9F4E542F765B7B4E9A5198572763766674633060813B44
6D995A273B6A322E7F88324F6F6344854725534638906768294157352055753F2C636C3F
379065354E967E337766484D8470588E4076A5663A99AD36283765733521B47E
9372605441763F47833E595541336D5A25592E5E502B56355D702F4160253C514A192B15
1E53461D41582B4477323924624D586225324B42386E2950616431345A2F234664546937
3A1A447D3C2D6B372F4950414516355F384A5649163F6B3E64834D915F71632DAB87383E
57334FA3B6523067475B416972394F6F3F46855F6B7063AA5E426D67476E71646A5C4F54
59756F4C9B6592433288592C2C8052337B7536A5856E4F7042374058494189478971425F
5456B587585A7CA4413D74A5A6525490597DD4AC69989A6E91733435854477632E895531
7B7B625BA35056945E5353396EA48468326C6C6449B59A94622E564860915F306380513B
458B5A3F557B724150893B698D642E6268425C5759A968643A5F78581B3C4A152380875E
357F6F3231606C2C5E5F3F25777461771E397442369F6B58222C5D5F241FA598
8D56525B428C3E4B8F444E5E4C22465427362252532656443D6A313359393D4937192C17
2B585421406F4D2772343F19595E546032223E55368A3E605D763E316276293A734F572E
4C1F2E7A4E1D75572C3B2F2E3C2D345E33505E5A16537345779E708F756A461C695A3545
634149B5C0733C723383516969357A6B4046884F886A5CC85958826C44726B6B8C5E4551
519B773BA541595238868C22339B7A3A917740A5976C60915A3A5B8462358B3E95834B5A
4F39A6AA825C97C85E35377F785782AA5B48B675467260434653364A554C8B79318F4B25
5D5B5C65A57227705078903F3D9846612E727E83406C4F8E5A557C443F5D39439192713E
477A2136446B71404C8A195897893B7370307F8561AC91452A4262611D3C463622528250
306D7346446378363E534D2D8D8C588137458B4C38A5836227315C602A2BA097
A782523D4A813E3D5D49738373243F4F4968455352306453425C2B4C62443A4624294C22
2B5851253D6C552D712C361B5D6D505F53454969309A445F516B453365772F3E77515744
71322F8B631D8781384E4644623935612E536F52155B523270926B9173746E247D65384B
613D2E928C552B4E2352357074293F5C435A953C717965D3513658615F96644C5D42535E
4D755349937369452F978447267B5D3685822C84723D5D875A3C495256508C2B7A985186
7D4F9AB7444599B1854D3574895058817A38C496747E7475453D2F836A3B5F4235A06630
655A2448856724723A465F4943AA4770384F62723F7F51A54A28635F576A652F6B987341
416C392E3F6A773F507B3D4E776E295F5F30684A46AB83662F41566622375230264D724A
337C604B495964433A796756A8875D9D5549854544A19668262F5E4512306F8E
755F56553B60455D66402D5467294A391D4D376222236768435B171B453D354924133130
2B601B1D446C533190254C223C2D547D5837583C2D87466267503443766B3C656A313837
702E3293351D767C40704E4643333F7023437D6020687A3B6A8E54A6AA99702094663370
7F3F43C6A96A2E6C3C502D40713B4752314C705B704262BB674657394D596D51593B395C
59735A47836A603224816E28267970337F83398C9C6B537263474A4E46317E4E83743358
6F599999615587A29364397D99545B856C49C7AA71937C73463E2E477B4261433185552C
655946688863276E4C5E6B403EBD5C683966718945866F8D6B37646F556578346D9A8445
4082623343797A404E8346628A7131526C467A4C51BA8E603A445869244B6E3B2542692B
3B72834F4D595E1F5EBA7345AC43539954478A3A3BAC9774282D5B6D371C598F
725F494F3957273A6B4C405C5A3887682C4133604C204E4252723933391E2F4F38183725
33554B202C4C17265D1D23204E49454E25407458236125537161352C9B372544684A4E30
50224787621680332C626C49401F3B7F44437754195977496E8943524467661D69592E42
4C4044B9B15D2F5C2E4F3C54773744644E4F865D5B8385CC69444C67515B7D4E5B39474E
54655F5795715B402B84773E25828648A67D4086A9695172533D4B57593391517B725154
4F4A9292504F7F7C8E4C60929A565F864D43B6A16380734C526D3440804D8168437C6B3C
7C625F659B5B3C855E65773160989A8A436A70763483929C6635594B476D663C69917942
488F5E304177724972995D598E5C42858A498563379491652E43563028466A61233E6D4C
2B68924652897528338555269B79449323689D5E43AEAD332A47727D3A23608D
668B5264397F48416D4344542D21565C2D2026705A275A42356D31203A29233A2D15373E
1F6163292D565827612422174B5C5159312D3549207449584D604C2F9C61314077624832
452D2686711E605D3C5230423C43337F3B405E77286A7843728C777E545D412B80733656
57312D9FBF57366528513B5282404F7B56578B68768A77BC694B6974645B74597042566F
65897866A573826B3C836F393D95943D90724388A364486A413B7E9F5C2B794AAB875752
5238809F804E866B845837679D5C60868348C2986B6E78752B401E275533616C316E6522
677E796293562D734E4E7B4737994662496F778353684C7B632B543E418D5F3563843549
73894F2F456B41314A70465F924B225C72357F5D469D7A543E5F7D671F3B5B342A717D54
3B9584262C5D7D493B82471F5F774C8B413994413FC2985F1F224C743A256496
658C546E3B874F42793B536A7E37525E31445F95602F5C643D71331D4555363E1D143E63
2C6372353567652B5E252E1D516B42583C484B5C2A88625D4F584F3EA8512F3E78614442
573729987D215F6D3C594538462F3C74313A65561B74724B80A172604F585E2563482453
5D261C6D967A446D245B57447833646946538635747C617C4E415D62514F6C445351595F
47766E458D3D4E593C827A24226E7A3771703B6C7D4C375A52293C554A235E2D5A714B52
745774727466908A9A764268935A80A99B4FC0BC509FBE753944171B58445D472745562B
5E496B66AF6D35794C6AA55639C8507A5684ACAD6A856F965825767651653D3088946340
4D79224C6F977C494E7D2A5EB275486B753B937250BDAA561D39565523445D2719396412
1D548B563E66885440B17129526841A7633C9D3A2D949F6625275F63172E8E94
848E53503C6E42404F316F83733B4B5A3E5A4F6B4D2B5D5A3A6939376757403E331E5E5E
28595A364F7E692A62374B265766458857594B4D2161494D64473022704E2F3262343455
582F2B8C45256B702C5A42436C3A3C7320497759255D57256694775A586A72298055306F
76514983AD7333753C4D4C387A3F4B4F525094664C825D7E63585B60505D7C70784F646D
63815B3E887256362872763E2F7A703F9A8A47739C44538050424C6E52317C5483726175
836C7B92683A86A68C5F456586353F6B4C46C7CD5F55725E454A2E635B405F5A3C766F35
7260253A916C348E3C3575634AC1A5B0493F5E7B3B8CBAB65E1F4A6F5B962B1C4569374B
7C8B35223860554C808D2D305A574979862571613CB6C4833B3D52472054806B23487C43
2C5D865447676D1332B77D264C36288E4D67C261329B9C4C3E436869282D6986
615954503257314F87464A4B5E2C492B2D5934642C1F746637782E2B5052394B55183A49
2952291836635C236E1B3D1B4E332955464E476C3C84535B6F6860377B603E3D7F5F4642
3B1D237B51215052306D433F43383C7A354271561E619A4C749D3F3A374C37299E801E33
54544492BC7326703C4C2F2E7946515327476B634F403D73595A643A455C696580265968
5B7F54327A476C552D67811A288E7A2E67783578B354274B57306DAD5232505E8D813155
655568905A5C7CB57B515D9AB3603F6D473A8B9F706085503665252247427B822A46521F
716B5A6FA549329263446E193B994C6F5E4A85812E343C7A8D375243399468314E812F2C
496F4B25456B2C1D78905D4C9A551F738043836B277E9C7739516938143154251550A690
366853163F9F8F3D2B8A622B6768548F1755B1703A9F9C1318468E833C22557A
695E4A593C793B2D67415E673D3069603B3A256642214E3F3C7E47446C2B2E535D24412D
274748233E6D342D682B3B1D69643B6A363F6C662D7022435B5043416C37254F72563D4B
4B19397E5D296529226157454F1A38873C3C7E5B154A734D8BB991542F4A171D73763C50
6F3335A7C08D466F296A593784346E735941695572795C6D454882725157603D95617B6D
40908052AB20496C47719018257B893D7B7C1A55A96D3E7171193A686D264F42C4A26952
624058BB886E88B97217417CA366469A963E979D6676AD72255434304F3282994F49541F
616E716FC85625816D5894403CB557804276928142906B8E5F3753493B6A3D2C527E6543
507032264965674547765B53BC622B5C7C3C7C725BBD99642365AB7E1C3B5E1F18346C55
3063703F2A465E4F2DA339227F6A73953231A53F3298823A1820446E3E2A6A89
7694523E3B6D42405A3594B06F2E3856344F2B553E265B5630592E4E895325435319595B
2A43552748714020501D27123545367D472F3957297659564F555048A54F382B6F623A47
5B3721786F32675C33614B4B65273B8D2D34886B3357761B5999817F8F7163286E44274F
6E29265A79924778124C623B6E1343775E427715579B727B4530466E4C5F572B5B6E9384
33596A5CB2704A514A8C91211A5876458F9B4969B33046697B44445A4A4F78287D8F7051
8C1835858E8290D095503A608B4871A1AA369BA7594159612D5C3C57652B4C7D4C7B8235
6D673C6BB88833854354786B41B960815A41545C489367C33E3B9674597E5832537E613E
5570394589A6824D4A8B245A96805D666D1B74A664BB965B1C425B651D3B643E253D712F
26817A5441646C2734B8531E7A5368BD513BA94C287B8B4B23294D6737296383
648252563C6E3F4763263E56673340321F5D3B6B221458733C721A265F74353F4A124054
3C481B114B8374296918471F483A3E95753B48490D7A6A546A491A1F68743B30612A396F
6F3F278565338478366E573C7B4F4794142B6B7B464B962D57A269484D5E5A20694A2E8B
A04255B3B16A3C90454A463A99543C563660936053505BAC705A5C5B74B984566232578E
5F5E42387770563D1C7C9C4E2C6F712184963B66904E2C4A48444B62361D554B8E855C7B
9A714D7A8F3D73C8996D375D8654306F9633859A755970803551254B81404E6B21567A31
6958414D8168337E524D6A6238A77179835B717467A665AB67295578558243398DA57945
5573291F43696F4A4B7F57478A8864706924545B3FA281483F56625A1F51775E26478163
2A698A5139566841259C621F794065BB4238AA4E277392381E26476F48266790
6167566E4066263F7D47566C71435A68284A3B594E294166376C41276B773A4B64214056
4846491D2E6767297B0F47224D2F2654542F4768316F555072686756575C44457132222E
624F298649116B652E7B61394D2F64A14D316E7025488E5463A583944C4D71286951193A
605B5DADB3721E794F3F2C1D69453D5835316272485F3543544F59432B4B664E5E46566B
5959705767625E341B48713F2B6F802F6F96404A9E6C34566C4C476B5A2C6556A4732950
89754C8D914B4EBC956E36639F64397C89348EB36F5B72863B52352957374F7548465326
5C4D4C7EAE5B347F4C5185493D977F7F85677264499AA69A753C5856477D562E56876D49
568A57294B63534F7095726270827581894A58795BB7B47A3F637D5A1D3B5F6B2D5B9D7B
316F8A4C357E874C309F5527886472CF2E58BA7C2E54A62F2235657138225180
6445475E3E783E3C88435458282877793A4B3F7771364A5A3D974927574233616B253820
2E665E2E3D7A2A1E86373D23696F357342367D8740662F395B85634C693C175079603F47
6E2F3D767233831B0A56714D5125297250376F6E223198535BA183937547482D7C61194B
68383BBAB93823652E4A3A2F854C40613D418174448A6367665451754E69775052425185
52606E618E6F65493A5991402F7892447CA54455A97B406F893F579E74386D5D80687456
58644E98977763D1A159518799576FB698377AAA536CA08B3F5E3838553654875F51632C
6D715A8AC55C33885352AA57398975816176A87E33775E8A5A447D48439C7C4B82A1533E
5F6E2744737F67595D766F5A726D3C8C965B628E5E9F9C691E4F6D5F1E3B552021556865
33664E1821596E57309E3919666D47CD4B319D8325478A5A2B1F386533284E88
6166474F3F8747437A325E8358273C51232C19424D26482E165C321C3F492330511C301B
1F3B552F35664E21562130135361337D1C13385C3A5F2C4F50735B4886673C2458483B4B
67221F72793677562D7C6A3E4E48399A4D2E686E3F3DA64D62B58B85AB5F5A266F402284
9A3C3EA0C43B3081376147379155576A5049977C4E986773714D6F8650707D4E61575CA3
51785C51A67953553E76B4562D7B743F8CA333509C3E2D4B5843476F4A42845882837379
88644AAB9E3D66CAA57336589B4E356EA04178915F404F682C4C3263782F54783C62743A
666A162C987A368E5334735B39857D806A4B68734B9476C248225D784F85301E4D7A6739
4F5E232643475B505D6F2A36796B4F8B841A47773C967D59384A6064203D5E5E2E567B59
2A634B3B32738B722FB5752A75814ED08B4795861E5790693029416A4F29528D
617C4E563F793E519547425271444B6D2A4847564B2A44543156311D3970414D521E4537
2A45552D466A562471243F2E7172389B5E27456C39564970827144396359433C6A392545
4F314E9F7029775E308967396F504CAE4B3B8778424DAF3740AE695984635C21804F1135
69554DA7B34D22674F49381E725047542D488E6D4B5C3F796556574F30607D5354363494
6259462D6464633F1F3C7A33266B803759993C5B7F6F3A58A0614C6C462D65618C6F3E4E
755746A2866A73C9A46C3D6AB16740728B4098CB6C586D902E4732304E3750873847472D
68573F6AB2612A81535691543D838871877C7D814FA682A16E3E61514C87663F609D753F
4A60412F4E4766575762794C7A88418885324C45259D753F3C4C5C622052786737586456
2785764F3D697D2B27AB742B784B2CAA805BBC8B2B388D493F647A6525317E8D
5F4F494C3C72373B844457582A377E6F2549335D52245949337A3E213E362D6061204122
2165511E2F62301A74212E2969312D9B392C605F30511E4E6E7656465D57202F57512C37
441222595F257030174F4D334A272681513B796C19307C4240AC7A63BA585E288F902F4C
895340C0C45134734C454C34866E485C504394875C81585F595B5880446568514D4C529D
605965457C5571453A47883121608A4A719C3C53AB873E509B5D497A563C827B91687B67
5E424EAE81838ABAA3565B8BAB703F7995406A936154659334453D3A6938529560535C34
6366549AA95A316A42558F433576848B8D746A6E46B7C79576466048438F5D3B5C906D4A
69874E36554A5B5B887B8C585B8858828F425B5238A3BB614749593D19687F69355A5D37
215B5B393A816C5826B0702F686933823A3992773745915C1F1A415535184883
5E864A583E915547955955506D4240623136212E402744391E62421C334F2946431B3536
2348643138665E1E5D173D33564C40B95132354F463C20766A6B524B5A704A24655F333A
4D3842948428734A2C5B39344D3F2B96473B70523435AD533AAD8C4858A1401D7748274F
9654177BAB5634793E64542C795C506552439E85598C5C575F54558A516D7255534B749E
627E754AA14D8A744644A0301F5D8F4A7D9A345EA6803E5A845D52904F33847B8A6C786C
545B5CBFAC8672B89C523861AD7641849B5381AE6C4F8CAC43523B37613653905D4A5329
63717599B97143A66B4D8050335E787069816D6237786A7878446935479662375D7D2C2C
4E693637544545305B78825B64843475A7507180327F826A485966601E3657553B928F75
286F4219308B875F2CA55E24848D536076619C893A61AE612D203A64331D5189
64954D44407A484A8045909F8F5048783C3B464A3623495C376D2D16316C3C524418394B
344C5E373968642B5B174240564C49BE5E46475951371B83716850498C883F266255333C
5D4C4EA183268B69277554395151299D573A6B6C5733AC4745B5934A6F846430873E1D3F
784C1B4EB05A4C99334B5A30782B71856748B15A4B925E6464588790516F824E675F5F8D
4E676A538E57665D584BB72E3086964C67932571A37D3268763772916134824C87978264
60473D99857258A9A55E396BA54F5C9CA85D8ABF5E2755782D313C417044709656435A33
73835C84BE5B33896C4585503E728C74557C9F8F509F92997B4566383F7F60385985693F
53613D48774C7D49526C8B4C6E80375A8C346E4C2A958447356D8A6F2038502B1E405E53
2F94634F2159624E3CBD671663395180816EC0A0334294633A27405A25267891
748750533D8B4D49904A45506D484373403E4B4E3C294D6034682F17315D404C4B1D5453
2E4C401C52775525581F4E51586444B16A414462565134777D692B264F7445467128254D
644854A46A209B792B88673E786A2CB3404B8D875A3DBF5035A77747B76A54288B401644
784C1D30983C44614B4E4F2B8A635055305198704B4B297278364F64408E94534F4975AD
79753D3A814E6C55285FA05A29587440858F2D72A276387D7167497C4135867B836C7960
5C5061B0A18158ABB58E467DA149365C794895CA773E5888334B37809A3A55854C6E7239
50752145C481429D4951B75442778D893767658E51BD91A96F489163568B623F80A87F42
5367253C7D918B5855718C46A38B4F61981C75795BA688553A4550532C6C895E24495A27
1F73826A1F6E6136297D62325156437F5E50D5A66A58AD393858705D411C427A
65544C54406B3D59A257566783433C402C6C474957357379376A44263F52334B54253C51
27626436335D52236C1B4C3B80543DB97F474A532A5237698E69473E4F85282E5C53332D
583D54AB6C1B875029776338443A2A9255366E852F2A845339A98D4ABA50462B916E1D41
7A843C3FA7739D9B445F682B6352667450368C7A508656576B436C7C3E5C75587944458F
589A72479E2E5472443DAA322B7169219CAA4277A140246A81655B90432D727DA47F4F69
9A6E68A6926B5BC4C1776598B7723759824070A06F404A9446685435774B6E96366C863A
55654250A36943AB39305E5F44728F78487B5D7649B691865F355045489B5B304F8B704B
706B3A2A3F515E4D96B13E21938F527399305A553C93C36545484936173E5C5826507F69
34908A6E1E7D6E733A93531F5662458B3640ADA45F67A95C2F2053753E1D4C98
69714750419047437C475A583C42808D3E46375D6C2A485123897926342E092257394037
0A2864353877351351274244812D228B5D41735F4C57224C6440343E4D803D296A5F3239
522D347C6C1E9453274D36404A5321945235688A58339E603DA79F4EAE7232238F872E47
83501E4B7196A1C02F385D2F6820497564388337277E595650346988517270294D62528A
38786E53834E57574B40C0323281873966893483B672216E642E639C822E65409EB7533D
838564A9BB8444AE9F404568AC593C55924D86C184403D8533413C2E633281984B5C6724
6D85687B984D61C15146744D3267A48F56845B783D9FB3AA773861404EA0803A4C833B39
5269472F4E6D4B2D57756F3A9C632E5774505F7F3C749968494E617126375043316B6F58
33737830257D714D328B63324D6A54B05A458D98357DB25E2C22427643306595
7C9A4C424277484A6B307C9874302E54385B212F5F3350591151532E4D5E28203F194A64
203556273F80641F5E1F42484D4B379469373F3E43653966654D363C5F813628655C1321
653E338271217B7133525445605F23985D45959E692B83373D848A49914D3621804F1D42
7C752356519ECD9A46485E316059455E60386D7559975D5C4F2E4771516E6A4C5A5A5C9C
6570724B8B6A70563E48BA2A236A7134557C1E69896A1C506D54466864376F406576815F
7D533273879358ADA15C4D65B06049719C5289AA824B7F8F4255542855375C844A556B26
44796975B24E3AA77B44754C22359EA35498AB913974638486487C8072977C4655967646
546E4D3A7084864D737B3746B48A4A6A885D7577568FA65E3563846F253B484520477067
2F6C87661B6D57433DAA8238693F20655D349096435E9E4F2F2940651E155483
608B4C3B376B48456B1E2E496A3D32300F50413B451475843C622F14345C331F410C265A
283448041B55632677114547461F147985494E3F0F49515B5E5817155C7E412361472338
683137804B1A706B365660286D64228035305C825322965F2F958A5A8A69462D9848224F
8274354641B6885F3A505A29615C4E605F3D899163A5615166496888495574647269509A
7A7F6F337F597151384BC022266C7B3157903C8A94651F647C6B476C5A38696DB4787D5A
83746BA1976557AC976B475F9B3B558D904782B861204D7F4B6855285C486D93584F723A
486C2F59AF663CAB4F527F68414FB99531596F764FBA86A75E246A887499563372A47344
5865171F3576804D6D775D4494683E60721E50715087BC903150606E2B566F6229488566
37A89F75206C571A1E6063396D6A2F8A633FB5A07A86BD77352B42784C2C4C90
619A4D5B4191494DA151525284433554253C413A4810426E39604119294C351E392B3264
3E2B4C2A225B5F257819462D554D3A7A77484E4A445553434B584C213075432A6A452A35
4F2E35776D34595F38584135566224825A498786352196643498943284854D2C8F412146
716D345536B43D654F4E4E21775D4A623B3E858C5B8E5079744A6883486A8E65785C60A0
6C795231907F665A2061BD5B39677C368CA140879C4C3370776248664B44657395826D57
A779609B817A48B5A263476DB4483A66863689CB743D5F8C4D5631438947668C34468D42
447A244FB55631964F4D7E5B484FC087506D667347B797A06E3A5C706E936D304988683F
48654A2E477D7B4671745C61A28A576E7D4F5D6E3D82D79D525566421F61825C264E6A32
1C73876F276C4F74347468459D8E388B4A2981A765A8C56939324B6F522B519C
686F4E5441662545744A5A4E674B557D3D434B395A304B813553402735552E355636365A
2B3B513D247168237C2F48275463417B6E456E3E335E55436A77722B377F1E317064424A
5E2C4D90724B5160397D53465B432E74613B899D42297D502994A8366E68552F87601725
556E523B5F852F5840484E21604838634C295769426A1C4D6647675F3A65824C6B504A8B
617C653D7D53625D453FAA4E2F616C246B8C3E808F4A40846D5F4A6A4E254B536D995052
9F88779F927254B59A3D4182B97D41659143609F8459848E3C6E3A355A41679B44448730
517C5B67BB4E34A155415E4C5683D598678F54672C879093803C6B5C66B8874E52A05849
6194733248903A3189905F5DB6412D5E7A624E832767B37A447BB17A232E56622348676F
2F82645664AE947737774C2EAD803B98642971A456B7C75A3553717C4B314F94
6160454F40813F3E7A48594D3916355F36392C526B3A4F7131796232344C0E123D3C3141
0A24523A266D3A0E62382E144F7238724B1E434A3F5A3422345F6A2D357C371B564F4854
5114467365573F592F4D4A3D6F4322734539A4AF551FA8363FB19B5C6169281B798F4535
5865254EA96D45602648682F6C3B406D6B36617A5199515B534180813D456C2665765D93
3C938C4A9F394F81624FCB503E8D95375F854084BC692D6B623E62B282375F4A7BA6634E
A271478EA08F5FC9AA484063AA5581A0A0486FBA6733909940433F215C447D9459549120
476D607AC04D2385756AA35B3A9FD6AF53915889449564916F3E695B405C804F5BA54536
3E786E2F498864385D745A50A9743F62665B5E845787B69542445B692739572A2E5B6D6E
338E5F3A214650613987745DB7533DAA844775B82F93BA8E382E4670384587A4
659D4C474380474477356A717A3C2B6238401F23613651631957442E4B6A44173D2D3C6F
3F35563737725527712B42354B60317B6944492E1E80523E4958362A54975B25551D336C
7B322D884D35497F4B655C2262662589201A608F5E2DC5431D778A5B53641D1470462B31
486D1A2376694B6D2A3B794A94382E5D794275511F8B5E683D1C568D44506B183B7C5CA5
38545E469A614E5B3C7ECF42205B80355C852839955F4D928B3A3A585D49884F487A8951
9E714786796C42AF93494A64A7433D5C823F83BE814C629945563F4F844B55833D599041
477E25409059297E3E3C61603A3EC89566B8AE7143AE7FB05B45A471647A825C95B38E4D
5577553F63948846646A3369AF8440605D2036562C91C0795F526D7A293E6559234A7230
24938F58297C8B2E29A18B57BE3E2167362E65B85E89CB6137698E741F1B4B93
5A8753423452404F5817374E804C343C1767553E69317C854672351C3E6F601F392A1F57
413A3E0E1455722975124B594A13107C7D4B6B4A0E58634275773B17296A4152842A233C
5E2F40853D26266B4A6B54296B6E30944F2A5D8A2223945A2495B65F7A9335258A3E2953
69874B2774593B78523549246D68325F3D335A6D4D893D81754A6479507C84585B534395
646A5937914F5A592F45A049346C7741979A3F48B54A2D738A6C475C5D304B4B826C6D4A
AD956596887756B7C04C4760A84F4A6F844689D0A067729F5B6D3D2A684A558338528B47
4C714A6FBC58328B474D614C4A50C972406059613FB18C924828625B57723F3166996B47
536A3321527F8D6466714059A873325A573D3D592F8DC997615A634A285F7F5B3486A243
1B507F313A94A64B1D633C2D7C77368D6A3F666B4581CF64362A3E67392C5995
5A5842453848365D76363A43664C6752133F423F3A0862804080431837665A47553A2E54
37615942264F571F762C4B4762461A63382D4944295037284B4F5C382F71211F57514657
5B33769D695931603FA5773A76402996663060726121BC602788944645734D228A3C192F
416C612C80432A6B3F3E4A307C602F54542D5B886189515859445E7C3E466E65554B56A4
6972694B966C6368545FC74931698333668F4052A76B3277926E47645D3D6F41606C764E
8288669D908771AEAE58546DB96E5B81732D64AA9E5174932F6D57347F5D5D895169A659
8B886D8CD0622476755471483561CB8C5C846D5F40BABC9B5E3F67495C77534D68A16642
6C785A2F788E3943A09A5465C47638748658448E3B6FD4AC5B6E865819314E2021567E7A
2F5A864F20606B6C2D7C5A2B4C6734887B498958347AAC4E423E58833D266495
62974D563C72434583424E503928436530373C72743056492F6E5024533F201A372F2E4E
1E2B4A35325E3B12553D39264E531B6C4E375035224E44407E747D42327E45256A5D505C
6829365D575741482757663D7F6E339B6A3060756622BE67298A725B3F8B361F864E4939
39845F317C5843732C3E47327C5D355355366F9C6A85686C6645588C505B6F705A6057A7
6D727B4D854A548A645DC6403A8C88397E854770B4753060862F5DAE8B40734B5F5F995B
81865E9F838E749BAE535C8EB87C8AB3A85C7FC583578CAD3B4E4C33754B899D5274862A
5F9A7D7FBC833982624B7F794F40A67D5A9E615F297D7D8C8F456B3B2E687B5081B2372C
4F734D3E878D5A364F713D71D05D134B675B5DA3396BAF5632708878253B5D4A2787A36C
3D5C8C50287392843F7E703B4F7A339A794897674985976F48476A8B3E2C6699
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.511614 0.0119048 0.97619 0.244193 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 0.003125 0 0.003125 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 .5 scale
320 string
320 160 8 [320 0 0 160 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage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P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.255807 0.488386 0.488095 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 0.003125 0 0.003125 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 .5 scale
320 string
320 160 8 [320 0 0 160 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage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P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.511614 0.255807 0.97619 0.488095 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 0.003125 0 0.003125 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 .5 scale
320 string
320 160 8 [320 0 0 160 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage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P
% End of image
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{600, 300},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40009H0001;2000`400?l00000o`00003ooooooooo
oooooe[ooooo003oooooooooooooF_ooool00?oooooooooooomJooooo`00oooooooooooooe[ooooo
003oooooooooooooF_ooool00?oooooooooooomJooooo`00oooooooooooooe[ooooo003ooooooooo
ooooF_ooool00?oooooooooooomJooooo`003_ooool0D`2g]kL0Ti>C0<g=c@1PH600Cdm?04Y:BP0a
<C40I6AT02l_;`1HF5P0Q8B4059BDP0O7al0EUIF04a<C00X:2P0<30`01LG5`1HF5P0FEUI03De=@1>
CTh08R8R02`/;01@D500J6QX06e]K@1ZJVX06ATI0451@@0m?Cd0=cLg01dM7@1@D500>3Ph05IFEP0o
?cl09BDU06UYJ@0o?cl0?3`l03Lg=`0J6QX0BTY:05QHF00U9BD0>3Ph07EeM@1NGUh0=CDe0410@01:
BTX0@41003/k>`1?Cdl07alO06][J`1CDe<0<c<c03hn?P14A4@02`/;051@D00Y:BT0B4Q802LW9`0/
;2`0N7Qh02l_;`0n?Sh0EEEE03Xj>P15ADD0;2`/04i>CP0[:b/0MGEe08R8R01KFe/0ADE504i>CP1B
DU80EEEE0080=cLg05@0:2PX06e]K@1<C4`0GUiN08b<S01WIfL0?Cdm071`L00m?Cd0IfMW065QH@1>
CTh0:b/[04Y:BP1PH600<C4a04i>CP1^KVh0ATI605aLG00i>CT0GEeM04U9B@0h>3P0EUIF04=3@`1@
D500De=C04m?C`13@d<0IFEU08V9R@1ADE40FUYJ06e]K@0l?3`0IfMW04E5A@26QXH0O7al04a<C02/
[:`0Hf=S02DU9@1KFe/0IFEU05YJFP23Ph<0Kfm_03`l?01]KFd092@T06UYJ@24Q8@0EeMG04A4A01_
Kfl0Cdm?0:6QX@1fMWH0Bd];07YjNP0_;bl0GUiN08:2PP1WIfL0HV9R071`L015ADD0@41007UiN@1Y
JFT0GemO02HV9P1?Cdl0N7Qh03Ph>01^KVh0HF5Q06YZJP31`L40VYZJ05YJFP2=SHd205EEE@1[06=S
H`1OGel0=cLg07]kN`1VIVH0SXj>05]KF`16ATH0GemO0451@@1?Cdl0QHF506UYJ@2EUID0@D5105mO
G`3=cLd0DU9B05eMG@0k>c/0;Bd]07YjNP1iNGT0Kfm_038b<P2?Shl0Fe]K03Xj>P1EEED0UiNG08b<
S00g=cL0CDe=06YZJP12@T80L71`06m_K`0_;bl0V9RH06UYJ@1IFET0];Bd08b<S01?Cdl0@41006QX
J00l?3`0>3Ph03`l?01ZJVX0Hf=S09^KV`1]KFd0@T9204E5A@1_Kfl0>c/k05aLG012@T80IfMW06QX
J01JFUX0?Shn0:6QX@1<C4`0>c/k03<c<`1aLG40Fe]K05UIF@2ATI40VYZJ05YJFP17AdL0MWIf07Ui
N@1;Bd/0U9BD05aLG01GEeL0;Rh^08n?S`1ADE40FUYJ05eMG@0W9bL0?clo06ATI014A4@0IVIV071`
L00Z:RX0Fe]K0:b/[01=CDd0;Rh^05EEE@20P800QHF504m?C`1iNGT0HV9R06=SH`1ADE40FEUI03`l
?02FUYH03_ooool0101]KFd0JFUY03Lg=`0Z:RX201dM7@0G01/K6`0[:b/0Rh^;04I6AP0S8b<071`L
01hN7P0W9bL08b<S03Lg=`0j>SX0?Shn03De=@2;Rh/0[jn_04a<C00V9RH0820P04i>CP1YJFT0D51@
03Hf=P0X:2P00P1CDe<03@1=CDd0=cLg03Hf=P18B4P0ZZZZ08N7Q`1XJ6P09RHV02@T900S8b<09BDU
02/[:`0^;Rh00P0V9RH03P0h>3P09RHV02@T900j>SX0;bl_061PH02ATI40Shn?08b<S01IFET0<C4a
04E5A@0I6AT061PH1@0E5AD0e@0V9RH0UIFE07MgM`1RHV80PH6106QXJ02DU9@0GUiN02/[:`0X:2P0
<C4a05]KF`1bLW80M7Ad06MWI`1?Cdl0GUiN03lo?`2EUID0QXJ608j>SP1@D500?Cdm03Hf=P1`L700
8b<S020P800L71`07QhN01dM7@0S8b<09RHV03Lg=`1UIFD0RHV90:JVYP2^[Zh0SHf=0;Vi^@1jNWX0
GemO05EEE@1QHF40/k>c08V9R@1EEED0IfMW09bLW037alL0XJ6Q09BDU019BDT0<30`02h^;P0b<S80
B4Q80451@@1@D500QhN7034a<@1EEED0MgMg05=CD`1cLg<0UIFE09NGU`2EUID0ZZZZ08B4Q02BTY80
U9BD05QHF01CDe<0P8200;6a/@3GemL0a/K60;2`/02HV9P0QHF5061PH01aLG40Lg=c03dm?@0^;Rh0
;bl_034a<@0h>3P0AdM705iNGP1:BTX0HF5Q0:^[Z`2a/K40WinO09^KV`2c/k<0OWin04a<C010@400
=SHf02PX:00l?3`0UYJF09jNWP1kNg/0Ti>C08b<S02^[Zh0GUiN09BDU02^[Zh0RXZ:04];B`1DE5@0
V9RH0;fm_@2HV9P0Ph>307]kN`2HV9P0^;Rh09>CT`2;Rh/0h>3P0<K6aP2IVIT0Ogmo0;fm_@3:b/X0
VIVI05]KF`1hN7P0gmoO0<k>cP2RXZ80SXj>0=GEe@2l_;`0clo?06e]K@17AdL0B4Q80861P@2GUiL0
LW9b04e=C@0o?cl0A4A405ADE01jNWX0XJ6Q0;Ng]`39bLT0gmoO0;bl_02NWYh0Hf=S06IVIP1jNWX0
LG5a06ATI01@D500NGUi0451@@16ATH0EeMG06e]K@1cLg<0QXJ604Y:BP1PH600IFEU08n?S`2OWil0
CTi>07=cL`2_[jl0Y:BT07alO01FEUH0De=C0820P026QXH0;2`/02DU9@0Y:BT0=CDe04a<C01NGUh0
De=C07inOP2BTY80M7Ad07MgM`0d=3@0;Rh^04Y:BP1DE5@0S8b<0:BTY02d];@0TI6A09ZJVP1nOWh0
SXj>0<o?c`1YJFT0B4Q80410@00o?cl0@41003Ph>00@ooooo`003_ooool0A02a/K40QhN70;Fe]@1B
DU80D51@04];B`0S8b<0HV9R03Ph>01]KFd0EEEE06QXJ00];Bd0Fe]K03dm?@0S8b<0820P01TI6@1O
Gel0EUIF03hn?P1:BTX0<C4a03Lg=`1KFe/0G5aL061PH02FUYH0?clo03De=@0h>3P0=CDe04E5A@16
ATH0=SHf04A4A01ADE4092@T06e]K@10@400=CDe04Y:BP1;Bd/0EeMG06QXJ00];Bd0=CDe06=SH`1V
IVH0ATI606=SH`1KFe/0@41004Y:BP1OGel0<c<c06QXJ01[Jf/0=cLg04Q8B01mOGd08b<S05aLG00_
;bl0@D5104U9B@0];Bd0R8R80P0n?Sh0;P1YJFT0C4a<04U9B@10@400GemO02d];@1oOgl0QXJ605]K
F`1;Bd/0BDU904a<C01LG5`0Bd];03`l?00/;2`0GEeM03hn?P13@d<0YjNW06m_K`1=CDd0Jf][030`
<01iNGT0F5QH06EUI@0L71`0?Shn06YZJP0n?Sh0D51@06e]K@0g=cL0MWIf04M7A`1gMgL0LW9b03De
=@1KFe/0?3`l06MWI`1kNg/0MgMg0492@P1HF5P208R8R01h05=CD`12@T80<S8b07AdM01CDe<0GEeM
04A4A015ADD0^k^k08J6QP11@D40Hf=S02/[:`0c<c<0J6QX07MgM`10@400Ph>304M7A`1RHV80N7Qh
061PH00k>c/0HV9R05UIF@1TI6@0IFEU065QH@1jNWX0DU9B069RHP1^KVh0IFEU06MWI`1GEeL0GEeM
07EeM@1gMgL0NGUi079bLP10@400QXJ60:2PX01cLg<0Kfm_04];B`0l?3`0^[Zj0::RXP1KFe/0Ti>C
03Ph>012@T80HF5Q06UYJ@1:BTX0LG5a03Lg=`1YJFT0HV9R04m?C`18B4P0>CTi03De=@1WIfL0RHV9
0;Jf]P0U9BD0?3`l0;Vi^@1NGUh0LW9b04M7A`0Q8B40@D5107YjNP1TI6@0?clo09ZJVP1QHF40DU9B
03Ph>01LG5`0IfMW03Lg=`1jNWX0WYjN05YJFP1ZJVX0OGem03Hf=P1mOGd0L71`04M7A`2=SHd0IfMW
0492@P0m?Cd0L71`07alO01IFET0BDU9071`L01OGel0^[Zj09:BTP1GEeL0@d=307]kN`0W9bL0HV9R
059BDP1NGUh0IVIV05IFEP0n?Sh0MGEe05QHF015ADD0;2`/0P1ZJVX09P1MGEd0HF5Q08V9R@1SHf<0
A4A406m_K`1JFUX0<S8b0861P@1dM7@0EEEE02d];@1hN7P0Q8B404e=C@1KFe/0:2PX03Hf=P1`L700
?Cdm06][J`1fMWH0>c/k06IVIP2QXJ40LG5a03De=@0];Bd0F5QH07QhN01FEUH0ZJVY09jNWP26QXH0
EEEE05YJFP0`<300VYZJ3_ooool07P24Q8@0ATI6034a<@0V9RH08B4Q01`L700J6QX07AdM08n?S`1A
DE409bLW01`L700P82007alO028R8P0];Bd0<c<c04I6AP11@D40Q8B40:JVYP29RHT0B4Q802DU9@1>
CTh0JVYZ03dm?@0_;bl0KVi^03<c<`80:b/[01/0<S8b03Lg=`0b<S80?Cdm09fMW@2JVYX0KFe]02l_
;`0U9BD092@T02<S8`0[:b/09bLW02TY:@10@400:RXZ028R8P0/;2`0G5aL051@D02`/;00Y:BT09NG
U`1LG5`0:2PX0492@P0f=SH00P0G5aL201HF5P1601@D500M7Ad0CTi>0:RXZ02/[:`0ZJVY0<g=c@2S
Xj<0Zj^[07AdM01aLG40Fe]K05=CD`10@400@d=304i>CP10@400@d=303Lg=`0`<300NWYj07moO`2?
Shl0HV9R0451@@0/;2`0I6AT05iNGP0Z:RX07alO024Q8@0V9RH0:2PX02`/;00_;bl0MWIf09VIV@2_
[jl0NGUi0:ZZZP1/K6`0ATI604M7A`0m?Cd0GEeM07inOP1eMGD0?clo04U9B@1/K6`0X:2P0;>c/`2A
TI40?Shn01/K6`0I6AT06a/K02DU9@0[:b/0<c<c03dm?@1XJ6P0<c<c04E5A@1TI6@0MGEe08R8R02J
VYX0/K6a0<;2`P80H61P00l0NWYj06QXJ01NGUh0QhN70<g=c@30`<00Zj^[0;jn_P2i^KT0SXj>07Ee
M@1HF5P0?clo03<c<`0/;2`00`0^;Rh0C00/;2`0@d=305]KF`1CDe<0R8R80:ZZZP1jNWX0K6a/08j>
SP2ZZZX0KFe]03`l?00];Bd0<30`09^KV`1/K6`0I6AT08N7Q`11@D40=cLg06][J`2VYZH0`L710861
P@15ADD0>SXj04U9B@1JFUX0YZJV0;6a/@2d];@0V9RH0:^[Z`32`/80Ng]k08:2PP3EeMD0HV9R06QX
J02GUiL0^[Zj0:ZZZP1=CDd0BTY:06QXJ02j^[X0clo?0<C4a02e]KD0f][J0=CDe01oOgl0<S8b030`
<00c<c<0A4A406YZJP2=SHd0Shn?075aL@1LG5`0DE5A07MgM`2n_[h0bLW90=[JfP3Kfm/0]kNg07Ee
M@1OGel0DE5A04Q8B012@T80B4Q805=CD`1mOGd205iNGP0_05MGE`11@D40=SHf06i^KP15ADD0<S8b
069RHP32`/80G5aL08V9R@2OWil0RHV907AdM01BDU80D51@04A4A01_Kfl0UiNG04Y:BP0];Bd0:BTY
034a<@10@400DU9B06YZJP2FUYH0_;bl05EEE@2PX:00<c<c02d];@0k>c/0SXj>0::RXP2l_;`0V9RH
05UIF@25QHD0JVYZ09nOW`1mOGd0T92@03Ph>00b<S80>3Ph03@d=00g=cL04?ooool000kooooo0>H0
XZ:R07]kN`2=SHd0GEeM05iNGP1^KVh0;2`/0861P@0m?Cd0Lg=c03@d=01[Jf/0=CDe05UIF@0k>c/0
6a/K02@T900P8200GUiN04m?C`12@T80AdM703/k>`0j>SX0F5QH06IVIP1FEUH0SHf=0451@@0`<300
9bLW02XZ:P1@D500?Shn02d];@0i>CT0JVYZ024Q8@1PH600AdM703hn?P1BDU80E5AD05]KF`1ADE40
9BDU03@d=01]KFd0GEeM04];B`1HF5P0<S8b03Ph>01=CDd0EUIF03<c<`1YJFT0<S8b02d];@1?Cdl0
L71`02PX:015ADD09bLW04A4A01HF5P0<C4a09>CT`0b<S80Bd];092@T01EEED0@d=304m?C`0h>3P0
8B4Q065QH@1_Kfl0CTi>05UIF@1DE5@0:RXZ03lo?`15ADD0?3`l03<c<`1QHF40IFEU03@d=022PX80
IVIV059BDP2;Rh/0@D5105UIF@11@D40Ogmo03Hf=P15ADD0GEeM020P801:BTX0QHF503Hf=P1MGEd0
:BTY06=SH`2ATI40@d=305=CD`0U9BD0A4A405ADE01kNg/0@d=304a<C01OGel0Q8B4055AD@1TI6@0
>3Ph04A4A00j>SX0CDe=05ADE00e=CD0RHV907inOP17AdL0IFEU075aL@1>CTh0IVIV07EeM@0Z:RX0
Rh^;05=CD`1`L700PX:204Q8B01HF5P0RXZ:07EeM@1lO7`0<S8b03Xj>P1WIfL0OGem06e]K@1LG5`0
@d=3071`L022PX80QXJ606QXJ028R8P0QHF506EUI@0];Bd0GEeM07=cL`1kNg/0NGUi09FEU@1JFUX0
bLW90:JVYP0k>c/0RHV905mOG`1:BTX0FUYJ07moO`1<C4`0RXZ:06=SH`1VIVH0I6AT059BDP1NGUh0
DE5A02d];@1QHF40Ng]k09^KV`1=CDd0ATI60;no_`1PH600RHV90861P@0];Bd0<c<c08^;R`1KFe/0
?Cdm07moO`1RHV80HF5Q03hn?P1aLG40LW9b04Y:BP18B4P0RXZ:069RHP1SHf<0LG5a04m?C`28R8P0
S8b<04];B`2IVIT0M7Ad03De=@1ADE40OGem092@T01DE5@0A4A4071`L01FEUH0W9bL08F5Q@1NGUh0
?3`l08N7Q`19BDT0Fe]K069RHP80FEUI02h0EeMG04E5A@1cLg<0Hf=S04i>CP0Z:RX0KVi^05eMG@1/
K6`0IfMW0820P01eMGD0@41006QXJ029RHT0Cdm?08:2PP27QhL0Bd];02d];@1/K6`0QHF505EEE@1a
LG40<c<c03`l?01dM7@0?Cdm06EUI@1ZJVX0;Rh^07UiN@2TY:@0QhN703lo?`0];Bd0GUiN02h^;P0l
?3`0KFe]0820P022PX80DU9B02l_;`0L71`0OWin3_ooool0CP1TI6@0?Shn03<c<`0h>3P09bLW01hN
7P0G5aL071`L09RHV01nOWh09bLW01dM7@0F5QH061PH01/K6`0P8200:BTY02l_;`1KFe/0Rh^;09NG
U`2UYJD0];Bd07YjNP1oOgl0LW9b030`<01_Kfl0Hf=S02TY:@0P82008R8R020P800V9RH0<c<c0492
@P1ZJVX0YjNW09NGU`1CDe<0>3Ph02PX:00V9RH09BDU02HV9P0P8200@d=302/[:`0K6a/07QhN06EU
I@2ATI40_kno0=7Ad@2e]KD0X:2P051@D00[:b/0B4Q801PH600C4a<051@D01`L7010@400FUYJ04E5
A@13@d<0PH6109jNWP2LW9`0YjNW0;^k^`1eMGD0?clo01lO7`0X:2P0;Rh^02d];@<09BDU02P07alO
02`/;01_Kfl0L71`075aL@25QHD0CDe=0410@019BDT0Ti>C05eMG@0a<C40;2`/02l_;`0g=cL0FEUI
06IVIP2VYZH0X:2P0;^k^`37alL0_Kfm08V9R@1WIfL0GEeM04m?C`2BTY80MgMg07IfMP1NGUh0GEeM
07YjNP2>SXh0Y:BT0;6a/@1XJ6P092@T01TI6@0H61P08B4Q0P0N7Qh02@0R8R80G5aL04i>CP0T92@0
>c/k09:BTP2LW9`0DE5A06i^KP0209fMW@1D05ADE013@d<0>SXj0410@02c/k<0clo?0<K6aP2h^;P0
]kNg0:n_[`1lO7`0C4a<03Ti>@0f=SH0<c<c03De=@0j>SX0?3`l03Xj>P0e=CD0BDU908B4Q02i^KT0
KVi^0:NWY`2VYZH0eMGE0;jn_P2i^KT0`L710;jn_P2?Shl0/;2`07EeM@1RHV80R8R807EeM@0k>c/0
7QhN02l_;`1>CTh0OGem09bLW01>CTh0?Shn059BDP1<C4`0_Kfm05QHF02d];@0TI6A0:6QX@31`L40
b<S809nOW`34a<@0KFe]09RHV02d];@0X:2P06MWI`0i>CT0B4Q806=SH`1KFe/0_[jn0<71`@3MgMd0
_;bl0=;BdP26QXH0;bl_030`<00d=3@0?Cdm04A4A015ADD0AdM705UIF@1]KFd0LW9b0;Jf]P2d];@0
]kNg0P3>c/h0>P2ZZZX0EEEE055AD@1YJFT0BTY:03dm?@0c<c<0Hf=S069RHP0n?Sh0=SHf0410@019
BDT0Bd];071`L01NGUh0@T9204m?C`2a/K40QHF505=CD`2GUiL0V9RH05]KF`11@D40ATI60451@@1o
Ogl0/;2`04E5A@0n?Sh0<C4a03@d=01DE5@0L71`0<;2`P38b<P0b/[:09RHV032`/80MGEe05=CD`1k
Ng/0X:2P05mOG`2?Shl0KVi^05]KF`1/K6`0H61P07UiN@19BDT0PX:206QXJ00b<S80=SHf03Lg=`0e
=CD@ooooo`003_ooool05`2XZ:P0J6QX06ATI01MGEd0JVYZ05QHF00V9RH0L71`0451@@1=CDd06ATI
05YJFP0k>c/0Cdm?03Lg=`0?3`l0:b/[02LW9`1CDe<0Bd];02LW9`11@D40Bd];0080?3`l02/0>c/k
04A4A023Ph<0@D5103/k>`0Z:RX0;Rh^04M7A`14A4@0:b/[02DU9@1]KFd07AdM05YJFP0n?Sh0<c<c
04A4A00l?3`0EUIF069RHP0d=3@09bLW05UIF@0l?3`0?clo08R8R01bLW80?Cdm04Y:BP1@D500<30`
06QXJ01ZJVX0>c/k0410@01ZJVX0<30`055AD@0m?Cd0EeMG05=CD`0[:b/0Ogmo0080@4100=00Jf][
03De=@18B4P0GUiN04m?C`0];Bd0Ph>306e]K@0^;Rh0Fe]K071`L01CDe<0ATI603lo?`0];Bd07QhN
03Lg=`1YJFT0@D5109JFUP1KFe/0A4A408j>SP1DE5@0H61P0410@01bLW80<c<c04e=C@1JFUX0?3`l
03@d=01/K6`0@T9206e]K@13@d<0DE5A079bLP0k>c/0GUiN0451@@1=CDd0DU9B05mOG`0k>c/0JFUY
05ADE01hN7P0B4Q808F5Q@12@T80FEUI03Lg=`1LG5`0MgMg051@D02KVi/0OGem03`l?01VIVH0Lg=c
04Q8B01^KVh0MgMg04Q8B02GUiL0CDe=07inOP29RHT0L71`03`l?01UIFD0K6a/07UiN@20P800IVIV
06][J`1mOGd0Ng]k09NGU`2=SHd0L71`06MWI`26QXH0Cdm?0861P@2KVi/0UYJF04=3@`1gMgL0GEeM
06YZJP1IFET0PH6105UIF@31`L40S8b<06YZJP27QhL0Hf=S04];B`1@D500JFUY04M7A`2QXJ40KVi^
05eMG@1XJ6P0Bd];04M7A`1JFUX0:RXZ05QHF01`L700Zj^[05eMG@12@T80SHf=05iNGP1RHV80IfMW
03Xj>P0S8b<0SXj>05MGE`10@400KVi^04a<C01:BTX0?Cdm06a/K01YJFT0CDe=04E5A@1oOgl0K6a/
06][J`1bLW80?clo06UYJ@2@T900FUYJ0820P01[Jf/0@d=303Ti>@1HF5P0P82005]KF`11@D40MGEe
06IVIP2GUiL0LW9b05mOG`0d=3@0U9BD04a<C01EEED0>3Ph04e=C@1ZJVX0Cdm?04Q8B01bLW80BTY:
03<c<`0_;bl0Ph>306EUI@1kNg/0K6a/08N7Q`1KFe/0>3Ph07EeM@2;Rh/0Bd];08V9R@1RHV80:BTY
03/k>`1`L700PH6105IFEP1eMGD0;Rh^03hn?P27QhL0@41005iNGP1[Jf/0>SXj079bLP2MWId0HF5Q
03Hf=P0];Bd0GUiN07EeM@15ADD0N7Qh08F5Q@1cLg<0EEEE051@D00U9BD0Kfm_3_ooool03P2NWYh0
NWYj05YJFP1@D500CDe=05YJFP1dM7@0TY:B0:6QX@2YZJT0Q8B405UIF@0X:2P061PH0P0F5QH04@0Q
8B4092@T02`/;01SHf<0TI6A09VIV@1XJ6P0b/[:0<71`@24Q8@0RHV9069RHP11@D40;Bd]02<S8`0Q
8B407alO00808B4Q02`09BDU02l_;`1RHV80UIFE09:BTP1KFe/0BTY:03De=@0Y:BT0=3@d03Ti>@17
AdL0De=C04a<C01LG5`0LW9b09jNWP2[Zj/0TY:B08:2PP1nOWh0Lg=c03dm?@1dM7@0>c/k03`l?00c
<c<0=3@d02`/;00f=SH0@T9204A4A00l?3`0KVi^09BDU028R8P0SXj>07]kN`1VIVH0JFUY05mOG`0f
=SH07QhN01lO7`<071`L07P0<c<c06UYJ@1FEUH0HF5Q05]KF`2FUYH0XJ6Q09bLW02HV9P0R8R805=C
D`1JFUX0FEUI04M7A`1NGUh0HV9R05MGE`28R8P0]kNg0:ZZZP2WYjL0SHf=06][J`1MGEd0EeMG06=S
H`2LW9`0UIFE06][J`1mOGd0Rh^;0;Vi^@2k^k/0WYjN09BDU01:BTX09RHV01XJ6P0M7Ad07QhN01lO
7`0R8R80:BTY04=3@`0m?Cd0>CTi0861P@2QXJ40ZJVY07inOP1QHF40SXj>0451@@1EEED0Kfm_07Ad
M02_[jl0QHF50;Fe]@2][Jd0O7al04=3@`1@D500QhN707inOP1>CTh0@T920451@@10@400>c/k03Lg
=`0j>SX0@4100:ZZZP2DU9@0Kfm_06QXJ02GUiL0YZJV079bLP1<C4`0E5AD055AD@22PX80`l?30::R
XP21PH40=3@d02PX:00e=CD0?Shn03`l?00g=cL0EEEE09>CT`2LW9`0PH6107EeM@27QhL0K6a/04i>
CP1fMWH0>SXj04a<C01RHV80?Shn09jNWP3@d=00b<S80;Rh^02CTi<0TY:B04I6AP0i>CT0CDe=04M7
A`0n?Sh0];Bd09fMW@3Ng]h20::RXP0@0=7Ad@2WYjL0MGEe06MWI`1VIVH0Hf=S05UIF@1NGUh0L71`
09JFUP2GUiL0TY:B0:f][@28R8P0[jn_0:NWY`80NWYj03H0Cdm?03hn?P12@T80?Shn05=CD`1WIfL0
?3`l03Hf=P0a<C40<c<c03/k>`18B4P0Jf][08>3P`1_Kfl0WinO07AdM01SHf<0RHV90<?3``30`<00
Ti>C07AdM01TI6@0Q8B408F5Q@1WIfL0X:2P06YZJP0b<S80@T9209NGU`38b<P0e=CD0=oOg`3GemL0
`l?30;Bd]02_[jl0V9RH09FEU@2YZJT0[:b/08R8R01/K6`0LG5a09:BTP2@T900@d=304I6AP1CDe<0
TI6A051@D00b<S8203Ti>A3ooooo000>ooooo`1Y0::RXP1dM7@0JFUY05MGE`1=CDd0FUYJ02h^;P18
B4P09RHV059BDP0F5QH0ATI603Ti>@1:BTX0G5aL01/K6`0e=CD0820P034a<@1JFUX0CDe=04E5A@0j
>SX0;Bd]05EEE@0o?cl0CDe=09ZJVP0c<c<0CDe=069RHP0Z:RX04a<C030`<01EEED08R8R06EUI@0E
5AD0E5AD05iNGP0e=CD0?Shn02d];@15ADD0Ng]k04i>CP0Y:BT0AdM705YJFP0i>CT0PH6107emO@1@
D500EeMG05ADE01OGel0D51@06][J`0g=cL0EEEE04i>CP0Q8B40@41004=3@`1>CTh0AdM702<S8`1]
KFd0CDe=04m?C`1XJ6P0MgMg0410@01EEED0LG5a03Ph>02OWil0R8R805]KF`1HF5P0GUiN03hn?P1B
DU80LW9b05=CD`0V9RH0;Bd]05]KF`0S8b<0Rh^;07QhN01>CTh0TY:B02PX:01IFET0Cdm?06YZJP0S
8b<0>SXj05IFEP1EEED0?clo06=SH`0`<300Jf][0080DU9B04@0PX:203dm?@1ADE40:RXZ04U9B@1U
IFD0Ng]k04U9B@1FEUH0ATI607emO@1;Bd/0FUYJ030`<01RHV80B4Q805IFEP1?Cdl0=3@d08F5Q@2U
YJD0KFe]05IFEP1gMgL0GemO06QXJ01YJFT0EEEE08N7Q`1mOGd0QXJ607YjNP2EUID0>CTi06ATI01e
MGD0P82009nOW`1eMGD0Lg=c096AT@2?Shl0U9BD08Z:RP1^KVh0R8R809bLW02:RXX0Rh^;06e]K@1o
Ogl0?Cdm07alO01UIFD0U9BD05mOG`2:RXX0ATI60:6QX@1iNGT0IfMW07moO`1VIVH0Cdm?05QHF02<
S8`0;bl_08B4Q080KFe]06D0OGem05=CD`11@D40IVIV028R8P1jNWX0IVIV0:JVYP1TI6@0AdM707Ee
M@1/K6`0HF5Q06QXJ00d=3@0;Rh^08Z:RP2LW9`0HV9R06QXJ02XZ:P0AdM7024Q8@0o?cl0EEEE04U9
B@1?Cdl0MgMg03@d=01NGUh0M7Ad051@D01LG5`0OWin04a<C023Ph<0VYZJ05aLG010@400Cdm?0820
P00j>SX0DE5A07MgM`1YJFT0][Jf08V9R@0Q8B40:b/[07MgM`1EEED0IfMW02l_;`0];Bd0@41005IF
EP0d=3@0MWIf07alO00m?Cd0820P05UIF@1TI6@0B4Q807AdM02KVi/0C4a<02DU9@1:BTX0Lg=c03/k
>`2>SXh0MWIf03hn?P0m?Cd0E5AD08N7Q`1BDU80Kfm_02XZ:P0k>c/0RXZ:04a<C01VIVH0HF5Q04Y:
BP2IVIT0WYjN07emO@16ATH0:2PX04U9B@0/;2`0GUiN0820P01`L700S8b<06UYJ@14A4@06ATI06IV
IP0>ooooo`0A0<C4a02/[:`0GemO04i>CP1_Kfl0MWIf06a/K02@T900?Cdm02HV9P2JVYX0_;bl07If
MP1GEeL0=SHf01TI6@0H61P00P0N7Qh04@0R8R80P82006m_K`1[Jf/0MGEe0<O7a`2EUID0ATI603Ph
>00f=SH0<C4a028R8P0N7Qh0820P01lO7`0M7Ad07QhN00807AdM02`0:b/[096AT@2LW9`0CTi>06AT
I02:RXX0IFEU05=CD`1HF5P0Ti>C06UYJ@0n?Sh0B4Q809VIV@1gMgL0NGUi07AdM01`L700^;Rh0:j^
[P2o_kl0OGem02HV9P0];Bd0@d=304Y:BP14A4@0;2`/01lO7`0N7Qh0D51@061PH029RHT0V9RH07=c
L`10@400?clo04];B`1/K6`0IFEU04A4A00f=SH0:b/[02DU9@80Cdm?02@0@D5104];B`14A4@0G5aL
08N7Q`2VYZH0`/;205YJFP0l?3`0BDU904i>CP1LG5`0SXj>03`l?010@400Ph>306YZJP2ATI40T92@
08F5Q@1oOgl0>CTi02/[:`0];Bd0@d=307UiN@1ADE40EEEE04=3@`1PH600`/;205mOG`2@T900PX:2
04Q8B00R8R8201dM7@807alO00<0820P02@T901dM7@00P22PX808@2UYJD0VYZJ0:n_[`2PX:00PH61
0:BTY02l_;`0>CTi03Lg=`2c/k<0QhN7069RHP16ATH0LG5a04e=C@0a<C40>CTi0;jn_P2MWId0GemO
051@D011@D40?3`l03Ti>@1ADE40YjNW09VIV@1<C4`0E5AD07UiN@1XJ6P0Zj^[07]kN`0202d];@05
034a<@25QHD0L71`09:BTP1MGEd00P0N7Qh05`0Q8B407AdM02`/;01CDe<0Ti>C09fMW@2VYZH0/K6a
0<k>cP28R8P0>CTi05MGE`1NGUh0<30`04i>CP2>SXh0IFEU0:NWY`3Lg=`0emOG0<g=c@2EUID0];Bd
0080JVYZ0580RHV907moO`2QXJ40bLW90<;2`P39bLT0S8b<055AD@2f][H0Ph>301lO7`0P82007alO
028R8P0Q8B409BDU02/[:`2GUiL0P82004Q8B01HF5P0Kfm_09RHV02:RXX0LW9b04A4A02:RXX0De=C
02`/;00n?Sh0PH61075aL@0m?Cd0=SHf03Ti>@0c<c<0=SHf04A4A01/K6`0De=C092@T01_Kfl0R8R8
0;Zj^P2>SXh0XJ6Q07emO@1`L700T92@0<O7a`27QhL0F5QH06=SH`2_[jl0d];B0<30`02e]KD0c/k>
0<S8b03<c<`0eMGE0=SHf02j^[X0UYJF08^;R`2NWYh0/k>c0;2`/02QXJ40QHF508^;R`2YZJT0`L71
07alO00U9BD0<c<c05aLG02a/K40`L7108B4Q01OGel0YjNW4?ooool000kooooo0?l0NWYj05aLG02_
[jl0MgMg051@D01`L700?3`l065QH@18B4P0Cdm?01`L700b<S807QhN030`<01EEED071`L034a<@0?
3`l06QXJ04];B`11@D40?Shn03@d=00Q8B40CDe=0410@01OGel0YZJV0451@@0b<S80Hf=S030`<00i
>CT0;bl_0410@00R8R80QHF502PX:01ADE40GemO04=3@`1GEeL0IVIV0492@P1]KFd0GEeM030`<01E
EED0P82004a<C02:RXX0?clo038b<P1=CDd0F5QH06][J`1>CTh0FEUI02XZ:P1/K6`0NWYj0410@014
A4@0;Bd]038b<P1FEUH0>CTi06i^KP0c<c<0@D5107QhN01iNGT0@410079bLP0f=SH06ATI07inOP20
P800ATI6061PH023Ph<0@d=3030`<00n?Sh0@d=302DU9@0i>CT0MGEe03@d=01iNGT0L71`04A4A02S
Xj<0G5aL04A4A00m?Cd0Lg=c05mOG`14A4@0DE5A03dm?@0c<c<0KFe]04e=C@1[Jf/0EeMG03dm?@1k
Ng/0IFEU05EEE@17AdL0@d=3059BDP22PX80HF5Q06m_K`15ADD0R8R805ADE01ZJVX0E5AD05EEE@18
B4P0EEEE0820P01FEUH0Q8B40:>SX`1dM7@0Bd];09FEU@1XJ6P0L71`06a/K01NGUh0Rh^;092@T01c
Lg<0NWYj09nOW`11@D40IFEU092@T01jNWX0R8R806MWI`1dM7@0RHV907UiN@2TY:@0PH6107AdM02d
];@0T92@08J6QP2CTi<0NWYj04m?C`15ADD0PX:2092@T02>SXh0GemO07AdM00Y:BT0PX:208Z:RP1P
H600O7al04U9B@1LG5`0OWin08R8R00[:b/0IfMW03lo?`1cLg<0QhN7051@D010@400E5AD02/[:`1m
OGd0G5aL09jNWP1NGUh07QhN05iNGP1ZJVX0JFUY075aL@0/;2`06QXJ07=cL`1fMWH0<S8b05UIF@1L
G5`0AdM703lo?`1FEUH0K6a/05iNGP10@400LW9b06m_K`1IFET0H61P04U9B@1hN7P0IVIV02HV9P12
@T80H61P06=SH`1>CTh0IVIV096AT@1>CTh0=cLg05mOG`1@D500UIFE08Z:RP1KFe/0=cLg079bLP1@
D500Bd];03lo?`11@D40F5QH04m?C`0/;2`0GemO059BDP17AdL0:BTY06e]K@1fMWH0LG5a06][J`2:
RXX0KVi^0451@@2>SXh0ZJVY075aL@2;Rh/0J6QX02XZ:P1ADE40P82009fMW@1]KFd001L0VIVI034a
<@0R8R80JFUY05YJFP1?Cdl0SHf=034a<@1[Jf/0MGEe06ATI018B4P0:2PX04Y:BP1LG5`0=3@d061P
H00m?Cd0Ng]k06MWI`1ZJVX0?3`l05eMG@0>ooooo`0Q0=kNgP39bLT0^[Zj092@T01gMgL0AdM703hn
?P1JFUX0K6a/028R8P0[:b/0U9BD0:ZZZP1@D500EeMG05YJFP1>CTh0FUYJ055AD@12@T80YJFU09^K
V`1]KFd0Jf][0;:b/P1_Kfl0DU9B03<c<`0];Bd0:BTY02HV9P0Q8B407AdM0080820P03T09bLW020P
800L71`07QhN02h^;P2SXj<0Zj^[09jNWP2TY:@0FUYJ04E5A@1@D500Ph>307QhN00l?3`0=CDe06IV
IP1oOgl0O7al08b<S02BTY80d=3@09:BTP2QXJ40/k>c09BDU02HV9P0Fe]K03Xj>P0Y:BT0820P01`L
700O7al0?Shn06IVIP29RHT0W9bL079bLP1IFET0@410024Q8@0];Bd0CDe=05IFEP1PH600M7Ad07mo
O`2i^KT0>SXj030`<00a<C40>SXj08N7Q`1KFe/0WIfM0::RXP1KFe/00`0n?Sh0F@0l?3`0Jf][05YJ
FP1GEeL0QHF504=3@`1WIfL0JFUY08R8R01iNGT0UYJF071`L015ADD0@T92065QH@1ADE40@41003Hf
=P10@400Shn?04M7A`1UIFD0ZJVY04U9B@10@4009BDU01dM7@0N7Qh07AdM01hN7P0P8200GemO0;Vi
^@2a/K40UiNG0;bl_03Jf]X0`L710;fm_@2QXJ40SXj>092@T02JVYX0dM7A0:VYZ@0f=SH0>3Ph06AT
I01GEeL0?3`l02XZ:P2@T900V9RH05MGE`1PH600KFe]07QhN02ATI40][Jf0:^[Z`1EEED0EUIF09>C
T`2EUID0LW9b0;Ng]`2f][H0BTY:03@d=016ATH0ZZZZ08B4Q01MGEd0Kfm_01hN7P0M7Ad07QhN01lO
7`0S8b<09BDU08F5Q@2IVIT0[Zj^096AT@2h^;P0d=3@0:NWY`2][Jd0WinO0080S8b<02<0bLW90=3@
d02^[Zh0c<c<0:>SX`2l_;`0^k^k06UYJ@0b<S809RHV02LW9`0[:b/0?Cdm08^;R`2:RXX0[:b/08n?
S`1<C4`0W9bL06m_K`0M7Ad071`L01hN7P0O7al08B4Q028R8P0T92@0TY:B07MgM`1LG5`0Ti>C09:B
TP2FUYH0T92@07MgM`0202TY:@0@07]kN`28R8P0Ng]k08V9R@1YJFT0A4A404E5A@17AdL0@41003Ph
>013@d<0DE5A05YJFP1gMgL0JVYZ0::RXP80a/K602D0Z:RX069RHP2DU9@0/[:b0;Fe]@1lO7`0IVIV
07MgM`37alL0g=cL0=oOg`3Ng]h0c<c<0<71`@2SXj<0b/[:0>7Qh@1jNWX0MWIf079bLP27QhL0W9bL
0:b/[01ZJVX0Cdm?05eMG@1VIVH0M7Ad0;Fe]@1NGUh0Ogmo09JFUP1aLG40GUiN09nOW`3@d=00Vi^K
013ooooo000>ooooo`2:07MgM`2OWil0d=3@08V9R@1PH600CTi>02/[:`1^KVh0K6a/05YJFP0R8R80
CTi>051@D00`<300Cdm?00T92@0o?cl08b<S01/K6`1GEeL0:RXZ03dm?@1MGEd0<c<c04=3@`0[:b/0
E5AD09FEU@1VIVH0>SXj03/k>`0Z:RX0H61P0451@@0_;bl0410@06IVIP0X:2P0EeMG034a<@0W9bL0
ATI606UYJ@1;Bd/0EEEE01DE5@0L71`0A4A405mOG`0h>3P0I6AT05=CD`11@D40ATI604Y:BP17AdL0
De=C07=cL`0Y:BT0G5aL05YJFP1CDe<0@D5104];B`0`<300IFEU04=3@`1GEeL0Lg=c04E5A@1hN7P0
KFe]0451@@1lO7`0HV9R03<c<`2IVIT0Ph>303De=@0k>c/0EUIF05mOG`0l?3`0@d=303Lg=`0I6AT0
8B4Q06=SH`1:BTX0SXj>079bLP0g=cL0WYjN06QXJ00h>3P0D51@07MgM`1mOGd0@D5104U9B@1kNg/0
A4A407=cL`1[Jf/0GemO07UiN@15ADD0Ogmo06][J`1HF5P0E5AD04U9B@1;Bd/0LG5a06EUI@1[Jf/0
@d=308R8R01QHF40IVIV05QHF018B4P0EUIF05aLG025QHD0Jf][079bLP2PX:00Ogmo04i>CP2XZ:P0
JFUY06][J`1_Kfl0CTi>07UiN@1aLG40Fe]K0P27QhL03P1?Cdl0Shn?0::RXP1UIFD0OWin04e=C@1k
Ng/0OGem06QXJ02@T900>SXj0492@P1bLW80OWin0P1kNg/0>02:RXX0M7Ad04I6AP1LG5`0Hf=S07Ui
N@1MGEd0Ti>C0451@@2GUiL0RHV904Y:BP23Ph<0Bd];038b<P0n?Sh0I6AT03Xj>P1`L700JFUY05]K
F`24Q8@0AdM705QHF01RHV808b<S06ATI01<C4`0YjNW079bLP0f=SH0ATI605=CD`1DE5@0L71`05IF
EP0M7Ad0NWYj08R8R016ATH0G5aL079bLP1BDU80EEEE06][J`1dM7@0Kfm_04E5A@1WIfL0O7al06i^
KP1UIFD0DU9B061PH02@T900I6AT0P1VIVH0@01JFUX0=3@d04i>CP2ATI40MGEe03Lg=`1JFUX0IfMW
09RHV02EUID0OGem04E5A@1`L700MWIf04e=C@1>CTh0ATI606][J`1cLg<0EEEE05eMG@1RHV80<30`
03hn?P25QHD0UiNG08f=S@1fMWH0Vi^K05IFEP0[:b/0BTY:09BDU01FEUH0X:2P09>CT`1<C4`0<C4a
0451@@1nOWh0ADE507inOP1BDU80;Rh^06MWI`11@D40<S8b08^;R`1:BTX0S8b<09ZJVP1ADE40>SXj
04M7A`1PH600Fe]K06YZJP2=SHd0?Shn07YjNP1>CTh0K6a/04E5A@1FEUH>ooooo`0M0=3@d02m_Kd0
[jn_07=cL`1DE5@0DE5A04Q8B016ATH0OWin02DU9@0O7al0<C4a0:n_[`2OWil0P82007QhN022PX80
TY:B09RHV02`/;00clo?0<g=c@2][Jd0SXj>0:NWY`27QhL0`<3004=3@`0X:2P00P0U9BD01P0S8b<0
71`L01`L700L71`0820P01lO7`<07AdM0580E5AD0:j^[P2DU9@0YjNW07IfMP1hN7P0XZ:R08N7Q`2;
Rh/0Ph>309nOW`2[Zj/0WinO0:j^[P30`<00ZZZZ0:2PX034a<@0f=SH0<c<c03:b/X0TI6A0492@P0S
8b<07AdM02l_;`0c<c<0@D5105MGE`1BDU80>SXj0410@01IFET0LW9b05eMG@17AdL092@T02LW9`0b
<S80=cLg034a<@0g=cL0NWYj092@T00_;bl08b<S06QXJ01[Jf/0CDe=0861P@1iNGT0;Bd]02XZ:P0X
:2P0:b/[03<c<`1KFe/0[jn_0<30`01EEED0<S8b05=CD`1EEED0C4a<059BDP2/[:`0Ogmo030`<00Y
:BT0C4a<05aLG00h>3P0:2PX02HV9P1dM7@0?Shn04Q8B01kNg/0O7al03hn?P0h>3P0:RXZ0P0N7Qh2
028R8P0@05mOG`2SXj<0DE5A04E5A@1eMGD0^;Rh0;Fe]@1mOGd0IFEU06MWI`1_Kfl0Ph>30<O7a`2Y
ZJT0?3`l02`/;080FEUI00X0ADE503lo?`1jNWX0BTY:02DU9@0R8R809BDU02PX:01=CDd0ZJVY0P1>
CTh05@21PH40H61P09>CT`2a/K40a/K60<g=c@2MWId0X:2P0<;2`P3Fe]H0^;Rh03`l?01`L7008b<S
028R8P0P82008R8R02<S8`0U9BD0MgMg09>CT`0208b<S00M0:FUY@39bLT0Cdm?07UiN@2LW9`0IVIV
0=CDe03IfMT0/;2`0:b/[035aLD0WYjN0<S8b02VYZH0Rh^;06i^KP0S8b<0820P01hN7P0T92@0K6a/
06IVIP2CTi<0][Jf059BDP20P800GemO01dM7@0L71`00P0M7Ad0A00O7al07QhN02/[:`2YZJT0K6a/
07=cL`2YZJT0OWin09JFUP2UYJD0RXZ:020P800N7Qh09RHV07IfMP3Kfm/0aLG509jNWP1hN7P0=SHf
02l_;`0c<c<0@d=304a<C015ADD0De=C05iNGP1XJ6P0Kfm_0:f][@1QHF40PX:20<o?c`34a<@0d];B
07UiN@2GUiL0QXJ60<30`03:b/X0aLG50<C4a03EeMD0e=CD09ZJVP1VIVH0e=CD0=[JfP30`<00JVYZ
03Lg=`16ATH0DE5A05YJFP24Q8@0BTY:04U9B@18B4P0G5aL0<K6aP2RXZ80H61P03dm?@0i>CT0?clo
05QHF02b/[80JFUY4?ooool000kooooo00/0F5QH079bLP3Cdm<0RHV905MGE`1HF5P0:BTY04i>CP1=
CDd0>SXj01@D50020451@@1302TY:@1HF5P07alO03Ti>@0>3Ph04a<C06e]K@0n?Sh09RHV03Ph>00Y
:BT0FUYJ02XZ:P1:BTX0Q8B406IVIP0n?Sh0Kfm_03@d=017AdL0;Bd]051@D00U9BD0Jf][01`L7017
AdL0F5QH03<c<`0i>CT0?Cdm04];B`1iNGT0GemO02<S8`0g=cL0EEEE03Ph>01hN7P0LW9b05MGE`1?
Cdl0@d=306QXJ017AdL0Ph>303Hf=P1SHf<0ADE502`/;00l?3`0IFEU03Xj>P1WIfL0@410061PH025
QHD0Cdm?07YjNP1;Bd/0;2`/0861P@1VIVH0>CTi08Z:RP25QHD0IFEU04];B`0205UIF@2:0451@@15
ADD0B4Q802/[:`0/;2`0Jf][05eMG@21PH40Xj>S04I6AP2IVIT0J6QX03@d=01FEUH0M7Ad08^;R`13
@d<0B4Q808b<S016ATH0L71`06m_K`1XJ6P0QhN704I6AP1jNWX0H61P06IVIP1PH600D51@04Q8B01Q
HF40@d=307MgM`0l?3`0O7al02`/;01VIVH0FUYJ03Xj>P0o?cl0CTi>07YjNP1MGEd0G5aL0:f][@1M
GEd0CTi>07moO`1RHV80FUYJ08N7Q`0o?cl0I6AT05eMG@1ADE40UiNG07UiN@0f=SH0De=C07YjNP1J
FUX0T92@05IFEP1SHf<0Kfm_05iNGP2h^;P0Shn?06MWI`2e]KD0Xj>S075aL@1dM7@0Zj^[07moO`1>
CTh0T92@07IfMP1eMGD0Cdm?08J6QP1<C4`0XZ:R0:2PX01lO7`0S8b<06][J`15ADD0@D51061PH00e
=CD0HF5Q07=cL`1QHF40S8b<04];B`0k>c/0IVIV02l_;`1FEUH0<C4a08J6QP22PX80@D5103dm?@1]
KFd0E5AD08B4Q01aLG407alO065QH@2`/;00FEUI06e]K@1eMGD0B4Q805EEE@1oOgl0O7al055AD@0n
?Sh0JVYZ069RHP1dM7@0V9RH04i>CP1_Kfl0ZJVY07alO01jNWX0MGEe069RHP0j>SX0G5aL09^KV`23
Ph<0?Cdm0P1/K6`0>02=SHd0WIfM06IVIP0n?Sh0Shn?07QhN01DE5@0CDe=02<S8`14A4@0EEEE06][
J`1dM7@0P82004e=C@0_;bl0GEeM07IfMP1:BTX0HF5Q09JFUP29RHT0=cLg051@D02/[:`0L71`08F5
Q@28R8P0MWIf04=3@`0o?cl0Ti>C05aLG027QhL0JVYZ03Ph>01FEUH0Hf=S04=3@`2ZZZX0GUiN08Z:
RP2EUID0IFEU05ADE00U9BD0@d=304Q8B01@D500Vi^K04U9B@23Ph<0GemO05UIF@0f=SH0Bd];3_oo
ool0>P3Bd]80a<C409fMW@1eMGD0EUIF05EEE@1@D500JVYZ09BDU00/;2`07AdM02@T902LW9`0bl_;
0:VYZ@20P800OWin06YZJP1QHF40MGEe05aLG028R8P0`L710;Fe]@26QXH0KVi^0<g=c@26QXH0IfMW
04Q8B00S8b<09BDU020P800N7Qh06a/K01`L700O7al0;Rh^01hN7P0Q8B40GEeM0:n_[`1_Kfl0Ogmo
04=3@`0l?3`0U9BD0;:b/P3;bl/0/k>c05eMG@2FUYH0KVi^07EeM@26QXH0RXZ:07=cL`2ZZZX20<C4
a00E0=GEe@1nOWh0IfMW02HV9P10@400AdM705MGE`14A4@09bLW04M7A`0c<c<0<C4a03/k>`0a<C40
FUYJ06=SH`1;Bd/09RHV02/[:`0/;2`0=cLg0080=CDe02P0TI6A09jNWP1fMWH0JFUY03Xj>P1?Cdl0
L71`07AdM00U9BD09bLW02DU9@0S8b<09RHV03<c<`1oOgl0[:b/07=cL`0Z:RX0@D5104Y:BP16ATH0
A4A408:2PP1fMWH0;2`/02@T900f=SH0K6a/04];B`0n?Sh09BDU08J6QP12@T80:BTY04Y:BP2>SXh0
Hf=S03hn?P0i>CT08B4Q0P0M7Ad09@0P8200Hf=S09^KV`1HF5P0Fe]K0:FUY@1CDe<0SHf=07IfMP16
ATH0?3`l03dm?@0j>SX0FEUI0;6a/@1lO7`0=3@d06EUI@1?Cdl0DU9B04U9B@2=SHd0?clo028R8P0O
7al092@T024Q8@0k>c/0S8b<04A4A01dM7@0CDe=04a<C01>CTh0TY:B0<o?c`34a<@00P1XJ6P08022
PX80]kNg09nOW`15ADD0MGEe02h^;P0Q8B408R8R030`<00d=3@0E5AD04];B`1fMWH0L71`06][J`1f
MWH0WIfM06MWI`2>SXh0QXJ60;bl_02^[Zh0cLg=08b<S02m_Kd0c/k>0<W9b@2>SXh0JFUY06EUI@1W
IfL0FEUI0`0N7Qh02@1ZJVX0F5QH09RHV031`L40TI6A06=SH`1eMGD07AdM01`L700301dM7@1301hN
7P0i>CT0Z:RX06e]K@1cLg<0IfMW07]kN`2EUID0UYJF04m?C`0Q8B407QhN01lO7`11@D40VIVI04E5
A@1LG5`0VIVI04e=C@0a<C40>SXj05mOG`1RHV80ADE501hN7P0a<C40CTi>06=SH`2IVIT0ADE506a/
K03Bd]80e]KF0<;2`P1XJ6P0XJ6Q0=GEe@30`<00SHf=0<?3``2VYZH0TI6A0;bl_039bLT0_[jn0:VY
Z@1XJ6P0Ng]k0;bl_027QhL0>CTi04];B`1EEED0GUiN065QH@1:BTX0?Cdm08>3P`39bLT0T92@02/[
:`0/;2`0<c<c03Ph>018B4P0QHF508j>SP0@ooooo`003_ooool0M`1DE5@0@T920:6QX@2OWil0CDe=
06][J`1BDU80@d=303/k>`1BDU80:2PX0451@@1>CTh08B4Q05=CD`0X:2P0?clo01TI6@0E5AD0C4a<
03@d=00P8200;bl_028R8P1=CDd0<S8b06ATI02<S8`0Bd];02TY:@1YJFT0<30`03/k>`0c<c<0@d=3
02HV9P1/K6`0?clo04=3@`1^KVh0=SHf02d];@1BDU80ATI606m_K`24Q8@0=SHf02`/;01@D500=CDe
07AdM01IFET0EeMG055AD@16ATH0P82004U9B@2MWId0A4A404U9B@1ADE40JFUY038b<P1bLW80>3Ph
06=SH`0e=CD0=cLg08n?S`1=CDd0MWIf06m_K`17AdL0JVYZ05UIF@0i>CT0S8b<08B4Q01/K6`0AdM7
04Q8B00l?3`0AdM7071`L01RHV80=3@d02d];@1ZJVX0ATI607AdM02SXj<0De=C07emO@19BDT0<c<c
05QHF01mOGd0Cdm?03@d=01HF5P0NWYj04A4A01PH600<C4a069RHP1_Kfl0@d=307EeM@10@400B4Q8
05MGE`14A4@0C4a<06=SH`18B4P0H61P03/k>`1iNGT0De=C0080F5QH01T0=cLg04=3@`19BDT0K6a/
05iNGP17AdL0Zj^[0820P01:BTX0P82004e=C@1;Bd/0S8b<06=SH`1cLg<0J6QX04E5A@1fMWH0OWin
05UIF@1ZJVX0M7Ad059BDP2;Rh/0JVYZ0080MGEe0880CDe=0:ZZZP2ATI40Hf=S0;Rh^02?Shl0J6QX
08:2PP2MWId0N7Qh05EEE@2<S8`0Ogmo09^KV`1PH600WYjN05]KF`2FUYH0_Kfm06MWI`2;Rh/0JVYZ
05QHF017AdL0MGEe04A4A01NGUh0MWIf05eMG@2OWil0IVIV04A4A01PH600=cLg05eMG@1ADE40QHF5
08B4Q01;Bd/0=SHf07=cL`19BDT0P82006QXJ00`<300LG5a0<W9b@1PH600PH6107MgM`1`L700:2PX
03/k>`1HF5P0@d=304Q8B020P800DU9B03dm?@1NGUh0Fe]K065QH@2DU9@0A4A406YZJP2VYZH0O7al
04m?C`17AdL0Ph>304a<C00c<c<0F5QH06EUI@2:RXX0[Jf]05mOG`0Y:BT0De=C04m?C`1@D500GemO
04M7A`18B4P0EUIF04Y:BP1GEeL0IfMW065QH@0j>SX0HV9R07IfMP1NGUh0LW9b08j>SP2?Shl0?Cdm
04I6AP2d];@0SXj>08N7Q`2CTi<0QHF504Q8B00l?3`0Ng]k05YJFP2:RXX0LW9b04I6AP19BDT0KVi^
04I6AP21PH40HF5Q06a/K02BTY80JFUY05EEE@0[:b/0B4Q8069RHP0l?3`0UYJF07YjNP23Ph<0GEeM
055AD@0d=3@0DE5A3_ooool0X`37alL0^;Rh0;>c/`1UIFD0E5AD04i>CP1<C4`0Bd];065QH@0d=3@0
?Cdm04a<C023Ph<0Ng]k07inOP1JFUX0AdM7051@D01IFET0FUYJ07]kN`28R8P0WYjN08f=S@27QhL0
Z:RX0<[:bP28R8P0>c/k05ADE01RHV80De=C03Lg=`0^;Rh08b<S01dM7@0Z:RX0GUiN02h^;P1@D500
BTY:09jNWP2_[jl0N7Qh03Lg=`0^;Rh0U9BD09ZJVP2c/k<0NGUi06EUI@1CDe<0<S8b020P800N7Qh0
ATI607YjNP1EEED0NWYj09BDU02XZ:P0a<C408V9R@1GEeL0IFEU04Q8B00X:2P07alO01/K6`0S8b<0
BDU903Lg=`0c<c<0;Rh^024Q8@0f=SH0GUiN04I6AP0W9bL0:2PX02l_;`0m?Cd0<S8b03lo?`2g]kL0
/[:b05ADE01KFe/0Ng]k0;Vi^@2PX:00=CDe02`/;00V9RH09BDU02XZ:P0m?Cd0PH6107=cL`2BTY80
EeMG034a<@0m?Cd0@D5104A4A01>CTh0MWIf03Ti>@0S8b<09BDU05mOG`1;Bd/0CTi>06=SH`3<c<`0
SHf=02h^;P0S8b<0D51@08j>SP1>CTh0>3Ph034a<@0L71`07AdM02XZ:P1IFET0WinO0;2`/02MWId0
H61P071`L01JFUX0BTY:02TY:@0`<300:RXZ02DU9@0^;Rh0FEUI0;Fe]@1LG5`0GUiN06ATI02LW9`0
MWIf0<71`@1MGEd0:BTY020P800`<300A4A405eMG@25QHD0IfMW06=SH`1=CDd0@d=306ATI02CTi<0
V9RH04i>CP1`L7000P12@T801@1GEeL0T92@06EUI@1`L700AdM700808b<S01<0@410051@D00l?3`0
<30`06QXJ01GEeL0CTi>07]kN`1eMGD0][Jf06i^KP1BDU80YJFU0=SHf039bLT0_kno0<o?c`2<S8`0
T92@0080GUiN05D0D51@03@d=01PH600HF5Q02TY:@0M7Ad0M7Ad055AD@2MWId0O7al08B4Q01/K6`0
Ti>C0451@@0Q8B4071`L01hN7P0L71`08B4Q065QH@2DU9@0JFUY07IfMP0o?cl0OGem0:2PX02BTY80
FEUI02<S8`0U9BD0?clo09:BTP2^[Zh0>c/k02l_;`14A4@0QXJ609FEU@2;Rh/0Ti>C07YjNP0l?3`0
7QhN02/[:`1;Bd/0Kfm_07UiN@13@d<0J6QX0=[JfP3Fe]H0UYJF0=[JfP3MgMd0fMWI0:j^[P2m_Kd0
`<300;Ng]`2f][H0W9bL0=7Ad@3Kfm/0OWin03/k>`0W9bL0>c/k08f=S@26QXH0?3`l051@D01BDU80
LG5a06e]K@1hN7P0YjNW096AT@2YZJT0>SXj038b<P1cLg<0Jf][059BDP2IVIT0^KVi013ooooo000>
ooooo`3h05MGE`15ADD0CTi>0;bl_01KFe/0J6QX06ATI00i>CT0QXJ605iNGP0h>3P0CTi>07MgM`0^
;Rh0CDe=0492@P13@d<0:b/[01DE5@1:BTX0?Shn01dM7@0^;Rh0;2`/03hn?P0h>3P0IFEU07IfMP1/
K6`0<30`061PH00n?Sh0AdM70451@@12@T80:BTY07EeM@1=CDd0=CDe07=cL`0j>SX0A4A406QXJ01C
De<0DU9B06IVIP0_;bl0=cLg05eMG@0Y:BT0Hf=S03`l?00g=cL0A4A404Q8B01_Kfl0@41008V9R@0a
<C40<30`07=cL`1eMGD0;2`/05mOG`0b<S80Fe]K04Q8B00c<c<0QhN703Hf=P1[Jf/0JVYZ04Y:BP1M
GEd0G5aL02HV9P1IFET0MWIf055AD@10@400D51@05ADE00f=SH0ATI604M7A`0X:2P0=cLg069RHP1B
DU80IVIV09jNWP13@d<0IFEU06UYJ@0h>3P0?clo06ATI01dM7@0?3`l0410@01?Cdl0<c<c06][J`1d
M7@0J6QX071`L00o?cl0KVi^08V9R@1LG5`0CDe=03/k>`16ATH0OWin08>3P`1YJFT0@T9206ATI01[
Jf/0Hf=S05iNGP11@D40FEUI05]KF`1]KFd0GEeM03dm?@2BTY80MWIf051@D02IVIT0Lg=c04];B`1m
OGd0EeMG06MWI`21PH40EeMG08:2PP1^KVh0AdM705iNGP2:RXX0FEUI09>CT`1MGEd0IVIV07UiN@1P
H600^;Rh06e]K@1?Cdl0UiNG09jNWP2;Rh/0Ph>309VIV@1UIFD0?Cdm06a/K01eMGD0SHf=059BDP2A
TI40?Shn05iNGP2KVi/0O7al07inOP1_Kfl0=cLg03/k>`1JFUX0C4a<065QH@1UIFD0HV9R07UiN@1B
DU80F5QH059BDP0X:2P0IfMW04m?C`28R8P0Vi^K03Ph>00g=cL0HV9R0451@@1nOWh0>c/k028R8P0l
?3`0]KFe0410@01mOGd0FEUI065QH@1EEED0Fe]K07EeM@1]KFd0@41006EUI@1PH600Fe]K08B4Q01;
Bd/0Jf][09FEU@1/K6`0AdM7061PH01KFe/0:BTY04i>CP1eMGD0Ogmo04M7A`16ATH0KFe]07QhN02/
[:`0Ogmo03@d=01^KVh0NGUi04e=C@1<C4`0>CTi04m?C`1UIFD0FEUI05YJFP1LG5`0?clo03hn?P1Q
HF40R8R806EUI@1kNg/0QhN708R8R013@d<0?Shn0;2`/026QXH207MgM`0L051@D00d=3@0Cdm?07If
MP1QHF40P82005MGE`0_;bl0F5QH07AdM013@d<0Ph>304U9B@1=CDd0TY:B07MgM`1JFUX0<30`04a<
C00e=CD0:2PX04e=C@1oOgl0P820051@D01YJFT0<C4a03dm?@kooooo0240Q8B406i^KP23Ph<0Ti>C
059BDP14A4@0?Shn04i>CP1dM7@0Lg=c05mOG`1:BTX0@D51051@D019BDT0?clo059BDP1[Jf/0PX:2
07alO01eMGD0QHF508R8R035aLD0_Kfm0:b/[01_Kfl0EeMG055AD@15ADD0@D5103De=@0m?Cd00P14
A4@0D`0h>3P0CTi>05eMG@1RHV80NWYj07QhN02TY:@0`L710;Fe]@2][Jd0/[:b0<30`028R8P0DU9B
03`l?00O7al06a/K01XJ6P0H61P06ATI01XJ6P0`<300KFe]08V9R@2XZ:P0/K6a0<[:bP34a<@0CTi>
02PX:00M7Ad06QXJ01/K6`0L71`05QHF01XJ6P0_;bl0>SXj03Lg=`0X:2P07alO038b<P1;Bd/0FEUI
04M7A`0a<C40;bl_05EEE@2QXJ40/;2`0:NWY`2_[jl0Lg=c05EEE@1/K6`0Hf=S061PH01ADE40@d=3
03Hf=P0h>3P0MWIf06QXJ01IFET0DU9B06a/K01YJFT0BTY:04A4A013@d<0@D5105aLG01HF5P08R8R
02HV9P1jNWX0WinO0;Ng]`2PX:00PX:206=SH`1gMgL0K6a/0080G5aL0280OWin071`L00f=SH0:RXZ
03`l?023Ph<0LW9b0<71`@2/[:`0Ng]k06a/K01aLG40Bd];06a/K00/;2`0:BTY02LW9`0U9BD09RHV
02h^;P1EEED0V9RH061PH02/[:`0/;2`0;Bd]02HV9P0L71`06QXJ01^KVh0IfMW05eMG@2>SXh0QHF5
0P1@D500=@0o?cl0D51@079bLP1]KFd0<30`02HV9P12@T80DE5A03lo?`0k>c/0DE5A07emO@27QhL0
HV9R05QHF01PH600HV9R05eMG@1BDU80EUIF09jNWP1lO7`0EEEE05UIF@1bLW80NGUi0861P@27QhL0
Z:RX0<g=c@30`<00/[:b0;6a/@2;Rh/0Ti>C05]KF`11@D40<C4a02`/;00f=SH0?Shn06=SH`1_Kfl0
]kNg08N7Q`1UIFD0GEeM09BDU02MWId0a<C40492@P0i>CT0;2`/008071`L04<08b<S09FEU@1YJFT0
E5AD08J6QP2<S8`0ZJVY0;Jf]P2a/K40YJFU07EeM@1hN7P0MWIf06m_K`2CTi<0Xj>S08R8R00n?Sh0
GemO09nOW`1FEUH0D51@07=cL`0j>SX0?clo03Hf=P1?Cdl0ZJVY079bLP18B4P0UIFE08Z:RP1^KVh0
YZJV0=CDe02LW9`0OGem09BDU033`l<0/k>c08j>SP2JVYX0^[Zj0<_;b`2j^[X0Xj>S030`<00R8R80
92@T02HV9P1OGel0WYjN09FEU@2?Shl0QhN70:^[Z`1`L700De=C06e]K@3:b/X0c/k>0;bl_02][Jd0
YJFU092@T02BTY80Z:RX013ooooo000>ooooo`1@05EEE@1>CTh0?Shn0:FUY@1SHf<0F5QH04Q8B00c
<c<0JVYZ03Ph>00E5AD0;bl_05IFEP0e=CD0C4a<04M7A`0k>c/0@41001LG5`0l?3`0<c<c01TI6@0a
<C40:b/[0492@P0I6AT0<c<c05]KF`1aLG40=SHf038b<P0B4Q80@T9203Lg=`0l?3`03@d=04E5A@14
A4@08R8R05eMG@0U9BD0;2`/04I6AP1;Bd/0GEeM03Ti>@0S8b<07AdM04E5A@0i>CT0E5AD06QXJ01N
GUh0>SXj03/k>`1EEED0>CTi09jNWP1DE5@07QhN04U9B@1PH600:RXZ08F5Q@11@D40E5AD06EUI@0X
:2P0V9RH05IFEP1TI6@0NGUi05mOG`1;Bd/0KFe]04M7A`1XJ6P0QhN702l_;`0];Bd203`l?01J04E5
A@1BDU80BDU9030`<00c<c<0AdM7059BDP1TI6@0[jn_06i^KP1ADE40HV9R02l_;`1MGEd0C4a<08f=
S@1@D500E5AD07inOP1?Cdl0IFEU07emO@1kNg/0WYjN05UIF@1NGUh0MGEe06ATI01XJ6P0@d=304I6
AP1LG5`0?3`l06YZJP19BDT0H61P04m?C`1;Bd/0JFUY04M7A`12@T80>CTi05mOG`1bLW80@D510:RX
Z01mOGd0?Shn0:VYZ@24Q8@0AdM707UiN@1`L700LG5a09:BTP1BDU80PX:207alO01`L700@T9206=S
H`1@D500SHf=08Z:RP1JFUX0W9bL04i>CP25QHD0V9RH07QhN02<S8`0T92@04e=C@1QHF40YZJV0820
P01ADE40IfMW071`L022PX80B4Q808Z:RP1KFe/0PH610:>SX`1TI6@0Jf][06e]K@0h>3P0<S8b0P1;
Bd/0G01JFUX0Ng]k04U9B@1gMgL0@410055AD@1IFET0?Shn06UYJ@1]KFd0OWin08V9R@0a<C4092@T
05ADE00`<300FEUI04=3@`0g=cL0A4A40<O7a`1@D500SHf=04I6AP1SHf<0<30`051@D020P800M7Ad
03Xj>P1EEED0De=C03Lg=`1oOgl0Fe]K075aL@1oOgl0;bl_03De=@22PX80R8R803`l?01;Bd/0HF5Q
05mOG`0a<C40GemO07QhN01bLW80_;bl061PH00H61P0QXJ607EeM@1;Bd/0Fe]K01lO7`0];Bd0@d=3
05MGE`1=CDd0HV9R04I6AP0Y:BT0>c/k051@D01TI6@0JFUY0;:b/P1eMGD0<30`028R8P28R8P0KFe]
06m_K`2:RXX0Kfm_03`l?00o?cl0LG5a0451@@2ATI40KVi^03dm?@1DE5@0JFUY01`L701bLW80J6QX
04];B`1oOgl0LG5a0P0j>SX02P1LG5`0CTi>0410@014A4@0U9BD09^KV`10@400L71`03dm?@0i>CT>
ooooo`0e055AD@16ATH0@D5105IFEP2FUYH0LG5a07YjNP1_Kfl0I6AT0492@P25QHD0>SXj02HV9P0k
>c/0F5QH05]KF`0m?Cd0<C4a03<c<`0a<C40>CTi06a/K03=cLd0dM7A06][J`2>SXh0>CTi059BDP2<
S8`0EEEE05UIF@1/K6`0GemO05EEE@1dM7@0PX:208f=S@1oOgl0RHV9096AT@2g]kL0_Kfm0:NWY`2U
YJD0Q8B4071`L02UYJD0KVi^01/K6`0F5QH05aLG01`L700K6a/00P0G5aL201PH600801TI6@10@400
U9BD0;no_`2b/[80ZJVY01`L700H61P201HF5P1N01TI6@0G5aL061PH01PH600G5aL06ATI03De=@12
@T809BDU02@T900S8b<09BDU04m?C`1ZJVX0GUiN07EeM@23Ph<0VYZJ075aL@2RXZ80De=C04m?C`23
Ph<0Ogmo079bLP1gMgL0Rh^;09:BTP2o_kl0YjNW03lo?`1@D500@41003<c<`1ADE40Vi^K0492@P0h
>3P0=SHf0451@@1RHV809BDU05QHF01>CTh0ADE5061PH021PH40IVIV04Q8B010@400GUiN051@D01l
O7`0[jn_0;Jf]P2k^k/0^;Rh0<_;b`3:b/X0a<C40=WIf@3<c<`0QhN707moO`1UIFD0Jf][06a/K020
P800K6a/02h^;P0_;bl0:BTY02@T900Y:BT0Bd];0:j^[P2NWYh0_Kfm08B4Q01[Jf/0H61P05EEE@1H
F5P0EeMG06i^KP2a/K40YjNW05mOG`14A4@0EUIF06i^KP1hN7P0:2PX01hN7P80820P03X0?Shn05]K
F`16ATH0@T9205]KF`2ATI40N7Qh06EUI@1DE5@0@D5103dm?@16ATH0Cdm?05=CD`1jNWX0a/K609nO
W`11@D40@d=309JFUP24Q8@0QXJ60<71`@2a/K40FEUI06ATI020P800OWin04E5A@0d=3@0@D5103dm
?@0g=cL0@41004a<C02QXJ40gmoO0=cLg02j^[X0XJ6Q0:n_[`3De=@0bl_;0:RXZ01HF5P0>CTi03Lg
=`0L71`0D51@0<O7a`12@T80GEeM08N7Q`3;bl/0_[jn075aL@2i^KT0SXj>0P1OGel04011@D40?Cdm
04E5A@1]KFd0Fe]K08n?S`2a/K40O7al03/k>`1QHF40b/[:07QhN01dM7@0XZ:R0<C4a039bLT209NG
U`0U096AT@0d=3@0>c/k08b<S03?cll0C4a<051@D013@d<0NWYj07moO`0o?cl0>SXj07UiN@2_[jl0
LW9b09NGU`29RHT07AdM01hN7P0L71`0820P07UiN@3GemL0Ti>C04i>CP1TI6@0M7Ad04];B`1?Cdl0
^k^k08j>SP0g=cL0<C4a02l_;`0l?3`0IVIV07alO00@ooooo`003_ooool05P1ADE40@d=303dm?@1f
MWH0AdM704m?C`1>CTh0:b/[06i^KP14A4@0@d=302d];@1CDe<0;2`/04Q8B016ATH092@T02l_;`0C
4a<0=3@d03Hf=P0I6AT202LW9`0M05IFEP0c<c<0@d=305mOG`1jNWX0>CTi055AD@15ADD0=3@d02LW
9`1:BTX0?Cdm06MWI`0b<S806ATI079bLP0o?cl0AdM703lo?`0g=cL0Hf=S07QhN016ATH0:2PX051@
D00X:2P0=SHf05]KF`1EEED00P15ADD0c017AdL0=CDe07]kN`1VIVH08b<S04U9B@1WIfL07QhN06YZ
JP0g=cL0Cdm?05QHF00Y:BT0Rh^;069RHP1KFe/0MgMg02d];@0g=cL0VIVI065QH@1OGel0Xj>S07Mg
M`12@T80ADE503/k>`0n?Sh0Kfm_07MgM`1EEED0=3@d03lo?`1<C4`0EUIF09JFUP24Q8@0BTY:06][
J`0Q8B40Hf=S0410@01DE5@0:2PX03lo?`1VIVH0ADE5065QH@0m?Cd0CDe=07UiN@1EEED0Fe]K075a
L@1;Bd/0EUIF04E5A@1KFe/0M7Ad079bLP1[Jf/0Cdm?069RHP27QhL0HF5Q06MWI`17AdL0HF5Q03Xj
>P1TI6@0G5aL03`l?02^[Zh0YJFU04Q8B02?Shl0IFEU04I6AP2?Shl0M7Ad05IFEP2FUYH0B4Q808B4
Q01aLG40N7Qh03De=@1]KFd0J6QX08^;R`2>SXh0Cdm?08:2PP1UIFD0SXj>0:BTY023Ph<0QHF508J6
QP1EEED0K6a/09nOW`1_Kfl0De=C06=SH`1^KVh0XJ6Q05MGE`2?Shl0PX:206e]K@2a/K40QXJ606EU
I@2?Shl0CDe=065QH@1cLg<0LG5a04M7A`1^KVh0FUYJ0:2PX01cLg<0FUYJ05MGE`11@D40PX:208^;
R`1kNg/0[Jf]0451@@18B4P0OWin02PX:01PH600;2`/01hN7P0Q8B40TI6A05IFEP2:RXX0AdM706i^
KP13@d<0?clo05mOG`1[Jf/0>SXj04e=C@1`L700EeMG06=SH`1IFET0MGEe08n?S`1SHf<0>CTi06IV
IP1kNg/0>3Ph04=3@`1CDe<0O7al04Y:BP0e=CD0NGUi05=CD`34a<@0Ng]k0451@@1KFe/0M7Ad04e=
C@1KFe/0GUiN04];B`1@D500FEUI04M7A`2FUYH0O7al04m?C`0k>c/0J6QX04I6AP0h>3P0VYZJ08>3
P`1CDe<09RHV08^;R`1_Kfl0CTi>08n?S`1dM7@0=SHf03hn?P1SHf<0?Cdm07QhN01MGEd0AdM703dm
?@80A4A40140Lg=c06i^KP15ADD0OWin092@T01NGUh0:BTY0410@00n?Sh0EeMG03De=@2QXJ40X:2P
05UIF@1EEED0>SXj03Ti>@0>ooooo`0@06=SH`1<C4`0@D5103hn?P16ATH0Rh^;0:b/[02;Rh/0Cdm?
08^;R`2RXZ80E5AD04];B`0l?3`0:BTY02XZ:P808B4Q0240<C4a05YJFP1]KFd0IFEU04i>CP2g]kL0
LW9b06e]K@1hN7P0820P03hn?P2>SXh0QXJ60:n_[`33`l<0UiNG08N7Q`2ATI40IVIV079bLP1hN7P0
?Cdm06i^KP1[Jf/0N7Qh08R8R00h>3P0@D5104M7A`24Q8@0D51@01TI6@0E5AD00P0G5aL02`0K6a/0
6ATI01HF5P0F5QH05aLG028R8P2OWil0R8R80861P@2SXj<071`L00D05ADE05/06ATI01LG5`0F5QH0
5ADE01PH600_;bl0ADE5028R8P0W9bL0;Bd]020P800k>c/0_kno07inOP0o?cl0TI6A0820P01NGUh0
9BDU02PX:01YJFT0GEeM03Lg=`0b<S80;bl_03Ti>@1SHf<0[Jf]07UiN@0g=cL0:2PX02LW9`0m?Cd0
Ng]k09>CT`1CDe<0>CTi03De=@1JFUX0MWIf06=SH`17AdL0Cdm?03@d=00g=cL0VIVI06][J`19BDT0
ATI606a/K01EEED0T92@09:BTP2XZ:P0YJFU0:NWY`2YZJT0_[jn0;Jf]P2_[jl0];Bd055AD@0d=3@0
<C4a03<c<`0l?3`0HV9R08J6QP1WIfL0<S8b02@T900W9bL0AdM70:^[Z`2XZ:P0e=CD0;fm_@2EUID0
S8b<079bLP1_Kfl0Fe]K04];B`1DE5@0Y:BT08Z:RP2MWId0`/;207emO@0f=SH07AdM008071`L02L0
7AdM02DU9@0k>c/0J6QX05=CD`1jNWX0UYJF07inOP0b<S80=SHf03dm?@0P82007QhN020P800R8R80
J6QX0:j^[P2c/k<0<S8b0451@@1mOGd0Shn?09VIV@1HF5P0O7al05iNGP1EEED0O7al07moO`1RHV80
@41003Lg=`13@d<0?3`l03/k>`22PX80clo?0<;2`P2TY:@00P2][Jd0<02g]kL0e=CD04Y:BP1OGel0
Z:RX071`L01FEUH0O7al0;Rh^03>c/h0OWin065QH@2GUiL0`<3009:BTP2[Zj/0KVi^04=3@`1?Cdl0
@d=306i^KP1PH600>SXj04I6AP1/K6`0NWYj0=3@d01IFET0M7Ad0;Bd]02LW9`0RHV90;no_`3=cLd0
^KVi0;bl_035aLD0Y:BT06e]K@0^;Rh0A4A40;Jf]P34a<@0FEUI04I6AP0S8b<0H61P07IfMP80A4A4
00H0De=C0:BTY01eMGD0QhN70;jn_P11@D4201hN7P0?02DU9@24Q8@0[Zj^061PH01<C4`0GUiN079b
LP1/K6`0I6AT0:FUY@1jNWX0:BTY02/[:`0_;bl0ADE50080Cdm?4?ooool000kooooo03H0E5AD06IV
IP1=CDd0QHF503dm?@1HF5P0LW9b03Lg=`19BDT0=SHf05MGE`0l?3`0GEeM01lO7`0[:b/0De=C03Ph
>00d=3@01`L702HV9P11@D408R8R03<c<`0Q8B40?3`l03Ti>@1:BTX0MWIf08V9R@0T92@0>CTi05=C
D`0g=cL0:BTY038b<P0Z:RX0MWIf06UYJ@0M7Ad0Lg=c02h^;P1EEED0LG5a0492@P14A4@0L71`04i>
CP1:BTX0IfMW0492@P0b<S80>c/k03Ph>012@T8205MGE`1i04A4A01lO7`0ATI601/K6`12@T80JFUY
02@T901]KFd0<C4a05ADE023Ph<0;bl_08R8R01SHf<0J6QX092@T01bLW80?3`l09NGU`0/;2`0B4Q8
08j>SP1CDe<0?Cdm075aL@1MGEd0;2`/03hn?P16ATH0BDU903Lg=`0o?cl0Fe]K059BDP25QHD0HF5Q
04Q8B01oOgl0ADE504Y:BP13@d<0GemO04];B`18B4P0MGEe05QHF01WIfL0UYJF05ADE020P800Bd];
05YJFP2<S8`0I6AT04I6AP0h>3P0@d=305QHF025QHD0I6AT04I6AP19BDT0R8R805IFEP1;Bd/0=3@d
05]KF`0m?Cd0I6AT05iNGP0i>CT0QHF5071`L012@T80Ogmo05MGE`0a<C40GemO06m_K`18B4P0]kNg
06QXJ029RHT0Q8B408>3P`0g=cL0QHF5071`L02ATI40RXZ:051@D02?Shl0Fe]K08f=S@29RHT0IfMW
07emO@2JVYX0MgMg07YjNP2WYjL0Jf][04=3@`1EEED0Rh^;09>CT`1QHF40[Zj^059BDP0d=3@0MGEe
08f=S@1OGel0V9RH02XZ:P11@D40F5QH05mOG`15ADD0Cdm?03<c<`02079bLP1S04m?C`1^KVh0=3@d
05iNGP21PH40JVYZ0;>c/`10@400=3@d08V9R@0[:b/0O7al05IFEP0l?3`0820P09bLW01=CDd0TY:B
07alO01jNWX0Lg=c05EEE@1hN7P0Kfm_03<c<`1LG5`0R8R806QXJ022PX80GEeM07IfMP2FUYH0R8R8
051@D025QHD0Ph>302d];@11@D40GEeM09>CT`1WIfL0=CDe05eMG@0h>3P0Vi^K0:6QX@1<C4`0LW9b
07=cL`0`<300Fe]K05eMG@13@d<0Cdm?05IFEP0k>c/0Hf=S05MGE`1?Cdl0?3`l06ATI01IFET0DU9B
09VIV@26QXH0CDe=02DU9@1jNWX0TY:B05MGE`2?Shl0S8b<03dm?@0o?cl0Fe]K05QHF023Ph<0MGEe
05]KF`1=CDd0Bd];06ATI01jNWX0KVi^04U9B@2BTY80W9bL07=cL`0c<c<0ADE505MGE`1OGel0?Cdm
0;bl_02XZ:P0FEUI05eMG@1BDU80@41000kooooo01D0@41004a<C01BDU80A4A403dm?@1XJ6P0`<30
0=;BdP2f][H0e=CD06a/K01GEeL0CDe=05ADE01YJFT0FEUI04E5A@1aLG40Jf][051@D00b<S800P0U
9BD0202VYZH0e=CD07IfMP24Q8@0ADE502`/;01<C4`0/;2`0P3:b/X0502>SXh0KFe]059BDP0l?3`0
QXJ60:ZZZP0k>c/0GUiN06IVIP1GEeL0HF5Q02`/;00a<C40>SXj04=3@`1^KVh0=SHf01LG5`0F5QH0
61PH0P0F5QH04@0H61P06QXJ03Ti>@1kNg/0`l?30:RXZ01lO7`0][Jf04m?C`17AdL0<c<c02/[:`0O
7al06QXJ01TI6@0G5aL05ADE00<05aLG03D0<c<c034a<@0W9bL0BDU904I6AP0g=cL0Xj>S06=SH`11
@D40Ph>30:b/[01`L700<c<c0410@01hN7P0WIfM06EUI@0j>SX0:BTY02PX:00d=3@0`L710;jn_P2D
U9@0NGUi07moO`1jNWX0SHf=07QhN02@T900NWYj061PH02IVIT0YJFU06QXJ01>CTh0<30`034a<@0W
9bL0<C4a07moO`2:RXX0EUIF04Y:BP1`L700Z:RX0;Zj^P2MWId0_Kfm0:j^[P2ATI40a<C40<O7a`02
0;Jf]P0N08>3P`0o?cl0:RXZ02XZ:P0b<S80>CTi0451@@1MGEd0Ng]k07IfMP1mOGd0XZ:R0<g=c@2a
/K40T92@08J6QP2ATI40]kNg08N7Q`1;Bd/0DE5A06ATI01[Jf/0SXj>08>3P`1nOWh0Y:BT07UiN@0P
820071`L0P0M7Ad0M00N7Qh0;2`/030`<01JFUX0IFEU096AT@2BTY80R8R803Ti>@0];Bd0<S8b020P
800S8b<071`L01hN7P0h>3P0JVYZ0;2`/01OGel0IVIV08B4Q01[Jf/0>c/k03`l?01XJ6P0GemO06QX
J01/K6`0UiNG08B4Q01:BTX0=cLg03/k>`12@T80S8b<0<71`@2e]KD0PH6107inOP2EUID0bl_;0:RX
Z02_[jl0HF5Q04e=C@1TI6@0/[:b0<c<c031`L40QHF50=;BdP2b/[80GEeM0;2`/01[Jf/0MgMg08j>
SP2OWil0CTi>04M7A`1KFe/0Cdm?061PH01YJFT0A4A404m?C`2RXZ80clo?0;Ng]`2b/[80MWIf04=3
@`1YJFT0[jn_0<g=c@2[Zj/0d=3@0=?Cd`31`L40@d=303hn?P2[Zj/0`/;20<W9b@1;Bd/08B4Q02HV
9P1JFUX0I6AT03Lg=`1?Cdl0Rh^;0;Jf]P1jNWX0KVi^096AT@2SXj<0=3@d01lO7`0];Bd0]KFe05]K
F`1@D500>SXj04M7A`2][Jd0XZ:R0:BTY02b/[80UYJF04U9B@1GEeL0PH6105mOG`1FEUH0IfMW4?oo
ool000kooooo0500DE5A096AT@1PH600NWYj034a<@19BDT0Hf=S03Ph>01XJ6P0Bd];04M7A`0e=CD0
MWIf0410@00f=SH0E5AD02d];@1EEED07QhN02LW9`1?Cdl03@d=02HV9P0^;Rh0@D5101@D500P8200
DU9B07moO`0i>CT0=CDe01dM7@0g=cL0?Shn02l_;`0@4100?clo05eMG@0N7Qh0IfMW02@T900/;2`0
@d=304A4A01ADE40@T9201TI6@0V9RH0?Shn0492@P0];Bd0>CTi01@D500K6a/0<c<c03lo?`0k>c/0
S8b<06e]K@0V9RH0<S8b05iNGP0X:2P0L71`02h^;P0a<C40IVIV030`<01nOWh0@41003<c<`1WIfL0
GEeM03lo?`2LW9`0G5aL06QXJ02n_[h0EeMG030`<080D51@09H0>CTi05EEE@15ADD09bLW02`/;00l
?3`0Lg=c06EUI@2GUiL0M7Ad03Ti>@23Ph<0Fe]K05=CD`1?Cdl0BDU904=3@`18B4P0CTi>03De=@1G
EeL0Rh^;059BDP1hN7P0L71`04i>CP2?Shl0IVIV04U9B@16ATH0C4a<05EEE@1cLg<0L71`06m_K`1L
G5`0RHV905aLG02ATI40EEEE05IFEP14A4@0Shn?08f=S@1>CTh0T92@07EeM@1?Cdl0YZJV08b<S013
@d<0N7Qh07YjNP10@400PH6105eMG@27QhL0WinO04e=C@1@D500P82006UYJ@1oOgl0NGUi030`<01e
MGD0IVIV08b<S02LW9`0>c/k04Q8B01dM7@0Kfm_07MgM`2l_;`0OWin051@D00k>c/0GemO092@T00`
<300RXZ:0:RXZ01=CDd0[:b/0:JVYP1=CDd0UYJF05eMG@14A4@0ADE50410@01FEUH0Rh^;0492@P22
PX80IFEU04Q8B027QhL0KFe]07AdM01/K6`0Fe]K0;bl_01VIVH0@d=307IfMP0W9bL0J6QX06m_K`14
A4@07QhN09JFUP1CDe<0B4Q8059BDP1mOGd0KFe]04m?C`1fMWH0NGUi02d];@0h>3P0Ng]k06EUI@2<
S8`0A4A407moO`2RXZ80QhN705QHF029RHT0PX:203Hf=P0o?cl0I6AT09FEU@1XJ6P0?Cdm06EUI@0h
>3P0RHV909fMW@1JFUX0Jf][07]kN`19BDT0EEEE04];B`16ATH205IFEP0/05UIF@1UIFD0HF5Q05mO
G`0b<S80Hf=S07YjNP1PH600V9RH07moO`1UIFD0;Bd]07UiN@2YZJT0J6QX0:>SX`25QHD0ADE505UI
F@20P800F5QH07emO@1]KFd0=CDe04A4A021PH40Lg=c09:BTP1YJFT07AdM06][J`1mOGd0NGUi03Lg
=`1>CTh0IFEU02d];@0[:b/0L71`06UYJ@16ATH0IfMW03Ti>@0S8b<>ooooo`0E05YJFP0m?Cd0<c<c
03Ph>00n?Sh0IVIV08n?S`1gMgL0EEEE08f=S@0i>CT0820P02<S8`0a<C40SHf=0:ZZZP2`/;00GEeM
02TY:@0S8b<092@T00808B4Q01d0QXJ60::RXP2e]KD0Ph>30:6QX@1JFUX0DU9B09:BTP2SXj<0[Jf]
06UYJ@1<C4`0EeMG09bLW02JVYX0PH6107=cL`1lO7`0QXJ606YZJP1RHV80:b/[02LW9`0X:2P0;bl_
03Lg=`1LG5`09BDU01PH600201LG5`1i01DE5@0L71`0CDe=05EEE@0j>SX0@41009nOW`2HV9P0^[Zj
08b<S01:BTX0<c<c03@d=00h>3P0?Cdm04U9B@1=CDd0B4Q803hn?P0d=3@0;bl_038b<P21PH40D51@
06IVIP1;Bd/0BTY:0;Bd]01eMGD0AdM706a/K02j^[X0VIVI04e=C@1QHF40QXJ60<_;b`24Q8@0DE5A
04M7A`0`<300@D510:n_[`1_Kfl0@41004i>CP1GEeL0Ng]k09RHV02m_Kd0XJ6Q0;jn_P3=cLd0Xj>S
09:BTP29RHT0BTY:03@d=00N7Qh071`L024Q8@0/;2`0@41006m_K`1iNGT0LG5a0<;2`P2LW9`0R8R8
09^KV`2WYjL0bLW90<30`02h^;P0[:b/0;jn_P1aLG40QhN706QXJ010@400;Rh^03@d=00a<C40=CDe
0410@01GEeL0UIFE0:VYZ@2e]KD0`<3005]KF`1;Bd/0ADE5051@D01kNg/0OWin06a/K01RHV80EeMG
071`L02CTi<0^KVi0;Bd]01OGel07AdM01hN7P0L71`07alO02`/;018B4P0@D51055AD@1;Bd/0SXj>
09FEU@1OGel0Hf=S02`/;00Y:BT0820P01`L700201dM7@0:038b<P1<C4`0T92@0<K6aP2WYjL0Jf][
02PX:00Z:RX0=3@d04I6AP80K6a/03X0C4a<096AT@2PX:00LG5a04e=C@1YJFT0V9RH0<30`02h^;P0
Y:BT08b<S024Q8@0HV9R06YZJP1`L700IVIV0;>c/`21PH40B4Q80:ZZZP35aLD0M7Ad0:VYZ@2][Jd0
emOG0;no_`2JVYX0B4Q80:BTY02KVi/0H61P08F5Q@13@d<0CDe=06IVIP1DE5@0Bd];07IfMP1VIVH0
/K6a08B4Q035aLD0YJFU055AD@1ZJVX0MGEe08R8R02][Jd0alO70=;BdP3Lg=`0V9RH030`<02XZ:P0
][Jf0:^[Z`2OWil0RXZ:0P0O7al07@17AdL0H61P06m_K`2>SXh0JVYZ08j>SP2MWId0H61P07QhN02a
/K40XJ6Q02LW9`1OGel0SHf=04];B`18B4P0>CTi059BDP39bLT0aLG50;fm_@34a<@0[Zj^075aL@1Y
JFT0LG5a07=cL`1gMgL0PH61013ooooo000>ooooo`1U059BDP1kNg/0IVIV08F5Q@13@d<0BTY:02h^
;P0L71`0@d=303`l?00N7Qh051@D0451@@0e=CD0=cLg04];B`0D51@0=3@d01DE5@0O7al0@41001<C
4`0N7Qh08b<S0451@@10@400>CTi051@D024Q8@0@d=30410@00k>c/0<C4a03lo?`0m?Cd0@41004a<
C01BDU8071`L06i^KP0m?Cd0Cdm?0410@00/;2`0HF5Q06EUI@14A4@08b<S03hn?P14A4@0:2PX04e=
C@19BDT0<30`0410@019BDT0820P06YZJP1kNg/0<30`03/k>`16ATH071`L071`L017AdL0<S8b05mO
G`0];Bd0PH6106i^KP1:BTX0MWIf038b<P0X:2P0Y:BT069RHP1UIFD0/[:b06MWI`1<C4`0GEeM04];
B`0f=SH0GUiN04M7A`0[:b/0<C4a038b<P1bLW80Fe]K09VIV@22PX80A4A407alO01IFET0HF5Q05MG
E`1DE5@0B4Q805EEE@1JFUX00P1EEED0C@1cLg<0BDU908N7Q`1eMGD0DU9B079bLP13@d<0FUYJ0451
@@1<C4`0F5QH03lo?`18B4P0HV9R0410@01ZJVX0>c/k06][J`1;Bd/0AdM702<S8`18B4P0IFEU0451
@@2@T900QXJ602XZ:P1LG5`0EeMG034a<@1gMgL0TY:B04Q8B02YZJT0=cLg0492@P1fMWH0RXZ:03Xj
>P1PH600=cLg07YjNP2TY:@0EEEE08B4Q01HF5P0MWIf0:2PX02MWId0Fe]K07emO@1nOWh0CTi>0;:b
/P2TY:@0Ogmo04U9B@1mOGd0UYJF05IFEP2=SHd0^;Rh051@D021PH40b<S805]KF`2?Shl0GUiN04I6
AP0h>3P0IfMW03`l?01^KVh0?3`l07EeM@1QHF40ATI60080Jf][03T0IVIV04A4A012@T80Vi^K061P
H010@400NGUi02LW9`1CDe<0Bd];03lo?`0M7Ad0T92@055AD@1UIFD0JFUY06MWI`0n?Sh0C4a<09>C
T`1fMWH0?clo03/k>`21PH40De=C08j>SP1VIVH0GUiN08J6QP1UIFD0A4A409JFUP2XZ:P0?3`l02`/
;01:BTX0OWin06e]K@16ATH0O7al04i>CP1XJ6P0Zj^[05UIF@20P800SHf=04M7A`1GEeL0@D5103dm
?@1lO7`0I6AT05MGE`1[Jf/0M7Ad05IFEP1?Cdl00P29RHT09@1NGUh0TI6A08n?S`0];Bd0:BTY07in
OP2HV9P0IVIV0:FUY@1]KFd0;Rh^03lo?`18B4P0@d=308b<S02NWYh0GUiN0410@01^KVh0:b/[08>3
P`1jNWX0E5AD08V9R@22PX80FUYJ02/[:`1YJFT0KVi^05YJFP0^;Rh0NGUi07moO`0i>CT0L71`05EE
E@0j>SX03_ooool04P2PX:00BDU903<c<`0Z:RX09BDU0451@@1UIFD0EEEE08F5Q@1>CTh0:BTY01`L
700M7Ad092@T065QH@0U9BD0<30`02DU9@808b<S0P0P8200U`0O7al0N7Qh03Xj>P0j>SX0LW9b08n?
S`2XZ:P0Shn?0:n_[`2c/k<0`L710:^[Z`2HV9P0Rh^;09JFUP27QhL0LG5a07=cL`27QhL0R8R80:FU
Y@1bLW80>3Ph02LW9`0R8R80:BTY02`/;00m?Cd0MgMg03@d=00I6AT07QhN03Ph>016ATH0;Bd]024Q
8@0P8200=cLg079bLP2b/[80]kNg0<71`@2i^KT0Rh^;051@D00_;bl0820P01lO7`0^;Rh0;Bd]03Ph
>00d=3@0=SHf04U9B@1NGUh0<c<c04E5A@1IFET0JFUY0;Fe]@1kNg/0MWIf07AdM02/[:`0VIVI07Yj
NP25QHD0X:2P09bLW02HV9P0QXJ606QXJ01PH600N7Qh07alO01?Cdl0;2`/02PX:00U9BD0?Shn05mO
G`2IVIT0][Jf0<k>cP3Fe]H0`<300;Ng]`2BTY80Hf=S030`<00N7Qh07AdM02<S8`0Y:BT08B4Q02HV
9P0o?cl0WinO0;no_`2DU9@0Ogmo0<71`@3QhN40dM7A0;no_`38b<P0XJ6Q0:NWY`1iNGT0@d=304i>
CP1UIFD0MGEe04m?C`0m?Cd0?clo0410@01OGel0ZZZZ08J6QP2ZZZX0d=3@0:JVYP1cLg<0E5AD04A4
A00m?Cd0ATI605mOG`1gMgL0Ng]k08j>SP3Cdm<0]KFe0<[:bP1aLG40;bl_02PX:00O7al08B4Q05=C
D`1FEUH0GUiN04a<C00h>3P0Ng]k05iNGP1;Bd/0EEEE04I6AP0P82000P0N7Qh0:P0L71`06ATI02h^
;P1<C4`0SXj>0>;RhP3Ph>00DU9B028R8P0U9BD0:RXZ03/k>`1mOGd0J6QX03/k>`0m?Cd0]kNg0=GE
e@3>c/h0]kNg09JFUP1bLW80Z:RX0820P00i>CT0>3Ph02XZ:P0o?cl0ADE506QXJ01NGUh0P82005MG
E`2?Shl0a<C407inOP2_[jl0HF5Q0;jn_P3Kfm/0NGUi05EEE@80KFe]03/0N7Qh05QHF01VIVH0>CTi
04U9B@1JFUX0E5AD059BDP3;bl/0WinO0;^k^`36a/H0`/;209^KV`1QHF40C4a<06IVIP2e]KD0[Zj^
0;Vi^@3IfMT0[Jf]07inOP2d];@0Q8B406MWI`1bLW80XZ:R03Lg=`0X:2P0Ogmo0:2PX028R8P0?3`l
02`/;01SHf<0WIfM08J6QP1MGEd0MGEe0;Jf]P2RXZ80ZJVY04m?C`19BDT0<S8b06][J`2DU9@0];Bd
0<o?c`2BTY80Z:RX0::RXP13@d<0?clo07EeM@1NGUh0JFUY06e]K@0@ooooo`003_ooool0:`1DE5@0
HF5Q03Ph>01RHV80B4Q805=CD`1]KFd0=SHf03dm?@0e=CD0KFe]03<c<`1IFET06a/K020P801CDe<0
:BTY04Y:BP0=3@d03Ph>0492@P0U9BD08B4Q02PX:011@D40C4a<03dm?@1GEeL0PX:202/[:`0o?cl0
Bd];030`<00U9BD09bLW03dm?@1ADE40FUYJ02TY:@1SHf<0@41004=3@`1;Bd/00P1:BTX0@01[Jf/0
De=C02@T900n?Sh0;Rh^01`L701HF5P0CDe=02l_;`0f=SH0Bd];04E5A@1^KVh0Rh^;03/k>`17AdL0
D51@02l_;`1bLW80De=C02/[:`1ZJVX0?clo05eMG@1[Jf/0C4a<065QH@1LG5`0ADE509bLW01fMWH0
GemO0;no_`1[Jf/0DE5A05UIF@10@400>3Ph08j>SP22PX80@41003<c<`0b<S80Lg=c04i>CP2BTY80
Vi^K04e=C@1kNg/08b<S069RHP1@D500GUiN01`L700_;bl0CTi>059BDP1>CTh0G5aL03/k>`1LG5`0
KVi^05ADE021PH4205ADE01j03Ph>01HF5P0Ng]k0861P@1TI6@0?Shn03dm?@1lO7`0Fe]K06e]K@0g
=cL0CTi>0451@@1HF5P0GemO01lO7`1]KFd0UYJF04Y:BP1OGel0KFe]04Q8B01LG5`0OGem05iNGP30
`<00OWin05ADE01WIfL0W9bL04=3@`1RHV80H61P059BDP2;Rh/0IVIV08J6QP1mOGd0HV9R09bLW02F
UYH0E5AD08:2PP1bLW80GUiN0;Zj^P2NWYh0SHf=04A4A01lO7`0S8b<06ATI01hN7P0`<3005EEE@1j
NWX0c<c<05ADE02;Rh/0Lg=c04];B`12@T80QHF504a<C01YJFT0>c/k06UYJ@2JVYX0E5AD06YZJP1U
IFD0J6QX07UiN@1DE5@0];Bd07inOP1<C4`0VYZJ03Lg=`1BDU80GemO03dm?@0W9bL0T92@07AdM01k
Ng/0JVYZ071`L00j>SX0:RXZ04m?C`1bLW80@d=303`l?01cLg<0ADE5055AD@1NGUh0EUIF09VIV@1a
LG40BTY:05]KF`1EEED0Fe]K02l_;`19BDT0JFUY038b<P0c<c<0I6AT05UIF@29RHT0Y:BT03dm?@1@
D500Jf][05=CD`1`L700ADE5038b<P19BDT205=CD`0[09fMW@26QXH0B4Q802TY:@1:BTX0Fe]K03`l
?02b/[80[:b/05aLG00/;2`0FUYJ08:2PP1CDe<0UiNG09>CT`1GEeL0@41004A4A00e=CD0BTY:06AT
I012@T80?clo05QHF015ADD0PX:2071`L01=CDd0^;Rh0::RXP1JFUX0:BTY034a<@14A4@0E5AD0451
@@1oOgl0Lg=c04e=C@1LG5`0?3`l02l_;`0>ooooo`0<06IVIP1oOgl0ATI602PX:00U9BD0@d=305iN
GP2<S8`0I6AT04A4A00M7Ad06a/K0P0L71`08@0I6AT05ADE020P800S8b<08B4Q01hN7P0P82009RHV
02TY:@1VIVH0:b/[02DU9@0Z:RX0AdM70:NWY`2=SHd0HV9R0;>c/`2e]KD0L71`06UYJ@1dM7@0LW9b
05eMG@1WIfL0NWYj07EeM@1aLG40Ph>309BDU01>CTh0@T92038b<P0202/[:`0F02l_;`0l?3`0X:2P
09bLW01?Cdl0:b/[02DU9@0P82008R8R02h^;P1<C4`0KVi^0:NWY`2l_;`0VYZJ07alO021PH40Y:BT
0:n_[`29RHT0>c/k01dM7@806QXJ01X071`L020P800U9BD0De=C02`/;00K6a/08B4Q04Q8B021PH40
Fe]K03@d=00j>SX0[jn_09NGU`0f=SH0?3`l06UYJ@0Z:RX09bLW03lo?`1EEED0BDU909NGU`1[Jf/0
IFEU03<c<`80:BTY02@0<S8b03Hf=P25QHD0WYjN0;Jf]P2h^;P0^k^k0;6a/@2=SHd0OWin04m?C`0o
?cl0>3Ph03Lg=`0d=3@0:RXZ02DU9@0];Bd0Zj^[0:f][@3>c/h0Vi^K08J6QP3>c/h0XJ6Q05ADE02E
UID0OWin06QXJ02TY:@0WinO03lo?`0i>CT0BDU906m_K`1[Jf/204m?C`0O09VIV@2BTY80^k^k07mo
O`2MWId0KFe]03Ti>@10@400G5aL06UYJ@1=CDd0@D5105QHF028R8P0^[Zj0;Vi^@2OWil0NGUi06MW
I`1SHf<0<C4a03`l?01IFET0X:2P06MWI`17AdL0@D5106m_K`2a/K40GEeM03Ti>@0204];B`0304=3
@`0N7Qh071`L00806ATI01D07alO059BDP2l_;`0b<S80:RXZ02>SXh08b<S01lO7`0W9bL0;bl_07]k
N`3Bd]80Vi^K07IfMP31`L40bl_;08R8R01/K6`0OGem0:RXZ02][Jd00P0b<S80B00f=SH0:RXZ038b
<P1PH600ATI602/[:`12@T80Rh^;04i>CP1iNGT0bLW90:FUY@1[Jf/0bl_;0=WIf@2QXJ40Kfm_03`l
?00e=CD0>c/k05YJFP1[Jf/0PX:203De=@0m?Cd0GEeM09JFUP1oOgl0`/;20<C4a02i^KT0PX:20<71
`@1NGUh0CDe=07emO@25QHD0_[jn0<o?c`2VYZH0`l?30;^k^`1UIFD0@41005EEE@1VIVH0JFUY09JF
UP1JFUX0[Zj^08B4Q014A4@0?Cdm03<c<`1<C4`0Rh^;08j>SP2CTi<0EeMG075aL@3AdM40WinO05QH
F01GEeL0@D51059BDP2QXJ40Xj>S08F5Q@1]KFd0^[Zj06e]K@809RHV00@0AdM703`l?00m?Cd0@T92
4?ooool000kooooo0?l0DE5A09JFUP1ZJVX0L71`04=3@`0o?cl0Fe]K0492@P11@D40FEUI07alO00b
<S80DU9B06][J`0e=CD0J6QX03lo?`1EEED0?3`l01@D5015ADD0:BTY024Q8@0i>CT0<30`05iNGP12
@T80LW9b09FEU@0h>3P0=cLg04U9B@0];Bd0De=C02/[:`0d=3@0B4Q8079bLP0f=SH0GemO0410@00h
>3P0EUIF05MGE`1RHV80NGUi04E5A@0W9bL0B4Q8059BDP0H61P0F5QH02h^;P0W9bL0;Rh^05QHF01:
BTX0JFUY079bLP0_;bl0?Shn055AD@0^;Rh0Jf][04Y:BP0Y:BT0GEeM0492@P1>CTh0FEUI04Y:BP1W
IfL0G5aL04A4A028R8P0S8b<03dm?@2OWil0N7Qh02d];@1=CDd0EUIF03Hf=P1FEUH0BDU903/k>`0Y
:BT0?3`l0861P@1@D500UIFE09VIV@0j>SX0MgMg06=SH`1@D500?3`l071`L01QHF40@d=304i>CP12
@T80C4a<08J6QP1PH600Jf][05EEE@15ADD0RXZ:05aLG01?Cdl0ATI604A4A01?Cdl0FEUI05=CD`1[
Jf/0?Shn08R8R01TI6@0HF5Q059BDP1/K6`0De=C06ATI01nOWh0KVi^07]kN`2:RXX0DE5A06i^KP26
QXH0IFEU061PH02GUiL0GEeM0;^k^`2ATI40EeMG06=SH`2VYZH0De=C08V9R@1hN7P0@D5108Z:RP1X
J6P0V9RH09>CT`1[Jf/0W9bL092@T01NGUh0X:2P07IfMP1VIVH0b/[:09NGU`1HF5P0@T9209:BTP2Z
ZZX0GemO06e]K@2TY:@0F5QH07EeM@37alL0DU9B09bLW01WIfL0ADE505EEE@1YJFT0Cdm?05ADE014
A4@0O7al09FEU@1ADE40Fe]K06a/K01aLG40Ph>304m?C`2][Jd0Jf][03/k>`2:RXX0>c/k05ADE01S
Hf<0ATI601XJ6P21PH40IVIV071`L01kNg/0N7Qh06m_K`1<C4`0MgMg092@T00k>c/0?Cdm07EeM@1Q
HF40LW9b05QHF01>CTh0WYjN09JFUP1ZJVX0Lg=c0861P@1PH600?3`l05]KF`26QXH0Q8B403Xj>P1[
Jf/0A4A407EeM@2UYJD0KFe]06a/K02BTY80@41006=SH`1GEeL0@41006m_K`1]KFd0ADE506MWI`1]
KFd0Bd];038b<P1WIfL0Ng]k04=3@`1ZJVX0QHF503/k>`0[:b/0GUiN09bLW01OGel0MGEe06e]K@12
@T80A4A405ADE01IFET001L0CDe=07=cL`11@D40>CTi07IfMP1NGUh0QHF5075aL@0h>3P0J6QX08f=
S@21PH40>CTi04e=C@1^KVh0?clo02l_;`1DE5@0MGEe04m?C`1ADE40?Cdm02HV9P0>ooooo`0>07al
O024Q8@0SXj>051@D00b<S80FUYJ09fMW@2;Rh/0B4Q803Hf=P0L71`09BDU028R8P0I6AT201DE5@0A
01`L700T92@07alO01hN7P0M7Ad0820P05eMG@1CDe<09RHV024Q8@0U9BD0=CDe09JFUP29RHT0MGEe
0:VYZ@2ATI400P1PH600KP0m?Cd0D51@059BDP1IFET0K6a/08n?S`2YZJT0WinO069RHP1VIVH0E5AD
04m?C`11@D40?Shn03Xj>P13@d<0IVIV0:JVYP19BDT0;2`/02d];@0/;2`0>3Ph03/k>`1GEeL0SXj>
09BDU02n_[h0[Zj^0:BTY02JVYX0Vi^K08^;R`28R8P0VYZJ09:BTP1lO7`0GUiN04];B`0h>3P0>CTi
04=3@`1]KFd0HV9R03<c<`1bLW80JVYZ0410@01QHF40TY:B0;Rh^02i^KT0WIfM07=cL`1YJFT0=cLg
01hN7P0O7al08b<S03@d=026QXH0S8b<08n?S`1hN7P0QHF505IFEP16ATH0?clo075aL@1lO7`0X:2P
09^KV`2BTY80Rh^;0:2PX01jNWX0GUiN04a<C012@T80@410051@D01HF5P0I6AT09^KV`33`l<0OGem
0:2PX017AdL0?clo09FEU@1cLg<0BDU9051@D00l?3`0=3@d04M7A`1GEeL0O7al07inOP1lO7`0DE5A
04Y:BP1CDe<0YZJV09FEU@2c/k<0/;2`06IVIP1[Jf/0>3Ph0P0M7Ad0;`0N7Qh0<30`08Z:RP2HV9P0
Q8B407IfMP20P800OGem09FEU@2LW9`0KVi^05]KF`2KVi/0c/k>0<g=c@2QXJ40H61P059BDP1HF5P0
OWin0;Zj^P2;Rh/0M7Ad07=cL`2:RXX0I6AT06][J`0o?cl0;Rh^0492@P1dM7@0WIfM0;no_`1PH600
[jn_06=SH`2:RXX0NGUi06m_K`1cLg<0aLG50::RXP1TI6@0GemO0;2`/02[Zj/0R8R80080RHV904`0
XZ:R09^KV`0`<3009BDU02`/;00Y:BT0;Rh^055AD@2RXZ80YjNW08Z:RP1^KVh0Q8B408n?S`3?cll0
d=3@0<W9b@3Fe]H0OGem0::RXP2LW9`0Jf][0492@P0[:b/0<30`04i>CP1aLG40XZ:R08j>SP1WIfL0
NWYj0:^[Z`35aLD0UYJF096AT@2GUiL0SHf=0;2`/02a/K40WIfM08^;R`1aLG40MGEe0820P02/[:`0
YjNW069RHP15ADD0@T9206EUI@2@T900`L7107inOP17AdL0Shn?08Z:RP0l?3`0<S8b02l_;`1IFET0
QhN708B4Q02FUYH0][Jf0=GEe@2a/K40N7Qh065QH@1cLg<0Shn?0;bl_02c/k<0GEeM07=cL`3AdM40
L71`0P0W9bL0100R8R80?3`l03De=@0i>CT@ooooo`003_ooool00`1ADE40IFEU03@d=00204Q8B01J
03Ti>@16ATH06ATI02<S8`12@T80;Bd]01<C4`0V9RH0@d=302HV9P1eMGD05QHF03Ti>@0b<S805ADE
04I6AP0@41003@d=02<S8`0];Bd0<c<c00l?3`0d=3@0JFUY0492@P1>CTh0?clo01dM7@16ATH0:2PX
03Xj>P0/;2`0?Shn01TI6@1<C4`0B4Q802LW9`0a<C409bLW0451@@1UIFD0Bd];024Q8@0`<300>c/k
01TI6@1IFET0HF5Q05EEE@0^;Rh0Bd];03Hf=P1ADE40S8b<05EEE@0g=cL0@d=3038b<P1?Cdl0I6AT
02`/;01>CTh07QhN03Ph>01QHF40?clo05UIF@1?Cdl08b<S05iNGP2NWYh0D51@0<O7a`2;Rh/0EEEE
05UIF@15ADD0@D5106a/K01TI6@0=SHf02TY:@0e=CD0QHF505aLG02VYZH0^k^k051@D01]KFd0K6a/
0P1JFUX0]@1bLW80M7Ad04=3@`1GEeL0N7Qh04e=C@25QHD0H61P06m_K`2>SXh0C4a<0861P@1SHf<0
TI6A07UiN@1BDU80CTi>05iNGP1MGEd0TI6A04=3@`22PX80@T9208B4Q01RHV80Fe]K03@d=01NGUh0
N7Qh06EUI@1RHV80SHf=03De=@1NGUh0MgMg06IVIP14A4@0TI6A04=3@`2LW9`0N7Qh04];B`1QHF40
TY:B04E5A@1PH600K6a/03Hf=P1dM7@0Fe]K069RHP1dM7@0JFUY09^KV`1`L700De=C07inOP2ATI40
G5aL0<g=c@38b<P0WIfM04E5A@2QXJ40][Jf05ADE028R8P0^k^k07moO`1kNg/0dM7A04m?C`2DU9@0
YZJV051@D01=CDd0CTi>02l_;`1mOGd0HF5Q07AdM02;Rh/0=3@d0861P@28R8P0I6AT06EUI@0];Bd0
KVi^07alO019BDT0Rh^;02LW9`0];Bd0FEUI03lo?`0n?Sh0Ph>308F5Q@1WIfL0K6a/07UiN@17AdL0
A4A407QhN02YZJT0GUiN02PX:01_Kfl0?3`l06IVIP1]KFd0<c<c07inOP27QhL09bLW03Ti>@1WIfL0
J6QX02d];@17AdL0QHF503/k>`0T92@0HF5Q05QHF01QHF40[jn_04U9B@0m?Cd0Fe]K05YJFP1IFET0
DE5A03Xj>P14A4@0Bd];05]KF`1ZJVX0K6a/05iNGP0S8b<0@41006IVIP1IFET0J6QX0861P@1DE5@0
?Shn03Hf=P23Ph<0H61P09FEU@2RXZ80KFe]04E5A@0g=cL0B4Q804A4A02VYZH0EUIF03lo?`2@T900
Cdm?05YJFP2GUiL0HF5Q05aLG02NWYh0OGem02l_;`0[:b/0EeMG05iNGP14A4@0CTi>05mOG`0n?Sh0
@d=3055AD@0];Bd03_ooool03020P800NWYj06UYJ@1dM7@0alO70=GEe@3GemL0HF5Q0410@00Y:BT0
71`L024Q8@<05ADE06@05QHF01/K6`0M7Ad07AdM01lO7`0M7Ad0<c<c06][J`1<C4`09BDU028R8P0/
;2`0Kfm_075aL@1KFe/0Z:RX0=GEe@2j^[X0=SHf02LW9`0`<300BDU903hn?P14A4@0AdM704Y:BP28
R8P0=CDe03dm?@10@400Bd];059BDP1LG5`0@41005YJFP1fMWH0L71`05YJFP2HV9P0CTi>03dm?@0o
?cl07AdM02TY:@1^KVh0JFUY09bLW01cLg<0KFe]096AT@2o_kl0_[jn0:^[Z`2YZJT0SHf=0861P@22
PX80Ogmo07MgM`2MWId0V9RH07IfMP1ZJVX0WYjN0<K6aP2l_;`0PX:209:BTP2RXZ80M7Ad05=CD`0o
?cl0EeMG0;Vi^@2_[jl0NWYj06EUI@0k>c/0<c<c0861P@1JFUX0MgMg0;Fe]@21PH40P8200::RXP2]
[Jd0W9bL06a/K01OGel0GUiN0861P@1iNGT0GEeM07YjNP1/K6`0IFEU059BDP0d=3@0;Bd]0P0d=3@0
X@0c<c<0ADE50:6QX@2i^KT0/;2`059BDP1PH600Z:RX0;6a/@1WIfL0GemO02/[:`0X:2P0:b/[038b
<P0g=cL0?clo07UiN@1JFUX0820P02@T9028R8P0M7Ad06IVIP1RHV80FUYJ04E5A@19BDT07alO02PX
:00k>c/0DU9B03dm?@0b<S80<c<c0492@P17AdL0B4Q8061PH01oOgl0[jn_09VIV@2YZJT0gmoO0<?3
``2QXJ40LG5a0:2PX035aLD0Y:BT07=cL`1LG5`0ADE5059BDP1=CDd0IFEU075aL@1;Bd/0UiNG0:NW
Y`2f][H0alO708j>SP28R8P0_kno051@D01MGEd0KFe]08^;R`34a<@0c<c<0<71`@28R8P0[Zj^0<K6
aP32`/80Kfm_04Q8B00^;Rh0=SHf069RHP1`L7007alO02d];@0W9bL0=CDe05UIF@1CDe<0^;Rh0;jn
_P3=cLd0^KVi09JFUP30`<00WinO08f=S@33`l<0NGUi0:FUY@1<C4`0DU9B08B4Q02;Rh/0IfMW038b
<P0o?cl0BTY:05eMG@0M7Ad0Shn?09jNWP2d];@0^KVi04i>CP0h>3P0GEeM0;Vi^@2l_;`0=CDe02h^
;P0[:b/0=SHf05YJFP2j^[X0Y:BT0;Jf]P2k^k/0RXZ:0:ZZZP2SXj<0NWYj06ATI00a<C40Ph>30=_K
f`2HV9P0ADE5034a<@1^KVh0a/K608n?S`2ZZZX0cLg=0=GEe@2f][H0VYZJ0;Fe]@2b/[80c/k>0<g=
c@2a/K40XJ6Q0=KFeP2g]kL0Zj^[03Lg=`0_;bl0:2PX02h^;P11@D40EeMG013ooooo000>ooooo`3o
04m?C`1oOgl0GEeM05mOG`1=CDd0=cLg05eMG@16ATH0<C4a02d];@1EEED0>3Ph02<S8`0Z:RX04Q8B
04A4A00Q8B40>SXj01LG5`0;2`/09RHV024Q8@0G5aL06ATI01HF5P0/;2`0:BTY06EUI@2;Rh/0<30`
02TY:@1ADE40;Rh^03Xj>P0I6AT0@41003lo?`1DE5@0BDU904Y:BP1IFET0<30`03<c<`1@D500E5AD
05]KF`1cLg<0>3Ph03Ti>@1PH6007QhN0492@P1RHV80FEUI034a<@13@d<0HV9R04e=C@2CTi<0I6AT
030`<01:BTX0@41005MGE`1fMWH0:2PX04Y:BP0j>SX0;bl_06][J`0g=cL0Cdm?05iNGP1PH600@D51
09nOW`1KFe/0bLW909^KV`1XJ6P0E5AD04=3@`1?Cdl0I6AT08n?S`1RHV80;Rh^03Lg=`1iNGT0Cdm?
09ZJVP2h^;P0GUiN05aLG01^KVh0GemO055AD@1UIFD0@d=3038b<P1;Bd/0L71`04e=C@1YJFT0>3Ph
04e=C@1dM7@0Bd];0820P01<C4`0Q8B4061PH01?Cdl0CTi>065QH@1EEED0RXZ:0451@@29RHT0GEeM
07EeM@1EEED0KFe]04e=C@1DE5@0J6QX05YJFP1PH600]KFe051@D01BDU80K6a/0451@@13@d<0XJ6Q
05IFEP2WYjL0Kfm_04A4A01cLg<0UYJF06i^KP1bLW80IVIV03Ph>02IVIT0Ph>307]kN`1EEED0CTi>
09:BTP2MWId0GemO0;bl_02PX:00<S8b08b<S02VYZH0GEeM04Y:BP2EUID0_Kfm069RHP1GEeL0RHV9
03<c<`1EEED0_;bl061PH01KFe/0Kfm_03<c<`11@D40Hf=S03Lg=`0o?cl0ATI608Z:RP2a/K40=cLg
03hn?P1SHf<0H61P0:2PX01PH600RHV907alO00/;2`0NGUi05aLG00g=cL0GUiN04a<C00G5aL0Cdm?
07inOP1/K6`0P82006IVIP1NGUh0DE5A04=3@`2GUiL0J6QX03@d=01RHV80L71`05iNGP1cLg<0>3Ph
08n?S`2ZZZX0Ti>C04a<C02:RXX0K6a/02l_;`1<C4`0Ph>307]kN`19BDT0K6a/04i>CP1iNGT0_kno
08b<S01OGel0EeMG04];B`20P800Kfm_06ATI01JFUX0GEeM04m?C`1bLW80WinO07IfMP0W9bL0@T92
06IVIP0o?cl0NGUi0:RXZ01lO7`0ADE503Ti>@23Ph<0<C4a07=cL`2=SHd0Ng]k04Q8B00i>CT0De=C
000D03Ti>@1bLW80G5aL03Xj>P2@T900QHF507UiN@1dM7@0GEeM02d];@24Q8@0Ogmo03/k>`0];Bd0
MWIf07]kN`1SHf<0CDe=05ADE01HF5P204E5A@0303<c<ooooooooooo00cooooo00`0T92@0820P02:
RXX0VIVI0=3@d03Kfm/0f][J09:BTP0];Bd071`L02HV9P0R8R8401DE5@808b<S00X07AdM01`L700L
71`0HF5Q04i>CP16ATH08b<S02@T900f=SH0U9BD0P2;Rh/07@28R8P0U9BD0:VYZ@1GEeL09BDU02@T
9010@4009RHV02l_;`0k>c/0@41005MGE`1HF5P0BDU9069RHP0l?3`0BTY:06MWI`20P800J6QX05UI
F@1XJ6P0Kfm_09BDU02ATI40Ogmo01`L700i>CT0GemO0080EEEE01d0[jn_07QhN01GEeL0NGUi0;Ng
]`2n_[h0[Jf]0;Fe]@2HV9P0Z:RX0<K6aP2GUiL0YjNW07=cL`1_Kfl0R8R80:n_[`2m_Kd0d=3@09ZJ
VP2;Rh/0MWIf04M7A`0h>3P0<C4a02`/;011@D40IfMW09RHV00207alO02206e]K@28R8P0d=3@06AT
I01LG5`0ZJVY0:6QX@1lO7`0TY:B07EeM@2;Rh/0NGUi092@T01<C4`0<C4a06=SH`1JFUX0FEUI06][
J`1IFET0EeMG05aLG00f=SH0;bl_034a<@0^;Rh0?clo09RHV03?cll0a/K607YjNP2XZ:P0TY:B0;6a
/@2][Jd0HF5Q03De=@0X:2P092@T02LW9`0g=cL0?clo03hn?P1HF5P0820P02@T901VIVH0NGUi05IF
EP1?Cdl0EeMG03Xj>P1hN7P0BTY:0492@P0g=cL0<c<c02`/;00^;Rh0:b/[02TY:@0_;bl0@41006MW
I`1^KVh0S8b<0;Fe]@3AdM40h^;R0<S8b02n_[h0TI6A07YjNP1JFUX0D51@04Y:BP1fMWH0JVYZ061P
H01SHf<0Cdm?03Hf=P0c<c<0Q8B40:f][@2d];@0d=3@0=7Ad@25QHD0H61P08^;R`1nOWh0[:b/0;no
_`2k^k/0]kNg0;Zj^P30`<00bLW90=KFeP3GemL0WinO09NGU`1PH600>c/k04e=C@1QHF40ADE502XZ
:P0d=3@0GEeM02d];@0b<S80/;2`06a/K02[Zj/0[:b/07emO@2l_;`0MGEe05mOG`2j^[X0HF5Q07Ee
M@15ADD0?3`l03hn?P1;Bd/0PH6108b<S080:RXZ02D0CTi>03Lg=`21PH40D51@08:2PP1TI6@0;Rh^
03@d=01NGUh0Kfm_08j>SP0V9RH0:2PX02HV9P0Z:RX0>3Ph0:>SX`22PX80fMWI0>OWi`3>c/h0Y:BT
05MGE`1^KVh0K6a/03Lg=`2OWil0/K6a0<;2`P2QXJ40K6a/0;:b/P1eMGD0IFEU09FEU@31`L40c/k>
0080^;Rh00d0_Kfm0;no_`39bLT0[:b/09VIV@2WYjL0UIFE0=[JfP3IfMT0[Jf]069RHP1=CDd0B4Q8
0080C4a<4?ooool000kooooo0?l0GUiN05mOG`1YJFT0IfMW05UIF@0`<300B4Q8061PH00S8b<0EUIF
0861P@1;Bd/08b<S04I6AP0U9BD0CTi>02PX:00g=cL07alO01<C4`0a<C40:BTY01XJ6P0R8R809BDU
0492@P0e=CD0Hf=S08F5Q@0`<300;bl_04e=C@0Z:RX0DU9B02TY:@0^;Rh0<c<c06MWI`1<C4`0A4A4
059BDP0X:2P0<C4a04i>CP1FEUH0E5AD06e]K@0k>c/0B4Q8065QH@0P8200@D5104E5A@13@d<0;bl_
055AD@1RHV80BDU90820P01;Bd/0>SXj06i^KP14A4@0Cdm?06a/K00a<C40I6AT069RHP0U9BD0M7Ad
03Ph>01DE5@0Ph>307=cL`0m?Cd0Xj>S0492@P2FUYH0VYZJ071`L01FEUH0Cdm?03/k>`1<C4`0Lg=c
06=SH`0`<300;bl_06a/K01DE5@0Rh^;0;6a/@1FEUH0A4A408J6QP1EEED0CDe=05=CD`1KFe/0?3`l
0451@@1HF5P0DU9B0861P@1fMWH0GEeM065QH@0f=SH0P82006][J`1hN7P0:RXZ03hn?P1]KFd0Shn?
06QXJ00m?Cd061PH06i^KP1fMWH0JFUY02/[:`1QHF40NGUi05ADE01CDe<05ADE03Hf=P2NWYh0K6a/
05mOG`1hN7P051@D02DU9@1JFUX0CDe=09NGU`27QhL0ADE50410@01YJFT0>SXj096AT@29RHT092@T
06ATI011@D40Jf][07moO`17AdL0MGEe04E5A@0d=3@0O7al08R8R01]KFd0UiNG0<30`01VIVH0?3`l
07MgM`2=SHd0Lg=c06EUI@2ZZZX0O7al06YZJP3>c/h0L71`04a<C02VYZH0CTi>03dm?@1gMgL0AdM7
03lo?`1ADE40E5AD096AT@1OGel0?Shn06MWI`1;Bd/0XJ6Q06i^KP2IVIT0VYZJ05iNGP1gMgL0J6QX
03dm?@2GUiL0N7Qh01`L701=CDd0QHF506MWI`2?Shl0RHV907]kN`1MGEd0EEEE0:BTY00l?3`0>3Ph
05eMG@1fMWH0IVIV05UIF@14A4@0RHV90:6QX@2ATI40CTi>09>CT`1UIFD0<c<c06UYJ@1oOgl0Shn?
04];B`1RHV80=3@d04m?C`2HV9P0R8R805QHF01QHF40ADE504i>CP1HF5P0GEeM05QHF01JFUX0Bd];
04A4A01cLg<0KFe]02/[:`0l?3`0IVIV03@d=01RHV80L71`05aLG01:BTX0;Rh^08n?S`1ZJVX0EeMG
08f=S@1fMWH0>CTi03De=@1FEUH001L0>c/k08J6QP1eMGD0>c/k09:BTP2NWYh0Ogmo06QXJ01IFET0
;2`/051@D025QHD0Bd];02`/;01hN7P0QXJ606QXJ015ADD0G5aL05QHF00k>c/0A4A403Ti>@0>oooo
o`0;07=cL`1jNWX0UIFE09BDU02FUYH0ZJVY0=7Ad@2IVIT0C4a<01hN7P0M7Ad00P0G5aL0;@0E5AD0
51@D01DE5@1EEED0Rh^;05MGE`0f=SH0<30`07YjNP1TI6@0=SHf038b<P1GEeL0JVYZ07EeM@1cLg<0
Ng]k05aLG015ADD0CDe=09^KV`1WIfL0:BTY02/[:`0Q8B4092@T03Ph>01FEUH0EEEE061PH00X:2P0
=3@d05YJFP1LG5`0B4Q8071`L01gMgL0?clo034a<@0c<c<0OGem07=cL`21PH40@T9204Y:BP0203hn
?P1F059BDP1QHF40G5aL05ADE01SHf<0QhN704];B`0d=3@0=cLg02HV9P0T92@0=SHf07alO02VYZH0
V9RH09ZJVP2XZ:P0aLG50=OGe`1lO7`0TI6A08Z:RP1CDe<0GEeM079bLP1QHF40CTi>06YZJP11@D40
ADE509RHV01`L700GemO059BDP25QHD0XZ:R06EUI@1/K6`0SXj>0:VYZ@2o_kl0ZJVY0:f][@1[Jf/0
TY:B0451@@0Z:RX0;bl_07=cL`1CDe<0FUYJ04Y:BP1@D500DE5A05QHF00`<3009RHV02h^;P0f=SH0
W9bL0=CDe03@d=00UiNG07MgM`1MGEd0K6a/06IVIP1eMGD0GUiN038b<P0Q8B40820P02HV9P0X:2P0
<S8b04M7A`13@d<0=cLg071`L01YJFT0@d=303lo?`1XJ6P0DE5A04=3@`0^;Rh202TY:@0B02`/;00Y
:BT0:2PX02TY:@0X:2P0:BTY02`/;01>CTh0L71`09BDU037alL0gmoO0;jn_P2k^k/0`l?30:n_[`20
P800Bd];0P1ADE40<P1ZJVX0Kfm_05ADE018B4P0CTi>03/k>`0f=SH0Lg=c09fMW@2SXj<0aLG50;no
_`2YZJT0R8R80=;BdP35aLD0`l?30<_;b`3De=@0d=3@0;jn_P2`/;00f][J0<c<c02=SHd0A4A405MG
E`22PX80YZJV0;6a/@2PX:00YZJV08R8R01iNGT0>SXj02l_;`1RHV80S8b<059BDP1_Kfl0_;bl0:>S
X`3Jf]X0E5AD04E5A@2c/k<0ADE504m?C`15ADD0=SHf0P0h>3P04P0o?cl0DE5A024Q8@0R8R80;bl_
0820P01@D500IFEU06YZJP0i>CT0;2`/038b<P1?Cdl0FEUI07EeM@0Q8B408b<S02DU9@80;Bd]01P0
Vi^K06YZJP2i^KT0`<300:2PX021PH40CTi>05iNGP1lO7`0MgMg03/k>`0W9bL0?clo06MWI`2`/;00
FUYJ05MGE`1MGEd0`<300<C4a027QhL0LG5a0:JVYP2PX:0209fMW@0<0:BTY02^[Zh0LG5a0<71`@3>
c/h0TY:B07=cL`1IFET0>c/k03@d=00i>CT0ATI64?ooool000kooooo02X0NGUi07=cL`24Q8@0OGem
08f=S@0l?3`0C4a<05mOG`0R8R80GEeM07UiN@1=CDd0:b/[06ATI00e=CD0De=C03<c<`18B4P0AdM7
01`L700Z:RX0?3`l01LG5`0g=cL0;Rh^04m?C`0_;bl0HF5Q08R8R00l?3`0;bl_03Ph>00P8200FUYJ
02`/;00l?3`09RHV09VIV@1FEUH0@41004i>CP0T92@2051@D03Z04U9B@1GEeL0BDU902/[:`1FEUH0
IfMW024Q8@0g=cL0@d=302DU9@0J6QX09bLW059BDP1@D500Q8B405UIF@0S8b<0?Shn03lo?`1;Bd/0
De=C024Q8@14A4@0@D5101dM7@1:BTX0820P02d];@1OGel0FEUI03<c<`2OWil0@T920:>SX`2VYZH0
H61P04m?C`2ATI40Fe]K04Y:BP2:RXX0D51@01XJ6P0U9BD0Hf=S06UYJ@2VYZH0e=CD04Q8B00i>CT0
WIfM05YJFP0[:b/0@41008F5Q@12@T80AdM70451@@0f=SH0P820071`L01>CTh0<S8b01TI6@1MGEd0
IVIV07IfMP0Q8B409RHV03dm?@1RHV80F5QH05mOG`0[:b/0G5aL06UYJ@1XJ6P0Jf][05]KF`17AdL0
EEEE07]kN`1VIVH0?clo08b<S00h>3P0E5AD08N7Q`1RHV80=CDe0492@P13@d<0Hf=S08B4Q01^KVh0
AdM707MgM`1KFe/0H61P08F5Q@0e=CD0DU9B06UYJ@1>CTh0KVi^05ADE01aLG40VIVI06i^KP2a/K40
RXZ:07emO@2@T900alO708>3P`16ATH0PH61061PH01bLW80O7al0<C4a01jNWX0I6AT0:j^[P1nOWh0
NWYj0;Jf]P1FEUH0?Cdm071`L01@D500De=C06m_K`1OGel0TY:B05IFEP1IFET0OGem04E5A@2CTi<0
B4Q809>CT`2=SHd0F5QH06m_K`0_;bl09bLW05MGE`10@40061PH03hn?P2;Rh/0ADE506e]K@1kNg/0
JFUY05IFEP1NGUh0Rh^;07MgM`0f=SH0@d=305iNGP1QHF40JVYZ02`/;01@D500ZJVY09BDU01IFET0
UiNG06m_K`0T92@0>SXj05UIF@2<S8`0FUYJ05QHF01gMgL0F5QH0;Vi^@2[Zj/0C4a<05QHF01gMgL0
Kfm_055AD@1WIfL0FUYJ05EEE@1WIfL0GUiN08R8R01hN7P0<S8b03Lg=`1WIfL0C4a<07alO01dM7@0
IFEU04m?C`14A4@0YjNW07IfMP1KFe/0MgMg06IVIP16ATH0MgMg07alO00n?Sh0Ogmo07EeM@1DE5@0
Z:RX0:VYZ@26QXH0GEeM059BDP1CDe<0Q8B408R8R01EEED0=3@d07]kN`29RHT0L71`04Q8B01aLG40
LW9b03hn?P1;Bd/0<C4a3_ooool02P2DU9@0^KVi09nOW`1kNg/0DE5A06EUI@2g]kL0/[:b09BDU00j
>SX201DE5@0g01<C4`0E5AD05ADE01XJ6P0a<C40DU9B07QhN029RHT0YjNW0;^k^`3AdM40O7al08J6
QP1gMgL0J6QX0;:b/P2][Jd0GEeM04A4A00d=3@0<C4a05]KF`2DU9@0Fe]K02PX:00P82009bLW03lo
?`1HF5P0DU9B06a/K00e=CD08R8R024Q8@0/;2`0HF5Q06a/K01YJFT0ATI6030`<00a<C40;Bd]03Ti
>@1lO7`0CTi>02`/;00^;Rh0ADE503Lg=`0H61P0:RXZ038b<P0c<c<0HV9R02HV9P0501DE5@1401dM
7@1FEUH0;bl_03lo?`22PX80d];B0=gMg@3AdM40V9RH06UYJ@10@400?clo04M7A`1EEED0LW9b08N7
Q`0l?3`0:2PX051@D023Ph<0LG5a05IFEP1cLg<0/[:b0:6QX@2HV9P0F5QH03lo?`1_Kfl0XJ6Q0:j^
[P1fMWH0Bd];05MGE`0/;2`09bLW055AD@1MGEd0?Cdm04m?C`0T92@09bLW04];B`1XJ6P092@T03hn
?P21PH40c<c<0<?3``2LW9`0Rh^;05=CD`21PH40[Jf]06ATI017AdL0FUYJ05MGE`0U9BD0820P024Q
8@0U9BD0:RXZ02h^;P1@D500X:2P051@D01HF5P203`l?018061PH01/K6`0@41002/[:`0];Bd0:BTY
02LW9`0P820092@T024Q8@0T92@0;2`/0492@P1PH600IfMW09fMW@35aLD0][Jf09JFUP2d];@0`l?3
0<30`03GemL0Z:RX08N7Q`2BTY80I6AT04U9B@11@D40EEEE04Y:BP1?Cdl0BTY:06][J`2HV9P0Vi^K
08n?S`2EUID0_;bl09NGU`2j^[X0bLW90;jn_P2a/K40PX:20820P02LW9`0f][J0=_Kf`2CTi<0R8R8
04Y:BP0h>3P0@41003dm?@1WIfL0TY:B0:6QX@2OWil0/[:b0;no_`2k^k/0c<c<06IVIP1DE5@0S8b<
0=;BdP3Hf=P0b<S80;^k^`2<S8`0^[Zj0P12@T80501@D500;bl_034a<@0d=3@0=cLg03dm?@0h>3P0
9BDU02PX:01LG5`0ADE5061PH01RHV80;2`/034a<@0c<c<0CTi>05MGE`1eMGD092@T0P0U9BD0:00X
:2P0;bl_0:FUY@1:BTX0P820069RHP2TY:@0W9bL04U9B@1HF5P0UYJF02l_;`0/;2`0Cdm?096AT@33
`l<0/[:b0:n_[`2VYZH0`<300<K6aP2ATI40a/K60820P01mOGd0Ti>C0;6a/@2KVi/0VYZJ08^;R`2i
^KT0YJFU08B4Q029RHT0GemO0410@01CDe<0IFEU05YJFP1DE5@@ooooo`003_ooool0EP2FUYH0O7al
07AdM01DE5@0Hf=S03Hf=P1HF5P0;2`/01DE5@0m?Cd0EUIF01/K6`0P8200EEEE02TY:@1KFe/0;Bd]
03<c<`19BDT05ADE01hN7P17AdL0410@02DU9@0/;2`0EEEE02TY:@0e=CD0EUIF04U9B@0b<S80?clo
02HV9P1DE5@0;2`/04e=C@0W9bL0IfMW0492@P0e=CD0H61P04m?C`2IVIT0FUYJ03hn?P1kNg/0KVi^
03Ti>@0c<c<0DU9B028R8P0b<S80LG5a06QXJ00^;Rh0<C4a03Lg=`0b<S80MWIf08b<S00d=3@0E5AD
02d];@1EEED0SHf=04A4A01CDe<0>SXj01PH601LG5`0:b/[03De=@1ZJVX0;2`/02HV9P2LW9`0IfMW
09bLW032`/808R8R02PX:01HF5P0JVYZ04E5A@2FUYH0GUiN0P0K6a/0_P0j>SX0GEeM09RHV03@d=00
IVIV02/[:`26QXH0J6QX04=3@`0f=SH0KFe]03Xj>P1WIfL0K6a/0410@01VIVH0?Shn04m?C`1aLG40
@T9205ADE01:BTX0MWIf06UYJ@0e=CD0A4A404i>CP0^;Rh0SXj>051@D01LG5`0EUIF04M7A`1mOGd0
FUYJ05IFEP0m?Cd0JVYZ07emO@1;Bd/0]kNg04M7A`19BDT0MWIf06=SH`1JFUX0GEeM069RHP1^KVh0
OGem03Xj>P1QHF40U9BD07moO`1VIVH0RHV90410@01bLW80Vi^K06=SH`1XJ6P0H61P0820P02LW9`0
Kfm_0;>c/`24Q8@0CDe=08R8R02i^KT0Ogmo03Ph>01mOGd0CTi>06UYJ@14A4@0UIFE06QXJ01FEUH0
IfMW07AdM01ADE40QhN705ADE00k>c/0De=C03/k>`1QHF40JFUY05iNGP2HV9P0=SHf04=3@`1PH600
>CTi06e]K@0o?cl0MGEe06YZJP0a<C40GemO055AD@0b<S80DE5A038b<P0J6QX0<c<c08>3P`1CDe<0
QhN707emO@1ADE40BTY:05eMG@1oOgl0GemO03Ph>015ADD0Hf=S051@D01eMGD0>SXj05EEE@2f][H0
VYZJ06ATI02BTY80HV9R02TY:@0o?cl0GUiN07]kN`1YJFT0E5AD07]kN`1=CDd0]KFe09^KV`1>CTh0
QHF506m_K`1]KFd0BDU904E5A@1mOGd0U9BD06e]K@1UIFD0OGem06i^KP0X:2P0E5AD08n?S`1RHV80
M7Ad06YZJP1aLG40;Bd]02<S8`1[Jf/0IfMW065QH@27QhL0G5aL028R8P0c<c<0CTi>03@d=028R8P0
J6QX02XZ:P1HF5P0PH6106e]K@1oOgl0CTi>034a<@1;Bd/0NGUi051@D00n?Sh0Q8B404i>CP0d=3@0
C4a<08>3P`1gMgL0>3Ph07MgM`0`<30>ooooo`0>04Q8B01iNGT0L71`09NGU`1@D500HV9R08:2PP2X
Z:P0TY:B07=cL`0P820092@T01dM7@0N7Qh201dM7@0^02DU9@0V9RH0:2PX03De=@2ATI40];Bd0;Jf
]P27QhL0Lg=c06QXJ01CDe<0];Bd0;Rh^01MGEd0C4a<04A4A00c<c<0<30`0492@P1lO7`0AdM7024Q
8@0U9BD0?Cdm051@D01=CDd0EeMG04M7A`0W9bL09BDU01lO7`0R8R80Kfm_0:ZZZP1nOWh0Hf=S02d]
;@0Y:BT0?clo0861P@18B4P09bLW038b<P0l?3`0;bl_01XJ6P8061PH00<09RHV059BDP0T92@00`0E
5AD201LG5`0<01TI6@1hN7P0H61P02PX:00V9RH0VYZJ0<;2`P2n_[h0WinO06e]K@0j>SX0<S8b0P0j
>SX0QP10@400C4a<05YJFP0k>c/0@41006=SH`2UYJD0K6a/08b<S02f][H0HF5Q04a<C01=CDd0AdM7
059BDP29RHT0QHF50;Vi^@2KVi/0EEEE061PH00/;2`0:2PX06QXJ00g=cL0>3Ph03lo?`0P82009BDU
07IfMP2ATI40YJFU09BDU02:RXX0Xj>S0;>c/`2/[:`0Ph>306IVIP1dM7@0DU9B04a<C018B4P0GEeM
051@D00`<3009bLW02@T900V9RH0:b/[03@d=01`L700@T9204Q8B013@d<0=SHf059BDP1WIfL0@d=3
02`/;00];Bd0;2`/02HV9P0T92@09bLW02d];@0h>3P0BTY:04a<C01QHF40[Jf]09ZJVP1lO7`0VYZJ
0:2PX02_[jl0/K6a09NGU`3Cdm<0bLW9096AT@1nOWh0LG5a05eMG@0i>CT0?Cdm04e=C@19BDT0EUIF
06m_K`1dM7@0GemO06YZJP2VYZH0Xj>S0;6a/@2e]KD0ZZZZ0;no_`3=cLd0bl_;0;Jf]P1RHV80YZJV
0<30`02CTi<0HV9R059BDP1;Bd/0ADE503/k>`15ADD0DU9B092@T02RXZ80OGem04Y:BP1WIfL0`l?3
07YjNP2b/[80b<S80<g=c@2j^[X0FUYJ08^;R`2n_[h0c<c<04M7A`0h>3P0>c/k03<c<`0[:b/0;Bd]
0P0a<C401`11@D40?Cdm038b<P1kNg/0EUIF055AD@1PH6000P0[:b/0400c<c<0De=C07=cL`2FUYH0
820P028R8P0S8b<0:BTY02h^;P2LW9`0@41007]kN`1EEED0Vi^K0;jn_P1MGEd209fMW@0M0:JVYP2E
UID0QHF5075aL@1JFUX0JVYZ0<_;b`3:b/X0clo?0;Jf]P2`/;00ZJVY09JFUP2=SHd0PH610;Zj^P2i
^KT0]KFe0<C4a02XZ:P0HV9R06ATI01fMWH0?Shn02/[:`0Y:BT0@d=30:^[Z`2SXj<04?ooool000ko
oooo0980aLG509>CT`24Q8@0OGem05QHF00Q8B40?Cdm059BDP0_;bl08b<S0451@@14A4@07AdM03Xj
>P0;2`/0=SHf04E5A@0m?Cd0FUYJ00<30`0:2PX0EEEE03@d=00T92@03Ph>04e=C@1LG5`0KFe]07]k
N`16ATH08b<S05UIF@1<C4`0?3`l01<C4`1PH600>SXj071`L00W9bL0820P03Ti>@1RHV80_kno061P
H00J6QX0@D5107=cL`1ADE40=CDe0492@P092@T06QXJ04e=C@1SHf<0<S8b03Xj>P10@400:BTY05QH
F01hN7P0=CDe05]KF`0o?cl0:BTY05mOG`0m?Cd0H61P04I6AP0I6AT0G5aL03hn?P15ADD0Q8B4030`
<00@4100PH61061PH01]KFd0_Kfm06UYJ@0m?Cd0De=C04];B`0o?cl0SHf=09FEU@1LG5`0>3Ph03Hf
=P0k>c/0De=C0:2PX029RHT0<c<c07QhN015ADD0F5QH03/k>`1BDU808b<S04=3@`1gMgL0De=C06IV
IP18B4P0@T9206][J`1CDe<0EEEE06MWI`25QHD0LG5a03hn?P1UIFD0OWin059BDP1mOGd0BDU905YJ
FP1aLG40Cdm?079bLP1DE5@0Hf=S0410@01NGUh0ADE504U9B@2UYJD0EeMG05ADE01[Jf/0AdM702d]
;@0m?Cd0DE5A05aLG02TY:@0LG5a0451@@1:BTX0NWYj06EUI@1fMWH0;bl_055AD@<0Lg=c0840@410
051@D022PX80JFUY0:>SX`1`L700LW9b08:2PP3>c/h0Q8B404=3@`1RHV80CTi>07inOP13@d<0YZJV
07alO01OGel0PX:2092@T01OGel0ZJVY06UYJ@0o?cl0B4Q8065QH@1EEED0NGUi05]KF`2>SXh0IVIV
04Q8B01UIFD0>3Ph07QhN01jNWX0U9BD06m_K`0b<S80Hf=S06EUI@0d=3@0E5AD04=3@`0X:2P0<C4a
08n?S`1;Bd/0SXj>07inOP1SHf<0>3Ph06QXJ028R8P0Ogmo03dm?@0n?Sh0HF5Q04i>CP2DU9@0?3`l
051@D02UYJD0P82005EEE@2[Zj/0LW9b038b<P0o?cl0EEEE08N7Q`17AdL0Lg=c07]kN`18B4P0/[:b
08F5Q@0b<S80Ph>3065QH@1kNg/0GemO020P8010@400BDU904a<C01hN7P0S8b<071`L00Y:BT0820P
05EEE@0k>c/0FUYJ07]kN`1LG5`0D51@02DU9@20P800A4A405eMG@1mOGd0MGEe02h^;P0`<300CTi>
02<S8`26QXH0MGEe04I6AP1UIFD0U9BD055AD@1RHV80LG5a05MGE`1fMWH0NGUi034a<@0e=CD0OGem
06ATI01:BTX0;2`/055AD@1UIFD0<S8b05mOG`0`<3003_ooool0:P0m?Cd0?3`l07inOP1`L700CTi>
06=SH`2EUID0bLW90<S8b02PX:00L71`05UIF@1<C4`0AdM703hn?P0g=cL0=3@d02TY:@0b<S80JFUY
0<O7a`2RXZ80VIVI0::RXP1=CDd0C4a<09NGU`13@d<0=SHf06YZJP17AdL0ATI604m?C`0l?3`09RHV
03Lg=`28R8P0CDe=02l_;`0m?Cd0Bd];04a<C080E5AD01T092@T02<S8`0S8b<09BDU03hn?P24Q8@0
`L710:f][@2[Zj/0Ogmo09jNWP2WYjL0IFEU04Q8B01FEUH0H61P04A4A00I6AT05QHF01DE5@0J6QX0
DU9B02l_;`0E5AD05QHF00<05ADE01D05QHF04E5A@2GUiL0UIFE03Ph>01CDe<0UIFE06UYJ@1gMgL0
MWIf04E5A@13@d<0A4A403lo?`0l?3`0?Cdm04Q8B01BDU80?Cdm04Q8B02WYjL00P21PH40@P1fMWH0
@T920451@@0j>SX0>3Ph09jNWP1XJ6P0B4Q806i^KP1cLg<0S8b<07UiN@15ADD08R8R08b<S027QhL0
M7Ad07inOP0b<S808B4Q03Xj>P2OWil0F5QH05aLG01]KFd0YjNW0;2`/02l_;`0E5AD02d];@0b<S80
DU9B04M7A`12@T80>3Ph05iNGP1SHf<0=CDe02DU9@0S8b<09RHV02TY:@1GEeL0CTi>0410@01ZJVX0
FUYJ08:2PP1ZJVX0@T9202`/;00X:2P0;2`/02TY:@0V9RH0;2`/02h^;P0h>3P0EEEE06m_K`2BTY80
MWIf06IVIP1SHf<0DE5A0:b/[080GEeM00`0dm?C0=KFeP2k^k/0][Jf03lo?`12@T80>3Ph020P800N
7Qh08R8R030`<01EEED208j>SP80Vi^K01X0[jn_09NGU`3AdM40LG5a05iNGP22PX80TI6A09RHV02N
WYh0@d=30:^[Z`2BTY80`/;207QhN01CDe<0D51@059BDP19BDT0?3`l0410@02i^KT0b<S807emO@0f
=SH0EEEE0;6a/@80_kno0380b<S80;fm_@2:RXX0IfMW059BDP1SHf<0[jn_0861P@0];Bd0:b/[03dm
?@0e=CD0;Rh^02d];@0i>CT0>SXj065QH@1EEED0VIVI06ATI01ADE40Fe]K02PX:00f=SH0M7Ad0;^k
^`3IfMT0a/K6051@D00W9bL07alO02LW9`0j>SX0Xj>S03dm?@1eMGD0DE5A07=cL`2NWYh0bLW906UY
J@2TY:@0IVIV059BDP1<C4`0?clo04A4A01HF5P0YjNW0:VYZ@80^KVi01<0YjNW08n?S`0n?Sh0<C4a
03Lg=`18B4P0RHV90<30`02k^k/0Shn?079bLP1oOgl0N7Qh06EUI@0e=CD0;Bd]02PX:01iNGT0@T92
013ooooo000>ooooo`090:n_[`3@d=00Vi^K07UiN@2VYZH0ATI603Xj>P1QHF40=cLg0080DE5A0440
IVIV02/[:`1HF5P0B4Q803hn?P0l?3`0:BTY04Q8B00W9bL05QHF04Q8B00O7al0=cLg02DU9@1:BTX0
I6AT05IFEP20P800IVIV028R8P0d=3@0?3`l055AD@0j>SX0GEeM02XZ:P27QhL0IFEU01`L700b<S80
;Bd]04A4A01XJ6P0;2`/04Q8B01LG5`0>SXj03Hf=P1UIFD0@41001XJ6P1;Bd/0=cLg02TY:@0l?3`0
Fe]K03Hf=P1]KFd0M7Ad02`/;00l?3`0E5AD03Xj>P1YJFT0=SHf03/k>`1OGel0;2`/04e=C@0W9bL0
;Bd]05IFEP1UIFD0>3Ph07QhN00204Y:BP1Z0;Ng]`1gMgL0?Cdm05iNGP1MGEd0@d=307MgM`1FEUH0
>c/k03<c<`1FEUH0Jf][09fMW@2l_;`0PX:202TY:@21PH40HF5Q04A4A00c<c<0Lg=c04Y:BP1BDU80
I6AT04i>CP1TI6@0MWIf04e=C@1<C4`0=SHf04Y:BP1XJ6P0LG5a04E5A@0a<C40?clo059BDP14A4@0
PX:2069RHP1]KFd0QXJ6055AD@20P800Fe]K08B4Q01:BTX0E5AD07moO`1=CDd0`<30069RHP1BDU80
JFUY071`L01<C4`0Jf][06EUI@1@D500/K6a09^KV`1OGel0DU9B09VIV@1TI6@0S8b<04e=C@1IFET0
KVi^04I6AP20P800Fe]K059BDP2@T900K6a/08b<S027QhL0QXJ60;Zj^P3>c/h0UIFE05=CD`1UIFD0
QhN708n?S`0o?cl0WYjN06][J`1;Bd/0V9RH0::RXP1FEUH0Z:RX05aLG018B4P0IfMW06m_K`1GEeL0
Kfm_06=SH`2SXj<0LW9b05=CD`1OGel0ATI608j>SP80Ti>C0100S8b<04Q8B01WIfL0Ogmo03Ti>@1Y
JFT0AdM702HV9P0/;2`0RHV904I6AP1oOgl0NWYj08:2PP1DE5@0CDe=0P1XJ6P07P0n?Sh0A4A407Qh
N014A4@0Ng]k03`l?019BDT0U9BD08>3P`1>CTh0JVYZ05IFEP0d=3@0ADE505aLG02EUID0FEUI04Y:
BP1OGel0=CDe09FEU@2UYJD0BDU906QXJ01IFET0Fe]K05QHF01KFe/0EEEE05eMG@80DU9B00@0IfMW
08^;R`15ADD08R8R0P1QHF40901HF5P0D51@06a/K01VIVH0;2`/08B4Q01TI6@0CDe=07AdM01YJFT0
;Rh^04m?C`1PH600<S8b06YZJP1DE5@0@41006UYJ@2UYJD0Lg=c06ATI01MGEd0GemO08:2PP1iNGT0
ADE503Hf=P1HF5P0E5AD04U9B@0a<C40JVYZ0861P@0l?3`0De=C02PX:0kooooo04<0>SXj038b<P0i
>CT0RHV90;bl_024Q8@0]KFe0:b/[02VYZH0VYZJ0<c<c03>c/h0[jn_08n?S`1UIFD0CTi>04A4A01C
De<0D51@07AdM02:RXX0[Zj^06QXJ01HF5P0J6QX08j>SP1bLW80;bl_01dM7@0g=cL0HV9R04Y:BP0U
9BD07AdM02DU9@0P8200Fe]K0:BTY01[Jf/0;Bd]03/k>`14A4@0ATI604i>CP0/;2`06a/K02XZ:P1;
Bd/0MWIf0:2PX030`<00VIVI096AT@1aLG40KVi^08Z:RP1hN7P0DU9B03hn?P0];Bd0<30`03Ti>@0b
<S805ADE01/K6`10@400FUYJ00805aLG00d06ATI01PH600Q8B40:2PX01lO7`0P8200=3@d07UiN@2I
VIT0_;bl09>CT`1^KVh09BDU00@07AdM01/08B4Q02@T900a<C40GemO096AT@1hN7P0Ng]k03Lg=`0W
9bL0Bd];04Y:BP0e=CD0<S8b030`<01=CDd0TY:B03Hf=P11@D40EUIF06i^KP1<C4`0GUiN096AT@2B
TY80JFUY07AdM024Q8@00P1QHF408026QXH0U9BD05iNGP17AdL0FEUI06i^KP1MGEd0RXZ:0492@P0/
;2`0:b/[02PX:010@400D51@03Ti>@0e=CD0?3`l059BDP1FEUH0>SXj02/[:`0l?3`0M7Ad09bLW02?
Shl0^;Rh092@T02>SXh0T92@07=cL`1ADE40?3`l0P0];Bd05@0^;Rh0>SXj04U9B@1CDe<0G5aL06e]
K@1hN7P0Lg=c08N7Q`28R8P0Zj^[0;Rh^02FUYH0QhN70=oOg`2o_kl0N7Qh03De=@0O7al0?3`l03Ph
>00401dM7@1K055AD@1UIFD0R8R8065QH@1MGEd0TY:B06IVIP2d];@0Hf=S02/[:`0l?3`0BDU905iN
GP1RHV80GemO0:RXZ01aLG40MWIf02TY:@0W9bL0:b/[04M7A`1oOgl0NGUi07]kN`2@T900/K6a09VI
V@2/[:`0/;2`0;^k^`3;bl/0a/K60;>c/`1LG5`0AdM705aLG01CDe<0BTY:04Q8B024Q8@0@41002LW
9`0a<C40@d=303Lg=`0/;2`0<S8b03hn?P1aLG40WIfM09:BTP1gMgL0N7Qh0:2PX02WYjL0[:b/0:FU
Y@2VYZH0Kfm_05MGE`28R8P0UIFE08B4Q01iNGT0/K6a0:2PX01eMGD0QHF5055AD@1WIfL0_[jn07al
O01GEeL0N7Qh03De=@0d=3@0;Bd]038b<P0f=SH0G5aL07]kN`2;Rh/0M7Ad079bLP1kNg/0@41002XZ
:P0[:b/0;Bd]03Ph>00208>3P`0;06YZJP1@D500=3@d04U9B@16ATH0A4A4059BDP1@D500JFUY0451
@@0e=CD04?ooool000kooooo03H0[jn_0<;2`P1mOGd0@d=307EeM@0k>c/0?Shn03lo?`0O7al0>CTi
03lo?`0i>CT05ADE04I6AP10@400?3`l02/[:`0?3`l0A4A402d];@0F5QH0Cdm?014A4@0Q8B40:b/[
04i>CP0f=SH071`L03dm?@1IFET092@T03lo?`0d=3@0<S8b034a<@1QHF408b<S05eMG@0n?Sh03Ph>
03De=@0:2PX0<S8b07YjNP0J6QX0@d=305YJFP1MGEd0<S8b05IFEP0R8R806ATI05MGE`1SHf<203hn
?P3N03Lg=`0G5aL0EUIF07inOP1@D500:BTY03lo?`0U9BD0IfMW04m?C`0l?3`0B4Q801HF5P1PH600
@41003Lg=`1VIVH0>c/k01dM7@1HF5P0NGUi03hn?P3:b/X0MWIf02PX:00n?Sh0E5AD04=3@`1aLG40
D51@03/k>`0X:2P0?clo06][J`2?Shl0][Jf08f=S@0h>3P0M7Ad05eMG@1KFe/0Bd];07QhN01TI6@0
Cdm?06][J`1TI6@0EUIF08n?S`1IFET0EUIF06a/K017AdL0L71`06EUI@1/K6`0CTi>04M7A`1?Cdl0
D51@08>3P`1hN7P0LW9b0:ZZZP1PH600SHf=065QH@1aLG40ADE505ADE01fMWH0Bd];0:VYZ@1VIVH0
@d=3069RHP1BDU80ADE506ATI01[Jf/0E5AD0;Rh^02PX:00I6AT05=CD`1jNWX0QXJ60;Jf]P1ZJVX0
@d=307]kN`0m?Cd0VIVI069RHP1:BTX0QXJ603/k>`1BDU80T92@07EeM@29RHT0bLW909jNWP0n?Sh0
JVYZ09VIV@1_Kfl0DU9B0:NWY`1hN7P0EeMG05eMG@25QHD0=SHf08n?S`1`L700=SHf03<c<`0m?Cd0
JFUY06EUI@0n?Sh0KVi^05ADE01EEED0U9BD03`l?01aLG40DU9B07QhN02XZ:P0GEeM05YJFP1nOWh0
?clo05]KF`1=CDd0=CDe01TI6@2:RXX0CDe=06ATI01aLG40QHF506a/K01bLW80LG5a08n?S`17AdL0
=3@d07=cL`1@D500QXJ605eMG@11@D40Ti>C09ZJVP1bLW80O7al06IVIP0h>3P0=3@d04U9B@2IVIT0
O7al061PH020P800B4Q808^;R`31`L40I6AT07inOP2>SXh0I6AT05eMG@1IFET0I6AT08j>SP1fMWH0
Bd];05eMG@2:RXX0DE5A03lo?`2GUiL0JVYZ06UYJ@1ADE40MgMg038b<P0O7al0Jf][06EUI@1GEeL0
Kfm_05QHF00U9BD0A4A405iNGP0o?cl0L71`069RHP0V9RH0@D5108J6QP1@D500XZ:R07MgM`19BDT0
KFe]07=cL`0I6AT0=CDe05]KF`1aLG40K6a/02h^;P1<C4`0MGEe03<c<`2>SXh0>c/k3_ooool0400h
>3P0@T920820P02a/K40SHf=0:2PX02TY:@0X:2P0;no_`2[Zj/0UYJF0<c<c01NGUh0CTi>05IFEP1n
OWh208j>SP110<71`@3AdM40KFe]08B4Q02ATI40EEEE0:j^[P2DU9@0?clo079bLP0e=CD08b<S01LG
5`1JFUX0@d=301`L700J6QX05QHF02@T902=SHd0dM7A07moO`19BDT0ATI605EEE@1OGel0LG5a06m_
K`22PX80Kfm_07QhN02WYjL0`<3009bLW01OGel0>CTi03Ph>00f=SH0F5QH08f=S@1[Jf/0EeMG03Hf
=P0a<C40@41003`l?01ADE40HF5Q09RHV01VIVH0M7Ad065QH@1UIFD0SHf=04M7A`15ADD0@T9204E5
A@1SHf<0VIVI0=OGe`35aLD0^;Rh07]kN`0T92@0100L71`201dM7@0T03Lg=`1`L700Ph>303lo?`0Z
:RX092@T06e]K@3De=@0UYJF059BDP18B4P0QhN70;no_`24Q8@0O7al079bLP1/K6`0P82004Q8B02=
SHd0S8b<05YJFP1LG5`09bLW02DU9@0W9bL0FEUI07IfMP1/K6`08R8R02d];@1<C4`0GEeM08j>SP1W
IfL09RHV0P0W9bL00`0U9BD0=SHf04E5A@0203@d=00T02/[:`19BDT0W9bL07alO02<S8`0Ph>308R8
R02QXJ40bLW909RHV01_Kfl0MWIf055AD@1MGEd0NGUi08Z:RP1XJ6P0@d=303`l?01LG5`0Ph>309RH
V013@d<0@41004E5A@1OGel0P8200;Vi^@2b/[80YJFU09^KV`3Fe]H0`l?3034a<@0N7Qh07AdM0P0l
?3`03P0M7Ad071`L01`L700N7Qh0Kfm_03`l?02GUiL0YjNW09ZJVP1VIVH0EeMG08V9R@2RXZ80EUIF
0P1@D500G01DE5@0H61P0:NWY`2_[jl0Shn?061PH00Z:RX09bLW03/k>`1MGEd0KVi^05=CD`1LG5`0
Rh^;09>CT`1fMWH0/[:b0:^[Z`2ZZZX0cLg=0<30`01OGel0:2PX03@d=015ADD0D51@05EEE@0m?Cd0
DE5A07emO@0S8b<0:BTY034a<@12@T80?Cdm04I6AP1fMWH0_kno0<c<c03Lg=`0f=SH0=GEe@35aLD0
Kfm_04Y:BP18B4P0EEEE08>3P`1BDU80E5AD05mOG`2;Rh/0d];B0820P00h>3P0PX:20:b/[02g]kL0
Ph>309FEU@2WYjL0a/K605=CD`0b<S80=3@d02h^;P0e=CD0A4A409RHV033`l<0WinO07QhN00k>c/0
>CTi05ADE00Y:BT09RHV02`/;00m?Cd0R8R806e]K@0k>c/0BTY:03Ti>@1@D500ADE504M7A`1DE5@0
Ph>303De=@0o?cl0;bl_4?ooool000kooooo06D0b<S80:ZZZP1`L700K6a/08V9R@0Y:BT0<30`04m?
C`0o?cl0:b/[02LW9`1NGUh0;Bd]04M7A`0N7Qh08R8R02h^;P0F5QH0<c<c00/;2`0?3`l0?Shn02HV
9P0W9bL05ADE03lo?`1MGEd0D51@06a/K01CDe<061PH03hn?P1CDe<0<S8b01LG5`1GEeL0=CDe07]k
N`13@d<0410@04];B`0T92@05aLG07QhN015ADD0@D5105MGE`1WIfL0>c/k05EEE@11@D404A4A03Ti
>@17AdL0>SXj03@d=01@D50092@T05iNGP25QHD0Hf=S02<S8`1QHF40;bl_08:2PP1YJFT0=SHf04a<
C00^;Rh0De=C04=3@`0n?Sh0Hf=S051@D00_;bl0>SXj08:2PP0m?Cd0^;Rh0:NWY`1EEED0CDe=05=C
D`0_;bl0I6AT05ADE01SHf<0>SXj04E5A@1jNWX0QXJ60:j^[P2QXJ40Bd];06i^KP1IFET0HF5Q059B
DP1eMGD0F5QH04m?C`0206][J`2205aLG026QXH0EeMG07EeM@1bLW80A4A406UYJ@1dM7@0U9BD05MG
E`1BDU80E5AD05eMG@29RHT0Q8B404a<C020P800@4100:FUY@1OGel0O7al03Xj>P1_Kfl0Ph>3059B
DP2LW9`0IVIV059BDP25QHD0P82004m?C`1HF5P0Fe]K02XZ:P1VIVH0Jf][04m?C`1EEED0GEeM05iN
GP2SXj<0L71`03lo?`2PX:00:RXZ065QH@1ADE40HV9R09nOW`1]KFd0GUiN09ZJVP0[:b/0KFe]0=_K
f`2o_kl0YJFU0:j^[P37alL0S8b<04I6AP1VIVH0T92@08J6QP21PH40/;2`03Xj>P1YJFT0SHf=0451
@@0j>SX0HF5Q0451@@27QhL0ADE507moO`21PH40BTY:05EEE@17AdL0K6a/096AT@29RHT0XZ:R07]k
N`1PH600Y:BT05=CD`1RHV80GEeM04A4A00i>CT0YZJV05QHF01XJ6P0NGUi0:^[Z`1CDe<0CDe=06EU
I@24Q8@0@d=30410@01VIVH0<S8b08>3P`1RHV80>c/k08^;R`25QHD0=cLg08N7Q`2?Shl0F5QH03De
=@13@d<0QHF502l_;`1YJFT0Ph>304Q8B022PX80[jn_03Lg=`1hN7P0Ng]k05MGE`1jNWX0ADE503lo
?`80DE5A02/0Fe]K08f=S@2KVi/0?clo01lO7`1@D500F5QH065QH@1UIFD0Shn?07alO00c<c<0Q8B4
07IfMP1PH600V9RH075aL@0c<c<0>CTi051@D00X:2P0FUYJ07YjNP1NGUh0BTY:0:^[Z`1BDU80NWYj
08:2PP1XJ6P0QXJ607inOP0g=cL092@T03lo?`1fMWH0LG5a02`/;01GEeL0MGEe02TY:@1nOWh0FUYJ
00kooooo0:/0DU9B05IFEP1;Bd/0PX:20;Zj^P2c/k<0Zj^[08n?S`2QXJ40VYZJ0;6a/@2d];@0a/K6
071`L00W9bL07QhN02LW9`1MGEd0alO70<K6aP1gMgL0LG5a0<[:bP2c/k<0c<c<0;bl_01cLg<0>c/k
02LW9`0E5AD051@D02XZ:P1NGUh07AdM01`L700I6AT0;2`/09VIV@3Fe]H0dm?C0;^k^`2l_;`0P820
05QHF02=SHd0U9BD0861P@1jNWX0PH610<71`@2k^k/0NGUi04=3@`0d=3@0;bl_03Ti>@18B4P0CTi>
061PH01HF5P0GEeM06ATI00k>c/0E5AD06UYJ@1BDU80@41003Lg=`1WIfL0D51@05UIF@1JFUX0Cdm?
05YJFP1EEED0FUYJ05mOG`2ATI40`/;208^;R`18B4P0NGUi07QhN00O7al071`L01TI6@0L71`09BDU
04I6AP23Ph<0U9BD09jNWP12@T80De=C075aL@2YZJT0WIfM0:FUY@2`/;00Zj^[0:FUY@2KVi/0@d=3
05IFEP1YJFT0RHV909nOW`1_Kfl0^KVi0:ZZZP1ZJVX0=cLg02h^;P0Q8B407alO05eMG@1dM7@0CDe=
034a<@0/;2`0=CDe08N7Q`2g]kL0Ph>303hn?P0a<C409BDU02HV9P0Y:BT0CTi>038b<P0g=cL0;2`/
034a<@1LG5`0a<C409VIV@1hN7P0F5QH06UYJ@39bLT0XZ:R03dm?@1@D500X:2P0820P02FUYH0[Zj^
09VIV@1RHV80K6a/0:b/[03>c/h0@d=302DU9@0a<C40NWYj0:NWY`36a/H0WYjN0;^k^`2FUYH0H61P
0;bl_02a/K40=cLg01dM7@0L71`0ADE505EEE@0O7al00P0N7Qh0F@0d=3@0RHV9059BDP1[Jf/0G5aL
09JFUP1dM7@0K6a/08J6QP3De=@0`L710<;2`P3@d=00e=CD0=3@d03IfMT0a/K60;Fe]@1UIFD0BDU9
04a<C01>CTh0C4a<07YjNP1ADE40:RXZ03Xj>P14A4@0ATI606EUI@2TY:@0c<c<0:2PX02BTY80Jf][
02l_;`0f=SH0@D5108Z:RP25QHD0O7al061PH02ATI40D51@02TY:@0X:2P0@T920::RXP2KVi/0^;Rh
0;2`/03Kfm/0Zj^[09BDU01VIVH0];Bd04I6AP0^;Rh0>3Ph05IFEP1iNGT0HV9R04];B`1ADE40Shn?
0<W9b@0o?cl0>3Ph096AT@3<c<`0_[jn0<30`031`L40d];B0=CDe02BTY80H61P06YZJP2:RXX0VYZJ
0;no_`2a/K40Rh^;09JFUP14A4@0<30`03<c<`1?Cdl0<S8b0080:2PX00H0@D5107YjNP1OGel0;Rh^
0410@016ATH204a<C00604Q8B01GEeL0M7Ad034a<@1FEUH0?Shn4?ooool000kooooo0540PX:20:FU
Y@1jNWX0Hf=S0:f][@1PH600<S8b05MGE`1@D500?Cdm02/[:`1_Kfl0<c<c04E5A@0c<c<0820P02l_
;`0I6AT06a/K01LG5`0C4a<0ATI6030`<00/;2`0:2PX0451@@1<C4`0Bd];06e]K@1ADE406ATI030`
<01IFET0:BTY038b<P1LG5`0<c<c06m_K`13@d<08B4Q04a<C01gMgL0<30`065QH@1BDU80EUIF055A
D@1SHf<0>c/k05ADE01JFUX05aLG03Hf=P0i>CT0=3@d02l_;`1DE5@0;Bd]05IFEP1iNGT0J6QX02PX
:01OGel0;Rh^07moO`1bLW80=3@d04a<C014A4@0DU9B03Lg=`16ATH0MgMg06i^KP1<C4`0>3Ph06a/
K00f=SH0VYZJ0;Rh^01LG5`00P1<C4`0D`0d=3@0Q8B409^KV`1[Jf/0>CTi04Y:BP1SHf<0MWIf0:b/
[02h^;P0HV9R061PH018B4P0LG5a051@D01iNGT0>c/k05mOG`1jNWX0Hf=S07UiN@1MGEd09BDU05eM
G@1SHf<0C4a<061PH012@T80KFe]03Xj>P1OGel0NWYj03/k>`1PH600?clo04i>CP1dM7@0<30`05EE
E@12@T80PX:2038b<P17AdL0BDU902`/;020P800Kfm_03Ti>@1HF5P0JVYZ03/k>`22PX80T92@03/k
>`2MWId0G5aL03dm?@1?Cdl0Ogmo06=SH`2FUYH0@d=302/[:`1oOgl0DU9B07MgM`19BDT0?3`l07=c
L`24Q8@0B4Q80:VYZ@1/K6`0CDe=0:BTY02n_[h0/k>c0<W9b@3Hf=P0^[Zj04A4A01VIVH0WIfM0080
RHV903h0_Kfm03/k>`1?Cdl0S8b<05aLG01VIVH0OGem03Xj>P21PH40ADE509JFUP1kNg/0E5AD04i>
CP1;Bd/0MWIf07YjNP1YJFT0UYJF07IfMP1RHV80WYjN05MGE`1KFe/0CTi>01TI6@0L71`0P82002d]
;@1FEUH0H61P08V9R@1RHV80DU9B04i>CP1kNg/0:2PX03Xj>P1hN7P0E5AD06=SH`0J6QX0>CTi09bL
W02PX:00G5aL06=SH`1]KFd0?3`l04I6AP1OGel0UIFE06YZJP16ATH0Ph>303Lg=`1lO7`0`/;2071`
L01YJFT0HF5Q04Y:BP80J6QX02h0CTi>06][J`1<C4`0?Shn079bLP1mOGd0LG5a03@d=01nOWh0IVIV
04Q8B01:BTX0P82007=cL`17AdL0UIFE06QXJ00g=cL0Fe]K06a/K01DE5@0CDe=06ATI01<C4`0GemO
071`L01OGel0FEUI0;Zj^P1gMgL0NWYj06i^KP17AdL0LG5a06i^KP1HF5P08b<S02h^;P1kNg/0<S8b
02h^;P1jNWX0Shn?03Xj>P1[Jf/0D51@3_ooool0WP1>CTh0De=C05MGE`2:RXX0WYjN0:RXZ03=cLd0
W9bL08R8R02ZZZX0WinO09VIV@2LW9`0UIFE05mOG`1@D500@D5102DU9@2BTY80`l?309ZJVP2/[:`0
[jn_0:ZZZP1MGEd0G5aL08R8R02EUID0Lg=c02l_;`0J6QX092@T08j>SP12@T808B4Q02HV9P1mOGd0
`L710:RXZ01^KVh0P8200:6QX@2c/k<0`l?30<C4a01eMGD0B4Q802l_;`0Z:RX0O7al08J6QP2OWil0
Kfm_03Xj>P0m?Cd0EeMG0410@00^;Rh0?clo05UIF@1KFe/0IFEU07EeM@1EEED0D51@05eMG@1`L700
F5QH04U9B@1?Cdl0CTi>04U9B@1<C4`0DE5A04m?C`1FEUH0Jf][06a/K02ZZZX0Jf][055AD@0Z:RX0
HV9R06YZJP0g=cL0?3`l05]KF`1JFUX0Cdm?03dm?@1CDe<0LW9b0<c<c036a/H0VIVI069RHP10@400
C4a<08>3P`2QXJ40YjNW0861P@1TI6@0@d=303Ph>00m?Cd0E5AD06EUI@3;bl/0`<3006][J`0];Bd0
9RHV02/[:`0l?3`0Ti>C09fMW@1aLG40CTi>030`<00o?cl0c/k>0:6QX@1OGel0EUIF04m?C`12@T80
ADE504Y:BP1PH600A4A405ADE01/K6`0GEeM09VIV@2g]kL0EeMG04e=C@19BDT0Bd];0;fm_@3Lg=`0
[jn_0:BTY02WYjL0ZJVY0;2`/02f][H0Zj^[0:NWY`3;bl/0jNWY0<c<c01^KVh0IfMW08>3P`1OGel0
XJ6Q0P2MWId09`2RXZ80W9bL09jNWP3Cdm<0f=SH0861P@0Q8B40820P07AdM033`l<0Vi^K07]kN`1f
MWH0Zj^[0;Jf]P1^KVh0E5AD06i^KP1lO7`0ATI605eMG@2RXZ80YJFU08N7Q`22PX80TI6A09RHV02M
WId0[Zj^0=SHf02o_kl0BTY:04=3@`18B4P0ADE50820P01]KFd09BDU01hN7P0201dM7@0;03Hf=P0h
>3P0BDU909VIV@1TI6@0R8R806=SH`1mOGd0KVi^05iNGP1JFUX00P1UIFD05P1dM7@0/[:b06i^KP1`
L700F5QH07alO02c/k<0]KFe0<30`03Fe]H0_[jn05IFEP1mOGd0ZZZZ05=CD`1ADE40?Cdm020P800T
92@0<c<c04U9B@0Y:BT202XZ:P0X08n?S`10@400HV9R0<c<c03EeMD0b<S80;no_`39bLT0cLg=0;bl
_01lO7`0ATI606EUI@2@T900UiNG0<30`026QXH0JFUY06m_K`0o?cl0;Bd]030`<00d=3@0>3Ph02HV
9P0Y:BT0?Cdm071`L01EEED0:BTY03<c<`1^KVh0Bd];04e=C@1KFe/0NWYj09VIV@1_Kfl0IFEU08B4
Q13ooooo000>ooooo`3F09nOW`22PX80Lg=c05]KF`2g]kL0LW9b03@d=01HF5P0GUiN03dm?@0e=CD0
N7Qh03@d=01?Cdl0BDU902HV9P0g=cL07alO024Q8@0X:2P06QXJ0492@P1>CTh0:BTY03Lg=`13@d<0
D51@04Y:BP1SHf<0FEUI028R8P0];Bd0Cdm?02DU9@0k>c/0D51@03Ph>01YJFT0Cdm?02<S8`0o?cl0
MWIf051@D01QHF40DU9B05QHF01BDU80EUIF03dm?@1=CDd0DE5A01XJ6P0d=3@0?Cdm020P800h>3P0
Kfm_03De=@1ADE40Q8B402l_;`0K6a/0FUYJ020P801gMgL0ADE503Ti>@1]KFd0EeMG051@D00U9BD0
?3`l069RHP1SHf<0=cLg03Hf=P1^KVh08B4Q071`L02CTi<0;bl_04M7A`1OGel071`L03dm?@1<C4`0
E5AD038b<P1OGel0HF5Q05MGE`1`L700V9RH03Lg=`1CDe<0Cdm?03Ti>@0S8b<0Jf][04A4A00d=3@0
CDe=03De=@1MGEd0Kfm_05IFEP1EEED0ATI603Lg=`1cLg<0>SXj069RHP0V9RH0@D51055AD@1@D500
Fe]K06i^KP0j>SX0Hf=S04I6AP1EEED0HV9R0:VYZ@1HF5P0CTi>06UYJ@1KFe/0RHV906IVIP1;Bd/0
KVi^06][J`15ADD0F5QH08N7Q`0k>c/0/;2`08b<S01MGEd0Cdm?08R8R026QXH0bl_;07moO`15ADD0
Lg=c04e=C@2LW9`0LG5a04=3@`1jNWX0A4A403@d=01cLg<0KFe]05UIF@1YJFT0JVYZ05aLG025QHD0
NGUi08b<S011@D40HV9R0820P00j>SX0B4Q809fMW@10@400AdM706e]K@0_;bl0@D5105IFEP0_;bl0
GEeM05=CD`2EUID0PH6105QHF01<C4`0AdM706MWI`27QhL0DU9B08Z:RP1gMgL0ADE509>CT`0`<300
Hf=S07YjNP1;Bd/06QXJ08j>SP13@d<0FEUI051@D01iNGT0A4A405YJFP1/K6`0OWin03hn?P0e=CD0
FEUI03Hf=P1lO7`0H61P03@d=01/K6`0VYZJ06UYJ@80Vi^K03h08b<S02`/;017AdL0MWIf0820P01I
FET0PH6103Hf=P1?Cdl0Vi^K06EUI@1eMGD0Rh^;04Y:BP1NGUh0@D5104I6AP1nOWh0J6QX0492@P1g
MgL0OWin03`l?00W9bL0ATI606i^KP1CDe<0D51@071`L00o?cl0:b/[05QHF01YJFT0>c/k06=SH`1O
Gel07QhN03hn?P1OGel0Bd];061PH01UIFD092@T030`<01dM7@0JVYZ06e]K@1kNg/0CTi>06EUI@2K
Vi/0O7al02/[:`0W9bL0IFEU075aL@0a<C40C4a<08B4Q00f=SH0Lg=c061PH0kooooo04H0EUIF055A
D@1@D500@d=308f=S@21PH40X:2P0451@@0k>c/0H61P09nOW`2QXJ40/[:b08^;R`0k>c/09BDU02XZ
:P1<C4`0R8R80<c<c02h^;P0[Zj^0<;2`P2;Rh/0EeMG0451@@0i>CT0?Shn04E5A@1WIfL0IFEU06a/
K02e]KD0ZJVY06QXJ01/K6`0[:b/07EeM@17AdL0?3`l059BDP1MGEd0MGEe0<30`01OGel0?Cdm034a
<@0W9bL09BDU069RHP1bLW80H61P069RHP1UIFD0AdM707YjNP0d=3@09bLW02<S8`0o?cl0Jf][05iN
GP1WIfL0D51@04U9B@1OGel0R8R803`l?015ADD0@D510P0o?cl0100i>CT0>c/k03/k>`16ATH206EU
I@090:RXZ01/K6`0F5QH05EEE@0V9RH0PX:20=;BdP1lO7`0=3@d00808B4Q02009RHV03hn?P2BTY80
[Jf]0;^k^`1dM7@0BTY:03hn?P12@T80QhN70:JVYP30`<00Jf][061PH01eMGD0I6AT03De=@0U9BD0
Ogmo0:FUY@2YZJT0R8R804i>CP0j>SX0GemO08:2PP1;Bd/0NGUi0;Jf]P28R8P0GUiN0:6QX@80Ti>C
01T0Q8B405YJFP1ADE40O7al08b<S02BTY80U9BD0<[:bP2LW9`0Fe]K08j>SP2j^[X0a/K6051@D00j
>SX0=cLg06IVIP38b<P0Rh^;07]kN`1nOWh0NGUi07YjNP2=SHd0UYJF0080Zj^[01H0YjNW0=CDe034
a<@0Xj>S08V9R@2GUiL0Ph>3096AT@1fMWH0EEEE04];B`2BTY80d=3@0>3Ph033`l<0[jn_07moO`14
A4@0Vi^K07QhN01LG5`0EEEE0P1CDe<05`2BTY80S8b<06m_K`2CTi<0K6a/04E5A@1HF5P0Z:RX0:>S
X`1fMWH0H61P07IfMP1YJFT0Jf][08B4Q02`/;00Zj^[08V9R@1GEeL0ATI60492@P1cLg<0AdM700<0
7AdM04h071`L01lO7`0k>c/0=CDe05iNGP1fMWH0Zj^[0;fm_@1OGel0QhN707UiN@1MGEd0HV9R05]K
F`1@D500UiNG0;Rh^01WIfL0WYjN0=?Cd`39bLT0cLg=0=?Cd`2/[:`0Ng]k04m?C`1oOgl0MGEe03dm
?@0/;2`0CTi>04=3@`0N7Qh08b<S06][J`0`<300?3`l03/k>`2XZ:P0N7Qh0<_;b`2n_[h0[Jf]0:n_
[`2HV9P0ZJVY08n?S`1hN7P0PH6107YjNP1ZJVX0C4a<07UiN@2g]kL0Z:RX03Ph>014A4@0MGEe02LW
9`0U9BD0;2`/03Hf=P0n?Sh0:RXZ03lo?`1cLg<0EeMG020P800`<300DU9B08V9R@1kNg/0UiNG08J6
QP3=cLd0bLW909bLW02CTi<@ooooo`003_ooool0;@34a<@0P820075aL@1VIVH0_;bl075aL@0e=CD0
D51@04i>CP17AdL0<C4a07=cL`0o?cl0EEEE06e]K@0Z:RX0De=C02d];@0m?Cd0@D5101XJ6P0o?cl0
@d=302`/;00o?cl0@d=306ATI00k>c/0GUiN07MgM`0_;bl0;Rh^03`l?00[:b/0CDe=03Lg=`0];Bd0
KFe]08F5Q@0b<S80=CDe06MWI`0J6QX0GEeM04Q8B00205UIF@2B04E5A@0o?cl0Lg=c05mOG`0T92@0
=CDe05=CD`0S8b<07QhN0410@00R8R80F5QH09jNWP1bLW807QhN04m?C`0W9bL0MWIf05iNGP0U9BD0
<c<c02/[:`1=CDd0:BTY03<c<`1IFET0IVIV01XJ6P0W9bL0MGEe03/k>`2ZZZX0Zj^[02<S8`0c<c<0
Bd];02`/;01FEUH0DE5A03Lg=`0O7al0@D5106=SH`1oOgl0ZJVY0:NWY`1:BTX0@d=305MGE`1;Bd/0
F5QH06ATI01;Bd/0?clo05MGE`1_Kfl0H61P08>3P`1aLG40GEeM07YjNP18B4P0OWin03/k>`1fMWH0
@41004E5A@1=CDd0EUIF06ATI020P800>c/k07inOP1KFe/0OGem06e]K@2[Zj/0IFEU061PH01`L700
IVIV08:2PP1IFET0DU9B0:b/[02QXJ40Fe]K05UIF@20P800820P096AT@2?Shl0H61P04Y:BP2=SHd0
E5AD08j>SP1VIVH0?clo06e]K@16ATH0O7al06m_K`11@D40K6a/08j>SP1<C4`0Vi^K07EeM@1DE5@0
ATI605EEE@1jNWX0^KVi0=;BdP2/[:`0BDU9075aL@2n_[h0XZ:R065QH@2^[Zh0>c/k07=cL`2PX:00
CTi>04A4A01dM7@0<c<c08V9R@0o?cl0Ogmo09ZJVP1NGUh0FUYJ05QHF01UIFD0Lg=c03De=@20P800
W9bL05iNGP2LW9`0>CTi04e=C@1UIFD202l_;`1C08f=S@1OGel0GEeM05=CD`2<S8`0?Shn0492@P15
ADD0KFe]02h^;P0[:b/0I6AT05ADE01JFUX0IVIV03Xj>P1/K6`0Ph>302XZ:P19BDT0Hf=S03dm?@0U
9BD0@T9207]kN`0e=CD0E5AD0861P@1NGUh0FUYJ0;Bd]010@400=cLg06EUI@15ADD0I6AT04U9B@15
ADD0Hf=S06IVIP17AdL0MgMg06QXJ01]KFd0=CDe04M7A`1`L700I6AT05ADE01EEED0Ng]k03lo?`1g
MgL0L71`03hn?P1[Jf/0FUYJ028R8P0a<C40@T9202DU9@1YJFT0CTi>03lo?`0l?3`0Ph>306IVIP1`
L700QhN705QHF01nOWh0XJ6Q07AdM00W9bL0:2PX069RHP23Ph<0=3@d061PH02MWId0@d=3071`L01h
N7P03_ooool0AP1;Bd/0?Shn03Hf=P1DE5@0SHf=03Xj>P1NGUh0E5AD02l_;`0X:2P0AdM70:j^[P3@
d=00UYJF03Ti>@0L71`061PH034a<@1hN7P0c/k>0=?Cd`3;bl/0Vi^K05ADE01NGUh0L71`05YJFP0o
?cl0>CTi03/k>`0d=3@07alO02/[:`2[Zj/0[Jf]0;:b/P1WIfL0?Cdm03lo?`17AdL0FEUI06e]K@28
R8P0X:2P08^;R`1>CTh0<c<c02`/;00V9RH0F5QH05]KF`14A4@0=3@d05eMG@1gMgL0IfMW030`<00U
9BD08b<S028R8P0W9bL0AdM7079bLP1eMGD0HF5Q05MGE`2JVYX0IVIV02@T900M7Ad301hN7P0M024Q
8@0Z:RX0<C4a04i>CP1RHV80TY:B05YJFP18B4P0H61P02HV9P0c<c<0]kNg04E5A@0O7al07QhN028R
8P0V9RH0EEEE08>3P`1]KFd0MGEe07IfMP16ATH0DE5A08R8R01[Jf/0Ng]k06QXJ01=CDd00P1LG5`0
@`1KFe/0LG5a08N7Q`3Bd]80MGEe09fMW@1VIVH0V9RH08V9R@27QhL0=cLg02TY:@0_;bl0IVIV08:2
PP2DU9@0XZ:R079bLP1/K6`0GEeM09VIV@2YZJT0Hf=S05UIF@1]KFd0ZZZZ0<;2`P2a/K40_;bl09FE
U@29RHT0XZ:R0:RXZ01GEeL0F5QH0=SHf034a<@0=cLg02XZ:P0/;2`0>CTi06i^KP2TY:@0OGem06][
J`1oOgl0Ng]k09BDU03<c<`0alO70<S8b02OWil0[:b/04m?C`11@D40?3`l069RHP2a/K40eMGE0<o?
c`2a/K40UiNG09nOW`3:b/X0KFe]02TY:@0U9BD00P0[:b/09@1KFe/0P82009jNWP2RXZ80LW9b07em
O@1KFe/0MgMg09VIV@1HF5P0LG5a08^;R`2HV9P0Ti>C07moO`2IVIT0a/K6096AT@1gMgL0;2`/01dM
7@0Q8B40FEUI034a<@0M7Ad07QhN024Q8@0N7Qh07AdM020P800e=CD0Jf][0<W9b@3Kfm/0gMgM0:FU
Y@1iNGT00P1?Cdl03P1KFe/0Kfm_05QHF02<S8`0EEEE0:6QX@2_[jl0`<300<K6aP3>c/h0alO7061P
H01GEeL0De=C0P1OGel0<P1YJFT09BDU02`/;01IFET0NWYj04e=C@1bLW80PH6109VIV@2a/K40f=SH
0<o?c`36a/H0VIVI07]kN`1cLg<0Rh^;071`L01ADE40B4Q806m_K`26QXH0S8b<0:6QX@3Bd]80c/k>
09nOW`0/;2`0=CDe05QHF00o?cl09BDU02`/;00c<c<09BDU02l_;`1EEED0OWin071`L00P8200;Bd]
03Ph>02PX:00dm?C0:b/[02g]kL0_Kfm0<71`@2QXJ40QhN74?ooool000kooooo0300_;bl04m?C`1R
HV80Bd];07inOP19BDT0:b/[04];B`0j>SX09RHV01hN7P1?Cdl0<C4a03Hf=P1>CTh06a/K04e=C@0L
71`0Cdm?03Ti>@0H61P0<C4a04A4A00I6AT0;2`/030`<01WIfL0>CTi04E5A@1/K6`0;bl_038b<P11
@D407QhN034a<@0P8200?3`l04I6AP0o?cl0;2`/020P801OGel0Bd];04E5A@0g=cL0D51@05UIF@1H
F5P204A4A03R04e=C@0O7al09BDU06EUI@1MGEd0<30`04m?C`16ATH0?clo08V9R@1mOGd0;Rh^06a/
K00M7Ad0IVIV08j>SP0m?Cd0EEEE04=3@`19BDT0>CTi03dm?@1EEED0LW9b05MGE`0g=cL0Ti>C04M7
A`2>SXh0^KVi05ADE00o?cl0A4A403@d=01HF5P0Ogmo06][J`14A4@0CDe=07AdM01HF5P0V9RH0<30
`01gMgL0>CTi05aLG01UIFD0G5aL05EEE@1VIVH0>3Ph06ATI01`L700EeMG07QhN01GEeL0HF5Q07mo
O`12@T80IFEU03De=@1jNWX0IVIV03dm?@15ADD0F5QH059BDP26QXH0=cLg06QXJ017AdL0GEeM04];
B`1eMGD0A4A40492@P1SHf<0BTY:05YJFP1_Kfl0<30`07AdM024Q8@0>CTi04Y:BP21PH40=CDe09>C
T`2>SXh0@41004Y:BP1mOGd0K6a/09ZJVP1IFET0?3`l06UYJ@2:RXX0Ogmo06IVIP17AdL0OWin08f=
S@16ATH0WIfM06QXJ00g=cL0F5QH0<W9b@36a/H0d];B0>CTi030`<00Cdm?05QHF02JVYX0HV9R05mO
G`2VYZH0@T9205EEE@1oOgl0@d=3051@D01QHF40>3Ph07IfMP17AdL0Q8B409VIV@17AdL0FEUI05EE
E@1YJFT0LG5a04=3@`1]KFd0V9RH04Q8B02PX:00Kfm_04U9B@1RHV80CDe=01/K6`27QhL0FUYJ05iN
GP20P800SXj>05UIF@1TI6@0F5QH07IfMP13@d<0;Bd]06m_K`21PH40J6QX07MgM`1:BTX0GUiN0:2P
X01dM7@0Lg=c096AT@1RHV80:b/[04E5A@1VIVH0M7Ad04E5A@1`L700@41005iNGP2h^;P0SXj>051@
D01PH600Bd];06e]K@1QHF40I6AT08F5Q@1TI6@0Bd];07emO@2FUYH0GemO038b<P10@400MWIf065Q
H@1ZJVX0SXj>092@T016ATH0Jf][07AdM00f=SH0MgMg07IfMP1@D500<S8b04a<C00h>3P0G5aL08F5
Q@1fMWH0BDU908n?S`1@D500YZJV0:6QX@1OGel0RHV909ZJVP1VIVH0;bl_04A4A024Q8@0PX:203Ph
>01NGUh0RHV902/[:`80Ph>33_ooool0o`1PH600Hf=S06a/K027QhL0[Jf]08F5Q@1EEED0D51@059B
DP0^;Rh0DU9B0:j^[P2g]kL0YjNW092@T01TI6@0FUYJ08N7Q`2a/K40f=SH0=CDe03Hf=P0^;Rh06AT
I00g=cL08b<S02`/;013@d<0ATI605]KF`1TI6@0@T9204Q8B01iNGT0Jf][0451@@15ADD0De=C05UI
F@1UIFD0H61P05aLG01VIVH0UiNG0;2`/00a<C40:BTY02l_;`0`<300Hf=S06UYJ@1DE5@0G5aL051@
D01JFUX0SHf=09VIV@25QHD0KVi^07QhN01nOWh0TI6A092@T01mOGd0?3`l0492@P1QHF40RHV907Ee
M@0U9BD08B4Q028R8P0W9bL0:b/[030`<00j>SX0IfMW0;Ng]`2UYJD0AdM704Y:BP1;Bd/0AdM702/[
:`1/K6`0UIFE02PX:00O7al0820P02HV9P1/K6`0D51@05IFEP1dM7@0L71`08:2PP2ATI40Ph>308F5
Q@1]KFd0@T9204m?C`1=CDd0@T9203dm?@1EEED0KFe]0;Rh^02NWYh0FUYJ06UYJ@2SXj<0JFUY05]K
F`1/K6`0G5aL04Q8B01/K6`0UIFE09NGU`1_Kfl0Bd];03De=@0m?Cd0Hf=S0:j^[P1DE5@0C4a<06AT
I024Q8@0]KFe0;bl_02j^[X0YjNW092@T027QhL0[jn_0<;2`P3EeMD0`/;20;Fe]@2:RXX0<c<c02DU
9@0V9RH0<C4a06EUI@1IFET0ATI602h^;P0c<c<0CTi>06IVIP23Ph<0XJ6Q0<k>cP2:RXX0IFEU06m_
K`21PH40OWin07AdM025QHD0Vi^K0:j^[P2g]kL0^KVi0:^[Z`2EUID0N7Qh05YJFP1ADE40Fe]K04m?
C`0n?Sh0SHf=08j>SP1SHf<0OWin06MWI`1eMGD0IfMW03Ph>00b<S80>c/k0410@01=CDd0Kfm_0;Zj
^P2OWil0YJFU09^KV`1mOGd0:BTY04Y:BP0l?3`07QhN01dM7@0R8R809RHV02DU9@0P82008B4Q05aL
G02FUYH0UIFE0=WIf@2f][H0]KFe0<;2`P2TY:@0Ph>3065QH@1;Bd/0FUYJ09ZJVP1[Jf/0QHF50<S8
b03Ogml0emOG09ZJVP2^[Zh0/K6a04A4A00U9BD09bLW03dm?@1OGel0FUYJ04I6AP1PH600YJFU09:B
TP2SXj<0clo?0<g=c@2i^KT0d=3@0<[:bP1_Kfl0De=C04U9B@0a<C40:RXZ03Lg=`1BDU80Fe]K065Q
H@24Q8@0[Zj^0:JVYP2OWil0]kNg07MgM`005`17AdL0;Bd]034a<@1JFUX0F5QH030`<01=CDd0Fe]K
08R8R02k^k/0a<C40:BTY01hN7P0IVIV09:BTP35aLD0b<S80<g=c@2][Jd0T92@05ADE035aLD0ZZZZ
013ooooo000>ooooo`3A0<c<c01GEeL0EEEE065QH@2WYjL0SXj>03@d=01=CDd0MgMg03dm?@0k>c/0
HF5Q05mOG`0/;2`0EeMG030`<012@T807QhN04Y:BP0`<3007QhN01lO7`0f=SH071`L03<c<`0e=CD0
De=C06IVIP1XJ6P0Ng]k04a<C00];Bd0=cLg02l_;`0k>c/08B4Q03dm?@16ATH0H61P05IFEP0W9bL0
@d=306=SH`1IFET0G5aL059BDP1;Bd/0CDe=05iNGP1EEED0Hf=S020P800L71`0EUIF04M7A`0U9BD0
Kfm_05QHF00h>3P0MWIf06m_K`0_;bl0HF5Q01dM7@19BDT0Hf=S020P8017AdL0FEUI04a<C00f=SH0
BDU906UYJ@2ATI40Ng]k03Ph>01fMWH08R8R04Q8B02WYjL0HV9R03Hf=P0m?Cd0<C4a02LW9`11@D40
CTi>0410@00f=SH0IFEU0492@P1cLg<0];Bd07IfMP0Z:RX0HF5Q05]KF`1^KVh0BTY:05mOG`0l?3`0
F5QH07YjNP12@T80J6QX08B4Q01CDe<0N7Qh059BDP1iNGT0?clo07QhN00l?3`0<c<c05ADE01cLg<0
GUiN06YZJP0l?3`0Jf][05QHF01VIVH0Hf=S07moO`1EEED0BDU905QHF01QHF40?Cdm06IVIP0b<S80
Ogmo07MgM`1FEUH0ADE508^;R`0Z:RX0Shn?0:RXZ01fMWH0EeMG07alO01YJFT0];Bd07inOP10@400
Lg=c08F5Q@1gMgL0QHF504a<C01hN7P0Shn?05mOG`2CTi<0QhN703Xj>P0o?cl0Ph>30>3Ph03Ogml0
iNGU0<S8b01MGEd0EEEE09fMW@26QXH0OWin09jNWP1BDU80GemO08n?S`1KFe/0@D51079bLP0l?3`0
Jf][03/k>`1OGel0WIfM04Q8B01@D500ATI605MGE`23Ph<0EUIF061PH02IVIT0@d=30:>SX`1KFe/0
@41007moO`1IFET07QhN08J6QP1OGel0;2`/05YJFP1fMWH0LG5a06ATI01KFe/0UIFE04a<C014A4@0
N7Qh075aL@1QHF400P12@T80@`1XJ6P0YJFU09:BTP1MGEd0V9RH03Ph>014A4@0Ogmo09ZJVP2ATI40
EEEE07MgM`0_;bl0DE5A0:j^[P2ATI40B4Q805aLG00a<C40EEEE06IVIP1MGEd0Ogmo05iNGP0Y:BT0
I6AT08b<S029RHT0A4A403De=@1SHf<0@T9204U9B@1bLW80T92@061PH01JFUX0NGUi03<c<`1bLW80
FEUI05YJFP0];Bd0BTY:04=3@`0k>c/0Ogmo08N7Q`1>CTh0QHF505iNGP2>SXh0NGUi05mOG`2IVIT0
[Jf]06e]K@0a<C409bLW06m_K`1gMgL0@T9205ADE02>SXh0ATI6079bLP1dM7@03_ooool0E`1FEUH0
D51@04a<C018B4P0ADE505]KF`1fMWH0PX:205YJFP1WIfL0ZJVY05aLG01TI6@0?Cdm04E5A@39bLT0
e=CD0<g=c@3Kfm/0`L7105eMG@19BDT0Jf][0:JVYP21PH40J6QX04A4A017AdL0@d=305QHF01XJ6P0
FUYJ07MgM`1SHf<0IfMW05IFEP1NGUh0U9BD06m_K`1OGel0Ti>C05=CD`0l?3`0BDU907moO`1fMWH0
7AdM01`L700M7Ad0EeMG04e=C@0L71`09bLW038b<P13@d<0MGEe0:VYZ@1NGUh0C4a<059BDP24Q8@0
WIfM09FEU@1^KVh07QhN01`L700P8200AdM709nOW`2YZJT0QhN708>3P`2>SXh0SHf=092@T02=SHd0
[Jf]0=;BdP3@d=00_kno05eMG@1GEeL0Bd];02`/;019BDT0Vi^K06MWI`0201lO7`0^03<c<`1DE5@0
>c/k03/k>`1;Bd/0bLW909bLW01bLW80MWIf05]KF`16ATH0?Shn03/k>`0j>SX0=CDe02<S8`0U9BD0
;Bd]06YZJP1/K6`0?3`l06IVIP2:RXX0ADE505=CD`1mOGd0b<S80=CDe02;Rh/0UYJF04i>CP1fMWH0
K6a/03Lg=`0b<S80@D510820P03=cLd0ZJVY07YjNP22PX80_;bl0=oOg`2MWId0PH610=3@d080QHF5
00`0GEeM0;^k^`1fMWH0>c/k05=CD`20P8009bLW02HV9P0/;2`0GEeM05ADE01;Bd/202HV9P80:BTY
02h0EEEE04Y:BP2SXj<0dM7A0:BTY01VIVH0<C4a02@T900Y:BT0>c/k0;fm_@2GUiL0WYjN0;^k^`2Q
XJ40KVi^05YJFP1QHF40XZ:R09jNWP1>CTh0QHF50861P@1SHf<0JFUY08R8R02WYjL0[Zj^04m?C`19
BDT0?3`l03@d=00m?Cd0GemO0:6QX@1hN7P0SXj>0:n_[`2NWYh0^KVi0;Rh^01gMgL0;2`/01`L700O
7al08B4Q0P0L71`04P0N7Qh0MGEe0:BTY02b/[80]kNg09fMW@2GUiL0U9BD0;bl_02][Jd0`L710<C4
a038b<P0`<300=3@d03<c<`0eMGE0=7Ad@80_[jn0100`/;207emO@29RHT0RXZ:02/[:`0V9RH0<30`
061PH037alL0[Jf]08j>SP2RXZ80CDe=03Ti>@36a/H0[Zj^0P38b<P01@2e]KD0@T9202d];@0O7al0
92@T00808b<S02009bLW02h^;P13@d<0Cdm?06m_K`26QXH0C4a<07]kN`29RHT0GEeM03@d=00W9bL0
>c/k08B4Q02e]KD0`<300;Bd]028R8P0GEeM05UIF@2>SXh0PX:20::RXP3=cLd0Z:RX09FEU@3Jf]X0
[:b/0:ZZZP3:b/X0Ph>3051@D13ooooo000>ooooo`1c0;Fe]@1FEUH0De=C05ADE02BTY80OWin03De
=@18B4P0ATI60410@00a<C40G5aL04Y:BP0o?cl0J6QX030`<00n?Sh05QHF0492@P0c<c<06a/K01PH
600k>c/0820P04Y:BP0m?Cd0GemO04I6AP1?Cdl0PH6104Y:BP0/;2`0@d=302HV9P17AdL09RHV02LW
9`0b<S80K6a/069RHP0M7Ad0=3@d034a<@1;Bd/0DE5A04e=C@1DE5@0B4Q803Ti>@1OGel0IFEU03Lg
=`0M7Ad0BDU902d];@0H61P0C4a<06IVIP0h>3P0N7Qh09>CT`1OGel0JFUY01XJ6P0n?Sh0VIVI04Q8
B01EEED0J6QX04E5A@0h>3P09bLW03`l?01XJ6P0@410028R8P1kNg/0Fe]K065QH@3=cLd0@d=302/[
:`1GEeL0G5aL03dm?@1_Kfl0B4Q80451@@13@d<0SHf=05aLG02XZ:P0bLW908R8R00_;bl0KVi^05EE
E@0f=SH0BTY:06e]K@0h>3P0FUYJ05ADE00c<c<0C4a<06][J`18B4P0G5aL03Xj>P1RHV80>CTi075a
L@1VIVH0<S8b03`l?00205iNGP2Q07UiN@1DE5@0Jf][04U9B@1UIFD0LW9b08F5Q@1EEED0H61P07Ui
N@1lO7`0@D5107]kN`11@D40UiNG0:^[Z`1_Kfl0@T9208:2PP0g=cL0OGem096AT@1ZJVX0GUiN07=c
L`17AdL0S8b<09:BTP1=CDd0F5QH055AD@1XJ6P0QHF504M7A`22PX80AdM703<c<`1iNGT0QXJ604U9
B@1?Cdl0N7Qh0>CTi03Vi^H0j^[Z0<;2`P1IFET0D51@0:b/[02EUID0R8R809BDU01;Bd/0OWin0;Vi
^@1[Jf/0@T9207AdM011@D40GemO04e=C@1VIVH0UYJF05=CD`1;Bd/0C4a<061PH01^KVh0@d=306MW
I`2HV9P0?3`l09ZJVP15ADD0=CDe06i^KP0n?Sh06ATI07inOP1BDU80A4A406QXJ01KFe/0C4a<05YJ
FP1YJFT0T92@04=3@`0];Bd0GEeM061PH01SHf<0=cLg02h^;P0n?Sh0LW9b05YJFP17AdL0Ph>301lO
7`0R8R80ATI605ADE01iNGT0DE5A061PH00/;2`0?clo0;^k^`2;Rh/0AdM706m_K`1<C4`0EUIF0410
@015ADD0N7Qh0820P01:BTX0HF5Q08Z:RP1TI6@0=3@d05ADE025QHD0IfMW05mOG`1eMGD0LW9b02l_
;`1SHf<0Ng]k03Lg=`1ZJVX0HF5Q0492@P1>CTh0PH6105MGE`1>CTh0Ogmo08R8R01CDe<0RHV907=c
L`2ATI40NGUi05MGE`2ATI40[jn_08>3P`0k>c/0;Bd]06i^KP1;Bd/0;bl_06][J`2IVIT0>CTi071`
L00o?cl03_ooool0@@1EEED0Bd];0492@P0i>CT0=cLg03@d=013@d<0N7Qh0:^[Z`37alL0Cdm?01/K
6`1:BTX0D51@02DU9@23Ph<0[Jf]08j>SP2KVi/0MgMg03/k>`0/;2`0<C4a04A4A01bLW80PX:204i>
CP15ADD0BTY:06EUI@1iNGT0>CTi02/[:`0R8R80820P02@T9018B4P0R8R8092@T02:RXX0BTY:02h^
;P0P82009BDU05]KF`2_[jl0?Cdm01XJ6P0V9RH0QHF504I6AP0K6a/0:b/[05QHF00m?Cd0Rh^;05QH
F00Z:RX0:BTY02<S8`0L71`0:b/[07=cL`28R8P0:BTY008071`L0840;2`/075aL@2_[jl0Jf][069R
HP1TI6@0K6a/071`L01_Kfl0MWIf0::RXP2j^[X0d=3@0<71`@1]KFd0?clo04e=C@12@T80M7Ad0:FU
Y@0b<S8092@T06=SH`15ADD0;Rh^03<c<`1@D500Rh^;08j>SP1lO7`0MgMg03@d=00U9BD0;Bd]02DU
9@0X:2P0:BTY02/[:`0P82008b<S059BDP1FEUH0=cLg05UIF@1>CTh0A4A403hn?P1YJFT0ZZZZ0;2`
/01[Jf/0G5aL05]KF`1IFET0Ogmo079bLP17AdL0OGem0:>SX`37alL0Z:RX04U9B@1NGUh0Hf=S0:6Q
X@1RHV80YZJV0;jn_P1NGUh0FEUI06IVIP1`L700IfMW03De=@0^;Rh0IfMW08J6QP0];Bd0;bl_05UI
F@1YJFT0Bd];02LW9`0[:b/092@T059BDP1ADE40@D5106i^KP3Lg=`0H61P02HV9P0U9BD08b<S02DU
9@0W9bL0S8b<0=;BdP28R8P0G5aL0<G5a@24Q8@0DE5A04];B`1YJFT0alO70:j^[P1dM7@0EeMG05IF
EP1IFET0KFe]08F5Q@1cLg<0Kfm_04m?C`1@D500IfMW07=cL`29RHT0S8b<04];B`18B4P0P82008>3
P`2ATI40a/K60:j^[P0d=3@00P0L71`0;00J6QX06ATI01`L7016ATH0WYjN08:2PP3Ph>00Ng]k09nO
W`2>SXh0FUYJ09:BTP2EUID0UYJF09jNWP2i^KT0e=CD0>;RhP2_[jl0]KFe08b<S02CTi<0Z:RX0:^[
Z`1HF5P0GEeM0:2PX01=CDd0:b/[03Ti>@1_Kfl0^k^k04M7A`0j>SX0GEeM02l_;`0];Bd0SXj>09BD
U030`<00clo?0;>c/`0m?Cd08R8R0P0Q8B408P0T92@08R8R020P800U9BD0=CDe03Ph>01JFUX0IVIV
06a/K01EEED0Shn?05iNGP1VIVH0EeMG08f=S@3@d=00g=cL09FEU@1jNWX0D51@04Y:BP1GEeL0MgMg
0:2PX03=cLd0fm_K0;6a/@2PX:00`l?30>3Ph03Hf=P0WIfM08F5Q@1lO7`@ooooo`003_ooool0:`1c
Lg<0Cdm?055AD@1=CDd0NGUi07QhN00d=3@0?Cdm04M7A`0a<C40;Bd]04U9B@0n?Sh09BDU04U9B@0j
>SX0Bd];01DE5@0[:b/0>c/k024Q8@0G5aL0;bl_01XJ6P0h>3P0>CTi04m?C`12@T80;bl_055AD@0g
=cL0;2`/04A4A00`<300>3Ph02/[:`0/;2`08b<S04];B`10@4006ATI030`<01<C4`00P1;Bd/0<`0l
?3`0Fe]K06EUI@1HF5P0?Shn05UIF@0k>c/08b<S04m?C`1DE5@0<30`04I6AP0i>CT0:BTY05EEE@1i
NGT0Fe]K05MGE`0G5aL0@d=309:BTP1CDe<0E5AD05YJFP1=CDd0GUiN03hn?P1;Bd/0R8R80410@00V
9RH0PX:2069RHP1]KFd0c<c<07=cL`14A4@0@D5104E5A@0g=cL0Q8B405mOG`0a<C40:RXZ06YZJP1B
DU80Zj^[0;Zj^P2>SXh0<30`04m?C`0206IVIP0Z0410@01EEED0;Bd]06EUI@21PH40<30`04M7A`1?
Cdl0>CTi08B4Q01UIFD0EUIF020P801RHV80Ng]k03dm?@10@400?3`l04U9B@1dM7@0Cdm?079bLP0m
?Cd0=SHf065QH@1XJ6P0@41003Lg=`1IFET0KFe]03/k>`1jNWX09BDU051@D020P800Hf=S03dm?@22
PX80:b/[05iNGP1nOWh0RXZ:0P1YJFT0R01UIFD0MgMg09VIV@15ADD0AdM70861P@1OGel0SXj>04U9
B@1RHV80GUiN05aLG01oOgl0Hf=S03/k>`0[:b/0Jf][0=kNgP3Hf=P0c/k>0861P@1PH600L71`0;jn
_P2SXj<0UIFE0;6a/@0h>3P0O7al0;Vi^@1^KVh0BTY:06QXJ01;Bd/0SXj>051@D01>CTh0[Jf]06YZ
JP1PH600JVYZ05MGE`2;Rh/0;2`/061PH02WYjL0FEUI09JFUP1WIfL0=SHf061PH011@D40820P0820
P01jNWX0CDe=07EeM@1CDe<0K6a/079bLP2GUiL0YjNW06QXJ00X:2P0FUYJ04];B`1_Kfl0Hf=S04A4
A017AdL0MgMg05aLG019BDT0QhN704e=C@0_;bl0BTY:05QHF02;Rh/0L71`07QhN01:BTX0DE5A0<30
`01WIfL0?Shn08Z:RP13@d<0FUYJ04M7A`0a<C40=CDe05MGE`1<C4`0IfMW07IfMP11@D40820P028R
8P1TI6@0Hf=S05eMG@1RHV80N7Qh05EEE@1BDU80M7Ad02l_;`1SHf<0Cdm?038b<P0O7al0A4A404m?
C`0o?cl0MgMg04I6AP0f=SH0IfMW05mOG`2BTY80L71`03Xj>P1MGEd0S8b<065QH@0_;bl0CTi>08b<
S01YJFT0<S8b02d];@1bLW80<C4a092@T01mOGd>ooooo`0E0451@@0i>CT0=cLg03Ti>@1?Cdl0I6AT
07IfMP2GUiL0`/;20<?3``0h>3P07alO028R8P1`L700IFEU09^KV`3>c/h0cLg=07moO`13@d<0:BTY
00809BDU02<09RHV02/[:`0j>SX0SHf=06YZJP1RHV80GUiN08>3P`0c<c<06QXJ01`L700J6QX07AdM
03Xj>P21PH40]KFe075aL@1gMgL0G5aL02d];@0S8b<0BDU9096AT@2d];@0RXZ:09bLW039bLT0NWYj
03dm?@1QHF40EEEE03dm?@1XJ6P0;2`/01/K6`0301TI6@0401`L701KFe/0EEEE06ATI08071`L02<0
<S8b05QHF02FUYH0W9bL0451@@0b<S80=cLg03<c<`0];Bd0;bl_03`l?01[Jf/0[Jf]0<g=c@3@d=00
FEUI055AD@1FEUH0P8200=[JfP35aLD0YJFU0<W9b@2=SHd0?Shn02h^;P0j>SX0I6AT0820P01lO7`0
RXZ:059BDP1>CTh0H61P0410@00202@T900802/[:`0U9BD07alO03De=@1FEUH0>CTi05QHF00/;2`2
02@T901d02h^;P1`L700Ogmo02h^;P0[:b/0;Rh^07=cL`39bLT0]KFe07emO@1EEED0Fe]K08f=S@35
aLD0OWin03@d=00X:2P0GEeM0:>SX`2f][H0N7Qh04Q8B01>CTh0BDU906a/K01oOgl0Cdm?03<c<`0b
<S80NGUi09JFUP0j>SX0?3`l065QH@1DE5@0:b/[024Q8@0R8R80EeMG04];B`16ATH0NGUi0<g=c@11
@D408B4Q02<S8`0T92@08b<S02LW9`1lO7`0i>CT0=[JfP1jNWX0Jf][0;Vi^@2EUID0RHV90;^k^`3B
d]80RXZ:0410@011@D40Bd];09bLW02;Rh/0BTY:05=CD`1<C4`0GemO04U9B@1hN7P0YZJV09jNWP1A
DE40?3`l06][J`3:b/X0Fe]K08b<S02a/K40WYjN06MWI`0Q8B406QXJ01`L700G5aL0;2`/08J6QP1T
I6@0`/;20;^k^`23Ph<0];Bd0<30`02=SHd0BTY:03Ph>00f=SH0@T9206UYJ@2SXj<0fm_K0<c<c02P
X:00UIFE08^;R`1;Bd/0De=C09>CT`1XJ6P0LG5a08B4Q00o?cl09bLW05iNGP2:RXX202@T900902l_
;`0k>c/0<C4a069RHP2^[Zh0b/[:0;Bd]02b/[80;bl_00807QhN02<0820P01hN7P0T92@08b<S02DU
9@0g=cL0=3@d04A4A01UIFD0PH6106i^KP2CTi<0WYjN0::RXP2n_[h0aLG50;bl_02HV9P0^[Zj09bL
W02g]kL0b/[:0=3@d036a/H0]kNg0:VYZ@2RXZ80^k^k0=;BdP2`/;00Xj>S0=?Cd`2TY:@0Lg=c071`
L00@ooooo`003_ooool0o`1FEUH0F5QH055AD@1WIfL0O7al08F5Q@0b<S80ADE506][J`16ATH0>3Ph
04U9B@1[Jf/0<S8b05eMG@0/;2`0Bd];02h^;P0f=SH0CTi>01/K6`0I6AT0BDU902PX:00j>SX06ATI
03/k>`1]KFd0HV9R075aL@0_;bl07alO03/k>`1HF5P0>CTi01/K6`0g=cL0<S8b05mOG`12@T806QXJ
01`L701KFe/0EEEE02TY:@0b<S80Fe]K06][J`1hN7P0C4a<06EUI@0N7Qh06QXJ03De=@15ADD0;Rh^
04e=C@1fMWH0?3`l05MGE`25QHD0FEUI06EUI@0_;bl0<S8b08:2PP0k>c/0Fe]K05aLG015ADD0B4Q8
03Lg=`1LG5`0OWin04i>CP0Y:BT0LW9b03Lg=`0e=CD0QHF507MgM`1;Bd/0@d=302PX:00T92@0CDe=
06MWI`1MGEd0:b/[06UYJ@0^;Rh0Lg=c08:2PP2GUiL0?3`l03hn?P1;Bd/0IVIV04E5A@18B4P09bLW
03hn?P1aLG40;Rh^04Y:BP1RHV80<C4a06UYJ@1UIFD0G5aL03Lg=`1EEED0MWIf03lo?`0l?3`0H61P
05MGE`1_Kfl0De=C07emO@1fMWH0EeMG065QH@1CDe<0Rh^;05iNGP1JFUX0PH6103dm?@1gMgL0@T92
05]KF`2;Rh/0G5aL03lo?`2OWil0<S8b0861P@1^KVh0G5aL04U9B@2:RXX0JVYZ06i^KP26QXH0@410
075aL@2BTY80GemO09JFUP1<C4`0JFUY079bLP1iNGT0GEeM06MWI`1SHf<0=cLg03hn?P2c/k<0Vi^K
0::RXP1YJFT0;Rh^04I6AP24Q8@0Xj>S06EUI@2^[Zh0Cdm?055AD@2IVIT0Lg=c055AD@1OGel0C4a<
08j>SP1BDU80@d=30::RXP1GEeL0ADE506EUI@1ADE40W9bL059BDP1DE5@0^KVi05MGE`2<S8`0VIVI
04E5A@1ZJVX0CTi>03<c<`2;Rh/0W9bL04a<C01VIVH0D51@06i^KP0f=SH0CDe=06QXJ01YJFT09BDU
071`L010@400EEEE07AdM017AdL0ATI607moO`0i>CT0?Shn0861P@1TI6@0<S8b04];B`1LG5`0GemO
03`l?01WIfL0JFUY04Y:BP2l_;`0LG5a03Ti>@1XJ6P0ATI6059BDP0o?cl0F5QH03dm?@1mOGd0HV9R
06MWI`1bLW80GEeM04U9B@0T92@0L71`07MgM`1/K6`0QhN70:NWY`1YJFT0DU9B075aL@0T92@0Q8B4
08j>SP1>CTh092@T051@D0005`18B4P0?Cdm0:FUY@2;Rh/0E5AD06IVIP1DE5@0W9bL09ZJVP1^KVh0
RHV90:^[Z`1kNg/07QhN02TY:@1DE5@0M7Ad03De=@0^;Rh0KVi^01DE5@1QHF40K6a/00kooooo01H0
KVi^07QhN01gMgL0MGEe06=SH`19BDT0=cLg038b<P0c<c<0FUYJ08f=S@25QHD0QXJ606UYJ@29RHT0
d=3@0=7Ad@3EeMD0Xj>S05EEE@0Y:BT08b<S0P0X:2P0200_;bl0;Rh^05IFEP0a<C40;2`/03Lg=`1@
D5008B4Q0P0K6a/05`0L71`07alO02<S8`0^;Rh0FEUI065QH@0];Bd0CDe=06IVIP0b<S80:BTY0:BT
Y03De=@0]kNg08Z:RP2KVi/0OWin0:2PX02a/K40IFEU05MGE`1;Bd/08R8R008071`L00`06QXJ01`L
700K6a/0EeMG02XZ:P1LG5`0CDe=02`/;01HF5P0E5AD061PH01_Kfl201XJ6P0C01/K6`0O7al08b<S
02XZ:P0/;2`0>c/k069RHP1`L700YZJV0;Jf]P1PH600WYjN0<K6aP2XZ:P0FUYJ05ADE01]KFd0^[Zj
09nOW`0207UiN@0B0820P01kNg/0Kfm_05]KF`0X:2P0:BTY0410@01@D500Cdm?02HV9P0Z:RX0?clo
024Q8@0S8b<0EeMG04Y:BP1/K6`092@T0P0P8200P`0T92@0@41006UYJ@0Z:RX0?Cdm08B4Q02XZ:P0
PX:20:6QX@3:b/X0_kno0:^[Z`3>c/h0/[:b08B4Q00P8200@d=308R8R02>SXh0KFe]04a<C00n?Sh0
=cLg02XZ:P1QHF40Ng]k07EeM@0a<C40;Bd]03@d=01/K6`0UYJF04m?C`1dM7@0J6QX04Y:BP0P8200
<c<c05UIF@1<C4`0De=C0410@02YZJT0D51@01lO7`0W9bL0:b/[02DU9@0];Bd0Shn?0>CTi03Ng]h0
`L710;bl_02b/[80^k^k0=?Cd`3Jf]X0[jn_0;Rh^01nOWh0JFUY07EeM@35aLD0J6QX02TY:@0/;2`0
>3Ph04a<C029RHT0]kNg0;Jf]P2OWil0HF5Q05YJFP2e]KD0PH6105=CD`20P800S8b<0;Zj^P2_[jl0
ATI602PX:00N7Qh0@D5108N7Q`2`/;00_;bl06IVIP1^KVh0LW9b05EEE@26QXH0M7Ad01`L700N7Qh0
<C4a030`<00i>CT0De=C0=[JfP3Bd]80T92@05ADE01aLG40EEEE03hn?P1NGUh0P82008:2PP1YJFT0
EEEE02l_;`1ADE40IfMW01`L700Q8B408R8R038b<P0k>c/0MgMg0<;2`P3QhN40X:2P08V9R@0b<S80
7AdM01`L700M7Ad071`L00807AdM01l09BDU034a<@0b<S80?clo05MGE`1YJFT0T92@0;bl_03Ti>@0
dm?C0<K6aP2BTY80FEUI04m?C`2XZ:P0IFEU09VIV@1SHf<0EeMG05EEE@1OGel0[Zj^0=7Ad@33`l<0
^KVi0:ZZZP2UYJD0Xj>S0;6a/@20P800T92@013ooooo000>ooooo`3K079bLP1>CTh0De=C06UYJ@1h
N7P0Ph>302l_;`0c<c<0EEEE03De=@0g=cL0@T92075aL@0];Bd0GemO04=3@`1@D500=CDe02d];@1H
F5P0=CDe01HF5P15ADD09RHV03Xj>P0W9bL0>SXj07UiN@1RHV80NWYj06e]K@0Z:RX0>3Ph06IVIP0o
?cl0>CTi03`l?00a<C40Lg=c06YZJP0n?Sh08R8R04Y:BP1BDU80IfMW02l_;`1ADE40M7Ad06=SH`1E
EED0N7Qh04e=C@0P8200<30`024Q8@0H61P0@D5105eMG@0`<300Fe]K07AdM00S8b<0ATI6030`<00a
<C40Ogmo02`/;00U9BD0EEEE04I6AP1?Cdl06ATI030`<01KFe/0AdM702PX:02CTi<0KVi^04];B`2S
Xj<0R8R803Ph>01PH600C4a<03Hf=P1NGUh0AdM704I6AP0P8200IVIV05IFEP1mOGd0V9RH0:b/[00g
=cL0Kfm_05mOG`1HF5P0DE5A07YjNP1UIFD0ATI606IVIP0d=3@0Kfm_09>CT`1>CTh0L71`04a<C01c
Lg<0EeMG05EEE@1MGEd0=CDe04e=C@25QHD0GemO07AdM00_;bl0GEeM08R8R01HF5P0ADE504Y:BP2:
RXX0GUiN05IFEP1DE5@0?Cdm0:j^[P1@D500F5QH06i^KP0c<c<09RHV07MgM`0W9bL0NGUi06i^KP1l
O7`0EUIF051@D01UIFD0GUiN08^;R`0k>c/0F5QH09NGU`1KFe/0Kfm_04M7A`2HV9P0VIVI08R8R01@
D500RXZ:09RHV01:BTX0Rh^;0:RXZ02DU9@0QhN709jNWP1bLW80DU9B08b<S01^KVh0@D510:JVYP1/
K6`0EeMG09fMW@1QHF40H61P08V9R@19BDT0M7Ad05aLG010@400YjNW05aLG00l?3`0JVYZ055AD@2O
Wil0OWin05aLG02b/[80GEeM0:n_[`2ZZZX0?Shn07inOP15ADD07alO06m_K`2@T900Bd];07moO`1S
Hf<0VIVI06i^KP1GEeL0UYJF08:2PP0o?cl0OGem08V9R@1JFUX0LG5a05IFEP1RHV80XZ:R08R8R013
@d<0LG5a05]KF`12@T80N7Qh0080X:2P02l0EUIF07emO@1YJFT0EUIF0;no_`2DU9@0HF5Q07moO`1M
GEd0I6AT07AdM01cLg<0>SXj05ADE01EEED0OWin0:^[Z`23Ph<0FEUI02@T901EEED0:RXZ06m_K`2J
VYX0];Bd06UYJ@0n?Sh0R8R801TI6@12@T80EEEE06IVIP0_;bl0E5AD06IVIP14A4@0LG5a07moO`1K
Fe/0K6a/06=SH`2HV9P0LG5a07AdM02@T900_Kfm07AdM00203Ph>00804A4A01aLG40@T9203hn?P1o
Ogl08b<S06QXJ01MGEd>ooooo`0606QXJ013@d<0<S8b02@T900O7al07AdM0P0J6QX06@0E5AD0@d=3
06YZJP2IVIT0Hf=S06a/K02WYjL0OGem09:BTP3De=@0_;bl09NGU`1lO7`0:RXZ02PX:00];Bd0:b/[
03dm?@14A4@0:BTY02LW9`0j>SX07alO01`L700K6a/00P0M7Ad0900O7al07QhN028R8P0e=CD0DE5A
04];B`0S8b<0<30`06m_K`23Ph<0WIfM09ZJVP22PX80Hf=S05iNGP1iNGT0LW9b0:2PX02ATI40Hf=S
02LW9`0Q8B40:2PX02`/;00T92@07alO01`L701MGEd09bLW04i>CP1`L700Hf=S04m?C`0n?Sh0Cdm?
06ATI0D071`L05L07AdM01lO7`0Q8B40@41005ADE01GEeL0Ti>C0=[JfP3Bd]80bl_;07AdM00g=cL0
=3@d03`l?01DE5@0QXJ60:6QX@2JVYX0]kNg0;:b/P2ZZZX0@D5102DU9@0V9RH08b<S02@T900^;Rh0
FUYJ059BDP0W9bL092@T02h^;P2YZJT0^KVi08R8R00Q8B40820P01dM7@0S8b<0?Cdm0;fm_@2[Zj/0
W9bL08J6QP1MGEd0FEUI08>3P`1dM7@0GEeM06EUI@2EUID0Ng]k09bLW01SHf<0M7Ad03lo?`1bLW80
HF5Q05YJFP0c<c<0:RXZ02PX:010@400IfMW09^KV`1HF5P0<C4a02d];@0g=cL0Lg=c0;6a/@2ATI40
XZ:R04e=C@13@d<0LW9b07QhN01RHV809BDU02h^;P26QXH0K6a/02<S8`0X:2P0;bl_030`<016ATH0
0P3;bl/02`2g]kL0Hf=S059BDP2YZJT0]KFe0;Vi^@3EeMD0e=CD0<W9b@2e]KD0WIfM0080clo?0100
Cdm?02TY:@0U9BD09RHV03Ti>@2:RXX0`/;20:6QX@25QHD0_Kfm0:2PX01VIVH07alO01dM7@0Z:RX0
T92@0P2TY:@0302HV9P0L71`04a<C01kNg/0`<300;Vi^@1FEUH0Kfm_07MgM`0e=CD0:RXZ04a<C080
E5AD0240=CDe02TY:@0[:b/0<C4a06i^KP3Cdm<0bl_;05eMG@1LG5`0De=C05IFEP1?Cdl0AdM705UI
F@1YJFT0XZ:R07MgM`1ADE40EeMG05eMG@0O7al07AdM01lO7`0U9BD0Fe]K0;no_`3Bd]80][Jf0:JV
YP1^KVh0Bd];01hN7P0L71`00`0M7Ad02@0Q8B408R8R02<S8`0V9RH0;bl_04I6AP1TI6@0][Jf0=cL
g0020<o?c`0E0=KFeP2LW9`0Zj^[0:n_[`2RXZ80UYJF08:2PP0i>CT0>c/k04I6AP22PX80];Bd0=CD
e037alL0^k^k0<O7a`2c/k<0X:2P0:>SX`3EeMD0gMgM013ooooo000>ooooo`3[07]kN`1?Cdl0CTi>
05=CD`21PH40UIFE04A4A00h>3P0DU9B03lo?`1OGel0AdM706ATI00_;bl0K6a/04e=C@0o?cl08b<S
02DU9@19BDT0;2`/01TI6@10@4007QhN04M7A`0_;bl0?Cdm05aLG00n?Sh0QhN707alO00Z:RX0=3@d
02l_;`0b<S80D51@02d];@0C4a<0@41007MgM`0n?Sh09BDU02TY:@0_;bl0BDU902PX:01DE5@0LG5a
034a<@0_;bl0@d=305]KF`0];Bd0?3`l05IFEP0n?Sh0?Cdm034a<@0K6a/0K6a/09ZJVP1gMgL0E5AD
03dm?@0[:b/0OWin071`L00^;Rh0LW9b04I6AP1cLg<0;Rh^03lo?`1eMGD0>3Ph01HF5P1^KVh0Ng]k
05=CD`2XZ:P0QXJ602l_;`1DE5@0GUiN0451@@1aLG40Cdm?04E5A@0T92@0MgMg06MWI`2VYZH0WinO
0<G5a@0a<C40IVIV05MGE`1BDU80?Shn065QH@1NGUh0@D5106QXJ00b<S80De=C06e]K@1>CTh0GEeM
04=3@`1CDe<0D51@04I6AP1lO7`0<S8b03@d=01SHf<0De=C05YJFP1VIVH0DE5A07YjNP1DE5@0Jf][
04m?C`23Ph<0EEEE05YJFP1ADE407alO08b<S01EEED0EUIF06YZJP0o?cl0<30`07inOP0/;2`0QXJ6
0:b/[029RHT0GEeM061PH01mOGd0X:2P0;^k^`1HF5P0DE5A08B4Q01TI6@0SXj>05EEE@1UIFD0T92@
07emO@16ATH0YZJV06QXJ01MGEd0OGem09:BTP0d=3@0>CTi07EeM@1OGel0DU9B07QhN020P800BTY:
09FEU@1ZJVX0K6a/0:NWY`1dM7@0:RXZ05IFEP12@T80HF5Q04=3@`1UIFD0WinO06][J`12@T80EeMG
05eMG@2BTY80MgMg059BDP2a/K40AdM706=SH`2=SHd0>SXj0:>SX`2<S8`0>c/k07AdM02/[:`0<S8b
06IVIP1^KVh0Shn?07QhN01^KVh0Ti>C0410@00O7al0C4a<08V9R@1]KFd0AdM701hN7P0f=SH0HF5Q
08:2PP0o?cl0Shn?038b<P0O7al0ATI606MWI`2BTY80I6AT07EeM@0b<S80=cLg08N7Q`2ATI40IfMW
06][J`0X:2P0<C4a059BDP1YJFT0@D510492@P0204e=C@0P05]KF`1QHF40EeMG024Q8@1MGEd0IfMW
071`L01DE5@0VYZJ06ATI01?Cdl0R8R804M7A`1DE5@0F5QH07IfMP0j>SX0Ogmo08^;R`1CDe<0RHV9
08J6QP1KFe/0T92@07IfMP2ZZZX0HV9R03Hf=P1WIfL0ZJVY08b<S01HF5P206MWI`0706a/K00n?Sh0
<c<c08>3P`11@D40MGEe051@D00>ooooo`0N04Y:BP15ADD0@T9203<c<`0P820071`L01TI6@0H61P0
5ADE05ADE02@T900P8200<G5a@3=cLd0VIVI02PX:01VIVH0XZ:R08:2PP2XZ:P0OGem071`L00h>3P0
<30`02HV9P12@T80;Bd]02PX:00Z:RX08B4Q0P0E5AD00`0F5QH06a/K01PH600201`L700R01dM7@0T
92@0D51@04i>CP0n?Sh07AdM03dm?@37alL0Ti>C09^KV`1UIFD0<C4a024Q8@0`<300CDe=05ADE01l
O7`0MWIf05ADE00e=CD09RHV028R8P0N7Qh071`L01TI6@1`L700@d=305UIF@1nOWh0T92@0410@00`
<300AdM706m_K`8071`L00<06QXJ01`L700L71`00`0L71`05P0T92@0F5QH05ADE020P800h>3P0=7A
d@2g]kL0E5AD03@d=00V9RH0=cLg03hn?P18B4P0O7al0:j^[P2m_Kd0[Jf]0;>c/`1PH600;Bd]02DU
9@0O7al2024Q8@0902/[:`0m?Cd0BDU907inOP2?Shl0ZJVY0;Bd]02UYJD0HV9R00<07alO01D0Hf=S
0<C4a021PH40EUIF04i>CP1;Bd/0B4Q807QhN012@T80?Shn04I6AP13@d<0Hf=S0451@@1lO7`0>c/k
02DU9@1/K6`0G5aL065QH@0Y:BT00P0T92@0400i>CT0LG5a07YjNP19BDT0D51@06YZJP1YJFT0QHF5
09VIV@2NWYh0O7al069RHP1MGEd0Kfm_08n?S`2:RXX20492@P1[07inOP29RHT0:2PX02TY:@0f=SH0
@d=307QhN034a<@0YJFU0:NWY`1KFe/0I6AT0:j^[P2FUYH0^[Zj0:^[Z`2GUiL0][Jf09VIV@1SHf<0
NWYj0;6a/@2MWId0?Cdm02d];@0Z:RX0:b/[06EUI@2^[Zh0^KVi0;Bd]03WinL0TY:B01lO7`0J6QX0
7AdM03<c<`32`/80g=cL0<C4a037alL0WYjN0:RXZ03<c<`0clo?08Z:RP2IVIT0[Jf]04M7A`0L71`0
8R8R02d];@0[:b/0NWYj07QhN01<C4`0I6AT08N7Q`2m_Kd0e=CD0<S8b01/K6`0B4Q804=3@`11@D40
CTi>05eMG@1DE5@0F5QH07inOP2CTi<0VYZJ09VIV@1WIfL0=SHf03Lg=`0^;Rh0;bl_07inOP2g]kL0
`<3008>3P`1/K6`0JVYZ0:b/[01DE5@0@41003/k>`0^;Rh092@T01hN7P0Q8B407alO01hN7P0V9RH0
BDU908^;R`2/[:`0b<S80:ZZZP2m_Kd0e=CD0;jn_P2UYJD0XZ:R07IfMP1=CDd00P24Q8@03@2@T900
`/;20;Bd]02][Jd0RHV907MgM`1[Jf/0UiNG0=3@d03IfMT0`L710;jn_P2=SHd04?ooool000kooooo
0=T0H61P04=3@`1>CTh0E5AD05iNGP1XJ6P0:b/[03Hf=P11@D40:b/[03Xj>P0o?cl0@d=301XJ6P0m
?Cd0AdM704e=C@0[:b/06ATI04I6AP0W9bL06ATI04e=C@0V9RH0<c<c020P800e=CD0EEEE038b<P1a
LG40FUYJ028R8P0k>c/0EUIF03`l?00i>CT0?Shn02@T901@D500GemO02d];@0^;Rh0FEUI04m?C`12
@T80:RXZ05YJFP1nOWh0MgMg04Y:BP1=CDd0ADE501hN7P0^;Rh0?clo03Hf=P0k>c/0I6AT03Ti>@1;
Bd/0MWIf08F5Q@1@D500?Shn02<S8`1gMgL0MGEe02d];@1KFe/0@T9208f=S@16ATH0?clo061PH01Q
HF40>CTi06a/K01HF5P0BTY:0;>c/`2TY:@0HF5Q0410@016ATH0ADE5069RHP1<C4`0DE5A02LW9`1R
HV80GUiN06i^KP2SXj<0_Kfm05iNGP1LG5`0EEEE06i^KP1:BTX0NGUi04=3@`1ADE40NGUi04];B`1Q
HF40Kfm_055AD@1KFe/0LW9b06=SH`1?Cdl0Bd];0820P01;Bd/0ATI604m?C`14A4@0GemO06UYJ@14
A4@0M7Ad03Ph>01WIfL0ATI607EeM@11@D40IFEU07UiN@16ATH0WIfM04A4A01FEUH0MWIf061PH013
@d<0PX:203Ph>01_Kfl0Ph>306m_K`1FEUH0RHV907]kN`1LG5`0TY:B051@D01?Cdl0RHV905eMG@1c
Lg<0CDe=07inOP2NWYh0Hf=S03`l?02?Shl0A4A404=3@`1WIfL0XZ:R07]kN`1;Bd/0RXZ:04M7A`1;
Bd/0Kfm_09ZJVP1BDU80UiNG04i>CP16ATH0RHV909BDU014A4@0JFUY03@d=01NGUh0IfMW03/k>`2K
Vi/0DU9B03lo?`1lO7`0>c/k096AT@0m?Cd0>SXj08>3P`1`L700JVYZ09BDU00/;2`0Hf=S07emO@0m
?Cd0Ng]k0:VYZ@1EEED0HV9R05aLG01VIVH0B4Q807EeM@22PX80HF5Q02`/;01>CTh0Hf=S05iNGP1[
Jf/0DU9B04E5A@29RHT0Kfm_0451@@2ATI40NWYj0080@T9203/0Fe]K09BDU01KFe/0LG5a07MgM`1H
F5P0YJFU07IfMP1;Bd/0OWin05aLG01;Bd/0BTY:0451@@10@400FUYJ06ATI01?Cdl0M7Ad06m_K`1C
De<0:b/[071`L01jNWX0JVYZ05=CD`2DU9@0MgMg0492@P2MWId0DU9B06IVIP1YJFT0De=C02/[:`1N
GUh0OWin04I6AP2GUiL0Q8B404i>CP1cLg<0IVIV09bLW01`L700Lg=c07UiN@2b/[80MgMg03/k>`0c
<c<0?Cdm075aL@1BDU809bLW07alO00h>3P0Kfm_07inOP0>ooooo`0`04];B`17AdL0@T9203Lg=`0j
>SX0=SHf03lo?`1BDU80Bd];04A4A010@400Ng]k0:6QX@2KVi/0Xj>S07AdM01eMGD0UYJF059BDP1;
Bd/0HV9R06EUI@1bLW80GUiN05aLG011@D40;2`/028R8P0O7al06QXJ01DE5@0D51@04a<C01DE5@0I
6AT06QXJ01/K6`0M7Ad071`L02LW9`16ATH0G5aL03Xj>P0^;Rh0MGEe05QHF01YJFT0@d=30P0l?3`0
2P14A4@0J6QX096AT@1ZJVX0CDe=05ADE01VIVH0I6AT06QXJ01LG5`2051@D00c0:>SX`1mOGd0Hf=S
05iNGP1SHf<0PX:207=cL`1IFET0ZJVY01lO7`0K6a/071`L01/K6`0L71`07AdM01hN7P0M7Ad0:RXZ
04a<C02LW9`0WIfM0:n_[`30`<00QHF507IfMP1aLG40EEEE05eMG@1_Kfl0Fe]K05ADE01YJFT0PH61
0:JVYP28R8P0IFEU06=SH`1BDU80>CTi02/[:`0O7al07QhN02h^;P1OGel0I6AT05MGE`12@T80NWYj
06][J`1XJ6P0L71`0080KVi^00d0Ogmo079bLP1>CTh0BTY:04a<C015ADD0EUIF06a/K00e=CD0=cLg
03lo?`0n?Sh0ATI60080HV9R04L0:b/[01lO7`1JFUX0G5aL06a/K00[:b/07QhN020P800U9BD0E5AD
05=CD`11@D409bLW03/k>`2@T900SXj>09>CT`25QHD0Hf=S03Xj>P0e=CD0;Rh^03Xj>P1BDU80IfMW
06IVIP2g]kL0bl_;092@T01kNg/0N7Qh09bLW02YZJT0G5aL03<c<`0m?Cd0QhN70;Ng]`20P800Lg=c
0:2PX01nOWh0JFUY04E5A@0/;2`0?3`l04];B`1^KVh0L71`08>3P`1cLg<0BDU904m?C`27QhL0MWIf
06m_K`19BDT0TY:B04Q8B00W9bL0GUiN08F5Q@2RXZ80^;Rh09FEU@30`<00b/[:0;Zj^P3Bd]80emOG
09bLW00206=SH`0306EUI@1TI6@0<C4a008071`L03d0820P03<c<`22PX80^;Rh08Z:RP1`L700Shn?
0<g=c@37alL0Vi^K05IFEP19BDT0ADE504Q8B01ADE40JVYZ09FEU@37alL0_kno0<K6aP1mOGd0TY:B
0861P@1nOWh0MGEe06UYJ@21PH40a<C40=CDe02TY:@0R8R809BDU01/K6`0AdM70410@013@d<0@410
04E5A@1?Cdl0GemO07MgM`1mOGd0Lg=c07emO@2b/[80alO70=[JfP2c/k<0];Bd05EEE@0c<c<0=SHf
02l_;`12@T80FUYJ08f=S@3Kfm/0emOG0;jn_P2DU9@0DU9B0080=CDe00L0?Shn04m?C`2][Jd0dm?C
096AT@2@T900NWYj013ooooo000>ooooo`3:07QhN01]KFd0AdM705mOG`1WIfL0OGem0410@00a<C40
C4a<030`<019BDT0?3`l07=cL`0c<c<0>CTi03Hf=P17AdL0DU9B01/K6`1ADE409RHV01DE5@10@400
>SXj03De=@13@d<0:BTY05aLG01GEeL0LW9b04a<C00P8200<S8b06UYJ@0g=cL0@d=303Ph>00O7al0
C4a<05iNGP0Z:RX07AdM051@D01=CDd0=CDe02<S8`11@D40GEeM075aL@12@T80GUiN04Y:BP0];Bd0
<30`03Ph>00b<S80A4A406YZJP12@T80ATI607]kN`1cLg<0H61P07=cL`0M7Ad0E5AD0820P00W9bL0
JFUY04a<C024Q8@0>CTi0492@P1aLG40De=C02`/;01DE5@0<C4a0410@02n_[h0WYjN05YJFP19BDT0
>CTi02<S8`18B4P0P82006UYJ@0i>CT0GUiN03lo?`0c<c<0CDe=079bLP1DE5@0EeMG03`l?01HF5P0
?clo07=cL`1?Cdl0?3`l051@D012@T80O7al09JFUP19BDT0EEEE069RHP1WIfL0Hf=S03hn?P21PH40
<c<c04m?C`23Ph<0B4Q805mOG`19BDT0@T9209>CT`1ZJVX0@T9203hn?P2DU9@0H61P06=SH`10@400
8b<S09>CT`1fMWH0EEEE07=cL`0d=3@06ATI079bLP1EEED0LG5a0<C4a01=CDd0=cLg04i>CP1aLG40
OWin0:6QX@12@T80=3@d06e]K@1ZJVX0G5aL04A4A01CDe<0Ph>308R8R01ADE40MGEe07QhN01`L700
T92@0<71`@29RHT0BTY:08b<S01hN7P0B4Q806=SH`2:RXX0C4a<09fMW@1bLW80EUIF08Z:RP2JVYX0
?Shn06IVIP14A4@0Bd];07QhN00j>SX0UYJF08>3P`15ADD0Lg=c03<c<`2?Shl0M7Ad05IFEP28R8P0
Fe]K07inOP2`/;00@T9206QXJ01cLg<0:2PX06a/K024Q8@0EUIF07YjNP2DU9@0NGUi04a<C080K6a/
00d0GemO03Xj>P1NGUh0K6a/06IVIP1]KFd0EeMG05aLG02KVi/0S8b<04];B`1lO7`0F5QH0080BTY:
03/0Hf=S0:JVYP1QHF40KFe]06ATI01?Cdl0[Zj^0:2PX023Ph<0JFUY05]KF`1MGEd0IVIV07]kN`18
B4P0D51@03`l?01;Bd/0UIFE08f=S@1kNg/08B4Q055AD@1SHf<0N7Qh09BDU035aLD0SXj>04Q8B02H
V9P0;Bd]051@D01>CTh0JFUY03`l?01DE5@0GEeM02l_;`1UIFD0OGem069RHP1bLW80NWYj0:2PX01D
E5@0KVi^08Z:RP37alL0V9RH05=CD`0h>3P0@41008J6QP1jNWX0?Cdm07QhN01IFET0JFUY06YZJP0>
ooooo`0L03`l?00j>SX0?Shn03Ph>00Z:RX0:BTY03dm?@1iNGT0BDU902`/;01bLW80D51@01lO7`0j
>SX0LG5a06i^KP33`l<0gMgM0=7Ad@2^[Zh0TI6A07moO`1hN7P0VYZJ09:BTP1OGel09RHV01`L7080
5QHF100E5AD0<P0G5aL06ATI01XJ6P0G5aL06QXJ01hN7P0W9bL0H61P06e]K@18B4P0Jf][07UiN@0Y
:BT05QHF01TI6@0Z:RX0<c<c061PH00a<C408b<S02/[:`0];Bd0=cLg051@D01]KFd0MgMg02<S8`0K
6a/0EeMG0:ZZZP2CTi<0U9BD08j>SP2LW9`0^KVi09fMW@3Jf]X0K6a/065QH@1IFET0>c/k03Hf=P0X
:2P09RHV02LW9`0Y:BT0;bl_04E5A@2/[:`0b/[:0P3=cLd03@3GemL0clo?0:VYZ@2<S8`0K6a/05]K
F`26QXH0NGUi07QhN02IVIT0MWIf07]kN`2=SHd00P2:RXX00`1UIFD0?Cdm04i>CP0206=SH`0=0820
P01@D500A4A40861P@2n_[h0N7Qh05EEE@1oOgl0`l?30;6a/@1fMWH0HV9R03Ph>00202l_;`1908J6
QP1MGEd0=3@d02l_;`0c<c<0A4A4059BDP2VYZH0JFUY0861P@28R8P0Z:RX05]KF`2CTi<0D51@028R
8P0P82007alO04U9B@1CDe<0G5aL02h^;P1/K6`0DE5A06MWI`1DE5@0?Cdm05EEE@1<C4`0E5AD05EE
E@1=CDd0>CTi03`l?01_Kfl0/K6a08F5Q@2c/k<0VYZJ0<C4a03;bl/0^;Rh07MgM`1ADE40O7al04Y:
BP1DE5@0MWIf05=CD`1YJFT0QhN704Q8B00L71`08B4Q02HV9P0U9BD0C4a<05QHF01GEeL0H61P0;Bd
]033`l<0[Zj^061PH01WIfL0:BTY07EeM@1oOgl0QHF5051@D01oOgl0MGEe06i^KP0207moO`1B09ZJ
VP2MWId0`<300<_;b`0o?cl0>CTi04e=C@0`<300?Shn06UYJ@13@d<07AdM02<S8`0m?Cd0Q8B40;Ng
]`2CTi<0AdM708^;R`39bLT0UiNG061PH02;Rh/0UYJF06][J`19BDT0Cdm?07IfMP2j^[X0`l?30;jn
_P2m_Kd0Ogmo06][J`2;Rh/0OGem08Z:RP32`/80/K6a071`L01oOgl0_;bl09nOW`1FEUH0MGEe04E5
A@16ATH0G5aL09BDU02a/K40bLW90<S8b02PX:00SHf=0:FUY@2GUiL0[:b/0;>c/`2f][H0d];B0<S8
b01kNg/0>c/k02<S8`0L71`06QXJ01lO7`0Q8B40bLW90:^[Z`2g]kL0R8R802h^;P0Z:RX0=3@d03Ph
>019BDT0WYjN0:f][@1NGUh0O7al06i^KQ3ooooo000>ooooo`1409>CT`1nOWh0Bd];06a/K01TI6@0
T92@06IVIP0n?Sh0De=C03Xj>P1KFe/0ADE508V9R@11@D40@41005YJFP1ADE40I6AT020P801CDe<0
@d=302<S8`0a<C40=cLg04U9B@0k>c/0=3@d05EEE@1DE5@0MGEe03Xj>P0L71`0>CTi06EUI@0j>SX0
De=C03`l?00I6AT0ATI608b<S018B4P0:b/[0451@@0n?Sh0AdM702l_;`1;Bd/0KFe]04e=C@0`<300
IfMW06YZJP15ADD0<C4a04A4A00M7Ad0=3@d05QHF00i>CT0>SXj07=cL`1;Bd/0?Cdm05eMG@0M7Ad0
@d=305QHF00J6QX204E5A@0@06e]K@0I6AT0;bl_04Y:BP1=CDd0@41006YZJP1DE5@0?3`l0:FUY@2D
U9@0@d=306QXJ02IVIT0?clo03Hf=P80AdM70;D0=CDe08F5Q@28R8P0RXZ:0;6a/@32`/80?clo06e]
K@1GEeL0FEUI01dM7@1<C4`0RHV905=CD`1>CTh07alO04I6AP1aLG40LW9b05YJFP0l?3`0>c/k07em
O@18B4P0Q8B402l_;`0Z:RX0FUYJ04];B`1OGel0I6AT03Ti>@1`L700I6AT061PH00g=cL0VIVI061P
H01FEUH0H61P05UIF@2EUID0M7Ad05aLG022PX80EEEE02`/;01`L700=cLg061PH03;bl/0SXj>05YJ
FP1hN7P0PX:20:NWY`32`/80RXZ:05ADE01OGel0L71`07QhN021PH40GemO08F5Q@23Ph<0GemO06m_
K`2?Shl0Q8B40;6a/@2l_;`0Ogmo03lo?`2NWYh0O7al04];B`22PX80X:2P03`l?02@T900JVYZ051@
D02=SHd0WIfM04e=C@1oOgl0C4a<0492@P1lO7`0?clo0:NWY`23Ph<0ATI606QXJ00k>c/0Ti>C07em
O@1BDU80SHf=059BDP1EEED0V9RH02PX:01fMWH0OWin03<c<`1XJ6P0LG5a04];B`1=CDd0DE5A059B
DP0n?Sh0P82006ATI01KFe/0;Rh^03dm?@1GEeL0GEeM05=CD`0f=SH0EEEE08J6QP1cLg<0@41007Yj
NP0m?Cd0<30`05UIF@1iNGT0QhN705IFEP1LG5`0FEUI03/k>`1[Jf/0HV9R06UYJ@1TI6@0?clo03dm
?@13@d<0FUYJ04A4A01EEED0FEUI03`l?01LG5`0L71`06m_K`0N7Qh0Cdm?04M7A`11@D40BDU907Ui
N@1kNg/0BDU908J6QP0l?3`0FUYJ05=CD`1oOgl0CTi>04];B`1@D500<C4a08J6QP1`L700FEUI06MW
I`21PH40VYZJ05UIF@1?Cdl0Ogmo0:n_[`2LW9`0GEeM0080>3Ph00L0IfMW03/k>`1<C4`0RHV905eM
G@1KFe/0GemO00kooooo0200@d=303@d=00_;bl0?Shn02l_;`0S8b<0F5QH05IFEP0g=cL0?Shn04Y:
BP0H61P06a/K03De=@1]KFd0DU9B0:>SX`2f][H0X:2P09ZJVP2MWId0[:b/0<W9b@2n_[h0OGem06][
J`0^;Rh06QXJ01DE5@0H61P06a/K01LG5`<05ADE044071`L01/K6`0M7Ad07AdM01hN7P0M7Ad0A4A4
0:j^[P2MWId0V9RH0<?3``1YJFT071`L03Lg=`0j>SX0ADE501PH600E5AD051@D01LG5`0O7al0>CTi
04A4A00c<c<0Bd];04Q8B00Q8B40GemO04m?C`1ZJVX0MgMg06MWI`20P800S8b<07QhN03Jf]X0`/;2
05aLG00a<C40?Cdm065QH@2@T900WYjN0:n_[`3>c/h0XJ6Q05MGE`35aLD0c<c<0;Fe]@2UYJD0^KVi
0=OGe`3MgMd0bLW90:BTY02VYZH0Ng]k061PH00S8b<0@T920:ZZZP2o_kl0SXj>08R8R00208:2PP2N
08V9R@1RHV80Ph>308^;R`2ZZZX0VYZJ06YZJP2;Rh/0XZ:R09^KV`1JFUX0Ph>30;2`/02l_;`0/k>c
08N7Q`1bLW80K6a/096AT@3?cll0DE5A02l_;`0b<S80:b/[03lo?`1ADE40Xj>S06][J`0o?cl0K6a/
0<o?c`2=SHd0NWYj065QH@17AdL0BDU903lo?`1RHV80KVi^08^;R`1QHF40Cdm?06QXJ01JFUX0<30`
02HV9P0o?cl09BDU02PX:00`<300>c/k03De=@0Y:BT0IfMW06YZJP1jNWX0YjNW09nOW`2GUiL0`<30
06UYJ@1MGEd0Xj>S09fMW@13@d<0;2`/06UYJ@1ADE40HV9R08B4Q01NGUh071`L01dM7@0T92@09BDU
030`<01NGUh0B4Q804I6AP1HF5P0T92@07AdM01[Jf/0ZJVY0:BTY03@d=00`/;20;Vi^@2:RXX0G5aL
04a<C01IFET0NGUi079bLP1RHV80@D5108:2PP3Cdm<0YZJV07=cL`12@T80>c/k03@d=01/K6`0UYJF
04m?C`0Z:RX0J6QX05iNGP1_Kfl0LG5a04A4A02HV9P0_;bl04Q8B00];Bd0B4Q808F5Q@22PX80QHF5
0:j^[P2a/K40Ogmo06][J`2ATI40TY:B096AT@2^[Zh0`<300:ZZZP2MWId0/k>c05iNGP2>SXh0Ogmo
06e]K@2GUiL0KFe]06ATI01iNGT0^k^k0<S8b02o_kl0W9bL08N7Q`24Q8@0SXj>09ZJVP2k^k/0bl_;
0=SHf039bLT0W9bL0;fm_@2[Zj/0W9bL07UiN@0S8b<201/K6`0@01lO7`0a<C40VYZJ06QXJ02<S8`0
KVi^02/[:`0X:2P0<C4a03Lg=`1=CDd0[:b/0:RXZ01EEED0L71`04=3@a3ooooo000>ooooo`0:05YJ
FP1DE5@0DU9B05EEE@0k>c/0EeMG04Y:BP0k>c/0EEEE01/K6`80>c/k0?l0EeMG01HF5P0^;Rh0CTi>
04Y:BP0e=CD04A4A04U9B@14A4@09RHV02l_;`0W9bL0820P02h^;P0];Bd0FEUI024Q8@0g=cL0Bd];
024Q8@1;Bd/0?3`l01LG5`12@T80>c/k01TI6@0[:b/0AdM703Hf=P0a<C40FUYJ0492@P0f=SH092@T
05mOG`21PH40IVIV03`l?01;Bd/0DU9B02XZ:P19BDT0N7Qh07=cL`0a<C40BTY:02LW9`17AdL0SXj>
09:BTP1HF5P0BTY:01`L701;Bd/0O7al038b<P1=CDd0?Shn08b<S01RHV80?Cdm06=SH`1LG5`04Q8B
03/k>`1bLW80E5AD0<?3``37alL0DE5A03@d=01CDe<0@41003`l?01DE5@0@T9202DU9@14A4@0OWin
05ADE02e]KD0bLW906m_K`1EEED0G5aL06QXJ01>CTh0GEeM07=cL`1ADE40LW9b071`L01RHV80Lg=c
07moO`1;Bd/0S8b<04m?C`2@T900BDU909NGU`1fMWH0AdM705iNGP1=CDd0Kfm_08f=S@1<C4`0NGUi
05mOG`1jNWX0ATI609^KV`1FEUH0IFEU08>3P`1TI6@0RHV905IFEP1KFe/0ZZZZ08f=S@18B4P0OGem
03De=@12@T80`l?307alO00o?cl0P82007YjNP1QHF40QHF506UYJ@1SHf<0YjNW05=CD`1^KVh0I6AT
075aL@2NWYh0I6AT04i>CP2;Rh/0RXZ:08^;R`36a/H0`l?308R8R00Y:BT0MgMg04m?C`0b<S80I6AT
07moO`1BDU80Lg=c05QHF019BDT0VIVI08N7Q`0k>c/0Jf][04e=C@1eMGD0EeMG03`l?026QXH0J6QX
03hn?P1cLg<0=3@d08:2PP17AdL0>CTi06UYJ@1[Jf/0B4Q808f=S@0Y:BT0B4Q806EUI@0];Bd0L71`
075aL@1XJ6P0JFUY069RHP1OGel0BTY:08>3P`1MGEd0Hf=S02h^;P12@T80GemO061PH01@D500CDe=
04];B`20P800LW9b03<c<`1`L700>SXj03lo?`0m?Cd0DE5A08j>SP1FEUH0H61P05iNGP13@d<0QXJ6
08Z:RP20P800IFEU06e]K@1GEeL0B4Q804e=C@1?Cdl0EUIF061PH01:BTX0IFEU07UiN@24Q8@08b<S
059BDP1nOWh0E5AD055AD@23Ph<0SXj>05UIF@27QhL0C4a<069RHP1LG5`0H61P0410@017AdL0F5QH
03De=@1gMgL0EEEE04I6AP1QHF40MgMg09jNWP1GEeL0H61P051@D02@T90000/0Ti>C04=3@`0c<c<0
=cLg07emO@1aLG40=cLg05mOG`0d=3@0ATI6051@D00>ooooo`0c07]kN`1cLg<0Lg=c07moO`2;Rh/0
UIFE0:j^[P2KVi/0RXZ:0:n_[`0Q8B40820P038b<P0W9bL04a<C02LW9`1cLg<0];Bd09BDU02NWYh0
XZ:R09:BTP24Q8@0]KFe09RHV02EUID0KFe]03Xj>P0R8R808b<S038b<P0Y:BT05aLG01TI6@0D51@0
5ADE01`L700K6a/07alO02/[:`1HF5P0Y:BT0;:b/P2[Zj/0ZJVY06YZJP2RXZ80OGem0492@P15ADD0
8R8R00@05ADE00<051@D01PH601;Bd/00P19BDT0101:BTX0Ph>307inOP1<C4`204=3@`0/08B4Q037
alL0RXZ:08V9R@20P800B4Q808Z:RP1kNg/0?Shn02DU9@0V9RH0;Bd]0410@01fMWH0W9bL0;no_`2i
^KT0d=3@0;Ng]`2UYJD0]kNg0;Fe]@3<c<`0e=CD0<S8b011@D40820P051@D00X:2P07alO06i^KP37
alL0]KFe07IfMP1TI6@0K6a/08J6QP2XZ:P0QhN70861P@2MWId0JFUY079bLP2f][H20:2PX01S09ZJ
VP1oOgl0U9BD0:6QX@30`<00fMWI0<[:bP32`/80TY:B08n?S`1lO7`0:b/[02`/;00];Bd0<S8b0451
@@2?Shl0IVIV05=CD`17AdL0W9bL0;Rh^028R8P0<S8b01lO7`0W9bL0@T9208Z:RP2l_;`0e=CD08F5
Q@1QHF40NWYj0410@00/;2`0:b/[059BDP0W9bL092@T02DU9@0^;Rh09RHV04i>CP1:BTX0ADE506=S
H`22PX80/K6a08Z:RP2e]KD0Jf][05UIF@1]KFd0/k>c07QhN01CDe<0BDU9055AD@2NWYh0Lg=c05AD
E00[:b/071`L02@T900M7Ad071`L02/[:`16ATH0>c/k04I6AP23Ph<0RHV909VIV@2XZ:P0QXJ60:^[
Z`3<c<`0d];B08^;R`1YJFT0J6QX05QHF01MGEd0L71`04i>CP1jNWX0^[Zj0:b/[02CTi<0S8b<09jN
WP2UYJD0Xj>S0<c<c03Fe]H0dm?C0;Rh^02<S8`0LG5a00<0DE5A01T0^KVi0:>SX`0/;2`09RHV02h^
;P11@D40GemO08V9R@1SHf<0ADE503Xj>P1BDU80KVi^07IfMP23Ph<0Z:RX09ZJVP2;Rh/0V9RH08J6
QP1VIVH0HF5Q06UYJ@22PX80Ogmo0080Rh^;02D0dm?C0:f][@26QXH0QHF508J6QP1gMgL0MGEe05mO
G`0k>c/0?clo05eMG@2j^[X0aLG508J6QP1BDU80JFUY06QXJ01mOGd0KVi^05]KF`0T92@06ATI06m_
K`1CDe<0IfMW0:FUY@1;Bd/0;2`/02h^;P0g=cL0=CDe07emO@2FUYH0/k>c07]kN`1IFET0>3Ph013o
oooo000>ooooo`3H06YZJP11@D40?Cdm061PH013@d<0GEeM02<S8`0[:b/0Bd];03hn?P0S8b<0;2`/
06UYJ@0e=CD092@T034a<@0h>3P0BTY:01/K6`15ADD09bLW01LG5`0`<300@41002HV9P0D51@04Q8B
05MGE`17AdL0H61P0451@@092@T0<C4a05UIF@0a<C40?Cdm02/[:`0e=CD0A4A406MWI`0U9BD05aLG
03`l?01HF5P07QhN00/;2`0Q8B40@D5106=SH`17AdL0Hf=S03lo?`0C4a<07QhN0451@@14A4@0=cLg
03dm?@0W9bL07AdM04Y:BP1dM7@0DE5A06][J`0R8R80:2PX06UYJ@0j>SX0Cdm?03`l?01`L700EEEE
0492@P1UIFD0Jf][04i>CP0k>c/0F5QH038b<P2?Shl0/;2`0861P@1BDU80BTY:03`l?010@400G5aL
05UIF@1FEUH0EEEE096AT@1DE5@0_[jn0<G5a@1mOGd0EUIF055AD@1fMWH0GUiN06MWI`1DE5@0CTi>
0820P01/K6`0JFUY0861P@1]KFd0De=C092@T01@D500KFe]03dm?@2ATI40JFUY04e=C@1QHF40>3Ph
04i>CP1jNWX0BDU909:BTP1DE5@0HF5Q03hn?P2HV9P0DE5A04A4A01WIfL0DU9B09nOW`1OGel0?clo
071`L01mOGd09RHV075aL@1UIFD0@4100<S8b029RHT0?clo065QH@1dM7@0Lg=c07inOP1?Cdl0>c/k
08:2PP1NGUh0HF5Q02h^;P0o?cl0MWIf06IVIP1;Bd/0Kfm_04Y:BP15ADD0OWin09bLW01eMGD0A4A4
0861P@23Ph<0ATI606EUI@2;Rh/0G5aL07AdM01_Kfl0@41009jNWP2LW9`0@41005EEE@0k>c/0>3Ph
06=SH`0c<c<0Ph>309NGU`1>CTh0N7Qh03Lg=`1aLG40N7Qh05IFEP1aLG40NGUi04M7A`27QhL0E5AD
04i>CP1gMgL0?Cdm06e]K@1YJFT0S8b<06][J`1]KFd0Fe]K05mOG`2BTY80Cdm?07alO00o?cl0=SHf
079bLP1]KFd0IVIV05iNGP1EEED0RHV907inOP10@400JFUY0P1NGUh0?00l?3`0De=C08>3P`1QHF40
IfMW06QXJ00m?Cd0Shn?08Z:RP1WIfL0LG5a07YjNP1TI6@0C4a<051@D01CDe<0MWIf069RHP12@T80
I6AT07MgM`1YJFT0>CTi07inOP1jNWX0E5AD04a<C01eMGD0Ph>305=CD`2@T900FEUI065QH@1HF5P0
Hf=S03hn?P1RHV80Ph>303dm?@1fMWH0DU9B02HV9P17AdL0K6a/08N7Q`1]KFd0J6QX04Y:BP1VIVH0
M7Ad04A4A01:BTX0C4a<0820P01PH600>3Ph06YZJP0c<c<0?clo06EUI@kooooo02l0JFUY05QHF01B
DU80EEEE055AD@1LG5`0NGUi05MGE`1PH600UiNG06MWI`0h>3P0<30`01HF5P0A4A40FUYJ065QH@2O
Wil0Jf][04a<C01dM7@0QXJ607EeM@2=SHd0XZ:R0:JVYP2@T900W9bL07=cL`0k>c/0;2`/02HV9P0O
7al06QXJ01DE5@0C4a<06QXJ03Xj>P1NGUh0Ng]k07YjNP1lO7`0P82007QhN02=SHd0TY:B07UiN@02
0:ZZZP0306ATI00E5AD04a<C00@05ADE0=h05QHF01lO7`1MGEd0/k>c0;Jf]P2e]KD0Rh^;04a<C017
AdL0ADE507MgM`1mOGd0C4a<08>3P`15ADD0=cLg02`/;00f=SH0J6QX07=cL`0b<S80:2PX061PH02S
Xj<0_kno09nOW`2MWId0e]KF0;:b/P2_[jl0LW9b069RHP1kNg/0[Jf]0:n_[`0h>3P071`L02@T901K
Fe/07QhN02DU9@2HV9P0Lg=c03Ti>@0j>SX0?Shn07alO02RXZ80TI6A08Z:RP1HF5P0Cdm?07UiN@3;
bl/0Xj>S07EeM@1lO7`0^[Zj0:n_[`35aLD0b/[:0=3@d02OWil0DU9B0451@@0o?cl0PH610410@00/
;2`0<30`02/[:`10@400Ti>C065QH@1fMWH0QHF506=SH`2e]KD0Xj>S04Q8B01`L700Kfm_06UYJ@2H
V9P0fm_K09RHV02KVi/0R8R808Z:RP0[:b/0:2PX0410@01?Cdl092@T02DU9@0^;Rh09BDU05=CD`1E
EED0=cLg02<S8`0n?Sh0DU9B09JFUP2_[jl0IFEU03Ti>@0e=CD0C4a<0:f][@2NWYh0Fe]K07=cL`2D
U9@0CDe=05=CD`1>CTh0ATI601dM7@0L71`06QXJ01TI6@0K6a/0EEEE061PH01ZJVX0[Jf]08j>SP2<
S8`0Ti>C0:ZZZP2f][H0d=3@08R8R01FEUH0D51@04i>CP1TI6@0H61P07inOP2EUID0YjNW08R8R027
QhL0[Jf]07]kN`1PH600JFUY0861P@2UYJD0N7Qh06m_K`2c/k<0fMWI0=CDe036a/H0^KVi0=3@d02m
_Kd0Ph>302`/;00S8b<0:2PX02h^;P0m?Cd0Jf][04A4A00n?Sh0CDe=0820P01MGEd0L71`06ATI01Z
JVX0R8R808f=S@2PX:00GUiN06MWI`1RHV80Bd];05IFEP1LG5`0M7Ad0;jn_P2[Zj/0MGEe06QXJ016
ATH0>CTi038b<P0c<c<0=CDe03<c<`1>CTh0QhN70;2`/02IVIT0QhN703De=@1<C4`0DU9B05QHF00i
>CT0AdM705mOG`22PX80QHF504a<C01gMgL0[:b/0410@00d=3@0;2`/02d];@0e=CD0][Jf08B4Q01l
O7`0De=C0;Fe]@2>SXh@ooooo`003_ooool0T01YJFT0LG5a04Q8B01[Jf/0Bd];07MgM`1GEeL092@T
03Xj>P13@d<09bLW02@T901CDe<0D51@02/[:`11@D40820P05]KF`0[:b/0GEeM034a<@0@41004Q8B
02XZ:P0X:2P0=cLg01/K6`1>CTh0Bd];06EUI@0k>c/04a<C02LW9`0Z:RX07QhN03De=@0[:b/0;Rh^
04E5A@1^KVh0@T9202HV9P0o?cl0DE5A059BDP0k>c/0?Shn059BDP1cLg<0Cdm?06m_K`0d=3@08R8R
02PX:01<C4`0AdM703dm?@1?Cdl0G5aL030`<01WIfL0RXZ:04A4A02ATI40@T9202XZ:P1_Kfl0@D51
05eMG@1<C4`0Ogmo05eMG@0j>SX0J6QX06UYJ@1CDe<0>SXj06UYJ@1BDU80QHF50;Ng]`20P800DU9B
05UIF@0j>SX0ADE507QhN02FUYH0F5QH04m?C`1hN7P0<30`0:2PX02WYjL0N7Qh05UIF@1:BTX0E5AD
05=CD`1]KFd0K6a/04E5A@1LG5`0De=C06IVIP2RXZ80L71`0451@@1kNg/0De=C08Z:RP0o?cl0R8R8
04A4A012@T80L71`051@D01JFUX0G5aL034a<@1dM7@0OWin04Q8B00^;Rh0QHF506e]K@19BDT0EeMG
03<c<`1eMGD0NGUi04i>CP20P800O7al02d];@1jNWX0E5AD03/k>`2c/k<0UIFE06m_K`1XJ6P0Ph>3
08:2PP80Kfm_01D0BDU907IfMP1MGEd0KVi^08J6QP1QHF40Q8B4079bLP1QHF40IfMW096AT@1gMgL0
_;bl0;no_`27QhL0@T9206i^KP26QXH0C4a<06QXJ029RHT00P1[Jf/0H@1iNGT0ATI609ZJVP2SXj<0
F5QH06=SH`1BDU80Cdm?07YjNP0i>CT0O7al0:6QX@1BDU80KFe]02l_;`1QHF40E5AD051@D01[Jf/0
KVi^04E5A@2DU9@0J6QX04Q8B01`L700?clo07MgM`2YZJT0Ti>C07UiN@22PX80IfMW05aLG02BTY80
OGem09jNWP1DE5@0>3Ph071`L01XJ6P0P82005mOG`1DE5@0XZ:R08n?S`1NGUh0QXJ607alO01NGUh0
>c/k04Y:BP23Ph<0DU9B06e]K@1eMGD0CTi>08^;R`1oOgl0Hf=S075aL@1iNGT0F5QH051@D016ATH0
=SHf05UIF@1DE5@0?clo071`L029RHT0K6a/030`<018B4P0LW9b05ADE01>CTh0LW9b061PH00b<S80
Kfm_04i>CP1QHF40IFEU04m?C`0S8b<0?3`l06=SH`0X:2P0JVYZ04Y:BP0d=3@0EUIF06ATI01nOWh0
TY:B06EUI@1?Cdl00P1[Jf/02P0S8b<0;2`/0492@P1eMGD0H61P03Ti>@1[Jf/0=SHf0492@P24Q8@>
ooooo`0e05]KF`1FEUH0DU9B04=3@`0m?Cd0QhN707AdM01PH600?Shn03`l?01oOgl0QHF503Lg=`0E
5AD0:RXZ07QhN011@D40I6AT03Lg=`0];Bd08B4Q03@d=01MGEd0V9RH09ZJVP2>SXh0[:b/06a/K01O
Gel0Kfm_05MGE`1EEED0D51@03Hf=P0_;bl0;2`/04A4A01/K6`0H61P04];B`1?Cdl0Fe]K06UYJ@2W
YjL0WinO0;jn_P1oOgl0OWin0=CDe01BDU804A4A01@D500C4a<00P0E5AD06@0F5QH05ADE01DE5@0O
7al0Shn?05=CD`1LG5`0V9RH092@T01DE5@0BTY:05UIF@1LG5`0UIFE04I6AP0V9RH0;bl_03Ti>@0m
?Cd0>3Ph03lo?`1oOgl0/K6a0<o?c`2TY:@00P2:RXX0^P2<S8`0f=SH0<;2`P27QhL0DE5A051@D01F
EUH0N7Qh08J6QP1iNGT08R8R01`L701EEED0:BTY01`L701KFe/0KFe]05UIF@0c<c<0;bl_05mOG`2V
YZH0U9BD08N7Q`1NGUh0KFe]09fMW@2LW9`0FEUI0492@P0e=CD0>c/k08>3P`2NWYh0[Zj^0:ZZZP1b
LW80DU9B03Lg=`0b<S80HF5Q0:f][@1WIfL0<S8b03Lg=`1LG5`0^;Rh08^;R`1oOgl0TY:B07IfMP2U
YJD0alO709^KV`1WIfL0EeMG09jNWP2>SXh0S8b<09ZJVP2FUYH0UIFE06=SH`0l?3`0;bl_04m?C`19
BDT092@T01/K6`0O7al0EUIF05]KF`18B4P08B4Q01hN7P0`<300AdM707emO@1fMWH0>c/k01dM7@0N
7Qh0<C4a05=CD`2KVi/0DU9B04i>CP1UIFD0DU9B04M7A`1PH600EEEE02l_;`0N7Qh0;Bd]0451@@19
BDT0ATI606QXJ01UIFD0UIFE0=CDe03AdM40`/;20<O7a`30`<00[jn_061PH00k>c/0;Rh^02d];@0d
=3@0CTi>08F5Q@22PX80Kfm_05iNGP1WIfL0S8b<07moO`1ADE40GUiN05aLG023Ph<0MgMg06][J`3:
b/X0fMWI0<C4a02i^KT0aLG50>3Ph02n_[h0/;2`06YZJP11@D40=3@d030`<010@400NWYj079bLP1W
IfL0RXZ:08j>SP2RXZ80OGem05MGE`1]KFd0SHf=0::RXP1hN7P0FUYJ061PH00n?Sh0>c/k0451@@1B
DU80K6a/0;jn_P2EUID0;Rh^02HV9P0U9BD0:RXZ02l_;`0e=CD0<c<c04m?C`1TI6@0MWIf0;6a/@2L
W9`0CDe=04];B`1MGEd0CTi>03Ti>@0f=SH0=cLg04M7A`1hN7P0TI6A08:2PP2CTi<0ZZZZ03Ti>@80
:RXZ00P0<S8b07YjNP2][Jd0Ti>C05=CD`1mOGd0XJ6Q08b<S13ooooo000>ooooo`0:071`L02ATI40
CDe=07YjNP1;Bd/0OGem07YjNP0e=CD0B4Q805EEE@80ATI60?P0P820069RHP0[:b/0DU9B03Hf=P1g
MgL0=3@d059BDP15ADD0=SHf02XZ:P0c<c<0;bl_03hn?P0U9BD0IfMW05iNGP1WIfL0G5aL02TY:@0o
?cl0@d=302d];@0g=cL0>c/k04E5A@0b<S80RHV9069RHP0Y:BT0>SXj059BDP1eMGD0GEeM0492@P1J
FUX0KFe]04I6AP23Ph<0HV9R04I6AP0_;bl0EeMG04Y:BP0m?Cd0De=C05iNGP0`<300KFe]07]kN`0Y
:BT0S8b<04e=C@0S8b<0GUiN038b<P1FEUH0CDe=08:2PP19BDT0<C4a06EUI@1dM7@0<c<c04Q8B026
QXH0L71`05aLG03:b/X0OWin05QHF02KVi/0EUIF03Ti>@1UIFD0HF5Q055AD@1[Jf/0XZ:R04e=C@30
`<00Z:RX07emO@1DE5@0IVIV04i>CP0[:b/0BTY:0::RXP1IFET0EUIF034a<@0l?3`0IVIV08B4Q01>
CTh0DE5A03<c<`2;Rh/0@D5108F5Q@1KFe/0?Cdm05QHF01TI6@0FEUI07MgM`1BDU80MWIf09NGU`1S
Hf<0AdM706e]K@1`L700C4a<06IVIP1iNGT0LG5a05YJFP1:BTX0QhN706m_K`0];Bd0Ogmo0492@P0b
<S80UIFE096AT@1_Kfl0K6a/08B4Q02>SXh0WIfM08f=S@1:BTX0QhN7069RHP1[Jf/0SHf=05eMG@1e
MGD0QHF507MgM`1`L700RXZ:06e]K@2_[jl0`/;208>3P`1;Bd/0OGem08n?S`1@D500Hf=S08:2PP1J
FUX0V9RH09ZJVP16ATH0RXZ:09fMW@1FEUH0L71`05YJFP1:BTX0M7Ad03Xj>P1iNGT0UIFE04e=C@1S
Hf<0?3`l07UiN@1^KVh0DE5A06][J`1NGUh0AdM708F5Q@1NGUh0@41006m_K`0Z:RX0ADE505=CD`1h
N7P0M7Ad0:FUY@1bLW80?Cdm05=CD`12@T80NWYj03hn?P0b<S80Fe]K0410@01FEUH0Cdm?065QH@2V
YZH0M7Ad02/[:`1HF5P0Bd];0451@@0e=CD0CTi>07UiN@1FEUH0F5QH07QhN015ADD0SHf=09JFUP1n
OWh0I6AT05eMG@1?Cdl0E5AD06=SH`1FEUH0EeMG06][J`11@D40Fe]K06EUI@1eMGD0>3Ph051@D01A
DE40@D5106MWI`25QHD0MGEe04Q8B01KFe/0;Bd]05EEE@1UIFD0Fe]K04=3@`10@400KFe]01dM7@1M
GEd0JVYZ04I6AP80Cdm?0100NWYj07MgM`1WIfL0E5AD08:2PP1jNWX0>CTi02`/;00j>SX0O7al07Qh
N00c<c<0HV9R02l_;`0c<c<0MWIf3_ooool04`1<C4`0AdM704m?C`1WIfL0F5QH07YjNP1FEUH0Jf][
04i>CP19BDT0<c<c04Y:BP1jNWX0Ph>30::RXP1GEeL0GUiN05=CD`0S8b<00P0[:b/201dM7@0M06][
J`2][Jd0[jn_09VIV@2ATI40M7Ad06][J`1nOWh0MGEe06QXJ01UIFD0GEeM05ADE01TI6@0GemO05YJ
FP1HF5P0FEUI059BDP1;Bd/0U9BD09ZJVP38b<P0U9BD06=SH`2QXJ40Bd];01@D500C4a<00P0A4A42
01DE5@<05aLG00/0Kfm_03Ti>@0a<C40OWin0:^[Z`2][Jd0Ti>C08:2PP2@T900E5AD020P800201dM
7@0a01lO7`0R8R809RHV03<c<`1/K6`0h>3P0:>SX`2IVIT0]KFe09BDU020P800N7Qh04a<C01RHV80
LW9b04Q8B01GEeL0G5aL08V9R@1cLg<0@41001dM7@0U9BD0>c/k01`L700_;bl0KFe]055AD@1FEUH0
AdM706UYJ@1gMgL0DU9B05YJFP2HV9P0Rh^;04Q8B01@D500;Rh^02d];@0c<c<0:b/[0492@P20P800
W9bL0:j^[P1VIVH0Lg=c05YJFP0203Xj>P0Y0::RXP2b/[80Q8B407IfMP2f][H0g=cL0<71`@2_[jl0
TI6A06ATI023Ph<0P82004e=C@14A4@0VYZJ09FEU@1@D500AdM707emO@1lO7`0RHV906IVIP1EEED0
MWIf07]kN`1OGel07alO01`L7018B4P0E5AD04I6AP0T92@07AdM01hN7P0i>CT0CDe=08B4Q01ZJVX0
?3`l01dM7@0Q8B400P0W9bL0300l?3`0SXj>05EEE@1NGUh0K6a/038b<P1HF5P0E5AD09:BTP1IFET0
GemO061PH080G5aL0100EEEE06UYJ@2@T900f=SH0=CDe03<c<`0bl_;09fMW@2EUID0GEeM03Hf=P0W
9bL09RHV03@d=01aLG40XZ:R0P1:BTX0A@0b<S80De=C07inOP1lO7`0GemO06MWI`29RHT0YZJV08j>
SP2m_Kd0f=SH0<O7a`2o_kl0Y:BT0;>c/`36a/H0b<S80<o?c`2l_;`0M7Ad059BDP1@D500GUiN07in
OP1^KVh0I6AT071`L01]KFd0QXJ608R8R01bLW80G5aL09BDU02>SXh0AdM703/k>`1RHV80@4100492
@P16ATH0De=C08^;R`37alL0SXj>02`/;00c<c<0;Bd]02l_;`0f=SH0CDe=0492@P1GEeL0MWIf0:>S
X`2CTi<0CDe=03Lg=`1EEED0E5AD03dm?@14A4@0@d=303Lg=`0^;Rh0=cLg05eMG@1/K6`0J6QX07Ui
N@0202PX:00902HV9P0j>SX0/;2`08b<S02i^KT0RXZ:06ATI013@d<0ATI6013ooooo000>ooooo`1Y
06ATI01^KVh0CDe=07emO@16ATH0L71`05]KF`0/;2`0AdM703Ti>@0f=SH0<C4a075aL@1;Bd/092@T
055AD@0V9RH0IfMW01dM7@12@T80?3`l02<S8`0[:b/0;2`/02@T900i>CT08B4Q05IFEP1DE5@0DE5A
071`L00T92@0=CDe04=3@`0Y:BT0<S8b02h^;P0h>3P07alO075aL@1NGUh07alO03<c<`0k>c/0FEUI
03dm?@13@d<0FEUI06a/K01:BTX0O7al055AD@12@T80:b/[05YJFP1LG5`0BDU904Y:BP18B4P0:BTY
06EUI@1jNWX0>3Ph07moO`14A4@09RHV075aL@1TI6@0>CTi04e=C@1]KFd0NGUi03Ti>@1EEED0PH61
04m?C`0e=CD0Lg=c06a/K017AdL0XJ6Q06e]K@1:BTX0DU9B05=CD`17AdL0MWIf05mOG`12@T80AdM7
09NGU`1JFUX0aLG50;:b/P2MWId0H61P06ATI01EEED0IVIV05eMG@1eMGD0De=C065QH@1mOGd0BDU9
0080LG5a07d0ATI606a/K01GEeL0NGUi04A4A01IFET0Ng]k04Q8B01ADE40HV9R05YJFP1WIfL0AdM7
079bLP24Q8@0HF5Q0410@01fMWH0K6a/04=3@`1^KVh0Ogmo06][J`1KFe/0B4Q807inOP1^KVh0@T92
079bLP1;Bd/0;bl_08F5Q@2ATI40KVi^06QXJ01dM7@0MGEe09FEU@2>SXh0@T9205aLG01QHF40NGUi
07QhN01ADE40Ng]k06e]K@1FEUH0NGUi08^;R`1UIFD0W9bL0<W9b@20P800@d=307AdM02<S8`0Bd];
06EUI@2?Shl0J6QX06m_K`1mOGd0?Cdm07EeM@2YZJT0GemO05]KF`1;Bd/0=SHf06QXJ016ATH0NWYj
08b<S013@d<0I6AT03Ph>01ADE40QhN705YJFP1XJ6P0MgMg03/k>`1iNGT0Jf][03Xj>P20P800EeMG
051@D01LG5`0Vi^K05EEE@23Ph<0LG5a05YJFP1MGEd0DU9B08n?S`1=CDd0=cLg07=cL`1UIFD0Ng]k
04e=C@1HF5P0Y:BT08B4Q013@d<0MWIf069RHP1<C4`0@d=304Y:BP1eMGD0HV9R07EeM@21PH40>3Ph
05MGE`27QhL0JVYZ069RHP24Q8@0DE5A04Q8B00204=3@`0/069RHP1eMGD0ATI6051@D01eMGD0Jf][
03Xj>P1MGEd0OWin05iNGP14A4@0HF5Q06EUI@0o?cl0BTY:06MWI`1aLG40R8R806i^KP0`<300@D51
07inOP1<C4`0CTi>061PH00g=cL0ATI606QXJ021PH40UIFE06=SH`1KFe/0UiNG09BDU01=CDd0?3`l
0410@01^KVh0De=C02LW9`1bLW80DU9B030`<01mOGd>ooooo`1506ATI01BDU80BDU904a<C02GUiL0
SXj>05]KF`0k>c/0;2`/02@T900_;bl092@T02<S8`1OGel0]KFe069RHP24Q8@0>SXj01hN7P0Q8B40
>SXj03lo?`18B4P0IfMW05EEE@1bLW80S8b<059BDP10@400De=C05YJFP1NGUh0L71`06e]K@26QXH0
UYJF06EUI@1RHV80Fe]K05QHF01KFe/0FEUI06a/K02LW9`0Lg=c0<O7a`22PX80De=C04Y:BP1?Cdl0
8R8R024Q8@0I6AT05aLG01`L700R8R808B4Q01/K6`0H61P09RHV061PH01GEeL0D51@069RHP2>SXh0
b/[:09FEU@1EEED0;Bd]00D07AdM05007QhN020P801`L700Jf][07EeM@2_[jl0ZZZZ079bLP1KFe/0
8R8R02`/;00_;bl0=CDe05IFEP1LG5`0D51@069RHP1mOGd0LW9b0410@00J6QX0@41001/K6`0Q8B40
P820051@D01<C4`0Hf=S0:FUY@1eMGD0F5QH04Y:BP1DE5@0/[:b05iNGP0Z:RX09RHV02@T900O7al0
820P024Q8@1@D500N7Qh0;Rh^02TY:@0Xj>S0:f][@2KVi/0M7Ad069RHP34a<@0^[Zj0<30`02GUiL0
QHF50:^[Z`2g]kL0`<300<71`@2c/k<0ZZZZ0:BTY035aLD0IVIV05ADE015ADD0@D5105eMG@2ZZZX0
VIVI0;2`/02f][H0/[:b09^KV`2DU9@0RXZ:0:b/[01hN7P0=3@d01lO7`807AdM00h07QhN03De=@1D
E5@0QXJ6061PH01cLg<0?3`l03Hf=P0d=3@0=CDe03`l?01XJ6P0OWin06=SH`80F5QH00H0Ng]k09>C
T`2WYjL0DE5A04M7A`0c<c<202DU9@0[02@T900W9bL0F5QH0=CDe031`L40];Bd0;Rh^01dM7@0ZZZZ
0;Fe]@2CTi<0M7Ad07=cL`1mOGd0VYZJ0:>SX`0W9bL0GemO03Ph>018B4P0Ph>305EEE@0X:2P0<S8b
06QXJ01/K6`0Shn?09fMW@2e]KD0/;2`0:FUY@1bLW80Jf][08b<S03Cdm<0`<3005]KF`1:BTX0AdM7
051@D01ADE40OGem06i^KP0204E5A@0^03hn?P1DE5@0Kfm_09:BTP2IVIT0QhN703<c<`0e=CD0A4A4
071`L01BDU80ATI604Q8B01=CDd0PH610<[:bP2_[jl0MWIf065QH@1KFe/0IFEU06MWI`1fMWH0QhN7
0;>c/`2e]KD0[Zj^04e=C@13@d<0<S8b02<S8`0l?3`0<30`03hn?P1BDU80/[:b05MGE`0a<C40G5aL
08Z:RP1^KVh0WinO05iNGP1IFET0FUYJ0:>SX`80Zj^[00D0K6a/03Xj>P0m?Cd0?clo04m?C`0@oooo
o`003_ooool0`P1MGEd0QHF504a<C01oOgl0ATI606m_K`29RHT0<c<c03`l?01CDe<0:RXZ038b<P1E
EED0EeMG02LW9`1CDe<092@T05EEE@0c<c<0=cLg04Y:BP0[:b/08B4Q02PX:00O7al0@d=303De=@17
AdL0BDU904a<C01]KFd07alO02/[:`0Z:RX06a/K01lO7`0Z:RX0=cLg028R8P1KFe/0LG5a02XZ:P0b
<S80?Cdm06MWI`1eMGD0Cdm?05EEE@1kNg/0Hf=S03Ti>@1RHV80D51@02DU9@1?Cdl0ATI604e=C@1;
Bd/0I6AT03@d=01GEeL0Lg=c03lo?`19BDT0EeMG020P801`L700MWIf02l_;`1RHV80GEeM07emO@0g
=cL0EeMG0820P01KFe/0=cLg06YZJP1ADE40<c<c07=cL`1jNWX0EeMG04];B`1DE5@0?Shn08:2PP2P
X:00EUIF05=CD`2;Rh/0D51@0:ZZZP2m_Kd0Y:BT061PH01ADE40FEUI06MWI`1DE5@0LG5a04Y:BP1K
Fe/0KFe]0410@01XJ6P0KFe]04E5A@1bLW80EUIF06UYJ@0o?cl0Jf][08B4Q01;Bd/0DE5A04i>CP14
A4@0G5aL04I6AP1CDe<0S8b<06QXJ01@D500F5QH08F5Q@0n?Sh0FUYJ07inOP1/K6`0Lg=c03/k>`1B
DU80JVYZ04e=C@1HF5P0OGem03hn?P1bLW80VYZJ08>3P`1mOGd0Ph>306=SH`2UYJD0RHV903Lg=`1B
DU80IVIV075aL@2:RXX0F5QH08V9R@24Q8@0SXj>07alO01hN7P0Fe]K08:2PP39bLT0SXj>05IFEP1Y
JFT0MGEe03Ph>01TI6@0WinO08B4Q025QHD0PH6103<c<`1FEUH0X:2P06m_K`1<C4`0De=C04Q8B026
QXH0EeMG06MWI`27QhL0>c/k07MgM`10@400BDU907]kN`0e=CD0Fe]K07MgM`18B4P0WYjN09BDU012
@T80HF5Q0P14A4@0DP1YJFT0TI6A05QHF01fMWH0Kfm_05IFEP0];Bd0=3@d06i^KP1QHF40;2`/06MW
I`1:BTX0DU9B04m?C`12@T80UiNG07=cL`0e=CD0CTi>065QH@1GEeL0Bd];03dm?@1ZJVX0De=C04E5
A@1^KVh0BTY:05=CD`2/[:`0JVYZ04i>CP1`L700FEUI07IfMP1IFET0@d=304i>CP1WIfL0ADE506AT
I02QXJ40PX:205=CD`11@D40Lg=c04];B`1TI6@0OGem09VIV@1DE5@0De=C05YJFP1>CTh0FUYJ06IV
IP1;Bd/0=CDe07]kN`0`<300CDe=07YjNP1<C4`0BDU905UIF@1GEeL0OWin075aL@1QHF40Ogmo08n?
S`0i>CT0A4A407alO02;Rh/0JVYZ01hN7P1hN7P0BDU902d];@27QhL>ooooo`0?04A4A010@400>3Ph
030`<01=CDd0GUiN05QHF00];Bd0:RXZ028R8P0];Bd0<C4a02PX:01TI6@0XZ:R0080/[:b02L0RHV9
02l_;`0P82009BDU02XZ:P0Y:BT0MgMg04M7A`1nOWh0^KVi09>CT`1jNWX0<c<c02<S8`0h>3P0OGem
075aL@1>CTh0X:2P07UiN@1hN7P0LG5a04A4A00^;Rh09BDU038b<P22PX80b/[:0;2`/02RXZ80][Jf
092@T01RHV80<C4a01dM7@0W9bL071`L01XJ6P0K6a/00P0L71`0800I6AT06QXJ04=3@`1XJ6P0ZJVY
096AT@2GUiL0c/k>0;Zj^P1LG5`0;2`/01`L700M7Ad071`L01dM7@0L71`06a/K01dM7@1/K6`0A4A4
05UIF@2=SHd0QXJ609jNWP0^;Rh071`L01dM7@0X:2P09BDU02XZ:P0l?3`0FUYJ0P1<C4`0:@1MGEd0
MWIf05EEE@20P800Kfm_05ADE01UIFD0O7al08j>SP2PX:00JFUY069RHP1hN7P0TI6A0:RXZ02j^[X0
I6AT034a<@0Q8B406QXJ01`L700N7Qh09BDU06IVIP2LW9`0VYZJ07MgM`2^[Zh0ZJVY0:ZZZP2c/k<0
_;bl0=CDe03GemL0_Kfm0820P01gMgL0Rh^;0;^k^`39bLT0][Jf0080Zj^[01<0SXj>096AT@1nOWh0
<S8b034a<@1FEUH0Cdm?07emO@1DE5@0Bd];07alO017AdL0@d=30410@01/K6`0d];B07UiN@0J6QX0
6a/K00<07AdM01H0:2PX06UYJ@1[Jf/0D51@06MWI`0e=CD0;2`/02DU9@0R8R8092@T024Q8@1gMgL0
LG5a08b<S01PH600WIfM071`L01cLg<0@D5102DU9@0Q8B407AdM0`0N7Qh01013@d<0_[jn06a/K01d
M7@2061PH0030;2`/026QXH0=CDe00<09RHV05@0>c/k09BDU00`<300CTi>04M7A`0o?cl0GemO04];
B`0O7al0IFEU0492@P0m?Cd0V9RH0:JVYP1^KVh0J6QX04A4A019BDT0M7Ad0;6a/@3Shn<0XJ6Q03Xj
>P0m?Cd0?3`l04=3@`1QHF40QhN703hn?P0U9BD0<c<c03Ti>@1OGel0EUIF08n?S`32`/80PX:202l_
;`0`<300;2`/08R8R02[Zj/0VYZJ096AT@25QHD0Z:RX0<o?c`2m_Kd0Fe]K04U9B@1ADE40GUiN04a<
C01EEED0HF5Q0:6QX@36a/H0[:b/03De=@0];Bd09RHV01`L700O7al0:2PX04U9B@1:BTX0_Kfm05EE
E@1@D500Ng]k09>CT`2GUiL0@d=30410@01[Jf/0_kno0<?3``2h^;P0CTi>03@d=00k>c/0@T9206][
J`2/[:`@ooooo`003_ooool04P1FEUH0UIFE04a<C028R8P0BDU9079bLP2FUYH0?3`l03dm?@1BDU80
:b/[04M7A`1RHV80KVi^038b<P1BDU80<30`05YJFP80=3@d0;`0?3`l034a<@0a<C40;2`/020P800e
=CD0<30`04];B`1@D500ATI606m_K`0Y:BT0:b/[02l_;`0[:b/08R8R03<c<`1=CDd0:BTY05ADE01i
NGT0<S8b02/[:`10@400E5AD06EUI@15ADD0DE5A079bLP1JFUX092@T07MgM`17AdL08R8R04E5A@1=
CDd0BDU905MGE`1UIFD0<S8b05]KF`1iNGT0@T9203Ph>01OGel07QhN071`L021PH40<30`061PH013
@d<0Ph>304I6AP1VIVH0WIfM06YZJP0l?3`0HF5Q04e=C@0a<C40E5AD08R8R01FEUH0B4Q80410@00c
<c<0GemO07EeM@1GEeL0DU9B07MgM`0h>3P0Ph>309^KV`2MWId0F5QH06IVIP1JFUX0Jf][04];B`1g
MgL0EUIF05]KF`25QHD0=SHf061PH01lO7`0C4a<06YZJP1JFUX0JFUY05eMG@13@d<0SXj>04];B`1:
BTX0PX:2051@D01OGel0>c/k05QHF02HV9P0GUiN03Ph>01IFET0UIFE04A4A01EEED0PH6106e]K@1f
MWH0D51@05EEE@2<S8`0FEUI06ATI01gMgL0?clo06e]K@28R8P0?Shn05=CD`1WIfL0Fe]K0<W9b@2T
Y:@0@41005aLG01/K6`0OGem069RHP16ATH0RHV908N7Q`28R8P0QHF505eMG@0l?3`0EEEE0;Vi^@2>
SXh0De=C07moO`2:RXX0EEEE06][J`2WYjL0J6QX07QhN026QXH0FEUI09VIV@3;bl/0LG5a06EUI@1P
H600:2PX071`L01ADE40J6QX092@T019BDT0E5AD034a<@1JFUX0RXZ:05UIF@1KFe/0JFUY02HV9P1P
H600MGEe03Ti>@1]KFd0D51@03/k>`1:BTX0VYZJ04E5A@24Q8@0Ph>305aLG00j>SX0D51@07]kN`1>
CTh0CDe=07IfMP80FUYJ04@0E5AD04=3@`2OWil0PH6104a<C01VIVH0H61P03hn?P1<C4`0BDU907Qh
N01YJFT0F5QH07AdM00k>c/0GemO09NGU`1nOWh0J6QX06ATI00m?Cd0BTY:05mOG`1SHf<0De=C06=S
H`0/;2`0>SXj071`L023Ph<0Hf=S0492@P1kNg/0H61P0451@@1@D500PX:205UIF@1HF5P0P82005=C
D`1XJ6P0K6a/04Q8B00l?3`0Jf][05MGE`0j>SX0Lg=c069RHP15ADD0Jf][06i^KP2EUID0NGUi06a/
K01mOGd0QhN705mOG`0W9bL0=3@d06MWI`1QHF408b<S07inOP24Q8@0=cLg07QhN0kooooo03P0=3@d
02d];@0Y:BT092@T020P8019BDT0Lg=c05IFEP15ADD0@41004e=C@1;Bd/0;bl_07moO`2RXZ80YZJV
05iNGP1PH600UIFE059BDP0`<300=SHf03Ti>@1DE5@0JVYZ09FEU@24Q8@0W9bL08:2PP1=CDd0GEeM
08^;R`1nOWh0[Zj^09FEU@30`<00WIfM07]kN`1bLW80MGEe06IVIP1QHF40Ti>C0:f][@2KVi/0SHf=
05aLG029RHT0VIVI09^KV`2EUID0AdM702l_;`0W9bL071`L01lO7`<071`L00`06QXJ04i>CP26QXH0
[Jf]08>3P`2OWil0]kNg09jNWP1^KVh0G5aL03dm?@0O7al201dM7@0:01XJ6P0K6a/0<S8b05mOG`1Q
HF40[Zj^0:VYZ@2WYjL0Jf][02HV9P807AdM0P0[:b/0@P0T92@0;2`/071`L01[Jf/0K6a/07inOP2W
YjL0OWin04i>CP2TY:@0/k>c05EEE@1^KVh0UIFE08j>SP1/K6`0?Cdm03lo?`1:BTX0SHf=0:n_[`29
RHT0Lg=c0410@00Z:RX0820P02HV9P0U9BD0SXj>08J6QP1LG5`0DE5A09nOW`3@d=00_;bl0:RXZ02l
_;`0g=cL0<71`@2RXZ80WIfM0:RXZ02MWId0AdM70::RXP2j^[X0dm?C0:b/[01hN7P0Ph>30;bl_02L
W9`0EEEE02PX:00L71`0GEeM04i>CP11@D40D51@05YJFP0T92@09BDU02PX:02=SHd0VYZJ020P8080
6QXJ0P0L71`01`0c<c<0MgMg08J6QP1DE5@0F5QH02@T900Q8B400P0M7Ad03P0O7al0Bd];0:f][@2i
^KT0[jn_05YJFP1XJ6P0Cdm?0861P@1FEUH0820P01`L700M7Ad071`L0P0M7Ad0H01DE5@0_kno05EE
E@1RHV80;2`/02LW9`1SHf<0O7al02@T900N7Qh08B4Q028R8P0/;2`0QhN705aLG014A4@0G5aL02d]
;@0l?3`0CDe=04Y:BP1QHF40<C4a08:2PP2EUID0`l?308N7Q`2FUYH0@D5102DU9@0_;bl0A4A40;Rh
^02SXj<0<c<c02l_;`0h>3P0@41005MGE`2=SHd0B4Q802@T900d=3@0@41005MGE`1BDU80WIfM06i^
KP1ZJVX0LG5a03De=@0[:b/0RHV90<g=c@3GemL0V9RH0:VYZ@3:b/X0c/k>0;^k^`2YZJT0Ogmo09:B
TP2EUID0WIfM0:^[Z`2m_Kd0fm_K08Z:RP1kNg/0C4a<02XZ:P0V9RH0820P024Q8@0i>CT0LG5a0:>S
X`30`<00G5aL08:2PP2n_[h0[Zj^07QhN01cLg<09bLW04=3@`33`l<0[:b/0;Rh^01JFUX0ATI603dm
?@1aLG40[jn_07moOa3ooooo000>ooooo`1F065QH@2RXZ80Bd];07IfMP16ATH0KFe]07alO00j>SX0
>CTi0451@@0n?Sh0Cdm?05MGE`1bLW80>CTi04M7A`0_;bl0GemO04M7A`0/;2`0B4Q802l_;`0];Bd0
=CDe020P800W9bL09BDU04I6AP1GEeL0AdM708N7Q`0T92@08b<S02l_;`0V9RH0:BTY03Ti>@1BDU80
;2`/05MGE`2ZZZX0B4Q802@T900n?Sh0F5QH06MWI`16ATH0Fe]K06][J`18B4P0?3`l07=cL`1WIfL0
=3@d03hn?P1LG5`0=CDe06IVIP1ZJVX0<S8b065QH@28R8P0>3Ph04A4A01NGUh07QhN065QH@1OGel0
=3@d06QXJ00g=cL0MgMg030`<012@T80RXZ:06i^KP0i>CT0G5aL08B4Q01IFET0@d=308>3P`1JFUX0
ADE505ADE015ADD206MWI`0J05QHF01BDU80MgMg06ATI02GUiL0/;2`0:6QX@1ADE40Ng]k061PH01V
IVH0DU9B06m_K`1CDe<0EEEE08b<S00j>SX0F5QH08b<S01HF5P0IfMW05QHF01]KFd0IFEU0451@@2@
T902055AD@1b07IfMP1:BTX0EeMG0492@P1HF5P0Rh^;05iNGP1?Cdl0?3`l06][J`12@T80HF5Q05mO
G`1MGEd0Lg=c04Q8B017AdL0Ti>C02/[:`18B4P0Lg=c04A4A01_Kfl0W9bL0820P01[Jf/0H61P034a
<@1_Kfl0Ogmo061PH01ZJVX0Hf=S079bLP1aLG40De=C071`L01]KFd0JFUY08j>SP20P800HV9R071`
L03:b/X0X:2P03<c<`1@D500JVYZ02HV9P1@D500QXJ608R8R01]KFd0K6a/03<c<`1ADE40OGem06m_
K`17AdL0HV9R03dm?@26QXH0?clo079bLP2@T900Fe]K05IFEP0b<S80>3Ph08^;R`1NGUh0G5aL07Yj
NP0b<S80GemO07QhN013@d<0Ti>C06UYJ@12@T80DU9B0:ZZZP17AdL0HV9R065QH@1GEeL0=cLg069R
HP21PH40E5AD034a<@1BDU80EEEE06a/K00b<S80<30`06QXJ01gMgL0ADE505]KF`1UIFD07QhN03Ti
>@1QHF40L71`06UYJ@1<C4`0MgMg05IFEP0k>c/0ZZZZ09jNWP1SHf<0TI6A0P1JFUX0;P1BDU80I6AT
05EEE@1LG5`0>CTi04Y:BP1_Kfl0QhN706m_K`10@400QXJ607inOP1=CDd0D51@09FEU@1QHF40I6AT
08N7Q`1FEUH0M7Ad069RHP0l?3`0>SXj0861P@15ADD0?Shn07=cL`1IFET0>SXj06ATI01eMGD0W9bL
079bLP1[Jf/0_Kfm0:f][@1[Jf/0;Rh^04I6AP1]KFd0Fe]K02l_;`2HV9P0MGEe030`<01gMgL>oooo
o`0704=3@`0b<S80:b/[02LW9`0N7Qh0;bl_04a<C00208N7Q`1007emO@1eMGD0G5aL05aLG02^[Zh0
/k>c07moO`1NGUh0>c/k055AD@1hN7P0M7Ad030`<01PH600IfMW0:^[Z`0a<C407AdM020P800/;2`0
9bLW04U9B@1nOWh0Shn?092@T02CTi<0a/K606e]K@0n?Sh0=CDe038b<P0k>c/0?Shn06MWI`1jNWX0
ATI60;Fe]@2BTY80GemO06QXJ02FUYH0NWYj06UYJ@1WIfL0HV9R05=CD`0l?3`0:b/[01hN7P0L71`0
?3`l08f=S@2LW9`0MWIf0492@P1DE5@0QHF507=cL`1>CTh0DU9B04];B`1JFUX0C4a<034a<@807QhN
04l0Jf][05YJFP28R8P0[Zj^0<g=c@2OWil0FUYJ02HV9P0L71`07AdM01`L700U9BD092@T034a<@2D
U9@0cLg=0:BTY02NWYh0Vi^K07QhN018B4P0PH61092@T01BDU80EUIF061PH01RHV80QHF507UiN@1;
Bd/0A4A408F5Q@3=cLd0GemO065QH@26QXH0M7Ad0410@00[:b/0CDe=09VIV@1:BTX0LG5a07emO@2W
YjL0`<3009>CT`1OGel0W9bL0=;BdP2^[Zh0IVIV04e=C@1YJFT0Lg=c06QXJ028R8P0SXj>0:j^[P1/
K6`0AdM705eMG@21PH40bLW902d];@0I6AT06a/K051@D01LG5`0?clo03Ph>01PH600BDU902DU9@0R
8R80?Cdm0:^[Z`0o?cl0820P008071`L01D0<S8b05QHF02YZJT0Y:BT061PH01ZJVX0820P01dM7@0O
7al0>SXj07YjNP1lO7`0GUiN09FEU@2RXZ80Ph>308j>SP1eMGD0]KFe05QHF00Q8B400`0L71`0HP0M
7Ad071`L06MWI`2e]KD0KFe]03Lg=`0V9RH08B4Q05IFEP2EUID0?Cdm01`L700P82007QhN02LW9`1V
IVH0ZZZZ09JFUP2`/;00>SXj03Ph>01jNWX0TI6A05eMG@26QXH0TY:B08V9R@2=SHd0Zj^[07QhN00`
<30092@T02LW9`0o?cl0[Zj^0<71`@1[Jf/0=3@d03De=@0k>c/0@D51069RHP1PH600:b/[03/k>`1L
G5`0BTY:05EEE@27QhL0?clo04];B`1SHf<0MWIf05mOG`2XZ:P0QHF50::RXP2^[Zh0]kNg0:ZZZP2i
^KT0YjNW0:BTY02XZ:P0Ph>308b<S021PH40O7al09NGU`30`<00`/;208N7Q`1ADE40?3`l02@T900X
:2P0@D51065QH@1[Jf/0VYZJ0;fm_@1XJ6P0SHf=0:n_[`1UIFD0K6a/0;Bd]021PH40U9BD0=OGe`3D
e=@0X:2P07]kN`2FUYH0YjNW0;Ng]`1OGel0NGUi4?ooool000kooooo0300EUIF08F5Q@17AdL0QHF5
04M7A`1dM7@0R8R803De=@0i>CT0?Cdm02@T900l?3`0?Shn05=CD`0Q8B40?clo03<c<`1SHf<0=CDe
030`<01KFe/0;bl_02LW9`0o?cl05aLG03`l?00e=CD0BDU9055AD@0X:2P0Hf=S03`l?00O7al0<30`
02d];@0P8200<S8b0492@P0F5QH0;2`/05YJFP0e=CD0820P03`l?01FEUH0HF5Q0410@01GEeL206a/
K02J034a<@1BDU80IfMW030`<00d=3@0EEEE0451@@19BDT0IVIV03Lg=`1YJFT0O7al03Ti>@0_;bl0
Cdm?01`L701NGUh0T92@03<c<`1/K6`0;Rh^08R8R01CDe<0BTY:07QhN01fMWH0ADE506m_K`21PH40
DE5A03`l?028R8P0GEeM04Q8B01IFET0C4a<06=SH`1QHF40KVi^05eMG@2:RXX0KFe]096AT@30`<00
];Bd04e=C@20P800De=C06ATI00_;bl0Ng]k04Q8B01QHF40FUYJ03Xj>P1JFUX0N7Qh05IFEP21PH40
C4a<0820P01KFe/0ADE508B4Q015ADD0@D51055AD@16ATH0J6QX05=CD`1GEeL0Ph>306ATI01WIfL0
<30`061PH00g=cL0JFUY08F5Q@1VIVH0H61P03De=@1<C4`0Ogmo04];B`1<C4`0Fe]K02/[:`10@400
TY:B051@D01]KFd0P82004m?C`2PX:00SHf=03Hf=P2;Rh/0MWIf04];B`1OGel0@D5109JFUP1UIFD0
?clo07MgM`1?Cdl05ADE038b<P2OWil0VIVI065QH@1MGEd0Shn?04];B`17AdL0RHV908R8R01`L700
O7al04=3@`1mOGd0ZJVY08B4Q01?Cdl0GemO020P801ZJVX0FEUI04];B`27QhL0Bd];08N7Q`1FEUH0
@d=308F5Q@0j>SX0FUYJ08J6QP11@D40GUiN08V9R@0^;Rh0E5AD069RHP14A4@0EEEE0:NWY`1LG5`0
K6a/06i^KP1]KFd0AdM706e]K@80PH6104P0ATI605eMG@1UIFD0SXj>06=SH`0T92@0Lg=c09VIV@13
@d<0F5QH065QH@1HF5P0BTY:03lo?`1LG5`0LG5a055AD@21PH40Jf][055AD@2OWil0K6a/05eMG@1o
Ogl0E5AD06i^KP15ADD0EEEE069RHP1KFe/0?clo04M7A`1]KFd0PX:206i^KP1:BTX0PX:207IfMP1H
F5P0DU9B08B4Q01KFe/0MgMg08Z:RP1>CTh0KFe]07AdM00^;Rh0B4Q80:>SX`1DE5@0>SXj06MWI`0[
:b/0:RXZ06m_K`1[Jf/0VYZJ07YjNP16ATH0Ph>309FEU@1CDe<0FEUI08F5Q@27QhL0<S8b01@D501R
HV80Ng]k02XZ:P23Ph<>ooooo`0d079bLP1=CDd0<c<c02<S8`0N7Qh08R8R03`l?018B4P08R8R02`/
;01?Cdl0HV9R07QhN02GUiL0^;Rh08F5Q@1fMWH0Bd];0451@@1BDU80IfMW0:>SX`2XZ:P0DE5A06MW
I`0T92@071`L01LG5`0P8200>SXj04e=C@1YJFT0Kfm_07UiN@13@d<0IVIV05YJFP0Y:BT09bLW02@T
900W9bL0;2`/038b<P1bLW80=CDe061PH02SXj<0KFe]06ATI02JVYX0TI6A06m_K`80FEUI03l0G5aL
05YJFP1EEED0EUIF07=cL`34a<@0Xj>S09BDU022PX80Fe]K07MgM`1YJFT0JVYZ02`/;01CDe<0Jf][
07=cL`1nOWh0QHF506i^KP1lO7`0ZJVY075aL@2PX:00YZJV08:2PP1oOgl0QXJ602DU9@0K6a/07AdM
01lO7`0`<300IVIV0:JVYP2EUID0Ph>309NGU`1fMWH0FUYJ07inOP2KVi/0J6QX09fMW@1>CTh0<S8b
02l_;`0h>3P0Cdm?03lo?`1MGEd0U9BD0<[:bP2OWil0Z:RX05aLG013@d<0KFe]09BDU024Q8@0]kNg
069RHP0^;Rh00P2m_Kd0W`2BTY80KVi^06MWI`1YJFT0QHF50=SHf03?cll0Hf=S03De=@18B4P0Bd];
06m_K`21PH40Y:BT07IfMP0Z:RX09BDU034a<@0k>c/0[:b/02PX:00H61P05QHF05=CD`2_[jl0A4A4
05iNGP0Z:RX0BDU903Hf=P0[:b/0=3@d0;bl_038b<P0Vi^K07]kN`1cLg<0Ph>30861P@2m_Kd0[Zj^
0:NWY`1aLG40>3Ph04];B`1/K6`0JVYZ04m?C`18B4P0C4a<08J6QP1ADE40SHf=09NGU`2i^KT0b/[:
06YZJP0N7Qh071`L01dM7@0K6a/07AdM02PX:020P800]KFe071`L00/;2`0:BTY02<S8`1?Cdl0R8R8
055AD@0Q8B40820P01dM7@0Q8B40IFEU0<71`@2n_[h0LG5a06a/K01DE5@0[Jf]0<o?c`1]KFd0Bd];
03Hf=P0/;2`0?clo08B4Q01KFe/08b<S02TY:@0i>CT0M7Ad0<g=c@25QHD0UiNG08V9R@0g=cL0:RXZ
030`<01IFET0Kfm_02h^;P0e=CD0Ng]k05QHF02VYZH0EeMG02/[:`1^KVh0Jf][08Z:RP2l_;`0c/k>
0<[:bP3Fe]H0`l?30;:b/P2UYJD0[:b/0;^k^`2QXJ40VYZJ0:JVYP1aLG40CTi>059BDP1CDe<0Rh^;
09VIV@2/[:`0YJFU06=SH`12@T80Bd];07QhN01fMWH0IFEU0861P@3;bl/0QhN708B4Q02[Zj/0ZZZZ
0=3@d03;bl/0^;Rh0<O7a`3Hf=P0YJFU0:2PX018B4P0BDU909jNWP1oOgl0GUiN08n?S`0@ooooo`00
3_ooool0o`1SHf<0Ti>C04I6AP1[Jf/0FUYJ0:NWY`2OWil08R8R02d];@1EEED0;bl_0451@@0l?3`0
Lg=c04M7A`0l?3`0=CDe065QH@15ADD071`L05]KF`0_;bl0820P055AD@0V9RH0De=C028R8P0f=SH0
GEeM03Ti>@1^KVh0;2`/01TI6@15ADD0>SXj02/[:`0U9BD0Bd];034a<@0c<c<0I6AT02TY:@0M7Ad0
BTY:05QHF00m?Cd0;Bd]05MGE`1/K6`0OWin03`l?01IFET0H61P01dM7@11@D40GemO04];B`14A4@0
ADE502<S8`1UIFD0MgMg0410@00/;2`0;bl_01DE5@1ADE40Ph>3030`<01NGUh0<30`096AT@1FEUH0
BTY:06EUI@1dM7@0=3@d05MGE`1dM7@0BDU90410@025QHD0EeMG05UIF@1PH600A4A407UiN@2=SHd0
JVYZ03dm?@1dM7@0D51@06][J`2^[Zh0XJ6Q04i>CP1GEeL0<S8b061PH01?Cdl0QXJ60492@P18B4P0
HF5Q03dm?@1?Cdl0KVi^02`/;01YJFT0F5QH08b<S00a<C40<S8b07emO@1IFET0E5AD04];B`0e=CD0
Bd];04U9B@1>CTh0Vi^K06QXJ01KFe/0ATI608n?S`1@D500CDe=07QhN01YJFT0UiNG059BDP0[:b/0
K6a/03/k>`1ADE40Vi^K05eMG@0l?3`0YjNW02/[:`1:BTX0I6AT04E5A@2KVi/0N7Qh03De=@1@D500
KVi^05UIF@1`L7009bLW059BDP1PH600OWin08^;R`1[Jf/0@T92069RHP2d];@0Vi^K071`L01QHF40
SXj>05IFEP14A4@0Rh^;08j>SP1jNWX0WIfM06ATI01hN7P0RXZ:092@T01CDe<0KVi^01PH601OGel0
EeMG04M7A`22PX80>SXj04Q8B00f=SH0=CDe061PH019BDT0B4Q806a/K00/;2`0DU9B09ZJVP11@D40
I6AT04i>CP16ATH0Ph>30:ZZZP1SHf<0Ph>308R8R01SHf<0:BTY04M7A`10@400O7al0451@@1=CDd0
>3Ph06a/K01YJFT08B4Q04E5A@2TY:@0;bl_05eMG@1hN7P0HV9R04M7A`19BDT0GemO05iNGP1QHF40
SHf=071`L014A4@0YjNW07=cL`16ATH0Ogmo051@D024Q8@0IfMW01dM7@12@T80GemO03Ti>@17AdL0
LW9b06QXJ00X:2P0@d=306e]K@1EEED0De=C04Y:BP25QHD09bLW059BDP1SHf<0BTY:08:2PP1WIfL0
:b/[038b<P005`1oOgl0?3`l03hn?P20P800G5aL02XZ:P1OGel0GUiN0:2PX02IVIT0M7Ad0:n_[`2Q
XJ40@d=302<S8`13@d<0Lg=c06YZJP0O7al0NGUi05eMG@0H61P0Ti>C00kooooo02H0?Cdm06=SH`1^
KVh0>3Ph01lO7`0T92@0DU9B071`L020P800LG5a06m_K`2@T900OWin05UIF@2EUID0SHf=07]kN`21
PH40@T9203hn?P1<C4`0Rh^;0:>SX`1bLW80Ph>304E5A@0a<C407AdM01/K6`0f=SH0MGEe05IFEP0m
?Cd0=3@d04m?C`0g=cL0E5AD024Q8@807AdM06@0820P02HV9P0d=3@0Hf=S04A4A00l?3`0Ng]k0:ZZ
ZP2QXJ40L71`04=3@`11@D40DU9B061PH01HF5P0E5AD065QH@1DE5@0J6QX0:RXZ02l_;`0_Kfm08R8
R02i^KT0Ti>C04a<C018B4P07AdM02<S8`0`<300>CTi05ADE01`L700Q8B409fMW@2[Zj/0bl_;08N7
Q`1EEED0Hf=S04i>CP1;Bd/0EEEE02TY:@1GEeL0KVi^06a/K01JFUX0=cLg030`<00[:b/0>SXj07Ee
M@2RXZ80TY:B0:RXZ02_[jl0_kno07IfMP0i>CT0<S8b02TY:@0e=CD0:2PX05aLG035aLD0/K6a059B
DP10@400Lg=c06IVIP1JFUX0GEeM0<?3``3?cll0AdM7061PH01aLG40KFe]07alO01JFUX0D51@05EE
E@1dM7@0bLW90=7Ad@1eMGD0;Bd]04E5A@1?Cdl0OGem069RHP2ZZZX0MWIf02LW9`0Q8B40:2PX02/[
:`24Q8@0>CTi0P0G5aL0K`1`L700d=3@0861P@17AdL0<S8b02HV9P1HF5P0<C4a08:2PP2_[jl0N7Qh
07EeM@1GEeL0Ti>C096AT@2TY:@0/[:b0;^k^`26QXH0[Zj^09jNWP1PH600A4A403dm?@0h>3P0?Shn
06MWI`0d=3@0;bl_06EUI@1JFUX0WYjN0;jn_P2WYjL0QHF5079bLP1nOWh0PH6108Z:RP2JVYX0a/K6
0=WIf@1bLW80<30`02XZ:P0V9RH0B4Q809BDU02f][H0Vi^K08J6QP1]KFd0N7Qh0:^[Z`36a/H0/K6a
07AdM028R8P0QHF509jNWP1kNg/0WYjN08:2PP1UIFD0E5AD06IVIP2/[:`0_Kfm09bLW02i^KT0_kno
0<k>cP3IfMT0KFe]02PX:02JVYX0XZ:R04A4A018B4P0VYZJ08n?S`13@d<0=SHf0:NWY`2g]kL0fm_K
0::RXP1mOGd0Ogmo07UiN@2LW9`0b/[:0:n_[`2KVi/0dM7A07IfMP2JVYX0VIVI0:RXZ02n_[h0Shn?
04M7A`1<C4`0BDU902l_;`0m?Cd0De=C05eMG@2>SXh0X:2P0:b/[00209RHV00G092@T01`L700J6QX
05UIF@1kNg/0f][J07moO`1/K6`0^k^k0;Vi^@3Kfm/0`l?30<C4a03;bl/0WinO08V9R@1ZJVX0NWYj
05eMG@2BTY80MgMg09^KV`1VIVH04?ooool000kooooo00D0F5QH08>3P`1<C4`0LG5a0410@00207=c
L`0>03Xj>P0];Bd0>c/k02`/;00Y:BT0:RXZ06=SH`0];Bd0<30`02LW9`1KFe/0=SHf01HF5P1FEUH2
02XZ:P0X05=CD`0N7Qh0B4Q802l_;`0k>c/0FUYJ038b<P1LG5`0;Rh^01XJ6P0/;2`0=CDe024Q8@0c
<c<0@410034a<@0[:b/0GemO05EEE@0o?cl0AdM703<c<`1?Cdl0=3@d04Q8B01WIfL0Hf=S03Ph>01<
C4`0IfMW03/k>`0d=3@0EUIF03Lg=`1=CDd0GEeM03@d=01MGEd0P82003Hf=P80AdM700`06ATI04M7
A`1oOgl09BDU03hn?P0W9bL0NWYj04Q8B00o?cl0K6a/06e]K@0b<S82075aL@1R03hn?P14A4@0P820
04];B`1RHV80GUiN038b<P1<C4`0Cdm?04i>CP0k>c/0N7Qh06][J`1>CTh0WinO0:VYZ@17AdL0OGem
05EEE@1GEeL0=cLg07inOP1`L700Cdm?05mOG`0`<300B4Q808b<S01ZJVX0F5QH03lo?`2:RXX0Ng]k
02h^;P1[Jf/0@D5104E5A@1hN7P0ATI6051@D019BDT0@41008b<S01[Jf/0IFEU03Lg=`1QHF40GUiN
05ADE01/K6`0D51@06QXJ01DE5@0>3Ph08:2PP1>CTh0?Shn096AT@13@d<0>SXj08Z:RP1nOWh0JFUY
079bLP1>CTh0VYZJ0:JVYP1aLG40De=C07emO@1MGEd0Ng]k06a/K01oOgl0Lg=c06YZJP1eMGD0XJ6Q
08F5Q@1[Jf/0`l?30:6QX@16ATH0G5aL08>3P`1;Bd/0ATI60861P@1`L700IVIV09^KV`1SHf<0JVYZ
0820P01bLW80ADE507AdM00N7Qh205]KF`04059BDP2BTY80HV9R059BDP80;2`/05d0MGEe06UYJ@17
AdL0LG5a02/[:`1@D500UIFE03Hf=P1XJ6P0HV9R03Lg=`1JFUX0XZ:R04Q8B01LG5`0JFUY05iNGP1@
D500Zj^[0820P02c/k<0BTY:06ATI01OGel0SXj>05]KF`0Y:BT0C4a<08B4Q01IFET0ADE505iNGP0h
>3P0?3`l05EEE@1WIfL0Hf=S05UIF@1eMGD0KFe]03Ti>@2WYjL0Rh^;04i>CP1cLg<0B4Q806m_K`1X
J6P0Jf][04i>CP1JFUX071`L0492@P1iNGT0LW9b05eMG@0j>SX0Bd];02PX:01@D500GemO0:JVYP1V
IVH0NWYj05IFEP0e=CD0Fe]K06=SH`1@D500@d=309RHV00Y:BT0<c<c06e]K@1GEeL0<c<c06i^KP1K
Fe/0Rh^;08n?S`1NGUh0Zj^[09nOW`1LG5`0CDe=07emO@28R8P0Lg=c038b<P22PX80Ng]k02l_;`1a
LG403_ooool02P1;Bd/0?3`l03@d=01[Jf/0Ng]k04U9B@12@T80D51@05eMG@0K6a/201LG5`2401XJ
6P0N7Qh09bLW07inOP1/K6`0RXZ:0<71`@2TY:@0Q8B409VIV@28R8P0SHf=09^KV`1NGUh0BDU905AD
E01>CTh0B4Q803@d=00a<C409BDU02@T900S8b<0?Shn05mOG`10@400:b/[01/K6`0M7Ad07alO02<S
8`16ATH0Ph>309:BTP2MWId0^KVi09ZJVP1:BTX0>SXj02/[:`0X:2P0;Rh^03Ph>00k>c/0A4A405QH
F02BTY80ZJVY09ZJVP2b/[80_Kfm09:BTP2=SHd0M7Ad05UIF@0Q8B4071`L01lO7`0U9BD09bLW02h^
;P0a<C40Bd];07moO`2RXZ80?3`l05IFEP1CDe<0C4a<02HV9P1GEeL0D51@02l_;`0Q8B407AdM01hN
7P0Q8B4092@T02HV9P0T92@0;bl_06IVIP2QXJ40^[Zj0=_Kf`2o_kl0YjNW0861P@1EEED0BTY:05EE
E@1iNGT0Ph>305=CD`0a<C40>SXj03Ph>011@D40FUYJ075aL@2?Shl0`l?30<S8b02XZ:P0_Kfm08^;
R`1EEED0ATI602XZ:P0N7Qh0;Rh^07EeM@35aLD0_[jn0:n_[`1kNg/0Cdm?06QXJ02HV9P0TY:B07Ad
M01DE5@0?Cdm02@T900X:2P09RHV055AD@1VIVH09RHV04m?C`1JFUX0:RXZ0P1DE5@0Q00m?Cd0;2`/
038b<P1gMgL0^;Rh07alO00/;2`08R8R059BDP3>c/h0[Jf]0:VYZ@2MWId0TY:B09FEU@2b/[80VIVI
07inOP2ATI40VIVI0:6QX@2/[:`0OGem02d];@0h>3P0LW9b04Y:BP0d=3@0BDU90<O7a`2h^;P0IFEU
03hn?P0j>SX0>3Ph0492@P2g]kL0Z:RX0:f][@0j>SX09RHV02LW9`1CDe<0^KVi0;Jf]P2WYjL0/k>c
0;^k^`2j^[X0M7Ad06IVIP2QXJ40_;bl0:ZZZP32`/80Y:BT0492@P11@D40E5AD06QXJ026QXH0YZJV
0<71`@2^[Zh0X:2P06MWI`21PH40Z:RX0<K6aP1YJFT0;Rh^06MWI`2l_;`0/k>c0:^[Z`2XZ:P0SXj>
0:RXZ02b/[80^KVi0:^[Z`1gMgL0HF5Q07UiN@26QXH0OGem06YZJP1WIfL0CTi>02LW9`1>CTh0M7Ad
09FEU@2GUiL0QHF50::RXP1dM7@0H61P065QH@1`L700Kfm_06i^KP1ZJVX0MGEe07MgM`1dM7@0LW9b
07=cL`2IVIT0Y:BT0:FUY@1IFET0HV9R0:j^[P0o?cl0UYJF0<k>cP34a<@0c<c<0<?3``21PH40I6AT
06=SH`1NGUh0K6a/04i>CP1PH600MWIf0:^[Z`32`/80DE5A02@T913ooooo000>ooooo`2F059BDP1M
GEd0<S8b06UYJ@16ATH0X:2P08j>SP0H61P0820P04M7A`0W9bL09BDU02XZ:P1^KVh0Cdm?02XZ:P0W
9bL0CTi>04=3@`0Q8B40ADE502TY:@0G5aL0EeMG03De=@1ADE407QhN02LW9`1UIFD0AdM7055AD@0h
>3P06ATI05=CD`1:BTX0>3Ph034a<@1@D500A4A4024Q8@11@D40?clo02XZ:P1HF5P0<S8b04e=C@0S
8b<0B4Q806][J`1YJFT0@d=303Ti>@1eMGD05ADE02LW9`1PH600ADE50410@00l?3`03`l?02/[:`1R
HV80?clo04E5A@0g=cL03Ph>02l_;`1WIfL0=cLg03Ph>00E5AD0DE5A05EEE@0a<C40G5aL06i^KP0/
;2`0AdM706m_K`1:BTX0FEUI096AT@14A4@0DU9B05IFEP13@d<0H61P06YZJP17AdL09bLW04Q8B01e
MGD0>SXj092@T02l_;`0EeMG07EeM@1CDe<0RHV906QXJ02?Shl0FEUI059BDP1PH600EUIF0492@P1b
LW80Hf=S05IFEP1LG5`0VYZJ08n?S`10@400PH6107EeM@12@T80EeMG03Ph>01JFUX0I6AT04U9B@2O
Wil0GemO08j>SP14A4@0FEUI02l_;`12@T80N7Qh05YJFP1XJ6P0@D5101XJ6P1TI6@0Kfm_04i>CP29
RHT08R8R024Q8@1iNGT0N7Qh05]KF`1cLg<0=3@d06MWI`1iNGT0KVi^03/k>`1YJFT0D51@0P1WIfL0
@@1^KVh0G5aL06ATI01XJ6P0ZZZZ092@T01NGUh0^k^k0:BTY01iNGT0IfMW08f=S@0h>3P0@d=307Qh
N02CTi<0M7Ad0;Bd]00i>CT0F5QH09:BTP2EUID0CTi>07QhN00^;Rh0IVIV06a/K00h>3P0T92@06m_
K`1YJFT0GUiN02/[:`1bLW80EUIF03lo?`2<S8`0FEUI04U9B@2VYZH0=CDe05ADE01aLG40?3`l071`
L02RXZ80Hf=S05UIF@1fMWH0Kfm_075aL@3Fe]H0Lg=c0;2`/019BDT0C4a<06i^KP27QhL0LG5a03<c
<`1;Bd/0Ogmo06UYJ@1;Bd/0JVYZ0080E5AD01D0Jf][08f=S@1bLW80Bd];05UIF@1[Jf/0<30`08F5
Q@26QXH0CTi>06][J`16ATH0DU9B061PH017AdL0F5QH05iNGP0W9bL0B4Q805mOG`1NGUh00P12@T80
901DE5@09bLW055AD@1?Cdl0Y:BT055AD@1TI6@0IFEU0492@P1WIfL0Jf][04E5A@0d=3@0LW9b03Ti
>@0n?Sh0O7al03dm?@0U9BD0GUiN03`l?028R8P0Rh^;06QXJ02d];@0]kNg05=CD`0S8b<0@T9205IF
EP1HF5P0:2PX06UYJ@1OGel071`L06i^KPkooooo0280@D5104M7A`1LG5`0V9RH0;Jf]P1VIVH0ATI6
06MWI`1ZJVX0C4a<03Lg=`0J6QX04a<C01DE5@0D51@06a/K07IfMP1BDU80bLW909NGU`17AdL0>c/k
05]KF`2GUiL0FUYJ0:BTY02n_[h0MgMg05QHF00j>SX08b<S01hN7P0L71`06ATI0P0L71`04P1;Bd/0
GUiN03De=@0_;bl09RHV02XZ:P0U9BD0DE5A0;Bd]02^[Zh0Ng]k07UiN@2d];@0?Cdm030`<00/;2`0
:2PX024Q8@809RHV01d0:2PX02h^;P12@T80[:b/09fMW@2^[Zh0/K6a0;no_`1^KVh092@T02d];@1C
De<09bLW01/K6`0M7Ad07alO028R8P0Q8B409BDU059BDP2m_Kd0NGUi0861P@1fMWH0LW9b030`<011
@D40=cLg01dM7@0301`L700g01dM7@0V9RH08b<S03Xj>P1UIFD0D51@05EEE@2^[Zh0clo?0;no_`39
bLT0Zj^[0;Rh^02i^KT0YZJV03`l?00N7Qh071`L01/K6`0J6QX06a/K01hN7P0X:2P0;Rh^059BDP2b
/[80_Kfm09jNWP3AdM40^[Zj05mOG`15ADD0;Bd]01dM7@0W9bL0LW9b0<?3``28R8P0VYZJ0;fm_@2M
WId0Rh^;09jNWP1_Kfl0GEeM05EEE@1CDe<0>c/k02TY:@0U9BD0I6AT09RHV01bLW80=cLg01dM7@02
01XJ6P2501hN7P1;Bd/0BDU903hn?P1XJ6P0_;bl07]kN`0k>c/0Kfm_07QhN03Bd]80bLW90<g=c@2J
VYX0Ph>307MgM`2JVYX0Y:BT096AT@1FEUH0?Shn03Ti>@0c<c<0GEeM09:BTP28R8P0MGEe0492@P0e
=CD0NWYj0:JVYP2YZJT0`L7108>3P`1MGEd0;2`/07EeM@2b/[80CDe=09bLW02EUID0?Shn02<S8`1>
CTh0T92@04E5A@0i>CT0M7Ad07UiN@2OWil0U9BD09RHV02c/k<0[Zj^0<?3``3QhN40^;Rh03hn?P0l
?3`0@T9204i>CP1SHf<0RHV90:BTY023Ph<0QhN70<S8b02`/;00Zj^[07emO@2f][H0/K6a05EEE@12
@T80DU9B0410@01?Cdl0VIVI0=[JfP3AdM40Lg=c04Y:BP11@D40HF5Q09FEU@2KVi/0Ng]k05mOG`0k
>c/04A4A00h>3P0@4100;Bd]05QHF02KVi/0bLW904Y:BP0M7Ad06QXJ01dM7@0Q8B40;bl_09NGU`0^
;Rh0:b/[03De=@0k>c/0HF5Q07MgM`2^[Zh0YjNW08j>SP3EeMD0gMgM0<[:bP1ADE40Shn?0<?3``2@
T900F5QH02LW9`0[:b/0;Rh^05IFEP1`L700GemO09NGU`28R8P0YZJV09bLW01nOWh0Q8B404A4A00@
ooooo`003_ooool0o`1FEUH0Jf][04i>CP1[Jf/0>SXj08Z:RP2HV9P0;2`/024Q8@12@T809RHV01XJ
6P0K6a/0LG5a05aLG00V9RH08B4Q03@d=01QHF40:2PX02XZ:P19BDT04A4A0451@@0l?3`0GEeM03De
=@0U9BD0Bd];03Lg=`0o?cl0>CTi00`<300Q8B40>3Ph03Lg=`0U9BD0=3@d051@D00S8b<0DU9B030`
<00F5QH0?Shn02TY:@19BDT06QXJ03@d=015ADD0LW9b05=CD`0a<C40E5AD02TY:@0P8200>c/k03dm
?@1<C4`0D51@02l_;`0j>SX0Hf=S03hn?P1?Cdl0Hf=S01XJ6P0`<300MgMg030`<00m?Cd09RHV06MW
I`1MGEd0;Bd]071`L01^KVh0@D5105YJFP1TI6@0=SHf05MGE`2JVYX0Hf=S06MWI`17AdL0;Bd]05mO
G`2;Rh/0JFUY03lo?`18B4P0GEeM02`/;01kNg/0Shn?05iNGP1KFe/05aLG06a/K01_Kfl0S8b<05QH
F00g=cL0M7Ad05eMG@14A4@0M7Ad03De=@1=CDd0IVIV0861P@1EEED09bLW06=SH`1XJ6P0@T9204A4
A00^;Rh0>CTi05eMG@0i>CT0Rh^;05ADE01^KVh0>c/k07EeM@1PH600ADE506UYJ@1LG5`0Xj>S05IF
EP0c<c<0KVi^05IFEP0i>CT0X:2P06QXJ00S8b<0Ph>306a/K016ATH0O7al05aLG01cLg<0U9BD04e=
C@15ADD0Q8B405mOG`1aLG40Bd];06][J`2ATI40Ph>306a/K02YZJT0ADE5059BDP32`/80[jn_0820
P01UIFD0Ti>C06a/K00i>CT0K6a/09BDU027QhL0_[jn07moO`1<C4`0O7al079bLP0m?Cd0Kfm_01PH
601JFUX0KFe]03/k>`2MWId0@D51051@D01VIVH0;bl_07QhN018B4P0;bl_079bLP0W9bL0Cdm?0:^[
Z`1=CDd0DE5A06][J`0f=SH0De=C09FEU@16ATH0Kfm_09RHV024Q8@0Hf=S0<G5a@1^KVh0WIfM05YJ
FP1fMWH0Ogmo08>3P`1OGel0>SXj06a/K02YZJT0R8R805EEE@1[Jf/09bLW02d];@12@T80AdM705iN
GP1:BTX0CDe=079bLP0o?cl0]kNg0<K6aP1<C4`0LW9b05IFEP1PH600IFEU05iNGP18B4P0EeMG02l_
;`1LG5`0WinO0861P@0N7Qh0@d=305eMG@0`<300G5aL08:2PP2][Jd0IVIV06][J`1CDe<0<C4a04i>
CP1BDU80@T9203Hf=P005`28R8P09RHV02/[:`1PH600EeMG034a<@1cLg<0?Cdm06UYJ@1fMWH0CDe=
0:n_[`2][Jd0J6QX03/k>`1CDe<0M7Ad07QhN00f=SH0K6a/07YjNP0a<C40NGUi00kooooo00h0];Bd
0;^k^`2c/k<0^[Zj092@T02b/[80VIVI05iNGP12@T80?Shn03`l?018B4P0?Shn02@T90805ADE04L0
LW9b09fMW@3AdM40QHF503hn?P1CDe<0L71`04i>CP1HF5P0UIFE05UIF@1@D500DE5A065QH@1LG5`0
EeMG05IFEP10@400<c<c02<S8`0[:b/0ZJVY09:BTP27QhL0SHf=09RHV02;Rh/0ZZZZ0<30`036a/H0
];Bd0;2`/03>c/h0U9BD03/k>`0S8b<0:2PX02/[:`0K6a/07QhN0492@P1NGUh0C4a<07UiN@1NGUh0
Bd];08Z:RP1?Cdl0QXJ603lo?`0X:2P0:RXZ04E5A@0I6AT06a/K01`L700T92@0=cLg05]KF`1_Kfl0
];Bd06][J`1WIfL0GUiN04];B`16ATH0EeMG06a/K00P820071`L01TI6@0201`L700?02DU9@0n?Sh0
F5QH04Q8B01DE5@0I6AT075aL@38b<P0YjNW0:JVYP1_Kfl0DU9B04i>CP2EUID08R8R008071`L00<0
6QXJ01HF5P0L71`00P0J6QX01@0R8R80>3Ph09:BTP2QXJ40NWYj0080];Bd00h0;bl_02HV9P0Q8B40
7alO03@d=023Ph<0]KFe07]kN`1]KFd0SXj>06m_K`1/K6`0TY:B06YZJP80DU9B00X0IFEU05eMG@13
@d<0GEeM07UiN@1VIVH0P82002@T900G5aL05ADE0`0G5aL0H00e=CD0IFEU09jNWP1SHf<0WinO0;Fe
]@2EUID0/[:b08:2PP1aLG40UiNG06][J`1?Cdl0A4A405EEE@23Ph<0Q8B404Q8B00X:2P09BDU028R
8P0b<S80Cdm?075aL@29RHT0Vi^K0;Jf]P3Fe]H0NGUi05QHF02QXJ40gmoO0=cLg035aLD0]KFe0<G5
a@1BDU80Fe]K0<o?c`1`L700=CDe079bLP1NGUh07AdM01/K6`0W9bL09RHV02DU9@0/;2`0^[Zj0:FU
Y@3@d=00g]kN0<K6aP2ATI40VYZJ07alO01XJ6P0C4a<0451@@1@D500Vi^K0:2PX02k^k/0f][J0=KF
eP1GEeL0K6a/0=;BdP2TY:@0EUIF03Ph>00P820071`L02@T901NGUh0`l?30=3@d02`/;00RHV907in
OP2@T900^[Zj0:>SX`2d];@0`L71092@T00Z:RX05aLG01<C4`0P8200HF5Q0;6a/@2TY:@0C4a<01dM
7@8071`L02406a/K024Q8@1GEeL07QhN01dM7@0Q8B4092@T03@d=01ZJVX0NGUi0<k>cP3IfMT0e]KF
0=CDe038b<P0bl_;0<30`02_[jl0IfMW02@T900L71`07AdM01/K6`0N7Qh0<c<c0:BTY037alL0/;2`
0;Jf]P32`/80PX:2071`L01LG5`04?ooool000kooooo0?l0F5QH07QhN01^KVh0NGUi03dm?@25QHD0
^;Rh071`L00f=SH0FEUI02/[:`17AdL0>c/k07]kN`1JFUX08R8R03Lg=`15ADD0MGEe02TY:@0K6a/0
92@T01<C4`1<C4`0?clo04U9B@0U9BD0?3`l069RHP0n?Sh0<30`034a<@0Y:BT0=cLg03Ti>@0h>3P0
;bl_03<c<`0n?Sh0:BTY06m_K`1XJ6P0>c/k059BDP0_;bl0FUYJ04a<C016ATH0E5AD06][J`18B4P0
;2`/06e]K@1@D500:RXZ03`l?00W9bL0DE5A06YZJP13@d<0@D5105eMG@0[:b/0DU9B07QhN00U9BD0
:b/[071`L00F5QH0>3Ph02h^;P1SHf<0@d=301/K6`1QHF40OWin04E5A@1fMWH0Ng]k03Lg=`1DE5@0
Z:RX051@D01jNWX0GUiN03De=@1<C4`0HF5Q03/k>`0g=cL0ADE506IVIP0m?Cd0/K6a0:2PX01JFUX0
Ph>305iNGP1eMGD0HF5Q08B4Q02;Rh/0BDU9071`L01QHF40@T9206][J`1LG5`0@D5105=CD`1NGUh0
Ti>C0492@P1]KFd0LG5a04];B`1bLW80H61P05IFEP1>CTh092@T08R8R01gMgL0DE5A034a<@1iNGT0
M7Ad04i>CP1VIVH0<S8b0:RXZ01iNGT0?Cdm07QhN01gMgL09bLW08b<S01[Jf/0<S8b07inOP27QhL0
B4Q806IVIP1TI6@0HV9R09jNWP1OGel0=cLg0861P@1RHV80LW9b04i>CP11@D40LG5a08R8R01]KFd0
/K6a04Q8B00m?Cd0[jn_0;Zj^P1nOWh0NWYj0:^[Z`22PX80?Cdm06ATI02;Rh/0Hf=S0:ZZZP2@T900
H61P09VIV@20P800E5AD06e]K@0Z:RX0De=C05aLG01<C4`0[:b/08Z:RP1fMWH0@d=302<S8`1eMGD0
G5aL04Y:BP2GUiL0@D5103`l?02a/K40@D5106ATI01eMGD0?Cdm04e=C@2SXj<0BTY:03hn?P1=CDd0
B4Q804];B`2b/[80MGEe09RHV00n?Sh0Bd];05YJFP2=SHd0EUIF02DU9@0W9bL0CDe=06a/K01DE5@0
G5aL03@d=011@D40@d=304Q8B01cLg<0E5AD055AD@1[Jf/0:2PX055AD@26QXH0FEUI07inOP0o?cl0
FEUI065QH@1YJFT0C4a<05MGE`0c<c<08R8R04M7A`1GEeL0AdM703@d=01KFe/05aLG02l_;`0k>c/0
QXJ606QXJ01YJFT0N7Qh03lo?`1@D500I6AT061PH014A4@001L0K6a/03Xj>P0];Bd0HV9R06EUI@0e
=CD0O7al05=CD`1kNg/0QXJ605EEE@2k^k/0b<S807alO010@400FUYJ07=cL`1iNGT0=SHf06QXJ026
QXH0>c/k07IfMP0>ooooo`0P07YjNP1iNGT0UIFE07moO`1bLW80F5QH07MgM`2GUiL0XZ:R05EEE@13
@d<0?Cdm02h^;P16ATH0Bd];02LW9`22PX80`L710=;BdP2f][H0K6a/075aL@1]KFd0:b/[0492@P2[
Zj/0QXJ604];B`0c<c<0=cLg04I6AP1HF5P204a<C00^055AD@1IFET0GEeM0:VYZ@1fMWH0K6a/055A
D@0l?3`0>CTi07=cL`2d];@0dM7A0<?3``2a/K40`l?30;>c/`2HV9P0KFe]02TY:@0P82009BDU051@
D01NGUh0>SXj02LW9`1?Cdl0GEeM02PX:01CDe<0MgMg06IVIP21PH40ADE503Ph>00o?cl0>SXj02h^
;P1@D500FUYJ05EEE@19BDT0@T9207inOP2n_[h0U9BD08b<S080Rh^;01/0YJFU0<W9b@1[Jf/0=CDe
020P800L71`06ATI01dM7@1=CDd0B4Q80492@P17AdL0OWin06YZJP3AdM40^[Zj0;2`/01fMWH0>SXj
034a<@1PH6008b<S01/K6`0L71`06a/K01`L700I6AT00P0L71`0D00J6QX0:2PX071`L01_Kfl0R8R8
08^;R`2GUiL0GemO02<S8`0O7al0=3@d05aLG011@D40FUYJ07UiN@1TI6@0Hf=S0410@00f=SH0@T92
06a/K01>CTh0Fe]K04Y:BP1bLW80Q8B406=SH`10@400?3`l05ADE01BDU805QHF01LG5`0E5AD061PH
01XJ6P0G5aL0HV9R09FEU@2;Rh/0NWYj0=CDe02k^k/0MgMg05iNGP0`<300E5AD0410@00d=3@0@410
03`l?01@D500HF5Q08f=S@27QhL0KFe]04e=C@1NGUh0YZJV0;2`/01mOGd0JFUY059BDP30`<00N7Qh
04E5A@1JFUX0alO70=cLg03Hf=P0]kNg0=GEe@2n_[h0SXj>0=SHf02n_[h0M7Ad092@T010@4005QHF
0`0G5aL0<00E5AD06a/K05MGE`1CDe<0e]KF0=gMg@2WYjL0IFEU0;Fe]@1jNWX0G5aL06MWI`1LG5`0
IFEU09FEU@1IFET0MGEe07inOP37alL0aLG50<_;b`2m_Kd0/K6a0492@P0U9BD07AdM01XJ6P0M7Ad0
HF5Q0:j^[P2l_;`0XJ6Q055AD@1DE5@0PH610:NWY`2QXJ40Q8B409^KV`38b<P0XJ6Q05iNGP1ADE40
QXJ609jNWP25QHD0NWYj05=CD`8071`L01L06a/K01dM7@0Y:BT07QhN020P800W9bL09BDU02<S8`0/
;2`0G5aL09RHV03Hf=P0`/;20:JVYP2DU9@0KVi^0<71`@3Ng]h0YZJV04a<C00N7Qh07AdM01`L7003
01dM7@0805]KF`3<c<`0d];B0<G5a@3AdM40S8b<05ADE01MGEd@ooooo`003_ooool0o`1>CTh0G5aL
05YJFP1PH600:2PX05UIF@25QHD0DU9B02l_;`0h>3P08b<S04A4A00d=3@0LG5a02/[:`0V9RH0B4Q8
034a<@1GEeL05aLG02<S8`0e=CD06a/K04E5A@0K6a/0@d=304I6AP11@D40JVYZ02h^;P0];Bd0IfMW
05UIF@0g=cL0@D5102/[:`18B4P0;bl_02h^;P0G5aL0?Shn05eMG@0b<S80>c/k00h>3P1<C4`0@D51
0410@01RHV80FEUI0451@@0G5aL0D51@04U9B@0X:2P0?Shn03Ti>@0n?Sh0B4Q804E5A@12@T80LW9b
0492@P0/;2`0GEeM02<S8`0X:2P0O7al034a<@0_;bl09bLW061PH01gMgL0;2`/05eMG@1hN7P08B4Q
05IFEP1cLg<0De=C06ATI033`l<0:b/[06UYJ@1nOWh0A4A405YJFP2CTi<0>c/k02HV9P1[Jf/0Rh^;
04Q8B01kNg/0[Zj^059BDP1UIFD0J6QX06UYJ@1OGel0EUIF08>3P`1?Cdl0LW9b030`<01:BTX0R8R8
08>3P`1GEeL0IVIV06UYJ@2YZJT0DU9B075aL@15ADD0BDU9071`L01MGEd0DE5A04I6AP0[:b/0Hf=S
079bLP1ZJVX0BTY:08B4Q01XJ6P0CDe=07EeM@1UIFD0XZ:R05=CD`11@D40Ogmo08>3P`14A4@0VIVI
07EeM@0`<300A4A406YZJP1VIVH0WYjN06EUI@1RHV80][Jf0861P@0l?3`0LW9b059BDP1]KFd0NGUi
055AD@23Ph<0EeMG04Y:BP2@T900E5AD02d];@1lO7`0TI6A06i^KP1VIVH0U9BD071`L01@D500Ti>C
0;2`/01^KVh0YJFU07=cL`0k>c/0GemO08J6QP1OGel0GUiN03De=@1bLW80N7Qh03Lg=`2?Shl0CDe=
065QH@1iNGT0;Rh^06i^KP1XJ6P0;bl_06e]K@1>CTh0>SXj0:b/[01;Bd/0>c/k06ATI00g=cL0BTY:
0:>SX`1/K6`0C4a<065QH@1EEED0IVIV0;Fe]@23Ph<0VIVI04A4A01HF5P0Kfm_0861P@1UIFD0BTY:
02PX:01ADE40O7al069RHP1QHF40=SHf04Y:BP11@D40AdM707YjNP1ADE40E5AD07UiN@18B4P0L71`
0:ZZZP19BDT0R8R805mOG`13@d<0B4Q805UIF@1BDU80C4a<0410@00k>c/0EUIF05mOG`0a<C40;2`/
05]KF`0X:2P0BDU904];B`2CTi<0NGUi06YZJP1oOgl0BTY:051@D01XJ6P0CTi>03hn?P005`1^KVh0
DE5A03Xj>P1OGel0EEEE03De=@2>SXh0CDe=07inOP22PX80=SHf06IVIP2SXj<0MgMg0451@@1JFUX0
M7Ad04Q8B00^;Rh0RXZ:09>CT`0j>SX0I6AT00kooooo02l0G5aL07IfMP1YJFT0MGEe05eMG@0h>3P0
;2`/02l_;`14A4@0U9BD05MGE`0/;2`0<S8b02/[:`0c<c<0WYjN0:BTY03Ph>00Ng]k0820P02UYJD0
V9RH05MGE`1PH600^KVi0:BTY01mOGd0WYjN0;Rh^01dM7@0?clo04I6AP12@T80>3Ph02h^;P0];Bd0
GEeM0:6QX@2RXZ80?Cdm04i>CP1KFe/0?Cdm059BDP2o_kl0cLg=0=3@d00209>CT`0h08V9R@2HV9P0
b<S80:JVYP1oOgl0U9BD06=SH`0V9RH06a/K01TI6@0R8R80@D5102LW9`0a<C40WYjN06e]K@1TI6@0
Ph>303hn?P0e=CD0FEUI06i^KP1MGEd0EUIF04m?C`14A4@0>c/k07EeM@33`l<0P82006m_K`1`L700
JVYZ06=SH`1NGUh0GemO07QhN01SHf<0?Shn02TY:@0^;Rh09BDU03/k>`10@400GEeM07QhN01mOGd0
i>CT0=?Cd`2FUYH0IfMW02l_;`0k>c/0B4Q804M7A`0N7Qh201`L700F01LG5`0J6QX06a/K01LG5`0K
6a/0?Cdm0;Fe]@1IFET0^;Rh092@T02;Rh/0OWin07QhN01FEUH0HF5Q03Ph>00Y:BT0<30`07IfMP1G
EeL0F5QH03Ph>080<S8b00d0Cdm?07emO@1XJ6P0S8b<06UYJ@2;Rh/0@d=303@d=00_;bl0>SXj06UY
J@0O7al05QHF00805aLG0P0H61P0:P1HF5P0DE5A08R8R01lO7`0d=3@092@T01TI6@0:RXZ028R8P0W
9bL0DE5A02LW9`0[:b/0?Cdm04E5A@11@D40;Bd]05EEE@2QXJ40e]KF0<K6aP3GemL0][Jf05mOG`0h
>3P0<S8b08Z:RP1RHV80BTY:04m?C`37alL0aLG505YJFP0d=3@0CDe=0861P@2ZZZX0hn?S0=oOg`2j
^[X0U9BD02PX:0@05aLG03H06QXJ01TI6@16ATH0QXJ60<?3``3AdM40/[:b07]kN`2TY:@0R8R8055A
D@0d=3@08R8R038b<P2IVIT0JVYZ08b<S02NWYh0a<C40<O7a`1lO7`0Ph>30<?3``16ATH0:b/[01XJ
6P0L71`07alO06m_K`2o_kl0`/;20<71`@1gMgL0U9BD07alO01GEeL0A4A4034a<@0^;Rh0EEEE05mO
G`1UIFD0^;Rh0;bl_02NWYh0Xj>S08j>SP27QhL0>3Ph02TY:@0P8200:2PX02<S8`0L71`202DU9@0@
02LW9`0S8b<0:RXZ05]KF`3?cll0b<S80:f][@2PX:00TY:B08Z:RP3=cLd0b/[:06m_K`0X:2P071`L
01dM7@8071`L00X07alO03@d=02LW9`0K6a/07EeM@2_[jl0_Kfm0;Bd]024Q8@0HV9R4?ooool000ko
oooo02@0NGUi05aLG015ADD0IfMW04U9B@17AdL0TY:B038b<P0];Bd0De=C02PX:00n?Sh0<30`06YZ
JP0e=CD05QHF03Lg=`0S8b<0CDe=028R8P0@4100:RXZ01LG5`19BDT0=SHf03De=@0o?cl092@T06=S
H`0f=SH08R8R06QXJ014A4@0BDU904];B`0e=CD202h^;P1/038b<P14A4@0I6AT05YJFP0a<C40E5AD
03hn?P1CDe<0:b/[04a<C021PH40OGem06QXJ00L71`0IFEU03@d=00Q8B40F5QH04=3@`1DE5@0ADE5
01dM7@0/;2`0KVi^04i>CP0b<S80Fe]K014A4@0T92@0NWYj04m?C`17AdL05ADE03Lg=`1cLg<0<S8b
059BDP1cLg<0?clo04M7A`1ZJVX0GUiN06QXJ03<c<`0HF5Q05]KF`1;Bd/0DE5A04];B`2QXJ40@410
028R8P0f=SH0KFe]03Ti>@1aLG40/;2`075aL@1^KVh0KFe]071`L021PH40L71`08B4Q01@D500GUiN
03Hf=P0/;2`0G5aL07AdM01FEUH0Bd];05IFEP2KVi/0DE5A07AdM01KFe/0?Shn05QHF00l?3`0EEEE
05ADE016ATH0LW9b071`L020P800BTY:09>CT`1fMWH0FUYJ08F5Q@1HF5P0^;Rh07emO@0e=CD0LW9b
08:2PP0[:b/0P82007IfMP0h>3P0@410069RHP14A4@0GUiN05]KF`13@d<0V9RH0P1:BTX0M023Ph<0
FUYJ055AD@1GEeL0C4a<08n?S`1hN7P0CTi>0;2`/01XJ6P0HV9R0;Vi^@2h^;P0OWin06i^KP2XZ:P0
HF5Q02TY:@1:BTX0V9RH065QH@2SXj<0Vi^K04Y:BP1[Jf/0V9RH07EeM@1NGUh0;bl_034a<@1EEED0
<30`08n?S`2FUYH0H61P07=cL`0f=SH0Cdm?06YZJP18B4P0PX:205eMG@0j>SX0WinO05mOG`1<C4`0
M7Ad0410@016ATH0]KFe07AdM01EEED0Kfm_079bLP1>CTh0X:2P07UiN@2NWYh0F5QH059BDP2<S8`0
QHF506QXJ01MGEd0;Rh^05MGE`26QXH0Jf][06MWI`11@D40Cdm?03`l?010@400LG5a05YJFP1KFe/0
L71`04E5A@1jNWX0^KVi055AD@1mOGd0KFe]04e=C@1BDU80G5aL04e=C@1BDU80=3@d03Lg=`1HF5P0
Fe]K03De=@0^;Rh0Lg=c02PX:015ADD0DU9B08N7Q`1gMgL0MGEe08N7Q`18B4P0DE5A06EUI@15ADD0
BTY:0:n_[`1UIFD0<C4a061PH00c<c<0820P06][J`17AdL0OWin0P1OGel0302<S8`0YZJV06m_K`11
@D40FEUI06][J`1^KVh0<S8b05MGE`1jNWX0>CTi069RHPkooooo01l0[jn_09NGU`1NGUh0GUiN09JF
UP1MGEd0;bl_02TY:@0W9bL0=SHf06][J`1nOWh0ATI602/[:`0X:2P0ADE508f=S@34a<@0_;bl09NG
U`2i^KT0c/k>0<K6aP2][Jd0H61P07alO01WIfL0L71`08b<S03Fe]H0XJ6Q0080R8R801P0U9BD0:^[
Z`2d];@0^[Zj07emO@1GEeL0JVYZ05ADE01JFUX0>3Ph04A4A02VYZH0_Kfm0;6a/@1=CDd0@41005]K
F`2MWId0aLG50<K6aP3=cLd0b/[:04a<C00N7Qh201`L700_01TI6@0S8b<0>CTi028R8P1kNg/0Vi^K
05EEE@1lO7`0][Jf08J6QP1kNg/0J6QX06=SH`10@400<C4a02`/;00T92@0Ogmo0;Jf]P2DU9@0LG5a
06ATI01@D500F5QH06m_K`1JFUX0CTi>06=SH`1mOGd0MGEe05eMG@0i>CT0<30`03Hf=P1<C4`0?3`l
09NGU`3GemL0d];B0<c<c01gMgL0ATI604U9B@0h>3P0F5QH02h^;P0L71`00P0J6QX04P0F5QH06a/K
024Q8@0W9bL0Hf=S0=7Ad@3=cLd0YZJV0<;2`P2d];@0_;bl0=CDe01aLG40?3`l02<S8`0L71`0;Bd]
079bLP80HV9R00l0;bl_02h^;P0W9bL0@41007=cL`1oOgl0JVYZ07inOP2@T900O7al04Q8B00g=cL0
<c<c05MGE`14A4@00P0E5AD201LG5`0801XJ6P1bLW80E5AD051@D01BDU80a<C405IFEP0M7Ad201/K
6`0G01TI6@0Z:RX0@d=302PX:00`<300@D5103hn?P0W9bL0:BTY03`l?02>SXh0Ng]k0<O7a`3:b/X0
YZJV03Ti>@0X:2P0NGUi06=SH`1MGEd0QXJ60=;BdP1TI6@00P0O7al0;`0R8R80ADE50451@@38b<P0
jNWY0>[ZjP2XZ:P08B4Q01dM7@0H61P05ADE01HF5P0J6QX0820P03lo?`2^[Zh0clo?0<30`01ADE40
9RHV05aLG01^KVh07AdM01hN7P0m?Cd0K6a/05EEE@19BDT0<30`08>3P`3EeMD0dM7A05=CD`1QHF40
`L710820P00j>SX08R8R01`L700R8R80NWYj0:RXZ02k^k/0e=CD0:ZZZP17AdL0;Bd]0080;Rh^00D0
:2PX02LW9`0b<S80ADE502h^;P020:NWY`0Q07inOP2=SHd0_[jn0<_;b`29RHT0PH6106i^KP1RHV80
7AdM02<S8`0a<C40=3@d0451@@1JFUX0U9BD0;Jf]P2j^[X0b/[:0;Bd]02k^k/0cLg=0:b/[02KVi/0
O7al038b<P0J6QX06ATI01`L700M7Ad071`L03<c<`2ATI40E5AD0080C4a<00D0NWYj0;jn_P2CTi<0
Kfm_07alO00@ooooo`003_ooool0o`1^KVh0GEeM06YZJP1GEeL0GEeM04a<C02c/k<0>CTi01dM7@0j
>SX0=3@d02DU9@0C4a<0HV9R05aLG00R8R80<30`01HF5P1DE5@0?3`l010@400j>SX03`l?03Ti>@1A
DE40;bl_05mOG`0C4a<0>c/k04];B`0a<C40NWYj030`<00X:2P0@41004Y:BP0`<300<C4a04e=C@1@
D500MgMg06=SH`0F5QH0?Shn03dm?@1?Cdl09BDU02XZ:P1=CDd0IfMW04Q8B00Q8B40IVIV03Hf=P0I
6AT0@D5103lo?`1>CTh0BTY:03Ti>@0c<c<0CTi>04I6AP0Z:RX0IFEU02<S8`0I6AT0I6AT04U9B@0a
<C409RHV03`l?01hN7P0<c<c05YJFP1lO7`0EEEE065QH@2>SXh0De=C06=SH`33`l<0MWIf07inOP1F
EUH0FUYJ059BDP2m_Kd0GEeM03Xj>P1FEUH0QHF504I6AP1fMWH0Ti>C05aLG01kNg/0GEeM05]KF`1b
LW80H61P0:2PX016ATH0I6AT05ADE01ADE40U9BD06YZJP11@D40D51@05eMG@2MWId0?clo06ATI01C
De<0G5aL08B4Q01RHV80BTY:04a<C00Q8B40DE5A06YZJP1PH600<C4a05aLG01fMWH0De=C069RHP0d
=3@0Ogmo05iNGP14A4@0L71`08j>SP0];Bd0K6a/07moO`10@400>c/k071`L01cLg<0MGEe07]kN`17
AdL0ZJVY069RHP10@400K6a/05MGE`1FEUH0O7al04i>CP2:RXX0Ogmo05ADE02GUiL0WYjN0861P@34
a<@0_kno09FEU@1^KVh0TY:B09ZJVP0m?Cd0EUIF0:j^[P1SHf<0UiNG09nOW`1=CDd0HF5Q09fMW@1k
Ng/0GUiN03Ti>@0k>c/0Fe]K030`<021PH40X:2P06=SH`1iNGT0=cLg04a<C01/K6`0C4a<07alO01G
EeL0<c<c09jNWP1RHV80ADE506MWI`0f=SH0C4a<0;Rh^01WIfL0E5AD071`L01WIfL0EeMG09jNWP2Y
ZJT0]kNg06][J`19BDT0RHV907]kN`24Q8@0K6a/03@d=01DE5@0QXJ6079bLP1dM7@0EeMG05UIF@10
@400=SHf075aL@1FEUH0GemO06e]K@0n?Sh0M7Ad0<?3``1TI6@0VIVI075aL@1MGEd0De=C06IVIP1/
K6`0U9BD05=CD`0h>3P0EeMG06QXJ00a<C40?clo08b<S01EEED0BTY:06ATI01aLG40DE5A04a<C01k
Ng/0A4A405aLG01iNGT0>c/k02HV9P005`1`L700@41002`/;02=SHd0ADE503Hf=P1/K6`0?3`l09ZJ
VP25QHD0L71`08V9R@2WYjL0EeMG04e=C@2EUID0Vi^K08:2PP0g=cL0I6AT07alO00f=SH0LG5a00ko
oooo00`0YZJV08R8R01WIfL0?Shn03<c<`1GEeL0GemO03Ti>@0R8R807alO028R8P19BDT20:NWY`0;
0::RXP1eMGD0:b/[03hn?P35aLD0h>3P0=7Ad@38b<P0_[jn0:f][@2SXj<00P2FUYH0802KVi/0Shn?
0;6a/@1PH600D51@061PH01RHV80Kfm_08J6QP34a<@0G5aL04];B`1<C4`0O7al09>CT`1]KFd0SHf=
04E5A@1mOGd0W9bL038b<P0/;2`0B4Q80;Zj^P2k^k/0c<c<05QHF021PH40YJFU06ATI00Z:RX07AdM
0P0J6QX0@`0j>SX09RHV030`<02:RXX0NWYj06m_K`33`l<0NWYj04e=C@1<C4`0DU9B075aL@0e=CD0
8b<S04e=C@1]KFd0K6a/04Y:BP1CDe<0F5QH065QH@1UIFD0TI6A06MWI`1lO7`0J6QX05UIF@24Q8@0
Xj>S05YJFP14A4@0@d=3055AD@2IVIT0`l?30;Jf]P2i^KT0_[jn0;^k^`1SHf<0Jf][03lo?`17AdL0
ATI601hN7P0K6a/05aLG01PH600N7Qh0:BTY065QH@2NWYh0Ng]k07UiN@1KFe/0[jn_0;jn_P38b<P0
b/[:06YZJP0];Bd07QhN020P8015ADD0GUiN09FEU@1VIVH00P0_;bl0Q`0/;2`0;Rh^0410@01oOgl0
PH6106=SH`0l?3`0MGEe07inOP1<C4`0?Shn04E5A@1HF5P06ATI01dM7@0^;Rh0B4Q805EEE@2TY:@0
V9RH08N7Q`2TY:@0WIfM02l_;`0J6QX061PH01TI6@0G5aL07QhN034a<@12@T80:BTY03Xj>P1<C4`0
=CDe02TY:@0^;Rh0@41009RHV02:RXX0?Cdm069RHP1[Jf/0=CDe05IFEP1UIFD0U9BD0:n_[`3@d=00
=CDe01lO7`0L71`08b<S03Lg=`1dM7@0clo?0=oOg`3EeMD0c/k>04];B`0J6QX05aLG01DE5@0G5aL0
61PH02/[:`1OGel0g=cL0=WIf@28R8P0;bl_028R8P0`<300EeMG02LW9`1FEUH0CTi>0451@@0l?3`0
BTY:04I6AP1YJFT0WinO07AdM01DE5@0ATI60;fm_@2n_[h0DE5A03Lg=`0X:2P0AdM704E5A@24Q8@0
OWin0<[:bP1cLg<08b<S02PX:00R8R807alO01`L700N7Qh092@T059BDP1/K6`0d=3@0<o?c`1eMGD0
=cLg059BDP1mOGd0<30`02XZ:P0_;bl0<C4a03De=@0l?3`0EEEE05mOG`1dM7@0SHf=0;2`/02TY:@0
[jn_0<_;b`3Fe]H0f=SH0=?Cd`2HV9P0PH6107IfMP0n?Sh061PH01XJ6P0201/K6`0;01dM7@1/K6`0
GemO04M7A`13@d<0CDe=08b<S032`/80QHF5069RHP1<C4`04?ooool000kooooo01`0J6QX05IFEP2:
RXX0IFEU06MWI`0o?cl0[Zj^08N7Q`0n?Sh0Bd];038b<P13@d<0AdM706=SH`1EEED09bLW04A4A00O
7al0FEUI03Hf=P0F5QH0>3Ph02PX:00h>3P0>c/k020P8011@D409RHV0P1EEED0400k>c/0Jf][05AD
E00d=3@0CDe=04i>CP0j>SX09bLW059BDP0Y:BT0LG5a07alO00j>SX0B4Q803De=@1ADE4204E5A@3V
0861P@1XJ6P0<c<c02<S8`1]KFd0B4Q802/[:`1=CDd0@41004i>CP1GEeL0De=C03hn?P1]KFd0BTY:
02DU9@1jNWX0=cLg01hN7P1NGUh0EeMG03De=@0^;Rh0?Shn075aL@0f=SH0B4Q806EUI@12@T80@d=3
05EEE@1ZJVX0BTY:0;2`/01KFe/0MgMg06=SH`1JFUX0ADE509fMW@1CDe<0>3Ph034a<@1]KFd0Fe]K
08J6QP2MWId0K6a/05ADE023Ph<0HV9R07QhN01:BTX0UIFE05=CD`1HF5P0EeMG03Ti>@1HF5P0P820
04Y:BP1TI6@0AdM708f=S@1PH600IFEU075aL@18B4P0DE5A05UIF@1:BTX0CTi>04e=C@1SHf<0Jf][
071`L01?Cdl0J6QX0861P@1JFUX0Jf][04i>CP2MWId0G5aL04U9B@1@D500UYJF03Hf=P1[Jf/0RXZ:
0451@@16ATH0OWin07emO@1cLg<0PX:204m?C`2f][H0TY:B04];B`22PX80FUYJ08V9R@2GUiL0Hf=S
09nOW`1kNg/0DU9B096AT@2@T900L71`0;Ng]`3;bl/0LG5a05aLG02LW9`0Vi^K03lo?`1?Cdl0ZZZZ
05IFEP2HV9P0Z:RX04m?C`1JFUX0RXZ:05QHF01PH600>CTi02h^;P1EEED0?Shn08>3P`2YZJT0Kfm_
06i^KP0^;Rh0E5AD06i^KP1BDU80O7al04Y:BP0f=SH0WYjN07IfMP0j>SX0Rh^;03`l?01=CDd0];Bd
05UIF@0o?cl0M7Ad07IfMP0o?cl0BDU9061PH02=SHd0Bd];059BDP1mOGd0F5QH08F5Q@1>CTh0?clo
06i^KP21PH40F5QH07UiN@1;Bd/0A4A40492@P1CDe<0IFEU04A4A01ZJVX0J6QX02l_;`1cLg<0`/;2
05QHF01^KVh0H61P03`l?01PH600JVYZ04i>CP1@D500CDe=02XZ:P1QHF40IVIV02h^;P0X:2P0Jf][
0451@@0a<C40H61P08F5Q@29RHT0JFUY0820P00j>SX0Cdm?06m_K`1IFET0@d=309:BTP0l?3`09BDU
06YZJP1LG5`0?Cdm06YZJP0g=cL0IVIV05iNGP1WIfL0NGUi09RHV01nOWh0:RXZ04m?C`1QHF40MGEe
03dm?@1TI6@0RHV904Q8B01QHF4>ooooo`1?059BDP1UIFD0JFUY07=cL`1/K6`0HF5Q055AD@1@D500
MWIf065QH@1WIfL0I6AT05mOG`1_Kfl0Shn?06=SH`0K6a/0ADE50;Fe]@2KVi/0YjNW0;fm_@26QXH0
@d=302LW9`0X:2P0:b/[03Ph>01ADE40I6AT09JFUP1LG5`0BDU903`l?00g=cL0>SXj0820P02<S8`0
@T9205eMG@1aLG40G5aL08R8R01LG5`06a/K03`l?02d];@0JVYZ05EEE@1@D500HV9R05YJFP1XJ6P0
HV9R04U9B@0l?3`0BTY:05iNGP1/K6`0HF5Q05MGE`1kNg/0KVi^065QH@23Ph<0][Jf0<O7a`3<c<`0
Xj>S0820P011@D40:RXZ02`/;01FEUH0QHF5071`L01=CDd0:b/[028R8P02020P801:024Q8@0b<S80
>3Ph04I6AP1@D500IfMW07IfMP2SXj<0ZJVY0;2`/02m_Kd0ZZZZ079bLP1GEeL0PX:2065QH@1EEED0
T92@0<O7a`2/[:`0_;bl04A4A00k>c/0@41004m?C`0l?3`0<S8b02/[:`1=CDd0MGEe06e]K@1JFUX0
F5QH02HV9P0U9BD0N7Qh0<c<c02NWYh0aLG509FEU@1QHF40;bl_05YJFP14A4@0?Shn09:BTP1QHF40
BTY:03`l?00a<C40;bl_03`l?01UIFD0U9BD096AT@18B4P0>SXj03/k>`1@D500MWIf06MWI`2ATI40
KFe]04=3@`0i>CT0;2`/024Q8@0i>CT0OWin0:n_[`3Bd]80/K6a05MGE`0O7al201LG5`1i01HF5P0G
5aL061PH02HV9P15ADD0@41004Y:BP0n?Sh0:2PX02XZ:P0k>c/0QHF502LW9`0K6a/071`L02/[:`1=
CDd0LG5a09bLW01nOWh0a/K60=gMg@1XJ6P0FUYJ05]KF`1_Kfl0UIFE0:j^[P2h^;P0/[:b0;no_`2m
_Kd0]KFe075aL@0Y:BT0820P02h^;P14A4@0L71`0;Ng]`3De=@0d];B09>CT`0a<C408b<S02d];@1g
MgL0FUYJ0410@00g=cL0<S8b034a<@0d=3@0BTY:0:2PX01UIFD0@d=30410@014A4@0WYjN0=3@d02<
S8`0Kfm_06YZJP1MGEd0<c<c07YjNP1/K6`0Rh^;06IVIP0Y:BT07AdM01PH600I6AT061PH01LG5`0E
5AD08b<S0=SHf02d];@0]KFe09NGU`1/K6`0PH6107alO016ATH0@T9204A4A01@D500LW9b08Z:RP1e
MGD0L71`07UiN@28R8P0Ogmo07]kN`1]KFd0L71`0=GEe@35aLD0d=3@0<C4a00a<C40Cdm?06IVIP0W
9bL06a/K020P801<C4`0KFe]05]KF`17AdL0@d=304U9B@1gMgL0a/K609ZJVP2WYjL0QHF506a/K00@
ooooo`003_ooool0>P1bLW80D51@08f=S@1]KFd0Rh^;03lo?`1oOgl0Ng]k03Hf=P1UIFD0A4A403`l
?013@d<0KFe]05QHF00_;bl0ATI602LW9`0j>SX0BDU9020P800_;bl08R8R03<c<`0j>SX07QhN04a<
C00I6AT0BDU905]KF`13@d<0TY:B03Hf=P0U9BD0CDe=05ADE00a<C405aLG0451@@0U9BD0FUYJ07Ad
M00];Bd0>3Ph055AD@1EEED0Bd];03dm?@1IFET0GEeM04E5A@0W9bL0MgMg04I6AP0F5QH0;Rh^04A4
A01@D50204m?C`2_02h^;P1ADE40GEeM020P801RHV80E5AD01XJ6P1ADE40G5aL03<c<`0^;Rh0<S8b
06a/K00n?Sh0@d=306IVIP1HF5P0ADE505EEE@1lO7`0BDU90<30`01mOGd0O7al0492@P1/K6`0>SXj
08N7Q`2?Shl0Cdm?038b<P1YJFT0HF5Q06YZJP2PX:00OGem04];B`2<S8`0FEUI08B4Q01DE5@0/K6a
05YJFP1CDe<0Lg=c0451@@1MGEd0RHV904I6AP1^KVh0A4A409FEU@1IFET0GUiN08J6QP1@D500AdM7
04E5A@1ADE40HV9R04M7A`1HF5P0JVYZ096AT@1MGEd0GemO08F5Q@11@D40LW9b065QH@2>SXh0OWin
038b<P15ADD0WinO069RHP1XJ6P0UYJF055AD@0f=SH0IFEU07UiN@1RHV80Kfm_04I6AP28R8P0UYJF
0451@@1[Jf/0=3@d079bLP1nOWh0EeMG08f=S@1eMGD0D51@08F5Q@2<S8`0LG5a0:n_[`3:b/X0WYjN
07YjNP2NWYh0Ti>C04=3@`1eMGD0`l?305EEE@0[:b/0Rh^;05MGE`2;Rh/0^;Rh07moO`1FEUH0CTi>
04U9B@1oOgl0?3`l06][J`2[Zj/0Ogmo09fMW@1VIVH0DE5A0861P@16ATH0P820075aL@0i>CT0O7al
07AdM00U9BD0L71`04Y:BP0l?3`0^k^k08n?S`1SHf<0HV9R075aL@1[Jf/0ATI606i^KP2@T900H61P
03hn?P2NWYh0GemO09BDU01TI6@0>SXj04i>CP22PX80GEeM079bLP1CDe<0E5AD02h^;P0T92@0DU9B
03dm?@1MGEd0FUYJ04a<C01JFUX0^;Rh05iNGP1IFET0NWYj03Ti>@10@400F5QH05EEE@02051@D00Y
03<c<`19BDT0H61P02h^;P0Q8B40Hf=S04];B`11@D40NGUi071`L020P800GEeM09JFUP1HF5P0FEUI
07inOP1OGel0<S8b09FEU@1SHf<0>SXj06MWI`0m?Cd0<C4a07UiN@13@d<0I6AT05=CD`17AdL08R8R
04a<C01`L700>SXj05eMG@1NGUh0B4Q801`L701]KFd0P82005MGE`1QHF403_ooool06P0k>c/0<C4a
02LW9`0X:2P0;Bd]04Q8B01RHV80U9BD0<;2`P2_[jl0;2`/01PH600G5aL05QHF01PH601>CTh0Hf=S
08:2PP1kNg/0CDe=02h^;P13@d<0TI6A06UYJ@1ADE40;Bd]0P0M7Ad05P0c<c<0Bd];059BDP25QHD0
@D5102TY:@0V9RH07alO04Y:BP30`<00Bd];03De=@17AdL0=CDe04=3@`1EEED071`L03/k>`27QhL0
9BDU04A4A01VIVH203De=@0b02`/;00c<c<0=CDe04a<C00i>CT0?Shn04Q8B015ADD0>SXj0492@P1Z
JVX0Bd];04A4A012@T80V9RH0;^k^`2JVYX0FEUI01dM7@0N7Qh07AdM02HV9P28R8P0WIfM03dm?@0O
7al07AdM02DU9@0c<c<0@T9203hn?P13@d<0CTi>02LW9`0[:b/0GemO0;Ng]`2i^KT0XJ6Q0;fm_@2T
Y:@0>CTi024Q8@1?Cdl0ADE503De=@1jNWX0b/[:0<c<c02b/[8207emO@0e0861P@2>SXh0M7Ad08B4
Q01iNGT0OWin07]kN`1RHV80<30`034a<@0L71`0@d=30;>c/`35aLD0];Bd0<c<c02OWil0UiNG08>3
P`25QHD0GEeM06QXJ01;Bd/0FEUI05aLG01:BTX0?clo05iNGP1mOGd0FUYJ06=SH`33`l<0Xj>S09^K
V`2?Shl0O7al0820P035aLD0^k^k03Xj>P0I6AT05QHF01PH600M7Ad071`L05iNGP2QXJ40_kno096A
T@2XZ:P0Ng]k024Q8@0E5AD00P0G5aL0300F5QH05ADE01`L701=CDd0?clo04i>CP11@D40;Bd]04=3
@`1YJFT06QXJ01TI6@8071`L02807AdM05ADE01gMgL0_[jn0=SHf02n_[h0NWYj06EUI@1ADE40@d=3
05]KF`1OGel0];Bd08n?S`16ATH0MgMg08:2PP3Cdm<0`L710:>SX`33`l<0VYZJ04a<C01NGUh0^;Rh
0::RXP0j>SX0HF5Q04M7A`1PH600G5aL02XZ:P0U9BD08b<S0P0X:2P0900a<C40HF5Q07]kN`0`<300
?Shn04m?C`1FEUH0Q8B40<71`@28R8P0V9RH0<K6aP2XZ:P0Rh^;0;^k^`39bLT0aLG509VIV@1DE5@0
6QXJ024Q8@0G5aL0820P01lO7`0M7Ad0C4a<0;6a/@1JFUX0QXJ6092@T02FUYH0WYjN0;2`/01RHV80
F5QH04e=C@80>SXj01l0?Shn0:BTY01SHf<0A4A403Xj>P0/;2`0A4A404Y:BP24Q8@0f=SH0=KFeP1d
M7@0Ogmo03`l?00S8b<0K6a/06UYJ@0f=SH0SHf=079bLP1<C4`0ADE504A4A01<C4`0L71`0<71`@1m
OGd0;bl_03Hf=P1bLW80XZ:R013ooooo000>ooooo`3o06QXJ01:BTX0NGUi05EEE@1lO7`0A4A408N7
Q`11@D408B4Q0410@00j>SX09bLW030`<00l?3`0FEUI02`/;011@D407alO061PH01EEED09bLW03Hf
=P0M7Ad0?Shn04m?C`0T92@0Cdm?02d];@0o?cl0EUIF04E5A@1iNGT0?3`l02PX:01;Bd/0GEeM034a
<@0N7Qh0ATI601dM7@1?Cdl0O7al04E5A@17AdL0IVIV0492@P1VIVH0De=C07]kN`1]KFd0=CDe028R
8P1bLW80ADE502@T9010@400DE5A04I6AP1<C4`0Hf=S0410@01SHf<0I6AT01dM7@1PH600GemO01dM
7@1<C4`0Kfm_03/k>`0^;Rh0<C4a07YjNP0l?3`0ADE507EeM@1SHf<0@T9204i>CP1TI6@0Bd];0;jn
_P1lO7`0PX:204M7A`1NGUh0=cLg08N7Q`3:b/X0FUYJ03Xj>P1SHf<0BTY:03Ti>@2ATI40SHf=04A4
A01hN7P0@d=308n?S`16ATH0QhN705IFEP1cLg<0KVi^0451@@1LG5`0KVi^03@d=01kNg/0@4100861
P@0b<S80FUYJ07inOP18B4P0?clo04M7A`0g=cL0De=C06=SH`13@d<0Hf=S07YjNP1@D500>c/k07Ui
N@17AdL0CTi>05IFEP27QhL0PX:204=3@`0_;bl0/[:b06=SH`1]KFd0VIVI075aL@1:BTX0N7Qh08R8
R01VIVH0Vi^K05]KF`26QXH0N7Qh03Lg=`1dM7@0CDe=0:FUY@24Q8@0IVIV0::RXP2KVi/0H61P08B4
Q01PH6008R8R071`L02e]KD0TY:B07IfMP2<S8`0NWYj02TY:@10@400Vi^K05EEE@15ADD0IVIV04];
B`17AdL0XZ:R07emO@1MGEd0>3Ph04e=C@1jNWX0>c/k06ATI02IVIT0BDU907=cL`1GEeL0BDU906e]
K@0[:b/0Jf][06YZJP10@400J6QX09BDU00k>c/0JVYZ05]KF`0e=CD0[Zj^08J6QP1TI6@0KFe]07em
O@2;Rh/0DE5A07emO@2HV9P0LW9b0410@02HV9P0FEUI09^KV`1mOGd0@T9206EUI@2>SXh0IfMW071`
L01HF5P0De=C03Lg=`0T92@0Fe]K04=3@`1?Cdl0E5AD0410@01YJFT0`/;205ADE01CDe<0RHV904a<
C016ATH0G5aL051@D01jNWX0IFEU03`l?01@D500KVi^02`/;00e=CD0T92@05mOG`11@D40RHV907Mg
M`1JFUX0=SHf07QhN014A4@0HV9R08R8R01TI6@09RHV000G04U9B@27QhL0GUiN06=SH`0l?3`0<C4a
04I6AP0l?3`0JVYZ05YJFP0h>3P0@D5107=cL`1_Kfl0;Rh^05MGE`1LG5`0Kfm_04A4A019BDT0V9RH
069RHP1HF5P03_ooool0300c<c<0:RXZ02@T900S8b<07AdM01`L700M7Ad0@d=308R8R024Q8@06a/K
01LG5`805ADE00d05aLG01dM7@2GUiL0Q8B404Y:BP0U9BD07AdM028R8P1ADE40J6QX079bLP1]KFd0
FEUI0080:b/[0840A4A403@d=019BDT0OWin02PX:00S8b<0820P02XZ:P2NWYh0Q8B402/[:`13@d<0
>SXj03<c<`1mOGd0ADE507]kN`2<S8`0BDU90492@P0o?cl0BTY:02`/;00U9BD0:RXZ02`/;00[:b/0
ADE502`/;00^;Rh0=CDe030`<00^;Rh0Hf=S04E5A@0e=CD0;bl_07MgM`2NWYh0N7Qh05YJFP0L71`0
6a/K01`L700d=3@0N7Qh08b<S029RHT0PX:205ADE01KFe/0DU9B03/k>`0_;bl0>3Ph05=CD`1LG5`0
;Rh^04i>CP1UIFD0S8b<07emO@1nOWh0YZJV06UYJ@0X:2P0>SXj06i^KP1ZJVX0[Zj^0=3@d03Rh^80
d=3@0:b/[02KVi/0];Bd0=7Ad@3;bl/0/[:b0<k>cP2/[:`0FUYJ03De=@0h>3P0B4Q807=cL`2BTY80
MGEe04=3@`1nOWh0/[:b0;^k^`2UYJD0Kfm_09fMW@3?cll0Vi^K01hN7P0Q8B40N7Qh0=[JfP3Fe]H0
bLW909nOW`2?Shl0Rh^;0;2`/01VIVH0H61P05]KF`1`L700WinO0<k>cP3De=@0W9bL02<S8`0F5QH0
5ADE01PH600^;Rh0LG5a0=7Ad@33`l<0Hf=S0410@01DE5@0De=C01PH600G5aL00`0E5AD02`0H61P0
@D5103Xj>P10@400>SXj02@T900W9bL0>SXj01LG5`0I6AT071`L0080820P0200D51@08N7Q`2j^[X0
ZJVY05=CD`14A4@0@T9204e=C@1?Cdl0GemO08^;R`30`<00Shn?04a<C01`L700HF5Q06=SH`3Lg=`0
jNWY0;jn_P1<C4`0AdM705EEE@2o_kl0SHf=06MWI`1[Jf/0UYJF0;:b/P1MGEd0:b/[01lO7`806a/K
04H08B4Q04=3@`24Q8@0>c/k02l_;`0Z:RX0=SHf06=SH`2`/;00[Jf]06e]K@1EEED0Shn?0<W9b@36
a/H0fMWI0=GEe@2/[:`0XJ6Q04U9B@0I6AT061PH01DE5@0G5aL05ADE01dM7@2@T900IFEU06=SH`1O
Gel0^k^k0;Bd]02GUiL0[Jf]09:BTP0X:2P07alO024Q8@0S8b<0:BTY08N7Q`0e=CD0<30`02d];@10
@400Fe]K0:VYZ@3IfMT0gmoO0<c<c01VIVH0@T9207alO00V9RH0HV9R07inOP2HV9P0G5aL0492@P1>
CTh0IFEU07]kN`0m?Cd0<S8b055AD@1?Cdl09bLW02`/;00d=3@0KVi^4?ooool000kooooo01X0MGEe
04U9B@2DU9@0GUiN09BDU01JFUX0Ng]k08j>SP1<C4`0Bd];04e=C@0^;Rh0FEUI059BDP1fMWH0@410
04m?C`0g=cL0>CTi05MGE`0S8b<0>SXj03Lg=`12@T80EUIF030`<080Cdm?02@0EUIF059BDP0o?cl0
RHV90451@@0W9bL0CDe=069RHP0m?Cd0?clo055AD@0`<300B4Q809fMW@1RHV80BDU905UIF@0o?cl0
L71`04=3@`1/K6`0G5aL02TY:@0[:b/0O7al061PH00e=CD0?3`l05YJFP17AdL0RHV908:2PP19BDT0
MGEe065QH@0Q8B42071`L02G02@T901;Bd/0EeMG0492@P1?Cdl0;bl_06m_K`0_;bl0BTY:079bLP1X
J6P0?Shn04e=C@0i>CT0:RXZ0820P01`L700Q8B404Q8B01:BTX0:BTY04=3@`2TY:@0GUiN04a<C01Y
JFT0DU9B02l_;`1mOGd0PH6104A4A025QHD0H61P0820P016ATH0Ng]k06QXJ01PH600NWYj04U9B@1h
N7P0SHf=04U9B@1TI6@0B4Q808j>SP1hN7P0HF5Q07moO`1ADE40O7al07=cL`1EEED0FUYJ03dm?@19
BDT0OGem08J6QP0g=cL0>CTi0861P@1@D500F5QH03Lg=`1gMgL0PH6105QHF00l?3`0];Bd030`<01H
F5P0Shn?075aL@11@D40M7Ad04M7A`16ATH0RHV904i>CP20P800IfMW03De=@24Q8@0E5AD08^;R`1T
I6@0=SHf07emO@1lO7`0HV9R08:2PP2>SXh0O7al08f=S@3:b/X0QhN707QhN02g]kL0WYjN06MWI`12
@T80WinO06MWI`1MGEd0XZ:R06QXJ01GEeL0ZZZZ08>3P`1KFe/0FEUI02h^;P1^KVh0>3Ph055AD@2e
]KD0MWIf06m_K`18B4P0AdM706UYJ@19BDT0NGUi06ATI00h>3P0Jf][096AT@0c<c<0JFUY05IFEP1D
E5@0]KFe0861P@1DE5@0GemO06YZJP0Z:RX0GEeM096AT@2][Jd0Ti>C059BDP29RHT0FUYJ0:b/[02:
RXX0@d=306IVIP24Q8@0@d=306MWI`0206QXJ00i03Ph>00P8200EEEE0410@01kNg/0IfMW0492@P1^
KVh0`<3005]KF`27QhL0Shn?04];B`17AdL0Hf=S059BDP18B4P0HV9R038b<P1=CDd0LW9b01lO7`0I
6AT0De=C069RHP19BDT0Q8B407MgM`2ATI40DE5A08:2PP13@d<0@T9207UiN@10@4009RHV08>3P`1C
De<0Kfm_08n?S`15ADD0AdM706IVIP1:BTX0LG5a05UIF@1=CDd0:RXZ04U9B@1@D500:RXZ05mOG`1d
M7@0MWIf04E5A@19BDT0ZJVY0080G5aL3_ooool202`/;00h02HV9P0S8b<07AdM01`L700H61P05QHF
04e=C@26QXH06a/K01DE5@0G5aL0ADE504e=C@0o?cl0O7al06EUI@0Y:BT06a/K01`L700N7Qh07AdM
05EEE@1[Jf/0DU9B04e=C@20P800@d=3028R8P0g=cL0?Shn05YJFP1?Cdl092@T028R8P0i>CT0T92@
0<G5a@1mOGd0MGEe07=cL`1XJ6P0SXj>06ATI02j^[X0WinO03`l?00Y:BT0820P02@T900l?3`0ADE5
02/[:`0V9RH0:b/[02d];@13@d<201`L700401/K6`0R8R80Lg=c02/[:`809bLW00D0GEeM08^;R`1C
De<0GUiN01/K6`0201TI6@1Z05mOG`1QHF40MGEe05]KF`0j>SX0:RXZ02LW9`0V9RH07QhN024Q8@0Y
:BT09BDU03`l?020P800R8R805=CD`1hN7P0Rh^;09NGU`1dM7@0ZZZZ04U9B@16ATH0Zj^[0<;2`P30
`<00YJFU08J6QP2/[:`0TI6A09ZJVP2j^[X0aLG50=WIf@3Zj^X0fm_K0<o?c`1WIfL0;Rh^02LW9`1<
C4`0Z:RX07inOP0h>3P0:b/[06a/K02GUiL0VIVI0820P029RHT0QXJ609RHV02ATI40:b/[04=3@`24
Q8@0R8R808Z:RP2_[jl0YjNW07YjNP1kNg/0YjNW0820P02;Rh/0MGEe08b<S035aLD0bLW906UYJ@1n
OWh0V9RH03hn?P0N7Qh0=cLg05aLG02DU9@0f=SH0=CDe026QXH0De=C03Lg=`10@400<c<c02TY:@0F
5QH04a<C01DE5@0H61P0<c<c0451@@1KFe/0>CTi01lO7`0];Bd0>SXj01TI6@0L71`07QhN024Q8@0Z
:RX0AdM7096AT@3@d=00GEeM02@T90808b<S06408R8R01lO7`0T92@0>CTi05=CD`2BTY80@T92051@
D02IVIT0Rh^;0<K6aP3Rh^80PH6103hn?P11@D40OWin0=3@d02[Zj/0PX:20;Fe]@2OWil0/[:b0:^[
Z`1iNGT0Bd];04I6AP1@D500IfMW0;fm_@3>c/h0=3@d02@T900Q8B40>SXj06a/K02YZJT0_Kfm07Mg
M`13@d<0XJ6Q04U9B@0m?Cd0OWin09NGU`1DE5@0EeMG05aLG00Z:RX0:2PX01TI6@0H61P06ATI04e=
C@2:RXX0HF5Q07UiN@1GEeL0[Zj^0;fm_@2_[jl0`/;20;^k^`2NWYh0TY:B09>CT`2LW9`0[:b/0;Rh
^02@T900U9BD0:JVYP2o_kl0/k>c0:n_[`2/[:`0Zj^[09>CT`2QXJ40@D5108^;R`24Q8@0VIVI08F5
Q@1IFET0AdM706IVIP2LW9`0]kNg04I6AP0S8b<07AdM028R8P10@4009RHV02h^;P0b<S80BTY:013o
oooo000>ooooo`3o06i^KP19BDT0TI6A069RHP1iNGT0EUIF09RHV02@T900EEEE03hn?P0e=CD0A4A4
06EUI@17AdL0IfMW02h^;P1PH600E5AD06EUI@1JFUX030`<03Lg=`1CDe<0FUYJ03/k>`0=3@d0?Cdm
055AD@19BDT0?3`l020P8022PX80HF5Q02XZ:P16ATH0HF5Q02<S8`0m?Cd0C4a<010@400S8b<0H61P
06MWI`0o?cl08b<S02XZ:P1NGUh0=3@d075aL@1YJFT0:2PX01`L7015ADD0O7al03dm?@1GEeL0EEEE
028R8P10@400GEeM03`l?01SHf<0IVIV01LG5`1DE5@0JVYZ02PX:00o?cl0FUYJ03Ti>@1HF5P0@410
075aL@1DE5@0>CTi05iNGP1`L700GEeM08N7Q`2IVIT0C4a<0;Vi^@1^KVh0SHf=08b<S02:RXX0@d=3
03dm?@3AdM40B4Q803/k>`1DE5@0MGEe04I6AP2a/K40T92@03De=@1iNGT0F5QH08N7Q`0_;bl0DU9B
06][J`1KFe/0>CTi02h^;P1>CTh0Kfm_04e=C@13@d<0:BTY05]KF`1XJ6P0CTi>06YZJP0a<C40@T92
05]KF`1<C4`0GUiN03lo?`15ADD0LG5a07alO01PH600AdM708j>SP1:BTX0DU9B05MGE`1aLG40Ogmo
0492@P0a<C40SXj>05IFEP1IFET0MWIf05eMG@0h>3P0De=C07YjNP1XJ6P0X:2P06MWI`1_Kfl0S8b<
04Q8B01kNg/0Fe]K07alO02?Shl0PX:208N7Q`1mOGd0Bd];06e]K@27QhL0Kfm_08B4Q02m_Kd0RHV9
04E5A@1`L700QHF504i>CP0l?3`0S8b<04U9B@1hN7P0X:2P06a/K01FEUH0Xj>S04i>CP1QHF40GUiN
02LW9`1MGEd0Bd];06e]K@2g]kL0Ng]k06UYJ@13@d<0GemO07EeM@1HF5P0M7Ad04a<C00/;2`0NGUi
08f=S@0f=SH0J6QX03dm?@0f=SH0KVi^07emO@1VIVH0G5aL06QXJ00n?Sh0<c<c04a<C01WIfL0S8b<
03dm?@1XJ6P0DU9B08^;R`27QhL0?Shn06IVIP1kNg/0EEEE04U9B@21PH40MgMg03De=@0M7Ad0CDe=
02TY:@14A4@0Fe]K05=CD`1MGEd0^k^k05eMG@1DE5@0M7Ad04a<C01;Bd/0FUYJ06a/K01FEUH0EeMG
02h^;P1;Bd/0K6a/03De=@0L71`0EeMG07=cL`1OGel0QHF508f=S@2XZ:P0D51@08^;R`0m?Cd0IVIV
09BDU01PH600:BTY000G06=SH`0m?Cd0De=C0:6QX@27QhL0HF5Q0820P00e=CD0H61P08F5Q@1ZJVX0
@T9205mOG`1FEUH05aLG0451@@1YJFT0K6a/0410@013@d<0Z:RX01TI6@1DE5@03_ooool0<@0c<c<0
:BTY02DU9@0Q8B407QhN01XJ6P0I6AT071`L024Q8@1UIFD0H61P01LG5`0E5AD0;bl_05mOG`14A4@0
SHf=07AdM00/;2`09BDU01hN7P0P82007alO059BDP0i>CT09RHV02/[:`16ATH0D51@01/K6`0M7Ad0
J6QX092@T02n_[h0Q8B406][J`1RHV80H61P0;>c/`2DU9@0WYjN0:6QX@2][Jd0ZJVY09VIV@37alL0
b<S804m?C`0O7al00`0J6QX07P0P8200@T9204Y:BP0];Bd09bLW03Hf=P1FEUH0ATI604A4A01RHV80
^[Zj02d];@0M7Ad07alO05QHF02GUiL0Bd];05]KF`0O7al05aLG02HV9P1eMGD0CDe=06i^KP0[:b/0
8B4Q02@T900P82008R8R01lO7`807AdM05h06a/K020P801KFe/0QXJ60:VYZ@1VIVH0XZ:R0;bl_02b
/[80VYZJ0:JVYP2`/;00^[Zj09FEU@1eMGD0BDU903Hf=P0n?Sh0N7Qh03Ph>011@D40GemO08j>SP2A
TI40f=SH09RHV02NWYh0MGEe0:NWY`2OWil0OGem03dm?@0M7Ad0;Bd]096AT@2/[:`0C4a<02d];@1g
MgL0V9RH09bLW032`/80Xj>S07]kN`1nOWh0UIFE0;fm_@36a/H0[Jf]06ATI01>CTh0Z:RX04U9B@11
@D40ATI607=cL`38b<P0S8b<05iNGP1EEED0Lg=c0<G5a@2VYZH0JFUY08V9R@1VIVH0I6AT0:VYZ@2]
[Jd0FEUI05iNGP0d=3@0=CDe02`/;00R8R805aLG01PH600G5aL09BDU07EeM@2:RXX0>c/k02/[:`0_
;bl0B4Q801hN7P0L71`07AdM020P800d=3@0PH6107YjNP2m_Kd0;bl_0P0M7Ad0F`0O7al07AdM01lO
7`0O7al08B4Q03hn?P2ZZZX0Q8B4055AD@1dM7@0^[Zj0=;BdP3Jf]X0I6AT05eMG@1dM7@0>CTi055A
D@3Fe]H0b/[:0;fm_@2[Zj/0WYjN0:f][@2CTi<0G5aL04Q8B013@d<0ATI608n?S`1oOgl0Hf=S024Q
8@0Z:RX0?Cdm05MGE`1[Jf/0d=3@0:FUY@2?Shl0EeMG02DU9@0N7Qh0:RXZ07moO`1@D500DE5A07If
MP0Y:BT0<30`02d];@0M7Ad0IVIV08j>SP1LG5`0Fe]K0;Bd]02BTY80Bd];05aLG02m_Kd0Ti>C059B
DP13@d<0BDU906QXJ01VIVH0E5AD0::RXP29RHT0P82006i^KP1LG5`0H61P0492@P1ZJVX0IFEU069R
HP1cLg<0Jf][06EUI@2g]kL0L71`06MWI`1BDU80M7Ad0;jn_P2OWil0EEEE02HV9P0P82000P0M7Ad0
1@0/;2`0:RXZ02`/;00b<S80IVIV013ooooo000>ooooo`1U06a/K012@T80GEeM06a/K01SHf<0;Rh^
07IfMP1ZJVX0>3Ph03hn?P0];Bd0;Rh^051@D014A4@0Hf=S02d];@0k>c/0:RXZ05]KF`1JFUX061PH
02TY:@0];Bd0Bd];06a/K00f=SH0=CDe03lo?`0j>SX0GEeM02LW9`2^[Zh0JFUY02HV9P1GEeL0K6a/
04I6AP0o?cl0E5AD04];B`10@400JVYZ04M7A`0n?Sh0GUiN03hn?P1UIFD06QXJ03hn?P1ZJVX0HV9R
03/k>`1JFUX0SHf=01LG5`0[:b/0GUiN03/k>`1;Bd/0Cdm?02DU9@0n?Sh0NWYj02LW9`1CDe<0HV9R
02/[:`1RHV80P8200492@P1LG5`0?3`l07AdM01]KFd0Bd];05UIF@1DE5@0<S8b04i>CP1kNg/0ADE5
0:j^[P1PH600Bd];04A4A01UIFD0B4Q803De=@3<c<`0JFUY03/k>`10@400KFe]04U9B@2MWId0XJ6Q
04M7A`1nOWh0JFUY08B4Q01JFUX00P1mOGd0[`1?Cdl0Ng]k04];B`1DE5@0NWYj05MGE`1CDe<0@410
06=SH`1`L700BDU907moO`1=CDd0BDU905=CD`16ATH0Kfm_04e=C@0m?Cd0MWIf08F5Q@1aLG40Cdm?
092@T019BDT0J6QX06IVIP1SHf<0SXj>034a<@0^;Rh0RHV9069RHP1ZJVX0NGUi05aLG01<C4`0LG5a
08n?S`1`L700QXJ605aLG02LW9`0]KFe051@D01gMgL0<c<c07QhN027QhL0MWIf08J6QP1jNWX0C4a<
069RHP24Q8@0P82006e]K@30`<00VIVI06][J`2PX:00Z:RX06m_K`0c<c<0LG5a07QhN01JFUX0YjNW
06i^KP1EEED0X:2P07=cL`0i>CT0EUIF028R8P1ADE40=cLg03dm?@2RXZ80MGEe06][J`1DE5@0;bl_
05EEE@1>CTh0OWin06EUI@14A4@0?Cdm09:BTP0g=cL0H61P07MgM`0T92@0FUYJ08R8R01bLW80G5aL
07UiN@1TI6@0C4a<07AdM01_Kfl0SHf=03De=@1jNWX0BTY:07alO027QhL0?Shn05eMG@1gMgL0FUYJ
04Y:BP1iNGT0NWYj03/k>`0K6a/0GUiN051@D01ADE40GUiN06i^KP0a<C40TI6A05mOG`1[Jf/0P820
04];B`13@d<0K6a/07QhN01KFe/0IfMW0451@@1GEeL0MGEe03`l?00P8200E5AD06YZJP0h>3P0J6QX
09FEU@2VYZH0C4a<08n?S`0M7Ad0<30`04Y:BP1HF5P0@D5106EUI@0U9BD08B4Q03lo?`1aLG40MgMg
08R8R00c<c<0AdM705mOG`1GEeL0@D5107alO01bLW8092@T03dm?@1>CTh0LG5a03`l?01/K6`0^[Zj
0492@P0l?3`03_ooool0<P0l?3`0;bl_02PX:00R8R8071`L01TI6@0G5aL05QHF01DE5@18B4P0TY:B
06EUI@0V9RH0=SHf05eMG@1]KFd0[Jf]0:j^[P1oOgl0Jf][05EEE@1?Cdl0CTi>06MWI`1^KVh0D51@
02@T900S8b<0@41002XZ:P0J6QX0>c/k07]kN`3>c/h0V9RH096AT@2n_[h0eMGE0=cLg03Kfm/0eMGE
0=OGe`3De=@0a<C409^KV`1CDe<0Ti>C07alO00Z:RX05aLG0`0E5AD06@0M7Ad0Cdm?07YjNP1?Cdl0
HV9R08B4Q02PX:00/K6a0<G5a@3<c<`0YJFU0492@P0J6QX0HV9R0;no_`1?Cdl0Fe]K028R8P0G5aL0
D51@06ATI019BDT0GemO028R8P0M7Ad00`0L71`0HP0M7Ad071`L01`L700K6a/0820P055AD@1<C4`0
O7al08j>SP19BDT0?3`l07MgM`2HV9P0alO70<;2`P24Q8@0CTi>04];B`10@400=CDe03@d=01IFET0
NGUi03@d=00h>3P0VIVI071`L02n_[h0]kNg0:BTY02a/K40W9bL04M7A`0^;Rh0<c<c05=CD`1WIfL0
LW9b0;bl_01JFUX08B4Q05YJFP2`/;00[Zj^0<[:bP3Cdm<0Y:BT08:2PP1UIFD0SXj>07EeM@1[Jf/0
CTi>04];B`1NGUh0Bd];05=CD`1SHf<0SHf=0=7Ad@1WIfL0>CTi03`l?02<S8`0`L710>3Ph038b<P0
CTi>03`l?013@d<0ATI6092@T01]KFd0AdM7059BDP0Z:RX0:2PX02d];@0Z:RX06QXJ01LG5`0i>CT0
bl_;0=GEe@11@D408b<S02PX:00j>SX0820P01dM7@0P82009bLW04a<C02e]KD0Lg=c08n?S`0O7al0
71`L0P0M7Ad0FP0O7al07QhN01dM7@0M7Ad0@D5108R8R01:BTX0Bd];06ATI02ZZZX0d];B0=_Kf`2Z
ZZX0J6QX02@T900M7Ad0Jf][08b<S021PH40[Zj^0<O7a`2g]kL0QhN70;Rh^020P800@d=302HV9P0U
9BD0F5QH06YZJP1aLG40AdM7038b<P0o?cl0FUYJ09^KV`2h^;P0XZ:R02d];@0R8R806a/K01LG5`0J
6QX0EEEE07AdM017AdL0CDe=034a<@0`<300OGem09RHV02LW9`0Fe]K06EUI@2IVIT0aLG507=cL`0m
?Cd0KFe]0:f][@2UYJD0DU9B04=3@`0n?Sh09BDU02/[:`1LG5`0V9RH04i>CP0o?cl0<c<c02HV9P0R
8R807alO038b<P1NGUh0SHf=04];B`21PH40BTY:09jNWP23Ph<0MWIf05YJFP1ZJVX0E5AD03De=@0d
=3@0;bl_020P80807AdM00D08R8R02/[:`0];Bd0>3Ph09nOW`0@ooooo`003_ooool0201fMWH0ATI6
079bLP1<C4`0Ogmo06YZJP36a/H0Jf][0P0d=3@0C014A4@0;Bd]03lo?`0a<C40IfMW04i>CP0m?Cd0
:RXZ02h^;P1LG5`0;2`/03@d=00b<S80>SXj04];B`0k>c/0:BTY03`l?00^;Rh0Cdm?02@T902RXZ80
SHf=01lO7`0Z:RX0@T9203Hf=P0W9bL0>3Ph04m?C`0o?cl0R8R8051@D00_;bl0Fe]K03<c<`1/K6`0
?Cdm05EEE@1UIFD0ATI602l_;`1@D500QXJ603<c<`0^;Rh0EEEE03lo?`1IFET0GUiN03lo?`0W9bL0
J6QX02DU9@14A4@0NWYj03De=@0d=3@0QXJ603Xj>P1HF5P0=cLg0820P01SHf<0AdM706QXJ01YJFT0
>CTi05EEE@1?Cdl0;Rh^07alO022PX80IfMW04e=C@1CDe<203Lg=`2n0:JVYP1lO7`0ATI603hn?P1c
Lg<0>c/k07QhN02IVIT0CDe=07AdM018B4P0NWYj05eMG@1dM7@0JVYZ06ATI01kNg/0D51@055AD@1V
IVH0B4Q805=CD`0m?Cd0FUYJ05MGE`18B4P0MgMg04E5A@16ATH0CDe=03dm?@1;Bd/0I6AT04Q8B01b
LW80H61P08:2PP1<C4`0Ogmo065QH@19BDT0Fe]K04m?C`2^[Zh0D51@038b<P1EEED0HV9R04M7A`2=
SHd0GUiN038b<P0l?3`0OWin06m_K`1cLg<0DE5A07YjNP22PX80=3@d075aL@1;Bd/0Fe]K07=cL`1T
I6@0LW9b0:6QX@1`L700HF5Q08J6QP0k>c/0CTi>096AT@2FUYH0HV9R0;^k^`2LW9`0AdM702/[:`1M
GEd0Ogmo05aLG02OWil0HV9R04A4A02HV9P0Q8B404];B`1@D500<30`06QXJ01VIVH0?Cdm0;:b/P1W
IfL0PX:206a/K00g=cL0H61P04A4A01aLG40Hf=S05eMG@0m?Cd0Shn?030`<01UIFD0KFe]028R8P1M
GEd0P82003`l?01;Bd/0Q8B406a/K013@d<0QHF506=SH`2OWil0<S8b07AdM013@d<0OGem08:2PP16
ATH0MGEe09NGU`1HF5P0FUYJ05UIF@1=CDd0A4A403/k>`1QHF40CTi>04a<C01:BTX0?clo030`<026
QXH0J6QX07YjNP1dM7@0:b/[051@D01RHV80GUiN06YZJP1YJFT0:RXZ03Ti>@1HF5P0CDe=02LW9`1U
IFD0Kfm_04I6AP1VIVH0DE5A07IfMP13@d<0NGUi05aLG01;Bd/0U9BD07YjNP0c<c<0MgMg04U9B@10
@400G5aL05=CD`1=CDd0KVi^03/k>`0l?3`0QXJ607moO`1QHF40KVi^06QXJ00C4a<0;Rh^051@D01^
KVh0@T9203hn?P2OWil0:BTY034a<@kooooo01<0?clo03De=@0X:2P08R8R01`L700K6a/061PH01HF
5P0E5AD0DU9B04m?C`2CTi<0/[:b09FEU@29RHT0R8R80:j^[P1OGel0Bd];0080@T9200T0@41003Ti
>@0c<c<0<30`03@d=012@T80C4a<05ADE01?Cdl00P0f=SH08P1?Cdl0LW9b02`/;00`<300BTY:08n?
S`3AdM40g]kN0<o?c`21PH40SXj>09fMW@0e=CD0820P01hN7P1?Cdl0QHF505iNGP1>CTh0EUIF05EE
E@1IFET0BDU903hn?P1LG5`0Q8B40:6QX@21PH40FEUI0451@@0o?cl0MWIf0:^[Z`2^[Zh20;^k^`09
09bLW01nOWh0S8b<071`L02@T900EEEE04i>CP0W9bL07QhN008071`L04L061PH01XJ6P0K6a/07AdM
01/K6`0H61P0;Rh^04e=C@1DE5@0Jf][07=cL`1nOWh0Y:BT09:BTP2GUiL0cLg=0:JVYP2XZ:P0^;Rh
0;no_`2][Jd0Cdm?03Ti>@0j>SX0LG5a0:^[Z`2PX:00RHV906MWI`1YJFT0]KFe09^KV`2g]kL0][Jf
0:>SX`1MGEd0BDU903Lg=`0i>CT0>SXj03Ph>01EEED0IFEU04e=C@2k^k/0_Kfm0<;2`P3@d=00U9BD
05iNGP13@d<0DE5A051@D01HF5P0EEEE055AD@1fMWH0Cdm?06a/K01PH600CTi>05UIF@1RHV80EeMG
03Ph>027QhL0PH610;^k^`2BTY80<S8b020P800201PH600A01hN7P1GEeL0N7Qh05EEE@1CDe<0AdM7
03Hf=P14A4@0Jf][06ATI01CDe<0ZJVY0=[JfP2l_;`0Ng]k06YZJP1mOGd00P1OGel0201gMgL0Q8B4
0;:b/P2n_[h0^k^k075aL@0O7al06a/K0P0M7Ad0AP0N7Qh07AdM01lO7`0P8200C4a<03@d=00X:2P0
:RXZ0492@P1dM7@0Shn?0;no_`0j>SX0<c<c03De=@1HF5P0IfMW05iNGP1:BTX0GemO09bLW02e]KD0
TY:B08Z:RP2PX:00K6a/02LW9`0O7al0?clo051@D01DE5@0PX:2079bLP2>SXh0/[:b0;Rh^037alL0
R8R805=CD`10@400A4A403dm?@0h>3P0ADE5096AT@1jNWX0HF5Q08R8R01gMgL0][Jf0:RXZ020P800
DU9B06EUI@2;Rh/0VYZJ08^;R`1GEeL0Shn?0:RXZ01TI6@0^k^k0;Vi^@1LG5`0@d=303Ph>00/;2`0
BDU902PX:00W9bL202DU9@0I02@T900O7al0820P03Xj>P19BDT0J6QX04Y:BP1[Jf/0T92@06i^KP1Y
JFT0>CTi03Lg=`12@T80<30`03<c<`0Z:RX06a/K01`L700M7Ad0820P02LW9`0d=3@0Lg=c0:^[Z`0@
ooooo`003_ooool0o`1UIFD0@d=308:2PP1IFET0NWYj04=3@`2UYJD0UIFE061PH00h>3P0B4Q802PX
:01NGUh0?3`l07YjNP0S8b<0@410051@D00g=cL0HF5Q00h>3P0X:2P0C4a<05IFEP1IFET02`/;01LG
5`1QHF40Cdm?059BDP0;2`/0IfMW09bLW01@D500>CTi05MGE`0X:2P0Cdm?03<c<`0];Bd07AdM075a
L@1eMGD0=cLg04E5A@0G5aL0Fe]K04=3@`1XJ6P0IVIV02@T900H61P0<c<c07inOP18B4P0:b/[059B
DP0j>SX0BDU904M7A`1ADE40:RXZ06UYJ@0^;Rh0=3@d07UiN@11@D40;bl_092@T00m?Cd0ADE502LW
9`1hN7P0RXZ:04I6AP1QHF40LW9b03Lg=`1UIFD0LW9b04A4A029RHT0SXj>07IfMP2:RXX0PH6107mo
O`1BDU80YjNW08F5Q@1:BTX0@T9209bLW01UIFD0YjNW0:j^[P17AdL0TI6A06UYJ@1dM7@0Fe]K06AT
I029RHT0Cdm?071`L019BDT0Hf=S08V9R@1_Kfl0FEUI04a<C01/K6`0TI6A04m?C`27QhL0DU9B05YJ
FP1`L700EEEE04m?C`1IFET0?clo09BDU01DE5@0J6QX051@D02;Rh/0KVi^051@D010@400D51@09fM
W@1`L700BDU905mOG`1IFET0:b/[06][J`24Q8@0A4A403lo?`2OWil0<C4a05IFEP1@D500P8200:n_
[`0a<C40MgMg05iNGP1PH600M7Ad03Hf=P10@400Jf][06i^KP1[Jf/0SHf=08b<S02>SXh0Y:BT0:2P
X00m?Cd0UIFE0:b/[021PH409bLW04A4A01JFUX0JVYZ0:FUY@2BTY80ATI607]kN`1aLG40I6AT06i^
KP0D51@0=CDe05=CD`0j>SX0Vi^K0451@@14A4@0JVYZ038b<P1dM7@0A4A404U9B@1iNGT0EeMG065Q
H@2LW9`0;Bd]03Xj>P1@D500:2PX07moO`21PH40D51@04E5A@1OGel0=CDe02DU9@1_Kfl0MgMg09^K
V`0/;2`0Kfm_0410@02:RXX0MWIf02PX:00^;Rh0JVYZ04a<C022PX80LG5a04=3@`0b<S808R8R04M7
A`0i>CT0I6AT08R8R01ADE407alO07MgM`1<C4`0IVIV07emO@0g=cL0DU9B06ATI00h>3P0DU9B0451
@@0P8200ADE507IfMP1=CDd0820P04=3@`1KFe/08R8R06QXJ02VYZH0WYjN059BDP1cLg<05aLG02TY
:@1IFET0L71`05ADE0005`1jNWX0;Bd]02@T9014A4@0GEeM06EUI@1^KVh0CDe=03Xj>P1^KVh0LW9b
03`l?01mOGd0MgMg02XZ:P0T92@0@T9206i^KP1QHF40ADE50;Ng]`1YJFT0@D5100kooooo00D0F5QH
05aLG00c<c<07alO01`L700201XJ6P0N01dM7@0W9bL0IVIV09>CT`2YZJT0]KFe0::RXP2FUYH0[Jf]
0:6QX@1CDe<0?Shn038b<P0^;Rh0:RXZ03De=@0Y:BT0820P02d];@0j>SX0B4Q8051@D02NWYh0VIVI
06MWI`23Ph<0DE5A05ADE014A4@0;bl_0P1hN7P01`1UIFD0OWin0:RXZ02BTY80VIVI06QXJ00W9bL0
0P0F5QH0I029RHT0JVYZ03dm?@12@T80:2PX01HF5P0G5aL05ADE01HF5P0R8R80Cdm?08f=S@1/K6`0
?Shn01/K6`0b<S80[Jf]0<c<c038b<P0Rh^;06UYJ@1`L700^k^k0=CDe02VYZH0NGUi05mOG`1;Bd/0
@d=3038b<P0Q8B406QXJ01TI6@0G5aL061PH01LG5`0L71`0DU9B05aLG01QHF40?Cdm030`<01?Cdl0
O7al09BDU017AdL0L71`04i>CP18B4P0MgMg0;6a/@2ATI40MGEe09>CT`31`L40ZZZZ0;Fe]@2ATI40
RXZ:06m_K`12@T80OWin0:RXZ02ZZZX0];Bd0:NWY`1OGel0;Bd]01XJ6P0F5QH05ADE01TI6@0X:2P0
<C4a0861P@24Q8@0RHV90:f][@3EeMD0^k^k06QXJ01`L700R8R805eMG@0d=3@0:b/[02HV9P1IFET0
H61P03Xj>P0^;Rh0>CTi04i>CP1CDe<0MGEe08B4Q02JVYX0OWin08V9R@1oOgl601DE5@0J034a<@1W
IfL0Cdm?04m?C`1PH600T92@0861P@0a<C40B4Q8059BDP1IFET0Y:BT0<S8b02VYZH0Hf=S05QHF01S
Hf<0De=C04E5A@26QXH0[jn_0=7Ad@2BTY80CDe=02HV9P0K6a/201`L700401dM7@0O7al0=SHf03Xj
>P809bLW00D0:RXZ07alO02YZJT0WIfM0:BTY00201dM7@0T02/[:`1bLW80RXZ:0:BTY02ZZZX0CTi>
0492@P1>CTh0G5aL07IfMP1ZJVX0UIFE08Z:RP0d=3@0JVYZ0:>SX`39bLT0cLg=0=CDe030`<00@D51
03`l?01eMGD0UYJF096AT@1;Bd/0@D5104=3@`17AdL0ATI604m?C`2PX:00P8200:FUY@1>CTh0YJFU
0P2QXJ40302OWil0I6AT03Ti>@17AdL0Hf=S08b<S02k^k/0PX:20451@@1`L700];Bd038b<P<07AdM
01h0?Cdm02DU9@0U9BD09BDU028R8P0Q8B40820P01hN7P0];Bd0?Cdm06=SH`1VIVH0EUIF07IfMP0[
:b/0KFe]0451@@0f=SH0@D5102@T900Y:BT07AdM01`L700N7Qh07AdM01lO7`0_;bl0GUiN0;Zj^P2X
Z:P@ooooo`003_ooool0o`1]KFd0@d=306QXJ01HF5P0MGEe038b<P1TI6@0FUYJ055AD@0a<C407alO
01TI6@0h>3P0;Rh^05MGE`0K6a/0:RXZ04E5A@0_;bl0G5aL01XJ6P0M7Ad0;2`/04i>CP1MGEd0>SXj
01XJ6P1:BTX0A4A406YZJP0Y:BT0ADE50:BTY00k>c/0?clo065QH@0m?Cd0BTY:02l_;`1PH600<C4a
06QXJ01DE5@0>3Ph061PH00h>3P0F5QH02@T901@D500NGUi06=SH`0n?Sh0>3Ph0;Jf]P0S8b<07QhN
05eMG@1BDU80RXZ:0492@P0W9bL06a/K06m_K`11@D40<C4a08B4Q00Q8B407QhN08N7Q`1?Cdl0ATI6
01/K6`1DE5@0LW9b0410@01VIVH0J6QX02PX:01NGUh0Hf=S0410@01hN7P0PX:203Ti>@16ATH0I6AT
06EUI@10@400_Kfm07moO`0j>SX0=CDe075aL@1IFET0WinO0;Jf]P0c<c<0De=C06UYJ@1/K6`0BTY:
04I6AP1WIfL0De=C055AD@0h>3P0=cLg05MGE`1RHV80B4Q802/[:`0m?Cd0Hf=S03dm?@1bLW80:2PX
02h^;P10@400ATI6059BDP16ATH092@T06m_K`18B4P0Jf][0410@01]KFd0K6a/04m?C`1UIFD0>CTi
07EeM@1QHF40BDU9069RHP23Ph<0;bl_06m_K`1WIfL0<S8b04U9B@2<S8`0GEeM04i>CP0_;bl0JFUY
09ZJVP1DE5@0IfMW03Xj>P1JFUX0OWin0861P@10@400Rh^;03`l?017AdL0Lg=c06e]K@1TI6@0/K6a
0;:b/P1PH600[Zj^0;^k^`1cLg<0<c<c05QHF01iNGT0@410069RHP1^KVh0@41009FEU@1iNGT0;Rh^
05YJFP0M7Ad0ADE5034a<@1<C4`0RXZ:06UYJ@1KFe/0De=C02l_;`1dM7@0N7Qh07=cL`2>SXh0>3Ph
02/[:`1QHF40@D5106EUI@1ZJVX061PH05iNGP2>SXh0;Rh^04M7A`2ATI40M7Ad034a<@1_Kfl0EUIF
08b<S00L71`0WYjN065QH@1[Jf/0D51@01lO7`18B4P0Z:RX04Q8B00a<C40Q8B405MGE`12@T80C4a<
07QhN00_;bl0>3Ph04Y:BP16ATH0:b/[09FEU@1hN7P0F5QH05aLG00W9bL0Fe]K06][J`1HF5P0FUYJ
05ADE00E5AD06a/K05ADE01jNWX0?clo04I6AP1DE5@03Ph>03@d=01GEeL0Q8B405YJFP1YJFT0A4A4
034a<@1cLg<0CDe=02h^;P005`2CTi<0DU9B055AD@1SHf<0JVYZ07MgM`1XJ6P0I6AT04];B`2CTi<0
LW9b03Lg=`24Q8@0Ng]k02LW9`0T92@0@D5106][J`16ATH0G5aL0;no_`1`L700@T9200kooooo01/0
Lg=c04e=C@1;Bd/0=3@d02PX:00Y:BT0=CDe02XZ:P0T92@0AdM70;^k^`38b<P0][Jf0;:b/P2DU9@0
/k>c06a/K01QHF40<C4a02PX:00f=SH0<30`02XZ:P10@400820P028R8P0X:2P00P0f=SH0602EUID0
RXZ:08>3P`2b/[80C4a<03De=@0Y:BT0CDe=04];B`0c<c<0J6QX05YJFP16ATH0L71`08B4Q015ADD0
E5AD02TY:@0G5aL0NWYj03Ph>00U9BD08R8R01TI6@@05ADE0P0F5QH0>@0d=3@0Shn?09RHV01_Kfl0
[jn_09^KV`27QhL0W9bL0=;BdP3<c<`0][Jf07alO02YZJT0c/k>0<C4a01XJ6P0=SHf02DU9@1@D500
MWIf05IFEP14A4@09bLW01XJ6P0L71`0?Cdm07moO`1bLW80E5AD03/k>`0P8200AdM706ATI01nOWh0
a/K604I6AP0e=CD092@T03Ph>01=CDd0Cdm?028R8P0^;Rh0WinO05QHF01`L700=3@d02PX:00k>c/0
/[:b09ZJVP2d];@0RXZ:092@T027QhL0Jf][02XZ:P0201DE5@805aLG01d0820P01dM7@15ADD0[:b/
09nOW`2a/K40inOW0>KViP2UYJD0AdM704=3@`14A4@0?Shn03Ti>@0X:2P0D51@02<S8`0L71`07AdM
01`L700O7al0<c<c06e]K@1CDe<0Ph>305mOG`1UIFD0OGem01HF5P0401DE5@1L01HF5P0E5AD0L71`
0451@@12@T80RXZ:0::RXP2?Shl0UIFE07MgM`14A4@0:b/[06ATI02>SXh0`l?308>3P`18B4P0;Bd]
03dm?@1_Kfl0alO70=;BdP2g]kL0VYZJ06a/K01=CDd0@D5102DU9@0L71`06ATI02HV9P1?Cdl0<C4a
02h^;P0n?Sh0K6a/07EeM@26QXH0^k^k0:>SX`1[Jf/0=CDe03`l?017AdL0E5AD06][J`1fMWH0?Cdm
03/k>`1CDe<0@d=301lO7`1HF5P0Ng]k0::RXP2m_Kd0VYZJ0::RXP3?cll0e=CD0=KFeP2@T9009bLW
06m_K`2:RXX0WinO09JFUP1SHf<0BTY:03dm?@12@T80BTY:05ADE02SXj<0ZZZZ0;^k^`23Ph<0V9RH
0<71`@2f][H0N7Qh034a<@0m?Cd0?Shn04m?C`2JVYX0UIFE079bLP13@d<0G5aL08Z:RP0Q8B4301dM
7@0702/[:`0V9RH09BDU02DU9@0S8b<08B4Q020P800202DU9@0E03@d=019BDT0UiNG05=CD`2a/K40
L71`0;:b/P1@D500=cLg034a<@0M7Ad09BDU01dM7@0L71`07QhN02@T900];Bd0KVi^0;Zj^P3@d=00
`L71013ooooo000>ooooo`0c06YZJP17AdL0L71`04i>CP2KVi/0HV9R07IfMP21PH40K6a/03`l?01T
I6@0>c/k03Ph>00T92@0F5QH04U9B@0m?Cd0=SHf02DU9@1SHf<0?Shn03Ph>00c<c<0>SXj079bLP1G
EeL071`L03lo?`0C4a<0BTY:03De=@0m?Cd0[Jf]01TI6@0Q8B40ATI603lo?`0X:2P05aLG05EEE@0e
=CD0IfMW05IFEP0M7Ad0@d=303dm?@1CDe<0?clo04M7A`1KFe/0DU9B0080?Cdm0<d0XJ6Q02h^;P0N
7Qh0AdM705EEE@2>SXh0D51@04m?C`0L71`0MgMg03hn?P0d=3@0LW9b0451@@0O7al0NGUi055AD@1;
Bd/0:b/[04Q8B01dM7@0ATI606MWI`1ZJVX0;Rh^05IFEP1VIVH0?clo06UYJ@2TY:@0H61P05iNGP1K
Fe/0JVYZ0492@P2DU9@0UYJF04e=C@11@D40PH6106=SH`1oOgl0]kNg05IFEP1/K6`0H61P07MgM`1J
FUX0GemO06][J`1fMWH0OWin06e]K@18B4P0KVi^05aLG01JFUX0Fe]K05=CD`1RHV80<C4a07QhN01?
Cdl0?3`l04m?C`0[:b/0Fe]K069RHP12@T80EeMG03Lg=`25QHD0De=C08>3P`1LG5`0?3`l06IVIP16
ATH0U9BD08R8R00W9bL0>c/k07emO@0g=cL0PX:20820P014A4@0<c<c08R8R01KFe/0N7Qh051@D01R
HV80IFEU05QHF01YJFT0E5AD04U9B@1FEUH0EeMG03lo?`2PX:00L71`05UIF@2KVi/07QhN01lO7`1Z
JVX0ZJVY07=cL`32`/80c<c<020P800K6a/0De=C07MgM`12@T80QHF506i^KP0H61P0De=C08B4Q00o
?cl0B4Q803Ph>01QHF40KVi^03Ph>02:RXX0A4A40:6QX@2@T900<30`071`L00n?Sh0A4A40::RXP1d
M7@0:RXZ069RHP0A4A40>3Ph06e]K@0H61P0JFUY09JFUP0M7Ad0;bl_05MGE`1mOGd0Bd];07IfMP1G
EeL0Ph>303dm?@1[Jf/0FUYJ071`L01GEeL0:2PX02LW9`1NGUh0G5aL04A4A02KVi/0AdM701/K6`0O
7al0De=C05mOG`0m?Cd0D51@03/k>`0M7Ad0I6AT07MgM`1;Bd/0LW9b01`L7016ATH0A4A4059BDP1O
Gel0@T92024Q8@0`<300FUYJ059BDP0P8200BDU905EEE@0c<c<0GEeM05aLG01dM7@0HV9R07alO01i
NGT0ADE506QXJ01ADE40820P05EEE@1ADE40GUiN0080Kfm_0180FUYJ06ATI01RHV80B4Q809jNWP1R
HV80=CDe075aL@1nOWh0=3@d01dM7@0h>3P0KVi^04=3@`1XJ6P0/;2`07UiN@0m?Cd>ooooo`0;08>3
P`1GEeL0G5aL061PH01dM7@0P82007QhN01dM7@0MgMg08J6QP37alL00P3:b/X09P3Fe]H0bl_;0;Rh
^01WIfL0Q8B406ATI01MGEd0C4a<0451@@0f=SH0A4A402`/;00Q8B40<30`0410@01;Bd/0Lg=c06=S
H`2QXJ40LG5a04e=C@0n?Sh0EEEE055AD@16ATH0DU9B03De=@1iNGT0N7Qh04];B`1XJ6P0WIfM05QH
F019BDT0<30`0820P00Y:BT061PH200E5AD0>@0G5aL0=3@d0;fm_@2TY:@0B4Q803Ti>@0h>3P0F5QH
0<?3``3Hf=P0Kfm_06IVIP2CTi<0d];B08b<S023Ph<0KFe]04I6AP0N7Qh09bLW02l_;`0h>3P0Bd];
059BDP1HF5P0NWYj0:6QX@1_Kfl0BDU9024Q8@0P82007alO04Y:BP1?Cdl0RXZ:05aLG00/;2`071`L
020P800l?3`0?clo04m?C`0l?3`0LG5a04Q8B01GEeL09BDU02HV9P0K6a/0IfMW05]KF`2=SHd0IFEU
05YJFP1[Jf/0AdM703Hf=P0201DE5@0B01PH600O7al071`L01PH600^;Rh0Jf][096AT@1gMgL0emOG
0=7Ad@1hN7P09BDU02HV9P0U9BD0A4A403/k>`0[:b/0@T92100M7Ad01@0L71`07AdM05YJFP1OGel0
Jf][0080DE5A00<0R8R801LG5`0F5QH00P0E5AD201HF5P0O01DE5@1DE5@0EEEE07=cL`2YZJT0QhN7
0:b/[02ZZZX0XZ:R0;Ng]`1QHF40KFe]08R8R024Q8@0^[Zj09RHV02SXj<0]KFe0<K6aP3Lg=`0fm_K
0;Ng]`2;Rh/0Ph>304];B`1MGEd0H61P04I6AP0a<C40DU9B05aLG00306e]K@0904E5A@1=CDd0I6AT
09fMW@2d];@0]kNg04e=C@1JFUX0=3@d0080Cdm?0340DE5A05QHF016ATH0M7Ad02@T900I6AT08B4Q
07YjNP2c/k<0YJFU08n?S`29RHT0UYJF08Z:RP1WIfL0R8R803`l?02ATI40][Jf0<g=c@3Hf=P0bLW9
0;Vi^@2?Shl0K6a/04m?C`1CDe<0]KFe08Z:RP39bLT0Shn?0;^k^`3IfMT0_Kfm08V9R@1LG5`0KVi^
0861P@2BTY80a/K604Q8B01`L700EEEE05eMG@2GUiL07alO01dM7@0O7al07AdM00808b<S0`0U9BD0
6@0S8b<0820P02DU9@0X:2P0<S8b069RHP3;bl/0Z:RX0;Jf]P2^[Zh0dM7A07YjNP18B4P0<C4a024Q
8@0T92@0:BTY02/[:`17AdL0E5AD08R8R02`/;00PX:207emO@33`l<04?ooool000kooooo02<0Fe]K
04Q8B01/K6`0@T92069RHP1ADE40IFEU079bLP0Y:BT07alO04a<C00j>SX0<30`01XJ6P0h>3P0FUYJ
03Ph>00g=cL05aLG05YJFP0h>3P0=SHf02/[:`0R8R80A4A405ADE00L71`0@D510492@P1IFET0Bd];
03Hf=P2BTY80<c<c03De=@0203lo?`1704U9B@0R8R80HF5Q04A4A01UIFD0H61P02h^;P15ADD0A4A4
05YJFP1KFe/0H61P06MWI`1JFUX0CTi>04Y:BP2KVi/0A4A402TY:@1FEUH0Fe]K06QXJ01=CDd0HF5Q
02l_;`26QXH0DE5A038b<P2?Shl0EeMG024Q8@1kNg/0CDe=05UIF@0_;bl0CTi>0820P019BDT0NWYj
08Z:RP0m?Cd0I6AT075aL@0S8b<0@D5108j>SP2BTY80QXJ608Z:RP0d=3@09BDU03hn?P2JVYX0PX:2
06e]K@2OWil0C4a<051@D02EUID0C4a<07alO016ATH0EEEE059BDP1MGEd0H61P04A4A01SHf<0CTi>
05UIF@2=SHd00P14A4@0Z018B4P0G5aL07inOP0Y:BT0G5aL0410@01DE5@0NGUi051@D014A4@0Fe]K
02TY:@21PH40D51@06e]K@0a<C40RXZ:09JFUP1>CTh0AdM703Hf=P2?Shl0T92@051@D01<C4`0Kfm_
02TY:@1_Kfl0NGUi04I6AP0b<S80Ng]k0451@@1DE5@0D51@06MWI`1MGEd0=3@d04E5A@1]KFd0E5AD
07QhN00n?Sh07QhN05EEE@1PH600BDU909NGU`1kNg/0RXZ:0:ZZZP2`/;00NWYj0;jn_P3:b/X0QhN7
03Ti>@0h>3P0OGem04Y:BP27QhL0P8200492@P1]KFd0S8b<04a<C01GEeL06ATI03<c<`1RHV80>CTi
07QhN01;Bd/0DU9B06UYJ@0R8R80EeMG03hn?P1<C4`0Rh^;05mOG`0/;2`0L71`04E5A@10@400C4a<
01HF5P1WIfL0SHf=03lo?`1BDU80Hf=S07]kN`13@d<0MgMg07AdM01lO7`0Bd];06MWI`1=CDd0LG5a
07QhN01PH600>c/k08Z:RP1DE5@0ATI6092@T01gMgL0Cdm?02LW9`1NGUh0IVIV03Xj>P1=CDd0GUiN
034a<@1dM7@0O7al04Q8B01/K6`0@41006QXJ01:BTX0BDU906QXJ01ADE40>SXj03De=@1XJ6P0IVIV
02PX:01[Jf/0Q8B4059BDP1HF5P0K6a/07=cL`0j>SX0Hf=S071`L012@T80FEUI03/k>`0V9RH0P820
05ADE01IFET0EEEE06i^KP0V9RH0@41007IfMP15ADD0Xj>S075aL@1DE5@0JVYZ08>3P`0i>CT08B4Q
04=3@`1`L700HF5Q03Hf=P2ATI40Hf=S02h^;Pkooooo0380K6a/06MWI`1>CTh0BTY:051@D01;Bd/0
CTi>04a<C019BDT0R8R80;bl_02n_[h0/;2`09jNWP2j^[X0WinO0:JVYP2=SHd0Fe]K05aLG01MGEd0
J6QX0820P010@400;bl_03De=@11@D40?Shn04Y:BP1hN7P0MgMg09BDU00e=CD0<c<c05IFEP1OGel0
E5AD04Q8B00N7Qh05aLG02PX:01@D500EEEE04Y:BP1WIfL0VIVI0::RXP2QXJ40VIVI020P80X05ADE
00d05aLG07emO@24Q8@0GUiN06][J`2?Shl0^k^k0=SHf03Ph>00/k>c0<?3``1RHV80I6AT0080R8R8
04X0RXZ:05aLG01NGUh0:BTY01TI6@0K6a/07AdM02@T901kNg/0W9bL0<K6aP1hN7P08b<S01`L700O
7al08R8R0410@011@D40IFEU09bLW00U9BD05ADE01LG5`0I6AT09BDU0492@P2JVYX0LG5a03hn?P0o
?cl07alO02XZ:P0I6AT0BDU902XZ:P1IFET0EEEE04e=C@1MGEd0>3Ph03lo?`0J6QX06ATI020P800M
7Ad071`L01XJ6P0W9bL0FEUI07IfMP2<S8`0_Kfm09JFUP15ADD09BDU024Q8@0U9BD0;2`/03Ti>@13
@d<0>c/k01dM7@0L71`06QXJ01LG5`0L71`07QhN038b<P24Q8@0FEUI05eMG@1@D500Rh^;01TI6@@0
5ADE07d05aLG01DE5@0[:b/0`L710;>c/`2FUYH0^KVi09^KV`1gMgL0WIfM0:FUY@1@D500OGem06IV
IP1;Bd/0M7Ad0>7Qh@2m_Kd0QXJ60861P@2i^KT0^;Rh0;Bd]01`L700LG5a08j>SP1@D500De=C065Q
H@2@T900UiNG096AT@1`L700BDU902XZ:P0n?Sh0>3Ph04i>CP1mOGd0N7Qh04I6AP1lO7`0OGem03Ti
>@0o?cl0@T9204A4A01VIVH0RXZ:07QhN00Q8B406ATI03<c<`2MWId0`/;206ATI00h>3P0ADE505UI
F@1YJFT0GEeM06ATI02RXZ80YZJV0;Ng]`33`l<0cLg=09:BTP25QHD0VYZJ0<C4a03=cLd0d=3@0=7A
d@3Kfm/0g]kN0=7Ad@3?cll0hN7Q0=;BdP2l_;`0Y:BT09NGU`2>SXh0V9RH0;Rh^01BDU80EeMG06QX
J01VIVH0T92@024Q8@0M7Ad07alO020P800N7Qh07alO028R8P0T92@07alO01`L700M7Ad08B4Q02<S
8`15ADD0_kno0;Bd]03<c<`0cLg=0<;2`P3EeMD0SHf=051@D01/K6`0OGem07MgM`25QHD0^KVi0;Bd
]03AdM40^;Rh06EUI@0n?Sh0MWIf0:6QX@0@ooooo`003_ooool0>P1EEED0@d=306IVIP1DE5@0RHV9
055AD@1]KFd0SHf=07]kN`0g=cL0EEEE03dm?@1<C4`0?Shn04A4A01JFUX0>CTi03lo?`0Q8B40JFUY
02TY:@0m?Cd0=CDe038b<P1KFe/0A4A4020P801NGUh0G5aL061PH01;Bd/0=SHf0:2PX016ATH0=CDe
03hn?P1:BTX0I6AT03<c<`1LG5`0=cLg05]KF`2HV9P0?3`l03hn?P0T92@0D51@05QHF01BDU80IVIV
03De=@0a<C40<S8b08F5Q@1KFe/0:b/[04m?C`0_;bl206e]K@1k06YZJP17AdL0XZ:R03<c<`0X:2P0
OGem061PH00[:b/0UiNG04M7A`1HF5P0=CDe05MGE`1SHf<0:RXZ0451@@1XJ6P0>c/k07emO@2UYJD0
FEUI04i>CP2=SHd0?clo04U9B@1KFe/0KFe]05YJFP0e=CD0Zj^[09NGU`1mOGd0KVi^071`L024Q8@0
clo?0410@01<C4`0L71`06EUI@15ADD0;bl_07EeM@1UIFD0GEeM01/K6`0a<C40Fe]K079bLP1=CDd0
6ATI038b<P1hN7P0De=C07EeM@0b<S8092@T04Y:BP1OGel0DE5A05]KF`0O7al0Lg=c059BDP1_Kfl0
?clo05IFEP1fMWH0AdM704m?C`0S8b<0J6QX07MgM`1;Bd/0De=C06ATI00P8200ATI60820P00o?cl0
:b/[05]KF`1gMgL0JFUY0820P01eMGD0FUYJ06EUI@1QHF40OGem05IFEP1YJFT0OGem03`l?028R8P0
Lg=c04A4A02;Rh/0OWin0:f][@2j^[X0/[:b07IfMP2i^KT0e]KF096AT@0o?cl0>SXj06QXJ01DE5@0
TI6A08f=S@1=CDd0K6a/07MgM`18B4P0Hf=S02<S8`15ADD0E5AD03<c<`1]KFd0Lg=c0080IFEU05d0
92@T05eMG@1?Cdl0Jf][0:^[Z`1]KFd0?Shn09:BTP1KFe/0BDU905EEE@0J6QX0JVYZ09VIV@1<C4`0
D51@05UIF@1[Jf/0=3@d04A4A01OGel0N7Qh0451@@1GEeL0=3@d07inOP1]KFd0Cdm?03Ti>@1jNWX0
BDU903Ti>@1nOWh0C4a<03Hf=P0X:2P0H61P04];B`0^;Rh0Bd];08R8R018B4P0Ogmo08N7Q`1@D500
[jn_06EUI@1fMWH0CTi>03hn?P1fMWH0NGUi05IFEP0X:2P0CDe=06i^KP0V9RH0FUYJ07inOP1YJFT0
BDU908B4Q01jNWX0A4A405iNGP1JFUX0?Cdm071`L01>CTh0=CDe06IVIP1XJ6P0B4Q8051@D02<S8`0
I6AT0410@01:BTX0;2`/07QhN020P800J6QX07inOP2MWId0DU9B01HF5P0M7Ad0DU9B065QH@1MGEd0
a<C405QHF00S8b<03_ooool0;00i>CT0HF5Q07]kN`2^[Zh0];Bd0861P@24Q8@0XJ6Q0:>SX`2UYJD0
`L710<?3``2k^k/0NWYj06a/K02VYZH0[Zj^08V9R@1:BTX0=SHf034a<@0o?cl0ATI605mOG`10@400
AdM703Ti>@0M7Ad06a/K04U9B@2n_[h0Kfm_038b<P1XJ6P0BDU903`l?01CDe<0<S8b02l_;`0K6a/0
6ATI01LG5`0I6AT0>CTi0P1=CDd0101MGEd0d=3@0:>SX`0g=cL201DE5@0301@D500E5AD05ADE0080
51@D03d04A4A01DE5@0D51@05ADE06QXJ01fMWH0TI6A08^;R`21PH40U9BD0<_;b`2j^[X0FEUI0410
@00Y:BT0>3Ph059BDP2TY:@0TY:B07QhN013@d<0K6a/059BDP0T92@05ADE01`L701lO7`0_[jn0=3@
d02m_Kd0QhN704Y:BP0Z:RX07QhN03Hf=P1JFUX0QXJ60;Zj^P0U9BD05ADE01TI6@0G5aL06a/K01dM
7@1RHV80IfMW051@D00S8b<04Q8B01<C4`0E5AD0>CTi028R8P1LG5`0N7Qh05=CD`1DE5@0?Cdm03Xj
>P0R8R809bLW00808B4Q0P0L71`02@0P8200I6AT09:BTP2OWil0^;Rh08f=S@0e=CD09RHV01`L7002
01lO7`0@02l_;`1FEUH0=CDe01lO7`0S8b<07QhN01/K6`0L71`07QhN02<S8`1hN7P0Hf=S08J6QP24
Q8@0[jn_020P80<05ADE05h04A4A01DE5@0E5AD0;Bd]0;Rh^02JVYX0_kno096AT@0i>CT0RXZ:08b<
S02NWYh0@410038b<P0i>CT0GUiN06][J`3Jf]X0H61P03Lg=`0j>SX0GEeM0:BTY02a/K40Hf=S05MG
E`2>SXh0I6AT0861P@2RXZ80UIFE09NGU`21PH40G5aL04U9B@0m?Cd0ADE502l_;`1KFe/0MgMg02h^
;P0/;2`0@4100;Vi^@3=cLd0KVi^04Q8B01CDe<0TI6A0=[JfP2;Rh/0GEeM02LW9`1^KVh0Hf=S096A
T@0g=cL0?clo03dm?@12@T80DU9B07AdM026QXH0WIfM0:ZZZP33`l<0Q8B408^;R`1fMWH0B4Q803/k
>`19BDT0IFEU08b<S03@d=00alO70;2`/02/[:`0U9BD0<c<c02l_;`0WIfM059BDP0m?Cd0@41004A4
A01lO7`0RXZ:04Q8B01cLg<0KVi^09fMW@0X:2P06ATI0P0M7Ad01@0H61P07AdM01dM7@0Q8B407AdM
008071`L01L0820P05]KF`2h^;P0S8b<07emO@3Fe]H0YJFU08b<S032`/80NGUi08>3P`2UYJD0VIVI
0:BTY02/[:`0clo?0<O7a`2>SXh0CDe=03<c<`1IFET0Q8B409nOW`0@ooooo`003_ooool0o`1HF5P0
C4a<05ADE015ADD0QXJ603Ti>@0f=SH0@D5104M7A`0m?Cd0@D51024Q8@0g=cL0>CTi03`l?00P8200
6ATI03Xj>P0M7Ad0GemO01XJ6P0L71`0;bl_04=3@`2WYjL0@T9201/K6`1=CDd0ADE506QXJ00U9BD0
6QXJ05MGE`1DE5@0>3Ph065QH@0o?cl0D51@02h^;P1>CTh07alO030`<01<C4`0;Bd]05ADE017AdL0
EUIF0410@01UIFD0Shn?06MWI`19BDT0>c/k09RHV01@D50071`L05iNGP13@d<0Cdm?04E5A@0b<S80
;2`/09>CT`1?Cdl07alO07QhN00h>3P08R8R069RHP1<C4`0De=C01TI6@0^;Rh0IFEU0410@01HF5P0
K6a/02DU9@18B4P0E5AD05EEE@1SHf<0WinO05IFEP1HF5P0GemO06YZJP1EEED09bLW09BDU03Ph>00
h^;R07alO02CTi<0MgMg0<_;b`1_Kfl0EUIF08n?S`1JFUX0Hf=S05IFEP1oOgl0FEUI06][J`1fMWH0
?Shn07IfMP2<S8`0Cdm?05UIF@17AdL0MgMg03dm?@1KFe/0G5aL03`l?01DE5@0FEUI04e=C@1:BTX0
;Rh^07EeM@1KFe/0QhN706=SH`1PH600JVYZ0410@01eMGD0C4a<07moO`1lO7`0>SXj05=CD`1[Jf/0
@d=305ADE02NWYh0AdM702d];@1BDU80Lg=c06IVIP1nOWh0PX:206EUI@25QHD0J6QX08B4Q01=CDd0
JFUY05=CD`11@D40PH6106m_K`1<C4`0TI6A08f=S@24Q8@0`L710<[:bP13@d<0QHF50<K6aP2GUiL0
@41003De=@1DE5@0>CTi07EeM@27QhL0CDe=069RHP1bLW80AdM705aLG00S8b<0?clo069RHP0m?Cd0
OWin07IfMP1cLg<0L71`02d];@1]KFd0Lg=c07IfMP2][Jd0@T9204I6AP2PX:00IfMW04M7A`1NGUh0
9bLW0:6QX@31`L40OWin055AD@1HF5P0KFe]034a<@0n?Sh0Ng]k09>CT`1?Cdl0DU9B02h^;P1bLW80
LG5a04i>CP0f=SH0NGUi05IFEP1QHF40Shn?04I6AP0o?cl09BDU05EEE@0`<300=CDe06i^KP1iNGT0
AdM707EeM@1/K6`0>SXj0861P@1PH600OGem075aL@1=CDd0C4a<05=CD`0h>3P08R8R04i>CP1bLW80
>SXj04Q8B01VIVH0<S8b04=3@`2?Shl0SHf=05mOG`26QXH0QhN702l_;`11@D40E5AD059BDP005`1M
GEd0N7Qh059BDP0g=cL0Kfm_07IfMP1ADE40Jf][0451@@26QXH0Ng]k02<S8`0Y:BT0MGEe059BDP0I
6AT07alO05ADE00Y:BT0C4a<0:BTY01kNg/0>c/k00kooooo02@0;2`/05MGE`3?cll0f][J0;Fe]@2<
S8`0IVIV04];B`0m?Cd0=3@d09NGU`1nOWh0Hf=S05EEE@2=SHd0Shn?0;no_`3>c/h0Ti>C04Q8B00U
9BD07alO01hN7P0h>3P0E5AD02l_;`0E5AD05aLG01PH601FEUH0Y:BT069RHP1mOGd0ATI604E5A@1<
C4`202DU9@0303@d=012@T807AdM00805aLG00h06a/K03Lg=`1GEeL0C4a<09:BTP2WYjL0Fe]K01LG
5`0C4a<04A4A01<C4`0A4A4051@D01DE5@804a<C00/051@D024Q8@1jNWX0<S8b03dm?@0a<C40:b/[
04A4A02SXj<0LG5a03Xj>P0201`L700i01/K6`0U9BD0EEEE0:ZZZP1TI6@0LG5a03Lg=`12@T80Jf][
04Y:BP1cLg<0ZJVY09JFUP1RHV80HF5Q06YZJP1_Kfl0JVYZ06IVIP2LW9`0bLW906e]K@2CTi<0@d=3
01LG5`0F5QH05ADE01LG5`0L71`0CDe=05EEE@1DE5@05aLG010@400E5AD0410@03Ph>00Q8B40SXj>
0<30`01lO7`0UIFE07moO`11@D40>CTi03`l?00R8R8071`L01DE5@0G5aL071`L06QXJ02_[jl0g]kN
0<_;b`2BTY80<c<c00806a/K00P06ATI024Q8@12@T80?Cdm02XZ:P0M7Ad071`L01LG5`805ADE00/0
61PH01hN7P1ADE40R8R806a/K01hN7P0c<c<05=CD`0a<C406ATI01LG5`0201DE5@15028R8P27QhL0
U9BD07]kN`2k^k/0VIVI05QHF01LG5`0A4A407moO`0O7al0820P02@T901:BTX0Shn?0:>SX`1PH600
<30`03hn?P1:BTX0QXJ608Z:RP1VIVH0C4a<05IFEP1BDU80Kfm_06i^KP1kNg/0P82004I6AP0l?3`0
AdM704U9B@1BDU80?Shn06EUI@1IFET07AdM01/K6`0l?3`0^k^k0?3`l03Vi^H0fMWI0<30`03@d=00
alO707=cL`1NGUh0OGem04i>CP1GEeL0Vi^K03De=@0b<S80;Bd]02@T900n?Sh0`/;20<O7a`2PX:00
J6QX071`L00g=cL0CDe=07]kN`12@T80@d=30080<c<c01T0?clo0;2`/034a<@0UIFE09BDU01SHf<0
NWYj0:BTY01OGel0;bl_030`<00a<C40=3@d04Y:BP1kNg/0De=C06QXJ020P800XZ:R038b<P0K6a/0
6ATI01`L700K6a/0<c<c00807alO0P0M7Ad0600Y:BT0Ti>C0;:b/P1mOGd0L71`09FEU@2j^[X0VYZJ
08f=S@2m_Kd0VIVI08j>SP3Bd]80ZZZZ0<?3``3Kfm/0]KFe06IVIP0e=CD0?Cdm04A4A01bLW80Q8B4
04E5AA3ooooo000>ooooo`0=059BDP1?Cdl0EUIF04];B`1TI6@0E5AD05EEE@1/K6`0D51@02d];@1;
Bd/0@T92030`<00202h^;P2/05EEE@0^;Rh0?Shn01`L701MGEd0ADE502HV9P0[:b/061PH04E5A@1M
GEd08b<S03/k>`0d=3@0@d=305IFEP0I6AT0EUIF04a<C00M7Ad0>CTi04Q8B011@D40;2`/0492@P19
BDT0DE5A06EUI@0/;2`0<30`05]KF`10@400EeMG0410@01[Jf/0I6AT0451@@0b<S80VIVI05aLG00V
9RH0@41003dm?@1JFUX0?Shn06=SH`0i>CT0Ti>C06YZJP0T92@0GUiN06IVIP0Q8B40HV9R05mOG`15
ADD0>3Ph03/k>`22PX809bLW04I6AP1`L700@T9205iNGP1jNWX0EUIF059BDP2`/;00KFe]075aL@1B
DU80IfMW03hn?P0T92@0NGUi0=GEe@3Shn<0WYjN09>CT`17AdL0`L7107IfMP1:BTX0U9BD059BDP1[
Jf/0EEEE08R8R01MGEd0Lg=c07UiN@10@400KFe]08^;R`1GEeL0FUYJ04Q8B01dM7@0>CTi05iNGP1L
G5`0@d=3051@D01JFUX0C4a<065QH@1DE5@0MgMg04U9B@1oOgl0IfMW0492@P29RHT0AdM707inOP1;
Bd/0Q8B40:BTY015ADD0@d=307inOP1:BTX0Fe]K09VIV@1=CDd0<C4a07UiN@1UIFD0LG5a055AD@1U
IFD0SHf=08>3P`1KFe/0PH6103Ti>@22PX80IfMW04A4A01hN7P0IVIV03De=@23Ph<0NWYj05UIF@2T
Y:@0`<3006e]K@2YZJT0dm?C07inOP15ADD0>CTi0820P01;Bd/0Lg=c09JFUP0o?cl0I6AT0861P@0n
?Sh0De=C02`/;00j>SX0F5QH03`l?01WIfL0O7al07moO`1mOGd0<C4a04m?C`80J6QX02X0WIfM06a/
K00n?Sh0T92@05ADE017AdL0KVi^020P8024Q8@0^k^k061PH01=CDd0HV9R07YjNP1<C4`0=CDe06UY
J@2?Shl0A4A405MGE`1:BTX0OWin08B4Q018B4P0=CDe06][J`1?Cdl0LG5a0:JVYP1gMgL0ADE502XZ
:P1FEUH0ADE503Ti>@1SHf<0MWIf03<c<`1YJFT0Lg=c0492@P2@T90205aLG00]04E5A@19BDT0IFEU
06UYJ@19BDT092@T03hn?P1SHf<0:b/[06EUI@2DU9@0JFUY04m?C`1dM7@0KVi^02`/;01GEeL0OWin
03hn?P1>CTh0O7al02@T901gMgL0UYJF075aL@0n?Sh0OWin03Ph>00d=3@0SXj>04I6AP2LW9`0QHF5
06QXJ01;Bd/0K6a/05eMG@0K6a/0:b/[04m?C`1aLG40>SXj09VIV@1iNGT0FEUI00kooooo02D0C4a<
06YZJP22PX80GemO04m?C`0m?Cd0=CDe030`<00_;bl0;Rh^069RHP1NGUh0@41003@d=00n?Sh0JFUY
08j>SP2a/K40LG5a05aLG01VIVH0TI6A0;2`/02YZJT0SXj>06=SH`0m?Cd0<c<c03/k>`1cLg<0alO7
0<c<c01^KVh0<c<c02XZ:P0T92@09BDU0080820P01P092@T01`L700F5QH05QHF01DE5@0F5QH07alO
04i>CP1hN7P0QXJ608R8R01bLW80J6QX051@D00m?Cd0@T9204M7A`1;Bd/0C4a<05MGE`1dM7@0ZZZZ
0;Jf]P0`<30202DU9@1Z02d];@18B4P0UiNG05]KF`0d=3@06ATI01XJ6P0I6AT071`L01hN7P0c<c<0
G5aL04I6AP1RHV80FUYJ05MGE`1mOGd0Q8B405=CD`1mOGd0IfMW055AD@1;Bd/0AdM704U9B@1@D500
B4Q807alO01[Jf/0QHF508>3P`12@T808b<S01HF5P0G5aL06ATI03Xj>P1IFET09BDU024Q8@0?3`l0
5ADE01@D500o?cl0O7al09^KV`1>CTh0D51@04U9B@0Y:BT0FUYJ0::RXP2EUID0OGem05]KF`1?Cdl0
:RXZ02l_;`1lO7`0b/[:0>?Sh`3UiND0eMGE09fMW@13@d<09bLW02TY:@11@D40<C4a01`L700K6a/0
8R8R02h^;P0P82006QXJ02<S8`0V9RH0;Rh^02l_;`12@T80Shn?0<c<c03MgMd0Zj^[0820P01MGEd0
BTY:051@D01eMGD0S8b<07EeM@1ADE40<S8b04m?C`28R8P0T92@0820P021PH40MGEe038b<P0S8b<0
9RHV02`/;01=CDd0=SHf06EUI@806a/K00/092@T03<c<`1ADE40F5QH061PH01bLW80T92@05ADE01R
HV80JFUY03/k>`0301dM7@0O02@T900i>CT0KFe]0:ZZZP1gMgL0FEUI01`L700^;Rh0/K6a0:2PX026
QXH0Q8B409fMW@2FUYH0Ng]k05YJFP1_Kfl0OWin092@T02MWId0XJ6Q04Q8B00`<300=3@d05iNGP2E
UID0Y:BT0<?3``2f][H0TY:B038b<P0202DU9@80?clo0340<c<c04M7A`2<S8`0Rh^;0:2PX035aLD0
V9RH04E5A@0g=cL0A4A406e]K@14A4@0=CDe02TY:@0[:b/0;2`/03Hf=P19BDT0J6QX079bLP2:RXX0
Z:RX065QH@0U9BD0@D51065QH@12@T80D51@05MGE`1DE5@0EUIF05UIF@1mOGd0Ng]k06m_K`28R8P0
XJ6Q0861P@2MWId0UYJF07]kN`2h^;P0ZJVY09FEU@39bLT0TY:B0:BTY03:b/X0OGem0080VYZJ0P28
R8P00`1RHV80EEEE05eMG@0@ooooo`003_ooool0e`1ADE40G5aL04i>CP1OGel0MgMg06e]K@1<C4`0
OGem08^;R`17AdL0EEEE04Y:BP0U9BD0BTY:03De=@1ZJVX0=CDe04Y:BP0^;Rh0Fe]K04E5A@0C4a<0
8B4Q01hN7P1EEED0I6AT02LW9`0d=3@0Bd];04i>CP1FEUH06QXJ03Hf=P1]KFd0;bl_04a<C01>CTh0
;2`/04E5A@19BDT0Fe]K04U9B@1DE5@0A4A403Xj>P1[Jf/0=cLg06a/K01LG5`0Ogmo079bLP13@d<0
<c<c08n?S`1@D500<S8b04i>CP0b<S80BDU907IfMP1lO7`0?Cdm0;6a/@1HF5P0:RXZ03lo?`1cLg<0
:RXZ04Q8B01CDe<0B4Q804i>CP0g=cL0N7Qh01/K6`1:BTX0UiNG04Q8B01LG5`0IVIV03Ph>00/;2`0
VIVI079bLP1dM7@0BTY:08j>SP1SHf<0=cLg04Y:BP2`/;00g=cL0<K6aP2:RXX0>3Ph07MgM`1TI6@0
?3`l08B4Q00k>c/0MWIf04a<C01bLW80>SXj06a/K01cLg<0?Cdm05QHF02:RXX0EUIF06EUI@12@T80
Ng]k04m?C`15ADD0K6a/04E5A@1ADE40J6QX0410@017AdL0GEeM07AdM01[Jf/0PX:205QHF00j>SX0
R8R8065QH@1IFET0?Shn06m_K`2OWil0EEEE04];B`1OGel0;bl_0492@P1jNWX0G5aL03De=@2QXJ40
HV9R05EEE@1VIVH0Kfm_06ATI02;Rh/0ATI609NGU`1=CDd0LW9b06YZJP0W9bL0De=C06e]K@1BDU80
YjNW09RHV01;Bd/0RXZ:0<C4a015ADD0PH610<W9b@2SXj<0Fe]K02d];@1DE5@0FUYJ08V9R@2BTY80
J6QX0492@P1NGUh06a/K02HV9P0I6AT0;Bd]04e=C@1GEeL0V9RH07AdM01;Bd/0EEEE03hn?P1mOGd0
PX:204m?C`2QXJ40FUYJ04Y:BP2g]kL0Q8B403lo?`1?Cdl05ADE07EeM@2RXZ80QXJ606EUI@1LG5`0
LW9b01TI6@0?3`l071`L08:2PP1jNWX0C4a<01HF5P0o?cl0IVIV05QHF00e=CD0H61P0080BDU903d0
UYJF06a/K01IFET09RHV04=3@`15ADD071`L02d];@1QHF40@41005YJFP1RHV80:BTY07alO01fMWH0
H61P05MGE`0g=cL0;bl_04Y:BP1PH600;bl_04E5A@1JFUX071`L03Lg=`1SHf<0F5QH04a<C029RHT0
Ng]k07=cL`1MGEd0K6a/03De=@1ZJVX0U9BD0410@01`L700WinO05MGE`1>CTh0X:2P07IfMP16ATH0
Kfm_030`<02LW9`0Z:RX07QhN01BDU80JFUY03lo?`0K6a/0;Rh^04I6AP1UIFD0EeMG0;fm_@29RHT0
:RXZ00kooooo01/0Bd];04E5A@0o?cl0DU9B05MGE`0`<30071`L01DE5@0G5aL05ADE01<C4`0D51@0
AdM704Q8B00h>3P0FEUI08F5Q@3Lg=`0e=CD0;Fe]@2ZZZX0[Jf]07alO01>CTh0FUYJ04=3@`0`<300
0P0a<C401@0c<c<0YjNW0<S8b02a/K40:b/[008071`L00P06a/K028R8P0R8R807QhN01TI6@0F5QH0
5ADE01@D50805ADE01P06QXJ05MGE`24Q8@0`<300:RXZ01bLW80GemO04m?C`15ADD0@D510492@P1G
EeL0Jf][08F5Q@2ZZZX0cLg=08f=S@0U9BD07alO02PX:01IFET0Shn?061PH00U9BD301LG5`0?01DE
5@0J6QX071`L04E5A@1NGUh0<S8b03`l?01SHf<0Vi^K0:6QX@1KFe/0ATI604A4A00O7al05ADE0080
6ATI02@05QHF01TI6@13@d<0SHf=05]KF`1YJFT0HV9R0820P01YJFT0;bl_01XJ6P10@400Lg=c01lO
7`0@41003Ph>01DE5@0o?cl0P82008>3P`1JFUX0De=C02PX:00E5AD04A4A02<S8`1>CTh0ATI609JF
UP3Bd]80[Jf]0:2PX02XZ:P0b/[:0<S8b02a/K420=;BdP050<71`@2h^;P0AdM704m?C`0a<C400`0G
5aL0900I6AT061PH03<c<`0d=3@092@T01PH600L71`07alO06IVIP3MgMd0hN7Q0;Fe]@29RHT0TY:B
0<O7a`3?cll0c<c<09NGU`2d];@0RHV902d];@0d=3@0=cLg04I6AP1JFUX0JFUY06][J`1CDe<0IFEU
04i>CP0Q8B40:2PX02h^;P0R8R80DU9B01TI6@805QHF06P08B4Q05ADE01bLW80YZJV0;Zj^P2=SHd0
^[Zj07IfMP1eMGD0;bl_01hN7P0V9RH0>c/k05EEE@2MWId0b/[:0<g=c@1fMWH0BTY:04i>CP1[Jf/0
c<c<092@T02FUYH0QhN70820P028R8P0Y:BT08R8R01<C4`0DE5A0;Zj^P2m_Kd0M7Ad05aLG02;Rh/0
YjNW09FEU@1nOWh0WYjN0<G5a@2e]KD0YjNW0820P016ATH09RHV028R8P12@T80;Bd]02/[:`1ZJVX0
QXJ607]kN`2LW9`0SXj>03Ti>@0d=3@0@41003Ti>@0L71`0<30`03De=@0`<300=3@d02/[:`0];Bd0
@41008b<S02`/;00_Kfm0;Fe]@2NWYh0QXJ603De=@0i>CT0J6QX07=cL`1WIfL0NGUi09VIV@28R8P0
K6a/07EeM@2ATI40VYZJ09BDU029RHT0>3Ph06MWI`35aLD0`/;20;:b/P39bLT0dm?C0<[:bP1UIFD0
G5aL06MWI`2SXj<0ZJVY08R8R02FUYH0Cdm?079bLQ3ooooo000>ooooo`15059BDP1ADE40Fe]K06UY
J@1cLg<0Jf][04E5A@1mOGd0U9BD04Y:BP1KFe/0DE5A034a<@1?Cdl0<c<c06MWI`0_;bl0Cdm?02TY
:@1MGEd0>CTi01LG5`0i>CT07QhN04M7A`1GEeL092@T04U9B@1GEeL0@d=3055AD@0H61P0?Cdm07Yj
NP0a<C40C4a<05YJFP0e=CD0Bd];04M7A`1>CTh0@T9207EeM@1>CTh0=cLg04U9B@0l?3`0MGEe05]K
F`1oOgl0A4A4024Q8@0U9BD0L71`06IVIP0a<C40Bd];02LW9`13@d<0JVYZ07EeM@0g=cL0]kNg02`/
;00I6AT0>CTi065QH@0];Bd0@d=30080DE5A03<0FEUI03Hf=P1cLg<0:b/[02h^;P1ZJVX0<30`05iN
GP1fMWH0OGem03hn?P2NWYh0@d=307MgM`1OGel0Shn?05=CD`1:BTX0Fe]K0;Vi^@3Bd]80P82009NG
U`1CDe<0/K6a07inOP0i>CT0T92@05]KF`1gMgL0DE5A08F5Q@16ATH0EUIF07UiN@0j>SX0FEUI09NG
U`1SHf<0FEUI0410@01nOWh0IVIV0451@@1DE5@0A4A405aLG01kNg/0D51@05EEE@0l?3`00P1QHF40
VP2<S8`0E5AD038b<P1NGUh0B4Q80861P@1JFUX0G5aL0;bl_01ADE40Cdm?07YjNP1KFe/0De=C08f=
S@0k>c/0<c<c08V9R@1aLG40KFe]06IVIP18B4P0]kNg0;Rh^019BDT0P82003Xj>P1gMgL0RXZ:04];
B`1]KFd0ADE501dM7@1EEED0RHV906UYJ@2JVYX0d=3@04U9B@1ZJVX0`l?30:BTY01VIVH0>CTi04=3
@`0b<S80CTi>06MWI`1[Jf/0@D5105QHF01=CDd0?Shn038b<P0R8R80B4Q8024Q8@16ATH0PX:2055A
D@18B4P0:RXZ03@d=021PH40G5aL0<G5a@1MGEd07alO08R8R02;Rh/0>CTi061PH00H61P0LW9b06MW
I`1PH600HV9R05MGE`1PH600FEUI02TY:@0[:b/0Kfm_06MWI`1<C4`0Kfm_079bLP28R8P0KFe]03Hf
=P1EEED0MgMg069RHP2f][H0RXZ:07MgM`0Z:RX0ATI608J6QP1JFUX0BTY:07inOP0g=cL0CDe=07]k
N`16ATH0IfMW06UYJ@19BDT0G5aL051@D01SHf<0DU9B051@D00`<300BTY:07UiN@0i>CT0@d=307al
O01:BTX0De=C0;fm_@2^[Zh0Lg=c06e]K@1^KVh092@T03Lg=`2?Shl0;Bd]0861P@2:RXX0BDU902/[
:`1/K6`0I6AT04Y:BP1WIfL0<c<c06a/K023Ph<0N7Qh04i>CP1]KFd0NWYj01`L700I6AT0<c<c05IF
EP0i>CT0OGem08>3P`1GEeL>ooooo`0O09JFUP25QHD0GEeM02/[:`0m?Cd092@T01dM7@0F5QH05ADE
01@D500@41004A4A010@4017AdL0EUIF04Q8B02=SHd0a<C40<g=c@2l_;`0f][J0;Fe]@1MGEd0IFEU
07]kN`1mOGd0NGUi07AdM01ZJVX0LW9b08^;R`0209bLW00808n?S`0c<c<0?Cdm02@T900I6AT07QhN
01lO7`0N7Qh201lO7`0:01/K6`0G5aL05aLG01/K6`1>CTh0L71`09>CT`2^[Zh0RHV904Y:BP80>c/k
00@0@D5103<c<`13@d<0>CTi0P0d=3@02@1GEeL0V9RH06EUI@0m?Cd0@d=307MgM`2>SXh0IVIV020P
800201PH600n01HF5P0E5AD05aLG01XJ6P0M7Ad0SXj>065QH@13@d<0<S8b0::RXP2VYZH06ATI01`L
7018B4P0:RXZ01LG5`0F5QH061PH01LG5`0J6QX09RHV04M7A`2FUYH0NWYj03<c<`1FEUH0HV9R07in
OP29RHT0YJFU06ATI00C4a<0410@02d];@1VIVH0Bd];03Ti>@1kNg/0IFEU03`l?00F5QH05ADE010@
400F5QH0ATI604Y:BP0a<C40Fe]K0<_;b`3Fe]H0^k^k09FEU@1ZJVX0G5aL09VIV@3>c/h0_[jn0;Ng
]`2f][H0_[jn01/K6`0H61P201DE5@0801HF5P0E5AD0;Bd]020P800I6AT05aLG01`L701JFUX20<g=
c@070=WIf@2l_;`0Ti>C0:^[Z`2][Jd0`<300<g=c@020=WIf@1A079bLP0a<C40>c/k0492@P13@d<0
BTY:04a<C01ADE40D51@03`l?01OGel0C4a<02HV9P0Z:RX0:BTY05eMG@0J6QX05ADE01HF5P0[:b/0
Fe]K0;no_`2d];@0ZJVY08B4Q01WIfL0W9bL09jNWP22PX80RXZ:0:>SX`2^[Zh0PX:205MGE`1JFUX0
T92@071`L00k>c/0YZJV0;2`/02YZJT0P82007=cL`1ZJVX0EUIF07alO02JVYX0RHV908R8R023Ph<0
`L710<G5a@2WYjL0NGUi06UYJ@3:b/X0XJ6Q06e]K@2YZJT0X:2P06i^KP2KVi/0TY:B075aL@1GEeL0
B4Q804U9B@10@400=3@d07inOP2EUID0S8b<09ZJVP2NWYh0>SXj038b<P17AdL0=3@d01dM7@0U9BD0
>c/k0080<S8b02P0:BTY02XZ:P0k>c/0YZJV0<S8b03@d=00c/k>0<W9b@1eMGD0RXZ:0:BTY02:RXX0
Hf=S05UIF@1MGEd0Kfm_0::RXP3;bl/0cLg=0=GEe@3Hf=P0fm_K0;Zj^P2<S8`0]KFe0<c<c03:b/X0
`l?30=7Ad@3De=@0_kno05iNGP25QHD0V9RH0;Jf]P2^[Zh0_kno0;fm_@2XZ:P0S8b<4?ooool000ko
oooo02/0D51@0410@00o?cl0H61P07UiN@0h>3P0:RXZ069RHP1UIFD0?3`l05=CD`0P8200=CDe05MG
E`0^;Rh0C4a<00h>3P16ATH0:2PX05ADE00m?Cd04a<C0410@00[:b/0A4A405IFEP0O7al0DU9B05QH
F00m?Cd0ATI600l?3`16ATH0P82003dm?@1<C4`0EeMG02l_;`0h>3P0@d=302`/;00L71`0CTi>0080
=SHf04`0@d=302`/;010@400=3@d07IfMP1BDU80=cLg028R8P25QHD0@T9201LG5`18B4P0EEEE05=C
D`1EEED0CTi>020P802?Shl0Cdm?02/[:`0j>SX0?clo01`L700f=SH0H61P04U9B@0k>c/0;Bd]05UI
F@0m?Cd0>CTi06m_K`1;Bd/0@T9205IFEP22PX80@T920;2`/017AdL0KFe]03`l?02:RXX0HF5Q04A4
A01>CTh0/[:b0820P00`<300IfMW04=3@`2_[jl0P82002h^;P28R8P0H61P07inOP0b<S80Lg=c05IF
EP1gMgL0Bd];02HV9P16ATH0OGem06YZJP0f=SH0:RXZ05ADE016ATH0C4a<075aL@0m?Cd0Cdm?059B
DP0h>3P0NGUi0P1eMGD0300Q8B40LG5a08:2PP0h>3P0Hf=S01PH601OGel0IfMW05IFEP2n_[h0BDU9
02d];@80IFEU08d0EeMG0:2PX011@D408b<S0:2PX01XJ6P0TI6A07inOP10@400?Shn07alO01@D500
QHF5024Q8@0o?cl0J6QX05=CD`1FEUH0N7Qh0410@01/K6`0TY:B06YZJP2DU9@0d];B08>3P`2FUYH0
g]kN0:>SX`1dM7@0K6a/09FEU@1MGEd0JVYZ08F5Q@1RHV80NWYj08J6QP1OGel0CDe=03Lg=`0T92@0
I6AT03Xj>P1FEUH0U9BD05IFEP1<C4`0EUIF04Y:BP2MWId0GemO0820P01RHV80@410096AT@1fMWH0
?Shn0861P@0Q8B40UIFE06UYJ@1iNGT0I6AT07inOP2=SHd0Fe]K0410@012@T80RXZ:07AdM01`L700
SHf=08B4Q02<S8`0IVIV03Hf=P1IFET0OGem05]KF`2a/K40PX:207]kN`0^;Rh0DE5A08N7Q`1HF5P0
D51@07inOP1:BTX0H61P0:6QX@1LG5`0Kfm_07EeM@1DE5@0UIFE06][J`1ZJVX0J6QX04Y:BP0Q8B40
@D5107MgM`1<C4`0?clo08j>SP0o?cl0=3@d075aL@1cLg<0JVYZ08J6QP2@T900FUYJ04A4A02<S8`0
>SXj08V9R@24Q8@0@d=302`/;01iNGT0>CTi04Y:BP2:RXX0C4a<08F5Q@2c/k<0M7Ad03<c<`24Q8@0
PX:203<c<`1>CTh0KFe]08Z:RP15ADD0QhN708Z:RP1EEED03_ooool0AP2:RXX0WIfM03Hf=P0R8R80
=SHf02<S8`0I6AT061PH01LG5`0E5AD04A4A010@400B4Q8092@T08V9R@2BTY80O7al04i>CP1IFET0
[Jf]0<[:bP3EeMD0OWin05YJFP1OGel0ADE503Ti>@0g=cL0@T9209nOW`28R8P0UiNG09RHV02KVi/0
YJFU05iNGP1IFET08b<S01`L700U9BD0;bl_02TY:@0Z:RX0;Rh^01dM7@0K6a/071`L04e=C@1eMGD0
MgMg05]KF`2=SHd0RXZ:04M7A`0j>SX0=SHf01lO7`0I6AT071`L024Q8@0M7Ad071`L071`L02<S8`0
UiNG0;2`/02NWYh0T92@06MWI`0N7Qh201DE5@0@01PH600G5aL05ADE01LG5`0J6QX0;2`/08Z:RP1F
EUH0?Cdm079bLP21PH406ATI01PH600l?3`0<C4a01XJ6P805ADE01L071`L01dM7@0E5AD05aLG05EE
E@2IVIT0<C4a028R8P18B4P0EUIF07inOP3>c/h0GUiN02l_;`1SHf<0GUiN02`/;00T92@0K6a/0;Jf
]P1hN7P05aLG01DE5@02014A4@0C01@D500g=cL0GUiN06e]K@12@T80GEeM0=KFeP2`/;00HF5Q03De
=@0b<S80CTi>0;Jf]P35aLD0Shn?0:ZZZP2d];@0@d=301LG5`0401DE5@0R04I6AP0M7Ad06ATI01XJ
6P1BDU80`L7107=cL`2?Shl0ZZZZ07QhN01CDe<0TY:B0;Jf]P2VYZH0^[Zj0;Rh^03AdM40Ogmo04I6
AP19BDT0<c<c01TI6@0M7Ad0>3Ph01hN7P10@400EUIF03dm?@1EEED0Jf][02l_;`0c<c<0FEUI01XJ
6P805aLG01l09BDU05EEE@3De=@0U9BD09nOW`36a/H0Vi^K0;6a/@3:b/X0_;bl08j>SP1NGUh0@410
02`/;00a<C40ATI606a/K02g]kL0X:2P08b<S02EUID0U9BD069RHP1KFe/0FUYJ06QXJ01kNg/0S8b<
0:FUY@2n_[h0Xj>S0080_Kfm01h0K6a/06IVIP31`L40a/K603Lg=`1<C4`0NGUi04m?C`0P8200?clo
0:ZZZP1JFUX0CDe=04E5A@1FEUH0Hf=S06YZJP2HV9P0/K6a0<71`@2m_Kd0XJ6Q02TY:@0];Bd0?3`l
03@d=00N7Qh07alO03Ph>00l?3`202h^;P0W03<c<`15ADD0WinO0<o?c`3;bl/0e=CD0=SHf02JVYX0
QHF5079bLP23Ph<0S8b<09JFUP24Q8@0PH6107QhN01kNg/0UiNG0<C4a02n_[h0^[Zj0;fm_@2^[Zh0
b/[:0=[JfP3YjNT0f][J08f=S@2o_kl0dm?C0<_;b`2h^;P0XJ6Q0<30`02^[Zh0_kno0;jn_P1fMWH0
H61P013ooooo000>ooooo`1U04a<C01EEED0B4Q8055AD@1/K6`0Bd];02TY:@1HF5P0BTY:02LW9`1J
FUX0<c<c02LW9`0Q8B406a/K05MGE`0T92@0;Rh^014A4@10@400Cdm?01LG5`11@D406ATI0410@01>
CTh0;bl_051@D00d=3@08R8R059BDP0N7Qh0820P06ATI00J6QX0;bl_04Q8B00S8b<07QhN02l_;`1@
D500?Shn05eMG@0d=3@08b<S05=CD`1=CDd0JVYZ03Xj>P1HF5P0EeMG02TY:@0P8200O7al04];B`0Z
:RX0?Shn05]KF`1UIFD0YJFU0820P00_;bl0Shn?05UIF@0d=3@0KFe]05IFEP0c<c<0;bl_05MGE`11
@D40Cdm?030`<01GEeL0;Rh^03Xj>P25QHD0GUiN0410@01GEeL0GEeM0451@@2e]KD0C4a<07]kN`0e
=CD0:RXZ01XJ6P0Q8B40CTi>075aL@0n?Sh0=3@d061PH00c<c<0UiNG0:FUY@0g=cL0NGUi03@d=02;
Rh/00P1>CTh06`0[:b/0E5AD06][J`0`<300ADE506m_K`0c<c<0GEeM03hn?P1MGEd0CDe=02h^;P1Q
HF40AdM705UIF@1=CDd0<S8b04a<C01JFUX0Lg=c06=SH`1fMWH0Lg=c04=3@`1`L700EUIF06UYJ@02
069RHP2B0<_;b`1/K6`0>SXj06][J`0f=SH0FEUI09jNWP1cLg<0:b/[0861P@1@D500HF5Q06i^KP1G
EeL0GUiN08R8R01DE5@0Ti>C06YZJP1LG5`0MWIf055AD@1LG5`0UIFE0451@@1jNWX0XJ6Q0451@@1I
FET0^[Zj061PH02YZJT0gmoO09ZJVP0o?cl0=CDe06IVIP1WIfL0VIVI09fMW@1GEeL0@41004m?C`1=
CDd0ADE503/k>`1:BTX0SHf=0410@01CDe<0SHf=07emO@29RHT0D51@0492@P1lO7`0;Bd]06ATI01R
HV80>c/k08b<S01<C4`0?Shn07IfMP0O7al0WYjN05eMG@1jNWX0;bl_04Q8B01dM7@0Bd];04m?C`1J
FUX0Rh^;065QH@14A4@0I6AT096AT@2_[jl0Bd];02/[:`1QHF40MgMg04a<C02o_kl0IFEU02/[:`0a
<C40K6a/07QhN01EEED0E5AD07emO@0S8b<0@T9208>3P`1BDU80Vi^K09nOW`0m?Cd0HF5Q0451@@1E
EED0N7Qh06][J`0g=cL0>3Ph05iNGP0d=3@0IFEU0<?3``29RHT0Jf][09jNWP2SXj<0@d=305mOG`27
QhL0CDe=07inOP2TY:@0LW9b08j>SP1?Cdl0FUYJ03`l?02m_Kd0Ph>304Y:BP1`L700:BTY07YjNP2E
UID0MWIf0410@020P800GEeM014A4@0M7Ad0?clo07IfMP0i>CT0Lg=c06QXJ00U9BD>ooooo`07051@
D01gMgL0EUIF028R8P0b<S809RHV01LG5`0201DE5@0m01@D500C4a<05QHF01hN7P0b<S80K6a/07Yj
NP1HF5P0>SXj04U9B@23Ph<0b/[:0=7Ad@2IVIT0=cLg09ZJVP2;Rh/0Jf][04i>CP1MGEd0T92@0;>c
/`1/K6`0BTY:06=SH`24Q8@0ZJVY05EEE@21PH40YJFU05eMG@1CDe<0JVYZ09bLW01RHV80@D5103De
=@0_;bl0GemO08R8R01eMGD0GEeM059BDP0j>SX0:BTY03De=@0^;Rh0@D5103Ph>00b<S80<C4a02<S
8`0V9RH0CTi>0;Bd]02e]KD0RHV906EUI@2HV9P0BDU9038b<P0601DE5@805aLG00D0B4Q809BDU01A
DE40ATI607MgM`0201LG5`0<02DU9@0i>CT06ATI01XJ6P0K6a/07AdM01lO7`0L71`05aLG01dM7@2F
UYH0LW9b0P0P82003`0n?Sh0<C4a06=SH`2c/k<0O7al04E5A@0i>CT0G5aL06i^KP1bLW80V9RH08:2
PP0/;2`05ADE01@D500201<C4`0[02@T901eMGD0^KVi09BDU02SXj<0d];B0:JVYP0g=cL0B4Q804=3
@`1ZJVX0SHf=0;Zj^P1[Jf/0Ogmo071`L02UYJD0Lg=c02<S8`0D51@07AdM03/k>`1`L700BTY:0492
@P15ADD0Y:BT05aLG00g=cL0W9bL06YZJP1jNWX0d];B0;bl_026QXH0UiNG09VIV@2KVi/0alO707al
O01?Cdl061PH01DE5@0201LG5`1J03De=@0L71`06ATI02HV9P19BDT0A4A406EUI@1DE5@0D51@06i^
KP0X:2P06ATI01/K6`0M7Ad0IfMW08j>SP2][Jd0VIVI0=CDe02h^;P0[:b/0<S8b033`l<0E5AD03De
=@0[:b/09BDU034a<@0[:b/0;Bd]0<K6aP2j^[X0K6a/08R8R01lO7`0@d=303/k>`1FEUH0JVYZ07If
MP2^[Zh0ZZZZ0<K6aP3Lg=`0T92@08>3P`2WYjL0UiNG05YJFP2:RXX0:BTY06EUI@1VIVH0;Bd]01`L
700];Bd0KVi^04A4A00S8b<0?Shn06QXJ02;Rh/0Ng]k0861P@2@T900W9bL0:n_[`2OWil0>SXj034a
<@10@400:b/[01lO7`0K6a/0:BTY03/k>`1DE5@0A4A404I6AP1:BTX0R8R80;^k^`2i^KT0c/k>0=3@
d037alL0_kno08R8R01XJ6P0DU9B0P12@T806@11@D40CTi>07=cL`2>SXh0Ph>308V9R@27QhL0@D51
08J6QP3MgMd0e=CD0=[JfP3Cdm<0X:2P0:JVYP2i^KT0Vi^K0:RXZ033`l<0QHF507YjNP2g]kL0_Kfm
0:^[Z`1IFET04?ooool000kooooo0280CTi>04U9B@1?Cdl0IfMW06EUI@1QHF40?Shn08N7Q`25QHD0
<30`04E5A@11@D40F5QH06ATI00X:2P0DU9B02`/;01FEUH0>c/k055AD@1VIVH05ADE04e=C@0d=3@0
?clo04A4A00G5aL0F5QH06ATI00i>CT0BTY:02TY:@12@T80WIfM0P0g=cL09P15ADD0:b/[03dm?@0[
:b/0@D5104=3@`1mOGd0E5AD02/[:`0a<C40;Bd]06=SH`0o?cl0D51@05]KF`0_;bl06a/K04=3@`1G
EeL0>3Ph03De=@0I6AT0:b/[065QH@1jNWX0@41008>3P`0a<C4071`L04a<C01bLW80BTY:02XZ:P1=
CDd0<S8b07inOP0h>3P0Hf=S0P0H61P0bP1JFUX0F5QH03Ti>@1JFUX0@d=302PX:02ATI40EeMG08B4
Q00_;bl0:RXZ02<S8`1DE5@0Xj>S07EeM@0o?cl0<30`05YJFP0U9BD0NWYj08b<S00e=CD0MgMg065Q
H@1iNGT0DE5A06ATI015ADD0Cdm?07EeM@0m?Cd0DE5A08>3P`1HF5P0CTi>0410@01cLg<0MWIf0410
@01NGUh0C4a<069RHP1oOgl0De=C06ATI01:BTX0G5aL07YjNP24Q8@0EeMG03Ph>01]KFd0GemO07Ad
M00f=SH0>3Ph09jNWP1ZJVX0>c/k075aL@1ADE40A4A408F5Q@1dM7@0;bl_0::RXP1^KVh0MgMg08b<
S01XJ6P0SXj>0::RXP0n?Sh0S8b<06EUI@1IFET0OWin04A4A00k>c/0LW9b03/k>`1kNg/0];Bd0861
P@2BTY80d];B06ATI021PH40a/K608f=S@1]KFd0<C4a06EUI@1<C4`0MGEe09^KV`1nOWh0C4a<059B
DP0l?3`0GUiN03Ti>@0T92@0E5AD04I6AP1UIFD0P82004A4A01@D500<S8b06UYJ@2;Rh/0>SXj04U9
B@1LG5`0?Cdm09FEU@1[Jf/0@d=306i^KP0e=CD0]kNg0:f][@2GUiL0Jf][06YZJP1^KVh0<S8b034a
<@12@T80J6QX04i>CP1;Bd/0@D5105EEE@24Q8@0FUYJ02`/;01?Cdl0O7al04e=C@2[Zj/0NWYj05iN
GP0^;Rh0B4Q806=SH`0V9RH0FEUI07UiN@0m?Cd0A4A4096AT@0i>CT0F5QH08J6QP1CDe<0XJ6Q07Ee
M@1?Cdl0CTi>07MgM`0a<C40D51@09^KV`1:BTX0:RXZ06IVIP1:BTX0>3Ph08J6QP2GUiL0Lg=c08N7
Q`26QXH0>CTi04E5A@2=SHd0M7Ad0;fm_@0g=cL0@41002/[:`1dM7@0N7Qh04U9B@1gMgL0DU9B06i^
KP2<S8`0Ogmo04E5A@1mOGd0PX:201lO7`0S8b<09bLW07MgM`13@d<0Kfm_08R8R01XJ6P>ooooo`17
04Q8B01=CDd0Ng]k02@T900/;2`0<30`01LG5`0F5QH05ADE01@D500^;Rh0?Shn0492@P1:BTX092@T
01LG5`0`<300F5QH03/k>`1gMgL0a<C40:n_[`38b<P0L71`04Q8B01JFUX0Kfm_05YJFP0j>SX09BDU
06e]K@29RHT0Bd];03De=@0h>3P0GUiN08n?S`2MWId0dm?C08n?S`1NGUh0ADE503Hf=P0`<300>3Ph
04E5A@1IFET0Ph>30:NWY`28R8P0QhN705IFEP0P82007AdM01XJ6P0K6a/05ADE01dM7@1@D500KVi^
03`l?00c<c<0BDU907MgM`2CTi<0WYjN08>3P`2@T900?3`l03Xj>P0Y:BT00P0E5AD0400H61P05QHF
01LG5`0H61P05ADE01/K6`1FEUH0OWin059BDP1`L700;Bd]01LG5`0I6AT0ATI603<c<`0L71`201PH
600@01LG5`0K6a/071`L01TI6@27QhL0_;bl04i>CP0T92@09bLW05aLG02YZJT0`l?30;:b/P30`<00
W9bL06EUI@80NGUi07h0?Cdm03/k>`1WIfL0>c/k028R8P0H61P04a<C01PH602JVYX0][Jf0:>SX`2R
XZ80c<c<0:j^[P1oOgl0S8b<09RHV02LW9`0Vi^K0:FUY@1PH600DU9B07MgM`2@T900Ti>C0:^[Z`1G
EeL0>CTi02TY:@0T92@09BDU02h^;P22PX80R8R804a<C01fMWH0ZZZZ05mOG`2:RXX0bLW90;Fe]@1J
FUX0@4100451@@1TI6@0/K6a06IVIP0c<c<05aLG01DE5@0D51@05aLG02l_;`0S8b<061PH01TI6@0Q
8B40C4a<055AD@1cLg<0ZZZZ0;Jf]P1cLg<0=CDe02`/;00l?3`0Lg=c03/k>`1UIFD0clo?0>7Qh@3?
cll0R8R809:BTP2WYjL0VYZJ051@D00k>c/0?3`l02<S8`0G5aL0>CTi0:j^[P34a<@0QXJ606MWI`1H
F5P0EEEE07inOP2JVYX0VIVI09bLW02NWYh0PX:208n?S`2UYJD0b<S80:RXZ02n_[h0MGEe03@d=01/
K6`0=3@d07alO01NGUh07QhN01XJ6P0/;2`0DE5A06ATI00O7al08b<S09FEU@2WYjL0KVi^02`/;00Y
:BT0De=C09nOW`2n_[h0GEeM04E5A@0e=CD07AdM01hN7P0L71`201dM7@0=03dm?@2g]kL0_;bl0:VY
Z@2l_;`0`/;20;no_`33`l<0bLW908n?S`1MGEd0@D5103`l?00203lo?`0J04E5A@1;Bd/0K6a/04Q8
B00X:2P0>SXj03Ph>01YJFT0<c<c06IVIP2XZ:P0NGUi09>CT`1[Jf/0[:b/0=KFeP3<c<`0YjNW0<c<
c01fMWH0EUIF06][J`2LW9`0N7Qh07inOP0h>3P@ooooo`003_ooool07`1=CDd0DU9B03hn?P1NGUh0
K6a/04e=C@0U9BD0=CDe04Y:BP0d=3@0?clo02HV9P0S8b<0F5QH02l_;`1@D50071`L03Ph>00m?Cd0
@d=305MGE`0;2`/0=3@d03Xj>P11@D40B4Q803De=@0Z:RX0B4Q80451@@1;Bd/00P0E5AD0301LG5`0
=cLg03Ph>015ADD06a/K03@d=00P8200@41003De=@13@d<0?Shn02d];@80D51@0>L0MWIf04M7A`1<
C4`0LG5a05YJFP0e=CD0@T9205ADE00i>CT0;bl_05mOG`0l?3`0J6QX07UiN@17AdL0KVi^05mOG`0a
<C40ATI6061PH01>CTh0:2PX06ATI00m?Cd0GUiN03<c<`1OGel0G5aL02h^;P1UIFD0IVIV055AD@27
QhL0V9RH04M7A`2_[jl0H61P09jNWP15ADD0=cLg01lO7`0`<300PH6108V9R@0f=SH0BDU906][J`1M
GEd0Y:BT09RHV00d=3@0Ph>3075aL@27QhL0ADE506QXJ01:BTX0EUIF05aLG00Y:BT0?Shn06YZJP1N
GUh0C4a<03<c<`16ATH0K6a/03`l?01aLG40>SXj04U9B@1YJFT0DE5A05YJFP1aLG40M7Ad08N7Q`28
R8P0Ti>C04I6AP2;Rh/0GUiN07QhN02=SHd0@D510;Bd]01cLg<0<S8b07moO`1nOWh0EeMG09:BTP1M
GEd0?Cdm0:>SX`1WIfL0NGUi08V9R@1DE5@0LG5a0:b/[01_Kfl0WinO06m_K`1^KVh0P820065QH@17
AdL0G5aL02PX:01IFET0Ng]k07EeM@2DU9@0dM7A071`L02<S8`0f=SH0820P01ZJVX0GemO0:VYZ@24
Q8@0MgMg0:j^[P1^KVh0LW9b09BDU01QHF40=cLg04i>CP0`<300Fe]K03De=@1BDU80Ti>C05eMG@1^
KVh0BTY:03Hf=P24Q8@0FEUI06QXJ01WIfL0@4100::RXP20P800BDU908:2PP0_;bl0QhN704a<C01m
OGd0>CTi06i^KP2<S8`0Fe]K03hn?P18B4P0N7Qh04i>CP1NGUh0LG5a06=SH`1aLG40ADE503Hf=P20
P800Ogmo05=CD`2LW9`0Hf=S04=3@`0n?Sh0MGEe08:2PP1;Bd/0BTY:07MgM`0I6AT0AdM70:BTY01G
EeL0Hf=S075aL@0h>3P0GEeM06a/K01;Bd/0C4a<07IfMP0c<c<0;Bd]04e=C@1<C4`0:b/[071`L01_
Kfl0FUYJ08n?S`1gMgL0QXJ609jNWP21PH40Lg=c03lo?`2WYjL0NWYj0::RXP0g=cL0F5QH034a<@20
P800Rh^;04i>CP27QhL0K6a/08^;R`2MWId0TY:B055AD@22PX80QXJ602PX:00U9BD09bLW07UiN@16
ATH0MgMg0:>SX`28R8P03_ooool01P1FEUH0A4A4059BDP16ATH0;Rh^03Hf=P805aLG00H05ADE01dM
7@1<C4`0FEUI038b<P0I6AT301DE5@0[02DU9@1VIVH0WinO0<K6aP3<c<`0alO70<W9b@2;Rh/0AdM7
04=3@`1ZJVX0;2`/01PH600M7Ad0@d=307QhN01?Cdl0CDe=03Ti>@26QXH0dM7A0;bl_02<S8`0MGEe
08B4Q01TI6@0ADE502DU9@0R8R809bLW07QhN01lO7`0UIFE09^KV`1PH60092@T01`L700J6QX071`L
01DE5@0A4A40=CDe01hN7P0201DE5@0_02DU9@0a<C40E5AD08F5Q@2UYJD0TI6A02TY:@0Q8B40ADE5
02PX:00H61P05ADE01@D500H61P05QHF01DE5@0I6AT071`L07MgM`1fMWH0LW9b06IVIP0L71`09BDU
09:BTP2GUiL0Cdm?02@T900G5aL061PH01XJ6P0K6a/0>3Ph0:NWY`3EeMD0[Zj^09nOW`2MWId0][Jf
09jNWP3>c/h0bl_;0:6QX@2<S8`0Lg=c03Ti>@0I6AT00P0J6QX0:00O7al0ADE5069RHP24Q8@0De=C
05EEE@2BTY80^k^k08:2PP1<C4`0[Jf]09BDU01VIVH0CDe=0410@00n?Sh0J6QX08^;R`1SHf<0D51@
05eMG@2ATI40QXJ607=cL`2g]kL0@41003Xj>P13@d<0DE5A09NGU`38b<P0Hf=S07=cL`1OGel0PH61
05EEE@0];Bd0OGem075aL@0S8b<202@T900603Hf=P1UIFD0LG5a038b<P0E5AD05QHF0P0E5AD2028R
8P1501TI6@0H61P06ATI02LW9`2RXZ80`l?30<O7a`2JVYX0Xj>S0<W9b@3?cll0d=3@06UYJ@0b<S80
IVIV0;6a/@2WYjL0SHf=0:ZZZP1fMWH0Y:BT08V9R@1gMgL0G5aL04M7A`0l?3`0>CTi04a<C02<S8`0
`/;20:b/[01DE5@0Lg=c09^KV`21PH40Cdm?03De=@0V9RH0@T9203@d=01[Jf/0/;2`0<71`@2VYZH0
VYZJ071`L01PH600Ph>308N7Q`1ZJVX0=CDe01/K6`0J6QX09BDU04=3@`1ZJVX0>c/k05EEE@1PH600
>3Ph09BDU021PH40Rh^;0<;2`P3Rh^80gmoO0;Zj^P2EUID0;Bd]00806a/K01D071`L01dM7@0M7Ad0
;Bd]04Y:BP11@D40DE5A0:6QX@2DU9@0[Jf]0:BTY02^[Zh0SXj>038b<P0^;Rh0>3Ph04Q8B01QHF40
F5QH04m?C`0W9bL00P0L71`05@0R8R80;bl_05iNGP0m?Cd0E5AD09>CT`1PH600KFe]03`l?021PH40
`/;20;Bd]03Fe]H0X:2P03@d=00];Bd0=CDe061PH02@T900LG5a0410@00@ooooo`003_ooool0o`1?
Cdl0I6AT04A4A01OGel0JVYZ07=cL`17AdL0ATI605QHF00f=SH0HV9R05YJFP12@T80HF5Q02XZ:P1T
I6@0=CDe030`<00[:b/0>c/k069RHP0R8R80A4A403@d=018B4P0EeMG051@D00d=3@0C4a<030`<01K
Fe/0=SHf01lO7`2>SXh0>3Ph02@T901?Cdl0>c/k03`l?00Y:BT0G5aL05QHF01VIVH0?Cdm03@d=01O
Gel0D51@06YZJP0j>SX0F5QH06i^KP1HF5P0?clo04E5A@1^KVh0>3Ph03Lg=`1GEeL0>SXj06IVIP1J
FUX0<30`05IFEP1ZJVX0>SXj0410@01PH600@T9202LW9`1^KVh0>CTi05UIF@0f=SH0I6AT0410@00]
;Bd0HF5Q04U9B@0c<c<0CTi>06EUI@0i>CT0YJFU051@D01^KVh0Hf=S07MgM`1QHF40=CDe08J6QP1/
K6`0<S8b04M7A`1NGUh0CDe=092@T02TY:@0?Cdm06e]K@1ADE40PH6106YZJP1EEED0@41006EUI@1o
Ogl0F5QH03lo?`1bLW80E5AD065QH@1@D500EUIF06QXJ00e=CD0Ng]k04E5A@1?Cdl0IFEU04E5A@1J
FUX0Fe]K071`L01/K6`0LW9b09FEU@14A4@0U9BD04Y:BP1bLW80P82004m?C`2c/k<0QHF503Ph>02=
SHd0HF5Q05IFEP2QXJ40LG5a030`<01gMgL0DE5A07UiN@2=SHd0I6AT05=CD`1lO7`0G5aL0:^[Z`1O
Gel0EeMG075aL@1@D500=CDe08N7Q`13@d<0HV9R08J6QP1EEED0IfMW0<[:bP1dM7@0R8R80=[JfP21
PH40KFe]02d];@1]KFd0Kfm_06MWI`2SXj<0Lg=c04i>CP1@D500F5QH030`<014A4@0?3`l07moO`1D
E5@0@4100820P01/K6`0LW9b05]KF`0c<c<0QXJ602h^;P1IFET0H61P03De=@1kNg/0Fe]K038b<P1W
IfL08b<S0:FUY@1<C4`0Jf][04i>CP1MGEd0B4Q805UIF@0i>CT0E5AD07emO@1RHV80?3`l06IVIP1]
KFd0L71`05MGE`0X:2P0ATI606UYJ@0i>CT0UiNG06i^KP14A4@08b<S04A4A01_Kfl0Bd];04Q8B01^
KVh0Cdm?03Lg=`2LW9`0GUiN05QHF019BDT0@41009VIV@1eMGD0AdM704Y:BP1VIVH0BTY:03/k>`1I
FET0CDe=02@T901XJ6P0Lg=c071`L02_[jl0MWIf06=SH`2>SXh0Y:BT07IfMP0f=SH0/[:b05UIF@00
5`1^KVh0Kfm_07IfMP0m?Cd0MgMg08:2PP1ADE40UiNG07=cL`24Q8@0QhN706=SH`0/;2`0Vi^K08F5
Q@0W9bL09RHV02d];@1QHF40<30`07UiN@2:RXX0OGem00kooooo00l0OWin06ATI01>CTh0L71`0861
P@1hN7P0C4a<04A4A00_;bl071`L03Hf=P0O7al06ATI01HF5P0E5AD00`0D51@02`0F5QH0MgMg0;6a
/@2JVYX0XZ:R09VIV@0d=3@06QXJ01`L700o?cl0Ph>30080JFUY01L0BDU905QHF01RHV80G5aL04i>
CP1ADE40XJ6Q09:BTP11@D40<30`02<S8`0U9BD0<c<c04=3@`1FEUH0Ng]k09bLW02_[jl0V9RH04i>
CP1LG5`0?Cdm01hN7P0201`L700D01XJ6P0S8b<071`L01HF5P0G5aL092@T0451@@0X:2P0:b/[051@
D02][Jd0UYJF04m?C`0O7al061PH02PX:00`<3009bLW01dM7@0G5aL401HF5P1?020P801hN7P0UYJF
09>CT`1hN7P0?Shn03Xj>P0];Bd0=SHf04A4A01KFe/0J6QX075aL@1oOgl0Lg=c05aLG01_Kfl0[:b/
0=kNgP3Ph>00f=SH0=GEe@3:b/X0Zj^[092@T01@D500;Bd]01dM7@0E5AD06a/K01TI6@0F5QH06QXJ
01hN7P0l?3`0OWin0<W9b@2i^KT0Ng]k0:6QX@2e]KD0dM7A051@D00Z:RX0;bl_02XZ:P0N7Qh07AdM
01LG5`1<C4`0JFUY05IFEP1LG5`0O7al06EUI@2SXj<0d];B0<[:bP1]KFd0Z:RX0=3@d035aLD0SXj>
03Ph>01LG5`0Jf][02PX:00V9RH0De=C06EUI@1GEeL0;Rh^02PX:00`<300?clo08:2PP16ATH0820P
01HF5P0201DE5@1K01HF5P0H61P05aLG01TI6@0d=3@0KFe]07MgM`24Q8@0VIVI0;2`/02a/K40^[Zj
0;fm_@33`l<0Xj>S06YZJP26QXH0OWin051@D014A4@0BTY:07UiN@1nOWh0Cdm?03hn?P11@D40De=C
03Hf=P0i>CT0GUiN075aL@1^KVh0d];B0;:b/P1XJ6P0:BTY01`L700M7Ad06QXJ01/K6`0_;bl0PX:2
061PH01LG5`0Ogmo0861P@1UIFD0Jf][08F5Q@2?Shl0][Jf071`L00O7al06ATI01XJ6P0G5aL08R8R
0492@P29RHT0EEEE02XZ:P18B4P0YJFU08B4Q01]KFd0MGEe09JFUP1TI6@0GemO06i^KP0i>CT07QhN
01PH600I6AT07QhN034a<@11@D40E5AD04];B`19BDT0Fe]K07EeM@2CTi<0Hf=S0451@@2@T900JFUY
03dm?@0j>SX0<S8b028R8P0201dM7@8071`L01H06ATI01`L700N7Qh0>SXj03@d=00l?3`0Ph>309jN
WP2>SXh0Kfm_09jNWP2e]KD0ZZZZ0=;BdP2/[:`0DU9B03Ti>@0k>c/0GEeM0:FUY@1]KFd0Cdm?4?oo
ool000kooooo0800De=C06EUI@14A4@0MWIf06e]K@1oOgl0@T92069RHP1fMWH0?Cdm0451@@1>CTh0
C4a<06IVIP0i>CT0FEUI02d];@0`<300;Rh^038b<P1FEUH08B4Q0410@00W9bL0=cLg04i>CP0S8b<0
<S8b06IVIP0k>c/0EeMG028R8P0X:2P0WYjN06e]K@0^;Rh0DE5A03Hf=P1NGUh0;Bd]05YJFP1<C4`0
OWin071`L00d=3@0HV9R04I6AP27QhL0JFUY05UIF@1EEED0=SHf02HV9P0U9BD0HV9R051@D019BDT0
=CDe01hN7P1GEeL0L71`06EUI@0l?3`0PH6104I6AP0[:b/0MGEe079bLP0T92@0JFUY038b<P1oOgl0
Bd];05mOG`16ATH08R8R06a/K01[Jf/0?clo07QhN02KVi/0AdM70:6QX@1BDU80QXJ60::RXP2=SHd0
ADE502/[:`24Q8@0KFe]04i>CP2<S8`0V9RH06ATI02OWil0WYjN03<c<`1]KFd0O7al069RHP1SHf<0
IFEU04e=C@1GEeL0NWYj03`l?01MGEd0SXj>05aLG01:BTX092@T05EEE@25QHD0BDU9059BDP0X:2P0
?Cdm06e]K@1LG5`0De=C04e=C@1CDe<0Shn?05]KF`1`L700=CDe07=cL`80I6AT07L0=CDe030`<01h
N7P0Ogmo03Ph>01eMGD0Jf][04M7A`1oOgl0QhN703De=@1iNGT0I6AT06][J`2<S8`0MGEe04A4A01S
Hf<0MgMg0:NWY`1QHF40I6AT05IFEP1KFe/0>c/k07moO`0m?Cd0FEUI06a/K01HF5P0ZZZZ0=CDe01V
IVH0VYZJ0=kNgP1YJFT0Ng]k03Lg=`1`L700MWIf06i^KP2h^;P0TY:B065QH@1EEED0GEeM02/[:`1K
Fe/0<C4a08N7Q`1XJ6P0BDU90:2PX01`L700PH6107=cL`0c<c<0O7al05EEE@1>CTh0Lg=c03`l?01o
Ogl0NWYj04A4A01bLW80<c<c0:VYZ@2CTi<0YZJV07MgM`1ZJVX0BDU905=CD`0c<c<0HF5Q08>3P`1E
EED0=CDe04e=C@1BDU80LW9b04Q8B00c<c<0Fe]K069RHP0T92@0NGUi069RHP0[:b/0:RXZ04a<C01Q
HF409bLW03Hf=P1RHV80GUiN04E5A@2GUiL0DU9B05ADE01YJFT0Bd];0:b/[01UIFD0=CDe07AdM027
QhL0Hf=S03Lg=`1<C4`0>c/k0451@@2LW9`0X:2P079bLP2NWYh0N7Qh06EUI@0207moO`0K0451@@0[
:b/0YjNW03`l?00g=cL0JVYZ06i^KP0f=SH0MGEe07IfMP0W9bL0OGem06MWI`1oOgl0YZJV07]kN`1;
Bd/0WinO06QXJ00b<S80ADE504i>CP1^KVh0>CTi05]KF`2VYZH0>CTi00kooooo02/0DE5A05eMG@1@
D500J6QX08n?S`1eMGD0IVIV05YJFP1;Bd/0<S8b01<C4`0M7Ad05aLG01@D500@41004A4A01@D500C
4a<05ADE04];B`25QHD0JFUY06e]K@1_Kfl0=CDe01/K6`0D51@0410@01<C4`0T92@0<c<c01hN7P0S
8b<05ADE01LG5`1JFUX0<S8b034a<@1mOGd0YZJV05ADE00P820061PH00805ADE024051@D03Ti>@1K
Fe/0GemO0;bl_01fMWH0DU9B04a<C016ATH08B4Q01`L700W9bL0>CTi02`/;00W9bL08B4Q02DU9@2/
[:`0YZJV0:BTY02d];@0a/K60<71`@1dM7@0:BTY01`L700H61P05aLG01TI6@0e=CD0:b/[01HF5P0D
51@00`0E5AD0700Q8B40[Zj^0;^k^`2[Zj/0Shn?06EUI@24Q8@0PH6106YZJP1/K6`0IfMW05aLG00f
=SH07AdM01TI6@0E5AD07alO0:b/[03=cLd0Zj^[0=3@d02i^KT0EeMG0820P01/K6`0>CTi03`l?00h
>3P202/[:`0?02d];@0V9RH08b<S01lO7`14A4@0YJFU0;Jf]P1>CTh0ATI60451@@2YZJT0De=C02`/
;00F5QH0410@00805ADE01l051@D01PH601RHV80VYZJ0;Fe]@2ATI40_[jn0<O7a`2LW9`0/k>c06a/
K01mOGd0M7Ad069RHP23Ph<0PH6104e=C@0V9RH0820P028R8P0`<300Ng]k075aL@17AdL0<C4a04Q8
B01:BTX0J6QX04];B`1DE5@09bLW00<05ADE00d061PH038b<P1OGel0N7Qh04Q8B00X:2P0<C4a0:f]
[@34a<@0dM7A0=KFeP2g]kL0Zj^[0080c<c<01D0M7Ad03/k>`0T92@0820P01hN7P0a<C40C4a<03<c
<`0J6QX05aLG028R8P1>CTh0K6a/05iNGP2:RXX0I6AT0:ZZZP1_Kfl0BDU902d];@0L71`00`0M7Ad0
<01CDe<0]kNg096AT@1KFe/0?Shn02h^;P1OGel0FUYJ06][J`2f][H0_Kfm08j>SP0H61P05aLG01/K
6`0J6QX06ATI020P800o?cl071`L04=3@`30`<00DE5A04m?C`1IFET0@D51020P8012@T80;Rh^03Hf
=P0n?Sh0=cLg03lo?`0d=3@0CTi>06MWI`29RHT0H61P04=3@`0g=cL0HV9R06e]K@1ZJVX0KVi^06e]
K@2][Jd0F5QH01LG5`<061PH00@06ATI01PH600J6QX061PH0P0I6AT0500N7Qh0;Bd]038b<P0b<S80
QhN70;Jf]P2XZ:P0X:2P05]KF`1lO7`0_;bl0<o?c`1[Jf/0U9BD09JFUP23Ph<0M7Ad02TY:@0R8R80
>SXj4?ooool000kooooo0480De=C04i>CP11@D40EeMG07IfMP1jNWX09bLW03De=@18B4P0?Cdm04];
B`0X:2P0:b/[05ADE00b<S80IVIV01@D500^;Rh0?Cdm03De=@1jNWX051@D03/k>`0j>SX0C4a<07mo
O`0m?Cd07QhN04U9B@0m?Cd0L71`03Hf=P0E5AD0K6a/06QXJ00Y:BT0EeMG028R8P1GEeL0<S8b055A
D@0W9bL0?Shn04E5A@0g=cL0OGem03/k>`1WIfL0ATI606e]K@1eMGD0IVIV04U9B@0c<c<0E5AD02DU
9@0S8b<0C4a<03De=@1=CDd0EEEE03De=@1=CDd0SHf=034a<@0R8R8204M7A`2B02TY:@1gMgL0=3@d
05mOG`0e=CD0HV9R05eMG@0Y:BT0HV9R03dm?@0S8b<0@41006UYJ@11@D40SHf=03Xj>P1;Bd/0@T92
06UYJ@1ZJVX0<c<c07QhN01;Bd/0;2`/04=3@`1?Cdl0GEeM0:>SX`2[Zj/0Cdm?06=SH`1QHF40C4a<
075aL@1TI6@0BTY:051@D01eMGD0FUYJ0451@@1/K6`0FEUI04U9B@15ADD0B4Q806YZJP1:BTX0HV9R
03`l?00m?Cd0GUiN05QHF017AdL0DE5A07EeM@2FUYH0E5AD0861P@1IFET0Ti>C05]KF`1XJ6P0Lg=c
0492@P2OWil0T92@03@d=01`L700Ogmo05=CD`1bLW80N7Qh04m?C`2WYjL0QhN7069RHP22PX80JVYZ
03/k>`1YJFT0PX:209^KV`1BDU80FEUI05aLG01WIfL0@d=3079bLP10@400IFEU0:JVYP22PX80XZ:R
0=?Cd`1@D500L71`0;2`/02a/K40WinO03Xj>P1KFe/0;Rh^07YjNP31`L40XZ:R069RHP1BDU80GEeM
059BDP1IFET0;bl_07moO`1_Kfl0J6QX0:ZZZP1]KFd0U9BD07alO00j>SX0SHf=05aLG01EEED0N7Qh
034a<@2FUYH0WIfM0492@P1YJFT0>CTi07emO@22PX80[:b/08Z:RP1bLW80Bd];04e=C@0D51@06a/K
055AD@2CTi<0>CTi065QH@17AdL0/;2`07EeM@0g=cL206UYJ@0302DU9@1KFe/0GemO0080:BTY03T0
BDU905IFEP0V9RH0@D5107inOP1MGEd0@T9208>3P`0m?Cd0TI6A0;>c/`1>CTh0WYjN05QHF00H61P0
B4Q807IfMP1JFUX0?Cdm069RHP13@d<09BDU065QH@1TI6@0B4Q808J6QP2NWYh0S8b<08F5Q@29RHT0
<c<c03Hf=P2GUiL0HF5Q03De=@1@D500Cdm?02@T9022PX80U9BD04e=C@1`L7006ATI04Q8B01_Kfl0
Shn?051@D02f][H0S8b<020P800M7Ad092@T061PH00c<c<0Ti>C0;Vi^@1HF5P03_ooool06P19BDT0
>SXj04a<C01LG5`0QhN707inOP1DE5@0DU9B05MGE`0Y:BT0ATI6020P800C4a<05QHF01DE5@0C4a<0
410@018B4P0E5AD0D51@075aL@1bLW80KVi^06=SH`16ATH05QHF0`0@4100:P0>3Ph04Q8B010@400E
5AD051@D01<C4`0I6AT0C4a<02@T900l?3`0LW9b0:JVYP1>CTh08B4Q01PH600E5AD04a<C02l_;`2U
YJD0OWin0::RXP1SHf<0RXZ:08n?S`1KFe/0BTY:0451@@0b<S80:BTY02h^;P0V9RH0820P04e=C@2F
UYH0IFEU05EEE@1WIfL0XZ:R09VIV@2ZZZX0Cdm?01TI6@807AdM03<06QXJ01TI6@0a<C40=3@d028R
8P0M7Ad071`L02l_;`1XJ6P0Vi^K0=?Cd`3<c<`0Z:RX061PH01fMWH0Ph>306a/K01EEED0A4A403hn
?P0/;2`08B4Q020P800J6QX0=SHf0:ZZZP3@d=00R8R80;Bd]01nOWh08b<S03Ph>01HF5P0OWin09ZJ
VP1iNGT0De=C055AD@1CDe<0E5AD06EUI@1KFe/0R8R805UIF@22PX80@D5102TY:@0k>c/0WIfM0492
@P0F5QH00P0@41004`0E5AD051@D01HF5P0D51@0;bl_0820P037alL0TY:B0<K6aP2_[jl0IVIV06i^
KP2NWYh0Ti>C05EEE@1?Cdl0WinO08B4Q00O7al00`0M7Ad01`0O7al0Lg=c04e=C@1GEeL0;2`/03Ti
>@1@D5000P1NGUh0:01CDe<0H61P02l_;`0E5AD07AdM03lo?`1OGel0B4Q803@d=00V9RH08b<S034a
<@2UYJD0X:2P0:^[Z`2KVi/0[jn_0;Vi^@2NWYh0`L710492@P0Q8B407alO024Q8@0M7Ad0:RXZ04E5
A@0d=3@06ATI01/K6`0I6AT0820P05mOG`2/[:`0U9BD05EEE@1hN7P0@T9204i>CP0c<c<201/K6`1?
01dM7@0d=3@0HF5Q03Xj>P0i>CT0Ng]k06QXJ01;Bd/0B4Q80861P@2YZJT0b<S809fMW@2QXJ40@D51
01LG5`0K6a/05aLG01TI6@0M7Ad0<S8b02<S8`2>SXh0Hf=S05EEE@1@D500:b/[01XJ6P0L71`0:b/[
04M7A`1FEUH0=CDe0410@01bLW80^k^k0<C4a03Fe]H0e=CD08N7Q`17AdL0?Cdm09FEU@2RXZ80^;Rh
0;jn_P21PH40CDe=06e]K@0I6AT05QHF01TI6@0G5aL06ATI01PH600I6AT07QhN020P800L71`07AdM
0410@00j>SX0BDU90:FUY@2OWil0`L710<C4a02DU9@0W9bL0<[:bP38b<P0L71`051@D01WIfL0VYZJ
07MgM`0N7Qh08B4Q0451@@0@ooooo`003_ooool0b@1;Bd/0FEUI04=3@`1EEED0ATI606IVIP0k>c/0
@d=303Hf=P0L71`0<c<c04e=C@0e=CD0>CTi01/K6`1:BTX0<S8b01hN7P0H61P06QXJ05aLG00W9bL0
>CTi024Q8@0f=SH0EEEE04i>CP0K6a/0@410028R8P1NGUh0Bd];01LG5`1`L700E5AD01@D5010@400
<30`03/k>`0S8b<0FUYJ04];B`1ADE40Cdm?028R8P1CDe<0?Shn05mOG`16ATH0C4a<06][J`1TI6@0
BTY:03@d=00n?Sh0>SXj02HV9P0a<C40=3@d03`l?01<C4`0ATI602HV9P1YJFT0>3Ph01`L700f=SH0
D51@01`L701RHV80?Cdm05EEE@0o?cl0B4Q806][J`0_;bl0DE5A06a/K00l?3`0B4Q807]kN`13@d<0
PX:2055AD@1dM7@0De=C07=cL`1kNg/0A4A408Z:RP28R8P0D51@05EEE@1TI6@0JVYZ09fMW@2QXJ40
EUIF06UYJ@1gMgL0@d=306a/K01fMWH0Fe]K055AD@1dM7@0KFe]065QH@1gMgL0FUYJ04U9B@12@T80
D51@079bLP17AdL0HV9R0492@P0l?3`0DE5A059BDP1CDe<0G5aL06IVIP2<S8`0C4a<08^;R`1MGEd0
SHf=06m_K`1ZJVX0Ogmo04U9B@2BTY80UYJF03Xj>P27QhL0Y:BT04i>CP1nOWh0HF5Q03<c<`1aLG40
L71`05mOG`25QHD0ATI605]KF`2b/[80QXJ6096AT@1TI6@0Lg=c08f=S@27QhL0C4a<06m_K`0Y:BT0
De=C0:^[Z`2ATI40PX:20=CDe022PX80O7al0=3@d01UIFD0Ph>304A4A01/K6`0EEEE05=CD`2CTi<0
WYjN065QH@1BDU80>CTi02DU9@0n?Sh071`L06e]K@15ADD0I6AT0:j^[P1DE5@0XZ:R03hn?P0l?3`0
VIVI06=SH`1DE5@0LG5a020P802:RXX0U9BD0410@01]KFd0;2`/07inOP0l?3`0NWYj05mOG`0208R8
R00N07IfMP0j>SX0ATI6069RHP23Ph<0>SXj05aLG00e=CD0L71`07YjNP0c<c<0JVYZ05IFEP0Y:BT0
SXj>08>3P`1VIVH0?clo04Y:BP1CDe<04Q8B02h^;P1BDU80QhN703hn?P2IVIT0:BTY06IVIP2<S8`0
Bd];0P25QHD0:`0_;bl0AdM7065QH@0I6AT071`L03/k>`1EEED07alO05IFEP1?Cdl0=CDe05=CD`1?
Cdl0GemO07]kN`2d];@0Kfm_01/K6`1aLG40EEEE04I6AP2LW9`0LG5a03<c<`1KFe/0IVIV04];B`2T
Y:@0JFUY04U9B@1`L700IVIV01LG5`2CTi<0PX:202LW9`0/;2`09BDU05IFEP0N7Qh0EEEE09bLW01[
Jf/03_ooool03P1_Kfl0G5aL059BDP1hN7P0QHF507QhN00m?Cd0;Bd]04U9B@16ATH07QhN01TI6@0>
3Ph05ADE0P0B4Q80KP0?3`l03Ph>02TY:@1WIfL0Hf=S09>CT`1gMgL0GEeM07IfMP0M7Ad04Q8B00l?
3`0>3Ph03`l?00h>3P0=3@d04A4A01DE5@0B4Q8051@D07alO02HV9P0Lg=c08B4Q02@T900ZZZZ071`
L00S8b<04Q8B01LG5`1NGUh0fMWI0=GEe@38b<P0VYZJ09fMW@2a/K40ZJVY0820P01YJFT0IfMW04a<
C00j>SX0?Cdm075aL@1_Kfl0B4Q804I6AP1ZJVX0VYZJ09^KV`1/K6`0QXJ608^;R`0Z:RX05ADE01LG
5`0I6AT071`L03Lg=`2MWId0KFe]075aL@2EUID0UiNG04=3@`1aLG40ZJVY0;2`/03@d=00LG5a03lo
?`0d=3@0<C4a03Hf=P12@T80EEEE061PH01ZJVX0E5AD05]KF`2:RXX0WYjN0<G5a@2e]KD0_;bl05AD
E00I6AT08B4Q02HV9P0/;2`0?clo07UiN@2TY:@0AdM7038b<P0^;Rh0;bl_04];B`2QXJ40@D51055A
D@1;Bd/071`L034a<@2@T900>CTi02DU9@0D51@04a<C0P0E5AD09@0A4A4051@D01LG5`17AdL0MgMg
06i^KP31`L40DU9B02XZ:P0S8b<0@D5109^KV`1`L700IVIV04Y:BP2QXJ40@D5101hN7P0E5AD08B4Q
01lO7`16ATH0C4a<051@D00`<3006QXJ020P800m?Cd0P82004];B`14A4@0GemO05iNGP1OGel0DE5A
05IFEP0];Bd00P0N7Qh0D`0R8R80C4a<06QXJ01bLW80GUiN08Z:RP2SXj<0YJFU06ATI02<S8`0DU9B
01dM7@0P82008R8R01dM7@0X:2P0BDU903@d=00K6a/071`L01DE5@0J6QX0?clo071`L01NGUh0FUYJ
06MWI`0d=3@0Bd];02@T900V9RH07AdM03<c<`1@D500A4A402@T900S8b<0A4A403lo?`13@d<0Fe]K
06=SH`3IfMT0ZJVY07=cL`1RHV80NGUi02DU9@0J6QX06ATI01`L700Y:BT0;2`/06a/K02BTY80DE5A
0410@00S8b<071`L01TI6@0F5QH07QhN04Q8B01KFe/0SXj>05eMG@1GEeL0`L710;^k^`2WYjL0YZJV
0;Fe]@2i^KT0b/[:0=7Ad@2ZZZX0/K6a0:b/[019BDT0=CDe03dm?@0a<C4061PH00<05ADE01T061PH
01/K6`0T92@08B4Q01dM7@0S8b<0@T9204E5A@1bLW80RHV905EEE@2^[Zh0CDe=020P800j>SX0];Bd
06][J`2WYjL0P82006m_K`25QHD0EUIF01dM7@0O7al0A4A4013ooooo000>ooooo`3o05MGE`1WIfL0
@T9207inOP1GEeL0TY:B0492@P13@d<0B4Q8051@D01?Cdl0IFEU030`<00m?Cd0@T9205iNGP0g=cL0
820P024Q8@0Y:BT0EUIF01hN7P0e=CD06QXJ038b<P1;Bd/0ATI601dM7@0k>c/0<S8b05iNGP1:BTX0
61PH065QH@1MGEd08b<S0492@P0Z:RX0?clo02PX:01WIfL0C4a<04A4A01LG5`0>CTi05MGE`0b<S80
K6a/05iNGP1UIFD0KFe]06][J`1@D500=3@d03dm?@12@T80:RXZ02l_;`0k>c/0<S8b05ADE01=CDd0
;2`/06i^KP0];Bd0:BTY05IFEP1RHV808B4Q05]KF`0a<C40E5AD04E5A@16ATH0EUIF02l_;`17AdL0
I6AT03`l?017AdL0L71`0492@P2EUID0L71`08Z:RP1HF5P0KVi^034a<@0Z:RX0LW9b07MgM`11@D40
Fe]K05aLG01DE5@0Q8B40;Zj^P1[Jf/0F5QH08R8R01BDU80NGUi06MWI`1NGUh0FEUI06MWI`1UIFD0
FEUI06EUI@1LG5`0K6a/04m?C`1@D500IVIV03De=@26QXH0@T9204Y:BP1?Cdl0AdM704A4A01YJFT0
GEeM07YjNP0i>CT0V9RH05iNGP1/K6`0Lg=c06=SH`2UYJD0D51@08Z:RP2;Rh/06a/K06ATI028R8P0
D51@08F5Q@2CTi<0DU9B07QhN01RHV80P8200:JVYP1/K6`09RHV05ADE01SHf<0Ph>304E5A@1FEUH0
UiNG08j>SP1;Bd/0QHF504i>CP17AdL0]KFe04=3@`0n?Sh0alO70861P@22PX80dM7A05IFEP0N7Qh0
<C4a08Z:RP1bLW80HV9R0:>SX`23Ph<0HV9R09RHV01VIVH0;Rh^05iNGP0L71`0=3@d05]KF`0k>c/0
SXj>05IFEP2n_[h0De=C034a<@2LW9`0IVIV07AdM02;Rh/0@d=307=cL`2<S8`0>3Ph09>CT`0j>SX0
U9BD051@D029RHT0De=C0451@@1OGel0IfMW03hn?P1KFe/0Q8B408N7Q`0g=cL0PX:205IFEP2ATI40
EEEE02<S8`1SHf<0IFEU034a<@1@D500QhN704I6AP0/;2`0GemO06m_K`1EEED0@41004Y:BP0c<c<0
9bLW07EeM@1MGEd0IVIV07QhN00X:2P0FEUI06EUI@0W9bL0IFEU07alO01RHV80:b/[04];B`14A4@0
<C4a08N7Q`2^[Zh0LW9b07UiN@1:BTX0EUIF04E5A@24Q8@0KVi^020P801cLg<0G5aL000G02l_;`2Q
XJ40Ti>C04E5A@1RHV80L71`02DU9@1gMgL0JFUY04i>CP1hN7P0EUIF04=3@`1fMWH0HV9R01lO7`0d
=3@0;Bd]05MGE`1:BTX0@T920:n_[`1kNg/03_ooool0700o?cl0A4A405]KF`1eMGD0TI6A06ATI01K
Fe/0:2PX01dM7@14A4@0>c/k01HF5P0A4A405aLG010@400U9BD07QhN03De=@2HV9P0Xj>S07EeM@2k
^k/0P82006e]K@2RXZ80Kfm_02LW9`0E5AD200h>3P1P00d=3@0>3Ph0410@00l?3`0B4Q806a/K07Yj
NP22PX80UYJF0;Ng]`36a/H0S8b<09BDU01gMgL0<30`08>3P`34a<@0g=cL0;no_`1UIFD0ADE5051@
D01EEED0Cdm?06EUI@21PH40JVYZ071`L01aLG40Zj^[0;^k^`1eMGD0GemO06i^KP1>CTh0AdM706e]
K@2FUYH0/;2`07IfMP2OWil0Bd];04A4A01GEeL0FEUI05=CD`0j>SX09bLW030`<00m?Cd0C4a<05AD
E02:RXX0T92@09BDU02n_[h0D51@01`L700X:2P0:BTY01LG5`0M7Ad0:2PX03@d=00g=cL0PX:209^K
V`1<C4`0XJ6Q0=OGe`3@d=00aLG503Ti>@0G5aL06ATI01lO7`0K6a/07AdM02HV9P1TI6@06a/K01DE
5@0F5QH05ADE01PH601TI6@09BDU02LW9`1TI6@06QXJ02`/;02=SHd0;Bd]03`l?00E5AD04a<C0`0A
4A40B@0D51@04Q8B03dm?@1?Cdl0LG5a0:JVYP1dM7@0Q8B407YjNP1bLW80Shn?03@d=00T92@08R8R
03/k>`1[Jf/0>SXj01LG5`0K6a/0<C4a02LW9`1[Jf/0ADE505EEE@0M7Ad06ATI0492@P1BDU80NGUi
03lo?`1gMgL0/k>c04E5A@1SHf<0G5aL01lO7`0K6a/071`L01XJ6P1GEeL0Lg=c08J6QP1MGEd0A4A4
03Ph>01KFe/0IVIV07IfMP21PH408R8R01XJ6P0O7al09BDU04M7A`1UIFD0De=C03Xj>P0U9BD05aLG
02@T901DE5@0IFEU06=SH`1DE5@0Hf=S02`/;01:BTX07AdM0451@@0j>SX0@T9204a<C00W9bL00P0Q
8B404`0];Bd0=CDe02PX:00/;2`0ADE50;Fe]@1LG5`0Cdm?04M7A`1FEUH0E5AD01PH600K6a/071`L
03Hf=P21PH40Xj>S06=SH`0f=SH00`0L71`201XJ6P0701hN7P0];Bd0MWIf09bLW01MGEd0@41008n?
S`020:>SX`0<0;Ng]`31`L40/;2`0:6QX@2l_;`0ZZZZ08B4Q01ZJVX0EeMG03hn?P0^;Rh0>c/k0P0E
5AD06`0K6a/05ADE01PH600L71`08b<S01dM7@0P820092@T059BDP18B4P0N7Qh04I6AP1PH600KFe]
01`L700R8R80CDe=0::RXP1DE5@0NGUi0;Rh^022PX80H61P02l_;`0S8b<0;bl_03Ph>00@ooooo`00
3_ooool0@01MGEd0H61P04=3@`25QHD0CTi>08:2PP13@d<0SXj>0:2PX01LG5`0CDe=05]KF`0i>CT0
E5AD04I6AP1JFUX0=SHf038b<P0[:b/0<S8b05]KF`0N7Qh0?Cdm02XZ:P11@D40DU9B04Q8B00Z:RX0
Hf=S03Ph>011@D40?Cdm024Q8@1SHf<0MGEe038b<P15ADD0;Bd]03`l?00c<c<0IFEU04a<C01=CDd0
Ph>303lo?`1IFET0>3Ph08N7Q`1fMWH0K6a/07inOP1TI6@0F5QH05=CD`1RHV80JVYZ02`/;01<C4`0
ADE503/k>`2>SXh0Kfm_02h^;P1mOGd2020P801C0451@@1aLG409RHV05=CD`0_;bl0N7Qh061PH00n
?Sh0K6a/02TY:@1]KFd0Ph>30492@P16ATH0GUiN01`L701dM7@0GUiN07emO@2<S8`0QHF507MgM`1<
C4`0Ogmo06a/K01=CDd0SHf=08N7Q`1]KFd0Hf=S09>CT`1NGUh0BTY:07IfMP11@D40QHF505aLG00a
<C40OGem08b<S01_Kfl0Bd];05MGE`1<C4`0MgMg05EEE@1:BTX0K6a/04a<C01PH600ADE505MGE`1[
Jf/0Hf=S05ADE017AdL0@D5108^;R`1:BTX0IFEU02TY:@1VIVH0Ng]k051@D00o?cl0;Rh^07AdM02F
UYH0De=C06IVIP0g=cL0:RXZ05IFEP1eMGD0L71`09fMW@1WIfL0C4a<05=CD`1SHf<0;2`/065QH@1E
EED00P2BTY80<`1PH600N7Qh038b<P0N7Qh0Hf=S061PH01:BTX0/;2`03Xj>P1UIFD0clo?07=cL`27
QhL0c/k>07AdM01eMGD0:b/[05YJFP1TI6@0KVi^0;2`/02:RXX0EUIF04U9B@15ADD0@d=304E5A@0W
9bL0EeMG06m_K`11@D40MgMg03hn?P2`/;00O7al03/k>`2TY:@08b<S03/k>`1_Kfl0@41007IfMP25
QHD09bLW05eMG@0c<c<0UIFE0:6QX@2WYjL0Q8B407=cL`0204e=C@1:02<S8`1:BTX0JFUY09:BTP0f
=SH0NGUi03@d=02@T900Ng]k03@d=016ATH0B4Q801LG5`14A4@0Ng]k05YJFP0/;2`0;Rh^04a<C00f
=SH0HF5Q08V9R@1`L700@41007YjNP0h>3P0P8200;>c/`1RHV80KVi^06IVIP0B4Q80=3@d06IVIP1X
J6P0;Bd]04U9B@0l?3`061PH04=3@`1YJFT0Jf][08R8R01>CTh0Q8B404i>CP2/[:`0Hf=S03@d=025
QHD0IfMW01dM7@1VIVH0Hf=S03hn?P1kNg/0H61P03lo?`1aLG40K6a/05aLG02UYJD0JVYZ05=CD`1V
IVH0OGem024Q8@1?Cdl0G5aL07UiN@17AdL0?Cdm0;no_`1QHF4>ooooo`0U03@d=00Y:BT0;Rh^07Ee
M@24Q8@0GEeM08j>SP0f=SH07AdM028R8P1NGUh0F5QH02`/;00o?cl0DU9B05aLG01]KFd0L71`08Z:
RP3Cdm<0^;Rh0<g=c@2`/;00Ogmo09FEU@10@4005aLG02/[:`0@41003Ph>00d=3@0>3Ph03@d=00l?
3`0^;Rh0H61P04=3@`0202HV9P0e04A4A021PH40/K6a09FEU@2d];@0d];B0:f][@31`L40dm?C0;bl
_019BDT0;bl_03@d=01KFe/0?Shn04e=C@1QHF40=cLg02@T900b<S80WinO08>3P`1FEUH0@d=304i>
CP0X:2P0?3`l06][J`1aLG40Ti>C08j>SP1aLG40YJFU08n?S`1VIVH0D51@0410@00a<C40?clo051@
D01RHV80MGEe06i^KP2=SHd0_Kfm0:2PX03<c<`0UIFE02`/;00Y:BT05QHF01<C4`0F5QH00P0Q8B40
3@0b<S80G5aL079bLP2FUYH0Zj^[09jNWP3:b/X0^k^k05]KF`0>3Ph04a<C01/K6`0O7al00P0W9bL0
400k>c/0BDU901XJ6P0I6AT05ADE01LG5`1DE5@08B4Q01/K6`1_Kfl0:BTY034a<@2_[jl0<30`03Lg
=`0I6AT301<C4`0U00h>3P0C4a<051@D04A4A00i>CT0RXZ:07inOP1HF5P0=3@d02HV9P0N7Qh0>c/k
01LG5`0N7Qh08b<S02d];@0[:b/0D51@04Y:BP0G5aL0:BTY03`l?01ZJVX0B4Q8069RHP1@D50071`L
04U9B@17AdL0BTY:07YjNP2NWYh0[Zj^06MWI`0n?Sh08B4Q01/K6`0202<S8`0U02`/;01kNg/0H61P
05mOG`0/;2`0=CDe02`/;00d=3@0J6QX08Z:RP2g]kL0I6AT038b<P0`<300@D5105EEE@1]KFd0EeMG
051@D011@D40B4Q805UIF@1fMWH0G5aL08F5Q@1=CDd0Kfm_02h^;P19BDT0<S8b04M7A`14A4@0@D51
03hn?P0R8R80;bl_01dM7@0202LW9`0D01lO7`0R8R80:BTY0:f][@1GEeL0>SXj05aLG00U9BD0F5QH
04E5A@0I6AT0<c<c09nOW`2@T900KVi^04Y:BP0G5aL061PH01LG5`0H61P201LG5`0C01`L700O7al0
Ng]k0;no_`1DE5@0ATI607YjNP2d];@0]kNg0;Vi^@33`l<0`L710;Zj^P2OWil0QhN70;>c/`2MWId0
PX:2061PH00203`l?00M024Q8@0G5aL05aLG01DE5@0J6QX071`L01hN7P0M7Ad08b<S02/[:`1TI6@0
O7al059BDP1;Bd/0RXZ:02DU9@0L71`0;Bd]0861P@1iNGT0BDU9071`L02UYJD0]KFe06a/K00a<C40
<30`02h^;P0g=cL04?ooool000kooooo00H0De=C05mOG`0k>c/0IVIV059BDP27QhL203`l?00Q04a<
C01ADE40BTY:051@D00];Bd0DU9B03Xj>P1=CDd06a/K02d];@0a<C408R8R06a/K00O7al0<C4a03Xj
>P0f=SH0I6AT04I6AP0Q8B40@41003`l?018B4P0<c<c020P801FEUH0MgMg02l_;`1>CTh0:b/[03hn
?P0[:b/0D51@0080;bl_05<0EUIF02PX:01DE5@0?Cdm06a/K01;Bd/0G5aL06e]K@0m?Cd0=cLg03hn
?P1FEUH0ATI602h^;P1BDU80>CTi02@T901>CTh0CDe=02@T901ZJVX0AdM702LW9`0n?Sh0>c/k01/K
6`1EEED0<C4a04U9B@15ADD0:b/[05mOG`0f=SH0A4A405MGE`0Z:RX0@D5107AdM01HF5P0Q8B403@d
=013@d<0DE5A06=SH`1LG5`0>3Ph08n?S`1aLG40?3`l0451@@0n?Sh0FUYJ09nOW`2l_;`0@41003Ph
>01iNGT0DE5A05YJFP14A4@0BDU9055AD@1VIVH0@41003Xj>P1GEeL0E5AD07moO`0];Bd0>SXj06a/
K01CDe<0HV9R03<c<`0e=CD0@d=304E5A@1:BTX0GemO03hn?P1_Kfl0=cLg08N7Q`0204e=C@2F08>3
P`1BDU80L71`04M7A`1nOWh0VYZJ03Ti>@1XJ6P0UIFE04Q8B01MGEd0Ng]k05=CD`1NGUh0K6a/07If
MP1iNGT0Fe]K02XZ:P1UIFD0L71`06m_K`1aLG40JFUY09bLW025QHD0DU9B069RHP14A4@0De=C0;>c
/`1nOWh0Rh^;0>3Ph01RHV80IfMW0:VYZ@21PH40R8R804a<C02:RXX0DU9B06ATI02FUYH0WYjN08B4
Q02RXZ80EUIF04I6AP1]KFd0820P02@T9017AdL0G5aL09>CT`1ADE40T92@05aLG00f=SH0YJFU06AT
I01jNWX0QHF5020P801WIfL0UYJF04E5A@1XJ6P0:BTY06YZJP1EEED0Jf][05YJFP1ZJVX0FUYJ065Q
H@0^;Rh0Bd];05=CD`1eMGD0;2`/05mOG`0_;bl0G5aL04Y:BP0j>SX0Fe]K06ATI00];Bd09BDU06=S
H`1LG5`0@T9203De=@1JFUX0?Shn03Ph>01ADE40O7al04I6AP28R8P0ADE505]KF`1dM7@0J6QX05aL
G01gMgL0C4a<05ADE01PH600G5aL02l_;`1BDU80Kfm_02<S8`13@d<0GemO05ADE01SHf<0H61P08Z:
RP15ADD0]KFe05iNGP0U9BD0AdM706a/K00[:b/0OWin05mOG`0i>CT0Ogmo08F5Q@1<C4`0JVYZ05YJ
FP15ADD0QHF506EUI@1IFET0GemO06m_K`0S8b<09bLW02XZ:P1IFET0C4a<03Xj>P2d];@0MgMg3_oo
ool07P0g=cL0;2`/02LW9`1PH600Cdm?06YZJP2WYjL0VYZJ07emO@1OGel0K6a/08>3P`1nOWh0L71`
05mOG`1MGEd0EUIF04e=C@1HF5P0GemO09jNWP3AdM40b<S806IVIP2;Rh/0TI6A01lO7`0M7Ad08b<S
030`<080;bl_00D0<S8b03@d=00^;Rh09BDU02<S8`0201hN7P0?028R8P0Y:BT0Kfm_07YjNP2d];@0
fm_K0<30`02f][H0X:2P09RHV02NWYh0J6QX05mOG`26QXH0SHf=0080N7Qh04L0C4a<03@d=00[:b/0
@D5107=cL`18B4P0?Shn03Xj>P12@T8092@T01/K6`0I6AT09BDU09FEU@2m_Kd0Xj>S07inOP2KVi/0
TI6A05ADE015ADD0>c/k03Hf=P0i>CT0BDU908F5Q@2BTY80T92@0;Rh^02UYJD0GUiN03dm?@1=CDd0
ATI603hn?P0b<S80:RXZ028R8P0`<300NGUi0:f][@2^[Zh0EeMG05ADE02_[jl0];Bd07UiN@0i>CT0
5aLG01TI6@0M7Ad08R8R02DU9@0V9RH0<S8b03Xj>P0h>3P0<c<c05=CD`1>CTh07AdM01PH601aLG40
U9BD0:JVYP2j^[X0QXJ604A4A00];Bd051@D018B4P0301<C4`1^02XZ:P2ATI40LW9b0:RXZ01GEeL0
<S8b02/[:`0T92@0;2`/03Ph>00Z:RX09bLW02DU9@0X:2P0;2`/03Lg=`1?Cdl0Fe]K02/[:`0V9RH0
EEEE06][J`2<S8`0bl_;0:ZZZP24Q8@0OGem05mOG`2RXZ80Zj^[07IfMP11@D40>3Ph03De=@0[:b/0
:2PX04=3@`1iNGT0M7Ad07alO01iNGT0>c/k03Hf=P0_;bl0;Rh^05UIF@37alL0ZJVY07inOP1dM7@0
OGem08N7Q`1/K6`0D51@04A4A010@400DU9B03`l?00o?cl0FEUI09bLW02][Jd0Jf][0;6a/@2ATI40
QhN70;Vi^@2NWYh0P820092@T01DE5@0CDe=05ADE01CDe<0EeMG05mOG`1aLG40Q8B40:^[Z`3Kfm/0
_kno061PH01_Kfl0Xj>S05aLG01jNWX0YJFU0;Zj^P2?Shl0LW9b04e=C@0M7Ad05QHF01XJ6P0G5aL0
6ATI01LG5`0H61P06ATI02TY:@1YJFT0]KFe0;Bd]02b/[80YjNW0<c<c02YZJT0TI6A07YjNP1ZJVX2
051@D00906=SH`1LG5`0KVi^07UiN@1jNWX0J6QX03`l?015ADD09BDU00806ATI01T06QXJ01lO7`0P
8200:BTY04Y:BP24Q8@0[Jf]0:JVYP1YJFT0O7al03Xj>P0S8b<09bLW03Lg=`2@T900B4Q805MGE`1E
EED0O7al04U9B@1mOGd0Ng]k055AD@16ATH0C4a<013ooooo000>ooooo`1;05aLG01/K6`0@41006IV
IP0e=CD0CTi>03Ti>@1DE5@0AdM702PX:00f=SH0F5QH03lo?`0i>CT04a<C028R8P0f=SH07alO01/K
6`0?3`l0<S8b0451@@0b<S80;Rh^01TI6@0a<C40BTY:02HV9P0_;bl071`L030`<01ADE408B4Q04a<
C01eMGD03Ph>02HV9P0m?Cd0;Bd]018B4P0o?cl0De=C0410@01HF5P05aLG03De=@0j>SX0H61P04M7
A`0l?3`0GUiN05]KF`17AdL0Cdm?05]KF`14A4@0:b/[04e=C@1FEUH0<S8b051@D00k>c/071`L05eM
G@0h>3P08b<S05EEE@1MGEd09BDU04];B`0d=3@0DU9B05MGE`0X:2P0IVIV0080@41001`0IVIV04M7
A`0n?Sh0HV9R03/k>`1hN7P0Lg=c05EEE@1KFe/0JFUY034a<@0K6a/0MGEe08>3P`1>CTh0EeMG0492
@P1YJFT0IVIV0;6a/@1XJ6P0?clo07emO@17AdL0Xj>S06UYJ@0R8R80BTY:0P1jNWX0Z`10@400De=C
03De=@29RHT0MGEe051@D01iNGT0<C4a06=SH`1EEED0C4a<04U9B@13@d<0=CDe05iNGP1>CTh0O7al
04i>CP24Q8@0L71`04I6AP2KVi/0DE5A07=cL`15ADD0OWin09nOW`17AdL0@41006i^KP1LG5`0VIVI
0:2PX01UIFD0Jf][05YJFP2DU9@0]kNg07moO`0e=CD0H61P075aL@1nOWh0NGUi04E5A@1NGUh0RHV9
05=CD`1]KFd0HF5Q03lo?`2FUYH0>c/k03De=@3?cll0O7al07]kN`2/[:`0FEUI03Hf=P0j>SX0P820
06m_K`1/K6`0TI6A09ZJVP0m?Cd0/K6a05YJFP12@T80IFEU03De=@0h>3P0K6a/030`<028R8P0IVIV
06YZJP1]KFd0;Bd]09:BTP1JFUX0Rh^;08Z:RP17AdL0Bd];08j>SP0^;Rh0OGem03hn?P1hN7P0HV9R
06][J`1QHF40ATI606EUI@1`L700B4Q806EUI@1QHF40WYjN03@d=023Ph<0C4a<06m_K`11@D408B4Q
059BDP1aLG40?Cdm02HV9P1`L700?Shn02DU9@19BDT0LW9b071`L01BDU80BTY:05=CD`0J6QX0WinO
06EUI@1IFET0OWin03Xj>P1LG5`0UIFE05]KF`1@D500E5AD061PH00`<300B4Q8079bLP0X:2P0?Cdm
07MgM`28R8P0KFe]04Y:BP27QhL0C4a<0<?3``20P800;Rh^04a<C022PX80:RXZ09VIV@1bLW80EEEE
07moO`20P800DU9B07UiN@1^KVh0CDe=09nOW`1=CDd0;bl_06UYJ@1iNGT0=SHf02/[:`0e=CD0I6AT
04i>CP0e=CD0YZJV096AT@0>ooooo`0P03Ph>00Y:BT0:BTY03/k>`2JVYX0CDe=0;Vi^@28R8P0TI6A
08n?S`1nOWh0RHV90;Fe]@34a<@0]kNg0:b/[01kNg/0JVYZ096AT@2HV9P0bLW90=oOg`2@T900P820
0=OGe`3Kfm/0bLW908Z:RP1DE5@09RHV01PH600E5AD201PH601<01HF5P0J6QX05aLG01PH600I6AT0
6QXJ01dM7@1HF5P0`L710;6a/@37alL0QHF504];B`11@D40Ogmo0<c<c032`/80Kfm_05eMG@2RXZ80
UYJF08n?S`24Q8@0Kfm_05IFEP0X:2P0FUYJ03Ph>00P82008b<S01dM7@0L71`05ADE014A4@0G5aL0
IVIV09ZJVP31`L40M7Ad06=SH`29RHT0VYZJ02LW9`0K6a/05aLG01/K6`0o?cl0De=C08F5Q@3Kfm/0
bLW904E5A@1:BTX05aLG01DE5@0H61P0;2`/03lo?`1MGEd0Y:BT0:^[Z`1BDU80Y:BT08V9R@1iNGT0
QhN704Y:BP1UIFD0TI6A08B4Q01KFe/0820P01XJ6P0L71`07AdM01lO7`808R8R06d07alO02@T9016
ATH0DE5A02l_;`1QHF40/k>c0:j^[P10@4008R8R07IfMP1QHF40JVYZ05=CD`0a<C406a/K014A4@0^
;Rh0FEUI05iNGP32`/80a/K605iNGP0o?cl0Ogmo07emO@1KFe/06QXJ01DE5@0B4Q808b<S02@T900U
9BD0:BTY02l_;`12@T80FEUI01LG5`0a<C40d];B0<[:bP2k^k/0NGUi04=3@`1:BTX0SXj>0:ZZZP2:
RXX0DE5A04i>CP1;Bd/0EUIF07]kN`2XZ:P0PX:207IfMP1_Kfl0BTY:03Ti>@0`<300@d=304U9B@13
@d<0R8R80::RXP2DU9@0L71`065QH@1=CDd0F5QH092@T02=SHd0UYJF04=3@`0Y:BT09bLW038b<P19
BDT0`l?308b<S02>SXh0SHf=08V9R@2:RXX0dm?C0<W9b@2UYJD0Vi^K0:FUY@21PH40IVIV07emO@1l
O7`0MgMg079bLP1lO7`0_[jn0=3@d022PX80/[:b0<c<c02OWil0IFEU0:^[Z`3Ti>@0cLg=09>CT`0a
<C40@41000<05aLG0`0E5AD03P0G5aL061PH061PH02CTi<0=SHf07YjNP34a<@0NGUi0<O7a`26QXH0
De=C03De=@0Z:RX0:b/[0P0];Bd08@0f=SH0CDe=05QHF01RHV80EUIF05eMG@1?Cdl0N7Qh03lo?`0d
=3@0=SHf03hn?P1RHV80LW9b07EeM@2XZ:P0_Kfm0;Ng]`2OWil0Bd];020P800W9bL0;Rh^05EEE@2I
VIT0BDU9065QH@1LG5`0A4A403dm?@16ATH0D51@020P800201lO7a3ooooo000>ooooo`1n06i^KP29
RHT0ADE508>3P`10@400LG5a03Ti>@1OGel0Ng]k04=3@`0];Bd0BTY:03@d=018B4P0EEEE03hn?P1C
De<09BDU04U9B@0Y:BT0BTY:0410@00c<c<0Cdm?02LW9`19BDT0GUiN034a<@0e=CD0CDe=03Lg=`1;
Bd/051@D04M7A`1^KVh0<C4a03Lg=`0];Bd0=cLg034a<@1>CTh0?clo03hn?P22PX80=3@d03Ti>@0e
=CD0Fe]K07IfMP1GEeL0NWYj020P800M7Ad0CDe=07AdM01WIfL0<c<c05]KF`16ATH0;Rh^06a/K01a
LG40;Bd]065QH@0j>SX071`L06ATI022PX80=3@d04M7A`0[:b/0Bd];06QXJ00M7Ad0KVi^02h^;P1<
C4`0Ng]k04I6AP0e=CD0FEUI02XZ:P1JFUX0GemO05ADE01VIVH0Ogmo0861P@1PH600OWin07=cL`1B
DU80Ng]k06m_K`1VIVH0Cdm?08J6QP1[Jf/0ADE508:2PP0k>c/0OGem06IVIP0m?Cd0<c<c05EEE@1[
Jf/0BTY:07AdM01LG5`0Ng]k06YZJP1EEED0RHV906YZJP1PH600E5AD06i^KP24Q8@0MGEe04i>CP1U
IFD0CTi>09VIV@1PH600NWYj0P1=CDd0UP2HV9P0H61P05YJFP0h>3P0OGem0:6QX@1bLW80@T9204e=
C@0e=CD0EUIF07MgM`1eMGD0IFEU05YJFP1]KFd0H61P069RHP0a<C40C4a<05IFEP1OGel0UiNG05iN
GP1bLW80EEEE02l_;`1PH600Ng]k04Y:BP2YZJT0Ph>306QXJ03Bd]80RXZ:096AT@2XZ:P0UiNG09JF
UP0h>3P0NGUi08F5Q@1SHf<0Q8B408n?S`1EEED0/[:b061PH019BDT0Cdm?02l_;`1FEUH0JVYZ030`
<01aLG40<S8b06EUI@1mOGd0>c/k09ZJVP1>CTh0IVIV092@T01PH600CDe=0:BTY018B4P0Hf=S03Lg
=`1[Jf/0L71`06QXJ01dM7@0FUYJ05QHF02BTY80Fe]K07MgM`1VIVH0Y:BT03/k>`22PX80De=C06MW
I`15ADD0=CDe04];B`1`L700A4A402HV9P1RHV80DU9B04=3@`0Y:BT0G5aL075aL@1ADE40B4Q8069R
HP0W9bL0Kfm_05aLG01EEED0MgMg03Lg=`13@d<0M7Ad05mOG`1LG5`0E5AD08N7Q`17AdL0D51@069R
HP0a<C40FUYJ08R8R01gMgL0Fe]K05=CD`1YJFT0=cLg08n?S`1^KVh0?Cdm05=CD`22PX80;2`/0::R
XP2BTY80D51@071`L023Ph<0B4Q808V9R@1_Kfl0A4A40:JVYP1FEUH0A4A4059BDP1hN7P0>CTi05UI
F@1YJFT0KVi^02h^;P0_;bl0Ti>C08:2PPkooooo02<0=CDe02XZ:P0Y:BT0?clo0=;BdP3=cLd0`<30
055AD@15ADD0LW9b08N7Q`3@d=00e=CD0<g=c@1jNWX0L71`09>CT`33`l<0^;Rh0<30`031`L40`<30
0;Fe]@2WYjL0PX:206a/K02i^KT0KVi^03Xj>P0N7Qh061PH01TI6@0E5AD07QhN01dM7@0201`L70<0
6ATI01L0;Bd]079bLP1fMWH0RHV90;Rh^02:RXX0G5aL05]KF`1aLG40VIVI0<o?c`2JVYX0P82009>C
T`2SXj<0/;2`0:j^[P0R8R808b<S02LW9`0l?3`0@T9202HV9P0201hN7P0G01/K6`0O7al08B4Q06EU
I@3Cdm<0YJFU0=;BdP2j^[X0WinO096AT@2SXj<0YjNW0:^[Z`2<S8`0PX:208F5Q@1kNg/0L71`09FE
U@36a/H0K6a/03Xj>P0a<C400`0E5AD0H00G5aL0;Bd]0:n_[`1fMWH0RXZ:0<?3``2][Jd0`<3006m_
K`0G5aL0:b/[096AT@2RXZ80UIFE06][J`0P82006a/K01`L700H61P071`L020P800];Bd0>SXj08>3
P`2=SHd0NGUi0;>c/`1/K6`0H61P03Hf=P0L71`0?3`l071`L01NGUh0DU9B0410@01=CDd0Cdm?06AT
I018B4P0De=C0:^[Z`2a/K40XJ6Q05aLG01bLW80Ph>308J6QP1:BTX06ATI01LG5`0I6AT092@T01lO
7`0Y:BT0>CTi05EEE@1gMgL0Q8B409fMW@2SXj<0PH6107moO`2<S8`0VIVI0:n_[`35aLD0clo?07If
MP1MGEd0O7al07UiN@1cLg<0PX:20<K6aP2m_Kd0@D5106][J`0k>c/0FUYJ02<S8`0Q8B40:b/[03Lg
=`29RHT0?Shn0410@01kNg/0Ph>306][J`17AdL0F5QH08R8R03EeMD0Q8B404M7A`80:2PX02@0EEEE
0:j^[P22PX80[:b/0<O7a`3GemL0a/K60;Ng]`25QHD0BDU904E5A@1OGel0GEeM02`/;00n?Sh0Bd];
055AD@1jNWX0Y:BT08^;R`1SHf<0YJFU0:BTY038b<P0[:b/05YJFP3;bl/0d=3@0=GEe@36a/H0JFUY
04Q8B00F5QH05aLG01DE5@0G5aL201DE5@0901TI6@0Z:RX0[jn_069RHP0P8200FEUI0:VYZ@1>CTh0
KFe]0080>SXj02D0@D5104];B`16ATH0=3@d02`/;00`<300?Cdm04A4A01BDU80KFe]06QXJ02MWId0
a/K60<71`@2KVi/0OWin06i^KP1VIVH0OWin0:ZZZP2UYJD0WinO09:BTP2MWId0>c/k04A4A00c<c<0
>SXj07AdM02EUID0CTi>09JFUP1cLg<0Cdm?038b<P16ATH0ADE500<071`L4?ooool000kooooo00X0
H61P06ATI00g=cL0H61P03Ti>@1JFUX08B4Q01lO7`13@d<0B4Q80P0c<c<0R`0=3@d0?Cdm05iNGP0h
>3P0=CDe03Lg=`1KFe/0<c<c04E5A@0];Bd0=3@d07UiN@0X:2P0BDU905iNGP0<30`0=SHf06EUI@0k
>c/0Cdm?01PH601LG5`0UiNG04Q8B00i>CT05ADE02XZ:P12@T80@d=3018B4P0M7Ad0HV9R03Hf=P0i
>CT0=cLg04I6AP1aLG40F5QH07EeM@0];Bd07AdM02XZ:P0o?cl0CDe=03Hf=P1mOGd0;Rh^01LG5`16
ATH0IfMW03Lg=`1lO7`0Cdm?01DE5@0f=SH0Fe]K03Lg=`1UIFD0?Shn03dm?@1RHV809bLW06MWI`0n
?Sh0:2PX06][J`11@D40ADE507inOP1dM7@0K6a/04U9B@0h>3P0C4a<05eMG@1kNg/0Fe]K092@T01K
Fe/0=SHf03/k>`0o?cl0Jf][09>CT`34a<@0JVYZ03Xj>P28R8P0De=C06MWI`19BDT0G5aL04Y:BP1X
J6P0@410030`<019BDT0F5QH08b<S018B4P0<C4a075aL@1YJFT0EUIF03Hf=P0n?Sh0A4A4055AD@17
AdL0F5QH04E5A@1nOWh0Fe]K06i^KP1FEUH0?Shn07inOP1MGEd0HF5Q04M7A`1LG5`0WinO07EeM@0g
=cL0NWYj04U9B@1YJFT0RXZ:06i^KP1UIFD0LW9b09BDU02=SHd0TI6A03Xj>P1?Cdl0IfMW0451@@2T
Y:@00P1MGEd05P2NWYh0KFe]07MgM`2;Rh/0Cdm?09nOW`2HV9P0MgMg0:f][@28R8P0U9BD0;6a/@2R
XZ80XJ6Q0451@@1nOWh0P82005]KF`24Q8@0WinO05MGE`2ATI4204A4A01U071`L00e=CD0<C4a06EU
I@0[:b/0EeMG04e=C@12@T80Ogmo03Xj>P2>SXh0P8200:NWY`2a/K40K6a/05=CD`2h^;P0Fe]K07Qh
N010@400I6AT08F5Q@1cLg<0PH6106QXJ01EEED0X:2P06ATI01cLg<0Fe]K0:6QX@0k>c/0Ng]k06AT
I01RHV80AdM703dm?@1BDU80LW9b03Ph>00O7al0JFUY065QH@1:BTX0<30`06IVIP26QXH0EeMG04M7
A`1YJFT0<30`08b<S028R8P0D51@07inOP1TI6@0D51@07emO@1EEED0Jf][07EeM@23Ph<0ADE505EE
E@1?Cdl092@T03Ph>01ZJVX0H61P04Y:BP1_Kfl0QhN703`l?02TY:@0EeMG038b<P1HF5P0LG5a01`L
701aLG40EUIF03hn?P23Ph<0SXj>04Y:BP28R8P0De=C04=3@`2e]KD0Shn?06=SH`1GEeL0NGUi03<c
<`0M7Ad0:b/[05ADE01;Bd/05QHF07]kN`1NGUh03_ooool04P11@D40B4Q805mOG`1YJFT0Ti>C0=7A
d@2n_[h0YJFU06=SH`1_Kfl0O7al0;bl_03>c/h0aLG509:BTP24Q8@0UYJF07moO`80NGUi0240FEUI
06IVIP2`/;00RXZ:04=3@`0d=3@0LW9b0410@01KFe/09RHV01hN7P0I6AT06a/K03dm?@18B4P0Bd];
04Y:BP1ADE40IfMW07inOP2h^;P0UIFE04Q8B033`l<0dm?C08^;R`17AdL0>3Ph04m?C`2FUYH0De=C
020P800e=CD00P0/;2`0<@0b<S80J6QX05IFEP0S8b<0820P0451@@0o?cl0@d=303De=@18B4P0@d=3
065QH@2KVi/0ZZZZ0;^k^`2LW9`0MWIf0820P02?Shl0ZZZZ0:^[Z`35aLD0OWin061PH01XJ6P0FEUI
03hn?P0o?cl0@d=307]kN`29RHT0DE5A034a<@0K6a/051@D01DE5@0F5QH0DU9B0492@P0U9BD0=SHf
07MgM`1OGel0GUiN07AdM00G5aL06ATI07IfMP3IfMT00P2f][H03`1XJ6P0CTi>04a<C014A4@0AdM7
04i>CP1DE5@0EUIF06=SH`1MGEd0Zj^[07EeM@1_Kfl0Y:BT04m?C`0201/K6`0:04Q8B01/K6`0GEeM
07inOP1IFET0EUIF07emO@2:RXX0]KFe0:FUY@80QHF50100HF5Q07EeM@1?Cdl0>CTi04E5A@19BDT0
;Rh^01@D500T92@0GUiN069RHP1gMgL0Q8B408^;R`2o_kl0Y:BT0P1iNGT0301jNWX0Q8B407EeM@2k
^k/0^KVi0<c<c037alL0L71`0451@@17AdL0Hf=S08N7Q`80b<S803P0Rh^;04M7A`0`<300ADE503dm
?@0K6a/09BDU07IfMP1^KVh0@T9204E5A@1eMGD0PX:208J6QP1LG5`0>SXj03Ti>@2b/[80_;bl08n?
S`1/K6`0=CDe07inOP2=SHd0GUiN08b<S03Jf]X0cLg=0;>c/`2d];@0Ph>306=SH`19BDT0BTY:05AD
E01YJFT0F5QH06e]K@2>SXh0X:2P065QH@14A4@0P8200=WIf@3AdM40WYjN0;Bd]03Ph>00bl_;0:2P
X02i^KT0Z:RX0:b/[02ATI40>c/k01LG5`805ADE01405QHF01hN7P1>CTh0]kNg0::RXP0U9BD0=SHf
06i^KP23Ph<0EeMG06QXJ010@400<c<c038b<P0X:2P0:RXZ03<c<`0203`l?00R02DU9@0e=CD0BDU9
06YZJP2DU9@0/[:b0;Bd]02>SXh0SHf=0:2PX02a/K40YZJV0;Fe]@2@T900IFEU075aL@2][Jd0N7Qh
05aLG013@d<0De=C04Y:BP2JVYX0R8R806MWI`1ADE40HF5Q03/k>`0f=SH0@41004=3@`0K6a/071`L
01dM7A3ooooo000>ooooo`3o05=CD`1iNGT0@D5106YZJP13@d<0QhN7059BDP13@d<0KVi^05]KF`0^
;Rh0FEUI03Ph>00V9RH0ADE5034a<@0e=CD051@D04U9B@0W9bL0@D5102`/;00N7Qh0FEUI024Q8@0j
>SX0JFUY03Ph>00U9BD0BDU904=3@`1KFe/0:RXZ02d];@1RHV80<c<c04A4A00l?3`0>3Ph038b<P1A
DE40BTY:02l_;`1>CTh071`L03hn?P1BDU80CTi>05UIF@0k>c/0F5QH05YJFP11@D40@T9205aLG00a
<C408B4Q055AD@1IFET0>c/k04I6AP1<C4`05QHF04i>CP19BDT06a/K034a<@13@d<05aLG04=3@`16
ATH0@41004];B`0Q8B40Cdm?05UIF@0i>CT0JVYZ02XZ:P0U9BD0C4a<07AdM01]KFd0IFEU04e=C@1J
FUX0DU9B06YZJP1`L700V9RH08N7Q`1WIfL0C4a<04I6AP26QXH0/K6a0<O7a`28R8P0@D5108Z:RP1K
Fe/0J6QX07IfMP1WIfL0CDe=05eMG@2>SXh0BDU905EEE@1_Kfl0TI6A08B4Q019BDT0P82007UiN@1E
EED0Hf=S05=CD`1>CTh0IVIV04Q8B01?Cdl0C4a<07EeM@1YJFT0IVIV059BDP0o?cl0Ti>C059BDP1^
KVh0EeMG06EUI@2`/;00PH61038b<P1nOWh0C4a<061PH02:RXX0LG5a08>3P`1[Jf/0TI6A0:>SX`2I
VIT0ADE504Y:BP1mOGd0DU9B0:j^[P1YJFT0HF5Q09fMW@1`L700NGUi05mOG`1DE5@0YjNW09JFUP1_
Kfl0UIFE07alO01LG5`0QXJ609VIV@2:RXX0?clo07EeM@10@400G5aL09FEU@2GUiL0FEUI09fMW@1A
DE40@41007QhN00j>SX0=cLg06=SH`0_;bl0Hf=S05aLG00m?Cd0IVIV03Hf=P29RHT0EeMG0:2PX02:
RXX0E5AD06][J`2k^k/0DU9B07IfMP0a<C40JFUY09JFUP2QXJ40P82006UYJ@1MGEd0U9BD03Xj>P1Z
JVX0LW9b0::RXP12@T80Q8B405aLG01dM7@0Kfm_04U9B@1CDe<0M7Ad04Q8B00U9BD0KVi^079bLP1S
Hf<0<C4a06MWI`24Q8@0I6AT06UYJ@1]KFd0;Rh^08N7Q`1bLW80FUYJ0;2`/01BDU80AdM7096AT@0^
;Rh0ATI604];B`1LG5`0<c<c06=SH`1kNg/0=3@d03Lg=`22PX80C4a<0451@@1EEED0Ogmo04I6AP2^
[Zh0O7al02h^;P0h>3P0Jf][000G02/[:`2LW9`0LG5a01/K6`1OGel0PH61051@D02@T900B4Q8034a
<@1mOGd0LG5a04];B`22PX80Ogmo04Q8B00Q8B40:b/[06QXJ01FEUH0C4a<08^;R`22PX803_ooool0
;`1hN7P0JVYZ05IFEP14A4@0>CTi06i^KP2YZJT0dm?C0<o?c`2XZ:P0K6a/0;Jf]P2@T900QhN706YZ
JP13@d<0@41004A4A01SHf<0Ph>306ATI00c<c<0:b/[04a<C00V9RH0@41005EEE@1`L700Lg=c075a
L@1BDU80C4a<07IfMP1VIVH0Cdm?04A4A00e=CD0;Bd]02<S8`0P8200Cdm?07MgM`2a/K40WinO0:JV
YP1UIFD0Bd];0080<30`02d0IVIV02DU9@0E5AD05QHF01TI6@0K6a/07alO02HV9P1/K6`0G5aL02/[
:`1SHf<0EEEE07=cL`19BDT0E5AD08V9R@2GUiL0L71`05IFEP1UIFD0_Kfm08F5Q@1fMWH0L71`04i>
CP1=CDd0][Jf09nOW`0l?3`0?Shn04e=C@0S8b<07QhN02d];@15ADD0L71`06IVIP1=CDd0>3Ph01XJ
6P0H61P0@T920451@@0Q8B400P0M7Ad04@15ADD0EEEE0451@@1dM7@0:b/[01LG5`1mOGd0dM7A0;:b
/P1oOgl0bLW908V9R@1_Kfl0Jf][06YZJP1RHV80MGEe0080JFUY00@0[Jf]09FEU@1XJ6P0O7al0P2A
TI40601EEED08b<S01hN7P13@d<0Ogmo0:2PX02_[jl0Xj>S08Z:RP2;Rh/0W9bL0::RXP1jNWX0BTY:
034a<@1TI6@0E5AD02/[:`0Q8B40;Bd]0410@00d=3@0=CDe06a/K080H61P05d0Bd];07inOP2][Jd0
SXj>07YjNP1mOGd0GemO02h^;P0Y:BT0K6a/06ATI01eMGD0U9BD034a<@0I6AT0;bl_04=3@`1<C4`0
SXj>0;Bd]02_[jl0<S8b02d];@0g=cL0GEeM02`/;01FEUH0MWIf03Ph>00U9BD0:BTY038b<P1SHf<0
JFUY07IfMP0e=CD0=SHf06m_K`1RHV80Xj>S0;Jf]P2[Zj/0_;bl08N7Q`2EUID0]kNg0<O7a`1lO7`0
M7Ad07IfMP24Q8@0RXZ:0:6QX@2^[Zh0WIfM09JFUP2^[Zh0cLg=0=;BdP1<C4`0820P03@d=02i^KT0
/;2`09NGU`2`/;00][Jf0=WIf@32`/80TY:B05IFEP1dM7@0];Bd07]kN`2JVYX0S8b<0451@@0Z:RX0
=cLg096AT@1kNg/0Bd];06e]K@0L71`0A4A406m_K`1gMgL0K6a/07QhN011@D40>CTi03lo?`0i>CT0
0P0X:2P0900P82009BDU01lO7`0V9RH0B4Q806m_K`2?Shl0`l?30;Vi^@2b/[80Ph>309bLW02RXZ80
bl_;0;^k^`1JFUX0EEEE0:j^[P1YJFT0E5AD08N7Q`2MWId0`/;20<S8b02KVi/0_Kfm05UIF@0R8R80
>c/k04=3@`12@T80=SHf04e=C@0H61P06ATI01dM7A3ooooo000>ooooo`0L06=SH`1]KFd0A4A405UI
F@0_;bl0FEUI04M7A`1CDe<0Fe]K0492@P0g=cL0KFe]04a<C00/;2`0;bl_02<S8`13@d<061PH02/[
:`0;2`/0;Rh^04i>CP0U9BD0?Cdm014A4@0_;bl0E5AD04A4A0809BDU05L08b<S05IFEP0a<C40<C4a
06YZJP0N7Qh0:BTY05ADE00f=SH06QXJ03Xj>P1MGEd0?3`l05IFEP0Z:RX09bLW04i>CP13@d<0K6a/
051@D01MGEd0LG5a04a<C013@d<0H61P05eMG@0d=3@0EeMG079bLP1MGEd0A4A404M7A`0X:2P0GEeM
04=3@`0I6AT0;Bd]0410@00W9bL0D51@0410@013@d<0G5aL02PX:01LG5`0DE5A030`<01SHf<0E5AD
030`<01DE5@0LG5a06YZJP1RHV80EEEE05mOG`1RHV80EeMG03@d=01]KFd0MWIf05IFEP1CDe<0AdM7
0820P02`/;00dM7A08f=S@17AdL0Q8B405aLG01aLG40NWYj07UiN@18B4P0HV9R09FEU@1DE5@0DU9B
05aLG02HV9P0QhN704Y:BP1hN7P0MWIf059BDP1PH6000P1ADE40W`1JFUX0@T9205UIF@15ADD0OWin
06=SH`1nOWh0G5aL03hn?P2KVi/0KFe]08F5Q@1TI6@0J6QX0;2`/01aLG40B4Q807moO`1:BTX0F5QH
07inOP1HF5P0U9BD07QhN02;Rh/0c<c<08f=S@1DE5@0OWin09bLW019BDT0QHF505]KF`2:RXX0Y:BT
079bLP1fMWH0?Cdm0410@01nOWh0LG5a05YJFP20P800RXZ:06YZJP1fMWH0H61P05QHF018B4P0N7Qh
06YZJP1ADE40R8R806][J`1BDU80W9bL04a<C00l?3`0NWYj03Ti>@0h>3P0G5aL0410@01ZJVX0C4a<
03Hf=P1>CTh0=cLg09RHV01EEED0Ti>C0:BTY00V9RH0?clo09>CT`10@400JVYZ02LW9`16ATH0LW9b
05QHF01BDU80FEUI069RHP2=SHd0@41005]KF`1EEED0Shn?03@d=01bLW80=CDe03dm?@1@D500=SHf
05EEE@1MGEd0<C4a04Q8B01fMWH0Lg=c05=CD`0j>SX0Jf][0410@00h>3P0BDU906IVIP17AdL0TI6A
05=CD`0i>CT0Ph>305IFEP0l?3`0QXJ6065QH@1^KVh0QXJ607YjNP0c<c<0A4A404a<C00`<300EUIF
09FEU@29RHT0ATI605MGE`1XJ6P0;Bd]0861P@1bLW809bLW03/k>`1XJ6P0=CDe09fMW@2KVi/0GEeM
06QXJ01jNWX0GEeM08f=S@1OGel0>CTi08:2PP1^KVh0@D5104i>CP1SHf<0ATI601lO7`0/;2`0GUiN
04A4A00T92@0O7al0820P00>ooooo`0Q05IFEP1EEED0Bd];02DU9@0L71`07QhN04i>CP21PH40`<30
08n?S`27QhL0NWYj03Hf=P10@400@D5103/k>`0a<C40<c<c06e]K@2l_;`0I6AT01DE5@0C4a<07alO
05=CD`0];Bd0ADE509bLW01jNWX0SXj>07=cL`12@T80=SHf0080;Bd]0180;Rh^030`<00f=SH0>3Ph
06UYJ@2QXJ40f][J0>7Qh@1SHf<0>3Ph092@T01EEED0;2`/02/[:`15ADD06ATI00h>3P0C4a<301DE
5@0a01/K6`0R8R80DU9B08N7Q`1OGel0P820071`L02ATI40[Jf]06YZJP19BDT0>3Ph02XZ:P0_;bl0
?Cdm071`L027QhL0GUiN0451@@0`<300P82009:BTP12@T805ADE03Lg=`0U9BD08b<S01`L700Y:BT0
FEUI071`L01]KFd0Fe]K02l_;`13@d<0<C4a02LW9`0I6AT07AdM01hN7P0f=SH0FUYJ04I6AP0h>3P0
UIFE06YZJP16ATH0^;Rh0>GUi@020=KFeP0b0<c<c02SXj<0UIFE096AT@2@T900WinO0;>c/`3Fe]H0
WYjN05aLG01PH600E5AD02DU9@0m?Cd0/K6a08>3P`1YJFT0I6AT06YZJP3AdM40_Kfm09^KV`20P800
Fe]K05aLG01RHV80>c/k01TI6@0H61P0?clo05mOG`0k>c/0;2`/02@T900Z:RX0?Cdm08J6QP1<C4`0
=cLg02DU9@0Z:RX0Ng]k0<o?c`2NWYh0W9bL09nOW`1dM7@0:b/[01/K6`0U9BD203De=@0W06YZJP1d
M7@06ATI01lO7`0h>3P0Ph>30<S8b036a/H0KVi^030`<00W9bL0=cLg05QHF02HV9P0NWYj04i>CP0U
9BD08B4Q02<S8`0P8200>CTi05YJFP1>CTh0HV9R02l_;`0N7Qh0CDe=04i>CP2j^[X0];Bd07alO02D
U9@0/k>c0=WIf@1dM7@0FEUI04m?C`1@D500;bl_008071`L02X0:2PX0492@P1`L700Ti>C0:>SX`3;
bl/07alO01DE5@1CDe<0TY:B09BDU00i>CT0=SHf06i^KP3Cdm<0UiNG03@d=00m?Cd0IVIV09ZJVP1;
Bd/0AdM707=cL`2b/[80b<S80=?Cd`1bLW807QhN03Hf=P1IFET0:BTY05QHF01^KVh0BTY:07MgM`1W
IfL0=cLg04=3@`0V9RH0:2PX020P800O7al201dM7@0R020P800T92@0B4Q8065QH@2IVIT0L71`08b<
S03?cll0YJFU09BDU01^KVh0Ph>30:VYZ@1CDe<0XJ6Q0::RXP0h>3P0?Shn079bLP3<c<`0S8b<0820
P02PX:00S8b<02PX:00P8200:2PX05MGE`1IFET0@T9207=cL`0_;bl0<c<c04e=CA3ooooo000>oooo
o`0P07]kN`2BTY80Cdm?069RHP0e=CD0IFEU05aLG01ZJVX0De=C06=SH`0f=SH0FEUI05]KF`0U9BD0
@T9202DU9@19BDT061PH030`<00N7Qh0<S8b0410@00V9RH0@D5102d];@0X:2P0EUIF04];B`0V9RH0
?Cdm03@d=01ADE4203@d=004075aL@1OGel0=SHf06i^KP80>c/k0280@410075aL@14A4@0De=C03Xj
>P0];Bd0CDe=03`l?020P800HF5Q07EeM@23Ph<0D51@04=3@`1MGEd0J6QX03Ph>01EEED0Lg=c06MW
I`17AdL0D51@03Lg=`1IFET0G5aL02LW9`0`<300Cdm?030`<01?Cdl0?Cdm04];B`1EEED0<c<c0P1B
DU802@0c<c<0HV9R05EEE@0`<300EUIF04a<C01ADE40PH6105IFEP0206IVIP2o08^;R`1gMgL0S8b<
07IfMP1GEeL0FUYJ04Y:BP1FEUH0GUiN0:6QX@22PX80AdM70861P@1=CDd0MWIf06ATI01<C4`0GemO
08n?S`28R8P0BDU9059BDP11@D40OGem08>3P`1:BTX0P820051@D019BDT0HV9R04];B`1BDU80Cdm?
03De=@1CDe<0FEUI0:RXZ00j>SX0IFEU05aLG017AdL0T92@03dm?@1[Jf/0FUYJ065QH@34a<@0Lg=c
034a<@1eMGD0>c/k06][J`21PH40Cdm?07IfMP1PH600Q8B40<g=c@23Ph<0=CDe03Hf=P1@D500FUYJ
0;Jf]P1BDU80D51@08^;R`1]KFd0TY:B0861P@1ADE40VYZJ04m?C`0a<C40HF5Q092@T01<C4`0LG5a
06a/K013@d<0@T9207UiN@1[Jf/0H61P0:BTY01eMGD0B4Q808J6QP0n?Sh0<30`051@D00d=3@0?Shn
04i>CP0b<S80H61P04Y:BP10@400IfMW02LW9`1TI6@0B4Q80::RXP2?Shl0EeMG04i>CP2YZJT0B4Q8
06IVIP0a<C40A4A408J6QP1CDe<0@D51061PH022PX80N7Qh04I6AP1dM7@0DU9B0:^[Z`0d=3@0SHf=
05=CD`19BDT0F5QH020P801DE5@0JFUY04E5A@0X:2P0DE5A07MgM`0P8200?3`l069RHP1hN7P0EeMG
059BDP1XJ6P06ATI069RHP20P800D51@0::RXP1`L700@41006YZJP1FEUH0Hf=S055AD@1kNg/0?Cdm
04];B`1MGEd0<S8b034a<@1`L700QHF5059BDP1iNGT0L71`04i>CP2JVYX0Ph>303@d=01JFUX0RXZ:
03Hf=P1kNg/0G5aL05QHF01/K6`0KFe]04];B`1nOWh0JFUY04A4A02EUID0WYjN06MWI`1MGEd0LW9b
03hn?P0k>c/0CDe=06][J`17AdL09RHV08R8R021PH403_ooool01010@400>c/k038b<P0O7al201XJ
6P0802DU9@1LG5`0TY:B07IfMP2IVIT0V9RH06ATI00Z:RX201DE5@0B01<C4`0A4A405ADE051@D02[
Zj/09BDU01<C4`0B4Q807alO05aLG020P800a/K60:ZZZP2<S8`0I6AT05YJFP1?Cdl0DE5A0P1NGUh0
>01BDU80RHV909^KV`3?cll0cLg=0=cLg03Fe]H0QhN702LW9`2CTi<0G5aL03<c<`0V9RH0@41001XJ
6P0Q8B406ATI014A4@0C4a<05ADE01TI6@0K6a/0820P07alO02VYZH0Ng]k07moO`2ZZZX0I6AT03dm
?@0b<S80;bl_02XZ:P0J6QX07AdM02HV9P1CDe<0GEeM03hn?P0^;Rh0IfMW0492@P1IFET0@41002`/
;00f=SH08R8R02<S8`0Q8B40:b/[05aLG019BDT0EeMG07AdM00V9RH08R8R0P0L71`0>`0R8R809RHV
02h^;P1NGUh0D51@03Hf=P21PH40@d=3024Q8@2NWYh0_kno0:n_[`3;bl/0c<c<05aLG01DE5@0FEUI
05MGE`1NGUh0P82009:BTP2=SHd0B4Q804Y:BP0];Bd07alO034a<@1FEUH0B4Q801TI6@1UIFD0CTi>
0:n_[`28R8P0HV9R05iNGP15ADD0>SXj051@D00m?Cd05ADE01HF5P0G5aL0;2`/04a<C00a<C409BDU
03De=@1SHf<0DU9B04];B`0_;bl0<C4a0451@@2UYJD0a/K607alO01oOgl0EEEE0080KVi^01@08b<S
01DE5@0T92@0=CDe04M7A`2_[jl0HF5Q03Ph>00];Bd0<c<c07moO`2TY:@0U9BD05YJFP0Y:BT0;bl_
0410@02g]kL0JVYZ02d];@807QhN01P07AdM01lO7`0R8R80BTY:05EEE@21PH40Jf][07alO02LW9`0
WIfM08:2PP2HV9P0alO70=WIf@3Ogml0[Zj^06IVIP1hN7P0I6AT07emO@0V9RH071`L01/K6`0J6QX2
01/K6`0U01`L700Q8B40NWYj04a<C00G5aL0Shn?075aL@1cLg<0820P01dM7@0a<C40];Bd0820P00X
:2P0;bl_04a<C02_[jl0BTY:024Q8@0X:2P0?Shn06MWI`34a<@0KFe]01TI6@10@400C4a<04E5A@1Y
JFT0?clo02/[:`2ZZZX0ATI602XZ:P0T92@09RHV01`L700401hN7P0R02<S8`0`<300DE5A07moO`2T
Y:@0MgMg09nOW`2`/;00/[:b06QXJ02^[Zh0bl_;0=7Ad@3Jf]X0emOG08^;R`0V9RH0<30`06MWI`1h
N7P07QhN03Xj>P2ATI40?Cdm03Lg=`0N7Qh08b<S06IVIP2ZZZX0Vi^K0;Rh^02=SHd0Q8B407=cLa3o
oooo000>ooooo`0o07AdM01nOWh0BTY:05mOG`0f=SH0IfMW04m?C`1HF5P0GUiN079bLP15ADD0GUiN
055AD@0[:b/0CTi>02PX:013@d<071`L04A4A00h>3P0@d=305ADE00P8200F5QH03Xj>P0k>c/0G5aL
0492@P0R8R80C4a<034a<@18B4P0:BTY02XZ:P1dM7@0IFEU03De=@1JFUX0:RXZ03/k>`0g=cL0EEEE
03De=@1QHF40>CTi03/k>`1FEUH0>3Ph07inOP1OGel0R8R805]KF`0l?3`0@T9205UIF@1QHF40?3`l
05iNGP1LG5`0CDe=04i>CP1cLg<0>CTi0080Hf=S06P09RHV03Xj>P1KFe/0=CDe055AD@12@T80C4a<
05iNGP0e=CD0@T9205ADE00f=SH0Jf][04m?C`0/;2`0H61P07QhN01RHV80TI6A03lo?`1XJ6P0MgMg
07YjNP1]KFd0WYjN07QhN01EEED0DU9B05EEE@1kNg/0XJ6Q0=SHf02DU9@0@D5108J6QP1PH600L71`
055AD@1bLW80C4a<05YJFP1^KVh0B4Q8069RHP1[Jf/0Rh^;07moO`13@d<0PX:208F5Q@1<C4`0G5aL
04I6AP1HF5P0Kfm_05UIF@1?Cdl0C4a<092@T01XJ6P0HF5Q06m_K`10@400Q8B407IfMP1IFET0B4Q8
04U9B@2`/;00TI6A03Ph>01^KVh0@41004Y:BP24Q8@0@T9208V9R@1VIVH0EUIF0<_;b`23Ph<0FUYJ
055AD@21PH40F5QH08N7Q`1BDU80MgMg08J6QP1NGUh0PX:207QhN01IFET0ZZZZ08j>SP1oOgl0ADE5
09FEU@1;Bd/0N7Qh08^;R`29RHT0BTY:07alO080IFEU0400[:b/09:BTP17AdL0Q8B4065QH@19BDT0
H61P03De=@1>CTh0N7Qh03Xj>P1bLW80BDU905eMG@1VIVH0?clo07emO@1EEED0U9BD092@T01JFUX0
CTi>0;2`/01BDU80Jf][04E5A@1EEED0VYZJ07YjNP21PH40HV9R06EUI@2:RXX0A4A40861P@1]KFd0
Z:RX03lo?`1LG5`0BDU904i>CP1WIfL0@T9204a<C01LG5`0@D5103De=@1PH600NWYj051@D00_;bl0
EEEE069RHP1;Bd/0J6QX07]kN`0b<S80GEeM061PH018B4P0UIFE06IVIP0n?Sh0O7al0P1oOgl0:@1E
EED0O7al034a<@1@D500KVi^03De=@0^;Rh0Fe]K051@D00m?Cd0De=C069RHP1BDU80PH6105IFEP0V
9RH0;bl_075aL@0j>SX0TY:B05MGE`16ATH0Kfm_09FEU@1TI6@0UiNG06][J`0c<c<0Lg=c07YjNP1G
EeL0GemO05=CD`0l?3`061PH02<S8`1PH600G5aL02/[:`2?Shl0MgMg00kooooo00@0=CDe03Lg=`0[
:b/08R8R0P0V9RH02P0Y:BT0>3Ph06i^KP2NWYh0UIFE05]KF`0K6a/051@D018B4P0?3`l2014A4@0P
018B4P0A4A40?3`l08j>SP19BDT04a<C010@400V9RH0SXj>08f=S@2g]kL0OWin07QhN00h>3P0:BTY
01lO7`0J6QX061PH01`L700Y:BT08b<S05=CD`1BDU80KFe]07AdM02FUYH0Xj>S0<?3``2/[:`0GEeM
03dm?@1GEeL201LG5`1101@D500>3Ph04A4A01<C4`0H61P06QXJ02PX:018B4P0MgMg0861P@2<S8`0
Hf=S04E5A@14A4@0:b/[01hN7P0H61P05QHF01DE5@0F5QH09BDU03lo?`11@D40@T9206QXJ00l?3`0
=CDe04Y:BP1UIFD0O7al02l_;`0Y:BT0<S8b0492@P1DE5@0NWYj06i^KP1mOGd0C4a<03De=@0];Bd0
:BTY02`/;00d=3@0>SXj06IVIP1MGEd0A4A406YZJP0`<3009bLW051@D017AdL0DE5A08J6QP3De=@0
SHf=06YZJP1IFET0De=C05eMG@1dM7@0IFEU0:^[Z`0i>CT00P0E5AD02@0N7Qh0<30`0410@01jNWX0
LG5a065QH@1NGUh0EEEE03/k>`0202d];@0?028R8P0L71`08B4Q04I6AP0V9RH05aLG01HF5P0E5AD0
71`L03<c<`0d=3@0?Cdm069RHP2>SXh0NGUi0080Ogmo00D0Y:BT0;jn_P2k^k/0MWIf03Ph>00202<S
8`0602XZ:P10@40061PH02`/;00T92@0:b/[0P1dM7@03P13@d<0GemO08n?S`2MWId0[:b/0<g=c@2^
[Zh0QXJ607QhN02DU9@0VIVI08:2PP1:BTX0CDe=0P15ADD0<00M7Ad05aLG02<S8`1CDe<0JFUY08:2
PP0n?Sh0:b/[04U9B@12@T80<S8b06a/K02XZ:P0emOG08R8R01jNWX0Kfm_07emO@2HV9P0C4a<01dM
7@0J6QX07QhN01dM7@0K6a/07AdM01hN7P0/;2`0KVi^0:JVYP29RHT0a/K607]kN`0Q8B4071`L03Lg
=`2MWId0MWIf059BDP16ATH0MgMg092@T01ZJVX0=3@d02XZ:P0Y:BT0;bl_04I6AP80P8200380PH61
0451@@1ADE40De=C05iNGP1nOWh0/K6a09RHV01IFET0@T9202LW9`0O7al0820P024Q8@0O7al0:BTY
05ADE01iNGT0SHf=0::RXP2?Shl0Kfm_04I6AP13@d<0P82008J6QP1>CTh0YJFU0=?Cd`2b/[80VYZJ
0:6QX@2KVi/0JVYZ075aL@1=CDd092@T06YZJP2DU9@09bLW04U9B@1RHV80GemO05aLG01aLG40X:2P
07AdM01@D500E5AD06=SHa3ooooo000>ooooo`1D06QXJ01UIFD0GEeM05YJFP0^;Rh0EeMG065QH@1H
F5P0GEeM07AdM015ADD0QHF505iNGP0S8b<0;Bd]02DU9@18B4P051@D02HV9P0=3@d0:b/[069RHP0P
8200@d=301TI6@0V9RH0AdM7051@D00P82009bLW02DU9@1FEUH0CDe=01`L701JFUX0@T9203De=@1Z
JVX0>c/k02@T900i>CT0J6QX04I6AP1?Cdl0:b/[02PX:016ATH0=cLg07MgM`1CDe<0I6AT071`L01B
DU80EUIF06i^KP1YJFT0;bl_04E5A@1`L700KVi^03lo?`1SHf<0:RXZ03lo?`1JFUX092@T02h^;P1=
CDd0820P0492@P1KFe/0De=C04m?C`0P8200?Shn05]KF`0i>CT0De=C05UIF@0U9BD0IfMW08:2PP1X
J6P0][Jf0P1[Jf/0`01ZJVX0I6AT03<c<`20P800Q8B405]KF`1:BTX0?3`l08b<S02][Jd0c/k>092@
T013@d<0Ogmo07AdM024Q8@0GemO08:2PP1@D500Hf=S08^;R`1OGel0G5aL07MgM`27QhL0WinO04Y:
BP1iNGT0Ng]k05=CD`1[Jf/0ATI604m?C`1/K6`0HF5Q071`L01JFUX0TY:B05YJFP1eMGD0J6QX0492
@P21PH40BDU906m_K`1JFUX0ATI60;jn_P1LG5`0=CDe06e]K@0f=SH0Bd];0820P00h>3P0LW9b06IV
IP0g=cL0/K6a08B4Q01>CTh0?3`l07emO@1<C4`0PH6106ATI02ZZZX0W9bL07QhN01kNg/0EeMG04Q8
B024Q8@0KFe]04E5A@10@400Vi^K05ADE027QhL0ZJVY06a/K011@D40SXj>08:2PP1cLg<0`l?307Ee
M@0c<c<0RXZ:04M7A`0];Bd0D51@03Lg=`0n?Sh0HV9R03Ph>01kNg/0@41005aLG00n?Sh0:RXZ04m?
C`0l?3`0W9bL09^KV`1IFET0?Cdm096AT@11@D40HF5Q030`<010@400QXJ604E5A@1aLG40GUiN06e]
K@25QHD0Hf=S07]kN`1LG5`0WYjN04m?C`1EEED0EeMG04Y:BP1YJFT0GemO0451@@1PH600Jf][03Ph
>013@d<0WYjN06IVIP0`<300C4a<08V9R@1SHf<0H61P08>3P`1[Jf/0OWin09fMW@1[Jf/0UYJF05AD
E00d=3@0P820071`L01dM7@0FUYJ08V9R@1KFe/0@D5105ADE00c<c<09RHV05eMG@28R8P0LW9b07Ui
N@1/K6`0FUYJ08j>SP28R8P0?3`l03@d=01YJFT0=CDe09BDU01FEUH0GUiN06a/K02@T900FEUI08B4
Q017AdL0>c/k06MWI`1hN7P0I6AT08F5Q@1VIVH0ATI601HF5P0T92@0JVYZ03lo?`1>CTh0WYjN092@
T0kooooo0140<30`03Lg=`0`<300=SHf04=3@`0j>SX09RHV01dM7@19BDT0Ph>30;Vi^@29RHT0:b/[
01<C4`0@41003Ph>018B4P02014A4@0S00h>3P0A4A40:b/[0:6QX@0_;bl04A4A02DU9@1YJFT0S8b<
08B4Q01JFUX0?clo01<C4`0A4A40410@01DE5@0B4Q805ADE01<C4`0@41005aLG02TY:@0i>CT0GUiN
065QH@1VIVH0^;Rh0=SHf02@T900Zj^[0;no_`1?Cdl0:RXZ01LG5`0>3Ph00P0D51@02P0B4Q80=3@d
05=CD`0b<S80UIFE07=cL`1oOgl0BTY:04];B`0T92@701DE5@1@02DU9@0S8b<0<c<c069RHP10@400
ADE503<c<`0o?cl0Shn?06ATI016ATH0De=C07MgM`2ATI40Z:RX0<C4a02b/[80IVIV03/k>`19BDT0
FEUI06IVIP1mOGd0U9BD0:>SX`1hN7P0AdM706YZJP0/;2`0De=C0492@P0T92@0:RXZ03dm?@2^[Zh0
8b<S02/[:`0T92@0?Cdm06EUI@12@T806QXJ06MWI`21PH409bLW01<C4`0D51@0HF5Q07inOP2h^;P0
IFEU05=CD`1eMGD0[Jf]06UYJ@0K6a/0:b/[02HV9P0Y:BT06a/K02`/;01jNWX092@T01HF5P0E5AD0
61PH03Hf=P1@D500OGem08:2PP2h^;P0Ng]k08n?S`2UYJD0NWYj06ATI02_[jl0S8b<02l_;`0E5AD2
01`L700H01TI6@0H61P0:BTY02`/;00[:b/0L71`01`L700E5AD05aLG028R8P1^KVh0GEeM0:2PX01X
J6P0@T9203/k>`1aLG40DU9B0:ZZZP1CDe<0;Bd]01lO7`0M7Ad07QhN0P0O7al03@0[:b/0Lg=c04e=
C@13@d<07AdM03Ti>@1SHf<0?clo04U9B@1hN7P0alO70:6QX@0o?cl00P0k>c/0DP1NGUh0Rh^;051@
D00L71`061PH020P800S8b<07AdM02<S8`0b<S80Fe]K0:RXZ01bLW80KFe]05]KF`1/K6`0MWIf08F5
Q@1aLG40@T9206e]K@1gMgL0=SHf02d];@0c<c<0Cdm?0451@@0V9RH0=CDe04E5A@1IFET0YJFU0>7Q
h@3De=@0/[:b08Z:RP1KFe/0Bd];09>CT`0i>CT0E5AD0820P02EUID0Ph>305IFEP0g=cL0EEEE07Qh
N02ATI40YZJV09fMW@1eMGD0GEeM028R8P0c<c<0FEUI02TY:@0T92@0XJ6Q0:2PX01WIfL0EUIF065Q
H@15ADD0JFUY09NGU`32`/80UiNG0;2`/03De=@0`l?30:BTY02MWId0TY:B06a/K02EUID0Xj>S09^K
V`0W9bL0:b/[04Y:BP1`L70@ooooo`003_ooool0o`2:RXX0LW9b075aL@1cLg<0>CTi0410@01FEUH0
D51@09>CT`1/K6`0;Bd]04e=C@1SHf<0;2`/02l_;`0L71`0Cdm?01LG5`0/;2`07AdM02d];@1QHF40
7AdM0410@00M7Ad0:RXZ04U9B@1JFUX092@T034a<@0[:b/0ATI605MGE`0I6AT0AdM706][J`0Y:BT0
JVYZ03<c<`13@d<0=SHf07UiN@1EEED0H61P04];B`0c<c<0EeMG04A4A01hN7P0H61P06EUI@1@D500
?clo03Hf=P16ATH0H61P03Ti>@14A4@0Cdm?03Ti>@1@D500T92@03dm?@11@D40EeMG02PX:01EEED0
MGEe03@d=00o?cl0BDU905EEE@1cLg<0CTi>034a<@1YJFT0@D5105mOG`1kNg/0;Bd]06UYJ@1GEeL0
Cdm?09FEU@1CDe<0KFe]07AdM022PX80KFe]09ZJVP1ZJVX0>3Ph06i^KP20P800V9RH06i^KP2j^[X0
RHV903Xj>P1bLW80?clo07inOP1OGel0IVIV051@D026QXH0RHV9055AD@1HF5P0H61P06m_K`2BTY80
BTY:07IfMP1LG5`0Bd];05eMG@15ADD0K6a/07AdM00k>c/0CDe=059BDP2ZZZX0FUYJ05eMG@12@T80
>CTi08Z:RP1SHf<0DU9B04Q8B01RHV80`/;208^;R`0d=3@0L71`034a<@1JFUX0RXZ:04=3@`23Ph<0
GemO04Q8B02TY:@0Shn?06i^KP0g=cL0IVIV04U9B@29RHT0EUIF06EUI@21PH40Fe]K08J6QP2<S8`0
De=C09ZJVP1RHV80<c<c04Y:BP2CTi<0IFEU08V9R@2?Shl0NWYj04U9B@1iNGT0Ogmo07=cL`35aLD0
LG5a03Lg=`2<S8`0A4A403/k>`1@D500>SXj07IfMP2EUID0=3@d06YZJP15ADD0MgMg05=CD`0m?Cd0
?clo03Ph>02_[jl0ZJVY069RHP0f=SH0Q8B40492@P1eMGD0>SXj03dm?@2:RXX0BDU907YjNP1NGUh0
Ph>309VIV@1TI6@0Ogmo05aLG02ATI40B4Q808V9R@17AdL0B4Q806MWI`19BDT0FEUI07=cL`1SHf<0
=3@d04E5A@29RHT0=3@d04E5A@1/K6`0Ng]k05=CD`1ADE40GEeM03lo?`1_Kfl0QhN705YJFP2@T900
PX:206ATI01gMgL0Q8B406e]K@1SHf<0Rh^;05UIF@17AdL0MgMg04=3@`0a<C40MWIf08f=S@1YJFT0
Rh^;071`L00h>3P0SXj>08Z:RP0m?Cd0?3`l07=cL`005`0k>c/0Y:BT0;2`/01VIVH0Hf=S08j>SP1B
DU80QhN707emO@1;Bd/0J6QX0:b/[01WIfL0H61P059BDP1<C4`09bLW02d];@1MGEd0A4A402HV9P1e
MGD0PH6100kooooo0480=3@d04m?C`1cLg<0@41003Lg=`0S8b<07AdM03hn?P1GEeL0JFUY08^;R`1f
MWH0HF5Q02DU9@0E5AD04A4A01<C4`0@41003Ph>00l?3`0A4A404a<C03Xj>P1dM7@051@D0410@01_
Kfl0_;bl065QH@1KFe/0@41001PH600@41004A4A01@D500E5AD071`L028R8P0T92@0:RXZ034a<@1C
De<0KVi^08Z:RP1bLW80OWin0=kNgP3Ti>@0d=3@0;2`/01_Kfl0J6QX069RHP0j>SX0;Rh^01`L700F
5QH0@D5108^;R`1ZJVX0/K6a05YJFP2OWil0EEEE0492@P0G5aL201<C4`0301DE5@0B4Q804a<C0080
5ADE03006ATI01/K6`0L71`0OWin065QH@1;Bd/0Q8B409FEU@2a/K40GEeM04M7A`0n?Sh0BTY:07=c
L`20P800Vi^K0;:b/P2JVYX0ADE502/[:`0`<300=cLg0492@P14A4@0AdM705eMG@2;Rh/0]KFe0861
P@1VIVH0=cLg02/[:`0_;bl0KVi^08V9R@0I6AT0<S8b06IVIP1PH600:BTY01lO7`0a<C40GemO06=S
H`25QHD07alO01PH6015ADD2075aL@060:NWY`27QhL0JVYZ04Y:BP1UIFD0=SHf0P14A4@05P0X:2P0
<C4a020P801GEeL0BDU901XJ6P0M7Ad0=cLg05=CD`1EEED0H61P05iNGP32`/80W9bL08Z:RP1VIVH0
DU9B03dm?@1=CDd0EeMG06m_K`0I6AT201/K6`0301`L700G5aL071`L0080=3@d01L0Lg=c01LG5`0E
5AD05ADE01lO7`2ATI40?Shn03lo?`1KFe/0B4Q804A4A01_Kfl0GemO07]kN`0d=3@06a/K01XJ6P0K
6a/06QXJ01hN7P0K6a/08B4Q071`L00204=3@`0^038b<P1KFe/0J6QX03@d=00b<S80TI6A0=KFeP2M
WId0A4A402h^;P0d=3@0Bd];08V9R@2QXJ40<c<c01TI6@0V9RH0=SHf03Ti>@12@T80AdM706ATI02H
V9P0QXJ602LW9`0U9BD0<S8b075aL@28R8P0Zj^[0:JVYP1BDU8092@T05ADE018B4P0>c/k04A4A01;
Bd/0DE5A03@d=00a<C40H61P0;Vi^@3Ogml0TI6A07IfMP80@D5100X0CDe=0;Bd]00^;Rh08b<S02TY
:@0k>c/0IVIV09ZJVP2KVi/0LG5a0P2g]kL08@2;Rh/0IFEU09VIV@0k>c/092@T03De=@0N7Qh071`L
01dM7@2VYZH0U9BD04i>CP1?Cdl0EeMG065QH@0h>3P0G5aL07inOP39bLT0c/k>08:2PP0e=CD0=cLg
05]KF`1eMGD0XZ:R0;>c/`33`l<0SHf=03`l?01RHV80GUiN07alO00@ooooo`003_ooool0R@1iNGT0
J6QX04M7A`1HF5P0=cLg03hn?P1BDU80>CTi04I6AP1NGUh0;Rh^055AD@15ADD0@T9206EUI@0X:2P0
AdM701dM7@11@D40:2PX024Q8@1DE5@06QXJ04];B`0_;bl0;Bd]0451@@14A4@09bLW04I6AP0[:b/0
?Shn03Lg=`0K6a/0HF5Q07moO`0V9RH0IVIV02d];@1BDU80=cLg061PH01:BTX0OGem05EEE@0[:b/0
?clo024Q8@1RHV80Fe]K06e]K@1ZJVX0Hf=S0410@01<C4`0Lg=c03@d=011@D40Jf][075aL@17AdL0
IFEU04U9B@12@T80J6QX03Hf=P0/;2`0Fe]K04];B`10@400D51@03Ti>@1KFe/0EUIF034a<@1VIVH0
=cLg07EeM@1/K6`0;2`/07alO01[Jf/0Fe]K09fMW@0c<c<0GEeM06i^KP1UIFD0GemO0:6QX@1/K6`0
CTi>030`<00h>3P0N7Qh08N7Q`33`l<0S8b<0492@P2KVi/0JVYZ05]KF`1PH600T92@04];B`1HF5P0
JVYZ04I6AP1UIFD0KVi^06UYJ@1gMgL0>3Ph07YjNP26QXH0DE5A0492@P0`<300DU9B06=SH`1ADE40
De=C0492@P1nOWh0IfMW05eMG@14A4@0=3@d07MgM`1XJ6P0F5QH04m?C`0j>SX0WYjN08N7Q`0d=3@0
Ng]k0080?Cdm04H0Kfm_03lo?`1nOWh0IFEU04A4A01SHf<0IFEU06=SH`0i>CT0Bd];04A4A023Ph<0
JVYZ07QhN01eMGD0<C4a06QXJ01nOWh0D51@09fMW@27QhL0JVYZ05UIF@23Ph<0ATI608J6QP2IVIT0
Ogmo04Y:BP22PX80F5QH06IVIP2e]KD0Q8B403hn?P28R8P0DE5A03hn?P1?Cdl0=cLg030`<01VIVH0
=cLg06m_K`0[:b/0FUYJ065QH@17AdL0D51@04U9B@1oOgl0VIVI05mOG`11@D40V9RH04Q8B01EEED0
ADE504m?C`2NWYh0Shn?09fMW@1DE5@0GUiN07MgM`1XJ6P0OWin06YZJP2ATI40=cLg0P15ADD0@`0k
>c/0G5aL03@d=00b<S80FEUI05EEE@0/;2`0BDU9079bLP0J6QX09bLW0410@01WIfL0<C4a04M7A`1I
FET0?3`l07QhN026QXH0AdM708F5Q@2FUYH0MGEe07QhN01;Bd/0D51@08:2PP25QHD0DU9B04I6AP1U
IFD0;bl_02d];@1?Cdl0IVIV04U9B@29RHT0Shn?05aLG02IVIT0MgMg03Xj>P16ATH0Kfm_03De=@1j
NWX0TI6A059BDP1UIFD0YjNW0492@P1jNWX0SHf=04Y:BP14A4@0OWin08J6QP2WYjL0Jf][03dm?@0M
7Ad0;Bd]05mOG`1<C4`0:RXZ08B4Q01bLW803_ooool04`1/K6`0Jf][051@D016ATH09RHV02<S8`1<
C4`0@T9201LG5`12@T80H61P05MGE`1HF5P0EeMG024Q8@0E5AD04Q8B014A4@0>3Ph00P0@41001@0C
4a<05aLG07=cL`1<C4`0PH610080Vi^K01@0=CDe05=CD`26QXH0LW9b03Ph>00e=CD0=cLg03dm?@0n
?Sh0@T9204E5A@18B4P0ATI604U9B@1ADE40IVIV08Z:RP30`<00gMgM0;Vi^@80^[Zj0100UiNG06e]
K@1IFET0C4a<04i>CP1CDe<0EeMG055AD@26QXH0V9RH0;Vi^@34a<@0b<S809bLW00T92@05ADE0P0D
51@301<C4`0801DE5@0C4a<05ADE01TI6@0[:b/0U9BD0;Bd]01mOGd209RHV0130:NWY`18B4P0<c<c
04=3@`1BDU80CTi>06EUI@22PX80XJ6Q07inOP2FUYH0Q8B403hn?P0N7Qh08B4Q02LW9`0f=SH0H61P
0;Bd]032`/80Xj>S08f=S@2EUID0Q8B407EeM@2d];@0ZJVY05MGE`0o?cl0Cdm?01hN7P0T92@08b<S
02d];@1BDU80BDU9061PH02ATI40HF5Q04E5A@1NGUh0V9RH0=gMg@3?cll0Fe]K02`/;00`<300H61P
03Xj>P0n?Sh0Cdm?02/[:`0T92@0<C4a0:ZZZP29RHT0U9BD07MgM`20P800XJ6Q09^KV`20P800e=CD
0<k>cP1QHF40:RXZ01PH600201XJ6P0<02h^;P1SHf<0F5QH01hN7P0I6AT071`L01dM7@0J6QX0?clo
04I6AP2ATI405aLG0P0E5AD0DP0M7Ad0IfMW02`/;00l?3`0D51@04=3@`0j>SX0M7Ad07=cL`1PH600
BDU903/k>`0];Bd0;2`/02d];@0Q8B408b<S03hn?P20P800Bd];07alO01FEUH0B4Q806e]K@18B4P0
YZJV0=7Ad@3EeMD0`/;206UYJ@0^;Rh0;Bd]0410@017AdL0L71`0;Zj^P23Ph<0Lg=c08B4Q01WIfL0
K6a/06=SH`2?Shl0_;bl05eMG@0O7al092@T04i>CP2OWil0RXZ:075aL@1WIfL05aLG01HF5P0N7Qh0
EUIF06YZJP0l?3`0C4a<04M7A`1RHV80ATI605YJFP2XZ:P0aLG509jNWP1DE5@0C4a<069RHP1`L700
QHF505MGE`13@d<0;bl_01lO7`0R8R80>c/k09jNWP35aLD0/;2`08B4Q01SHf<209>CT`0N03Hf=P0S
8b<07AdM01XJ6P0M7Ad0A4A40:JVYP3;bl/0YZJV09JFUP2;Rh/0EUIF04Q8B01ADE40IfMW08N7Q`2Y
ZJT0?clo02PX:00R8R80E5AD05UIF@2=SHd0Rh^;0;6a/@2ATI40Ogmo06MWI`1YJFT0?3`l4?ooool0
00kooooo03<0JFUY06m_K`1mOGd0G5aL03dm?@0k>c/0O7al08:2PP1iNGT0SXj>02`/;01=CDd0ADE5
028R8P0o?cl08R8R02l_;`0J6QX0@D5103hn?P0];Bd0GemO01@D5016ATH08R8R04i>CP1ZJVX0GUiN
02<S8`0l?3`09BDU04Q8B01IFET05QHF03<c<`1[Jf/08B4Q04i>CP0U9BD0?Cdm02XZ:P14A4@0<S8b
04E5A@0k>c/0<C4a05=CD`0j>SX0I6AT05YJFP2UYJD00P1MGEd0B018B4P0CTi>059BDP0W9bL0IfMW
07UiN@1kNg/0AdM7071`L00f=SH0EEEE0861P@16ATH0<C4a04Y:BP0[:b/0DE5A06][J`1;Bd/0FUYJ
0410@00l?3`0O7al03Lg=`1QHF40CDe=02PX:025QHD0P820055AD@2][Jd0?Shn05mOG`1DE5@0CDe=
06EUI@2NWYh0Kfm_06EUI@18B4P0@T9207MgM`2OWil0bLW906m_K`0S8b<0Fe]K05aLG01=CDd0A4A4
05aLG017AdL0GEeM07=cL`11@D40BTY:059BDP1<C4`0U9BD04=3@`1/K6`0SHf=059BDP1VIVH0ATI6
04e=C@1SHf<0Bd];05EEE@1FEUH0YZJV07YjNP80IfMW09L0DU9B08F5Q@1kNg/0GemO06][J`15ADD0
ZZZZ06QXJ00e=CD0L71`03Lg=`16ATH0Ogmo0410@01^KVh0GEeM03hn?P1HF5P0MWIf06i^KP0f=SH0
Fe]K04A4A01cLg<0IfMW07EeM@1cLg<0EeMG075aL@1XJ6P0Bd];09ZJVP2?Shl0KFe]061PH02GUiL0
Ph>308b<S02MWId0QHF505MGE`1nOWh0TI6A06=SH`34a<@0Vi^K03hn?P1dM7@0IfMW05EEE@1XJ6P0
?Cdm03@d=01dM7@0@D5107AdM00o?cl0EEEE05QHF00c<c<0>CTi05=CD`2l_;`0_[jn05QHF00k>c/0
M7Ad05MGE`1PH600?Cdm02d];@1WIfL0=cLg07inOP1UIFD0OWin09FEU@1PH600MgMg09VIV@2XZ:P0
D51@0492@P17AdL0?3`l079bLP16ATH0=SHf06YZJP0l?3`0;bl_05mOG`1TI6@09RHV03De=@19BDT0
FEUI01dM7@1QHF40PH6104e=C@1RHV80[Jf]03Lg=`1^KVh0PX:204M7A`1`L700G5aL05ADE01JFUX0
O7al03Lg=`1?Cdl0Ogmo04I6AP0X:2P0HV9R03/k>`0j>SX0Lg=c07IfMP0Z:RX0Ogmo05eMG@11@D40
CTi>079bLP0L71`0IVIV08:2PP1[Jf/0ATI60861P@0c<c<0LG5a096AT@1LG5`0DE5A0;Bd]021PH40
Ng]k065QH@1EEED0>3Ph03hn?P1`L700C4a<03@d=02EUID0QhN700kooooo04@0[jn_05EEE@14A4@0
;bl_02`/;01FEUH0=3@d01HF5P0D51@0DU9B01lO7`0o?cl0ADE504];B`1BDU807alO01@D500C4a<0
4Q8B01@D500>3Ph04a<C018B4P0e=CD0UIFE0:JVYP2SXj<0IfMW028R8P1KFe/0W9bL06a/K01GEeL0
@T92034a<@0X:2P071`L02DU9@0n?Sh09BDU01dM7@0e=CD0IFEU02h^;P1:BTX0`l?30=?Cd`2ZZZX0
QHF5071`L01DE5@0`/;20<30`01]KFd0B4Q80492@P1>CTh0IFEU07inOP2TY:@0aLG508^;R`2XZ:P0
WYjN01`L700A4A405ADE014A4@804a<C00L06a/K01TI6@0I6AT05ADE05eMG@2g]kL0/[:b0080H61P
04D0[:b/0<k>cP3AdM40SXj>05]KF`1EEED0C4a<04M7A`1[Jf/0`/;208J6QP1dM7@0JVYZ06UYJ@2L
W9`0IVIV04Q8B01_Kfl0M7Ad0820P02f][H0aLG50<[:bP29RHT0R8R807inOP2:RXX0>CTi030`<01L
G5`0KFe]01LG5`0C4a<04A4A010@400I6AT0=SHf03Ph>00U9BD0OWin059BDP0O7al0EeMG0:b/[03?
cll0/k>c0:ZZZP0m?Cd0:RXZ02@T901=CDd0:b/[03Ti>@10@4008b<S02HV9P2c/k<0T92@08N7Q`1;
Bd/0H61P09jNWP1iNGT0Ti>C09BDU02YZJT0Ng]k01LG5`0D51@00P0G5aL02`0K6a/0<c<c06YZJP1?
Cdl06QXJ01/K6`0M7Ad08B4Q05iNGP1/K6`0/[:b00804a<C00d05ADE01LG5`0g=cL0;Rh^02`/;01=
CDd0=CDe02`/;01iNGT0P82003Ph>00X:2P0:RXZ0080;bl_00P0=CDe03`l?015ADD0F5QH09bLW020
P800JFUY04=3@`80=SHf01d0VIVI07alO02GUiL0WIfM0;6a/@35aLD0V9RH02`/;00d=3@0:BTY03Lg
=`3>c/h0VYZJ03Ti>@0k>c/0H61P09VIV@3AdM40Z:RX0;Fe]@0i>CT09bLW051@D021PH40YJFU07Yj
NP0P8200>c/k01LG5`0201DE5@0k01LG5`1>CTh0EEEE03Ph>00l?3`0KVi^0:NWY`2i^KT0a/K60<O7
a`25QHD0BDU907inOP2UYJD0Hf=S02HV9P0l?3`0HF5Q04A4A00h>3P0?Cdm05]KF`2IVIT0]kNg0;Rh
^01[Jf/0GUiN03/k>`1@D500>c/k01`L700I6AT06a/K0451@@1iNGT0Lg=c04];B`0a<C40=CDe04=3
@`19BDT0U9BD065QH@0k>c/0O7al05YJFP1BDU80Hf=S06][J`1iNGT0B4Q80:BTY02ZZZX0A4A403`l
?01VIVH0D51@02XZ:P0J6QX04?ooool000kooooo02X0L71`05EEE@1^KVh0Bd];02d];@0Y:BT0Cdm?
06][J`1]KFd0MgMg03Ph>01dM7@0P82004M7A`1;Bd/0=3@d06IVIP0U9BD0=CDe00h>3P0N7Qh0IVIV
02DU9@12@T808R8R02DU9@0a<C40EEEE02PX:00f=SH0:BTY061PH01RHV806a/K030`<01PH600820P
06=SH`0h>3P0;Bd]03Hf=P1_Kfl205EEE@2Y01lO7`0L71`0>CTi03<c<`1RHV80<S8b059BDP1PH600
DU9B0492@P1?Cdl0IVIV01TI6@11@D40FUYJ05ADE00h>3P0Lg=c034a<@0k>c/0GemO02LW9`0V9RH0
ADE502HV9P0`<300Bd];0451@@1IFET0@T9201dM7@1TI6@0<30`051@D01FEUH08b<S06=SH`29RHT0
A4A40<?3``1WIfL0KVi^05UIF@1MGEd0EUIF09fMW@2<S8`0K6a/04e=C@17AdL0NWYj096AT@2c/k<0
B4Q803De=@1iNGT0HV9R05YJFP1UIFD0MgMg04Q8B01IFET0Shn?06EUI@1SHf<0LG5a04I6AP2PX:00
Cdm?06i^KP2=SHd0De=C08:2PP1?Cdl0De=C07EeM@1OGel0F5QH06EUI@2CTi<0NGUi0820P01RHV80
CTi>0861P@1jNWX0DU9B05YJFP11@D40Zj^[05YJFP0h>3P0MGEe03hn?P15ADD0Jf][03Ph>027QhL0
Ph>304a<C00h>3P0Ogmo071`L00k>c/0Fe]K04U9B@28R8P0Lg=c08F5Q@2<S8`0MWIf06e]K@1`L700
CTi>08Z:RP2=SHd0IVIV07MgM`2QXJ40Q8B408j>SP2ZZZX0Shn?05UIF@2?Shl0T92@061PH036a/H0
QhN704A4A01mOGd0Jf][02h^;P1@D500>SXj02d];@1^KVh0?3`l06EUI@12@T80DU9B05iNGP13@d<0
>SXj05YJFP2CTi<0];Bd08>3P`17AdL0R8R805QHF01dM7@0E5AD030`<01eMGD0>c/k07EeM@1ADE40
PH6109JFUP1VIVH0Jf][06a/K01nOWh0@D5105]KF`15ADD0;Rh^05IFEP0203Hf=P0o07inOP1^KVh0
ADE504e=C@1LG5`05aLG03hn?P1KFe/0HV9R02l_;`0o?cl0GEeM04U9B@1EEED0/[:b0451@@1QHF40
L71`04I6AP1WIfL0Cdm?05IFEP23Ph<0Shn?05=CD`0o?cl0Fe]K02/[:`0a<C40D51@055AD@1:BTX0
Kfm_09RHV01DE5@0S8b<05mOG`0c<c<0=cLg06MWI`0f=SH0LW9b08:2PP1fMWH0E5AD09RHV010@400
F5QH0861P@0a<C40:RXZ08B4Q02IVIT0XZ:R07IfMP0o?cl06ATI02XZ:P1NGUh0@41002<S8`1NGUh0
KVi^00kooooo01<0TI6A07]kN`1_Kfl0EEEE075aL@0c<c<06QXJ01LG5`0Y:BT0PH6101dM7@0G5aL0
>SXj04=3@`19BDT0DU9B01XJ6P0A4A404a<C00803Ph>00d0410@00h>3P0c<c<0bl_;0:VYZ@2QXJ40
AdM701hN7P10@400BTY:02/[:`0J6QX05ADE00804a<C00@051@D01LG5`0I6AT06ATI0P0K6a/03@0^
;Rh0HF5Q02`/;02c/k<0YZJV071`L01cLg<0HF5Q0492@P2FUYH0QXJ605mOG`1HF5P00P1JFUX0D@1^
KVh0JFUY06e]K@1iNGT0W9bL0:n_[`2:RXX0De=C03Lg=`0m?Cd0@T9203lo?`0h>3P0?Cdm03Ph>00e
=CD0;bl_03De=@1OGel0[Zj^04A4A00V9RH0WYjN0<K6aP35aLD0Zj^[071`L01/K6`0JFUY05eMG@0n
?Sh0<C4a04e=C@0i>CT0ATI607]kN`1jNWX0X:2P09BDU02BTY80]KFe0;fm_@3?cll0`L710;fm_@2E
UID0Ti>C0;Jf]P1;Bd/0CTi>06MWI`16ATH0C4a<061PH016ATH07AdM01<C4`0D51@0820P06=SH`0g
=cL0Ti>C08F5Q@1KFe/0Ng]k07UiN@2FUYH0RXZ:06IVIP1fMWH0EEEE02/[:`13@d<0@D5103<c<`0M
7Ad0:RXZ01dM7@1RHV80;Rh^03@d=00205]KF`030820P02JVYX0SHf=0080:BTY00@0BDU904m?C`0R
8R805ADE0P0F5QH0200M7Ad0?3`l06IVIP10@4007AdM02DU9@0/;2`0C4a<0P3:b/X0=`18B4P07AdM
03Ph>00/;2`0;Rh^02@T900W9bL0ATI602DU9@0c<c<0Rh^;0:2PX01KFe/0Hf=S06MWI`1MGEd0FUYJ
075aL@28R8P0W9bL0;bl_02VYZH0ADE50451@@0/;2`08R8R02TY:@2HV9P0IfMW03Lg=`10@400W9bL
0:ZZZP2e]KD0D51@020P800K6a/0@T920<30`018B4P05aLG01DE5@0R8R80EeMG09NGU`24Q8@0Y:BT
03<c<`1^KVh0NWYj03/k>`0d=3@0IVIV01lO7`0[:b/0100E5AD0>P0G5aL0Hf=S0410@00k>c/0?Cdm
0:^[Z`3EeMD0f=SH0=GEe@2e]KD0_kno0<g=c@1TI6@08b<S01`L700O7al0:RXZ04I6AP1YJFT0Q8B4
09jNWP2KVi/0/k>c096AT@12@T807QhN01TI6@0Y:BT0=CDe01/K6`0N7Qh0=CDe06e]K@1NGUh0Jf][
01dM7@0Y:BT0:2PX03/k>`1UIFD0YZJV06][J`0a<C40C4a<03Ti>@12@T80DU9B07AdM02IVIT0L71`
07emO@1[Jf/0Xj>S092@T02DU9@0TI6A09:BTP21PH4@ooooo`003_ooool0iP2DU9@0FEUI09>CT`1G
EeL0JVYZ03<c<`1oOgl0@41006=SH`1^KVh0:BTY0410@01JFUX0>3Ph03Ti>@0U9BD0E5AD02DU9@0]
;Bd0:b/[02DU9@1ZJVX0:2PX03hn?P0m?Cd0>c/k03lo?`1SHf<0;Bd]03`l?016ATH0De=C05eMG@0J
6QX0:BTY05IFEP0S8b<0CDe=03Hf=P0`<300;2`/05YJFP1DE5@0HF5Q05eMG@0f=SH0A4A40451@@1K
Fe/0H61P06IVIP1ZJVX0De=C03Xj>P1LG5`0Kfm_0451@@12@T80JVYZ05ADE00n?Sh0I6AT03lo?`0l
?3`0K6a/038b<P0^;Rh0E5AD03lo?`0c<c<0Fe]K03dm?@1SHf<0LG5a02PX:01jNWX0<c<c06IVIP1d
M7@0<c<c06IVIP23Ph<0>3Ph0;:b/P1IFET0NGUi07YjNP1ZJVX0HV9R09nOW`1NGUh0E5AD05iNGP1O
Gel0RHV908:2PP32`/80NGUi03Ph>01kNg/0G5aL04U9B@1MGEd0NWYj03dm?@1NGUh0MWIf05IFEP1O
Gel0M7Ad0492@P24Q8@0ADE506MWI`22PX80DE5A05aLG012@T80HV9R07EeM@1:BTX0EeMG04e=C@23
Ph<0HV9R06i^KP1;Bd/0ATI607QhN01WIfL0Hf=S04m?C`15ADD0XZ:R071`L00f=SH0KVi^030`<018
B4P0Lg=c04Y:BP1_Kfl0LW9b04I6AP0b<S80Ogmo06=SH`0m?Cd0FUYJ04M7A`2<S8`0Jf][071`L01n
OWh0G5aL06UYJ@25QHD0E5AD08j>SP1jNWX0FUYJ06m_K`2=SHd0FEUI06a/K02e]KD0JFUY0410@01f
MWH0NWYj05EEE@2l_;`0P82003hn?P1`L700IFEU034a<@1<C4`0?Shn03dm?@21PH40BDU906a/K00c
<c<0CDe=05MGE`16ATH0=3@d04a<C02KVi/0Y:BT06EUI@11@D40R8R805QHF01GEeL0F5QH03Lg=`26
QXH0DU9B0861P@1ADE40Jf][08F5Q@1[Jf/0KVi^08N7Q`2?Shl0HF5Q03Ti>@1IFET0>3Ph065QH@1X
J6P0;bl_06i^KP1XJ6P0ATI6051@D01ZJVX0FEUI02h^;P16ATH0J6QX02PX:016ATH0GUiN06EUI@1C
De<0/[:b0P16ATH0501mOGd0F5QH06i^KP1XJ6P0De=C061PH01oOgl0G5aL055AD@27QhL0EEEE020P
8011@D40?3`l02LW9`1<C4`0Vi^K05UIF@25QHD0Jf][0P12@T80601nOWh0@T9205ADE02;Rh/0G5aL
03Xj>P22PX80Bd];05YJFP1fMWH0De=C03/k>`2QXJ40S8b<09bLW01_Kfl0GUiN01lO7`0T92@0EeMG
03dm?@0[:b/0MgMg07inOPkooooo0180ZZZZ06a/K01LG5`0PX:205ADE01]KFd0F5QH04Q8B00n?Sh0
>SXj03/k>`0c<c<09BDU024Q8@0e=CD0BTY:0451@@0E5AD2010@400M018B4P0J6QX0<C4a08^;R`2V
YZH0WYjN0;2`/02JVYX0Jf][04Q8B00R8R805ADE01<C4`0A4A40410@014A4@0G5aL061PH01TI6@0H
61P05aLG01HF5P0I6AT09bLW07EeM@39bLT0GEeM02`/;00i>CT00P1fMWH04P1BDU80;Rh^01LG5`0H
61P0820P038b<P1LG5`0KFe]0:n_[`2l_;`0ZJVY08:2PP0];Bd0IFEU02<S8`0J6QX05ADE01HF5PL0
5ADE00T0<30`04e=C@1RHV80GEeM0;Jf]P1kNg/0>c/k01PH600G5aL00`0E5AD0=00A4A40CDe=04m?
C`12@T80A4A409RHV02=SHd0SXj>07AdM02CTi<0Zj^[0:NWY`2][Jd0OWin05aLG015ADD0E5AD0451
@@0L71`0<30`061PH00X:2P06a/K028R8P0];Bd0<30`02/[:`0k>c/0XZ:R09jNWP2/[:`0[jn_05UI
F@1JFUX0Bd];02PX:00`<300@D5103lo?`1<C4`0EUIF03lo?`1NGUh0MGEe04A4A01HF5P0RXZ:06m_
K`1/K6`0NGUi08:2PP1VIVH203dm?@0803`l?00C4a<051@D01LG5`0M7Ad0A4A405mOG`0S8b<201TI
6@11024Q8@11@D40Kfm_06UYJ@1MGEd0QhN70;6a/@2VYZH0U9BD06=SH`1_Kfl0Rh^;06i^KP1DE5@0
?Cdm038b<P2FUYH0YJFU09ZJVP2d];@0e=CD065QH@1CDe<0Bd];0492@P0m?Cd0@D5104Q8B01@D500
GUiN09fMW@1OGel0;bl_02h^;P1GEeL0Shn?05QHF00L71`06ATI05IFEP28R8P0U9BD0:f][@2`/;00
R8R807inOP1iNGT0ZJVY06IVIP0_;bl0=SHf03dm?@0o?cl0FUYJ07]kN`2b/[80VYZJ03/k>`0K6a/0
5QHF01`L701>CTh0>CTi02/[:`0F5QH0100E5AD0>@0S8b<0IVIV07IfMP15ADD0MgMg0<K6aP2e]KD0
YjNW0<C4a02k^k/0LW9b034a<@0H61P06a/K020P800b<S80C4a<04i>CP1UIFD0Vi^K05EEE@1oOgl0
H61P02`/;00K6a/05aLG01/K6`0S8b<0C4a<04=3@`1^KVh0M7Ad08f=S@17AdL0BTY:0410@01<C4`0
HV9R06YZJP1VIVH0H61P04e=C@1mOGd0>c/k03De=@18B4P0M7Ad09ZJVP1=CDd07alO02LW9`1oOgl0
JVYZ02d];@0V9RH0:b/[06MWI`0@ooooo`003_ooool00`1mOGd0E5AD06i^KP0204i>CP3703dm?@2D
U9@0<c<c04Q8B01^KVh0=CDe04m?C`1=CDd09bLW05UIF@0a<C40@41002<S8`0h>3P0820P01`L701S
Hf<0820P03Ti>@0Y:BT0@d=304Q8B01CDe<06a/K0451@@0l?3`0Hf=S04i>CP0F5QH0?Shn06EUI@0_
;bl0@T9202HV9P0j>SX0;Rh^04];B`0^;Rh0A4A404Y:BP0d=3@0D51@04A4A00j>SX0DE5A0820P01K
Fe/0D51@03Hf=P1CDe<0HV9R02<S8`1@D500IVIV055AD@0e=CD0@d=30451@@1:BTX0NGUi034a<@0L
71`0>SXj04e=C@12@T80M7Ad038b<P1;Bd/0E5AD02d];@1fMWH0=SHf05UIF@1SHf<0:BTY069RHP21
PH40>CTi08V9R@1KFe/0IfMW06QXJ01EEED0IVIV0;Rh^029RHT0HV9R04U9B@17AdL0L71`09BDU036
a/H0MGEe034a<@1`L700O7al04E5A@1EEED0MgMg03/k>`1DE5@0Jf][061PH018B4P0LG5a03Hf=P2?
Shl0CDe=05MGE`25QHD0DE5A05aLG018B4P0C4a<06a/K01;Bd/0CDe=05ADE01^KVh0NWYj07]kN`1]
KFd0LW9b07EeM@1dM7@0EEEE05iNGP0l?3`0S8b<07=cL`0b<S80P8200410@017AdL0LG5a06ATI01^
KVh0Ng]k059BDP0T92@0Bd];05iNGP0e=CD0GUiN0410@02;Rh/0NWYj07EeM@2=SHd0NGUi061PH01m
OGd0EEEE08n?S`1mOGd0MgMg08B4Q02VYZH0J6QX04Y:BP3:b/X0Ogmo069RHP1gMgL0Rh^;059BDP3K
fm/0Z:RX04m?C`1lO7`0PH6103De=@1aLG40BTY:02XZ:P1jNWX0EEEE06i^KP0a<C40?Cdm05EEE@0i
>CT0@D5104i>CP2MWId0/;2`04i>CP0k>c/0S8b<07EeM@1PH600?Cdm02@T901[Jf/0;Bd]05aLG01<
C4`0NGUi07AdM002069RHP0X08N7Q`1gMgL0Cdm?05QHF01_Kfl0?clo06a/K01DE5@0=CDe06][J`2:
RXX0I6AT05MGE`1SHf<0EEEE03`l?011@D40Ogmo04a<C014A4@0DU9B08:2PP1>CTh0/k>c06QXJ00m
?Cd0Shn?06MWI`26QXH0JVYZ06EUI@1^KVh0Lg=c03Xj>P0g=cL0LG5a05YJFP0Q8B40>c/k03Lg=`80
=SHf00l0ZJVY06QXJ01^KVh0RHV905IFEP0m?Cd0Jf][03Lg=`0m?Cd0SXj>07alO012@T80TI6A04M7
A`1BDU800P2;Rh/03@1DE5@0Kfm_07alO02DU9@0ZZZZ06i^KP0W9bL0;2`/05eMG@1GEeL0:2PX07If
MP1nOWh03_ooool09@2b/[80I6AT04m?C`1@D500EeMG08>3P`0`<30071`L01TI6@0E5AD04A4A018B
4P0T92@0@T92038b<P0V9RH0CTi>05iNGP0[:b/0=cLg04a<C01BDU80@d=304E5A@0Y:BT0F5QH0:6Q
X@2BTY80YJFU08R8R00O7al051@D01DE5@0B4Q804a<C00h>3P0D51@00P0E5AD201HF5P0M01DE5@0H
61P05QHF075aL@1VIVH0>SXj01HF5P0G5aL0=cLg05QHF02][Jd0SHf=05iNGP1=CDd0HV9R061PH01F
EUH0MgMg051@D01ADE40[Jf]08N7Q`2g]kL0TY:B01TI6@0E5AD04a<C01DE5@0C4a<01P0E5AD0200A
4A406QXJ06UYJ@2ZZZX0ZJVY09RHV00X:2P05ADE0P0C4a<09@0E5AD04A4A01DE5@13@d<07AdM02@T
901:BTX0NGUi03dm?@1RHV809BDU02l_;`1<C4`0LG5a05MGE`1gMgL0J6QX04e=C@0m?Cd0HF5Q02/[
:`13@d<092@T02XZ:P0c<c<09RHV02<S8`0Q8B407AdM06MWI`1TI6@0HF5Q07moO`3?cll0RHV906YZ
JP1SHf<00P1BDU80G01?Cdl0EeMG05EEE@1ADE40Cdm?051@D02TY:@0YZJV0::RXP2@T900_;bl06][
J`0l?3`07AdM01`L700G5aL0>CTi024Q8@0C4a<0410@01<C4`0D51@06ATI04I6AP1nOWh0>c/k01`L
700O7al0D51@05]KF`2c/k<0cLg=0<G5a@24Q8@0;Bd]01lO7`1LG5`0SXj>09>CT`24Q8@0WYjN0<30
`03Fe]H0e=CD05iNGP1@D500WinO0<C4a02_[jl0D51@03hn?P0/;2`09bLW02/[:`0W9bL0;Bd]03@d
=021PH40GEeM09fMW@2^[Zh0Shn?05IFEP0T92@05aLG04a<C013@d<0=3@d02PX:01HF5P0Z:RX07=c
L`2g]kL0aLG50;Zj^P2JVYX0EUIF05UIF@1fMWH0QXJ60<[:bP3MgMd0d];B0:2PX02`/;00TY:B08B4
Q02<S8`0a/K609nOW`0i>CT0:RXZ100E5AD06@0C4a<05ADE07emO@32`/80]KFe0=GEe@3Lg=`0_;bl
07IfMP10@400@D5106=SH`1aLG40;Rh^06ATI01]KFd0?clo02`/;00Y:BT0ADE506UYJ@1VIVH0Cdm?
03<c<`0K6a/00P0I6AT07P0O7al0Bd];08N7Q`1jNWX0[Zj^06ATI00m?Cd0Rh^;08>3P`1NGUh0>c/k
038b<P0^;Rh0>SXj09JFUP2k^k/0I6AT04Y:BP1bLW80[Jf]075aL@0m?Cd071`L034a<@1SHf<06ATI
01DE5@0F5QH08B4Q07AdM13ooooo000>ooooo`0U09RHV01<C4`0Ogmo05EEE@1kNg/0Bd];0:BTY017
AdL0E5AD06a/K00_;bl0>c/k04M7A`0Z:RX0A4A402DU9@12@T806a/K02/[:`0S8b<071`L071`L00d
=3@0<30`01TI6@0f=SH0A4A405eMG@0_;bl0=SHf024Q8@17AdL0BDU901hN7P0V9RH0D51@02PX:002
03`l?02H028R8P0K6a/0Cdm?04a<C01@D500=3@d02PX:00l?3`0Cdm?03Ti>@1;Bd/0IFEU051@D01I
FET0=SHf059BDP17AdL0:BTY03Lg=`1QHF40C4a<02TY:@1@D500@D5102`/;01PH600;Rh^01lO7`0j
>SX0<c<c02TY:@1GEeL0:2PX051@D01?Cdl06QXJ05mOG`0b<S80E5AD07IfMP0d=3@0F5QH08B4Q00j
>SX0OGem06m_K`1]KFd0De=C05IFEP11@D40/k>c08n?S`1TI6@0E5AD04a<C020P800YJFU0<?3``1/
K6`0?3`l07=cL`1mOGd0DU9B05YJFP25QHD0ADE505IFEP1fMWH0IVIV04i>CP1iNGT0=SHf08J6QP1J
FUX0G5aL07IfMP1BDU80FEUI04M7A`1?Cdl0P82006QXJ01@D500Hf=S06][J`1aLG40PX:207UiN@1c
Lg<0NGUi06QXJ01ZJVX0E5AD03hn?P2DU9@0K6a/03/k>`29RHT0@T92059BDP1^KVh0Cdm?07UiN@2A
TI40B4Q802l_;`13@d<0De=C030`<01LG5`0?Cdm096AT@1VIVH0FUYJ07moO`1_Kfl0I6AT05YJFP1E
EED0VYZJ07MgM`1aLG40Q8B40::RXP1;Bd/0B4Q80=OGe`2HV9P0J6QX07inOP1QHF40@D5107=cL`1/
K6`0@D5109FEU@1oOgl07alO03hn?P0l?3`0=cLg06][J`11@D40K6a/034a<@14A4@0Jf][02DU9@0f
=SH0@d=308^;R`2?Shl202d];@1E0861P@1FEUH0J6QX04E5A@0W9bL0LW9b02d];@1JFUX0BTY:09:B
TP2:RXX0MWIf06m_K`2EUID0PH6105aLG00g=cL0Kfm_04M7A`1`L700FEUI03Ti>@1_Kfl0NGUi05AD
E01NGUh0F5QH06=SH`1<C4`0IFEU08J6QP1ADE40?Cdm0492@P1SHf<0?Cdm0:RXZ01[Jf/0>c/k08J6
QP1EEED0JVYZ07UiN@1TI6@0Lg=c06i^KP1SHf<0>SXj07AdM01DE5@07alO03hn?P1MGEd0B4Q803/k
>`2WYjL0HV9R05aLG02ATI40FEUI0451@@1fMWH0EEEE03<c<`2>SXh0Ng]k04a<C027QhL0D51@069R
HP2LW9`0WIfM05]KF`2e]KD0N7Qh08n?S`2a/K40Lg=c02PX:00Z:RX0KVi^04Y:BP0i>CT0RXZ:08:2
PP0>ooooo`0:0<K6aP1HF5P0>CTi02l_;`2XZ:P0V9RH030`<00L71`05QHF01DE5@804A4A01@03Ph>
01DE5@0j>SX0F5QH06a/K024Q8@0GemO02`/;00U9BD09RHV02`/;00W9bL05QHF06][J`27QhL0Zj^[
09VIV@22PX80Cdm?01lO7`@05ADE00<05QHF01DE5@0D51@00P0E5AD06P0C4a<05ADE01TI6@1dM7@0
ATI603Lg=`0I6AT05ADE01`L7010@400/[:b0:2PX01cLg<0GemO06QXJ014A4@092@T038b<P1`L700
;Bd]03Lg=`2<S8`0cLg=0<?3``1JFUX07AdM0P0E5AD00`0D51@05ADE01DE5@0201DE5@804a<C00L0
8R8R04i>CP2YZJT0`l?30:JVYP2RXZ80:RXZ00<04a<C01/04Q8B01<C4`0e=CD0A4A401hN7P0M7Ad0
;bl_06m_K`0o?cl0LW9b02/[:`0Y:BT0=3@d04Y:BP1oOgl0NGUi096AT@10@400FUYJ05=CD`11@D40
9RHV018B4P0G5aL0=CDe02TY:@0R8R800P0P8200701MGEd0MGEe05]KF`1lO7`0Ogmo03Xj>P1ZJVX0
7QhN01DE5@0D51@061PH02PX:00k>c/0<c<c04i>CP1lO7`0YZJV0=?Cd`3Fe]H0b/[:08b<S00T92@0
6ATI01DE5@0M7Ad0;2`/0492@P0F5QH201DE5@1:01<C4`0B4Q804A4A02HV9P1>CTh0RHV903dm?@0K
6a/0DE5A06QXJ03@d=00e]KF07moO`14A4@0<C4a04];B`0l?3`0TY:B0:6QX@2EUID0QXJ60:JVYP3D
e=@0b/[:04I6AP1XJ6P0>SXj02`/;01`L700_;bl05UIF@0k>c/0?Cdm03Ti>@0d=3@0?3`l05QHF02D
U9@0ZJVY0<_;b`2i^KT0Bd];02LW9`0T92@0AdM704a<C00`<300;bl_02HV9P0U9BD0W9bL0<;2`P2L
W9`0Xj>S0:VYZ@2b/[80^;Rh0:JVYP10@400GEeM0=;BdP2/[:`0T92@04i>CP1KFe/0PX:209>CT`2G
UiL0aLG50<k>cP1aLG40>CTi01HF5P0C4a<201DE5@0T01@D500H61P0DU9B0>7Qh@3Rh^80h>3P0>7Q
h@2PX:00?Cdm02HV9P0/;2`0;Rh^055AD@2IVIT0NWYj038b<P0T92@09bLW01lO7`0^;Rh0IVIV08J6
QP14A4@0=3@d01/K6`0I6AT06a/K03hn?P1`L700UiNG0;^k^`20P800<S8b08B4Q02c/k<0IfMW0P0^
;Rh03P0[:b/0;Rh^038b<P29RHT0a<C40<;2`P31`L40XZ:R0;Jf]P1@D500Jf][02<S8`1PH600@410
0P0F5QH00`0I6AT06QXJ05iNGP0@ooooo`003_ooool0A02g]kL0CTi>09NGU`1ADE40Y:BT06i^KP2U
YJD0AdM705eMG@2:RXX0@41003`l?01?Cdl0<S8b03hn?P0h>3P0DE5A02TY:@0/;2`09RHV01hN7P1U
IFD0<30`02h^;P0b<S80=3@d03Hf=P1VIVH0<C4a03dm?@0n?Sh0@d=304m?C`0R8R80820P0451@@0b
<S80>CTi04U9B@0X:2P08b<S05UIF@1OGel0De=C04];B`0h>3P0A4A405iNGP13@d<0FUYJ079bLP1M
GEd0J6QX03<c<`1MGEd0J6QX03lo?`0l?3`0KVi^03hn?P0_;bl0K6a/03hn?P0Z:RX0KFe]03Hf=P0b
<S80D51@0P0Y:BT0J`1JFUX09BDU051@D01GEeL08B4Q06IVIP0[:b/0G5aL07MgM`0];Bd0DE5A08j>
SP15ADD0MGEe0861P@1NGUh0LW9b075aL@1JFUX0[jn_0861P@1ADE40K6a/08J6QP2JVYX0TI6A0<W9
b@2;Rh/0A4A407=cL`1TI6@0HF5Q079bLP1oOgl0ATI6065QH@1jNWX0GEeM04e=C@1^KVh0?Cdm08j>
SP1IFET0GemO08V9R@1>CTh0I6AT0492@P1=CDd0MWIf04i>CP1ADE40GemO08j>SP1>CTh0GemO06EU
I@1JFUX0NWYj069RHP1ADE40=CDe04Q8B02YZJT0Ogmo03<c<`1`L700:2PX06EUI@1^KVh0HV9R05EE
E@1^KVh0B4Q803Ph>01JFUX0DE5A03dm?@1YJFT0@T9209ZJVP20P800IfMW07inOP1TI6@0JFUY07mo
O`1GEeL0Rh^;05UIF@1VIVH0ZJVY0::RXP0k>c/0ADE50;Rh^02VYZH0HV9R0::RXP1[Jf/0@T9209NG
U`23Ph<0?3`l07UiN@1VIVH0820P0080@d=306<0Cdm?08:2PP18B4P0K6a/028R8P1@D500MgMg03Ti
>@0Y:BT0<c<c07alO028R8P0C4a<03Xj>P1jNWX0BDU9075aL@1=CDd08B4Q07MgM`0_;bl0EEEE03Hf
=P1jNWX0IVIV06][J`1OGel0W9bL08N7Q`1NGUh0=SHf06IVIP1;Bd/0K6a/06IVIP11@D40H61P07mo
O`1NGUh0DU9B05UIF@1OGel0;Rh^0410@01_Kfl0?3`l02XZ:P0k>c/0GUiN03dm?@2;Rh/0E5AD0410
@01ZJVX0@D51079bLP1hN7P0JVYZ075aL@1nOWh0Jf][03Ti>@1[Jf/0De=C01lO7`0n?Sh0FEUI03Ph
>00g=cL0UIFE05IFEP1eMGD0SHf=05EEE@0n?Sh0Kfm_04Y:BP0g=cL0/K6a092@T01@D500Ogmo04=3
@`1hN7P0/k>c08b<S01VIVH0a<C406=SH`1eMGD0TI6A05iNGP0Z:RX0;bl_06=SH`0R8R80=cLg07]k
N`1lO7`03_ooool02@1bLW80I6AT03lo?`1WIfL0RXZ:06IVIP1DE5@0?3`l01LG5`0201@D50<04a<C
01H06ATI08>3P`2<S8`0MWIf0;Bd]01jNWX0?3`l02`/;00];Bd08R8R05YJFP2m_Kd0cLg=0<[:bP1m
OGd0D51@08b<S01?Cdl0<S8b01TI6@0E5AD04Q8B100E5AD0;@0D51@04a<C01DE5@0Z:RX0IfMW03`l
?00[:b/061PH01@D500E5AD0>SXj06IVIP1FEUH0MWIf030`<00O7al06a/K01HF5P0K6a/0?clo05=C
D`0W9bL0GemO0;Rh^02l_;`0N7Qh059BDP0H61P05ADE01<C4`0E5AD06a/K01hN7P0J6QX09BDU04A4
A01kNg/0Q8B40::RXP3AdM40_kno0;Zj^P1NGUh07QhN01LG5`0201DE5@0i01/K6`1fMWH0;Bd]024Q
8@0M7Ad0?Cdm071`L01?Cdl0B4Q804a<C00g=cL0ATI6061PH02DU9@0TY:B096AT@0n?Sh0:RXZ03Ph
>01UIFD05aLG010@400B4Q805QHF02d];@0h>3P0=CDe03dm?@1iNGT0U9BD06YZJP28R8P0>SXj05]K
F`14A4@051@D01<C4`0A4A404a<C01DE5@0M7Ad0:2PX02DU9@0Q8B40;Rh^0=SHf037alL0Xj>S03hn
?P0F5QH05ADE01XJ6P0/;2`0Cdm?03`l?00D51@05aLG00@05ADE06406ATI02LW9`1MGEd0Ti>C03/k
>`1mOGd0`/;20=3@d03EeMD0IVIV05ADE01OGel0NGUi07]kN`1fMWH0LW9b07AdM02][Jd0`/;207If
MP30`<00GEeM028R8P0L71`06a/K01lO7`0_;bl0YjNW06EUI@0i>CT0=3@d03`l?01ADE40[Jf]0:j^
[P2d];@0UYJF08R8R01MGEd06a/K0451@@1BDU80=SHf034a<@0[:b/092@T01dM7@20P800QXJ607mo
O`1_Kfl0IVIV05YJFP1oOgl0^[Zj0::RXP3Bd]80SXj>04];B`0c<c<0FUYJ04e=C@1;Bd/0MWIf09:B
TP2LW9`0OGem06m_K`2:RXX0BDU901dM7@0P8200;Bd]0492@P0i>CT0a/K60=[JfP3?cll0hn?S0>GU
i@24Q8@061PH01/K6`0U9BD0:BTY02h^;P1iNGT0?clo02XZ:P0S8b<0820P01`L700O7al0@41008>3
P`0k>c/0=cLg00805QHF01l06a/K05EEE@2DU9@0Zj^[0;Zj^P2GUiL0XJ6Q07IfMP0e=CD0CDe=06i^
KP13@d<09BDU02PX:015ADD0Zj^[0:j^[P0j>SX0@d=307]kN`2[Zj/0CDe=0;Ng]`26QXH0[:b/02l_
;`0E5AD05QHF01DE5@0J6QX0C4a<013ooooo000>ooooo`3o09^KV`1<C4`0MgMg04];B`26QXH0R8R8
0;Bd]00l?3`0Kfm_09>CT`0k>c/0?clo05IFEP0e=CD0ATI604M7A`1CDe<0<C4a02LW9`0_;bl07alO
06IVIP0b<S80<30`03/k>`0n?Sh0?3`l061PH00C4a<0AdM703hn?P0e=CD0CTi>01lO7`0];Bd0EUIF
03`l?00e=CD0AdM702l_;`0^;Rh0Cdm?04e=C@1SHf<0IfMW03hn?P13@d<0AdM70492@P1SHf<0Q8B4
071`L01<C4`09BDU04I6AP1KFe/0<c<c04E5A@1YJFT0IfMW03Lg=`1;Bd/0>CTi030`<01kNg/0DE5A
01/K6`0g=cL0CTi>02h^;P1IFET092@T065QH@1VIVH0820P06IVIP0a<C40F5QH079bLP0^;Rh0BDU9
08n?S`0f=SH0EUIF06YZJP1@D500GemO05ADE01aLG40[jn_08V9R@1BDU80?Shn04m?C`1oOgl0PH61
0;fm_@2HV9P0@410079bLP1]KFd0BTY:05iNGP2EUID0EeMG055AD@1IFET0=cLg05MGE`2CTi<0ADE5
07emO@10@400IfMW08f=S@1DE5@0EeMG03<c<`1ZJVX0RHV905YJFP1:BTX0BDU909nOW`1MGEd0FEUI
05QHF015ADD0Ng]k079bLP1CDe<0@T9203@d=02>SXh0MgMg03De=@24Q8@0>SXj03hn?P1]KFd0Hf=S
05UIF@1gMgL0Fe]K05EEE@1YJFT0KVi^02XZ:P1@D500>3Ph0861P@1hN7P0IfMW06][J`1aLG40FEUI
08>3P`1ADE40O7al06QXJ01RHV80LG5a096AT@1=CDd0:b/[09nOW`2RXZ80ADE50820P01jNWX0AdM7
09jNWP2?Shl0I6AT09JFUP1mOGd092@T0451@@1@D500;Bd]079bLP1BDU80KFe]02h^;P15ADD0PH61
03lo?`0^;Rh0C4a<07emO@2<S8`0@T9203dm?@1dM7@0JFUY05=CD`10@4008b<S06i^KP0b<S80EUIF
04E5A@1ZJVX0Cdm?05YJFP1LG5`0[Jf]07AdM01HF5P0=SHf04m?C`0d=3@0JFUY06QXJ01FEUH0IVIV
092@T01cLg<0CTi>04];B`1UIFD0>c/k0492@P1gMgL0EeMG02TY:@12@T80NWYj04U9B@2DU9@0O7al
055AD@1[Jf/0MgMg07EeM@25QHD0GUiN08:2PP2<S8`0M7Ad03Lg=`1ZJVX0DU9B024Q8@1>CTh0G5aL
03/k>`0d=3@0QXJ601LG5`1>CTh0VIVI06a/K00m?Cd0I6AT0040<c<c0@0V9RH207UiN@0C04Y:BP1f
MWH0<c<c05EEE@2BTY80PH6107AdM035aLD0P82007YjNP2MWId0GUiN02LW9`0c<c<0E5AD03dm?@0Y
:BT0Kfm_08R8R00>ooooo`0405]KF`1<C4`0Fe]K0:JVYP80GemO00@0<30`03Ti>@0k>c/07alO0P0E
5AD201<C4`0E01HF5P1OGel0H61P09BDU02WYjL0U9BD0;bl_02LW9`0QHF508>3P`2KVi/0c<c<0;fm
_@2g]kL0][Jf04=3@`1KFe/0KFe]02@T900M7Ad05QHF00P05ADE00h0=cLg065QH@0g=cL0;2`/01DE
5@0C4a<06a/K06EUI@0i>CT0>SXj04M7A`0V9RH071`L01XJ6P8071`L04806ATI02d];@0X:2P0=3@d
09NGU`2QXJ40Rh^;06a/K01FEUH07AdM01PH600U9BD0;bl_0410@01DE5@0F5QH05EEE@0h>3P06a/K
03/k>`2TY:@0^;Rh0<O7a`27QhL0FUYJ06ATI012@T809BDU05]KF`1CDe<092@T02d];@0/;2`0GEeM
04m?C`0e=CD06ATI02HV9P1JFUX0Lg=c08R8R02][Jd0/;2`07QhN01ZJVX0H61P06UYJ@2c/k<0M7Ad
03dm?@0k>c/0<C4a04=3@`1XJ6P0GemO0492@P1WIfL0[Zj^0:^[Z`2PX:00CTi>06=SH`0G5aL04a<C
01DE5@0D51@301<C4`0o01/K6`0X:2P09bLW02/[:`2g]kL0N7Qh05]KF`1?Cdl06a/K01DE5@0K6a/0
:BTY05MGE`0a<C405QHF01LG5`0E5AD05QHF01DE5@0A4A405QHF020P800];Bd0Ph>30=;BdP2j^[X0
TY:B07UiN@2f][H0T92@06IVIP22PX80TI6A08B4Q01SHf<0X:2P0:n_[`2/[:`0UiNG0;Rh^03<c<`0
:b/[01DE5@0H61P06QXJ04e=C@1CDe<0K6a/0<W9b@2k^k/0Kfm_059BDP1ZJVX0/[:b0:^[Z`2LW9`0
FUYJ05=CD`1`L700AdM706][J`10@400=SHf0080;2`/01408B4Q02DU9@1_Kfl0GemO06a/K01ADE40
@d=30410@01=CDd0cLg=0<O7a`1YJFT0<c<c02PX:00/;2`0?3`l06UYJ@0208B4Q00n05=CD`0a<C40
>CTi065QH@1aLG40_[jn09:BTP1KFe/0A4A405IFEP32`/80ZZZZ09BDU02n_[h0a<C40=cLg02][Jd0
8B4Q01DE5@0T92@0De=C04E5A@1LG5`0MgMg04I6AP0_;bl07AdM01`L700M7Ad0820P07MgM`1NGUh0
?Cdm01HF5P0G5aL07alO05eMG@2k^k/0`L7108n?S`35aLD0a<C403Hf=P0P82009BDU03Xj>P1CDe<0
A4A402/[:`1RHV80SXj>07EeM@0/;2`0:BTY038b<P1SHf<0EeMG0::RXP3De=@0cLg=05EEE@0L71`2
01DE5@0301HF5P0d=3Cooooo00oooooo000>ooooo`0G09NGU`1;Bd/0LG5a04];B`1GEeL0JFUY0:VY
Z@13@d<0DE5A05EEE@0e=CD0I6AT06=SH`0P8200ADE504A4A00o?cl08b<S030`<01;Bd/092@T08B4
Q017AdL00P0S8b<04P0o?cl0HF5Q071`L00i>CT0;Rh^02DU9@1?Cdl0CTi>02@T900J6QX0EUIF0451
@@1:BTX0=cLg02h^;P0];Bd0FEUI04Y:BP80C4a<0780AdM706IVIP1UIFD0?Cdm04U9B@1bLW80LG5a
06MWI`0f=SH0@41005ADE00`<300=cLg06MWI`1UIFD0<S8b04];B`1>CTh0820P07AdM01GEeL0820P
030`<016ATH0>SXj06UYJ@0n?Sh0D51@04I6AP0M7Ad0JVYZ04E5A@1ADE40Jf][02PX:01DE5@0R8R8
04Y:BP1QHF40M7Ad05mOG`1WIfL0C4a<05]KF`2HV9P0SHf=065QH@0n?Sh0D51@06IVIP2@T900e=CD
0:ZZZP0h>3P0N7Qh06][J`1;Bd/0C4a<08:2PP1VIVH0EUIF05EEE@19BDT0@d=307=cL`0n?Sh0M7Ad
0492@P1@D500Ng]k055AD@1CDe<0>3Ph04U9B@1UIFD0EUIF0410@01?Cdl0K6a/05YJFP1DE5@0JVYZ
05QHF01WIfL0K6a/05aLG01ADE40?Shn09BDU01lO7`0=CDe07moO`15ADD0De=C079bLP1iNGT0EUIF
065QH@11@D40C4a<05IFEP1NGUh0:RXZ05ADE00j>SX0P820075aL@1LG5`0KFe]06i^KP1UIFD0QhN7
05EEE@22PX82079bLP0R07QhN02DU9@0E5AD03hn?P2/[:`0]KFe06EUI@23Ph<0SHf=05IFEP2n_[h0
`L7106m_K`2RXZ80RXZ:02DU9@11@D40FUYJ02/[:`1bLW80I6AT06QXJ00l?3`0BTY:07AdM00d=3@0
;Rh^05EEE@1YJFT0S8b<06e]K@1;Bd/0Ng]k07emO@80De=C0540=cLg08N7Q`24Q8@0UIFE06IVIP24
Q8@0F5QH06ATI01BDU80c/k>0:b/[01^KVh0=CDe04m?C`10@400NGUi07moO`1PH600GUiN08n?S`1`
L700A4A404I6AP1UIFD0@4100492@P1]KFd0E5AD034a<@1;Bd/0R8R804Y:BP1oOgl0O7al059BDP1`
L700FEUI07EeM@1/K6`0Hf=S09bLW02FUYH0LG5a034a<@1GEeL0:2PX02LW9`1VIVH0K6a/03@d=00_
;bl0MWIf05MGE`1WIfL0Z:RX06UYJ@0i>CT0NGUi06EUI@0`<300[:b/07]kN`1:BTX0J6QX071`L01g
MgL0[Zj^09BDU01lO7`0eMGE0::RXP1_Kfl0Zj^[05MGE`1@D500De=C06a/K01WIfL0;Bd]07alO02I
VIT03_ooool0:@2HV9P0^KVi0=?Cd`2QXJ40MgMg06m_K`0l?3`0;Rh^02l_;`0`<3005ADE00h>3P0?
3`l07alO038b<P0U9BD06a/K04Q8B020P800SXj>096AT@2WYjL0/;2`06IVIP1MGEd0Q8B408V9R@36
a/H0`L7107inOP0R8R80QXJ608b<S01JFUX0C4a<03Ti>@0[:b/071`L01LG5`0C4a<051@D00805ADE
00`0?Shn04i>CP0c<c<08B4Q01PH600E5AD0;Rh^05mOG`0U9BD0:BTY03dm?@0[:b/201LG5`1a02l_
;`0O7al05ADE01<C4`0A4A4061PH0820P01fMWH0U9BD08>3P`22PX80UYJF061PH01>CTh0?Shn03<c
<`0P820061PH01HF5P0E5AD06a/K03Hf=P1]KFd0/[:b09bLW02@T900UiNG08F5Q@1fMWH0W9bL0:ZZ
ZP1EEED0@41003Hf=P0j>SX0@T9201dM7@0I6AT05ADE02/[:`0n?Sh0L71`08^;R`2JVYX0]kNg07in
OP2n_[h0UiNG0:>SX`2ZZZX0LG5a065QH@1MGEd0MGEe06e]K@20P800UiNG0:ZZZP2RXZ80[Zj^0;jn
_P35aLD0UIFE034a<@0E5AD04a<C01DE5@0C4a<04A4A010@400A4A405ADE03@d=00W9bL0:b/[09jN
WP1jNWX0Fe]K0:j^[P1`L700DU9B03lo?`1VIVH0K6a/02XZ:P0E5AD061PH01DE5@0G5aL05ADE01<C
4`0F5QH07AdM02PX:01bLW80`l?305aLG01CDe<0BTY:0:^[Z`3MgMd0e=CD0;fm_@2TY:@0/[:b08J6
QP1NGUh0Cdm?04=3@`0^;Rh0VIVI0<71`@0n?Sh00P0K6a/0JP0Q8B40:b/[03<c<`1;Bd/0PX:20;Bd
]02DU9@0<S8b06MWI`2f][H0NWYj06QXJ01KFe/0LW9b09JFUP39bLT0X:2P0:JVYP2SXj<0]kNg0:6Q
X@1eMGD0Hf=S09FEU@2ATI40=SHf01hN7P0L71`0:b/[03De=@2[Zj/0Lg=c02d];@0Y:BT09RHV02`/
;00`<300Cdm?07YjNP12@T808b<S02@T9011@D40Kfm_08R8R025QHD0YJFU0<O7a`2FUYH0[:b/0:FU
Y@1gMgL0TI6A07YjNP33`l<0c<c<0:ZZZP21PH409BDU01DE5@0T92@0@D5108^;R`2UYJD0L71`02/[
:`0L71`06a/K01PH600M7Ad0D51@069RHP0k>c/05aLG01XJ6P0^;Rh0J6QX0;bl_02GUiL0QXJ606a/
K02?Shl07QhN01dM7@0Q8B407AdM02XZ:P0^;Rh0@d=305aLG02BTY80FEUI02`/;00Z:RX0=3@d04E5
A@1VIVH0_[jn0=GEe@3<c<`0alO707YjNP0L71`05ADE01/K6`0X:2P@ooooo`003_ooool0=@2i^KT0
C4a<08V9R@1?Cdl0MgMg075aL@2m_Kd0BDU905mOG`1WIfL0;bl_0451@@1WIfL0<C4a02l_;`0Y:BT0
DE5A02d];@0O7al092@T01HF5P1]KFd0BDU902TY:@0M7Ad07QhN02l_;`1OGel0>c/k02LW9`0H61P0
;Bd]05UIF@0c<c<071`L04m?C`0a<C40>3Ph0451@@0R8R807AdM04a<C01CDe<0E5AD03De=@0g=cL0
AdM706EUI@0o?cl0AdM706EUI@1WIfL0KVi^0080?3`l05l0Cdm?04Q8B00h>3P0GUiN05MGE`0Z:RX0
A4A403dm?@0L71`0KFe]04E5A@0P8200;Bd]04Q8B00Y:BT0Cdm?02TY:@1?Cdl0CDe=020P801IFET0
@41005EEE@1PH600<S8b03`l?01fMWH0BTY:06ATI01jNWX0Kfm_06a/K01?Cdl0EEEE08Z:RP2MWId0
JVYZ03`l?01IFET0OWin08:2PP3AdM40UiNG0451@@1MGEd0LG5a04a<C01IFET0P82006a/K01CDe<0
Cdm?04];B`1<C4`0QXJ60492@P1iNGT0Fe]K05IFEP1lO7`0DU9B04m?C`0m?Cd0DU9B07alO01UIFD0
?clo05IFEP1eMGD0H61P04m?C`1dM7@0IFEU06IVIP1`L700F5QH02l_;`0`<300NGUi06MWI`0^;Rh0
NWYj03Xj>P15ADD0Ng]k07EeM@1RHV80P82004U9B@17AdL0Fe]K04E5A@1FEUH0SXj>03/k>`020861
P@1505ADE01kNg/0Q8B404i>CP1RHV80EeMG09NGU`2@T900I6AT07IfMP2FUYH0@D5103Lg=`1oOgl0
YZJV06ATI023Ph<0KVi^02DU9@1VIVH0Rh^;059BDP2OWil0M7Ad038b<P1VIVH0I6AT02DU9@1LG5`0
B4Q809:BTP0g=cL0CTi>065QH@0/;2`0FEUI05aLG013@d<0O7al05eMG@11@D40S8b<06MWI`1DE5@0
ADE503/k>`1gMgL0HV9R07moO`1IFET0T92@07EeM@1=CDd06ATI04=3@`1UIFD0Q8B403hn?P1?Cdl0
61PH065QH@29RHT0EEEE05UIF@1]KFd0@d=304m?C`1oOgl0SHf=0080A4A403L0IFEU030`<00m?Cd0
L71`08:2PP1BDU80Ng]k04m?C`17AdL0QHF505eMG@1cLg<0Hf=S04m?C`2[Zj/0X:2P071`L00U9BD0
De=C02/[:`0N7Qh0?Shn04I6AP0Q8B40=3@d07alO01NGUh0JVYZ0:j^[P0e=CD0:RXZ07=cL`1/K6`0
<c<c0;2`/01MGEd0?3`l07alO027QhL0S8b<0;Bd]01MGEd0R8R80=3@d02IVIT0KVi^0:f][@15ADD0
9bLW0410@01[Jf/0I6AT03Ti>@23Ph<0P82000kooooo00P0C4a<0;bl_02UYJD0JFUY079bLP1[Jf/0
CDe=03hn?P80;2`/0300D51@059BDP0f=SH09RHV01`L700G5aL05ADE01PH601OGel0]kNg0:BTY02U
YJD0Zj^[08f=S@1SHf<0[Jf]0<[:bP1oOgl0WYjN08J6QP23Ph<0J6QX06][J`1IFET0E5AD04];B`1>
CTh0EUIF04i>CP0j>SX09BDU018B4P0A4A40CTi>03dm?@0/;2`09BDU01PH600E5AD0ATI603hn?P0O
7al07AdM04I6AP0W9bL06ATI034a<@0Q8B4201DE5@0<014A4@0D51@05QHF08B4Q01ADE40QXJ609nO
W`1FEUH0/;2`06ATI00E5AD051@D0`0E5AD04`0C4a<05ADE03Ti>@1WIfL0QhN702h^;P0];Bd0Cdm?
07IfMP22PX80/[:b0<k>cP3;bl/0[jn_06i^KP1KFe/0Bd];020P800Y:BT00P0P82006P0j>SX0<S8b
03/k>`1kNg/0LW9b07moO`2OWil0]kNg09VIV@1eMGD0FUYJ04i>CP1=CDd0@41004m?C`1FEUH0JFUY
07MgM`2FUYH0`L710:6QX@2@T900YJFU0<71`@0K6a/051@D0P0E5AD200h>3P0@014A4@0C4a<04a<C
02h^;P1OGel0;bl_0:NWY`2>SXh0JFUY09RHV01gMgL0Q8B409BDU02n_[h0[jn_02/[:`D05ADE02d0
51@D01HF5P0M7Ad09BDU061PH02DU9@0ADE504a<C00n?Sh0V9RH0<G5a@2MWId0TY:B08j>SP0b<S80
7AdM01TI6@0N7Qh0?Cdm05ADE01/K6`0/K6a0:VYZ@0c<c<0?3`l02PX:00O7al08b<S02LW9`0m?Cd0
IVIV09:BTP1PH600Ogmo079bLP2=SHd0CTi>04Y:BP2EUID0dm?C0>7Qh@3Zj^X0i^KV0>c/k03emOD0
0P3`l?00AP3Vi^H0UYJF096AT@0R8R8071`L01/K6`0Q8B40<c<c0:BTY012@T809bLW02DU9@0Y:BT0
9bLW030`<01EEED0<30`04i>CP1QHF40Fe]K07alO01@D500ATI606UYJ@2?Shl0/K6a0=cLg02f][H0
H61P071`L02HV9P0XJ6Q07UiN@2DU9@0Shn?08b<S026QXH09BDU01LG5`0O7al0<c<c08J6QP16ATH0
8B4Q01dM7@0L71`06ATI01hN7P1XJ6P0Ph>3059BDP0T92@09bLW055AD@21PH40/[:b05=CD`1cLg<0
AdM706YZJP0N7Qh071`L01hN7P0M7Ad08R8R03/k>`1IFET0EeMG0:2PX01CDe<202h^;P0<03<c<`12
@T80P8200:j^[P2MWId0G5aL06IVIP26QXH0JVYZ01/K6`0E5AD07alO4?ooool000kooooo0380[:b/
04e=C@1fMWH0CTi>08N7Q`1^KVh0alO705=CD`1SHf<0Kfm_059BDP0i>CT0Cdm?03De=@0Z:RX0?3`l
04A4A00[:b/08R8R03De=@0R8R80IfMW04Y:BP0S8b<0;Bd]038b<P0d=3@0Hf=S03hn?P0^;Rh0<C4a
02d];@17AdL0=cLg020P8016ATH0BTY:02TY:@1FEUH0:2PX03Xj>P0o?cl0Hf=S05UIF@1DE5@0<S8b
04];B`1iNGT0?Shn055AD@80Jf][00<0NWYj04M7A`0^;Rh00P1?Cdl0RP0b<S80I6AT075aL@1=CDd0
EeMG04=3@`0L71`0O7al065QH@0Y:BT0<30`04U9B@0f=SH0Fe]K03@d=01OGel0HV9R024Q8@1;Bd/0
@T92069RHP1^KVh0=cLg03/k>`1iNGT0?Shn07inOP2JVYX0J6QX07alO01VIVH0>c/k06IVIP2PX:00
NWYj03Ti>@1KFe/0Ng]k05MGE`2m_Kd0ZZZZ05MGE`1FEUH0EEEE04A4A01ZJVX0Rh^;04A4A01DE5@0
GUiN03lo?`0n?Sh0LG5a04];B`2IVIT0IfMW05eMG@2?Shl0@41006YZJP1ADE40F5QH07alO00m?Cd0
>3Ph05IFEP1YJFT0@41004e=C@1`L700F5QH05IFEP1QHF40LW9b04U9B@0d=3@0R8R807inOP0n?Sh0
P82004];B`1>CTh0Shn?061PH01;Bd/0Lg=c05IFEP1@D500G5aL069RHP0Y:BT0DE5A03`l?02:RXX0
Lg=c0410@01dM7@0OGem059BDP13@d<0=3@d05eMG@1`L700MWIf06UYJ@2VYZH0NWYj03/k>`1>CTh0
_kno07EeM@2CTi<0V9RH04M7A`2PX:00ZZZZ05UIF@29RHT0R8R802@T9015ADD0CDe=02DU9@1KFe/0
H61P06MWI`0i>CT0DU9B07alO014A4@0:b/[04Q8B00g=cL0Ph>307=cL`1<C4`0MGEe06ATI080Cdm?
0540:BTY06][J`0l?3`0GEeM04M7A`2>SXh0Ogmo07IfMP1=CDd0QhN706QXJ01QHF40A4A406QXJ00l
?3`0IVIV069RHP1;Bd/0GEeM07=cL`1?Cdl0<30`055AD@1PH600>CTi04i>CP1hN7P0AdM702HV9P14
A4@0Jf][05eMG@2=SHd0GUiN034a<@1YJFT0MgMg07IfMP1cLg<0Bd];07MgM`1hN7P0EeMG02/[:`1R
HV80DU9B01lO7`16ATH0De=C02l_;`0W9bL0N7Qh061PH01GEeL0[:b/075aL@11@D40K6a/04i>CP0Z
:RX0[Zj^07MgM`0m?Cd0LG5a07AdM02@T900]kNg06YZJP1WIfL0dm?C06YZJP1YJFT0XZ:R04];B`0n
?Sh0Cdm?06e]K@0i>CT0@410096AT@27QhL03_ooool0202;Rh/0O7al05=CD`1>CTh0?3`l02`/;00U
9BD0;Rh^0P0d=3@0;`1?Cdl0Rh^;071`L00e=CD06a/K01dM7@0k>c/0E5AD051@D01;Bd/0>CTi03Hf
=P0c<c<0Ng]k07QhN01FEUH0OGem096AT@29RHT0UYJF0:^[Z`2OWil0MGEe02`/;00[:b/0;Bd]03Ti
>@10@400BDU904I6AP1CDe<0D51@079bLP1oOgl0?clo03dm?@19BDT0=3@d0410@01NGUh0:RXZ02h^
;P0g=cL0O7al03/k>`0G5aL051@D00804a<C00`051@D014A4@0E5AD0C4a<08^;R`1VIVH0EeMG0:RX
Z02IVIT0QXJ60861P@14A4@204I6AP0U04Q8B01CDe<0GemO08f=S@2l_;`0R8R801dM7@0F5QH04a<C
02d];@1SHf<0PH610;no_`2?Shl0GEeM06IVIP2GUiL0Ph>306IVIP1aLG40DE5A02/[:`0k>c/0C4a<
04i>CP0l?3`0>SXj04a<C026QXH0^KVi0:f][@21PH40A4A403lo?`19BDT0De=C04e=C@0203hn?P09
05]KF`1NGUh0G5aL09:BTP2@T900][Jf07MgM`1]KFd0AdM700<051@D0`0@41004`0B4Q804a<C03hn
?P28R8P0S8b<0=KFeP3:b/X0[:b/08n?S`0f=SH0B4Q808>3P`2o_kl0X:2P04Q8B00`<3008b<S020P
800I6AT00P0E5AD04P0F5QH07alO030`<01eMGD0Hf=S04U9B@1HF5P0JVYZ0:FUY@2RXZ80E5AD05=C
D`10@4006a/K01XJ6P0F5QH06ATI01lO7`80F5QH01H0P82008>3P`2<S8`0IfMW02/[:`0S8b<0;Bd]
0492@P0b<S80=CDe04I6AP1WIfL0YZJV0492@P1:BTX0M7Ad07EeM@2@T900bLW90=[JfP3QhN40fm_K
0P3MgMd03P3Shn<0[Zj^09NGU`1eMGD0LW9b024Q8@0G5aL06a/K024Q8@10@400UiNG03Ti>@0W9bL0
:RXZ0P0[:b/0AP0o?cl0820P01hN7P0T92@0Jf][0;2`/01oOgl0GEeM06e]K@1:BTX0S8b<09FEU@2h
^;P0Ti>C04M7A`14A4@0IVIV04Y:BP0[:b/0<c<c071`L02VYZH0Shn?07UiN@1XJ6P0=3@d024Q8@19
BDT0N7Qh04a<C01ADE40GemO059BDP1XJ6P0XJ6Q09nOW`1WIfL0N7Qh09>CT`21PH40Y:BT04];B`1O
Gel0IfMW04=3@`1QHF40<30`01`L700N7Qh0=3@d04a<C017AdL0>c/k079bLP2YZJT0Kfm_04Y:BP1@
D500Ng]k08j>SP2e]KD0^k^k06][J`1LG5`0Cdm?05IFEP1JFUX0H61P05IFEP1DE5@@ooooo`003_oo
ool09P2XZ:P0DU9B05UIF@0g=cL0Fe]K05=CD`2KVi/0;Bd]05EEE@1_Kfl0Bd];034a<@0h>3P0820P
03/k>`1GEeL0ADE5024Q8@0o?cl0JFUY03Ti>@1]KFd0EUIF02l_;`1OGel0?clo0451@@1WIfL05aLG
0410@013@d<0;Rh^03Ph>00T92@0;Rh^05mOG`17AdL0@4100P0a<C40kP0f=SH0=cLg02h^;P12@T80
Lg=c02HV9P10@400FUYJ02`/;01DE5@0D51@06UYJ@1DE5@0<C4a04I6AP1dM7@0De=C02LW9`1OGel0
=3@d028R8P1OGel0G5aL01/K6`1dM7@0?Shn03dm?@1NGUh0I6AT04=3@`1HF5P08B4Q06IVIP1_Kfl0
9BDU03`l?00c<c<0AdM706QXJ00e=CD0AdM707QhN00l?3`0EeMG07]kN`1NGUh0X:2P06][J`1QHF40
I6AT08N7Q`1TI6@0;Rh^059BDP1SHf<0L71`0<c<c02VYZH0=3@d05EEE@1VIVH0CTi>051@D02=SHd0
Bd];04Q8B01BDU80@D5102d];@1iNGT0E5AD09JFUP1fMWH0De=C08>3P`16ATH0EeMG04e=C@1_Kfl0
U9BD065QH@0l?3`0CTi>07EeM@1HF5P0CTi>06IVIP14A4@0JVYZ07=cL`1[Jf/0@T9203Ph>024Q8@0
P8200410@02@T900CTi>0492@P1hN7P0I6AT05ADE021PH40@d=306EUI@1nOWh0OGem03Lg=`1=CDd0
@T9208b<S029RHT0E5AD05mOG`26QXH0Cdm?07QhN013@d<0GEeM05aLG01KFe/0JVYZ09fMW@1/K6`0
>SXj03hn?P2f][H0>CTi07MgM`20P800?3`l09VIV@2`/;00NGUi0861P@2BTY80:RXZ04e=C@1YJFT0
8R8R06EUI@1TI6@0KFe]034a<@1:BTX0K6a/05IFEP0[:b/0GEeM0492@P29RHT0MWIf04M7A`1TI6@0
O7al059BDP1NGUh09BDU06EUI@14A4@0FUYJ05MGE`21PH40Kfm_08R8R01FEUH0[jn_0820P01dM7@0
;Rh^05IFEP1<C4`0IFEU06i^KP1XJ6P0A4A409FEU@1kNg/0>SXj055AD@1MGEd0>3Ph0492@P1gMgL0
@d=302TY:@1PH600NGUi05IFEP2=SHd0NWYj03hn?P1fMWH0N7Qh07YjNP1MGEd0GUiN09JFUP1nOWh0
MWIf03Ti>@1GEeL0D51@03<c<`1XJ6P0IFEU02TY:@0g=cL0S8b<06e]K@1ADE40YjNW02XZ:P0e=CD0
RHV9071`L00b<S80[:b/06UYJ@0/;2`0QXJ606YZJP1mOGd0];Bd04=3@`27QhL0alO708b<S01PH600
UiNG0492@P0U9BD0>c/k05UIF@1;Bd/0=cLg09ZJVP1bLW8>ooooo`0H0::RXP1;Bd/0ADE505YJFP0`
<3009BDU02HV9P0O7al071`L02PX:01XJ6P0M7Ad0:FUY@2DU9@0VIVI07YjNP1<C4`0=3@d03Hf=P0Y
:BT09RHV02LW9`0N7Qh0BTY:0`0C4a<06P0I6AT0BDU90;2`/028R8P0WinO08Z:RP2JVYX0>c/k01lO
7`0N7Qh08R8R02d];@0b<S80=cLg03Xj>P2_[jl0JVYZ06ATI01fMWH0XZ:R0;Jf]P36a/H0U9BD05aL
G00b<S808R8R0P0M7Ad0:`0F5QH051@D01DE5@0C4a<05ADE01`L701MGEd0ATI601hN7P1CDe<0Cdm?
08n?S`3EeMD0/[:b07EeM@1IFET0E5AD06EUI@1[Jf/0Hf=S07YjNP2k^k/0d];B0<K6aP1VIVH08R8R
01@D500L71`0?3`l08N7Q`3AdM40hn?S0:^[Z`1SHf<0U9BD04M7A`1DE5@0JVYZ0:RXZ02@T900?3`l
01DE5@0C4a<00`0E5AD05@1SHf<0LG5a07=cL`2GUiL0L71`04Y:BP0o?cl0<c<c02h^;P0Y:BT092@T
01lO7`0S8b<0CDe=08f=S@2ATI40FEUI05eMG@1=CDd0QHF504A4A00201<C4`0T01DE5@0A4A404a<C
01XJ6P19BDT0>c/k05aLG02d];@0TY:B0;Vi^@3Fe]H0/k>c0;^k^`3@d=00`L7109JFUP1^KVh0E5AD
03Ph>00m?Cd0;Bd]03@d=013@d<0>c/k01dM7@0Q8B40DU9B0861P@1RHV80G5aL08R8R01jNWX0`L71
05UIF@1`L7008b<S0P0E5AD03P0D51@05QHF01LG5`0K6a/0@d=305MGE`1[Jf/0M7Ad04m?C`21PH40
YJFU03hn?P1GEeL0<c<c0P0G5aL0I00F5QH05aLG0410@01XJ6P0BDU908J6QP28R8P0][Jf09bLW039
bLT0QhN708^;R`2OWil0SHf=08B4Q02EUID0[:b/09NGU`2c/k<0Bd];02LW9`0M7Ad08B4Q05ADE01M
GEd0ADE503<c<`0T92@09RHV03hn?P0f=SH07QhN01dM7@0Q8B40CTi>07YjNP0f=SH0<C4a02`/;00T
92@0EEEE0410@01cLg<0A4A402<S8`1UIFD0=3@d02@T900R8R808b<S04a<C02=SHd0PH61079bLP1L
G5`0[Jf]05IFEP0f=SH0WinO0:n_[`29RHT0LG5a08^;R`1TI6@0;Bd]051@D02/[:`0UYJF0;bl_03G
emL0WIfM03Hf=P1`L700De=C038b<P1JFUX0BTY:020P801<C4`0CDe=03Xj>P15ADD0U9BD04Q8B027
QhL0^k^k061PH027QhL0N7Qh0;no_`34a<@0_Kfm030`<00Q8B409BDU03`l?01RHV80Cdm?04];B`1W
IfL@ooooo`003_ooool05@2FUYH0DE5A071`L012@T80IFEU059BDP2VYZH0GemO04i>CP1UIFD0Bd];
03De=@0j>SX06ATI01/K6`10@400?clo01TI6@0F5QH0@T9201hN7P0206EUI@0A02HV9P15ADD0;Bd]
034a<@1JFUX0=CDe02PX:00^;Rh08R8R04Q8B017AdL071`L03/k>`0/;2`0Bd];04U9B@0R8R800P0Y
:BT0>011@D40@d=305IFEP0L71`0=cLg06QXJ00n?Sh0FEUI03dm?@1cLg<0JFUY03<c<`0X:2P0>SXj
0492@P0e=CD0IfMW05aLG00e=CD0GUiN03Ti>@0J6QX0HF5Q0492@P0^;Rh0:RXZ04I6AP0i>CT0HV9R
02d];@1<C4`0EEEE02LW9`0n?Sh0B4Q806EUI@1PH60092@T03hn?P1^KVh0DU9B06e]K@20P800?Cdm
0861P@1UIFD0GEeM06IVIP2m_Kd0Lg=c0451@@1XJ6P0L71`07QhN02e]KD0bl_;0P0o?cl0IP1_Kfl0
CDe=04U9B@1UIFD0FEUI04A4A01ZJVX0>CTi02l_;`23Ph<0CTi>08B4Q01<C4`0DE5A09VIV@1GEeL0
Hf=S03Ph>018B4P0Jf][075aL@12@T80EUIF07inOP1LG5`0C4a<07moO`1IFET0E5AD07EeM@1nOWh0
CTi>02XZ:P1iNGT0GEeM02l_;`2DU9@0FEUI0492@P27QhL0Ng]k05eMG@2=SHd0EEEE04];B`1JFUX0
Fe]K03<c<`15ADD0<C4a092@T02:RXX0DU9B06ATI01QHF40EEEE06m_K`12@T80GEeM07EeM@1FEUH0
Fe]K08n?S`11@D40B4Q808>3P`3<c<`0NGUi07alO023Ph<0?Cdm0:6QX@2i^KT0N7Qh07YjNP2LW9`0
<c<c04m?C`1dM7@07AdM05YJFP1`L700Jf][03lo?`1CDe<0K6a/05MGE`0g=cL0LW9b04A4A01fMWH0
NWYj03<c<`1_Kfl0P820055AD@1YJFT0:RXZ06UYJ@1aLG40Ng]k05aLG080GEeM04X0MgMg03dm?@1b
LW80JVYZ06UYJ@0a<C40E5AD034a<@1bLW80QHF506=SH`14A4@0PH6104Y:BP1;Bd/0LW9b08F5Q@11
@D40AdM707inOP1TI6@0<30`04Y:BP21PH40De=C09BDU01ADE40@D510820P01FEUH0L71`05MGE`1?
Cdl0WinO08R8R01_Kfl0=3@d06i^KP15ADD06a/K04E5A@0j>SX03@d=02<S8`1oOgl0IFEU05YJFP2m
_Kd0@d=302DU9@1DE5@0BTY:05MGE`3Bd]80Y:BT03/k>`14A4@0IFEU08R8R02QXJ40<C4a03/k>`2G
UiL0CDe=071`L02GUiL0G5aL02HV9P0f=SH0GEeM04M7A`1kNg/0][Jf07MgM`kooooo01X0b/[:0:2P
X02@T900N7Qh04I6AP0j>SX0?3`l03/k>`0`<3007AdM03@d=01OGel0@D5109RHV02UYJD0?3`l02LW
9`0o?cl0>SXj02/[:`0Y:BT06a/K01HF5P15ADD0410@014A4@805ADE00T05QHF075aL@2a/K40SXj>
09nOW`2HV9P0YZJV06MWI`0g=cL00P0];Bd04P0Y:BT0<S8b09^KV`2RXZ80MGEe07AdM02DU9@0][Jf
07inOP1KFe/0RHV90:j^[P2GUiL0QXJ606IVIP0l?3`0=CDe01TI6@805ADE00d0;Rh^06e]K@1QHF40
7AdM01DE5@0P8200WIfM0:JVYP2e]KD0/[:b0:2PX023Ph<0O7al00<0N7Qh01D0Vi^K0<?3``1kNg/0
SXj>0:n_[`28R8P0BDU901dM7@0S8b<0E5AD0;2`/03Shn<0inOW0=?Cd`2d];@0GemO05]KF`11@D40
@d=306e]K@0M7Ad0100E5AD0?00R8R80Lg=c05QHF014A4@0Bd];02LW9`0S8b<09RHV02XZ:P0];Bd0
>c/k038b<P13@d<0IFEU05YJFP1VIVH0DE5A05QHF01EEED0LW9b06QXJ02JVYX08b<S01DE5@0@4100
4A4A03Hf=P15ADD0>CTi03@d=02EUID0EEEE059BDP2j^[X0eMGE0;Rh^02GUiL0b/[:07EeM@0`<300
6a/K01lO7`12@T80A4A402TY:@0L71`06a/K024Q8@0b<S80I6AT0;:b/P2YZJT0OWin09ZJVP2DU9@0
Kfm_0:BTY01SHf<0D51@01DE5@8051@D00l0410@01<C4`0H61P071`L028R8P1OGel0M7Ad0:^[Z`1>
CTh0@410075aL@2a/K40;bl_01@D500C4a<00`0E5AD0HP0f=SH0Ogmo02d];@1OGel0TY:B0;:b/P2G
UiL0b<S808>3P`2CTi<0Fe]K04A4A00m?Cd0FEUI0:f][@3>c/h0Zj^[08>3P`2HV9P0De=C02/[:`1M
GEd0=CDe02d];@0T92@08B4Q038b<P25QHD0:RXZ01dM7@0K6a/08b<S05]KF`1iNGT0=cLg02@T900H
61P0=3@d079bLP0];Bd0M7Ad03<c<`0o?cl0@d=3028R8P0O7al08B4Q02DU9@0_;bl0U9BD07UiN@2F
UYH0MgMg04Y:BP2a/K40W9bL08N7Q`2m_Kd0ZJVY0:6QX@1VIVH09bLW02XZ:P0l?3`0E5AD0:j^[P3E
eMD0hN7Q08F5Q@0j>SX0TY:B04Q8B00[:b/0AdM705QHF01NGUh0Bd];03De=@1KFe/0QhN705UIF@2=
SHd0IfMW08J6QP2PX:00MGEe08V9R@2^[Zh0alO70;no_`11@D40>CTi02<S8`0O7al0De=C0:NWY`2U
YJD0_;bl4?ooool000kooooo0540[Zj^05IFEP1iNGT0@D5105aLG00n?Sh0IVIV05IFEP1JFUX0M7Ad
04I6AP13@d<0I6AT04A4A00/;2`0E5AD04A4A00X:2P08b<S03Ti>@0I6AT0HF5Q05]KF`0X:2P0>3Ph
02LW9`0m?Cd0G5aL04U9B@0X:2P08B4Q02l_;`1NGUh0BTY:02<S8`10@400:RXZ04Y:BP1>CTh09RHV
02LW9`0a<C40HF5Q04m?C`19BDT06a/K04E5A@1`L700@T9204U9B@0h>3P0S8b<07QhN01:BTX0;2`/
04U9B@1GEeL06ATI04Y:BP1PH600ADE507MgM`0g=cL06ATI06UYJ@1DE5@0=SHf038b<P11@D40=CDe
06m_K`0b<S80ADE504];B`0I6AT0<S8b04E5A@1/K6`0GEeM03Lg=`0T92@00P1VIVH0R01[Jf/0XZ:R
06=SH`2IVIT0Fe]K04i>CP1:BTX0UIFE06e]K@0T92@0@41005mOG`1gMgL0];Bd0<_;b`1FEUH0@T92
07=cL`1;Bd/0I6AT08:2PP1UIFD0ATI607AdM010@400;Rh^07=cL`1GEeL0QXJ606=SH`13@d<0QHF5
05iNGP1bLW80CTi>05aLG01kNg/0IFEU03lo?`1ADE40N7Qh04i>CP1EEED0U9BD05MGE`1?Cdl0IVIV
07moO`1EEED0>SXj07AdM01nOWh0>CTi09fMW@1LG5`0@T9208Z:RP1JFUX0@d=308b<S01RHV80ADE5
05mOG`1CDe<0=3@d04];B`0i>CT0L71`08^;R`1>CTh0KFe]06m_K`1@D500B4Q8038b<P11@D40Bd];
069RHP1XJ6P0YjNW05=CD`0a<C40BDU909nOW`1gMgL0MWIf09VIV@1BDU80`L710=3@d01dM7@0J6QX
09bLW011@D40NWYj071`L00O7al0A4A406QXJ021PH40>c/k055AD@1ZJVX0E5AD02h^;P1NGUh0F5QH
06e]K@1lO7`0C4a<06MWI`26QXH0HV9R075aL@0e=CD0J6QX06=SH`1gMgL0FEUI0:VYZ@20P800Kfm_
04Y:BP2@T900IFEU06QXJ01<C4`0NGUi05mOG`1@D500GUiN02d];@1JFUX0Shn?06IVIP0f=SH205aL
G00W02XZ:P1ZJVX0SHf=06ATI00i>CT0@d=306IVIP1;Bd/0UiNG07QhN00g=cL0K6a/04a<C01_Kfl0
SHf=06][J`2DU9@0QhN703<c<`0K6a/0D51@05aLG00L71`0Bd];04e=C@0W9bL09RHV07AdM019BDT0
ATI60;>c/`1cLg<0=CDe05UIF@1VIVH0=3@d09nOW`23Ph<0Cdm?0080Kfm_0100XJ6Q0;>c/`1GEeL0
@T9209FEU@16ATH0D51@08j>SP1@D5009RHV03<c<`1DE5@0=cLg03Ph>01]KFd0Hf=S3_ooool04`3B
d]80d=3@0:n_[`1cLg<0De=C04m?C`11@D40;2`/02TY:@0`<300?Cdm07UiN@27QhL0SXj>07YjNP18
B4P0A4A405IFEP0n?Sh00P0m?Cd00`0N7Qh08B4Q04I6AP02014A4@0?01DE5@0C4a<04a<C03Ti>@1N
GUh0X:2P0:BTY02RXZ80_;bl0=WIf@2[Zj/0UiNG0:>SX`2i^KT0e]KF0080emOG0P2OWil03`2g]kL0
WinO0:BTY02MWId0G5aL03Ph>00k>c/0DE5A07=cL`2IVIT0WinO06e]K@0j>SX0H61P06m_K`0202@T
900K01DE5@0G5aL0;bl_09RHV02^[Zh0UYJF08b<S01/K6`0BDU9034a<@0Z:RX0:BTY02h^;P0d=3@0
QXJ607]kN`1PH600E5AD05mOG`1_Kfl0H61P04e=C@2MWId0clo?0=[JfP3Ti>@0f][J0080J6QX00D0
Q8B409BDU026QXH0QXJ605ADE00204Q8B00^04];B`1=CDd0S8b<0;:b/P1YJFT0ADE504Y:BP0E5AD0
6ATI01LG5`0E5AD05QHF02PX:01iNGT0QHF505]KF`0e=CD0=3@d01LG5`0J6QX0A4A403/k>`1[Jf/0
YZJV0820P014A4@0<C4a04=3@`0e=CD0:2PX02DU9@1XJ6P0C4a<04m?C`1>CTh0[Jf]0<K6aP2_[jl0
][Jf0;fm_@1XJ6P06a/K01LG5`0I6AT07AdM03@d=080CDe=04h0>c/k020P800W9bL0HV9R0:RXZ02]
[Jd0clo?0;Fe]@21PH40NWYj0:BTY01:BTX06QXJ018B4P0C4a<05ADE01@D500@41004A4A01LG5`0M
7Ad0M7Ad08^;R`2i^KT0Vi^K09:BTP2:RXX0Ph>30451@@0W9bL07alO024Q8@0];Bd0DE5A0:2PX02m
_Kd0De=C06=SH`16ATH0Ng]k06EUI@2YZJT0K6a/03/k>`0m?Cd0Fe]K08j>SP37alL0bl_;09bLW012
@T80C4a<06m_K`2l_;`0]KFe09fMW@0/;2`07alO01`L700_;bl0MGEe0::RXP0i>CT07alO01dM7@0[
:b/0HV9R06YZJP0b<S80:BTY06IVIP29RHT0QhN707UiN@1fMWH0:BTY04=3@`0R8R82024Q8@0d02<S
8`0T92@09bLW05IFEP2?Shl0I6AT0:NWY`2VYZH0Q8B40<k>cP32`/80/[:b0:j^[P1cLg<0I6AT04E5
A@18B4P0?Shn0492@P1EEED0dM7A0=oOg`2`/;00Zj^[0:f][@2c/k<0Kfm_04E5A@21PH40TI6A08>3
P`2VYZH0eMGE08j>SP25QHD0@41001`L700_;bl0TY:B0:2PX02][Jd0[Zj^0:6QX@1GEeL0Bd];051@
D013@d<0DU9B08>3P`2MWId0b/[:0=?Cda3ooooo000>ooooo`0A0:>SX`1<C4`0IVIV04M7A`1jNWX0
@d=307=cL`1iNGT0D51@079bLP0o?cl0<30`05MGE`10@4009BDU03Ti>@0h>3P00P0S8b<0800f=SH0
5ADE06YZJP1XJ6P0<30`03lo?`0L71`0:b/[04M7A`0g=cL09RHV01dM7@0P8200BTY:03Lg=`0Q8B40
@T9203Xj>P1=CDd0GEeM02DU9@0o?cl09RHV05iNGP1ADE40CDe=01lO7`0S8b<0FEUI04=3@`1IFET0
=3@d0P1UIFD0eP0l?3`0>c/k06i^KP1XJ6P06a/K04i>CP1PH600De=C08Z:RP1>CTh05ADE06UYJ@1S
Hf<0>CTi03Hf=P1UIFD0AdM7071`L00`<300A4A404Q8B00W9bL0<C4a04=3@`1SHf<0LW9b04e=C@0P
8200HV9R05eMG@1lO7`0XZ:R05iNGP2PX:00HF5Q07YjNP1HF5P0Xj>S05mOG`0a<C40Bd];06i^KP1D
E5@0Z:RX0<K6aP1KFe/0B4Q806][J`12@T80Hf=S09jNWP10@400?3`l07IfMP1GEeL0=cLg07EeM@0k
>c/0Ogmo051@D00n?Sh0Ph>304U9B@1IFET0BTY:06MWI`2HV9P0DE5A04Q8B01LG5`0KVi^04];B`1:
BTX0Ph>305]KF`1=CDd0QXJ607UiN@1BDU80@D5108f=S@1nOWh0>SXj08f=S@1GEeL0=cLg0820P01_
Kfl0ATI609FEU@1QHF40@T9207=cL`1FEUH0@D5105aLG00l?3`0Kfm_08>3P`17AdL0Ph>308F5Q@1H
F5P0NGUi059BDP16ATH0J6QX04Q8B01bLW80[jn_05mOG`11@D40C4a<09nOW`1RHV80Vi^K09jNWP0c
<c<0Ti>C0:^[Z`1MGEd0N7Qh0;Zj^P0f=SH0DE5A065QH@0g=cL0G5aL055AD@1MGEd0<30`069RHP2;
Rh/0Lg=c030`<01IFET09RHV05UIF@2CTi<0De=C06EUI@2;Rh/0Bd];08j>SP0l?3`0GUiN07=cL`1`
L700?3`l04e=C@1iNGT0IfMW0451@@1cLg<0EUIF05YJFP0Y:BT0Kfm_079bLP1JFUX0IVIV05eMG@0l
?3`0Ph>306YZJP18B4P0LW9b06e]K@0[:b/0>c/k079bLP1OGel0=cLg04Q8B01UIFD0?Cdm08B4Q01X
J6P0;Rh^07UiN@16ATH0FUYJ07moO`1NGUh0WYjN08N7Q`1hN7P0<S8b05YJFP1]KFd07alO05EEE@1[
Jf/0=SHf02/[:`1jNWX0HF5Q05QHF033`l<0MGEe03Lg=`1BDU80Lg=c02l_;`2;Rh/0ZZZZ05MGE`1^
KVh0Fe]K07inOP2:RXX0BTY:0410@027QhL206=SH`09096AT@13@d<08b<S03Lg=`1PH600CTi>0410
@02j^[X0Ph>300kooooo01@0RXZ:07]kN`2HV9P0YZJV05IFEP1=CDd0EeMG065QH@1UIFD0>3Ph04m?
C`1dM7@0TY:B0:BTY027QhL0ADE502h^;P0f=SH0<S8b030`<080:b/[00<0<30`03De=@0C4a<00P0G
5aL0@@0C4a<05ADE03Ph>00U9BD0?clo0:j^[P3<c<`0f][J0<71`@2:RXX0NWYj092@T02d];@0e=CD
0=KFeP3Ogml0dm?C09fMW@1IFET0Bd];07IfMP2PX:00GUiN02TY:@0Q8B4092@T02TY:@0j>SX0CDe=
04Q8B02IVIT0alO703/k>`0M7Ad05QHF01DE5@0H61P0@T9205aLG017AdL0Bd];0820P01bLW80H61P
04A4A00i>CT0;Rh^02/[:`0U9BD0DE5A09FEU@1]KFd0EeMG04U9B@16ATH0C4a<04];B`2DU9@0e]KF
0=?Cd`32`/80Ph>309BDU01:BTX0CDe=0080H61P00`0N7Qh04U9B@0h>3P0=SHf02XZ:P0e=CD0ATI6
08f=S@2^[Zh0Xj>S07QhN00K6a/201DE5@8051@D07<05aLG0410@00d=3@0;bl_03Xj>P0S8b<05ADE
01LG5`0W9bL0IFEU03De=@1iNGT0Ph>30::RXP1YJFT0?Cdm03De=@0f=SH0H61P07MgM`1FEUH0DE5A
08>3P`22PX80P82006m_K`2c/k<0YZJV08j>SP17AdL08B4Q01LG5`0K6a/071`L01lO7`0e=CD0DU9B
04M7A`1FEUH0VIVI0<O7a`2l_;`0aLG50<S8b01<C4`0[jn_0<S8b01QHF40820P01LG5`0A4A405ADE
018B4P0D51@04A4A00h>3P0N7Qh0KVi^096AT@1WIfL0MWIf05mOG`1GEeL0N7Qh08Z:RP1lO7`0L71`
07=cL`1bLW80IFEU059BDP2WYjL0_kno092@T01=CDd0BTY:04];B`2ATI40IVIV05ADE025QHD0`<30
0;Jf]P28R8P0M7Ad05=CD`1_Kfl0<c<c0410@01/K6`0W9bL0;jn_P0h>3P0>c/k05IFEP1RHV80=3@d
02TY:@1QHF40>3Ph01hN7P0[:b/0G5aL08f=S@1ADE40KVi^04A4A00f=SH0=3@d07inOP36a/H0<S8b
01LG5`0Q8B407AdM008092@T03<09bLW03Lg=`0`<300<30`08J6QP2OWil0a<C40:2PX02n_[h0dm?C
0=cLg02EUID0L71`06UYJ@1VIVH0OWin07moO`2?Shl0YZJV0;Ng]`3Kfm/0^KVi07MgM`1GEeL0RXZ:
0<C4a038b<P0_[jn09JFUP2DU9@0/K6a04A4A00Y:BT08R8R02/[:`0e=CD0>CTi075aL@2/[:`0UiNG
07emO@2_[jl0UYJF07AdM01mOGd0Jf][07YjNP2BTY80_;bl08b<S01fMWH04?ooool000kooooo01T0
/[:b05eMG@1VIVH0?3`l071`L014A4@0JVYZ079bLP1@D500U9BD065QH@0g=cL0J6QX055AD@0];Bd0
HV9R04U9B@0^;Rh09RHV051@D00W9bL0F5QH075aL@0g=cL0D51@0080<30`07T0H61P02h^;P0i>CT0
;Rh^02TY:@1?Cdl0@d=301lO7`13@d<0?3`l05IFEP1UIFD08B4Q0492@P0U9BD0IFEU05]KF`2=SHd0
;Rh^03@d=01RHV80=SHf05mOG`0c<c<0IfMW079bLP1ADE40;Bd]055AD@1=CDd08B4Q05EEE@1JFUX0
?Shn05=CD`1OGel05aLG07=cL`1jNWX0?Shn05=CD`1]KFd0Cdm?09:BTP0f=SH0>c/k055AD@0e=CD0
=SHf03hn?P1VIVH0PX:205QHF00f=SH0OWin04E5A@1jNWX0T92@05IFEP2HV9P0UIFE08F5Q@14A4@0
Z:RX03`l?00k>c/0MgMg08F5Q@1YJFT0X:2P0=7Ad@0g=cL0DE5A0:6QX@1CDe<0>c/k06a/K01eMGD0
A4A407AdM00d=3@0=CDe07]kN`1IFET0Rh^;02LW9`0];Bd0TI6A06IVIP1RHV808B4Q055AD@2=SHd0
Shn?04i>CP17AdL0MGEe07YjNP1?Cdl0M7Ad06YZJP1IFET0Ph>307alO00d=3@061PH069RHP1fMWH0
AdM708J6QP0o?cl0@T920:2PX02;Rh/0?clo09bLW01JFUX0LG5a08^;R`1NGUh0>CTi0080@T9201l0
Ph>308n?S`1:BTX0HF5Q05mOG`1UIFD0Rh^;061PH01QHF40QXJ603Xj>P0n?Sh0P82004Y:BP0V9RH0
C4a<08f=S@1=CDd0Cdm?08Z:RP15ADD0YjNW0<C4a01JFUX0?clo071`L00o?cl0H61P055AD@0W9bL0
F5QH0080JVYZ01/07QhN03<c<`1OGel0JVYZ02h^;P1PH600GemO06=SH`2JVYX0FUYJ05QHF027QhL0
B4Q805eMG@0i>CT0MGEe09RHV02DU9@0CTi>09:BTP1HF5P0NWYj04=3@`2PX:00I6AT069RHP0^;Rh0
0P1SHf<0@P1IFET0Q8B406MWI`0o?cl0MWIf034a<@0k>c/0FUYJ07YjNP0l?3`0@41007MgM`1EEED0
CDe=08B4Q021PH40D51@09VIV@29RHT0@d=307=cL`1QHF40JVYZ075aL@1NGUh0YjNW08f=S@1^KVh0
<c<c05]KF`1JFUX08b<S07AdM01eMGD0>CTi02XZ:P1iNGT0M7Ad05aLG030`<00T92@03Lg=`1BDU80
MWIf04I6AP1JFUX0^k^k06=SH`1]KFd0QhN708V9R@2RXZ80DU9B03Xj>P1jNWX0J6QX06YZJP2ATI40
@D5102LW9`0h>3P0IVIV05ADE00k>c/0`<3009>CT`kooooo03<0Ng]k06m_K`17AdL0KFe]07inOP1h
N7P0E5AD051@D01QHF40RHV907EeM@27QhL0YZJV0=SHf032`/80Ti>C04A4A01CDe<0A4A403lo?`0e
=CD0=cLg07alO00P82004Q8B01PH600Z:RX0:2PX028R8P12@T80=3@d01dM7@1@D500d];B0=?Cd`3;
bl/0UiNG0410@00g=cL0=3@d03Hf=P2PX:00g]kN0<G5a@1]KFd0BTY:04i>CP1EEED0WYjN065QH@0^
;Rh00P0T92@0=P0f=SH0DU9B051@D011@D40;Rh^04a<C01ZJVX0KVi^07AdM01>CTh0F5QH06][J`15
ADD0=cLg06ATI01aLG40:b/[03`l?00o?cl0FEUI03<c<`0O7al08B4Q028R8P2?Shl0Ng]k06IVIP0j
>SX0=CDe02XZ:P0S8b<0MWIf0<G5a@39bLT0VIVI05iNGP2<S8`0R8R804I6AP18B4P0?Shn059BDP1=
CDd0;bl_02h^;P0k>c/0@d=306IVIP2;Rh/0ZJVY09JFUP1eMGD0ADE501dM7@<05ADE01006a/K0451
@@0e=CD0@41003Hf=P0F5QH05ADE01HF5P0W9bL0>SXj02PX:01RHV80MWIf09FEU@3@d=00b<S80P3<
c<`03`3Jf]X0RHV904e=C@1^KVh0Xj>S02@T900G5aL0CDe=0:^[Z`1^KVh0B4Q808F5Q@1iNGT0@410
01hN7P0201`L700O01dM7@0U9BD0;Bd]06m_K`1YJFT0bl_;0<W9b@3GemL0eMGE0<k>cP1gMgL0L71`
08Z:RP1mOGd0Kfm_03lo?`0U9BD05QHF01<C4`0@41004A4A01<C4`0g=cL0MgMg04Y:BP0n?Sh0DE5A
065QH@10@400;bl_03<c<`0203@d=01=03Ti>@11@D40ADE509fMW@2^[Zh0c<c<0;2`/01CDe<0LG5a
02LW9`2JVYX0V9RH09^KV`1lO7`0Jf][05YJFP1GEeL0@d=308b<S01UIFD0=cLg0451@@1FEUH0[Zj^
0<k>cP25QHD0=cLg01PH600J6QX071`L02/[:`2ATI40Lg=c05]KF`2g]kL0[:b/08J6QP17AdL09RHV
024Q8@0Y:BT0CTi>0:ZZZP1fMWH06ATI024Q8@0S8b<08R8R02<S8`0W9bL0ATI601dM7@0L71`0FUYJ
0;Rh^02e]KD0WinO08:2PP2l_;`0aLG505eMG@0N7Qh06ATI01lO7`0c<c<0EeMG09bLW02k^k/0R8R8
0<S8b01cLg<0h>3P0=SHf039bLT0^k^k0<?3``36a/H00P2g]kL05@10@4008B4Q038b<P1EEED0UYJF
09bLW01QHF40SHf=0:ZZZP2`/;00a/K60;no_`2BTY80FEUI06QXJ01_Kfl0U9BD08b<S00h>3P0;Bd]
03Ph>00@ooooo`003_ooool0_@2:RXX0ADE507AdM015ADD0BTY:03Ti>@1XJ6P0GemO03Hf=P1<C4`0
=SHf038b<P2:RXX0ATI603`l?014A4@0EeMG034a<@0d=3@0CTi>02l_;`2IVIT0QHF503Lg=`13@d<0
;2`/06IVIP21PH40ADE502HV9P0P8200Bd];05UIF@0O7al09BDU05aLG00j>SX0HF5Q06QXJ00N7Qh0
<S8b02l_;`1DE5@0?Shn05QHF00U9BD0GUiN07alO01;Bd/0C4a<03@d=023Ph<0N7Qh05MGE`0a<C40
De=C03`l?00`<300IVIV07EeM@1HF5P0JVYZ0451@@0R8R80KVi^07MgM`0^;Rh0<S8b04Q8B00^;Rh0
L71`05iNGP14A4@0?Cdm01@D500^;Rh0De=C05aLG01XJ6P07alO02LW9`0h>3P0Cdm?061PH02<S8`0
@41005mOG`15ADD0M7Ad04=3@`2YZJT0FEUI02DU9@0Z:RX0@41005]KF`2DU9@0c/k>04M7A`0_;bl0
EEEE0492@P14A4@0FEUI055AD@0i>CT0H61P055AD@0U9BD0GUiN055AD@1jNWX0HV9R02l_;`1_Kfl0
GUiN069RHP1GEeL0@D5108J6QP1mOGd0C4a<04i>CP1UIFD0Jf][04e=C@1SHf<0G5aL05YJFP23Ph<0
Ogmo051@D00h>3P0Lg=c05eMG@0k>c/0ZJVY059BDP0e=CD0NWYj06m_K`1GEeL0Xj>S0410@00`<300
B4Q805MGE`0g=cL0ATI602<S8`16ATH0PX:204Q8B01WIfL0IVIV05eMG@1iNGT0>CTi05EEE@1QHF40
HV9R05EEE@2OWil0D51@03Lg=`0k>c/0QhN708Z:RP1FEUH0RXZ:03<c<`2@T900`L7107moO`1DE5@0
NGUi0410@026QXH0IFEU02PX:00i>CT0HF5Q05aLG00`<300De=C06][J`1_Kfl0:RXZ04e=C@02079b
LP1G09RHV01IFET0E5AD0:JVYP1HF5P0I6AT02d];@19BDT0KFe]05UIF@1?Cdl0UIFE05ADE01cLg<0
;Rh^061PH01BDU80KFe]03Ti>@1JFUX0=SHf04E5A@1BDU80Cdm?03Ti>@1jNWX0GEeM03Lg=`1NGUh0
UIFE03dm?@16ATH0MGEe04e=C@0X:2P0EeMG07IfMP1?Cdl0OGem069RHP1DE5@0T92@059BDP1EEED0
L71`05aLG02n_[h0/;2`07inOP0j>SX0EeMG03dm?@0J6QX0C4a<061PH01=CDd0:BTY075aL@1@D500
E5AD0;jn_P1hN7P0?Shn06e]K@26QXH0@d=308^;R`2@T900GemO04=3@`2:RXX0NGUi0:6QX@1;Bd/0
HV9R09FEU@1jNWX0IfMW09ZJVP0l?3`09bLW04Y:BP1aLG40CDe=03hn?P23Ph<0V9RH00kooooo01P0
[Zj^07inOP1MGEd0GemO03Ti>@1?Cdl0NGUi06ATI01mOGd0TI6A0=[JfP2e]KD0TI6A08f=S@30`<00
KFe]07UiN@22PX80CDe=04E5A@19BDT0P82009BDU00/;2`203<c<`0=04m?C`0h>3P0<30`05YJFP0j
>SX0=SHf07AdM01oOgl0R8R80<G5a@1HF5P04Q8B01TI6@0201/K6`0H03lo?`2NWYh0TY:B04];B`1]
KFd0Ogmo08N7Q`34a<@0_[jn07IfMP1UIFD0KFe]0:^[Z`15ADD0;Rh^04Y:BP1OGel0CDe=05IFEP1C
De<0HV9R09bLW03Ng]h0UiNG0P0a<C40<01fMWH0AdM7020P800G5aL071`L02HV9P0o?cl0@D5101lO
7`0F5QH0AdM707]kN`1]KFd0Fe]K04i>CP0d=3@0820P09^KV`32`/80^k^k0451@@0Q8B40Cdm?0;Jf
]P1XJ6P0LW9b09jNWP2`/;00Ogmo05=CD`1@D500BTY:04M7A`0`<30092@T08Z:RP1XJ6P0Kfm_05EE
E@12@T80>3Ph02l_;`0Y:BT0<C4a030`<00g=cL0>3Ph01HF5P<05ADE0:00:2PX03De=@0G5aL0=CDe
09RHV02<S8`0UiNG0;Vi^@2ZZZX0];Bd0<71`@2OWil0WYjN09>CT`2DU9@06ATI01DE5@10@400MWIf
06YZJP0O7al09BDU0410@01eMGD0G5aL02LW9`0P820071`L01/K6`0R8R80M7Ad0<C4a03IfMT0E5AD
07IfMP2SXj<0[Zj^024Q8@0O7al0;2`/02PX:00c<c<0>3Ph05YJFP1NGUh0DU9B0451@@0c<c<0<30`
04I6AP1JFUX0A4A402HV9P0Q8B40>3Ph0820P01ZJVX0?3`l02LW9`0];Bd0<C4a03@d=016ATH0R8R8
08:2PP2UYJD0bl_;0<O7a`1^KVh0E5AD08n?S`1hN7P0E5AD06e]K@1IFET0JVYZ05mOG`1?Cdl0Fe]K
09NGU`17AdL0@D5104Y:BP20P800]kNg02DU9@0I6AT05ADE01LG5`0R8R807AdM03@d=01gMgL0[:b/
08V9R@1WIfL0YJFU07moO`1RHV80G5aL06e]K@2SXj<0c/k>09nOW`1ADE40=cLg02XZ:P0N7Qh08R8R
03<c<`0T92@06a/K01hN7P14A4@0NWYj03Ti>@1eMGD0`l?30<K6aP2DU9@0PX:201dM7@0F5QH06a/K
024Q8@0S8b<0@d=308N7Q`1DE5@0]KFe0:f][@27QhL0Kfm_08f=S@2_[jl0_[jn0=OGe`3?cll0RXZ:
06EUI@1BDU80H61P06EUI@1eMGD0Lg=c08J6QP24Q8@0O7al0=;BdP3EeMD0[Zj^06a/K00O7al0=CDe
03Lg=`1CDe<0ATI602`/;00`<300=CDe4?ooool000kooooo02d0WYjN06ATI01UIFD0CDe=08b<S00o
?cl0BDU908B4Q016ATH0DE5A04i>CP0S8b<0?Shn04];B`0V9RH0C4a<0410@00g=cL09RHV04M7A`0R
8R80EUIF05]KF`11@D40Cdm?02`/;00f=SH0BTY:03/k>`0S8b<06ATI02@T901EEED0Bd];01hN7P0o
?cl0>c/k03Lg=`1KFe/08R8R02/[:`0N7Qh0F5QH059BDP1JFUX00P0^;Rh0PP17AdL0@D5104i>CP0/
;2`0BDU905iNGP1LG5`0;bl_04];B`1QHF4071`L03Xj>P1cLg<0F5QH07=cL`18B4P0410@059BDP1_
Kfl0@41004I6AP1cLg<0ATI605YJFP0X:2P0<S8b04=3@`0a<C40:RXZ0451@@1;Bd/0EUIF051@D00H
61P0DE5A05QHF01GEeL0Rh^;06m_K`2LW9`0Fe]K06m_K`0k>c/0YZJV07UiN@0d=3@0?3`l05aLG01@
D500T92@0<[:bP1KFe/0=cLg05UIF@0m?Cd0CDe=06ATI00l?3`0;bl_06YZJP1IFET0:RXZ06UYJ@1<
C4`0LW9b06ATI00c<c<0OWin05YJFP1jNWX0FUYJ04Y:BP2?Shl0FEUI04M7A`1BDU80IfMW055AD@1C
De<0MgMg05]KF`1TI6@0NGUi07IfMP1<C4`0>SXj08R8R01cLg<0;bl_0861P@1CDe<0>SXj0861P@1N
GUh0=SHf08R8R01PH600@D51065QH@13@d<0?Cdm05QHF014A4@0?3`l08>3P`0l?3`0P82007inOP1S
Hf<0KFe]06UYJ@2@T900?Shn04];B`1GEeL0YJFU04U9B@0`<300?3`l096AT@26QXH0GEeM09bLW017
AdL0UYJF0<C4a01jNWX0EUIF0820P01<C4`204i>CP0=02DU9@0d=3@0H61P06IVIP16ATH0@41009RH
V01EEED0:b/[065QH@1fMWH0GemO07YjNP020451@@1D092@T01ADE40NGUi03De=@1GEeL0QXJ604m?
C`0d=3@0O7al06UYJ@20P800@D5109bLW01LG5`0MWIf03`l?01ZJVX0PX:205aLG01XJ6P0CTi>04Y:
BP2=SHd0L71`0451@@1LG5`0MgMg03Lg=`1;Bd/0P82006i^KP16ATH0HF5Q08:2PP12@T80Ng]k079b
LP0h>3P0J6QX05QHF01@D500FUYJ04M7A`2SXj<0QHF507inOP0h>3P0GUiN05EEE@0M7Ad0C4a<05AD
E00c<c<0:b/[07]kN`1PH600B4Q80<30`021PH40>CTi05=CD`1jNWX0>CTi0:b/[01/K6`0DU9B03`l
?025QHD0GUiN08:2PP0j>SX0?3`l06m_K`1HF5P0J6QX0820P01<C4`092@T038b<P1KFe/0?clo03Xj
>P2o_kl0SHf=3_ooool0;01RHV80G5aL05iNGP1OGel0I6AT061PH01WIfL0UYJF09>CT`24Q8@0Ti>C
08j>SP0m?Cd08b<S02`/;01WIfL0GemO06EUI@1BDU80DE5A05MGE`2;Rh/0QhN705ADE014A4@0=3@d
02l_;`0j>SX0:2PX02@T901EEED0Ph>307moO`18B4P0>CTi08V9R@0Z:RX09RHV02<S8`0A4A403`l?
00h>3P0C4a<0CTi>0P0M7Ad01`0L71`0?clo09:BTP0k>c/05aLG01DE5@0F5QH00P0b<S80:`0[:b/0
<c<c01LG5`0F5QH06QXJ01dM7@0O7al0<30`09FEU@2KVi/0;bl_04Y:BP11@D40820P01DE5@0B4Q80
51@D01HF5P0G5aL06QXJ02PX:00i>CT0?Cdm05UIF@1YJFT0Kfm_05UIF@1LG5`0J6QX0:>SX`1]KFd0
Kfm_06IVIP0N7Qh0BTY:0:RXZ01mOGd0Fe]K04Y:BP19BDT0ADE501/K6`0I6AT00P0G5aL0J00H61P0
6ATI03De=@1EEED0NGUi0;Vi^@2LW9`0OGem06=SH`1PH600Ph>307IfMP1LG5`0ADE503De=@0K6a/0
5ADE01LG5`0O7al0:b/[01PH6016ATH0<c<c04I6AP1>CTh0Cdm?07moO`3:b/X0V9RH061PH00d=3@0
:RXZ065QH@16ATH07AdM0492@P1LG5`0De=C01hN7P0M7Ad06a/K01dM7@0n?Sh0GemO06UYJ@1@D500
?Cdm06ATI02<S8`0FUYJ06IVIP25QHD0NGUi0;Bd]02SXj<08b<S01dM7@0Q8B40:BTY02XZ:P0O7al0
61PH01lO7`0T92@0;2`/04a<C024Q8@0a<C406QXJ00Q8B40:BTY024Q8@0S8b<0?Shn07inOP1;Bd/0
;bl_02`/;015ADD0J6QX06YZJP2=SHd0Kfm_06e]K@1aLG40Shn?0<c<c01gMgL0>CTi038b<P12@T80
Xj>S0;no_`3:b/X0c<c<0<K6aP2YZJT0^KVi0;>c/`2MWId0Y:BT07IfMP1nOWh0:b/[0P0E5AD01`0F
5QH06ATI04U9B@1eMGD0S8b<0:VYZ@2?Shl00P36a/H0?`22PX80Ng]k06e]K@1SHf<0F5QH061PH01h
N7P0IFEU06ATI017AdL09bLW02<S8`0Q8B406a/K01`L700Y:BT0GEeM03hn?P0S8b<0<30`09:BTP28
R8P0VIVI0:6QX@1[Jf/0:RXZ01dM7@0^;Rh0@T9207moO`2:RXX0WYjN0:^[Z`22PX805aLG01XJ6P0I
6AT09BDU0492@P25QHD0Zj^[09BDU01iNGT0GEeM04Q8B00l?3`0<c<c038b<P0o?cl0GEeM09jNWP2Q
XJ40J6QX03lo?`1TI6@09RHV01TI6@0J6QX07QhN04A4A01=CDd0EeMG04=3@`0@ooooo`003_ooool0
b02<S8`0I6AT05eMG@0m?Cd0LW9b03/k>`1:BTX0LW9b05IFEP27QhL0JVYZ02TY:@10@400DU9B02TY
:@1XJ6P0ADE50451@@0b<S80EeMG03<c<`1<C4`0DU9B03Xj>P1KFe/0>SXj04Y:BP17AdL0<C4a03Xj
>P0P8200;Bd]05ADE01?Cdl08R8R04Y:BP1DE5@0<c<c06=SH`0T92@0@d=302PX:01GEeL0GEeM09:B
TP0j>SX0<30`04Y:BP0/;2`0IfMW03`l?01MGEd0CDe=03@d=00i>CT0H61P069RHP0W9bL0>SXj05aL
G011@D40F5QH06=SH`0F5QH0@d=306=SH`0S8b<0?clo06MWI`17AdL0EeMG02l_;`0m?Cd0Hf=S03dm
?@0Z:RX0ATI6069RHP1lO7`0GEeM01TI6@1>CTh0D51@07=cL`2?Shl0GUiN09VIV@1^KVh0NGUi0492
@P29RHT0Jf][03hn?P1DE5@0I6AT05ADE01ZJVX0^[Zj04i>CP0d=3@0E5AD04=3@`19BDT0IVIV061P
H00b<S80GEeM05iNGP0a<C40MGEe05EEE@1eMGD0Cdm?0410@029RHT0G5aL06e]K@1CDe<0B4Q809NG
U`1XJ6P0EEEE04M7A`1EEED0EUIF051@D01RHV80Bd];03/k>`1dM7@0K6a/03De=@0/;2`0OWin079b
LP0m?Cd0Shn?05UIF@0d=3@0Ogmo08:2PP10@400UYJF04e=C@1BDU80PX:205QHF016ATH0Bd];04A4
A00f=SH0Ogmo0492@P26QXH0MWIf05mOG`1eMGD0O7al0;^k^`1MGEd0ATI606MWI`2TY:@0GemO03lo
?`0d=3@0Q8B406m_K`16ATH0R8R804Y:BP2:RXX0a<C408j>SP1NGUh0K6a/061PH01LG5`0BTY:02d]
;@0h>3P0K6a/05QHF00`<300:BTY08F5Q@1?Cdl09BDU05QHF01SHf<0F5QH0;Ng]`1]KFd0>CTi09VI
V@14A4@0J6QX03lo?`1CDe<0SXj>0P15ADD0C020P800FUYJ08>3P`15ADD0Vi^K05UIF@1^KVh09bLW
05eMG@1ZJVX0HV9R06m_K`16ATH0?clo0820P01FEUH0>c/k04i>CP1jNWX0;bl_03Ti>@1UIFD0JFUY
04E5A@1OGel0NGUi04A4A02EUID0Q8B403lo?`1/K6`0CTi>04Y:BP1XJ6P0DU9B0<O7a`2a/K40O7al
02DU9@1?Cdl0JVYZ02TY:@1DE5@0K6a/04I6AP0S8b<0JVYZ03`l?01:BTX0Zj^[09BDU011@D40EEEE
075aL@0R8R80Ogmo05ADE01?Cdl0ATI6065QH@1`L700[Zj^06MWI`10@400Ng]k04a<C01VIVH0SXj>
06i^KP0^;Rh0@T9207AdM01:BTX0;bl_0::RXP2;Rh/>ooooo`0Y04=3@`0c<c<0ADE502h^;P0W9bL0
:BTY04a<C022PX80LW9b08F5Q@1FEUH0EeMG06YZJP0];Bd07alO01<C4`18B4P0P82004m?C`12@T80
Fe]K0:^[Z`1DE5@07QhN010@400G5aL05QHF01<C4`0=3@d061PH05]KF`1aLG40CDe=05QHF00b<S80
Ph>302DU9@0E5AD0<c<c01DE5@0>3Ph00P0=3@d0201RHV80BDU901`L700G5aL06a/K05EEE@16ATH0
5ADE0P0D51@01`0J6QX05ADE01DE5@0F5QH05ADE014A4@0F5QH00P0I6AT01P0M7Ad0A4A40861P@10
@400F5QH024Q8@804a<C00<04A4A01@D500A4A400P0A4A405@0E5AD04a<C01PH600U9BD0GEeM04E5
A@0Z:RX0:b/[01LG5`1JFUX0?3`l04a<C02BTY80Ng]k071`L01MGEd06QXJ020P800Z:RX0;Bd]02<S
8`0301DE5@0X01<C4`0E5AD05ADE01LG5`18B4P0@d=3075aL@1[Jf/0CDe=051@D02GUiL0Ogmo03Ti
>@0Q8B407QhN02l_;`1MGEd09RHV01LG5`0K6a/0:BTY01XJ6P1ADE406QXJ024Q8@0O7al0820P0451
@@2RXZ80K6a/02d];@1;Bd/0EEEE0451@@1bLW80CTi>055AD@0c<c<0FUYJ02TY:@@06a/K038071`L
02DU9@1cLg<0fMWI0;fm_@0_;bl09RHV02LW9`0a<C40QXJ60=?Cd`2QXJ40?Shn02<S8`0O7al092@T
02h^;P0];Bd0820P01XJ6P0N7Qh0:BTY04A4A01^KVh0c/k>08B4Q00];Bd0JVYZ034a<@0Y:BT0<c<c
069RHP0F5QH08B4Q05=CD`1mOGd0Kfm_08F5Q@1iNGT0Xj>S06i^KP0U9BD0>3Ph09bLW00T92@0:RXZ
0451@@1CDe<0AdM703Lg=`80=SHf01L0?Cdm05UIF@25QHD0YJFU059BDP16ATH0EEEE09^KV`1:BTX0
6ATI01DE5@0G5aL092@T061PH028R8P0ZJVY0;Rh^02n_[h0d];B0;no_`1oOgl09bLW01dM7@0201lO
7`0j01`L7027QhL0MgMg03Xj>P15ADD0BTY:01`L700I6AT061PH01lO7`19BDT0Hf=S01lO7`0M7Ad0
:RXZ05aLG00e=CD0;bl_096AT@3Fe]H0]KFe05mOG`1dM7@0KVi^08n?S`1aLG40SXj>06a/K01QHF40
=3@d01dM7@0I6AT06QXJ01lO7`2@T900IFEU05IFEP0L71`06QXJ01/K6`0I6AT071`L024Q8@0a<C40
K6a/0;Jf]P1PH600@d=303Ti>@1=CDd0IFEU01DE5@0K6a/0:BTY02PX:00Y:BT0@D51051@D13ooooo
000>ooooo`3o092@T01eMGD0GEeM055AD@1mOGd0?Cdm05ADE01KFe/0<c<c04i>CP1EEED08R8R04=3
@`0^;Rh0:RXZ065QH@0o?cl08b<S03Lg=`1QHF40@T9204i>CP1;Bd/0;2`/05ADE00f=SH0EUIF05UI
F@0Y:BT0;Bd]01hN7P0l?3`0GUiN01XJ6P0U9BD0Kfm_05IFEP0i>CT0GemO02LW9`16ATH0820P03Lg
=`0e=CD0IVIV03dm?@0/;2`0ATI6038b<P1ADE40;Rh^06ATI01WIfL0Cdm?02/[:`0a<C40Bd];03@d
=016ATH0J6QX04];B`1DE5@0?clo01DE5@0g=cL0L71`03dm?@0f=SH0OWin04M7A`19BDT0:BTY02XZ
:P18B4P0>CTi02h^;P0o?cl0<c<c05MGE`1ADE407alO06IVIP13@d<0EeMG07emO@1EEED0SHf=06m_
K`1aLG40@d=308B4Q01PH600:b/[03Lg=`1KFe/0FUYJ08J6QP2j^[X0EeMG030`<01JFUX0@T9203dm
?@1?Cdl0HV9R02d];@1>CTh0I6AT02l_;`1OGel0GUiN06=SH`1XJ6P0Bd];08N7Q`1GEeL0Kfm_05UI
F@0e=CD0K6a/06IVIP1UIFD0GUiN065QH@0o?cl0DU9B06e]K@1NGUh0@d=308:2PP1bLW80C4a<0410
@029RHT0KVi^034a<@25QHD0FUYJ03Lg=`20P800Kfm_03dm?@2LW9`0IFEU0410@01HF5P0BDU903`l
?019BDT0CDe=03dm?@2@T900Bd];08F5Q@1]KFd0IfMW07IfMP1ZJVX0_[jn06e]K@19BDT0EeMG09RH
V01EEED0=cLg03Hf=P21PH40T92@04m?C`2CTi<0F5QH0861P@39bLT0V9RH069RHP1^KVh0L71`069R
HP16ATH0<S8b05QHF024Q8@0FEUI03<c<`0^;Rh0S8b<06MWI`0d=3@0EUIF03@d=010@400VIVI05YJ
FP0j>SX0OGem059BDP1VIVH0?clo04i>CP2OWil0CTi>0492@P1fMWH0Fe]K07]kN`1>CTh0VYZJ055A
D@1hN7P09bLW06=SH`1fMWH0IVIV07AdM01SHf<0B4Q80861P@0k>c/0A4A406e]K@2IVIT0@D5103/k
>`1ZJVX0<S8b02h^;P1oOgl0R8R80451@@1_Kfl0Hf=S04A4A025QHD0AdM703`l?016ATH0>3Ph092@
T01WIfL0J6QX02TY:@1<C4`0=CDe020P801EEED0MGEe03lo?`0/;2`0J6QX03lo?`0g=cL0T92@06EU
I@0e=CD0LW9b000G07inOP0c<c<0MgMg06IVIP18B4P0CDe=07YjNP1HF5P0SXj>0410@01fMWH0YJFU
06IVIP1ZJVX0[Jf]03Hf=P0X:2P0=cLg06EUI@1DE5@08B4Q0;Bd]01nOWh03_ooool0600l?3`0<30`
03Xj>P0S8b<071`L01TI6@0`<300BTY:04M7A`1aLG40EEEE04I6AP1/K6`0?3`l02PX:00E5AD0ADE5
09NGU`2TY:@0SHf=0;jn_P3Cdm<0FUYJ01XJ6P805ADE01X03@d=00h>3P0=3@d0:b/[05iNGP0U9BD0
6ATI03Ti>@0e=CD0Q8B402@T900C4a<04Q8B00l?3`0@41003@d=00l?3`1GEeL0RHV902<S8`0K6a/0
6ATI03<c<`1WIfL04A4A010@40804A4A00l04a<C01@D500D51@051@D01<C4`0P82008B4Q01HF5P0M
7Ad0D51@06ATI01ADE40B4Q801dM7@0E5AD00P0@4100500D51@05ADE01DE5@0A4A405QHF014A4@0=
3@d04A4A0451@@1?Cdl0<S8b01hN7P0E5AD0EEEE0410@00V9RH0FUYJ0<o?c`2BTY8092@T0P0N7Qh0
0`0Z:RX0=3@d020P800701DE5@0X04E5A@0j>SX0Jf][08B4Q01fMWH0C4a<092@T01FEUH0ADE502PX
:00Q8B408R8R03/k>`1dM7@0IFEU034a<@14A4@0:BTY0492@P0I6AT07alO02@T900N7Qh0;2`/08V9
R@1TI6@08B4Q01XJ6P0P8200@T9207IfMP1ZJVX0<S8b02LW9`1>CTh0FEUI01dM7@0L71`07AdM01hN
7P807AdM0300=SHf0<_;b`28R8P06ATI02LW9`0`<300=SHf0:^[Z`2DU9@0G5aL05MGE`0i>CT0C4a<
05]KF`1<C4`0GEeM04Y:BP0`<3009RHV030`<00i>CT0L71`0=oOg`2c/k<0=SHf034a<@1GEeL0=SHf
05=CD`2:RXX0H61P06IVIP13@d<09RHV06YZJP1hN7P0ADE505EEE@2CTi<0O7al06QXJ02VYZH0A4A4
061PH01GEeL0<C4a01LG5`0N7Qh201dM7@0H01lO7`0_;bl0ATI607MgM`1CDe<0=3@d04i>CP2SXj<0
Zj^[05MGE`0M7Ad0=SHf05]KF`1=CDd0Hf=S09FEU@2j^[X0S8b<0;Ng]`2h^;P0BTY:028R8P0N7Qh0
9BDU0P0m?Cd09P1mOGd0FEUI06=SH`0d=3@0>SXj02<S8`0K6a/061PH02PX:01FEUH0>SXj01`L700N
7Qh0;bl_03`l?00U9BD09RHV05MGE`3EeMD0MWIf01dM7@0R8R80;bl_06UYJ@1WIfL0SXj>05=CD`1P
H600D51@04e=C@0n?Sh08R8R02l_;`1QHF40C4a<05mOG`0P82005QHF0P0E5AD04@0L71`09BDU03hn
?P2f][H0S8b<04Y:BP0c<c<0<S8b0492@P1^KVh06QXJ020P800];Bd08R8R02@T900Y:BT0DE5A013o
oooo000>ooooo`3:09>CT`1bLW80H61P04];B`1fMWH0?clo04M7A`23Ph<0?Shn05MGE`11@D40<c<c
06e]K@1JFUX09BDU05UIF@16ATH0D51@02/[:`1FEUH0=CDe05eMG@1`L700>3Ph061PH00U9BD0?3`l
055AD@1:BTX08R8R01DE5@0N7Qh0De=C04I6AP0M7Ad0@D510492@P14A4@0MgMg038b<P0i>CT092@T
069RHP1CDe<0HV9R02DU9@0b<S80Bd];0492@P1CDe<0:BTY051@D01QHF40I6AT034a<@0d=3@0ADE5
02<S8`16ATH0I6AT05ADE01YJFT0>CTi01XJ6P14A4@0OGem03`l?00];Bd0Jf][03<c<`19BDT0D51@
0451@@15ADD05QHF03De=@1<C4`0BTY:05IFEP19BDT05QHF03lo?`1EEED0I6AT08>3P`1=CDd0TI6A
05mOG`1aLG40B4Q80:^[Z`27QhL0>3Ph03hn?P1GEeL0@D510:>SX`2f][H0DU9B030`<01WIfL0AdM7
04i>CP1YJFT0LW9b03Ti>@1?Cdl0Kfm_03lo?`1VIVH0GemO06][J`1`L700Hf=S0:ZZZP1@D500KFe]
06MWI`17AdL0KVi^075aL@1TI6@0Hf=S04m?C`1DE5@0FEUI07EeM@1_Kfl0C4a<0820P02BTY80@d=3
038b<P28R8P0FEUI02`/;020P800DU9B03<c<`1kNg/0MGEe03Hf=P2EUID0KVi^04m?C`1`L700@T92
03Lg=`1<C4`0BDU90451@@29RHT0AdM708V9R@1aLG40DE5A05ADE01FEUH0]KFe08N7Q`1HF5P0FUYJ
092@T011@D40?Cdm07AdM02UYJD0YZJV05=CD`2@T900FEUI07emO@3De=@0[:b/06UYJ@2IVIT0KVi^
096AT@1cLg<0=3@d03De=@25QHD0GUiN06=SH`0^;Rh0RHV905EEE@0a<C40Ng]k069RHP1KFe/0Xj>S
051@D01FEUH0U9BD05UIF@1CDe<0>CTi06i^KP2TY:@0Q8B404e=C@80K6a/0400I6AT04U9B@2e]KD0
VYZJ07]kN`0^;Rh0EUIF04Q8B01PH600TI6A05mOG`1:BTX0P820055AD@0k>c/0ADE508^;R`1=CDd0
EEEE07]kN`1bLW80@D51051@D029RHT0DU9B08f=S@1TI6@0;Rh^069RHP1XJ6P0Cdm?05MGE`1IFET0
ZJVY06QXJ01TI6@0>SXj06a/K01HF5P06a/K03`l?01:BTX05ADE02<S8`24Q8@0GUiN03De=@1oOgl0
Kfm_038b<P19BDT0K6a/02`/;01NGUh0GemO03lo?`0U9BD0MWIf065QH@1gMgL07QhN03Ti>@1dM7@0
@T920P1[Jf/0201HF5P08R8R02`/;01MGEd0@T9201lO7`2UYJD0V9RH3_ooool0:00f=SH0<c<c03Lg
=`0R8R806QXJ01TI6@0T92@0D51@03hn?P0f=SH0U9BD0492@P1lO7`0Kfm_02`/;00S8b<0L71`0<S8
b03Cdm<0`/;20=OGe`2/[:`0UIFE06i^KP1NGUh0?clo02l_;`0X:2P07QhN04M7A`0W9bL05ADE01<C
4`0K6a/0;2`/08:2PP0/;2`061PH018B4P0D51@200h>3P0<014A4@10@400ZZZZ03/k>`0G5aL06QXJ
061PH02JVYX08B4Q014A4@0@41004a<C0P0A4A40400D51@05ADE01DE5@0I6AT0>3Ph01XJ6P0K6a/0
KVi^08b<S01gMgL0<c<c01LG5`0E5AD04Q8B014A4@0@410401DE5@0E014A4@0>3Ph0410@01`L701I
FET0>CTi01`L700J6QX0EEEE0492@P0`<300;bl_09ZJVP1DE5@06ATI01/K6`0P8200:b/[030`<00O
7al05QHF00H05ADE00/0?clo03hn?P1ADE40<c<c02d];@0c<c<0EUIF03<c<`0W9bL08B4Q02<S8`02
02LW9`0>04A4A01TI6@0Shn?09bLW01oOgl0<S8b01HF5P0M7Ad0;Bd]01dM7@0Q8B40Ph>308b<S00S
8b<201`L700:01hN7P0d=3@0DU9B03/k>`0O7al0;Bd]0:VYZ@1KFe/0820P01PH60<07alO01l0@D51
08n?S`1^KVh06ATI02DU9@0d=3@0Bd];08R8R01=CDd0VYZJ03Hf=P14A4@0FUYJ09RHV02VYZH0WIfM
0<G5a@1OGel09BDU02/[:`0m?Cd0WYjN0=cLg03IfMT0OWin02h^;P1QHF40OWin05iNGP1dM7@0F5QH
00805aLG03D061PH06m_K`1=CDd0FUYJ051@D00U9BD0K6a/0;jn_P2_[jl0FEUI0451@@0Q8B406a/K
01XJ6P0L71`07AdM02<S8`0X:2P0;bl_04E5A@2o_kl0d=3@09VIV@23Ph<0dM7A0>;RhP2f][H0W9bL
06][J`19BDT0DE5A06i^KP1PH600^[Zj08n?S`1IFET0SXj>08f=S@19BDT0A4A405mOG`20P800W9bL
09>CT`1DE5@0M7Ad08j>SP1BDU80E5AD03Xj>P0/;2`0;Bd]06ATI002024Q8@0802<S8`1HF5P0@410
03Lg=`0];Bd0Ng]k09VIV@0^;Rh201/K6`0I04A4A01/K6`0OWin05UIF@0m?Cd0AdM706ATI00];Bd0
?3`l06YZJP2WYjL0CTi>03Xj>P1fMWH0CDe=024Q8@0[:b/0>SXj079bLP2ZZZX0e=CD0=OGe`2c/k<0
?Cdm02XZ:P0203Lg=`0807QhN00a<C408b<S02HV9P0N7Qh07alO02/[:`1cLg<@ooooo`003_ooool0
:P2=SHd0EUIF059BDP1?Cdl0S8b<03hn?P1;Bd/0Shn?04A4A01FEUH0C4a<028R8P16ATH0E5AD02LW
9`0f=SH0>SXj05=CD`0V9RH0EUIF04A4A00m?Cd0JVYZ038b<P1IFET0>CTi03dm?@19BDT0=cLg02<S
8`0G5aL0:b/[05QHF01DE5@08B4Q0410@01NGUh09bLW079bLP0d=3@0?clo01TI6@80FEUI03/0H61P
038b<P0R8R80?Shn05EEE@1PH600?Shn061PH01MGEd0MWIf03hn?P0a<C40K6a/02TY:@0j>SX0Lg=c
04m?C`1GEeL0?Cdm01lO7`0^;Rh0NWYj04i>CP0M7Ad0MGEe0492@P0k>c/0;bl_02h^;P0l?3`0;Bd]
03@d=019BDT0D51@05iNGP1JFUX05QHF05=CD`1LG5`0MgMg09jNWP1`L700Shn?07EeM@1ZJVX0<C4a
06UYJ@1JFUX0=CDe04E5A@1SHf<0ADE50;Fe]@30`<00Lg=c03`l?01bLW80<c<c06YZJP0206UYJ@0?
03De=@1jNWX0Jf][0410@01WIfL0Cdm?08R8R01ZJVX0G5aL0<S8b01IFET0PX:206a/K014A4@0LW9b
0080Jf][0480MGEe04E5A@1ADE40DE5A09^KV`1gMgL0>c/k07=cL`1IFET0DU9B03Ph>026QXH0S8b<
02/[:`2KVi/0NWYj03Xj>P2ATI40MgMg0410@02NWYh0K6a/061PH02ATI40FUYJ03Xj>P1`L700HV9R
03De=@2;Rh/0?Shn09FEU@23Ph<0De=C04m?C`0i>CT0YZJV0:ZZZP22PX80G5aL0;2`/01NGUh0=CDe
03Lg=`1oOgl0N7Qh06e]K@2ZZZX0Fe]K04Q8B02f][H0MGEe04I6AP1YJFT0@d=304I6AP1CDe<0=SHf
04Y:BP1EEED0K6a/07UiN@0a<C40Shn?04];B`0U9BD205aLG01H06EUI@2UYJD0LW9b02LW9`1`L700
I6AT092@T00o?cl0?Cdm09RHV016ATH0B4Q8079bLP1nOWh0Ph>30410@01/K6`0Cdm?07AdM01EEED0
O7al04A4A00o?cl0GEeM03Ti>@1ZJVX0TY:B075aL@0n?Sh0AdM707YjNP0/;2`0A4A406][J`1aLG40
@41004a<C02:RXX0>CTi09NGU`29RHT0>c/k07=cL`1`L700F5QH08F5Q@1QHF40[:b/096AT@15ADD0
:RXZ059BDP1QHF407AdM03`l?016ATH0=SHf028R8P1ZJVX0D51@030`<01]KFd0Lg=c04I6AP1DE5@0
N7Qh03Hf=P0n?Sh0De=C04e=C@0];Bd0SHf=05QHF021PH40=cLg04E5A@2;Rh/0C4a<06m_K`23Ph<0
HV9R02LW9`0a<C40G5aL04E5A@0[:b/0X:2P09NGU`kooooo0200=cLg0410@00a<C40820P01XJ6P0G
5aL0:RXZ030`<01hN7P0@4100<G5a@3:b/X0a<C40<71`@23Ph<0LG5a09BDU03>c/h0a/K60=CDe02l
_;`0@T92079bLP1ADE40Hf=S075aL@1/K6`0Shn?08b<S02LW9`05aLG01@D50805ADE01L0;Rh^08B4
Q00b<S806ATI01/K6`0k>c/05aLG00l?3`0E5AD0>SXj0;Ng]`17AdL0?Shn096AT@3>c/h0g=cL0;Fe
]@20P800GemO034a<@0L71`04a<C01DE5@0201<C4`0<01LG5`0i>CT0=cLg02DU9@2@T900[:b/07=c
L`0[:b/05aLG01<C4`0A4A404a<C0`0E5AD00`0F5QH04A4A010@4002010@400;01DE5@1LG5`0ADE5
020P800K6a/0EeMG05ADE00[:b/0>3Ph0:RXZ01:BTX00P0L71`0100M7Ad08b<S03@d=00Q8B4601DE
5@0<01TI6@15ADD0BDU904m?C`0k>c/0:BTY01/K6`0[:b/0Fe]K02XZ:P0Y:BT0=SHf0P0d=3@0PP0o
?cl0I6AT05MGE`1nOWh0^KVi04I6AP0I6AT0=CDe03/k>`0P82008b<S08V9R@3:b/X0DE5A01PH600J
6QX06a/K02LW9`0_;bl0Bd];024Q8@0T92@0P8200<c<c02:RXX0@D5101hN7P0O7al08b<S06e]K@1`
L700NWYj02h^;P0Q8B40<c<c075aL@1=CDd0=3@d079bLP0h>3P0<S8b02`/;00g=cL0L71`0::RXP30
`<00N7Qh0410@01:BTX0YZJV0<c<c034a<@0WIfM0<_;b`2^[Zh0Kfm_092@T01lO7`0GemO06][J`0Y
:BT08B4Q02d];@2DU9@0BTY:08j>SP1<C4`0;2`/02HV9P2TY:@0^k^k09BDU00b<S806a/K01TI6@0K
6a/07AdM02HV9P0d=3@0DU9B071`L028R8P0_kno0<K6aP1QHF40NGUi0>CTi037alL0`l?30;Ng]`1k
Ng/0PX:206=SH`14A4@0LW9b0<_;b`16ATH05aLG075aL@3:b/X0/;2`09>CT`1UIFD0@41003`l?016
ATH0MGEe09JFUP30`<00^;Rh0;Jf]P34a<@0ZJVY0;:b/P2NWYh092@T02HV9P15ADD0RHV905ADE01C
De<0EeMG09NGU`1LG5`05ADE01XJ6P0b<S80Cdm?07alO01iNGT202LW9`0H03De=@1hN7P0CDe=02XZ
:P0k>c/0De=C05eMG@0[:b/0T92@0=[JfP30`<00b/[:0<k>cP37alL0[Zj^0;Fe]@2UYJD0NGUi03lo
?`13@d<0?clo04=3@`1UIFD0HF5Q0`0M7Ad00`0f=SH0DU9B0;6a/@0@ooooo`003_ooool0:`2WYjL0
PX:2059BDP14A4@0PH6103hn?P0m?Cd0GEeM04U9B@1kNg/0Lg=c02@T900o?cl0Cdm?04U9B@1XJ6P0
C4a<059BDP0`<300I6AT05=CD`12@T80G5aL03`l?01RHV80A4A403Xj>P16ATH092@T03/k>`0R8R80
:b/[05QHF01ADE409BDU03dm?@1QHF40;Bd]075aL@0/;2`0=SHf01/K6`1MGEd00P1OGel0[@1CDe<0
ADE504U9B@1YJFT0IFEU04A4A01OGel0DE5A06][J`15ADD0<c<c06i^KP0_;bl0?Shn07MgM`1ADE40
EeMG05]KF`0b<S80;bl_08^;R`1SHf<07AdM08N7Q`1MGEd0CTi>04I6AP14A4@0HV9R03Ti>@0e=CD0
B4Q805=CD`1_Kfl0DU9B01DE5@1KFe/0@T92071`L02BTY80Jf][096AT@1cLg<0M7Ad04U9B@1mOGd0
IFEU03Ph>01;Bd/0HF5Q03Hf=P2BTY80S8b<05EEE@0[:b/0CTi>02<S8`14A4@0L71`07AdM00Y:BT0
?clo05aLG013@d<0N7Qh03`l?01aLG40NGUi06EUI@3Cdm<0A4A405QHF01QHF40GemO09JFUP1TI6@0
C4a<051@D01CDe<0GUiN04e=C@1eMGD0De=C04U9B@23Ph<0JFUY04E5A@0_;bl0UiNG08B4Q00g=cL0
Ng]k05eMG@0f=SH0QHF508:2PP0/;2`0Ng]k03dm?@1MGEd0QhN705YJFP0l?3`0CTi>05IFEP1@D500
S8b<02/[:`1jNWX0V9RH06a/K01mOGd0Cdm?09ZJVP2g]kL0A4A404E5A@2UYJD0QHF504e=C@0e=CD0
M7Ad08V9R@1DE5@0PH6107YjNP0h>3P0a<C409JFUP1dM7@0NGUi07EeM@15ADD0?Cdm02l_;`23Ph<0
JVYZ04e=C@12@T80=CDe0:2PX01VIVH0<30`061PH00T92@0B4Q808F5Q@1WIfL092@T079bLP10@400
GemO04U9B@13@d<0ZZZZ04M7A`1DE5@0Cdm?069RHP1bLW80?clo07moO`1ADE40N7Qh02PX:01SHf<0
GemO05MGE`1ZJVX0IFEU04e=C@2HV9P0Lg=c0080@D5100`0K6a/03@d=00o?cl0JVYZ07MgM`0o?cl0
D51@07]kN`16ATH0MgMg06i^KP0Y:BT205mOG`0/04a<C01:BTX0ATI60:^[Z`23Ph<0IVIV02l_;`1<
C4`0IVIV028R8P0g=cL0DU9B030`<00V9RH0H61P04Y:BP0c<c<0O7al061PH01;Bd/0DE5A06ATI013
@d<0>SXj07UiN@1WIfL0EUIF09RHV01MGEd0WIfM05EEE@19BDT0QHF504E5A@1cLg<0UYJF06QXJ00V
9RH0;bl_05iNGP0/;2`0<30`06m_K`2>SXh>ooooo`0P0492@P18B4P0;Bd]024Q8@0L71`06a/K0410
@00o?cl0HV9R0;Zj^P3Fe]H0UIFE0:BTY02VYZH0VIVI0;Jf]P3GemL0/k>c0=GEe@3Ogml0Ti>C02<S
8`1?Cdl09bLW01dM7@0J6QX06ATI02DU9@0I6AT0Ng]k05YJFP0H61P201DE5@0301/K6`2;Rh/0=CDe
00805ADE024061PH01/K6`0C4a<071`L04M7A`2e]KD0cLg=0<O7a`3>c/h0d];B0=?Cd`32`/80_Kfm
0<30`02j^[X0WIfM07inOP1LG5`0<c<c038b<P16ATH0Shn?0:JVYP1>CTh0Y:BT08B4Q01HF5P08B4Q
01<C4`0E5AD04Q8B01<C4`0D51@00`0E5AD200l?3`0D00h>3P0A4A403@d=03Xj>P1^KVh08R8R01LG
5`1YJFT0Jf][04];B`2PX:00e=CD06EUI@0k>c/06ATI01`L700M7Ad0;bl_02<S8`0D51@401DE5@0P
01<C4`0F5QH0Jf][09BDU01dM7@0;Rh^03<c<`0N7Qh06ATI07QhN020P800FEUI05ADE01;Bd/0De=C
06UYJ@2BTY80Y:BT0;Zj^P36a/H0NWYj04i>CP23Ph<0VYZJ02l_;`1QHF40/k>c0:2PX02=SHd0=SHf
01`L700G5aL201hN7P0G04M7A`0/;2`07alO071`L01CDe<0ATI60820P01[Jf/0<S8b02l_;`1YJFT0
H61P06MWI`0f=SH0820P05ADE01oOgl0B4Q802h^;P1MGEd05ADE02HV9P1=CDd00P0o?cl0DP1@D500
Jf][09bLW02;Rh/0_;bl0861P@1ADE40[jn_09NGU`2NWYh0Xj>S0=7Ad@2c/k<0SXj>08n?S`1SHf<0
QHF508R8R02b/[80`L7105iNGP2l_;`0;Rh^01`L7012@T80Shn?09fMW@1CDe<0GemO01XJ6P0G5aL0
6a/K01XJ6P0M7Ad0AdM704Y:BP10@400G5aL08j>SP2EUID0^[Zj0;Fe]@2_[jl0bLW90<?3``3:b/X0
fMWI0<c<c01iNGT0TI6A07MgM`2/[:`0:BTY01dM7@24Q8@0AdM7030`<00/;2`0@d=306YZJP1]KFd0
DU9B06UYJ@2b/[80Vi^K06UYJ@1FEUH0G5aL08b<S01GEeL0VIVI06a/K010@400RXZ:0451@@15ADD0
DE5A08f=S@2QXJ40ADE501/K6`0R8R8204U9B@0R079bLP1HF5P07alO01`L700Z:RX0CTi>09NGU`0m
?Cd0BDU904A4A016ATH0A4A40;Ng]`3Fe]H0][Jf0:NWY`2YZJT0TI6A08>3P`2ZZZX0Cdm?07EeM@1X
J6P0?Cdm02DU9@11@D40DU9B08f=S@0/;2`0;Rh^0451@@1BDU80M7Ad09VIVA3ooooo000>ooooo`2N
07AdM01OGel0D51@04I6AP1LG5`0=SHf04a<C01YJFT0ATI604Q8B01QHF40<C4a06UYJ@1ADE409BDU
04M7A`1;Bd/0=cLg028R8P1KFe/0EEEE04];B`1WIfL0:2PX03lo?`0^;Rh0<S8b04a<C00^;Rh09BDU
02/[:`0_;bl0Fe]K03<c<`0O7al0>3Ph04U9B@0/;2`0MgMg024Q8@0h>3P08B4Q04E5A@14A4@0IVIV
03lo?`0l?3`0IVIV04Y:BP1>CTh0=SHf05]KF`1/K6`0FEUI03De=@0h>3P0KFe]034a<@1EEED0JFUY
03hn?P13@d<0BTY:02PX:00m?Cd0SHf=04a<C00J6QX0Ng]k04M7A`1YJFT0GEeM04Q8B012@T80:BTY
03dm?@1FEUH0@d=307YjNP1JFUX07AdM065QH@1NGUh0K6a/08b<S01<C4`0O7al07MgM`20P800ADE5
07moO`1PH600<C4a05UIF@1VIVH0@T920<30`02][Jd0I6AT02l_;`1TI6@0=CDe04=3@`1:BTX0M7Ad
03Ti>@16ATH0Fe]K0410@01UIFD0G5aL06IVIP1SHf<0M7Ad0<C4a01GEeL0DU9B051@D01?Cdl0FUYJ
07EeM@1@D500BTY:0410@01EEED0EeMG06a/K01MGEd0Cdm?07emO@1NGUh0>CTi02PX:023Ph<0Lg=c
02d];@1nOWh0Ng]k03hn?P2CTi<0P82003dm?@2FUYH0JVYZ059BDP1bLW80Fe]K0492@P1?Cdl0D51@
038b<P28R8P0D51@07moO`1cLg<0C4a<05mOG`1BDU8209JFUP0j05UIF@1BDU80RHV9096AT@1HF5P0
CDe=08R8R02JVYX0FEUI08J6QP1MGEd0ATI60;no_`2VYZH0JVYZ0861P@1PH600C4a<05IFEP0a<C40
A4A407inOP1MGEd0EUIF03Xj>P21PH40H61P03@d=01XJ6P0De=C06MWI`2BTY80GemO038b<P1jNWX0
G5aL075aL@0i>CT0Cdm?0:^[Z`1kNg/0G5aL06QXJ01aLG40P82003dm?@25QHD0PH6107moO`0f=SH0
GemO05eMG@1>CTh0JFUY06m_K`1BDU80UYJF07moO`80A4A403X0RHV904U9B@12@T80N7Qh07IfMP15
ADD0H61P08j>SP1HF5P0S8b<06MWI`0j>SX0K6a/07]kN`1TI6@0F5QH04A4A02WYjL0T92@06=SH`0d
=3@0CTi>04e=C@0V9RH0BDU906a/K01>CTh092@T05IFEP0l?3`0<c<c06e]K@2;Rh/0Bd];065QH@1Z
JVX092@T04U9B@2PX:00I6AT03Hf=P21PH40C4a<09JFUP0l?3`0F5QH09BDU01<C4`0MWIf0::RXP1D
E5@0:BTY03Xj>P1WIfL0EeMG020P801MGEd0SXj>3_ooool0C@1MGEd0OWin04M7A`0W9bL07QhN01lO
7`0_;bl092@T028R8P1jNWX0RXZ:09JFUP22PX80Hf=S04Q8B01^KVh0W9bL06YZJP1bLW80QHF50:f]
[@1WIfL0>SXj03hn?P0X:2P09BDU030`<00d=3@0820P034a<@2>SXh0QXJ602DU9@0G5aL0?clo0<[:
bP1HF5P061PH01LG5`0E5AD0820P05UIF@0k>c/0S8b<0:f][@39bLT0b/[:09>CT`1QHF40>3Ph02h^
;P0g=cL0?Shn0410@01<C4`0F5QH06=SH`1dM7@0O7al08F5Q@2EUID0X:2P0;Rh^03AdM40Zj^[07al
O01CDe<05QHF014A4@0@41004a<C010@400D51@0820P01@D500?3`l04A4A0080410@01404A4A02/[
:`2g]kL0D51@03`l?02DU9@0WinO0:BTY02i^KT0iNGU09^KV`1:BTX0?clo03/k>`17AdL0Fe]K03hn
?P0201@D501Y01LG5`0H61P08R8R03<c<`1VIVH0];Bd0<G5a@2YZJT0Lg=c0492@P0k>c/07AdM06a/
K00n?Sh09RHV02PX:00^;Rh0>CTi04E5A@1bLW80WYjN0::RXP2e]KD0`<3007]kN`0i>CT0MGEe0:j^
[P2<S8`0=CDe024Q8@18B4P0LW9b05EEE@12@T80;2`/01dM7@0j>SX0FUYJ05QHF01oOgl0KVi^04M7
A`1UIFD0OGem08V9R@2j^[X0VIVI096AT@2JVYX0^[Zj0;2`/03EeMD0PX:203Lg=`0_;bl0DU9B01HF
5P0E5AD05aLG03Lg=`2JVYX0Ogmo04U9B@1ZJVX0][Jf05ADE010@400ATI605=CD`2UYJD0][Jf09nO
W`2i^KT0UiNG07inOP1ADE40:b/[02l_;`0g=cL0CDe=0<71`@3;bl/0b/[:06MWI`18B4P0DU9B06=S
H`2KVi/0?Shn03Lg=`0K6a/05aLG01PH600I6AT08R8R02TY:@0n?Sh0DU9B07EeM@2>SXh0]KFe0<_;
b`22PX80QHF50080f=SH02T0eMGE0<_;b`3?cll0bl_;034a<@0M7Ad0UiNG05YJFP1]KFd0ADE502DU
9@0I6AT061PH01LG5`0F5QH071`L079bLP2i^KT0VYZJ09RHV01iNGT0L71`04];B`1gMgL0RXZ:09>C
T`32`/80X:2P03lo?`0N7Qh0:2PX07]kN`2WYjL0I6AT03Xj>P10@400?Shn02h^;P1KFe/0NGUi02/[
:`0301`L700M02TY:@1_Kfl0SHf=05aLG00j>SX0FEUI0:ZZZP2^[Zh0NWYj04I6AP0k>c/0?3`l02XZ
:P0S8b<0<C4a05EEE@1KFe/0Lg=c06QXJ00o?cl0Fe]K08R8R02NWYh0NWYj05aLG01CDe<0AdM7038b
<P0[:b/04?ooool000kooooo07T0IVIV08^;R`1BDU80Cdm?07moO`18B4P0@D5106e]K@13@d<0C4a<
02d];@0Q8B40EUIF05aLG00];Bd0820P04];B`1JFUX09bLW05YJFP12@T80=CDe06e]K@0Y:BT0>SXj
02TY:@0S8b<0>SXj02d];@0V9RH0?Shn01lO7`1QHF40Hf=S02TY:@0];Bd0EeMG02LW9`1QHF4092@T
028R8P0G5aL0Bd];05MGE`1IFET0<C4a02d];@0e=CD0BDU904Y:BP19BDT0F5QH04e=C@1PH600C4a<
02l_;`1oOgl0<C4a0410@01gMgL0HV9R04Q8B00l?3`0;Bd]02HV9P26QXH0LG5a01hN7P1PH600CDe=
059BDP0`<300@T9203`l?013@d<0<c<c05eMG@10@400GUiN07MgM`0X:2P0JVYZ05iNGP1bLW80S8b<
07MgM`1nOWh0E5AD05eMG@0f=SH0P82007=cL`0f=SH0EUIF05MGE`0_;bl0WinO0;no_`1GEeL0=SHf
06EUI@0X:2P0ATI6059BDP22PX80@41004m?C`1kNg/0EUIF075aL@1XJ6P0MWIf08Z:RP1gMgL0_;bl
05YJFP1YJFT0M7Ad06ATI01KFe/0M7Ad0080FEUI09/0EUIF06m_K`1UIFD0RHV907QhN01VIVH0S8b<
08:2PP1[Jf/0?3`l08>3P`1_Kfl0>c/k09FEU@2DU9@0?Cdm092@T01bLW80@d=309JFUP1TI6@0B4Q8
06YZJP11@D40>c/k08n?S`1LG5`0:b/[07UiN@1:BTX0Zj^[08N7Q`1EEED0DU9B03Ph>020P800WinO
0820P01>CTh0NGUi08B4Q01HF5P0=cLg06MWI`2MWId0GUiN08J6QP23Ph<0B4Q80<;2`P2HV9P0Jf][
07=cL`1eMGD0:b/[0410@00N7Qh09bLW05EEE@1:BTX0K6a/034a<@1^KVh0IFEU028R8P1cLg<0NGUi
069RHP2CTi<0EUIF02d];@1cLg<0CTi>07]kN`17AdL0=cLg09VIV@16ATH0EUIF06m_K`1gMgL0Ph>3
05=CD`1XJ6P0C4a<06m_K`0[:b/0E5AD03hn?P11@D40SHf=05mOG`1<C4`0Q8B403De=@19BDT0Lg=c
08V9R@0o?cl0ADE506][J`11@D40<C4a04Y:BP1`L700De=C09:BTP1;Bd/08R8R05aLG01bLW80FUYJ
05eMG@16ATH0WIfM07YjNP1DE5@0?Shn06i^KP1WIfL07alO03/k>`1KFe/0=3@d02XZ:P1gMgL0E5AD
03/k>`2EUID0Q8B402HV9P15ADD0OGem04U9B@0k>c/0PX:204M7A`0O7al0Jf][04a<C02;Rh/0@D51
03Ti>@2DU9@0@D510861P@2HV9P0GemO01lO7`0R8R80C4a<05MGE`0U9BD0I6AT09JFUP0>ooooo`0L
08b<S01UIFD0RXZ:030`<00L71`0<S8b02LW9`0e=CD0G5aL03`l?016ATH0:2PX01PH600`<300820P
06UYJ@1:BTX07QhN01HF5P0I6AT0C4a<08R8R02ZZZX0aLG509FEU@1oOgl0SXj>0820P080>c/k01/0
D51@096AT@2^[Zh0TY:B0<O7a`39bLT0/;2`05mOG`0W9bL06ATI020P801PH600]kNg07inOP1NGUh0
OGem0;fm_@2LW9`0A4A40492@P0l?3`0?clo03Lg=`0O7al06QXJ01LG5`0D51@00P0E5AD04P0O7al0
<30`04m?C`0X:2P0IfMW0;>c/`32`/80cLg=09VIV@1gMgL0A4A403De=@0`<300A4A40:6QX@1KFe/0
E5AD010@40<04Q8B0200ADE506EUI@2:RXX0UYJF0=WIf@3EeMD0VYZJ0<_;b`2^[Zh0Xj>S06IVIP1Y
JFT0CTi>02DU9@1bLW80[jn_07IfMP1jNWX0Ph>308^;R`27QhL0Hf=S06ATI01GEeL0WIfM09^KV`1c
Lg<0Q8B4061PH013@d<0L71`01`L70<05ADE01H05QHF02HV9P0n?Sh0V9RH06ATI02FUYH0^;Rh0;^k
^`39bLT0ZZZZ09>CT`2VYZH0<S8b01XJ6P0I6AT0;2`/06YZJP17AdL0G5aL079bLP1oOgl0BDU90P2]
[Jd04@1mOGd0SHf=07alO01cLg<0Vi^K0<_;b`1QHF40@D51079bLP1NGUh0>c/k05=CD`1eMGD0FEUI
04I6AP1XJ6P06ATI00805ADE03X0<C4a04=3@`0M7Ad06QXJ05aLG01bLW80:BTY03Lg=`1;Bd/0VYZJ
0;no_`1`L700De=C051@D02OWil0DE5A03Ti>@24Q8@0Cdm?05=CD`2KVi/0eMGE0:BTY01jNWX0;2`/
09>CT`1]KFd0L71`0<30`01ADE40>3Ph01PH600E5AD06a/K01`L700Q8B40D51@069RHP2[Zj/0OGem
06YZJP1`L700>3Ph059BDP1_Kfl0`<300<c<c03Fe]H0g]kN0=WIf@2TY:@0DE5A04i>CP1QHF40U9BD
02d];@0T92@071`L0P0F5QH07@0G5aL061PH03hn?P21PH40NWYj04i>CP1CDe<0Xj>S096AT@0o?cl0
M7Ad0:FUY@30`<00_Kfm0:RXZ02FUYH0?clo08F5Q@2f][H0NWYj01dM7@0c<c<0WinO03/k>`0o?cl0
Z:RX0:>SX`0_;bl06a/K00805QHF01<06a/K02TY:@15ADD0SHf=0:6QX@2ZZZX0Z:RX06][J`0n?Sh0
;Bd]01lO7`0K6a/08B4Q01dM7@0S8b<0:BTY05eMG@14A4@0JFUY0080[Zj^00@0Y:BT08V9R@1BDU80
8b<S0P0N7Qh00`0U9BD0;bl_ooooo`0?ooooo`003_ooool08P1UIFD0S8b<05ADE01EEED0QhN704m?
C`12@T80NGUi03/k>`1OGel0OWin03Lg=`1BDU80GUiN034a<@14A4@0NWYj061PH00_;bl0G5aL06AT
I00m?Cd0LG5a02PX:015ADD0EEEE03Hf=P0n?Sh07AdM02TY:@1SHf<0;2`/06=SH`1bLW8203De=@2^
06IVIP0[:b/0GUiN02DU9@0^;Rh07AdM055AD@1GEeL0F5QH03`l?018B4P0Bd];05aLG01IFET0HV9R
05eMG@1?Cdl0F5QH04m?C`0n?Sh0OGem02l_;`0n?Sh0N7Qh065QH@14A4@0CDe=03Lg=`0Y:BT0V9RH
07emO@0Q8B40GemO05EEE@1IFET0ADE503Ph>016ATH0;bl_03`l?01CDe<0>SXj06EUI@1FEUH06a/K
07AdM01OGel0P8200:6QX@1bLW80H61P04m?C`1HF5P0@T9206=SH`18B4P092@T05=CD`1MGEd08B4Q
06e]K@2FUYH0NWYj04A4A01]KFd092@T05UIF@14A4@0N7Qh03<c<`1TI6@0JFUY04I6AP1]KFd0=CDe
07AdM01lO7`0HF5Q07alO018B4P0GEeM069RHP1ADE40Cdm?06a/K014A4@0DU9B05UIF@1OGel0AdM7
07IfMP1^KVh0ADE506EUI@1>CTh0FEUI03`l?022PX80NWYj02<S8`1^KVh0NWYj03Lg=`1aLG40L71`
03/k>`1eMGD0C4a<03Lg=`1JFUX0DU9B02TY:@19BDT0BTY:02<S8`1NGUh0;Bd]05YJFP1aLG40Cdm?
07AdM01GEeL0M7Ad079bLP1dM7@0IVIV08f=S@2JVYX0MWIf0492@P1XJ6P0Ti>C06e]K@2YZJT0Vi^K
04m?C`30`<00_;bl051@D02_[jl0MGEe03Ti>@14A4@05aLG01/K6`1HF5P0DE5A04M7A`0W9bL0ADE5
05IFEP0[:b/0E5AD06][J`1VIVH0[jn_06e]K@0e=CD0NGUi05]KF`2UYJD0EUIF03Ti>@38b<P0D51@
06QXJ024Q8@0[:b/0:f][@1ZJVX0QHF506m_K`1gMgL09BDU0P1fMWH0@P1ADE40IFEU03dm?@1LG5`0
U9BD06=SH`10@400CDe=07UiN@0g=cL0Kfm_09NGU`1lO7`0BDU904i>CP1mOGd0A4A40;:b/P1eMGD0
B4Q806][J`1eMGD0IfMW079bLP1@D500_Kfm0:ZZZP1FEUH07AdM04Q8B01EEED08b<S04A4A01MGEd0
9bLW01TI6@1?Cdl04Q8B01dM7@1DE5@0Rh^;05IFEP1BDU80R8R805ADE010@400/K6a075aL@0Y:BT0
GEeM0451@@2WYjL0Hf=S03`l?02MWId0>SXj065QH@2OWil0IVIV02DU9@0W9bL0GemO03dm?@0^;Rh0
SXj>09BDU0kooooo01T0Cdm?06e]K@1DE5@0K6a/01lO7`15ADD0EUIF07IfMP1<C4`0>3Ph02<S8`0G
5aL05ADE01`L700K6a/05ADE03@d=00R8R805aLG01HF5P0E5AD0CDe=08N7Q`3GemL0jNWY0080mOGe
01L0hn?S0:RXZ01lO7`0GEeM08b<S03EeMD0a/K607emO@14A4@0ADE508j>SP31`L40Xj>S0<;2`P3L
g=`0/k>c08N7Q`15ADD0<S8b04A4A02d];@0C4a<01lO7`02028R8P0l02PX:00N7Qh07QhN01LG5`0C
4a<04A4A01DE5@0G5aL06ATI06i^KP2>SXh0CDe=0;no_`33`l<0];Bd08V9R@1^KVh0GemO06a/K02Q
XJ40bl_;0=KFeP1VIVH0FEUI028R8P0e=CD0D51@06IVIP2IVIT0ZZZZ0;Ng]`3Bd]80i^KV0>CTi03M
gMd0^KVi04];B`1>CTh0Z:RX06i^KP1RHV80UIFE0;Bd]02b/[80QHF505YJFP1<C4`0Bd];04M7A`1J
FUX0UIFE04=3@`2KVi/0RXZ:0410@01IFET0QHF508Z:RP1jNWX092@T0P0E5AD0=`0G5aL06ATI03Lg
=`13@d<0TI6A06ATI01ZJVX0C4a<07]kN`3De=@0VIVI05MGE`2QXJ40Ogmo02h^;P0K6a/071`L024Q
8@1^KVh0NGUi08B4Q02e]KD0YJFU0:j^[P2m_Kd0QXJ60:FUY@28R8P0HF5Q04I6AP2VYZH0aLG507in
OP2<S8`08b<S01dM7@0V9RH0HF5Q071`L01VIVH0T92@05EEE@0G5aL06QXJ04e=C@0d=3@06QXJ01DE
5@1aLG40Lg=c034a<@0_;bl0>SXj08f=S@1KFe/00P1LG5`0;@0o?cl0Hf=S096AT@12@T80K6a/0=_K
f`3Hf=P0g=cL0<o?c`2EUID0S8b<07IfMP1oOgl0I6AT0;Vi^@3Ti>@0Shn?04m?C`0[:b/0<S8b05AD
E013@d<0KVi^06MWI`23Ph<0HV9R07IfMP1JFUX07QhN028R8P0U9BD0>CTi0:FUY@2b/[80Y:BT0:j^
[P24Q8@0UYJF0<S8b01iNGT0@d=30;Ng]`0^;Rh07alO01XJ6P0201HF5P0W01TI6@18B4P0H61P06IV
IP0e=CD0?Cdm06=SH`10@400B4Q807YjNP1ADE40E5AD07=cL`2^[Zh0Xj>S0;^k^`3GemL0dm?C0<W9
b@2<S8`07alO01/K6`1LG5`0De=C09jNWP2:RXX0TY:B08J6QP0e=CD06QXJ01TI6@0H61P0:b/[02`/
;013@d<0UiNG0;:b/P26QXH0<30`00807AdM01<08R8R02h^;P0N7Qh07AdM01hN7P0P8200=3@d069R
HP1<C4`0UiNG0820P02XZ:P0NWYj01lO7`0L71`07QhN024Q8@0X:2P0DU9B013ooooo000>ooooo`0l
08B4Q02>SXh0De=C04I6AP1^KVh0@T920410@01?Cdl0<C4a07UiN@1cLg<0>c/k04];B`1JFUX0?Shn
05YJFP1MGEd0CDe=02/[:`1MGEd0FUYJ03Xj>P1YJFT0>3Ph06MWI`1GEeL0@41003hn?P0c<c<0?Shn
05iNGP0X:2P0FEUI05YJFP0f=SH0Cdm?07AdM00Z:RX0HV9R03Lg=`1;Bd/09RHV05MGE`1FEUH0R8R8
05MGE`1IFET0Bd];04e=C@11@D40BDU904e=C@1TI6@0AdM7030`<00R8R80GemO02l_;`0b<S80HV9R
0P0d=3@0?01GEeL0;bl_02/[:`2<S8`0ADE502DU9@1[Jf/0CTi>05YJFP12@T80@d=306a/K00j>SX0
?3`l04Y:BP19BDT0MgMg05UIF@0U9BD0GEeM03hn?P1VIVH0U9BD07MgM`1JFUX0F5QH06YZJP1>CTh0
P82005EEE@0`<300Kfm_07IfMP1=CDd0Ph>30:f][@1cLg<0<c<c07EeM@0l?3`0CDe=03Ph>01jNWX0
?clo04];B`1?Cdl0DU9B079bLP1VIVH0C4a<08:2PP1MGEd0OWin05iNGP1KFe/0H61P051@D01MGEd0
O7al071`L080I6AT09X0KFe]06=SH`21PH40Fe]K03hn?P1mOGd0EUIF03Hf=P0X:2P0LW9b07IfMP0g
=cL0NWYj071`L00o?cl0VYZJ08Z:RP17AdL0R8R804A4A01CDe<0P820051@D012@T80GEeM059BDP0a
<C40O7al05ADE023Ph<0LW9b06][J`23Ph<0K6a/07]kN`2BTY80J6QX03Xj>P2FUYH0S8b<05mOG`15
ADD0IFEU08J6QP0j>SX0Jf][04a<C016ATH0alO70<g=c@1OGel0I6AT05iNGP15ADD0BTY:02h^;P1S
Hf<0Fe]K051@D01JFUX0?3`l07IfMP1_Kfl0=CDe06UYJ@0U9BD0>SXj096AT@1/K6`0=3@d08j>SP0i
>CT0MGEe06=SH`1:BTX0`L710:FUY@1mOGd0?clo05iNGP1kNg/0>c/k08b<S02j^[X0RXZ:01lO7`1:
BTX0Kfm_05]KF`2FUYH0:b/[034a<@1YJFT0=cLg04];B`1lO7`0Rh^;02`/;00h>3P0H61P05EEE@1<
C4`0P82008f=S@0_;bl0FUYJ05MGE`19BDT0NGUi08J6QP1;Bd/0HF5Q03`l?02f][H0a<C408>3P`0k
>c/0B4Q804M7A`0P8200E5AD0820P01[Jf/08b<S069RHP13@d<0;2`/05eMG@26QXH0E5AD05MGE`1]
KFd04a<C038b<P2g]kL0OGem02HV9P11@D40:2PX08j>SP1=CDd0IfMW0<;2`P1QHF40IfMW09bLW01<
C4`0?Shn04=3@`1XJ6P0BDU902d];@1YJFT0QXJ63_ooool02P2MWId0P820071`L01fMWH0PH610;:b
/P2YZJT0F5QH05EEE@0m?Cd301DE5@1802PX:00P82007alO07emO@1/K6`0?Shn03<c<`0T92@05QHF
04m?C`2n_[h0k^k^0?Gem@3WinL0clo?0<?3``2LW9`0PH6108>3P`2n_[h0[:b/02`/;00I6AT07AdM
034a<@11@D40d];B0>7Qh@3Ph>00d];B0:VYZ@0];Bd08R8R01LG5`1eMGD0Shn?024Q8@0I6AT05aLG
01DE5@0L71`06a/K01DE5@0F5QH05ADE01LG5`0V9RH0;2`/034a<@1KFe/0eMGE0<G5a@2[Zj/0`<30
0;Fe]@2[Zj/0OWin07UiN@2ATI40Xj>S0:f][@2IVIT0H61P06=SH`1;Bd/0?clo03Hf=P1SHf<0VIVI
0:6QX@34a<@0_Kfm0P3Kfm/0ZP2JVYX0JFUY03<c<`1IFET0Z:RX0820P02@T900d];B03Ph>00e=CD0
?clo04I6AP17AdL0>SXj02d];@1EEED0J6QX0<C4a01VIVH0=SHf034a<@1>CTh0Shn?08:2PP0_;bl0
7alO01LG5`0J6QX092@T03hn?P1>CTh0HF5Q08Z:RP1aLG409RHV02PX:02VYZH0XJ6Q02h^;P0m?Cd0
WYjN07emO@0Y:BT07QhN024Q8@0T92@0OGem0:f][@2^[Zh0f][J0=KFeP2DU9@0JFUY04i>CP0l?3`0
<S8b03`l?01@D500`L710<o?c`2HV9P07QhN04U9B@1IFET0ATI609ZJVP2?Shl0aLG50820P01hN7P0
Xj>S08N7Q`0V9RH05aLG01DE5@1VIVH0>3Ph028R8P1EEED0Lg=c02d];@0H61P07AdM03Hf=P1<C4`0
BTY:06m_K`1oOgl0@4100<W9b@3Lg=`0T92@05ADE02<S8`0EeMG0:f][@1GEeL0MgMg0=GEe@2c/k<0
];Bd0;Fe]@2@T900V9RH0:ZZZP3>c/h0MGEe07moO`1/K6`0NGUi04U9B@0N7Qh071`L01lO7`0S8b<0
<c<c07QhN03Fe]H0RHV9071`L017AdL0[Zj^0:JVYP2g]kL0PX:208n?S`1[Jf/0@T92020P800E5AD0
7alO04];B`1JFUX0De=C04E5A@1<C4`0@d=3020P800G5aL07alO05]KF`1nOWh0V9RH0;Rh^037alL0
a/K60:ZZZP2l_;`0];Bd09NGU`2k^k/0J6QX030`<01TI6@0VIVI065QH@0Y:BT0;Rh^06QXJ01bLW80
KVi^020P800J6QX08B4Q030`<00o?cl0MGEe09fMW@0`<3007QhN0P0M7Ad00`0O7al0=cLg02LW9`02
01dM7@0>01hN7P0V9RH0Jf][05eMG@1iNGT0Ng]k04I6AP22PX806a/K01`L700O7al0:2PX04=3@`2<
S8`@ooooo`003_ooool0o`1QHF40FEUI05ADE011@D40EeMG034a<@1?Cdl0QhN704I6AP1;Bd/0GUiN
02`/;019BDT0:b/[02d];@1IFET0C4a<02`/;00O7al0M7Ad06IVIP0g=cL0N7Qh02d];@1@D500DU9B
03Ti>@1;Bd/0EEEE02TY:@19BDT0:BTY059BDP0Y:BT061PH03Hf=P1PH6008b<S06i^KP0K6a/0?Cdm
01/K6`1>CTh0;Rh^05EEE@16ATH0CTi>04M7A`1/K6`0H61P05=CD`1KFe/0Kfm_06QXJ01PH600=cLg
06i^KP0n?Sh0?Cdm07moO`1OGel0ATI603lo?`0M7Ad08b<S07]kN`1ADE408B4Q051@D011@D40KFe]
04=3@`0o?cl0@d=303Ph>00l?3`0F5QH0492@P1aLG40EUIF01hN7P1QHF40Lg=c07AdM02MWId0?clo
03Xj>P0g=cL0C4a<030`<02NWYh0P82001hN7P0c<c<0E5AD04a<C02BTY80_;bl07=cL`0V9RH0L71`
03`l?00n?Sh0;Rh^07UiN@16ATH0DE5A05=CD`0W9bL0FEUI06=SH`1?Cdl0@41003dm?@1cLg<0FUYJ
06ATI00j>SX0ADE505aLG01YJFT0IFEU05=CD`1IFET0J6QX05]KF`1oOgl0E5AD038b<P1QHF40K6a/
05EEE@0];Bd0IfMW0861P@0Q8B40SXj>07YjNP0^;Rh0IfMW07QhN00e=CD0UYJF05ADE00W9bL0Bd];
05MGE`0`<300SHf=059BDP0b<S80D51@05iNGP2=SHd0PH6104=3@`1UIFD0EEEE06QXJ02@T900FUYJ
05aLG02IVIT0Ng]k055AD@1MGEd0VYZJ0;>c/`1@D500KFe]04M7A`0j>SX0Rh^;09nOW`1`L700Lg=c
051@D00f=SH0IFEU02DU9@0R8R80AdM705mOG`22PX80:RXZ04I6AP1BDU807alO06i^KP1JFUX0Kfm_
0:FUY@19BDT0<S8b09:BTP1DE5@0KVi^01TI6@0k>c/0VIVI04a<C01WIfL0BTY:08F5Q@21PH40;Rh^
03@d=00l?3`0Q8B403Lg=`1BDU80@d=303Ti>@2DU9@0J6QX0410@021PH40;bl_02`/;019BDT0Kfm_
03Ph>015ADD0Jf][02`/;00M7Ad0N7Qh092@T01EEED0VYZJ05EEE@0O7al0Lg=c0820P01SHf<0Jf][
02LW9`1nOWh0W9bL07MgM`0i>CT0GEeM03Ph>00D51@0<C4a05ADE00U9BD05ADE07]kN`2@T900=SHf
06QXJ01CDe<05QHF06m_K`005`2?Shl0?Cdm02/[:`2:RXX0HV9R02/[:`1XJ6P0E5AD08n?S`0G5aL0
EEEE0;6a/@1`L700KFe]09bLW00C4a<061PH04I6AP2>SXh0H61P028R8P1EEED0NWYj00kooooo00X0
/[:b0::RXP2JVYX0QXJ60<g=c@2FUYH0SHf=08b<S01ZJVX0AdM70P0E5AD00`0M7Ad0De=C0492@P02
04m?C`0B03/k>`1HF5P0IVIV05mOG`11@D40<C4a0;Zj^P3MgMd0gmoO0<W9b@1nOWh0D51@02`/;00b
<S80?Cdm0492@P2GUiL0FUYJ0P0E5AD05`0G5aL0>CTi0=7Ad@3QhN40emOG0<S8b02SXj<0LW9b01/K
6`0C4a<0Fe]K0<c<c02BTY80O7al075aL@1dM7@0Jf][06EUI@1EEED0:BTY01LG5`0I6AT092@T0080
6a/K00l08R8R0<;2`P3;bl/0UIFE0=CDe03Hf=P0clo?08V9R@1ADE40BDU904e=C@1XJ6P0NGUi0820
P00b<S800P0L71`04@0_;bl0@T9204I6AP2IVIT0Y:BT0;jn_P3Fe]H0c/k>0:f][@1>CTh0M7Ad06UY
J@1`L700^k^k06MWI`1nOWh0?3`l008071`L05T07AdM01lO7`0N7Qh09BDU04Y:BP2VYZH0fMWI09RH
V021PH40MgMg06ATI02XZ:P0OWin065QH@0T92@09RHV03Hf=P10@400GemO06i^KP1KFe/0P82005AD
E00[:b/06QXJ0451@@2ZZZX0KFe]02h^;P1EEED0`/;207moO`1<C4`0Fe]K05iNGP1dM7@0Q8B408Z:
RP3AdM40dm?C0<C4a02=SHd0GemO05QHF01?Cdl0CDe=05]KF`25QHD0YJFU0;Bd]01CDe<0BTY:06AT
I01lO7`0YZJV08Z:RP2UYJD0CTi>0492@P1bLW80XZ:R03Xj>P0E5AD05QHF04];B`0h>3P0Kfm_055A
D@0Y:BT05QHF01LG5`0F5QH06QXJ028R8P15ADD0PX:2065QH@1OGel0I6AT08Z:RP0];Bd09RHV0410
@01MGEd0PX:206IVIP34a<@0YJFU055AD@0204Q8B00802l_;`1=CDd0Ph>30;jn_P3IfMT0[Zj^06=S
H`1HF5P201`L700i01dM7@0Q8B40;2`/06UYJ@39bLT0/;2`0:^[Z`1PH600VIVI08B4Q02m_Kd0f=SH
055AD@0^;Rh0>c/k05MGE`1QHF40>CTi05mOG`1ADE40E5AD05QHF017AdL08b<S01`L700F5QH05ADE
01TI6@0V9RH0@d=30<O7a`2^[Zh0^k^k0;Rh^01jNWX0?Shn07EeM@1FEUH0IVIV0<30`02d];@0Vi^K
069RHP0O7al07AdM01hN7P0[:b/0De=C06MWI`1`L700AdM704Q8B01UIFD0QhN70:2PX02LW9`07alO
008071`L00@07QhN01dM7@0W9bL0;2`/0`0M7Ad03@0T92@0Fe]K08f=S@2ATI40FEUI02DU9@1LG5`0
>3Ph01`L700P8200<S8b07emO@1iNGT04?ooool000kooooo06d0JFUY05iNGP1:BTX0Bd];07UiN@0k
>c/0;Bd]06MWI`11@D40Hf=S03dm?@0`<300JFUY061PH00k>c/0>SXj04I6AP12@T808B4Q04i>CP0o
?cl0?3`l07inOP16ATH0K6a/02/[:`0^;Rh0De=C05eMG@0c<c<0;Bd]02LW9`17AdL0B4Q802<S8`0n
?Sh0DE5A02d];@1XJ6P0:b/[03/k>`0M7Ad0JFUY051@D01ZJVX0=SHf03lo?`1/K6`0IVIV04m?C`0R
8R80@d=305]KF`1@D500@d=30451@@1BDU809BDU04m?C`1bLW80EUIF03dm?@1;Bd/06ATI03Ti>@1n
OWh0GEeM02TY:@1UIFD09RHV065QH@1GEeL0ADE504m?C`0J6QX0>3Ph069RHP0l?3`0OWin05]KF`0E
5AD0BTY:061PH02;Rh/0^KVi096AT@1DE5@0;bl_04Y:BP0J6QX0Lg=c07IfMP0l?3`0D51@06m_K`0d
=3@0YjNW0<30`02=SHd0ATI606m_K`0Y:BT0HV9R03Lg=`24Q8@0=3@d06i^KP1cLg<0FEUI0080EEEE
00/0LW9b07UiN@1LG5`0KFe]04M7A`22PX80LW9b055AD@1GEeL0H61P03dm?@0207]kN`0E06e]K@10
@400T92@0820P01BDU80IVIV04U9B@1/K6`0AdM7075aL@2@T9007alO07]kN`29RHT0?Cdm07]kN`1l
O7`06QXJ07moO`1]KFd0?Shn0080LG5a08<06ATI055AD@1]KFd09RHV04m?C`12@T80a<C40::RXP1N
GUh0HV9R0410@01HF5P0^k^k08R8R01^KVh0XJ6Q079bLP0G5aL0@D5107alO02SXj<0EUIF09ZJVP2F
UYH0?Shn09NGU`2MWId0IVIV09:BTP1bLW809BDU05ADE00d=3@0<30`04m?C`1JFUX0VIVI04m?C`19
BDT0E5AD01lO7`1XJ6P0LG5a06m_K`38b<P0EUIF02DU9@21PH40Hf=S09BDU010@400?3`l0;Fe]@1G
EeL0HF5Q07IfMP2BTY80PH610492@P2@T900Jf][07MgM`0g=cL0De=C04U9B@0k>c/0JVYZ03dm?@0o
?cl0OWin06EUI@13@d<0D51@071`L00/;2`0BDU906EUI@1WIfL0ADE504M7A`1fMWH0EeMG0;bl_01R
HV80:b/[05aLG01lO7`0G5aL079bLP1KFe/0_Kfm09VIV@1TI6@08b<S08R8R01nOWh071`L03/k>`1N
GUh07alO01PH601@D500EEEE030`<01SHf<0L71`03lo?`0h>3P0GUiN04m?C`0];Bd0Xj>S03Ti>@0R
8R80MGEe07=cL`2EUID0<S8b034a<@2UYJD0?clo06EUI@22PX80>SXj01PH600P8200A4A405IFEP0Z
:RX0JVYZ08V9R@0>ooooo`0A04Y:BP2HV9P0[jn_09ZJVP2m_Kd0Q8B40<K6aP2b/[80WIfM096AT@18
B4P0?Shn038b<P10@400?3`l04E5A@0Z:RX00P0U9BD08P0R8R80<c<c04e=C@1nOWh0clo?0=oOg`3:
b/X0];Bd02LW9`0K6a/06QXJ024Q8@0[:b/09BDU04I6AP1fMWH07alO01@D500B4Q809bLW0<W9b@2h
^;P0a<C40;Bd]01nOWh0EEEE010@400=3@d0<30`055AD@0b<S80?Cdm03`l?00n?Sh302d];@0I02l_
;`0W9bL0;Bd]03<c<`0b<S80:2PX05=CD`2m_Kd0YjNW05aLG03Fe]H0gmoO08R8R01JFUX0F5QH05]K
F`17AdL0=CDe04e=C@1aLG40ZJVY06=SH`0e=CD0<S8b03lo?`02061PH0060<S8b02RXZ80S8b<09RH
V028R8P0QhN70P1TI6@01@1JFUX0P8200:f][@1FEUH0@d=300<07AdM024071`L01lO7`0R8R80SXj>
0<G5a@2^[Zh0ZZZZ08>3P`24Q8@0]KFe0=WIf@2@T900UYJF07QhN01YJFT0/K6a08N7Q`1iNGT0Ph>3
07QhN01<C4`0DU9B03`l?00M7Ad0>SXj09jNWP2OWil0CDe=03De=@1^KVh0`<300:FUY@1dM7@00`1X
J6P07`2/[:`0dM7A0::RXP2][Jd0UYJF08:2PP28R8P0KFe]03`l?00[:b/0<S8b0:FUY@23Ph<0LG5a
04m?C`1@D500Hf=S0;>c/`2YZJT0Jf][07QhN01`L700J6QX05ADE01PH600ATI602DU9@27QhL0KVi^
02d];@0N7Qh00P0G5aL201DE5@0901HF5P0G5aL05aLG03/k>`25QHD0TI6A03Lg=`1<C4`09BDU0080
8B4Q00`0GemO096AT@2e]KD0[jn_05UIF@1;Bd/0@D5102`/;00Y:BT0@d=304A4A016ATH20:f][@0M
05=CD`0L71`07alO028R8P0Y:BT0;2`/05mOG`2NWYh0<S8b01TI6@1JFUX0RXZ:06ATI03?cll0][Jf
0<O7a`11@D408B4Q01hN7P0S8b<0>SXj0;:b/P1RHV80ADE504i>CP1OGel0:BTY01DE5@0F5QH00P0E
5AD07`0G5aL07QhN03Hf=P1kNg/0DE5A08B4Q033`l<0dM7A06a/K02c/k<0VYZJ0::RXP33`l<0f][J
06i^KP0k>c/09RHV028R8P0S8b<0;Rh^04a<C02DU9@0L71`06m_K`1gMgL0KVi^08j>SP2VYZH0b/[:
04=3@`0N7Qh00P0O7al04`0M7Ad071`L038b<P0N7Qh07alO02LW9`0X:2P0PH610<W9b@2TY:@0;bl_
02h^;P0a<C40Ogmo02@T900[:b/0F5QH08^;R`0_;bl04?ooool000kooooo04L0MWIf09BDU01BDU80
?Cdm06e]K@12@T80@41005YJFP0e=CD0XZ:R06m_K`0^;Rh0>3Ph05IFEP0d=3@0Cdm?0410@00n?Sh0
9RHV05]KF`1FEUH0<30`05UIF@0n?Sh0RHV905=CD`0U9BD0@d=305=CD`0i>CT0Fe]K02XZ:P13@d<0
EEEE02LW9`18B4P0FEUI020P801@D5007AdM02LW9`0B4Q80=CDe03hn?P1mOGd0AdM702l_;`0i>CT0
EeMG051@D01IFET0EUIF04m?C`1EEED0D51@04Q8B01jNWX0>3Ph02/[:`1_Kfl0HV9R03Xj>P1ADE40
=cLg024Q8@1hN7P0Kfm_038b<P1WIfL0B4Q8065QH@0204];B`3=06EUI@0W9bL0>c/k05eMG@0d=3@0
R8R806][J`0c<c<0EeMG04U9B@1IFET0VIVI0861P@1oOgl0Shn?075aL@16ATH0KVi^04A4A00W9bL0
Cdm?06i^KP0X:2P0FUYJ07UiN@2BTY80AdM707QhN00B4Q80EeMG03/k>`1^KVh04a<C04=3@`1gMgL0
GUiN05eMG@0E5AD0EeMG09^KV`1bLW80Ng]k03/k>`16ATH0KVi^04a<C01OGel0EeMG02/[:`1UIFD0
Ti>C08B4Q00c<c<0FEUI06YZJP1LG5`0TI6A04Y:BP1ADE40BTY:08b<S02ATI407QhN05QHF01fMWH0
ADE508n?S`2KVi/0BDU908j>SP0`<300ATI606UYJ@1kNg/0A4A404m?C`1:BTX0Cdm?07QhN00X:2P0
OGem08n?S`1QHF40S8b<01PH600e=CD0QHF508j>SP22PX80/;2`09FEU@1@D500>SXj061PH02;Rh/0
GEeM0:6QX@2ZZZX0=SHf09^KV`2WYjL0FEUI04e=C@1QHF40;Bd]05aLG00l?3`0EeMG06EUI@0l?3`0
OGem04a<C01kNg/0PX:203De=@1ZJVX0?3`l06][J`2h^;P0R8R803<c<`25QHD0C4a<07QhN01[Jf/0
@D510;Vi^@1PH600KVi^0451@@1DE5@0G5aL04Q8B02CTi<0IfMW0861P@0k>c/0UYJF07AdM01IFET0
OWin05QHF013@d<0OWin065QH@0n?Sh0EEEE071`L00o?cl0RHV90:JVYP22PX80CDe=04Y:BP2;Rh/0
?clo09JFUP20P800GEeM06IVIP1]KFd0B4Q80:JVYP1TI6@0^k^k09JFUP1KFe/071`L04m?C`1UIFD0
7AdM03/k>`1TI6@0?Shn02DU9@1GEeL0;bl_02HV9P21PH40NWYj05ADE01CDe<0K6a/02LW9`0d=3@0
^;Rh05=CD`0N7Qh0IfMW06QXJ02m_Kd0DE5A03/k>`2YZJT0C4a<059BDP2;Rh/0Bd];02<S8`0Y:BT0
CDe=04m?C`0Y:BT0Hf=S08>3P`0>ooooo`0A04];B`1dM7@0UYJF0;Zj^P3<c<`0TY:B07EeM@31`L40
_kno09^KV`1GEeL0AdM702h^;P0[:b/06ATI020P800K6a/00P0M7Ad09@0L71`07AdM02HV9P1=CDd0
^k^k0<o?c`3>c/h0a<C4071`L01CDe<0DU9B01`L700K6a/0;bl_02DU9@19BDT0B4Q801LG5`0H61P0
@D510<[:bP2?Shl0][Jf0<g=c@2EUID0820P014A4@0D51@0@D5101@D500@41005ADE01<C4`0E5AD0
410@018B4P0C4a<00`0B4Q8201LG5`1201/K6`14A4@0[:b/05=CD`1VIVH0`/;20;:b/P0f=SH0:2PX
03dm?@1>CTh0C4a<04A4A010@400GEeM0;Fe]@2o_kl0IfMW05YJFP1=CDd0?clo08^;R`16ATH06QXJ
03/k>`1QHF40ZZZZ0:RXZ01XJ6P092@T01HF5P0G5aL0E5AD0:FUY@1QHF408B4Q01hN7P0M7Ad06a/K
028R8P1TI6@0]KFe06YZJP1LG5`0FUYJ08N7Q`1oOgl0P82008f=S@2h^;P0`<300<O7a`2DU9@0a/K6
07UiN@1/K6`0V9RH0:RXZ01:BTX0@T9208F5Q@14A4@0=cLg05QHF02WYjL0Y:BT0P1IFET0902RXZ80
>3Ph03dm?@16ATH0>c/k02HV9P2<S8`0alO70:f][@2f][H0ZZZZ08f=S@1BDU80=3@d01`L700^;Rh0
Bd];08b<S02FUYH0Ph>302LW9`0L71`06QXJ034a<@2@T900Ph>3071`L01QHF40@T9203lo?`11@D40
CDe=09^KV`2IVIT0>SXj01/K6`L05ADE05H051@D01LG5`0_;bl0WinO09bLW01IFET0@D51055AD@0`
<300?clo0<?3``36a/H0HF5Q07=cL`29RHT0Ti>C07alO01>CTh0DU9B0:6QX@2NWYh0M7Ad06a/K00f
=SH07AdM028R8P0Z:RX0;Bd]0410@026QXH071`L01DE5@0F5QH061PH07=cL`1fMWH0^k^k0;Bd]03I
fMT0WYjN05ADE00n?Sh0@T9205IFEP2l_;`0MGEe04];B`1PH6008b<S01LG5`0E5AD061PH01LG5`0I
6AT05QHF024Q8@1/K6`0FEUI07MgM`24Q8@0/[:b0;Jf]P1jNWX0cLg=0<[:bP37alL0bLW90<S8b02g
]kL0>c/k02XZ:P0g=cL0EUIF05aLG00o?cl0;Rh^03`l?01IFET0P8200;6a/@2l_;`0e=CD0<G5a@2M
WId0:RXZ01hN7P807AdM01808b<S03@d=00X:2P08B4Q02d];@1EEED0ZJVY0<S8b02c/k<0De=C03Lg
=`0c<c<0I6AT05]KF`0m?Cd0Y:BT04Q8B00X:2P@ooooo`003_ooool0301SHf<0MGEe05ADE01@D500
JVYZ03<c<`13@d<0L71`03Lg=`1FEUH0K6a/03/k>`80CDe=02h092@T05ADE01?Cdl0>3Ph01lO7`1=
CDd0KFe]03Xj>P1_Kfl0:b/[06EUI@1fMWH0>3Ph04E5A@1GEeL0;Bd]05EEE@12@T80AdM7038b<P0G
5aL0?Cdm079bLP0Y:BT0LW9b01@D5017AdL08B4Q04];B`0d=3@0MGEe06EUI@0e=CD0B4Q805UIF@1:
BTX0H61P059BDP1^KVh0FEUI0451@@0k>c/0I6AT0410@00k>c/0JFUY0P0^;Rh0?`1LG5`0AdM702PX
:026QXH0EeMG028R8P1hN7P0DE5A07EeM@1LG5`0>c/k06ATI00o?cl0EUIF06IVIP0^;Rh0KFe]07If
MP0f=SH0BTY:06YZJP1MGEd0Y:BT07IfMP1^KVh0CDe=05IFEP15ADD0JFUY04i>CP0T92@0Hf=S0820
P01DE5@0/;2`0;:b/P1^KVh0;Bd]08F5Q@1:BTX0?clo02`/;021PH40CDe=03dm?@1GEeL0=SHf069R
HP1YJFT0CTi>05QHF018B4P0N7Qh05aLG01KFe/0Cdm?051@D022PX80MGEe059BDP1>CTh0EeMG07em
O@0205aLG01Y05UIF@18B4P0K6a/05YJFP0i>CT071`L069RHP27QhL0>SXj06m_K`1iNGT0:2PX07Yj
NP2FUYH0?Shn07QhN01MGEd0<30`051@D01JFUX0B4Q805QHF018B4P09BDU05eMG@1ADE40VIVI07al
O01EEED0TY:B07=cL`1=CDd0Q8B4092@T014A4@0TY:B09NGU`1^KVh0=cLg061PH02CTi<0BDU907If
MP2@T900=3@d08Z:RP2WYjL0LW9b06IVIP23Ph<0>3Ph059BDP0];Bd0>SXj06a/K016ATH0L71`03De
=@1>CTh0IfMW02`/;01KFe/0AdM706IVIP2HV9P0HV9R03@d=01oOgl0Cdm?07QhN01FEUH0>c/k09nO
W`1hN7P0P820065QH@1bLW80K6a/05QHF02PX:00QXJ608V9R@0c<c<0EeMG06MWI`1>CTh0P82004e=
C@1CDe<0UYJF07=cL`17AdL0EUIF07moO`0b<S80AdM706IVIP1QHF40CDe=05iNGP2:RXX0GEeM07em
O@25QHD0KFe]0080NGUi02`0AdM706YZJP1=CDd0[Jf]09^KV`1QHF40?clo06MWI`1JFUX07QhN04I6
AP1[Jf/0IFEU02XZ:P1`L700Kfm_02h^;P1/K6`0RXZ:04m?C`1ADE40N7Qh04M7A`0[:b/0WYjN05aL
G00S8b<0JVYZ06a/K035aLD0>3Ph04Q8B02b/[80IFEU04Q8B02LW9`0=3@d020P800^;Rh0EUIF05QH
F00T92@0G5aL08R8R0kooooo0100CTi>04Q8B01FEUH0OGem0:NWY`1bLW80QXJ60<30`02h^;P0Zj^[
07IfMP19BDT0@d=302/[:`0R8R809bLW0P0[:b/08@0X:2P0:b/[038b<P16ATH0X:2P0:VYZ@2j^[X0
bLW90::RXP1<C4`0De=C079bLP1ADE40:RXZ028R8P17AdL0M7Ad0820P014A4@0BTY:05UIF@2^[Zh0
TY:B0;no_`36a/H0XJ6Q04=3@`0g=cL0DU9B04Q8B00F5QH0410@018B4P0201<C4`0500h>3P0A4A40
4a<C00l?3`0>3Ph00`0?3`l0:P0>3Ph05QHF04e=C@0N7Qh0@d=309NGU`1kNg/04a<C010@400?3`l0
4Q8B01DE5@0H61P04A4A00h>3P0Z:RX0NWYj0:NWY`2c/k<0_[jn09BDU00`<3004a<C014A4@0B4Q80
5aLG02XZ:P1cLg<0EeMG04Q8B016ATH08B4Q01PH601EEED0V9RH06e]K@0e=CD0De=C07UiN@2XZ:P0
[jn_08>3P`80ATI601L0@D5104=3@`1EEED0FEUI06a/K01jNWX0UiNG09RHV02j^[X0aLG508f=S@0m
?Cd0=CDe03/k>`1/K6`0WIfM07MgM`1IFET0F5QH08J6QP2<S8`0c<c<09jNWP0208R8R00S05]KF`0_
;bl0820P020P800/;2`0Xj>S06EUI@1FEUH0WinO06UYJ@1?Cdl0J6QX08n?S`1eMGD0=cLg02`/;00c
<c<0>CTi05aLG01XJ6P0@d=302TY:@0Q8B40D51@03hn?P1eMGD0FUYJ03`l?01@D500S8b<0;no_`2k
^k/0RXZ:05mOG`0M7Ad01@0E5AD04@0F5QH05ADE01DE5@0E5AD0820P07EeM@38b<P0d];B0=kNgP3P
h>00f][J0=kNgP3Lg=`0e]KF0861P@1<C4`0;Bd]00809bLW00h0@41007QhN033`l<0YZJV03`l?00Q
8B40@D5102d];@0Y:BT0<30`07EeM@1gMgL09BDU01HF5P805ADE01405aLG02`/;037alL0bLW90<K6
aP23Ph<0DU9B05UIF@1EEED0G5aL06IVIP1iNGT0YjNW092@T014A4@06ATI01HF5P0201LG5`0901PH
600E5AD061PH02/[:`1YJFT0P82007emO@21PH40TI6A0080C4a<00<0HV9R07]kN`2_[jl00P2FUYH0
9P2EUID0Jf][065QH@1IFET0ATI6034a<@0d=3@0?3`l04M7A`1QHF40Ng]k0;fm_@3Hf=P0emOG0;^k
^`2@T900TI6A03dm?@0Q8B4092@T02/[:`17AdL0G5aL07QhN02JVYX0[jn_0:b/[02RXZ80Z:RX07Yj
NP23Ph<0SXj>09bLW03<c<`0[Zj^08b<S01:BTX0LG5a4?ooool000kooooo00X0I6AT04E5A@17AdL0
CTi>07QhN00n?Sh0?3`l08R8R013@d<0EUIF0P0X:2P0Y`1gMgL0NGUi03Xj>P1;Bd/0Fe]K075aL@0f
=SH0BTY:05YJFP0m?Cd0UiNG03Ph>01GEeL0@T9203<c<`1QHF40Jf][02l_;`0P8200;Rh^06IVIP1N
GUh0;Rh^03dm?@1BDU807QhN08J6QP0g=cL0?Cdm02<S8`1YJFT0DU9B07=cL`12@T80=SHf07emO@27
QhL0De=C02l_;`0i>CT0Fe]K08F5Q@1SHf<0C4a<05=CD`0G5aL0D51@07UiN@1PH600?clo05]KF`0_
;bl0?Cdm07IfMP1bLW80<c<c08>3P`0C4a<0EUIF075aL@1=CDd0DE5A02DU9@0Y:BT0HF5Q03Lg=`1_
Kfl0KVi^028R8P0a<C40MWIf05]KF`2QXJ40Ph>309>CT`1eMGD0AdM703/k>`1lO7`0HF5Q01TI6@1;
Bd/0J6QX03Xj>P2j^[X0^KVi03Ph>00S8b<0IFEU02h^;P12@T80;bl_08F5Q@1<C4`0@410065QH@0m
?Cd0HF5Q07AdM014A4@0RXZ:06=SH`1WIfL0GEeM055AD@1eMGD0CTi>06UYJ@1gMgL0D51@04Y:BP1A
DE40QHF5059BDP1PH600KVi^065QH@1oOgl0IFEU04U9B@0j>SX0FEUI096AT@0h>3P0N7Qh09:BTP14
A4@0O7al0:FUY@14A4@0Ogmo07]kN`10@400Kfm_08V9R@0o?cl0Ng]k07AdM00h>3P0KFe]05eMG@20
P800J6QX06EUI@1HF5P0I6AT04i>CP2HV9P0UiNG07MgM`2JVYX0XJ6Q05UIF@1ADE40QhN709VIV@1S
Hf<0][Jf09RHV00g=cL0NWYj0:ZZZP1CDe<0QXJ608^;R`0o?cl0GUiN0080>3Ph0640EEEE04E5A@27
QhL0GemO055AD@1SHf<0;2`/06m_K`1JFUX0RXZ:0<G5a@1LG5`0<c<c08R8R01CDe<0ZZZZ05MGE`0i
>CT0RHV907EeM@1aLG40MWIf0:RXZ01nOWh0<c<c07MgM`1NGUh0LW9b04A4A01mOGd0B4Q804=3@`2L
W9`0O7al06MWI`2QXJ40De=C03hn?P1OGel0KVi^03Hf=P1cLg<0Ogmo06MWI`1IFET0GEeM07IfMP1U
IFD0LW9b06e]K@0l?3`0S8b<09JFUP1OGel0SXj>05iNGP2OWil0W9bL06UYJ@0N7Qh0GUiN05mOG`0N
7Qh0>c/k05EEE@0P82008B4Q05mOG`1UIFD0<c<c06IVIP1>CTh061PH03dm?@1^KVh0EeMG030`<02N
WYh0>CTi01TI6@1ZJVX0AdM70<g=c@1;Bd/0<C4a09fMW@23Ph<0=SHf08Z:RP1JFUX0:b/[01lO7`0h
>3P0C4a<02PX:01>CTh0R8R800kooooo0100=cLg02d];@0Y:BT0EEEE08j>SP0];Bd09RHV05aLG01V
IVH0O7al0451@@0P82004Q8B010@400>3Ph0820P100A4A409`0D51@0<C4a0:2PX01CDe<0N7Qh07Ee
M@1]KFd07AdM01LG5`0Z:RX07alO02`/;00c<c<0GUiN06ATI011@D40=3@d02l_;`0];Bd0W9bL0:j^
[P1^KVh0bl_;08F5Q@0S8b<06ATI01@D500O7al0=cLg010@400=3@d0410@01<C4`0=3@d03`l?00h>
3P0?3`l0410@00l?3`0200h>3P0;00d=3@0G5aL0@41001/K6`10@400Shn?07]kN`0H61P03Ph>01<C
4`0A4A400`0@41002P0=3@d03Ph>03Ti>@1lO7`0C4a<0:>SX`1DE5@04A4A00h>3P0A4A4200h>3P0i
01TI6@0W9bL0>SXj01DE5@0];Bd0MGEe04i>CP0h>3P0DU9B0=OGe`3Hf=P0_;bl09ZJVP18B4P0A4A4
05QHF00N7Qh06ATI01/K6`0G5aL05ADE03/k>`1^KVh0/[:b0;bl_02g]kL0d];B0<g=c@21PH40ATI6
02PX:00V9RH0?Shn06ATI01XJ6P0ADE502LW9`1mOGd0Kfm_0:BTY036a/H0^;Rh07inOP1fMWH0H61P
0451@@0c<c<0?clo06EUI@1TI6@0W9bL06e]K@1JFUX08b<S024Q8@1bLW80GemO00805aLG01<05ADE
01PH601CDe<0Lg=c07alO01DE5@0>c/k079bLP1IFET0NGUi0<K6aP1^KVh0IfMW0:b/[03;bl/0_kno
07MgM`13@d<0ADE500<05ADE00<051@D01DE5@0E5AD00P0E5AD0700G5aL0@d=30::RXP3=cLd0e=CD
0;2`/018B4P0HF5Q0>3Ph03Shn<0[:b/05iNGP1?Cdl0Bd];03Hf=P0L71`05aLG03Ph>01TI6@0GemO
03/k>`0U9BD0>SXj07inOP11@D40SXj>05EEE@0F5QH401DE5@0A01LG5`0h>3P0/K6a0;bl_02IVIT0
MGEe04U9B@1BDU80SHf=07EeM@17AdL0:BTY030`<020P800JFUY03dm?@0D51@01@0E5AD0<`0P8200
EeMG07moO`1hN7P0Q8B406=SH`13@d<0;Rh^03Xj>P1@D500Ti>C065QH@32`/80[Zj^0;Fe]@1TI6@0
HV9R06EUI@25QHD0WinO079bLP11@D40<c<c0:j^[P2o_kl0/;2`0:b/[02ATI40];Bd0;:b/P3Bd]80
Y:BT09fMW@2i^KT0ZZZZ09BDU01jNWX0[:b/0=SHf02k^k/0b/[:05=CD`0V9RH0<30`03Ph>01MGEd0
]kNg0:ZZZP37alL0K6a/055AD@0@ooooo`003_ooool0NP1QHF40IVIV04M7A`17AdL0QhN704M7A`13
@d<0NWYj038b<P1aLG40F5QH02LW9`0l?3`0DE5A02<S8`0/;2`0;Rh^04e=C@0V9RH0B4Q802h^;P0F
5QH0G5aL02LW9`0o?cl0BDU902<S8`0`<300DE5A02HV9P0K6a/07alO03/k>`1EEED0;bl_03De=@1J
FUX08B4Q05IFEP0Q8B40<30`01<C4`1CDe<0BTY:07emO@0L71`04a<C03Ph>01LG5`0CDe=02`/;01?
Cdl0D51@07=cL`1KFe/0B4Q807MgM`0l?3`092@T05QHF018B4P0>c/k05UIF@0R8R807alO079bLP1i
NGT0=SHf07MgM`12@T80O7al06YZJP0n?Sh0CTi>04Q8B00i>CT0M7Ad02h^;P1XJ6P0KVi^03lo?`0m
?Cd0NWYj069RHP2e]KD0Rh^;08F5Q@2[Zj/0GemO0410@01_Kfl0@410028R8P24Q8@0VYZJ03dm?@2P
X:00a<C403/k>`0`<300PH6103Lg=`1DE5@0=cLg096AT@1EEED0EeMG06YZJP1@D500L71`07alO01>
CTh0V9RH06MWI`1cLg<0GemO06m_K`26QXH0D51@071`L01mOGd0CTi>0P1LG5`0N@2SXj<0DE5A07Qh
N01LG5`0DE5A092@T01CDe<0EEEE03hn?P1fMWH0];Bd0492@P1kNg/0M7Ad03lo?`2<S8`0Xj>S03<c
<`1fMWH0?Shn02d];@1;Bd/0F5QH04=3@`1KFe/0BTY:0492@P24Q8@0F5QH08:2PP23Ph<0MWIf08R8
R01TI6@0BTY:0:^[Z`2NWYh0?Cdm09RHV02UYJD0Lg=c03Hf=P1HF5P0Vi^K0492@P1^KVh0X:2P0451
@@1hN7P0TI6A05mOG`18B4P0J6QX02`/;01<C4`0<S8b06=SH`1hN7P0@T9207QhN00l?3`0HV9R07Ad
M00j>SX0J6QX01HF5P0/;2`0V9RH07YjNP0f=SH0SXj>04A4A01cLg<0Fe]K03Ti>@25QHD0OGem07Ee
M@1;Bd/0J6QX07=cL`1;Bd/0U9BD07IfMP25QHD08R8R05eMG@1hN7P0Cdm?08F5Q@0`<300=SHf07Yj
NP1WIfL0>CTi04m?C`1NGUh09BDU04=3@`17AdL0Fe]K051@D01MGEd0Kfm_030`<01iNGT0Jf][04m?
C`2;Rh/0Q8B4034a<@1gMgL0?3`l09JFUP1mOGd0FEUI03Ph>01EEED0I6AT020P800m?Cd00P1NGUh0
7`0^;Rh0JFUY05UIF@0Z:RX0Hf=S04];B`0k>c/0De=C08^;R`1bLW80;bl_0;Fe]@1eMGD0:RXZ07]k
N`1>CTh0d=3@08^;R`17AdL0UIFE08J6QP0k>c/0T92@06UYJ@0`<300:BTY0451@@1MGEd0:BTY059B
DP2=SHd03_ooool0400e=CD0:RXZ02HV9P2RXZ80b/[:069RHP0Q8B40@41004a<C01FEUH0;Bd]01LG
5`0=3@d02PX:00d=3@0H61P300h>3P09014A4@0>3Ph05aLG05mOG`0g=cL0GEeM02h^;P1PH6005aLG
00803Ph>0P0@41005@0C4a<0CTi>02PX:00M7Ad0:RXZ02/[:`0d=3@0T92@0;Rh^01]KFd0bLW90;^k
^`2CTi<0CTi>02d];@0I6AT08b<S01TI6@0=3@d03Ph>00l?3`0500h>3P0<014A4@0=3@d03@d=00h>
3P0@4100B4Q802XZ:P17AdL0W9bL07AdM00E5AD03`l?0P0@41002P0>3Ph04A4A00h>3P0=3@d0410@
02/[:`1ZJVX09bLW06m_K`1FEUH3014A4@803Ph>02004Q8B02HV9P0b<S804a<C01DE5@0R8R80QHF5
0;6a/@2@T900g]kN0>;RhP29RHT0DE5A04];B`0h>3P08b<S03Hf=P0O7al061PH01`L700^;Rh0>CTi
03<c<`1@D500ZZZZ0:6QX@2OWil0/k>c07=cL`1>CTh0F5QH03dm?@80EUIF01P0DE5A04A4A01XJ6P0
[Jf]0:6QX@2RXZ80YJFU0:2PX024Q8@0Ng]k03<c<`0M7Ad071`L01dM7@1?Cdl0AdM706i^KP2[Zj/0
Ng]k02`/;00X:2P0@d=307IfMP0I6AT201DE5@0C01PH600M7Ad0@d=307moO`29RHT0D51@05mOG`1i
NGT0Zj^[08>3P`15ADD0>CTi05eMG@2DU9@0a/K608>3P`0d=3@0B4Q803Ti>@0401DE5@0H014A4@0C
4a<0:RXZ055AD@11@D40Ng]k0<c<c03EeMD0`<3006=SH`2@T900`/;20<W9b@37alL0DE5A03/k>`0e
=CD0;bl_03dm?@1<C4`0UYJF05mOG`1<C4`0EeMG0P0_;bl0101OGel0VIVI0820P00C4a<501DE5@0A
01XJ6P1IFET0SXj>0;Bd]02b/[80^[Zj06ATI01bLW80W9bL03<c<`0T92@08B4Q02DU9@19BDT0LW9b
06QXJ00Z:RX00`0E5AD0800D51@05aLG02@T9014A4@0IfMW06UYJ@1gMgL0=cLg03<c<`0j>SX0=CDe
06YZJP25QHD0Y:BT0>7Qh@3Fe]H0W9bL03De=@1eMGD0SXj>06UYJ@11@D408b<S02/[:`2][Jd0ZJVY
06UYJ@1JFUX0PX:209>CT`2VYZH0/k>c0P3Bd]801`3=cLd0_Kfm0:BTY01WIfL0N7Qh0;Fe]@3Lg=`0
0P3De=@02P1HF5P0@410061PH02SXj<0]KFe0=kNgP3Shn<0hN7Q0<?3``2o_kl@ooooo`003_ooool0
^@1QHF40O7al04i>CP1;Bd/0NGUi03hn?P1ADE40UIFE04M7A`1:BTX0LG5a04A4A01;Bd/0KFe]02XZ
:P18B4P0Cdm?04];B`0Z:RX0A4A405ADE00a<C40EUIF02LW9`0i>CT0L71`0451@@1=CDd0DU9B038b
<P0g=cL0:RXZ04E5A@1EEED0;Bd]04I6AP1PH60092@T075aL@0T92@0?clo02h^;P1aLG40EEEE09^K
V`1NGUh09bLW04E5A@1/K6`0B4Q804U9B@1`L700PX:2075aL@14A4@0>CTi05iNGP13@d<0?3`l06YZ
JP0i>CT09BDU04Y:BP0a<C40CTi>09nOW`1`L700:BTY07MgM`17AdL0RHV906MWI`0i>CT0Kfm_051@
D01<C4`0OGem03/k>`27QhL0N7Qh0492@P1=CDd0Lg=c0410@02^[Zh0JFUY05UIF@24Q8@0Hf=S03lo
?`20P800Cdm?014A4@0e=CD0JFUY055AD@2WYjL0/k>c04e=C@0R8R80IfMW04m?C`11@D407QhN079b
LP1@D500AdM705ADE00];Bd0Jf][06e]K@1;Bd/0G5aL03lo?`1iNGT0GUiN05MGE`1?Cdl0<30`061P
H01mOGd0De=C04E5A@0d=3@0U9BD069RHP1IFET0ATI602d];@1TI6@0Hf=S03lo?`0O7al0?3`l07Yj
NP0];Bd0Jf][0820P00g=cL0FEUI09VIV@0l?3`0KFe]06m_K`0j>SX0F5QH0:2PX01QHF40G5aL04I6
AP0];Bd0IFEU065QH@2<S8`0Kfm_04I6AP1eMGD0EeMG04I6AP2RXZ80QXJ606YZJP2NWYh0Y:BT06a/
K00m?Cd0JVYZ0;6a/@1DE5@0LW9b08^;R`10@400V9RH0<_;b`1/K6`0Hf=S092@T00^;Rh0AdM7038b
<P0`<300CTi>04A4A027QhL0>3Ph0080AdM700h0;Bd]061PH00o?cl0JVYZ0;:b/P1QHF40:RXZ0861
P@1EEED0TI6A05ADE00m?Cd0Ph>308R8R080O7al01H0OGem0861P@1?Cdl0YZJV08:2PP28R8P0?Shn
065QH@1ADE40C4a<08N7Q`1VIVH0D51@09fMW@1eMGD0?clo04Y:BP1PH600>3Ph04i>CP17AdL0IVIV
0P1GEeL0<`1RHV80Hf=S07YjNP28R8P0@D5108R8R025QHD0?clo04E5A@0U9BD0WIfM07EeM@0o?cl0
?3`l05ADE01RHV80820P059BDP1hN7P0IfMW03Lg=`1NGUh0EUIF02LW9`25QHD0MWIf04m?C`1CDe<0
OGem02/[:`0W9bL0Zj^[07AdM00[:b/0HV9R02`/;02ZZZX0P82005]KF`2l_;`0Rh^;038b<P2=SHd0
BDU903lo?`1TI6@0NWYj04E5A@0a<C40OWin08f=S@0>ooooo`0<034a<@0Z:RX0<S8b07YjNP3=cLd0
c/k>07moO`0n?Sh0FUYJ05IFEP0P82005ADE100=3@d00`0<30`03Ph>00h>3P0200h>3P0N014A4@1=
CDd0>CTi096AT@0_;bl0:BTY01DE5@0@41003Ph>00d=3@0>3Ph07alO02@T900E5AD04A4A01<C4`0@
41004A4A02d];@2[Zj/0f=SH0;Rh^028R8P0K6a/07=cL`1^KVh0EeMG02d];@0R8R803@d=0P0>3Ph0
0`0=3@d03Ph>00h>3P0200l?3`0@014A4@0>3Ph03Ph>00h>3P0?3`l0<30`0492@P1hN7P0clo?0<K6
aP1<C4`03`l?00h>3P0=3@d03`l?00h>3P803@d=00H03`l?01lO7`1UIFD09RHV04=3@`1OGel2014A
4@1C00h>3P0=3@d0410@01<C4`0R8R80=3@d00h>3P0@41004A4A01`L702b/[80hn?S0=_Kf`2ZZZX0
_[jn04Q8B00O7al05aLG01PH600K6a/0;Rh^01TI6@0K6a/0<S8b02DU9@0V9RH0;Bd]061PH00k>c/0
IFEU05YJFP22PX80=CDe04=3@`1HF5P0L71`04a<C016ATH0J6QX07YjNP1LG5`0NGUi0;Fe]@37alL0
SHf=071`L01ADE40?Cdm01LG5`0E5AD05aLG04m?C`0k>c/0AdM707emO@2=SHd0;bl_02LW9`0d=3@0
MGEe02XZ:P0I6AT08b<S01dM7@0K6a/071`L03De=@2BTY80^[Zj09JFUP2OWil0O7al03Hf=P17AdL0
D51@09jNWP2QXJ40IFEU07EeM@1QHF40@41004];B`0V9RH00P0E5AD01`0F5QH09bLW0451@@0l?3`0
820P04m?C`23Ph<00P2c/k<02@15ADD0>3Ph05]KF`3?cll0`L710<O7a`1_Kfl0F5QH06i^KP0207If
MP0;05]KF`28R8P0/K6a07inOP1FEUH0ATI603<c<`1NGUh0_kno03Xj>P0@41000P0C4a<02@0E5AD0
5QHF01lO7`0k>c/0RXZ:07inOP2TY:@0Ogmo06e]K@0201/K6`1105eMG@0h>3P08b<S01lO7`0U9BD0
9RHV05UIF@1TI6@0F5QH02LW9`0E5AD051@D01DE5@0M7Ad0@T9204];B`1?Cdl0Hf=S03Ph>00d=3@0
Rh^;06ATI020P800ZJVY0<_;b`2h^;P0[Zj^0:^[Z`2o_kl0MWIf05=CD`0V9RH071`L01/K6`0L71`0
K6a/08V9R@1=CDd0:BTY034a<@22PX80K6a/0:j^[P2h^;P0f=SH0=KFeP3AdM40g=cL0=WIf@2g]kL0
c<c<0=7Ad@3Ogml0fm_K0:VYZ@28R8P0UIFE0;Ng]`2m_Kd0O7al08B4Q03Kfm/0f=SH07MgM`1aLG40
4?ooool000kooooo0>00GemO04m?C`19BDT0A4A4079bLP0g=cL0>c/k08B4Q014A4@0F5QH02XZ:P0g
=cL0OWin06m_K`0U9BD0BDU904Q8B01BDU8092@T05UIF@19BDT0<c<c07YjNP0`<300?Shn03Hf=P0]
;Bd0H61P065QH@0a<C408R8R024Q8@1UIFD0DE5A01hN7P0_;bl0BDU901XJ6P1dM7@08B4Q02h^;P0Y
:BT0JFUY02l_;`2KVi/0>CTi02`/;01PH600GemO0451@@0N7Qh0CTi>06i^KP1fMWH0EUIF04I6AP1J
FUX0820P02l_;`1GEeL0DE5A02`/;00n?Sh04Q8B028R8P1IFET0GemO02DU9@1`L70092@T04m?C`1=
CDd0<c<c04Y:BP0W9bL09RHV06UYJ@0k>c/0NGUi06a/K00I6AT0<30`05mOG`10@400[:b/07YjNP1S
Hf<0^[Zj05QHF013@d<0Shn?092@T00_;bl0C4a<08V9R@1:BTX0`<300<C4a01ADE40=3@d07=cL`1<
C4`0BDU903@d=026QXH0KVi^04Q8B01LG5`0D51@06a/K027QhL0G5aL0861P@1HF5P0GemO05YJFP1H
F5P0P82004A4A01UIFD0J6QX055AD@1=CDd0DU9B09fMW@1PH600FEUI06EUI@15ADD0JFUY075aL@15
ADD0>SXj04M7A`28R8P0:BTY061PH02:RXX0BTY:075aL@2LW9`0?3`l07moO`27QhL0?Shn051@D02K
Vi/0GEeM069RHP1FEUH0?3`l08:2PP1kNg/0TI6A06QXJ01aLG40GUiN0492@P1>CTh0[Zj^0861P@23
Ph<0XZ:R0:>SX`1FEUH0Fe]K08^;R`2[Zj/0F5QH07UiN@2EUID0@41006YZJP2CTi<0HF5Q05eMG@2C
Ti<0=3@d04E5A@0m?Cd0>SXj06UYJ@15ADD0UIFE061PH01CDe<0G5aL03@d=01UIFD0E5AD09ZJVP2Y
ZJT0FUYJ034a<@1ZJVX0C4a<08n?S`13@d<0=CDe07IfMP24Q8@0S8b<07AdM01ZJVX0KVi^04I6AP2g
]kL0alO708J6QP16ATH0H61P04Q8B013@d<0Shn?05eMG@1<C4`0T92@06e]K@1:BTX0JFUY08N7Q`12
@T80EEEE04Y:BP80Fe]K01T0R8R807]kN`1bLW80Fe]K08R8R01HF5P0PX:208n?S`1?Cdl0DU9B03Ph
>02SXj<0^k^k065QH@17AdL0DE5A03dm?@0I6AT0J6QX07moO`1YJFT0=CDe05aLG00g=cL08B4Q0080
Fe]K01T0>CTi05iNGP1/K6`0F5QH02HV9P2`/;00L71`02l_;`1YJFT0<c<c08:2PP0j>SX0>CTi09:B
TP1gMgL0?Shn096AT@1LG5`07alO01XJ6P11@D40ADE501PH6018B4P0Ph>300kooooo00`0>c/k03lo
?`17AdL0M7Ad06YZJP1UIFD0U9BD0;Ng]`20P800LG5a03Hf=P0A4A4300d=3@0>00`<300?3`l03`l?
014A4@0D51@07alO034a<@1UIFD0VYZJ0;6a/@2ZZZX0OGem0410@00M7Ad2010@400X024Q8@0a<C40
5QHF01DE5@0C4a<0410@014A4@0D51@0<C4a08N7Q`2OWil0_Kfm0:RXZ01RHV80DU9B05ADE01GEeL0
E5AD03`l?00[:b/0410@00d=3@0>3Ph04Q8B01TI6@0S8b<0820P01XJ6P0E5AD04A4A00d=3@0A4A40
?Shn08j>SP2HV9P0QhN707inOP1lO7`0BTY:01<C4`803Ph>01/0410@00h>3P0>3Ph03Ph>01lO7`1B
DU808R8R030`<01LG5`0410@00h>3P0=3@d03Ph>00d=3@0@41006ATI03Ph>00G5aL04a<C010@400C
4a<0S8b<0>KViP3Hf=P0`/;20:RXZ00V9RH00`0E5AD07P0G5aL071`L03Hf=P0P8200@41002DU9@0Y
:BT0<S8b069RHP0X:2P0C4a<092@T02FUYH0FEUI055AD@1VIVH0OWin07]kN`1nOWh0H61P03/k>`0i
>CT0ADE509FEU@3:b/X0IVIV05aLG01iNGT0EUIF02l_;`805aLG0340BTY:03@d=00h>3P0HF5Q0;>c
/`2IVIT0LW9b06=SH`27QhL0FUYJ03<c<`0b<S8092@T020P800N7Qh08B4Q0410@02m_Kd0eMGE0:RX
Z01^KVh0HV9R08J6QP2KVi/0fMWI08V9R@0N7Qh0AdM707=cL`1`L700DE5A05IFEP0Q8B40:2PX04M7
A`13@d<0:BTY02LW9`1gMgL0SXj>07inOP1SHf<0IfMW07alO00k>c/0G5aL0;2`/02g]kL0RXZ:00<0
5ADE01d06ATI02d];@1>CTh0S8b<0;2`/02l_;`0[Jf]08n?S`1PH600][Jf08^;R`1gMgL07QhN01HF
5P0J6QX0;Bd]03Ti>@15ADD0TI6A0;>c/`2RXZ80/;2`07YjNP14A4@05aLG01/K6`0b<S80Ph>3038b
<P0302@T900k02`/;01^KVh0CDe=05=CD`1>CTh0<C4a02<S8`1;Bd/0D51@04E5A@18B4P0FEUI04U9
B@21PH40H61P03<c<`0_;bl0?3`l07AdM01;Bd/0KFe]0820P02n_[h0_;bl06i^KP0k>c/0:b/[024Q
8@0[:b/0TY:B03/k>`0Y:BT0820P02HV9P1KFe/0DU9B0:JVYP2g]kL0^k^k0=;BdP3>c/h0d=3@0=KF
eP37alL0_kno0<W9b@2h^;P0SHf=08b<S01eMGD0LW9b04Q8B01lO7`0^;Rh0<c<c032`/80_kno09JF
UP1GEeL04?ooool000kooooo0=T0GUiN08J6QP1:BTX0Bd];096AT@1EEED0AdM709FEU@1IFET0De=C
06e]K@12@T80@410069RHP0a<C40=SHf02PX:010@4009bLW04A4A00i>CT07QhN069RHP0_;bl0<c<c
04m?C`0Y:BT0ATI604=3@`0X:2P0=SHf02<S8`18B4P0I6AT034a<@0h>3P0HV9R01hN7P1MGEd05aLG
03dm?@0c<c<0EUIF04I6AP2i^KT0DE5A038b<P0e=CD0Cdm?0451@@0P8200MWIf06YZJP1[Jf/0DU9B
04];B`1UIFD0BTY:02@T901UIFD0GemO03<c<`14A4@0>3Ph0492@P2DU9@0Q8B402PX:01cLg<0>c/k
05]KF`0i>CT0=3@d04e=C@0o?cl0:b/[06m_K`0k>c/0L71`059BDP0d=3@0=CDe0820P00j>SX0[Jf]
08b<S018B4P0F5QH0:6QX@0_;bl0MgMg04Q8B00W9bL0Cdm?09JFUP0f=SH0Ng]k0:^[Z`1FEUH0=3@d
07UiN@0n?Sh0G5aL02`/;01iNGT0G5aL051@D01UIFD0DU9B075aL@25QHD0FEUI08b<S01LG5`0EeMG
05YJFP1EEED0RXZ:055AD@1]KFd0LW9b05EEE@1?Cdl0M7Ad09jNWP1RHV80OWin07EeM@1:BTX0MgMg
08Z:RP1dM7@0ATI604A4A02PX:00:2PX05eMG@2?Shl0BTY:07emO@2JVYX0=3@d08:2PP20P800?Shn
05YJFP24Q8@0GEeM075aL@1?Cdl0<c<c08B4Q01kNg/0RXZ:06a/K01bLW80E5AD05]KF`1LG5`0_kno
0:b/[026QXH0UIFE09bLW01BDU80>3Ph065QH@2][Jd0G5aL08B4Q02KVi/0De=C0861P@2^[Zh0K6a/
06i^KP2/[:`0@d=3059BDP0k>c/0=cLg065QH@15ADD0T92@05eMG@1:BTX0De=C02TY:@1ZJVX0MGEe
09VIV@2i^KT0LG5a04=3@`2VYZH0G5aL0820P01@D500<c<c05iNGP1hN7P0KFe]0861P@1]KFd0HV9R
03Lg=`1hN7P0JVYZ07QhN014A4@0JFUY03De=@17AdL0UYJF069RHP1:BTX0OGem0080;2`/00@0CTi>
06UYJ@0g=cL0E5AD0P15ADD0400`<300Fe]K07QhN01_Kfl0I6AT08B4Q00d=3@0MGEe0:NWY`1QHF40
P820038b<P1oOgl0PX:206YZJP18B4P2061PH00S01hN7P0f=SH0EeMG05EEE@0k>c/0TI6A07EeM@0X
:2P0Kfm_0492@P0I6AT0GUiN08N7Q`1OGel0;2`/0:FUY@1NGUh092@T08V9R@1CDe<0H61P07IfMP1Q
HF40W9bL08V9R@1>CTh0[Zj^065QH@0];Bd0820P03Xj>P1<C4`07AdM055AD@29RHT03_ooool03P19
BDT0?3`l03@d=00U9BD0EEEE04m?C`19BDT0H61P09RHV02^[Zh0:RXZ014A4@0=3@d030`<0P0=3@d0
0`0g=cL0GUiN04];B`0203hn?P0=0410@01]KFd0NWYj04A4A01HF5P0KFe]07EeM@1fMWH0J6QX06EU
I@1`L7006ATI01<C4`03010@400D014A4@0K6a/0I6AT04E5A@0X:2P0YjNW09bLW02BTY80Ng]k05=C
D`1IFET0GEeM05=CD`1:BTX0?clo028R8P0>3Ph05ADE01/K6`0O7al202TY:@0?03/k>`11@D40;Bd]
04M7A`1mOGd0Ng]k05UIF@1cLg<0FUYJ05ADE01@D500H61P02<S8`0@41003Ph>00<0410@00H0<c<c
0410@00Q8B409BDU05QHF00A4A42010@40803Ph>00/0410@01TI6@0b<S80<c<c01DE5@0F5QH0AdM7
0;Vi^@3QhN40_;bl0:f][@0206m_K`1E024Q8@0E5AD051@D01DE5@0G5aL0:b/[04I6AP0k>c/06a/K
02PX:00b<S80Hf=S0451@@1[Jf/0F5QH065QH@1nOWh0BDU9059BDP19BDT0QHF509RHV02@T900Rh^;
08N7Q`2@T900JVYZ07MgM`1>CTh0DU9B05]KF`1[Jf/0Cdm?03dm?@0I6AT0GEeM03/k>`0h>3P0Jf][
07UiN@0j>SX0?clo06QXJ02=SHd0T92@07alO01oOgl0Hf=S06ATI01^KVh0I6AT08Z:RP39bLT0hN7Q
0=SHf034a<@0SXj>07MgM`1BDU80QhN70:NWY`1GEeL0;Rh^05QHF01EEED0JFUY07=cL`2PX:00C4a<
02h^;P0U9BD0>3Ph08V9R@1hN7P0J6QX065QH@0U9BD0<c<c069RHP2PX:00^KVi055AD@1SHf<0N7Qh
01LG5`0401DE5@1I01PH601UIFD0Cdm?07=cL`20P800[jn_0;bl_00f=SH07alO071`L00m?Cd0=CDe
04Y:BP1LG5`0NGUi08R8R02d];@0c/k>0=OGe`24Q8@0QhN7051@D00K6a/092@T034a<@2ATI40UiNG
03lo?`0];Bd092@T02HV9P1=CDd0I6AT04M7A`1;Bd/0HV9R09:BTP2EUID0E5AD03`l?01UIFD0VIVI
07UiN@19BDT07alO01/K6`0L71`0820P05YJFP1<C4`0OWin09fMW@2m_Kd0[Jf]0<71`@2c/k<0Q8B4
09bLW02i^KT0F5QH02`/;00R8R807alO03De=@0f=SH0DU9B0;fm_@2GUiL0Zj^[0<g=c@36a/H0e=CD
0=7Ad@36a/H0OWin08F5Q@1]KFd0?Cdm03hn?P19BDT0OWin07moO`1]KFd0_kno0<S8b02ZZZX0BTY:
04E5A@1RHV804?ooool000kooooo04T0I6AT09FEU@1=CDd0@T9207YjNP18B4P0BTY:0820P015ADD0
V9RH08n?S`1@D500B4Q807QhN00l?3`0>c/k04Q8B00f=SH08b<S04U9B@1LG5`0=cLg06e]K@0R8R80
<C4a06a/K00l?3`0DU9B04A4A00Y:BT0Bd];03@d=01<C4`0GUiN03Lg=`0i>CT0IVIV02/[:`1KFe/0
5aLG0492@P10@400EUIF04];B`2n_[h0GUiN04I6AP17AdL0FEUI04A4A00K6a/0Ph>3075aL@1XJ6P0
D51@04U9B@2:RXX0?clo02HV9P1RHV80EEEE03<c<`1=CDd0C4a<04i>CP2QXJ40Ph>302HV9P2;Rh/0
B4Q807EeM@1DE5@0>CTi0080DE5A09P0:BTY07YjNP0j>SX0Jf][06a/K01GEeL0<c<c07YjNP15ADD0
]KFe09>CT`1:BTX0Kfm_08B4Q01:BTX0QhN703hn?P0M7Ad0?clo07QhN00d=3@0CTi>0;2`/01JFUX0
C4a<09VIV@0c<c<0De=C030`<01hN7P0:b/[075aL@25QHD0IfMW07emO@1JFUX0Bd];09:BTP1NGUh0
I6AT05iNGP27QhL0T92@055AD@1_Kfl0PX:204i>CP1SHf<0GemO08f=S@1>CTh0IfMW06YZJP1CDe<0
Lg=c06IVIP1MGEd0F5QH04];B`2g]kL0;bl_08J6QP2FUYH0C4a<06MWI`2CTi<09BDU08Z:RP1mOGd0
<S8b06QXJ01fMWH0=cLg08:2PP1QHF40=3@d08:2PP1<C4`0QhN709NGU`1cLg<0H61P04M7A`0m?Cd0
VIVI08F5Q@1bLW80PH610:FUY@1NGUh0>CTi06][J`2UYJD0EUIF09bLW02XZ:P0GEeM08Z:RP2o_kl0
GUiN03hn?P1hN7P0;Bd]034a<@0l?3`0@D51071`L01JFUX0UYJF05IFEP13@d<0FUYJ03<c<`1kNg/0
G5aL08B4Q02n_[h0Fe]K03<c<`29RHT0FEUI08F5Q@1@D500?Shn079bLP2<S8`0IFEU07alO02OWil0
Shn?051@D02OWil0TY:B08Z:RP15ADD0IVIV03Ph>00o?cl0Ogmo061PH019BDT0QHF506UYJ@0o?cl0
De=C065QH@13@d<0MgMg04a<C01mOGd0BDU9059BDP80K6a/02`0KVi^0820P00g=cL0FUYJ08b<S01A
DE40C4a<02XZ:P2EUID0Q8B404M7A`0e=CD0O7al06m_K`0P8200>3Ph051@D00[:b/07QhN04m?C`1C
De<0;bl_09BDU01SHf<0Cdm?03dm?@1RHV80CTi>03`l?02m_Kd0IfMW01HF5P1>CTh0DE5A0820P021
PH40KVi^0<30`02PX:00>c/k09BDU01SHf<0>SXj02LW9`80@41000<09RHV07QhN02ATI403_ooool0
8@0h>3P0<30`02`/;00R8R807alO04Y:BP2EUID0P820071`L02@T900;Bd]01<C4`0=3@d01PH600X:
2P0C4a<05QHF02HV9P0b<S80=CDe03dm?@1[Jf/0];Bd06YZJP1IFET0FUYJ06IVIP0n?Sh0;bl_03Lg
=`1nOWh0S8b<04U9B@0201<C4`0E00h>3P0A4A4051@D05=CD`0h>3P07AdM01DE5@1QHF40=CDe02DU
9@1>CTh0Rh^;07alO01:BTX0EEEE04];B`0f=SH0?clo04U9B@0b<S807AdM00<06ATI00X0820P03`l
?01hN7P0LG5a059BDP1JFUX0FEUI06UYJ@2c/k<0L71`0P0[:b/03P1>CTh0DE5A03@d=00F5QH0410@
01HF5P1?Cdl0;2`/01`L700N7Qh0BTY:01<C4`0>3Ph04Q8B0P0>3Ph0G`0@4100:RXZ04Y:BP2FUYH0
NGUi07=cL`1KFe/0YjNW0;Vi^@3<c<`0/[:b06i^KP1ADE40L71`03`l?00D51@05ADE01@D500S8b<0
>3Ph04A4A00I6AT05QHF03De=@21PH40EEEE04I6AP0_;bl0<C4a05EEE@0F5QH08b<S059BDP1XJ6P0
G5aL05eMG@1kNg/0/;2`0;Rh^01cLg<07AdM01<C4`0_;bl0GemO07UiN@1fMWH0DE5A06][J`2KVi/0
U9BD08:2PP2k^k/0BDU902PX:00^;Rh0H61P0:BTY02LW9`0SXj>06UYJ@1[Jf/0Cdm?0410@015ADD0
HF5Q0<;2`P3Jf]X0Vi^K06IVIP0V9RH0820P02<S8`0Z:RX0JFUY0:n_[`1RHV80E5AD06=SH`1WIfL0
DU9B09>CT`1HF5P0=3@d05YJFP1aLG40TY:B071`L01bLW80=SHf03De=@0m?Cd0C4a<0:6QX@2o_kl0
;Rh^0080DU9B00<05ADE01@D500C4a<00P0E5AD0;00O7al0QXJ605]KF`15ADD0ATI604Q8B01cLg<0
:2PX02`/;013@d<0GEeM07IfMP1kNg/0A4A403Xj>P1?Cdl0_Kfm0:JVYP1oOgl0KVi^0;fm_@2/[:`0
KFe]07emO@25QHD0MgMg07moO`2LW9`0WIfM07IfMP1NGUh0HF5Q09>CT`1_Kfl0Ng]k08J6QP1mOGd0
/;2`07IfMP1PH600U9BD06][J`1oOgl09bLW0P0K6a/0:`0L71`0;Bd]079bLP1_Kfl0[Zj^0<71`@2?
Shl0P82007=cL`1bLW80QhN70:>SX`3;bl/0;bl_028R8P0M7Ad08B4Q04E5A@0];Bd0IfMW0:j^[P2U
YJD0[Jf]0<S8b033`l<0b<S80;^k^`2<S8`08R8R01lO7`0e=CD0JVYZ05=CD`1[Jf/0L71`05UIF@12
@T80Ti>C0;6a/@2GUiL0<C4a03@d=01<C4`04?ooool000kooooo06T0KFe]06i^KP1>CTh0BTY:07]k
N`15ADD0DE5A09VIV@1ADE40EEEE07QhN016ATH0@41005YJFP0f=SH0EEEE04];B`1:BTX0;bl_061P
H01]KFd0=SHf06UYJ@0];Bd0>3Ph05QHF00j>SX0C4a<051@D00e=CD0DU9B02/[:`1GEeL0DU9B02TY
:@13@d<0GemO02@T901RHV807AdM04e=C@16ATH0K6a/04m?C`2e]KD0MGEe04A4A017AdL0Fe]K04U9
B@0f=SH0L71`08J6QP1YJFT0>CTi038b<P1VIVH0=cLg03Xj>P1WIfL0?Shn02`/;01>CTh0@d=305AD
E02XZ:P0Jf][01hN7P2ATI40B4Q80820P01UIFD0>c/k05iNGP1BDU80:b/[07MgM`11@D40OWin08J6
QP15ADD0=3@d07YjNP0g=cL0Z:RX08:2PP19BDT0^KVi05eMG@0l?3`0SXj>05MGE`0J6QX0@d=307Ui
N@1;Bd/0>3Ph0:2PX01HF5P0LG5a07inOP18B4P0FUYJ02/[:`1gMgL00P1KFe/03P1UIFD0@41006][
J`1eMGD0CTi>06UYJ@10@400IFEU05QHF01NGUh0L71`03lo?`1eMGD0QHF50P1FEUH0701MGEd0WYjN
06UYJ@28R8P0F5QH0451@@1WIfL0H61P06ATI00f=SH0CTi>0:FUY@0h>3P0IFEU06m_K`0a<C40TI6A
09fMW@0h>3P0S8b<05]KF`0^;Rh0M7Ad07UiN@1VIVH0K6a/0492@P0a<C4207alO01m09BDU01fMWH0
IFEU07]kN`1OGel0IFEU0:^[Z`2JVYX0MWIf08V9R@2k^k/0Ph>305IFEP2;Rh/0[:b/04];B`1KFe/0
OWin04A4A023Ph<0]KFe07=cL`18B4P0SXj>03dm?@1JFUX0ATI605]KF`29RHT0De=C08j>SP11@D40
KFe]07alO00j>SX0H61P038b<P1;Bd/0];Bd07EeM@13@d<0Y:BT0451@@2;Rh/0FUYJ04=3@`1eMGD0
SXj>065QH@1aLG40HF5Q08:2PP1=CDd0^[Zj096AT@20P800?clo075aL@1DE5@0Cdm?09>CT`1OGel0
D51@09ZJVP1hN7P0AdM7069RHP1YJFT0<S8b05iNGP1aLG40MGEe05=CD`1fMWH0TI6A04e=C@2KVi/0
SHf=055AD@1ZJVX0VIVI04M7A`1WIfL0C4a<09fMW@2VYZH0GEeM0410@01:BTX0ADE5028R8P1EEED0
Lg=c05]KF`0U9BD0GEeM04Q8B00Z:RX0PX:208J6QP1/K6`0Bd];06QXJ01EEED0<S8b08R8R01KFe/0
:BTY05QHF014A4@0QHF504Y:BP18B4P0`L710:FUY@1SHf<0Zj^[04];B`0d=3@0?3`l069RHP1EEED0
7AdM04M7A`29RHT03_ooool0C`0c<c<0:b/[02HV9P0Y:BT0>c/k05EEE@26QXH0UIFE09JFUP2EUID0
KVi^05IFEP1MGEd0?Shn05IFEP0[:b/08R8R03De=@0g=cL0>SXj03lo?`0l?3`0BDU906=SH`1dM7@0
PX:207YjNP15ADD0:b/[03lo?`14A4@0<C4a05iNGP1VIVH0;Bd]01<C4`0O7al0EEEE03Xj>P0L71`0
5QHF01DE5@0a<C40;bl_00l?3`0@41004a<C030`<01?Cdl0FUYJ04M7A`1QHF40@T9204e=C@1SHf<0
I6AT059BDP1DE5@0GUiN06i^KP1hN7P0HF5Q05YJFP1CDe<0FEUI08^;R`1KFe/0S8b<06MWI`1BDU80
=cLg0451@@14A4@0@41004=3@`15ADD0@T9208J6QP0i>CT00P0H61P00`0n?Sh08B4Q00h>3P0200h>
3P2000l?3`0F5QH0:b/[05UIF@2JVYX0EeMG06UYJ@1fMWH0IFEU0:2PX02[Zj/0Lg=c06a/K01ZJVX0
HV9R06=SH`1LG5`0Cdm?04a<C01hN7P0@D5102h^;P0U9BD0?Cdm07inOP24Q8@0R8R8079bLP1WIfL0
JFUY04M7A`0g=cL0ATI6071`L018B4P0Bd];03dm?@0h>3P0?Cdm08>3P`1CDe<0:2PX01DE5@0D51@0
;Rh^04a<C01kNg/0Z:RX07YjNP13@d<0BTY:05IFEP1bLW80VYZJ0:NWY`2DU9@0Ph>309:BTP27QhL0
M7Ad04m?C`1<C4`0De=C07inOP1VIVH0R8R807alO01`L700QHF5059BDP0U9BD07alO03dm?@0_;bl0
9bLW069RHP2ZZZX0Ogmo06a/K01RHV80SHf=07UiN@1gMgL0XJ6Q06a/K022PX80N7Qh05IFEP12@T80
>SXj03dm?@0g=cL0?Shn08:2PP2;Rh/0EUIF04A4A01NGUh071`L01PH600R8R80:2PX02`/;00`<300
=3@d05MGE`1OGel0Fe]K04e=C@1:BTX0>CTi01TI6@0Q8B40P82009bLW02EUID0I6AT06e]K@23Ph<0
YZJV08F5Q@1eMGD0Q8B407moO`0h>3P0<30`03Hf=P0k>c/203Hf=P0L03Lg=`0n?Sh0BTY:07emO@2=
SHd0L71`08>3P`2TY:@0/k>c0:n_[`32`/80_[jn0;:b/P1_Kfl0IfMW06YZJP1HF5P0CTi>04A4A00o
?cl0LW9b0;Jf]P2WYjL0];Bd09nOW`2CTi<0OGem04e=C@80?3`l0200BDU909FEU@1@D5009RHV03Ti
>@1<C4`0<30`03`l?01DE5@0YjNW0:6QX@2b/[80]kNg07moO`1/K6`0GEeM0861P@0Z:RX0<C4a06MW
I`2CTi<0OGem08>3P`2g]kL0ZJVY08j>SP31`L40[:b/06EUI@0b<S80=3@d02l_;a3ooooo000>oooo
o`2206UYJ@1aLG40AdM704U9B@2@T900AdM704=3@`1lO7`0AdM705UIF@0l?3`0@T920820P02=SHd0
?Shn04I6AP1:BTX0K6a/02XZ:P18B4P0DE5A02<S8`29RHT0D51@03@d=00^;Rh02@T9028R8P1GEeL0
?Cdm03Lg=`0:2PX0:2PX06ATI00e=CD0>3Ph05IFEP0C4a<0DE5A02LW9`12@T80A4A40861P@0X:2P0
Rh^;05eMG@11@D40Lg=c05mOG`1BDU808R8R04a<C01TI6@0@41003@d=00n?Sh0IfMW03dm?@0Y:BT0
JVYZ05mOG`0b<S80ATI602d];@0d=3@0O7al06a/K00N7Qh0U9BD03dm?@1=CDd0=SHf0410@01:BTX0
De=C024Q8@1cLg<0=CDe06QXJ02:RXX0F5QH03<c<`1oOgl0?Cdm0:NWY`2OWil0CTi>0:j^[P1bLW80
:b/[08n?S`27QhL0;Rh^04M7A`23Ph<0=cLg04];B`1aLG40UYJF0:6QX@30`<00;bl_04];B`0_;bl0
J6QX020P8019BDT0MGEe06ATI01NGUh0=cLg02LW9`1nOWh0FEUI05IFEP12@T80JFUY08R8R01ADE40
LW9b071`L00Y:BT0F5QH059BDP2:RXX0>3Ph07QhN01^KVh0De=C06UYJ@80EeMG0980Bd];0410@030
`<00<S8b0861P@27QhL0>CTi06IVIP29RHT0=3@d09fMW@1bLW808B4Q06i^KP1TI6@0;Rh^0820P022
PX80;Rh^06EUI@10@400WYjN0;Ng]`18B4P0Ph>308F5Q@1TI6@0ZJVY0;^k^`24Q8@0NGUi09nOW`10
@400ADE506QXJ02/[:`0Bd];05EEE@2BTY80CDe=08J6QP31`L40Q8B403lo?`25QHD0<c<c0451@@0l
?3`0;Rh^06=SH`1JFUX0V9RH04];B`1LG5`0IfMW02@T901iNGT0J6QX07]kN`2HV9P0CDe=065QH@31
`L40C4a<07AdM01=CDd0<S8b06MWI`2TY:@0Lg=c08B4Q01KFe/0N7Qh03dm?@2OWil0/k>c096AT@0h
>3P0HF5Q0410@01>CTh0X:2P0820P013@d<0Ph>303/k>`0i>CT0DU9B06UYJ@0k>c/0CTi>06e]K@1;
Bd/0;Bd]05MGE`1eMGD0EEEE09bLW01SHf<0;Rh^05MGE`1dM7@0F5QH07moO`0l?3`0M7Ad09VIV@1X
J6P0BDU905QHF01aLG409RHV03Lg=`1@D500@d=3034a<@1]KFd0F5QH03<c<`1cLg<0N7Qh030`<01A
DE40LG5a04e=C@0b<S80Rh^;06=SH`0b<S80G5aL05ADE02`/;00FUYJ04E5A@2=SHd0V9RH05UIF@2b
/[80GUiN02`/;00R8R80@T9205eMG@0`<300IFEU09FEU@kooooo0140<c<c03@d=018B4P0H61P03Hf
=P0g=cL0ATI604i>CP1QHF40Q8B407IfMP2SXj<0/[:b07MgM`2OWil0?Shn02<S8`0201DE5@0E01<C
4`0E5AD05ADE01DE5@0G5aL092@T06ATI01_Kfl0EEEE04M7A`0d=3@0@D51038b<P0T92@0CDe=0:^[
Z`1BDU80JVYZ03/k>`0M7Ad05QHF00805aLG00D06ATI03<c<`0@4100410@01<C4`0200h>3P0f02`/
;020P800_kno07MgM`1HF5P0K6a/08F5Q@2k^k/0e=CD0<G5a@2MWId0QHF506EUI@17AdL0?Shn05EE
E@2;Rh/0KVi^038b<P0I6AT0ADE506EUI@1GEeL0?clo03Xj>P1BDU80GemO03Lg=`1@D500JFUY03@d
=00M7Ad0=cLg04=3@`0I6AT092@T038b<P0h>3P0>CTi055AD@28R8P0DE5A08R8R02ATI40JVYZ06=S
H`1kNg/0@T9201lO7`0_;bl0De=C04U9B@13@d<0L71`0P2RXZ80;`2NWYh0U9BD07=cL`2;Rh/0NWYj
04I6AP1FEUH0CDe=069RHP1NGUh0<C4a02XZ:P0M7Ad0820P0451@@1bLW80NWYj05]KF`0/;2`0F5QH
09NGU`19BDT0De=C061PH00h>3P05ADE03Hf=P2d];@0Z:RX08R8R010@400?Shn03Ti>@1CDe<0_[jn
0;Fe]@2@T900CDe=05=CD`19BDT0Hf=S051@D00_;bl07AdM024Q8@1_Kfl0M7Ad0080<C4a02@0FUYJ
07YjNP0n?Sh0=SHf03@d=00/;2`0:BTY02h^;P1FEUH0VYZJ0:>SX`2EUID0TY:B08R8R01_Kfl0V9RH
0:n_[`1MGEd0:RXZ01LG5`0D51@05ADE01@D500E5AD05aLG02`/;01JFUX0Ng]k03lo?`1PH600AdM7
038b<P0S8b<0820P028R8P0S8b<202HV9P0l02d];@0f=SH0@d=30451@@1XJ6P0<30`06m_K`1eMGD0
LW9b07alO00];Bd0=3@d03lo?`1>CTh0V9RH0<30`03Bd]80@T9201`L700M7Ad06QXJ01TI6@0H61P0
7AdM02/[:`14A4@0J6QX04Q8B00l?3`0Hf=S09jNWP2DU9@0X:2P0;bl_035aLD0XJ6Q08n?S`1<C4`0
DE5A05aLG01[Jf/0NWYj0:>SX`2d];@0/k>c0;^k^`3EeMD0emOG0:b/[01GEeL0HV9R08>3P`2>SXh0
Lg=c05EEE@1MGEd0X:2P09nOW`2ZZZX0ATI60P0_;bl0601ADE40_kno0:n_[`2BTY80d=3@0>3Ph02f
][H0I6AT0:JVYP2RXZ80IVIV0492@P0e=CD0>SXj05eMG@1[Jf/0YJFU0<k>cP3<c<`0e]KF05EEE@0X
:2P0;Bd]04M7Aa3ooooo000>ooooo`1507alO02JVYX0C4a<0492@P1gMgL0B4Q804Y:BP1[Jf/0<30`
08Z:RP1dM7@0<30`02h^;P1DE5@0>3Ph05]KF`0X:2P0GemO03<c<`1@D500FEUI014A4@1ADE40@D51
04e=C@1NGUh0:2PX020P800o?cl0<S8b06ATI00P8200=CDe05IFEP0W9bL0?clo079bLP0O7al0GUiN
01lO7`12@T80B4Q804e=C@11@D40U9BD06UYJ@0g=cL0?clo03hn?P1DE5@0>CTi06IVIP1UIFD0CDe=
03Hf=P0l?3`0L71`03Hf=P0X:2P0IFEU05aLG00C4a<0@d=303hn?P0c<c<0PX:2075aL@0Q8B40Ng]k
0080DU9B0<l0E5AD04E5A@1PH600GemO02<S8`1kNg/0ADE509FEU@2NWYh0JFUY02/[:`1MGEd0?Cdm
08B4Q02:RXX0BDU9096AT@1=CDd0;2`/0820P01?Cdl07AdM0492@P1lO7`0C4a<05IFEP1ADE40WYjN
0<g=c@2JVYX0ATI605=CD`0a<C40H61P05UIF@15ADD0GUiN061PH01CDe<0MGEe05UIF@2GUiL0GEeM
05aLG00o?cl0AdM7075aL@1ADE40KVi^06YZJP1<C4`0FUYJ05aLG02LW9`0IFEU071`L01bLW80Bd];
07]kN`1`L700EUIF03hn?P18B4P0^[Zj02LW9`1ZJVX0LG5a03@d=01EEED0O7al01hN7P1iNGT0JVYZ
01`L701@D500KFe]05ADE01GEeL0I6AT03Lg=`1_Kfl0@41006EUI@1fMWH0L71`07emO@1CDe<0<S8b
07=cL`27QhL0Ti>C08>3P`2QXJ40G5aL04e=C@1UIFD0/;2`05EEE@1aLG40W9bL059BDP29RHT0ZZZZ
08:2PP1UIFD0Shn?0492@P1EEED0E5AD02PX:01EEED0BTY:08B4Q01:BTX0EEEE06][J`0V9RH0GemO
06UYJ@1eMGD0/[:b04i>CP0j>SX0YjNW061PH01eMGD0C4a<028R8P0e=CD0WYjN07alO02HV9P0Zj^[
096AT@0i>CT0M7Ad06=SH`25QHD0B4Q807alO020P800LW9b09NGU`1lO7`0CTi>09JFUP1fMWH0ATI6
05ADE01^KVh0A4A4071`L024Q8@0QXJ604e=C@1cLg<0Ng]k03lo?`2d];@0RXZ:04Y:BP1ZJVX0R8R8
06UYJ@1gMgL0EUIF08n?S`2VYZH0GUiN03De=@1dM7@0Kfm_02DU9@0k>c/0B4Q804E5A@0P8200G5aL
06MWI`0_;bl0K6a/08N7Q`1VIVH0A4A405MGE`13@d<0?Cdm0:ZZZP22PX80>3Ph05ADE00P8200IFEU
05eMG@0d=3@0T92@09JFUP1ADE40WYjN04m?C`0_;bl0:BTY0410@012@T805ADE05ADE023Ph<03_oo
ool05@1QHF40M7Ad05]KF`0n?Sh0@41003Hf=P0U9BD06ATI01PH6011@D40=cLg08N7Q`2l_;`0Vi^K
0;Vi^@0`<3006ATI01DE5@0G5aL05ADE01@D500201PH600A034a<@18B4P0J6QX04A4A019BDT0EEEE
05ADE01>CTh0>c/k03Ti>@0W9bL0De=C0=;BdP2<S8`092@T01LG5`0A4A400`0E5AD0I`0e=CD03Ph>
00d=3@0=3@d0410@00h>3P0A4A40BTY:0<k>cP1nOWh0=CDe03<c<`15ADD0VIVI08J6QP1lO7`0E5AD
01/K6`0E5AD04a<C02DU9@1OGel0;Rh^02DU9@1=CDd092@T024Q8@0f=SH0FUYJ06IVIP16ATH0;Rh^
028R8P0X:2P0:RXZ04A4A01VIVH0NWYj09^KV`2QXJ40HV9R05YJFP1=CDd0C4a<04U9B@1@D500Q8B4
08V9R@1VIVH0Bd];04M7A`1LG5`0K6a/03Xj>P0/;2`07QhN01`L700k>c/0E5AD05aLG01lO7`0VIVI
0;Zj^P3>c/h0_Kfm09^KV`1aLG40=3@d04=3@`25QHD0IFEU02`/;00V9RH0;Bd]01LG5`0F5QH0;Rh^
065QH@1gMgL0L71`0820P01>CTh0>3Ph04A4A00d=3@0<S8b05YJFP1oOgl0XZ:R08J6QP2FUYH0OGem
06EUI@0a<C409RHV04E5A@2QXJ40PX:209:BTP1@D500C4a<059BDP1<C4`00P0a<C40600V9RH071`L
04a<C027QhL08R8R02`/;01`L700_;bl0:6QX@1QHF40D51@03lo?`15ADD0@T9202@T900_;bl0JFUY
0:NWY`2?Shl0cLg=0;Fe]@2ATI40F5QH01LG5`H05ADE00d05QHF024Q8@1LG5`0GEeM05UIF@1fMWH0
EEEE028R8P0Q8B409BDU02PX:00P82006ATI00805aLG01T06ATI04M7A`0h>3P0>3Ph0;>c/`2/[:`0
OWin0:NWY`2LW9`0?clo04A4A018B4P0CDe=09bLW02h^;P0Ti>C07QhN01IFET092@T01LG5`0I6AT0
6a/K03@d=01OGel0BTY:0080:RXZ02@08b<S02PX:00h>3P0UIFE0<c<c02o_kl0Rh^;0=;BdP2e]KD0
Shn?09nOW`2CTi<0RHV9079bLP1IFET0UIFE09JFUP1KFe/0T92@0<?3``2YZJT0LG5a08:2PP2QXJ40
^KVi0<K6aP2f][H0[:b/0=3@d02TY:@0Xj>S08n?S`27QhL0S8b<08n?S`1MGEd206QXJ00D0<30`01n
OWh0SXj>0:BTY038b<P0SXj>05IFEP1:BTX0@41004I6AP1GEeL0CTi>0;Zj^P3Jf]X0fm_K0=[JfP2=
SHd0ADE5055AD@1lO7`@ooooo`003_ooool0U01PH600Rh^;04a<C00i>CT0Jf][04Q8B015ADD0Jf][
01hN7P0l?3`0JVYZ03dm?@0b<S80<30`00l?3`1@D500?Shn04E5A@0D51@0MGEe08B4Q00l?3`0HV9R
028R8P0d=3@0G5aL03<c<`0O7al0@D5101TI6@1JFUX0:2PX03@d=018B4P010@401/K6`1LG5`09RHV
07MgM`0A4A40ADE504M7A`16ATH06QXJ07UiN@25QHD0BDU904i>CP0o?cl0;2`/055AD@1KFe/0GUiN
05QHF00G5aL05ADE06e]K@11@D408b<S065QH@17AdL08b<S051@D00a<C40=cLg0820P01;Bd/06QXJ
071`L01ADE40EUIF061PH00X:2P0KFe]06ATI00R8R80Fe]K030`<01LG5`0PX:205=CD`0R8R80Ng]k
02l_;`2EUID0RXZ:05YJFP2:RXX0JFUY03Xj>P2HV9P0B4Q8028R8P1?Cdl0PX:205EEE@16ATH0@D51
0;Jf]P28R8P0GemO03Xj>P1EEED0:BTY065QH@1LG5`0CTi>061PH01OGel0Hf=S096AT@1SHf<0YJFU
065QH@1ADE40F5QH06QXJ028R8P0BDU905EEE@1dM7@0I6AT06i^KP1@D500VYZJ07YjNP1oOgl0Kfm_
03<c<`1/K6`0LG5a055AD@0h>3P0Bd];0<30`00T92@0K6a/07]kN`0a<C40EeMG092@T00l?3`0Shn?
06EUI@0O7al0I6AT07alO01[Jf/205YJFP2003Ph>01YJFT0KFe]0;Bd]01hN7P0K6a/08>3P`1dM7@0
Jf][0:6QX@2GUiL0IFEU08:2PP2GUiL0Jf][04M7A`1OGel0Vi^K04Q8B02=SHd0T92@04M7A`22PX80
^;Rh065QH@0g=cL0Ogmo04];B`1XJ6P0EEEE02PX:01LG5`0Fe]K09>CT`1HF5P0Cdm?079bLP0j>SX0
FUYJ02l_;`1IFET0[jn_06IVIP0l?3`0Zj^[055AD@1oOgl0J6QX0451@@1?Cdl0^KVi06=SH`1IFET0
Kfm_07IfMP1?Cdl0^[Zj08J6QP23Ph<092@T06YZJP28R8P0M7Ad09VIV@1FEUH0De=C0:BTY01cLg<0
A4A405QHF01UIFD06a/K03De=@1fMWH0P82004e=C@1]KFd0MgMg055AD@2DU9@0J6QX03hn?P1PH600
LW9b03Lg=`1aLG40D51@08N7Q`2l_;`0T92@034a<@1HF5P0KVi^02/[:`1FEUH0Kfm_069RHP0Y:BT0
IfMW06IVIP0g=cL0Z:RX09nOW`1eMGD0ATI605MGE`0J6QX07QhN061PH01SHf<0>CTi06a/K00_;bl0
RXZ:06=SH`0o?cl0]KFe0:2PX020P800_Kfm07MgM`0e=CD0:b/[0492@P1RHV80;2`/04a<C02@T90>
ooooo`0>06][J`1<C4`0BTY:038b<P0M7Ad071`L01XJ6P0G5aL05ADE01TI6@1YJFT0UiNG0:BTY022
PX8207UiN@0M03Xj>P0a<C40D51@04i>CP0i>CT0?Cdm04e=C@1VIVH0EUIF06EUI@1?Cdl09BDU02TY
:@1BDU80LW9b05UIF@0o?cl0=SHf02l_;`31`L40RXZ:01TI6@0E5AD04A4A010@400C4a<06QXJ05=C
D`0C4a<00P0>3Ph04P0?3`l04a<C01<C4`0j>SX0^KVi09:BTP1nOWh0MgMg07moO`2<S8`0N7Qh08N7
Q`1eMGD07alO018B4P0L71`0IVIV02l_;`803Ph>04408B4Q0410@00L71`05aLG01<C4`0U9BD0EeMG
06MWI`13@d<0;2`/0451@@17AdL0alO70:ZZZP2PX:00W9bL05=CD`1HF5P0DU9B03Lg=`0[:b/09RHV
075aL@2PX:00QhN705QHF00o?cl0;Bd]04a<C00H61P05aLG01dM7@0P82009bLW03Ph>01PH600@d=3
04A4A01jNWX0`L710<;2`P2a/K40FEUI04m?C`1IFET0?Shn04=3@`17AdL092@T02PX:00`<3006a/K
01LG5`10@400J6QX08b<S01?Cdl071`L01PH600c<c<0>c/k03`l?00b<S80IFEU0<K6aP0207UiN@0U
09>CT`27QhL0I6AT03hn?P1gMgL0S8b<0820P023Ph<0KVi^04i>CP1[Jf/0<30`01lO7`0M7Ad0;2`/
034a<@1OGel0MGEe024Q8@0j>SX0MGEe0:n_[`34a<@0Ti>C0:>SX`27QhL0U9BD08n?S`1OGel0M7Ad
06m_K`12@T80@D5105QHF02YZJT0EUIF03<c<`0301DE5@0301<C4`0E5AD05ADE00805ADE06<071`L
055AD@0o?cl0BTY:06m_K`2RXZ80DE5A03`l?00c<c<0<30`02@T900O7al07QhN01TI6@0E5AD06ATI
03hn?P0i>CT0Zj^[075aL@1QHF40[Zj^0;^k^`1_Kfl0K6a/05eMG@1oOgl0UiNG0;no_`1]KFd0Cdm?
04U9B@1SHf<0FUYJ030`<00Z:RX0HV9R02h^;P0M7Ad09RHV030`<00O7al07QhN028R8P1eMGD0`L71
0:2PX01`L700emOG0=GEe@2MWId0BTY:02l_;`0h>3P0A4A405MGE`1oOgl0GEeM02@T900Z:RX0Hf=S
05]KF`30`<00Y:BT08j>SP1lO7`0OGem08:2PP2FUYH0]kNg09ZJVP25QHD0KFe]075aL@1oOgl0TI6A
04m?C`1IFET0TI6A069RHP12@T80=cLg07UiN@36a/H0_Kfm08R8R01NGUh0I6AT04];B`1IFET0WinO
0=WIf@3Jf]X0aLG50;Ng]`33`l<0W9bL0:ZZZP1KFe/04?ooool000kooooo0?l0HF5Q09ZJVP1=CDd0
CTi>096AT@19BDT0CDe=0:6QX@1ADE40DU9B08B4Q013@d<0=CDe05ADE00U9BD0?3`l03hn?P18B4P0
410@0492@P1^KVh0>CTi061PH00];Bd0:BTY04a<C00e=CD07QhN03Ti>@0_;bl0I6AT03hn?P0[:b/0
C4a<02XZ:P0R8R80GEeM02DU9@1hN7P06ATI04I6AP0];Bd0EEEE04A4A01jNWX0MgMg04Q8B01>CTh0
BTY:04e=C@1CDe<0@d=304];B`1HF5P0C4a<024Q8@1CDe<0@d=302XZ:P1ZJVX0ADE502XZ:P12@T80
;Rh^03De=@1gMgL0KFe]03@d=01IFET0C4a<05QHF011@D40=CDe05IFEP1RHV8092@T06i^KP19BDT0
QhN708J6QP0e=CD08B4Q07emO@0d=3@0V9RH09BDU00b<S80Q8B408F5Q@0m?Cd0Shn?0451@@0Q8B40
ATI6075aL@1ADE40EEEE03Hf=P2d];@0?Cdm06EUI@1?Cdl0CTi>024Q8@1gMgL0GEeM04Y:BP1RHV80
>c/k069RHP2<S8`0Fe]K08j>SP1@D500NGUi05mOG`1XJ6P0Ph>304Q8B01ZJVX0SXj>06EUI@1ZJVX0
H61P0:2PX01/K6`0NGUi059BDP0a<C40R8R806IVIP1JFUX0820P065QH@2m_Kd0BTY:06MWI`1lO7`0
=SHf08b<S02QXJ40@41009:BTP1<C4`0<c<c071`L01gMgL0HV9R05MGE`1;Bd/0A4A406EUI@1cLg<0
UIFE08:2PP1RHV80YjNW07UiN@1PH600Vi^K0861P@1jNWX0Ogmo0::RXP1SHf<0AdM706e]K@2d];@0
@D5106IVIP26QXH0=SHf08V9R@3;bl/0M7Ad04i>CP2<S8`0CDe=05IFEP0a<C40@d=308V9R@1GEeL0
S8b<03@d=016ATH0SHf=0492@P1OGel092@T04m?C`2e]KD0EUIF034a<@2FUYH0CTi>07inOP1KFe/0
B4Q804m?C`30`<00K6a/06e]K@1VIVH0Lg=c04M7A`2g]kL0UiNG08N7Q`0j>SX0G5aL071`L01^KVh0
Ti>C06e]K@0m?Cd0R8R806QXJ00o?cl0B4Q806EUI@0l?3`0AdM707emO@1kNg/0ATI6075aL@1dM7@0
GemO0::RXP2:RXX0EeMG06i^KP1mOGd0EUIF06i^KP0m?Cd0PX:20=OGe`2MWId0DU9B05iNGP12@T80
7alO065QH@22PX80G5aL02HV9P1LG5`0<S8b01`L701cLg<0QhN706m_K`1:BTX001L0Cdm?07AdM00d
=3@0M7Ad06QXJ015ADD0UYJF03Ph>02;Rh/0BTY:02TY:@21PH40YjNW08N7Q`35aLD0JFUY03Ti>@0b
<S80Bd];065QH@0[:b/0DE5A09bLW00>ooooo`0707EeM@18B4P0<S8b02LW9`0K6a/06ATI01LG5`02
01DE5@0h018B4P0b<S80VIVI0:2PX02GUiL0RHV907=cL`0g=cL0;bl_02`/;00V9RH07AdM01PH600I
6AT0;2`/01hN7P0J6QX0:b/[03dm?@0[:b/08R8R05]KF`1ZJVX0LG5a071`L02<S8`0b<S80<71`@1S
Hf<0>c/k02TY:@0X:2P0;Bd]0492@P1dM7@0GEeM04Q8B00c<c<07alO010@400O7al0IfMW04];B`0E
5AD08b<S03`l?01bLW80SHf=061PH01;Bd/0CDe=05UIF@0V9RH0IVIV02PX:00>3Ph300d=3@1I03`l
?00_;bl051@D01<C4`0>3Ph03@d=01`L7014A4@0O7al07QhN02c/k<0XZ:R08Z:RP2[Zj/0D51@059B
DP0n?Sh0BDU904a<C017AdL0TY:B05MGE`1oOgl0PX:206e]K@0k>c/071`L05UIF@0M7Ad05ADE01@D
500E5AD0:b/[01dM7@12@T80Jf][055AD@2EUID0`L710<G5a@23Ph<0JVYZ05mOG`0U9BD04a<C010@
400^;Rh0@T9201`L700Y:BT0@41003Lg=`0X:2P0NWYj07alO01CDe<0:RXZ01DE5@0F5QH06ATI02@T
9010@400Rh^;0;6a/@2XZ:P0?3`l0:2PX03IfMT0Zj^[08B4Q02QXJ40UIFE08f=S@25QHD0L71`07in
OP1KFe/0>3Ph03<c<`0_;bl0<S8b0410@02EUID0R8R805EEE@1`L700G5aL0861P@2LW9`00P15ADD0
3@11@D40>3Ph04I6AP10@400>3Ph03Ti>@17AdL0EUIF03De=@2>SXh0De=C02h^;P0E5AD00`0C4a<4
01DE5@0=01/K6`1KFe/0I6AT04=3@`1:BTX0VYZJ0;^k^`1aLG40Shn?075aL@1:BTX0<c<c02d];@02
02HV9P1>01TI6@0l?3`0Q8B40;Rh^01mOGd0O7al0=7Ad@3QhN40Kfm_05YJFP17AdL0GEeM0::RXP1o
Ogl0L71`055AD@1?Cdl0Bd];05QHF02GUiL0`/;2059BDP0L71`07alO02TY:@0[:b/07QhN02<S8`0g
=cL0Fe]K0:2PX025QHD0UIFE0<K6aP2CTi<0HF5Q03Xj>P18B4P0Bd];04m?C`1cLg<0@d=303lo?`0R
8R806a/K034a<@1VIVH0Lg=c05UIF@1?Cdl0ATI6059BDP1NGUh0O7al07AdM02=SHd0R8R804m?C`11
@D40HF5Q08^;R`1MGEd0M7Ad09FEU@1LG5`0D51@03Xj>P1HF5P0VYZJ07alO02GUiL0^;Rh0;:b/P2[
Zj/0/;2`0;no_`2n_[h0_Kfm0P34a<@0103>c/h0f=SH08j>SP0W9bL@ooooo`003_ooool0^01XJ6P0
Kfm_04i>CP1;Bd/0IVIV02DU9@15ADD0M7Ad04Y:BP1DE5@0IfMW04];B`1EEED0OGem03dm?@13@d<0
@T9205YJFP0`<300Bd];0861P@0e=CD0De=C03@d=00e=CD0EEEE02h^;P0e=CD0EUIF03Hf=P1JFUX0
:b/[03/k>`1ADE40?Cdm02@T901]KFd08b<S07alO00_;bl0B4Q802LW9`1DE5@0DU9B07]kN`1^KVh0
ADE506i^KP0n?Sh0BDU905EEE@13@d<0JVYZ07MgM`1bLW80:b/[05]KF`0N7Qh0<C4a071`L01TI6@0
@T9205ADE00/;2`0CDe=092@T01bLW80Bd];055AD@1=CDd0OGem05=CD`16ATH0Fe]K04=3@`0^;Rh0
Jf][03/k>`29RHT0WIfM0492@P0Y:BT0IfMW02TY:@2DU9@0Z:RX03Hf=P1^KVh0J6QX0492@P27QhL0
H61P01LG5`0U9BD0EEEE061PH00k>c/0GemO08F5Q@0_;bl0F5QH0410@01;Bd/08B4Q061PH018B4P0
>3Ph06=SH`1<C4`0@41006UYJ@12@T80JVYZ01`L701=CDd0EeMG06MWI`1OGel0>SXj06EUI@22PX80
C4a<05iNGP1:BTX0Rh^;065QH@1lO7`0IFEU03dm?@1XJ6P0HV9R05eMG@15ADD0?clo0:ZZZP0o?cl0
HF5Q06a/K00T92@0Jf][08b<S00n?Sh0R8R804Y:BP10@400Q8B406e]K@1OGel0FUYJ04i>CP0U9BD0
Bd];05=CD`1]KFd0VIVI055AD@2OWil0R8R807MgM`2OWil0TY:B079bLP25QHD0VYZJ03dm?@11@D40
PX:20;Vi^@1OGel0IFEU096AT@13@d<0H61P09nOW`24Q8@0Kfm_08j>SP0l?3`0KVi^03Xj>P0e=CD0
FUYJ05ADE02KVi/204A4A01L08N7Q`0`<300IfMW05]KF`1WIfL0^k^k04i>CP0d=3@0XJ6Q04];B`1N
GUh0C4a<05IFEP23Ph<0eMGE0820P02?Shl0E5AD06MWI`0/;2`0QhN7092@T02:RXX0?3`l06][J`1L
G5`0IVIV0;Rh^027QhL0D51@0:2PX01HF5P0BDU9065QH@2DU9@0De=C04Q8B02@T900>SXj034a<@29
RHT0T92@05iNGP2f][H0@D5102d];@1NGUh0NWYj05QHF023Ph<09bLW06MWI`2c/k<0NWYj04A4A02F
UYH0NWYj02<S8`0^;Rh0EUIF069RHP0S8b<0F5QH06m_K`0_;bl0PX:206ATI01FEUH0RHV909BDU01g
MgL0=cLg07MgM`1<C4`0;Rh^09NGU`0k>c/0V9RH06ATI00Y:BT0LG5a0:BTY027QhL0alO705YJFP0e
=CD0De=C075aL@1TI6@0<C4a04m?C`2DU9@>ooooo`0A08V9R@0m?Cd0=SHf02TY:@0M7Ad071`L01DE
5@0F5QH05ADE01<C4`0K6a/0T92@09^KV`22PX80DE5A05=CD`0^;Rh00P0@41001@0A4A403Ph>00h>
3P0B4Q805aLG00<061PH02H0:2PX03`l?00W9bL0:BTY0492@P18B4P0Q8B40<K6aP3Jf]X0alO708N7
Q`1/K6`0KFe]07=cL`1PH600EUIF03dm?@0V9RH0?3`l04E5A@1=CDd0BTY:06][J`2>SXh04a<C010@
400>3Ph05QHF06MWI`1^KVh0Y:BT09BDU01/K6`0YjNW09jNWP2GUiL03`l?00h>3P<03@d=00H0;Bd]
05mOG`0G5aL04a<C014A4@0@410201DE5@1Q05MGE`30`<00`L7107moO`2ZZZX0P82009nOW`30`<00
RHV902TY:@0E5AD04a<C071`L00S8b<0?3`l06MWI`1>CTh0De=C02DU9@1EEED0CDe=04i>CP0a<C40
8b<S030`<00Y:BT06a/K09>CT`3:b/X0[Zj^0<K6aP3>c/h0Zj^[069RHP0V9RH08B4Q01dM7@0H61P0
3Ph>030`<01:BTX08b<S03Ti>@1>CTh0GUiN0451@@1HF5P0FUYJ04m?C`0W9bL071`L01lO7`0L71`0
A4A40<O7a`2TY:@0[Zj^0861P@2?Shl0QHF505QHF01bLW80bl_;0:^[Z`1oOgl0H61P06m_K`12@T80
NGUi05mOG`1KFe/0GEeM07=cL`2FUYH0dM7A0<W9b@24Q8@0Lg=c04U9B@1PH600WYjN04I6AP1ADE40
;bl_01`L700G5aL07QhN024Q8@0S8b<092@T03/k>`1>CTh0WIfM08B4Q01;Bd/08B4Q02TY:@0301DE
5@0A01TI6@0Q8B40>c/k05ADE01bLW80IVIV04a<C01`L700YZJV051@D00S8b<0;bl_051@D022PX80
QHF50861P@1`L7000P17AdL01`1RHV80[Zj^08J6QP2?Shl0YZJV0=SHf03Shn<00P1nOWh03@1/K6`0
KFe]03Hf=P0^;Rh0Jf][05MGE`1YJFT0HV9R08F5Q@2HV9P0U9BD04e=C@0K6a/00P0M7Ad0>`0S8b<0
;Rh^04];B`0f=SH0I6AT069RHP11@D40Xj>S0;>c/`2LW9`0MgMg08Z:RP1lO7`0BTY:03@d=012@T80
;Bd]034a<@0e=CD0820P028R8P1aLG40IFEU04E5A@16ATH0@41004U9B@1RHV80KFe]05aLG01EEED0
Kfm_07emO@1/K6`0PH6108Z:RP1>CTh0UiNG065QH@10@400DE5A06IVIP1iNGT0Kfm_05YJFP0o?cl0
De=C0;Bd]03:b/X0][Jf08N7Q`1mOGd0Ti>C055AD@1fMWH0QHF50<30`02EUID0H61P013ooooo000>
ooooo`3o06=SH`1oOgl0BDU904M7A`21PH40@d=30451@@1iNGT0?clo065QH@1JFUX0:BTY030`<01Q
HF40=cLg03dm?@0a<C40IVIV03Ph>01@D500JVYZ02DU9@1XJ6P0@T920410@01KFe/0:BTY01DE5@0m
?Cd0=SHf05QHF00U9BD0;Bd]05ADE00i>CT0;bl_05aLG00K6a/0JVYZ038b<P0h>3P09BDU04e=C@1?
Cdl0MgMg05YJFP0a<C40ATI603`l?01>CTh0@d=3030`<00o?cl0G5aL051@D00/;2`0J6QX04U9B@0P
8200EUIF03Hf=P0n?Sh0Hf=S02<S8`0j>SX0OWin05UIF@16ATH0A4A405EEE@1IFET0De=C030`<01Y
JFT0EEEE02@T901IFET0:RXZ08:2PP2OWil0FUYJ02HV9P1jNWX0;Rh^09BDU02CTi<0G5aL05YJFP1W
IfL07QhN07EeM@1[Jf/0>3Ph03<c<`1@D500ADE503Ti>@2@T900Jf][04Q8B01WIfL0:2PX05YJFP0m
?Cd0P82003Xj>P0g=cL0IFEU079bLP1DE5@0IVIV03Ph>02BTY80F5QH069RHP0l?3`0Jf][08N7Q`11
@D40Bd];06a/K00O7al0IFEU05eMG@2LW9`0>SXj07AdM01eMGD0B4Q807EeM@1?Cdl0KVi^04m?C`1W
IfL0cLg=03`l?01dM7@0Rh^;03Hf=P1NGUh0QHF503@d=024Q8@0I6AT03dm?@1oOgl0MgMg03`l?01Z
JVX0L71`0410@01dM7@0CDe=069RHP2@T900Hf=S0:2PX01aLG40AdM708Z:RP2=SHd0OWin08N7Q`2O
Wil0BDU904E5A@1TI6@0ZJVY05IFEP1nOWh0TI6A04A4A01iNGT0_;bl07AdM01MGEd0VIVI04=3@`1=
CDd0?clo03Ph>01`L700FEUI08b<S01;Bd/0EeMG096AT@0a<C40GemO04=3@`1MGEd0Z:RX05=CD`0V
9RH0PX:205MGE`22PX80GUiN03Xj>P1_Kfl0clo?0820P02UYJD0Ph>307emO@14A4@0XZ:R079bLP23
Ph<0@T9208N7Q`1VIVH0DU9B06][J`21PH40IfMW0:b/[01ZJVX0@T9204Y:BP1hN7P0CDe=05IFEP2>
SXh0MWIf03lo?`1QHF40Kfm_059BDP2/[:`0O7al0410@01QHF40HV9R04A4A01]KFd0@T9208b<S02k
^k/0QhN7055AD@1HF5P0LW9b02PX:00l?3`0GUiN0492@P0Y:BT0HV9R04m?C`0/;2`0TI6A07MgM`19
BDT0@d=3000G06i^KP18B4P0<C4a09BDU020P800FUYJ08:2PP0_;bl0RHV905eMG@0k>c/0KFe]0;Rh
^01[Jf/0`l?307QhN00h>3P0C4a<06YZJP1?Cdl0<30`06UYJ@2LW9`03_ooool05`2@T900EEEE03lo
?`0j>SX0:RXZ02<S8`0N7Qh05aLG01DE5@0F5QH0>CTi06QXJ02KVi/0Xj>S06YZJP1RHV80ATI601dM
7@0@41003`l?00h>3P0D51@05QHF00805ADE05P05aLG01DE5@0E5AD06a/K03<c<`0Y:BT0GemO09NG
U`2k^k/0[:b/0<S8b02?Shl0FUYJ04M7A`16ATH0EUIF05YJFP1[Jf/0I6AT05ADE01:BTX0C4a<051@
D02:RXX0Zj^[04Y:BP0h>3P0;Bd]02XZ:P1;Bd/0@D5107=cL`0f=SH09bLW030`<02BTY80`L710::R
XP1^KVh0De=C03@d=00O7al04A4A010@401mOGd0W9bL05iNGP16ATH0=SHf0492@P1LG5`0S8b<0;Vi
^@3De=@0gMgM0;Vi^@0g=cL06QXJ02@T9017AdL03`l?00h>3P0?3`l0IVIV04M7A`0f=SH0K6a/08B4
Q01mOGd0U9BD061PH00E5AD06a/K02PX:013@d<0Fe]K04m?C`1QHF40M7Ad03hn?P1OGel0[:b/0;6a
/@2@T9008b<S02TY:@0`<300:BTY0P0S8b<0;P0U9BD0?Cdm07alO02HV9P0];Bd034a<@0L71`0Cdm?
0;Jf]P2RXZ80S8b<04m?C`13@d<0?clo04];B`2^[Zh0[jn_0<G5a@2VYZH0Xj>S0:n_[`1^KVh0Hf=S
06EUI@1fMWH0>c/k02<S8`1XJ6P0^KVi0;Jf]P1oOgl0O7al05mOG`1HF5P0FEUI07]kN`2RXZ80]KFe
09bLW01bLW80FUYJ09^KV`1OGel0F5QH03Hf=P0P820201lO7`0502@T900M7Ad07alO02`/;01GEeL0
0P2_[jl04`1hN7P0IVIV04a<C00X:2P08B4Q02LW9`0[:b/09bLW01lO7`0L71`0820P055AD@2PX:00
ZJVY0;bl_00Y:BT05aLG01<C4`0G5aL00P0E5AD0700P8200<S8b051@D01]KFd0Y:BT0<k>cP2f][H0
ZZZZ06YZJP1OGel0W9bL0<_;b`1BDU80<C4a02@T900O7al0<S8b069RHP1UIFD0IVIV06][J`1EEED0
A4A404Q8B01?Cdl0?Shn01dM7@0F5QH201LG5`0c02h^;P1MGEd0RXZ:06IVIP2ATI40Ogmo03/k>`1H
F5P0D51@02h^;P13@d<09RHV01dM7@0Y:BT0;Rh^02d];@0d=3@0=cLg03De=@1SHf<0MgMg03Ph>01:
BTX0Hf=S07emO@29RHT0P82008j>SP1]KFd0IVIV05UIF@1nOWh0CTi>06QXJ02ATI40H61P04U9B@0h
>3P0?Cdm05=CD`1lO7`0LG5a065QH@10@400ATI605aLG01_Kfl0YJFU07]kN`1IFET0?3`l0080>3Ph
00@0D51@0::RXP1@D500KFe]4?ooool000kooooo04<0FUYJ08N7Q`1CDe<0>c/k059BDP10@400Cdm?
05QHF00G5aL0@d=30820P01<C4`0=3@d03`l?00G5aL0IfMW04Y:BP1YJFT0<C4a07alO025QHD0ATI6
079bLP0Y:BT0?Shn06m_K`1PH6007alO03Ti>@0U9BD0EeMG0451@@0j>SX0?Shn00h>3P0D51@0I6AT
02TY:@1eMGD04Q8B04];B`1IFET0BTY:018B4P1lO7`0OGem04];B`1[Jf/0BTY:03<c<`1SHf<0@T92
07EeM@1gMgL0>c/k01LG5`1:BTX0@D51059BDP24Q8@0:RXZ02<S8`1=CDd0;bl_0410@025QHD0?Cdm
00809RHV01T0Fe]K06][J`1DE5@0:BTY06][J`1^KVh0<30`079bLP0Z:RX0GEeM08Z:RP0R8R808b<S
07MgM`0T92@0UIFE0;Jf]P1OGel0NWYj09>CT`0];Bd0RXZ:03hn?P0Y:BT0De=C0080JFUY03L09bLW
07AdM01IFET0>c/k07QhN01BDU80?clo02@T901]KFd0J6QX038b<P1OGel0?Cdm04M7A`1]KFd0CDe=
08V9R@0m?Cd0PH61061PH01TI6@0NGUi051@D01lO7`0Q8B405QHF01GEeL0@d=309FEU@1TI6@0JVYZ
05UIF@0g=cL0L71`05YJFP1IFET0;bl_04E5A@2PX:00?clo06a/K01gMgL0@D5109NGU`2JVYX0?clo
07moO`1:BTX0;Bd]07=cL`2:RXX0K6a/059BDP1MGEd0<30`0080Bd];07d0PX:206a/K01LG5`0[Jf]
09FEU@1UIFD0UYJF08R8R01gMgL0QhN70<30`01<C4`0AdM7061PH02XZ:P0CDe=06m_K`24Q8@0ATI6
08V9R@3@d=00X:2P06e]K@2OWil0Fe]K06e]K@0m?Cd0:RXZ06QXJ01@D500Ph>303Ph>01BDU80Rh^;
04M7A`1OGel0BTY:06m_K`2l_;`0F5QH038b<P2;Rh/0BTY:065QH@1<C4`0BTY:051@D039bLT0FEUI
061PH01IFET0HF5Q03lo?`2a/K40S8b<06e]K@0X:2P0HV9R05]KF`1GEeL0LW9b03lo?`1<C4`0VIVI
06][J`17AdL0De=C06YZJP0Z:RX0DU9B07moO`2=SHd0I6AT06IVIP1aLG40CDe=0:RXZ01cLg<0<S8b
05YJFP1GEeL0?Cdm05UIF@0_;bl0SHf=0<W9b@2GUiL0HF5Q05mOG`1:BTX0:2PX05mOG`1oOgl0Fe]K
03@d=02DU9@0@d=301/K6`1@D500Ogmo034a<@1WIfL0YZJV04];B`0M7Ad0Hf=S03`l?00];Bd0NWYj
03Hf=P2=SHd0JVYZ03lo?`1VIVH0Jf][06=SH`3?cll0I6AT03Hf=P0Z:RX0?Shn051@D00/;2`0FEUI
09FEU@0>ooooo`0907YjNP2:RXX0@d=303Lg=`0];Bd0:BTY02PX:00S8b<06ATI0080;Rh^02P0?clo
03Lg=`1?Cdl0GEeM05mOG`1GEeL0AdM701DE5@0=3@d03Ph>01@D500E5AD04Q8B010@400D51@06ATI
03/k>`10@400B4Q806=SH`3;bl/0^;Rh05UIF@0m?Cd0Ph>30<[:bP1RHV80C4a<05]KF`14A4@0=CDe
03<c<`0o?cl0HF5Q08Z:RP2a/K40b/[:0=OGe`3<c<`0;bl_0P0E5AD02`0[:b/0FEUI09:BTP38b<P0
H61P02d];@1=CDd0O7al03De=@0U9BD0EUIF0080KFe]0700Jf][069RHP1HF5P0WinO0;fm_@2BTY80
QHF508:2PP23Ph<0OWin07MgM`1eMGD0XZ:R07inOP2[Zj/0WIfM02TY:@0K6a/0?Shn01HF5P0a<C40
D51@07MgM`26QXH0RHV90:2PX032`/80X:2P08B4Q01XJ6P0HV9R04m?C`0c<c<08b<S03hn?P0e=CD0
Cdm?0;Vi^@1XJ6P0GUiN09:BTP2YZJT0/;2`03De=@11@D40BDU905UIF@1JFUX0GUiN0451@@0R8R80
7AdM03<c<`2@T900P82007UiN@26QXH0RXZ:06a/K01kNg/06QXJ01PH600E5AD06ATI05IFEP2i^KT0
/K6a0:2PX02XZ:P0V9RH02<S8`0G5aL05ADE01TI6@1=CDd0SXj>08N7Q`29RHT0[jn_06YZJP0e=CD0
;bl_03Lg=`0f=SH0>SXj04Y:BP2=SHd0_Kfm0;2`/02;Rh/0DU9B01hN7P0B4Q80A4A406m_K`0W9bL0
6ATI01`L700N7Qh0:2PX04I6AP28R8P0clo?0:2PX01JFUX0GUiN04E5A@0P82005aLG01/K6`0I6AT0
5aLG0`0J6QX01P0K6a/0U9BD0:^[Z`30`<00;Rh^01lO7`D05ADE01L05QHF01DE5@0I6AT0MGEe08f=
S@1lO7`0ATI601`L700M7Ad0?Cdm0<S8b01cLg<0Ogmo08n?S`28R8P0PH6108F5Q@1IFET0Bd];028R
8P0S8b<0@d=304U9B@0202`/;00m05aLG00Y:BT05aLG02PX:01UIFD0Jf][07inOP36a/H0clo?0410
@00X:2P0>c/k05MGE`0T92@0;Rh^03dm?@0N7Qh07alO04Y:BP0^;Rh0=cLg04I6AP26QXH0OWin0<S8
b01oOgl0U9BD09ZJVP2?Shl0P82007inOP1?Cdl0>CTi04M7A`1eMGD0S8b<06a/K02;Rh/0HF5Q038b
<P0X:2P0:b/[034a<@0i>CT0D51@06a/K00o?cl0=cLg05UIF@1WIfL0RHV908N7Q`1ZJVX0<30`02d]
;@0Y:BT0:b/[03@d=01nOWh0<C4a06EUI@0@ooooo`003_ooool0J`1JFUX0F5QH0492@P0o?cl0B4Q8
03Hf=P1MGEd0MWIf03Hf=P0o?cl0IVIV04a<C01WIfL0DU9B01<C4`0o?cl0@D5103Xj>P0820P0HV9R
0820P010@400P82002h^;P0g=cL0IVIV05YJFP17AdL0EEEE03@d=01DE5@0=cLg065QH@1IFET0@T92
02HV9P1CDe<07alO07IfMP0/;2`0Bd];04M7A`1RHV80<30`06=SH`0h>3P0;Bd]04U9B@14A4@0?Cdm
03Lg=`0X:2P0Bd];04m?C`1LG5`0>3Ph051@D00Q8B407alO05MGE`1ADE40ATI605UIF@0c<c<0MWIf
09fMW@1YJFT0FEUI034a<@1@D500YJFU07MgM`0j>SX0MWIf0410@00Y:BT0OWin030`<01PH600LW9b
065QH@0Q8B40SXj>02LW9`28R8P0U9BD04I6AP15ADD0Lg=c03Ph>02:RXX0?3`l01TI6@0_;bl0@D51
06MWI`0/;2`0P82004=3@`0Z:RX0Jf][03lo?`14A4@0<30`07alO01PH600;bl_0080E5AD0:T0A4A4
08R8R01QHF40RHV9055AD@1HF5P0Cdm?05iNGP1lO7`0?Shn04I6AP1^KVh0IFEU051@D01FEUH0Y:BT
06UYJ@1bLW80JFUY04];B`21PH40Hf=S06QXJ01DE5@0GemO0<O7a`0m?Cd0JFUY08>3P`0c<c<0IVIV
08n?S`10@400OGem06][J`0b<S80MgMg09:BTP1^KVh0EUIF05eMG@0m?Cd0Kfm_0451@@1PH600K6a/
069RHP22PX80R8R806IVIP2MWId0T92@08N7Q`2@T900[Zj^05QHF01DE5@0KFe]0;Vi^@1UIFD0PH61
07=cL`0];Bd0I6AT0:ZZZP2NWYh0Hf=S09>CT`0_;bl0KFe]05MGE`0d=3@0Ogmo05eMG@29RHT0DE5A
06UYJ@2VYZH0FEUI08Z:RP1]KFd0S8b<0=3@d01RHV8092@T07IfMP1UIFD0LG5a04Q8B00e=CD0HF5Q
0<_;b`1dM7@0Q8B406e]K@1OGel0@4100;Zj^P2l_;`0OGem03lo?`1WIfL0BDU905aLG01gMgL0De=C
05]KF`2QXJ40IVIV0492@P1/K6`0N7Qh04E5A@1hN7P0SXj>03Ti>@13@d<0X:2P09ZJVP1MGEd0a<C4
07IfMP0h>3P0M7Ad08J6QP1>CTh0SXj>03/k>`1_Kfl0e=CD0:b/[01KFe/0NWYj05QHF00I6AT0<C4a
04i>CP0P82008B4Q06YZJP1jNWX0;bl_05YJFP26QXH0Cdm?0410@01[Jf/0K6a/02d];@1lO7`0FUYJ
02/[:`1JFUX0=3@d08R8R01kNg/0BDU908V9R@1HF5P0EeMG0:b/[01>CTh0@T9203hn?P1HF5P0H61P
02HV9P1TI6@0UIFE00kooooo00h0MGEe08n?S`1BDU80<C4a02LW9`0T92@09BDU02XZ:P19BDT0CTi>
03Ph>00J6QX04Q8B01dM7@80@T9201D0IFEU04i>CP0o?cl04Q8B00h>3P0=3@d03Ph>01LG5`0I6AT0
7QhN02TY:@0d=3@0<C4a05IFEP32`/80XZ:R04I6AP0Y:BT0>3Ph06QXJ02UYJD00P2ATI408P1QHF40
>CTi020P800L71`08B4Q04];B`1BDU80DE5A065QH@1_Kfl0^[Zj0;bl_023Ph<0N7Qh08F5Q@1mOGd0
OWin0:2PX02LW9`0Vi^K0:RXZ020P800?3`l02<S8`0N7Qh0=3@d055AD@13@d<0;2`/02DU9@2<S8`0
>3Ph02@T900b<S8203Ti>@0N03Xj>P14A4@0U9BD0:6QX@1SHf<0>3Ph075aL@2PX:00G5aL06ATI01E
EED0Fe]K03@d=00e=CD0=SHf04M7A`1LG5`0IFEU08Z:RP1YJFT0BDU902@T900];Bd0AdM706a/K01a
LG40KFe]055AD@2UYJD0Ng]k0P1<C4`01@20P800g=cL08j>SP16ATH0>CTi0080@T9200d0?Cdm038b
<P0N7Qh06a/K02DU9@1/K6`0F5QH06UYJ@2e]KD0RXZ:08:2PP29RHT071`L00805aLG03h08b<S07=c
L`2e]KD0Z:RX0:f][@3=cLd0F5QH05]KF`14A4@09RHV01LG5`0K6a/0NWYj05ADE00l?3`0Ti>C0:f]
[@1:BTX0@D5102l_;`0U9BD0:RXZ03De=@10@400QHF50:NWY`2h^;P0UiNG02/[:`0B4Q804a<C01TI
6@1RHV80?clo02/[:`0V9RH0>c/k07UiN@2XZ:P0_Kfm0<o?c`2CTi<0M7Ad06][J`1=CDd0@41002`/
;00J6QX06a/K01`L700H61P06ATI02l_;`1dM7@0Shn?0<K6aP16ATH0:BTY02HV9P0G5aL04A4A01<C
4`805ADE03D05aLG020P801TI6@0UYJF0;Ng]`1OGel0820P01`L7012@T80K6a/01lO7`0Q8B40:BTY
05IFEP25QHD0EUIF06][J`1@D5008B4Q01/K6`0Q8B40ATI603`l?00V9RH0<c<c06e]K@1GEeL0HF5Q
06i^KP2f][H0^[Zj0:b/[03=cLd0G5aL024Q8@0f=SH0GemO02PX:00Q8B40?clo03De=@0N7Qh0De=C
05ADE012@T80K6a/0:2PX02c/k<0bl_;06i^KP1jNWX0SHf=07YjNP0208N7Q`80<30`01`0=SHf03`l
?02_[jl0_[jn07EeM@1gMgL0FUYJ03De=@0a<C40<c<c03Xj>P17AdL0Ng]k03@d=01MGEd0E5AD08f=
S@24Q8@0B4Q805MGE`0k>c/0?Cdm0410@00o?cl0>3Ph07IfMP11@D40L71`4?ooool000kooooo0480
HV9R09NGU`1=CDd0BDU9079bLP13@d<0ADE508>3P`12@T80Cdm?03Ti>@0X:2P0@d=306EUI@0`<300
=cLg05MGE`1dM7@0<30`05IFEP19BDT0;bl_06i^KP0j>SX0De=C03lo?`0P82006QXJ03Lg=`0_;bl0
CTi>01hN7P0[:b/0BTY:03De=@0b<S80CDe=018B4P1EEED0?Cdm03Ti>@0V9RH0CTi>03Lg=`1/K6`0
CTi>03Lg=`1@D500=CDe03Ph>014A4@0@41007inOP1dM7@0OGem0492@P1HF5P0ADE502DU9@1ZJVX0
GEeM051@D01RHV80:BTY03Hf=P1MGEd205MGE`0I0451@@0h>3P0EeMG06IVIP0m?Cd0Ogmo06i^KP0c
<c<0Ph>3030`<01PH600MGEe06IVIP0R8R80Ti>C02TY:@2:RXX0LW9b05]KF`0o?cl0Rh^;02/[:`26
QXH0CTi>04U9B@0203Ti>@0j079bLP0a<C40O7al05QHF013@d<0Lg=c02`/;013@d<0<S8b07alO01M
GEd0=CDe05=CD`1EEED0De=C09bLW01ZJVX0QHF506QXJ01/K6`0EUIF05QHF02<S8`0D51@05]KF`1_
Kfl0L71`05eMG@1GEeL0YjNW06e]K@1bLW80Ng]k04e=C@1XJ6P0E5AD08Z:RP1TI6@0GEeM0<K6aP0m
?Cd0S8b<08R8R00i>CT0OWin08F5Q@17AdL0TY:B07EeM@0`<300H61P08J6QP0_;bl0QXJ608^;R`10
@400Lg=c04];B`80GemO0240NWYj0861P@26QXH0GUiN09nOW`23Ph<0SXj>08R8R02^[Zh0De=C05aL
G02>SXh0^;Rh08>3P`2c/k<0Z:RX05aLG01oOgl0aLG508>3P`1bLW80[Jf]03/k>`1>CTh0C4a<03<c
<`1eMGD0JVYZ09fMW@1BDU80M7Ad08J6QP0Z:RX00P1mOGd0401oOgl0_;bl08>3P`0i>CT0PX:205MG
E`1oOgl0NGUi04m?C`10@400YZJV06a/K02NWYh0HF5Q05mOG`0Y:BT207emO@0k08j>SP15ADD0Jf][
03/k>`0^;Rh0J6QX07]kN`1YJFT0/[:b03Lg=`0/;2`0Cdm?07=cL`16ATH0QhN708f=S@1JFUX0=SHf
04m?C`1aLG40EeMG0=3@d01MGEd04a<C04];B`1WIfL0G5aL0:>SX`0i>CT0Jf][0:n_[`1FEUH0<S8b
07alO01hN7P09BDU03/k>`1MGEd0BTY:02LW9`2EUID0K6a/03dm?@1LG5`0S8b<051@D01>CTh0TY:B
08B4Q00o?cl0OWin071`L00k>c/0IFEU03<c<`2JVYX0NGUi04Q8B02GUiL00P1WIfL02@2GUiL0Kfm_
04Q8B017AdL0JVYZ06EUI@0/;2`0IVIV09VIV@0>ooooo`0;06ATI023Ph<0^KVi05IFEP0j>SX0;Rh^
03De=@1KFe/0NGUi04U9B@0G5aL00P0A4A403P0>3Ph05QHF0492@P1CDe<0JFUY05mOG`0b<S804a<C
014A4@0D51@061PH01DE5@0N7Qh07alO0P0U9BD0X@1jNWX0Ti>C07QhN01YJFT0>c/k06][J`1VIVH0
?clo01LG5`0X:2P0H61P07alO01bLW80@410028R8P0_;bl0=CDe03<c<`0o?cl0>3Ph08j>SP3Hf=P0
VYZJ05eMG@1<C4`0>SXj03hn?P1BDU80XJ6Q08>3P`2UYJD0RHV9071`L01RHV80EEEE04E5A@1>CTh0
?clo03Lg=`0`<300LG5a03/k>`14A4@0@T9204Q8B017AdL0A4A404E5A@1bLW80[Jf]05mOG`1?Cdl0
9bLW07]kN`3Kfm/0X:2P07]kN`0O7al061PH01lO7`0R8R80410@02LW9`1JFUX0PH6107YjNP20P800
EeMG05IFEP1KFe/0Hf=S05eMG@1SHf<0T92@08b<S02`/;00UYJF0;>c/`33`l<0e]KF08:2PP1=CDd0
?Shn03`l?00o?cl0E5AD065QH@0Z:RX07AdM03@d=01^KVh0FEUI07=cL`2LW9`0Ti>C0;Jf]P2a/K40
8b<S01dM7@0T92@0Hf=S09FEU@2CTi<0[Jf]0:NWY`2FUYH0>CTi02/[:`19BDT0=cLg01hN7P0O7al0
H61P05EEE@0b<S80Ph>309BDU01:BTX0D51@065QH@1JFUX0IVIV05ADE00j>SX0HV9R0:ZZZP36a/H0
`<3007MgM`15ADD08B4Q018B4P0G5aL0JFUY03Xj>P0];Bd0OWin09bLW01eMGD0]kNg0<30`01KFe/0
>SXj03@d=01GEeL0KVi^07MgM`1]KFd0EEEE03@d=00P82006QXJ03/k>`1DE5@0]KFe0=CDe02k^k/0
J6QX02/[:`0a<C405ADE00807alO00/0;2`/03<c<`1<C4`0^[Zj0<g=c@1DE5@0;Rh^04];B`0[:b/0
ATI602/[:`0201`L700:02@T900g=cL0NGUi05UIF@1CDe<0I6AT02PX:00S8b<06a/K02DU9@80P820
01@0SHf=0;:b/P2?Shl0P82008b<S01>CTh0@41005EEE@2f][H0MgMg03Lg=`0a<C40MWIf02HV9P0L
71`09bLW04I6AP0/;2`0JFUY092@T080S8b<02L0X:2P0;Jf]P3:b/X0WYjN08F5Q@2NWYh0[:b/0;jn
_P36a/H0?clo02LW9`0];Bd0?3`l0:VYZ@2SXj<0QHF508Z:RP2=SHd0`<300:6QX@21PH40NGUi08n?
S`2`/;00GemO055AD@1/K6`0WYjN05UIF@0/;2`0Ph>305iNGP13@d<0@D5104E5A@1DE5@0U9BD04i>
CP1cLg<04?ooool00?oooooooooooomJooooo`00oooooooooooooe[ooooo003oooooooooooooF_oo
ool00?oooooooooooomJooooo`00oooooooooooooe[ooooo003oooooooooooooF_ooool000kooooo
02@0Zj^[05iNGP1NGUh0P8200;Zj^P1^KVh0>CTi05aLG01nOWh0WinO04I6AP0`<300DU9B06UYJ@1V
IVH0/K6a04e=C@1>CTh0Ph>305YJFP2VYZH0c<c<04i>CP18B4P0[Zj^08f=S@2[Zj/0QHF503<c<`1M
GEd0[jn_05YJFP20P800?Cdm03De=@1WIfL206a/K02707moO`0I6AT0BDU905eMG@1/K6`0DU9B07=c
L`0T92@0D51@06UYJ@1QHF40GEeM04E5A@1OGel0KVi^09RHV00c<c<0AdM704A4A01JFUX0I6AT04=3
@`0b<S80JVYZ04A4A01VIVH0Q8B403Ph>00l?3`0DU9B04=3@`1QHF40YZJV03<c<`0o?cl0Ng]k0492
@P0^;Rh0VYZJ04=3@`0_;bl0B4Q804e=C@1ADE40RXZ:02l_;`13@d<0F5QH05IFEP1VIVH0WYjN04Q8
B01FEUH0EeMG07AdM01YJFT0VIVI03Xj>P1ADE40IVIV06ATI02_[jl0EEEE02LW9`1PH600EEEE06a/
K02TY:@0?clo024Q8@15ADD0KFe]0:ZZZP2g]kL0=CDe0451@@1>CTh0GemO0:RXZ033`l<0>CTi04=3
@`1KFe/0JVYZ0861P@1oOgl0=SHf04i>CP1ZJVX0MGEe06ATI02BTY80ADE5065QH@2VYZH0NWYj08N7
Q`2MWId0>SXj07UiN@1lO7`0J6QX0:2PX037alL0@41006][J`2LW9`0UiNG0:FUY@2ATI40ATI6075a
L@31`L40Q8B40<W9b@0b<S80@T9208>3P`2KVi/0G5aL096AT@0b<S80BTY:06IVIP26QXH0G5aL0;Fe
]@1DE5@0D51@07EeM@1iNGT0OGem08f=S@14A4@0F5QH0080HF5Q06L0E5AD04Q8B01?Cdl0E5AD07Ad
M016ATH0=SHf04E5A@1?Cdl0LW9b07QhN019BDT0MgMg059BDP2:RXX0/k>c04U9B@15ADD0Hf=S03lo
?`2QXJ40K6a/04=3@`1aLG40CDe=06YZJP2WYjL0>c/k07QhN01WIfL0R8R80;:b/P2l_;`0EeMG08J6
QP1ADE40XZ:R079bLP0Z:RX0EeMG061PH01RHV80][Jf04a<C00a<C40O7al06m_K`2b/[80]KFe04U9
B@1:BTX0UIFE075aL@2PX:00K6a/0492@P1@D500N7Qh07AdM02=SHd0BDU90451@@1BDU80HF5Q079b
LP2ZZZX0H61P03/k>`1=CDd0KFe]09ZJVP33`l<0JFUY03/k>`1bLW80P82009JFUP3Fe]H0JVYZ03dm
?@1jNWX0M7Ad065QH@32`/80AdM703hn?P1]KFd0IVIV0:JVYP3<c<`0Cdm?02XZ:P1]KFd0SXj>09VI
V@3Cdm<0FEUI01`L701SHf<0MWIf055AD@2f][H0Hf=S00kooooo00X0BDU906ATI02CTi<0]KFe08R8
R01;Bd/0JFUY05mOG`1EEED0I6AT0P1CDe<0D01KFe/0OWin06=SH`1aLG40U9BD0:6QX@21PH40IVIV
05UIF@0`<300B4Q80451@@0h>3P0EUIF02/[:`1?Cdl0>3Ph05]KF`1NGUh0?Cdm03De=@14A4@0?clo
05MGE`1?Cdl0?Cdm04e=C@10@400>CTi02h^;P1;Bd/0DE5A03lo?`1FEUH0D51@06][J`1?Cdl0<30`
04a<C013@d<0D51@04e=C@1@D500@D5107EeM@1?Cdl0CDe=0410@01FEUH0DE5A04A4A01:BTX0CTi>
07UiN@1HF5P0IFEU09NGU`18B4P0H61P05EEE@1[Jf/0HV9R055AD@1ZJVX0OGem07MgM`17AdL0JFUY
08F5Q@24Q8@0K6a/055AD@1=CDd0LG5a071`L01]KFd0LG5a05MGE`1jNWX0IFEU0P1dM7@03023Ph<0
IFEU04m?C`1fMWH0NWYj08B4Q01YJFT0Ng]k0820P01XJ6P0Vi^K06QXJ080QXJ602l0X:2P092@T01`
L700NWYj0:>SX`1TI6@0SHf=08^;R`2_[jl0EeMG07IfMP1OGel0Hf=S05aLG01VIVH0PH610492@P2J
VYX0M7Ad07moO`2k^k/0Ogmo0<K6aP0a<C40Ng]k0861P@0f=SH0C4a<05aLG01<C4`0?clo05mOG`16
ATH0@T9205IFEP1oOgl0Cdm?04Y:BP18B4P0KVi^05]KF`1RHV80CTi>05YJFP1KFe/0@410065QH@02
061PH01i01lO7`1WIfL0I6AT06=SH`1bLW80B4Q804m?C`1nOWh0LW9b08R8R02/[:`09bLW04U9B@1[
Jf/0BTY:06UYJ@14A4@0GUiN04m?C`19BDT0Cdm?069RHP0Z:RX0LW9b05UIF@1OGel0<C4a04I6AP1G
EeL0=cLg04m?C`1KFe/0QhN706IVIP21PH40NWYj02/[:`19BDT0EEEE055AD@0o?cl0QhN7059BDP22
PX80MWIf08R8R010@400Kfm_06=SH`1JFUX0Q8B405YJFP1LG5`0D51@065QH@1ZJVX0EEEE06QXJ01R
HV80AdM7051@D01LG5`0LG5a07AdM01nOWh0W9bL0;Zj^P1XJ6P0/;2`0820P02DU9@0P8200:6QX@1]
KFd0VYZJ06ATI01aLG40NWYj09FEU@20P800E5AD065QH@1gMgL0MWIf07emO@2>SXh0GEeM075aL@2:
RXX0N7Qh08B4Q01[Jf/0Ng]k06YZJP2GUiL0NGUi0<30`01cLg<0JFUY0:NWY`2NWYh0;2`/06MWI`1b
LW80KVi^0:>SX`12@T80F5QH04E5A@1QHF40QhN709BDU023Ph<0N7Qh08B4Q01FEUH0M7Ad08V9R@2=
SHd0[Zj^0:n_[`0@ooooo`003_ooool0>P1kNg/0PX:207IfMP1FEUH0Q8B40820P019BDT0MWIf03hn
?P1CDe<0L71`06EUI@1fMWH0:RXZ02DU9@2@T900LW9b05=CD`19BDT07QhN03`l?02EUID0J6QX0410
@00g=cL0FEUI06UYJ@2:RXX0DE5A05aLG01;Bd/0DU9B0;>c/`1UIFD09bLW0492@P1ADE40NWYj0;Vi
^@0];Bd08b<S071`L02CTi<0LW9b0;:b/P0[:b/0;bl_07EeM@1RHV80VIVI07emO@0P8200?Cdm07mo
O`1ADE40L71`05YJFP0n?Sh2061PH01H05MGE`2a/K40<S8b069RHP1XJ6P0Cdm?07YjNP21PH408B4Q
04E5A@25QHD0O7al0820P02h^;P092@T03hn?P1RHV80Q8B407moO`2TY:@0<30`03Hf=P1]KFd0DE5A
075aL@2RXZ805aLG0492@P14A4@0EEEE0;Zj^P23Ph<08R8R05ADE01fMWH0J6QX0:6QX@0W9bL0>SXj
07UiN@2?Shl0Bd];05MGE`0X:2P0?clo075aL@2[Zj/0GemO07=cL`0k>c/0DE5A09RHV01mOGd0IfMW
07AdM00j>SX0EUIF0::RXP1jNWX0GUiN08N7Q`10@400IFEU0:JVYP1DE5@0Fe]K071`L01;Bd/0ADE5
0;fm_@1=CDd0IfMW06][J`12@T80?Shn09nOW`0j>SX0Rh^;05ADE00i>CT0@d=309FEU@1OGel0E5AD
034a<@1ADE40HV9R09ZJVP80FUYJ03<0<30`07QhN02PX:00OGem03/k>`1cLg<0@T92061PH03;bl/0
Shn?034a<@1TI6@0L71`07AdM03Lg=`0IfMW03Xj>P1/K6`0LW9b0<K6aP3UiND0>3Ph03De=@1dM7@0
KFe]092@T02XZ:P0<30`06IVIP1^KVh0Kfm_08Z:RP29RHT0AdM706a/K02@T900S8b<0<o?c`1SHf<0
D51@09jNWP1YJFT0];Bd0;Ng]`0a<C40HV9R0;fm_@2VYZH0]kNg05]KF`0c<c<00P24Q8@0B`2n_[h0
GUiN03De=@1/K6`0YJFU09BDU01XJ6P0A4A405eMG@1oOgl0Bd];0:n_[`1aLG40AdM70861P@2BTY80
Jf][07EeM@0g=cL0IVIV09RHV028R8P0`l?30;bl_01JFUX0LW9b0;no_`20P800^;Rh06m_K`0n?Sh0
IVIV07moO`28R8P0[Zj^038b<P1DE5@0F5QH08B4Q021PH40`L7103/k>`17AdL0^;Rh0;Bd]01oOgl0
Xj>S04e=C@1ADE40WinO0:^[Z`1NGUh0=SHf0492@P1;Bd/0[Jf]0:>SX`16ATH0G5aL055AD@1EEED0
TY:B0861P@0g=cL0DE5A0410@01=CDd0V9RH0:b/[01@D500BDU905=CD`1[Jf/0L71`05eMG@0>oooo
o`0804i>CP1GEeL0bl_;0<o?c`2b/[80E5AD07EeM@28R8P206][J`0507EeM@1dM7@0I6AT07UiN@1X
J6P00P1jNWX04P1UIFD0FUYJ04m?C`1LG5`0@d=3055AD@1KFe/0AdM706=SH`1:BTX0HV9R04a<C01Z
JVX0Ogmo04];B`0k>c/0J6QX03Ph>080GemO0<X0=SHf06][J`1;Bd/0<C4a04=3@`1QHF40FEUI04E5
A@1dM7@09bLW05aLG01NGUh0De=C06EUI@1RHV80:RXZ04e=C@1jNWX0MWIf05YJFP16ATH0D51@04=3
@`1LG5`0DU9B05ADE013@d<0HV9R061PH01dM7@0TI6A0;Ng]`1JFUX0Z:RX0;Jf]P23Ph<0R8R808:2
PP1MGEd0Ogmo0:ZZZP1WIfL0UiNG0;:b/P2`/;00QXJ607=cL`1LG5`0RHV908B4Q00m?Cd0HF5Q04Q8
B01RHV80@41003lo?`1TI6@0J6QX0410@01<C4`0FUYJ04=3@`1ADE40CTi>05iNGP1=CDd0D51@06IV
IP0m?Cd0FEUI08:2PP1PH600E5AD03lo?`1dM7@0OWin075aL@2PX:00N7Qh0:6QX@2?Shl0X:2P07Ad
M028R8P0Rh^;06][J`1_Kfl0AdM706ATI01_Kfl0VYZJ0;Fe]@1PH600FUYJ04Q8B01aLG40QHF508N7
Q`2<S8`0FUYJ075aL@18B4P0K6a/059BDP1EEED0BTY:04a<C01=CDd0ADE503/k>`1WIfL0GUiN05YJ
FP1<C4`0HV9R03lo?`16ATH0D51@0410@01bLW808B4Q08:2PP1oOgl0@T9205YJFP1DE5@0=cLg055A
D@1>CTh0Bd];06MWI`0];Bd0Kfm_079bLP14A4@0N7Qh06EUI@22PX80G5aL061PH00n?Sh0EUIF038b
<P2b/[80L71`069RHP1HF5P0GemO08F5Q@1ADE40HV9R075aL@2HV9P0B4Q808n?S`2:RXX0CDe=079b
LP1dM7@0MWIf04M7A`1fMWH0Lg=c0:j^[P1lO7`0IfMW034a<@1cLg<0IFEU04E5A@1>CTh0A4A405MG
E`1NGUh0HV9R06EUI@1XJ6P0MGEe06][J`1LG5`0GemO06a/K01YJFT0CTi>05YJFP3Hf=P0^KVi07Ee
M@1XJ6P0YjNW0<G5a@1oOgl0[jn_096AT@2NWYh0FUYJ07QhN02DU9@0XJ6Q08>3P`1RHV80RHV90P2_
[jl09@1FEUH0GemO05=CD`1hN7P0Ti>C08R8R01GEeL0K6a/0:ZZZP1LG5`0Rh^;07moO`2FUYH0ZZZZ
079bLP1CDe<0Lg=c03Ph>02^[Zh0UYJF0861P@2m_Kd0De=C07emO@15ADD0@T9206UYJ@25QHD0GEeM
07emO@1OGel0ADE504e=C@2:RXX0K6a/08Z:RP2[Zj/04?ooool000kooooo02`0MWIf079bLP2PX:00
Fe]K05YJFP1oOgl0Rh^;0861P@14A4@0?Cdm06EUI@1hN7P0Y:BT03lo?`0M7Ad0;Rh^07QhN01dM7@0
D51@01/K6`0X:2P0De=C07EeM@2OWil0Cdm?03hn?P1ZJVX0O7al07inOP2[Zj/0>CTi03/k>`1TI6@0
XJ6Q059BDP2UYJD0ATI603`l?01bLW80NGUi05IFEP2GUiL0JFUY03Lg=`80MGEe0>P0K6a/08N7Q`0e
=CD0?Cdm09FEU@1FEUH0J6QX04A4A00L71`0;bl_06UYJ@17AdL0H61P01/K6`0V9RH0De=C06EUI@1:
BTX0GemO028R8P1BDU80PH6104Q8B00o?cl0CTi>04E5A@1FEUH0U9BD03Xj>P0];Bd0GemO03De=@1C
De<0TY:B05UIF@0[:b/0EUIF06a/K027QhL0XZ:R05MGE`15ADD0L71`04U9B@2CTi<0Q8B4030`<014
A4@0GUiN061PH02UYJD0FUYJ02/[:`1mOGd0Ng]k05EEE@2DU9@0<c<c03De=@1NGUh0MgMg069RHP2D
U9@0Cdm?04U9B@12@T80Q8B407UiN@2o_kl0@D5104M7A`1hN7P0I6AT07emO@2h^;P0FUYJ03`l?02N
WYh0Kfm_07IfMP2GUiL0=SHf02/[:`2CTi<0Ng]k08R8R02<S8`0<30`05UIF@1gMgL0KFe]0:RXZ01_
Kfl0=CDe06e]K@24Q8@0Ph>30;6a/@0];Bd0>SXj06MWI`1[Jf/0OWin0:6QX@0[:b/0DU9B06IVIP2Y
ZJT0Ph>309JFUP0o?cl0DU9B07IfMP33`l<0N7Qh0<30`01VIVH0CDe=09fMW@3MgMd0Lg=c09fMW@1@
D500MWIf0<;2`P38b<P0IVIV096AT@13@d<0IVIV09FEU@1ZJVX0OGem08>3P`0`<300EeMG09JFUP1d
M7@0P820030`<01@D500Zj^[09^KV`2>SXh0V9RH03Ph>01>CTh0W9bL06e]K@1_Kfl0=SHf069RHP2=
SHd0UYJF04i>CP1MGEd0I6AT0<g=c@2OWil0CTi>09:BTP1NGUh0Ng]k0<71`@1GEeL0MGEe07QhN01;
Bd/0KVi^08N7Q`0m?Cd0Jf][04M7A`27QhL0alO705EEE@2CTi<0HV9R03@d=01aLG40a<C409RHV01^
KVh0IVIV05UIF@24Q8@0^KVi06ATI01_Kfl0@D5107]kN`3Bd]80IfMW04E5A@1ZJVX0IfMW08b<S03@
d=00FUYJ03/k>`23Ph<0NGUi08B4Q03Bd]80GUiN03Ti>@1gMgL0U9BD0:NWY`1YJFT0:BTY04U9B@1a
LG40XJ6Q06MWI`2LW9`0@410055AD@1bLW80Zj^[06e]K@2UYJD0ADE503/k>`1KFe/0Shn?09JFUP2A
TI40Bd];02d];@19BDT0De=C3_ooool0F`1@D500GEeM08B4Q01`L700W9bL07MgM`1eMGD0IfMW0492
@P1^KVh0M7Ad061PH01GEeL0QhN707YjNP1fMWH0IfMW079bLP1nOWh0GUiN04U9B@0h>3P0HV9R04M7
A`0i>CT0K6a/055AD@0g=cL0;Rh^07emO@1DE5@0>c/k04Y:BP2EUID0Fe]K05mOG`1TI6@0AdM7061P
H01;Bd/0>CTi03Ph>01VIVH0HF5Q04a<C01VIVH0@d=307UiN@1XJ6P0DE5A092@T02CTi<0BTY:06i^
KP2GUiL0O7al07]kN`1aLG40QXJ604Y:BP28R8P0CTi>055AD@1:BTX0AdM706m_K`20P800VYZJ0:^[
Z`0m?Cd0QhN70<?3``1kNg/0C4a<04];B`1@D500QhN707EeM@12@T80GemO07emO@2h^;P0G5aL05MG
E`1CDe<0Cdm?059BDP1:BTX0C4a<0451@@1EEED00P1FEUH0W`1fMWH0IFEU04];B`1DE5@0Hf=S051@
D01NGUh0EeMG07EeM@1KFe/0IVIV07AdM01EEED0MWIf0:VYZ@2UYJD0J6QX0451@@1bLW80O7al04M7
A`1/K6`0N7Qh071`L01?Cdl0J6QX05eMG@1;Bd/0@41005IFEP2BTY80>3Ph08j>SP1fMWH0N7Qh0;Vi
^@1EEED0RHV903Ti>@1RHV80T92@05eMG@1QHF40Cdm?06m_K`1lO7`0MGEe05QHF02>SXh0Lg=c05AD
E01^KVh0FEUI0451@@1fMWH0LW9b06UYJ@1CDe<0EeMG03hn?P1[Jf/0K6a/051@D01^KVh0EEEE0<W9
b@32`/80HV9R05aLG01HF5P08R8R06UYJ@1/K6`0G5aL092@T00O7al0?Cdm051@D00m?Cd0HF5Q0410
@02>SXh0GemO04Y:BP1HF5P0Kfm_01dM7@1QHF40BDU908j>SP16ATH0@D51051@D00d=3@0PH6107Ui
N@1nOWh0EUIF06][J`1cLg<0FUYJ04Y:BP1KFe/0Kfm_05EEE@1bLW80DE5A08F5Q@2OWil0PH6104a<
C02=SHd0Kfm_03lo?`1KFe/0E5AD06m_K`1BDU80EUIF08B4Q01UIFD0KVi^07]kN`1IFET0GemO06m_
K`1aLG40Fe]K06=SH`2=SHd0Ogmo04Y:BP1/K6`0RHV90820P02GUiL0Zj^[05YJFP1fMWH0=CDe06IV
IP1]KFd0N7Qh08j>SP2?Shl0U9BD05YJFP1^KVh0JVYZ08j>SP0n?Sh0NWYj09RHV01ZJVX0U9BD08j>
SP2MWId0[Jf]0:b/[00206e]K@0H08j>SP22PX80Hf=S06MWI`1EEED0MWIf07=cL`1mOGd0Shn?04Q8
B01/K6`0I6AT0492@P1_Kfl0NGUi06a/K02SXj<0Shn?04M7A`1lO7`0Zj^[07MgM`2QXJ40b<S84?oo
ool000kooooo07T0Xj>S06][J`1eMGD0Ogmo0:JVYP22PX80=CDe07IfMP26QXH0Ph>302XZ:P13@d<0
H61P06MWI`19BDT0WinO04M7A`1PH600SXj>04=3@`28R8P0VIVI03Ph>01FEUH0][Jf065QH@2=SHd0
Ogmo04A4A02GUiL0U9BD05MGE`2BTY80DE5A02h^;P1SHf<0KVi^05UIF@1]KFd0?3`l03lo?`1_Kfl0
TI6A059BDP1iNGT0>SXj04Q8B022PX80NWYj05iNGP15ADD0F5QH07YjNP2=SHd0BDU905mOG`13@d<0
HV9R08J6QP14A4@0ATI606IVIP19BDT0KVi^08R8R016ATH0CDe=07AdM00l?3`0I6AT07]kN`0a<C40
BDU907IfMP19BDT0@41009jNWP1EEED0BDU906UYJ@16ATH0DU9B09:BTP0b<S80DU9B05mOG`18B4P0
FEUI08R8R019BDT0FUYJ061PH00h>3P0IfMW0:NWY`19BDT0HF5Q061PH01:BTX0V9RH075aL@0l?3`0
F5QH06=SH`1^KVh0Ogmo0410@01LG5`0LG5a05UIF@1iNGT0WYjN03lo?`1;Bd/0Kfm_06QXJ021PH40
U9BD03Xj>P1BDU80JFUY0080K6a/09/0RXZ:0410@01ADE40KFe]06][J`1OGel0KVi^04Q8B01XJ6P0
NGUi05MGE`1hN7P0Lg=c04a<C01mOGd0OWin055AD@1gMgL0Zj^[04E5A@21PH40Q8B405mOG`2ATI40
SXj>0492@P21PH40S8b<065QH@2VYZH0Kfm_04e=C@2GUiL0ZZZZ06MWI`2c/k<0CDe=03lo?`23Ph<0
Shn?07AdM02GUiL0AdM705MGE`2k^k/0L71`07alO026QXH0EEEE06][J`34a<@0QXJ607=cL`1GEeL0
BTY:07emO@33`l<0K6a/05MGE`1NGUh0Jf][09>CT`1lO7`0DU9B061PH01`L700Ti>C0<K6aP1ZJVX0
MWIf0:VYZ@1WIfL0Shn?08j>SP17AdL0TY:B05mOG`1YJFT0R8R804];B`1cLg<0Ogmo06IVIP2b/[80
Jf][05]KF`2EUID0JFUY092@T02=SHd0@T9206EUI@1eMGD0LG5a0<30`01HF5P0@410075aL@1PH600
YZJV08Z:RP0k>c/0@T9209NGU`1^KVh0UIFE05eMG@1<C4`0M7Ad09nOW`1/K6`0XZ:R05aLG013@d<0
Rh^;07EeM@1WIfL0XJ6Q06e]K@1PH600TI6A07IfMP1^KVh0][Jf06i^KP1:BTX0NWYj08>3P`20P800
];Bd06MWI`1HF5P0Ti>C04E5A@1RHV80V9RH04M7A`0m?Cd0M7Ad05QHF027QhL0VIVI03/k>`16ATH0
L71`06a/K01oOgl0SXj>03hn?P0];Bd0HV9R07YjNP1>CTh0SHf=04I6AP0>ooooo`1004i>CP1OGel0
_Kfm0<O7a`2/[:`0DE5A06ATI01iNGT0DU9B071`L01cLg<0FEUI05EEE@24Q8@0De=C075aL@25QHD0
V9RH06][J`1=CDd0E5AD02d];@13@d<0@T9203hn?P1PH600:b/[034a<@0b<S80EeMG06MWI`0e=CD0
A4A406YZJP1BDU80C4a<05eMG@0l?3`0FEUI04Y:BP0e=CD0<S8b04m?C`1BDU80A4A405QHF013@d<0
JFUY069RHP13@d<0DU9B05IFEP14A4@0DU9B05UIF@1DE5@0JVYZ04a<C01TI6@0A4A405IFEP1?Cdl0
D51@0492@P80F5QH0;/0Lg=c0820P02<S8`0H61P07YjNP27QhL0MgMg07IfMP1TI6@0EEEE0820P02e
]KD0HV9R071`L01bLW80YZJV06m_K`1=CDd0CTi>05mOG`1WIfL0BDU905QHF01ADE40Fe]K051@D01A
DE40IFEU05UIF@18B4P0@41004a<C017AdL0FEUI051@D01HF5P0DE5A04M7A`1JFUX0EEEE05aLG01b
LW80Jf][05mOG`11@D40DU9B05mOG`1RHV80OWin06UYJ@1ZJVX0N7Qh07=cL`1MGEd0EEEE05mOG`1E
EED0P82003`l?023Ph<0EEEE07]kN`2VYZH0Hf=S09>CT`1:BTX0Fe]K06EUI@1ZJVX0K6a/04M7A`1A
DE40D51@06UYJ@1GEeL0KVi^07AdM01ADE40J6QX05eMG@1UIFD0UIFE08F5Q@1YJFT0Hf=S06YZJP1E
EED0;Rh^05EEE@1MGEd0OWin05MGE`1aLG40IfMW05aLG01GEeL0LW9b04U9B@1cLg<0PX:206a/K01j
NWX0>CTi0451@@1ZJVX0GemO08Z:RP1iNGT0QhN706QXJ01GEeL0?Shn05=CD`0];Bd0QHF505EEE@1N
GUh0FUYJ05MGE`1XJ6P0EeMG06QXJ01bLW80Shn?06ATI028R8P0NWYj04];B`27QhL0LW9b08>3P`1^
KVh0V9RH06e]K@2:RXX0QXJ6075aL@1GEeL0IFEU06MWI`1OGel0OGem069RHP1FEUH0IVIV07AdM01/
K6`0L71`07emO@1eMGD0GEeM06EUI@1TI6@0K6a/07IfMP1jNWX0ZJVY092@T01dM7@0VYZJ08N7Q`2M
WId0L71`08V9R@1/K6`0PH6104Y:BP2:RXX0JVYZ08N7Q`28R8P0G5aL07AdM01gMgL0OGem05EEE@1W
IfL0Jf][07MgM`1jNWX0LW9b07=cL`1bLW80PH6107MgM`2][Jd0LW9b0080Y:BT00T0Jf][06YZJP1m
OGd0D51@07MgM`2NWYh0QXJ609jNWP1BDU800P1`L700301IFET0Ogmo08N7Q`1LG5`0Kfm_05MGE`1=
CDd0FUYJ08N7Q`1^KVh0Shn?09JFUQ3ooooo000>ooooo`3o06e]K@2MWId0W9bL05IFEP1EEED0Lg=c
05ADE02@T900;Rh^04i>CP1XJ6P0Bd];07QhN00_;bl08b<S05]KF`1]KFd0KVi^05QHF00I6AT0:b/[
092@T02DU9@0GUiN028R8P1CDe<0Hf=S071`L01:BTX0B4Q803/k>`1_Kfl0LG5a055AD@0e=CD0?3`l
03hn?P1oOgl0/[:b04e=C@0^;Rh0@d=304I6AP1SHf<0bLW904];B`0n?Sh0Kfm_059BDP20P800IfMW
020P801;Bd/0GUiN03<c<`2?Shl0MGEe03De=@1MGEd0?clo05eMG@1kNg/0=cLg05iNGP1RHV80D51@
05MGE`2KVi/0GemO034a<@22PX80Lg=c08:2PP2h^;P0820P02XZ:P1DE5@0KFe]09nOW`3Ng]h08B4Q
02@T901NGUh0D51@09fMW@31`L409BDU0410@01<C4`0Jf][0<g=c@26QXH07QhN04Y:BP1kNg/0HF5Q
06YZJP0X:2P092@T04];B`2<S8`0HV9R08Z:RP0V9RH0;2`/07QhN02;Rh/0JFUY09NGU`1HF5P0DE5A
05eMG@1fMWH0J6QX0:2PX01@D500OGem0:JVYP1BDU80HV9R0::RXP1?Cdl0EeMG09fMW@1WIfL0Ti>C
0:>SX`0k>c/0E5AD07YjNP1LG5`0Ti>C09FEU@0g=cL0@T920;Jf]P1NGUh0UIFE065QH@1:BTX0EEEE
0:JVYP1hN7P0J6QX0451@@1<C4`0HF5Q09ZJVP1QHF40H61P024Q8@15ADD0MWIf0:2PX01TI6@0S8b<
02d];@1MGEd0^[Zj08:2PP1=CDd0OWin05]KF`1_Kfl0d=3@06YZJP0n?Sh0LW9b05EEE@2GUiL0d];B
06MWI`1<C4`0M7Ad04U9B@1_Kfl0Zj^[04I6AP1OGel0De=C05EEE@2GUiL0LW9b069RHP20P800IVIV
05MGE`2XZ:P0K6a/0492@P1kNg/0VYZJ09BDU02SXj<0HF5Q061PH028R8P0SXj>0<g=c@1JFUX0@T92
08j>SP1/K6`0]KFe05ADE00n?Sh0U9BD0:JVYP2OWil0K6a/03Ti>@1FEUH0Y:BT07MgM`2n_[h0IfMW
03/k>`2>SXh0Ti>C06i^KP2BTY80;Bd]069RHP2[Zj/0Shn?0:ZZZP2LW9`0:2PX05=CD`1mOGd0RHV9
0;jn_P1kNg/0?clo079bLP2TY:@0U9BD0;2`/00_;bl0>3Ph05YJFP1mOGd0T92@0<o?c`0j>SX0=3@d
04m?C`2WYjL0KVi^07MgM`0n?Sh0DE5A0;Zj^P2RXZ80L71`000G05mOG`0h>3P0BDU90=;BdP2WYjL0
I6AT065QH@12@T80EUIF0;no_`2MWId0HF5Q0861P@0l?3`0A4A409>CT`2^[Zh0De=C06MWI`11@D40
?Cdm07alO01nOWh03_ooool04`1:BTX0H61P0::RXP2DU9@0Y:BT05mOG`1TI6@0QhN706i^KP1_Kfl0
N7Qh08:2PP1PH600KFe]061PH023Ph<0U9BD08B4Q01eMGD00P1WIfL0HP14A4@0F5QH04Q8B010@400
D51@03Hf=P1>CTh0@41005iNGP1VIVH0@41004I6AP20P800Jf][05YJFP1_Kfl0Bd];07QhN00h>3P0
<30`02`/;01KFe/0Hf=S04M7A`1LG5`07AdM06i^KP1fMWH0Cdm?07QhN02IVIT0B4Q8069RHP27QhL0
HF5Q06e]K@1WIfL0PH6103`l?01kNg/0BDU904Y:BP12@T80@d=304E5A@1NGUh0OGem0<30`01EEED0
UYJF0:j^[P25QHD0NGUi07=cL`23Ph<0MGEe0<G5a@1<C4`0KVi^09JFUP2f][H0QXJ605iNGP15ADD0
JVYZ06m_K`1;Bd/0HF5Q03Ph>01QHF40De=C04E5A@14A4@0F5QH0492@P0[:b/0@T9204e=C@1RHV80
F5QH05UIF@1NGUh0EUIF06YZJP1?Cdl0EUIF03hn?P1FEUH0FEUI05eMG@1PH600GEeM04e=C@1WIfL0
>3Ph03`l?01;Bd/0DE5A0P1FEUH0W@0l?3`0Fe]K07moO`0o?cl0Ng]k04Q8B01PH600SHf=079bLP2A
TI40<c<c04=3@`1BDU80BTY:04m?C`0o?cl0@D5104U9B@1HF5P0Cdm?04I6AP1KFe/0FEUI04e=C@1D
E5@0EUIF05mOG`1UIFD0SHf=051@D01VIVH0EUIF05=CD`1ZJVX0;Rh^0492@P10@400];Bd0:FUY@1E
EED0PX:205aLG01=CDd0Ph>307inOP1LG5`0L71`05MGE`0m?Cd0LG5a079bLP1FEUH0HV9R07AdM01I
FET0H61P06UYJ@1WIfL0FEUI0861P@1IFET0Kfm_06YZJP0o?cl0ATI605EEE@1hN7P0RHV908b<S02E
UID0ZJVY09^KV`1ADE40O7al08Z:RP2LW9`0GemO08R8R02CTi<0aLG508V9R@18B4P0H61P092@T02?
Shl0D51@05mOG`1_Kfl0JFUY065QH@1gMgL0PX:207YjNP1[Jf/0KFe]04i>CP1hN7P0Ng]k071`L01H
F5P0J6QX0;Bd]01fMWH0@D5105mOG`1gMgL0Y:BT09NGU`2ZZZX0TY:B0861P@1FEUH0I6AT09NGU`21
PH40Jf][08B4Q02>SXh0XZ:R0;fm_@1OGel0LW9b0861P@2>SXh0T92@06][J`1LG5`0OWin08N7Q`1a
LG40^KVi06UYJ@1PH600YjNW08j>SP2DU9@0XJ6Q06UYJ@2EUID0_kno0:n_[`2][Jd0Fe]K07IfMP1;
Bd/0Cdm?07=cL`1]KFd0BTY:05IFEP1CDe<0@d=306e]K@2NWYh0OGem08n?S`2CTi<04?ooool000ko
oooo0;d0RHV906IVIP1cLg<0Lg=c06UYJ@1KFe/0CTi>09^KV`1HF5P0?3`l04i>CP1MGEd0WIfM0451
@@0U9BD0;Bd]05EEE@26QXH0T92@02TY:@11@D40DE5A08^;R`2YZJT0CDe=02LW9`1UIFD0IVIV075a
L@2/[:`0:BTY03@d=01hN7P0KFe]05]KF`2FUYH0D51@034a<@1EEED0KFe]06=SH`2f][H0DE5A04=3
@`1gMgL0SXj>08:2PP2g]kL0<c<c04U9B@21PH40@T9207moO`1cLg<06ATI0451@@1dM7@0Hf=S06][
J`0F5QH0>CTi055AD@19BDT0JFUY06YZJP0D51@0?clo04U9B@1nOWh0HV9R09:BTP0X:2P0?clo0:^[
Z`1OGel0@41008:2PP0^;Rh0?3`l0;no_`2SXj<0AdM706ATI00f=SH0@41009NGU`1=CDd0>CTi06YZ
JP0h>3P0MWIf09nOW`1PH600DU9B05YJFP12@T80NWYj05iNGP1JFUX0MWIf03Hf=P0n?Sh0LG5a04Y:
BP1>CTh0K6a/0410@018B4P0O7al05=CD`1<C4`0HV9R034a<@0n?Sh0PH6108j>SP1oOgl0RXZ:04U9
B@1MGEd0NGUi05=CD`1ADE40D51@05eMG@1/K6`0[jn_05IFEP0m?Cd0GemO04Q8B02VYZH0YJFU03dm
?@11@D40LW9b06][J`2f][H0K6a/02d];@16ATH0DU9B079bLP22PX80=3@d03Hf=P10@400SXj>08F5
Q@2n_[h0:b/[03Xj>P1CDe<0QhN706UYJ@3>c/h0AdM704U9B@1PH600Shn?08B4Q03IfMT0J6QX07em
O@2`/;00TI6A07AdM02ZZZX0?Shn08J6QP2_[jl0OGem09NGU`2i^KT0=cLg05mOG`28R8P0PH610;Zj
^P2=SHd09bLW0492@P2HV9P0Ogmo0:JVYP1>CTh0@41004a<C02][Jd00P2BTY80?P0/;2`0HF5Q09>C
T`2ATI40Ph>30451@@1:BTX0QHF50:2PX01PH600LG5a05]KF`2>SXh0Ti>C075aL@2QXJ40BDU907If
MP2f][H0JFUY0:6QX@1iNGT0<C4a07EeM@1nOWh0SHf=0:NWY`11@D40Kfm_0:^[Z`1XJ6P0QHF505=C
D`15ADD0H61P0=;BdP1cLg<0MGEe04i>CP19BDT0Rh^;0<g=c@1KFe/0O7al04Y:BP1]KFd0WinO06m_
K`0e=CD0I6AT05MGE`1mOGd0b<S807]kN`14A4@0LW9b06i^KP2KVi/0]kNg04a<C01DE5@0HV9R0P29
RHT05`2i^KT0>c/k034a<@20P800FUYJ0820P02RXZ80>SXj04=3@`2@T900Ng]k06EUI@2SXj<0MGEe
0410@01QHF40GUiN07inOP2UYJD0F5QH02TY:@0l?3`0HV9R00kooooo0240GEeM05EEE@2j^[X0UYJF
07QhN01UIFD0LG5a051@D013@d<0G5aL06MWI`1EEED0Bd];06][J`1bLW80Kfm_04Y:BP0m?Cd0FEUI
05eMG@10@400=SHf04e=C@15ADD0=cLg05UIF@1ADE40=SHf03Lg=`1GEeL0DE5A03Xj>P10@4000P1O
Gel0Z@1NGUh0JFUY04a<C01GEeL0<C4a03De=@0f=SH0IFEU05mOG`1?Cdl0IFEU03Ti>@1/K6`0Hf=S
0410@01/K6`0L71`04A4A016ATH0G5aL065QH@1MGEd0>CTi07emO@12@T80Ph>305=CD`1>CTh0Bd];
04a<C01?Cdl0Fe]K0820P02k^k/0FEUI08F5Q@2f][H0U9BD07IfMP1GEeL0GemO04m?C`27QhL0D51@
06=SH`2@T900aLG509:BTP1gMgL0De=C096AT@28R8P0CDe=07moO`1MGEd0MWIf05MGE`1NGUh0PH61
07IfMP1:BTX0C4a<06QXJ01CDe<0LW9b05ADE01]KFd0De=C05UIF@1mOGd0F5QH05eMG@1ZJVX0Ph>3
06UYJ@1@D500F5QH0861P@1ADE40J6QX05eMG@1jNWX0Bd];05IFEP10@400IFEU059BDP1<C4`0MWIf
04U9B@27QhL0Kfm_0:RXZ02d];@0SXj>0<?3``0];Bd0EeMG05QHF01?Cdl0D51@03Ti>@0e=CD0<c<c
06][J`1NGUh0ADE505ADE01iNGT0@T9204Y:BP1<C4`0Ng]k05IFEP1<C4`0E5AD079bLP0b<S80:BTY
04];B`0[:b/0Bd];02DU9@1SHf<0Fe]K05eMG@1JFUX0JFUY04M7A`1kNg/0HF5Q04e=C@1YJFT0:BTY
06ATI01eMGD0?clo06a/K01iNGT0Jf][06MWI`1?Cdl0D51@07emO@12@T80Q8B406=SH`2CTi<0NGUi
04Y:BP1=CDd0H61P07moO`1eMGD0Ph>307UiN@2PX:00MgMg04M7A`1bLW80UIFE096AT@1MGEd0VYZJ
05MGE`1SHf<0GUiN055AD@15ADD00P1>CTh08@15ADD0Xj>S06MWI`1VIVH0De=C075aL@1]KFd0Hf=S
06QXJ01QHF40Cdm?055AD@18B4P0De=C05ADE01KFe/0R8R806IVIP18B4P0Zj^[071`L022PX80L71`
07]kN`1JFUX0H61P07inOP2:RXX0J6QX08>3P`1mOGd0NGUi06MWI`02065QH@0U06i^KP2@T900EEEE
0:JVYP2o_kl0@T9208>3P`2UYJD0UYJF08F5Q@21PH40De=C07inOP1SHf<0EUIF08Z:RP29RHT0KVi^
0:FUY@2JVYX0NWYj06m_K`0f=SH0EEEE03Lg=`0o?cl0KVi^09NGU`1GEeL0Jf][06EUI@1OGel0GUiN
07inOP21PH40JVYZ0861P@0@ooooo`003_ooool0?@34a<@0G5aL075aL@1mOGd0ZJVY04a<C019BDT0
D51@07QhN01ZJVX09RHV04Q8B01?Cdl0OGem069RHP1hN7P0?3`l04i>CP1EEED0A4A4055AD@1bLW80
<C4a07=cL`2m_Kd0HF5Q07EeM@1KFe/0?Shn08>3P`2_[jl0@41006MWI`18B4P0<30`059BDP1bLW80
@D51069RHP0d=3@0:2PX06QXJ01aLG40DU9B03Ph>00_;bl0@D5109NGU`2:RXX0BDU905iNGP19BDT0
Ng]k09BDU01HF5P0?Shn04m?C`1aLG40TI6A03<c<`0Z:RX00P1:BTX0e`1mOGd0^[Zj065QH@0T92@0
BDU904i>CP1IFET0Rh^;03Hf=P0U9BD0CTi>05ADE00o?cl0VYZJ03Ph>00Z:RX0ADE50861P@1YJFT0
VYZJ03Ti>@11@D40E5AD07MgM`1WIfL0]KFe03Ti>@0o?cl0Kfm_04i>CP29RHT0VIVI03@d=01@D500
JFUY07YjNP2a/K40<C4a03De=@1QHF40CDe=07alO02m_Kd0CDe=04E5A@24Q8@0JFUY09^KV`2^[Zh0
=CDe02d];@1HF5P0KVi^096AT@2l_;`0?3`l04M7A`1]KFd0SHf=09VIV@1TI6@0@T9206MWI`1OGel0
PH6105QHF02<S8`0;bl_06i^KP2n_[h0SHf=0820P01[Jf/0BTY:061PH02b/[80IfMW08f=S@26QXH0
?3`l08V9R@27QhL0FUYJ07emO@1[Jf/0@d=30:JVYP2TY:@0D51@09FEU@1MGEd0<c<c07=cL`2XZ:P0
I6AT09fMW@0/;2`0@d=30:^[Z`2DU9@0MgMg0;Zj^P14A4@0EEEE0<S8b020P800JVYZ08:2PP10@400
Cdm?0<[:bP2/[:`0EEEE065QH@1:BTX0H61P0;jn_P1HF5P0=SHf05ADE01QHF40WYjN08R8R013@d<0
GUiN05eMG@2j^[X0dM7A0410@01LG5`0L71`05IFEP2MWId0G5aL04];B`1iNGT0IVIV07EeM@2[Zj/0
>c/k08J6QP1nOWh0QhN70=KFeP1WIfL0BTY:09NGU`1^KVh0`<3009JFUP0^;Rh0FEUI05EEE@2GUiL0
eMGE07AdM01VIVH0Cdm?06IVIP2l_;`0DE5A03`l?01PH600LG5a07inOP2^[Zh0@T9205IFEP1XJ6P0
ZZZZ06EUI@2f][H0?Shn05IFEP1aLG40XJ6Q07emO@2l_;`0EEEE069RHP2BTY80^KVi09VIV@38b<P0
/[:b05MGE`3Cdm<0SXj>07]kN`2d];@0H61P05=CD`2c/k<0U9BD06=SH`3?cll0FEUI03Xj>P1nOWh0
Kfm_08J6QP2DU9@0:RXZ04a<C01YJFT0H61P071`L032`/80FUYJ01dM7@1cLg<0QHF504Y:BP1NGUh0
D51@00kooooo00L0MGEe05EEE@2g]kL0c/k>0:FUY@1@D500De=C0080JFUY00h0F5QH05iNGP1iNGT0
HV9R06EUI@1@D500GUiN09:BTP1TI6@0Bd];04Q8B01VIVH0I6AT051@D080H61P0;H0EeMG03<c<`1F
EUH0GUiN04m?C`1MGEd0GUiN04a<C017AdL0Fe]K05MGE`1HF5P0>CTi05aLG00m?Cd0=SHf03Ph>011
@D40BDU904Q8B01:BTX0?Cdm05iNGP1JFUX0ATI606i^KP1fMWH0ATI6055AD@1FEUH0BDU9061PH01@
D500IVIV04Q8B01=CDd0BTY:059BDP0m?Cd0C4a<051@D01TI6@0E5AD06i^KP1IFET0G5aL08:2PP26
QXH0K6a/0861P@2g]kL0MWIf08^;R`1BDU80Fe]K07YjNP1NGUh0IFEU04e=C@1CDe<0Kfm_06][J`1K
Fe/0M7Ad065QH@1FEUH0F5QH051@D01lO7`0KVi^04e=C@1ADE40E5AD059BDP1SHf<0EeMG05aLG01:
BTX0C4a<05iNGP1FEUH0G5aL07]kN`29RHT0UiNG079bLP1YJFT0Kfm_051@D01TI6@0G5aL08N7Q`1T
I6@0M7Ad06EUI@1gMgL0EUIF059BDP22PX80;2`/06i^KP1FEUH0QhN70<[:bP1ZJVX0Ng]k059BDP1N
GUh0FUYJ071`L01fMWH0EEEE08>3P`17AdL0JFUY03/k>`13@d<0CDe=03Ti>@11@D40@d=303Ph>00j
>SX0=cLg05YJFP0i>CT0Cdm?03@d=00i>CT0Cdm?03Lg=`1CDe<0=cLg07QhN01fMWH0@d=303`l?01D
E5@0CTi>07alO01KFe/0MgMg08N7Q`0J6QX0?Cdm059BDP1;Bd/0M7Ad04e=C@1PH600G5aL065QH@1N
GUh0HV9R04e=C@2_[jl0Ogmo06ATI014A4@0L71`0861P@1BDU80@d=306MWI`2NWYh0Cdm?05EEE@1B
DU80<C4a08V9R@1XJ6P0NWYj06=SH`20P800KVi^0;>c/`1]KFd0<c<c04E5A@25QHD0I6AT03`l?080
D51@04D0HV9R06=SH`1ZJVX0EeMG05ADE01]KFd0DU9B05YJFP1CDe<0L71`069RHP1iNGT0O7al0<k>
cP2=SHd0CTi>069RHP2BTY80`<3009FEU@27QhL0M7Ad09FEU@1/K6`0KVi^07UiN@31`L40OGem05QH
F02;Rh/0UIFE09NGU`17AdL0BDU9055AD@23Ph<0Xj>S071`L02FUYH0N7Qh06YZJP0k>c/0LG5a07Ee
M@1lO7`0T92@03Xj>P19BDT0HF5Q04A4A02CTi<0QHF507inOP1fMWH0Fe]K07QhN013@d<0>3Ph055A
D@1SHf<0IVIV06][J`1nOWh0@T9206][J`2>SXh0IFEU09>CT`2k^k/04?ooool000kooooo0;00TY:B
08N7Q`1nOWh0JFUY07=cL`26QXH0>3Ph069RHP15ADD0G5aL04m?C`0[:b/0EEEE04I6AP1ADE40QHF5
04a<C00f=SH0FUYJ05ADE01WIfL0Ogmo03Lg=`11@D40VIVI07MgM`2[Zj/0S8b<0492@P1:BTX0WinO
08B4Q02/[:`0?Cdm02XZ:P0/;2`0Ph>30861P@37alL0=cLg038b<P0h>3P0CTi>08n?S`2PX:005ADE
0492@P1DE5@0GemO0:NWY`1GEeL0820P04Y:BP1bLW80DE5A0;:b/P0d=3@0BTY:05eMG@1FEUH0KVi^
0;2`/00[:b/0EUIF07=cL`2NWYh0KVi^0<G5a@0^;Rh0:BTY07UiN@1fMWH0Q8B40;:b/P0T92@0:b/[
06IVIP2LW9`0Ogmo0:ZZZP0R8R80<S8b06ATI01YJFT0MgMg08F5Q@0W9bL0D51@05ADE01PH600HF5Q
02h^;P0h>3P0M7Ad08R8R01KFe/0Lg=c01TI6@1<C4`0YjNW0;>c/`1?Cdl0IFEU02<S8`1LG5`0Ti>C
07inOP1ZJVX0LG5a04a<C01IFET0YZJV061PH00l?3`0J6QX04M7A`1cLg<0W9bL055AD@18B4P0I6AT
04I6AP2LW9`0S8b<03Xj>P1IFET0P82005ADE01]KFd0`<300410@011@D40JFUY04a<C01aLG40Shn?
0451@@1ZJVX0GemO06m_K`2GUiL0f][J04i>CP1QHF40Jf][06IVIP28R8P0X:2P03Ti>@1MGEd0IFEU
05UIF@2^[Zh0PH6103Ti>@19BDT0JVYZ08^;R`3;bl/0De=C03hn?P16ATH0OGem0:j^[P3Shn<0CDe=
04A4A017AdL0Rh^;0:b/[02c/k<0?clo05mOG`1RHV80TY:B09jNWP80AdM706@0K6a/08b<S020P800
SXj>055AD@0e=CD0GEeM0:VYZ@2JVYX0c<c<06ATI01FEUH0S8b<07moO`3=cLd0/[:b04Y:BP1hN7P0
Ogmo08R8R02GUiL0Bd];04M7A`2o_kl0T92@0:RXZ01YJFT0@41008n?S`3;bl/0K6a/07QhN00n?Sh0
CDe=09:BTP1KFe/0YJFU04I6AP1>CTh0SXj>08f=S@1>CTh0RXZ:03`l?02EUID0Rh^;05mOG`1`L700
DE5A04m?C`2YZJT0UYJF05aLG02JVYX0B4Q804E5A@2GUiL0De=C061PH02;Rh/0>CTi04m?C`1TI6@0
F5QH061PH01GEeL0DU9B04A4A01OGel0Vi^K051@D01EEED0MGEe07inOP2JVYX0OWin03`l?01?Cdl0
Jf][05QHF03:b/X0J6QX03Ti>@19BDT0LW9b0:b/[03Ng]h0MWIf02l_;`1FEUH0B4Q806=SH`3=cLd0
QHF502XZ:P0l?3`0HV9R065QH@1cLg<0?clo3_ooool0R@1hN7P0G5aL08>3P`2=SHd0XZ:R059BDP1J
FUX0EeMG03Lg=`1LG5`0MgMg05aLG01GEeL0Kfm_06MWI`20P800E5AD069RHP1HF5P0EeMG061PH016
ATH0E5AD04=3@`0o?cl0DE5A03/k>`11@D40CDe=055AD@12@T809bLW03`l?01MGEd0K6a/08N7Q`21
PH40@d=306][J`1PH600>SXj04i>CP2:RXX0HV9R04Y:BP1CDe<0AdM704a<C01?Cdl0A4A405YJFP1[
Jf/0<C4a04M7A`1RHV80O7al05=CD`1IFET0LG5a04e=C@1KFe/0ADE504a<C019BDT0@d=303lo?`1f
MWH0O7al09VIV@18B4P0XZ:R0<;2`P26QXH0@T92075aL@3De=@0V9RH06i^KP0a<C40CDe=08J6QP32
`/80KVi^05ADE017AdL0GemO065QH@0j>SX0B4Q8055AD@1=CDd0C4a<04=3@`1RHV80F5QH04E5A@0g
=cL0ATI603dm?@1;Bd/0@d=3055AD@17AdL0?Cdm055AD@11@D40DE5A07MgM`1kNg/0F5QH05EEE@1C
De<0IVIV055AD@1OGel0GUiN06][J`2DU9@0TI6A05IFEP1WIfL0NWYj06IVIP1nOWh0>c/k07AdM01b
LW80Rh^;0<c<c01iNGT0SXj>03Ph>01RHV80WIfM075aL@1XJ6P0EeMG0080M7Ad00@0NWYj05ADE01C
De<0F5QH0P1ADE40Q@1LG5`0AdM7069RHP1;Bd/0Xj>S04i>CP1]KFd0>SXj04e=C@1UIFD0<c<c05eM
G@0Q8B40Jf][07inOP0d=3@0D51@05aLG00/;2`0C4a<03lo?`1DE5@0GEeM04e=C@1HF5P0H61P04e=
C@1`L700EUIF06EUI@1OGel0I6AT04A4A01<C4`0?clo05]KF`1FEUH0IVIV05UIF@16ATH0B4Q803Lg
=`1nOWh0R8R808^;R`1EEED0Shn?092@T016ATH0De=C06ATI01hN7P0G5aL07IfMP1<C4`0I6AT0820
P00n?Sh0@41004Y:BP1;Bd/0@T9205mOG`1JFUX0JVYZ04m?C`1RHV80IVIV05mOG`1YJFT0IfMW04Y:
BP1TI6@0IFEU05aLG01GEeL0IfMW06e]K@1TI6@0CDe=071`L020P800FEUI06UYJ@23Ph<0?Cdm05eM
G@14A4@0N7Qh07AdM01hN7P0K6a/0820P01bLW80GemO06UYJ@20P800Rh^;05EEE@1OGel0I6AT02`/
;025QHD0KFe]05IFEP1PH600Vi^K05]KF`1cLg<0T92@05UIF@1NGUh0DU9B0492@P1OGel0Rh^;0861
P@1GEeL0A4A405mOG`12@T80>SXj079bLP2;Rh/0GemO09JFUP1FEUH0B4Q807MgM`2][Jd0Ogmo0:n_
[`33`l<04?ooool000kooooo07d0MgMg08B4Q028R8P0NWYj04a<C025QHD0UiNG0;2`/00o?cl0CTi>
06e]K@1_Kfl0Xj>S03De=@10@400FUYJ069RHP25QHD0CTi>02h^;P12@T80MgMg08N7Q`1XJ6P0;bl_
0410@01LG5`0UYJF075aL@1YJFT0:2PX06YZJP27QhL0NWYj04I6AP1]KFd0>SXj05aLG01nOWh0RHV9
04Q8B01gMgL0?Shn0492@P28R8P0EeMG05iNGP1oOgl0>3Ph05QHF02PX:00@d=308N7Q`0a<C4092@T
04m?C`1UIFD0=cLg04Q8B00e=CD0>3Ph069RHP0k>c/0@T9205]KF`13@d<0?3`l069RHP20P800DU9B
04E5A@12@T80AdM707moO`1PH6009BDU04E5A@1RHV80H61P07moO`15ADD0:BTY03lo?`1EEED0Ti>C
08f=S@0Z:RX0<C4a05=CD`1`L700alO705MGE`0M7Ad0>SXj079bLP1fMWH0RHV9028R8P0b<S80A4A4
07IfMP1YJFT0Ogmo02l_;`15ADD0EUIF07UiN@2:RXX0/;2`04=3@`1;Bd/0GUiN07]kN`1iNGT0h>3P
04e=C@1MGEd0XJ6Q07=cL`2RXZ80aLG504U9B@1JFUX0^;Rh05eMG@020:JVYP0;051@D00g=cL0ZZZZ
08N7Q`2d];@0W9bL04e=C@0l?3`0V9RH07inOP2l_;`00P10@400CP1_Kfl0P82009bLW02WYjL0AdM7
0492@P1GEeL0QhN7075aL@2RXZ80DE5A03lo?`1cLg<0b/[:096AT@2<S8`0@41005ADE020P800NWYj
05mOG`27QhL0EUIF05eMG@2/[:`0YZJV07EeM@2QXJ40B4Q807inOP26QXH0O7al059BDP25QHD0:2PX
07moO`2NWYh0Bd];04Q8B01eMGD0<S8b07MgM`1hN7P0A4A406UYJ@0k>c/0EEEE09NGU`22PX80W9bL
05YJFP14A4@0GemO0;^k^`22PX80P8200451@@1jNWX0]KFe05EEE@0i>CT0LW9b06ATI03:b/X0RXZ:
04E5A@1MGEd0Kfm_08f=S@20P800B4Q804M7A`21PH40@D510;jn_P1^KVh0CTi>07inOP80NGUi03X0
`l?303hn?P20P800Ph>308f=S@2KVi/0T92@03Lg=`1XJ6P0KFe]08>3P`2_[jl0=CDe03<c<`1/K6`0
NGUi09JFUP2n_[h0:BTY03lo?`21PH40JVYZ0:FUY@2f][H0:b/[04=3@`1RHV80MWIf09NGU`2n_[h0
92@T04A4A01cLg<0Kfm_08R8R024Q8@06QXJ05IFEP2b/[80[Zj^06ATI027QhL08b<S04Q8B03?cll0
emOG05YJFP1gMgL0>c/k03`l?01FEUH0Xj>S06EUI@1mOGd0CTi>05=CD`1=CDd0JFUY3_ooool0R`1?
Cdl0IFEU0=WIf@3Ogml0alO709>CT`2<S8`0TI6A061PH01aLG40EEEE04e=C@1>CTh0LG5a05IFEP1V
IVH0NGUi08b<S01kNg/0JVYZ06MWI`15ADD0Bd];05IFEP1?Cdl0NWYj05QHF01@D500K6a/071`L01Y
JFT0ATI606ATI01^KVh0NWYj065QH@1]KFd0@d=306i^KP1>CTh0;bl_03Hf=P1@D500DE5A04M7A`1i
NGT0Cdm?05=CD`1LG5`0GEeM06UYJ@1gMgL0>c/k04I6AP18B4P0<C4a0451@@1IFET0HV9R05QHF01Q
HF40CDe=04E5A@1@D500@D5104Q8B00i>CT0=cLg06=SH`15ADD0FUYJ05aLG01XJ6P0E5AD05MGE`0c
<c<0=3@d02TY:@0c<c<0EeMG08F5Q@2c/k<0NGUi061PH01VIVH0E5AD05iNGP1ZJVX0I6AT04a<C01C
De<0HV9R059BDP1ZJVX0LW9b05UIF@19BDT0?Shn04i>CP1VIVH0ADE504M7A`10@400=CDe051@D01c
Lg<0Hf=S069RHP1iNGT0TY:B06][J`1HF5P0I6AT05eMG@1_Kfl0BDU905mOG`1eMGD0E5AD04U9B@1D
E5@0KFe]05YJFP1WIfL0<S8b08B4Q01XJ6P0GEeM07moO`1iNGT0UYJF04Q8B01<C4`0HF5Q08:2PP1b
LW80?Shn04e=C@1cLg<00P1DE5@0D@0l?3`0AdM704Y:BP16ATH0F5QH05iNGP1`L700B4Q805ADE01I
FET0IFEU04e=C@1GEeL0MGEe03hn?P1OGel0ADE509NGU`2h^;P0Cdm?05=CD`1lO7`0H61P07YjNP1h
N7P0HF5Q06IVIP0k>c/08b<S03/k>`17AdL0EUIF06=SH`2@T900IVIV04];B`12@T80IVIV0492@P1h
N7P0FEUI06e]K@1>CTh0BDU904Y:BP16ATH0ADE5061PH01kNg/0C4a<0410@012@T80CDe=0:NWY`1O
Gel0J6QX05eMG@2SXj<0FEUI08N7Q`28R8P0PX:204=3@`1PH600Fe]K06ATI022PX80IFEU05QHF01?
Cdl0GemO04i>CP1HF5P0LW9b05eMG@1:BTX0EEEE05=CD`1EEED0GemO07EeM@0209nOW`0:069RHP1m
OGd0Shn?09NGU`1cLg<0Hf=S0492@P1lO7`0C4a<07alO080JFUY02X0Hf=S05mOG`25QHD0N7Qh07in
OP1DE5@0F5QH04];B`1hN7P0JVYZ04e=C@2JVYX0JFUY06MWI`1GEeL0Ng]k0861P@1/K6`0Ti>C08V9
R@1ZJVX0PX:206MWI`29RHT0/k>c09NGU`1FEUH0BTY:05iNGP0k>c/0HV9R07AdM021PH40@T9204i>
CP1<C4`0CTi>08F5Q@1eMGD0Kfm_0:2PX02EUID@ooooo`003_ooool0=02`/;00Lg=c065QH@1nOWh0
^[Zj05ADE00n?Sh0GemO06e]K@1/K6`0=SHf02l_;`15ADD0H61P03Lg=`1[Jf/0>CTi04E5A@1OGel0
<c<c04e=C@1ZJVX0>SXj06IVIP1bLW80CTi>06][J`1>CTh0@d=3071`L01OGel0A4A405eMG@1>CTh0
@410065QH@1SHf<0MGEe07MgM`1=CDd0Cdm?0820P02?Shl0Cdm?04a<C00m?Cd0H61P0:ZZZP1dM7@0
BDU904=3@`12@T8208n?S`3P03/k>`1BDU80B4Q805IFEP23Ph<0@T9203<c<`19BDT0?clo071`L02e
]KD0HF5Q03`l?01kNg/0B4Q80861P@2OWil0C4a<03lo?`1WIfL0FEUI04Y:BP2IVIT0?Cdm03Xj>P1W
IfL0KFe]05iNGP2IVIT09bLW04Y:BP2;Rh/0LW9b05eMG@2HV9P0>3Ph05eMG@23Ph<0C4a<07]kN`2:
RXX0?clo07MgM`1QHF40GemO09ZJVP1VIVH0:RXZ065QH@10@400OWin09^KV`0Y:BT0<30`06QXJ01W
IfL0WYjN09>CT`0`<300?3`l06e]K@1cLg<0RHV90:j^[P0n?Sh0A4A407EeM@2KVi/0LG5a07]kN`0i
>CT0De=C096AT@2=SHd0Fe]K07AdM00l?3`0FEUI09jNWP2IVIT0QXJ606EUI@1GEeL0Rh^;09^KV`1/
K6`0P82008V9R@1FEUH0MGEe07=cL`1LG5`0HF5Q08>3P`1EEED0QHF5061PH01=CDd0YjNW05]KF`1?
Cdl0QXJ605]KF`1FEUH0UIFE04U9B@1JFUX0V9RH0861P@20P800Y:BT06QXJ02CTi<0];Bd08N7Q`29
RHT0WYjN06MWI`29RHT0[Zj^06i^KP1OGel0De=C06UYJ@2DU9@0YJFU05=CD`1:BTX0Jf][06QXJ02[
Zj/0IVIV0451@@1RHV80Kfm_0;Bd]02e]KD0AdM706UYJ@2;Rh/0KFe]09jNWP1cLg<0DU9B09bLW027
QhL0GemO08V9R@0o?cl0VYZJ09RHV01]KFd0Zj^[08j>SP1?Cdl0QXJ606m_K`2UYJD0Vi^K03dm?@1R
HV80LW9b08N7Q`2LW9`0?3`l04E5A@1RHV80J6QX0:NWY`10@400<c<c075aL@1WIfL0GemO08j>SP1;
Bd/0AdM709:BTP2QXJ40K6a/09bLW01@D500K6a/0;2`/02ZZZX0IVIV09FEU@1>CTh0GemO0:f][@34
a<@0NWYj0<C4a01SHf<0Bd];09BDU02ATI40J6QX09bLW01IFET0Fe]K0:b/[01kNg/0IVIV09bLW01T
I6@0EUIF092@T00m?Cd0DU9B08b<S01RHV80D51@096AT@1LG5`0IFEU08n?S`1HF5P0D51@06EUI@1N
GUh0C4a<06MWI`1IFET>ooooo`0504M7A`1SHf<0W9bL0<_;b`2GUiL00P1NGUh0:@1XJ6P0EeMG06a/
K01/K6`0E5AD04Y:BP1gMgL0Lg=c0861P@20P800R8R806e]K@1>CTh0C4a<03/k>`1:BTX0ADE504I6
AP1UIFD0>CTi04Q8B013@d<0GEeM051@D017AdL0CDe=06][J`1PH600LW9b07MgM`1?Cdl0JVYZ05eM
G@12@T80ATI607]kN`1YJFT0D51@06UYJ@0j>SX0DE5A0080Fe]K00d0KFe]06UYJ@0l?3`0Bd];05YJ
FP1CDe<0G5aL05=CD`1ZJVX0Bd];05YJFP1CDe<0E5AD0080ADE504<0CTi>05QHF01/K6`0ZZZZ04U9
B@2GUiL0][Jf08j>SP1UIFD0Hf=S06m_K`18B4P0EeMG05EEE@1RHV80Ogmo0;Bd]01kNg/0HF5Q061P
H01OGel0JFUY04i>CP1NGUh0Bd];069RHP1=CDd0C4a<061PH01RHV80D51@04m?C`1IFET0ADE505UI
F@18B4P0EUIF04A4A00n?Sh0Bd];04e=C@1JFUX0L71`07inOP1JFUX0Cdm?06IVIP1mOGd0GEeM075a
L@1EEED0FUYJ03`l?01RHV80>c/k051@D012@T80C4a<071`L00a<C40MgMg06MWI`20P800];Bd06e]
K@1PH600<S8b0080FUYJ00@0BTY:04a<C01=CDd0@D510P19BDT0RP11@D40DE5A05IFEP0k>c/0@d=3
07moO`1@D500EUIF071`L02@T900EUIF06][J`1/K6`0BTY:06i^KP1MGEd0Jf][06IVIP21PH40Q8B4
05]KF`1NGUh0K6a/04U9B@24Q8@0NGUi05ADE01gMgL08R8R0410@01NGUh0C4a<07MgM`1KFe/0M7Ad
061PH01]KFd0DE5A05MGE`0d=3@0O7al04i>CP1VIVH0FEUI069RHP1`L700GemO06YZJP1SHf<0RHV9
05aLG01aLG40D51@055AD@1eMGD0O7al06YZJP1TI6@0Xj>S0861P@2SXj<0V9RH059BDP0k>c/0I6AT
071`L01WIfL0T92@06IVIP1WIfL0Kfm_079bLP1nOWh0LG5a06m_K`1eMGD0EeMG04Q8B00f=SH0JFUY
07emO@1fMWH0YjNW08:2PP1[Jf/0/;2`09JFUP2OWil0SXj>0820P01VIVH0MGEe06e]K@27QhL0OWin
092@T01^KVh0I6AT07=cL`1lO7`0SXj>04m?C`1HF5P0ADE508N7Q`1eMGD0<c<c08J6QP1hN7P0NGUi
05QHF01hN7P0IVIV08n?S`2QXJ40GemO079bLP21PH40GemO09jNWP1nOWh0S8b<05mOG`1DE5@0EeMG
03`l?01=CDd0HF5Q06m_K`1^KVh0MgMg05YJFP1<C4`0LG5a07UiN@1RHV80SHf=09:BTQ3ooooo000>
ooooo`37071`L02UYJD0S8b<06ATI025QHD0`/;205]KF`27QhL0;Bd]05ADE02FUYH0?clo06ATI00f
=SH09RHV05ADE028R8P0Fe]K04e=C@0N7Qh0>SXj08B4Q01BDU80CTi>03Ti>@1BDU80PH61092@T00]
;Bd0D51@05ADE01@D500TY:B07inOP0P8200ATI605]KF`22PX80_[jn04=3@`0b<S80<C4a05ADE01a
LG40/K6a02<S8`10@400CDe=05]KF`2<S8`0L71`02DU9@1EEED0MWIf05QHF01dM7@0H61P0451@@1U
IFD0B4Q8059BDP21PH407QhN03Ti>@1BDU80Ph>307inOP3IfMT0;Rh^02d];@1RHV80QhN706QXJ030
`<00;bl_01LG5`1aLG40VYZJ07inOP2TY:@0:BTY02@T901QHF40Ng]k08>3P`21PH407QhN04i>CP1P
H600EEEE06QXJ014A4@09bLW05=CD`26QXH0K6a/06i^KP0c<c<0@T9205EEE@2<S8`0EeMG07]kN`0c
<c<0@41009:BTP1EEED0E5AD08^;R`0l?3`0I6AT0:f][@2?Shl0LW9b0:^[Z`0b<S80GemO0861P@1T
I6@0F5QH0:6QX@13@d<0EUIF08f=S@1^KVh0GemO07inOP0^;Rh0AdM707alO00m?Cd0Jf][05eMG@13
@d<0Ti>C07EeM@14A4@0S8b<03hn?P0g=cL0Lg=c08F5Q@1GEeL0OWin03`l?01WIfL0QXJ609JFUP0o
?cl0MgMg03lo?`1BDU80PH610820P00`<300GemO05ADE01bLW80c<c<06i^KP0`<300GemO06=SH`1l
O7`0h>3P05EEE@16ATH0Hf=S075aL@1hN7P0d=3@03Lg=`15ADD0SXj>05MGE`2YZJT0IfMW02PX:014
A4@0MWIf071`L02FUYH0JFUY03dm?@1JFUX0HF5Q08:2PP3EeMD0H61P075aL@2VYZH0HV9R0<;2`P2^
[Zh0@D51069RHP2:RXX0P8200<W9b@02069RHP1=0;Bd]01QHF40`/;2075aL@0o?cl0Ti>C0;fm_@1`
L700J6QX03hn?P1GEeL0YJFU06e]K@1jNWX0De=C05eMG@2[Zj/0RHV9092@T01RHV80>3Ph06i^KP2k
^k/0HV9R07EeM@0f=SH0B4Q8071`L02GUiL0VYZJ0<K6aP0o?cl0EEEE07emO@2EUID0JVYZ09^KV`0j
>SX0EeMG06MWI`1SHf<0O7al06QXJ00`<300J6QX06MWI`2CTi<0HF5Q06m_K`0o?cl0J6QX08:2PP2M
WId0IVIV05IFEP0Y:BT0EUIF0=;BdP2RXZ80@d=305mOG`0h>3P0B4Q80:RXZ02@T900AdM706MWI`0o
?cl0Cdm?08F5Q@2CTi<0=cLg05=CD`1<C4`0Jf][07QhN01GEeL03_ooool0G@0n?Sh0Jf][09:BTP1o
Ogl0a/K605]KF`1EEED0JVYZ04E5A@1JFUX0E5AD04Q8B01DE5@0RHV906=SH`2@T900YJFU0:^[Z`1]
KFd0FEUI05MGE`0Z:RX0FEUI04Q8B00k>c/0IFEU04M7A`0m?Cd0B4Q804];B`1KFe/0?Shn0492@P1Z
JVX0DE5A06ATI01ZJVX0ATI606a/K01ADE40@d=303Lg=`1QHF40GUiN059BDP1FEUH07QhN04Y:BP1R
HV80Bd];05aLG01SHf<0=3@d04Q8B01=CDd0?Cdm06YZJP1_Kfl0QhN7059BDP1/K6`0JVYZ05mOG`1>
CTh0E5AD05aLG01QHF40L71`09NGU`1EEED0UIFE09^KV`2WYjL0Rh^;05iNGP1fMWH0LW9b03dm?@0l
?3`0D51@079bLP2OWil0Ph>305EEE@1FEUH0IfMW07=cL`1VIVH0NGUi05=CD`1hN7P0IfMW05UIF@02
06][J`0b05EEE@1<C4`0GEeM051@D01DE5@0C4a<05aLG01HF5P0B4Q806QXJ01[Jf/0P82005aLG024
Q8@0M7Ad06][J`1/K6`0QhN706YZJP2=SHd0De=C06a/K020P800JFUY06e]K@1hN7P0HF5Q04Y:BP1l
O7`0;Rh^0::RXP2:RXX0QXJ60:b/[01iNGT0OWin030`<01MGEd0ADE506a/K01gMgL0>3Ph059BDP1L
G5`0HF5Q04E5A@1gMgL0GUiN038b<P19BDT204Q8B01I04M7A`1kNg/0IFEU04M7A`1KFe/0@D5103Lg
=`1MGEd0CDe=06YZJP1OGel0UYJF0861P@14A4@0De=C06IVIP12@T80HF5Q04M7A`10@400D51@02XZ
:P0g=cL0BTY:04Q8B01SHf<0F5QH07inOP1GEeL0FEUI059BDP1ZJVX0CTi>07YjNP1GEeL0IfMW05eM
G@1;Bd/0CTi>04e=C@29RHT0M7Ad0;no_`1FEUH0MGEe061PH00/;2`0IFEU04i>CP1BDU80DE5A08B4
Q01HF5P0PX:206][J`1SHf<0@41004Y:BP1>CTh0LW9b0:VYZ@1QHF40EEEE05UIF@1cLg<0GUiN05]K
F`1jNWX0EeMG05UIF@1DE5@0D51@05]KF`1SHf<0J6QX0:j^[P1mOGd0SHf=0:j^[P24Q8@0Xj>S08R8
R01TI6@0F5QH059BDP1oOgl0SHf=061PH025QHD00P1QHF40:01[Jf/0XJ6Q09fMW@1@D500C4a<06a/
K02/[:`0QHF5055AD@23Ph<0P82008^;R`1JFUX0JFUY07YjNP1fMWH0L71`05IFEP1VIVH0E5AD04I6
AP2LW9`0Ogmo0861P@1EEED0ATI604m?C`12@T80@41006YZJP28R8P0JFUY05aLG01gMgL0GemO08f=
S@21PH40PX:207EeM@1oOgl@ooooo`003_ooool0o`1SHf<0Ogmo09NGU`1[Jf/0CDe=09FEU@1RHV80
W9bL05EEE@0`<300QhN704m?C`2FUYH0DU9B02`/;013@d<0GemO09fMW@2OWil05aLG04I6AP1IFET0
K6a/096AT@0i>CT0:BTY06=SH`2IVIT0N7Qh07IfMP0K6a/0A4A404];B`25QHD0F5QH05mOG`0i>CT0
?Shn03dm?@1@D500K6a/07emO@17AdL0=SHf06i^KP1QHF40MWIf0;:b/P0d=3@0?3`l0::RXP19BDT0
QhN707alO00[:b/0<30`05eMG@1OGel0Ph>301hN7P0[:b/0PH6105ADE01GEeL0NGUi02DU9@0a<C40
SXj>08F5Q@1OGel0@T9203<c<`1MGEd0SXj>08N7Q`0o?cl0Hf=S0451@@1ZJVX0UYJF06=SH`0_;bl0
BDU905MGE`1fMWH0Zj^[04E5A@0V9RH0IVIV05mOG`26QXH0Bd];03<c<`1;Bd/0AdM706UYJ@2EUID0
=3@d04A4A01UIFD0HV9R05MGE`2GUiL0@41004M7A`1NGUh0FUYJ06=SH`2[Zj/0?Cdm04I6AP1CDe<0
G5aL07]kN`2g]kL0>SXj061PH026QXH0EeMG075aL@3Bd]80Q8B405iNGP1kNg/0Hf=S08f=S@3Rh^80
H61P03Lg=`1SHf<0DU9B0::RXP2n_[h0@41004Y:BP1fMWH0Ng]k0=kNgP1MGEd0>CTi05aLG01lO7`0
S8b<09fMW@0e=CD0@41007QhN02UYJD0RHV909RHV00h>3P0EEEE06i^KP2j^[X0V9RH09fMW@0a<C40
BDU905]KF`2YZJT0M7Ad09ZJVP1UIFD0BTY:08:2PP20P800HV9R0:n_[`1?Cdl0FEUI07emO@1_Kfl0
DE5A0:ZZZP0n?Sh0H61P0;:b/P1jNWX0^;Rh06ATI00Z:RX0EUIF0:BTY01XJ6P0OWin02d];@0U9BD0
Hf=S09:BTP27QhL0IVIV0410@01QHF40b<S809ZJVP1eMGD0>c/k04I6AP28R8P0YJFU051@D02MWId0
CTi>0<o?c`2h^;P0GemO09fMW@26QXH0HF5Q0:b/[01;Bd/0C4a<069RHP1]KFd0NGUi05mOG`13@d<0
Fe]K071`L02FUYH0Y:BT0410@01ADE40OGem06][J`2:RXX0FUYJ02PX:01JFUX0HV9R06a/K03<c<`0
Ti>C03dm?@1[Jf/0NWYj07MgM`37alL0>SXj03De=@1_Kfl0Fe]K092@T02d];@09bLW0492@P2RXZ80
T92@07]kN`2[Zj/0<c<c03Lg=`2XZ:P0][Jf092@T0003P1mOGd09RHV02h^;P1hN7P0/K6a07]kN`2N
WYh0=CDe05QHF01OGel0P82008Z:RP33`l<0HF5Q0P1EEED01`1IFET0F5QH08R8R01YJFT0;bl_06m_
K`1XJ6P03_ooool08@0n?Sh0FEUI0<71`@2ATI40_[jn05=CD`1IFET0Q8B406][J`1YJFT0IfMW06EU
I@1[Jf/0Ng]k065QH@2:RXX0ZZZZ07emO@1]KFd0GemO065QH@13@d<0H61P05=CD`17AdL0LW9b03Xj
>P1DE5@0Bd];065QH@1/K6`0G5aL04Y:BP0208N7Q`0W07UiN@1eMGD0Cdm?09bLW01@D500?Cdm04Y:
BP1OGel0IVIV05iNGP1XJ6P09RHV05QHF01[Jf/0H61P05ADE01nOWh0:RXZ05YJFP1^KVh0BTY:05UI
F@16ATH0Fe]K03Hf=P0o?cl0DU9B05QHF012@T80De=C05YJFP1CDe<0GemO09NGU`18B4P0NWYj08^;
R`1WIfL0DE5A0080GemO00`0G5aL05EEE@1gMgL0Ph>307alO036a/H0L71`05IFEP1:BTX0GEeM07Ad
M01EEED3065QH@0d059BDP1YJFT0PX:206EUI@1LG5`0Ng]k07IfMP1KFe/0HV9R061PH01`L700GemO
05]KF`1QHF40@d=306IVIP2;Rh/0UiNG061PH01RHV80Z:RX09>CT`1DE5@0KFe]08^;R`1]KFd0>c/k
05UIF@1DE5@0RXZ:09nOW`1lO7`0IVIV04Y:BP1MGEd0K6a/09bLW03De=@0IfMW07emO@1JFUX0/;2`
06=SH`1>CTh0F5QH07QhN01EEED0QHF507]kN`1ZJVX0@T9205ADE080GemO00T0De=C0410@01/K6`0
KVi^05]KF`1ADE40K6a/04M7A`0e=CD00P1=CDd09`1HF5P0Hf=S0<W9b@2BTY80FUYJ06EUI@1JFUX0
CDe=08F5Q@1[Jf/0Bd];04M7A`0Y:BT0?Shn04M7A`1EEED0RHV905UIF@1WIfL0Hf=S0820P01DE5@0
KVi^055AD@2[Zj/0GemO05=CD`1dM7@0OGem09FEU@1<C4`0GUiN04Q8B01JFUX0CTi>07UiN@1GEeL0
EEEE09FEU@0205aLG00@06EUI@2PX:00L71`07moO`1RHV80KFe]03lo?`1gMgL0J6QX03@d=00n?Sh0
?clo05]KF`1_Kfl0Lg=c06UYJ@80Jf][03l0F5QH055AD@1DE5@0<c<c05ADE01bLW80I6AT09BDU01T
I6@0AdM705aLG024Q8@0MWIf07YjNP29RHT0N7Qh08:2PP1TI6@0Cdm?06=SH`20P800E5AD05EEE@1D
E5@0@T9205iNGP1=CDd0DU9B04M7A`0l?3`0B4Q804a<C02PX:00Lg=c05iNGP1^KVh0QXJ606YZJP1g
MgL0Ph>307QhN026QXH0K6a/05aLG01nOWh0H61P05=CD`1DE5@0FEUI06e]K@1WIfL0NGUi07EeM@1T
I6@0NWYj06QXJ01EEED0>3Ph06UYJ@2/[:`0GUiN09NGU`2WYjL04?ooool000kooooo0:D0^KVi05QH
F01IFET0Ti>C0<K6aP1EEED0BTY:06e]K@2TY:@0E5AD03Lg=`0d=3@0GemO06UYJ@1GEeL0Rh^;04i>
CP1<C4`0JVYZ03dm?@0a<C40Fe]K03Lg=`2HV9P0OWin03<c<`1EEED0BDU906i^KP2PX:00AdM702/[
:`19BDT0B4Q804=3@`2NWYh0G5aL04I6AP1EEED0>c/k04];B`2UYJD0YZJV03Ti>@18B4P0<c<c0820
P03/k>`0QHF502l_;`1ZJVX0ATI609BDU02h^;P0<c<c0410@01@D500JVYZ0;^k^`1?Cdl051@D03/k
>`1LG5`0GEeM07emO@0/;2`08B4Q04M7A`1dM7@0NWYj08b<S00L71`0:BTY05]KF`2g]kL0KVi^09RH
V00g=cL0F5QH07YjNP2ZZZX0E5AD06m_K`16ATH0Lg=c0;:b/P2:RXX0B4Q80820P017AdL0RHV9092@
T01@D500WinO03lo?`18B4P0Rh^;05aLG01]KFd0WYjN03Hf=P0c<c<0IFEU08>3P`1lO7`0OWin02TY
:@1KFe/0ZZZZ07emO@21PH40LG5a02TY:@1IFET0N7Qh08V9R@2WYjL0TI6A034a<@1>CTh0P82009RH
V02IVIT0O7al03dm?@1>CTh0OGem09fMW@1aLG40P8200410@01CDe<0Zj^[09RHV01hN7P0Fe]K04a<
C01YJFT0Zj^[051@D02MWId0F5QH0451@@24Q8@0U9BD038b<P1SHf<0J6QX04E5A@1fMWH0De=C06e]
K@24Q8@0>SXj04U9B@25QHD0EeMG0492@P23Ph<0?Cdm06YZJP2IVIT0Ogmo05aLG02BTY800P1PH600
K`3De=@0EEEE061PH01[Jf/0H61P08:2PP2YZJT0DU9B04A4A01QHF40I6AT0;>c/`25QHD0;Bd]04i>
CP2=SHd0Z:RX0=WIf@1MGEd0;2`/04E5A@2;Rh/0bLW90;Ng]`0l?3`0@D5107UiN@2][Jd0_[jn05MG
E`0e=CD0FEUI0:BTY028R8P0SHf=03Ph>01EEED0O7al079bLP2c/k<0Q8B404E5A@28R8P0GemO07mo
O`1mOGd08R8R0451@@1XJ6P0Xj>S08R8R00g=cL0HF5Q05MGE`26QXH0YJFU03lo?`0i>CT0Ng]k0::R
XP1GEeL0SXj>04e=C@1SHf<0YJFU0=OGe`1YJFT0T92@04=3@`1JFUX0WIfM0;Bd]01UIFD0Ph>305eM
G@1PH600RXZ:07YjNP1DE5@0RXZ:05=CD`1EEED0TI6A05EEE@1;Bd/0F5QH03/k>`1<C4`0Shn?04M7
A`13@d<0FEUI04M7A`1EEED0OWin03Hf=P1:BTX0HF5Q03Xj>P1@D500H61P06YZJP0_;bl0JFUY04I6
AP0[:b/0Cdm?08J6QP0m?Cd0H61P03/k>`0>ooooo`1204U9B@1RHV80EeMG06i^KP1KFe/0GEeM07Qh
N01CDe<0=cLg065QH@1UIFD0@41004M7A`1iNGT0OWin079bLP1WIfL0OWin05eMG@1DE5@0>CTi030`
<01FEUH0?Shn03<c<`1[Jf/0Cdm?0451@@0m?Cd0IFEU05YJFP11@D40@T920861P@1bLW80QXJ608B4
Q01?Cdl0U9BD061PH015ADD0Hf=S08B4Q01`L700IfMW06UYJ@1LG5`0Lg=c07AdM01`L700De=C08b<
S00c<c<0EUIF07YjNP1QHF40JFUY04Q8B02DU9@0IFEU05aLG01[Jf/0E5AD03<c<`1HF5P0L71`0P1L
G5`0Y`31`L40C4a<07alO02?Shl0[:b/08Z:RP1_Kfl0NGUi0820P01IFET0AdM705aLG02RXZ80clo?
08V9R@1LG5`0C4a<0861P@20P800CDe=06e]K@1>CTh0QHF504M7A`1fMWH0JVYZ07AdM01JFUX0>c/k
055AD@1<C4`0EEEE04e=C@1WIfL0CTi>04E5A@18B4P0DE5A05QHF01EEED0HV9R04i>CP1QHF40BDU9
06QXJ01EEED0IVIV04=3@`1=CDd0DE5A05UIF@21PH40Q8B4055AD@1IFET0RHV904Y:BP1^KVh0I6AT
06=SH`2RXZ80NWYj03Xj>P0^;Rh0FEUI07=cL`1nOWh0I6AT059BDP0d=3@0C4a<07alO01gMgL0EUIF
061PH01>CTh0?Cdm07=cL`14A4@0?clo055AD@1kNg/0=CDe06][J`1;Bd/0JFUY08R8R01DE5@0KFe]
02l_;`1nOWh0LW9b02XZ:P1=CDd0MGEe03Ti>@1mOGd0G5aL0492@P1kNg/0;Bd]05YJFP1RHV80=SHf
04m?C`15ADD0MgMg04];B`1KFe/0FUYJ06MWI`1>CTh0EUIF0451@@1UIFD0E5AD04Q8B01BDU80=SHf
07]kN`1KFe/0UIFE04Y:BP2:RXX0=SHf04M7A`16ATH0LG5a059BDP0b<S80Fe]K07=cL`36a/H0Q8B4
04M7A`1CDe<0H61P04i>CP1OGel0TI6A07alO023Ph<0Kfm_06a/K01`L700H61P07EeM@1IFET0A4A4
06EUI@2?Shl0EUIF04I6AP11@D40]KFe07emO@26QXH0_Kfm065QH@2CTi<0Fe]K05UIF@14A4@0G5aL
06YZJP1aLG40Lg=c0080HV9R02T0MWIf06IVIP1aLG40RHV907EeM@2;Rh/0Jf][08Z:RP1YJFT0G5aL
065QH@1TI6@0Kfm_059BDP1gMgL0IfMW08Z:RP2JVYX0<c<c03Ti>@16ATH0E5AD09FEU@24Q8@0RXZ:
05=CD`0m?Cd0G5aL03lo?`19BDT0Hf=S07EeM@2=SHd0QHF505iNGP1EEED0TI6A09>CT`2@T900WYjN
0:2PX00@ooooo`003_ooool0o`2@T900PH61055AD@1SHf<0Ph>307]kN`0g=cL0C4a<05]KF`25QHD0
=3@d01lO7`12@T80ADE503Hf=P2TY:@0EUIF04=3@`28R8P0GUiN07EeM@28R8P0=CDe05aLG031`L40
MgMg08F5Q@1ZJVX0<30`08R8R02/[:`0GEeM08b<S00g=cL0?clo05UIF@1mOGd0YjNW0::RXP0M7Ad0
@D5104E5A@2?Shl0O7al055AD@0`<300?Shn059BDP2HV9P0QXJ603Ph>00e=CD0HF5Q08V9R@1iNGT0
IFEU03De=@1LG5`0W9bL0<G5a@1HF5P0Lg=c01hN7P1>CTh0IfMW05MGE`19BDT0Shn?02@T901GEeL0
QhN7055AD@1iNGT0XJ6Q02`/;01FEUH0GUiN03Ph>01/K6`0UIFE02HV9P17AdL0KFe]02d];@1aLG40
?Shn04A4A01YJFT0QhN703dm?@1TI6@0=3@d03Ph>01VIVH0NWYj0410@01JFUX0AdM7051@D02DU9@0
XZ:R04I6AP1WIfL0>CTi09NGU`2l_;`0?3`l030`<01OGel0IVIV09jNWP2k^k/0>3Ph04Y:BP1ZJVX0
F5QH0;:b/P3EeMD0=SHf03Ph>01NGUh0EEEE08f=S@2m_Kd0<C4a04E5A@1<C4`0LG5a08F5Q@2][Jd0
<S8b06ATI01aLG40NGUi0;Bd]01mOGd0BTY:05QHF023Ph<0JFUY0<S8b02][Jd0@D5105YJFP23Ph<0
J6QX08j>SP2PX:00@T9207IfMP1YJFT0RXZ:0<;2`P1CDe<0?Cdm07UiN@24Q8@0W9bL0<;2`P0g=cL0
92@T08N7Q`2XZ:P0b/[:0=cLg01ADE40BDU909nOW`2m_Kd0`<3009jNWP0f=SH0Cdm?0:BTY02e]KD0
QXJ603Xj>P12@T80EUIF09FEU@1XJ6P0WYjN06UYJ@1bLW80FEUI0<;2`P1DE5@0bLW904I6AP2<S8`0
/K6a06a/K01mOGd0LG5a02TY:@1hN7P0`l?305EEE@2/[:`0>SXj04m?C`30`<00I6AT09nOW`12@T80
C4a<08:2PP3Rh^80QHF507UiN@1HF5P0Ogmo0;Zj^P1FEUH0EEEE04Q8B01`L700[Jf]05=CD`1GEeL0
?Cdm05YJFP2TY:@0WYjN03hn?P1=CDd0De=C06EUI@1nOWh0L71`0<C4a03EeMD0^KVi08N7Q`2NWYh0
?clo04a<C01gMgL0IFEU05ADE02IVIT0I6AT04e=C@1/K6`0FEUI06UYJ@3Fe]H0Ph>303Ph>01^KVh0
I6AT08F5Q@39bLT0GUiN04I6AP005`1RHV80DU9B06YZJP3;bl/0EEEE03hn?P1iNGT0Cdm?09>CT`2k
^k/0>SXj0451@@1RHV80KVi^06MWI`2a/K40BTY:03<c<`14A4@0HF5Q07inOP2`/;00CTi>00kooooo
0340?Shn05MGE`15ADD0/;2`09NGU`0h>3P0>SXj05]KF`1<C4`0DE5A04U9B@17AdL0D51@061PH015
ADD0?Shn092@T01mOGd0K6a/05EEE@1HF5P0?Shn03@d=00b<S80=3@d04E5A@0c<c<0@d=303@d=00Z
:RX0DE5A04Q8B00i>CT0O7al04i>CP1CDe<0Hf=S071`L01_Kfl0Cdm?04=3@`0n?Sh0>SXj05IFEP1I
FET0De=C024Q8@1aLG40HF5Q0080GEeM00<0F5QH04=3@`16ATH00P0m?Cd0:@1ADE40BDU908F5Q@0m
?Cd0>SXj04=3@`1FEUH0CTi>061PH01YJFT0OGem07inOP2g]kL0CTi>0820P02FUYH0XZ:R0:n_[`29
RHT0RXZ:06ATI00a<C40JVYZ06QXJ027QhL0QHF507EeM@1SHf<0DE5A071`L022PX80DU9B06e]K@1O
Gel0Q8B4059BDP1kNg/0PH6109BDU01YJFT0B4Q80080Fe]K0;<0M7Ad04m?C`1bLW80CTi>05EEE@1[
Jf/0CTi>06][J`1RHV80SHf=069RHP1/K6`0GEeM05mOG`1BDU80K6a/055AD@1XJ6P0Bd];05YJFP1U
IFD0P82005UIF@1BDU80Lg=c01`L701_Kfl0NWYj0:>SX`2o_kl0RXZ:071`L01GEeL0S8b<07moO`27
QhL0Ti>C07IfMP1;Bd/0C4a<07UiN@1>CTh0E5AD03lo?`1WIfL0?clo04Q8B00d=3@0M7Ad06UYJ@1U
IFD0KVi^09FEU@13@d<0?3`l04A4A01;Bd/0Hf=S04E5A@29RHT0Q8B406e]K@26QXH0Cdm?03De=@1i
NGT0J6QX04=3@`2?Shl06QXJ03hn?P1CDe<0B4Q80:2PX01FEUH0SHf=06YZJP1YJFT0BTY:06=SH`0l
?3`0Shn?05iNGP1cLg<0@D5104U9B@1GEeL0=SHf0820P01ZJVX0_kno05ADE01IFET07AdM02/[:`1@
D500Bd];04M7A`1XJ6P0Rh^;03hn?P1/K6`0PX:205YJFP0f=SH0EUIF04Q8B01CDe<0Ng]k05=CD`1=
CDd0Hf=S06YZJP1LG5`0EUIF07moO`17AdL0DU9B055AD@1oOgl0@D5108b<S01VIVH0Rh^;07EeM@1U
IFD0OGem0861P@3MgMd0Hf=S07alO024Q8@0PX:2092@T029RHT0HF5Q08B4Q01MGEd0DU9B07AdM02U
YJD0SHf=05aLG01eMGD0Lg=c0<?3``1cLg<0>3Ph05mOG`1KFe/0JFUY03dm?@2MWId0QhN70:n_[`1^
KVh0BTY:05IFEP1JFUX0:BTY06MWI`23Ph<0W9bL03lo?`1JFUX0Ogmo04M7A`1;Bd/0LG5a09jNWP1M
GEd0FUYJ06YZJP1OGel0De=C0820P01>CTh0Q8B409FEU@0@ooooo`003_ooool0602>SXh0_;bl08F5
Q@1KFe/0NWYj0<[:bP1[Jf/0WIfM03De=@1SHf<0IfMW04M7A`1[Jf/0?Cdm038b<P0Z:RX0EUIF07Ad
M015ADD09BDU05IFEP1HF5P0@41005IFEP80?clo0;/0QXJ6061PH017AdL0KFe]02PX:01=CDd0Hf=S
055AD@0V9RH0JVYZ04I6AP1JFUX0VYZJ038b<P0_;bl0CDe=04a<C01[Jf/0[Zj^02PX:00l?3`0@T92
04Y:BP24Q8@0Ti>C02LW9`1UIFD0GemO04E5A@1lO7`0J6QX038b<P1>CTh0KVi^05QHF02EUID0:BTY
038b<P1aLG40S8b<05mOG`2k^k/0I6AT02l_;`23Ph<0Fe]K07inOP3Hf=P07AdM03De=@25QHD0De=C
09NGU`3Cdm<0>c/k03Ti>@1fMWH0FEUI0<g=c@1iNGT0820P03Xj>P2>SXh0RXZ:09nOW`0^;Rh0?Shn
04E5A@1kNg/0MWIf0861P@0W9bL0DE5A061PH02CTi<0IfMW09BDU00X:2P0Hf=S0;^k^`2@T900JFUY
08j>SP0o?cl0Kfm_0:FUY@21PH40GemO071`L00n?Sh0QXJ60:FUY@1`L700K6a/0:2PX01=CDd0K6a/
0:FUY@25QHD0Shn?0820P00k>c/0E5AD0:2PX01JFUX0Vi^K05MGE`1>CTh0Fe]K09bLW01oOgl0LG5a
03De=@18B4P0MgMg0:VYZ@1hN7P0Kfm_030`<019BDT0Cdm?09ZJVP1WIfL0DE5A034a<@18B4P0I6AT
09VIV@19BDT0Rh^;02l_;`1SHf<0e]KF09FEU@0i>CT0OWin055AD@1[Jf/0^[Zj08:2PP1;Bd/0HF5Q
05=CD`2GUiL0f=SH06=SH`1:BTX0LG5a04U9B@1nOWh0UYJF03`l?01PH600Cdm?04M7A`2ATI40Ph>3
03dm?@1RHV80KFe]07inOP33`l<0MWIf05MGE`2CTi<0QXJ60<O7a`2YZJT0<S8b05MGE`2l_;`0NWYj
0=KFeP1YJFT0>3Ph07IfMP1bLW80gMgM03/k>`12@T80NGUi08F5Q@2UYJD0Q8B403dm?@2CTi<0`L71
0080Ogmo03l0AdM706ATI02[Zj/0Rh^;0;Rh^01gMgL0>3Ph06YZJP2k^k/0ZJVY09NGU`1:BTX0;Rh^
06ATI00j>SX0F5QH08j>SP27QhL0BTY:07emO@2>SXh0GEeM0:ZZZP0V9RH0>c/k04=3@`25QHD0Ogmo
08F5Q@0b<S80B4Q804m?C`24Q8@0Hf=S0820P00m?Cd0ADE508j>SP2f][H0P82006=SH`1<C4`0F5QH
06e]K@2DU9@0Bd];05QHF00b<S80CDe=0820P01_Kfl0G5aL0:VYZ@1CDe<0@D51061PH029RHT0?Cdm
08b<S013@d<0CDe=06IVIP1kNg/03_ooool0K@18B4P0G5aL06ATI02e]KD0f][J05YJFP1QHF40NWYj
07IfMP1PH600KVi^0820P01bLW80Kfm_05eMG@1EEED0Ng]k03Xj>P0e=CD0=cLg05QHF00k>c/0?clo
03Ti>@10@400Cdm?03`l?017AdL0CTi>051@D01[Jf/0Cdm?03Xj>P1ZJVX0FUYJ06][J`1NGUh09BDU
04U9B@1>CTh0<C4a03dm?@1UIFD0F5QH03Ph>01NGUh0>CTi02HV9P0f=SH0ATI603De=@0k>c/0:b/[
03lo?`18B4P0A4A4059BDP15ADD0GUiN04a<C014A4@0EEEE04e=C@0i>CT0EeMG05eMG@1>CTh0BDU9
065QH@1FEUH0GemO06UYJ@1CDe<0E5AD069RHP1WIfL0LW9b03/k>`0o?cl0Hf=S08J6QP2EUID0DE5A
04M7A`0o?cl0EEEE06e]K@1BDU80GEeM051@D00k>c/0@T92051@D01gMgL0GUiN055AD@1NGUh0Fe]K
0410@01<C4`0CTi>05QHF00j>SX0De=C061PH018B4P0FEUI07IfMP1oOgl00P1BDU82071`L02104M7
A`1HF5P0DE5A07AdM01cLg<0N7Qh06ATI02=SHd0I6AT05aLG01mOGd0E5AD07alO01jNWX0/K6a0;Jf
]P1EEED0C4a<05QHF02=SHd0GEeM08>3P`2BTY80OGem0;2`/01YJFT0Y:BT07]kN`1iNGT0Hf=S061P
H016ATH0BDU904e=C@1NGUh0JFUY05UIF@17AdL0HF5Q04=3@`1QHF40G5aL04I6AP1PH600MGEe08Z:
RP1_Kfl0<c<c059BDP1_Kfl0I6AT0:NWY`1aLG40IVIV08>3P`0g=cL0L71`06i^KP14A4@0HF5Q05eM
G@2JVYX0F5QH05IFEP10@400KFe]02LW9`1YJFT0AdM707MgM`1GEeL0Ph>309RHV01SHf<0]KFe08>3
P`1]KFd0B4Q809NGU`0l?3`0DU9B05IFEP1bLW80F5QH04];B`1gMgL0PH610=CDe021PH40B4Q804e=
C@2/[:`0LG5a06EUI@1NGUh0NWYj0:FUY@1:BTX0H61P08^;R`1IFET0J6QX05]KF`1=CDd0KFe]07in
OP1RHV80H61P065QH@2XZ:P0Rh^;05iNGP1dM7@0Jf][065QH@1XJ6P0O7al04Y:BP19BDT0CDe=06MW
I`2CTi<0W9bL08J6QP20P800IFEU071`L01^KVh0ADE50080FEUI0280MGEe04m?C`0a<C40[jn_06MW
I`1kNg/0B4Q80:NWY`2JVYX0YjNW0:2PX01HF5P0UIFE092@T01RHV80ZJVY08>3P`1jNWX0Bd];08F5
Q@1UIFD0=cLg04=3@`16ATH0GemO06=SH`1UIFD0Fe]K04U9B@28R8P0S8b<0820P02LW9`0^;Rh4?oo
ool000kooooo00@0Ogmo06ATI02ATI40NWYj0P1IFET0DP1MGEd0][Jf08N7Q`12@T80EeMG04E5A@2Q
XJ40PH6102DU9@1FEUH0?Shn09JFUP2`/;008b<S03<c<`1^KVh0F5QH08R8R01QHF408B4Q03Xj>P1`
L700BDU90;Rh^01EEED08b<S06i^KP29RHT0CTi>0;fm_@1DE5@0DU9B0861P@1SHf<0MgMg09ZJVP10
@400Bd];09FEU@1ZJVX0Rh^;08f=S@0U9BD0@41008f=S@1DE5@0_[jn07EeM@0`<300De=C06EUI@1T
I6@0S8b<01hN7P0c<c<0C4a<04=3@`23Ph<0O7al02DU9@0f=SH0A4A406a/K01@D500FUYJ01lO7`0U
9BD0CDe=03Lg=`19BDT0GUiN03De=@1ADE40GUiN055AD@0d=3@0GemO030`<01_Kfl0T92@02XZ:P1W
IfL204a<C02L0:2PX025QHD0;2`/08B4Q01>CTh0ADE5075aL@1?Cdl0@41005aLG00m?Cd0<S8b07mo
O`1?Cdl0<c<c05mOG`1>CTh0MgMg0:RXZ014A4@0AdM705UIF@1MGEd0[jn_0;Jf]P1<C4`0CDe=08>3
P`22PX80Q8B409bLW016ATH0EEEE09fMW@1_Kfl0UiNG0:RXZ013@d<0?Shn09RHV01RHV80fMWI05mO
G`0h>3P0ADE507inOP2OWil0/;2`0410@00/;2`0Jf][0:BTY024Q8@0`l?304Y:BP0a<C40NWYj09nO
W`2DU9@0R8R802l_;`0o?cl0GEeM092@T01LG5`0WIfM03Hf=P0o?cl0QHF50:>SX`2=SHd0_;bl05=C
D`1GEeL0];Bd09fMW@37alL0gmoO05IFEP1MGEd0[Jf]0;2`/02/[:`0fm_K04Q8B01VIVH0`/;2092@
T030`<00CDe=03`l?01aLG40UiNG07UiN@2f][H0;Bd]04e=C@2:RXX0Z:RX0;Ng]`1lO7`0=3@d092@
T03GemL0UIFE08:2PP1=CDd0@41007EeM@2d];@0KFe]07IfMP12@T80dm?C0:ZZZP1=CDd0Y:BT06AT
I01GEeL0c/k>07QhN02JVYX0R8R80492@P2FUYH0J6QX071`L024Q8@0HF5Q08j>SP2PX:00E5AD06m_
K`1>CTh0DE5A06][J`2`/;00ATI605iNGP0/;2`0>SXj05UIF@2MWId0Q8B409nOW`23Ph<0QHF50<K6
aP28R8P0=SHf06=SH`1BDU80QHF50<S8b01;Bd/0;2`/0P1`L7006`2:RXX0[jn_03lo?`0h>3P0EEEE
08b<S02BTY80IFEU02h^;P18B4P0MgMg07]kN`1SHf<0R8R802d];@0h>3P0IVIV07AdM01@D500c<c<
05YJFP0o?cl0De=C09JFUP1<C4`0Z:RX061PH00204a<C00308>3Pooooooooooo00cooooo0280C4a<
04e=C@1LG5`0L71`0;Bd]01iNGT0PX:2059BDP0i>CT0EUIF05YJFP10@400>CTi06MWI`1gMgL0E5AD
05=CD`1TI6@0CDe=04Y:BP0i>CT09bLW04m?C`0m?Cd0@d=305QHF00c<c<0CTi>04A4A01LG5`0BDU9
03lo?`17AdL0Lg=c0P1mOGd06@20P800C4a<05aLG01@D500=cLg0410@01<C4`0EEEE04I6AP1GEeL0
@410059BDP17AdL0C4a<05iNGP1_Kfl0:b/[055AD@1^KVh0EEEE05UIF@0n?Sh0NGUi05MGE`1FEUH0
0P13@d<07`1EEED0@T9203<c<`17AdL0FUYJ09FEU@0i>CT0KVi^09RHV01jNWX0Fe]K04A4A013@d<0
IFEU04U9B@14A4@0EeMG09ZJVP3GemL0Jf][04e=C@17AdL0?Cdm03Xj>P18B4P0Cdm?02XZ:P0m?Cd0
AdM703De=@0l?3`00P15ADD02P0P8200<C4a04A4A01;Bd/0:BTY03@d=00m?Cd0;Bd]030`<00l?3`2
03Xj>P1V04a<C019BDT0@d=30492@P1/K6`0QXJ606EUI@0g=cL0=CDe03Lg=`0m?Cd0E5AD06QXJ01Y
JFT0H61P06ATI00K6a/0GemO051@D01nOWh0Ti>C07IfMP0];Bd0CTi>06e]K@1VIVH0HV9R06IVIP2E
UID0OWin09fMW@1[Jf/0Lg=c07YjNP1iNGT0M7Ad06EUI@1?Cdl0G5aL08B4Q01fMWH0WinO05YJFP1T
I6@0ATI606QXJ01jNWX0Bd];059BDP1OGel0cLg=09BDU013@d<0J6QX07QhN015ADD0HF5Q07AdM01N
GUh0M7Ad03/k>`10@400E5AD05eMG@1hN7P0G5aL08N7Q`1/K6`0C4a<04=3@`1CDe<08b<S08J6QP1h
N7P0FUYJ05IFEP1<C4`0DU9B05aLG01TI6@0H61P09FEU@1IFET0I6AT07IfMP1RHV80DU9B065QH@1d
M7@0J6QX07QhN01bLW80PH6109VIV@0k>c/0CTi>04e=C@1FEUH0ATI605mOG`1WIfL0HV9R0P1EEED0
6@1;Bd/0GEeM05]KF`18B4P0?Shn03lo?`0e=CD0BTY:06m_K`1CDe<0NGUi061PH01KFe/0UIFE05UI
F@2RXZ80M7Ad08V9R@1NGUh0PX:205ADE01gMgL0KVi^07inOP14A4@00P1DE5@09`1XJ6P0WinO051@
D01_Kfl0GemO09bLW01KFe/0=CDe06=SH`1FEUH0LG5a07QhN02>SXh0IFEU08>3P`35aLD0LW9b05eM
G@1NGUh0HV9R07=cL`22PX80Xj>S01dM7@1BDU80HV9R03@d=01:BTX0OWin08b<S01`L700J6QX06e]
K@1ADE40Shn?08:2PP2;Rh/0[Jf]0;6a/@0@ooooo`003_ooool0G@2BTY80R8R806e]K@24Q8@0][Jf
05UIF@1:BTX0TY:B08V9R@1eMGD0FUYJ03lo?`1VIVH0KVi^04e=C@2SXj<0FUYJ03lo?`1MGEd0EUIF
07AdM02BTY80=SHf05aLG024Q8@0EeMG09NGU`23Ph<09RHV069RHP2RXZ80Kfm_0<?3``0g=cL0<c<c
05MGE`1NGUh0PX:20820P00X:2P0?Cdm07AdM01PH600Kfm_051@D00m?Cd0HV9R09RHV01eMGD0KFe]
03/k>`0o?cl0OGem09^KV`1FEUH0DU9B04m?C`1GEeL0MgMg07=cL`13@d<0M7Ad03Ph>01QHF40MGEe
04A4A013@d<0MWIf04I6AP1>CTh0VIVI05UIF@1TI6@0Ng]k03@d=01@D500RXZ:04E5A@1gMgL0OGem
04i>CP1LG5`0Ng]k03lo?`1iNGT0KVi^03<c<`1bLW80SHf=05QHF01WIfL0Fe]K04M7A`0208b<S00`
04];B`1XJ6P0BTY:06MWI`2OWil0MWIf03lo?`1YJFT0CTi>059BDP2;Rh/0@D5103Ph>01WIfL0L71`
08^;R`2OWil0Bd];05=CD`28R8P0JVYZ0;Jf]P2PX:00?Cdm051@D01jNWX0HF5Q08N7Q`2BTY80@d=3
06MWI`2=SHd0JVYZ079bLP1kNg/0?3`l07UiN@2HV9P0KVi^08Z:RP1oOgl0@d=3071`L022PX80LW9b
0;Rh^01OGel0BDU90P1lO7`0Q@1MGEd0XJ6Q075aL@1IFET0RHV9071`L01cLg<0ZJVY038b<P0T92@0
Lg=c079bLP26QXH0a<C403dm?@0_;bl0V9RH0:2PX02HV9P0g=cL04Q8B019BDT0Ng]k09NGU`2:RXX0
Kfm_03`l?01DE5@0Rh^;071`L01dM7@0E5AD05IFEP28R8P0RXZ:05=CD`23Ph<0Jf][05mOG`2TY:@0
ZZZZ05=CD`2DU9@0KVi^0:ZZZP2[Zj/0EUIF07=cL`2>SXh0CTi>08>3P`1NGUh0FEUI08:2PP1DE5@0
EeMG0;6a/@1NGUh0UYJF06m_K`1HF5P0a<C409NGU`1LG5`0Kfm_05MGE`2GUiL0[Zj^05EEE@1iNGT0
GemO0861P@2FUYH0Cdm?04i>CP1aLG40GUiN08N7Q`1IFET0?3`l06IVIP1[Jf/0DE5A075aL@2QXJ40
[jn_0:NWY`2UYJD0^;Rh0;Ng]`17AdL0HV9R07AdM02@T900IFEU0:ZZZP1EEED0FEUI06UYJ@1`L700
K6a/0:FUY@1bLW80AdM7075aL@1HF5P0NGUi0;Jf]P19BDT0Bd];07EeM@1BDU80R8R80;:b/P1OGel0
@41009:BTP1EEED0KFe]06i^KP14A4@0>c/k06UYJ@2=SHd0L71`09^KV`0o?cl0?3`l04m?C`1PH600
G5aL07alO010@4003_ooool0:01CDe<0DE5A09ZJVP2CTi<0Rh^;04e=C@1WIfL0M7Ad04E5A@1=CDd0
Fe]K051@D01ADE40Jf][05]KF`15ADD0O7al07IfMP1UIFD0Bd];059BDP0a<C40AdM703Lg=`0k>c/0
G5aL03Lg=`0i>CT0>SXj05aLG01OGel0@D51030`<01JFUX0GEeM05QHF01KFe/0A4A405mOG`1=CDd2
03dm?@0_04i>CP1IFET0Bd];05mOG`0O7al0DU9B05ADE01>CTh0AdM7059BDP0l?3`0A4A403hn?P0m
?Cd0BTY:04];B`1EEED0?clo04];B`1?Cdl0CDe=04I6AP1<C4`0BDU906YZJP1@D500L71`04];B`1B
DU80KFe]07IfMP1YJFT0IFEU05eMG@1NGUh0EUIF05]KF`1WIfL0SXj>08:2PP1UIFD0F5QH05EEE@1H
F5P0EeMG0492@P1=CDd00P13@d<07`1:BTX0@D5106i^KP19BDT0CTi>05YJFP1;Bd/0?clo059BDP1;
Bd/0D51@04=3@`16ATH0AdM704A4A019BDT0H61P06i^KP1?Cdl0DU9B06a/K01^KVh0GemO059BDP1C
De<0>c/k05]KF`1:BTX0ATI605aLG01QHF400P14A4@0700F5QH0Ogmo06ATI02`/;00O7al06e]K@0W
9bL0?Shn07IfMP1BDU80NWYj07]kN`1BDU80ATI605iNGP18B4P0<c<c05]KF`1TI6@0Ogmo05EEE@1c
Lg<0G5aL07inOP1WIfL0SHf=05IFEP1hN7P204i>CP0V05EEE@1CDe<0Fe]K04A4A01lO7`0GemO05AD
E01SHf<0KFe]04U9B@1hN7P0OGem05eMG@2<S8`0EUIF059BDP1TI6@0A4A407AdM01YJFT0Ph>306UY
J@1MGEd0JVYZ08j>SP1WIfL0O7al07=cL`1_Kfl0Jf][04m?C`1ZJVX0HV9R07MgM`1LG5`0Vi^K05eM
G@1hN7P2051@D00?075aL@1/K6`0IFEU05iNGP1oOgl0N7Qh09VIV@25QHD0ATI605IFEP1TI6@0GemO
06a/K01hN7P0J6QX0080EUIF04<0K6a/07UiN@1NGUh0Ng]k05UIF@1?Cdl0Fe]K06IVIP17AdL0Ph>3
05mOG`2LW9`0Jf][04E5A@1eMGD0GEeM08n?S`14A4@0G5aL069RHP1eMGD0IVIV07YjNP1dM7@0Rh^;
04i>CP1ZJVX0FUYJ07QhN02BTY80I6AT08B4Q020P800Y:BT05IFEP0^;Rh0N7Qh061PH01eMGD0ADE5
0861P@22PX80Shn?07alO01MGEd0R8R808>3P`16ATH0Ogmo08J6QP1iNGT0>CTi05aLG01nOWh0EeMG
06IVIP1dM7@0PH61061PH01WIfL0FUYJ061PH022PX80NWYj07alO02NWYh0UiNG013ooooo000>oooo
o`0L05iNGP2VYZH0]kNg06EUI@1TI6@0XZ:R07alO02m_Kd0A4A405EEE@2SXj<0K6a/0<G5a@0a<C40
Cdm?04i>CP1SHf<0QHF505UIF@0C4a<0>CTi09NGU`1SHf<0QXJ604a<C00c<c<0FEUI0;fm_@80MgMg
02P0@D5104U9B@2PX:00];Bd05mOG`1bLW80?clo07=cL`2b/[80G5aL05MGE`1_Kfl0;bl_06i^KP2c
/k<0A4A406][J`2<S8`0A4A406i^KP1nOWh0DU9B07QhN01TI6@0A4A408b<S02@T900>c/k065QH@0T
92@0<c<c07]kN`0g=cL0CTi>04A4A01BDU80JFUY0:VYZ@1:BTX0;2`/0P1SHf<0cP2;Rh/0e]KF038b
<P0h>3P0HF5Q06=SH`2FUYH0QhN702/[:`0/;2`0G5aL05iNGP2;Rh/0RHV9028R8P0];Bd0BTY:07Ui
N@2WYjL0ATI6024Q8@0b<S80GEeM075aL@2VYZH0>CTi03Xj>P19BDT0PX:207emO@2FUYH0:BTY055A
D@1PH600F5QH05aLG02MWId0=cLg05QHF01PH600JFUY06a/K023Ph<0=CDe0861P@2_[jl0I6AT08b<
S02i^KT0AdM7055AD@31`L40GemO09^KV`2g]kL0EeMG06=SH`2e]KD0MGEe0<O7a`19BDT0@T9207Ui
N@22PX80P82008^;R`0i>CT0;bl_08>3P`2c/k<0K6a/06IVIP1ADE40BDU908:2PP2YZJT0GemO061P
H014A4@0HF5Q0::RXP2k^k/0EUIF07emO@0e=CD0De=C05iNGP26QXH0EUIF08b<S014A4@0Hf=S09:B
TP3Jf]X0UiNG0;Jf]P1HF5P0T92@0=CDe01RHV80RHV908:2PP0f=SH0WYjN0;>c/`1;Bd/0P82006=S
H`0a<C40TI6A09nOW`1;Bd/0F5QH04];B`0l?3`0_[jn06a/K01DE5@0K6a/06QXJ02;Rh/0`/;206EU
I@1oOgl0FUYJ07alO039bLT0I6AT03dm?@1iNGT0S8b<0;Jf]P1@D500<30`07UiN@1kNg/0Xj>S08^;
R`0f=SH0C4a<0861P@1SHf<0/K6a05iNGP0f=SH0K6a/08V9R@2^[Zh0Rh^;03De=@1WIfL0YJFU05UI
F@2OWil0S8b<03`l?00e=CD0?Cdm02/[:`1?Cdl0QHF507alO02c/k<0KFe]0<;2`P37alL0?Shn0451
@@2CTi<0J6QX08b<S02:RXX08B4Q03De=@1kNg/0VYZJ07YjNP1IFET0?Cdm03`l?02;Rh/0/k>c07=c
L`0o?cl0:BTY04e=C@20P800Z:RX06a/K01bLW80ATI605YJFP1VIVH0PX:203hn?P1YJFT0EeMG061P
H02LW9`0Ng]k04i>CP1lO7`0DU9B05MGE`1eMGD0LW9b3_ooool0S@1[Jf/0C4a<0:FUY@1HF5P0Shn?
06QXJ01gMgL0P82006YZJP1FEUH0H61P05ADE015ADD0FUYJ06EUI@14A4@0CTi>05UIF@1TI6@0HF5Q
059BDP0h>3P0DU9B04Y:BP16ATH0JVYZ03lo?`1CDe<0?Cdm07MgM`1TI6@0ATI604=3@`1FEUH0FUYJ
06i^KP1aLG40CTi>07MgM`12@T80?clo04i>CP1_Kfl0EUIF05]KF`1`L700:BTY06a/K01dM7@0LG5a
071`L022PX80<c<c05aLG01oOgl0Cdm?05UIF@16ATH0NWYj03dm?@1MGEd0F5QH04U9B@14A4@0Bd];
03dm?@19BDT0F5QH09BDU011@D40MgMg08j>SP2EUID0G5aL04m?C`1CDe<0LW9b05EEE@18B4P0IfMW
0:ZZZP3AdM40Ng]k069RHP1?Cdl0D51@05eMG@1<C4`0H61P03Xj>P1;Bd/0>c/k04Q8B019BDT0A4A4
04];B`0X:2P0?Cdm03`l?01:BTX0=3@d04a<C00j>SX0?Shn03@d=013@d<0C4a<069RHP1KFe/0ADE5
04Y:BP1WIfL0KVi^07alO015ADD0GemO020P801:BTX0CDe=05YJFP1PH600>c/k04];B`1OGel06a/K
04U9B@0m?Cd0W9bL09>CT`1RHV80;Bd]05ADE02l_;`0>SXj07alO01TI6@0R8R803hn?P1JFUX0?Cdm
02d];@0206ATI00n0410@011@D40JFUY04e=C@1WIfL0HF5Q096AT@1=CDd0DE5A04e=C@26QXH0NWYj
0492@P1=CDd0BTY:0<c<c02@T900>CTi03`l?01RHV80ATI607UiN@1^KVh0Ng]k07EeM@0a<C40DU9B
07EeM@2KVi/0XJ6Q092@T01oOgl0Ph>30861P@1PH600M7Ad034a<@2:RXX0LW9b07=cL`2;Rh/0QXJ6
08F5Q@1PH600YZJV05eMG@2?Shl0D51@08b<S01gMgL0I6AT07UiN@1fMWH0Hf=S06MWI`29RHT0X:2P
0<S8b01XJ6P0GemO07=cL`1lO7`204Q8B01706=SH`1FEUH0JFUY055AD@1LG5`0MGEe05ADE01PH600
J6QX04U9B@27QhL0W9bL065QH@1KFe/0F5QH092@T01NGUh0G5aL0:6QX@1<C4`0FUYJ051@D01cLg<0
KVi^06e]K@1DE5@0HF5Q07UiN@1kNg/0GemO07=cL`1UIFD0YjNW096AT@1dM7@0QXJ609NGU`2;Rh/0
DU9B04I6AP25QHD0N7Qh08b<S01BDU80Kfm_071`L02<S8`0V9RH07UiN@1hN7P0_;bl07emO@2b/[80
][Jf0:FUY@1CDe<0LW9b06UYJ@0`<300ATI605]KF`1eMGD0IVIV06YZJP1FEUH0@41006ATI029RHT0
J6QX0;:b/P2CTi<04?ooool000kooooo08<0YjNW05MGE`1eMGD0X:2P08N7Q`17AdL0CTi>08J6QP2A
TI40Hf=S0451@@12@T80KVi^08B4Q00j>SX0C4a<04I6AP1^KVh0Kfm_02<S8`0_;bl0Fe]K04i>CP2<
S8`0>3Ph028R8P0m?Cd0GemO06i^KP25QHD0<30`03Ti>@1?Cdl0MWIf075aL@2LW9`0>SXj03dm?@1f
MWH0@T9207inOP2][Jd0;Rh^0451@@20P800JVYZ08B4Q02SXj<09bLW071`L02<S8`0LW9b0;Rh^01[
Jf/07AdM03Xj>P27QhL0YjNW09JFUP0U9BD0=SHf061PH01YJFT0S8b<05QHF00d=3@0?clo07emO@29
RHT0QhN707AdM00T92@0DE5A0;:b/P2;Rh/0HF5Q07]kN`0P8200PX:20;Zj^P1VIVH0GEeM06m_K`10
@400IFEU0:NWY`1DE5@0JFUY08B4Q00e=CD0J6QX0<S8b01DE5@0MWIf0410@01;Bd/0KVi^09:BTP1E
EED0UiNG02l_;`0c<c<0Y:BT05]KF`1@D500NWYj02/[:`1EEED0[jn_06][J`1PH600Jf][04e=C@1m
OGd0XJ6Q04M7A`1GEeL0Fe]K04e=C@1UIFD0J6QX06EUI@1JFUX0GEeM05IFEP25QHD0[Jf]05MGE`17
AdL0EUIF06YZJP0209VIV@0J04m?C`1^KVh0LG5a0:f][@2MWId0H61P0451@@1QHF40EeMG06UYJ@2X
Z:P0?clo04I6AP22PX80OGem07UiN@2>SXh0<30`051@D01ZJVX0SXj>059BDP2/[:`0;Bd]04a<C01m
OGd205iNGP1e0<K6aP1MGEd0De=C061PH02MWId0V9RH0<_;b`1dM7@0N7Qh09VIV@1eMGD0cLg=0;Ng
]`0e=CD0E5AD0:2PX01jNWX0c<c<06EUI@0m?Cd0E5AD0:RXZ02MWId0aLG504I6AP0o?cl0D51@09fM
W@2VYZH0Ng]k02@T901ADE40S8b<0:>SX`2LW9`0FEUI03Xj>P1iNGT0VYZJ09:BTP1UIFD0E5AD08n?
S`25QHD0WYjN09nOW`1=CDd0@T920<g=c@1SHf<0/K6a055AD@17AdL0IfMW079bLP1NGUh0NWYj028R
8P1FEUH0_[jn075aL@20P800?Shn04I6AP1?Cdl0IVIV0492@P1kNg/0Z:RX0:b/[02a/K40Ph>30:JV
YP2QXJ40>c/k051@D029RHT0IVIV059BDP20P800=SHf07QhN02h^;P0F5QH04A4A01@D500EUIF06EU
I@1kNg/0<S8b0451@@1RHV80>c/k08^;R`1SHf<09bLW030`<01^KVh0GemO06IVIP2LW9`0:BTY04i>
CP1EEED0D51@03<c<`2FUYH0NWYj01`L701UIFD0L71`05YJFP2EUID0De=C02d];@1;Bd/0LW9b00ko
oooo00/0G5aL0410@01gMgL0FEUI07moO`2<S8`0XZ:R051@D00a<C40?3`l04U9B@0204a<C004069R
HP1`L700=3@d05mOG`80Q8B401l0I6AT05mOG`0l?3`0CTi>03lo?`0e=CD0CTi>03hn?P0k>c/0=3@d
055AD@1LG5`0@D5104U9B@1OGel0Cdm?04i>CP1DE5@0DU9B05]KF`12@T80@41003Hf=P1[Jf/0Hf=S
05iNGP1_Kfl0<c<c0820P01ZJVX0KFe]0080KVi^00`0=3@d04e=C@1^KVh0OGem06i^KP1;Bd/0S8b<
05=CD`1`L700F5QH055AD@18B4P205YJFP0:06IVIP1GEeL0/k>c04i>CP1cLg<0MgMg07moO`1^KVh0
KFe]07EeM@80JVYZ02L0B4Q805iNGP2JVYX0e]KF08Z:RP1eMGD0FUYJ079bLP1`L700EEEE07=cL`1B
DU80MWIf05ADE01IFET0Jf][07=cL`1QHF40B4Q804Y:BP1ADE40De=C04A4A01SHf<0BTY:04];B`1C
De<0CDe=04m?C`1OGel0Kfm_04m?C`1CDe<0K6a/07inOP1`L700@T92051@D00M7Ad00P1:BTX0L019
BDT0IFEU05MGE`0m?Cd0C4a<02HV9P1_Kfl0Hf=S08V9R@23Ph<0NWYj02/[:`15ADD0Jf][06QXJ02=
SHd0NWYj06][J`0K6a/0CDe=04U9B@17AdL0=cLg03Xj>P1<C4`0=3@d06IVIP0j>SX0Ph>305EEE@1M
GEd0?Shn055AD@0[:b/0AdM70410@012@T80DE5A03Ph>01]KFd0DU9B03/k>`1>CTh0HF5Q03@d=01U
IFD0>c/k051@D01fMWH0=cLg04];B`1FEUH0ADE504];B`1YJFT0VYZJ06][J`1QHF40JVYZ07moO`1<
C4`0BDU904a<C01dM7@0DU9B04=3@`1:BTX0>c/k04M7A`1?Cdl0];Bd04a<C01CDe<0EEEE0451@@1B
DU80IVIV055AD@1BDU80I6AT05=CD`1JFUX0P820059BDP17AdL0EUIF04M7A`1PH600UYJF05EEE@11
@D40D51@05eMG@1BDU80De=C06a/K018B4P0Cdm?04a<C01:BTX0D51@07alO011@D40OGem055AD@1C
De<0Kfm_05ADE01oOgl0Ph>308:2PP1<C4`205]KF`0F069RHP1ZJVX0UIFE04i>CP1GEeL0Fe]K09:B
TP1lO7`0E5AD06MWI`2^[Zh0hn?S05mOG`0f=SH0Ng]k05iNGP1fMWH0PX:209^KV`1mOGd0Ogmo0;:b
/P80R8R801D0O7al05YJFP1lO7`0Y:BT0;Rh^00k>c/0@d=3075aL@1JFUX0KVi^06UYJ@1/K6`0MGEe
07MgM`1?Cdl0BTY:06a/K01IFET0PX:206QXJ01kNg/04?ooool000kooooo00/0WYjN07emO@1GEeL0
PH610:b/[01ZJVX0B4Q805QHF01@D500W9bL04=3@`0203lo?`80Hf=S05L0_Kfm05EEE@1CDe<0I6AT
07YjNP1YJFT0OWin01TI6@1SHf<0alO707UiN@2][Jd0Shn?02DU9@1iNGT0Q8B409RHV02c/k<07AdM
05MGE`1bLW80Lg=c08^;R`1gMgL09BDU05IFEP2;Rh/0QhN706][J`10@4007QhN04a<C01EEED0CTi>
05mOG`0_;bl0BTY:07]kN`2TY:@0Cdm?05EEE@0l?3`0DE5A06IVIP1hN7P0=3@d05mOG`0Z:RX0NWYj
05aLG00];Bd0@d=3079bLP14A4@0NGUi08F5Q@0e=CD0>3Ph05QHF00c<c<0Fe]K08Z:RP0W9bL0BDU9
05MGE`1VIVH0J6QX09JFUP0g=cL0CTi>05YJFP1:BTX0GUiN09FEU@0g=cL0AdM705MGE`2>SXh0]kNg
069RHP0Z:RX0PH61055AD@1`L700bLW906MWI`0H61P00P1QHF40P02:RXX0Z:RX034a<@0n?Sh0GEeM
07EeM@2ZZZX0PH6103@d=018B4P0M7Ad08Z:RP2][Jd0L71`03hn?P1jNWX0SHf=06e]K@2CTi<0O7al
02h^;P1IFET0^KVi08n?S`2IVIT0KFe]04=3@`1`L700]kNg08:2PP2i^KT0JVYZ05IFEP2@T900ZZZZ
071`L02e]KD0GemO04];B`1iNGT0ZZZZ07]kN`36a/H0ADE5061PH02e]KD0NWYj07emO@22PX8092@T
0451@@1fMWH0JFUY06][J`2:RXX09BDU05=CD`1IFET0L71`08B4Q029RHT0:BTY06MWI`1PH600O7al
08F5Q@1kNg/08b<S08F5Q@3Ti>@0YjNW07IfMP19BDT0=cLg0820P031`L40H61P07EeM@1KFe/0CTi>
071`L02QXJ40DU9B08J6QP17AdL0^k^k0:JVYP0l?3`0F5QH08^;R`19BDT0VYZJ07emO@14A4@0T92@
06EUI@1VIVH0U9BD04=3@`2LW9`0GEeM08N7Q`2i^KT0FUYJ04=3@`22PX80FUYJ0;jn_P2;Rh/09BDU
06MWI`1CDe<0Q8B40;jn_P0c<c<0:b/[07inOP1/K6`0^KVi03De=@1HF5P0/K6a09BDU015ADD0WYjN
09bLW02GUiL0^;Rh0P2LW9`0802b/[80>SXj065QH@1XJ6P0CDe=05iNGP2k^k/0@T9204U9B@2OWil0
D51@065QH@1/K6`0;bl_04i>CP2^[Zh0JVYZ08b<S020P800?Cdm04i>CP20P800Kfm_08:2PP26QXH0
Bd];0451@@29RHT0CTi>0861P@22PX80;Bd]0P0c<c<0102;Rh/0MGEe0:6QX@0h>3P203lo?`04069R
HP1XJ6P0OWin02l_;`kooooo03H0AdM704I6AP21PH40A4A405aLG01>CTh0FEUI07YjNP1[Jf/0>SXj
05mOG`1^KVh0J6QX07QhN01MGEd0=CDe06i^KP1ZJVX0E5AD04E5A@1OGel0?Cdm05=CD`12@T80@D51
069RHP0/;2`0@41003hn?P1WIfL0MGEe051@D01<C4`0EEEE06=SH`1QHF40H61P02/[:`1BDU80?Cdm
03`l?00g=cL0D51@05ADE014A4@0EeMG02TY:@1@D500F5QH06EUI@1^KVh0EeMG03Hf=P0o?cl204I6
AP0[055AD@1IFET0EUIF04Q8B01CDe<0BTY:04i>CP1OGel0C4a<04Y:BP1gMgL0IfMW08F5Q@1GEeL0
LG5a08b<S01bLW80ADE506EUI@1/K6`0P82007inOP1_Kfl0OGem09:BTP2LW9`0MgMg05QHF017AdL0
Fe]K065QH@1;Bd/0E5AD04Y:BP0n?Sh0>c/k04=3@`1cLg<0C4a<0492@P1BDU80B4Q80410@00204M7
A`1:05IFEP12@T80Cdm?04i>CP14A4@0?Shn069RHP1_Kfl0Fe]K04a<C01oOgl0OWin07moO`1EEED0
Hf=S03<c<`1<C4`0Fe]K05=CD`25QHD0GEeM04a<C01UIFD0:BTY08b<S015ADD0M7Ad0:RXZ02/[:`0
<c<c06][J`2PX:00SHf=09ZJVP2=SHd0GemO01TI6@1ADE40Lg=c07emO@18B4P0?Shn06a/K00Y:BT0
Bd];038b<P1gMgL0D51@05]KF`1GEeL0Jf][03dm?@1ADE40B4Q80492@P1IFET0:2PX08N7Q`1XJ6P0
@D5106EUI@0k>c/0>CTi07QhN01:BTX0De=C051@D00O7al0AdM705QHF00i>CT0LW9b04e=C@1GEeL2
059BDP0L03/k>`1LG5`0>3Ph09jNWP1iNGT0DU9B0451@@1PH600KVi^03hn?P1MGEd0Cdm?08V9R@1H
F5P0KVi^07AdM01XJ6P0^;Rh069RHP0e=CD0HV9R08R8R01VIVH0NWYj055AD@1OGel0CTi>05iNGP80
>3Ph04L0Bd];04E5A@1CDe<0LG5a06=SH`1ZJVX0C4a<07IfMP19BDT0H61P059BDP1TI6@0@d=308^;
R`1DE5@0Rh^;05aLG017AdL0IVIV0410@02ZZZX0HV9R04e=C@1SHf<0Kfm_08j>SP1mOGd0K6a/09ZJ
VP1BDU80I6AT059BDP1gMgL0/[:b075aL@2KVi/0Kfm_05]KF`14A4@0B4Q807]kN`1aLG40Hf=S03/k
>`1[Jf/0NWYj071`L01fMWH0TI6A0;Bd]02FUYH0FEUI07IfMP2:RXX0RHV905ADE01QHF40GEeM03Ti
>@1ADE40JFUY08V9R@16ATH0De=C05QHF01UIFD0P82009FEU@20P800VYZJ09FEU@0@ooooo`003_oo
ool0;`1aLG40ZJVY0:BTY01^KVh0NGUi08n?S`2>SXh0^;Rh03Xj>P1UIFD0KVi^05eMG@1oOgl09BDU
03@d=01SHf<0PH6106UYJ@0m?Cd0BTY:08^;R`2CTi<0F5QH04A4A00n?Sh0;Rh^07EeM@2RXZ80@410
03Xj>P16ATH0Xj>S0:FUY@14A4@0=CDe05aLG00g=cL0V9RH0<G5a@0h>3P0=cLg06QXJ01@D500W9bL
0:^[Z`0M7Ad0=CDe0080EUIF07`0X:2P05MGE`0K6a/0DU9B09ZJVP1hN7P0U9BD04A4A012@T80GEeM
07QhN01KFe/0`L7102`/;01PH600SXj>08J6QP2EUID0_Kfm04E5A@10@400X:2P075aL@2;Rh/0Y:BT
03lo?`0_;bl0U9BD06a/K02[Zj/0MgMg03Lg=`13@d<0IfMW0861P@2n_[h0Jf][030`<010@400O7al
06UYJ@2JVYX0B4Q802d];@1[Jf/0Q8B405UIF@2OWil0@d=302/[:`1_Kfl0[Jf]065QH@1mOGd0ATI6
04U9B@2UYJD0LG5a04e=C@2MWId0=cLg08B4Q02VYZH0CDe=05]KF`1bLW80Cdm?092@T02WYjL0ATI6
05EEE@1/K6`0AdM708F5Q@2i^KT0SXj>06m_K`1eMGD0?3`l069RHP2f][H0@T9208B4Q01NGUh0@T92
09^KV`20P800AdM706][J`12@T80KFe]09BDU025QHD0A4A4069RHP1IFET0Q8B40<30`02RXZ80@d=3
06e]K@1WIfL0SHf=0<C4a01:BTX0:b/[05UIF@0m?Cd0VYZJ0;Fe]@0j>SX0<C4a07=cL`1]KFd0Vi^K
0<o?c`17AdL0>SXj08N7Q`1[Jf/0ZJVY0;Ng]`0V9RH0?clo0`2UYJD07P14A4@0>CTi04a<C02IVIT0
IVIV0:FUY@1FEUH0ATI605EEE@1lO7`0N7Qh0=;BdP1=CDd0MWIf0:^[Z`1QHF40NWYj0:b/[00/;2`0
LW9b0<71`@1`L700_Kfm04];B`1JFUX0SXj>07alO03<c<`0=cLg04A4A080bl_;03`0VIVI07moO`1A
DE40Cdm?09bLW01DE5@0Y:BT04i>CP1lO7`0ZJVY07UiN@1RHV80LG5a034a<@29RHT0QXJ606ATI038
b<P0Bd];03dm?@0a<C40FEUI06m_K`1GEeL0HV9R08^;R`28R8P0>c/k06QXJ027QhL0;Rh^07AdM027
QhL0GEeM07alO02KVi/0AdM7065QH@2d];@0IFEU04Q8B019BDT0Cdm?0820P033`l<0H61P04];B`10
@400AdM706e]K@2YZJT0SHf=04=3@`1>CTh0BDU905EEE@1oOgl0;bl_02<S8`18B4P2051@D0080:6Q
X@1XJ6P0;Rh^05mOG`1GEeL0HV9R09^KV`1>CTh>ooooo`0Z04U9B@12@T80P82005UIF@1_Kfl0MGEe
08F5Q@1VIVH0@d=303@d=01>CTh0D51@04=3@`1FEUH0E5AD03Xj>P1OGel0M7Ad06EUI@1CDe<0FUYJ
03Hf=P19BDT0BTY:04E5A@1KFe/0;Rh^05=CD`0k>c/0HF5Q05YJFP19BDT0De=C05]KF`1gMgL0QHF5
075aL@0^;Rh0GUiN04];B`0c<c<0AdM70P1TI6@0H@1EEED0O7al03Lg=`1]KFd0EeMG05IFEP1<C4`0
EEEE02LW9`0o?cl0EEEE069RHP1NGUh0=CDe06QXJ01CDe<0FUYJ059BDP19BDT0DE5A05ADE01=CDd0
Cdm?05MGE`2;Rh/0?3`l09>CT`2d];@0YjNW06][J`1ADE40GemO06i^KP1ZJVX0?Cdm06a/K02][Jd0
e=CD07]kN`1YJFT0EeMG05eMG@1IFET0EEEE061PH01:BTX0B4Q803Xj>P0k>c/0De=C04a<C015ADD0
=cLg04U9B@0k>c/0EEEE0492@P1<C4`0=cLg04E5A@0e=CD0BTY:03hn?P1OGel0HF5Q04M7A`1=CDd0
DE5A05mOG`1_Kfl0EUIF06m_K`0c<c<0HV9R05YJFP1>CTh0MWIf07EeM@1?Cdl0EUIF02TY:@1lO7`0
EeMG06MWI`2BTY80OGem02<S8`1=CDd0Z:RX06ATI02EUID0TI6A08V9R@1LG5`0F5QH06e]K@1<C4`0
0P1ADE40Q`1BDU80?Cdm05IFEP1:BTX0QXJ606=SH`1MGEd0CTi>05iNGP10@400C4a<055AD@1BDU80
LG5a05YJFP2>SXh0KVi^04];B`11@D40J6QX05=CD`27QhL0D51@05=CD`1KFe/0@410065QH@1MGEd0
JFUY0:2PX01iNGT0RXZ:06IVIP1iNGT0@T92055AD@0[:b/0CDe=05IFEP1PH600FEUI03dm?@19BDT0
@D5109RHV00l?3`0RHV905=CD`2;Rh/0NGUi05ADE01IFET0KVi^03Hf=P15ADD0H61P07emO@2g]kL0
J6QX04e=C@1SHf<0RXZ:04E5A@13@d<0H61P05YJFP1ZJVX0ADE506MWI`29RHT0G5aL06m_K`1KFe/0
IVIV065QH@1WIfL0E5AD07EeM@1NGUh0XJ6Q065QH@1;Bd/0Kfm_05eMG@23Ph<0IfMW06e]K@20P800
QhN705ADE01QHF40JVYZ07inOP1KFe/0UIFE079bLP1hN7P0YjNW09jNWP2d];@0Xj>S0;Vi^@1?Cdl0
<30`08j>SP1HF5P0IFEU04m?C`1dM7@0Ti>C07inOP25QHD0K6a/08V9R@2d];@0HV9R09>CT`2MWId0
Z:RX04=3@`1aLG40JVYZ04=3@`1CDe<0HV9R06a/K01eMGD0GUiN04Q8B014A4@0Lg=c0:BTY02ATI40
YZJV08f=S@0@ooooo`003_ooool0201^KVh0GemO07MgM`1iNGT0FUYJ04];B`1CDe<0[jn_0P1:BTX0
[`1RHV80Vi^K0=GEe@1CDe<0;2`/04a<C01aLG40ZZZZ03Lg=`0Z:RX0@T9209jNWP23Ph<0JFUY034a
<@17AdL0?clo0:^[Z`2PX:00W9bL01HF5P16ATH0IFEU07IfMP1ZJVX0X:2P03Ph>00d=3@0Lg=c05mO
G`1^KVh0MWIf020P800o?cl0T92@05eMG@1UIFD0GEeM01hN7P1OGel0N7Qh06a/K025QHD0>SXj030`
<01EEED0L71`05YJFP1dM7@0?3`l03Ti>@1YJFT0M7Ad06a/K01OGel0:2PX04i>CP1?Cdl0OGem05AD
E01LG5`092@T05MGE`2QXJ40FEUI04E5A@1ADE40<c<c06e]K@2UYJD0@D51051@D019BDT0>c/k06UY
J@2^[Zh0ADE504=3@`1IFET0F5QH071`L02BTY80>SXj0492@P1dM7@0Bd];09VIV@1WIfL0@41005=C
D`1^KVh0AdM707inOP0e=CD0>c/k05iNGP11@D40H61P0:VYZ@17AdL0D51@07emO@1:BTX0VYZJ09>C
T`0l?3`0C4a<096AT@1VIVH0PX:20:6QX@0l?3`0CDe=08Z:RP2YZJT0/[:b0;Bd]00k>c/0>3Ph08^;
R`1oOgl0b/[:06m_K`0l?3`0F5QH08Z:RP2BTY80^k^k030`<00j>SX0Shn?0:j^[P2CTi<0XJ6Q04U9
B@1CDe<0RHV90=CDe020P800N7Qh04=3@`1dM7@0^KVi0:>SX`1EEED0X:2P02PX:01PH600aLG508^;
R`1XJ6P0[Zj^05QHF026QXH0a/K60<30`01YJFT0c/k>059BDP1mOGd0dM7A07moO`1nOWh0PX:20492
@P2][Jd0^[Zj06=SH`1iNGT0D51@04];B`2DU9@0XZ:R059BDP2][Jd00P1<C4`0F`2l_;`0Ph>307in
OP1hN7P0D51@08n?S`38b<P0QHF506IVIP13@d<0CTi>0::RXP1lO7`0B4Q806IVIP1fMWH0b<S808Z:
RP0k>c/0MGEe051@D02@T900Ti>C04E5A@1hN7P0JFUY04m?C`36a/H0LG5a05=CD`2YZJT0Jf][08N7
Q`2FUYH0:RXZ05iNGP1?Cdl0I6AT0:j^[P1ZJVX07AdM024Q8@0e=CD0<c<c03Ti>@0n?Sh0<C4a08F5
Q@1XJ6P0R8R80;jn_P0l?3`0ADE509RHV024Q8@0Y:BT0<G5a@1GEeL0CTi>07emO@1iNGT0LW9b03Ti
>@1;Bd/0D51@08N7Q`23Ph<0T92@04M7A`0^;Rh0G5aL04e=C@1jNWX0G5aL07]kN`0Y:BT0EEEE06m_
K`1QHF40E5AD092@T019BDT0?clo051@D01WIfL0E5AD07QhN013@d<0Cdm?09bLW01iNGT03_ooool0
MP16ATH0ADE507emO@1IFET0MWIf04i>CP1CDe<0Jf][055AD@1>CTh0JFUY05YJFP1<C4`0NWYj08R8
R01aLG40H61P08N7Q`1gMgL0F5QH07=cL`1CDe<0G5aL05=CD`1ADE40NWYj051@D010@400?clo04I6
AP1IFET0B4Q804i>CP1OGel0M7Ad06e]K@1TI6@0:2PX05eMG@0c<c<0:2PX03Ti>@1;Bd/0CDe=04m?
C`1GEeL09BDU06i^KP1UIFD0N7Qh05iNGP2CTi<0:b/[04U9B@1fMWH0@T92055AD@0l?3`0FUYJ04=3
@`1KFe/0DU9B04Y:BP1WIfL0CDe=04I6AP23Ph<0RHV90;Vi^@1@D500J6QX07alO02YZJT0MgMg08N7
Q`1mOGd0GUiN06][J`29RHT0O7al09nOW`3dm?@0NWYj07MgM`1dM7@0KFe]06UYJ@1FEUH0M7Ad04a<
C01[Jf/0DU9B04i>CP1`L700LG5a05UIF@0g=cL0CDe=05EEE@1bLW80CDe=05]KF`1ADE40@d=303Ph
>01BDU80AdM7061PH01YJFT0G5aL04m?C`1SHf<0MWIf07YjNP1KFe/0IFEU03Ph>011@D4204a<C00I
06MWI`16ATH0Bd];061PH00E5AD0MGEe04A4A01CDe<0Jf][08N7Q`1?Cdl0G5aL0::RXP1;Bd/0MgMg
079bLP2@T900JFUY0:>SX`1lO7`0F5QH06QXJ01BDU80EUIF05MGE`0205UIF@2309RHV01lO7`0UIFE
06e]K@1_Kfl0IfMW09jNWP21PH40L71`0820P00j>SX0/K6a09FEU@0c<c<0EEEE05eMG@1DE5@0Ogmo
05YJFP1KFe/0IfMW04i>CP11@D40Cdm?04M7A`0o?cl0IVIV0:RXZ01OGel0EUIF05aLG021PH40CTi>
08F5Q@1mOGd0O7al05=CD`17AdL0Cdm?03De=@1?Cdl0A4A408J6QP15ADD0Jf][075aL@0c<c<0EUIF
06EUI@19BDT0IFEU05YJFP1MGEd0OGem06UYJ@0c<c<0B4Q8055AD@1=CDd0?Shn071`L01PH600J6QX
05ADE01YJFT0LW9b06e]K@1ZJVX0KVi^05eMG@22PX80X:2P06ATI028R8P0GemO08V9R@1FEUH0Cdm?
08B4Q01/K6`0Xj>S07]kN`2@T900H61P06MWI`2CTi<0M7Ad06=SH`29RHT0FUYJ075aL@1LG5`0TY:B
09fMW@1OGel0SHf=0:6QX@31`L40?Shn034a<@2PX:00De=C061PH01aLG40Vi^K0820P02:RXX0[:b/
09>CT`2b/[80TY:B065QH@2VYZH0VIVI0;Zj^P1>CTh0GUiN0861P@19BDT0GEeM07QhN028R8P0Lg=c
08J6QP1OGel0D51@092@T02QXJ40TY:B0<O7a`2KVi/04?ooool000kooooo00h0b/[:06MWI`1SHf<0
WYjN09nOW`1;Bd/0D51@07AdM02:RXX0Xj>S03Hf=P0c<c<0KVi^0:b/[080SHf=02l0B4Q804A4A01f
MWH0=CDe065QH@1TI6@0Bd];05IFEP1XJ6P0>CTi06e]K@0b<S80?Shn07YjNP1<C4`0=3@d055AD@11
@D40Cdm?09:BTP1OGel0GUiN04E5A@18B4P0HV9R09JFUP1DE5@0BDU904];B`1KFe/0R8R80;bl_00m
?Cd0E5AD06][J`1SHf<0];Bd07UiN@0a<C40>SXj05MGE`1iNGT0MGEe02`/;00i>CT0@D5106UYJ@02
07emO@0602PX:00d=3@0<c<c08R8R02SXj<0TI6A0P0i>CT0G01<C4`0HV9R08:2PP1[Jf/09BDU04Y:
BP1LG5`0F5QH0861P@20P800<S8b0410@01RHV80H61P0:JVYP2?Shl0>CTi04U9B@24Q8@0Lg=c09BD
U027QhL0>c/k079bLP1GEeL0Bd];08J6QP0f=SH0<C4a07QhN01SHf<0PX:207inOP1;Bd/0@T9207Mg
M`29RHT0Xj>S07MgM`0a<C40@D5107emO@1UIFD0OWin07moO`13@d<0DU9B06=SH`1bLW80L71`04M7
A`12@T80FEUI08V9R@21PH40Jf][07EeM@11@D40Z:RX0<g=c@1TI6@0L71`07=cL`1EEED0`L710<K6
aP1IFET0MGEe05aLG01CDe<0U9BD06=SH`1GEeL0Ogmo051@D01PH600OWin0492@P1QHF40KFe]04];
B`1:BTX0/K6a03Xj>P10@400IVIV04=3@`1bLW80YJFU04E5A@1:BTX0EEEE0P1IFET0EP2k^k/0F5QH
05=CD`1XJ6P0O7al0:NWY`2g]kL0?clo05MGE`2;Rh/0Xj>S0<_;b`1kNg/0:BTY03Ti>@1jNWX0S8b<
0>GUi@22PX80;Bd]04a<C020P800f=SH09bLW00Z:RX0KVi^07alO01bLW80_;bl05QHF00Y:BT0NWYj
0;Zj^P26QXH0KVi^03dm?@2=SHd0YZJV07UiN@2WYjL0@D5105mOG`31`L40QHF509bLW01[Jf/0;2`/
065QH@2CTi<0V9RH09FEU@0`<300@T920:f][@1gMgL0T92@02/[:`0m?Cd0I6AT08N7Q`1=CDd0P820
0>g]k@3Bd]80[jn_05UIF@1UIFD0OGem03hn?P1OGel0JVYZ04i>CP1DE5@0Z:RX05ADE01MGEd0L71`
05mOG`1BDU80]KFe03lo?`1@D500KFe]03`l?01PH600U9BD0P1LG5`05`1RHV80<c<c04e=C@21PH40
B4Q8059BDP2YZJT0>SXj05ADE01HF5P0?clo04Q8B027QhL0PH6103`l?027QhL0>SXj04i>CP1[Jf/0
Cdm?04Q8B01jNWX0EUIF00kooooo0P15ADD0EP2/[:`0IFEU07UiN@1GEeL0GUiN08J6QP20P800DU9B
05]KF`1HF5P0B4Q806QXJ01@D500?Shn06QXJ01/K6`0L71`07QhN01RHV80?Shn05EEE@1:BTX0AdM7
06i^KP0m?Cd0>SXj02l_;`1QHF40JVYZ04U9B@1YJFT0H61P07=cL`1MGEd0G5aL03dm?@1<C4`0<S8b
02`/;00^;Rh0D51@04m?C`0a<C40FEUI02PX:014A4@0EEEE05eMG@1GEeL0Jf][04I6AP1HF5P0G5aL
04Y:BP1KFe/0DE5A05=CD`14A4@0Fe]K069RHP1WIfL0FUYJ0820P01FEUH0EeMG05YJFP26QXH0G5aL
07moO`2PX:00OGem05QHF01YJFT0NGUi07IfMP1mOGd0?Shn05QHF01^KVh0XZ:R07EeM@1KFe/0D51@
06QXJ01WIfL0D51@0P1BDU80<@0h>3P0@T9203hn?P1oOgl0Cdm?03dm?@1EEED0EeMG03dm?@1FEUH0
DU9B05EEE@0n?Sh0DU9B04U9B@0k>c/0Cdm?08:2PP1TI6@0@T9204M7A`1OGel0Hf=S05eMG@1QHF40
KVi^04U9B@1XJ6P0HV9R03`l?01YJFT0GemO04Y:BP1OGel0;Bd]09>CT`13@d<0PH6107AdM01mOGd0
71`L04E5A@1fMWH0BDU906m_K`1UIFD0EeMG04a<C01cLg<00P1FEUH0R@2@T900N7Qh04Y:BP1>CTh0
NGUi05EEE@2ZZZX0GUiN06a/K01UIFD0Hf=S0451@@13@d<0Bd];05ADE01OGel0?clo08>3P`1aLG40
EEEE06m_K`1iNGT0=cLg08Z:RP1WIfL0>3Ph06EUI@0e=CD0BTY:05mOG`16ATH0IVIV06UYJ@1gMgL0
F5QH04m?C`0n?Sh0Hf=S04=3@`1lO7`0HV9R05QHF010@400IFEU06YZJP0U9BD0T92@04M7A`1mOGd0
De=C0820P02;Rh/0GemO08B4Q01HF5P0>c/k07UiN@1jNWX0LG5a08J6QP1SHf<0BTY:04e=C@1TI6@0
ATI6069RHP23Ph<0IFEU05mOG`1ADE40Kfm_07inOP1IFET0KVi^06IVIP1LG5`0FUYJ06UYJ@1BDU80
SHf=03Lg=`2IVIT0=cLg04];B`2:RXX0BTY:0:>SX`1dM7@0HV9R065QH@1]KFd0De=C06UYJ@1fMWH0
XJ6Q06EUI@28R8P0Hf=S06YZJP1oOgl0Kfm_0:^[Z`3Kfm/0kno_05IFEP0Z:RX0XJ6Q06EUI@1[Jf/0
BDU9075aL@27QhL0O7al0;Ng]`23Ph<0Q8B408n?S`1]KFd0UIFE0:6QX@2h^;P0EUIF06=SH`1PH600
I6AT07MgM`1LG5`0RHV905]KF`1CDe<0LG5a06EUI@1NGUh0NWYj08R8R02HV9P0XJ6Q013ooooo000>
ooooo`3o07EeM@2KVi/0Ph>305MGE`1lO7`0MgMg06a/K01^KVh0=SHf05aLG01BDU80EeMG07EeM@0[
:b/0B4Q806YZJP1_Kfl0JFUY0410@01CDe<0MgMg0:VYZ@1XJ6P0C4a<03dm?@1IFET0SXj>08N7Q`17
AdL0CDe=05ADE01lO7`0R8R803hn?P1DE5@0H61P04i>CP2=SHd0Ti>C02/[:`19BDT0Fe]K051@D025
QHD0IfMW03lo?`1ADE40Hf=S05YJFP1oOgl0@T9202h^;P16ATH0JFUY06QXJ02DU9@0@d=303lo?`1<
C4`0De=C06][J`2j^[X0:BTY05]KF`1lO7`0MGEe07QhN02i^KT0<30`04e=C@23Ph<0NGUi08B4Q02]
[Jd0;2`/03hn?P23Ph<0HV9R08J6QP1nOWh0<30`0410@026QXH0OGem08j>SP10@400>3Ph05QHF02A
TI40HV9R09VIV@1VIVH0<S8b05YJFP1^KVh0EeMG08n?S`0g=cL0Bd];07MgM`1lO7`0@D5106e]K@0d
=3@0FEUI08N7Q`1aLG40FUYJ08^;R`1:BTX0J6QX092@T01BDU80G5aL06UYJ@1:BTX0P8200:f][@13
@d<0MGEe06][J`1?Cdl0Ogmo0:NWY`1>CTh0Kfm_07AdM01KFe/0Jf][09>CT`1MGEd0LG5a06EUI@1V
IVH0LG5a08Z:RP1>CTh0LG5a07MgM`1dM7@0VYZJ0:BTY01VIVH0UIFE05QHF01JFUX0UYJF0:JVYP1_
Kfl0UIFE04=3@`1UIFD0V9RH05YJFP11@D40Shn?06UYJ@20P800Xj>S04I6AP0m?Cd0M7Ad07inOP2I
VIT0[jn_051@D01=CDd0LW9b08j>SP2_[jl0MgMg038b<P1VIVH0V9RH08F5Q@28R8P0E5AD03Ti>@1]
KFd0ZZZZ07AdM026QXH0GemO04U9B@2;Rh/0Ti>C071`L02WYjL0DE5A08j>SP30`<00HV9R0:f][@2<
S8`0<C4a075aL@2b/[80MWIf09FEU@0/;2`0CTi>08:2PP1MGEd0Shn?04a<C01LG5`0][Jf06m_K`1j
NWX0Cdm?05UIF@2EUID0ZJVY04a<C01VIVH0Bd];06ATI02YZJT0L71`05iNGP1eMGD0GUiN08Z:RP18
B4P0EUIF07alO015ADD0Bd];07QhN01gMgL0Y:BT0:NWY`24Q8@0QHF5092@T018B4P0IVIV0861P@1:
BTX0KFe]09NGU`1fMWH0QhN708n?S`1MGEd0MGEe0;Jf]P25QHD0CDe=06a/K016ATH0Kfm_08R8R01P
H600Hf=S000C04e=C@15ADD0IFEU0:BTY01YJFT0>3Ph07]kN`1;Bd/0RHV907=cL`0j>SX0=3@d04i>
CP1FEUH0GemO0<30`01fMWH0?3`l06a/K00205UIF@0308n?S`1KFe_ooooo00gooooo0140D51@05mO
G`1dM7@0C4a<071`L01KFe/0I6AT08b<S01JFUX0@T9205MGE`1OGel0De=C07emO@1oOgl0Hf=S05UI
F@0206EUI@2F04];B`1>CTh0>SXj05=CD`16ATH0Bd];06YZJP11@D40B4Q803lo?`1RHV80J6QX04];
B`1QHF40IVIV075aL@1UIFD0GemO03`l?01GEeL0>3Ph02l_;`0i>CT0EEEE05ADE01BDU80Jf][02h^
;P1NGUh0HV9R06UYJ@1MGEd0L71`030`<01:BTX0IFEU04I6AP1JFUX0@T9206UYJ@0m?Cd0I6AT04i>
CP1;Bd/0De=C06UYJ@1@D500EUIF06ATI02FUYH0CDe=06][J`25QHD0/k>c07IfMP1/K6`0Rh^;06a/
K021PH40B4Q807emO@2UYJD0dm?C07alO01TI6@0Cdm?06QXJ01OGel0CDe=05mOG`1:BTX0FEUI04Y:
BP12@T80HF5Q05mOG`1@D500?Cdm04];B`1:BTX0I6AT04Y:BP1GEeL0AdM704U9B@0l?3`0Bd];03hn
?P1RHV80I6AT04U9B@1=CDd0K6a/06=SH`1ZJVX0Fe]K04i>CP0h>3P0ADE505]KF`11@D40IfMW06=S
H`1ADE40B4Q802DU9@1gMgL0=cLg09^KV`1dM7@0NWYj02l_;`19BDT0SXj>04A4A02:RXX0Kfm_075a
L@0i>CT0Hf=S05MGE`1KFe/0CDe=05IFEP1DE5@0>c/k05iNGP0o?cl0PX:204Y:BP1^KVh0G5aL06][
J`1;Bd/0IVIV06=SH`1NGUh0Kfm_05ADE02YZJT0Ph>303dm?@1DE5@0G5aL05QHF024Q8@0Fe]K05mO
G`25QHD0@d=30P1FEUH0?00l?3`0OGem06UYJ@2>SXh0H61P069RHP12@T80QhN703Ti>@2>SXh0KFe]
06m_K`0n?Sh0HV9R05eMG@0b<S80GUiN02/[:`1hN7P0Bd];07QhN01SHf<0@41005MGE`1ZJVX0@D51
05UIF@1LG5`0LG5a092@T01YJFT0<S8b05EEE@1aLG40CDe=04=3@`1eMGD0EeMG06IVIP1PH600LW9b
09nOW`24Q8@0Vi^K075aL@1VIVH0LW9b0861P@1ZJVX0V9RH05=CD`2^[Zh0@T9205]KF`1lO7`0G5aL
0:FUY@1SHf<0Jf][061PH080OGem02d0Ogmo07EeM@2?Shl0J6QX08R8R01YJFT0SHf=0:FUY@27QhL0
U9BD09ZJVP2k^k/0FEUI05ADE02PX:00RHV9075aL@1cLg<0Ti>C09JFUP2>SXh0a<C40:RXZ02[Zj/0
b<S808>3P`2XZ:P0YJFU0:j^[P1;Bd/0Kfm_061PH019BDT0GUiN04a<C01[Jf/0JFUY079bLP0m?Cd0
ADE507=cL`21PH40Ogmo0;bl_02WYjL04?ooool000kooooo03D0YZJV05=CD`2RXZ80R8R80820P019
BDT0EeMG0:f][@1[Jf/0D51@02`/;00f=SH0LG5a05YJFP0d=3@0BTY:03hn?P1OGel0IfMW02<S8`12
@T80Hf=S04=3@`2MWId0=CDe030`<01<C4`0KFe]06a/K01kNg/0;Rh^04Q8B01BDU80Bd];09FEU@37
alL0;Bd]07EeM@2WYjL0CDe=07UiN@2CTi<0?clo04a<C01QHF40Ng]k0:n_[`2BTY8071`L04a<C01i
NGT0QXJ609bLW00203De=@3C04];B`1oOgl0MgMg07UiN@0E5AD0@d=307UiN@1aLG40U9BD05UIF@0U
9BD0ATI608V9R@2MWId0Ti>C04=3@`0k>c/0De=C09bLW02<S8`0EeMG04];B`0k>c/0LG5a08N7Q`1K
Fe/0BTY:03Lg=`1HF5P0PX:206][J`0b<S80F5QH05]KF`0f=SH0EUIF08:2PP0l?3`0EEEE075aL@1@
D500eMGE0:VYZ@1?Cdl0GEeM04e=C@0Y:BT0H61P04];B`1HF5P0ATI603Xj>P19BDT0T92@04U9B@10
@400?clo04I6AP1YJFT0TY:B03Xj>P1?Cdl0EUIF05eMG@26QXH0]kNg05IFEP1=CDd0FUYJ05IFEP2A
TI40]KFe04m?C`17AdL0J6QX07]kN`2MWId0WYjN03Ti>@1OGel0JFUY0;^k^`2a/K40EEEE03/k>`1G
EeL0Cdm?06m_K`3:b/X0BTY:04a<C01ADE40MgMg0:j^[P2d];@0B4Q8055AD@1PH600JVYZ069RHP2O
Wil0>c/k04U9B@1dM7@0MgMg07YjNP38b<P0J6QX04E5A@2=SHd0TY:B0:n_[`2e]KD0DE5A06EUI@2V
YZH0Lg=c0;6a/@29RHT0AdM708Z:RP2i^KT0PH6109jNWP1_Kfl0@D5106e]K@33`l<0Q8B40<_;b`1U
IFD0ADE506=SH`2:RXX0UIFE08:2PP15ADD0AdM708Z:RP1JFUX0MWIf03/k>`1;Bd/0[:b/07MgM`1E
EED0QHF504A4A02SXj<0UiNG06a/K022PX80>c/k06ATI02MWId0EeMG075aL@17AdL0DU9B0:f][@1Y
JFT0FEUI07MgM`0k>c/0J6QX09fMW@1<C4`0KFe]04U9B@1KFe/0Z:RX03<c<`0S8b<0@T92020P800k
>c/0DU9B03De=@1GEeL0JFUY07]kN`2OWil0ZJVY02XZ:P1;Bd/0IVIV05]KF`2>SXh0/;2`02XZ:P14
A4@0FUYJ08Z:RP26QXH0J6QX02/[:`1@D500IVIV08V9R@2KVi/0EEEE028R8P1>CTh0IVIV0;Vi^@1`
L700TY:B03dm?@17AdL0IfMW06UYJ@0207EeM@0:0451@@0_;bl0DE5A09FEU@1OGel0PX:204U9B@16
ATH0EeMG07YjNPkooooo00D0De=C07UiN@2JVYX0HV9R07moO`0205aLG03g08^;R`1SHf<0AdM705EE
E@1iNGT0J6QX09NGU`2=SHd0GEeM07EeM@1kNg/0N7Qh055AD@1]KFd0AdM7051@D019BDT0Cdm?05eM
G@1:BTX0B4Q8055AD@1VIVH0HF5Q04a<C01cLg<0Jf][079bLP1^KVh0G5aL03Hf=P1:BTX0<30`024Q
8@0/;2`0CTi>04a<C010@400GEeM02TY:@1;Bd/0Fe]K06=SH`1ADE40Lg=c03<c<`14A4@0G5aL04=3
@`1DE5@0BTY:04e=C@0l?3`0EeMG05YJFP1NGUh0DU9B06i^KP1VIVH0OGem0861P@2CTi<0EeMG05=C
D`1MGEd0NGUi05]KF`1NGUh0WIfM069RHP2<S8`0Kfm_09JFUP2@T900]kNg0820P01KFe/0BTY:06EU
I@1SHf<0Bd];05mOG`1>CTh0?Shn04a<C01:BTX0PH6104];B`1:BTX0CTi>05EEE@13@d<0EEEE04Q8
B01HF5P0@d=305=CD`11@D40DE5A03/k>`1cLg<0IFEU05EEE@1<C4`0O7al06QXJ01CDe<0BDU905EE
E@0n?Sh0ATI605MGE`0l?3`0CTi>059BDP19BDT0DU9B02h^;P2FUYH0ATI60<71`@1hN7P0GUiN03De
=@1ADE40UiNG03Ph>027QhL0MgMg06EUI@19BDT0DU9B06YZJP1cLg<0?clo04Y:BP1TI6@0:2PX04e=
C@0c<c<0Xj>S030`<01BDU80L71`06i^KP0n?Sh0IfMW05]KF`1aLG40T92@04M7A`1nOWh0N7Qh0492
@P1PH600L71`04a<C01/K6`0Hf=S04A4A01RHV80?Cdm04I6AP1?Cdl0De=C096AT@1TI6@0EeMG05QH
F01[Jf/0=CDe06QXJ00f=SH0PX:206UYJ@1EEED0?Cdm04U9B@1ADE40A4A40:BTY00k>c/0MgMg04U9
B@1aLG40I6AT05aLG025QHD0Cdm?02`/;01lO7`0EeMG04M7A`1ZJVX0EUIF03Lg=`1?Cdl0HF5Q03De
=@0o?cl0CDe=03dm?@1ADE40IfMW05eMG@20P800Y:BT0;>c/`1GEeL0GemO055AD@1KFe/0O7al0:^[
Z`1IFET0MgMg04Y:BP23Ph<0W9bL05mOG`21PH40R8R806m_K`1<C4`0IFEU06i^KP2<S8`0KVi^09^K
V`1BDU80IfMW065QH@1LG5`0Lg=c05MGE`1oOgl0/k>c0<o?c`1<C4`0GUiN0:6QX@24Q8@0N7Qh04a<
C01XJ6P0O7al0820P0020:6QX@0F0:RXZ01lO7`0R8R80:^[Z`2=SHd0/k>c05=CD`1NGUh0HV9R04Q8
B01OGel0LG5a0:6QX@25QHD0Q8B405UIF@1HF5P0N7Qh09FEU@1_Kfl0UiNG0:>SXa3ooooo000>oooo
o`220;6a/@1jNWX0GEeM08:2PP35aLD0ATI603/k>`1^KVh0PX:207YjNP0f=SH09RHV05ADE01dM7@0
NGUi07QhN00];Bd0G5aL0:NWY`1oOgl0/;2`0:>SX`1CDe<0EeMG07YjNP1YJFT0UiNG03hn?P12@T80
KVi^04M7A`1XJ6P0OWin02HV9P1IFET0KVi^05UIF@1VIVH0Ogmo03Hf=P0j>SX0IVIV03lo?`1@D500
=CDe01dM7@1MGEd0ZZZZ05EEE@1HF5P0>c/k02/[:`1aLG40Ti>C04M7A`0o?cl0=cLg04I6AP24Q8@0
ADE502PX:01FEUH0BTY:09BDU02>SXh0:b/[02d];@1EEED0@41005iNGP1cLg<09BDU0451@@1BDU80
E5AD061PH01^KVh071`L0451@@1jNWX0?Cdm06IVIP1OGel0=3@d07]kN`21PH40CDe=09JFUP21PH40
>3Ph09VIV@2KVi/0H61P09jNWP1eMGD0=3@d05mOG`2:RXX0E5AD09bLW01>CTh09bLW05MGE`1YJFT0
Ph>30:RXZ00^;Rh0=3@d08>3P`2:RXX0/[:b08F5Q@0`<300FUYJ06e]K@1kNg/0VIVI06QXJ015ADD0
DE5A09NGU`2EUID0S8b<03dm?@0o?cl0GEeM08Z:RP2<S8`0PH6105]KF`80J6QX0980UiNG06UYJ@2S
Xj<0EUIF05QHF024Q8@0QHF5071`L02e]KD0E5AD05=CD`2;Rh/0UiNG06=SH`2JVYX0<c<c065QH@2f
][H0QXJ606=SH`2[Zj/0<C4a02`/;02EUID0J6QX06MWI`1bLW80;Bd]03hn?P1iNGT0L71`05mOG`1P
H600;Bd]075aL@2d];@0FUYJ075aL@1ADE40>c/k075aL@2e]KD0FUYJ05aLG01]KFd0FEUI07YjNP2S
Xj<0<c<c0451@@1MGEd0T92@0<W9b@1aLG40@D5109NGU`1OGel0^k^k0;bl_00c<c<0E5AD06EUI@1H
F5P0YJFU03hn?P0e=CD0KFe]075aL@1NGUh0YJFU03hn?P1dM7@0OWin08f=S@3>c/h0BTY:05QHF02@
T900R8R80;Bd]01QHF40=SHf06=SH`22PX80[jn_09RHV010@400CDe=08b<S02CTi<0TI6A03/k>`1S
Hf<0OWin07moO`1FEUH0Kfm_04A4A01EEED0EUIF04Q8B01DE5@0S8b<03Ph>01JFUX0Ng]k06EUI@28
R8P0_Kfm03/k>`1KFe/0Ph>306MWI`1cLg<0YZJV04E5A@1TI6@0YjNW06YZJP1[Jf/0LW9b03Xj>P1D
E5@0Y:BT04m?C`1MGEd0PH6105=CD`1IFET0NWYj03lo?`1YJFT0Jf][02h^;P1<C4`0HV9R055AD@1D
E5@0Hf=S02l_;`0h>3P0F5QH04I6AP13@d<0EeMG03Ph>0kooooo01d0DE5A06IVIP1bLW80G5aL07Qh
N01HF5P0FUYJ0:FUY@1DE5@0A4A405IFEP1dM7@0BDU906ATI01`L700De=C055AD@1HF5P0MWIf06YZ
JP1FEUH0:b/[055AD@11@D40?clo06QXJ00j>SX0=cLg0451@@0207UiN@0]0492@P1YJFT0HV9R08N7
Q`1mOGd0KVi^04I6AP1MGEd0>3Ph02LW9`0c<c<0KFe]05EEE@1BDU80Kfm_02l_;`1IFET0Jf][06EU
I@1FEUH0MgMg034a<@1:BTX0EeMG04A4A01ADE40?Shn06i^KP12@T80IVIV059BDP1?Cdl0C4a<069R
HP1ADE40EEEE06][J`2TY:@0Bd];07IfMP1JFUX0PH61059BDP1HF5P0Vi^K0080RHV90100ADE507em
O@2OWil0f][J07UiN@1SHf<0BTY:06=SH`1WIfL0BTY:05mOG`18B4P0A4A404I6AP0n?Sh0FUYJ0P1@
D5001P0h>3P0EEEE0410@01LG5`0>CTi04i>CP80@T9206X0<S8b04A4A00d=3@0KFe]06EUI@16ATH0
Cdm?06=SH`1GEeL0IfMW06IVIP1YJFT0Cdm?05MGE`1VIVH0Bd];04M7A`1RHV80EUIF04Q8B00e=CD0
T92@0410@02l_;`0Ogmo065QH@1=CDd0FUYJ0:BTY00k>c/0Rh^;07QhN01gMgL0A4A406MWI`19BDT0
C4a<04U9B@1EEED0BTY:030`<01DE5@0ATI607IfMP0k>c/0De=C04M7A`1HF5P0@d=3055AD@1NGUh0
ATI605mOG`1@D500WIfM06i^KP1HF5P0MWIf071`L01RHV80UIFE07IfMP1FEUH0MGEe06=SH`1<C4`0
HF5Q05ADE01OGel0L71`05UIF@1]KFd0EeMG03Xj>P1]KFd0BTY:09FEU@1cLg<0KVi^05]KF`1=CDd0
D51@03Ph>01hN7P0>SXj07IfMP1DE5@0SHf=05]KF`0m?Cd0DE5A06=SH`0a<C40JFUY05mOG`1_Kfl0
X:2P05UIF@0];Bd0Hf=S07MgM`0o?cl0?Shn055AD@1=CDd0IFEU0P1PH6009028R8P0];Bd0;Ng]`1c
Lg<0G5aL08>3P`21PH40JVYZ079bLP13@d<0RXZ:03De=@1=CDd0J6QX05ADE01kNg/0J6QX07YjNP24
Q8@0TY:B07EeM@26QXH0MWIf0;Bd]01NGUh0QXJ606IVIP1dM7@0SHf=069RHP1`L700WIfM0<o?c`2J
VYX0Ng]k09JFUP80R8R801`0De=C092@T02`/;00`L710<W9b@2KVi/0Z:RX0:>SX`25QHD0XZ:R0;Rh
^036a/H0EeMG08>3P`1fMWH0Cdm?05MGE`1<C4`0J6QX07=cL`1gMgL0>3Ph04U9B@1MGEd0PX:2092@
T02>SXh0^KVi4?ooool000kooooo0?l0NWYj0::RXP1fMWH0Hf=S07]kN`24Q8@0K6a/06][J`0d=3@0
EUIF05EEE@1HF5P0Kfm_02TY:@1JFUX0UiNG04=3@`17AdL0ATI60451@@1cLg<0[Jf]03lo?`16ATH0
?Cdm06MWI`2f][H0W9bL030`<01;Bd/0>SXj07emO@2a/K408R8R03lo?`1DE5@0@4100:2PX02g]kL0
:BTY03`l?017AdL0G5aL08R8R01VIVH0;Bd]04I6AP1_Kfl0MGEe07YjNP0[:b/0B4Q805=CD`1`L700
Jf][06UYJ@0c<c<0GEeM07moO`1lO7`0B4Q808f=S@0W9bL0G5aL0;Zj^P2YZJT0IVIV08R8R00Q8B40
DU9B09NGU`1]KFd0S8b<0:2PX00T92@0BTY:06=SH`14A4@0SHf=06EUI@1=CDd0Bd];05aLG01NGUh0
OGem02d];@1?Cdl0DE5A096AT@1[Jf/0KVi^030`<01:BTX0GemO08b<S01HF5P0EEEE02`/;015ADD0
Ti>C09BDU00k>c/0@d=303Lg=`1@D500L71`04m?C`0n?Sh0AdM703/k>`1CDe<0Kfm_03hn?P14A4@0
HV9R02`/;02@T900UYJF03/k>`15ADD0BDU904Y:BP1hN7P0UYJF02l_;`14A4@0DU9B06=SH`1MGEd0
Vi^K03De=@1SHf<0Fe]K06e]K@28R8P0QhN703Ti>@1_Kfl0EeMG05mOG`2f][H0/k>c03Ti>@1kNg/0
LG5a07IfMP3?cll0QHF503`l?02NWYh0Jf][079bLP3@d=00>CTi03@d=01YJFT0FEUI09^KV`2l_;`0
8R8R04I6AP1QHF40N7Qh0<O7a`2VYZH08B4Q03lo?`1TI6@0K6a/0;Bd]01WIfL0>SXj065QH@23Ph<0
JVYZ09BDU01ADE40BDU9079bLP27QhL0OGem08n?S`1CDe<0J6QX0:>SX`2RXZ80NWYj0:f][@1GEeL0
Lg=c0=oOg`1aLG40WinO04m?C`0d=3@0V9RH09NGU`1dM7@0LG5a02TY:@11@D40XZ:R06a/K02ZZZX0
>3Ph06][J`27QhL0T92@096AT@1?Cdl0E5AD0:f][@2EUID0D51@069RHP1@D500NWYj0<K6aP1TI6@0
GEeM0451@@1FEUH0S8b<05=CD`14A4@0M7Ad03/k>`1VIVH0UYJF05ADE01[Jf/0PX:205iNGP18B4P0
YZJV03De=@1SHf<0KVi^04e=C@1UIFD0Xj>S05ADE01YJFT0C4a<05UIF@25QHD0Z:RX05UIF@11@D40
De=C04U9B@2ATI40`<3004M7A`0f=SH001L0I6AT04a<C01[Jf/0ZJVY034a<@0g=cL0IVIV05=CD`23
Ph<0RHV9028R8P0e=CD0DE5A06MWI`1oOgl0[Zj^02l_;`0];Bd0C4a<06EUI@20P800Ng]k03Xj>P0>
ooooo`0Y04M7A`1QHF40PX:206MWI`27QhL0FUYJ05eMG@2RXZ80De=C051@D01?Cdl0K6a/04i>CP24
Q8@0MgMg0:6QX@1KFe/0LW9b07EeM@1?Cdl0DE5A03Xj>P1?Cdl0>3Ph03hn?P1SHf<0C4a<03Hf=P14
A4@0De=C06=SH`0n?Sh0GEeM04e=C@1YJFT0EEEE05aLG011@D40DE5A03Xj>P19BDT00P1ADE40;`1J
FUX0GEeM05aLG00o?cl0FUYJ07QhN01JFUX0ADE505]KF`0T92@0B4Q806MWI`10@400F5QH04e=C@1S
Hf<0D51@079bLP1:BTX0F5QH05ADE01]KFd0EEEE061PH01fMWH0]KFe04i>CP2OWil0HF5Q07]kN`1P
H600KFe]08V9R@1JFUX0Q8B406YZJP2DU9@0]kNg0>;RhP2=SHd0L71`06=SH`1?Cdl0Kfm_069RHP1h
N7P0Bd];0080H61P06`0DU9B071`L01/K6`0GemO051@D01]KFd0H61P06IVIP1>CTh0H61P05YJFP1@
D500=CDe05EEE@12@T80MWIf06=SH`1ADE40D51@06IVIP1FEUH0JFUY08n?S`1_Kfl0EUIF04Q8B01B
DU80Fe]K051@D01:BTX0EUIF059BDP0c<c<0RXZ:03Xj>P32`/80UiNG05ADE020P800IFEU09^KV`1X
J6P0/[:b07MgM`1JFUX0ADE505ADE01?Cdl0ATI60410@019BDT0D51@02HV9P1HF5P0@D510:6QX@0a
<C40E5AD05UIF@1[Jf/0@d=304Y:BP1GEeL0Hf=S07AdM01CDe<0`L7108:2PP0c<c<0FEUI06YZJP0m
?Cd0EEEE05QHF01>CTh0QhN705]KF`1bLW80J6QX03Ph>01bLW80JVYZ0;2`/02QXJ40EUIF03<c<`29
RHT0@410092@T01NGUh0Bd];03hn?P1<C4`0EEEE03/k>`2=SHd0BDU907AdM013@d<0FEUI05IFEP15
ADD0C4a<06UYJ@15ADD0MGEe05YJFP1:BTX206IVIP0801lO7`17AdL0F5QH02l_;`0c<c<0HV9R03Lg
=`17AdL205UIF@0E07emO@1ZJVX0Q8B404m?C`1@D500C4a<04m?C`1QHF40Kfm_05UIF@2n_[h0A4A4
05UIF@20P800EUIF08R8R01RHV80GemO059BDP1OGel0Cdm?0080KFe]02/0UYJF08B4Q029RHT0GEeM
06UYJ@1cLg<0Rh^;08:2PP2;Rh/0];Bd08B4Q02m_Kd0`l?307IfMP1VIVH0K6a/0:6QX@2MWId0d=3@
0<c<c026QXH0SXj>0;Bd]03AdM40emOG08j>SP2QXJ40CTi>05QHF01:BTX0DE5A06UYJ@1:BTX0O7al
08R8R01nOWh0?clo05ADE01QHF40UiNG092@T02?Shl0^KVi013ooooo000>ooooo`1G08f=S@1iNGT0
QHF506][J`1EEED0NGUi07alO02LW9`0:BTY051@D01NGUh0MWIf0;>c/`0m?Cd08B4Q07=cL`25QHD0
XZ:R05YJFP15ADD0=SHf08J6QP2>SXh0YZJV03Xj>P0[:b/0:2PX09:BTP25QHD0R8R802l_;`0h>3P0
GEeM05MGE`1_Kfl0Y:BT02@T900^;Rh0QHF506IVIP1BDU80LW9b01hN7P0i>CT0JFUY04a<C01_Kfl0
OWin01dM7@1hN7P0J6QX02l_;`1bLW80CTi>02HV9P1@D500JVYZ02h^;P1>CTh0>CTi04U9B@1NGUh0
<30`03dm?@1FEUH0?Shn06EUI@1cLg<0BTY:059BDP1NGUh0;bl_08V9R@3Ng]h0J6QX030`<01TI6@0
GEeM09FEU@2[Zj/0;bl_03`l?01LG5`0JFUY0:FUY@1=CDd0;Bd]0080DU9B0:40NGUi0<_;b`0b<S80
=SHf06i^KP1`L700MgMg0;2`/00b<S807QhN055AD@2@T900RHV906a/K00W9bL0ADE505YJFP1SHf<0
MWIf09bLW00k>c/0G5aL05EEE@1=CDd0M7Ad0:^[Z`0e=CD0Fe]K06a/K01?Cdl0R8R80861P@0c<c<0
Cdm?08J6QP20P800UYJF06m_K`0k>c/0B4Q80:FUY@1[Jf/0Y:BT04e=C@0l?3`0Hf=S09RHV026QXH0
VIVI02HV9P1:BTX0JFUY08n?S`24Q8@0PH6102`/;01HF5P0L71`0:n_[`1ZJVX0MgMg04a<C01JFUX0
LW9b09>CT`1@D500H61P03hn?P1ADE40Vi^K05]KF`1FEUH0Lg=c03lo?`1WIfL0Y:BT05eMG@1WIfL0
S8b<04A4A01lO7`0];Bd0451@@0k>c/0KFe]04E5A@1mOGd0VIVI03Lg=`1?Cdl0FEUI03Ti>@27QhL0
VIVI06EUI@2QXJ40H61P06QXJ02d];@0IFEU05aLG02LW9`0U9BD0:JVYP2ZZZX0=cLg05]KF`1oOgl0
WinO09RHV00d=3@0B4Q806m_K`1dM7@0fm_K05eMG@10@400UIFE06EUI@2QXJ40K6a/03dm?@1GEeL0
N7Qh051@D02FUYH0D51@061PH02;Rh/0MWIf0:ZZZP1OGel0@d=30820P01TI6@0TI6A0:VYZ@0S8b<0
?Cdm07moO`1HF5P0S8b<06QXJ00c<c<0@D5108b<S01@D500ZZZZ05eMG@0h>3P0De=C08F5Q@1eMGD0
W9bL06MWI`0];Bd0D51@092@T02TY:@0L71`069RHP0204E5A@0J08>3P`24Q8@0FUYJ02LW9`0l?3`0
BDU908>3P`1dM7@0EEEE055AD@12@T80IFEU08n?S`1QHF40@41009:BTP11@D40IFEU07moO`1`L700
EUIF07moO`0`<300JVYZ096AT@1PH60>ooooo`3o04=3@`1TI6@0L71`06IVIP1TI6@0HF5Q06][J`2>
SXh0Bd];03`l?019BDT0MgMg06a/K022PX80IFEU08F5Q@1^KVh0Ogmo07alO01QHF40GemO02/[:`19
BDT0=3@d03`l?01SHf<0CDe=03`l?01JFUX0K6a/07EeM@14A4@0N7Qh06QXJ02CTi<0Kfm_06UYJ@0m
?Cd0EeMG02LW9`0T92@0:b/[05QHF01CDe<0?clo05MGE`0S8b<0@d=3069RHP1^KVh0EUIF096AT@0o
?cl0BDU90820P010@400EeMG03lo?`1aLG40>c/k06i^KP1CDe<0EUIF04E5A@1cLg<0J6QX07emO@1M
GEd0W9bL05]KF`2EUID0IfMW07EeM@1<C4`0GUiN0:ZZZP1^KVh0MgMg0410@01dM7@0W9bL0=gMg@23
Ph<0Hf=S05=CD`24Q8@0Ng]k04m?C`1RHV80JVYZ04U9B@1?Cdl0@d=308R8R017AdL0EEEE05MGE`1Z
JVX0C4a<05EEE@1:BTX0GUiN04m?C`1LG5`0AdM704e=C@0m?Cd0UIFE075aL@1JFUX0DE5A071`L01U
IFD0Q8B406m_K`1WIfL0=CDe04a<C01`L700CTi>05]KF`1jNWX0G5aL06UYJ@0n?Sh0T92@03De=@2n
_[h0UIFE05eMG@2OWil0KVi^09jNWP1HF5P0[jn_07AdM01HF5P0AdM7065QH@1MGEd0E5AD03dm?@1L
G5`0K6a/04I6AP1EEED0HF5Q0;no_`17AdL0E5AD06EUI@1BDU80@d=306m_K`1VIVH0?Cdm05MGE`0o
?cl0^[Zj08F5Q@0l?3`0KVi^05MGE`0g=cL0Ti>C06e]K@1>CTh0PH610492@P0m?Cd0ADE503<c<`1T
I6@0BDU906e]K@1lO7`0AdM702HV9P1[Jf/0?Cdm0;2`/01^KVh0HF5Q04Y:BP1NGUh0IFEU04=3@`2:
RXX0@T92079bLP1FEUH0Ogmo05EEE@1@D500IFEU05]KF`19BDT0LW9b069RHP1NGUh0NGUi06e]K@0b
<S80D51@05iNGP0n?Sh0CTi>07YjNP13@d<0ATI607emO@1WIfL0I6AT04U9B@1mOGd0G5aL051@D01Q
HF40JFUY07]kN`1XJ6P0L71`0;Vi^@1<C4`0EEEE07QhN01WIfL0NGUi06ATI01kNg/0E5AD08:2PP1/
K6`0Ti>C08N7Q`2_[jl0Jf][08N7Q`2?Shl0Y:BT07alO01MGEd0I6AT06a/K02_[jl0Ng]k075aL@2?
Shl0OWin05mOG`1MGEd0UiNG0:JVYP2SXj<0^;Rh000>09BDU02JVYX0Zj^[08j>SP2g]kL0UIFE0:FU
Y@1MGEd0IfMW04];B`1FEUH0FUYJ04m?C`1hN7P205EEE@0704U9B@1LG5`0FEUI07moO`2:RXX0OWin
0:2PX00@ooooo`003_ooool06`31`L40CTi>07=cL`29RHT0T92@03lo?`1HF5P0J6QX06ATI015ADD0
:b/[05=CD`1]KFd0Ph>305ADE00m?Cd0=CDe06EUI@2IVIT0F5QH055AD@1QHF40<c<c06EUI@2XZ:P0
?clo05]KF`0203Hf=P2=09fMW@0d=3@0@D5103/k>`17AdL0GEeM0:BTY013@d<0@T9203Xj>P1NGUh0
JFUY0<?3``0h>3P0?Shn04m?C`0b<S80UiNG0<30`00c<c<0CTi>07UiN@19BDT0SHf=04I6AP0K6a/0
<c<c05UIF@27QhL0N7Qh01LG5`0Z:RX0F5QH04m?C`27QhL0VYZJ03/k>`11@D40EEEE065QH@1jNWX0
QhN703/k>`13@d<0Fe]K08N7Q`1nOWh0Ng]k03Xj>P1CDe<0W9bL06=SH`1eMGD0JFUY03`l?01SHf<0
X:2P06EUI@2i^KT0;2`/04E5A@29RHT0YJFU04Q8B02YZJT0IfMW0492@P2CTi<0Q8B4051@D01fMWH0
Bd];02`/;02<S8`0@d=305IFEP1MGEd0=SHf04Y:BP1cLg<0De=C07=cL`1dM7@0:b/[06IVIP31`L40
NWYj08j>SP1RHV80;bl_07YjNP31`L40OGem07AdM01NGUh0CTi>08n?S`32`/80E5AD04i>CP1lO7`0
?Shn09:BTP2`/;00>3Ph06=SH`1FEUH0MgMg0:VYZ@1SHf<0<c<c06ATI025QHD0Hf=S0<c<c01JFUX0
>3Ph06m_K`1dM7@0IFEU0<?3``1DE5@0>3Ph07moO`1FEUH0OGem0;^k^`0W9bL0EeMG0861P@1IFET0
O7al0=;BdP12@T80MGEe08f=S@1aLG40VIVI0<o?c`1ADE40J6QX0080Q8B406X0WinO0::RXP0h>3P0
GUiN0;Fe]@2CTi<0W9bL05=CD`0g=cL0D51@07EeM@2QXJ40cLg=02<S8`1JFUX0TI6A08R8R02h^;P0
Shn?03Hf=P2EUID0`/;207UiN@1fMWH0<30`07inOP2/[:`0X:2P07YjNP1^KVh0?3`l07alO02=SHd0
JFUY08^;R`0d=3@0AdM70:RXZ01KFe/0N7Qh04m?C`0a<C40G5aL069RHP2:RXX0N7Qh03@d=01/K6`0
RXZ:0451@@1FEUH0;bl_05ADE032`/80MWIf04M7A`1VIVH0=CDe08N7Q`2SXj<0DU9B04Q8B01EEED0
KFe]09JFUP2CTi<0=cLg0451@@1TI6@0DU9B0:FUY@2i^KT0>3Ph0410@01WIfL0EUIF07UiN@2XZ:P0
B4Q804E5A@2>SXh0HF5Q08F5Q@1WIfL0;bl_05eMG@1[Jf/0J6QX05QHF020P8008b<S04Y:BP1dM7@0
HV9R05MGE`2VYZH0Fe]K03/k>`2@T900J6QX04M7A`2^[Zh0@D5103Ti>@1IFET0EeMG3_ooool02P10
@400IfMW07EeM@1aLG40IVIV04a<C01>CTh0JFUY05eMG@10@40204i>CP2[03hn?P0l?3`0EeMG07If
MP1eMGD0JVYZ05eMG@1BDU80H61P04Y:BP1BDU80>c/k04Q8B01EEED0:b/[03lo?`1ADE40GemO05=C
D`1;Bd/0Hf=S065QH@1UIFD0Lg=c07QhN015ADD0Fe]K03De=@0Y:BT0>CTi08B4Q01OGel0DU9B07Yj
NP0d=3@0G5aL07EeM@1jNWX0DU9B0::RXP0[:b/0E5AD07IfMP0i>CT0FEUI04M7A`1gMgL0?Cdm07Yj
NP1ADE40D51@04=3@`1RHV80FUYJ06=SH`1TI6@0WYjN04i>CP2LW9`0K6a/07MgM`1VIVH0HF5Q09fM
W@2EUID0Ph>304Q8B022PX80U9BD0<o?c`2HV9P0KVi^05=CD`1TI6@0PH6104m?C`1TI6@0C4a<03Hf
=P1DE5@0BDU907IfMP14A4@0FEUI0492@P1GEeL0AdM7055AD@19BDT0ADE504U9B@1OGel0=3@d05EE
E@0g=cL0J6QX05EEE@1;Bd/0Cdm?08^;R`12@T80GUiN04Q8B01KFe/0@410055AD@1eMGD0DU9B05IF
EP1BDU80GemO069RHP1]KFd0Vi^K05]KF`3:b/X0X:2P05UIF@3@d=00OWin0:RXZ01DE5@0b/[:08>3
P`1_Kfl0BDU905UIF@15ADD0Cdm?059BDP1HF5P0EUIF03dm?@1FEUH0EEEE0:FUY@1FEUH0MGEe06EU
I@1PH600ATI604U9B@1?Cdl0@d=306a/K012@T80Vi^K07]kN`0o?cl0F5QH05eMG@15ADD0QXJ6069R
HP1LG5`0YJFU055AD@1TI6@0G5aL030`<020P800B4Q80861P@2GUiL0HF5Q02`/;01iNGT0=3@d05mO
G`11@D40MgMg0080A4A402<0BDU902l_;`1bLW80@D51071`L01<C4`0GUiN05MGE`1=CDd0J6QX07Ee
M@0h>3P0Lg=c06EUI@1cLg<0QhN706e]K@0`<300IfMW06][J`0m?Cd0>3Ph05UIF@14A4@0F5QH061P
H01KFe/0NWYj07EeM@1JFUX0GEeM05aLG01ZJVX0LW9b071`L00206UYJ@0h0;Vi^@1HF5P0PX:208N7
Q`1;Bd/0JVYZ061PH01YJFT0EeMG06QXJ01CDe<0LW9b07MgM`2FUYH0FUYJ07emO@1ADE40EeMG05UI
F@1VIVH0Fe]K06ATI02][Jd0RHV907AdM02/[:`0OWin069RHP1CDe<0NWYj08V9R@2BTY80_;bl0:NW
Y`2a/K40`L710:2PX034a<@0QhN70<K6aP1UIFD0N7Qh05=CD`1OGel0GUiN04E5A@1[Jf/0J6QX06IV
IP18B4P0F5QH06ATI01SHf<0RHV905iNGP2WYjL@ooooo`003_ooool02026QXH0Rh^;08Z:RP1TI6@0
PH6105]KF`1ADE40R8R80P1PH60203/k>`3o05YJFP14A4@0DU9B08>3P`1?Cdl0GEeM04E5A@1DE5@0
NGUi0:6QX@16ATH0Cdm?04Q8B01WIfL0Ti>C07EeM@0i>CT0JVYZ04a<C01bLW80SHf=02TY:@0k>c/0
GemO04U9B@2IVIT0SHf=02h^;P10@400J6QX06ATI01_Kfl0BTY:02d];@1ADE40KVi^06MWI`22PX80
@D5103<c<`1ZJVX0OGem05QHF027QhL0<S8b0410@027QhL0MGEe065QH@2BTY809RHV051@D01PH600
Ti>C071`L032`/80;2`/05=CD`1cLg<0Fe]K08R8R02EUID0;bl_04e=C@1gMgL0@D5109nOW`1DE5@0
;Rh^04e=C@1kNg/0JFUY08F5Q@0g=cL0FUYJ09NGU`29RHT0IfMW08F5Q@1<C4`0A4A409NGU`2^[Zh0
F5QH05eMG@0Z:RX0G5aL08:2PP1/K6`0B4Q806UYJ@11@D40JVYZ08f=S@14A4@0CTi>05aLG01BDU80
UIFE09jNWP1<C4`0BTY:065QH@1=CDd0SXj>08f=S@11@D40E5AD06i^KP1PH600P82006a/K01:BTX0
LG5a08F5Q@1HF5P0J6QX07MgM`1:BTX0Kfm_05mOG`1HF5P0WYjN06UYJ@1JFUX0KFe]05IFEP1QHF40
Xj>S08n?S`1?Cdl0MGEe07emO@1iNGT0a<C4061PH017AdL0Jf][07]kN`1^KVh0ZZZZ059BDP0l?3`0
L71`065QH@2;Rh/0QHF503`l?00g=cL0IFEU07MgM`2HV9P0RXZ:034a<@1<C4`0Fe]K07YjNP3:b/X0
GemO03De=@1LG5`0O7al07EeM@2IVIT0GUiN04a<C01MGEd0ZZZZ07YjNP21PH40Kfm_061PH02FUYH0
X:2P07moO`2a/K40GemO07alO02^[Zh0HV9R09FEU@1`L700ADE508B4Q01SHf<0Q8B40820P01EEED0
Ng]k08:2PP1VIVH0VIVI04i>CP1DE5@0[Zj^09FEU@1hN7P0Hf=S04i>CP1mOGd0Z:RX05aLG01cLg<0
FEUI09ZJVP2[Zj/0EUIF07EeM@23Ph<0HV9R08n?S`1<C4`0Fe]K08J6QP1QHF40GemO08N7Q`0j>SX0
Hf=S08F5Q@1fMWH0HF5Q08b<S011@D40IfMW08:2PP1GEeL0G5aL0;Vi^@15ADD0IfMW08V9R@1?Cdl0
KVi^0;Fe]@1VIVH0FEUI07=cL`1HF5P0LW9b0;fm_@1YJFT0GEeM07YjNP1:BTX0SHf=0:BTY01JFUX0
@d=306UYJ@1@D500L71`06EUI@0^;Rh0@D51000;051@D01NGUh0Ng]k08j>SP0d=3@0>CTi04e=C@1?
Cdl0EUIF08:2PP1;Bd/03_ooool0:`13@d<0GemO06i^KP1ADE40G5aL05QHF01NGUh0M7Ad04Q8B01;
Bd/0CTi>05YJFP1?Cdl0GemO05IFEP1]KFd0JVYZ06=SH`1UIFD0E5AD04e=C@0a<C40Cdm?03<c<`0e
=CD0De=C04m?C`14A4@0EUIF06ATI01RHV80@d=306ATI01EEED0LW9b069RHP1OGel0@d=304Y:BP0f
=SH0;Bd]0451@@1IFET00P1;Bd/0?`1aLG40;2`/0492@P1[Jf/0LG5a05]KF`1fMWH0?Cdm04U9B@1S
Hf<0A4A405=CD`14A4@0MGEe05EEE@1]KFd0CDe=05IFEP1GEeL0HV9R065QH@1oOgl0Kfm_0861P@1@
D500NGUi06YZJP1nOWh0Jf][05eMG@1dM7@0Hf=S05iNGP1:BTX0LG5a06i^KP2g]kL0N7Qh05aLG01?
Cdl0Lg=c079bLP1ADE40KFe]05EEE@1TI6@0IfMW0451@@1^KVh0EUIF04e=C@1HF5P0E5AD04I6AP1?
Cdl0ADE504m?C`1@D500Fe]K04Y:BP1MGEd0=3@d079bLP02069RHP2S051@D01[Jf/0@d=308:2PP1I
FET0G5aL03dm?@1BDU80FEUI06=SH`1QHF40EeMG05iNGP1VIVH0F5QH06=SH`0i>CT0]kNg07UiN@1N
GUh0]kNg07AdM02^[Zh0FUYJ0:BTY01gMgL0JFUY03`l?01ADE40IfMW07IfMP1BDU80DE5A059BDP0g
=cL0Bd];04m?C`2TY:@0BDU905iNGP1cLg<0J6QX04A4A01=CDd0Fe]K06QXJ01nOWh0Hf=S0;bl_02K
Vi/0MgMg07AdM01oOgl0DU9B08b<S023Ph<0DE5A08F5Q@1FEUH0ATI605ADE00m?Cd0KVi^05]KF`1C
De<0HF5Q069RHP0[:b/0O7al03Xj>P1oOgl0D51@069RHP0k>c/0EUIF05UIF@0h>3P0R8R805mOG`1]
KFd0De=C071`L01UIFD0EEEE061PH01]KFd0;2`/07EeM@19BDT0JFUY06i^KP1PH600;2`/065QH@1N
GUh0:BTY05ADE01XJ6P0@D5104Q8B01SHf<0FEUI06m_K`1NGUh0HV9R05IFEP1LG5`0I6AT06YZJP1e
MGD0NWYj06m_K`2WYjL0CTi>06e]K@1cLg<0GUiN07alO01MGEd0Jf][0410@01MGEd0DE5A07IfMP1a
LG40Ti>C06ATI02>SXh0HF5Q08N7Q`1UIFD0KFe]05]KF`17AdL0ZJVY07=cL`1mOGd0YJFU08N7Q`1`
L700HF5Q09>CT`2MWId0[:b/0;Ng]`2<S8`0Ph>308j>SP2LW9`0[:b/05aLG02XZ:P0PX:207UiN@1H
F5P0?clo055AD@1QHF40N7Qh06IVIP1IFET0CDe=05ADE00206i^KP0306m_K`1WIfL0[Jf]013ooooo
000>ooooo`0A09:BTP1_Kfl0PH6107AdM01UIFD0JFUY05iNGP2QXJ40@41004U9B@18B4P0EEEE09^K
V`0`<3007QhN030`<014A4@00P2CTi<04P0];Bd0A4A405=CD`1UIFD0X:2P02PX:00Y:BT0@D5108b<
S01KFe/0WYjN020P8017AdL0DE5A04a<C01iNGT0`L7103De=@80IFEU0<<0I6AT08>3P`2j^[X0>CTi
04i>CP1oOgl0ADE507inOP27QhL09RHV05ADE02;Rh/0GEeM08N7Q`0a<C407AdM06UYJ@1jNWX0FUYJ
04i>CP0X:2P0:b/[03`l?01CDe<0[jn_04E5A@0F5QH0;Rh^05EEE@22PX80EEEE0492@P0d=3@0MgMg
08Z:RP1JFUX0CTi>05EEE@0Y:BT0EEEE08b<S00d=3@0GemO04i>CP0/;2`0Ng]k06m_K`0m?Cd0GUiN
02PX:00h>3P0GUiN051@D00_;bl0EeMG04e=C@1UIFD0/;2`04Q8B00d=3@0H61P04];B`0d=3@0Rh^;
02l_;`0];Bd0DE5A05eMG@1UIFD0][Jf03Lg=`0n?Sh0MWIf05mOG`1hN7P0][Jf03hn?P1UIFD0J6QX
051@D02BTY80`<3004i>CP1OGel0LG5a05UIF@2KVi/0e]KF03De=@1>CTh0TI6A07IfMP37alL0JFUY
02DU9@1PH600W9bL07emO@2DU9@0?clo034a<@1JFUX0clo?08f=S@2EUID08R8R06IVIP2SXj<0_[jn
09nOW`2WYjL0?3`l06IVIP1[Jf/0Vi^K05=CD`2EUID0<C4a07YjNP1^KVh0Hf=S06QXJ02h^;P0=SHf
07EeM@2][Jd0Ph>307inOP2?Shl0A4A408^;R`2m_Kd0EUIF07AdM01RHV80<S8b07emO@32`/80Hf=S
0;Ng]`14A4@0BTY:06QXJ02CTi<0O7al0820P00[:b/0KVi^09bLW01oOgl0[Jf]06QXJ017AdL0T92@
0=;BdP1TI6@0E5AD0410@01aLG40_;bl06e]K@13@d<0EUIF06i^KP3AdM40PH6103Lg=`1CDe<0F5QH
06i^KP27QhL0?Shn04U9B@1>CTh0LG5a0<71`@1SHf<0FEUI06YZJP1]KFd0[jn_0;6a/@19BDT0S8b<
065QH@1kNg/0aLG5079bLP0V9RH0OWin05QHF023Ph<0VYZJ03Ph>00m?Cd00P1gMgL0:P2a/K40FUYJ
038b<P17AdL0HV9R07MgM`2n_[h0TI6A0410@00e=CD0GEeM09^KV`2BTY80KVi^03dm?@0o?cl0]KFe
0<W9b@2YZJT0?clo030`<01;Bd/0YjNW0;6a/@1eMGD0N7Qh024Q8@0i>CT0QHF508:2PP1jNWX0/k>c
04E5A@0f=SH0Fe]K07alO018B4P0Ogmo03Hf=P0_;bl0@D5105eMG@kooooo0100C4a<04E5A@1eMGD0
IfMW07AdM01PH600KFe]08V9R@1GEeL0DU9B04];B`1ADE40@41006YZJP1mOGd0S8b<0`1fMWH02@1_
Kfl0I6AT03Ti>@1FEUH0=cLg04=3@`1NGUh0<c<c04e=C@0206MWI`0g06YZJP1HF5P0NGUi05UIF@2>
SXh0M7Ad06i^KP16ATH0D51@03Ti>@0c<c<0@41006e]K@1FEUH0A4A405iNGP0e=CD0=cLg055AD@1J
FUX0Fe]K0820P00R8R80@41005iNGP0n?Sh0EEEE04e=C@1OGel0ADE506MWI`19BDT0F5QH04A4A01]
KFd0JFUY07YjNP1bLW80RHV904U9B@2:RXX0Lg=c07emO@1FEUH0G5aL07EeM@1iNGT0TY:B06MWI`25
QHD0MgMg0>3Ph01nOWh0IFEU04m?C`0205iNGP2004Y:BP1QHF40GUiN04Q8B01QHF40A4A407AdM00h
>3P0ADE506e]K@1GEeL0>CTi05EEE@18B4P0?Cdm04U9B@1HF5P0?Shn04e=C@1?Cdl0TI6A05IFEP1>
CTh0EEEE06m_K`1:BTX0>CTi051@D01PH600GemO05EEE@1^KVh0AdM705QHF01lO7`0EUIF03`l?01[
Jf/0I6AT024Q8@2QXJ40D51@079bLP37alL0MgMg092@T01=CDd0]kNg06UYJ@1EEED0@D5105aLG01A
DE40MGEe06a/K01?Cdl0C4a<03Hf=P1ADE40>CTi0:FUY@1?Cdl0L71`06e]K@1>CTh0Cdm?07MgM`1a
LG40DU9B051@D01QHF40g=cL09bLW013@d<0F5QH06UYJ@19BDT0[Zj^0:f][@12@T80SHf=04Y:BP1@
D500MGEe05mOG`1mOGd0U9BD06EUI@1NGUh0Hf=S02XZ:P1[Jf/0<C4a07IfMP1:BTX0QXJ604M7A`1:
BTX0CTi>03@d=01YJFT0GUiN06][J`0o?cl0LW9b05YJFP0e=CD0E5AD061PH01>CTh0KVi^04a<C01Y
JFT0EeMG08f=S@14A4@0Jf][061PH01BDU80CTi>05]KF`1bLW80Fe]K04m?C`1TI6@0QHF505ADE01;
Bd/204i>CP0L06=SH`1YJFT0GUiN06MWI`1PH600[Jf]04M7A`1QHF40IFEU05mOG`24Q8@0ADE504e=
C@1BDU80IfMW04i>CP1PH600Ng]k09:BTP1SHf<0U9BD06MWI`1CDe<0ADE505UIF@16ATH0E5AD09ZJ
VP80JFUY01l0Ph>305mOG`1dM7@0GemO069RHP1eMGD0PX:20<C4a02JVYX0^;Rh0:^[Z`2BTY80XZ:R
07MgM`2GUiL0K6a/06UYJ@1MGEd0CTi>05MGE`1?Cdl0N7Qh0:BTY02ATI40De=C05mOG`1/K6`0Jf][
07moO`2>SXh0[Zj^013ooooo000>ooooo`200;Jf]P1OGel0KVi^092@T02l_;`0ATI606EUI@2ZZZX0
N7Qh09BDU00V9RH0CTi>08J6QP1gMgL0N7Qh09FEU@0l?3`0IVIV09nOW`1SHf<0SXj>0::RXP0^;Rh0
E5AD07QhN01DE5@0Ogmo03Xj>P0/;2`0NWYj04=3@`0i>CT0Kfm_02DU9@0[:b/0S8b<06a/K01=CDd0
?Shn020P8018B4P0Shn?08Z:RP1>CTh0?3`l02`/;01UIFD0Shn?03<c<`19BDT0>CTi05QHF02WYjL0
X:2P024Q8@0i>CT0IFEU071`L01hN7P0<C4a03Lg=`0l?3`0?Shn0<?3``25QHD07AdM02d];@16ATH0
CDe=07moO`1gMgL07QhN03dm?@1EEED0A4A4079bLP2CTi<0<C4a04U9B@1?Cdl0E5AD09JFUP2<S8`0
?3`l04Y:BP1LG5`0QHF50<G5a@1nOWh0@D5104a<C01MGEd0Jf][09fMW@2ATI40<c<c071`L01TI6@0
F5QH09BDU01NGUh0;Rh^059BDP1KFe/0Ti>C075aL@0Z:RX0?Shn06EUI@1^KVh0Rh^;075aL@0d=3@0
@d=303lo?`1IFET0T92@061PH00c<c<0@T9205YJFP1kNg/0R8R805]KF`0c<c<0@D51096AT@23Ph<2
05EEE@1D04Q8B01bLW80QHF504I6AP1lO7`0ADE5061PH01kNg/0L71`03Hf=P1VIVH0Fe]K05QHF026
QXH0@D510492@P2EUID0@41006m_K`2KVi/0CDe=0492@P1jNWX0CDe=06UYJ@27QhL0;Rh^055AD@1`
L700DE5A075aL@2_[jl0AdM70410@01_Kfl0LW9b07=cL`2c/k<0ATI605=CD`2FUYH0PH610::RXP2A
TI40;bl_05UIF@1cLg<0MGEe0:n_[`2<S8`0@41005aLG020P800QhN70:n_[`0V9RH0EEEE09FEU@1k
Ng/0c/k>05]KF`0b<S80KVi^0:JVYP2PX:00SHf=02`/;01ZJVX0TI6A06m_K`2YZJT0IFEU0451@@23
Ph<0TI6A08F5Q@2LW9`0B4Q806a/K03>c/h0Kfm_0:n_[`1JFUX09BDU0P2CTi<0?P2PX:00Fe]K038b
<P1MGEd0ZZZZ08^;R`1kNg/0<S8b05MGE`29RHT0cLg=09:BTP27QhL0@T9205UIF@1nOWh0QHF505eM
G@1^KVh0<C4a08>3P`2@T900AdM7059BDP2JVYX0AdM705eMG@2][Jd0C4a<04m?C`1LG5`0BDU905QH
F02RXZ80BTY:03hn?P1mOGd0GEeM079bLP1fMWH0>3Ph05=CD`1?Cdl0D51@05ADE01kNg/0<C4a04M7
A`1IFET0<30`06YZJP2m_Kd0N7Qh05YJFP1?Cdl0ATI603Ph>02IVIT0IVIV03/k>`1MGEd0J6QX3_oo
ool0o`1?Cdl0FUYJ07IfMP1KFe/0EEEE065QH@1bLW80K6a/04E5A@18B4P0Bd];04i>CP1:BTX0H61P
04U9B@1:BTX0EEEE059BDP1OGel0FUYJ03Ti>@0];Bd0J6QX038b<P0k>c/0MgMg03@d=00l?3`0C4a<
079bLP1YJFT0B4Q806IVIP2:RXX0N7Qh06YZJP1XJ6P0IFEU05mOG`1:BTX0A4A405mOG`1UIFD0G5aL
055AD@1WIfL0>3Ph03dm?@1RHV80KVi^04a<C023Ph<0;bl_04U9B@1UIFD0C4a<05QHF00o?cl0MgMg
04e=C@29RHT0AdM704e=C@15ADD0GEeM05IFEP1`L700Lg=c0;6a/@1>CTh0d];B0:ZZZP2OWil0IFEU
051@D022PX80CTi>05iNGP0l?3`0HV9R06i^KP3>c/h0RHV906][J`1LG5`0Ph>307AdM01IFET0KFe]
04e=C@1SHf<0FUYJ03/k>`1KFe/0@T92051@D017AdL0GUiN04Y:BP1SHf<0CDe=05QHF01EEED0EeMG
069RHP1PH600>CTi06UYJ@1@D500I6AT04U9B@1oOgl0@d=305iNGP1GEeL0Jf][03De=@1;Bd/0Hf=S
05UIF@1PH600P82005MGE`18B4P0A4A405eMG@1DE5@0[Jf]06QXJ011@D40TY:B07]kN`2IVIT0De=C
07=cL`1_Kfl0Rh^;05IFEP1PH600A4A404a<C01OGel0EEEE04A4A00f=SH0EUIF03Ph>02?Shl0Bd];
05YJFP1jNWX0EeMG03De=@17AdL0ADE505ADE01FEUH0=CDe06e]K@0o?cl0=SHf05YJFP1]KFd092@T
079bLP1VIVH0=CDe06UYJ@1@D500IFEU06UYJ@0m?Cd0IVIV06ATI01hN7P0K6a/06YZJP1>CTh0TI6A
03De=@20P800Cdm?061PH00n?Sh0C4a<05=CD`0f=SH0Kfm_06i^KP2>SXh0GEeM08N7Q`1UIFD0AdM7
05iNGP1JFUX07alO06IVIP0Z:RX0DU9B03hn?P1eMGD0;Bd]06YZJP1NGUh07AdM05MGE`1NGUh0=cLg
03`l?01?Cdl0C4a<07]kN`0o?cl0Bd];04m?C`16ATH0FEUI05mOG`1VIVH0MGEe079bLP2g]kL0>SXj
06][J`1SHf<0DE5A08J6QP19BDT0@T920451@@1FEUH0F5QH06EUI@1dM7@0SXj>06i^KP2KVi/0ATI6
06ATI01@D500MWIf08R8R01_Kfl0VYZJ08n?S`22PX80/k>c0:VYZ@22PX80QHF508V9R@1dM7@0XZ:R
0=WIf@004`2DU9@0Kfm_09BDU02KVi/0b<S8096AT@2WYjL0G5aL06MWI`1VIVH0J6QX06][J`1VIVH0
IFEU07alO01OGel0Bd];055AD@1UIFD00P1bLW800`1`L700Ph>3ooooo`0?ooooo`003_ooool01P2I
VIT0OGem092@T01HF5P0VIVI05]KF`80KVi^09X0?clo07MgM`1=CDd0>SXj05IFEP0e=CD0>CTi092@
T011@D40@T9205mOG`12@T80MgMg0;no_`1HF5P0E5AD06UYJ@2=SHd0a/K606i^KP0R8R80GemO05MG
E`2JVYX0aLG502DU9@0l?3`0DE5A079bLP31`L40T92@024Q8@17AdL0De=C05MGE`2BTY80De=C034a
<@1FEUH0DE5A07IfMP2ATI406QXJ03lo?`1:BTX0IfMW05]KF`2QXJ40=SHf03Lg=`15ADD0G5aL06AT
I01kNg/09BDU05=CD`2TY:@0GUiN07IfMP1kNg/09BDU03hn?P29RHT0FUYJ07IfMP1ZJVX08B4Q04e=
C@2HV9P0BTY:075aL@13@d<0:RXZ04Q8B02>SXh0>CTi06IVIP0M7Ad0EeMG08^;R`22PX80FUYJ07Yj
NP0e=CD0;bl_06UYJ@2RXZ80De=C04e=C@0d=3@0ATI605ADE02ATI40@d=305MGE`0U9BD0Ng]k09^K
V`13@d<0=CDe04i>CP15ADD0S8b<0;>c/`1FEUH0@D5106=SH`11@D40PX:20:6QX@0[:b/0CDe=07If
MP1LG5`0TY:B09nOW`15ADD0EUIF05eMG@24Q8@0[jn_08j>SP0l?3`0EEEE06e]K@18B4P0^KVi06i^
KP15ADD0EUIF07]kN`1JFUX0_;bl08F5Q@19BDT0J6QX07=cL`27QhL0eMGE0451@@1IFET0WYjN0:NW
Y`2RXZ80aLG503<c<`0n?Sh0Shn?05YJFP2ATI40OWin03Xj>P1XJ6P0TI6A0P2CTi<0C@1GEeL0?Cdm
06e]K@1OGel0YZJV09VIV@0m?Cd0L71`09BDU02MWId0F5QH06a/K01:BTX0C4a<07moO`2h^;P0Ph>3
07AdM01XJ6P0W9bL08^;R`1gMgL0P8200<K6aP1<C4`0YjNW0<C4a01;Bd/0LG5a07AdM01WIfL0WYjN
059BDP1NGUh0LW9b04M7A`1MGEd0WinO04E5A@1FEUH0AdM705=CD`2`/;00Kfm_05YJFP1_Kfl0MGEe
09bLW02MWId0<S8b06IVIP1_Kfl0SHf=09RHV00/;2`0>3Ph07inOP20P800UYJF03hn?P1FEUH0J6QX
07EeM@1PH600[jn_0410@01;Bd/0F5QH07YjNP1NGUh0TI6A03De=@1:BTX0OGem055AD@1MGEd0/;2`
0080>3Ph00@0J6QX061PH022PX80_Kfm0P1=CDd07@1lO7`0L71`07]kN`2/[:`0@41005iNGP27QhL0
@D5108V9R@1jNWX09bLW04e=C@1oOgl0OGem07EeM@1FEUH0;2`/03`l?01BDU80Kfm_07]kN`1hN7P0
:b/[02HV9P18B4P0IFEU04M7A`2GUiL0ATI600kooooo02D0Lg=c05MGE`1iNGT0GemO08>3P`1JFUX0
F5QH092@T018B4P0EUIF04A4A01aLG40J6QX07QhN01GEeL0LG5a06UYJ@1kNg/0L71`04E5A@1:BTX0
<C4a04i>CP1:BTX0?Shn05ADE01;Bd/0>c/k05iNGP1EEED0EeMG0492@P1PH600@D5105eMG@1>CTh0
G5aL0080>CTi01D0:2PX03Hf=P0l?3`0C4a<05QHF01DE5@0EEEE03Ph>00i>CT0DE5A051@D01:BTX0
D51@03Ph>01>CTh0C4a<0451@@1KFe/0FUYJ05IFEP17AdL00`1WIfL0e`11@D40KVi^07emO@2@T900
J6QX0:^[Z`1cLg<0LG5a08F5Q@2;Rh/0Q8B4069RHP1SHf<0Cdm?08F5Q@2;Rh/0NWYj04U9B@2m_Kd0
SXj>05iNGP1HF5P0R8R807IfMP19BDT0GEeM05YJFP1:BTX0EeMG03De=@1RHV809BDU04];B`1EEED0
F5QH059BDP1_Kfl0B4Q804m?C`1>CTh0BTY:04e=C@1EEED0?Cdm07EeM@16ATH0JFUY04E5A@1^KVh0
ADE5075aL@18B4P0G5aL03hn?P17AdL0GemO07EeM@1gMgL0D51@05MGE`1PH600DE5A06=SH`15ADD0
Ng]k04m?C`1RHV80P820069RHP2ATI40JVYZ09ZJVP20P800JFUY03De=@18B4P0Jf][06m_K`1BDU80
EeMG05aLG00h>3P0D51@01XJ6P1jNWX0EeMG06YZJP1`L700BTY:0451@@1VIVH0D51@0492@P1<C4`0
BTY:0<[:bP1nOWh0@d=306a/K012@T809bLW08J6QP1cLg<0@T9209FEU@0o?cl0Bd];04Q8B00a<C40
IfMW05MGE`1UIFD0AdM705IFEP0i>CT0M7Ad06=SH`28R8P0F5QH079bLP0m?Cd0Cdm?059BDP0n?Sh0
LG5a05eMG@1jNWX0EEEE07MgM`1IFET0FUYJ0820P01PH600:RXZ075aL@0m?Cd0IVIV04m?C`1_Kfl0
;Rh^06a/K01JFUX09BDU04Q8B01>CTh0>SXj04m?C`1cLg<0D51@06e]K@1HF5P0J6QX051@D01EEED0
I6AT06MWI`22PX80UIFE05YJFP26QXH0@41007inOP1VIVH0Bd];06i^KP19BDT0AdM704Q8B01KFe/0
EeMG04e=C@27QhL0Zj^[06ATI02?Shl0MGEe08N7Q`1lO7`0EEEE0492@P1eMGD0bLW906][J`1WIfL0
M7Ad05UIF@1<C4`0D51@06i^KP2g]kL0UIFE0:NWY`2YZJT0b<S80:j^[P1fMWH0WIfM07UiN@3Fe]H0
DU9B06EUI@0n?Sh0BTY:04I6AP1=CDd0De=C0820P01PH600@D51055AD@1[Jf/0I6AT05IFEP2<S8`0
RHV9013ooooo000>ooooo`3o06YZJP1cLg<0`/;205QHF01cLg<0Hf=S08F5Q@37alL0<C4a071`L02>
SXh0Rh^;09bLW00Z:RX0=SHf04Y:BP1lO7`0V9RH03@d=010@400@D5104A4A01^KVh0U9BD03<c<`0/
;2`0EEEE09VIV@1cLg<0FEUI03@d=019BDT0DU9B04i>CP10@400Jf][02TY:@2ATI40V9RH03Hf=P0m
?Cd0L71`030`<01;Bd/0VYZJ03Xj>P1[Jf/0CDe=03De=@1aLG40CDe=03<c<`1?Cdl0;2`/0410@01a
LG40AdM703Xj>P0m?Cd0=3@d06a/K02>SXh09RHV03/k>`0h>3P0>SXj09FEU@3AdM40=3@d03`l?017
AdL0ATI60:RXZ037alL08B4Q01TI6@1^KVh0IFEU06i^KP1KFe/0:BTY03dm?@1aLG40PH610:^[Z`0a
<C40;Bd]03lo?`1`L700PH610:n_[`1>CTh0:2PX0492@P1jNWX0QhN708:2PP0P8200A4A404m?C`2H
V9P0IFEU09RHV00S8b<0=3@d05]KF`1XJ6P0CTi>06i^KP0n?Sh0ATI6079bLP2>SXh0W9bL0<;2`P13
@d<0L71`09bLW01RHV80Vi^K0<[:bP10@400Fe]K0:VYZ@1gMgL0VYZJ0861P@0o?cl0Kfm_0;Vi^@1Y
JFT0b<S806e]K@19BDT0@d=3092@T01jNWX0Kfm_02TY:@18B4P0I6AT0:6QX@1NGUh0Shn?03Ph>01>
CTh0G5aL08^;R`2:RXX0OWin03`l?01SHf<0`/;206IVIP13@d<0HF5Q03Xj>P1/K6`0Xj>S02d];@0g
=cL0LW9b04e=C@2KVi/0/[:b02h^;P0m?Cd0KFe]05ADE02MWId0Shn?03<c<`1OGel0PH6106e]K@2Z
ZZX0<S8b03dm?@18B4P0MgMg06IVIP2e]KD0OWin04];B`1hN7P0Kfm_07alO02e]KD0ATI6055AD@1U
IFD0Hf=S0;Zj^P2FUYH0<C4a08R8R01KFe/0];Bd0:>SX`0g=cL0BTY:07=cL`2BTY80Ph>302XZ:P19
BDT0P82009bLW02HV9P0DU9B03Lg=`1OGel0]kNg05=CD`2/[:`0FEUI0:6QX@30`<00NGUi075aL@1G
EeL0CDe=08:2PP1DE5@0SXj>09bLW00`<300C4a<07=cL`1TI6@0UIFE0:f][@0a<C40Bd];07moO`1G
EeL0P82005YJFP0^;Rh0>CTi07emO@1eMGD0N7Qh06a/K00U9BD0B4Q807emO@2>SXh0E5AD0410@00g
=cL0EEEE08j>SP2ZZZX0DE5A000G04];B`1ADE40Jf][09:BTP1;Bd/0=3@d059BDP1@D500L71`07in
OP0^;Rh0:RXZ05aLG01OGel0PH6109^KV`12@T808B4Q04=3@`1QHF40C4a<08^;R`2ATI403_ooool0
3`1LG5`0AdM709:BTP1fMWH0LW9b05=CD`1LG5`0KVi^06a/K01BDU80EEEE05aLG014A4@0GUiN07=c
L`0207AdM01D092@T01dM7@0BTY:05eMG@0n?Sh0DU9B04i>CP19BDT0GEeM03@d=01=CDd0HV9R06e]
K@1eMGD0EEEE07moO`1HF5P0S8b<06=SH`1cLg<0BTY:04=3@`0U9BD0:2PX059BDP1KFe/0GemO0492
@P1kNg/0<c<c0492@P1EEED0IfMW04U9B@27QhL092@T0410@01YJFT0A4A404Q8B00U9BD0E5AD03hn
?P1cLg<0<C4a03Hf=P0k>c/0@d=303<c<`1EEED0EeMG065QH@0a<C40FEUI071`L024Q8@0?Cdm03Lg
=`1jNWX0EeMG04m?C`0U9BD0D51@06EUI@35aLD0Fe]K04i>CP15ADD0BDU904a<C01;Bd/0BDU903`l
?014A4@0BTY:03<c<`1VIVH0:b/[04e=C@1^KVh0GemO04A4A01WIfL0H61P0P1@D500J@1PH600B4Q8
05aLG00i>CT0MgMg03Xj>P1PH600CTi>07alO010@400J6QX04Q8B01WIfL0=3@d05EEE@1aLG40FUYJ
07=cL`20P800EEEE03Hf=P0n?Sh0=SHf02HV9P2TY:@0K6a/05mOG`2NWYh0K6a/09jNWP1;Bd/0Z:RX
09fMW@1jNWX0GemO06UYJ@1PH600FUYJ06=SH`1BDU80E5AD04U9B@1SHf<092@T08f=S@1NGUh0F5QH
05YJFP1CDe<0>c/k04M7A`1;Bd/0I6AT06IVIP1NGUh0d];B08J6QP0f=SH0IVIV07IfMP0i>CT0Ti>C
07UiN@16ATH0DU9B03Ph>01CDe<0Cdm?04Y:BP1eMGD0ATI607moO`1>CTh0DU9B02DU9@1aLG40B4Q8
08R8R01=CDd0HV9R03Hf=P1RHV80GUiN04I6AP1eMGD0I6AT079bLP1:BTX0N7Qh06e]K@15ADD0CDe=
05eMG@0/;2`0DU9B03lo?`1KFe/0O7al08>3P`1;Bd/0KVi^05mOG`0f=SH0BTY:051@D00203lo?`14
04];B`1@D500IfMW04A4A01=CDd0BTY:04E5A@1OGel0F5QH05aLG01HF5P0D51@08N7Q`13@d<0HV9R
06IVIP1DE5@0N7Qh05eMG@1BDU80>CTi05]KF`1RHV80H61P06MWI`1NGUh0DE5A079bLP1HF5P0FUYJ
04m?C`1EEED0Hf=S05]KF`23Ph<0GEeM05iNGP1oOgl0NWYj06a/K01CDe<0G5aL05ADE01HF5P0VYZJ
06EUI@1@D500IFEU05QHF01kNg/0@D5109FEU@1>CTh0JVYZ04=3@`1<C4`0E5AD051@D01IFET0S8b<
06ATI010@400DE5A079bLP1mOGd0EUIF08F5Q@1mOGd@ooooo`003_ooool0CP2NWYh0F5QH05]KF`26
QXH0MGEe0492@P15ADD0R8R809bLW01UIFD0:BTY05aLG027QhL0IFEU03hn?P1XJ6P0ATI608F5Q@2`
/;00>c/k03hn?P13@d<0BTY:07]kN`1QHF40;bl_04m?C`1PH600J6QX0;>c/`0X:2P0<30`051@D00g
=cL0MWIf0<_;b`0n?Sh0?clo08:2PP14A4@0NGUi0<k>cP0n?Sh0@D510:BTY017AdL0^;Rh08n?S`0^
;Rh0E5AD061PH02?Shl0Q8B403Hf=P0R8R80J6QX04a<C02FUYH0LW9b01<C4`0e=CD0GemO05eMG@2N
WYh0A4A402DU9@0e=CD0K6a/07emO@1QHF40BDU901lO7`1?Cdl0UiNG065QH@1<C4`0I6AT02/[:`80
KVi^03<0?clo061PH01`L700EUIF092@T022PX80EUIF08:2PP1@D500?clo06UYJ@1TI6@0Bd];09FE
U@15ADD0?Cdm071`L013@d<0?clo06m_K`0n?Sh0=CDe069RHP0e=CD0@T9204e=C@0`<300Lg=c0:NW
Y`0l?3`0IfMW06][J`0T92@0Cdm?09ZJVP10@400EeMG04];B`14A4@0F5QH0:JVYP1GEeL0DE5A04e=
C@1@D500YZJV0;^k^`18B4P0A4A4069RHP1WIfL00P2OWil0500l?3`0F5QH04=3@`2GUiL0Zj^[02h^
;P0h>3P0EUIF07EeM@1lO7`0c<c<04Q8B01@D500L71`09>CT`2=SHd0W9bL03<c<`1>CTh0GemO0P2A
TI405P1jNWX0ADE5055AD@1ZJVX0GUiN06m_K`2^[Zh0@T9205eMG@1gMgL0N7Qh08R8R02LW9`0Bd];
06UYJ@2HV9P0IFEU0;:b/P2SXj<0@d=307IfMP26QXH208>3P`0i04e=C@0l?3`0H61P08V9R@1NGUh0
Ng]k01XJ6P17AdL0YjNW079bLP2n_[h0>CTi03Ph>01FEUH0X:2P06YZJP1bLW80@41006YZJP3:b/X0
S8b<055AD@1HF5P0I6AT0;fm_@2IVIT0B4Q807UiN@0k>c/0N7Qh08b<S00g=cL0GemO04Q8B01GEeL0
Shn?05QHF01VIVH0UYJF04Y:BP2QXJ40WYjN051@D01QHF40Bd];06IVIP2a/K40F5QH04U9B@2>SXh0
Kfm_096AT@3?cll0AdM704Q8B01MGEd0Fe]K0080V9RH02P0;bl_0451@@1FEUH0GUiN08b<S02;Rh/0
:RXZ04M7A`1aLG40RHV90;>c/`1PH600>SXj065QH@1JFUX0GemO0:BTY00i>CT0D51@08B4Q02NWYh0
MGEe07QhN020P800?Cdm05=CD`23Ph<0E5AD07UiN@2g]kL0ATI605]KF`1hN7P0De=C05QHF02=SHd0
BTY:02d];@0l?3`0LG5a3_ooool0S`14A4@0B4Q807]kN`1EEED0R8R804a<C01GEeL0VIVI04U9B@1W
IfL0DU9B06EUI@16ATH0HF5Q05IFEP1RHV80DU9B05eMG@1^KVh0De=C04A4A00`<300EEEE0410@00m
?Cd0Hf=S04Y:BP0j>SX0HF5Q079bLP1PH600?Shn05eMG@1VIVH0IfMW06m_K`1eMGD0ADE505QHF012
@T80:2PX04Q8B01XJ6P0I6AT04A4A01CDe<0?clo05EEE@1iNGT0GEeM055AD@1mOGd0?Shn04M7A`1P
H600CTi>051@D00b<S80H61P05ADE01^KVh0?clo055AD@14A4@0Lg=c05YJFP1XJ6P0G5aL0:6QX@1:
BTX0MgMg06ATI02UYJD0G5aL04I6AP1IFET0CTi>061PH011@D40GemO07YjNP3Fe]H0Lg=c06][J`1H
F5P0IFEU06UYJ@1OGel0Jf][05EEE@1hN7P0KVi^03Hf=P1TI6@0;Rh^06MWI`15ADD0DE5A04Y:BP1M
GEd0@41004a<C017AdL0GemO05IFEP1QHF40;2`/03hn?P0l?3`0NGUi061PH01kNg/0FUYJ07QhN01F
EUH0M7Ad03Lg=`14A4@0Fe]K06EUI@1NGUh0IVIV05iNGP12@T80B4Q8051@D00/;2`0ZJVY07EeM@12
@T80IVIV071`L02WYjL0KFe]0;jn_P2@T900V9RH05EEE@1SHf<0AdM705QHF01cLg<0J6QX0080=SHf
04P0F5QH02/[:`1eMGD0Cdm?06EUI@1/K6`0M7Ad055AD@1MGEd0F5QH06ATI02CTi<0NGUi0?Win@2e
]KD0BTY:06EUI@1dM7@0@T9207alO01gMgL0GUiN061PH01WIfL0EEEE05IFEP13@d<0Cdm?051@D02R
XZ80IVIV05MGE`0^;Rh0GUiN034a<@3Bd]80EUIF05eMG@0j>SX0?clo05UIF@1^KVh0MGEe06MWI`2V
YZH0HV9R07moO`1_Kfl0D51@05aLG01UIFD0BDU904];B`13@d<0GEeM07AdM01kNg/0>3Ph09ZJVP1Y
JFT06a/K04m?C`1ADE40=3@d04Q8B01CDe<0B4Q806i^KP19BDT0AdM705UIF@1BDU8206YZJP0h06][
J`1mOGd0PH6109BDU01<C4`0IFEU065QH@1XJ6P0UiNG065QH@1JFUX0HV9R06i^KP1IFET0DE5A07Yj
NP2MWId0Kfm_096AT@15ADD0EeMG04Y:BP1EEED0G5aL06MWI`2LW9`0@41006IVIP26QXH0OGem07Mg
M`1DE5@0N7Qh07alO02KVi/0YJFU07MgM`1mOGd0IFEU05eMG@1lO7`0KVi^09^KV`0j>SX0NGUi04I6
AP1:BTX0DU9B05ADE01;Bd/0PX:2055AD@15ADD0IFEU07QhN01WIfL207MgM`03092@T?oooooooooo
00kooooo000>ooooo`1`0;Rh^01EEED0G5aL07]kN`2n_[h0ADE504U9B@1CDe<0K6a/0;Ng]`0h>3P0
?Cdm04=3@`1fMWH0QhN70:JVYP0n?Sh0De=C09nOW`22PX80clo?08N7Q`0d=3@0EUIF0;6a/@1WIfL0
ZZZZ03dm?@0n?Sh0OWin07YjNP1XJ6P0[Jf]020P800`<300N7Qh0820P01SHf<0M7Ad02DU9@0^;Rh0
OGem08f=S@1BDU80=cLg01`L7013@d<0SHf=06][J`1gMgL08B4Q05IFEP2ZZZX0WYjN04i>CP1eMGD0
?Cdm05]KF`2DU9@0ADE50492@P18B4P0>SXj06UYJ@25QHD0;Rh^05MGE`13@d<0C4a<06a/K01NGUh0
=cLg05eMG@1IFET0BDU904M7A`2NWYh0<C4a04Y:BP17AdL0=CDe075aL@1]KFd0<S8b06EUI@1EEED0
GEeM092@T02VYZH0=CDe04i>CP1KFe/0IVIV0;Bd]02@T900?Shn04i>CP1<C4`0JFUY0;>c/`17AdL0
=CDe05mOG`19BDT0MgMg06QXJ00M7Ad0:b/[07AdM01UIFD0/[:b09^KV`80:2PX0:@0Bd];05]KF`2i
^KT0NWYj038b<P1PH600QHF509BDU02FUYH0Jf][04A4A01MGEd0Rh^;07MgM`1RHV80H61P04=3@`1m
OGd0LG5a06ATI02OWil0F5QH055AD@2c/k<0Xj>S05]KF`2TY:@0E5AD04Y:BP2@T900Ti>C05aLG027
QhL0=cLg06MWI`2LW9`0Ogmo08N7Q`2<S8`0;2`/03/k>`2/[:`0B4Q806QXJ01TI6@0<30`04m?C`2R
XZ80FEUI06=SH`0l?3`0A4A4069RHP2^[Zh0HV9R08B4Q01=CDd0>SXj06a/K02HV9P0?3`l04a<C00c
<c<0SHf=0;bl_013@d<0:BTY06e]K@1PH600Ti>C0<c<c018B4P0;bl_07]kN`1bLW80alO708N7Q`0S
8b<0HV9R08N7Q`1cLg<0SXj>02`/;00n?Sh0Hf=S06a/K02IVIT0Ogmo038b<P1MGEd0U9BD09ZJVP39
bLT0@D5104A4A02;Rh/0Kfm_0;^k^`26QXH0@D5106IVIP1UIFD0VIVI0;Vi^@0b<S80<c<c07alO02:
RXX0NWYj030`<01EEED0IfMW0:RXZ01SHf<0RXZ:03hn?P1bLW80MgMg0861P@1aLG40Rh^;04I6AP2C
Ti<0^k^k05EEE@1QHF40/k>c04A4A01PH600Z:RX08:2PP1aLG40`l?303`l?01XJ6P0Xj>S04a<C02>
SXh0ZJVY03lo?`1BDU80LW9b030`<01IFET0OWin04Q8B01TI6@0U9BD03Xj>P1SHf<0N7Qh04M7A`0m
?Cd0T92@071`L01HF5P0Jf][0492@P1<C4`0MWIf08N7Q`1?Cdl0QXJ605ADE0kooooo03h0@d=304U9
B@1lO7`0Kfm_07EeM@1<C4`0D51@08B4Q01<C4`0Fe]K05EEE@1XJ6P0EUIF07MgM`1RHV80JVYZ07mo
O`1fMWH0HF5Q04a<C01ADE40:2PX04i>CP0f=SH0:BTY04Q8B00L71`0@D5104a<C01IFET0NWYj04e=
C@1iNGT0BTY:07MgM`1HF5P0GemO04a<C01JFUX0>c/k03hn?P1?Cdl0ATI605=CD`1GEeL0IFEU0410
@00n?Sh0BDU905UIF@0g=cL0D51@03lo?`1=CDd0E5AD03/k>`1GEeL0<c<c06EUI@17AdL0NGUi04I6
AP80CDe=01D0G5aL059BDP2@T900N7Qh0:n_[`14A4@0QXJ608B4Q02FUYH0Kfm_06IVIP1SHf<0A4A4
07UiN@1EEED0GEeM05MGE`2QXJ40Kfm_05IFEP1BDU800P1]KFd0C@0i>CT0EEEE051@D01=CDd0GemO
04E5A@1bLW80ADE504Q8B01<C4`0D51@04a<C01CDe<0EeMG05mOG`1ADE40JVYZ04E5A@1BDU80@T92
07inOP1NGUh0FEUI059BDP1fMWH0I6AT08Z:RP1DE5@0IFEU03@d=01;Bd/0Fe]K07UiN@1fMWH0Jf][
0820P01bLW80SXj>069RHP13@d<0S8b<075aL@0n?Sh0EUIF06ATI02?Shl0DE5A08b<S01QHF40KVi^
03dm?@15ADD0JFUY07=cL`1CDe<0CDe=059BDP0b<S80De=C01PH602>SXh0De=C08>3P`2FUYH0NGUi
04Q8B017AdL0HF5Q08B4Q01mOGd0QHF50=KFeP2SXj<0DE5A06UYJ@1cLg<0A4A40080OGem0340@T92
06m_K`13@d<0CTi>051@D019BDT0PX:204i>CP24Q8@0AdM703`l?013@d<0Q8B403dm?@1WIfL0?3`l
06UYJ@1CDe<0DU9B05eMG@1;Bd/0IFEU059BDP1WIfL0@410061PH01SHf<0EeMG07UiN@1]KFd0A4A4
069RHP1FEUH0OGem06m_K`22PX80B4Q805YJFP1@D500AdM707UiN@1]KFd0I6AT061PH01BDU80EeMG
06IVIP1SHf<0Bd];0080CDe=03d0DU9B05QHF01`L700Lg=c05EEE@1]KFd0>c/k06a/K01NGUh0CDe=
07inOP1_Kfl0D51@055AD@16ATH0De=C05MGE`1VIVH0SHf=061PH02DU9@0C4a<055AD@0K6a/0EeMG
05iNGP1VIVH0T92@05UIF@1lO7`0V9RH08f=S@1eMGD0De=C07alO02=SHd0Ph>309jNWP1eMGD0QHF5
04i>CP1`L700W9bL05YJFP2UYJD0D51@09jNWP1LG5`0AdM7051@D01[Jf/0N7Qh075aL@1OGel0EEEE
04m?C`1PH600LG5a06m_K`2ATI40QHF5013ooooo000>ooooo`1@08j>SP28R8P0Zj^[05mOG`23Ph<0
LW9b07QhN02:RXX0<30`06UYJ@1lO7`0LG5a0:JVYP0_;bl0>SXj07IfMP1jNWX0XZ:R06][J`0c<c<0
I6AT0:FUY@1XJ6P0NGUi03Ph>00n?Sh0JFUY0:^[Z`1JFUX0Hf=S04];B`1LG5`0Ng]k05eMG@1>CTh0
IVIV05QHF026QXH0NGUi03dm?@15ADD0JVYZ04e=C@1OGel0JVYZ03Xj>P1HF5P0Fe]K03hn?P1nOWh0
Fe]K0451@@1:BTX0ATI6059BDP1cLg<0@D5105UIF@1YJFT0:b/[05mOG`2@T900BTY:05mOG`1HF5P0
@D5107UiN@2UYJD0>3Ph04e=C@1CDe<0>SXj08J6QP2GUiL0=cLg03dm?@1=CDd0@T9208^;R`1kNg/2
034a<@0=06][J`1PH600Q8B403hn?P0d=3@0CTi>06i^KP20P800UiNG03/k>`0f=SH0BTY:06UYJ@02
07AdM02402LW9`1:BTX0De=C06MWI`1LG5`0S8b<02h^;P1CDe<0NWYj04=3@`1[Jf/0T92@03Ph>01K
Fe/0SXj>05iNGP0b<S805aLG02<S8`1HF5P0O7al055AD@24Q8@0OGem0451@@1dM7@0SXj>05mOG`24
Q8@0Xj>S04M7A`1YJFT0/[:b051@D02BTY80JFUY03Xj>P1YJFT0QhN706=SH`1ZJVX0@d=305eMG@21
PH40Ph>306=SH`2=SHd0H61P06MWI`1lO7`0M7Ad07inOP2GUiL0De=C07QhN02UYJD0G5aL05EEE@2;
Rh/0C4a<07EeM@2BTY80ATI604a<C02GUiL0H61P08n?S`1[Jf/0B4Q804e=C@1_Kfl0I6AT0:VYZ@1K
Fe/0=3@d05mOG`1jNWX0GUiN0861P@0o?cl0DE5A0861P@1]KFd0D51@0::RXP1PH600DU9B08:2PP1n
OWh0JVYZ09NGU`11@D40Jf][096AT@1dM7@0Ph>305mOG`0f=SH0Jf][06QXJ01gMgL0WYjN0451@@0k
>c/0PH6106EUI@1/K6`0=CDe04a<C01^KVh0PX:209>CT`1HF5P0>3Ph06=SH`2<S8`0EEEE09ZJVP13
@d<0HF5Q0:VYZ@1UIFD0NWYj06EUI@1HF5P0ZZZZ071`L01TI6@0O7al04I6AP14A4@0YJFU0P1/K6`0
;`25QHD0?3`l05]KF`26QXH0?Cdm06QXJ01RHV80Cdm?061PH02XZ:P0KVi^08N7Q`1gMgL0BDU905AD
E033`l<0RXZ:05iNGP1RHV80ADE507QhN02TY:@0KFe]059BDP1ADE40C4a<07IfMP24Q8@0@T9203`l
?01NGUh0AdM7065QH@2:RXX0Bd];04E5A@1FEUH0GEeM07]kN`2GUiL0Bd];03Ti>@1^KVh0FUYJ069R
HP2d];@0J6QX00kooooo00D0@d=306][J`2@T900JFUY071`L00205]KF`2?07EeM@1HF5P0H61P06IV
IP1GEeL0@D5105UIF@1bLW80Fe]K055AD@1YJFT0Jf][05eMG@1CDe<0@41006][J`1;Bd/0ADE506a/
K00S8b<0@T9205YJFP1ZJVX0I6AT0451@@1XJ6P0EeMG07AdM01WIfL0IFEU059BDP1VIVH0?Shn02h^
;P17AdL0D51@05MGE`18B4P0N7Qh04m?C`1BDU80Hf=S06][J`1KFe/0T92@03dm?@12@T80HF5Q05eM
G@1LG5`0>3Ph05mOG`1=CDd0O7al03dm?@1ADE40BTY:06YZJP1KFe/0I6AT05IFEP2BTY80ADE5075a
L@1]KFd0RHV9061PH01?Cdl0N7Qh05EEE@1QHF40AdM706QXJ01lO7`0_[jn079bLP1QHF40FUYJ069R
HP1_Kfl0BDU905aLG01?Cdl0KFe]06=SH`0[:b/0O7al03Ti>@1HF5P0B4Q804i>CP1GEeL0G5aL05MG
E`0c<c<0BDU9051@D015ADD0JVYZ04Y:BP21PH40BTY:06MWI`1IFET0Ti>C05eMG@1cLg<0HF5Q07Ad
M00f=SH0F5QH06QXJ014A4@0HV9R0861P@1aLG40P82008>3P`2CTi<0FUYJ09nOW`21PH40De=C096A
T@1^KVh0V9RH05]KF`2_[jl0PH6105iNGP1ADE40G5aL02l_;`13@d<0EUIF04];B`13@d<0=CDe05AD
E00Q8B40Lg=c05mOG`1aLG400P1EEED05`16ATH0FEUI05QHF01PH600Lg=c07QhN03Hf=P0Rh^;04a<
C01aLG40HF5Q030`<01iNGT0DU9B04m?C`23Ph<0BTY:05mOG`1;Bd/0<c<c06a/K012@T80RHV90080
Cdm?06D08B4Q05IFEP0c<c<0UIFE04I6AP1ZJVX0?clo061PH01QHF40Hf=S07=cL`14A4@0NWYj0492
@P1ZJVX0Jf][05EEE@1[Jf/0GUiN03De=@1`L700EEEE07emO@1RHV80MWIf04a<C01fMWH0Fe]K02l_
;`1bLW80CDe=04A4A01>CTh0HF5Q05EEE@1]KFd0DE5A04Q8B01BDU80ATI606ATI01NGUh0Cdm?09:B
TP1[Jf/0MGEe02HV9P1QHF40FUYJ06ATI02:RXX0Jf][04a<C01QHF40IFEU06MWI`1CDe<0KFe]07Ad
M01MGEd0Ng]k05=CD`1IFET0@41005EEE@0g=cL0Hf=S07IfMP11@D40K6a/08J6QP1bLW80Kfm_05eM
G@1ZJVX0QHF50:ZZZP2a/K40EUIF08J6QP1_Kfl0R8R80:^[Z`1/K6`0][Jf05YJFP20P800CTi>05=C
D`1JFUX0Jf][05mOG`1kNg/0EEEE04i>CP1:BTX0KVi^06UYJ@1OGel0Lg=c07UiN@0@ooooo`003_oo
ool0o`3@d=00FUYJ06YZJP2?Shl0WIfM04];B`16ATH0ZZZZ092@T01ZJVX0@D5103Hf=P1UIFD0]KFe
03hn?P2>SXh0@41004E5A@2MWId0EeMG05]KF`1XJ6P0;bl_05YJFP1]KFd0AdM709:BTP0i>CT0EEEE
06UYJ@0k>c/0FUYJ08B4Q00O7al0@41008j>SP1UIFD0A4A405iNGP0f=SH0FUYJ09bLW01>CTh0=SHf
071`L010@400V9RH0:f][@0/;2`0>3Ph05eMG@2GUiL0Ti>C03@d=00U9BD0IVIV05ADE02MWId0MgMg
01dM7@0O7al0DU9B05UIF@2FUYH0EEEE02d];@0^;Rh0JVYZ06IVIP2;Rh/0B4Q803Ti>@12@T80EUIF
06e]K@1kNg/0Hf=S04a<C01:BTX0WYjN07MgM`1nOWh0GemO04=3@`1HF5P0PX:205=CD`2XZ:P0Bd];
03Hf=P1hN7P0Xj>S05QHF02LW9`0>c/k03Ph>02CTi<0FUYJ07IfMP24Q8@0@D5103Ph>02HV9P0K6a/
079bLP1:BTX0:b/[03Ph>01GEeL0HF5Q0;no_`1FEUH0;Bd]02h^;P17AdL0DU9B069RHP1cLg<0<S8b
07MgM`2YZJT0Fe]K05UIF@1MGEd0EeMG07emO@2`/;00K6a/03hn?P18B4P0De=C08j>SP1mOGd0>3Ph
04M7A`1GEeL0MGEe09RHV014A4@0>3Ph065QH@0e=CD0NWYj0:RXZ01YJFT0>SXj059BDP1KFe/0QHF5
0:b/[00b<S80C4a<03Xj>P1NGUh0IfMW0:2PX00m?Cd0EUIF05=CD`1nOWh0GEeM0<g=c@11@D40EUIF
05EEE@1eMGD0`l?30;jn_P17AdL0HF5Q05QHF01UIFD0_;bl0:>SX`0X:2P0L71`04m?C`2GUiL0`l?3
04Q8B016ATH0GemO09fMW@2c/k<0VIVI02h^;P1CDe<0Lg=c0820P02VYZH0CDe=0410@01nOWh0_;bl
06][J`1oOgl0@D5106QXJ02PX:00alO706QXJ01VIVH0GemO0;Zj^P20P800KVi^06IVIP0h>3P0T92@
0:^[Z`1[Jf/0_;bl03dm?@19BDT0YZJV08R8R025QHD0LW9b05ADE01dM7@0[Jf]06MWI`2=SHd0DE5A
05aLG02`/;00QHF504i>CP2@T900DU9B05mOG`34a<@0OGem04m?C`1WIfL0?3`l07EeM@23Ph<0=SHf
0451@@1MGEd0A4A407YjNP2HV9P0:RXZ034a<@1ZJVX0MWIf06MWI`2JVYX0<S8b05QHF01ZJVX0PH61
09NGU`001`1NGUh0>3Ph05=CD`1QHF40De=C0820P01]KFd00P0g=cL03P1BDU80De=C05EEE@1mOGd0
>3Ph03De=@1YJFT0SXj>05IFEP2?Shl0?Cdm02h^;P1]KFd0Vi^K3_ooool0Z00j>SX0JVYZ05QHF01>
CTh0JFUY04=3@`17AdL0R8R804=3@`1JFUX0IVIV04e=C@16ATH0K6a/06e]K@1_Kfl0Hf=S05YJFP14
A4@0?3`l0492@P0_;bl0D51@03dm?@0h>3P0C4a<02LW9`0c<c<0AdM70410@01?Cdl0>CTi05MGE`0l
?3`0D51@04m?C`1KFe/0@41004e=C@0b<S80ATI604M7A`1@D500EEEE07EeM@1]KFd0<30`04E5A@1m
OGd0EUIF04I6AP1PH600BDU9061PH01KFe/0=cLg05aLG01;Bd/0G5aL04e=C@1PH600@D5105UIF@18
B4P0DU9B05mOG`2ATI40Q8B40;Vi^@16ATH0Ph>308F5Q@2PX:00Ng]k069RHP26QXH0DE5A079bLP1=
CDd0Q8B4075aL@2BTY80JFUY05=CD`1IFET0QXJ606e]K@1DE5@0HV9R05QHF01fMWH0IFEU02`/;02I
VIT0CDe=05UIF@1:BTX0D51@06UYJ@1LG5`0DU9B04e=C@1;Bd/0D51@0492@P1KFe/0BDU908Z:RP1^
KVh0PH6105iNGP2TY:@0Kfm_08j>SP1`L700Q8B404Y:BP1?Cdl0R8R805IFEP1nOWh0QHF50820P01M
GEd0Shn?0=;BdP1dM7@0^k^k08f=S@1?Cdl0Lg=c05YJFP2>SXh0?Shn09JFUP1bLW80I6AT065QH@1C
De<0?3`l05MGE`2EUID0J6QX04];B`18B4P0H61P03De=@23Ph<0HV9R07emO@1]KFd0@41003`l?01>
CTh0D51@06ATI01YJFT0GEeM0;2`/01SHf<0?clo05]KF`1CDe<0;Bd]05mOG`1DE5@0BTY:06IVIP80
Bd];04@0=3@d03hn?P1kNg/0EUIF05UIF@13@d<0A4A403De=@1eMGD0>c/k08F5Q@1FEUH0Jf][03/k
>`1>CTh0C4a<04m?C`1FEUH0B4Q807MgM`0n?Sh0FEUI05aLG01FEUH0Jf][04Y:BP0m?Cd0Jf][04M7
A`1EEED0CTi>04];B`0b<S80FEUI04Y:BP0I6AT0CTi>03@d=00_;bl0?clo05QHF016ATH0De=C05mO
G`1<C4`0CDe=04=3@`1:BTX0BDU906=SH`1oOgl0EEEE04e=C@14A4@0S8b<05ADE01<C4`0HF5Q05]K
F`1IFET0D51@04];B`1bLW80QHF508N7Q`2BTY80Ogmo08R8R080MWIf02H0J6QX061PH01KFe/0Ng]k
07YjNP0Z:RX0PX:20:JVYP2JVYX0PH6108N7Q`1aLG40Y:BT07MgM`2VYZH0OWin0;6a/@2<S8`0Ng]k
08f=S@1mOGd0Ph>304I6AP2QXJ40DU9B04Q8B017AdL0GUiN061PH02<S8`0E5AD04=3@`1>CTh0Q8B4
06QXJ01CDe<0HF5Q07YjNQ3ooooo000>ooooo`0B0;:b/P1PH600JVYZ061PH02`/;00FUYJ03/k>`1<
C4`0Jf][09VIV@0k>c/0<C4a04=3@`1]KFd0B4Q80;:b/P1<C4`0EUIF0P1KFe/0o`2FUYH0TY:B02`/
;01>CTh0HF5Q06i^KP3:b/X0G5aL02h^;P1CDe<0AdM70:BTY038b<P0:RXZ0451@@1BDU80GEeM0:^[
Z`2][Jd06a/K03dm?@1^KVh0U9BD07inOP1aLG40<C4a03hn?P1fMWH0PX:208Z:RP0a<C40;Rh^08F5
Q@2CTi<0Hf=S07moO`0f=SH0CDe=08^;R`1IFET0Cdm?061PH00U9BD0Bd];07=cL`18B4P0NGUi079b
LP0C4a<0EEEE05IFEP0/;2`0GUiN06IVIP0`<300PH6107UiN@0j>SX0EUIF03De=@1ADE40GemO07Qh
N00^;Rh0CDe=03Ti>@18B4P0Ng]k0:f][@19BDT0[jn_07QhN01TI6@0][Jf08Z:RP0e=CD0EeMG04i>
CP1lO7`0]KFe04I6AP0e=CD0FUYJ05UIF@20P800RXZ:02XZ:P0m?Cd0ATI604i>CP32`/80KVi^02HV
9P11@D40DU9B069RHP2f][H0MGEe0410@01<C4`0J6QX06m_K`2BTY80SHf=03@d=01]KFd0R8R806QX
J025QHD0QhN703`l?02@T900[jn_06EUI@2JVYX0HF5Q06][J`2PX:00MGEe071`L02c/k<0NGUi059B
DP25QHD0[:b/08:2PP2MWId092@T079bLP2g]kL0R8R80:ZZZP1QHF40?Shn04Q8B02YZJT0LG5a08R8
R01MGEd07alO04i>CP2][Jd0KVi^0;6a/@1QHF40>CTi05=CD`33`l<0Ogmo08J6QP13@d<0?clo05UI
F@2[Zj/0TI6A06YZJP0[:b/0Bd];06UYJ@1_Kfl0De=C08R8R013@d<0X:2P0;fm_@1TI6@0C4a<08J6
QP10@400^KVi0:j^[P0k>c/0F5QH0492@P1NGUh0Q8B404];B`1>CTh0Lg=c04i>CP2ATI40^;Rh03Lg
=`1MGEd0KFe]079bLP1oOgl0<S8b05YJFP1PH600J6QX0>GUi@1bLW808b<S06][J`1lO7`0T92@06QX
J00[:b/0A4A406e]K@26QXH0^[Zj04U9B@0j>SX0F5QH09JFUP1]KFd0b/[:04U9B@15ADD0Hf=S065Q
H@2FUYH0Rh^;04I6AP1NGUh0MGEe05MGE`2JVYX0XZ:R03dm?@1>CTh0F5QH06YZJP1aLG40]KFe03/k
>`1<C4`0Vi^K06][J`2KVi/0V9RH02h^;P1UIFD0MWIf07QhN02TY:@0XJ6Q04A4A011@D40GemO04=3
@`1QHF40Hf=S03<c<`13@d<0De=C05aLG00i>CT0AdM703Lg=`1JFUX0M7Ad06MWI`000`14A4@0Jf][
04i>CP0>ooooo`1=0451@@1`L700Rh^;05iNGP2FUYH0GEeM05YJFP2LW9`0D51@05eMG@1PH600JVYZ
05UIF@1MGEd0K6a/071`L01cLg<0M7Ad07=cL`1FEUH0De=C02HV9P1HF5P0?clo03/k>`1TI6@06a/K
04a<C01_Kfl0PH61065QH@11@D40J6QX04e=C@1hN7P0JFUY06e]K@1DE5@0DU9B028R8P0_;bl0@D51
03Ti>@1:BTX0>SXj06YZJP14A4@0C4a<04m?C`1;Bd/0Cdm?06YZJP0a<C40@d=306ATI01<C4`0EUIF
03dm?@1ADE40@D5105QHF01:BTX0JFUY04Q8B01KFe/0EUIF04m?C`19BDT0IfMW06IVIP1SHf<0JVYZ
05MGE`13@d<0BDU904M7A`1ADE400P1GEeL03`1VIVH0Ogmo0:^[Z`1ZJVX0F5QH04];B`1GEeL0HF5Q
0451@@1;Bd/0GemO05QHF01BDU80GEeM08N7Q`0204a<C01805aLG01GEeL0DE5A05ADE01TI6@0Bd];
059BDP1JFUX0FEUI06a/K01dM7@0TY:B04M7A`1PH600TI6A0;2`/01lO7`0T92@09FEU@2@T900G5aL
06QXJ02EUID0Ng]k08F5Q@22PX80NWYj0820P02ZZZX0Xj>S04Y:BP1]KFd0Ng]k06i^KP20P800OGem
0:NWY`1jNWX0aLG509VIV@2BTY80E5AD06UYJ@1MGEd0IFEU05YJFP1CDe<0EUIF03hn?P1>CTh07QhN
08j>SP1UIFD0QHF506i^KP1aLG40Cdm?055AD@1QHF40KFe]06IVIP24Q8@0c<c<07inOP1=CDd0RHV9
07MgM`15ADD0KVi^08R8R01IFET0Shn?0P1>CTh0K01EEED0GEeM08N7Q`18B4P0PX:2059BDP1ADE40
;Bd]04e=C@0a<C40RXZ:034a<@1OGel0?Cdm08f=S@1cLg<0HV9R06i^KP1MGEd0Ogmo0451@@1LG5`0
IVIV04m?C`26QXH0IfMW03De=@1]KFd0DE5A06MWI`0n?Sh0JFUY03Ph>01JFUX0DU9B028R8P1NGUh0
=cLg03@d=01:BTX0KVi^055AD@1HF5P0FEUI059BDP1ADE40ATI606ATI019BDT0DU9B07AdM01DE5@0
JFUY03Ti>@1hN7P0F5QH05EEE@1[Jf/0LG5a04E5A@1EEED0F5QH08>3P`0j>SX0E5AD06QXJ02;Rh/0
P82004a<C01CDe<0Cdm?07emO@19BDT0H61P06e]K@18B4P0OGem08b<S02=SHd0F5QH06][J`1dM7@0
ZJVY07emO@2h^;P0NGUi0;Ng]`2SXj<0`L710;bl_01KFe/0RXZ:0451@@1[Jf/0GEeM079bLP1LG5`0
De=C059BDP1PH600@T9204Y:BP1hN7P0PX:2065QH@1DE5@0Jf][07EeMA3ooooo000>ooooo`3N08f=
S@29RHT0/k>c05]KF`2ATI40/k>c0861P@2=SHd0>3Ph07AdM02a/K40NGUi0;Bd]00d=3@0@41007=c
L`29RHT0/K6a03hn?P12@T80P82006a/K01SHf<0QhN702l_;`1FEUH0O7al09FEU@13@d<0D51@03Lg
=`1QHF40OGem065QH@0g=cL0EUIF03dm?@1WIfL0Q8B405]KF`0h>3P0ATI6024Q8@1oOgl0Zj^[034a
<@0j>SX0@d=303<c<`2LW9`0H61P03Lg=`18B4P0<c<c03Lg=`1fMWH0BDU903<c<`1@D500?Cdm08V9
R@2m_Kd0:BTY04E5A@1FEUH0@T9209^KV`2g]kL07alO04Q8B01UIFD0?Cdm08Z:RP2DU9@09RHV02PX
:02:RXX0IVIV0;Vi^@1cLg<0;Bd]03@d=02LW9`0QXJ609jNWP0`<300BTY:05IFEP26QXH0QHF50<O7
a`0j>SX0EUIF059BDP2MWId0K6a/07emO@0Q8B40EUIF08F5Q@2?Shl0IVIV079bLP13@d<0EeMG06=S
H`1DE5@0IFEU071`L00n?Sh0Bd];07YjNP1LG5`0P82007alO00g=cL0DU9B08^;R`13@d<0Lg=c06i^
KP0a<C40I6AT0;fm_@2<S8`0QXJ6096AT@13@d<0HV9R0:j^[P1IFET0QHF504E5A@0j>SX0N7Qh07in
OP1RHV80MGEe038b<P1=CDd0G5aL09NGU`1=CDd0QHF503Ph>02ATI40[jn_07inOP1FEUH0HF5Q04m?
C`2g]kL0U9BD02h^;P0g=cL0MGEe04Y:BP2GUiL0Rh^;024Q8@17AdL0LG5a06IVIP2n_[h0Ng]k024Q
8@14A4@0HF5Q05IFEP2o_kl0?Cdm02PX:01MGEd0J6QX05MGE`2f][H09BDU04Q8B01kNg/0GEeM06e]
K@3>c/h0B4Q805iNGP2_[jl0NGUi06a/K032`/80=cLg08F5Q@2g]kL0IFEU08F5Q@1>CTh0<c<c05iN
GP1aLG40TI6A07emO@0h>3P0Cdm?09ZJVP1SHf<0VYZJ03/k>`1ADE40Vi^K07moO`1YJFT0Fe]K04Y:
BP1SHf<0/;2`0820P02b/[80Fe]K08:2PP28R8P0DE5A06ATI01<C4`0Cdm?0P2QXJ40=P1YJFT0PX:2
02l_;`10@400L71`08B4Q01kNg/0WIfM03De=@11@D40[Jf]05MGE`1^KVh0De=C030`<014A4@0WYjN
05eMG@1YJFT0LW9b04I6AP18B4P0X:2P08>3P`1:BTX0BDU903Lg=`1mOGd0^k^k05QHF01OGel0EEEE
05IFEP2FUYH0clo?05UIF@10@400H61P04E5A@1mOGd0^[Zj0451@@0h>3P0HF5Q05YJFP29RHT0alO7
03Ph>012@T80De=C06EUI@1aLG40clo?059BDPkooooo04L0BDU9079bLP2?Shl0Ph>307QhN01CDe<0
G5aL06ATI019BDT0GEeM07EeM@1BDU80ADE506][J`28R8P0MgMg061PH01RHV80KVi^05EEE@1<C4`0
:2PX06=SH`1DE5@0CDe=075aL@0U9BD0@T9205UIF@1cLg<0Cdm?03dm?@1@D500BDU9051@D01PH600
Lg=c04a<C01NGUh0BDU903`l?017AdL0CDe=065QH@15ADD0KFe]04M7A`1:BTX0IfMW04m?C`1WIfL0
Ng]k03Hf=P1=CDd0Jf][05ADE01QHF40@T9207AdM01;Bd/0NWYj03/k>`1DE5@0Bd];055AD@1JFUX0
FEUI05EEE@2SXj<0@d=307IfMP0208R8R00M04];B`19BDT0@D5105YJFP1EEED0ADE507emO@29RHT0
/K6a079bLP1SHf<0DE5A07]kN`2<S8`0@T9206YZJP1FEUH0LW9b06=SH`1GEeL0Ogmo0451@@1CDe<0
?clo05EEE@1FEUH0Lg=c07AdM01?Cdl00P1RHV803P16ATH0NWYj05EEE@1aLG40DE5A06ATI01IFET0
KVi^05aLG01HF5P0Jf][08>3P`1ADE40BDU90P1dM7@04@2@T900ZJVY065QH@1HF5P0U9BD09JFUP1C
De<0aLG508V9R@19BDT0Jf][069RHP2<S8`0Bd];071`L01JFUX0GUiN0080BDU908/0De=C04e=C@1`
L700K6a/06EUI@1@D500HV9R03@d=021PH40M7Ad08^;R`1/K6`0Ogmo065QH@25QHD0L71`07=cL`2O
Wil0/K6a0;Vi^@1eMGD0GemO07UiN@1lO7`0FEUI09VIV@22PX80GEeM08R8R01?Cdl0De=C03dm?@12
@T80SXj>05eMG@1eMGD0Hf=S071`L00b<S80EEEE055AD@2c/k<0DU9B06IVIP15ADD0BTY:05mOG`1Y
JFT0MgMg05QHF027QhL0AdM706ATI01ADE40?3`l08B4Q01^KVh0F5QH06YZJP1WIfL0OGem04Q8B01R
HV80DE5A08^;R`1aLG40BDU906=SH`1ADE40>c/k05EEE@1WIfL0DU9B079bLP1^KVh0IFEU06a/K01:
BTX0IfMW051@D01RHV80K6a/07MgM`1nOWh0@T92061PH01<C4`0HF5Q06YZJP1@D500>SXj051@D01A
DE40IfMW06=SH`28R8P0S8b<05MGE`1=CDd0CTi>06i^KP11@D40=SHf02d];@2LW9`0Ti>C04a<C021
PH40PX:2069RHP1_Kfl0OGem08V9R@2;Rh/0JVYZ0;6a/@1[Jf/0Zj^[08b<S01jNWX0SXj>04];B`2`
/;00MgMg07AdM01:BTX0C4a<05eMG@1hN7P0MGEe06e]K@1UIFD0Lg=c06][J`1nOWh0KFe]05mOG`1W
IfL0NGUi013ooooo000>ooooo`1R0820P01HF5P0IFEU06UYJ@1JFUX0DE5A04];B`25QHD0>3Ph03hn
?P1RHV80MgMg09:BTP10@400;Bd]03lo?`1ZJVX0T92@05YJFP0K6a/0B4Q805ADE00j>SX0^KVi02h^
;P0P8200HV9R07emO@1aLG40U9BD01PH601FEUH0MGEe06IVIP24Q8@0_[jn04I6AP1aLG40WIfM08B4
Q02BTY80/K6a03Lg=`0m?Cd0cLg=05=CD`1oOgl0Jf][01lO7`1[Jf/0alO7065QH@20P8005ADE02/[
:`14A4@0Hf=S05aLG01[Jf/0;bl_03@d=01^KVh0HF5Q0820P014A4@0;Bd]04I6AP1kNg/0MGEe061P
H00f=SH092@T03De=@1_Kfl0HV9R06YZJP1NGUh0ATI604m?C`1_Kfl0Bd];07AdM01>CTh0MGEe0::R
XP2CTi<0>SXj06QXJ01:BTX0D51@0:FUY@20P800ADE505EEE@11@D40F5QH0;2`/01hN7P204M7A`1A
04I6AP1bLW80Q8B402d];@1<C4`0Cdm?05ADE01gMgL0_[jn0451@@1GEeL0De=C04E5A@2=SHd0TI6A
03Ti>@1JFUX0MgMg06UYJ@2;Rh/0/[:b04A4A01:BTX0MGEe06=SH`2FUYH0alO703dm?@0f=SH0Q8B4
06e]K@3:b/X0BTY:02LW9`0f=SH0QHF509VIV@27QhL0:BTY04=3@`1;Bd/0NWYj07MgM`34a<@0?Cdm
051@D01nOWh0Ph>30;Fe]@2MWId07AdM075aL@2l_;`0Ng]k0820P02][Jd0;bl_07AdM02/[:`0De=C
07EeM@2BTY80?clo07]kN`3Fe]H0QXJ606m_K`2KVi/0AdM707]kN`2FUYH0CDe=09RHV01jNWX08B4Q
07inOP1QHF40GEeM06m_K`0];Bd0AdM70080M7Ad05l0Cdm?06IVIP0i>CT0Jf][0::RXP1MGEd0QHF5
05UIF@14A4@0ZJVY0:^[Z`1<C4`0Ogmo02h^;P1iNGT0bLW906ATI01=CDd0J6QX06i^KP3;bl/0FUYJ
03/k>`17AdL0G5aL0:b/[02>SXh0CTi>07MgM`0e=CD0Cdm?0<_;b`1_Kfl0J6QX08^;R`1`L700NGUi
07IfMP0f=SH0Lg=c05IFEP1`L700f][J04Y:BP0e=CD0EeMG06EUI@2IVIT0ZZZZ03@d=019BDT0QXJ6
06EUI@37alL0RHV903Ph>016ATH0Hf=S07alO02EUID0XZ:R03hn?P1PH600OWin07moO`2>SXh0Ph>3
030`<01<C4`0X:2P08V9R@26QXH0@d=305aLG02?Shl0TI6A08R8R01fMWH0Cdm?05]KF`1MGEd0F5QH
07AdM01=CDd0JFUY03hn?P1UIFD0Lg=c06IVIP0m?Cd0L71`030`<018B4P0Ng]k09FEU@0>ooooo`2H
04i>CP1VIVH0LW9b059BDP1?Cdl0De=C05MGE`1WIfL0CDe=04Y:BP18B4P0EUIF05UIF@1:BTX0Cdm?
05ADE02ATI40FEUI061PH01FEUH0Bd];03Lg=`1EEED0@41004A4A01UIFD0=3@d04m?C`1OGel0GEeM
04m?C`17AdL0B4Q8059BDP0m?Cd0BDU904i>CP1FEUH0ADE503De=@0b<S8092@T038b<P1=CDd0BTY:
06][J`1=CDd0HF5Q08n?S`1GEeL0@T9206YZJP0c<c<0A4A4055AD@0o?cl0ATI60451@@1SHf<0EeMG
05iNGP0a<C40F5QH05UIF@1@D500GEeM07AdM01TI6@0QHF504U9B@1iNGT0LG5a0820P01=CDd0EUIF
04];B`1@D500C4a<04i>CP1=CDd0KFe]0:>SX`1TI6@0C4a<04i>CP1SHf<0GUiN04e=C@1FEUH0CTi>
06ATI01YJFT0EUIF08J6QP1:BTX0CTi>04A4A01EEED0Hf=S069RHP1hN7P0Ng]k05eMG@1WIfL0NWYj
06IVIP1FEUH0TY:B079bLP1jNWX0J6QX092@T02CTi<0WYjN09VIV@1kNg/0Bd];04Y:BP1OGel0RXZ:
06m_K`1EEED0O7al05UIF@24Q8@0UYJF04=3@`24Q8@0Lg=c06IVIP2d];@0OWin09fMW@2:RXX0^[Zj
07moO`1SHf<0AdM705UIF@1kNg/0GemO051@D01^KVh0Fe]K03De=@1CDe<06QXJ0:RXZ01:BTX0G5aL
09VIV@1jNWX204e=C@1l04M7A`29RHT0Shn?09VIV@3Kfm/0R8R804I6AP1/K6`0M7Ad02DU9@1VIVH0
E5AD055AD@29RHT0@d=305UIF@0d=3@0>CTi08F5Q@0f=SH0Ogmo0492@P1NGUh0B4Q806MWI`0l?3`0
M7Ad02h^;P1LG5`0DE5A04a<C01NGUh0JVYZ08j>SP0o?cl0MGEe05YJFP2=SHd0F5QH03hn?P1`L700
IVIV03<c<`1]KFd0M7Ad08B4Q01FEUH0Hf=S03De=@1BDU80DE5A03Lg=`1XJ6P0?3`l05UIF@1VIVH0
JFUY06IVIP1[Jf/0FUYJ06MWI`1DE5@0?clo06IVIP1HF5P0HF5Q08>3P`1@D500EeMG028R8P1jNWX0
C4a<059BDP1VIVH0JFUY05MGE`1?Cdl0EeMG08J6QP1WIfL0K6a/06EUI@1aLG40GUiN05YJFP1dM7@0
EEEE05iNGP1FEUH0IFEU05EEE@0g=cL0QHF507inOP1fMWH0NWYj07alO01VIVH0W9bL09ZJVP2`/;00
IfMW08b<S01aLG40fMWI0;Ng]`2KVi/0];Bd04];B`2XZ:P0F5QH07MgM`1PH600Fe]K07IfMP1VIVH0
D51@04i>CP1:BTX0SHf=071`L01eMGD0JFUY08f=SA3ooooo000>ooooo`180;6a/@1DE5@0EUIF08^;
R`2[Zj/0CDe=05UIF@1RHV80LW9b08j>SP0W9bL0BDU907MgM`20P800MGEe0;Fe]@0d=3@0N7Qh08j>
SP1VIVH0OWin08V9R@0Q8B40JFUY07MgM`1@D500MgMg02TY:@1:BTX0IfMW03Hf=P1ADE40IfMW01LG
5`0N7Qh0H61P05MGE`1;Bd/0Jf][02h^;P0j>SX0G5aL06a/K00n?Sh0@d=304a<C01?Cdl0Ng]k038b
<P0o?cl0<C4a071`L02>SXh0AdM7024Q8@15ADD0=SHf07YjNP2DU9@0;2`/02l_;`0l?3`0=cLg092@
T01/K6`08B4Q0410@019BDT0EEEE07alO01iNGT08b<S0P12@T80I@1EEED0OGem09>CT`0P8200DE5A
06EUI@1<C4`0PX:207inOP0V9RH0EUIF09:BTP1KFe/0Xj>S07AdM00e=CD0Ph>30::RXP2SXj<0ZJVY
03Ph>017AdL0T92@08:2PP2IVIT0[jn_02@T901>CTh0YjNW061PH02VYZH0Rh^;028R8P1;Bd/0[Zj^
07]kN`2UYJD0?Cdm03Xj>P1NGUh0W9bL09JFUP26QXH0^k^k04=3@`1TI6@0IVIV0820P02FUYH0CTi>
04A4A01KFe/0QhN708>3P`1dM7@0BDU905aLG02YZJT0XJ6Q06IVIP27QhL0@d=306IVIP2NWYh0JFUY
051@D01WIfL0@d=304i>CP2TY:@0QhN705aLG01nOWh0;2`/07moO`3De=@0IFEU04a<C01JFUX0BDU9
05MGE`2m_Kd0ADE505QHF01eMGD0ADE5071`L037alL0>3Ph05mOG`1RHV80>c/k09ZJVP3Lg=`0@D51
06][J`1WIfL0>CTi09FEU@3:b/X0?3`l0080EEEE06D0PH610=7Ad@0`<300<S8b04A4A01RHV80YZJV
0>?Sh`0Z:RX0;Rh^08F5Q@1lO7`0gmoO05MGE`1@D500DE5A092@T01PH600UiNG03lo?`1MGEd0NWYj
07EeM@2=SHd0NWYj05UIF@2WYjL0NGUi08n?S`1mOGd0=SHf06a/K02>SXh0HV9R0:6QX@0g=cL0?clo
05UIF@29RHT0QHF506=SH`0j>SX0Jf][09BDU01MGEd0Lg=c04a<C00o?cl0EeMG08f=S@1>CTh0SXj>
02LW9`13@d<0S8b<059BDP1@D500G5aL0451@@1YJFT0Ogmo04e=C@1EEED0UiNG04E5A@1KFe/0SXj>
05YJFP1HF5P0TY:B03Hf=P1ZJVX0^;Rh055AD@1]KFd0PH6105IFEP1^KVh0HV9R03Ph>01[Jf/0KFe]
05]KF`1cLg<0KFe]03Xj>P1@D500G5aL06MWI`1]KFd0WYjN05]KF`0f=SH0KFe]05aLG01QHF40X:2P
03Xj>P0X:2P0EUIF08j>SP0>ooooo`0303hn?P1SHf<0QhN70080QHF50P1;Bd/0;`1MGEd0BDU905YJ
FP1JFUX0Cdm?04M7A`1PH600QXJ606QXJ01:BTX0CTi>05ADE01:BTX0E5AD04=3@`1JFUX0De=C04e=
C@1KFe/0BTY:0492@P1[Jf/0EEEE04Y:BP0h>3P0B4Q804=3@`1IFET0GUiN06i^KP0o?cl0?Cdm030`
<00W9bL07QhN03Lg=`1=CDd0@T92055AD@0m?Cd0>CTi04];B`0m?Cd0KVi^05MGE`14A4@0CDe=0080
B4Q801/0EEEE05ADE01MGEd0BDU905UIF@1@D500Fe]K04];B`1HF5P0IVIV05UIF@1HF5P0IfMW05YJ
FP1jNWX0Ph>307inOP1MGEd0EEEE05UIF@1GEeL0GEeM06MWI`1VIVH0JFUY08^;R`1NGUh00P1;Bd/0
101JFUX0IfMW03lo?`1KFe/203lo?`2k05IFEP10@400IVIV0451@@1>CTh0>CTi051@D01FEUH0FUYJ
04e=C@15ADD0@T9203`l?00^;Rh0FUYJ07emO@2QXJ40EeMG059BDP1LG5`0FEUI03`l?01OGel0CTi>
05YJFP11@D40J6QX08N7Q`1`L700UYJF0:BTY01[Jf/0O7al0;Rh^02m_Kd0K6a/09RHV01`L700FUYJ
08F5Q@1hN7P0RXZ:08J6QP28R8P0S8b<04i>CP1cLg<0HV9R03dm?@0g=cL0GEeM04i>CP1EEED0ADE5
0492@P0M7Ad0LW9b04Q8B012@T80CDe=04Y:BP1<C4`0G5aL03`l?01?Cdl0IVIV071`L02GUiL0IFEU
092@T02;Rh/0De=C04U9B@2QXJ40EUIF04U9B@1MGEd0CTi>07YjNP1?Cdl0IVIV08B4Q01?Cdl0UYJF
04e=C@1CDe<0=3@d059BDP1TI6@0_[jn059BDP1VIVH0:RXZ04e=C@1FEUH0FUYJ06i^KP1NGUh0NWYj
059BDP23Ph<0GEeM05]KF`1dM7@0GUiN02/[:`28R8P0De=C06e]K@0^;Rh0E5AD04=3@`1QHF40BTY:
04U9B@26QXH0FEUI03/k>`19BDT0L71`04a<C01VIVH0Fe]K05YJFP1FEUH0CTi>079bLP1YJFT0IVIV
08>3P`1PH600HV9R04M7A`25QHD0KFe]06UYJ@1?Cdl0K6a/05]KF`1ADE40L71`06MWI`1LG5`0RXZ:
06EUI@1<C4`0I6AT06][J`1VIVH0Bd];0492@P1HF5P0U9BD07EeM@0j>SX0OWin06MWI`24Q8@0G5aL
05UIF@13@d<0JVYZ061PH02PX:00?Shn0:j^[P2HV9P0WinO0:BTY01cLg<0Vi^K05IFEP23Ph<0<30`
04M7A`0k>c/0OGem07AdM026QXH0C4a<07=cL`1IFET0OWin07QhN01KFe/0Kfm_069RHP0@ooooo`00
3_ooool0JP2YZJT0KVi^07QhN01KFe/0OGem07MgM`1WIfL0MgMg03hn?P1QHF40Cdm?03hn?P25QHD0
C4a<04U9B@21PH40Hf=S06i^KP1CDe<0>SXj08:2PP2_[jl0@D5104Y:BP16ATH0PH610:2PX00h>3P0
7alO04a<C010@400S8b<0:b/[00Q8B407AdM03`l?01hN7P0NWYj09^KV`0O7al0@D5104U9B@1XJ6P0
QhN707alO00I6AT0F5QH075aL@14A4@0O7al03Lg=`0S8b<0H61P06m_K`1MGEd0QXJ603De=@0o?cl0
QHF5065QH@24Q8@0WIfM038b<P1SHf<0ZZZZ05eMG@25QHD0PX:202/[:`1FEUH0Z:RX05aLG01IFET0
EUIF02<S8`13@d<0T92@04Q8B01lO7`0@T9202d];@1:BTX0TI6A055AD@2DU9@0>SXj03lo?`1IFET0
SHf=061PH02=SHd0<c<c03lo?`1gMgL0I6AT04Q8B01BDU80>3Ph051@D01kNg/0Cdm?02PX:01ADE40
>SXj06IVIP2SXj<203<c<`0G04a<C01BDU80Q8B406IVIP0W9bL0?Cdm04E5A@1ADE40JVYZ092@T00Y
:BT0>c/k05=CD`15ADD0RXZ:08F5Q@0U9BD0Hf=S06][J`1UIFD0QXJ608:2PP0g=cL00P1QHF40T@1^
KVh0XJ6Q04Y:BP10@400K6a/08:2PP1nOWh0]kNg0492@P1<C4`0T92@07AdM01oOgl0`<3003Xj>P1S
Hf<0];Bd07inOP2i^KT0Ph>303De=@1=CDd0[Zj^06][J`2XZ:P0G5aL04M7A`1MGEd0Ph>307inOP2c
/k<0?clo03`l?014A4@0_Kfm0820P027QhL0=CDe038b<P1XJ6P0f=SH0;no_`21PH40CDe=05QHF01`
L700NWYj06i^KP2YZJT0@T9204m?C`2QXJ40WIfM061PH01VIVH0@4100:BTY03;bl/0H61P07]kN`0h
>3P0AdM70:BTY01VIVH0MWIf06m_K`12@T80D51@07]kN`1<C4`0KFe]05MGE`1]KFd0O7al04=3@`11
@D40E5AD07AdM02h^;P0HV9R02LW9`1EEED0KVi^0;Fe]@2BTY80;2`/03@d=010@400O7al0:JVYP1]
KFd0?3`l05MGE`1jNWX0F5QH0:j^[P1>CTh0?Shn06e]K@11@D40EeMG08J6QP0P8200=cLg06IVIP1I
FET0I6AT0;Jf]P0`<300ADE507AdM01cLg<0NGUi0=_Kf`1GEeL0AdM70:6QX@26QXH0S8b<0:6QX@11
@D40Bd];08:2PP1MGEd0V9RH08>3P`0d=3@0GUiN07=cL`0l?3`0Bd];04A4A00W9bL0@T92071`L01f
MWH0BDU9061PH00U9BD0@d=3065QH@1cLg<0FUYJ075aL@13@d<03_ooool0X@0m?Cd0QhN707emO@11
@D40E5AD04Y:BP1CDe<0GUiN03Xj>P1TI6@0IFEU069RHP1NGUh0L71`07emO@1cLg<0Hf=S06QXJ01^
KVh0DE5A04Q8B00^;Rh0EEEE04A4A01=CDd0JVYZ03dm?@0j>SX0JFUY07YjNP1QHF40>c/k05mOG`1J
FUX0Fe]K03dm?@15ADD0B4Q805=CD`0f=SH0<30`038b<P13@d<0EUIF03/k>`1CDe<0AdM704Q8B01S
Hf<0ATI603Lg=`1;Bd/0>3Ph04I6AP1BDU80GEeM05EEE@0[:b/0C4a<04e=C@19BDT0:BTY06=SH`1P
H600>c/k04a<C01DE5@0JVYZ07QhN00m?Cd0Ng]k0:f][@21PH40LW9b04E5A@0m?Cd0=SHf04Q8B013
@d<0Bd];0:RXZ03AdM40D51@04I6AP0m?Cd0?clo04Q8B017AdL0A4A4030`<00h>3P0@T9203`l?01;
Bd/0>c/k04e=C@11@D40BDU906][J`19BDT0D51@04];B`0e=CD0>c/k03Lg=`19BDT0ATI604a<C00j
>SX0HF5Q05YJFP1cLg<0DE5A09JFUP1XJ6P0N7Qh03hn?P0o?cl0GemO05QHF01VIVH0EEEE05QHF01P
H600HV9R09^KV`1=CDd0X:2P08V9R@1[Jf/0K6a/06m_K`2ZZZX0Q8B40:JVYP2ATI40S8b<061PH019
BDT0;Bd]05MGE`2LW9`0GUiN06MWI`1UIFD0EeMG02l_;`2PX:00H61P06EUI@1OGel0I6AT05iNGP2;
Rh/0Fe]K05MGE`1QHF40Rh^;0=gMg@1WIfL0?Shn0080Cdm?07<0>CTi06e]K@1WIfL0CDe=069RHP12
@T80BDU903/k>`19BDT0HV9R05EEE@2g]kL0MWIf06ATI016ATH0C4a<05=CD`1`L700<c<c05MGE`0k
>c/0KFe]07=cL`2PX:00Vi^K04U9B@1LG5`0HV9R08V9R@1jNWX0DE5A051@D01VIVH0=cLg06a/K01Z
JVX0WYjN05eMG@1NGUh0EEEE08b<S01UIFD0@41005=CD`19BDT0GUiN0492@P15ADD0GUiN079bLP1F
EUH0GUiN04m?C`19BDT0UiNG07inOP1DE5@0IFEU0:b/[02<S8`0B4Q8071`L01aLG40YjNW06IVIP1N
GUh0H61P06=SH`1>CTh0IfMW05QHF01hN7P0Fe]K07MgM`1WIfL0HF5Q09RHV01SHf<0M7Ad03hn?P17
AdL0C4a<03Ti>@2KVi/0Ng]k061PH01WIfL0UIFE05IFEP1eMGD0F5QH09jNWP0S8b<0L71`06=SH`2e
]KD0XZ:R06MWI`1YJFT0I6AT08:2PP0l?3`0C4a<03Xj>P19BDT0EUIF05EEE@16ATH0@d=3059BDP2H
V9P0IfMW07MgM`2CTi<0OWin013ooooo000>ooooo`2_09VIV@1SHf<0Vi^K06YZJP1fMWH0DU9B06e]
K@2YZJT0F5QH05UIF@1:BTX0GUiN08>3P`16ATH0C4a<07]kN`1LG5`0YJFU08N7Q`0n?Sh0AdM707Mg
M`18B4P0MgMg055AD@0i>CT0JVYZ06m_K`1?Cdl0NGUi03De=@0d=3@0I6AT06UYJ@1=CDd0Lg=c03/k
>`18B4P0Ogmo05IFEP1JFUX0NGUi02`/;00o?cl0M7Ad05eMG@24Q8@0MGEe02l_;`1?Cdl0Fe]K05=C
D`24Q8@0=cLg02PX:01DE5@0EeMG06e]K@1aLG408B4Q038b<P1IFET0DU9B06UYJ@1IFET0:RXZ03`l
?01lO7`0Cdm?05iNGP1MGEd0=3@d06i^KP24Q8@0@410059BDP1KFe/0ATI606YZJP1gMgL0CDe=06EU
I@1KFe/0Hf=S07]kN`24Q8@0AdM706YZJP0j>SX0De=C0:BTY01JFUX0?Cdm07AdM01?Cdl0IfMW08n?
S`11@D40E5AD07moO`1RHV80G5aL07UiN@0a<C40De=C06m_K`1TI6@0HF5Q08R8R00n?Sh0HV9R06MW
I`1IFET0M7Ad0;Bd]017AdL0JVYZ07QhN01<C4`0Jf][08:2PP12@T80JVYZ08^;R`18B4P0Ph>309>C
T`0i>CT0EeMG06e]K@1/K6`0WIfM051@D00Z:RX0H61P06YZJP1QHF40QHF503<c<`12@T80IVIV08B4
Q01mOGd0QXJ603Ph>01=CDd0LW9b06e]K@27QhL0Vi^K03hn?P1RHV80NWYj06e]K@1TI6@0PH610492
@P1hN7P0Vi^K055AD@1bLW80]kNg04A4A02:RXX0J6QX04Y:BP1iNGT0`l?304a<C02IVIT0HF5Q05QH
F02FUYH0GemO03Ti>@0207YjNP0^05=CD`1cLg<0ATI605MGE`22PX80Lg=c05IFEP2<S8`0Bd];06=S
H`27QhL0E5AD06YZJP1WIfL0Jf][0:FUY@1RHV80=cLg06=SH`17AdL0TY:B0:j^[P1RHV80GemO06YZ
JP1_Kfl0`<30065QH@1ADE40KFe]04e=C@26QXH0J6QX04U9B@1oOgl0OWin05ADE01iNGT0BTY:059B
DP20P800LW9b07MgM`1WIfL0@d=308V9R@80Ng]k03D0]kNg03Lg=`0[:b/0QXJ6079bLP1lO7`0X:2P
03Ph>00d=3@0M7Ad06YZJP28R8P0OGem03De=@1CDe<0PH6106=SH`2BTY80MgMg03Hf=P1=CDd0OWin
08B4Q01fMWH0@d=30410@01DE5@0SHf=06e]K@1jNWX0=cLg05mOG`1cLg<0WIfM07YjNP1DE5@0Hf=S
05ADE01hN7P0Shn?051@D012@T80GEeM051@D01YJFT0Y:BT05UIF@0`<300C4a<05ADE01bLW80Zj^[
06IVIP0>ooooo`1C03lo?`1VIVH0Lg=c05iNGP1TI6@0DE5A059BDP1/K6`0ADE504e=C@18B4P0EEEE
051@D01/K6`0NWYj07QhN01QHF40Fe]K07EeM@1WIfL0AdM703De=@1RHV80@D5103/k>`1WIfL0Cdm?
03hn?P1HF5P0Jf][05IFEP12@T80C4a<06=SH`1GEeL0EEEE06MWI`1ADE40Bd];03<c<`14A4@0ATI6
0410@019BDT0E5AD06EUI@0n?Sh0CDe=06EUI@1IFET0Bd];06EUI@14A4@0D51@04e=C@10@400DU9B
05=CD`1GEeL0AdM7051@D00n?Sh0E5AD0492@P1GEeL0GUiN05eMG@1@D500Hf=S05aLG01HF5P0Lg=c
07inOP1?Cdl0G5aL034a<@1:BTX0Lg=c06e]K@1KFe/0QHF506a/K01>CTh00P0k>c/05`1@D500AdM7
02l_;`0f=SH0ATI603Ti>@0n?Sh0FUYJ069RHP0e=CD0A4A405YJFP16ATH0@d=305MGE`1MGEd0@410
04M7A`15ADD0>CTi04I6AP1UIFD0OWin0080AdM704T0De=C05]KF`1DE5@0HV9R059BDP1LG5`0Cdm?
04e=C@1NGUh0Jf][075aL@26QXH0JFUY06YZJP2;Rh/0/K6a061PH02JVYX0I6AT05UIF@1SHf<0EEEE
065QH@13@d<0Rh^;075aL@1[Jf/0QhN707emO@1[Jf/0HF5Q069RHP1MGEd0LG5a06EUI@1YJFT0AdM7
0:2PX01eMGD0OGem0861P@1mOGd0GEeM071`L023Ph<0Rh^;096AT@2e]KD0[jn_08V9R@1IFET0GEeM
07AdM01CDe<0S8b<08Z:RP2CTi<0Rh^;06EUI@1@D500F5QH06QXJ02<S8`0De=C06YZJP1CDe<0E5AD
069RHP1FEUH0G5aL09FEU@18B4P0LG5a0080AdM704d0D51@05]KF`1eMGD0IfMW07AdM01NGUh0LG5a
05YJFP0o?cl0E5AD05]KF`17AdL0M7Ad06UYJ@1_Kfl0Fe]K069RHP19BDT0F5QH04];B`0j>SX0I6AT
03/k>`10@400A4A406=SH`1@D500HV9R05]KF`1PH600EEEE04Y:BP2ATI40RHV908:2PP2<S8`0E5AD
06MWI`1BDU80RHV906e]K@1SHf<0DU9B05eMG@0f=SH0D51@04U9B@1bLW80HV9R065QH@19BDT0CTi>
04];B`24Q8@0PX:204E5A@1ADE40FUYJ07IfMP1OGel0=cLg07UiN@11@D40IVIV05mOG`1MGEd0Kfm_
09BDU01LG5`0TI6A06MWI`28R8P0JFUY096AT@2=SHd0MgMg096AT@0207IfMP0>05=CD`1YJFT0AdM7
06a/K01bLW80M7Ad04=3@`1RHV80De=C09JFUP1aLG40OGem09>CT`1lO7`@ooooo`003_ooool0P`2l
_;`0IFEU05iNGP20P800Shn?0492@P1;Bd/0G5aL05IFEP29RHT0>CTi03Ph>02?Shl0Jf][07moO`3=
cLd0?3`l069RHP2KVi/0LW9b0861P@2m_Kd0>c/k065QH@32`/80QHF50;Rh^00a<C40A4A407MgM`1Y
JFT0S8b<096AT@0G5aL0FEUI08>3P`1HF5P0OGem0;2`/00[:b/0DE5A0::RXP2=SHd0Ng]k06IVIP0X
:2P0A4A40:f][@1ZJVX0EeMG02/[:`0];Bd0HF5Q09BDU018B4P0Kfm_02h^;P0o?cl0KFe]061PH01O
Gel0Q8B403Ph>028R8P0Y:BT04i>CP1NGUh0JVYZ030`<01^KVh0YjNW04];B`15ADD0HF5Q02l_;`1:
BTX0PX:202/[:`0k>c/0=cLg04E5A@1ADE40QXJ602/[:`1NGUh0ADE504I6AP2RXZ80WIfM03Hf=P1/
K6`0HV9R04];B`2>SXh0GEeM03`l?01gMgL0I6AT07QhN02k^k/0<30`02l_;`1NGUh0BTY:08n?S`1d
M7@0;bl_03dm?@1KFe/0Hf=S0:n_[`1cLg<0:RXZ02l_;`1OGel0Kfm_0=GEe@2NWYh0>3Ph03@d=021
PH40SHf=09VIV@1bLW80<C4a05mOG`2VYZH0MgMg08V9R@1VIVH0?3`l0080IFEU02D0C4a<07emO@1D
E5@0E5AD09fMW@1VIVH0H61P06e]K@1NGUh0HV9R0=?Cd`2OWil0OGem08^;R`0i>CT0IFEU0<g=c@1c
Lg<0P82005iNGP0j>SX0Fe]K09FEU@17AdL0Kfm_05MGE`0/;2`0HF5Q08V9R@1eMGD0ZJVY05UIF@0]
;Bd0DU9B0;2`/01ZJVX0Jf][0080=3@d03`0GemO08^;R`0o?cl0DE5A03De=@1fMWH0SXj>03`l?00/
;2`0B4Q804M7A`1gMgL0c/k>0451@@0e=CD0LW9b069RHP3De=@0Xj>S02d];@1ADE40OGem06MWI`24
Q8@0;2`/03Xj>P20P800V9RH0;Rh^01/K6`0;Rh^061PH01jNWX0]KFe0:>SX`0X:2P0B4Q809>CT`2F
UYH0/;2`04A4A00Y:BT0FUYJ07EeM@2HV9P0F5QH02PX:01PH600KVi^05mOG`24Q8@0?clo04Q8B01[
Jf/0YjNW05=CD`2JVYX0ATI605]KF`1iNGT206MWI`0/06][J`17AdL0GemO09NGU`1SHf<0EeMG0::R
XP0f=SH0C4a<07emO@18B4P0GEeM07moO`11@D40A4A408F5Q@0e=CD0DU9B06i^KP0b<S80GemO06QX
J00/;2`0FEUI06MWI`18B4P0Cdm?065QH@0];Bd0ATI6059BDP11@D40HV9R07]kN`0o?cl0<S8b05MG
E`1@D500B4Q80820P00h>3P0LW9b05MGE`1;Bd/>ooooo`1S0410@02BTY80Q8B405ADE01OGel0CDe=
05IFEP1VIVH0?clo055AD@1BDU80FUYJ05mOG`1nOWh0IfMW08J6QP1]KFd0LW9b05EEE@14A4@0@410
03/k>`1jNWX0C4a<04i>CP1bLW80G5aL03Ti>@1;Bd/0Lg=c05UIF@17AdL0D51@069RHP1JFUX0DE5A
069RHP1SHf<0EeMG030`<018B4P0GemO05EEE@1DE5@0ATI607MgM`0h>3P0FUYJ06i^KP1UIFD0H61P
07QhN011@D40C4a<04M7A`1BDU80C4a<03De=@1RHV80CDe=06IVIP0e=CD0GemO051@D019BDT0MWIf
05QHF01JFUX0Ti>C05ADE01dM7@0Vi^K0;2`/01HF5P0EeMG02h^;P1HF5P0E5AD06EUI@2=SHd0b/[:
09JFUP1FEUH0ADE504m?C`17AdL0Hf=S03Lg=`1ADE40@d=303hn?P14A4@0=3@d03hn?P0g=cL0>3Ph
03@d=00e=CD0CTi>0080?Shn06<0@d=303/k>`16ATH0@T92055AD@1KFe/0F5QH038b<P19BDT0DU9B
065QH@16ATH0FEUI04Q8B00m?Cd0?Shn0451@@1HF5P0QHF506IVIP1kNg/0G5aL04Y:BP1`L700Q8B4
06ATI02c/k<0F5QH04U9B@1mOGd0TI6A0:j^[P1MGEd0P82006=SH`1[Jf/0CTi>0492@P16ATH0CTi>
09FEU@1EEED0>3Ph05EEE@1=CDd0;bl_08B4Q01SHf<0GemO05=CD`1<C4`0?Shn03lo?`15ADD0IVIV
07UiN@3=cLd0kno_09bLW023Ph<0R8R806QXJ01BDU80X:2P04];B`1@D500GemO03Ti>@1IFET0Bd];
05QHF01/K6`0EeMG0820P01OGel0N7Qh03hn?P0Z:RX0MGEe0:^[Z`1:BTX0GemO04a<C01cLg<0IVIV
06ATI01UIFD0>SXj06QXJ01ZJVX0S8b<06QXJ01OGel0SHf=0820P01PH600UYJF0:^[Z`2VYZH00P1S
Hf<02@1GEeL0FEUI04Y:BP16ATH0B4Q803dm?@16ATH0CTi>05QHF003069RHP0H05]KF`1QHF40JFUY
0:j^[P2WYjL0J6QX04a<C025QHD0Shn?05mOG`1IFET0EeMG07AdM00o?cl0FUYJ03Lg=`1NGUh0BDU9
06YZJP1aLG40IVIV05iNGP1XJ6P0@4100P1FEUH09P11@D40F5QH04=3@`1OGel0=CDe02LW9`2WYjL0
@d=305=CD`19BDT0Shn?0492@P1kNg/0>CTi09jNWP18B4P0Ng]k09NGU`3IfMT0QhN7075aL@2KVi/0
[jn_096AT@1kNg/0U9BD03Lg=`1CDe<0Kfm_05IFEP11@D40A4A4069RHP2/[:`0MGEe08Z:RP1nOWh0
KFe]4?ooool000kooooo0=`0[jn_0:ZZZP2j^[X0EeMG08N7Q`2?Shl0QXJ604e=C@1<C4`0MGEe07If
MP1oOgl0Hf=S05ADE01:BTX0U9BD05eMG@1lO7`0@D5103hn?P1KFe/0Rh^;04m?C`1ADE40<C4a04m?
C`1UIFD0ADE502XZ:P1:BTX0:BTY096AT@3=cLd0IFEU02`/;01>CTh0:b/[06QXJ02g]kL0C4a<05IF
EP1dM7@0HF5Q061PH022PX807alO03Ph>01PH600DU9B09^KV`0b<S809bLW0410@01ZJVX0E5AD0:RX
Z010@400IfMW06=SH`10@400MGEe0<C4a00Y:BT0<C4a06][J`1>CTh0UIFE09bLW00V9RH0C4a<0;Ng
]`1YJFT0S8b<0:NWY`0R8R80<30`08B4Q01KFe/0Ti>C07]kN`0Q8B40?Cdm07=cL`1IFET0XJ6Q06e]
K@15ADD0@D51096AT@1oOgl0VIVI02l_;`1EEED0EeMG08Z:RP2SXj<0MGEe024Q8@1@D500Kfm_06UY
J@1OGel0OWin030`<01SHf<0Ti>C05]KF`1YJFT0T92@03`l?01gMgL0Z:RX05eMG@29RHT0N7Qh03hn
?P1XJ6P0[jn_06i^KP1`L700Lg=c03lo?`1IFET0]kNg05ADE01VIVH0Cdm?03hn?P1=CDd0Vi^K05IF
EP2EUID0A4A403`l?02:RXX0EUIF0451@@1aLG40G5aL05eMG@2RXZ80DU9B0451@@1`L700Cdm?065Q
H@2m_Kd0ATI604M7A`23Ph<0Lg=c0820P02?Shl0<30`03/k>`1lO7`0HV9R0;Bd]02JVYX06QXJ03Ph
>01UIFD0FUYJ0<W9b@2SXj<0:2PX0451@@1DE5@0G5aL0;Rh^01DE5@0:b/[05EEE@1MGEd0FUYJ0:6Q
X@0W9bL0>c/k05EEE@1FEUH0DU9B07AdM017AdL0KFe]0:6QX@25QHD0MWIf0:^[Z`0h>3P0Shn?0=WI
f@1_Kfl0NWYj02l_;`0c<c<0MGEe07=cL`1`L700K6a/02l_;`1fMWH0Ogmo06EUI@1JFUX0@D5105MG
E`2=SHd0GUiN0861P@0m?Cd0DE5A07]kN`24Q8@0BTY:07EeM@1?Cdl0Ti>C0:j^[P17AdL0EUIF0P1P
H600>02WYjL0DU9B030`<01^KVh0E5AD03Ti>@2g]kL0GemO04A4A01fMWH0GEeM04a<C01kNg/0;bl_
03hn?P1LG5`0I6AT069RHP2UYJD0AdM70410@00o?cl0Fe]K06e]K@2/[:`0M7Ad0410@01?Cdl0Hf=S
092@T038b<P0?Cdm069RHP1XJ6P0I6AT0;Rh^033`l<0@d=304a<C01ZJVX0D51@09:BTP2PX:00;2`/
03`l?01EEED0JFUY0861P@2f][H0?Cdm05ADE01_Kfl0K6a/08n?S`1bLW80=SHf3_ooool0IP14A4@0
LW9b07alO01CDe<0Lg=c04];B`1ADE40U9BD04];B`1SHf<0FEUI0451@@1;Bd/0Hf=S06a/K01cLg<0
N7Qh08:2PP1cLg<0EeMG04Y:BP0V9RH0Bd];04Q8B01EEED0FUYJ05ADE00a<C40FUYJ06MWI`1EEED0
?3`l06QXJ018B4P0I6AT059BDP1KFe/0AdM7055AD@17AdL0?3`l04=3@`1=CDd0EEEE061PH01EEED0
=SHf05YJFP26QXH0FUYJ04e=C@1/K6`0EUIF069RHP1DE5@0?Shn05mOG`1cLg<0LG5a05UIF@1OGel0
Fe]K07EeM@1CDe<0F5QH07alO01YJFT0G5aL08^;R`1BDU80Jf][08N7Q`2OWil0Rh^;065QH@0m?Cd0
@d=307emO@2ATI40WIfM09RHV02RXZ80NGUi04i>CP1ADE40Hf=S05iNGP10@400E5AD05=CD`0m?Cd0
E5AD05aLG01`L700AdM7059BDP1=CDd0J6QX05]KF`1IFET0Kfm_04Y:BP80D51@0:@0B4Q805=CD`1F
EUH0Lg=c05YJFP1=CDd0FEUI071`L015ADD0JVYZ05YJFP1KFe/0AdM704e=C@1NGUh0@D5104A4A01B
DU80NGUi03Ti>@1DE5@0OWin05QHF03Ng]h0QXJ6051@D01?Cdl0ADE5069RHP0n?Sh0O7al08F5Q@1Z
JVX0IFEU06MWI`2EUID0FUYJ051@D01JFUX0Cdm?04I6AP15ADD08B4Q0=3@d011@D40EeMG08f=S@1U
IFD0@d=304A4A00m?Cd0AdM704A4A01bLW80Ogmo04Y:BP16ATH0CDe=0820P01ADE40Bd];0492@P1k
Ng/0Ph>306=SH`1kNg/0GUiN06UYJ@2:RXX0J6QX09jNWP1^KVh0M7Ad04i>CP1]KFd0De=C06a/K01?
Cdl0Ng]k05UIF@0k>c/0J6QX08V9R@1iNGT0F5QH07alO01/K6`0QXJ605iNGP1;Bd/0FEUI06][J`1Z
JVX0Ogmo04];B`1VIVH0Ng]k075aL@1FEUH0GEeM0410@00l?3`0J6QX04=3@`1=CDd0Bd];06EUI@1D
E5@0BTY:05UIF@1=CDd0Cdm?04a<C01BDU80PH6106UYJ@1RHV80B4Q804e=C@0b<S80NGUi04i>CP1G
EeL0>SXj04e=C@0h>3P0EeMG06QXJ021PH40D51@05iNGP16ATH0B4Q803Hf=P1OGel0Fe]K04E5A@17
AdL0F5QH07=cL`0f=SH08b<S059BDP0];Bd0LW9b04I6AP1<C4`0?clo092@T01EEED0MgMg06EUI@29
RHT0Jf][06i^KP1]KFd0Ng]k0:^[Z`2RXZ80JFUY0410@01HF5P0?Cdm071`L080NGUi00P0PH6109>C
T`1/K6`0O7al07]kN`1KFe/0LW9b08V9RA3ooooo000>ooooo`0J08Z:RP1UIFD0R8R8065QH@1ZJVX0
GemO09fMW@1[Jf/09BDU04m?C`0l?3`0TI6A0:RXZ00W9bL0@D5105ADE01<C4`0]kNg08F5Q@0[:b/0
IFEU07alO01DE5@0RXZ:0410@00_;bl2059BDP3:07EeM@2@T90061PH03lo?`1NGUh0N7Qh05EEE@28
R8P0<30`03De=@26QXH0QhN705aLG03>c/h0BTY:04E5A@1YJFT0BTY:06][J`2OWil0>c/k05iNGP1h
N7P0JFUY07inOP0_;bl0;Bd]03Ph>01VIVH0D51@05iNGP0K6a/0;2`/04m?C`1XJ6P0FEUI055AD@0X
:2P0?Shn07QhN01OGel0?Cdm04];B`0f=SH0BTY:09NGU`1JFUX0?clo04M7A`1EEED0Jf][09bLW00f
=SH0=cLg059BDP17AdL0T92@0:>SX`0_;bl0@T9205EEE@1HF5P0[:b/0410@012@T80M7Ad04U9B@1h
N7P0c/k>05ADE014A4@0F5QH04Q8B01FEUH0TY:B03De=@0m?Cd0K6a/04U9B@2ATI40`l?304E5A@1G
EeL0IFEU04];B`2ATI40_[jn03Ti>@12@T80L71`05EEE@2ATI40a/K604e=C@1EEED0WYjN07=cL`2j
^[X0T92@03De=@1JFUX0I6AT07QhN03@d=00N7Qh03hn?P1KFe/0XJ6Q0820P02@T900A4A40492@P2F
UYH0^k^k07MgM`1nOWh0>c/k04A4A02QXJ40`L710861P@26QXH0?Cdm08>3P`21PH40B4Q805mOG`1V
IVH0A4A40861P@2i^KT0KVi^07YjNP22PX80A4A4061PH03Cdm<0M7Ad09JFUP2DU9@0C4a<09NGU`3=
cLd0I6AT07QhN01VIVH0<S8b09BDU02MWId0IFEU07inOP0a<C40?Cdm096AT@22PX80E5AD0861P@19
BDT0CDe=0::RXP1BDU80G5aL05ADE01oOgl0`l?306ATI00];Bd0H61P04e=C@27QhL0ZJVY03@d=013
@d<0KFe]09bLW02/[:`0ATI604a<C02>SXh0FEUI0;bl_01=CDd0>c/k04I6AP1UIFD0NGUi0<C4a01C
De<0Kfm_08R8R01hN7P0Zj^[09NGU`0d=3@0/K6a09ZJVP1PH600^;Rh04A4A01>CTh0Ng]k07YjNP80
SXj>02h0:b/[05]KF`21PH40AdM70:b/[01`L7007AdM04I6AP2>SXh0L71`09>CT`1gMgL0;Bd]05QH
F020P800Ph>307MgM`0g=cL0;Bd]06UYJ@2BTY80S8b<06m_K`15ADD0=cLg04M7A`29RHT0OGem04m?
C`1QHF40?Cdm03Lg=`26QXH0IfMW03Hf=P1/K6`0>3Ph02XZ:P1PH600P82003@d=01QHF40:b/[06i^
KP2@T900Fe]K3_ooool04012@T80Ti>C09FEU@1PH600Kfm_05ADE01CDe<0G5aL0410@01UIFD0DU9B
0492@P1ADE40R8R809>CT`28R8P205=CD`0N05eMG@1EEED0CDe=03Lg=`1EEED0B4Q80492@P1SHf<0
HV9R038b<P1QHF40MgMg05QHF01ADE40FUYJ05UIF@1eMGD0L71`07UiN@1>CTh0DE5A028R8P0l?3`0
FEUI04=3@`1CDe<0A4A40861P@14A4@0HV9R0P1WIfL0=@1OGel0MGEe03De=@1=CDd0DU9B03lo?`1H
F5P0<c<c05QHF018B4P0GemO04Q8B01CDe<0?clo04I6AP1WIfL0D51@04a<C01aLG40Jf][05iNGP1c
Lg<0NGUi06a/K01GEeL0?Cdm061PH00m?Cd0CTi>09JFUP3Jf]X0UiNG05ADE00l?3`0>SXj03De=@18
B4P0>3Ph04I6AP1;Bd/0>c/k04E5A@1@D500C4a<03hn?P0l?3`0C4a<03dm?@19BDT0DE5A06QXJ00k
>c/0B4Q80080?Cdm00l0Cdm?071`L01lO7`0?clo059BDP1kNg/0Jf][04M7A`1LG5`0Bd];05iNGP12
@T80EUIF07]kN`1/K6`00P1^KVh0H@1NGUh0ADE506ATI028R8P0CDe=09fMW@1cLg<0GEeM04i>CP28
R8P0WYjN03@d=02@T900Ti>C08R8R01bLW80OWin09JFUP1nOWh0HV9R04U9B@1VIVH0EUIF0492@P13
@d<0L71`05UIF@1CDe<0Bd];05MGE`19BDT0Hf=S03/k>`1HF5P0DE5A09bLW02a/K40E5AD07UiN@1k
Ng/0Bd];03Lg=`1ZJVX0=CDe05ADE01IFET0=CDe03dm?@0b<S80OGem07]kN`1MGEd0QhN703hn?P1d
M7@0NWYj05]KF`1aLG40PH6104E5A@19BDT0=cLg05ADE016ATH0@T9205ADE011@D40BDU905ADE01X
J6P0@T9203hn?P1nOWh0D51@03Lg=`2DU9@0EEEE06QXJ00o?cl0Fe]K04U9B@1<C4`0H61P05]KF`1G
EeL0ADE50410@00j>SX0EeMG0492@P1ADE40HV9R05MGE`1LG5`0FUYJ04Y:BP1MGEd00P1^KVh0>@1P
H600?Shn02`/;01IFET0B4Q807moO`1TI6@0IfMW05ADE01`L700E5AD05=CD`1UIFD0FUYJ03dm?@1R
HV80?Shn0820P01]KFd0;Rh^04A4A01RHV80TI6A04E5A@0b<S80M7Ad03@d=01DE5@0EeMG08f=S@1J
FUX0KVi^04Y:BP2YZJT0GemO0;>c/`1HF5P0^k^k08>3P`1ZJVX0Ng]k09^KV`1YJFT0>SXj059BDP1;
Bd/0Rh^;06a/K01EEED0De=C0492@P1>CTh0XJ6Q07]kN`1LG5`0PH6108>3P`0@ooooo`003_ooool0
<@34a<@0EUIF07YjNP2HV9P0NWYj0451@@1EEED0XJ6Q05iNGP1dM7@0>SXj02l_;`1iNGT0G5aL07Ui
N@2SXj<0@41006IVIP2ZZZX0F5QH08j>SP1hN7P0;2`/069RHP1aLG40EUIF0820P00f=SH08b<S07If
MP0o?cl0?Shn03/k>`19BDT0ADE508B4Q01EEED0:2PX0492@P19BDT0Bd];079bLP1KFe/0>SXj04A4
A00a<C40IfMW0;Zj^P12@T800P14A4@0g`1QHF40_;bl071`L00M7Ad0?clo04I6AP26QXH0UiNG02PX
:00Y:BT0D51@05iNGP1mOGd0R8R801TI6@0U9BD0EUIF07QhN01bLW80IVIV02/[:`1EEED0JFUY06AT
I01/K6`0OWin02l_;`1IFET0SHf=069RHP1cLg<0MWIf0410@01TI6@0[jn_07EeM@1bLW80>SXj03Ph
>02?Shl0ZZZZ09NGU`39bLT0?clo03dm?@2MWId0Lg=c0:ZZZP1bLW80;Bd]03Lg=`1jNWX0J6QX08:2
PP1BDU809BDU03hn?P29RHT0LW9b08F5Q@10@400:b/[03Ti>@1WIfL0Lg=c08V9R@1LG5`0=CDe04Q8
B023Ph<0QhN706ATI00m?Cd0E5AD06e]K@2LW9`0FUYJ05]KF`1ADE40>SXj07AdM01lO7`0@D5107If
MP0l?3`0AdM709JFUP1GEeL0?clo059BDP0i>CT0HF5Q07moO`13@d<0>SXj08F5Q@11@D40Ti>C09nO
W`0h>3P0D51@07QhN01>CTh0UIFE08n?S`0U9BD0EUIF09VIV@1OGel0W9bL0<S8b00l?3`0AdM7075a
L@2BTY80ZZZZ0=gMg@16ATH0Ti>C0;Rh^01iNGT0/K6a07alO00X:2P0Ng]k07emO@2KVi/0`L7102XZ
:P0a<C40GemO0:>SX`2XZ:P0[Zj^02TY:@1KFe/0S8b<07alO02l_;`0FUYJ03dm?@2BTY80WYjN05iN
GP1nOWh0=cLg04I6AP2XZ:P0W9bL07YjNP1LG5`0Ogmo0;2`/01kNg/0RHV906IVIP0k>c/0R8R80:BT
Y01LG5`0I6AT02/[:`17AdL0aLG509RHV024Q8@0PH6104E5A@1@D500aLG507QhN02?Shl0ADE504i>
CP1SHf<0O7al03De=@1aLG40@D5106QXJ02c/k<0GEeM03Lg=`1UIFD0D51@09jNWP2a/K40<c<c02HV
9P1XJ6P0IVIV07IfMP2YZJT0>c/k02/[:`1YJFT0J6QX07MgM`1RHV807QhN03Ti>@1MGEd0De=C09VI
V@1@D500;Rh^04M7A`1DE5@0Fe]K05mOG`27QhL0?Cdm06][J`1PH600GemO06EUI@2TY:@0G5aL04=3
@`1CDe<0Ng]k07IfMP2[Zj/00P0[:b/00`2?Shl0PH61ooooo`0=ooooo`1N04Y:BP2GUiL0PX:2051@
D01^KVh0Hf=S079bLP1gMgL0@D5104a<C00k>c/0?clo055AD@1/K6`0EEEE05ADE01OGel0P82006a/
K01>CTh0@d=303Hf=P1CDe<0EeMG055AD@1FEUH0De=C0492@P1KFe/0JFUY05=CD`0j>SX0De=C069R
HP1XJ6P0D51@05YJFP1?Cdl0LG5a0451@@13@d<0HV9R04U9B@1WIfL0FEUI092@T018B4P0IFEU08>3
P`1JFUX0IVIV06e]K@1EEED0FUYJ065QH@1YJFT0KVi^04e=C@1ZJVX0E5AD09JFUP1CDe<0KFe]05=C
D`1:BTX0O7al05MGE`19BDT0MgMg05MGE`1aLG40TI6A08n?S`10@400DU9B02@T901=CDd0GEeM09JF
UP2?Shl0YZJV09:BTP1PH600C4a<055AD@1?Cdl0KFe]03Ti>@1FEUH0CDe=03Hf=P1<C4`0AdM707Ad
M080C4a<03X0?Cdm04i>CP1;Bd/0JVYZ05iNGP13@d<0AdM703lo?`0g=cL0C4a<03`l?026QXH0B4Q8
04e=C@1FEUH0I6AT04Q8B01/K6`0Ng]k04i>CP10@400B4Q806i^KP27QhL0LG5a0820P01oOgl0EUIF
06m_K`2BTY80Cdm?0;Vi^@20P800FEUI05QHF01LG5`0N7Qh05=CD`2_[jl0IFEU069RHP1NGUh0EeMG
05EEE@1:BTX0H61P04I6AP0m?Cd0GEeM06=SH`0b<S80a<C40451@@0l?3`0Kfm_051@D013@d<0H61P
0P1UIFD0NP1IFET0Zj^[0<_;b`0h>3P0@d=3038b<P1MGEd0ATI609JFUP1YJFT0Jf][092@T01GEeL0
H61P04a<C01FEUH0BTY:03hn?P1dM7@0B4Q805IFEP0n?Sh0:b/[06i^KP2/[:`0@d=305]KF`1:BTX0
@d=30410@029RHT0Ph>304a<C01HF5P0PX:209jNWP1IFET0E5AD07EeM@1BDU80?clo05aLG01[Jf/0
UYJF05ADE01>CTh0IfMW06][J`14A4@0Bd];04Q8B00o?cl0De=C0410@00j>SX0F5QH061PH018B4P0
CDe=051@D00n?Sh0@d=30410@01aLG40^KVi07UiN@1JFUX0B4Q8061PH01CDe<0B4Q8030`<01EEED0
;2`/03lo?`1BDU80OWin04e=C@1JFUX0L71`0861P@1HF5P0/[:b07alO00i>CT0@D5105iNGP1TI6@0
CTi>04U9B@30`<00HV9R06i^KP1?Cdl0Bd];061PH02DU9@0KFe]075aL@1>CTh0OWin08f=S@3[jn/0
UIFE06=SH`27QhL0OGem04i>CP14A4@0De=C04Q8B01SHf<0Ng]k06EUI@14A4@0De=C07EeM@2_[jl0
M7Ad08F5Q@1iNGT0RXZ:4?ooool000kooooo08<0UiNG06][J`1oOgl0GemO09:BTP1<C4`0EEEE04m?
C`0o?cl0RXZ:03Hf=P0i>CT0De=C05IFEP1VIVH0Xj>S02d];@0o?cl0GUiN08^;R`2a/K40UYJF02XZ
:P11@D40P82005MGE`2a/K40<S8b02/[:`1NGUh0E5AD096AT@1iNGT0;2`/03Ti>@1CDe<0Kfm_07If
MP2n_[h0<c<c02`/;01lO7`0V9RH079bLP1UIFD0=CDe0451@@2OWil0QHF508>3P`0c<c<08R8R0492
@P2g]kL0J6QX08Z:RP0d=3@0@d=307UiN@2LW9`0EeMG08J6QP0g=cL0@T9206QXJ016ATH0IFEU096A
T@0W9bL0E5AD06a/K00g=cL0EEEE08j>SP0V9RH0<C4a07moO`0c<c<0EeMG06e]K@0d=3@0@d=306][
J`0c<c<0MgMg03Lg=`0o?cl0AdM706MWI`1>CTh0LW9b05=CD`1ADE40Ph>305=CD`0h>3P0E5AD03De
=@2>SXh0^[Zj02d];@0Z:RX0AdM703lo?`2:RXX0QhN701dM7@0g=cL0De=C05EEE@2_[jl0Ng]k02DU
9@0m?Cd0ADE506ATI038b<P0QhN702<S8`17AdL0G5aL06IVIP2IVIT0IFEU02DU9@1LG5`0WIfM06m_
K`2DU9@0HV9R0410@00207]kN`0B075aL@2f][H0DU9B0451@@1UIFD0Jf][06QXJ02OWil0?3`l04U9
B@2LW9`0cLg=06IVIP39bLT0;Rh^06e]K@2DU9@0OWin0P27QhL02@0R8R80CTi>09RHV01CDe<0Shn?
04a<C00Y:BT0C4a<0:j^[P0207YjNP1C030`<01BDU80FEUI0;:b/P1lO7`0Kfm_030`<01ADE40FEUI
0;Jf]P1VIVH0J6QX03dm?@0o?cl0TY:B079bLP0k>c/0A4A404I6AP1[Jf/0];Bd05ADE00b<S80NGUi
051@D03?cll0^;Rh03Lg=`16ATH0@41003hn?P1gMgL0=3@d03Lg=`1=CDd0EeMG0:FUY@1_Kfl0A4A4
08B4Q01dM7@0b<S809nOW`0Z:RX0DE5A06e]K@2KVi/0];Bd03hn?P0a<C40Hf=S096AT@2TY:@0Ph>3
03`l?01MGEd0P82009NGU`2n_[h0<30`0410@01lO7`0RHV905MGE`2OWil0?clo05MGE`1QHF40Vi^K
06][J`28R8P0=SHf061PH02:RXX0/k>c07YjNP2RXZ80Cdm?04i>CP1lO7`0PH6106QXJ02d];@00P1=
CDd07@1aLG40CTi>07alO01/K6`0?clo06=SH`2j^[X0EEEE08b<S025QHD0>c/k0451@@2=SHd0BDU9
07MgM`1lO7`0;Bd]03De=@1PH600N7Qh05EEE@21PH40<S8b02/[:`2ATI40KVi^061PH00/;2`0?clo
00kooooo01l0Bd];05IFEP1PH600Cdm?04e=C@0m?Cd0B4Q808N7Q`1CDe<0GemO05QHF01EEED0De=C
05iNGP1ZJVX0MgMg08>3P`1eMGD0M7Ad06EUI@1;Bd/0=SHf075aL@11@D40AdM7079bLP1NGUh0>3Ph
04a<C00l?3`0D51@0080AdM701h0<C4a05]KF`1@D500HF5Q05aLG01?Cdl0<c<c04Y:BP1BDU80EEEE
07UiN@1]KFd0F5QH04I6AP1NGUh0TI6A06ATI01DE5@0K6a/04U9B@1HF5P0CDe=03lo?`1?Cdl0Bd];
06QXJ01ZJVX0QXJ608N7Q`1mOGd205iNGP2K08:2PP1^KVh0HV9R07EeM@1fMWH0E5AD075aL@2EUID0
GEeM06UYJ@0_;bl0AdM703dm?@1/K6`0CDe=04U9B@2ZZZX0IFEU04=3@`1=CDd0Fe]K065QH@1:BTX0
DE5A06=SH`16ATH0C4a<05]KF`1RHV80D51@04];B`1<C4`0DU9B05mOG`2CTi<0Lg=c04];B`1<C4`0
CTi>03lo?`1>CTh0CDe=09FEU@1IFET0E5AD079bLP20P800De=C096AT@1gMgL0E5AD03`l?018B4P0
R8R807MgM`1iNGT0J6QX05iNGP18B4P0I6AT09ZJVP22PX80[Jf]0861P@23Ph<0PX:206YZJP2?Shl0
EEEE09FEU@2?Shl0KVi^06=SH`1VIVH0][Jf0820P01TI6@0DE5A05ADE01ADE40@T92034a<@2PX:00
?clo03<c<`1`L700FUYJ04Q8B01HF5P0CDe=09fMW@1DE5@0V9RH0;>c/`1NGUh0D51@059BDP1]KFd0
Cdm?06ATI011@D40Lg=c09:BTP1TI6@0KVi^04I6AP1KFe/0HF5Q06i^KP2k^k/0H61P06ATI026QXH0
Cdm?05UIF@1VIVH0Bd];079bLP14A4@0?clo03Xj>P1@D500B4Q804I6AP1OGel0DE5A04i>CP1<C4`0
AdM7051@D01jNWX0IfMW07QhN01IFET0KVi^05eMG@1IFET0?clo03Xj>P0d=3@0@d=305=CD`0o?cl0
ADE504M7A`13@d<0BDU904M7A`1CDe<0CTi>04];B`19BDT0Lg=c03Xj>P15ADD00P1MGEd0>016ATH0
ADE506UYJ@0d=3@0DE5A03<c<`1KFe/0@T9204m?C`1SHf<0Vi^K05ADE01fMWH0IfMW04Y:BP1EEED0
M7Ad06MWI`1dM7@0KFe]05MGE`1LG5`0;bl_03Lg=`1oOgl0De=C06m_K`1?Cdl0C4a<05iNGP2?Shl0
I6AT08B4Q01<C4`0JFUY051@D01]KFd0PH6107MgM`2GUiL0V9RH09BDU029RHT0PH6106ATI01RHV80
UIFE06YZJP1BDU80Lg=c075aL@2OWil0Ph>308Z:RP29RHT0QhN74?ooool000kooooo0;00S8b<071`
L02LW9`0QXJ608N7Q`1VIVH0Rh^;096AT@13@d<0KFe]05QHF01gMgL0VYZJ03@d=013@d<0IfMW06QX
J01lO7`0DU9B05=CD`1DE5@0Ng]k061PH02ATI40;2`/03@d=01MGEd0OGem04I6AP25QHD08b<S03Hf
=P1LG5`0H61P04a<C02@T900=CDe04E5A@1NGUh0JFUY05ADE02TY:@0H61P04e=C@2;Rh/0@D5108Z:
RP1_Kfl0<C4a05QHF01ZJVX0<c<c06=SH`0d=3@0;Rh^06UYJ@1DE5@0Kfm_05iNGP15ADD0;Bd]06YZ
JP1EEED0JVYZ05QHF00V9RH0BDU908f=S@1TI6@0Cdm?06EUI@0j>SX0Fe]K09RHV01DE5@0?clo04m?
C`11@D40HF5Q07moO`12@T80CDe=05ADE00k>c/0Rh^;06m_K`0j>SX0Fe]K05EEE@1GEeL0T92@03`l
?00h>3P0IVIV05ADE01KFe/0O7al038b<P1;Bd/0GUiN05ADE01ADE40JFUY03Ti>@1EEED0JVYZ059B
DP1HF5P0QhN703Ph>01JFUX0MGEe051@D01TI6@0MgMg02l_;`1CDe<0MGEe05IFEP1aLG40S8b<04=3
@`1HF5P0Z:RX06IVIP23Ph<0Fe]K034a<@1BDU80J6QX05=CD`2@T900ATI603<c<`1WIfL0LG5a08J6
QP2RXZ80=SHf0492@P1aLG40T92@05UIF@1`L700=cLg05EEE@2TY:@0Rh^;06ATI01cLg<0DU9B05mO
G`2BTY80C4a<04Q8B01FEUH0@41007EeM@1cLg<0=3@d04i>CP1ZJVX0F5QH09RHV02:RXX0@d=306MW
I`1RHV80KFe]0:VYZ@1SHf<0BDU905QHF01YJFT0DE5A09JFUP80EUIF04d0K6a/06QXJ01;Bd/0RHV9
06e]K@1fMWH0R8R807=cL`1ZJVX0Vi^K05MGE`2>SXh0NWYj06MWI`2c/k<0J6QX030`<01UIFD0K6a/
08^;R`2BTY80>CTi04e=C@1fMWH0WYjN07IfMP0n?Sh0IVIV08^;R`1TI6@0Vi^K0410@01<C4`0TI6A
0861P@1PH600UIFE059BDP1SHf<0U9BD07UiN@2MWId0IfMW03hn?P2EUID0RHV906EUI@2JVYX0A4A4
04];B`20P800M7Ad079bLP2PX:00=cLg03<c<`2FUYH0CDe=07moO`1jNWX0;bl_03/k>`1XJ6P0Ng]k
07emO@1VIVH0<c<c05eMG@1iNGT0T92@079bLP1=CDd0=SHf071`L02ZZZX0EeMG06UYJ@0205UIF@0E
08Z:RP2EUID0H61P0492@P1NGUh0>SXj061PH029RHT0EEEE03hn?P1FEUH0Fe]K05ADE02<S8`0Fe]K
0451@@1cLg<0A4A408n?S`2NWYh0D51@00kooooo01H0CDe=06MWI`1fMWH0EEEE06a/K01;Bd/0Cdm?
05mOG`0l?3`0FUYJ04a<C00j>SX0AdM705eMG@1XJ6P0IVIV0410@01BDU80BDU90410@00k>c/0<c<c
0P12@T80GP15ADD0B4Q804A4A00_;bl0D51@04Q8B015ADD0>CTi059BDP17AdL0E5AD04e=C@1EEED0
A4A404a<C00V9RH0@d=303lo?`0l?3`0EeMG055AD@1LG5`0A4A405QHF01ZJVX0J6QX05YJFP24Q8@0
BTY:06YZJP1NGUh0>3Ph05EEE@0l?3`0EeMG061PH01bLW80EEEE06EUI@1@D500B4Q807IfMP1MGEd0
FEUI08B4Q01DE5@0KVi^08n?S`2BTY80De=C069RHP17AdL0HF5Q04U9B@1>CTh0I6AT05YJFP2CTi<0
D51@04E5A@19BDT0ATI604i>CP17AdL0De=C04U9B@0d=3@0AdM70410@019BDT0@41004m?C`15ADD0
BTY:05ADE01EEED0JVYZ05=CD`1>CTh0A4A404=3@`1?Cdl0GemO06m_K`10@400DE5A08J6QP1XJ6P0
H61P0820P01QHF40De=C0451@@1EEED207UiN@0H08>3P`2;Rh/0HF5Q06ATI01VIVH0/k>c09^KV`2^
[Zh0KFe]06UYJ@1ADE40KVi^09ZJVP17AdL0/[:b07]kN`27QhL0JFUY06e]K@1JFUX0Jf][06IVIP18
B4P0F5QH0P1JFUX0>P1EEED0Zj^[05YJFP0g=cL0OGem05QHF01IFET0SXj>03De=@1[Jf/0Lg=c0<_;
b`3AdM40CDe=06EUI@1KFe/0G5aL04U9B@2:RXX0<S8b06ATI01`L700FUYJ06i^KP0];Bd0Jf][06a/
K00i>CT0JFUY03`l?01FEUH0De=C03lo?`1YJFT0MWIf03Hf=P1ADE40?Shn04a<C00h>3P0EeMG05YJ
FP1CDe<0I6AT05YJFP1VIVH0ADE50492@P1OGel0C4a<05UIF@2:RXX0LW9b08b<S01ADE40GEeM05=C
D`1PH6020410@018051@D00e=CD0?3`l02l_;`18B4P0@41004Q8B01EEED0Bd];04a<C011@D40Fe]K
02d];@1]KFd0J6QX05ADE00`<300<S8b06ATI00n?Sh0F5QH02d];@1EEED0A4A4059BDP20P800XJ6Q
05YJFP1gMgL0KVi^06YZJP1KFe/0TY:B07UiN@1FEUH0DU9B061PH01SHf<0BDU906YZJP29RHT0F5QH
061PH01<C4`0KVi^05eMG@2=SHd0MWIf0:b/[01cLg<0Shn?079bLP2UYJD0Ti>C09jNWP2g]kL0YJFU
07AdM01:BTX0Hf=S04m?C`1aLG40QXJ606YZJP1CDe<0FEUI06IVIP2CTi<0Jf][06ATI020P800Q8B4
4?ooool000kooooo0?l0alO705EEE@1kNg/0ZJVY0:b/[00g=cL0FEUI065QH@1aLG40Shn?02TY:@1:
BTX0QhN706UYJ@2;Rh/0IfMW02l_;`1;Bd/0QHF506EUI@2BTY80VIVI01TI6@29RHT0Hf=S05ADE02N
WYh0=cLg01/K6`1hN7P0Bd];06IVIP1QHF408b<S034a<@1YJFT0Q8B407IfMP24Q8@092@T02/[:`1a
LG40RXZ:04m?C`0i>CT0=CDe04m?C`1`L700EeMG06=SH`11@D40EUIF09nOW`2?Shl0>3Ph05=CD`14
A4@0Ph>308^;R`19BDT0:b/[04I6AP0o?cl0QHF508b<S00W9bL0:RXZ0451@@1KFe/0HV9R06UYJ@0T
92@0;Bd]055AD@1QHF40HV9R08f=S@0U9BD0<S8b05IFEP1SHf<0LW9b07]kN`0`<300>c/k05mOG`1>
CTh0MGEe08n?S`0^;Rh0EeMG07moO`1_Kfl0Rh^;05=CD`0j>SX0H61P05EEE@2:RXX0Vi^K02@T900c
<c<0Bd];05UIF@22PX80K6a/02LW9`0_;bl0Cdm?06YZJP2TY:@0KFe]034a<@19BDT0FUYJ05mOG`2T
Y:@0FUYJ02l_;`1BDU80EeMG0861P@2>SXh0<c<c0410@01kNg/0VIVI04e=C@1ZJVX0Cdm?0410@01l
O7`0Vi^K05EEE@1jNWX0>c/k05QHF02[Zj/0RHV903hn?P1HF5P0BTY:05YJFP29RHT0FEUI051@D01b
LW80?Cdm06MWI`2PX:00;bl_04Q8B015ADD0>3Ph07alO029RHT09RHV05iNGP1KFe/0ATI609:BTP2]
[Jd0:2PX03hn?P1SHf<0B4Q809ZJVP20P800>3Ph08>3P`2<S8`0LW9b09jNWP1gMgL0<30`04m?C`1k
Ng/0PH610<O7a`1;Bd/0<C4a04U9B@1PH600e=CD0;fm_@0[:b/0EEEE08f=S@1KFe/0hn?S05EEE@0W
9bL0L71`0::RXP1MGEd0IFEU04=3@`1WIfL0]KFe07MgM`1LG5`0@T92065QH@2j^[X0QXJ608N7Q`1]
KFd0B4Q806i^KP2?Shl0Ng]k06i^KP0n?Sh0C4a<05aLG02DU9@0Vi^K071`L00m?Cd0K6a/0;>c/`1W
IfL0N7Qh04A4A01;Bd/0M7Ad0;^k^`1>CTh0D51@030`<01=CDd0Ogmo07emO@1:BTX0Cdm?03dm?@2J
VYX0][Jf05eMG@0c<c<0HF5Q04I6AP1hN7P0`l?305YJFP14A4@0Kfm_04M7A`1]KFd0[Jf]05MGE`18
B4P0De=C04e=C@1hN7P001L0?3`l030`<019BDT0IFEU03dm?@20P800Shn?01lO7`0i>CT0G5aL04=3
@`1=CDd0]KFe07YjNP0_;bl0AdM704E5A@1PH600XZ:R05ADE00i>CT0EeMG05]KF`0>ooooo`0904Q8
B01RHV80Rh^;04i>CP1SHf<0Cdm?05]KF`1UIFD0A4A40080?clo00L0AdM705=CD`1<C4`0ATI604];
B`1OGel0EEEE0080EeMG00T0BDU902XZ:P1IFET0ADE504U9B@1[Jf/0FEUI03Xj>P18B4P00P1@D500
200o?cl0D51@04M7A`1SHf<0F5QH05]KF`1IFET0De=C0P1;Bd/09@1UIFD0E5AD061PH01PH600PX:2
04i>CP1TI6@0LW9b06=SH`0k>c/0G5aL05iNGP1PH600CTi>038b<P18B4P0=SHf04e=C@1BDU80I6AT
04M7A`12@T80CDe=03lo?`1KFe/0C4a<04i>CP1dM7@0D51@059BDP1XJ6P0MgMg069RHP1gMgL0=SHf
05ADE01MGEd00P1ZJVX0``2?Shl0KVi^04e=C@18B4P0CDe=0451@@1BDU80>SXj051@D01:BTX0Cdm?
04M7A`1<C4`0EEEE04=3@`11@D40@d=304];B`10@400E5AD05mOG`15ADD0C4a<05aLG01>CTh0De=C
06IVIP1nOWh0IFEU05mOG`10@400FUYJ08V9R@2GUiL0C4a<04m?C`1DE5@0FUYJ065QH@1_Kfl0LW9b
0820P021PH40HF5Q06e]K@2g]kL0YJFU0<[:bP23Ph<0PX:2079bLP1UIFD0K6a/03lo?`2>SXh0JVYZ
07=cL`1hN7P0Fe]K0:ZZZP1kNg/0Hf=S05]KF`16ATH0IVIV05ADE01UIFD0g]kN05QHF00o?cl0Ph>3
03Ti>@0l?3`0QXJ602d];@2=SHd0DE5A09NGU`2_[jl0DU9B05MGE`1VIVH0KFe]0451@@2ATI40BDU9
069RHP27QhL0E5AD04A4A00a<C40KVi^071`L00n?Sh0/k>c055AD@1SHf<0NGUi04I6AP1ZJVX0KVi^
03hn?P1[Jf/0ATI604M7A`15ADD0OGem04U9B@13@d<0H61P079bLP2<S8`0EUIF04Q8B01:BTX0EUIF
06a/K02=SHd0?Cdm05aLG00n?Sh0GemO04Q8B00k>c/0AdM703/k>`1BDU80=3@d03dm?@0n?Sh0CTi>
04=3@`11@D40Cdm?04Y:BP16ATH0F5QH055AD@0a<C40NWYj059BDP1fMWH0;2`/03Ph>01IFET0:RXZ
0451@@1?Cdl0E5AD04=3@`15ADD0I6AT061PH01ADE40K6a/06e]K@1EEED0CTi>06m_K`1]KFd0?3`l
04I6AP1LG5`0HV9R065QH@1/K6`0TI6A04m?C`1JFUX0@410055AD@1MGEd0SXj>06ATI024Q8@0De=C
06MWI`2NWYh0/;2`09RHV036a/H0UYJF09NGU`2ATI40D51@05]KF`1>CTh0Kfm_07moO`1SHf<0>CTi
05IFEP1cLg<0WinO075aL@0206][J`0309ZJV_oooooooooo00kooooo000>ooooo`0M09^KV`1fMWH0
Rh^;05UIF@2SXj<0M7Ad061PH01LG5`0=3@d07QhN01DE5@0HF5Q05IFEP0`<300GEeM09JFUP0n?Sh0
@D5104A4A00l?3`0M7Ad0:>SX`15ADD0CTi>06ATI01RHV80UIFE05YJFP0Q8B400P1@D500m`1ZJVX0
N7Qh028R8P0^;Rh0=SHf05=CD`2/[:`0]KFe02DU9@0a<C40@d=304Y:BP1oOgl0Jf][02/[:`1LG5`0
M7Ad05QHF02FUYH092@T04E5A@1MGEd0JFUY06=SH`26QXH0A4A403De=@1jNWX0T92@05mOG`3=cLd0
?Shn04A4A01cLg<0G5aL08b<S01mOGd0>CTi0492@P1iNGT0DU9B08j>SP2SXj<0820P0410@01^KVh0
EeMG08F5Q@2GUiL0:b/[04Q8B01`L700BTY:0861P@0/;2`0<30`051@D024Q8@0H61P05YJFP0R8R80
BDU905IFEP1ZJVX0Cdm?03hn?P0j>SX0De=C079bLP1DE5@0=CDe04];B`0c<c<0VYZJ0:n_[`1DE5@0
;Rh^0451@@0b<S80U9BD0:b/[00j>SX0?Shn05iNGP12@T80VIVI0:^[Z`0^;Rh0@d=305eMG@1=CDd0
U9BD0;Zj^P0m?Cd0F5QH06ATI01DE5@0V9RH06UYJ@0k>c/0KVi^07inOP1EEED0]kNg04];B`0g=cL0
R8R8092@T01TI6@0Zj^[065QH@1IFET0R8R807alO01iNGT0c/k>03Ph>01HF5P0Xj>S09:BTP2MWId0
FEUI0451@@0h>3P0Z:RX05YJFP1oOgl0Fe]K03Xj>P0a<C40YjNW0820P01iNGT0L71`03De=@1>CTh0
RHV906i^KP1aLG40=CDe04m?C`1WIfL0Xj>S06UYJ@1SHf<08b<S059BDP1`L700SXj>04e=C@1gMgL0
>c/k05aLG02UYJD0GemO04m?C`1gMgL0?Shn08B4Q02UYJD0EEEE06UYJ@1:BTX0:BTY07AdM00e=CD0
A4A406a/K01CDe<0TY:B07]kN`0j>SX0GUiN04I6AP2BTY80ZZZZ02l_;`1>CTh0Cdm?07moO`2m_Kd0
GEeM02/[:`1MGEd0KFe]0:FUY@30`<00BTY:04a<C01MGEd0R8R80:JVYP0c<c<0=CDe04M7A`25QHD0
Hf=S0;Rh^00d=3@0@T9208J6QP2<S8`0H61P0:b/[00Y:BT0EEEE0:>SX`2HV9P0MWIf0:FUY@1LG5`0
CDe=08^;R`2JVYX0P8200<g=c@1=CDd0G5aL0:j^[P3=cLd0ZZZZ071`L00T92@0GemO07MgM`1<C4`0
P82009^KV`0/;2`0>3Ph08f=S@1@D500I6AT061PH00e=CD0<C4a04m?C`25QHD0JFUY08^;R`0];Bd0
=CDe065QH@1cLg<0RXZ:06YZJP0k>c/03_ooool0;P11@D40A4A4059BDP1DE5@0@d=302h^;P0m?Cd0
E5AD04i>CP1ADE40Bd];0492@P0e=CD0ATI605=CD`15ADD0@41004A4A00];Bd0:BTY04a<C00f=SH0
G5aL03`l?00`<300@T9204];B`1<C4`0CTi>03Lg=`14A4@0CTi>04Y:BP0b<S80Bd];05EEE@1MGEd0
?Shn0492@P0Q8B408b<S03Xj>P11@D40AdM703Xj>P19BDT204a<C00F0451@@1DE5@0C4a<07AdM00k
>c/0?3`l04];B`1>CTh0FEUI04A4A01CDe<0EeMG07]kN`1KFe/0FUYJ06=SH`17AdL0J6QX05mOG`1M
GEd0QHF506UYJ@80K6a/09X0FUYJ06i^KP1ZJVX0=CDe055AD@1RHV80KFe]09:BTP2ATI40@D5104M7
A`0m?Cd0=3@d03lo?`0m?Cd0B4Q803`l?017AdL0;2`/05=CD`1GEeL0L71`05QHF01<C4`0?Cdm05MG
E`1RHV80IVIV07EeM@1=CDd0CTi>04Y:BP15ADD0F5QH07]kN`1lO7`0=cLg04a<C01]KFd0Bd];03/k
>`1NGUh0ADE50410@00d=3@0D51@055AD@1KFe/0OGem09fMW@1=CDd0Cdm?07UiN@3;bl/0OGem09fM
W@1?Cdl0KVi^05aLG02<S8`0RXZ:04];B`2WYjL0O7al061PH028R8P0G5aL04];B`1DE5@0MWIf05AD
E01PH600FEUI069RHP1]KFd0NWYj06MWI`1IFET0O7al05mOG`1>CTh0Ogmo051@D01dM7@0MGEe0=3@
d036a/H0EeMG08j>SP1SHf<0DE5A04A4A02PX:00CDe=09:BTP2CTi<0DE5A05MGE`19BDT0M7Ad07al
O01dM7@0Fe]K04e=C@1?Cdl0FUYJ04m?C`1_Kfl0K6a/04=3@`1KFe/0@d=304M7A`0h>3P0KVi^061P
H01:BTX0?Cdm04m?C`1CDe<0BTY:05=CD`1aLG40GEeM06QXJ02`/;00IFEU08b<S015ADD0DE5A07em
O@1lO7`0>SXj05YJFP1cLg<0@T9204=3@`0m?Cd0Cdm?04e=C@17AdL0H61P05YJFP1DE5@0C4a<06=S
H`0j>SX0Rh^;05mOG`1VIVH0@41003Lg=`1@D500=CDe0P1DE5@0<P1OGel0>3Ph04E5A@1QHF40S8b<
04Y:BP1hN7P0DU9B05]KF`1TI6@0JFUY05ADE014A4@0B4Q805]KF`1VIVH0:BTY05aLG02:RXX0@d=3
05mOG`0e=CD0DU9B03De=@2=SHd0?Cdm0861P@1ZJVX0NGUi05IFEP1^KVh0MWIf09bLW02DU9@0/[:b
08R8R016ATH0NGUi0492@P1]KFd0T92@06ATI017AdL0GUiN06QXJ01jNWX0QXJ6069RHP27QhL0WIfM
4?ooool000kooooo04`0Lg=c08:2PP2LW9`0DU9B07AdM025QHD0O7al08b<S00a<C40Jf][08b<S02=
SHd0S8b<03Xj>P1?Cdl0JFUY07YjNP1SHf<071`L04I6AP1gMgL0YJFU05YJFP27QhL09bLW04Y:BP22
PX80X:2P051@D01hN7P0=3@d04];B`1fMWH0G5aL04E5A@1iNGT0:b/[05MGE`2?Shl0LG5a04E5A@2Q
XJ40ADE504Y:BP2VYZH0?Shn04U9B@1IFET0<30`07IfMP2BTY80<S8b04U9B@0j>SX092@T059BDP1D
E5@0:b/[03dm?@0[:b/0>CTi0:BTY01CDe<0=cLg03Ti>@0Q8B40GEeM0:BTY01IFET0>SXj04Y:BP0c
<c<0DE5A09jNWP1:BTX08b<S0P0m?Cd0/01GEeL0U9BD03<c<`0m?Cd0B4Q8079bLP2RXZ80BDU902DU
9@18B4P0EEEE07MgM`2QXJ40;bl_04I6AP1CDe<0GemO08B4Q01[Jf/0<C4a05]KF`1dM7@0GUiN051@
D01^KVh0;2`/05=CD`1NGUh0OGem08f=S@27QhL0@41005]KF`1^KVh0FEUI07]kN`2UYJD0?Shn04Y:
BP21PH40OGem08b<S02RXZ80?3`l0861P@2EUID0YJFU0<?3``2GUiL0@D5104A4A02?Shl0MgMg08b<
S014A4@0@41005ADE029RHT0Lg=c07UiN@17AdL0BTY:05eMG@2ZZZX0DU9B06m_K`0`<300De=C08V9
R@2MWId0HV9R06YZJP1:BTX0O7al07moO`0k>c/0@D5104i>CP1IFET0Q8B406m_K`0P8200:BTY05aL
G01ZJVX0RHV906m_K`0];Bd0CDe=055AD@2;Rh/0WYjN04a<C00o?cl0EEEE07moO`21PH40TY:B02l_
;`0X:2P0FEUI06UYJ@1ZJVX0T92@04a<C00a<C40G5aL06MWI`1hN7P0^;Rh030`<01KFe/0QHF507Ui
N@2d];@0LG5a02/[:`1HF5P0Fe]K09nOW`2BTY80<S8b04E5A@26QXH0Shn?09jNWP1@D500?clo08B4
Q02GUiL0Y:BT04=3@`15ADD0EEEE08R8R01]KFd0Y:BT03Ph>01ADE40PX:206IVIP2SXj<0GUiN038b
<P29RHT0VYZJ061PH02MWId0CDe=0451@@23Ph<0UYJF05UIF@1aLG40>CTi03lo?`1eMGD0A4A4079b
LP1HF5P0=SHf038b<P1]KFd0I6AT05EEE@1?Cdl0;2`/04Y:BP20P800Kfm_04m?C`1GEeL0ADE508V9
R@3Fe]H0JFUY0P1;Bd/05P1DE5@0SHf=0:b/[01<C4`08B4Q055AD@1:BTX0JVYZ08V9R@0Y:BT0<30`
0492@P1IFET0MWIf0<c<c019BDT08B4Q03Lg=`1BDU80R8R808V9R@0l?3`>ooooo`0[05EEE@1YJFT0
FUYJ03Ph>011@D40EEEE06ATI012@T807alO05iNGP11@D40:2PX03Ph>01[Jf/0OWin05iNGP0/;2`0
@D5104E5A@0n?Sh0<30`02PX:018B4P0A4A404e=C@1@D500:b/[01@D5012@T80C4a<04E5A@0Y:BT0
A4A404Y:BP1=CDd0=SHf03lo?`1:BTX0CTi>02LW9`0d=3@0ATI6051@D002055AD@1@05=CD`1EEED0
IVIV06MWI`1OGel0DE5A07UiN@11@D40DU9B05mOG`1CDe<0@D5103<c<`1<C4`0GemO06=SH`17AdL0
Cdm?05ADE00n?Sh0GEeM06EUI@1WIfL0Kfm_04];B`1gMgL0QhN705mOG`15ADD0Jf][05IFEP1bLW80
?clo04M7A`1QHF40Fe]K04A4A00j>SX0BTY:05ADE01;Bd/0BDU903`l?01;Bd/0>3Ph028R8P0j>SX0
AdM7051@D01>CTh0?Shn04Q8B015ADD0EeMG04Q8B01FEUH0Bd];03lo?`12@T80ADE505UIF@1=CDd0
DE5A03lo?`1FEUH0IVIV04A4A01UIFD0MGEe04i>CP1<C4`0@D5104e=C@1RHV80NWYj04m?C`1<C4`0
IfMW034a<@0j>SX206IVIP0g09NGU`1WIfL0M7Ad04M7A`1_Kfl0PX:204U9B@2/[:`0EeMG05EEE@1L
G5`0@T9203/k>`1KFe/0GUiN051@D00l?3`0KVi^075aL@1?Cdl0][Jf08n?S`1eMGD0K6a/04e=C@1i
NGT0^KVi079bLP1mOGd0TI6A0>g]k@3Hf=P0E5AD061PH01EEED0I6AT05=CD`2YZJT0<c<c071`L01a
LG40G5aL05QHF018B4P0NGUi06EUI@1ZJVX0PX:204m?C`1KFe/0FEUI03Hf=P2;Rh/0Ph>303/k>`02
06UYJ@0c075aL@1VIVH0W9bL05iNGP1/K6`0DE5A05EEE@1JFUX0Bd];04a<C01KFe/0EUIF04A4A01/
K6`0A4A406ATI01@D500C4a<04Y:BP1@D500@T9203`l?01<C4`0@d=304A4A00j>SX0BDU905mOG`19
BDT0E5AD05mOG`1GEeL0CDe=06YZJP0X:2P0HF5Q04E5A@1lO7`0;bl_05iNGP1UIFD0JVYZ04Q8B014
A4@0N7Qh059BDP1mOGd0KVi^079bLP1DE5@0Hf=S0080FUYJ01l0IfMW06YZJP1hN7P0F5QH05ADE01J
FUX0I6AT04E5A@1DE5@0S8b<02XZ:P1IFET0=3@d04m?C`19BDT0XJ6Q061PH024Q8@0MGEe0:>SX`1f
MWH0YJFU05ADE01^KVh0Ng]k0:^[Z`26QXH0<c<c06][J`0`<300G5aL0080H61P00P0EeMG05=CD`2>
SXh0VIVI069RHP2YZJT0O7al06e]KA3ooooo000>ooooo`1>0::RXP1ADE40HV9R08R8R020P800A4A4
05YJFP2MWId0N7Qh04m?C`0k>c/0EeMG079bLP0h>3P0C4a<08^;R`17AdL0WIfM07alO00a<C40De=C
05eMG@0W9bL0Jf][04U9B@0^;Rh0CTi>04Y:BP0n?Sh0Rh^;03/k>`0f=SH0BTY:02l_;`0j>SX0QHF5
03@d=00f=SH0>c/k03Ti>@1IFET0aLG506YZJP0Z:RX0QhN704I6AP2SXj<0_Kfm02PX:016ATH0Vi^K
06EUI@2^[Zh0<C4a01/K6`0`<300QXJ607AdM01lO7`06ATI01lO7`13@d<0R8R809NGU`1]KFd092@T
020P800[:b/0Kfm_07=cL`1CDe<0=SHf04A4A00e=CD0Ph>305aLG01GEeL0:2PX0P1;Bd/01019BDT0
EUIF065QH@0c<c<208^;R`3003Lg=`1NGUh0EEEE0410@02EUID0F5QH03hn?P1dM7@0<S8b0492@P20
P800DE5A04E5A@1UIFD09bLW04U9B@1YJFT0<S8b05EEE@1WIfL08B4Q05UIF@2_[jl0DE5A06=SH`1=
CDd0;Bd]03hn?P2FUYH0A4A406EUI@12@T80?Cdm09:BTP2YZJT0CDe=051@D012@T80BTY:0:BTY034
a<@0CDe=0492@P1=CDd0DU9B0;>c/`1kNg/0;Bd]04];B`1@D500M7Ad0:^[Z`0c<c<0;2`/0451@@1H
F5P0Lg=c08f=S@0U9BD0?Shn05YJFP1SHf<0RXZ:08n?S`0T92@0Hf=S0:b/[01NGUh0Ph>305]KF`0/
;2`0Jf][0;bl_01UIFD0Kfm_09JFUP0m?Cd0MGEe0:FUY@1EEED0X:2P06YZJP1@D500[Jf]0=GEe@1c
Lg<0`/;204m?C`1LG5`0U9BD092@T01YJFT0PX:203hn?P1FEUH0S8b<06][J`23Ph<0Shn?01/K6`1S
Hf<0d];B0861P@1aLG40>c/k04i>CP2GUiL0a/K605eMG@1@D500?clo06QXJ02i^KT0LG5a04i>CP0n
?Sh0G5aL0<71`@1oOgl0L71`05MGE`0o?cl0O7al08^;R`1=CDd0JFUY03dm?@1dM7@0]KFe06QXJ02N
WYh0JFUY05aLG02YZJT0WinO04=3@`1VIVH0JVYZ06=SH`32`/80TI6A03Xj>P1CDe<0MWIf0:FUY@3:
b/X0=CDe034a<@1OGel0ADE50<?3``3:b/X09bLW01lO7`1^KVh0I6AT0:^[Z`2WYjL0;bl_02/[:`22
PX80RHV908f=S@1LG5`0=3@d0410@029RHT0NGUi071`L00U9BD0;Rh^05EEE@2OWil0NGUi06ATI01Q
HF40<30`04e=C@2BTY80SXj>04Y:BP2DU9@0>SXj03Ph>01kNg/0MWIf04E5A@1XJ6P0;bl_04i>CP27
QhL0Kfm_3_ooool02P1HF5P0IVIV06QXJ00o?cl0BDU903Ti>@16ATH0De=C03De=@1<C4`202l_;`04
05IFEP1BDU80@d=305IFEP80FEUI02T0IFEU05IFEP0g=cL0>CTi06UYJ@0i>CT0De=C08V9R@1;Bd/0
>CTi059BDP1WIfL0EeMG03Lg=`1BDU80E5AD06IVIP1KFe/0De=C06YZJP1EEED0;bl_065QH@1IFET0
A4A404];B`1MGEd0@D5103`l?01:BTX0ATI603Xj>P0Y:BT0ATI605UIF@1@D500?Shn01lO7`0m?Cd0
@T9205ADE002059BDP2Q04m?C`12@T80?clo03<c<`1:BTX0A4A404I6AP1OGel0NGUi03lo?`1LG5`0
D51@04i>CP1=CDd09BDU0410@014A4@0B4Q804A4A00i>CT0>SXj03Hf=P11@D40@410059BDP14A4@0
;Bd]04A4A01=CDd0@d=304e=C@1IFET0D51@03/k>`0m?Cd0=SHf03hn?P0o?cl0E5AD06][J`1FEUH0
DE5A05QHF01<C4`0Bd];06MWI`2:RXX0De=C05UIF@1SHf<0KFe]075aL@1lO7`0JVYZ06a/K01=CDd0
De=C05aLG012@T80HF5Q07emO@1CDe<0>3Ph05MGE`2KVi/0KFe]0;Zj^P1]KFd0O7al03`l?01LG5`0
Fe]K03De=@2m_Kd0M7Ad06UYJ@1bLW80GUiN06=SH`1QHF40I6AT06MWI`18B4P0De=C04A4A01oOgl0
^k^k05IFEP1ADE40WYjN05EEE@1CDe<0MgMg05UIF@2QXJ40NGUi08^;R`2:RXX0IfMW05MGE`1:BTX0
KVi^04M7A`1fMWH0Jf][0:JVYP2ATI40GEeM04U9B@0I6AT0GemO04M7A`0n?Sh0GemO071`L01hN7P0
IVIV05=CD`1bLW80QHF506=SH`2GUiL0ATI604Q8B01?Cdl0UIFE05]KF`13@d<0Fe]K05YJFP1kNg/0
@41005]KF`1TI6@0KVi^075aL@2BTY80G5aL05]KF`1;Bd/0I6AT04m?C`1CDe<0=CDe05MGE`1XJ6P0
=3@d03`l?00h>3P0C4a<04E5A@1:BTX0I6AT051@D01;Bd/0AdM704I6AP13@d<0MGEe03dm?@26QXH0
0P1DE5@02`1MGEd0;2`/04=3@`12@T80GemO04m?C`1SHf<0QhN707emO@1/K6`0GUiN0080HV9R02T0
CDe=0820P01fMWH0@T9204Y:BP1MGEd0IFEU02XZ:P1KFe/0Ogmo04E5A@2:RXX0MGEe07inOP1MGEd0
LW9b06e]K@2YZJT0Jf][07]kN`1VIVH0];Bd05MGE`2NWYh0OWin09NGU`27QhL0HF5Q0:NWY`1PH600
YjNW0:6QX@1WIfL0CTi>055AD@1UIFD0LG5a07]kN`1cLg<0PX:208B4Q00@ooooo`003_ooool0@P32
`/80E5AD05QHF01kNg/0[Zj^0410@017AdL0De=C08>3P`2HV9P0:RXZ04M7A`1QHF40Hf=S07inOP2H
V9P0@T9205UIF@1kNg/0HF5Q0:n_[`2<S8`09RHV05ADE01^KVh0H61P09jNWP0W9bL0;Rh^059BDP0i
>CT0IVIV075aL@0B4Q80=CDe04m?C`1ADE40Vi^K0;2`/00I6AT0>CTi04m?C`27QhL0E5AD051@D00M
7Ad0<C4a08N7Q`1UIFD0G5aL02`/;00I6AT0IfMW0:6QX@0m?Cd0@T9203dm?@13@d<0MWIf05mOG`0e
=CD0B4Q803Ti>@1=CDd0LW9b02d];@80>SXj0<D0<C4a03hn?P1HF5P0:b/[03hn?P1EEED0<c<c05mO
G`1FEUH061PH04Q8B01KFe/0;Rh^05aLG01gMgL0820P04=3@`1VIVH0AdM706i^KP1UIFD0<30`05EE
E@1`L700IfMW09fMW@0_;bl0<c<c07]kN`1`L700QhN70:n_[`0S8b<09BDU071`L01HF5P0JVYZ0861
P@0R8R80:RXZ06m_K`1/K6`0[:b/05YJFP0U9BD0=cLg07]kN`1lO7`0YZJV04i>CP0V9RH0E5AD0:6Q
X@2DU9@0MWIf030`<01:BTX0HV9R08b<S024Q8@0Ng]k055AD@1DE5@0OWin07YjNP1VIVH0WIfM03Ti
>@1MGEd0Hf=S07]kN`1OGel0QXJ602DU9@1ADE40JFUY04=3@`1<C4`0QHF5038b<P1LG5`0]kNg05MG
E`1DE5@0J6QX03Ti>@16ATH0P82002/[:`1DE5@0EUIF02PX:013@d<0WYjN03/k>`1SHf<0A4A4034a
<@1_Kfl0Zj^[03Ph>019BDT0EUIF04i>CP1dM7@0OGem03Ti>@1HF5P0>SXj09BDU02n_[h0@d=302TY
:@1YJFT0K6a/09>CT`34a<@0<C4a02`/;021PH40OGem0;Bd]00U9BD0<S8b07moO`2g]kL0GUiN06UY
J@0N7Qh0A4A409RHV025QHD0NWYj03Ti>@19BDT0RHV90861P@2GUiL0MgMg02TY:@1iNGT0TI6A08N7
Q`2@T900;bl_04Y:BP1lO7`0RHV90<71`@1:BTX0AdM708R8R02KVi/0Ph>307UiN@0o?cl0@410071`
L033`l<0TI6A0:6QX@0l?3`0GEeM0:ZZZP1oOgl0Cdm?05iNGP0];Bd0CTi>0:FUY@1^KVh0<C4a08Z:
RP1BDU80ADE508R8R01ADE40AdM707]kN`15ADD0FUYJ0820P015ADD0LG5a05ADE00l?3`0Q8B4055A
D@0e=CD0K6a/06=SH`1GEeL0?Shn08Z:RP0W9bL0;Rh^03Hf=P0205IFEP0;08J6QP1PH6009bLW065Q
H@1mOGd0IFEU08n?S`1;Bd/0?3`l04I6AP1QHF403_ooool0P01NGUh0C4a<06e]K@17AdL0K6a/04M7
A`1CDe<0GemO04U9B@1EEED0DU9B04E5A@1?Cdl0D51@05IFEP1NGUh0HF5Q05ADE011@D40?Shn04Y:
BP0_;bl0DE5A03lo?`0k>c/0@D5104e=C@11@D40Fe]K065QH@1?Cdl0?Cdm05mOG`15ADD0K6a/05=C
D`1>CTh0AdM704];B`0M7Ad0>3Ph04Q8B016ATH0D51@05aLG01`L700Cdm?07]kN`1lO7`0LG5a04a<
C01VIVH0<c<c03`l?015ADD0BTY:06EUI@1XJ6P0De=C04A4A01IFET0Kfm_055AD@1;Bd/0IFEU07Ee
M@17AdL0?clo04U9B@1>CTh0D51@05eMG@1FEUH0IFEU08B4Q00i>CT0GemO05eMG@1KFe/0E5AD02XZ
:P0];Bd0?clo02TY:@0_;bl0DE5A03Xj>P0/;2`0;Bd]05QHF00c<c<0@41006i^KP1HF5P0>3Ph04m?
C`1QHF40>SXj03Ph>01SHf<0IFEU03Lg=`1:BTX0?3`l038b<P16ATH0T92@07EeM@0k>c/0DE5A06e]
K@0d=3@0ATI6079bLP1>CTh0HV9R059BDP1YJFT0FEUI06YZJP1mOGd0Shn?06EUI@1=CDd0QHF50;Ng
]`1`L700RXZ:0P1GEeL0>`1UIFD0RXZ:08B4Q01?Cdl0RHV905mOG`1XJ6P0Ti>C06][J`1VIVH0MWIf
06QXJ00h>3P0ATI605UIF@1GEeL0FUYJ07alO01[Jf/0CDe=06MWI`1=CDd0IFEU08F5Q@0e=CD0BTY:
04I6AP3Xj>P0Y:BT03dm?@1oOgl0G5aL07moO`1MGEd0Ti>C04Q8B01bLW80JVYZ04e=C@15ADD0D51@
0:JVYP29RHT0SXj>06EUI@1JFUX0DE5A06UYJ@13@d<0Ogmo071`L01ADE40EUIF04Y:BP1CDe<0B4Q8
05iNGP1SHf<0JFUY0080AdM701H0BDU904];B`1UIFD0J6QX03hn?P1@D500M7Ad065QH@24Q8@0De=C
051@D01kNg/0OGem04M7A`1MGEd0EUIF04];B`0n?Sh0>SXj055AD@15ADD0A4A40P1DE5@0?`1>CTh0
Cdm?04e=C@1ADE40[:b/05QHF024Q8@08b<S05QHF01MGEd0B4Q806EUI@1]KFd0F5QH03lo?`1KFe/0
EeMG05QHF019BDT0JVYZ04i>CP0o?cl0EEEE08n?S`1iNGT0I6AT05aLG02GUiL0Ph>306ATI01]KFd0
SHf=05UIF@1gMgL0@T9204U9B@1]KFd0T92@06QXJ02:RXX0G5aL0820P022PX80]kNg0861P@25QHD0
VIVI0;Rh^01lO7`0PH6109fMW@1/K6`0Kfm_05EEE@1UIFD0H61P065QH@1_Kfl0UiNG05mOG`1iNGT0
QhN7079bLP0@ooooo`003_ooool0H01lO7`0RXZ:0;Fe]@1PH600XJ6Q08F5Q@1lO7`0K6a/02d];@1i
NGT0G5aL071`L01_Kfl0<c<c06ATI02YZJT0G5aL04];B`0g=cL0;bl_06YZJP36a/H0MgMg04a<C00a
<C40EEEE08F5Q@1UIFD0=cLg04a<C017AdL0GUiN07alO00c<c<0;Bd]07EeM@0i>CT0VYZJ08F5Q@18
B4P0>c/k05QHF019BDT0MWIf06UYJ@0W9bL0:b/[03hn?P1?Cdl0R8R804];B`0K6a/0;bl_05IFEP1V
IVH0P82003dm?@1BDU80P82007EeM@1GEeL0YZJV030`<01OGel0JFUY04E5A@23Ph<0^;Rh02<S8`10
@400De=C04A4A01lO7`0`<3003Ph>00W9bL0EeMG04U9B@2OWil0[Jf]02PX:00i>CT0MgMg05UIF@2@
T900=3@d038b<P17AdL0U9BD06YZJP1lO7`08B4Q06QXJ020P800PH6106QXJ080B4Q804P0D51@079b
LP1IFET0>c/k04Y:BP0n?Sh0CDe=05]KF`1DE5@0H61P075aL@0l?3`0F5QH0820P00j>SX0CDe=05eM
G@0k>c/0Ogmo08Z:RP0V9RH0GUiN06MWI`0n?Sh0X:2P06YZJP0Z:RX0H61P07]kN`17AdL0NGUi09FE
U@0m?Cd0MGEe06ATI01<C4`0R8R8071`L00m?Cd0IFEU05aLG01QHF40/K6a09VIV@0g=cL0Q8B405]K
F`1[Jf/0`<3006=SH`1;Bd/0Z:RX06m_K`1hN7P0R8R8038b<P0];Bd0SHf=051@D02ATI40EUIF03@d
=01@D500OWin08:2PP2JVYX0?clo055AD@24Q8@0Z:RX07=cL`26QXH203Lg=`1008F5Q@3>c/h0JVYZ
096AT@0h>3P0>c/k07]kN`2^[Zh0H61P07QhN01KFe/0Jf][07QhN01^KVh0HV9R07YjNP10@400];Bd
0:RXZ00[:b/0E5AD03hn?P0o?cl0IFEU034a<@1@D500H61P03/k>`2CTi<0Lg=c04U9B@1PH600@T92
06UYJ@2SXj<0BDU9061PH018B4P0J6QX08B4Q018B4P0=CDe06][J`19BDT0_[jn08J6QP0^;Rh0ATI6
075aL@1]KFd0^KVi03Ph>00Y:BT0AdM706IVIP1dM7@0VYZJ03Ph>01;Bd/0Lg=c03lo?`1DE5@0VIVI
02h^;P80DE5A02P0KFe]04A4A02=SHd0FUYJ04=3@`1SHf<0MWIf05eMG@2c/k<0IVIV04m?C`1]KFd0
De=C0:FUY@1nOWh0@410061PH02JVYX0D51@08V9R@1XJ6P0@d=30492@P1eMGD0CTi>069RHP1VIVH0
5aLG0410@01;Bd/0K6a/06i^KP2MWId0=SHf04];B`1>CTh0HF5Q06EUI@1>CTh0;2`/3_ooool03@1G
EeL0EUIF06UYJ@1>CTh0B4Q804M7A`1:BTX0@T9202TY:@1ADE40;bl_034a<@1;Bd/00P1MGEd0:01G
EeL0CTi>061PH01EEED0B4Q803De=@0U9BD0DU9B04m?C`1;Bd/0EEEE04I6AP15ADD0GEeM07AdM01<
C4`0?Cdm05aLG01CDe<0EeMG05IFEP1BDU80De=C071`L00U9BD0B4Q8065QH@1>CTh0G5aL06a/K01b
LW80E5AD08Z:RP2FUYH0KFe]05mOG`2:RXX0BDU906MWI`1;Bd/2055AD@0Y03/k>`1=CDd0C4a<05]K
F`1OGel0D51@04I6AP0o?cl0IVIV05MGE`1@D500EeMG06=SH`1QHF40Jf][07AdM01;Bd/0F5QH03@d
=01?Cdl0@41004U9B@1>CTh0?Shn02<S8`0o?cl0>c/k04E5A@1=CDd0De=C03lo?`11@D40B4Q802h^
;P10@400>CTi04M7A`15ADD0<c<c03/k>`0n?Sh00P1>CTh0PP1XJ6P0BTY:04e=C@1<C4`0@d=305=C
D`19BDT0=SHf038b<P1GEeL0RHV902`/;01EEED0GUiN05MGE`1KFe/0<c<c04I6AP1OGel0QhN704e=
C@1RHV80EEEE04I6AP1CDe<0MWIf07]kN`3Jf]X0U9BD07MgM`17AdL0UiNG08J6QP1?Cdl0U9BD05iN
GP27QhL0GEeM04=3@`0m?Cd0De=C0861P@18B4P0<C4a051@D01;Bd/0IFEU09^KV`11@D40>SXj06][
J`0n?Sh0Y:BT0<c<c00h>3P0KVi^05]KF`2SXj<0VIVI02`/;01BDU80@D5106YZJP1^KVh0Q8B402/[
:`21PH40MGEe061PH01=CDd0AdM706ATI01MGEd0UIFE05mOG`1YJFT0EeMG051@D018B4P0/;2`0:JV
YP1SHf<0KFe]075aL@1XJ6P0B4Q80:FUY@20P800BDU904A4A01JFUX0Kfm_030`<00j>SX0BTY:051@
D01[Jf/0U9BD06=SH`1TI6@0BTY:05aLG01<C4`0Bd];03@d=01:BTX0EUIF03@d=018B4P0?clo04U9
B@1GEeL0CDe=05=CD`1HF5P0G5aL05]KF`1dM7@0BDU906EUI@0m?Cd0I6AT03dm?@23Ph<0JVYZ05EE
E@1NGUh0<C4a04i>CP1GEeL205]KF`0E04m?C`1FEUH0GEeM05iNGP1<C4`0KVi^09jNWP1^KVh0K6a/
06QXJ01YJFT0MgMg0492@P1^KVh0HF5Q02`/;01MGEd0AdM706a/K01>CTh0J6QX0080MGEe00`0IfMW
07alO02GUiL0clo?08b<S02FUYH0Ph>308f=S@1MGEd0A4A405UIF@0o?cl2069RHP090492@P0l?3`0
De=C08>3P`2HV9P0NWYj09VIV@2?Shl0O7al013ooooo000>ooooo`2M09RHV01FEUH0J6QX071`L01i
NGT0DE5A06][J`1dM7@0C4a<07inOP1BDU80H61P07]kN`0m?Cd0FEUI071`L01XJ6P0PH61051@D00l
?3`0GemO05=CD`1KFe/0Q8B403lo?`11@D40IFEU06YZJP1>CTh0MWIf03Lg=`1?Cdl0IfMW04I6AP0k
>c/0XJ6Q04a<C010@400J6QX03Ph>01?Cdl0SHf=075aL@0o?cl0M7Ad04a<C027QhL0RHV903<c<`1I
FET0P82004a<C02>SXh0GEeM02HV9P0k>c/0I6AT05QHF01]KFd09bLW03hn?P1EEED0I6AT07IfMP1G
EeL0<S8b03dm?@1eMGD0KFe]065QH@1YJFT09bLW04I6AP1dM7@0JVYZ04];B`1NGUh0>CTi06IVIP2D
U9@0IVIV04];B`1ZJVX0@41005QHF01fMWH0?3`l05ADE01@D500CDe=08J6QP1UIFD0Bd];06e]K@0k
>c/0Fe]K07QhN00n?Sh0De=C06UYJ@1<C4`0De=C07moO`0k>c/0HV9R05IFEP0h>3P0G5aL09fMW@15
ADD0DE5A05UIF@14A4@0Jf][09nOW`1ADE40J6QX05eMG@0e=CD0M7Ad0:VYZ@1CDe<0H61P0861P@10
@400MWIf08N7Q`1>CTh0De=C06EUI@1LG5`0T92@06e]K@1;Bd/0HV9R061PH020P800Xj>S04Y:BP0k
>c/0J6QX09JFUP1eMGD0RXZ:03Ph>014A4@0IVIV0820P01lO7`0T92@03Hf=P18B4P0JVYZ06a/K01Q
HF40IVIV03hn?P0206a/K01C04I6AP1kNg/0/k>c03Lg=`22PX80UiNG03dm?@1mOGd0R8R805ADE02n
_[h0XJ6Q07QhN021PH40U9BD04I6AP2<S8`0LW9b06a/K02<S8`0CDe=04a<C01jNWX0N7Qh07=cL`2Q
XJ40D51@07UiN@33`l<0M7Ad07moO`1NGUh0GemO0;6a/@2FUYH0@41006][J`16ATH0MgMg0;Jf]P1H
F5P0FUYJ071`L024Q8@0N7Qh03/k>`1HF5P0L71`04];B`2;Rh/0A4A404=3@`1TI6@0BTY:06=SH`2B
TY80CTi>07IfMP1nOWh0EUIF09BDU01aLG40>CTi07QhN02=SHd0IFEU0:2PX013@d<0:BTY079bLP1W
IfL0QHF50;^k^`1:BTX0<30`07AdM017AdL0ZZZZ06m_K`0W9bL0>SXj061PH01NGUh00P2<S8`08P0T
92@0:b/[07emO@21PH40LW9b059BDP0o?cl0GUiN0:f][@1]KFd0GemO03`l?00m?Cd0HV9R08B4Q018
B4P0CTi>06e]K@0_;bl0HF5Q07MgM`1SHf<0D51@09^KV`1:BTX0A4A407moO`1MGEd0Bd];06m_K`1:
BTX0L71`08b<S01ADE4>ooooo`1V05QHF01JFUX0MWIf0410@01LG5`0A4A405IFEP1NGUh0@T9205=C
D`11@D40@d=305eMG@1?Cdl0DE5A06YZJP1UIFD0CTi>059BDP1@D500@D5103Ph>01PH600A4A404M7
A`1BDU80AdM70451@@1IFET0Hf=S05MGE`0o?cl0GemO05EEE@1UIFD0H61P04m?C`1BDU80Bd];02LW
9`18B4P0De=C0492@P1?Cdl0G5aL05=CD`17AdL0H61P07inOP1CDe<0@41005aLG01BDU80FEUI04E5
A@1;Bd/0IFEU06UYJ@1?Cdl0AdM705EEE@1aLG40CDe=0451@@1EEED0EUIF04Q8B019BDT0DE5A06=S
H`1TI6@0JFUY06=SH`1KFe/0GUiN030`<01BDU80B4Q805MGE`18B4P0?clo02d];@12@T80=3@d034a
<@1EEED0C4a<03De=@13@d<0Fe]K04A4A01?Cdl0Kfm_065QH@0o?cl0DE5A05=CD`12@T80?Cdm06EU
I@1_Kfl0=SHf0P13@d<0E@0c<c<0>CTi071`L01XJ6P0@D5104e=C@1UIFD0>3Ph05MGE`1fMWH0HV9R
07moO`1;Bd/0E5AD05]KF`1TI6@0GUiN07EeM@1GEeL0=SHf05eMG@2LW9`0EEEE0=?Cd`1lO7`0OGem
04=3@`2NWYh0OWin04E5A@2>SXh0EeMG05eMG@1VIVH0De=C07alO01TI6@0D51@04i>CP14A4@0GUiN
03@d=01TI6@0PH6104Y:BP0l?3`0OGem04E5A@1XJ6P0S8b<04];B`2=SHd0FEUI0<o?c`2^[Zh0IVIV
08n?S`1OGel0Hf=S06e]K@26QXH0@D5108Z:RP1bLW80C4a<0451@@0k>c/0Ng]k06=SH`1`L700?Shn
05ADE01@D500MGEe02XZ:P2WYjL0R8R80451@@1YJFT0IVIV071`L01DE5@0HF5Q06=SH`1cLg<00P1>
CTh02`1BDU80>c/k05YJFP1RHV80EeMG051@D01QHF40Lg=c09>CT`1PH600DU9B0080EeMG01d0<S8b
05UIF@1NGUh0>c/k03dm?@1:BTX0DE5A04m?C`0o?cl0GemO06ATI01;Bd/0C4a<04M7A`1CDe<0NWYj
03Ph>01kNg/0=cLg065QH@1IFET0;2`/05YJFP1CDe<0AdM7051@D01HF5P0K6a/04m?C`0205ADE00[
04Y:BP1BDU80EEEE08B4Q01ADE40ATI605MGE`1WIfL0JFUY02LW9`1ZJVX0QHF50451@@1TI6@0EeMG
06=SH`1bLW80OGem09fMW@1_Kfl0M7Ad06YZJP1/K6`0R8R806MWI`2JVYX0N7Qh09BDU01LG5`0@T92
06m_K`1HF5P0MGEe05ADE01<C4`0CTi>05QHF01XJ6P0MgMg06m_K`1jNWX0MGEe0820P00@ooooo`00
3_ooool06`2>SXh0IFEU07MgM`1HF5P0YjNW05QHF01CDe<0CTi>04A4A024Q8@0=3@d04U9B@18B4P0
DE5A06YZJP2IVIT0>SXj05=CD`1JFUX0EeMG08:2PP2?Shl09BDU04a<C01>CTh0KVi^08^;R`0203Xj
>P3<04Y:BP1DE5@0LG5a071`L00T92@05aLG0410@01KFe/0TI6A0<G5a@0I6AT0;2`/03De=@1?Cdl0
Q8B407=cL`0S8b<0>CTi05ADE01OGel0S8b<04A4A00M7Ad0CTi>07YjNP1LG5`0HV9R03dm?@18B4P0
LW9b055AD@0d=3@0Hf=S02DU9@19BDT0GUiN03De=@1BDU80IVIV028R8P11@D40OWin04a<C01>CTh0
LG5a03Lg=`16ATH0GemO028R8P11@D40FEUI02h^;P0h>3P0De=C02d];@1<C4`0:2PX0492@P1<C4`0
Ph>303lo?`1;Bd/0>SXj04e=C@1UIFD0@D510492@P13@d<0E5AD0861P@1nOWh0;2`/03Xj>P1=CDd0
LW9b08Z:RP1]KFd09bLW03Ti>@1SHf<0KVi^08n?S`2KVi/07AdM03Ti>@0o?cl0J6QX0<30`020P800
;Bd]04];B`1LG5`0GEeM0;6a/@1MGEd0?clo08:2PP1nOWh0NWYj0:n_[`1^KVh0@d=306YZJP1hN7P0
FUYJ0;Rh^01`L700ADE509BDU02GUiL0KVi^0<30`01EEED0?clo0:b/[02<S8`0MgMg0<;2`P1;Bd/0
DE5A09VIV@2OWil0Ng]k07moO`0S8b<0?Cdm08Z:RP1NGUh0[:b/06ATI00Q8B40<c<c0;Rh^01[Jf/0
SXj>04m?C`0e=CD0?clo09ZJVP1EEED0Kfm_04i>CP0g=cL0FEUI08^;R`1IFET0Hf=S03hn?P0o?cl0
PH6109JFUP0a<C40I6AT04I6AP12@T80W9bL03lo?`0^;Rh0GemO07AdM03?cll0EEEE02`/;01GEeL0
TY:B05EEE@2<S8`0:b/[04U9B@1RHV80FEUI0:>SX`1UIFD0CDe=079bLP1UIFD0XZ:R09NGU`0e=CD0
Hf=S079bLP1eMGD0SXj>038b<P19BDT0KFe]08:2PP2[Zj/0LW9b03/k>`1EEED0WinO08n?S`2XZ:P0
:2PX04Q8B02WYjL0Ogmo04];B`2WYjL0:BTY05YJFP2j^[X0Lg=c0P1CDe<0:`0k>c/0M7Ad0<O7a`1R
HV80BDU90<30`00n?Sh0EUIF09fMW@1kNg/0CDe=0:BTY01/K6`0GUiN0;Bd]015ADD0IfMW08J6QP0o
?cl0GEeM07=cL`13@d<0SHf=0:NWY`11@D40:RXZ09>CT`1?Cdl0CDe=06EUI@0_;bl0;Bd]09FEU@1K
Fe/0CTi>07UiN@0_;bl0Bd];0:>SX`1]KFd0@T92061PH01=CDd03_ooool0:`1IFET0GUiN0820P01O
Gel0JFUY04U9B@1>CTh0BDU903`l?01FEUH0>CTi03dm?@1SHf<0I6AT05MGE`22PX80CTi>04];B`17
AdL0BDU904A4A00_;bl0DE5A05EEE@1KFe/0I6AT04m?C`1WIfL0OGem07UiN@1?Cdl0C4a<05aLG017
AdL0Hf=S04m?C`1IFET0D51@06=SH`0^;Rh0@T9205UIF@1<C4`00P1LG5`02@1YJFT0Cdm?069RHP23
Ph<0De=C06m_K`1YJFT0?Shn05mOG`0204a<C00506a/K01TI6@0CDe=04];B`1XJ6P00P1MGEd03`1f
MWH0CDe=07YjNP1XJ6P0De=C05]KF`1>CTh0CDe=05aLG014A4@0?clo07YjNP1IFET0MWIf04E5A@02
05EEE@0c03`l?019BDT0D51@03/k>`0X:2P0F5QH03lo?`12@T80@D5104M7A`16ATH0EEEE055AD@1K
Fe/0ATI605=CD`12@T80Bd];04Q8B01ADE40IVIV03hn?P16ATH0AdM704U9B@1HF5P0FEUI04m?C`13
@d<0DU9B07EeM@13@d<0AdM706IVIP1@D500IVIV03/k>`11@D40KVi^08f=S@1aLG40Ng]k04I6AP1=
CDd0T92@0<C4a02TY:@0inOW08j>SP1cLg<0@4100080JVYZ02l0CDe=09NGU`1?Cdl0I6AT05=CD`18
B4P0Fe]K059BDP20P800C4a<02TY:@1:BTX0=CDe06IVIP24Q8@0?Shn04E5A@1lO7`0@410059BDP1M
GEd0H61P07=cL`1`L700j^[Z0<71`@0o?cl0CDe=038b<P1UIFD0PH610820P00g=cL0TI6A06][J`1^
KVh09bLW02TY:@1MGEd0BDU905QHF01WIfL0A4A405aLG02MWId0GUiN08>3P`0206YZJP0R07YjNP17
AdL0BTY:04i>CP1ZJVX0>c/k04=3@`0U9BD0EEEE05aLG00V9RH0B4Q805]KF`1SHf<0EUIF06=SH`0j
>SX0IVIV05mOG`1nOWh0WinO08R8R018B4P0E5AD05]KF`17AdL0Bd];03Xj>P14A4@0Fe]K04];B`1D
E5@0Fe]K055AD@80De=C00X0C4a<04U9B@0/;2`0IVIV030`<01;Bd/0ATI6061PH01QHF40>SXj0P0f
=SH05012@T80Fe]K05aLG01DE5@0EeMG05IFEP1VIVH0CDe=05mOG`1LG5`0<C4a05IFEP1EEED0I6AT
03`l?01jNWX0SHf=04];B`1JFUX0ADE50P1PH6000`2>SXh0MWIf06IVIP0207QhN00E06i^KP2>SXh0
LG5a08^;R`1XJ6P0TI6A06m_K`16ATH0FUYJ04E5A@1OGel0EeMG03hn?P19BDT0DE5A07IfMP2SXj<0
MGEe08Z:RP1bLW80P820013ooooo000>ooooo`0a0861P@1_Kfl0a/K605IFEP28R8P0N7Qh08R8R01`
L700?3`l07YjNP1XJ6P0X:2P08R8R00P8200CDe=08^;R`1fMWH0R8R802XZ:P16ATH0EUIF09JFUP1g
MgL0RXZ:02DU9@0m?Cd0E5AD08B4Q012@T80De=C02@T901BDU80H61P051@D00g=cL0JFUY02DU9@1;
Bd/0TI6A07=cL`1<C4`0HF5Q02`/;00h>3P0X:2P03/k>`1=CDd0HV9R02TY:@0206][J`1V03De=@0f
=SH0@41002l_;`1>CTh0DU9B03Lg=`1BDU80>SXj05YJFP2`/;00>3Ph04E5A@1XJ6P0AdM708b<S02]
[Jd07alO02d];@1EEED0F5QH06a/K02/[:`0=cLg02l_;`1NGUh0DE5A07UiN@2?Shl09bLW02h^;P1X
J6P0Jf][09BDU00j>SX0:2PX04E5A@1oOgl0NGUi08Z:RP0K6a/0>SXj051@D01bLW80Kfm_05ADE00Q
8B40E5AD069RHP1WIfL0E5AD05QHF00];Bd0GemO05aLG01DE5@0DE5A08j>SP0X:2P0FUYJ06e]K@1:
BTX0FEUI06][J`0e=CD0EEEE08Z:RP16ATH0MGEe08N7Q`0_;bl0IFEU08>3P`0b<S80QXJ606IVIP15
ADD0JVYZ09:BTP17AdL0GemO02h^;P0k>c/0BDU9065QH@14A4@0EeMG030`<00o?cl0UYJF09VIV@0k
>c/0BDU903`l?01@D500HF5Q06YZJP13@d<0AdM706=SH`2YZJT0XJ6Q0P0Z:RX0N`16ATH0Cdm?0;Vi
^@2LW9`0:RXZ03`l?01RHV80DE5A0;jn_P1_Kfl0;Bd]04M7A`1EEED0N7Qh0<W9b@1IFET0@41005MG
E`1WIfL0IVIV0;^k^`14A4@0@d=305=CD`2ATI40OGem0;^k^`1MGEd0OGem09:BTP1bLW80Q8B40;Jf
]P0c<c<0Y:BT08b<S01RHV80W9bL0451@@0h>3P0T92@05aLG02TY:@0EUIF02h^;P14A4@0Ph>307Ee
M@1:BTX0<c<c04];B`1oOgl0IfMW08b<S00d=3@0H61P07alO01ADE40Cdm?07moO`1<C4`0N7Qh09VI
V@1DE5@0RHV906UYJ@1SHf<0V9RH061PH01=CDd0VYZJ04I6AP1VIVH0V9RH03lo?`1>CTh0VYZJ06YZ
JP1`L700Hf=S02l_;`14A4@0E5AD06QXJ01RHV80Z:RX0492@P0k>c/0Cdm?05iNGP1=CDd0_Kfm06m_
K`14A4@0CDe=06UYJ@2QXJ40`L7105EEE@1@D500BDU903lo?`2DU9@0_[jn03hn?P0h>3P0K6a/065Q
H@2?Shl0ZJVY03lo?`0Z:RX0?Shn07MgM`2MWId0`l?305MGE`0V9RH0?clo07EeM@1nOWh0LG5a03Ph
>00>ooooo`3205IFEP1lO7`0N7Qh04U9B@1LG5`0H61P06e]K@1bLW80@D5106UYJ@0i>CT0ATI6055A
D@0o?cl0ADE504m?C`1PH600EeMG05eMG@1UIFD0>SXj02XZ:P1KFe/0ATI604m?C`1MGEd0ATI603dm
?@1VIVH0Rh^;05IFEP16ATH0IfMW05iNGP1/K6`0EUIF059BDP1JFUX0BTY:02LW9`1?Cdl0GemO05MG
E`1EEED0BTY:03Xj>P0d=3@0EUIF08R8R01UIFD0Bd];079bLP18B4P0DU9B069RHP1WIfL0HF5Q079b
LP1QHF40L71`07AdM01QHF40D51@071`L01<C4`0JFUY07MgM`1UIFD0@T92061PH01GEeL0K6a/07If
MP1:BTX0Bd];02HV9P1;Bd/0@T9204Q8B016ATH0GUiN03@d=012@T80DU9B03lo?`1KFe/0EEEE05IF
EP0n?Sh0GEeM03<c<`1?Cdl0L71`06QXJ00k>c/0De=C055AD@1>CTh0B4Q806UYJ@25QHD0C4a<04Y:
BP13@d<0I6AT03dm?@1TI6@0OGem03hn?P1DE5@0MWIf04M7A`1<C4`0H61P03Xj>P1oOgl0?clo04m?
C`1>CTh0FUYJ0410@01EEED0O7al04e=C@1CDe<0OWin05mOG`2TY:@0Ph>307moO`1@D500Ti>C07in
OP1MGEd0XZ:R069RHP26QXH0Hf=S069RHP2j^[X0OWin05=CD`1ZJVX0FUYJ05eMG@13@d<0MGEe07em
O@1SHf<0=SHf07emO@0a<C40JVYZ07MgM`1XJ6P0S8b<03hn?P1kNg/0EUIF03De=@2?Shl0BDU905UI
F@1lO7`0RXZ:01dM7@2OWil0GUiN04I6AP0S8b<09bLW06m_K`0h>3P0FUYJ05EEE@1ADE40BDU905EE
E@10@400T92@06a/K01=CDd0Hf=S051@D01ADE40Cdm?06a/K00f=SH0Jf][02`/;017AdL0Fe]K02LW
9`1KFe/204A4A00h06MWI`1iNGT0M7Ad07]kN`11@D40Bd];05aLG017AdL0@41007EeM@1]KFd0=cLg
0492@P14A4@0?3`l04];B`14A4@0BTY:059BDP0m?Cd0D51@0410@01=CDd0EeMG02PX:02RXZ80DE5A
05UIF@1ADE40=3@d04I6AP0n?Sh0E5AD04Q8B01UIFD0TI6A04Y:BP1GEeL0E5AD04M7A`1<C4`0DE5A
06e]K@1WIfL0H61P06IVIP1GEeL0FEUI04=3@`1bLW80PX:204=3@`1GEeL0?clo04i>CP1dM7@209NG
U`0H05ADE01EEED0FUYJ06a/K01iNGT0D51@06MWI`1MGEd0MgMg04i>CP0g=cL0GUiN04e=C@1^KVh0
Cdm?0410@01<C4`0FEUI065QH@1BDU80P82006YZJP21PH40QHF54?ooool000kooooo00P0TY:B059B
DP1iNGT0HV9R06m_K`18B4P0GUiN08n?S`80DU9B00/0CTi>06QXJ02=SHd0>SXj03Ph>01LG5`0EEEE
0:FUY@1LG5`0:BTY051@D00203`l?00j09JFUP15ADD09BDU03@d=01YJFT0NGUi08:2PP0M7Ad09BDU
04E5A@1VIVH0IFEU0<g=c@0f=SH08B4Q05iNGP1PH600MGEe0<;2`P10@400:b/[05=CD`18B4P0S8b<
0:NWY`0N7Qh0<30`06QXJ01GEeL0NGUi05QHF00L71`0>SXj04a<C01RHV80U9BD0410@00U9BD0Bd];
04Y:BP1cLg<0I6AT02XZ:P0k>c/0K6a/079bLP1SHf<0H61P01`L7018B4P0EUIF06UYJ@0e=CD0Bd];
03<c<`14A4@0Lg=c03lo?`80D51@0<<0Fe]K06=SH`1XJ6P0:b/[05ADE00d=3@0CTi>07EeM@1;Bd/0
=cLg06i^KP0];Bd0Fe]K06m_K`0e=CD0ATI606IVIP13@d<0I6AT07moO`0X:2P0DE5A05eMG@1ADE40
L71`0=OGe`0n?Sh0BTY:05IFEP0o?cl0QXJ60;jn_P11@D40C4a<06MWI`1CDe<0WIfM0<;2`P0f=SH0
E5AD06a/K018B4P0];Bd0;2`/00b<S80HV9R079bLP2=SHd0[jn_03<c<`0Z:RX0FUYJ06QXJ01hN7P0
RXZ:02`/;00];Bd0MGEe0:f][@2ATI40Ti>C03Lg=`17AdL0K6a/08b<S01dM7@0Q8B403dm?@12@T80
SHf=08B4Q01NGUh0Hf=S03`l?01ZJVX0]kNg06UYJ@2<S8`0O7al03Ph>01hN7P0`l?3051@D02:RXX0
Fe]K03dm?@2:RXX0_[jn05]KF`2HV9P0GemO03<c<`1UIFD0WinO051@D02FUYH0=CDe03<c<`1=CDd0
IVIV096AT@1RHV80>c/k05mOG`29RHT0LG5a07inOP13@d<0AdM70;^k^`2=SHd0<30`06MWI`1<C4`0
/K6a09nOW`0`<300?3`l08b<S038b<P0SHf=03<c<`1@D500EeMG05QHF034a<@0AdM703hn?P2<S8`0
I6AT07]kN`2c/k<0@D51055AD@23Ph<0GUiN0;^k^`23Ph<0Bd];09:BTP1lO7`0RXZ:0>3Ph00o?cl0
@D5107inOP1EEED0JFUY0;Zj^P0e=CD0HV9R096AT@1>CTh0YZJV06e]K@0m?Cd0FUYJ09FEU@1/K6`0
/K6a09BDU00Q8B40?3`l09:BTP2HV9P0QhN707alO00_;bl0EeMG09BDU01jNWX0Ogmo03lo?`0Q8B40
@T9207EeM@24Q8@0Cdm?05EEE@0g=cL0@D5108b<S01hN7P0De=C06ATI00a<C40?3`l07YjNP1OGel0
?3`l05eMG@0m?Cd0I6AT08Z:RP11@D403_ooool0IP1TI6@0JFUY06QXJ017AdL0CTi>0410@01:BTX0
DE5A03Ti>@17AdL0B4Q804A4A01JFUX0EUIF04m?C`1kNg/0Bd];04M7A`0c<c<0@T9204=3@`0b<S80
BDU903hn?P13@d<0D51@04E5A@18B4P0De=C059BDP1=CDd0Cdm?04Q8B017AdL0De=C065QH@1RHV80
ADE5051@D00h>3P0@D5104=3@`11@D40CTi>04E5A@18B4P0ADE505EEE@1`L700GemO069RHP1bLW80
FUYJ079bLP1=CDd0@d=305IFEP1KFe/0AdM705=CD`1gMgL0JFUY05eMG@28R8P0EEEE07AdM01RHV80
De=C020P801LG5`0P82007inOP1PH600KVi^06EUI@0/;2`0@T9204M7A`1?Cdl0@41004e=C@0f=SH0
>c/k03@d=00O7al0DE5A03<c<`1>CTh0?Cdm04m?C`0m?Cd0Bd];061PH01MGEd0FEUI04Y:BP16ATH0
CDe=055AD@1JFUX0EEEE04E5A@80CDe=02H0C4a<04U9B@1SHf<0R8R804=3@`1BDU80M7Ad05YJFP1B
DU80WinO05=CD`1iNGT0IfMW07=cL`1VIVH0Shn?05iNGP1iNGT0CTi>04I6AP1hN7P0YZJV08V9R@30
`<00Ti>C0:2PX01UIFD0Shn?06m_K`1YJFT0Y:BT06ATI01_Kfl0SHf=069RHP1GEeL0Jf][05MGE`80
B4Q808H0GUiN01XJ6P25QHD0PX:206UYJ@15ADD0RXZ:030`<02VYZH0/;2`03hn?P1BDU80GEeM0<K6
aP2UYJD0?Shn05YJFP14A4@0EEEE08>3P`20P800ADE504a<C01:BTX0R8R805QHF01;Bd/0PX:203dm
?@1NGUh0Lg=c051@D00g=cL0HV9R03De=@2/[:`0NWYj05iNGP1ZJVX0Fe]K05IFEP1:BTX0Jf][0492
@P15ADD0CDe=09NGU`2UYJD0?Shn06EUI@1LG5`0IVIV04e=C@1:BTX0ADE505eMG@1bLW80@T9207If
MP1`L7006a/K04m?C`15ADD0;Rh^03Ti>@1CDe<0ATI604e=C@0h>3P0D51@05]KF`13@d<0E5AD03/k
>`19BDT0JVYZ020P801IFET05aLG05IFEP0m?Cd0=cLg04Y:BP1@D500F5QH055AD@1@D500HV9R05aL
G01OGel0CDe=04m?C`1JFUX0FEUI07inOP1;Bd/0>c/k05UIF@1JFUX0MgMg0410@01KFe/0JFUY03Ti
>@1[Jf/0EEEE06EUI@1ADE40LW9b0:2PX01PH600Ph>306=SH`1JFUX0P82005ADE01ZJVX0FUYJ06AT
I01@D500Hf=S06a/K01SHf<0K6a/0410@01ADE40HF5Q06UYJ@1hN7P0QhN706IVIP1kNg/0GEeM06a/
K13ooooo000>ooooo`2A0;6a/@1EEED0M7Ad08^;R`2GUiL0>3Ph04a<C02FUYH0IVIV079bLP0o?cl0
GEeM0:JVYP1_Kfl0Jf][08Z:RP19BDT0EEEE09jNWP1eMGD0[Zj^06ATI00_;bl0GUiN0:BTY01NGUh0
UiNG04E5A@0o?cl0KFe]051@D01WIfL0A4A402h^;P0f=SH0Hf=S06YZJP1?Cdl0J6QX02<S8`0b<S80
KFe]08B4Q01@D500@T920410@019BDT0LG5a05QHF01=CDd0=CDe04Y:BP2;Rh/0]kNg04Q8B01NGUh0
?Shn04i>CP1mOGd0V9RH03Hf=P1LG5`0@D5106UYJ@2ATI40De=C03`l?01GEeL0ADE506ATI02<S8`0
;2`/02LW9`1[Jf/0LW9b05eMG@1eMGD0=3@d02d];@1XJ6P0BDU907AdM01kNg/0:b/[04M7A`2CTi<0
DE5A0::RXP1XJ6P0;Rh^06][J`1PH600IFEU09jNWP0_;bl0>3Ph07]kN`1KFe/0LW9b09FEU@0V9RH0
@D5106m_K`1QHF40PH6106IVIP0Y:BT0Cdm?07]kN`1aLG40[Zj^059BDP0Z:RX0Bd];07alO01gMgL0
Q8B403dm?@0d=3@0HF5Q0:b/[02:RXX0S8b<04];B`0j>SX0C4a<09:BTP1QHF40I6AT04i>CP1OGel0
SXj>08R8R016ATH0DU9B04a<C01@D500Ph>306MWI`1EEED0LW9b0410@01VIVH0Xj>S04i>CP0204e=
C@05059BDP1jNWX0UYJF04m?C`1MGEd00P1UIFD0O01XJ6P0Kfm_01lO7`19BDT0Cdm?04];B`2?Shl0
UYJF034a<@1ADE40FEUI05MGE`26QXH0YjNW03lo?`1CDe<0KFe]05UIF@2IVIT0`<300492@P19BDT0
N7Qh06i^KP3Cdm<0Q8B403Xj>P1`L700GEeM0:b/[02SXj<06ATI04Y:BP1]KFd0Ng]k0;jn_P0n?Sh0
CDe=07]kN`2d];@0I6AT07YjNP0Z:RX0KFe]0<C4a01dM7@0P82005aLG01CDe<0[:b/07]kN`1lO7`0
EUIF03Hf=P1/K6`0Kfm_07alO024Q8@0:2PX038b<P28R8P0Ng]k08f=S@0f=SH0<30`06EUI@2n_[h0
KVi^08^;R`0m?Cd0Cdm?07emO@25QHD0@T92069RHP0h>3P0HF5Q0;Vi^@1JFUX0@D51079bLP0d=3@0
RHV908>3P`0X:2P0>3Ph071`L00b<S80NWYj0<;2`P1IFET092@T07AdM01OGel0N7Qh06a/K00[:b/0
CTi>07UiN@1PH600UIFE08V9R@0U9BD0@41005mOG`1XJ6P0K6a/09FEU@0Q8B40=cLg05=CD`1OGel0
EEEE096AT@0h>3P0C4a<05eMG@1NGUh0G5aL0;6a/@14A4@0BTY:07=cL`18B4P>ooooo`1g05QHF01W
IfL0K6a/05=CD`1JFUX0BDU905ADE01NGUh0BTY:05mOG`13@d<0?3`l04a<C01NGUh0HV9R06=SH`1G
EeL0JVYZ06MWI`1CDe<0CDe=04];B`1NGUh0B4Q804e=C@1EEED0Cdm?059BDP1YJFT0OWin04a<C015
ADD0L71`05QHF027QhL0Cdm?06ATI01CDe<0F5QH03<c<`12@T80FEUI05IFEP1MGEd0Fe]K05aLG018
B4P0HF5Q05YJFP1=CDd0Hf=S05iNGP0g=cL0ADE5065QH@1YJFT0CTi>05mOG`1?Cdl0GEeM05YJFP1A
DE40DU9B05mOG`0a<C40Kfm_0;Ng]`1^KVh09BDU03hn?P1DE5@0K6a/05eMG@15ADD0NGUi055AD@1b
LW80=3@d04M7A`1UIFD0NGUi04e=C@16ATH0FUYJ03De=@1IFET0;Rh^051@D00c<c<0ATI602TY:@0n
?Sh0B4Q8069RHP1=CDd0@41004a<C00l?3`0DU9B05ADE01TI6@0De=C04Y:BP17AdL0B4Q803dm?@1I
FET0NGUi06ATI01ZJVX0U9BD05ADE029RHT0JVYZ04=3@`1bLW80?Cdm05QHF01CDe<00P1gMgL07`2_
[jl0I6AT05iNGP20P800YJFU07]kN`2][Jd0HF5Q09JFUP1SHf<0UIFE071`L01JFUX0Xj>S05aLG01R
HV80CDe=04Y:BP1JFUX0SHf=08^;R`1=CDd0<c<c08f=S@1>CTh0TI6A08>3P`15ADD0BDU907AdM00S
8b<00P1NGUh0K`1ADE40Hf=S059BDP3Bd]80XJ6Q04E5A@1hN7P0F5QH06a/K02<S8`0[:b/04m?C`26
QXH0Fe]K04];B`0e=CD0;Bd]069RHP0o?cl0LW9b08f=S@1BDU80A4A406][J`1;Bd/0^;Rh07inOP1T
I6@0LG5a03lo?`1<C4`0G5aL059BDP1HF5P0D51@02`/;01CDe<0GEeM03@d=019BDT0>3Ph0451@@1W
IfL0BTY:05IFEP1HF5P0Ph>305]KF`1MGEd0CTi>01DE5@1ADE40C4a<024Q8@19BDT0A4A403<c<`1=
CDd0@d=304Q8B01HF5P0BDU906IVIP11@D40>CTi05=CD`0K6a/0Jf][01lO7`1WIfL0F5QH06UYJ@1M
GEd0L71`075aL@1GEeL0B4Q806=SH`1GEeL0CDe=04U9B@1]KFd0H61P06YZJP1bLW80?Cdm0410@01T
I6@0SHf=08Z:RP0m?Cd0J6QX07moO`0];Bd0EUIF05mOG`1DE5@0F5QH07AdM02d];@0N7Qh05QHF01V
IVH0Jf][06i^KP1aLG40SHf=06ATI02FUYH0Cdm?04A4A002051@D00;05iNGP1>CTh0CDe=0451@@1M
GEd0NGUi09jNWP1_Kfl0JVYZ07=cL`1LG5`04?ooool000kooooo03`0QXJ606=SH`2DU9@0CDe=08B4
Q01SHf<0GemO04e=C@0a<C40Lg=c04E5A@1SHf<0Cdm?03Ph>00l?3`0IfMW04Y:BP1NGUh0=3@d05UI
F@1KFe/0K6a/034a<@1?Cdl0=3@d03hn?P1ZJVX0De=C02l_;`0h>3P0=cLg04i>CP1eMGD08b<S01hN
7P19BDT0EUIF06MWI`2GUiL0=SHf01lO7`1DE5@0FEUI06QXJ02FUYH09RHV04m?C`1dM7@0CTi>09NG
U`0g=cL06a/K05]KF`2HV9P0Fe]K0;>c/`1;Bd/07alO06EUI@2b/[8207=cL`1I03Xj>P16ATH0LG5a
07emO@26QXH0YjNW01lO7`0j>SX0PH61071`L01TI6@0TY:B03@d=00`<300EUIF05=CD`1cLg<0FEUI
02@T900k>c/0Cdm?04e=C@1?Cdl09BDU030`<01LG5`0Ph>304m?C`1;Bd/09RHV04Q8B01TI6@0Q8B4
04I6AP0e=CD0?Cdm06m_K`1fMWH0B4Q803@d=017AdL0@D5107=cL`29RHT0:BTY03@d=014A4@0=3@d
0861P@1lO7`09bLW02l_;`0m?Cd0CTi>0:NWY`1nOWh0<30`03Ph>01CDe<0MWIf08N7Q`1HF5P0;bl_
04U9B@18B4P0FUYJ08j>SP1gMgL0<S8b069RHP1[Jf/0KFe]0820P01iNGT0ADE5071`L01]KFd0UIFE
0<O7a`1/K6`0>3Ph06i^KP1NGUh0Vi^K0:f][@0j>SX0Hf=S09VIV@21PH400P2PX:00O@0l?3`0A4A4
08V9R@1NGUh0VIVI05YJFP0U9BD0@T920:n_[`1mOGd0^KVi05eMG@0g=cL0DU9B07UiN@1aLG40TY:B
03@d=00a<C40BTY:08>3P`1kNg/0Kfm_055AD@0m?Cd0CDe=0861P@1FEUH0VYZJ051@D02:RXX0[:b/
05iNGP1ADE40N7Qh03`l?02UYJD0P82003Xj>P1TI6@0OWin0451@@1dM7@0=CDe04];B`1LG5`0GemO
07inOP1/K6`0=SHf061PH01HF5P0Shn?09fMW@0Z:RX0G5aL05MGE`1dM7@0[jn_03Lg=`0M7Ad0HV9R
06m_K`3@d=00IfMW02LW9`0h>3P0N7Qh07emO@2CTi<0<30`04U9B@1]KFd0RHV907AdM02g]kL08B4Q
05IFEP2LW9`0F5QH06IVIP2JVYX07alO04Y:BP2HV9P0CDe=07=cL`32`/80@d=304a<C02:RXX0MgMg
04I6AP2XZ:P0B4Q8065QH@2=SHd0@41008>3P`1^KVh0=SHf06][J`2o_kl0BTY:069RHP1oOgl0;Rh^
04M7A`28R8P0ATI606MWI`1JFUX0:BTY04=3@`1CDe<0J6QX05YJFP1ADE40<C4a0410@025QHD0Jf][
06IVIP14A4@0ADE500kooooo00d0DE5A06ATI01^KVh0AdM706ATI01YJFT0GemO04M7A`0d=3@0ATI6
04=3@`0h>3P0IFEU0080DU9B00h0PH6105UIF@1@D500>3Ph04M7A`0j>SX0:b/[03lo?`0m?Cd0ADE5
059BDP0o?cl0A4A404e=C@<0BDU900P0EeMG06MWI`1NGUh0EEEE05YJFP1VIVH0Fe]K02d];@80DU9B
0:L0EUIF06MWI`1XJ6P0EEEE04i>CP1WIfL0QXJ605mOG`1?Cdl0OGem0451@@1]KFd0BDU904A4A015
ADD0?clo05IFEP1[Jf/0NWYj069RHP1BDU80S8b<03dm?@1bLW80NGUi07EeM@0m?Cd0GemO06MWI`2H
V9P0K6a/04];B`1mOGd0@41005aLG01RHV80GEeM051@D01JFUX0CDe=05=CD`1HF5P0=CDe06ATI01@
D500EEEE04a<C00];Bd0?3`l03lo?`0a<C40BDU9069RHP0g=cL0>3Ph04];B`1OGel0?3`l04U9B@1K
Fe/0?clo04i>CP1/K6`0@d=3055AD@1=CDd0?clo04i>CP25QHD0DE5A061PH01eMGD0?Cdm051@D01W
IfL0MgMg05UIF@1^KVh0Cdm?07AdM01BDU80:RXZ08N7Q`2YZJT0IFEU096AT@1@D500PX:203`l?01f
MWH0EEEE07AdM02VYZH0CTi>05MGE`1WIfL0Hf=S06IVIP1aLG40Hf=S04a<C00O7al0CDe=01<C4`1l
O7`0IfMW051@D00m?Cd0P82002@T901QHF40IVIV04A4A01`L700R8R80>SXj02KVi/0;Bd]07EeM@1Q
HF40MGEe092@T026QXH0@D5108Z:RP1fMWH0Fe]K034a<@11@D40Fe]K04];B`1GEeL0Hf=S04a<C018
B4P0QhN705EEE@22PX80LG5a07IfMP1lO7`0CTi>05eMG@1KFe/0J6QX04e=C@1OGel0B4Q8079bLP1l
O7`0:2PX04U9B@0c<c<0@410075aL@1?Cdl0J6QX06m_K`2QXJ40I6AT06YZJP1SHf<0:RXZ05=CD`1D
E5@0;bl_0410@00203dm?@0<051@D00/;2`0@D5105IFEP0g=cL0D51@04Y:BP10@400IVIV02PX:02N
WYh07QhN0P1=CDd0:P1;Bd/0EUIF05IFEP1YJFT0F5QH03Xj>P1=CDd0M7Ad06MWI`1LG5`0EeMG05AD
E01EEED0CDe=055AD@1gMgL0GemO03Xj>P1=CDd0<S8b05eMG@1kNg/0:2PX05UIF@1;Bd/0IFEU04m?
C`1iNGT0XZ:R05eMG@1UIFD0FUYJ09fMW@2QXJ40CDe=04E5A@1TI6@0Ph>306i^KP19BDT0HF5Q05QH
F080IVIV00T0Jf][04a<C01MGEd0RXZ:08:2PP1fMWH0N7Qh06IVIP1WIfL04?ooool000kooooo0:<0
P82006QXJ02WYjL0Ng]k08:2PP1ADE40Hf=S07YjNP1IFET0NGUi04i>CP1^KVh0P82004i>CP1]KFd0
MgMg051@D024Q8@0HF5Q034a<@15ADD0GemO04Q8B01iNGT0E5AD02<S8`1FEUH0Ph>3059BDP1VIVH0
9RHV03Ph>01KFe/0ATI6051@D02IVIT0?Shn0451@@1VIVH0DE5A05IFEP2CTi<0@d=303De=@1CDe<0
F5QH06IVIP1bLW8092@T04M7A`1[Jf/0<c<c04e=C@1>CTh092@T0410@01SHf<0Fe]K06e]K@0X:2P0
=cLg061PH01VIVH0NWYj05eMG@0a<C40B4Q808f=S@1_Kfl0C4a<05MGE`0_;bl0BTY:09jNWP1IFET0
B4Q804I6AP15ADD0MgMg09BDU00h>3P0D51@04m?C`1CDe<0Lg=c065QH@0a<C40D51@0492@P1GEeL0
OWin0410@012@T80F5QH04];B`1QHF40TY:B0492@P18B4P0Lg=c04i>CP14A4@0KFe]030`<012@T80
F5QH03Ph>01SHf<0VYZJ03@d=01JFUX0EUIF04Y:BP25QHD0WIfM04=3@`1BDU80HF5Q04m?C`27QhL0
VIVI03dm?@1SHf<0Rh^;05=CD`2IVIT0Ogmo03dm?@1NGUh0QhN706i^KP2:RXX0=3@d03De=@1?Cdl0
KVi^07MgM`1`L7009BDU03Ph>01UIFD0Ph>30861P@2?Shl0=3@d03dm?@1dM7@0HF5Q06a/K01HF5P0
;Bd]05MGE`1_Kfl0JFUY06i^KP1/K6`0<30`05iNGP2MWId0De=C07QhN01aLG40Bd];0080P82002@0
Cdm?07UiN@18B4P0GEeM09FEU@1NGUh0@d=3069RHP1=CDd0BDU909JFUP1KFe/0D51@079bLP11@D40
A4A409^KV`28R8P0N7Qh075aL@1EEED0S8b<0;jn_P1FEUH0L71`04m?C`1DE5@0WIfM08j>SP1ADE40
IVIV04E5A@23Ph<0N7Qh03Lg=`1eMGD207=cL`1;08J6QP14A4@0Bd];07alO01UIFD0U9BD03Ph>01<
C4`0Lg=c06e]K@1QHF40Ogmo0410@01lO7`0Shn?07YjNP2LW9`0LG5a04];B`2=SHd0FEUI06m_K`2X
Z:P0FEUI0492@P1jNWX0Ng]k065QH@2TY:@0CDe=051@D023Ph<0J6QX092@T01dM7@0:b/[0410@01P
H600Cdm?071`L01[Jf/0<C4a04I6AP1oOgl0GEeM05EEE@1]KFd0CTi>07YjNP2PX:00IVIV05QHF01N
GUh0B4Q808>3P`2JVYX0G5aL04];B`1`L700>CTi07emO@2b/[80D51@055AD@1YJFT0Bd];07alO02Q
XJ40GEeM04m?C`1SHf<0DE5A08B4Q01^KVh0@d=300kooooo02<0GEeM07IfMP26QXH0CDe=06MWI`15
ADD0GemO059BDP0m?Cd0CTi>04E5A@0h>3P0EeMG051@D01?Cdl0M7Ad05YJFP1KFe/0D51@069RHP0g
=cL0>c/k059BDP12@T80BDU903lo?`1=CDd0B4Q808>3P`1ZJVX0F5QH04i>CP1`L700FUYJ06UYJ@02
06MWI`0A055AD@1@D5008B4Q03`l?01@D500EUIF05MGE`1FEUH0D51@04=3@`1EEED0GemO05EEE@1;
Bd/0HV9R04M7A`1PH6000P14A4@0Z@1DE5@0DE5A04M7A`1BDU80I6AT05ADE01EEED0HV9R03`l?01?
Cdl0Lg=c06a/K00i>CT0IFEU08V9R@2BTY80LW9b05mOG`1bLW80>3Ph05UIF@0i>CT0EEEE0451@@13
@d<0Bd];03`l?015ADD0>3Ph061PH00b<S80Bd];03Ph>019BDT0>c/k04Q8B01=CDd0BTY:04i>CP0i
>CT0@T9204A4A01JFUX0ATI605ADE01FEUH0Bd];04E5A@1EEED0Cdm?07=cL`1_Kfl0HV9R06UYJ@1h
N7P0D51@07YjNP26QXH0CTi>065QH@1;Bd/0N7Qh05aLG01HF5P0Kfm_07UiN@1RHV80>3Ph09>CT`1h
N7P0G5aL09jNWP1aLG40Rh^;04M7A`22PX80GEeM079bLP2CTi<0F5QH061PH01WIfL0D51@04Q8B01R
HV80IVIV04a<C00h>3P0N7Qh03hn?P2<S8`0GemO06QXJ014A4@0GUiN02l_;`1gMgL0Ng]k05mOG`1K
Fe/0GEeM0:ZZZP26QXH0BDU907EeM@1;Bd/0K6a/07moO`1XJ6P0@41008j>SP1UIFD0GUiN03Xj>P1<
C4`0MWIf03Ph>01YJFT0NWYj05eMG@16ATH0NWYj07EeM@2XZ:P0KVi^07=cL`1lO7`0GEeM05iNGP1@
D500Hf=S01<C4`1_Kfl09RHV06YZJP1eMGD0>3Ph06YZJP1KFe/0DE5A08J6QP1UIFD0LG5a05iNGP1Z
JVX0Cdm?069RHP1JFUX0=3@d075aL@1QHF40:2PX03/k>`1:BTX0>SXj055AD@0o?cl0F5QH05EEE@11
@D40EeMG04M7A`15ADD0E5AD02TY:@1bLW80<C4a05=CD`0j>SX00P1CDe<09@1>CTh0LW9b05ADE014
A4@0Ogmo06][J`1HF5P0GEeM059BDP1?Cdl0ATI605aLG01CDe<0=cLg05aLG01NGUh0LW9b03hn?P1U
IFD0JFUY04A4A01FEUH0AdM704U9B@21PH40Ph>309jNWP1MGEd0F5QH06i^KP2<S8`0O7al05]KF`1P
H600Fe]K06e]K@0j>SX00P1IFET0301HF5P0HF5Q06ATI01BDU80CTi>059BDP1bLW80N7Qh06m_K`29
RHT0NGUi09RHV13ooooo000>ooooo`1T0:VYZ@1JFUX0GEeM05aLG02VYZH0D51@04M7A`0n?Sh0?clo
07inOP0];Bd0C4a<05YJFP17AdL0KVi^0=oOg`0n?Sh0GUiN08f=S@1fMWH0T92@079bLP0e=CD0Cdm?
0:2PX01LG5`0V9RH07UiN@0`<300EEEE05YJFP2=SHd0_[jn03Xj>P0Y:BT0FUYJ0820P01GEeL0PH61
03<c<`0N7Qh0J6QX0:NWY`1PH600Ogmo02PX:01QHF40ZZZZ08b<S01]KFd092@T051@D01CDe<0J6QX
059BDP1YJFT0<C4a03<c<`25QHD0XJ6Q05MGE`2DU9@0@41005=CD`2PX:00J6QX06ATI01iNGT0=cLg
04Y:BP2BTY80;bl_0410@01MGEd0BDU9055AD@1<C4`0:RXZ03hn?P1?Cdl0B4Q806=SH`1TI6@08b<S
038b<P16ATH0I6AT0:BTY01VIVH0:BTY03lo?`16ATH0K6a/0;Vi^@1>CTh0;Rh^04m?C`12@T80S8b<
0<O7a`80<S8b0180IFEU05UIF@2OWil0Rh^;02<S8`0a<C40De=C06IVIP2e]KD0<30`03/k>`0m?Cd0
Cdm?0861P@2VYZH0>c/k03Ti>@0h>3P208>3P`2L08J6QP0j>SX0@D5105QHF01QHF40EEEE0:VYZ@1a
LG40>c/k06i^KP1lO7`0D51@09BDU01>CTh0?clo06e]K@22PX80LG5a08J6QP0V9RH0B4Q809BDU01U
IFD0KVi^07UiN@0U9BD0D51@0:RXZ01IFET0GEeM06=SH`0`<300D51@09JFUP17AdL0M7Ad04E5A@1;
Bd/0JFUY0:2PX01>CTh0NWYj04E5A@1gMgL0UYJF0<?3``19BDT0NWYj07emO@1SHf<0[:b/08B4Q00[
:b/0E5AD059BDP1kNg/0[Jf]06IVIP0^;Rh0>c/k06UYJ@2^[Zh0^KVi02PX:00o?cl0De=C06a/K02g
]kL0<S8b03hn?P1LG5`0PH6108:2PP2DU9@08b<S06MWI`1hN7P0ADE50:RXZ01VIVH0EeMG0;jn_P1j
NWX0M7Ad06EUI@0_;bl0NGUi096AT@2ZZZX0Zj^[03Ph>01BDU80R8R807MgM`2BTY80?Cdm02XZ:P23
Ph<0/K6a0:2PX02o_kl0B4Q804=3@`2QXJ40ZZZZ061PH01[Jf/0>c/k05ADE026QXH0HV9R04i>CP1[
Jf/0=3@d08N7Q`2o_kl0B4Q804e=C@1cLg<0<c<c06e]K@2_[jl0CDe=03De=@1MGEd0ADE507QhN01U
IFD0;Bd]04a<C01RHV80BDU907YjNP1MGEd09RHV03hn?P1RHV80>CTi07MgM`2TY:@09bLW030`<01E
EED0EUIF06MWI`2<S8`0ADE503`l?01oOgl0LW9b09^KV`37alL0>3Ph0492@P1TI6@0E5AD3_ooool0
=`1IFET0Lg=c05YJFP17AdL0De=C04m?C`1EEED0C4a<0451@@1EEED0@T9203Ph>01KFe/0OWin05YJ
FP1VIVH0CDe=05iNGP20P800IfMW0451@@0k>c/0@d=304e=C@1PH600>CTi04Q8B01BDU80G5aL07=c
L`1CDe<0@41006YZJP25QHD0Jf][04];B`1MGEd0I6AT0410@00V9RH0BDU904e=C@17AdL0C4a<05EE
E@16ATH0?Cdm04e=C@1_Kfl0E5AD05UIF@1/K6`0?Cdm05ADE01:BTX00P1EEED05`1CDe<0BDU905iN
GP1RHV80DU9B05QHF01[Jf/0?Shn03/k>`1;Bd/0K6a/03<c<`1IFET0Ph>308b<S02BTY80DE5A08Z:
RP12@T80NGUi051@D01HF5P0BDU90080EUIF03`0D51@05UIF@0h>3P0KVi^03/k>`1@D500>c/k04e=
C@13@d<0F5QH05UIF@1?Cdl0ATI60410@01ADE40Bd];06IVIP1DE5@0JVYZ055AD@1;Bd/0FEUI06][
J`1DE5@0Kfm_06i^KP1CDe<0CTi>07MgM`1;Bd/0R8R805mOG`0j>SX0EEEE04a<C01QHF40P82006m_
K`1TI6@0Jf][071`L00W9bL0Lg=c071`L01OGel0QHF504U9B@25QHD0@T9206=SH`0h>3P0Kfm_0820
P011@D40HF5Q06e]K@1MGEd0KFe]061PH01kNg/2051@D00G07QhN01:BTX0UYJF06UYJ@1_Kfl0GUiN
05QHF00_;bl0IVIV05IFEP1YJFT0M7Ad08N7Q`3dm?@0Xj>S059BDP1CDe<0G5aL08F5Q@1RHV80Bd];
05IFEP22PX800P1DE5@0B00g=cL0JFUY096AT@18B4P0HF5Q051@D01dM7@0D51@07=cL`1hN7P0`/;2
07UiN@1bLW80L71`08^;R`1ZJVX0Fe]K05mOG`0n?Sh0N7Qh02`/;02EUID0NGUi02l_;`1BDU80@D51
059BDP22PX80Ph>3075aL@1cLg<0Vi^K05UIF@1^KVh0EeMG02XZ:P19BDT0ATI6030`<00g=cL0ATI6
06a/K01EEED0;bl_05ADE01GEeL0ADE505ADE01CDe<0E5AD071`L00h>3P0Ng]k038b<P2EUID0De=C
06YZJP1JFUX0FEUI05MGE`17AdL0:RXZ06ATI01DE5@0D51@05eMG@1BDU80BDU906m_K`2:RXX0;bl_
03@d=080K6a/02<0N7Qh030`<01LG5`0BDU903<c<`1<C4`0F5QH05eMG@1BDU80Lg=c08b<S019BDT0
I6AT06QXJ02f][H0XJ6Q08:2PP39bLT0IfMW09RHV00o?cl0CDe=03Lg=`17AdL0BTY:059BDP1ADE40
Bd];06e]K@2IVIT0/k>c0820P01dM7@0G5aL09VIV@0@ooooo`003_ooool0I@1]KFd0UIFE0<[:bP1@
D500DE5A0820P02CTi<0HF5Q03Ti>@1MGEd0MWIf09fMW@1VIVH0;2`/04U9B@29RHT0R8R806ATI00c
<c<0?Shn05]KF`21PH40ATI606m_K`0Z:RX08B4Q05QHF02HV9P0A4A403hn?P0T92@0A4A404U9B@1K
Fe/0@D5104Q8B00h>3P0Bd];07QhN017AdL0=3@d04Q8B011@D40C4a<08F5Q@0^;Rh0?Cdm04];B`0i
>CT0W9bL04i>CP0I6AT0CDe=05mOG`1CDe<0UYJF04e=C@0H61P0C4a<05eMG@1YJFT0_[jn03Ti>@0b
<S80GEeM071`L02QXJ40/[:b01hN7P13@d<0KVi^06UYJ@2OWil0_kno02HV9P1;Bd/0GUiN05IFEP1i
NGT0EEEE020P801<C4`0OWin065QH@1hN7P0=cLg034a<@27QhL0^k^k06MWI`1RHV808B4Q034a<@1W
IfL0alO7069RHP0h>3P0;2`/03Xj>P1YJFT0Q8B40080CTi>08L0=cLg04U9B@1lO7`0BTY:05=CD`1c
Lg<0<C4a0;Zj^P2FUYH0;bl_0410@00n?Sh0ATI60;6a/@1gMgL0=cLg05=CD`1HF5P0C4a<09BDU01Q
HF40<C4a06YZJP1IFET0>SXj08R8R02LW9`0=SHf05iNGP0o?cl0De=C07MgM`1aLG40FEUI061PH00_
;bl0HV9R0<[:bP1LG5`0?clo05QHF00j>SX0Ng]k0<o?c`10@400AdM707EeM@1bLW80Shn?0;Zj^P0Q
8B4092@T06IVIP1aLG40WinO07moO`0Z:RX0FUYJ069RHP1`L700/;2`03@d=00e=CD0QHF50:NWY`2B
TY80/[:b03Hf=P17AdL0UIFE0=3@d01[Jf/0RXZ:03De=@0[:b/0Ng]k08f=S@1SHf<0Lg=c03/k>`1U
IFD0UYJF06ATI01^KVh0VYZJ03Xj>P1lO7`0UYJF06EUI@2EUID0DU9B06UYJ@2OWil0G5aL06ATI01;
Bd/0?clo07=cL`2=SHd0Fe]K05eMG@18B4P0UYJF09ZJVP14A4@0EEEE03Ti>@1]KFd0Xj>S04Y:BP0d
=3@0EeMG05EEE@2HV9P0FUYJ020P801HF5P0G5aL07AdM035aLD0>3Ph03<c<`1NGUh0NWYj08Z:RP2?
Shl0:RXZ04e=C@1bLW80OWin07UiN@1eMGD0<C4a051@D01fMWH00P2=SHd09P2^[Zh0?Cdm04];B`1j
NWX0W9bL079bLP2g]kL0<30`04];B`2LW9`0FEUI07inOP2:RXX0<30`069RHP2i^KT0C4a<079bLP2>
SXh0?Shn02l_;`1_Kfl0De=C07UiN@17AdL08B4Q0410@01ADE40HV9R06][J`1hN7P0:b/[03<c<`1T
I6@0G5aL0861P@0`<300@D513_ooool0C@1OGel0N7Qh07emO@15ADD0IFEU05MGE`1RHV80ATI603lo
?`1=CDd0Cdm?0451@@1DE5@0FEUI055AD@1eMGD0Hf=S04];B`11@D40A4A402d];@13@d<0D51@03Xj
>P1>CTh0>CTi04m?C`1;Bd/0Fe]K07MgM`1LG5`0A4A405MGE`1>CTh0IVIV059BDP1LG5`0Bd];04Y:
BP0T92@0ATI604a<C01;Bd/0GEeM05MGE`0m?Cd0@d=3059BDP1lO7`0BTY:03`l?01QHF40@D5105EE
E@0l?3`0?clo05IFEP1BDU80=3@d03lo?`1HF5P0HF5Q04I6AP1NGUh0IFEU065QH@1XJ6P0UYJF065Q
H@1aLG40Fe]K04Q8B019BDT0Jf][05mOG`11@D40D51@0080E5AD00<0@41003/k>`1ADE400P17AdL0
`P1DE5@0KFe]03Lg=`17AdL0>SXj06IVIP10@400CTi>06YZJP18B4P0=3@d04e=C@1EEED0@41004Y:
BP1jNWX0GemO059BDP1=CDd0B4Q805aLG017AdL0Ti>C06i^KP1`L700FEUI06IVIP1FEUH0UIFE05iN
GP1<C4`0M7Ad04i>CP1kNg/0JVYZ07YjNP1PH600HV9R04a<C00n?Sh0Shn?08R8R02@T900_Kfm05eM
G@2/[:`0DE5A08:2PP1>CTh0QHF508N7Q`1[Jf/0Cdm?05YJFP1?Cdl0?clo05IFEP1PH600@410059B
DP1gMgL0<c<c09RHV01SHf<0NWYj04];B`1QHF40?3`l079bLP1RHV80L71`06EUI@1gMgL0YjNW09RH
V01<C4`0GemO04M7A`1aLG40M7Ad06m_K`0m?Cd0Q8B403Xj>P1FEUH0?Cdm0451@@1YJFT0<c<c059B
DP0i>CT0Hf=S03Xj>P29RHT0E5AD0;:b/P2<S8`0Rh^;09NGU`1gMgL0J6QX04Q8B017AdL0;Bd]061P
H01<C4`0DU9B065QH@1<C4`0HV9R05YJFP1FEUH0QHF506ATI01ZJVX0IVIV08N7Q`17AdL0KFe]051@
D018B4P0G5aL04m?C`0i>CT0?3`l069RHP1aLG40@d=304I6AP1JFUX0EeMG04i>CP1;Bd/0?Shn04];
B`1<C4`0:BTY0:BTY01>CTh0FUYJ05=CD`13@d<0E5AD030`<01ADE40@T9204E5A@1NGUh0Hf=S05YJ
FP1aLG40=3@d03Ti>@1XJ6P0HF5Q030`<01:BTX0Kfm_0:>SX`1oOgl0>c/k079bLP1WIfL0H61P05]K
F`10@400H61P06QXJ02CTi<0^[Zj05IFEP1]KFd0MGEe09RHV01`L700Hf=S0820P01TI6@0GEeM05aL
G01`L700GEeM04Q8B01ZJVX0HV9R04U9B@1UIFD0Kfm_0:b/[02e]KD0QHF508f=S@1GEeL0MGEe4?oo
ool000kooooo0:H0P82005EEE@26QXH0RXZ:07QhN00m?Cd0HF5Q09RHV00o?cl0GUiN03/k>`24Q8@0
[jn_02/[:`10@400DU9B03Xj>P2KVi/0CDe=02l_;`0Z:RX0De=C05EEE@2>SXh0?3`l02/[:`0k>c/0
Ng]k071`L02>SXh04Q8B030`<017AdL0EUIF07IfMP2^[Zh0@41003dm?@1LG5`0@T9206MWI`2/[:`0
EEEE03<c<`1OGel0C4a<06MWI`2NWYh0:b/[03Ti>@2;Rh/0CDe=06QXJ00[:b/0;Bd]04Y:BP1`L700
Cdm?06a/K00P82007QhN05MGE`1XJ6P0CTi>03`l?00H61P0Cdm?0:RXZ01]KFd0Fe]K03hn?P0X:2P0
ADE508^;R`1=CDd0=CDe03Xj>P0];Bd0HF5Q08J6QP0o?cl0:BTY0410@01bLW80X:2P04];B`0J6QX0
<30`071`L01]KFd0U9BD02LW9`0N7Qh0=cLg05ADE024Q8@0Fe]K01TI6@0f=SH0A4A405IFEP1RHV80
MGEe01TI6@1CDe<0Fe]K04i>CP22PX80Lg=c03<c<`1JFUX0SHf=05IFEP2CTi<0@41003/k>`1GEeL0
PX:205IFEP2BTY80E5AD03hn?P1VIVH0OGem07IfMP2^[Zh0IVIV02h^;P1ZJVX0_;bl08b<S02UYJD0
>3Ph03/k>`1UIFD0OGem08N7Q`2ATI40=3@d03Lg=`1NGUh0Ti>C07IfMP1kNg/0:RXZ04];B`2DU9@0
SXj>07EeM@0o?cl0>CTi05UIF@2SXj<0Kfm_05YJFP1MGEd0BTY:06i^KP2ATI40C4a<06EUI@1gMgL0
Cdm?09ZJVP2GUiL0@4100P1RHV80KP14A4@0[Zj^0:f][@0i>CT0BDU905=CD`1>CTh0Jf][05iNGP0Z
:RX0EEEE04a<C01]KFd0^KVi055AD@1YJFT0Hf=S04a<C02=SHd0bl_;0410@01UIFD0JVYZ06][J`2c
/k<0MGEe038b<P1WIfL0DE5A0;Bd]02ATI408B4Q05eMG@28R8P0Vi^K06=SH`0N7Qh0ATI609>CT`1I
FET0SHf=02d];@18B4P0K6a/075aL@1jNWX0UIFE03Hf=P1iNGT0VIVI04i>CP2BTY80E5AD04m?C`2n
_[h0R8R807UiN@22PX80?3`l059BDP2EUID0DU9B04Q8B02CTi<0=cLg06QXJ02WYjL0@T9207]kN`1G
EeL0=3@d06m_K`1YJFT0FEUI06ATI012@T80A4A405UIF@2GUiL0J6QX05MGE`10@400GEeM06QXJ02;
Rh/0ATI604];B`0l?3`0?clo04m?C`29RHT0CDe=03Ti>@12@T80AdM709jNWP1nOWh0;Bd]04E5A@17
AdL0GemO08n?S`2GUiL0@d=30451@@1HF5P0De=C0:b/[01?Cdl0<C4a3_ooool0201LG5`0DE5A06UY
J@1MGEd0FUYJ04E5A@1TI6@0EUIF0P1<C4`0K@12@T80=CDe05YJFP1_Kfl0FEUI06i^KP1DE5@0C4a<
051@D01SHf<0E5AD04U9B@0i>CT0>SXj04E5A@0h>3P0@D5104Q8B01PH600MgMg06e]K@1UIFD0N7Qh
05MGE`1OGel0G5aL065QH@1BDU80ATI602PX:019BDT0HF5Q05mOG`1ADE40H61P04U9B@15ADD0Cdm?
04U9B@1;Bd/0AdM705ADE00k>c/0AdM7030`<015ADD0?Shn04U9B@0e=CD0CTi>05QHF01>CTh0DE5A
061PH00c<c<0Bd];05]KF`22PX80?clo04e=C@1TI6@0MWIf055AD@0l?3`0G5aL03Lg=`19BDT0>SXj
04m?C`0];Bd0>SXj04=3@`0i>CT0DE5A04i>CP1=CDd0:2PX03hn?P0c<c<0?Shn03Lg=`18B4P0BDU9
03hn?P0e=CD0@41004I6AP0g=cL0CDe=04];B`1FEUH0D51@0410@01CDe<0F5QH04];B`2BTY80GEeM
03Ph>01WIfL0SXj>03lo?`1HF5P0LW9b07MgM`23Ph<0LW9b07QhN01ZJVX00P1/K6`0JP25QHD0Hf=S
04E5A@2[Zj/0Kfm_08j>SP33`l<0EeMG061PH01BDU80VYZJ05mOG`20P800GUiN03`l?017AdL0CTi>
04A4A01<C4`0PX:208F5Q@0m?Cd09BDU07emO@0f=SH0YjNW0410@01RHV80B4Q8061PH00X:2P0Ph>3
07emO@1^KVh0L71`08J6QP3Ph>00O7al06EUI@1jNWX0E5AD075aL@23Ph<0GemO04a<C027QhL0DE5A
05]KF`0I6AT0ATI608:2PP1;Bd/0HV9R04Y:BP1RHV80@T9205YJFP1TI6@0h>3P08n?S`1IFET0DU9B
04i>CP17AdL0>CTi06IVIP0W9bL0KVi^04m?C`1SHf<0Fe]K055AD@1dM7@0@d=304];B`1dM7@0?clo
08Z:RP1_Kfl0T92@04e=C@2g]kL0MgMg02PX:01IFET0EEEE02HV9P0];Bd0^k^k08Z:RP19BDT09bLW
04i>CP1ADE40?Shn03Xj>P0c<c<0IFEU06=SH`0Z:RX0J6QX02`/;01WIfL0C4a<04Y:BP1YJFT2055A
D@0a05=CD`10@400IVIV07AdM01ZJVX0LG5a0492@P1NGUh0Jf][06ATI016ATH0>SXj06QXJ029RHT0
MWIf051@D01EEED0B4Q802l_;`1LG5`0IfMW06QXJ01cLg<0X:2P0>3Ph025QHD0J6QX08F5Q@2=SHd0
QXJ608^;R`2PX:00GEeM06=SH`1>CTh0FEUI05mOG`1WIfL0PH61075aL@1IFET0B4Q805MGE`2l_;`0
YJFU06e]K@2CTi<0R8R808>3P`0@ooooo`003_ooool0:@2a/K40DU9B06IVIP2CTi<0/k>c03/k>`1?
Cdl0KVi^051@D02ATI40<c<c03De=@23Ph<0E5AD08R8R02UYJD0<S8b05YJFP2=SHd0OWin07moO`1T
I6@07AdM05QHF021PH40C4a<0;Fe]@1fMWH0;Bd]06MWI`12@T80@41005mOG`0j>SX0;bl_08N7Q`2<
S8`0IfMW06a/K00V9RH0<C4a0080OGem0>80EEEE04a<C00[:b/0HV9R0;Zj^P1QHF40?clo04Y:BP0f
=SH0MgMg075aL@0R8R80=cLg04e=C@1WIfL0VYZJ04M7A`0V9RH0@D5105mOG`1eMGD0Lg=c01`L700`
<300ATI607IfMP2=SHd0MGEe02/[:`1DE5@0Hf=S055AD@1UIFD0CTi>01/K6`16ATH0KFe]05YJFP1e
MGD0GEeM02`/;01]KFd0WinO05MGE`1WIfL0:RXZ034a<@2GUiL0S8b<059BDP2SXj<0;bl_0492@P2T
Y:@0>SXj051@D01]KFd08B4Q03De=@1dM7@0CTi>05ADE00l?3`092@T04=3@`25QHD0A4A405=CD`0]
;Bd0>CTi05]KF`1mOGd0DE5A069RHP0g=cL0E5AD061PH01jNWX0>3Ph05EEE@1SHf<0DU9B08N7Q`2U
YJD0@T9204E5A@0o?cl0EUIF0;Zj^P1]KFd0:RXZ04I6AP0j>SX0UiNG0:6QX@0k>c/0<C4a055AD@16
ATH0OGem09jNWP0W9bL0;Bd]05EEE@1OGel0VIVI0492@P0a<C40EEEE06MWI`1/K6`0Q8B4075aL@0`
<300JVYZ06IVIP1DE5@0WinO0<[:bP0k>c/0RHV90:2PX01oOgl0/K6a07=cL`0k>c/0NGUi08^;R`2M
WId0b/[:04U9B@0l?3`0Fe]K0:2PX01YJFT0c/k>059BDP19BDT0MWIf065QH@2MWId0EeMG03Ti>@1U
IFD0b<S808f=S@1iNGT0=cLg04=3@`1VIVH0^k^k05aLG01QHF40BTY:065QH@20P800CTi>06MWI`10
@400S8b<0>GUi@1cLg<0Cdm?06a/K01@D500OGem0492@P1>CTh0Hf=S03/k>`1ADE40Z:RX07QhN01[
Jf/0ADE505UIF@2CTi<0Ph>304I6AP20P800?Cdm08V9R@3<c<`0CDe=0451@@1OGel0JFUY0861P@2k
^k/0?Cdm038b<P1cLg<0JVYZ0:>SX`2d];@0:2PX03Lg=`1@D500Hf=S0:FUY@2ATI408R8R03Hf=P1W
IfL0IFEU07MgM`1>CTh0:b/[04a<C02?Shl0Ogmo09jNWP19BDT0;bl_05ADE02@T900JVYZ079bLP1N
GUh0>c/k06IVIP2LW9`0De=C06e]K@1SHf<0:b/[0P1EEED01`1JFUX0IVIV079bLP0f=SH0Bd];0:j^
[P19BDT03_ooool0301EEED0F5QH061PH015ADD0DU9B05YJFP1HF5P0ADE504=3@`1:BTX0A4A403Lg
=`80De=C01X0C4a<061PH01JFUX0C4a<04U9B@1GEeL0;Rh^03Hf=P1FEUH0BTY:05EEE@10@400ATI6
04Q8B01HF5P0JFUY051@D019BDT0FUYJ069RHP1QHF40E5AD06ATI01BDU80@D5102@T9080?Shn00<0
>SXj04Y:BP18B4P00P0e=CD08012@T80Fe]K04U9B@1CDe<0IVIV03/k>`17AdL0>CTi06EUI@19BDT0
B4Q804Y:BP1KFe/0GUiN05aLG017AdL0GEeM04=3@`1EEED0De=C08N7Q`0_;bl0GemO07alO02;Rh/0
9RHV0492@P1XJ6P0EeMG05aLG00k>c/0BTY:0P11@D40a@1<C4`0?Cdm05ADE01=CDd0Cdm?02/[:`1<
C4`06ATI04M7A`13@d<0B4Q805IFEP0k>c/0ADE503@d=018B4P0=3@d04m?C`1PH600Fe]K079bLP17
AdL0J6QX07moO`0j>SX0R8R805ADE01:BTX0CDe=06IVIP1CDe<0P82005aLG015ADD0CTi>04Q8B01D
E5@0L71`05aLG017AdL0EeMG06UYJ@10@400R8R805]KF`1iNGT0^KVi06UYJ@2@T900QhN708V9R@26
QXH0[jn_09RHV01FEUH0BDU907UiN@1UIFD0PX:205QHF01OGel0IVIV02HV9P1FEUH0<C4a0:FUY@1?
Cdl0JVYZ061PH01^KVh0ATI605eMG@1VIVH0Ng]k07alO01JFUX0YjNW07AdM018B4P0EEEE04A4A01f
MWH0Kfm_04=3@`1;Bd/0TI6A05IFEP1>CTh06ATI06UYJ@2@T900?3`l055AD@0S8b<0IFEU04a<C01T
I6@0?Cdm0<O7a`1dM7@0LG5a069RHP1CDe<0B4Q803/k>`1mOGd0@T9206m_K`0[:b/0GUiN05aLG00Z
:RX0EEEE03Hf=P1XJ6P0NWYj05iNGP1:BTX0LW9b09>CT`1=CDd0>CTi04];B`0o?cl0EEEE04=3@`0f
=SH0ATI60>?Sh`2:RXX0CDe=04M7A`1>CTh0FEUI04E5A@1GEeL0@41004a<C01=CDd07QhN07emO@0W
9bL0I6AT04i>CP1hN7P0Jf][04e=C@1GEeL0De=C03dm?@1>CTh0EEEE05mOG`1HF5P0LG5a03dm?@17
AdL0IFEU01@D500k>c/0FEUI05=CD`1UIFD07AdM08j>SP25QHD0EUIF0492@P19BDT0GemO079bLP1O
Gel0LW9b06][J`1QHF40HV9R0=?Cd`3>c/h0QhN709^KV`1fMWH0YZJV08Z:RP1nOWh0DU9B03Xj>P1M
GEd0IfMW04m?C`19BDT0C4a<05iNGP1cLg<0M7Ad06IVIP24Q8@0SHf=013ooooo000>ooooo`0709:B
TP1dM7@0WinO059BDP2:RXX0LG5a05UIF@0204A4A01E05mOG`0a<C40DU9B05IFEP1>CTh0HF5Q08J6
QP0g=cL0AdM704A4A015ADD0MGEe08n?S`0W9bL0?Cdm07alO012@T80UiNG079bLP0a<C40@d=305aL
G01IFET0UYJF03@d=00X:2P0B4Q807QhN025QHD0alO702PX:00F5QH0De=C07moO`1fMWH0PX:201LG
5`0e=CD0QXJ60861P@25QHD09BDU03`l?014A4@0Bd];05MGE`1^KVh0?Shn04E5A@1GEeL0R8R8071`
L038b<P0;bl_04E5A@1HF5P0D51@06MWI`20P8008B4Q0451@@1iNGT0DU9B07moO`2?Shl0>c/k0492
@P1>CTh0@41004Q8B00`<300=SHf04];B`1XJ6P0>SXj04U9B@0g=cL0CDe=07UiN@1YJFT0:BTY03hn
?P1eMGD0EeMG0861P@0204Y:BP1405ADE01[Jf/0JVYZ07EeM@0c<c<08R8R05IFEP1HF5P0PH6104];
B`0W9bL0ATI604a<C01_Kfl0[:b/02`/;00X:2P0EeMG06m_K`1iNGT0OGem02`/;013@d<0Ogmo08N7
Q`1hN7P0Y:BT04E5A@1>CTh0FEUI08j>SP20P800/;2`06UYJ@0e=CD0CDe=07=cL`1RHV80Zj^[03Lg
=`1:BTX0XJ6Q0:FUY@1`L700NGUi03hn?P1?Cdl0W9bL05MGE`1YJFT0S8b<01`L701OGel0OWin04Q8
B02ATI40LW9b02/[:`0a<C40Lg=c04U9B@2HV9P0@d=3030`<010@400/K6a07AdM01VIVH203Lg=`1>
09:BTP2j^[X0Ogmo0820P00h>3P0<c<c08:2PP2_[jl0EUIF04E5A@0j>SX0?3`l07inOP2EUID0>SXj
05UIF@19BDT0b<S807YjNP0Q8B40?Shn051@D01EEED0bl_;05eMG@0a<C40ATI606i^KP2BTY80Ph>3
01hN7P1HF5P0NGUi05aLG02/[:`0DU9B03Hf=P1jNWX0OWin08:2PP1TI6@07AdM07moO`1MGEd0R8R8
07]kN`0];Bd0D51@069RHP29RHT0PH6102DU9@0n?Sh0UiNG09JFUP21PH40YjNW04E5A@1:BTX0Ti>C
08j>SP1]KFd0MGEe0451@@1HF5P0LG5a069RHP1OGel0M7Ad04A4A01RHV80Lg=c05mOG`1jNWX0N7Qh
04I6AP1XJ6P0S8b<0P15ADD0801aLG40?Shn06QXJ01iNGT0@d=306MWI`1XJ6P0<30`06a/K02MWId0
>c/k03lo?`1lO7`0>c/k05IFEP1]KFd0=CDe05QHF015ADD0ATI605IFEP1eMGD0>SXj04E5A@1<C4`0
Fe]K06MWI`2RXZ80@D5105EEE@1GEeL0GUiN3_ooool07@1OGel0A4A4059BDP1ADE40F5QH0451@@1Z
JVX0C4a<06e]K@1HF5P0B4Q804I6AP1MGEd0EeMG05ADE01MGEd0KFe]059BDP19BDT0Cdm?038b<P0i
>CT0?Cdm03Lg=`1;Bd/0AdM704U9B@1>CTh0SXj>0080FUYJ0=80AdM705YJFP1?Cdl0K6a/061PH01T
I6@0E5AD04];B`0Z:RX0>3Ph03Ti>@12@T80Bd];059BDP0o?cl0@D5104];B`1EEED0@T9203Xj>P1>
CTh0@T9203lo?`0Z:RX0@d=304M7A`1ADE40@41004Q8B01:BTX0@D5104E5A@1<C4`0?clo0410@01G
EeL0LW9b04U9B@1>CTh0C4a<04e=C@0e=CD0ADE506ATI00^;Rh0B4Q804M7A`1IFET0:BTY03Lg=`13
@d<0>3Ph0492@P1KFe/0Cdm?02`/;00m?Cd071`L04=3@`0g=cL0?clo05ADE00m?Cd0@T9202TY:@0o
?cl0=CDe04e=C@1OGel0@T9204];B`0c<c<0BTY:04i>CP1HF5P0T92@06IVIP1ZJVX0FEUI06MWI`10
@400Hf=S04=3@`0m?Cd0GEeM06e]K@1cLg<0H61P06=SH`1IFET0K6a/05]KF`0j>SX0T92@055AD@26
QXH0`<3007YjNP2TY:@0O7al07inOP1FEUH0UIFE06][J`1IFET0CTi>03lo?`17AdL0ADE5065QH@1H
F5P0FUYJ03<c<`1KFe/07QhN09:BTP1FEUH0@d=304Y:BP1ADE40:RXZ069RHP1OGel0G5aL06a/K01k
Ng/0dm?C07YjNP15ADD0Jf][04E5A@1VIVH0YJFU065QH@10@400IFEU02l_;`1kNg/0=CDe03hn?P1K
Fe/0CDe=05iNGP0i>CT0H61P05YJFP1mOGd0F5QH0<71`@1XJ6P0UiNG08n?S`1JFUX0Cdm?03Xj>P1F
EUH0@T9206MWI`1ADE40MWIf0861P@1=CDd0CTi>04A4A01MGEd0QHF507EeM@1KFe/0F5QH08^;R`1A
DE40EeMG059BDP0i>CT0IVIV0410@00`<300IVIV0<[:bP1?Cdl0EEEE055AD@1CDe<0MWIf04a<C01E
EED0CDe=04Y:BP17AdL0=CDe08f=S@1<C4`0G5aL038b<P0l?3`0DE5A034a<@10@400FEUI0492@P1M
GEd0N7Qh069RHP1aLG40B4Q804M7A`1IFET0DE5A03dm?@1ZJVX206MWI`0S05aLG01FEUH0A4A405YJ
FP1@D500HF5Q06=SH`1DE5@0HV9R06EUI@2n_[h0HV9R08>3P`22PX80Jf][07MgM`2MWId0[jn_07Qh
N02?Shl0G5aL055AD@1SHf<0NWYj092@T01SHf<0OWin08:2PP1[Jf/0SXj>08>3P`1KFe/0HF5Q071`
L01TI6@04?ooool000kooooo0?l0PX:206MWI`3:b/X0CTi>06ATI01[Jf/0/;2`06IVIP0m?Cd0JFUY
065QH@2TY:@0R8R802TY:@1GEeL0JVYZ071`L02KVi/0:RXZ03Xj>P0k>c/0M7Ad07IfMP2OWil07AdM
04A4A01/K6`0QHF504a<C01MGEd05QHF02l_;`1=CDd0GUiN03lo?`1;Bd/0:BTY03Lg=`1ADE40RHV9
05ADE029RHT0@41003@d=01jNWX0>c/k04a<C012@T80;Rh^05]KF`1[Jf/0<S8b04U9B@0_;bl0?clo
04i>CP1:BTX0<c<c04A4A00m?Cd0B4Q807MgM`17AdL0ATI604a<C00l?3`0Ogmo0:VYZ@13@d<0<C4a
0492@P14A4@0R8R80<71`@0X:2P0>3Ph05iNGP1_Kfl0SHf=06UYJ@0G5aL0>SXj06m_K`1YJFT0N7Qh
03Xj>P0d=3@0Fe]K096AT@1iNGT0OWin01dM7@1:BTX0ADE506i^KP2BTY80B4Q802PX:019BDT0=cLg
05QHF01/K6`0HF5Q02XZ:P1UIFD0LW9b038b<P1jNWX0?clo03dm?@21PH40Ng]k03`l?018B4P0?Cdm
0492@P2`/;00LG5a059BDP1iNGT0DE5A02l_;`1[Jf/0F5QH0451@@1QHF40?3`l04=3@`1FEUH0J6QX
0410@01aLG40>SXj051@D01YJFT0D51@04M7A`1=CDd0B4Q807IfMP2<S8`0@41004E5A@1QHF40IFEU
07=cL`2JVYX0>CTi0492@P1=CDd0=CDe092@T02SXj<0?3`l02d];@19BDT0EeMG0:RXZ01ZJVX0=3@d
04e=C@1BDU80WIfM0<O7a`1@D500;Bd]05mOG`24Q8@0M7Ad0<[:bP14A4@0;2`/05IFEP2EUID0MWIf
06YZJP0^;Rh0EEEE05ADE02>SXh0IfMW09RHV00g=cL0KFe]09^KV`1?Cdl0PX:206MWI`0h>3P0F5QH
06a/K01NGUh0LG5a02d];@0`<300YjNW05iNGP1jNWX0@T9202XZ:P1YJFT0IfMW061PH00k>c/0;Bd]
07moO`2b/[80D51@05eMG@0a<C40BTY:09:BTP1?Cdl0?clo06YZJP1DE5@0RXZ:06MWI`0X:2P0BTY:
04i>CP1_Kfl0/K6a05aLG01>CTh0DE5A05YJFP1BDU80[jn_02h^;P1FEUH0H61P04M7A`1YJFT0TY:B
02XZ:P1NGUh0A4A4059BDP2UYJD0Vi^K03Lg=`1WIfL0Lg=c04i>CP2ATI40clo?059BDP1DE5@0UIFE
055AD@21PH40RHV902`/;00f=SH001L0Xj>S07moO`1bLW80SHf=03dm?@1JFUX0Ng]k061PH01nOWh0
A4A402<S8`1ADE40P82008Z:RP23Ph<0BDU903Ph>014A4@0UIFE07AdM02<S8`0<c<c04M7A`0>oooo
o`1>05eMG@1LG5`0Ng]k05QHF01ZJVX0Cdm?06EUI@1@D500BDU904i>CP16ATH0=cLg05=CD`1KFe/0
EUIF06i^KP1<C4`0?Shn03dm?@1HF5P0@D5103De=@0];Bd0?clo05=CD`0b<S80De=C06ATI01gMgL0
JVYZ05mOG`1NGUh0N7Qh05IFEP1nOWh0RXZ:0820P01OGel0ADE502LW9`1@D500CTi>05YJFP16ATH0
DU9B03dm?@1=CDd0@T9203hn?P14A4@0CDe=04m?C`0l?3`0ADE5038b<P1=CDd0=cLg055AD@12@T80
EUIF05UIF@1DE5@0Cdm?06QXJ00k>c/0;Bd]06MWI`1jNWX0<C4a06IVIP1TI6@0HV9R03Xj>P1UIFD0
G5aL05IFEP17AdL0:b/[0P14A4@0Q@1GEeL0B4Q8034a<@1<C4`0=SHf03Ti>@0[:b/0@D5101`L700`
<300@d=303`l?010@400?3`l04e=C@0n?Sh0Cdm?0410@01FEUH0LG5a06][J`1HF5P0D51@059BDP1W
IfL0DU9B0;:b/P1<C4`0E5AD06=SH`1`L700Cdm?07moO`1gMgL0C4a<05YJFP1:BTX0Cdm?05QHF01M
GEd0L71`06YZJP1BDU80KFe]0;Vi^@1HF5P0Rh^;0::RXP1@D500MgMg07alO02VYZH0IfMW0;Jf]P1F
EUH0De=C03`l?01>CTh0DU9B0451@@1YJFT0SHf=06=SH`1[Jf/0UYJF07AdM02/[:`0GUiN06e]K@1T
I6@0EeMG05eMG@2MWId0Kfm_05mOG`1^KVh0P8200=OGe`1dM7@0@41004a<C016ATH0JVYZ06i^KP1@
D500C4a<07]kN`0k>c/0C4a<020P801=CDd0I6AT04];B`20P800@d=306a/K014A4@0HV9R05=CD`3F
e]H0M7Ad05UIF@1HF5P0J6QX055AD@12@T80R8R805EEE@2LW9`0FUYJ051@D01IFET0LG5a07UiN@0m
?Cd0A4A405aLG01GEeL0UIFE06e]K@23Ph<0?Cdm06][J`1FEUH0GemO071`L01PH600<S8b04];B`1K
Fe/0Hf=S051@D010@4000P1QHF40?`1ADE40ADE504e=C@1BDU80Bd];02TY:@1RHV80:2PX07inOP0m
?Cd0CTi>05eMG@1RHV80FEUI04=3@`0X:2P0B4Q807QhN01bLW80EEEE04=3@`1EEED0RXZ:05]KF`1O
Gel0@41006=SH`1LG5`0M7Ad07EeM@1VIVH0F5QH05=CD`1=CDd0FEUI04];B`1/K6`0O7al0>_[j`1G
EeL0L71`04Y:BP3;bl/0Zj^[05YJFP1]KFd0H61P06YZJP1GEeL0FUYJ05EEE@19BDT0HV9R051@D00c
<c<0@41003hn?P2=SHd0TY:B06a/K01YJFT0J6QX07moO`0@ooooo`003_ooool0>`2IVIT0Cdm?06e]
K@25QHD0Ph>303hn?P0l?3`0L71`05QHF01^KVh0=SHf04e=C@1eMGD0C4a<07QhN029RHT0>CTi06i^
KP1/K6`0Cdm?075aL@12@T80>CTi069RHP28R8P0DE5A07QhN016ATH0Bd];07EeM@1ADE40CDe=06][
J`19BDT0Cdm?061PH01QHF40CTi>07]kN`1:BTX0CDe=07alO01MGEd0De=C06IVIP0n?Sh0Lg=c0:>S
X`1=CDd0EEEE051@D00l?3`0Rh^;06UYJ@0[:b/0@D51051@D01LG5`0RHV90080>3Ph01L0GemO059B
DP1/K6`0MWIf02TY:@0j>SX0IfMW06e]K@1VIVH0I6AT020P8011@D40NWYj055AD@1PH600Cdm?02d]
;@1FEUH0JVYZ03Ti>@1;Bd/0DE5A0451@@0205QHF03003dm?@1fMWH0=SHf04=3@`1kNg/0EeMG05UI
F@2DU9@0;2`/04m?C`1aLG40>3Ph065QH@1oOgl08R8R03`l?01`L700<C4a07IfMP0f=SH0>SXj06IV
IP1KFe/0A4A408>3P`17AdL0EEEE06i^KP1KFe/0ATI604];B`1CDe<0DE5A09FEU@2HV9P0ADE506AT
I01LG5`0FEUI0:6QX@2WYjL0?Cdm05=CD`1SHf<0G5aL0:6QX@1dM7@0@D5107MgM`17AdL0Ng]k08b<
S018B4P0@d=306IVIP1ADE40OGem07UiN@0n?Sh0JFUY07emO@1QHF40OWin07emO@14A4@0Fe]K065Q
H@1BDU80J6QX071`L00k>c/0JVYZ05ADE01?Cdl0RXZ:07inOP16ATH0IVIV08V9R@1:BTX0RHV909nO
W`0o?cl0GUiN061PH01VIVH0X:2P07moO`0k>c/0De=C04Y:BP1lO7`0YjNW04M7A`11@D40FUYJ05AD
E01oOgl0OGem0410@01jNWX0TY:B06e]K@2QXJ40>3Ph05=CD`2FUYH0JVYZ059BDP1PH600?Shn09bL
W02VYZH0?Cdm05iNGP1MGEd0FEUI0861P@14A4@0P82006=SH`18B4P0RXZ:04E5A@1HF5P0IFEU04I6
AP1EEED0WinO06EUI@27QhL0JFUY051@D01nOWh0O7al04Y:BP2BTY80HV9R07MgM`2QXJ40A4A40410
@01fMWH0B4Q80861P@29RHT0>SXj04M7A`1YJFT0CTi>0:2PX02CTi<0=SHf04m?C`1hN7P0GemO0:6Q
X@2?Shl0;bl_03dm?@1NGUh0IfMW07QhN01GEeL0@T9205mOG`2BTY80I6AT07]kN`1BDU80?Shn051@
D01mOGd0FEUI06MWI`1:BTX0?Shn07IfMP2SXj<0DE5A06a/K01>CTh0?clo06YZJP20P800CDe=05AD
E01LG5`0CDe=07alO01bLW80@D513_ooool04@1HF5P0FEUI065QH@1<C4`0GemO05eMG@1aLG40B4Q8
06IVIP1HF5P0Cdm?0451@@1/K6`0L71`06m_K`24Q8@0K6a/0080CDe=02/0G5aL02/[:`0h>3P0<30`
03/k>`1HF5P0>c/k04M7A`1CDe<0RXZ:06a/K01VIVH0DE5A07YjNP1_Kfl0LG5a05QHF01QHF40MWIf
04U9B@0c<c<0CDe=03lo?`16ATH0De=C05iNGP12@T80CTi>04U9B@1JFUX0D51@04i>CP14A4@0<C4a
03hn?P0g=cL0De=C03/k>`1=CDd0B4Q804m?C`1=CDd0E5AD0080D51@06X0<C4a02d];@1jNWX0H61P
03De=@1KFe/0I6AT06i^KP0d=3@0Fe]K05aLG00a<C40BDU904Q8B01HF5P092@T04A4A019BDT0>CTi
04=3@`10@400?Cdm03<c<`18B4P08R8R03dm?@19BDT0@T9204i>CP0i>CT0C4a<02XZ:P13@d<0B4Q8
04Y:BP1DE5@0ADE505ADE010@400DE5A05eMG@18B4P0O7al04U9B@1PH600Fe]K07=cL`1eMGD0Shn?
061PH014A4@0CTi>069RHP1NGUh0EEEE06ATI01YJFT0E5AD05MGE`1SHf<0]kNg05ADE02JVYX0QHF5
07EeM@28R8P0F5QH06MWI`1OGel0YjNW061PH016ATH0@D5104I6AP10@400IFEU051@D01KFe/0GEeM
04m?C`1dM7@0>SXj0:>SX`10@400E5AD05iNGP19BDT0>CTi07UiN@1TI6@0RHV908^;R`2<S8`0aLG5
07EeM@1;Bd/0M7Ad05iNGP1gMgL0Vi^K061PH01FEUH0L71`03<c<`1WIfL09RHV0P1`L700DP13@d<0
GUiN03Ph>01FEUH0HV9R07MgM`13@d<0XZ:R07]kN`2NWYh0JVYZ051@D01IFET0Jf][092@T01QHF40
De=C04Q8B01oOgl0Jf][03dm?@1?Cdl0>SXj05ADE01bLW80Hf=S06][J`1TI6@0MWIf04E5A@1]KFd0
F5QH059BDP1VIVH0B4Q803`l?01=CDd0Cdm?04U9B@18B4P0@T9204A4A01LG5`0@d=305EEE@1=CDd0
D51@04i>CP0f=SH0RXZ:03Lg=`1IFET0@D5106m_K`1JFUX0GemO04U9B@1GEeL0DE5A05ADE019BDT0
Cdm?05iNGP1?Cdl0?3`l05iNGP1FEUH0;2`/055AD@1MGEd0EEEE05QHF019BDT0LW9b05ADE014A4@0
ATI604i>CP1?Cdl0JFUY05aLG02ZZZX206QXJ00F05MGE`24Q8@0Lg=c06QXJ021PH40HV9R06m_K`0e
=CD0Kfm_04];B`1GEeL0DE5A05MGE`15ADD0Cdm?04Y:BP1]KFd0Hf=S06IVIP1gMgL0K6a/06e]KA3o
oooo000>ooooo`0<08j>SP2IVIT0][Jf04];B`1cLg<0Ng]k09BDU00i>CT0=CDe06=SH`1CDe<0UIFE
0P0j>SX0;@1[Jf/0S8b<06i^KP1GEeL0AdM705QHF01OGel0Kfm_03Ph>00g=cL0?Cdm06EUI@1hN7P0
I6AT024Q8@0h>3P0;2`/06ATI02][Jd0?Cdm02TY:@14A4@0<C4a06a/K026QXH0FEUI02d];@1>CTh0
@D5106QXJ02][Jd09bLW03hn?P1cLg<0FUYJ092@T012@T806ATI04];B`26QXH0IfMW07QhN00j>SX0
?clo04];B`0206EUI@0N0:FUY@0W9bL0D51@05mOG`16ATH0PH610:NWY`0L71`0EEEE05eMG@15ADD0
F5QH09:BTP0P8200?clo065QH@1PH600UIFE05UIF@0Y:BT0ATI606][J`1ADE40IVIV03<c<`0a<C40
C4a<096AT@1<C4`0KFe]0P0l?3`0^@1DE5@0IFEU061PH00i>CT0=CDe03lo?`1?Cdl0AdM704Q8B01U
IFD0=SHf08V9R@1WIfL09RHV055AD@17AdL0@41009:BTP1EEED0:b/[05EEE@14A4@0KVi^08Z:RP0m
?Cd0?3`l05ADE01QHF40M7Ad08V9R@0g=cL0:b/[05eMG@2>SXh0KVi^09RHV01OGel09RHV06YZJP2=
SHd0FEUI0:FUY@0i>CT0@410079bLP1NGUh0LG5a0<71`@15ADD0?Shn0:>SX`2:RXX0Y:BT0=;BdP0_
;bl0BTY:0820P01ZJVX0d=3@09FEU@0O7al0D51@07emO@1_Kfl0WinO05YJFP0l?3`0EeMG0:RXZ01_
Kfl0][Jf05EEE@0m?Cd0IVIV0;Ng]`23Ph<0U9BD02l_;`0i>CT0FUYJ0;:b/P2BTY80H61P03@d=01>
CTh0JFUY0:JVYP1RHV80KFe]03De=@2=SHd0YZJV05EEE@1XJ6P0Cdm?05QHF02`/;00WIfM04];B`22
PX80=cLg05MGE`1kNg/0:b/[06ATI01KFe/0D51@09NGU`1eMGD0@d=30451@@15ADD0[Zj^08J6QP0N
7Qh0Bd];05YJFP2?Shl0/[:b02d];@0X:2P0EeMG07MgM`2PX:00Kfm_03Ph>015ADD0LG5a07emO@2W
YjL0@T9203Ti>@1[Jf/0MGEe0820P01mOGd0?3`l0410@01jNWX0HF5Q06i^KP1fMWH0A4A4051@D02c
/k<0IfMW09VIV@1WIfL0@41006IVIP2l_;`0De=C09FEU@1mOGd0>CTi05IFEP2l_;`0BDU905mOG`2B
TY80@41004i>CP2MWId0E5AD07QhN02IVIT0>3Ph04E5A@2?Shl0E5AD06][J`0c<c<0=cLg05ADE01a
LG40>SXj06IVIP13@d<0AdM7055AD@1hN7P0?Shn06=SH`0j>SX0Hf=S00kooooo08H0E5AD055AD@1U
IFD0DE5A04a<C00k>c/0?clo03Xj>P1FEUH0LW9b04E5A@0i>CT0CTi>04I6AP1<C4`0DU9B05EEE@1=
CDd0@D5103hn?P0`<300C4a<04=3@`0h>3P0HV9R04E5A@17AdL0B4Q805=CD`1FEUH0E5AD04=3@`1I
FET0ADE505]KF`1CDe<0EeMG05ADE00d=3@0;Rh^03hn?P0c<c<0?Cdm03`l?00o?cl0;2`/04I6AP0g
=cL0C4a<04M7A`1?Cdl0B4Q805iNGP1=CDd0=3@d04];B`1JFUX0H61P05ADE01@D500@D5104Y:BP17
AdL0FEUI05iNGP0o?cl0Ph>3069RHP1?Cdl0OGem06=SH`2>SXh0D51@03hn?P1HF5P0>SXj04];B`1>
CTh0FUYJ03De=@0_;bl0@41004A4A011@D40E5AD0492@P0b<S80BTY:030`<01BDU80CDe=0410@01V
IVH0>SXj05=CD`19BDT0@T9204E5A@13@d<0Fe]K0492@P1DE5@0G5aL04a<C01WIfL0HF5Q0:b/[01;
Bd/0Hf=S05=CD`1dM7@0GemO08>3P`0h>3P0>c/k03Ph>01DE5@0JVYZ05YJFP1iNGT0HV9R05UIF@0n
?Sh0B4Q808b<S01GEeL0][Jf071`L01CDe<0IVIV08N7Q`24Q8@0Kfm_0:b/[080O7al03X09bLW04m?
C`0m?Cd0?Shn055AD@1?Cdl0>SXj034a<@17AdL0=cLg092@T00/;2`0Cdm?065QH@10@400Bd];07in
OP1XJ6P0LW9b07]kN`2HV9P0iNGU07AdM01LG5`0MgMg04=3@`1oOgl0W9bL04Q8B01GEeL0JVYZ01dM
7@1EEED0820P04m?C`1<C4`0@D51061PH00i>CT0H61P04U9B@1BDU80OGem0=oOg`21PH40OGem06UY
J@1FEUH0KVi^06IVIP20P800LW9b065QH@0l?3`0BTY:051@D01GEeL0N7Qh0P16ATH07P1NGUh0C4a<
07inOP1QHF40KFe]04];B`1]KFd0EUIF03<c<`0j>SX0@d=303hn?P0m?Cd0De=C04U9B@1ADE40@d=3
04];B`1ZJVX0CTi>05iNGP16ATH0Fe]K05YJFP0/;2`0Hf=S03Ph>01oOgl0DE5A04i>CP80C4a<0380
@d=3059BDP0d=3@0Lg=c04Y:BP13@d<0FUYJ04U9B@1HF5P0L71`05QHF019BDT0>SXj04];B`1ZJVX0
HF5Q0410@01PH600EUIF04E5A@1@D500J6QX03`l?01OGel0DE5A0<[:bP1mOGd0TY:B04M7A`2OWil0
M7Ad07moO`31`L40DE5A05iNGP0V9RH0EEEE03dm?@1/K6`0HF5Q059BDP0Z:RX0<C4a03Ph>02HV9P0
NWYj06EUI@2:RXX0IFEU07=cLa3ooooo000>ooooo`2c06QXJ01RHV80^KVi05EEE@1IFET0E5AD09:B
TP1^KVh0;2`/04Y:BP1UIFD0bl_;09jNWP0J6QX0>c/k08:2PP1nOWh0QhN702XZ:P1;Bd/0JFUY08R8
R01TI6@0X:2P02h^;P15ADD0QXJ606UYJ@1TI6@0UIFE01HF5P1<C4`0G5aL07inOP1QHF40XZ:R02l_
;`0T92@0<c<c075aL@26QXH0XZ:R03<c<`0j>SX0O7al07inOP1kNg/0LW9b01`L701IFET0PH6103dm
?@1NGUh09RHV02/[:`1PH600MWIf05QHF01BDU807QhN055AD@1HF5P0Bd];04M7A`1?Cdl0:2PX04M7
A`2LW9`0E5AD04Y:BP0n?Sh0E5AD061PH02LW9`0G5aL051@D015ADD0>c/k06ATI01UIFD07alO03Ph
>017AdL0Ph>308j>SP0d=3@07alO02DU9@1OGel0K6a/092@T00N7Qh0@T92059BDP0o?cl0VIVI03dm
?@0V9RH0ATI605eMG@1BDU80SHf=0820P00k>c/0JFUY07alO01CDe<0Ti>C05MGE`0Q8B40Q8B406m_
K`24Q8@0UYJF03De=@17AdL0[:b/09JFUP2ATI40^;Rh051@D012@T80WIfM09JFUP1`L700V9RH04m?
C`0d=3@0Kfm_07emO@1WIfL0VIVI04a<C00b<S80GEeM05UIF@1QHF40NGUi03De=@12@T80L71`08:2
PP1JFUX0C4a<038b<P1NGUh0LW9b06QXJ01KFe/0=SHf02`/;01YJFT0SHf=030`<015ADD0HV9R03Ph
>02@T900PH6102XZ:P1<C4`0G5aL04a<C02VYZH0Jf][03Ph>01RHV80MGEe06UYJ@2LW9`0@d=302l_
;`1XJ6P0?clo05IFEP2JVYX0820P04i>CP1OGel00P1iNGT0EP2e]KD0>3Ph05IFEP27QhL0OGem0:BT
Y01RHV80;bl_08^;R`2XZ:P0G5aL0:JVYP0b<S80:RXZ07inOP1EEED0WinO05QHF00X:2P0CTi>06EU
I@2>SXh0@410024Q8@1NGUh0V9RH05ADE021PH40<30`04A4A02IVIT0Shn?071`L02c/k<0=3@d05aL
G01[Jf/0E5AD07]kN`1JFUX0@D510<?3``2?Shl0O7al08f=S@13@d<0DU9B09>CT`1FEUH0PH6107Ee
M@0i>CT0De=C08:2PP0l?3`0EUIF04I6AP1?Cdl0Kfm_08n?S`16ATH0BDU90410@017AdL0H61P09:B
TP13@d<0DU9B04];B`1JFUX0JVYZ09nOW`1:BTX0?clo06MWI`1MGEd0OGem0;6a/@14A4@0>3Ph06AT
I01LG5`0Shn?04];B`0K6a/0ADE50P1`L7002@2RXZ80HF5Q02<S8`10@400SXj>075aL@2e]KD0?Shn
034a<@0>ooooo`3=04a<C01TI6@0L71`05YJFP1GEeL0ATI605mOG`16ATH0CTi>03hn?P1:BTX0A4A4
05QHF01jNWX0L71`06ATI01LG5`0FEUI04I6AP1ADE40>SXj03@d=00[:b/0;Bd]04m?C`0g=cL0Bd];
04m?C`22PX80PH6105QHF01<C4`0IFEU05MGE`1ZJVX0FUYJ069RHP1SHf<0<c<c03De=@1<C4`0?3`l
04A4A01CDe<0D51@02TY:@1=CDd0?clo06EUI@1>CTh0D51@03dm?@0/;2`0<C4a02/[:`17AdL09bLW
04i>CP16ATH0EeMG04];B`1CDe<0Cdm?04];B`0a<C4092@T07inOP1IFET0:b/[065QH@1MGEd0Q8B4
04E5A@1OGel0IFEU04U9B@1>CTh0IFEU06QXJ00U9BD0=3@d04i>CP0n?Sh0=CDe04m?C`1:BTX0<C4a
04i>CP0^;Rh0?Cdm04U9B@18B4P0D51@02l_;`1?Cdl0;Rh^034a<@0n?Sh0BDU907inOP18B4P0D51@
04Q8B01FEUH0Fe]K059BDP1dM7@0Bd];05QHF01^KVh0PX:205UIF@1nOWh0JFUY07EeM@1?Cdl0De=C
04a<C01YJFT0GemO06][J`1gMgL0Lg=c07moO`36a/H0>CTi0>KViP1`L700De=C05eMG@1QHF40EeMG
05QHF02BTY80DE5A05iNGP19BDT0Cdm?03lo?`1DE5@0@4100451@@1JFUX0@d=3065QH@0a<C40V9RH
01/K6`1=CDd0RXZ:03`l?00k>c/0OGem04A4A01lO7`0IVIV08R8R02c/k<0Jf][04M7A`1NGUh0JFUY
079bLP2>SXh0IfMW07]kN`1oOgl0>3Ph05QHF00e=CD0EeMG05=CD`1ADE40NGUi02XZ:P1HF5P0EUIF
055AD@1FEUH0XJ6Q08:2PP2=SHd0IfMW0:JVYP1ADE40K6a/092@T01?Cdl0MWIf051@D01>CTh0ADE5
034a<@16ATH0<C4a04E5A@1kNg/0@D5105iNGP1WIfL0NWYj0451@@0n?Sh0BDU904=3@`0204];B`0R
03/k>`12@T80HF5Q04];B`19BDT0DU9B04I6AP1SHf<0=cLg04M7A`13@d<0ADE5055AD@18B4P0Rh^;
0492@P1BDU80@T9205iNGP1PH600I6AT04E5A@13@d<0<30`06a/K011@D40Bd];05aLG01VIVH0DE5A
06MWI`1?Cdl0;Rh^03Lg=`80ADE502<0BDU903dm?@1PH600>3Ph03Ti>@0Y:BT0C4a<04];B`29RHT0
CTi>09BDU01IFET0EeMG055AD@2JVYX0EeMG05QHF01cLg<0IfMW09>CT`17AdL0Hf=S02LW9`1=CDd0
>c/k06=SH`0m?Cd0A4A404m?C`1JFUX0D51@06i^KP1HF5P0GEeM07]kN`0@ooooo`003_ooool0e`2M
WId0EeMG07YjNP25QHD0KVi^03lo?`1NGUh0S8b<069RHP1lO7`0:2PX07IfMP34a<@0IfMW09NGU`1V
IVH0;bl_07QhN01CDe<0A4A407MgM`0i>CT0:2PX065QH@1MGEd0Hf=S0861P@0/;2`0@d=3061PH015
ADD0@41008J6QP0I6AT0@4100:VYZ@1MGEd0Cdm?05EEE@0S8b<0D51@09fMW@1cLg<0;bl_04E5A@1E
EED0KFe]0:b/[00i>CT0>SXj07EeM@1bLW80Lg=c0410@00U9BD0C4a<05=CD`28R8P0[Jf]02<S8`0i
>CT0G5aL07AdM02`/;00QHF501XJ6P0g=cL0P82005]KF`2=SHd0B4Q8020P8017AdL0Rh^;05YJFP2C
Ti<0BDU903Hf=P18B4P0HF5Q059BDP2d];@0BDU903hn?P1NGUh0LW9b06IVIP1hN7P0820P0451@@1l
O7`0H61P05aLG01IFET07AdM05]KF`1fMWH0<S8b06][J`12@T80:RXZ04i>CP1MGEd0CDe=075aL@0[
:b/0:RXZ08:2PP2IVIT0@d=306YZJP0Y:BT0?clo08:2PP1lO7`0=CDe03@d=00l?3`0B4Q807=cL`1J
FUX09RHV04Y:BP1?Cdl0IVIV08Z:RP1>CTh0:b/[04=3@`1>CTh0Jf][09fMW@1[Jf/0<C4a03hn?P1N
GUh0Ng]k09JFUP0f=SH0:RXZ05aLG01jNWX0OWin04U9B@0e=CD0@41005eMG@1dM7@0^[Zj04];B`0T
92@0Jf][0:BTY01EEED0RXZ:07moO`0d=3@0MGEe0;6a/@1RHV80TY:B071`L010@400TI6A08b<S01Z
JVX0b<S808b<S01=CDd0Cdm?07IfMP1LG5`0^KVi05YJFP11@D40Kfm_05mOG`1WIfL0PX:204I6AP0i
>CT0G5aL07QhN02=SHd0HF5Q04M7A`1VIVH0YJFU06a/K01kNg/0DE5A04M7A`2ATI40KVi^04];B`1?
Cdl0AdM70<30`02ATI40>c/k069RHP1;Bd/0SHf=06m_K`0c<c<0GUiN08n?S`0l?3`0RXZ:03Hf=P0]
;Bd0Bd];034a<@19BDT0/k>c0080EUIF03d0?3`l05aLG02e]KD0[Jf]0492@P20P800I6AT08n?S`3:
b/X0?clo03Xj>P1fMWH0MGEe0:^[Z`2@T9009bLW03Xj>P1hN7P0NGUi0;2`/01gMgL08R8R04I6AP23
Ph<0JFUY0;2`/028R8P08B4Q05IFEP2k^k/0SXj>0:2PX01=CDd0;Rh^051@D02OWil0SXj>08Z:RP0o
?cl0=3@d06YZJP2`/;00IFEU05]KF`12@T80Bd];075aL@2WYjL0Hf=S07alO00j>SX0>3Ph05aLG01`
L700AdM7069RHP1LG5`0:b/[08Z:RP1nOWh0?3`l00kooooo01T0Bd];05]KF`1WIfL0EEEE07IfMP1_
Kfl0Ng]k0492@P1ZJVX0Cdm?04U9B@0h>3P0Cdm?055AD@1GEeL0Lg=c069RHP1:BTX0CTi>04U9B@0e
=CD0D51@055AD@17AdL0IfMW0080@41006/0ATI604a<C01PH600B4Q80451@@1UIFD0DE5A05=CD`1;
Bd/0F5QH07MgM`0i>CT0<S8b04i>CP0b<S80@D5103dm?@1<C4`0?Cdm0410@00h>3P0CTi>03dm?@17
AdL0?Cdm04I6AP0c<c<0:b/[04i>CP0h>3P0Cdm?0410@01:BTX0CTi>05iNGP1:BTX0GUiN04U9B@0Q
8B40OWin05QHF00];Bd0F5QH05mOG`1QHF40>3Ph04m?C`1IFET0>c/k04Y:BP0j>SX0EEEE02TY:@0g
=cL0ADE503hn?P0^;Rh0J6QX04I6AP0k>c/0@T9203Ti>@1>CTh0@T9204Y:BP1[Jf/0>3Ph04A4A019
BDT0De=C04];B`11@D40KFe]04];B`1DE5@0F5QH05aLG01OGel0GEeM0:ZZZP1JFUX0F5QH069RHP1b
LW80GUiN07=cL`0b<S80B4Q80451@@1ADE40B4Q8065QH@19BDT0JFUY04U9B@18B4P0R8R80<O7a`1i
NGT0WYjN069RHP1YJFT0OWin075aL@1dM7@0Kfm_0:NWY`0206][J`0402@T9018B4P0EeMG08N7Q`80
BTY:03L0AdM704i>CP1HF5P0Cdm?08R8R00h>3P0BDU905EEE@0o?cl0BDU907QhN00c<c<0[Jf]0861
P@1jNWX0[Jf]05mOG`13@d<0FUYJ03`l?01=CDd0TI6A04I6AP1@D500ADE502DU9@1OGel0;Rh^08F5
Q@1^KVh0BDU904Q8B00n?Sh0GEeM069RHP1OGel0A4A40;Jf]P1IFET0VIVI05ADE01;Bd/0HV9R07Ad
M029RHT0B4Q804E5A@0k>c/0Ti>C08>3P`12@T80AdM703hn?P0o?cl0P8200080FEUI01h0@D5106=S
H`1CDe<0DE5A0410@017AdL0JFUY03hn?P0O7al0=SHf07]kN`15ADD0CTi>061PH01KFe/0J6QX05EE
E@1:BTX0EUIF08b<S01?Cdl0<C4a06IVIP0_;bl0KFe]030`<00h>3P0@D510861P@1GEeL204i>CP0_
071`L01?Cdl0Bd];05=CD`14A4@0;Bd]09>CT`1cLg<0:BTY061PH01<C4`0CDe=051@D00i>CT0O7al
04i>CP17AdL0A4A404U9B@0f=SH0Fe]K04E5A@2YZJT0IfMW0820P01QHF40Xj>S07YjNP1eMGD0VIVI
055AD@1RHV80@41007inOP19BDT0EeMG05QHF01EEED0H61P06UYJ@14A4@0Ng]k07alO020P800R8R8
09VIV@1_Kfl04?ooool000kooooo0;L0Vi^K06UYJ@1oOgl0De=C08J6QP1:BTX0EEEE03Xj>P19BDT0
HV9R030`<00l?3`0ATI605=CD`1eMGD0N7Qh02`/;01?Cdl0IfMW05QHF02SXj<0Jf][02TY:@11@D40
HF5Q08b<S03:b/X0=CDe02LW9`1CDe<0ADE507alO02n_[h09BDU01LG5`1@D500Jf][08j>SP2BTY80
5QHF030`<01QHF40Ng]k079bLP1HF5P0820P03hn?P1_Kfl0Ng]k07emO@13@d<0;Rh^06IVIP27QhL0
ADE506m_K`0d=3@0@41006i^KP1eMGD0FEUI06ATI00Z:RX0GUiN08j>SP18B4P0@41005ADE00Q8B40
F5QH079bLP0];Bd0A4A403/k>`0S8b<0De=C08F5Q@0m?Cd0Bd];02`/;00Z:RX0NWYj06IVIP0h>3P0
EUIF0492@P1JFUX0LG5a03dm?@0i>CT0@D5103/k>`20P800YjNW02XZ:P1:BTX0Lg=c04I6AP2ATI40
LG5a01dM7@0_;bl0Hf=S06m_K`2o_kl0;Bd]02PX:016ATH0KFe]06m_K`2QXJ40;2`/02l_;`0o?cl0
LG5a06MWI`13@d<07alO04U9B@1PH600NGUi05eMG@2ZZZX0>CTi05MGE`1ZJVX0Ph>308J6QP2VYZH0
F5QH04i>CP1kNg/0/K6a07emO@2c/k<0>CTi06][J`1dM7@0N7Qh05YJFP2RXZ80;2`/07EeM@1jNWX0
B4Q808B4Q01gMgL0=3@d08^;R`2SXj<0JFUY0:2PX01^KVh0<c<c05]KF`2;Rh/0ADE507YjNP18B4P0
?Cdm079bLP34a<@0CDe=07YjNP1BDU80ATI609>CT`3@d=00GUiN05mOG`0k>c/0LW9b0<_;b`1PH600
=CDe06UYJ@17AdL0OGem0;Zj^P11@D40>CTi055AD@1QHF400P29RHT0G@0[:b/0IVIV06MWI`20P800
a/K603/k>`1/K6`0TI6A0861P@1oOgl0GUiN01/K6`1WIfL0Rh^;06i^KP2XZ:P0CDe=04M7A`2YZJT0
Ogmo0:>SX`1fMWH08R8R05UIF@1[Jf/0HF5Q07alO00W9bL0>SXj055AD@2@T900/;2`05QHF00e=CD0
ATI609>CT`2:RXX0MgMg03Ph>013@d<0MGEe069RHP1ZJVX0GUiN03@d=015ADD0QXJ604e=C@1QHF40
P820030`<01WIfL0Ph>303/k>`1FEUH0I6AT03Ti>@1VIVH0YjNW04U9B@1BDU80B4Q804a<C01jNWX0
ZJVY0451@@1LG5`0HF5Q04];B`1YJFT0VIVI02DU9@1EEED0IFEU06=SH`2>SXh0JFUY03@d=01CDe<0
MgMg075aL@2KVi/0MgMg02`/;01gMgL0JFUY06MWI`1oOgl0Ti>C02h^;P1MGEd0/K6a07inOP0>oooo
o`0Y055AD@1QHF40QHF507QhN01JFUX0ATI606YZJP1PH600IfMW06EUI@1RHV80EUIF05eMG@1^KVh0
BTY:055AD@0c<c<0?3`l04A4A01CDe<0?clo0410@00[:b/0@T9204e=C@18B4P0GemO06i^KP1[Jf/0
KFe]05MGE`1ZJVX0OGem055AD@1XJ6P0RHV906EUI@1`L700;bl_03<c<`1CDe<00P1GEeL0>`14A4@0
CDe=04I6AP1DE5@0?clo05EEE@13@d<0EEEE03Ti>@0h>3P0=CDe02l_;`1;Bd/0?Shn05ADE00n?Sh0
AdM70451@@1IFET0D51@04i>CP19BDT0=cLg07]kN`1KFe/0?3`l05aLG01QHF40Jf][04=3@`1=CDd0
N7Qh04E5A@1GEeL0B4Q806EUI@0_;bl0=CDe04M7A`0m?Cd0=CDe05QHF00e=CD0;Rh^03Xj>P0_;bl0
>SXj04E5A@0n?Sh0E5AD03Ph>01<C4`0=3@d03Lg=`11@D40@T9207UiN@19BDT0Cdm?0080CDe=06/0
EEEE05=CD`25QHD0@T92059BDP1<C4`0KFe]05EEE@1mOGd092@T096AT@1@D500GUiN06a/K02FUYH0
De=C07IfMP1@D500C4a<08:2PP34a<@0CTi>0<[:bP1NGUh0DE5A06ATI02LW9`0IFEU051@D02:RXX0
?Cdm05iNGP14A4@0@T92034a<@1?Cdl0AdM7069RHP1:BTX0DE5A04a<C00m?Cd0QHF501`L700m?Cd0
JVYZ02l_;`1XJ6P0QhN703/k>`1BDU80DE5A09VIV@3EeMD0J6QX08>3P`2EUID0G5aL06MWI`2OWil0
Cdm?06IVIP1PH6007AdM06][J`0S8b<0CTi>03De=@0k>c/0D51@03Lg=`1PH600CTi>05YJFP26QXH0
kno_05mOG`2ATI40GUiN065QH@0o?cl0DU9B06e]K@0g=cL0JFUY055AD@1CDe<0EEEE06=SH`1hN7P0
@d=303dm?@1WIfL0Bd];06IVIP1=CDd0G5aL0451@@1<C4`0>CTi03De=@0n?Sh0>CTi038b<P0l?3`0
EEEE0410@00203/k>`0E04Q8B01RHV80<c<c04=3@`0d=3@0C4a<04A4A017AdL0MgMg05EEE@1UIFD0
?Shn04U9B@0h>3P0CTi>0451@@1IFET0AdM705YJFP18B4P0F5QH0080FUYJ0280IfMW0820P01OGel0
CTi>0451@@1LG5`0OWin05=CD`0`<300?Shn03Lg=`18B4P0FEUI07UiN@0f=SH0HV9R04m?C`2e]KD0
Cdm?04A4A00l?3`0GemO04m?C`1IFET0GUiN079bLP1/K6`0=CDe05UIF@1PH600YjNW05UIF@1GEeL0
;bl_0P12@T801P25QHD0IFEU069RHP1cLg<0GemO08>3Pa3ooooo000>ooooo`1:08B4Q02HV9P0f=SH
04Y:BP1TI6@0LW9b0<K6aP19BDT0;Bd]08b<S021PH40`l?30410@00d=3@0A4A4071`L022PX80TI6A
03@d=00`<300ADE507AdM01]KFd0OGem03hn?P1DE5@0IFEU0:>SX`1@D500Kfm_038b<P1OGel0KVi^
04m?C`19BDT0JVYZ030`<01EEED0S8b<04e=C@0];Bd0B4Q803`l?01NGUh0TY:B02l_;`0c<c<0=CDe
02@T901TI6@0IfMW01hN7P0f=SH0>c/k03@d=01bLW80De=C02HV9P0k>c/0A4A4069RHP2GUiL0>3Ph
055AD@1OGel0Kfm_08B4Q02UYJD08R8R02h^;P24Q8@0H61P0;Vi^@1^KVh201/K6`3:051@D02QXJ40
[Jf]03@d=00T92@0;Rh^05eMG@1]KFd0WIfM02XZ:P0];Bd0BTY:06YZJP2EUID0IFEU02h^;P0g=cL0
LW9b05eMG@1/K6`0@d=303<c<`1CDe<0N7Qh0451@@1WIfL0CTi>03dm?@2DU9@0O7al04i>CP1bLW80
ATI603Ti>@1IFET0KFe]04U9B@1fMWH0=SHf06=SH`2LW9`0EeMG04E5A@1hN7P0:2PX03dm?@1cLg<0
ATI60410@01LG5`092@T05UIF@1aLG40KFe]04Q8B01XJ6P0EEEE075aL@2LW9`0?3`l05YJFP1BDU80
FUYJ05mOG`2n_[h0FEUI04I6AP1ZJVX0C4a<0:b/[029RHT0=CDe04Q8B01ADE40IFEU0<S8b02/[:`0
9bLW03Ti>@1aLG40JVYZ0<S8b01>CTh0=SHf04i>CP2WYjL0Xj>S0;no_`19BDT0C4a<06i^KP2UYJD0
YjNW09ZJVP0W9bL0ADE508j>SP2NWYh0GemO0:>SX`17AdL0?3`l07UiN@1oOgl0JVYZ07MgM`0a<C40
N7Qh0:^[Z`1DE5@0WYjN071`L00h>3P0`l?309bLW01kNg/0O7al03<c<`1GEeL0LW9b051@D01aLG40
>SXj03hn?P22PX80OGem061PH00`<300AdM70:VYZ@2b/[80@d=305aLG010@400FEUI0:>SX`1LG5`0
EUIF07MgM`1BDU80[Zj^071`L011@D40EEEE04=3@`27QhL0[Zj^0451@@1?Cdl0Fe]K06QXJ02NWYh0
XJ6Q03hn?P1<C4`0FUYJ05IFEP2EUID0SHf=030`<01?Cdl0MgMg04Y:BP2a/K40b<S804E5A@1:BTX0
GUiN04Y:BP20P800/;2`034a<@1HF5P0OGem051@D026QXH0^;Rh03dm?@1RHV80]KFe06i^KP2GUiL0
SXj>030`<01ZJVX0GEeM04U9B@1kNg/0=CDe03hn?P1^KVh0I6AT04I6AP28R8P0?clo04=3@`1[Jf/0
PH6105QHF01aLG40>c/k04m?C`kooooo0540D51@05aLG01?Cdl0?Cdm04e=C@1:BTX0E5AD03hn?P13
@d<0B4Q803lo?`0/;2`0EEEE08b<S01nOWh0IFEU05UIF@1NGUh0FEUI04E5A@0R8R80=SHf03Ph>00/
;2`0BTY:03Xj>P0T92@0@T9204Y:BP1CDe<09bLW03/k>`17AdL0C4a<0492@P0g=cL0EEEE04U9B@0O
7al0=CDe04A4A00a<C40<S8b03De=@0i>CT06a/K030`<00c<c<0AdM703/k>`1;Bd/0:RXZ0410@00c
<c<0BTY:05aLG00j>SX0GemO06YZJP1BDU80A4A405IFEP1CDe<0EUIF05UIF@0k>c/0YjNW06MWI`18
B4P0IfMW05mOG`1bLW80@41004e=C@1UIFD0LG5a06MWI`1ADE40De=C038b<P1BDU800P13@d<0WP0j
>SX0H61P03lo?`0i>CT0CTi>03Ti>@0o?cl0D51@04e=C@1BDU80@T9205IFEP0j>SX0=SHf04A4A01=
CDd0LG5a04E5A@1DE5@0HV9R05EEE@1WIfL0LG5a0:6QX@1;Bd/0D51@05eMG@1lO7`0Hf=S06e]K@1;
Bd/0JFUY05EEE@1;Bd/0ADE5075aL@15ADD0De=C05UIF@1EEED0SXj>0;2`/01]KFd0dm?C05iNGP1A
DE40ADE506a/K013@d<0JFUY0;:b/P16ATH0H61P04A4A01ADE40GUiN05]KF`1GEeL0F5QH06=SH`1D
E5@0@T9207alO02PX:00C4a<04I6AP1nOWh0B4Q805EEE@1MGEd0@T9207AdM01TI6@0ZZZZ0=cLg01N
GUh0BDU904a<C018B4P0EEEE06IVIP0e=CD0GUiN05UIF@0b<S80O7al01lO7`1LG5`0DU9B03Ti>@1A
DE40:b/[03dm?@1SHf<0H61P04E5A@1EEED0RXZ:0=oOg`1>CTh0=3@d03Hf=P1KFe/0MgMg04E5A@1f
MWH0?Cdm059BDP1ADE40@d=304I6AP0l?3`0@d=306i^KP1BDU80EUIF03dm?@1WIfL0Cdm?07IfMP0h
>3P0=cLg04I6AP12@T80:BTY02/[:`1QHF40?Shn04I6AP1CDe<0DU9B06e]K@12@T80JVYZ03Xj>P1<
C4`0@d=302DU9@1QHF40<C4a04];B`0Q8B40JVYZ04E5A@16ATH0=SHf04e=C@16ATH0IFEU04m?C`1E
EED0Hf=S07MgM`1>CTh0GUiN05UIF@0X:2P0@T920P10@4001P12@T80@41008>3P`11@D40A4A402LW
9`80EUIF01/0MWIf06IVIP3AdM40JVYZ05UIF@15ADD0Xj>S06=SH`1VIVH0MgMg04Q8B01@D500>CTi
06a/K00m?Cd0DU9B04E5A@1NGUh0ATI604Y:BP0j>SX0PH6105eMG@1fMWH0H61P07]kN`25QHD04?oo
ool000kooooo0?l0Jf][06IVIP22PX80M7Ad04U9B@15ADD0IVIV08^;R`0i>CT0BTY:03Lg=`1nOWh0
V9RH034a<@1BDU80@d=303`l?02DU9@0CDe=02<S8`1EEED0L71`03Hf=P1[Jf/0=cLg02XZ:P12@T80
KVi^08F5Q@1YJFT05ADE03dm?@1SHf<0QHF508:2PP2j^[X0?Shn04a<C01>CTh0FUYJ05IFEP28R8P0
@41003@d=01PH600M7Ad075aL@25QHD08B4Q0451@@1oOgl0FUYJ09NGU`0a<C40;Bd]05QHF01JFUX0
JFUY06e]K@0N7Qh0=CDe05UIF@1TI6@0a<C408:2PP0S8b<0@D5107EeM@1MGEd0BTY:02<S8`0a<C40
I6AT09nOW`0l?3`0@T92030`<019BDT0Ng]k07MgM`0/;2`0?Shn03Ti>@12@T80P82006QXJ00a<C40
>CTi03<c<`1`L700HV9R01lO7`0d=3@0>CTi038b<P1dM7@0B4Q8024Q8@0a<C40GEeM03Xj>P24Q8@0
J6QX02LW9`1>CTh0H61P04];B`2CTi<0GUiN02`/;01YJFT0NWYj06e]K@2c/k<0@41003`l?01SHf<0
GUiN09bLW02YZJT0<30`04E5A@1VIVH0PH6109VIV@3>c/h0?Cdm04I6AP1^KVh0Rh^;0:VYZ@2g]kL0
;bl_06=SH`2GUiL0PX:20:VYZ@2?Shl0:BTY04i>CP2GUiL0Vi^K08V9R@0d=3@0:b/[04i>CP2/[:`0
Lg=c07MgM`0f=SH0@D5106=SH`2l_;`0Kfm_0861P@1EEED0B4Q808b<S02[Zj/0A4A4075aL@1OGel0
C4a<0:n_[`1^KVh0@d=307EeM@1<C4`0Cdm?0:j^[P1NGUh0?Shn06QXJ00h>3P0A4A409:BTP0f=SH0
X:2P07]kN`1TI6@0LW9b09:BTP0j>SX0EEEE04i>CP1FEUH0][Jf08b<S00j>SX0HV9R07=cL`2>SXh0
^k^k03Ph>00l?3`0Hf=S07YjNP35aLD0B4Q8024Q8@1NGUh0OGem0<c<c01RHV807AdM04m?C`2`/;00
Ogmo09JFUP0l?3`0BTY:05mOG`1UIFD0N7Qh09fMW@0g=cL0SHf=08j>SP24Q8@0[Jf]055AD@1<C4`0
YZJV075aL@1nOWh0KFe]024Q8@0o?cl0QHF507]kN`2VYZH0QXJ6024Q8@19BDT0XZ:R05YJFP28R8P0
DE5A02/[:`0f=SH0RHV906YZJP1MGEd0DE5A0451@@13@d<0OWin06i^KP1>CTh0IFEU04e=C@1cLg<0
Zj^[06=SH`1<C4`001L0M7Ad05]KF`26QXH0Xj>S04];B`13@d<0ADE506YZJP1mOGd0FUYJ02LW9`1<
C4`0KVi^05EEE@2[Zj/0IFEU020P800b<S80K6a/06YZJP2_[jl0>CTi034a<@0>ooooo`0B05IFEP1X
J6P0Kfm_06ATI01CDe<0CTi>06][J`0n?Sh0IFEU05ADE014A4@0?Cdm05EEE@1OGel0KFe]06ATI01W
IfL0AdM70P15ADD0o`0`<300CTi>03`l?00f=SH0DE5A051@D00o?cl0ADE506IVIP1RHV80B4Q804E5
A@1DE5@0HV9R061PH01IFET0HV9R07YjNP16ATH0BDU9069RHP19BDT0CDe=04M7A`1BDU80=CDe04m?
C`18B4P0FEUI03dm?@1ADE40<30`0451@@0Z:RX09bLW03Ti>@0d=3@0>SXj03@d=00k>c/0>SXj051@
D013@d<0D51@061PH014A4@0IfMW055AD@0n?Sh0F5QH0:2PX02BTY80ADE505=CD`0i>CT0<S8b04=3
@`0o?cl0CTi>01dM7@0b<S80=3@d03De=@0_;bl0De=C02h^;P10@400?clo03Xj>P0n?Sh0?clo04m?
C`1@D500E5AD05IFEP19BDT0EEEE05]KF`1>CTh0LG5a05mOG`1XJ6P0Fe]K07alO01]KFd0OGem0:2P
X01TI6@0OGem06MWI`1nOWh0QhN707YjNP16ATH0Ogmo05EEE@1RHV80De=C08V9R@1=CDd0LW9b03Lg
=`1LG5`0ZZZZ0<71`@2?Shl0d=3@05MGE`22PX80LG5a07QhN015ADD0Hf=S09fMW@1DE5@0FUYJ04Y:
BP1<C4`0EUIF06i^KP1EEED0De=C04Q8B01QHF40>3Ph07IfMP2i^KT0?Cdm038b<P1>CTh0;Rh^05YJ
FP1jNWX0@4100:6QX@1VIVH0QHF50;>c/`1@D500CDe=05UIF@11@D40Hf=S0:FUY@0n?Sh0JFUY06EU
I@12@T80G5aL0492@P1SHf<0HV9R0492@P1/K6`0:BTY04Y:BP1`L700GemO051@D035aLD0D51@05mO
G`16ATH0Bd];02`/;02HV9P0][Jf04i>CP1:BTX0B4Q806YZJP1]KFd0ATI604A4A01SHf<0CDe=07Ee
M@0i>CT0De=C03Ph>01aLG40@d=303Xj>P16ATH0AdM704a<C00o?cl0<S8b04=3@`1LG5`0>c/k03@d
=00m?Cd0@D5106YZJP0W9bL0@41002TY:@1<C4`0AdM703Ti>@1iNGT0>3Ph05EEE@0N7Qh0?Cdm02PX
:016ATH0<c<c04M7A`0d=3@0Cdm?04I6AP1BDU80Jf][05ADE00i>CT0GEeM05QHF012@T80RXZ:055A
D@1BDU80>3Ph03Hf=P1YJFT0<c<c05=CD`1:BTX0EUIF02h^;P1HF5P0De=C08f=S@1^KVh0DU9B04U9
B@2?Shl0Jf][07YjNP2NWYh0JVYZ06a/K00o?cl0EEEE04e=C@1aLG40BTY:04=3@`0n?Sh0F5QH034a
<@1WIfL0I6AT061PH0000`1`L700LG5a07MgM`0@ooooo`003_ooool05`2OWil0IVIV08:2PP1`L700
FUYJ04A4A01HF5P0IFEU04Y:BP1]KFd0=3@d04U9B@1QHF40B4Q807IfMP1jNWX0@d=304i>CP1ZJVX0
DU9B092@T02HV9P0>c/k0080BDU900X0JVYZ09nOW`0j>SX0;Rh^04A4A01DE5@0UIFE05YJFP0P8200
<c<c0P1FEUH08@1jNWX0T92@01lO7`0Z:RX0Bd];07MgM`1KFe/0MGEe02XZ:P1KFe/0HV9R04a<C01g
MgL0>c/k02DU9@1aLG40L71`0451@@1UIFD0?clo04M7A`1dM7@0DE5A05UIF@1XJ6P0<S8b069RHP1Z
JVX0=3@d04M7A`1CDe<0C4a<06i^KP0203`l?03>06a/K01HF5P0B4Q805EEE@0n?Sh0B4Q806i^KP1B
DU80?3`l05mOG`1ADE40=CDe05QHF019BDT0ATI607EeM@0^;Rh0@D5105iNGP16ATH0MGEe06=SH`0T
92@0A4A406=SH`1GEeL0PX:207EeM@0N7Qh0@D5106a/K01ADE40M7Ad02<S8`0W9bL0ATI608Z:RP1R
HV80JFUY028R8P0];Bd0H61P08N7Q`1]KFd0EUIF024Q8@1:BTX0L71`079bLP1NGUh0QhN702DU9@1O
Gel0K6a/06IVIP1GEeL0U9BD04I6AP1>CTh0TY:B08J6QP1JFUX0OWin034a<@1eMGD0QXJ604];B`1S
Hf<0R8R804M7A`1cLg<0PX:204Y:BP1^KVh0LW9b05YJFP1oOgl0GemO061PH01nOWh0LW9b05QHF01j
NWX0UIFE04U9B@2FUYH0KVi^05eMG@29RHT0Ti>C05ADE02<S8`0IVIV05MGE`2MWId0TI6A05EEE@1e
MGD0GUiN08B4Q02b/[80EEEE04I6AP1UIFD0DE5A0:2PX02ATI40:RXZ04E5A@1`L700Hf=S09RHV016
ATH0=3@d05ADE01TI6@0LW9b08V9R@0g=cL0BDU907IfMP1jNWX0Ogmo06EUI@0_;bl0Ph>308^;R`1W
IfL0Q8B404a<C01ZJVX0QXJ606YZJP2HV9P0FUYJ02h^;P1aLG40AdM706m_K`1`L7009RHV04I6AP2<
S8`0IVIV09>CT`0b<S80?Shn07AdM01YJFT0J6QX06EUI@1:BTX0M7Ad09:BTP1[Jf/0FEUI07AdM01O
Gel0QhN70;>c/`1<C4`0IFEU065QH@1SHf<0Ph>308N7Q`1=CDd0D51@08F5Q@1FEUH0UIFE092@T00m
?Cd0F5QH07=cL`1KFe/0Lg=c09^KV`0n?Sh0De=C05MGE`1QHF40Rh^;06YZJP0j>SX0EEEE05mOG`1;
Bd/0N7Qh06a/K00b<S80LW9b07UiN@1JFUX0Rh^;079bLP0`<300De=C07QhN01UIFD0MWIf04e=C@0d
=3@0HV9R07moO`1LG5`>ooooo`1U05MGE`1WIfL0JVYZ06EUI@1`L700K6a/06MWI`18B4P0HF5Q05eM
G@17AdL0@T9206IVIP1lO7`0P82006QXJ01MGEd0FEUI05MGE`1JFUX0;bl_04E5A@0k>c/0@41005mO
G`1;Bd/0@41004e=C@1YJFT0IVIV04a<C01CDe<0EeMG05YJFP1LG5`0IfMW05UIF@1SHf<0=CDe0492
@P1DE5@0BTY:05QHF00k>c/0BDU903Hf=P19BDT0<C4a05]KF`14A4@0D51@03De=@1=CDd0=CDe03<c
<`14A4@0@d=305iNGP16ATH0DU9B04A4A01ADE40De=C05mOG`1eMGD0IVIV09VIV@1PH600AdM706QX
J01JFUX0JVYZ04M7A`1BDU80L71`04Q8B01ZJVX0BDU905aLG00l?3`0AdM70451@@14A4@0@T9205AD
E00d=3@0?clo04A4A00m?Cd0?3`l04];B`1:BTX0ATI60410@015ADD0@41003<c<`1;Bd/0ATI6069R
HP0k>c/00P1?Cdl0[`1HF5P0JFUY07UiN@2ATI40E5AD059BDP1RHV80OGem06=SH`1jNWX0I6AT06IV
IP1>CTh0Cdm?05QHF01`L700?Cdm079bLP19BDT0DU9B092@T028R8P0SXj>0<O7a`0h>3P0Jf][06IV
IP1cLg<0@T9208Z:RP2UYJD0ATI6055AD@1BDU80A4A403lo?`1QHF40GUiN04Y:BP14A4@0KFe]04e=
C@1eMGD0QhN704e=C@1<C4`0LW9b03Lg=`2<S8`0UYJF02LW9`1oOgl0H61P0:VYZ@39bLT0FEUI061P
H01UIFD0DE5A069RHP20P800@T9205iNGP1KFe/0=CDe05mOG`0^;Rh0G5aL061PH01KFe/0PH610492
@P1ADE40J6QX05aLG01YJFT0YZJV05MGE`2KVi/0F5QH059BDP0d=3@0LG5a07moO`0Z:RX0DU9B04I6
AP1gMgL0Jf][04U9B@1:BTX0Cdm?04U9B@1UIFD0B4Q806=SH`0g=cL0D51@04a<C01EEED0=3@d04M7
A`1?Cdl0=3@d02`/;00b<S80EEEE04];B`12@T80B4Q804i>CP1`L700=CDe05UIF@0h>3P0CTi>04U9
B@0i>CT0Hf=S03hn?P1DE5@0;bl_05EEE@0/;2`0ATI604A4A016ATH0;2`/04M7A`1=CDd0BTY:05QH
F01MGEd0ATI606=SH`1;Bd/0>CTi04=3@`19BDT0FUYJ04E5A@0o?cl0KVi^03De=@0n?Sh0<S8b04M7
A`0n?Sh0LG5a059BDP2RXZ80JFUY05=CD`10@400T92@05MGE`1DE5@0F5QH04a<C01BDU80A4A405mO
G`17AdL0F5QH03/k>`1BDU80@41004a<C017AdL0O7al05QHF01WIfL0FUYJ06a/K01oOgl04?ooool0
00kooooo05D0JFUY07MgM`2c/k<0GEeM06UYJ@1KFe/0QhN70410@00b<S80LG5a06m_K`2_[jl0A4A4
03Hf=P0m?Cd0JFUY06MWI`1jNWX07QhN03lo?`0i>CT0Ogmo06a/K02JVYX0820P04=3@`1OGel0O7al
06ATI01XJ6P08b<S05eMG@1dM7@0BTY:06YZJP2ZZZX0:b/[04e=C@1EEED0FEUI05ADE01YJFT09BDU
04M7A`1_Kfl0BTY:04A4A013@d<07alO09ZJVP2HV9P0;Bd]051@D00W9bL0820P05UIF@1XJ6P0<C4a
03@d=00e=CD0BTY:069RHP0d=3@0CTi>03`l?01:BTX0IFEU092@T00Z:RX0>SXj0492@P1IFET0X:2P
08J6QP0S8b<0A4A404I6AP1cLg<0Vi^K04E5A@0^;Rh0<30`05EEE@1fMWH0Ti>C00809BDU01h0=SHf
06UYJ@27QhL0CTi>024Q8@17AdL0CTi>03Hf=P1UIFD08B4Q020P801BDU80YZJV08Z:RP1lO7`0Fe]K
03hn?P1oOgl0KFe]05IFEP1TI6@08B4Q0492@P2YZJT0Jf][06MWI`1FEUH0=CDe0410@02FUYH20410
@02O04i>CP0M7Ad0ATI607moO`1ADE40CDe=06ATI00f=SH0CDe=07IfMP1iNGT0@41005MGE`16ATH0
>3Ph07MgM`1PH600Hf=S069RHP0d=3@0@d=309VIV@19BDT0BTY:03Xj>P18B4P0LG5a09VIV@0`<300
?Cdm04=3@`1HF5P0/k>c09fMW@0b<S80ATI605IFEP1jNWX0cLg=05YJFP0g=cL0HV9R051@D01YJFT0
gMgM07YjNP0W9bL0IFEU09^KV`1fMWH0^k^k03dm?@0^;Rh0OWin0:VYZ@1XJ6P0SHf=03Ti>@1ADE40
XJ6Q05IFEP2CTi<0U9BD03<c<`2ATI40NWYj09:BTP34a<@0KFe]04e=C@3=cLd0/;2`0:VYZ@1XJ6P0
9bLW05mOG`1ZJVX0GEeM0820P00f=SH0AdM70861P@1`L700Lg=c02<S8`0_;bl0RXZ:0;Rh^01EEED0
OWin02/[:`11@D40Xj>S06EUI@1GEeL0F5QH03Ti>@1kNg/0MGEe0410@02QXJ40;2`/07MgM`1kNg/0
=cLg04Y:BP1FEUH0BTY:05mOG`1dM7@0?Shn071`L019BDT0@D510:2PX01_Kfl0:RXZ04a<C01ADE40
=CDe07UiN@2o_kl0>c/k059BDP2<S8`0De=C07alO02h^;P0?Cdm04U9B@1nOWh0GEeM08R8R02CTi<0
<C4a05ADE02XZ:P0ATI60820P01SHf<09BDU04=3@`29RHT0G5aL0;bl_01IFET0<C4a04Y:BP2FUYH0
LG5a09BDU00R8R807AdM055AD@2RXZ80D51@061PH00m?Cd0>SXj00kooooo0>l0EeMG06YZJP1NGUh0
E5AD05MGE`1CDe<0H61P03Hf=P1XJ6P0G5aL04I6AP0l?3`0KFe]0861P@2EUID0NGUi06EUI@1EEED0
FEUI05]KF`0K6a/0@D5102XZ:P0a<C40B4Q804I6AP0R8R80@d=306MWI`1KFe/0ADE504Y:BP1;Bd/0
Cdm?05EEE@1:BTX0DU9B065QH@0g=cL0AdM7055AD@10@400BDU904=3@`0m?Cd0:RXZ03Ti>@0k>c/0
G5aL03`l?00d=3@0ATI605EEE@0R8R80DE5A04U9B@12@T80@D510410@01?Cdl0DE5A06YZJP1=CDd0
De=C06e]K@1YJFT0W9bL059BDP11@D40Hf=S08j>SP2JVYX0HF5Q06YZJP1GEeL0>3Ph04Y:BP0g=cL0
EeMG02LW9`11@D40=SHf03`l?014A4@0Fe]K0410@017AdL0?clo04E5A@1EEED0Bd];059BDP1UIFD0
CTi>04];B`17AdL0GEeM06EUI@16ATH0FEUI04];B`1MGEd0AdM707YjNP1VIVH0Ogmo0:FUY@1BDU80
K6a/04m?C`1fMWH0Lg=c0;Rh^01XJ6P0Hf=S071`L01UIFD0ADE506IVIP1[Jf/0R8R808V9R@2FUYH0
b/[:0:b/[02l_;`0e]KF069RHP31`L40QhN7065QH@1GEeL0VYZJ08^;R`0l?3`0@41006EUI@0f=SH0
CTi>051@D01ADE40=3@d04Y:BP1jNWX0AdM708:2PP1oOgl0GUiN05]KF`1LG5`0=cLg08B4Q01jNWX0
>c/k05eMG@1ADE40`L710=WIf@1^KVh0CTi>06ATI01ZJVX0LW9b09ZJVP0n?Sh0QhN705IFEP0[:b/0
Fe]K034a<@1OGel0DU9B05aLG02=SHd0@D5105IFEP1RHV80EEEE06UYJ@2b/[80FEUI08b<S01RHV80
C4a<04I6AP2>SXh0Shn?04m?C`1KFe/0BTY:08b<S01RHV80?clo0492@P1>CTh0CDe=08V9R@18B4P0
EeMG03hn?P24Q8@0EEEE051@D01ADE40Bd];04M7A`13@d<0>c/k0410@01IFET0De=C04Y:BP1?Cdl0
CTi>07MgM`0_;bl0DE5A03Ti>@1BDU80=cLg03Ti>@1ADE40?clo0410@00T92@0CTi>03De=@17AdL0
@T9203Lg=`0i>CT0K6a/03Ti>@17AdL0EeMG059BDP0h>3P0IfMW06EUI@0204Q8B00B059BDP1GEeL0
>SXj04A4A01LG5`0=SHf0492@P0i>CT0ATI602h^;P1<C4`0NWYj09FEU@1KFe/0AdM706a/K02@T900
MGEe0P2HV9P04@1UIFD0CDe=069RHP1bLW80Ogmo07QhN012@T80Fe]K04];B`1LG5`0H61P08b<S01F
EUH0FUYJ06m_K`1eMGD0L71`013ooooo000>ooooo`1X09>CT`1SHf<0U9BD0820P01ADE40CDe=06IV
IP1_Kfl0EEEE06UYJ@0d=3@0LW9b06i^KP0l?3`0EUIF03<c<`0j>SX0L71`061PH00W9bL0JFUY03lo
?`1;Bd/0IfMW028R8P12@T80HF5Q04e=C@1RHV80K6a/01PH600f=SH0Fe]K03<c<`23Ph<0]KFe034a
<@0k>c/0JVYZ05]KF`1IFET0WinO06IVIP0/;2`0JFUY06EUI@23Ph<0QHF502@T900o?cl0ZJVY05=C
D`2_[jl0CDe=02LW9`13@d<0Hf=S07MgM`1PH60061PH04i>CP1PH600DU9B0;Ng]`1^KVh08b<S04A4
A01aLG40FUYJ08B4Q00];Bd0>CTi05]KF`27QhL0FUYJ07=cL`0[:b/0ATI607AdM02BTY80Fe]K06e]
K@0/;2`0@T9207alO01iNGT0<C4a075aL@0M7Ad0ADE508F5Q@0a<C40@41003dm?@0S8b<0HV9R06QX
J01ADE40K6a/05eMG@0O7al0G5aL06m_K`0c<c<203hn?P0405eMG@1kNg/0BTY:02HV9P80CDe=00H0
KFe]0861P@0b<S80<c<c06QXJ01MGEd209BDU02N02@T900S8b<0G5aL05ADE029RHT0Y:BT02l_;`0`
<300@d=307MgM`29RHT0SXj>02LW9`19BDT0Bd];061PH02h^;P0Kfm_038b<P1;Bd/0HV9R08B4Q02K
Vi/0@D5103Ph>01bLW80VIVI061PH01VIVH0>c/k04A4A01fMWH0/;2`06UYJ@2@T900HV9R04=3@`1o
Ogl0/K6a04];B`26QXH0@T9204=3@`2BTY80g=cL061PH02CTi<0A4A403hn?P1gMgL0VYZJ07EeM@2A
TI40<C4a04i>CP1PH600>SXj07MgM`1RHV80@41006e]K@2g]kL0Jf][07]kN`13@d<0Bd];0:>SX`2g
]kL0?clo06=SH`1UIFD0R8R80:b/[00c<c<0=SHf05aLG01_Kfl0a/K608^;R`0`<300AdM7079bLP2T
Y:@0GUiN02/[:`1DE5@0BDU905QHF02TY:@0:b/[034a<@1NGUh0KFe]09NGU`2_[jl0>c/k075aL@1O
Gel0P8200<c<c01^KVh0BDU90:j^[P1kNg/0Y:BT08V9R@16ATH0EUIF08N7Q`1cLg<0Rh^;051@D00^
;Rh0=cLg0::RXP1/K6`0Y:BT07UiN@0b<S80;Bd]08>3P`1hN7P0Jf][05iNGP0Z:RX0?clo09bLW02Y
ZJT0GemO03`l?00X:2P0B4Q808N7Q`1:BTX0EeMG0492@P0k>c/0Kfm_09^KV`1;Bd/0G5aL0492@P0a
<C40Cdm?08F5Q@13@d<0Bd];06m_K`1<C4`0_;bl07alO012@T80@D5103`l?01ADE40WinO08N7Q`12
@T8>ooooo`0f06][J`1_Kfl0KFe]06IVIP1]KFd0IVIV07YjNP15ADD0NGUi05eMG@17AdL0>c/k07=c
L`1kNg/0RXZ:09BDU01bLW80E5AD05iNGP1MGEd0:BTY04m?C`16ATH0@T9205=CD`1DE5@0:RXZ0451
@@1oOgl0Ph>304m?C`1PH600L71`05YJFP1PH600BTY:06MWI`1nOWh0:BTY04Y:BP1GEeL0@41005UI
F@12@T80BDU902HV9P1<C4`0<S8b06ATI00l?3`0<S8b02d];@1:BTX0:BTY0P0f=SH0>`1ADE40>c/k
04i>CP19BDT0A4A405EEE@1DE5@0ADE506i^KP1fMWH0YJFU03lo?`1<C4`0FUYJ04a<C01HF5P0=CDe
04];B`1VIVH0<S8b05mOG`14A4@0MWIf03<c<`14A4@0=cLg03dm?@0k>c/0E5AD03<c<`11@D40;bl_
04e=C@0e=CD0>3Ph04m?C`0a<C40@D5103Hf=P14A4@09bLW04a<C00i>CT0De=C02TY:@15ADD0@D51
05IFEP19BDT0TI6A09JFUP1>CTh0GUiN05MGE`1hN7P0IFEU07YjNP1aLG40QHF50080BTY:04T0<30`
04m?C`0o?cl0OGem03Ph>01GEeL0hn?S08V9R@3<c<`0`L7105IFEP2JVYX0N7Qh08N7Q`11@D40JVYZ
07]kN`1dM7@0BTY:04];B`17AdL0@41003`l?01IFET0Cdm?03`l?01KFe/0>SXj07IfMP28R8P0AdM7
03hn?P1WIfL0=CDe092@T021PH40>c/k07=cL`1YJFT0O7al08^;R`1NGUh0K6a/06IVIP1GEeL0IVIV
08V9R@0h>3P0MGEe05=CD`12@T80H61P05EEE@29RHT0De=C05]KF`2:RXX0Cdm?04];B`1/K6`0EeMG
07=cL`2VYZH0ADE509BDU01aLG40Shn?030`<01TI6@0F5QH05ADE024Q8@0De=C0080EUIF03/0GEeM
05UIF@1?Cdl0?3`l06EUI@16ATH0G5aL02LW9`1<C4`0@d=304i>CP0m?Cd0FEUI05aLG00g=cL0;2`/
03lo?`1[Jf/0CDe=03hn?P1PH600F5QH06i^KP16ATH0D51@03/k>`1[Jf/0A4A402l_;`19BDT0IVIV
05QHF00T92@0CTi>01/K6`1QHF40?3`l03/k>`0g=cL0D51@04=3@`0o?cl0ATI604U9B@0j>SX0Ng]k
05eMG@0Z:RX0@d=303/k>`1ZJVX0B4Q804e=C@1IFET0=SHf07emO@15ADD0?Shn034a<@0205=CD`0I
08b<S01FEUH0Fe]K061PH02FUYH0Bd];04Y:BP1<C4`0HF5Q051@D019BDT0FEUI05YJFP1TI6@0HV9R
06MWI`14A4@0CTi>04];B`1[Jf/0B4Q806YZJP1ADE40EEEE09^KV`0@ooooo`003_ooool0C`2JVYX0
Bd];05MGE`1iNGT0F5QH03Ph>01:BTX0GemO06=SH`1oOgl0<30`03Ph>01FEUH0JFUY09^KV`0k>c/0
<C4a05EEE@1MGEd0K6a/0<G5a@17AdL0:RXZ079bLP14A4@0[:b/0>7Qh@0b<S80@D5107EeM@1>CTh0
[jn_05EEE@0H61P0=CDe069RHP1[Jf/0YZJV096AT@0I6AT0;bl_03dm?@1bLW80GUiN05EEE@0O7al0
@41005QHF00l?3`0DU9B02d];@0h>3P0CTi>065QH@0h>3P0GEeM04M7A`1@D500UiNG05mOG`1KFe/0
GUiN04E5A@1hN7P0SXj>03Lg=`0n?Sh0A4A403lo?`1TI6@0ATI6028R8P15ADD0?3`l04=3@`1lO7`0
A4A402`/;01:BTX00P1jNWX0a@1cLg<0>CTi02LW9`18B4P0K6a/04=3@`2YZJT09bLW02HV9P1ZJVX0
EUIF07UiN@1SHf<06ATI03dm?@2PX:00Q8B40;:b/P15ADD0:BTY04e=C@1TI6@0MWIf08N7Q`0P8200
?3`l06EUI@2>SXh0GemO04E5A@0U9BD0>c/k08:2PP2YZJT0EUIF03Lg=`0c<c<0FUYJ05mOG`17AdL0
=SHf05UIF@0R8R80Cdm?07alO021PH40Fe]K09FEU@1>CTh0B4Q806][J`1CDe<0?Shn04e=C@0/;2`0
Fe]K0820P01;Bd/0E5AD06EUI@0U9BD0Fe]K06UYJ@0h>3P0Ng]k05UIF@1?Cdl0Rh^;04M7A`1DE5@0
IfMW04A4A01`L700Ogmo06ATI00e=CD0GEeM07=cL`22PX80WinO06=SH`0g=cL0D51@092@T027QhL0
Zj^[07emO@11@D40E5AD061PH02WYjL0f=SH03`l?012@T80QHF505mOG`2JVYX0QXJ601dM7@1@D500
W9bL09ZJVP2YZJT0?clo038b<P1JFUX0WinO0:b/[02DU9@0ATI605mOG`22PX80NWYj09fMW@1IFET0
>3Ph06ATI01mOGd0QHF509^KV`17AdL0RXZ:08J6QP1HF5P0V9RH04=3@`0S8b<0XJ6Q06e]K@1^KVh0
IFEU02TY:@1:BTX0WYjN079bLP1YJFT08R8R03hn?P1cLg<0^KVi06i^KP11@D40<C4a065QH@1bLW80
@D51059BDP1NGUh0Cdm?0:6QX@27QhL0<30`04M7A`1:BTX0ADE509FEU@2VYZH0E5AD04Q8B00j>SX0
BTY:08F5Q@2WYjL0=cLg02d];@1PH600JVYZ08>3P`1NGUh092@T03Xj>P1iNGT0IVIV09ZJVP10@400
<C4a03Xj>P1cLg<0F5QH0:>SX`1XJ6P0>CTi04=3@`2ATI40H61P07YjNP2?Shl0<30`08:2PP1TI6@0
ATI60:RXZ01WIfL0=3@d071`L02l_;`0I6AT00kooooo09l0IfMW07alO01cLg<0K6a/069RHP1SHf<0
NGUi06m_K`1VIVH0FEUI06m_K`1[Jf/0Ng]k092@T01/K6`0Shn?06=SH`1GEeL0F5QH06=SH`0g=cL0
@d=3030`<019BDT0IfMW05=CD`1bLW80JFUY05iNGP1WIfL0AdM7055AD@1MGEd0Hf=S071`L01fMWH0
Hf=S05]KF`0^;Rh0C4a<04i>CP1HF5P0FEUI03lo?`0f=SH0@d=303hn?P0_;bl0HF5Q05eMG@0e=CD0
<c<c05ADE00S8b<0Bd];05YJFP14A4@0@41004i>CP19BDT0HV9R06YZJP1IFET0EeMG05QHF01RHV80
S8b<03dm?@16ATH0H61P05MGE`1QHF40?clo05UIF@0f=SH0>3Ph05QHF013@d<0G5aL02LW9`1:BTX0
>3Ph04M7A`1=CDd0J6QX03Ti>@13@d<0;bl_03Xj>P0_;bl0=cLg06=SH`0e=CD0A4A403dm?@1?Cdl0
:2PX061PH017AdL0IFEU03Ph>01ADE40A4A405eMG@1QHF40Shn?08N7Q`1?Cdl0GemO04];B`1`L700
OWin0:^[Z`1MGEd0IfMW05eMG@1ADE40CTi>05]KF`0e=CD0I6AT02HV9P0j>SX0S8b<069RHP39bLT0
ZJVY04Y:BP2IVIT0J6QX08F5Q@12@T80IfMW02l_;`0h>3P0E5AD069RHP16ATH0?Cdm03/k>`1LG5`0
@T9203hn?P1nOWh0?Cdm08b<S01lO7`0De=C04Q8B01NGUh0=3@d0:RXZ02ATI404a<C065QH@1BDU80
`L710<[:bP1;Bd/00P10@400H`1DE5@0GEeM07MgM`0W9bL0SHf=04];B`0c<c<0BTY:03De=@1LG5`0
@D51079bLP22PX80DU9B04m?C`1JFUX0>3Ph08Z:RP2YZJT0FEUI0:n_[`1MGEd0C4a<02@T901YJFT0
EeMG02`/;019BDT0<C4a05YJFP0k>c/0C4a<04U9B@1GEeL0A4A407=cL`0e=CD0E5AD02PX:01_Kfl0
CTi>05eMG@14A4@0ADE506IVIP1:BTX0<30`03Ph>00n?Sh0CTi>03Lg=`16ATH0BDU9069RHP0[:b/0
H61P038b<P16ATH0=3@d03/k>`1?Cdl0B4Q802l_;`0m?Cd0Cdm?04A4A01ADE40?clo03De=@0_;bl0
Bd];03Hf=P0l?3`0Fe]K05IFEP0o?cl0IVIV06][J`1?Cdl0@d=3069RHP2OWil0Kfm_061PH01IFET0
=cLg059BDP0h>3P0@d=3059BDP1RHV80E5AD0820P01_Kfl0MgMg05=CD`1[Jf/08R8R05eMG@0204A4
A00703<c<`17AdL0Cdm?04];B`1LG5`0>SXj04a<C00204E5A@070492@P1oOgl0ATI6069RHP1@D500
G5aL07UiN@0@ooooo`003_ooool0o`1eMGD0MWIf06UYJ@1?Cdl0PH6105]KF`2=SHd0<S8b04m?C`1_
Kfl0Cdm?069RHP0f=SH0;Rh^069RHP2YZJT0A4A403`l?00_;bl0?clo06i^KP24Q8@0BTY:04E5A@0E
5AD0:b/[07inOP1lO7`0@D5104A4A00S8b<0G5aL0861P@0Y:BT0=3@d04i>CP0e=CD0De=C096AT@0o
?cl09RHV05=CD`0l?3`0P82006i^KP0H61P0;bl_04E5A@0e=CD0NWYj05YJFP0Z:RX0BTY:03hn?P0h
>3P0UIFE05eMG@0m?Cd0A4A405IFEP1mOGd0[Jf]02d];@19BDT0EUIF05QHF02EUID0YJFU01lO7`0n
?Sh0Jf][06e]K@3<c<`0I6AT02@T901JFUX0WinO0820P02FUYH0820P04I6AP13@d<0EUIF0820P01Z
JVX061PH04I6AP1VIVH0K6a/05iNGP0`<300=CDe04m?C`1GEeL0:BTY05=CD`0J6QX0?Shn06YZJP1Y
JFT0>c/k05UIF@1:BTX0AdM7079bLP15ADD0<c<c04A4A00c<c<0CTi>06ATI00V9RH0:RXZ03Xj>P0l
?3`0KVi^06][J`0R8R80@D5104e=C@1OGel0Cdm?08n?S`0S8b<0<C4a05eMG@1?Cdl0G5aL07moO`1Z
JVX0IFEU06][J`1@D500V9RH08b<S00Y:BT0MGEe05MGE`1NGUh0/k>c09NGU`0_;bl0D51@06ATI01G
EeL0gmoO0:^[Z`0Q8B40Q8B408>3P`2YZJT0emOG05QHF00_;bl0J6QX08R8R020P800b<S8061PH013
@d<0S8b<0:BTY022PX80P820038b<P1FEUH0WYjN0;>c/`1WIfL0J6QX03<c<`1EEED0Z:RX0:n_[`1H
F5P0DU9B04];B`1[Jf/0VIVI04M7A`1^KVh0H61P03Lg=`2GUiL0JFUY04Y:BP1]KFd0>c/k04U9B@2I
VIT0GEeM05]KF`11@D40@T920861P@1eMGD0<C4a04i>CP1;Bd/0E5AD0:f][@15ADD0Fe]K06IVIP1_
Kfl0/k>c07emO@0P8200GUiN08f=S@2i^KT0TI6A02TY:@0U9BD0Hf=S092@T02[Zj/0;2`/02XZ:P1D
E5@0QHF50:JVYP1cLg<0<C4a05]KF`1^KVh0NGUi07]kN`23Ph<0?3`l05YJFP23Ph<0BTY:0:JVYP1L
G5`09RHV04M7A`1iNGT0Hf=S0;:b/P24Q8@09RHV04E5A@2ATI40PH6107QhN01eMGD0:2PX05]KF`1U
IFD0EEEE07QhN01cLg<0<c<c059BDP005`1oOgl0DE5A06a/K00f=SH0;bl_04A4A01RHV80Cdm?065Q
H@0`<300?clo0451@@1kNg/0D51@07moO`0n?Sh0A4A404U9B@1]KFd0>3Ph04e=C@18B4P0HF5Q00ko
oooo00P0J6QX05YJFP13@d<0@d=3051@D01EEED0E5AD03Xj>P80EEEE00/0?3`l03Lg=`1JFUX0GEeM
05MGE`1XJ6P0Bd];04M7A`0o?cl0@T9202`/;0020492@P0G03Hf=P1TI6@0De=C04M7A`1BDU80HV9R
075aL@12@T80D51@05UIF@1PH600AdM704a<C01OGel0IfMW03dm?@1>CTh0DE5A03/k>`1DE5@0@410
03Lg=`0[:b/00P0j>SX04`1ADE40=CDe03`l?00c<c<0@T9201HF5P0n?Sh0<S8b03Xj>P0m?Cd0LW9b
04U9B@1/K6`0FEUI03Ti>@1VIVH0EUIF05QHF01iNGT00P12@T80X01OGel0Shn?06YZJP0e=CD0CDe=
03<c<`1QHF40D51@04M7A`0i>CT0<C4a04=3@`10@400>c/k0451@@1OGel0=cLg03lo?`0i>CT0Bd];
03Ph>00e=CD0EeMG03De=@0j>SX0;bl_04e=C@14A4@0BDU905=CD`1KFe/0@41004Y:BP18B4P0F5QH
04Y:BP2QXJ40L71`06=SH`1dM7@0Cdm?069RHP2;Rh/0VYZJ05MGE`20P800IFEU05EEE@16ATH0FUYJ
04=3@`1KFe/0>CTi061PH02/[:`0JVYZ0;2`/020P800BTY:0=KFeP1/K6`0IVIV061PH02CTi<0DU9B
04Y:BP0j>SX0GemO051@D023Ph<0I6AT06UYJ@1VIVH0HV9R07=cL`1=CDd0QHF50861P@1OGel0DE5A
05EEE@16ATH0PH6105mOG`0Q8B40M7Ad05QHF02NWYh0UiNG03dm?@1PH600?3`l055AD@17AdL0M7Ad
02DU9@27QhL0Jf][04U9B@1OGel0Fe]K07MgM`19BDT0F5QH05MGE`1@D500E5AD05mOG`17AdL0R8R8
0:b/[01EEED0M7Ad06IVIP1MGEd0<30`092@T01GEeL0:b/[04M7A`0j>SX0R8R804A4A017AdL0Bd];
04A4A00f=SH0EUIF0492@P1HF5P0<30`05QHF00k>c/0?clo034a<@0n?Sh0BTY:02`/;00e=CD0@410
04I6AP1>CTh0ATI605eMG@1HF5P0GEeM04M7A`1HF5P0A4A405]KF`0k>c/0?clo055AD@1KFe/0A4A4
03De=@0o?cl05ADE069RHP0e=CD203Hf=P0C06=SH`0l?3`0GEeM05YJFP0n?Sh0AdM705eMG@1QHF40
>c/k051@D00j>SX0FUYJ03<c<`1FEUH0@D5103`l?01YJFT0;Bd]0451@@0204M7A`0G05mOG`1lO7`0
AdM706=SH`1HF5P0HF5Q02@T902g]kL0EeMG04M7A`0c<c<0G5aL05eMG@1:BTX0GUiN03Hf=P1FEUH0
@d=304i>CP1DE5@0Ng]k04Y:BP1XJ6P00P1IFET00`2=SHgoooooooooo`0>ooooo`003_ooool0HP1n
OWh0M7Ad0;Rh^01KFe/0CTi>05=CD`2e]KD0DU9B03Hf=P1SHf<0EEEE0861P@0j>SX0;bl_04];B`1Y
JFT0M7Ad06=SH`0J6QX0=cLg04m?C`17AdL0E5AD0::RXP0L71`0<S8b03Ti>@1WIfL0VIVI07MgM`0b
<S80G5aL06IVIP1DE5@0Ph>30:6QX@0Q8B40AdM705UIF@1aLG40NGUi0:>SX`0d=3@0=SHf09VIV@0m
?Cd0GemO05]KF`0Q8B40EEEE07YjNP17AdL0U9BD030`<00P8200BTY:08R8R01[Jf/0B4Q8020P800n
?Sh0I6AT061PH01[Jf/0?3`l01hN7P0c<c<0QXJ605ADE01<C4`07QhN030`<01?Cdl0QhN703lo?`1M
GEd0<S8b0410@01XJ6P0Fe]K01dM7@11@D40<30`09bLW02][Jd0?3`l02`/;01=CDd0>c/k07YjNP1<
C4`0820P03Ti>@10@400<C4a08J6QP0[:b/08B4Q0P1HF5P0/P15ADD0[Zj^07EeM@0n?Sh0G5aL051@
D01[Jf/0[jn_05eMG@0n?Sh0LW9b055AD@1eMGD0^k^k04M7A`11@D40Vi^K04i>CP1iNGT0G5aL02LW
9`0o?cl0Shn?08F5Q@1dM7@0MWIf02h^;P0l?3`0F5QH096AT@2EUID0NGUi030`<01PH600U9BD05mO
G`2TY:@0CDe=03`l?020P800`<3006ATI01WIfL0<c<c03Xj>P2LW9`0e]KF06IVIP19BDT0AdM705AD
E02JVYX0Vi^K05aLG01WIfL0BDU9065QH@2CTi<0QHF503<c<`1:BTX0FEUI06ATI033`l<0E5AD034a
<@1?Cdl0N7Qh071`L02;Rh/0=SHf03Xj>P1IFET0R8R808j>SP27QhL07alO059BDP1bLW80EUIF08:2
PP1cLg<09bLW04E5A@1]KFd0HV9R09jNWP1CDe<0@D51055AD@1OGel0_;bl08:2PP0H61P0AdM706m_
K`1QHF40Ng]k02d];@0g=cL0Hf=S0861P@2JVYX07alO03/k>`21PH40YZJV04U9B@1gMgL0:RXZ06QX
J02PX:00Ng]k079bLP22PX80>c/k08Z:RP1VIVH0F5QH06ATI00X:2P0CTi>079bLP1MGEd0RXZ:05YJ
FP0^;Rh0FEUI07]kN`1NGUh0MWIf05]KF`0d=3@0KVi^092@T00g=cL0F5QH04M7A`16ATH0Ogmo09NG
U`0i>CT0BTY:04I6AP1;Bd/0PH610;no_`0o?cl0Cdm?05QHF01EEED0QHF50;Bd]00_;bl0ADE505EE
E@1NGUh0X:2P05YJFP0V9RH0De=C07IfMP28R8P0d=3@03Ph>00];Bd0FEUI04m?C`1GEeL0UiNG04I6
AP0X:2P0Cdm?069RHP1lO7`0ZJVY08:2PP0l?3`>ooooo`0l04M7A`1kNg/0J6QX06YZJP1FEUH0Fe]K
07UiN@1:BTX0N7Qh05iNGP1WIfL0HV9R06][J`1/K6`0R8R807MgM`1YJFT0DE5A05ADE01CDe<0:b/[
03`l?00n?Sh0ATI605QHF01WIfL0L71`08J6QP21PH40Hf=S04e=C@1VIVH0LW9b06ATI01OGel0EEEE
06ATI01/K6`0=cLg04Y:BP1LG5`0BTY:06UYJ@10@400A4A40451@@0k>c/0;2`/059BDP0m?Cd0:RXZ
04Q8B01MGEd0;bl_03/k>`0g=cL0BDU902d];@0b<S80;Rh^0P1;Bd/0T`11@D40GEeM05EEE@1gMgL0
WinO03lo?`1=CDd0ATI6028R8P13@d<0ADE505EEE@19BDT0;2`/04=3@`18B4P0AdM702HV9P0Z:RX0
<c<c03Xj>P0l?3`0Fe]K02h^;P0];Bd07AdM0451@@0/;2`09bLW05QHF00P8200<C4a02LW9`19BDT0
7AdM03dm?@0o?cl0DE5A02/[:`0g=cL0?3`l03Lg=`0i>CT0VIVI07IfMP0g=cL0D51@04e=C@0e=CD0
@d=3055AD@1MGEd0Q8B4065QH@1NGUh0Bd];05iNGP1FEUH0Kfm_02HV9P12@T80clo?075aL@3@d=00
PX:205]KF`2g]kL0O7al07inOP11@D40EeMG02HV9P1=CDd0E5AD04U9B@0U9BD0@41003@d=016ATH0
@d=303`l?01iNGT0>3Ph09NGU`1eMGD0DU9B04i>CP1BDU80B4Q807IfMP29RHT0ATI605iNGP16ATH0
[jn_0;Rh^00h>3P0BDU903dm?@1UIFD0EEEE08:2PP1JFUX0Rh^;05iNGP1VIVH0KFe]04a<C01hN7P0
BTY:04i>CP1^KVh0JVYZ04Q8B01OGel0ATI60;2`/02[Zj/0@d=309BDU01=CDd0C4a<03Ph>01HF5P0
EEEE03`l?01LG5`0ATI6055AD@0d=3@0Fe]K04E5A@0m?Cd0<C4a05=CD`11@D40O7al020P801EEED0
<30`04Y:BP0f=SH0AdM706EUI@0f=SH0:b/[03lo?`0204E5A@1302l_;`13@d<0>c/k05EEE@0/;2`0
CDe=03@d=01=CDd0<S8b034a<@12@T80I6AT03Ph>00e=CD0@41001HF5P1;Bd/0@D5104];B`1BDU80
FUYJ04e=C@0m?Cd0E5AD05]KF`0i>CT0WIfM06UYJ@0Y:BT0>3Ph04Y:BP22PX80BDU906=SH`0n?Sh0
;bl_05MGE`0m?Cd0?Shn055AD@0h>3P0E5AD09jNWP1PH600L71`04m?C`1FEUH0;Bd]07AdM01<C4`0
Bd];04Q8B01ADE40Bd];059BDP1eMGD0BDU905EEE@1OGel0G5aL05eMG@2:RXX0GUiN06YZJP1UIFD0
F5QH0:b/[00@ooooo`003_ooool0^`2JVYX0HV9R07EeM@1eMGD0Fe]K04I6AP1IFET0GUiN06ATI01[
Jf/0>3Ph04U9B@1ADE40CDe=096AT@1/K6`0<30`04];B`18B4P0PH610:FUY@19BDT0<C4a05IFEP1C
De<0OWin075aL@0W9bL0=cLg06i^KP1=CDd0SHf=05MGE`0L71`0Bd];08N7Q`1SHf<0MgMg05IFEP0U
9BD0?3`l06i^KP1LG5`0Hf=S03Lg=`12@T80D51@06IVIP0i>CT0Kfm_03dm?@15ADD0OWin05mOG`0e
=CD0Hf=S04M7A`1WIfL0Ph>303/k>`1HF5P0EeMG059BDP2@T900O7al02PX:01OGel0CDe=0451@@2>
SXh0Cdm?03lo?`1HF5P0DE5A051@D024Q8@0ADE503hn?P1KFe/0CDe=04Y:BP21PH40@d=303dm?@1c
Lg<0G5aL06UYJ@2SXj<09BDU03/k>`1IFET0BDU908>3P`1<C4`071`L04A4A018B4P0C4a<07emO@11
@D40;Bd]04=3@`1ZJVX0HV9R07AdM00M7Ad092@T05QHF02;Rh/0GemO06QXJ00T92@0?Shn05=CD`21
PH40HF5Q05mOG`0P8200CTi>05mOG`0o?cl0?Shn05aLG00X:2P0CTi>06][J`1cLg<0BTY:08F5Q@12
@T80KFe]0;Fe]@1PH600F5QH06QXJ00m?Cd0Rh^;08j>SP0j>SX0EUIF061PH01@D500PX:203lo?`0f
=SH0J6QX075aL@1KFe/0Q8B4038b<P1FEUH0KFe]05]KF`1RHV80XZ:R06m_K`16ATH0RXZ:07YjNP1g
MgL0XJ6Q08J6QP1GEeL0NWYj079bLP1nOWh0XJ6Q04E5A@1BDU80Q8B406e]K@2GUiL0c/k>02d];@10
@400Shn?05EEE@26QXH0A4A4051@D01nOWh0UiNG07AdM02OWil0>c/k04Y:BP1lO7`00P21PH40D`1R
HV80;2`/05iNGP2VYZH0O7al07inOP1LG5`0EUIF0:BTY01fMWH0JFUY07QhN014A4@0Ng]k05eMG@1I
FET0MgMg03/k>`0o?cl0Rh^;05EEE@1iNGT0KFe]059BDP1`L700YZJV06m_K`1eMGD0CTi>07YjNP2g
]kL0H61P06YZJP19BDT0De=C0:f][@3:b/X0GUiN061PH01YJFT0P8200:NWY`2MWId0B4Q806m_K`1D
E5@0LG5a0:j^[P1mOGd0BTY:051@D023Ph<0E5AD09FEU@2KVi/0?3`l03dm?@1iNGT0H61P08J6QP10
@400=CDe05=CD`1]KFd0I6AT07]kN`0h>3P0?clo06=SH`1ZJVX0IVIV08:2PP10@400CTi>06EUI@1J
FUX0ADE508N7Q`1gMgL0<C4a06QXJ01kNg/0Cdm?0080Ph>300@0;2`/05MGE`2UYJD0H61P3_ooool0
901BDU80N7Qh069RHP1LG5`0EEEE05ADE01bLW80D51@06ATI01BDU80HF5Q05eMG@1cLg<0K6a/08:2
PP1dM7@0IfMW051@D01OGel0E5AD02/[:`14A4@0@T9204A4A01HF5P0De=C04e=C@1SHf<0KFe]06i^
KP11@D40F5QH05ADE01UIFD0De=C05YJFP80Fe]K0800?3`l04U9B@1>CTh0@d=305QHF013@d<0?clo
04I6AP0g=cL0;Bd]05MGE`19BDT0:RXZ0492@P1DE5@0820P04Y:BP17AdL0B4Q803Xj>P13@d<0ATI6
069RHP1ADE40CDe=075aL@1RHV80LG5a09RHV00m?Cd0AdM706IVIP1PH600G5aL0410@01GEeL0<C4a
04E5A@1JFUX0C4a<05IFEP0e=CD0<S8b03Lg=`15ADD0?clo05UIF@0l?3`0BDU902l_;`0o?cl0@410
03Xj>P1MGEd0;2`/05EEE@0f=SH0ATI603Hf=P1PH600EEEE05UIF@1;Bd/0BDU903Xj>P1CDe<0E5AD
0<71`@2;Rh/0H61P06m_K`1>CTh0<c<c06][J`1>CTh0<30`06=SH`0l?3`0AdM7079bLP1^KVh0?Shn
0451@@0f=SH0@D5107UiN@0o?cl0_kno07AdM01PH600VYZJ069RHP1WIfL0KVi^09:BTP0d=3@0Fe]K
04Y:BP1>CTh0<C4a051@D01:BTX0HF5Q04M7A`1;Bd/0JFUY05iNGP2b/[80KFe]05MGE`15ADD0A4A4
05iNGP1nOWh0MgMg02PX:01aLG40JVYZ0;Jf]P2VYZH0IFEU05aLG015ADD0HV9R04I6AP2YZJT0Fe]K
0:FUY@80CDe=03d0EUIF0492@P1ZJVX0?Shn06][J`1WIfL0GUiN0451@@1RHV8092@T08J6QP1iNGT0
ATI6096AT@1:BTX0Bd];03hn?P22PX80Cdm?03lo?`16ATH0@d=305eMG@1CDe<0@d=303dm?@1?Cdl0
BTY:05YJFP11@D40IfMW04];B`1SHf<0C4a<05IFEP0e=CD0@410051@D00O7al0:BTY03De=@14A4@0
Cdm?04=3@`1=CDd0EEEE069RHP0g=cL0I6AT0451@@1?Cdl0>3Ph05EEE@1FEUH0KVi^0492@P10@400
AdM702h^;P1IFET0@d=30080=cLg00P0Bd];04A4A017AdL0E5AD04e=C@14A4@0GUiN04Y:BP80D51@
02D0B4Q8061PH013@d<0IVIV04=3@`0o?cl0Hf=S03lo?`1;Bd/0GemO05IFEP1TI6@0S8b<07=cL`1Z
JVX0IVIV07EeM@1IFET0[:b/05=CD`1PH600AdM707EeM@1DE5@0F5QH06i^KP18B4P0Fe]K04i>CP1N
GUh0JFUY07QhN01IFET0FUYJ06YZJP1OGel0M7Ad013ooooo000>ooooo`3o08F5Q@23Ph<0X:2P059B
DP1aLG40LW9b0;Vi^@19BDT0=3@d06MWI`24Q8@0Vi^K02LW9`0`<300:b/[07inOP1eMGD0IVIV034a
<@18B4P0JFUY07AdM01SHf<0MWIf01`L701ADE40Vi^K07EeM@1SHf<0@41003lo?`1@D500O7al04E5
A@1=CDd0D51@0410@01OGel0Ph>30410@018B4P0Hf=S028R8P1nOWh0O7al02d];@1DE5@0=cLg03lo
?`1fMWH0TI6A03@d=018B4P0<30`06IVIP2b/[80>3Ph04Y:BP1JFUX0;bl_07EeM@2EUID0820P03/k
>`1JFUX0A4A40:ZZZP2DU9@08b<S03<c<`1VIVH0M7Ad08^;R`0k>c/09bLW03Ti>@1WIfL0NGUi07al
O00M7Ad08b<S0451@@1NGUh0NGUi07emO@0K6a/09BDU04];B`1NGUh0Ph>303Xj>P0I6AT0EUIF07mo
O`14A4@0G5aL02h^;P0[:b/0>3Ph05eMG@1<C4`0NWYj05EEE@0b<S80OGem06EUI@19BDT0F5QH02h^
;P0d=3@0RHV905UIF@12@T80HV9R02DU9@14A4@0YJFU059BDP1<C4`0E5AD02HV9P0j>SX0K6a/034a
<@0e=CD0BTY:038b<P0h>3P0I6AT06MWI`1XJ6P0CDe=03Xj>P24Q8@0/k>c030`<01<C4`0GUiN05]K
F`2`/;00Rh^;02DU9@1;Bd/0FEUI07EeM@3@d=00C4a<02XZ:P15ADD0FUYJ09ZJVP2b/[80>c/k02`/
;012@T80KVi^09jNWP2b/[80;bl_0492@P1ZJVX0IFEU096AT@2l_;`0=cLg05=CD`25QHD0O7al0861
P@1LG5`0@D5106m_K`2ZZZX0MgMg06QXJ01UIFD0=3@d08R8R02PX:00@T9208:2PP0e=CD0?clo071`
L02b/[80DE5A05iNGP0O7al0De=C0:JVYP1UIFD0AdM70410@00g=cL0DE5A07IfMP0k>c/0GUiN02d]
;@1aLG40WIfM05iNGP1[Jf/0:RXZ051@D02WYjL0SHf=04I6AP1KFe/0@410096AT@2`/;00A4A403/k
>`1MGEd0De=C08>3P`0b<S80=CDe051@D01]KFd0WinO03<c<`0Y:BT0BDU9079bLP1OGel0/;2`07Mg
M`0g=cL0K6a/0::RXP1iNGT0d=3@06][J`0X:2P0D51@08>3P`1eMGD0U9BD0;>c/`0m?Cd0CDe=06e]
K@1TI6@0ZZZZ09:BTP0i>CT0EEEE06EUI@1RHV80[Zj^04Y:BP0a<C40N7Qh000G07AdM01VIVH0JVYZ
01lO7`0Z:RX0Ph>307emO@2:RXX0I6AT02h^;P0`<300BDU908F5Q@1LG5`0L71`02TY:@0o?cl0@d=3
06MWI`1RHV80QHF504M7A`1;Bd/03_ooool0A@1aLG40NWYj06i^KP1]KFd0FEUI059BDP1ZJVX0G5aL
05YJFP1PH600A4A404=3@`1PH600Lg=c08V9R@2^[Zh0Kfm_04m?C`1CDe<0EEEE02HV9P0m?Cd0<S8b
059BDP1FEUH0F5QH07AdM022PX80SHf=075aL@19BDT0Kfm_05IFEP1^KVh0GemO05IFEP1RHV80IFEU
04I6AP1GEeL0FUYJ0410@01XJ6P0AdM705]KF`14A4@0EEEE04Q8B01bLW80CTi>04U9B@0d=3@0Cdm?
02LW9`0o?cl0>CTi04I6AP0f=SH0A4A4055AD@1/K6`0BDU9065QH@1OGel0DE5A09FEU@2RXZ80;2`/
03Ti>@0205IFEP0P05]KF`0h>3P0IVIV03Hf=P0c<c<0G5aL0451@@1SHf<0:b/[02@T900f=SH0AdM7
03hn?P1<C4`0:RXZ03lo?`0c<c<0@d=303Lg=`0m?Cd0C4a<02d];@1<C4`0:BTY04A4A00Y:BT0@D51
04=3@`1BDU80AdM703dm?@12@T8205UIF@16096AT@1^KVh0A4A405iNGP1EEED0@d=304M7A`0j>SX0
AdM707EeM@1HF5P0G5aL055AD@1^KVh0H61P05ADE01LG5`0J6QX09^KV`21PH40c/k>08^;R`2UYJD0
clo?08b<S01iNGT0KFe]09RHV01=CDd0K6a/05aLG00i>CT0>3Ph05]KF`0l?3`0C4a<04A4A01?Cdl0
S8b<04];B`2XZ:P0IfMW06UYJ@1?Cdl0:b/[04A4A01ADE40De=C0410@02/[:`0Jf][0;Ng]`37alL0
D51@06UYJ@1BDU80T92@05YJFP2VYZH0IFEU05aLG01BDU80M7Ad06IVIP1/K6`0Ogmo065QH@29RHT0
QHF508j>SP80Hf=S06D0@D510:BTY01bLW80EUIF07alO01WIfL0E5AD04E5A@1DE5@0AdM703Lg=`18
B4P0FEUI071`L00m?Cd0BDU903De=@1;Bd/0CDe=07EeM@1ADE40Ogmo061PH01lO7`0Cdm?04i>CP1F
EUH0?clo05EEE@14A4@0ADE504e=C@1BDU80Fe]K03@d=01LG5`0ATI605iNGP0i>CT0EEEE05ADE01n
OWh0=3@d051@D015ADD0SXj>0451@@1>CTh0D51@03Lg=`26QXH0?3`l03/k>`10@400HF5Q059BDP16
ATH0D51@04e=C@0_;bl0VIVI05mOG`0c<c<0@41003/k>`1JFUX07alO05]KF`0Q8B40F5QH08B4Q01H
F5P0AdM704e=C@0a<C40QXJ608N7Q`19BDT0AdM7092@T01fMWH0>c/k055AD@1>CTh0Jf][04a<C01:
BTX0Bd];05eMG@1SHf<0<C4a04Q8B015ADD0K6a/071`L01nOWh0Fe]K04m?C`1ADE40DU9B071`L00@
ooooo`003_ooool0G@1]KFd0DU9B06m_K`1/K6`0?Cdm04I6AP1JFUX0GUiN05mOG`1?Cdl0A4A407Qh
N01`L700BTY:05aLG00_;bl0ADE5051@D00T92@0@D5104m?C`16ATH0I6AT07=cL`0R8R80?Cdm04Q8
B01XJ6P0NGUi05aLG00V9RH0A4A403/k>`1_Kfl0M7Ad08:2PP0N7Qh0@d=305IFEP1@D500LG5a07Ad
M00Z:RX0@D5105eMG@1gMgL0YZJV02d];@0/;2`0ADE507IfMP22PX80SHf=03<c<`0W9bL0N7Qh06YZ
JP27QhL0I6AT03lo?`15ADD0N7Qh05ADE01QHF40;Rh^02d];@1EEED0OGem06m_K`1IFET05ADE034a
<@1?Cdl0Fe]K04a<C01fMWH0:BTY04Y:BP2DU9@0De=C02`/;01WIfL0:BTY04U9B@2?Shl0?Cdm03Xj
>P16ATH0<C4a05=CD`1@D50071`L03Lg=`0202d];@2g07]kN`0b<S80<30`03dm?@0j>SX0B4Q809FE
U@1UIFD0>SXj05IFEP15ADD0Hf=S09bLW015ADD0?3`l05YJFP1KFe/0Rh^;09ZJVP0V9RH0ADE507=c
L`1CDe<0Ti>C0820P00L71`0;Rh^07UiN@1QHF40T92@0:RXZ00_;bl0?Shn05ADE01hN7P0g]kN0<30
`01<C4`0?Shn07EeM@21PH40][Jf03Hf=P1FEUH0MWIf08j>SP1IFET0O7al038b<P1IFET0/[:b0;2`
/01kNg/0JFUY03`l?01cLg<0TI6A07=cL`1CDe<0K6a/0451@@1LG5`0]kNg07UiN@1DE5@0T92@0492
@P1<C4`0_[jn09^KV`1UIFD0VIVI03/k>`26QXH0QhN704U9B@1bLW80Vi^K04a<C01TI6@0MWIf02l_
;`1EEED0GEeM05YJFP2;Rh/0J6QX02TY:@1OGel0NWYj06][J`2[Zj/0E5AD03Ti>@1dM7@0RXZ:0:j^
[P1_Kfl09bLW06IVIP1nOWh0RHV909^KV`0`<300=3@d08>3P`27QhL0VYZJ01hN7P0f=SH0GEeM06AT
I01RHV80GemO024Q8@1<C4`0ZJVY08j>SP2][Jd0Rh^;03`l?01cLg<0LG5a05aLG01ZJVX09bLW06YZ
JP1jNWX0@d=308:2PP1=CDd0;bl_03/k>`1fMWH0E5AD05UIF@10@400?Shn0410@01jNWX0CDe=0820
P00h>3P0;bl_051@D01cLg<0A4A405IFEP0c<c<0;Bd]06EUI@1mOGd0?Cdm04e=C@0/;2`0BDU9075a
L@2PX:00@d=303lo?`0k>c/0QHF50;2`/00];Bd06a/K03Hf=P1IFET0T92@0:>SX`0R8R80:RXZ0451
@@1?Cdl0G5aL0<o?c`1@D500;Bd]03/k>`11@D40HV9R0:b/[01QHF40>c/k00kooooo0P1oOgl0EP1H
F5P0FUYJ05YJFP1WIfL0NWYj0451@@1gMgL0LW9b051@D01?Cdl0N7Qh06e]K@23Ph<0EEEE05]KF`1:
BTX0GUiN04i>CP0T92@0?3`l03Xj>P0g=cL0CDe=04a<C015ADD0H61P05=CD`1EEED0EUIF06QXJ01B
DU80De=C05ADE01QHF40E5AD055AD@0k>c/0DE5A05YJFP1TI6@0Hf=S04=3@`0c<c<0EeMG03Ti>@0g
=cL0JVYZ07AdM00n?Sh0ADE506IVIP0];Bd0FEUI04A4A01?Cdl0?Shn07]kN`1JFUX0HF5Q03`l?018
B4P0EEEE04];B`1IFET0Jf][03Hf=P19BDT0J6QX05iNGP21PH40?Cdm075aL@0m?Cd0K6a/069RHP15
ADD0EeMG05ADE00d=3@0>CTi055AD@0i>CT0De=C03Ph>01SHf<09BDU0P11@D40_00k>c/0GUiN03dm
?@1NGUh0;Rh^04Q8B010@400E5AD05eMG@1YJFT0EUIF05YJFP1BDU80Kfm_079bLP2j^[X0SHf=05IF
EP1ADE40@d=3028R8P1ZJVX09bLW02d];@1IFET0>c/k05IFEP1PH600DU9B055AD@1HF5P0CTi>06MW
I`2?Shl0Kfm_0<S8b01kNg/0Ogmo08N7Q`1YJFT0L71`09>CT`2HV9P08b<S05]KF`1DE5@0DU9B05IF
EP1SHf<0D51@07QhN01KFe/0BTY:06m_K`0h>3P0U9BD03Hf=P1KFe/0AdM704Y:BP1ZJVX0R8R807Ui
N@0M7Ad0FEUI04U9B@2?Shl0LG5a03hn?P1DE5@0:RXZ061PH00o?cl0UIFE04a<C026QXH0ATI605MG
E`1:BTX0>3Ph05MGE`1]KFd0W9bL06EUI@0n?Sh0?3`l06ATI00P8200Ph>307inOP1XJ6P0OWin04Q8
B01GEeL0=cLg05mOG`0b<S80CDe=06EUI@16ATH0EEEE03De=@1[Jf/0B4Q806EUI@0k>c/0A4A4034a
<@1PH600E5AD06e]K@1ADE40D51@02d];@1YJFT0WinO02PX:00c<c<0?clo05IFEP1KFe/0@D5104=3
@`1:BTX0G5aL02d];@1NGUh0=SHf04Q8B00b<S80IfMW04i>CP1HF5P0:2PX038b<P0c<c<051@D05QH
F00k>c/092@T02h^;P0e=CD0=cLg024Q8@18B4P0BDU902PX:01@D500@T9204=3@`14A4@0FUYJ08N7
Q`0g=cL0MgMg028R8P0i>CT0CDe=0451@@1;Bd/0EeMG03Lg=`1<C4`0BTY:05YJFP0g=cL0PX:208Z:
RP1KFe/0IFEU05IFEP1NGUh0G5aL05ADE01BDU80GEeM079bLP1BDU80Cdm?04a<C01IFET0Kfm_06=S
H`1=CDd0Fe]K05ADE01EEED0N7Qh4?ooool000kooooo08`0Zj^[06=SH`1[Jf/0SXj>05YJFP0o?cl0
C4a<05EEE@1XJ6P0L71`0492@P1GEeL0JVYZ07MgM`2HV9P0>c/k04M7A`1ZJVX0I6AT09^KV`2ZZZX0
6ATI04Y:BP1bLW80GemO08J6QP25QHD09RHV0451@@1gMgL0EEEE0::RXP13@d<06a/K059BDP20P800
A4A408:2PP0^;Rh0:BTY04A4A023Ph<0GUiN04Q8B00[:b/09BDU05=CD`1hN7P0?clo05aLG00/;2`0
<S8b071`L014A4@0>c/k05eMG@0j>SX0GEeM06ATI00];Bd0DE5A07QhN01?Cdl0UIFE03`l?00S8b<0
@T9204A4A00g=cL0VYZJ02`/;00Y:BT0ADE505EEE@1<C4`0WinO02l_;`0Q8B40B4Q805aLG01NGUh0
PX:203@d=00W9bL0CDe=069RHP2NWYh0HV9R01HF5P1=CDd0C4a<05UIF@2EUID0G5aL010@400b<S80
EeMG069RHP1eMGD06a/K02@T900_;bl0J6QX06YZJP1=CDd06QXJ03Lg=`13@d<0GEeM04i>CP1VIVH0
:BTY04a<C02CTi<0M7Ad05MGE`1=CDd0>CTi0492@P2ATI40HV9R03Lg=`1LG5`0=cLg06=SH`30`<00
UiNG061PH02BTY80>c/k08F5Q@2UYJD0GUiN055AD@1@D500BTY:08N7Q`1UIFD09RHV06ATI080CTi>
00T0SXj>05=CD`0f=SH0DU9B04M7A`2ATI40RHV902TY:@1DE5@00P1fMWH0O@22PX80c/k>0492@P0k
>c/0Ogmo09JFUP25QHD0d=3@04];B`15ADD0UiNG08^;R`2MWId0_kno05ADE01>CTh0]kNg07QhN02a
/K40ZZZZ03hn?P1iNGT0XZ:R07UiN@2l_;`0A4A403Xj>P1^KVh0/[:b08b<S02QXJ40ATI603hn?P1H
F5P0P82007moO`1WIfL0=CDe06QXJ02:RXX0EeMG07alO01^KVh0<c<c08>3P`1QHF40DE5A06YZJP0h
>3P0T92@05eMG@11@D40GEeM02@T901NGUh0T92@04=3@`1]KFd0LG5a04=3@`1dM7@0ZJVY06=SH`1F
EUH0=CDe08f=S@2j^[X0=CDe05mOG`1IFET0F5QH0:f][@2DU9@0;bl_04U9B@1QHF40MGEe0;:b/P1T
I6@0?clo05ADE01jNWX0M7Ad0;Fe]@1MGEd092@T04M7A`1cLg<0IfMW0;Rh^01JFUX0:BTY069RHP2<
S8`0AdM708^;R`0R8R80@T92079bLP2GUiL0Lg=c08B4Q00c<c<0@410069RHP1UIFD0B4Q805mOG`12
@T80EEEE05]KF`1jNWX0A4A4079bLP1dM7@0>c/k07IfMP1`L700Bd];071`L015ADD0;Rh^08>3P`31
`L40G5aL00kooooo00X0R8R808J6QP1GEeL0FUYJ05eMG@1EEED0I6AT04U9B@1XJ6P0DU9B0P1DE5@0
=`1`L700OWin079bLP1NGUh0NWYj05]KF`1`L700FEUI02PX:019BDT0>CTi04=3@`1ADE40ATI604Y:
BP1NGUh0EeMG06MWI`15ADD0FEUI051@D01RHV80CTi>05QHF01TI6@0MGEe065QH@1GEeL0EEEE03dm
?@1LG5`0A4A403<c<`0n?Sh0=SHf034a<@1PH600A4A403@d=01>CTh0@D5103`l?01]KFd0@d=304Y:
BP0];Bd0ADE503lo?`1JFUX0D51@05EEE@1@D500IVIV09>CT`2GUiL00P1:BTX0M@1kNg/0S8b<06a/
K00g=cL0CTi>03Hf=P0c<c<0DE5A0410@01HF5P0>c/k04U9B@0m?Cd0>c/k03Xj>P19BDT09bLW0492
@P0d=3@0ATI6030`<015ADD0EeMG02/[:`0d=3@0:RXZ04=3@`1FEUH0FUYJ055AD@1MGEd0I6AT034a
<@1RHV80Ogmo04i>CP2QXJ40QHF507YjNP1aLG40CDe=03hn?P1SHf<0=cLg038b<P1BDU80BDU904I6
AP1NGUh0KVi^05]KF`1YJFT0EEEE06YZJP1[Jf/0@4100;Ng]`1FEUH0TY:B08Z:RP1iNGT0HV9R09NG
U`2c/k<0ADE506ATI01<C4`0QHF503/k>`1ADE40=cLg04a<C00[:b/0:BTY06ATI00d=3@0Shn?04I6
AP23Ph<0<c<c024Q8@0m?Cd0C4a<06YZJP13@d<0LG5a04e=C@3GemL0^;Rh02h^;P1@D500;Bd]0:6Q
X@1dM7@0IFEU065QH@24Q8@0@D5105=CD`12@T80Fe]K07inOP15ADD0Kfm_06QXJ01GEeL0CDe=065Q
H@1@D500U9BD08V9R@1;Bd/0IVIV0451@@13@d<0A4A407inOP0203Ti>@0/04Y:BP1DE5@0KVi^02d]
;@0n?Sh0=CDe04e=C@19BDT0KVi^03`l?01[Jf/0KFe]08Z:RP1:BTX0DU9B03De=@0`<300>SXj02TY
:@18B4P0>SXj03<c<`1?Cdl0@410055AD@15ADD0CDe=034a<@15ADD0>CTi05EEE@0Y:BT0BTY:04U9
B@1RHV80=3@d0492@P10@400;Rh^06YZJP0b<S806QXJ03Lg=`0b<S820410@00/05ADE017AdL0>3Ph
061PH01<C4`0F5QH04i>CP0l?3`0CTi>05QHF029RHT0BTY:03dm?@1@D500@41003Xj>P1=CDd09BDU
06m_K`16ATH0DU9B05QHF02RXZ80GEeM079bLP2=SHd0AdM705MGE`0h>3P0GemO04M7A`1MGEd0GUiN
05YJFP1/K6`0C4a<055AD@1KFe/0Jf][04a<C01MGEd0H61P05IFEP1hN7P@ooooo`003_ooool0m@2c
/k<0YZJV09jNWP1IFET0IVIV05IFEP23Ph<0=3@d04U9B@1lO7`0FEUI07=cL`0W9bL0D51@06a/K026
QXH0FUYJ05]KF`0b<S80FUYJ07=cL`1VIVH0@410051@D019BDT0IVIV08Z:RP0c<c<0<S8b04U9B@1>
CTh0PH6105YJFP0D51@0:2PX06EUI@1CDe<0][Jf04U9B@0M7Ad0<C4a061PH01KFe/0N7Qh02LW9`0k
>c/0>SXj05IFEP26QXH0VYZJ02<S8`0V9RH0>CTi055AD@1nOWh0];Bd024Q8@18B4P0KVi^05ADE02B
TY80UIFE03`l?019BDT0GEeM05iNGP2MWId0A4A402LW9`1SHf<0MGEe07IfMP2?Shl0<S8b02PX:01?
Cdl0JFUY07emO@1^KVh061PH03hn?P1UIFD0L71`05eMG@1NGUh06a/K069RHP2ATI40?3`l05EEE@0V
9RH09BDU065QH@28R8P0>SXj04e=C@0Y:BT0?Cdm0861P@1?Cdl08b<S03hn?P11@D40IfMW07emO@0l
?3`0;Bd]05aLG00m?Cd0M7Ad07MgM`0O7al0:BTY04m?C`1CDe<0NWYj05mOG`0Y:BT0>3Ph05ADE01Q
HF40E5AD06i^KP0_;bl0J6QX07MgM`1oOgl0PH610<C4a00i>CT0G5aL06YZJP1aLG40U9BD09RHV00^
;Rh0Ng]k08j>SP1/K6`0YjNW04I6AP0V9RH0C4a<065QH@21PH40UIFE04A4A00c<c<0R8R805EEE@28
R8P0LG5a051@D01?Cdl0Lg=c07IfMP1ZJVX0NWYj0492@P1`L700Xj>S0;>c/`26QXH0IVIV04U9B@1C
De<0XJ6Q06e]K@1EEED0@d=305MGE`1bLW80W9bL051@D00f=SH0De=C07EeM@2WYjL0Lg=c030`<01G
EeL0?Shn05iNGP30`<00JFUY02l_;`1?Cdl0@D5107QhN02^[Zh0=CDe04A4A01?Cdl0>c/k07alO01/
K6`0?Cdm06QXJ01NGUh0MWIf0:^[Z`13@d<0JVYZ05=CD`1_Kfl0LG5a01dM7@0K6a/0Lg=c06=SH`2G
UiL0EEEE03Xj>P1MGEd0GUiN06UYJ@1eMGD08R8R04U9B@2WYjL0ADE508J6QP19BDT0A4A407emO@33
`l<0HV9R09NGU`1bLW80Fe]K07moO`31`L40IFEU0:f][@1<C4`0KFe]09>CT`2i^KT0GemO08j>SP1J
FUX0>SXj07UiN@2VYZH0AdM70080IVIV01l0ADE506YZJP18B4P0@D5106IVIP0h>3P0>SXj08Z:RP0c
<c<0HF5Q07=cL`0g=cL0AdM708j>SP10@400BDU906ATI015ADD0CDe=09BDU02TY:@0<30`07MgM`1T
I6@0ADE509NGU`2PX:00:RXZ04m?C`1lO7`0K6a/00kooooo0:d0NGUi071`L01IFET0JVYZ04i>CP1G
EeL0LW9b04i>CP1[Jf/0IfMW059BDP1ADE40H61P05iNGP1eMGD0J6QX06YZJP13@d<0Fe]K04m?C`0D
51@0?Cdm03Xj>P0h>3P0ADE504Y:BP1CDe<0IFEU05=CD`1DE5@0Cdm?06][J`16ATH0Fe]K07=cL`1f
MWH0D51@06QXJ017AdL0B4Q804M7A`1<C4`0FUYJ065QH@15ADD0Bd];0410@00m?Cd0NWYj05mOG`14
A4@0@T9205eMG@0Y:BT0CDe=04M7A`1YJFT0?Cdm059BDP0e=CD0Jf][04M7A`1<C4`0I6AT04=3@`2K
Vi/0W9bL04E5A@1KFe/0CTi>051@D01UIFD0BDU907QhN00j>SX0=CDe051@D019BDT0E5AD0492@P0]
;Bd0Bd];04M7A`0h>3P0D51@024Q8@17AdL0410@04Q8B00b<S80CDe=04Y:BP0X:2P0E5AD01dM7@0i
>CT0?Shn04a<C010@400<c<c04=3@`0Y:BT0B4Q805mOG`1DE5@0Y:BT06e]K@18B4P0IVIV06MWI`0e
=CD0Cdm?03/k>`0h>3P0HF5Q04i>CP1CDe<0FEUI04e=C@1QHF40H61P06i^KP1iNGT0Kfm_06ATI02]
[Jd0PX:20<71`@2[Zj/0Kfm_075aL@2IVIT0[jn_03dm?@1XJ6P0ADE504];B`0Z:RX0CDe=03dm?@1U
IFD0>CTi0410@01/K6`0Cdm?092@T019BDT0>SXj0451@@0];Bd0NWYj09bLW01VIVH0>c/k07UiN@1Z
JVX0P82008V9R@1EEED0H61P055AD@1UIFD0A4A4055AD@0h>3P0H61P034a<@1FEUH0BTY:04=3@`1K
Fe/0?Cdm07moO`0205mOG`1W04Y:BP1LG5`07QhN0<o?c`1/K6`0G5aL07IfMP1NGUh0HV9R03hn?P19
BDT0:b/[03Hf=P12@T80>CTi04];B`0V9RH0FUYJ04=3@`1HF5P0Cdm?07IfMP1IFET0N7Qh061PH01b
LW80?Shn04Y:BP0k>c/0=3@d05YJFP0U9BD0>c/k03Ph>00d=3@0EEEE02LW9`17AdL0@T92055AD@0[
:b/0Bd];03hn?P1NGUh0:2PX071`L010@400MGEe02<S8`14A4@0;bl_00l?3`1IFET0<S8b02/[:`1?
Cdl0B4Q8055AD@13@d<0BDU904Y:BP0n?Sh0Ph>304M7A`0o?cl0?Shn05=CD`1EEED0DE5A07UiN@14
A4@0DU9B06a/K011@D40@d=305ADE00<30`0IFEU059BDP1]KFd0CDe=0:JVYP21PH40QHF50;Fe]@16
ATH0EUIF01XJ6P1LG5`0>CTi05YJFP1NGUh0Cdm?069RHP1GEeL0DE5A06][J`1oOgl0E5AD05eMG@1F
EUH0DU9B07QhN00@ooooo`003_ooool04P1NGUh0N7Qh0;6a/@1NGUh0GEeM06a/K030`<00AdM702d]
;@1JFUX0JFUY09bLW00h>3P07alO03dm?@1FEUH0KVi^07UiN@80;2`/0680DE5A069RHP1mOGd0RHV9
01/K6`0d=3@0JVYZ079bLP2PX:00CTi>05MGE`1jNWX0Vi^K05IFEP1_Kfl0Hf=S030`<01/K6`0NGUi
05EEE@1]KFd0CTi>02LW9`1;Bd/0ADE50410@01>CTh05ADE03Hf=P1EEED0EUIF06YZJP1aLG40:BTY
03lo?`2;Rh/0J6QX075aL@0T92@0820P04M7A`1]KFd0:BTY055AD@0R8R80?Shn08b<S02IVIT09bLW
03dm?@0];Bd0FEUI0:FUY@1CDe<07AdM03De=@1<C4`0/[:b09NGU`0Y:BT071`L03lo?`1IFET0[Zj^
07inOP0I6AT0;Rh^03Ph>01<C4`0T92@02PX:00M7Ad0>SXj059BDP10@400RXZ:01lO7`0c<c<0LG5a
07emO@1<C4`0QHF5030`<01;Bd/0Q8B407=cL`1aLG40J6QX03Hf=P16ATH0Q8B404E5A@1EEED0SHf=
03Hf=P1:BTX0V9RH05EEE@80KFe]00@07QhN03Ph>02@T900@D510P1TI6@0?@0W9bL0De=C08f=S@1V
IVH0N7Qh0410@00o?cl0QHF5092@T010@400EUIF02d];@1=CDd0^;Rh0;jn_P0];Bd0E5AD02l_;`1^
KVh0_;bl055AD@0_;bl0C4a<03lo?`1nOWh0QhN7034a<@1EEED0GemO07AdM02EUID0S8b<02h^;P1E
EED0HV9R08f=S@2HV9P0TY:B03Ph>016ATH0V9RH0;>c/`2KVi/0FEUI03lo?`1GEeL0MGEe08R8R01f
MWH0JFUY02XZ:P27QhL0Z:RX05YJFP2[Zj/0Cdm?01lO7`1`L700M7Ad05mOG`2ATI400P0f=SH0=P33
`l<0K6a/08f=S@14A4@07alO04=3@`2ATI40HV9R079bLP0];Bd0@41005MGE`1CDe<0G5aL024Q8@1;
Bd/0Ng]k04M7A`1ADE40Bd];038b<P28R8P0UIFE04I6AP1hN7P0H61P05YJFP1hN7P0?Cdm05YJFP1Y
JFT0@41008b<S01KFe/0Bd];07=cL`1RHV80DU9B069RHP1PH600?3`l05MGE`1EEED0De=C0::RXP1j
NWX0<S8b04];B`1SHf<0Cdm?0<?3``2OWil0=cLg05EEE@80JVYZ0240[jn_09jNWP0n?Sh0I6AT05aL
G01fMWH0/k>c051@D00/;2`0PX:206m_K`1iNGT0U9BD02TY:@0[:b/0GemO06=SH`1cLg<0Q8B403Lg
=`0^;Rh0?clo0:NWY`2<S8`0c/k>05ADE00j>SX0>c/k06m_K`1NGUh0Rh^;03dm?@0l?3`03_ooool0
C@27QhL0OGem05eMG@1JFUX0Cdm?05QHF01[Jf/0?Shn08Z:RP1gMgL0C4a<059BDP1lO7`0NGUi08Z:
RP1eMGD0KFe]05EEE@1[Jf/0J6QX03Ti>@19BDT0@d=303hn?P18B4P0Bd];03lo?`1SHf<0MWIf06QX
J012@T80Kfm_05YJFP1WIfL0BTY:05ADE01/K6`0Kfm_04];B`1PH600DU9B05=CD`1EEED0<c<c020P
801?Cdl08R8R03hn?P1VIVH0FUYJ0492@P1;Bd/0De=C04e=C@1HF5P0<C4a059BDP1EEED0EUIF02XZ
:P1BDU80>SXj05ADE01>CTh0=cLg069RHP1XJ6P0:BTY03dm?@22PX80N7Qh05aLG00^;Rh0Hf=S03dm
?@1@D500CTi>0080?3`l0600@T9201LG5`0h>3P0ADE504A4A012@T80;bl_04E5A@0[:b/0CTi>03Hf
=P14A4@0@41003dm?@1GEeL0?Shn05ADE01/K6`0KVi^06=SH`1KFe/0JVYZ051@D01CDe<0KFe]06QX
J021PH40Lg=c07moO`1/K6`0Bd];07alO02IVIT0=SHf06ATI01HF5P0=SHf03Ph>01TI6@0B4Q809nO
W`1HF5P0De=C07=cL`2ZZZX0RHV909bLW01CDe<0IfMW06EUI@1YJFT0P8200;Jf]P2ZZZX0De=C08Z:
RP1SHf<0C4a<05=CD`19BDT0?3`l05]KF`0n?Sh0=SHf07=cL`0g=cL0SXj>04A4A01YJFT0>3Ph01dM
7@1;Bd/0MWIf075aL@0_;bl0H61P04m?C`36a/H0V9RH02/[:`13@d<06QXJ0:JVYP18B4P0Fe]K05AD
E02?Shl0B4Q8059BDP17AdL0N7Qh079bLP1?Cdl0QHF505QHF01^KVh205eMG@0e02TY:@2KVi/0Cdm?
06][J`1WIfL0@D5103lo?`10@400FUYJ03`l?00n?Sh0?clo04=3@`1XJ6P0BDU903Lg=`1=CDd0GEeM
04=3@`17AdL0<30`05EEE@1OGel0KFe]0492@P1HF5P0<S8b03`l?01IFET06QXJ03/k>`0_;bl09bLW
04I6AP0b<S80CDe=04A4A01=CDd0=cLg04U9B@0m?Cd0De=C03Ti>@11@D40<c<c05mOG`0J6QX0Bd];
03@d=00W9bL0FEUI03dm?@0c<c<00P0o?cl0;P15ADD0>c/k04E5A@0e=CD0;Rh^05EEE@0k>c/0HV9R
04m?C`1QHF40QHF505YJFP1YJFT08b<S038b<P15ADD0@d=305ADE01PH600<S8b079bLP0l?3`0E5AD
04m?C`37alL0OGem07]kN`2JVYX0F5QH06QXJ00e=CD0G5aL034a<@1_Kfl0Jf][05ADE01FEUH0@d=3
051@D02:RXX0UYJF051@D01hN7P0Fe]K059BDP23Ph<@ooooo`003_ooool0N01/K6`0KVi^0::RXP2G
UiL0ATI605]KF`1IFET0O7al08F5Q@1ADE40BDU9079bLP1oOgl0G5aL07UiN@0H61P0AdM707=cL`1?
Cdl0Jf][071`L00X:2P0:b/[07inOP13@d<0I6AT03hn?P11@D40I6AT075aL@0i>CT0CDe=02l_;`0n
?Sh0IVIV07AdM00V9RH0Bd];04Q8B00f=SH0M7Ad0:NWY`0_;bl0BDU904];B`1GEeL0WinO04m?C`0R
8R80F5QH03lo?`2g]kL0P82001lO7`10@400C4a<065QH@2n_[h0=SHf02HV9P1<C4`0M7Ad07QhN02]
[Jd0DU9B02h^;P0m?Cd0I6AT04Q8B01^KVh08B4Q02l_;`1DE5@0PX:206i^KP2?Shl08b<S04m?C`2E
UID0Ogmo051@D02;Rh/0=CDe051@D01hN7P0@d=304M7A`0e=CD09bLW06MWI`1^KVh0<30`05=CD`0j
>SX0;Bd]075aL@1LG5`0?Cdm04e=C@0^;Rh0BDU907]kN`1DE5@0?Shn02h^;P0n?Sh0D51@07AdM015
ADD0>c/k06UYJ@1ADE40?Cdm07YjNP0];Bd0:b/[051@D01:BTX0MWIf08n?S`809bLW05P0GUiN03hn
?P29RHT0TI6A02/[:`0c<c<0Fe]K07EeM@30`<00Fe]K02<S8`1ADE40LG5a06UYJ@1kNg/0:BTY04E5
A@1ADE40VYZJ07QhN02@T900<S8b03Lg=`1>CTh0O7al07MgM`1RHV80=CDe04];B`1RHV80FEUI09jN
WP25QHD0<S8b07IfMP1dM7@0E5AD06YZJP1nOWh0=3@d05iNGP2h^;P0HF5Q06e]K@28R8P0@41005iN
GP1_Kfl0DE5A092@T029RHT0:2PX05YJFP26QXH0CDe=07emO@1@D500>CTi07inOP22PX80ATI605QH
F016ATH0DU9B0;:b/P1dM7@0CTi>07MgM`1EEED0[jn_07EeM@0N7Qh0>c/k05MGE`1jNWX0Zj^[03@d
=015ADD0F5QH06EUI@2e]KD0>SXj034a<@1VIVH0F5QH08Z:RP29RHT092@T0P1fMWH09`1[Jf/0d];B
04i>CP16ATH0[Jf]06YZJP2RXZ80GEeM03hn?P1]KFd0RHV907inOP3=cLd0I6AT03@d=01OGel0TI6A
09:BTP2GUiL0>c/k03Ti>@21PH40Ti>C07AdM02RXZ80=3@d03`l?026QXH0[Jf]05UIF@2CTi<0C4a<
038b<P1^KVh0YjNW06IVIP1GEeL07alO05EEE@02092@T00604Y:BP1OGel0?3`l05EEE@21PH40CTi>
0`16ATH0401EEED0OWin03Ti>@0V9RH0@41004i>CP1NGUh0TI6A07alO00_;bl0@d=304M7A`1UIFD0
X:2P071`L00m?Cd>ooooo`3o08^;R`1nOWh0EEEE05YJFP1EEED0BTY:05UIF@12@T80H61P05ADE01:
BTX0CTi>079bLP1gMgL0JVYZ05iNGP1eMGD0Fe]K05eMG@1:BTX0?Cdm05QHF018B4P0?clo05mOG`14
A4@0>CTi04m?C`1MGEd0J6QX03`l?01GEeL0F5QH05iNGP1>CTh0F5QH05ADE01MGEd0AdM704e=C@14
A4@0?clo04Y:BP1@D500<C4a03hn?P0U9BD0E5AD05=CD`1OGel0<30`03/k>`0l?3`0:b/[059BDP0X
:2P0K6a/03hn?P0l?3`0?clo065QH@14A4@0AdM7071`L01?Cdl0T92@07AdM00/;2`0B4Q806EUI@1n
OWh0H61P0410@01MGEd0:2PX034a<@1HF5P0EEEE04U9B@14A4@08R8R04Q8B00i>CT0@T9204M7A`10
@400EeMG0451@@1MGEd0BDU906i^KP1?Cdl0;Bd]04Y:BP0T92@0?Cdm03`l?00k>c/0@41003<c<`13
@d<0;2`/04a<C01WIfL0G5aL08j>SP1GEeL0BTY:055AD@1=CDd0>3Ph04Y:BP1RHV80AdM703hn?P1J
FUX0AdM705EEE@0k>c/0C4a<04Q8B01]KFd0KVi^061PH01HF5P0XJ6Q04M7A`2@T900R8R806a/K01K
Fe/0a/K60;:b/P0V9RH0EUIF05aLG01VIVH0BDU9065QH@18B4P0Cdm?03Lg=`12@T80MGEe0451@@2A
TI40@d=305eMG@16ATH09bLW05ADE01@D500?clo02h^;P1[Jf/0De=C09jNWP2;Rh/0<30`04=3@`0g
=cL0Q8B405=CD`1<C4`0De=C071`L013@d<0Fe]K04e=C@1JFUX0Kfm_03lo?`1iNGT0?3`l069RHP19
BDT0?clo03hn?P2h^;P0Cdm?07moO`1fMWH0?Cdm034a<@17AdL0LW9b04i>CP0g=cL0<S8b03hn?P16
ATH0>3Ph03Ti>@0g=cL0@d=303hn?P1MGEd0:b/[04e=C@12@T80IFEU02l_;`0b<S806a/K02HV9P0g
=cL0410@034a<@11@D40>c/k05ADE014A4@0@d=3055AD@1BDU80=3@d05aLG00l?3`0Cdm?02TY:@1T
I6@0B4Q805eMG@0W9bL0@T920410@00m?Cd0CTi>02d];@0X:2P0B4Q804=3@`1;Bd/0<c<c04A4A00h
>3P09BDU05YJFP0_;bl0@D5103lo?`11@D40FUYJ06m_K`2<S8`0:BTY03dm?@16ATH0=SHf06UYJ@2A
TI40EUIF07EeM@0m?Cd0S8b<000C0451@@1cLg<0HF5Q06a/K02;Rh/0;bl_04e=C@0K6a/0E5AD01hN
7P1OGel0Cdm?04a<C01VIVH0C4a<04m?C`23Ph<0I6AT05UIF@0205=CD`03059BDP1PH63ooooo00oo
oooo000>ooooo`1908B4Q01jNWX0QhN706i^KP24Q8@0LG5a07QhN013@d<0H61P07IfMP1QHF40IVIV
0410@01JFUX0IfMW06YZJP1NGUh0Bd];04I6AP1`L700Ph>30451@@14A4@0E5AD05IFEP2GUiL0RXZ:
03/k>`1NGUh0Fe]K069RHP2EUID0@T9202PX:01OGel0IVIV05YJFP1fMWH0?clo03Ph>01IFET0De=C
05]KF`1_Kfl08b<S03Ti>@1/K6`0Lg=c04m?C`1BDU806ATI03hn?P24Q8@0F5QH05aLG02:RXX09RHV
05ADE01KFe/0;Rh^07inOP1SHf<0:b/[05YJFP1aLG40EEEE0820P00];Bd0=CDe05EEE@0n?Sh0Ogmo
09NGU`0203hn?P0?06ATI01CDe<0HV9R05]KF`0];Bd0F5QH07inOP1@D500GUiN05YJFP10@400L71`
08j>SP0b<S80I6AT0080@4100;X0MgMg061PH00];Bd0I6AT04=3@`1JFUX0Ti>C05IFEP0f=SH0GemO
05=CD`27QhL0IfMW02d];@0a<C40GUiN079bLP26QXH0IFEU02DU9@13@d<0J6QX06EUI@1YJFT0@D51
02PX:01EEED0Hf=S05UIF@18B4P0KFe]02TY:@0n?Sh0Hf=S05YJFP1SHf<0TI6A04=3@`1FEUH0Ph>3
055AD@2ATI40TY:B03Xj>P2JVYX0GemO06][J`2NWYh0DE5A03Lg=`1oOgl0IVIV08F5Q@1]KFd092@T
05]KF`1jNWX0I6AT08R8R01>CTh0ATI6071`L02DU9@0IVIV08>3P`1hN7P0E5AD0820P02m_Kd0QXJ6
06QXJ01/K6`0MWIf096AT@2g]kL0I6AT08:2PP1KFe/0DU9B0:NWY`2m_Kd0ATI604i>CP1SHf<0MWIf
09nOW`1`L700?Cdm051@D016ATH0NGUi08j>SP0o?cl0@T9205UIF@1^KVh0Ng]k08^;R`0l?3`0M7Ad
07moO`1DE5@0TI6A04M7A`1ADE40P82005YJFP1UIFD0P82003Ph>01fMWH0G5aL06IVIP2DU9@0@410
03/k>`2IVIT0EeMG0:f][@1MGEd0>CTi069RHP1iNGT0JVYZ06QXJ00_;bl0FUYJ08>3P`1>CTh0MgMg
0451@@0k>c/0LG5a09fMW@1JFUX0RHV906m_K`1NGUh0QXJ607moO`14A4@0UiNG06YZJP1LG5`0WIfM
09JFUP1JFUX0OGem06YZJP1CDe<0QXJ607UiN@1=CDd0GemO065QH@1ZJVX0PX:204Y:BP1;Bd/0I6AT
06=SH`1@D500KVi^03De=@1NGUh0JVYZ0451@@1CDe<0Ng]k04U9B@1JFUX0Lg=c06ATI012@T80Ph>3
05MGE`0b<S80IfMW05UIF@1EEED0Ogmo05MGE`0_;bl0FEUI07alO01PH60>ooooo`0c08R8R023Ph<0
LW9b07IfMP1<C4`0Cdm?07UiN@1DE5@0NGUi05mOG`1KFe/0GemO06e]K@1TI6@0LG5a079bLP1HF5P0
Bd];05eMG@1_Kfl09RHV059BDP17AdL0@T9205IFEP1CDe<0@T9205IFEP26QXH0L71`0492@P1iNGT0
IFEU07]kN`1OGel0P82006][J`1PH600DU9B05mOG`1LG5`0GEeM06IVIP1=CDd0@D51051@D00`<300
A4A405YJFP1>CTh0AdM70080DE5A0>40@41005IFEP10@400G5aL05YJFP1PH600@d=3051@D01>CTh0
B4Q804U9B@1@D500UYJF065QH@0j>SX0EEEE075aL@1gMgL0KVi^04U9B@1aLG40BDU9055AD@1@D500
@T9204A4A01:BTX09bLW0451@@0o?cl0ATI603dm?@0/;2`0ATI601dM7@1HF5P0>c/k05iNGP0n?Sh0
?clo04];B`0/;2`0=3@d05IFEP15ADD0EeMG02/[:`1@D500=CDe04A4A01MGEd0EUIF07QhN01HF5P0
MgMg05]KF`1GEeL0MgMg05iNGP0j>SX0EUIF06=SH`17AdL0Fe]K075aL@1BDU80NWYj05eMG@1SHf<0
GUiN05EEE@1YJFT0Shn?03dm?@2LW9`0DE5A06UYJ@1OGel0UYJF09>CT`0l?3`0QhN705eMG@1?Cdl0
AdM704Y:BP15ADD0GUiN03/k>`1BDU80IFEU05MGE`1oOgl0GEeM065QH@1<C4`0A4A407IfMP1aLG40
JVYZ04];B`24Q8@0EUIF0;Rh^02LW9`0?Cdm051@D00b<S80QHF5051@D01bLW80CDe=06e]K@0];Bd0
JFUY051@D01>CTh0HF5Q03Ph>027QhL0AdM707IfMP1YJFT0C4a<04M7A`2XZ:P0ATI606YZJP1CDe<0
G5aL04e=C@14A4@0E5AD02l_;`0m?Cd0CDe=0451@@1CDe<0<C4a06EUI@18B4P0Fe]K02`/;00l?3`0
=CDe04M7A`1ADE40MGEe03Ti>@0n?Sh0:2PX04A4A01DE5@0=SHf04U9B@18B4P0?clo055AD@0j>SX0
E5AD05]KF`1<C4`0@D5104M7A`18B4P0F5QH034a<@1MGEd0EUIF07YjNP0c<c<0CDe=03`l?00i>CT0
GUiN02h^;P12@T80HF5Q0451@@1;Bd/0AdM705=CD`16ATH0D51@07UiN@0`<300A4A403lo?`1GEeL0
JFUY04a<C01XJ6P09RHV05QHF01cLg<0FEUI04U9B@1FEUH0:RXZ08b<S01:BTX0Lg=c03dm?@21PH40
N7Qh07AdM02QXJ40CTi>061PH00n?Sh0N7Qh04a<C01fMWH0J6QX06=SH`1[Jf/0D51@04m?C`1XJ6P0
Fe]K05QHF01WIfL0Fe]K05aLG01WIfL04?ooool000kooooo09<0FEUI07AdM01fMWH0NGUi05ADE01@
D500ZJVY05IFEP1ADE40?3`l061PH02LW9`0B4Q803Hf=P10@400Kfm_05mOG`1bLW80:BTY04A4A01?
Cdl0FEUI06=SH`1TI6@0:BTY05iNGP1XJ6P0DE5A0820P01?Cdl0@D5104m?C`1;Bd/0HF5Q0=[JfP1@
D500820P05=CD`1jNWX0Lg=c0;Zj^P11@D406ATI065QH@1HF5P0LG5a05YJFP0O7al0?Cdm0861P@0n
?Sh0GemO03Hf=P0_;bl0BTY:06e]K@1MGEd0Vi^K030`<00[:b/0E5AD07IfMP15ADD0M7Ad03Ti>@1B
DU80Ogmo08V9R@0m?Cd0EEEE02/[:`1BDU80HV9R07EeM@0Y:BT0N7Qh038b<P1@D500KVi^03Ti>@13
@d<0?Cdm02h^;P1jNWX0HF5Q01hN7P10@400@D5103`l?01oOgl0;2`/01dM7@0`<300>SXj04Q8B027
QhL0:b/[02h^;P1LG5`0Rh^;06a/K02GUiL0<c<c03lo?`1dM7@0RHV9075aL@2IVIT0;bl_04M7A`28
R8P0GUiN08F5Q@2DU9@08b<S04e=C@2KVi/0GUiN0861P@22PX808R8R0451@@1mOGd0Fe]K0:f][@1h
N7P0;2`/059BDP1PH600PH610:JVYP11@D40ADE5079bLP2LW9`0RHV907YjNP0c<c<0B4Q808B4Q02k
^k/0F5QH08N7Q`0];Bd0G5aL0;6a/@1RHV800P1MGEd0>018B4P0Lg=c0::RXP17AdL0DU9B06][J`13
@d<0M7Ad0:6QX@0g=cL0<c<c04i>CP0m?Cd0Hf=S08B4Q00l?3`0@d=305YJFP1[Jf/0WIfM05YJFP0Y
:BT0AdM707AdM01nOWh0Ph>305]KF`0[:b/0D51@07]kN`2;Rh/0WIfM05MGE`10@400Jf][05ADE01j
NWX0/k>c02/[:`1[Jf/0Z:RX07UiN@2a/K40DU9B04=3@`1MGEd0MGEe08N7Q`24Q8@0CTi>05=CD`2B
TY80P82008^;R`0^;Rh0D51@0P1[Jf/0601gMgL0AdM70451@@1kNg/0[:b/04U9B@1JFUX08B4Q04Q8
B01nOWh0Bd];07EeM@13@d<0>c/k07IfMP1]KFd0EEEE07EeM@0j>SX0Cdm?069RHP2SXj<0G5aL06UY
J@80?3`l02d0OWin07EeM@15ADD0GUiN04Q8B01?Cdl0O7al09BDU00l?3`0G5aL04i>CP1`L700YZJV
07AdM00_;bl0CTi>061PH01aLG40^;Rh05UIF@0`<300CTi>05mOG`2?Shl0[jn_01lO7`18B4P0DU9B
04i>CP22PX80[Jf]024Q8@10@400HV9R05MGE`1PH600^;Rh01dM7@14A4@0IfMW05eMG@18B4P0[Zj^
02XZ:P1>CTh03_ooool0LP2?Shl0QXJ6059BDP15ADD0Fe]K059BDP1=CDd0<C4a06YZJP1KFe/0@T92
051@D01ZJVX0GUiN09VIV@1SHf<0PH6104U9B@1ZJVX0IVIV02HV9P15ADD0C4a<0492@P1=CDd0Cdm?
09^KV`24Q8@0JVYZ05YJFP1SHf<0MWIf05iNGP1=CDd0<30`065QH@1HF5P0L71`06EUI@1IFET0CTi>
04Y:BP1LG5`0AdM703<c<`0n?Sh0;Rh^04I6AP1FEUH0EEEE03<c<`0j>SX0Bd];071`L01WIfL0Bd];
06YZJP16ATH0J6QX04A4A01=CDd0Hf=S059BDP1>CTh0EEEE09NGU`1ZJVX0?Shn04U9B@2HV9P0X:2P
06=SH`1;Bd/0EEEE02d];@0c<c<0EUIF04i>CP11@D40De=C03Hf=P12@T80@41004i>CP0j>SX0=cLg
0451@@0c<c<0De=C04I6AP1lO7`0@D5103Hf=P11@D40>c/k04I6AP0e=CD0?clo05eMG@1@D500=SHf
03lo?`1>CTh0KFe]06ATI026QXH0Fe]K04i>CP1EEED0G5aL0:BTY01/K6`0?3`l06EUI@80ADE506d0
CTi>06m_K`12@T80KVi^06YZJP1GEeL0NGUi07EeM@1VIVH0K6a/06=SH`36a/H0GemO061PH01@D500
VYZJ08^;R`1KFe/0N7Qh07AdM01TI6@0J6QX03hn?P0Z:RX0=cLg03Ph>00e=CD0CDe=06MWI`2SXj<0
Cdm?05ADE01CDe<0=cLg051@D01WIfL0PH6106EUI@1EEED0DE5A0<W9b@20P800>3Ph05QHF00n?Sh0
Shn?05=CD`1FEUH0IFEU08F5Q@0[:b/0B4Q803De=@1gMgL0OWin04];B`28R8P0:b/[05mOG`1[Jf/0
C4a<03Ti>@2QXJ40>3Ph07=cL`1IFET0Cdm?034a<@1KFe/0MWIf028R8P0W9bL0BDU905]KF`1XJ6P0
;Rh^05EEE@0k>c/0AdM70492@P0a<C40<c<c0451@@16ATH0J6QX02XZ:P0_;bl092@T04I6AP0n?Sh0
;Bd]04];B`0d=3@0<30`03dm?@0c<c<0ATI604M7A`1<C4`0>3Ph0410@015ADD0FUYJ02l_;`1RHV80
?Cdm04e=C@0`<300IFEU0080@41003<0>c/k030`<011@D40DU9B02l_;`0e=CD0@T9204e=C@0g=cL0
Bd];069RHP0S8b<0D51@04I6AP0m?Cd0IVIV04a<C01fMWH0;Rh^04];B`1<C4`0>3Ph03hn?P1LG5`0
AdM709>CT`0e=CD0FUYJ04I6AP20P800IFEU079bLP2>SXh0BDU905IFEP1<C4`0LG5a059BDP1KFe/0
EUIF06m_K`1nOWh0@D51051@D01[Jf/0GemO04i>CP1WIfL0CTi>04];B`1LG5`04?ooool000kooooo
07P0Ph>3069RHP1nOWh0TI6A065QH@0o?cl0JFUY06IVIP2ATI40DU9B069RHP1dM7@0IFEU08b<S01l
O7`0<C4a0410@01lO7`0DE5A08>3P`1]KFd08B4Q05eMG@1>CTh0@T920:NWY`1LG5`0<C4a075aL@1X
J6P0IFEU09RHV018B4P07QhN06UYJ@2`/;00H61P05aLG00c<c<0>CTi08J6QP1iNGT0;Bd]0451@@0X
:2P0DU9B07UiN@0n?Sh0;bl_04i>CP0j>SX0HF5Q05aLG00N7Qh0=3@d04];B`16ATH0W9bL03hn?P0O
7al0Bd];04a<C01BDU80ZJVY03De=@0Q8B40C4a<06][J`1ADE40JVYZ02PX:00l?3`0G5aL07emO@1B
DU80][Jf02TY:@16ATH0Hf=S05UIF@2<S8`0[Jf]02LW9`1@D500NWYj05MGE`1/K6`0@T9201hN7P1K
Fe/0NGUi06=SH`1kNg/08B4Q02l_;`1=CDd0N7Qh06][J`1QHF408R8R02PX:01LG5`0JVYZ08N7Q`0i
>CT07AdM03Lg=`1<C4`0HV9R05mOG`0W9bL07alO04Y:BP1SHf<0DE5A04M7A`0a<C40=CDe069RHP1`
L70203Lg=`2L06=SH`14A4@0WYjN09fMW@1:BTX0@41005YJFP0j>SX0VYZJ05eMG@0S8b<0JVYZ08F5
Q@1OGel0UYJF02l_;`0];Bd0EeMG07EeM@1HF5P0W9bL030`<00i>CT0EEEE05MGE`1_Kfl0IfMW04E5
A@1QHF40KFe]07moO`2OWil0/[:b04U9B@1SHf<0X:2P08F5Q@29RHT0W9bL01dM7@1IFET0UiNG08F5
Q@2?Shl0RXZ:02PX:01QHF40Q8B405MGE`27QhL0GUiN0410@024Q8@0JVYZ030`<02RXZ80DU9B02/[
:`25QHD0[:b/07AdM01DE5@0@D5106ATI02GUiL0OWin08:2PP14A4@0>3Ph096AT@2YZJT0A4A406m_
K`0c<c<0EUIF0;Fe]@1:BTX0GEeM05MGE`15ADD0QHF504];B`0m?Cd0HF5Q04U9B@2MWId0Kfm_034a
<@1EEED0HF5Q07EeM@2@T900=SHf03@d=01RHV80Cdm?09JFUP1RHV80;2`/05eMG@1kNg/0MgMg09nO
W`1@D500@41004M7A`1dM7@0Ph>30;Fe]@0f=SH0Cdm?06IVIP2HV9P0KFe]0:2PX00e=CD0GEeM0861
P@2OWil0JVYZ096AT@12@T80=cLg08Z:RP2i^KT0F5QH06YZJP0V9RH0F5QH05iNGP27QhL0De=C065Q
H@0/;2`0Q8B40:ZZZP1IFET0>3Ph06QXJ00j>SX0BDU908Z:RP1BDU80=3@d06ATI015ADD0CTi>0:b/
[01OGel09bLW03lo?`1ADE40BDU90;^k^`1oOgl0?3`l3_ooool0301_Kfl0O7al05UIF@1IFET0BDU9
04Q8B01?Cdl0@D5106i^KP1[Jf/0ADE505QHF080QXJ609h0UYJF06=SH`22PX80KVi^07UiN@1ZJVX0
=cLg05aLG01?Cdl0@D5105mOG`11@D40@d=305iNGP1MGEd0JVYZ04I6AP1<C4`0FUYJ075aL@1OGel0
MWIf05aLG01VIVH0EeMG061PH01EEED0G5aL061PH014A4@0?3`l05IFEP15ADD0B4Q805EEE@14A4@0
@d=303lo?`13@d<0=SHf05MGE`15ADD0IfMW0492@P1eMGD0DU9B05=CD`1VIVH0EeMG05aLG01EEED0
WIfM07AdM014A4@0BTY:05MGE`1TI6@0F5QH04a<C01XJ6P0@d=304Q8B01VIVH0HF5Q03`l?01OGel0
=SHf05]KF`0g=cL0?3`l03<c<`0T92@0>3Ph02XZ:P19BDT0AdM706YZJP1ADE40ADE505IFEP17AdL0
Bd];05MGE`1?Cdl0JVYZ0410@01LG5`0?clo04E5A@18B4P0I6AT08f=S@1JFUX0De=C06][J`1SHf<0
B4Q807AdM013@d<0SXj>075aL@1@D500CDe=09NGU`1dM7@0OWin059BDP1`L700M7Ad05UIF@2OWil0
UiNG06=SH`1cLg<0GemO05MGE`0Z:RX0Fe]K05ADE00c<c<0Ogmo079bLP1ZJVX0D51@065QH@1`L700
Cdm?03Ti>@1EEED0MGEe07moO`1VIVH0E5AD06=SH`1ADE407alO07QhN01fMWH0H61P04A4A01PH600
NGUi0>[ZjP2UYJD0@d=304M7A`0n?Sh0Ph>304=3@`19BDT0Bd];06QXJ00i>CT0F5QH04U9B@1HF5P0
D51@0451@@80MGEe00l0IFEU061PH00m?Cd0IVIV09ZJVP0Z:RX0E5AD04E5A@1/K6`0ATI605aLG01S
Hf<0:BTY05EEE@1:BTX00P0];Bd01P0X:2P0BDU9034a<@19BDT0<30`02@T9080>CTi04l0DU9B08F5
Q@11@D40BDU902PX:010@400?clo01XJ6P0e=CD0<S8b04Y:BP19BDT0?Shn05UIF@1OGel0@41003hn
?P0l?3`0AdM704e=C@0h>3P0G5aL03lo?`1XJ6P0;Bd]03<c<`0a<C40BDU903Xj>P0O7al0=CDe059B
DP0m?Cd0ATI603lo?`13@d<0ADE505mOG`1eMGD0E5AD04E5A@0n?Sh0CTi>06EUI@1dM7@0RXZ:04a<
C01WIfL0Kfm_03Lg=`12@T80EeMG051@D02b/[80@d=3071`L00n?Sh0Kfm_03lo?`1JFUX0Q8B403<c
<`19BDT092@T05MGE`0a<C40H61P04e=C@1EEED0E5AD04U9B@1GEeL0LW9b05=CD`1EEED0Hf=S05aL
G01?Cdl0P820013ooooo000>ooooo`0@09:BTP2?Shl0M7Ad05UIF@1ZJVX0K6a/06YZJP0j>SX0=3@d
06][J`1ADE40JFUY03Ph>00d=3@0GEeM034a<@80<30`0900?clo06a/K02<S8`08B4Q03<c<`1>CTh0
Fe]K09fMW@35aLD09bLW030`<01HF5P0Q8B407QhN00Y:BT06ATI03lo?`1BDU80KVi^0;Fe]@0a<C40
92@T05aLG01[Jf/0DE5A0820P00W9bL0<S8b06UYJ@1MGEd0MWIf06UYJ@0Z:RX0De=C055AD@0d=3@0
JVYZ06MWI`0[:b/0HV9R07]kN`1CDe<0MGEe03Lg=`0[:b/0Ph>306EUI@1LG5`0HV9R01DE5@10@400
HV9R03Lg=`0l?3`0EUIF01dM7@1;Bd/0JVYZ0492@P1KFe/0@41002<S8`1TI6@0HV9R05=CD`1PH600
=SHf0492@P1JFUX0FEUI02`/;019BDT0820P051@D02IVIT0BTY:01dM7@0Y:BT0>c/k07alO02ATI40
>SXj02LW9`13@d<0GEeM0:j^[P29RHT0:BTY03Xj>P16ATH0?Cdm0:>SX`1RHV808b<S04a<C01GEeL0
HV9R09FEU@0a<C406ATI03Hf=P1aLG40NWYj06m_K`20P800<c<c05=CD`2QXJ40Rh^;08N7Q`2YZJT0
ATI605QHF01cLg<0DU9B0:b/[01GEeL0EEEE08R8R01kNg/0P82009FEU@0d=3@0@D510;:b/P1kNg/0
PH6107]kN`0S8b<0F5QH08b<S01IFET0SXj>04a<C00n?Sh0EUIF075aL@1DE5@0X:2P06ATI013@d<0
I6AT08N7Q`1^KVh2055AD@1M05YJFP1`L700[Zj^0451@@1>CTh0C4a<07IfMP2MWId0Xj>S03<c<`12
@T80IVIV04i>CP33`l<0S8b<02<S8`0m?Cd0Jf][07UiN@1dM7@07QhN04e=C@27QhL0M7Ad0:6QX@0o
?cl06ATI05UIF@2?Shl0NGUi0;Vi^@0m?Cd0BDU907inOP1/K6`0X:2P08f=S@0b<S80C4a<075aL@24
Q8@0UYJF02<S8`14A4@0L71`06a/K02_[jl0=cLg03Xj>P2FUYH0O7al05UIF@1DE5@092@T061PH02K
Vi/0A4A4069RHP0_;bl0CDe=07alO02:RXX0FUYJ07]kN`1CDe<0GEeM07alO01EEED0@D51061PH00n
?Sh0HV9R08j>SP1MGEd0:RXZ04m?C`1:BTX0MGEe0:FUY@1QHF40;Bd]03Ti>@1TI6@0Ogmo0861P@0W
9bL0=CDe07]kN`1lO7`0FEUI06m_K`0R8R80=CDe0080FUYJ01<0Fe]K09:BTP0e=CD0?Cdm07MgM`1_
Kfl0K6a/0;^k^`1PH600>SXj07AdM028R8P0DU9B0:>SX`1GEeL0=CDe05ADE02KVi/0H61P00kooooo
04d0QXJ607UiN@1ZJVX0J6QX04m?C`1ADE40HF5Q0451@@1CDe<0D51@04];B`15ADD0E5AD04Q8B01U
IFD0PX:207=cL`0n?Sh0I6AT05YJFP0G5aL0=cLg03Hf=P1:BTX0?Shn03Ti>@1=CDd0BTY:04=3@`1:
BTX0>c/k04m?C`1<C4`0CTi>0492@P1IFET0CDe=03hn?P13@d<0AdM705ADE01NGUh0ATI60410@00^
;Rh0CDe=01hN7P1HF5P0DE5A04M7A`0a<C40AdM704m?C`0b<S80De=C04I6AP1]KFd0Bd];05MGE`1:
BTX0H61P05aLG01MGEd0LG5a06EUI@2MWId0Bd];0410@01;Bd/0MWIf08^;R`1XJ6P0GemO06=SH`0o
?cl0AdM705eMG@0205ADE01805iNGP1BDU80B4Q803Xj>P1;Bd/0=cLg038b<P1;Bd/07QhN05IFEP14
A4@0R8R803dm?@16ATH0?clo02XZ:P0d=3@0EeMG03Xj>P1RHV80>3Ph06ATI00o?cl0=SHf04e=C@1S
Hf<0/k>c05YJFP1PH600Kfm_08R8R01VIVH0BTY:03`l?026QXH0@d=3065QH@1@D500MWIf04E5A@2?
Shl0FUYJ07AdM01RHV80SHf=0:RXZ02ATI40M7Ad09>CT`15ADD0H61P05eMG@1nOWh0O7al04a<C01V
IVH0B4Q804Y:BP13@d<0>CTi03/k>`1TI6@0ADE5055AD@1;Bd/0MWIf0:VYZ@1@D500RHV903/k>`13
@d<0D51@0P1[Jf/0G`1RHV80NWYj04a<C01nOWh0OGem04I6AP1LG5`0?Cdm07=cL`1QHF40Rh^;07=c
L`1nOWh0FUYJ0861P@1EEED0H61P05iNGP0`<300P82005eMG@1^KVh0HV9R04M7A`1GEeL0Ogmo0410
@01/K6`0JFUY04E5A@0i>CT0Bd];051@D00S8b<0@T9204U9B@1MGEd0EeMG030`<00o?cl0DE5A05iN
GP0k>c/0;2`/055AD@14A4@0?3`l05mOG`0b<S80<C4a02@T9016ATH0Bd];01/K6`0];Bd0:2PX03De
=@1<C4`0;2`/04I6AP1MGEd0BTY:04=3@`1=CDd0CTi>05QHF00i>CT0S8b<04e=C@1<C4`0>CTi06YZ
JP0`<300EEEE02h^;P0d=3@0CTi>065QH@0i>CT0?Shn03`l?01;Bd/0>3Ph04A4A01<C4`06a/K03/k
>`19BDT0FEUI06a/K00e=CD0I6AT02h^;P1BDU80QhN70080EUIF01`0FUYJ05aLG01oOgl0@D5104I6
AP0l?3`0QXJ604I6AP1KFe/0HV9R034a<@1OGel0<S8b05QHF00l?3`0Ng]k065QH@1>CTh0A4A405=C
D`0i>CT0I6AT055AD@1CDe<0AdM705eMG@1lO7`0L71`4?ooool000kooooo06T0JVYZ0861P@2PX:00
GUiN059BDP28R8P0XZ:R03@d=00h>3P0D51@08F5Q@2k^k/0<C4a04A4A01EEED0BTY:09FEU@0g=cL0
?3`l05]KF`1hN7P0F5QH06m_K`1KFe/0:RXZ08N7Q`2SXj<0KVi^08B4Q00];Bd0@D5107YjNP1ZJVX0
E5AD09bLW01gMgL0;Bd]07emO@2<S8`0EUIF09NGU`18B4P04a<C075aL@1KFe/0D51@05YJFP0V9RH0
;bl_07inOP1<C4`0Bd];03hn?P0`<300Fe]K05aLG00h>3P0H61P03Hf=P1=CDd0VIVI04e=C@0Z:RX0
Hf=S04Y:BP1WIfL0Xj>S04A4A00O7al0CDe=075aL@23Ph<0PH6102LW9`0Q8B40@T9205ADE02WYjL0
LW9b01hN7P0Y:BT0E5AD06UYJ@2JVYX0<c<c02LW9`1IFET0OWin04M7A`1_Kfl05aLG030`<01WIfL0
G5aL04Q8B00e=CD071`L02PX:01cLg<0JFUY06ATI01SHf<09bLW04a<C020P8000P1PH600Z`1NGUh0
<C4a04];B`2d];@0Hf=S06a/K01WIfL09BDU04U9B@1VIVH0<c<c05mOG`19BDT0=cLg04Y:BP1bLW80
=3@d079bLP1BDU80Fe]K07AdM02TY:@0@T9205QHF00k>c/0Bd];07moO`2FUYH0ADE507EeM@10@400
TI6A0;^k^`17AdL0>c/k061PH01HF5P0ZZZZ0;6a/@0];Bd0ADE505eMG@1^KVh0]kNg08N7Q`0a<C40
EEEE05ADE01eMGD0alO709BDU00];Bd0EUIF06IVIP2>SXh0UIFE09>CT`0T92@0IVIV09VIV@2MWId0
S8b<0451@@1<C4`0QHF50;Bd]01RHV80UIFE05YJFP13@d<0OGem0:VYZ@1;Bd/0Hf=S02h^;P1BDU80
[Jf]05UIF@1NGUh0Ng]k01dM7@1hN7P0ZZZZ04i>CP1jNWX0@T9203<c<`1?Cdl0GemO06e]K@1JFUX0
?Shn04e=C@2EUID0Cdm?061PH00i>CT0DE5A0:2PX01lO7`0EeMG05iNGP1HF5P0Vi^K08:2PP10@400
E5AD069RHP1VIVH0LW9b02LW9`1;Bd/0LW9b04];B`2FUYH0;2`/034a<@14A4@0IFEU06IVIP2c/k<0
GemO05YJFP2>SXh0Ng]k07=cL`2[Zj/0@D5103lo?`20P800OGem07EeM@2k^k/0L71`03hn?P2BTY80
^k^k07emO@2?Shl0@d=303Ph>02XZ:P0M7Ad05QHF02XZ:P0<c<c03hn?P23Ph<0@41006ATI01UIFD0
<30`04U9B@1VIVH0EEEE069RHP1KFe/0@d=304];B`1DE5@0JFUY04];B`20P800=CDe04I6AP18B4P0
Kfm_02`/;01EEED0>CTi03@d=00>ooooo`0808N7Q`2JVYX0GEeM05]KF`1OGel0JVYZ06QXJ00n?Sh2
069RHP1A03/k>`1IFET0Shn?09>CT`1dM7@0Cdm?05=CD`1:BTX0EEEE04];B`0W9bL0@D51034a<@0d
=3@0ATI604a<C010@400De=C07=cL`1LG5`0>3Ph06EUI@1NGUh0Lg=c04Y:BP1^KVh0MWIf055AD@1K
Fe/0F5QH03Hf=P0c<c<0=CDe03Hf=P0/;2`0Bd];02h^;P19BDT0BTY:04e=C@0g=cL0Bd];051@D01A
DE40Bd];04E5A@1JFUX0Bd];05YJFP1DE5@0DU9B05eMG@1>CTh0AdM706ATI02EUID0?clo04Q8B01Y
JFT0VIVI07inOP1CDe<0ATI606YZJP1GEeL0Jf][07EeM@1aLG40F5QH061PH00f=SH0D51@04E5A@1C
De<0@T92055AD@1BDU80>CTi05QHF01CDe<0LW9b0080B4Q802D0EEEE038b<P15ADD0GEeM051@D01[
Jf/0ATI605mOG`15ADD0?clo04m?C`1<C4`0LW9b05ADE01MGEd0GUiN06e]K@2DU9@0MWIf04I6AP28
R8P0BDU903@d=00n?Sh0HV9R02HV9P1KFe/0?Cdm06m_K`1]KFd0K6a/0:n_[`1oOgl0NWYj096AT@1X
J6P0RXZ:0080GEeM01d0Jf][06=SH`25QHD0IVIV05IFEP1=CDd0BDU907IfMP1PH600@41003lo?`1Y
JFT0U9BD09FEU@14A4@0De=C04i>CP0Y:BT0Cdm?04M7A`0k>c/0@d=304Y:BP1>CTh0^[Zj06EUI@1I
FET0M7Ad055AD@0206MWI`1N04A4A01LG5`0GemO03<c<`1KFe/0>SXj065QH@1LG5`0;bl_08F5Q@12
@T80Bd];06MWI`1UIFD0NGUi06e]K@1LG5`0[:b/04a<C00l?3`0:RXZ06ATI01kNg/08b<S03hn?P0h
>3P0EUIF04a<C00/;2`0De=C02LW9`1:BTX0?clo03`l?018B4P0=SHf055AD@1RHV80Cdm?05]KF`1:
BTX0Fe]K04Q8B00];Bd0B4Q80410@00n?Sh0B4Q803hn?P12@T80GEeM05ADE00n?Sh0>3Ph05eMG@1Q
HF407alO05QHF00e=CD0EeMG03Hf=P0h>3P092@T04m?C`0[:b/0=SHf02PX:01:BTX0=CDe0451@@1F
EUH0@T9203@d=01DE5@0DU9B03@d=01KFe/0BTY:04U9B@1bLW80KVi^0:2PX00Y:BT0E5AD05UIF@10
@400IVIV08B4Q01=CDd0O7al03De=@1RHV80AdM705iNGP80EeMG01<0H61P04Q8B01IFET0C4a<06=S
H`17AdL0GUiN04M7A`1;Bd/0CDe=04A4A01;Bd/0L71`04e=C@1?Cdl0EeMG065QH@1jNWX0FUYJ013o
oooo000>ooooo`1J05mOG`1eMGD0L71`09nOW`15ADD0CTi>07AdM01gMgL0G5aL04a<C00j>SX0LG5a
06a/K01^KVh0H61P02/[:`1LG5`0Cdm?03Ti>@1jNWX0?3`l04=3@`1HF5P0De=C03@d=01PH600?3`l
04m?C`22PX80HV9R03Lg=`1RHV80@D5103/k>`26QXH0L71`03Ti>@10@400@d=304i>CP2`/;00H61P
01/K6`0e=CD0?Cdm09JFUP2@T9009BDU02`/;01?Cdl0DE5A0:2PX029RHT06QXJ03Lg=`1_Kfl0LG5a
0;Ng]`0`<3008b<S07=cL`2:RXX0Hf=S07YjNP0g=cL0JFUY07QhN028R8P0L71`05]KF`0g=cL0EeMG
08B4Q029RHT0H61P07QhN00X:2P0@D5107moO`1EEED0NGUi05YJFP0d=3@0Hf=S065QH@0`<300ADE5
02l_;`0c<c<0NGUi0P0f=SH0^P1:BTX0=3@d04Q8B01CDe<08R8R0451@@0n?Sh0FUYJ07EeM@1mOGd0
9BDU02HV9P1GEeL0CDe=08F5Q@2=SHd092@T02l_;`1DE5@0CTi>08:2PP1mOGd092@T05IFEP1XJ6P0
ATI608N7Q`2ATI40<c<c059BDP2KVi/0GUiN0<?3``1aLG40=cLg05IFEP2SXj<0M7Ad09JFUP0`<300
9bLW06IVIP2DU9@0QHF50:RXZ013@d<0?Shn079bLP2NWYh0LG5a08N7Q`0[:b/0C4a<08n?S`1KFe/0
FEUI04];B`1=CDd0JVYZ08B4Q01BDU80TI6A0:>SX`0l?3`0IFEU09>CT`1ADE40FEUI06UYJ@0V9RH0
GUiN0:2PX01?Cdl0HV9R07EeM@0d=3@0Ph>304Y:BP0j>SX0KVi^03hn?P1DE5@0MGEe04Q8B00Z:RX0
Fe]K06i^KP1dM7@0QHF504m?C`0a<C40GemO06IVIP1cLg<0[jn_02d];@0l?3`0Ti>C07emO@3Ng]h0
O7al0410@012@T80HV9R0;jn_P2EUID0;Rh^04U9B@1IFET0QXJ60;6a/@0V9RH0F5QH06UYJ@1[Jf/0
[Zj^07=cL`17AdL0Ogmo07=cL`23Ph<0_[jn04m?C`19BDT0][Jf06QXJ02>SXh0I6AT04E5A@1iNGT0
LW9b05]KF`1`L700;bl_04e=C@1CDe<0OWin07AdM01SHf<0>c/k06][J`2:RXX0MWIf05eMG@1NGUh0
>c/k05aLG01[Jf/0Ogmo03<c<`11@D40=cLg08F5Q@2DU9@0SHf=038b<P11@D40ATI606ATI02b/[80
EUIF038b<P1ADE40GemO08>3P`25QHD06a/K034a<@16ATH0EEEE069RHP1eMGD07AdM0451@@1DE5@0
H61P05aLG02ZZZX0:b/[0451@@1XJ6P0Kfm_06=SH`2e]KD0<C4a03hn?Pkooooo0340Jf][0:>SX`1G
EeL0De=C0410@01@D500F5QH04I6AP1TI6@0EeMG03Xj>P1SHf<0K6a/04i>CP1TI6@0FUYJ07emO@1W
IfL0Q8B404Q8B00e=CD0HF5Q04U9B@10@400F5QH04U9B@0g=cL0BDU905=CD`1[Jf/0>CTi04Y:BP1@
D500H61P04A4A01HF5P0BTY:04a<C015ADD0>CTi03lo?`1;Bd/0BDU905]KF`0];Bd0A4A401PH601I
FET0=3@d0080>c/k00P0BTY:02d];@0O7al08b<S04U9B@1PH600;Bd]04E5A@80JVYZ04/0C4a<05=C
D`1TI6@0IfMW07IfMP1CDe<0LW9b08:2PP1>CTh0I6AT07YjNP1oOgl0GUiN02LW9`0`<300ATI605iN
GP0k>c/0C4a<03dm?@1IFET0<30`04];B`0_;bl0:RXZ02`/;01FEUH0L71`04I6AP2TY:@0@d=303<c
<`0h>3P0EUIF05]KF`0l?3`0@D510861P@16ATH0Cdm?061PH01KFe/0DE5A08b<S02g]kL0H61P07mo
O`1aLG40Ng]k08Z:RP1bLW80@D5109:BTP1/K6`0I6AT04a<C01PH600C4a<07]kN`1CDe<0SXj>07em
O@1RHV80e]KF0861P@2HV9P0cLg=03/k>`1NGUh0AdM70;Vi^@1WIfL0<30`05IFEP1IFET00P1CDe<0
9011@D40?3`l04I6AP1ZJVX0C4a<055AD@2?Shl0WYjN06ATI01cLg<0C4a<03hn?P1_Kfl0Jf][06IV
IP1@D500C4a<06ATI02g]kL0PH6104Q8B01>CTh0@41005UIF@1GEeL0=CDe05IFEP1QHF40>SXj05]K
F`1UIFD0]KFe07AdM01JFUX0MWIf079bLP80DE5A03`0EEEE08>3P`2@T900Fe]K05eMG@12@T80A4A4
03Ph>01UIFD0HV9R02d];@1IFET0<C4a04Q8B01@D500BTY:06UYJ@10@400De=C04Y:BP0c<c<0DE5A
059BDP23Ph<0PX:203lo?`0g=cL06a/K03lo?`11@D409BDU04];B`1ADE40@d=3051@D011@D40E5AD
06a/K01ADE40Bd];03dm?@1;Bd/0C4a<02@T901VIVH0CTi>06EUI@0e=CD0AdM703<c<`1;Bd/05ADE
034a<@16ATH0K6a/04M7A`12@T80HV9R04Q8B015ADD205iNGP0D04=3@`0j>SX0De=C04m?C`1ADE40
G5aL06IVIP1eMGD0C4a<06e]K@0n?Sh0=SHf05EEE@1;Bd/0OWin05IFEP1]KFd0=3@d06ATI01;Bd/2
06][J`0B06EUI@1WIfL0IfMW06YZJP10@400Fe]K03Ph>01<C4`0A4A404I6AP17AdL0?Cdm04a<C01>
CTh0Hf=S04e=C@1EEED0H61P4?ooool000kooooo04L0UIFE07UiN@1OGel0OGem092@T01ZJVX0Lg=c
04I6AP1^KVh0Cdm?06EUI@1JFUX0;2`/05]KF`25QHD0>SXj04=3@`0f=SH0@41008j>SP2=SHd0?Shn
06EUI@1CDe<0Hf=S0:>SX`1cLg<0;Bd]05IFEP1JFUX0I6AT09:BTP1ADE40@T9205MGE`1FEUH0HV9R
07UiN@0j>SX0<c<c055AD@1QHF40CDe=075aL@0Z:RX0:b/[05ADE015ADD0C4a<05aLG00c<c<0E5AD
051@D00f=SH0IVIV05iNGP0/;2`0IFEU0451@@18B4P0X:2P069RHP0k>c/0S8b<051@D01gMgL0PX:2
02d];@13@d<0Jf][0492@P0208B4Q00L02HV9P1EEED0YJFU04m?C`1UIFD0@T92034a<@1ZJVX0UYJF
051@D01LG5`0=CDe0492@P2BTY80IFEU02h^;P11@D40@T9206][J`2>SXh0>c/k038b<P1BDU80?Shn
07QhN017AdL0:b/[03hn?P80HV9R03/0UYJF03`l?00c<c<0@T9206QXJ01/K6`0KVi^0410@00Q8B40
CTi>06ATI01<C4`0NWYj02l_;`0R8R80BTY:06QXJ01<C4`0H61P06UYJ@0k>c/0LW9b07EeM@1>CTh0
R8R809RHV00j>SX0P82006m_K`1<C4`0LG5a03`l?011@D40MgMg05YJFP1OGel0IfMW034a<@1CDe<0
S8b<05mOG`1dM7@0KFe]03Ph>021PH40MWIf05IFEP1jNWX0F5QH04E5A@1lO7`0SXj>051@D02CTi<0
HV9R04i>CP24Q8@0UiNG0451@@02069RHP1b05mOG`28R8P0VYZJ03Xj>P1SHf<0EUIF06IVIP2ZZZX0
OGem03`l?01]KFd0GemO069RHP2ATI40De=C04U9B@1_Kfl0HV9R06EUI@1CDe<0<30`05mOG`22PX80
IFEU07=cL`0S8b<0=SHf08:2PP2?Shl0J6QX08:2PP0h>3P0EeMG08F5Q@1/K6`0LG5a06UYJ@15ADD0
UYJF05]KF`25QHD0TY:B02`/;00n?Sh0WYjN07EeM@1aLG40=3@d04A4A01QHF40MWIf06ATI012@T80
@d=308F5Q@2GUiL0BDU9071`L012@T80Fe]K08Z:RP1jNWX0BDU907]kN`1?Cdl0OWin09nOW`1HF5P0
D51@07UiN@1OGel0TI6A09^KV`1?Cdl0EEEE07]kN`1QHF40IfMW08^;R`1IFET0BTY:07UiN@1WIfL0
LG5a07EeM@0i>CT0H61P07inOP1fMWH0HF5Q075aL@0^;Rh0C4a<06UYJ@1CDe<0D51@08R8R00f=SH0
EUIF05iNGP1LG5`0FEUI09FEU@1/K6`0=CDe05UIF@1SHf<0@d=308f=S@1_Kfl0<S8b05]KF`26QXH0
De=C3_ooool206a/K00Q051@D01CDe<0E5AD06ATI01OGel0CTi>061PH01DE5@0@D5105MGE`1bLW80
MGEe06YZJP1WIfL0HF5Q0410@01CDe<0?Cdm038b<P18B4P0=CDe03Lg=`14A4@0@d=303`l?01DE5@0
EeMG061PH00i>CT0CTi>06UYJ@1NGUh0AdM70080EUIF03H0@d=304];B`1<C4`0@41004i>CP0l?3`0
@d=302PX:00e=CD07QhN04E5A@14A4@0?clo03Hf=P15ADD0>CTi04U9B@12@T80ATI605MGE`1:BTX0
CTi>05aLG01HF5P0DU9B04Y:BP1@D500MWIf08N7Q`10@400?3`l04A4A01/K6`0GEeM03hn?P0m?Cd0
DE5A03`l?019BDT0EEEE05IFEP1ADE40LW9b03lo?`1RHV80BTY:059BDP1;Bd/0AdM704E5A@0k>c/0
FEUI055AD@1_Kfl204Q8B01d05=CD`0l?3`0BDU905QHF016ATH0H61P04E5A@1DE5@0ATI60451@@1=
CDd0F5QH06][J`1<C4`0I6AT05aLG01XJ6P0SHf=09jNWP1MGEd0Rh^;04i>CP1HF5P0C4a<065QH@0f
=SH0JFUY05aLG022PX80MGEe05aLG038b<P0MWIf08J6QP2UYJD0EUIF07MgM`1:BTX0OWin05aLG01B
DU80LW9b04=3@`10@400G5aL04E5A@1<C4`0BTY:05MGE`1ADE40>c/k0820P02?Shl0EUIF065QH@1K
Fe/0>SXj07emO@1nOWh0F5QH04m?C`1/K6`0IVIV08j>SP18B4P0GUiN05eMG@1EEED0I6AT07YjNP1>
CTh0E5AD05eMG@13@d<0N7Qh04m?C`1VIVH0DU9B04A4A01iNGT0E5AD05iNGP1PH600Bd];06EUI@2I
VIT0JVYZ07MgM`1;Bd/0GemO059BDP1[Jf/0Hf=S02LW9`1HF5P0Bd];07inOP1>CTh0=CDe04I6AP1@
D500HF5Q04i>CP0m?Cd0EUIF03Ph>01NGUh0HV9R03Ti>@0f=SH020P80492@P0e=CD09bLW03lo?`12
@T8204E5A@0G03hn?P1:BTX0GemO04E5A@0o?cl0;bl_04];B`1BDU808R8R061PH00k>c/0?clo03Hf
=P16ATH0;Rh^05eMG@0l?3`0>SXj04Y:BP1KFe/0De=C04i>CP1`L7000P1=CDd0:P1iNGT0DE5A03Ti
>@11@D40A4A405MGE`1aLG40G5aL07alO00g=cL0EEEE07UiN@1ADE40GUiN07emO@1RHV80PH610410
@01BDU80FUYJ07QhN014A4@0E5AD05]KF`0m?Cd0DU9B04I6AP1BDU80@D5106YZJP1EEED0GUiN05mO
G`1?Cdl0Bd];06a/K01ADE40E5AD05iNGP1JFUX0K6a/06m_Ka3ooooo000>ooooo`0O069RHP1WIfL0
K6a/05iNGP18B4P0G5aL071`L01CDe<0GEeM04U9B@1EEED0S8b<04a<C00k>c/0=SHf03<c<`1fMWH0
9RHV01dM7@10@400?clo061PH01aLG40ATI603<c<`19BDT0PH6108J6QP1TI6@08b<S02HV9P02065Q
H@3e07UiN@2KVi/0=CDe034a<@1LG5`0MgMg06e]K@2_[jl0=CDe01LG5`1]KFd0CDe=07]kN`1NGUh0
:b/[03Hf=P1jNWX0@d=309^KV`0_;bl0:b/[04E5A@1UIFD0@T9206EUI@0J6QX0;2`/061PH01nOWh0
EEEE0451@@0O7al0=3@d071`L01LG5`0<C4a04=3@`0e=CD0GUiN07MgM`19BDT09RHV04=3@`0j>SX0
QHF5069RHP0N7Qh0=CDe03hn?P1QHF40];Bd03Hf=P0D51@0@D5105MGE`1NGUh0Jf][01PH600Z:RX0
KVi^06EUI@1OGel0BDU903Ti>@0o?cl0FEUI061PH022PX80Rh^;03<c<`1NGUh0NWYj07inOP1kNg/0
FEUI02PX:01RHV80WYjN069RHP22PX80MWIf03Hf=P1SHf<0ZZZZ0410@02XZ:P0GEeM03hn?P0o?cl0
NGUi05MGE`2LW9`0G5aL0492@P1YJFT0WIfM075aL@2ATI40=CDe04Y:BP1VIVH0Ph>304m?C`1KFe/0
A4A404m?C`2CTi<0DE5A04m?C`1HF5P0>SXj06UYJ@25QHD0ADE50451@@1FEUH0A4A407QhN01aLG40
=3@d05iNGP1TI6@0GEeM0:b/[01bLW80=3@d069RHP1MGEd0B4Q80;6a/@1;Bd/0:b/[06UYJ@27QhL0
IVIV08b<S00n?Sh0?Cdm05mOG`1HF5P0Hf=S0;fm_@1IFET0>SXj05eMG@1XJ6P0JVYZ0;^k^`1NGUh0
A4A406MWI`1@D500WYjN08V9R@0R8R80Ng]k04m?C`1KFe/0QXJ604A4A010@400QXJ6065QH@24Q8@0
>SXj03/k>`1MGEd0NGUi06=SH`1EEED0;bl_04i>CP2RXZ80KVi^05YJFP18B4P0O7al0:j^[P21PH40
GemO08:2PP0e=CD0GUiN0861P@0[:b/0E5AD03De=@0];Bd0J6QX0451@@0c<c<0CTi>034a<@1^KVh0
ZJVY07EeM@0n?Sh0MGEe04Q8B01gMgL0`/;205MGE`0m?Cd0IfMW07emO@2NWYh0Y:BT04M7A`14A4@0
GemO0820P01oOgl0ZZZZ04E5A@0S8b<0O7al06MWI`2UYJD0ZJVY03<c<`1?Cdl0Q8B406m_K`1hN7P0
GEeM03lo?`1?Cdl0I6AT06i^KP1hN7P0EUIF01lO7`1=CDd0JFUY09bLW01LG5`0Ph>3038b<P18B4P0
LW9b09ZJVP1DE5@0QhN702LW9`12@T803_ooool0AP1HF5P0MWIf04m?C`1LG5`0EEEE06MWI`1]KFd0
Bd];07QhN01PH600A4A406UYJ@1ZJVX0C4a<06i^KP1GEeL0P82004e=C@1cLg<0Bd];04Y:BP0k>c/0
A4A404I6AP1ADE40Cdm?03Hf=P13@d<0>c/k04U9B@0o?cl0JVYZ05=CD`1aLG40De=C07EeM@1HF5P0
A4A404m?C`1:BTX0?3`l04Y:BP12@T80@D5102/[:`10@40051@D05IFEP0Y:BT0>CTi04Y:BP1LG5`0
:b/[03@d=00g=cL0<C4a03`l?00k>c/0De=C06ATI01QHF40De=C05MGE`1`L700K6a/071`L00S8b<0
DU9B05UIF@0m?Cd204M7A`0H04a<C018B4P092@T038b<P1LG5`0H61P03@d=013@d<0;bl_04i>CP0i
>CT0EeMG02d];@0i>CT0=CDe03`l?017AdL0C4a<07alO016ATH0@D5103De=@10@400B4Q80P11@D40
6`1VIVH0@D5104Y:BP1=CDd0@D5104Q8B01nOWh0TY:B05ADE01QHF40G5aL06=SH`2VYZH0X:2P05=C
D`2?Shl0BTY:04m?C`1=CDd0L71`04a<C01MGEd0ATI606EUI@1@D500P82009nOW`0207UiN@1F0;:b
/P1TI6@0Q8B407IfMP2l_;`0QHF50410@01>CTh0F5QH02l_;`0k>c/0<S8b04E5A@1:BTX0>3Ph065Q
H@0h>3P0GUiN04e=C@1CDe<0CTi>06ATI00a<C40F5QH05ADE018B4P0Jf][07moO`1EEED0ZJVY03`l
?015ADD0EEEE04E5A@1MGEd0Fe]K02HV9P1eMGD0I6AT04E5A@1aLG40JFUY09BDU01@D500@410065Q
H@1FEUH0Cdm?0861P@1ADE40KFe]09BDU01bLW80Ti>C03lo?`16ATH07AdM07]kN`1]KFd0:2PX06e]
K@0k>c/0Fe]K03Lg=`0n?Sh0Cdm?0410@01?Cdl0E5AD0492@P1WIfL0B4Q807AdM01OGel0@T9204m?
C`0Y:BT0CTi>03dm?@0a<C40F5QH03/k>`12@T80D51@04I6AP1@D50205iNGP0304I6AP19BDT0BDU9
0080@T9203X0Kfm_05IFEP12@T80EeMG06a/K00c<c<0DU9B03dm?@0h>3P0=cLg05UIF@16ATH0BDU9
06ATI01;Bd/0A4A4065QH@1BDU80>3Ph04A4A01DE5@0@41004U9B@1SHf<0QHF504m?C`1TI6@0KVi^
0410@00l?3`0GemO09>CT`2KVi/0I6AT07MgM`1CDe<0M7Ad04M7A`1[Jf/0ATI605QHF017AdL0>3Ph
03Hf=P0k>c/0IFEU03Ph>015ADD0>c/k06ATI019BDT0I6AT05IFEP1JFUX0F5QH06QXJ01[Jf/0EUIF
4?ooool000kooooo0?l0G5aL08V9R@1nOWh0W9bL03`l?01NGUh0MgMg06EUI@2l_;`0?Shn05=CD`1M
GEd0DU9B08R8R01jNWX0;2`/04U9B@2BTY80EUIF09ZJVP1=CDd0;Bd]0410@01FEUH0C4a<07UiN@0j
>SX0=SHf06][J`1TI6@0B4Q805ADE00a<C40;Rh^06QXJ01cLg<0=cLg02<S8`0O7al0>3Ph07MgM`2E
UID0>c/k03lo?`0W9bL0Hf=S08V9R@0b<S807QhN04Y:BP1HF5P0RHV907=cL`0N7Qh0=cLg04Y:BP1b
LW80W9bL02HV9P0N7Qh0ATI605QHF01lO7`0Ph>3024Q8@0[:b/0D51@06][J`1/K6`0T92@024Q8@19
BDT0N7Qh069RHP1/K6`0NWYj028R8P0d=3@0Kfm_051@D02MWId0IVIV02HV9P1@D500PH61059BDP2J
VYX0<30`02LW9`1SHf<0>3Ph04Q8B019BDT0;Bd]02/[:`1CDe<0<C4a06UYJ@0N7Qh0<C4a04i>CP1P
H600CTi>07emO@0I6AT0;Bd]07IfMP1XJ6P0?3`l04E5A@0d=3@0De=C05EEE@1SHf<09BDU03lo?`1?
Cdl0AdM708N7Q`1bLW8071`L03@d=011@D40DU9B09NGU`1NGUh0:BTY05MGE`2<S8`0NWYj0=cLg00m
?Cd0ADE507]kN`2@T900Kfm_08J6QP0g=cL0D51@07UiN@22PX80RHV908>3P`0e=CD0G5aL07YjNP1R
HV80OGem06UYJ@1:BTX0NGUi06YZJP18B4P0ZZZZ0;Fe]@0_;bl0RHV908J6QP1/K6`0[:b/0;jn_P0i
>CT0IFEU0:6QX@1:BTX0QhN706e]K@0a<C40Fe]K04a<C01=CDd0RHV9034a<@0g=cL0FEUI06i^KP1A
DE40IVIV02`/;016ATH0NGUi09FEU@19BDT0;bl_03lo?`1fMWH0Vi^K04e=C@19BDT0?Shn05MGE`2o
_kl0NGUi030`<015ADD0DU9B08^;R`2OWil0;bl_051@D01XJ6P0Lg=c07QhN00T92@0?clo05aLG01[
Jf/0GUiN07UiN@0];Bd0NWYj07]kN`1gMgL0^KVi02LW9`1IFET0IVIV04Q8B02WYjL0C4a<01hN7P1Z
JVX0KVi^06EUI@2LW9`0:b/[02XZ:P1@D500VYZJ07emO@2@T900=CDe04e=C@1:BTX0M7Ad096AT@1^
KVh0<30`06UYJ@1VIVH0GUiN05UIF@1VIVH06a/K04U9B@31`L40[:b/04i>CP1?Cdl08B4Q05aLG02Z
ZZX0DE5A03Hf=P13@d<001L0;2`/065QH@2=SHd0>3Ph03<c<`1@D500>SXj06IVIP25QHD0=cLg030`
<01@D500DU9B06=SH`2ZZZX0@T92020P800b<S80FUYJ04I6AP2ZZZX0;2`/03/k>`0>ooooo`0606=S
H`1QHF40G5aL05YJFP1PH600KFe]0P1[Jf/206m_K`0805UIF@1dM7@0O7al06UYJ@2<S8`0WIfM07If
MP1<C4`206MWI`2o03Xj>P1@D500EeMG055AD@14A4@0KVi^06ATI01mOGd0Kfm_051@D01HF5P0HF5Q
05iNGP1LG5`0ATI606i^KP1GEeL0@T9204m?C`0i>CT0EEEE05eMG@15ADD0DU9B03/k>`1;Bd/071`L
04M7A`0`<300>CTi02TY:@1CDe<0?Cdm04];B`1RHV80BDU905MGE`13@d<0FEUI05MGE`1ADE40GUiN
04e=C@15ADD0Ng]k07QhN00V9RH0<S8b03lo?`1MGEd0LW9b06ATI01JFUX0KVi^03De=@12@T80BTY:
04M7A`15ADD0DU9B0492@P1BDU80<C4a05]KF`0Y:BT0?Shn02h^;P1<C4`0A4A40492@P1dM7@0<30`
04E5A@0^;Rh0ATI603lo?`1=CDd0=SHf05UIF@16ATH0A4A404E5A@0j>SX0@410059BDP1WIfL0@d=3
051@D01=CDd0DU9B05=CD`1ZJVX0IFEU09nOW`17AdL0Lg=c04];B`1hN7P0=3@d071`L01@D500TI6A
06UYJ@1^KVh0`l?307IfMP25QHD0ZZZZ05mOG`21PH40C4a<071`L01MGEd0>3Ph04I6AP12@T80=3@d
04i>CP0b<S80ATI603De=@19BDT0Lg=c05YJFP2^[Zh0Q8B4055AD@0j>SX0CDe=02<S8`1TI6@0MgMg
051@D01<C4`0I6AT06=SH`2^[Zh0@D5106IVIP1CDe<0Cdm?03Ph>02:RXX0?Cdm04Y:BP1@D500GUiN
09BDU01>CTh0BTY:04U9B@1^KVh0NWYj05iNGP1SHf<0E5AD05aLG01`L700VIVI08:2PP2EUID0EUIF
06UYJ@0Q8B40FEUI05MGE`0L71`0VIVI02`/;01dM7@0>SXj05]KF`1PH600BDU905UIF@1RHV80CTi>
05QHF00k>c/0UIFE07UiN@1:BTX0HV9R02/[:`1GEeL0@41003Xj>P1:BTX0De=C04m?C`0g=cL00P1:
BTX0@@1aLG40>CTi04Y:BP0c<c<0Fe]K06=SH`16ATH0Fe]K07EeM@18B4P0=3@d04A4A01=CDd0MGEe
04i>CP0n?Sh0C4a<05IFEP1<C4`0BDU9055AD@1GEeL0E5AD0::RXP28R8P0DE5A05aLG019BDT0AdM7
03Hf=P1@D500WIfM04A4A01cLg<0JFUY06m_K`1<C4`0G5aL0:2PX02IVIT0DU9B04e=C@1@D500E5AD
03De=@1dM7@0GUiN04Q8B01/K6`0ADE504M7A`0o?cl0K6a/05YJFP1?Cdl0=cLg04Q8B019BDT0HF5Q
03lo?`14A4@0FUYJ05UIF@1UIFD0DU9B013ooooo000>ooooo`0@09>CT`1gMgL0BTY:07EeM@1CDe<0
@d=304M7A`0n?Sh0L71`05eMG@14A4@0Bd];02HV9P1TI6@0Vi^K0410@080=3@d06X0DE5A051@D019
BDT0:BTY04=3@`0g=cL0Lg=c0:RXZ01@D5008b<S0410@01;Bd/0PX:207QhN00W9bL0?Cdm04Q8B020
P800MWIf07YjNP0I6AT0BDU904E5A@1ZJVX0F5QH07QhN00/;2`0@T9204A4A01XJ6P0GEeM069RHP0n
?Sh0CTi>055AD@0i>CT0M7Ad065QH@0R8R80O7al05ADE01FEUH0Ng]k02LW9`0e=CD0SHf=05ADE01=
CDd0Bd];02<S8`10@400TY:B065QH@21PH40@d=302@T901IFET0/k>c04a<C01FEUH0@d=303De=@1n
OWh0YjNW03Hf=P1[Jf/0=CDe05QHF02?Shl0Kfm_01lO7`1MGEd0>c/k08j>SP1bLW808b<S02l_;`0m
?Cd0F5QH0;Jf]P0c<c<061PH02h^;P1GEeL0HV9R0;fm_@0X:2P08b<S03hn?P1EEED0I6AT09NGU`0/
;2`0>CTi069RHP1gMgL0JFUY079bLP0[:b/0;bl_06][J`26QXH0E5AD09:BTP1TI6@0=CDe0P2=SHd0
K@1]KFd0W9bL07MgM`12@T80bLW908f=S@1BDU80/k>c065QH@14A4@0Jf][05YJFP1QHF40EeMG03`l
?01OGel0MWIf03Ti>@1lO7`0IVIV03Hf=P1;Bd/0PH6103/k>`1^KVh0ADE502d];@1UIFD0][Jf04=3
@`1OGel0De=C055AD@1jNWX0ZZZZ04E5A@1?Cdl0ADE503Ti>@2JVYX0^[Zj03`l?01?Cdl0AdM706][
J`2>SXh0A4A402XZ:P14A4@0Jf][06UYJ@1aLG40:2PX02XZ:P1TI6@0TY:B06i^KP0Q8B409BDU04m?
C`1^KVh0@T9208j>SP0N7Qh0>3Ph075aL@2QXJ40LG5a079bLP0g=cL0G5aL0;6a/@1_Kfl0Ti>C05MG
E`17AdL0Shn?04];B`1cLg<0=SHf02h^;P1KFe/0MgMg05EEE@1>CTh0>SXj04A4A01/K6`0Lg=c04e=
C@0^;Rh0=CDe071`L024Q8@0=3@d05=CD`0j>SX0FEUI07inOP1cLg<0<c<c08>3P`1DE5@0SXj>0:6Q
X@1EEED0>c/k06i^KP1;Bd/00P2^[Zh0:@0d=3@0AdM706QXJ01JFUX0N7Qh0;Jf]P0g=cL0;Rh^05UI
F@1nOWh0KVi^09RHV00a<C40@T9207EeM@2GUiL0PH6108B4Q012@T80E5AD06e]K@1[Jf/0QXJ609NG
U`0d=3@0DE5A05eMG@1_Kfl0HF5Q0;6a/@1BDU80>CTi08:2PP1^KVh0D51@06e]K@1^KVh0;bl_04a<
C01fMWH0>c/k00kooooo00<0GemO07AdM01<C4`00P1GEeL0201LG5`0FUYJ04m?C`1MGEd0Cdm?03dm
?@1PH600MgMg0P1_Kfl03@1;Bd/0PX:2079bLP1cLg<0@D5103Ti>@1KFe/0@T9204Y:BP1RHV80DU9B
03Xj>P1EEED00P1_Kfl01`19BDT0IVIV06e]K@1YJFT0DE5A06YZJP1_Kfl00P1EEED03P0Y:BT0?Cdm
055AD@1@D500DU9B03Xj>P11@D407AdM04m?C`15ADD0BDU9038b<P19BDT0=CDe0P1ADE408P1PH600
G5aL06m_K`1GEeL0PH61069RHP1XJ6P0HF5Q06IVIP2/[:`0S8b<02<S8`11@D40EEEE07EeM@1/K6`0
AdM704I6AP1NGUh0E5AD05iNGP1ZJVX0EEEE04];B`1RHV80>3Ph0492@P0i>CT0>c/k03<c<`19BDT0
=SHf0410@019BDT204];B`0G03lo?`1RHV80<C4a03lo?`14A4@0Hf=S03<c<`1EEED0IFEU05QHF00j
>SX0?Shn04M7A`1UIFD0MgMg059BDP1RHV80AdM704];B`1QHF40Jf][04M7A`1TI6@00P0n?Sh0601I
FET0EUIF02/[:`1@D500BTY:06ATI01gMgL0EUIF0;Zj^P0n?Sh0Hf=S08^;R`1@D500P82005ADE02E
UID0De=C04U9B@1JFUX0?Shn0410@01[Jf/0?Cdm07IfMP80BDU908P0JVYZ04M7A`2PX:00JFUY071`
L01OGel0Fe]K03Lg=`2HV9P0Y:BT04A4A01CDe<0EUIF05eMG@2`/;00:BTY05ADE011@D40>3Ph04Q8
B01IFET0HF5Q05iNGP1/K6`0AdM705]KF`1:BTX0JVYZ04M7A`0m?Cd0BTY:04A4A010@400L71`04];
B`1JFUX0Lg=c09:BTP35aLD0LG5a03/k>`0e=CD0[Jf]08>3P`0k>c/0JVYZ059BDP2XZ:P0BTY:03`l
?016ATH0JVYZ07IfMP1EEED0FEUI09^KV`1<C4`0Kfm_04U9B@2CTi<0K6a/03`l?01?Cdl0A4A4038b
<P1;Bd/0?Shn05eMG@12@T80?3`l03dm?@1VIVH0ADE504U9B@0l?3`0BTY:04U9B@1BDU80Rh^;04E5
A@1:BTX0@d=306m_K`0j>SX0FUYJ03Hf=P0g=cL0>c/k04i>CP0f=SH0@T92069RHP1?Cdl0?Cdm05=C
D`19BDT0@41007MgM`0h>3P0NGUi03`l?011@D40M7Ad01dM7@12@T80E5AD05eMG@1cLg<0N7Qh07]k
N`1gMgL0EeMG05eMG@1UIFD0N7Qh059BDP2XZ:P0KFe]05ADE01ZJVX0E5AD030`<00f=SH0E5AD03Ti
>@16ATH0=cLg04I6AP1;Bd/0KVi^04];B`1<C4`0D51@06=SH`1fMWH0JFUY4?ooool000kooooo03d0
LW9b0<?3``1EEED0IfMW07MgM`2OWil0GUiN03hn?P0i>CT0IVIV0;Jf]P1SHf<0;Bd]02TY:@1IFET0
AdM706a/K00N7Qh0<c<c059BDP1JFUX0@41008B4Q00k>c/08b<S06ATI025QHD0F5QH08^;R`0a<C40
CDe=08:2PP1mOGd0FUYJ06IVIP0Y:BT0FEUI07YjNP1BDU80GUiN06IVIP0S8b<0<30`05aLG00`<300
?clo04M7A`0/;2`0ATI607AdM00d=3@0EUIF03Ph>00`<300NGUi08:2PP0O7al0B4Q803<c<`1XJ6P0
WinO0080:b/[0280>CTi06][J`1fMWH0M7Ad01dM7@0Z:RX0EeMG06i^KP2ZZZX0FEUI02TY:@0e=CD0
EEEE07IfMP2>SXh0?Cdm03Ph>01?Cdl0O7al07]kN`27QhL0>3Ph03<c<`1dM7@0SXj>06=SH`1iNGT0
820P04=3@`2DU9@0FEUI08N7Q`1EEED0<30`0P1PH600@`14A4@0MgMg04i>CP1;Bd/0KVi^096AT@0m
?Cd0IFEU038b<P0S8b<0FEUI05aLG00`<300LG5a03Ph>00f=SH0Hf=S03dm?@1?Cdl0T92@04Q8B011
@D40FEUI09NGU`0l?3`0K6a/04=3@`10@400TY:B0:BTY00`<300JVYZ04a<C015ADD0VIVI06][J`0c
<c<0MWIf05mOG`1nOWh0Rh^;02TY:@1DE5@0QhN7065QH@2SXj<0RXZ:02h^;P1KFe/0O7al065QH@3=
cLd0A4A404E5A@16ATH0L71`07alO02=SHd0EUIF02d];@1eMGD0TY:B03lo?`2KVi/0<30`03Ti>@1?
Cdl00P2KVi/0CP1PH6007QhN061PH01bLW80NWYj04];B`2=SHd0@41006UYJ@1NGUh0Cdm?06QXJ026
QXH0>c/k04m?C`1dM7@0?Cdm07moO`1DE5@0;Bd]061PH00g=cL0C4a<05YJFP1;Bd/0AdM705MGE`1:
BTX0Jf][04E5A@0`<300RXZ:096AT@17AdL0IFEU05=CD`2?Shl0O7al02LW9`15ADD0GemO07alO02C
Ti<0;bl_03Lg=`1:BTX0H61P08R8R00m?Cd0<C4a04I6AP1BDU80F5QH08:2PP0O7al0Bd];06][J`2A
TI40H61P0:ZZZP18B4P0FEUI0861P@1eMGD0Hf=S09BDU00i>CT0Lg=c0:FUY@1[Jf/0NGUi08J6QP0f
=SH0Bd];0;Jf]P2[Zj/0RXZ:0::RXP80<S8b01h0Ng]k055AD@1XJ6P0Ng]k01dM7@18B4P0Q8B403Ph
>01ADE40G5aL024Q8@1?Cdl0Shn?04Y:BP1QHF40@d=302h^;P1<C4`0I6AT05UIF@0j>SX0DE5A03hn
?P1FEUH0Jf][07inOP0`<300HV9R05IFEP1CDe<>ooooo`3Z05UIF@1mOGd0DE5A06MWI`1HF5P0OGem
06i^KP1CDe<0PX:205aLG015ADD0PX:208>3P`1TI6@0SXj>0861P@25QHD0@d=306=SH`17AdL0?clo
0451@@1?Cdl0AdM704U9B@1TI6@0?clo055AD@1?Cdl0D51@0492@P1SHf<0DE5A061PH01:BTX0O7al
05UIF@0m?Cd0Bd];059BDP0m?Cd0Cdm?04];B`19BDT0=SHf04E5A@0];Bd0FEUI02`/;011@D40Cdm?
05YJFP0c<c<0?3`l04E5A@1@D500?Cdm04I6AP1VIVH0EeMG05EEE@1HF5P0EEEE05UIF@1UIFD0GemO
04U9B@1KFe/0GUiN05=CD`1]KFd0BDU904E5A@1FEUH0:RXZ0410@01mOGd0I6AT03/k>`1BDU80?Shn
051@D00g=cL0E5AD03Lg=`15ADD0;Bd]05=CD`0m?Cd0CDe=06m_K`0e=CD0D51@030`<01FEUH0C4a<
04=3@`0l?3`0FEUI04U9B@1:BTX0D51@055AD@1?Cdl0V9RH08F5Q@1;Bd/0I6AT05IFEP1EEED0GemO
05UIF@17AdL0O7al04Q8B01KFe/0?Cdm069RHP0/;2`0EUIF05mOG`1WIfL0DE5A04=3@`34a<@08R8R
05iNGP1MGEd0:BTY05EEE@1DE5@0Zj^[055AD@0g=cL0@T9204a<C00S8b<0ATI602d];@19BDT0CTi>
05iNGP1VIVH0<S8b0:6QX@0g=cL0B4Q803Lg=`1MGEd0?Cdm04Q8B01?Cdl0De=C04Y:BP1ADE40MgMg
0<;2`P0Z:RX0OWin07UiN@1RHV80V9RH06a/K00o?cl0L71`061PH00i>CT0H61P04I6AP1gMgL0<S8b
03/k>`1FEUH0@41004m?C`1DE5@0GEeM071`L02ATI40Lg=c07]kN`1JFUX0A4A402l_;`19BDT0GUiN
04];B`1kNg/0BDU9075aL@0f=SH0CTi>03dm?@0j>SX0@D5106MWI`1>CTh0BTY:05IFEP2o_kl0UiNG
07QhN01LG5`09bLW04];B`0d=3@0>SXj05ADE017AdL0FUYJ04a<C016ATH0@T92071`L011@D40C4a<
0492@P1ZJVX0ATI603lo?`1>CTh0EUIF03dm?@0h>3P0Hf=S03dm?@1PH600FEUI03Xj>P0f=SH0FEUI
04=3@`12@T80Fe]K0P0j>SX08`1nOWh0F5QH02`/;00];Bd07alO04i>CP0c<c<0Jf][08j>SP12@T80
F5QH055AD@1SHf<0CDe=051@D02k^k/0UYJF04Y:BP1ADE40L71`06MWI`2?Shl0bLW907QhN01`L700
Fe]K04i>CP0k>c/0EEEE06][J`0c<c<0JVYZ04e=C@1BDU80@D510080F5QH00D0De=C05aLG01BDU80
GEeM06QXJ00@ooooo`003_ooool0o`1HF5P0MgMg08^;R`1]KFd0BTY:05=CD`25QHD0B4Q806EUI@0/
;2`0BTY:06e]K@0c<c<0CDe=04Y:BP0^;Rh0GUiN04=3@`0l?3`0Cdm?038b<P1>CTh0J6QX06=SH`0Y
:BT0C4a<055AD@1CDe<0RHV903hn?P15ADD0AdM703hn?P1XJ6P0Y:BT03Hf=P0^;Rh0IfMW0492@P1[
Jf/0Ti>C05eMG@0J6QX0GEeM069RHP1cLg<0OWin01XJ6P0Z:RX0E5AD061PH02b/[80<S8b02XZ:P0h
>3P0G5aL07=cL`1@D5008B4Q03dm?@1`L700MGEe07alO00n?Sh061PH05YJFP2DU9@0IfMW04A4A015
ADD0CTi>075aL@2;Rh/0@41004=3@`18B4P0:BTY04Y:BP1<C4`0;Rh^05YJFP14A4@0>3Ph065QH@1B
DU80=CDe07YjNP0];Bd0=3@d05ADE00K6a/0>CTi0492@P0];Bd0E5AD079bLP0V9RH0?clo0451@@1?
Cdl0H61P04E5A@0f=SH0DE5A04m?C`1aLG40^KVi06=SH`0K6a/0@41003lo?`1?Cdl0_;bl05eMG@0N
7Qh0DE5A04=3@`1NGUh0/[:b061PH00c<c<0>3Ph07AdM01TI6@0bl_;04i>CP13@d<0Cdm?08b<S02@
T900UIFE03Lg=`1HF5P0IfMW0;>c/`1aLG40IFEU03Ti>@1ADE40HV9R05YJFP1=CDd0G5aL03<c<`1A
DE40MGEe051@D01jNWX0EUIF04M7A`1bLW80OWin04E5A@1iNGT0FUYJ04i>CP1dM7@0N7Qh03hn?P1b
LW80Q8B4051@D021PH40Jf][03<c<`1jNWX0M7Ad03`l?02a/K40ADE503dm?@1bLW80IfMW059BDP2S
Xj<0HF5Q03dm?@1JFUX0?3`l08J6QP3;bl/0E5AD03<c<`1NGUh0EEEE0<W9b@1mOGd0<C4a05=CD`1Y
JFT0YZJV0=OGe`1KFe/0;2`/04U9B@1QHF40alO707UiN@0`<300>CTi06IVIP29RHT0QhN702HV9P1_
Kfl0_[jn08R8R022PX80F5QH07EeM@2^[Zh0M7Ad06a/K01eMGD0:2PX06UYJ@1oOgl0DE5A09fMW@1C
De<0=CDe06MWI`1EEED0Fe]K07moO`0^;Rh0GEeM092@T01SHf<0?clo06][J`0o?cl0HV9R07UiN@19
BDT0EeMG06UYJ@1KFe/0FUYJ07emO@1=CDd0ADE5069RHP0n?Sh0H61P0;^k^`1BDU80=3@d059BDP1X
J6P0OGem08f=S@0Q8B40<C4a05aLG0005`1fMWH0Ng]k08^;R`0S8b<0=3@d05YJFP1QHF40QXJ607=c
L`0M7Ad0Bd];079bLP1ZJVX0L71`0:BTY00b<S809bLW069RHP2DU9@0Kfm_0;Ng]`0c<c<0=3@d00ko
oooo01`0EeMG0820P01[Jf/0KVi^065QH@1nOWh0KVi^05eMG@27QhL0HV9R05ADE01nOWh0QXJ607Ad
M01aLG40E5AD06YZJP1TI6@0Lg=c04Q8B017AdL0CTi>055AD@1KFe/0GEeM07EeM@1<C4`0GemO0P1W
IfL0_P1BDU80O7al071`L01/K6`0IVIV07moO`1RHV80BDU905EEE@1IFET0ADE505UIF@1:BTX0De=C
03Lg=`15ADD0;Rh^065QH@10@400BTY:03`l?01IFET0<c<c0492@P1QHF40G5aL05MGE`1EEED0HV9R
06YZJP1HF5P0HF5Q055AD@1aLG40RXZ:08f=S@11@D40>3Ph05YJFP1SHf<0FUYJ051@D01WIfL0Fe]K
03<c<`0o?cl0DE5A06MWI`1:BTX0HV9R04E5A@1@D500>3Ph06EUI@0_;bl0B4Q802d];@18B4P0>3Ph
04];B`1IFET0:RXZ051@D00P8200=CDe03Ti>@1CDe<0:2PX04Q8B01BDU80?clo04U9B@0o?cl0?Cdm
05YJFP1LG5`0?clo05YJFP1=CDd0D51@05EEE@1MGEd0HF5Q07moO`1:BTX0C4a<04e=C@1gMgL0>SXj
05mOG`12@T80F5QH08F5Q@1:BTX0W9bL02@T901gMgL0Ogmo04E5A@23Ph<0>SXj09JFUP1ADE40GUiN
04i>CP0n?Sh0<30`04U9B@0n?Sh0IfMW034a<@0j>SX0FUYJ03hn?P2WYjL0AdM707EeM@1HF5P0EeMG
03/k>`1bLW80M7Ad04m?C`1RHV80IVIV06][J`2:RXX0Cdm?075aL@18B4P0AdM70492@P1RHV807AdM
05=CD`1@D500BTY:06MWI`13@d<0EEEE03lo?`1iNGT0IfMW04];B`1EEED0Kfm_05iNGP21PH40Ti>C
08V9R@2EUID0Bd];04m?C`0n?Sh0JVYZ061PH01KFe/0Xj>S06QXJ02<S8`0CTi>06=SH`19BDT0FEUI
05ADE01=CDd0AdM707YjNP1ADE40Hf=S06IVIP1;Bd/0D51@02DU9@1_Kfl0D51@04i>CP1FEUH0F5QH
06UYJ@0l?3`0B4Q804M7A`1aLG40@D5104U9B@0[:b/0@T9204I6AP15ADD2069RHP0Y04=3@`0o?cl0
Cdm?04Y:BP1SHf<0<C4a02`/;00[:b/0@D5102l_;`0c<c<0EUIF05ADE00c<c<0GemO04i>CP12@T80
CTi>03/k>`1iNGT0Bd];04M7A`1QHF408B4Q05IFEP1NGUh0JVYZ05UIF@1PH600L71`08:2PP1ZJVX0
IVIV079bLP2=SHd0L71`08:2PP0h>3P0?clo03Ph>016ATH00P0h>3P03@1:BTX0>c/k055AD@0i>CT0
Bd];061PH01lO7`0FUYJ04U9B@1ADE40EUIF06IVIP1LG5`04?ooool000kooooo02P0FUYJ05IFEP1c
Lg<0RHV903dm?@0o?cl0LW9b06YZJP2GUiL0>3Ph0410@01?Cdl0>c/k0:2PX02GUiL07AdM04Q8B01@
D500Ng]k09BDU00K6a/0AdM705ADE01]KFd0MGEe04a<C00V9RH0?Shn06IVIP1cLg<0Rh^;04E5A@0c
<c<0@T9207UiN@2MWId0JVYZ04M7A`0N7Qh0B4Q80P1`L7005P1ADE40F5QH01dM7@16ATH0De=C03`l
?00c<c<0<S8b024Q8@1eMGD0IfMW03@d=01GEeL0=CDe03<c<`1nOWh0:RXZ03De=@17AdL0;Rh^06i^
KP1hN7P201/K6`3D04e=C@1NGUh0IFEU06=SH`0V9RH0<C4a061PH01DE5@0NGUi0820P00W9bL0Bd];
05EEE@0c<c<0SHf=06a/K00^;Rh0BDU905iNGP1<C4`0`L7102`/;00^;Rh0F5QH059BDP1_Kfl0?clo
01hN7P0a<C40LG5a05MGE`1gMgL07QhN02`/;019BDT0H61P06=SH`2k^k/09RHV02/[:`1<C4`0Ph>3
04M7A`1bLW807alO02XZ:P1KFe/0MWIf030`<01ADE408R8R024Q8@1NGUh0Lg=c04=3@`1GEeL0A4A4
03<c<`1MGEd0JFUY03Xj>P1LG5`0ATI6051@D02TY:@0E5AD03/k>`18B4P0CDe=05EEE@1]KFd0:b/[
04U9B@1MGEd0@d=30820P01hN7P09bLW05mOG`1DE5@0EeMG09nOW`1/K6`0CTi>06][J`1_Kfl0DU9B
0;Jf]P28R8P0@d=306m_K`1^KVh0EUIF0=CDe02YZJT0;Bd]04m?C`1^KVh0DU9B0;:b/P2MWId0DU9B
06UYJ@1TI6@0Ti>C08^;R`0f=SH0>c/k06YZJP1bLW80GemO05MGE`0[:b/0@T920:2PX028R8P0HV9R
02@T900/;2`0Shn?09jNWP1BDU80EEEE03Hf=P19BDT0^[Zj0;Rh^01aLG40Cdm?03<c<`1gMgL0[Zj^
05EEE@2;Rh/0=cLg05]KF`2h^;P0CTi>065QH@0l?3`0BDU907alO01;Bd/0E5AD07UiN@0l?3`0OWin
0:f][@0m?Cd0A4A405ADE017AdL0Vi^K0451@@0[:b/0EEEE06IVIP1jNWX0Ti>C038b<P0_;bl0GemO
079bLP1jNWX0U9BD03Ti>@0g=cL0Ph>306QXJ03?cll0VIVI0492@P1KFe/0FEUI075aL@2OWil0SXj>
04];B`1=CDd0H61P08J6QP20P800LG5a030`<01DE5@0IVIV06e]K@1GEeL0GemO034a<@12@T80FUYJ
04E5A@19BDT0QHF503Lg=`1;Bd/0MGEe04Q8B019BDT0HF5Q03@d=010@400GUiN061PH00b<S80BTY:
04A4A00o?cl0OGem04m?C`0[:b/>ooooo`0Y05ADE01dM7@0DU9B05]KF`1RHV80MgMg05]KF`1CDe<0
J6QX05EEE@0j>SX0I6AT06e]K@1EEED0LG5a04i>CP1^KVh0ADE505QHF013@d<0>3Ph04U9B@1KFe/0
GemO055AD@1XJ6P0IfMW06YZJP1PH600Hf=S03`l?01=CDd0De=C05aLG017AdL0F5QH04m?C`0m?Cd0
FUYJ04U9B@11@D400P1CDe<03`1NGUh0@41004=3@`0V9RH0F5QH030`<011@D40A4A404i>CP0l?3`0
Cdm?05YJFP1]KFd0D51@04E5A@0206i^KP1l05mOG`1JFUX0H61P07MgM`2[Zj/0RHV903@d=014A4@0
HV9R07MgM`1CDe<0B4Q8059BDP1[Jf/0B4Q805YJFP1VIVH0Hf=S04a<C01IFET0=CDe055AD@15ADD0
HV9R04M7A`1>CTh0>CTi05ADE01;Bd/0EeMG07MgM`11@D40EUIF03/k>`1>CTh0CDe=06MWI`17AdL0
Fe]K05=CD`1OGel0CDe=051@D01FEUH0VIVI08F5Q@1ADE40I6AT04I6AP1>CTh0Hf=S03Lg=`0n?Sh0
Ogmo04i>CP15ADD0D51@05eMG@0f=SH0EeMG05YJFP1YJFT0G5aL059BDP32`/80=3@d065QH@15ADD0
;bl_05mOG`13@d<0UiNG05iNGP1?Cdl0Fe]K0451@@0b<S80I6AT02PX:01CDe<0C4a<04m?C`1FEUH0
ATI60;>c/`0R8R80GUiN03@d=014A4@0GUiN0:6QX@2LW9`0GEeM08B4Q01cLg<0Rh^;0;Ng]`0n?Sh0
Xj>S04E5A@18B4P0BTY:06e]K@1@D500RXZ:06EUI@0m?Cd0@T9204];B`2;Rh/0C4a<04];B`1=CDd0
AdM704a<C02FUYH0J6QX08>3P`2OWil0[jn_09>CT`1NGUh0Fe]K04];B`80JFUY01l0DU9B09JFUP2C
Ti<0`l?304A4A015ADD0:2PX04Y:BP14A4@0BDU904Q8B01bLW80@T9208^;R`1kNg/0VIVI07EeM@0n
?Sh0@d=303hn?P11@D40C4a<0451@@15ADD0<30`03Xj>P0`<300F5QH03De=@13@d<0=3@d0080CTi>
01<0FEUI07EeM@12@T80;bl_04=3@`1UIFD0;Bd]06QXJ00W9bL092@T030`<019BDT0@41004I6AP1i
NGT0ATI604=3@`2ATI40LG5a0080Hf=S02H0;2`/061PH00V9RH0IFEU0:f][@0Y:BT0D51@05YJFP20
P800GUiN059BDP2o_kl0RHV907=cL`1UIFD0JFUY05EEE@28R8P0Z:RX03@d=01EEED0BTY:04e=C@0X
:2P0AdM705YJFP1QHF40RXZ:05=CD`14A4@0C4a<05aLG00n?Sh0@d=3051@D01[Jf/0MgMg05IFEQ3o
oooo000>ooooo`3L07emO@2[Zj/0H61P07AdM01PH600P82004I6AP13@d<0IFEU07AdM02KVi/0GUiN
02PX:01EEED0M7Ad03dm?@1JFUX08b<S03@d=01QHF40De=C03Lg=`29RHT0>CTi05MGE`25QHD0Jf][
04=3@`1nOWh0>SXj092@T02;Rh/0G5aL05=CD`1TI6@0DE5A05mOG`1bLW80@T9204a<C01^KVh0=SHf
03@d=01LG5`0;bl_03hn?P1>CTh0;Bd]04Y:BP1bLW80;Rh^05EEE@13@d<0>SXj06UYJ@1>CTh0>3Ph
059BDP0g=cL0GUiN08:2PP0b<S80@T9204a<C016ATH0F5QH07UiN@0k>c/0=3@d04m?C`1@D500OGem
06UYJ@0b<S80@d=304];B`1CDe<0ZJVY05aLG00d=3@0EEEE06][J`1HF5P0SHf=028R8P0m?Cd0HV9R
06m_K`1fMWH0Fe]K02h^;P12@T80IfMW05UIF@1NGUh0DE5A03hn?P1TI6@0GEeM03Lg=`1MGEd0?clo
04I6AP22PX80KVi^03dm?@1ZJVX0>c/k055AD@2HV9P0K6a/03Hf=P1JFUX0De=C05IFEP2:RXX0>SXj
034a<@1BDU80ADE503/k>`1SHf<0EUIF03Lg=`1OGel0ATI604U9B@1eMGD0O7al0492@P1IFET0B4Q8
05QHF01mOGd0?Shn04=3@`1DE5@0CDe=05IFEP1jNWX09RHV03Hf=P1^KVh0CDe=0:>SX`1LG5`0:b/[
051@D01TI6@0FUYJ0:RXZ01@D5007alO05aLG026QXH0EeMG09NGU`1:BTX0<c<c05IFEP2CTi<0MgMg
07alO00a<C40;Rh^06][J`2VYZH0JVYZ06ATI00`<300FEUI065QH@19BDT0B4Q808b<S010@400IVIV
05YJFP0e=CD0@41006UYJ@16ATH0KVi^04Y:BP0d=3@0Q8B406IVIP0k>c/0IVIV02h^;P19BDT0MgMg
06QXJ01lO7`0TY:B04A4A01WIfL0IVIV04m?C`29RHT0FUYJ04];B`1QHF40EEEE09NGU`1lO7`0;Bd]
05mOG`1[Jf/0O7al09nOW`0i>CT0=cLg06UYJ@1BDU80M7Ad030`<00[:b/0EEEE07IfMP80KFe]03P0
=cLg05mOG`1VIVH0CTi>08V9R@2[Zj/0=SHf04];B`2CTi<0LW9b06i^KP2SXj<0>c/k065QH@2;Rh/0
J6QX07moO`1kNg/0;2`/04I6AP1hN7P0CDe=09FEU@1WIfL09bLW04m?C`21PH40@d=306YZJP1LG5`0
@41006][J`2NWYh0CDe=08:2PP14A4@0@T9205eMG@2;Rh/0E5AD069RHP1GEeL0?Shn061PH02EUID0
K6a/04=3@`1FEUH0@T9205QHF01mOGd0KVi^0410@01jNWX0EUIF04];B`kooooo01D0Cdm?07QhN01^
KVh0I6AT05YJFP1aLG40I6AT05eMG@1NGUh0De=C05IFEP1]KFd0OGem07MgM`28R8P0QXJ607MgM`1:
BTX0F5QH04Y:BP0o?cl00P18B4P02P1ADE40HF5Q05QHF01:BTX0FEUI04M7A`1HF5P0EEEE06a/K01I
FET2061PH01307alO01OGel0DE5A05iNGP1KFe/0De=C06EUI@1>CTh0ATI604A4A010@400<30`05QH
F010@400@d=303lo?`1?Cdl0=3@d0410@017AdL0GUiN04U9B@14A4@0@D5106MWI`1FEUH0D51@05eM
G@1TI6@0TY:B08F5Q@10@400Bd];06IVIP1KFe/0H61P04a<C01HF5P0FUYJ03/k>`19BDT0JFUY06EU
I@1=CDd0De=C03Lg=`1ADE40@41005QHF00f=SH0?Cdm02DU9@1DE5@0>c/k04];B`1YJFT0<S8b0492
@P0O7al0BTY:03hn?P1@D500>SXj05MGE`19BDT0?3`l05QHF00204a<C00`08V9R@1bLW80Bd];05YJ
FP18B4P0CTi>051@D010@400CTi>071`L012@T80Bd];04=3@`1aLG40AdM706e]K@1JFUX0K6a/08f=
S@1UIFD0alO704m?C`2CTi<0G5aL04];B`1YJFT0EeMG0;6a/@1;Bd/0L71`05YJFP18B4P0>CTi06AT
I00m?Cd0FEUI03`l?01DE5@0Jf][05QHF02UYJD0AdM706QXJ01PH600GemO04];B`1SHf<0IFEU0P1/
K6`0O01SHf<0OGem0:FUY@0n?Sh0GemO02h^;P1NGUh0?Shn05UIF@0i>CT0FUYJ055AD@15ADD0E5AD
059BDP2HV9P0@D51055AD@1LG5`0EEEE065QH@1HF5P0LG5a09^KV`1hN7P0NWYj08B4Q010@400AdM7
034a<@1oOgl0EEEE05MGE`2IVIT0EUIF06UYJ@11@D40G5aL0492@P1@D500@410061PH015ADD0GEeM
01hN7P1eMGD0N7Qh04Q8B00h>3P06a/K04Y:BP0e=CD0:BTY04A4A017AdL0Cdm?02h^;P1<C4`0?3`l
061PH00a<C40@d=303dm?@1JFUX0@d=30451@@1EEED0F5QH02PX:00c<c<0ATI6034a<@1eMGD0AdM7
03hn?P15ADD0D51@0451@@0n?Sh0I6AT04M7A`1@D500QXJ6065QH@12@T80@d=304];B`1RHV80?Shn
051@D01dM7@0>3Ph06e]K@19BDT0FUYJ05aLG013@d<0Rh^;071`L01YJFT0DE5A05eMG@1SHf<0N7Qh
08J6QP0a<C40FUYJ04Y:BP1QHF40?clo04i>CP1IFET0@41006a/K00n?Sh0ADE506IVIP2;Rh/0A4A4
04e=C@1RHV80CTi>07UiN@1FEUH@ooooo`003_ooool0501IFET0JFUY07emO@2LW9`0BDU905IFEP1Z
JVX0IFEU09:BTP0e=CD0?Cdm065QH@13@d<0L71`03Ti>@0^;Rh0@T9204Y:BP1BDU80FUYJ0P0V9RH0
L00d=3@0E5AD05MGE`1?Cdl0;Rh^0410@01XJ6P0>3Ph05ADE00^;Rh0:2PX079bLP1iNGT0@T9204a<
C00b<S806a/K04a<C025QHD0Bd];024Q8@14A4@0FEUI06YZJP1bLW805aLG03De=@1MGEd0QhN70<30
`011@D40820P04Y:BP11@D40NWYj08j>SP0B4Q80;Bd]05iNGP11@D40WinO03/k>`0D51@0>CTi06i^
KP1fMWH0TI6A08:2PP0];Bd0De=C08B4Q01SHf<0^KVi08n?S`0c<c<0I6AT08J6QP1CDe<0SHf=03Hf
=P0n?Sh0GUiN04m?C`1JFUX0PH6102<S8`0n?Sh0Jf][03De=@1ADE40?Cdm01TI6@1IFET0S8b<02`/
;01CDe<08b<S04E5A@1ZJVX0M7Ad02h^;P1BDU80>CTi05YJFP1cLg<0A4A401TI6@0l?3`0LW9b07If
MP1lO7`09bLW02XZ:P18B4P0IfMW07AdM02^[Zh0<C4a024Q8@0k>c/0EUIF07QhN026QXH092@T01/K
6`0n?Sh0O7al07moO`20P800:2PX020P801PH60208:2PP1C038b<P0L71`0DE5A06MWI`1CDe<0MWIf
06UYJ@0c<c<0J6QX09:BTP1CDe<0YjNW05EEE@17AdL0Hf=S0:NWY`1KFe/0NGUi06][J`0o?cl0FUYJ
07QhN00o?cl0PX:205EEE@0_;bl0LG5a069RHP1IFET0MgMg04a<C017AdL0NWYj05EEE@1YJFT0JVYZ
03dm?@1PH600PX:206][J`1BDU80IFEU03@d=01QHF40VYZJ03Ti>@1:BTX0>CTi03Xj>P2h^;P0Lg=c
02l_;`1ADE40>SXj08Z:RP2UYJD0DU9B038b<P17AdL0GemO0;6a/@1fMWH0>CTi05ADE01:BTX0QHF5
06IVIP0F5QH0CDe=07IfMP1>CTh0_kno04a<C00k>c/0[Jf]092@T02EUID0Ph>302DU9@1PH600I6AT
051@D01MGEd00P0k>c/0>@23Ph<0G5aL09VIV@27QhL0@d=303/k>`2:RXX0KFe]06EUI@1nOWh0?clo
04Q8B02XZ:P0BDU908:2PP0m?Cd0DU9B09nOW`2RXZ80?Cdm07=cL`12@T80D51@0:6QX@2SXj<0<S8b
03<c<`1@D500GemO07alO01QHF408b<S03Lg=`1FEUH0IVIV08N7Q`1YJFT07QhN03lo?`1QHF40Hf=S
0:2PX01fMWH0:RXZ04Q8B01[Jf/0Fe]K069RHP1`L700:RXZ01lO7`19BDT0QXJ606EUI@2h^;P0<C4a
03dm?@0>ooooo`0/05IFEP1JFUX0=cLg04m?C`1cLg<0GUiN04E5A@19BDT0GUiN05IFEP0m?Cd0JVYZ
05MGE`18B4P0Jf][07inOP1[Jf/0Hf=S04U9B@0c<c<0Cdm?04a<C012@T80B4Q8051@D01:BTX0DE5A
059BDP1@D500E5AD03`l?01KFe/0Hf=S05aLG01BDU80H61P05mOG`1:BTX0De=C051@D00f=SH0De=C
04Q8B01;Bd/2038b<P0>03dm?@1GEeL0>SXj0451@@0m?Cd0@D5103Ti>@1;Bd/0A4A406QXJ01:BTX0
>CTi04=3@`1bLW8204];B`1k05EEE@1NGUh0UIFE071`L00b<S80?Cdm05QHF027QhL0D51@04Q8B01;
Bd/0C4a<0492@P1?Cdl0DE5A065QH@1YJFT0Hf=S0451@@1BDU80ADE505QHF019BDT0CTi>01`L701B
DU80@T9205QHF01eMGD0ADE5061PH00M7Ad0De=C04U9B@1aLG40>3Ph05ADE01LG5`0?Cdm05]KF`1Q
HF40EEEE0:^[Z`1]KFd0GemO06IVIP10@400F5QH07QhN00h>3P0Bd];06a/K00c<c<0ATI604m?C`1A
DE40<S8b04e=C@1TI6@0HV9R05aLG01IFET0c/k>06ATI01OGel0>SXj05IFEP2GUiL0L71`09VIV@1k
Ng/0Jf][07UiN@1@D500@d=305IFEP1=CDd0H61P02l_;`0n?Sh0N7Qh06=SH`2i^KT0?Cdm0820P00a
<C40I6AT07AdM02PX:00`<3005EEE@2?Shl0G5aL07IfMP2][Jd0>SXj08:2PP0i>CT0Cdm?03hn?P1P
H600<c<c05ADE01EEED0DU9B06EUI@1PH600];Bd05IFEP1bLW80Hf=S05]KF`1EEED0AdM704=3@`2<
S8`0P82008R8R01nOWh0Jf][07AdM00b<S80R8R80080BTY:05/0R8R805MGE`29RHT0Bd];04m?C`10
@400LW9b03lo?`1OGel0@D5105aLG00e=CD0PX:20820P01IFET0AdM703Ph>019BDT0BTY:03Hf=P0n
?Sh0ADE506UYJ@0X:2P0D51@04Y:BP1TI6@0?Cdm05ADE010@400Fe]K03/k>`0m?Cd0IFEU05UIF@0n
?Sh0?Cdm04e=C@19BDT0Shn?05=CD`12@T80B4Q804I6AP0j>SX0>CTi05mOG`1@D500>3Ph07YjNP1<
C4`0?Cdm04U9B@0Q8B40IFEU03lo?`1<C4`0Ph>303lo?`1OGel0:BTY05]KF`1EEED0De=C06i^KP1S
Hf<0FUYJ05=CD`1UIFD0Kfm_06i^KP1mOGd0820P04i>CP0c<c<0E5AD02/[:`11@D40Bd];02<S8`1E
EED0<C4a03lo?`1OGel0MGEe05EEE@17AdL0EEEE055AD@1YJFT0GUiN013ooooo000>ooooo`80T92@
02l0E5AD0820P012@T80FUYJ04A4A017AdL0Ogmo059BDP14A4@0>3Ph05IFEP29RHT0O7al02<S8`11
@D40BDU906m_K`2^[Zh0?Shn02d];@1FEUH0B4Q80820P01`L7009BDU03De=@1UIFD0FEUI09bLW027
QhL04a<C04I6AP1YJFT0GEeM07UiN@1>CTh07alO03hn?P0i>CT0G5aL0410@019BDT0:RXZ04I6AP1?
Cdl0Bd];05=CD`0204=3@`1i08b<S01_Kfl0?Shn06IVIP0f=SH0=cLg09nOW`1LG5`0C4a<05iNGP0R
8R80@d=305eMG@0R8R80>3Ph04I6AP0T92@0BDU9061PH00j>SX0Bd];03Ph>00j>SX0Bd];07emO@0k
>c/0NGUi051@D01LG5`0OWin04M7A`11@D40?3`l03Ph>025QHD0/;2`02`/;00P8200DU9B04];B`29
RHT0OGem01dM7@0X:2P0LG5a05iNGP24Q8@07QhN01TI6@0`<300JFUY075aL@2KVi/06a/K03@d=01;
Bd/0OWin061PH02[Zj/09bLW04m?C`1ZJVX0I6AT04Q8B01kNg/0;Bd]04M7A`1]KFd0FUYJ03Ti>@1e
MGD0;bl_04U9B@2YZJT0MGEe038b<P1:BTX0>CTi051@D02@T900E5AD0410@01:BTX09BDU06UYJ@2?
Shl0:b/[03De=@1>CTh0<30`03hn?P1TI6@0;2`/04];B`19BDT0@d=307EeM@1KFe/0=cLg04M7A`1B
DU80Jf][0820P01bLW80<C4a05]KF`1QHF40GemO09jNWP2WYjL07AdM04U9B@1RHV80JVYZ0=3@d01?
Cdl0BTY:06a/K01nOWh0WinO0861P@0204=3@`1908:2PP25QHD0Q8B404i>CP0P8200C4a<09FEU@1Z
JVX0V9RH04A4A00[:b/0L71`08:2PP1?Cdl0L71`02HV9P0/;2`0S8b<0:ZZZP1[Jf/0MgMg02`/;01V
IVH0_Kfm06a/K02^[Zh0?clo06QXJ02g]kL0BTY:07=cL`0o?cl0:2PX05YJFP1ADE40HV9R04Q8B00f
=SH0CDe=07QhN00o?cl0I6AT0492@P25QHD0VYZJ051@D00i>CT0EEEE059BDP2HV9P0U9BD02/[:`16
ATH0HV9R05YJFP2ZZZX0R8R803Ph>018B4P0HV9R07]kN`2`/;00AdM7034a<@1@D500Ti>C05iNGP2e
]KD0PX:203lo?`1FEUH0Vi^K065QH@0207EeM@0M038b<P1`L700UIFE05eMG@2>SXh0EUIF04I6AP1L
G5`0U9BD04Q8B01DE5@0UIFE038b<P1dM7@0TY:B0410@00i>CT0P82004a<C016ATH0N7Qh05QHF018
B4P0J6QX03<c<`0o?cl0Ph>306i^KP0n?Sh03_ooool0O`1=CDd0Q8B404M7A`1JFUX0@d=305mOG`1@
D500Cdm?055AD@17AdL0@D5108Z:RP1gMgL0Hf=S06e]K@1aLG40OWin0451@@1?Cdl0BDU903hn?P1=
CDd0DE5A05iNGP1eMGD0Bd];03`l?01<C4`0BDU905YJFP1@D500F5QH05ADE01PH600FUYJ06EUI@1D
E5@0BTY:04U9B@0m?Cd0ATI605ADE00n?Sh0@41004a<C00`<300<C4a04=3@`0a<C40BDU903lo?`16
ATH0;bl_0451@@1MGEd0LG5a04i>CP17AdL0>3Ph061PH01JFUX0HV9R06m_K`1[Jf/0YJFU09BDU01O
Gel0NGUi075aL@1GEeL0I6AT04U9B@1LG5`0A4A403/k>`1=CDd0IVIV079bLP0m?Cd0EeMG04];B`1>
CTh0BDU906YZJP11@D40>3Ph02/[:`1RHV80F5QH05iNGP23Ph<0ADE503hn?P0k>c/0E5AD04U9B@1D
E5@0ATI6055AD@17AdL0ATI6055AD@1>CTh0CDe=0:n_[`1VIVH0B4Q8061PH00l?3`0F5QH05IFEP18
B4P0C4a<05eMG@1;Bd/0IFEU05=CD`1DE5@0?Shn0492@P1dM7@0JFUY071`L00[:b/0]kNg04A4A01P
H6000P11@D40:@1UIFD0M7Ad0;Fe]@1_Kfl0Hf=S04i>CP1VIVH0<S8b05YJFP0i>CT0De=C03Hf=P17
AdL0A4A405mOG`2BTY80>SXj09JFUP0n?Sh0EeMG08b<S036a/H0P82006=SH`2a/K40GUiN071`L01j
NWX0B4Q80:BTY00h>3P0O7al03Xj>P1[Jf/0AdM706a/K015ADD0DU9B069RHP1GEeL0Rh^;0080CTi>
06X0DU9B05UIF@1ADE40GemO04];B`2:RXX0ADE508R8R02FUYH0DE5A04E5A@0Q8B40I6AT03dm?@1D
E5@0Zj^[05YJFP1gMgL0ATI6061PH00M7Ad0@d=302XZ:P1VIVH0JFUY07AdM00Z:RX0PX:206a/K01T
I6@0:b/[02@T901?Cdl0>CTi030`<01=CDd0CTi>04I6AP11@D40De=C04U9B@1BDU80=cLg04I6AP1?
Cdl0K6a/04a<C01>CTh0GEeM06ATI00`<300FUYJ06UYJ@1=CDd0UIFE02TY:@0_;bl0D51@05]KF`17
AdL0@D5106IVIP1>CTh0C4a<08B4Q018B4P0JFUY05EEE@1;Bd/0H61P01lO7`11@D40UiNG02`/;01:
BTX0=3@d04=3@`1DE5@0;2`/07moO`1NGUh0C4a<03Xj>P1_Kfl0JFUY06][J`1kNg/0=3@d05=CD`0f
=SH0KVi^01`L7019BDT0ADE5034a<@1SHf<0BTY:05YJFP1/K6`0M7Ad03lo?`10@400GemO04a<C020
P800Ph>34?ooool000kooooo08<0WYjN0:f][@1@D500Lg=c06QXJ02FUYH0CDe=0410@01]KFd0QhN7
08>3P`17AdL0:BTY08:2PP32`/80J6QX05mOG`0];Bd0ADE508f=S@1iNGT0Cdm?05mOG`0Z:RX0F5QH
09BDU01]KFd0Bd];07YjNP0/;2`0FUYJ07emO@1BDU80GEeM05MGE`0^;Rh0CDe=07EeM@0[:b/0>3Ph
04Y:BP0f=SH0EUIF07QhN00O7al0BTY:04I6AP14A4@0IVIV07UiN@0M7Ad0C4a<059BDP1^KVh0UIFE
02h^;P0a<C40LG5a04I6AP1dM7@0SXj>01DE5@1<C4`0IfMW03/k>`1[Jf/0EEEE01/K6`0a<C40CTi>
06UYJ@2][Jd0ADE501dM7@13@d<0GUiN06i^KP32`/80Fe]K01DE5@1FEUH0Q8B406IVIP1UIFD08b<S
04E5A@1YJFT0K6a/04U9B@10@400:2PX05UIF@1bLW80EUIF05UIF@0c<c<0@41006IVIP1/K6`0<30`
06MWI`0[:b/092@T05]KF`1GEeL0:2PX05]KF`0U9BD08R8R05UIF@0l?3`0:b/[04M7A`0f=SH0ADE5
05QHF00R8R80?clo08B4Q017AdL0BTY:0:RXZ01:BTX0:BTY065QH@1`L700J6QX07UiN@0/;2`0=CDe
08>3P`020861P@2A0:2PX00c<c<0AdM7096AT@19BDT0KVi^08R8R00`<300>c/k07alO01hN7P0_[jn
03Lg=`0k>c/0GEeM06m_K`1lO7`0/k>c03lo?`0/;2`0F5QH08f=S@1ADE40Y:BT0451@@0a<C40CTi>
071`L01:BTX0G5aL02DU9@1@D500JVYZ07QhN00k>c/0CTi>034a<@1OGel0PH6105QHF00i>CT0O7al
0451@@1cLg<0Rh^;03hn?P1>CTh0AdM702XZ:P1^KVh0AdM703dm?@1GEeL0@d=305MGE`1aLG407alO
03Lg=`1MGEd0F5QH05]KF`2ATI40;2`/03lo?`1cLg<0HV9R0<G5a@1QHF40=CDe08f=S@1^KVh0X:2P
04e=C@0];Bd0DU9B05IFEP2OWil0Rh^;01TI6@1?Cdl0WYjN04e=C@1jNWX07QhN02@T901?Cdl0P820
065QH@1iNGT0:2PX05UIF@2BTY80D51@06QXJ02CTi<0<c<c04e=C@1mOGd0G5aL09RHV02QXJ40=SHf
08F5Q@1SHf<0@41005mOG`15ADD0=SHf05aLG02=SHd0Bd];071`L01NGUh0<30`03lo?`1IFET0@D51
04];B`10@400;Rh^075aL@1WIfL0;Rh^05eMG@1ADE40@d=3069RHP2RXZ80DU9B04i>CP1^KVh0HF5Q
05eMG@2@T900B4Q8034a<@1?Cdl0Ng]k07MgM`2l_;`0EEEE02<S8`1YJFT0GEeM06a/K00>ooooo`1;
051@D028R8P0C4a<05eMG@1ADE40MgMg06i^KP1[Jf/0I6AT04];B`17AdL0QhN7075aL@1SHf<0NGUi
07YjNP2>SXh0C4a<05QHF00g=cL0BTY:04i>CP1CDe<0CTi>05MGE`1;Bd/0?Shn05YJFP19BDT0E5AD
05QHF01lO7`0J6QX06ATI01>CTh0GEeM05UIF@1;Bd/0Cdm?05aLG016ATH0I6AT04=3@`1NGUh0BTY:
05MGE`1:BTX0HV9R03hn?P1>CTh0?Shn04Q8B00`<300?clo0451@@1CDe<0=3@d03lo?`0U9BD0GemO
05EEE@1?Cdl0G5aL065QH@32`/80L71`03<c<`0m?Cd0FUYJ05aLG01OGel0GUiN07]kN`1CDe<0B4Q8
0080DE5A0640GEeM05EEE@1hN7P0ATI604i>CP15ADD0E5AD0410@016ATH0:BTY04I6AP0j>SX0AdM7
05ADE00b<S80@d=303De=@1RHV80CDe=05mOG`11@D40AdM705QHF010@400IFEU055AD@1QHF40Q8B4
05iNGP1CDe<0KVi^055AD@1MGEd0G5aL071`L020P800Ph>3055AD@0l?3`0ATI605UIF@1JFUX0IVIV
05UIF@1XJ6P0Q8B404E5A@2d];@0H61P09jNWP15ADD0EEEE06m_K`1?Cdl0NGUi04A4A01`L700AdM7
04A4A00a<C40IVIV0492@P1HF5P0@41005ADE01;Bd/0EeMG09:BTP0S8b<0AdM702XZ:P1@D500JFUY
06MWI`1JFUX0BTY:06EUI@1FEUH0Ogmo07MgM`16ATH0IfMW03Ti>@19BDT0<30`07alO00n?Sh0CDe=
0492@P19BDT0CDe=051@D020P800CTi>071`L00k>c/0CDe=0080C4a<01l0B4Q806YZJP1KFe/0Q8B4
08j>SP17AdL0De=C0451@@1bLW80>SXj05ADE026QXH0JVYZ08:2PP18B4P0E5AD04I6AP1bLW80;Bd]
06e]K@15ADD0EUIF0451@@29RHT0NWYj03`l?00R8R807QhN04i>CP12@T80ADE50080@D5103L0KVi^
020P8017AdL0?3`l06ATI00a<C40BDU903dm?@1DE5@0;Bd]0451@@1/K6`0IVIV03Ti>@1:BTX0D51@
02l_;`1KFe/0<S8b02`/;017AdL0>CTi04E5A@0g=cL0DE5A05IFEP1;Bd/0Q8B404A4A015ADD0BDU9
02d];@1ADE409RHV03Ti>@1TI6@0<30`03Ti>@0V9RH0EEEE07inOP0m?Cd0YJFU06QXJ01GEeL0ATI6
07inOP1MGEd0EUIF06MWI`0k>c/0HF5Q03Hf=P1ADE40:RXZ0080FUYJ00`0?clo051@D00[:b/0=SHf
08N7Q`21PH40>SXj04e=C@1WIfL0Cdm?06m_K`1?Cdl@ooooo`003_ooool0o`1BDU80/[:b07]kN`1P
H600BDU907EeM@26QXH0?Shn07=cL`16ATH0Lg=c05MGE`0Y:BT0Cdm?07EeM@1FEUH0I6AT01lO7`0R
8R80>SXj04E5A@1<C4`0Lg=c028R8P0P8200GemO05]KF`1/K6`0T92@01XJ6P0m?Cd0JVYZ055AD@2a
/K40UiNG02TY:@0`<300MgMg04=3@`2IVIT0NGUi01TI6@0/;2`0EeMG04m?C`2][Jd0E5AD01hN7P0]
;Bd0MWIf04Q8B02FUYH0=3@d02TY:@1gMgL0Ti>C04m?C`12@T80:RXZ03hn?P1fMWH0FEUI04a<C00K
6a/08B4Q04U9B@1[Jf/0AdM703hn?P0f=SH0@d=307EeM@1YJFT09RHV0451@@0_;bl0@d=30:f][@1n
OWh0820P051@D00^;Rh0F5QH0820P00/;2`0;bl_04m?C`10@400EEEE04=3@`0I6AT0@d=303Lg=`10
@400Lg=c04a<C00U9BD0ATI605EEE@18B4P0VYZJ05iNGP0Y:BT0AdM707emO@1EEED0QhN702TY:@0R
8R80De=C06a/K02>SXh0RHV902d];@0c<c<0GemO04=3@`1HF5P0O7al04I6AP0m?Cd0EUIF08>3P`1e
MGD0Ng]k02PX:01PH600SHf=08R8R01BDU80=CDe0492@P1EEED0Jf][06MWI`1HF5P07alO02<S8`0f
=SH0N7Qh05UIF@1:BTX0De=C03Ti>@20P800PH6103Lg=`1TI6@0ATI6051@D02VYZH0Hf=S03De=@1@
D500?clo05ADE01[Jf/0OWin02/[:`1<C4`0@T9204E5A@2/[:`0:BTY03De=@1jNWX0S8b<05IFEP2G
UiL06QXJ03De=@1cLg<0QXJ605]KF`31`L40=CDe03Xj>P22PX80EEEE09jNWP1/K6`07alO03Ti>@1/
K6`0RXZ:0;Zj^P0k>c/0;2`/07=cL`1GEeL0Lg=c07emO@0`<300AdM7075aL@1dM7@0UIFE02/[:`1:
BTX0C4a<061PH01YJFT0EUIF03hn?P24Q8@0TI6A04i>CP1[Jf/0>CTi08R8R02RXZ80@T9204Y:BP1M
GEd0ADE507emO@1FEUH0<S8b05QHF00e=CD0BDU90820P00j>SX0D51@05IFEP1;Bd/0XJ6Q0:BTY00f
=SH0De=C06EUI@0l?3`0Rh^;0:j^[P0j>SX0LG5a07MgM`1FEUH0WIfM06UYJ@0P8200BDU906ATI01R
HV80UIFE096AT@0O7al0@T92069RHP1SHf<0QHF505EEE@0N7Qh0CDe=07moO`005`1MGEd0NGUi04U9
B@0R8R80DE5A05EEE@1XJ6P0WYjN08N7Q`0P8200A4A406=SH`1XJ6P0HF5Q06QXJ00[:b/0CTi>05mO
G`1fMWH0De=C08:2PP0h>3P0?Shn00kooooo00H0HV9R07]kN`1<C4`0Cdm?05EEE@1WIfL204e=C@3o
069RHP1@D500ATI605]KF`1UIFD0GemO06i^KP25QHD0LG5a05eMG@1;Bd/0?Cdm04a<C01>CTh0@d=3
04U9B@1RHV80Fe]K05aLG00k>c/0HF5Q05]KF`0k>c/0DE5A065QH@1SHf<0NWYj061PH01=CDd0A4A4
05aLG01ADE40CDe=05iNGP1<C4`0FEUI04i>CP0m?Cd0?clo04M7A`0i>CT0AdM703Xj>P0i>CT0;Rh^
0451@@0l?3`0H61P03hn?P19BDT0=cLg05iNGP1OGel0De=C06=SH`1cLg<0RXZ:06QXJ00m?Cd0AdM7
07]kN`1WIfL0OWin04a<C01SHf<0ADE503Xj>P1?Cdl0GUiN05aLG013@d<0GEeM04U9B@1DE5@0BTY:
065QH@1;Bd/0AdM702l_;`1=CDd0A4A404];B`1dM7@0ATI604i>CP0n?Sh0EeMG05IFEP1OGel0@d=3
04Y:BP1LG5`0CDe=06e]K@1KFe/0KFe]0:JVYP1aLG40HV9R06UYJ@15ADD0Hf=S05QHF01_Kfl0C4a<
05YJFP0d=3@0BDU905aLG01>CTh0=SHf0451@@1dM7@0I6AT061PH02=SHd0emOG04];B`1RHV80@d=3
05EEE@1/K6`0NGUi0:JVYP1IFET0E5AD059BDP18B4P092@T04E5A@0e=CD0DU9B06=SH`1=CDd0KFe]
06QXJ02DU9@0A4A406][J`0n?Sh0CDe=05eMG@1GEeL0JVYZ04m?C`26QXH0E5AD08F5Q@1kNg/0>SXj
07YjNP0o?cl0GemO038b<P1CDe<0?3`l08B4Q01:BTX0B4Q805=CD`1OGel0PH6102l_;`1EEED0@410
04U9B@16ATH0E5AD034a<@1RHV80BDU909>CT`1mOGd0GUiN05aLG018B4P0Ti>C05YJFP1>CTh0MgMg
06UYJ@1bLW80@T9204U9B@0^;Rh0AdM7038b<P1@D500KVi^06EUI@18B4P0TY:B06m_K`17AdL0ATI6
03De=@1OGel0BTY:03`l?01>CTh0@d=304Q8B00a<C40Cdm?0410@01CDe<0=3@d04=3@`1>CTh0Fe]K
0410@01LG5`0KVi^04e=C@0Z:RX0D51@05=CD`0f=SH0J6QX01dM7@0j>SX0@41003`l?012@T8092@T
04e=C@0o?cl0<S8b059BDP0[:b/0GemO05=CD`0W9bL0H61P024Q8@0j>SX0UIFE02h^;P17AdL0<C4a
03Xj>P1<C4`0?Cdm04m?C`1MGEd0IFEU04E5A@1SHf<0@D51065QH@1RHV8092@T04a<C014A4@0NGUi
000?02HV9P0g=cL0?Cdm04=3@`1aLG40?3`l0410@01HF5P0FEUI04Q8B01UIFD0JVYZ051@D01ZJVX0
H61P013ooooo000>ooooo`3@05EEE@1mOGd0O7al08:2PP0m?Cd0EeMG0:ZZZP1OGel0Xj>S02l_;`1[
Jf/0Ti>C0492@P2MWId0C4a<02h^;P1EEED0DU9B07IfMP26QXH09BDU02TY:@1HF5P0QHF506][J`0e
=CD0<S8b05=CD`1bLW80EUIF07inOP0b<S809RHV04Y:BP28R8P0Hf=S04Y:BP0V9RH071`L055AD@1W
IfL0>3Ph03Xj>P0o?cl0>SXj075aL@1gMgL0:RXZ03Hf=P0e=CD0HV9R09>CT`18B4P0:BTY03lo?`0j
>SX0GUiN04];B`0S8b<0=CDe05UIF@1VIVH0MGEe020P800C4a<0BDU907MgM`1HF5P0O7al04m?C`0U
9BD0M7Ad08Z:RP1@D500XZ:R065QH@0Z:RX0IFEU08V9R@19BDT0alO704=3@`0Z:RX0GemO05EEE@1i
NGT0RHV9018B4P0k>c/0EeMG03Hf=P2/[:`0?Cdm02TY:@0U9BD0FUYJ05EEE@1eMGD06QXJ02LW9`1L
G5`0QHF504I6AP1BDU80820P04Y:BP1_Kfl0N7Qh03dm?@1KFe/06a/K05ADE02/[:`0AdM7020P801<
C4`0>c/k04E5A@20P800>SXj02/[:`0k>c/0?Shn04e=C@29RHT0:2PX034a<@1ADE40BDU907]kN`1F
EUH07QhN03dm?@1SHf<0HV9R08j>SP1@D50071`L04M7A`1MGEd0KVi^07QhN01jNWX0ADE505IFEP1Z
JVX0H61P07moO`1?Cdl0FEUI09BDU020P800I6AT08N7Q`21PH40:BTY04U9B@1PH600GUiN09ZJVP24
Q8@0820P065QH@1EEED0H61P06UYJ@0e=CD0?Cdm07AdM01LG5`0HF5Q04a<C00U9BD0DE5A07UiN@1X
J6P0G5aL02h^;P0j>SX0Cdm?06ATI016ATH0H61P01lO7`1MGEd0Rh^;055AD@0g=cL0@41003dm?@1O
Gel0Ph>303Lg=`1SHf<0J6QX04i>CP2@T900De=C03Hf=P1JFUX0EEEE092@T01lO7`08b<S04e=C@1:
BTX0HV9R0<[:bP80ADE504@0QXJ6055AD@2@T900WinO034a<@1HF5P0IFEU0861P@2b/[80ATI603`l
?01NGUh0K6a/0<O7a`2DU9@0820P04a<C023Ph<0JFUY0:b/[01oOgl0?3`l04A4A01WIfL0HV9R09NG
U`0l?3`0C4a<05IFEP1oOgl0ATI6065QH@0n?Sh0ATI606e]K@2VYZH0Cdm?059BDP12@T80B4Q809NG
U`1lO7`09RHV03lo?`10@400Bd];07emO@1bLW80;Bd]02HV9P1BDU80=SHf09:BTP2i^KT0<30`02TY
:@1]KFd0E5AD05mOG`2WYjL0<c<c02TY:@1@D500DE5A07YjNP2OWil0Cdm?02/[:`kooooo04/0JVYZ
06a/K018B4P0BTY:03Xj>P1BDU80ADE503Ti>@0h>3P0@41004A4A01_Kfl0Hf=S05]KF`1oOgl0W9bL
07UiN@0m?Cd0F5QH04U9B@0^;Rh0=cLg0451@@15ADD0@41004Y:BP15ADD0G5aL03hn?P1QHF40O7al
06ATI01LG5`0EeMG055AD@1GEeL0CDe=04];B`15ADD0BDU904e=C@1<C4`0@D5103Lg=`0_;bl09bLW
051@D017AdL0:RXZ03lo?`19BDT0=CDe02XZ:P0i>CT0>c/k04a<C00k>c/0KFe]02l_;`1OGel0EEEE
03lo?`1TI6@0UYJF09FEU@1gMgL0C4a<05IFEP1`L700GEeM06IVIP1[Jf/0MgMg03Ti>@10@4000`1;
Bd/0K`0];Bd0DE5A04i>CP0k>c/0B4Q8061PH00o?cl0AdM702LW9`1@D500?Shn0451@@1TI6@0>SXj
04=3@`0/;2`0K6a/05IFEP1ADE40Cdm?05]KF`1GEeL0?Shn06MWI`1IFET0D51@09nOW`1[Jf/0EUIF
05YJFP1=CDd0FEUI05mOG`0o?cl0EUIF07IfMP19BDT0De=C04m?C`1]KFd0B4Q8079bLP1NGUh0C4a<
0820P01GEeL0]KFe04E5A@1]KFd0C4a<04=3@`1BDU80G5aL08^;R`1gMgL0NWYj07inOP1`L700DE5A
07YjNP1IFET0@41003Ph>01NGUh0KVi^07moO`2WYjL0?clo079bLP1<C4`0GemO06][J`20P800MgMg
061PH02BTY80BDU90820P01YJFT0AdM709bLW00d=3@0JVYZ028R8P1HF5P0=cLg05EEE@15ADD0KVi^
071`L00e=CD0D51@03De=@1HF5P0ADE505ADE019BDT0ATI603hn?P1GEeL0=SHf08^;R`1mOGd0=cLg
04A4A00L71`0UIFE03lo?`1dM7@0Rh^;06a/K00206ATI01G07EeM@0U9BD0Cdm?01DE5@1^KVh0Bd];
06a/K00U9BD0JVYZ06=SH`1LG5`0:BTY03hn?P17AdL0>CTi04Y:BP1DE5@0Cdm?065QH@0d=3@0GUiN
0410@01>CTh0<S8b04A4A01]KFd0TI6A03Ti>@1ADE40HF5Q05QHF00g=cL0BDU905QHF011@D40Ti>C
03Ph>00f=SH0BDU9030`<019BDT0=SHf04E5A@0m?Cd0@T9208J6QP0d=3@0@T92030`<00I6AT0BDU9
02/[:`1JFUX0SXj>03Ph>01KFe/0<S8b05QHF01OGel0A4A406a/K01TI6@0GemO05QHF01PH600DU9B
07inOP2<S8`0@d=304];B`1?Cdl0CTi>02h^;P19BDT0De=C03dm?@1KFe/0D51@05]KF`1iNGT0Q8B4
06=SH`1;Bd/0Fe]K04E5A@1aLG40I6AT013ooooo000>ooooo`0O07IfMP1cLg<0Cdm?06e]K@16ATH0
GUiN0451@@16ATH0IVIV05YJFP12@T80=CDe030`<01_Kfl0Ti>C03Hf=P16ATH09bLW03Lg=`1/K6`0
AdM701`L7017AdL0?Shn075aL@1UIFD0;Bd]03Xj>P13@d<0Bd];0::RXP0202TY:@0R034a<@1MGEd0
KFe]08f=S@0P82004Q8B04=3@`1iNGT0Kfm_07MgM`1EEED061PH05YJFP24Q8@0De=C08f=S@1CDe<0
:BTY06=SH`1VIVH0Hf=S08b<S00Q8B40>CTi065QH@0b<S80DU9B06][J`0K6a/0FUYJ07alO017AdL0
MWIf04M7A`80>3Ph06l0JFUY05ADE024Q8@0>c/k04E5A@18B4P0N7Qh0451@@1hN7P0:RXZ02d];@1f
MWH0MgMg0410@00^;Rh0=CDe05=CD`1eMGD0@d=302LW9`0i>CT0C4a<0:BTY02PX:00:b/[02h^;P0m
?Cd0MWIf0;Fe]@0l?3`06a/K03<c<`1JFUX0G5aL06UYJ@0U9BD0@T9204];B`1GEeL0B4Q808j>SP0S
8b<09RHV05MGE`1gMgL0GemO071`L00U9BD0GUiN06i^KP1GEeL0HV9R0:>SX`0i>CT0@T9209ZJVP1/
K6`0D51@08R8R00a<C40A4A408^;R`1OGel0GEeM06i^KP0F5QH0MgMg0:>SX`1>CTh0GUiN04m?C`0i
>CT0G5aL075aL@15ADD0OGem03Hf=P0Y:BT0QhN706ATI00l?3`0Jf][034a<@0i>CT0R8R80:2PX00o
?cl0JVYZ034a<@18B4P0Q8B408n?S`0c<c<0C4a<03Ti>@1]KFd0Vi^K06][J`0];Bd0EUIF069RHP1j
NWX0OGem01lO7`1IFET0V9RH079bLP29RHT0CDe=024Q8@1aLG400P1TI6@0H02MWId071`L01dM7@1j
NWX0I6AT065QH@1ZJVX09BDU05EEE@2<S8`0IFEU09ZJVP2BTY80@d=308J6QP20P800D51@05]KF`0N
7Qh0EeMG0;6a/@1=CDd0QXJ603Xj>P0W9bL0OGem07IfMP1JFUX0=SHf03@d=019BDT0Shn?05=CD`1M
GEd0;Rh^04I6AP23Ph<0MWIf04Y:BP1ZJVX0=3@d06QXJ02n_[h0MgMg0410@01IFET0>CTi07EeM@2F
UYH0De=C05MGE`1KFe/0HF5Q09ZJVP18B4P0;Rh^04m?C`1LG5`0GemO07QhN00k>c/0=cLg0492@P1[
Jf/0H61P07alO018B4P0;2`/06YZJP2BTY80EUIF079bLP15ADD0;Rh^07YjNP23Ph<0Hf=S07EeM@24
Q8@0;Rh^055AD@2GUiL0N7Qh05MGE`26QXH0D51@02l_;`1oOgl0G5aL05mOG`1`L700>3Ph02l_;`1O
Gel0Hf=S061PH0kooooo0?D0G5aL065QH@1lO7`0K6a/04a<C01YJFT0IFEU05mOG`1JFUX0GemO079b
LP1mOGd0Jf][06MWI`2IVIT0XZ:R07AdM01=CDd0DE5A0451@@10@400AdM705=CD`1OGel0HF5Q08B4
Q025QHD0KFe]05EEE@14A4@0DU9B07MgM`1bLW80OGem069RHP1eMGD0G5aL04i>CP1:BTX0FUYJ04m?
C`1CDe<0?clo05MGE`13@d<0=cLg03Ti>@1>CTh0<c<c03hn?P0a<C40@D51055AD@1QHF40BTY:05IF
EP0m?Cd0D51@03<c<`1DE5@0EUIF04E5A@1LG5`0MGEe09NGU`1LG5`0:2PX03Lg=`1CDe<0GemO05EE
E@0a<C40CDe=03Ph>013@d<0D51@04I6AP19BDT0=SHf05QHF00g=cL0>CTi03/k>`13@d<0@41003Ti
>@0H61P0:b/[02LW9`0`<300=SHf02h^;P11@D408B4Q030`<00e=CD0@410034a<@11@D40CDe=03dm
?@1=CDd0B4Q804a<C01hN7P0C4a<05EEE@1MGEd0CDe=055AD@1`L700CDe=065QH@1]KFd0@410051@
D01BDU80HF5Q04i>CP15ADD0HV9R079bLP2^[Zh0Z:RX0=OGe`1EEED0QhN706][J`1FEUH0MgMg0820
P02DU9@0?clo05UIF@16ATH0LG5a04e=C@1DE5@0DU9B07inOP15ADD0CDe=0861P@1mOGd0Vi^K06i^
KP1eMGD0EUIF05YJFP21PH40VYZJ09JFUP1XJ6P0Q8B406=SH`2:RXX0K6a/03`l?01PH600>c/k05YJ
FP0d=3@0Hf=S03`l?01NGUh0EEEE051@D01:BTX0GUiN07AdM019BDT0G5aL04i>CP16ATH0C4a<04m?
C`1OGel0P82005mOG`1fMWH0IVIV0492@P1CDe<0@d=30820P01?Cdl0Lg=c09bLW02;Rh/0Q8B40492
@P16ATH0>CTi07MgM`0H61P0AdM7055AD@1FEUH0?clo09RHV01LG5`0?Shn03Ph>00c<c<0JFUY04E5
A@0k>c/0@T9204];B`1KFe/0:BTY04I6AP13@d<0BDU902XZ:P0m?Cd0?Shn04e=C@0`<300CTi>06IV
IP1nOWh0@D5104a<C019BDT0<S8b04Y:BP0S8b<0<c<c05ADE00^;Rh0?clo034a<@1=CDd0CTi>04=3
@`1ADE40?clo03dm?@14A4@0Bd];08B4Q00d=3@0DU9B065QH@0204a<C00O04A4A010@400GUiN05YJ
FP22PX80OGem05=CD`1OGel0JVYZ02`/;01GEeL0Rh^;04M7A`1HF5P0A4A4055AD@0g=cL0CDe=05AD
E01MGEd0IVIV0492@P14A4@0M7Ad05EEE@15ADD0@D5105IFEP1@D500QXJ605]KF`0@ooooo`003_oo
ool0hP1]KFd0QHF506EUI@1bLW80De=C08b<S022PX80F5QH08J6QP1BDU80RHV906UYJ@0g=cL0EeMG
059BDP1RHV80RHV90451@@0m?Cd0F5QH05YJFP15ADD0Kfm_02TY:@1=CDd0LG5a04A4A01bLW80MGEe
03@d=01=CDd0HF5Q04Q8B027QhL0M7Ad024Q8@1EEED0KFe]04I6AP1XJ6P0H61P020P8011@D40KVi^
04I6AP1_Kfl0ATI602@T901FEUH0PX:2051@D01XJ6P0820P03dm?@1bLW80KVi^061PH016ATH0=SHf
06][J`2;Rh/0?Shn07AdM00W9bL0=CDe07moO`1mOGd0=cLg06IVIP1fMWH0D51@07alO01WIfL0;2`/
06m_K`15ADD0FEUI0;2`/01RHV80:BTY05eMG@17AdL0GUiN071`L00W9bL0?clo06=SH`17AdL0C4a<
05ADE00S8b<0CDe=07=cL`1SHf<0KVi^05ADE00Z:RX0Cdm?05aLG017AdL0QHF505ADE00/;2`0Cdm?
04I6AP1JFUX0Rh^;02h^;P0b<S80G5aL059BDP1TI6@0KVi^02PX:00h>3P0MWIf03dm?@1=CDd0M7Ad
03Lg=`0];Bd0HF5Q07EeM@1IFET0Jf][03Lg=`1;Bd/0IfMW07AdM01:BTX0HV9R03<c<`1TI6@0S8b<
071`L01BDU80Bd];03Ph>01^KVh0Lg=c03De=@1QHF40G5aL05MGE`2JVYX0P820034a<@1/K6`0G5aL
069RHP2OWil0G5aL034a<@1DE5@0L71`05IFEP2KVi/0J6QX02/[:`1IFET0HV9R04=3@`2?Shl0B4Q8
03/k>`1HF5P0K6a/04a<C01lO7`0>3Ph059BDP1RHV80Cdm?05UIF@2SXj<0ADE505iNGP1hN7P0<C4a
07]kN`18B4P06a/K06=SH`1PH600IVIV08:2PP0/;2`0CDe=07AdM01:BTX0GUiN07UiN@0m?Cd0JVYZ
08R8R01GEeL0LG5a0492@P1BDU80PH6106][J`1OGel0BTY:05YJFP2GUiL0H61P04I6AP1OGel0De=C
071`L020P800;Bd]061PH01OGel0B4Q807alO01VIVH0ATI606MWI`1GEeL0H61P06EUI@0_;bl0ATI6
07AdM01UIFD207]kN`0=03Xj>P1KFe/0IVIV05QHF02@T900Ogmo03dm?@1kNg/0KVi^05UIF@2NWYh0
IVIV03<c<`0205mOG`0S05YJFP1_Kfl0G5aL02LW9`1>CTh0JVYZ05EEE@23Ph<0HF5Q030`<01IFET0
N7Qh04=3@`1nOWh0LW9b03Lg=`1<C4`0KVi^06e]K@1YJFT0CTi>03@d=010@400Hf=S061PH01=CDd0
GemO03Ph>01<C4`0I6AT05eMG@14A4@0H61P034a<@1VIVH03_ooool0?@1/K6`0Lg=c04];B`1HF5P0
E5AD06i^KP17AdL0A4A4069RHP0o?cl0@D5107AdM022PX80NGUi07AdM02;Rh/0RXZ:061PH01IFET0
<S8b03`l?018B4P0C4a<04e=C@1GEeL0BDU904=3@`19BDT0FUYJ05ADE01IFET0J6QX07QhN01[Jf/0
GemO071`L01GEeL0E5AD05=CD`1GEeL0Bd];06=SH`1BDU80Bd];04I6AP0f=SH0Cdm?04];B`0l?3`0
Cdm?04M7A`0n?Sh0;Rh^04E5A@1MGEd0R8R805IFEP1ADE40ATI606EUI@1PH6000P1EEED0K`27QhL0
^[Zj08f=S@17AdL0ATI606e]K@1hN7P0EeMG05IFEP1[Jf/0DE5A04i>CP1QHF40CTi>04Y:BP12@T80
FUYJ04i>CP1BDU80DE5A05eMG@15ADD0BDU902PX:01BDU80=cLg0451@@1QHF40=SHf04e=C@0];Bd0
EEEE051@D01;Bd/0@D5105EEE@1NGUh0AdM705UIF@1LG5`0FEUI09bLW01VIVH0J6QX06EUI@1>CTh0
H61P07IfMP12@T80CDe=04];B`14A4@0DE5A04i>CP0j>SX0ADE50410@01YJFT0A4A407=cL`1iNGT0
^k^k05=CD`1YJFT0Hf=S04E5A@1:BTX0Jf][0:f][@0n?Sh0FUYJ03`l?014A4@09BDU04Y:BP1;Bd/0
De=C06ATI01MGEd0NWYj08Z:RP22PX80H61P0861P@1>CTh0DE5A09NGU`2HV9P0Shn?07QhN02:RXX0
ATI608>3P`1NGUh0C4a<06UYJ@0g=cL0GEeM02DU9@1IFET0?Cdm06MWI`1<C4`0GEeM04m?C`1;Bd/0
GemO04a<C01EEED0?Shn05IFEP0205QHF01V04E5A@25QHD0AdM708F5Q@1TI6@0D51@05mOG`1;Bd/0
M7Ad05iNGP1YJFT0QhN707YjNP1dM7@0De=C05QHF014A4@0E5AD03Ph>01ADE40GUiN055AD@0j>SX0
Jf][034a<@1=CDd071`L02/[:`16ATH0<30`04=3@`15ADD0@d=3055AD@0[:b/0Cdm?0492@P15ADD0
=SHf0410@01ADE40J6QX03De=@1QHF40JFUY04e=C@0U9BD0D51@05UIF@14A4@0Lg=c02DU9@0h>3P0
Fe]K04E5A@19BDT0D51@06][J`1CDe<0FEUI06a/K01>CTh0D51@04M7A`0c<c<0C4a<02`/;01@D500
G5aL03Lg=`19BDT0=3@d051@D01FEUH0<c<c07EeM@10@400EeMG04a<C01[Jf/0D51@06IVIP1kNg/0
>c/k05eMG@0f=SH0Bd];02DU9@16ATH0>c/k05ADE01eMGD0A4A404I6AP23Ph<0HF5Q04a<C01QHF40
IfMW05EEE@1eMGD0IVIV4?ooool000kooooo0?l0IfMW06UYJ@1WIfL0Ti>C03`l?017AdL0K6a/06MW
I`2YZJT0=CDe04Y:BP1TI6@0EeMG08j>SP19BDT08B4Q04Y:BP1_Kfl0WIfM08V9R@0S8b<09RHV04i>
CP1LG5`0`l?304U9B@0K6a/0BTY:075aL@1?Cdl0OGem024Q8@0Y:BT0HV9R08V9R@11@D40KVi^014A
4@0W9bL0DE5A05ADE01NGUh0MGEe04i>CP0d=3@0@d=305QHF00e=CD0GEeM03lo?`1PH600O7al03/k
>`19BDT0GEeM01lO7`1BDU80/K6a0410@00_;bl0>c/k02l_;`1`L700EeMG034a<@19BDT0<30`01/K
6`1GEeL0KFe]02LW9`16ATH0BTY:06UYJ@2OWil0Jf][02<S8`13@d<0G5aL07MgM`2/[:`0FUYJ03Ti
>@11@D40>CTi08>3P`2MWId05QHF01/K6`1@D500EUIF0;Ng]`0n?Sh0410@02@T901=CDd0GUiN08V9
R@0G5aL08R8R03hn?P1gMgL0H61P06QXJ00H61P0;Rh^05iNGP24Q8@0=cLg05ADE00U9BD0E5AD065Q
H@1VIVH0@T9204M7A`0i>CT0BTY:05mOG`1YJFT0=CDe05UIF@0_;bl0<c<c08>3P`1<C4`0;bl_05MG
E`0h>3P0ADE509^KV`1ADE40Cdm?05aLG00l?3`0Hf=S07EeM@0e=CD0B4Q804=3@`0a<C40HV9R05]K
F`0a<C40C4a<04];B`1MGEd0UIFE06e]K@0h>3P0DU9B06EUI@1^KVh0V9RH07MgM`0f=SH0@d=307If
MP1NGUh0`/;20:>SX`0_;bl0C4a<06a/K020P800YZJV0451@@0d=3@0LW9b065QH@1]KFd0Fe]K02PX
:01CDe<0J6QX06m_K`29RHT0Rh^;038b<P0j>SX0PX:206MWI`1^KVh0:BTY03Xj>P1ADE40CDe=03hn
?P1CDe<0;Bd]05QHF01mOGd0EEEE051@D01ADE40=SHf08n?S`1/K6`0EEEE071`L00b<S80Hf=S096A
T@1KFe/0HF5Q03/k>`1]KFd0b<S804=3@`1IFET0HF5Q0410@028R8P0XJ6Q024Q8@0m?Cd0Cdm?05iN
GP2TY:@0;Rh^04=3@`1]KFd0Cdm?09JFUP2;Rh/0;Rh^0451@@1cLg<0GEeM08N7Q`1PH600;Rh^05UI
F@2BTY80IFEU07alO00n?Sh0GemO069RHP2QXJ40C4a<07alO00k>c/0BDU9065QH@25QHD0DU9B07Ad
M01?Cdl0;Bd]07inOP2JVYX0E5AD06ATI01DE5@001L0?clo05UIF@1iNGT0<30`0451@@1dM7@0<c<c
06m_K`2QXJ40<c<c03Ti>@1>CTh0>c/k03Ph>02IVIT0<c<c030`<01cLg<0?3`l04Q8B02RXZ80?3`l
03<c<`0>ooooo`0U06a/K01lO7`0FEUI06EUI@17AdL0IFEU075aL@1IFET0L71`04Y:BP1bLW80S8b<
07alO01eMGD0ZZZZ0=?Cd`21PH40F5QH07EeM@1FEUH0C4a<051@D01KFe/0D51@05YJFP1kNg/0G5aL
05UIF@1CDe<0Hf=S08B4Q01`L700OWin08B4Q023Ph<0RHV906QXJ00205=CD`0605mOG`1LG5`0Ng]k
04Y:BP1PH600EEEE0P1:BTX0<01BDU80?Shn04];B`14A4@0<30`03Ti>@12@T80Bd];05MGE`0i>CT0
D51@04=3@`1@D500EEEE03Xj>P1XJ6P0Jf][06IVIP1PH600FUYJ05]KF`1GEeL0DE5A05IFEP14A4@0
Jf][02/[:`0n?Sh0IVIV04];B`0n?Sh0<c<c05ADE018B4P0>3Ph04=3@`1ADE40@D5105=CD`0m?Cd0
@d=305=CD`13@d<0H61P04M7A`1ADE40?3`l04U9B@80Bd];0;D0DU9B05IFEP1NGUh0DU9B075aL@1R
HV80I6AT08Z:RP1eMGD0IfMW06QXJ01WIfL0GUiN07inOP0o?cl0Lg=c07AdM00l?3`0>3Ph055AD@1R
HV80@T9204=3@`1GEeL0>SXj08j>SP1cLg<0alO704U9B@22PX80GemO05ADE01CDe<0P8200:BTY00W
9bL0S8b<03@d=013@d<09BDU05]KF`0l?3`0@D5104];B`1<C4`0MgMg09fMW@1nOWh0>CTi055AD@0l
?3`0B4Q805mOG`2?Shl0QXJ605=CD`1EEED0ATI609VIV@1LG5`0@41005aLG016ATH0E5AD03De=@18
B4P0?Shn0861P@10@400GEeM055AD@19BDT0GemO05eMG@1aLG40C4a<051@D011@D40MWIf07EeM@29
RHT0IfMW092@T01fMWH0CTi>04I6AP0f=SH0PH6105aLG01GEeL0P82008F5Q@1`L700JVYZ06=SH`0o
?cl0CTi>01PH601NGUh0Fe]K05=CD`1@D500NGUi04Y:BP1?Cdl0B4Q803Hf=P1CDe<0BDU904i>CP1H
F5P0AdM705eMG@0i>CT0D51@04];B`1BDU80@41004U9B@1>CTh0G5aL02l_;`1NGUh0K6a/06EUI@16
ATH0D51@05=CD`1KFe/0Kfm_03hn?P0b<S80F5QH0410@01BDU80Bd];04A4A01<C4`0<C4a05YJFP0_
;bl0@D51059BDP12@T80GemO0410@00g=cL0?3`l0492@P1jNWX0AdM704Y:BP1ADE40ADE50:6QX@11
@D40LG5a05ADE01^KVh07alO055AD@1cLg<0=CDe05ADE01@D500Bd];04=3@`1<C4`0EeMG06ATI01M
GEd0B4Q805YJFP2IVIT0Cdm?03`l?01=CDd0Fe]K059BDP24Q8@0De=C013ooooo000>ooooo`2b0820
P02IVIT0D51@07alO01FEUH0X:2P04I6AP17AdL0H61P05eMG@1^KVh0>SXj03hn?P1>CTh0FUYJ0451
@@1ADE407QhN03Xj>P23Ph<0LG5a04I6AP1CDe<071`L04m?C`2;Rh/0A4A402`/;013@d<0>3Ph09ZJ
VP0k>c/051@D03Hf=P1>CTh0<C4a08V9R@1?Cdl06a/K03Ti>@0h>3P0B4Q807inOP1^KVh0:BTY059B
DP0o?cl0>3Ph08F5Q@1WIfL0820P06e]K@16ATH0IFEU06m_K`0P8200C4a<08R8R01cLg<0EEEE03Lg
=`0U9BD0>c/k06EUI@1cLg<0Y:BT02d];@0W9bL0:BTY05IFEP16ATH0Ph>3020P800[:b/0N7Qh08B4
Q01:BTX0MWIf02TY:@1DE5@0IVIV069RHP1XJ6P0@T9202d];@13@d<0RXZ:07=cL`13@d<0AdM703@d
=01JFUX0Shn?0451@@12@T80>CTi03Xj>P24Q8@0C4a<02/[:`1<C4`0>3Ph04a<C02UYJD0=SHf030`
<013@d<0AdM705UIF@1oOgl06ATI02h^;P1BDU80=3@d07inOP2GUiL06QXJ04=3@`11@D40BTY:065Q
H@2j^[X0Bd];0451@@1BDU80Hf=S0861P@2/[:`0=3@d03dm?@1ZJVX0KFe]08N7Q`2PX:006ATI04E5
A@1dM7@0IfMW08j>SP19BDT0;2`/04A4A027QhL0Jf][08N7Q`0O7al0=SHf061PH01`L700KVi^08F5
Q@0Y:BT0=SHf07QhN01iNGT0?clo08V9R@0U9BD0:2PX04];B`1hN7P0CDe=06YZJP0K6a/0BTY:05UI
F@1WIfL0=SHf05UIF@0i>CT0Fe]K09NGU`0^;Rh0>SXj07=cL`11@D40Hf=S09:BTP80@d=30680<C4a
03@d=01cLg<0>3Ph02l_;`1MGEd0A4A407UiN@24Q8@06ATI04e=C@1OGel0C4a<09>CT`28R8P0:2PX
04Q8B01FEUH0KFe]0;2`/00g=cL0@d=306IVIP21PH40W9bL024Q8@0^;Rh0MgMg079bLP1_Kfl0=cLg
02DU9@1JFUX0W9bL07moO`1RHV80?Cdm069RHP20P800KFe]05MGE`1OGel0?clo04Q8B01/K6`0B4Q8
06YZJP1TI6@0=SHf08B4Q01dM7@0>SXj05mOG`1VIVH0?clo092@T01GEeL0?clo07UiN@1=CDd0<c<c
08b<S01ZJVX0?clo06YZJP1FEUH0=3@d05aLG01iNGT0<30`059BDP1QHF40@T9205]KF`1_Kfl07alO
03Ti>@1OGel0D51@061PH02JVYX0=3@d03Lg=`1[Jf/0FEUI06QXJ02LW9`0?clo038b<P1YJFT0GEeM
06=SH`2XZ:P0<c<c04=3@`1;Bd/0EEEE06ATI0kooooo08X0PX:206i^KP1GEeL0EEEE05QHF01BDU80
AdM704a<C01ZJVX0@41003lo?`1RHV80TI6A07UiN@1ADE40BDU905UIF@1GEeL0C4a<03/k>`1=CDd0
>c/k02XZ:P0o?cl0ATI602PX:00c<c<0@d=305iNGP16ATH0CDe=059BDP1OGel0D51@06e]K@1SHf<0
G5aL04A4A012@T80ADE504A4A01MGEd0@d=3051@D01VIVH0@d=3051@D01EEED0FEUI04i>CP19BDT0
C4a<09:BTP2f][H0Bd];07alO01FEUH0FEUI02TY:@1FEUH0IVIV04I6AP1YJFT0RXZ:08j>SP2UYJD0
B4Q804U9B@1oOgl0T92@04M7A`19BDT0IfMW05QHF01bLW80Shn?05IFEP1KFe/0G5aL065QH@1;Bd/0
De=C055AD@1WIfL0BDU905EEE@0k>c/0De=C0410@01@D500K6a/03@d=01YJFT0@41005IFEP1;Bd/0
G5aL059BDP1GEeL0GEeM05ADE01MGEd0Hf=S07=cL`2BTY80C4a<05mOG`2?Shl0GUiN061PH01WIfL0
MWIf07=cL`1TI6@0>3Ph04Y:BP1hN7P0G5aL04=3@`1<C4`0MGEe05=CD`1;Bd/0/K6a0>7Qh@0i>CT0
ADE504=3@`1KFe/0K6a/08F5Q@2OWil0;2`/06IVIP0];Bd0=cLg01LG5`0i>CT2034a<@1307]kN`1<
C4`0EEEE09^KV`20P800B4Q806m_K`19BDT0EEEE05IFEP1<C4`0Bd];05iNGP1TI6@0BTY:08f=S@1Q
HF40DE5A05YJFP15ADD0N7Qh03Ti>@1GEeL0FUYJ06a/K013@d<0F5QH04a<C01IFET0@T9203Hf=P1?
Cdl07alO059BDP1DE5@0GEeM05mOG`2@T900@d=309FEU@1YJFT0ADE504I6AP0o?cl0OWin065QH@2<
S8`0RXZ:05=CD`1UIFD0CTi>04];B`1ADE40E5AD02LW9`15ADD0H61P05QHF01FEUH0Ogmo04=3@`1M
GEd0:BTY04A4A01=CDd0<30`04Q8B00204Y:BP1503lo?`0m?Cd0B4Q803lo?`11@D40A4A404=3@`1C
De<0G5aL03Ti>@1LG5`0IVIV04m?C`0Q8B40DU9B04Y:BP12@T80IfMW02`/;00l?3`0BDU903Xj>P0c
<c<0=SHf04Q8B00g=cL0?Cdm05]KF`1:BTX0GUiN04Q8B00P8200Bd];02LW9`1FEUH0GemO055AD@1W
IfL0CTi>05aLG017AdL0:b/[05EEE@1<C4`0KFe]051@D01dM7@0?clo051@D01WIfL0ATI607IfMP11
@D40?Cdm02d];@0e=CD0BDU906MWI`1MGEd0>3Ph03/k>`2IVIT0@T9203dm?@1PH600EeMG05aLG01c
Lg<0FEUI013ooooo000>ooooo`1E05UIF@1hN7P0DU9B05ADE01:BTX0Ng]k06EUI@12@T80E5AD06MW
I`2FUYH0@d=302`/;01ADE40ADE506=SH`2PX:00:b/[018B4P0i>CT0DU9B06MWI`2][Jd0;Rh^03De
=@1@D500IFEU092@T028R8P09RHV05=CD`1mOGd0CTi>0:n_[`1YJFT0>SXj04=3@`1dM7@0J6QX071`
L00b<S8092@T061PH01TI6@0CDe=0<?3``0f=SH07QhN03Hf=P25QHD0EeMG04m?C`0T92@0<S8b04E5
A@0K6a/0DE5A02d];@0k>c/0UYJF08V9R@0Y:BT0?clo01hN7P19BDT0WinO05IFEP0J6QX0Bd];04A4
A01^KVh0^;Rh05=CD`0K6a/0<c<c05QHF01VIVH0ZJVY02DU9@0`<300DE5A0492@P24Q8@0CDe=01/K
6`0204Q8B00J04m?C`1QHF40EUIF02@T9015ADD0@d=30451@@1_Kfl0IfMW02l_;`1HF5P0Fe]K05aL
G02JVYX0DE5A03@d=01QHF40GemO07QhN01fMWH06ATI030`<01CDe<0ATI605EEE@1oOgl202/[:`2S
08>3P`0e=CD0DU9B03Hf=P0/;2`0:BTY06=SH`1[Jf/0EEEE05QHF00U9BD0De=C08F5Q@27QhL0Fe]K
03lo?`0`<300Lg=c0<C4a01nOWh0JFUY03@d=00];Bd0MgMg08N7Q`0/;2`0@d=304E5A@25QHD0d];B
05]KF`0X:2P0A4A4065QH@1oOgl0];Bd0410@00e=CD0D51@05mOG`1cLg<0VIVI04A4A00a<C40C4a<
059BDP1LG5`0WYjN02<S8`0i>CT0FEUI07QhN018B4P0Lg=c030`<01`L700Zj^[069RHP1YJFT0SXj>
02XZ:P1?Cdl0Y:BT075aL@1dM7@0@D5101hN7P1YJFT0N7Qh05YJFP2LW9`0=SHf02/[:`2OWil0;Bd]
06UYJ@1OGel0ADE505iNGP2^[Zh0@T92061PH00b<S80>SXj09VIV@1oOgl0E5AD03De=@1;Bd/0VYZJ
079bLP0e=CD0EEEE04];B`2;Rh/0I6AT024Q8@1=CDd0@D5104M7A`1hN7P0>3Ph04U9B@1EEED0CDe=
07emO@1hN7P0=SHf05IFEP1]KFd0EEEE092@T02YZJT0;Rh^05QHF01WIfL0GUiN0:j^[P2FUYH0>SXj
055AD@1PH600LW9b0<O7a`1XJ6P0=CDe04U9B@1aLG40I6AT0:RXZ01TI6@0ADE5065QH@1]KFd0KVi^
09ZJVP1WIfL0=SHf06ATI02BTY80DE5A09bLW01]KFd0:b/[04];B`1jNWX0DE5A06=SH`1>CTh07QhN
03Lg=`1`L700LW9b05aLG01dM7@0>3Ph03<c<`1TI6@0Ti>C051@D00l?3`092@T055AD@0>ooooo`19
0861P@1VIVH0BTY:06ATI02:RXX0Lg=c04i>CP1MGEd0UYJF051@D01>CTh0IFEU06e]K@1WIfL0MWIf
0861P@1[Jf/0KFe]05UIF@13@d<0De=C05]KF`16ATH0EeMG05IFEP1YJFT0<c<c04m?C`2SXj<0IfMW
055AD@1hN7P0QHF505eMG@1TI6@0L71`07AdM01=CDd0DU9B04e=C@18B4P0Kfm_07QhN01VIVH0GEeM
03dm?@2BTY80CTi>03@d=018B4P0BTY:02<S8`0_;bl0ADE505=CD`2UYJD0Cdm?05MGE`0c<c<0EeMG
04e=C@1<C4`0JVYZ07AdM02MWId0MGEe04];B`1=CDd0K6a/061PH01;Bd/0BDU906a/K002030`<01h
061PH01QHF40>c/k03De=@1:BTX0?Cdm03/k>`1;Bd/0G5aL03/k>`13@d<0=3@d04=3@`1=CDd0BDU9
05iNGP1>CTh0DE5A03hn?P1<C4`0Cdm?0410@01<C4`0F5QH05UIF@1ADE40EEEE06=SH`1KFe/0VIVI
06QXJ01dM7@0HV9R05iNGP1aLG40Xj>S03`l?01lO7`0SHf=04];B`1=CDd0JVYZ034a<@12@T80?Cdm
061PH012@T80Shn?08Z:RP2QXJ40DE5A04U9B@1iNGT0KVi^08^;R`2=SHd0bLW905IFEP1lO7`0?clo
03Ph>00I6AT0=SHf03@d=00`<300>c/k030`<01JFUX0];Bd06QXJ011@D40J6QX03`l?011@D40EUIF
034a<@0Q8B40GEeM0:RXZ00e=CD0N7Qh03dm?@1MGEd0JVYZ03Ti>@1?Cdl0;2`/05IFEP0c<c<0KVi^
03<c<`1VIVH0BDU902l_;`0a<C40?Shn05IFEP0b<S80IVIV04i>CP1HF5P0DE5A09FEU@0k>c/0MGEe
05eMG@1KFe/0LG5a04A4A01WIfL0<c<c04];B`1?Cdl0LW9b06=SH`1DE5@0EUIF04m?C`15ADD0>SXj
0P12@T80D@1;Bd/0CDe=04i>CP0G5aL0<C4a01DE5@0e=CD0CTi>020P800n?Sh0CDe=04Q8B01>CTh0
>CTi05QHF01ADE40ADE5059BDP13@d<0PH6108:2PP1<C4`0GUiN04e=C@1CDe<071`L03Hf=P0`<300
>c/k06i^KP1=CDd0B4Q804A4A00k>c/0ATI605mOG`1LG5`0ATI606e]K@2MWId0?clo020P800M7Ad0
:b/[04E5A@0[:b/0LW9b075aL@16ATH0Bd];03Ti>@14A4@0G5aL03<c<`1GEeL0ATI60;Jf]P1ADE40
JVYZ04M7A`1oOgl0Ng]k03Hf=P10@400>3Ph04];B`0c<c<0@41004=3@`1UIFD0HF5Q0410@01JFUX0
d=3@05iNGP1CDe<0QHF506IVIP19BDT0NGUi061PH00@ooooo`003_ooool08@1HF5P0Q8B408:2PP24
Q8@0@d=304i>CP29RHT0UiNG09RHV00b<S80I6AT05iNGP1XJ6P0V9RH04=3@`0`<300Lg=c06i^KP1Q
HF40Hf=S02TY:@0g=cL0Bd];06MWI`1`L700>c/k03@d=01MGEd0IfMW038b<P1MGEd0;Rh^03Xj>P02
04m?C`1:02<S8`19BDT0;bl_05YJFP1oOgl0>SXj020P800i>CT0@d=306i^KP2;Rh/0ATI601DE5@0Z
:RX0>3Ph08b<S02FUYH0;2`/0492@P1OGel0>SXj09:BTP0l?3`0<30`03Xj>P1MGEd0=SHf07inOP0J
6QX0;2`/05mOG`1aLG40Cdm?0=GEe@1GEeL0Cdm?07MgM`2HV9P0HF5Q08b<S00N7Qh0B4Q806m_K`1Y
JFT0<S8b05=CD`0T92@0D51@092@T00g=cL0HV9R03Lg=`0D51@0?3`l07QhN011@D40MGEe03<c<`0F
5QH0D51@09JFUP0a<C40@D5102PX:00_;bl0MWIf07]kN`0h>3P0;2`/02DU9@1DE5@0XZ:R04M7A`0M
7Ad204m?C`2306e]K@1iNGT07QhN01/K6`1JFUX0>c/k07YjNP1PH60071`L02LW9`11@D40Jf][05mO
G`1mOGd07AdM024Q8@1IFET0Ogmo07inOP1SHf<06ATI0492@P1jNWX0T92@09ZJVP0j>SX0?clo051@
D01QHF40BDU907=cL`1<C4`0;Rh^06ATI01HF5P0Cdm?06EUI@0Z:RX0DE5A07moO`1]KFd0IFEU0:j^
[P1>CTh0>c/k07MgM`1YJFT0>c/k0861P@0j>SX0;bl_07inOP1_Kfl0DU9B071`L00W9bL0B4Q806AT
I00m?Cd0EUIF07]kN`0O7al0EUIF0:JVYP1;Bd/0DE5A059BDP0[:b/0QHF508N7Q`0S8b<0?clo04A4
A013@d<0W9bL06MWI`0N7Qh0BTY:03dm?@1nOWh0FUYJ02@T900i>CT0IVIV05MGE`2<S8`09RHV038b
<P1JFUX0G5aL09nOW`0_;bl0ADE505eMG@1hN7P0Ph>3092@T012@T80Lg=c08:2PP1ADE40ZJVY075a
L@0O7al0P82006QXJ02>SXh0RXZ:02h^;P13@d<0QhN706QXJ01cLg<0DE5A02h^;P1HF5P0XZ:R075a
L@1oOgl0=SHf03Ph>01[Jf/0X:2P04i>CP1CDe<0;bl_04Q8B01VIVH0UIFE04Y:BP1IFET00P0d=3@0
8P2RXZ80]KFe03Xj>P0k>c/0C4a<03/k>`2ATI40[Jf]03@d=00a<C40J6QX04=3@`2<S8`0VYZJ030`
<00Y:BT0DE5A05EEE@2BTY80QXJ601/K6`0e=CD0@T9205IFEP1[Jf/0Z:RX02HV9P0o?cl0DU9B07Ee
M@28R8P0Q8B401XJ6P0g=cL>ooooo`2M075aL@2DU9@0FUYJ05iNGP1<C4`0MgMg07UiN@1SHf<0K6a/
04m?C`1ZJVX0MGEe07MgM`1lO7`0Xj>S0;Vi^@23Ph<0E5AD06UYJ@15ADD0?Cdm03Ti>@1CDe<0CTi>
05mOG`1eMGD0C4a<05YJFP1<C4`0E5AD069RHP1?Cdl0EeMG065QH@1]KFd0De=C04M7A`16ATH0=cLg
04];B`1CDe<0J6QX03/k>`10@400BDU90410@017AdL0?Shn04A4A013@d<0AdM702d];@16ATH0@410
03hn?P13@d<0;Bd]03Xj>P0d=3@0>CTi03Ph>00g=cL0IfMW06EUI@2;Rh/0S8b<092@T02OWil0IVIV
06MWI`1eMGD0LW9b06e]K@0Y:BT0=CDe05=CD`14A4@0@41003Ph>017AdL0DE5A03Lg=`1ADE40I6AT
03`l?00i>CT0<30`05MGE`0Z:RX0=SHf05YJFP0g=cL0DE5A03Ph>01KFe/0>c/k03dm?@1FEUH0Cdm?
04a<C01KFe/0CDe=05EEE@1UIFD0ZZZZ04M7A`1KFe/0NGUi07AdM01ADE40IFEU04A4A020P800Q8B4
05eMG@1QHF40L71`05UIF@1OGel0>c/k06][J`1DE5@0RXZ:09RHV02l_;`0E5AD05iNGP1cLg<0EeMG
06m_K`2UYJD0e]KF05mOG`1TI6@0K6a/05ADE00X:2P0=SHf04E5A@16ATH0E5AD04a<C01RHV80PH61
05eMG@14A4@0HV9R04m?C`13@d<0Kfm_05ADE00l?3`0?Cdm07EeM@0c<c<0QXJ60410@00204U9B@0V
030`<01HF5P08b<S04=3@`0f=SH0JVYZ04A4A01JFUX0ATI605aLG01EEED0Kfm_07MgM`1:BTX0A4A4
04U9B@1IFET0M7Ad0;>c/`11@D40QXJ605UIF@0i>CT0?3`l0410@01gMgL0=SHf06ATI02?Shl0SXj>
061PH02?Shl0EEEE05QHF01IFET0=3@d05mOG`1LG5`206IVIP0605aLG00B4Q80Cdm?01dM7@15ADD0
DU9B0P1LG5`0301`L700F5QH051@D00o?cl0>c/k055AD@0g=cL0=SHf03hn?P1=CDd0ADE503Lg=`80
I6AT03T0GUiN04Y:BP1EEED0F5QH065QH@20P800B4Q804Y:BP1[Jf/0De=C04U9B@10@400>3Ph04=3
@`0b<S80Bd];03Ti>@19BDT0@41002<S8`19BDT0>3Ph0492@P1DE5@0@d=305IFEP0i>CT0HV9R07]k
N`1;Bd/0De=C03`l?01BDU80Fe]K07=cL`10@400]KFe079bLP0n?Sh0GEeM04Y:BP11@D40>3Ph04Q8
B01DE5@0Ti>C092@T01EEED0FEUI07EeM@1TI6@0FEUI05EEE@1/K6`0Ng]k096AT@2EUID04?ooool0
00kooooo0?l0IVIV07MgM`15ADD0SXj>04=3@`1QHF40>c/k059BDP2ATI40=CDe04Q8B014A4@0>c/k
0:>SX`1TI6@08b<S065QH@17AdL0GemO09BDU00/;2`0=3@d02XZ:P12@T80]kNg065QH@0F5QH09bLW
04e=C@1OGel0`<3002XZ:P0Q8B40B4Q807alO01:BTX0T92@01dM7@0Z:RX0HV9R07alO01NGUh0VYZJ
055AD@0/;2`0IVIV05QHF00k>c/0Rh^;03Hf=P0l?3`0PH6104I6AP1_Kfl0@D5102DU9@1]KFd0GEeM
04Q8B01/K6`0Kfm_028R8P1/K6`0>c/k03`l?01QHF40;Bd]02/[:`1EEED0E5AD02TY:@1<C4`0=3@d
06][J`21PH40BDU902/[:`0i>CT0BDU906IVIP2@T900:RXZ02TY:@19BDT0EUIF08>3P`1bLW808B4Q
020P801?Cdl0@d=30:>SX`0f=SH071`L02d];@1kNg/0AdM707QhN00C4a<0:BTY04I6AP1dM7@0NWYj
05YJFP0J6QX0@41007=cL`1PH600ADE505mOG`0R8R80De=C0:2PX01gMgL0BTY:0492@P0j>SX0BTY:
0:f][@1FEUH0@41007YjNP0e=CD0@T9207=cL`1ADE40EeMG079bLP0`<300CTi>06i^KP0o?cl0K6a/
04I6AP0S8b<0O7al05mOG`0[:b/0C4a<055AD@0`<300YJFU08^;R`0o?cl0GemO0410@01NGUh0_Kfm
04M7A`0k>c/0GUiN04Q8B01SHf<0_kno071`L00e=CD0LW9b05ADE01IFET0];Bd07MgM`0c<c<0Fe]K
06UYJ@1QHF40SXj>03Ti>@0Z:RX0RHV907YjNP2@T900^k^k038b<P0/;2`0LW9b06YZJP21PH40De=C
01lO7`0h>3P0MWIf04M7A`1jNWX06a/K020P801VIVH0TY:B04Y:BP1SHf<0;bl_04=3@`1ZJVX0A4A4
05eMG@15ADD0?Shn07IfMP1>CTh0ADE507IfMP0m?Cd0GUiN07inOP1UIFD0PX:202DU9@1ADE40YjNW
05aLG01/K6`0C4a<03`l?01aLG40Y:BT04Q8B01JFUX0BTY:04e=C@1nOWh0ATI603Ph>00o?cl0IFEU
0;fm_@2JVYX0:b/[03Ph>01DE5@0G5aL0;>c/`1^KVh07alO03Ti>@1eMGD0LW9b0:RXZ00a<C40;Rh^
04A4A01^KVh0JVYZ09VIV@0a<C409RHV04E5A@27QhL0D51@05YJFP1IFET0=CDe06i^KP2OWil0KVi^
04m?C`1FEUH001L0;Rh^03/k>`1EEED0?Cdm05=CD`2RXZ80=SHf04e=C@1/K6`0?3`l04m?C`1PH600
8b<S02l_;`1aLG40BTY:02/[:`1CDe<07AdM05QHF01fMWH0=CDe04Y:BP0>ooooo`0D06e]K@1QHF40
DU9B059BDP1HF5P0G5aL04=3@`1MGEd0Ph>304I6AP1ADE40M7Ad09NGU`1dM7@0FUYJ05IFEP1XJ6P0
MGEe04E5A@0k>c/205aLG00K04I6AP1UIFD0Lg=c05=CD`0o?cl0De=C0:BTY01LG5`0EeMG071`L02@
T900MWIf05aLG01fMWH0GUiN04A4A00m?Cd0AdM704U9B@1EEED0CTi>05iNGP1VIVH0@D5105EEE@1;
Bd/0F5QH0080D51@09H09bLW06UYJ@1IFET0>3Ph05YJFP0i>CT0IFEU028R8P16ATH0D51@03/k>`1=
CDd0H61P09JFUP1`L700;2`/02l_;`18B4P0GUiN030`<00X:2P0Jf][03Ph>00o?cl0K6a/051@D01?
Cdl0?Shn04a<C00a<C40@41003hn?P13@d<0B4Q803Ti>@0W9bL0<C4a03`l?00d=3@0@D5103dm?@14
A4@0820P02TY:@0m?Cd0<c<c03Hf=P1CDe<0EeMG03De=@0b<S80DU9B03Ph>00i>CT0C4a<06m_K`1H
F5P0BTY:05]KF`1`L700@T9206][J`1RHV80B4Q805eMG@24Q8@0ATI604Y:BP17AdL0Shn?05=CD`1Q
HF40[Jf]0;Vi^@1[Jf/0QHF504];B`2;Rh/0Ti>C06i^KP2NWYh0EEEE08n?S`1IFET0H61P065QH@1b
LW80Kfm_04M7A`1ADE40Ng]k07EeM@2b/[80MgMg04m?C`1WIfL0DU9B05YJFP24Q8@0Lg=c06m_K`1I
FET0GemO05MGE`23Ph<0FUYJ05iNGP1PH600?Shn05IFEP0`<300D51@0410@01SHf<0@41005]KF`1B
DU80FEUI02h^;P0h>3P0?3`l02DU9@18B4P0Bd];06MWI`0l?3`0IfMW06ATI02ATI40Hf=S03`l?016
ATH0GEeM07]kN`0i>CT0S8b<0:^[Z`2HV9P0I6AT05eMG@1SHf<0EEEE0410@011@D40M7Ad05UIF@1F
EUH0V9RH09JFUP18B4P205=CD`1;075aL@2ATI40F5QH05mOG`12@T80CDe=05QHF015ADD0AdM7069R
HP1=CDd0ADE5051@D01BDU80FUYJ0451@@1ZJVX0I6AT07IfMP0n?Sh0DU9B04U9B@0j>SX0F5QH04I6
AP0m?Cd0C4a<02`/;012@T80=CDe04I6AP1GEeL0ADE505ADE017AdL0Bd];05EEE@1MGEd0M7Ad00d=
3@0j>SX0Ogmo04=3@`1aLG40C4a<03dm?@1GEeL0L71`08F5Q@1IFET0M7Ad07EeM@20P800EeMG06m_
K`26QXH0CTi>069RHP1KFe/0QHF504E5A@1<C4`0;Bd]04M7A`1GEeL0FEUI04m?C`2c/k<0GEeM05IF
EP1XJ6P0I6AT05EEE@1^KVh0GEeM013ooooo000>ooooo`1j06QXJ02YZJT0FUYJ065QH@1DE5@0RXZ:
06][J`1<C4`0FEUI05ADE02=SHd0DU9B038b<P1KFe/0GUiN03lo?`2<S8`0;bl_03@d=01IFET0IFEU
04m?C`1iNGT0:b/[0492@P1TI6@0BTY:06EUI@1VIVH0;bl_05MGE`1nOWh0AdM707QhN01KFe/06a/K
04U9B@1;Bd/0?Shn08Z:RP0Z:RX05QHF0492@P1EEED0CTi>07=cL`0[:b/0>c/k08J6QP1MGEd0Hf=S
03dm?@0k>c/0J6QX05iNGP0V9RH0AdM703@d=013@d<0RHV906i^KP0[:b/0AdM703@d=01MGEd0TI6A
04E5A@0X:2P0D51@05]KF`1VIVH0Shn?0410@00c<c<0A4A403dm?@1XJ6P0TI6A02DU9@0a<C40DU9B
0451@@1ZJVX0=cLg01/K6`0n?Sh0FUYJ069RHP1QHF40MWIf02DU9@0h>3P0G5aL03`l?01dM7@0CTi>
04U9B@1hN7P0BTY:03Hf=P2;Rh/0?clo0492@P2;Rh/0>SXj05ADE01[Jf/092@T0492@P1fMWH0;bl_
06YZJP1`L700@d=304];B`1bLW80=cLg05QHF01LG5`0@d=3055AD@26QXH305EEE@2I03dm?@1ADE40
NGUi07]kN`1:BTX0EUIF03@d=01bLW80R8R803De=@1DE5@0ATI604e=C@1iNGT0JFUY02l_;`1GEeL0
Lg=c071`L023Ph<0=CDe03Xj>P1fMWH0Hf=S07YjNP1kNg/0>SXj038b<P1=CDd0VIVI05eMG@2KVi/0
?3`l02d];@1@D500MgMg04A4A01]KFd0:BTY03hn?P1HF5P0GEeM05aLG01ZJVX09BDU03hn?P1`L700
H61P07EeM@1dM7@09bLW05IFEP23Ph<0DE5A06=SH`1:BTX0:BTY071`L01FEUH0=CDe05mOG`0e=CD0
?clo08^;R`0k>c/0PX:204a<C012@T80MWIf08>3P`0h>3P0JVYZ03Xj>P1GEeL0PH6104=3@`1DE5@0
>SXj06QXJ028R8P0C4a<04];B`1dM7@0E5AD0:^[Z`1OGel0<C4a05IFEP1QHF40;Bd]08V9R@19BDT0
De=C06a/K01BDU80RHV907alO00^;Rh0H61P08>3P`1JFUX0Kfm_08j>SP14A4@0FUYJ08^;R`1/K6`0
Lg=c079bLP12@T80Hf=S07QhN01EEED0UYJF0492@P0`<300E5AD05mOG`19BDT0Jf][05=CD`0e=CD0
F5QH07alO00h>3P0EUIF05=CD`0c<c<0DE5A061PH012@T80JVYZ03`l?00k>c/0E5AD08b<S01GEeL0
GemO04M7A`11@D40J6QX08N7Q`1JFUX0DU9B04i>CP0h>3P0Cdm?0820P01<C4`0@D5105YJFP0[:b/0
De=C00kooooo0300L71`06a/K01>CTh0G5aL06QXJ01cLg<0CDe=05UIF@1gMgL0EUIF051@D020P800
N7Qh07YjNP2IVIT0/[:b08F5Q@1BDU80Hf=S03Ph>012@T80>SXj04e=C@1BDU80EEEE06][J`14A4@0
IVIV05YJFP1LG5`0H61P069RHP1kNg/0IfMW05mOG`1VIVH0F5QH04];B`1?Cdl0DE5A04];B`1NGUh0
EUIF05EEE@1;Bd/0Cdm?07=cL`15ADD204=3@`0Z04a<C00U9BD0>c/k0451@@14A4@0L71`02h^;P1D
E5@0=CDe06YZJP1=CDd0?3`l05aLG01RHV80PX:2069RHP1PH600L71`06QXJ01`L700ADE505IFEP1R
HV80?Shn03Ph>01@D500EUIF04E5A@0d=3@0>c/k04M7A`0/;2`0@41004Y:BP0e=CD0:RXZ02`/;016
ATH0>3Ph030`<018B4P0<c<c0P0d=3@0F@1?Cdl0?3`l024Q8@1=CDd0@d=30451@@18B4P0A4A404U9
B@1FEUH0RHV905ADE01FEUH0EEEE05mOG`1ADE40JFUY038b<P1VIVH0H61P05ADE01KFe/0DE5A03lo
?`1DE5@0<S8b051@D00d=3@0L71`08N7Q`1bLW80De=C05eMG@11@D40>c/k05aLG01oOgl0/k>c05EE
E@1DE5@0>CTi04I6AP0h>3P0ATI6069RHP1`L700OGem079bLP2@T900alO70;Jf]P25QHD0NWYj07Ui
N@1/K6`0QhN708j>SP20P800GUiN075aL@1HF5P0S8b<05=CD`1^KVh0Kfm_04Q8B01YJFT0A4A406MW
I`1=CDd0LW9b04a<C01HF5P0Bd];06i^KP13@d<0Cdm?06QXJ011@D40D51@061PH01GEeL0Hf=S09:B
TP1@D500QXJ6071`L018B4P0BTY:0080D51@00D0ATI608^;R`2PX:00S8b<06=SH`0205QHF00K05iN
GP1<C4`0>SXj04i>CP1LG5`0GEeM06][J`20P800B4Q804a<C00^;Rh0CDe=069RHP0`<300EeMG04Q8
B01:BTX0B4Q804A4A01>CTh0H61P0492@P1;Bd/0?clo06EUI@1MGEd0?Shn0080IFEU00X0GemO04A4
A01SHf<0ATI60492@P1RHV80:2PX03Ph>01@D500=CDe0P1:BTX0;@1KFe/0BDU904e=C@1FEUH0ATI6
04Q8B00n?Sh0?3`l04];B`0e=CD0EeMG09BDU01>CTh0KVi^04m?C`1@D500GUiN04a<C01]KFd0AdM7
07EeM@1]KFd0YJFU05eMG@24Q8@0CTi>05UIF@23Ph<0DU9B05EEE@17AdL0HF5Q03hn?P1ADE40IFEU
05QHF01VIVH0Xj>S06ATI01LG5`0L71`06EUI@1EEED0IFEU079bLP0@ooooo`003_ooool0d@1?Cdl0
Q8B406UYJ@1fMWH0>3Ph05UIF@23Ph<0LW9b07inOP0g=cL0CDe=08b<S01VIVH0^KVi030`<00V9RH0
H61P06YZJP1gMgL0FEUI02/[:`0^;Rh0Bd];05eMG@23Ph<0:2PX02`/;01?Cdl0G5aL04a<C01cLg<0
7AdM03dm?@1OGel0QHF5051@D01VIVH0820P03/k>`1kNg/0QXJ6034a<@15ADD0>SXj03Lg=`1gMgL0
AdM702`/;011@D40<S8b07MgM`2:RXX0:BTY03Ti>@12@T80BDU908j>SP0h>3P0820P059BDP1cLg<0
HV9R08V9R@0N7Qh0:2PX05MGE`1JFUX0F5QH0:f][@1GEeL0<C4a06YZJP22PX80Q8B40:n_[`0L71`0
Cdm?0861P@1WIfL0IFEU05EEE@0?3`l0K6a/08J6QP13@d<0PX:20492@P0^;Rh0<c<c06a/K01KFe/0
XZ:R024Q8@0Y:BT0B4Q805=CD`11@D40@T9201TI6@0_;bl0NWYj06IVIP0k>c/0<c<c02h^;P0l?3`0
NGUi04A4A00L71`0@T9203<c<`1KFe/0Ng]k02HV9P0U9BD0@T92030`<01]KFd0SHf=02l_;`0^;Rh0
C4a<04M7A`1gMgL0Ti>C02PX:00a<C40EEEE07QhN01lO7`0MGEe01lO7`0h>3P0C4a<05=CD`2?Shl0
92@T03Ph>01YJFT0K6a/05=CD`2WYjL0BTY:04Q8B020P800Lg=c06YZJP24Q8@0?Cdm069RHP2a/K40
Hf=S07AdM02@T900DE5A051@D01dM7@0Jf][06EUI@2CTi<0>CTi04Y:BP21PH40E5AD051@D02:RXX0
7alO03dm?@2=SHd0MgMg0861P@1[Jf/08b<S04m?C`2h^;P0Lg=c06IVIP1PH600820P05iNGP1hN7P0
8b<S04e=C@0l?3`0Jf][09jNWP18B4P0:BTY04I6AP10@400U9BD09ZJVP0Z:RX0ADE504m?C`1UIFD0
XZ:R03dm?@0i>CT0E5AD04Q8B02OWil0<C4a03Ph>01HF5P0FUYJ07alO02LW9`0:BTY0080NWYj04<0
LW9b0:BTY01OGel06QXJ05QHF01@D500NGUi07YjNP0`<300<S8b08:2PP1UIFD0OWin05YJFP0m?Cd0
F5QH09:BTP1VIVH0OGem04A4A01ADE40IVIV08J6QP1FEUH0AdM7034a<@1@D500M7Ad08V9R@1:BTX0
PH6102XZ:P0i>CT0C4a<06IVIP13@d<0Hf=S04E5A@0b<S80KVi^05]KF`0R8R80ATI603/k>`1PH600
]kNg05]KF`0O7al0;bl_05ADE01=CDd0[Jf]06e]K@0R8R80?Cdm04M7A`0e=CD0KFe]07MgM`0G5aL0
>SXj04I6AP0l?3`0MWIf06MWI`0O7al0A4A400kooooo02L0L71`06m_K`1`L700GEeM06IVIP1aLG40
IVIV06UYJ@2GUiL0Cdm?04i>CP1UIFD0SHf=06QXJ01]KFd0KVi^06e]K@1HF5P0D51@04=3@`1;Bd/0
?Cdm04U9B@1RHV80J6QX05iNGP1<C4`0I6AT08b<S016ATH0BDU905ADE01cLg<0CTi>055AD@1IFET0
G5aL03`l?01<C4`00P17AdL0JP1>CTh0ADE505ADE01?Cdl0D51@05QHF01=CDd0DU9B04A4A01CDe<0
ATI604U9B@1<C4`0AdM706UYJ@1;Bd/0K6a/04=3@`1<C4`0DU9B04e=C@1KFe/0IfMW0;Fe]@1cLg<0
<S8b03Xj>P1QHF40MWIf04Q8B00o?cl0KFe]0492@P1:BTX0K6a/05IFEP1HF5P0>c/k04Y:BP1<C4`0
ATI604];B`19BDT0=cLg04I6AP11@D40Cdm?03@d=00h>3P0CTi>030`<00h>3P0<c<c04U9B@0m?Cd0
6ATI059BDP18B4P0C4a<03Hf=P10@400BDU9059BDP2>SXh0Cdm?05ADE01MGEd0BTY:069RHP1mOGd0
De=C08>3P`1QHF40Bd];05MGE`1_Kfl0<30`04];B`0k>c/0QhN702l_;`1WIfL0[Zj^0:2PX017AdL0
E5AD02h^;P11@D40HV9R07AdM02SXj<0E5AD08j>SP1=CDd0DU9B0410@01IFET0CTi>05EEE@21PH40
DU9B07UiN@2=SHd0E5AD0861P@1RHV82051@D080Kfm_07l0IVIV04Y:BP1OGel0JVYZ04a<C015ADD0
ATI60492@P17AdL0EeMG0410@01DE5@0@d=305QHF01PH600Q8B405iNGP1UIFD0DU9B04U9B@1WIfL0
C4a<06EUI@1QHF40GUiN06e]K@2CTi<0DU9B08R8R01HF5P0@D5105=CD`1>CTh0FEUI09>CT`33`l<0
/k>c07MgM`1IFET0AdM704U9B@1SHf<0BDU903Ph>01DE5@0CDe=04Q8B01cLg<0UYJF04I6AP1BDU80
<S8b04E5A@1KFe/0<c<c06=SH`0j>SX0EEEE04i>CP0m?Cd0?3`l07]kN`15ADD0Bd];04U9B@1GEeL0
HF5Q03`l?01hN7P0GemO055AD@0l?3`0KVi^03Xj>P1>CTh0G5aL0451@@1:BTX0EUIF04I6AP1JFUX0
@41003dm?@1HF5P0ADE505YJFP19BDT0?clo059BDP11@D40JFUY01LG5`1MGEd0RHV905UIF@1_Kfl0
C4a<04m?C`1<C4`0IVIV09jNWP1ZJVX0EUIF04e=C@1eMGD0G5aL08n?S`1`L700A4A406][J`1?Cdl0
Jf][065QH@2;Rh/0@T9205mOG`1FEUH0ATI6075aL@2k^k/0IFEU05aLG01mOGd0EeMG03lo?`1?Cdl0
Fe]K013ooooo000>ooooo`3906i^KP1aLG40EEEE07IfMP1NGUh0GemO04A4A01JFUX0PX:204=3@`1?
Cdl0>SXj04I6AP2[Zj/0FUYJ02LW9`0n?Sh0<S8b08V9R@1mOGd0>c/k02TY:@11@D40AdM708F5Q@1[
Jf/0820P03/k>`1DE5@0JVYZ0:^[Z`11@D404Q8B04E5A@1cLg<0?3`l09NGU`0E5AD05QHF04A4A01e
MGD0ADE507IfMP1EEED0EeMG06m_K`1@D500@d=307AdM00];Bd0@T9205]KF`0o?cl0EEEE02l_;`0N
7Qh0G5aL06ATI013@d<0QHF505ADE00^;Rh0J6QX0410@014A4@0Hf=S02<S8`0`<300B4Q805MGE`1K
Fe/0EUIF02`/;01:BTX0F5QH0492@P0d=3@0>3Ph03@d=02JVYX0YjNW03lo?`0b<S807alO02/[:`1c
Lg<0EEEE01HF5P0a<C40@41004m?C`2>SXh0GemO01dM7@13@d<0DU9B09JFUP2=SHd05ADE024Q8@12
@T80ATI60820P01DE5@09BDU02l_;`10@400LW9b05iNGP1MGEd092@T03Ti>@24Q8@0LW9b06YZJP1L
G5`0A4A403`l?01MGEd0JVYZ07MgM`1nOWh0<S8b055AD@1=CDd0FEUI0410@01dM7@0;bl_04U9B@26
QXH0CDe=05EEE@18B4P061PH04E5A@1^KVh0<S8b0861P@16ATH0;Rh^06e]K@1<C4`0E5AD03`l?00k
>c/0ATI608N7Q`14A4@0@d=303`l?012@T80@41007alO01ZJVX0<c<c059BDP11@D40E5AD0:NWY`1/
K6`08b<S06ATI01ZJVX0HF5Q0;Bd]014A4@06a/K05eMG@1FEUH0ZJVY07alO00V9RH0820P04];B`1e
MGD0TY:B06i^KP0E5AD0@d=306ATI01CDe<0XJ6Q03Ti>@0_;bl0Ti>C08J6QP1;Bd/0GUiN02XZ:P0m
?Cd0PX:205mOG`27QhL0MgMg03Xj>P1nOWh0EEEE07EeM@1IFET0<C4a0080GEeM04/0KFe]04=3@`0M
7Ad0ADE506][J`1KFe/0Ph>30451@@0g=cL0O7al0:FUY@12@T80FUYJ03/k>`1GEeL0T92@04a<C01F
EUH0De=C05IFEP2FUYH0UiNG0451@@19BDT0GEeM06YZJP2RXZ80J6QX02/[:`1QHF40EEEE06YZJP2N
WYh0>c/k030`<01YJFT0EeMG051@D02GUiL0<S8b02l_;`14A4@0Jf][055AD@1fMWH0FEUI02XZ:P14
A4@0IFEU02l_;`1RHV80<C4a03Xj>P1NGUh0FEUI02TY:@15ADD0De=C03<c<`1KFe/0TY:B03Hf=P1B
DU80HV9R02DU9@0e=CD0KFe]034a<@1MGEd0DU9B01/K6`1GEeL0XZ:R04U9B@1UIFD03_ooool0T01P
H600MgMg04=3@`19BDT0DU9B07YjNP1?Cdl0JVYZ09ZJVP1HF5P0B4Q809RHV02EUID0Shn?08b<S02O
Wil0SHf=06i^KP1MGEd0?Cdm04m?C`1BDU80@d=3051@D01PH600GEeM04U9B@1_Kfl0IFEU05MGE`1S
Hf<0LG5a08R8R01PH600F5QH06e]K@1CDe<0>CTi04];B`19BDT0>SXj05=CD`18B4P0Bd];04I6AP1E
EED0FUYJ0492@P0c<c<0B4Q805=CD`0X:2P0820P03hn?P1>CTh0JFUY02h^;P15ADD0<S8b06UYJ@1;
Bd/0<c<c059BDP1NGUh0HV9R05UIF@16ATH0BTY:06IVIP1HF5P0?3`l04Q8B01EEED08R8R02<S8`11
@D40EEEE03lo?`0d=3@0@D5104];B`0b<S80Cdm?0451@@0c<c<0<S8b04Y:BP1@D500@D5104=3@`1=
CDd0?Shn03/k>`1@D500Fe]K03hn?P0b<S80HF5Q05ADE01GEeL0Cdm?05UIF@1KFe/0MWIf09NGU`1T
I6@0G5aL06][J`1MGEd0K6a/06EUI@17AdL0Lg=c07QhN01JFUX0FEUI065QH@15ADD0DU9B03De=@1;
Bd/0;Bd]07IfMP1mOGd0Q8B404U9B@1EEED0<30`02@T9013@d<0LW9b0;fm_@13@d<0Fe]K03lo?`1Y
JFT0BDU905mOG`1PH600EUIF06EUI@1CDe<0J6QX096AT@80EEEE01P0Cdm?05iNGP1;Bd/0PX:209:B
TP1LG5`0EEEE07IfMP1UIFD0/K6a079bLP1WIfL0P82005MGE`1OGel0DE5A06YZJP1DE5@0M7Ad04m?
C`17AdL0>c/k06=SH`0l?3`206e]K@0_05=CD`1<C4`0Jf][06IVIP1^KVh0VIVI05aLG02@T900KVi^
05aLG01UIFD0CDe=05MGE`1hN7P0De=C06YZJP1NGUh0De=C04U9B@0e=CD0GUiN05aLG00m?Cd0@D51
07QhN01;Bd/0De=C09JFUP19BDT0E5AD02<S8`1;Bd/0EUIF02HV9P1=CDd0A4A40492@P0e=CD0?Cdm
03De=@14A4@0<C4a03`l?00k>c/0DE5A05YJFP17AdL00P1VIVH0>@1SHf<0>c/k05IFEP1=CDd0A4A4
069RHP0Z:RX0=CDe05iNGP0b<S80CTi>05EEE@1FEUH0?3`l05YJFP1ZJVX0GUiN05eMG@0l?3`0@d=3
05ADE00j>SX0ADE506UYJ@10@400Jf][07AdM01LG5`0Cdm?05]KF`2<S8`0C4a<05aLG01VIVH0^KVi
05aLG01cLg<0<30`07=cL`2YZJT0FEUI03Xj>P0_;bl0@D51034a<@1RHV80G5aL0492@P18B4P0QHF5
04I6AP1JFUX0E5AD06ATI01ADE40De=C08>3P`0@ooooo`003_ooool0_`1hN7P0OWin055AD@1CDe<0
K6a/08F5Q@11@D40>3Ph069RHP1QHF40LG5a030`<00Z:RX0IFEU07moO`1eMGD0YJFU024Q8@15ADD0
OWin06ATI01ZJVX0UiNG038b<P1MGEd0TI6A04e=C@27QhL0HV9R02d];@1hN7P0VYZJ05=CD`2DU9@0
GUiN02h^;P1FEUH0?3`l02/[:`1OGel092@T01DE5@0Y:BT0@T9202XZ:P16ATH03`l?02LW9`0m?Cd0
AdM703@d=00h>3P0EEEE09NGU`1?Cdl05aLG038b<P0a<C40H61P0;Jf]P0h>3P0:BTY03dm?@0j>SX0
K6a/0:j^[P0Q8B40<S8b04A4A01GEeL0UiNG0;Bd]00K6a/0<30`04M7A`1GEeL0MWIf07moO`0S8b<0
<S8b08Z:RP1XJ6P0IfMW02@T901;Bd/0E5AD07inOP1:BTX0JVYZ04e=C@0h>3P0HF5Q061PH00g=cL0
J6QX03De=@11@D40Ph>3051@D01EEED0JVYZ02DU9@14A4@0MGEe03Ph>017AdL0G5aL018B4P12@T80
VYZJ034a<@1BDU80B4Q802<S8`17AdL0QXJ6024Q8@0o?cl0BDU902HV9P1CDe<0YZJV04A4A01=CDd0
AdM7059BDP1CDe<0^k^k07EeM@0Z:RX0C4a<075aL@28R8P0FUYJ01lO7`0j>SX0ATI604];B`2NWYh0
@D5103@d=01eMGD0DE5A08B4Q02EUID07alO04e=C@1XJ6P0TY:B0:^[Z`1LG5`071`L04e=C@1UIFD0
KVi^07EeM@1fMWH09BDU04a<C01ADE40OGem05]KF`1QHF40;Rh^04e=C@28R8P0Ti>C05aLG010@400
<S8b05]KF`2IVIT0LG5a0410@01NGUh09BDU05mOG`2<S8`0@4100492@P0];Bd09bLW05QHF00d=3@0
<30`05]KF`0];Bd0DU9B08:2PP0/;2`0CDe=0080>3Ph05D0C4a<071`L00];Bd0@d=304E5A@2ATI40
Rh^;03@d=018B4P0CDe=0;Fe]@2XZ:P0:RXZ02`/;014A4@0D51@0<[:bP1SHf<0:BTY05UIF@2LW9`0
DU9B09fMW@0b<S80=3@d07AdM01]KFd0WYjN079bLP0h>3P0AdM709NGU`1hN7P0WYjN09NGU`0n?Sh0
Jf][0:f][@1ZJVX0`<3007UiN@13@d<0MWIf05]KF`1BDU80Rh^;0492@P0e=CD0EUIF05ADE01MGEd0
RXZ:04Q8B00o?cl0EeMG06UYJ@0`<300R8R804E5A@0m?Cd0IVIV069RHP0j>SX0OGem06MWI`1=CDd0
H61P07=cL`0^;Rh0?Shn04];B`10@400J6QX08>3P`0W9bL0;2`/04];B`0j>SX0TI6A07MgM`0Q8B40
<S8b06m_K`1=CDd0IFEU00kooooo0480I6AT07AdM01GEeL0B4Q804m?C`25QHD0Cdm?06QXJ02=SHd0
F5QH04];B`1jNWX0UYJF07UiN@30`<00hN7Q07moO`1<C4`0IFEU05YJFP0o?cl0=SHf055AD@1SHf<0
M7Ad075aL@1EEED0PH6107IfMP1YJFT0RXZ:08>3P`1eMGD0O7al08>3P`1/K6`0FUYJ04];B`1=CDd0
FEUI04M7A`1NGUh0ATI605YJFP1@D500CDe=04Y:BP11@D40=cLg03Xj>P1BDU80:b/[02PX:01:BTX0
BDU906YZJP0k>c/0G5aL02PX:019BDT0Cdm?02d];@1PH600JVYZ07moO`1cLg<2059BDP0507YjNP1X
J6P0F5QH059BDP1QHF400P0o?cl0b`1BDU80D51@04m?C`0Z:RX0ADE504];B`0];Bd0C4a<03Hf=P0Y
:BT0=3@d02l_;`13@d<0;bl_03hn?P1@D500BTY:04E5A@0i>CT0A4A404Y:BP0[:b/0FUYJ04M7A`10
@400DE5A0410@00m?Cd0F5QH09ZJVP1;Bd/0ADE504e=C@1UIFD0ADE504];B`14A4@0[Jf]05UIF@13
@d<0Fe]K06QXJ010@400EEEE04m?C`1GEeL0DU9B0820P02b/[80W9bL075aL@21PH40Fe]K04M7A`1Q
HF40FEUI08N7Q`1QHF40IVIV034a<@14A4@0=3@d079bLP22PX80IVIV04U9B@1DE5@0JFUY0<71`@1o
Ogl0EUIF03De=@0^;Rh0=CDe05=CD`1:BTX0ATI6069RHP1nOWh0BTY:05eMG@14A4@0WYjN0861P@0m
?Cd0KFe]059BDP1OGel0BTY:05EEE@0k>c/0FEUI04U9B@22PX80>3Ph0492@P1<C4`0>SXj04U9B@17
AdL0CDe=06MWI`1lO7`0MWIf09FEU@1PH600>SXj061PH021PH40I6AT07MgM`1?Cdl0G5aL04];B`10
@400IVIV06ATI019BDT0@T92030`<010@400BTY:03<c<`0o?cl0LG5a03dm?@16ATH09RHV04a<C01I
FET0<c<c055AD@0o?cl0E5AD04M7A`13@d<0?3`l05YJFP0o?cl0BDU904Y:BP1aLG40MWIf04];B`1X
J6P0EUIF07inOP1>CTh0Fe]K04E5A@13@d<0GEeM03Xj>P13@d<0DU9B03De=@0l?3`0<S8b03/k>`1:
BTX0B4Q804];B`0m?Cd0<c<c02HV9P0_;bl0EEEE03hn?P1fMWH0PX:204];B`1<C4`0BDU903/k>`1>
CTh0NGUi09FEU@1EEED0TY:B059BDP1cLg<0EeMG06QXJ017AdL0ADE5069RHP1EEED0Fe]K03lo?`1;
Bd/06ATI038b<P0j>SX0FEUI05EEE@2DU9@0@T9204M7A`22PX80HV9R04m?C`1^KVh0EeMG013ooooo
000>ooooo`1]05EEE@1ZJVX0GemO06=SH`1>CTh0Hf=S06a/K01IFET0OWin03Ti>@1EEED0GEeM04e=
C@1eMGD0<S8b03lo?`1fMWH0EUIF05=CD`1[Jf/0;Rh^03Ph>02IVIT0A4A405aLG00d=3@0;bl_05]K
F`2;Rh/0;bl_06a/K010@400:2PX08B4Q01DE5@0:BTY04I6AP0k>c/0Cdm?09:BTP1:BTX071`L04=3
@`1<C4`0GemO09fMW@0Q8B40=SHf04e=C@1<C4`0M7Ad03Xj>P0F5QH0@T9204M7A`0e=CD0Kfm_01/K
6`0c<c<0Cdm?04];B`1@D500OWin01XJ6P0l?3`0K6a/06ATI014A4@0PX:203dm?@1<C4`0QhN704U9
B@1DE5@0J6QX01hN7P1=CDd0NGUi0492@P15ADD0DU9B02d];@1dM7@0IFEU02l_;`1>CTh0>c/k03@d
=01>CTh0Jf][01XJ6P0_;bl0:RXZ02@T9028R8P0ADE503@d=01:BTX0@41006EUI@2TY:@0BTY:02/[
:`1MGEd0@d=3069RHP1jNWX08B4Q01lO7`0206EUI@0k08j>SP1[Jf/061PH02PX:024Q8@0CDe=06e]
K@1IFET071`L0492@P1gMgL0GUiN05]KF`0m?Cd0;Rh^03/k>`1nOWh0b/[:07=cL`0c<c<0820P05]K
F`1bLW80=cLg05]KF`0U9BD0>CTi05iNGP1dM7@0=SHf07EeM@0c<c<0FEUI07QhN01@D500?Cdm079b
LP0h>3P0/k>c0;Fe]@1DE5@0:RXZ05=CD`1BDU80D51@0;Jf]P1LG5`0@T9206QXJ00n?Sh0F5QH08Z:
RP0W9bL0BTY:05=CD`1TI6@0K6a/069RHP0i>CT00P1ADE40JP13@d<0K6a/08^;R`0W9bL0GemO06QX
J01?Cdl0Vi^K04U9B@0G5aL0EUIF075aL@1ZJVX0Z:RX03<c<`0S8b<0T92@06a/K02PX:00>SXj02TY
:@13@d<0Ogmo07EeM@1`L7005aLG04I6AP2MWId0MGEe069RHP0j>SX0=cLg09NGU`1hN7P0G5aL069R
HP0j>SX0R8R80::RXP1WIfL0PH6103Xj>P13@d<0O7al079bLP1IFET0De=C034a<@0i>CT0N7Qh03Ph
>01?Cdl0ADE503Ti>@1/K6`0^;Rh0451@@16ATH0@41004A4A02`/;00W9bL03Hf=P1HF5P0>c/k04U9
B@2m_Kd0VYZJ02h^;P1DE5@0B4Q805EEE@2YZJT0HF5Q02<S8`1=CDd0GemO05QHF035aLD0H61P02<S
8`14A4@0FUYJ06YZJP3;bl/0FEUI03@d=00j>SX0F5QH07QhN02:RXX0@T9204M7A`1HF5P0NGUi06i^
KP1JFUX0<30`02HV9P1<C4`0NGUi04Q8B01RHV80<30`02d];@1IFET>ooooo`0:06ATI023Ph<0JFUY
04Y:BP1FEUH0UiNG07]kN`1NGUh0JFUY05iNGP80LG5a03l0Ti>C05eMG@15ADD0?Shn05EEE@1ADE40
>c/k04A4A01:BTX0<30`05ADE01RHV80EEEE05]KF`1CDe<0GUiN079bLP14A4@0FEUI079bLP1_Kfl0
C4a<04I6AP1LG5`0DE5A02h^;P16ATH0Bd];05MGE`1KFe/0Bd];05iNGP1`L700DU9B05UIF@1<C4`0
?Cdm04I6AP1DE5@092@T03lo?`1HF5P0EEEE06i^KP0b<S80GemO03/k>`1IFET0F5QH03Ph>01SHf<0
Ogmo092@T01fMWH0ADE504i>CP1XJ6P0IVIV03Ph>01:BTX0CDe=034a<@17AdL00P1>CTh0b@1PH600
FEUI04U9B@18B4P0;bl_04Q8B00i>CT0;2`/03<c<`0f=SH0CDe=03@d=017AdL0FUYJ04I6AP13@d<0
@T9204a<C01@D500@D5105iNGP1:BTX0Bd];04M7A`0j>SX0AdM704Q8B02BTY80BDU9055AD@1RHV80
J6QX069RHP1^KVh0@d=3071`L01HF5P0BDU905eMG@1jNWX0=3@d04e=C@0k>c/0Kfm_03Ph>01JFUX0
^;Rh09bLW016ATH0EeMG03Ph>00c<c<0F5QH06QXJ02<S8`0IVIV07moO`1GEeL0EEEE02DU9@1EEED0
CTi>05UIF@1ADE40G5aL07inOP2j^[X0GemO06i^KP0n?Sh0?Cdm03lo?`1OGel0EUIF04];B`11@D40
CDe=03lo?`2GUiL0=cLg04a<C01;Bd/0?Cdm069RHP0n?Sh0GEeM03Ph>01JFUX0B4Q808j>SP13@d<0
BDU902@T901:BTX0EUIF03Xj>P1UIFD0>c/k04U9B@1`L700JVYZ0410@01iNGT0?clo03Xj>P0i>CT0
>3Ph04];B`1<C4`0J6QX04];B`24Q8@0?Shn03<c<`0i>CT0BDU904=3@`0m?Cd0B4Q805IFEP13@d<0
F5QH08Z:RP1ZJVX0De=C01dM7@1WIfL0G5aL03hn?P17AdL0@d=305=CD`19BDT0Cdm?04U9B@1MGEd0
>c/k04];B`14A4@0@d=30492@P0d=3@0SXj>05mOG`1?Cdl0>SXj08:2PP0h>3P0ATI6065QH@18B4P0
E5AD07emO@1KFe/0FEUI061PH01GEeL0EUIF04i>CP1OGel0BTY:04];B`0V9RH0@d=306IVIP0R8R80
G5aL055AD@1[Jf/0Hf=S065QH@1LG5`0@T9204I6AP1ZJVX0I6AT04e=C@1cLg<0YJFU06=SH`2CTi<0
F5QH04M7A`1hN7P0Cdm?04E5A@1DE5@0LG5a03Hf=P1JFUX0D51@03Hf=P1=CDd0[jn_04=3@`19BDT0
JVYZ05]KF`1`L700M7Ad05]KF`0@ooooo`003_ooool0W01bLW80IFEU06a/K01oOgl0C4a<05IFEP1I
FET0PX:209bLW00/;2`0BDU904Q8B01mOGd0[Jf]02l_;`0Q8B40@D510410@02CTi<0TY:B02PX:00D
51@0ATI606YZJP2c/k<0?Cdm02TY:@0a<C40QXJ607UiN@2f][H0:RXZ03lo?`0i>CT0@T9205]KF`1o
Ogl06ATI038b<P0W9bL0DE5A0410@02ATI40?Shn02XZ:P15ADD0?clo05=CD`1kNg/0?clo06m_K`1V
IVH0=cLg055AD@0o?cl08R8R0861P@1=CDd0@T9205iNGP0[:b/0<S8b07AdM00V9RH0:RXZ04Y:BP0/
;2`0>c/k0:2PX01>CTh0<c<c04m?C`1EEED0LG5a09:BTP0V9RH0;Bd]05eMG@1>CTh0PH6107MgM`0N
7Qh0A4A4075aL@1QHF40V9RH03Ph>00P8200AdM706IVIP1UIFD0Rh^;01PH600G5aL0;bl_05eMG@1L
G5`0IFEU01HF5P0[:b/0G5aL05QHF010@400;Rh^024Q8@1?Cdl0K6a/04I6AP12@T80ADE503De=@14
A4@0RHV904U9B@0d=3@0BTY:03Ph>01=CDd0Kfm_02h^;P0];Bd0DE5A05ADE01GEeL0GEeM01lO7`0b
<S80EEEE04E5A@27QhL0IfMW01hN7P0g=cL0DE5A04U9B@29RHT06QXJ0410@01NGUh0Kfm_05=CD`2V
YZH0?Cdm03lo?`2CTi<0Hf=S069RHP3Bd]80=SHf05YJFP2:RXX0IfMW06UYJ@2DU9@0C4a<04U9B@80
MGEe07P0GEeM09>CT`0i>CT0>CTi07EeM@1FEUH0GEeM0820P00P8200=SHf07moO`1<C4`0XZ:R05iN
GP11@D40@T9207EeM@1FEUH0KVi^02XZ:P0g=cL0EEEE05iNGP0c<c<0Bd];034a<@1ADE40PX:206i^
KP0o?cl0ADE505IFEP1lO7`0RXZ:0451@@1GEeL0FEUI06EUI@1jNWX0<C4a03/k>`0f=SH0CDe=06IV
IP0];Bd0=CDe0451@@1DE5@0HF5Q08^;R`0/;2`0Hf=S05QHF01YJFT0aLG5071`L00j>SX0FUYJ06EU
I@28R8P0P82003Ph>01:BTX0JVYZ05mOG`2_[jl0Jf][0451@@17AdL0EeMG06MWI`2/[:`0>SXj04I6
AP1>CTh0SHf=05eMG@1RHV809bLW03Ti>@1<C4`0YJFU05iNGP1YJFT0>3Ph03dm?@1>CTh0N7Qh03/k
>`1kNg/0EEEE02h^;P1eMGD0UYJF03@d=01FEUH0:RXZ03Ti>@1iNGT0RXZ:03Ph>012@T80EeMG02`/
;02HV9P0Cdm?024Q8@0j>SX0CTi>02l_;`1/K6`0B4Q8020P800i>CT0>3Ph0451@@1SHf<0@T9202DU
9@0l?3`>ooooo`0M07]kN`1OGel0JVYZ051@D01EEED0B4Q802/[:`1ADE40RXZ:04A4A00o?cl0OGem
08j>SP26QXH0V9RH0820P01mOGd0DE5A05eMG@0e=CD0BTY:02/[:`13@d<0EUIF06QXJ01ADE40E5AD
07alO02GUiL00P1MGEd0>`1/K6`0JFUY06][J`1QHF40EeMG04e=C@0b<S80@41004I6AP0l?3`0ADE5
04Q8B00n?Sh0:b/[04U9B@1JFUX0A4A404];B`0m?Cd0ADE502/[:`0e=CD0?Cdm04m?C`1HF5P0CDe=
069RHP0W9bL0BTY:05MGE`1EEED0G5aL05eMG@22PX80IFEU04A4A00o?cl0G5aL06MWI`15ADD0BDU9
055AD@0K6a/09RHV04];B`16ATH0?Cdm02d];@0c<c<0>SXj0410@014A4@0?Cdm04e=C@0[:b/0?clo
04e=C@0o?cl0>3Ph0080AdM706X0:BTY03Xj>P1;Bd/0De=C02/[:`1IFET0GEeM04a<C019BDT0AdM7
06=SH`1LG5`0Ti>C075aL@1lO7`0DE5A061PH01eMGD0Cdm?0451@@2:RXX0IFEU08:2PP1[Jf/0MgMg
03Xj>P1BDU80@T9205MGE`17AdL0LW9b0:6QX@1oOgl0B4Q807EeM@1CDe<0:b/[04M7A`24Q8@0Z:RX
04m?C`1UIFD0=cLg06MWI`1JFUX0R8R80;Rh^02>SXh0LW9b05YJFP1dM7@0Z:RX07EeM@12@T80BDU9
065QH@18B4P0>CTi04I6AP0e=CD0BDU9071`L00c<c<0J6QX030`<01VIVH0Bd];03Lg=`15ADD0=CDe
059BDP18B4P0I6AT02HV9P0m?Cd09RHV07=cL`0e=CD0JVYZ05iNGP0/;2`0=cLg03dm?@1SHf<0M7Ad
06m_K`1SHf<0VIVI04=3@`1GEeL0IfMW061PH00j>SX0M7Ad07YjNP0a<C40E5AD02@T901BDU80ATI6
079bLP1<C4`0<c<c03dm?@1aLG40AdM70P1OGel0CP1BDU80KFe]04a<C01ADE40FEUI03/k>`1;Bd/0
>c/k05QHF00n?Sh0De=C05MGE`1fMWH0CTi>03lo?`1<C4`0Cdm?051@D012@T80IVIV069RHP1QHF40
AdM704e=C@18B4P0FEUI07AdM00l?3`0=SHf05iNGP0c<c<0<30`03lo?`0a<C40=SHf05IFEP1LG5`0
ADE504e=C@0Q8B40;2`/038b<P0A4A40A4A40492@P10@400ATI604M7A`1WIfL0KFe]05eMG@29RHT0
FUYJ08F5Q@1bLW80JFUY05mOG`1CDe<0DU9B06m_K`2CTi<0E5AD05QHF00e=CD0ADE502TY:@19BDT0
>SXj04Y:BP0n?Sh0EeMG04a<C01KFe/0FEUI05EEE@1CDe<0Ng]k065QHA3ooooo000>ooooo`0L09:B
TP1oOgl0Bd];06QXJ01bLW80N7Qh03@d=01GEeL0J6QX05QHF01OGel0;2`/03Xj>P1_Kfl0OWin03dm
?@1OGel0:2PX03/k>`25QHD0H61P04I6AP0d=3@0<c<c05eMG@20P8008b<S059BDP80=SHf0:P0Ti>C
059BDP0g=cL0QhN702TY:@0U9BD0VYZJ02PX:00R8R80BDU9030`<00e=CD0UYJF06=SH`0Z:RX0AdM7
03lo?`1DE5@0]KFe03`l?00i>CT0EUIF04];B`2@T900Cdm?02DU9@1BDU80Cdm?08>3P`2k^k/0;Bd]
02@T901KFe/0EUIF07alO02RXZ809BDU02h^;P17AdL0F5QH08Z:RP1OGel08B4Q04Q8B01MGEd0H61P
071`L012@T80:RXZ03dm?@1lO7`0=SHf04E5A@0I6AT0;bl_05EEE@1nOWh0;bl_05IFEP12@T80@410
07moO`1FEUH0;Bd]04];B`0k>c/0Hf=S096AT@0_;bl0;2`/04=3@`1=CDd0Hf=S04Q8B00G5aL07QhN
04A4A00g=cL0Jf][06i^KP0L71`08R8R059BDP1<C4`0De=C07AdM00W9bL0ADE5069RHP1SHf<0JFUY
0:>SX`0h>3P0BDU907YjNP1cLg<0FUYJ092@T00c<c<0>SXj092@T01GEeL0UYJF05YJFP0^;Rh0G5aL
051@D01MGEd0TI6A02h^;P0`<300IfMW06UYJ@1ADE40ADE501lO7`0f=SH0VYZJ07MgM`1VIVH0H61P
03De=@19BDT0KFe]05eMG@0g=cL0?Cdm04I6AP1]KFd0Ph>304e=C@0g=cL0@D5103dm?@1CDe<0]kNg
04i>CP0Z:RX0@41007QhN02TY:@0IVIV01hN7P0o?cl0GEeM05QHF02CTi<0<30`01TI6@19BDT0E5AD
03@d=01gMgL07AdM02`/;01OGel0Hf=S05iNGP1ZJVX0=CDe06a/K029RHT0KFe]08^;R`25QHD0>SXj
08>3P`15ADD206EUI@0c02HV9P1XJ6P0D51@06ATI01WIfL0;Bd]061PH01aLG40De=C07emO@0n?Sh0
=SHf08>3P`2d];@0@D51055AD@0k>c/0D51@09^KV`1RHV80FEUI0451@@14A4@0FEUI07inOP17AdL0
G5aL06YZJP1KFe/0V9RH07=cL`0n?Sh0MWIf059BDP1SHf<0MWIf04M7A`0c<c<0HV9R05QHF01@D500
OGem03Hf=P0g=cL0P820061PH00e=CD0Ng]k06MWI`0^;Rh0EEEE0080CTi>01T0WIfM03hn?P0k>c/0
ADE505UIF@1iNGT0PH6108j>SP0^;Rh0Lg=c096AT@1:BTX0IFEU0:f][@0U9BD0@41005=CD`12@T80
QhN706][J`0L71`0Cdm?06][J`0l?3`0Q8B400kooooo0300S8b<06EUI@1?Cdl0EeMG06IVIP1aLG40
B4Q805aLG01eMGD0De=C04A4A01]KFd0SXj>07]kN`2BTY80UYJF0820P01UIFD0I6AT04U9B@1ADE40
ADE504U9B@1>CTh0H61P059BDP1HF5P0Fe]K06MWI`1NGUh0Jf][07]kN`1CDe<0CTi>06EUI@1EEED0
DU9B0451@@14A4@0E5AD04E5A@1QHF40?Cdm04i>CP13@d<0Cdm?0410@00m?Cd203Ti>@0V0451@@0W
9bL0B4Q804I6AP1DE5@0IVIV020P8012@T80;bl_05UIF@10@4009BDU04a<C01/K6`0GemO05]KF`10
@400?Shn04=3@`1BDU80B4Q805MGE`1:BTX0=cLg06EUI@1MGEd0CTi>0410@00j>SX0ADE504e=C@0W
9bL0@D5102XZ:P0M7Ad0;bl_03Xj>P1?Cdl203Lg=`0_04a<C00i>CT0;Bd]04a<C01RHV80?clo02`/
;01FEUH0=cLg02l_;`1;Bd/0<c<c02PX:01PH600SXj>03@d=00l?3`0D51@05YJFP12@T80De=C04Q8
B01iNGT0?3`l03<c<`1ADE40IVIV04m?C`1HF5P0:2PX04I6AP0];Bd0L71`09jNWP2;Rh/0NWYj08J6
QP1@D500?3`l04i>CP0d=3@0F5QH051@D01JFUX0<S8b03De=@0j>SX00P1HF5P0G01GEeL0C4a<04a<
C01cLg<0[Zj^05=CD`1<C4`0<30`03Lg=`11@D40MgMg06][J`1DE5@0F5QH071`L010@400PX:202l_
;`1XJ6P0ATI603hn?P1?Cdl0=CDe05QHF00_;bl0BTY:038b<P1QHF40;2`/04E5A@0M7Ad0Bd];03dm
?@0k>c/0FEUI03/k>`1?Cdl0W9bL06i^KP1RHV80Shn?04A4A01MGEd0FUYJ04U9B@0W9bL0Bd];06QX
J00/;2`0L71`02XZ:P19BDT0;bl_03Xj>P0l?3`0BDU904I6AP1MGEd0>SXj0861P@1/K6`0=cLg04=3
@`0_;bl0R8R806MWI`0a<C40CTi>04E5A@1ADE40>SXj05YJFP0o?cl0Fe]K030`<00j>SX0?clo06m_
K`1RHV80>3Ph06m_K`1ADE40HV9R038b<P1KFe/0<c<c04U9B@1^KVh0>3Ph03Xj>P0m?Cd0?clo0P16
ATH0;@0i>CT0Bd];05iNGP1XJ6P0CDe=04=3@`0l?3`0H61P05ADE00A4A40Jf][04E5A@19BDT0?clo
05=CD`1:BTX0=cLg04U9B@1mOGd0JFUY08R8R01JFUX0JVYZ08^;R`2CTi<0DE5A05=CD`1[Jf/0IFEU
05MGE`1>CTh0GemO03Hf=P1<C4`0AdM703dm?@12@T80NWYj03Hf=P19BDT0B4Q805EEE@1mOGd0OWin
05MGE`0@ooooo`003_ooool0o`1RHV80PX:205YJFP1MGEd0DE5A06UYJ@1VIVH0CTi>06a/K017AdL0
N7Qh04Q8B00h>3P0O7al05MGE`19BDT0Q8B403De=@1EEED0GEeM03`l?01FEUH0HF5Q03De=@1@D500
EEEE04M7A`1dM7@0EUIF04A4A01]KFd0@D5104Q8B02ZZZX0DU9B02`/;01:BTX0?3`l05eMG@29RHT0
<30`02d];@18B4P0EUIF05MGE`1PH6007alO038b<P1ADE40GemO092@T00d=3@09bLW04];B`1DE5@0
@D5106IVIP0];Bd0>SXj07MgM`1XJ6P0=cLg04Y:BP0b<S80@41005UIF@1:BTX0<C4a07moO`12@T80
LG5a08B4Q014A4@0GUiN05IFEP0Y:BT0KFe]08J6QP0i>CT0@T9204m?C`0g=cL0Ng]k05iNGP0];Bd0
IfMW0492@P0b<S80MWIf05iNGP0b<S80FUYJ03lo?`17AdL0KFe]030`<00h>3P0D51@04U9B@1[Jf/0
Rh^;03Ti>@0k>c/0KVi^05=CD`1nOWh0EUIF020P800i>CT0GemO04I6AP23Ph<0MGEe028R8P0b<S80
HV9R03<c<`1ZJVX0NGUi02HV9P0o?cl0FUYJ04a<C01JFUX0DE5A02LW9`1PH600Lg=c055AD@1EEED0
9bLW03<c<`1WIfL0MWIf03hn?P1LG5`0;Rh^04i>CP1kNg/0Jf][04Q8B01lO7`0@d=307QhN02NWYh0
DU9B03Ti>@21PH40DE5A08:2PP2a/K40Cdm?034a<@26QXH0HV9R06ATI01kNg/0<C4a04=3@`1_Kfl0
IfMW05aLG01mOGd0<C4a03Ph>01hN7P0KVi^06IVIP1HF5P09bLW065QH@1eMGD0I6AT06MWI`1`L700
;bl_05mOG`1LG5`0ATI607EeM@1=CDd09RHV055AD@1=CDd0HV9R07UiN@0f=SH0;Bd]069RHP1CDe<0
Ogmo0410@00Y:BT0E5AD071`L01EEED0O7al03@d=01@D500P82004E5A@1QHF40<C4a05iNGP2=SHd0
C4a<05YJFP1cLg<0@41009:BTP1FEUH0Bd];05eMG@1IFET0Bd];07EeM@14A4@0G5aL05QHF01KFe/0
NGUi065QH@1ADE40S8b<06MWI`1FEUH0PX:20:2PX014A4@0Ogmo05MGE`1CDe<0Zj^[0:6QX@16ATH0
Fe]K04=3@`1HF5P0VIVI08J6QP1;Bd/0GUiN05UIF@15ADD0VYZJ06EUI@0f=SH0=3@d04e=C@1IFET0
S8b<04I6AP0W9bL0Bd];04M7A`005`1OGel0MgMg03<c<`0Z:RX0Bd];06=SH`1HF5P0Q8B40451@@0`
<300A4A405aLG01OGel0L71`03@d=00c<c<0@D51071`L01DE5@0Fe]K03De=@0j>SX0EUIF00kooooo
05h0SHf=07moO`1QHF40BDU905EEE@1bLW80De=C05eMG@1eMGD0GUiN06][J`2:RXX0PH61065QH@2C
Ti<0SHf=07UiN@1ADE40DU9B03Ti>@1IFET0BDU905]KF`1IFET0GemO04m?C`1XJ6P0IFEU07YjNP1F
EUH0HF5Q07EeM@1cLg<0Fe]K06=SH`1LG5`0F5QH02<S8`0g=cL0?Cdm0451@@1LG5`0@D5105UIF@1;
Bd/0CDe=055AD@11@D40@d=303Xj>P15ADD092@T02PX:016ATH0C4a<05QHF00i>CT0HF5Q02TY:@18
B4P0D51@04a<C01BDU80Jf][09fMW@1[Jf/0BDU904];B`1GEeL0FUYJ04=3@`18B4P0D51@02XZ:P0[
:b/0B4Q804];B`0e=CD0?clo0451@@10@400?clo04=3@`0l?3`0?Shn034a<@0X:2P0?Cdm038b<P0i
>CT0?clo04=3@`0b<S80=3@d0P10@4001P0b<S80CDe=04Y:BP14A4@0ATI60410@080Cdm?08`0Jf][
04e=C@1MGEd0Bd];04m?C`1GEeL0Cdm?03/k>`1VIVH0>CTi055AD@1CDe<0LG5a03@d=019BDT0@d=3
04a<C01;Bd/0GUiN08V9R@26QXH0DE5A06][J`0i>CT0:2PX055AD@1mOGd0UIFE04I6AP1XJ6P0=3@d
04Q8B00m?Cd0IVIV0861P@1GEeL0ATI6051@D01[Jf/0/k>c07MgM`1<C4`0De=C04I6AP11@D40R8R8
08F5Q@14A4@0Cdm?06EUI@1IFET0SXj>05ADE01WIfL0F5QH065QH@1gMgL0E5AD06YZJP0g=cL0DU9B
03lo?`1PH600FUYJ05IFEP0g=cL0GUiN06ATI014A4@0@410055AD@1RHV80MgMg06YZJP1YJFT0RXZ:
05=CD`0h>3P0Bd];05MGE`1SHf<0BTY:05EEE@1:BTX0O7al03Hf=P11@D40<S8b04M7A`0k>c/0>CTi
0451@@1PH600;Bd]05aLG01<C4`0>c/k05EEE@0P8200Bd];0492@P0a<C40D51@03Ti>@1CDe<0A4A4
04i>CP16ATH0IfMW0410@00e=CD0B4Q804i>CP14A4@0>3Ph07moO`1?Cdl0FEUI03Ph>01JFUX0>3Ph
05IFEP1[Jf/0=3@d03Ti>@0m?Cd0A4A40451@@1GEeL0:2PX0451@@1>CTh0EUIF04m?C`1>CTh0>CTi
03/k>`0m?Cd0;Rh^05QHF080Bd];00`0C4a<05MGE`1?Cdl0De=C04=3@`1nOWh0DE5A06][J`1LG5`0
HF5Q0492@P1JFUX205IFEP0B079bLP19BDT0F5QH04a<C01HF5P0>3Ph061PH01NGUh0ADE5065QH@2:
RXX0Bd];061PH01CDe<0FUYJ06a/K01TI6@0GEeM4?ooool000kooooo04L0IFEU061PH01GEeL0GEeM
05]KF`1JFUX0@41005EEE@1XJ6P0CTi>06e]K@0m?Cd0D51@08:2PP1ZJVX0;Bd]04=3@`0d=3@0Fe]K
08R8R00P8200:2PX03/k>`1ADE40YZJV03Xj>P0N7Qh0H61P05iNGP1hN7P0/[:b01PH600n?Sh0L71`
06m_K`1TI6@0GemO01`L700S8b<0BTY:06a/K01<C4`0U9BD030`<00k>c/0PX:207UiN@1SHf<0PH61
03<c<`1GEeL0]kNg05]KF`1OGel0>SXj01/K6`1_Kfl0=3@d04A4A01YJFT08B4Q02LW9`1oOgl09bLW
03hn?P1EEED0<30`055AD@1jNWX0>CTi03Lg=`020492@P3=05eMG@2=SHd0;2`/02l_;`1ADE40E5AD
08:2PP1WIfL05QHF03<c<`1[Jf/0EEEE0:^[Z`0^;Rh04Q8B04M7A`1:BTX0Hf=S09NGU`0O7al09BDU
06m_K`1HF5P0JVYZ05aLG00E5AD09BDU05QHF01kNg/0DU9B02@T900H61P0AdM705]KF`0j>SX0B4Q8
04a<C00G5aL0A4A408n?S`1IFET0@d=304M7A`0Z:RX0<C4a08V9R@1MGEd0Cdm?06i^KP0[:b/0FUYJ
07=cL`11@D40ADE505IFEP0m?Cd0De=C05YJFP0U9BD0BDU904e=C@0d=3@0NGUi02DU9@0c<c<0FEUI
03`l?016ATH0S8b<055AD@0e=CD0DE5A05aLG01XJ6P0UiNG04e=C@1?Cdl0Fe]K06UYJ@1CDe<0ZZZZ
06UYJ@0k>c/0Cdm?03Ph>020P800[jn_02l_;`0];Bd0MWIf05mOG`1hN7P0alO704A4A00k>c/0SXj>
07AdM02_[jl0Rh^;024Q8@1CDe<0Rh^;04i>CP1YJFT06a/K02HV9P1PH600J6QX06ATI01hN7P0<C4a
04a<C02k^k/0R8R8055AD@1`L700FEUI07AdM02PX:00AdM707YjNP0k>c/0?Cdm09JFUP1SHf<0De=C
06ATI00_;bl0KFe]04Y:BP16ATH0D51@03lo?`1YJFT0VIVI03lo?`1IFET0>c/k07emO@34a<@0GEeM
03Xj>P1=CDd0Bd];0:>SX`2ATI40@41004U9B@1MGEd0H61P0;^k^`2EUID0=cLg069RHP2@T900G5aL
0:FUY@1^KVh0:BTY05eMG@23Ph<0Kfm_08F5Q@0[:b/0>CTi04];B`2MWId0K6a/08b<S00[:b/0@T92
05MGE`1iNGT0HF5Q0:NWY`1BDU80BDU908n?S`1]KFd0?3`l07QhN00b<S80>3Ph08n?S`21PH40A4A4
05]KF`1VIVH0;bl_061PH024Q8@0@T9205eMG@13@d<06ATI05IFEP1RHV80=SHf06ATI00a<C40;Bd]
06UYJ@1_Kfl05aLG03Hf=P0>ooooo`0G09FEU@1iNGT0EUIF07]kN`2:RXX0Lg=c04e=C@1>CTh0FUYJ
03`l?017AdL0GemO05UIF@1?Cdl0UYJF0:b/[01ADE40A4A4051@D00Z:RX0ATI603Hf=P1ADE400P1O
Gel0@@0i>CT0?Shn02/[:`19BDT0C4a<05mOG`1FEUH0J6QX04a<C01IFET0JFUY04e=C@0d=3@0AdM7
04e=C@16ATH0D51@04Q8B016ATH0CTi>06QXJ01WIfL0@41004A4A01ADE40FEUI02HV9P0a<C40?3`l
05YJFP1UIFD0;bl_051@D013@d<0HV9R04Q8B00i>CT0Bd];04Y:BP1SHf<0EEEE06MWI`1aLG40EEEE
04];B`0a<C40>CTi05EEE@0O7al0<c<c05QHF00^;Rh08B4Q03Xj>P1@D500DE5A03Ti>@1EEED0=CDe
02LW9`0a<C40?Shn05ADE011@D40@4100080Bd];03H0ADE505EEE@1SHf<0CTi>04E5A@1JFUX0CTi>
04];B`17AdL0ATI60492@P1QHF40MWIf04=3@`1EEED0?3`l04Q8B01ADE40BDU904A4A01CDe<0DU9B
04Y:BP1HF5P0IfMW06=SH`1MGEd0?clo03Ti>@0j>SX0Cdm?09FEU@1`L700DU9B07inOP1PH600@410
061PH01OGel0Kfm_04a<C01VIVH0@T9203De=@19BDT0?Cdm05MGE`14A4@0De=C059BDP1nOWh0X:2P
065QH@1SHf<204i>CP0M04I6AP1eMGD0N7Qh04Q8B01IFET0J6QX04M7A`1?Cdl0@T9206ATI01<C4`0
>SXj04a<C01EEED0I6AT04Q8B01=CDd0@T9205UIF@1@D500QhN7055AD@1IFET0FUYJ03dm?@18B4P0
>CTi04I6AP1cLg<00P1PH6006P24Q8@0820P04i>CP11@D40EUIF04U9B@2OWil0Z:RX08:2PP2;Rh/0
?clo07=cL`0f=SH0BDU902l_;`1JFUX0@T9205iNGP0[:b/0VIVI05MGE`1KFe/0BTY:04E5A@1TI6@0
H61P0P13@d<0A`15ADD0DU9B034a<@1]KFd0ADE506=SH`0n?Sh0=3@d04E5A@20P800GUiN03hn?P1e
MGD0ATI606i^KP12@T80EeMG02DU9@10@400FUYJ02h^;P17AdL0BDU904];B`1:BTX0HV9R059BDP13
@d<0HF5Q069RHP0f=SH0B4Q803Ti>@0l?3`0Cdm?03@d=01[Jf/0:2PX0451@@17AdL0DE5A051@D00n
?Sh0B4Q807EeM@1nOWh0/K6a07UiN@1@D500ATI604a<C00a<C40GEeM07EeM@1IFET0IFEU04a<C01G
EeL09bLW065QH@1JFUX0A4A407UiN@2SXj<0BDU903lo?`18B4P0BTY:065QH@1[Jf/0N7Qh013ooooo
000>ooooo`1S09JFUP29RHT0Cdm?0861P@2=SHd0][Jf03Hf=P1=CDd0Q8B407MgM`21PH40?clo03Ti
>@1MGEd0Vi^K05mOG`1XJ6P0;2`/038b<P1`L700MWIf04Y:BP19BDT08b<S04M7A`24Q8@0=3@d07]k
N`0];Bd0>CTi06i^KP0l?3`0=CDe06ATI00Z:RX0@T9206][J`14A4@0@4100451@@0b<S80@D5107Qh
N015ADD0>CTi05aLG00_;bl0@d=309:BTP0_;bl0<30`04a<C018B4P0Q8B4055AD@0G5aL0CTi>059B
DP1VIVH0SXj>02HV9P0^;Rh0GUiN04a<C01nOWh0NWYj01XJ6P12@T80EeMG06YZJP2:RXX0;2`/030`
<01>CTh0Fe]K06IVIP1gMgL06ATI03Ph>01RHV80MGEe03<c<`1EEED03Ph>02DU9@1OGel0NWYj02d]
;@1MGEd0?3`l03dm?@14A4@0>c/k03<c<`1CDe<0<30`051@D01/K6`0<30`0080>SXj04l08b<S04a<
C01NGUh03@d=02LW9`0i>CT0<c<c065QH@1OGel08b<S02`/;017AdL0CDe=0492@P1DE5@07alO04Q8
B01=CDd0Bd];07moO`2DU9@0:BTY03`l?01FEUH0M7Ad0:ZZZP2CTi<09bLW06MWI`26QXH0F5QH09FE
U@13@d<0:BTY071`L011@D40Jf][0:j^[P0_;bl0;Bd]06i^KP25QHD0MGEe0820P00a<C40Bd];06i^
KP24Q8@0/[:b08J6QP0g=cL0@d=309>CT`2d];@0LW9b055AD@0[:b/0CDe=0:NWY`2ZZZX0H61P06m_
K`0/;2`0D51@08B4Q01cLg<0BDU903Xj>P0h>3P0J6QX07AdM015ADD0GemO04E5A@11@D40XJ6Q05IF
EP0c<c<0BTY:0080>c/k0600Ng]k01/K6`0b<S80EeMG051@D01FEUH0MGEe03<c<`1HF5P0N7Qh05YJ
FP2TY:@0K6a/02d];@1FEUH0BTY:08b<S02>SXh09BDU05ADE01WIfL0R8R807emO@0c<c<0BTY:06YZ
JP19BDT0Hf=S01lO7`0j>SX0PH6106MWI`1FEUH0Hf=S02XZ:P1GEeL0UIFE05UIF@1bLW80DE5A03Xj
>P1]KFd0[Jf]0820P02TY:@0NGUi04Q8B01@D500MgMg075aL@2b/[80Hf=S04I6AP2<S8`0AdM705mO
G`1XJ6P0:BTY03/k>`1/K6`0EeMG04];B`1`L700<c<c02PX:01GEeL0LW9b02h^;P1FEUH0BTY:03hn
?P1`L700E5AD02`/;01BDU80F5QH04A4A01iNGT0R8R8038b<P0o?cl0?Cdm079bLP2XZ:P0Ng]k02<S
8`0j>SX0?clo055AD@2/[:`0L71`01/K6`1<C4`0IVIV05EEE@2DU9@>ooooo`1008f=S@1hN7P0Fe]K
04M7A`1LG5`0OGem05ADE01PH600JFUY03lo?`1CDe<0EeMG06QXJ01GEeL0F5QH05=CD`1bLW80HV9R
059BDP0_;bl0HV9R03/k>`1OGel0De=C04];B`1=CDd0H61P04];B`1LG5`0EeMG06QXJ01WIfL0Fe]K
05YJFP1bLW80Hf=S051@D00Q8B40?3`l04I6AP1GEeL0EUIF03lo?`1HF5P0E5AD02XZ:P0e=CD0>SXj
04I6AP0e=CD0C4a<038b<P0e=CD0Cdm?059BDP1ZJVX08b<S05MGE`0V9RH0;Rh^03De=@0l?3`0>c/k
05MGE`80KFe]00@0=cLg0492@P12@T80GemO0P14A4@0?P1MGEd0<c<c04m?C`1IFET0@41003Hf=P0o
?cl0=cLg04I6AP0k>c/0=SHf02TY:@0d=3@0;Rh^02`/;013@d<0<C4a038b<P0g=cL0A4A4034a<@0^
;Rh0>c/k03`l?00g=cL0>c/k051@D014A4@0Fe]K034a<@14A4@0F5QH061PH016ATH0Cdm?04Q8B013
@d<0Bd];0492@P0n?Sh0BDU902d];@11@D40Fe]K0861P@0m?Cd0GemO03lo?`1<C4`0@410061PH02c
/k<0UYJF03hn?P1PH600N7Qh04M7A`1VIVH0Rh^;096AT@1?Cdl0Shn?0P1=CDd02@1?Cdl0=3@d04];
B`0d=3@0<C4a05=CD`1YJFT0a<C404A4A00203Lg=`0Z03/k>`0l?3`0`/;209JFUP0Y:BT0>SXj0451
@@0l?3`0Q8B404U9B@1YJFT0De=C05]KF`1RHV80G5aL065QH@0g=cL0?3`l02/[:`1/K6`0;bl_04];
B`0V9RH0Fe]K055AD@1OGel0KVi^03Lg=`1=CDd0]KFe05mOG`1_Kfl0R8R803De=@1DE5@0HV9R05eM
G@10@400=CDe06ATI01>CTh0MgMg0P0l?3`0E`15ADD0DU9B03lo?`0m?Cd0?clo05EEE@0j>SX0Ph>3
05UIF@1PH600HF5Q03dm?@28R8P0C4a<034a<@0`<300>SXj051@D012@T80FUYJ04m?C`1EEED0A4A4
04=3@`1ADE40GUiN04M7A`0n?Sh0KFe]06ATI01DE5@0?clo05QHF00i>CT0GemO07YjNP1;Bd/0C4a<
05MGE`1CDe<0ADE507QhN00S8b<0BDU905UIF@1FEUH0@D5104A4A00m?Cd0Jf][05UIF@0U9BD0De=C
02DU9@1=CDd0EEEE05QHF01=CDd0?clo05MGE`1/K6`0BTY:05MGE`1NGUh0GEeM09bLW02ATI40Q8B4
05QHF01DE5@0JFUY05mOG`1ADE40HF5Q03dm?@2=SHd0LG5a05QHF02BTY80_[jn04e=C@1JFUX0NWYj
06MWI`2:RXX0IVIV061PH00@ooooo`003_ooool0^01fMWH0Vi^K06][J`1DE5@0IfMW08N7Q`2:RXX0
FEUI04e=C@13@d<0GEeM051@D00];Bd0EUIF03@d=00m?Cd0HF5Q03/k>`13@d<0B4Q803/k>`0g=cL0
NWYj038b<P1BDU80BTY:0451@@1/K6`0?Cdm034a<@1EEED0ATI607=cL`2TY:@0?clo01hN7P1@D500
C4a<08:2PP2WYjL09bLW03De=@1LG5`0K6a/09bLW01fMWH0<S8b03dm?@1LG5`0MgMg08n?S`0V9RH0
<c<c04i>CP1_Kfl0EeMG08F5Q@0];Bd0C4a<08>3P`1lO7`0?3`l07EeM@0_;bl0?3`l06UYJ@1FEUH0
=3@d06][J`0e=CD0MWIf06i^KP0`<300BDU906m_K`0c<c<0HV9R03Ti>@0O7al0CTi>05aLG00k>c/0
Ng]k02l_;`0P8200?3`l0410@016ATH0UYJF04e=C@0l?3`0EeMG04E5A@1>CTh0NWYj02@T900d=3@0
Jf][02l_;`1KFe/0NGUi01LG5`0n?Sh0Rh^;03<c<`1EEED0<30`010@400V9RH0KFe]03hn?P1TI6@0
ATI6028R8P0j>SX0Rh^;04I6AP2IVIT0@D5102LW9`0k>c/0RHV905=CD`1JFUX0?Shn02l_;`1/K6`0
Xj>S07emO@14A4@0:BTY03De=@1DE5@0NGUi04M7A`1>CTh06a/K05]KF`2ATI40EeMG02TY:@1EEED0
;Rh^05mOG`1eMGD0;Bd]01TI6@1RHV80>3Ph05iNGP1kNg/0;2`/03De=@1cLg<0@D5105=CD`1nOWh0
92@T02HV9P1;Bd/0@d=304];B`2BTY80:RXZ02d];@1YJFT0TY:B07=cL`1RHV80:RXZ03lo?`27QhL0
DE5A09VIV@26QXH0;bl_06a/K01QHF40@D5108J6QP0a<C409RHV06i^KP0m?Cd203hn?P0501`L700f
=SH0Ti>C05MGE`1bLW800P0j>SX0E@2>SXh0U9BD05MGE`18B4P09BDU0492@P26QXH0GemO04i>CP0^
;Rh0CTi>08F5Q@1ADE40BDU908J6QP1:BTX0TI6A0492@P0f=SH0Hf=S04I6AP1TI6@0Kfm_02@T900m
?Cd0>CTi04I6AP1jNWX0B4Q8038b<P13@d<0GemO06a/K02<S8`0Rh^;03Lg=`1TI6@0BDU905aLG02i
^KT0PX:2030`<01kNg/0EUIF06ATI02XZ:P0>c/k03De=@1_Kfl0JVYZ06e]K@3<c<`0@d=303De=@18
B4P0I6AT06a/K01^KVh09BDU0451@@1ZJVX0DE5A04E5A@1ZJVX09bLW02<S8`1NGUh0P82006][J`28
R8P0>3Ph01lO7`1OGel0];Bd05UIF@1HF5P08b<S03Lg=`0h>3P0H61P03Ti>@1;Bd/07QhN02d];@1P
H6003_ooool09`2=SHd0GemO04i>CP1LG5`0MWIf05YJFP15ADD0Hf=S09nOW`1;Bd/0A4A405iNGP2:
RXX0GEeM079bLP1hN7P0BDU904Q8B01:BTX08R8R05YJFP0o?cl0?Cdm05QHF01=CDd0;Rh^04a<C013
@d<0IfMW04U9B@10@400J6QX05YJFP19BDT0DE5A04e=C@1]KFd09RHV03Xj>P0203hn?P0W051@D01T
I6@0HV9R04I6AP10@400@d=30410@019BDT0=cLg04m?C`0k>c/0AdM704e=C@1>CTh0IfMW03/k>`1Z
JVX0>c/k04e=C@1EEED0E5AD05IFEP1SHf<0PX:2075aL@1;Bd/0DE5A05QHF01eMGD0@T9204A4A01G
EeL0>3Ph030`<015ADD0AdM703@d=00k>c/0;2`/0080Bd];0380B4Q803dm?@15ADD0;bl_02LW9`13
@d<0?3`l03Ti>@0k>c/0@410034a<@0g=cL0>c/k03Lg=`0c<c<0@d=305YJFP0h>3P0>CTi030`<018
B4P0De=C06a/K01QHF40IfMW038b<P19BDT0B4Q805=CD`0^;Rh0@D5101/K6`0e=CD0ATI6051@D00k
>c/0B4Q804U9B@1ADE40Fe]K06a/K01]KFd0L71`05EEE@1VIVH0NGUi06][J`1kNg/0Ph>309:BTP80
CDe=00h0DE5A051@D01fMWH0GemO05QHF01JFUX0Bd];06IVIP1BDU80Lg=c03<c<`1:BTX0E5AD0451
@@80Bd];07D0DE5A02d];@18B4P0D51@0492@P1OGel0ATI605MGE`1ADE40IFEU05MGE`1>CTh0B4Q8
04];B`1SHf<0?clo04Q8B011@D40WinO03Ti>@1VIVH0CDe=02<S8`0R8R80:RXZ03Ti>@16ATH0De=C
06UYJ@29RHT0@d=3034a<@1?Cdl0WYjN04U9B@1UIFD0QXJ604E5A@1gMgL05QHF04Q8B00b<S80C4a<
04A4A01DE5@0FEUI051@D01=CDd0QhN7030`<019BDT0B4Q802LW9`1GEeL0@T9203De=@14A4@0CTi>
065QH@1=CDd0GemO05]KF`1kNg/0DE5A04I6AP1IFET0JVYZ06e]K@1;Bd/0KVi^06ATI01mOGd0CTi>
07MgM`0V9RH0IVIV05eMG@0a<C40@41005iNGP1<C4`0DU9B09NGU`18B4P0?Cdm06UYJ@1WIfL0D51@
05iNGP0X:2P0AdM703Lg=`11@D40FEUI04Q8B02=SHd0DU9B07]kN`1HF5P0?clo0451@@1<C4`0LW9b
0;6a/@1?Cdl0C4a<05MGE`1FEUH0=CDe05mOG`1ZJVX0C4a<03De=@19BDT0FUYJ038b<P14A4@00P0i
>CT02@1lO7`0_[jn04];B`18B4P0HV9R05IFEP1GEeL0IfMW09BDU00@ooooo`003_ooool0?01EEED0
IFEU08V9R@1]KFd0>3Ph06a/K023Ph<0OGem07IfMP0`<300Jf][09ZJVP1VIVH0YZJV03<c<`0e=CD0
Hf=S069RHP1]KFd0OGem02TY:@19BDT0ATI605mOG`2e]KD06a/K02LW9`1>CTh0>SXj059BDP1VIVH0
5QHF028R8P1kNg/0NGUi051@D016ATH04Q8B02TY:@1aLG40EeMG051@D01hN7P092@T03@d=026QXH0
C4a<05mOG`1TI6@0=SHf04m?C`1KFe/0=3@d04m?C`0];Bd09bLW09fMW@0_;bl0<C4a03dm?@80;Bd]
0=P0PH6102TY:@10@400Cdm?03Hf=P1gMgL0gmoO03dm?@15ADD0JVYZ05QHF02CTi<0V9RH03<c<`1@
D500Cdm?05aLG02?Shl0GemO010@401MGEd0HF5Q06EUI@2UYJD0ATI6018B4P1[Jf/0RXZ:06ATI01E
EED0:2PX04i>CP1XJ6P0@d=3051@D01;Bd/0=cLg055AD@1mOGd0EEEE04I6AP0i>CT0B4Q805EEE@28
R8P0:RXZ01lO7`0k>c/09BDU03@d=01gMgL0:2PX02h^;P19BDT0C4a<07IfMP17AdL0820P02@T9012
@T80CTi>06IVIP0j>SX07AdM02TY:@1VIVH0UYJF07MgM`0R8R808b<S034a<@1VIVH0E5AD06QXJ00S
8b<0?Shn055AD@1PH600@D5108b<S00^;Rh0@41007inOP1`L700ATI60;Zj^P12@T80JVYZ05IFEP1D
E5@0CTi>0<O7a`0i>CT0E5AD07MgM`16ATH0?Shn06i^KP0W9bL0>CTi08N7Q`1GEeL0F5QH06=SH`0g
=cL0AdM706YZJP0X:2P0GUiN02l_;`0k>c/0MGEe06e]K@12@T80AdM702l_;`11@D40K6a/075aL@1:
BTX0AdM703lo?`2;Rh/0UYJF03dm?@0[:b/0ATI606QXJ02ZZZX0Ph>302/[:`0k>c/0G5aL06e]K@1h
N7P0;Rh^03Ph>01:BTX0GEeM07AdM00K6a/0CTi>05YJFP1GEeL0MGEe08V9R@15ADD0IVIV05YJFP1O
Gel0MgMg03@d=00S8b<0JVYZ06a/K02m_Kd0DU9B03hn?P19BDT0GemO07=cL`2DU9@0D51@03hn?P1h
N7P0QhN705IFEP2>SXh0>c/k04A4A01OGel0Shn?07EeM@1JFUX0<c<c04A4A01EEED0J6QX03Ti>@1C
De<0<C4a04U9B@1BDU80G5aL03Ti>@1ZJVX0A4A404i>CP2XZ:P0K6a/03<c<`1EEED0C4a<05]KF`2S
Xj<0EeMG01DE5@0f=SH0C4a<05MGE`2UYJD0Cdm?020P800f=SH0?clo04A4A01oOgl09BDU03De=@1D
E5@0Fe]K0451@@1mOGd0=CDe01hN7P0o?cl>ooooo`1309:BTP29RHT0B4Q8051@D01UIFD0GemO03dm
?@1KFe/0EeMG03Ti>@0k>c/0HV9R05QHF01GEeL0MGEe075aL@1fMWH0DE5A05eMG@0S8b<0AdM703Hf
=P0i>CT0?Cdm05IFEP0k>c/0GemO04I6AP0l?3`0>SXj05aLG01bLW80Jf][04a<C01EEED0H61P04i>
CP0b<S80ATI6055AD@11@D40E5AD034a<@0n?Sh0?clo05EEE@1OGel0>3Ph03lo?`1@D500Cdm?024Q
8@0Z:RX0Bd];06IVIP1XJ6P0:BTY04e=C@0c<c<0IFEU055AD@1<C4`0De=C05YJFP1LG5`0EUIF061P
H00205eMG@0k05=CD`1OGel0Kfm_05EEE@0^;Rh0>SXj06IVIP12@T808b<S038b<P0l?3`0FEUI05EE
E@1GEeL0@T9203<c<`0`<300@D5105QHF019BDT0@T92059BDP17AdL0;2`/04Y:BP1PH600Bd];04=3
@`1JFUX0CTi>04I6AP1BDU80B4Q804E5A@1lO7`0QhN704E5A@1CDe<0BDU905UIF@1?Cdl0CDe=03dm
?@18B4P0<c<c05]KF`1?Cdl0K6a/03dm?@1DE5@0=cLg03De=@1<C4`0J6QX0:b/[01gMgL0?Shn075a
L@1RHV800P1<C4`03P2=SHd0WinO03lo?`1MGEd0<S8b04U9B@0k>c/0GEeM07YjNP1WIfL0AdM705=C
D`1bLW80V9RH0`1ADE40:@1SHf<0EeMG07]kN`1_Kfl0N7Qh055AD@1FEUH0DU9B07EeM@0k>c/0MGEe
05=CD`16ATH0BTY:04i>CP1EEED0B4Q805aLG00b<S80HF5Q02l_;`1OGel0>c/k04U9B@16ATH0>c/k
04=3@`1?Cdl0EEEE079bLP0c<c<0JFUY08N7Q`0e=CD0AdM704a<C01EEED0@D510:^[Z`2`/;00@410
0080@D5105P0Ogmo038b<P1UIFD0EeMG06MWI`1VIVH0NGUi06MWI`28R8P0Fe]K05aLG01QHF40=SHf
06ATI01cLg<0De=C05IFEP1IFET0De=C05MGE`2:RXX0Cdm?04i>CP1NGUh0CDe=05IFEP2QXJ40J6QX
059BDP1nOWh0IfMW06i^KP0g=cL0DU9B02/[:`1[Jf/0JVYZ038b<P0m?Cd0ADE50492@P0g=cL0DU9B
02DU9@12@T80BDU904=3@`0l?3`0>SXj03dm?@1UIFD0DU9B04e=C@1YJFT0=CDe04Q8B00b<S80Bd];
04U9B@11@D40Fe]K05aLG01FEUH0EEEE07=cL`1_Kfl0MgMg08J6QP1<C4`0=CDe0492@P1]KFd0De=C
04M7A`1?Cdl0?Cdm051@D01ADE40ADE505]KF`1[Jf/0?Shn059BDP1UIFD0CDe=055AD@1?Cdl0Ogmo
4?ooool000kooooo0600U9BD05aLG01BDU80Jf][06MWI`1IFET0B4Q805MGE`21PH40ATI605EEE@10
@400D51@08b<S01iNGT0;Rh^05]KF`1;Bd/0Fe]K06UYJ@0h>3P08B4Q04A4A00^;Rh0NWYj04];B`0G
5aL0C4a<0451@@1=CDd0QHF501/K6`0Q8B40?Shn03Xj>P1=CDd0Lg=c01/K6`0U9BD0?Cdm04U9B@1S
Hf<0][Jf02`/;00T92@0ATI6051@D02EUID0[Jf]024Q8@0e=CD0BTY:04A4A01;Bd/0<c<c02h^;P1?
Cdl0@41005=CD`27QhL07QhN030`<01EEED0JFUY07moO`2QXJ40:RXZ03dm?@1gMgL0I6AT05EEE@0E
5AD0:b/[04A4A01]KFd0=cLg05MGE`0Y:BT09BDU06i^KP1hN7P0;Rh^0492@P0h>3P0Bd];07QhN01<
C4`08B4Q03De=@13@d<0JFUY08b<S00G5aL0<30`04U9B@1=CDd208:2PP0502TY:@17AdL0DE5A059B
DP1fMWH00P0b<S80M00o?cl0Fe]K05MGE`1cLg<0DU9B02TY:@0f=SH0J6QX05EEE@1iNGT0EUIF03Ph
>01KFe/0Q8B404a<C026QXH0Ph>303Hf=P1HF5P0UYJF04];B`1/K6`0F5QH02/[:`1OGel0E5AD04U9
B@1fMWH0D51@030`<01[Jf/0?3`l05UIF@2=SHd0=3@d04e=C@1SHf<0ADE504m?C`1HF5P0<C4a030`
<023Ph<0KFe]05MGE`1/K6`0:BTY04Q8B02;Rh/0R8R80492@P1BDU80?Cdm05]KF`2KVi/0=3@d024Q
8@19BDT0FUYJ069RHP1eMGD0;2`/0410@01XJ6P0HV9R08R8R010@40061PH03Ti>@21PH40D51@05UI
F@0Y:BT0AdM705ADE01WIfL0EUIF06EUI@0b<S80EUIF07YjNP1WIfL0=SHf05iNGP0o?cl0E5AD07in
OP1BDU80G5aL03De=@0i>CT0MWIf04];B`1iNGT0Cdm?02TY:@1aLG40ATI606a/K01]KFd0;Bd]04E5
A@2b/[80D51@06IVIP0_;bl0?Shn0820P01UIFD0@D5104=3@`0m?Cd0Lg=c06][J`0b<S80CDe=0P17
AdL0702>SXh0EeMG03De=@1>CTh0DE5A05iNGP2ZZZX0@d=304E5A@1GEeL0HF5Q07UiN@2?Shl0<C4a
04E5A@1>CTh0FEUI061PH02ATI40=CDe0410@01MGEd0I6AT055AD@2NWYh0A4A403Ti>@1aLG42061P
H00I06m_K`1FEUH0:b/[06a/K029RHT0?3`l065QH@12@T80=SHf06EUI@2BTY80@T9207YjNP0n?Sh0
:b/[05MGE`1NGUh0>SXj06UYJ@10@400;bl_051@D01EEED0<30`05IFEP0>ooooo`0H09^KV`1nOWh0
BDU905EEE@1oOgl0M7Ad04Y:BP1nOWh0M7Ad03lo?`15ADD0PH61079bLP1_Kfl0/k>c0:VYZ@1iNGT0
E5AD05iNGP10@400D51@03`l?00n?Sh0C4a<0P1UIFD08P20P800FEUI069RHP1;Bd/0MGEe08n?S`20
P800HV9R07MgM`1]KFd0EUIF02d];@0b<S80EeMG06ATI01PH600>SXj059BDP11@D40Bd];06a/K014
A4@0?Shn03Ti>@1;Bd/0:b/[02`/;013@d<0CTi>05eMG@0M7Ad0CDe=03`l?01ADE4203lo?`0T04Y:
BP1EEED0K6a/05aLG00o?cl0ATI604i>CP1BDU80<C4a03hn?P1WIfL09RHV0410@01RHV80BTY:03Ph
>00_;bl0?clo05EEE@17AdL0Bd];03Ti>@0e=CD0;Rh^02/[:`10@400;Rh^03@d=00k>c/0>3Ph03<c
<`0W9bL0<c<c03Ti>@1?Cdl0A4A40P1<C4`0XP1EEED0ADE504U9B@1aLG40LW9b04Q8B01LG5`0BTY:
05EEE@1=CDd0BTY:05ADE01aLG40@T9204=3@`18B4P0JFUY03Ti>@1BDU80AdM704A4A01>CTh0GUiN
0:f][@1_Kfl0<S8b06EUI@1ZJVX0FUYJ05=CD`1KFe/0MGEe02h^;P1aLG40;Rh^03<c<`1/K6`0CDe=
061PH00l?3`0<30`04a<C01bLW80UYJF05UIF@1LG5`0@D51038b<P0f=SH0Zj^[08R8R011@D40MGEe
08^;R`1RHV80VYZJ04U9B@1BDU80AdM704i>CP1?Cdl0GEeM05=CD`19BDT0De=C05iNGP2BTY80Cdm?
07IfMP18B4P0F5QH06][J`1BDU80I6AT051@D01>CTh0S8b<04=3@`1]KFd0RHV904=3@`1<C4`0>SXj
06][J`0j>SX0Cdm?07inOP0o?cl0Fe]K028R8P11@D40@d=304A4A00];Bd0HV9R05eMG@1[Jf/0MGEe
0<30`01NGUh0Kfm_05mOG`1FEUH0L71`069RHP13@d<0CTi>05aLG01VIVH0<C4a04a<C013@d<0GEeM
03<c<`0W9bL0AdM7059BDP1CDe<0@D5107moO`1@D500Jf][04U9B@1GEeL0=3@d04Y:BP1IFET0=CDe
03Ph>019BDT0ATI60451@@1cLg<0BDU903dm?@1ADE40EUIF059BDP1NGUh0;2`/069RHP1EEED0B4Q8
05aLG00];Bd0N7Qh04Y:BP1mOGd0Bd];04E5A@1PH600EUIF05=CD`2CTi<0JFUY05YJFP2KVi/0U9BD
03Ti>@1NGUh0CTi>04e=C@80ADE500h0KVi^034a<@16ATH0?Shn05aLG01kNg/0XJ6Q02h^;P0e=CD0
MWIf04i>CP1HF5P0CTi>0:JVYQ3ooooo000>ooooo`2N07IfMP2:RXX0CDe=05iNGP2<S8`0TY:B03Xj
>P17AdL0FEUI05eMG@27QhL0<c<c04A4A01FEUH0IfMW04E5A@1TI6@0;2`/04Q8B01YJFT0F5QH05aL
G01^KVh0=SHf06IVIP2GUiL0FUYJ0:>SX`16ATH0<c<c06m_K`19BDT0IVIV08^;R`0U9BD0=3@d07in
OP15ADD0H61P05YJFP0N7Qh0@D5105mOG`1<C4`0B4Q803hn?P0I6AT0=SHf08B4Q012@T80G5aL024Q
8@0l?3`0QHF505eMG@0G5aL0D51@03De=@1;Bd/0Ph>304I6AP0K6a/0B4Q802d];@1`L700a<C403Hf
=P0U9BD0JFUY06QXJ02;Rh/09bLW01LG5`0W9bL0Hf=S04U9B@1[Jf/06a/K01XJ6P18B4P0U9BD05MG
E`22PX807alO02`/;01gMgL0X:2P06MWI`2ZZZX0>SXj0451@@1LG5`0HV9R04Y:BP1kNg/0:2PX0492
@P1`L700?Cdm04M7A`1KFe/07QhN02l_;`1CDe<0<S8b05QHF013@d<05ADE02@T901PH6006a/K04M7
A`1<C4`06QXJ03De=@1iNGT0<C4a05YJFP0b<S80;bl_04M7A`28R8P0?Shn06QXJ01:BTX0@410065Q
H@2<S8`0<S8b04m?C`0d=3@0ADE5096AT@1^KVh06a/K03Ph>00k>c/0T92@09fMW@16ATH08R8R05AD
E019BDT0R8R809nOW`0g=cL0A4A4051@D01dM7@0XJ6Q09RHV00g=cL0?Shn04e=C@1ZJVX0RHV906][
J`0l?3`205ADE01f071`L016ATH0LG5a02LW9`1IFET0MWIf04i>CP0o?cl0@d=3051@D02QXJ40OGem
030`<01PH600<30`04I6AP1hN7P08B4Q01`L7011@D40=CDe03Xj>P1lO7`09RHV038b<P1EEED0=cLg
051@D01eMGD0<C4a04Y:BP11@D40Hf=S08V9R@1PH600>CTi0492@P1EEED0Ogmo05mOG`0];Bd0@D51
05iNGP1]KFd0N7Qh03Hf=P0i>CT0GEeM05aLG02MWId0:BTY02DU9@1PH600IVIV05EEE@1BDU80?3`l
06e]K@1TI6@0@d=306UYJ@0o?cl0E5AD05=CD`1QHF40CDe=09>CT`0m?Cd0@D51079bLP1ZJVX0Jf][
096AT@1:BTX0Bd];08n?S`1GEeL0Ng]k08R8R00c<c<09BDU079bLP19BDT0EUIF07moO`0j>SX09bLW
0:>SX`1YJFT0>CTi05iNGP0f=SH0=CDe05aLG01KFe/0D51@06a/K00a<C40DU9B06UYJ@1DE5@0=3@d
03<c<`13@d<0BDU908n?S`1TI6@09bLW04A4A00i>CT0E5AD06EUI@1LG5`08b<S04a<C01;Bd/0H61P
0:ZZZPkooooo01@0W9bL08R8R01RHV80H61P06ATI01[Jf/0@D5105ADE024Q8@0CDe=05MGE`1VIVH0
RXZ:06m_K`1cLg<0HV9R06e]K@1<C4`0EeMG02LW9`80@D510680Fe]K04a<C011@D40=cLg06QXJ01T
I6@0PH6104=3@`1DE5@0Rh^;05=CD`13@d<0F5QH05IFEP1]KFd0AdM7051@D017AdL0>CTi04A4A01J
FUX0DU9B04=3@`1>CTh0EeMG04E5A@1CDe<0B4Q805iNGP0i>CT0=SHf04m?C`1;Bd/0GemO02l_;`1Q
HF40>SXj05UIF@1LG5`0CDe=05mOG`1ZJVX0Jf][06ATI016ATH0F5QH069RHP1MGEd0BTY:04m?C`1]
KFd0?3`l03@d=01@D500B4Q802h^;P1:BTX0B4Q805aLG00l?3`0EeMG04A4A00l?3`0<c<c03Ph>01G
EeL0>c/k04Q8B01IFET0A4A403hn?P16ATH0CTi>04a<C01EEED0GemO04U9B@1;Bd/0FEUI04I6AP1C
De<0Jf][04Y:BP0n?Sh0Jf][04i>CP1FEUH0BTY:07alO019BDT0G5aL03lo?`1:BTX0FEUI05YJFP0h
>3P0D51@05aLG080M7Ad01P0P8200=CDe02f][H0De=C06=SH`1XJ6P0JVYZ079bLP1fMWH0MgMg04a<
C01]KFd0EEEE03`l?013@d<0:BTY04i>CP0c<c<0C4a<0451@@1BDU80cLg=03/k>`1?Cdl204=3@`80
>CTi0500A4A402/[:`0k>c/0@d=304M7A`29RHT0IVIV075aL@1/K6`0EeMG04Y:BP1BDU80CDe=04I6
AP1OGel0<C4a04Y:BP0];Bd0Kfm_05=CD`1lO7`0LG5a03dm?@11@D40>CTi05QHF02RXZ80?clo07Yj
NP2ATI40ADE506IVIP1SHf<0Ogmo03/k>`1]KFd0QXJ6071`L01MGEd0D51@04m?C`1<C4`0JFUY04=3
@`1BDU80ATI60492@P0k>c/0GemO04M7A`1hN7P0HV9R0492@P21PH40Cdm?04];B`0i>CT0?clo04M7
A`12@T80GUiN04U9B@1@D500DE5A03Lg=`1:BTX0L71`06IVIP1:BTX0QXJ604M7A`1_Kfl0DE5A06UY
J@0/;2`0Hf=S06IVIP0e=CD0Bd];0451@@80@d=302h0YJFU03lo?`0g=cL0GUiN04];B`0_;bl0<30`
03Ti>@1CDe<0?clo0451@@1DE5@0:BTY0410@012@T80ADE505UIF@1CDe<0GemO03hn?P1mOGd0_kno
08^;R`1UIFD0Hf=S04m?C`0d=3@0SXj>06a/K01HF5P0ADE503Ph>01<C4`0B4Q805UIF@17AdL0@T92
05eMG@21PH40BDU904Q8B01RHV80De=C05ADE01KFe/0MWIf4?ooool000kooooo0?l0LW9b06i^KP1_
Kfl0LG5a04Y:BP1VIVH0CDe=08:2PP2ATI40ADE505MGE`1ADE40HF5Q08J6QP0`<300<c<c05IFEP18
B4P0TY:B06UYJ@0Z:RX0?Shn04Y:BP19BDT0YZJV03`l?00[:b/0@T9204U9B@1SHf<0J6QX02LW9`0a
<C40FEUI051@D01>CTh0J6QX0492@P1WIfL0MGEe05ADE01KFe/0H61P03hn?P1cLg<0KFe]03<c<`0h
>3P0@D5104E5A@1^KVh0=cLg02LW9`16ATH0@T9204e=C@1]KFd092@T02l_;`1<C4`0BDU904m?C`23
Ph<06QXJ03`l?01HF5P0CDe=05ADE02n_[h0=SHf0451@@1:BTX0@T9207QhN01ADE40;bl_05ADE01F
EUH0>3Ph09bLW011@D408b<S055AD@0m?Cd0?clo06=SH`0Q8B40?clo05MGE`1@D500DU9B04m?C`0_
;bl0@T9208V9R@10@400B4Q804E5A@0^;Rh0Kfm_09RHV01>CTh0ATI603/k>`0j>SX0K6a/08b<S00n
?Sh0:b/[04];B`0m?Cd0Lg=c07IfMP0d=3@0De=C04U9B@15ADD0OGem05EEE@0/;2`0AdM705]KF`18
B4P0RHV905MGE`0];Bd0DE5A06QXJ01@D500P820028R8P0Y:BT0ATI606QXJ01?Cdl0HV9R01lO7`0f
=SH0I6AT06m_K`1KFe/0Shn?03Xj>P18B4P0MWIf0451@@0n?Sh0Shn?03Hf=P1?Cdl0RXZ:05YJFP1m
OGd0VYZJ03Hf=P1ADE40ZJVY05UIF@1LG5`0NGUi03Ph>00e=CD0RXZ:05iNGP1ZJVX0QhN703lo?`1N
GUh0G5aL04m?C`2?Shl0De=C03dm?@2@T900LW9b03Ph>01TI6@0?Cdm04Y:BP1dM7@0@d=302d];@1S
Hf<0@D5105EEE@1mOGd0=cLg04I6AP1YJFT0De=C08F5Q@1@D500<C4a05ADE01WIfL0FUYJ051@D00_
;bl0E5AD061PH01<C4`0JFUY02PX:01TI6@0TY:B05aLG01`L700QHF50451@@28R8P0KVi^05]KF`25
QHD0?Cdm04a<C01jNWX0@T9208^;R`1BDU80=CDe07alO01_Kfl0I6AT09BDU01=CDd0@T9209ZJVP2P
X:00F5QH07YjNP13@d<0D51@09JFUP1/K6`0DE5A05eMG@15ADD0GUiN07moO`1JFUX0>c/k05aLG00m
?Cd0D51@06][J`19BDT0;Rh^05QHF01JFUX0G5aL061PH00l?3`0@41005=CD`1?Cdl000P0@d=306m_
K`0[:b/0:2PX05aLG01;Bd/0DE5A06][J`809RHV00d0Bd];07]kN`18B4P0EeMG02@T900h>3P0DE5A
06UYJ@0n?Sh0LG5a02XZ:P0R8R80Fe]K00kooooo02l0UIFE07moO`18B4P0E5AD06e]K@1XJ6P0CDe=
05aLG01KFe/0@D5105MGE`1iNGT0Ti>C05iNGP2DU9@0Ph>306m_K`1FEUH0F5QH02DU9@1BDU80>CTi
04];B`1:BTX0De=C0451@@1]KFd0Fe]K05ADE018B4P0Bd];06a/K01VIVH0GUiN06i^KP1SHf<0CTi>
030`<00d=3@0CTi>04a<C01NGUh0@d=304a<C00g=cL0CDe=05MGE`0203Ti>@0d0410@013@d<09bLW
038b<P14A4@0GEeM05QHF00Z:RX0E5AD03hn?P1TI6@0DU9B04I6AP1?Cdl0E5AD06ATI01JFUX0?clo
051@D01HF5P0FEUI04Q8B01GEeL0D51@028R8P0Y:BT0CDe=04i>CP0];Bd0CTi>04A4A01>CTh0@410
05EEE@0j>SX0@41002PX:01:BTX0CDe=03Ph>00o?cl0@D5103/k>`0c<c<0BTY:055AD@0m?Cd0BTY:
04m?C`1:BTX0>c/k069RHP80B4Q801T0P82005MGE`12@T80FUYJ04e=C@1MGEd0BTY:04M7A`1>CTh0
JVYZ04A4A01EEED0CTi>05YJFP0d=3@0GEeM04Q8B014A4@0KVi^07YjNP2ATI40PX:2038b<P1EEED0
Ph>30080IFEU03T0KVi^08R8R01CDe<0MGEe051@D01;Bd/0QHF5075aL@1VIVH0BTY:0492@P1CDe<0
Hf=S09RHV01<C4`0De=C04a<C01TI6@0Cdm?075aL@1HF5P0820P0410@01VIVH09bLW06a/K018B4P0
OWin04M7A`11@D40@T920410@01HF5P0@d=3051@D00e=CD0N7Qh03/k>`1XJ6P0>3Ph059BDP0l?3`0
>3Ph05YJFP0o?cl0EEEE09FEU@0g=cL0DE5A079bLP0b<S80GUiN04i>CP1ADE40C4a<096AT@2VYZH0
0P1SHf<0<011@D40J6QX02d];@1YJFT0<30`05eMG@1NGUh0IVIV04m?C`2@T900>CTi051@D01?Cdl0
BTY:07AdM017AdL0?clo05QHF01BDU80De=C0492@P1MGEd0Cdm?05IFEP1:BTX0=3@d04];B`1bLW80
FUYJ03`l?01dM7@0BTY:06QXJ00n?Sh0Fe]K028R8P1EEED0FEUI03Ph>016ATH0ADE504A4A00k>c/0
J6QX02l_;`0n?Sh0G5aL05UIF@80D51@00l0AdM706IVIP1CDe<0HF5Q05eMG@10@400HF5Q03De=@1G
EeL0?Shn04Q8B01ZJVX0C4a<04m?C`1]KFd00P1QHF42079bLP0D03hn?P1<C4`0>SXj05QHF01:BTX0
ADE505ADE00G5aL0B4Q80410@01;Bd/0LW9b0:6QX@14A4@0CDe=07AdM01EEED0GUiN05YJFP1[Jf/@
ooooo`003_ooool0o`2SXj<0H61P05EEE@1NGUh0NGUi0820P00e=CD0G5aL08R8R01_Kfl0TI6A03Hf
=P19BDT0YZJV08N7Q`1DE5@0FUYJ01dM7@1XJ6P0VYZJ05ADE01HF5P0?Cdm01`L701UIFD0Ti>C04E5
A@1jNWX0;Rh^04e=C@2=SHd0<S8b03`l?01JFUX06a/K03Lg=`1nOWh08b<S03@d=00j>SX0;Bd]06=S
H`2PX:00=SHf03Lg=`13@d<0E5AD09VIV@26QXH0:b/[059BDP0m?Cd0HF5Q0:f][@0e=CD06ATI04a<
C019BDT0IFEU0820P00W9bL0;Bd]08J6QP17AdL0SXj>065QH@0N7Qh0<C4a08:2PP2<S8`0J6QX01DE
5@0_;bl0De=C079bLP1;Bd/0JFUY018B4P0a<C40Bd];061PH01EEED0GEeM01XJ6P0d=3@0HV9R055A
D@0_;bl0LG5a03lo?`1UIFD0MWIf02/[:`0c<c<0DE5A03De=@1eMGD0JVYZ02h^;P0i>CT0>c/k0492
@P1OGel0=cLg02`/;016ATH0GUiN05]KF`1GEeL0BDU903`l?00h>3P0KVi^055AD@1gMgL0C4a<03Ti
>@0j>SX0Fe]K05ADE02=SHd0O7al03@d=01GEeL0KFe]04Q8B02;Rh/0PX:201TI6@1_Kfl0EUIF04e=
C@2MWId0?Shn028R8P1fMWH0>3Ph04Y:BP2BTY80@D5102l_;`2:RXX0JFUY06a/K01RHV807QhN02TY
:@1cLg<0JVYZ06ATI018B4P07AdM03Lg=`1dM7@0GemO03<c<`0i>CT0>3Ph04];B`2DU9@0Cdm?02LW
9`11@D40<C4a0820P02c/k<0F5QH02`/;01>CTh0JVYZ0<;2`P1NGUh0;Bd]03Xj>P0o?cl0EUIF0:2P
X00Z:RX0=CDe059BDP17AdL0DU9B08F5Q@0_;bl0F5QH06UYJ@1JFUX0P82007]kN`1KFe/0R8R806AT
I01ADE40SXj>051@D011@D40MgMg04A4A01YJFT0HV9R04A4A01RHV80DU9B08j>SP1;Bd/0?3`l04I6
AP2FUYH0OGem07]kN`13@d<0AdM707moO`1eMGD0Cdm?051@D00_;bl0Hf=S07inOP1DE5@0J6QX04A4
A01HF5P0UiNG069RHP0l?3`0GUiN03hn?P1SHf<0SXj>05=CD`1=CDd0IFEU065QH@1NGUh0MGEe03hn
?P11@D40De=C04];B`1XJ6P0VIVI02TY:@10@400BDU906ATI025QHD0XJ6Q03Ph>01CDe<0Fe]K04I6
AP1bLW80/K6a04];B`005`0^;Rh0I6AT06MWI`1DE5@0T92@02`/;00e=CD0LW9b07IfMP1/K6`0LW9b
03Ph>00/;2`0FUYJ079bLP0k>c/0Cdm?02TY:@0X:2P0Jf][07alO00Z:RX0H61P00kooooo03X0VYZJ
08f=S@1ADE40LG5a07IfMP1OGel0>CTi05iNGP1gMgL0BDU905IFEP1`L700IVIV0820P01/K6`0IFEU
06YZJP1FEUH0D51@01hN7P18B4P0>SXj04e=C@14A4@0?clo04E5A@1QHF40GemO09BDU018B4P0CDe=
07moO`1gMgL0Bd];05=CD`1KFe/0HF5Q02l_;`17AdL0ADE503Ph>017AdL0E5AD05UIF@11@D40ADE5
05YJFP1BDU80Fe]K04Q8B01UIFD0=CDe038b<P1CDe<0FUYJ065QH@0U9BD0DE5A0P0c<c<0I@14A4@0
B4Q804A4A018B4P0MGEe05aLG01DE5@0DE5A05=CD`1:BTX0;Bd]02XZ:P1?Cdl0@41004=3@`1<C4`0
BTY:030`<013@d<0;Rh^03Xj>P1;Bd/0@D5103`l?00m?Cd05aLG02HV9P0i>CT0<C4a02`/;00];Bd0
?Shn02LW9`0X:2P0>3Ph03De=@0a<C40=3@d04A4A00];Bd0>3Ph02XZ:P14A4@0DU9B03dm?@13@d<0
DU9B0410@016ATH0AdM704I6AP1XJ6P0Ogmo04Y:BP1EEED0EeMG05iNGP0d=3@0De=C06IVIP15ADD0
=SHf04=3@`2j^[X0Ph>302@T901YJFT0I6AT07moO`1]KFd0IFEU075aL@0e=CD0CDe=04I6AP12@T80
?Cdm05QHF01mOGd0Ng]k05IFEP1gMgL0U9BD0<71`@1IFET0KFe]03/k>`1FEUH0A4A407UiN@1UIFD0
EeMG071`L022PX80O7al09>CT`1[Jf/0TI6A05UIF@0o?cl0B4Q80080H61P06P0EeMG06][J`12@T80
HF5Q05iNGP1iNGT0Jf][06ATI01VIVH0<30`05EEE@1FEUH0FUYJ06m_K`11@D40Rh^;08j>SP0n?Sh0
>c/k051@D02CTi<0>c/k051@D01OGel0=3@d04=3@`0L71`0@D5103@d=01;Bd/0>CTi05IFEP1JFUX0
K6a/04Q8B01NGUh0=cLg06MWI`1DE5@0:b/[04i>CP0f=SH0CDe=02h^;P18B4P0CTi>02d];@1<C4`0
<c<c03lo?`15ADD0:b/[03hn?P1]KFd0DU9B03Xj>P2KVi/0?Shn05IFEP0m?Cd0CTi>01TI6@1MGEd0
@D5102<S8`0l?3`0C4a<04Y:BP1ADE40H61P05QHF01;Bd/0FEUI05=CD`1LG5`0MGEe02h^;P1GEeL0
CDe=05ADE00m?Cd09bLW04m?C`0d=3@0J6QX05QHF00l?3`0ADE504=3@`1;Bd/0CDe=05UIF@1/K6`0
Ti>C06][J`14A4@0T92@04i>CP1CDe<09RHV04i>CP1XJ6P0?3`l05mOG`80@d=300T0LW9b0:2PX00o
?cl0?Shn059BDP1CDe<0EEEE059BDP1jNWX04?ooool000kooooo0400I6AT0;Fe]@1^KVh0GEeM069R
HP2OWil0FUYJ09bLW01GEeL0B4Q8075aL@0k>c/0?Shn04Q8B00c<c<0RHV906ATI00G5aL0;Rh^04];
B`13@d<0UIFE06m_K`0U9BD0BTY:05=CD`1dM7@0`L7102XZ:P0f=SH0Lg=c03lo?`1lO7`0WinO024Q
8@0X:2P0EeMG04e=C@21PH40GemO01hN7P0h>3P0I6AT07AdM02<S8`0Ph>301lO7`1:BTX0RXZ:05EE
E@1[Jf/09BDU02HV9P1BDU80FUYJ0410@018B4P0820P03Xj>P2IVIT0K6a/04A4A01FEUH0A4A40P1V
IVH0LP0S8b<0?clo04e=C@1?Cdl0QXJ606EUI@0P8200BDU903lo?`16ATH0TY:B02HV9P0c<c<0DU9B
04U9B@1MGEd0V9RH02h^;P0V9RH0EeMG07QhN01CDe<0`/;202PX:012@T80RHV904I6AP1bLW80S8b<
02@T9012@T80SXj>03Lg=`1bLW80FUYJ02TY:@1:BTX0Hf=S0451@@1jNWX0=CDe01LG5`1:BTX0Hf=S
03Xj>P1mOGd08b<S01lO7`1<C4`0DE5A04I6AP1VIVH0>c/k02DU9@17AdL0UIFE05YJFP2BTY80:b/[
02d];@1ADE40MGEe034a<@0f=SH0;bl_04I6AP1MGEd0ATI602<S8`19BDT0:2PX03<c<`25QHD0CTi>
024Q8@15ADD0@D5105iNGP24Q8@07QhN020P801@D500EEEE0:NWY`1cLg<08b<S03@d=01QHF40Jf][
07YjNP10@4008R8R030`<01lO7`0UYJF04Y:BP1VIVH07alO03Ph>022PX80RXZ:07MgM`0c<c<0=cLg
04M7A`1[Jf/0Cdm?0:>SX`1CDe<0<S8b05aLG01VIVH0BDU905EEE@80:BTY03l0K6a/03/k>`18B4P0
BTY:02h^;P10@400MgMg03lo?`1CDe<0=CDe03dm?@1MGEd0Lg=c04M7A`0n?Sh0C4a<07=cL`1jNWX0
?clo04M7A`13@d<0a/K609ZJVP0b<S80B4Q805]KF`1ZJVX0`<3003Ti>@1?Cdl0J6QX05YJFP1TI6@0
M7Ad02l_;`1EEED0De=C05IFEP2a/K40@41005ADE02PX:00Jf][07=cL`2l_;`0HF5Q04=3@`1CDe<0
JVYZ06i^KP2<S8`0=SHf04Q8B01TI6@0IVIV09nOW`2;Rh/08B4Q03Xj>P1fMWH0MgMg0:BTY020P800
0P0];Bd07`1jNWX0IFEU04Q8B01>CTh0>c/k03Lg=`1]KFd0NGUi04];B`1OGel0<C4a0410@01ZJVX0
J6QX03/k>`0j>SX08B4Q05=CD`1`L700KVi^03<c<`10@400>3Ph07EeM@2?Shl0@41001/K6`1<C4`0
E5AD06MWI`2n_[h03_ooool0o`25QHD0JFUY04I6AP1FEUH0K6a/061PH01=CDd0KFe]061PH01<C4`0
LW9b096AT@2;Rh/0R8R80:BTY02DU9@0MGEe05IFEP1GEeL0<30`0451@@0k>c/0?Shn059BDP10@400
=cLg05aLG01>CTh0CDe=0492@P13@d<0De=C07IfMP1ADE40EeMG06][J`1GEeL092@T03Xj>P18B4P0
>3Ph05=CD`0j>SX0Cdm?03De=@1>CTh0HF5Q03hn?P0d=3@0ADE505=CD`0a<C40>CTi04];B`1LG5`0
FEUI034a<@1GEeL0?3`l04];B`1EEED0F5QH05ADE01UIFD0WYjN06][J`0a<C40>CTi04U9B@1RHV80
<c<c03`l?017AdL07QhN028R8P1<C4`0FUYJ03dm?@15ADD0ATI604];B`0i>CT0ATI603@d=00i>CT0
71`L059BDP16ATH0;Rh^03/k>`0d=3@0>CTi02DU9@1:BTX0Cdm?03Hf=P11@D40A4A404I6AP0f=SH0
ADE502<S8`12@T80K6a/03hn?P1=CDd0FUYJ0451@@1?Cdl0EUIF04=3@`10@400J6QX02d];@1^KVh0
ADE504M7A`0h>3P0@T9202l_;`0/;2`0FUYJ04I6AP1cLg<0B4Q8024Q8@1>CTh0=cLg02l_;`0k>c/0
SHf=06e]K@0U9BD0FUYJ030`<00i>CT0GemO05=CD`1XJ6P0C4a<08F5Q@1IFET0CDe=09jNWP0o?cl0
DE5A0451@@1@D500<30`05ADE01ADE40:2PX07EeM@2RXZ80?3`l06YZJP1XJ6P0P82007AdM01IFET0
ADE506ATI01OGel0FEUI05ADE00/;2`0HV9R02h^;P2MWId0@D5105mOG`1WIfL0DE5A065QH@1:BTX0
CDe=08F5Q@1?Cdl0IfMW08^;R`15ADD0BTY:04Q8B01:BTX0J6QX09>CT`2IVIT0>SXj04Q8B00d=3@0
D51@03Hf=P1HF5P0ATI605UIF@1>CTh0De=C05QHF01lO7`0?3`l08b<S01dM7@0AdM706EUI@0c<c<0
?Shn02l_;`0m?Cd0JFUY059BDP1DE5@0Bd];05iNGP1@D500A4A4069RHP1aLG40FUYJ05UIF@1lO7`0
FUYJ05eMG@1?Cdl0GUiN034a<@1WIfL0FUYJ04E5A@1FEUH0CTi>05QHF01:BTX0F5QH03Hf=P18B4P0
K6a/05YJFP13@d<0@D5102XZ:P1EEED0CTi>05]KF`0g=cL0;Rh^05IFEP15ADD0MgMg04];B`17AdL0
NWYj04e=C@1?Cdl0GEeM06][J`005`1VIVH0KVi^06m_K`0h>3P0@T9203/k>`1IFET0>c/k04I6AP1A
DE407QhN05IFEP0d=3@0HV9R059BDP1]KFd0De=C05aLG01BDU80JVYZ05IFEP1VIVH0Xj>S013ooooo
000>ooooo`0S05iNGP1oOgl0Vi^K06MWI`12@T80JVYZ071`L03Jf]X0PX:203Hf=P1aLG40Jf][0:>S
X`2;Rh/06a/K03@d=01/K6`0EeMG08>3P`1>CTh08b<S03Hf=P1=CDd0>c/k06UYJ@0V9RH0=cLg05aL
G01<C4`0>SXj03@d=00Q8B40Cdm?0:f][@1jNWX00P0`<300I@0/;2`0=cLg07QhN00h>3P0<C4a03`l
?010@400GUiN08F5Q@1<C4`0>c/k04Q8B01MGEd0YJFU02h^;P0M7Ad0>CTi03`l?017AdL0PH6101LG
5`0I6AT0>c/k04I6AP1EEED0UYJF028R8P1:BTX0Fe]K04=3@`2FUYH0b/[:03<c<`1dM7@0NGUi05EE
E@2e]KD0C4a<01PH601SHf<0@d=306QXJ02JVYX0BTY:02d];@1nOWh0@D5104i>CP1TI6@09bLW03Ph
>01nOWh0H61P055AD@0f=SH0;bl_0492@P1MGEd0?Shn05=CD`0e=CD0=cLg05QHF01kNg/0:b/[034a
<@0m?Cd0EeMG09BDU01@D5006ATI03De=@1;Bd/0G5aL09fMW@0g=cL06a/K04A4A013@d<0LG5a08b<
S00_;bl09BDU03De=@13@d<0H61P0=oOg`16ATH0=3@d07emO@1nOWh0@d=308f=S@0M7Ad0<C4a07=c
L`1JFUX0ADE5071`L00E5AD0A4A40080EeMG04L0?Shn0:b/[00b<S80G5aL07MgM`1<C4`0?Cdm06m_
K`0Y:BT0AdM707YjNP0j>SX0Cdm?05iNGP0g=cL0CDe=09FEU@1@D500<30`05]KF`0T92@0@D5106EU
I@0^;Rh0=3@d04A4A00l?3`0F5QH04E5A@0c<c<0@d=3024Q8@0e=CD0RXZ:09FEU@0l?3`0GUiN04Q8
B01mOGd0UYJF03`l?00S8b<0AdM703dm?@20P800NGUi02`/;01=CDd0FUYJ06i^KP2l_;`0;2`/03Hf
=P1VIVH0Ng]k07UiN@16ATH0:b/[06][J`2?Shl0IfMW05EEE@0U9BD0KVi^0;Fe]@1_Kfl0JVYZ06QX
J00j>SX0MWIf05mOG`0206a/K01102HV9P0n?Sh0G5aL04a<C020P8007alO034a<@2IVIT0Fe]K051@
D01ZJVX0>c/k0492@P2IVIT0[Zj^0451@@1WIfL0>3Ph055AD@2m_Kd0OWin0451@@1ADE40=cLg06i^
KP32`/80HF5Q03lo?`13@d<0=CDe07MgM`3:b/X0CDe=034a<@1?Cdl0GemO0:^[Z`1FEUH06a/K03Ph
>01@D500HV9R06EUI@2MWId09BDU034a<@1dM7@0JFUY05QHF01:BTX08R8R02PX:01KFe/0PX:205UI
F@1iNGT07alO02LW9`1WIfL0YZJV04m?C`24Q8@0?clo02LW9`1IFET03_ooool0G`27QhL0Ogmo03De
=@1=CDd0MWIf06IVIP1CDe<0P82007AdM01;Bd/0G5aL06UYJ@2=SHd0K6a/05]KF`1CDe<0FUYJ05=C
D`1KFe/0820P04=3@`0e=CD0BTY:04i>CP1EEED0IfMW08:2PP1dM7@0Fe]K04a<C018B4P0Kfm_06IV
IP1<C4`0H61P07AdM01JFUX0@410051@D01CDe<0Bd];05IFEP0i>CT0G5aL03@d=011@D40HV9R04A4
A00o?cl0@d=305IFEP0];Bd0:BTY04a<C01IFET0J6QX020P800o?cl0CDe=05EEE@1:BTX0ADE504U9
B@1;Bd/0CTi>055AD@19BDT0A4A404Y:BP1DE5@0Cdm?051@D01QHF40=SHf03@d=01:BTX0EEEE05MG
E`1DE5@0@T9204A4A00j>SX0Bd];03Xj>P0h>3P0;2`/02LW9`15ADD0;Rh^03hn?P0h>3P0>c/k04M7
A`0h>3P0ATI60080?Shn0P14A4@02`0e=CD0DE5A01dM7@10@400IVIV03/k>`0n?Sh0Cdm?04U9B@1L
G5`0@4100080BTY:04`0SHf=02l_;`0n?Sh0CDe=05iNGP1>CTh0F5QH04M7A`1;Bd/0JFUY05]KF`1c
Lg<0K6a/0410@01/K6`0JFUY061PH01KFe/0De=C051@D00^;Rh0JVYZ03hn?P0/;2`0OGem03dm?@1d
M7@0B4Q803@d=013@d<0K6a/09>CT`12@T80KFe]03Lg=`1FEUH0=cLg08:2PP1FEUH0820P03Xj>P18
B4P09RHV065QH@0j>SX0TI6A04=3@`0i>CT0<c<c05eMG@18B4P0D51@05ADE014A4@0T92@0410@01Q
HF40>SXj06a/K01FEUH0?Shn06IVIP0c<c<0>SXj0861P@0f=SH0Kfm_06a/K00[:b/0JVYZ065QH@1I
FET0>c/k059BDP1PH600@4100P12@T805P1DE5@0:BTY05iNGP0_;bl0IFEU05YJFP1;Bd/0Fe]K07in
OP0f=SH0EeMG06EUI@10@400IVIV05=CD`1ADE40GUiN05QHF01ZJVX0EeMG06e]K@1<C4`204m?C`0Y
02d];@1>CTh0PH6107QhN01QHF40U9BD079bLP1hN7P0Fe]K07EeM@1<C4`0JFUY05eMG@1VIVH0De=C
03`l?01UIFD0CDe=04m?C`1KFe/0Hf=S04a<C01BDU80K6a/06YZJP10@400LG5a04e=C@2/[:`0LG5a
03`l?01cLg<0>CTi04m?C`0n?Sh0LG5a08Z:RP1UIFD0?clo04];B`1`L7000P1ZJVX05@1KFe/0CTi>
0:BTY01RHV80A4A40492@P1]KFd0N7Qh04i>CP1DE5@0@41005QHF022PX80SXj>04];B`17AdL0H61P
05EEE@1iNGT0O7al06UYJ@0@ooooo`003_ooool0W@2WYjL0AdM706IVIP1cLg<0J6QX05]KF`0g=cL0
JVYZ0:FUY@19BDT0>3Ph04A4A019BDT0R8R8051@D01:BTX0B4Q803Ti>@1ZJVX0Hf=S024Q8@12@T80
>SXj04=3@`2ZZZX0Bd];01XJ6P1>CTh0FUYJ07emO@1HF5P05ADE02<S8`1KFe/0SHf=06IVIP1MGEd0
8R8R02d];@1XJ6P0F5QH08J6QP2IVIT0;Rh^04m?C`28R8P0EEEE05MGE`0l?3`0Cdm?07moO`1>CTh0
BDU904a<C00K6a/0<c<c06MWI`0a<C40HV9R05iNGP0L71`0>SXj07moO`0m?Cd0Jf][04];B`0Z:RX0
GemO09NGU`1[Jf/0HF5Q02d];@13@d<0Ogmo06EUI@0d=3@0D51@028R8P1QHF40[:b/04i>CP0E5AD0
DE5A02d];@1FEUH0RHV901LG5`0L71`0?3`l059BDP2YZJT0M7Ad01DE5@0g=cL0IFEU05YJFP2l_;`0
D51@01HF5P0n?Sh0LG5a05ADE01cLg<08b<S02l_;`1NGUh0TY:B04Y:BP2?Shl0G5aL03dm?@1MGEd0
Ph>303`l?01/K6`0>c/k06=SH`1nOWh0Ogmo02DU9@0b<S80DU9B02d];@19BDT0LW9b03Lg=`1<C4`0
ADE503`l?01RHV80De=C030`<01/K6`0<30`03/k>`2CTi<0=cLg0410@01SHf<0=SHf030`<02[Zj/0
Kfm_0410@01MGEd0EEEE051@D030`<00DE5A04I6AP1<C4`0B4Q805IFEP2HV9P0<C4a04U9B@1KFe/0
0P1fMWH01@2d];@0;bl_03lo?`1OGel0Jf][0080MWIf00T0:2PX03lo?`1@D500HV9R08j>SP0a<C40
;Bd]02l_;`1bLW800P1SHf<02@0K6a/0:b/[07MgM`1^KVh0ATI607=cL`10@400A4A4079bLP0204A4
A01J0410@019BDT0G5aL04e=C@0/;2`0FEUI03`l?01@D500MWIf02/[:`13@d<0FEUI03Ph>01kNg/0
:BTY05IFEP10@400<S8b06m_K`2SXj<0Bd];05=CD`13@d<0JFUY08Z:RP0/;2`0>CTi05eMG@1_Kfl0
]KFe03dm?@0X:2P0J6QX06m_K`2:RXX0WYjN03Hf=P1<C4`0E5AD08^;R`22PX80LG5a03Ph>015ADD0
J6QX0:VYZ@1nOWh0CDe=04Q8B01>CTh0I6AT09BDU022PX80LG5a03Hf=P10@400J6QX08Z:RP1DE5@0
JVYZ03<c<`1;Bd/0QXJ604m?C`1IFET0NWYj0492@P1GEeL0OGem05]KF`11@D40JFUY03dm?@1;Bd/0
MgMg03Ph>00b<S80BTY:03@d=00b<S80GemO02/[:`0V9RH0=cLg02XZ:P0W9bL0OWin03De=@0/;2`0
?clo3_ooool09P2EUID0PX:20492@P1ADE40EEEE05ADE01;Bd/0Fe]K07AdM00m?Cd0GemO08f=S@2a
/K40T92@07EeM@1gMgL0MGEe05IFEP1EEED09RHV04=3@`0c<c<0@T9204E5A@0n?Sh0@41006EUI@1K
Fe/0JFUY05mOG`1IFET0HV9R06i^KP1LG5`0Ogmo05QHF01EEED0=SHf0P12@T805`0a<C40ADE503Lg
=`1=CDd0<c<c04E5A@1CDe<0CDe=04];B`0k>c/0IfMW03Hf=P0Y:BT0BTY:04U9B@1LG5`0;Rh^04M7
A`1DE5@0DE5A06MWI`1OGel0FUYJ0080KFe]00`0L71`03/k>`11@D40Bd];04m?C`0b<S80>c/k05EE
E@0V9RH0:RXZ04];B`1=CDd204Y:BP0903dm?@15ADD0ADE504U9B@18B4P0@D5102PX:00g=cL0DU9B
0080?3`l07l0@41004E5A@0d=3@0=SHf0410@01:BTX0ADE505IFEP1@D500=CDe04A4A00W9bL0ATI6
06UYJ@1@D500ATI605YJFP0g=cL0FEUI04Q8B01YJFT0:2PX07IfMP1ADE40Ogmo05ADE01OGel0<C4a
04A4A01JFUX0?Cdm05=CD`0j>SX0WIfM075aL@0X:2P0EUIF05MGE`1[Jf/0KVi^065QH@16ATH08b<S
07AdM017AdL0@T9205MGE`0/;2`0CTi>02`/;01;Bd/0=SHf05]KF`3=cLd0>3Ph04A4A013@d<0P820
03Ti>@1SHf<0EeMG01LG5`1NGUh0L71`01dM7@1lO7`0B4Q803Ph>00b<S80C4a<04I6AP14A4@0=3@d
055AD@1<C4`07QhN04Q8B00N7Qh0Ph>305MGE`1lO7`0FUYJ0492@P1]KFd0CTi>05]KF`2FUYH0Fe]K
09jNWP26QXH0?Cdm04A4A01QHF40RHV906a/K024Q8@0Ph>305iNGP1JFUX0>3Ph03Ti>@0b<S80N7Qh
055AD@1>CTh0Cdm?05QHF00l?3`0Fe]K038b<P2TY:@0U9BD051@D01gMgL0Bd];069RHP0g=cL0AdM7
05YJFP0g=cL0B4Q80492@P1GEeL0ADE502LW9`1;Bd/0G5aL0080CDe=00/0RXZ:04A4A01=CDd0@T92
065QH@0i>CT0F5QH04Q8B00m?Cd0A4A403Ph>00204I6AP0^05]KF`18B4P0BTY:07inOP1XJ6P0D51@
04A4A00n?Sh0AdM703dm?@1hN7P0B4Q803De=@1CDe<0;Rh^08>3P`15ADD0?Cdm03`l?013@d<0?clo
055AD@1:BTX0HV9R07alO01:BTX0<C4a04a<C01FEUH0GUiN03Ph>00l?3`0C4a<030`<01:BTX07AdM
04e=C@2HV9P0[Jf]05MGE`1OGel0MGEe05YJFP1TI6@0DU9B07UiNA3ooooo000>ooooo`3I0:j^[P2D
U9@0Bd];07alO026QXH0OWin034a<@1IFET0Ti>C0820P01QHF40ADE505aLG01BDU80H61P07]kN`1<
C4`0:b/[06QXJ01dM7@0;bl_08N7Q`0h>3P0=cLg08Z:RP1aLG40CDe=05]KF`0O7al0?Cdm08:2PP15
ADD0DE5A05ADE00b<S80=SHf05mOG`0V9RH0?3`l03Ph>00Z:RX0@41007QhN00m?Cd0?3`l03Ph>01I
FET0L71`05iNGP0c<c<0Jf][038b<P1IFET0OWin028R8P0G5aL0GUiN03`l?02>SXh0J6QX01`L700K
6a/0K6a/05iNGP2[Zj/0@410020P800_;bl0RXZ:0820P02][Jd07QhN02@T9016ATH0IFEU04=3@`1]
KFd092@T02l_;`1;Bd/0TY:B04M7A`1dM7@092@T02h^;P23Ph<0DE5A04m?C`1WIfL0:2PX05]KF`2Q
XJ40<c<c03Ti>@1IFET0:RXZ08^;R`2IVIT0;Bd]05QHF00n?Sh092@T04Q8B010@400:b/[0410@00]
;Bd0820P03Xj>P1OGel0<c<c0410@00_;bl0@D51055AD@14A4@0>CTi04a<C01?Cdl0E5AD08J6QP1L
G5`0<C4a04Q8B01HF5P0EeMG0:VYZ@1:BTX08B4Q04M7A`0l?3`0DU9B092@T00g=cL09bLW04m?C`13
@d<0NWYj09NGU`0f=SH0@D510410@01SHf<0K6a/03hn?P0U9BD0?clo08R8R02=SHd0OGem061PH00`
<300ADE508B4Q01hN7P0I6AT03hn?P0`<300G5aL08J6QP1dM7@0?Shn04e=C@0l?3`0CDe=09^KV`0n
?Sh0?clo03dm?@11@D40QXJ6079bLP0f=SH0EEEE0451@@19BDT0X:2P0492@P0c<c<0ATI60492@P0l
?3`0LW9b02@T901=CDd0Cdm?04e=C@1oOgl0Ng]k04I6AP1WIfL0I6AT06=SH`2GUiL0EEEE03hn?P23
Ph<0B4Q808:2PP2CTi<0=CDe051@D00o?cl0UiNG05UIF@0c<c<0B4Q809JFUP1fMWH0Q8B403`l?01>
CTh0Hf=S071`L01EEED0Cdm?04];B`0206YZJP0k04A4A023Ph<0@T9208:2PP2VYZH0HV9R05MGE`2@
T900>CTi06IVIP2SXj<0H61P03lo?`1QHF40ATI607UiN@1VIVH0<S8b04a<C00f=SH0@41006YZJP2G
UiL0=3@d0451@@15ADD0A4A40410@029RHT0ADE50492@P14A4@0EeMG05=CD`2@T900@41005UIF@1U
IFD0GUiN06e]K@2KVi/0@T9203hn?P1_Kfl0W9bL05iNGP27QhL0?Cdm034a<@1ADE40F5QH05UIF@25
QHD0;bl_020P801NGUh0QhN705UIF@1mOGd03_ooool0^@1gMgL0QXJ604I6AP1?Cdl0Kfm_05mOG`0d
=3@0DU9B06EUI@1HF5P0IfMW06UYJ@1IFET0EeMG09:BTP27QhL0Fe]K04U9B@1@D500>CTi055AD@1>
CTh0@d=304A4A018B4P0A4A406m_K`1NGUh0PX:2059BDP0o?cl0J6QX04];B`1BDU80IfMW05QHF01D
E5@0>CTi04a<C01?Cdl0AdM705UIF@12@T80FUYJ03Ti>@1<C4`0EeMG03hn?P0f=SH0>3Ph05ADE00_
;bl0>SXj04A4A01@D500HF5Q038b<P1ADE40@d=304i>CP1GEeL0EEEE04];B`1?Cdl0KVi^05iNGP0h
>3P0?Shn04a<C01IFET0@41004a<C01bLW80;Bd]02l_;`1@D500E5AD04a<C01;Bd/0?3`l04m?C`0h
>3P0BTY:03lo?`0n?Sh071`L038b<P1@D500>3Ph03lo?`19BDT0BTY:0451@@0b<S80@T9204i>CP1;
Bd/0FEUI0451@@10@400D51@04e=C@1;Bd/0JVYZ04];B`1MGEd0Jf][04E5A@1PH600DU9B06a/K00b
<S80Kfm_04i>CP1;Bd/0E5AD06QXJ00n?Sh0H61P04M7A`0o?cl0H61P055AD@2GUiL0MWIf02`/;01Q
HF40MGEe06][J`1SHf<0[Jf]08R8R013@d<0MGEe04U9B@1DE5@0Jf][04i>CP1=CDd0;Rh^069RHP1<
C4`0FEUI0:ZZZP1ADE40ATI603@d=01JFUX0<30`06m_K`1@D500;Bd]06m_K`1jNWX07alO07alO00k
>c/0N7Qh03dm?@1KFe/0BTY:05QHF016ATH0K6a/05QHF00];Bd0Rh^;03`l?01]KFd0=CDe04Q8B00m
?Cd0ADE5065QH@17AdL0BTY:06a/K00`<300HV9R06m_K`0h>3P0ADE50451@@1=CDd0BDU90080Ti>C
05/0E5AD03Hf=P0X:2P0DE5A018B4P1DE5@0:RXZ05ADE00m?Cd0ATI604i>CP1eMGD08R8R04I6AP0d
=3@09bLW059BDP0S8b<0@410034a<@1>CTh0C4a<04M7A`1BDU80BTY:04i>CP1BDU80@41005MGE`1X
J6P0HF5Q05aLG028R8P0ADE5055AD@1<C4`0E5AD02<S8`1YJFT0@T9203`l?01GEeL0>CTi04i>CP1:
BTX0R8R804];B`1@D500IfMW05=CD`0n?Sh0B4Q80451@@1_Kfl0E5AD09bLW017AdL0<c<c04m?C`0m
?Cd0Cdm?06][J`1EEED0GemO04M7A`1KFe/0H61P05QHF01<C4`0VIVI03Ti>@0g=cL0HF5Q03`l?016
ATH0>CTi04Y:BP1<C4`06QXJ075aL@0M7Ad0>SXj04m?C`1RHV80>CTi03Xj>P1;Bd/0ADE505iNGP1;
Bd/0M7Ad013ooooo000>ooooo`3o07emO@2EUID0RXZ:061PH01KFe/0MgMg05]KF`2ATI40C4a<05=C
D`1ZJVX0EUIF065QH@1^KVh08B4Q05=CD`1ZJVX0B4Q807MgM`1YJFT0=3@d05aLG01GEeL0BDU907Yj
NP12@T80D51@08B4Q018B4P0CTi>03`l?011@D40Fe]K08^;R`12@T80B4Q804U9B@1<C4`0H61P05eM
G@0b<S80@D5104e=C@1LG5`0[Jf]06=SH`0^;Rh0AdM704a<C01BDU80Q8B4030`<012@T80FEUI03hn
?P0g=cL0Lg=c02DU9@13@d<0Cdm?051@D016ATH0GEeM02d];@1LG5`0GemO0492@P21PH40IVIV038b
<P1KFe/0?Shn04Q8B02EUID0>CTi02d];@1fMWH0?3`l051@D01hN7P0<S8b05YJFP1gMgL0=CDe061P
H01LG5`0?Cdm061PH01iNGT0ADE5059BDP1XJ6P0=SHf05QHF02=SHd0?3`l05UIF@1RHV80<30`04m?
C`1^KVh0;bl_04A4A00f=SH0FEUI08>3P`17AdL09RHV03/k>`0o?cl0J6QX09FEU@10@400;Rh^051@
D013@d<0CDe=09ZJVP19BDT08B4Q051@D025QHD0D51@08Z:RP0i>CT09BDU05iNGP25QHD0@d=305AD
E00e=CD0@d=307MgM`1LG5`0Cdm?05ADE00S8b<0BTY:07MgM`1SHf<0ATI609:BTP11@D40GUiN08j>
SP1OGel0EeMG09>CT`0o?cl0KFe]08:2PP1JFUX0@T9205]KF`15ADD0G5aL071`L00b<S80I6AT06EU
I@0];Bd0CTi>06e]K@0j>SX0KFe]04U9B@17AdL0EeMG05IFEP12@T80IFEU03/k>`1?Cdl0M7Ad069R
HP15ADD0LG5a04m?C`1MGEd0X:2P04M7A`0c<c<0IVIV03hn?P1iNGT0UIFE0492@P17AdL0HF5Q06i^
KP2/[:`0B4Q803Ph>019BDT0HV9R07emO@1hN7P0;bl_051@D028R8P0IVIV075aL@0b<S80GemO06a/
K015ADD0Shn?06QXJ016ATH0QHF5069RHP1iNGT0Lg=c02h^;P1KFe/0Ng]k04U9B@1VIVH0>c/k059B
DP2?Shl0KFe]05YJFP1bLW80Bd];07MgM`2YZJT0GEeM05EEE@1MGEd0De=C06][J`2IVIT0IFEU065Q
H@1NGUh0GemO09>CT`1/K6`0B4Q805YJFP1lO7`0Fe]K06m_K`2;Rh/0@d=30451@@1[Jf/0Fe]K07]k
N`1[Jf/0;Rh^03lo?`1>CTh0FEUI000G07alO01LG5`0:2PX04m?C`1CDe<0Cdm?05ADE01ZJVX08R8R
03Ti>@1SHf<0MGEe0451@@1aLG407alO03Hf=P1UIFD0L71`03Lg=`1[Jf/0;Bd]04a<C01VIVH03_oo
ool0B@1/K6`0MWIf04m?C`1DE5@0EUIF05ADE015ADD0GUiN07MgM`11@D40FEUI07MgM`21PH40Kfm_
061PH01RHV80IfMW05eMG@1;Bd/09RHV03/k>`0`<300B4Q804a<C01:BTX0G5aL06YZJP1SHf<0QHF5
05YJFP1CDe<0K6a/05MGE`1JFUX0Kfm_05]KF`1ZJVX0=SHf03De=@11@D40>c/k055AD@0i>CT0CTi>
03/k>`0l?3`0@d=30492@P0d=3@0=SHf04m?C`0d=3@0;bl_0410@01=CDd0Fe]K02l_;`0i>CT0@D51
04i>CP1FEUH0Cdm?05=CD`1QHF40KFe]06IVIP10@400BTY:05EEE@1IFET0B4Q8051@D01JFUX00P0`
<3009P1CDe<0EEEE04Q8B017AdL0?clo04U9B@0o?cl0AdM70410@012@T80;2`/03Ti>@0o?cl0<c<c
03/k>`0c<c<0AdM704=3@`0c<c<0?3`l04A4A013@d<0DE5A04E5A@0^;Rh0CTi>02PX:00m?Cd0KVi^
04Y:BP19BDT0Fe]K03/k>`1LG5`0Bd];06][J`0U9BD0KFe]0P1>CTh0H@1FEUH0FEUI04E5A@1GEeL0
CDe=059BDP1fMWH0HF5Q0:VYZ@1oOgl0@d=308f=S@23Ph<0I6AT05QHF02MWId0N7Qh02XZ:P1]KFd0
?3`l04=3@`1`L700E5AD079bLP14A4@0DU9B05QHF01kNg/0d];B05aLG01^KVh0CDe=06EUI@11@D40
NWYj06=SH`0M7Ad0IVIV06YZJP0T92@0MgMg04A4A01^KVh0@D5103lo?`0e=CD0HV9R04e=C@1OGel0
A4A401HF5P1SHf<09BDU071`L00[:b/0IFEU03Ti>@0/;2`0J6QX051@D01EEED0T92@04m?C`2FUYH0
RHV903@d=01;Bd/0CTi>06a/K01=CDd0];Bd09ZJVP15ADD0B4Q802PX:00g=cL0:2PX05mOG`0l?3`0
F5QH04i>CP1JFUX0?3`l09JFUP19BDT0PX:206QXJ018B4P0M7Ad03`l?01MGEd0A4A405IFEP1JFUX0
>SXj05=CD`19BDT00P14A4@0@00X:2P0Bd];07=cL`1PH600EeMG09>CT`1:BTX0DU9B03`l?01DE5@0
7alO05eMG@0m?Cd0<c<c0451@@0m?Cd0DU9B04M7A`1PH600A4A403/k>`1fMWH0Bd];03`l?017AdL0
;2`/04a<C011@D40JVYZ03lo?`11@D40KFe]04A4A01JFUX0Bd];04Y:BP21PH40@41004M7A`1LG5`0
HF5Q03`l?021PH40>c/k04E5A@1cLg<0C4a<05=CD`15ADD0CDe=05YJFP0h>3P0JVYZ02l_;`1ADE40
QHF509:BTP1;Bd/0BDU9059BDP1=CDd0HV9R05=CD`1UIFD@ooooo`003_ooool08@31`L40Bd];06e]
K@1fMWH0^[Zj06MWI`0k>c/0ZZZZ0;Ng]`1]KFd0BDU904=3@`1JFUX0bLW908^;R`1>CTh0D51@03Lg
=`1>CTh0SXj>03Ti>@1CDe<0>c/k02l_;`27QhL0?3`l03<c<`1OGel0>3Ph065QH@1@D5007AdM04e=
C@0203hn?P3Z05ADE01cLg<09BDU04E5A@1BDU80G5aL0820P02?Shl0=3@d07emO@2BTY80MGEe08f=
S@0o?cl0;bl_065QH@1OGel0H61P09JFUP0W9bL0:b/[06=SH`11@D40U9BD06QXJ00K6a/0<c<c07in
OP1NGUh0T92@04a<C00O7al0D51@0;6a/@1ZJVX0Ng]k02<S8`0m?Cd0Hf=S06][J`14A4@0DE5A02l_
;`1>CTh0Jf][051@D00a<C40G5aL0451@@1IFET0U9BD02l_;`0^;Rh0G5aL03lo?`1bLW80Zj^[04M7
A`0M7Ad0CDe=03Ti>@22PX80R8R802DU9@0l?3`0?Cdm04M7A`22PX806a/K03<c<`1OGel0Shn?04e=
C@1lO7`0BDU902PX:01SHf<0NGUi04e=C@1mOGd0@d=303dm?@1SHf<0NWYj051@D01dM7@0EEEE02DU
9@1:BTX0JVYZ04=3@`1YJFT0>CTi034a<@18B4P08b<S059BDP1DE5@07alO02l_;`1FEUH0<S8b079b
LP1]KFd07QhN04=3@`1JFUX0DE5A04U9B@0b<S809RHV05EEE@1bLW80De=C07AdM010@400:b/[0820
P02LW9`0<C4a03lo?`11@D40;Rh^071`L02j^[X0F5QH038b<P1:BTX0GemO0:ZZZP2[Zj/0<c<c04=3
@`1QHF40EeMG0;>c/`1@D500:BTY04Q8B01UIFD0Shn?0;2`/00f=SH0<S8b05QHF023Ph<0DE5A07Mg
M`0Y:BT0CDe=08n?S`1^KVh0IfMW03Ti>@1EEED0Kfm_0:FUY@27QhL0OGem0410@00n?Sh0PX:206IV
IP2CTi<0>SXj03Hf=P21PH40@d=3075aL@10@400=SHf06=SH`3:b/X0IfMW06m_K`19BDT0Ph>307al
O01<C4`0>SXj05]KF`1EEED0S8b<04U9B@0_;bl0DE5A051@D02DU9@0/[:b03Lg=`0l?3`0LG5a069R
HP1iNGT0R8R8034a<@12@T80LG5a079bLP26QXH0AdM702/[:`10@400DE5A05]KF`1kNg/0MWIf03<c
<`0g=cL0JFUY06ATI01`L700V9RH0451@@1XJ6P0HV9R05ADE01WIfL0OWin03Xj>P1ADE40PH6107=c
L`1[Jf/0IVIV01/K6`17AdL0EUIF061PH01WIfL0QhN702h^;P0Z:RX0MGEe0P1DE5@01`1RHV80<C4a
03Xj>P1PH600@41003`l?01YJFT03_ooool0Q`1QHF40C4a<02DU9@1aLG40HV9R05=CD`0d=3@0ADE5
05=CD`0b<S80B4Q805]KF`1OGel0MGEe051@D014A4@0H61P059BDP11@D407AdM04I6AP0[:b/0<S8b
05MGE`0g=cL0ADE505YJFP19BDT0Bd];04M7A`10@400GUiN04E5A@1:BTX0EUIF071`L01]KFd0:b/[
04e=C@0n?Sh0?3`l04e=C@1FEUH0Hf=S03Xj>P1=CDd0GUiN03lo?`0c<c<0CTi>05=CD`0V9RH09BDU
03Lg=`14A4@0CTi>02/[:`0m?Cd0>CTi04a<C014A4@0=SHf05EEE@1GEeL0LW9b05aLG015ADD0Bd];
05EEE@1;Bd/0LW9b07moO`1KFe/0:2PX02TY:@1@D500>SXj04A4A019BDT0AdM704m?C`0d=3@0@D51
03Lg=`0d=3@0<S8b02`/;00j>SX0;bl_03Ph>00n?Sh0=3@d03lo?`0S8b<0;bl_03`l?011@D40ATI6
04Q8B013@d<0>c/k04I6AP10@400Hf=S055AD@18B4P0FEUI04a<C01BDU80@d=305MGE`0R8R80FEUI
03Ti>@1<C4`0ADE504=3@`0e=CD0=cLg03dm?@0f=SH0MGEe05MGE`27QhL0J6QX034a<@20P800LW9b
051@D018B4P0IFEU065QH@0S8b<0HV9R030`<0020492@P2=03Ti>@1IFET0?clo03Ti>@1>CTh0Ti>C
0:f][@1bLW80Kfm_05IFEP1jNWX0AdM706m_K`1]KFd0?3`l05]KF`1?Cdl09RHV05]KF`0i>CT0JFUY
02l_;`1GEeL0=3@d04=3@`0f=SH0@41003hn?P0K6a/0Q8B402DU9@1^KVh0>CTi061PH00j>SX0;bl_
06ATI011@D40>CTi06i^KP0i>CT0IfMW075aL@0O7al0>SXj03De=@1NGUh0@D51075aL@1[Jf/0>CTi
03hn?P0d=3@0?clo018B4P1HF5P0<30`06a/K00j>SX0EeMG01PH602RXZ80=CDe0820P01aLG40=cLg
05iNGP0m?Cd0DU9B04e=C@1?Cdl0Fe]K05EEE@1HF5P0FUYJ05iNGP1ADE40?clo04A4A02JVYX0IVIV
03lo?`1bLW80@d=307=cL`1=CDd0F5QH04i>CP2NWYh0I6AT04Y:BP1;Bd/0<S8b04I6AP0c<c<0DU9B
03dm?@0g=cL0EeMG03Ti>@0[:b/0>CTi04U9B@2;Rh/0EEEE09jNWP0h>3P0;Rh^051@D01<C4`0IfMW
05YJFP1VIVH0MgMg04=3@`15ADD0GUiN07MgM`1HF5P0I6AT05iNGP12@T80HF5Q03Xj>P1YJFT0<C4a
03<c<`0l?3`0;Bd]05ADE01:BTX0ATI606MWI`1hN7P0EEEE05]KF`1hN7P0Fe]K05UIF@1?Cdl0GUiN
013ooooo000>ooooo`3o08V9R@1fMWH0Bd];05iNGP26QXH0QhN703Xj>P1@D500F5QH07QhN01PH600
=CDe0492@P1>CTh0IVIV08n?S`1@D500=3@d03dm?@1UIFD0EeMG08J6QP10@400<S8b05MGE`1@D500
GUiN05=CD`0d=3@0Bd];07AdM014A4@0W9bL03hn?P0c<c<0@T92061PH00e=CD0HV9R03Lg=`0K6a/0
@41005EEE@12@T80KVi^01HF5P0Z:RX0JVYZ08^;R`11@D40BDU901XJ6P19BDT0DU9B04a<C00j>SX0
>3Ph03De=@1<C4`0IFEU03Ph>00];Bd0=SHf04e=C@28R8P0EEEE01DE5@0`<300?Cdm06MWI`2QXJ40
>SXj03Ti>@1FEUH0CTi>05EEE@26QXH07QhN02d];@1iNGT0KFe]055AD@28R8P07alO03Hf=P1/K6`0
IFEU075aL@3<c<`0<c<c051@D02<S8`0VYZJ065QH@2h^;P0@410051@D01]KFd0Cdm?05EEE@1HF5P0
:BTY059BDP1RHV80>SXj051@D00Y:BT0<S8b06][J`1PH600=cLg05=CD`0c<c<0<C4a08N7Q`1TI6@0
>SXj06IVIP0i>CT0=CDe09JFUP2h^;P0>CTi059BDP0`<300@4100:NWY`2LW9`08b<S04U9B@0];Bd0
HF5Q0;^k^`1FEUH08b<S06IVIP0n?Sh0U9BD09JFUP0Y:BT0:RXZ07AdM01hN7P0RHV90410@00S8b<0
E5AD07UiN@1lO7`0^[Zj05IFEP0b<S80ADE506][J`1PH600IVIV02l_;`0X:2P0G5aL09JFUP1OGel0
EeMG0410@00Y:BT0IVIV0:JVYP1nOWh0CTi>04Q8B013@d<0EUIF075aL@1QHF40Rh^;03dm?@1<C4`0
KFe]03Ti>@1BDU80HF5Q02LW9`0i>CT0N7Qh03lo?`1YJFT0=3@d04M7A`1kNg/0Fe]K0451@@1:BTX0
?Cdm05UIF@2RXZ80HV9R02/[:`0g=cL0FEUI0;Jf]P2GUiL0>CTi05QHF01OGel0b/[:06ATI00W9bL0
?clo06QXJ02MWId0SHf=03@d=01IFET0KVi^04e=C@1cLg<0QXJ60451@@1JFUX0Ph>3069RHP2XZ:P0
BDU9061PH021PH40O7al08F5Q@2EUID0@41006][J`28R8P0HV9R08Z:RP1PH600>3Ph06QXJ01]KFd0
Bd];0861P@1IFET0>CTi03dm?@2d];@0EeMG05]KF`17AdL0:b/[03Lg=`1YJFT0FEUI04i>CP1MGEd0
=CDe05MGE`2GUiL0CTi>000G055AD@0[:b/0BTY:09FEU@22PX80BTY:03/k>`19BDT092@T04E5A@2a
/K40Kfm_020P8016ATH0=3@d06][J`2PX:00<30`02PX:01PH600KVi^06=SH`2PX:003_ooool0201^
KVh0H61P02l_;`0l?3`0HF5Q06a/K01IFET0JVYZ0P1]KFd0AP21PH40T92@07alO01mOGd0Ng]k06UY
J@1XJ6P0MGEe06IVIP12@T80@410034a<@0c<c<0D51@04Y:BP11@D40SHf=0861P@1jNWX0DE5A065Q
H@21PH40I6AT05ADE01aLG40O7al05IFEP0Q8B40>SXj04];B`0f=SH0B4Q803dm?@16ATH0<C4a03Ti
>@13@d<0?Cdm02d];@0j>SX0ATI602@T900K6a/0ADE507alO01nOWh0;2`/03Xj>P0f=SH0C4a<0451
@@0c<c<0A4A404i>CP1MGEd0EEEE03Ti>@1<C4`0ADE504a<C00Y:BT0<c<c07QhN00V9RH0:BTY05eM
G@1CDe<0DU9B03hn?P10@40203lo?`2D05aLG016ATH0@T9202@T901@D500>3Ph02XZ:P11@D40<c<c
04Q8B00b<S80@41004];B`16ATH0BDU904M7A`0a<C40?Shn05MGE`0U9BD0;Rh^065QH@11@D40?3`l
05QHF00[:b/0>CTi04M7A`18B4P0D51@096AT@15ADD0?Cdm04I6AP1EEED0?Cdm04A4A017AdL0?Shn
079bLP1IFET0QXJ606i^KP0o?cl0SXj>07]kN`1NGUh0G5aL09FEU@1`L700=3@d08B4Q00a<C40<c<c
07=cL`1ADE40TY:B03Ph>00b<S80>3Ph06m_K`2i^KT0FUYJ07EeM@1FEUH0GEeM05QHF01oOgl0EEEE
02/[:`1TI6@0Bd];01`L701eMGD0C4a<05UIF@0Y:BT0A4A403De=@16ATH0?clo05MGE`1=CDd0<S8b
09jNWP0m?Cd0N7Qh03@d=01DE5@0DU9B04a<C01LG5`0ATI605UIF@2BTY80?clo07AdM01cLg<0<C4a
06IVIP1>CTh0GemO04a<C028R8P0VIVI05EEE@16ATH07AdM02d];@0E5AD0GUiN030`<01YJFT0@d=3
04e=C@0E5AD0Lg=c03Hf=P1fMWH0EeMG03dm?@1SHf<05aLG04I6AP0e=CD0GEeM0492@P0a<C40FUYJ
04A4A00k>c/0E5AD02l_;`0o?cl0CDe=04Q8B010@400JFUY02LW9`16ATH0?Cdm04i>CP0W9bL0BTY:
03@d=00b<S80@T9203Hf=P80?clo01h0Jf][034a<@0o?cl0NWYj05ADE01:BTX0=SHf05IFEP1KFe/0
ATI605eMG@0a<C40<c<c059BDP0K6a/0EeMG07UiN@1KFe/0LG5a05eMG@1IFET0EUIF05iNGP0n?Sh0
Y:BT06i^KP1MGEd0Ogmo04Y:BP1MGEd203dm?@0>04];B`0J6QX0FUYJ01hN7P14A4@0P82008j>SP19
BDT0F5QH08V9R@18B4P0D51@04=3@`1ADE4@ooooo`003_ooool0>P2HV9P0Q8B407MgM`1HF5P0MWIf
09FEU@1OGel0X:2P04A4A01=CDd0XJ6Q05iNGP23Ph<0NWYj01/K6`1<C4`0Kfm_05QHF02KVi/09BDU
03Hf=P1PH600F5QH055AD@1eMGD0;Rh^04A4A01GEeL0<C4a04U9B@0P8200CTi>06EUI@1BDU807alO
03Ph>017AdL0E5AD07=cL`2;Rh/08b<S038b<P1ADE40De=C0;6a/@17AdL09RHV04i>CP1KFe/0J6QX
09>CT`0C4a<0?clo069RHP1EEED0Fe]K08n?S`0F5QH204M7A`0407AdM01lO7`0G5aL02h^;P80GEeM
0=@0De=C07EeM@0m?Cd0<c<c08V9R@1dM7@0QHF50:JVYP0H61P09RHV061PH01WIfL0Q8B40::RXP0d
=3@0?3`l06EUI@0l?3`0GemO06m_K`0P8200?Shn06ATI00`<300CDe=03Hf=P0`<300A4A407IfMP13
@d<0EEEE03Hf=P0m?Cd0Ti>C09BDU00U9BD0ADE503De=@1ADE40V9RH03Hf=P0P8200FEUI071`L01P
H600WIfM02TY:@0M7Ad0C4a<0820P01UIFD0ZZZZ02XZ:P0O7al0FEUI0;6a/@1_Kfl0YjNW024Q8@0k
>c/0DE5A09ZJVP1GEeL0EUIF02LW9`1HF5P0JVYZ06IVIP1@D500De=C03Lg=`1^KVh0Lg=c04E5A@1E
EED0QhN703De=@1UIFD0JFUY03De=@1IFET0NGUi03/k>`1gMgL0Xj>S04a<C01/K6`0F5QH02l_;`2H
V9P0Lg=c02HV9P10@400GemO0410@01XJ6P0Kfm_03Lg=`0j>SX0CTi>06ATI01cLg<0I6AT03Ph>01J
FUX0DU9B05QHF02n_[h0UIFE03hn?P2GUiL0IFEU06][J`1lO7`08b<S03lo?`1<C4`0BDU90;6a/@1H
F5P09RHV05EEE@1RHV80GUiN06MWI`0M7Ad0;bl_06ATI02;Rh/0HV9R03Ph>00_;bl0@T9206UYJ@1;
Bd/0EUIF01TI6@18B4P0JFUY04E5A@26QXH0DE5A03`l?02=SHd0BTY:04e=C@1:BTX0:b/[03Hf=P1d
M7@0?3`l05=CD`0W9bL0De=C08B4Q01IFET0Lg=c05=CD`16ATH0H61P0;Jf]P1lO7`0@D5105mOG`0i
>CT0Ogmo0:RXZ00l?3`0ATI606QXJ01DE5@0VIVI05EEE@0];Bd0ATI606m_K`1hN7P0TI6A0861P@0P
8200EEEE06m_K`1XJ6P0QhN706e]K@0W9bL0I6AT08^;R`1[Jf/0W9bL03De=@0];Bd0CDe=08b<S01T
I6@0BTY:03Ti>@0W9bL0;Bd]05UIF@2b/[80FUYJ05mOG`0Q8B409RHV065QH@1]KFd0=SHf04m?C`0L
71`0;Rh^03Xj>Pkooooo00@0J6QX09NGU`0k>c/0@D510P1BDU802010@400GUiN06IVIP0n?Sh0CDe=
07moO`27QhL0IfMW0P1OGel0h01VIVH0EUIF059BDP0i>CT0@D5103lo?`1MGEd0BTY:04];B`1PH600
MWIf06MWI`1dM7@0Fe]K069RHP20P800IVIV04i>CP1RHV80KVi^06ATI00f=SH0:2PX03/k>`18B4P0
A4A403De=@1BDU80@D5104I6AP1FEUH0C4a<04];B`0o?cl0H61P02LW9`0G5aL0B4Q804M7A`1<C4`0
:b/[034a<@0b<S80>CTi051@D00`<300E5AD06QXJ01LG5`0IFEU03Ph>01:BTX0Jf][04i>CP0n?Sh0
CTi>06QXJ016ATH0CDe=06=SH`1HF5P0D51@059BDP1@D500E5AD02h^;P0o?cl0>SXj03hn?P0Q8B40
:RXZ03<c<`0Q8B40=3@d03De=@0c<c<0:2PX02h^;P0m?Cd0?clo03/k>`1FEUH0>SXj04E5A@1<C4`0
<C4a03Lg=`1eMGD0?clo04Q8B01UIFD0Cdm?06m_K`0m?Cd0>SXj02LW9`10@40092@T03lo?`1MGEd0
EEEE034a<@1@D500JVYZ06IVIP1dM7@0Kfm_0:f][@2NWYh0AdM705IFEP1gMgL0TY:B08>3P`2BTY80
GemO03dm?@1FEUH0Cdm?02h^;P1:BTX09RHV04I6AP0_;bl0?3`l04Q8B02?Shl0P82005eMG@1?Cdl0
@T9206a/K00^;Rh0?clo04U9B@0h>3P0ADE503<c<`0C4a<0Ti>C0492@P1YJFT0<30`04M7A`19BDT0
B4Q80451@@1QHF40?clo020P801iNGT0H61P09ZJVP0S8b<0CTi>05QHF00];Bd0H61P0492@P1MGEd0
LG5a05mOG`1TI6@0L71`03/k>`0h>3P0CDe=05UIF@1UIFD0VIVI0:JVYP1GEeL0AdM703Ph>01KFe/0
>CTi05IFEP0o?cl0Fe]K04A4A01<C4`09bLW09JFUP15ADD0QhN706ATI00];Bd0E5AD01<C4`13@d<0
:RXZ04E5A@17AdL0=cLg04I6AP1BDU80?clo04E5A@0n?Sh0B4Q8075aL@1CDe<0=cLg06EUI@0E5AD0
BTY:03Xj>P1@D5009BDU06i^KP11@D40=SHf04];B`0e=CD0ATI604=3@`1aLG40ATI604M7A`1fMWH0
ADE504A4A011@D408R8R0P16ATH0901`L700<S8b03dm?@24Q8@0<30`04A4A018B4P0E5AD06a/K01Q
HF40NGUi07MgM`1KFe/0D51@07AdM02BTY80C4a<04Y:BP15ADD0Cdm?04I6AP1=CDd0EUIF05QHF01M
GEd0@T9204m?C`1hN7P0K6a/03<c<`0o?cl0G5aL0410@01?Cdl0Bd];05ADE13ooooo003ooooooooo
ooooF_ooool00?oooooooooooomJooooo`00oooooooooooooe[ooooo003oooooooooooooF_ooool0
0?oooooooooooomJooooo`00oooooooooooooe[ooooo003oooooooooooooF_ooool00001\
\>"],
  ImageRangeCache->{{{0, 599}, {299, 0}} -> {-0.0517664, -0.0250054, \
0.00351174, 0.00351174}, {{14.6875, 292.5}, {291.875, 152.938}} -> {-16.9197, \
-8.22585, 1.15187, 1.15187}, {{306.438, 584.25}, {291.875, 152.938}} -> \
{-352.977, -8.22585, 1.15187, 1.15187}, {{14.6875, 292.5}, {146, 7.0625}} -> \
{-16.9197, -176.255, 1.15187, 1.15187}, {{306.438, 584.25}, {146, 7.0625}} -> \
{-352.977, -176.255, 1.15187, 1.15187}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Import externally computed test labels", "Subsection"],

Cell[BoxData[
    \(\(testData = \((Map[ReadList[#, Number] &, testFiles, \ {2}]\  // \ 
              Transpose)\) + 1;\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Visualize individual histogram classifications of test images\
\>", \
"Subsection"],

Cell["\<\
The graphics below illustrate the classification of each test image \
by band (HL1, LH1, HH1, HL2, LH2, HH2) down the columns.\
\>", "Text"],

Cell[BoxData[
    \(\(probs = \(AnalyzeOneTestVector[#, \ hist, \ rowLength] &\)\  /@ \ 
          testData;\)\)], "Input"],

Cell[BoxData[
    \(\(results = 
        Map[VisualizeResults[#, \ colors, \ 
              DisplayFunction \[Rule] Identity] &, \ 
          probs, \ {2}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(gimages = \(ShowImage[#, 1, 255, 0, 
              DisplayFunction \[Rule] Identity] &\)\  /@ \ 
          testImages;\)\), "\n", 
    \(\(Show[GraphicsArray[gimages], ImageSize \[Rule] 640, 
        DisplayFunction \[Rule] $DisplayFunction];\)\), \
"\[IndentingNewLine]", 
    \(\(gresults = Map[Last, results, {2}];\)\), "\[IndentingNewLine]", 
    \(\(Show[GraphicsArray[Transpose[gresults]], ImageSize \[Rule] 640, 
        DisplayFunction \[Rule] $DisplayFunction, 
        GraphicsSpacing \[Rule] 0];\)\)}], "Input"],

Cell[CellGroupData[{

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .11628 
%%ImageSize: 640 74.4186 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.238095 0.00276855 0.238095 [
[ 0 0 0 0 ]
[ 1 .11628 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .11628 L
0 .11628 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.00276855 0.245293 0.113511 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 0.003125 0 0.003125 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 .5 scale
320 string
320 160 8 [320 0 0 160 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage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P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.267442 0.00276855 0.488926 0.113511 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 0.003125 0 0.003125 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 .5 scale
320 string
320 160 8 [320 0 0 160 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage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P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.511074 0.00276855 0.732558 0.113511 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 0.003125 0 0.003125 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 .5 scale
320 string
320 160 8 [320 0 0 160 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
B793CD6F504F4A31642F664A84521F564C28401F175859354E22154250686D6A192C553D
371D503856562725693F3C371A3B59582538755E2247404A403B4F1F5B7A53333E440B2F
712948272C782F3C3F553A452C4E2B7773885B454E523B6F3737286D4C5E997F673D703D
675B674E2B4A60314E8457465C395D493856554350534F43804989515A6D3C673653867C
4CAC6325486E655A836F3C776324698457446F5747A1764B7A2F3686826762704540668C
695F264F78383E9E616AC19A5A94865555635F377B65678E5B465F414FA2676995415FCD
683C5D3B2D7A796F441F8F5B3A5597A176374D6A42706F3B23986959B48C4F2B54683C38
3C6A497C9B6D42456F3B66524267685A3E9BA64C3B33715B597EA49A5A4776794B8F985C
572E8F51387B5D273F6444666A762A5BAC4D2E55649B854F796263376B593C96
B187B5643F504B236238766455682D5B3D232916195F563E4A31234A5B5C60963F284138
35454636446141246D40354A4B5459682D3563662C5F635B404A5F334C846B37487D2336
812F41492D883E3844694C49405F2D708D865B4B494C3F794B3C2C5D3E43A0AD6F4D6B30
795E51651C3E6A3E5084563776477772355D583C677B774277388888534232745F465D44
45BB8641497C2B336877409472476278603B62753C6465617A523F846E6567575D755598
79724086A0733BA34B3CBAA25B8D98384261694A71303E69624F483935725C89B6253CB9
7F3D724721417A644D309A6152385C7559377A9E5A6A7D402C7D70478D67422456707C59
49703C81BA9257437B275D67525E66563E6A7F58452C6A6A5D5B668963446F5A32847E74
552D78843A605B2836703D6B66853B66A171352D2F807856A99E8639715A309A
A27B8D74455E6E2C813D7D68346B35593B1B291E205E4F42473B2B495866568D412E3227
2A503E2D3981522160473E525460565125346D5D23725832384D5633438E322D4F70282D
5C274458319332395C9055434F382150716F4E59542A2D51453C33616534758F66528B41
5940417F36455D204A8486365D2963914364412544547B4366315F845164384433404D54
35897E474981714E66752A898C53708248588AA1487C323A677D4E8B5C43708286685CB3
85652D5D737B40B2955AC9A63B7D955F4A5A7F4C8A76506664525E512D71507B9B4D46BF
7F4189812D338B5B4F2B7F62613E716B794A488A6263714D51888C4B9974352F737D9054
447053589C855E3C8749566062595957456383634E2A6E5D6C755980754068894F6B9887
4B2D6C85416871333C743D65646F2E79A4873F2D37842E3C6D8082376C2F1C7E
BD8A6E7C4963762D7C416F63205F3D553514282A275F4D30505432475056578E3E383324
3350422029826820541C2C47405558512727583A1969846334475B4147916E3B3F773329
5F2F51592D89212842734256583D25738E7439667E593A563A291D3B724C91A256408D4F
614035733545512B315982386036557D4551553D484D6E4A82565C785093454B313C6B80
529C6F30578989527A7B3E8EA23E83A461375E99496D705B668662A99F8C73619A5957BF
AD9C3B6E5C462F7E8C61C1916A748E634A544C54A17F5A5E4D3F51552B683972A86D459C
8432637639278C5A503A7C554E3A675C774B4785666A6A452A6A9152886E40234C5E7D57
40686163916C5A3195545A55444D5D4A487872413C26764C6D81588780396987427C8E7A
3744848F4E616B21307547657B642B60984D31384A8977396A7D562A7959216B
934559674770391F63404A1711553849380926322647481D3141284E281F317743374730
283E453520715819606039403843687341275A3D19618C80454D451E4086653A415C2C36
834B5D4C28745F405563273A6360347B8D662350614D355243311E326035899C60478E58
5E4F3B703154624D37527F4C794F4D6630626D445257502B68644B783F763E673D314D6E
4E9A8B4741745D3D61735278B15B796D7E406C8745848F71707450918E8E6D6D71453FAF
88904A7F5E8D3A7B7551C0866A75A1634B4C8539A18A525C82563D5E2869526DAD4D3F7E
8E3360583B1F8F543540604345407068684E4279726C7A5636678F61786746294B52825F
42817A599C776436924459492B4D7753477269532937907E89765D873636808F537F974A
1B325B72386E7F3B4B99385758714884A1753B222F757350868D8F5757472873
A27469723C4D5A2E4826544F1646394A5C1B393120315A4D453A1743553F4D9A33336762
2A13305522804915545E353E2D29607B4E29475A2849817D5057545F346B6B37554E211D
62434E47236D4D386568774055713893AB885B585E3E3F647253262D5B2376A0784E9228
5965396A233A56553F5F67306B5252823D435F2A49657B496943467D4B5A3062513F564F
3485A56D4864775F6869554FBF7D867A9539648A5F809F7573918796948A6E889C8A49CC
6D7F3D7C659449748A46A1796758A6664F588C2F848F4A6D7D534166227D7766A6644775
9A3D6168342E8A9C6F5468A847213F3B6F494F77345E7469375C7E4C839A5C37484F803A
51778D44B689212B7755616C2F2D40563465867C3D205964485A8D9B4C254A733B819B76
3E3D548750546F2A3B8A4C66566B4A999E7D46282D652C5E80708C5E74441966
7A5CAF9E4F50703C61483F5F1C321E30551C3B270F1A4B413E34152D4D405FA641224163
30392F4022838728515F4357663B496D5D3055804D4B8A3F324D586B326A592A6C7A401F
682D3256396E3322607879407236195F9C804660834327383E432539753469897044A35C
444D2C735F44513D334B8E4D6B573D7B65406A47435282616C714588546A5455533C5580
5684A374385E9568706C5E47CE90737A9F4165AA757A886774896C86A48174B490865DC8
7A4F4582908E4D707429828A6047B0495C7E882B67463873875040542B70895C9E5E1E5E
8B4869712C1A73763033595C473F5646915E40726F5960464C7866264260633D5F66914E
375F6838958A5B37725045503F41584F2C635B5247296D767150868A6E418EA971819568
2A51809D6A70993122695A4F9C7D316B756448282C675C34603D7B66686A3C5D
779FD0B85A604E2B6E6C565E224E50304F093846231B572A3D5D3135432B5495662E453B
2A60412F105B702857312746693C5955151C445F3837645341464A47287D73295C5A5320
624B30654357733F4B786D417C62337DB583353B565F364143371921634A79A272379E68
385F41777D41497B444E986B5F79457F6B456A54494B7165607643886166584867455C85
6B72A07F3764A8696B6F4E43AF715B87874F8FC77C657E4D7B7D626E903A42727E7B48AE
8A74465C63794A70934197894A5DA84B323E643A70864C5B84475862235C6B4CA7723646
772F5470561D7A883B515C7252556B4B9C6F45677C6E656D3760906466665A2B3D4E9175
375A894498957D45707646544E466B73555A5F62303E85978D4F9D9B562B4A945683BC93
4C31417E404A7E522E674132997D4A8C9A513A47516F5B6A8D3E7A5D3F6C4556
5872D3B9585758294E4D413214414129581F31400E136D3E263819385A2A4A846628546F
34472D502574621C475833393D296C795F233755442C7872574F4368226B833663452C1E
59653A674060854E507A4B2C8166396CA885654B5959433F45482B2C6B5D6F92A3469968
34604B748B43488C46518F6F6887467A605675605048614367873C7C2C665A3A55294E7A
5D5CAD5D38637F625A873F388F5D51977936537F745A9056636F5963B88F67B5A37146A1
AB7F4E9076754459864CA2A07C63B56B45416035619055618C4B3B662F4F5D318682413D
7D5C5484711F61B05E546D7548557F6692513E6A6274986A326FA97C7A756234405C9B83
3D6C98408D9D663E8F7846614D2344556B598F804D2F5D764A3D8496893750AC706AA088
76433F937444876A38566343B1A35E8A95655425305548509B4983803D59364B
5442A1BC814D6B52433B366E28414E2153282F4F19154C34202F1D264D32648C4B183969
303B33432665733F436E362D5237546F84362C504524745957514680325F9D444951691B
4972386335378F4158766F476A593965B3846C47483C3D507062342D6A465493A3537D49
33664A7D4F34587A44556A31626F437540315F57444C634848773B79535858375233496C
5E47AB804153804D4B8C634E986845767E596A8563528B6A75755248AA9163B88F6862A2
9D78558C7F9B625D9E5B96BD6753C36A584775445E96565D9F664460373D7D5185844B36
83634980683071C96858817770283B3E71434880523D5E83326194446AA67C613D47834C
335887438AAD5F29534F3A665F4748564A3D7067613A62765E578C8E8F3D46B48E709D93
85483C7B6A498A7246496E469B67616C9269552B335C623C967A838337513451
534454C7AF586B5F3D8740813444722A4E362661261550411D2C2C2E3C3469735A203364
373C3F412A5A7E4339783C3161495D4C7B3026552B1959464B56507C364E8C424090831C
4079456C413596295D7D7B556B60283E706F5441483A1D3D455235336135456587447B63
3147347665313C5C3F65856764693F7D83497741374683774C793A5A64544C3C6C425D5C
45398C7E455A9A6D4D87494C9A7F5783683377919D5A965F7D90754CB16F51A8A99060B2
9A5F2E70869A5850852950907B47CC74344862425C7A397F8E60544423539E627DA1394E
85674CA2251940A659417F75805653589075416D685A765441869B5A41555D2A395A8476
4246765B72A3833A64834457553F45535D44735D3F3B536D626491828F5A39BC8C68997E
74465D8884547A7641677E4D8764515CA98E6332366B41235C8A94642E4E1B3C
5B4548C0B95D656935853E773C577B314C4E21602F15433A1D302929403C61787D33365C
45514243274EAB57316D38566E446058512E476444186D3122313F623744851F1F56671C
38451F494F3077173A59593F4E58232D877C4E3E586E3B42473C1B3B636E608CB4414F6E
3F54295182474442274B75806C763F5E8E4569583F45798E5488496D727270466D48597D
7440976E336B9879487364377B835781735B456395578F5A4E616C50BF6B4C85928659A0
976A4B6864803C619C526BA57C34AE6A392E515566914E4464435C602C31783B92943620
6733365A512A37C432337B3D4254636C7E643E5C585C916E294F8E7E4D6A581D21426588
4B4671707DB47A2D776F375E4332587654456A5A3E416EA367678E8C812B42A47F4D8B70
2B224064654385371C48693880A0403D7A5F512E3359292D3E746C713983473E
554E3EA1A9635848336A3040152F56354C47195C40173C3319312A2B4219335B713B3032
1242373C0D286244225D252C463E585D39231D45541D54685E3A3B552D449E541E49601F
34854154652898614B64795F4B6D473799872F2D3C3C43475249303347524781AF6E5162
2F66534C8D50547E4F55747D7B9E595E75487F6843465C3C498A49604F4B69474D37395F
7241A87D3546A98447797049999252827C70425570508D8A5A9C5D3E8598788C904D3D85
A68051677082513F8A5B81A3643D986D38324B4B5A8D6849774051593E32A06D7E893124
5F493059433744C75A458D466330507D82743A5553377F7C39717F2F35828856224B615F
315F836D72BC601886753C5A5B1F2D43573B5E6246293B5064587AB2753022886D57868A
6F3C3F71552C916E3D54691C558E684B7F713A3A52664E4044949B512E703D39
51433D806B474F4E2B6E2E59432D532C484613352F133436192729245633435F7A3D3551
4534274A3D37973219723F473F294563784628503B14365B5545454738327B662349671C
206A374F58298B70545B772D3799612F8EA37742453B2C4F6F7755343F4C3A7296844A6B
2163634054283F6645536E3D4D79555B71326456455B74724B8A4F628761674771503A64
5C3CAEA54A458F65468F744467964884717835508A688B8E4F827C4D8EA4838586556077
9F6F53636EA16E408F826DB18642878F4D61737147716A5AA0735A57414CB88B7BAD4148
867528602C1E219169438A476E433F526C6B3A4D705763743E758F6339667B492643537C
4A357C7553C47B415B743B5E5B5E4B5059305D967C4F3B68462B449A8353268B6F2D6E8F
74363E634E2B785D473D444463826E457E905E292A553E5735A1A08031553A39
54664D8D7C3D587237491E4E573C5D1F2B532A453407264122332A173C394A7689212639
533729322A35B7691D732E5571523244704E4A676024323B3842575755337C461B42692A
1E6D3154832F88665F6890723C972C1F708E533D715D32263E4649373F5B43608561487F
455340435F4B48755857769654804B5A8C4C7C4638435885487F464988564B34674E3D64
5E398570493B7F57315F6F434DB768898483375EAC70918A508F6B4B8D89677D9A77757F
A76B43558B936E54AE5234758D3C82982A41585F45643A3372724F6E343289816AB34034
967B2B7C563C209C6634927C7A7355717E6F335C88688276437696885085833723415D93
67357247389BA14C72731B455B5D434F56284E63574F3C6459673D99864D257A92604E8F
8C3D3F5B723E83755B4D4B6470836E49929C73333159575F3DBCA87B365D5240
5191607C7831496338683D594735764036542039551E274F0D263E1E411420527F462C35
1D373E2F1018665D1E67242C435334514219263E5C272D39141B333F4B2B8C6D26325E2F
21702E3166307E493633675D3F9C5C2EA2BE573050503D345545272C3C73547597743983
5B51554F4943484E3546678B5278704E8F616A49464C5573518F6F5C895C91555953448F
8D4E90754855A68C43787A2A56815D879F4D506C93697F793075804B8C9C3B48746F6D81
BC7E503B5F9027398AA84DACA63A5F965D44454056987E42826548876D40A86C5BBC6643
816A27686F441E968B1B48527D6D4F7676792D387B658C58307FA28758898255173F6495
683D527838899D5A6B7B3F52554B465656506165615F32637A7A45987F652D79A9745BA3
85455980753B7D6D354481737CA8691D6B7D79373B60652D2B70695B31673923
527B667C8E434A2E1C433A3D1E144135374B0F1934151F40131E2B1B4140395084553140
3B313F3D402077521C6E3D4F40292E616544233E5B2D284D49304049231D6A7B303B461C
1C7047325F2D8188534A763228A4623397B2674C5D4B313A5E472B3132724B6A9982447C
595969575448555A5541697349877552722D595A414C583F2E6162406A3B6B4B513D2348
654190862E255C57317792404FA93742768A3A4C73377AA45584733D76A09D5B7D7E3E5D
B2A47F497D96723A8DB85081C861558F5E4638673C5C7F3C7561466B6B399244429B6040
9C5627534B3F1D907A276569673E4C9D88763F3B81538E7A525E86654496A851272C4A7E
6D4667914E68AB59808D345957413D7C64545A6B74564F89897943918F2D297E986864A5
6D2E3F48582D8C9E5E406E2B669F7A5489825A2B547E6E5A2E797F561B70553A
546138438148536D363D26436D33591B2053272B4A0D0E422521311E414C3D57822F273F
4B3025273D1F825A296340434B62314A6B53243E47141C584D2F364B60296E8B3B47503E
1F72532B6A3F5D81554C615C459C763886BF6B515940333D8E8240333273316A929B4D7B
234D76505E1C2F4E523E5D5C3B5C6E54815B4D5438587B8151773E3D7C5B6D37504C4158
5F1F6D96652E5F6D485C7D506BC07E54679C43517260528B66868871629C965482726458
BA9E8D447C8C874078C0557ACC5C4C8B734B42854C4D853B699A546A653E917954B47E4C
A49037525F3D27909B4D7B6A703A2A564872433C7345515F5D5699714A5B55773F2F4969
32334B7D5989A43D506B4066704532495355509D8648294A5B4235B2AC5C2C5A82357097
9357404450194A64423F5845659F704DB8A25A29283A4454417F7355445C3C2F
50986E4D8C3743624342604B9036505F345F4A314C2A1243291F322F2E65436585342131
4B2A492C351E7F69345B4D3C4C78365576672B4B6D311750322B3A6E742D6564234C6450
1D683D2D7352516B45647C6B4C8F7917337F692C5F6B3D1F45454B2F4B8D35497E88327B
5D4A5330775F4353443C758E61723A458B6D4B452C3B4D563C6A57387E5D56475A704A57
76657C885F3661786A699F6B67C59A585FA948538872468F6E82968071A1A5607F68705B
CA8E6644ABB4783E6DA14A7AC35D4496514C6B774C41704D999F54596247AB864CA26F2D
838F364A4F301A80913396A18E704367558829548558695F6C5AAE9258706D89484D7599
8E3152643C92A57C637D234A736F51645E33336676692D5D7247276E814C284A9F6C4E6C
745B546A7B3B5270504D7E695AAA5C194D7C823A3B466B2825598F7246584131
5193654E964F3A54403F7345672D5376357147385D4E154729224B373156417FA556323D
462F5D2A3219677B376333335F76376E7B2322446941185F292221425C286C803B2F3D2B
1E6D562547314A82302F514D3B809E5057BF862D3B413E3C664D2B222D745969ACA94172
684F52476862434840304F7D5F63704488774058604C515B465F7E43916B6B5C568D5B71
8576798C74377A935F568E4659B0885666A25D8CBC7D3C8561AEB38165967B5CC183705B
C9A04A40789F863D6DA6656FCA673EA17C3D3F5B513B623B5F8B4E5C76438F8051B86748
73A33F5E775B1982B71D4955626E558499974D25656A7B652F428E997C76958D202A4172
7A426B874B58A45E74A6424E523F2E7A7B6B4268642C3770836A316689292D7299714F7E
6629343E5F4D477531256E5251BB8557829D80374D637156384E5A414049391B
516534305F48394619234A3A2D13264326751D0E39321546100D24212D330F34695A2A4E
3F1D46283A13453E194C4827312C2241654B21305B1B195961552E4B4B21518C5537434B
194F642C4E1E38833E3F594F235E9E5F40C78B555945423F6C6436293585585FA6BB506D
6C4A695A72744357783D5C85606F8E4C8171559179524E5E44759143824284624670345E
7865628D3B2E5E7766449142449C784B619245516E6C36745B627E69699B70537E91684F
CDC89D45A1B6713688BB7F7BD1702E94A6504D4E2F639761748B3481884D7B652D6E7C49
6DA9272D593F3E83BB4F676C7947444EA2A95E286F3C666C6E337E87273967765A2D4785
3B243F825861AF493D5B625159513A444B70456A6C5E234066773B6881543E3683685795
A26D45375C3344A6563F904F268E97615C9E7D2F262F575E444E5F562543512D
4F7F5D49744D375D463132285538232A124429193A170B2621172111162C29658B461929
512E3A1940265854494A5930336936545B7338395E621E4262593143774C4D9364304A66
1A5776284A3A2F9B3B374F5E60419F7540C99B6854435A43648F622E3779514C9AB85E5C
6E4479516543324B703B5F69384D744B805D3B84604F4E614C5E8A41895D755545954D54
685A60B56A36526C4143A16448A76F4473966E5A89663899837B79314E929D5FBCA03331
8CA65D4A95BD943057893355BC9C235B6F334163372558468AB1373E633887A060897C2C
5F935C375E4C174FAD4E6C80665E51434397683462705E6283388FAA934C8A87512F4C83
7B4947904E79BF8C5F574B4B806F645A5D613D729F76274266532B79A87C4539831E4373
8D7B4839565039725C3A9085569B745D2D847F3B2634767B634D545060454533
5E5F69527B59304860236D3E814B2346254E331C371F1331291A241F2542356385441C2F
4D2A52292E1F46674C445228315F3D56546D3B48625F204145432F51754F49804B3A6E6E
194F6C316462259453385483733DA35331969A70564F50264C7363302F6C5C4B8BB15644
8653564D535B3C4158386C81765D6136809045782A3E6D8F6E623D186E76692B2D957954
5315369E94435F7814255A603A97874540693A7AA7892464416B9E60477545347C888554
97C0663C778D9A4C65AA7C6ACEB22D4CA64E3D774725595154915F3E674650A16E999A5E
509E683D97781C4DCC3D678F897B5D5D4DA43C385D766669494489A1914E93A62333697F
8F4B447F344F9888586150394E585D585A623344736D2B3C66422562705C4A2E8F7C5757
8D7639354B603B86753B929E699468592C50854B292F788668455C4F603B4439
79738465958D3C4C5F227940794D2B643553462048471C2A3C172F3F2E4F2F618851262F
38205A2C3C16359956404E2450653A4957492B56735B213743251A27594A508459233E61
1C4B532144411D751E202D5F59339F5232A3A6604F91793D4A8A501A2563824FA6D44839
9D5E562B4085424741185480704E32195D8A427621263D625A565F2B5C69686B2F864755
7B663F8C4A255487623542612463846E47775B447B853552694E6B705471996EB18A9C5E
90C7834681608D577CC47A64AEB7457AB6563D70502E786F5F9256597D48419348938D58
4D912F275740183EC84D456D7B69566B518B7736435E6163712C50A99459978B52243A59
8C5A406F7758B9AB4C588A636F51675A557A535E887832376760387C74654F44A7905B5B
7766467788703E7F7554A8A96C9F5D525384885533347B8970487173703E4B31
967C7442656336582C154B2E561B2055295B4C0E3349151E47101A302C55293556632F32
3F26542C4D2528674235604F99882B3E7B6E39334B59223271682E31571632768C345448
11558D44533A186D4A2B356A2C269C804E9CC22228587C5845965E1B1B3A744598D0662B
86577843366D3A676C3A46663E4F714254652F766935444E213B8E505C56477D3D77563D
6A7D4BB74F3E4976635A5D863D6E7D3A61947F46868940729B636F6160809C6FB3845544
88B97F387D4E8B464495685667B4335187543B533B3D85695E983643603E346D3F756A31
407E513251321A33C93D53877D514A65557F5F3845635067833A55B69A64928242293F5E
7B693F697B4DB59B4E8586586D49457D948356657D6E28548F7C48746A712D236B785661
875C223342593488682A588141987F4E314B79503C40844E344C838F5E387730
C59384689158213D522F2F1741441D3A0B365D2D3D5A030A553425230E4D5C6D7B800C23
594C3C1360452F7027203962BFAC141A417351355331091A4D63323A522D295878355B67
16295F3D60461951673E458430108178486DBD693D535E373F8D955C3836522453A08933
78305A583B522343775A4C6648426B53558C4185713E657E61437D495A714F7231776340
5E4549A56746546B472D3D6F325CA471414A7A4188762F5173737A6B40508269A3707A6A
82CE8443624E8D6E43A67C5F82CE515FA9693F48612D7C795B8E6648654926787A946F32
4185653454432831CD514B8E7E6338557B887F3D3E614E94943C50A58055AB9E46323F55
87475D887B48B285328372507B5F2040495641788C7029205555215A7B5C5025804E395D
7D752E304B512386754665942F7262715776793127437D644A2C519336325F30
A5D6A8609BA542386B3D5942586D284C443E4E3031442315401F29351E4369596E7A381D
31455035573A26875C1F31394183542E40665944836938194E372B4D636736615D124582
283852173957255D2E272F506731803C1F3FA46D3F5B58323B6F525037497D4D9CBC7432
8D6261523E75424C704E5F7D6F4D491C528C46823C284054543C7D5257704B76336A7949
547848BD6648556E734B618C2B57B39A5956993E8985475D66526C8658558D678C73789F
CED8946367588279409F8166A9D67158A474464A6C36837A48907150595E248690947753
55A6893C6343283FA77B4D8F82884133536D553E517B3F7D533058A5702F5655523E5773
9845295E3E2B9C9C3C5F4F27595954525E6A294B63833D225E7249514F6F6D2779684447
706E2937455326674B3654A2638059646680743F2B3D5E59643D6C95743E5125
B9C98E5493A7493B5631574F4A5E2E644C3D4928204B2B16501F2F4D2C515F5392905526
3633513F63291B866D183320475B6D2A50511A284460481A4836272B455C35798A45337C
293B80543C66335751262B5B623F707F4B54CA803B618E5C4A7E59262F6288579EBC901F
75635B3627705158584D3B4B7D4D4E4F417D4D5F4E393D504D308771839B578958748E4A
538651C286544E636C4C74A33649AF9B654E99398F925254763A77945D4E93708C9A9768
A5C4964363B6B18C3E9C552F86C17E53AC434984712F73647DB67256654F46969592A13D
386874356F4A23189D653F6F727C673E6E637B3D3674499389473A83956D7E6B46293345
916C3A667F3E8EAD5670826B5D5761585C634E596B934D226377515F50685E318E885952
786333676E783D6D5C497EA863836F5657847D4B2944524F2E2468897239552A
AFC27D374F753B3E3F1F353D3F391546403C480D0F442D164F11122F2B4E361C3D625024
3F3432316130155D3E0E350A326D861A435A5D324666221957633E3E432B17567E502967
1725674F3C48164B744037663B1D588B663ECA76283E51564371503B283F7F568FB68D38
7466535B4B78644F6B8048568F59566C477D63656C4E474F6936837872AA608D675B7145
54764BA98B404362524564944254B8A064537A4BC0B66A437B3D91A1624A863B52908A60
89C99E3E6A99924B52A778575DCC3E368F7036333D3998653E6E5455945523715278A85D
466E7E3F5B4D3519987B4D6471856C43A0718F47347350808C5D41939A727C7359383449
997C536D80488BC1647E9E7D645D59648E88634B5D8A513F97854F695177321F6B745657
6F5825444E6E3F70622641862A75A277496D7319204A5B716C2E4CA04A338E3B
C8AA7052858929304F3F381E275E2D471E223C2016330B0F3E26202D153F5D506C772F18
3E533217574C1E7B43104B241741AE454157673B4466411139473A343B64245E85632387
3B2F8269364C2E3471433E63502F3A9C673DB8A7554D4D582F6454633A458B6886AEA14B
6E6B47615275584F6B894C5C86577572447B567494575254764489844C8040A56D517C3A
6F83529C9D2F5285804F5892232A666B4F555D3488A3703FA02A536E51629F6D5E9A3B1B
6DDBBFA5AEC79F794666908681D28E3A698D413A611F6287457F814A5569256C9189A27B
4D72A453625D4439A2A9586879AB532F6B6584434066328383623B8B8537879A84583543
852F567C834882AF3778896C577A453F515B475B8D9B3F1F50674961658F7C33848C5F60
98713339405F285A7A5E4AAB3D677A8268867E3718303F76712C57B238297E5A
82A57A4D79AD6032575032472B6F33453320411D191B17134630243328414C4B6D683A19
305929325C4A1B6F43214C77302C95525651633B42665A173639342F3D6A2D5679682880
3E2E7F72344C442D763746776E4C389F39369AB85C4C4B4D34849B6B394A774F76ACB862
60464A7150793B5F7A7B4B795D255D634C8F31426D3A5F7A571F603F4E743055374C8232
47492C80944939586A3B82AB753B9D5C3D4F7F4C7A96432B7F52727B493C738448A97C5C
4DA4BEB3C9D8D0A344669D8989D8A23B4F8C5C667D264E8145967B544E6134767A699676
378D9E575B4E191C758A2D56608962356E4E7B283A78546D591A399CA05C636F6B3C465F
956A3A5283377CC2706972504A68684E6B5C3B3E727D71347E814A484A807347959A3537
5B6C544D57704C5F705F59BA707D7A6E47716E581B2B2E7B322E7AA17C3A6B50
9F82734D69B77234585E28513578344F492648251F21281A424E1B373743504A63714122
2D4F253B50541B694F233F76503F82525852563D4059511A343D203867773551842F1B6E
45207745396D572E71253C62633736A6362170932F4769551C3D4C54325F843E57709837
53653839236B44344D3B2F5D6F5655463776703A622641516B345B6E3A634655754EA958
4E695B8978534B6E6B45589D713BB08C5D4F884AC2CB7F45734D88B071437A4434737664
59696A5C857994844162803A48B08940476D2F4156213D5D539581584C5E2F6787528A77
2E5C9330637A4B1A77A44359507944377C6C7E3E3559367E7960346C9A699BC471232C47
7680496981364F9B6575A2734A5E41467E6B6542777E3C274672695350703F2B587E543B
635F1E3E516C4B60652430745E756D7B4E659B7C2E28276571314C9573367360
C48071507BBC7135504E385531733F556D2A5F472D3D411A3F4318403F43643B5E96572F
2E3C2B4D3747136D853235671A3288485959453F697D5F243553231E354A22589E721E66
3727765E25332B2773293359661A2799503BAAAB23334F472C5651371F41695D7FA9A74A
436C424B58644B3F5786586083715D7A487C7F3B7640454D644764803B7E5B7D8E4BAB65
60706682634F52ACA15B59BC4420918F604A8D47618E663F6D466A8D6F416C8E4C9B7773
5446557AB9D2CA8E4971BEA261D4883B73A04E44742C3A893F7F9A5E5A7A36657335809C
54679C394D652F2F76A35F5D538C3E245F456D2E2B64546054663A6C832A49695C3D2542
7B353D6B815E5AB440375A6F45644945635B714777686D35476F706454557B3F778D533E
6B5A22313E4625694E3F3C8359737087587EA1742F1F2862833460AC8E437078
B44F6F3C3E603B30541D1D2B1F4F1C3B58256E381043411A3D430C1E2A3E711529535825
3E471827244C1840341720572C273F1F4B5E4C272C4E39162B67552D3E56313B8D872F72
611D6C8A35452D1C6E2F41535F4334B371459CAD3B3E4844315D5D49314D7D72609AB968
3C874D64625F7144768F635E816D788C427452377151444858324A84346B34546438772B
3D695A607C3E296B944F4F9A693A988B45548D716B9F4A3B7587709A5E4F8D933F986848
3076D2ABD0E4DC934752AB8F77D083354E82526B672452924C9495436D80568765366F92
2E518F6550614C1958B25C5F5C7B4E3C7C57743828757D7B5E7F476CA35F7E9476532741
6E72413E68415EAF874F6455465F505B885B7C4A75816332457A8B6A5C6E8A46739B4241
92884035424E415D6264468E5758A9A45E809266311D2373823461A8873E968A
C34E55624F9C562641572D231C4F463143103C52285A301525451B1C2E225C5C60778B39
253B233A1B51374B4A412066693A714F54546461385C60271E62643245625A4285722D71
6A1D60914464582275423956846B39AA7C4980C46C403A473752A18C564D7F5F4F95C685
35693165564B5A2C5177574F6F41497242537932837B35416D685988396559655439735D
475C39539F61377D73224598662F8D903B406C735F95673D5C8C67886D3F6E874CA27776
3E3ABFE1D4E4E1AF575D883546AA984F5B7C34345B34345A41749C4A45522C4A7C4F6A9E
5747B07842624D1D4DC3585DA4A1644B8A58774D326884586E6E4D509C886895A2702E48
5E75454F773E5DC194506165507A716D8254634C84AA5B313A725F5877AD954562AF442A
5B64602F49542E5BA8874B90523BA39E5F91A1652B436494823C5B9F5318707B
CC57556E53A78E344D772F4A3B615F2C5730463D1E4A301E1F36191E3335536668688D4C
2D372F3B214535466056274363387A5C524B4D5E377263201C564725617C5838766F2F52
701D49632047592671364969917B389A512248A7623637433127414E40366E5C4273B476
2A675B5B6E4A5F3C58886B4268845378525F923F783C335483625E6A3C6B58667A4B7F55
4958613D814A327F775645AA6C2A8FA876577C6F63B47E4073855C92854C788F5F938688
3A3F83E0DFE5E3AD5D559D867EBE7D525F8F5B417247316B3B5F9D485056365783566099
4243A35B407F591E3FCC5F2C5A76714D7B5B954C4478714F73424268A5925D969A38447F
9A915159772F51AE91485B5D3155665D7F5F5D29648C8944357C4A42497290605A995833
72595A2D5142433B7F874E8574488E795F99AD6D461B276F7742549E7E467274
B55653594E927E354846334D315C4A3F68303F3D1642331B183B132D4A3D5F464F837F4A
2C43264726311D326C621D34312F67514D5448393D8165371D492D18385F663878935F4F
831A3E99485568286138273C68402281755B61CD432B4C615C3D6F4841438C8E5CA8C988
2F796355364A6D385A6740334C6B485C3A467E397166323C605C5E79546B4965974C8555
60797C4193624197AB6F42B54E377D916A5E733A538C924D58513A958547824733799B71
494F78E4E6EAD9AB5950AC9588AE7A4B7EB96B42744F335F4D6696534B5A3D606E436798
393E9A45356E3E1933C95244685B4C31826990432D5D60497D372E3E725A47877E1F2246
5479475B602C3FBB8B4759844C56404578946B4A618A6434549476675F75722F63926437
6A61424E837F574E7F885389935291795791AF83581D2D6E4B2F6BAC8639703F
734F5158427978343D47283A2D493E25493A4154152B3B21172F171D38394F422F455C37
2C4430382B3225234B4019304C31644B3C5B65582853593B234F54304745392955795B3E
6F17439253545A29705E3E4B8840267C88626DCC7344433E4537845F312A4F8452ABBA8E
305647666640552D658D7430474F3984654A6220627B3D40522549744F723D3681416840
37596D3B9460255080633DA75C2B5E7E8A69698941779945478148768E49625E5C7F7B4B
3B2B6BDED8CE748E6070BEA395CB97387CB96E4A6851458E504EAD6A608B49578B2C60A7
6D449667366041202DD37A4D75536C4D9797A768285A4B4B92634447775C4996784D2F4A
588B528E784A51C0673E9380435A4731354D614C67764120225F68635D6278555293542F
634F321F47414F3F77463667853992703A5D8C612E304E8C69322D786B31907D
56585179557C8532456B4A4138496B325D2C36602E364E1B19492B243A193B6D62677B2F
1F3B58391B333B325F421A1C5B426729325B6B783662651E1A35452E5148763C5785593C
8D2F32823B5B5C2B5F48375C7E4E297470373585774B424328244D675D2B4B862E738297
3C57244B664548273E70712E4A623169654B6D3755763F3C566A576F537D7657833E538B
5E5A813D9955425B8B5C3FB38A32816E5C498A90446E864071924F6E964C6972795D6964
63373EB39BA25D752E4684A365BBA14F519973515F50488E5243A257457356519C5254B9
6D418C99456A4E3349CD9C4C66506E2D3F4D686925704034757447467F393E828064324B
5C5F2A4D67694ABC71395A7646523F583D6D8C6267725D49246080776C87A769529C4524
848E4E245A45483DA58B546678309C9A6E89AB7B2319295474352E895315616C
724E53765B78832F33553C2E3742712D5F43316F352D583516452B213A273A796262916D
2A38663F39324531736A3E224A2C78672F5174633377784D203021183D445D305B742322
6930317F2C255535574F19305B47287DA96E4BA388385E624C365E4746203E8E567D98AC
376F6E5F58517A65466F5D346F934E704C539357555D354D907A5F742F5D885863264A8A
5E56543DCC8F50586E3326846A27796E7C565088415E8B3B58974F676F47989988507E96
984A8BA894878AB172528C6E419FAC6C579D6160896032745C40A75C3C6C67519F7E5CB2
6951AFAA3E7E451F20BD904B7F639974675796823F7D89456E715662A288436F725B4278
A0A056567D6956BF9461738A5D6474733A3A6D557EAB83592452572A6F9AB4693EC24E19
4255662F63456644717F5B6C8D3898717490BD74531D384471423E9F5E23685D
7B4F4E594D819544385237475F47642F6C4D28562325492C19401A21472F3D5C3E739B7C
2A342F325031291340773E2529124B4928547131223C435B2D3C563E4138311B6C9A7727
803D2B7E702E723953732E3F75381657857B53A8862F41665E41714F4524519C67A69FC5
315A7157523E615E41795632536D4E5D43357050467C32345075535A66517A5475614F83
555A511F928555566A3F3096652C86AC895D609B5FA0BB585184596E8E5565907D46A9A3
685D7D92343961885F5278804A96936A6CA7742A565B296143659F6B42723B5D927752B1
602D638D3AA38C3B20C8AC32666E8F6E826E93401F4C894A8F471E3661823F829C321F46
67926E5A7532378791676E68283152694130544D4D5B61572151686770549A644F9E7247
5458763A827C8B5389865B90A349AA623667A98C6B4567676C3E339373417550
5A475863374E603840401D294A4132124051326A20154E2E1A541E10312944551F497365
2D4D513E3E39562451613943743C6E65306C917C504C4C481C31423B4141643649718027
802F2D997B2D6331608F49405F5C355E84674FBA986234474C4A7247522443826A7FADBC
5C60664F6352824D416A8449607361597A4D796546854B404C6B60776C4F6F4193654C78
507C8A46C2644E6D94643E875C1B5B676E5465953B4F805C4A632970875E769C3E3A8F99
696087A16B4377A048608BAC4EA09A4144889A45702D3878603994414D9B843B92343B91
BC4C728B175A924324C5AF4D5E4F654B6891986B29516B3C897B5A448B7E499BA3884140
5B91497F837A569F76538D9A5F4A4343495B6F64506A604A397B94786E548C553AA9864F
54563C20605E7D4B9D793F6B9D46BF897C70A56C351E2C3C5C4124887A327687
663F446D466D6F1D2C41382B293D5422393C2A6E361D3D1F1745462235172654447E874F
17295B39342243234F5C21193D375A1E23486B7250394E41202B3B303136643C4D7A8A2D
6C4C18556F2C522A4B8A433E61663D4F7B4944ACAF6042433F405150502A4679515C99BD
6058525B784270396181734D626B405C6A5073384F7B4B4B4C3627476639782E4E523F71
324D6745BA91393F585C47909355839E6F58ADBD5E68A44353AF69715F3B85A0883E7D94
1F2646A38B52769A46355287558E925A47898E43614D1D436D3DA162305D733B90453975
763F619D406C673648BBA25D6568663239586C562F4A5B42705B4A45875F39798D6C4343
5B974F555F735AAA76425B74594C513F363A62644D7E7D5B1C3E707B66529B994ABF8555
777C69364F697F41918E5C7A892978576981BE8176203A3D856329816A3E6875
786D47744A677D40314C431D493C733339361D70521B512615404F243543295C576A794C
203269374326491F4C5E2A1D5049503523415D7151335E4A2D3038323E4A6A42467B7338
88731D54802769574084394271532C3A6D3140BE9E5A49493923488069394A723F334D72
54545A3C583F734F3C4956427C964955624A83633E81334F8A7C485F4942936A592B3E94
6063402386A07655733419548F5571C44D374E835E7EA142346D676C5C44538388516584
787090C1894A80987848638A4C87B372568A9A3E6665224B783A9683456C79338F745688
82337EB0426873282BAC84567A94795A3D6C6C5F3A5E6C507C6D575C9B8C4B7682584A4A
63A66E536D644FAEA0836A685B5D667B4848573C4B958D7B214260637894C58E48C26E2D
504E693C37715D2F657D6272A152A0546E8AC7987C2A3840867A3D797659696A
937E4B87506490663E534D275B458941405A2E746420534323314924493B3455546A803A
1C39653A532B4C19468C482B412B51472F4B6D4D2B35676A4531441D2C3B58393A734B23
565D1D43581A4552376D192F4A4D403A99543CA594434E82993F364747356D9D888AB1C2
3F4E8B57591D4C89535B401F4671725A3C21557D48842F2A4B684B5F64397064645C3799
605660599297745C82552C56893760CB8E5A78A45FA7C28A545F796778815F85835F528C
8F84B1BC7F3F8FAD7C4B82A03C849B6A508D9D4D7F6F28427C3FA783466F613B937D528D
8122559828767E3329A7714B4D5152433880645B2E3D57447553365586734071833D3059
79875E4E5C593B6B62695F693F3D435A445258593C5C706F1E3D61474149797B49A3683C
5A537F4E2F67503186705967A9599A594F7FAF9C82373838673B4C927F5D5B5F
5A545278313B574A3B551C1A3B3B57162E4E2E6635114944262F3C11202E2D59211E504B
214B3C17422A4C192B4736315A414236245F816657214B522A4978732E334A27478E923F
704A1C4B7C324D413A8C623D635C12274F7254C3C751254253403C54422533547E54B5C9
6F37725C684E5D7351558E7062737F4B8C485590499776474B704D6F8D4C795F7083469B
566583649081565BAA8D4868913542C37C3F80A05461856963A75E476E64719E644E71A4
8A8BC6C3882957974F32647F527373584999873B6B574375573C86683E74713482473969
8A4B488D2948652D2BB4716869625F474D835D632E425F4C74504D4B8072336B743A3F3D
518E604C605E43868A8049816D57484D4F5655604A657984233F657E5451838E5994794C
625C604026675835775546619C529E5760509093563033377D7137546A344650
6A413D714E435D232B4B5329232C6935243117584A1B45271730542C2614125747408041
093159313D0F4635446725173C5D521E0B2141636627633F131E41442C423D271D4A743E
646B2228693A4F591F705542656B4E215558328FB081485C4A3C405C59564C5E9154BEC5
7D426A51765E67544E6E916C69816D53904F506D3D91694D5A67384E7A4992545D653E98
514467528CB25F3F707D2658896540C8893F617F68737E4F3B826F4D612E3F76664B36A7
4A457E9C75446B968346658B5C77716F409E9C4055502538633383974E5C933771785671
9C564787544E773D27B3698C6B6D5B6559924F7C3F36726773665E55897E40666B5E5E3C
5383725067683D8F8A6742A07A644C505379736242647769395F9C7A544C758353918F59
6158633E3B88833D765226478255876D684A667450374A4C8060385E75333F65
6971487F574B7757243A5C29272453502B41112F5B2B5D311012312228371B4E4B40893B
13272A1E3524322E456E42263F633F523B3E52736F2E6F3422284C472F4A4F5C30678A48
3F91422A6F415D6E2A7F5D3A6869532054695285B7804A5A593A457896584E507830A0A7
7841714A54536D6C4551665366A270417B495C8A3F884442637C505A5C31747E57392E85
6D4957335397794E807C2D5F95543BB3956F688283826F6F497665546E86618472613994
9177BCBF87425B81864C68896B607579469AA3586371334F7A397CA15256832F6154506B
8C4F45946848703F31BCA993798267793F927D9E5438705778805F54A28F5E8A817C5E3B
4A835E466D754E8B7F63449D79585046365260543F70896C30345C72544E72603257864E
61654F231C5B63286A4A345677507E92654F6B6B291D2C427560395284364284
70914D995B4B7D7A35487634464680622B522D3F77345245362A392D2F3E25675E3F8F5C
293F432D37413445328962293A6440755D425A6D5B318362462F574A2C4E535E306D7B29
288C4D235E32566D2D824931657433276986705CCA7E3A759B5639656151577EA24DC0A8
7D3177664E2B4AA259505B313C66844E5122438B41855B3D48686459775276975175476D
704C66794E945A4A876F2D6598423295916F6C7B8C8E9D8D4A877B496B8D5D758577439C
8A6DAFC2834B59A08F5063825A81AF9A468A9D56707A3A4A743A79954D596C3C796E516B
893347855E406F2A1F6B537874A572522753427A3E325B3A46564F61A6742B525E4B4135
4E796B405878458D967E45825D4F54635658566B415B6575383C63514167857548387E2D
55655B43205F6D1D5D6A464F6E2F7A776754827A4B272C3A7C7833447F2F3376
6E8751B9694F817537594F2F474276512B5420336D194944302F321D2B422B6D572B8974
373F4A2B3A472F351D71682445482F58394B6F70523B735447376E5E414A45372A737945
289042317F623C692473733454843A204171754DB36A3D4755514B725C3136499157C4AD
9C4A726B4F5F5F88554C71814B6E7E476C594C8644647E454A5C68545C44728B4C6F4278
703C677C449451487A5F355B8B3E2D90935D71855572979147687A5A918454787C6350A0
9071A7CC7C3E63948A4A628B534E7E7F37709750616A353A6C4E8E88455B712E52825567
7E4B356A653A7545236F518E4D877B6045654A8A353F6E6556664E6AB87D3A6F5D473746
5A847949596E26567D6B3A787750494A565E58784453738745397C7D6443666642277861
606D6C342B7B925254643346805C838556609B94772A2B39623230688949306C
5A55487B543C5F402034432124206B441D4D1D2760213A34152639251C2F163F5129666C
112B3B272A21203B2171531921452D5A403A42686436844D3D1F465942534E5827577B47
296D461A6365357532677E3E557D642A477463408F7051514E544379615247559C5DC5B6
9D51705D5A6D5A6251596B79477463456C67506B444D774A4C505B607149727C46833D73
674974823C986447827C4E508E5830798E7F5F6A8977938A3D4F703D616C4E7E5D48569E
855991C5844855818D4C68937C5D907A437ABA6E54563532633D668F404E72424F8B5E68
9A78418770398A682D7E66A85D7F67883A545A94652E7868708F4C458F8A4B85867C613F
3A66754F91944957906942919052463B2F626D71474C774F2F3D817E57445B643B1E686D
82A36F2B19456A45485C3B456C567EA4705592944F424D477A731E32A35A2F8E
5D854C8E6F466F89333C6C392A3255572753291F5533374A2B2128271F4335474937616D
1F2B2A1B1F391B37225B712A324B2F67754F557B7D48396250254F4642574B643457734A
3449572070762F873D5D7D3757805B28466A5133737A5C524B543E82A0564B5A8B50AABD
A44F7051596754714A52646D40686D4572604C693F6B844B524F4E445C46538C48885058
853E5A7E3F99733B526A4D37797D3E729A837D7F8763A5893752814B718A5889848E4DAA
785B82C98E565B777538649F845EAB813356A06F4C6D3948865767873B678640497B355B
7F6F489E944261442E59699158766F7F2D2D346E612C675241524F42977335495361574B
3D6A7134456E4A53AC6A2F6C70597659434755674564A18253235F734B647D99543B6B5A
4E5A664B23467B304D7A4C497939577E71617F8F4031447C8B6A1E36BA492D87
56954C9F704972963C3D693A2B47626E32523C245A34343C31312E2A2035304B5039536F
292B2F2B22461F4D295479322B4D325465455172724124774722454D40515765325B794C
37385F1E7081307B44438346669D6A3246614D31548859524840335F7557455E7738839B
9D4A65665A6B4B77564D698536607C4C6A7043695D438E4B474C82505F3B58984D6F3859
9544558144957650558C593D8A773F6D883E5358755BC9A4405C8A4E7D62468987884EBC
5D3C55B98E536C918A556BA768618F865999CB7165823E2870516890494860315A8A595B
8C45266075396D5020564A9A45848370473A507B4E4D76664E5A54439F814C6567603E4C
4978894958743B5F977E4F80643D4A5F634E58632C3A708363265E7B60415082594B6480
53686C4832466B573A7362456F666E95796C7D87813C27346761233BC0843778
61A24B9754466D7C3A394F323E4F57723947411C5F472C482F2D3F2B2027254657395487
24232F26295021522C57AA48244E2E5867465B6B533C3C7367343E5C2B3F666A32618836
39445E1E615F34814E377730428A6E343E5C845943836153455445676758455E776497B0
A14C567B6066526F534C5D8C3A588C586770406D654190514959764A5742588B4A724F3C
6B42615F3683734847932B2D6373446F9C806B655B316F7F606A893D727153706D6957C4
806270CAA03346596A265086884F8B6C33517D6F4791333D863F72905B4C6032388B5E5C
955E325F78439369226152AA4762616B4337628154315262476C32306877455264651E39
6170864B4C77563BAA9E4E78915A5A52644D5C5C394A6F876F255B867E4D5095615A6D87
5674623C363E81453E73593A7156759C726BBDAD94412E466D5B2F68C7753077
5685479D6C47748835394D2D243C3E53213F46206335305B2F27502E173C354951203063
3C1F302D204A1A42162C5A35204433566140576C6D6B315267303455443E496637697C41
302F4F1C5E90337B5D2E88534A78763E4B6F81513C88625748594C63616E4C6D8A6D91C0
B442578053642F7B48526F5A3A5A785681593F805B4584453B4751466853578334946730
60376985448760354C7F4B32655B2B4092506D817F4FA08D368BAD3F4B5F4196653F3FAF
4F15329F99615A608F4B4789884B947C437DA9844F8F2F206A594B874B75985643853A5A
927A415E892E5462256355A75C6C6E8257476D8181465D7A4F8E6324739943486761584A
3F5C8E5351816B519F6C41787F546E45554B785B3F476D826E2C6882765852845B66878A
4E6D742E345BA3543A672B2A746A6B9A7A468395693D5985873214388C7B2A83
6393466E685AA79F222D5B4E2F413C73473C492161451C5B2F20524F265322365D3A376E
2C19453A2B38124B313364291D4450583D2D576C7C803C59601D415F4C49444523657746
3A2C2F155183306C4F3091564A657434345774494085515D596044798D6A2E4B74506BAE
A154475732604F86423A56613D4F6E2C696F418C31327D594C5C4B354B494E9B5B755B46
8F504D78458C97522B6C3B267C9B5D3CA72B4A5275459B7835508F4D59702752607E4ACB
6B4262B49B7058698E56448B8E53A19D64788A90538A51185F5747823A38583635604948
76612C529A41644E2A6283AA63838893322947407C414D3F316C692145A42F4179786247
495F8834618D7044A7732F5C7F5084671D30545F39477268282F576D55534A85273A6963
4A82672B31337F3C3E805C2A7E405EA09974AFA155312343736A1F3EB45D1893
5C884D754536505C322643363C20286B443F3C0C5B3C1564372A5C462054324155404075
281826322D461F423F37784B213D4B3F4931456082784962763B34544B4B666C36557134
3C534E173E781E3F542474394F84684236755F2F3F745963514B26434E55394A6D595495
974E5080565D41917642626E3E5E97655B5F3CA67B2D672C3F5E8347533D3A7A60694E2B
5F53525D34486D5A3082441B4D8B433479726C52994F968F5A5DA65D58856D83768C4CBD
A7906DC3A0535685945D477756407E99696F76634DB2631D636747915B4D5732387E604B
78632159903F6D48183B4E9D354F6C81534E9B75AC537E554B775E2E4997522E4A5A2B32
6580885A4C718541BA6A50457E619562603D5757205695765C284C472E617CB96970803A
355F71644C3C902139845D2D7D504D849564BDAA894C2F4E6F7D2F43AE722A6B
547E4B8B7F499689413349291B322B5B16203C185A2F16481D29693D1C3C2B355F251F43
331B31371542243E221E455E5C593A2655364B6E5A362736573B34572420344D31648E46
223B401A50862C28392A80572F54721F2E6D834C498B294672713E5550462147827C47A8
BB4B317A53502D6A693C5B502131806E553E216E7A2F6F5636406D444C54469E72707C42
6369557B665E624D3F8258315C97423F9B896652884C9EBC8749A24C62716B7A6F473593
9A7969C2A23932627239458A8962789D5D648A803D68511F524F5C9369495A26206B7242
787134469A2D627C355166A65B69666A3752BA8BBA414A8456A558234F715F3962614445
444E72386678553193AB5E44672F486D763F655C182E5C6D5D274C4F213E429363728571
3556543C3A4AA0302C5651398C5D6892895799945D3E6BABA1683467AD833377
525D325B7646A08E18204D4127252A6E4F2A39144E43214529175C5235511E27655C3151
3819534A38441D504421413F2A4967324D23486B646D4339751527605831403C0F2B625B
2345370E2F6737353A155155315C6E3423476F4A59914147525643606A47282648753A90
BC6D41755389688F593C6760564272635670489A8F408175424157385A64499F427B8E44
592F4278605468411A646F455689222179785B71743467796E3B8E445067676E5C64438D
AA905EBBA4794E808D384378937078B4395892954E8C632E666C38906F626F5E2B72563F
898F5949A6355471294E70A2635976845971D673B0494C8A518771334B7F69385D6A5454
6B8DA0434B596B30858648536B46526047347C5E27485F5E422E565427514FA4514C7C65
42676B45343472393E7C3D2576453C888B68B4B7654023425658283D955F1C6E
566B4E577F3A8A982C213D46261A1B715C263A083461282A4911364B3C5D35254B442A3F
390C2138373811345023523016215B29491A34456E7653315429203B50294C502F3A6352
294F631A307730374226675D2D706E532F5A6436579A6D5967472D5F8B694439485D2C7B
8F79425B176C6F8C581954745D4474354D854681552763683E46442E395D398B4C5B6E3B
756045696751A356336E562B46A06823836C4670885C73944D4596725F714B6B91835780
A94552C2AF805673936C396C94749ABE7F4C7C723D805D185A6D3B9D413C64662F78482F
6F74274FAB4D516B2A415395466F98A46363C56E9D5A769F5F835F3A6CA9883A6F6B272D
424763584A4D723FB7C64C4B725660655E355A572F5C9F811E2D595D305C82AD66577F53
314E52423C2F88262B60573193533D69764DAFAD77593B537478363F997A3179
58786E58993D85B8703651612B473B7B5A22412D4575291B241350483F49253C62552730
3129373938342A333E296F683B475D2F5A4C4654795D482C6D502A3C301D516A43415D35
215278252B70161E512E63431B617E5435767B3754A85B457A5E354C613B3B3345663DB1
A0783C835E7561848B424F7061426B5C415C495E93426D71464F7260564E248887665131
79744E662340A8793D787715388C6B327E8748587464629E5F378D7462724E4171885F7A
B1483DAFBA7E52A2AB823D648B5571AA906099805456842A535C4CAC8A6D7F4323755C4A
A38A413CB14164753D3C4DA34A3E4D424D4BB275983E4B684B8D5625274D6C396E5C3441
43487F6654516B2851865F537E3F596169415757332247574727405B172F3B8668508178
3F50646054336C3A2D6265358770537B8655BBC89067405A7379363E91863B76
4E5C5A506F285985522F4729234434712B26513F31571723351B3E4B1B4346416A3F1C2D
675937412B3D532F2E173E5D3242340E4C414062733F41175049283E49293E4845427262
212C5D23287C311F3F2760772C5D78212056735364C3272F697E445A933B2A216B8B487B
AE83206568695F56834A5372304A888357745869A95271453C55705D51462B638A5A6A4A
84684D758149A253417F835137997530446A6699A26562B6813C9A49526D795183574350
90542D7C916E418A94705093B07567A5733B5F865F5666357278378F4D526F792E6E682F
5D7C4E3AAC4B3B64333A4AA36C4C61535766B583994458815C81654A28517C467E61364A
41478C685154794870AA4B47885F434859465D4C403B565F3121375B28494B93794C877F
4A50684E48346E513A5F553599824D7E823666A38F5F415A74482E66AD933A64
795C45517D494792322D574E283E306A35163737234D22102A173E5436353F24634C1F22
6844494B3532292E3244645A3147613E532B4C817F7B681C653421584F3654451D2C6E6D
2F325B11247A4F2569153773325273522C476A5E68CC774A5B4B514BA1402C18366D3971
B0A0416E6D7081708448585E362C5C74566A2C569B51745B3745583C555446728C54804A
93765A85614FB87D3572822B2A807638406244526A5B43984A4AA1645A51574C8F784F4C
B06862B9B87E4C8FA861294A98665CA39B4A6B987553682F3155308F965C6473364F6A48
71925D3A9F5F4C74483746B574556F667E4EA0799E58529E7985685D2E5786498D67414F
3C40736E5A5B70457AB9534E7D6D4D525C3D5C523437585B35243773284552877748A287
48516545385BAF65316033206F66477E5F5F8CA6815C41596B6E32347A7A3962
6E5D6A3E6F5D4CB3391D2E46342513625C22402016543C103A0F1D54512F5F133B5D3931
7A3028404A352B314D5077631627553D4F252A4D66684821663619414C314E4A39334E5E
2D2A6523196449263C263C78335A7C6F3A618E5363C389637E565A52BD5D492B56854676
9380377B5D5B7260A04348645451946A415D435D9D3F6453447484624A4C2151894B6031
5C76536233357F5E44708E34256C7F403B70736F7B7B47A962407D5B57567C4E8A7F5D4A
979E81C4BF955388929A3D56AE7055979F4D619D7B4F6D393B5B3081A0666079374C6C4C
659257339E62456736354CB86754705E70579EA9B76B499A777B846C345486598B745759
4036667C565F6D3E74C3745499715D53664791945338576831304E8C554A6471512F687B
445C793B1E2E70402C8D45367E593C9A857089A76C414D959B82373F897C3671
6C5B8E46868246B78940455737403A61612B4D2A18643D17412C205741264525576A4438
624C33425639372047368F8E3A3750294E4D446A7B422D236D4F2B3F3E2752605C416662
232C814125744B3035344B7B304C6C45203F4E584997503D84814840A244341D2E63528B
9C91284D785E7842825B44573B345374495561448258645A3B3D4D5E4A555165944B724D
647B5F795646A055476191362C63823E39797C7A768C49A67C4985755D698B5E9F827446
979276BAC48C508EAA9B4154AF696DC0AF4E5A8D7758863932673C7FA4676D703452634A
5897513FB17D3A7835275BAD6A509C9E83344858855963A93C5F876E395B8038817F555C
403D5F40417C763B6EC94B487C6B45545F3750515834777B211D2B6D4B407C9B8B488B7A
294E705D4D3A742F1D6E6740775B2B5D5E70919EA14A25475A7035427B7C395E
645185457F4C38A4843C48452C45546449235245264E2F152F241A4C351A3D2753693A3E
735B3558462D492D5D1C536A3A3E5841543D46978D5638236C402A5B5444494D493B7365
3A1E722D164762303D2730673B445E6D35465C7C4AC88B536A456B4998615E3133776480
9DA64E5A8E667852A76D3F58733E5C8B4A59804998676588575054544A47486188436D50
6C87555D4E4C9A634B3E9A423172924352827E6A717855C6A84C967756A9A2679F74424A
8A8D6AB4D15528678D9A3C49A44A3770A04F5A86383A67382A424086AD67806B28557959
50AF422D8B6F399E6C273EBB482E4C4C684A4967943C408E7F51832D4580814D5673402B
43688A694658592277BB76535F55326C7551604F41204A503A292D693621436E874A8685
4B4F6D55443EAF492D6650396B68436E5E5D6192A3682E57677A445082965664
72508D47928B3F7F7B365C6E443C436D582F4645273A49202F221A4C3A1E4C1949655043
9236254D54352D1741255A742D294751554B3D59605A45277746162E4344504F4F2E516A
4F2062541A515C31342E326C3E4366743C45557C49C0A4557C426C3A878F69353269616A
A09C5E4B8C598454B170445373415D89465A814495595E86524E4745516247589241915D
5F854172754D8E7E32459F794B68965136657962626F46889641775E34727E578D75653B
858C71AFCA9E668E9E934375C389212B8B578BB87F4F5D4E497F3C6BAB5F9F9D66518146
5AA571397C74257061333CBB8F6362657D6B466E90603E9F9C5F94643A4E825861725354
2E244B583D5D5A4C5AB86556597A394058426750503349602E1D25634B417970803D7C96
58597E5F303495633A673D31727F43645347224C79673A5D5E481C5188805761
684A793A707C44874121324E3A27303C592C39491F605527361D19624F244F2D3F595245
793C284B5D461C1E461D4F7C453F4F664266537B6D6D3522724524405150464C63406374
541D605F1D4C6F373E2E317A3C45758144424E644BBEA25682475E3787CA72423A634A39
91A9714478438F46875853736E415C6E34777F4081325A7E4A453F473753634384417A50
3B79474E58548782432FB274516D99714A788857749B5B8678376C7B4DA58466A29B803F
84602270B5925E8E8C7A29409B822745664B47A27D6059384D7A3B649938597357496D2B
45906A4068943B6A754035AE86646D5B9F8B517D9872409898599B7D42658E6667705853
37244A6B434F544069C25C4C53894C465C3F617A653C506E2C2545905F4189775A224A78
446288641C2F49875E633C313F4C3C6A5A3841737E602E575C6F44385A986258
7549944279945A7B8E4C3D584D2E595276403469373957233A371F6556304F4F5662423F
8941274D6247323F5130489D623D54593F70436C6355292B7C60353C6252478982497575
4C217070244B572B594F2F6F2F4A72864A3E4D392A808D5384484A2943A4704C4C69522F
7D946E44856080467B834C607A49788D495771488E78617F51517C73555A3D499C5D8637
398150583F2E7781583CB43B25588F714174473B50894E8067358484548B64367D7C8341
828E7C8DCA87638CB79E67429F8E405DA26857AA835263592E6E3851B56F7C6F48476949
579A64386B9133696F3D54B581545F567D2A5D91AD93528A885AAC8A4366843551676868
38204169407B67426EC06055878F4B476351534862324D721F131F536249847791425F82
434279401D2F83536F8F454764684A71594D2A495A462A5F7476453959A95C5C
6E499144807956989055374535446547672E3D8354655A0C3753387C3B0D3D51495F1920
82612A4661281E3D4C10236067314C232A5E3471A130281C457C3D57783122405D3C6388
4317546A283F5A2B4758407154395E8E525D87994CB9A03C8D8C8A433DD16C243B547546
B1BB65357958872F527A5C5B392E4E6F4D4D39295B684E6A332E425B4C5E3F4582607C60
478E4A525657717F42318E5C4F59765D38537A6A66A0676F8C48698C5B7C8F82877D583E
6D876F84BD89424870854E3C8C573A78A06C56A34E4F725E275D4B6DB76F8669435F7558
53944C2C798D36685B1E366E7D665C557B3E334C678C3D7C53528B873E667B5752498177
351D3B5F29445B535DBB764354744C4B5A6D6B56572E4B6C35191E57735F858DA84B558B
3D6694602929633D53A1876177893560856A425F94171741696C403650A81954
6C425D4395632E766A383A412D2E5044632D25512A5B5A18292D25706C36353F3A5D5C27
AE6926576C58343F544B406A4727545E3E651A3E6E66623B5A8D172B5F5D3B4B4F253E79
7B2753622B628041425C3C746D4B597D2A324E7B45AE912E4B44654835CC81503B406D49
9DBF83477E69845A7D9B5E4F7B4B547A57485D406370497F5C3D4953466F4D3D8E5D8571
4F9049686E5E638E312E896B586A795C4C718F716E865C9CB550707D33788776867A5A3E
6284806DC0998452A0A86F337197585AA76E55A0733140562251373DA2707A6B542F554E
57A465443D9237608668245A88725C5F92644C746F8D356D874A7C873E5D77664E4A797A
3B1B358650515E6E319167576B804B436C88675B674157753C1F21546A386895A645528F
1D304A5859296525213F7177789833475F57417C8E55243D4E713C3F99BA423C
6A4664314D725DBD60172C3A2E393A2E53402E4B10234527332E18313E322D2E162B4820
B38910203847252135463F7D401B2E542F672E455E6149304E84312F5A4C39484B31256F
5C243C682B337B3F4347327D78537A7B583A5456399FA45E6A4B5A4940B88F6C47416E39
86B49350733C81597D83578689545167435B623C5D4C4F734C3A47482E4B6341823F577A
437F553D566255AD522860695B4E89502D386F59766338707C385A82385873777A998F50
67844D4FA69A8442B5934B3B79A7645F9B564EB99844516820586542B88172776B345A48
45A8575B3F993A59755925557A5B5B86A673477B629A3B6574437D8B4264866545505256
40212D7B4851474C3597644F7B7743697A855C64641B3B60553D26727034514E7C52527E
63427F634B2E9658524746425E7E5256A48261649D1D13426E773A2744873940
82467F525F8B76CF7551283F5A2143347B5B2C4F44397231343538645744254945586E27
90912D334C501A2C3A583E9260353C6137714C646E65422E5288342D5853446A714C297B
57264B8B3F35902342683B824D3A55715D3855482358A65F634E4C252D939361453B773C
6A889549745373616B8348426D4C50644D3C503D5761417A573344524B4A644E877F6889
547F6D54565D48AE4D3C49764A3F916B37408C6287836A83883052955E5D73506AA98D51
5A87294C7C917D42C0A5421B40915F58A26E3976704353383F776637AB52548C6C3A653F
379E6F5F3A8426708C561E65861C3B5D85643F8E63A4297D77437C794985A87A3B636044
48545A7F53474C312A747F6A79711236495044706E38364F451523576E587B546F244373
5554A990341E573A2D705F5757812321687B6178AA38131931654A3357B71822
654382575B7A43A595602E4248285E3C7A23225D5037610E284C4369590B17614F62420B
679C5039572F204F332D1D7175353945175B43687E4D2418337E482B5A4A3A4947512A6E
632E3479412F9046344527788A4661766E3765724489C458768A817F52A78D7C4A429C65
A7B3A8479169745B64AF624F704963896F4D654C6C914F875B495A70554F593F98905468
508B6E504C33509D70495F921F2B6B84443F9F2240565080AF3156985E607436406B8A52
6B8D8C8EA4A0413995AC8127447B396AA592467B7153746E1435533A9B5130446A327444
2A677957619C2D3A6936287F8150455A6335256F779B2C4796408A76282E6A6037827143
3222305E396488511F776137667D375264383752412045764D241C435B2268A69E5B4973
1729597075337A2D24445D6569734D3A6E723C7D96572A24426E613258B76941
6D43685C537532645A51273A1F19382E571B1241452F5C1A1D2C306B5D3A1A4A44716329
45A43B3F6151284A2F60316854284760385824508170633E38B6231E5E5C528A42271B66
77413184211E87603D461B547240668947285E634078CF353946646540BDBD413A357159
9FBBB13353696C4A467657535138375762444B2B3D633D722F202E40465246246D71486B
406D6C4F6564397561496296702F6F6732498C5C5E4E2F699A543A933A5A7E81408B4A2D
47736D64B1B27D42AEBB733358945D40626E40957927345A1D45314C8A844D5B532F7478
43A38E382B614165963D185E8E2E4782A074316F568C1C8CB0616B501F48A86F21318457
424C5E912F384A462B95945C585C275B6B6C435A54151B547A5E1F46540E3457845D5669
4431734D2C30935251636A776A65644B93723784B0452724416B462F89BF7042
6A4770425A9B6276816C2A4E643B382458492F4B3625633E3833215372571C3F133C5835
3DAD19214659252817553567561D1D433D533F475561523D3DA12E1E3D51558E504F1C79
743E3472411F7968394B2B4874466771622E56663F69D374605E5B6A4294B5764D418163
7FBDB1566C60775A5F8254767E6D486E5C595B5B5362317864393C4F2B5B62426B423785
53835C3C5E6D469488273B8E6C3782804433884D6978506265583998544956573FA08A55
599B1E1F6AA99650C2CC201B53866742856E185384433A4838616E388A5B2DA19030703E
1177A2742A6211387C5D1869961D2F505D7D4B7657833D42945A705728275E7740449B47
1B1F3F675F3D503B1D64806E4B721C464464405F4221305A521F204955335D5C745D677C
79456851211F55515E6F6F5A497F62489E623571A15A341D386E432CA3B0793D
5B486C4B3862516572291A244C3A301A385A313E37175A38362B113244541C4142535F4B
369233353F5925492261446560332845445A5B606E605A4E4A9B442945675B684D612F8E
7E51328F57217B6633592F4E80497A9B793D64712341A97392868A34253E84AF826D9F4C
508AA04C7C4655525D8E3144634E598D443A4D485C7E295C4E31547950445B29748D506D
318A964E5C32368F90504CAA33296F7946327B41415450675D341C6E6D54783E1E556F51
49977B8AAAB0955FBECA8739387F7A4A8780426D8C6236571933623978682D526922573E
296F8B5F2C704540633416678D3F5271557B4377747C4B3E8F4D7178603B8A7434469077
4F273C7F663A4D5E31748176486C40684A613068513A35686629266B8452586C733A3A63
7042593B2329805459556E2618677645A371546AA66039214370611C4F91632E
554366644389516D8D7B333B553D4C3E445A323F3F2169293D3523415B44205E5C625D4B
36A046353E603464335C375B98492F3E245058527B50353132855B2B4E502F6D6D6A47AE
9633287D602B97622C583557632A417B553B7DA5594ED2483F495B6D5A3581D5977D6E70
84D0CE404C7065452F8663655D1B315B7249511932785375461E244A5F515B1F5B8A526F
3F5676475F3F2368774B538C3C2046803F2B5B8B626980755A653A877D56697D3C888264
448B7EADBAB2915AB9D6913F3A7F5154918D4D6C77622E63234554336D8E576565245D4F
4B8AAB6D3E925B497A301A6A994C506E446B34445F7841317C347E6D4F397A6929397E4C
36283F814B2E4B88487F937B50AF65764E5625767956284D6E26265A7E6949847A40475E
5A3D704E3435666848508C642B544A2C7880687EC57452161D52612990C45823
584C544A4086393641472950412137393C2012203A1D5F1A1C2F295DA7421B4D45636D25
1A575438615C22502E4E1F304C302A5447564065A87667493B98501C5B60434F45322C95
914F1F783822626C2C53192E6540587B5C2548545563D36B56585F6A55276DBBE0E27C93
77CBCB6F568F5A63569668596B763E768C4D515947773D5B6D4A3C54594D4A2E56945B87
63606A406B7E4C7F7C3A53736343549E472D527D68667E8265854A85844D695341817C61
4C918D84C1CA572F85C69740356F393975874D627259355C233F623D7E925A73702D6D73
5D8EAD4246A067477A4127A1C17E516E426D313E7B934F30742E72714E36797636618F46
3F25347630356E79477587503A81607D716C2E4C5338224E725221486632438F8D665786
872F415462425D7852376F763D656B41867B23296F7A52191F54291A7DA47B3B
534C5B4335554B555A3416314F41361C31472C2A320F45382A310A1A4259244A1D294A4C
1A644516374D3E46183D3C526921142A56443F2D545D5943329A571E344B3D583F532D80
7A5C21705D1E678232442E3C8A304756743F617E615FD5887170517F502653BCD9E38294
55C4CF6F4F96535856A5665569763E70884E52534970345A6A3E41525B57553F5A944D7B
694A8754838C4C8496485B858954549A493E7F856B664E86B9893F778749996144786729
2D71836DB8C47C4DA5D191532C6F8B486B9052768C33254E21355246859D577875255061
5689A56E36825F4C9645196EA757496F6B7F4A27577E3F40854E6B5D523D7C6F276D9D74
513247845035425E266D8A693D733C5243653F765F3821375C3F2A719855506D65162357
8846517A37299DB1774276492B5D9C4B9F7F51485E876916234265234286714D
515151783A725C557D6B3138474329402B62372A4929755122251C204761222B4A3F5A5F
184753233A563E3B4046555060473236603C6F5287736F3F3298602D463B3C5B3D7345C0
9078274C6E245C8D38454239791E447979455A754B44D48E6971534E2C22369DD0E3BA91
38A2CD7D4492517D53A969647D7C42698E4F6C6147783E62794B444D58416D694D9F4583
65398B3965814A83B2412A69804061985023734E557C5344617D446F7B296A6C44779671
3D9471448FBC9357ADD46B364582804D7B9B2B52755B4358363E5E32499F5D86853D4D6E
4777956A459D48418657269ACE69505D51744D437AA04924734690AA3E2D596E3475AF79
382127653A3C848D3F64805747AC7B6546631B5372592744692022588F7C4D7B76393D57
74364A7E1815507A6A39852610377F40988B7F4E5374501F325B7C3151AC8165
515C4E823B776D4C7D8B4747554A254A356A3E2C4A2E5B4513211E1E556427344B435956
1A366D2F4C57452C45495B49544C3C3A6B376C5C7F7E7243338F503245573249767C3DC9
99582A3F732A487134484E37781B4AA589485C66382CB97972744A8E6337256EB0DCC68A
386688643C843B764C994B3A6C733D588A426965427B4F457E5945516840475D41A76B82
583A8861585A3E6F9F554B506E2F427A5C35A17C4755666F648B2269974D726A27537367
52A7984B8AC4612981C9A35B2D56525A899268425E261026192D4D5798B0384B553E7D82
5548A15A4AB7843F7F1E1575A286655F5872190F1C827A2870163F665835606F2249966C
59262561451C2D61405A8C37297C76605759152F4A602F455A21173763584C897B90555D
6C356A944936709F574EA076414A6F309CA8785242903F1B2E46652B83BD892A
52515B9041736B457D944B485B51314F336735284F295D3917391F1D47572449573B4B51
183D7A314C575C354B474E4275693237493C755B74894421257066313E5727436A7537BA
B32C1939612D437C25515936732B2E6A69305E767D3ECD6E43775F8F534A3D78B9D28097
53A1C17E39905B77518E7C4656793A5997596C59407E66417335445C7B50553C3B86618C
54325E4881805A5CBC514F6E865B538D3B33898E546D6648B7B84B46803A778A4B6D6524
1D5589699AD0662B6AC3A466395B2B324E676B41586F2A3E3222482146986C51482A3481
6E49C55D1F888B39833C1872676062535A6059292B6F672D6B6F72886D3655846A62B68A
772A2F5D865A4A7E374D9066466769495C653A635250304A79432E437C4A53BDAE93526D
6E24378F3624818A492B6C64425167336C83784E46937A1C193356264B7D8357
50403F7D4379382A6265393E532035572E4C09124628543D1340371E44561F52582F4A46
0F46803D4C67462F38432C1C4E4C1F36432C4034698252372285421738575553554E208B
934F2B3A3F1C36843C493B2D593D3969744B42568242CC94476D3C8A61443567B2803067
4398C6802E88607E326C7956774B26467D5083362A54464C8D553D4F5238797547A22171
823863183D806756BE492D56736557A04123A07758917E403E7C663A85213F6853567977
406C926A94D29C6A96DEA3746C9F8B5D6A85627A867E404D3724643A56AA7E564C564A9D
754880624091763EA55D2195697964609C8D5B40428A7446998D848C6636598C6D5BB182
7B2E3D658758507E4A60B1905C6F7554957D586A684A21417764333F8E3F347173805386
905A448C3D378984432C79392D678A4C85B3743361A782334E6D8A4644878A55
4C55486A376C4B29584A19345A3327211B572D1B2E11404F1741151D404E2F5034113252
1E20641A2F4A45231E2F503E5D462123534D6A3A4C645729207C4B2A24575B65A5802F67
B659346D56332F7439414F30572E3A81885E40575D41C1A84C7B352A1A212D6F713E3460
3377B7A53779348B4E52492B546B30456F20455D3E5D4D2E675A47594D324C5A3DA96376
734370566A686262CB6C3A518436599E732B81544C616E575E885651936A5C76515CA287
417AA14159BA843BA9DF9A3F35597267999D57404F5743453B4A8D4053A6747D8950427C
2C2D64623B8C4C3E9B511F9E5D7A2F3956744B4F5A8B6127616491AF4B2B6189644CBF65
2B315B7D7855547D2342897C529B9F3D615C2555786B37385E3E2A65C3896B9EA350355F
874D7EA480648E4F5A3CBD83504370297A9576405CA45D111D3F76373B736825
4E494F8A4465613E87852F304541586428522B2C563B5166154D47213F4417586440324A
29429D373748422B3D2B41437D80272B312D633F504F5B2F1B4357382643192B617A403F
C7311C4C724A2A702A327E38631818456E58395A43288C9657842F2A235481C4753F305A
255F948C35776179514780454F753D518359574E407376407D3E4C627F53644A2C8B7A84
57386D5F7C6C36389E6A3B578B514485742FA27269778C688EA249338C65597E443B6C78
3B7BB48192D27A4E81C68D6D315D6D4C759B7E4C5243355E3924544665A35C445032698B
3E35495C3D956B43974535B7AD976B4C876E323142684E2C694155845A2C4F8E6A4DAB7A
5E2E3F50632659793D44BA6739588653A18C5D4F4E7731509B60332A664A388697875E87
8639458D8A5EBD37402B7478484A77526E8C7F457782821F23277752336F8868
4D523E79426C4D25354A38303F2623582F502A0E383D43570B344E254148352A48463C4B
15155C37384A3F1B342040354355262D505076474A4D715A35425439213D5F3C6879472C
B05F3146604E286A5E3D5E335F5C2E5D6C6651879847B9A4609E45371F3057AA8936496B
5D8DBB9834837187454B844A565C293E6A63594C33466C3C79683A4969515A7146A18788
93468B5E7C748D41B473325BA37E57925D3DA37C5179895471ACA4399F6F6E8061477B3C
28597B7594D17E618CD8806A5FA2B08477AE6E72947A48374E305B355296905D6E4A3684
6150686740A28049BA4A2F874C7D3948938C5B3E48784E308B716371453680BA43539C63
433E6C7D824B4A771947A8A0576371385D6D6A4B4C76332D4D5C3C2B706F5A8F77927A9E
81733FA78C68A2375831808B5448876C8B9D9251867D86282527795E2D77A388
526C4487468685484563592F5B633F71346B5D1A3E46416F2548562A4450533959493454
361F81352D4F553B3E225E5C6758283864556F42527D7E673E3D682A3B575C3B5E552A46
9B6E3B4E674C338C743D593A78433A5A582B2C4B7347BEA748747B80672761916C324C64
5F7ACAA437725D9A6040804D5F7D573E71646459515764368B814645522D52643EA04972
9B3C9046629D8B43A66B358CBA835AA6662E775E529FBA7554828E44AC41506E7047A678
48619E3B3FC19E5489DF725125478D6F6D9F5C3F4C5D493433527F4F3E98466E8E613396
2323606A347F5426795323A54C6F50416D485F3F537F662C4F72747556203C6A603D9A6A
3A1E383E694B4A733732A987565947388A78794E486A453E596E48256B7572B9789177B3
A4783CAC894E753B633D7D894E4C9173878B7841ACA28D29242A6423409A9D8A
4B5B4376364D6045474C212C695145501F5C4119221035541F3F311D4C5D4C2F3E142C61
361E9B3A1A3F563A3A2F5A5365531D305A4B653136585D49404C74461C2D52386E5F361F
5466383259371B4C6A3459324F3C1F6767663950562A8CA258684B6D5A428E956B324157
3B49AFA34268446774424F326A805840733D4A684F546B34716F4359785D615139A68F72
8F4C974E4E79745AC09E3A4C9F3F519C7B31765E3452605351755C41A97C5D742F237687
3D626E6F8FD3855586D58F88354F906E60A68A5C5366512C54267F584196896956553276
323F52553676623E894523A54C674C4084475233547B5D2E4759656A582F507960349372
4E274F47754B4569673CBA8566564B48A8736E3F4B624F375947332265706DA5745B2769
A3742FB7642866A2883D707B4E5B9D7281834D178D8F7D24272F5E3E1E58766F
536544A24A6D7F4262765326414E4C6639593D1C302E325621402F1E374E233266423457
22289E6D2E4061365E2D5A4C7EA33C346246876941705536262562503160351E57706529
4E81462B757224577A327F4B5F462262766B3F789B479BA65286A28D452B6A9D6D4E8C98
6479C59E336D7C626347834D577A3C5D8E71464A245585496B39283D6D5C534D22838F5B
7035736451763530787F384F9B6B477F87357966626B8C7544638B63A76164565B3B9569
3D596C58AAD475569ADE697B374D93766EB892615567532B5B31876849A29D708173337C
51594E733C7F7A44875C33A993A6774E864953336183552D3C4D527248335B7153247962
2B2A4F48612736625E45B27B5254694BAC6C5E35748763374C4A2C419CA0729E7872577F
7F412BA76117376A6E3675761E2F7D677FA67B4B9CA26832454E6E541D5BA639
534E416D40767A2735484A2F4B282B5432661D0B2E3D357A143B42324C7F3D1E49364370
36156C6829585622573251273E5D2D377D3B67464E8C756649335425182E4C354D553532
678D3122474729707D345F35625D29556F3D2340694185953A4B42696A33539C4B2C434F
5D7FC7AB4F63614C7145824A50755A416C723F4945486A4A556E3C3D5E58475149A19654
8159935B4C8473429F903449977F5372784FA7A46A62826A3B698F759B52595C67438D57
4065A682A2D3871970B0B19F3A4D692E7AC1A26252685252592F7F6F68B4A06D947C3A8D
5C5C557831969D42725F397D82AC8A53914B4D141B51933D346147B07537697E54255B5F
29294F425626417E5D42AB5B3D91B34E9E7D331848765A3D62642125616448869E9B7C85
89333697784935504F248294504A7019486F8F50A0CC8C201D24603A2C93B958
4B5943773346663B43361B1C334D35391B4A4B191E181A5C2739320F36554E1B4020235E
4B177054142D52303B235A4B51702D22533E5F4636616B644A343E3A1F2C31343C4C4620
2C69381C36501C487C3D553F486B2F4A576C3C487B437192517453737B44789B88505564
6A73C7A1566977436C4AA15B51746D61776F444942507247576C423C5152535C418A8C4C
8B5D8D6F52817F4992963A5BB3A44E7E613371914F5F85465BB2A8649164738D874C8657
2953AB9182D4AD567CD065834456815553939E61525A17253E1C6D4564ABB054A23E3C99
6C5A5471208A94406D6C2C7E3C7A5F52BD88763A4662833F355C35707A336A7D2E298E83
663F514353122E52873EB37E29668C4B859D6D2F4761191C3B59511F564F35534F77477B
B46F1B716A40469C71335B66385DA4694970661774B282272C25562715559C6B
576742AB515792424348712F4F65303D425E4E1F202129561E351D17324B461D3B2F355E
4A18615D23384C2A3F28674C4477413957326C5E52776D6B50343D4223302F3B32544D2A
2D6E2D29566221427431544546562F3B53643C47704280AA708A586E312A509477415B5C
5459AFBA6B5888527946875E596765596565526C4F5066357696424A4F47446951697A39
985E6C733392A5508A8B1B3790885085935278893A80A66C26547D48834556978E4B8E7C
4E47B5433EC7AB5782D1561E31749F7262A3836298804C2E5E1C345B3B849756BE53319C
7458748B43738C3893933A9450895329585F673E5B8487214D8256915523637555315087
462C59646F55404A33276E7B5D66782859696127657C622B4B51373187AE72794A634845
846E2073991F2FA193456270183277694E78564337B4621F342D577A1942AF7B
5D6043AF5A4E82438EA07F384D5B3954465A4C1F322B325B1E3D31234152482A633C3441
3D2163753236532D3C33654C4DA4623F5938877657807E645853626A282F4C453B8E6F36
267D2020417126396C2F78603E6C2950898342465E1C598E5E7D8C85774C58A56C4D8D87
6D5076935E4A7641854573317D8C6F4B57564177554A6C4C605F45576B63544721608B4A
6529667B3A653F2E7496533A92372A5675709DA12D4C53632C61674292926078321E467F
604AB03A65CF9F4687CE74752B3F75646EB08A56494B3F434527576F41A7473EB07C3BA4
31143B6F4076852752673395A1A7845C8A4D4D234A6992412B7934907B34466D2317447B
5A2C302C4C3661897040886B3880B3626E7457123466682D494E2A1843696B884E83854E
AC633485923B1D66633E7B603647716C5CA56A53319A7D214F5C79721C3DBF61
5853387A504A74322E576D2A48321A57414F13093B442D7D14345E3C3C5D3A184759403B
242673903E3158204635441D2A665731572F66644F80681A2E49627128485725204B5729
37874924374D264A7E2347533B6C2A36575F2F59885559AE24324567703D59BC5625323C
5F7CCDC836397A514C26654C475C39446D6750761D346860655B1D28405253592E427437
813F4E7730755C3E779A504F9E8645577058668F656D735E378298524E7975996D4C4884
655EBE769FDE942570C08D9038558E2E5CA5A388836D38546A1A234769BC794997573BAB
58576C7C1F789D4D67601D5D4F676653764A5D273B4A6E2D296424565A3C5D6E46252360
563C34344E0F265A7F4E907E24517664615C733A4F6466324B5B6825465E6D7A7A9A7042
B3481F45774A297F5C288291594A6B3A4472685D2E8D711E20254F661C3CB865
4D6B3D8B425999454A407C304B6E3F4C334B35191F1D165B2A2D400D306B512A39292D55
421A395E20406B3535205B403373251F514A71322C73715F3F334A1A14344D4C2750420E
1E4D442945291026663E4B361A5242324E4F24285F5A54B244545D5E48326EB68C53503F
5476BCAF4A367751672C56455A6F4730405245873D3F6F45586E4942463741643B4C6937
8C5A4C8F356D8350849A213B77A34A62864E5670487E7F571D48755E90685C9F9C584478
2347A88677E1A22D5E9274805F89B9756B87997FC0832D376F2525474F8391598860309E
7967888E21568E3C5D7A34775A6E4E4D906965355A5B7C28315A39623A37597365352765
62483834656D4A48783E97786564726C577F8C5E595C512C587C7A213F603B4B458E8D47
B6742B4997552D7C624A7C793F4B697A469761552A976D272D2F62832838B088
5C6C408447354E395447311F36583F39132243281F1B0F3241324B1019314A262F1E1930
51214C750E12393D2D123F5340803017353A60472A4E5E5B474F5B44282D4D5632503B1E
1A5D3823555D252C693452572866402C5366473E623B45AB73555B69311B4F9B834E5742
693E8DB1683F7D47A34F83224A7A7A40534E1B89755079314184554C4943355E56467C4E
8470469B505173457E9F47245B6E5C99A0656B6F4594B77F356071707E79455E89534F8B
613F963B35CFAB4C7BAC59363A69976F6C919A3DB1793A426535386C30818F666A6D2D92
64508B8A474B8E2E7A7F3E78626B61325A65704865619E1F49834C6F4121527C663D2670
3E2539597270524A531A94A965597E3A5C9B8F5B50546030486A79283D77886D4A838B4C
C3802E4CAB582A9972557F80594B796E4D9F4D2F409179362B3564673535A691
6E89459A6B4071395F7B5B2B2D4A3448553E72342549294A40335E4027495E31354A5037
4B14476E3126482D37314E3F3E90733439355B76436B7A201D4D746734325B462E6C713A
1F613A1C6482342D612B4B681D6E2E2B6D7B4635592A2B895F54667F816055A773527B6F
664954866B45823B7D48833D33556B4A7474447B6A55896A407F546E84754E6544579960
7A4D4D985E625A387DA1722B594D35567775656B496D6062314C63495F975E72552F368A
7B4AA98368D2B65E91A89796384FA38563848F55B27F41494F2F566A308F5232657D3B9A
603C6690604DA4486561376B706874595A58925B7766A4492D82536745354B7769442662
5243252C5C71514862275C825C557737436E795F5C548747505E66315A88775B538B4637
8F6E3D53AB592CA2925070834E42896F44A65644287C783959696E3B212F9382
606437615F395A211F435F3133330D3D5E384B1F375B33452D34876B28495E0C3679513B
4F185C97482D45152A4243121D596A36393746713A75752D1D2A3F4D2C3F7D2E17466748
257C4F15365B373F8B3E3D6227673E17396B41457E743A9D49384C5D7B5B66B95B363B3F
6B82A3C46A3A88536730625C4A684030495F508C483171694567363E4451475842477E5B
6E563E7E566361475C9F7527477A49698A6E657371948D913A4F5C7241A45D5D9E6D5995
8B4F9F9877ADAA6594B1A2A1415D9F805B849F5791513744703531652B723B4D427F3A8E
9070A7B16C53B85B747B4064857381636C55A064735BA14C2A7B6462473D527D66381F69
614A2B3566865747693061B688507E64507B7E556B75834555653924386A604A6F98763C
A457325890521C71563E838E5A3A885343B58F632C8179331D2B54672F167B5E
5379416C67438752436E7B3A2E59382645314D1D144927412C1E654C213A693825533F43
5B2A2D62332C5C3C3832514A2F5B411C3E524E592154585A41425C31182951593B464C18
144E491B314317236246404B214F59284A6A2A254C743B9F654D5A526A7082AD87674C46
8697CBC788418A5B68608B674D5D8E4955766891844980794C5D63534E66484F4A4D7569
66523F93455F6E5765B08123407E4C608A7183766091A399454A758452AE69619D70648E
5F54A7966F95AC4B5C86998A3F5694405C9597599D5E4340783A37632F75515C3D663689
634AA08A546BBB527478316996A18058795D943A6A72A25F25845C746F49537A6E48256E
72632D35678464696D2E66A8725AB052479C852E464B5C33637B7A3437824C4155897446
AE7C2E3876602B9C711B5F81613E9048317D714B4EB57F48212B6868434C8B82
636D44634F2F5947535B5A29376D4C2C2F235432182B0B2E4E255D1D112F544425391123
5631316A1E153C54361A3A5D3C66462A274E436C3D625D714C43605D3B2D57725D444734
1B5D43192D4027306F40435C285C511F4063543054713A9962555F6257344B8E76565347
808DD2D18D47845C716291794862955452645398874A7876554F6051515A425938527E63
7E5C3E9B5981856468B07138587F4A587E5894856A8BCC8D547EA79049855B8AA4725D8E
3D407E715A80A56E6A766058485D926A51886B529C6D2A3C7A39385C40864D4C364E3798
644693A4263F93406A6A2746725852436F628D405B558F432572353D5036557347314876
735334406B40384966476BB7533983563C8884616E867A3344593E305695894657844C2D
8172273B725E359D9B5D687A6D4C8D5F39826E41297263461F2C5E5236247C80
7B924F507335655C6A537F4636595B2542255B3618301E3240264D342D28564B26403934
513434715F2A416E3B3B407144564F3A2D4D3C80457D758350435D68452A55736747504F
1F595C27304F302F6E3D4B55335252273F625530564C2E74815666668B7760B776575A4A
56427AA18247814D7658704C5F8F88495259297D834A80503F53624B524F35535F53A83A
655C479032486B5A61C473233E753B6B814F767A4584CD83353658475AB652508B6D7EA6
81519A4F31619E814C716C4342688A6B60A475488649333050343E4E32546C4A40672764
5937A28F574EA9486963314486534139868278467452AB23458D53495820547B56452851
77202F496278575268194C788050A27040795B5663517B3D4B5D5C323170855279627D4E
9A83345A9380367B5C586C6D5F367E6944959E673585723E3B4D6B5637268881
8081555480376A647771A1674A606922472A583F193B39414E205F483D304C4D295C4E30
4736348E73283D6D343E39704961953E3559449A447F7F653F506F6E522952503B60925A
1F5458234867322C6437526234464C2644675A2E6B7C348E803A7592876F65CF6A466571
765CCDDB8C4996576A40556A465763477289648B75478B844C474947696E534E483E9066
65593C8375715A324BA398293C75484E8A407274506ACF86573D6B7A578E59717F455DA3
6B4E9E83884BAD8048808789495399867CC3A843786F453B563569963C707762556B3D78
744AB1AE7953C1536D7E4F4AAB6D744D665F86468761AE462155475A73394B594D253D79
6D422733556752848A2646744C3B90814C79728579538B425C8B743B2F603722455D734C
895219306A733896574F86AC91499A7C3A7E8B6736855D2F16205872352D8E65
687A3F4760356439394A714E3F5C393355264A281F4C3744591F6F4736456C361B473C32
481C1F5A562A45461F3835392038553340532C6131849150393342543B3969675F3B5345
23716E282B4E373A794D4659363D5C2F3E6E442A5474398AA1445A5C6C6A6DD585633F39
809BBFD59B38756875654979515D79485165578B883F78854F4C6E4447705E5063458F6A
5D844484856B585E46BD8A32486737467E449F854D42C77F5D65889659804B7C8C7759AE
8563B699753EB96D4E708F884A649E444D947B4A8F814D566934325938736D3064614181
6E2877723B499F504E703A6088878D65706A8D427B79A26B26624B415A4A4C59715C2D46
865D2837545C444C6C3D41757455994A308C777884576D204465512F2C55695760427258
795A332E6A7C3C8E563D587E7240935A2C6769463B8449491926688343299089
68655D476D2E5761585D8F5945855E232D25593614260D2B6220572F1926475020351925
564D1C5A4222476A3B243968463C622B284637774461647052566E693A2445706E3F633D
173F5A242E4D20275D5B534F203E5B343D5359256782468AB66B6B6A643355AA845B4A3C
8C93C6CE90437F7484734B8250638B5F5C6D80879F4A797B4F566B464F6C61707440925A
7568428139586F5A46BE5C2B3E6D364B803872814A37B1844E3C75844C8164AA9C7853A3
5748846D4540A2945487A96C417E9E8273C375338A5C312D50373E62388472405C3E2A4F
51279C9B593D9141566C3040864571615B6D85637B5C9E6B3255574A695F414F716B3843
9E662B354C896360836B4AB19D6B965434837D70745A895B415C4C33265D88727953845A
8E883C346B673594565E6C90753D84473B67786458B2664616246A4D314E9E90
8A72714E983940565093A0372D4D632C2F1C5549172C1D2D611D58281D2A495A24332E2B
465719476B2A286A33433679553D834B33574478516F65503F3646604F23444F39509056
23415728557534215D4955734E316939485F7B2D69572F6F95536D74826D63D16A386E80
984794BA893A723F7E734A66508689515869576F924A765C45515D456C743B4D5D47AA5A
5D42398A6F56524862C28B343470315A8A43837E3F48A48F6E37567549895665815B5CAF
8C539A62334A949165898F7A49628F7F73C571378C57303B503A7695345E764577533D3F
501FAFA96236844273763A3D8A497A4E6E8399647F5C9149468947486749597F67633445
893433576C7B53515D3F37A6875A9082647578846D638B59477C724331768D698B8D5338
8E8A3D3C72743BA4B066638E6C37877D4B68AC67417E524C272D5D5136267581
7968473F70373E5239466C4F2E51454265284D411D412821541A60352F2D41442756362B
3E371B617F311B662D5237604A50AA552B3F216250656D6A63404C734A1E416B7147656B
264268362C5B4B205F50395B5631662E40756C2C7C6B387E9D335D6E655F69D86C4E3038
7850BDC38C429B6A5B665A904B586A46657C5F6977387A864E544230526351534E357E67
5D4434776B65584F3A9E8732357B3D3D6F3F7E6B5F4463656339583D44836A7875313699
7E509D876A59AE574686997F4A71925866B5843E886E333E4F3730663765782B5A614750
69297F995F4198484466454F9E8F9D594F5E77687E6A91432A45453B5C3432436F552C49
721A1E3040673147593C37B9864785967579774B508285524674562F2D4F66498999845C
99773A46736B357A915265A75C277A8D4A447E8693BB6B3D1D2D5F593E2A8472
696F7D3F793D3B7C8279A6762C4D45223F22431B1A413E2D5F144B40224E6A5E233C3B25
485916336B2C164E253D2A44322F5B3B31533A643F75A55D5D484E52301D67797B47704E
1E558146314A2B2D756B4B5A403C7C343A614D288580386AAD3E5F544D6565D76F654842
776FCEC96F235B5C4D4C3C5C475D73414A564D4C94436C8D4F5466464D634B556646A67A
676752858C6A5F6B45AA6836347037467F406E58613E58766E36526444736775735740A2
684B9A8F6D609E8F838C9D855766969163C49B3E748F3F55683D3474416A7D3F55583339
683EBCBE583B745748773D2D67377E6C5E7E95607799A86A3642473C72463651823C2F5F
6426284249591D61814D299BAD376E82476D735C545A7C374F827C4628623B3A73A3492A
7F5D414E786C1C66826B4681461F71915C51B48162936155383E706335349587
70556E3A5B2D294F6B6D9459387480474B34656725350E1E662543402225315528492229
60621B3060231D63382D366F553E6B1F1C3933622E36526052424F661B17415A54387345
1D3B5F272645261A454B4159421D64283850562363893552C3676E595D566BCF8C6C4D47
7A61C1B3483579625A70597748598F6563726F46A04F6E8D4F56824F53755F587B4F9379
80624E818C67525A41AB5A3639753E456B388794724C387F703B585D498873858C763E9B
704E8A8D6677B38F848EAA8F5974A99060C687447D9C3A2E503A2D6E3C5C6E42525E433A
6E4593B4834788584E995430753B7548598196666B6C7E4D355B452E56363669936E454D
5C1728545B622F3F5D492485B241617046616D4F56838F533F60562B3150514A6FAA8554
8C5F33375D71367282765498542C5881312A84997DC7763F192A5E5728235E6E
995D923A735F2F703D5B7C351E384F372E154B5C233134256E2B384845383D6A2B3B3339
53641B28592015492F2421554E396F4E2F403A5F68526776683F596D592F3F776B405D3F
264162252745271D47533A5767277F3338596C326482383FC06784736D5D58C663615748
8149ABCA713C7A574C735C7945738A675C77714C994C667948616A4F69834A5B64379D50
624C418270556F5652BB81343D7030507842697B5842419152456A524B975F687A58469C
8B55917F4D5F8C7A617DB66B365F867859BB7331627F33324D3B49924A6476365A504435
583597A768408C543E6F5D2F763C7B6B4470977877809275453D5D4866622B4E8D815155
6D42252E43682B4F6C623184C6494D89686D6F6149557B5A446B8A551D433F2257A88151
8D6846385B89405B894F3573542B5D7A5234AA916FCB6E5D1D215B5130277C74
8F5493387675378E426B966F33476538433451562629222566253343343D405B2E3B4552
52561929523D17503C3C375F5A48906B3C4848577450645D3D345F705229455D3D3C6B5F
3737763E356257204462406E7A28742B35737B3467843631A44A6E8066668AD459466475
915FB5B981347C604655507B3448614561865E376F3E688A4E4C4E355B67495361356874
7A494B6D8154564738885F35306C2F406E5175797B49226D733453564380777881603689
7F538A75667FB971515BB4674958977B51BD8D4B7D9C432F4B403070475A7B2F405D4733
59469EA06242835C406F523E9567875A3766725D648E8C83473554285C6E33518A4E3B4A
676F2E354868243C4F673F589D423E704867746F5C6A835D5D8F9655223E392B41BD8260
7C6D3E4B7B98434C8C683F90713857715442988668CA6F5E2127533E312E7187
7D546E3A614E3D9433488359354F4D27593147392338201C63203041294348531B443E3C
634E163E653F1E42263A2E4B2E32554A3450443A5F42805B50365362261F506651354353
2E4A79311C3A4D275C74324B542D763C2F59632962814C25895B6768556692DD89624947
7059CEC67531707C455753773B546B60487071368F4D5785544D5C484C6C4B4D64436E7A
7B6D72758B5D555E3C8C73362D80404771646E7B7B52244B5E355764408B7A758D793888
7D558F7D7784C28A684ACA7F6259958B52DBA84F7CA75A35714A2A7A556B71313D553941
504C9DB04E3B8C753F803D246B2D5C593F7974626287776836586F3F6C543550868A6457
63553A3D417F4C4452824457B3683D8F677A916A656E733A37558D5A213B373636C09268
6E89563D5A7B373D8E7C42915B33528B8B546F7C5BCDAA6E272C5D5D5028767E
984C7F406A7B4BA447547C5C2F3B472A442544401B2B231C7034253B1936445D2F402B21
47491E26502D223C3C221B4F4C415E34283C4F3956406550593652472C2537614C295051
312C602E1F3A331C355728504F1A5F3231547634588453217D6F6D53564190D68F64544C
807ACFC36C3C737D525B5885455676664E767B36865A5C76554E59474F806850774E6B71
8279737977586A543E946C32438942526E4F79AC75482F4353305C5B3D91665A7F6F3691
5A559A777184C67E4B48D79868609B6141736C4195B04D1F3E3C376B41696F31446B2536
4B3A8B8F2D2D8156428D4527722D5A4152928A766F95817A3E376F477059395F7E79545E
586340586586513D42631B5FA86B3B865563707964736E633A5A8D541F3E5D483BB89562
5C915941628A55338E7B4C87653B629C9D5BB5785BC2B173282A6E593A398A82
B74E973C65A46EA5475D8D87403C4F323E384260292C261E65301D3F32343666313E3B3E
434F2220413E2539492823595F495C4B38445E43694A725D68335D6848363C6E3E2F6C53
282A6D363250291C355A255057216625315C772D518E5F2A75815E72715A88D581516C86
9A5BC6C98B447364617F657F46617A5D4D7E5E3D8E595F89485364424D764E5179448E4E
5F655A7A7053513548A97F3A2B7028656E62556B7148385A513D656C429A80677E643E93
7F578B5966A9C97B3B45B8A66282C26B4297833C7989432043434F824856822250773929
45217C884C3A7A49578B4D21772F5529427A666B5F9C877D3E36664B6C66414E727F5E52
595F2B31406F3C2A3B5E28528B54406A416B79786A717E6B394E88531F3E593837AC7D56
758D553E5E7F4A37B190507F5B2A78B38C66C4633AB0915E2A2F633311377B7C
9B4C77375E8688B43C6F9C8A3B3F56354647416431272F1F66321A463B3E3C60133F4F3E
354E1F2D564B2D35472F2E4F4D5571673E434742873E84704C25465B3F27456967374B51
21307B511B374E2A31592461662066382958722E498F4922566A505F547185D989523E4F
7F54ADBD9840726D4A5F5D955751593757998C457D40678D594F57336A895A4A60319F5D
5958457B8C585342348E773D2D843A3E6D6359737A5B55696E2A564A388178676B713081
83517C686271AB774D2B9FA24560A07A479E8F649696642441502D7252508A2E45813F2E
4B4C7D8C423D74693E6840236E325640496A4F5A5CAD748030364F346968564A8290734E
4B65403642775729427A345E947C516B776D7D855E828C7437478C52214E5C3B34947717
4E996C3D596E332679794A76511455928174C58045AF9D5E2733544237296F88
974B713F575769A943515D4D3564632045443D4023304B2484471035233F617039273425
4F4E241A56473B4A372E2D594A47514C4766653D6230727167364054342B376765324B64
38207457203046274C693E50461D6A5D2D516B2854886D2661745F674C5B5AD68D613E50
6658C7D4AA38786B4B4F498266565549436B7B3E7442507B554D5338496556405F3E6C5A
546A58677E5A5C513E947C38317F4553727956665C414C565E2A54543A80715C6D6E3D8C
875582727278B86F543EACB56562A48D56BEC16FA29A7925415A2B726450803C4A74342E
4F5A698C6D4B7B7D3F675337878495795284586452CEAC9A41354F40797F604D6E8F7044
46654C34426D54314B882D677F7C527059608A6C639C967131416C2827666C342F727A57
67A869396F836530AC7B4A68697777AE947CD5A24896AB5750536C75582D7C99
B94C89435B7771BD495F626B2F4167312F2949582D1F24166D49153D1D1E2F5F3B232B18
2D59331C4F3B273841221D4C534D5A353747653F553865676E3C3C4F4E42385E572A4451
291C6D45202D48232F4F294F4D205958275560323C76692A647A6F6C4F5547CC9D6A3C59
7E64A0D197415D714C5B56806C534F4B4C7F8D42795B567C62424F3D527C653F5E4D7560
4F7465667C63582F3079672C2F7A3A457B7562966A49475B45569D7F3B8181547B843567
625797906476C46741377FA66466A06E25668B529F8C5C326664255C48958F374E612C59
6750437C5D418C672F79453B77627F634E90754D1943659D6B3E4F186189554E636D434F
7F8D51374465303D70822D777B4F47855D5690634FABA070253F672B1E3E46213478805E
6AAE352A578F6C33B05D3C7C79958CB45D88D099409BAD4527406B7751398380
A74B8241568B71C64F574E854C3B533428313446251F261A5E411229232B2F643E202F2E
2A4C381D4059362950253440614C6044344B773F4A446F6C743F304E54473462693C4D4F
231C7751272C3F23395B3A515420525625535E343A7B6C3C769B7176654D3CC2C5893C60
884B8ADCBA5F58645397598E616A5D364269904EA26D6092534A7956568544437B466E45
61865C576B5180433688763F53994F4C854B3855734E4C5551213E673A8A63358F85327C
2528547C7466C286793955C3746AA18B45A0A86192AC741F3C3E1F50584C723B4E71352B
4D3C3B8A754A7066348D5027723A60353E888E78486C5E87395777314B5C46538963342B
51784149587F31213C66438F9B4C2B698259AF69386270452F527C4A1D425632265E9378
6CB981415688602BBD6F376065858AB6696DD56850929F4D29356546294DA087
B04E6A3B63826AC8576E5B8A57374B352C463A59302444296F531E31363838623E234534
304136224C6037285B2A3F3D655068632F4B7B3C6A4B676A7F4E2C50554B2F66795E6163
371C80702B34533C3F5A2D6C7021445D2F707D3A3B77262A85995F8266281E7C7A6A3655
6D32519D9A4F544534684E87263E5E4839606A488F605A8B342D5B4B5973362C593E633B
385953547651644F328885342E66464F98745D8B785E5363733048553D8A834A59742C6B
61406564776BC18E7B3C46BA757FBEA449A0AC507F8D6F294D5B2B66685981365586532B
593D337B714D7A6125524E2A643D5A5E4A946F7451A171715130584681684F3C59826935
5048273344705C2A4B702C757E6F36696B45897D5E8C7F68273C7C5A21494F2C28599448
419F60404F813C289E7E4381707395B76B60D06C4679A549526974472A338186
A852592B435B539B2D556B734B3138203B573951213F69396D561C415F3F416717255A43
2E38242E5F622B40313136372E275D7326405A2C6A3D5069543146745C49275F34225F7A
3D1B743E3D5E64473E5821666F253C452047683547785028577B5EA06B61408887642E52
63528ECCA63455664E5A458D4B4852412D5C9554967653835438574D6F94613C603C7558
4E66446A84626B423884804739904E42786454A06143657E7D374C4D428C89545F86316D
78435D5C5B6AB2886C3A3EB6395698803C99B07981998B2A4D692265644C8E314A6C562B
5E5C42897647647C376D5E2565445A5E4F816F8856AF8085632E564C656E683651957B3A
515D452B4277432960793F6C8D7A3E76785E965D5E967E7639436B5033686529377B9D6D
51A72A356BA67032AC692C8691427DB44387C78C437D9742253B595F37379A72
9651703F446552A65F4E58714B353A191B40314C1916421E65651833452D315A3517382E
2248471C3B3D1B4B49222929413550561C37683E6A473D736933283A4A3A35675C355E4D
241A61422E2A462F43622D4C55273E5C336560243E6E60436D803D81655D3E8DBD734168
705A96B5CB3F3F6F4D533F6559446A392F778E4E844C519964496338486B714264477E5C
4C7F595482687E4E2A795D2934945942877B5DB466554B5A5B33414931908A5264613179
6F425D75565BBC61414883CC796097833DA1B9787A9C9B334F741D5A704C893F536C5737
756E44767A336F803270692A69717B70485D5D773D726A785A3154317285633850814A4B
7285532E477E64304A81376E9451418056518E574F9F886F34607B451B453A0D235E9F65
5ABD4325396F4A57D2A43B44666388A1313B974D409F975C26365D4D417BB677
AE567942405C3E66565A697E464364442C54335428233919615B1A3538273D5C491C3321
2F5E4A23403E154A4E262731613E5F491B457042632F388C784A2C495E4F194A60457752
1B19695436324124466F32454B19325A2F6C5D37246677546BA263995B4E2E66956D2440
5F618CB4CB5642734B794F82654674402E588E57866343855E5E724E5C7B653F5E43784E
5594574F85467F553A747E2C469D5C428A5A43A37462455F5334445239708B4E6D6F356B
4832414B6268C5895331499F77608C9952C1D0746898A0417A701F4468629F3B516A542E
5C60586D7C4C67862E9671356863776A47A9806F4A9065923D4C795F505E2D456F8F6636
5C5C163D6A8D6439436622749778376C4C4E8F8D6B9487331B37685C1C4B4D27265A8E49
46B373353E7466349F834F6F706EA1B3574295462F718E502633544C21386D63
A34C6646487A437379506B783F3057402539254A232336156A681F403F1C2B47371B301D
204A372142551E4D5D253F265E39694D1F2359435C553465653C3B6E6D621B4E60538A65
3615696339366555387030444827315F2663724D206267537CA25EA0617A5659A35F314B
6E4661A8C65B486B427B4B9E403C7657376A803B7F503E834C46594A6798514875426E4B
4A835B4D9A727952418D7E40338D5737806F46C169614273564155633C6F834783853C74
795246684872C39B5F414C9F6276BF9E3393AB5D78CAAA365161375C51457430628B7330
773A26599353658B21758E3C5E737052264D7967417356763D296F725A665D3047836A48
726D35213B725F3748652A4F84682E7946367D7F5E9E8778323F756D1F556B362B678D61
58C375373A69732F8BAA576E6B4B7E8A4A4087633B8B91432337605E3D40BA83
B25D66383F70446A72508E9A613768512D6231612E26502758711C51503030602E284A2E
294F431F43572056652142256532848D2E3462365A64336772512D514B4E21555A3E5368
5617737A3E536D584692363B5135365329668258367E58317A90569895854C3BA83C3B77
856C65A0D13751A153581D6C75447434356F87598B272D9178536221518D8F4E6A24757A
4F746A599F677C341862765934863F42A08B3FC2765A718B5E393D4642838F4A615F3693
8B6061863A3E8C734A264C8D4D3B638A45A7C45A3F58873F605127586A41931E335F6A2E
645C5F639A5A5887315F5D39759894594292587A43A06473502E6363598467384576313B
5A7A4F294077554D8481366A99894373615B78715EA78D6E33476E5A237475392A599974
5CC09037366D76465ABB636D799489A2523A7A683F95914127386671363BC093
8A45744A404A39685F36455236328A463C44317D31344E2F99851F4E432C6681451A3120
4B591F255C532161681E322F541C6058255E7C4B52453483785731535C1C306675586A4E
34226E772E32481B40705E443D142E6D395C681F27386439608C405F45745531A959252A
404B6A94CE472F554251365951396051254973517A622F6F6557625741867D4C5D3E656B
4D635C5A976E7F5038735D3640A952357A6F57CB7B4030485737414B2346824867663188
793955616255BE8050373B878A48648A3390C17F5471814086652839613B7D30536B6F2A
51497272985954A64966642D496D595C429554732E60528654395A3645524F31417A5D37
5E954E2B46754D285776336A7D625490523F6B705CBEB07E3A4B633D1A4C604D29657C50
54BE783E4A8F86438B905F437C9879A14B62957A3A949A3C274A715D3D3E8398
935A65583F843C487A45494B301B364427361C5535243E11575D216E3F1926443E1B2C11
1D53481D4046133756211E1B54326D511C22443F40411F3E5855314D665017396E56704B
480F4D5F373672502F592B334032265827515547164E7B37528D6F9656765A39C188373E
56446B8ECA543863465C37653E35695328548561726931786577985C4F8F6C465E3C6E63
51735766955980563B82762D23765A388153347A755D3A5E434A5F7142437E3584854B84
74677C414759C1863A244B9B8E475F913E97C1895569884442502338734D96414B905F34
704D786C81314495405A6931578455432B887D85459E5B9B57365F8456665A3446876D3E
5D743E2640777142628B295772552F614D3B665A36927C7A344070601D4E563D3381754F
4BBE71312A697B3AAD74483582876484243B6D6C3C9B86462433535B3335B882
A86E6554489441498E46456B6B2B465125612B6238274F3354591D565E3F464F37182C21
2B564D1E3D5936375F2338205B446562403949433D76385363622C49606E213B7759753C
4E11577F4955735F365A253146302D5B284456591953733A5B886EA16067391D8A693139
623D5091CA62364E3368376339286A5F2C547637725F358449415B57458E46477438603F
557B5E629F576B42388E7043298C4B3B806938B47B624864422F3B55463588437B77516C
656BA33A4E54B299583C2C867D4D7FA74F95C76A578C8253594B26304D41724A359F4A21
5F67735172503D8B416788385688493E2370547A3C9A5C9E4442757F626941416C937344
5B79423556886A4A5F783253838E406F624C525A58B38D813B5671491C495128226B8970
45C291403D787938AB635B427F8B587F503C704431977A512330624D203EC698
8C645D3C3E723B4A7256828C6A29405229682B5E413257334C5221535B3A4A4731284B20
2D544F224A693E3363244328574872923A304A2C4F7F3C5D4D343960675C273A5C415864
62164363233F675C31572F3D633D2A5B31627C5D194E6040738F5E996E796221896B3E54
6440686ABA4E345443573A6660325D5E31608A55754F408967516D53489768555D305556
50624B3B83656C352C7E724D2D8F59347F8240AA814D528258463F5644367F428676516D
757CBB5D4667B4945F3F34846F3E4D884A8AC48E5E686F605C4A2D386C3F703029854F25
5F516358B76D39993C4C683F538E455832805A83459B599745275D6A626F46285680563B
4E7A3B23396569455F79256295843F6C4E2B686852C7B17C253D616A29546C462352813C
4AAB9441426771227F544F46724F70AE67407B4C38948E6E2E4274672C2FA28B
90755D60417D3D545B3343595522432E2A612856233761424E4B144354365659291E3C1E
3C5E1A256F6D3E395F274620371C4E663D2C4632346D2E64674F2B314C493446684B5434
4A1537703D367E632A49292A48392E59253357511F665A2C577D558D6F71671F84602B37
5B476D86BA57305A424F2B4F622D4E642F3F7F5E63684B875F4F6F59356C66656755613F
526D5E43956F724C40896E352D855A37806F3DB385654058493C3E544D3D904B856D517D
766ABE6D49579E9155373681904B52935881C998626A717062463258843F73332E8C6734
67453440995A3A7D4D57663F4E9F4E542F765B7B4E9A5198572763766674633060813B44
6D995A273B6A322E7F88324F6F6344854725534638906768294157352055753F2C636C3F
379065354E967E337766484D8470588E4076A5663A99AD36283765733521B47E
9372605441763F47833E595541336D5A25592E5E502B56355D702F4160253C514A192B15
1E53461D41582B4477323924624D586225324B42386E2950616431345A2F234664546937
3A1A447D3C2D6B372F4950414516355F384A5649163F6B3E64834D915F71632DAB87383E
57334FA3B6523067475B416972394F6F3F46855F6B7063AA5E426D67476E71646A5C4F54
59756F4C9B6592433288592C2C8052337B7536A5856E4F7042374058494189478971425F
5456B587585A7CA4413D74A5A6525490597DD4AC69989A6E91733435854477632E895531
7B7B625BA35056945E5353396EA48468326C6C6449B59A94622E564860915F306380513B
458B5A3F557B724150893B698D642E6268425C5759A968643A5F78581B3C4A152380875E
357F6F3231606C2C5E5F3F25777461771E397442369F6B58222C5D5F241FA598
8D56525B428C3E4B8F444E5E4C22465427362252532656443D6A313359393D4937192C17
2B585421406F4D2772343F19595E546032223E55368A3E605D763E316276293A734F572E
4C1F2E7A4E1D75572C3B2F2E3C2D345E33505E5A16537345779E708F756A461C695A3545
634149B5C0733C723383516969357A6B4046884F886A5CC85958826C44726B6B8C5E4551
519B773BA541595238868C22339B7A3A917740A5976C60915A3A5B8462358B3E95834B5A
4F39A6AA825C97C85E35377F785782AA5B48B675467260434653364A554C8B79318F4B25
5D5B5C65A57227705078903F3D9846612E727E83406C4F8E5A557C443F5D39439192713E
477A2136446B71404C8A195897893B7370307F8561AC91452A4262611D3C463622528250
306D7346446378363E534D2D8D8C588137458B4C38A5836227315C602A2BA097
A782523D4A813E3D5D49738373243F4F4968455352306453425C2B4C62443A4624294C22
2B5851253D6C552D712C361B5D6D505F53454969309A445F516B453365772F3E77515744
71322F8B631D8781384E4644623935612E536F52155B523270926B9173746E247D65384B
613D2E928C552B4E2352357074293F5C435A953C717965D3513658615F96644C5D42535E
4D755349937369452F978447267B5D3685822C84723D5D875A3C495256508C2B7A985186
7D4F9AB7444599B1854D3574895058817A38C496747E7475453D2F836A3B5F4235A06630
655A2448856724723A465F4943AA4770384F62723F7F51A54A28635F576A652F6B987341
416C392E3F6A773F507B3D4E776E295F5F30684A46AB83662F41566622375230264D724A
337C604B495964433A796756A8875D9D5549854544A19668262F5E4512306F8E
755F56553B60455D66402D5467294A391D4D376222236768435B171B453D354924133130
2B601B1D446C533190254C223C2D547D5837583C2D87466267503443766B3C656A313837
702E3293351D767C40704E4643333F7023437D6020687A3B6A8E54A6AA99702094663370
7F3F43C6A96A2E6C3C502D40713B4752314C705B704262BB674657394D596D51593B395C
59735A47836A603224816E28267970337F83398C9C6B537263474A4E46317E4E83743358
6F599999615587A29364397D99545B856C49C7AA71937C73463E2E477B4261433185552C
655946688863276E4C5E6B403EBD5C683966718945866F8D6B37646F556578346D9A8445
4082623343797A404E8346628A7131526C467A4C51BA8E603A445869244B6E3B2542692B
3B72834F4D595E1F5EBA7345AC43539954478A3A3BAC9774282D5B6D371C598F
725F494F3957273A6B4C405C5A3887682C4133604C204E4252723933391E2F4F38183725
33554B202C4C17265D1D23204E49454E25407458236125537161352C9B372544684A4E30
50224787621680332C626C49401F3B7F44437754195977496E8943524467661D69592E42
4C4044B9B15D2F5C2E4F3C54773744644E4F865D5B8385CC69444C67515B7D4E5B39474E
54655F5795715B402B84773E25828648A67D4086A9695172533D4B57593391517B725154
4F4A9292504F7F7C8E4C60929A565F864D43B6A16380734C526D3440804D8168437C6B3C
7C625F659B5B3C855E65773160989A8A436A70763483929C6635594B476D663C69917942
488F5E304177724972995D598E5C42858A498563379491652E43563028466A61233E6D4C
2B68924652897528338555269B79449323689D5E43AEAD332A47727D3A23608D
668B5264397F48416D4344542D21565C2D2026705A275A42356D31203A29233A2D15373E
1F6163292D565827612422174B5C5159312D3549207449584D604C2F9C61314077624832
452D2686711E605D3C5230423C43337F3B405E77286A7843728C777E545D412B80733656
57312D9FBF57366528513B5282404F7B56578B68768A77BC694B6974645B74597042566F
65897866A573826B3C836F393D95943D90724388A364486A413B7E9F5C2B794AAB875752
5238809F804E866B845837679D5C60868348C2986B6E78752B401E275533616C316E6522
677E796293562D734E4E7B4737994662496F778353684C7B632B543E418D5F3563843549
73894F2F456B41314A70465F924B225C72357F5D469D7A543E5F7D671F3B5B342A717D54
3B9584262C5D7D493B82471F5F774C8B413994413FC2985F1F224C743A256496
658C546E3B874F42793B536A7E37525E31445F95602F5C643D71331D4555363E1D143E63
2C6372353567652B5E252E1D516B42583C484B5C2A88625D4F584F3EA8512F3E78614442
573729987D215F6D3C594538462F3C74313A65561B74724B80A172604F585E2563482453
5D261C6D967A446D245B57447833646946538635747C617C4E415D62514F6C445351595F
47766E458D3D4E593C827A24226E7A3771703B6C7D4C375A52293C554A235E2D5A714B52
745774727466908A9A764268935A80A99B4FC0BC509FBE753944171B58445D472745562B
5E496B66AF6D35794C6AA55639C8507A5684ACAD6A856F965825767651653D3088946340
4D79224C6F977C494E7D2A5EB275486B753B937250BDAA561D39565523445D2719396412
1D548B563E66885440B17129526841A7633C9D3A2D949F6625275F63172E8E94
848E53503C6E42404F316F83733B4B5A3E5A4F6B4D2B5D5A3A6939376757403E331E5E5E
28595A364F7E692A62374B265766458857594B4D2161494D64473022704E2F3262343455
582F2B8C45256B702C5A42436C3A3C7320497759255D57256694775A586A72298055306F
76514983AD7333753C4D4C387A3F4B4F525094664C825D7E63585B60505D7C70784F646D
63815B3E887256362872763E2F7A703F9A8A47739C44538050424C6E52317C5483726175
836C7B92683A86A68C5F456586353F6B4C46C7CD5F55725E454A2E635B405F5A3C766F35
7260253A916C348E3C3575634AC1A5B0493F5E7B3B8CBAB65E1F4A6F5B962B1C4569374B
7C8B35223860554C808D2D305A574979862571613CB6C4833B3D52472054806B23487C43
2C5D865447676D1332B77D264C36288E4D67C261329B9C4C3E436869282D6986
615954503257314F87464A4B5E2C492B2D5934642C1F746637782E2B5052394B55183A49
2952291836635C236E1B3D1B4E332955464E476C3C84535B6F6860377B603E3D7F5F4642
3B1D237B51215052306D433F43383C7A354271561E619A4C749D3F3A374C37299E801E33
54544492BC7326703C4C2F2E7946515327476B634F403D73595A643A455C696580265968
5B7F54327A476C552D67811A288E7A2E67783578B354274B57306DAD5232505E8D813155
655568905A5C7CB57B515D9AB3603F6D473A8B9F706085503665252247427B822A46521F
716B5A6FA549329263446E193B994C6F5E4A85812E343C7A8D375243399468314E812F2C
496F4B25456B2C1D78905D4C9A551F738043836B277E9C7739516938143154251550A690
366853163F9F8F3D2B8A622B6768548F1755B1703A9F9C1318468E833C22557A
695E4A593C793B2D67415E673D3069603B3A256642214E3F3C7E47446C2B2E535D24412D
274748233E6D342D682B3B1D69643B6A363F6C662D7022435B5043416C37254F72563D4B
4B19397E5D296529226157454F1A38873C3C7E5B154A734D8BB991542F4A171D73763C50
6F3335A7C08D466F296A593784346E735941695572795C6D454882725157603D95617B6D
40908052AB20496C47719018257B893D7B7C1A55A96D3E7171193A686D264F42C4A26952
624058BB886E88B97217417CA366469A963E979D6676AD72255434304F3282994F49541F
616E716FC85625816D5894403CB557804276928142906B8E5F3753493B6A3D2C527E6543
507032264965674547765B53BC622B5C7C3C7C725BBD99642365AB7E1C3B5E1F18346C55
3063703F2A465E4F2DA339227F6A73953231A53F3298823A1820446E3E2A6A89
7694523E3B6D42405A3594B06F2E3856344F2B553E265B5630592E4E895325435319595B
2A43552748714020501D27123545367D472F3957297659564F555048A54F382B6F623A47
5B3721786F32675C33614B4B65273B8D2D34886B3357761B5999817F8F7163286E44274F
6E29265A79924778124C623B6E1343775E427715579B727B4530466E4C5F572B5B6E9384
33596A5CB2704A514A8C91211A5876458F9B4969B33046697B44445A4A4F78287D8F7051
8C1835858E8290D095503A608B4871A1AA369BA7594159612D5C3C57652B4C7D4C7B8235
6D673C6BB88833854354786B41B960815A41545C489367C33E3B9674597E5832537E613E
5570394589A6824D4A8B245A96805D666D1B74A664BB965B1C425B651D3B643E253D712F
26817A5441646C2734B8531E7A5368BD513BA94C287B8B4B23294D6737296383
648252563C6E3F4763263E56673340321F5D3B6B221458733C721A265F74353F4A124054
3C481B114B8374296918471F483A3E95753B48490D7A6A546A491A1F68743B30612A396F
6F3F278565338478366E573C7B4F4794142B6B7B464B962D57A269484D5E5A20694A2E8B
A04255B3B16A3C90454A463A99543C563660936053505BAC705A5C5B74B984566232578E
5F5E42387770563D1C7C9C4E2C6F712184963B66904E2C4A48444B62361D554B8E855C7B
9A714D7A8F3D73C8996D375D8654306F9633859A755970803551254B81404E6B21567A31
6958414D8168337E524D6A6238A77179835B717467A665AB67295578558243398DA57945
5573291F43696F4A4B7F57478A8864706924545B3FA281483F56625A1F51775E26478163
2A698A5139566841259C621F794065BB4238AA4E277392381E26476F48266790
6167566E4066263F7D47566C71435A68284A3B594E294166376C41276B773A4B64214056
4846491D2E6767297B0F47224D2F2654542F4768316F555072686756575C44457132222E
624F298649116B652E7B61394D2F64A14D316E7025488E5463A583944C4D71286951193A
605B5DADB3721E794F3F2C1D69453D5835316272485F3543544F59432B4B664E5E46566B
5959705767625E341B48713F2B6F802F6F96404A9E6C34566C4C476B5A2C6556A4732950
89754C8D914B4EBC956E36639F64397C89348EB36F5B72863B52352957374F7548465326
5C4D4C7EAE5B347F4C5185493D977F7F85677264499AA69A753C5856477D562E56876D49
568A57294B63534F7095726270827581894A58795BB7B47A3F637D5A1D3B5F6B2D5B9D7B
316F8A4C357E874C309F5527886472CF2E58BA7C2E54A62F2235657138225180
6445475E3E783E3C88435458282877793A4B3F7771364A5A3D974927574233616B253820
2E665E2E3D7A2A1E86373D23696F357342367D8740662F395B85634C693C175079603F47
6E2F3D767233831B0A56714D5125297250376F6E223198535BA183937547482D7C61194B
68383BBAB93823652E4A3A2F854C40613D418174448A6367665451754E69775052425185
52606E618E6F65493A5991402F7892447CA54455A97B406F893F579E74386D5D80687456
58644E98977763D1A159518799576FB698377AAA536CA08B3F5E3838553654875F51632C
6D715A8AC55C33885352AA57398975816176A87E33775E8A5A447D48439C7C4B82A1533E
5F6E2744737F67595D766F5A726D3C8C965B628E5E9F9C691E4F6D5F1E3B552021556865
33664E1821596E57309E3919666D47CD4B319D8325478A5A2B1F386533284E88
6166474F3F8747437A325E8358273C51232C19424D26482E165C321C3F492330511C301B
1F3B552F35664E21562130135361337D1C13385C3A5F2C4F50735B4886673C2458483B4B
67221F72793677562D7C6A3E4E48399A4D2E686E3F3DA64D62B58B85AB5F5A266F402284
9A3C3EA0C43B3081376147379155576A5049977C4E986773714D6F8650707D4E61575CA3
51785C51A67953553E76B4562D7B743F8CA333509C3E2D4B5843476F4A42845882837379
88644AAB9E3D66CAA57336589B4E356EA04178915F404F682C4C3263782F54783C62743A
666A162C987A368E5334735B39857D806A4B68734B9476C248225D784F85301E4D7A6739
4F5E232643475B505D6F2A36796B4F8B841A47773C967D59384A6064203D5E5E2E567B59
2A634B3B32738B722FB5752A75814ED08B4795861E5790693029416A4F29528D
617C4E563F793E519547425271444B6D2A4847564B2A44543156311D3970414D521E4537
2A45552D466A562471243F2E7172389B5E27456C39564970827144396359433C6A392545
4F314E9F7029775E308967396F504CAE4B3B8778424DAF3740AE695984635C21804F1135
69554DA7B34D22674F49381E725047542D488E6D4B5C3F796556574F30607D5354363494
6259462D6464633F1F3C7A33266B803759993C5B7F6F3A58A0614C6C462D65618C6F3E4E
755746A2866A73C9A46C3D6AB16740728B4098CB6C586D902E4732304E3750873847472D
68573F6AB2612A81535691543D838871877C7D814FA682A16E3E61514C87663F609D753F
4A60412F4E4766575762794C7A88418885324C45259D753F3C4C5C622052786737586456
2785764F3D697D2B27AB742B784B2CAA805BBC8B2B388D493F647A6525317E8D
5F4F494C3C72373B844457582A377E6F2549335D52245949337A3E213E362D6061204122
2165511E2F62301A74212E2969312D9B392C605F30511E4E6E7656465D57202F57512C37
441222595F257030174F4D334A272681513B796C19307C4240AC7A63BA585E288F902F4C
895340C0C45134734C454C34866E485C504394875C81585F595B5880446568514D4C529D
605965457C5571453A47883121608A4A719C3C53AB873E509B5D497A563C827B91687B67
5E424EAE81838ABAA3565B8BAB703F7995406A936154659334453D3A6938529560535C34
6366549AA95A316A42558F433576848B8D746A6E46B7C79576466048438F5D3B5C906D4A
69874E36554A5B5B887B8C585B8858828F425B5238A3BB614749593D19687F69355A5D37
215B5B393A816C5826B0702F686933823A3992773745915C1F1A415535184883
5E864A583E915547955955506D4240623136212E402744391E62421C334F2946431B3536
2348643138665E1E5D173D33564C40B95132354F463C20766A6B524B5A704A24655F333A
4D3842948428734A2C5B39344D3F2B96473B70523435AD533AAD8C4858A1401D7748274F
9654177BAB5634793E64542C795C506552439E85598C5C575F54558A516D7255534B749E
627E754AA14D8A744644A0301F5D8F4A7D9A345EA6803E5A845D52904F33847B8A6C786C
545B5CBFAC8672B89C523861AD7641849B5381AE6C4F8CAC43523B37613653905D4A5329
63717599B97143A66B4D8050335E787069816D6237786A7878446935479662375D7D2C2C
4E693637544545305B78825B64843475A7507180327F826A485966601E3657553B928F75
286F4219308B875F2CA55E24848D536076619C893A61AE612D203A64331D5189
64954D44407A484A8045909F8F5048783C3B464A3623495C376D2D16316C3C524418394B
344C5E373968642B5B174240564C49BE5E46475951371B83716850498C883F266255333C
5D4C4EA183268B69277554395151299D573A6B6C5733AC4745B5934A6F846430873E1D3F
784C1B4EB05A4C99334B5A30782B71856748B15A4B925E6464588790516F824E675F5F8D
4E676A538E57665D584BB72E3086964C67932571A37D3268763772916134824C87978264
60473D99857258A9A55E396BA54F5C9CA85D8ABF5E2755782D313C417044709656435A33
73835C84BE5B33896C4585503E728C74557C9F8F509F92997B4566383F7F60385985693F
53613D48774C7D49526C8B4C6E80375A8C346E4C2A958447356D8A6F2038502B1E405E53
2F94634F2159624E3CBD671663395180816EC0A0334294633A27405A25267891
748750533D8B4D49904A45506D484373403E4B4E3C294D6034682F17315D404C4B1D5453
2E4C401C52775525581F4E51586444B16A414462565134777D692B264F7445467128254D
644854A46A209B792B88673E786A2CB3404B8D875A3DBF5035A77747B76A54288B401644
784C1D30983C44614B4E4F2B8A635055305198704B4B297278364F64408E94534F4975AD
79753D3A814E6C55285FA05A29587440858F2D72A276387D7167497C4135867B836C7960
5C5061B0A18158ABB58E467DA149365C794895CA773E5888334B37809A3A55854C6E7239
50752145C481429D4951B75442778D893767658E51BD91A96F489163568B623F80A87F42
5367253C7D918B5855718C46A38B4F61981C75795BA688553A4550532C6C895E24495A27
1F73826A1F6E6136297D62325156437F5E50D5A66A58AD393858705D411C427A
65544C54406B3D59A257566783433C402C6C474957357379376A44263F52334B54253C51
27626436335D52236C1B4C3B80543DB97F474A532A5237698E69473E4F85282E5C53332D
583D54AB6C1B875029776338443A2A9255366E852F2A845339A98D4ABA50462B916E1D41
7A843C3FA7739D9B445F682B6352667450368C7A508656576B436C7C3E5C75587944458F
589A72479E2E5472443DAA322B7169219CAA4277A140246A81655B90432D727DA47F4F69
9A6E68A6926B5BC4C1776598B7723759824070A06F404A9446685435774B6E96366C863A
55654250A36943AB39305E5F44728F78487B5D7649B691865F355045489B5B304F8B704B
706B3A2A3F515E4D96B13E21938F527399305A553C93C36545484936173E5C5826507F69
34908A6E1E7D6E733A93531F5662458B3640ADA45F67A95C2F2053753E1D4C98
69714750419047437C475A583C42808D3E46375D6C2A485123897926342E092257394037
0A2864353877351351274244812D228B5D41735F4C57224C6440343E4D803D296A5F3239
522D347C6C1E9453274D36404A5321945235688A58339E603DA79F4EAE7232238F872E47
83501E4B7196A1C02F385D2F6820497564388337277E595650346988517270294D62528A
38786E53834E57574B40C0323281873966893483B672216E642E639C822E65409EB7533D
838564A9BB8444AE9F404568AC593C55924D86C184403D8533413C2E633281984B5C6724
6D85687B984D61C15146744D3267A48F56845B783D9FB3AA773861404EA0803A4C833B39
5269472F4E6D4B2D57756F3A9C632E5774505F7F3C749968494E617126375043316B6F58
33737830257D714D328B63324D6A54B05A458D98357DB25E2C22427643306595
7C9A4C424277484A6B307C9874302E54385B212F5F3350591151532E4D5E28203F194A64
203556273F80641F5E1F42484D4B379469373F3E43653966654D363C5F813628655C1321
653E338271217B7133525445605F23985D45959E692B83373D848A49914D3621804F1D42
7C752356519ECD9A46485E316059455E60386D7559975D5C4F2E4771516E6A4C5A5A5C9C
6570724B8B6A70563E48BA2A236A7134557C1E69896A1C506D54466864376F406576815F
7D533273879358ADA15C4D65B06049719C5289AA824B7F8F4255542855375C844A556B26
44796975B24E3AA77B44754C22359EA35498AB913974638486487C8072977C4655967646
546E4D3A7084864D737B3746B48A4A6A885D7577568FA65E3563846F253B484520477067
2F6C87661B6D57433DAA8238693F20655D349096435E9E4F2F2940651E155483
608B4C3B376B48456B1E2E496A3D32300F50413B451475843C622F14345C331F410C265A
283448041B55632677114547461F147985494E3F0F49515B5E5817155C7E412361472338
683137804B1A706B365660286D64228035305C825322965F2F958A5A8A69462D9848224F
8274354641B6885F3A505A29615C4E605F3D899163A5615166496888495574647269509A
7A7F6F337F597151384BC022266C7B3157903C8A94651F647C6B476C5A38696DB4787D5A
83746BA1976557AC976B475F9B3B558D904782B861204D7F4B6855285C486D93584F723A
486C2F59AF663CAB4F527F68414FB99531596F764FBA86A75E246A887499563372A47344
5865171F3576804D6D775D4494683E60721E50715087BC903150606E2B566F6229488566
37A89F75206C571A1E6063396D6A2F8A633FB5A07A86BD77352B42784C2C4C90
619A4D5B4191494DA151525284433554253C413A4810426E39604119294C351E392B3264
3E2B4C2A225B5F257819462D554D3A7A77484E4A445553434B584C213075432A6A452A35
4F2E35776D34595F38584135566224825A498786352196643498943284854D2C8F412146
716D345536B43D654F4E4E21775D4A623B3E858C5B8E5079744A6883486A8E65785C60A0
6C795231907F665A2061BD5B39677C368CA140879C4C3370776248664B44657395826D57
A779609B817A48B5A263476DB4483A66863689CB743D5F8C4D5631438947668C34468D42
447A244FB55631964F4D7E5B484FC087506D667347B797A06E3A5C706E936D304988683F
48654A2E477D7B4671745C61A28A576E7D4F5D6E3D82D79D525566421F61825C264E6A32
1C73876F276C4F74347468459D8E388B4A2981A765A8C56939324B6F522B519C
686F4E5441662545744A5A4E674B557D3D434B395A304B813553402735552E355636365A
2B3B513D247168237C2F48275463417B6E456E3E335E55436A77722B377F1E317064424A
5E2C4D90724B5160397D53465B432E74613B899D42297D502994A8366E68552F87601725
556E523B5F852F5840484E21604838634C295769426A1C4D6647675F3A65824C6B504A8B
617C653D7D53625D453FAA4E2F616C246B8C3E808F4A40846D5F4A6A4E254B536D995052
9F88779F927254B59A3D4182B97D41659143609F8459848E3C6E3A355A41679B44448730
517C5B67BB4E34A155415E4C5683D598678F54672C879093803C6B5C66B8874E52A05849
6194733248903A3189905F5DB6412D5E7A624E832767B37A447BB17A232E56622348676F
2F82645664AE947737774C2EAD803B98642971A456B7C75A3553717C4B314F94
6160454F40813F3E7A48594D3916355F36392C526B3A4F7131796232344C0E123D3C3141
0A24523A266D3A0E62382E144F7238724B1E434A3F5A3422345F6A2D357C371B564F4854
5114467365573F592F4D4A3D6F4322734539A4AF551FA8363FB19B5C6169281B798F4535
5865254EA96D45602648682F6C3B406D6B36617A5199515B534180813D456C2665765D93
3C938C4A9F394F81624FCB503E8D95375F854084BC692D6B623E62B282375F4A7BA6634E
A271478EA08F5FC9AA484063AA5581A0A0486FBA6733909940433F215C447D9459549120
476D607AC04D2385756AA35B3A9FD6AF53915889449564916F3E695B405C804F5BA54536
3E786E2F498864385D745A50A9743F62665B5E845787B69542445B692739572A2E5B6D6E
338E5F3A214650613987745DB7533DAA844775B82F93BA8E382E4670384587A4
659D4C474380474477356A717A3C2B6238401F23613651631957442E4B6A44173D2D3C6F
3F35563737725527712B42354B60317B6944492E1E80523E4958362A54975B25551D336C
7B322D884D35497F4B655C2262662589201A608F5E2DC5431D778A5B53641D1470462B31
486D1A2376694B6D2A3B794A94382E5D794275511F8B5E683D1C568D44506B183B7C5CA5
38545E469A614E5B3C7ECF42205B80355C852839955F4D928B3A3A585D49884F487A8951
9E714786796C42AF93494A64A7433D5C823F83BE814C629945563F4F844B55833D599041
477E25409059297E3E3C61603A3EC89566B8AE7143AE7FB05B45A471647A825C95B38E4D
5577553F63948846646A3369AF8440605D2036562C91C0795F526D7A293E6559234A7230
24938F58297C8B2E29A18B57BE3E2167362E65B85E89CB6137698E741F1B4B93
5A8753423452404F5817374E804C343C1767553E69317C854672351C3E6F601F392A1F57
413A3E0E1455722975124B594A13107C7D4B6B4A0E58634275773B17296A4152842A233C
5E2F40853D26266B4A6B54296B6E30944F2A5D8A2223945A2495B65F7A9335258A3E2953
69874B2774593B78523549246D68325F3D335A6D4D893D81754A6479507C84585B534395
646A5937914F5A592F45A049346C7741979A3F48B54A2D738A6C475C5D304B4B826C6D4A
AD956596887756B7C04C4760A84F4A6F844689D0A067729F5B6D3D2A684A558338528B47
4C714A6FBC58328B474D614C4A50C972406059613FB18C924828625B57723F3166996B47
536A3321527F8D6466714059A873325A573D3D592F8DC997615A634A285F7F5B3486A243
1B507F313A94A64B1D633C2D7C77368D6A3F666B4581CF64362A3E67392C5995
5A5842453848365D76363A43664C6752133F423F3A0862804080431837665A47553A2E54
37615942264F571F762C4B4762461A63382D4944295037284B4F5C382F71211F57514657
5B33769D695931603FA5773A76402996663060726121BC602788944645734D228A3C192F
416C612C80432A6B3F3E4A307C602F54542D5B886189515859445E7C3E466E65554B56A4
6972694B966C6368545FC74931698333668F4052A76B3277926E47645D3D6F41606C764E
8288669D908771AEAE58546DB96E5B81732D64AA9E5174932F6D57347F5D5D895169A659
8B886D8CD0622476755471483561CB8C5C846D5F40BABC9B5E3F67495C77534D68A16642
6C785A2F788E3943A09A5465C47638748658448E3B6FD4AC5B6E865819314E2021567E7A
2F5A864F20606B6C2D7C5A2B4C6734887B498958347AAC4E423E58833D266495
62974D563C72434583424E503928436530373C72743056492F6E5024533F201A372F2E4E
1E2B4A35325E3B12553D39264E531B6C4E375035224E44407E747D42327E45256A5D505C
6829365D575741482757663D7F6E339B6A3060756622BE67298A725B3F8B361F864E4939
39845F317C5843732C3E47327C5D355355366F9C6A85686C6645588C505B6F705A6057A7
6D727B4D854A548A645DC6403A8C88397E854770B4753060862F5DAE8B40734B5F5F995B
81865E9F838E749BAE535C8EB87C8AB3A85C7FC583578CAD3B4E4C33754B899D5274862A
5F9A7D7FBC833982624B7F794F40A67D5A9E615F297D7D8C8F456B3B2E687B5081B2372C
4F734D3E878D5A364F713D71D05D134B675B5DA3396BAF5632708878253B5D4A2787A36C
3D5C8C50287392843F7E703B4F7A339A794897674985976F48476A8B3E2C6699
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.754707 0.00276855 0.97619 0.113511 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 0.003125 0 0.003125 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 .5 scale
320 string
320 160 8 [320 0 0 160 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage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P
% End of image
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{640, 74.375},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000:00000BR000`400?l00000o`00003ooooooooo
oooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`004oooool0C@22PX80Jf][
08^;R`1EEED0L71`05=CD`1GEeL0M7Ad05]KF`1ADE40TI6A04a<C01cLg<0RHV905ADE01ZJVX0MWIf
0492@P1UIFD0H61P04m?C`1/K6`0GUiN05QHF01cLg<0DE5A075aL@1?Cdl0CTi>05YJFP1ZJVX0B4Q8
05eMG@1SHf<0B4Q807MgM`1mOGd0=CDe07EeM@1WIfL0@T9206QXJ01/K6`0ATI605YJFP1kNg/0EEEE
06ATI01PH600LW9b05IFEP1BDU80LG5a05]KF`1GEeL0Ti>C05MGE`1CDe<0VIVI05IFEP1SHf<0OWin
04e=C@1WIfL0S8b<05MGE`1dM7@0RXZ:05iNGP1MGEd0T92@05EEE@1^KVh0SHf=05MGE`1lO7`0QhN7
0080IfMW00/0R8R8059BDP1eMGD0U9BD05]KF`24Q8@0O7al05UIF@1_Kfl0G5aL06MWI`0205iNGP08
07YjNP27QhL0HF5Q0861P@25QHD0JFUY08F5Q@1dM7@207inOP0U065QH@2PX:00MWIf06IVIP1JFUX0
LG5a07UiN@1OGel0H61P09ZJVP1hN7P0R8R808f=S@1cLg<0IfMW079bLP1cLg<0EeMG07MgM`1cLg<0
Hf=S0:NWY`1]KFd0KVi^0:>SX`1ADE40FEUI09NGU`1IFET0KFe]08b<S018B4P0Jf][09fMW@16ATH0
HV9R07YjNP0>ooooo`0605EEE@2l_;`0MgMg079bLP1[Jf/0H61P0P1^KVh0:P26QXH0IFEU04];B`18
B4P0DU9B04Y:BP1ADE40GUiN04M7A`1<C4`0F5QH04U9B@0g=cL0FEUI059BDP1@D500EUIF04A4A01:
BTX0HF5Q06QXJ019BDT0EUIF04a<C01@D500Kfm_07emO@1ZJVX0Ng]k06MWI`1nOWh0M7Ad09:BTP1j
NWX0G5aL06m_K`1EEED0IVIV05MGE`1/K6`0B4Q806ATI080IfMW03L0Kfm_069RHP1iNGT0Jf][08B4
Q021PH40UIFE07moO`1`L700KFe]05mOG`1iNGT0RHV908f=S@1JFUX0LG5a06IVIP1BDU80FEUI04m?
C`1JFUX0ADE507MgM`1;Bd/0EUIF04i>CP1HF5P0Bd];06=SH`1BDU80F5QH06EUI@1FEUH0EEEE05aL
G01SHf<0EUIF05=CD`1PH600GUiN05IFEP1IFET0HF5Q07YjNP1fMWH0CDe=06EUI@1RHV80OWin07Ui
N@1IFET0HV9R06UYJ@18B4P0H61P0080IfMW01l0EEEE06IVIP1]KFd0^[Zj08F5Q@2@T900Vi^K08j>
SP1^KVh0VIVI06m_K`29RHT0JVYZ06QXJ022PX80NGUi07MgM`1jNWX0Rh^;08R8R023Ph<0GemO07al
O01nOWh0F5QH06m_K`23Ph<0MgMg05QHF025QHD0YJFU00kooooo0440XJ6Q07alO01>CTh0AdM7055A
D@1PH600@41003Hf=P0S8b<0FUYJ04=3@`0S8b<0EEEE07AdM00[:b/0?clo03hn?P12@T80B4Q804E5
A@0m?Cd0?Shn04];B`1BDU80ADE505=CD`17AdL0@41006YZJP18B4P0?3`l05IFEP0i>CT0C4a<03Ti
>@1ADE40@4100492@P1jNWX0LG5a04a<C01IFET0?Shn04M7A`1gMgL0PH6105YJFP1GEeL0Fe]K03@d
=01?Cdl0KVi^05ADE01EEED0D51@04a<C01RHV80HF5Q06EUI@1RHV80B4Q805MGE`1YJFT0Ogmo051@
D00205mOG`0U05UIF@1RHV80LW9b04i>CP1KFe/0NGUi06QXJ00n?Sh0MWIf06EUI@1FEUH0MWIf04i>
CP1lO7`0BTY:06i^KP2^[Zh0MWIf065QH@1SHf<0G5aL04m?C`1RHV80BDU906IVIP20P800@D5109bL
W01CDe<0@T92079bLP0i>CT0J6QX065QH@1jNWX0CTi>06YZJP0205YJFP0?07EeM@1RHV80?clo06AT
I01CDe<0HF5Q09JFUP1GEeL0BTY:05ADE01UIFD0GUiN07=cL`0h>3P0I6AT0080NGUi0140GEeM06IV
IP1jNWX0@T9206=SH`1UIFD0<S8b05IFEP1`L700BTY:08>3P`0o?cl0QhN7079bLP1kNg/0EEEE06MW
I`0>ooooo`0N06UYJ@0/;2`07QhN020P801mOGd092@T024Q8@0[:b/0>c/k069RHP2;Rh/0;Bd]04U9
B@0m?Cd0D51@03Ti>@0f=SH0FEUI09:BTP1BDU809BDU02/[:`0X:2P0<S8b028R8P16ATH0QXJ607Mg
M`0];Bd0;2`/0P0F5QH03P0G5aL0NGUi08^;R`2RXZ80LW9b0492@P1>CTh0GUiN04e=C@0g=cL0QXJ6
069RHP0m?Cd0IVIV0P0R8R80900X:2P0DE5A09^KV`2LW9`0NWYj04i>CP24Q8@0FUYJ06i^KP2XZ:P0
J6QX02DU9@0_;bl0?Shn05QHF01ADE40MWIf09>CT`2][Jd0MWIf07YjNP1_Kfl0a/K60;Rh^02JVYX0
J6QX04i>CP0^;Rh0<S8b04I6AP1EEED0Y:BT08V9R@2RXZ80H61P02l_;`80OGem03H0IVIV08R8R02[
Zj/0K6a/06ATI02/[:`0VIVI0;2`/01jNWX0_[jn08^;R`2k^k/0Lg=c08F5Q@31`L40XZ:R0=3@d01V
IVH0ATI607alO01]KFd0DU9B07UiN@2h^;P0c/k>08Z:RP1OGel0Fe]K06a/K01PH600EEEE069RHP1Q
HF40EeMG08J6QP2:RXX0U9BD05eMG@23Ph<0BTY:02TY:@17AdL0FUYJ09VIV@1iNGT0<30`05YJFP2T
Y:@0RXZ:08R8R02OWil0GemO03Xj>P0i>CTDooooo`004oooool0DP21PH40N7Qh08N7Q`1VIVH0KFe]
04Y:BP1YJFT0FUYJ05eMG@1HF5P0MGEe06=SH`1dM7@0P82008J6QP1RHV80MgMg05YJFP1KFe/0H61P
069RHP1LG5`0HV9R06a/K01HF5P0Fe]K07AdM015ADD0DU9B05QHF01=CDd0EeMG05YJFP1TI6@0D51@
059BDP1WIfL0?clo07=cL`1QHF40BDU906][J`1]KFd0D51@05]KF`1]KFd0B4Q806EUI@1[Jf/0J6QX
05aLG01TI6@0J6QX05QHF01RHV80M7Ad05QHF01[Jf/0OGem05ADE01eMGD0KFe]05MGE`1ZJVX0Jf][
06IVIP1UIFD0LW9b075aL@1QHF40RHV906a/K01MGEd0Ph>306MWI`1TI6@0RHV908B4Q01cLg<0RXZ:
07EeM@1iNGT208V9R@0Q06a/K02;Rh/0SXj>06m_K`1PH600UYJF05QHF01_Kfl0N7Qh06EUI@24Q8@0
R8R808J6QP1kNg/0H61P0861P@1aLG40LW9b07AdM027QhL0NWYj07EeM@1VIVH0MGEe08:2PP1EEED0
K6a/08n?S`15ADD0TI6A07YjNP1UIFD0R8R80080O7al01@0IVIV07MgM`2;Rh/0IFEU08b<S01aLG40
Kfm_08^;R`1hN7P0Bd];07QhN01_Kfl0Hf=S07IfMP1[Jf/0HV9R06][J`1QHF40GEeM061PH0kooooo
01d0EUIF0:ZZZP23Ph<0KVi^061PH01XJ6P0KFe]06YZJP20P800HV9R0451@@16ATH0CDe=03De=@17
AdL0EEEE05mOG`1DE5@0GemO04a<C00e=CD0FUYJ055AD@1IFET0IFEU04Q8B01IFET0IfMW06m_K`02
05YJFP0`051@D01;Bd/0MGEe07EeM@1ZJVX0OWin05QHF022PX80Ng]k08B4Q01YJFT0D51@05IFEP1;
Bd/0EeMG05=CD`1OGel0@T9204m?C`1JFUX0E5AD05YJFP1MGEd0Lg=c05IFEP1RHV80J6QX071`L01]
KFd0F5QH05EEE@1QHF40Lg=c07IfMP29RHT0DE5A06e]K@1NGUh0GEeM069RHP1eMGD0GUiN05YJFP2F
UYH0HF5Q05UIF@1?Cdl0Hf=S0P1bLW8205iNGP0C075aL@1HF5P0ATI606QXJ01hN7P0IFEU059BDP1@
D500Fe]K05iNGP1FEUH0NWYj07IfMP1aLG40HV9R06YZJP1jNWX0MWIf08j>SP0206EUI@0C079bLP1D
E5@0JVYZ079bLP1fMWH0GemO06][J`1bLW80UIFE07emO@1lO7`0UIFE079bLP1RHV80NGUi07alO01g
MgL0G5aL06QXJ00207inOP0?07YjNP2KVi/0Ph>3096AT@1_Kfl0I6AT08N7Q`1dM7@0KFe]06i^KP1c
Lg<0MGEe05QHF028R8P0WYjN00kooooo01h0RXZ:06=SH`1SHf<0Cdm?05=CD`14A4@0ATI602HV9P0W
9bL0DU9B03Xj>P0i>CT0B4Q806QXJ00k>c/0;bl_04I6AP0[:b/0K6a/03hn?P0o?cl0C4a<04e=C@16
ATH0<S8b075aL@1;Bd/0@T9206QXJ016ATH204e=C@1/04A4A01GEeL0>CTi06ATI01:BTX0>3Ph07Mg
M`1DE5@0EeMG04A4A00j>SX0ATI6065QH@1mOGd0Jf][051@D01IFET0?clo04=3@`1dM7@0De=C04U9
B@1MGEd0AdM704m?C`1LG5`0De=C06ATI01LG5`0@D51065QH@1^KVh0F5QH07UiN@1KFe/0FUYJ07al
O020P800E5AD06m_K`1`L700HF5Q06UYJ@25QHD0L71`06m_K`29RHT0HF5Q06UYJ@1DE5@0NWYj0:b/
[01YJFT0K6a/069RHP1aLG40JVYZ05aLG01CDe<0Bd];07IfMP1>CTh0TY:B05=CD`15ADD0Lg=c03lo
?`1MGEd0EEEE06a/K019BDT0LG5a061PH01GEeL0NGUi05IFEP11@D40MGEe05YJFP1QHF40R8R8065Q
H@1BDU80CTi>05]KF`1HF5P0I6AT03De=@1aLG40JVYZ07EeM@1EEED0Kfm_0861P@0e=CD0IVIV06i^
KP0g=cL0H61P06MWI`1EEED0QhN703Hf=P1_Kfl0Fe]K07inOP1>CTh0Bd];3_ooool02P1iNGT0BDU9
04=3@`1PH600XJ6Q06UYJ@0M7Ad06ATI02DU9@1FEUH209RHV00B09jNWP1YJFT0?clo02<S8`0Q8B40
:b/[04=3@`24Q8@0AdM702/[:`0];Bd0ATI60410@01LG5`0YZJV08j>SP1WIfL0A4A40P0[:b/0>P0k
>c/0B4Q808b<S02CTi<0NWYj04m?C`0c<c<08R8R01hN7P0`<300IFEU06m_K`1nOWh0SHf=065QH@17
AdL0D51@06YZJP2YZJT0[Zj^07]kN`1IFET0QHF507QhN01lO7`0ZZZZ09NGU`0`<30071`L01hN7P0^
;Rh0?Shn075aL@2LW9`0MgMg06e]K@1CDe<0RHV90:f][@2KVi/0Hf=S06e]K@12@T80?Cdm03Xj>P0n
?Sh0WYjN07IfMP2YZJT0O7al07EeM@2RXZ80T92@055AD@0e=CD0>c/k069RHP21PH4206i^KP0`06EU
I@1eMGD0OGem0;fm_@2WYjL0M7Ad03Ti>@1<C4`0/;2`0=gMg@2^[Zh0QhN705UIF@1GEeL0Fe]K08V9
R@2NWYh0Y:BT0:RXZ01^KVh0CTi>04=3@`1LG5`0>c/k03De=@1@D500Lg=c079bLP1jNWX0];Bd09VI
V@1OGel0NGUi08>3P`16ATH0MGEe0<30`03EeMD0][Jf08V9R@2IVIT0SXj>07EeM@21PH40CTi>07=c
L`10@400>3Ph5?ooool001?ooooo01l0Ogmo071`L01SHf<0M7Ad04E5A@1HF5P0HV9R03@d=01YJFT0
Bd];05ADE028R8P0>c/k06QXJ01`L700@T9206YZJP1IFET0CDe=075aL@1SHf<0DE5A08N7Q`1jNWX0
FEUI05=CD`1WIfL0AdM706MWI`18B4P0FEUI0080C4a<02h0GemO05IFEP1/K6`0RXZ:03/k>`1RHV80
WinO04a<C01GEeL0SXj>03/k>`1EEED0Vi^K04M7A`1JFUX0Fe]K051@D01IFET0IVIV05QHF01PH600
IfMW05QHF01DE5@0NWYj08^;R`1GEeL0L71`061PH01aLG40NGUi065QH@1hN7P0K6a/07UiN@26QXH0
DU9B06EUI@1eMGD0?Shn07=cL`1lO7`0<c<c07emO@2FUYH0AdM70P2;Rh/01`1XJ6P0WinO07AdM01Q
HF40VYZJ08F5Q@1JFUX00P1oOgl04@1FEUH0OWin07MgM`1SHf<0PH6107EeM@2;Rh/0M7Ad06UYJ@2C
Ti<0L71`08>3P`1VIVH0RXZ:08f=S@1WIfL0LW9b0080P82001h0KFe]06IVIP1jNWX0N7Qh08>3P`1U
IFD0QhN707UiN@1gMgL0UIFE04=3@`1VIVH0ZZZZ04a<C01fMWH0Rh^;04a<C02BTY80PX:203`l?02E
UID0O7al04I6AP2JVYX0Ng]k055AD@20P800N7Qh04A4A01VIVH>ooooo`1Z05QHF02MWId0N7Qh06a/
K01OGel0K6a/065QH@1aLG40Jf][06EUI@19BDT0CDe=04Q8B013@d<0Bd];059BDP1JFUX0HF5Q06a/
K01>CTh0<S8b065QH@1IFET0C4a<071`L018B4P0H61P06YZJP1UIFD0FUYJ06MWI`1<C4`0B4Q8069R
HP28R8P0MGEe09:BTP1[Jf/0HV9R07YjNP2CTi<0S8b<05aLG01]KFd0EEEE06ATI01DE5@0HF5Q03Ph
>01CDe<0HF5Q05UIF@1VIVH0EeMG05aLG01SHf<0IVIV05IFEP1CDe<0D51@05EEE@1?Cdl0H61P06m_
K`1lO7`0[Zj^03lo?`1ADE40BDU903dm?@1MGEd0DU9B05UIF@1@D500KVi^05YJFP1HF5P0CDe=03De
=@1PH600KFe]05]KF`1]KFd0Hf=S05ADE01HF5P0GEeM06m_K`1PH600HV9R06UYJ@1VIVH0Kfm_05=C
D`23Ph<0OWin09:BTP1RHV80T92@08:2PP25QHD0Lg=c051@D01SHf<0OWin069RHP80KFe]0200IfMW
05YJFP1NGUh0HV9R08J6QP1ZJVX0NGUi09jNWP1dM7@0MgMg07moO`1jNWX0Q8B406MWI`1fMWH0XJ6Q
07MgM`2:RXX0S8b<06][J`25QHD0TY:B08F5Q@2MWId0K6a/05ADE01YJFT0JVYZ05mOG`1LG5`0RHV9
08B4Q0kooooo05X0Ng]k07=cL`1JFUX0ADE50451@@0i>CT0=CDe03/k>`0Z:RX0?Cdm04E5A@0W9bL0
@d=307]kN`0c<c<0C4a<02/[:`0g=cL0MGEe04I6AP14A4@0?Shn061PH00n?Sh0E5AD06QXJ01@D500
GUiN04m?C`1ADE40Bd];0492@P0l?3`0CTi>03Lg=`1ZJVX0GemO03lo?`2?Shl0Jf][05YJFP11@D40
DU9B0410@01>CTh0QHF5079bLP1>CTh0DE5A0410@017AdL0J6QX05EEE@1?Cdl0Fe]K04m?C`1ADE40
JVYZ051@D01ZJVX0DU9B04m?C`1OGel0IFEU079bLP1dM7@0J6QX06=SH`28R8P0O7al06a/K01hN7P0
RHV9075aL@24Q8@0T92@08R8R02?Shl0QXJ605ADE01mOGd0KVi^06=SH`2:RXX0FUYJ07=cL`1kNg/0
DU9B05IFEP1/K6`204m?C`0;0861P@1:BTX0Ogmo05ADE010@400Q8B404Y:BP1SHf<0?Shn065QH@1>
CTh00P1QHF405P1FEUH0FUYJ06m_K`17AdL0LW9b055AD@1MGEd0V9RH04a<C01CDe<0Bd];0492@P1=
CDd0K6a/03@d=01TI6@0L71`07QhN01@D500MgMg08B4Q00n?Sh207=cL`0;02l_;`1WIfL0KFe]061P
H01nOWh0=3@d05EEE@1MGEd0KVi^069RHP17AdL03_ooool0S036a/H0NGUi06UYJ@1UIFD0>c/k06IV
IP1bLW80@D5103Hf=P0e=CD0S8b<05IFEP2PX:00@D5102l_;`0W9bL0820P024Q8@0O7al0=SHf08n?
S`2<S8`0K6a/071`L01HF5P0?Shn07emO@21PH40a<C40:>SX`1PH600D51@03dm?@0Q8B40De=C07Ui
N@1cLg<0>c/k04e=C@1CDe<0Cdm?08B4Q00o?cl0D51@07QhN02l_;`0C4a<04E5A@1XJ6P0C4a<079b
LP1mOGd0PX:205]KF`1>CTh0GemO04A4A01^KVh0J6QX071`L00Y:BT07QhN01lO7`16ATH0WYjN0:>S
X`2k^k/0XJ6Q08n?S`2FUYH0LW9b05aLG015ADD0KFe]07emO@1GEeL0H61P09NGU`1ZJVX0OWin08n?
S`1^KVh0=SHf08V9R@22PX80:BTY01lO7`0b<S80SHf=0::RXP2h^;P0Ph>306m_K`2=SHd0ZJVY0;Zj
^P2i^KT0Kfm_04Q8B01OGel0Z:RX0;^k^`1_Kfl0Jf][01hN7P0P820092@T08V9R@1BDU80O7al08R8
R00g=cL0HF5Q06UYJ@1KFe/0?3`l03dm?@1;Bd/0EeMG07UiN@2ZZZX0YJFU07UiN@2`/;00K6a/09VI
V@3De=@0alO70<71`@3GemL0T92@08b<S02[Zj/0N7Qh08V9R@2HV9P0CTi>08F5Q@2@T900WIfM5?oo
ool001?ooooo0540Shn?07EeM@1lO7`0CDe=069RHP0k>c/0FUYJ06e]K@13@d<0DU9B06][J`1=CDd0
S8b<08:2PP1JFUX0QHF506MWI`0f=SH0LW9b06IVIP0o?cl0Jf][04M7A`1LG5`0MgMg04Q8B01cLg<0
IfMW04];B`1NGUh0IFEU04Y:BP2<S8`0J6QX0410@01/K6`0Jf][03Ti>@1hN7P0IfMW051@D01`L700
IFEU05IFEP1OGel0D51@065QH@1PH600DU9B08R8R01JFUX0EEEE08Z:RP1OGel0IFEU09:BTP10@400
G5aL09NGU`18B4P0JVYZ08n?S`1?Cdl0KVi^092@T018B4P0PH6107]kN`1FEUH0NWYj0:6QX@1EEED0
KFe]08F5Q@1EEED0KFe]08V9R@1ADE40PH6107YjNP1MGEd00P2<S8`0901[Jf/0QhN706i^KP1gMgL0
QXJ604m?C`26QXH0IfMW051@D02<S8`0PX:206i^KP28R8P0Ti>C07MgM`1/K6`0LG5a09RHV01oOgl0
K6a/0:JVYP1ZJVX0Kfm_06e]K@1NGUh0Ti>C06i^KP1JFUX0S8b<05]KF`1TI6@0UiNG06e]K@1WIfL0
QhN706e]K@80JVYZ00X0Jf][07]kN`2NWYh0JFUY0820P02;Rh/0Fe]K06m_K`2GUiL0EUIF0P25QHD0
1P1?Cdl0Fe]K09bLW00n?Sh0JVYZ05UIF@kooooo00l0J6QX0:RXZ01kNg/0G5aL05]KF`1RHV80I6AT
06IVIP1^KVh0DE5A05aLG01?Cdl0E5AD0492@P1DE5@00P1;Bd/01@1ZJVX0KFe]051@D00n?Sh0HV9R
0080C4a<00X0DU9B04m?C`1CDe<0H61P05YJFP1IFET0CTi>04m?C`16ATH0GemO0P1YJFT09P20P800
X:2P08N7Q`1PH600MgMg06YZJP1?Cdl0GEeM05EEE@1MGEd0CTi>06=SH`10@400Bd];05ADE019BDT0
Cdm?059BDP1iNGT0LW9b06EUI@1CDe<0MWIf0820P01WIfL0H61P05YJFP1[Jf/0QhN709BDU01CDe<0
MgMg06MWI`1]KFd0GEeM051@D017AdL0B4Q80P1ADE40>@1;Bd/0Cdm?0451@@1?Cdl0Fe]K04A4A01K
Fe/0FUYJ04i>CP1;Bd/0GemO05]KF`1NGUh0FEUI06IVIP1WIfL0Fe]K05EEE@1/K6`0QhN706MWI`1E
EED0IVIV075aL@1eMGD0MgMg03hn?P1EEED0FEUI05]KF`1OGel0Hf=S05aLG01GEeL0IVIV079bLP2<
S8`0Jf][07alO02=SHd0JFUY06IVIP2BTY80LG5a07inOP1TI6@0GemO07]kN`1dM7@0HV9R06YZJP1d
M7@0MgMg05MGE`1NGUh0PX:205aLG00205QHF005079bLP1fMWH0Fe]K08n?S`2`/;003_ooool03`1i
NGT0XJ6Q05QHF014A4@0EEEE03Ti>@0k>c/0=3@d030`<00m?Cd0<c<c03Ph>00k>c/0KVi^04U9B@02
04I6AP0X02h^;P1YJFT0?Cdm03Lg=`13@d<0EeMG02l_;`1<C4`0D51@05=CD`1@D500CDe=05UIF@18
B4P0?clo04M7A`1GEeL0HV9R06ATI01KFe/0CDe=09:BTP1YJFT0DU9B03lo?`12@T80>c/k06ATI02C
Ti<0KVi^05EEE@1NGUh0Hf=S05]KF`1bLW80JFUY06=SH`1VIVH0IFEU04a<C080H61P01/0IFEU061P
H017AdL0Jf][06IVIP1_Kfl0O7al06EUI@1QHF40K6a/07=cL`1RHV80Q8B407AdM01QHF40JVYZ07]k
N`1eMGD0Q8B408>3P`1RHV80LW9b05mOG`1XJ6P0VIVI069RHP1]KFd00P1ADE40=01^KVh0IFEU05MG
E`10@400JFUY05]KF`1NGUh0CDe=05ADE025QHD0D51@069RHP1UIFD0N7Qh055AD@1^KVh0MWIf04m?
C`1oOgl0IVIV03Ph>01fMWH0GUiN06YZJP2FUYH0IfMW05mOG`1EEED0AdM705mOG`1dM7@0>c/k07Mg
M`1/K6`0QHF503lo?`1nOWh0WYjN04i>CP1QHF40IFEU04U9B@1IFET0Ng]k071`L01jNWX0?clo05YJ
FP1iNGT0HV9R05]KF`18B4P>ooooo`170<W9b@25QHD0E5AD059BDP1MGEd08b<S0:j^[P29RHT0Ogmo
08R8R02QXJ40Z:RX09FEU@2FUYH0@T9202PX:00N7Qh07AdM01lO7`0Q8B40Q8B408V9R@1_Kfl0W9bL
0::RXP2>SXh0Ti>C09nOW`2:RXX0a<C40:b/[01EEED0:RXZ0451@@1;Bd/0?Cdm04M7A`1IFET0>3Ph
02/[:`0d=3@0F5QH07alO01HF5P0EUIF08F5Q@0Y:BT0:2PX03/k>`2WYjL0BDU9051@D01:BTX0RHV9
02XZ:P1NGUh0>c/k05EEE@0_;bl0M7Ad04I6AP0V9RH07alO0451@@1ZJVX0LW9b09BDU01XJ6P0De=C
07QhN025QHD00P1@D5007`1TI6@0=CDe02<S8`0e=CD0LW9b06=SH`1TI6@0]kNg0:ZZZP2;Rh/0^[Zj
05QHF010@4009BDU03De=@1QHF40P8200:6QX@1TI6@0U9BD0<[:bP2RXZ80alO70::RXP1bLW80?Shn
01lO7`1UIFD0UYJF09NGU`1CDe<00P0M7Ad01@0U9BD0Rh^;07=cL`23Ph<0Rh^;00808B4Q01/0RHV9
08B4Q01UIFD0=3@d05EEE@0g=cL0DU9B07EeM@1gMgL0clo?09jNWP2^[Zh0]kNg0;fm_@2l_;`0WYjN
0;2`/02IVIT0BTY:05=CD`1TI6@0BTY:0:ZZZP1`L700=SHf0492@P2<S8`05?ooool001?ooooo00@0
O7al07moO`1XJ6P0Q8B40P1BDU80J`1TI6@0@410055AD@11@D40AdM706YZJP0i>CT0Fe]K06a/K00k
>c/0LG5a061PH01<C4`0M7Ad07=cL`19BDT0GemO08n?S`15ADD0IfMW08>3P`0m?Cd0G5aL04i>CP15
ADD0DU9B05iNGP1CDe<0Ng]k05ADE01FEUH0FUYJ06IVIP1RHV80IfMW05UIF@2=SHd0CDe=05mOG`2=
SHd0B4Q8065QH@1VIVH0FUYJ05ADE01SHf<0GEeM05MGE`1]KFd0HF5Q05MGE`1oOgl0PX:206UYJ@2D
U9@0Kfm_06MWI`2?Shl0IfMW07EeM@24Q8@0J6QX08^;R`1DE5@0K6a/08B4Q017AdL0IVIV07]kN`14
A4@0QhN708Z:RP17AdL0MGEe08>3P`1OGel0WIfM0:^[Z`1NGUh0RXZ:092@T01DE5@0NWYj0820P019
BDT0M7Ad06MWI`1QHF40VYZJ05YJFP26QXH0Vi^K05mOG`2EUID0Kfm_08B4Q01gMgL0NWYj0820P01a
LG40H61P08>3P`1fMWH0Fe]K06e]K@1/K6`0O7al0080IfMW01P0Rh^;069RHP1gMgL0Zj^[05IFEP1W
IfL0ZZZZ05YJFP1cLg<0SXj>05MGE`1dM7@0Rh^;04=3@`2IVIT0QHF5030`<02SXj<0Ng]k0451@@1b
LW80KVi^04];B`1WIfL>ooooo`0M05YJFP3GemL0TI6A07YjNP1hN7P0GemO06IVIP1/K6`0SHf=06UY
J@1BDU80Cdm?05UIF@1>CTh0GemO05MGE`1KFe/0Lg=c07MgM`1CDe<0?Cdm06UYJ@1JFUX0ATI606=S
H`1RHV80Cdm?04A4A01=CDd00P1JFUX0C01>CTh0B4Q8051@D01_Kfl0JVYZ07EeM@1=CDd0ADE503dm
?@1mOGd0NGUi065QH@1NGUh0Cdm?05UIF@1@D500JVYZ04U9B@1:BTX0Cdm?0451@@16ATH0GUiN06m_
K`1XJ6P0JVYZ05MGE`1PH600EUIF0492@P1BDU80Cdm?06a/K01bLW80Ng]k04E5A@1LG5`0D51@04m?
C`1;Bd/0D51@04Q8B01CDe<0Lg=c05UIF@1UIFD0EEEE069RHP1TI6@0KFe]065QH@1]KFd0IFEU04E5
A@0j>SX0Cdm?06e]K@1QHF40CTi>05mOG`1IFET0G5aL05mOG`1KFe/0Ng]k059BDP1VIVH0JFUY07in
OP20P800SXj>05QHF01WIfL0TY:B05QHF01UIFD206e]K@0O059BDP1MGEd0NGUi09^KV`1oOgl0RXZ:
0820P01XJ6P0MGEe07moO`1]KFd0RHV904a<C01=CDd0PX:207AdM01VIVH0JVYZ07emO@2;Rh/0OGem
07UiN@2@T900DE5A04I6AP1/K6`0LW9b06IVIP1WIfL0P82008n?S`0>ooooo`0404a<C02LW9`0GemO
04e=C@80CTi>08H0@d=30410@00f=SH0<S8b02d];@0Z:RX0=SHf07AdM00h>3P0ATI603/k>`0e=CD0
KFe]055AD@0d=3@0CDe=06QXJ00^;Rh0ATI604=3@`1BDU80IFEU04I6AP1OGel0M7Ad034a<@1KFe/0
B4Q806ATI01[Jf/0IFEU04M7A`1bLW80KFe]0492@P0j>SX0Fe]K04A4A016ATH0RHV906ATI01:BTX0
GEeM04M7A`1BDU80Jf][05QHF01GEeL0GEeM05aLG014A4@0GemO05YJFP1SHf<0E5AD04M7A`1LG5`0
BDU907EeM@1bLW80EEEE071`L01cLg<0IVIV05eMG@1oOgl0M7Ad06m_K`1dM7@0OGem08F5Q@2<S8`0
SHf=05MGE`21PH40O7al06ATI02=SHd0GemO07moO`1JFUX0D51@06QXJ01hN7P0Cdm?03/k>`1hN7P0
Jf][06=SH`1HF5P0E5AD08V9R@1GEeL0NWYj04=3@`1VIVH0DE5A061PH01[Jf/0EeMG06YZJP1fMWH0
@D5106][J`1>CTh0IfMW09>CT`1EEED0E5AD05eMG@15ADD0CTi>06YZJP16ATH0I6AT07MgM`2<S8`0
ATI6092@T02BTY80HV9R06e]K@1YJFT0G5aL06MWI`1[Jf/0GemO08f=S@10@400Fe]K05EEE@27QhL0
DU9B03Ph>0kooooo04h0ZZZZ08N7Q`19BDT0Cdm?06ATI01<C4`0IFEU05mOG`1:BTX0K6a/07IfMP2I
VIT0WIfM07alO01>CTh0B4Q804M7A`0_;bl0:b/[04I6AP1jNWX0ZZZZ07MgM`2IVIT0R8R804i>CP0Z
:RX09RHV061PH01lO7`0];Bd0:^[Z`1JFUX09bLW01/K6`0Z:RX0=cLg02LW9`1?Cdl0B4Q802h^;P1E
EED0U9BD0861P@1iNGT0VIVI04e=C@0W9bL0>CTi06m_K`1SHf<0?Cdm04A4A01LG5`0;Rh^04Y:BP23
Ph<0Ti>C06i^KP14A4@0Kfm_03Lg=`0M7Ad0CDe=0:JVYP1bLW80I6AT038b<P0[:b/0:2PX06a/K01R
HV80RHV90:VYZ@1UIFD0>CTi04Y:BP1`L70205IFEP0F07]kN`1<C4`0CDe=065QH@1dM7@0IFEU04A4
A010@400;bl_05iNGP1ZJVX0WYjN06m_K`3AdM40a<C40:BTY01[Jf/0>CTi04e=C@1?Cdl0Lg=c0861
P@80LG5a0180:RXZ01dM7@0j>SX0MGEe07alO01[Jf/0W9bL05YJFP1[Jf/0Ti>C03`l?01FEUH0TI6A
06][J`0Y:BT0?Shn06UYJ@1fMWH20<30`00=0<;2`P2c/k<0^[Zj0<30`03Jf]X0GEeM02/[:`1hN7P0
Fe]K06=SH`2:RXX0QHF509bLW00208j>SP030:FUYOoooooooooo01;ooooo000Cooooo`0b08b<S01a
LG40U9BD06ATI01EEED0EeMG05UIF@1<C4`0IVIV04Y:BP1WIfL0H61P05ADE021PH40Fe]K05=CD`1Y
JFT0A4A4061PH01SHf<0D51@05iNGP1JFUX0IfMW069RHP1>CTh0M7Ad04];B`1=CDd0Fe]K05EEE@0l
?3`0MWIf07UiN@1;Bd/0IfMW07QhN00n?Sh0Kfm_07EeM@0g=cL0IFEU07AdM015ADD0Fe]K05aLG01:
BTX0IVIV05eMG@1QHF42069RHP1G07UiN@1;Bd/0G5aL08V9R@1RHV80H61P08N7Q`1EEED0QHF5075a
L@1OGel0O7al06i^KP1MGEd0O7al07YjNP1oOgl0HF5Q08J6QP1_Kfl0G5aL08b<S01WIfL0EEEE08f=
S@1_Kfl0FUYJ08F5Q@1UIFD0MGEe08R8R01cLg<0Ng]k07YjNP1iNGT0MgMg05iNGP21PH40HF5Q06YZ
JP1^KVh0KFe]079bLP1jNWX0UYJF06YZJP1TI6@0SXj>08J6QP23Ph<0MgMg0::RXP1VIVH0Fe]K07Ui
N@1QHF40Ng]k075aL@1GEeL0Q8B4069RHP19BDT0MgMg08j>SP1UIFD0Ogmo08J6QP1NGUh0IfMW07in
OP1[Jf/0UiNG06YZJP1HF5P0UIFE06e]K@1DE5@0VYZJ075aL@1?Cdl0Ph>3071`L01NGUh0MgMg05IF
EP1LG5`0HF5Q00kooooo08@0D51@09VIV@28R8P0IVIV05]KF`1IFET0J6QX07UiN@2>SXh0HV9R04A4
A01>CTh0DE5A0451@@1ADE40EUIF06=SH`1WIfL0Kfm_06=SH`11@D40K6a/05YJFP11@D40J6QX04a<
C01BDU80EUIF065QH@1JFUX0EeMG05QHF01:BTX0Fe]K06i^KP1XJ6P0QhN706e]K@1GEeL0EUIF08b<
S01jNWX0F5QH06=SH`1JFUX0IFEU05]KF`1UIFD0DU9B05]KF`1LG5`0E5AD059BDP1PH600MgMg06IV
IP22PX80JVYZ06QXJ01MGEd0J6QX069RHP1EEED0OGem08J6QP2>SXh0GUiN05YJFP1OGel0Fe]K05YJ
FP1EEED0AdM705eMG@1^KVh0GEeM05YJFP1EEED0D51@08Z:RP1fMWH0EeMG06YZJP1JFUX0A4A403dm
?@1XJ6P0JFUY05mOG`1WIfL0IVIV05QHF01VIVH0GEeM06a/K01^KVh0HF5Q06IVIP1UIFD0KVi^07in
OP1hN7P0CDe=05eMG@1WIfL0DU9B06a/K01[Jf/0GemO055AD@1GEeL0JFUY08f=S@27QhL0OWin08^;
R`1YJFT0Jf][07QhN01KFe/0KVi^059BDP1EEED0K6a/06m_K`1eMGD0Jf][07EeM@24Q8@0KFe]07If
MP1gMgL205QHF006069RHP1_Kfl0GemO061PH01hN7P0TY:B3_ooool0@01?Cdl0OWin05UIF@13@d<0
E5AD04=3@`19BDT0BTY:03De=@0Y:BT0:2PX02/[:`0d=3@0EeMG05UIF@0k>c/0?3`l03De=@1SHf<0
ATI603`l?015ADD0F5QH034a<@1=CDd0=CDe04I6AP1;Bd/0>CTi055AD@1IFET09RHV04e=C@16ATH0
Jf][05=CD`1BDU80GEeM061PH01_Kfl0ADE50451@@1FEUH0?Shn059BDP2DU9@0FUYJ05=CD`1:BTX0
Fe]K04e=C@1RHV80J6QX06YZJP1NGUh0IVIV04Q8B01QHF40JVYZ06UYJ@1UIFD0D51@05QHF01EEED2
07MgM`1:05mOG`1fMWH0Kfm_06UYJ@1aLG40GemO07AdM01WIfL0MWIf07]kN`1jNWX0N7Qh07inOP1R
HV80JFUY071`L01[Jf/0SXj>05YJFP1`L700CDe=05YJFP1cLg<0GemO05YJFP0j>SX0PX:206UYJ@1?
Cdl0@4100492@P1dM7@0B4Q8069RHP1:BTX0LG5a05ADE01FEUH0NGUi04i>CP1QHF40Lg=c034a<@1B
DU80EUIF07IfMP2DU9@0Fe]K05MGE`1HF5P0@41004i>CP1ZJVX0BTY:05iNGP1OGel0SXj>03@d=01_
Kfl0PX:204I6AP1EEED0G5aL04I6AP1CDe<0I6AT055AD@1iNGT0?Cdm04a<C00o?cl0SXj>05UIF@0m
?Cd>ooooo`0605IFEP1MGEd0Jf][06m_K`1ZJVX0I6AT0P0n?Sh04P11@D40De=C06i^KP2f][H0WIfM
05QHF01KFe/0JVYZ06ATI01]KFd0MgMg07emO@2GUiL0X:2P08V9R@2:RXX0:2PX01LG5`<061PH00@0
I6AT0:f][@2][Jd071`L0P0G5aL0FP0H61P08b<S034a<@0T92@0<c<c05EEE@1QHF40RHV907]kN`1B
DU80LW9b051@D01NGUh0VYZJ05ADE014A4@0NWYj055AD@0n?Sh0C4a<05IFEP1VIVH0KVi^04m?C`1T
I6@0Ng]k092@T02ATI40/k>c0;Bd]01jNWX0DU9B05iNGP1?Cdl0=CDe02XZ:P27QhL0_Kfm09:BTP1V
IVH0KFe]08>3P`22PX80IfMW06QXJ00Z:RX0820P03/k>`1:BTX0PX:206UYJ@1@D500AdM704i>CP21
PH40LG5a06a/K02ATI40X:2P06YZJP29RHT0GEeM03Ti>@0g=cL0J6QX0<C4a02IVIT0Xj>S0:2PX01N
GUh0>c/k07EeM@1LG5`0SXj>0;Zj^P2@T900F5QH055AD@1IFET0MGEe08n?S`1RHV80OWin079bLP2D
U9@0WinO08b<S010@400];Bd055AD@80KFe]00<0QXJ608f=S@2=SHd00P0U9BD0202/[:`0OWin06YZ
JP1>CTh0[Jf]05EEE@1?Cdl0M7Ad5?ooool001?ooooo00X0RHV906i^KP27QhL0Cdm?07=cL`0`<300
E5AD071`L01?Cdl0I6AT0P1SHf<0@P1kNg/0I6AT07emO@1OGel0EUIF05mOG`1`L700E5AD05iNGP1`
L700@41007moO`1/K6`0B4Q808j>SP1EEED0CDe=09^KV`17AdL0ADE505YJFP1@D500GemO065QH@1X
J6P0JVYZ06EUI@1eMGD0Fe]K05eMG@1eMGD0H61P05QHF01]KFd0I6AT04a<C01SHf<0O7al05EEE@1S
Hf<0U9BD03dm?@1hN7P0Ti>C0492@P1aLG40/;2`03`l?01cLg<0UYJF0451@@1fMWH0SXj>04];B`2<
S8`0NWYj05IFEP1jNWX0Vi^K051@D01]KFd0Ogmo05ADE01]KFd0MGEe06][J`80Kfm_01P0K6a/09>C
T`28R8P0R8R808^;R`1XJ6P0R8R806m_K`1ADE40RHV9065QH@21PH40UIFE06ATI01oOgl0Q8B406YZ
JP2;Rh/0JFUY07UiN@2CTi<0I6AT05mOG`2WYjL206i^KP0Q06a/K01DE5@0Ti>C055AD@1TI6@0RXZ:
051@D01]KFd0NGUi0:VYZ@2PX:00NWYj06m_K`1eMGD0L71`06UYJ@1UIFD0SXj>05YJFP1SHf<0Kfm_
05UIF@1BDU80L71`06][J`1IFET0I6AT05EEE@1QHF40Jf][03dm?@1/K6`0IfMW00kooooo00H0DE5A
07UiN@1RHV80F5QH04U9B@1DE5@205eMG@1007alO01MGEd0@41003lo?`11@D40@T9203hn?P1=CDd0
GemO06MWI`1dM7@0JFUY04];B`1TI6@0GUiN05UIF@1]KFd0G5aL05ADE01FEUH0GUiN069RHP1ADE40
EEEE04m?C`1]KFd0QHF506EUI@2MWId0Ogmo079bLP1?Cdl0Rh^;07moO`1HF5P0KVi^05aLG01dM7@0
Hf=S07QhN01<C4`0EEEE061PH01>CTh0E5AD05UIF@1RHV80Hf=S05aLG01BDU80G5aL05ADE01kNg/0
EEEE05UIF@1_Kfl0PX:208>3P`1KFe/0OWin07AdM016ATH0Kfm_055AD@80CDe=0480JVYZ05IFEP1T
I6@0GEeM05QHF01OGel0I6AT05]KF`1SHf<0De=C051@D016ATH0G5aL06QXJ01KFe/0GUiN05aLG01J
FUX0F5QH04I6AP1hN7P0Ogmo04i>CP12@T80GUiN05YJFP1iNGT0Ph>304];B`1EEED0PX:205eMG@1Y
JFT0Jf][051@D01CDe<0JFUY05ADE02=SHd0PH6108J6QP1[Jf/0JVYZ07QhN01]KFd0HV9R0861P@1Y
JFT0N7Qh08^;R`1HF5P0IVIV06UYJ@26QXH0I6AT04a<C01OGel0SXj>04];B`1IFET0IfMW08B4Q01Z
JVX0IVIV0861P@2FUYH>ooooo`1P06QXJ01bLW80A4A4059BDP14A4@0BDU904e=C@13@d<0>3Ph01dM
7@0`<300<C4a02/[:`1=CDd0G5aL03<c<`0f=SH09bLW05]KF`1?Cdl0=CDe05MGE`18B4P0=SHf05UI
F@0[:b/0<30`04Q8B010@400EEEE04m?C`0W9bL0D51@04e=C@1^KVh0CDe=065QH@1_Kfl0F5QH07in
OP1@D500<c<c04Q8B00h>3P0FUYJ07emO@1FEUH0HF5Q04Y:BP1>CTh0D51@05mOG`1bLW80KFe]06QX
J01NGUh0A4A4061PH01aLG40KVi^06ATI01BDU80GUiN05eMG@1]KFd0KVi^05IFEP1aLG40HV9R07If
MP1mOGd0KVi^07]kN`1QHF40O7al07UiN@1JFUX0OGem08b<S01PH600GemO06m_K`1eMGD0JFUY06][
J`1_Kfl0BDU904m?C`1]KFd0H61P06ATI015ADD0O7al08J6QP1TI6@0CTi>0P1LG5`0401ADE40L71`
065QH@1gMgL0DU9B06IVIP1gMgL0Fe]K07emO@25QHD0<S8b055AD@1WIfL0HF5Q06EUI@1fMWH2059B
DP0:04e=C@1<C4`0GUiN05YJFP1GEeL0FEUI08j>SP11@D40PH6107MgM`80IFEU00`0GemO05eMG@1H
F5P0OGem051@D026QXH0Bd];05iNGP0m?Cd0T92@05IFEP0l?3`>ooooo`0T04U9B@10@400De=C0:NW
Y`2UYJD0I6AT03hn?P1oOgl0M7Ad0492@P0W9bL0CDe=0:f][@1lO7`0A4A4096AT@2LW9`0FEUI06a/
K01oOgl0LW9b075aL@0c<c<0=CDe05ADE00^;Rh05aLG01PH600d=3@0HF5Q09NGU`2h^;P0G5aL030`
<00];Bd0<c<c0P0[:b/02P1IFET0BTY:06MWI`1/K6`0GUiN07emO@1BDU80[jn_05iNGP0`<3020861
P@1J06i^KP2CTi<0VYZJ09ZJVP2FUYH0G5aL02d];@0d=3@0HF5Q06=SH`2CTi<0VYZJ0:b/[02h^;P0
^k^k09JFUP1FEUH0<c<c03Lg=`1DE5@0N7Qh0;Rh^02HV9P0JVYZ08f=S@1VIVH0HV9R08F5Q@2QXJ40
ATI601dM7@0O7al0=CDe05MGE`2OWil0JFUY02h^;P0P82007AdM03Xj>P2JVYX0R8R804a<C018B4P0
KFe]05mOG`21PH40BTY:06i^KP2`/;00Q8B407alO02DU9@0Ogmo071`L03=cLd0YZJV0:ZZZP2UYJD0
M7Ad0820P015ADD0F5QH05]KF`1jNWX0/[:b0;Fe]@1KFe/0TY:B0;^k^`37alL0>SXj09:BTP2f][H0
>CTi03`l?01KFe/0Q8B409NGU`1fMWH0Rh^;03Xj>P1jNWX0@d=309VIV@2[Zj/0[jn_05mOG`1dM7@0
KVi^5?ooool001?ooooo08/0S8b<07IfMP1nOWh0QhN705IFEP1CDe<0Lg=c03Hf=P1NGUh0D51@05AD
E01SHf<0@41006ATI01jNWX0?3`l06][J`1WIfL0D51@06e]K@1TI6@0E5AD07IfMP1VIVH0E5AD06MW
I`1nOWh0DU9B05eMG@1JFUX0G5aL0492@P1XJ6P0IVIV04e=C@1FEUH0Lg=c03De=@1SHf<0QhN703lo
?`1NGUh0TY:B03/k>`1/K6`0OGem04e=C@1PH600FUYJ05ADE01KFe/0L71`06a/K01`L700GemO06e]
K@1kNg/0GemO07moO`1iNGT0M7Ad07IfMP27QhL0JVYZ0820P02<S8`0Cdm?07UiN@2@T900>c/k08b<
S027QhL0?Cdm0861P@1hN7P0>SXj07]kN`1nOWh0@T920:FUY@20P800EeMG0:BTY02EUID0HF5Q0:NW
Y`2ATI40I6AT08V9R@1_Kfl0HV9R07alO01]KFd0QhN708b<S01cLg<0][Jf06YZJP1aLG40Vi^K06YZ
JP2IVIT0GUiN08:2PP2<S8`0NGUi08>3P`1YJFT0PH6108f=S@1MGEd0NGUi07moO`1LG5`0DU9B05mO
G`1mOGd0S8b<09FEU@18B4P0Hf=S09FEU@0m?Cd0KFe]08:2PP0o?cl0RXZ:07emO@17AdL0OGem06QX
J00j>SX0Lg=c08V9R@1?Cdl0MGEe071`L01@D500K6a/00kooooo0140D51@08:2PP2IVIT0K6a/055A
D@1UIFD0HF5Q061PH01OGel0@D5103dm?@0n?Sh0B4Q80492@P1?Cdl0De=C05QHF002071`L00W04U9
B@0j>SX0Hf=S04U9B@0m?Cd0@T9204a<C013@d<0E5AD05IFEP1EEED0CDe=04Q8B01:BTX0C4a<061P
H01JFUX0Jf][05ADE01/K6`0@T9208b<S01NGUh0AdM704i>CP16ATH0@D5104A4A01EEED0@41004A4
A015ADD0?3`l04A4A016ATH0FUYJ04];B`1TI6@0DE5A0080F5QH01@0K6a/065QH@1EEED0JVYZ08N7
Q`1HF5P0J6QX06][J`20P800SHf=0820P01bLW80H61P055AD@1nOWh0H61P05ADE01XJ6P0CTi>08^;
R`80GemO02l0MGEe06][J`1DE5@0EEEE05mOG`1gMgL0HV9R055AD@1?Cdl0HF5Q06EUI@1[Jf/0S8b<
06e]K@1SHf<0EeMG06YZJP1XJ6P0Ti>C08f=S@18B4P0J6QX06e]K@1eMGD0GEeM05mOG`1BDU80CTi>
05eMG@1JFUX0Q8B406a/K01aLG40P82005eMG@1/K6`0QHF506QXJ01`L700Bd];061PH01_Kfl0H61P
06MWI`1nOWh0Ti>C09:BTP0207QhN00908^;R`1@D500CDe=05aLG01eMGD0IVIV06e]K@27QhL0[Jf]
00kooooo08`0HF5Q06QXJ01;Bd/0?3`l03Ph>00i>CT0>c/k03hn?P0Y:BT051@D02l_;`0W9bL0=cLg
04Q8B01SHf<0>SXj038b<P0i>CT0Cdm?04];B`14A4@0ATI604m?C`0h>3P0B4Q8028R8P15ADD0@d=3
03<c<`1RHV80ATI602HV9P1OGel0@d=307YjNP1@D500CTi>08J6QP1SHf<0TI6A051@D00g=cL0KVi^
03<c<`1IFET0P82006a/K01JFUX0HV9R04I6AP1>CTh0DU9B05EEE@1bLW80Fe]K05EEE@18B4P0FEUI
05iNGP1IFET0G5aL04Q8B011@D40BDU9079bLP14A4@0D51@07UiN@1cLg<0De=C0861P@1FEUH0FEUI
07]kN`22PX80MWIf07UiN@1lO7`0MWIf09BDU01RHV80QhN707moO`1YJFT0UIFE06e]K@13@d<0GEeM
06a/K01SHf<0G5aL059BDP1YJFT0S8b<07=cL`1ADE40CDe=05UIF@1SHf<0KFe]03`l?01aLG40Fe]K
05IFEP1_Kfl0FEUI06][J`1lO7`0C4a<03`l?01FEUH0L71`06MWI`1dM7@0Fe]K069RHP0n?Sh0G5aL
06e]K@1VIVH0GEeM04e=C@2PX:00>3Ph06e]K@24Q8@0HV9R04U9B@17AdL0JFUY059BDP1ZJVX0Hf=S
09ZJVP11@D40Fe]K04Q8B01kNg/0Cdm?03lo?`kooooo00T0M7Ad030`<00d=3@0JVYZ065QH@0/;2`0
7alO02d];@0W9bL00P0R8R80601<C4`0A4A405eMG@2SXj<0U9BD0:FUY@21PH40Q8B4079bLP1nOWh0
V9RH04a<C00Z:RX0;Rh^04a<C01DE5@0=3@d02LW9`0^;Rh0Shn?0;Zj^P2UYJD0NWYj04A4A0809bLW
06L0<C4a05UIF@0`<300IVIV06][J`1FEUH0RXZ:07AdM01cLg<0HV9R05aLG02BTY80ADE502d];@1;
Bd/0WIfM0<G5a@2a/K40MgMg03Ph>00_;bl09bLW02h^;P2^[Zh0VIVI0;jn_P2RXZ80TY:B09BDU01L
G5`0EeMG05]KF`18B4P0SHf=09VIV@2QXJ40D51@055AD@1?Cdl0MGEe0;Fe]@2XZ:P0Fe]K02h^;P1F
EUH0IfMW04i>CP2;Rh/0A4A403hn?P0N7Qh06a/K030`<02m_Kd0/[:b02`/;00_;bl0UYJF07]kN`2R
XZ80WinO08Z:RP26QXH0=cLg02XZ:P16ATH0E5AD07EeM@23Ph<0TI6A0=3@d02=SHd0G5aL05MGE`1Y
JFT0Cdm?05iNGP2e]KD0[jn_0;6a/@1aLG40I6AT0:ZZZP2o_kl0W9bL07QhN01UIFD0GUiN07IfMP1l
O7`0>c/k07moO`21PH40OGem0:^[Z`1BDU80CDe=0:^[Z`2=SHd0WIfM03@d=01HF5P0EUIF01Cooooo
000Cooooo`1`08V9R@1mOGd0QXJ607inOP1/K6`0H61P071`L01ZJVX0FUYJ04i>CP1jNWX0GEeM05iN
GP2<S8`0G5aL079bLP2<S8`0DE5A065QH@1UIFD0G5aL06ATI01HF5P0OGem06YZJP1;Bd/0PX:205]K
F`1LG5`0I6AT05YJFP13@d<0EeMG06][J`14A4@0LW9b08F5Q@0n?Sh0KFe]0861P@17AdL0I6AT07]k
N`14A4@0JFUY06QXJ01DE5@0Hf=S061PH01]KFd0IfMW06IVIP1XJ6P0AdM706IVIP22PX80F5QH06MW
I`2UYJD0EeMG07AdM024Q8@0HV9R07moO`23Ph<0LG5a06m_K`21PH40J6QX06=SH`2ATI40GemO069R
HP2=SHd0IFEU06MWI`23Ph<0H61P06UYJ@1lO7`0J6QX08>3P`2NWYh0NGUi08F5Q@1/K6`0MgMg07Ee
M@1SHf<0PH6106QXJ01`L700RXZ:07MgM`1hN7P0NGUi08f=S@1cLg<0HF5Q07UiN@23Ph<0QhN705QH
F02MWId0MGEe06EUI@1lO7`0KFe]07QhN01XJ6P0GUiN07MgM`80H61P00d0J6QX08N7Q`2OWil0UIFE
08V9R@1WIfL0Lg=c06m_K`1HF5P0Rh^;05iNGP1JFUX0RXZ:0080G5aL00X0S8b<07AdM01KFe/0HV9R
05=CD`1eMGD0MgMg051@D01JFUX0IFEU3_ooool01@1EEED0RHV9075aL@1bLW80BDU90080FEUI0180
DE5A06a/K01KFe/0@d=30451@@1;Bd/0?Cdm051@D01HF5P0AdM705iNGP1RHV80DU9B0410@01KFe/0
EeMG03lo?`1VIVH204e=C@0O04a<C01?Cdl0CDe=059BDP1;Bd/0B4Q8059BDP1RHV80De=C07AdM01L
G5`0I6AT05IFEP29RHT0K6a/05MGE`1ADE40A4A404Q8B014A4@0E5AD04M7A`12@T80Bd];03lo?`12
@T80AdM705aLG01>CTh0K6a/059BDP02051@D01@05EEE@1:BTX0<S8b069RHP23Ph<0D51@06UYJ@1R
HV80KVi^051@D00k>c/0HV9R05aLG01TI6@0Lg=c06UYJ@1MGEd0HF5Q051@D01_Kfl0GEeM05IFEP1Y
JFT0L71`06YZJP1EEED0LW9b07alO01bLW80LG5a06][J`1aLG40KVi^06YZJP1iNGT0LW9b06MWI`1U
IFD0Kfm_06e]K@2GUiL0O7al05MGE`1QHF40LW9b05IFEP1VIVH0IfMW05iNGP1LG5`0JVYZ05aLG020
P800Hf=S05YJFP1OGel0K6a/06m_K`1kNg/0H61P07emO@1YJFT0QhN707QhN01IFET0NGUi069RHP1m
OGd0PX:208f=S@1dM7@0S8b<05=CD`1QHF40K6a/06m_K`1QHF40IfMW07]kN`2MWId>ooooo`0k06][
J`1OGel0@D5104A4A01;Bd/0A4A4051@D01>CTh0>SXj02LW9`0c<c<0:2PX03Ti>@17AdL0JFUY03Xj
>P0k>c/0<30`03lo?`1=CDd0>c/k05ADE01CDe<0=SHf05=CD`0T92@0A4A40451@@13@d<0GEeM04A4
A00_;bl0FUYJ03dm?@1QHF40@D51055AD@26QXH0@d=309FEU@19BDT0>CTi05QHF00d=3@0J6QX07]k
N`1cLg<0HV9R051@D01/K6`0CDe=04a<C01cLg<0JVYZ06][J`1HF5P0D51@04i>CP1aLG400P1SHf<0
C`1QHF40EEEE07AdM01bLW80CTi>07=cL`1fMWH0Lg=c06i^KP21PH40K6a/05UIF@1cLg<0Shn?07mo
O`1`L700RXZ:08F5Q@2;Rh/0KFe]09BDU01dM7@0KVi^09VIV@1`L700CDe=059BDP1QHF40Lg=c05MG
E`1OGel0Kfm_08F5Q@1KFe/0JFUY04i>CP1DE5@0LG5a07AdM01FEUH0MgMg071`L01EEED0JVYZ04m?
C`1mOGd0K6a/06=SH`16ATH0HF5Q06MWI`1LG5`0MgMg065QH@1GEeL0BTY:05mOG`1CDe<0GemO05AD
E01:BTX0MgMg03Xj>P1`L700JFUY06QXJ01BDU80A4A406IVIP1MGEd0K6a/069RHP1kNg/0D51@05MG
E`10@400H61P04Y:BP0f=SH03_ooool0I01oOgl0J6QX07AdM02GUiL0@41002<S8`0J6QX05ADE01dM
7@0N7Qh08R8R061PH00^;Rh0>3Ph08Z:RP2=SHd0NGUi04=3@`1=CDd0E5AD08^;R`1/K6`0DU9B04e=
C@0m?Cd0DE5A07=cL`0e=CD0<S8b0451@@2<S8`0YJFU09^KV`2PX:00UiNG08j>SP1[Jf/0FEUI05AD
E01aLG40MGEe05]KF`20P800Shn?08f=S@1OGel0;bl_059BDP1iNGT0Rh^;07=cL`1NGUh0LG5a08j>
SP25QHD0R8R805MGE`0o?cl0BDU906UYJ@2TY:@0J6QX07=cL`1^KVh0?Shn03Lg=`1HF5P0LW9b04a<
C01`L700TY:B07=cL`1>CTh0820P02`/;01TI6@0IfMW06a/K028R8P0NWYj0;Zj^P2n_[h0MWIf08>3
P`2JVYX0IFEU06e]K@1HF5P0DE5A08R8R02>SXh0QXJ6071`L023Ph<0a<C408B4Q02QXJ40NGUi06][
J`1^KVh202PX:00V0492@P2JVYX0SXj>09JFUP30`<00YJFU0:>SX`1bLW80C4a<04E5A@1cLg<0Kfm_
0;2`/02b/[80NGUi0:JVYP2@T900Hf=S0861P@2VYZH0MGEe06][J`2OWil0ATI609RHV011@D40AdM7
09:BTP2^[Zh0_[jn0820P02FUYH0Y:BT0861P@2VYZH0:b/[02XZ:P0n?ShDooooo`004oooool02@27
QhL0PX:207YjNP1MGEd0Ng]k0492@P1VIVH0JVYZ05ADE00205eMG@0^065QH@1WIfL0MWIf06i^KP1R
HV80IVIV07IfMP1KFe/0EUIF08>3P`1HF5P0DU9B079bLP1HF5P0GUiN096AT@0o?cl0JVYZ06MWI`1;
Bd/0GEeM04I6AP1MGEd0N7Qh05EEE@1UIFD0GemO06=SH`1hN7P0IFEU06][J`1^KVh0FUYJ05=CD`1d
M7@0Q8B40492@P1eMGD0RXZ:03Ti>@1fMWH0Ogmo03`l?01gMgL0OWin059BDP80M7Ad0500Fe]K079b
LP1dM7@0FEUI09>CT`1VIVH0KVi^08V9R@1jNWX0SHf=06m_K`1cLg<0KFe]06][J`23Ph<0OWin05iN
GP1XJ6P0NGUi05ADE01SHf<0MGEe06QXJ01/K6`0Rh^;07YjNP1[Jf/0_;bl06QXJ01WIfL0WIfM05YJ
FP1nOWh0R8R805YJFP27QhL0HF5Q075aL@2?Shl0EEEE08>3P`1mOGd0LG5a08:2PP20P800I6AT08b<
S01BDU80I6AT08N7Q`0f=SH0OWin06a/K01XJ6P0HF5Q09>CT`2/[:`0VIVI08F5Q@1YJFT0FEUI05aL
G027QhL0E5AD055AD@1]KFd0Fe]K05MGE`1^KVh0DU9B06QXJ01nOWh0DU9B05IFEP1<C4`0L71`06a/
K01LG5`0EEEE05]KF`kooooo04d0Bd];05iNGP1^KVh0LW9b03Xj>P1FEUH0I6AT04i>CP1/K6`0HF5Q
04i>CP1:BTX0ATI603Xj>P1BDU80EEEE05=CD`1IFET0FUYJ04M7A`0k>c/0HV9R05MGE`1;Bd/0IfMW
06][J`1:BTX0GUiN05mOG`1KFe/0FUYJ051@D01DE5@0H61P07QhN01RHV80LW9b06e]K@1eMGD0J6QX
09BDU021PH40G5aL061PH01ADE40G5aL04];B`1OGel0DE5A04U9B@1@D500ATI604m?C`18B4P0HF5Q
05ADE01jNWX0FUYJ04U9B@1?Cdl0HV9R04a<C011@D40JFUY08^;R`1UIFD0Kfm_08R8R01iNGT0=CDe
061PH00m?Cd0ATI604Q8B01_Kfl0CTi>04U9B@0204Q8B00905EEE@1>CTh0C4a<05MGE`1;Bd/0ATI6
04A4A01?Cdl0JVYZ0080GUiN0380Fe]K061PH01GEeL0CDe=05ADE01dM7@0GEeM06e]K@1TI6@0G5aL
06ATI01YJFT0DU9B04e=C@1WIfL0ADE5061PH01QHF40BDU905ADE01RHV80Bd];06i^KP1FEUH0P820
075aL@1OGel0KVi^08B4Q01GEeL0O7al06=SH`28R8P0NGUi05UIF@1eMGD0LG5a07AdM02WYjL0U9BD
06YZJP2MWId0CDe=05UIF@1^KVh0JFUY05eMG@1WIfL0NWYj08F5Q@kooooo0P1SHf<0L014A4@0De=C
03dm?@1;Bd/0;Rh^03<c<`0[:b/05ADE02TY:@0N7Qh092@T04Y:BP1WIfL0<S8b03Ti>@0/;2`0>CTi
051@D011@D40Bd];04e=C@1EEED0DE5A0410@01<C4`0?Cdm069RHP1TI6@0BDU90492@P1RHV80ATI6
065QH@10@400I6AT071`L01?Cdl0YZJV05UIF@15ADD0KFe]0492@P1UIFD0L71`08R8R01MGEd0G5aL
05IFEP1;Bd/0D51@06IVIP1PH600J6QX05mOG`16ATH0IVIV069RHP1EEED0KFe]05YJFP1LG5`0Bd];
07alO01;Bd/0Cdm?05UIF@1[Jf/0H61P06e]K@1bLW80De=C075aL@20P800GEeM05iNGP2HV9P0IVIV
08^;R`1UIFD0WYjN06MWI`1LG5`0][Jf05QHF010@400EUIF03lo?`1dM7@0ADE5059BDP1lO7`0SXj>
04m?C`1nOWh0GEeM03@d=022PX80NGUi069RHP1<C4`0N7Qh05QHF01TI6@0DU9B09NGU`1^KVh0JFUY
04I6AP1`L700IfMW05=CD`26QXH205EEE@0806=SH`1MGEd0BTY:07inOP15ADD0>SXj07emO@1JFUX2
05UIF@0>08>3P`13@d<0BDU907=cL`1VIVH0Ph>306EUI@1<C4`0Cdm?05EEE@1dM7@0DE5A051@D00n
?Sh>ooooo`2<0820P02DU9@0^k^k0;Jf]P0_;bl07QhN01HF5P0E5AD0ADE5038b<P1:BTX0EeMG034a
<@1[Jf/0PH6107IfMP1jNWX0=cLg03Hf=P0];Bd0DE5A04Q8B017AdL0EUIF07moO`1CDe<0HV9R0861
P@0_;bl0BDU906i^KP1ZJVX0HV9R08N7Q`1bLW80G5aL092@T026QXH0V9RH0<;2`P2HV9P0IFEU05AD
E019BDT0EUIF08R8R01^KVh0TI6A06MWI`2BTY80VIVI09fMW@22PX80>c/k05MGE`1NGUh0De=C04E5
A@0];Bd0=CDe0;Jf]P2OWil0Q8B408f=S@1=CDd09RHV02XZ:P0f=SH0ATI604e=C@1OGel0EeMG059B
DP0i>CT0<C4a02`/;00Z:RX0<C4a069RHP2WYjL0e=CD0;fm_@2RXZ80DU9B055AD@1aLG40De=C03Lg
=`1kNg/0];Bd0;2`/02IVIT0Z:RX0<O7a`33`l<0]kNg0<O7a`1eMGD0LG5a07]kN`1XJ6P0EeMG03dm
?@1]KFd0WinO0:FUY@21PH40QHF5069RHP0o?cl0BTY:06m_K`0V9RH0EeMG05mOG`1]KFd0<30`07If
MP11@D409RHV05EEE@1fMWH0clo?0:FUY@1gMgL0BTY:06e]K@26QXH0Rh^;06ATI02g]kL0W9bL0:n_
[`2a/K40WYjN09^KV`35aLD0MgMg0492@P16ATHDooooo`004oooool0R`1jNWX0O7al06a/K02CTi<0
D51@079bLP1cLg<0A4A407moO`0j>SX0P82005iNGP0j>SX0PH6106e]K@1BDU80LG5a06MWI`17AdL0
M7Ad065QH@1LG5`0KVi^05ADE01XJ6P0EUIF06][J`1UIFD0EUIF069RHP1`L700EeMG06MWI`1lO7`0
DU9B06EUI@2:RXX0@d=306MWI`2=SHd0@d=305YJFP2ATI40?clo06ATI01iNGT0ATI6061PH01iNGT0
C4a<075aL@1YJFT0HV9R05QHF01PH600OWin06ATI01JFUX0QHF5065QH@1UIFD0O7al09NGU`1YJFT0
QHF508B4Q01CDe<0PH610820P017AdL0UYJF07=cL`1OGel0/;2`06MWI`1[Jf/0Vi^K06QXJ01HF5P0
T92@06m_K`1bLW80Z:RX07moO`1`L700S8b<07YjNP2GUiL0IfMW06a/K01kNg/0K6a/07=cL`2EUID0
KVi^07IfMP2PX:00IVIV075aL@2:RXX0K6a/0::RXP1ADE40UiNG096AT@1HF5P0Jf][07alO029RHT0
M7Ad04Y:BP1`L700T92@04A4A00c<c<0CDe=05MGE`1eMGD0SXj>04U9B@1kNg/0X:2P05ADE024Q8@0
De=C05]KF`2<S8`0H61P051@D020P800GUiN04M7A`1PH600E5AD04];B`1aLG40EeMG055AD@20P800
3_ooool0D`16ATH0IVIV06][J`1[Jf/0>3Ph05YJFP1HF5P0HV9R06YZJP1RHV80EUIF051@D01GEeL0
ADE504Q8B01BDU80EeMG07QhN01[Jf/0@d=303Lg=`1NGUh0FEUI04i>CP1_Kfl0DU9B04];B`1LG5`0
F5QH04a<C01GEeL0BTY:05MGE`1QHF40MWIf065QH@2@T900KFe]06IVIP1WIfL0ZZZZ07]kN`1[Jf/0
GemO059BDP1LG5`0ATI605]KF`13@d<0BTY:05EEE@15ADD0?Shn04U9B@1IFET0EEEE06=SH`1QHF40
Fe]K05=CD`1NGUh0De=C03dm?@1TI6@0LW9b069RHP1mOGd0Kfm_08F5Q@1aLG40I6AT05EEE@1?Cdl0
EEEE08J6QP1XJ6P0GUiN06m_K`1YJFT0MGEe065QH@19BDT0LW9b0080EUIF03L0CTi>06ATI025QHD0
Hf=S061PH01CDe<0Jf][05QHF014A4@0Hf=S05]KF`1YJFT0CTi>06YZJP1ADE40Q8B406UYJ@1;Bd/0
De=C06ATI01RHV80IVIV06UYJ@1UIFD0JVYZ07EeM@1OGel0NGUi06ATI01iNGT0PX:207AdM01]KFd0
MWIf06m_K`25QHD0Ogmo0::RXP22PX80Fe]K06=SH`1dM7@0Ph>309bLW02QXJ40P8200:ZZZP1IFET0
GUiN06YZJP1hN7P0Hf=S06IVIP2MWId0YZJV00kooooo07l0P82006m_K`1IFET0C4a<03lo?`16ATH0
@T9204=3@`0e=CD0<C4a03<c<`0M7Ad0?Cdm03`l?01UIFD0<c<c03dm?@0i>CT09bLW05QHF019BDT0
MWIf04];B`17AdL0DU9B03hn?P17AdL0:2PX04A4A01VIVH0>c/k03Hf=P1PH600?clo04I6AP0];Bd0
B4Q806IVIP1EEED0ZZZZ055AD@1SHf<0K6a/034a<@1UIFD0MGEe09FEU@1PH600Cdm?05YJFP1?Cdl0
>c/k06EUI@1ADE40EEEE05QHF00j>SX0B4Q8061PH01=CDd0G5aL06=SH`1<C4`0Lg=c06MWI`11@D40
LG5a04a<C01JFUX0Jf][06e]K@1VIVH0HF5Q079bLP1SHf<0JFUY08F5Q@2MWId0LW9b0:6QX@1OGel0
LW9b07AdM01WIfL0XJ6Q06][J`19BDT0E5AD05]KF`1eMGD0BTY:05UIF@1EEED0P82004I6AP1TI6@0
?Shn02TY:@27QhL0Jf][05MGE`1NGUh0N7Qh04U9B@1OGel0EeMG08V9R@1jNWX0JFUY038b<P1[Jf/0
EUIF06IVIP2=SHd0O7al06IVIP1BDU80MGEe05aLG01kNg/0Cdm?05MGE`1dM7@0EEEE06][J`1LG5`0
L71`0080Cdm?00/0NWYj061PH023Ph<0GemO05EEE@1IFET0GEeM05mOG`1RHV80HF5Q04U9B@0>oooo
o`0H08B4Q029RHT0Fe]K0:FUY@2:RXX08R8R01TI6@0K6a/0<C4a05QHF02ZZZX0XZ:R07]kN`29RHT0
Shn?04I6AP0l?3`0K6a/03Ph>00/;2`0DU9B05UIF@0W9bL0=3@d0P1iNGT03`0g=cL0GUiN04];B`0d
=3@09RHV024Q8@0Y:BT0<C4a01DE5@0F5QH0<c<c03hn?P1EEED0`l?309ZJVP0203dm?@1405UIF@0n
?Sh0?Cdm07moO`2>SXh0T92@06=SH`1XJ6P0VYZJ08J6QP1>CTh0B4Q804M7A`0k>c/0@D5106UYJ@1o
Ogl0[jn_09nOW`1`L700MWIf059BDP1:BTX09RHV02LW9`0d=3@0JFUY04Q8B01?Cdl0EUIF02d];@0Z
:RX092@T02/[:`11@D40KVi^09nOW`2RXZ80]KFe0;no_`2SXj<0Ogmo04m?C`1;Bd/0CTi>06e]K@20
P800YjNW0:ZZZP2m_Kd0/k>c08f=S@2PX:00WIfM04Q8B013@d<0EeMG09fMW@2HV9P0Rh^;09BDU02m
_Kd0cLg=09jNWP24Q8@0?Cdm03`l?00a<C4203Lg=`0K04e=C@1CDe<0D51@034a<@1mOGd0=cLg02@T
901CDe<0ADE506e]K@2WYjL0M7Ad08R8R01VIVH0QXJ60:b/[030`<00/;2`09ZJVP28R8P0/;2`0:f]
[@2HV9P0NWYj04=3@`15ADD0P82001Cooooo000Cooooo`1O09:BTP1_Kfl0PX:205MGE`1nOWh0>3Ph
06MWI`1nOWh0Bd];05MGE`1oOgl0BDU905mOG`1aLG40E5AD06MWI`1FEUH0IVIV06m_K`1?Cdl0Kfm_
06IVIP16ATH0OWin06QXJ01:BTX0OGem065QH@1:BTX0Fe]K06e]K@1DE5@0IVIV06=SH`1aLG40MWIf
06QXJ01LG5`0K6a/069RHP1CDe<0O7al059BDP1UIFD0I6AT05iNGP1XJ6P0EUIF05]KF`1UIFD0?Shn
05UIF@1nOWh0@T9206UYJ@26QXH0>SXj05iNGP2FUYH0ADE5065QH@1lO7`0ADE507MgM`1oOgl0GemO
08n?S`1lO7`0LG5a09RHV02;Rh/0HV9R06IVIP29RHT0LW9b05YJFP1OGel0QXJ606EUI@1IFET0MgMg
07EeM@1MGEd0NWYj07inOP1KFe/0ZJVY06UYJ@1ZJVX0Zj^[05ADE01]KFd0Zj^[05]KF`27QhL00P1o
Ogl0:P2>SXh0MWIf07UiN@1iNGT0HV9R07]kN`24Q8@0KVi^092@T01MGEd0IfMW09FEU@0o?cl0Ng]k
06ATI01?Cdl0HV9R0:NWY`3EeMD0QHF506QXJ01_Kfl0GUiN07QhN01aLG40IVIV092@T01KFe/0FUYJ
06e]K@1OGel0EEEE07MgM`23Ph<0J6QX051@D01EEED0JVYZ07emO@1LG5`0FUYJ06MWI`kooooo03@0
Cdm?06IVIP1VIVH0MWIf05ADE01KFe/0I6AT05]KF`1NGUh0J6QX04U9B@1>CTh0EeMG04=3@`1BDU80
FEUI06UYJ@1eMGD0Hf=S03lo?`0b<S80GEeM051@D00o?cl0J6QX05iNGP1@D500EEEE05aLG01;Bd/0
EeMG05EEE@1RHV80EEEE06e]K@1OGel0Q8B407=cL`1lO7`0HV9R08j>SP1aLG40De=C05MGE`13@d<0
Bd];03lo?`1CDe<0@d=304M7A`1BDU80?Cdm0P15ADD0=P1GEeL0Bd];06ATI01?Cdl0Lg=c05YJFP1=
CDd0EUIF03`l?01SHf<0LW9b06UYJ@1ZJVX0Hf=S06e]K@1OGel0H61P06ATI019BDT0EeMG07]kN`1U
IFD0EUIF04m?C`1JFUX0IFEU05YJFP1EEED0Lg=c05UIF@1OGel0A4A4065QH@1gMgL0EEEE05ADE01S
Hf<0J6QX05aLG016ATH0FEUI05ADE01bLW80HV9R05]KF`1IFET0N7Qh05iNGP1ADE40GEeM079bLP1J
FUX0Lg=c07alO080K6a/01h0PX:203dm?@1/K6`0IFEU07YjNP1aLG40LW9b075aL@2;Rh/0Q8B407Ui
N@1bLW80YjNW08j>SP1aLG40Q8B406][J`2>SXh0[jn_09RHV024Q8@0];Bd06UYJ@1LG5`0HV9R06][
J`1NGUh0G5aL08:2PP2_[jl>ooooo`0L0;Fe]@25QHD0FEUI04e=C@0m?Cd0ATI603<c<`0a<C40?clo
034a<@0^;Rh0:BTY02h^;P1SHf<0NGUi02d];@18B4P0@41003Hf=P1:BTX0@T9208b<S00c<c<0HF5Q
05ADE00g=cL0?3`l03Xj>P80De=C06h0>SXj04Y:BP15ADD0@T9203/k>`0o?cl0D51@055AD@17AdL0
Q8B405EEE@18B4P0GUiN05IFEP1DE5@0FEUI09NGU`1EEED0FUYJ055AD@1@D500EEEE06MWI`1ADE40
GEeM06ATI015ADD0GEeM05mOG`1DE5@0GemO05eMG@1JFUX0G5aL07IfMP1>CTh0IFEU04I6AP1HF5P0
VIVI04m?C`1aLG40GEeM069RHP1[Jf/0GEeM07QhN025QHD0SHf=0;2`/01LG5`0KVi^07=cL`1aLG40
];Bd07MgM`1IFET0F5QH05UIF@1dM7@0G5aL05YJFP1SHf<0Q8B404M7A`27QhL0B4Q8030`<02@T900
M7Ad05YJFP1FEUH0Lg=c055AD@1XJ6P0EEEE07]kN`1RHV80Hf=S0492@P1YJFT0Fe]K04a<C01jNWX0
JFUY05=CD`1;Bd/0E5AD04a<C01iNGT0>SXj04e=C@1NGUh0Hf=S05EEE@19BDT0MGEe03Ph>00l?3`0
K6a/04i>CP1dM7@0IFEU06e]K@18B4P0EeMG05]KF`1:BTX0H61P0492@Pkooooo0280>c/k06EUI@1d
M7@0[Jf]0:n_[`2MWId0IfMW04E5A@17AdL0E5AD0:ZZZP23Ph<0L71`05YJFP0g=cL0EEEE03@d=00/
;2`0PX:204U9B@18B4P0EUIF02l_;`0S8b<0KVi^0:n_[`2DU9@0Xj>S06a/K018B4P0=CDe01TI6@0G
5aL0=SHf0P0F5QH01P0Q8B40IfMW06UYJ@1mOGd0HF5Q038b<P80<C4a0600De=C06UYJ@1mOGd0G5aL
05ADE00k>c/0HV9R03`l?01QHF40MgMg06YZJP2JVYX0SHf=04Q8B01kNg/0EEEE08:2PP2MWId0?Cdm
03Lg=`12@T80@410055AD@0[:b/09RHV06=SH`1hN7P0RXZ:06e]K@0];Bd0:RXZ02TY:@0l?3`0GUiN
0861P@1oOgl0T92@04];B`2`/;00UiNG02XZ:P0k>c/07alO03/k>`27QhL0Q8B409>CT`2^[Zh0D51@
07alO01`L700M7Ad07alO013@d<0Cdm?05IFEP2g]kL0SXj>075aL@2n_[h0`<3007YjNP1JFUX0QHF5
02d];@0a<C40=3@d03Lg=`1GEeL0MgMg05eMG@1OGel0T92@0:f][@24Q8@0BTY:06a/K012@T80HF5Q
08R8R02?Shl0KVi^0492@P10@400O7al09RHV02VYZH0ATI6034a<@1RHV80UIFE05iNGP1QHF40CDe=
03Lg=`1@D50Dooooo`004oooool06028R8P0Ogmo071`L01gMgL0Fe]K04E5A@1SHf<0E5AD05YJFP1L
G5`0PH6106][J`15ADD0KVi^06=SH`18B4P0EeMG07UiN@1>CTh0JVYZ05iNGP1BDU80F5QH07moO`80
DU9B0140J6QX055AD@1HF5P0CDe=069RHP1NGUh0G5aL06e]K@1SHf<0EUIF07QhN01@D500EeMG07Yj
NP19BDT0F5QH07]kN`0205IFEP0@092@T01HF5P0FEUI08j>SP1UIFD0Cdm?069RHP1^KVh0EUIF079b
LP1aLG40Fe]K069RHP1bLW80IfMW0820P080K6a/04`0O7al06][J`1nOWh0IVIV071`L024Q8@0FUYJ
07QhN020P800FUYJ07AdM02>SXh0F5QH07IfMP1hN7P0F5QH075aL@1_Kfl0IFEU07alO029RHT0K6a/
06UYJ@24Q8@0Lg=c071`L01nOWh0G5aL07YjNP2@T900MGEe07MgM`1fMWH0OGem06MWI`1CDe<0NGUi
07QhN01dM7@0KVi^07moO`1dM7@0MWIf061PH01aLG40PX:2065QH@1bLW80Hf=S07emO@14A4@0BTY:
05EEE@1ADE40N7Qh07moO`1DE5@0KFe]08j>SP1GEeL0Ogmo06e]K@1CDe<0LW9b06i^KP15ADD0Ogmo
06a/K01;Bd/0FUYJ05UIF@17AdL0Lg=c05MGE`18B4P0IfMW3_ooool06P1NGUh0JFUY06YZJP1iNGT0
C4a<05YJFP1XJ6P0Kfm_06EUI@1TI6@0Bd];04Q8B01ADE40@41005UIF@1OGel0IFEU06e]K@1OGel0
A4A402d];@1GEeL0De=C0451@@1^KVh0Cdm?0P1ADE40K01DE5@0CDe=05eMG@1FEUH0GemO06IVIP1k
Ng/0G5aL07EeM@1nOWh0Jf][06e]K@2UYJD0PH6105YJFP1PH600DE5A05QHF019BDT0GEeM04e=C@1>
CTh0E5AD04M7A`15ADD0@d=305mOG`1@D500IfMW04e=C@1MGEd0EEEE051@D01CDe<0?Shn06=SH`1_
Kfl0Lg=c075aL@1SHf<0Kfm_04];B`1FEUH0DE5A04E5A@1<C4`0Fe]K055AD@1FEUH0I6AT06MWI`1j
NWX0IVIV05aLG01cLg<0G5aL06QXJ01=CDd0GEeM06UYJ@1VIVH0GUiN06ATI01QHF40EeMG04Q8B01H
F5P0G5aL05mOG`1DE5@0Hf=S05EEE@1jNWX0Hf=S03lo?`1<C4`0CTi>05aLG01aLG40Y:BT06UYJ@1Q
HF40MWIf05ADE01[Jf/0JFUY07EeM@1nOWh0K6a/07EeM@2>SXh0IFEU07YjNP1aLG40UYJF09nOW`21
PH40N7Qh07=cL`2NWYh0_kno0:f][@2KVi/0[Jf]05UIF@1JFUX0K6a/07MgM`80GemO00<0Ph>30:6Q
XOooool03Oooool0<@2l_;`0Jf][06=SH`12@T80>CTi0492@P0k>c/0=cLg02PX:00a<C40;bl_01lO
7`0g=cL0BDU9075aL@0g=cL0?Cdm04I6AP0Y:BT0AdM702/[:`14A4@0EEEE051@D00l?3`0Cdm?03Ti
>@0e=CD0B4Q805YJFP0o?cl0DE5A05ADE01=CDd0>3Ph051@D01<C4`0@T9207EeM@25QHD0GEeM05mO
G`1HF5P0@T9206YZJP1gMgL0YJFU05YJFP1BDU800P1KFe/0>01LG5`0KFe]05UIF@1QHF40H61P05eM
G@1DE5@0H61P07=cL`1eMGD0Lg=c065QH@1_Kfl0Q8B404M7A`1/K6`0F5QH05YJFP2DU9@0F5QH079b
LP24Q8@0IfMW05EEE@1YJFT0N7Qh0820P01^KVh0^[Zj071`L02ZZZX0K6a/06MWI`2QXJ40JFUY05IF
EP1EEED0E5AD071`L01PH600IFEU06=SH`2EUID0D51@07YjNP1BDU80;2`/092@T01SHf<0NGUi05eM
G@1gMgL0CTi>06IVIP1`L700Lg=c0P1_Kfl07`0i>CT0IVIV05iNGP1SHf<0Q8B408>3P`1WIfL0GEeM
06=SH`1DE5@0Ph>304a<C01RHV80GEeM06YZJP1UIFD0FEUI079bLP15ADD0DU9B06QXJ01>CTh0LW9b
07YjNP1[Jf/0ADE504M7A`1PH600@T92061PH01KFe/03_ooool07P11@D40R8R80:ZZZP2XZ:P0YJFU
0:n_[`2>SXh0D51@05=CD`2YZJT0PX:20861P@2ATI40FEUI02LW9`0a<C40;bl_01XJ6P0_;bl0WinO
06YZJP1FEUH0H61P06i^KP2?Shl0_Kfm06UYJ@10@400GUiN05QHF080@D5100X0@410055AD@19BDT0
AdM704U9B@0n?Sh0GUiN0;^k^`24Q8@0?Shn0P0M7Ad0D@0k>c/0M7Ad06a/K014A4@0UIFE06m_K`21
PH40EUIF05mOG`1dM7@0T92@05iNGP14A4@0=SHf09JFUP19BDT0?clo08R8R01:BTX0:BTY03/k>`0j
>SX0<c<c05eMG@1kNg/0Q8B40:VYZ@1cLg<0M7Ad075aL@1iNGT0Cdm?0861P@1KFe/0E5AD08V9R@2g
]kL0W9bL0;Zj^P0^;Rh07QhN0410@00M7Ad0Bd];04Q8B01nOWh0K6a/0:RXZ01`L700NGUi08^;R`2d
];@0I6AT03@d=01?Cdl0JVYZ07IfMP1`L700YjNW0;Vi^@1FEUH0<S8b05YJFP1KFe/0MGEe02l_;`14
A4@0Fe]K08Z:RP2g]kL0/K6a0:2PX01GEeL0I6AT05QHF025QHD0RXZ:09:BTP2UYJD0Y:BT09fMW@02
04Q8B00=06][J`2IVIT0OWin03lo?`0f=SH0:BTY061PH01>CTh0ATI604];B`1=CDd0IVIV03`l?00D
ooooo`004oooool01P29RHT0Kfm_07AdM01lO7`0AdM7065QH@80HV9R00l0JVYZ04Y:BP1hN7P0LG5a
04=3@`23Ph<0IFEU04Q8B01RHV80JVYZ04A4A01cLg<0DU9B05EEE@1CDe<00P1YJFT0CP13@d<0HF5Q
05=CD`1<C4`0HF5Q061PH00g=cL0OWin079bLP19BDT0IFEU0:FUY@12@T80GEeM08f=S@11@D40GemO
06MWI`18B4P0LW9b06MWI`1DE5@0Kfm_05UIF@1BDU80NWYj04Y:BP1HF5P0FUYJ04e=C@1MGEd0DE5A
04e=C@1lO7`0EUIF059BDP1bLW80Hf=S05QHF026QXH0JVYZ05EEE@27QhL0LW9b04Y:BP2IVIT0JFUY
05]KF`2MWId0IFEU065QH@24Q8@0FEUI065QH@1fMWH0G5aL06m_K`1bLW80IVIV06e]K@1VIVH0Hf=S
07QhN01KFe/0JFUY07=cL`1aLG40TY:B096AT@1QHF40XJ6Q092@T01SHf<0Rh^;05YJFP1bLW80VYZJ
055AD@80QHF500T0IVIV06a/K01OGel0Rh^;07UiN@1>CTh0LG5a07emO@11@D400P1iNGT02017AdL0
O7al07MgM`1@D500M7Ad06m_K`1GEeL0SXj>0P1IFET03@24Q8@0AdM704m?C`1eMGD0E5AD061PH01W
IfL0EeMG05iNGP1dM7@0E5AD05UIF@1ZJVX03_ooool05@1DE5@0K6a/06YZJP1nOWh0@d=305aLG01`
L700PH6106a/K01YJFT0ATI603dm?@1:BTX0@d=305iNGP1EEED0Hf=S071`L01SHf<0@D5103/k>`02
05EEE@0T0410@01eMGD0CDe=051@D01JFUX0F5QH059BDP1PH600EeMG05mOG`1XJ6P0P82007MgM`1V
IVH0P82006ATI01ZJVX0[jn_08R8R01SHf<0J6QX05YJFP1RHV80DU9B06=SH`1KFe/0HF5Q05QHF01E
EED0BTY:04U9B@1YJFT0F5QH06EUI@1bLW80GUiN0P1FEUH0C`1LG5`0C4a<06=SH`21PH40Rh^;08B4
Q029RHT0K6a/055AD@1BDU80C4a<04Y:BP1DE5@0LG5a05=CD`1ADE40GUiN05MGE`24Q8@0GUiN05=C
D`1TI6@0FEUI07EeM@1=CDd0DE5A07EeM@2?Shl0DU9B071`L01FEUH0EEEE04i>CP1/K6`0FEUI05YJ
FP1BDU80Hf=S05iNGP1RHV80JVYZ03/k>`16ATH0DU9B04I6AP1YJFT0KVi^05IFEP1DE5@0IfMW06EU
I@22PX80IVIV06a/K01_Kfl0HV9R071`L02CTi<0P82007alO01dM7@0N7Qh09VIV@2EUID0K6a/0861
P@2_[jl0/K6a0::RXP2l_;`0WYjN05]KF`1@D500IfMW06i^KP1GEeL0Fe]K08b<S02JVYX03_ooool0
N`2CTi<0IfMW08j>SP16ATH0@T92051@D00o?cl0<C4a02/[:`0O7al0;Rh^030`<00h>3P0CDe=06][
J`0^;Rh0?Shn04M7A`0e=CD0B4Q805mOG`0k>c/0IfMW05QHF00o?cl0Fe]K02h^;P0_;bl0HV9R05MG
E`0g=cL0GEeM055AD@1@D500?Shn055AD@1MGEd0?Cdm06i^KP1IFET0IfMW05IFEP15ADD0FEUI06YZ
JP1GEeL0Vi^K05ADE01;Bd/0EUIF04m?C`1<C4`0JFUY04a<C01NGUh0B4Q804a<C01MGEd0A4A404i>
CP1DE5@0De=C06i^KP1BDU80L71`04Q8B01WIfL0EeMG075aL@2>SXh0CTi>07YjNP29RHT0FEUI06MW
I`1VIVH0KVi^06IVIP1TI6@0SXj>09RHV02^[Zh0JFUY07UiN@2GUiL0H61P069RHP1OGel0=CDe071`
L01ZJVX0EeMG065QH@1gMgL0EEEE08N7Q`1HF5P09bLW08N7Q`1=CDd0JVYZ05IFEP1]KFd0AdM705iN
GP1CDe<0Lg=c075aL@24Q8@0=CDe06m_K`1@D500GEeM0820P01mOGd0EUIF05aLG01QHF40B4Q807Yj
NP1=CDd0JVYZ04i>CP0206UYJ@0?03lo?`1SHf<0ATI605eMG@1VIVH0A4A407emO@1eMGD0Fe]K05aL
G00[:b/0HF5Q04Y:BP1^KVh0I6AT00kooooo00T0DU9B061PH02FUYH0Ti>C071`L02HV9P0W9bL04A4
A00g=cL00P2[Zj/0P@2`/;00IfMW06IVIP1EEED0ADE508b<S017AdL0UIFE07QhN01dM7@0Y:BT09BD
U00d=3@0=cLg08Z:RP1XJ6P0EUIF03Xj>P0h>3P0Hf=S06i^KP1@D500HF5Q04U9B@17AdL0A4A404Y:
BP1XJ6P0RHV905IFEP15ADD0OWin04];B`0e=CD0?Shn0:2PX02=SHd0ATI606IVIP2JVYX0HV9R05EE
E@0k>c/0Xj>S09:BTP0g=cL0EeMG08F5Q@20P800G5aL0:6QX@1UIFD0F5QH06UYJ@2<S8`0KVi^092@
T02KVi/0@T9208j>SP2OWil0TI6A09VIV@2YZJT0[jn_0=CDe021PH40Q8B409:BTP1hN7P0Y:BT0<c<
c01dM7@0CDe=09JFUP1WIfL0TY:B07emO@1bLW80DE5A0:f][@22PX80Ph>308B4Q02j^[X0NGUi04I6
AP1jNWX0<c<c01dM7@0[:b/0?Cdm0861P@23Ph<0LG5a06ATI01QHF40YJFU0820P02PX:00`/;20<K6
aP1jNWX0NGUi03Lg=`0l?3`0>CTi0492@P0c<c<0QHF50<G5a@2o_kl0/k>c09fMW@1WIfL0Jf][0:>S
X`1]KFd0FUYJ02/[:`0e=CD0;Rh^05QHF00g=cL0CDe=06MWI`1mOGd0Zj^[08N7Q`0Dooooo`004ooo
ool0R`24Q8@0N7Qh06YZJP1LG5`0Hf=S03Lg=`1RHV80L71`03dm?@1^KVh0SHf=04Y:BP1dM7@0Kfm_
071`L01DE5@0B4Q806ATI01LG5`0C4a<07YjNP1CDe<0>SXj08J6QP1MGEd0CDe=07QhN012@T80E5AD
07=cL`1CDe<0DE5A07inOP1]KFd0G5aL05eMG@1NGUh0J6QX05iNGP1ZJVX0EUIF069RHP1TI6@0UiNG
04=3@`1kNg/0PH61059BDP1WIfL0L71`04E5A@1VIVH0KFe]03dm?@1PH600SHf=04M7A`1lO7`0Ti>C
04];B`2<S8`0NGUi069RHP29RHT0Jf][069RHP1hN7P0Hf=S07inOP1RHV80Q8B407UiN@1^KVh0OGem
06m_K`1/K6`0NWYj08V9R@1DE5@0Hf=S0820P01/K6`0JVYZ08j>SP1bLW80Hf=S09JFUP1WIfL0O7al
08F5Q@1CDe<0MWIf09RHV01jNWX0QXJ607]kN`21PH40PX:2059BDP2ATI40QHF505YJFP1`L700P820
069RHP1hN7P0DE5A05mOG`2BTY80CDe=06m_K`1DE5@0MgMg06][J`1FEUH0QHF506ATI01OGel0TI6A
04];B`1JFUX0XJ6Q05=CD`1hN7P0P82006ATI01iNGT0OGem061PH01fMWH0KFe]04I6AP1ADE40I6AT
05aLG01UIFD0I6AT04I6AP1IFET03_ooool0>P1DE5@0HF5Q05UIF@1NGUh0AdM704m?C`15ADD0J6QX
06e]K@1NGUh0AdM70410@014A4@0@410061PH01FEUH0HF5Q06e]K@1VIVH0@d=3038b<P1NGUh0EEEE
04A4A024Q8@0HV9R05mOG`1PH600GUiN05IFEP1UIFD0DU9B05ADE01YJFT0Ng]k07MgM`1kNg/0Lg=c
08V9R@1MGEd0WIfM08f=S@1QHF40Kfm_05YJFP1LG5`0Cdm?06EUI@1BDU80EEEE04m?C`1<C4`0CDe=
04U9B@1JFUX0F5QH05mOG`1JFUX205IFEP80G5aL01/0I6AT06][J`26QXH0ZJVY09>CT`20P800O7al
04U9B@1FEUH0E5AD04E5A@1HF5P0NGUi06MWI`1HF5P0DU9B06EUI@1lO7`0L71`05UIF@1^KVh0Hf=S
07QhN018B4P0FUYJ05eMG@21PH400P1FEUH02`1DE5@0EeMG0410@01WIfL0FUYJ05mOG`1QHF40IfMW
05IFEP1hN7P0GUiN0080BDU902@0CTi>05=CD`1SHf<0HV9R05=CD`1NGUh0J6QX06UYJ@20P800J6QX
06UYJ@1VIVH0EEEE06IVIP25QHD0M7Ad05QHF01^KVh0Fe]K09FEU@2<S8`0M7Ad075aL@2IVIT0]kNg
09fMW@2d];@0[:b/06e]K@1<C4`0I6AT065QH@1FEUH0H61P071`L025QHD>ooooo`0P09:BTP1LG5`0
LW9b04=3@`0a<C40@d=303/k>`16ATH0@T920451@@0/;2`0:b/[03Lg=`1:BTX0GEeM03dm?@0b<S80
=3@d030`<018B4P0ADE502PX:01>CTh0De=C03lo?`1IFET0>CTi034a<@15ADD0FEUI059BDP1LG5`2
04Y:BP0g02LW9`1?Cdl0DE5A02l_;`24Q8@0LW9b05aLG019BDT0ADE50410@01MGEd0K6a/0:VYZ@1L
G5`0D51@061PH01HF5P0MGEe075aL@1]KFd0Jf][055AD@1OGel0Cdm?04Y:BP1MGEd0GemO065QH@1]
KFd0GUiN06][J`13@d<0U9BD05EEE@1UIFD0TI6A055AD@1`L700N7Qh05MGE`1ZJVX0K6a/08N7Q`1`
L700Fe]K07UiN@2/[:`0emOG06UYJ@2WYjL0WinO061PH01aLG40Hf=S03Lg=`0207EeM@0J075aL@1S
Hf<0EeMG06a/K01iNGT0EUIF034a<@29RHT0G5aL06EUI@1?Cdl0H61P04=3@`1/K6`0IVIV06][J`1E
EED0M7Ad03Lg=`1IFET0BTY:06=SH`1kNg/0DE5A05ADE01HF5P2061PH00E07IfMP1;Bd/0NWYj061P
H01cLg<0Kfm_04I6AP1hN7P0=cLg03lo?`1NGUh0BTY:06e]K@2BTY80KFe]06QXJ00R8R80MgMg04];
B`1mOGd0PX:200kooooo02h0BTY:079bLP1hN7P0DU9B038b<P1LG5`0_[jn03Xj>P0Y:BT0ZJVY0=KF
eP21PH40@d=3051@D01<C4`0?clo06QXJ02=SHd0EEEE04e=C@1IFET0Kfm_08J6QP0e=CD0>SXj07=c
L`1:BTX0P82004Q8B00];Bd0@T9207emO@1`L700M7Ad02`/;00N7Qh08B4Q02`/;015ADD0Q8B4051@
D00b<S80Hf=S03@d=00Q8B40AdM70P1YJFT0B`1QHF40OWin05QHF01;Bd/0De=C07emO@2VYZH0Lg=c
08N7Q`18B4P0:BTY071`L02<S8`0EeMG06i^KP1oOgl0JVYZ0<?3``2j^[X0QhN708R8R02ATI40f=SH
03De=@0];Bd0L71`06m_K`1GEeL0V9RH0;Ng]`2_[jl0BDU906YZJP2a/K40][Jf0:ZZZP2g]kL0ADE5
02XZ:P1RHV80TI6A07moO`1VIVH0J6QX05ADE01cLg<0UYJF0:VYZ@1bLW809bLW04e=C@0Q8B409BDU
024Q8@0X:2P0ZZZZ0=7Ad@2VYZH0CTi>05eMG@2BTY80IfMW0<G5a@2k^k/0UiNG0492@P1PH600A4A4
03lo?`1kNg/0Xj>S0:f][@2n_[h0QhN706MWI`0205eMG@0?07UiN@2][Jd0_;bl05]KF`0l?3`0@410
034a<@0c<c<0TI6A05IFEP1?Cdl0a/K60;Vi^@2PX:00[Zj^01Cooooo000Cooooo`0h08V9R@1kNg/0
Lg=c08>3P`1NGUh0ADE506MWI`1?Cdl0GUiN05eMG@1ZJVX0I6AT04=3@`1XJ6P0IfMW0451@@1EEED0
Kfm_05=CD`1GEeL0LW9b05eMG@19BDT0M7Ad05IFEP1CDe<0MGEe055AD@1GEeL0D51@05ADE01GEeL0
Bd];05mOG`1MGEd0E5AD07moO`1;Bd/0I6AT075aL@1CDe<0IVIV06YZJP1]KFd0HF5Q061PH01NGUh0
Jf][06ATI01QHF40Cdm?061PH01XJ6P0D51@05aLG01hN7P2059BDP0W06m_K`1EEED0KFe]06i^KP1N
GUh0N7Qh06i^KP1dM7@0CTi>07UiN@1aLG40A4A408N7Q`1/K6`0CTi>09>CT`1hN7P0DE5A07=cL`1^
KVh0GUiN06IVIP1fMWH0FUYJ07]kN`1cLg<0K6a/07UiN@1aLG40Kfm_07EeM@26QXH0Jf][07inOP1Z
JVX0Shn?0861P@1NGUh0Xj>S0080JVYZ02P0Ng]k06IVIP2>SXh0HF5Q07UiN@23Ph<0GEeM06a/K01n
OWh0PH6107=cL`1bLW80K6a/08J6QP1]KFd0MgMg07YjNP1IFET0LW9b07UiN@1<C4`0Kfm_08>3P`1:
BTX0Lg=c06][J`14A4@0WYjN06a/K01;Bd/0OGem06MWI`17AdL0HV9R07alO01LG5`0I6AT05MGE`1:
BTX0HV9R3_ooool02P1>CTh0IVIV065QH@1dM7@0A4A405EEE@1IFET0Hf=S06EUI@1RHV8204=3@`1C
04Y:BP11@D40GEeM05mOG`1YJFT0Lg=c06YZJP1;Bd/0@T9206QXJ01GEeL0=CDe06e]K@1=CDd0CTi>
04e=C@1@D500EUIF05eMG@1DE5@0FUYJ06e]K@1WIfL0L71`075aL@1YJFT0Fe]K061PH02:RXX0O7al
05QHF01PH600EUIF05YJFP1BDU80CDe=059BDP19BDT0CDe=04m?C`1DE5@0Bd];069RHP1LG5`0GEeM
05IFEP1LG5`0GemO05YJFP1PH600De=C04Y:BP1WIfL0V9RH08Z:RP1lO7`0K6a/059BDP1RHV80EUIF
0492@P15ADD0KVi^069RHP1<C4`0FEUI05QHF022PX80J6QX065QH@1gMgL0IVIV07QhN01DE5@0Fe]K
06a/K01PH600GemO05QHF01EEED0FEUI04I6AP1lO7`00P1QHF40:`1=CDd0J6QX04Y:BP1JFUX0Ng]k
04i>CP1<C4`0BTY:05IFEP1UIFD0E5AD05EEE@1KFe/0L71`06e]K@1kNg/0J6QX05mOG`1IFET0De=C
06YZJP1oOgl0OGem06ATI01VIVH0GEeM08b<S029RHT0P82007EeM@2LW9`0][Jf09NGU`2PX:00Rh^;
07AdM01MGEd0IfMW07IfMP1NGUh0HF5Q06m_K`2<S8`03_ooool0<@2;Rh/0F5QH08V9R@0l?3`0ATI6
0451@@12@T80?Shn03Hf=P0o?cl07AdM02/[:`0_;bl0FEUI06YZJP1BDU80<S8b030`<00h>3P0D51@
03Hf=P15ADD0GEeM059BDP16ATH0JVYZ03@d=00g=cL0FUYJ05aLG01MGEd0J6QX03De=@1KFe/0@T92
05ADE01IFET0BTY:0861P@1CDe<0Kfm_04M7A`0o?cl0GUiN061PH01OGel0ZJVY05mOG`1FEUH00P1?
Cdl0201WIfL0DE5A05UIF@1GEeL0GUiN06a/K01>CTh0GemO0P1NGUh04@1PH600Jf][05YJFP1WIfL0
BTY:08N7Q`19BDT0J6QX092@T01HF5P0IFEU08f=S@1CDe<0IFEU075aL@1fMWH0KFe]0080Jf][01P0
h>3P0>?Sh`1dM7@0QXJ6096AT@1YJFT0MGEe05ADE00k>c/0FEUI07=cL`1ADE40EeMG05YJFP1`L700
M7Ad05UIF@0d=3@0Q8B405aLG01^KVh0Hf=S07AdM01>CTh3065QH@0Q06][J`2BTY80>SXj06i^KP1@
D500DE5A08R8R01CDe<0FEUI061PH01/K6`0DU9B08:2PP1=CDd0LG5a05]KF`28R8P0K6a/04=3@`1T
I6@0Cdm?051@D01]KFd0Jf][06a/K02>SXh0N7Qh07EeM@0c<c<0M7Ad04a<C01]KFd0Jf][00kooooo
08`0GEeM04e=C@1TI6@0IfMW07EeM@1aLG40IfMW08:2PP2a/K40^k^k079bLP1kNg/0IFEU03Xj>P15
ADD0Hf=S051@D011@D40Cdm?07AdM01OGel0Bd];07IfMP0S8b<0=SHf06a/K00n?Sh0GEeM09^KV`1X
J6P0OGem08Z:RP1<C4`0=CDe092@T01VIVH0GUiN06YZJP2BTY80a<C407YjNP19BDT0@d=3069RHP0R
8R80FEUI03dm?@24Q8@0SHf=06a/K012@T80@41002l_;`0`<300HV9R04a<C01ZJVX0OGem0:VYZ@24
Q8@0KFe]059BDP19BDT0XZ:R07AdM022PX80Z:RX0;6a/@29RHT0UiNG06YZJP1SHf<0=cLg04=3@`1O
Gel09bLW03<c<`1MGEd0Z:RX075aL@13@d<0<C4a08j>SP2MWId0VYZJ06][J`2DU9@0IVIV07YjNP1_
Kfl0Rh^;05ADE012@T80FEUI08:2PP20P800WinO08R8R00c<c<07QhN01dM7@0[:b/0OGem0:j^[P2;
Rh/0VYZJ0;2`/02NWYh0aLG50;Fe]@2m_Kd0Z:RX08^;R`19BDT0A4A4096AT@1^KVh0IVIV08n?S`2m
_Kd0WIfM02/[:`0S8b<092@T03Lg=`1DE5@0NGUi07IfMP1_Kfl0?Shn07]kN`2DU9@0Shn?07alO02C
Ti<0^[Zj0:b/[036a/H0YJFU08J6QQCooooo000Cooooo`1707emO@1cLg<0Lg=c08^;R`1PH600JFUY
05EEE@1CDe<0Hf=S04I6AP1_Kfl0KVi^05IFEP29RHT0EUIF05aLG01SHf<0CTi>06a/K01PH600DU9B
061PH01GEeL0F5QH07=cL`0g=cL0C4a<06a/K00m?Cd0ADE507MgM`0a<C40GEeM09JFUP0h>3P0G5aL
09BDU00Z:RX0MGEe061PH00g=cL0M7Ad05QHF01DE5@0LG5a06a/K011@D40N7Qh04U9B@1<C4`0MGEe
05aLG01ZJVX0C4a<04];B`1nOWh0LG5a05eMG@2<S8`0GUiN07MgM`26QXH0Jf][06UYJ@2LW9`0LG5a
05]KF`1oOgl0HV9R059BDP2BTY800P1QHF401020P800Ng]k07]kN`29RHT206EUI@0306=SH`1YJFT0
OGem0080GUiN03L0QhN705aLG025QHD0OGem0492@P1nOWh0QHF507=cL`1kNg/0WYjN05YJFP1fMWH0
SXj>07]kN`1`L700IFEU07MgM`1]KFd0A4A408n?S`1aLG40JVYZ07MgM`1`L700XZ:R05ADE01XJ6P0
Jf][07YjNP1EEED0O7al07YjNP1FEUH0LW9b06UYJ@16ATH0K6a/065QH@1DE5@0QXJ605EEE@1NGUh0
RHV905eMG@1RHV80H61P05]KF`1]KFd0C4a<04E5A@1_Kfl0HF5Q03De=@1KFe/0Ogmo00kooooo01D0
G5aL07AdM01YJFT0KFe]05=CD`1KFe/0HF5Q06a/K01fMWH0GEeM04I6AP1<C4`0Cdm?0492@P1UIFD0
FEUI05mOG`1WIfL0J6QX03Ph>00k>c/00P1IFET0B00j>SX0H61P04Y:BP1;Bd/0Cdm?055AD@18B4P0
GEeM04m?C`1<C4`0I6AT06e]K@1KFe/0MgMg05iNGP1CDe<0HF5Q07moO`1eMGD0De=C05aLG017AdL0
DE5A04=3@`18B4P0F5QH05ADE01JFUX0Cdm?051@D01:BTX0FUYJ05iNGP1LG5`0De=C05aLG01KFe/0
Kfm_05aLG01:BTX0@T9206ATI02:RXX0P82007inOP25QHD0DU9B06IVIP1GEeL0CDe=03dm?@1/K6`0
Hf=S04M7A`1CDe<0EEEE096AT@1PH600HV9R05mOG`1MGEd0G5aL04E5A@1FEUH0HF5Q04];B`1BDU80
Kfm_05YJFP1GEeL0CDe=075aL@80I6AT01/0G5aL05mOG`14A4@0I6AT07UiN@1EEED0AdM703/k>`1=
CDd0GUiN05EEE@1=CDd0F5QH06e]K@1EEED0IFEU069RHP1KFe/0FEUI04i>CP1RHV80PX:206i^KP1Z
JVX0EEEE05iNGP25QHD00P1YJFT03P1LG5`0QHF509^KV`2;Rh/0NWYj09RHV01LG5`0ATI605UIF@1R
HV80DU9B061PH01XJ6P0QXJ63_ooool03`1MGEd0EeMG07emO@0j>SX0BDU903hn?P11@D40@d=303dm
?@0k>c/06a/K02l_;`0/;2`0Cdm?051@D0020492@P1k02h^;P0b<S80@410020P8017AdL0B4Q8069R
HP1<C4`0GEeM02l_;`0n?Sh0BTY:04];B`1]KFd0GUiN02l_;`1SHf<0@d=305iNGP1KFe/0>3Ph07Yj
NP1?Cdl0MgMg0492@P0c<c<0G5aL03`l?01SHf<0UiNG061PH018B4P0EEEE04A4A01iNGT0?3`l04M7
A`1RHV80BTY:06][J`15ADD0C4a<069RHP1YJFT0F5QH069RHP1CDe<0KVi^04Q8B01^KVh0Cdm?071`
L01cLg<0IVIV06i^KP22PX80D51@06][J`1`L700IfMW06e]K@1NGUh0@d=30<;2`P2ATI40HF5Q0::R
XP2PX:00KFe]08f=S@1IFET0C4a<06=SH`25QHD0JFUY06QXJ01=CDd0Shn?06YZJP1QHF40>CTi08J6
QP1]KFd0H61P05ADE01mOGd0DU9B04=3@`1hN7P0F5QH04Q8B024Q8@0ADE505iNGP1FEUH0IFEU07em
O@1JFUX0H61P04e=C@19BDT0IfMW06i^KP0h>3P0JVYZ06UYJ@23Ph<0KVi^03`l?01bLW80?3`l04Q8
B01VIVH0EeMG05iNGP2>SXh0I6AT06IVIP0a<C40L71`034a<@1FEUH0MgMg00kooooo00X0FEUI05MG
E`1HF5P0GUiN08N7Q`1[Jf/0HF5Q09NGU`3@d=00Kfm_0P0W9bL0K@0];Bd0H61P04A4A01=CDd071`L
01dM7@0_;bl0KFe]05ADE00l?3`0E5AD0;>c/`2LW9`0WYjN079bLP1BDU809bLW01/K6`0L71`0FUYJ
051@D00e=CD0De=C09NGU`0Z:RX0:BTY02d];@14A4@0XJ6Q08N7Q`1ZJVX0YjNW08B4Q02YZJT0HV9R
06YZJP1oOgl0F5QH04M7A`0i>CT0:2PX02TY:@16ATH0C4a<02DU9@0c<c<0Jf][030`<01dM7@0ZJVY
09NGU`2IVIT0NWYj02d];@26QXH0NWYj04=3@`1ADE40L71`030`<01/K6`0EeMG07EeM@0c<c<0@T92
04e=C@1lO7`0=CDe02LW9`0T92@0RHV90;fm_@2[Zj/0ZZZZ0<K6aP20P800EEEE08B4Q00m?Cd0BDU9
07AdM02WYjL0CTi>079bLP21PH40YZJV08^;R`0V9RH0@d=309bLW02BTY80QhN70:>SX`19BDT0<c<c
06IVIP3Jf]X0];Bd06a/K019BDT0MgMg06UYJ@13@d<0IfMW028R8P0^;Rh0F5QH0<O7a`1nOWh00P0N
7Qh04@0O7al09RHV03Ph>01@D500Ogmo0<C4a02o_kl0W9bL09fMW@2EUID0UYJF096AT@2k^k/0`l?3
0:VYZ@2f][H0OGem01Cooooo000Cooooo`2;0861P@1dM7@0Kfm_05]KF`2;Rh/0?clo06YZJP1eMGD0
D51@08B4Q01fMWH0Fe]K06YZJP1VIVH0PH6104m?C`1FEUH0Ng]k05YJFP1HF5P0OGem05=CD`1@D500
QXJ605eMG@1JFUX0PH6105ADE01FEUH0F5QH04Q8B01NGUh0B4Q804m?C`1VIVH0@41006MWI`1@D500
IFEU05aLG01BDU80H61P07AdM01bLW80D51@05iNGP1oOgl0CDe=05ADE01cLg<0?3`l05EEE@1LG5`0
>c/k06EUI@28R8P0<C4a05YJFP1fMWH0C4a<08J6QP1/K6`0IVIV08:2PP1LG5`0LW9b06a/K01MGEd0
TY:B05YJFP1aLG40O7al069RHP1aLG40QHF5069RHP1[Jf/0Q8B4059BDP1GEeL0Q8B405ADE01LG5`0
SHf=065QH@1RHV80UiNG05IFEP1cLg<0M7Ad04e=C@1fMWH0Jf][06IVIP2=SHd0AdM707QhN01jNWX0
ADE508Z:RP1_Kfl0K6a/08B4Q01`L700KFe]071`L01JFUX0J6QX09BDU01DE5@0N7Qh05IFEP1eMGD0
Jf][07YjNP2CTi<0HF5Q06QXJ02PX:00BTY:069RHP28R8P0FEUI07moO`29RHT0GUiN06UYJ@1_Kfl0
F5QH07MgM`21PH40C4a<06IVIP20P800Fe]K05mOG`1ZJVX0FUYJ069RHP0>ooooo`0A04M7A`1ZJVX0
IVIV079bLP1DE5@0H61P05eMG@1QHF40JFUY05aLG00l?3`0ATI60492@P0i>CT0Fe]K061PH01RHV80
0P1ZJVX01`1;Bd/0@410061PH01CDe<0ADE506QXJ016ATH00P1@D500801DE5@0C4a<05aLG01?Cdl0
FEUI06UYJ@1eMGD0IVIV07emO@1JFUX0BDU905aLG021PH40LG5a05]KF`1PH600D51@06EUI@1CDe<0
EUIF04e=C@1>CTh0De=C04a<C01JFUX0BDU905YJFP1TI6@0K6a/06IVIP1JFUX0DU9B0P1]KFd02P1S
Hf<0BDU906MWI`1KFe/0Ph>308:2PP1lO7`0Cdm?05mOG`1MGEd203`l?00906QXJ01^KVh0HV9R05QH
F01kNg/0];Bd07emO@1KFe/0Lg=c0080IFEU01@0Cdm?05QHF01aLG40EeMG055AD@1ZJVX0DU9B05AD
E01LG5`0H61P07MgM`1SHf<0H61P06UYJ@1>CTh0L71`07moO`1MGEd0DE5A04a<C080FUYJ0240D51@
05=CD`1GEeL0KFe]06a/K01SHf<0HF5Q06MWI`1aLG40FUYJ061PH01oOgl0OWin04E5A@1FEUH0HV9R
079bLP1mOGd0QHF506MWI`28R8P0Vi^K06i^KP1jNWX0S8b<06UYJ@1=CDd0IFEU05mOG`1CDe<0Hf=S
06i^KP28R8P03_ooool0S01SHf<0FUYJ08F5Q@0h>3P0AdM704A4A01=CDd0EeMG03lo?`0m?Cd09BDU
03<c<`0_;bl0E5AD05iNGP1WIfL0@D5103lo?`0e=CD0Cdm?034a<@0];Bd0CDe=06=SH`0k>c/0G5aL
03/k>`0d=3@0?clo04I6AP1ZJVX0CDe=03<c<`1BDU80CTi>05]KF`11@D40<30`0861P@1RHV80MWIf
05YJFP19BDT0De=C03De=@1jNWX0UiNG075aL@1VIVH0Cdm?05aLG01WIfL0BTY:06MWI`1ADE40Jf][
05iNGP1<C4`0JFUY05UIF@1/K6`0EeMG06YZJP1IFET0IFEU06m_K`1RHV80<S8b061PH027QhL0Kfm_
06UYJ@2ATI40D51@075aL@1XJ6P0RXZ:06MWI`2@T900K6a/0861P@1[Jf/0Lg=c07emO@1aLG40NWYj
08N7Q`1KFe/0ATI605YJFP24Q8@0EeMG06][J`1YJFT0S8b<06ATI01hN7P0Bd];079bLP1`L700Ogmo
07=cL`1lO7`0DU9B06UYJ@1^KVh0F5QH06=SH`20P800?3`l07moO`1MGEd0I6AT07moO`1[Jf/0GUiN
05MGE`1;Bd/0FEUI07EeM@1?Cdl0FEUI05eMG@2?Shl0MGEe04Q8B01ADE40GEeM06e]K@1UIFD0Kfm_
079bLP26QXH0JFUY08J6QP11@D40Hf=S03dm?@1WIfL0Hf=S3_ooool04@1?Cdl0<C4a01lO7`0J6QX0
7AdM07moO`2IVIT0NWYj096AT@2PX:00IFEU030`<00b<S80<C4a02TY:@0R8R806a/K00807AdM0140
=CDe03lo?`0g=cL0S8b<08R8R013@d<0C4a<07IfMP1FEUH0;2`/02DU9@0M7Ad0IfMW04E5A@1XJ6P0
=SHf06IVIP0201`L700X01dM7@0O7al0BTY:08Z:RP3Hf=P0M7Ad038b<P13@d<0Ng]k0:b/[02d];@0
?Cdm02<S8`0X:2P0ATI604E5A@2EUID0YjNW034a<@0V9RH0UiNG08R8R01OGel0Kfm_05EEE@1QHF40
LW9b05mOG`1ADE40GemO02`/;00b<S80O7al04I6AP1>CTh0TI6A09RHV01UIFD0Ng]k05EEE@80DE5A
01L0;Rh^03/k>`2[Zj/0QhN708>3P`2_[jl0^k^k0;Bd]020P800WYjN02h^;P0Z:RX0N7Qh0:FUY@32
`/80HV9R02<S8`2e]KD0];Bd092@T02f][H0XZ:R08>3P`0203Xj>P0P05QHF019BDT0NGUi0=CDe02H
V9P0CTi>04m?C`1EEED0OGem07UiN@1[Jf/0:b/[03<c<`2VYZH0/k>c07]kN`0i>CT0;Bd]02@T900Q
8B408R8R03Xj>P2ZZZX0aLG50<k>cP2[Zj/0UYJF07]kN`1SHf<0W9bL0:j^[P2IVIT20;Vi^@030<30
`?oooooooooo01;ooooo000Cooooo`1R08^;R`1hN7P0Ng]k0861P@1FEUH0MgMg06][J`1HF5P0QhN7
04m?C`25QHD0Lg=c03/k>`2:RXX0H61P05ADE01/K6`0FUYJ06MWI`1KFe/0F5QH05UIF@1BDU80FUYJ
06QXJ01>CTh0B4Q806=SH`19BDT0BTY:07QhN01ADE40CDe=08n?S`16ATH0AdM708n?S`0j>SX0BDU9
08>3P`0a<C40IfMW065QH@11@D40NWYj06UYJ@10@400L71`04i>CP1?Cdl0HV9R06a/K01WIfL0EeMG
05EEE@1eMGD0B4Q801lO7`1ZJVX0Jf][05mOG`1mOGd0LW9b07EeM@2JVYX0LG5a059BDP1mOGd0IfMW
051@D022PX80LG5a06ATI01hN7P0QXJ606IVIP21PH40NGUi065QH@1/K6`0LW9b071`L01JFUX0GUiN
07]kN`18B4P0KFe]07EeM@18B4P0MgMg07UiN@1KFe/0P8200861P@1?Cdl0Ph>3075aL@1IFET2071`
L080JFUY01H0@D5107QhN01oOgl0CTi>075aL@1PH600QHF5071`L01LG5`0SHf=071`L015ADD0TY:B
07UiN@1<C4`0KVi^06EUI@1HF5P0PH6107moO`1?Cdl0YjNW0P1OGel03@29RHT0DE5A065QH@1SHf<0
De=C05ADE01[Jf/0CTi>06MWI`1aLG40E5AD05eMG@2>SXh03_ooool0:P1GEeL0MWIf06IVIP1XJ6P0
EEEE06ATI01=CDd0IfMW05UIF@1TI6@0BTY:05aLG01CDe<0<c<c05mOG`1LG5`0H61P06i^KP1UIFD0
DU9B03/k>`1IFET0FUYJ055AD@1fMWH0EEEE05YJFP1OGel0G5aL04a<C01MGEd0DE5A05YJFP1LG5`0
KFe]05UIF@1bLW80I6AT05EEE@1DE5@0Rh^;079bLP80GUiN0600Bd];06QXJ017AdL0Fe]K04e=C@1C
De<0CDe=04U9B@1;Bd/0FUYJ06=SH`1UIFD0N7Qh06][J`1IFET0H61P05=CD`1fMWH0PX:207MgM`1c
Lg<0Ogmo08>3P`25QHD0MGEe05MGE`0i>CT0Cdm?03`l?00k>c/0IfMW06=SH`1>CTh0FEUI06IVIP2X
Z:P0K6a/05YJFP1RHV80DE5A06a/K01>CTh0D51@06IVIP1?Cdl0@T9206ATI01DE5@0Fe]K069RHP1D
E5@0IfMW05eMG@1PH600GUiN05=CD`1`L700MWIf065QH@1DE5@0>3Ph05UIF@1QHF40CDe=059BDP1E
EED0JVYZ06][J`1>CTh0H61P06a/K01ZJVX0Hf=S05eMG@1hN7P0K6a/04m?C`1EEED0CDe=05aLG01]
KFd0Ogmo06ATI02MWId0R8R807]kN`2JVYX0WIfM06e]K@1ADE40JVYZ061PH01BDU80G5aL06IVIP1f
MWH>ooooo`80DU9B00D0Hf=S034a<@0f=SH0=SHf02l_;`020492@P0B030`<00P8200@41002@T9015
ADD0Bd];05=CD`19BDT0>c/k03hn?P16ATH0;Rh^04Y:BP15ADD0K6a/06ATI018B4P0<30`0P0k>c/0
K`1=CDd0OWin051@D00a<C40GUiN04E5A@1bLW80C4a<04e=C@1/K6`0DE5A09>CT`10@400ATI605EE
E@0f=SH0KVi^08V9R@1mOGd0FEUI05aLG01JFUX0NGUi05IFEP1PH600GemO06=SH`1VIVH0BTY:04];
B`1TI6@0G5aL05MGE`1NGUh0De=C07moO`1PH600IVIV059BDP1YJFT0NGUi06=SH`20P800MgMg051@
D01YJFT0IVIV08j>SP1@D500MWIf05EEE@2?Shl0HF5Q06EUI@25QHD0MGEe06=SH`2?Shl0EeMG03@d
=01EEED0MgMg05iNGP1]KFd0?3`l08j>SP1HF5P0HF5Q05eMG@1kNg/0MGEe065QH@18B4P0O7al04U9
B@1ADE40M7Ad061PH01HF5P0TI6A05iNGP1VIVH0Fe]K07AdM01lO7`0H61P079bLP1;Bd/0A4A406MW
I`1RHV80Cdm?071`L01bLW80M7Ad07YjNP1/K6`0J6QX04M7A`1aLG40Kfm_06UYJ@1ZJVX0QXJ607If
MP20P800:BTY05MGE`0m?Cd0IFEU07MgM`0>ooooo`1o04U9B@0j>SX0>3Ph04U9B@1:BTX0E5AD09jN
WP2<S8`0P82004m?C`1TI6@0L71`04m?C`0W9bL07alO01DE5@0D51@06QXJ01/K6`0P8200DE5A02h^
;P1HF5P0EUIF03`l?010@400MWIf055AD@1VIVH0Hf=S051@D02SXj<0L71`06a/K01QHF40QXJ601dM
7@0L71`07AdM01hN7P1KFe/0WIfM0;Rh^01iNGT0EUIF06UYJ@1HF5P0RXZ:092@T01TI6@0?Cdm01lO
7`0V9RH0HV9R051@D024Q8@0K6a/07EeM@1gMgL0C4a<04U9B@1TI6@0=SHf03lo?`1MGEd0@41003dm
?@1TI6@09bLW02<S8`1DE5@0;bl_06IVIP2ATI40O7al0451@@0d=3@0GEeM0861P@2^[Zh0NWYj0:ZZ
ZP1SHf<0HV9R0:NWY`1oOgl0M7Ad03Ti>@0l?3`0Hf=S07]kN`1@D500K6a/05aLG023Ph<08R8R08:2
PP2WYjL0`<300<O7a`2j^[X0Hf=S06ATI00k>c/071`L02`/;02GUiL0P8200<g=c@2a/K40C4a<04Q8
B01NGUh0];Bd09jNWP2;Rh/0P82006MWI`2h^;P0_;bl08>3P`17AdL0@T9204=3@`1QHF40Ng]k07Qh
N0020<K6aP0;08F5Q@0e=CD0?3`l07AdM03IfMT0SXj>0451@@0j>SX0OWin0;fm_@25QHD05?ooool0
01?ooooo04L0SXj>07AdM029RHT0K6a/07EeM@1HF5P0Lg=c06YZJP1RHV80G5aL07UiN@1HF5P0EeMG
0820P01ADE40FUYJ06UYJ@1BDU80GUiN05mOG`1KFe/0HF5Q05eMG@1YJFT0IfMW059BDP1TI6@0FUYJ
04i>CP1GEeL0IVIV04I6AP1>CTh0NGUi059BDP16ATH0JVYZ055AD@1ZJVX0L71`05MGE`1^KVh0GEeM
06EUI@1TI6@0S8b<07AdM01GEeL0GUiN0861P@1BDU80K6a/06][J`11@D40DU9B08:2PP15ADD0De=C
07]kN`1BDU80JVYZ07UiN@1/K6`0Ng]k07EeM@1]KFd0HV9R06][J`1mOGd0D51@07AdM00306m_K`0?
07alO01dM7@0E5AD07emO@1XJ6P0E5AD07]kN`1`L700EUIF08Z:RP1^KVh0ATI608N7Q`1gMgL0C4a<
0080MGEe02@0LW9b08>3P`1gMgL0K6a/06i^KP1bLW80JVYZ065QH@1jNWX0QXJ6079bLP1UIFD0IfMW
07=cL`26QXH0HV9R0820P01oOgl0DU9B06i^KP26QXH0K6a/065QH@25QHD0O7al05mOG`1dM7@0KFe]
051@D01OGel0KVi^04a<C01nOWh0HV9R065QH@1fMWH206i^KP0:07IfMP1OGel0CTi>05iNGP1FEUH0
D51@06UYJ@1RHV80E5AD07]kN`kooooo08`0F5QH06a/K01WIfL0K6a/04];B`1TI6@0FUYJ07AdM01W
IfL0F5QH03/k>`19BDT0DE5A034a<@1SHf<0DE5A051@D01MGEd0JFUY04I6AP10@400DE5A05YJFP14
A4@0IFEU04U9B@1BDU80EUIF05aLG01>CTh0E5AD05eMG@1?Cdl0HF5Q07AdM01QHF40LW9b05IFEP1D
E5@0EeMG08V9R@1]KFd0EUIF06EUI@1EEED0JFUY05EEE@1QHF40DE5A05QHF01KFe/0EEEE059BDP1S
Hf<0JVYZ075aL@22PX80P82006a/K01^KVh0NGUi07emO@21PH40QHF507MgM`1eMGD0Ogmo07QhN01e
MGD0EUIF05ADE01XJ6P0Cdm?04A4A01iNGT0LW9b05YJFP1QHF40L71`0:6QX@1PH600Hf=S06][J`1U
IFD0KVi^04E5A@1XJ6P0HV9R05=CD`1<C4`0Jf][05IFEP1HF5P0GemO05QHF01aLG40E5AD06ATI01P
H600EUIF05QHF01LG5`0E5AD04];B`0/;2`0CDe=05]KF`1LG5`0FEUI055AD@1OGel0H61P05ADE01P
H600FUYJ05aLG01BDU80KFe]07inOP1dM7@0GEeM05]KF`1RHV80GemO07moO`24Q8@0NWYj096AT@2J
VYX0Xj>S09RHV023Ph<0Jf][05EEE@1RHV80IFEU05EEE@1dM7@0HV9R06m_K`kooooo00<0LG5a05UI
F@1WIfL00P15ADD0Q`0h>3P0D51@0451@@1<C4`0=3@d02d];@0n?Sh0;bl_04A4A01;Bd/0ATI604];
B`0k>c/0>SXj0410@00`<300ADE503Xj>P1NGUh0E5AD04E5A@11@D40BDU903Ph>016ATH0N7Qh055A
D@0n?Sh0Bd];055AD@1MGEd0AdM703lo?`1CDe<0C4a<08n?S`1=CDd0C4a<04m?C`10@400LG5a07If
MP1]KFd0HF5Q04a<C01MGEd0FUYJ05ADE01dM7@0DE5A071`L01WIfL0CDe=05iNGP1WIfL0HF5Q061P
H01[Jf/0GUiN06QXJ01oOgl0LW9b04I6AP1SHf<0TI6A05aLG01eMGD0Ti>C05]KF`1cLg<0I6AT07Mg
M`1XJ6P0OWin09NGU`2SXj<0FEUI06m_K`1nOWh0I6AT06m_K`2CTi<0FEUI04Y:BP1EEED0R8R805YJ
FP1UIFD0FUYJ08:2PP1:BTX0IFEU051@D01/K6`0IfMW06QXJ017AdL0Kfm_04U9B@1FEUH0JFUY06=S
H`1MGEd0N7Qh0492@P1/K6`0FUYJ065QH@1aLG40JVYZ06EUI@1@D500De=C05EEE@1LG5`0GEeM055A
D@1OGel0OWin07moO`1FEUH0EeMG04Y:BP1QHF40E5AD065QH@1eMGD0NWYj06UYJ@2BTY80<c<c05YJ
FP1ADE40J6QX05aLG00>ooooo`80Cdm?04T0C4a<07=cL`1FEUH0Bd];02/[:`1?Cdl0ZZZZ0:NWY`2H
V9P0YZJV08n?S`0c<c<06QXJ01hN7P0F5QH05aLG01/K6`0N7Qh0DE5A08n?S`2;Rh/0DE5A03<c<`0h
>3P08R8R01lO7`0^;Rh0D51@03hn?P1WIfL0K6a/08F5Q@24Q8@0/K6a06][J`0n?Sh0BTY:05MGE`1h
N7P0^KVi0<K6aP2o_kl0a<C40:JVYP1VIVH0C4a<07IfMP2SXj<0NWYj06m_K`1gMgL0RHV906QXJ02h
^;P0P82009nOW`2ZZZX0RHV907=cL`1hN7P0;bl_03De=@1hN7P0EeMG08f=S@29RHT0@D5103<c<`1c
Lg<0Kfm_06=SH`19BDT0@4100080>c/k03l0=cLg06MWI`2;Rh/0Xj>S0:VYZ@1`L700N7Qh05=CD`1U
IFD0JFUY020P800T92@0=CDe04a<C02CTi<0Ph>308:2PP2/[:`0[Zj^06e]K@1YJFT0MWIf08>3P`2P
X:00J6QX061PH01mOGd0HF5Q05mOG`1eMGD0M7Ad0;Rh^01;Bd/0J6QX079bLP1gMgL0QHF509RHV026
QXH0VYZJ09BDU01hN7P0Ph>307emO@26QXH0Shn?09fMW@2CTi<0NWYj092@T02f][H0WIfM092@T01A
DE4092@T02l_;`28R8P0JFUY02/[:`0f=SH0OWin09FEU@1MGEd05?ooool001?ooooo01`0N7Qh071`
L01YJFT0Hf=S061PH01CDe<0Lg=c06IVIP1QHF40Fe]K06UYJ@1`L700ATI6061PH01cLg<0?Shn05MG
E`1`L700De=C06m_K`1gMgL0A4A406a/K01]KFd0ATI607=cL`1KFe/0=SHf0P1GEeL0K@14A4@0H61P
0410@01CDe<0Jf][03hn?P1;Bd/0J6QX065QH@1MGEd0I6AT05iNGP28R8P0JFUY04a<C02CTi<0NWYj
04U9B@2?Shl0MWIf04];B`1kNg/0Lg=c04i>CP1kNg/0Hf=S05]KF`1gMgL0QHF506MWI`1gMgL0Jf][
06MWI`24Q8@0H61P092@T01OGel0HV9R08Z:RP1:BTX0G5aL07inOP1LG5`0Jf][0:BTY01@D500MgMg
09fMW@1@D500Kfm_092@T01<C4`0Ogmo0:6QX@1DE5@0HF5Q09fMW@1FEUH0IFEU06QXJ01=CDd0L71`
06a/K01aLG40Ph>3055AD@2>SXh0NGUi055AD@28R8P0L71`08n?S`1XJ6P0EUIF08f=S@1bLW80?clo
08>3P`1gMgL0I6AT05eMG@1RHV80NGUi05MGE`1TI6@0QHF504Q8B01WIfL0SXj>04U9B@1ZJVX0R8R8
05ADE01bLW80QhN704m?C`21PH40KFe]06ATI01iNGT0IVIV055AD@1TI6@0LG5a04e=C@1YJFT0K6a/
03Hf=P1nOWh03_ooool01`1FEUH0LG5a05eMG@1JFUX0Cdm?059BDP1CDe<00P1UIFD0601EEED0AdM7
055AD@1ADE40A4A405eMG@1>CTh0BTY:051@D01OGel0@41002LW9`13@d<0Cdm?04U9B@1VIVH0AdM7
04e=C@19BDT0CTi>05ADE01?Cdl0FUYJ05=CD`80HV9R03X0IfMW06m_K`1DE5@0E5AD05QHF01dM7@0
HF5Q04e=C@1HF5P0Bd];055AD@1FEUH0FUYJ04E5A@1HF5P0H61P051@D01CDe<0IFEU07MgM`1TI6@0
Kfm_07AdM01JFUX0IVIV0820P01gMgL0QHF50861P@1_Kfl0RHV908R8R02EUID0MgMg05iNGP1DE5@0
G5aL04];B`0d=3@0JFUY05]KF`1HF5P0C4a<075aL@2OWil0LG5a06EUI@1WIfL0DE5A05QHF01FEUH0
Jf][06MWI`18B4P0DE5A07EeM@1ADE40Cdm?0P1NGUh0601/K6`0J6QX061PH01RHV80FUYJ05iNGP1L
G5`0Bd];05MGE`0j>SX0HV9R06EUI@1NGUh0EEEE05YJFP1]KFd0F5QH04U9B@1YJFT0H61P06EUI@1J
FUX0JVYZ07]kN`80H61P01<0DE5A06YZJP1?Cdl0K6a/08^;R`1PH600Rh^;0861P@2ATI40]KFe09fM
W@1dM7@0DU9B06][J`1/K6`0F5QH06a/K01`L700LW9b00kooooo01`0HV9R05YJFP1IFET0<c<c04i>
CP0W9bL0DE5A034a<@0i>CT0>SXj020P800b<S809bLW03De=@15ADD0ATI603Lg=`0a<C40:b/[04];
B`0Z:RX0Cdm?03Lg=`0m?Cd0Jf][04Y:BP0X:2P0AdM70P0j>SX07@1/K6`0Bd];05QHF016ATH0GEeM
04m?C`1EEED0GUiN03/k>`1BDU80UIFE059BDP17AdL0HV9R059BDP1^KVh0NGUi07alO01NGUh0GemO
05aLG01^KVh0I6AT0861P@1OGel0JFUY06IVIP1EEED0Fe]K0080H61P0340Fe]K06=SH`1CDe<0Fe]K
08V9R@1PH600E5AD06e]K@1nOWh0Lg=c07AdM01iNGT0DE5A07=cL`1LG5`0IfMW065QH@1/K6`0OWin
09JFUP1CDe<0Lg=c07UiN@1XJ6P0Hf=S09jNWP1DE5@0AdM704i>CP2>SXh0EEEE069RHP1MGEd0PH61
04e=C@1XJ6P0FEUI079bLP24Q8@0KFe]05iNGP1lO7`0FUYJ06QXJ01eMGD0KVi^06][J`1gMgL0IVIV
0080FUYJ01`0KFe]07=cL`1DE5@0SHf=05IFEP1BDU80Hf=S05IFEP1PH600Hf=S06EUI@1QHF40Hf=S
071`L01PH600?Cdm07=cL`1BDU80?Shn061PH022PX80FUYJ05eMG@0c<c<0HF5Q04e=C@1IFET0Fe]K
3_ooool06`1MGEd0CTi>05eMG@1ZJVX0EeMG06e]K@0U9BD0A4A4065QH@17AdL0E5AD07=cL`2PX:00
TI6A069RHP15ADD0<30`024Q8@0_;bl0G5aL06UYJ@2:RXX0UIFE08R8R02XZ:P07AdM01@D500201HF
5P0>05=CD`27QhL0TY:B05iNGP1OGel0M7Ad04E5A@0e=CD0EEEE05UIF@23Ph<0WinO09FEU@39bLT2
079bLP0_0:>SX`0l?3`0=SHf01lO7`1eMGD0?Cdm06e]K@2LW9`0LW9b08f=S@2DU9@0Fe]K09FEU@2j
^[X0Xj>S04M7A`1NGUh0HF5Q034a<@1oOgl0Kfm_07emO@2a/K40L71`07AdM02TY:@0VIVI08>3P`0`
<300D51@02`/;00c<c<0De=C02PX:012@T80Shn?03hn?P0a<C40Shn?05IFEP1XJ6P0DU9B03Xj>P0M
7Ad0;2`/05UIF@1jNWX00P2b/[80;P2e]KD0XZ:R0492@P1BDU80PX:208>3P`2:RXX0OGem05mOG`1n
OWh0SXj>0=CDe034a<@0bl_;09VIV@0o?cl0=3@d07=cL`1GEeL0O7al06MWI`1YJFT0TY:B07=cL`1C
De<0B4Q808:2PP2PX:00CDe=03@d=00c<c<0>SXj06a/K02GUiL0CDe=05=CD`11@D40C4a<08B4Q01f
MWH0GEeM02`/;014A4@0WinO06QXJ02LW9`Dooooo`004oooool05`23Ph<0K6a/06m_K`20P800D51@
05]KF`1WIfL0IFEU07]kN`19BDT0N7Qh05aLG01<C4`0MgMg04e=C@1CDe<0NGUi04];B`1PH600Hf=S
04e=C@1MGEd0HV9R0080Fe]K06/0A4A405ADE01QHF40?clo05IFEP1dM7@0?3`l05mOG`1kNg/0>c/k
06MWI`1oOgl0<C4a06MWI`1/K6`0>SXj075aL@1dM7@0ADE506EUI@1]KFd0De=C06ATI01BDU80IVIV
05QHF01@D500OGem05ADE01@D500NWYj05EEE@1CDe<0MgMg059BDP1EEED0QXJ605]KF`1NGUh0OWin
06MWI`1<C4`0Lg=c065QH@1MGEd0PH6106ATI01QHF40N7Qh07]kN`1iNGT0K6a/07YjNP1`L700FUYJ
07UiN@1mOGd0ADE50861P@1mOGd0>CTi07alO023Ph<0CTi>06QXJ029RHT0D51@08j>SP1lO7`0E5AD
092@T01ZJVX0KFe]07alO01HF5P0LW9b08:2PP1IFET0JVYZ065QH@2<S8`0HV9R05iNGP2HV9P0GemO
05aLG02HV9P0JVYZ04e=C@2EUID0IfMW0451@@1dM7@0EUIF05ADE01QHF40I6AT05mOG`2BTY80DU9B
06i^KP1iNGT0E5AD07IfMP1VIVH0Fe]K05eMG@0204E5A@0506QXJ01DE5@0ADE507EeM@1TI6@03_oo
ool0C@1GEeL0EUIF05EEE@1NGUh0B4Q805UIF@1WIfL0R8R806EUI@1ZJVX0?3`l05MGE`1CDe<0ATI6
06UYJ@1MGEd0GUiN055AD@1@D500ATI604=3@`1;Bd/0DU9B04U9B@1fMWH0@41004m?C`1=CDd0Bd];
04A4A00m?Cd0F5QH04i>CP1HF5P0EeMG05=CD`1`L700ADE50492@P1EEED0Lg=c04U9B@0k>c/0@D51
03Lg=`0o?cl0ADE504Q8B00m?Cd0De=C04i>CP12@T80?Shn04Y:BP1HF5P0D51@05iNGP1[Jf/0EeMG
06ATI01gMgL0Jf][07AdM01nOWh0NWYj06m_K`1bLW80RHV907emO@1TI6@0@D5106MWI`1VIVH0DE5A
08>3P`1aLG40GemO0080LG5a03d0YZJV05QHF01IFET0KFe]06i^KP1UIFD0C4a<05mOG`1[Jf/0GemO
05UIF@1[Jf/0GEeM05MGE`1]KFd0GUiN05YJFP1WIfL0C4a<06=SH`1NGUh0LG5a05YJFP1OGel0Cdm?
03Ph>01EEED0FUYJ05]KF`1@D500OGem071`L01nOWh0G5aL079bLP1QHF40GemO05MGE`1VIVH0J6QX
05aLG01UIFD0HV9R04];B`19BDT0Kfm_07UiN@1fMWH0Rh^;06IVIP1[Jf/0X:2P07emO@1[Jf/0Fe]K
04a<C01[Jf/0B4Q807UiN@1ZJVX0OWin00kooooo0700OWin07=cL`1VIVH0EeMG05YJFP0o?cl0Jf][
03lo?`16ATH0>c/k03`l?00b<S80<C4a04U9B@18B4P0HF5Q04=3@`0c<c<0>c/k04];B`16ATH0Bd];
04i>CP1FEUH0Hf=S05UIF@17AdL0G5aL0492@P11@D40MGEe030`<01[Jf/0C4a<05YJFP1BDU80Cdm?
05YJFP0m?Cd0N7Qh08>3P`1=CDd0IVIV05YJFP14A4@0Ogmo08Z:RP1mOGd0Kfm_04Y:BP1[Jf/0FUYJ
04m?C`1fMWH0CDe=06UYJ@1LG5`0DU9B06ATI01QHF40H61P075aL@1LG5`0FUYJ06ATI01eMGD0IFEU
04a<C01VIVH0HV9R07alO01eMGD0SHf=06a/K01jNWX0Lg=c06QXJ01gMgL0O7al08f=S@2TY:@0DU9B
07alO01eMGD0H61P07IfMP2<S8`0GUiN05EEE@1BDU80RXZ:059BDP1IFET0IVIV07MgM`0o?cl0GEeM
05EEE@17AdL0IfMW08N7Q`18B4P0DE5A059BDP1KFe/0EUIF07MgM`1FEUH0H61P04=3@`1KFe/0G5aL
0P1VIVH06P1KFe/0M7Ad051@D01HF5P0IFEU04e=C@1SHf<0EUIF05eMG@1VIVH0C4a<065QH@1RHV80
@T9207MgM`19BDT0DU9B05QHF01oOgl0GEeM07emO@0[:b/0EEEE04A4A01MGEd0DU9B3_ooool0@P18
B4P0DE5A07inOP1JFUX0>SXj03De=@1TI6@0PX:205YJFP0T92@0=CDe02XZ:P2?Shl0NWYj06QXJ01/
K6`0MWIf07UiN@1`L700IVIV05mOG`25QHD0XJ6Q06][J`1iNGT07AdM01DE5@0K6a/07alO02/[:`13
@d<0IfMW0:BTY02GUiL0=CDe01dM7@0O7al09bLW06e]K@2SXj<0WinO07MgM`19BDT0EUIF051@D01[
Jf/0JFUY034a<@0P8200BDU904];B`1mOGd0FUYJ065QH@1^KVh0;Rh^02/[:`0o?cl0PH6109:BTP1l
O7`0I6AT09bLW02HV9P0^k^k0;>c/`80WinO04P0Lg=c08R8R01HF5P0Rh^;08Z:RP2<S8`0Vi^K051@
D01FEUH0:RXZ01dM7@13@d<0N7Qh030`<00U9BD0;Bd]06i^KP1ZJVX0ADE508J6QP1FEUH0Cdm?0410
@01FEUH0aLG50<O7a`2@T900XZ:R05mOG`28R8P0Xj>S0451@@1WIfL0JFUY04U9B@2ATI40bl_;0<K6
aP2FUYH0^KVi0<o?c`1lO7`0Bd];07QhN01/K6`0H61P08b<S01eMGD0Ng]k04];B`1EEED0ADE5092@
T02IVIT0>SXj03Hf=P1>CTh0M7Ad0:6QX@1BDU80=CDe0492@P18B4P0BDU904];B`1VIVH0CDe=02TY
:@25QHD0Xj>S05eMG@13@d<Dooooo`004oooool00`2FUYH0NWYj07=cL`0206a/K00>04m?C`1YJFT0
OGem05UIF@1[Jf/0OWin05IFEP1XJ6P0HF5Q05mOG`1FEUH0IFEU05MGE`18B4P207AdM00I05aLG01A
DE40NWYj05iNGP0j>SX0Kfm_05MGE`1BDU80G5aL06a/K01DE5@0H61P06][J`1NGUh0Kfm_06UYJ@13
@d<0I6AT06=SH`1BDU80HV9R07=cL`1TI6@0GEeM071`L00206=SH`1K06ATI01hN7P0BDU9069RHP1^
KVh0CDe=05aLG01iNGT0DU9B06QXJ02CTi<0CTi>06e]K@1hN7P0EEEE071`L01]KFd0Kfm_04a<C01M
GEd0Kfm_05ADE01/K6`0Ng]k065QH@1`L700PX:206YZJP1MGEd0NWYj06m_K`1RHV80NWYj075aL@1K
Fe/0Ph>306QXJ01?Cdl0NGUi06=SH`1IFET0GUiN051@D01QHF40OGem06UYJ@1_Kfl0LG5a06i^KP1T
I6@0DE5A08J6QP1bLW80IFEU06e]K@1PH600KFe]071`L01ADE40HF5Q07EeM@1@D500Fe]K06YZJP1`
L700DU9B07inOP1iNGT0DE5A05mOG`1kNg/0EEEE06MWI`1/K6`0DE5A07QhN01SHf<0EEEE06m_K`1[
Jf/0IVIV0861P@1TI6@0EUIF06UYJ@1RHV80DE5A07MgM`1IFET0EEEE06i^KP0>ooooo`07061PH01X
J6P0Fe]K06ATI01@D500DU9B05mOG`0206][J`0_05iNGP0m?Cd0De=C04e=C@14A4@0H61P05EEE@1F
EUH0F5QH06IVIP19BDT0A4A4059BDP1EEED0ATI606EUI@1=CDd0EUIF04];B`1JFUX0Fe]K05=CD`1U
IFD0Cdm?06=SH`1[Jf/0J6QX08V9R@17AdL0Bd];07IfMP2KVi/0Hf=S04U9B@1=CDd0A4A404Q8B01<
C4`0CDe=04Y:BP1;Bd/0Cdm?04=3@`14A4@0F5QH05EEE@1>CTh00P1FEUH0=@1>CTh0AdM705MGE`1X
J6P0G5aL07UiN@1bLW80IfMW075aL@1OGel0LG5a06][J`1`L700HF5Q05IFEP11@D40OGem06][J`1M
GEd0F5QH06a/K02g]kL0M7Ad06MWI`1YJFT0KVi^06MWI`1KFe/0N7Qh07EeM@1SHf<0EUIF07UiN@1H
F5P0EeMG06QXJ01UIFD0L71`075aL@1LG5`0Jf][075aL@1cLg<0K6a/05MGE`1=CDd0?3`l055AD@1M
GEd0Fe]K05QHF01TI6@0OWin0080HF5Q01/0K6a/04Y:BP10@400De=C06IVIP1GEeL0C4a<06ATI01@
D500IVIV03Hf=P1CDe<0G5aL05MGE`1fMWH0K6a/08>3P`2=SHd0S8b<092@T01PH600GUiN079bLP1R
HV80N7Qh07YjNP25QHD03_ooool206YZJP0705QHF01CDe<0C4a<02l_;`1MGEd0?3`l03Ph>002038b
<P2102`/;00Z:RX0>3Ph0451@@1>CTh0;Rh^02@T900Z:RX0@D5104U9B@0j>SX0CTi>04U9B@1bLW80
ATI604A4A01<C4`0Bd];04Y:BP1XJ6P0?Shn059BDP1DE5@0EeMG04Y:BP1MGEd0MWIf03Ti>@1PH600
HF5Q05EEE@1<C4`0I6AT04m?C`1bLW80NGUi08V9R@1TI6@0HV9R05]KF`1NGUh0G5aL06a/K01OGel0
FUYJ051@D01NGUh0FEUI05]KF`1ADE40LG5a059BDP1RHV80H61P08>3P`1EEED0IVIV06MWI`1GEeL0
OWin075aL@29RHT0IfMW06UYJ@1SHf<0JVYZ07MgM`1jNWX0I6AT0:NWY`1HF5P0L71`08>3P`1/K6`0
MGEe09RHV01SHf<0De=C059BDP1jNWX0Bd];05EEE@1ZJVX0MWIf04m?C`1YJFT0I6AT04M7A`1_Kfl0
NGUi05aLG014A4@0G5aL06UYJ@1OGel0Hf=S069RHP1YJFT0GUiN055AD@1OGel0LG5a06QXJ01HF5P0
Ng]k059BDP1>CTh0I6AT04I6AP1[Jf/0C4a<06QXJ01IFET0EeMG06][J`1/K6`0ADE505]KF`15ADD0
GUiN05QHF01oOgl0JVYZ07YjNP0l?3`0IVIV04E5A@1_Kfl0Fe]K00kooooo01/0C4a<03Ti>@1OGel0
CTi>02h^;P0d=3@0AdM70:2PX020P800?Shn030`<01NGUh0IFEU09:BTP1?Cdl0CDe=07MgM`2=SHd0
MgMg065QH@18B4P0LG5a0:JVYP29RHT0N7Qh05]KF`0V9RH00P0L71`0<00K6a/0HV9R08b<S01nOWh0
Ti>C038b<P0P820071`L020P801ZJVX0IVIV09>CT`0d=3@07QhN02/[:`1=CDd0HV9R06][J`1OGel0
IVIV06YZJP1mOGd0Q8B406=SH`1eMGD0MgMg03<c<`0O7al0;bl_08R8R02;Rh/0][Jf09^KV`2j^[X0
`L7108J6QP2OWil0^[Zj0:j^[P2JVYX0KFe]03Lg=`1YJFT0K6a/07YjNP1=CDd0OWin055AD@8071`L
03d0C4a<06UYJ@1KFe/0;bl_02h^;P1VIVH0QhN707YjNP1oOgl0C4a<02<S8`0O7al08R8R08f=S@1d
M7@0GUiN0:>SX`0l?3`0>c/k09BDU017AdL0CTi>04m?C`12@T80Hf=S09^KV`22PX80FUYJ0:ZZZP2W
YjL0?3`l04i>CP29RHT0>SXj04A4A01YJFT0YJFU05mOG`0o?cl0YZJV08N7Q`2RXZ80^k^k07inOP1j
NWX0PH610:f][@2SXj<0>3Ph024Q8@0[:b/0GUiN0820P01]KFd0UIFE05iNGP2@T900^;Rh05QHF012
@T80Ng]k01Cooooo000Cooooo`2408>3P`1nOWh0HF5Q0820P01GEeL0CTi>06YZJP1]KFd0GUiN06e]
K@1jNWX0I6AT04i>CP1GEeL0LG5a03Hf=P1GEeL0JVYZ03`l?01VIVH0NWYj04I6AP1;Bd/0Rh^;04];
B`1RHV80MgMg034a<@1MGEd0Cdm?03hn?P1VIVH0AdM704e=C@1NGUh0ADE506QXJ01KFe/0F5QH07al
O01ADE40FEUI09:BTP1FEUH0B4Q808Z:RP26QXH0De=C08f=S@1^KVh0@D5107AdM01PH600D51@07Ad
M01QHF40D51@06][J`1VIVH0FEUI08R8R01RHV80O7al06m_K`1KFe/0OWin05eMG@1]KFd0O7al04Q8
B01gMgL0LW9b03lo?`2:RXX0SHf=05=CD`1TI6@0NWYj055AD@28R8P0V9RH04I6AP2CTi<0[Jf]069R
HP2IVIT0Ph>306YZJP27QhL0IfMW05QHF02<S8`0JFUY07IfMP1cLg<0Fe]K09JFUP1QHF40EEEE08V9
R@1SHf<0WYjN06IVIP1RHV80U9BD055AD@1DE5@0T92@0820P01_Kfl0Lg=c08R8R01cLg<0DE5A06i^
KP2>SXh0AdM706IVIP2KVi/0;bl_071`L02;Rh/0>c/k07IfMP1YJFT0?3`l07AdM01aLG40?Cdm06i^
KP1[Jf/0AdM70P1[Jf/01@14A4@0K6a/06i^KP0f=SH0Ogmo00kooooo01T0KVi^06UYJ@1VIVH0IfMW
04Q8B01>CTh0IVIV075aL@1NGUh0G5aL0410@01>CTh0EEEE04];B`1WIfL0FUYJ05]KF`1XJ6P0JVYZ
04];B`1?Cdl0E5AD06IVIP1EEED0M7Ad0080FEUI03D0EeMG06=SH`1@D500IFEU061PH019BDT0Fe]K
06MWI`1YJFT0M7Ad04=3@`1GEeL0H61P0:>SX`1JFUX0ADE504M7A`14A4@0C4a<04];B`1BDU80?clo
04Q8B01EEED0AdM703`l?01BDU80G5aL05iNGP1HF5P0HF5Q04U9B@1RHV80MGEe079bLP1LG5`0KVi^
07MgM`1UIFD0Ogmo079bLP1iNGT0JVYZ06e]K@1@D500EUIF04M7A`1/K6`0EUIF04e=C@1IFET0Fe]K
0;6a/@02059BDP0G05iNGP1HF5P0E5AD04e=C@1WIfL0IVIV04=3@`1GEeL0Q8B4051@D017AdL0EEEE
055AD@1DE5@0JFUY069RHP1ADE40G5aL06EUI@1EEED0FUYJ059BDP0i>CT00P1@D5008@1DE5@0EUIF
04Y:BP1aLG40KFe]04=3@`1JFUX0Fe]K04U9B@1FEUH0G5aL05eMG@1OGel0Jf][04=3@`1^KVh0A4A4
06QXJ01RHV80IFEU08>3P`1SHf<0QXJ60;2`/01hN7P0M7Ad04U9B@1XJ6P0IfMW04a<C025QHD0MGEe
08:2PP0>ooooo`1W07QhN01/K6`0G5aL05eMG@14A4@0>3Ph05MGE`0h>3P0@D5103Hf=P11@D40>CTi
02TY:@0k>c/0@D5104e=C@0Y:BT0:2PX03Hf=P0j>SX0HF5Q03Hf=P1?Cdl0CTi>06IVIP16ATH0E5AD
04U9B@0l?3`0DU9B07AdM00i>CT0BDU9059BDP1IFET0ADE505QHF01jNWX0?clo07EeM@1NGUh0G5aL
04m?C`1KFe/0EUIF07AdM01oOgl0SXj>06][J`1PH600EeMG05]KF`1GEeL0KVi^06EUI@1LG5`0DU9B
05eMG@1JFUX0E5AD059BDP1^KVh0BDU9059BDP1LG5`0N7Qh04E5A@1RHV80E5AD04Q8B020P800Kfm_
06ATI01YJFT0Rh^;05MGE`1WIfL0GemO07YjNP17AdL0ZJVY04e=C@1LG5`0JVYZ06][J`1ZJVX0OGem
06=SH`1QHF40DE5A07IfMP1JFUX0EEEE06YZJP1hN7P0De=C05iNGP1]KFd0@d=3071`L01XJ6P0HV9R
05ADE00206=SH`0S06ATI01<C4`0IVIV05YJFP1?Cdl0De=C06e]K@1VIVH0Kfm_06MWI`20P800GUiN
059BDP1SHf<0@d=307IfMP16ATH0Ogmo059BDP1^KVh0PX:2069RHP17AdL0L71`04E5A@1HF5P0EeMG
08J6QP1XJ6P0UIFE0492@P1`L700=CDe07]kN`1EEED03_ooool0=@1>CTh0:2PX02@T901FEUH0E5AD
05eMG@27QhL0W9bL05eMG@1GEeL0K6a/071`L01KFe/07AdM02PX:01BDU80PX:20861P@1TI6@0;bl_
03<c<`1aLG40H61P0861P@1hN7P0UYJF06=SH`1BDU80@41005IFEP2HV9P0O7al05]KF`1QHF40E5AD
06ATI01GEeL0K6a/06IVIP2OWil0SHf=02HV9P0M7Ad0;bl_07QhN02VYZH0U9BD07inOP1ZJVX0M7Ad
04Q8B01ZJVX0F5QH0080RXZ:00@0GemO06i^KP1oOgl0Bd];0P2JVYX05P1/K6`0/k>c09:BTP1ADE40
JVYZ08n?S`1_Kfl0?3`l08f=S@0R8R80?clo06ATI018B4P0?Shn02h^;P2LW9`0EEEE04i>CP2@T900
UiNG051@D00e=CD205aLG00g07AdM02>SXh0[:b/069RHP0N7Qh07AdM03Lg=`2EUID0;Bd]030`<01l
O7`0;Bd]024Q8@1UIFD0YZJV04e=C@2CTi<0Rh^;06=SH`1kNg/0QHF502/[:`2<S8`0Xj>S071`L00f
=SH0GUiN04];B`1BDU80SXj>03/k>`1RHV80UYJF0;Jf]P2k^k/0/;2`0;6a/@2RXZ80RXZ:06QXJ02D
U9@0Y:BT05IFEP0d=3@0IFEU07]kN`2ATI40Vi^K09>CT`2WYjL0cLg=0:n_[`1YJFT0X:2P079bLP0D
ooooo`004oooool0IP1mOGd0LW9b07YjNP1kNg/0JVYZ05]KF`1JFUX0Lg=c05mOG`1_Kfl0Ti>C059B
DP1<C4`0O7al05MGE`1CDe<0IVIV05=CD`1HF5P0Kfm_05IFEP1[Jf/0E5AD05mOG`1bLW80<c<c05aL
G01cLg<0?3`l071`L01[Jf/0A4A4055AD@25QHD0@41005aLG027QhL0;Bd]05iNGP1hN7P0?Cdm05UI
F@1fMWH0ADE5061PH01QHF40FUYJ05ADE01=CDd0OWin04U9B@1>CTh0N7Qh03Xj>P1?Cdl0QhN704=3
@`1<C4`0Shn?04E5A@1MGEd0QhN704m?C`1SHf<0Kfm_05]KF`1GEeL0Lg=c06IVIP1FEUH0M7Ad05IF
EP1dM7@0P82006IVIP1aLG40HV9R06UYJ@1]KFd0BTY:069RHP21PH40AdM706][J`25QHD0?clo07Mg
M`1kNg/0@D5107alO01UIFD0GEeM09FEU@1oOgl0K6a/08b<S01RHV80J6QX06EUI@1@D500O7al06][
J`80OGem02<0Hf=S09FEU@1>CTh0JVYZ09NGU`1XJ6P0Jf][092@T01OGel0HF5Q07moO`1SHf<0H61P
07YjNP1aLG40K6a/06m_K`1GEeL0IFEU08f=S@1DE5@0IfMW06][J`1aLG40NWYj06UYJ@1PH600HV9R
06MWI`0m?Cd0K6a/06QXJ01@D500L71`05ADE00>ooooo`0805IFEP1SHf<0DE5A05UIF@1?Cdl0C4a<
04e=C@1UIFD205MGE`1F03lo?`1?Cdl0D51@04U9B@1;Bd/0A4A404=3@`1BDU80Hf=S04A4A019BDT0
IVIV061PH01BDU80Ph>305MGE`1SHf<0D51@055AD@1EEED0Lg=c06a/K01ADE40Hf=S06UYJ@1SHf<0
MWIf05UIF@1KFe/0Cdm?079bLP1QHF40Bd];055AD@1>CTh0Cdm?04Y:BP1<C4`0AdM7055AD@1]KFd0
CTi>04M7A`1GEeL0IFEU06=SH`1]KFd0LG5a04A4A01`L700P82006MWI`1`L700YjNW08F5Q@1fMWH0
R8R806a/K02=SHd0HV9R07YjNP1>CTh0EUIF04M7A`1`L700CDe=04i>CP1LG5`0LG5a0;Bd]01QHF40
GUiN06=SH`1BDU80IFEU059BDP1WIfL0KVi^04a<C01SHf<0IfMW055AD@1:BTX0ADE503Xj>P1TI6@2
05ADE00S06ATI01YJFT0GemO05mOG`14A4@0BDU903Lg=`16ATH0A4A4051@D01:BTX0De=C05QHF01C
De<0?3`l051@D018B4P0BTY:06MWI`1lO7`0N7Qh05YJFP1eMGD0FUYJ05aLG01ADE40KVi^06ATI01V
IVH0SXj>065QH@1YJFT0Q8B409^KV`2ATI400P1cLg<01@1nOWh0HF5Q08R8R01hN7P0P82000kooooo
01/0MgMg05aLG01RHV80E5AD03dm?@0h>3P0DE5A0492@P0m?Cd0;Bd]04Y:BP0n?Sh0@D5103<c<`1;
Bd/0B4Q802PX:00a<C40;2`/03`l?01DE5@0<30`04Y:BP0o?cl0Lg=c061PH01BDU800P1;Bd/03@19
BDT0JFUY03`l?010@400BDU903lo?`0n?Sh0GEeM079bLP0h>3P0J6QX05=CD`1KFe/00P1FEUH0?01:
BTX0GUiN05]KF`25QHD0G5aL059BDP1UIFD0CDe=05=CD`1]KFd0Fe]K06a/K013@d<0E5AD069RHP17
AdL0A4A407=cL`17AdL0IFEU05=CD`1fMWH0@T9205aLG01NGUh0A4A4071`L01WIfL0LW9b06IVIP1i
NGT0Hf=S05]KF`1hN7P0R8R806UYJ@2][Jd0GEeM07IfMP1TI6@0GUiN08J6QP1jNWX0IFEU07alO019
BDT0JFUY04M7A`11@D40HV9R065QH@15ADD0I6AT05YJFP0h>3P0Lg=c079bLP1WIfL0EeMG06QXJ080
F5QH0280ATI6071`L01=CDd0F5QH05MGE`1XJ6P0Jf][07YjNP1HF5P0J6QX07=cL`19BDT0FEUI03hn
?P1WIfL0?clo04i>CP1OGel0JFUY069RHP1IFET0DE5A061PH00f=SH0L71`055AD@1dM7@0PX:209VI
V@0d=3@0GEeM04i>CP1cLg<0D51@3_ooool0S015ADD0FUYJ04A4A01PH600G5aL04Q8B017AdL0P820
07inOP1/K6`0PH6108n?S`1eMGD0:RXZ03Ti>@1BDU80;bl_03dm?@1HF5P08b<S028R8P0f=SH0H61P
0:FUY@28R8P0AdM703lo?`1<C4`0F5QH08J6QP2i^KT0XJ6Q09^KV`1@D5008R8R02l_;`1?Cdl0UIFE
0;fm_@1JFUX0Bd];05mOG`1=CDd0A4A402d];@0Y:BT0?clo09^KV`2f][H0Xj>S03/k>`0^;Rh0IVIV
08n?S`1DE5@0Jf][075aL@2][Jd0LW9b06YZJP1=CDd0@D5109nOW`2ZZZX0ATI6071`L021PH40Bd];
028R8P1;Bd/0<30`05EEE@24Q8@0?Shn03De=@1/K6`0Ogmo05ADE02VYZH0/K6a0:2PX02k^k/0MGEe
05UIF@1hN7P0?Cdm06EUI@1mOGd0];Bd07MgM`1ZJVX0UIFE0;^k^`0/;2`0;bl_0861P@30`<00VIVI
08f=S@2OWil0WYjN09fMW@1[Jf/0N7Qh0::RXP2l_;`0V9RH0<[:bP1YJFT0ATI609JFUP2MWId0QHF5
09fMW@2GUiL0Q8B408f=S@2VYZH0Kfm_09^KV`29RHT0a<C408B4Q01RHV80?Shn05]KF`21PH40Shn?
0:JVYP1kNg/0M7Ad09FEU@2WYjL0c/k>0:FUY@2LW9`0IFEU06IVIP2QXJ40HV9R5?ooool001?ooooo
02X0RHV909bLW01^KVh0Jf][071`L016ATH0M7Ad069RHP16ATH0FEUI075aL@1mOGd0B4Q8061PH01/
K6`0@41004=3@`1UIFD0FUYJ05]KF`1hN7P0FEUI05EEE@26QXH0BTY:05MGE`1jNWX0>CTi06YZJP1L
G5`0=CDe065QH@17AdL0?clo06UYJ@10@400C4a<06ATI01CDe<0Bd];05iNGP1@D50205UIF@0F04Q8
B01/K6`0JFUY04I6AP1JFUX0Lg=c03Hf=P1EEED0J6QX04M7A`1NGUh0L71`051@D01FEUH0JVYZ05YJ
FP1bLW80J6QX07IfMP1QHF40EeMG07AdM080GEeM01@0VIVI04e=C@1]KFd0LW9b04U9B@22PX80OGem
04M7A`1GEeL0Jf][04M7A`1@D500RHV90451@@1[Jf/0Ph>3061PH01hN7P0M7Ad05YJFP80M7Ad0340
A4A407moO`1UIFD0J6QX07moO`0o?cl0TY:B04i>CP1LG5`0RXZ:055AD@2HV9P0J6QX05YJFP23Ph<0
J6QX065QH@20P800LW9b07EeM@1XJ6P0JVYZ0820P01RHV80GemO08R8R018B4P0GUiN09^KV`0i>CT0
HV9R09bLW015ADD0IVIV07moO`17AdL0JFUY06ATI01>CTh0Jf][07=cL`0b<S80Fe]K07AdM01CDe<0
G5aL07YjNP17AdL0Jf][00kooooo0P1HF5P06P1JFUX0Hf=S04];B`1;Bd/0FUYJ061PH01LG5`0DE5A
03Hf=P19BDT0AdM703/k>`1;Bd/0ATI604i>CP1HF5P0D51@04M7A`1<C4`0DU9B05QHF01BDU80Jf][
04e=C@1QHF40?clo0P1@D500KP1OGel0F5QH04e=C@1RHV80IVIV06=SH`1gMgL0CTi>05=CD`1IFET0
Kfm_04U9B@16ATH0AdM704U9B@1<C4`0BDU9055AD@0o?cl0CTi>051@D01<C4`0Bd];05IFEP1NGUh0
GemO06EUI@1YJFT0BTY:05YJFP1cLg<0NGUi05ADE02OWil0PX:207alO01_Kfl0Ogmo06YZJP1aLG40
P82005iNGP1HF5P0G5aL08f=S@1YJFT0D51@05aLG01kNg/0_kno04m?C`1OGel0JFUY055AD@1YJFT0
F5QH06a/K01ZJVX0B4Q804m?C`1_Kfl0EeMG04];B`1FEUH0AdM706IVIP1SHf<0AdM7055AD@1aLG40
JVYZ05YJFP1FEUH0@D5102h^;P12@T80B4Q8051@D01;Bd/0EEEE04e=C@1[Jf/0<S8b04a<C01ADE40
ADE505YJFP1SHf<0M7Ad05YJFP1XJ6P0DU9B06=SH`1FEUH0HF5Q05ADE01GEeL0UIFE07QhN028R8P0
YZJV0:b/[02ATI40DE5A06YZJP1_Kfl0Bd];07inOP1ZJVX0R8R83_ooool0=P1QHF40I6AT079bLP1M
GEd0>c/k02/[:`18B4P0;2`/03Lg=`0[:b/0=CDe0492@P0m?Cd0:RXZ059BDP0h>3P0;Rh^03@d=00b
<S80<c<c04U9B@1:BTX0ADE503`l?01VIVH0A4A40492@P15ADD0=3@d03De=@1PH600>SXj03hn?P14
A4@0<C4a02d];@1@D500I6AT02XZ:P1ZJVX0Hf=S03hn?P1SHf<0De=C03Hf=P1>CTh0GEeM096AT@1I
FET0H61P06=SH`1?Cdl0BTY:05YJFP80GEeM01X0GemO05iNGP1BDU80BDU9059BDP1jNWX0I6AT05EE
E@1SHf<0GemO03Xj>P1/K6`0HF5Q034a<@26QXH0Jf][069RHP2;Rh/0KFe]05YJFP1_Kfl0GUiN07Yj
NP1eMGD0/K6a04e=C@80Hf=S03P0O7al07UiN@1fMWH0JFUY06YZJP1;Bd/0KVi^065QH@14A4@0JVYZ
065QH@1TI6@0GUiN06UYJ@10@400NWYj06QXJ01PH600UIFE07inOP1YJFT0MGEe04Y:BP1^KVh0CDe=
05UIF@1LG5`0I6AT065QH@1OGel0MWIf05]KF`14A4@0EEEE06MWI`0^;Rh0G5aL03hn?P0k>c/0BDU9
079bLP1fMWH0CTi>04Q8B01TI6@0=CDe05QHF01FEUH0L71`07EeM@2?Shl0@d=307YjNP17AdL0MgMg
04i>CPkooooo07d0BDU906EUI@1gMgL0ATI6051@D00T92@05ADE02PX:01[Jf/0_kno06EUI@2CTi<0
MgMg08J6QP1;Bd/09bLW020P800S8b<0EUIF03Ti>@0S8b<0:2PX09>CT`2=SHd0NGUi03De=@0Y:BT0
;2`/034a<@1[Jf/0WinO0;Vi^@2DU9@0>CTi01hN7P0O7al09BDU0410@02YZJT0IVIV04m?C`13@d<0
:BTY01`L700S8b<092@T03lo?`25QHD0alO70<30`02HV9P0U9BD055AD@0O7al07QhN030`<01MGEd0
/[:b0<_;b`24Q8@0?Cdm02<S8`2KVi/0YjNW08f=S@2CTi<0NGUi051@D00_;bl0@41006e]K@0m?Cd0
7AdM04A4A00i>CT0Lg=c04i>CP1DE5@0c/k>0::RXP23Ph<0Xj>S0861P@20P800Ogmo065QH@1HF5P0
ADE50;Vi^@2>SXh0=SHf08^;R`2ATI40D51@02l_;`2;Rh/0IfMW0:6QX@27QhL0WYjN0<G5a@2b/[80
?3`l051@D02/[:`0UIFE09RHV02RXZ80TI6A07QhN01cLg<0Ogmo0:RXZ02KVi/0GemO06e]K@1iNGT0
?Shn010@400X:2P0XJ6Q06=SH`0Z:RX0?3`l07AdM00205aLG00=07]kN`2PX:00U9BD0;>c/`1`L700
`L7107moO`15ADD0EUIF07=cL`27QhL0WinO04a<C00Dooooo`004oooool0401oOgl0L71`07UiN@1e
MGD0G5aL06EUI@1DE5@0Jf][05aLG013@d<0Ng]k05eMG@18B4P0LW9b05mOG`17AdL2055AD@0:04Y:
BP1SHf<0JVYZ055AD@1QHF40N7Qh05mOG`1MGEd0G5aL04M7A`80EeMG06d0IfMW05eMG@1:BTX0I6AT
059BDP1>CTh0Kfm_04Q8B01>CTh0L71`0451@@1IFET0LW9b0451@@1PH600GemO059BDP1IFET0DE5A
061PH017AdL0D51@07AdM01GEeL0EEEE07EeM@19BDT0FEUI07AdM01RHV80JVYZ07alO01mOGd0L71`
06QXJ01cLg<0EUIF06MWI`1gMgL0DE5A079bLP1ZJVX0D51@07alO01eMGD0JVYZ06EUI@1WIfL0HV9R
065QH@1VIVH0Jf][05mOG`1[Jf/0NWYj06ATI01mOGd0Ng]k04a<C01[Jf/0L71`05ADE01XJ6P0QhN7
05YJFP1[Jf/0RHV9059BDP1JFUX0LW9b06MWI`1kNg/0I6AT06e]K@1aLG40KFe]05eMG@1/K6`0Rh^;
075aL@1NGUh0N7Qh071`L01PH600O7al0861P@1<C4`0JFUY08f=S@1:BTX0Jf][07QhN01BDU80Ph>3
06e]K@1eMGD0NGUi05QHF01NGUh0NGUi05aLG01ADE40IfMW04i>CP1MGEd0MGEe04E5A@1MGEd0IVIV
00kooooo00P0EUIF04a<C012@T80CTi>0451@@14A4@0C4a<05QHF080A4A400D0=cLg04Y:BP1<C4`0
>SXj04Y:BP0204Q8B00K055AD@1@D500@T9204=3@`1=CDd0BDU904m?C`1BDU80ADE5059BDP17AdL0
B4Q804e=C@1KFe/0CTi>04Q8B01IFET0Jf][069RHP1OGel0CTi>05MGE`1@D500FEUI03dm?@13@d<0
@T920080?clo00D0CTi>059BDP0i>CT0CTi>05UIF@0204Y:BP1G05eMG@1HF5P0GEeM05QHF01MGEd0
BTY:05EEE@1QHF40HV9R04];B`25QHD0MGEe06e]K@1eMGD0NWYj06QXJ01SHf<0EEEE05iNGP1EEED0
Hf=S08:2PP1eMGD0FUYJ07MgM`24Q8@0_[jn06ATI01LG5`0JVYZ06i^KP1WIfL0DU9B06UYJ@1TI6@0
FEUI05QHF01kNg/0JFUY05MGE`1]KFd0CDe=05aLG01EEED0G5aL065QH@1/K6`0GEeM05IFEP1OGel0
Cdm?03Xj>P13@d<0C4a<05]KF`1BDU80E5AD05=CD`1hN7P0B4Q804i>CP1QHF40Cdm?06YZJP1VIVH0
J6QX05eMG@1RHV80C4a<061PH016ATH0HF5Q05mOG`1EEED0Shn?069RHP1gMgL0O7al08N7Q`2GUiL0
IfMW06][J`1hN7P0De=C07]kN`1iNGT0Q8B400kooooo02`0GemO06MWI`1GEeL0MWIf03Xj>P0f=SH0
DE5A03Ph>00g=cL0A4A402d];@0[:b/0@d=303Hf=P1CDe<0=CDe030`<00g=cL0=3@d034a<@1DE5@0
<c<c0410@00k>c/0I6AT059BDP11@D40<C4a03@d=019BDT0De=C03Ti>@1FEUH0@D5102PX:00h>3P0
CDe=06i^KP0h>3P0JVYZ06e]K@1:BTX0J6QX05YJFP80@D5101P0EEEE08f=S@1ADE40G5aL075aL@15
ADD0HF5Q05mOG`1NGUh0GEeM069RHP1IFET0F5QH04e=C@0o?cl0L71`051@D01bLW80Fe]K07MgM`1>
CTh0M7Ad05IFEP1=CDd2071`L00L06EUI@24Q8@0JFUY06IVIP1HF5P0O7al07MgM`15ADD0YjNW06AT
I029RHT0G5aL08:2PP2>SXh0IFEU06MWI`1YJFT0DE5A06i^KP1IFET0E5AD06MWI`1IFET0GUiN06QX
J01NGUh0>3Ph06][J`80H61P00h0Rh^;07=cL`1aLG40Kfm_04M7A`1dM7@0FEUI04Y:BP12@T80KVi^
055AD@1SHf<0OGem06EUI@80De=C01H0EeMG03Xj>P1FEUH0=SHf0451@@1@D500LW9b07QhN017AdL0
DU9B05ADE00f=SH0GUiN06IVIP1PH600I6AT09bLW01@D500IVIV04i>CP1jNWX0E5AD3_ooool05P29
RHT0T92@071`L01UIFD0Hf=S04a<C00d=3@0?Cdm09ZJVP2UYJD0LW9b05EEE@1eMGD0KVi^06EUI@1?
Cdl0AdM703Xj>P1<C4`0R8R806EUI@1IFET20;Fe]@1d0:j^[P25QHD0F5QH04M7A`12@T80?Shn04A4
A01JFUX0MGEe04e=C@0m?Cd0>SXj04I6AP1>CTh0SHf=096AT@1^KVh0MGEe06YZJP0e=CD07alO03@d
=016ATH0GemO0;:b/P2`/;00CTi>06YZJP0V9RH071`L01TI6@0K6a/0:BTY08j>SP2@T900Ng]k03Ti
>@10@400GEeM06i^KP1FEUH0Cdm?065QH@1SHf<0Jf][05=CD`1@D500:RXZ01HF5P0H61P0;Bd]0861
P@2XZ:P0TY:B051@D01HF5P0>SXj04];B`1PH600GemO06m_K`2=SHd0L71`08n?S`1aLG40Ph>30;jn
_P2EUID0T92@0<C4a021PH40Hf=S01XJ6P0L71`0820P09FEU@37alL0Ng]k09>CT`1>CTh0MgMg08B4
Q02][Jd0VIVI0;6a/@1oOgl08R8R020P802@T900XZ:R07QhN02ATI40OWin08F5Q@1:BTX0Jf][09:B
TP1RHV808b<S01lO7`0W9bL08R8R02@T901RHV80^k^k0;bl_02OWil0bl_;071`L00N7Qh071`L02HV
9P2JVYX0/K6a0;Jf]P1VIVHDooooo`004oooool0?`27QhL0MWIf08J6QP1QHF40N7Qh051@D01JFUX0
RHV9051@D01ADE40Vi^K051@D01FEUH0K6a/04I6AP1ADE40CDe=04Y:BP1RHV80IVIV04M7A`1VIVH0
@41004U9B@1ZJVX0:b/[06ATI01JFUX0@T9207=cL`1OGel0@41005ADE01/K6`0>3Ph04M7A`1dM7@0
?Cdm04Y:BP1lO7`0>c/k06=SH`1LG5`0C4a<05eMG@1IFET0O7al055AD@1OGel0Ogmo03Lg=`1GEeL0
IFEU0410@01MGEd0N7Qh03Ti>@1QHF40P8200410@01gMgL0MWIf04i>CP0207EeM@1:069RHP1TI6@0
JFUY05UIF@1VIVH0OWin05EEE@1FEUH0MgMg079bLP1UIFD0EeMG06e]K@1IFET0@T9206a/K01YJFT0
@T9209:BTP1LG5`0ADE509:BTP1YJFT0DE5A08^;R`1KFe/0KFe]07MgM`1SHf<0Ng]k04];B`1cLg<0
GemO03Ti>@1^KVh0IVIV05mOG`1aLG40Hf=S06e]K@21PH40?Shn06e]K@2DU9@0DE5A07emO@1[Jf/0
BDU908b<S01lO7`0LW9b05UIF@1QHF40IVIV05EEE@1XJ6P0GEeM05]KF`1gMgL0H61P06a/K01DE5@0
Lg=c05mOG`1JFUX0JVYZ0492@P13@d<0FUYJ05eMG@1]KFd0I6AT05ADE019BDT>ooooo`1[05IFEP1E
EED0DE5A051@D019BDT0@d=3059BDP1KFe/0F5QH04M7A`0e=CD0ADE504a<C017AdL0H61P05ADE01E
EED0G5aL051@D014A4@0Bd];05QHF01NGUh0Jf][08>3P`1LG5`0F5QH04];B`1KFe/0CTi>065QH@1A
DE40CTi>05MGE`1:BTX0Hf=S061PH01CDe<0EeMG055AD@0j>SX0?clo03Hf=P14A4@0@41004E5A@1:
BTX0ATI604M7A`10@400EUIF04a<C010@400GEeM04U9B@1ZJVX0?clo06EUI@16ATH0FUYJ06m_K`1V
IVH0Fe]K08N7Q`21PH40IFEU08^;R`1_Kfl0IfMW06QXJ01LG5`0DE5A04U9B@1IFET0JFUY05aLG01V
IVH0L71`05UIF@2c/k<0Cdm?06QXJ01dM7@0FUYJ05mOG`1ADE40O7al07YjNP1RHV80DU9B09:BTP1S
Hf<0G5aL06EUI@1HF5P0GUiN04i>CP1EEED0AdM706a/K01RHV80EUIF06ATI01=CDd0?clo04=3@`1<
C4`00P1EEED03P1KFe/0H61P07AdM01?Cdl0G5aL05eMG@13@d<0FUYJ05IFEP1PH600E5AD07alO01R
HV80Ogmo0P1QHF403`1IFET0GEeM08Z:RP1FEUH0QXJ60:FUY@2>SXh0R8R806m_K`1KFe/0GEeM05AD
E029RHT0MWIf0861P@0>ooooo`0:06UYJ@1MGEd0Cdm?06e]K@0d=3@0>CTi04A4A00c<c<0;2`/0410
@0808B4Q0100@d=303Hf=P1BDU80@T9203Xj>P13@d<0;bl_03hn?P1WIfL0;Rh^04i>CP0b<S80KFe]
06EUI@0n?Sh0=CDe0P11@D401017AdL0BTY:04M7A`0h>3P203Ti>@1X04a<C01WIfL0BTY:06QXJ022
PX80K6a/05mOG`1aLG40ADE503@d=01MGEd0SHf=05UIF@1VIVH0O7al04];B`1CDe<0G5aL061PH019
BDT0LW9b05IFEP1VIVH0D51@0492@P1ZJVX0EeMG07MgM`1LG5`0Kfm_05IFEP1mOGd0ADE505aLG01D
E5@0IFEU05QHF01lO7`0JVYZ05YJFP1LG5`0QHF506a/K01kNg/0_[jn06e]K@2=SHd0@T9208N7Q`26
QXH0FEUI08Z:RP1NGUh0@D5105aLG01nOWh0FUYJ05eMG@1KFe/0MWIf06QXJ01LG5`0>c/k07YjNP1S
Hf<0KFe]07UiN@2:RXX0MGEe07alO01IFET0KVi^06m_K`1MGEd0A4A4075aL@1KFe/0Hf=S096AT@1^
KVh0HV9R059BDP1eMGD0?Shn04m?C`0g=cL0H61P059BDP1OGel0Ph>304m?C`19BDT0Jf][0451@@1J
FUX0EUIF06MWI`1]KFd0SXj>04];B`1mOGd0EeMG06QXJ01ADE4>ooooo`0X09jNWP1EEED0H61P03`l
?00V9RH0B4Q8071`L01QHF40J6QX0;no_`38b<P0`L7108f=S@23Ph<0Xj>S09BDU01hN7P0UIFE0<30
`01NGUh0L71`05UIF@22PX80M7Ad03/k>`28R8P0];Bd092@T011@D407QhN02d];@0T92@0MGEe08V9
R@1bLW80GemO04e=C@0a<C40MWIf0820P080GUiN03H0LG5a06MWI`2=SHd0EeMG03`l?01`L700aLG5
0<K6aP1]KFd0?3`l04E5A@0L71`06ATI028R8P1SHf<0UYJF0:2PX02n_[h0WinO02LW9`0/;2`0LW9b
0410@00];Bd0BDU907]kN`1]KFd0KVi^03hn?P1>CTh061PH02XZ:P1FEUH0PX:209JFUP0`<3006ATI
01hN7P0h>3P0<S8b03dm?@0_;bl0RHV908N7Q`1[Jf/0C4a<06ATI02RXZ80=SHf01lO7`10@400f=SH
0=cLg01ZJVX201PH600Z02DU9@2SXj<0LW9b02@T901EEED0=SHf055AD@18B4P0NGUi07alO01DE5@0
[jn_038b<P0d=3@0Ng]k0:VYZ@0a<C40:BTY02DU9@0f=SH0B4Q80;bl_01gMgL0YJFU05YJFP1=CDd0
=SHf04A4A01XJ6P0ZZZZ0<?3``38b<P0^KVi0820P00Y:BT06a/K02@T901bLW80Bd];06QXJ02_[jl0
IfMW5?ooool001?ooooo00d0MgMg06e]K@1UIFD0L71`06e]K@1IFET0G5aL06EUI@1`L700ATI605UI
F@1fMWH0@4100080J6QX00h0@d=305MGE`1^KVh0B4Q8051@D01^KVh0D51@061PH028R8P0C4a<06IV
IP1fMWH0<C4a061PH080BTY:0500JVYZ04E5A@1IFET0IFEU04Q8B01MGEd0Fe]K055AD@1mOGd0FEUI
05aLG01WIfL0B4Q804i>CP1fMWH0G5aL051@D01KFe/0GUiN051@D01XJ6P0G5aL04Y:BP1RHV80EEEE
05QHF01[Jf/0Hf=S05EEE@1nOWh0Jf][05]KF`1[Jf/0GemO07IfMP1LG5`0HF5Q09:BTP13@d<0Ogmo
07alO00n?Sh0Lg=c08>3P`0o?cl0Jf][06ATI01EEED0FEUI09NGU`1@D500JVYZ08>3P`1gMgL0SHf=
08^;R`1dM7@0Kfm_06e]K@1SHf<0MWIf07UiN@2NWYh0N7Qh05mOG`2TY:@0G5aL05mOG`27QhL0JFUY
07inOP1:BTX0GUiN07=cL`1DE5@0EeMG06EUI@1jNWX0MGEe04a<C080Ph>301X0=SHf06i^KP2PX:00
?Cdm06EUI@2=SHd0<C4a05iNGP2<S8`0:2PX07moO`1WIfL0Cdm?08f=S@1CDe<0D51@06IVIP1SHf<0
B4Q806e]K@1fMWH0B4Q806i^KP1MGEd0EUIF06m_K`kooooo03`0FEUI059BDP1@D500FUYJ04i>CP15
ADD0EUIF05iNGP1MGEd0DE5A03dm?@1ADE40BTY:04A4A01HF5P0EeMG05YJFP1SHf<0Cdm?0451@@1>
CTh0C4a<059BDP1DE5@0KVi^04A4A01HF5P0B4Q806EUI@1EEED0Hf=S04e=C@1;Bd/0CDe=05EEE@1X
J6P0HV9R04M7A`1BDU80D51@03`l?012@T80<c<c04M7A`1<C4`0De=C05mOG`1?Cdl0E5AD0410@01I
FET0@d=303/k>`1EEED0DU9B05UIF@18B4P0MgMg05iNGP1HF5P2065QH@0<04Y:BP1iNGT0Ti>C06UY
J@2>SXh0JVYZ05UIF@1MGEd0L71`04m?C`1ADE40C4a<0P1LG5`03`1GEeL0K6a/07alO02o_kl0Ng]k
06a/K01[Jf/0IVIV05iNGP14A4@0Kfm_05MGE`1DE5@0D51@09RHV002069RHP0_05]KF`1TI6@0J6QX
04U9B@1NGUh0EeMG065QH@1jNWX0DU9B05MGE`1>CTh0>3Ph04Q8B01:BTX0HV9R04];B`1:BTX0EeMG
07]kN`1<C4`0BTY:05IFEP13@d<0HV9R04i>CP1EEED0E5AD05iNGP1GEeL0J6QX04U9B@1YJFT0EUIF
06][J`2FUYH0EEEE06][J`1hN7P0Ph>307QhN01IFET0HF5Q04m?C`1CDe<0L71`07=cL`1kNg/03_oo
ool0J@1OGel0LG5a05=CD`2KVi/0@d=30451@@1EEED0?Shn03Ph>018B4P09bLW02<S8`0j>SX0=3@d
061PH01<C4`0A4A404i>CP0j>SX0=SHf07MgM`0k>c/0BDU904E5A@23Ph<0@41004Q8B00l?3`0@410
05=CD`1FEUH0BTY:051@D00l?3`0A4A403@d=01<C4`0EeMG0492@P1<C4`0MgMg05]KF`1]KFd0JFUY
04i>CP0/;2`0I6AT09FEU@18B4P0Lg=c079bLP1CDe<0F5QH04U9B@1PH600G5aL07MgM`1PH600C4a<
059BDP1>CTh0J6QX061PH01eMGD0GEeM07=cL`1CDe<0Lg=c04M7A`1VIVH0HV9R07IfMP1YJFT0Y:BT
06e]K@1cLg<0OGem08f=S@1WIfL0P8200<71`@1GEeL0TI6A04M7A`22PX80QXJ605EEE@1aLG40H61P
03lo?`1QHF40R8R805aLG01QHF40De=C07MgM`1^KVh0Hf=S03`l?01cLg<0FUYJ07IfMP14A4@0J6QX
07MgM`0205]KF`0Q07QhN01HF5P0HV9R04Q8B01UIFD0EeMG05UIF@2BTY80GUiN04i>CP1JFUX0E5AD
03`l?01BDU80:2PX05IFEP19BDT0LW9b08F5Q@15ADD0Fe]K06MWI`0a<C40Hf=S05]KF`1?Cdl0Hf=S
096AT@12@T80F5QH05UIF@1`L700EEEE00kooooo04h0G5aL06m_K`1WIfL0DE5A06a/K01SHf<0IfMW
07UiN@0Y:BT0Z:RX0;:b/P1[Jf/0;2`/038b<P1GEeL0M7Ad04=3@`0i>CT0P82006ATI01WIfL0SXj>
02TY:@2?Shl0CTi>05iNGP1QHF40@d=304Y:BP1TI6@0JFUY06i^KP1bLW80`l?309jNWP1DE5@0>CTi
07]kN`1=CDd09bLW020P800b<S80?clo06ATI02XZ:P0Zj^[0;Bd]01bLW80Jf][07=cL`2j^[X0A4A4
03hn?P16ATH0<S8b06][J`1TI6@0=3@d059BDP2e]KD0[Jf]03lo?`1?Cdl0J6QX051@D00c<c<0=SHf
07emO@1hN7P0>c/k06=SH`23Ph<0F5QH03<c<`0Q8B40P8200<;2`P0k>c/201LG5`0l01lO7`13@d<0
@T9202TY:@1[Jf/0>c/k02@T901OGel0W9bL0;Jf]P1QHF40GemO09BDU02e]KD0^;Rh0;no_`0n?Sh0
>CTi071`L03:b/X0HV9R02<S8`1BDU80ATI6034a<@0c<c<0L71`0492@P14A4@0]kNg07MgM`1MGEd0
EeMG07AdM00T92@06QXJ01TI6@0G5aL0Ph>30;Fe]@21PH40O7al04A4A01:BTX0Q8B406IVIP21PH40
NWYj06YZJP3=cLd0b/[:04E5A@17AdL07QhN065QH@19BDT0ATI609nOW`2NWYh0NGUi5?ooool001?o
oooo08/0MgMg06e]K@1dM7@0K6a/069RHP1OGel0EeMG06QXJ01TI6@0AdM706e]K@1XJ6P0@D5105mO
G`1KFe/0@41005=CD`1MGEd0A4A4071`L01QHF40B4Q805MGE`1XJ6P0E5AD04];B`1HF5P0@d=304Y:
BP1LG5`0FEUI0492@P1=CDd0Kfm_0492@P1CDe<0JFUY0451@@1=CDd0IFEU03Lg=`1FEUH0F5QH03lo
?`1LG5`0DU9B051@D01DE5@0B4Q806IVIP1<C4`0EeMG06IVIP1<C4`0HF5Q06i^KP1:BTX0FEUI07Ui
N@1KFe/0IfMW07IfMP1YJFT0K6a/07alO01]KFd0A4A406a/K01YJFT0CDe=096AT@1WIfL0F5QH07Qh
N01RHV80LG5a06][J`1JFUX0M7Ad06MWI`1^KVh0J6QX05iNGP1UIFD0KFe]05mOG`1_Kfl0LG5a05=C
D`1gMgL0HV9R061PH01ZJVX0Lg=c07EeM@1DE5@0PX:206][J`1XJ6P0F5QH07=cL`1iNGT0EeMG06IV
IP1kNg/0LW9b05EEE@1ZJVX0PH6106a/K01bLW80NGUi08^;R`1DE5@0N7Qh08F5Q@1OGel0GEeM07al
O01HF5P0KFe]07IfMP1=CDd0TI6A06a/K01PH600O7al05mOG`1FEUH0NWYj05mOG`1FEUH0Jf][055A
D@1JFUX0Kfm_059BDP1PH600Fe]K00kooooo0680I6AT05aLG01EEED0F5QH051@D014A4@0EeMG05mO
G`1ADE40C4a<03Lg=`19BDT0De=C04a<C01WIfL0GEeM05ADE01LG5`0FUYJ04I6AP1=CDd0F5QH04];
B`1?Cdl0N7Qh05ADE01NGUh0De=C065QH@1KFe/0JVYZ05UIF@1UIFD0IfMW04I6AP1TI6@0GemO04i>
CP1FEUH0BTY:04I6AP15ADD0=cLg04e=C@15ADD0BTY:05QHF01FEUH0AdM704a<C01IFET0BDU904i>
CP1EEED0F5QH069RHP1<C4`0IfMW061PH01LG5`0K6a/065QH@1PH600U9BD09VIV@1aLG40Ng]k07in
OP1TI6@0Hf=S075aL@1IFET0CTi>05=CD`1QHF40IFEU05IFEP1/K6`0J6QX0:NWY`1ADE40GEeM06][
J`1IFET0F5QH04e=C@1GEeL0H61P04a<C01EEED0Ph>3079bLP1ADE40G5aL03hn?P1<C4`0F5QH05IF
EP80FEUI02H0Hf=S05UIF@1cLg<0Cdm?03dm?@15ADD0AdM705ADE015ADD0C4a<04A4A01fMWH0@d=3
04U9B@1@D500?clo065QH@1GEeL0Cdm?05IFEP1IFET0G5aL05mOG`1GEeL0IFEU04];B`1OGel0SHf=
04e=C@1WIfL0Kfm_06IVIP1ZJVX0EUIF05=CD`1@D500EeMG07QhN080MWIf3_ooool0NP1NGUh0KFe]
06YZJP1gMgL0?Cdm04E5A@1<C4`0Bd];03dm?@1<C4`0;2`/024Q8@14A4@0@41005EEE@1LG5`0<S8b
05ADE00e=CD0<S8b06YZJP0o?cl0D51@05=CD`1XJ6P0Bd];04m?C`0_;bl0D51@05]KF`1@D500HF5Q
04=3@`14A4@0BDU903lo?`19BDT0Fe]K065QH@19BDT0KFe]07UiN@1TI6@0EEEE08b<S00o?cl0GUiN
07alO01HF5P0K6a/07IfMP1MGEd0Jf][05EEE@1RHV80H61P075aL@1UIFD0DE5A051@D01DE5@0HV9R
06UYJ@1SHf<0EEEE06EUI@1RHV80M7Ad05QHF01nOWh0F5QH061PH01[Jf/0PH6105UIF@1]KFd0I6AT
08Z:RP1YJFT0K6a/0:VYZ@1jNWX0U9BD04a<C02IVIT0DU9B05iNGP2FUYH0FUYJ05IFEP1=CDd0P820
075aL@1OGel0B4Q808:2PP1VIVH0Cdm?05ADE01oOgl0HV9R07=cL`1IFET0QXJ606i^KP1hN7P0LW9b
07=cL`1HF5P0IFEU03/k>`1FEUH0DE5A05QHF02?Shl0G5aL06IVIP1=CDd0GEeM04Y:BP1;Bd/09BDU
0P1SHf<0401QHF40M7Ad04m?C`1CDe<0E5AD065QH@1IFET0De=C05MGE`1>CTh0DU9B04I6AP1QHF40
CTi>06][J`1LG5`>ooooo`0e030`<00U9BD09bLW0451@@2HV9P0?Cdm01hN7P0a<C40Ng]k03lo?`0W
9bL0G5aL05eMG@12@T80=cLg04M7A`1:BTX08R8R03Xj>P1mOGd0D51@06e]K@1MGEd0IFEU0410@00j
>SX0<C4a038b<P0i>CT0=3@d02l_;`1ZJVX0>SXj06m_K`1nOWh09RHV02LW9`20P800HF5Q03lo?`0g
=cL0;bl_03dm?@1@D500OWin09^KV`2HV9P0ADE506QXJ02=SHd0_Kfm09>CT`2KVi/00P2e]KD0E@24
Q8@0>CTi05eMG@1fMWH0N7Qh0:b/[025QHD0TY:B05aLG01/K6`0Rh^;09ZJVP1gMgL0Xj>S07alO02?
Shl0/k>c05aLG00N7Qh0:RXZ07emO@2SXj<0H61P038b<P0J6QX05ADE02HV9P18B4P0<C4a04i>CP0O
7al07QhN03Ph>025QHD0V9RH04M7A`0l?3`0CTi>09BDU01DE5@0MgMg0;^k^`26QXH0AdM709NGU`1a
LG40Ogmo08N7Q`0f=SH0=3@d059BDP1IFET0=cLg069RHP2/[:`0PX:209BDU02KVi/0WYjN03Ti>@0P
82006ATI03Xj>P23Ph<0J6QX0:NWY`2e]KD0N7Qh059BDP1gMgL0N7Qh05UIF@1aLG40Xj>S0<C4a01Z
JVX0DU9B07UiN@1TI6@0JFUY06EUI@0m?Cd0H61P02/[:`1NGUh05?ooool001?ooooo08/0O7al07Ad
M01^KVh0I6AT06EUI@1ADE40J6QX06MWI`1?Cdl0J6QX07MgM`1:BTX0GEeM06MWI`19BDT0D51@04=3
@`10@400GUiN05MGE`15ADD0HV9R051@D01PH600J6QX03De=@2>SXh0KVi^0410@02;Rh/0GemO04M7
A`1eMGD0L71`04E5A@1[Jf/0Hf=S04e=C@1JFUX0FEUI04Q8B01GEeL0E5AD061PH01ADE40Cdm?06a/
K018B4P0DU9B07alO00i>CT0F5QH07UiN@0f=SH0EeMG0820P00c<c<0HF5Q07YjNP10@400P82007Mg
M`10@400IVIV06YZJP1QHF40JVYZ06EUI@1SHf<0L71`07YjNP1RHV80IFEU069RHP1lO7`0M7Ad06YZ
JP1_Kfl0F5QH04M7A`26QXH0IVIV055AD@1nOWh0HF5Q04];B`2:RXX0GEeM05iNGP2:RXX0IfMW06e]
K@1bLW80HF5Q07QhN01BDU80QHF506][J`16ATH0OGem06e]K@1VIVH0N7Qh05EEE@1TI6@0RHV902`/
;01eMGD0Ogmo03hn?P22PX80Hf=S065QH@1aLG40KFe]08B4Q01GEeL0IfMW07alO019BDT0FUYJ08F5
Q@1?Cdl0LG5a079bLP1@D500H61P08b<S015ADD0GEeM07emO@12@T80CTi>05mOG`1ADE40D51@07Ee
M@1HF5P0DU9B00kooooo01X0GUiN05MGE`1LG5`0EeMG04e=C@13@d<0FEUI06IVIP1JFUX0Cdm?0410
@018B4P0Bd];04U9B@1JFUX0EEEE069RHP1SHf<0GemO04e=C@1@D500HV9R05]KF`1HF5P0KFe]04i>
CP80EUIF05X0BTY:05MGE`1RHV80DU9B05MGE`1nOWh0>3Ph05mOG`1UIFD0HV9R06MWI`1?Cdl0Fe]K
04e=C@17AdL0D51@03lo?`0e=CD0?Shn055AD@0j>SX0Cdm?059BDP15ADD0De=C04];B`1PH600KVi^
06ATI01KFe/0FEUI05mOG`1[Jf/0KVi^06ATI02<S8`0O7al07IfMP1eMGD0QXJ605QHF01IFET0L71`
05]KF`1;Bd/0G5aL05MGE`1NGUh0@T9205MGE`1[Jf/0_[jn05QHF01bLW80PH6106QXJ01GEeL0A4A4
059BDP1_Kfl0Cdm?05YJFP2;Rh/0PX:2055AD@1MGEd0Cdm?051@D01EEED0@4100451@@1MGEd0Ng]k
061PH01OGel0@D5103`l?00n?Sh0@41004i>CP10@400DE5A03lo?`25QHD0?Cdm05QHF01XJ6P0Hf=S
04e=C@1JFUX205eMG@0D05IFEP1RHV80J6QX04i>CP1GEeL0De=C05UIF@2@T900De=C079bLP1dM7@0
IVIV07IfMP1@D500Cdm?06a/K01BDU80RHV9079bLP1XJ6P>ooooo`0E05YJFP1]KFd0F5QH08:2PP10
@400>SXj055AD@1:BTX0BDU905eMG@0Q8B40>3Ph04m?C`11@D40D51@059BDP17AdL0Fe]K03De=@0o
?cl0C4a<0080@T9202`0ADE5071`L018B4P0EUIF03Hf=P1JFUX0A4A40451@@1`L700>SXj04];B`1B
DU80D51@05]KF`1?Cdl0HV9R05YJFP22PX80IfMW06YZJP1BDU80Xj>S03/k>`1NGUh0Shn?05UIF@1_
Kfl0IVIV07AdM01HF5P0AdM705]KF`10@400IVIV05QHF00n?Sh0D51@05MGE`1SHf<0MGEe06e]K@1H
F5P0Hf=S065QH@80GUiN02D0Jf][059BDP1dM7@0Ng]k09BDU01MGEd0KFe]08B4Q022PX80EEEE07mo
O`2LW9`0NWYj07UiN@1<C4`0Ng]k079bLP1RHV80PX:204e=C@17AdL0Bd];08n?S`1WIfL0EUIF05AD
E01kNg/0HF5Q04i>CP1JFUX0G5aL06ATI01aLG40BDU907=cL`1GEeL0GUiN0080KVi^0200F5QH06AT
I014A4@0EUIF04A4A01GEeL0Rh^;05MGE`1SHf<0HV9R065QH@1<C4`0E5AD01hN7P1SHf<0HV9R0820
P01aLG40?3`l05UIF@17AdL0=SHf07IfMP1lO7`0A4A406YZJP1[Jf/0:RXZ04i>CP1GEeL0MWIf03/k
>`kooooo0440<S8b02<S8`0L71`06ATI03Hf=P1LG5`0:BTY05aLG02IVIT0Cdm?03Ti>@10@400EEEE
02l_;`0j>SX0;Bd]0:JVYP26QXH0T92@0;Bd]02k^k/0_[jn09fMW@2:RXX07alO01PH600Q8B40DE5A
03/k>`1TI6@0R8R80<?3``1=CDd0HV9R05=CD`0o?cl0@41005UIF@0W9bL0820P01hN7P0M7Ad071`L
04Y:BP2=SHd0LW9b08V9R@2^[Zh0[:b/06UYJ@0l?3`0JFUY04Y:BP1`L700[:b/09bLW02KVi/0FEUI
03<c<`1VIVH0Q8B405=CD`2UYJD0`L7108:2PP0209FEU@0K05=CD`1`L700D51@0:NWY`1KFe/0IFEU
0<?3``25QHD0FEUI08b<S01KFe/0?Cdm02TY:@0M7Ad061PH08N7Q`0c<c<0?clo02@T900R8R80KVi^
07MgM`1>CTh07QhN01lO7`0N7Qh0KFe]0080G5aL02`0b/[:07EeM@1<C4`0JFUY0:j^[P2i^KT0Xj>S
07=cL`0e=CD0EUIF05aLG00f=SH0FEUI0:BTY01ZJVX0820P01lO7`1MGEd0Bd];02`/;01HF5P0KFe]
07IfMP2PX:00MGEe09fMW@1;Bd/0ATI607EeM@1dM7@0FEUI03dm?@0n?Sh0KFe]06][J`2?Shl0MGEe
06=SH`1[Jf/0<c<c01lO7`0R8R80:b/[05aLG1Cooooo000Cooooo`0k07AdM029RHT0JVYZ06m_K`1^
KVh0GUiN06MWI`1bLW80HF5Q04U9B@1BDU80JFUY03`l?01]KFd0GemO02l_;`1ADE40MGEe03lo?`1L
G5`0MGEe03Ti>@1FEUH0K6a/03/k>`1?Cdl0CTi>038b<P1PH600Bd];04a<C01/K6`0AdM706][J`1H
F5P0@d=307AdM01ADE40ADE508J6QP14A4@0D51@06YZJP11@D40D51@05mOG`1=CDd0FEUI06YZJP1N
GUh0BDU905UIF@1=CDd0CTi>05IFEP1HF5P0J6QX065QH@1JFUX00P1[Jf/03`1`L700MWIf05iNGP1i
NGT0O7al03De=@1SHf<0M7Ad02l_;`1dM7@0QHF503Ti>@1[Jf/0IFEU0492@P0206a/K00m04M7A`1h
N7P0MGEe05aLG01XJ6P0HF5Q071`L01ADE40F5QH07alO01FEUH0FUYJ071`L023Ph<0Hf=S0:FUY@1O
Gel0DU9B09JFUP1OGel0I6AT05QHF01cLg<0OGem04Q8B01aLG40MGEe061PH01WIfL0EEEE08J6QP2;
Rh/0EeMG07=cL`2<S8`0CTi>07YjNP23Ph<0Cdm?07YjNP22PX80=SHf05ADE01^KVh0?3`l06i^KP1M
GEd0I6AT08>3P`1JFUX0IVIV07]kN`1SHf<0Fe]K0861P@1MGEd0Fe]K07moO`1IFET0HV9R05UIF@0>
ooooo`2<06YZJP1PH600EUIF05UIF@18B4P0@T9205ADE01RHV80FUYJ05UIF@0i>CT0CTi>0451@@1;
Bd/0KVi^065QH@1UIFD0J6QX06MWI`11@D40ATI605mOG`1?Cdl0C4a<06ATI016ATH0F5QH04A4A01D
E5@0CTi>05aLG01EEED0Cdm?06EUI@13@d<0Ph>307IfMP1EEED0FEUI04M7A`15ADD0?3`l03lo?`1:
BTX0@d=30410@01;Bd/0B4Q803/k>`1?Cdl0D51@04];B`1=CDd0HF5Q07AdM01OGel0IFEU06MWI`1F
EUH0JVYZ06ATI01YJFT0IVIV06YZJP29RHT0JFUY071`L023Ph<0Fe]K05aLG01EEED0E5AD05QHF01U
IFD0G5aL05iNGP1CDe<0KFe]05]KF`2HV9P0GemO06i^KP1UIFD0IfMW05UIF@1BDU80EeMG079bLP1M
GEd0IfMW08^;R`28R8P0GEeM05YJFP0i>CT0De=C05eMG@1SHf<0DE5A07AdM01TI6@0Cdm?05iNGP1G
EeL0=cLg03dm?@13@d<0EeMG0451@@1?Cdl0@D51079bLP12@T80Bd];05aLG01WIfL0HV9R05mOG`1J
FUX0Bd];04e=C@1LG5`0J6QX059BDP1NGUh0@d=306EUI@2=SHd0ATI607emO@1/K6`0G5aL05ADE01I
FET0DE5A071`L01@D500MGEe07MgM`29RHT>ooooo`0A05iNGP1IFET0MGEe09VIV@0_;bl0>SXj03Ph
>01KFe/0=cLg04E5A@0`<300;Bd]04I6AP0c<c<0>3Ph061PH01FEUH00P0f=SH0I00j>SX0I6AT03`l
?011@D40EEEE05iNGP1EEED0EeMG034a<@1EEED0DU9B03<c<`1XJ6P0=CDe04a<C01LG5`0CTi>05iN
GP1HF5P0JFUY04M7A`1CDe<0PX:205YJFP10@400W9bL05MGE`1IFET0L71`075aL@1NGUh0KVi^06YZ
JP1`L700DE5A05=CD`1;Bd/0FEUI05]KF`0o?cl0ADE505QHF01TI6@0LG5a06IVIP1EEED0F5QH07mo
O`14A4@0FEUI07IfMP0g=cL0M7Ad069RHP1nOWh0ADE506IVIP1/K6`0RXZ:06][J`1QHF40L71`08b<
S028R8P0>CTi07emO@1dM7@0De=C08j>SP1;Bd/0CDe=059BDP2>SXh0L71`04a<C011@D40Ph>306=S
H`1;Bd/0KFe]05EEE@14A4@0Q8B405QHF022PX80DU9B05eMG@1/K6`0QXJ605QHF01JFUX0A4A4065Q
H@1=CDd0?Shn07alO01/K6`0D51@05mOG`1MGEd204Y:BP0C02LW9`1ZJVX0EUIF05IFEP1RHV80E5AD
06m_K`1?Cdl0ADE505QHF01SHf<0?3`l08>3P`1lO7`07alO03lo?`1HF5P0GUiN02d];@0>ooooo`1B
03Xj>P0T92@071`L01LG5`0O7al0M7Ad0:BTY029RHT0U9BD04M7A`11@D40=cLg038b<P17AdL0DE5A
03hn?P1QHF40;Rh^04Y:BP2o_kl0Z:RX09ZJVP1>CTh0=cLg07EeM@1CDe<0EeMG04A4A01LG5`0SHf=
04e=C@0o?cl0TI6A0;Ng]`25QHD0SHf=079bLP0k>c/07QhN01`L700J6QX07AdM01XJ6P15ADD0J6QX
07moO`2KVi/0];Bd09nOW`2l_;`0IfMW03Xj>P2>SXh0UIFE08>3P`2VYZH0]kNg071`L00m?Cd0>SXj
04M7A`1EEED0_;bl0<W9b@1WIfL0BTY:05ADE01CDe<0JFUY06IVIP1DE5@0FUYJ061PH02NWYh0IfMW
020P800K6a/0J6QX05ADE00o?cl0F5QH05aLG080W9bL00H0N7Qh06IVIP1nOWh0^;Rh0;^k^`0i>CT2
01dM7@0`01lO7`10@400:2PX03Lg=`2FUYH0=cLg03De=@1OGel0EEEE09bLW02TY:@0QhN702<S8`0_
;bl0HV9R0820P02b/[80`<3005iNGP14A4@0>c/k071`L020P800M7Ad0:n_[`1SHf<0OWin09>CT`25
QHD0I6AT0;Zj^P1@D500?Cdm03<c<`0V9RH08b<S01lO7`12@T80FEUI08B4Q01/K6`0>3Ph03De=@0a
<C4071`L01lO7`0[:b/0Shn?5?ooool001?ooooo08/0PX:206YZJP1bLW80GemO05eMG@1TI6@0CDe=
08R8R01RHV80GEeM07QhN01DE5@0DU9B0861P@14A4@0E5AD06QXJ013@d<0F5QH05eMG@11@D40H61P
05MGE`1UIFD0NGUi03Lg=`1JFUX0JVYZ03Lg=`1dM7@0QHF503lo?`1fMWH0SHf=03hn?P1VIVH0Ogmo
04=3@`1>CTh0EeMG04I6AP1RHV80BTY:075aL@1PH600@d=3071`L01NGUh0?Shn07inOP1>CTh0E5AD
07alO00m?Cd0De=C08j>SP0j>SX0FUYJ08>3P`0o?cl0Jf][07QhN01=CDd0De=C08f=S@1@D500F5QH
06a/K01JFUX0HV9R07EeM@1DE5@0Hf=S06a/K01RHV80KVi^05aLG01WIfL0Lg=c04m?C`1bLW80LG5a
04Y:BP2RXZ80Ng]k05QHF02VYZH0Hf=S04a<C027QhL0D51@06EUI@1gMgL0IfMW06][J`1KFe/0LW9b
08j>SP1CDe<0DU9B0820P01dM7@0Lg=c05eMG@1VIVH0VYZJ0492@P1YJFT0LG5a04Y:BP29RHT0PH61
05iNGP24Q8@0I6AT06MWI`1ZJVX0G5aL08B4Q01/K6`0EUIF07UiN@1UIFD0Hf=S06a/K01DE5@0FUYJ
07QhN019BDT0FUYJ07QhN019BDT0ADE5069RHP1DE5@0IVIV07moO`1BDU80Bd];00kooooo00H0JFUY
05=CD`1HF5P0De=C04E5A@1:BTX2069RHP0=05MGE`1LG5`0>c/k04I6AP17AdL0CTi>05mOG`1QHF40
IVIV05aLG01MGEd0@D5104];B`0204m?C`1:04Q8B01]KFd0@d=305ADE017AdL0EUIF04];B`1LG5`0
Cdm?05aLG01SHf<0D51@079bLP1WIfL0G5aL06EUI@1DE5@0Bd];04a<C01;Bd/0DE5A04m?C`1:BTX0
Fe]K0492@P1BDU80Cdm?05mOG`1<C4`0Fe]K06QXJ01YJFT0EUIF071`L01JFUX0EUIF079bLP1/K6`0
HF5Q05YJFP1jNWX0QXJ606m_K`1KFe/0Ogmo05MGE`1MGEd0FEUI05EEE@1UIFD0Jf][06QXJ01VIVH0
DE5A06EUI@1cLg<0][Jf05ADE01iNGT0GemO06MWI`19BDT0D51@05iNGP1@D500K6a/06=SH`2OWil0
TY:B06i^KP1CDe<0CDe=05UIF@1PH60205ADE00Y07MgM`1oOgl0D51@065QH@18B4P0>c/k04A4A01B
DU80EeMG04U9B@1=CDd0BTY:08n?S`1LG5`0EUIF04m?C`1:BTX0CDe=05]KF`1JFUX0EeMG04a<C01^
KVh0PX:204m?C`1KFe/0@d=305iNGP24Q8@0DE5A08^;R`1nOWh0Ng]k06EUI@1EEED0BDU906ATI01S
Hf<0[:b/08:2PP1aLG403_ooool0501FEUH0HV9R05YJFP1oOgl0@T92034a<@10@400A4A403`l?019
BDT071`L03/k>`1=CDd0>3Ph05YJFP11@D40Lg=c04a<C00b<S80@T920P1:BTX0C00n?Sh0AdM706i^
KP1BDU80E5AD02h^;P1HF5P0DU9B030`<01/K6`0=SHf04Q8B01`L700@41005MGE`1DE5@0KVi^04U9
B@1UIFD0RHV9061PH01UIFD0[:b/055AD@1RHV80Ti>C07EeM@1^KVh0GEeM07QhN01IFET0AdM7065Q
H@1:BTX0J6QX05QHF00m?Cd0De=C055AD@1WIfL0G5aL069RHP1MGEd0BDU907UiN@1EEED0F5QH079b
LP10@400HF5Q04U9B@2=SHd0ATI606YZJP1[Jf/0GUiN05IFEP1oOgl0TY:B08B4Q027QhL0E5AD06][
J`1QHF40HV9R07moO`19BDT0?Shn04=3@`1oOgl0Cdm?05=CD`1;Bd/0QHF505YJFP14A4@205eMG@0K
04=3@`1hN7P0@41007UiN@17AdL0NWYj05IFEP1RHV80BTY:06][J`0o?cl0H61P0451@@19BDT0PH61
05EEE@1>CTh0Fe]K04Y:BP0c<c<0De=C030`<01?Cdl0<c<c07UiN@1OGel0;Rh^0080GEeM00/0?Shn
05aLG01/K6`0CTi>07YjNP1eMGD0?3`l03<c<`1QHF40M7Ad05QHF00>ooooo`0A05iNGP0a<C408R8R
02HV9P0/;2`0WYjN0;2`/02SXj<0N7Qh03<c<`0`<300=SHf02LW9`0h>3P0IVIV092@T01eMGD00P0n
?Sh0201PH600LW9b09>CT`1JFUX071`L07YjNP0a<C408b<S0P0F5QH0;P0k>c/0MGEe04E5A@2;Rh/0
[Zj^09ZJVP2EUID0/k>c05=CD`0d=3@0AdM702d];@0I6AT092@T06ATI00b<S80>SXj07emO@1ZJVX0
;bl_06a/K01OGel0/;2`08Z:RP1NGUh0Jf][0:ZZZP2NWYh0JFUY01TI6@0F5QH07QhN04m?C`2FUYH0
alO709:BTP1OGel0ADE502d];@1>CTh0:BTY034a<@1RHV80OWin07=cL`1:BTX201DE5@0o01dM7@1P
H600GemO096AT@1QHF40Cdm?09RHV02SXj<0Cdm?055AD@2d];@0a<C406ATI00X:2P071`L02/[:`0a
<C40?3`l06IVIP2SXj<0=cLg03@d=01aLG40Jf][03lo?`1ADE40IfMW08Z:RP1kNg/0`<300;no_`0d
=3@0Jf][08:2PP16ATH0@d=306UYJ@2[Zj/0Q8B40:RXZ025QHD0?Cdm04i>CP2BTY80@d=308>3P`0T
92@08b<S030`<00U9BD08R8R020P800a<C40Jf][07moO`1_Kfl0@41002XZ:P0T92@07AdM02@T901S
Hf<0_Kfm01Cooooo000Cooooo`0D07YjNP1mOGd0L71`06=SH`1QHF40AdM706ATI01aLG40AdM705QH
F01VIVH0DU9B05=CD`1lO7`0G5aL03lo?`1EEED0HV9R06UYJ@1KFe/205eMG@0Z04Y:BP1jNWX0EUIF
04Y:BP1GEeL0A4A405ADE01RHV80FEUI05ADE012@T80I6AT05mOG`1>CTh0KVi^04U9B@0n?Sh0FEUI
04Y:BP1BDU80JFUY04e=C@19BDT0KVi^05]KF`1CDe<0F5QH05UIF@0l?3`0G5aL05MGE`16ATH0EeMG
061PH01EEED0Fe]K05YJFP1UIFD0HF5Q06i^KP1aLG40K6a/0P1fMWH0901@D500Fe]K08Z:RP1BDU80
H61P07QhN012@T80JVYZ06e]K@1CDe<0Jf][06EUI@1NGUh0EeMG08V9R@1TI6@0IVIV08V9R@1WIfL0
JVYZ07]kN`1DE5@0EeMG07UiN@1ZJVX0Jf][04m?C`23Ph<0L71`051@D020P800JFUY05aLG01TI6@0
J6QX08F5Q@80G5aL01`0KVi^06MWI`1CDe<0Lg=c06MWI`1`L700IVIV07inOP28R8P0FEUI06a/K01k
Ng/0CDe=07=cL`20P800Cdm?05]KF`1hN7P0@41006YZJP1RHV80DE5A06YZJP1/K6`0@T9206a/K01H
F5P0FEUI0P1LG5`01@1<C4`0GUiN06YZJP1BDU80GEeM00kooooo0P1EEED0101ADE40F5QH055AD@16
ATH205aLG00705UIF@1BDU80?Cdm04=3@`12@T80B4Q806UYJ@0205iNGP1Q061PH01SHf<0@41004A4
A01>CTh0B4Q804A4A01DE5@0@41004Q8B010@400ADE504U9B@1BDU80BDU904Y:BP1PH600?clo06EU
I@1>CTh0DE5A04m?C`18B4P0?3`l03Xj>P1@D500?Shn03Lg=`0k>c/0Bd];03Ph>010@400@T9205MG
E`0n?Sh0GEeM06a/K01IFET0I6AT05QHF01NGUh0H61P06YZJP1OGel0IVIV06][J`1aLG40TY:B07Qh
N01jNWX0QhN704e=C@1DE5@0Hf=S05]KF`18B4P0IVIV05ADE01IFET0Cdm?06a/K01cLg<0XJ6Q05eM
G@1[Jf/0IVIV06ATI01CDe<0AdM7065QH@19BDT0HV9R05QHF02SXj<0Ogmo04m?C`1ADE40C4a<05mO
G`1LG5`0CTi>05YJFP1YJFT0O7al04i>CP1UIFD0B4Q803Ti>@2=SHd0EeMG05QHF015ADD0AdM704E5
A@1kNg/0C4a<05ADE00204e=C@0H04i>CP1ZJVX0Fe]K059BDP19BDT0Hf=S06YZJP1BDU80FEUI059B
DP1VIVH0T92@06=SH`29RHT0UIFE0820P01hN7P0H61P06QXJ01ZJVX0Fe]K08^;R`1]KFd0OWin3_oo
ool0901BDU80EUIF06e]K@1gMgL0@T9204A4A00c<c<0CTi>03Xj>P11@D40=cLg02LW9`1?Cdl0=3@d
04I6AP18B4P0JFUY03Ph>013@d<0@T9204m?C`1ADE40=CDe05=CD`1OGel0De=C05aLG00e=CD0Cdm?
05iNGP18B4P0Rh^;03Ph>01BDU80MGEe04A4A080D51@0240M7Ad051@D01KFe/0UIFE06EUI@1GEeL0
MWIf092@T01eMGD0PH610820P01RHV80EeMG06YZJP1UIFD0AdM705eMG@19BDT0GemO05ADE00e=CD0
DU9B05MGE`1IFET0GemO05aLG01OGel0@41008F5Q@15ADD0FEUI07=cL`0j>SX00P1TI6@0701UIFD0
@41006YZJP1RHV80FEUI065QH@1iNGT0QXJ60:6QX@2RXZ80CDe=069RHP1YJFT0Fe]K07=cL`17AdL0
@T9204a<C01eMGD0HF5Q04A4A00h>3P0S8b<05aLG00n?Sh0FUYJ05aLG00o?cl205aLG00U06e]K@0o
?cl0JVYZ069RHP1EEED0De=C06QXJ00l?3`0Fe]K04U9B@1:BTX0OGem065QH@1RHV80F5QH04i>CP1:
BTX0CTi>02DU9@1VIVH0DU9B06ATI01LG5`0EeMG04Q8B01:BTX0F5QH06YZJP1<C4`0D51@08:2PP1k
Ng/0De=C02PX:01NGUh0LG5a04];B`0>ooooo`0<069RHP1]KFd0MWIf079bLP1lO7`0]kNg0;6a/@2X
Z:P0T92@05ADE01;Bd/0F5QH0P0g=cL0301;Bd/0PX:204];B`1=CDd0De=C038b<P14A4@0?Shn06QX
J01lO7`0IfMW01TI6@@05ADE02d0EUIF08Z:RP1WIfL0V9RH0:JVYP1mOGd0L71`08Z:RP1NGUh0@410
02`/;00a<C40QXJ608V9R@0Z:RX0820P04M7A`1[Jf/061PH020P8014A4@0KVi^04=3@`0O7al0<c<c
05=CD`1MGEd0Bd];02HV9P0M7Ad07QhN028R8P1fMWH0Z:RX069RHP0O7al0:RXZ03`l?00a<C407QhN
01/K6`0a<C40Jf][06EUI@1KFe/00P0E5AD0;P0F5QH0FEUI0:2PX02>SXh0T92@08R8R01QHF40IfMW
0:^[Z`1nOWh0/[:b0;Bd]01dM7@0HF5Q06m_K`1jNWX0GemO04I6AP0n?Sh0LG5a05iNGP25QHD0EeMG
05mOG`2>SXh0FUYJ04Y:BP2HV9P0EUIF06ATI01bLW80OWin0:j^[P2][Jd0PX:207]kN`2MWId0_;bl
0;no_`36a/H0WYjN06a/K01eMGD0MgMg06EUI@1oOgl2020P800601`L700R8R807alO020P801>CTh0
W9bL0P2k^k/01`1_Kfl0JVYZ06i^KP2DU9@0Rh^;06QXJ02EUID05?ooool001?ooooo00H0LW9b07em
O@1QHF40MgMg05QHF01QHF4205aLG00Y069RHP0o?cl0DE5A07YjNP0o?cl0JVYZ06UYJ@0W9bL0G5aL
05YJFP0n?Sh0Jf][07YjNP0m?Cd0F5QH079bLP11@D40EeMG059BDP0i>CT0FEUI04a<C01FEUH0HV9R
04M7A`1hN7P0G5aL0451@@27QhL0BTY:055AD@1dM7@0<c<c05QHF01CDe<0?clo065QH@1FEUH0B4Q8
06MWI`12@T800P1;Bd/0F01LG5`0F5QH05]KF`0j>SX0KFe]05QHF00m?Cd0Ng]k07=cL`19BDT0Hf=S
07AdM01>CTh0Jf][07]kN`17AdL0GUiN06a/K015ADD0IFEU06QXJ01TI6@0N7Qh06=SH`1DE5@0M7Ad
05IFEP1_Kfl0FEUI05]KF`20P800EeMG071`L01/K6`0CTi>075aL@1OGel0BDU906UYJ@1bLW80Cdm?
06a/K029RHT0D51@05iNGP1fMWH0Hf=S07emO@1=CDd0HF5Q05eMG@19BDT0MWIf05mOG`1PH600G5aL
06EUI@20P800EeMG05EEE@2@T900KFe]04a<C020P800IFEU051@D01YJFT0JVYZ05mOG`1QHF40N7Qh
05=CD`27QhL0GEeM06=SH`1eMGD0E5AD06EUI@1^KVh0EEEE06a/K01=CDd0EUIF071`L015ADD0De=C
07MgM`0o?cl>ooooo`0A05iNGP1LG5`0FEUI05]KF`1>CTh0BTY:05aLG01YJFT0DU9B04m?C`0g=cL0
?3`l03hn?P1CDe<0L71`05]KF`1YJFT00P1`L70204Q8B00U04e=C@15ADD0ATI604U9B@16ATH0?Cdm
04A4A00g=cL0Bd];04m?C`1@D500BTY:05iNGP0j>SX0IfMW04];B`1<C4`0@d=303hn?P11@D40=3@d
0451@@0h>3P0:RXZ03dm?@13@d<0>CTi03lo?`1=CDd0FEUI04U9B@1KFe/0KFe]05eMG@1ZJVX0LW9b
06=SH`0205eMG@1@065QH@1GEeL0Ogmo07MgM`2:RXX0IVIV07emO@28R8P0A4A405EEE@1QHF40Hf=S
06ATI02;Rh/0GemO05MGE`1QHF40LW9b07emO@2_[jl0CDe=07=cL`1TI6@0J6QX04Y:BP0n?Sh0De=C
059BDP1LG5`0E5AD09FEU@1_Kfl0E5AD07]kN`1]KFd0IVIV05MGE`1@D500E5AD06][J`1aLG40A4A4
06YZJP1CDe<0ADE50492@P1BDU80Fe]K051@D01CDe<0AdM706EUI@19BDT0GemO05EEE@1:BTX0CDe=
06MWI`1EEED0Fe]K04E5A@1PH600J6QX06][J`1EEED0@d=305eMG@26QXH0GEeM07IfMP1mOGd0JVYZ
05]KF`1FEUH0EeMG04E5A@18B4P0N7Qh06MWI`1^KVh>ooooo`10055AD@19BDT0F5QH04Y:BP0_;bl0
?Shn034a<@0f=SH0:BTY03@d=00V9RH09BDU05EEE@0f=SH0?3`l03Xj>P1FEUH0>SXj04=3@`0l?3`0
>3Ph0492@P0j>SX0AdM706ATI01UIFD0GEeM03Ti>@1?Cdl0AdM703Ph>029RHT0>c/k04Y:BP1^KVh0
?Shn04I6AP14A4@0JFUY04A4A01RHV80YjNW06ATI01PH600A4A40<g=c@2a/K40UYJF09jNWP1cLg<0
FEUI08>3P`1QHF40FUYJ06i^KP1DE5@0GUiN05EEE@12@T80EeMG055AD@1QHF40Jf][05iNGP80JFUY
0380QXJ604a<C01ZJVX0N7Qh03lo?`1aLG40IVIV08Z:RP1VIVH0L71`06i^KP1PH600DE5A08F5Q@2K
Vi/0VIVI08Z:RP1_Kfl0D51@05aLG01^KVh0MWIf03Xj>P0k>c/0CTi>08f=S@1[Jf/0A4A4069RHP2K
Vi/0HV9R05MGE`1VIVH0M7Ad05aLG01SHf<0FUYJ069RHP10@400GUiN069RHP1KFe/0DE5A07moO`16
ATH0GUiN03/k>`1DE5@0OGem04e=C@80I6AT01H0C4a<04U9B@14A4@0>3Ph06i^KP17AdL0IfMW05IF
EP1RHV80E5AD05YJFP1mOGd0F5QH059BDP1UIFD0R8R804U9B@1`L7007AdM04U9B@1LG5`0GEeM3_oo
ool06P1CDe<0/[:b079bLP10@400=SHf071`L01;Bd/0K6a/0:BTY01bLW80GUiN05EEE@1fMWH0;Bd]
03dm?@2ZZZX0G5aL03Ti>@0U9BD0;2`/01/K6`0G5aL0:b/[079bLP2<S8`0BDU90P0E5AD0L00D51@0
6QXJ071`L00[:b/0;bl_05UIF@1GEeL08b<S01dM7@1JFUX0GUiN05mOG`1EEED0R8R808B4Q01MGEd0
JFUY06][J`2?Shl0RHV9024Q8@0F5QH09RHV05IFEP0a<C404Q8B02HV9P1lO7`0MgMg06=SH`1ZJVX0
A4A401hN7P0K6a/0Rh^;0=cLg01UIFD06a/K02XZ:P0g=cL08R8R01hN7P0H61P092@T05YJFP25QHD0
];Bd03/k>`0H61P0A4A40861P@1dM7@0T92@05iNGP1>CTh08b<S04m?C`1mOGd0:b/[055AD@1/K6`0
FEUI05mOG`1bLW80EUIF030`<010@400Cdm?05aLG00X:2P0][Jf0=7Ad@37alL0YJFU071`L01_Kfl0
MGEe03<c<`11@D40a/K60:^[Z`1SHf<0ADE503lo?`13@d<0Ng]k0:RXZ02BTY80FUYJ05mOG`0/;2`0
;bl_05UIF@1PH600UiNG02`/;00];Bd0@T9203<c<`0g=cL0NWYj07moO`2=SHd0ZJVY0:6QX@2LW9`0
[:b/0=?Cd`1lO7`0Cdm?06YZJP1LG5`Dooooo`004oooool0R`27QhL0LW9b071`L01IFET0IVIV05eM
G@1>CTh0Ti>C05]KF`1PH600JFUY03lo?`1`L700JFUY0451@@1NGUh0I6AT03dm?@1FEUH0IfMW0451
@@1SHf<0G5aL06YZJP1fMWH0;Rh^07]kN`1IFET0>SXj069RHP1YJFT0=cLg05ADE01]KFd0BDU904a<
C01UIFD0?clo05EEE@1VIVH0@41005aLG01IFET0GUiN051@D01UIFD0MgMg051@D01;Bd/0JFUY03Ti
>@1ADE40L71`03lo?`1ADE40LG5a04];B`1PH600CTi>05MGE`1lO7`0IfMW05ADE023Ph<0GEeM05AD
E01aLG40De=C06YZJP1TI6@0K6a/06][J`1bLW80K6a/06][J`1lO7`0Bd];06i^KP1XJ6P0ATI6096A
T@1YJFT0EeMG0:FUY@1ZJVX0Bd];09fMW@1KFe/0G5aL08Z:RP1@D500HF5Q06a/K01VIVH0RXZ:05EE
E@1QHF40L71`04];B`1[Jf/0L71`05QHF022PX80BTY:06ATI026QXH0:b/[07MgM`1hN7P0CDe=07Ee
M@1XJ6P0L71`05mOG`1LG5`0O7al055AD@1QHF40VIVI069RHP1IFET0X:2P05IFEP1BDU80PH6104a<
C01YJFT0Ng]k05=CD`1UIFD0LW9b05IFEP1RHV80KFe]04E5A@1UIFD0Kfm_04U9B@1QHF403_ooool0
O01CDe<0EEEE05IFEP1>CTh0IVIV051@D01SHf<0JVYZ05aLG01<C4`0>3Ph03Xj>P19BDT0C4a<06m_
K`1NGUh0IfMW069RHP1MGEd0BDU90492@P1=CDd0AdM704Y:BP1HF5P0CDe=04Y:BP11@D40CDe=05=C
D`1@D500C4a<04i>CP1GEeL0E5AD07MgM`1TI6@0C4a<051@D01IFET0?3`l0451@@17AdL0@D5103lo
?`16ATH0CTi>03hn?P1;Bd/0A4A4059BDP1>CTh0EeMG07YjNP1IFET0EeMG07alO010@400?clo05QH
F01`L700Cdm?08f=S@1mOGd0P82005UIF@1VIVH0RHV905IFEP0j>SX0BDU9059BDP1MGEd0GUiN05EE
E@16ATH0JFUY071`L02;Rh/0V9RH06][J`1SHf<0NWYj05eMG@1:BTX0Cdm?05IFEP1@D500H61P069R
HP2TY:@0UYJF05IFEP1_Kfl0FUYJ059BDP1SHf<0De=C04I6AP1XJ6P0K6a/04E5A@1JFUX0DU9B055A
D@17AdL0@T9205ADE012@T80CDe=04M7A`28R8P0EeMG04Q8B01NGUh0ATI604e=C@1>CTh0ATI605eM
G@1@D500E5AD05IFEP1BDU82051@D00405ADE026QXH0NWYj07MgM`80L71`00P0BTY:05QHF01IFET0
AdM703hn?P1mOGd0LW9b06m_K`kooooo03H0E5AD05QHF01]KFd0IfMW05aLG017AdL0=SHf04];B`0k
>c/0BTY:03<c<`0P8200ADE502/[:`16ATH0ADE5051@D00f=SH0CDe=0410@01CDe<0DU9B04M7A`1D
E5@0KFe]07EeM@1HF5P0A4A404Q8B01<C4`0GEeM0:f][@19BDT0BTY:06UYJ@0o?cl0De=C038b<P28
R8P0DE5A04e=C@2VYZH0LG5a05MGE`0^;Rh0Xj>S0=3@d01^KVh0SXj>06IVIP1KFe/0OWin069RHP1L
G5`206MWI`1D061PH01MGEd0C4a<05QHF01>CTh0Kfm_06i^KP1=CDd0IfMW05UIF@2ATI40@d=305UI
F@1^KVh0@D5106YZJP1SHf<0GemO05aLG01UIFD0KFe]04i>CP1XJ6P0TY:B06][J`2MWId0SHf=06QX
J01CDe<0JVYZ075aL@1JFUX0=SHf03@d=01>CTh0SHf=061PH01BDU80Hf=S07inOP1aLG40De=C05mO
G`1eMGD0JVYZ05YJFP1CDe<0EeMG04A4A01@D500JFUY04M7A`1=CDd0PH6104];B`15ADD0=SHf05]K
F`1cLg<0@41008N7Q`1KFe/0;Bd]05iNGP16ATH07alO065QH@1LG5`0MGEe059BDP1CDe<0G5aL04=3
@`1gMgL0K6a/03dm?@1HF5P0VYZJ05eMG@1UIFD09bLW065QH@1_Kfl0IfMW3_ooool04@1>CTh0BTY:
04I6AP0/;2`0820P024Q8@11@D40De=C0:VYZ@2?Shl0QHF509bLW01LG5`0;bl_02d];@2IVIT0GEeM
00808B4Q03P07alO01LG5`0E5AD05ADE04I6AP2?Shl0ZJVY07IfMP1KFe/0Hf=S08>3P`36a/H0IFEU
02@T901LG5`0C4a<01TI6@0H61P071`L05=CD`13@d<0KVi^08J6QP1MGEd0@410020P800T92@0A4A4
07AdM01`L700CDe=02XZ:P1XJ6P06ATI018B4P1OGel0OWin04I6AP0N7Qh0G5aL09BDU02SXj<0LW9b
0<?3``3De=@0`/;205ADE00j>SX06a/K01XJ6P0Y:BT0:2PX028R8P0f=SH0d];B0;Vi^@80YjNW03l0
WYjN06=SH`0f=SH0HF5Q08B4Q01YJFT0;bl_02h^;P0f=SH061PH01dM7@1DE5@0Q8B409JFUP1/K6`0
?Shn02LW9`19BDT0S8b<08V9R@11@D40_[jn07MgM`1jNWX0Rh^;05mOG`2LW9`0Jf][05]KF`23Ph<0
ZJVY0;Jf]P20P800:b/[04=3@`0^;Rh0QHF509^KV`1VIVH0>CTi03Ph>00b<S80;bl_03dm?@2<S8`0
YjNW0820P01GEeL0@41006UYJ@1oOgl0LW9b08Z:RP2BTY80K6a/09JFUP2ZZZX0^;Rh0:f][@27QhL0
/[:b0861P@1PH6005?ooool001?ooooo08H0MWIf07=cL`1MGEd0JVYZ05YJFP1OGel0JVYZ075aL@1E
EED0CDe=07=cL`1LG5`0Fe]K06m_K`1PH600AdM706EUI@1SHf<0E5AD04a<C01aLG40D51@05QHF01c
Lg<0De=C05UIF@1LG5`0B4Q805QHF01LG5`0EEEE05UIF@1BDU80GEeM05YJFP15ADD0HV9R05QHF01@
D500DU9B05IFEP1BDU80G5aL059BDP18B4P0EeMG06QXJ01EEED0C4a<06=SH`16ATH0HV9R06QXJ01F
EUH0EEEE06=SH`1RHV80De=C05UIF@1cLg<0DU9B07UiN@1dM7@0EUIF06MWI`1hN7P0HV9R05YJFP1g
MgL0AdM706m_K`1XJ6P0CTi>06ATI01mOGd0GEeM06ATI01bLW80GUiN065QH@1kNg/0H61P05iNGP2H
V9P0HV9R05=CD`1lO7`0JFUY05ADE01_Kfl0HV9R06EUI@1VIVH0Ng]k079bLP1QHF40O7al05eMG@1Z
JVX0HF5Q05iNGP1eMGD0GUiN06=SH`1VIVH0GUiN04m?C`1_Kfl0IVIV06EUI@1YJFT0O7al08B4Q01?
Cdl0K6a/0820P015ADD0KFe]07=cL`19BDT0GEeM07QhN01:BTX0Lg=c07EeM@1@D500MWIf06MWI`1C
De<0N7Qh05aLG01MGEd0I6AT06YZJP80F5QH00<0L71`05YJFP1TI6@03_ooool02P1GEeL0J6QX05]K
F`1MGEd0FEUI04e=C@1PH600GemO04i>CP1<C4`203hn?P0n04=3@`1BDU80K6a/05]KF`1PH600I6AT
061PH017AdL0BDU904Y:BP13@d<0A4A4051@D018B4P0>3Ph03`l?00d=3@0BTY:055AD@1>CTh0BTY:
05EEE@0l?3`0JFUY05MGE`1EEED0D51@05MGE`0h>3P0?Shn04M7A`0m?Cd0@T9203/k>`1@D500>c/k
03lo?`14A4@0Fe]K04m?C`1HF5P0LG5a05MGE`1XJ6P0JVYZ05QHF019BDT0IFEU065QH@1HF5P0YJFU
08f=S@1YJFT0JVYZ065QH@1eMGD0EEEE04E5A@1EEED0CTi>055AD@1VIVH204E5A@1006MWI`1MGEd0
Lg=c096AT@1UIFD0HF5Q06][J`1TI6@0CDe=04];B`1=CDd0B4Q805aLG01KFe/0UYJF07]kN`1NGUh0
M7Ad04m?C`1IFET0FUYJ04U9B@11@D40H61P05eMG@17AdL0ADE504I6AP0n?Sh0B4Q804U9B@1IFET0
@41004I6AP1>CTh0N7Qh059BDP14A4@0Hf=S0451@@1EEED0DU9B05ADE01UIFD0BTY:04m?C`1EEED0
Bd];0492@P1=CDd0EEEE06][J`1QHF40K6a/065QH@1ZJVX0Fe]K05MGE`1NGUh0CTi>04a<C01]KFd0
L71`07IfMPkooooo03H0CTi>05MGE`1QHF40CTi>051@D014A4@0>CTi0451@@0[:b/0>CTi02`/;00a
<C40?3`l03Xj>P0k>c/0>3Ph05IFEP0h>3P0De=C03Hf=P0l?3`0FUYJ03De=@1>CTh0De=C06EUI@13
@d<0?3`l04E5A@1LG5`0CDe=09bLW00h>3P0@T9205YJFP0n?Sh0BTY:03@d=01dM7@0BTY:05aLG02/
[:`0GUiN059BDP0h>3P0QhN706UYJ@1]KFd0YZJV05iNGP1MGEd0KFe]055AD@1;Bd/205]KF`1D04m?
C`1HF5P0B4Q805]KF`1>CTh0IfMW06ATI01DE5@0EeMG06e]K@2?Shl0BTY:06ATI01hN7P0?Shn075a
L@1cLg<0GUiN07IfMP1KFe/0JVYZ05QHF01MGEd0Ogmo06m_K`2VYZH0SHf=07moO`1MGEd0HV9R079b
LP1LG5`0De=C03Ti>@1>CTh0NWYj06MWI`13@d<0LW9b06MWI`1PH600EUIF079bLP1JFUX0HV9R05mO
G`1VIVH0EEEE04i>CP1fMWH0OGem04I6AP1iNGT0R8R805QHF01FEUH0KFe]05MGE`1eMGD0IVIV07em
O@1QHF40Cdm?05mOG`1:BTX0?Cdm07AdM01FEUH0SHf=05eMG@1SHf<0J6QX06ATI01QHF40GUiN055A
D@1GEeL0RHV905UIF@29RHT09RHV05iNGP1BDU80HF5Q3_ooool0M`22PX80<c<c02h^;P0H61P05QHF
018B4P0U9BD0L71`07=cL`2?Shl0c<c<06ATI01/K6`0FEUI06ATI02HV9P0PX:2071`L016ATH0@410
03Ph>013@d<0:b/[03/k>`1fMWH0PX:205UIF@0h>3P0<C4a02h^;P0];Bd0LW9b08R8R025QHD0AdM7
01PH600F5QH05aLG01XJ6P1FEUH0GEeM08Z:RP0I6AT0:b/[01LG5`0J6QX07AdM05ADE01cLg<0>c/k
07EeM@1lO7`0?Shn04Q8B01/K6`0Q8B4020P800B4Q809RHV06UYJ@20P800_;bl05UIF@1KFe/0[:b/
09JFUP1lO7`0:2PX01HF5P0g=cL0:RXZ04Y:BP1mOGd0YZJV08Z:RP2TY:@0/;2`0<G5a@1WIfL0<C4a
02PX:00d=3@0>3Ph04a<C01;Bd/0@d=301/K6`0N7Qh0GEeM0:ZZZP2TY:@0/[:b09nOW`1SHf<0@T92
0451@@25QHD0TY:B09>CT`1MGEd0Kfm_09bLW02VYZH0[Jf]08^;R`1lO7`0N7Qh075aL@0i>CT0Lg=c
04];B`1HF5P0HV9R09RHV02g]kL0FEUI03Ti>@0T92@0:b/[0080>SXj00D0VYZJ0<O7a`3@d=00UiNG
0820P00206UYJ@0708R8R02][Jd0]KFe09BDU03>c/h0eMGE0;jn_P020:JVYP030:>SX`2l_;`0PX:2
01Cooooo000Cooooo`0N07alO01iNGT0FUYJ0820P01HF5P0Ogmo051@D01CDe<0L71`03Hf=P1PH600
JFUY03dm?@1_Kfl0K6a/03Ti>@1ZJVX0O7al0410@01QHF40EEEE04E5A@1VIVH0F5QH0492@P1HF5P0
Cdm?04];B`1BDU80@d=30P1RHV803@1MGEd0N7Qh03dm?@1?Cdl0Rh^;02h^;P1BDU80MGEe02l_;`1?
Cdl0K6a/03Ti>@1aLG400P16ATH0G01SHf<0>3Ph05MGE`1JFUX0CTi>071`L01XJ6P0?Cdm06][J`1h
N7P0BDU906i^KP21PH40=cLg06YZJP21PH40@D5107QhN025QHD0EUIF07=cL`1jNWX0CDe=09>CT`1O
Gel0GEeM07QhN01FEUH0MGEe0820P01QHF40PX:205eMG@1bLW80SHf=059BDP1fMWH0RHV904=3@`1c
Lg<0PX:205MGE`1fMWH0NWYj04I6AP1YJFT0U9BD05MGE`27QhL0JFUY05mOG`20P800>CTi07UiN@1`
L700EeMG08V9R@1UIFD0JVYZ06ATI01LG5`0Shn?079bLP1OGel0PX:206IVIP1JFUX0PH6107EeM@1F
EUH0O7al06QXJ01HF5P0Lg=c05IFEP1DE5@0M7Ad05eMG@1VIVH0PH6106a/K01jNWX0K6a/05=CD`1[
Jf/0?Shn06=SH`1WIfL0@T9206=SH`1aLG40?Cdm3_ooool00`1KFe/0EEEE04i>CP0204m?C`1b04=3
@`1VIVH0KVi^05aLG01:BTX0=SHf03Ph>010@400>c/k05]KF`15ADD0De=C04e=C@1LG5`0AdM704Q8
B013@d<0=cLg03lo?`18B4P0@d=303@d=012@T80=cLg04a<C019BDT0Bd];05QHF01QHF40ATI607]k
N`1LG5`0DE5A05EEE@1=CDd0=cLg03`l?018B4P0?Shn04=3@`0m?Cd0D51@04Y:BP16ATH0Cdm?061P
H01OGel0JVYZ08b<S01RHV80KVi^07EeM@1PH600Cdm?06EUI@1FEUH0De=C0:^[Z`2XZ:P0IfMW05]K
F`1EEED0Ng]k05IFEP1EEED0GUiN05IFEP1<C4`0Ti>C04M7A`15ADD0HV9R06IVIP1fMWH0T92@04m?
C`1DE5@0G5aL069RHP1NGUh0CDe=04a<C015ADD0A4A405aLG01mOGd0PH6103Hf=P29RHT0De=C05AD
E01FEUH0Bd];04Q8B01JFUX0E5AD04U9B@1<C4`0@d=303/k>`12@T80@D5105]KF`0e=CD0A4A40410
@01]KFd0@d=303Ti>@1;Bd/0<S8b04a<C01JFUX0Hf=S0P1ADE403019BDT0@d=305UIF@10@400AdM7
05EEE@1hN7P0JFUY071`L01/K6`0JVYZ04e=C@80De=C00D0Bd];04A4A01[Jf/0JVYZ07YjNP0>oooo
o`0d04i>CP1IFET0H61P04Q8B01>CTh0=cLg04i>CP0o?cl0<30`04=3@`0h>3P0ATI603Ph>00d=3@0
CDe=03Hf=P1EEED0>3Ph04E5A@0[:b/0?clo06IVIP0a<C40EUIF04Q8B01JFUX0>c/k04Q8B00b<S80
AdM705mOG`1iNGT0>3Ph05EEE@1;Bd/0?Shn05MGE`0_;bl0GemO059BDP1MGEd0X:2P07MgM`0_;bl0
EeMG09^KV`0l?3`0GUiN09fMW@1IFET0IfMW06IVIP80CTi>04<0GemO05=CD`1<C4`0Hf=S04];B`1U
IFD0FEUI06ATI024Q8@0FUYJ079bLP1YJFT0Lg=c05=CD`1cLg<0KFe]051@D01mOGd0PX:206m_K`2?
Shl0IfMW079bLP1:BTX0EEEE0861P@27QhL0YjNW08n?S`1jNWX0I6AT07YjNP2>SXh0JFUY04m?C`0n
?Sh0Bd];07emO@1RHV80AdM706a/K01>CTh0JVYZ05iNGP1iNGT0Kfm_06i^KP1/K6`0HV9R04m?C`0n
?Sh0JVYZ069RHP1=CDd0OWin07YjNP1BDU80GemO05IFEP1BDU80O7al06IVIP1]KFd0Jf][05UIF@1R
HV80C4a<0080Bd];0140De=C08Z:RP20P800KFe]07QhN02:RXX0@T9205ADE01NGUh0Kfm_08V9R@1a
LG40PX:202@T901@D500E5AD08N7Q`0>ooooo`0304a<C016ATH0<30`00805aLG0P0f=SH07P0J6QX0
;2`/06i^KP32`/80^k^k06e]K@1NGUh0:2PX04=3@`1BDU80B4Q808^;R`2_[jl0J6QX04Q8B00b<S80
GemO08n?S`2ATI40<c<c01/K6`0E5AD0=CDe02LW9`0g=cL0I6AT09:BTP0[:b/0>3Ph01XJ6P805aLG
02P061PH05eMG@1aLG40<S8b051@D00/;2`06ATI01XJ6P0b<S80dm?C071`L025QHD0bLW90;Rh^020
P800AdM702PX:012@T80AdM702`/;02XZ:P0QHF50:ZZZP1`L700KFe]08>3P`1CDe<0OWin08j>SP1^
KVh0<c<c04m?C`25QHD0IVIV07QhN01/K6`0RXZ:03<c<`1/K6`0FEUI0P0F5QH0>`0[:b/06ATI02`/
;026QXH0[:b/08F5Q@22PX80Kfm_07AdM02i^KT0SXj>09FEU@1HF5P0?3`l02TY:@2<S8`0O7al06IV
IP1jNWX0I6AT075aL@1/K6`0/[:b09:BTP1KFe/0KVi^03@d=00K6a/0>SXj04E5A@1kNg/0NGUi06UY
J@3?cll0YZJV04m?C`0L71`07AdM02TY:@1oOgl0[jn_0;6a/@2h^;P0FUYJ03Ti>@1:BTX0CTi>03`l
?00X:2P0C4a<055AD@26QXH0JVYZ0;:b/P36a/H0QXJ604U9B@20P800FUYJ01Cooooo000Cooooo`0m
08>3P`1fMWH0Cdm?05mOG`1QHF40?clo05EEE@1hN7P0AdM705iNGP2DU9@0A4A405YJFP1]KFd0?Cdm
07EeM@0m?Cd0ADE5069RHP1HF5P0>c/k06ATI01@D500GemO06QXJ00`<300LG5a05=CD`11@D40Hf=S
065QH@12@T80Cdm?04e=C@1SHf<0?3`l069RHP1?Cdl0@D51061PH01>CTh0AdM7055AD@1NGUh0>SXj
059BDP1/K6`0>CTi05eMG@1lO7`0<30`06=SH`1<C4`0=cLg06m_K`16ATH0AdM707IfMP0l?3`0GEeM
06QXJ00206IVIP1<05mOG`1PH600L71`071`L01:BTX0OWin05MGE`1XJ6P0N7Qh05aLG01VIVH0M7Ad
06m_K`1UIFD0S8b<071`L01VIVH0SXj>06IVIP1OGel0UiNG061PH01aLG40Q8B405]KF`1iNGT0GUiN
05]KF`1eMGD0?Cdm05aLG01iNGT0@41007QhN01]KFd0FEUI07UiN@1NGUh0N7Qh0861P@14A4@0Hf=S
071`L00f=SH0Lg=c06=SH`1IFET0J6QX05QHF023Ph<0HV9R06e]K@2MWId0FEUI069RHP25QHD0Cdm?
06EUI@2CTi<0Bd];06MWI`20P800BDU905aLG020P800B4Q805EEE@1`L700DU9B06YZJP1oOgl0>c/k
06m_K`1RHV80@d=306i^KPkooooo0380GemO06IVIP1^KVh0F5QH05eMG@1>CTh0LG5a07AdM01KFe/0
FEUI03Xj>P10@400C4a<04M7A`1WIfL0EeMG05UIF@1RHV80FEUI04U9B@1?Cdl0CTi>03/k>`0m?Cd0
CTi>04a<C00m?Cd0?3`l04=3@`1BDU80Bd];05=CD`1ZJVX0MGEe04a<C01/K6`0EUIF05aLG01ZJVX0
De=C0451@@14A4@0Bd];03dm?@11@D40?clo04Q8B00o?cl0?Cdm04U9B@80Cdm?00d0HF5Q08F5Q@1J
FUX0KFe]06m_K`1OGel0C4a<06ATI01HF5P0Cdm?08b<S02[Zj/0G5aL0080IVIV04T0MWIf055AD@12
@T80H61P04I6AP1MGEd0OWin05IFEP1;Bd/0TI6A05=CD`1hN7P0TI6A06=SH`1JFUX0HF5Q05eMG@1B
DU80B4Q805iNGP1RHV80DE5A06ATI01oOgl0Ph>304=3@`1hN7P0@41004Y:BP1TI6@0CDe=04U9B@1K
Fe/0A4A404a<C015ADD0@d=303@d=00o?cl0AdM705mOG`0e=CD0BDU904E5A@1SHf<0BDU903/k>`1<
C4`0?Shn03Xj>P1ADE40F5QH05EEE@12@T80BDU9051@D01GEeL0=cLg03dm?@1HF5P0Jf][06e]K@1X
J6P0EUIF051@D01;Bd/0De=C04Y:BP1=CDd0BTY:06YZJP1SHf<0MWIf00kooooo00P0FUYJ05IFEP1_
Kfl0AdM704];B`1>CTh0DU9B04M7A`80<c<c05X0@T9204A4A011@D40@T9203hn?P10@400EeMG02h^
;P1?Cdl0=CDe05YJFP1NGUh0?Cdm05eMG@0o?cl0KFe]04U9B@1CDe<0=cLg059BDP15ADD0EUIF059B
DP1;Bd/0BTY:05ADE01ADE40=cLg065QH@0n?Sh0Cdm?0:FUY@1OGel0JFUY05MGE`20P800?Cdm05YJ
FP2a/K40EEEE06UYJ@1EEED0De=C06a/K01FEUH0G5aL05=CD`1NGUh0GemO05YJFP1@D500JFUY06m_
K`1]KFd0IfMW08J6QP21PH40GemO07inOP1lO7`0DE5A05eMG@23Ph<0G5aL07UiN@1_Kfl0I6AT04=3
@`1`L700M7Ad05iNGP2^[Zh0TY:B07MgM`0l?3`0MgMg08^;R`1?Cdl0F5QH03/k>`1ZJVX0K6a/06e]
K@17AdL0FEUI04E5A@1KFe/0AdM706MWI`1LG5`205eMG@0V055AD@1IFET0B4Q805UIF@1WIfL0=cLg
06UYJ@1XJ6P0AdM704A4A01MGEd0EeMG07EeM@1bLW80DE5A071`L01NGUh0F5QH04Y:BP1=CDd0ATI6
079bLP29RHT0Kfm_08f=S@1eMGD0EEEE07=cL`1JFUX0HF5Q08F5Q@26QXH0I6AT08n?S`0W9bL0AdM7
04Q8B024Q8@>ooooo`0V075aL@1UIFD0OGem04Q8B00W9bL09BDU01PH600E5AD051@D04M7A`2VYZH0
TI6A05IFEP0^;Rh0NWYj05]KF`1MGEd0EEEE055AD@1lO7`0:RXZ02PX:017AdL0S8b<0:b/[01[Jf/0
?Cdm01`L700J6QX071`L01hN7P11@D40:RXZ08J6QP0k>c/061PH02XZ:P0J6QX201HF5P0G03Xj>P2F
UYH0QXJ603Lg=`0j>SX0DU9B06e]K@1`L700I6AT0<G5a@3Jf]X0b/[:09jNWP0o?cl06ATI01`L700H
61P0?Shn0;>c/`2JVYX0Zj^[07]kN`0];Bd00P0T92@01`1UIFD0FEUI075aL@2b/[80W9bL0;bl_02P
X:000P1:BTX09P0X:2P0J6QX02/[:`0h>3P0KVi^01/K6`0E5AD06ATI01LG5`1ADE40Ng]k09^KV`2f
][H0`<300:>SX`1jNWX0BTY:05YJFP1bLW80@41004M7A`0e=CD0JVYZ0<;2`P1XJ6P08B4Q01/K6`0_
;bl0LG5a07AdM01MGEd0Ph>309:BTP0L71`06QXJ01dM7@1PH600:RXZ0P1gMgL06P22PX80GEeM05AD
E00J6QX071`L03Ti>@1=CDd0Fe]K08B4Q01BDU80IfMW03`l?00Z:RX07AdM01`L700K6a/0;2`/03Ti
>@2ATI40Ogmo0:>SX`37alL0Ogmo03Xj>P1eMGD0GUiN5?ooool001?ooooo08/0MWIf0820P01IFET0
J6QX05mOG`1bLW80BDU904a<C01MGEd0>CTi05QHF01eMGD0<S8b06=SH`1XJ6P0=3@d05ADE02DU9@0
=CDe05eMG@1^KVh0?Shn069RHP1TI6@0E5AD075aL@1MGEd0>CTi061PH00i>CT0EeMG05QHF016ATH0
Jf][05EEE@11@D40Ph>304i>CP15ADD0NWYj04Y:BP19BDT0K6a/0410@01JFUX0BTY:04E5A@1ADE40
?clo075aL@1QHF40EeMG05]KF`1UIFD0=CDe06a/K01[Jf/0>3Ph06MWI`1^KVh0:b/[06EUI@1hN7P0
>CTi06m_K`1/K6`0?3`l061PH01lO7`0?Cdm07=cL`1QHF40FEUI075aL@1<C4`0LW9b07]kN`1QHF40
Ph>306ATI01OGel0MgMg07QhN01fMWH0Kfm_05mOG`1kNg/0KFe]04m?C`1WIfL0Ti>C065QH@1WIfL0
Xj>S069RHP2BTY80G5aL05]KF`25QHD0Cdm?06e]K@1WIfL0CTi>06a/K01_Kfl0GEeM071`L01CDe<0
L71`08b<S01JFUX0MgMg07AdM01BDU80LG5a06IVIP1BDU80M7Ad06m_K`14A4@0LG5a06YZJP1?Cdl0
Shn?05MGE`1FEUH0IVIV061PH01MGEd0G5aL05iNGP1WIfL0EEEE06MWI`1gMgL0@41005aLG01PH600
D51@00kooooo02l0IfMW061PH01PH600EUIF06=SH`1=CDd0MgMg08b<S01TI6@0G5aL03De=@0m?Cd0
A4A403@d=01^KVh0G5aL05YJFP1CDe<0GEeM04i>CP1:BTX0Bd];03Ti>@0m?Cd0CTi>03Ph>00k>c/0
@T9204Y:BP16ATH0EUIF055AD@1MGEd0Lg=c04m?C`1ZJVX0H61P04Y:BP1EEED0De=C03Ph>00m?Cd0
C4a<03`l?01:BTX0@D5104i>CP0204A4A00e04e=C@1;Bd/0A4A4061PH02CTi<0De=C06a/K023Ph<0
Kfm_05ADE01;Bd/0K6a/06m_K`2i^KT0b<S808^;R`1_Kfl0IfMW06i^KP1;Bd/0@d=304i>CP13@d<0
EUIF0820P01JFUX0ADE508B4Q01BDU80Lg=c08b<S01QHF40J6QX06MWI`1ZJVX0D51@05UIF@1HF5P0
JVYZ055AD@1TI6@0OWin08F5Q@1FEUH0MgMg06EUI@1KFe/0GUiN04m?C`15ADD0GEeM04i>CP1<C4`0
0P1;Bd/01@0k>c/0D51@04m?C`1SHf<0>c/k0080ATI601d0CDe=051@D00c<c<0EEEE034a<@1;Bd/0
BTY:04M7A`1EEED0Bd];04M7A`1ADE40DU9B04U9B@0n?Sh0@41006a/K01KFe/0OGem06UYJ@1XJ6P0
De=C06YZJP1PH600EEEE05IFEP1VIVH0HF5Q07IfMP0>ooooo`0b05MGE`1KFe/0N7Qh0451@@1GEeL0
>CTi04e=C@1;Bd/09BDU03De=@12@T80?3`l04I6AP0/;2`0BTY:04A4A01BDU80C4a<05ADE00m?Cd0
B4Q806m_K`19BDT0G5aL051@D01bLW80>c/k04];B`0f=SH0?clo05QHF015ADD0HV9R04];B`16ATH0
F5QH05]KF`0l?3`0J6QX04=3@`1QHF40UiNG05QHF01/K6`0AdM707emO@1CDe<0L71`0;Jf]P1EEED2
06EUI@0/051@D01RHV80JVYZ04M7A`12@T80HF5Q04=3@`1BDU80E5AD05eMG@1fMWH0H61P06IVIP1Y
JFT0Hf=S05mOG`1cLg<0Hf=S065QH@1nOWh0MgMg05QHF01jNWX0N7Qh05aLG01@D500IFEU07AdM02:
RXX0ZJVY0861P@2=SHd0@d=306a/K02/[:`0Fe]K05iNGP13@d<0N7Qh08:2PP28R8P0F5QH06i^KP1G
EeL206QXJ00Z06i^KP1aLG40UIFE06m_K`1=CDd0A4A405EEE@18B4P0JVYZ04a<C01VIVH0B4Q803`l
?01:BTX0@T9205UIF@1jNWX0FUYJ0861P@1hN7P0>3Ph06m_K`1@D500?clo05UIF@1`L700S8b<07If
MP1OGel0JVYZ03<c<`1NGUh0EEEE05=CD`1LG5`0N7Qh07]kN`2IVIT0;Bd]051@D01<C4`0OGem3_oo
ool0H`1=CDd0Fe]K08>3P`1JFUX0ATI602@T900E5AD04a<C018B4P0b<S80MGEe05]KF`1;Bd/04A4A
01<C4`0M7Ad06ATI02PX:00o?cl0G5aL05YJFP0J6QX05ADE05YJFP2BTY80S8b<06ATI00c<c<0;Bd]
02`/;00];Bd0XJ6Q08:2PP2UYJD0Kfm_028R8P0J6QX0:b/[02HV9P0J6QX0:b/[0:FUY@32`/80Ogmo
07emO@1QHF40DU9B02DU9@0J6QX0Jf][0;Vi^@2LW9`0CDe=05mOG`1GEeL0?Shn03lo?`16ATH0OGem
079bLP0k>c/0J6QX01XJ6P0C4a<051@D034a<@2VYZH0ZZZZ0:RXZ02;Rh/0MgMg07inOP1MGEd08B4Q
020P801GEeL0?clo04M7A`1NGUh0C4a<01`L700N7Qh0ATI604M7A`0U9BD0R8R80;bl_02c/k<0]KFe
071`L00R8R808B4Q03`l?00J6QX092@T05EEE@1kNg/0MGEe04a<C00202<S8`0W05YJFP1_Kfl0E5AD
06=SH`1jNWX0[:b/028R8P0K6a/06ATI02PX:01YJFT0Lg=c0410@00W9bL0?Cdm0410@01JFUX0P820
0:NWY`1ADE40O7al08f=S@27QhL0De=C01LG5`0I6AT061PH01/K6`0L71`0:b/[03/k>`2QXJ40];Bd
096AT@3AdM40L71`08N7Q`1JFUX0<30`01Cooooo000Cooooo`0D07IfMP1ZJVX0F5QH06QXJ01TI6@0
De=C0492@P1^KVh0Cdm?0451@@1^KVh0Hf=S0410@01bLW80Hf=S05MGE`1>CTh0H61P04i>CP1QHF42
05IFEP0B04i>CP1<C4`0D51@04M7A`1EEED0CDe=05iNGP1DE5@0K6a/051@D01=CDd0JFUY04U9B@13
@d<0K6a/04m?C`1HF5P0H61P0P1>CTh0BP1KFe/0F5QH04m?C`13@d<0EEEE04Y:BP1<C4`0JVYZ05EE
E@1LG5`0EeMG05iNGP1DE5@0BDU906=SH`1KFe/0D51@05YJFP13@d<0JVYZ065QH@1CDe<0LG5a06i^
KP1GEeL0D51@06m_K`1WIfL0JFUY05UIF@1eMGD0Lg=c06][J`27QhL0GUiN06UYJ@1hN7P0Hf=S07Ui
N@1/K6`0G5aL08^;R`1RHV80EEEE08f=S@1_Kfl0Fe]K069RHP1cLg<0HV9R05=CD`1^KVh0Kfm_06YZ
JP1JFUX0KVi^05ADE01GEeL0Jf][06e]K@1hN7P0De=C07=cL`26QXH0B4Q8075aL@1SHf<0DU9B07If
MP1ADE40Jf][05QHF01WIfL0NGUi0P1GEeL05@1_Kfl0JFUY04];B`1^KVh0G5aL06MWI`1RHV80EeMG
06MWI`1RHV80G5aL04Q8B01ZJVX0K6a/04a<C01XJ6P0GEeM04A4A01YJFT0Fe]K06a/K00>ooooo`1h
06MWI`1NGUh0FUYJ05mOG`1PH600FEUI06m_K`1dM7@0FEUI055AD@0h>3P0@D51059BDP1WIfL0KFe]
05IFEP1SHf<0GEeM069RHP1=CDd0Cdm?051@D00k>c/0=cLg04U9B@11@D40>SXj0451@@12@T80@d=3
05aLG016ATH0FUYJ06IVIP1=CDd0FUYJ04m?C`0o?cl0CTi>04Q8B00e=CD0?clo059BDP0d=3@0>SXj
034a<@15ADD0?Cdm03hn?P1;Bd/0B4Q804=3@`1>CTh0RHV904m?C`1JFUX0Ng]k06YZJP1GEeL0De=C
05EEE@0o?cl0S8b<0:VYZ@1hN7P0O7al05iNGP0o?cl0D51@04Y:BP1@D500Bd];05eMG@24Q8@0EUIF
04U9B@24Q8@0B4Q8071`L01jNWX0ATI6051@D01JFUX0Lg=c05YJFP1HF5P0DU9B06=SH`1>CTh0IfMW
07inOP1lO7`0ADE506=SH`16ATH0?clo051@D018B4P0>CTi05EEE@14A4@0?Cdm04=3@`18B4P0=SHf
0410@016ATH0De=C03De=@18B4P0?clo04];B`19BDT0=SHf055AD@0e=CD0Bd];051@D01<C4`0Fe]K
0P1=CDd04P1VIVH0De=C03hn?P11@D40CDe=05iNGP1hN7P0FUYJ05EEE@19BDT0ATI6051@D01GEeL0
B4Q8055AD@1XJ6P0HV9R07IfMPkooooo08`0Fe]K06=SH`1VIVH0CTi>05aLG014A4@0?3`l04I6AP0_
;bl092@T03hn?P0b<S80=SHf03De=@0l?3`0@d=30451@@13@d<0@T92034a<@1GEeL0J6QX04E5A@1E
EED0E5AD07=cL`1BDU80C4a<02TY:@0j>SX0GUiN02/[:`1?Cdl0@T92034a<@1DE5@0D51@04U9B@1G
EeL0F5QH04i>CP24Q8@0Kfm_06][J`1?Cdl0Rh^;05YJFP1UIFD0YJFU05eMG@1YJFT0J6QX05QHF01b
LW80H61P05IFEP1>CTh0G5aL05eMG@1BDU80DE5A05MGE`1VIVH0J6QX06=SH`1jNWX0I6AT06EUI@1b
LW80Fe]K06EUI@1aLG40LW9b04];B`1nOWh0K6a/07IfMP1EEED0I6AT07inOP1NGUh0^k^k08b<S01Q
HF40AdM706m_K`2HV9P0HV9R04a<C00d=3@0EeMG07AdM021PH40BDU907emO@1<C4`0I6AT065QH@1d
M7@0HF5Q07]kN`1YJFT0IFEU055AD@1PH600EUIF06MWI`15ADD0EUIF04A4A01IFET0B4Q804M7A`1I
FET0Hf=S05]KF`21PH40IfMW04=3@`1LG5`0?Cdm04M7A`12@T80JFUY06IVIP1KFe/0QXJ6065QH@0b
<S80PX:205=CD`19BDT0MgMg06][J`1<C4`0TI6A02h^;P1>CTh0?clo08j>SPkooooo01X0AdM706MW
I`1iNGT0B4Q803<c<`0f=SH07alO02`/;00f=SH0QhN709fMW@22PX80J6QX01/K6`0?3`l03@d=010@
400V9RH0JFUY071`L02PX:00Ogmo06IVIP2m_Kd0ZJVY06=SH`80JVYZ0580F5QH05eMG@1mOGd0CTi>
055AD@1bLW80TI6A07EeM@1?Cdl0@d=305QHF01;Bd/0IVIV08>3P`2TY:@0N7Qh02d];@0O7al0;Rh^
04I6AP1jNWX0SXj>0<71`@1=CDd061PH024Q8@18B4P0E5AD020P8011@D40Bd];04Q8B00X:2P0E5AD
02DU9@0C4a<04Q8B01@D501<C4`0SHf=05QHF014A4@0Fe]K038b<P1@D500<30`024Q8@11@D40C4a<
02DU9@1<C4`0EeMG08V9R@1EEED0?Shn01lO7`0f=SH0LG5a059BDP1PH600IVIV06YZJP0U9BD0<S8b
055AD@0a<C40:RXZ04Q8B01RHV80C4a<04Y:BP0a<C40@d=302l_;`0T92@0<S8b03/k>`26QXH0H61P
04a<C00X:2P07alO0P1ZJVX0200^;Rh06a/K01XJ6P0X:2P0RHV905iNGP2f][H0[jn_0P2j^[X0102Q
XJ40NGUi03lo?`0V9RH201LG5`0<024Q8@0O7al0?Cdm06=SH`1TI6@0C4a<06EUI@1^KVh0TY:B07mo
O`0d=3@0<c<c5?ooool001?ooooo0140OWin07EeM@1ADE40G5aL06QXJ011@D40Cdm?07moO`0i>CT0
IFEU07MgM`1:BTX0JFUY04a<C01LG5`0KFe]03Xj>P0206UYJ@1h03hn?P1EEED0H61P03dm?@1KFe/0
GEeM0451@@1_Kfl0C4a<055AD@1OGel0F5QH06IVIP1BDU80EUIF07AdM017AdL0EEEE06YZJP12@T80
E5AD06IVIP11@D40J6QX08J6QP0d=3@0DE5A06QXJ00g=cL0BTY:05mOG`0h>3P0IfMW04U9B@0n?Sh0
KFe]04I6AP19BDT0Jf][0410@01GEeL0?clo04Y:BP1MGEd0GemO05QHF02ATI40G5aL04];B`2=SHd0
B4Q805QHF01[Jf/0Cdm?06IVIP1XJ6P0HV9R05mOG`20P800J6QX07UiN@2DU9@0HV9R07QhN01cLg<0
HV9R08:2PP1nOWh0M7Ad06i^KP1:BTX0Rh^;08:2PP13@d<0Ogmo07IfMP15ADD0TI6A06MWI`1mOGd0
L71`05EEE@1/K6`0J6QX03dm?@1iNGT0Lg=c065QH@21PH40HV9R09VIV@1WIfL0CTi>0;bl_01OGel0
L71`0::RXP1LG5`0GEeM09JFUP1=CDd0Hf=S09RHV00m?Cd0KFe]06i^KP14A4@0JFUY06EUI@1ADE40
J6QX05IFEP1JFUX0D51@07moO`13@d<0IFEU08>3P`0j>SX0Ph>33_ooool0;P1UIFD0GUiN05EEE@1Q
HF40GUiN05UIF@1`L700Ng]k05mOG`1JFUX0>3Ph04E5A@1>CTh0F5QH06m_K`1GEeL0GEeM05MGE`1K
Fe/0Bd];04U9B@1=CDd0@d=3038b<P1;Bd/0?Shn03dm?@19BDT0B4Q80451@@1JFUX0CDe=06YZJP1]
KFd0EeMG069RHP1ADE40>c/k05YJFP1ADE40=CDe04E5A@1<C4`0?3`l0451@@0l?3`204E5A@090410
@01KFe/0Cdm?03Xj>P1DE5@0YZJV05aLG015ADD0GEeM0080A4A40540L71`0410@00m?Cd0G5aL09ZJ
VP1WIfL0N7Qh0820P018B4P0Bd];0451@@1FEUH0BDU906ATI02@T900B4Q8055AD@1kNg/0CDe=08>3
P`26QXH0DE5A04m?C`1aLG40NGUi05IFEP1CDe<0EEEE06=SH`11@D40FEUI061PH01[Jf/0B4Q806UY
J@15ADD0@T92055AD@16ATH0BTY:05IFEP1GEeL0C4a<04I6AP0j>SX0:BTY0492@P1:BTX0G5aL03Lg
=`1CDe<0BDU905IFEP1HF5P0?3`l05EEE@0f=SH0@D5104m?C`1:BTX0DE5A04i>CP18B4P0DU9B05EE
E@16ATH0C4a<05]KF`1/K6`0Ogmo06i^KP23Ph<0G5aL05iNGP1FEUH0H61P04e=C@1TI6@0JFUY05mO
G`1ZJVX03_ooool02`1IFET0F5QH06e]K@0l?3`0HF5Q0492@P10@400A4A4024Q8@0Z:RX0ATI60080
@41003T0=3@d0451@@0k>c/0>c/k05=CD`1>CTh0<C4a0410@01>CTh0?Shn05EEE@15ADD0M7Ad03/k
>`1>CTh0;Rh^03Xj>P1>CTh092@T05UIF@0e=CD0:RXZ059BDP0n?Sh0Bd];04A4A011@D40FUYJ08B4
Q00l?3`0Fe]K04i>CP2EUID0?clo05]KF`31`L40?3`l065QH@14A4@0CDe=05=CD`1?Cdl0IFEU03@d
=01OGel0C4a<03`l?018B4P0BDU905=CD`1ZJVX0EUIF075aL@1SHf<0JVYZ07MgM`1CDe<00P1WIfL0
;@1hN7P0@d=307IfMP1XJ6P0Ph>306a/K01HF5P0Ph>308F5Q@2n_[h0J6QX08F5Q@1NGUh0Lg=c09ZJ
VP2CTi<0EUIF04I6AP0f=SH0OWin07MgM`19BDT0QhN706e]K@1IFET0KVi^06QXJ01=CDd0Hf=S06YZ
JP1NGUh0A4A4051@D016ATH0AdM704];B`1TI6@0:BTY05IFEP0e=CD0C4a<05MGE`1]KFd0H61P06QX
J00206IVIP0E05YJFP1:BTX0Kfm_03<c<`1JFUX0LW9b069RHP2:RXX0DE5A03hn?P1_Kfl0G5aL05mO
G`1RHV80IFEU04i>CP21PH409BDU0492@P16ATH0UYJF00kooooo01l0<S8b04e=C@1MGEd0XJ6Q06e]
K@1`L700MgMg05iNGP1CDe<0G5aL0;Rh^02RXZ80NGUi01hN7P0Z:RX0;bl_03<c<`0Z:RX08b<S01lO
7`1<C4`0QXJ60=7Ad@2[Zj/0X:2P07inOP1kNg/0N7Qh04a<C00e=CD0GUiN0080?Shn01h07AdM04];
B`2k^k/0U9BD07=cL`15ADD0>CTi05QHF02BTY80Y:BT06][J`1;Bd/0@D5102h^;P0k>c/0X:2P08>3
P`2BTY80NGUi02PX:00K6a/08b<S02h^;P0i>CT0C4a<02h^;P15ADD0WIfM09NGU`0i>CT201<C4`0=
01lO7`22PX80P82002l_;`0X:2P0<C4a02HV9P0Z:RX0@d=304e=C@0a<C40O7al0;^k^`020861P@0P
096AT@0m?Cd0<S8b03Hf=P1nOWh0Lg=c03Ti>@0_;bl0FEUI0:FUY@1iNGT0OWin059BDP19BDT0?Shn
04a<C02ATI40XJ6Q08N7Q`2MWId0LW9b04e=C@1DE5@0HV9R08f=S@3>c/h0QHF50820P01jNWX0/;2`
06m_K`0M7Ad301PH600I028R8P2?Shl0[Zj^0;^k^`2MWId0IfMW051@D01PH600M7Ad05mOG`11@D40
7alO01/K6`0Q8B40>SXj09VIV@1mOGd0Fe]K02DU9@1NGUh0BTY:06UYJ@1SHf<0KFe]04U9B@0Doooo
o`004oooool0H@1bLW80KFe]05YJFP1_Kfl0EEEE07MgM`15ADD0CTi>05aLG00h>3P0GEeM06m_K`0c
<c<0LG5a05UIF@0o?cl0Fe]K06EUI@0k>c/0Fe]K06EUI@1<C4`0H61P05QHF01@D500L71`04m?C`1^
KVh0FEUI04];B`1bLW80@D5103lo?`25QHD0AdM704U9B@1LG5`0BDU9055AD@1PH600?Shn04e=C@1I
FET0?Cdm05mOG`0`<300Cdm?05]KF`0_;bl0A4A406UYJ@1@D500H61P06QXJ00i>CT0IVIV07=cL`0k
>c/0L71`06e]K@0[:b/0IFEU06i^KP0f=SH0JFUY09FEU@1BDU80K6a/06IVIP1fMWH0IVIV05YJFP2D
U9@0JFUY05=CD`2EUID0C4a<05MGE`2IVIT0D51@06UYJ@29RHT0Hf=S07IfMP1mOGd0Fe]K08>3P`1[
Jf/0DE5A06MWI`1[Jf/0ADE507YjNP1PH600GUiN07YjNP14A4@00P1`L700:011@D40O7al05ADE01S
Hf<0IfMW04i>CP2>SXh0Kfm_05IFEP1WIfL0De=C075aL@17AdL0Hf=S05mOG`1VIVH0JVYZ079bLP1P
H600Hf=S069RHP1LG5`0DE5A06UYJ@1RHV80CTi>07=cL`1HF5P0FUYJ08:2PP0U9BD0IVIV08V9R@0[
:b/0F5QH07=cL`0j>SX0BTY:07EeM@1<C4`>ooooo`0=07]kN`1SHf<0G5aL065QH@1WIfL0EUIF06i^
KP24Q8@0GEeM05IFEP0a<C40?Cdm04i>CP0206MWI`0?069RHP1KFe/0Fe]K05]KF`1@D500EUIF05=C
D`1;Bd/0A4A4065QH@1IFET0@D5104I6AP1;Bd/0D51@0080EEEE05l0E5AD06EUI@1FEUH0EeMG05aL
G015ADD0GemO051@D00g=cL0C4a<04I6AP0j>SX0>c/k0410@015ADD0@d=303dm?@1>CTh0De=C04Y:
BP1UIFD0TY:B04i>CP13@d<0ADE504e=C@1CDe<0GEeM05QHF01QHF40QhN70:NWY`29RHT0QhN708b<
S01>CTh0DU9B05=CD`1DE5@0CTi>061PH01gMgL0DU9B04e=C@1ZJVX0FEUI07=cL`1bLW80Cdm?061P
H01^KVh0GemO069RHP1JFUX0OGem06i^KP1?Cdl0Hf=S06a/K01PH600EEEE04a<C01KFe/0ADE5059B
DP1<C4`0A4A405YJFP1XJ6P0D51@04U9B@1GEeL0?3`l059BDP1=CDd0De=C03<c<`1@D500DE5A04Y:
BP1DE5@0>3Ph05YJFP0a<C40@D5104E5A@17AdL0DE5A04E5A@1;Bd/0CTi>04I6AP1BDU80D51@04=3
@`0205MGE`0;08F5Q@1CDe<0GUiN059BDP1FEUH0E5AD04Q8B01ZJVX0Hf=S05ADE01TI6@03_ooool0
5`1`L700I6AT04i>CP1:BTX0E5AD03Ph>00n?Sh0<S8b03Ti>@0W9bL0@41004A4A00/;2`0@41003<c
<`0m?Cd0<c<c04U9B@0_;bl0;Rh^04Q8B01TI6@0>SXj0080Bd];01d0IFEU03lo?`1OGel0;bl_04I6
AP1JFUX09BDU04a<C01?Cdl0;Bd]04];B`16ATH0DE5A04I6AP1LG5`0A4A406UYJ@1OGel0IFEU04Q8
B02>SXh0GEeM05MGE`27QhL0EUIF071`L01XJ6P0>3Ph06i^KP0205UIF@1D069RHP1KFe/0K6a/069R
HP1?Cdl0EEEE075aL@1_Kfl0JFUY065QH@1OGel0Ogmo04];B`1QHF40OGem05mOG`27QhL0DU9B061P
H01aLG40GEeM05QHF01YJFT0Lg=c05MGE`31`L40PX:206m_K`1;Bd/0P820092@T01nOWh0GEeM04I6
AP1IFET0G5aL069RHP1EEED0O7al061PH01TI6@0JVYZ071`L01MGEd0Jf][051@D01`L700HF5Q06][
J`1OGel0EEEE03dm?@1aLG40=3@d04a<C00i>CT0K6a/059BDP1=CDd0O7al06UYJ@1JFUX0LG5a05EE
E@1DE5@0JVYZ03dm?@20P800JFUY05IFEP2@T900IVIV0492@P2@T900IVIV05mOG`1hN7P0MWIf0410
@020P800?Cdm05aLG00j>SX0U9BD3_ooool03`0a<C40=SHf0::RXP2EUID0MgMg092@T037alL0U9BD
08b<S02YZJT0bl_;09>CT`2WYjL0WinO03Ph>00201TI6@1N01/K6`0J6QX06ATI04E5A@28R8P0/;2`
055AD@29RHT0b/[:08F5Q@2OWil0VIVI03Xj>P14A4@08b<S020P800J6QX0K6a/0:2PX02QXJ40UYJF
06MWI`1KFe/0GEeM07]kN`30`<00@41001DE5@0M7Ad0>3Ph08Z:RP2FUYH0W9bL055AD@2ATI40QXJ6
038b<P0L71`07QhN02@T901PH600De=C08f=S@1QHF40ATI606MWI`16ATH0=cLg059BDP28R8P0WYjN
05aLG01iNGT09BDU01/K6`0S8b<0;bl_059BDP1UIFD0/K6a092@T01cLg<0];Bd06a/K01UIFD0K6a/
0:b/[01IFET0Fe]K02h^;P0l?3`0RHV9071`L01WIfL0GEeM09FEU@1>CTh0:2PX0451@@2OWil0[Jf]
0:VYZ@2QXJ40NGUi06m_K`1DE5@0JVYZ09bLW02NWYh0[Zj^08>3P`2k^k/0e]KF06YZJP805aLG01/0
5ADE01TI6@1HF5P0De=C07inOP24Q8@0E5AD03Xj>P0a<C40<30`04Y:BP1QHF40Jf][092@T01NGUh0
GEeM08B4Q02[Zj/0X:2P03/k>`0a<C40KFe]06ATI01IFET0?clo03`l?00N7Qh05?ooool001?ooooo
05@0RXZ:07UiN@1KFe/0IFEU065QH@1WIfL0B4Q806ATI01OGel0DE5A06=SH`1WIfL0DE5A065QH@1K
Fe/0Kfm_04Y:BP1NGUh0Fe]K0492@P1IFET0E5AD03@d=01;Bd/0IVIV03`l?01?Cdl0KVi^04Y:BP14
A4@0LG5a04U9B@0o?cl0KFe]05EEE@1<C4`0I6AT04U9B@1HF5P0DU9B051@D01JFUX0Bd];06EUI@1F
EUH0=3@d06IVIP1:BTX0?Shn06=SH`11@D40D51@05UIF@1@D500DU9B05=CD`1OGel0Fe]K05EEE@1O
Gel0ATI605mOG`1gMgL0JFUY06IVIP1bLW80JVYZ059BDP1ZJVX0L71`05ADE01LG5`0MgMg04e=C@1R
HV80MgMg05YJFP1aLG40MWIf06=SH`2@T900Ng]k061PH02BTY8206a/K00e07UiN@1^KVh0P82005]K
F`1cLg<0Kfm_05=CD`1MGEd0MgMg05QHF01=CDd0Lg=c05eMG@1DE5@0EUIF05aLG01QHF40>3Ph069R
HP1lO7`0F5QH06UYJ@1?Cdl0Ogmo06IVIP1ADE40Shn?05aLG01RHV80PH6106m_K`1VIVH0PX:206MW
I`1HF5P0OWin06=SH`1TI6@0HV9R06a/K01DE5@0LG5a06][J`10@400F5QH06ATI019BDT0PH6106QX
J01CDe<0KVi^04m?C`1[Jf/03_ooool07P20P800GEeM05EEE@1IFET0J6QX05]KF`1aLG40LW9b06EU
I@1QHF40=SHf0410@014A4@0EUIF05YJFP1NGUh0FEUI05eMG@1PH600G5aL04a<C01=CDd0A4A403Hf
=P1GEeL0DU9B04M7A`1=CDd0EeMG0451@@80DU9B00T0DE5A06i^KP1JFUX0Lg=c05IFEP0l?3`0D51@
04Q8B00j>SX00P13@d<00`0a<C40>SXj0492@P0203hn?P0V04i>CP1GEeL0Bd];05EEE@1TI6@0QhN7
06ATI01BDU80De=C04Y:BP17AdL0FUYJ06YZJP1[Jf/0Lg=c08R8R01jNWX0P8200:VYZ@1MGEd0EUIF
04E5A@1:BTX0=CDe05EEE@1[Jf/0ATI60451@@1`L700E5AD07IfMP1/K6`0@T9206e]K@1IFET0GEeM
05IFEP1EEED2065QH@0d055AD@1YJFT0I6AT05QHF01:BTX0Bd];04a<C00h>3P0B4Q804a<C019BDT0
FUYJ06i^KP13@d<0A4A403Ph>00b<S80>c/k04A4A019BDT0<C4a0492@P18B4P0?Shn04M7A`0d=3@0
Bd];02l_;`10@400B4Q804A4A01@D500Bd];051@D01MGEd0FUYJ04=3@`1;Bd/0?3`l04];B`1]KFd0
TY:B08f=S@1GEeL0ADE504m?C`1TI6@0CDe=065QH@1XJ6P0HV9R06QXJ0kooooo02/0IVIV05EEE@1A
DE40?Cdm061PH00m?Cd0<30`03/k>`0b<S80=3@d03Xj>P1;Bd/0=CDe04I6AP11@D40@410038b<P1E
EED0>3Ph03<c<`12@T80ATI603Xj>P18B4P0CDe=06ATI012@T80Cdm?02h^;P1@D500C4a<02LW9`1F
EUH0<C4a02`/;01KFe/0?clo05ADE00`<300E5AD05MGE`1[Jf/0DU9B0080GUiN04T0SHf=04e=C@1_
Kfl0aLG5065QH@1^KVh0J6QX05YJFP1/K6`0E5AD07IfMP1IFET0J6QX05QHF01<C4`0DE5A04a<C01_
Kfl0I6AT05mOG`1_Kfl0GEeM096AT@1ADE40EEEE07MgM`1fMWH0WIfM06a/K01]KFd0N7Qh06MWI`28
R8P0L71`079bLP1kNg/0X:2P07IfMP25QHD0Cdm?075aL@2;Rh/0N7Qh04];B`1?Cdl0Bd];05MGE`17
AdL0Cdm?0861P@1jNWX0Jf][07inOP1cLg<0G5aL07EeM@1SHf<0NGUi05aLG01eMGD0DE5A05MGE`19
BDT0K6a/03De=@1VIVH0=3@d06][J`1JFUX0Cdm?096AT@1^KVh0Hf=S0080IfMW01@0EEEE05aLG00h
>3P0M7Ad06EUI@1ADE40Q8B404m?C`12@T80PX:205IFEP1RHV80L71`06IVIP1EEED0PH61030`<015
ADD0@T9207]kN`kooooo03T0FEUI04];B`1QHF40WIfM09FEU@2ATI40QXJ6061PH01FEUH0PH6104e=
C@1FEUH0?Shn06ATI01XJ6P0C4a<04A4A010@400>c/k04M7A`2HV9P0YjNW08F5Q@0o?cl0B4Q805MG
E`0L71`08B4Q03Ph>018B4P0FEUI069RHP1TI6@0JVYZ06IVIP1oOgl0NGUi06IVIP2YZJT0KFe]03`l
?00Z:RX0?clo061PH00^;Rh08B4Q03Hf=P0S8b<0?clo05=CD`19BDT0JFUY0<S8b02^[Zh0NGUi06a/
K01bLW800P28R8P03@1ZJVX0JFUY04];B`1ADE40RHV908b<S01mOGd0Shn?05]KF`15ADD0ATI6038b
<P0^;Rh00P1DE5@0;`1JFUX0ZJVY08J6QP1MGEd0CTi>07MgM`1jNWX0:BTY055AD@2k^k/0Jf][030`
<01>CTh0GUiN04=3@`0e=CD0IfMW05MGE`1:BTX0OGem09NGU`29RHT0VYZJ09nOW`1mOGd0JVYZ05aL
G01VIVH0PH61092@T00b<S80MgMg09FEU@2k^k/0]kNg07=cL`29RHT0GemO05]KF`1MGEd0CTi>07em
O@0e=CD0KFe]06UYJ@0l?3`0;Bd]0080:BTY01409RHV06QXJ02RXZ80[:b/09JFUP2OWil0Shn?08F5
Q@1PH600PH6108V9R@27QhL0=cLg04A4A012@T80ADE502PX:00Dooooo`004oooool0J@1bLW80Rh^;
05MGE`1OGel0Hf=S04a<C01SHf<0D51@04e=C@1UIFD0EUIF05EEE@1aLG40ATI606YZJP1`L700;Rh^
071`L01IFET0?clo07YjNP1BDU80ADE508J6QP19BDT0G5aL069RHP1BDU80GemO03Ti>@1NGUh0IfMW
04];B`1:BTX0F5QH03Xj>P1OGel0Kfm_034a<@1TI6@0N7Qh03Ph>01OGel0J6QX04Q8B01CDe<0EUIF
05=CD`1IFET0D51@05EEE@1VIVH0<S8b05]KF`1_Kfl0BDU905QHF01BDU80@D51075aL@13@d<0E5AD
07MgM`17AdL0H61P08^;R`1EEED0HV9R06QXJ01HF5P0IfMW07UiN@18B4P0IVIV05iNGP1CDe<0MGEe
05UIF@1YJFT0KFe]075aL@1UIFD0RHV9079bLP1^KVh0Q8B406QXJ01bLW80QhN704A4A01^KVh0Kfm_
04E5A@29RHT0Jf][079bLP1UIFD0E5AD07moO`1;Bd/0EEEE07UiN@1XJ6P0G5aL08:2PP0207]kN`0P
05QHF01WIfL0R8R805MGE`1QHF40VIVI04e=C@1kNg/0MGEe05QHF020P800RXZ:059BDP1`L700P820
04Q8B021PH40GemO05]KF`1fMWH0FUYJ06MWI`1FEUH0DE5A071`L01>CTh0GUiN05aLG01KFe/0DU9B
05aLG01WIfL>ooooo`0<07moO`1LG5`0Cdm?05ADE01PH600FEUI07AdM01fMWH0GUiN06=SH`12@T80
BTY:0P19BDT08@1`L700Fe]K06MWI`1HF5P0IFEU05]KF`17AdL0EEEE04M7A`0l?3`0C4a<059BDP10
@400>CTi06EUI@1;Bd/0D51@04e=C@1CDe<0K6a/05mOG`1^KVh0H61P03@d=01BDU80B4Q803Ph>00i
>CT0@D5103Xj>P18B4P0DE5A03`l?00203Ti>@0_04I6AP0f=SH0AdM705aLG01aLG40De=C059BDP1C
De<0CDe=055AD@1JFUX0H61P06IVIP1KFe/0JVYZ07=cL`1[Jf/0XZ:R055AD@1IFET0CDe=04Y:BP16
ATH0GEeM06i^KP16ATH0Cdm?05MGE`1BDU80K6a/06=SH`0l?3`0GemO05QHF01>CTh0F5QH051@D01L
G5`0J6QX05YJFP1`L700IVIV05QHF01CDe<0EEEE04E5A@0c<c<00P1:BTX02`0c<c<0@d=3075aL@0g
=cL0;Rh^034a<@0k>c/0Bd];04a<C01DE5@0@4100080Cdm?01X0Bd];04=3@`10@400A4A403Hf=P1@
D500BTY:0410@01RHV80?Cdm04Q8B01SHf<0De=C04Y:BP1?Cdl0Fe]K075aL@1WIfL0L71`08f=S@19
BDT0E5AD04a<C01YJFT0DU9B06QXJ080HV9R00<0H61Pooooooooool03?ooool05P24Q8@0GemO04m?
C`1YJFT0NGUi055AD@13@d<0=cLg03Xj>P0a<C40@D5103dm?@0e=CD0Cdm?03/k>`11@D40=CDe07Ee
M@0d=3@0C4a<05MGE`1BDU8204Y:BP0W06UYJ@1lO7`0EeMG06UYJ@0k>c/0F5QH06=SH`0j>SX0FEUI
03lo?`0`<300DU9B04Y:BP1>CTh0=CDe05mOG`1ADE40FEUI06EUI@1lO7`0Jf][09ZJVP1HF5P0G5aL
0;6a/@1XJ6P0Kfm_05QHF01GEeL0MGEe05iNGP1VIVH0Hf=S06MWI`1BDU80E5AD059BDP1?Cdl0MWIf
00<0H61P00@0CTi>09fMW@18B4P0D51@0P1WIfL0A02SXj<0IVIV04i>CP1dM7@0G5aL06m_K`23Ph<0
MWIf06=SH`1nOWh0K6a/071`L01BDU80OWin0::RXP1SHf<0C4a<03lo?`1SHf<0CDe=05aLG01>CTh0
Jf][07IfMP1cLg<0PX:206i^KP1PH600GUiN05eMG@1hN7P0GemO075aL@1BDU80EeMG03lo?`1NGUh0
=3@d05QHF00e=CD0IVIV06QXJ00e=CD0KVi^071`L01XJ6P0Jf][061PH01MGEd0KVi^03@d=01SHf<0
De=C06=SH`1oOgl0CTi>05UIF@1bLW80IfMW06a/K020P800IFEU06IVIP1gMgL0<30`04e=C@11@D40
Ogmo3_ooool0900k>c/092@T01/K6`0j>SX0UiNG07emO@0a<C4051@D014A4@0S8b<0KVi^01LG5`0U
9BD0WYjN0:RXZ01jNWX0@d=303hn?P0j>SX0O7al0:j^[P2c/k<0Ng]k05iNGP0`<300=cLg01LG5`0C
4a<06ATI03De=@1jNWX0Ogmo075aL@0f=SH07QhN01TI6@80>c/k03h0JFUY04A4A019BDT07alO028R
8P17AdL0K6a/04E5A@0O7al08B4Q02XZ:P1GEeL0CTi>01lO7`28R8P0YJFU08Z:RP1DE5@0De=C08b<
S01FEUH0:BTY02PX:01@D500?clo06EUI@1GEeL0?clo03De=@0];Bd05QHF02/[:`0/;2`0Bd];05MG
E`1BDU80RHV90;Fe]@1CDe<0D51@034a<@0/;2`0IfMW06][J`1VIVH0Ti>C09NGU`1;Bd/0Q8B404E5
A@0O7al0820P02`/;01LG5`0OGem075aL@1OGel0T92@0<;2`P1oOgl0PX:204U9B@<071`L0180BTY:
06][J`2PX:00>CTi06a/K02CTi<0=3@d08:2PP0T92@0;Bd]07=cL`0X:2P0GemO05EEE@1]KFd0SHf=
02DU9@0Q8B4201hN7P0A03Ti>@2=SHd0SXj>07moO`25QHD0V9RH0=SHf02k^k/0?clo069RHP0a<C40
EUIF038b<P1<C4`0VIVI09fMW@1kNg/05?ooool001?ooooo00X0MgMg07IfMP1eMGD0M7Ad05iNGP26
QXH0@D5106UYJ@1XJ6P0@4100P1MGEd0O`1BDU80K6a/05QHF01BDU80CTi>07AdM00l?3`0F5QH069R
HP1KFe/0ADE504A4A01PH600=SHf04i>CP1bLW80Cdm?03Xj>P1fMWH0@d=305aLG01gMgL0BTY:05UI
F@1gMgL0BTY:05eMG@1@D500BTY:05mOG`0l?3`0EeMG06UYJ@0X:2P0De=C06EUI@0c<c<0OWin06MW
I`10@400Lg=c065QH@1>CTh0I6AT079bLP1<C4`0FEUI06i^KP0j>SX0I6AT06YZJP1?Cdl0KVi^06=S
H`1WIfL0LW9b06UYJ@26QXH0IVIV06YZJP2FUYH0ATI606UYJ@29RHT0Bd];06][J`2EUID0B4Q80820
P01iNGT0CTi>0820P01lO7`0CTi>08>3P`1JFUX0EUIF07MgM`1KFe/0GUiN05IFEP1LG5`0Ph>3079b
LP1@D500JFUY07IfMP1KFe/0KVi^05YJFP1ZJVX0De=C05MGE`2?Shl0Bd];05UIF@1MGEd0EEEE075a
L@1@D500IfMW07IfMP1mOGd0K6a/07=cL`1LG5`0Kfm_07emO@1BDU80HV9R07alO01ADE40G5aL0820
P00m?Cd0EEEE08>3P`0d=3@0De=C07QhN00f=SH0HF5Q05iNGP10@400KFe]059BDP1ADE403_ooool0
;P2>SXh0JVYZ059BDP1JFUX0H61P05YJFP1RHV80KFe]06=SH`1QHF40>c/k04Q8B01;Bd/0KVi^06YZ
JP1XJ6P0HV9R06IVIP1LG5`0Fe]K05eMG@1=CDd0ATI603Xj>P1EEED0BDU904m?C`1NGUh0Hf=S055A
D@1EEED0B4Q805=CD`1RHV80JFUY08^;R`1HF5P0B4Q805IFEP17AdL0@T9204A4A016ATH0=cLg04M7
A`1UIFD20410@00<04A4A01?Cdl0@D5104Y:BP1SHf<0JFUY05mOG`1iNGT0CTi>05=CD`1ADE40F5QH
0P1WIfL0=`1YJFT0JVYZ07MgM`1UIFD0VYZJ06EUI@1VIVH0D51@04I6AP0o?cl0G5aL079bLP1CDe<0
FEUI07EeM@1[Jf/0OGem06MWI`19BDT0Fe]K06a/K01IFET0GEeM05]KF`1WIfL0@T9206YZJP1NGUh0
DE5A04E5A@1;Bd/0@d=304i>CP12@T80BTY:04Q8B00i>CT0>SXj04U9B@0_;bl0;Bd]04A4A00j>SX0
@T920451@@1>CTh0>3Ph0410@01:BTX0CDe=05EEE@17AdL0;Bd]04=3@`15ADD00P17AdL05@1>CTh0
>c/k04i>CP1GEeL0C4a<055AD@1CDe<0BDU9061PH01@D500Ng]k07alO01KFe/0De=C065QH@1`L700
CTi>05QHF01CDe<0FUYJ05UIF@0>ooooo`0706a/K01DE5@0B4Q804];B`1ZJVX0F5QH03`l?00203@d
=00=02h^;P1;Bd/0@T9203<c<`1?Cdl0;Rh^03Lg=`0Z:RX0De=C03Hf=P0l?3`0A4A403Lg=`020410
@00P05ADE01gMgL0CTi>05aLG00e=CD0Hf=S04a<C00_;bl0GUiN03Hf=P0_;bl0H61P04A4A01=CDd0
>3Ph05QHF01FEUH0IFEU08:2PP1UIFD0F5QH0:>SX`1BDU80Lg=c0<_;b`1ZJVX0MgMg05]KF`1XJ6P0
LG5a05aLG01kNg/206e]K@0A05mOG`18B4P0IfMW05YJFP1jNWX0L71`069RHP1SHf<0DE5A09VIV@19
BDT0@41005mOG`1cLg<0OGem06a/K01XJ6P00P1fMWH0?`26QXH0NWYj07YjNP1`L700MgMg06m_K`2;
Rh/0G5aL07IfMP2BTY80IVIV05MGE`17AdL0Bd];05YJFP1HF5P0@d=306ATI01IFET0G5aL075aL@1P
H600FEUI07=cL`1UIFD0Ng]k06QXJ026QXH0A4A405=CD`1>CTh0Hf=S04a<C01XJ6P0<30`069RHP1Q
HF40CDe=08j>SP1lO7`0F5QH07MgM`1[Jf/0E5AD05QHF00];Bd0JFUY05mOG`1ZJVX0Ng]k04M7A`1F
EUH0M7Ad05QHF01YJFT0Kfm_04i>CP1KFe/0NWYj030`<017AdL0BTY:09:BTP0>ooooo`0K03De=@0]
;Bd0>CTi02PX:01cLg<0T92@01XJ6P0@41004A4A010@400k>c/0GEeM01LG5`1oOgl0PH6103Ph>00B
4Q805ADE01HF5P0G5aL0=CDe06=SH`2NWYh0YJFU0861P@1XJ6P05QHF008051@D00D0=cLg08f=S@2=
SHd0C4a<024Q8@0201DE5@1101lO7`0h>3P0I6AT03dm?@1PH600De=C04A4A01oOgl0/;2`07IfMP14
A4@0Cdm?06][J`1mOGd0Bd];0451@@0e=CD0@41007alO018B4P0G5aL06UYJ@1gMgL06ATI034a<@2?
Shl0N7Qh07emO@0^;Rh09RHV01dM7@17AdL071`L01PH600o?cl0M7Ad0:6QX@2CTi<0RXZ:092@T00R
8R8071`L01dM7@0X:2P0CDe=020P800X:2P0SXj>0;^k^`1iNGT0PX:208N7Q`1CDe<0=SHf01`L700o
?cl0Hf=S02d];@15ADD0?clo0::RXP1<C4`0Bd];065QH@0L71`00P0O7al01`0g=cL0S8b<096AT@1G
EeL0MWIf04A4A01WIfL00P18B4P0700`<300ATI60;Fe]@2^[Zh0NGUi07MgM`2;Rh/0LW9b04m?C`0]
;Bd0BDU9092@T02>SXh0B4Q803dm?@1GEeL0WIfM08F5Q@1VIVH0UIFE08^;R`2>SXh0OWin08Z:RP1e
MGD0TY:B03@d=01>CThDooooo`004oooool0BP22PX80MWIf05iNGP1MGEd0H61P06EUI@1HF5P0Cdm?
0492@P1PH600Cdm?04a<C01iNGT0GemO05]KF`1oOgl0:b/[06][J`1bLW80<C4a079bLP1EEED0=CDe
061PH01IFET0ATI60410@01DE5@0D51@04];B`1ADE40G5aL04I6AP1<C4`0G5aL03Hf=P1CDe<0L71`
03hn?P18B4P0P82004I6AP1CDe<0FEUI04=3@`1DE5@0H61P03@d=01WIfL0E5AD03`l?01fMWH0@T92
0451@@1aLG40<c<c05ADE01WIfL0;2`/05mOG`1FEUH0H61P08b<S01XJ6P0EUIF075aL@1SHf<0HF5Q
079bLP1WIfL0DE5A08b<S01SHf<0DE5A0P1[Jf/0?`1]KFd0IFEU08:2PP1^KVh0K6a/05mOG`1hN7P0
JFUY05eMG@1mOGd0GUiN06IVIP1iNGT0Bd];07MgM`1`L700ADE508B4Q01gMgL0Cdm?07inOP1GEeL0
MGEe06e]K@1>CTh0I6AT06m_K`1DE5@0IFEU06MWI`1fMWH0CDe=04];B`1mOGd0De=C05aLG01nOWh0
EeMG06QXJ01`L700EUIF06YZJP1`L700EUIF075aL@1gMgL0BTY:07inOP1LG5`0FUYJ06a/K01BDU80
L71`059BDP1?Cdl0HV9R05UIF@1bLW80GEeM05UIF@1SHf<0EeMG06i^KP0>ooooo`0E07]kN`1QHF40
CDe=04e=C@1OGel0DU9B06YZJP1iNGT0KVi^06UYJ@0i>CT0B4Q804A4A01>CTh0F5QH04a<C01JFUX0
GEeM05EEE@1ADE40FEUI0080ATI60100@D5104Q8B015ADD0@41004i>CP1[Jf/0E5AD05iNGP1JFUX0
HV9R06YZJP1LG5`0K6a/05mOG`15ADD0HV9R0P1BDU80=P0i>CT0?Cdm030`<010@400JVYZ04M7A`13
@d<0BDU905IFEP19BDT0?Shn05QHF01mOGd0JFUY06MWI`1@D500JFUY05ADE01bLW80Kfm_06e]K@2<
S8`0P82006IVIP1@D500Jf][05YJFP1LG5`0C4a<05QHF01<C4`0KVi^06e]K@1EEED0Cdm?06a/K01P
H600OWin06][J`18B4P0EEEE06YZJP1OGel0H61P05EEE@1JFUX0JVYZ065QH@1UIFD0Bd];055AD@1C
De<0@41005QHF080>c/k02/0Bd];03De=@0i>CT0GEeM03Xj>P0b<S80=SHf01lO7`17AdL0ADE5059B
DP11@D40B4Q805EEE@16ATH0AdM704E5A@0R8R80@d=304];B`0m?Cd0Bd];05YJFP0o?cl0E5AD05mO
G`1JFUX0G5aL04m?C`1HF5P0K6a/04E5A@1OGel0IfMW04U9B@12@T80De=C05QHF014A4@0FUYJ05=C
D`1HF5P0Kfm_00kooooo02P0N7Qh069RHP0l?3`0CDe=07YjNP10@400ATI603Hf=P0k>c/0;Bd]0451
@@0l?3`0:b/[04Y:BP0f=SH0=CDe034a<@1UIFD0:2PX04a<C01BDU80C4a<05eMG@0o?cl0GemO06YZ
JP1>CTh0FUYJ03Lg=`1BDU80I6AT04=3@`1ADE40B4Q8034a<@1FEUH0EEEE04e=C@0l?3`0Kfm_0P1F
EUH206i^KP0d06IVIP2PX:00BDU906EUI@2d];@0IFEU06i^KP1PH600IFEU07EeM@1OGel0I6AT06=S
H`1VIVH0DU9B04e=C@1IFET0C4a<07]kN`1@D500HV9R05YJFP1=CDd0WIfM04Y:BP12@T80GemO06a/
K01UIFD0LW9b04Q8B01RHV80J6QX065QH@1kNg/0MgMg069RHP1jNWX0LG5a08V9R@1JFUX0NGUi09bL
W01SHf<0Bd];0492@P1YJFT0C4a<05eMG@1>CTh0De=C05iNGP80JVYZ02X0IFEU05iNGP1lO7`0FUYJ
07emO@1]KFd0K6a/051@D01>CTh0@d=306IVIP16ATH0Cdm?03Lg=`1TI6@0CTi>03/k>`2LW9`0KVi^
07]kN`1`L700JFUY069RHP1^KVh0=3@d06m_K`1mOGd0Fe]K08^;R`1@D500E5AD09RHV01PH600IFEU
08>3P`1ADE40OGem07inOP0d=3@0ATI603dm?@1lO7`>ooooo`0R05MGE`1=CDd0:BTY03/k>`0i>CT0
Kfm_04i>CP0G5aL04Q8B00l?3`0B4Q80De=C05ADE02IVIT0DE5A05IFEP0T92@0:2PX02d];@0e=CD0
ADE506][J`28R8P0emOG0<G5a@2DU9@0FEUI03/k>`0g=cL0LG5a0::RXP2d];@0EUIF01DE5@804a<C
02H05ADE01lO7`2:RXX0N7Qh09^KV`1CDe<0?clo065QH@2KVi/0W9bL06=SH`0a<C40=3@d04m?C`2U
YJD0NWYj05mOG`1bLW80De=C059BDP0c<c<0;Rh^06MWI`1VIVH0@D5107]kN`2g]kL0Cdm?04Q8B00o
?cl0=CDe03<c<`1/K6`0F5QH07=cL`1dM7@0`<3005mOG`80=3@d0180DE5A01`L700K6a/0:b/[061P
H00F5QH061PH05UIF@1DE5@0@d=306i^KP1GEeL0:b/[02@T900P8200EUIF05mOG`1;Bd/205EEE@0@
0;fm_@13@d<0<C4a06QXJ01SHf<0CDe=05MGE`1RHV80TY:B04I6AP1?Cdl0S8b<08J6QP0Y:BT0BDU9
0410@080C4a<01P0aLG50;>c/`1FEUH0Lg=c06YZJP0`<300BDU909fMW@2XZ:P0Rh^;07AdM00U9BD0
8b<S04a<C02d];@0MgMg06MWI`1?Cdl0SXj>08n?S`0c<c<0KFe]09bLW02UYJD2061PH1Cooooo000C
ooooo`2;071`L021PH40GEeM06MWI`1ADE40IFEU05EEE@1?Cdl0FEUI06ATI01ADE40GEeM06YZJP1H
F5P0I6AT06a/K016ATH0MWIf05EEE@1;Bd/0M7Ad04M7A`1>CTh0De=C059BDP1EEED0A4A405=CD`1J
FUX0>SXj07AdM01JFUX0C4a<06YZJP1DE5@0EUIF05mOG`1SHf<0ATI6051@D01ZJVX0CTi>0492@P1R
HV80EUIF04E5A@1DE5@0CTi>0492@P1=CDd0JVYZ04];B`1ADE40MWIf04i>CP1JFUX0LG5a04Y:BP12
@T80KFe]04U9B@1VIVH0MgMg05MGE`1VIVH0IFEU04Y:BP1VIVH0Hf=S06e]K@1MGEd0Jf][07UiN@1I
FET0EeMG07YjNP1VIVH0Hf=S08Z:RP1PH600IfMW07alO01XJ6P0Lg=c06e]K@1hN7P0Hf=S06YZJP1Q
HF40IVIV06EUI@1KFe/0GemO06UYJ@1TI6@0Ph>305mOG`1ADE40QHF504U9B@1UIFD0KFe]06m_K`1W
IfL0Lg=c079bLP1]KFd0EUIF06ATI01hN7P0Hf=S059BDP1QHF40GEeM06i^KP1XJ6P0LW9b06QXJ01U
IFD0N7Qh05mOG`1WIfL0PH6106EUI@1VIVH0Ph>305]KF`1MGEd0KVi^03Xj>P1>CTh0JFUY04M7A`1R
HV80GEeM04=3@`1RHV80EEEE04E5A@0>ooooo`1D07moO`1JFUX0EUIF05QHF01SHf<0EEEE079bLP1W
IfL0GemO05QHF00m?Cd0>SXj04Q8B01;Bd/0HV9R051@D01RHV80EeMG05aLG01=CDd0AdM70451@@0n
?Sh0>c/k04E5A@0o?cl0@41003Xj>P1EEED0DU9B05YJFP1=CDd0HF5Q06=SH`1VIVH0Jf][05IFEP12
@T80G5aL05IFEP1@D500@41004Q8B011@D40C4a<06e]K@12@T80Bd];04I6AP1IFET0C4a<04M7A`1E
EED0IfMW06EUI@21PH40IVIV06YZJP1;Bd/0Cdm?05eMG@1gMgL0Ti>C07alO01nOWh0IfMW07MgM`1X
J6P0E5AD04U9B@1FEUH0Bd];06m_K`1hN7P0G5aL05=CD`1IFET0CTi>075aL@1KFe/0FUYJ05QHF01T
I6@0De=C0P1PH6004P1NGUh0GemO061PH01bLW80LG5a04];B`1EEED0B4Q805EEE@0l?3`0AdM70492
@P19BDT0A4A406i^KP14A4@0<c<c03Ti>@80B4Q80280AdM705QHF010@400@T9204I6AP10@400@d=3
0410@00T92@0?3`l051@D01>CTh0ATI606MWI`16ATH0Bd];06QXJ01VIVH0EeMG04I6AP1SHf<0IfMW
04Y:BP1GEeL0I6AT05eMG@1HF5P0EUIF04m?C`18B4P0F5QH059BDP1MGEd0I6AT3_ooool0<`1^KVh0
Hf=S03Xj>P1VIVH0Ogmo04M7A`19BDT0<C4a03lo?`0b<S80ATI6038b<P0i>CT0E5AD03@d=00k>c/0
?Cdm04Q8B00O7al0>c/k0492@P0o?cl0@d=303hn?P1EEED0IFEU05MGE`1JFUX0;2`/065QH@1FEUH0
<c<c05UIF@0f=SH09BDU05MGE`1>CTh0D51@03De=@1JFUX0GemO04I6AP2:RXX0I6AT061PH02RXZ80
Fe]K061PH030`<00BTY:05mOG`0205]KF`0L07AdM01OGel0EeMG079bLP1ZJVX0IFEU059BDP1SHf<0
F5QH08F5Q@1UIFD0L71`065QH@1CDe<0SXj>04];B`10@400FUYJ071`L015ADD0M7Ad04e=C@1SHf<0
Kfm_071`L01bLW80LG5a07EeM@80Ph>30140UIFE05]KF`1oOgl0YJFU05MGE`1TI6@0ADE505]KF`1C
De<0F5QH04];B`1?Cdl0L71`07EeM@1]KFd0I6AT051@D00205YJFP0A08R8R01eMGD0O7al0492@P1?
Cdl0?Shn079bLP1CDe<0C4a<03@d=01GEeL0CDe=03lo?`2RXZ80D51@06QXJ01UIFD00P1LG5`04`1b
LW80=CDe04];B`1]KFd0Fe]K07EeM@1;Bd/0E5AD07EeM@1EEED0De=C07=cL`12@T80RXZ:09:BTP0k
>c/0BDU903/k>`1iNGT03_ooool0<`26QXH0Cdm?04i>CP0a<C40@T9204A4A00g=cL0ADE501dM7@0B
4Q80410@01TI6@2QXJ40RHV903Lg=`1>CTh09bLW01/K6`0H61P0820P01hN7P0m?Cd0Lg=c08F5Q@1b
LW80IVIV06m_K`0o?cl0BDU907QhN02MWId0XZ:R061PH00V9RH08B4Q02<S8`0P8200CTi>08f=S@1=
CDd0[Zj^08:2PP1;Bd/0@41005MGE`1WIfL0H61P08j>SP22PX80WYjN0;bl_00208>3P`1G04M7A`1@
D500>CTi02PX:00L71`0DE5A07=cL`1SHf<0NGUi08F5Q@1UIFD0?clo04I6AP0f=SH0=cLg07MgM`1Q
HF40NGUi07AdM019BDT0?3`l038b<P0I6AT0=3@d04a<C00W9bL0E5AD0;:b/P10@400=3@d02/[:`0m
?Cd0F5QH0:2PX01FEUH0D51@04a<C01XJ6P0WinO06MWI`0^;Rh0Lg=c06e]K@1bLW80];Bd06MWI`0j
>SX0S8b<02HV9P0h>3P0O7al08J6QP15ADD0C4a<04=3@`0_;bl05QHF01`L701=CDd0Bd];08n?S`35
aLD0][Jf09JFUP0m?Cd08b<S03`l?01YJFT0QHF509bLW00m?Cd0;Bd]02h^;P0/;2`0Jf][04M7A`0b
<S80EEEE07UiN@1=CDd0A4A407UiN@1/K6`0JVYZ07]kN`1UIFD0DU9B01Cooooo000Cooooo`0_08:2
PP1UIFD0MWIf06=SH`1DE5@0Ng]k03Xj>P1>CTh0C4a<03`l?01XJ6P0DU9B05ADE01eMGD0>SXj055A
D@1UIFD0G5aL04];B`1LG5`0H61P069RHP17AdL0@D51065QH@1NGUh0>3Ph06QXJ015ADD0=CDe08B4
Q01:BTX0Fe]K06EUI@1>CTh0J6QX06EUI@1BDU80IVIV03/k>`1BDU80Ng]k02LW9`1RHV80Cdm?02`/
;01/K6`00P10@4009`1ADE40HV9R051@D01NGUh0CTi>05MGE`1IFET0Hf=S06IVIP1BDU80J6QX04U9
B@1ZJVX0OGem06][J`1cLg<0J6QX075aL@1;Bd/0IFEU07MgM`13@d<0IVIV08F5Q@12@T80IfMW07Qh
N017AdL0Hf=S08V9R@1:BTX0Fe]K07=cL`1=CDd0LG5a06ATI01JFUX0HV9R06=SH`0205mOG`0a075a
L@1FEUH0FUYJ05mOG`1YJFT0FEUI05]KF`1UIFD0GEeM06QXJ01MGEd0De=C08j>SP18B4P0Kfm_07Ui
N@1LG5`0JFUY04M7A`1YJFT0Jf][03`l?01TI6@0RXZ:05IFEP1iNGT0SHf=059BDP1mOGd0Ph>304I6
AP22PX80M7Ad03dm?@1_Kfl0Ph>30492@P1]KFd0HF5Q051@D01VIVH0EUIF04M7A`2<S8`0De=C04M7
A`2ATI40Cdm?05MGE`0>ooooo`0R06i^KP1?Cdl0HF5Q05=CD`1HF5P0Cdm?06i^KP1PH600HF5Q04i>
CP14A4@0<c<c04M7A`0k>c/0E5AD04U9B@1RHV80E5AD05QHF017AdL0>c/k0410@00e=CD0;bl_03Hf
=P0k>c/0@D5104M7A`1BDU80CDe=05ADE01CDe<0IVIV04a<C080F5QH04P0C4a<03dm?@1FEUH0DE5A
04m?C`11@D40B4Q803dm?@1=CDd0KVi^04Q8B01:BTX0B4Q805QHF01<C4`0A4A406=SH`1bLW80IFEU
08>3P`1nOWh0GEeM05=CD`1ADE40FUYJ06UYJ@2CTi<0QHF50820P01VIVH0QHF504I6AP14A4@0BDU9
04E5A@1GEeL0E5AD05iNGP1CDe<0EEEE065QH@1^KVh0O7al04a<C01=CDd0EEEE05]KF`1:BTX0Jf][
069RHP1LG5`0B4Q806e]K@1_Kfl0N7Qh03/k>`1EEED0BTY:04e=C@1OGel0AdM705YJFP1?Cdl0BDU9
05mOG`0l?3`0;bl_03`l?013@d<0>c/k04M7A`1HF5P204I6AP0N04i>CP18B4P0AdM704a<C00n?Sh0
B4Q805ADE01GEeL0C4a<04m?C`10@400De=C05UIF@1NGUh0FUYJ05MGE`1dM7@0MGEe055AD@1TI6@0
DU9B04=3@`14A4@0CDe=059BDP1<C4`0De=C055AD@1JFUX0JVYZ3_ooool0S01]KFd0HV9R04i>CP1V
IVH0M7Ad04a<C015ADD0@D5103hn?P0Y:BT0@d=302/[:`0k>c/0CTi>03<c<`1<C4`0=CDe04Q8B00]
;Bd0?3`l04A4A011@D40EeMG04U9B@1JFUX0GUiN04Q8B01QHF40<30`055AD@18B4P0ATI605eMG@0j
>SX0>c/k05iNGP1DE5@0D51@03<c<`1[Jf/0GEeM03Hf=P1eMGD0JVYZ07YjNP2UYJD0EUIF06UYJ@31
`L40EeMG06=SH`1DE5@0FEUI05YJFP1QHF40D51@06=SH`1XJ6P0DE5A04Y:BP1JFUX0D51@075aL@1[
Jf/0LW9b05QHF017AdL0NWYj04U9B@0n?Sh0IVIV07QhN00i>CT0Hf=S04E5A@1RHV80N7Qh07YjNP1Z
JVX0LG5a07AdM02PX:00IFEU09VIV@1HF5P0NGUi0;Bd]01UIFD0NGUi04M7A`1ADE40FEUI04a<C01=
CDd0B4Q807AdM01ZJVX0NGUi05aLG01CDe<0GUiN04];B`1bLW80Kfm_07alO013@d<0De=C04];B`1]
KFd0GUiN05eMG@0g=cL0GEeM04=3@`1LG5`0P82004Y:BP1^KVh0N7Qh06MWI`1KFe/0PX:203dm?@0k
>c/0I6AT07MgM`1hN7P0De=C06ATI01YJFT0FUYJ06ATI01ZJVX0GUiN07QhN02][Jd0ADE50451@@0n
?Sh0O7al3_ooool04`2<S8`0L71`06a/K00i>CT071`L01hN7P0^;Rh0<C4a04I6AP0P8200=SHf05iN
GP1RHV80X:2P08R8R00`<3004a<C010@400E5AD00P0F5QH03P0H61P0Fe]K03Hf=P0d=3@0FEUI069R
HP0j>SX0B4Q8071`L02QXJ40OWin079bLP0E5AD051@D0`0E5AD0IP0F5QH0D51@08n?S`0Y:BT05ADE
01@D500W9bL0>CTi03@d=01fMWH0AdM706][J`1jNWX0FUYJ0410@00o?cl0=3@d02XZ:P0Q8B40:RXZ
08B4Q02=SHd0Vi^K059BDP0g=cL0@T92051@D01;Bd/0TY:B092@T028R8P0Fe]K01lO7`0_;bl04a<C
01DE5@0g=cL0DE5A01dM7@1EEED0YZJV0:n_[`1BDU80LG5a08Z:RP2>SXh0WinO09ZJVP2g]kL0H61P
03/k>`0/;2`0;bl_05ADE01kNg/0OWin05aLG00H61P0ATI606=SH`2WYjL0Z:RX0:NWY`19BDT0Cdm?
09BDU02l_;`0QHF5059BDP20P800<c<c01/K6`0E5AD051@D05mOG`2SXj<0^;Rh0:^[Z`1[Jf/0@D51
03<c<`1FEUH0D51@05eMG@1PH6009RHV01TI6@0i>CT0NGUi06][J`1OGel0HV9R04i>CP0o?cl0Ti>C
04Q8B020P800FEUI02@T9017AdL05aLG04Y:BQCooooo000Cooooo`80O7al0480CDe=05UIF@1UIFD0
D51@05UIF@1QHF40Bd];06EUI@0k>c/0ADE507UiN@0i>CT0EeMG06IVIP0a<C40JFUY05ADE00_;bl0
Kfm_05EEE@0o?cl0FUYJ04i>CP1LG5`0AdM705IFEP1PH600?3`l055AD@1XJ6P0?3`l04];B`1mOGd0
De=C055AD@1mOGd0?3`l04i>CP2;Rh/0?Shn05QHF01cLg<0C4a<05=CD`13@d<0?clo06][J`0W9bL0
CDe=06m_K`0U9BD0GEeM06ATI00k>c/0IFEU059BDP0l?3`0Ng]k04i>CP1=CDd0Q8B406=SH`1ZJVX0
W9bL079bLP1TI6@2065QH@0N06YZJP1UIFD0KVi^05QHF01TI6@0IVIV069RHP1SHf<0LG5a07UiN@1j
NWX0HV9R06YZJP1VIVH0DE5A07AdM00f=SH0Hf=S06EUI@0a<C40PX:204Y:BP1;Bd/0IfMW05MGE`1`
L700Kfm_04a<C02>SXh0HF5Q0P1NGUh0401ADE40EEEE06IVIP1PH600NGUi055AD@15ADD0NWYj04e=
C@1JFUX0KVi^04e=C@1lO7`0Kfm_051@D01eMGD2061PH00C06ATI01LG5`0IFEU07YjNP1KFe/0L71`
06QXJ01EEED0K6a/05QHF01;Bd/0IfMW04i>CP1ZJVX0IVIV04Q8B01JFUX0D51@04I6AP0>ooooo`0M
06IVIP1GEeL0HF5Q061PH01OGel0GUiN07=cL`1iNGT0LW9b065QH@18B4P0BDU905UIF@1LG5`0J6QX
05aLG01UIFD0G5aL06=SH`15ADD0D51@04];B`18B4P0=3@d04E5A@0a<C40AdM705QHF01IFET00P1O
Gel0201GEeL0MGEe04];B`1FEUH0LG5a05UIF@1;Bd/0FUYJ0P1;Bd/01`0f=SH0?Cdm03lo?`13@d<0
EeMG04I6AP11@D400P16ATH0FP1<C4`0?3`l051@D01MGEd0DE5A05=CD`1QHF40GUiN05MGE`1<C4`0
EeMG07AdM02ATI40IfMW08:2PP1OGel0KVi^04U9B@0n?Sh0Bd];04m?C`1RHV80O7al06a/K01DE5@0
De=C06a/K01GEeL0NWYj04U9B@17AdL0E5AD05mOG`1IFET0J6QX04];B`1LG5`0DU9B069RHP1ZJVX0
W9bL05aLG01EEED0CDe=05mOG`1XJ6P0Bd];065QH@1LG5`0Fe]K079bLP1_Kfl0CTi>04A4A01<C4`0
CDe=0451@@1FEUH0BTY:04I6AP1LG5`0GEeM04=3@`1GEeL0>CTi04E5A@1:BTX0CDe=05ADE01bLW80
DE5A061PH011@D40GEeM05aLG01ZJVX0O7al07AdM013@d<0EEEE07YjNP1QHF40ADE504i>CP15ADD0
?Shn05UIF@17AdL0F5QH06IVIPkooooo00h0Ng]k06IVIP1gMgL0MWIf06m_K`1=CDd0?Cdm03Ph>00l
?3`0:BTY04A4A00Z:RX0<c<c051@D080>3Ph0P0f=SH0KP0];Bd0=SHf03Xj>P1>CTh0A4A404a<C01?
Cdl0FEUI04m?C`1HF5P0=3@d04e=C@15ADD0@41004U9B@0e=CD09BDU04M7A`13@d<0@D5102l_;`1X
J6P0FUYJ0410@01iNGT0I6AT06e]K@2_[jl0GUiN07IfMP39bLT0GUiN06YZJP1VIVH0I6AT06YZJP1S
Hf<0E5AD079bLP1TI6@0F5QH04Y:BP1[Jf/0Fe]K06m_K`1`L700LW9b05YJFP14A4@0Rh^;04i>CP19
BDT0I6AT07MgM`0n?Sh0DE5A04a<C01YJFT0JVYZ07EeM@1XJ6P0MGEe06YZJP2`/;00F5QH0;Bd]01[
Jf/0LW9b07moO`1SHf<0Lg=c03Hf=P1MGEd0De=C04m?C`1@D500?3`l05UIF@1DE5@0IfMW06e]K@1;
Bd/0@d=30410@01oOgl0Ng]k0820P00k>c/0HF5Q04m?C`1XJ6P0Jf][05]KF`0n?Sh0IfMW03hn?P12
@T80NWYj04m?C`1SHf<0MgMg06e]K@1MGEd0L71`03Xj>P1=CDd0I6AT07=cL`1lO7`0DU9B0P1[Jf/0
2P1ZJVX0GEeM08^;R`1kNg/0OGem0:f][@19BDT0Bd];03Lg=`21PH4>ooooo`0507emO@16ATH0S8b<
03Lg=`0F5QH00P0C4a<0Q@1<C4`0O7al06m_K`0];Bd0:2PX051@D02c/k<0LG5a061PH00N7Qh05ADE
01HF5P0E5AD051@D01DE5@1:BTX09BDU01DE5@0];Bd0Ogmo04I6AP0`<300=3@d03Ph>01fMWH0`/;2
03dm?@0E5AD051@D01TI6@0T92@0Cdm?08R8R02i^KT0A4A401LG5`0E5AD0E5AD01lO7`0Z:RX0HF5Q
04E5A@0g=cL0MgMg096AT@11@D40CDe=01HF5P0O7al0;2`/03Hf=P2:RXX0Lg=c05ADE00^;Rh051@D
01/K6`0_;bl0B4Q80:2PX033`l<0CDe=01PH600b<S808b<S01HF5P0D51@05ADE04M7A`1:BTX0OWin
0=;BdP1PH600EEEE06YZJP2:RXX0YjNW0;Zj^P1aLG40:2PX05mOG`20P800@D510410@02>SXh0[Zj^
07]kN`0Q8B40B4Q803De=@0S8b<0SXj>09BDU027QhL0W9bL096AT@2GUiL0FUYJ05EEE@20P800[:b/
071`L019BDT071`L02/[:`1KFe/0g]kN0=?Cd`17AdL09BDU04M7A`24Q8@0:BTY02<S8`1DE5@0Hf=S
02PX:00T92@0KVi^0:j^[P1lO7`0N7Qh05ADE00a<C40=SHf0:ZZZP20P800PX:207AdM01^KVh09bLW
01HF5P0h>3P05?ooool001?ooooo0P1cLg<0A`1NGUh0HF5Q04m?C`1VIVH0BDU904e=C@1@D500Fe]K
0410@01OGel0BTY:055AD@1LG5`0GEeM05=CD`1[Jf/0EeMG04];B`1TI6@0CTi>04M7A`14A4@0BDU9
05iNGP0k>c/0FUYJ04];B`0j>SX0G5aL05QHF012@T80GemO051@D01MGEd0EUIF05QHF01<C4`0?clo
06IVIP1DE5@0ADE506m_K`1GEeL0?clo04Y:BP1SHf<0C4a<0410@01IFET0GEeM051@D01[Jf/0BTY:
03/k>`1^KVh0B4Q803hn?P1eMGD0ADE5069RHP1]KFd0EEEE06IVIP1ZJVX0Fe]K05mOG`1GEeL0IVIV
055AD@1RHV80LG5a0080HV9R00@0NWYj06=SH`1QHF40LW9b0P1ZJVX0>P1^KVh0GemO07UiN@1ZJVX0
L71`05=CD`1ZJVX0FEUI055AD@22PX80CTi>06i^KP1KFe/0DU9B08N7Q`1QHF40C4a<08B4Q01FEUH0
I6AT06MWI`1iNGT0H61P05eMG@1_Kfl0KFe]05YJFP1?Cdl0IVIV05]KF`1EEED0J6QX06a/K01MGEd0
J6QX06e]K@1QHF40Lg=c06e]K@1NGUh0NWYj07inOP1CDe<0HF5Q075aL@0o?cl0GemO06=SH`0i>CT0
Fe]K06EUI@0n?Sh0GEeM05YJFP10@400J6QX05iNGP12@T8>ooooo`0`06UYJ@1QHF40KFe]05mOG`1/
K6`0GEeM075aL@1^KVh0J6QX05IFEP14A4@0CDe=065QH@1HF5P0GemO05MGE`1PH600HV9R05QHF01<
C4`0Bd];051@D016ATH0@d=30492@P0o?cl0ADE505YJFP1<C4`0GEeM05]KF`1GEeL0NWYj05IFEP1M
GEd0IFEU05EEE@10@400IfMW05IFEP1=CDd0?3`l04a<C013@d<0ATI605mOG`14A4@0@d=30P17AdL0
<01>CTh0Bd];06QXJ01PH600FUYJ051@D01>CTh0GEeM04a<C01>CTh0EeMG06EUI@20P800B4Q805iN
GP17AdL0NGUi04e=C@0k>c/0@D5104e=C@1GEeL0Lg=c04Y:BP1?Cdl0EeMG06EUI@1TI6@0RXZ:069R
HP1DE5@0I6AT05aLG01>CTh0C4a<05EEE@1GEeL0Bd];07=cL`1lO7`0SHf=05=CD`1>CTh0Fe]K07Yj
NP26QXH0ADE504U9B@80EEEE02P0QXJ607AdM017AdL0B4Q805ADE01<C4`0?clo05=CD`18B4P0ADE5
059BDP1CDe<0?3`l05ADE00d=3@0<30`04A4A019BDT0AdM706a/K00o?cl0HV9R04Y:BP1LG5`0GemO
06=SH`1mOGd0O7al055AD@1hN7P0NGUi04i>CP10@400DE5A05ADE019BDT0FUYJ04e=C@1GEeL0I6AT
3_ooool05@1hN7P0G5aL07AdM01kNg/0JVYZ04m?C`17AdL0?Cdm0410@00^;Rh0B4Q802/[:`0c<c<0
EEEE02h^;P0e=CD0?Cdm03/k>`0j>SX0>3Ph03dm?@0204i>CP1e05IFEP1EEED0EUIF05EEE@1;Bd/0
>CTi05IFEP0m?Cd0@D5104a<C00e=CD0>3Ph04Y:BP17AdL0@D5103dm?@1PH600De=C04=3@`1YJFT0
G5aL065QH@2c/k<0CTi>065QH@2n_[h0KVi^069RHP1FEUH0MWIf05ADE01KFe/0IFEU069RHP1WIfL0
Cdm?04Q8B01[Jf/0Bd];06m_K`1NGUh0K6a/05YJFP0l?3`0OWin04a<C013@d<0LW9b06][J`1=CDd0
GEeM05=CD`1NGUh0Jf][07AdM01TI6@0KVi^075aL@2GUiL0D51@0::RXP1OGel0MGEe09ZJVP20P800
OWin04E5A@1;Bd/0GUiN05QHF01DE5@0BDU905]KF`1QHF40MgMg05=CD`1ADE40Jf][05MGE`1_Kfl0
IFEU08Z:RP11@D40CTi>06YZJP1OGel0Lg=c05eMG@10@400FEUI04E5A@1HF5P0MgMg05iNGP1YJFT0
OGem07YjNP1XJ6P0G5aL02d];@1FEUH0Bd];061PH023Ph<0Cdm?06a/K01UIFD0Fe]K06QXJ02=SHd0
O7al08f=S@2TY:@0A4A405=CD`1;Bd/0Ph>300kooooo01/0P82009BDU01[Jf/0>c/k02h^;P0X:2P0
7AdM02/[:`0k>c/0T92@0:FUY@2CTi<0QXJ60:f][@2GUiL0HV9R059BDP0h>3P0<30`02PX:00G5aL0
820P04Q8B00O7al05aLG04i>CP0c<c<00P0Q8B404P0L71`06a/K024Q8@21PH40OGem06YZJP0b<S80
;Bd]02/[:`0Q8B40=cLg08N7Q`1gMgL0NGUi08R8R01SHf<0?3`l03hn?P809BDU00`0DE5A09>CT`27
QhL0Ti>C08V9R@1FEUH0BTY:06ATI027QhL0XJ6Q0:^[Z`1SHf<201@D5009014A4@0I6AT0>SXj06][
J`1`L700MWIf04m?C`1eMGD07AdM00805QHF048051@D01lO7`2EUID0KFe]075aL@2/[:`0VIVI06QX
J01KFe/0HF5Q0:JVYP1[Jf/08b<S03De=@1:BTX0SHf=05iNGP2FUYH0Lg=c08V9R@2YZJT0WYjN09RH
V01nOWh0RHV9051@D00/;2`0?3`l09ZJVP0a<C40;Rh^04U9B@1HF5P0AdM7061PH01mOGd0UiNG09>C
T`2?Shl0WinO0:j^[P26QXH0<30`03Xj>P20P800?clo01`L700O7al0JFUY02l_;`0e=CD0Xj>S08V9
R@1ZJVX07alO02@T900h>3P0De=C07]kN`0/;2`0CDe=08j>SP2e]KD0L71`02XZ:P0P820Dooooo`00
4oooool0R`2DU9@0KFe]071`L015ADD0Kfm_07emO@0o?cl0FEUI04Q8B01;Bd/0ADE505ADE01^KVh0
EeMG055AD@2>SXh0F5QH05]KF`1UIFD0AdM7059BDP1>CTh0=cLg03hn?P1TI6@0@T9203lo?`1LG5`0
@41004];B`1JFUX0ADE504m?C`1JFUX0ATI606MWI`1>CTh0AdM706m_K`18B4P0H61P06UYJ@0`<300
JFUY06ATI00h>3P0H61P05EEE@1ADE40BTY:04i>CP1JFUX0EUIF05=CD`20P800DE5A05MGE`1hN7P0
=SHf04a<C01?Cdl0C4a<05=CD`1OGel0EEEE055AD@1[Jf/0@D51055AD@1XJ6P0;Rh^06=SH`20P800
?Shn061PH01lO7`0?Shn06IVIP1aLG40ADE508F5Q@1SHf<0CDe=08R8R01CDe<0GEeM04U9B@1IFET0
H61P03/k>`1HF5P0HV9R05aLG01ADE40De=C061PH01bLW80GUiN06MWI`1/K6`0D51@05]KF`2:RXX0
ATI6071`L01/K6`0D51@06UYJ@0^;Rh0IVIV06e]K@1;Bd/0FUYJ09ZJVP0m?Cd0Ph>308V9R@18B4P0
NWYj07emO@0m?Cd0Hf=S07inOP0k>c/0K6a/06=SH`1FEUH0Ph>306=SH`1@D500IVIV05eMG@1?Cdl0
PH6104U9B@1=CDd0MWIf05=CD`1ADE403_ooool0D`1TI6@0IfMW06IVIP1QHF40FUYJ05aLG01jNWX0
QhN706QXJ01ADE40A4A404Q8B01LG5`0DU9B04];B`1QHF40GEeM06i^KP1OGel0FEUI05aLG017AdL0
ADE504A4A01=CDd0=SHf0451@@1CDe<0BDU904m?C`1CDe<0GEeM07]kN`1KFe/0GEeM065QH@1EEED0
@T9206UYJ@1IFET0BDU90410@017AdL0?Cdm0492@P1LG5`0>SXj03dm?@0o?cl0BDU904Y:BP1<C4`0
Jf][05mOG`1CDe<0BTY:04M7A`1DE5@0AdM705aLG01TI6@0PX:209JFUP1aLG40IfMW04e=C@1nOWh0
IFEU04=3@`1ADE40Bd];06=SH`1oOgl0F5QH06][J`1BDU80JFUY06QXJ02:RXX0Cdm?04I6AP17AdL0
DU9B0080Bd];01d0JFUY05MGE`1KFe/0F5QH0861P@1oOgl0@d=305=CD`1FEUH0Kfm_06QXJ01:BTX0
B4Q8061PH010@400MgMg04Q8B00Z:RX0>3Ph04m?C`12@T80?Cdm04U9B@13@d<0C4a<04Q8B01HF5P0
;Rh^04m?C`020492@P0H04U9B@1:BTX0DE5A07AdM015ADD0HV9R04M7A`1FEUH0HF5Q055AD@1WIfL0
Kfm_04I6AP1^KVh0G5aL04m?C`1@D500E5AD04];B`16ATH0NGUi04U9B@1KFe/0J6QX3_ooool0101m
OGd0Fe]K07]kN`2=SHd205aLG0260492@P0c<c<0>c/k02`/;015ADD0<C4a02l_;`1<C4`0<30`03<c
<`0o?cl0<c<c04Y:BP0h>3P0?Cdm05UIF@1>CTh0HV9R04a<C01JFUX0I6AT03lo?`1?Cdl0F5QH059B
DP13@d<0C4a<03hn?P0m?Cd0C4a<059BDP0f=SH0@T9206QXJ01>CTh0?Shn08b<S01^KVh0=3@d0:2P
X01JFUX0G5aL09JFUP21PH40D51@06YZJP1XJ6P0FEUI04];B`1TI6@0P82006EUI@1CDe<0EEEE05eM
G@1<C4`0EEEE05mOG`1PH600HV9R03lo?`23Ph<0E5AD04e=C@1hN7P0IVIV05=CD`1OGel0=SHf065Q
H@1JFUX0NGUi04e=C@19BDT0Lg=c09JFUP1ZJVX0QhN707EeM@25QHD0YJFU075aL@28R8P0?Cdm0410
@01JFUX0EeMG061PH00o?cl0>SXj06][J`1]KFd0Cdm?04a<C01=CDd0AdM708N7Q`1^KVh0LW9b04E5
A@1IFET0EeMG05eMG@1QHF40Bd];03Ti>@1SHf<0>c/k059BDP28R8P0B4Q806EUI@28R8P0HF5Q05=C
D`1RHV80>CTi04e=C@0k>c/0NWYj08:2PP1GEeL0JVYZ06a/K01JFUX0Lg=c0:BTY01YJFT0Ph>309BD
U015ADD0GUiN03/k>`2<S8`>ooooo`0M08B4Q01@D500=3@d02XZ:P0c<c<0Fe]K05=CD`0L71`0@d=3
04i>CP0h>3P0?Shn07moO`27QhL0S8b<0::RXP1ADE40;2`/03Ti>@15ADD0DU9B08Z:RP18B4P0?clo
05YJFP0i>CT0Bd];02TY:@0D51@00P0C4a<0K@1<C4`0NGUi08R8R02IVIT0CDe=04M7A`1IFET0SHf=
0::RXP0G5aL0;Bd]079bLP2>SXh0LW9b09VIV@1/K6`0>CTi04U9B@15ADD0FEUI0;Vi^@2GUiL0@T92
055AD@11@D40CDe=06YZJP2HV9P0UiNG05YJFP0D51@04Q8B010@400I6AT0L71`0;6a/@2k^k/0LG5a
06IVIP2`/;00>3Ph028R8P0G5aL05ADE028R8P1/K6`0DE5A07emO@1kNg/0BTY:01/K6`0H61P0?3`l
05MGE`2CTi<0BDU902PX:012@T80>SXj08N7Q`0n?Sh0K6a/0:f][@3Ogml0g=cL0=oOg`2SXj<0MGEe
03Ti>@0N7Qh0@41006QXJ00Y:BT0=CDe030`<00j>SX0V9RH05eMG@1LG5`0WYjN09>CT`16ATH0BDU9
04a<C02ATI40Q8B403hn?P0g=cL0HV9R05IFEP1XJ6P0X:2P071`L02@T900O7al06=SH`1=CDd07AdM
02TY:@1:BTX0GemO08b<S01=CDd0Ti>C0;^k^`1TI6@0De=C05aLG01EEED05?ooool001?ooooo03T0
NGUi08:2PP1?Cdl0EeMG05]KF`17AdL0IFEU0492@P15ADD0Lg=c02h^;P1;Bd/0IVIV03<c<`1CDe<0
JVYZ03dm?@1QHF40EeMG034a<@1JFUX0?clo04e=C@19BDT0C4a<08J6QP14A4@0BTY:06e]K@0j>SX0
B4Q806EUI@0Z:RX0E5AD071`L012@T80EeMG08B4Q01:BTX0HV9R06=SH`14A4@0FUYJ065QH@1?Cdl0
EeMG03`l?01ZJVX0FEUI02PX:01NGUh0G5aL03Ti>@1PH600HF5Q05QHF01MGEd00P1DE5@01@1ZJVX0
@d=304e=C@1cLg<0=SHf0080Hf=S04T0@d=306YZJP13@d<0Cdm?04i>CP1DE5@0I6AT06e]K@1@D500
JFUY071`L01:BTX0I6AT071`L01BDU80GemO08>3P`1<C4`0L71`07emO@19BDT0Ng]k06=SH`11@D40
MGEe061PH01TI6@0GEeM051@D01jNWX0IfMW04i>CP2CTi<0JVYZ06IVIP1PH600GUiN06ATI01?Cdl0
Kfm_07QhN01BDU80GemO065QH@1KFe/0N7Qh06=SH`1LG5`0L71`061PH01WIfL0M7Ad0492@P2:RXX0
OGem05EEE@2>SXh0Cdm?05YJFP1iNGT0DU9B05IFEP1iNGT0?clo05]KF`27QhL0>c/k06EUI@1OGel0
>c/k05]KF`1[Jf/0AdM700kooooo0100HF5Q04e=C@1MGEd0C4a<05aLG01BDU80GemO079bLP1YJFT0
ADE504Y:BP19BDT0FUYJ04];B`1JFUX0CDe=0P1MGEd01`1JFUX0BTY:04U9B@14A4@0?Cdm0492@P11
@D400P0o?cl0=P1OGel0C4a<04e=C@1EEED0HV9R0861P@1RHV80JFUY06MWI`1<C4`0AdM705aLG01O
Gel0DE5A04Q8B01CDe<0@41005EEE@1[Jf/0BTY:04M7A`1@D500BDU904i>CP1HF5P0Ogmo05mOG`1D
E5@0GEeM04I6AP1EEED0DE5A04I6AP1JFUX0IVIV08>3P`1JFUX0I6AT06IVIP2>SXh0IVIV051@D01>
CTh0AdM705EEE@24Q8@0I6AT05UIF@1lO7`0OWin07moO`1nOWh0JVYZ051@D080E5AD00/0D51@059B
DP1hN7P0GUiN05MGE`1CDe<0IfMW08V9R@1TI6@0D51@04Y:BP0207=cL`0L04E5A@18B4P0HV9R04m?
C`1lO7`0GemO03@d=010@400CTi>04Y:BP16ATH0Bd];04e=C@1FEUH0CTi>05mOG`12@T80J6QX0451
@@0m?Cd0BDU904e=C@1<C4`0IFEU04Q8B01SHf<0>c/k069RHP80Bd];00h0G5aL05]KF`1=CDd0KFe]
05=CD`0n?Sh0@d=304M7A`0l?3`0CTi>06i^KP17AdL0EUIF07=cL`kooooo0180NGUi04Y:BP1JFUX0
M7Ad05YJFP1@D500:b/[03`l?00f=SH0@D5104];B`0l?3`0@D5104e=C@0R8R80>CTi03/k>`0i>CT2
03Xj>P0b030`<00c<c<0BTY:04m?C`1@D500ADE505EEE@17AdL0Bd];04Q8B016ATH0Bd];04=3@`0n
?Sh0B4Q804m?C`1;Bd/0<S8b03hn?P1MGEd0C4a<04M7A`1`L700KFe]04m?C`2GUiL0DU9B05mOG`2Z
ZZX0NWYj05IFEP1=CDd0IVIV059BDP17AdL0LW9b07MgM`1YJFT0Cdm?051@D01eMGD0C4a<06YZJP1P
H600IVIV06e]K@0m?Cd0MgMg05IFEP1;Bd/207QhN00e059BDP1/K6`0?Shn05aLG01_Kfl0Jf][05eM
G@1@D500HF5Q08f=S@1JFUX0TI6A05YJFP1bLW80ZJVY07alO02GUiL0Cdm?0410@01KFe/0G5aL069R
HP1>CTh0E5AD06MWI`1dM7@0DU9B055AD@1SHf<0FUYJ06][J`1dM7@0NWYj04Q8B01EEED0L71`04A4
A01gMgL0GEeM03dm?@1PH600AdM705aLG01oOgl0De=C06UYJ@1fMWH0NGUi06ATI01SHf<0>CTi05AD
E00l?3`00P24Q8@03@0m?Cd0M7Ad08:2PP1NGUh0IfMW09NGU`1>CTh0MGEe09>CT`0k>c/0BTY:04i>
CP2?Shl03_ooool0B@2FUYH0JVYZ03Hf=P0_;bl08b<S04Y:BP25QHD0OGem03Xj>P0a<C409BDU02/[
:`0X:2P05QHF05]KF`2JVYX0UiNG05YJFP0[:b/0;2`/051@D02>SXh0MGEe0:>SX`2BTY80R8R805AD
E00Z:RX05aLG01TI6@1BDU80<S8b02XZ:P2MWId0XJ6Q07YjNP1^KVh0OGem0;no_`2YZJT0FUYJ01`L
701?Cdl0clo?0:>SX`1iNGT0EeMG06IVIP0H61P05ADE03@d=01UIFD0IfMW0492@P0`<300<C4a02h^
;P1EEED0Ogmo05UIF@1/K6`0=cLg01@D500K6a/0>3Ph05QHF01lO7`0b<S80;6a/@2d];@0E5AD03`l
?00f=SH00P0/;2`0@@1?Cdl0Ph>308F5Q@2:RXX0GemO01TI6@0C4a<05ADE02PX:01CDe<0O7al07If
MP1NGUh092@T01HF5P0Y:BT0F5QH06UYJ@2WYjL0[jn_08>3P`1]KFd0TY:B0;:b/P1mOGd0;bl_05UI
F@11@D40:BTY05=CD`0N7Qh0<C4a05UIF@0[:b/09bLW05YJFP0l?3`0@d=302DU9@0X:2P0L71`0861
P@1mOGd0L71`0:FUY@29RHT0ATI603Ti>@2ATI40d];B04I6AP1XJ6P0Bd];0410@017AdL0FEUI06m_
K`2<S8`0Ogmo0:j^[P2l_;`0<c<c02TY:@1UIFD0QHF501Cooooo000Cooooo`0L09BDU01XJ6P0L71`
05EEE@1ZJVX0IFEU04U9B@1nOWh0CTi>04a<C01GEeL0?3`l05]KF`1PH600A4A4061PH01]KFd0DU9B
055AD@1@D500ADE505]KF`1JFUX0?clo06QXJ01BDU80B4Q807AdM080BDU906d0GEeM04Q8B014A4@0
BDU903lo?`1dM7@0?Cdm04E5A@1iNGT0A4A405IFEP1VIVH0<C4a05UIF@16ATH0>3Ph051@D01<C4`0
ADE504m?C`1SHf<0=CDe05EEE@1<C4`0ADE505UIF@1=CDd0DE5A0410@01TI6@0HV9R05mOG`1MGEd0
M7Ad04];B`1OGel0NGUi04m?C`17AdL0JFUY04A4A01SHf<0OWin04e=C@1NGUh0Ogmo04=3@`1IFET0
M7Ad03hn?P1ADE40IFEU05]KF`1VIVH0FEUI069RHP1MGEd0G5aL06m_K`1PH600@41005IFEP1gMgL0
@D51061PH01QHF40Bd];061PH01MGEd0L71`05EEE@1TI6@0PH6104m?C`1QHF40Hf=S06IVIP1[Jf/0
<c<c05QHF01_Kfl0@d=3065QH@2ATI40?Shn05mOG`2MWId0CTi>05aLG01[Jf/0ADE506QXJ01nOWh0
=CDe05UIF@1JFUX0@d=306ATI01LG5`0AdM706QXJ01iNGT0BDU906UYJ@1?Cdl0Fe]K07YjNP1?Cdl0
D51@00kooooo00<0KVi^05=CD`1QHF400P1MGEd0Q`1DE5@0IVIV07YjNP1aLG40ATI604e=C@17AdL0
EeMG04a<C01MGEd0E5AD06EUI@1RHV80E5AD059BDP1FEUH0Cdm?059BDP1=CDd0Cdm?03Ph>00k>c/0
CDe=03Ti>@16ATH0EUIF061PH028R8P0CDe=06ATI01ZJVX0Fe]K04U9B@1GEeL0EEEE05MGE`18B4P0
DE5A03lo?`16ATH0EEEE04=3@`1FEUH0C4a<04I6AP1IFET0EUIF07inOP1RHV80GemO05YJFP1[Jf/0
Cdm?04e=C@1>CTh0GUiN06i^KP2HV9P0Jf][06IVIP1BDU80L71`05QHF00m?Cd0BDU905ADE01IFET0
NWYj04M7A`1@D500DE5A05QHF01VIVH0Ogmo05UIF@18B4P0CTi>05UIF@18B4P0@41006][J`1ADE40
BTY:051@D01BDU80RHV905MGE`1OGel0CTi>06UYJ@1nOWh0ADE504M7A`1OGel0E5AD08:2PP12@T80
;bl_04Y:BP1>CTh0ATI603/k>`18B4P0ATI604m?C`1ADE40FEUI0410@01=CDd0<c<c03Hf=P10@400
?Cdm04e=C@1BDU80@d=305UIF@0_;bl0EUIF0410@01?Cdl0G5aL05eMG@1DE5@0EUIF04];B`15ADD0
AdM704];B`12@T80?Shn06m_K`1=CDd0H61P06=SH`0>ooooo`0A07emO@1CDe<0EUIF06][J`1PH600
De=C051@D00h>3P0=SHf02h^;P0o?cl0@41003<c<`13@d<0;Rh^02PX:01;Bd/00P0b<S80N@13@d<0
<30`04i>CP14A4@0CDe=059BDP0l?3`0I6AT04Q8B01PH600@T9206MWI`13@d<0@41004=3@`18B4P0
F5QH0410@00Y:BT0A4A406QXJ013@d<0HV9R08^;R`1dM7@0De=C07inOP19BDT0EUIF09>CT`2ATI40
E5AD06ATI01bLW80Fe]K04];B`1XJ6P0Kfm_05iNGP1LG5`0EeMG07=cL`1@D500H61P06i^KP1XJ6P0
K6a/04U9B@20P800F5QH04a<C01eMGD0MgMg05=CD`1OGel0A4A405MGE`1YJFT0NGUi05UIF@13@d<0
F5QH09VIV@1QHF40MGEe06a/K022PX80];Bd06e]K@2[Zj/0FEUI03hn?P1IFET0GUiN071`L01=CDd0
ATI606UYJ@1hN7P0EeMG05mOG`1`L700Bd];07alO01VIVH0LG5a03`l?01VIVH0E5AD04];B`1iNGT0
FEUI02/[:`1ZJVX0Bd];04I6AP1hN7P0DU9B05EEE@26QXH0Ogmo05MGE`1EEED0@T9205IFEP0n?Sh0
LW9b08F5Q@1=CDd0KVi^06ATI01eMGD0JFUY09RHV019BDT0GEeM08N7Q`0h>3P0B4Q80410@024Q8@0
3_ooool0S02ZZZX0U9BD059BDP1:BTX0@41004m?C`2CTi<0I6AT03dm?@0h>3P0;Bd]04=3@`0G5aL0
5ADE02@T901?Cdl0/;2`0<c<c02KVi/0WYjN0=KFeP3Bd]80Vi^K0861P@2>SXh0>3Ph03dm?@1`L700
Fe]K075aL@0O7al05ADE02TY:@1gMgL0O7al065QH@0c<c<0;Bd]06a/K01hN7P0EeMG05IFEP1cLg<0
e=CD0:BTY01NGUh0LW9b06i^KP13@d<0?Cdm06i^KP2<S8`0Jf][01PH600E5AD06a/K05eMG@0l?3`0
71`L02PX:01@D500QXJ6061PH00k>c/0Bd];065QH@1MGEd0WYjN09bLW02GUiL09bLW01/K6`0_;bl0
A4A403Ph>01mOGd0`<3009>CT`2PX:00>SXj01@D500E5AD04Q8B01TI6@1[Jf/0Vi^K07EeM@1bLW80
DU9B04];B`1ZJVX0Y:BT06][J`1NGUh0U9BD04Q8B026QXH0Y:BT07QhN01@D500WinO0:j^[P0`<300
B4Q805YJFP0l?3`0>3Ph05mOG`1<C4`0Fe]K07emO@0^;Rh0:2PX028R8P0V9RH0?Cdm06][J`2OWil0
/k>c0<W9b@1nOWh0EUIF065QH@1dM7@0c/k>09VIV@2ATI40T92@0:>SX`2LW9`0MGEe0410@00_;bl0
U9BD09bLW02FUYH0I6AT05mOG`2FUYH0Z:RX5?ooool001?ooooo05@0N7Qh06][J`1RHV80GUiN061P
H01^KVh0De=C05iNGP1EEED0Hf=S03hn?P1CDe<0Jf][04M7A`1@D500FEUI03Xj>P1LG5`0Fe]K03Xj
>P1EEED0FUYJ04M7A`1@D500G5aL05ADE012@T80G5aL04];B`14A4@0FUYJ05=CD`1JFUX0De=C061P
H01^KVh0C4a<06MWI`1UIFD0@T9207QhN01BDU80AdM7065QH@1<C4`0Hf=S06UYJ@1>CTh0Kfm_03hn
?P1OGel0Cdm?03`l?01MGEd0HV9R03<c<`1QHF40FUYJ03Ti>@1HF5P0FUYJ04M7A`1TI6@0FEUI04e=
C@1BDU80Kfm_05YJFP1DE5@0J6QX059BDP1SHf<0LG5a05EEE@1OGel0Lg=c05aLG01OGel0HF5Q05mO
G`1TI6@0Hf=S06MWI`1FEUH206QXJ00@04=3@`1^KVh0I6AT04=3@`1:BTX0IfMW05iNGP1?Cdl0DU9B
06EUI@1/K6`0FUYJ06e]K@1NGUh0EUIF06MWI`80Hf=S0240GemO04e=C@1eMGD0H61P04e=C@1iNGT0
HV9R05mOG`22PX80JVYZ05YJFP1/K6`0HV9R06MWI`1<C4`0IFEU05mOG`1:BTX0K6a/05MGE`1<C4`0
MgMg059BDP1=CDd0MgMg04];B`1EEED0Lg=c04U9B@1UIFD0EEEE04Y:BP1IFET00P1EEED>ooooo`0/
06MWI`1HF5P0De=C04a<C01:BTX0G5aL06i^KP2=SHd0KVi^04a<C011@D40Bd];05QHF01LG5`0Fe]K
05eMG@1]KFd0IVIV05aLG01BDU80De=C04U9B@10@400Bd];04I6AP0h>3P0?clo055AD@11@D40C4a<
05=CD`1MGEd0T92@059BDP1OGel0Hf=S055AD@1DE5@0AdM70492@P1:BTX0BDU904a<C00i>CT20451
@@80@T9200X0B4Q804I6AP1@D500F5QH075aL@1OGel0EeMG06UYJ@1<C4`0CTi>0P1?Cdl06P1OGel0
Ng]k0:n_[`1UIFD0GemO05QHF01kNg/0G5aL059BDP1KFe/0GemO07=cL`2EUID0J6QX06MWI`1YJFT0
PX:2075aL@1hN7P0GUiN04e=C@15ADD0BTY:05]KF`1;Bd/0FEUI0P1<C4`0=01>CTh0Fe]K07inOP1F
EUH0Fe]K06=SH`1_Kfl0Rh^;04];B`0k>c/0E5AD04e=C@1UIFD0CDe=02h^;P15ADD0De=C04E5A@0m
?Cd0>SXj03hn?P1LG5`0DE5A05mOG`12@T80EeMG03De=@0h>3P0?Shn04];B`1?Cdl0FEUI0410@01M
GEd0@d=305EEE@1;Bd/0DU9B05]KF`1JFUX0HV9R05EEE@1RHV80Bd];03dm?@19BDT0Bd];04e=C@1e
MGD0DU9B05UIF@1`L70>ooooo`0:07MgM`1ADE40E5AD06IVIP1BDU80AdM705IFEP0m?Cd0@41003Xj
>P80B4Q803@0@T92055AD@0];Bd0;Bd]04M7A`0e=CD0=SHf04E5A@0[:b/0@d=305ADE01?Cdl0B4Q8
03hn?P1QHF40@41004a<C01:BTX0FUYJ04i>CP0o?cl0C4a<059BDP1HF5P0@41002`/;016ATH0Hf=S
03lo?`1?Cdl0NGUi06][J`1WIfL0M7Ad04U9B@1DE5@0P82008Z:RP1CDe<0@d=305eMG@1CDe<0A4A4
06e]K@1WIfL0GEeM06YZJP19BDT0Q8B404];B`1]KFd0GemO0P1bLW80BP0l?3`0L71`05ADE014A4@0
O7al07QhN01>CTh0GEeM04A4A01=CDd0I6AT06m_K`1[Jf/0GUiN05QHF01oOgl0D51@06][J`1MGEd0
I6AT0:j^[P1RHV80MWIf05ADE00e=CD0EeMG05IFEP1^KVh0BDU9061PH01]KFd0Lg=c04m?C`1@D500
MgMg04M7A`1jNWX0Jf][075aL@14A4@0FUYJ069RHP0l?3`0JFUY05eMG@0i>CT0G5aL05IFEP1IFET0
M7Ad05MGE`1LG5`0K6a/07alO01[Jf/0FEUI03Ph>01OGel0B4Q8071`L02:RXX0Cdm?07YjNP1TI6@0
MgMg07=cL`28R8P0A4A406=SH`2DU9@0=3@d051@D017AdL0W9bL3_ooool09`1oOgl0IFEU06EUI@1O
Gel0PX:209nOW`1mOGd0S8b<065QH@1?Cdl0FEUI069RHP0g=cL0>c/k02l_;`19BDT0NWYj09jNWP1N
GUh08R8R03Lg=`2IVIT0E5AD05aLG02QXJ40C4a<04I6AP0l?3`0@41003hn?P1@D500I6AT09NGU`16
ATH0>CTi020P800/;2`0:RXZ02@T900205mOG`1S04e=C@1aLG40UIFE0451@@2ATI40Lg=c07IfMP0o
?cl0=3@d03`l?01mOGd0MGEe06=SH`0l?3`0@D5105EEE@0Z:RX05ADE01hN7P0Y:BT0HF5Q08j>SP2Y
ZJT0]kNg06QXJ01kNg/071`L05UIF@19BDT0ATI604M7A`0e=CD07alO04U9B@2OWil0S8b<0;Ng]`0k
>c/0AdM703Xj>P0b<S80<C4a0451@@1jNWX0=cLg03Ph>01OGel0ADE5038b<P1<C4`0Q8B40:NWY`2E
UID0LW9b07=cL`1mOGd0RHV907EeM@1gMgL0F5QH08Z:RP1fMWH06a/K02<S8`1;Bd/0QXJ609nOW`1^
KVh0E5AD08R8R024Q8@0B4Q802@T900];Bd08R8R03Ti>@1[Jf/0SHf=0;Ng]`0o?cl071`L02`/;01j
NWX0Xj>S07alO01hN7P0WYjN0;jn_P22PX80E5AD06UYJ@1ZJVX0Ph>30:f][@22PX80=SHf04a<C01H
F5P0=CDe01Cooooo000Cooooo`0g06QXJ020P800C4a<07UiN@1KFe/0GEeM06ATI00i>CT0I6AT06QX
J00a<C40Bd];05mOG`15ADD0DE5A05QHF01VIVH0HF5Q04i>CP1@D500Cdm?05=CD`1FEUH0?Shn05aL
G01fMWH0?3`l04m?C`1^KVh0?Shn04Y:BP1aLG40>3Ph04A4A01_Kfl0>SXj05]KF`1cLg<0?Cdm069R
HP1NGUh0@T9204e=C@16ATH0?Shn05aLG00];Bd0AdM704i>CP0X:2P0J6QX04M7A`0l?3`0K6a/04E5
A@0205aLG01B055AD@1BDU80Hf=S03lo?`1IFET0G5aL04e=C@1IFET0Hf=S05QHF01[Jf/0ATI604];
B`1DE5@0DE5A06IVIP1_Kfl0DU9B06a/K01VIVH0BDU9071`L01]KFd0Bd];05mOG`1]KFd0?3`l069R
HP1JFUX0BDU906][J`1bLW80De=C05UIF@1ADE40Jf][03`l?01HF5P0Ng]k0451@@1JFUX0Kfm_05eM
G@1ZJVX0Fe]K06EUI@20P800C4a<05]KF`1mOGd0ATI606EUI@1KFe/0FEUI05YJFP1BDU80I6AT05UI
F@1;Bd/0R8R8061PH01IFET0S8b<05YJFP1=CDd0KFe]04U9B@1DE5@0RHV904A4A01JFUX0MWIf03Ti
>@1FEUH0P82003<c<`1DE5@0H61P038b<P1JFUX0HV9R03Ph>0kooooo03/0M7Ad05eMG@1QHF40EUIF
05=CD`1TI6@0NWYj0::RXP1gMgL0De=C04A4A01=CDd0FEUI04m?C`1DE5@0IFEU07MgM`1bLW80EEEE
055AD@1PH600CDe=04Y:BP1;Bd/0B4Q804M7A`0k>c/0K6a/04e=C@1FEUH0CTi>05eMG@2<S8`0H61P
065QH@1XJ6P0E5AD04];B`1@D500?Cdm04e=C@16ATH0CDe=0492@P15ADD0CDe=0451@@17AdL0BTY:
05=CD`1KFe/0FUYJ07]kN`1YJFT0HF5Q061PH01DE5@0EEEE04e=C@0204a<C00g06QXJ025QHD0GEeM
06YZJP10@400IVIV05IFEP10@400ATI60410@01UIFD0OWin05aLG01EEED0LG5a06a/K01GEeL0PH61
05EEE@1:BTX0BDU905MGE`19BDT0>CTi05YJFP1IFET0EEEE06QXJ01VIVH0Lg=c04U9B@1BDU80FUYJ
07EeM@2;Rh/0EUIF04A4A01LG5`0D51@05UIF@1>CTh0<S8b04M7A`1DE5@0BDU904M7A`19BDT0B4Q8
061PH01ADE40GEeM0451@@1LG5`0?Shn04M7A`0204Q8B00F04e=C@1<C4`0@41005IFEP0j>SX0AdM7
055AD@17AdL0JFUY05QHF01@D500CDe=05]KF`17AdL0@T9204];B`1LG5`0F5QH06m_K`1<C4`0Fe]K
06UYJ@kooooo08`0PX:204m?C`1QHF40IFEU05UIF@1KFe/0Bd];04I6AP13@d<0@4100492@P0k>c/0
D51@04Y:BP0Z:RX0;Rh^059BDP0b<S80CTi>03hn?P0a<C40D51@065QH@10@400?clo05]KF`1DE5@0
A4A406EUI@13@d<0FUYJ05ADE00a<C40D51@06IVIP17AdL0?Cdm038b<P14A4@0GUiN0492@P1DE5@0
NWYj07UiN@1OGel0EEEE055AD@1>CTh0KFe]08f=S@10@400CDe=05UIF@19BDT0CDe=06=SH`1VIVH0
GUiN05MGE`1>CTh0MGEe04i>CP1CDe<0HF5Q06MWI`1UIFD0=cLg07alO01FEUH0ADE507]kN`1hN7P0
De=C061PH00l?3`0ATI606ATI01lO7`0I6AT09:BTP1;Bd/0RHV906IVIP1KFe/0GUiN06i^KP2[Zj/0
K6a/07YjNP1EEED0;bl_04m?C`1:BTX0L71`0451@@1TI6@0KVi^06MWI`1=CDd0F5QH06UYJ@0k>c/0
J6QX07AdM01kNg/0?Cdm06][J`1HF5P0Bd];07QhN01JFUX0=SHf061PH01KFe/0De=C07AdM01UIFD0
EUIF05mOG`29RHT0Kfm_05YJFP0o?cl0EeMG03hn?P1^KVh0P82005aLG01cLg<0HF5Q05UIF@1SHf<0
OWin04e=C@1ADE40SHf=04E5A@1BDU80?Cdm0:>SX`kooooo02@0A4A40451@@10@400QHF508b<S01W
IfL0@D5102DU9@1PH600ADE507alO01BDU80820P01dM7@0C4a<0DE5A05]KF`1LG5`09RHV02<S8`0>
3Ph092@T0451@@0J6QX0Fe]K020P800G5aL09BDU028R8P0E5AD06ATI01dM7@1eMGD0?Cdm01<C4`0B
4Q8201@D500;02TY:@0o?cl0Bd];02h^;P1>CTh0HV9R04m?C`1^KVh0:2PX034a<@0G5aL00P0E5AD0
F@0N7Qh0De=C08b<S01PH600PH6103Ph>00i>CT0;2`/01TI6@0P8200<C4a02PX:010@400NWYj03lo
?`12@T80DU9B04Q8B00l?3`06QXJ01`L7016ATH0Rh^;09BDU00c<c<0MGEe0;Bd]00];Bd08R8R02/[
:`0K6a/07AdM05IFEP2n_[h0?Cdm02PX:01;Bd/09bLW04=3@`1dM7@0S8b<05eMG@22PX80;2`/030`
<01SHf<0MgMg07inOP2UYJD0S8b<06UYJ@1XJ6P05QHF030`<01kNg/0/;2`0<_;b`20P800BDU9020P
801oOgl0EUIF03Xj>P0J6QX071`L04m?C`0T92@0CTi>04m?C`2JVYX0@41006=SH`2FUYH0PX:20492
@P0J6QX0@D5108N7Q`1JFUX092@T01dM7@0/;2`0Shn?05ADE016ATH0<S8b01lO7`0^;Rh0EEEE01Co
oooo000Cooooo`1007]kN`1PH600Kfm_04i>CP1OGel0HF5Q04=3@`1ADE40GEeM0492@P1MGEd0A4A4
05QHF01JFUX0@D5106YZJP1BDU80BDU906EUI@1<C4`0=SHf06][J`1SHf<0<c<c06i^KP1=CDd0@D51
03`l?01CDe<0IfMW04A4A00o?cl0P820038b<P17AdL0O7al02l_;`1RHV80Jf][03De=@1GEeL0IfMW
03De=@1UIFD0D51@03hn?P1EEED0De=C059BDP1<C4`0G5aL05IFEP1@D500AdM705eMG@0i>CT0CDe=
07YjNP0j>SX0?3`l06YZJP1FEUH0CDe=08:2PP80DU9B04T0V9RH05=CD`1?Cdl0MWIf03Ph>01/K6`0
MGEe0451@@1]KFd0JFUY04e=C@1OGel0Hf=S03dm?@1MGEd0HV9R04i>CP1HF5P0EUIF08B4Q01=CDd0
Cdm?07YjNP1KFe/0<C4a06MWI`1QHF40B4Q806e]K@1MGEd0HF5Q05MGE`1<C4`0OGem055AD@1JFUX0
Lg=c04a<C01YJFT0>CTi069RHP1ZJVX0BDU906ATI01[Jf/0ADE505iNGP22PX80ATI605eMG@23Ph<0
CTi>05]KF`1nOWh0F5QH061PH01cLg<0CTi>06MWI`1]KFd0CTi>06QXJ01YJFT0BTY:05YJFP1EEED0
ATI6071`L01DE5@0Bd];07EeM@17AdL0C4a<00kooooo0540MWIf05UIF@1ZJVX0D51@065QH@1CDe<0
N7Qh06EUI@1RHV80BDU904i>CP0h>3P0DU9B03hn?P1fMWH0DU9B06EUI@1ZJVX0J6QX04U9B@1FEUH0
H61P05=CD`1RHV80DE5A04=3@`1MGEd0L71`059BDP1:BTX0DU9B06YZJP2;Rh/0HF5Q079bLP1UIFD0
DU9B06EUI@1LG5`0BTY:04];B`1>CTh0DU9B04E5A@1>CTh0EEEE04m?C`1:BTX0CDe=05ADE01HF5P0
Hf=S07moO`1RHV80JFUY07UiN@1YJFT0EEEE06=SH`15ADD0F5QH069RHP2`/;00ATI607QhN01^KVh0
Ng]k05ADE00];Bd0=SHf04I6AP1?Cdl0SXj>05UIF@1JFUX0?Shn04Y:BP1SHf<0LW9b05eMG@1BDU80
0P1ADE40>@1=CDd0>SXj04I6AP0n?Sh0EeMG05]KF`1SHf<0M7Ad05aLG01FEUH0Fe]K05UIF@1bLW80
DE5A03lo?`14A4@0DU9B04Y:BP17AdL0;Rh^03hn?P1>CTh0ADE504I6AP17AdL0@d=306e]K@1CDe<0
DE5A038b<P1=CDd0@41004E5A@0l?3`0BDU904Y:BP1QHF40<c<c04Q8B017AdL0FUYJ05ADE01:BTX0
@d=306IVIP1LG5`0EUIF05QHF017AdL0>SXj0451@@1CDe<0B4Q808>3P`1NGUh0FUYJ06YZJP0>oooo
o`0408>3P`1CDe<0G5aL05mOG`80A4A401H0CTi>04=3@`10@400AdM704U9B@10@400@d=304m?C`0[
:b/0:BTY04E5A@0m?Cd0C4a<04Q8B00a<C40@D5104e=C@0l?3`0>3Ph04e=C@1MGEd0<C4a0P18B4P0
>@1HF5P0?3`l02/[:`1=CDd0HF5Q03/k>`0n?Sh0;Bd]04I6AP1;Bd/0?Shn04a<C01_Kfl0KFe]06][
J`1OGel0De=C04I6AP1jNWX0QhN704M7A`16ATH0Hf=S0451@@1<C4`0LG5a06YZJP1`L700G5aL04m?
C`1]KFd0HV9R055AD@1TI6@0KFe]07IfMP11@D40MgMg04];B`16ATH0N7Qh07UiN@1IFET0EEEE03dm
?@1;Bd/0Jf][07]kN`1OGel0SHf=06IVIP1cLg<0H61P07AdM01eMGD0JVYZ0:NWY`0208B4Q00D06e]
K@0m?Cd0I6AT04m?C`1eMGD0DU9B05IFEP1[Jf/0JFUY05IFEP1BDU80QXJ604Q8B01[Jf/0MgMg08J6
QP14A4@0H61P079bLP19BDT206ATI00M04M7A`1NGUh0DE5A061PH01]KFd0Cdm?05EEE@1DE5@0Lg=c
05EEE@1LG5`0<c<c05EEE@11@D40IFEU05mOG`1:BTX0N7Qh04i>CP1DE5@0Hf=S071`L014A4@0GUiN
09BDU00f=SH0BTY:03dm?@2LW9`03_ooool08P0f=SH0:RXZ01XJ6P0l?3`0ATI605eMG@1UIFD08b<S
07MgM`2b/[80a/K606EUI@0c<c<07AdM01/K6`0l?3`0820P05YJFP0X:2P04a<C00l?3`0F5QH0K6a/
01XJ6P1lO7`0=cLg014A4@0C4a<051@D01LG5`0S8b<071`L06a/K00o?cl2018B4P0=01@D500E5AD0
4A4A010@400n?Sh07AdM03Lg=`0k>c/0[jn_02l_;`0N7Qh0;Bd]01HF5P0301DE5@0/03lo?`1MGEd0
Bd];06i^KP0^;Rh09BDU04];B`1SHf<0HV9R02<S8`0T92@08b<S069RHP0K6a/09bLW05YJFP0`<300
H61P02d];@0M7Ad07QhN07EeM@1;Bd/09BDU06i^KP1fMWH0A4A406MWI`1kNg/0N7Qh02TY:@1QHF40
h>3P06MWI`1KFe/0K6a/05aLG00^;Rh0J6QX065QH@1GEeL0WinO04U9B@1=CDd201lO7`0802@T901K
Fe/0OGem06UYJ@31`L40IfMW051@D01MGEd209JFUP0J06a/K00d=3@0FUYJ08R8R01RHV80C4a<02HV
9P0W9bL0ATI602@T901<C4`0;2`/07]kN`0L71`0;Bd]06m_K`1EEED0GemO03dm?@1OGel0@d=3055A
D@0N7Qh0?Cdm07UiN@2d];@203Ph>00404e=C@0V9RH0820P04I6AQCooooo000Cooooo`2;07alO01b
LW80GemO05mOG`1IFET0EEEE06=SH`1=CDd0IVIV069RHP13@d<0ADE506a/K00n?Sh0AdM705iNGP0n
?Sh0@d=304a<C01FEUH0?clo04m?C`1IFET0?clo055AD@1TI6@0E5AD0410@01PH600GEeM04E5A@1F
EUH0Hf=S03dm?@1[Jf/0Hf=S05=CD`1GEeL0DU9B04=3@`1?Cdl0CDe=051@D01ADE40Fe]K05MGE`0g
=cL0IFEU05ADE00X:2P0HV9R05aLG00f=SH0HF5Q05=CD`10@400IfMW04i>CP15ADD0GemO05UIF@1:
BTX0FEUI05mOG`1IFET0Cdm?075aL@1YJFT0CTi>069RHP1LG5`0Jf][05UIF@1YJFT0Bd];05]KF`1d
M7@0EUIF06QXJ01TI6@0D51@05mOG`1RHV80A4A405iNGP1jNWX0@41005YJFP1XJ6P0@d=304i>CP1H
F5P0D51@05=CD`1DE5@0H61P05UIF@1@D500GemO06MWI`1TI6@0GUiN06IVIP1eMGD0IfMW04U9B@1n
OWh0HV9R05]KF`1UIFD0GUiN06m_K`1RHV80I6AT06QXJ01DE5@0JVYZ06ATI013@d<0IFEU06ATI00g
=cL0Lg=c04e=C@1?Cdl0MgMg04a<C01ADE40HV9R04I6AP1IFET0JVYZ04I6AP1>CTh0F5QH04A4A01B
DU80JFUY03lo?`0>ooooo`03079bLP1JFUX0IFEU0080FEUI06@0HF5Q07UiN@2;Rh/0Kfm_04a<C013
@d<0C4a<065QH@1;Bd/0Hf=S05EEE@1[Jf/0HF5Q05=CD`19BDT0EUIF05=CD`1@D500EeMG051@D013
@d<0?Shn04i>CP0c<c<0@4100492@P1HF5P0OWin06IVIP1SHf<0GUiN05EEE@1<C4`0EUIF03lo?`11
@D40ATI604Q8B00h>3P0?Shn03Xj>P0o?cl0?Shn03Xj>P1:BTX0ADE5051@D01TI6@0F5QH05YJFP1V
IVH0EEEE06=SH`1TI6@0CDe=059BDP1WIfL0YZJV061PH01]KFd0HV9R08J6QP1NGUh0DU9B05ADE01Q
HF40MgMg09FEU@1VIVH0JVYZ061PH01IFET0F5QH07]kN`1KFe/0CTi>04];B`1?Cdl0DU9B04E5A@1U
IFD0DU9B04m?C`1MGEd0K6a/07UiN@15ADD0Cdm?05iNGP2HV9P0MWIf06=SH`13@d<0GUiN069RHP1C
De<0DU9B04I6AP1KFe/0EEEE0P1:BTX08@11@D40@T9205UIF@1KFe/0GemO059BDP1EEED0?Shn04M7
A`12@T80AdM704];B`1?Cdl0B4Q805ADE00g=cL0Hf=S05eMG@1NGUh0HV9R05]KF`1`L700JVYZ05]K
F`1EEED0DE5A04e=C@1NGUh0Fe]K07MgM`1NGUh0Hf=S07emO@0>ooooo`1V07]kN`1<C4`0GEeM05mO
G`1:BTX0Cdm?04M7A`0o?cl0ADE504A4A01;Bd/0@d=30492@P11@D408b<S02d];@1>CTh0<C4a061P
H017AdL0;bl_059BDP1GEeL0@4100492@P1YJFT0HF5Q03/k>`1_Kfl0Cdm?04Y:BP1>CTh0;Bd]055A
D@1fMWH0ADE504=3@`0f=SH0BDU905iNGP15ADD0De=C08>3P`21PH40Kfm_04m?C`1<C4`0EEEE07Mg
M`27QhL0@T9204m?C`1WIfL0A4A404i>CP1YJFT0KFe]0861P@1GEeL0E5AD071`L01IFET0D51@06YZ
JP1XJ6P0H61P03Ti>@25QHD0BTY:059BDP23Ph<0LW9b05aLG01`L700AdM7055AD@1RHV80QXJ605eM
G@1eMGD0QXJ6079bLP1oOgl0FEUI06m_K`1fMWH0PX:207]kN`1iNGT0D51@04M7A`1FEUH0DU9B07Ad
M017AdL0CTi>079bLP1;Bd/0De=C059BDP1lO7`0Cdm?0P1YJFT06P1hN7P0Bd];05aLG01QHF40F5QH
08R8R01DE5@0>CTi05UIF@1GEeL0G5aL071`L01EEED0EUIF06YZJP22PX80G5aL04m?C`13@d<0Bd];
03Lg=`1KFe/0EEEE05MGE`1SHf<0?Cdm0P1bLW80201eMGD0BDU905UIF@2KVi/0BDU904I6AP0h>3P0
QHF53_ooool08P1?Cdl0:2PX01/K6`0b<S80GUiN0;:b/P2DU9@0Lg=c0;Zj^P38b<P0LW9b04U9B@0e
=CD0ADE5055AD@1HF5P05QHF065QH@0i>CT06a/K02DU9@0N7Qh0X:2P09nOW`2/[:`0VIVI07]kN`1N
GUh0E5AD04i>CP23Ph<0JVYZ09fMW@0f=SH2018B4P0<01@D500B4Q80410@010@4011@D408b<S0451
@@1ZJVX0[Jf]03Lg=`0Z:RX0;Rh^0`0E5AD0C00T92@0Ph>305YJFP0S8b<0BTY:04U9B@12@T80EUIF
08Z:RP2^[Zh0DU9B05iNGP1IFET0P82004i>CP0V9RH08R8R03lo?`1=CDd0HF5Q06a/K01KFe/0NGUi
079bLP1EEED0GemO04U9B@0R8R80>CTi0820P01nOWh0Ogmo07EeM@2KVi/0Z:RX0:FUY@1kNg/0DE5A
059BDP2GUiL0S8b<03De=@2KVi/0DU9B038b<P0J6QX0:b/[04=3@`1_Kfl0[Zj^08B4Q02e]KD0`l?3
0:VYZ@1`L700K6a/05=CD`1XJ6P0CTi>0451@@1MGEd0UIFE08Z:RP27QhL0UYJF08F5Q@1IFET0A4A4
071`L01CDe<09bLW04A4A01VIVH08B4Q0492@P1UIFD204];B`0;0;^k^`2HV9P0S8b<07inOP1^KVh0
ADE503Ph>01fMWH0>c/k04E5A@1/K6`05?ooool001?ooooo00`0Lg=c06UYJ@1CDe<0MWIf04a<C01j
NWX0LG5a030`<01ZJVX0EEEE03dm?@1DE5@204e=C@0A04i>CP1OGel0IfMW05YJFP1CDe<0Jf][03`l
?01;Bd/0JVYZ02h^;P1?Cdl0LW9b03Hf=P15ADD0EeMG0410@01TI6@00`1>CTh09P1KFe/0G5aL06AT
I01?Cdl0KFe]04a<C00i>CT0KFe]04=3@`0h>3P0I6AT05=CD`0d=3@0FUYJ051@D00k>c/0MGEe03hn
?P16ATH0EEEE04e=C@1MGEd0De=C055AD@1FEUH0DE5A04e=C@1PH600BTY:04U9B@1cLg<0?Shn055A
D@1TI6@0;2`/06MWI`1LG5`0D51@0P1[Jf/07`1KFe/0O7al065QH@1MGEd0LW9b05YJFP1<C4`0K6a/
05aLG013@d<0MWIf069RHP1LG5`0JVYZ061PH01GEeL0ADE505EEE@1^KVh0<S8b05mOG`25QHD0<S8b
05IFEP1JFUX0AdM706MWI`0e=CD0GEeM07UiN@1ADE400P1/K6`0801ADE40Fe]K059BDP1mOGd0FUYJ
059BDP24Q8@0G5aL069RHP1nOWh0B4Q805QHF020P800EeMG04];B`1XJ6P0ADE505MGE`1WIfL0<c<c
059BDP1cLg<0;Bd]069RHP1_Kfl0>c/k069RHP1UIFD0:BTY05YJFP1HF5P0;Bd]3_ooool0;01`L700
FEUI06YZJP1QHF40M7Ad061PH020P800T92@071`L01<C4`0B4Q804Y:BP1YJFT0GemO07EeM@1FEUH0
LW9b061PH01LG5`0ATI604m?C`17AdL0Cdm?04e=C@1>CTh0A4A40451@@1GEeL0>SXj05UIF@19BDT0
GemO07IfMP1OGel0IfMW05eMG@17AdL0CDe=04Y:BP14A4@0?clo04=3@`0o?cl0?3`l0P0h>3P03P0b
<S80<c<c038b<P1<C4`0ATI604U9B@1bLW80DE5A05=CD`1ZJVX0GemO05ADE01MGEd0AdM70P1FEUH0
>027QhL0EeMG065QH@1HF5P0Q8B405iNGP10@400F5QH06MWI`1bLW80VYZJ07MgM`1ZJVX0LW9b06MW
I`1[Jf/0EeMG05eMG@1BDU80FUYJ059BDP1NGUh0Bd];05MGE`1@D500Fe]K05YJFP1ZJVX0LW9b04e=
C@1CDe<0RHV909>CT`1fMWH0DU9B04=3@`1EEED0HF5Q079bLP1;Bd/0?3`l055AD@19BDT0D51@05AD
E01:BTX0B4Q805YJFP1DE5@0I6AT04m?C`1=CDd0@41004];B`18B4P0BTY:0P18B4P0500o?cl0H61P
0451@@1/K6`0Cdm?04U9B@1`L700HF5Q08J6QP1kNg/0FUYJ06IVIP1IFET0EEEE04i>CP1cLg<0Ti>C
06YZJP1IFET0F5QH3_ooool0:`1aLG40Bd];059BDP1EEED0ADE504=3@`1YJFT0;bl_04e=C@0l?3`0
@d=305=CD`0];Bd0=cLg02DU9@0a<C40FEUI02TY:@15ADD0>3Ph020P801@D500@d=304I6AP0j>SX0
DE5A05eMG@0h>3P0M7Ad04Y:BP1ADE40?Shn030`<01IFET0G5aL04M7A`16ATH0=3@d061PH01GEeL0
CTi>05mOG`1nOWh00P1KFe/0?01=CDd0C4a<04Q8B01cLg<0RHV90410@016ATH0J6QX04=3@`1LG5`0
MWIf07QhN02>SXh0DU9B06EUI@1ZJVX0De=C05MGE`1ZJVX0N7Qh075aL@15ADD0O7al05=CD`1XJ6P0
RXZ:07YjNP1JFUX0G5aL055AD@1GEeL0JVYZ08R8R01CDe<0IFEU0861P@1YJFT0L71`06EUI@1kNg/0
Kfm_0861P@22PX80MgMg059BDP0_;bl0EUIF05aLG01_Kfl0DE5A06i^KP1aLG40FEUI05QHF01FEUH0
QHF505iNGP1aLG40J6QX07inOP80Bd];00d0L71`051@D01aLG40IFEU04A4A01JFUX0DU9B06a/K01f
MWH0@d=306EUI@1aLG40K6a/0080GUiN0180>3Ph059BDP13@d<0Hf=S05UIF@1EEED0MWIf03Ph>01Y
JFT0GEeM06a/K012@T80Hf=S0:j^[P0g=cL0BTY:04Q8B01jNWX>ooooo`0[07AdM011@D4092@T02d]
;@1@D500BTY:05]KF`1RHV80C4a<02l_;`22PX80P82006YZJP1MGEd0:BTY06e]K@1TI6@0[Jf]08j>
SP0V9RH0>SXj09RHV026QXH0/;2`04E5A@0b<S80:BTY02@T900W9bL0>SXj061PH01VIVH0/k>c08f=
S@0k>c/09BDU03`l?00e=CD04A4A018B4P1;Bd/0PH6109nOW`020<30`01O0820P017AdL0MgMg04m?
C`1;Bd/0F5QH07UiN@2HV9P0UYJF055AD@1<C4`071`L01LG5`0N7Qh0K6a/09>CT`2f][H0RXZ:09^K
V`0P8200?Cdm05YJFP1:BTX0FEUI09^KV`27QhL0G5aL09bLW01bLW80Xj>S08J6QP1TI6@0C4a<01LG
5`0P8200CTi>034a<@1cLg<0ATI608f=S@2IVIT0SXj>055AD@1JFUX0E5AD0<71`@1_Kfl0HF5Q08J6
QP14A4@08B4Q01TI6@0a<C40FEUI08R8R024Q8@0O7al0;Bd]03Cdm<0`l?307MgM`1VIVH0>c/k01lO
7`0G5aL07AdM079bLP26QXH0RXZ:04U9B@25QHD0]KFe06MWI`10@400V9RH04a<C00o?cl0GUiN05mO
G`0J6QX06a/K05EEE@1jNWX0Z:RX06EUI@0Z:RX07alO024Q8@1?Cdl0Kfm_08n?S`2GUiL0EEEE02d]
;@0V9RH05?ooool001?ooooo04`0MWIf05mOG`1/K6`0ATI605eMG@1;Bd/0H61P05iNGP1JFUX0Lg=c
04e=C@1<C4`0N7Qh04a<C01ADE40P820051@D01?Cdl0GEeM03Xj>P0m?Cd0CDe=04i>CP0c<c<0ADE5
04=3@`1PH600;Bd]04Q8B01hN7P0>CTi04];B`1aLG40:RXZ055AD@1oOgl0<C4a04m?C`1GEeL0BDU9
07MgM`0n?Sh0D51@04e=C@19BDT0GUiN04E5A@19BDT0NWYj03Lg=`16ATH0Hf=S03De=@18B4P0KFe]
03Ti>@1IFET0JFUY03Ti>@1>CTh0O7al0492@P1@D500LG5a0492@P1TI6@0H61P03@d=01=CDd0JVYZ
055AD@1HF5P0Bd];065QH@1_Kfl0?Shn0P1LG5`0?@1<C4`0IVIV05YJFP1KFe/0Ng]k05ADE01;Bd/0
QhN703hn?P1@D500Ng]k03Lg=`13@d<0IFEU05ADE01EEED0FEUI055AD@1KFe/0BDU905MGE`1kNg/0
B4Q807=cL`1=CDd0BDU9071`L011@D40QXJ606][J`0o?cl0L71`06YZJP1KFe/0NWYj07MgM`1UIFD0
Jf][07alO01WIfL0ATI606ATI01UIFD0ADE5065QH@1HF5P0Cdm?05iNGP1=CDd0A4A406MWI`17AdL0
@D5105iNGP1<C4`0CDe=03lo?`1GEeL0@T9205]KF`1HF5P03_ooool0501/K6`0BTY:06UYJ@1LG5`0
OWin05mOG`2=SHd0/k>c07QhN01GEeL0ATI604e=C@1WIfL0I6AT06a/K01]KFd0MgMg06i^KP1GEeL0
BTY:0P1=CDd05@1?Cdl0BTY:0410@015ADD0<C4a05]KF`0e=CD0B4Q803lo?`1IFET0Jf][05EEE@1Q
HF40GEeM04Q8B00n?Sh0De=C03/k>`0o?cl0CTi>03De=@0203hn?P0Y04I6AP12@T80AdM703`l?01F
EUH0CTi>04a<C020P800EEEE05]KF`1JFUX0Jf][05YJFP1QHF40C4a<04Y:BP1XJ6P0U9BD06ATI01A
DE40BDU907YjNP1=CDd0Cdm?06e]K@1KFe/0HV9R08Z:RP1<C4`0D51@05EEE@1GEeL0JFUY07QhN022
PX80F5QH06MWI`1FEUH0Bd];05EEE@1KFe/00P1BDU80101JFUX0MGEe07emO@1CDe<206EUI@0905=C
D`1IFET0De=C04=3@`11@D40CTi>06ATI01=CDd0>CTi0080AdM700L0ADE504=3@`0n?Sh0@d=306EU
I@1JFUX0LG5a0080Cdm?01X0>c/k04I6AP0m?Cd0ATI604Y:BP1EEED0>c/k05EEE@1=CDd0GUiN05YJ
FP1:BTX0OWin05eMG@20P800I6AT04a<C01VIVH0@D5103Hf=P10@400H61P06UYJ@1NGUh0GEeM06MW
I`kooooo08`0PH61051@D01RHV80De=C04a<C01SHf<0EUIF04A4A01UIFD0ADE504i>CP1BDU80AdM7
03dm?@0Q8B40B4Q8069RHP0l?3`0KFe]03hn?P0`<300GemO05EEE@1>CTh0?Shn06IVIP1EEED0>3Ph
06i^KP17AdL0A4A4059BDP0_;bl0GEeM06YZJP17AdL0BDU903Ti>@1?Cdl0F5QH04];B`1?Cdl0QhN7
069RHP1UIFD0CDe=03dm?@1HF5P0EUIF08f=S@1:BTX0De=C05aLG019BDT0De=C06i^KP1`L700IfMW
05EEE@1IFET0IFEU05]KF`1SHf<0JFUY06IVIP1EEED0?Cdm07UiN@15ADD0F5QH08:2PP1ZJVX0AdM7
05mOG`0m?Cd0BDU905MGE`1aLG40IfMW06e]K@1hN7P0KVi^08N7Q`1FEUH0Jf][07EeM@28R8P0Shn?
08V9R@1@D500<C4a04i>CP1;Bd/0HF5Q04a<C01?Cdl0M7Ad05eMG@1:BTX0I6AT0:2PX01cLg<0M7Ad
07]kN`2NWYh0?clo06=SH`1IFET0AdM706IVIP1TI6@0<S8b06QXJ01NGUh0FEUI06MWI`1HF5P0H61P
07IfMP20P800L71`04Q8B00h>3P0GUiN03Ph>01>CTh0?clo061PH01aLG40=cLg09JFUP12@T80J6QX
05EEE@1UIFD0Vi^K04I6AP1=CDd0?3`l08F5Q@kooooo00H0MgMg06a/K01VIVH0M7Ad04Q8B00S8b<2
01`L700A059BDP0X:2P0:2PX092@T03[jn/0i>CT0:BTY01oOgl0`/;20492@P0b<S80RHV90=SHf02i
^KT0C4a<069RHP18B4P00P0N7Qh0:@0J6QX05ADE01PH600j>SX0MGEe0:ZZZP2`/;00OWin075aL@2a
/K40Kfm_04=3@`14A4@0NWYj0:b/[03?cll0cLg=04Y:BP1TI6@0OWin09jNWP1RHV80Bd];04E5A@1e
MGD0Ph>305YJFP1RHV80TI6A028R8P0I6AT0;bl_065QH@1cLg<0BDU90:f][@1IFET0F5QH01hN7P0e
=CD0RXZ:0080][Jf04H0NGUi059BDP1NGUh0]KFe05aLG00_;bl0KFe]09>CT`1OGel0IFEU02<S8`0N
7Qh0IVIV04e=C@1@D500?Cdm04m?C`1aLG40EUIF0<_;b`2BTY80NWYj07inOP3:b/X0UYJF06a/K01j
NWX0Lg=c07QhN01IFET07alO02@T901SHf<0Xj>S07inOP2c/k<0RXZ:03lo?`0M7Ad06ATI04Y:BP1F
EUH0@D5103Hf=P1_Kfl0JVYZ0:VYZ@2j^[X0bLW90:ZZZP13@d<0MWIf06e]K@1/K6`0?Cdm01XJ6P0Z
:RX0Ogmo07YjNP0R8R807alO02PX:00N7Qh09RHV06ATI02@T900Ng]k01dM7@0Q8B40De=C5?ooool0
01?ooooo00@0IVIV06i^KP1EEED0JFUY0P1ADE40301kNg/0<C4a05IFEP1iNGT0:BTY05mOG`1QHF40
>CTi06EUI@1SHf<0BTY:04M7A`80Cdm?07L0?Shn055AD@1KFe/0>c/k051@D01QHF40>CTi03Ph>01Q
HF40BDU903hn?P1KFe/0AdM704A4A01OGel0EUIF05UIF@1@D500C4a<05IFEP16ATH0HF5Q05iNGP0b
<S80GemO04m?C`0P8200E5AD051@D00j>SX0JFUY03/k>`16ATH0E5AD04U9B@1KFe/0EUIF05ADE01@
D500ADE504i>CP1IFET0>SXj055AD@21PH40=SHf05IFEP1QHF40;bl_06QXJ01ZJVX0?clo071`L01i
NGT0DU9B07moO`1OGel0CDe=0820P01WIfL0@d=306YZJP1OGel0BTY:065QH@1YJFT0CDe=05ADE01H
F5P0GUiN03dm?@1ADE40NWYj0451@@1WIfL0N7Qh0451@@1]KFd0CDe=05IFEP1SHf<0=CDe06UYJ@1[
Jf/0BTY:07=cL`1lO7`0G5aL06m_K`1@D500HV9R06IVIP1;Bd/0LG5a061PH011@D40RXZ:051@D00i
>CT0Ti>C05IFEP15ADD0MGEe04Y:BP1BDU80S8b<03dm?@17AdL0RXZ:03Xj>P1DE5@0KFe]03lo?`1J
FUX0EUIF03<c<`1?Cdl0E5AD03Xj>P0>ooooo`04075aL@1GEeL0Hf=S05IFEP80HV9R00H0PX:206MW
I`1QHF40ADE503`l?01<C4`2065QH@0m07AdM01HF5P0I6AT05MGE`1?Cdl0?clo04e=C@1;Bd/0Cdm?
055AD@1:BTX0@41003Xj>P1KFe/0@41004a<C01@D500GemO07alO01CDe<0GemO05QHF010@400@T92
04];B`19BDT0?Cdm04I6AP0l?3`0?Cdm0410@015ADD0?clo0492@P1?Cdl0F5QH04M7A`1EEED0Lg=c
069RHP1NGUh0KVi^061PH01QHF40LW9b04A4A01ADE40FEUI09jNWP1GEeL0CTi>055AD@1kNg/0GEeM
05ADE01iNGT0Jf][06m_K`2>SXh0CTi>05ADE00204=3@`0O04a<C01VIVH0DU9B04M7A`1JFUX0AdM7
04m?C`16ATH0Cdm?04M7A`15ADD0EUIF06IVIP1UIFD0DU9B04M7A`1YJFT0CTi>05MGE`1;Bd/0@410
0492@P1GEeL0JVYZ061PH019BDT0Bd];04Q8B01?Cdl0FEUI04E5A@0204U9B@0N05eMG@1HF5P0EEEE
05ADE015ADD0FEUI04I6AP1<C4`0ATI605=CD`0h>3P0C4a<03De=@1@D500Fe]K05YJFP1VIVH0Hf=S
08J6QP1ZJVX0F5QH06=SH`1:BTX0ATI604=3@`19BDT0IVIV05YJFP1KFe/0Jf][3_ooool06@1QHF40
BDU904m?C`1KFe/0C4a<04M7A`1@D500?clo04=3@`11@D40AdM705]KF`17AdL0@T9203`l?00f=SH0
@d=302`/;01HF5P0?Cdm02`/;01DE5@0ADE5055AD@1?Cdl00P1KFe/02017AdL0H61P051@D01?Cdl0
ADE502DU9@1CDe<0CDe=0P18B4P05`0b<S80GEeM05aLG013@d<0JVYZ09JFUP1;Bd/0>CTi05IFEP1D
E5@0CDe=06e]K@2OWil0B4Q8051@D01IFET0Cdm?04e=C@1SHf<0GemO05UIF@1RHV80DE5A0080H61P
0P1[Jf/05@1ZJVX0B4Q804i>CP1kNg/0@T9205aLG01fMWH0JVYZ04Y:BP1QHF40?Cdm05eMG@1@D500
WYjN05MGE`1FEUH0S8b<07AdM01fMWH0KFe]08N7Q`0206MWI`0:08F5Q@22PX80Cdm?02/[:`13@d<0
KVi^04i>CP14A4@0JVYZ07emO@80Fe]K02T0CTi>07emO@1TI6@0MWIf05iNGP1/K6`0EeMG04I6AP1Y
JFT0@41006EUI@1BDU80<c<c05QHF019BDT0L71`07inOP11@D40HV9R07QhN01`L700J6QX07=cL`0j
>SX0B4Q802TY:@1nOWh0CDe=0451@@1eMGD0>CTi07=cL`1FEUH0Ng]k03@d=01XJ6P0UYJF020P801>
CTh0BDU9075aL@0>ooooo`0508j>SP2GUiL0ZJVY0:FUY@26QXH00P0i>CT05P15ADD0>3Ph0492@P18
B4P0R8R80=OGe`29RHT0;bl_02TY:@1BDU80@D5101DE5@1KFe/0c/k>0:j^[P1EEED0:b/[0820P01I
FET0E5AD04E5A@0/;2`202DU9@0603Xj>P2m_Kd0^KVi09FEU@1MGEd0CTi>0P1^KVh03P0e=CD0@T92
07emO@2c/k<0ZZZZ065QH@1eMGD0PX:2075aL@0O7al07AdM020P801/K6`0_Kfm0P2BTY80BP2_[jl0
IFEU061PH01WIfL0LW9b05eMG@0T92@0LG5a069RHP28R8P0Ogmo06IVIP1bLW80[Jf]0;Zj^P22PX80
I6AT04A4A021PH40SXj>04i>CP2?Shl0TI6A061PH01dM7@0?Shn03Hf=P1RHV80;2`/01LG5`0F5QH0
9RHV06UYJ@1hN7P0Bd];02/[:`1lO7`0UYJF06MWI`14A4@0;Rh^06EUI@2i^KT0Fe]K02PX:00U9BD0
ATI60861P@1dM7@0VYZJ0:f][@1[Jf/0=CDe04i>CP1/K6`0De=C03Ph>00K6a/05QHF038b<P1]KFd0
T92@09>CT`1dM7@0OWin0:n_[`1?Cdl08R8R02LW9`1]KFd0HF5Q06EUI@2VYZH0JVYZ0P0N7Qh02@0[
:b/07QhN02@T902DU9@0Jf][03Ti>@0n?Sh0<30`06a/K00Dooooo`004oooool0;P1jNWX0G5aL06EU
I@1CDe<0EUIF05MGE`1HF5P0EEEE061PH01KFe/0C4a<03hn?P1LG5`0DU9B04=3@`1QHF40L71`03Lg
=`1EEED0GemO030`<01MGEd0D51@03<c<`1MGEd0FEUI04];B`15ADD0DU9B071`L012@T80BTY:06=S
H`0g=cL0EEEE07UiN@18B4P0BTY:06i^KP0j>SX0BTY:05eMG@18B4P0A4A405aLG01?Cdl204=3@`07
05IFEP1;Bd/0FEUI051@D01?Cdl0?3`l05]KF`0204e=C@0?05UIF@17AdL0EUIF061PH01:BTX0Cdm?
06e]K@1:BTX0ADE5079bLP1@D500ATI606e]K@1KFe/0F5QH0080HF5Q0440IVIV06i^KP1LG5`0F5QH
04m?C`1TI6@0EeMG05QHF01WIfL0Fe]K04];B`1dM7@0De=C04a<C01OGel0Cdm?03dm?@1BDU80H61P
0451@@1KFe/0J6QX05=CD`1JFUX0G5aL06MWI`1FEUH0CTi>065QH@1WIfL0GEeM04U9B@1`L700GEeM
04e=C@1VIVH0GUiN06ATI01PH600PX:205mOG`1FEUH0T92@05IFEP18B4P0Ogmo05mOG`17AdL0NWYj
0451@@1BDU80HF5Q04e=C@19BDT0De=C055AD@1GEeL0KFe]04A4A01QHF40E5AD04Q8B01HF5P0DU9B
051@D00>ooooo`040861P@1BDU80IVIV05IFEP80J6QX01L0MgMg096AT@1`L700Bd];04m?C`1BDU80
F5QH069RHP1YJFT0I6AT05mOG`1NGUh0EUIF03Ti>@1@D500BTY:04m?C`16ATH0A4A403Lg=`0d=3@0
EUIF034a<@0204E5A@0Z051@D01jNWX0DE5A051@D01BDU80@d=304M7A`1<C4`0>c/k03lo?`12@T80
=CDe03Hf=P0h>3P0?3`l03Ph>00m?Cd0?clo04Y:BP10@400@T9206EUI@1<C4`0CDe=05=CD`1FEUH0
ATI6061PH016ATH0@T9205QHF02<S8`0JFUY04m?C`14A4@0IFEU04];B`10@400JVYZ051@D01SHf<0
SXj>0P1:BTX05@1TI6@0CTi>05aLG01dM7@0GEeM061PH01UIFD0@d=304e=C@19BDT0BTY:04a<C019
BDT0Fe]K075aL@1]KFd0CDe=04i>CP1CDe<0FEUI05QHF00203dm?@0B0492@P1CDe<0B4Q804m?C`12
@T80ADE50492@P1?Cdl0De=C03Ti>@15ADD0E5AD05EEE@1LG5`0B4Q8055AD@0o?cl0<c<c0P13@d<0
5P19BDT0EEEE04A4A015ADD0@D5104Y:BP1>CTh0Bd];065QH@1NGUh0IfMW06ATI01EEED0G5aL059B
DP19BDT0CDe=051@D01TI6@0E5AD061PH01TI6@>ooooo`2<07AdM01=CDd0DU9B05IFEP1=CDd0H61P
04];B`0n?Sh0ADE503Ti>@1>CTh0Bd];05YJFP0k>c/0>3Ph04Q8B012@T80;Rh^06UYJ@0g=cL0;2`/
03lo?`1GEeL0>c/k03dm?@1YJFT0FUYJ04=3@`1/K6`0BTY:03Ti>@1HF5P0=3@d05QHF01]KFd0D51@
05=CD`12@T80Hf=S05=CD`1BDU80GemO07inOP1`L700HV9R04Q8B00h>3P0GEeM06i^KP2=SHd0Cdm?
04M7A`1FEUH0?3`l04i>CP1SHf<0H61P069RHP1>CTh0E5AD075aL@1@D500MGEe05EEE@1VIVH0H61P
03Hf=P1mOGd0@T9205EEE@2=SHd0JVYZ04=3@`1OGel0AdM704Y:BP1EEED0RHV906][J`1=CDd0RXZ:
06EUI@2FUYH0C4a<07=cL`1gMgL0HV9R08V9R@1NGUh0EUIF03/k>`1@D500EEEE06MWI`1;Bd/0CTi>
07emO@1JFUX0CTi>05]KF`2<S8`0NWYj05mOG`1bLW80U9BD04Y:BP1SHf<0J6QX04e=C@1nOWh0G5aL
03Lg=`1/K6`0Fe]K06e]K@1^KVh0MGEe05aLG01_Kfl0Ph>306ATI01OGel0?Shn05IFEP13@d<0L71`
04m?C`1JFUX0KVi^03De=@20P800D51@09JFUP12@T80IfMW0861P@0^;Rh0@41004E5A@1bLW8>oooo
o`0H059BDP24Q8@0V9RH0:2PX02g]kL0KVi^03@d=00R8R809RHV02DU9@0c<c<0UIFE0<71`@23Ph<0
GUiN04=3@`0a<C40JFUY03De=@1[Jf/0YjNW0<?3``1:BTX0@D51100B4Q82010@4004018B4P0U9BD0
Cdm?0820P0806a/K01H09bLW02@T901HF5P0TY:B08b<S01>CTh071`L03Hf=P20P800?Cdm02XZ:P1O
Gel0Kfm_03Lg=`1iNGT0U9BD051@D01BDU80IFEU08b<S02VYZH0/[:b0P1KFe/05`1`L700H61P069R
HP2ZZZX0Shn?065QH@0`<300:RXZ09BDU02=SHd0LG5a06][J`16ATH0@d=3061PH01@D5009BDU05eM
G@1[Jf/0A4A408J6QP2`/;00BDU900@05ADE01T07QhN08>3P`2^[Zh0ZJVY0;jn_P2d];@0De=C04Q8
B013@d<0UiNG04A4A00m?Cd0:BTY065QH@15ADD05aLG02DU9@2f][H0a<C407IfMP1@D500HF5Q09>C
T`1;Bd/05QHF00805aLG01X0@41007MgM`1oOgl0MGEe07=cL`2VYZH0YJFU05]KF`1>CTh0ADE503Ph
>01FEUH0Vi^K0=3@d02SXj<0DE5A020P800h>3P0De=C08B4Q02d];@0P82006a/K02HV9P0S8b<055A
DACooooo000Cooooo`0507YjNP1VIVH0OWin04I6AP1YJFT00P1KFe/0J`1UIFD0BDU9069RHP12@T80
?Cdm06][J`1<C4`0DU9B05eMG@1BDU80D51@05=CD`0l?3`0Bd];05=CD`1EEED0E5AD04m?C`1SHf<0
HV9R030`<01EEED0JFUY02PX:01MGEd0Hf=S03Ph>01JFUX0C4a<04Q8B01OGel0BDU905aLG01:BTX0
@T92065QH@0j>SX0B4Q805mOG`0];Bd0CTi>065QH@0Z:RX0C4a<04Y:BP0U9BD0@D5105IFEP0[:b/0
GemO04Y:BP0a<C40H61P07AdM00e=CD0H61P07MgM`1>CTh0F5QH05aLG01MGEd0@D5105eMG@1LG5`0
I6AT05QHF01_Kfl0LG5a065QH@1eMGD0JFUY0451@@29RHT0FUYJ06ATI025QHD0Cdm?05mOG`26QXH0
ATI6059BDP1OGel0@D51055AD@1MGEd0I6AT061PH01SHf<0NGUi071`L00o?cl0IfMW07EeM@1?Cdl0
H61P051@D01^KVh0C4a<04E5A@23Ph<0De=C05aLG01`L700FEUI075aL@1oOgl0Q8B40080H61P01L0
OWin06YZJP13@d<0GemO06m_K`14A4@0GEeM069RHP1EEED0J6QX05UIF@0m?Cd0Kfm_05MGE`18B4P0
FEUI06m_K`0o?cl0Bd];06IVIP0k>c/0H61P04i>CP0>ooooo`1408F5Q@1NGUh0MWIf05ADE01=CDd0
CTi>05YJFP1lO7`0GEeM03lo?`17AdL0E5AD05=CD`15ADD0De=C05aLG01KFe/0IVIV04m?C`0m?Cd0
DE5A04Q8B01?Cdl0ATI604M7A`18B4P0=cLg05mOG`0Z:RX0ATI603dm?@13@d<0GEeM05UIF@1;Bd/0
CTi>0410@01=CDd0>3Ph03De=@13@d<0ADE502l_;`0k>c/0>SXj03dm?@12@T80?clo04i>CP1=CDd0
AdM704=3@`1TI6@0C4a<04A4A01?Cdl0B4Q804a<C01TI6@0EeMG0451@@1?Cdl0U9BD05aLG01UIFD0
GemO06MWI`1RHV8204E5A@0A03lo?`1YJFT0Ph>304Q8B019BDT0Lg=c0492@P1<C4`0GEeM05aLG01A
DE40KVi^0492@P1=CDd0EeMG05=CD`1BDU800P1:BTX0<`1dM7@0FEUI05UIF@1;Bd/0N7Qh07MgM`1M
GEd0C4a<0492@P11@D40GUiN05]KF`1FEUH0G5aL04=3@`10@400CTi>05UIF@0o?cl0Bd];065QH@1G
EeL0HF5Q04e=C@1>CTh0A4A404e=C@1<C4`0DE5A051@D01<C4`0AdM705ADE016ATH0=cLg04a<C01>
CTh0HF5Q06a/K01YJFT0L71`05]KF`1RHV80FEUI04M7A`1PH600L71`07inOP1GEeL0HV9R06][J`0>
ooooo`2<05eMG@1:BTX0HV9R065QH@1:BTX0AdM7061PH00f=SH0CDe=03`l?00g=cL0G5aL03`l?00j
>SX0@41002PX:01FEUH0=3@d05IFEP0n?Sh0:BTY06=SH`19BDT0@41005QHF019BDT0GEeM05EEE@1U
IFD0A4A404U9B@1JFUX0;2`/061PH01KFe/0@d=305MGE`0d=3@0Jf][051@D017AdL0N7Qh08F5Q@2<
S8`0DU9B051@D00h>3P0HF5Q07EeM@1mOGd0ADE504a<C01OGel0Cdm?055AD@22PX80KFe]06YZJP1I
FET0IfMW07AdM01BDU80MWIf05mOG`1cLg<0J6QX04I6AP26QXH0DE5A06=SH`2DU9@0IVIV04Y:BP1W
IfL0B4Q8065QH@1ZJVX0O7al06m_K`1HF5P0WinO061PH02MWId0FUYJ07MgM`20P800Fe]K07alO01W
IfL0EeMG0492@P1=CDd0HF5Q06=SH`1?Cdl0De=C07alO01PH600Bd];061PH020P800M7Ad07EeM@1Z
JVX0R8R80492@P1MGEd0M7Ad04m?C`1fMWH0De=C02d];@1OGel0G5aL061PH01OGel0FEUI075aL@1/
K6`0M7Ad05iNGP1JFUX0@D5104];B`0o?cl0JFUY04Y:BP0f=SH0LW9b04];B`21PH40F5QH08f=S@1D
E5@0Jf][06i^KP18B4P0<C4a04I6AP1^KVh>ooooo`0>034a<@1PH600NWYj03Ti>@1IFET0A4A400l?
3`0E5AD0410@014A4@0K6a/0H61P05QHF011@D4201XJ6P0903dm?@0k>c/0<30`055AD@2ATI40a<C4
05]KF`0O7al092@T00D03`l?00D03Ph>01@D500d=3@0LG5a01<C4`02010@400400h>3P0?3`l0De=C
05eMG@<04A4A03T0>SXj01XJ6P0g=cL0Jf][0:j^[P30`<00EEEE03hn?P0S8b<06QXJ03Ti>@1YJFT0
/;2`0;>c/`1?Cdl0>CTi05ADE01=CDd0Kfm_09JFUP2TY:@0NGUi03dm?@0[:b/0FEUI08V9R@1:BTX0
B4Q80492@P0F5QH07alO065QH@1KFe/0JFUY09RHV01EEED0Vi^K0;Ng]`16ATH07QhN01DE5@0D51@0
6QXJ03/k>`2?Shl0bLW906QXJ03=cLd0YjNW04=3@`0`<300;bl_061PH00j>SX0=CDe07IfMP0U9BD0
0P0E5AD02@0X:2P0YZJV0;Jf]P1`L700MGEe03Hf=P0X:2P0HV9R01lO7`0201DE5@0J03<c<`1VIVH0
N7Qh04=3@`0`<300MGEe0;:b/P2YZJT0GEeM07UiN@1ADE40De=C0:2PX02=SHd0Y:BT0<?3``2h^;P0
[Zj^07alO036a/H0[Zj^03Hf=P1KFe/0ZZZZ0<_;b`2@T90Dooooo`004oooool0R`1cLg<0JFUY06EU
I@1iNGT0AdM706e]K@1TI6@0=cLg05aLG01NGUh0>CTi051@D01LG5`0BDU903lo?`16ATH0KVi^04Y:
BP12@T80LG5a0451@@1CDe<0Lg=c04M7A`1JFUX0F5QH0492@P17AdL0L71`051@D015ADD0HF5Q04e=
C@1>CTh0GEeM06EUI@1IFET0G5aL059BDP1GEeL0@T92061PH01JFUX0>3Ph079bLP1CDe<0@d=304Q8
B01?Cdl0ADE505UIF@1=CDd0BTY:03dm?@14A4@0?clo0492@P1HF5P0JFUY03@d=01;Bd/0EEEE034a
<@1XJ6P0K6a/02TY:@1JFUX0EeMG03Hf=P1WIfL0De=C03Ti>@1UIFD0GemO04Y:BP1EEED0L71`04a<
C01HF5P0B4Q803`l?01cLg<0EEEE065QH@1BDU80EEEE06MWI`1>CTh0DE5A05eMG@15ADD0DE5A06QX
J00_;bl0J6QX07MgM`1<C4`0JFUY04E5A@1CDe<0H61P03Xj>P1RHV80M7Ad05UIF@1=CDd0EEEE05mO
G`1SHf<0A4A405QHF01PH600EeMG08J6QP1GEeL0BDU908^;R`1JFUX0CDe=06UYJ@1CDe<0EEEE071`
L018B4P0EeMG06YZJP1=CDd0Cdm?06=SH`0c<c<0Hf=S06i^KP0f=SH0IFEU05ADE00n?Sh0CTi>055A
D@0k>c/03_ooool03@22PX80DU9B06IVIP1@D500Hf=S04Q8B01VIVH0MGEe05UIF@0l?3`0ATI603dm
?@1:BTX00P1>CTh0?`1BDU80EeMG05EEE@1NGUh0?3`l04U9B@1=CDd0BTY:04M7A`18B4P0ADE503`l
?01QHF40<S8b04m?C`1BDU80EEEE06MWI`1KFe/0F5QH069RHP1<C4`0ATI604E5A@0b<S80AdM7051@
D00n?Sh0>c/k03hn?P15ADD0>SXj03lo?`18B4P0DU9B0410@017AdL0LG5a05QHF018B4P0D51@03hn
?P13@d<0CDe=04E5A@12@T80Hf=S0861P@1KFe/0Lg=c061PH01`L700C4a<055AD@1TI6@0EEEE061P
H01TI6@0DE5A05=CD`1KFe/0D51@04m?C`0205aLG00S05ADE01IFET0C4a<04M7A`1OGel0FEUI03dm
?@0h>3P0AdM705QHF01RHV80A4A405YJFP2BTY80F5QH03`l?01@D500EUIF061PH01]KFd0CDe=05AD
E017AdL0Cdm?05=CD`1KFe/0IFEU055AD@1HF5P0NGUi06ATI01fMWH0E5AD059BDP0k>c/00P16ATH0
1`1BDU80AdM704i>CP17AdL0ADE505ADE01EEED00P1>CTh03P1ADE40JVYZ06ATI01[Jf/0ATI605]K
F`16ATH0AdM70492@P1IFET0KFe]05UIF@1BDU80Lg=c3_ooool0S01SHf<0BDU905UIF@1ZJVX0BTY:
04];B`1ZJVX0>CTi04e=C@0k>c/0@D51051@D00j>SX0AdM703dm?@0b<S80E5AD034a<@1EEED0?3`l
038b<P1PH600GUiN03lo?`1>CTh0A4A406i^KP17AdL0G5aL04=3@`0g=cL0A4A402d];@1IFET0GUiN
04Q8B01?Cdl0>c/k07=cL`1NGUh0BTY:06YZJP1gMgL0NWYj05iNGP1GEeL0B4Q805YJFP1nOWh0QXJ6
04e=C@15ADD0De=C05ADE01:BTX0QXJ6069RHP1MGEd0FEUI055AD@1[Jf/0C4a<06i^KP1MGEd0EEEE
06ATI00h>3P0HV9R03lo?`1SHf<0P82006=SH`1LG5`0NGUi05EEE@1BDU80LG5a07inOP1QHF40C4a<
09JFUP1kNg/0V9RH06ATI02=SHd0KFe]065QH@2;Rh/0H61P05QHF00g=cL0BTY:06ATI01@D500C4a<
05EEE@25QHD0DU9B055AD@1LG5`0PX:208B4Q01fMWH0Ng]k0:2PX01JFUX0DU9B07MgM`1>CTh0Q8B4
05aLG00n?Sh0CTi>06=SH`1iNGT0GemO06][J`1^KVh0D51@06QXJ01SHf<0De=C03Lg=`1]KFd0DU9B
05ADE01:BTX0CTi>07EeM@0e=CD0T92@059BDP1SHf<0E5AD07=cL`1WIfL0@D5104m?C`0j>SX0MWIf
3_ooool01P0f=SH0I6AT09^KV`23Ph<0IFEU03Ph>0803@d=02h03Ph>014A4@0L71`0GEeM0820P00o
?cl051@D018B4P0S8b<051@D014A4@0L71`0Zj^[0:ZZZP1fMWH0G5aL03`l?00R8R803Ph>018B4P0I
6AT05ADE00l?3`0@4100D51@09bLW01<C4`071`L00l?3`0>3Ph03`l?03hn?P0o?cl04A4A00l?3`0@
41007alO030`<00B4Q80R8R80>;RhP2e]KD08R8R01LG5`0O7al08b<S0P0/;2`0C`1:BTX0M7Ad06QX
J01EEED0KVi^06=SH`1CDe<0PX:208j>SP1VIVH0=cLg01LG5`16ATH0F5QH08:2PP1<C4`0H61P02TY
:@0P82008R8R09>CT`2][Jd0IfMW06m_K`2YZJT0ADE506MWI`16ATH06QXJ02h^;P0d=3@0GUiN0820
P01nOWh0Bd];0;no_`2<S8`0?Cdm04a<C01EEED0WIfM07MgM`1:BTX0Ph>301HF5P0G5aL09BDU06QX
J02SXj<0SXj>02l_;`1CDe<0:b/[02@T9016ATH0De=C03Ph>00V9RH0BDU904Y:BP1@D500H61P051@
D02:RXX0SHf=0;no_`1mOGd0:b/[03/k>`1hN7P0<30`051@D02:RXX0`L710=;BdP3GemL0`l?30<g=
c@2ATI400P2?Shl00`2QXJ40cLg=07IfMP0Dooooo`004oooool0:P1jNWX0IFEU06i^KP1GEeL0FUYJ
059BDP1OGel0ATI605=CD`1KFe/0@D5104E5A@1]KFd0CDe=03lo?`1;Bd/0DU9B03@d=01KFe/0F5QH
03/k>`1DE5@0D51@03hn?P1HF5P0BDU904Q8B017AdL0CTi>05aLG00l?3`0D51@0861P@11@D40HV9R
04];B`0f=SH0BTY:05=CD`17AdL0D51@04];B`80CTi>00/0Cdm?05EEE@11@D40B4Q805eMG@1=CDd0
DE5A04U9B@1;Bd/0BDU904e=C@0204A4A01B05aLG01JFUX0@d=306UYJ@1GEeL0CTi>071`L01GEeL0
?Cdm075aL@18B4P0BTY:079bLP1DE5@0C4a<05aLG01>CTh0Hf=S06ATI01OGel0Hf=S05aLG01PH600
@T9206MWI`1TI6@0BDU904m?C`1`L700DU9B04I6AP1=CDd0DE5A03lo?`1OGel0GUiN03lo?`1RHV80
H61P05iNGP1@D500C4a<06YZJP1=CDd0Fe]K065QH@1UIFD0IfMW0451@@1QHF40F5QH059BDP1>CTh0
Fe]K06=SH`1@D500MgMg05aLG01DE5@0L71`06ATI01GEeL0IVIV05iNGP1;Bd/0GUiN05eMG@1IFET0
De=C055AD@1BDU80Fe]K05EEE@14A4@0CTi>05aLG01=CDd0ATI605=CD`12@T80CTi>05mOG`kooooo
01D0Rh^;05EEE@1]KFd0EEEE05aLG01GEeL0L71`09fMW@1YJFT0A4A404U9B@1<C4`0Cdm?061PH01K
Fe/0FEUI05]KF`1PH600EeMG03hn?P1?Cdl00P18B4P0<P1?Cdl0BTY:04=3@`0i>CT0EUIF02/[:`1<
C4`0D51@05ADE01/K6`0DU9B05IFEP1DE5@0ADE504I6AP1:BTX0>CTi04I6AP1?Cdl0?Cdm03/k>`0j
>SX0A4A403/k>`0h>3P0EEEE055AD@1=CDd0D51@06IVIP1DE5@0Cdm?05IFEP1KFe/0C4a<05ADE014
A4@0D51@06a/K02IVIT0C4a<07=cL`1RHV80Hf=S05]KF`1;Bd/0C4a<03Ti>@1EEED0Kfm_0P1>CTh0
@@1IFET0?clo05UIF@1^KVh0J6QX04i>CP1QHF40A4A405YJFP1BDU80G5aL04e=C@1:BTX0E5AD06EU
I@1QHF40E5AD055AD@1gMgL0HV9R04E5A@18B4P0AdM705EEE@1ZJVX0EeMG05UIF@1QHF40BTY:04U9
B@1BDU80EEEE0451@@1<C4`0JFUY05UIF@1cLg<0DE5A04e=C@0n?Sh0ADE50492@P1FEUH0@d=304M7
A`1;Bd/0De=C05UIF@15ADD0BTY:051@D01BDU80KVi^06e]K@21PH40De=C05EEE@14A4@0@d=304U9
B@1MGEd0KVi^065QH@1GEeL0K6a/00kooooo0380NGUi04U9B@1[Jf/0Jf][05QHF01TI6@0FUYJ03Xj
>P0m?Cd0>3Ph0410@01?Cdl0>3Ph0492@P0a<C40?3`l055AD@0g=cL0IFEU03<c<`13@d<0EUIF07=c
L`0o?cl0E5AD03<c<`1bLW80@D51071`L01:BTX0?3`l04i>CP18B4P0JVYZ07IfMP1?Cdl0CDe=03lo
?`1fMWH0G5aL051@D021PH40MGEe06m_K`1/K6`0EEEE030`<01MGEd0=cLg07]kN`80E5AD05P0EUIF
05EEE@1FEUH0RHV906ATI01NGUh0F5QH06EUI@1nOWh0E5AD08F5Q@1[Jf/0J6QX06=SH`1BDU80O7al
04E5A@1XJ6P0QXJ606UYJ@1JFUX0LW9b05MGE`1JFUX0N7Qh0861P@1YJFT0DU9B0:BTY01`L700UIFE
05IFEP20P800Lg=c06m_K`2DU9@0DU9B05UIF@17AdL0Fe]K06][J`1JFUX0BTY:06EUI@2<S8`0Jf][
05IFEP1PH600NWYj06IVIP1iNGT0MgMg08b<S01NGUh0EUIF07IfMP1ADE40MGEe051@D00i>CT0IfMW
05UIF@1mOGd0IVIV069RHP1VIVH0Kfm_06QXJ01NGUh0HV9R0492@P1GEeL0ATI605mOG`1BDU80A4A4
071`L00n?Sh0PH6104i>CP1QHF40JFUY08Z:RP1mOGd0ADE50492@P0k>c/0L71`3_ooool0?P0h>3P0
9RHV03hn?P2=SHd0UiNG05MGE`0C4a<06a/K02h^;P10@400D51@08B4Q01QHF40Fe]K04=3@`1MGEd0
=SHf014A4@0B4Q80=CDe028R8P1^KVh0@41005YJFP1KFe/0C4a<03hn?P0d=3@08b<S02`/;01HF5P0
GUiN05UIF@1bLW80EeMG03dm?@0a<C40:2PX02DU9@12@T80;Bd]01HF5P0@41003`l?02`/;01`L700
BTY:09>CT`37alL0RHV905=CD`0V9RH06a/K03@d=00T92@0=CDe07QhN01EEED0J6QX03Hf=P1MGEd0
Hf=S0P1iNGT0C00^;Rh0ATI606MWI`1TI6@0Kfm_07EeM@1PH600Jf][09bLW01mOGd0IFEU06=SH`2Q
XJ40^KVi06a/K00k>c/0FEUI07emO@1MGEd0IFEU0861P@1?Cdl0MWIf06EUI@1@D500BTY:0:j^[P10
@400C4a<01DE5@0I6AT092@T06=SH`1XJ6P0Ogmo038b<P1UIFD0L71`06EUI@2SXj<0T92@07moO`1F
EUH0IVIV07EeM@1PH600GemO07=cL`1jNWX0WIfM07AdM01aLG40PH6102h^;P0O7al0F5QH08n?S`2O
Wil0PH61079bLP27QhL0:2PX02HV9P0o?cl0TY:B0:JVYP2f][H0YJFU06e]K@16ATH0Jf][0861P@1Z
JVX0/[:b07EeM@14A4@Dooooo`004oooool03P1fMWH0I6AT06UYJ@1CDe<0KFe]05mOG`1oOgl0EEEE
04];B`2DU9@0@T9203<c<`1lO7`0Bd];0P1MGEd0N`15ADD0CTi>06=SH`1>CTh0IVIV04i>CP1CDe<0
HV9R03Lg=`19BDT0Hf=S02HV9P1=CDd0EUIF03<c<`1]KFd0FUYJ03@d=01nOWh0?3`l05QHF01EEED0
BTY:05eMG@1;Bd/0CDe=05aLG00h>3P0DU9B06e]K@0/;2`0De=C065QH@0c<c<0HF5Q04];B`0n?Sh0
JFUY04M7A`0o?cl0JFUY055AD@0o?cl0EEEE06e]K@12@T80EUIF06IVIP1QHF40>CTi05MGE`1OGel0
=3@d05eMG@1`L700EEEE04Y:BP1/K6`0DE5A051@D01oOgl0@410065QH@1iNGT0=CDe059BDP2IVIT0
@41006ATI02;Rh/0?clo05UIF@1GEeL0ATI604];B`1GEeL0HV9R065QH@1PH600KFe]05=CD`18B4P0
De=C05QHF01QHF40EEEE05YJFP1nOWh0HF5Q05EEE@1gMgL0FEUI06MWI`1EEED0G5aL08:2PP17AdL0
KFe]07MgM`1;Bd/0L71`06=SH`11@D40EEEE07AdM00e=CD0EeMG071`L011@D40E5AD075aL@0o?cl0
Fe]K059BDP0m?Cd0JFUY04e=C@15ADD0FUYJ0492@P11@D40K6a/03Ph>00>ooooo`0909BDU01LG5`0
KVi^059BDP1FEUH0GEeM0861P@1RHV80HF5Q0080?3`l06P0BTY:04=3@`1^KVh0Hf=S05iNGP1XJ6P0
JVYZ05eMG@11@D40Cdm?05ADE01;Bd/0DE5A04i>CP11@D40>3Ph05iNGP0Q8B40C4a<0451@@17AdL0
D51@04e=C@18B4P0BTY:04I6AP10@400CDe=03dm?@15ADD0D51@03/k>`0e=CD0<C4a03Ti>@0e=CD0
=SHf04Q8B00o?cl0?Shn0410@01MGEd0B4Q804I6AP1:BTX0J6QX055AD@1DE5@0BDU9055AD@1MGEd0
WIfM04Q8B01kNg/0ADE505eMG@1HF5P0CTi>07UiN@1PH600OGem096AT@1ADE40B4Q8061PH01;Bd/0
GUiN07alO01OGel0?clo06][J`1:BTX0E5AD05QHF01YJFT0B4Q805EEE@1GEeL0Jf][05aLG01HF5P0
De=C08F5Q@1MGEd0>CTi05EEE@19BDT0GEeM06IVIP1>CTh0De=C04m?C`14A4@0BTY:04U9B@17AdL0
<c<c03lo?`1FEUH0EeMG05=CD`1=CDd0?Cdm034a<@80AdM700L0EUIF04Q8B01OGel0E5AD04I6AP1=
CDd0BTY:0080BDU900h0F5QH05EEE@1eMGD0JFUY05ADE01=CDd0@D5104M7A`1:BTX0H61P06e]K@19
BDT0D51@06EUI@kooooo02<0IFEU04Y:BP1PH600Kfm_05ADE01FEUH0Hf=S04M7A`1ADE40AdM704U9
B@1]KFd0=CDe02/[:`13@d<0=3@d05EEE@0f=SH0EeMG03@d=014A4@0HF5Q06][J`1BDU80BTY:0451
@@1SHf<0?Shn06UYJ@11@D40ATI604A4A00m?Cd0GemO071`L00204E5A@1@03Lg=`1dM7@0DE5A05AD
E01fMWH0LW9b08N7Q`1?Cdl0G5aL055AD@1TI6@0>CTi08V9R@22PX80F5QH04Y:BP19BDT0EeMG075a
L@1OGel0FUYJ05ADE01UIFD0KFe]059BDP1]KFd0JFUY06m_K`1:BTX0EeMG07YjNP19BDT0De=C08j>
SP1gMgL0?Shn071`L01EEED0HV9R06IVIP2NWYh0IVIV071`L02XZ:P0I6AT09NGU`1[Jf/0S8b<05iN
GP1QHF40UYJF0492@P1PH600?Cdm05IFEP1`L700KVi^04Q8B01UIFD0MGEe08B4Q010@400De=C08B4
Q01ZJVX0KVi^07MgM`2MWId0DE5A04e=C@26QXH0@T9206m_K`1JFUX0=cLg05=CD`1HF5P0MGEe06=S
H`1MGEd206ATI00E061PH01dM7@0D51@03Xj>P19BDT0B4Q806a/K01KFe/0DE5A07MgM`1;Bd/0MGEe
04];B`1fMWH0ATI608:2PP2@T900ATI603Hf=P1:BTX0N7Qh00kooooo08`0<S8b0492@P1:BTX0Cdm?
08R8R01hN7P0QhN705eMG@0R8R807alO01XJ6P0U9BD0E5AD06YZJP0d=3@0?clo051@D01GEeL0BTY:
02h^;P0F5QH0>SXj014A4@0@4100:RXZ05iNGP1XJ6P0LW9b09NGU`2QXJ40KFe]04e=C@1PH600LG5a
0451@@1GEeL0B4Q804I6AP0i>CT0IfMW02PX:00d=3@05aLG02@T9017AdL0NGUi07emO@1UIFD0QhN7
04E5A@1CDe<0Kfm_08b<S02?Shl0FUYJ079bLP1JFUX0GUiN030`<00b<S80H61P05IFEP0f=SH0MgMg
03/k>`0L71`0Kfm_08b<S00b<S80D51@0;Ng]`1^KVh0HF5Q055AD@0X:2P0BDU905MGE`1CDe<0EEEE
03@d=00/;2`0B4Q809NGU`2?Shl0Hf=S09VIV@1OGel0<30`01TI6@0F5QH0DU9B05QHF016ATH0:2PX
02/[:`0Q8B40;bl_04I6AP13@d<0@T9208>3P`2=SHd0?clo06][J`2e]KD0Ng]k01dM7@0L71`0820P
030`<01HF5P0L71`08R8R02`/;00[:b/08N7Q`1ADE40Kfm_08F5Q@2c/k<0`/;207MgM`1eMGD0GUiN
075aL@1XJ6P0IFEU03<c<`28R8P0YZJV0;jn_P1cLg<0P820061PH01ADE40Q8B40;^k^`38b<P0B4Q8
03<c<aCooooo000Cooooo`1Y07inOP1^KVh0K6a/092@T01>CTh0HV9R06i^KP13@d<0F5QH05=CD`1=
CDd0?clo04a<C01cLg<0>CTi0492@P26QXH0?Cdm0451@@1TI6@0<30`05MGE`1mOGd0?Cdm05iNGP1I
FET0=3@d04Q8B01<C4`0BTY:05=CD`1PH600GEeM04m?C`1JFUX0MWIf05eMG@0k>c/0Hf=S069RHP18
B4P0Kfm_04a<C01;Bd/0Hf=S05EEE@1;Bd/0B4Q805MGE`1>CTh0Bd];04A4A01ZJVX0;Rh^04m?C`1/
K6`09BDU04Y:BP1[Jf/0;Bd]05EEE@1lO7`0=CDe075aL@1DE5@0<C4a05aLG01?Cdl0;Rh^065QH@1E
EED0ATI605YJFP18B4P0DU9B05=CD`1ADE40GEeM059BDP1>CTh0DU9B055AD@1=CDd0IVIV03Ti>@16
ATH0KFe]04];B`1JFUX0KFe]02d];@1@D500Hf=S03Xj>P1SHf<0GEeM04i>CP1XJ6P0?clo07YjNP1?
Cdl0IfMW07emO@1NGUh0Jf][0080EEEE01H0H61P05EEE@0o?cl0Jf][06a/K01GEeL0VYZJ04];B`1I
FET0P82004a<C01[Jf/0M7Ad059BDP1bLW80LG5a04Y:BP1ZJVX0?clo04a<C01VIVH0EEEE0P1DE5@0
200b<S80DU9B05mOG`1<C4`0IFEU05ADE01NGUh0JVYZ3_ooool0:P1mOGd0Bd];06YZJP1TI6@0DU9B
06][J`23Ph<0NWYj065QH@11@D40?Cdm04Y:BP1>CTh0J6QX05=CD`1;Bd/0GemO06IVIP1IFET0@T92
04a<C012@T80BTY:059BDP14A4@0B4Q803Ti>@1NGUh0:BTY04];B`1@D500Cdm?06MWI`19BDT0CDe=
05ADE011@D40>c/k05QHF01<C4`0@T9204U9B@80>3Ph05T0=3@d03/k>`0h>3P0>CTi04Y:BP15ADD0
=SHf0451@@1CDe<0CTi>04Y:BP1>CTh0H61P04I6AP1;Bd/0B4Q803`l?01FEUH0IfMW04M7A`1CDe<0
DU9B04E5A@1=CDd0ATI605]KF`1DE5@0F5QH06e]K@1>CTh0>c/k05MGE`0n?Sh0De=C05mOG`1cLg<0
@d=306i^KP17AdL0FEUI05YJFP1RHV80E5AD059BDP14A4@0IVIV05YJFP1MGEd0Cdm?07QhN016ATH0
?3`l04Y:BP1=CDd0E5AD069RHP1FEUH0K6a/05EEE@16ATH0Cdm?06][J`1GEeL0ADE505QHF01aLG40
KVi^06][J`1PH600EUIF05YJFP18B4P0EUIF05]KF`1:BTX0EeMG04m?C`1=CDd0K6a/04e=C@17AdL0
FUYJ06i^KP1CDe<0Lg=c06UYJ@1KFe/0Cdm?04e=C@0204i>CP0506ATI01WIfL0EUIF069RHP1lO7`0
3_ooool01`21PH40@d=305eMG@1JFUX0?Shn05eMG@0i>CT00P0m?Cd0;014A4@0Bd];04i>CP10@400
9bLW02XZ:P0n?Sh0@T92028R8P1XJ6P0=cLg04I6AP10@400HF5Q04A4A00d=3@0CTi>05eMG@0X:2P0
Kfm_0492@P0g=cL0BTY:03Lg=`1DE5@0LW9b04E5A@1CDe<0@d=306][J`1JFUX0G5aL06a/K01RHV80
NGUi04e=C@1JFUX0=SHf06YZJP0m?Cd0NWYj07QhN01DE5@0AdM70P1CDe<0E@1oOgl0OWin05]KF`1:
BTX0IFEU06UYJ@1PH600NGUi07AdM01WIfL0JVYZ04Q8B024Q8@0?Cdm05EEE@1^KVh0IfMW04=3@`1X
J6P0E5AD05IFEP1RHV80PX:207AdM01JFUX0SHf=06YZJP2?Shl0EUIF07YjNP1gMgL0JVYZ09:BTP1>
CTh0I6AT03lo?`1=CDd0Jf][065QH@0k>c/0H61P07]kN`1bLW80F5QH05MGE`1gMgL0L71`08>3P`1e
MGD0QXJ605ADE01iNGT0PH61055AD@29RHT0EeMG030`<01XJ6P0Jf][069RHP1eMGD0GemO061PH01/
K6`0Kfm_07AdM01VIVH0C4a<05=CD`10@400LG5a05mOG`1PH600HV9R02l_;`1lO7`0De=C051@D01=
CDd0RXZ:092@T01;Bd/0=SHf0492@P1]KFd03_ooool0E`1SHf<0@D5102`/;00L71`0820P05]KF`2P
X:00MgMg02LW9`0c<c<0:BTY03hn?P1GEeL0@D5104E5A@1GEeL0=SHf08J6QP2;Rh/05aLG01@D500f
=SH06a/K00l?3`0@4100;2`/08f=S@1LG5`0Ti>C07MgM`0I6AT061PH04M7A`0U9BD09RHV02DU9@14
A4@0?clo02XZ:P0n?Sh0V9RH09nOW`1JFUX0A4A405=CD`2@T900IFEU04A4A01?Cdl0820P038b<P1N
GUh0J6QX0;6a/@2QXJ40B4Q805QHF00l?3`0:2PX01lO7`0j>SX0NGUi04m?C`0b<S80>CTi065QH@2J
VYX0R8R806QXJ018B4P0S8b<06e]K@1FEUH0<c<c02TY:@0g=cL0D51@05=CD`2e]KD0NWYj06QXJ01H
F5P0E5AD06m_K`2UYJD0MWIf01HF5P0301DE5@0b03/k>`1BDU80JFUY03De=@0];Bd09RHV01XJ6P0T
92@0>c/k07emO@2>SXh0[:b/05QHF01aLG40/K6a061PH015ADD092@T04];B`0n?Sh09bLW02<S8`1I
FET0YjNW0<K6aP2e]KD0Jf][05aLG01aLG40GEeM07]kN`2IVIT0UYJF09nOW`2ZZZX0/k>c08V9R@2@
T900I6AT06m_K`1iNGT0P8200;Ng]`1RHV80CDe=06e]K@3Bd]80cLg=09JFUP1eMGDDooooo`004ooo
ool0502=SHd0KVi^079bLP1;Bd/0MgMg04U9B@1PH600GUiN0492@P1[Jf/0AdM703`l?01lO7`0ADE5
059BDP1=CDd0BDU905MGE`1;Bd/0@d=30P1BDU800`1CDe<0GUiN04E5A@0205]KF`0=028R8P1DE5@0
KVi^02@T901XJ6P0JFUY03Hf=P1oOgl0EeMG04a<C01ADE40Bd];069RHP0204m?C`0j06ATI00e=CD0
A4A408N7Q`0^;Rh0Bd];08f=S@0g=cL0BTY:04m?C`0n?Sh0Bd];061PH014A4@0C4a<04Y:BP1JFUX0
DU9B05YJFP0h>3P0EUIF069RHP18B4P0ADE505UIF@1LG5`0CDe=05YJFP1HF5P0Hf=S04I6AP1fMWH0
Jf][04U9B@1^KVh0EUIF051@D023Ph<0BDU905EEE@1fMWH0>CTi04Y:BP1^KVh0>SXj05MGE`1IFET0
AdM704e=C@1MGEd0EEEE05IFEP1GEeL0GUiN061PH017AdL0G5aL06e]K@80EUIF02D0@41007emO@1T
I6@0CTi>06=SH`1ADE40MgMg04E5A@1RHV80U9BD04m?C`1ZJVX0Ng]k04M7A`1_Kfl0L71`04E5A@1L
G5`0Ogmo03/k>`1OGel0HV9R0410@01LG5`0KVi^04U9B@1WIfL0KFe]04=3@`1_Kfl0FEUI03Hf=P1A
DE40Cdm?02TY:@1YJFT0DE5A00kooooo01P0QHF504a<C01NGUh0C4a<05IFEP1WIfL0OGem08:2PP1P
H600@T9204I6AP19BDT0@41006ATI01ZJVX0EeMG05YJFP1VIVH0EEEE04=3@`16ATH0A4A404U9B@1>
CTh204E5A@1I03Ti>@1FEUH0<30`04e=C@1MGEd0De=C06IVIP1;Bd/0BTY:04e=C@16ATH0?Shn055A
D@1;Bd/0@T9204Y:BP12@T80>3Ph03hn?P10@400@T9203Xj>P1DE5@0DE5A0410@019BDT0G5aL05]K
F`1:BTX0HF5Q051@D01JFUX0FEUI04E5A@18B4P0C4a<08N7Q`13@d<0M7Ad06MWI`1=CDd0GemO059B
DP16ATH0@T92059BDP21PH40@D5104I6AP1HF5P0ADE504Y:BP1YJFT0B4Q804i>CP1@D500BDU904Q8
B01ADE40HF5Q04Q8B01IFET0De=C06e]K@1KFe/0Bd];05eMG@2<S8`0EeMG03Hf=P14A4@0B4Q804I6
AP1DE5@0D51@06ATI01ADE40?clo04Q8B01ADE40CDe=0410@01ADE40GEeM065QH@1?Cdl0DE5A04M7
A`10@4000P19BDT02@1HF5P0HF5Q05=CD`1;Bd/0BDU906UYJ@13@d<0D51@05ADE00204Y:BP0<061P
H01WIfL0ATI604m?C`0n?Sh0>CTi03lo?`1MGEd0J6QX05MGE`1EEED0Hf=S3_ooool06@1eMGD0CDe=
05YJFP1ZJVX0DU9B05YJFP1GEeL0>CTi04I6AP0d=3@0F5QH04i>CP0e=CD0;Rh^03dm?@13@d<0@D51
02`/;01UIFD0@D5103Ti>@1GEeL0I6AT05=CD`10@4000P1GEeL07`13@d<0EeMG04A4A016ATH0C4a<
03Lg=`1WIfL0Fe]K059BDP1;Bd/0?Shn06e]K@1UIFD0DE5A079bLP1SHf<0KVi^06ATI01MGEd0=cLg
05QHF016ATH0M7Ad04Y:BP1=CDd0AdM704Y:BP19BDT0MGEe06IVIP1GEeL00P1IFET0801hN7P0H61P
07IfMP1aLG40GUiN06MWI`18B4P0PX:204m?C`1ADE40RXZ:07=cL`13@d<0MWIf05EEE@1>CTh0GUiN
08R8R01bLW80MgMg09BDU01RHV80S8b<05iNGP2=SHd0IFEU05UIF@2IVIT0GemO06MWI`0i>CT0Fe]K
0P1XJ6P0:`12@T80GUiN07MgM`1XJ6P0CDe=05]KF`2JVYX0MWIf06i^KP1]KFd0UiNG05UIF@1WIfL0
T92@04M7A`1jNWX0GEeM04Q8B01XJ6P0G5aL071`L026QXH0E5AD06UYJ@1XJ6P0E5AD08V9R@1jNWX0
@41005YJFP15ADD0GEeM055AD@1PH600P82005IFEP1XJ6P0MgMg06IVIP10@400Ogmo0;^k^`1=CDd0
0P1ADE400`1dM7Coooooooooo`0<ooooo`0;065QH@0/;2`071`L01HF5P0D51@0ADE509JFUP1XJ6P0
;2`/01dM7@0E5AD00P0L71`0C`0[:b/0;Bd]03lo?`1/K6`0_Kfm0<C4a01LG5`0Bd];05ADE019BDT0
>CTi034a<@1XJ6P07QhN04];B`1nOWh0L71`075aL@1oOgl05ADE00d=3@1ADE4071`L014A4@0S8b<0
JVYZ0:^[Z`2FUYH0N7Qh07MgM`0b<S80HF5Q04a<C01ZJVX0=CDe05IFEP0[:b/092@T02h^;P27QhL0
/k>c09bLW01>CTh07QhN02<S8`0h>3P0<S8b04=3@`1VIVH0BTY:01`L700V9RH0Ng]k0:^[Z`1lO7`0
X:2P09VIV@2CTi<0LG5a06EUI@1:BTX0BDU907inOP2;Rh/0Hf=S07moO`19BDT0<C4a02l_;`0^;Rh0
@41006ATI01_Kfl07AdM01@D500E5AD07QhN051@D01LG5`00P1XJ6P0;P1PH600I6AT05EEE@0d=3@0
Cdm?09FEU@2c/k<0h^;R079bLP1TI6@0GUiN06IVIP1QHF40XZ:R051@D00M7Ad09BDU03Lg=`1<C4`0
P82009VIV@2CTi<0HV9R04U9B@1?Cdl0<S8b03`l?01/K6`0CDe=04U9B@1UIFD0M7Ad075aL@1eMGD0
Jf][0861P@1@D500H61P07QhN01iNGT0]kNg0:VYZ@2SXj<0U9BD0;Fe]@1[Jf/Dooooo`004oooool0
3028R8P0Kfm_07IfMP1eMGD0HV9R05MGE`1fMWH0Bd];05MGE`1_Kfl0AdM704E5A@80GemO02@0CTi>
03`l?01TI6@0ATI604m?C`1HF5P0Bd];05EEE@28R8P0DU9B04I6AP1LG5`0GEeM03De=@1HF5P0Fe]K
03lo?`1IFET0Hf=S05ADE01HF5P0CTi>06ATI00o?cl0F5QH069RHP14A4@0G5aL06IVIP14A4@0DE5A
06e]K@10@400Cdm?06][J`0j>SX204Y:BP0a069RHP19BDT0AdM706a/K016ATH0C4a<06QXJ015ADD0
JFUY059BDP0l?3`0IfMW04U9B@0l?3`0FEUI04m?C`10@400H61P069RHP1<C4`0GUiN06m_K`1CDe<0
Fe]K06ATI01;Bd/0B4Q807EeM@10@400F5QH07EeM@18B4P0E5AD065QH@1DE5@0GemO06IVIP1/K6`0
C4a<05YJFP1oOgl0DU9B05UIF@1oOgl0EUIF069RHP1NGUh0JVYZ06][J`0204m?C`0T08F5Q@1PH600
Jf][05MGE`1VIVH0H61P04=3@`1[Jf/0NWYj05ADE01kNg/0I6AT059BDP1jNWX0IVIV069RHP1IFET0
Lg=c05EEE@1WIfL0IFEU05eMG@1BDU80EEEE05UIF@1EEED0FEUI04U9B@1=CDd0G5aL04];B`1LG5`0
DU9B04Y:BP1FEUH0EeMG3_ooool02`1XJ6P0E5AD05MGE`1:BTX0D51@05IFEP1dM7@0FUYJ065QH@0f
=SH0=cLg0080Bd];06/0GemO06MWI`1DE5@0De=C06ATI01[Jf/0>SXj04E5A@17AdL0B4Q804M7A`0l
?3`0ADE5038b<P1@D500;Rh^0492@P1;Bd/0De=C06IVIP1@D500CDe=05eMG@1=CDd0@41004a<C018
B4P0@T9204E5A@0m?Cd0=CDe03Hf=P0h>3P0@D5103<c<`18B4P0BTY:03Xj>P0n?Sh0Fe]K055AD@16
ATH0GUiN04E5A@1?Cdl0CDe=04I6AP18B4P0IFEU08Z:RP1SHf<0K6a/069RHP1<C4`0DE5A04a<C01K
Fe/0ATI607=cL`2DU9@0GemO04E5A@1[Jf/0ADE504=3@`1NGUh0E5AD03hn?P1PH600?Cdm04=3@`1:
BTX0EUIF03<c<`1IFET0Bd];075aL@1GEeL0ATI6055AD@2DU9@0CDe=02h^;P0l?3`0Bd];04M7A`1Q
HF40HV9R06][J`1GEeL0BDU904m?C`1FEUH0D51@03`l?019BDT0LG5a061PH01MGEd0Bd];04i>CP0n
?Sh0@D5104M7A`1=CDd0BTY:03hn?P0204Q8B00B05aLG01?Cdl0E5AD04i>CP1PH600Cdm?06EUI@1O
Gel0FEUI05=CD`11@D40@T9204e=C@1UIFD0K6a/05MGE`1FEUH0Fe]K3_ooool0=`1aLG40ATI605YJ
FP1JFUX0@T92055AD@13@d<0?3`l04Q8B01;Bd/0DU9B05YJFP13@d<0:RXZ03Lg=`0l?3`0?clo02PX
:01OGel0?3`l03Xj>P1<C4`0EeMG04I6AP0c<c<0FUYJ04m?C`0g=cL0GEeM04U9B@0d=3@0G5aL038b
<P1IFET0De=C051@D01>CTh0@d=3065QH@1@D500EeMG07UiN@1PH600MGEe04m?C`1<C4`0B4Q805QH
F00a<C40MWIf04e=C@1@D500EeMG03lo?`1DE5@00P1WIfL0D`1KFe/0DE5A06ATI01XJ6P0DE5A07Mg
M`1LG5`0KFe]051@D01FEUH0SHf=04i>CP1OGel0Ogmo06UYJ@1:BTX0O7al04a<C01NGUh0FEUI07Qh
N01gMgL0IfMW08f=S@1WIfL0Shn?05ADE01kNg/0MGEe065QH@2SXj<0HV9R06YZJP0i>CT0Fe]K06EU
I@1bLW80@D5105eMG@1hN7P0F5QH059BDP1JFUX0VIVI07=cL`22PX80M7Ad08^;R`1:BTX0IFEU06a/
K01NGUh0R8R805IFEP11@D40Kfm_06ATI01JFUX0Ph>305YJFP1CDe<0EUIF06ATI02;Rh/0FUYJ04I6
AP1HF5P0BDU906IVIP1ADE40CDe=07UiN@0j>SX0MgMg06i^KP1NGUh0Cdm?071`L02YZJT0E5AD04i>
CP14A4@0Ogmo00kooooo02h0MgMg03/k>`0X:2P07QhN01LG5`0l?3`0PH6106=SH`11@D40410@014A
4@0E5AD051@D01`L700^;Rh0E5AD09jNWP2LW9`0Xj>S055AD@1?Cdl0FUYJ05UIF@1cLg<0Zj^[07in
OP0Y:BT0EUIF08n?S`0h>3P0Shn?07QhN01QHF40>CTi03Xj>P2f][H0GUiN05mOG`24Q8@0/k>c09VI
V@0O7al0<c<c01XJ6P1XJ6P0DU9B0P2;Rh/0900f=SH0;2`/04E5A@1KFe/0K6a/0861P@1lO7`0;2`/
03Ti>@0e=CD0;bl_06UYJ@1jNWX0E5AD09bLW01RHV80<S8b06YZJP2h^;P0YJFU08^;R`1:BTX0C4a<
06=SH`2_[jl0KVi^04e=C@1FEUH0UYJF09NGU`1bLW80ADE503Xj>P0N7Qh07alO02`/;080UiNG0180
EUIF024Q8@0S8b<08B4Q01dM7@1VIVH0/;2`028R8P0E5AD05QHF01hN7P17AdL0U9BD0:6QX@1=CDd0
O7al09BDU017AdL2069RHP0<055AD@16ATH0@d=303De=@0K6a/0@41007AdM027QhL0=CDe05eMG@0c
<c<08b<S0P0a<C40501CDe<0TY:B05UIF@1lO7`0Ph>306EUI@1OGel0JVYZ08:2PP18B4P0=cLg065Q
H@1RHV80ADE507moO`21PH40@D5103@d=01QHF40F5QH5?ooool001?ooooo00`0QhN705eMG@26QXH0
Hf=S05aLG01VIVH0I6AT05QHF01GEeL0FEUI05eMG@15ADD205=CD`0f0492@P1:BTX0IFEU03<c<`1?
Cdl0IVIV02`/;01=CDd0H61P0492@P1OGel0@d=304m?C`1>CTh0Cdm?051@D01FEUH0ADE506IVIP14
A4@0AdM707IfMP1_Kfl0>CTi06=SH`1dM7@0C4a<05iNGP1MGEd0CTi>05ADE01HF5P0G5aL05iNGP1C
De<0GEeM04m?C`1<C4`0GEeM02d];@1VIVH0HV9R02/[:`1_Kfl0K6a/02h^;P2GUiL0HV9R03@d=02<
S8`0BDU904A4A01iNGT0DE5A0P1NGUh01@1OGel0HF5Q0451@@1XJ6P0L71`0080GUiN0280KFe]03`l
?01UIFD0F5QH04Y:BP1UIFD0JFUY04i>CP1MGEd0PH61055AD@1YJFT0IfMW034a<@1HF5P0JVYZ051@
D01IFET0@d=306ATI01GEeL0?Shn07inOP1@D500Hf=S04m?C`1PH600MgMg04U9B@1<C4`0L71`04];
B`1HF5P0J6QX0P1ZJVX06P22PX80H61P06YZJP1OGel0GEeM06m_K`15ADD0MgMg06m_K`15ADD0FEUI
06ATI01:BTX0N7Qh06EUI@0f=SH0O7al04U9B@0d=3@0GemO06MWI`15ADD0IFEU03lo?`1ADE40Fe]K
3_ooool0S01dM7@0>c/k05eMG@1IFET0FUYJ071`L01jNWX0JFUY06MWI`1=CDd0?Cdm04i>CP1<C4`0
Ng]k06YZJP1VIVH0Fe]K071`L01MGEd0=CDe04=3@`0o?cl0A4A404U9B@14A4@0@D5102d];@1TI6@0
;2`/03Xj>P0n?Sh0C4a<05aLG01;Bd/0CDe=04M7A`1=CDd0CTi>05IFEP1=CDd0@d=304i>CP10@400
<S8b02`/;00l?3`0=SHf03`l?018B4P0@d=30410@00c<c<0EUIF055AD@14A4@0ADE505=CD`10@400
EUIF0451@@1FEUH0JFUY092@T01KFe/0NWYj07]kN`1ADE40EeMG0410@01DE5@0=SHf06YZJP1mOGd0
EeMG04I6AP1JFUX0A4A402`/;01`L700AdM704=3@`1@D500@d=305]KF`1[Jf/0F5QH04U9B@1ADE40
Fe]K06YZJP1DE5@0DU9B05ADE02IVIT0E5AD03<c<`12@T80CDe=04A4A01@D500GemO05iNGP19BDT0
;Rh^04M7A`11@D40B4Q80492@P14A4@0EeMG0410@018B4P0ATI603lo?`0g=cL0ADE50492@P1@D500
Fe]K04Y:BP13@d<0ATI604e=C@1BDU80B4Q805aLG01WIfL0H61P05QHF026QXH0GEeM059BDP14A4@0
ADE504I6AP1TI6@0GUiN061PH018B4P0CDe=3_ooool06`1[Jf/0ADE504e=C@1WIfL0@41004M7A`1I
FET0;bl_051@D00l?3`0Cdm?065QH@0m?Cd0=cLg03hn?P0i>CT0C4a<03@d=01@D500?3`l03dm?@1C
De<0@T920410@00b<S80AdM705ADE00205EEE@0H03dm?@1ADE40GUiN0492@P1WIfL0AdM705UIF@1R
HV80@d=30861P@18B4P0De=C096AT@1IFET0GEeM065QH@1EEED0>c/k04i>CP18B4P0IVIV04U9B@14
A4@0EeMG0P19BDT0201lO7`0MgMg061PH01@D500IVIV061PH01NGUh0Ogmo0P1_Kfl03@1LG5`0Jf][
09>CT`1BDU80GUiN07inOP1aLG40DE5A07alO01ADE40JFUY05aLG01SHf<00P1eMGD04P2DU9@0Ogmo
08j>SP1[Jf/0UiNG08V9R@1LG5`0YJFU06YZJP1VIVH0@d=306MWI`1cLg<0Ph>305aLG023Ph<0UIFE
05IFEP80GUiN02@0QHF5071`L01lO7`0IFEU09VIV@1MGEd0D51@06a/K01RHV80OGem05UIF@16ATH0
QhN705EEE@1XJ6P0Vi^K04Q8B01VIVH0MGEe05ADE023Ph<0Ng]k04A4A016ATH0@d=308:2PP19BDT0
E5AD07EeM@1GEeL0LG5a055AD@1SHf<0HV9R06EUI@2ATI42059BDP0304m?C`1nOWkooooo00gooooo
08`0Ng]k05YJFP0];Bd0>3Ph05YJFP0[:b/051@D03Lg=`1LG5`0>CTi010@400D51@06QXJ02TY:@0e
=CD0VIVI04E5A@1MGEd0LW9b05]KF`1BDU809bLW03Lg=`19BDT0Jf][0<W9b@1fMWH0GEeM07YjNP2L
W9`0KVi^04=3@`0l?3`0@D5105YJFP1HF5P0>SXj0410@015ADD0UIFE051@D02>SXh0Shn?04A4A00_
;bl0:BTY055AD@1hN7P0Bd];04Y:BP1XJ6P0LG5a09RHV01iNGT0/K6a065QH@0n?Sh0BDU903@d=00U
9BD0Hf=S09FEU@2FUYH0GemO02HV9P1XJ6P0Z:RX0:j^[P17AdL0>c/k01`L701MGEd0HF5Q07IfMP19
BDT0ADE50451@@2?Shl0^KVi03lo?`0H61P0AdM703`l?01dM7@0UiNG0;6a/@1DE5@0EEEE059BDP0g
=cL06a/K04e=C@2LW9`0IFEU02XZ:P0H61P071`L02h^;P2?Shl0P82002d];@14A4@0=CDe02l_;`1j
NWX0C4a<03De=@0Z:RX0GemO05YJFP1lO7`0OGem0820P02?Shl0:b/[05ADE00S8b<0<C4a04=3@`1]
KFd0Vi^K0<_;b`23Ph<0UIFE08J6QP0Z:RX0=cLg0:6QX@1kNg/0N7Qh069RHP1UIFD0L71`05aLG01/
K6`0DE5A04M7A`12@T80Fe]K05eMGACooooo003oooooooooooooP_ooool00001\
\>"],
  ImageRangeCache->{{{0, 639}, {73.375, 0}} -> {-0.126578, -0.0116304, \
0.00665596, 0.00665596}, {{19, 158.75}, {71.625, 1.6875}} -> {-43.5083, \
-4.07955, 2.28983, 2.28983}, {{172.75, 312.5}, {71.625, 1.6875}} -> \
{-395.569, -4.07955, 2.28983, 2.28983}, {{326.438, 466.188}, {71.625, \
1.6875}} -> {-747.487, -4.07955, 2.28983, 2.28983}, {{480.188, 619.938}, \
{71.625, 1.6875}} -> {-1099.55, -4.07955, 2.28983, 2.28983}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.51364 
%%ImageSize: 640 968.727 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.238095 0.036039 0.238095 [
[ 0 0 0 0 ]
[ 1 1.51364 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.51364 L
0 1.51364 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.036039 0.261905 0.276299 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .50436 L
.2381 .50436 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .50436 L
.2381 .50436 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .01189 L
.47619 .01189 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .01189 L
.47619 .01189 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .02607 L
.71429 .02607 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .02607 L
.71429 .02607 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .09949 L
.95238 .09949 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .09949 L
.95238 .09949 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FFFF0000FF00000000FFFF8000FF0000FF0000FF0000
FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF00000000FFFF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF8000
FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF00000000FF0000FFFF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF80000000FFFF0000FF0000FF80000000FFFF8000FF0000FF0000
FF0000FF0000FF00000000FFFF0000
FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF80000000FFFF8000FF0000FF8000FF0000FF0000FF0000
FF0000FF00000000FF0000FFFF0000
FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF8000FF00000000FFFF8000FF80000000FFFF8000FF0000FF0000
FF0000FF0000FF0000FF80000000FF
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF8000FF8000FF0000FF0000
FF00000000FFFF0000FF8000FF0000
FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF00000000FF
FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF8000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF8000FF0000FF0000FF0000
FF0000FF00000000FFFF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF8000FF0000
FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF8000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000
0000FFFF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF80000000FFFF0000FF8000FF8000
FF8000FF0000FF8000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF8000FF0000FF8000FF0000FF8000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000
FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF00000000FFFF8000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF8000
FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000
FF0000FF8000FF0000FF0000FF0000FF8000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF8000FF8000FF0000FF0000
FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000
FF80000000FFFF00000000FFFF8000FF8000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF80000000FF
0000FF0000FF0000FFFF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000
FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
0000FF0000FF0000FFFF8000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000
FF0000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF00000000FFFF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF80000000FFFF8000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF80000000FFFF0000FF0000FF0000FF0000
FF80000000FFFF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF8000
FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000
FF80000000FFFF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF00000000FF0000FF
FF80000000FFFF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF80000000FF
FF8000FF80000000FFFF80000000FF
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF8000FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF8000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF8000FF0000FF0000
FF8000FF8000FF80000000FFFF0000FF0000FF0000FF80000000FFFF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000
0000FFFF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF8000FF8000FF80000000FF0000FFFF0000FF0000FF0000FF8000
FF8000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000
FF0000FF8000FF8000FF0000FF0000FF8000FF8000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FFFF8000FF8000FF8000FF0000FF8000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF00000000FF0000FFFF8000FF8000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF00000000FFFF0000
0000FFFF0000FF0000FF0000FF0000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.261905 0.036039 0.5 0.276299 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .01251 L
.2381 .01251 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .01251 L
.2381 .01251 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .35042 L
.47619 .35042 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .35042 L
.47619 .35042 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .26031 L
.71429 .26031 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .26031 L
.71429 .26031 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .01856 L
.95238 .01856 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .01856 L
.95238 .01856 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FFFF800000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF00
0000FF00FF000000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF00
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF000000FF0000FF
00FF000000FF00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF000000FF00FF00
0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FFFF0000FF80000000FF
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF00
0000FF00FF0000FF000000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF80000000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF000000FF00FF000000FF0000FF00FF0000FF00
0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF00
0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF00
0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF00FF0000FF0000FF000000FF
00FF000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FFFF80000000FF
0000FFFF80000000FF0000FF0000FF
0000FFFF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF00
0000FF00FF0000FF000000FF0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF
0000FF00FF0000FF000000FF00FF0000FF0000FF000000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF0000FF000000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF000000FF00FF000000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF00FF000000FF0000FF0000FF0000FF00FF00
0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF
0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF0000FF0000FF0000FF000000FF
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF00
0000FF00FF0000FF000000FF00FF0000FF0000FF000000FF0000FF0000FFFF800000FF00
0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF00FF000000FF00FF0000FF0000FF000000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF00
0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF00
00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF00FF00
0000FF00FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF00
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FFFF80000000FF00FF00
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF800000FF00
0000FF0000FF00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF000000FF00FF00
0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF000000FF00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
00FF000000FF00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF00FF0000FF0000FF00
00FF0000FF000000FF00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF000000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF00FF0000FF00
0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF000000FF00FF000000FF0000FF0000FF00FF000000FF
0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF00FF00
0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF00FF000000FF0000FF
0000FFFF80000000FF0000FF00FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF00FF000000FF
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00
0000FF00FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF80000000FF
0000FF00FF000000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FFFF00000000FF0000FF
FF800000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF000000FF0000FF
0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
FF80000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF00FF0000FF000000FF0000FFFF00000000FF0000FF
FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF00
00FF000000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF00FF000000FF00FF0000FF00
00FF000000FF0000FF0000FF0000FF0000FF00FF000000FF00FF00FF80000000FF0000FF
FF800000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF00FF000000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF
0000FF0000FF0000FFFF80000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF000000FF0000FF0000FF0000FF00FF000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF000000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF000000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF000000FF
00FF000000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF00FF0000FF0000FF00
00FF0000FF000000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF00
0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF00
00FF0000FF000000FF0000FF0000FF00FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF000000FF0000FF00FF000000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FFFF800000FF0000FF00
0000FF0000FFFF80000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF000000FF0000FF
00FF000000FF00FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF000000FF
0000FF00FF000000FF00FF0000FF00
00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF00FF00
00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF000000FF00FF00
00FF0000FF0000FF0000FF0000FF000000FF00FF000000FF00FF00FF80000000FF0000FF
0000FF00FF000000FF00FF000000FF
00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF00
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
FF800000FF0000FF000000FF00FF00
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FFFF80000000FF0000FF
0000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FFFF80000000FF00FF0000FF000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF000000FF00FF00
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF00FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF
0000FF0000FF00FF0000FF0000FF00
00FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FFFF80000000FFFF80000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF
0000FF0000FFFF80000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF00
00FF000000FF00FF0000FF0000FF00
00FF000000FF00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FFFF80000000FF0000FF00FF000000FF0000FF0000FF0000FF00FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF00FF00
00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.036039 0.738095 0.276299 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .01752 L
.2381 .01752 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .01752 L
.2381 .01752 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .07801 L
.47619 .07801 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .07801 L
.47619 .07801 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .48684 L
.71429 .48684 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .48684 L
.71429 .48684 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .05944 L
.95238 .05944 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .05944 L
.95238 .05944 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
0000FF00FF000000FF00FF0000FF000000FF0000FF0000FF0000FFFF80000000FF0000FF
FF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF
FF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF80000000FF0000FFFF80000000FF0000FF0000FF0000FFFF80000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FFFF80000000FF
FF80000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF80000000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF
0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF80000000FF
0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF
FF80000000FF0000FF00FF0000FF000000FF0000FF0000FFFF80000000FF0000FF0000FF
0000FF0000FF0000FF0000FFFF8000FF80000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FFFF8000FF80000000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF
0000FFFF80000000FF0000FF0000FF0000FF0000FFFF8000FF8000FF8000FF0000FF0000
0000FF0000FF0000FF0000FF0000FF
FF800000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FFFF8000FF8000FF0000
FF80000000FFFF80000000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF
0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FFFF8000FF0000FF00000000FF
0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF80000000FF0000FFFF80000000FF0000FFFF8000FF8000FF8000FF00000000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF
FF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF80000000FF
0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF80000000FF0000FFFF80000000FFFF80000000FFFF8000FF80000000FFFF80000000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000FF8000FF0000FF8000
FF8000FF80000000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF80000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00
0000FF0000FF0000FFFF80000000FFFF8000FF8000FF8000FF0000FF8000FF80000000FF
0000FFFF80000000FFFF80000000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FFFF0000FF8000
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF80000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF0000FF0000FF0000FF8000FF80000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF80000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FFFF8000
0000FF0000FF0000FF00FF000000FF
0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FFFF8000FF8000FF8000FF0000FF8000FF80000000FF
FF80000000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FFFF8000
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FFFF80000000FF0000FF
0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000FF0000FF0000FF80000000FF
FF80000000FFFF8000FF80000000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FFFF80000000FF
0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF8000FF0000FF80000000FF
FF80000000FF0000FF0000FF0000FFFF80000000FFFF8000FF80000000FFFF80000000FF
0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF
0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF
0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00
0000FF0000FF0000FFFF8000FF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FFFF80000000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
00FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000
0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF80000000FFFF8000FF80000000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF80000000FF0000FF00FF000000FF0000FF0000FFFF80000000FF0000FF
0000FF0000FF0000FFFF80000000FF0000FFFF80000000FF0000FF00FF0000FF000000FF
0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF80000000FF0000FFFF80000000FF0000FF0000FF00FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF8000
FF80000000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FFFF0000FF8000
FF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FFFF80000000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF80000000FF0000FF0000FF00FF000000FF0000FF0000FFFF80000000FFFF80000000FF
0000FF0000FFFF8000FF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FFFF8000FF80000000FFFF8000FF80000000FF0000FF0000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000
FF8000FF80000000FFFF80000000FF0000FFFF80000000FF0000FF0000FF0000FFFF8000
0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF80000000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FFFF80000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF80000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF
0000FF00FF000000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FFFF0000
FF0000FF80000000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF80000000FFFF80000000FFFF8000FF80000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF
0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF80000000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF
FF00000000FF0000FFFF80000000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF
0000FF0000FF0000FFFF80000000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FFFF80000000FF0000FF
00FF0000FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF000000FF00FF0000FF000000FF0000FF0000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FFFF8000
FF80000000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF80000000FF0000FF0000FFFF0000FF0000FF80000000FFFF8000
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FFFF80000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF80000000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF80000000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF00FF00
0000FF0000FF0000FFFF00000000FF0000FFFF80000000FFFF80000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF
FF8000FF80000000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF00FF0000FF000000FF0000FF00FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF8000FF0000FF8000FF8000
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF8000FF80000000FF0000FFFF8000
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF80000000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF00FF00
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FFFF8000FF80000000FFFF8000
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FFFF80000000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF00FF0000FF000000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FFFF80000000FFFF80000000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF00FF000000FF00FF00
0000FF0000FF0000FF0000FFFF8000
0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000
0000FF0000FF0000FF00FF000000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FFFF8000FF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FFFF80000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FFFF80000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000
0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF8000FF80000000FF0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FFFF8000
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FFFF80000000FF00FF000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF
00FF0000FF000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FFFF80000000FF00FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF000000FF0000FF00FF00
0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FFFF80000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF0000FF000000FF0000FF00FF000000FF00FF000000FF00FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF00FF00
FF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF
0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF00
0000FF00FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.738095 0.036039 0.97619 0.276299 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .2357 L
.2381 .2357 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .2357 L
.2381 .2357 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .01189 L
.47619 .01189 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .01189 L
.47619 .01189 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .06403 L
.71429 .06403 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .06403 L
.71429 .06403 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .33019 L
.95238 .33019 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .33019 L
.95238 .33019 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000
0000FFFF8000FF0000FF8000FF0000FF8000FF8000FF80000000FF0000FFFF00000000FF
FF8000FF0000FF8000FF0000FF80000000FFFF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF00000000FFFF8000
FF8000FF8000FF0000FF80000000FFFF8000FF0000FF0000FF0000FF0000FF8000FF8000
FF8000FF0000FF8000FF0000FF0000FF8000FF8000FF8000FF0000FF8000FF0000FF8000
FF0000FF8000FF0000FF8000FF0000
FF8000FF8000FF0000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF0000FF8000
FF0000FF8000FF80000000FFFF8000FF8000FF8000FF80000000FFFF0000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF0000
FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF8000
FF8000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF0000FF8000
FF0000FF0000FF8000FF8000FF0000
FF0000FF00000000FFFF8000FF8000FF0000FF8000FF0000FF0000FF8000FF8000FF8000
FF0000FF8000FF8000FF8000FF80000000FFFF8000FF8000FF8000FF8000FF00000000FF
FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF0000FF8000FF8000FF80000000FF
FF0000FF8000FF8000FF0000FF8000FF0000FF80000000FFFF8000FF0000FF8000FF8000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF0000FF8000FF0000
FF0000FF0000FF8000FF8000FF0000FF8000FF0000FF8000FF0000FF8000FF8000FF0000
FF8000FF0000FF0000FF8000FF8000
FF0000FF00000000FF0000FF0000FFFF0000FF0000FF0000FF0000FF8000FF0000FF8000
FF8000FF8000FF80000000FFFF80000000FFFF8000FF8000FF0000FF0000FF0000FF8000
FF8000FF0000FF0000FF8000FF8000FF8000FF0000FF8000FF80000000FFFF80000000FF
FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF80000000FFFF8000FF8000
FF0000FF8000FF0000FF8000FF0000FF0000FF8000FF8000FF8000FF0000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF0000
FF0000FF0000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF0000FF0000FF8000FF8000FF0000FF0000FF8000FF0000
FF80000000FFFF80000000FFFF8000FF8000FF8000FF80000000FFFF8000FF8000FF0000
FF8000FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF0000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF8000FF8000
FF8000FF80000000FFFF8000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000
0000FFFF0000FF8000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF0000FF8000FF8000FF0000FF8000FF8000FF0000FF8000FF0000FF0000FF0000FF0000
FF8000FF0000FF8000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF8000FF0000FF8000FF8000FF0000
FF8000FF0000FF8000FF80000000FFFF0000FF8000FF00000000FFFF0000FF80000000FF
FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF8000
FF0000FF0000FF8000FF0000FF0000
FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF0000FF0000FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF0000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF8000FF00000000FF0000FFFF0000FF0000FF0000FF0000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF0000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF0000FF80000000FFFF0000
FF8000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF00000000FFFF80000000FFFF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000
FF8000FF0000FF8000FF80000000FFFF8000FF0000FF0000FF0000FF0000FF8000FF8000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF0000FF8000FF0000
FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF0000FF80000000FFFF0000
FF8000FF0000FF0000FF0000FF8000
FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF80000000FFFF8000FF8000FF0000
FF0000FF0000FF80000000FFFF8000FF8000FF0000FF8000FF0000FF0000FF0000FF8000
FF8000FF8000FF0000FF80000000FFFF8000FF0000FF0000FF0000FF0000FF8000FF0000
0000FFFF8000FF8000FF0000FF0000FF8000FF0000FF8000FF0000FF0000FF8000FF8000
FF8000FF80000000FFFF0000FF0000FF0000FF80000000FFFF8000FF8000FF8000FF0000
FF0000FF8000FF8000FF8000FF8000FF0000FF80000000FF0000FF0000FF0000FF0000FF
FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF0000FF80000000FFFF8000FF80000000FF0000FFFF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000
FF8000FF0000FF0000FF8000FF8000FF0000FF8000FF8000FF8000FF0000FF8000FF0000
FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF8000
0000FFFF0000FF0000FF8000FF8000FF0000FF80000000FFFF0000FF8000FF0000FF0000
FF0000FF8000FF80000000FFFF8000FF8000FF8000FF8000FF0000FF80000000FF0000FF
FF80000000FF0000FFFF8000FF0000
FF0000FF8000FF0000FF8000FF8000FF8000FF80000000FF0000FFFF8000FF8000FF0000
FF8000FF80000000FFFF8000FF0000FF8000FF8000FF00000000FFFF0000FF0000FF0000
FF8000FF0000FF0000FF8000FF8000FF0000FF0000FF8000FF0000FF00000000FFFF8000
FF0000FF0000FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF0000FF8000FF8000
0000FFFF8000FF8000FF8000FF80000000FF0000FFFF8000FF8000FF8000FF8000FF0000
FF0000FF8000FF80000000FFFF8000FF8000FF0000FF80000000FFFF8000FF80000000FF
FF8000FF80000000FF0000FF0000FF
FF0000FF8000FF0000FF80000000FFFF8000FF8000FF8000FF8000FF0000FF8000FF0000
FF8000FF80000000FFFF0000FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF0000
FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF0000
FF0000FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF0000
FF0000FF0000FF8000FF00000000FFFF8000FF8000FF8000FF0000FF8000FF80000000FF
FF8000FF8000FF8000FF0000FF8000
0000FFFF00000000FFFF0000FF0000FF8000FF8000FF8000FF8000FF0000FF8000FF0000
FF80000000FFFF8000FF80000000FFFF8000FF8000FF0000FF00000000FFFF80000000FF
FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF8000FF0000FF8000FF80000000FF
FF8000FF8000FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF0000FF8000FF0000
FF8000FF8000FF8000FF0000FF80000000FFFF8000FF8000FF0000FF8000FF0000FF0000
FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF8000FF8000
FF8000FF0000FF0000FF8000FF8000
0000FFFF8000FF8000FF0000FF8000FF0000FF80000000FFFF0000FF8000FF8000FF0000
FF8000FF8000FF80000000FFFF80000000FFFF8000FF0000FF0000FF8000FF8000FF8000
FF0000FF0000FF8000FF8000FF8000FF0000FF8000FF8000FF0000FF8000FF00000000FF
FF0000FF8000FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF0000FF8000FF8000FF0000FF8000FF8000FF0000FF8000FF0000
FF0000FF0000FF8000FF8000FF8000FF8000FF0000FF80000000FF0000FFFF8000FF0000
FF0000FF0000FF8000FF0000FF8000
FF0000FF0000FF8000FF0000FF80000000FFFF8000FF0000FF8000FF0000FF8000FF0000
FF0000FF8000FF8000FF8000FF0000FF0000FF0000FF00000000FFFF8000FF80000000FF
FF0000FF0000FF8000FF8000FF0000FF8000FF0000FF0000FF0000FF8000FF80000000FF
FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000
FF8000FF0000FF8000FF0000FF8000FF0000FF0000FF8000FF00000000FFFF0000FF0000
FF0000FF0000FF8000FF0000FF8000
0000FFFF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF0000FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF00000000FFFF8000FF8000
FF8000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF80000000FF
FF8000FF8000FF0000FF80000000FFFF8000FF80000000FFFF80000000FFFF0000FF0000
FF8000FF8000FF80000000FFFF0000FF0000FF0000FF8000FF8000FF0000FF8000FF8000
FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF0000FF8000FF0000FF0000
FF0000FF0000FF8000FF8000FF8000
0000FFFF8000FF8000FF0000FF8000FF0000FF8000FF80000000FFFF0000FF0000FF0000
FF0000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
0000FF0000FFFF8000FF00000000FF0000FFFF8000FF8000FF0000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF0000FF80000000FFFF8000
FF8000FF0000FF8000FF8000FF8000
0000FF0000FFFF8000FF8000FF0000FF8000FF8000FF8000FF0000FF8000FF8000FF0000
FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF80000000FF0000FFFF0000
FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF0000FF80000000FF
0000FFFF8000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF80000000FFFF0000
FF80000000FFFF80000000FFFF8000FF0000FF0000FF0000FF0000FF0000FF8000FF8000
FF0000FF8000FF0000FF8000FF8000FF0000FF8000FF8000FF0000FF8000FF80000000FF
FF80000000FF0000FFFF80000000FF
FF8000FF8000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF0000
FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF00000000FFFF0000
FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF0000FF8000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF0000
FF80000000FF0000FFFF8000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000
FF0000FF8000FF0000FF8000FF80000000FF0000FFFF8000FF80000000FF0000FF0000FF
0000FFFF8000FF8000FF8000FF0000
FF80000000FFFF0000FF8000FF0000FF8000FF0000FF0000FF8000FF8000FF0000FF8000
0000FFFF0000FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF0000FF8000
FF8000FF0000FF0000FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF80000000FF
FF0000FF0000FF0000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF0000
FF0000FF0000FF80000000FFFF0000FF0000FF0000FF8000FF8000FF0000FF8000FF0000
FF0000FF0000FF0000FF8000FF80000000FFFF8000FF80000000FF0000FF0000FF0000FF
FF8000FF8000FF0000FF8000FF8000
FF8000FF0000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF0000FF8000FF8000FF0000FF8000FF8000FF8000FF0000FF80000000FFFF8000
FF8000FF0000FF0000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF80000000FFFF0000FF0000
FF0000FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF8000FF8000FF0000FF0000
FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF00000000FFFF80000000FF0000FF
FF0000FF00000000FFFF8000FF8000
FF0000FF0000FF8000FF00000000FFFF8000FF8000FF8000FF8000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF8000FF8000FF0000FF80000000FFFF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF0000FF8000FF80000000FFFF8000FF0000FF8000FF0000FF0000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF80000000FFFF8000FF8000FF8000
FF8000FF0000FF0000FF8000FF0000
FF0000FF0000FF8000FF8000FF8000FF8000FF00000000FFFF8000FF8000FF0000FF0000
FF8000FF0000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF0000FF0000FF8000
FF80000000FFFF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF0000
FF8000FF8000FF0000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF80000000FFFF8000FF8000FF8000
FF0000FF0000FF8000FF8000FF0000
FF0000FF0000FF8000FF0000FF8000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF8000FF0000FF8000FF0000FF8000FF0000FF8000FF80000000FFFF00000000FFFF0000
FF8000FF0000FF8000FF8000FF8000FF0000FF8000FF8000FF0000FF8000FF8000FF8000
FF80000000FF0000FFFF80000000FFFF8000FF8000FF0000FF8000FF80000000FF0000FF
0000FFFF0000FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF0000FF0000FF8000
FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF80000000FFFF8000FF8000FF0000
FF0000FF0000FF8000FF8000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000
FF8000FF0000FF8000FF8000FF8000FF0000FF0000FF80000000FFFF8000FF8000FF0000
FF8000FF0000FF8000FF0000FF8000FF0000FF8000FF8000FF0000FF8000FF80000000FF
FF8000FF0000FF0000FF8000FF8000FF0000FF8000FF8000FF0000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF8000FF0000FF8000
FF0000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF0000FF0000
FF8000FF0000FF8000FF8000FF0000
FF8000FF8000FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF0000
FF8000FF0000FF8000FF8000FF0000FF0000FF80000000FF0000FF0000FFFF8000FF8000
0000FFFF8000FF8000FF0000FF0000FF0000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF8000
FF8000FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF80000000FFFF8000FF8000
FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF80000000FFFF0000FF0000FF0000FF0000FF0000FF8000FF8000FF80000000FFFF0000
FF0000FF0000FF8000FF8000FF0000FF0000FF80000000FF0000FF0000FFFF8000FF8000
0000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000
FF0000FF0000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF80000000FFFF0000FF8000
FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF80000000FF
FF8000FF8000FF0000FF8000FF8000
FF00000000FF0000FFFF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF0000
FF0000FF0000FF8000FF8000FF8000FF0000FF8000FF8000FF80000000FFFF8000FF8000
FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF0000FF8000FF8000FF0000
FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF0000FF8000FF0000FF8000
FF8000FF8000FF0000FF8000FF8000FF0000FF8000FF8000FF8000FF0000FF8000FF8000
FF8000FF0000FF80000000FF0000FF
FF8000FF80000000FFFF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF80000000FFFF0000FF8000FF8000
FF0000FF8000FF8000FF8000FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF8000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF80000000FF0000FF0000FF
FF8000FF0000FF8000FF8000FF0000FF8000FF0000FF8000FF0000FF8000FF8000FF8000
FF0000FF8000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF00000000FF
FF8000FF80000000FFFF0000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000
FF0000FF80000000FFFF0000FF0000FF0000FF80000000FF0000FF0000FFFF0000FF0000
FF0000FF00000000FFFF80000000FFFF0000FF0000FF0000FF8000FF8000FF80000000FF
FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF0000
FF8000FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF80000000FF
FF8000FF80000000FFFF8000FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF80000000FFFF8000
FF8000FF0000FF0000FF8000FF80000000FFFF8000FF8000FF8000FF0000FF8000FF0000
FF0000FF80000000FFFF0000FF0000FF0000FF80000000FFFF8000FF8000FF0000FF0000
FF0000FF0000FF80000000FFFF8000FF0000FF8000FF0000FF8000FF8000FF8000FF0000
FF8000FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF8000FF0000
FF0000FF8000FF0000FF0000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF0000
FF8000FF8000FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF8000FF8000FF80000000FFFF8000
FF0000FF0000FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF0000FF0000FF8000
FF0000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000
FF0000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF80000000FFFF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000
FF0000FF0000FF0000FF8000FF8000FF8000FF00000000FFFF8000FF0000FF00000000FF
FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF0000
FF8000FF8000FF80000000FFFF8000
FF0000FF8000FF8000FF8000FF80000000FFFF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF80000000FFFF8000FF80000000FFFF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF80000000FFFF0000FF0000FF0000
FF8000FF8000FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF0000
FF8000FF0000FF0000FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF0000FF0000
FF8000FF8000FF0000FF8000FF8000FF8000FF0000FF0000FF8000FF0000FF0000FF0000
FF8000FF8000FF8000FF0000FF8000
FF0000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF0000FF8000FF0000
FF0000FF80000000FFFF8000FF8000FF8000FF8000FF80000000FFFF0000FF8000FF0000
FF0000FF8000FF0000FF0000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF8000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000
FF8000FF8000FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF8000
FF8000FF80000000FFFF8000FF8000
FF8000FF80000000FFFF8000FF80000000FFFF0000FF8000FF8000FF0000FF8000FF8000
FF0000FF8000FF80000000FFFF80000000FF0000FFFF8000FF8000FF0000FF8000FF8000
FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000
FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF0000
FF0000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000
FF0000FF00000000FF0000FF0000FF0000FFFF8000FF8000FF0000FF0000FF0000FF8000
FF8000FF0000FF8000FF8000FF8000
0000FF0000FFFF80000000FFFF80000000FFFF8000FF0000FF0000FF0000FF8000FF8000
FF80000000FF0000FF0000FF0000FFFF80000000FFFF0000FF0000FF0000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF80000000FFFF8000FF8000FF0000FF8000FF8000
FF8000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF80000000FF
FF8000FF8000FF8000FF0000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000
FF80000000FF0000FFFF80000000FFFF8000FF8000FF8000FF8000FF80000000FFFF8000
FF8000FF8000FF8000FF8000FF8000
0000FFFF8000FF80000000FFFF00000000FFFF8000FF8000FF0000FF0000FF8000FF0000
FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF0000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF0000FF80000000FF0000FFFF8000
FF80000000FFFF8000FF8000FF0000FF0000FF8000FF0000FF80000000FF0000FFFF8000
FF0000FF8000FF8000FF8000FF0000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.276299 0.261905 0.516558 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .45388 L
.2381 .45388 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .45388 L
.2381 .45388 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .01647 L
.47619 .01647 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .01647 L
.47619 .01647 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .14413 L
.71429 .14413 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .14413 L
.71429 .14413 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .02732 L
.95238 .02732 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .02732 L
.95238 .02732 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
0000FF0000FF0000FFFF00000000FFFF0000FF0000FF0000FF0000FF0000FF00000000FF
0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF80000000FFFF0000
FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF8000FF0000
FF80000000FFFF0000FF0000FF0000FF0000FF8000FF00000000FFFF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF8000
FF0000FF00000000FFFF0000FF00000000FF0000FFFF0000FF00000000FFFF00000000FF
FF0000FF0000FF0000FF00000000FFFF00000000FFFF0000FF0000FF80000000FF0000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000
FF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000FF8000FF8000FF8000FF0000
FF0000FF0000FF8000FF0000FF0000
0000FFFF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF00000000FF0000FF0000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF0000FF0000FF0000
FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
0000FFFF0000FF00000000FF0000FF0000FFFF0000FF0000FF8000FF8000FF80000000FF
FF0000FF0000FF8000FF8000FF8000
FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FFFF0000FF0000FF0000FF8000FF00000000FF0000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF8000
0000FFFF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF00000000FFFF0000
0000FFFF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000FF00000000FF0000FF
FF0000FF0000FF0000FF8000FF8000
FF00000000FFFF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FFFF0000FF0000FF8000FF8000FF0000FF0000FF0000
FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF8000FF8000
0000FF0000FFFF0000FF8000FF00000000FFFF0000FF0000FF0000FF00000000FF0000FF
0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FF
0000FFFF0000FF0000FF0000FF8000
FF00000000FFFF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FFFF00000000FFFF0000FF8000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF
0000FFFF0000FF0000FF0000FF0000FF0000FF00000000FFFF00000000FFFF0000FF0000
FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000FF00000000FF
FF0000FF00000000FFFF0000FF8000FF0000FF0000FF00000000FFFF0000FF00000000FF
FF0000FF00000000FFFF00000000FF
FF00000000FFFF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00000000FF
FF0000FF00000000FFFF00000000FFFF0000FF0000FF00000000FFFF0000FF0000FF0000
FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF00000000FF
FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF00000000FFFF0000
FF0000FF0000FF00000000FFFF0000
0000FFFF00000000FFFF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FFFF0000FF0000
FF0000FF00000000FFFF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FFFF00000000FFFF0000FF0000FF0000FF0000FF00000000FF
FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FF
0000FFFF00000000FFFF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000FF00000000FFFF0000FF0000
FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
0000FF0000FF0000FFFF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF00000000FF
0000FF0000FF0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FF
0000FF0000FF0000FF0000FFFF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF00000000FFFF00000000FFFF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF00000000FF0000FF
0000FFFF00000000FFFF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
0000FFFF00000000FFFF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000
0000FFFF0000FF00000000FFFF0000FF0000FF00000000FFFF0000FF00000000FFFF0000
FF0000FF0000FF00000000FFFF0000
FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF00000000FF0000FFFF00000000FFFF0000FF0000FF0000FF0000FF00000000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FF
FF0000FF0000FF00000000FF0000FF
0000FF0000FF0000FFFF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF00000000FFFF00000000FF
FF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF00000000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF80000000FFFF0000FF0000FF0000
0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FFFF0000
0000FF0000FF0000FFFF0000FF0000FF00000000FFFF0000FF0000FF8000FF00000000FF
FF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000FF00000000FFFF0000FF0000
FF00000000FF0000FF0000FF0000FFFF0000FF0000FF80000000FFFF0000FF0000FF0000
FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF00000000FFFF0000FF0000FF0000FF0000FF00000000FF0000FFFF00000000FF
0000FFFF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
0000FF00FF00FF0000FF0000FF0000FF00000000FFFF0000FF00000000FFFF00000000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FFFF00000000FFFF0000FF0000FF00000000FF0000FFFF0000
FF0000FF00000000FFFF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FFFF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000
FF0000FF0000FF0000FF80000000FFFF0000FF0000FF0000FF0000FF00000000FFFF0000
FF0000FF0000FF8000FF0000FF0000
00FF0000FF00FF0000FF0000FF0000FF00000000FFFF00000000FF0000FFFF00000000FF
FF0000FF00000000FFFF00000000FF0000FFFF0000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FFFF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF00000000FFFF0000
FF0000FF8000FF8000FF80000000FF0000FFFF0000FF0000FF00000000FF0000FF0000FF
FF0000FF0000FF8000FF8000FF0000
00FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF
FF0000FF00000000FFFF00000000FFFF00000000FFFF00000000FFFF0000FF0000FF0000
0000FF0000FFFF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FF0000FF0000FFFF0000FF00000000FFFF00000000FF
FF0000FF0000FF0000FF80000000FF
00FF0000FF000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FFFF00000000FFFF00000000FFFF0000FF00000000FF0000FFFF0000
FF0000FF00000000FF0000FFFF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
0000FF0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FFFF0000FF0000FF00000000FF0000FFFF0000FF0000FF00000000FF
FF0000FF0000FF0000FF00000000FF
00FF000000FFFF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FF0000FFFF00000000FFFF0000FF00000000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF
0000FFFF0000FF0000FF00000000FF
00FF000000FFFF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000FF00000000FF
0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
0000FF0000FFFF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000
0000FF0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF00000000FF0000FFFF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FFFF0000FF0000
00FF000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF
0000FFFF00000000FF0000FF0000FFFF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF8000FF0000
0000FFFF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FFFF0000FF80000000FFFF0000FF00000000FF0000FFFF0000FF0000
FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
00FF0000FF00FF0000FF0000FF80000000FFFF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FF0000FFFF0000FF0000FF00000000FFFF00000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF
0000FFFF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF00000000FF
FF00000000FF0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000
FF0000FF8000FF0000FF0000FF0000
00FF00FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF00000000FFFF0000FF00000000FF0000FFFF00000000FFFF0000FF0000
FF0000FF00000000FFFF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF80000000FFFF0000FF0000FF0000FF00000000FF
0000FFFF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000
FF0000FF0000FF80000000FFFF0000
00FF00FF0000FF0000FF00000000FFFF00000000FFFF0000FF00000000FF0000FF0000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF8000
0000FFFF0000FF0000FF0000FF0000FF00000000FFFF00000000FFFF0000FF00000000FF
0000FFFF0000FF00000000FFFF0000FF00000000FF0000FFFF0000FF00000000FF0000FF
FF0000FF00000000FFFF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FF0000FF
00FF000000FFFF00000000FFFF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000
FF0000FF00000000FF0000FFFF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF80000000FF
0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
0000FFFF0000FF0000FF00000000FFFF00000000FFFF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
0000FFFF0000FF00000000FF0000FF
0000FF0000FFFF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FFFF0000FF0000FF0000FF00000000FF0000FFFF00000000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF
0000FFFF0000FF00000000FF0000FF0000FF0000FFFF0000FF0000FF0000FF00000000FF
FF00000000FFFF0000FF0000FF00000000FF0000FFFF00000000FFFF0000FF0000FF0000
FF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FF
0000FF0000FFFF00000000FF0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FFFF00000000FFFF0000FF0000FF0000FF00000000FFFF0000
0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
0000FF0000FFFF00000000FF0000FF0000FF0000FFFF0000FF0000FF0000FF00000000FF
FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000
FF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
0000FFFF0000FF00000000FFFF0000FF0000FF0000FF0000FF80000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF00000000FF0000FFFF0000
0000FF0000FFFF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF00000000FF
FF0000FF0000FF0000FF00000000FFFF00000000FFFF0000FF0000FF00000000FF0000FF
FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000
FF00000000FF0000FFFF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FF
0000FFFF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
0000FFFF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF00000000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF
FF0000FF00000000FFFF0000FF0000FF00000000FFFF0000FF00000000FF0000FFFF0000
FF000000FF000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FF
0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000
FF00000000FF0000FFFF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
0000FFFF0000FF00000000FFFF00000000FF0000FFFF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000
0000FF0000FF0000FFFF0000FF00000000FF0000FF0000FFFF00000000FF0000FFFF0000
FF00000000FF0000FFFF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000
FF0000FF80000000FFFF0000FF0000
0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF00000000FF0000FF0000FFFF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000
0000FFFF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000
FF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000
FF0000FF00000000FFFF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
FF0000FF00000000FFFF0000FF0000
0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF00000000FF
0000FFFF00000000FF0000FFFF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000
0000FFFF0000FF8000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000
FF0000FF0000FF00000000FF0000FF0000FFFF0000FF8000FF80000000FF0000FFFF0000
FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FFFF0000FF00000000FF
FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF
FF0000FF00000000FFFF0000FF0000
0000FF0000FFFF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000
0000FFFF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF00000000FF
0000FFFF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000FF00000000FF0000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FFFF0000FF000000FF00
FF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000
0000FFFF00000000FF0000FFFF00000000FFFF0000FF00000000FFFF0000FF0000FF0000
0000FF0000FF0000FFFF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
0000FFFF00000000FF0000FF0000FF0000FFFF0000FF00000000FFFF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF00000000FFFF0000
FF00000000FF0000FFFF0000FF00000000FF0000FFFF00000000FFFF0000FF00000000FF
0000FF0000FF0000FF0000FFFF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000
FF0000FF00000000FF0000FFFF0000
FF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
0000FF0000FF0000FFFF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
0000FFFF00000000FFFF00000000FF0000FFFF0000FF00000000FFFF00000000FF0000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF00000000FF0000FF
FF00000000FF0000FFFF0000FF00000000FF0000FFFF0000FF00000000FF0000FF0000FF
0000FF0000FFFF00000000FF0000FFFF0000FF00000000FF0000FFFF0000FF0000FF0000
FF00000000FFFF0000FF0000FF0000
0000FF0000FF0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
0000FF0000FF0000FFFF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FF0000FF0000FFFF0000FF0000FF00000000FFFF0000FF00000000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000
FF00000000FF0000FFFF0000FF00000000FF0000FF0000FF0000FF00FF00FF0000FF0000
0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000
FF00000000FF0000FF0000FFFF0000
0000FF0000FFFF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000
00FF00FF00000000FFFF0000FF0000FF0000FF0000FF0000FF00000000FFFF00000000FF
FF0000FF0000FF0000FF00000000FFFF0000FF0000FF00000000FF0000FF0000FF0000FF
FF0000FF00000000FF0000FF0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FFFF00000000FF0000FF0000FFFF0000FF00000000FF0000FFFF0000
FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000
FF0000FF00000000FF0000FFFF0000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.261905 0.276299 0.5 0.516558 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .07217 L
.2381 .07217 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .07217 L
.2381 .07217 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .27012 L
.47619 .27012 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .27012 L
.47619 .27012 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .28576 L
.71429 .28576 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .28576 L
.71429 .28576 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .01376 L
.95238 .01376 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .01376 L
.95238 .01376 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
0000FF0000FF0000FFFF0000FF00000000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FFFF0000FF00000000FF00FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FFFF0000FF00000000FFFF0000FF00000000FF
0000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF000000FF00FF000000FF0000FF0000FF0000FFFF00000000FF0000FF0000FF0000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000FF00000000FF
0000FF0000FF0000FFFF00000000FF
0000FF0000FFFF00000000FFFF00000000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FFFF0000FF00000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000
0000FF0000FFFF0000FF0000FF00000000FF0000FF0000FF0000FF0000FF0000FF00FF00
0000FF0000FF00FF00FF00000000FF0000FF0000FF0000FF0000FFFF00000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FF0000FF
FF00000000FF0000FF0000FF0000FF
0000FF0000FFFF00000000FFFF0000FF000000FF000000FF00FF0000FF0000FF00FF0000
0000FF0000FFFF000000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
FF00000000FFFF0000FF0000FF000000FF000000FF0000FF00FF0000FF000000FF00FF00
00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
00FF000000FF00FF000000FF0000FFFF0000FF00000000FF0000FFFF0000FF00000000FF
0000FFFF000000FF0000FF00FF0000
0000FF0000FF0000FF0000FF0000FFFF000000FF0000FF000000FF0000FF00FF000000FF
0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF00FF000000FF00FF0000FF00
0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF
FF0000FF0000FF0000FF0000FF00000000FFFF00000000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF
0000FF0000FF0000FFFF000000FF000000FFFF00000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FFFF0000
0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF00
FF0000FF00000000FF0000FF0000FFFF0000FF00000000FF0000FF0000FFFF000000FF00
0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FFFF00000000FF
0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF00
0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF000000FF
0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF00FF000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
FF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF000000FF00FF00FF00000000FF
0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF
00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000
0000FF00FF0000FF0000FF0000FF000000FF0000FFFF0000FF000000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FFFF0000FF0000FF000000FF00
00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FFFF00000000FF0000FF0000FF00FF0000FF000000FF
0000FFFF0000FF00000000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FFFF0000FF00000000FF0000FF00FF00
0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FFFF0000FF0000FF000000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF00000000FF0000FFFF00000000FF0000FF0000FF0000FFFF00000000FF
0000FFFF0000FF00000000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF000000FF0000FF00FF0000FF000000FF0000FFFF00000000FF0000FF0000FF
0000FF00FF000000FF0000FF0000FF0000FFFF0000FF0000FF00000000FFFF0000FF0000
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00000000FF0000FFFF000000FF00
00FF0000FF0000FF000000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF00000000FF00FF0000FF00
FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FFFF00000000FF0000FFFF00000000FF0000FFFF0000
0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FFFF00000000FF0000FFFF0000FF0000FF8000FF8000FF80000000FF0000FFFF0000
00FF0000FF000000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FF
0000FF0000FF0000FF00FF0000FF00
FF000000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00FF00000000FF0000FF0000FFFF0000FF0000FF0000
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FFFF0000FF0000FF0000FF80000000FFFF0000FF0000
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF00FF000000FFFF0000FF00000000FF
FF0000FF00000000FFFF000000FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FFFF0000FF0000FF0000FF00000000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF000000FF0000FFFF0000FF00000000FF0000FFFF00000000FFFF0000
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF000000FF00FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF00FF000000FFFF0000FF0000FF0000
FF0000FF00000000FF00FF0000FF00
00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FFFF0000FF000000FF0000FF000000FF0000FF0000FF0000FF0000FFFF000000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF00FF0000FF00000000FF0000FF0000FFFF0000FF0000
0000FFFF000000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000FF0000FF0000
0000FFFF0000FF00000000FFFF0000
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000
0000FFFF00000000FF0000FF00FF000000FFFF00000000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF000000FFFF00000000FF0000FFFF00000000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FFFF000000FF000000FF0000FFFF00000000FF
0000FF0000FF0000FF00FF000000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF00FF000000FFFF00000000FF0000FFFF000000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF000000FF000000FF00FF0000FF00FF00000000FFFF0000FF0000FF00000000FF
0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF000000FF00FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FFFF0000
0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF00FF0000FF0000FF00FF00000000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF00FF000000FF0000FF0000FFFF00000000FF0000FF0000FF0000FF00FF00
0000FF0000FF0000FF0000FFFF000000FF000000FF00FF0000FF000000FF00FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF00000000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF000000FF0000FF0000FF0000FF00FF00000000FF0000FF0000FF0000FF0000FF
00FF00FF00000000FF0000FFFF000000FF000000FF0000FF00FF00FF00000000FF00FF00
FF000000FF000000FF0000FF00FF0000FF0000FF0000FF000000FF00FF000000FFFF0000
0000FFFF00000000FF0000FFFF0000
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00
00FF000000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF
FF00000000FF00FF0000FF0000FF0000FF000000FF00FF0000FF000000FF0000FF0000FF
FF00000000FFFF00000000FF0000FF
0000FFFF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF00FF000000FF0000FF0000FF00FF000000FF0000FF00FF0000FF0000FF000000FF
00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF
0000FF00FF000000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FFFF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FFFF000000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FFFF0000FF0000FF0000
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF00FF0000FF0000FF00
0000FF00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
FF00000000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FFFF00000000FF0000FFFF000000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF000000FF0000FF0000FFFF0000FF0000FF0000
0000FF00FF000000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF00
0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF000000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FFFF00000000FF00FF00
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF0000FF0000FF00FF00FF00000000FF0000FFFF0000
0000FF0000FF0000FF00FF000000FF00FF000000FF00FF0000FF0000FF000000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FFFF000000FF000000FF
0000FF00FF0000FF0000FF000000FF00FF000000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FFFF00000000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF000000FF0000FFFF00000000FF0000FF0000FF00FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF000000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FFFF0000
0000FF0000FF0000FF0000FF0000FF
00FF0000FF000000FF0000FFFF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF00000000FF00FF0000FF00FF00000000FFFF0000FF000000FF0000FF0000FF00
0000FF0000FF00FF00FF000000FF0000FF0000FF0000FF000000FF0000FFFF000000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF0000FF
0000FF0000FF00FF0000FF000000FF
00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF
0000FFFF000000FF0000FF000000FF0000FF0000FFFF0000FF00000000FF0000FF0000FF
00FF000000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF00
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF000000FF0000FF
0000FF0000FF00FF000000FF0000FFFF00000000FF0000FF0000FF00FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF
0000FF0000FFFF000000FF0000FF00
00FF000000FF0000FF00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FFFF00000000FF00FF0000FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF
0000FF0000FF0000FFFF000000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF00
00FF0000FF00FF00000000FF0000FFFF00000000FF0000FF0000FF00FF000000FF00FF00
00FF000000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF00
0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FFFF00000000FF
FF00000000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF0000FFFF0000
0000FF0000FFFF00000000FFFF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00
0000FF0000FF00FF000000FF0000FF0000FF00FF0000FF000000FF00FF000000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF
FF00000000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF
FF0000FF0000FF0000FF0000FF00000000FFFF0000FF000000FF0000FF000000FF00FF00
0000FF00FF00FF00000000FF0000FFFF0000FF0000FF00000000FFFF0000FF000000FF00
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FFFF00000000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FF0000FF0000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF000000FF000000FF0000FF00FF00
00FF0000FF000000FF0000FF0000FFFF0000FF00000000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF00
00FF00FF00000000FF0000FFFF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF0000FF0000FF0000FF80000000FF0000FF00FF000000FF00FF000000FF
00FF000000FFFF00000000FF0000FF0000FF0000FFFF00000000FF0000FF0000FF00FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF00
0000FF0000FF0000FF0000FF00FF00
0000FFFF00000000FF0000FFFF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FF0000FF0000FF
FF00000000FFFF0000FF8000FF8000FF00000000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF000000FF00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF00FF00
0000FF0000FFFF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00000000FF0000FFFF0000
0000FF0000FFFF00000000FF0000FFFF80000000FF0000FFFF000000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF000000FF0000FF000000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
0000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF00FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF00FF00000000FF0000FFFF0000
FF00000000FFFF0000FF0000FF00000000FFFF00000000FF0000FFFF00000000FF0000FF
0000FF0000FF00FF000000FF0000FFFF00000000FF0000FFFF000000FF0000FF000000FF
0000FF00FF0000FF000000FF00FF000000FF00FF000000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00000000FF0000FF0000FF
0000FF0000FFFF0000FF0000FF00000000FFFF0000FF00000000FFFF0000FF000000FF00
0000FF0000FF0000FF0000FF0000FFFF00000000FF0000FF0000FFFF000000FF000000FF
0000FF00FF000000FF0000FF0000FF00FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FFFF00000000FF0000FF0000FF
FF00000000FF0000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FFFF00000000FF0000FF0000FFFF0000FF00000000FF
0000FF0000FF0000FF0000FFFF00000000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.276299 0.738095 0.516558 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .08406 L
.2381 .08406 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .08406 L
.2381 .08406 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .19669 L
.47619 .19669 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .19669 L
.47619 .19669 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .34875 L
.71429 .34875 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .34875 L
.71429 .34875 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .0123 L
.95238 .0123 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .0123 L
.95238 .0123 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
0000FF0000FF0000FF00FF000000FF00FF0000FF0000FF0000FF0000FF000000FF00FF00
00FF000000FF00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF000000FF
00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FFFF0000FF00000000FFFF0000FF00000000FF0000FF00FF000000FF
00FF0000FF000000FF00FF00FF00000000FFFF00000000FFFF0000FF00000000FF0000FF
0000FF0000FF00FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FFFF0000
0000FF00FF00FF00000000FF0000FF
0000FF0000FF0000FFFF000000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF00FF000000FF0000FF000000FF0000FF
FF000000FF000000FF0000FF0000FF00FF0000FF000000FFFF00000000FFFF0000FF0000
0000FFFF00000000FF0000FFFF0000FF00000000FF0000FF0000FF00FF0000FF000000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF000000FF
FF0000FF0000FF0000FF00000000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF0000FF000000FF00FF00
0000FF00FF000000FF00FF000000FF0000FF00FF000000FF0000FFFF00000000FF0000FF
0000FFFF000000FF0000FF000000FF0000FF0000FF0000FFFF00000000FF0000FF0000FF
00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FFFF000000FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
FF00000000FFFF0000FF00000000FF00FF0000FF0000FF0000FF000000FF0000FFFF0000
FF00000000FF0000FF0000FF00FF000000FF0000FF0000FF00FF000000FF0000FF0000FF
FF0000FF000000FF0000FF0000FF00FF00000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF00FF000000FF00
0000FF00FF000000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF00
0000FF0000FF00FF000000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
FF0000FF00000000FF0000FF0000FF0000FF00FF0000FF0000FF000000FFFF0000FF0000
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF00FF00000000FF
0000FFFF00000000FF0000FF0000FFFF00000000FFFF0000FF00000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FFFF0000FF0000FF000000FF00
00FF0000FF00FF00000000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF000000FF00FF000000FF00FF000000FF0000FF00FF000000FF
0000FF0000FF0000FF00FF0000FF00
FF0000FF00000000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FFFF0000
FF00000000FF0000FF0000FFFF00000000FF0000FFFF00000000FF00FF00FF00000000FF
0000FF0000FF0000FFFF00000000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF000000FFFF0000FF0000FF00000000FF0000FF00FF00
00FF000000FFFF00000000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF00FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF00FF00000000FF0000FFFF00000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FF0000FFFF00000000FF
0000FF0000FF0000FFFF00000000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FFFF00000000FF0000FFFF0000FF0000
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000
0000FF0000FFFF0000FF00000000FFFF00000000FFFF0000FF00000000FF0000FF00FF00
00FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF000000FF00FF0000FF00FF00000000FF0000FF0000FFFF0000
0000FFFF0000FF0000FF00000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000
FF00000000FF0000FF0000FFFF00000000FFFF00000000FF0000FF0000FF0000FFFF0000
0000FF0000FF0000FF00FF000000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF00000000FFFF0000FF0000FF0000FF0000FF0000FF00000000FF
0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FFFF00000000FF0000FF0000FFFF00000000FF0000FF00FF00FF0000
0000FF0000FFFF00000000FF0000FF00FF0000FF0000FF0000FF0000FF000000FFFF0000
FF00000000FF0000FF0000FF00FF000000FFFF00000000FFFF0000FF00000000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF000000FF00FF0000FF0000FF00000000FF0000FFFF000000FF00
0000FF0000FF0000FF0000FFFF0000
00FF0000FF000000FF00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF00
FF00000000FF0000FFFF00000000FF00FF0000FF000000FF0000FF0000FF0000FFFF0000
0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF00FF0000
0000FF0000FF0000FF00FF000000FF0000FF0000FFFF00000000FF0000FF0000FF0000FF
00FF0000FF0000FF000000FF0000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FFFF00000000FFFF00000000FF0000FF0000FF00FF00
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF00
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF00FF0000FF0000FF0000FF000000FF0000FF00FF00
00FF000000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF00FF00
0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FFFF000000FF00
00FF0000FF0000FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF000000FF0000FF00FF000000FF00FF0000FF000000FF
0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
00FF000000FF00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF00000000FFFF0000FF00000000FF00FF00
00FF000000FF0000FF00FF000000FF00FF00FF00000000FF0000FFFF00000000FF0000FF
00FF0000FF0000FF000000FF00FF000000FF00FF000000FF0000FF0000FF0000FF00FF00
00FF0000FF000000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FFFF00000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FFFF00000000FFFF0000FF00000000FF00FF00
0000FF0000FF00FF000000FF0000FF00FF0000FF000000FF0000FFFF0000FF00000000FF
FF000000FF000000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF00000000FF0000FFFF000000FF0000FF0000FF0000FF000000FF
0000FF0000FF00FF000000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF00
00FF0000FF000000FF0000FF0000FF0000FFFF0000FF00000000FFFF00000000FF00FF00
00FF000000FF0000FF0000FF0000FF00FF000000FFFF00000000FFFF0000FF00000000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FFFF0000FF0000FF0000
0000FFFF0000FF00000000FF0000FF
0000FF0000FF0000FFFF00000000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FFFF0000FF000000FF0000FF0000FF000000FF0000FF0000FF0000FFFF00000000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF0000FFFF00000000FF0000FF0000FFFF000000FF00
0000FF0000FF0000FF0000FF00FF0000FF0000FF00FF00000000FFFF0000FF00000000FF
0000FF00FF0000FF0000FF000000FF00FF0000FF000000FF0000FF0000FFFF0000FF0000
FF00000000FFFF00000000FF00FF00
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF0000FF000000FF
0000FFFF0000FF00000000FF0000FFFF00000000FF00FF000000FF0000FFFF00000000FF
0000FF0000FF0000FF00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF
00FF00FF00000000FF0000FF0000FF0000FFFF00000000FF0000FFFF00000000FF00FF00
0000FF0000FF00FF000000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FFFF00000000FF0000FF0000FF
FF0000FF00000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF000000FF
0000FF0000FFFF00000000FF0000FFFF000000FF000000FF0000FF0000FF0000FFFF0000
0000FFFF0000FF0000FF000000FF000000FF00FF0000FF0000FF0000FF0000FF000000FF
FF00000000FF00FF0000FF000000FF0000FFFF00000000FF0000FF0000FFFF0000FF0000
0000FF0000FFFF00000000FF00FF0000FF0000FF0000FF000000FF0000FFFF00000000FF
00FF000000FFFF00000000FFFF0000FF00000000FFFF00000000FF0000FF0000FF0000FF
0000FFFF00000000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF00FF00FF0000
0000FFFF0000FF00000000FF00FF00FF000000FF000000FF0000FFFF00000000FFFF0000
FF0000FF0000FF0000FF00000000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF
FF00000000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF000000FF00FF000000FF00FF000000FF00FF00FF00000000FF0000FF0000FF
00FF0000FF000000FF0000FFFF0000FF0000FF00000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FFFF0000
0000FFFF0000FF00000000FF0000FF0000FF0000FF0000FF0000FFFF00000000FFFF0000
FF00000000FFFF00000000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF
FF00000000FF0000FF0000FF0000FF00FF00FF00000000FF0000FF0000FF00FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FF0000FF0000FF
0000FF0000FF0000FF0000FFFF00000000FF0000FFFF00000000FF0000FFFF00000000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF000000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
FF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FFFF0000
FF80000000FFFF80000000FF0000FF0000FF00FF0000FF000000FF00FF0000FF000000FF
FF00000000FF0000FFFF00000000FF00FF00FF00000000FF0000FFFF00000000FF0000FF
0000FF0000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FF0000FFFF00000000FF
0000FF00FF000000FF0000FF0000FF
0000FF0000FFFF000000FF0000FF0000FF0000FF0000FF0000FF00FF00000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FFFF0000FF00000000FF0000FF
FF00000000FFFF0000FF00000000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FFFF00000000FF0000FF
0000FF0000FF0000FFFF0000FF00000000FFFF0000FF00000000FF0000FFFF00000000FF
0000FF00FF000000FF00FF000000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FFFF00000000FF0000FF00FF000000FF0000FFFF00000000FFFF0000
FF00000000FF0000FFFF00000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF0000FF0000FFFF00000000FFFF00000000FF0000FFFF0000FF0000FF0000FF0000
0000FF0000FF00FF00FF00000000FF0000FFFF00000000FFFF0000FF00000000FF00FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FFFF00000000FF0000FF0000FF0000FF
0000FF00FF000000FF0000FF00FF00
0000FF00FF000000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FFFF00000000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF
FF00000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000FF00000000FF0000FF
0000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF000000FF0000FFFF00000000FF0000FFFF00000000FF0000FFFF00000000FF
0000FF0000FF00FF0000FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF00FF0000FF0000FF000000FF00FF0000FF000000FF00FF0000FF000000FF0000FF
FF00000000FF0000FF0000FF0000FF0000FFFF00000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF00FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF00000000FF0000FF00FF0000FF000000FF0000FF0000FFFF00000000FF0000FF
00FF0000FF0000FF000000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF000000FF
0000FFFF000000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF00FF00FF00000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF00FF000000FF0000FF0000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00FF00000000FF0000FFFF0000
00FF000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF00000000FF
0000FFFF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF00FF0000FF0000FF00FF00000000FF00FF0000FF000000FF0000FF
0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF00FF0000FF00
00FF000000FFFF000000FF000000FFFF00000000FF00FF000000FF00FF00FF000000FF00
0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF00000000FF0000FF0000FF00FF000000FF00FF0000FF000000FF00FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF
FF00000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FFFF0000FF00000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF000000FF00FF0000FF00FF00000000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF000000FF0000FF
0000FF0000FFFF00000000FF0000FF0000FF00FF000000FF00FF0000FF000000FF0000FF
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF
0000FF0000FFFF000000FF0000FF0000FF000000FF00FF000000FF00FF0000FF000000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF00000000FF00FF0000FF0000FF000000FF00FF000000FF0000FF0000FF
0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF00FF000000FF00FF00
00FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF00FF000000FF0000FF0000FF
0000FF00FF0000FF0000FF000000FF00FF000000FF0000FF00FF00FF00000000FF00FF00
FF00000000FF0000FFFF000000FF0000FF000000FF00FF00FF000000FF000000FF00FF00
00FF0000FF0000FF000000FF0000FF00FF000000FF0000FF0000FFFF0000FF0000FF0000
FF000000FF000000FF0000FF00FF0000FF000000FF00FF00FF00000000FF00FF0000FF00
0000FFFF00000000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FFFF0000
FF00000000FF0000FFFF00000000FF
0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FFFF000000FF00FF00000000FF0000FFFF000000FF0000FF000000FF
0000FFFF000000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FFFF000000FF0000FF000000FF0000FF00FF00FF0000
0000FF0000FF0000FF0000FF0000FFFF000000FF0000FF00FF0000FF00000000FFFF0000
FF00000000FF0000FF0000FF0000FF
0000FF0000FFFF000000FF0000FF000000FF00FF000000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF000000FF0000FFFF000000FF0000FF000000FF00FF0000FF00
0000FF0000FF0000FF0000FF00FF00FF00000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FFFF0000
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FF0000FFFF00000000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF00FF00000000FF00FF00FF00000000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF000000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FFFF0000FF0000
0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF0000FF0000FF0000FF00000000FF0000FF
0000FF00FF000000FF0000FF0000FF
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF0000FF000000FF00FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FFFF00000000FFFF00000000FF0000FFFF0000
FF00000000FF0000FFFF00000000FF0000FFFF000000FF000000FF0000FF0000FFFF0000
00FF0000FF000000FF0000FF00FF00FF00000000FFFF00000000FFFF0000FF00000000FF
FF000000FF000000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
FF00000000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF00FF000000FF000000FFFF00000000FFFF0000
FF00000000FF0000FFFF00000000FFFF0000FF000000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FFFF00000000FFFF0000FF00000000FF0000FF0000FF0000FF
00FF0000FF000000FF0000FF0000FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.738095 0.276299 0.97619 0.516558 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .05944 L
.2381 .05944 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .05944 L
.2381 .05944 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .01189 L
.47619 .01189 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .01189 L
.47619 .01189 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .01856 L
.71429 .01856 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .01856 L
.71429 .01856 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .55192 L
.95238 .55192 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .55192 L
.95238 .55192 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF8000FF0000FF8000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF80000000FFFF0000FF0000FF8000FF8000FF8000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF80000000FFFF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000
FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF8000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF8000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF80000000FFFF8000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF80000000FF0000FFFF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF0000
FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF
FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF80000000FFFF0000FF0000FF0000FF8000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF0000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF80000000FFFF0000FF8000
FF8000FF80000000FFFF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF0000FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF80000000FF0000FF0000FFFF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000
FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF0000FF0000
FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF80000000FFFF0000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF80000000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF00000000FFFF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF8000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF0000FF
FF8000FF0000FF80000000FF0000FF
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
0000FFFF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF0000FFFF0000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF80000000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF0000FF0000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF00000000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.516558 0.261905 0.756818 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .4664 L
.2381 .4664 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .4664 L
.2381 .4664 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .05965 L
.47619 .05965 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .05965 L
.47619 .05965 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .03066 L
.71429 .03066 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .03066 L
.71429 .03066 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .0851 L
.95238 .0851 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .0851 L
.95238 .0851 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF00000000FF00FF00FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00
00FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF8000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF80000000FF0000FF0000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000
FF0000FF0000FF8000FF00000000FF
FF00000000FFFF0000FF000000FF0000FF00FF8000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF000000FF00
FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00000000FFFF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF00000000FF0000FF0000FFFF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00
00FF0000FF00FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF8000
FF00000000FFFF8000FF8000FF8000
FF00000000FF00FF0000FF0000FF00FF0000FF8000FF0000FF0000FF8000FF0000FF8000
FF0000FF0000FF8000FF8000FF0000FF00000000FFFF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF80000000FFFF0000FF8000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF000000FF00
00FF0000FF0000FF00FF0000FF0000FF0000FF8000FF8000FF8000FF0000FF0000FF8000
FF0000FF0000FF8000FF8000FF8000
00FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF80000000FFFF0000FF0000FF0000FF8000FF0000FF8000FF0000FF0000
00FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF80000000FF0000FFFF8000FF80000000FFFF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF00000000FF
00FF0000FF0000FF00FF8000FF8000FF0000FF80000000FFFF8000FF0000FF0000FF8000
FF0000FF8000FF0000FF0000FF0000
00FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FF00FF00FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000
FF0000FF0000FF8000FF0000FF80000000FF0000FFFF8000FF8000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF000000FF0000FF00FF0000FF8000FF80000000FFFF8000FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF0000FF0000
00FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FFFF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF8000FF8000FF0000FF0000
FF0000FF0000FF8000FF0000FF8000FF8000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF8000FF00000000FFFF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF0000FF0000
00FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF8000FF8000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF0000FF0000FF8000FF8000FF0000FF8000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF8000FF0000
FF00000000FFFF0000FF8000FF8000
00FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
00FF00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF80000000FF00FF00FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF8000
00FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF000000FF0000FF00
FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF00000000FFFF8000FF0000
FF0000FF0000FF0000FF8000FF0000FF00000000FF00FF0000FF00FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000
FF0000FF8000FF8000FF0000FF8000
00FF00FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF000000FF00FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF8000FF0000
FF0000FF0000FF0000FF8000FF0000FF00000000FF00FF00FF0000FF0000FF0000FF0000
FF00000000FFFF0000FF0000FF0000FF0000FF0000FF8000FF0000FF00000000FFFF0000
FF0000FF8000FF8000FF0000FF8000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF8000FF0000FF8000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF8000FF0000
FF0000FF8000FF0000FF8000FF0000FF00000000FF00FF00FF8000FF0000FF0000FF0000
FF000000FF00FF0000FF0000FF0000FF8000FF0000FF8000FF0000FF0000FF8000FF8000
FF0000FF8000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF000000FF00FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF00000000FF0000FFFF000000FF00FF0000FF0000FF0000
FF00000000FFFF0000FF0000FF0000FF8000FF0000FF8000FF8000FF0000FF8000FF0000
FF0000FF80000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF00000000FF00FF0000FF00FF0000FF0000FF0000
FF000000FF00FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF000000FF00FF0000
0000FFFF80000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF00000000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF80000000FF00FF0000FF00FF0000FF0000
FF000000FF00FF0000FF0000FF0000FF80000000FF0000FF0000FFFF0000FF0000FF8000
0000FF0000FFFF8000FF0000FF0000
FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF00FF0000
FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF8000FF0000FF0000FF0000FF0000
0000FF0000FF0000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF00000000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000
FF0000FF0000FF8000FF0000FF0000FF000000FF00FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF8000FF8000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF00000000FF00FF00
00FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF00FF0000FF00
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF0000FF0000
00FF00FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
00FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF00000000FF00FF00FF0000
FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000
FF00000000FFFF0000FF0000FF0000
00FF00FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
00FF00FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000
FF000000FF00FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000
FF00000000FFFF8000FF0000FF0000
00FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF000000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF80000000FFFF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000
00FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF000000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000
FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF80000000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
00FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000
FF0000FF00000000FF00FF00FF0000FF000000FF00FF000000FF00FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF80000000FFFF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000
00FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000
FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF0000FF00FF0000FF0000
FF8000FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FF
FF8000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000
FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000
FF8000FF0000FF0000FF8000FF8000FF0000FF8000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF800000FF00
FF000000FF00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF0000FF0000FF8000FF0000FF0000
FF8000FF0000FF0000FF8000FF0000
FF000000FF00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF000000FF00FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF000000FF00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000
FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000
FF8000FF0000FF80000000FFFF0000
00FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00
00FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF000000FF00FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF000000FF00FF0000
00FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF00FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000
FF000000FF00FF0000FF0000FF8000FF0000FF0000FF8000FF8000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF8000FF0000FF8000
FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF000000FF00FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF00000000FFFF8000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF8000FF0000FF0000
FF000000FF00FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF8000FF0000FF0000
0000FF00FF00FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF0000FF0000FF0000FF8000FF0000
FF0000FF000000FF00FF0000FF000000FF0000FF00FF0000FF0000FF8000FF0000FF0000
FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000
0000FFFF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF8000FF0000
FF000000FF0000FF00FF000000FF00FF000000FF00FF000000FF00FF0000FF0000FF0000
FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000
FF0000FF0000FF0000FF0000FF0000
00FF00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000
00FF00FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF8000FF00000000FFFF0000FF0000FF0000FF8000FF0000
FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
00FF0000FF00FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF800000FF00FF0000FF0000
00FF0000FF00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FFFF8000FF0000FF8000FF8000FF0000FF0000FF000000FF00
FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000
00FF0000FF00FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF00000000FFFF0000FF0000FF0000
00FF0000FF00FF000000FF0000FF00FF0000FF0000FF00000000FF00FF00FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00
FF0000FF0000FF00000000FFFF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000
FF000000FF0000FF0000FF00FF0000
00FF00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF00
00FF00FF0000FF0000FF0000FF0000FF0000FF800000FF00FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FFFF0000FF0000FF0000FF8000FF0000FF000000FF00FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF000000FF00FF000000FF00FF0000
00FF00FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF00000000FF0000FFFF0000FF0000FF8000FF0000FF0000FF000000FF0000FF00
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF000000FF0000FF00FF0000FF0000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.261905 0.516558 0.5 0.756818 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .19002 L
.2381 .19002 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .19002 L
.2381 .19002 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .36711 L
.47619 .36711 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .36711 L
.47619 .36711 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .0486 L
.71429 .0486 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .0486 L
.71429 .0486 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .03608 L
.95238 .03608 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .03608 L
.95238 .03608 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF80000000FF00FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000
FF0000FF800000FF0000FF0000FF00FF0000FF0000FF00000000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000
FF00000000FF00FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF00
FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF80000000FF
FF80000000FF00FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000
FF0000FF000000FF0000FF0000FF0000FF00FF000000FF000000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000
FF00000000FF00FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF8000FF0000
FF80000000FF00FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000
FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FFFF8000
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000
FF00000000FF00FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF8000FF0000
FF0000FF8000FF8000FF8000FF8000
FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF0000
FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00000000FF0000FF0000FFFF8000
00FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF00FF0000
FF00000000FF00FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF00
FF0000FF000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF00000000FF0000FF0000FF0000FFFF0000
FF0000FF8000FF8000FF8000FF0000
FF000000FF0000FF0000FF00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000
FF000000FF0000FF0000FF0000FF0000FF00FF000000FF000000FF0000FF0000FFFF8000
00FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF0000FF0000
FF000000FF0000FF00FF0000FF0000FF000000FF00FF0000FF0000FF000000FF00FF0000
FF0000FF000000FF0000FF00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000
FF0000FF000000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FFFF8000FF0000
FF0000FF8000FF0000FF8000FF0000
00FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000
FF0000FF000000FF0000FF0000FF0000FF00FF000000FF000000FF0000FF0000FFFF8000
00FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF0000FF0000
FF000000FF0000FF00FF0000FF0000FF000000FF00FF0000FF0000FF000000FF00FF0000
FF0000FF00000000FF00FF00FF0000FF000000FF0000FF0000FF00FF000000FF00FF0000
FF0000FF000000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FFFF8000FF0000
0000FFFF80000000FFFF8000FF0000
00FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF00FF0000FF0000
FF000000FF0000FF0000FF0000FF0000FF00FF000000FF000000FF0000FF0000FFFF8000
00FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF00FF0000
FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000
FF8000FF00000000FF00FF0000FF00FF000000FF0000FF00FF000000FF0000FF0000FF00
FF0000FF000000FF0000FF0000FF0000FF0000FF00FF8000FF00000000FF0000FFFF0000
0000FFFF80000000FFFF8000FF0000
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000
FF000000FF0000FF0000FF0000FF00FF0000FF000000FF000000FF0000FF0000FFFF8000
00FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF0000
FF000000FF0000FF00FF8000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000
FF8000FF00000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
FF000000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00000000FFFF0000
0000FFFF80000000FFFF8000FF0000
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00FF0000FF000000FF000000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000
FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000
FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF8000FF0000
00FF0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF00FF0000
FF000000FF0000FF0000FF00FF000000FF00FF0000FF0000FF0000FF000000FF0000FF00
FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000
FF0000FF8000FF0000FF8000FF0000
00FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF0000
FF000000FF0000FF00FF0000FF0000FF0000FF8000FF0000FF0000FF000000FF0000FF00
FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000
FF00000000FFFF8000FF0000FF0000
00FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00FF000000FF0000FF00FF00000000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF0000FF0000
FF000000FF0000FF00FF0000FF0000FF0000FF8000FF0000FF00000000FF00FF0000FF00
FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF0000FF0000FF0000FF000000FF0000FF00FF0000FF00000000FFFF0000
FF00000000FFFF8000FF0000FF0000
00FF0000FF00FF0000FF000000FF00FF0000FF000000FF0000FF0000FF0000FF00FF0000
00FF0000FF0000FF0000FF00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF8000
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000
FF0000FF000000FF00FF0000FF0000FF00000000FF0000FF0000FF0000FF00FF0000FF00
FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF0000FF0000FF000000FF0000FF0000FF00FF0000FF00000000FFFF0000
FF0000FF0000FF8000FF0000FF0000
00FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000
FF000000FF0000FF0000FF00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF8000
00FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF00FF000000FF00
FF000000FF0000FF00FF0000FF00000000FF0000FF0000FF0000FF0000FF00FF0000FF00
FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF0000FF000000FF0000FF0000FF00FF0000FF0000FF00000000FF00FF00
0000FFFF0000FF0000FF0000FF0000
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000
FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF00FF0000FF0000FF0000
00FF00FF000000FF00FF0000FF0000FF00000000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
FF000000FF00FF0000FF000000FF0000FF0000FF00FF0000FF0000FF00000000FFFF0000
0000FF00FF0000FF00FF0000FF0000
FF000000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000
FF000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF00FF000000FF00FF0000FF0000FF00000000FF0000FF0000FFFF800000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF00
FF0000FF0000FF00000000FF00FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000
0000FF00FF0000FF0000FF00FF0000
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000
FF000000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000
FF000000FF0000FF0000FF00FF0000FF00000000FF0000FF0000FFFF800000FF0000FF00
00FF0000FF00FF0000FF00000000FF00FF0000FF0000FF0000FF0000FF00FF0000FF0000
FF0000FF0000FF00000000FF00FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000
0000FF00FF0000FF0000FF0000FF00
00FF0000FF00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000
00FF0000FF0000FF0000FF00FF0000FF0000FF00000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF8000
FF0000FF00000000FF00FF0000FF00FF00000000FF0000FF0000FFFF800000FF0000FF00
00FF0000FF00FF0000FF00000000FF00FF0000FF0000FF0000FF0000FF00FF0000FF0000
FF0000FF0000FF00000000FF00FF00FF0000FF000000FF0000FF00FF0000FF0000FF0000
0000FF00FF0000FF0000FF00FF0000
00FF00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF00
00FF0000FF0000FF0000FF00FF0000FF0000FF00000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF8000FF0000FF8000
FF8000FF00000000FF00FF0000FF00FF00000000FF0000FF0000FFFF800000FF00FF0000
00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00FF000000FF00
FF0000FF0000FF000000FF0000FF0000FF00FF0000FF000000FF00FF0000FF0000FF0000
0000FF00FF0000FF0000FF0000FF00
00FF00FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF00
00FF0000FF0000FF0000FF00FF0000FF0000FF00000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF8000FF0000FF8000
FF8000FF00000000FF00FF0000FF00FF00000000FF0000FF0000FFFF800000FF0000FF00
00FF0000FF00FF0000FF00000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF0000FF0000FF0000
00FF0000FF0000FF0000FF0000FF00
00FF00FF0000FF000000FF0000FF0000FF00FF0000FF000000FF0000FF00FF000000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF00
00FF0000FF0000FF00FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF8000
FF8000FF000000FF0000FF0000FF00FF00000000FF0000FF0000FFFF800000FF0000FF00
00FF0000FF00FF0000FF00000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF0000FF000000FF00
00FF0000FF0000FF0000FF0000FF00
FF0000FF0000FF000000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00
00FF0000FF0000FF00FF0000FF0000FF00000000FF00FF00FF0000FF8000FF0000FF8000
FF8000FF000000FF0000FF0000FF00FF00000000FF0000FF0000FFFF800000FF0000FF00
00FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF00FF0000
00FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF000000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF00
FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00000000FF00FF0000FF0000FF00
00FF0000FF0000FF00FF8000FF0000FF80000000FF00FF0000FF0000FF0000FF00FF0000
FF800000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FFFF800000FF0000FF00
FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00000000FF00FF00FF0000
00FF0000FF0000FF0000FF00FF0000
00FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00FF0000FF8000FF00000000FF00FF0000FF0000FF00
00FF0000FF0000FF00FF8000FF0000FF00000000FF00FF0000FF0000FF0000FF0000FF00
FF0000FF0000FF000000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF00
FF0000FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00
FF000000FF0000FF00FF0000FF0000FF0000FF8000FF00000000FF00FF0000FF0000FF00
00FF0000FF0000FF00FF0000FF0000FF00000000FF00FF0000FF0000FF0000FF00FF0000
FF000000FF00FF000000FF0000FF0000FF00FF00000000FF0000FFFF800000FF0000FF00
FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00
00FF0000FF0000FF00FF0000FF0000FF0000FF8000FF00000000FF00FF0000FF0000FF00
00FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00
FF000000FF0000FF0000FF0000FF0000FF00FF8000FF0000FF8000FF800000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF00
FF000000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00
00FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF00FF000000FF00
00FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF800000FF0000FF00
FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF00FF0000
FF000000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF8000
FF0000FF000000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF00000000FFFF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00
00FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000
0000FF00FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF00FF0000
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000
FF0000FF000000FF0000FF0000FF00
00FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF00000000FFFF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00
00FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF00FF8000FF0000
FF0000FF000000FF0000FF00FF0000FF0000FF0000FF800000FF00FF0000FF0000FF0000
0000FF00FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000
FF0000FF00000000FFFF0000FF0000
00FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FFFF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF00FF8000FF0000
FF000000FF00FF0000FF0000FF0000FF0000FF0000FF800000FF00FF0000FF0000FF0000
0000FF00FF0000FF0000FF00FF0000FF0000FF000000FF00FF000000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF0000
FF0000FF0000FF0000FF0000FF0000
FF000000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF8000FF0000
FF00000000FFFF0000FF0000FF0000FF8000FF0000FF800000FF0000FF00FF000000FF00
00FF0000FF0000FF0000FF00FF0000FF0000FF8000FF0000FF00000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF0000
FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF8000FF0000
FF00000000FFFF0000FF0000FF0000FF8000FF0000FF000000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF0000FF0000FF8000FF0000FF00000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF00
FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000FF0000
FF0000FF80000000FF0000FF0000FFFF8000FF0000FF000000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF0000FF0000FF8000FF0000FF0000FF800000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF00
00FF0000FF00FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000
FF0000FF80000000FF0000FFFF0000FF8000FF0000FF000000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF0000FF000000FF00FF0000FF0000FF000000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF00
00FF0000FF00FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FF00FF0000FF0000FF00FF000000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000
FF0000FF80000000FF0000FFFF8000FF8000FF0000FF000000FF0000FF0000FF0000FF00
00FF00FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF000000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF00
00FF0000FF00FF0000FF0000FF8000
0000FFFF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF00
00FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000
FF0000FF00000000FFFF0000FF0000FF8000FF0000FF000000FF0000FF0000FF00FF0000
FF0000FF00000000FFFF000000FF00FF000000FF0000FF00FF0000FF000000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF0000FF0000FF8000
FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF00000000FF00FF00
00FF00FF0000FF0000FF000000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF00
00FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000
00FF00FF0000FF0000FF0000FF8000FF8000FF0000FF00000000FFFF000000FF00FF0000
FF0000FF0000FF8000FF0000FF0000FF000000FF0000FF00FF0000FF000000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF0000FF0000FF0000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.516558 0.738095 0.756818 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .26699 L
.2381 .26699 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .26699 L
.2381 .26699 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .09031 L
.47619 .09031 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .09031 L
.47619 .09031 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .13474 L
.71429 .13474 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .13474 L
.71429 .13474 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .14976 L
.95238 .14976 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .14976 L
.95238 .14976 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
00FF0000FF0000FF00FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF8000FF0000FF00000000FF0000FF0000FFFF8000FF0000
FF00000000FF00FF0000FF00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF8000
FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF8000FF00000000FF0000FF0000FFFF80000000FF00FF00FF0000FF0000
FF00000000FF0000FF0000FF0000FFFF0000FF80000000FFFF80000000FF0000FF0000FF
FF8000FF00000000FF0000FFFF8000
00FF0000FF0000FF00FF0000FF0000FF0000FF00000000FFFF0000FF000000FF00FF0000
FF0000FF0000FF0000FF0000FF000000FF00FF80000000FF0000FF0000FFFF8000FF0000
FF0000FF0000FF000000FF00FF0000FF000000FF0000FF0000FF0000FF00FF8000FF0000
FF8000FF0000FF0000FF000000FF0000FF00FF8000FF8000FF0000FF8000FF8000FF0000
FF0000FF00000000FF0000FF0000FF0000FF0000FFFF8000FF0000FF0000FF0000FF8000
FF00000000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF80000000FF0000FF0000FFFF8000
00FF0000FF0000FF0000FF00FF0000FF0000FF00000000FFFF0000FF0000FF000000FF00
FF0000FF0000FF0000FF0000FF000000FF00FF00000000FF0000FFFF8000FF8000FF0000
FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF000000FF000000FFFF0000
FF8000FF0000FF0000FF0000FF000000FF000000FF0000FFFF8000FF8000FF8000FF8000
FF8000FF00000000FF0000FF0000FFFF8000FF8000FF8000FF0000FF0000FF00000000FF
0000FF0000FFFF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF0000FF0000
0000FF0000FF0000FFFF80000000FF
FF8000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FF00FF00FF0000FF80000000FFFF8000FF80000000FF
0000FF0000FFFF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF
0000FFFF8000FF0000FF0000FF0000FF00000000FF0000FF0000FFFF8000FF8000FF8000
FF8000FF0000FF80000000FF0000FFFF8000FF8000FF8000FF0000FF0000FF00000000FF
0000FF0000FF0000FFFF000000FF00FF8000FF0000FF0000FF000000FF00FF0000FF0000
FF8000FF8000FF8000FF8000FF8000
FF8000FF800000FF00FF000000FF00FF0000FF0000FF0000FF8000FF0000FF0000FF8000
FF8000FF0000FF0000FF0000FF000000FF00FF8000FF8000FF8000FF8000FF80000000FF
0000FF0000FFFF8000FF000000FF00FF0000FF0000FF000000FF00FF0000FF0000FF0000
FF8000FF8000FF8000FF0000FF0000FF80000000FF0000FF0000FF0000FFFF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF0000FF0000FF0000FF0000
FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF0000
FF0000FF8000FF8000FF8000FF8000
FF80000000FF00FF0000FF0000FF00FF0000FF00000000FF0000FFFF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF80000000FF
0000FF0000FFFF800000FF0000FF00FF000000FF00FF000000FF0000FF00FF0000FF0000
FF80000000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF
FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF8000FF8000FF0000FF0000
FF80000000FFFF0000FF00000000FFFF00000000FF0000FFFF8000FF0000FF8000FF0000
FF0000FF00000000FF0000FF0000FF
FF8000FF800000FF0000FF0000FF00FF0000FF0000FF80000000FF00FF00FF8000FF8000
FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF800000FF00FF0000FF000000FF00FF000000FF0000FF0000FF00FF0000
0000FF0000FF0000FFFF80000000FFFF80000000FF0000FF0000FF0000FFFF80000000FF
FF8000FF80000000FFFF8000FF00000000FF0000FF0000FFFF8000FF8000FF000000FF00
FF00000000FF0000FF0000FF0000FFFF00000000FF0000FFFF80000000FF0000FFFF8000
FF0000FF00000000FF0000FF0000FF
FF80000000FF00FF0000FF0000FF00FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF0000FF0000FF8000FF0000FF000000FF0000FF00FF000000FF0000FF0000FF00FF0000
0000FF0000FF0000FFFF8000FF8000FF80000000FF0000FF0000FF0000FFFF80000000FF
FF80000000FF0000FFFF8000FF00000000FF0000FF0000FF0000FF0000FF00FF0000FF00
FF0000FF00000000FFFF80000000FFFF00000000FFFF8000FF80000000FF0000FF0000FF
FF0000FF00000000FF0000FF0000FF
FF000000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF00
00FF00FF0000FF8000FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF0000
FF80000000FFFF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000
0000FF0000FF0000FF0000FFFF0000FF0000FF80000000FF0000FFFF8000FF8000FF8000
FF80000000FF00FF00FF8000FF0000FF80000000FF0000FF0000FFFF000000FF0000FF00
FF0000FF0000FF8000FF800000FF0000FF00FF0000FF8000FF8000FF8000FF80000000FF
00FF00FF0000FF80000000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF00
00FF00FF0000FF8000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF8000
FF8000FF8000FF000000FF00FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000
FF0000FF8000FF80000000FF00FF0000FF00FF00000000FF0000FFFF8000FF8000FF0000
FF00000000FF00FF00FF8000FF0000FF8000FF0000FF80000000FFFF0000FF0000FF0000
FF0000FF0000FF00000000FF00FF0000FF00FF0000FF0000FF8000FF8000FF80000000FF
00FF0000FF00FF8000FF00000000FF
FF000000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF0000FF8000FF0000
FF8000FF80000000FF00FF0000FF0000FF00FF0000FF0000FF0000FF0000FF8000FF0000
0000FFFF8000FF0000FF800000FF00FF0000FF00000000FF0000FFFF8000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000
FF0000FF8000FF000000FF0000FF0000FF000000FF0000FF0000FFFF8000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF000000FF0000FF00FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF8000FF8000FF800000FF00
FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF8000FF00000000FFFF0000
FF80000000FFFF8000FF0000FF0000FF8000FF00000000FFFF80000000FFFF0000FF0000
00FF00FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF8000
FF0000FF0000FF000000FF00FF0000FF0000FF8000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF000000FF0000FF0000FF00FF0000FF00000000FFFF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF0000FF0000
FF0000FF00000000FFFF8000FF00000000FF0000FFFF0000FF8000FF0000FF8000FF0000
FF80000000FF0000FFFF0000FF8000FF8000FF00000000FFFF8000FF8000FF0000FF0000
00FF00FF8000FF0000FF0000FF8000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF8000
FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF80000000FFFF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF000000FF00FF0000FF00000000FFFF0000
FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000
FF8000FF00000000FFFF0000FF00000000FF0000FF0000FFFF8000FF0000FF8000FF0000
FF8000FF80000000FF0000FF0000FFFF80000000FF0000FFFF8000FF80000000FFFF0000
FF0000FF8000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000
FF0000FF00000000FFFF8000FF0000FF8000FF0000FF0000FF80000000FF0000FFFF0000
0000FF0000FF0000FFFF8000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF8000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF8000
FF0000FF0000FF00000000FF0000FF0000FF0000FF0000FFFF8000FF0000FF0000FF0000
FF8000FF00000000FF0000FF0000FFFF80000000FF0000FFFF8000FF80000000FFFF0000
FF00000000FFFF0000FF0000FF8000
00FF00FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF0000FF8000
FF8000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF8000FF80000000FFFF0000
0000FF0000FF0000FFFF8000FF0000FF0000FF000000FF00FF000000FF00FF0000FF0000
FF0000FF80000000FFFF0000FF0000FF0000FF0000FF00000000FF0000FFFF8000FF8000
FF000000FF0000FF000000FF0000FFFF8000FF8000FF8000FF800000FF00FF0000FF0000
0000FFFF00000000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF0000FF0000FF0000FF80000000FF
00FF0000FF00FF000000FF00FF0000FF0000FF8000FF0000FF8000FF8000FF0000FF8000
FF00000000FFFF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF80000000FF
0000FF0000FF0000FFFF8000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000
FF8000FF8000FF8000FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FFFF8000
0000FFFF0000FF0000FF8000FF8000FF0000FF8000FF8000FF8000FF0000FF0000FF0000
0000FF0000FFFF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF0000
FF00000000FFFF8000FF00000000FF
FF0000FF000000FF0000FF0000FF00FF0000FF8000FF0000FF8000FF800000FF00FF0000
0000FF0000FFFF80000000FFFF0000FF0000FF0000FF8000FF0000FF80000000FF0000FF
0000FFFF8000FF8000FF8000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000
FF0000FF8000FF8000FF8000FF0000FF0000FF00000000FF0000FF0000FFFF8000FF8000
0000FFFF8000FF0000FF8000FF8000FF0000FF0000FF8000FF8000FF0000FF0000FF0000
0000FF0000FF0000FF0000FFFF000000FF00FF0000FF000000FF00FF00000000FF0000FF
0000FFFF8000FF0000FF80000000FF
FF0000FF000000FF0000FF0000FF0000FF00FF8000FF0000FF8000FF800000FF00FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF00000000FFFF8000FF00000000FF0000FF
0000FFFF8000FF8000FF8000FF000000FF0000FF0000FF0000FF00FF000000FF00FF0000
FF00000000FF0000FFFF80000000FFFF000000FF000000FF0000FF0000FFFF8000FF8000
FF0000FF8000FF0000FF8000FF8000FF0000FF0000FF8000FF0000FF80000000FF0000FF
0000FF0000FF0000FF0000FFFF0000FF000000FF0000FF0000FF00FF0000FF8000FF0000
FF8000FF80000000FF0000FF0000FF
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF8000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF00000000FF0000FF
0000FFFF8000FF8000FF800000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000
FF00000000FF0000FFFF80000000FF00FF0000FF000000FF0000FF0000FFFF8000FF8000
FF0000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF0000FF80000000FF0000FF
0000FFFF8000FF80000000FFFF8000FF0000FF0000FF000000FF00FF0000FF8000FF8000
FF8000FF80000000FF0000FF0000FF
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF8000FF0000FF8000FF0000FF0000FF00000000FF00FF00FF0000FF00000000FF0000FF
0000FFFF8000FF8000FF000000FF00FF0000FF000000FF0000FF0000FF0000FF000000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FFFF8000
FF0000FF8000FF0000FF8000FF8000FF0000FF0000FF8000FF0000FF80000000FF0000FF
0000FFFF8000FF8000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF8000FF80000000FF0000FFFF8000
00FF0000FF00FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF8000
0000FF00FF00FF0000FF0000FF0000FF80000000FF00FF00FF0000FF00000000FF0000FF
0000FFFF8000FF0000FF000000FF0000FF00FF0000FF000000FF0000FF0000FF000000FF
0000FF0000FFFF8000FF000000FF0000FF00FF00000000FF0000FF0000FFFF8000FF8000
FF0000FF8000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF8000
0000FFFF8000FF8000FF0000FF0000FF0000FF8000FF8000FF0000FF8000FF0000FF0000
FF0000FF00000000FF0000FFFF8000
00FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF8000
00FF0000FF00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF00000000FF0000FF
FF8000FF8000FF0000FF800000FF0000FF0000FF00FF0000FF000000FF0000FF000000FF
0000FF0000FF0000FFFF0000FF0000FF0000FF00000000FFFF80000000FFFF8000FF8000
FF0000FF8000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000
0000FFFF8000FF8000FF0000FF0000FF8000FF8000FF8000FF0000FF8000FF0000FF0000
FF8000FF0000FF80000000FFFF8000
00FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF8000FF0000FF0000
00FF00FF8000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00000000FF
FF8000FF8000FF00000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FFFF0000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000
0000FFFF80000000FF0000FFFF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF0000FF00000000FF00FF00FF0000FF0000FF8000FF0000FF0000
FF8000FF0000FF80000000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00000000FF
FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF00000000FF
0000FF0000FFFF00000000FF00FF0000FF0000FF0000FF00FF000000FF00FF00000000FF
0000FF0000FF0000FFFF0000FF0000FF8000FF8000FF0000FF0000FF00000000FF0000FF
0000FFFF8000FF8000FF80000000FF0000FFFF8000FF0000FF000000FF0000FF00FF0000
FF0000FF0000FF8000FF0000FF00000000FF00FF00FF0000FF80000000FF0000FFFF8000
FF8000FF0000FF80000000FF0000FF
0000FF00FF0000FF0000FF00FF000000FF0000FF00FF0000FF0000FF0000FF00000000FF
FF0000FF00000000FF0000FF00FF0000FF0000FF00FF0000FF8000FF00000000FF0000FF
0000FF0000FF0000FFFF800000FF0000FF00FF0000FF0000FF0000FF0000FF00000000FF
0000FF0000FF0000FFFF0000FF8000FF8000FF0000FF0000FF8000FF00000000FF0000FF
0000FFFF8000FF8000FF80000000FF0000FFFF8000FF000000FF00FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF000000FF0000FF00FF00000000FF0000FF0000FFFF8000
FF80000000FF0000FF0000FF0000FF
0000FFFF000000FF0000FF00FF0000FF0000FF0000FF000000FF00FF000000FF0000FF00
00FF00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF00000000FF0000FF
0000FF0000FF0000FFFF800000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF8000
0000FF0000FF0000FFFF0000FF0000FF0000FF0000FF00000000FF0000FFFF80000000FF
0000FFFF8000FF8000FF8000FF0000FF8000FF8000FF000000FF0000FF00FF8000FF8000
FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF00000000FF0000FFFF8000
FF80000000FF0000FF0000FF0000FF
0000FFFF000000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00
00FF0000FF00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF8000FF00000000FF
FF0000FF80000000FF0000FF00FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000
FF0000FF80000000FFFF000000FF00FF0000FF000000FF000000FF0000FF0000FFFF8000
0000FFFF8000FF8000FF8000FF0000FF80000000FF00FF0000FF00FF0000FF0000FF0000
FF0000FF8000FF8000FF000000FF0000FF00FF0000FF0000FF80000000FF0000FF0000FF
FF8000FF8000FF8000FF80000000FF
0000FFFF000000FF00FF0000FF0000FF00000000FFFF0000FF000000FF0000FF0000FF00
FF000000FF0000FF000000FFFF0000FF00000000FFFF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF80000000FFFF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000
FF0000FF80000000FF00FF0000FF00FF0000FF000000FF000000FF0000FF0000FFFF8000
FF0000FF8000FF8000FF0000FF00000000FF00FF00FF0000FF0000FF0000FF00000000FF
0000FF0000FFFF8000FF800000FF0000FF00FF8000FF0000FF00000000FF0000FF0000FF
FF80000000FFFF80000000FF0000FF
0000FFFF000000FF00FF0000FF0000FF0000FF8000FF0000FF0000FF000000FF0000FF00
FF000000FF0000FF000000FF0000FF0000FF0000FFFF0000FF00000000FF00FF000000FF
FF00000000FF0000FF0000FFFF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FF0000FFFF0000FF0000FF8000FF80000000FF0000FFFF8000FF8000
FF0000FF8000FF8000FF0000FF00000000FFFF0000FF0000FF0000FF8000FF00000000FF
0000FF0000FFFF8000FF8000FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FF
FF8000FF0000FF00000000FF0000FF
FF8000FF000000FF00FF0000FF0000FF0000FF8000FF0000FF000000FF0000FF00FF0000
FF000000FF0000FF000000FF0000FF0000FF0000FFFF0000FF00000000FF00FF00FF0000
FF00000000FF0000FF0000FFFF0000FF0000FF8000FF0000FF000000FF00FF0000FF0000
FF0000FF8000FF000000FF00FF0000FF0000FF8000FF80000000FF0000FFFF8000FF8000
FF000000FF00FF8000FF0000FF000000FF00FF0000FF0000FF0000FF8000FF00000000FF
0000FF0000FFFF80000000FFFF0000FF00000000FF0000FF0000FFFF8000FF00000000FF
FF8000FF0000FF80000000FF0000FF
FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF000000FF0000FF00
00FF0000FF00FF00000000FF0000FF0000FF0000FFFF0000FF00000000FFFF0000FF0000
FF80000000FFFF80000000FF0000FF00FF00FF0000FF0000FF000000FF00FF0000FF0000
FF8000FF80000000FFFF0000FF0000FF0000FF80000000FFFF8000FF8000FF8000FF0000
FF000000FF00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF8000FF00000000FF
0000FF0000FF0000FFFF8000FF0000FF0000FF00000000FF0000FFFF8000FF00000000FF
FF00000000FFFF80000000FF0000FF
0000FFFF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF000000FF0000FF00
FF0000FF0000FF0000FF8000FF00000000FFFF8000FF8000FF0000FF8000FF0000FF0000
FF80000000FF0000FF0000FFFF800000FF00FF0000FF0000FF000000FF0000FF00FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF80000000FF0000FFFF8000FF80000000FF
0000FF0000FFFF000000FF00FF000000FF0000FF00FF0000FF0000FF8000FF00000000FF
0000FF0000FF0000FFFF8000FF0000FF0000FF00000000FF0000FF00FF0000FF000000FF
0000FF0000FFFF80000000FF0000FF
00FF00FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF80000000FFFF8000FF0000FF0000FF8000FF0000FF0000
FF8000FF8000FF80000000FFFF000000FF00FF000000FF0000FF00FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF8000FF0000FF80000000FF0000FFFF8000FF80000000FF
0000FF0000FFFF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF00000000FF
0000FFFF8000FF8000FF8000FF0000FF0000FF0000FF00000000FF00FF0000FF000000FF
0000FF0000FFFF80000000FF0000FF
0000FFFF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF80000000FFFF00000000FFFF8000FF0000
FF8000FF0000FF000000FF00FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF80000000FF
0000FFFF8000FF0000FF00000000FF00FF0000FF0000FF00FF0000FF0000FF00000000FF
0000FFFF8000FF8000FF8000FF0000FF0000FF0000FF0000FF8000FF000000FF000000FF
0000FF0000FFFF80000000FF0000FF
0000FF00FF00FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000
FF8000FF00000000FFFF8000FF0000FF00000000FF0000FF0000FF0000FFFF8000FF0000
FF8000FF8000FF000000FF00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF80000000FF
0000FF0000FFFF0000FF00000000FF00FF0000FF0000FF0000FF00FF0000FF00000000FF
0000FFFF8000FF8000FF8000FF000000FF00FF8000FF0000FF0000FF000000FF000000FF
FF0000FF0000FF00000000FF0000FF
0000FF00FF00FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FFFF8000FF8000FF00000000FF0000FF0000FF0000FFFF8000FF8000
0000FFFF80000000FF00FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00
00FF00FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF00000000FF0000FF
0000FFFF8000FF0000FF00000000FF00FF00FF0000FF0000FF0000FF0000FF00000000FF
0000FF0000FFFF8000FF8000FF0000FF0000FF8000FF0000FF0000FF000000FF00FF0000
FF0000FF0000FF00000000FF0000FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.738095 0.516558 0.97619 0.756818 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .13975 L
.2381 .13975 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .13975 L
.2381 .13975 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .04025 L
.47619 .04025 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .04025 L
.47619 .04025 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .08614 L
.71429 .08614 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .08614 L
.71429 .08614 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .37566 L
.95238 .37566 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .37566 L
.95238 .37566 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF8000FF0000FF0000FF8000FF0000FF0000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF00000000FFFF000000FF00FF8000FF0000FF8000FF0000
FF0000FF8000FF0000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF0000
FF000000FF000000FFFF0000FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF0000
FF8000FF0000FF0000FF8000FF0000FF80000000FFFF8000FF8000FF8000FF8000FF8000
FF0000FF8000FF8000FF0000FF8000
FF8000FF0000FF000000FF00FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF80000000FFFF8000FF0000FF00000000FF0000FF00FF00FF0000FF8000FF8000FF0000
FF8000FF0000FF00000000FFFF0000FF0000FF8000FF8000FF8000FF0000FF000000FF00
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF
FF0000FF0000FF8000FF0000FF0000FF8000FF8000FF8000FF0000FF8000FF8000FF0000
FF8000FF8000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF0000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF80000000FFFF0000FF00000000FF0000FF00FF00FF8000FF0000FF8000FF8000
FF8000FF8000FF0000FF800000FF00FF8000FF0000FF8000FF8000FF0000FF8000FF0000
FF8000FF8000FF8000FF0000FF8000FF00000000FFFF8000FF8000FF8000FF8000FF0000
FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF0000
FF8000FF8000FF0000FF8000FF8000FF0000FF8000FF80000000FFFF80000000FFFF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF80000000FFFF8000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF00000000FFFF0000FF0000FF00000000FF00FF00FF8000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF000000FF00FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF800000FF000000FFFF8000FF8000FF0000FF8000FF8000
00FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000
FF8000FF800000FF00FF00000000FFFF8000FF8000FF80000000FFFF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF0000FF8000FF8000FF0000FF0000FF00000000FF00FF0000FF0000FF0000FF00FF0000
FF0000FF0000FF0000FF8000FF8000FF0000FF8000FF80000000FF0000FF0000FFFF8000
FF0000FF0000FF8000FF0000FF0000FF00000000FF0000FF0000FF0000FF0000FFFF8000
FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000FF80000000FFFF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF8000FF0000
FF8000FF8000FF0000FF8000FF8000FF0000FF8000FF80000000FF0000FF0000FFFF8000
FF8000FF0000FF8000FF8000FF8000FF800000FF000000FFFF80000000FFFF8000FF8000
00FF00FF8000FF8000FF0000FF8000FF8000FF8000FF80000000FF0000FFFF8000FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
00FF00FF8000FF8000FF80000000FF
FF8000FF0000FF8000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF0000
0000FFFF8000FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000FF0000FF8000FF00000000FF0000FFFF8000FF8000
0000FFFF8000FF0000FF8000FF8000FF8000FF8000FF0000FF80000000FFFF8000FF8000
FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF800000FF00FF8000FF8000
FF0000FF8000FF8000FF8000FF0000
FF0000FF0000FF8000FF0000FF0000FF80000000FFFF0000FF0000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF0000FF0000FF8000FF0000
FF000000FF0000FF00FF0000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF80000000FFFF8000FF8000FF8000FF8000FF0000FF80000000FF0000FFFF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF80000000FF
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000
FF8000FF8000FF80000000FFFF0000
00FF0000FF00FF8000FF0000FF0000FF800000FF00FF000000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF8000FF0000FF8000FF8000FF0000FF0000FF80000000FFFF8000
FF8000FF8000FF8000FF8000FF8000FF80000000FFFF80000000FFFF8000FF8000FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000
FF8000FF80000000FFFF8000FF8000
00FF00FF0000FF8000FF8000FF0000FF8000FF00000000FF00FF00FF0000FF8000FF8000
FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF80000000FFFF8000FF0000FF8000
FF8000FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF0000FF80000000FFFF8000
FF80000000FFFF8000FF8000FF8000FF80000000FFFF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF80000000FFFF8000FF8000FF8000FF0000FF8000
FF8000FF0000FF80000000FFFF8000
FF0000FF8000FF0000FF8000FF0000FF80000000FFFF8000FF000000FF00FF0000FF0000
0000FF0000FF00FF00FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF00000000FF
FF8000FF0000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF80000000FFFF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF8000FF8000FF0000FF8000
FF8000FF0000FF80000000FFFF8000
FF0000FF8000FF8000FF8000FF8000FF80000000FFFF8000FF0000FF0000FF8000FF0000
00FF000000FF00FF00FF0000FF8000FF0000FF8000FF0000FF0000FF0000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF80000000FF0000FFFF8000
FF80000000FF0000FFFF8000FF8000FF8000FF8000FF8000FF0000FF80000000FFFF8000
FF00000000FFFF80000000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF0000FF8000FF0000FF8000FF8000FF80000000FFFF0000FF0000FF0000FF8000FF8000
FF0000FF800000FF00FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF800000FF00
00FF00FF8000FF0000FF0000FF80000000FF0000FFFF0000FF80000000FFFF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF800000FF00FF0000
FF80000000FFFF8000FF8000FF8000FF8000FF80000000FFFF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF80000000FFFF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF80000000FFFF0000FF0000FF0000FF80000000FF
FF8000FF800000FF00FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF800000FF00
FF0000FF8000FF8000FF8000FF80000000FFFF00000000FF0000FF0000FFFF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF0000FF8000
0000FF0000FF0000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000FF00000000FF
FF8000FF8000FF8000FF8000FF8000FF80000000FFFF8000FF8000FF80000000FFFF8000
0000FF0000FFFF8000FF8000FF8000
FF0000FF8000FF8000FF8000FF8000FF80000000FFFF0000FF0000FF8000FF8000FF8000
FF8000FF8000FF0000FF0000FF8000FF0000FF8000FF8000FF0000FF80000000FFFF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF00000000FF0000FF0000FFFF8000FF8000
FF8000FF0000FF8000FF0000FF8000FF8000FF800000FF00FF8000FF0000FF0000FF8000
0000FFFF80000000FFFF80000000FF0000FF0000FFFF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF80000000FFFF0000FF8000FF8000FF8000FF80000000FFFF8000
FF80000000FFFF8000FF8000FF8000
FF0000FF80000000FFFF0000FF000000FF00FF0000FF0000FF8000FF8000FF8000FF8000
FF8000FF800000FF0000FF00FF8000FF0000FF8000FF8000FF0000FF000000FF0000FF00
FF8000FF0000FF8000FF8000FF8000FF8000FF0000FF8000FF80000000FFFF80000000FF
FF8000FF0000FF80000000FFFF8000FF8000FF800000FF00FF8000FF0000FF8000FF8000
0000FF0000FFFF8000FF80000000FF0000FF0000FFFF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF800000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000
0000FF0000FFFF8000FF8000FF8000
FF8000FF80000000FFFF00000000FFFF0000FF0000FF8000FF8000FF8000FF8000FF8000
FF800000FF0000FF0000FF00FF80000000FF0000FFFF800000FF0000FF0000FF000000FF
FF8000FF0000FF8000FF8000FF8000FF8000FF0000FF8000FF80000000FF0000FF0000FF
FF800000FF00FF80000000FF0000FFFF8000FF80000000FFFF8000FF8000FF8000FF8000
0000FFFF8000FF8000FF00000000FF0000FFFF8000FF8000FF0000FF0000FF8000FF0000
FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF0000FF8000FF00000000FF0000FF
FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF80000000FF00FF00FF0000FF0000FF8000FF8000FF8000FF8000FF8000
FF800000FF0000FF0000FF00FF80000000FF0000FFFF800000FF000000FFFF00000000FF
FF8000FF00000000FFFF80000000FFFF8000FF80000000FFFF80000000FF0000FFFF8000
FF8000FF0000FF80000000FF0000FFFF8000FF8000FF0000FF8000FF8000FF8000FF8000
FF80000000FFFF8000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000
FF8000FF8000FF8000FF0000FF8000FF8000FF0000FF8000FF0000FF00000000FF00FF00
FF8000FF0000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
0000FF0000FFFF000000FF000000FF0000FF0000FFFF800000FF00FF8000FF0000FF0000
FF8000FF00000000FFFF8000FF8000FF8000FF8000FF0000FF80000000FFFF8000FF8000
FF8000FF0000FF00000000FF0000FF0000FFFF8000FF8000FF800000FF00FF00000000FF
0000FF0000FFFF8000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF80000000FFFF8000FF0000FF00000000FF00FF00
0000FFFF8000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF0000FF8000FF8000FF00000000FF0000FF0000FFFF8000FF8000FF8000FF00000000FF
FF8000FF00000000FFFF80000000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF00000000FFFF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF00000000FF
FF0000FF0000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF80000000FF00FF00
00FF00FF8000FF8000FF8000FF8000
0000FF0000FFFF8000FF8000FF0000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF00000000FF0000FFFF0000FF8000FF80000000FFFF800000FF00FF80000000FF0000FF
FF8000FF0000FF80000000FFFF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000
FF00000000FFFF00000000FFFF80000000FFFF8000FF8000FF8000FF0000FF0000FF8000
FF8000FF8000FF8000FF0000FF0000FF00000000FFFF0000FF0000FF8000FF0000FF8000
FF8000FF0000FF8000FF8000FF80000000FF0000FFFF8000FF80000000FF0000FF00FF00
00FF00FF8000FF8000FF8000FF8000
00FF000000FFFF0000FF8000FF8000FF8000FF8000FF8000FF0000FF00000000FFFF8000
FF0000FF80000000FFFF0000FF8000FF8000FF8000FF80000000FFFF80000000FFFF8000
FF8000FF0000FF80000000FF0000FFFF8000FF0000FF8000FF8000FF8000FF8000FF8000
FF0000FF8000FF8000FF8000FF8000FF8000FF80000000FF0000FFFF8000FF0000FF8000
FF0000FF0000FF8000FF8000FF0000FF00000000FF00FF0000FF00FF8000FF0000FF8000
FF8000FF0000FF8000FF00000000FF0000FFFF000000FF000000FF0000FF0000FF00FF00
00FF00FF8000FF8000FF8000FF8000
FF0000FF800000FF00FF8000FF8000FF8000FF80000000FFFF0000FF0000FF8000FF8000
FF8000FF80000000FFFF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF00000000FF0000FF0000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF8000
FF8000FF0000FF8000FF8000FF0000FF00000000FFFF8000FF0000FF8000FF8000FF8000
FF80000000FF0000FFFF00000000FF0000FFFF0000FF0000FF80000000FF0000FFFF0000
FF8000FF8000FF8000FF8000FF8000
FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF00000000FFFF8000FF8000
FF8000FF80000000FF00FF000000FFFF0000FF000000FF0000FF00FF0000FF8000FF0000
FF8000FF8000FF8000FF8000FF0000
FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF80000000FF
FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF80000000FFFF8000FF8000FF8000
FF0000FF8000FF8000FF8000FF80000000FFFF8000FF8000FF8000FF8000FF8000FF8000
FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF000000FF00FF8000FF8000FF80000000FFFF8000FF8000FF8000FF8000FF8000
FF80000000FF0000FF0000FF0000FFFF0000FF000000FF00FF0000FF0000FF80000000FF
FF8000FF8000FF0000FF0000FF8000
FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF80000000FFFF0000FF8000FF8000
FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF00000000FF0000FF0000FFFF8000
FF0000FF8000FF80000000FFFF00000000FFFF8000FF8000FF8000FF80000000FFFF8000
FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF80000000FF
FF8000FF0000FF0000FF0000FF8000FF80000000FFFF8000FF8000FF8000FF8000FF8000
FF80000000FFFF8000FF8000FF8000FF8000FF000000FF00FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF80000000FFFF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF80000000FF0000FFFF8000FF8000
FF8000FF8000FF800000FF0000FF000000FF00FF00FF0000FF0000FF0000FF8000FF8000
FF0000FF8000FF8000FF8000FF0000FF8000FF8000FF00000000FFFF8000FF0000FF8000
FF8000FF0000FF0000FF8000FF8000FF00000000FF0000FF0000FFFF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000FF8000FF000000FF00FF0000FF8000FF8000FF8000
FF8000FF0000FF80000000FF0000FF
0000FF0000FFFF0000FF0000FF8000FF00000000FFFF0000FF8000FF8000FF00000000FF
FF8000FF00000000FF00FF00FF8000FF0000FF80000000FF0000FFFF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF000000FF0000FF00FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF000000FF00FF8000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF800000FF00FF0000FF80000000FFFF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF00000000FFFF8000FF8000FF8000
FF0000FF00000000FF0000FF0000FF
00FF00FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF800000FF0000FF0000FF00FF0000FF00000000FFFF0000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF000000FF00FF000000FF00FF00000000FFFF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF
FF80000000FFFF0000FF800000FF0000FF00FF8000FF8000FF80000000FFFF8000FF0000
0000FFFF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF80000000FFFF8000
FF0000FF00000000FF0000FF0000FF
FF0000FF8000FF8000FF0000FF8000FF0000FF8000FF00000000FF0000FF0000FF0000FF
FF8000FF00000000FF00FF00FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF0000
FF0000FF8000FF0000FF8000FF0000FF8000FF0000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF80000000FFFF8000FF8000FF8000FF80000000FF0000FFFF8000
FF80000000FFFF0000FF800000FF0000FF00FF00000000FFFF80000000FF0000FF0000FF
0000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF0000FFFF8000
FF8000FF8000FF80000000FFFF8000
FF0000FF8000FF8000FF8000FF8000FF0000FF8000FF00000000FF0000FF0000FF0000FF
FF8000FF80000000FFFF0000FF0000FF00000000FFFF0000FF8000FF0000FF8000FF0000
FF8000FF8000FF0000FF8000FF0000FF80000000FF0000FFFF0000FF8000FF8000FF0000
FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF0000FF800000FF00FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000
FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF0000FF0000FF
FF80000000FF00FF00FF8000FF80000000FFFF8000FF8000FF8000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF0000FF80000000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF0000FF
0000FFFF8000FF8000FF8000FF8000
0000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF0000FF0000FF
FF800000FF0000FF00FF00000000FF0000FFFF8000FF8000FF80000000FFFF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF0000FF8000FF80000000FF0000FFFF8000FF80000000FF0000FF0000FFFF8000FF0000
FF0000FF8000FF8000FF0000FF8000FF0000FF0000FF8000FF8000FF80000000FFFF8000
FF8000FF800000FF00FF0000FF0000FF8000FF8000FF0000FF8000FF8000FF80000000FF
0000FFFF80000000FF0000FFFF8000
FF8000FF800000FF00FF0000FF8000FF8000FF8000FF8000FF0000FF80000000FF0000FF
FF80000000FF00FF0000FF000000FF0000FF0000FFFF80000000FF0000FF0000FFFF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF8000FF0000FF8000
FF8000FF8000FF80000000FF0000FFFF8000FF80000000FFFF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF000000FF00FF8000FF0000FF8000FF0000FF8000FF8000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF80000000FF0000FF
FF80000000FF00FF00FF8000FF80000000FF0000FFFF8000FF0000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF800000FF00FF000000FF00FF0000FF0000FF8000FF8000
FF8000FF8000FF8000FF80000000FFFF8000FF8000FF8000FF8000FF8000FF800000FF00
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF8000FF8000
FF8000FF8000FF0000FF0000FF0000FF800000FF0000FF00FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF800000FF0000FF00FF8000FF0000FF0000FF8000FF00000000FFFF80000000FF
FF8000FF800000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000
FF80000000FFFF0000FF0000FF8000FF000000FF000000FFFF0000FF0000FF8000FF0000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF800000FF00
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF80000000FFFF000000FF00FF0000FF800000FF00FF0000FF0000FF0000FF8000FF8000
FF0000FF8000FF8000FF8000FF8000
FF000000FF0000FF0000FF000000FF00FF000000FF0000FF0000FFFF8000FF8000FF0000
FF0000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF0000
FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF0000FF8000FF0000FF0000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF000000FF00FF0000
FF0000FF0000FF0000FF0000FF0000FF00000000FFFF8000FF0000FF8000FF8000FF0000
0000FF00FF0000FF00FF0000FF0000FF0000FF8000FF0000FF80000000FFFF0000FF0000
FF8000FF0000FF8000FF8000FF8000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.756818 0.261905 0.997078 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .56506 L
.2381 .56506 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .56506 L
.2381 .56506 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .01293 L
.47619 .01293 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .01293 L
.47619 .01293 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .01209 L
.71429 .01209 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .01209 L
.71429 .01209 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .05173 L
.95238 .05173 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .05173 L
.95238 .05173 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF00000000FFFF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF8000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF8000FF8000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF8000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF8000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF8000FF8000FF8000FF8000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF0000
FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF8000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF000000FF00FF800000FF00
FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000
00FF00FF8000FF0000FF8000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
00FF00FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF8000FF0000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.261905 0.756818 0.5 0.997078 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .02273 L
.2381 .02273 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .02273 L
.2381 .02273 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .39506 L
.47619 .39506 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .39506 L
.47619 .39506 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .09657 L
.71429 .09657 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .09657 L
.71429 .09657 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .12744 L
.95238 .12744 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .12744 L
.95238 .12744 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
00FF00FF800000FF0000FF0000FF00FF800000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF00FF0000FF0000FF00FF8000FF8000FF8000FF0000FF8000FF800000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF8000FF800000FF00FF8000FF8000FF8000
FF8000FF800000FF0000FF00FF8000FF0000FF0000FF0000FF000000FF0000FF0000FF00
FF800000FF0000FF0000FF0000FF0000FF00FF8000FF8000FF000000FF0000FF0000FF00
0000FFFF80000000FFFF8000FF800000FF0000FF00FF8000FF8000FF8000FF8000FF8000
FF80000000FF00FF0000FF0000FF00
00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF800000FF0000FF00FF0000FF8000FF800000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF00FF8000FF8000
FF800000FF00FF8000FF8000FF8000FF0000FF8000FF00000000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF00FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000
00FF0000FF0000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF0000FF800000FF00
FF800000FF0000FF0000FF0000FF00
FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF8000FF8000FF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000FF8000
FF8000FF8000FF0000FF8000FF8000FF0000FF8000FF800000FF0000FF0000FF0000FF00
00FF0000FF00FF8000FF8000FF800000FF00FF0000FF8000FF0000FF8000FF800000FF00
00FF0000FF0000FF00FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000FF8000
FF8000FF8000FF800000FF0000FF00
FF80000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF00FF8000FF8000
FF0000FF8000FF0000FF8000FF8000FF0000FF800000FF0000FF0000FF0000FF0000FF00
FF8000FF800000FF00FF8000FF8000FF0000FF0000FF8000FF8000FF800000FF0000FF00
00FF0000FF0000FF00FF800000FF00FF8000FF80000000FF00FF0000FF0000FF0000FF00
00FF00FF0000FF8000FF80000000FF
0000FF0000FF00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF800000FF0000FF0000FF0000FF00FF8000FF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF000000FF0000FF00FF00
FF80000000FF0000FF0000FF00FF0000FF0000FF00FF800000FF0000FF0000FF0000FF00
FF8000FF8000FF80000000FF0000FF0000FFFF8000FF80000000FF0000FF0000FF0000FF
00FF0000FF0000FF000000FF0000FF0000FFFF800000FF0000FF000000FF0000FF0000FF
FF800000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF
0000FFFF8000FF800000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF00FF00FF80000000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF00
00FF0000FF00FF800000FF000000FF
00FF000000FF00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00FF800000FF00FF0000FF800000FF0000FF0000FF00
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF00FF0000FF0000FF000000FF0000FF0000FFFF80000000FF0000FFFF80000000FF
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF00FF8000
FF000000FF0000FF0000FF000000FF
00FF0000FF00FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF800000FF0000FF00FF800000FF00FF8000FF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FFFF800000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF00
FF80000000FF00FF00FF800000FF00
00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF0000FF00FF800000FF000000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00FF800000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF00FF0000FF0000FF0000FF00FF80000000FFFF8000FF80000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF00FF800000FF0000FF0000FF00FF8000FF8000FF80000000FF00FF00
00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF00FF00
00FF0000FF000000FF00FF00FF800000FF000000FFFF80000000FFFF8000FF8000FF8000
FF80000000FFFF8000FF800000FF00
00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF8000FF800000FF00FF8000FF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF00FF8000FF800000FF00FF8000FF8000FF0000FF800000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF80000000FF0000FFFF800000FF00
00FF0000FF00FF800000FF00FF8000FF800000FF00FF800000FF00FF8000FF8000FF0000
FF0000FF80000000FF0000FF00FF00
0000FFFF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF00FF8000FF8000
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF00
00FF0000FF0000FF00FF8000FF0000FF8000FF0000FF8000FF8000FF800000FF0000FF00
00FF0000FF0000FF00FF80000000FFFF8000FF800000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FFFF800000FF00FF8000FF8000
FF0000FF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FFFF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF00FF800000FF0000FF00FF8000
FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF0000FF00
0000FFFF8000FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF00
00FF0000FF00FF80000000FFFF8000FF80000000FF0000FF0000FFFF8000FF8000FF8000
FF8000FF800000FF0000FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FFFF80000000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000
FF800000FF0000FF00FF0000FF0000FF8000FF8000FF800000FF00FF800000FF0000FF00
FF8000FF0000FF8000FF80000000FF0000FF00FF0000FF0000FF0000FF00FF800000FF00
FF800000FF0000FF00FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
FF80000000FF00FF00FF800000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000FF80000000FFFF8000
0000FFFF8000FF8000FF0000FF8000FF8000FF0000FF8000FF8000FF800000FF00FF8000
FF800000FF00FF8000FF800000FF0000FF0000FF00FF80000000FFFF800000FF0000FF00
00FF0000FF0000FF00FF800000FF0000FF0000FF000000FF0000FFFF80000000FF0000FF
FF800000FF00FF800000FF0000FF00
00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF00
00FF0000FF0000FF00FF80000000FFFF8000FF8000FF8000FF0000FF800000FF00FF8000
FF800000FF00FF800000FF0000FF0000FF000000FF00FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF00FF8000FF800000FF00
FF800000FF0000FF00FF800000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF00FF800000FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
FF8000FF800000FF00FF800000FF00FF8000FF8000FF0000FF0000FF80000000FFFF8000
FF8000FF8000FF8000FF80000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF00FF8000FF8000
00FF00FF800000FF00FF800000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF00FF00FF800000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00FF80000000FF0000FF0000FF00FF0000FF0000FF00
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF000000FF
00FF0000FF0000FF0000FF0000FF000000FF00FF00FF80000000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF000000FFFF8000FF8000
FF8000FF8000FF800000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF00FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF800000FF00
FF8000FF8000FF8000FF000000FF0000FF0000FF00FF800000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF00
FF8000FF8000FF80000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FFFF800000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FFFF8000FF800000FF00FF8000FF8000FF8000FF0000
FF8000FF80000000FFFF8000FF0000FF8000FF8000FF8000FF80000000FF0000FF0000FF
00FF00FF8000FF0000FF0000FF800000FF000000FFFF800000FF00FF80000000FF0000FF
FF800000FF000000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FFFF8000
0000FFFF80000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF80000000FF00FF0000FF0000FF0000FF000000FF00FF000000FF
FF80000000FFFF8000FF800000FF000000FF00FF0000FF0000FF000000FF0000FF00FF00
00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF00FF800000FF0000FF0000FF00
FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00FF8000FF80000000FFFF80000000FF0000FF0000FF
0000FF0000FF0000FFFF80000000FFFF8000FF800000FF00FF800000FF00FF8000FF8000
FF8000FF8000FF0000FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FFFF80000000FFFF8000
00FF0000FF00FF80000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF0000FF00FF800000FF00FF8000FF80000000FF
0000FF00FF0000FF0000FF000000FFFF8000FF8000FF8000FF8000FF8000FF80000000FF
0000FFFF800000FF00FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF000000FFFF80000000FFFF8000
FF8000FF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FFFF8000FF8000FF8000FF8000FF8000
0000FF0000FF00FF0000FF0000FF00FF8000FF8000FF8000FF800000FF00FF800000FF00
00FF00FF800000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF000000FF0000FF00FF0000FF0000FF0000FF00FF8000
FF80000000FF0000FF00FF0000FF00
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF000000FF0000FFFF80000000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000FF8000FF8000FF8000FF8000
00FF0000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
00FF00FF8000FF80000000FF0000FF00FF00FF80000000FF0000FF0000FF0000FF00FF00
00FF0000FF000000FF00FF00FF800000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF00FF800000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FFFF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF0000FFFF8000FF800000FF00
00FF0000FF00FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
00FF00FF80000000FF0000FFFF8000
00FF000000FF0000FF00FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF00FF8000FF80000000FF0000FF
FF8000FF8000FF0000FF8000FF000000FF00FF8000FF0000FF8000FF8000FF80000000FF
0000FF0000FFFF8000FF800000FF0000FF0000FF0000FF0000FF00FF800000FF0000FF00
00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00FF800000FF0000FF00
FF800000FF0000FF000000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF00FF80000000FF0000FFFF80000000FF0000FF00FF0000FF0000FF0000FF0000FF00
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF00FF8000FF80000000FF0000FF
0000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF0000FF00FF00
FF800000FF0000FF00FF800000FF00FF80000000FFFF800000FF00FF8000FF800000FF00
00FF0000FF0000FF000000FF00FF0000FF000000FFFF80000000FF0000FFFF80000000FF
0000FFFF800000FF0000FF000000FF
00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF00FF80000000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF00FF8000FF8000FF800000FF0000FF0000FF0000FF00
00FF000000FFFF80000000FFFF80000000FF0000FFFF8000FF0000FF8000FF800000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FFFF800000FF000000FF
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000FF8000
0000FF0000FFFF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF00FF000000FF
00FF000000FF0000FFFF8000FF8000FF80000000FFFF80000000FFFF8000FF800000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00FF8000FF800000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF80000000FFFF8000FF8000FF800000FF00
00FF0000FF0000FF0000FF000000FFFF80000000FF0000FFFF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF00
00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF00FF8000FF800000FF0000FF00
00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00FF800000FF0000FF0000FF00
00FF0000FF0000FF00FF800000FF00FF8000FF800000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000
00FF0000FF00FF800000FF0000FF00FF800000FF0000FF00FF800000FF000000FF0000FF
00FF0000FF00FF80000000FF0000FFFF80000000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF000000FF0000FF00FF00
00FF0000FF0000FF00FF800000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF00
0000FF0000FF00FF0000FF0000FF000000FF0000FF00FF00FF8000FF0000FF800000FF00
FF8000FF800000FF0000FF00FF800000FF00FF000000FF0000FF0000FF0000FF0000FF00
FF800000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF00
FF8000FF8000FF800000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF00FF8000FF8000
00FF00FF8000FF80000000FFFF8000FF8000FF8000FF8000FF800000FF0000FF0000FF00
00FF000000FF00FF0000FF0000FF00FF8000FF800000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF00
0000FF00FF0000FF0000FF0000FF00
00FF00FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00FF800000FF00FF800000FF0000FF0000FF0000FF00
00FF0000FF00FF800000FF0000FF000000FF00FF0000FF000000FFFF800000FF0000FF00
FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FFFF80000000FFFF8000FF8000
00FF0000FF00FF80000000FFFF8000FF8000FF8000FF8000FF80000000FF0000FF0000FF
00FF000000FF0000FF00FF0000FF00FF80000000FF0000FF00FF00FF800000FF0000FF00
00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000
00FF00FF8000FF800000FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF00FF8000
FF8000FF800000FF0000FF00FF8000FF8000FF0000FF8000FF800000FF0000FF0000FF00
FF800000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF00FF80000000FF
FF8000FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000
00FF0000FF00FF800000FF0000FF00
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.756818 0.738095 0.997078 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .0123 L
.2381 .0123 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .0123 L
.2381 .0123 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .28826 L
.47619 .28826 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .28826 L
.47619 .28826 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .2795 L
.71429 .2795 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .2795 L
.71429 .2795 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .06174 L
.95238 .06174 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .06174 L
.95238 .06174 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF000000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF00FF00
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF00FF80000000FFFF80000000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000FF80000000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
0000FF0000FF0000FF00FF000000FF0000FF00FF0000FF000000FF00FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FFFF8000
0000FF00FF00FF80000000FFFF8000FF80000000FF0000FFFF800000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF00
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF800000FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF800000FF00
00FF00FF00000000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF80000000FFFF8000FF80000000FFFF80000000FFFF8000FF8000
00FF00FF800000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF
FF8000FF8000FF80000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF00FF8000
0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
FF800000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF800000FF000000FF
0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF80000000FF0000FFFF8000FF800000FF0000FF000000FF0000FF0000FF
FF8000FF8000FF8000FF80000000FF0000FF0000FF00FF0000FF000000FFFF8000FF8000
FF8000FF80000000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF00FF000000FF
0000FF0000FF00FF0000FF0000FF00
00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FFFF8000FF80000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF00FF800000FF00FF8000FF80000000FFFF8000
FF8000FF8000FF80000000FF0000FF0000FF0000FFFF8000FF80000000FFFF80000000FF
0000FF0000FF0000FF00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF00FF8000
0000FF0000FF00FF00FF80000000FF
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FFFF80000000FF0000FFFF8000FF8000FF8000FF8000FF80000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF000000FF0000FF0000FFFF800000FF0000FF00FF80000000FFFF8000
FF80000000FFFF80000000FF0000FFFF80000000FFFF8000FF800000FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
0000FF0000FF00FF00FF80000000FF
00FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF00
FF8000FF8000FF80000000FFFF80000000FFFF8000FF8000FF8000FF80000000FF0000FF
FF80000000FF0000FF0000FFFF8000FF80000000FF0000FF0000FF00FF00FF800000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF800000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FFFF8000
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF000000FF0000FF0000FF0000FF00FF00FF8000FF80000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF00
00FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF00FF0000FF0000FF00
FF80000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF00FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF80000000FFFF8000FF80000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF
0000FFFF80000000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF80000000FF0000FF0000FFFF80000000FF0000FF00FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF00FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF
FF80000000FF0000FFFF80000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF00FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00
0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF00
FF80000000FF0000FFFF800000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF
FF80000000FF0000FF0000FF0000FF0000FF0000FFFF800000FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF00FF0000FF00
0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF80000000FF00FF000000FF0000FF0000FF00FF000000FF00FF00FF8000
0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF00
0000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF00FF0000FF0000FF000000FF0000FF00FF0000FF000000FF0000FF00FF00
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FFFF8000
FF80000000FF0000FF0000FF0000FFFF80000000FF0000FF00FF0000FF000000FF0000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF
00FF0000FF0000FF000000FF0000FF0000FFFF80000000FF00FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF0000FF000000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF00
00FF00FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF00
00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FFFF80000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
FF8000FF80000000FF0000FF00FF00
00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF80000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FFFF8000FF800000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF00
0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF000000FFFF800000FF00
FF800000FF000000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF00
0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
FF80000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FFFF8000
FF80000000FFFF8000FF800000FF000000FFFF8000FF80000000FF0000FFFF8000FF8000
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FFFF8000
0000FF00FF000000FF0000FF0000FF00FF0000FF000000FF0000FF0000FFFF8000FF8000
FF8000FF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF
0000FF00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FFFF8000FF8000FF80000000FF0000FFFF8000FF8000
00FF00FF80000000FFFF8000FF80000000FF0000FF0000FFFF8000FF8000FF80000000FF
00FF0000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF0000FF0000FF0000FF
FF8000FF80000000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF80000000FFFF8000FF800000FF0000FF0000FF0000FF000000FF
00FF00FF80000000FF0000FFFF800000FF0000FF000000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF000000FF0000FFFF800000FF00FF800000FF0000FF00FF8000FF8000
0000FFFF80000000FF0000FFFF8000
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF00FF0000FF000000FF00FF000000FF0000FF0000FF00FF000000FF0000FF00FF00
00FF000000FF0000FF0000FF00FF0000FF000000FF00FF0000FF000000FF0000FF0000FF
0000FF0000FFFF800000FF0000FF0000FF00FF800000FF0000FF0000FF00FF8000FF8000
0000FF0000FF0000FF00FF00FF8000FF80000000FF0000FFFF8000FF80000000FF0000FF
0000FF0000FF0000FF0000FFFF8000FF80000000FF0000FF0000FF0000FFFF8000FF8000
0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF800000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF80000000FF0000FFFF800000FF0000FF000000FF0000FFFF800000FF00
00FF00FF8000FF80000000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FFFF800000FF0000FF00FF80000000FFFF8000FF8000FF8000FF80000000FF
FF80000000FF0000FF0000FF00FF000000FF0000FF0000FF0000FFFF80000000FFFF8000
0000FF0000FF0000FFFF800000FF000000FF00FF000000FF0000FFFF8000FF80000000FF
0000FF0000FF0000FF00FF0000FF00
0000FF0000FFFF800000FF000000FF00FF00FF80000000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF000000FFFF80000000FF
0000FF00FF000000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FFFF8000FF800000FF00FF80000000FF00FF00FF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF000000FF00FF00
00FF0000FF000000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FFFF8000FF80000000FF0000FFFF8000FF80000000FFFF8000FF8000FF800000FF00
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF00FF800000FF000000FF00FF0000FF000000FF
00FF000000FF00FF0000FF0000FF00
0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF00FF000000FF
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF
0000FF00FF00FF80000000FF0000FF0000FF0000FF0000FF0000FFFF8000FF80000000FF
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF00FF0000FF00
00FF0000FF0000FF00FF8000FF80000000FF00FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF000000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
00FF0000FF0000FF0000FF000000FF0000FF00FF000000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF00FF0000FF0000FF00
0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
0000FF0000FF00FF000000FF00FF0000FF000000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF00FF0000FF0000FF000000FF0000FF00FF0000FF00FF80000000FF0000FF00FF00
00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF
0000FF0000FFFF80000000FF00FF00
0000FF00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FFFF800000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF
00FF0000FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
00FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
00FF0000FF000000FF0000FFFF80000000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF0000FF00
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FFFF800000FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00
0000FF0000FF00FF0000FF0000FF000000FF00FF0000FF0000FF000000FF00FF0000FF00
00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF80000000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000FF0000FF8000
00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF00FF800000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FFFF800000FF00
00FF0000FF000000FF00FF0000FF0000FF000000FF0000FFFF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF8000FF800000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00FF800000FF0000FF00FF8000FF8000FF800000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF00FF000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FFFF80000000FF
00FF00FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF00FF80000000FF0000FF0000FF
0000FF00FF0000FF00FF80000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF0000FF00
00FF0000FF0000FF00FF80000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF00
00FF000000FF00FF00FF80000000FF0000FF0000FFFF800000FF0000FF0000FF0000FF00
FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF0000FF00
00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FFFF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
0000FF0000FF00FF0000FF0000FF00
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.738095 0.756818 0.97619 0.997078 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .11284 L
.2381 .11284 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .11284 L
.2381 .11284 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .15143 L
.47619 .15143 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .15143 L
.47619 .15143 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .04547 L
.71429 .04547 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .04547 L
.71429 .04547 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .33207 L
.95238 .33207 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .33207 L
.95238 .33207 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
00FF0000FF0000FF0000FF00FF8000FF8000FF0000FF00000000FF00FF00FF8000FF8000
FF800000FF0000FF0000FF00FF800000FF00FF8000FF800000FF0000FF0000FF00FF8000
FF8000FF8000FF8000FF0000FF800000FF00FF8000FF8000FF0000FF8000FF800000FF00
FF0000FF000000FF0000FF00FF800000FF0000FF0000FF0000FF00FF80000000FFFF8000
FF8000FF8000FF0000FF8000FF8000FF80000000FFFF8000FF800000FF0000FF00FF8000
00FF00FF8000FF000000FF00FF8000FF8000FF8000FF8000FF8000FF0000FF000000FF00
FF8000FF0000FF8000FF8000FF8000
FF000000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000
FF8000FF8000FF000000FF00FF8000FF800000FF000000FF00FF0000FF0000FF0000FF00
00FF00FF800000FF00FF8000FF8000FF80000000FFFF8000FF0000FF8000FF0000FF8000
FF8000FF0000FF000000FF0000FF00FF8000FF800000FF0000FF00FF800000FF0000FF00
00FF00FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF8000FF8000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF0000FF8000
FF0000FF8000FF8000FF8000FF8000
FF800000FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000
FF8000FF0000FF0000FF0000FF8000FF80000000FF0000FFFF8000FF8000FF8000FF8000
FF800000FF0000FF00FF0000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF80000000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF00FF8000FF0000FF0000FF0000FF8000FF8000FF0000FF8000FF8000FF8000FF0000
FF0000FF8000FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF0000
FF0000FF0000FF0000FF0000FF8000
00FF0000FF0000FF000000FFFF8000FF8000FF8000FF0000FF0000FF8000FF800000FF00
FF0000FF0000FF0000FF800000FF0000FF0000FF0000FF00FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF800000FF0000FF0000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000
00FF00FF8000FF8000FF8000FF800000FF00FF8000FF8000FF0000FF8000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF8000FF8000FF8000
FF8000FF0000FF800000FF00FF8000FF8000FF0000FF0000FF8000FF0000FF0000FF8000
FF8000FF0000FF8000FF000000FF000000FF00FF0000FF00FF800000FF00FF800000FF00
00FF00FF8000FF8000FF8000FF000000FF00FF8000FF8000FF8000FF8000FF800000FF00
FF8000FF8000FF800000FF0000FF0000FF00FF80000000FFFF8000FF8000FF8000FF8000
00FF00FF8000FF0000FF8000FF0000FF8000FF8000FF0000FF8000FF8000FF8000FF0000
FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000
FF8000FF8000FF8000FF80000000FF
00FF00FF800000FF0000FF00FF0000FF8000FF0000FF0000FF8000FF800000FF0000FF00
00FF0000FF00FF800000FF0000FF0000FF000000FF0000FFFF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF800000FF00FF800000FF00FF8000FF800000FF00FF0000FF0000FF8000FF8000
FF8000FF80000000FFFF8000FF800000FF0000FF0000FF0000FF0000FF00FF0000FF0000
FF8000FF800000FF00FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF8000
FF8000FF8000FF0000FF800000FF00
FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
FF8000FF0000FF8000FF800000FF0000FF00FF80000000FFFF8000FF800000FF00FF0000
FF800000FF00FF800000FF00FF800000FF00FF8000FF8000FF8000FF0000FF0000FF0000
FF0000FF8000FF8000FF0000FF800000FF00FF8000FF0000FF0000FF8000FF8000FF8000
00FF00FF80000000FFFF800000FF0000FF0000FF0000FF00FF8000FF8000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF0000FF8000FF0000
00FF00FF800000FF00FF8000FF8000
00FF00FF8000FF0000FF0000FF80000000FF0000FF0000FF0000FF0000FFFF8000FF8000
FF8000FF8000FF8000FF800000FF0000FF000000FF00FF00FF8000FF8000FF8000FF8000
FF000000FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF0000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF0000FF0000
FF8000FF800000FF00FF800000FF00FF8000FF800000FF00FF8000FF0000FF0000FF8000
FF8000FF8000FF0000FF0000FF800000FF0000FF0000FF000000FF00FF0000FF0000FF00
00FF00FF8000FF8000FF8000FF8000
00FF00FF0000FF0000FF0000FF000000FF0000FF0000FF000000FF0000FFFF8000FF0000
FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF0000
FF800000FF0000FF0000FF0000FF000000FF00FF00FF8000FF8000FF8000FF0000FF0000
FF0000FF0000FF8000FF8000FF800000FF00FF80000000FFFF8000FF0000FF0000FF0000
FF800000FF000000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF800000FF000000FF00FF000000FF0000FF00FF0000FF00
FF8000FF800000FF00FF000000FF00
00FF00FF8000FF0000FF8000FF8000FF80000000FF00FF0000FF0000FF00FF80000000FF
FF8000FF8000FF800000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF8000FF8000FF800000FF00FF800000FF0000FF00FF800000FF00FF8000FF0000
FF8000FF8000FF8000FF000000FF0000FF000000FF00FF00FF8000FF0000FF0000FF8000
FF000000FF000000FF00FF00FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00
FF8000FF8000FF8000FF80000000FFFF80000000FF0000FF00FF0000FF0000FF00FF8000
FF8000FF8000FF8000FF800000FF00
00FF00FF0000FF8000FF0000FF0000FF8000FF8000FF800000FF0000FF0000FF0000FF00
FF800000FF0000FF0000FF0000FF00FF8000FF8000FF800000FF00FF8000FF8000FF8000
FF0000FF8000FF800000FF00FF8000FF0000FF0000FF8000FF8000FF000000FF00FF8000
FF8000FF8000FF0000FF800000FF00FF80000000FFFF8000FF8000FF8000FF0000FF8000
00FF0000FF0000FF00FF8000FF80000000FFFF8000FF800000FF00FF8000FF0000FF0000
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
FF800000FF0000FF00FF8000FF8000
0000FFFF8000FF8000FF8000FF0000FF800000FF0000FF0000FF0000FF00FF8000FF8000
FF8000FF800000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF000000FF00FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF000000FF0000FF0000FF00FF8000FF80000000FF
FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000FF8000FF8000FF0000
FF8000FF8000FF0000FF8000FF8000FF8000FF800000FF0000FF0000FF00FF800000FF00
FF8000FF80000000FFFF8000FF8000
00FF0000FF00FF8000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF800000FF00
00FF00FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000FF80000000FF
FF8000FF000000FF00FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF80000000FF
FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF000000FF0000FF0000FF000000FFFF8000FF800000FF0000FF00FF0000FF0000FF0000
FF8000FF0000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF800000FF0000FF00
00FF0000FF0000FF00FF8000FF8000
FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF800000FF0000FF00FF8000
FF8000FF0000FF8000FF800000FF0000FF00FF8000FF8000FF8000FF800000FF0000FF00
FF0000FF00000000FFFF8000FF800000FF00FF8000FF8000FF8000FF800000FF000000FF
0000FFFF8000FF800000FF00FF0000FF800000FF00FF800000FF00FF8000FF8000FF8000
FF0000FF800000FF0000FF0000FF0000FF0000FF00FF8000FF8000FF80000000FFFF8000
FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF800000FF0000FF0000FF00
0000FF0000FFFF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF800000FF00FF8000FF8000FF0000
FF0000FF8000FF8000FF800000FF0000FF00FF800000FF0000FF00FF800000FF0000FF00
FF8000FF0000FF8000FF0000FF8000FF8000FF8000FF0000FF800000FF00FF800000FF00
00FF0000FF00FF8000FF8000FF8000FF0000FF8000FF800000FF0000FF00FF8000FF8000
FF0000FF0000FF800000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF8000FF80000000FFFF8000FF8000FF8000FF0000FF8000FF800000FF00FF8000
00FF00FF8000FF000000FF00FF8000
FF8000FF0000FF0000FF8000FF0000FF8000FF8000FF800000FF0000FF00FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF800000FF0000FF00FF80000000FFFF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF800000FF00
00FF0000FF00FF8000FF8000FF0000FF800000FF00FF8000FF0000FF8000FF8000FF8000
FF800000FF00FF80000000FFFF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000
00FF0000FF00FF80000000FF0000FF0000FFFF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF000000FF00FF8000
00FF00FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000FF0000FF8000FF8000
FF800000FF000000FF0000FF00FF00FF8000FF8000FF0000FF8000FF800000FF0000FF00
FF8000FF0000FF8000FF8000FF8000FF0000FF8000FF8000FF800000FF00FF8000FF8000
00FF000000FFFF0000FF8000FF8000FF000000FF00FF8000FF8000FF8000FF8000FF0000
FF80000000FFFF80000000FFFF8000FF8000FF0000FF8000FF8000FF8000FF000000FF00
FF8000FF8000FF800000FF0000FF00FF800000FF00FF8000FF80000000FF0000FFFF8000
FF8000FF8000FF80000000FFFF8000
FF8000FF8000FF8000FF8000FF80000000FFFF800000FF00FF8000FF8000FF8000FF8000
00FF00FF800000FF000000FF0000FFFF8000FF8000FF8000FF8000FF0000FF0000FF8000
FF8000FF0000FF0000FF8000FF8000FF8000FF0000FF000000FF00FF000000FF0000FF00
0000FF0000FFFF8000FF80000000FFFF0000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF800000FF0000FF00FF800000FF00FF0000FF8000FF8000FF8000FF8000
FF80000000FFFF8000FF8000FF80000000FF0000FFFF8000FF800000FF0000FF00FF8000
FF8000FF8000FF800000FF00FF8000
00FF00FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF800000FF000000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000FF8000FF0000FF8000FF8000
FF800000FF00FF0000FF8000FF0000FF0000FF0000FF000000FF00FF8000FF8000FF8000
00FF00FF8000FF8000FF8000FF8000FF0000FF80000000FFFF8000FF8000FF8000FF8000
FF800000FF0000FF0000FF0000FF00FF8000FF0000FF0000FF0000FF8000FF8000FF8000
FF8000FF8000FF800000FF0000FF0000FF000000FF0000FFFF800000FF0000FF00FF8000
00FF000000FFFF8000FF8000FF8000
00FF000000FF0000FFFF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000
FF8000FF8000FF800000FF0000FF0000FF00FF8000FF800000FF00FF0000FF8000FF8000
00FF00FF000000FF00FF8000FF8000FF0000FF0000FF000000FF00FF8000FF8000FF8000
00FF00FF8000FF8000FF0000FF0000FF8000FF8000FF80000000FFFF8000FF800000FF00
00FF0000FF00FF8000FF8000FF8000FF80000000FFFF8000FF8000FF0000FF0000FF8000
FF8000FF800000FF00FF000000FF0000FF000000FF0000FFFF800000FF0000FF00FF8000
0000FFFF8000FF8000FF8000FF8000
00FF0000FF000000FF0000FF0000FF00FF00FF800000FF00FF8000FF8000FF8000FF8000
00FF00FF800000FF0000FF0000FF00FF800000FF0000FF0000FF0000FF0000FF00FF8000
00FF00FF800000FF00FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF8000FF0000FF0000FF8000FF000000FF0000FF0000FF0000FF0000FF00FF8000
FF8000FF8000FF8000FF800000FF00FF800000FF0000FF0000FF000000FFFF8000FF8000
FF8000FF80000000FFFF8000FF800000FF0000FF00FF8000FF8000FF8000FF800000FF00
00FF000000FFFF8000FF8000FF8000
00FF0000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000
00FF00FF800000FF0000FF0000FF00FF8000FF8000FF8000FF8000FF000000FF00FF8000
00FF00FF8000FF8000FF8000FF80000000FF0000FFFF800000FF00FF800000FF00FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000
00FF000000FF0000FFFF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000
FF8000FF8000FF8000FF800000FF00FF8000FF800000FF0000FF0000FF0000FF0000FF00
00FF00FF8000FF8000FF8000FF8000
FF800000FF0000FF00FF8000FF8000FF8000FF800000FF00FF8000FF8000FF0000FF0000
FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000FF8000FF800000FF00FF8000
FF8000FF8000FF0000FF800000FF0000FF0000FF00FF800000FF00FF8000FF800000FF00
0000FFFF8000FF8000FF8000FF8000FF800000FF00FF0000FF0000FF8000FF800000FF00
00FF0000FF0000FF0000FF0000FF000000FFFF800000FF0000FF0000FF00FF8000FF0000
FF8000FF0000FF0000FF0000FF800000FF0000FF000000FF0000FF0000FFFF8000FF8000
00FF00FF8000FF0000FF8000FF0000
FF8000FF0000FF8000FF8000FF8000FF8000FF800000FF000000FFFF8000FF0000FF0000
FF800000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000
00FF00FF8000FF8000FF800000FF0000FF0000FF00FF8000FF8000FF0000FF8000FF8000
00FF000000FF0000FFFF8000FF8000FF8000FF800000FF00FF8000FF000000FF00FF8000
0000FF00FF0000FF00FF800000FF0000FF00FF8000FF800000FF0000FF00FF8000FF0000
FF0000FF8000FF8000FF0000FF8000FF800000FF000000FF00FF0000FF0000FF00FF8000
FF8000FF0000FF0000FF0000FF8000
FF8000FF0000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF0000FF0000
FF00000000FFFF8000FF800000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000
FF80000000FFFF8000FF8000FF800000FF0000FF00FF8000FF0000FF0000FF8000FF0000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000
00FF0000FF0000FF00FF8000FF8000FF0000FF800000FF0000FF00FF8000FF8000FF0000
FF8000FF8000FF0000FF8000FF8000FF8000FF800000FF0000FF0000FF00FF8000FF0000
FF0000FF8000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF800000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF800000FF00FF800000FF00FF0000FF800000FF00
0000FF00FF00FF8000FF800000FF0000FF00FF800000FF00FF8000FF0000FF0000FF0000
FF8000FF800000FF00FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000
FF8000FF0000FF0000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF8000FF0000FF8000
FF0000FF8000FF0000FF8000FF8000
FF0000FF0000FF8000FF8000FF8000FF8000FF0000FF000000FF0000FF0000FF0000FF00
FF800000FF0000FF0000FF0000FF00FF800000FF0000FF00FF8000FF8000FF0000FF8000
00FF00FF800000FF00FF8000FF80000000FFFF8000FF80000000FF00FF0000FF00FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF0000FF800000FF00FF8000FF8000FF8000FF0000FF8000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF80000000FFFF0000FF8000FF8000
FF8000FF0000FF8000FF800000FF00
FF8000FF000000FF00FF0000FF8000FF8000FF0000FF8000FF800000FF000000FFFF8000
FF80000000FF00FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF80000000FF0000FF0000FF00FF0000FF0000FF0000FF00FF000000FF00
FF8000FF8000FF8000FF80000000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF0000FF8000FF000000FF00FF8000FF000000FF00FF8000FF8000FF8000FF0000FF8000
FF0000FF8000FF8000FF8000FF8000FF0000FF000000FF0000FF00FF8000FF8000FF0000
00FF0000FF0000FF0000FF00FF8000
FF8000FF8000FF0000FF8000FF8000FF0000FF8000FF0000FF8000FF80000000FFFF8000
0000FF00FF00FF8000FF8000FF8000FF800000FF00FF800000FF000000FFFF8000FF8000
FF0000FF8000FF800000FF0000FF000000FF00FF000000FF0000FFFF8000FF8000FF8000
FF0000FF800000FF00FF8000FF8000FF8000FF8000FF800000FF00FF800000FF00FF8000
0000FFFF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF0000FF8000FF8000
FF0000FF8000FF8000FF0000FF8000FF8000FF0000FF800000FF00FF8000FF0000FF0000
FF8000FF8000FF800000FF00FF8000
00FF00FF8000FF0000FF8000FF8000FF8000FF0000FF8000FF000000FF0000FF00FF8000
0000FF00FF00FF800000FF00FF800000FF00FF800000FF0000FF000000FFFF8000FF8000
FF8000FF8000FF800000FF0000FF0000FF0000FF00FF8000FF8000FF8000FF8000FF8000
FF8000FF800000FF00FF8000FF8000FF0000FF0000FF800000FF000000FFFF800000FF00
FF8000FF0000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF0000FF0000
FF8000FF0000FF8000FF8000FF8000FF0000FF800000FF00FF8000FF0000FF0000FF8000
FF8000FF800000FF0000FF0000FF00
FF8000FF8000FF0000FF8000FF8000FF8000FF800000FF0000FF00FF800000FF00FF8000
00FF0000FF00FF80000000FFFF8000FF800000FF0000FF0000FF0000FF0000FF00FF8000
FF0000FF800000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF00FF8000FF8000
FF0000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF0000FF8000
FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF800000FF00FF80000000FFFF8000
FF8000FF0000FF8000FF8000FF80000000FFFF80000000FF0000FFFF80000000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF00FF800000FF000000FF0000FFFF8000FF0000
FF8000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF8000FF8000FF8000FF8000
FF0000FF8000FF8000FF8000FF8000FF8000FF000000FF00FF8000FF8000FF8000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF8000FF8000FF0000
FF8000FF8000FF800000FF00FF0000FF8000FF0000FF0000FF8000FF800000FF0000FF00
00FF0000FF0000FF0000FF00FF8000
FF8000FF8000FF8000FF80000000FFFF8000FF800000FF0000FF00FF8000FF8000FF8000
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000FF0000
FF8000FF0000FF0000FF00000000FF00FF000000FFFF8000FF8000FF8000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF8000FF8000
FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF0000
FF800000FF000000FFFF8000FF0000FF8000FF0000FF0000FF0000FF800000FF0000FF00
00FF0000FF00FF8000FF8000FF8000
FF8000FF0000FF8000FF80000000FF00FF00FF8000FF8000FF8000FF8000FF800000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000FF0000
FF0000FF0000FF0000FF000000FF00FF800000FF0000FF0000FF00FF8000FF8000FF8000
FF0000FF8000FF8000FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000
FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF8000
FF800000FF00FF80000000FFFF8000FF0000FF0000FF0000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF0000
FF0000FF800000FF0000FF00FF800000FF00FF8000FF800000FF0000FF0000FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000FF8000
FF8000FF0000FF8000FF8000FF0000FF800000FF00FF8000FF8000FF8000FF8000FF8000
FF0000FF0000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF0000
00FF00FF80000000FF0000FFFF8000FF8000FF8000FF8000FF0000FF0000FF8000FF0000
00FF00FF8000FF8000FF0000FF0000
00FF00FF8000FF800000FF0000FF000000FF00FF00FF800000FF00FF800000FF0000FF00
00FF00FF800000FF0000FF0000FF0000FF00FF8000FF8000FF8000FF8000FF0000FF8000
00FF00FF0000FF0000FF800000FF00FF8000FF0000FF8000FF8000FF8000FF8000FF8000
FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF800000FF00
FF8000FF800000FF0000FF0000FF00FF8000FF8000FF8000FF0000FF8000FF0000FF0000
FF800000FF00FF800000FF00FF8000FF800000FF00FF8000FF8000FF0000FF8000FF0000
FF8000FF0000FF8000FF8000FF8000
0000FFFF800000FF0000FF0000FF000000FFFF8000FF8000FF8000FF800000FF0000FF00
00FF00FF0000FF800000FF00FF8000FF800000FF00FF8000FF800000FF00FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF800000FF00FF8000FF8000
FF0000FF8000FF8000FF800000FF000000FF0000FFFF800000FF00FF8000FF0000FF8000
0000FFFF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000FF0000
FF8000FF800000FF00FF8000FF8000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.997078 0.261905 1.23734 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .59989 L
.2381 .59989 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .59989 L
.2381 .59989 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .01272 L
.47619 .01272 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .01272 L
.47619 .01272 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .0123 L
.71429 .0123 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .0123 L
.71429 .0123 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .01689 L
.95238 .01689 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .01689 L
.95238 .01689 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF00000000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.261905 0.997078 0.5 1.23734 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .02586 L
.2381 .02586 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .02586 L
.2381 .02586 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .32727 L
.47619 .32727 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .32727 L
.47619 .32727 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .25468 L
.71429 .25468 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .25468 L
.71429 .25468 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .034 L
.95238 .034 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .034 L
.95238 .034 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF000000FF0000FF000000FF00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF8000FF000000FF000000FF
00FF000000FF0000FF00FF00FF8000FF8000FF8000FF800000FF0000FF0000FF0000FF00
FF0000FF000000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF
00FF000000FF00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00FF000000FF000000FF00FF00
0000FFFF800000FF0000FF0000FF00
FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF00FF00
00FF000000FF0000FF00FF0000FF0000FF000000FFFF0000FF000000FF0000FF000000FF
00FF000000FF0000FF00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00
FF0000FF0000FF000000FF000000FF0000FF0000FF00FF0000FF0000FF000000FF00FF00
0000FF0000FF00FF0000FF000000FF0000FF00FF0000FF00FF0000FF000000FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF000000FF00
00FF0000FF0000FF0000FF0000FF00
FF000000FF0000FF0000FF000000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF00FF000000FF00FF000000FF0000FF0000FF0000FFFF8000FF80000000FF0000FF
00FF0000FF000000FF00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF00FF0000
FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF0000FF00FF8000FF8000FF000000FF0000FF00
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF
00FF0000FF0000FF000000FF00FF00
0000FF00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF00FF00
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
00FF000000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
FF0000FF8000FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF00FF000000FF00FF000000FF00FF000000FF00FF00
0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF00FF00FF8000FF8000
FF8000FF800000FF000000FF0000FF
0000FFFF80000000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF00FF000000FF00FF0000FF000000FF00FF000000FF00FF000000FF00FF000000FF
0000FF00FF00FF80000000FF0000FFFF800000FF0000FF000000FF00FF000000FF00FF00
00FF000000FF00FF0000FF0000FF000000FF00FF000000FF0000FFFF0000FF000000FF00
00FF000000FF00FF000000FF00FF00
0000FF00FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF
00FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF00FF000000FF0000FF0000FF
FF0000FF8000FF80000000FFFF8000FF80000000FFFF80000000FF0000FF00FF000000FF
0000FF0000FF00FF000000FF0000FF00FF000000FF0000FF00FF000000FF0000FF00FF00
00FF0000FF000000FF0000FF0000FF
00FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF0000FF00FF8000FF0000FF8000FF80000000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00FF0000
FF000000FF0000FF000000FF0000FF
00FF000000FF00FF000000FF0000FF00FF0000FF0000FF000000FF0000FF00FF000000FF
00FF0000FF0000FF000000FF0000FF0000FF00FF00FF8000FF800000FF0000FF000000FF
00FF0000FF000000FF00FF0000FF0000FF0000FF000000FF00FF000000FF0000FF00FF00
00FF000000FF00FF000000FF0000FF00FF000000FF0000FF00FF000000FF00FF000000FF
00FF0000FF000000FF00FF000000FF00FF000000FF0000FF00FF0000FF000000FF00FF00
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF00FF8000
00FF0000FF000000FF0000FF0000FF
00FF000000FF00FF000000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF000000FF00FF000000FF00FF000000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF00FF000000FF00FF0000FF0000FF000000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF00FF000000FF
00FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF
00FF0000FF000000FF0000FF00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF00FF000000FF0000FF000000FF0000FF0000FFFF8000FF8000FF800000FF000000FF
00FF0000FF0000FF0000FF000000FF00FF0000FF000000FF00FF000000FF0000FF00FF00
00FF0000FF000000FF00FF000000FF0000FF00FF0000FF00FF8000FF8000FF8000FF8000
FF0000FF000000FF0000FF000000FF
00FF000000FF00FF000000FF0000FF00FF000000FF0000FF00FF0000FF000000FF00FF00
0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FFFF8000FF8000FF800000FF00FF0000
0000FFFF000000FF0000FF000000FF0000FF00FF00FF8000FF800000FF0000FF00FF0000
FF000000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF000000FF00FF000000FF00FF0000FF0000FF0000FF00
FF0000FF0000FF000000FF000000FF
00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF00FF0000FF000000FF00FF0000FF000000FF00FF0000FF000000FF00FF00
00FF000000FF0000FF0000FF0000FF0000FF0000FFFF8000FF8000FF800000FF00FF8000
FF0000FF8000FF800000FF000000FF0000FF0000FFFF8000FF0000FF800000FF0000FF00
FF000000FF0000FF000000FF00FF0000FF0000FF0000FF000000FF0000FF00FF0000FF00
00FF000000FF0000FF00FF000000FF0000FF00FF000000FF00FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF000000FF00FF0000FF000000FF0000FF00FF0000FF000000FF0000FF
0000FFFF80000000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF000000FF00FF000000FF0000FF00FF000000FF0000FFFF0000FF0000
FF0000FF00000000FF00FF000000FF00FF0000FF00FF0000FF0000FF80000000FFFF0000
FF00000000FFFF80000000FFFF8000FF80000000FF0000FF00FF000000FF00FF0000FF00
00FF000000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00
0000FF00FF0000FF000000FF00FF00
0000FF00FF000000FF0000FF00FF000000FF00FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FFFF8000FF80000000FF00FF00
00FF0000FF0000FF000000FF00FF0000FF000000FF0000FF00FF0000FF00FF000000FF00
00FF000000FF00FF0000FF000000FF0000FFFF8000FF000000FF00FF80000000FFFF8000
FF800000FF00FF800000FF00FF8000FF800000FF0000FF0000FF000000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF000000FF0000FF00FF0000FF00
00FF0000FF0000FF000000FF0000FF
00FF000000FF0000FF0000FF00FF000000FF0000FF00FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF000000FF0000FF00FF000000FFFF8000FF8000FF8000
FF80000000FF00FF0000FF000000FF00FF00FF0000FF000000FF000000FF00FF0000FF00
00FF000000FF0000FF00FF0000FF0000FF000000FF00FF0000FF000000FF0000FF00FF00
0000FF00FF000000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF000000FF0000FF00FF00
00FF000000FF0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF00FF000000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF00FF000000FF
00FF000000FF0000FF0000FF00FF0000FF0000FF000000FF00FF0000FF000000FF00FF00
0000FF00FF0000FF0000FF000000FF0000FF00FF0000FF000000FF0000FF0000FF00FF00
00FF0000FF0000FF000000FFFF8000
00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF000000FF00FF00
0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF00FF000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF0000FF0000FF00FF00
0000FFFF8000FF800000FF000000FF0000FF00FF000000FF0000FF0000FF00FF0000FF00
00FF0000FF000000FF0000FF0000FF00FF000000FF0000FF0000FF00FF000000FF00FF00
0000FF0000FF0000FFFF80000000FF0000FF0000FF00FF0000FF000000FF00FF0000FF00
FF000000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF000000FF00FF0000FF00
00FF000000FF00FF0000FF000000FF00FF0000FF0000FF000000FF0000FF00FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF00FF80000000FF0000FF00FF0000FF0000FF00
00FF0000FF000000FF00FF000000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF
00FF0000FF0000FF000000FF00FF00
00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF000000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF000000FF00FF000000FF0000FF00FF000000FF00FF0000FF0000FF000000FF00FF00
00FF000000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF00
00FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF00FF000000FF00FF00
00FF0000FF0000FF0000FF000000FF
00FF000000FF0000FF0000FF00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF00FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF0000FF00FF0000FF000000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF00FF00FF000000FF0000FF000000FF0000FF0000FF00FF000000FF
0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF00FF000000FF
FF8000FF8000FF80000000FF0000FF
00FF0000FF000000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF000000FF0000FF0000FF0000FF00FF00FF8000
00FF0000FF0000FF000000FF0000FF00FF000000FF00FF0000FF000000FF0000FF0000FF
00FF000000FF0000FF00FF0000FF000000FF0000FF00FF0000FF000000FF0000FF00FF00
0000FF0000FF0000FF00FF0000FF0000FF000000FF00FF000000FF0000FF0000FF0000FF
00FF0000FF0000FF000000FF00FF00
0000FF0000FF00FF000000FF0000FF00FF0000FF000000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF00FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF000000FF00FF0000FF00
00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00FF800000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF00FF0000FF00
0000FFFF800000FF000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF0000FF000000FF00FF000000FF00FF000000FF00FF00
00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF
0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF000000FF00FF00
0000FF0000FF0000FF00FF0000FF00
00FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF00FF0000FF00
00FF0000FF000000FF0000FF00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF00FF000000FF0000FF00FF000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF
00FF000000FF0000FF0000FF00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF0000FF000000FF00FF0000FF0000FF0000FF000000FF00FF00
FF80000000FF0000FF0000FF0000FF0000FF0000FFFF8000FF8000FF800000FF0000FF00
0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF000000FF00FF000000FF0000FF0000FF0000FFFF80000000FF0000FF
0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF00FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF00FF000000FF00FF000000FFFF800000FF00
0000FF00FF0000FF000000FF00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF00
00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF000000FF00FF000000FFFF8000
FF8000FF80000000FF0000FF00FF00
0000FF00FF000000FF00FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF
00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF00
00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF00
00FF0000FF000000FF00FF0000FF000000FF00FF000000FF00FF0000FF0000FF000000FF
00FF000000FF0000FF0000FF0000FF
00FF000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF00FF000000FF0000FF00FF0000FF00
FF800000FF000000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF0000FF00FF000000FF0000FF
00FF0000FF000000FF0000FF0000FF00FF0000FF0000FF000000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF000000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF000000FF00FF0000FF000000FF00FF0000FF000000FF0000FF00FF000000FF
0000FF0000FF0000FF00FF000000FF
00FF0000FF0000FF000000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF00FF00
00FF0000FF000000FF00FF0000FF0000FF000000FF00FF000000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF00FF0000
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF
0000FF0000FF00FF000000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FFFF80000000FFFF80000000FF
0000FF0000FF0000FF00FF000000FF
00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF00FF000000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF
0000FF0000FF0000FF00FF0000FF0000FF000000FF00FF000000FF0000FF00FF0000FF00
00FF0000FF000000FF00FF0000FF0000FF0000FF000000FF00FF000000FF00FF0000FF00
00FF000000FF0000FF00FF000000FF
0000FF00FF0000FF000000FF00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF00FF000000FF0000FF00FF0000FF000000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF0000FF000000FF00FF000000FF00FF0000FF00
00FF00FF80000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF000000FF
00FF000000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF00
00FF0000FF000000FF0000FF00FF000000FF0000FF00FF0000FF000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF00
0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF000000FFFF8000
00FF0000FF000000FF0000FF00FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF000000FF00FF00
00FF00FF00000000FF00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF00
0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF00
0000FF00FF000000FF00FF000000FFFF0000FF0000FF000000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF00
00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF000000FF00FF00FF8000FF8000
FF000000FF0000FF0000FF000000FF
00FF00FF80000000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF00FF00
00FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00
00FF000000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FFFF8000
FF800000FF000000FF0000FF00FF00
0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF00
0000FF00FF000000FF00FF00FF8000FF800000FF0000FF0000FF0000FF000000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF
00FF000000FF0000FF00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00FF0000
00FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF000000FF0000FFFF00000000FF00FF0000FF000000FF0000FF00FF00
00FF000000FFFF8000FF800000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF000000FF00FF000000FF00FF0000FF0000FF0000FF000000FFFF80000000FF0000FF
0000FF00FF0000FF0000FF0000FF00
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.997078 0.738095 1.23734 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .01335 L
.2381 .01335 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .01335 L
.2381 .01335 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .25885 L
.47619 .25885 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .25885 L
.47619 .25885 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .3402 L
.71429 .3402 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .3402 L
.71429 .3402 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .02941 L
.95238 .02941 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .02941 L
.95238 .02941 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF
0000FF00FF0000FF000000FF0000FF00FF000000FF0000FFFF80000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF00FF0000FF0000FF0000FF000000FF00FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF000000FF
00FF000000FF00FF0000FF000000FF0000FF00FF0000FF000000FF0000FF00FF000000FF
0000FF0000FF00FF0000FF000000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF000000FF00FF00
00FF000000FF00FF000000FF00FF000000FF00FF000000FF0000FF0000FF00FF0000FF00
0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF000000FF00FF000000FF
00FF0000FF000000FF0000FF00FF0000FF000000FF0000FF00FF0000FF000000FF00FF00
0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF00FF000000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF00FF000000FF00FF0000FF000000FF0000FF00FF000000FF0000FF
0000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF
0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF00FF80000000FFFF80000000FF00FF0000FF00FF800000FF0000FF00
0000FF00FF0000FF000000FF0000FF0000FFFF80000000FFFF80000000FF00FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF000000FF0000FFFF8000FF8000FF80000000FF
00FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF0000FF0000FF0000FF000000FF
00FF000000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF00FF000000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00
0000FF00FF000000FF00FF0000FF000000FFFF8000FF8000FF8000FF8000FF8000FF8000
0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF00FF00
0000FFFF800000FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF000000FF
00FF000000FF00FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF000000FF
00FF000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FFFF800000FF00FF8000
FF80000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000FF8000FF000000FF0000FF00
0000FF0000FF00FF0000FF0000FF000000FF0000FF00FF0000FF000000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF80000000FF0000FF00FF0000FF00
00FF000000FF0000FF0000FF0000FF
00FF000000FF00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF000000FF
0000FF0000FF0000FF00FF000000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF
00FF000000FF00FF0000FF000000FFFF8000FF80000000FFFF8000FF800000FF0000FF00
0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF000000FF
00FF0000FF0000FF0000FF000000FF00FF000000FFFF80000000FFFF80000000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FFFF80000000FF00FF0000FF000000FF00FF000000FF0000FF0000FF00FF000000FF
00FF000000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF00FF0000FF0000FF00
00FF000000FF00FF000000FF00FF0000FF000000FF0000FF00FF000000FF0000FF0000FF
00FF000000FF0000FF00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
0000FF00FF0000FF0000FF0000FF000000FF00FF000000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF0000FF0000FF000000FF0000FF
0000FF00FF000000FF00FF000000FF
0000FF00FF0000FF0000FF0000FF000000FF00FF000000FF00FF0000FF000000FF0000FF
00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FF00FF000000FF00FF0000FF000000FF0000FF0000FF00FF0000FF000000FF
0000FF00FF0000FF0000FF0000FF000000FF00FF0000FF000000FF00FF000000FF00FF00
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF00FF00
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF00
00FF000000FF00FF000000FF0000FF0000FF00FF000000FF00FF0000FF0000FF000000FF
FF800000FF00FF80000000FF00FF0000FF0000FF0000FF000000FF0000FF00FF0000FF00
0000FF0000FF00FF000000FF0000FFFF80000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF800000FF000000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF000000FF
00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FFFF8000
0000FFFF8000FF80000000FF0000FFFF80000000FF0000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF00FF0000FF00
00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF00FF0000FF000000FF00FF000000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF000000FF00FF0000FF00
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF00FF00
0000FF00FF000000FF0000FF00FF000000FF0000FF0000FF00FF0000FF0000FF000000FF
0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF000000FF0000FF00FF000000FF00FF0000FF000000FF0000FF00FF000000FF
0000FF0000FF0000FF00FF000000FF
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF
0000FF0000FF00FF000000FF0000FF00FF000000FF00FF000000FF0000FF00FF0000FF00
00FF0000FF000000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
00FF000000FF00FF000000FF00FF00
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF00
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF00FF000000FF0000FF
0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF000000FF
0000FF00FF000000FF00FF0000FF000000FF0000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF0000FF000000FF
0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF00
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF00
0000FF00FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF00FF000000FF0000FF00FF000000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF000000FF00FF000000FF00FF000000FF0000FF
0000FF0000FF0000FFFF80000000FF00FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF0000FF0000FF0000FF
00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF000000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF00FF8000FF8000
FF80000000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF0000FF00
00FF000000FF0000FF00FF000000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
0000FF0000FF0000FF0000FF00FF00
0000FF00FF000000FF0000FF0000FF0000FF00FF0000FF000000FF00FF000000FF0000FF
0000FF00FF000000FF0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF00FF00
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF000000FFFF800000FF0000FF000000FF00FF00FF80000000FF00FF00
00FF0000FF0000FF0000FF000000FF0000FF00FF000000FF0000FF0000FFFF80000000FF
00FF0000FF000000FF00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF00
0000FF00FF0000FF000000FF0000FF
00FF0000FF0000FF000000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF
00FF0000FF0000FF000000FF0000FF0000FF00FF000000FF00FF0000FF000000FFFF8000
FF800000FF000000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF00
00FF000000FF0000FF0000FF0000FF0000FFFF00000000FFFF8000FF800000FF0000FF00
00FF000000FF0000FF00FF000000FF00FF000000FF0000FF0000FF00FF000000FF0000FF
FF8000FF80000000FF0000FF00FF000000FF0000FF00FF0000FF00FF8000FF8000FF8000
0000FF0000FF0000FF0000FF00FF00
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF00FF000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF00
FF800000FF000000FF0000FF0000FF00FF000000FF0000FF00FF0000FF0000FF0000FF00
0000FF0000FF00FF000000FFFF800000FF00FF80000000FF00FF000000FF00FF000000FF
00FF000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF0000FF0000FF00
0000FF00FF00FF80000000FF00FF00
0000FF0000FF00FF000000FF00FF0000FF000000FF00FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF00FF8000
0000FF00FF00FF80000000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF00FF00FF0000FF0000FF0000
00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00FF80000000FFFF8000
0000FF00FF0000FF000000FF00FF000000FF0000FF00FF000000FF0000FF00FF00FF8000
0000FF00FF0000FF000000FF0000FF
00FF000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF00
00FF000000FF0000FFFF8000FF80000000FF0000FF00FF000000FF0000FF0000FF0000FF
00FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00
0000FF0000FF00FF000000FF0000FF00FF000000FF00FF0000FF00FF800000FF0000FF00
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF800000FF00
0000FF0000FF0000FF00FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF000000FF00FF0000FF00
0000FF00FF00FF800000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00
0000FF0000FF0000FF00FF0000FF000000FF00FF0000FF000000FF0000FF0000FF00FF00
00FF0000FF0000FF000000FF0000FF00FF000000FF0000FF0000FF00FF000000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF
FF80000000FF0000FF00FF0000FF00
0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000
0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF
00FF000000FF00FF00FF800000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF000000FF0000FF
0000FF00FF000000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF00FF000000FF
0000FF0000FF0000FF00FF000000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF
00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF000000FF00FF0000FF000000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF00FF000000FF00FF00
00FF000000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF00FF0000FF000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF00
0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
0000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF00FF00
0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF00FF00
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF
0000FF00FF000000FF0000FF00FF000000FF00FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF00FF00
00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF000000FF00FF000000FF00FF0000FF000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF00FF00
0000FF00FF000000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF00FF000000FF00FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF00FF0000FF00
00FF0000FF0000FF000000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF000000FF00FF000000FF0000FF00FF000000FF0000FF00FF0000FF0000FF00
0000FF00FF0000FF000000FF00FF00
00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF0000FF0000FF00FF0000FF0000FF000000FF00FF0000FF000000FF00FF0000FF00
00FF0000FF0000FF000000FF00FF000000FF00FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF000000FF0000FF0000FF00FF00
00FF000000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF
0000FF0000FF00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF00FF00
00FF000000FF0000FF00FF0000FF0000FF000000FF00FF0000FF000000FF0000FF00FF00
0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF
0000FF00FF000000FF00FF000000FF00FF000000FF0000FF00FF000000FF0000FF0000FF
00FF0000FF000000FF0000FF00FF00
00FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF00FF00
0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF00FF000000FF00FF00
00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF000000FF00FF0000FF000000FF
0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF
0000FF0000FF00FF000000FF00FF00
0000FF00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF00FF000000FF0000FF
0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF00FF000000FF0000FF
0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF00
00FF000000FF0000FF00FF000000FF0000FF00FF0000FF000000FF00FF0000FF000000FF
00FF0000FF000000FF00FF000000FF
00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF
0000FF0000FF00FF000000FF0000FF00FF0000FF0000FF000000FF00FF000000FF0000FF
0000FF0000FF00FF000000FF00FF0000FF000000FF00FF000000FF00FF0000FF000000FF
00FF0000FF0000FF000000FF0000FF0000FF00FF0000FF000000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FFFF8000FF8000FF8000FF8000
0000FF00FF0000FF0000FF00FF800000FF000000FF00FF00FF80000000FFFF80000000FF
0000FF0000FF00FF000000FF00FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF
FF80000000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF00FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF
00FF0000FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00
00FF000000FF0000FF00FF0000FF000000FF0000FF00FF000000FF00FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF00FF0000FF0000FF00
00FF000000FF00FF0000FF000000FF
00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF000000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF000000FF0000FF00FF0000FF000000FF0000FF00FF000000FF00FF000000FF0000FF
00FF000000FF00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF000000FF0000FF00FF0000FF0000FF0000FF000000FF
00FF00FF000000FF0000FF00FF8000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.738095 0.997078 0.97619 1.23734 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .12786 L
.2381 .12786 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .12786 L
.2381 .12786 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .04359 L
.47619 .04359 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .04359 L
.47619 .04359 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .04109 L
.71429 .04109 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .04109 L
.71429 .04109 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .42927 L
.95238 .42927 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .42927 L
.95238 .42927 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF0000FF8000FF0000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
0000FFFF8000FF8000FF8000FF8000FF80000000FF0000FFFF8000FF8000FF8000FF0000
FF8000FF0000FF0000FF00000000FF0000FF0000FFFF8000FF8000FF800000FF00FF8000
FF8000FF0000FF800000FF00FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF0000FF0000FF0000FF8000
FF8000FF8000FF8000FF8000FF800000FF00FF80000000FFFF0000FF000000FF0000FF00
FF800000FF0000FF00FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF80000000FFFF0000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF80000000FFFF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF00000000FF00FF000000FF
0000FF0000FF00FF00FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF000000FF00FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000
FF8000FF8000FF8000FF800000FF000000FF0000FF00FF00FF0000FF0000FF8000FF0000
0000FFFF800000FF00FF8000FF0000FF0000FF00000000FFFF8000FF8000FF8000FF0000
00FF000000FF00FF0000FF00FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000
FF8000FF0000FF0000FF0000FF000000FF00FF800000FF000000FFFF0000FF0000FF0000
FF0000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF0000FF8000FF8000
FF8000FF8000FF8000FF800000FF00FF8000FF8000FF0000FF0000FF8000FF8000FF8000
FF8000FF8000FF80000000FF0000FFFF0000FF8000FF8000FF8000FF8000FF8000FF8000
00FF000000FFFF8000FF8000FF0000FF0000FF8000FF0000FF8000FF8000FF8000FF8000
FF80000000FFFF8000FF8000FF8000FF0000FF00000000FF0000FFFF0000FF0000FF0000
FF0000FF0000FF8000FF800000FF00
00FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
0000FF00FF0000FF000000FFFF80000000FFFF8000FF0000FF0000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF0000FF00
0000FF0000FFFF0000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000
00FF00FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF800000FF0000FF00
00FF00FF8000FF8000FF8000FF00000000FF00FF00FF0000FF0000FF0000FF8000FF8000
FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF800000FF000000FF00FF000000FF0000FFFF8000FF8000FF8000
FF00000000FF00FF000000FF00FF0000FF000000FF0000FF0000FF00FF00FF000000FF00
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF8000FF8000FF800000FF000000FF00FF00FF8000FF8000FF8000FF8000FF0000
FF00000000FFFF8000FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000
FF0000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000FF8000
FF8000FF800000FF00FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF000000FF00FF8000
FF0000FF0000FF8000FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF000000FF00FF800000FF0000FF00
FF8000FF8000FF8000FF8000FF800000FF000000FFFF8000FF8000FF0000FF0000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF80000000FFFF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF800000FF00
FF0000FF0000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF8000FF8000
FF0000FF000000FF0000FF0000FF00FF0000FF8000FF8000FF0000FF8000FF000000FF00
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF800000FF000000FFFF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF0000FF0000FF8000
FF8000FF8000FF0000FF0000FF800000FF000000FFFF80000000FF00FF00FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF0000FF0000FF0000
FF0000FF8000FF8000FF0000FF0000FF80000000FFFF80000000FFFF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF800000FF000000FF0000FF0000FFFF0000FF8000FF8000FF8000
FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000
FF0000FF0000FF000000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF0000FF8000FF8000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF8000FF8000FF0000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF80000000FFFF8000FF8000
00FF0000FF000000FFFF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF0000FF8000FF000000FF00
0000FF0000FFFF8000FF8000FF800000FF0000FF00FF800000FF00FF8000FF8000FF8000
FF0000FF8000FF8000FF800000FF00FF8000FF0000FF8000FF8000FF8000FF8000FF8000
FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF000000FF000000FFFF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF8000FF0000FF800000FF00FF8000FF0000FF8000FF8000FF8000
FF80000000FFFF8000FF0000FF8000FF0000FF8000FF800000FF00FF8000FF0000FF0000
FF0000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000
FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF0000FF8000FF8000FF8000
FF0000FF0000FF00000000FF0000FFFF8000FF8000FF0000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF000000FF00FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF0000
FF0000FF8000FF8000FF0000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000
FF8000FF80000000FFFF8000FF8000
FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF80000000FFFF80000000FFFF8000FF8000
FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF0000FF0000FF000000FF00
FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF8000FF8000FF8000FF0000FF0000
FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF8000
FF8000FF8000FF0000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF800000FF00
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000
FF800000FF000000FFFF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF800000FF000000FF00FF0000FF00
FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF00000000FF
FF00000000FF0000FF00FF00FF80000000FFFF8000FF8000FF8000FF8000FF8000FF8000
00FF000000FFFF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
0000FFFF8000FF0000FF0000FF0000FF0000FF8000FF800000FF00FF800000FF000000FF
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF000000FF00
00FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000
FF80000000FFFF8000FF8000FF8000FF8000FF8000FF80000000FF0000FF00FF00FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF0000FF8000FF8000FF0000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF0000FF8000FF0000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF8000FF8000
FF8000FF800000FF00FF8000FF800000FF00FF0000FF0000FF0000FF80000000FF00FF00
FF8000FF8000FF8000FF8000FF800000FF0000FF000000FF0000FFFF8000FF8000FF8000
FF8000FF80000000FFFF8000FF8000FF8000FF0000FF0000FF8000FF00000000FFFF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000
00FF0000FF00FF0000FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000FF0000FF8000
FF0000FF0000FF8000FF8000FF80000000FFFF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
FF800000FF00FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF00000000FFFF00000000FF
0000FFFF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
00FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF000000FF0000FF00FF8000FF8000FF8000FF8000FF80000000FFFF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
00FF0000FF00FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF8000
0000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF0000FF00FF00FF8000
FF8000FF8000FF8000FF8000FF8000
0000FFFF0000FF0000FF0000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000
FF000000FF000000FFFF8000FF80000000FFFF8000FF8000FF8000FF8000FF8000FF0000
FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
00FF000000FFFF8000FF800000FF00FF0000FF0000FF0000FF0000FF8000FF80000000FF
0000FF0000FFFF0000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF000000FFFF8000
FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF000000FF00FF0000FF8000FF8000FF8000
FF800000FF00FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF0000FF0000FF8000
FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000
FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF80000000FFFF8000FF8000FF0000FF8000FF80000000FF0000FF0000FF
FF800000FF00FF8000FF800000FF0000FF000000FFFF8000FF8000FF8000FF8000FF8000
FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF800000FF000000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF0000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF0000
FF0000FF8000FF8000FF8000FF8000FF0000FF000000FF0000FF0000FF0000FF00FF0000
FF0000FF8000FF8000FF8000FF8000
0000FF0000FFFF8000FF8000FF8000FF8000FF800000FF000000FFFF80000000FF0000FF
FF00000000FFFF0000FF0000FF8000FF8000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF8000FF0000FF8000FF800000FF00FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF80000000FFFF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF8000FF80000000FF0000FF
FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF8000
FF8000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF0000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF0000FF0000FF8000
FF8000FF8000FF8000FF0000FF0000FF8000FF80000000FFFF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF0000FF8000FF8000FF800000FF00FF800000FF000000FF00FF0000FF00FF8000FF8000
FF8000FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF8000
FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF8000FF8000
FF0000FF00000000FF0000FF0000FFFF80000000FFFF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000
FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF0000FF0000FF8000FF0000FF8000FF8000
FF0000FF8000FF0000FF8000FF8000
00FF00FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF80000000FFFF800000FF00FF8000FF8000FF8000FF8000
00FF0000FF00FF8000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF0000FF8000FF8000FF8000FF8000
FF8000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF800000FF00
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF0000FF0000FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF80000000FF
00FF00FF0000FF8000FF8000FF8000FF0000FF0000FF8000FF0000FF8000FF8000FF0000
FF0000FF0000FF0000FF0000FF800000FF0000FF000000FF0000FF0000FFFF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF0000FF0000FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF000000FF000000FF0000FF0000FF
FF8000FF8000FF8000FF8000FF0000FF800000FF00FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF0000FF8000FF8000
FF0000FF0000FF8000FF0000FF0000FF8000FF0000FF8000FF8000FF0000FF0000FF8000
FF0000FF0000FF80000000FFFF0000FF8000FF8000FF8000FF8000FF8000FF000000FF00
0000FF0000FF0000FFFF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF0000
FF0000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF80000000FFFF8000FF0000FF8000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF80000000FF
FF80000000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000
FF8000FF8000FF0000FF0000FF0000
FF8000FF80000000FF0000FF0000FFFF8000FF8000FF8000FF8000FF0000FF0000FF8000
FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF800000FF00FF8000FF8000FF8000FF8000FF800000FF0000FF00FF0000FF8000FF8000
FF8000FF8000FF8000FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 1.23734 0.261905 1.4776 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .47787 L
.2381 .47787 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .47787 L
.2381 .47787 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .02169 L
.47619 .02169 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .02169 L
.47619 .02169 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .12577 L
.71429 .12577 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .12577 L
.71429 .12577 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .01647 L
.95238 .01647 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .01647 L
.95238 .01647 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF00000000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF
0000FFFF0000FF00000000FF0000FF
FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF00000000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FFFF0000
0000FF00FF00FF0000FF0000FF0000FF0000FF00000000FFFF00000000FFFF0000FF0000
0000FF0000FFFF0000FF0000FF0000FF00000000FFFF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000
FF00000000FFFF0000FF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
0000FF00FF00FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
0000FFFF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF00000000FFFF0000
FF00000000FFFF0000FF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF8000FF8000FF0000FF00000000FF0000FFFF0000FF0000FF0000
FF0000FF00000000FFFF0000FF0000
0000FF0000FFFF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000
0000FFFF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF00000000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FFFF0000FF00000000FFFF0000
FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF00000000FF0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000
FF00000000FFFF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF00000000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF
FF00000000FF0000FFFF0000FF0000FF00000000FF0000FFFF0000FF00000000FF0000FF
0000FF00FF00FF00000000FFFF0000
0000FF0000FFFF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000
FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FFFF0000FF0000FF00000000FF0000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FFFF00000000FF
FF00000000FF0000FFFF0000FF0000FF0000FF00000000FFFF0000FF00000000FF0000FF
FF0000FF0000FF0000FF8000FF0000
0000FF0000FFFF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF00000000FF0000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FFFF0000FF0000FF00000000FFFF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF00000000FF
0000FF0000FF0000FFFF0000FF0000FF0000FF00000000FFFF0000FF0000FF00000000FF
FF0000FF0000FF0000FF800000FF00
0000FF0000FFFF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000
FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF00000000FF0000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FFFF0000FF00000000FFFF0000FF0000FF0000FF0000
FF0000FF00000000FF0000FF0000FF0000FFFF0000FF00000000FF0000FFFF00000000FF
0000FF0000FF0000FFFF0000FF0000FF0000FF00000000FFFF0000FF0000FF00000000FF
FF0000FF0000FF0000FF0000FF0000
0000FF0000FFFF00000000FFFF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000
FF00000000FFFF0000FF0000FF0000FF00000000FFFF0000FF00000000FFFF0000FF0000
FF0000FF00000000FF0000FF0000FF0000FFFF0000FF00000000FF0000FFFF00000000FF
0000FF0000FFFF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF80000000FFFF0000FF0000
0000FF0000FFFF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000
FF0000FF0000FF00000000FFFF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000
FF0000FF00000000FFFF0000FF00000000FF0000FFFF00000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FF0000FFFF0000FF0000FF0000
0000FFFF0000FF0000FF0000FF00000000FFFF0000FF00000000FFFF0000FF00000000FF
FF0000FF80000000FFFF0000FF0000
0000FF0000FFFF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000
FF00000000FFFF0000FF0000FF0000FF00000000FF0000FFFF00000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF00000000FF0000FFFF00000000FF0000FFFF00000000FFFF0000FF0000FF0000
0000FF0000FFFF00000000FF0000FF0000FFFF00000000FF0000FFFF00000000FFFF0000
FF0000FF0000FF0000FF0000FF00000000FF0000FFFF00000000FF0000FFFF00000000FF
0000FFFF80000000FFFF00000000FF
0000FF0000FFFF0000FF00000000FFFF0000FF0000FF0000FF00000000FF0000FFFF0000
0000FF0000FFFF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF00000000FFFF0000FF00000000FF0000FFFF00000000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000
0000FF00FF000000FF0000FF0000FF
0000FF00FF00FF0000FF00000000FFFF0000FF0000FF0000FF00000000FF0000FFFF0000
0000FFFF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF00000000FFFF00000000FFFF0000FF00000000FF0000FFFF00000000FFFF0000
FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FFFF0000FF0000FF0000
0000FF00FF0000FF000000FFFF0000
0000FF00FF00FF0000FF00000000FFFF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF00000000FFFF0000
FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000
FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FF0000FFFF00000000FF
0000FF0000FF0000FFFF0000FF0000
0000FF00FF00FF0000FF0000FF0000FF0000FF8000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF00000000FFFF0000
FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FF0000FFFF0000FF00000000FF
FF0000FF00000000FFFF0000FF0000
0000FF00FF00FF0000FF0000FF8000FF0000FF8000FF0000FF00000000FFFF0000FF0000
FF00000000FFFF0000FF0000FF0000FF0000FF00000000FFFF0000FF000000FF00FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000
FF0000FF0000FF00000000FF0000FFFF00000000FF0000FF0000FF0000FFFF00000000FF
0000FF0000FFFF0000FF0000FF00000000FF0000FF0000FF0000FFFF0000FF00000000FF
FF0000FF0000FF0000FF0000FF0000
0000FF00FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF00000000FFFF00000000FF0000FFFF00000000FF0000FF0000FF0000FFFF0000FF0000
FF00000000FFFF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF00000000FF
FF0000FF0000FF0000FF0000FF0000
0000FFFF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000
FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
00FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FF0000FF0000FFFF0000FF00000000FF0000FFFF0000FF0000
FF00000000FFFF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF00000000FF
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FF0000FF0000FF0000FFFF00000000FF0000FFFF00000000FF
0000FFFF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF00000000FF
0000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF
0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF
FF0000FF0000FF0000FF00000000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF00000000FFFF0000FF0000FF00000000FF0000FF0000FF0000FF0000FF0000FFFF0000
0000FFFF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FF
FF0000FF0000FF800000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF
FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
FF0000FF00000000FFFF0000FF00000000FF0000FFFF00000000FF0000FFFF0000FF0000
0000FFFF0000FF0000FF0000FF00000000FFFF00000000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FF0000FF
0000FFFF0000FF800000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF00000000FFFF0000FF00000000FF0000FFFF0000FF0000
00FF00FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000
FF00000000FFFF00000000FF0000FF
FF8000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF800000FF00FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FF
0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF00000000FFFF00000000FF0000FF
FF8000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF00000000FF0000FFFF0000FF8000FF0000FF0000FF0000FF0000FF0000FF00000000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FFFF00000000FFFF00000000FF0000FF0000FF0000FF0000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00
0000FF0000FF0000FFFF00000000FF
00FF00FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000
FF00000000FFFF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF00000000FFFF00000000FFFF00000000FF0000FF0000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF00000000FF
00FF00FF0000FF00000000FFFF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FFFF00000000FF
0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
00FF000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FFFF0000FF0000
0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
00FF000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000
0000FFFF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF00000000FF
0000FFFF0000FF0000FF0000FF0000
00FF000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FFFF00000000FF0000FFFF00000000FF00FF00FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF
0000FFFF0000FF0000FF0000FF0000
00FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF00000000FFFF0000FF0000FF0000FF00000000FF0000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF
FF0000FF00000000FF0000FFFF0000
00FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF
FF0000FF0000FF0000FF0000FF0000
00FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FF0000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000
00FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FF0000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF
FF0000FF0000FF0000FF0000FF0000
00FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FF0000FF
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF000000FF00FF8000FF0000
00FF0000FF00FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FF0000FF
0000FFFF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
0000FFFF0000FF8000FF0000FF0000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.261905 1.23734 0.5 1.4776 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .02836 L
.2381 .02836 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .02836 L
.2381 .02836 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .28847 L
.47619 .28847 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .28847 L
.47619 .28847 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .14809 L
.71429 .14809 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .14809 L
.71429 .14809 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .17688 L
.95238 .17688 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .17688 L
.95238 .17688 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF8000FF800000FF0000FF0000FF00FF800000FF0000FF0000FF00FF800000FF00FF8000
FF8000FF8000FF8000FF800000FF000000FF0000FF0000FF00FF0000FF0000FF00FF8000
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF00FF800000FF000000FF
0000FFFF800000FF0000FF0000FF0000FF000000FF00FF0000FF000000FF0000FF0000FF
00FF0000FF0000FF000000FF00FF0000FF000000FFFF00000000FF0000FFFF800000FF00
00FF0000FF0000FF000000FF00FF00FF8000FF8000FF8000FF800000FF00FF800000FF00
00FF000000FF0000FFFF800000FF00
FF8000FF800000FF0000FF0000FF00FF800000FF0000FF0000FF0000FF0000FF00FF8000
FF8000FF8000FF8000FF800000FF0000FF0000FF000000FF00FF0000FF0000FF00FF8000
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF00FF800000FF000000FF
0000FFFF8000FF800000FF0000FF0000FF00FF800000FF0000FF0000FF000000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF000000FFFF00000000FF0000FFFF800000FF00
00FF0000FF000000FF0000FFFF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00
00FF000000FF00FF00FF800000FF00
0000FFFF800000FF0000FF0000FF00FF800000FF0000FF0000FF0000FF0000FF00FF8000
FF8000FF8000FF8000FF800000FF0000FF0000FF000000FFFF800000FF00FF800000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF00FF800000FF000000FF
0000FFFF8000FF800000FF0000FF0000FF00FF000000FF0000FF0000FF000000FF0000FF
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FFFF800000FF00
00FF0000FF000000FF0000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
00FF0000FF0000FF00FF800000FF00
FF8000FF8000FF800000FF00FF8000FF8000FF800000FF0000FF0000FF0000FF00FF8000
FF8000FF8000FF800000FF0000FF0000FF0000FF000000FFFF800000FF00FF800000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF8000FF800000FF000000FF00FF000000FF
0000FF0000FFFF8000FF8000FF8000FF00000000FF00FF0000FF000000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF0000FFFF800000FF00
00FF0000FF000000FF0000FFFF8000FF8000FF8000FF800000FF00FF80000000FFFF8000
00FF0000FF0000FF00FF800000FF00
0000FF00FF00FF800000FF00FF800000FF00FF800000FF0000FF0000FF0000FF00FF8000
FF8000FF8000FF800000FF0000FF0000FF0000FF000000FFFF800000FF00FF800000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF80000000FF0000FF0000FF0000FF
0000FF0000FFFF800000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000
00FF0000FF00FF00000000FF00FF000000FF0000FF00FF00FF0000FF8000FF8000FF8000
00FF0000FF000000FF0000FFFF8000FF8000FF8000FF800000FF000000FF0000FFFF8000
00FF000000FF00FF00FF800000FF00
0000FF00FF00FF800000FF00FF800000FF00FF800000FF0000FF0000FF0000FF00FF8000
00FF00FF0000FF8000FF800000FF00FF800000FF000000FFFF800000FF00FF800000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF80000000FF0000FF0000FF0000FF
0000FF0000FFFF800000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000
0000FF0000FFFF00000000FF00FF000000FF0000FF00FF000000FFFF8000FF8000FF8000
00FF0000FF000000FF0000FFFF8000FF8000FF8000FF800000FF000000FF0000FFFF8000
00FF0000FF0000FF00FF800000FF00
0000FF00FF00FF800000FF00FF800000FF00FF800000FF0000FF0000FF00FF8000FF8000
0000FFFF0000FF8000FF800000FF00FF800000FF000000FFFF800000FF00FF800000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF8000FF80000000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FFFF00000000FF0000FF0000FF0000FF0000FFFF8000
0000FF0000FFFF00000000FF00FF000000FF0000FF00FF000000FF0000FFFF8000FF8000
00FF0000FF000000FF0000FFFF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
00FF0000FF0000FF00FF8000FF8000
0000FF0000FFFF800000FF00FF800000FF00FF800000FF0000FF0000FF00FF800000FF00
0000FFFF0000FF800000FF0000FF00FF00000000FF0000FFFF800000FF00FF800000FF00
00FF0000FF00FF800000FF0000FF0000FF00FF8000FF8000FF800000FF000000FF0000FF
0000FF0000FF0000FF00FF000000FFFF00000000FF0000FF0000FF0000FF0000FFFF8000
0000FF0000FF0000FF0000FF00FF00FF00000000FF00FF000000FF0000FFFF8000FF8000
00FF0000FF000000FF0000FFFF8000FF8000FF800000FF00FF8000FF8000FF800000FF00
00FF0000FF0000FF00FF8000FF8000
FF8000FF8000FF800000FF00FF8000FF8000FF800000FF0000FF0000FF00FF8000FF8000
0000FFFF0000FF800000FF0000FF00FF000000FF00FF8000FF800000FF00FF800000FF00
00FF0000FF00FF800000FF0000FF0000FF00FF8000FF800000FF0000FF000000FF0000FF
0000FFFF8000FF800000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF00FF00000000FF00FF000000FF0000FFFF8000FF8000
00FF0000FF000000FF0000FFFF8000FF8000FF800000FF00FF8000FF0000FF000000FF00
00FF0000FF00FF0000FF800000FF00
FF800000FF00FF800000FF00FF8000FF8000FF8000FF800000FF0000FF0000FF00FF8000
FF8000FF8000FF800000FF0000FF00FF800000FF00FF8000FF8000FF800000FF0000FF00
00FF0000FF00FF8000FF800000FF0000FF00FF8000FF800000FF0000FF0000FF000000FF
0000FFFF8000FF800000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF000000FF00FF00FF000000FF0000FF000000FF00FF0000FF000000FFFF8000FF8000
00FF0000FF0000FF00FF800000FF00FF800000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FFFF00000000FF00FF00
FF800000FF00FF800000FF00FF8000FF8000FF8000FF800000FF0000FF0000FF00FF8000
FF8000FF800000FF0000FF0000FF0000FF0000FF00FF8000FF8000FF800000FF00FF8000
00FF0000FF0000FF0000FF0000FF00FF8000FF800000FF0000FF0000FF0000FF000000FF
0000FFFF8000FF800000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF00FF8000FF800000FF0000FF0000FF00FF8000FF8000FF8000FF8000
00FF0000FF0000FF00FF8000FF800000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF800000FF00
FF800000FF0000FF0000FF00FF800000FF00FF800000FF0000FF0000FF0000FF00FF8000
FF8000FF800000FF0000FF0000FF0000FF0000FF00FF8000FF8000FF800000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FFFF0000
0000FFFF8000FF800000FF000000FF0000FF0000FF0000FF0000FF0000FFFF00000000FF
0000FF0000FFFF80000000FFFF800000FF0000FF0000FF0000FF00FF8000FF800000FF00
00FF0000FF0000FF00FF8000FF800000FF0000FF0000FF000000FFFF00000000FF0000FF
00FF000000FF0000FFFF800000FF00
FF800000FF0000FF0000FF00FF800000FF00FF800000FF0000FF0000FF0000FF0000FF00
FF8000FF800000FF0000FF0000FF00FF800000FF0000FF00FF8000FF800000FF00FF8000
00FF0000FF0000FF0000FF0000FF0000FF00FF800000FF0000FF0000FF000000FFFF0000
0000FF0000FFFF800000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000
0000FF0000FFFF80000000FFFF80000000FFFF800000FF0000FF00FF800000FF0000FF00
00FF0000FF0000FF00FF8000FF800000FF00FF800000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FFFF800000FF00
FF800000FF0000FF0000FF0000FF0000FF00FF800000FF0000FF0000FF0000FF0000FF00
00FF00FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF800000FF00FF80000000FF0000FF0000FF
0000FF0000FFFF800000FF0000FF0000FF000000FF0000FF0000FFFF00000000FF0000FF
FF00000000FF0000FF0000FFFF8000FF8000FF8000FF800000FF00FF800000FF0000FF00
FF800000FF00FF0000FF800000FF00FF800000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FFFF8000FF8000
00FF0000FF0000FF0000FF0000FF0000FF00FF800000FF0000FF0000FF0000FF0000FF00
00FF00FF8000FF800000FF00FF8000FF800000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
FF00000000FFFF00000000FFFF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00
FF800000FF000000FF0000FFFF800000FF0000FF0000FF0000FF00FF800000FF000000FF
0000FFFF00000000FFFF8000FF8000
00FF0000FF0000FF0000FF0000FF0000FF00FF8000FF8000FF800000FF0000FF0000FF00
00FF0000FF00FF800000FF00FF8000FF800000FF0000FF0000FF00FF800000FF0000FF00
00FF0000FF0000FF0000FF00FF800000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF00000000FFFF0000FF000000FF0000FF00FF8000FF80000000FFFF8000FF800000FF00
FF800000FF000000FF0000FFFF800000FF0000FF0000FF0000FF00FF800000FF000000FF
0000FFFF000000FF00FF8000FF8000
00FF0000FF0000FF0000FF0000FF0000FF00FF8000FF8000FF800000FF0000FF0000FF00
00FF0000FF00FF800000FF00FF8000FF800000FF0000FF0000FF00FF800000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF000000FF0000FF0000FF
0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF00FF80000000FF0000FF0000FFFF8000FF8000
FF800000FF000000FF0000FFFF800000FF0000FF00FF8000FF8000FF8000FF8000FF8000
00FF00FF800000FF00FF8000FF8000
00FF0000FF0000FF0000FF0000FF0000FF00FF8000FF800000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF800000FF0000FF00FF800000FF00FF800000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF8000FF800000FF000000FF0000FF0000FF
0000FFFF800000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000
0000FF0000FF0000FF0000FF00FF0000FF00FF80000000FF0000FF0000FFFF8000FF8000
FF800000FF000000FF0000FFFF800000FF0000FF0000FF00FF8000FF800000FF00FF8000
00FF00FF8000FF8000FF8000FF8000
00FF0000FF0000FF0000FF0000FF0000FF00FF800000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF8000FF800000FF00FF800000FF00FF800000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF8000FF800000FF0000FF000000FF0000FF
0000FFFF800000FF0000FF000000FF0000FFFF0000FF0000FF000000FF000000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF0000FF00FF80000000FFFF8000
FF800000FF000000FF0000FFFF800000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF0000FFFF8000FF8000FF8000
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF80000000FF0000FFFF800000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF800000FF0000FF0000FF000000FF0000FF
0000FFFF800000FF0000FF000000FFFF0000FF00000000FFFF000000FF000000FF0000FF
0000FF0000FF0000FF0000FFFF80000000FF00FF0000FF00FF8000FF800000FF0000FF00
00FF000000FF0000FF0000FFFF800000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF0000FFFF8000FF800000FF00
FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FFFF800000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF800000FF0000FF000000FF0000FF0000FF
0000FFFF800000FF00FF80000000FF0000FFFF00000000FF0000FF0000FF0000FFFF8000
0000FF0000FF0000FF0000FFFF8000FF800000FF0000FF00FF8000FF800000FF0000FF00
00FF000000FF0000FF0000FFFF8000FF8000FF800000FF00FF800000FF0000FF0000FF00
0000FF0000FFFF8000FF800000FF00
FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF00FF800000FF0000FF0000FF000000FF0000FFFF800000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF800000FF0000FF000000FF0000FF0000FF
0000FF0000FF00FF00FF00000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000
0000FF0000FF0000FF0000FFFF8000FF800000FF0000FF00FF8000FF800000FF0000FF00
00FF000000FF0000FF0000FFFF8000FF8000FF800000FF00FF8000FF800000FF000000FF
0000FFFF0000FF8000FF8000FF8000
FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
FF8000FF800000FF0000FF0000FF000000FF0000FFFF800000FF00FF800000FF0000FF00
00FF00FF800000FF00FF8000FF800000FF00FF800000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FFFF0000FF8000
0000FFFF00000000FF0000FFFF8000FF800000FF0000FF0000FF00FF800000FF00FF8000
00FF0000FF0000FF00FF8000FF8000FF8000FF800000FF00FF8000FF8000FF00000000FF
0000FF0000FFFF8000FF8000FF8000
FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
FF800000FF0000FF0000FF00FF8000FF80000000FFFF8000FF8000FF800000FF00FF8000
FF8000FF8000FF8000FF8000FF800000FF00FF800000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF000000FF0000FFFF8000FF8000FF800000FF0000FF0000FF00FF800000FF0000FF00
00FF0000FF000000FFFF8000FF8000FF8000FF800000FF00FF8000FF00000000FF0000FF
0000FF0000FF0000FFFF8000FF8000
FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000
FF8000FF8000FF8000FF8000FF800000FF0000FF0000FF0000FF0000FF00FF00000000FF
FF8000FF8000FF800000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF000000FF0000FF0000FF0000FFFF800000FF0000FF0000FF00FF8000FF800000FF00
FF8000FF80000000FFFF8000FF8000FF8000FF800000FF00FF800000FF000000FF0000FF
0000FF0000FFFF8000FF8000FF8000
FF8000FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000
FF8000FF8000FF800000FF00FF800000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FFFF800000FF0000FF0000FF000000FF00FF000000FF0000FF0000FFFF000000FF00
00FF0000FF0000FF000000FF00FF00FF800000FF0000FF0000FF00FF8000FF800000FF00
FF8000FF8000FF8000FF800000FF00FF8000FF800000FF0000FF00FF00000000FF0000FF
0000FFFF8000FF8000FF8000FF8000
00FF00FF8000FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00
FF80000000FFFF800000FF00FF800000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FFFF800000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF00
FF800000FF0000FF000000FF0000FF0000FFFF00000000FF00FF00FF8000FF800000FF00
00FF00FF800000FF00FF8000FF8000FF8000FF800000FF0000FF00FF0000FF0000FF0000
FF0000FF8000FF8000FF8000FF8000
00FF0000FF0000FF00FF800000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF00FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000FF8000
FF8000FF8000FF800000FF00FF800000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FFFF800000FF000000FF0000FF0000FF0000FF0000FF0000FFFF8000FF8000FF8000
FF800000FF0000FF000000FF0000FF0000FF00FF000000FF0000FFFF8000FF800000FF00
00FF0000FF0000FF000000FF0000FFFF8000FF800000FF0000FF0000FF00FF0000FF0000
FF0000FF0000FF8000FF8000FF8000
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
FF8000FF8000FF8000FF8000FF800000FF00FF800000FF0000FF0000FF0000FF00FF8000
FF800000FF00FF800000FF00FF800000FF00FF800000FF0000FF000000FF0000FF0000FF
0000FFFF800000FF000000FF0000FF0000FF0000FF0000FF0000FFFF8000FF8000FF8000
00FF00FF800000FF000000FF0000FF0000FF00FF000000FF0000FFFF8000FF800000FF00
00FF0000FF0000FF000000FF0000FFFF80000000FF00FF0000FF000000FF0000FF00FF00
0000FFFF8000FF8000FF8000FF8000
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
FF800000FF00FF800000FF0000FF0000FF00FF800000FF0000FF0000FF0000FF0000FF00
FF800000FF0000FF0000FF00FF80000000FFFF800000FF0000FF0000FF0000FF000000FF
0000FFFF800000FF000000FF0000FF0000FF0000FF0000FF0000FFFF0000FF8000FF8000
00FF00FF800000FF000000FF0000FF0000FF00FF000000FF0000FFFF8000FF800000FF00
00FF0000FF0000FF000000FF0000FFFF8000FF800000FF00FF80000000FF0000FF00FF00
00FF0000FF000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF800000FF00FF800000FF0000FF0000FF00
FF800000FF0000FF0000FF00FF80000000FFFF800000FF00FF800000FF0000FF000000FF
0000FFFF800000FF0000FF0000FF000000FF0000FF0000FFFF0000FF00000000FF00FF00
00FF000000FF0000FFFF8000FF80000000FFFF8000FF800000FF00FF8000FF800000FF00
00FF0000FF0000FF000000FF0000FFFF8000FF8000FF8000FF8000FF800000FF000000FF
00FF00FF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF800000FF00FF800000FF0000FF0000FF00
FF800000FF0000FF0000FF00FF80000000FFFF800000FF00FF800000FF0000FF000000FF
0000FFFF8000FF800000FF0000FF00FF8000FF800000FF00FF0000FF00000000FF00FF00
0000FF0000FF0000FFFF8000FF8000FF8000FF8000FF800000FF00FF8000FF800000FF00
00FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00
00FF00FF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF00FF800000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF800000FF00FF8000FF8000FF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF80000000FFFF800000FF000000FF00FF0000FF000000FF
0000FFFF800000FF000000FF0000FFFF800000FF0000FF000000FFFF00000000FF00FF00
0000FF00FF0000FF00FF8000FF8000FF800000FF00FF800000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF800000FF0000FF00
00FF00FF800000FF00FF800000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF800000FF00FF8000FF8000FF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF80000000FFFF800000FF000000FF0000FF00FF000000FF
0000FFFF800000FF000000FF0000FFFF800000FF0000FF000000FF00FF000000FF00FF00
00FF0000FF00FF00000000FF00FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000FF800000FF00
00FF00FF8000FF8000FF8000FF8000
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF800000FF00FF8000FF8000FF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF00FF800000FF00FF800000FF000000FF00FF0000FF000000FF
0000FFFF800000FF000000FF0000FFFF800000FF0000FF000000FF0000FF0000FF00FF00
00FF0000FF000000FF0000FF00FF00FF800000FF000000FF0000FFFF8000FF8000FF8000
00FF0000FF00FF800000FF00FF8000FF800000FF0000FF00FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF8000FF800000FF0000FF00FF8000FF8000FF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FFFF000000FF00FF00000000FF0000FF0000FFFF0000
FF000000FF00FF00000000FF00FF00FF800000FF000000FF0000FFFF8000FF800000FF00
00FF000000FF0000FFFF8000FF8000FF800000FF0000FF00FF800000FF00FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF00FF8000FF800000FF0000FF00FF8000FF8000FF800000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00FF8000FF8000FF80000000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FFFF000000FF000000FF0000FF0000FF0000FF0000FF
FF000000FF00FF00000000FF0000FF0000FF00FF000000FF0000FFFF8000FF800000FF00
00FF000000FF0000FFFF8000FF800000FF0000FF0000FF0000FF0000FF00FF8000FF8000
FF800000FF00FF800000FF0000FF00
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1.23734 0.738095 1.4776 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .08635 L
.2381 .08635 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .08635 L
.2381 .08635 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .1264 L
.47619 .1264 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .1264 L
.47619 .1264 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .34583 L
.71429 .34583 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .34583 L
.71429 .34583 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .08322 L
.95238 .08322 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .08322 L
.95238 .08322 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
0000FFFF0000FF0000FF800000FF00FF8000FF800000FF00FF000000FF00FF8000FF8000
FF8000FF00000000FF00FF000000FF0000FFFF0000FF000000FF00FF8000FF800000FF00
0000FF0000FF0000FF0000FF00FF00FF8000FF0000FF000000FF00FF00000000FF0000FF
0000FFFF00000000FFFF0000FF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF00FF0000FF0000
0000FF00FF000000FFFF8000FF8000FF800000FF0000FF0000FF000000FFFF00000000FF
0000FF0000FFFF0000FF00000000FF
0000FFFF8000FF0000FF800000FF0000FF00FF8000FF0000FF00000000FF0000FFFF8000
00FF00FF00000000FF00FF000000FF0000FFFF0000FF0000FF0000FF8000FF800000FF00
0000FF0000FF0000FFFF8000FF8000FF8000FF0000FF000000FF00FF00000000FF0000FF
FF00000000FFFF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF00FF00000000FF
0000FF00FF000000FF0000FFFF8000FF800000FF0000FF000000FF0000FF0000FFFF0000
0000FF0000FF00FF000000FF0000FF
FF0000FF0000FF0000FF800000FF0000FF000000FFFF0000FF00000000FF0000FF0000FF
00FF00FF0000FF000000FF000000FF0000FF0000FF0000FF0000FF0000FFFF800000FF00
0000FF0000FFFF8000FF8000FF8000FF8000FF0000FF8000FF80000000FF0000FFFF0000
00FF000000FFFF0000FF0000FF00000000FF00FF000000FFFF00000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF800000FF000000FF0000FF
FF000000FF000000FF0000FF0000FFFF800000FF000000FF0000FF0000FF0000FFFF0000
0000FF0000FFFF80000000FF00FF00
0000FFFF0000FF0000FF8000FF800000FF000000FFFF0000FF0000FF0000FF0000FF8000
00FF00FF000000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FFFF800000FF00
00FF000000FFFF8000FF800000FF0000FF0000FF00FF800000FF000000FF0000FF0000FF
00FF000000FFFF0000FF00000000FF0000FF00FF00FF8000FF00000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF800000FF000000FF0000FF
FF000000FF000000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FFFF8000
00FF00FF000000FF00FF8000FF8000FF8000FF80000000FFFF0000FF00000000FF0000FF
0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
0000FF0000FFFF8000FF800000FF0000FF00FF00000000FF00FF000000FF0000FF00FF00
0000FF0000FFFF0000FF0000FF00000000FF00FF00FF80000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF00
FF000000FF000000FF0000FFFF800000FF0000FF000000FF00FF000000FF0000FF0000FF
00FF000000FF0000FFFF0000FF8000
FF8000FF0000FF0000FF8000FF8000FF8000FF800000FF0000FF00FF00000000FF00FF00
FF800000FF0000FF000000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF00FF00
0000FFFF0000FF8000FF800000FF0000FF00FF0000FF000000FF000000FF0000FF0000FF
FF0000FF0000FF00000000FFFF000000FF000000FF0000FF0000FF0000FF0000FFFF0000
FF00000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF000000FFFF0000
FF00000000FF0000FFFF0000FF8000FF800000FF0000FF0000FF00FF800000FF000000FF
00FF000000FF0000FFFF00000000FF
FF800000FF0000FF00FF8000FF8000FF800000FF0000FF0000FF000000FF0000FF0000FF
FF800000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF000000FFFF8000FF800000FF0000FF0000FF00FF00000000FF0000FFFF00000000FF
0000FF0000FF0000FF0000FFFF800000FF000000FF0000FF0000FF00FF000000FFFF0000
FF00000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FFFF0000
FF0000FF00000000FFFF0000FF8000FF8000FF800000FF0000FF0000FF00FF800000FF00
00FF00FF00000000FFFF00000000FF
FF8000FF800000FF00FF8000FF800000FF00FF800000FF0000FF0000FF0000FF0000FF00
FF800000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF800000FF00
00FF000000FF0000FFFF800000FF0000FF00FF0000FF00000000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF00000000FF00FF00FF80000000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FF
FF0000FF00000000FF0000FFFF8000FF8000FF800000FF0000FF000000FF0000FF00FF00
00FF000000FF0000FF0000FF0000FF
FF80000000FF00FF0000FF00FF800000FF0000FF0000FF00FF800000FF0000FF0000FF00
FF800000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF000000FF0000FFFF800000FF0000FF0000FF00FF0000FF00000000FF0000FF0000FF
0000FF0000FF0000FFFF00000000FF0000FFFF8000FF8000FF800000FF000000FFFF0000
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FF
0000FFFF00000000FF00FF00FF8000FF800000FF000000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
FF00000000FF00FF000000FFFF800000FF0000FF000000FFFF8000FF8000FF8000FF8000
FF800000FF0000FF0000FF000000FFFF00000000FF0000FF0000FF0000FF0000FF00FF00
0000FF0000FFFF8000FF800000FF0000FF000000FFFF800000FF0000FF0000FF000000FF
0000FF0000FF0000FFFF00000000FFFF0000FF000000FF00FF800000FF000000FF0000FF
0000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF00FF8000FF8000FF8000FF000000FF000000FF0000FFFF0000
00FF000000FF0000FF0000FF0000FF
0000FFFF000000FF000000FFFF800000FF0000FF00FF0000FF000000FF0000FF00FF8000
00FF0000FF0000FF000000FF0000FF0000FF0000FFFF00000000FF0000FF0000FF0000FF
0000FF0000FFFF8000FF800000FF0000FF000000FF0000FF0000FF00FF0000FF000000FF
0000FFFF00000000FFFF00000000FFFF00000000FF00FF000000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF
FF00000000FF00FF0000FF0000FF00FF8000FF8000FF800000FF000000FFFF0000FF0000
00FF000000FF0000FF0000FF0000FF
0000FF0000FF00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF00FF8000
FF800000FF0000FF00FF00000000FFFF0000FF0000FF00000000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF800000FF0000FF000000FFFF00000000FF00FF0000FF000000FF
0000FFFF0000FF0000FF0000FF00000000FF0000FF0000FF0000FF0000FF0000FFFF0000
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00FF0000FF0000
FF0000FF800000FF000000FF00FF00FF8000FF8000FF800000FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF
FF8000FF800000FF000000FF0000FFFF8000FF800000FF0000FF0000FF0000FF00FF8000
FF8000FF8000FF0000FF00000000FF0000FF0000FF0000FF00FF00FF00000000FFFF0000
0000FF0000FF0000FF0000FF00FF0000FF000000FFFF0000FF00000000FFFF00000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF00000000FF00FF000000FF00FF000000FF0000FF0000FF0000FF0000FFFF0000FF8000
FF00000000FF00FF000000FF0000FFFF8000FF8000FF8000FF800000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF
FF0000FF8000FF800000FF0000FF00FF8000FF8000FF8000FF8000FF800000FF00FF8000
00FF000000FFFF000000FF000000FF0000FF0000FFFF000000FF000000FF0000FFFF0000
FF0000FF00000000FF0000FF00FF000000FF0000FF0000FF0000FF0000FFFF00000000FF
0000FF0000FF0000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FFFF00000000FFFF0000FF0000
FF0000FF800000FF0000FF000000FFFF8000FF800000FF00FF800000FF000000FF0000FF
0000FF0000FFFF0000FF00000000FF
00FF00FF8000FF800000FF00FF800000FF0000FF000000FF0000FFFF8000FF8000FF8000
00FF000000FF00FF0000FF0000FF000000FF0000FFFF000000FF000000FF0000FF0000FF
0000FFFF00000000FFFF0000FF00000000FF0000FF0000FF0000FF0000FFFF00000000FF
0000FF0000FF0000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF0000FF00000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000FF0000
FF0000FF800000FF0000FF00FF8000FF8000FF800000FF00FF800000FF000000FF0000FF
0000FF0000FFFF00000000FF0000FF
0000FF0000FFFF000000FF00FF800000FF0000FF000000FF0000FFFF000000FF00FF8000
00FF000000FFFF0000FF000000FF000000FF0000FFFF80000000FF0000FF00FF000000FF
0000FF0000FF00FF00FF0000FF80000000FF0000FFFF8000FF80000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FFFF0000FF00000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF0000FF800000FF000000FF0000FFFF800000FF0000FF0000FF000000FF0000FFFF0000
0000FF0000FF00FF000000FFFF0000
0000FF0000FFFF000000FF000000FF00FF0000FF00FF00000000FFFF000000FF00FF8000
0000FF0000FFFF0000FF00000000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FFFF800000FF0000FF0000FF00FF80000000FF0000FFFF0000
FF00000000FF0000FF0000FF0000FFFF0000FF00000000FFFF0000FF00000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF8000FF800000FF000000FF0000FFFF800000FF0000FF0000FF000000FF0000FFFF0000
FF0000FF000000FF000000FF0000FF
FF8000FF800000FF0000FF000000FFFF800000FF000000FF0000FFFF000000FF00FF8000
0000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF00000000FF0000FF0000FFFF800000FF0000FF00FF800000FF000000FFFF0000FF0000
FF0000FF0000FF000000FF0000FF000000FF0000FFFF0000FF8000FF800000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000
FF000000FF0000FF000000FF0000FFFF800000FF0000FF000000FF0000FFFF00000000FF
FF0000FF000000FF000000FF0000FF
0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FFFF8000FF8000
0000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF80000000FF0000FFFF00000000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF000000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000
FF00000000FF0000FF0000FF0000FF0000FF00FF0000FF000000FFFF00000000FFFF0000
0000FF0000FF00FF000000FF0000FF
0000FFFF8000FF8000FF800000FF00FF800000FF000000FF0000FF0000FFFF8000FF8000
00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FFFF0000FF00000000FF0000FF0000FF0000FF
00FF0000FF00FF00000000FF00FF000000FF0000FF0000FF0000FFFF8000FF80000000FF
00FF0000FF000000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
FF8000FF8000FF800000FF00FF8000FF800000FF000000FF00FF00FF800000FF0000FF00
00FF00FF800000FF0000FF000000FFFF0000FF00000000FF0000FF0000FFFF00000000FF
0000FF00FF000000FF0000FFFF8000FF800000FF00FF800000FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF00FF0000FF0000FF00000000FF0000FF0000FF00FF00
FF800000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF00FF8000
00FF0000FF000000FF0000FF0000FF0000FFFF800000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FFFF8000FF800000FF00FF8000FF800000FF000000FF00FF00FF800000FF0000FF00
FF8000FF8000FF800000FF000000FFFF0000FF00000000FF00FF00FF0000FF0000FF8000
FF800000FF000000FF0000FFFF0000FF800000FF0000FF0000FF000000FF0000FF0000FF
0000FFFF0000FF0000FF00000000FFFF00000000FF0000FF0000FF0000FFFF00000000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF00FF0000
00FF0000FF0000FF000000FF0000FF0000FFFF800000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FFFF8000FF800000FF0000FF0000FF00FF800000FF0000FF00FF800000FF0000FF00
0000FF0000FFFF800000FF000000FF00FF000000FF0000FF00FF00FF0000FF0000FF8000
FF800000FF00FF80000000FFFF0000FF000000FF0000FF0000FF000000FF0000FFFF0000
FF0000FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF800000FF0000FF0000FF000000FF0000FFFF8000FF8000FF80000000FF0000FF0000FF
0000FFFF00000000FF0000FF0000FF
0000FFFF8000FF800000FF0000FF0000FF00FF800000FF00FF8000FF000000FF0000FF00
0000FFFF800000FF0000FF000000FF0000FF0000FF0000FF00FF00FF00000000FFFF8000
00FF0000FF0000FF000000FF0000FF00FF0000FF00FF800000FF000000FF0000FF0000FF
0000FF00FF000000FFFF0000FF0000FF00000000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF800000FF00FF000000FF000000FF0000FFFF000000FF00FF800000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FFFF0000FF800000FF000000FF0000FFFF80000000FFFF0000FF000000FF0000FF00
FF8000FF800000FF0000FF000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF
00FF00FF800000FF0000FF00FF000000FF000000FF0000FF00FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF80000000FFFF00000000FF0000FFFF00000000FF00FF000000FFFF00000000FF0000FF
0000FF0000FF00FF000000FF0000FF
FF00000000FFFF800000FF000000FF0000FFFF8000FF8000FF0000FF000000FF0000FF00
FF8000FF800000FF0000FF0000FF0000FF00FF00000000FF0000FF00FF000000FF0000FF
00FF00FF800000FF0000FF00FF000000FF000000FF0000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FFFF000000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF0000FF00000000FF0000FF0000FF00FF000000FF0000FFFF00000000FFFF8000
00FF000000FFFF00000000FFFF0000FF00000000FF00FF000000FF0000FF0000FF0000FF
0000FF0000FF00FF000000FF0000FF
0000FFFF0000FF800000FF000000FF0000FFFF800000FF0000FF00FF800000FF0000FF00
FF8000FF800000FF0000FF0000FF0000FF000000FFFF00000000FF00FF000000FF0000FF
0000FF0000FFFF0000FF0000FF800000FF000000FF0000FFFF800000FF0000FF0000FF00
0000FF0000FF0000FFFF800000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF000000FF0000FF00FF0000FF000000FF0000FFFF0000FF8000FF8000
FF8000FF800000FF000000FFFF00000000FF0000FF00FF00FF00000000FFFF00000000FF
FF00000000FF0000FF0000FF0000FF
0000FFFF800000FF0000FF000000FF0000FFFF800000FF0000FF0000FF0000FF0000FF00
00FF00FF800000FF0000FF000000FF00FF000000FF00FF000000FF0000FF0000FF0000FF
0000FF0000FFFF00000000FF0000FFFF00000000FF0000FFFF800000FF0000FF000000FF
0000FFFF00000000FFFF800000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF00
FF800000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FF
FF00000000FF0000FF0000FF0000FF
0000FFFF8000FF800000FF0000FF0000FF00FF800000FF00FF8000FF800000FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF000000FF00FF000000FFFF00000000FF0000FF
FF000000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FFFF8000
00FF0000FF0000FF000000FFFF00000000FF0000FF0000FF0000FF0000FFFF00000000FF
FF00000000FF0000FF0000FF0000FF
FF8000FF8000FF800000FF0000FF0000FF00FF800000FF0000FF00FF80000000FF00FF00
00FF00FF00000000FFFF00000000FF0000FFFF000000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF800000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FFFF0000FF0000FF00000000FF0000FF0000FFFF8000
00FF0000FF0000FF000000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF
FF0000FF00000000FF0000FF0000FF
FF8000FF0000FF0000FF000000FF0000FF00FF800000FF0000FF000000FF0000FF0000FF
00FF00FF0000FF0000FF0000FF00000000FFFF0000FF00000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF800000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF
0000FF0000FF00FF000000FFFF0000FF00000000FF0000FF0000FF0000FF0000FFFF8000
FF800000FF0000FF000000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF00000000FF0000FF0000FF
0000FF0000FFFF00000000FF00FF0000FF00FF8000FF800000FF000000FF0000FF0000FF
0000FFFF0000FF0000FF0000FF0000FF0000FF00000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FFFF8000FF800000FF000000FF
0000FF0000FF0000FF0000FF0000FFFF00000000FF0000FFFF000000FF000000FF0000FF
0000FFFF000000FF000000FF0000FF0000FF0000FF0000FFFF00000000FF0000FFFF8000
FF800000FF0000FF000000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF80000000FFFF8000FF8000FF8000FF800000FF000000FF0000FF0000FF
0000FF0000FFFF0000FF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF8000FF8000FF800000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FFFF00000000FF0000FF0000FF
0000FFFF000000FF000000FF0000FF0000FF0000FF0000FF0000FFFF00000000FFFF8000
FF800000FF0000FF0000FF00FF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF80000000FFFF80000000FFFF8000FF800000FF000000FF0000FF0000FF
0000FF0000FFFF0000FF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF00FF8000FF800000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FFFF00000000FF0000FF0000FF
0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FFFF8000
FF8000FF800000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF80000000FF0000FF0000FFFF8000FF80000000FF0000FF0000FF0000FF
0000FF00FF00FF800000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF0000FF0000FF800000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000FF00000000FFFF8000
FF8000FF800000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF80000000FF0000FF0000FFFF80000000FF00FF000000FF0000FF0000FF
0000FFFF8000FF800000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000FF000000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FFFF8000
FF8000FF800000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FFFF80000000FF0000FF0000FFFF800000FF0000FF000000FF0000FF0000FF
0000FFFF8000FF800000FF00FF000000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FFFF000000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FFFF8000
FF8000FF800000FF0000FF00FF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.738095 1.23734 0.97619 1.4776 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .02941 L
.2381 .02941 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .02941 L
.2381 .02941 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .16687 L
.47619 .16687 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .16687 L
.47619 .16687 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .02586 L
.71429 .02586 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .02586 L
.71429 .02586 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .41967 L
.95238 .41967 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .41967 L
.95238 .41967 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF800000FF00FF800000FF0000FF0000FF00FF8000FF800000FF00FF8000FF8000FF8000
FF8000FF800000FF00FF8000FF8000FF8000FF800000FF0000FF00FF8000FF800000FF00
FF8000FF8000FF800000FF0000FF0000FF0000FF0000FF00FF800000FF00FF800000FF00
FF800000FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
00FF00FF8000FF0000FF000000FF0000FF00FF8000FF800000FF000000FF0000FFFF0000
FF8000FF800000FF00FF8000FF800000FF00FF800000FF00FF0000FF0000FF800000FF00
0000FFFF8000FF800000FF0000FF00
FF800000FF00FF800000FF0000FF0000FF00FF8000FF800000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF800000FF00
FF8000FF8000FF8000FF800000FF0000FF0000FF0000FF00FF8000FF8000FF800000FF00
FF800000FF00FF80000000FFFF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
00FF00FF8000FF0000FF0000FF8000FF8000FF8000FF800000FF000000FFFF0000FF0000
0000FFFF8000FF8000FF8000FF8000FF8000FF800000FF00FF0000FF8000FF800000FF00
FF8000FF8000FF8000FF000000FF00
FF800000FF00FF800000FF0000FF0000FF00FF800000FF0000FF0000FF00FF8000FF8000
FF800000FF00FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000FF8000
FF8000FF800000FF00FF800000FF0000FF0000FF0000FF00FF8000FF8000FF8000FF8000
FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
00FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF0000FFFF0000
0000FFFF800000FF00FF8000FF8000FF8000FF8000FF8000FF0000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF800000FF00FF800000FF0000FF00FF8000FF8000FF800000FF00FF800000FF0000FF00
FF800000FF00FF8000FF8000FF800000FF00FF800000FF00FF8000FF8000FF8000FF8000
FF800000FF0000FF00FF8000FF8000FF800000FF00FF8000FF800000FF0000FF00FF8000
FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000FF8000FF8000FF800000FF00
00FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF0000
FF8000FF800000FF00FF8000FF8000FF8000FF80000000FFFF0000FF0000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000
FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00
FF8000FF8000FF8000FF8000FF800000FF0000FF0000FF0000FF0000FF00FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF800000FF00FF800000FF00
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF800000FF00FF800000FF00FF8000FF0000FF0000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF800000FF00FF800000FF00FF8000FF800000FF0000FF00FF800000FF0000FF00
00FF0000FF00FF8000FF8000FF8000FF800000FF0000FF0000FF0000FF00FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
00FF00FF8000FF8000FF8000FF800000FF0000FF00FF8000FF800000FF00FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF800000FF00FF800000FF00FF0000FF00000000FFFF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
00FF00FF800000FF0000FF0000FF00FF8000FF800000FF0000FF00FF8000FF8000FF8000
FF8000FF8000FF800000FF00FF8000FF8000FF800000FF0000FF0000FF00FF800000FF00
FF800000FF00FF8000FF8000FF8000FF800000FF00FF800000FF00FF800000FF00FF8000
00FF00FF8000FF800000FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000FF8000
00FF00FF8000FF800000FF00FF8000
00FF00FF800000FF00FF0000FF8000FF800000FF0000FF0000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF800000FF00
FF8000FF800000FF0000FF00FF8000FF8000FF800000FF0000FF00FF8000FF8000FF8000
FF8000FF8000FF800000FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF800000FF00FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF800000FF0000FF00FF8000FF800000FF0000FF0000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000
FF8000FF8000FF800000FF0000FF0000FF0000FF00FF8000FF800000FF00FF8000FF8000
FF800000FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
00FF00FF800000FF0000FF00FF8000FF800000FF0000FF0000FF00FF8000FF8000FF8000
FF8000FF800000FF00FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF800000FF0000FF0000FF0000FF00FF800000FF0000FF00FF8000
FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000
FF800000FF00FF800000FF00FF8000
00FF00FF8000FF800000FF00FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000
FF8000FF800000FF00FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF800000FF0000FF0000FF00FF8000FF800000FF00FF8000
FF8000FF8000FF8000FF800000FF00FF800000FF00FF800000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF800000FF00FF800000FF00FF8000FF8000FF8000FF8000
0000FFFF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF800000FF0000FF00
FF800000FF00FF000000FF00FF8000
00FF00FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF800000FF00FF8000
FF800000FF00FF8000FF800000FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF800000FF00FF800000FF00FF800000FF00FF8000FF800000FF00FF8000
0000FF0000FFFF8000FF800000FF00FF800000FF00FF800000FF00FF800000FF0000FF00
FF800000FF00FF0000FF8000FF8000
00FF00FF8000FF800000FF00FF0000FF8000FF8000FF800000FF0000FF00FF8000FF8000
FF8000FF800000FF00FF800000FF00FF800000FF00FF8000FF800000FF00FF8000FF8000
00FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
FF800000FF00FF8000FF800000FF00FF800000FF00FF8000FF8000FF8000FF800000FF00
0000FFFF8000FF800000FF00FF000000FF00FF000000FF00FF8000FF8000FF8000FF8000
0000FFFF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
FF800000FF00FF8000FF8000FF8000
00FF0000FF00FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000
FF8000FF800000FF00FF800000FF00FF800000FF0000FF0000FF0000FF00FF800000FF00
00FF00FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF800000FF00
FF800000FF00FF8000FF800000FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF800000FF0000FF00FF80000000FFFF0000FF8000FF8000FF8000FF8000
0000FFFF0000FF800000FF0000FF0000FF0000FF00FF8000FF8000FF8000FF800000FF00
00FF0000FF00FF8000FF8000FF8000
00FF00FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF0000FF00FF8000FF8000
00FF00FF800000FF00FF8000FF8000FF8000FF8000FF800000FF0000FF00FF800000FF00
00FF00FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000FF8000
FF800000FF00FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000FF800000FF00
FF000000FF00FF8000FF8000FF0000FF8000FF800000FF00FF8000FF800000FF0000FF00
0000FF00FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
00FF0000FF00FF8000FF8000FF8000
00FF00FF8000FF8000FF8000FF8000FF8000FF800000FF00FF800000FF0000FF00FF8000
00FF00FF800000FF00FF8000FF8000FF8000FF8000FF800000FF0000FF0000FF0000FF00
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF800000FF00
FF800000FF00FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF800000FF00
00FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF800000FF00FF800000FF00FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF800000FF00FF8000
00FF00FF800000FF00FF8000FF8000FF800000FF00FF8000FF8000FF800000FF00FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF800000FF00
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
FF800000FF00FF800000FF0000FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000
00FF0000FF00FF800000FF0000FF0000FF00FF80000000FF0000FF0000FFFF8000FF8000
FF800000FF00FF8000FF800000FF00
00FF00FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF800000FF00FF8000
00FF00FF800000FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF800000FF00FF8000FF8000FF8000FF8000FF800000FF00FF8000FF800000FF00
FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF800000FF00FF800000FF0000FF00FF8000FF8000FF800000FF0000FF0000FF00FF8000
00FF0000FF00FF8000FF800000FF00FF8000FF80000000FF0000FFFF8000FF8000FF8000
FF800000FF0000FF00FF8000FF8000
00FF00FF8000FF8000FF8000FF800000FF00FF800000FF00FF8000FF800000FF00FF8000
00FF0000FF00FF8000FF800000FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000
00FF0000FF0000FF00FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000
FF800000FF0000FF00FF800000FF00FF8000FF8000FF8000FF800000FF00FF8000FF8000
FF8000FF8000FF800000FF0000FF00FF800000FF00FF8000FF800000FF0000FF00FF8000
00FF00FF8000FF800000FF0000FF00FF8000FF80000000FF0000FF0000FFFF800000FF00
FF800000FF0000FF00FF8000FF8000
00FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000
00FF0000FF0000FF00FF800000FF00FF8000FF8000FF8000FF8000FF800000FF00FF8000
00FF00FF800000FF00FF8000FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000
FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000
FF8000FF8000FF800000FF00FF8000FF800000FF00FF8000FF8000FF800000FF0000FF00
FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF800000FF00FF800000FF00
FF8000FF8000FF8000FF0000FF8000
00FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
00FF0000FF00FF800000FF0000FF00FF8000FF800000FF00FF800000FF0000FF00FF8000
FF8000FF800000FF00FF8000FF8000FF800000FF00FF800000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF800000FF00FF8000FF8000
FF8000FF8000FF800000FF0000FF0000FF0000FF00FF800000FF00FF800000FF0000FF00
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF800000FF00
FF8000FF80000000FFFF0000FF8000
FF8000FF800000FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000
00FF00FF8000FF8000FF800000FF00FF800000FF00FF800000FF0000FF0000FF00FF8000
FF8000FF8000FF800000FF00FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000
FF8000FF8000FF8000FF8000FF800000FF0000FF00FF800000FF00FF8000FF8000FF8000
FF8000FF0000FF800000FF00FF0000FF8000FF8000FF8000FF800000FF00FF800000FF00
FF8000FF8000FF8000FF0000FF8000
FF8000FF800000FF00FF800000FF00FF8000FF8000FF800000FF00FF8000FF800000FF00
00FF00FF8000FF8000FF800000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF00
FF800000FF0000FF0000FF00FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF800000FF0000FF00FF800000FF00FF8000FF0000FF8000
0000FFFF0000FF8000FF8000FF0000FF0000FF8000FF800000FF0000FF00FF800000FF00
FF800000FF000000FFFF0000FF8000
FF8000FF8000FF8000FF8000FF800000FF00FF8000FF800000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF800000FF00FF800000FF0000FF00FF8000FF800000FF0000FF00
00FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF0000FF0000FFFF800000FF00
FF8000FF8000FF8000FF8000FF8000FF80000000FFFF800000FF00FF8000FF0000FF0000
FF0000FF8000FF8000FF8000FF0000FF0000FF8000FF800000FF00FF8000FF800000FF00
FF800000FF000000FFFF0000FF8000
FF8000FF800000FF00FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
00FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF0000FFFF800000FF00
FF8000FF800000FF0000FF00FF0000FF80000000FFFF800000FF00FF0000FF0000FF0000
FF8000FF8000FF8000FF800000FF00FF0000FF8000FF800000FF00FF8000FF8000FF8000
FF800000FF0000FF00FF0000FF8000
FF800000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000
FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000FF800000FF00FF800000FF00
00FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF800000FF00FF8000FF800000FF00FF8000FF8000FF80000000FF0000FF0000FF00FF00
FF8000FF800000FF0000FF00FF8000FF8000FF8000FF800000FF00FF0000FF0000FF0000
FF8000FF8000FF8000FF8000FF8000FF000000FF00FF8000FF8000FF8000FF8000FF8000
00FF0000FF00FF0000FF0000FF0000
FF800000FF0000FF0000FF00FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000
FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF800000FF00FF8000FF8000
00FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
00FF0000FF00FF8000FF800000FF00FF8000FF800000FF000000FF00FF000000FF00FF00
FF8000FF8000FF8000FF8000FF800000FF00FF8000FF0000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF800000FF00FF000000FF00FF8000FF8000FF8000FF8000FF8000
00FF0000FF0000FF00FF0000FF8000
FF800000FF0000FF0000FF00FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000
FF8000FF8000FF8000FF8000FF800000FF00FF800000FF00FF8000FF8000FF8000FF8000
00FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
0000FF0000FFFF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF000000FF00
FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000
00FF00FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000FF8000FF8000
00FF00FF800000FF00FF800000FF00
00FF0000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF800000FF00FF8000FF8000FF800000FF00FF800000FF0000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF800000FF00FF800000FF0000FF0000FF00FF8000FF8000FF8000
0000FF0000FFFF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF000000FF00
FF8000FF8000FF8000FF8000FF800000FF00FF80000000FFFF80000000FFFF8000FF8000
00FF0000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
00FF0000FF0000FF0000FF0000FF00
FF8000FF8000FF8000FF8000FF800000FF00FF800000FF00FF8000FF8000FF8000FF8000
FF800000FF00FF8000FF800000FF0000FF00FF800000FF0000FF00FF8000FF8000FF8000
FF8000FF800000FF00FF800000FF0000FF0000FF0000FF0000FF0000FF00FF8000FF8000
FF00000000FFFF8000FF8000FF8000FF8000FF800000FF00FF8000FF800000FF0000FF00
FF8000FF800000FF000000FF00FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000
00FF0000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF800000FF0000FF00
FF800000FF00FF8000FF8000FF800000FF00FF800000FF000000FF0000FFFF8000FF8000
FF8000FF8000FF8000FF800000FF0000FF00FF8000FF800000FF00FF8000FF80000000FF
FF8000FF800000FF00FF800000FF00FF8000FF8000FF8000FF800000FF00FF0000FF0000
FF8000FF8000FF8000FF8000FF800000FF00FF000000FF00FF8000FF8000FF800000FF00
FF8000FF800000FF000000FFFF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000
FF8000FF800000FF00FF8000FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000
00FF00FF800000FF0000FF0000FF00
FF800000FF00FF8000FF8000FF8000FF8000FF800000FF000000FF0000FFFF8000FF8000
FF800000FF00FF8000FF800000FF0000FF00FF8000FF800000FF00FF8000FF8000FF8000
00FF00FF800000FF00FF800000FF0000FF00FF8000FF8000FF80000000FFFF0000FF8000
FF8000FF8000FF8000FF8000FF800000FF00FF8000FF0000FF800000FF00FF8000FF8000
FF8000FF800000FF000000FFFF8000FF800000FF00FF8000FF800000FF00FF0000FF8000
FF8000FF800000FF00FF8000FF8000FF800000FF00FF8000FF800000FF00FF8000FF8000
00FF00FF800000FF00FF8000FF8000
00FF00FF8000FF8000FF8000FF800000FF00FF800000FF000000FF0000FFFF8000FF8000
FF800000FF0000FF00FF8000FF8000FF8000FF8000FF800000FF00FF800000FF00FF8000
FF8000FF800000FF00FF8000FF800000FF0000FF00FF800000FF000000FFFF0000FF8000
FF8000FF8000FF0000FF800000FF00FF800000FF0000FF0000FF00FF8000FF800000FF00
FF8000FF8000FF00000000FFFF8000FF800000FF00FF8000FF800000FF00FF0000FF8000
FF800000FF0000FF00FF8000FF8000FF800000FF00FF8000FF800000FF0000FF0000FF00
00FF00FF800000FF00FF8000FF8000
00FF00FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF800000FF0000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF800000FF00FF0000FF8000
FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF0000FF0000FF800000FF00
FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000
FF800000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00FF8000
00FF00FF800000FF0000FF00FF8000
00FF00FF8000FF800000FF00FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000
FF800000FF0000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF800000FF00
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF800000FF0000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
00FF00FF800000FF00FF8000FF8000
00FF0000FF00FF800000FF00FF800000FF00FF8000FF8000FF8000FF8000FF800000FF00
FF800000FF0000FF0000FF00FF8000FF8000FF800000FF00FF8000FF8000FF8000FF8000
FF800000FF0000FF00FF8000FF800000FF0000FF00FF800000FF00FF8000FF800000FF00
FF8000FF800000FF00FF800000FF0000FF0000FF00FF8000FF8000FF8000FF800000FF00
FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000
FF800000FF00FF8000FF8000FF8000FF800000FF0000FF00FF8000FF8000FF8000FF8000
FF8000FF800000FF00FF8000FF8000
00FF0000FF00FF800000FF00FF800000FF00FF8000FF8000FF8000FF8000FF800000FF00
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF800000FF0000FF00FF8000FF8000FF800000FF00FF800000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF800000FF0000FF0000FF00FF8000FF8000FF8000FF800000FF00
00FF00FF800000FF000000FF0000FFFF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000FF8000FF80000000FFFF8000
00FF00FF800000FF00FF8000FF8000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{640, 968.688},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000:00003b2000`400?l00000o`00003ooooooooo
oooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo
003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003ooooooooo
ooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool0
0?oooooooooooon2ooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooo
oon2ooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00
oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooo
oh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003o
ooooooooooooP_ooool00?oooooooooooon2ooooo`008_oooombo`0002OoooooLOl0000Wooooog7o
00009ooooombo`0002?ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo0005oooooaKo0?l0
1?oooolGo`00o`Cooooo5oooO`04ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
0_oooolFool000Gooooo5_l0o`05oooooaKo003o1?oooolGoomo00Cooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`02oooooaOoo`001?oooolFo`3o00Gooooo5_l00?l5oooooaKoogl0
1?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo0004oooooaOo0?l0
1?oooolFo`00o`Gooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oo
oooooooo00?ooooo5_oo000OoooooaOo003o1?oooolGoomo00Cooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00o`Cooooo5oooO`04ooooo`03o`000?oo
oooooooo02Gooooo00?o0000ooooooooool00_oooolGool000Cooooo5_l0o`05oooooaKo003o1Ooo
oolFoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02oooooaOoo`007ooo
oolFo`00o`Gooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooo
oooo00?ooooo5_oo000OoooooaOo003o1?oooolGoomo00Cooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00o`Gooooo5_ooO`04ooooo`03o`000?oooooo
oooo02Gooooo00?o0000ooooooooool00_oooolGool001oooooo5_l00?l5oooooaKoogl01Oooool0
0ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00>_oooolGoomo
00Cooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00
oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00
o`Gooooo5_ooO`04ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolGool0
01oooooo5_l00?l5oooooaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003o
ooooooooo`03oooooaKoo`00>_oooolGoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`0MoooooaKo0?l01OoooolFo`00o`Gooooo5_ooO`04ooooo`03o`000?oooooooooo02Go
oooo00?o0000ooooooooool00_oooolGool001oooooo5_l00?l5oooooaKoogl01Oooool00ol0003o
ooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00>_oooolGoomo00Cooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00o`Gooooo
5_ooO`04ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolGool001oooooo
5_l00?l5oooooaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003ooooooooo
o`03oooooaKoo`00>_oooolGoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`0MoooooaKo0?l01OoooolFo`00o`Gooooo5_ooO`04ooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool00_oooolGool001oooooo5_l00?l5oooooaKoogl01Oooool00ol0003ooooooooo
o`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00>_oooolGoomo00Cooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`02oooooaOoo`00>_oooolFoomo00Gooooo00?o0000
ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool003[ooooo5oooO`04oooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lOoooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lOoooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolGool003[ooooo5_ooO`05oooo
o`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo000joooooaOo
ogl01?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo
003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolGool003[ooooo5_ooO`05ooooo`03
o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo000joooooaOoogl0
1?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o
7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool00_oooolGool003[ooooo5_ooO`05ooooo`03o`00
0?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oo
oooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`000?oo
oooooooo02Gooooo00?o0000ooooooooool0>?oooolFo`00oaoooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`02oooooaOoo`00>_oooolFoomo00Gooooo00?o0000ooooooooool08Ooo
ool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo00;ooooo5ooo000joooooaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0
003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
03Sooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolGool0
03[ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?l007oo001o00?o
001o5_oo001Eo`00O`03o`000?oooooooooo02Gooooo00?o0000o`00Ool007l07Ol007lFo`3o00Go
001o5_l00?lOo`00O`03o`000?oooooooooo02Gooooo00?o0000o`00Ool007l0>?l007lFo`00oaoo
001o00?o0000ooooooooool09Oooool00ol0003o001oo`00O`02o`00OaOoo`00>_l007lFoomo00Go
001o00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo000joooooaKoogl01Oooool00ol0003ooooo
ooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool00_oooolGool003[ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000R
ooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool0>?oooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02
oooooaOoo`00>_oooolFoomo00Gooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000oooo
ooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0Moooo
oaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0hoooo
oaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo000joooo
oaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKo
o`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo
003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolGool003[ooooo5_ooO`05ooooo`03
o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0>?oooolFo`00oaoooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`02oooooaOoo`00>_oooolFoomo00Gooooo00?o0000ooooooooool0
8Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo00;ooooo5ooo000joooooaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool0
0ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo03Sooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolG
ool003[ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo
00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o
00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0>?oooolFo`00
oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02oooooaOoo`00>_oooolFoomo
00Gooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Go
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoo
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003o
ooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo000joooooaKoogl01Oooool00ol0003o
ooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool00_oooolGool003[ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo
000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool0>?oooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`1CoooooaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003o001oo`00O`03
o`00OaKoo`00EOl007l00ol0003oooooooooo`0Uooooo`03o`000?l007oo001o01go001o5_l0o`05
o`00OaKo003o7ol007l00ol0003oooooooooo`0Uooooo`03o`000?l007oo001o03So001o5_l00?lO
o`00O`03o`000?oooooooooo02Gooooo00?o0000o`00Ool007l0Dol007lFoomo00Go001o00?o0000
ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03
o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000oooooooo
ool07OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0hoooooaKo
003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03
o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o03[ooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oo
oooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07Ooo
oolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0hoooooaKo003o7ooo
ool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oo
oooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooo
oooo02Gooooo00?o0000ooooooooool07OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo
00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o
03[ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo
027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Go
oooo00?o0000ooooooooool07OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?o
oooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo
5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o03[ooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo
000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool07OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_oo
O`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001E
ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o03[ooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000oooo
ooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0Moooo
oaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOoooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qoooo
o`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0hoooooaKo
003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06kooooo00?o0000oooooooo
ool08Oooool002;ooooo00?o0000o`00Ool007l00ol007lFool005Go001o00?o0000ooooooooool0
9Oooool00ol0003o001oo`00O`1]o`00O`03o`000?oooooooooo02Gooooo00?o0000o`00Ool007l0
>?l007lFo`00oaoo001o00?o0000ooooooooool09Oooool00ol0003o001oo`00O`1^o`00O`03o`00
0?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oo
oooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo03Sooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_oo
ool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo06kooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool0
05Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0>?oooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`1^ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?o
oooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003o
ooooooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOooooo`03o`000?oooooooooo02Go
oooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003ooooo
ooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool08Oooool002;ooooo00?o
0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0>?oooolFo`00oaoooooo00?o
0000ooooooooool09Oooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo027ooooo000R
ooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lO
ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Q
ooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0hoooo
oaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06kooooo00?o0000oooo
ooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000oooooooo
ool0>?oooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1^ooooo`03
o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo03Sooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOoo
ool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo06kooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolF
ool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooo
oooo02Gooooo00?o0000ooooooooool0>?oooolFo`00oaoooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`1^ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo
00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003o
001oo`00O`03o`00OaKoo`00EOl007l00ol0003oooooooooo`0Uooooo`03o`000?l007oo001o06go
001o00?o0000ooooooooool09Oooool00ol0003o001oo`00O`0ho`00OaKo003o7ol007l00ol0003o
ooooooooo`0Uooooo`03o`000?l007oo001o06ko001o00?o0000ooooooooool08Oooool002;ooooo
00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool08Oooool002;ooooo00?o
0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000
ooooooooool0K_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`1^oooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1^oooo
o`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo06kooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000oooooooo
ool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06kooooo00?o0000oooooooo
ool08Oooool002;ooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_oo
ool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOoo
ool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooo
oooo06kooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooo
oooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo06kooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool0K_ooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0
003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo02Go
oooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06go
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo027o
oooo000Rooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool08Oooool002;ooooo00?o
0000ooooooooool0K_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o
0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`1^
ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1^
ooooo`03o`000?oooooooooo027ooooo000Rooooog;o00009ooooomao`0002OoooooLOl0000Woooo
og;o00008oooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003oooooooooooooP_oo
ool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooo
oooooon2ooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool001Kooooo5P3o00000`3o
P000o`000?l000050?l00080oh005P3o00020?n00100o`000`3oP00V0?l00080oh002@3o00000`3o
P000o`000?l000090?l00080oh003_oooolN003o00T0003o0P3oP00;003o00L0003o1P00o`000`00
0?l00?l0003o00040000o`<00?l01`000?l4003o0080003o0P00o`0@0000o`<00?l05P000?l3003o
00@0003o3_ooool20000o`800?l000<0003o003o0000o`000P00o`070000o`80oh0010000?l00`3o
P000003o0000o`0K0000o`80oh002`000?l00`3oP000003o0000o`0F0000o`030?n000000?l0003o
0080003o0P3oP0070000o`80oh0080000?l>ooooo`D0o`000P3oP0060?l000030?n0003o0000o`00
00@0o`000P3oP0000`000?l0oh000?n000020?l00080oh0000<0o`000?n0003oP000103oP0030000
o`80o`000P000?l20?n00080o`0000<0oh000?l0003o00000P3oP0020000o`D0o`000P3oP0070?l0
0080oh000P3o00090?n000030?l000000?l0003o00H0oh0000<0o`000?n0003oP0000P000?l20?n0
00L0o`001@3oP0020?l00080oh000`3o00060?n00080o`0000<0oh000?l0003o00000P3oP0020?l0
0080oh0000<0o`000?n0003oP0000P3o000Hooooo`005_oooolF0?l000030?n0003o0000o`0000D0
o`000P3oP00F0?l00080oh00403o00030?n002H0o`000P3oP0090?l000030?n0003o0000o`0000T0
o`000P3oP00>oooooah00?l02@000?l20?n000/00?l01`000?l6003o00030000o`00o`000?l000@0
003o0`00o`070000o`@00?l00P000?l2003o0100003o0`00o`0F0000o`<00?l010000?l>ooooo`80
003o0P00o`000`000?l00?l0003o0002003o00L0003o0P3oP0040000o`030?n000000?l0003o01/0
003o0P3oP00;0000o`030?n000000?l0003o01H0003o00<0oh000000o`000?l00P000?l20?n000L0
003o0P3oP00P0000o`kooooo1@3o00020?n000H0o`0000<0oh000?l0003o0000103o00020?n00003
0000o`3oP000oh000080o`000P3oP0000`3o0000oh000?n000040?n000<0003o0P3o00020000o`80
oh000P3o00000`3oP000o`000?l000020?n00080003o1@3o00020?n000L0o`000P3oP0020?l000T0
oh0000<0o`000000o`000?l01P3oP0000`3o0000oh000?n000020000o`80oh001`3o00050?n00080
o`000P3oP0030?l000H0oh000P3o00000`3oP000o`000?l000020?n00080o`000P3oP0000`3o0000
oh000?n000020?l001Sooooo000Fooooo`80o`000P3oP00I0?l000030000o`3o0000o`000080o`00
0P000?l20?n000L0o`000`3oP0060?l000<0oh006@3o00020?n000H0o`0000<0oh000?l0003o0000
103o00030?n000d0o`000`3oP0040?l00080oh003P3o00020?n000kooooo0P00o`020000o`<00?l0
10000?l7003o0080003o0P00o`030000o`L00?l01P000?l00`3o0000oh000?n000020000o`/00?l0
1`000?l4003o00030000o`00o`000?l000800?l00P000?l5003o01d0003o1@00o`0B0000o`80oh00
1`000?l>ooooo`T00?l02@000?l20?n00080003o00<0oh000000o`000?l01@000?l2003o00d0003o
0P3oP0050000o`80oh00<P000?l20?n000<0003o0P3oP00G0000o`80oh000P000?l>ooooo`@0oh00
00<0o`000?n0003oP000103oP0020?l000D0oh000P3o00020?n000030?l0003oP000oh000080oh00
0P000?l70?n00080003o00<0o`000?n0003oP0000P3o00090?n00080o`002@3oP0090?l000D0oh00
1`3o00040?n000<0o`000P3oP0040?l00080oh001@3o00020?n000D0o`000P3oP0030?l00080oh00
1P3o00000`3oP000o`000?l000020?n000@0o`000`3oP0020?l001Sooooo000Fooooo`80o`000P3o
P00I0?l000030000o`3o0000o`000080o`000P000?l20?n000L0o`000`3oP0060?l000<0oh006@3o
00020?n000H0o`0000<0oh000?l0003o0000103o00030?n000d0o`000`3oP0040?l00080oh003P3o
00020?n000kooooo0P00o`020000o`<00?l010000?l7003o0080003o0P00o`030000o`L00?l01P00
0?l00`3o0000oh000?n000020000o`/00?l01`000?l4003o00030000o`00o`000?l000800?l00P00
0?l5003o01d0003o1@00o`0B0000o`80oh001`000?l>ooooo`T00?l02@000?l20?n00080003o00<0
oh000000o`000?l01@000?l2003o00d0003o0P3oP0050000o`80oh00<P000?l20?n000<0003o0P3o
P00G0000o`80oh000P000?l>ooooo`@0oh0000<0o`000?n0003oP000103oP0020?l000D0oh000P3o
00020?n000030?l0003oP000oh000080oh000P000?l70?n00080003o00<0o`000?n0003oP0000P3o
00090?n00080o`002@3oP0090?l000D0oh001`3o00040?n000<0o`000P3oP0040?l00080oh001@3o
00020?n000D0o`000P3oP0030?l00080oh001P3o00000`3oP000o`000?l000020?n000@0o`000`3o
P0020?l001Sooooo000Fooooo`80o`000P3oP00<0?l00080oh007@3o00000`3oP000o`000?l00004
0?l00080oh008P3o00000`3oP000o`000?l0000;0?l00080oh00303o00020?n00080o`000P000?l9
0?l000030?n0003o0000o`000080oh003_ooool40000o`h00?l01`000?l7003o0080003o0P00o`03
0000o`@00?l00P000?l7003o00T0003o1000o`0A0000o`@00?l050000?l5003o00h0003o0P3oP00;
0000o`kooooo0P000?l5003o00/0003o0P3oP0020000o`030?n000000?l0003o0080003o0`00o`06
0000o`80oh0030000?l40?n00180003o00<0oh000000o`000?l010000?l30?n000T0003o0P00o`0;
0000o`80oh000`000?l20?n000T0003o1@3oP0040?l000T0003o3_ooool40?l000030000o`3oP000
oh000080oh000P3o00020?n000<0o`001P3oP0000`3o0000oh000?n000050?n00080003o1`3oP002
0?l00080003o1@3oP0040?l000@0oh0000<0o`000?n0003oP000103oP0020000o`030?l0003oP000
oh000080oh000P3o00000`3oP000o`000?l000020?n00080003o0P3oP0000`3o0000oh000?n00002
0?n000D0o`000P3oP0060?l000<0oh000P3o00020?n000D0o`00103oP0000`3o0000oh000?n00002
0?l00080oh000P3o00030?n00080o`000P3oP0030?l000@0oh006?ooool001Kooooo103oP00N0?l0
0080oh002@3o00030000o`H0o`000P000?l90?l000030?n0003o0000o`0001L0o`000P3oP00;0?l0
0080oh001`3o00020?n000030000o`3o0000o`000080o`000P3oP0020000o`030?n0003o0000o`00
00L0o`000P000?l20?l000kooooo10000?l>003o0080003o3000o`090000o`800?l01`000?l4003o
00L0003o1000o`070000o`<00?l010000?l5003o0080003o0P00o`090000o`800?l01@000?l2003o
00h0003o0P3oP0030000o`80oh0010000?l20?n000D0003o3_ooool20?n000D00?l02@000?l40?n0
0080o`0000<0oh000000o`000?l00P3oP0090000o`80oh004`000?l2003o0080003o0P00o`070000
o`80oh001`000?l20?n000<0o`001P000?l3003o0100003o0P3oP0040000o`030?n000000?l0003o
0080003o1@3oP0020?l000/0003o3_ooool40?l000D0003o1`3o00020?n00080o`001`3oP0020000
o`030?n000000?l0003o00@0oh001@3o00040?n000<0o`001P3oP0020?l000<0oh000P000?l20?n0
0080003o303oP0020000o`80oh0000<0003o0?n0003oP0000P3oP0020?l00080oh0000<0o`000?n0
003oP000103o00050?n00080o`001@3oP0020?l000/0oh000P3o00000`3oP000o`000?l000060?l0
00D0oh006?ooool001Kooooo103oP00N0?l00080oh002@3o00030000o`H0o`000P000?l90?l00003
0?n0003o0000o`0001L0o`000P3oP00;0?l00080oh001`3o00020?n000030000o`3o0000o`000080
o`000P3oP0020000o`030?n0003o0000o`0000L0o`000P000?l20?l000kooooo10000?l>003o0080
003o3000o`090000o`800?l01`000?l4003o00L0003o1000o`070000o`<00?l010000?l5003o0080
003o0P00o`090000o`800?l01@000?l2003o00h0003o0P3oP0030000o`80oh0010000?l20?n000D0
003o3_ooool20?n000D00?l02@000?l40?n00080o`0000<0oh000000o`000?l00P3oP0090000o`80
oh004`000?l2003o0080003o0P00o`070000o`80oh001`000?l20?n000<0o`001P000?l3003o0100
003o0P3oP0040000o`030?n000000?l0003o0080003o1@3oP0020?l000/0003o3_ooool40?l000D0
003o1`3o00020?n00080o`001`3oP0020000o`030?n000000?l0003o00@0oh001@3o00040?n000<0
o`001P3oP0020?l000<0oh000P000?l20?n00080003o303oP0020000o`80oh0000<0003o0?n0003o
P0000P3oP0020?l00080oh0000<0o`000?n0003oP000103o00050?n00080o`001@3oP0020?l000/0
oh000P3o00000`3oP000o`000?l000060?l000D0oh006?ooool001Kooooo1@3o00040?n001T0o`00
0P3oP0070?l000D0oh003`3o00020?n000<0o`000P3oP00>0?l00080oh003@3o00000`3oP000o`00
0?l0000;0?l000<0oh000P000?l20?n00080o`000P3oP0090?l000<0003o0P3o000>ooooo`80003o
0P3oP00>003o0080003o4000o`020000o`03003o00000?l0003o00T0003o1000o`070000o`@00?l0
00<0003o003o0000o`000P00o`030000o`800?l00P000?l2003o00T0003o0`00o`020000o`H00?l0
00<0003o003o0000o`0080000?l2003o0080003o3_ooool40000o`D00?l00P000?l2003o00T0003o
00<0oh000000o`000?l03@000?l30?n000L0003o0P3oP00?0000o`030?n000000?l0003o0080003o
0P3oP0000`000?l0oh000?n000020000o`@0oh0000<0003o0?n0003oP0001`000?l2003o00H0003o
00<0oh000000o`000?l050000?l20?l000030?n0003o0000o`0000H0oh001@000?l>ooooo`80oh00
0P3o00030?n000@0o`000`3oP0040?l00080oh000P3o00000`3oP000003o0000o`020?n00080003o
1`3oP0020000o`<0oh000P3o00020?n00080o`000`3oP0060?l000`0oh000P3o00020?n000030?l0
003oP000oh0000T0oh000P3o00040?n000030?l0003oP000oh0000@0oh001`3o00070?n00080003o
1@3oP0060?l000D0oh000P000?l20?l000030?n0003o0000o`000080oh006?ooool001Kooooo1@3o
00040?n001T0o`000P3oP0070?l000D0oh003`3o00020?n000<0o`000P3oP00>0?l00080oh003@3o
00000`3oP000o`000?l0000;0?l000<0oh000P000?l20?n00080o`000P3oP0090?l000<0003o0P3o
000>ooooo`80003o0P3oP00>003o0080003o4000o`020000o`03003o00000?l0003o00T0003o1000
o`070000o`@00?l000<0003o003o0000o`000P00o`030000o`800?l00P000?l2003o00T0003o0`00
o`020000o`H00?l000<0003o003o0000o`0080000?l2003o0080003o3_ooool40000o`D00?l00P00
0?l2003o00T0003o00<0oh000000o`000?l03@000?l30?n000L0003o0P3oP00?0000o`030?n00000
0?l0003o0080003o0P3oP0000`000?l0oh000?n000020000o`@0oh0000<0003o0?n0003oP0001`00
0?l2003o00H0003o00<0oh000000o`000?l050000?l20?l000030?n0003o0000o`0000H0oh001@00
0?l>ooooo`80oh000P3o00030?n000@0o`000`3oP0040?l00080oh000P3o00000`3oP000003o0000
o`020?n00080003o1`3oP0020000o`<0oh000P3o00020?n00080o`000`3oP0060?l000`0oh000P3o
00020?n000030?l0003oP000oh0000T0oh000P3o00040?n000030?l0003oP000oh0000@0oh001`3o
00070?n00080003o1@3oP0060?l000D0oh000P000?l20?l000030?n0003o0000o`000080oh006?oo
ool001Kooooo1`3o00040?n001L0o`000P3oP0070?l00080oh002@3o00030?n000/0o`000P3oP00>
0?l000@0oh000`3o00020?n00180o`00103oP0000`3o0000003o0000o`040?n00080003o00<0oh00
0?l0003o00001`3o00020?n00080003o3_oooolB003o0080003o7000o`020000o`@00?l01@000?l;
003o0080003o1`00o`020000o`03003o00000?l0003o00L0003o0P00o`040000o`800?l01`000?l2
003o00<0003o0P00o`0<0000o`80oh002@000?l2003o00kooooo10000?l5003o00d0003o00<0oh00
0000o`000?l03@000?l30?n00180003o1P00o`050000o`80oh0000<0003o0?n0003oP000103oP002
0?l000<0oh0010000?l20?n000030000o`3oP000oh0000@0003o0P3oP00>0000o`80oh002P000?l2
0?l00080oh002@000?l>ooooo`T0o`002`3oP0030?l000@0oh000P3o00030?n00080o`000P3oP007
0?l00080oh000P3o00030?n000@0o`000P3oP0050?l000T0oh000P3o00000`3oP000o`000?l00004
0?n00080o`000`3oP0020?l00080oh000P3o00030?n00080003o0P3o00020?n000030?l000000?l0
003o0080o`000P3oP0020000o`P0oh00203o00050?n000@0o`0000<0oh000?l0003o00000P3o000H
ooooo`005_ooool70?l000@0oh005`3o00020?n000L0o`000P3oP0090?l000<0oh002`3o00020?n0
00h0o`00103oP0030?l00080oh004P3o00040?n000030?l000000?l0003o00@0oh000P000?l00`3o
P000o`000?l000070?l00080oh000P000?l>oooooa800?l00P000?lL003o0080003o1000o`050000
o`/00?l00P000?l7003o0080003o00<00?l00000o`000?l01`000?l2003o00@0003o0P00o`070000
o`800?l00`000?l2003o00`0003o0P3oP0090000o`800?l03_ooool40000o`D00?l03@000?l00`3o
P000003o0000o`0=0000o`<0oh004P000?l6003o00D0003o0P3oP0000`000?l0oh000?n000040?n0
0080o`000`3oP0040000o`80oh0000<0003o0?n0003oP00010000?l20?n000h0003o0P3oP00:0000
o`80o`000P3oP0090000o`kooooo2@3o000;0?n000<0o`00103oP0020?l000<0oh000P3o00020?n0
00L0o`000P3oP0020?l000<0oh00103o00020?n000D0o`002@3oP0020?l000030?n0003o0000o`00
00@0oh000P3o00030?n00080o`000P3oP0020?l000<0oh000P000?l20?l00080oh0000<0o`000000
o`000?l00P3o00020?n00080003o203oP0080?l000D0oh00103o00000`3oP000o`000?l000020?l0
01Sooooo000Fooooo`80oh00403o00020?n000L0o`002@3oP0070?l00080oh002@3o00020?n00140
o`000P3oP0090?l000@0oh00803o00020000o`<0oh001P3o00020000o`030?l0003oP000oh000080
o`003_ooool7003o0080003o6`00o`020000oa/00?l01`000?l3003o0080003o1`00o`020000o`/0
0?l02@000?l2003o00030000o`00o`000?l000800?l00P000?l5003o00D0003o0P3oP002003o00L0
003o0P00o`0>ooooo`@0003o0`00o`0?0000o`030?n000000?l0003o00d0003o0`3oP00@0000o`80
oh0040000?l60?l000<0oh0060000?l00`3oP000003o0000o`070000o`800?l01`000?l20?n000D0
003o0P00o`020000o`kooooo103oP0050?l000/0oh001@3o00050?n000@0o`00103oP0030?l000l0
oh001@3o00020?n000T0o`0000<0oh000?l0003o000010000?l70?l000T0oh000P3o00050?n00080
o`002@3oP0020?l000L0oh000P3o00000`3oP000003o0000o`020?l00080oh001`3o000Hooooo`00
5_ooool40?n000`0o`000P3oP0090?l00080oh000`3o00020?n000L0o`000P000?l20?n000T0o`00
0P3oP00A0?l00080oh001P3o00050?n000T0o`000P3oP0090?l00080oh00303o00020000o`80oh00
2@3o00000`000?l0oh000?n000020?l000kooooo>`00o`090000o`800?l00P000?l5003o00@0003o
00<00?l00000o`000?l01`000?l=003o00L0003o2@00o`070000o`800?l02@000?l>ooooo`80003o
1@00o`0X0000o`030?n000000?l0003o01L0003o1P3oP0020?l000<0oh000P000?l20?n000L0003o
0P3oP00W0000o`80oh003_ooool20?l00080oh001@3o000;0?n000<0o`000P000?l20?n00080003o
1@3o00020?n000D0o`00103oP0020?l000<0oh000P000?l20?n000L0o`00103oP0070?l000030?n0
003o0000o`0000@0o`000P3oP0000`3o0000oh000?n000020?l000D0oh001P3o00070?n00080o`00
0`3oP0020?l00080oh001@3o00040?n00080o`0000<0oh000000o`000?l00P3o00020?n000D0o`00
0P3oP00Hooooo`005_ooool40?n000`0o`000P3oP0090?l00080oh000`3o00020?n000L0o`000P00
0?l20?n000T0o`000P3oP00A0?l00080oh001P3o00050?n000T0o`000P3oP0090?l00080oh00303o
00020000o`80oh002@3o00000`000?l0oh000?n000020?l000kooooo>`00o`090000o`800?l00P00
0?l5003o00@0003o00<00?l00000o`000?l01`000?l=003o00L0003o2@00o`070000o`800?l02@00
0?l>ooooo`80003o1@00o`0X0000o`030?n000000?l0003o01L0003o1P3oP0020?l000<0oh000P00
0?l20?n000L0003o0P3oP00W0000o`80oh003_ooool20?l00080oh001@3o000;0?n000<0o`000P00
0?l20?n00080003o1@3o00020?n000D0o`00103oP0020?l000<0oh000P000?l20?n000L0o`00103o
P0070?l000030?n0003o0000o`0000@0o`000P3oP0000`3o0000oh000?n000020?l000D0oh001P3o
00070?n00080o`000`3oP0020?l00080oh001@3o00040?n00080o`0000<0oh000000o`000?l00P3o
00020?n000D0o`000P3oP00Hooooo`005_oooolI0?l000@0oh001@3o00020?n001@0o`0000<0003o
0?l0003o00006@3o00040?n000`0o`00103oP0020?l00080oh00303o00020?n00100o`003_oooolF
003o00<0003o3@00o`000`000?l00?l0003o0006003o00<0003o2@00o`040000o`800?l000<0003o
003o0000o`000P00o`020000o`03003o00000?l0003o0100003o1P00o`020000oaD00?l00P000?l2
003o00`0003o0P00o`0>ooooob00003o0P3oP0040000o`030?n000000?l0003o00H0003o00<0oh00
0000o`000?l02@000?l20?n00100003o1@3o00020?n00080003o0P3oP0020000o`<0oh002`000?l2
0?n00200003o3_ooool20?l00080oh0000<0o`000?n0003oP0001`3oP0020000o`@0oh001@3o0002
0?n00080003o0`3oP0020?l00080oh001@3o00060?n000030?l0003oP000oh000080003o0P3oP007
0?l00080oh000P3o00000`000?l0oh000?n000020?n000<0o`000P3oP0020?l00080oh000`3o0008
0?n000030000o`3o0000o`0000@0o`000P3oP0000`000?l0oh000?n000040?n000<0o`001`3oP002
0?l00080oh002@000?l90?n001Sooooo000FoooooaT0o`00103oP0050?l00080oh00503o00000`00
0?l0o`000?l0000I0?l000@0oh00303o00040?n00080o`000P3oP00<0?l00080oh00403o000>oooo
oaH00?l00`000?l=003o00030000o`00o`000?l000H00?l00`000?l9003o00@0003o0P00o`000`00
0?l00?l0003o0002003o0080003o00<00?l00000o`000?l040000?l6003o0080003o5@00o`020000
o`800?l030000?l2003o00kooooo80000?l20?n000@0003o00<0oh000000o`000?l01P000?l00`3o
P000003o0000o`090000o`80oh0040000?l50?l00080oh000P000?l20?n00080003o0`3oP00;0000
o`80oh0080000?l>ooooo`80o`000P3oP0000`3o0000oh000?n000070?n00080003o103oP0050?l0
0080oh000P000?l30?n00080o`000P3oP0050?l000H0oh0000<0o`000?n0003oP0000P000?l20?n0
00L0o`000P3oP0020?l000030000o`3oP000oh000080oh000`3o00020?n00080o`000P3oP0030?l0
00P0oh0000<0003o0?l0003o0000103o00020?n000030000o`3oP000oh0000@0oh000`3o00070?n0
0080o`000P3oP0090000o`T0oh006?ooool001Kooooo2@3o00040?n000<0o`000P3oP00T0?l00080
oh006@3o00030000ob@0o`000P3oP00;0?l000<0oh000P3o000>ooooob@00?l00P000?l00`00o`00
003o0000o`0B003o00@0003o0P00o`070000o`03003o00000?l0003o00H0003o00<0oh000000o`00
0?l00P00o`090000o`h00?l01P000?l5003o00h0003o0P3oP002003o00kooooo1`000?l2003o00L0
003o0P00o`0>0000o`80oh001@000?l20?n000H0003o00<0oh000000o`000?l02@000?l20?n00100
003o103oP0000`3o0000oh000?n000020000o`80oh001`000?l20?n00080003o0`3oP0020000o`80
oh000`000?l20?n00180003o1000o`050000o`kooooo0P3oP0030?l00080oh000P000?l40?n000<0
003o5P3oP0000`3o0000oh000?n000040?n000<0o`00103oP0020?l000D0oh000P3o00020?n000<0
o`00103oP0090?l00080oh0000<0o`000?n0003oP0000P3oP0020000o`<0o`00103oP0020?l00080
oh0000<0003o0?l0003o00000P3oP0050?l000@0oh000P000?l70?n00080o`0000<0oh000000o`00
0?l00P000?l20?n000<0003o0P3oP0020?l001Sooooo000Fooooo`T0o`00103oP0030?l00080oh00
903o00020?n001T0o`000`000?lT0?l00080oh002`3o00030?n00080o`003_oooolT003o0080003o
00<00?l00000o`000?l04P00o`040000o`800?l01`000?l00`00o`00003o0000o`060000o`030?n0
00000?l0003o00800?l02@000?l>003o00H0003o1@00o`0>0000o`80oh000P00o`0>ooooo`L0003o
0P00o`070000o`800?l03P000?l20?n000D0003o0P3oP0060000o`030?n000000?l0003o00T0003o
0P3oP00@0000o`@0oh0000<0o`000?n0003oP0000P000?l20?n000L0003o0P3oP0020000o`<0oh00
0P000?l20?n000<0003o0P3oP00B0000o`@00?l01@000?l>ooooo`80oh000`3o00020?n00080003o
103oP0030000oaH0oh0000<0o`000?n0003oP000103oP0030?l000@0oh000P3o00050?n00080o`00
0P3oP0030?l000@0oh002@3o00020?n000030?l0003oP000oh000080oh000P000?l30?l000@0oh00
0P3o00020?n000030000o`3o0000o`000080oh001@3o00040?n00080003o1`3oP0020?l000030?n0
00000?l0003o0080003o0P3oP0030000o`80oh000P3o000Hooooo`005_ooool70?l000@0oh001@3o
00020000obH0o`0000<0oh000?l0003o00002`3o00020?n000T0o`0000<0oh000?l0003o0000503o
00020?n000h0o`0010000?l90?l000030?n0003o0000o`000080o`003_ooool40000o`<00?l00P00
0?l9003o0080003o0P00o`030000o`T00?l01`000?lB003o00T0003o1@00o`0?0000o`800?l000<0
003o003o0000o`001000o`020000oa000?l05`000?l>oooooa00003o1000o`0<0000o`80oh002`00
0?l20?n002/0003o00<0oh000000o`000?l01`000?l20?n001T0003o1P00o`050000o`kooooo0P3o
00020?n000030?l0003oP000oh0000H0oh000`000?l40?n00080o`000`3oP0020000o`80oh0000<0
o`000?n0003oP0000P3oP0020?l00080003o1@3o00020?n000<0o`00103oP0040?l000030?n0003o
0000o`000080o`000P000?l20?n000<0o`001@3oP0020?l00080oh001@3o00040?n00080003o1`3o
P0040000o`L0oh000`3o00040?n00080003o0`3oP0020?l00080oh000P000?l30?n00080003o103o
P0050000oaSooooo000Fooooo`L0o`00103oP0050?l00080003o9P3o00000`3oP000o`000?l0000;
0?l00080oh002@3o00000`3oP000o`000?l0000D0?l00080oh003P3o00040000o`T0o`0000<0oh00
0?l0003o00000P3o000>ooooo`@0003o0`00o`020000o`T00?l00P000?l2003o00<0003o2@00o`07
0000oa800?l02@000?l5003o00l0003o0P00o`000`000?l00?l0003o0004003o0080003o4000o`0G
0000o`kooooo40000?l4003o00`0003o0P3oP00;0000o`80oh00:`000?l00`3oP000003o0000o`07
0000o`80oh006@000?l6003o00D0003o3_ooool20?l00080oh0000<0o`000?n0003oP0001P3oP003
0000o`@0oh000P3o00030?n00080003o0P3oP0000`3o0000oh000?n000020?n00080o`000P000?l5
0?l00080oh000`3o00040?n000@0o`0000<0oh000?l0003o00000P3o00020000o`80oh000`3o0005
0?n00080o`000P3oP0050?l000@0oh000P000?l70?n000@0003o1`3oP0030?l000@0oh000P000?l3
0?n00080o`000P3oP0020000o`<0oh000P000?l40?n000D0003o6?ooool001Kooooo4P3o00020?n0
0080o`0000<0oh000?l0003o00000P3oP0030?l00080oh00603o00000`3oP000o`000?l000040?l0
0080oh001@3o00020?n000h0o`000P3oP00@0?l00080oh001@3o00020?n000T0o`000`3oP0040?l0
0080003o0P3o00000`3oP000o`000?l000020?l000kooooo0P00o`020000o`<00?l010000?lG003o
00L0003o3@00o`0>0000o`@00?l05@000?l6003o00<0003o2@00o`020000o`800?l000<0003o003o
0000o`0050000?l>ooooo`D0003o0P3oP00;0000o`800?l01@000?l20?n00180003o0P3oP00<0000
o`H00?l01@000?l30?n00180003o1000o`050000o`80oh003@000?l7003o00L0003o1000o`050000
o`kooooo0P3o00020?n000030?l0003oP000oh000080003o1`3oP0020?l00080oh000P3o00030?n0
0080003o0P3o00050?n00080o`000P3oP0000`3o0000oh000?n000020?l000D0oh001P3o00020?n0
00D0o`002`3oP0050?l000L0oh00103o00050?n00080o`000P3oP0020?l000L0oh001@3o00020?n0
0080o`0000<0003o0?n0003oP000103oP0020?l000<0oh000P000?l50?n00080o`000P3oP00Hoooo
o`005_ooool@0?l00080oh001@3o00020?n000D0o`000P3oP0040?l000<0oh00:03o00020000o`L0
o`000P3oP00Y0?l000<0oh00103o000>ooooo`T0003o0P00o`030000oa@00?l01`000?lB003o00h0
003o0P00o`070000o`800?l02`000?l4003o00030000o`00o`000?l000T00?l01@000?l6003o00<0
003o1000o`090000o`kooooo50000?l20?l000<0003o0P3oP00B0000o`80oh0000<0003o0?n0003o
P0000P3oP0050000o`P00?l01@000?l20?n00180003o0`00o`0]0000o`kooooo0P000?l20?l00003
0000o`3o0000o`000080o`001`3oP0020?l00080oh000P3o00000`3oP000003o0000o`040?n00003
0000o`3oP000oh000080oh00103o00000`000?l0oh000?n000020000o`L0o`00103oP0020?l00003
0?n0003o0000o`0000@0oh000P000?l30?n00080o`000P3oP0050?l000@0oh0000<0o`000?n0003o
P0000P3o00050?n00080o`000P3oP0020000o`<0oh000P3o00020?n000L0o`003@3oP0000`000?l0
oh000?n000040?n000<0o`00103oP00Hooooo`005_ooool@0?l00080oh001@3o00020?n000D0o`00
0P3oP0040?l000<0oh00:03o00020000o`L0o`000P3oP00Y0?l000<0oh00103o000>ooooo`T0003o
0P00o`030000oa@00?l01`000?lB003o00h0003o0P00o`070000o`800?l02`000?l4003o00030000
o`00o`000?l000T00?l01@000?l6003o00<0003o1000o`090000o`kooooo50000?l20?l000<0003o
0P3oP00B0000o`80oh0000<0003o0?n0003oP0000P3oP0050000o`P00?l01@000?l20?n00180003o
0`00o`0]0000o`kooooo0P000?l20?l000030000o`3o0000o`000080o`001`3oP0020?l00080oh00
0P3o00000`3oP000003o0000o`040?n000030000o`3oP000oh000080oh00103o00000`000?l0oh00
0?n000020000o`L0o`00103oP0020?l000030?n0003o0000o`0000@0oh000P000?l30?n00080o`00
0P3oP0050?l000@0oh0000<0o`000?n0003oP0000P3o00050?n00080o`000P3oP0020000o`<0oh00
0P3o00020?n000L0o`003@3oP0000`000?l0oh000?n000040?n000<0o`00103oP00Hooooo`005_oo
ool20?n000`0o`000P3oP00D0?l000<0oh00<`3o00000`3oP000o`000?l0000=0?l00080oh005@3o
00020?n00080003o00<0o`000?n0003oP0000P3oP00>ooooo`800?l030000?l8003o00030000o`00
o`000?l000T00?l02@000?l@003o00/0003o0`00o`020000o`800?l01@000?l2003o00H0003o00<0
0?l00000o`000?l00P00o`070000o`T00?l06`000?l>ooooo`D0003o0P00o`0B0000o`80oh000`00
0?l2003o00H0003o00<0oh000000o`000?l01`000?l20?n000@0003o0P3oP0030000o`@00?l01@00
0?l20?n000h0003o0P3oP00d0000o`kooooo0P000?l30?n00080o`000P3oP0020?l00080oh0000<0
003o0?l0003o0000103oP0020?l000D0oh000P000?l00`3oP000003o0000o`020?n000@0o`001@3o
P0040?l000D0oh000P3o00030?n00080o`000P3oP0020?l00080003o00<0o`000?n0003oP0000P3o
00020?n000D0o`001`3oP0020?l000D0oh000P3o00040?n00080o`000`3oP0020?l00080oh001@3o
00070?n00080o`000P3oP0030000o`80oh001P3o00000`3oP000o`000?l000020?n001Sooooo000F
ooooo`80oh00303o00020?n001@0o`000`3oP00c0?l000030?n0003o0000o`0000d0o`000P3oP00E
0?l00080oh000P000?l00`3o0000oh000?n000020?n000kooooo0P00o`0<0000o`P00?l000<0003o
003o0000o`002@00o`090000oa000?l02`000?l3003o0080003o0P00o`050000o`800?l01P000?l0
0`00o`00003o0000o`02003o00L0003o2@00o`0K0000o`kooooo1@000?l2003o0180003o0P3oP003
0000o`800?l01P000?l00`3oP000003o0000o`070000o`80oh0010000?l20?n000<0003o1000o`05
0000o`80oh003P000?l20?n003@0003o3_ooool20000o`<0oh000P3o00020?n00080o`000P3oP000
0`000?l0o`000?l000040?n00080o`001@3oP0020000o`030?n000000?l0003o0080oh00103o0005
0?n000@0o`001@3oP0020?l000<0oh000P3o00020?n00080o`000P000?l00`3o0000oh000?n00002
0?l00080oh001@3o00070?n00080o`001@3oP0020?l000@0oh000P3o00030?n00080o`000P3oP005
0?l000L0oh000P3o00020?n000<0003o0P3oP0060?l000030?n0003o0000o`000080oh006?ooool0
01Kooooo0P3oP0020?l000030?n0003o0000o`000080oh006`3o00020?n000L0o`000P3oP00/0?l0
00@0oh00103o00000`3oP000o`000?l000020?n00080o`000`3oP00M0?l000kooooo1`00o`070000
o`P00?l05@000?l=003o00030000o`00o`000?l000@0003o0P00o`000`000?l00?l0003o0002003o
00`0003o1000o`0B0000o``00?l00P000?l2003o00@0003o00<00?l00000o`000?l010000?l20?n0
00<0003o0P00o`0>oooooc00003o0P3oP0040000o`80oh001@000?l2003o01<0003o1P3oP0050000
o`800?l0:`000?l>ooooo`@0o`0000<0oh000?l0003o00000P3oP0020000o`80oh0000<0o`000?n0
003oP0000P3o00020?n000<0o`001P3oP0070?l00080003o0`3oP0020000o`@0o`000`3oP0020?l0
0080oh001@3o00040?n00080003o00<0o`000?n0003oP0001`3oP0040?l000h0oh002`3o00050?n0
0080o`000P3oP0020?l000030?n0003o0000o`000080o`000P3oP0020?l000030000o`3o0000o`00
00H0o`0000<0oh000?l0003o00000P3oP00Hooooo`005_ooool20?n00080o`0000<0oh000?l0003o
00000P3oP00K0?l00080oh001`3o00020?n002`0o`00103oP0040?l000030?n0003o0000o`000080
oh000P3o00030?n001d0o`003_ooool7003o00L0003o2000o`0E0000o`d00?l000<0003o003o0000
o`0010000?l2003o00030000o`00o`000?l000800?l030000?l4003o0180003o3000o`020000o`80
0?l010000?l00`00o`00003o0000o`040000o`80oh000`000?l2003o00kooooo<0000?l20?n000@0
003o0P3oP0050000o`800?l04`000?l60?n000D0003o0P00o`0[0000o`kooooo103o00000`3oP000
o`000?l000020?n00080003o0P3oP0000`3o0000oh000?n000020?l00080oh000`3o00060?n000L0
o`000P000?l30?n00080003o103o00030?n00080o`000P3oP0050?l000@0oh000P000?l00`3o0000
oh000?n000070?n000@0o`003P3oP00;0?l000D0oh000P3o00020?n00080o`0000<0oh000?l0003o
00000P3o00020?n00080o`0000<0003o0?l0003o00001P3o00000`3oP000o`000?l000020?n001So
oooo000Fooooo`@0oh003P3o00040?n000<0o`00103oP0090?l000030?n0003o0000o`000080o`00
0P3oP0070?l00080oh001@3o00020?n001h0o`000P3oP00M0?l00080oh002P3o00020?n00080o`00
3_ooool7003o00L0003o2000o`0>0000oa800?l05`000?l20?n000<0003o0P00o`020000o`800?l0
3P000?l6003o00030000o`00o`000?l000800?l00P000?l00`00o`00003o0000o`0?0000o`D00?l0
3_oooolG0000o`H00?l030000?l20?n001/0003o0P3oP00:0000o`80o`00103oP00R0000o`800?l0
3P000?l>ooooo`80003o1@3o000=0?n000<0o`001P3oP0050?l00080oh000P3o00000`000?l0oh00
0?n000040?n00080o`000`3oP0070?l000H0oh000P000?l30?n00080o`000P3oP0000`000?l0oh00
0?n000020?n00080003o0P3oP0000`000?l0o`000?l000020?l000D0oh000P000?l60?l000<0oh00
0P3o00040?n000<0o`00103oP0050?l00080oh000P3o00000`3oP000o`000?l000060?l000D0oh00
6?ooool001Kooooo103oP00>0?l000@0oh000`3o00040?n000T0o`0000<0oh000?l0003o00000P3o
00020?n000L0o`000P3oP0050?l00080oh007P3o00020?n001d0o`000P3oP00:0?l00080oh000P3o
000>ooooo`L00?l01`000?l8003o00h0003o4P00o`0G0000o`80oh000`000?l2003o0080003o0P00
o`0>0000o`H00?l000<0003o003o0000o`000P00o`020000o`03003o00000?l0003o00l0003o1@00
o`0>oooooaL0003o1P00o`0<0000o`80oh006`000?l20?n000X0003o0P3o00040?n00280003o0P00
o`0>0000o`kooooo0P000?l50?l000d0oh000`3o00060?n000D0o`000P3oP0020?l000030000o`3o
P000oh0000@0oh000P3o00030?n000L0o`001P3oP0020000o`<0oh000P3o00020?n000030000o`3o
P000oh000080oh000P000?l20?n000030000o`3o0000o`000080o`001@3oP0020000o`H0o`000`3o
P0020?l000@0oh000`3o00040?n000D0o`000P3oP0020?l000030?n0003o0000o`0000H0o`001@3o
P00Hooooo`005_ooool@0?l000H0oh005@3o00020?n000L0o`000P3oP0050?l00080oh002`3o0000
0`3oP000o`000?l000020?l00080oh00503o00000`3oP000o`000?l0000T0?l000kooooo5`00o`0=
0000oa800?l040000?l7003o01T0003o103oP0030000o`800?l01@000?l2003o00d0003o1`00o`0>
ooooo`T0003o0P3oP0030000o`80oh003@000?l00`00o`00003o0000o`0f0000o`80oh001@000?l2
003o00D0003o0P3oP0020000o`80oh001@000?l40?n001L0003o3_ooool20000o`<0oh000P3o0002
0?n00080o`000`3oP0020000o`L0o`00103oP0030?l000/0oh000P3o00050?n000@0o`003@3oP003
0000o`80oh000P3o00030000o`@0oh000P3o00030?n00080o`003P3oP0020?l000H0oh000`3o0004
0?n000D0o`000P3oP0020?l000030?n000000?l0003o00@0oh000P3o00050?n001Sooooo000Foooo
o`L0o`000P3oP0090?l00080oh001@3o00020?n000L0o`000P000?l20?n000D0o`000P3oP00L0?l0
0080oh002@3o00040?n00280o`000P3oP00>0?l000kooooo7P00o`080000oa000?l09`000?l2003o
00L0003o0P3o00040000o`030?n00000o`000?l000T00?l00P000?l3003o00T0003o1000o`0>oooo
o`L0003o103oP0020000o`<0oh001`000?l2003o0100003o1P3oP0000`000?l0oh000?n000040000
o`80oh001`000?l20?n000X0003o103oP00<0000o`@00?l01@000?l20?n000h0003o0P3oP00?0000
o`030?n000000?l0003o0080003o3_ooool40000o`<0oh000P3o00050?n00080o`00103oP0030?l0
0080oh001@3o00080?n000<0003o0P3o00070?n000@0o`001@3oP0020?l00080oh000`000?l60?n0
00T0o`0000<0oh000000o`000?l00P3o00020?n00080003o00<0oh000000o`000?l00P3oP0090?l0
00@0oh0000<0o`000?n0003oP0000P3o00030?n00080o`00103oP0020?l000<0oh000P000?l20?n0
00<0003o0P3oP0020000oaSooooo000Fooooo`L0o`000P3oP0090?l00080oh001@3o00020?n000L0
o`000P000?l20?n000D0o`000P3oP00L0?l00080oh002@3o00040?n00280o`000P3oP00>0?l000ko
oooo7P00o`080000oa000?l09`000?l2003o00L0003o0P3o00040000o`030?n00000o`000?l000T0
0?l00P000?l3003o00T0003o1000o`0>ooooo`L0003o103oP0020000o`<0oh001`000?l2003o0100
003o1P3oP0000`000?l0oh000?n000040000o`80oh001`000?l20?n000X0003o103oP00<0000o`@0
0?l01@000?l20?n000h0003o0P3oP00?0000o`030?n000000?l0003o0080003o3_ooool40000o`<0
oh000P3o00050?n00080o`00103oP0030?l00080oh001@3o00080?n000<0003o0P3o00070?n000@0
o`001@3oP0020?l00080oh000`000?l60?n000T0o`0000<0oh000000o`000?l00P3o00020?n00080
003o00<0oh000000o`000?l00P3oP0090?l000@0oh0000<0o`000?n0003oP0000P3o00030?n00080
o`00103oP0020?l000<0oh000P000?l20?n000<0003o0P3oP0020000oaSooooo000Fooooo`D0oh00
0P3o00020?n001T0o`000P3oP0030?l000H0oh002@3o00020?n00100o`0000<0oh000?l0003o0000
603o00000`3oP000o`000?l0000K0?l000@0oh001@3o000>ooooo`L00?l00P000?l]003o00/0003o
00<0oh000000o`000?l00P000?lB003o00L0003o1000o`030000o`80o`0010000?l00`3oP0000?l0
003o000=003o00030000o`00o`000?l000@00?l00P000?l5003o00kooooo3@000?l00`3oP000003o
0000o`070000o`800?l01`000?l20?n000L0003o103o00020?n000030000o`3oP000oh000100003o
0P3oP0020000o`80oh0000<0003o0?n0003oP0000P3oP00K0000o`80oh001@000?l20?n00100003o
0P3oP0070000o`kooooo103oP0050?l00080oh001@3o00040?n00080o`000`3oP0050?l000P0oh00
00<0o`000000o`000?l01`3o00090?n00080o`001P3oP0000`3o0000oh000?n000050?l00080oh00
1`3o00020?n00080o`000P3oP0030000o`H0oh001@3o00060?n000030?l0003oP000oh000080o`00
0`3oP0040000o`@0oh001`000?l50?n00080o`006?ooool001Kooooo1@3oP0020?l00080oh006@3o
00020?n000<0o`001P3oP0090?l00080oh00403o00000`3oP000o`000?l0000H0?l000030?n0003o
0000o`0001/0o`00103oP0050?l000kooooo1`00o`020000obd00?l02`000?l00`3oP000003o0000
o`020000oa800?l01`000?l4003o00<0003o0P3o00040000o`030?n00000o`000?l000d00?l000<0
003o003o0000o`001000o`020000o`D00?l03_ooool=0000o`030?n000000?l0003o00L0003o0P00
o`070000o`80oh001`000?l40?l00080oh0000<0003o0?n0003oP00040000?l20?n00080003o0P3o
P0000`000?l0oh000?n000020?n001/0003o0P3oP0050000o`80oh0040000?l20?n000L0003o3_oo
ool40?n000D0o`000P3oP0050?l000@0oh000P3o00030?n000D0o`00203oP0000`3o0000003o0000
o`070?l000T0oh000P3o00060?n000030?l0003oP000oh0000D0o`000P3oP0070?l00080oh000P3o
00020?n000<0003o1P3oP0050?l000H0oh0000<0o`000?n0003oP0000P3o00030?n000@0003o103o
P0070000o`D0oh000P3o000Hooooo`005_ooool70?l00080oh000P3o00050?n000T0o`000P3oP00>
0?l00080oh001@3o00020?n001l0o`000`3oP0040?l00080oh001@3o00020?n00080o`0000<0oh00
0?l0003o0000503o00020?n000T0o`0000<0oh000?l0003o00000P3o000>ooooocH00?l03P000?l7
003o00@0003o0P00o`000`000?l00?l0003o0004003o00T0003o0P00o`000`000?l00?l0003o0002
0?n000@0003o00<0oh00003o0000o`000P00o`020000o`T00?l02@000?l00`00o`00003o0000o`02
0000o`kooooo1@000?l2003o01T0003o0P3oP0090000o`80o`000`000?l20?n000T0003o0P3oP009
0000o`80oh000`000?l20?n000h0003o1P00o`050000o`80oh0010000?l5003o0080003o00<00?l0
0000o`000?l050000?l>ooooo`80oh000P000?l00`3o0000oh000?n000020?l00080oh000`3o0004
0?n00080o`000P3oP0000`000?l0o`000?l000050?l00080oh000P3o00050?n00080o`00103oP003
0?l000H0oh000P3o00000`3oP000o`000?l000040?n00080003o1@3o00050?n00080o`001`3oP006
0?l000030?n000000?l0003o00H0o`000`3oP0020?l00080oh001`3o00030?n00080003o103oP007
0000o`@0oh0000<0o`000?n0003oP0000P3oP00Hooooo`005_ooool70?l00080oh000P3o00050?n0
00T0o`000P3oP00>0?l00080oh001@3o00020?n001l0o`000`3oP0040?l00080oh001@3o00020?n0
0080o`0000<0oh000?l0003o0000503o00020?n000T0o`0000<0oh000?l0003o00000P3o000>oooo
ocH00?l03P000?l7003o00@0003o0P00o`000`000?l00?l0003o0004003o00T0003o0P00o`000`00
0?l00?l0003o00020?n000@0003o00<0oh00003o0000o`000P00o`020000o`T00?l02@000?l00`00
o`00003o0000o`020000o`kooooo1@000?l2003o01T0003o0P3oP0090000o`80o`000`000?l20?n0
00T0003o0P3oP0090000o`80oh000`000?l20?n000h0003o1P00o`050000o`80oh0010000?l5003o
0080003o00<00?l00000o`000?l050000?l>ooooo`80oh000P000?l00`3o0000oh000?n000020?l0
0080oh000`3o00040?n00080o`000P3oP0000`000?l0o`000?l000050?l00080oh000P3o00050?n0
0080o`00103oP0030?l000H0oh000P3o00000`3oP000o`000?l000040?n00080003o1@3o00050?n0
0080o`001`3oP0060?l000030?n000000?l0003o00H0o`000`3oP0020?l00080oh001`3o00030?n0
0080003o103oP0070000o`@0oh0000<0o`000?n0003oP0000P3oP00Hooooo`005_ooool50?l000@0
oh001@3o00020?n000T0o`000P3oP00@0?l00080oh002`3o00000`3oP000o`000?l0000F0?l00003
0?n0003o0000o`0000d0o`000`3oP00F0?l00080oh001`3o00020?n000D0o`003_oooold003o0180
003o1`00o`020000o`T00?l040000?lR003o00kooooo1@000?l6003o0080003o0`00o`070000o`80
0?l03@000?l00`3oP000003o0000o`040?n00080003o00<0oh000000o`000?l050000?l20?n000D0
003o103o00000`3oP000003o0000o`020?n000d0003o00<0oh000000o`000?l00P3oP00K0000o`80
oh001@000?l>ooooo`80oh000P3o00030?n00080o`002@3oP0020?l000<0oh000P3o00040?n00003
0?l0003oP000oh0000@0oh000P3o00000`3oP000003o0000o`040?n000<0o`001P3oP0020?l000L0
oh000P3o000<0?n00080o`000P3oP0000`000?l0o`000?l000040?l000D0oh000P3o00020?n00080
o`001@3oP0040?l000030?n0003o0000o`000080o`001`3oP0020?l00080003o00<0oh000000o`00
0?l00P000?l40?l000030000o`3oP000oh000080oh006?ooool001Kooooo1@3o00040?n000D0o`00
103oP0050?l00080oh00B03o00020?n000T0o`0000<0oh000?l0003o00002`3o00020?n000L0o`00
0`3oP0040?l000kooooo7@00o`070000oa800?l040000?l;003o00030000o`00o`000?l000L0003o
0P3oP0060000o`h00?l00P000?l3003o0080003o4P00o`0>ooooo`D0003o2`00o`0;0000o`800?l0
3P000?l20?n000D0003o1000o`090000o`800?l01@000?l20?l000<0003o0P3oP0020000o`80oh00
9@000?l2003o0080003o0P00o`050000o`@0oh001@000?l>ooooo`@0o`0000<0oh000?l0003o0000
0P000?l70?n000T0o`000P3oP0020?l000<0oh000P3o00020?n00080003o2P3oP0020?l000L0oh00
0P3o00090?n00080o`000`3oP0020000o`80oh000P3o00020?n000D0o`003@3oP0000`3o0000oh00
0?n000020?l000`0oh000P3o00040?n00080003o1`3oP0030?l00080oh000P3o000Hooooo`005_oo
ool50?l000@0oh001@3o00040?n000D0o`000P3oP0180?l00080oh002@3o00000`3oP000o`000?l0
000;0?l00080oh001`3o00030?n000@0o`003_oooolM003o00L0003o4P00o`0@0000o`/00?l000<0
003o003o0000o`001`000?l20?n000H0003o3P00o`020000o`<00?l00P000?lB003o00kooooo1@00
0?l;003o00/0003o0P00o`0>0000o`80oh001@000?l4003o00T0003o0P00o`050000o`80o`000`00
0?l20?n00080003o0P3oP00U0000o`800?l00P000?l2003o00D0003o103oP0050000o`kooooo103o
00000`3oP000o`000?l000020000o`L0oh002@3o00020?n00080o`000`3oP0020?l00080oh000P00
0?l:0?n00080o`001`3oP0020?l000T0oh000P3o00030?n00080003o0P3oP0020?l00080oh001@3o
000=0?n000030?l0003oP000oh000080o`00303oP0020?l000@0oh000P000?l70?n000<0o`000P3o
P0020?l001Sooooo000Fooooo`@0oh002P3o00020?n000L0o`00103oP0070?l00080oh001`3o0002
0?n001h0o`000P3oP0070?l00080oh00903o00030?n000H0o`000P3oP0050?l000kooooo1000o`03
0000oa800?l00P000?l2003o00L0003o0P00o`000`000?l00?l0003o000=003o00L0003o0P00o`07
0000o`h00?l02`000?lG003o00d0003o1@00o`020000o`kooooo1@000?l2003o0080003o1`00o`0R
0000o`800?l010000?l3003o00@0003o0P00o`0<0000o`@0oh0000<0o`000?n0003oP0000P3oP005
0000o`800?l05P000?l00`00o`00003o0000o`0;0000o`@0oh000`000?l20?n000kooooo103o0007
0?n00080o`0000<0003o0?n0003oP0000P3oP0040?l000030?n0003o0000o`0000L0oh000P3o0004
0?n000<0o`00103oP0020000o`030?n0003o0000o`0000/0oh000P3o000:0?n000H0o`001@3oP002
0?l000@0oh0000<0o`000?n0003oP000103o000@0?n00080o`0000<0oh000?l0003o0000103oP002
0000o`D0oh00103o00030?n00080o`006?ooool001Kooooo103oP00:0?l00080oh001`3o00040?n0
00L0o`000P3oP0070?l00080oh007P3o00020?n000L0o`000P3oP00T0?l000<0oh001P3o00020?n0
00D0o`003_ooool4003o00<0003o4P00o`020000o`800?l01`000?l2003o00030000o`00o`000?l0
00d00?l01`000?l2003o00L0003o3P00o`0;0000oaL00?l03@000?l5003o0080003o3_ooool50000
o`800?l00P000?l7003o0280003o0P00o`040000o`<00?l010000?l2003o00`0003o103oP0000`3o
0000oh000?n000020?n000D0003o0P00o`0F0000o`03003o00000?l0003o00/0003o103oP0030000
o`80oh003_ooool40?l000L0oh000P3o00000`000?l0oh000?n000020?n000@0o`0000<0oh000?l0
003o00001`3oP0020?l000@0oh000`3o00040?n00080003o00<0oh000?l0003o00002`3oP0020?l0
00X0oh001P3o00050?n00080o`00103oP0000`3o0000oh000?n000040?l00100oh000P3o00000`3o
P000o`000?l000040?n00080003o1@3oP0040?l000<0oh000P3o000Hooooo`005_ooool20?n00080
003o00<0o`000000o`000?l0103oP0030?l00080oh005`3o00020?n000/0o`000P3oP0050?l00080
003o1`3o00020?n000`0o`00103oP00;0?l00080oh004@3o00020?n000L0o`000P3oP0040?l00080
oh001@3o000>oooooad00?l02P000?l=003o00T0003o0P00o`020000o`03003o00000?l0003o00T0
0?l04P000?l5003o0080003o4000o`0=0000o`800?l000<0003o003o0000o`000P00o`0>ooooo`T0
003o0P00o`0>0000o`@00?l04P000?l00`3oP000003o0000o`040000o`L00?l00P000?l2003o0003
0000o`00o`000?l000/0003o0`3oP0020000o`80oh001@000?l2003o0180003o0P3oP00I0000o`ko
oooo103o00000`3oP000o`000?l000020?n000D0o`000P3oP0060?l000030?n0003o0000o`000080
oh000P3o00000`3oP000o`000?l000040?n00080003o00<0o`000000o`000?l00P3o00020?n00080
o`001@3oP0020?l000<0oh000P3o00070?n000@0003o0P3oP0000`000?l0oh000?n000020?n00080
o`000`3oP0060000o`T0o`000P3oP0000`3o0000oh000?n000040?l00080oh0000<0o`000?n0003o
P0000P3oP0050?l000@0oh000P000?l30?n000H0o`000`3oP0020?l001Sooooo000Fooooo`80oh00
0P000?l00`3o0000003o0000o`040?n000<0o`000P3oP00G0?l00080oh002`3o00020?n000D0o`00
0P000?l70?l00080oh00303o00040?n000/0o`000P3oP00A0?l00080oh001`3o00020?n000@0o`00
0P3oP0050?l000kooooo7@00o`0:0000o`d00?l02@000?l2003o0080003o00<00?l00000o`000?l0
2@00o`0B0000o`D00?l00P000?l@003o00d0003o0P00o`000`000?l00?l0003o0002003o00kooooo
2@000?l2003o00h0003o1000o`0B0000o`030?n000000?l0003o00@0003o1`00o`020000o`800?l0
00<0003o003o0000o`002`000?l30?n00080003o0P3oP0050000o`800?l04P000?l20?n001T0003o
3_ooool40?l000030?n0003o0000o`000080oh001@3o00020?n000H0o`0000<0oh000?l0003o0000
0P3oP0020?l000030?n0003o0000o`0000@0oh000P000?l00`3o0000003o0000o`020?l00080oh00
0P3o00050?n00080o`000`3oP0020?l000L0oh0010000?l20?n000030000o`3oP000oh000080oh00
0P3o00030?n000H0003o2@3o00020?n000030?l0003oP000oh0000@0o`000P3oP0000`3o0000oh00
0?n000020?n000D0o`00103oP0020000o`<0oh001P3o00030?n00080o`006?ooool001Kooooo1`3o
00040?n000<0o`000P3oP00O0?l000030?n0003o0000o`0000T0o`0010000?lT0?l000030?n0003o
0000o`0000h0o`000P3oP0070?l00080oh001`000?l40?l000kooooo=000o`040000o`03003o0000
0?l0003o00T00?l00P000?l3003o01@0003o6`00o`050000o`@00?l00P000?l7003o00kooooo2@00
0?l2003o00D0003o0P3oP0090000o`800?l030000?l20?n000/0003o0P00o`030000o`@00?l00`00
0?l2003o00/0003o0P3oP0000`000?l0oh000?n000020000oa000?l03@000?l00`00o`00003o0000
o`02003o00@0003o00<00?l00000o`000?l00P00o`070000o`80oh003_ooool40?l000030?n0003o
0000o`0000H0o`000`3oP0060?l000030?n0003o0000o`0000D0oh00103o00020?n00080003o0`3o
P0020?l00080oh000P3o00000`3oP000o`000?l000020?n00080o`000`3oP0020?l000@0oh000P00
0?l00`3oP000o`000?l000020?l000<0oh000P3o00040?n00080o`000`3oP0020?l000T0oh000P3o
00020?n000<0o`000P3oP0020?l00080oh0000<0o`000?n0003oP0001@3oP0020?l000L0oh00103o
00020?n00080o`000`3oP0020?l001Sooooo000Fooooo`L0o`00103oP0030?l00080oh007`3o0000
0`3oP000o`000?l000090?l000@0003o903o00000`3oP000o`000?l0000>0?l00080oh001`3o0002
0?n000L0003o103o000>oooooc@00?l010000?l00`00o`00003o0000o`09003o0080003o0`00o`0D
0000oa/00?l01@000?l4003o0080003o1`00o`0>ooooo`T0003o0P00o`050000o`80oh002@000?l2
003o00`0003o0P3oP00;0000o`800?l00`000?l4003o00<0003o0P00o`0;0000o`80oh0000<0003o
0?n0003oP0000P000?l@003o00d0003o00<00?l00000o`000?l00P00o`040000o`03003o00000?l0
003o00800?l01`000?l20?n000kooooo103o00000`3oP000o`000?l000060?l000<0oh001P3o0000
0`3oP000o`000?l000050?n000@0o`000P3oP0020000o`<0oh000P3o00020?n00080o`0000<0oh00
0?l0003o00000P3oP0020?l000<0oh000P3o00040?n00080003o00<0oh000?l0003o00000P3o0003
0?n00080o`00103oP0020?l000<0oh000P3o00090?n00080o`000P3oP0030?l00080oh000P3o0002
0?n000030?l0003oP000oh0000D0oh000P3o00070?n000@0o`000P3oP0020?l000<0oh000P3o000H
ooooo`005_ooool50?l00080oh001P3o00030?n000L0o`000P3oP00P0?l000@0003o1`3o00020000
oa80o`000`3oP00?0?l00080oh005@3o00050000o`80oh000P3o000>oooooa800?l00P000?l5003o
00D0003o0P00o`020000o`h00?l00P000?l2003o00@0003o00<00?l00000o`000?l03P00o`090000
o`@00?l02@000?lI003o00d0003o1@00o`0>ooooo`D0003o2000o`070000o`X00?l010000?l2003o
01L0003o0P00o`0G0000o`80oh001@000?l2003o00@0003o0P00o`0:0000o`@0oh003P000?l4003o
00L0003o3_ooool40?n000030?l0003oP000oh0000H0o`001`3oP0020?l000030?n0003o0000o`00
00@0oh000`3o00020?n000D0003o103oP0020000o`<0oh001P3o00030?n00080o`001`3oP0020?l0
00@0oh002@3o00000`3oP000o`000?l000040?n000<0o`000P3oP0020?l000L0oh000P000?l40?n0
00030?l0003oP000oh000080oh001@3o000D0?n001Sooooo000Fooooo`D0o`000P3oP00?0?l000<0
oh00403o00020?n00100o`00103oP00@0?l00080oh006@3o00020?n000P0o`000P3oP0020?l00080
oh002@3o00020?n000D0o`003_oooolM003o00030000o`00o`000?l000800?l00P000?lG003o00/0
003o1@00o`060000o`030?n00000o`000?l000800?l010000?l00`3oP000003o0000o`0M003o00<0
003o3@00o`0>ooooo`T0003o1000o`0:0000o`H00?l01`000?l2003o00T0003o00<0oh000000o`00
0?l010000?l9003o02@0003o00<00?l00000o`000?l040000?l2003o00/0003o0P3oP0050000o`ko
oooo0P3oP0020000o`T0o`001@3oP0020000o`D0o`00103oP0030?l00080oh001@000?l40?n00080
003o3P3oP0020?l000@0oh000`3o00040?n000<0o`001`3oP0040?l00080oh001`3o00040?n00003
0?l0003oP000oh000080003o0P3o00070?n000D0o`000P3oP0020?l000D0oh000P000?l40?n00003
0?l0003oP000oh000080oh006?ooool001Kooooo1@3o00020?n000l0o`000`3oP00@0?l00080oh00
403o00040?n00100o`000P3oP00I0?l00080oh00203o00020?n00080o`000P3oP0090?l00080oh00
1@3o000>oooooad00?l000<0003o003o0000o`000P00o`020000oaL00?l02`000?l5003o00H0003o
00<0oh00003o0000o`000P00o`040000o`030?n000000?l0003o01d00?l00`000?l=003o00kooooo
2@000?l4003o00X0003o1P00o`070000o`800?l02@000?l00`3oP000003o0000o`040000o`T00?l0
90000?l00`00o`00003o0000o`0@0000o`800?l02`000?l20?n000D0003o3_ooool20?n00080003o
2@3o00050?n00080003o1@3o00040?n000<0o`000P3oP0050000o`@0oh000P000?l>0?n00080o`00
103oP0030?l000@0oh000`3o00070?n000@0o`000P3oP0070?l000@0oh0000<0o`000?n0003oP000
0P000?l20?l000L0oh001@3o00020?n00080o`001@3oP0020000o`@0oh0000<0o`000?n0003oP000
0P3oP00Hooooo`005_ooool20?l00080oh00703o00020?n000L0o`000P3oP00>0?l000@0oh001@3o
00060?n000L0o`000P3oP00>0?l00080oh00303o00020000o`80o`000P3oP0030?l00080oh002@3o
00020?n00080003o00<0oh000?l0003o00000P3o000>oooooa/00?l01`000?lD003o0080003o1@00
o`040000o`03003o00000?l0003o00800?l01@000?l;003o00L0003o0P00o`020000oaL00?l01@00
0?l2003o00@0003o0P00o`000`000?l00?l0003o0002003o00kooooo2@000?l7003o00/0003o0P00
o`090000o`03003o00000?l0003o00/0003o1`00o`0I0000o`80oh0010000?l00`00o`00003o0000
o`0]0000o`kooooo0P3o00030000o`H0oh000P3o00070?n000D0o`001@3oP0020?l000H0oh0000<0
003o0?n0003oP0000P3oP0020?l00080oh001@3o00040?n000D0o`000P3oP0090?l000030?n0003o
0000o`0000@0oh000P3o00030?n00080o`000P3oP0070?l000D0oh000P3o00020?n00080o`001@3o
P0020?l000<0oh000P3o00060?n000030?l0003oP000oh0000@0oh000P3o00000`3oP000003o0000
o`020000oaSooooo000Fooooo`80o`000P3oP00L0?l00080oh001`3o00020?n000h0o`00103oP005
0?l000H0oh001`3o00020?n000h0o`000P3oP00<0?l00080003o0P3o00020?n000<0o`000P3oP009
0?l00080oh000P000?l00`3oP000o`000?l000020?l000kooooo6`00o`070000oa@00?l00P000?l5
003o00@0003o00<00?l00000o`000?l00P00o`050000o`/00?l01`000?l2003o0080003o5`00o`05
0000o`800?l010000?l2003o00030000o`00o`000?l000800?l03_ooool90000o`L00?l02`000?l2
003o00T0003o00<00?l00000o`000?l02`000?l7003o01T0003o0P3oP0040000o`03003o00000?l0
003o02d0003o3_ooool20?l000<0003o1P3oP0020?l000L0oh001@3o00050?n00080o`001P3oP000
0`000?l0oh000?n000020?n00080o`000P3oP0050?l000@0oh001@3o00020?n000T0o`0000<0oh00
0?l0003o0000103oP0020?l000<0oh000P3o00020?n000L0o`001@3oP0020?l00080oh000P3o0005
0?n00080o`000`3oP0020?l000H0oh0000<0o`000?n0003oP000103oP0020?l000030?n000000?l0
003o0080003o6?ooool001Kooooo0P3oP00;0?l000030?n0003o0000o`0001T0o`000P3oP0050?l0
0080oh007@3o00030?n002@0o`000P3oP0020000o`L0o`000P3oP0020000o`D0o`003_ooool2003o
00L0003o5`00o`060000oa800?l00`000?l2003o00`0003o3`00o`0B0000oa<00?l02`000?l2003o
00kooooo0P00o`020000o``00?l00P000?l2003o01T0003o0`3oP0060000o`800?l000<0003o003o
0000o`003P000?l2003o01T0003o0P3oP0040000o`80oh007P000?l>ooooo`@0oh0000<0003o0?n0
003oP0000P3oP0050?l000H0oh000P3o00080?n00080o`00103oP0020000o`030?l0003oP000oh00
0080oh000P3o00050?n00080o`00103oP0050?l000D0oh002`3o00040?n000D0003o0P3oP0020?l0
00<0oh000P3o00020?n00080o`0000<0oh000?l0003o00001P3oP0000`3o0000oh000?n000020?l0
0080oh00403o00030?n00080o`000P000?lHooooo`005_ooool20?n000/0o`0000<0oh000?l0003o
00006@3o00020?n000D0o`000P3oP00M0?l000<0oh00903o00020?n00080003o1`3o00020?n00080
003o1@3o000>ooooo`800?l01`000?lG003o00H0003o4P00o`030000o`800?l030000?l?003o0180
003o4`00o`0;0000o`800?l03_ooool2003o0080003o3000o`020000o`800?l06@000?l30?n000H0
003o0P00o`000`000?l00?l0003o000>0000o`800?l06@000?l20?n000@0003o0P3oP00N0000o`ko
oooo103oP0000`000?l0oh000?n000020?n000D0o`001P3oP0020?l000P0oh000P3o00040?n00080
003o00<0o`000?n0003oP0000P3oP0020?l000D0oh000P3o00040?n000D0o`001@3oP00;0?l000@0
oh001@000?l20?n00080o`000`3oP0020?l00080oh000P3o00000`3oP000o`000?l000060?n00003
0?l0003oP000oh000080o`000P3oP00@0?l000<0oh000P3o00020000oaSooooo000Fooooo`80oh00
2@3o00030?n000P0o`0000<0oh000?l0003o00003P3o00020?n000H0o`000`3oP00B0?l000@0oh00
1`3o00030?n00080o`00103oP0070?l00080oh001@3o00020?n000`0o`00103oP0090?l00080oh00
0P000?l50?l000kooooo0P00o`020000obh00?l07@000?l9003o00T0003o0`00o`020000o`800?l0
0P000?l:003o0080003o0P00o`020000o`800?l000<0oh000000o`000?l010000?l2003o00030000
o`00o`000?l00080003o3_ooool4003o00X0003o0P00o`0=0000o`X00?l01P000?l20?n000030000
o`00o`000?l000L0003o0P00o`060000o`<00?l01`000?l2003o03T0003o1000o`0>ooooo`@0oh00
00<0003o0?l0003o00002@3oP0070?l00080oh000P000?l50?l00080oh001@000?l80?l000030000
o`3oP000oh000080003o1@3o00060?n00080003o00<0oh000?l0003o00001`3o00090?n00080o`00
0P3oP0030?l00080oh000P3o000;0?n00080003o0`3oP0020000o`<0oh00203o00000`3oP000o`00
0?l000060?l000030?n000000?l0003o0080oh006?ooool001Kooooo0P3oP0090?l000<0oh00203o
00000`3oP000o`000?l0000>0?l00080oh001P3o00030?n00180o`00103oP0070?l000<0oh000P3o
00040?n000L0o`000P3oP0050?l00080oh00303o00040?n000T0o`000P3oP0020000o`D0o`003_oo
ool2003o0080003o;P00o`0M0000o`T00?l02@000?l3003o0080003o0P00o`020000o`X00?l00P00
0?l2003o0080003o0P00o`000`3oP000003o0000o`040000o`800?l000<0003o003o0000o`000P00
0?l>ooooo`@00?l02P000?l2003o00d0003o2P00o`060000o`80oh0000<0003o003o0000o`001`00
0?l2003o00H0003o0`00o`070000o`800?l0>@000?l4003o00kooooo103oP0000`000?l0o`000?l0
00090?n000L0o`000P3oP0020000o`D0o`000P3oP0050000o`P0o`0000<0003o0?n0003oP0000P00
0?l50?l000H0oh000P000?l00`3oP000o`000?l000070?l000T0oh000P3o00020?n000<0o`000P3o
P0020?l000/0oh000P000?l30?n00080003o0`3oP0080?l000030?n0003o0000o`0000H0o`0000<0
oh000000o`000?l00P3oP00Hooooo`005_ooool;0?l00080oh00303o00020?n000`0o`000P3oP006
0?l000030?n0003o0000o`0000/0o`000P3oP00>0?l00080oh001`3o00020?n000T0o`000P3oP003
0?l00080oh00303o00040?n000D0o`0010000?l20?n00080003o00<0o`000?n0003oP0000P3o000>
ooooo`800?l01`000?lG003o0080003o5`00o`0D0000o`T00?l02`000?lG003o0100003o0P00o`0>
ooooo`D00?l010000?l;003o00L0003o2@00o`090000o`80oh0000<0003o003o0000o`000P00o`02
0000o`800?l03P000?l2003o00030000o`00o`000?l000@00?l06@000?l7003o00T0003o0P00o`0@
0000o`kooooo0P3oP0030?l000@0oh000P000?l50?n00080o`000P3oP0030?l00080oh000P000?l5
0?l00080oh000P000?l30?n000P0o`0000<0oh000000o`000?l00P3oP0020?l00080oh0000<0o`00
0?n0003oP000103oP0020?l000<0oh001`3o00040?n000<0o`000P3oP0040?l00080oh000`3o0004
0?n000<0o`001P3oP0020?l000<0oh000P3o00030?n000H0o`000P3oP0050?l000D0oh000P000?l2
0?n001Sooooo000Foooooa@0o`000P3oP0030?l000@0oh002P3o00020?n002@0o`000P3oP00D0?l0
00<0oh005`3o00020?n00080003o103oP0000`000?l0oh000?n000020000o`kooooo10000?lb003o
00/0003o00<00?l00000o`000?l02`000?l9003o00T0003o5@00o`0;0000o`80oh000`00o`020000
o`800?l03_ooool>0000o`P00?l020000?l4003o0080003o1@00o`0F0000o`800?l000<0003o003o
0000o`002@000?l2003o03/0003o3_ooool40?l000D0oh000P3o00020?n000L0o`000P3oP0000`3o
0000oh000?n000020?n00080o`002@3oP0000`3o0000oh000?n000020?n00080o`001@3oP0020?l0
00/0oh000P3o00000`3oP000003o0000o`0;0?n000<0o`00103oP0050?l000H0oh000P3o00000`00
0?l0oh000?n000040?l00080003o1@3o00070?n00080o`000P3oP0050?l000D0oh000P000?l20?n0
01Sooooo000Foooooa@0o`000P3oP0030?l000@0oh002P3o00020?n002@0o`000P3oP00D0?l000<0
oh005`3o00020?n00080003o103oP0000`000?l0oh000?n000020000o`kooooo10000?lb003o00/0
003o00<00?l00000o`000?l02`000?l9003o00T0003o5@00o`0;0000o`80oh000`00o`020000o`80
0?l03_ooool>0000o`P00?l020000?l4003o0080003o1@00o`0F0000o`800?l000<0003o003o0000
o`002@000?l2003o03/0003o3_ooool40?l000D0oh000P3o00020?n000L0o`000P3oP0000`3o0000
oh000?n000020?n00080o`002@3oP0000`3o0000oh000?n000020?n00080o`001@3oP0020?l000/0
oh000P3o00000`3oP000003o0000o`0;0?n000<0o`00103oP0050?l000H0oh000P3o00000`000?l0
oh000?n000040?l00080003o1@3o00070?n00080o`000P3oP0050?l000D0oh000P000?l20?n001So
oooo000Fooooo`@0o`0000<0oh000?l0003o00003@3o00020?n000P0o`000P000?l70?l00080oh00
1P3o00000`3oP000o`000?l0000?0?l000L0oh001@3o00040?n000D0o`000P3oP00W0?l000@0oh00
1`3o000>ooooo`L0003o=000o`0D0000o`L00?l02`000?lE003o00H0003o00<0oh000000o`000?l0
1P000?l00`00o`00003o0000o`020000o`kooooo1@000?l2003o00L0003o2@00o`090000o`800?l0
0P000?l3003o02d0003o0P3oP00d0000o`kooooo0P3o00070?n00080003o0P3oP00:0?l000H0oh00
00<0003o0?n0003oP0000P3oP0020000o`80oh003P3o00050?n00080003o1P3o00030?n000D0o`00
0P3oP0020?l000L0oh000P3o00020?n000<0o`001P3oP0020?l000030?n0003o0000o`0000H0o`00
0`3oP0020?l000D0oh00103o00020?n000D0o`001@3oP0020?l00080oh006?ooool001Kooooo103o
00000`3oP000o`000?l0000=0?l00080oh00203o00020000o`L0o`000P3oP0060?l000030?n0003o
0000o`0000l0o`001`3oP0050?l000@0oh001@3o00020?n002L0o`00103oP0070?l000kooooo1`00
0?ld003o01@0003o1`00o`0;0000oaD00?l01P000?l00`3oP000003o0000o`060000o`03003o0000
0?l0003o0080003o3_ooool50000o`800?l01`000?l9003o00T0003o0P00o`020000o`<00?l0;@00
0?l20?n003@0003o3_ooool20?l000L0oh000P000?l20?n000X0o`001P3oP0000`000?l0oh000?n0
00020?n00080003o0P3oP00>0?l000D0oh000P000?l60?l000<0oh001@3o00020?n00080o`001`3o
P0020?l00080oh000`3o00060?n00080o`0000<0oh000?l0003o00001P3o00030?n00080o`001@3o
P0040?l00080oh001@3o00050?n00080o`000P3oP00Hooooo`005_ooool20?l000<0oh006@3o0002
0?n000@0o`000P3oP0000`3o0000oh000?n000020?l00080oh002`3o00000`000?l0o`000?l00002
0?n000@0o`001@3oP0020000o`D0o`000P3oP0020000o`D0o`000P3oP00L0?l00080oh002@3o0002
0?n000T0o`003_ooool90000oc@00?l040000?l7003o00/0003o0P3oP0020000o`<00?l00P000?l@
003o00/0003o0`00o`020000o`800?l03_ooool50000o`800?l01`000?l=003o00<0003o2@00o`15
0000o`03003o00000?l0003o00/0003o0P3oP00>0000o`kooooo0P3o00070?n00080o`001@3oP002
0?l00080oh000`3o00020?n00080003o2@3oP0020000o`030?l0003oP000oh0000@0o`000P3oP003
0?l000H0oh001`3o00070?n00080o`002@3oP0030?l000H0oh0000<0o`000?n0003oP0001`3o000;
0?n00080o`000`3oP0080?l000T0oh0000<0003o0?n0003oP0000P3oP00Hooooo`005_ooool20?l0
00<0oh006@3o00020?n000@0o`000P3oP0000`3o0000oh000?n000020?l00080oh002`3o00000`00
0?l0o`000?l000020?n000@0o`001@3oP0020000o`D0o`000P3oP0020000o`D0o`000P3oP00L0?l0
0080oh002@3o00020?n000T0o`003_ooool90000oc@00?l040000?l7003o00/0003o0P3oP0020000
o`<00?l00P000?l@003o00/0003o0`00o`020000o`800?l03_ooool50000o`800?l01`000?l=003o
00<0003o2@00o`150000o`03003o00000?l0003o00/0003o0P3oP00>0000o`kooooo0P3o00070?n0
0080o`001@3oP0020?l00080oh000`3o00020?n00080003o2@3oP0020000o`030?l0003oP000oh00
00@0o`000P3oP0030?l000H0oh001`3o00070?n00080o`002@3oP0030?l000H0oh0000<0o`000?n0
003oP0001`3o000;0?n00080o`000`3oP0080?l000T0oh0000<0003o0?n0003oP0000P3oP00Hoooo
o`005_ooool20000o`L0o`00103oP00/0?l00080oh00403o00040?n000L0o`000P3oP00G0?l000L0
oh0010000?l50?l000@0oh001@3o00020?n000kooooo2@000?ld003o00/0003o1@00o`040000o`<0
0?l03`000?l00`00o`00003o0000o`020000oa800?l00P000?l00`00o`00003o0000o`060000o`D0
0?l03_ooool50000o`800?l01`000?l2003o00@0003o0P00o`030000o`800?l01`000?l4003o0100
003o0P3oP0030000o`80oh003P000?l20?n00100003o0P00o`0=0000o`03003o00000?l0003o01/0
003o3_ooool40?n000030000o`3oP000oh000080oh000P000?l20?l000<0oh000P3o00040?n00003
0?l0003oP000oh000080oh000P000?l00`3oP000003o0000o`020000o`@0oh0000<0o`000?n0003o
P0000P3oP0040?l000030?n0003o0000o`0000H0oh0000<0o`000?n0003oP0001`3oP0070?l000@0
oh001@3o00020?n000@0o`001`3oP0050?l000P0oh000`3o00070000o`@0oh001@3o00040?n00080
o`001@3oP00Hooooo`005_ooool@0?l00080oh004P3o00030?n000H0o`000`3oP0040?l000@0oh00
00<0o`000?n0003oP0006`3o00020?n00080o`0000<0oh000?l0003o0000503o00020000o`D0oh00
0P3o00020?n000<0o`000P3oP0090?l000kooooo0P00o`050000oc@00?l01`000?l20?n00080003o
0P3oP0090000o`03003o00000?l0003o00H0003o00<0oh000000o`000?l01P000?lE003o00L0003o
1000o`020000o`D00?l03_oooolI0000o`800?l02@000?l5003o01H0003o0P00o`0>0000o`800?l0
00<0003o003o0000o`001`00o`0?0000o`D00?l010000?l00`00o`00003o0000o`02003o0080003o
1@00o`090000o`800?l03_ooool40000o`030?n000000?l0003o0080oh000P000?l20?n000D0o`00
1@3oP0070000o`80oh000P000?l50?l000T0oh000P3o00040?n00080003o0`3oP0020?l000D0oh00
1P3o00030?n000L0o`000P3oP0020000o`D0oh000P3o00040?n000<0o`002@3oP0040000o`80oh00
00<0003o0?n0003oP0001`3oP0020000o`/0oh006?ooool001Kooooo403o00020?n00180o`000`3o
P0060?l000<0oh00103o00040?n000030?l0003oP000oh0001/0o`000P3oP0020?l000030?n0003o
0000o`0001@0o`000P000?l50?n00080o`000P3oP0030?l00080oh002@3o000>ooooo`800?l01@00
0?ld003o00L0003o0P3oP0020000o`80oh002@000?l00`00o`00003o0000o`060000o`030?n00000
0?l0003o00H0003o5@00o`070000o`@00?l00P000?l5003o00kooooo6@000?l2003o00T0003o1@00
o`0F0000o`800?l03P000?l2003o00030000o`00o`000?l000L00?l03`000?l5003o00@0003o00<0
0?l00000o`000?l00P00o`020000o`D00?l02@000?l2003o00kooooo10000?l00`3oP000003o0000
o`020?n00080003o0P3oP0050?l000D0oh001`000?l20?n00080003o1@3o00090?n00080o`00103o
P0020000o`<0oh000P3o00050?n000H0o`000`3oP0070?l00080oh000P000?l50?n00080o`00103o
P0030?l000T0oh0010000?l20?n000030000o`3oP000oh0000L0oh000P000?l;0?n001Sooooo000F
oooooc@0o`0010000?l30?n000L0o`000P3oP00@0?l00080oh001@3o00040?n001@0o`0000<0oh00
0?l0003o00001`3o00020000o`80o`000P000?l70?l000kooooo0P00o`020000o`T00?l000<0003o
003o0000o`00:`00o`070000o`80oh0010000?l00`00o`00003o0000o`060000o`<00?l02`000?l7
003o00@0003o1@00o`040000o`L00?l00P000?l>003o00kooooo0P3oP00U0000o`800?l01`000?l2
0?n00200003o1000o`020000o`<00?l00P000?l2003o00d0003o00<00?l00000o`000?l01`000?l7
003o00d0003o3_ooool20000o`<0oh000P000?l20?l00080003o0`3oP0040?l00080oh000P3o0000
0`3oP000o`000?l000050?l000@0oh001`3o000;0?n00080o`000P3oP0050?l00080oh001@3o0007
0?n000H0o`000`3oP0020?l000D0oh000P3o00060?n000030?l0003oP000oh0000@0003o0`3oP002
0000o`@0oh000`3o00020?n00080o`000P3oP0030000o`80oh000P3o00050?n00080o`006?ooool0
01Kooooo=03o00040000o`<0oh001`3o00020?n00100o`000P3oP0050?l000@0oh00503o00000`3o
P000o`000?l000070?l00080003o0P3o00020000o`L0o`003_ooool2003o0080003o2@00o`000`00
0?l00?l0003o000[003o00L0003o0P3oP0040000o`03003o00000?l0003o00H0003o0`00o`0;0000
o`L00?l010000?l5003o00@0003o1`00o`020000o`h00?l03_ooool20?n002D0003o0P00o`070000
o`80oh0080000?l4003o0080003o0`00o`020000o`800?l03@000?l00`00o`00003o0000o`070000
o`L00?l03@000?l>ooooo`80003o0`3oP0020000o`80o`000P000?l30?n000@0o`000P3oP0020?l0
00030?n0003o0000o`0000D0o`00103oP0070?l000/0oh000P3o00020?n000D0o`000P3oP0050?l0
00L0oh001P3o00030?n00080o`001@3oP0020?l000H0oh0000<0o`000?n0003oP00010000?l30?n0
0080003o103oP0030?l00080oh000P3o00020?n000<0003o0P3oP0020?l000D0oh000P3o000Hoooo
o`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooo
oooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo
003oooooooooooooP_ooool002;oooooL_l0000Wooooog7o00009ooooomao`0002OoooooLOl0000T
ooooo`008_ooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`0Mooooo`03o`3o0?oooooooooo01?ooooo00?o0?l0oooo
ooooool0=oooool00ol0003oooooooooo`0Rooooo`008_ooool00ol0003oooooooooo`03oooooaKo
o`001OoooolFo`3o00Cooooo5ol00?l4oooooaOoogl01?ooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo00;ooooo5_oo0005oooooaKo0?l01OoooolFo`00o`Cooooo5oooO`04ooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolGool000Cooooo5_l0o`05oooooaKo
003o1OoooolFoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02oooooaOo
o`001?oooolFo`3o00Gooooo5_l00?l5oooooaKoogl01?ooool00ol0003oooooooooo`0Rooooo`00
8_ooool00ol0003oooooooooo`03oooooaKoo`007ooooolGo`00o`Cooooo5oooO`04ooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool00_oooolFool000Gooooo5_l0o`05oooooaKo003o
7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo0004oooooaKo0?l0
1OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02oooooaOoo`00
>_oooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooo
oolFool001oooooo5ol00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oo
oolFool000Gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo00;ooooo5ooo0004oooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`02oooooaOoo`00>_oooolFoomo00Cooooo00?o0000ooooooooool08_oo
ool002;ooooo00?o0000ooooooooool00ooooolFool001oooooo5ol00?lOooooo`03o`000?oooooo
oooo02Gooooo00?o0000ooooooooool00_oooolFool000Gooooo5_l0o`05oooooaKo003o7oooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo0004oooooaKo0?l01OoooolF
o`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02oooooaOoo`00>_oooolF
oomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool0
01oooooo5ol00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolFool0
00Gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
00;ooooo5ooo0004oooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`02oooooaOoo`00>_oooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;o
oooo00?o0000ooooooooool00ooooolFool001oooooo5ol00?lOooooo`03o`000?oooooooooo02Go
oooo00?o0000ooooooooool00_oooolFool000Gooooo5_l0o`05oooooaKo003o7oooool00ol0003o
ooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo0004oooooaKo0?l01OoooolFo`00oaoo
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02oooooaOoo`00>_oooolFoomo00Co
oooo00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool001oooooo
5ol00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolFool000Gooooo
5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo
5ooo0004oooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`1CoooooaKoogl01?ooool00ol0003oooooooooo`0Rooooo`008_ooool00ol0003ooooooooo
o`03oooooaKoo`007ooooolGo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`02oooooaOoo`001?oooolFo`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000
ooooooooool00ooooolFool001oooooo5ol00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool00_oooolGool000Cooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo05?ooooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo000Roooo
o`03o`000?oooooooooo00?ooooo5_oo000OoooooaOo003o7oooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo05?ooooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo000Rooooo`03
o`000?oooooooooo00?ooooo5_oo000OoooooaOo003o7oooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo05?ooooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo000Rooooo`03o`00
0?oooooooooo00?ooooo5_oo000OoooooaOo003o7oooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo05?ooooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo000Rooooo`03o`000?oo
oooooooo00?ooooo5_oo000OoooooaOo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo05?ooooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo000Rooooo`03o`000?l007oo
001o00?o001o5_oo000Oo`00OaOo003o7ol007l00ol0003oooooooooo`0Uooooo`03o`000?l007oo
001o01go001o5_l0o`05o`00OaKo003o7ol007l00ol0003oooooooooo`0Uooooo`03o`000?l007oo
001o01go001o5_l0o`05o`00OaKo003o7ol007l00ol0003oooooooooo`0Uooooo`03o`000?l007oo
001o05?o001o5_ooO`04o`00O`03o`000?oooooooooo02;ooooo000Rooooo`03o`000?oooooooooo
00?ooooo5_oo000OoooooaOo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
05?ooooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo000Rooooo`03o`000?oooooooooo00?o
oooo5_oo000OoooooaOo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01go
oooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01go
oooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?o
oooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo000Rooooo`03o`000?oooooooooo00?ooooo
5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo
5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo
5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Cooooo
00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o
0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o
0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o
0000ooooooooool09Oooool00ol0003oooooooooo`1CoooooaKoogl01?ooool00ol0003ooooooooo
o`0Rooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo05?ooooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo000Roooo
o`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000oooooooo
ool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo
0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo
0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1CoooooaKo
ogl01?ooool00ol0003oooooooooo`0Rooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00
EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o
7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool0
8_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo05?ooooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo000Rooooo`03o`000?oooooo
oooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolF
o`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0>?oooolF
o`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1CoooooaKoogl01?ooool0
0ol0003oooooooooo`0Rooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;o
oooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?o
oooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo000Rooooo`03o`000?l007oo001o00?o001o
5_oo001Eo`00O`03o`000?oooooooooo02Gooooo00?o0000o`00Ool007l07Ol007lFo`3o00Go001o
5_l00?lOo`00O`03o`000?oooooooooo02Gooooo00?o0000o`00Ool007l0>?l007lFo`00oaoo001o
00?o0000ooooooooool09Oooool00ol0003o001oo`00O`1Co`00OaKoogl01?l007l00ol0003ooooo
ooooo`0Rooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000
ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0M
oooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0h
oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`04
ooooo`03o`000?oooooooooo02;ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eoooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lOoooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0>?oooolFo`00oaoooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`1CoooooaKoogl01?ooool00ol0003oooooooooo`0Roooo
o`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo03Sooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000oooooooo
ool0>?oooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1CoooooaKo
ogl01?ooool00ol0003oooooooooo`0Rooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00
EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0>?oooolFo`00oaoooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`1CoooooaKoogl01?ooool00ol0003oooooooooo`0Rooooo`008_oo
ool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0hoooooaKo003o7ooo
ool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`04ooooo`03o`000?oo
oooooooo02;ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooo
oooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo03Sooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolF
oomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool0
05Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0>?oooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`1CoooooaKoogl01?ooool00ol0003oooooooooo`0Rooooo`008_ooool00ol0003o
ooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06go
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003o
ooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`04ooooo`03o`000?oooooooooo02;o
oooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo03Sooooo
5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Cooooo
00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o
0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool0>?oooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`1CoooooaKoogl01?ooool00ol0003oooooooooo`0Rooooo`008_ooool00ol0003oooooooooo`03
oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo05?ooooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo000Roooo
o`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`1CoooooaKoogl01?ooool00ol0003oooooooooo`0Roooo
o`008_ooool00ol0003o001oo`00O`03o`00OaKoo`00EOl007l00ol0003oooooooooo`0Uooooo`03
o`000?l007oo001o06go001o00?o0000ooooooooool09Oooool00ol0003o001oo`00O`1]o`00O`03
o`000?oooooooooo02Gooooo00?o0000o`00Ool007l0Dol007lFoomo00Co001o00?o0000oooooooo
ool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`04ooooo`03o`00
0?oooooooooo02;ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oo
oooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1CoooooaKoogl01?oo
ool00ol0003oooooooooo`0Rooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolF
oomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool0
05Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
05?ooooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo000Rooooo`03o`000?oooooooooo00?o
oooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003o
ooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`1CoooooaKoogl01?ooool00ol0003oooooooooo`0Rooooo`008_ooool00ol0003ooooo
ooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o
0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo000R
ooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`1CoooooaKoogl01?ooool00ol0003oooooooooo`0R
ooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]oooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000oooo
ooooool08_ooool002;ooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000oooooooo
ool08_ooool002;ooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool0
8_ooool002;ooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oo
oooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool08_oo
ool002;ooooo00?o0000o`00Ool007l0K_l007l00ol0003oooooooooo`0Uooooo`03o`000?l007oo
001o06go001o00?o0000ooooooooool09Oooool00ol0003o001oo`00O`1]o`00O`03o`000?oooooo
oooo02Gooooo00?o0000o`00Ool007l0Dol007lFoomo00Co001o00?o0000ooooooooool08_ooool0
02;ooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;o
oooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06go
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Go
oooo00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo
00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o
0000ooooooooool0K_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o
0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000
ooooooooool0K_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000oooo
ooooool0K_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000oooooooo
ool0K_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000oooooooo
ool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool0
K_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
KOooool00ol0003oooooooooo`0Rooooo`008_ooool00ol0003oooooooooo`1^ooooo`03o`000?oo
oooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oo
oooooooo02;ooooo000Rooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool08_ooool0
02;ooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Rooooo`008_ooool00ol0003o
ooooooooo`1^ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003o
ooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`1]ooooo`03o`000?oooooooooo02;ooooo000Rooooo`03o`000?oooooooooo06kooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo
00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool0K_ooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003ooooooooo
o`0Rooooo`008_oooombo`0002OoooooLOl0000Wooooog7o00009ooooomao`0002Cooooo003ooooo
ooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003oooooooooooooP_oo
ool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooo
oooooon2ooooo`00oooooooooooooh;ooooo000Fooooo`D0003o0P3o00020000o`/0o`000`000?l?
0?l000030?n000000?l0003o00L0o`000P000?lM0?l00080003o00<0o`000?n0003oP0000P3o0002
0?n00080003o1`3o00020?n000030?l000000?l0003o00/0o`000P3oP0050?l000@0oh0000<0o`00
0?n0003oP0002@3o00020?n000kooooo1@000?l40?l000L0003o4000o`020000o`@0o`0000<0003o
003o0000o`002@00o`040000o`<0o`000P000?l40?l000D0003o0P3o000>0000o`@00?l00P000?l0
0`00o`00003o0000o`060000o`030?l000000?l0003o00L0003o0P00o`0=0000o`<0o`001`000?l2
0?l00080003o3_ooool50000o`800?l00P000?l9003o0080003o0`00o`020000o`L00?l00P000?l7
003o0080003o1`00o`020000o`L00?l02`000?l30?l00080003o103o00030000o`800?l00P000?l4
003o00030000o`00o`000?l00080o`000P000?l20?l000030000o`3o0000o`000080o`001`000?l4
003o00030000o`00o`000?l000T0003o0P3o00020000o`800?l000<0o`000000o`000?l00P000?l>
ooooo`h0oh000P3o00070?n00080o`003P3oP0020?l000/0oh000P3o00020?n000<0o`000P3oP002
0?l000L0oh000P3o000Y0?n00080003o0`3o00060?n000030?l0003oP000oh0000/0oh006?ooool0
01Kooooo1@000?l20?l00080003o2`3o00030000o`l0o`0000<0oh000000o`000?l01`3o00020000
oad0o`000P000?l00`3o0000oh000?n000020?l00080oh000P000?l70?l00080oh0000<0o`000000
o`000?l02`3o00020?n000D0o`00103oP0000`3o0000oh000?n000090?l00080oh003_ooool50000
o`@0o`001`000?l@003o0080003o103o00000`000?l00?l0003o0009003o00@0003o0`3o00020000
o`@0o`001@000?l20?l000h0003o1000o`020000o`03003o00000?l0003o00H0003o00<0o`000000
o`000?l01`000?l2003o00d0003o0`3o00070000o`80o`000P000?l>ooooo`D0003o0P00o`020000
o`T00?l00P000?l3003o0080003o1`00o`020000o`L00?l00P000?l7003o0080003o1`00o`0;0000
o`<0o`000P000?l40?l000<0003o0P00o`020000o`@00?l000<0003o003o0000o`000P3o00020000
o`80o`0000<0003o0?l0003o00000P3o00070000o`@00?l000<0003o003o0000o`002@000?l20?l0
0080003o0P00o`000`3o0000003o0000o`020000o`kooooo3P3oP0020?l000L0oh000P3o000>0?n0
0080o`002`3oP0020?l00080oh000`3o00020?n00080o`001`3oP0020?l002T0oh000P000?l30?l0
00H0oh0000<0o`000?n0003oP0002`3oP00Hooooo`005_ooool40?l000030000o`3o0000o`000080
o`0010000?l30?l00080003o0P3o00020000o`L0o`0000<0003o0?l0003o00000P000?l40?l00003
0?n000000?l0003o0080003o2`3o00030?n001P0o`0000<0oh000?l0003o0000403o00020000o`/0
o`000`000?l60?l000D0oh001P3o00000`3oP000o`000?l000020?l000kooooo10000?l00`3o0000
003o0000o`020?l000D0003o2000o`0:0000o`@0o`006`000?l20?l000<0003o1@3o000;0000o`80
0?l010000?l00`00o`00o`000?l000090000o`80o`0050000?l00`3o0000003o0000o`020000o`80
o`001`000?l>ooooo`D0003o0P3o000=003o00L0003o1`00o`020?l000<00?l010000?l20?l00080
0?l01@000?l4003o00030000o`3o0000o`000080003o103o00000`000?l0o`000?l000040000o`<0
o`001P000?l3003o00@0003o0P00o`0[0000o`<00?l00P000?l>oooooaL0oh000P3o00090?n00080
o`000P3oP0030?l001/0oh000P000?l30?l000/0oh000P3o000D0?n00080o`007P3oP00Hooooo`00
5_ooool40?l000030000o`3o0000o`000080o`0010000?l30?l00080003o0P3o00020000o`L0o`00
00<0003o0?l0003o00000P000?l40?l000030?n000000?l0003o0080003o2`3o00030?n001P0o`00
00<0oh000?l0003o0000403o00020000o`/0o`000`000?l60?l000D0oh001P3o00000`3oP000o`00
0?l000020?l000kooooo10000?l00`3o0000003o0000o`020?l000D0003o2000o`0:0000o`@0o`00
6`000?l20?l000<0003o1@3o000;0000o`800?l010000?l00`00o`00o`000?l000090000o`80o`00
50000?l00`3o0000003o0000o`020000o`80o`001`000?l>ooooo`D0003o0P3o000=003o00L0003o
1`00o`020?l000<00?l010000?l20?l000800?l01@000?l4003o00030000o`3o0000o`000080003o
103o00000`000?l0o`000?l000040000o`<0o`001P000?l3003o00@0003o0P00o`0[0000o`<00?l0
0P000?l>oooooaL0oh000P3o00090?n00080o`000P3oP0030?l001/0oh000P000?l30?l000/0oh00
0P3o000D0?n00080o`007P3oP00Hooooo`005_ooool20000o`80o`0000<0003o0?l0003o00005P3o
00000`000?l0o`000?l000060?l000D0003o2`3o00050?n000T0o`000P000?l<0?l000@0oh003@3o
00000`000?l0o`000?l000060?l000030000o`3o0000o`000080o`001@000?l40?l000D0oh000P00
0?l40?l000D0oh003_ooool40000o`030?l000000?l0003o00@0o`000P00o`000`000?l00?l0003o
0004003o0080o`000`000?l20?l000800?l04P000?l00`3o0000003o0000o`0=0000o`800?l000<0
o`000000o`000?l01@3o0002003o00@0003o0`00o`020000o`H00?l00`000?l=003o0080003o00<0
0?l00000o`000?l00P00o`030000o`@0o`0010000?l30?l000@0003o0P3o0003003o0080o`003_oo
ool70?l000@0003o2@00o`090000o`03003o00000?l0003o00@0003o0`00o`020000o`800?l00P00
0?l2003o00030000o`00o`000?l000@0003o0P00o`030000o`80o`001@000?l20?l000@00?l01`00
0?l20?l000D0003o1@00o`0;0000o`80o`002`00o`000`000?l00?l0003o0002003o0180003o3_oo
ool90?n00080o`002@3oP0020?l00200oh000`3o00080?n000<0o`000P000?l>0?l000L0oh000P3o
000@0?n00080o`00103oP0030?l001@0oh006?ooool001Kooooo2`3o00020000oa00o`0000<0003o
0?l0003o0000403o00020000o`H0o`0000<0oh000?l0003o000010000?lB0?l000D0oh000P000?l;
0?l000<0003o0P3o00020000o`80o`0000<0003o0?l0003o00001@3o00040000o`D0o`0010000?l5
0?l000@0oh003_ooool90000o`80o`000`00o`040000o`800?l02@000?l3003o00@0003o0P00o`00
0`000?l00?l0003o0002003o0080003o0P00o`000`000?l00?l0003o000;0000o`800?l00P000?l8
0?l00080003o0P3o00050000o`/00?l01`000?l4003o00T0003o0P3o00000`00o`00003o0000o`02
0?l00100003o0P3o000>ooooo`80o`000P000?l30?l00080003o1`00o`040000o`<0o`001P000?l0
0`00o`00003o0000o`040000o`800?l01@000?l40?l000D00?l00P3o000@0000o`800?l02@000?l0
0`00o`00o`000?l00002003o0080003o0P00o`050000o`800?l02@000?l2003o00<0003o0P00o`03
0000o`P00?l03P000?l>ooooo`h0oh000P000?l20?n00080o`00;P3oP0020?l000L0oh000P3o000V
0?n000030?l0003oP000oh0000L0oh000P3o000;0?n001Sooooo000Fooooo`/0o`000P000?l@0?l0
00030000o`3o0000o`000100o`000P000?l60?l000030?n0003o0000o`0000@0003o4P3o00050?n0
0080003o2`3o00030000o`80o`000P000?l20?l000030000o`3o0000o`0000D0o`0010000?l50?l0
00@0003o1@3o00040?n000kooooo2@000?l20?l000<00?l010000?l2003o00T0003o0`00o`040000
o`800?l000<0003o003o0000o`000P00o`020000o`800?l000<0003o003o0000o`002`000?l2003o
0080003o203o00020000o`80o`001@000?l;003o00L0003o1000o`090000o`80o`0000<00?l00000
o`000?l00P3o000@0000o`80o`003_ooool20?l00080003o0`3o00020000o`L00?l010000?l30?l0
00H0003o00<00?l00000o`000?l010000?l2003o00D0003o103o0005003o0080o`0040000?l2003o
00T0003o00<00?l00?l0003o00000P00o`020000o`800?l01@000?l2003o00T0003o0P00o`030000
o`800?l00`000?l8003o00h0003o3_ooool>0?n00080003o0P3oP0020?l002h0oh000P3o00070?n0
0080o`009P3oP0000`3o0000oh000?n000070?n00080o`002`3oP00Hooooo`005_ooool20?l00080
003o1`3o00030000o`l0o`0000<0003o0?l0003o00000P3o00040?n000X0o`000P000?lK0?l00080
003o0`3o00040?n000@0003o00<0o`000?n0003oP0000P3o00020000o`L0o`001@000?l@0?l000H0
003o1@3o00020?n000kooooo2`000?l9003o00L0003o1`00o`0;0000o`h00?l00P000?l4003o00<0
o`001@000?l40?l000D0003o0P3o0002003o00D0003o1P00o`070000o`80o`000`000?l20?l001T0
003o0P00o`0>ooooo`@0o`001`000?l5003o0080003o103o00050000o`800?l02@000?l00`00o`00
o`000?l000040000o`80o`001@000?l20?l00080003o0`3o000@0000o`80o`000P000?l50?l000H0
0?l000<0o`000000o`000?l00P000?l20?l000h0003o1000o`000`000?l00?l0003o00020000o`80
0?l00`000?l2003o00L0003o1000o`0>oooooa80oh000P3o00050?n00080o`000P3oP0000`3o0000
oh000?n0000D0?n00080003o0P3oP0000`3o0000oh000?n000090?n000L0o`00>`3oP0040?l001So
oooo000Fooooo`80o`000P000?l70?l000<0003o3`3o00000`000?l0o`000?l000020?l000@0oh00
2P3o00020000oa/0o`000P000?l30?l000@0oh0010000?l00`3o0000oh000?n000020?l00080003o
1`3o00050000oa00o`001P000?l50?l00080oh003_ooool;0000o`T00?l01`000?l7003o00/0003o
3P00o`020000o`@00?l00`3o00050000o`@0o`001@000?l20?l000800?l01@000?l6003o00L0003o
0P3o00030000o`80o`006@000?l2003o00kooooo103o00070000o`D00?l00P000?l40?l000D0003o
0P00o`090000o`03003o003o0000o`0000@0003o0P3o00050000o`80o`000P000?l30?l00100003o
0P3o00020000o`D0o`001P00o`000`3o0000003o0000o`020000o`80o`003P000?l4003o00030000
o`00o`000?l00080003o0P00o`030000o`800?l01`000?l4003o00kooooo4P3oP0020?l000D0oh00
0P3o00020?n000030?l0003oP000oh0001@0oh000P000?l20?n000030?l0003oP000oh0000T0oh00
1`3o000k0?n000@0o`006?ooool001Kooooo0P3o00020000o`L0o`000P000?l<0?l00080003o0P3o
00000`000?l0o`000?l000020?n000@0o`0000<0003o0?l0003o00005P3o00030000o`/0o`000P00
0?l20?l000030000o`3o0000o`0000T0o`0010000?l90?l00080003o0`3o00020000o`80o`0000<0
oh000?l0003o0000103o00020000o`<0o`000P000?l40?l000030000o`3o0000o`000080003o3_oo
ool40000o`T00?l000<0003o003o0000o`001000o`070000o`D00?l01P000?l00`00o`00003o0000
o`0?003o00T0003o00<0o`000000o`000?l04P000?l5003o0080003o1`00o`020000o`800?l00P3o
00090000o`<00?l00P000?l;003o00L0003o3_ooool40?l000L0003o1@00o`040000o`<0o`001P00
0?l00`3o0000003o0000o`020000o`80o`000P000?l00`00o`00o`000?l000070000o`80o`001000
0?l20?l000X0003o1`00o`020000o`H0o`000`000?l4003o0080003o00<0o`000000o`000?l00P00
0?l4003o00h0003o1`00o`040000o`<00?l02`000?l>oooooa80oh000P3o000P0?n00080o`004P3o
P0030?l000/0oh000P3o00020?n000030?l0003oP000oh0000l0oh0000<0o`000?n0003oP000403o
P0020?l000T0oh006?ooool001Kooooo0P3o00020000o`L0o`000P000?l<0?l00080003o0P3o0000
0`000?l0o`000?l000020?n000@0o`0000<0003o0?l0003o00005P3o00030000o`/0o`000P000?l2
0?l000030000o`3o0000o`0000T0o`0010000?l90?l00080003o0`3o00020000o`80o`0000<0oh00
0?l0003o0000103o00020000o`<0o`000P000?l40?l000030000o`3o0000o`000080003o3_ooool4
0000o`T00?l000<0003o003o0000o`001000o`070000o`D00?l01P000?l00`00o`00003o0000o`0?
003o00T0003o00<0o`000000o`000?l04P000?l5003o0080003o1`00o`020000o`800?l00P3o0009
0000o`<00?l00P000?l;003o00L0003o3_ooool40?l000L0003o1@00o`040000o`<0o`001P000?l0
0`3o0000003o0000o`020000o`80o`000P000?l00`00o`00o`000?l000070000o`80o`0010000?l2
0?l000X0003o1`00o`020000o`H0o`000`000?l4003o0080003o00<0o`000000o`000?l00P000?l4
003o00h0003o1`00o`040000o`<00?l02`000?l>oooooa80oh000P3o000P0?n00080o`004P3oP003
0?l000/0oh000P3o00020?n000030?l0003oP000oh0000l0oh0000<0o`000?n0003oP000403oP002
0?l000T0oh006?ooool001Kooooo0P3o00020000o`L0o`000P00o`070?l00080003o0`3o00020000
o`80o`0000<0003o0?l0003o0000103o00020000o`T0o`0000<0003o0?l0003o0000;@3o00020000
o`T0o`000P000?l70?l000030000o`3o0000o`0000L0o`000P000?l70?l00080003o0P3o000>oooo
o`@00?l00`000?l=003o0080o`0000<0003o003o0000o`001@00o`040000o`@0o`004000o`020000
o`L00?l030000?l20?l00080003o1@3o0009003o0080003o1@00o`0@0000o`80o`001P000?l3003o
00@0003o0`3o00040000o`kooooo1`000?l2003o0080o`000`000?l20?l00180003o0P3o00030000
o`80o`001`000?l20?l000@0003o0P3o000@0000o`03003o00000?l0003o0080003o0P3o00030000
o`@0o`0050000?l20?l000<0003o103o00000`000?l0o`000?l000020000o`@0o`000`000?l4003o
00L0003o3_oooolI0?n000@0o`00:`3oP0030?l001H0oh000P3o000W0?n001Sooooo000Fooooo`80
003o0P3o00000`000?l0o`000?l000020?l000@0003o5@3o00050000o`P0o`0000<0003o0?l0003o
00000P3o00020000obD0o`000P000?l20?l00080003o2@3o00020000o`L0o`0000<0003o0?l0003o
00004P3o00020000o`kooooo10000?lJ003o00@0003o103o00030000o`800?l00P000?l9003o01D0
003o0P3o00020000o`D0o`003`00o`0<0000o`80o`000`000?l20?l000L0003o0P3o00040000o`<0
o`0010000?l>ooooo`/0003o0`00o`080000o`L00?l000<0003o003o0000o`000P00o`020?l000D0
003o0P3o00020000o`D0o`003@000?l30?l000H0003o00<0o`000000o`000?l00P3o00070000o`80
o`001@000?l2003o00@0003o0`00o`0=0000o`80o`0000<0003o0?l0003o00002@3o000;0000o`ko
ooooB03oP0000`3o0000oh000?n0000X0?n000030?l0003oP000oh0000d0oh000P3o00050?n001So
oooo000Fooooo`80003o0P3o00000`000?l0o`000?l000020?l000@0003o5@3o00050000o`P0o`00
00<0003o0?l0003o00000P3o00020000obD0o`000P000?l20?l00080003o2@3o00020000o`L0o`00
00<0003o0?l0003o00004P3o00020000o`kooooo10000?lJ003o00@0003o103o00030000o`800?l0
0P000?l9003o01D0003o0P3o00020000o`D0o`003`00o`0<0000o`80o`000`000?l20?l000L0003o
0P3o00040000o`<0o`0010000?l>ooooo`/0003o0`00o`080000o`L00?l000<0003o003o0000o`00
0P00o`020?l000D0003o0P3o00020000o`D0o`003@000?l30?l000H0003o00<0o`000000o`000?l0
0P3o00070000o`80o`001@000?l2003o00@0003o0`00o`0=0000o`80o`0000<0003o0?l0003o0000
2@3o000;0000o`koooooB03oP0000`3o0000oh000?n0000X0?n000030?l0003oP000oh0000d0oh00
0P3o00050?n001Sooooo000Fooooo`80003o0P3o00000`000?l0o`000?l000020?l000@0003o403o
00030000o`H0o`0000<0003o0?l0003o00001P3o00000`000?l0o`000?l000090?l00080003o103o
00050000o`<0o`000P3oP00D0?l00080003o2@3o00070000oaL0o`000P000?l>oooooa@00?l01`00
0?l2003o0100003o0P00o`030000o`@00?l00`000?l20?l000L0003o0P00o`070000o`H0o`0000<0
003o0?l0003o00000P3o000=003o00030?l000000?l0003o0080003o0P3o0007003o00<0003o0P3o
000=0000o`80o`0000<0003o003o0000o`000P00o`0>ooooo`T00?l03@000?l00`00o`00003o0000
o`020000o`80o`001@000?l20?l000<0003o0P00o`020?l000@0003o00<0o`000000o`000?l00P00
0?l9003o0080003o0`3o00060000o`03003o00000?l0003o0080o`000P000?l30?l000T0003o0P00
o`0D0000o`80o`0000<00?l00?l0003o0000103o00030000o`80o`000P00o`070000o`80o`003_oo
ool_0?n000030?l0003oP000oh000280oh000P3o000I0?n00080o`006@3oP00Hooooo`005_ooool2
0000o`80o`0000<0003o0?l0003o00000P3o00040000oa00o`000`000?l60?l000030000o`3o0000
o`0000H0o`0000<0003o0?l0003o00002@3o00020000o`@0o`001@000?l30?l00080oh00503o0002
0000o`T0o`001`000?lG0?l00080003o3_oooolD003o00L0003o0P00o`0@0000o`800?l00`000?l4
003o00<0003o0P3o00070000o`800?l01`000?l60?l000030000o`3o0000o`000080o`003@00o`00
0`3o0000003o0000o`020000o`80o`001`00o`030000o`80o`003@000?l20?l000030000o`00o`00
0?l000800?l03_ooool9003o00d0003o00<00?l00000o`000?l00P000?l20?l000D0003o0P3o0003
0000o`800?l00P3o00040000o`030?l000000?l0003o0080003o2@00o`020000o`<0o`001P000?l0
0`00o`00003o0000o`020?l00080003o0`3o00090000o`800?l050000?l20?l00003003o003o0000
o`0000@0o`000`000?l20?l000800?l01`000?l20?l000kooooo;`3oP0000`3o0000oh000?n0000R
0?n00080o`006@3oP0020?l001T0oh006?ooool001Kooooo1`000?l60?l000030000o`3o0000o`00
00d0o`0000<0003o0?l0003o00008P3o00020000o`80o`000P000?l30?l000@0003o3P3o00020000
o`T0o`001@000?l20?l00080003o1@3o00020000oa80o`003_ooool20?l001/00?l000<0003o0?l0
003o000010000?l20?l000<0003o0P3o00020000o`/00?l02P000?l20?l000@0003o0`3o00060?n0
00<0003o0P3o0004003o00030000o`00o`000?l000800?l03@000?l3003o00h0003o0P3o00070000
o`@00?l03_ooool4003o00030000o`00o`000?l000800?l01`000?l6003o00030?l000000?l0003o
0080003o0P3o00000`000?l00?l0003o0002003o00L0003o0P3o00050000o`P00?l000<0003o003o
0000o`001000o`020?l000D0003o0P00o`050000o`80o`001`000?l5003o00H0003o0P3o000A0000
o`80o`000P000?l20?l000D0003o0P00o`090000o`kooooo5`3oP0040000o`80o`009@3oP0040?l0
00`0oh000P000?l40?l001T0oh000P3o000<0?n00080003o0P3o00070?n001Sooooo000Fooooo`L0
003o1P3o00000`000?l0o`000?l0000=0?l000030000o`3o0000o`000280o`000P000?l20?l00080
003o0`3o00040000o`h0o`000P000?l90?l000D0003o0P3o00020000o`D0o`000P000?lB0?l000ko
oooo0P3o000K003o00030000o`3o0000o`0000@0003o0P3o00030000o`80o`000P000?l;003o00X0
003o0P3o00040000o`<0o`001P3oP0030000o`80o`001000o`000`000?l00?l0003o0002003o00d0
003o0`00o`0>0000o`80o`001`000?l4003o00kooooo1000o`000`000?l00?l0003o0002003o00L0
003o1P00o`000`3o0000003o0000o`020000o`80o`0000<0003o003o0000o`000P00o`070000o`80
o`001@000?l8003o00030000o`00o`000?l000@00?l00P3o00050000o`800?l01@000?l20?l000L0
003o1@00o`060000o`80o`004@000?l20?l00080003o0P3o00050000o`800?l02@000?l>oooooaL0
oh0010000?l20?l002D0oh00103o000<0?n00080003o103o000I0?n00080o`00303oP0020000o`80
o`001`3oP00Hooooo`005_ooool20000o`80o`0000<0003o0?l0003o0000103o00020000oa00o`00
0`000?l[0?l00080003o403o00020000o`d0o`0000<0003o0?l0003o00000P3o00020000o`D0o`00
0P000?l30?l00080003o1`3o00020000o`80o`003_ooool20?l000T00?l01@000?l>003o0080o`00
1P000?l50?l000T00?l05@000?l60?l00080oh0000<0003o0?l0003o00000P3o000;003o00T0003o
1@00o`090000o`800?l00P000?l30?l00080003o103o00000`000?l0o`000?l00002003o00kooooo
10000?l:003o00H0003o0`00o`0B0000o`80o`001@000?l9003o00d0003o0P00o`000`000?l00?l0
003o0006003o00<0003o1000o`020000o`<00?l0;@000?l>oooooa80oh002`3o000W0?n00080o`00
3P3oP0020?l003@0oh006?ooool001Kooooo0P000?l20?l000030000o`3o0000o`0000@0o`000P00
0?l@0?l000<0003o:`3o00020000oa00o`000P000?l=0?l000030000o`3o0000o`000080o`000P00
0?l50?l00080003o0`3o00020000o`L0o`000P000?l20?l000kooooo0P3o0009003o00D0003o3P00
o`020?l000H0003o1@3o0009003o01D0003o1P3o00020?n000030000o`3o0000o`000080o`002`00
o`090000o`D00?l02@000?l2003o0080003o0`3o00020000o`@0o`0000<0003o0?l0003o00000P00
o`0>ooooo`@0003o2P00o`060000o`<00?l04P000?l20?l000D0003o2@00o`0=0000o`800?l000<0
003o003o0000o`001P00o`030000o`@00?l00P000?l3003o02d0003o3_oooolB0?n000/0o`009`3o
P0020?l000h0oh000P3o000d0?n001Sooooo000Fooooo`@0o`0000<0003o0?l0003o0000403o0004
0000o`80o`0000<0003o0?l0003o00001`3o00020000o`d0o`000P3oP00W0?l00080003o6`3o0005
0000o`D0o`0010000?l>oooooa00003o3P00o`040000o`L0o`002@000?l2003o00h0003o1000o`03
0000o`@0o`0010000?l00`3o0000003o0000o`020?l000T00?l01@000?l2003o0080003o2`00o`05
0000o`800?l00P000?l90?l000030000o`00o`000?l000800?l03_ooool2003o00L0003o1@00o`06
0000o`800?l01`000?l5003o00h0003o2@00o`0;0000o`L00?l01`000?l20?l000T00?l00P000?l2
003o00/0003o4`00o`0;0000o`kooooo0P3o000E0?n000H0o`00903oP0000`3o0000oh000?n0000P
0?n00080o`00903oP00Hooooo`005_ooool50000o`H0o`000P000?l@0?l000030000o`3o0000o`00
00H0o`0000<0003o0?l0003o00000P000?l40?l000<0003o6@3o00020000o`D0o`000P000?l?0?l0
00030?n000000?l0003o00H0o`0000<0003o0?l0003o00005`3o00020000o`80o`003_ooool4003o
00030000o`00o`000?l000d00?l00`000?l40?l000<00?l02@000?l20?l000/00?l040000?l5003o
00@0o`001@000?l40?l00080003o0P3o0005003o00L0003o2@00o`0=0000o`D0o`000P000?l30?l0
0080003o0P3o000>ooooo`80003o1`00o`040000o`03003o00000?l0003o00@00?l00`000?l9003o
00/0003o0P00o`020000o`T00?l000<0003o003o0000o`004000o`020?l00080003o0`3o00020000
o`@00?l00`000?l2003o0080003o0P00o`020?l000<0003o0P3o00040000o`D00?l00P000?l00`00
o`00003o0000o`02003o00L0003o1P00o`050000o`kooooo6`3oP0070?l000L0oh000P3o00070?n0
0080003o1@3o00020?n00080o`00C@3oP00Hooooo`005_ooool50000o`H0o`000P000?l@0?l00003
0000o`3o0000o`0000H0o`0000<0003o0?l0003o00000P000?l40?l000<0003o6@3o00020000o`D0
o`000P000?l?0?l000030?n000000?l0003o00H0o`0000<0003o0?l0003o00005`3o00020000o`80
o`003_ooool4003o00030000o`00o`000?l000d00?l00`000?l40?l000<00?l02@000?l20?l000/0
0?l040000?l5003o00@0o`001@000?l40?l00080003o0P3o0005003o00L0003o2@00o`0=0000o`D0
o`000P000?l30?l00080003o0P3o000>ooooo`80003o1`00o`040000o`03003o00000?l0003o00@0
0?l00`000?l9003o00/0003o0P00o`020000o`T00?l000<0003o003o0000o`004000o`020?l00080
003o0`3o00020000o`@00?l00`000?l2003o0080003o0P00o`020?l000<0003o0P3o00040000o`D0
0?l00P000?l00`00o`00003o0000o`02003o00L0003o1P00o`050000o`kooooo6`3oP0070?l000L0
oh000P3o00070?n00080003o1@3o00020?n00080o`00C@3oP00Hooooo`005_ooool50000o`H0o`00
0P000?l30?l00080oh000P3o00020000o`D0o`000`000?l80?l000030000o`3o0000o`0000@0o`00
1`000?l40?l000030?n000000?l0003o00L0o`000P000?l=0?l000030000o`3o0000o`0000H0o`00
00<0003o0?l0003o00001`3o00040000o`80o`000`000?l40?l00080oh005`3o000>oooooa@00?l0
0P3o00000`000?l0o`000?l000040000o`03003o00000?l0003o0080o`002@000?l9003o0100003o
1@00o`020000o`80o`0010000?l00`3o0000003o0000o`060000o`T00?l000<0003o003o0000o`00
2@00o`070000o`80o`000P00o`030000o`80o`001`000?l2003o0080003o3_ooool40000ob<00?l0
1P000?l20?l000030000o`00o`000?l001@00?l01@000?l20?l00080003o0`3o00020000o`800?l0
10000?l00`00o`00003o0000o`020000o`@00?l00`000?l40?l00080003o00<0o`00003o0000o`00
0P000?l3003o00T0003o1000o`090000o`kooooo7P3oP0020?l00180oh000P3o00070?n000@0o`00
B`3oP00Hooooo`005_ooool50000o`H0o`000P000?l30?l00080oh000P3o00020000o`D0o`000`00
0?l80?l000030000o`3o0000o`0000@0o`001`000?l40?l000030?n000000?l0003o00L0o`000P00
0?l=0?l000030000o`3o0000o`0000H0o`0000<0003o0?l0003o00001`3o00040000o`80o`000`00
0?l40?l00080oh005`3o000>oooooa@00?l00P3o00000`000?l0o`000?l000040000o`03003o0000
0?l0003o0080o`002@000?l9003o0100003o1@00o`020000o`80o`0010000?l00`3o0000003o0000
o`060000o`T00?l000<0003o003o0000o`002@00o`070000o`80o`000P00o`030000o`80o`001`00
0?l2003o0080003o3_ooool40000ob<00?l01P000?l20?l000030000o`00o`000?l001@00?l01@00
0?l20?l00080003o0`3o00020000o`800?l010000?l00`00o`00003o0000o`020000o`@00?l00`00
0?l40?l00080003o00<0o`00003o0000o`000P000?l3003o00T0003o1000o`090000o`kooooo7P3o
P0020?l00180oh000P3o00070?n000@0o`00B`3oP00Hooooo`005_ooool20000o`800?l01`3o0002
0000o`<0o`000P000?l20?l00080003o6P3o00020000o`80o`000P000?l50?l000@0003o1@3o0002
0000o`L0o`000P000?l<0?l00080003o1`3o00020000o`/0o`000P3oP0000`000?l0o`000?l00007
0?l00080003o1P3o00000`3oP000o`000?l000020?l000kooooo5000o`070000o`800?l000<0003o
0?l0003o000010000?l20?l000h00?l03P000?l20?l000800?l00P000?l3003o0080o`000P000?l5
0?l000T0003o1000o`020000o`03003o00000?l0003o00T00?l02@000?l2003o00D0003o0P3o0009
0000o`kooooo1@000?l20?l000@0003o0P3o0007003o00D0003o0P00o`020000o`<00?l040000?l=
003o0080003o1@00o`070000o`@0o`000P000?l00`3o0000003o0000o`04003o00L0003o0P00o`02
0000o`030?l000000?l0003o00@0o`001@000?l9003o00@0003o1@3o00020000o`<0o`0010000?l>
oooooaL0oh00103o000E0?n00080o`002@3oP0020000o`80o`001@3oP0020000o`<0o`00;`3oP002
0?l00100oh006?ooool001Kooooo0P000?l2003o00L0o`000P000?l30?l00080003o0P3o00020000
oaX0o`000P000?l20?l00080003o1@3o00040000o`D0o`000P000?l70?l00080003o303o00020000
o`L0o`000P000?l;0?l00080oh0000<0003o0?l0003o00001`3o00020000o`H0o`0000<0oh000?l0
003o00000P3o000>oooooa@00?l01`000?l2003o00030000o`3o0000o`0000@0003o0P3o000>003o
00h0003o0P3o0002003o0080003o0`00o`020?l00080003o1@3o00090000o`@00?l00P000?l00`00
o`00003o0000o`09003o00T0003o0P00o`050000o`80o`002@000?l>ooooo`D0003o0P3o00040000
o`80o`001`00o`050000o`800?l00P000?l3003o0100003o3@00o`020000o`D00?l01`000?l40?l0
0080003o00<0o`000000o`000?l01000o`070000o`800?l00P000?l00`3o0000003o0000o`040?l0
00D0003o2@00o`040000o`D0o`000P000?l30?l000@0003o3_oooolG0?n000@0o`005@3oP0020?l0
00T0oh000P000?l20?l000D0oh000P000?l30?l002l0oh000P3o000@0?n001Sooooo000Fooooo`@0
0?l01`3o00020000o`030?l000000?l0003o0080003o0P3o00020000o`<0o`000P000?l20?l000<0
003o1P3o00000`000?l0o`000?l0000M0?l00080003o0`3o00040000oaD0o`000P000?l20?l00080
003o0`3o00060?n000<0003o1P3o00050000o`@0o`000`3oP0020?l000kooooo4P00o`070000o`D0
0?l00P3o00070000oa800?l02`000?l2003o00D0003o0P3o00070000o`800?l01`000?l20?l00080
0?l00P000?l3003o0080003o1`00o`0B0000o`80o`001`000?l>ooooo`D0003o0P3o00020000o`/0
0?l00`000?l40?l000D00?l01`000?l20?l000H0003o4`00o`090000o`80o`001@000?l20?l00080
0?l01`000?l6003o00030?l000000?l0003o00@0o`000`000?l6003o00030000o`00o`000?l00080
0?l01@000?l60?l00080003o00<0o`000000o`000?l00P00o`0>oooooc/0oh000P000?l20?l000D0
oh000P3o00090?n00080o`008`3oP0060000o`80o`003P3oP00Hooooo`005_ooool4003o00L0o`00
0P000?l00`3o0000003o0000o`020000o`80o`000P000?l30?l00080003o0P3o00030000o`H0o`00
00<0003o0?l0003o00007@3o00020000o`<0o`0010000?lE0?l00080003o0P3o00020000o`<0o`00
1P3oP0030000o`H0o`001@000?l40?l000<0oh000P3o000>oooooa800?l01`000?l5003o0080o`00
1`000?lB003o00/0003o0P00o`050000o`80o`001`000?l2003o00L0003o0P3o0002003o0080003o
0`00o`020000o`L00?l04P000?l20?l000L0003o3_ooool50000o`80o`000P000?l;003o00<0003o
103o0005003o00L0003o0P3o00060000oa<00?l02@000?l20?l000D0003o0P3o0002003o00L0003o
1P00o`000`3o0000003o0000o`040?l000<0003o1P00o`000`000?l00?l0003o0002003o00D0003o
1P3o00020000o`030?l000000?l0003o00800?l03_oooolk0?n00080003o0P3o00050?n00080o`00
2@3oP0020?l002<0oh001P000?l20?l000h0oh006?ooool001Kooooo1000o`0R0?l000030000o`3o
0000o`0001H0o`000P000?l30?l00080003o0P3o00000`000?l0o`000?l000020000o`80o`000P00
0?l50?l000@0003o2@3o00000`000?l0o`000?l0000=0?l000D0003o103o00000`000?l0o`000?l0
00020000o`D0o`000P3oP0020000o`kooooo7P00o`060000oa000?l03P000?l20?l000L00?l00P3o
00090000o`800?l00P3o00030000o`80o`000P00o`030000o`800?l00P3o00020000o`800?l000<0
o`00003o0000o`0010000?l7003o0080003o00<00?l00000o`000?l00P3o00020000o`80o`000`00
0?l20?l000kooooo2@000?l2003o00D0003o1000o`030000o`@0o`000`000?l20?l00080003o0P00
o`030000o`80o`001`000?l2003o00@0003o2@00o`020000o`03003o003o0000o`0000L0003o0P3o
00040000o`030?l000000?l0003o00800?l010000?l00`00o`00003o0000o`020000o`@00?l04@00
0?l4003o0080003o0P3o00050000o`@0o`001@000?l>ooooo`/0oh000P3o000G0?n00080o`007P3o
P0040?l000`0oh000P3o000R0?n000@0o`003P3oP00Hooooo`005_ooool4003o00030000o`3o0000
o`000180o`000P000?l20?l000030000o`3o0000o`000080003o103o00030000o`H0o`000`000?l4
0?l000<0003o7@3o00040000oaD0o`000P000?l50?l000@0003o1@3o00020000o`L0o`000P000?l>
oooooa@00?l00P000?l7003o00T0003o4000o`0>0000o`D00?l02@000?l6003o00/0003o1@00o`04
0000o`030?l000000?l0003o00L00?l00P000?l4003o00D0003o0P3o00020000o`030?l000000?l0
003o0080003o3_ooool=0000o`<00?l00P000?l2003o00D0003o0P3o00030000o`80o`000P00o`07
0000o`80o`000P000?l50?l000800?l00P000?l9003o0080003o00<0o`000000o`000?l01000o`03
0000o`80o`001@000?l40?l000@0003o00<0o`000000o`000?l01`00o`040000o`80o`000P000?l0
0`00o`00003o0000o`020?l00080003o0`3o00020000o`80o`002@000?l20?l000D0003o3_ooool;
0?n000<0o`000P3oP0020?l00180oh000P3o00090?n000030?l0003oP000oh0000L0oh000P3o000[
0?n000@0o`003P3oP0020000o`80o`003P3oP00Hooooo`005_ooool4003o00030000o`3o0000o`00
0180o`000P000?l20?l000030000o`3o0000o`000080003o103o00030000o`H0o`000`000?l40?l0
00<0003o7@3o00040000oaD0o`000P000?l50?l000@0003o1@3o00020000o`L0o`000P000?l>oooo
oa@00?l00P000?l7003o00T0003o4000o`0>0000o`D00?l02@000?l6003o00/0003o1@00o`040000
o`030?l000000?l0003o00L00?l00P000?l4003o00D0003o0P3o00020000o`030?l000000?l0003o
0080003o3_ooool=0000o`<00?l00P000?l2003o00D0003o0P3o00030000o`80o`000P00o`070000
o`80o`000P000?l50?l000800?l00P000?l9003o0080003o00<0o`000000o`000?l01000o`030000
o`80o`001@000?l40?l000@0003o00<0o`000000o`000?l01`00o`040000o`80o`000P000?l00`00
o`00003o0000o`020?l00080003o0`3o00020000o`80o`002@000?l20?l000D0003o3_ooool;0?n0
00<0o`000P3oP0020?l00180oh000P3o00090?n000030?l0003oP000oh0000L0oh000P3o000[0?n0
00@0o`003P3oP0020000o`80o`003P3oP00Hooooo`005_ooool2003o0080003o1`3o00020000oaT0
o`0000<0003o0?l0003o00001P3o00030000oa00o`000P000?lI0?l000@0003o0P3o00000`000?l0
o`000?l000020?l00080003o5@3o00040000o`D0o`000P000?l>ooooo`80003o0P3o000@003o0080
003o2000o`080000oa<00?l02@000?l2003o00D0003o0P00o`040000o`D00?l00P000?l2003o00T0
003o1@00o`070000o`800?l01@000?l8003o00h0003o3_ooool70000o`@00?l01`000?l2003o0080
o`0000<0003o0?l0003o00000P3o00020000o`03003o003o0000o`0000800?l010000?l00`3o0000
003o0000o`090?l000H0003o1`00o`020000o`030?l000000?l0003o00@0003o0`00o`0=0000o`80
0?l000<0003o003o0000o`000P000?l2003o0080003o00<00?l00?l0003o00001P000?l3003o00@0
003o1@3o000B0000o`kooooo3@3oP0030?l000H0oh0000<0o`000?n0003oP000:@3oP0070?l002l0
oh000P3o00030?n000H0o`001`3oP00Hooooo`005_ooool2003o0080003o1`3o00020000oaT0o`00
00<0003o0?l0003o00001P3o00030000oa00o`000P000?lI0?l000@0003o0P3o00000`000?l0o`00
0?l000020?l00080003o5@3o00040000o`D0o`000P000?l>ooooo`80003o0P3o000@003o0080003o
2000o`080000oa<00?l02@000?l2003o00D0003o0P00o`040000o`D00?l00P000?l2003o00T0003o
1@00o`070000o`800?l01@000?l8003o00h0003o3_ooool70000o`@00?l01`000?l2003o0080o`00
00<0003o0?l0003o00000P3o00020000o`03003o003o0000o`0000800?l010000?l00`3o0000003o
0000o`090?l000H0003o1`00o`020000o`030?l000000?l0003o00@0003o0`00o`0=0000o`800?l0
00<0003o003o0000o`000P000?l2003o0080003o00<00?l00?l0003o00001P000?l3003o00@0003o
1@3o000B0000o`kooooo3@3oP0030?l000H0oh0000<0o`000?n0003oP000:@3oP0070?l002l0oh00
0P3o00030?n000H0o`001`3oP00Hooooo`005_ooool2003o0080003o1`3o00030000o`H0o`000`00
0?l?0?l000030000o`3o0000o`000080o`0010000?l:0?l00080003o1P3o00030000o`d0o`0000<0
oh000?l0003o00001P3o00030000o`H0o`000`000?l=0?l000030000o`3o0000o`0000l0o`0000<0
003o0?l0003o00000P3o000>ooooo`80003o0P3o000J003o00H0003o0P3o000<003o00T0003o1P3o
000:0000o`800?l010000?l5003o0080003o0P00o`070000o`D00?l02@000?l;003o00L0003o0P3o
00070000o`kooooo1`000?l2003o00/0003o0P3o00000`000?l0o`000?l000020?l000/0003o00<0
o`000000o`000?l0103o00020000o`030?l000000?l0003o00L0003o1000o`040000o`030?l00000
0?l0003o00D0003o0P00o`020?l000D0003o0P00o`0@0000o`80o`003@000?l00`3o0000003o0000
o`020000o`80o`000`000?l20?l000/0003o3_ooool;0?n00080003o0`3o000T0?n00080o`00;@3o
P0000`3o0000oh000?n0000I0?n00080o`001@3oP0020000o`80o`006?ooool001Kooooo0P00o`02
0000o`L0o`000`000?l60?l000<0003o3`3o00000`000?l0o`000?l000020?l000@0003o2P3o0002
0000o`H0o`000`000?l=0?l000030?n0003o0000o`0000H0o`000`000?l60?l000<0003o3@3o0000
0`000?l0o`000?l0000?0?l000030000o`3o0000o`000080o`003_ooool20000o`80o`006P00o`06
0000o`80o`003000o`090000o`H0o`002P000?l2003o00@0003o1@00o`020000o`800?l01`000?l5
003o00T0003o2`00o`070000o`80o`001`000?l>ooooo`L0003o0P00o`0;0000o`80o`0000<0003o
0?l0003o00000P3o000;0000o`030?l000000?l0003o00@0o`000P000?l00`3o0000003o0000o`07
0000o`@00?l010000?l00`3o0000003o0000o`050000o`800?l00P3o00050000o`800?l040000?l2
0?l000d0003o00<0o`000000o`000?l00P000?l20?l000<0003o0P3o000;0000o`kooooo2`3oP002
0000o`<0o`00903oP0020?l002d0oh0000<0o`000?n0003oP0006@3oP0020?l000D0oh000P000?l2
0?l001Sooooo000Fooooo`800?l00P000?l@0?l000<0003o0P3o00050000o`P0o`0000<0003o0?l0
003o00002@3o00020000o`T0o`000P3oP0020?l000030000o`3o0000o`0000@0o`000`3oP00?0?l0
00030000o`3o0000o`000080oh000P000?l30?l000@0003o2`3o00000`000?l0o`000?l0000D0?l0
00kooooo10000?l5003o0080003o4P00o`000`000?l0o`000?l000040000o`80o`003000o`020000
o`800?l01@000?l60?l000030000o`00o`000?l00080003o0`00o`0;0000o`800?l01`000?l4003o
0080003o00<00?l00000o`000?l05P00o`090000o`030?l000000?l0003o00800?l03_ooool40000
o`030?l00000o`000?l000L00?l020000?l00`3o0000003o0000o`0=0000o`030?l000000?l0003o
0080o`000P3oP0020000o`030?n000000?l0003o00@0003o0`00o`020000o`@00?l00P000?l00`3o
0000003o0000o`020000o`80o`0000<0003o003o0000o`000P3o00040000o`030?l000000?l0003o
00L0003o0P3o000K0000o`80o`000`000?l20?l000@0003o0P00o`050000o`kooooo2`3o00020000
oaD0oh000P000?l20?n000030?l0003oP000oh0000d0oh000P3o00150?n000@0003o0P3oP0020?l0
00030?n000000?l0003o0080003o6?ooool001Kooooo1000o`030?l00080oh000P000?l>0?l000@0
003o1@3o00020000o`80o`0000<0003o0?l0003o00006`3o00020000o`h0o`0010000?l90?l00080
003o1`3o00020000o`030?l000000?l0003o0080003o303o00020000o`@0o`000P3oP0050?l000ko
oooo10000?lG003o00T0003o2@00o`0;0000o`03003o003o0000o`0000@0003o0P3o00050000o`80
0?l000<0003o003o0000o`000P000?l5003o00P0003o1`00o`070000oa@00?l02@000?l00`3o0000
0?l0003o00020000o`kooooo10000?l00`3o00000?l0003o0009003o0080o`003P000?l2003o0080
003o0`3o00040000o`80o`000P000?l30?l00080003o2`00o`070000o`80o`0030000?l4003o0080
003o00<00?l00000o`000?l02@000?l20?l000T0003o0`3o00020000o`@0o`000`000?l20?l000@0
003o0P00o`000`000?l00?l0003o00020000o`koooooO`3oP0020?l00080003o0P3o00050?n001So
oooo000Fooooo`@00?l00`3o00020?n00080003o3P3o00040000o`D0o`000P000?l20?l000030000
o`3o0000o`0001/0o`000P000?l>0?l000@0003o2@3o00020000o`L0o`000P000?l00`3o0000003o
0000o`020000o``0o`000P000?l40?l00080oh001@3o000>ooooo`@0003o5`00o`090000o`T00?l0
2`000?l00`00o`00o`000?l000040000o`80o`001@000?l2003o00030000o`00o`000?l00080003o
1@00o`080000o`L00?l01`000?lD003o00T0003o00<0o`00003o0000o`000P000?l>ooooo`@0003o
00<0o`00003o0000o`002@00o`020?l000h0003o0P00o`020000o`<0o`0010000?l20?l00080003o
0`3o00020000o`/00?l01`000?l20?l000`0003o1000o`020000o`03003o00000?l0003o00T0003o
0P3o00090000o`<0o`000P000?l40?l000<0003o0P3o00040000o`800?l000<0003o003o0000o`00
0P000?l>ooooogl0oh000P3o00020000o`80o`001@3oP00Hooooo`005_ooool2003o00L0o`000P00
0?l50?l00080oh002@3o00020000o`<0o`0010000?l20?l000030000o`3o0000o`0000H0o`0000<0
003o0?l0003o00001P3o00000`000?l0o`000?l0000>0?l00080oh000P000?l70?l000@0003o0`3o
00020?n000/0o`000P000?lC0?l00080003o1P3o00000`3oP000003o0000o`020?l000kooooo0P00
0?l5003o0080003o0P00o`030000oa000?l020000?l:003o00T0003o0P3o00070000o`L00?l01000
0?l20?l000D0003o1P00o`0>0000o`80o`003000o`090000o`80o`002@000?l>ooooo`D0003o2`00
o`0;0000o`80o`000`000?l2003o00@0003o00<0o`000000o`000?l0103o00030000o`80o`000P00
0?l=003o00D0003o0P3o00000`000?l0o`000?l000040000o`L0o`0010000?l00`00o`00o`000?l0
00040000o`80o`0000<0003o0?l0003o00000P3o00020000o`D00?l02@000?l20?l000T0003o0P00
o`030000o`800?l03_oooom80?n000<0o`00403oP0020?l002d0oh006?ooool001Kooooo0P00o`07
0?l00080003o1@3o00020?n000T0o`000P000?l30?l000@0003o0P3o00000`000?l0o`000?l00006
0?l000030000o`3o0000o`0000H0o`0000<0003o0?l0003o00003P3o00020?n00080003o1`3o0004
0000o`<0o`000P3oP00;0?l00080003o4`3o00020000o`H0o`0000<0oh000000o`000?l00P3o000>
ooooo`80003o1@00o`020000o`800?l00`000?l@003o00P0003o2P00o`090000o`80o`001`000?l7
003o00@0003o0P3o00050000o`H00?l03P000?l20?l000`00?l02@000?l20?l000T0003o3_ooool5
0000o`/00?l02`000?l20?l000<0003o0P00o`040000o`030?l000000?l0003o00@0o`000`000?l2
0?l00080003o3@00o`050000o`80o`0000<0003o0?l0003o000010000?l70?l000@0003o00<00?l0
0?l0003o000010000?l20?l000030000o`3o0000o`000080o`000P000?l5003o00T0003o0P3o0009
0000o`800?l00`000?l2003o00koooooB03oP0030?l00100oh000P3o000]0?n001Sooooo000Foooo
o`800?l01@3o00020000o`80o`000P000?l30?l000H0003o4@3o00020000o`d0o`000P000?l70?l0
0080oh0000<0003o0?l0003o00001`3o00020000o`80o`000P000?l30?l000@0003o0`3o00020000
o`@0o`000`000?l40?l000@0003o0`3o00020000o`L0o`000P000?l<0?l000@0003o3_ooool4003o
00<0003o0P3o000@003o00T0003o4000o`0@0000o`80o`000P000?l3003o0080o`000P000?l40?l0
00D00?l010000?l00`00o`00o`000?l00007003o00@0003o0P3o000>003o0080003o0P00o`090000
o`<00?l00P000?l>ooooo`80003o0P00o`000`000?l00?l0003o0007003o00/0003o0P3o00030000
o`@00?l02@000?l20?l000T0003o3000o`0=0000o`<0o`0020000?l00`3o0000003o0000o`0@0000
o`800?l010000?l00`3o0000003o0000o`020000o`80o`000`000?l20?l000H0003o1@00o`0>oooo
ob@0oh000P3o000R0?n000030?l0003oP000oh000080o`002`3oP0020?l00280oh0000<0o`000?n0
003oP0002`3oP00Hooooo`005_ooool2003o00D0o`000P000?l20?l00080003o0`3o00060000oa40
o`000P000?l=0?l00080003o1`3o00020?n000030000o`3o0000o`0000L0o`000P000?l20?l00080
003o0`3o00040000o`<0o`000P000?l40?l000<0003o103o00040000o`<0o`000P000?l70?l00080
003o303o00040000o`kooooo1000o`030000o`80o`004000o`090000oa000?l040000?l20?l00080
003o0`00o`020?l00080003o103o0005003o00@0003o00<00?l00?l0003o00001`00o`040000o`80
o`003P00o`020000o`800?l02@000?l3003o0080003o3_ooool20000o`800?l000<0003o003o0000
o`001`00o`0;0000o`80o`000`000?l4003o00T0003o0P3o00090000o``00?l03@000?l30?l000P0
003o00<0o`000000o`000?l040000?l2003o00@0003o00<0o`000000o`000?l00P000?l20?l000<0
003o0P3o00060000o`D00?l03_oooolT0?n00080o`008P3oP0000`3o0000oh000?n000020?l000/0
oh000P3o000R0?n000030?l0003oP000oh0000/0oh006?ooool001Kooooo0P00o`020000o`030?l0
00000?l0003o00T0o`0010000?l50?l000@0003o1@3o00040000o`h0o`000P000?l70?l00080oh00
0`000?lD0?l00080003o1@3o00020000o`80o`000P000?l>0?l00080003o3P3o00020000o`<0o`00
10000?l>ooooo`L00?l00P000?lG003o0080003o4000o`090000o`@00?l00`000?l20?l000@00?l0
1@000?l40?l000D0003o0P00o`050000o`800?l02`000?l>003o0100003o0P00o`0>ooooo`80003o
5000o`000`000?l00?l0003o0004003o00030000o`00o`000?l000800?l00P000?l3003o00@0003o
0P3o00090000o`80o`0000<0003o003o0000o`003P00o`0?0000o`800?l02@000?l00`00o`00003o
0000o`020000o`T00?l05@000?l2003o0080003o3_oooolh0?n000<0003o0P3o00050?n00080o`00
<P3oP0020?l00180oh006?ooool001Kooooo0P00o`020000o`030?l000000?l0003o00T0o`001000
0?l50?l000@0003o1@3o00040000o`h0o`000P000?l70?l00080oh000`000?lD0?l00080003o1@3o
00020000o`80o`000P000?l>0?l00080003o3P3o00020000o`<0o`0010000?l>ooooo`L00?l00P00
0?lG003o0080003o4000o`090000o`@00?l00`000?l20?l000@00?l01@000?l40?l000D0003o0P00
o`050000o`800?l02`000?l>003o0100003o0P00o`0>ooooo`80003o5000o`000`000?l00?l0003o
0004003o00030000o`00o`000?l000800?l00P000?l3003o00@0003o0P3o00090000o`80o`0000<0
003o003o0000o`003P00o`0?0000o`800?l02@000?l00`00o`00003o0000o`020000o`T00?l05@00
0?l2003o0080003o3_oooolh0?n000<0003o0P3o00050?n00080o`00<P3oP0020?l00180oh006?oo
ool001Kooooo10000?l30?l00080003o403o00020000o`L0o`0010000?l00`3o0000003o0000o`0F
0?l000<0003o103o00070000o`L0o`000P000?l20?l00080003o1@3o00040000o`030?l000000?l0
003o00T0o`0010000?lE0?l00080003o3_ooool40000oa800?l000<0003o003o0000o`007@00o`02
0000o`D00?l01`000?l2003o00<0003o0P3o00060000o`D00?l02@000?l4003o00h0003o2@00o`02
0000o`03003o00000?l0003o00H0003o00<0o`00003o0000o`000P00o`0>ooooo`80003o9@00o`0=
0000o`@00?l000<0003o003o0000o`002@00o`0D0000o`80o`000`000?l4003o00D0003o0P3o0004
0000o`D00?l01@000?l2003o00L0003o2@00o`020000o`kooooo0P3oP0020?l000030?n0003o0000
o`000080oh000P3o000>0?n000T0o`005`3oP0020?l000d0oh0000<0o`000?n0003oP000?`3oP00H
ooooo`005_ooool40000o`030?l000000?l0003o0080003o4P3o00020000o`030?l000000?l0003o
00L0o`000P000?l20?l00080003o503o00030000o`80o`001`000?l70?l00080003o103o00000`00
0?l0o`000?l0000;0?l00080003o1@3o00040000oaL0o`003_ooool2003o00<0003o0P00o`020000
o`h00?l00P000?lM003o01<0003o0P3o00020000o`T00?l050000?l5003o0080003o2@00o`020000
o`03003o00000?l0003o00L0003o0P3o0002003o00kooooo0P000?l20?l00003003o00000?l0003o
0080003o4P00o`020000o`h00?l00P000?l2003o00030?l000000?l0003o00H0003o3000o`040000
o`03003o00000?l0003o00H0003o00<00?l00000o`000?l01`000?l9003o0080o`0010000?l20?l0
0003003o00000?l0003o00D0003o0P00o`0@0000o`80o`000P000?l>ooooo`80oh000P3o000<0?n0
00@0o`001@3oP0020?l001/0oh002@3o000m0?n000030000o`3oP000oh0000/0oh006?ooool001Ko
oooo10000?l00`3o0000003o0000o`020000oa80o`000P000?l00`3o0000003o0000o`070?l00080
003o0P3o00020000oa@0o`000`000?l20?l000L0003o1`3o00020000o`@0o`0000<0003o0?l0003o
00002`3o00020000o`D0o`0010000?lG0?l000kooooo0P00o`030000o`800?l00P000?l>003o0080
003o7@00o`0C0000o`80o`000P000?l9003o01@0003o1@00o`020000o`T00?l00P000?l00`00o`00
003o0000o`070000o`80o`000P00o`0>ooooo`80003o0P3o00000`00o`00003o0000o`020000oa80
0?l00P000?l>003o0080003o0P00o`000`3o0000003o0000o`060000o``00?l010000?l00`00o`00
003o0000o`060000o`03003o00000?l0003o00L0003o2@00o`020?l000@0003o0P3o00000`00o`00
003o0000o`050000o`800?l040000?l20?l00080003o3_ooool20?n00080o`00303oP0040?l000D0
oh000P3o000K0?n000T0o`00?@3oP0000`000?l0oh000?n0000;0?n001Sooooo000Fooooo`80003o
0`3o00020000o`L0o`000P3oP0020000o``0o`000P000?l60?l000<0003o0P3o00040000o`D0o`00
0P000?l90?l00080003o1`3o00000`000?l0o`000?l000020000o`D0o`0010000?l40?l000030000
o`3o0000o`0000T0o`0010000?l30?l000@0003o0P3o00000`000?l0o`000?l0000B0?l00080003o
3_oooolk003o0080003o1`00o`020?l000<0003o0P3o00060000o`03003o00000?l0003o00@00?l0
0`000?l8003o00<0003o1P00o`070000o`<00?l01`000?l20?l00080003o0P3o00050000o`800?l0
3_ooool20000o`80o`006@00o`070000o`L00?l00P000?l3003o00P0003o2P00o`040000o`D00?l0
0P3o00020000o`<00?l02@000?l2003o0080003o0P00o`070000o`D00?l00P000?l20?l000800?l0
00<0003o0?l0003o00000P000?l2003o0080003o00<00?l00?l0003o00000P00o`090000o`kooooo
0P3oP0020?l003L0oh00103o001;0?n001Sooooo000Fooooo`80003o0`3o00020000o`L0o`000P3o
P0020000o``0o`000P000?l60?l000<0003o0P3o00040000o`D0o`000P000?l90?l00080003o1`3o
00000`000?l0o`000?l000020000o`D0o`0010000?l40?l000030000o`3o0000o`0000T0o`001000
0?l30?l000@0003o0P3o00000`000?l0o`000?l0000B0?l00080003o3_oooolk003o0080003o1`00
o`020?l000<0003o0P3o00060000o`03003o00000?l0003o00@00?l00`000?l8003o00<0003o1P00
o`070000o`<00?l01`000?l20?l00080003o0P3o00050000o`800?l03_ooool20000o`80o`006@00
o`070000o`L00?l00P000?l3003o00P0003o2P00o`040000o`D00?l00P3o00020000o`<00?l02@00
0?l2003o0080003o0P00o`070000o`D00?l00P000?l20?l000800?l000<0003o0?l0003o00000P00
0?l2003o0080003o00<00?l00?l0003o00000P00o`090000o`kooooo0P3oP0020?l003L0oh00103o
001;0?n001Sooooo000Fooooo`80003o0`3o00020000oa/0o`000P000?l70?l00080003o1`3o0002
0000o`T0o`000P000?lC0?l00080003o103o00000`000?l0o`000?l000040?l00080003o0`3o0004
0000o`<0o`000P00o`020000oaL0o`000P000?l>ooooocH00?l00`000?l2003o00@0003o0P3o0003
0000o`80o`000P000?l70?l000L00?l010000?l00`00o`00003o0000o`040000o`<00?l00P000?l2
003o00@0003o3P00o`020000o`03003o00000?l0003o0080003o0P3o00050000o`800?l03_ooool4
0000o`030?l000000?l0003o00@0003o0P00o`000`000?l00?l0003o0002003o0080003o2@00o`07
0000o`<00?l010000?l20?l000T0003o3P00o`050000o`@0o`003P000?l2003o0080003o0P00o`07
0000o`L00?l00P000?l3003o0080o`003@000?l3003o00@0003o3_oooolI0?n00080o`00;@3oP000
0`3o0000oh000?n0000Y0?n00080o`00503oP00Hooooo`005_ooool20000o`<0o`000P000?lK0?l0
0080003o1`3o00020000o`L0o`000P000?l90?l00080003o4`3o00020000o`@0o`0000<0003o0?l0
003o0000103o00020000o`<0o`0010000?l30?l000800?l00P000?lG0?l00080003o3_oooolf003o
00<0003o0P00o`040000o`80o`000`000?l20?l00080003o1`3o0007003o00@0003o00<00?l00000
o`000?l010000?l3003o0080003o0P00o`040000o`h00?l00P000?l00`00o`00003o0000o`020000
o`80o`001@000?l2003o00kooooo10000?l00`3o0000003o0000o`040000o`800?l000<0003o003o
0000o`000P00o`020000o`T00?l01`000?l3003o00@0003o0P3o00090000o`h00?l01@000?l40?l0
00h0003o0P00o`020000o`800?l01`000?l7003o0080003o0`00o`020?l000d0003o0`00o`040000
o`kooooo6@3oP0020?l002d0oh0000<0o`000?n0003oP000:@3oP0020?l001@0oh006?ooool001Ko
oooo0P000?l<0?l00080003o1`3o00040000o`L0o`000P000?l70?l00080003o0`3o00020000o`80
o`0010000?lJ0?l00080003o0P3o00050000o`@0o`001@000?l20?l000@0003o0`3o00040000o`T0
o`000`000?l60?l00080oh0000<0003o0?l0003o00000P3o000>oooooaL00?l00P000?lG003o00T0
003o0P00o`060000o`P0o`000P000?l40?l000<00?l00P000?l2003o0080003o0P00o`000`3o0000
003o0000o`020000o`D0o`000P000?l40?l000800?l01`000?l00`00o`00003o0000o`040000o`80
o`0030000?l2003o00kooooo10000?l00`3o0000003o0000o`040000o`800?l000<0003o003o0000
o`000P00o`040000o`L00?l01`000?l3003o00/0003o4000o`0;0000o`800?l02`000?l00`3o0000
0?l0003o0004003o0080003o00<00?l00000o`000?l01000o`070000o`800?l01`000?l2003o00h0
003o3_ooool>0?n00080o`00403oP0020?l00080oh000`3o000R0?n000H0o`00903oP0000`3o0000
oh000?n0000D0?n001Sooooo000Fooooo`80003o303o00020000o`L0o`0010000?l70?l00080003o
1`3o00020000o`<0o`000P000?l20?l000@0003o6P3o00020000o`80o`001@000?l40?l000D0003o
0P3o00040000o`<0o`0010000?l90?l000<0003o1P3o00020?n000030000o`3o0000o`000080o`00
3_oooolG003o0080003o5`00o`090000o`800?l01P000?l80?l00080003o103o0003003o0080003o
0P00o`020000o`800?l000<0o`000000o`000?l00P000?l50?l00080003o103o0002003o00L0003o
00<00?l00000o`000?l010000?l20?l000`0003o0P00o`0>ooooo`@0003o00<0o`000000o`000?l0
10000?l2003o00030000o`00o`000?l000800?l010000?l7003o00L0003o0`00o`0;0000oa000?l0
2`000?l2003o00/0003o00<0o`00003o0000o`001000o`020000o`03003o00000?l0003o00@00?l0
1`000?l2003o00L0003o0P00o`0>0000o`kooooo3P3oP0020?l00100oh000P3o00020?n000<0o`00
8P3oP0060?l002@0oh0000<0o`000?n0003oP000503oP00Hooooo`005_ooool20000oaD0o`001P00
0?l30?l000@0003o1`3o00020000o`/0o`000`000?l;0?l000<0003o7@3o00040000o`D0o`000P00
0?l70?l00080003o2`3o00000`000?l0o`000?l000020?l000kooooo1`00o`020000oaD00?l00P00
0?l2003o00@0003o00<00?l00000o`000?l03`000?l00`3o0000003o0000o`040000o`X0o`000P3o
P0040?l000800?l00`000?l6003o00D0003o103o00050000o``00?l050000?l2003o00kooooo1000
0?l00`3o0000003o0000o`06003o00030000o`00o`000?l000L0003o1000o`090000o`L00?l01`00
0?lB003o01/0003o1000o`050000o``00?l00P000?l2003o00@0003o00<00?l00000o`000?l01`00
o`040000o`kooooo3P3oP0020?l00640oh000P3o000G0?n001Sooooo000Fooooo`80003o3P3o0002
0000o`80o`000`000?l20?l000@0003o0`3o00040000o`L0o`000P000?l20?l000030?n0003o0000
o`0000L0o`000P00o`0;0?l000D0003o0P3o00040?n000<0003o2`3o00070000o`@0o`000P000?l3
0?l00080003o3P3o00020000o`@0o`0000<0003o0?l0003o00000P3o000>ooooo`800?l00P3o0003
0000o`80o`004P00o`050000o`T00?l01`000?l2003o0180003o1@3o00020?n000@0003o0P00o`00
0`000?l00?l0003o00020000o`800?l00P000?l00`3o0000003o0000o`060000o`030?l000000?l0
003o00@0003o1`00o`090000o`800?l000<0003o003o0000o`000P00o`070000o`800?l03_ooool5
0000o`P00?l000<0003o003o0000o`001`000?l6003o00030000o`00o`000?l000@0003o0P00o`00
0`3o0000003o0000o`02003o0080o`000`000?l20?l000@00?l00P000?l00`00o`00o`000?l00002
003o0080003o1`00o`030000o`800?l01P000?l70?l000800?l00`000?l4003o0080003o00<00?l0
0?l0003o00000P000?l4003o00030000o`3o0000o`000080003o2`00o`030000o`@0o`000`000?l2
0?l00080003o3_oooolb0?n00080o`000P3oP0020?l001`0oh000P3o000W0?n00080o`002`3oP00H
ooooo`005_ooool20000o`h0o`000P000?l20?l000<0003o0P3o00040000o`<0o`0010000?l70?l0
0080003o0P3o00000`3oP000o`000?l000070?l000800?l02`3o00050000o`80o`00103oP0030000
o`/0o`001`000?l40?l00080003o0`3o00020000o`h0o`000P000?l40?l000030000o`3o0000o`00
0080o`003_ooool2003o0080o`000`000?l20?l001800?l01@000?l9003o00L0003o0P00o`0B0000
o`D0o`000P3oP0040000o`800?l000<0003o003o0000o`000P000?l2003o0080003o00<0o`000000
o`000?l01P000?l00`3o0000003o0000o`040000o`L00?l02@000?l2003o00030000o`00o`000?l0
00800?l01`000?l2003o00kooooo1@000?l8003o00030000o`00o`000?l000L0003o1P00o`000`00
0?l00?l0003o00040000o`800?l000<0o`000000o`000?l00P00o`020?l000<0003o0P3o0004003o
0080003o00<00?l00?l0003o00000P00o`020000o`L00?l00`000?l2003o00H0003o1`3o0002003o
00<0003o1000o`020000o`03003o003o0000o`000080003o1000o`000`000?l0o`000?l000020000
o`/00?l00`000?l40?l000<0003o0P3o00020000o`kooooo<P3oP0020?l00080oh000P3o000L0?n0
0080o`009`3oP0020?l000/0oh006?ooool001Kooooo10000?l:0?l00080003o1P3o00000`000?l0
o`000?l000090?l00080003o1@3o00040000o`L0o`000P000?l@0?l000<0003o203o00030000o`d0
o`001@000?l40?l000800?l00`3o00040000o`T0o`0000<0003o0?l0003o00002`3o000>ooooo`80
003o0P3o00030000o`80o`003@00o`0>0000o`<00?l04P000?l20?l000H0003o00<0o`000000o`00
0?l00P3o00030?n00080o`0010000?l;003o0100003o0`00o`020000o`L00?l00P000?l7003o00T0
003o0P00o`0>ooooo`D0003o1P00o`0>0000o`@00?l01`000?l2003o00X0003o0P3o0002003o0080
o`000`000?l20?l000@00?l00`000?l20?l000T00?l04P000?l20?l000<00?l010000?l2003o0080
o`0020000?l20?l000@00?l00`3o00020000o`@0o`001`000?l>ooooof40oh000P3o000I0?n000<0
o`002`3oP00Hooooo`005_ooool40000o`X0o`000P000?l60?l000030000o`3o0000o`0000T0o`00
0P000?l50?l000@0003o1`3o00020000oa00o`000`000?l80?l000<0003o3@3o00050000o`@0o`00
0P00o`030?l000@0003o2@3o00000`000?l0o`000?l0000;0?l000kooooo0P000?l20?l000<0003o
0P3o000=003o00h0003o0`00o`0B0000o`80o`001P000?l00`3o0000003o0000o`020?l000<0oh00
0P3o00040000o`/00?l040000?l3003o0080003o1`00o`020000o`L00?l02@000?l2003o00kooooo
1@000?l6003o00h0003o1000o`070000o`800?l02P000?l20?l000800?l00P3o00030000o`80o`00
1000o`030000o`80o`002@00o`0B0000o`80o`000`00o`040000o`800?l00P3o00080000o`80o`00
1000o`030?l00080003o103o00070000o`koooooH@3oP0020?l001T0oh000`3o000;0?n001Sooooo
000Fooooo`80003o0P3o00030000o`80o`000P000?l30?l00080003o1P3o00050000o`/0o`0000<0
003o0?l0003o00000P3o00020000o`80o`001`000?l30?l00080003o3@3o00000`000?l0o`000?l0
00070?l00080003o103o00030000o`@0o`0010000?l00`3o0000003o0000o`040?l000T0003o1`3o
00030000o`P0o`000`000?l20?l000kooooo10000?l00`3o00000?l0003o000F003o00D0003o5`00
o`020?l000@0003o0P3o00030000o`80o`000`000?l20?n000@0003o0P3o0005003o00h0003o0P3o
0004003o0080003o3000o`020000o`L00?l00P000?l2003o00D0003o3_ooool40000o`030?l00000
o`000?l000800?l00P000?l2003o00L0003o2@00o`030000o`80o`001000o`000`000?l00?l0003o
0002003o00L0003o0P00o`020?l000h0003o2@00o`0D0000o`800?l01`000?l20?l000`0003o0P3o
00030000o`80o`002`000?l>ooooocl0oh000P3o000`0?n00080003o5`3oP00Hooooo`005_ooool2
0000o`80o`000`000?l20?l00080003o0`3o00020000o`H0o`001@000?l;0?l000030000o`3o0000
o`000080o`000P000?l20?l000L0003o0`3o00020000o`d0o`0000<0003o0?l0003o00001`3o0002
0000o`@0o`000`000?l40?l000@0003o00<0o`000000o`000?l0103o00090000o`L0o`000`000?l8
0?l000<0003o0P3o000>ooooo`@0003o00<0o`00003o0000o`005P00o`050000oaL00?l00P3o0004
0000o`80o`000`000?l20?l000<0003o0P3oP0040000o`80o`001@00o`0>0000o`80o`001000o`02
0000o``00?l00P000?l7003o0080003o0P00o`050000o`kooooo10000?l00`3o00000?l0003o0002
003o0080003o0P00o`070000o`T00?l00`000?l20?l000@00?l000<0003o003o0000o`000P00o`07
0000o`800?l00P3o000>0000o`T00?l050000?l2003o00L0003o0P3o000<0000o`80o`000`000?l2
0?l000/0003o3_oooolo0?n00080o`00<03oP0020000oaL0oh006?ooool001Kooooo103o00030000
o`T0o`000P000?l40?l000D0003o2`3o00000`000?l0o`000?l000020?l00080003o0P3o00000`00
0?l0o`000?l000040000o`<0o`000P000?l20?l000@0003o3P3o00020000o`030?l000000?l0003o
0080003o0P3o00030000o`@0o`0010000?l30?l000T0003o0P3o00030000o`@0o`0010000?l70?l0
0080003o1@3o000>ooooo`T0003o1`00o`020000o`T00?l01`000?l@003o0080003o1@00o`020?l0
00@0003o0`3o00020000o`D0o`000P000?l20?l000@0003o00<0o`000000o`000?l01P000?l00`00
o`00003o0000o`020000o`80o`000`000?l20?l000@00?l00`000?l4003o0080003o00<00?l00000
o`000?l00P00o`040000o`h00?l03_ooool70000oa@00?l00P3o00000`000?l00?l0003o00020?l0
0080003o0P00o`0@0000o`<00?l00P000?l2003o00L0003o2@00o`050000o`@0o`002`000?l3003o
0180003o1`3o00060000o`800?l01@000?l>oooooaT0oh000P3o000P0?n000H0o`004`3oP0020000
oc@0oh006?ooool001Kooooo1@000?lA0?l000D0003o0`3o00020000o`l0o`001@000?l50?l00080
003o103o00020000o`h0o`000P000?l70?l000<0003o103o00070000o`800?l0103o00000`000?l0
o`000?l0000>0?l00080003o103o00050000o`80o`003_ooool70000oa@00?l01@000?lI003o0080
o`002@000?l50?l00080003o103o00020000o`<0o`000P00o`090000o`80o`001@000?l20?l00080
0?l00`000?l2003o00D0003o0P00o`020000o`d00?l01@000?l>ooooo`L00?l01`000?l?003o0003
0000o`00o`000?l000800?l00P000?l5003o00T0003o1000o`0@0000o`800?l00`000?l20?l00080
003o0P3o00030000o`@0o`000`000?l20?l000@0003o0P3o00000`00o`00003o0000o`040000o`80
o`000`00o`040000o`03003o003o0000o`000080003o0P3o00020000o`<0o`000P000?l20?l00080
0?l01@000?l>oooooaH0oh000`3o00070?n000T0o`006`3oP0020?l000h0oh000P3o000d0?n001So
oooo000Fooooo`D0003o4@3o00050000o`<0o`000P000?l?0?l000D0003o1@3o00020000o`@0o`00
0P000?l>0?l00080003o1`3o00030000o`@0o`001`000?l2003o00@0o`0000<0003o0?l0003o0000
3P3o00020000o`@0o`001@000?l20?l000kooooo1`000?lD003o00D0003o6@00o`020?l000T0003o
1@3o00020000o`@0o`000P000?l30?l000800?l02@000?l20?l000D0003o0P3o0002003o00<0003o
0P00o`050000o`800?l00P000?l=003o00D0003o3_ooool7003o00L0003o3`00o`000`000?l00?l0
003o0002003o0080003o1@00o`090000o`@00?l040000?l2003o00<0003o0P3o00020000o`80o`00
0`000?l40?l000<0003o0P3o00040000o`80o`0000<00?l00000o`000?l010000?l20?l000<00?l0
10000?l00`00o`00o`000?l000020000o`80o`000P000?l30?l00080003o0P3o0002003o00D0003o
3_oooolF0?n000<0o`001`3oP0090?l001/0oh000P3o000>0?n00080o`00=03oP00Hooooo`005_oo
ool40000o`L0o`000`000?l80?l00003003o003o0000o`000080003o2`3o00000`000?l0o`000?l0
00020000o`L0o`000P000?l50?l000P0003o0`3o00050000oa40o`0000<0003o0?l0003o00001P00
0?l30?l000@0003o0`3o00020000o`h0o`000P00o`060?l000<0003o0P3o000>ooooo`80003o1`00
o`040000ob/00?l000<0003o0?l0003o00001P000?l00`3o0000003o0000o`040000o`030?l00000
0?l0003o01@0003o0P3o00050000o`@0o`002@000?l00`3o0000003o0000o`0D003o00kooooo5P00
o`000`3o0000003o0000o`090000o`800?l00P000?l00`00o`00003o0000o`090000o`/00?l03000
0?l2003o0080o`000P00o`020000o`030?l000000?l0003o00@0o`000`000?l20?l00080003o103o
0005003o00/0003o00<0o`000000o`000?l0103o00070000o`@00?l01@000?l>ooooo`@0oh0000<0
o`000000o`000?l0P`3oP00Hooooo`005_ooool40000o`L0o`000`000?l80?l00003003o003o0000
o`000080003o2`3o00000`000?l0o`000?l000020000o`L0o`000P000?l50?l000P0003o0`3o0005
0000oa40o`0000<0003o0?l0003o00001P000?l30?l000@0003o0`3o00020000o`h0o`000P00o`06
0?l000<0003o0P3o000>ooooo`80003o1`00o`040000ob/00?l000<0003o0?l0003o00001P000?l0
0`3o0000003o0000o`040000o`030?l000000?l0003o01@0003o0P3o00050000o`@0o`002@000?l0
0`3o0000003o0000o`0D003o00kooooo5P00o`000`3o0000003o0000o`090000o`800?l00P000?l0
0`00o`00003o0000o`090000o`/00?l030000?l2003o0080o`000P00o`020000o`030?l000000?l0
003o00@0o`000`000?l20?l00080003o103o0005003o00/0003o00<0o`000000o`000?l0103o0007
0000o`@00?l01@000?l>ooooo`@0oh0000<0o`000000o`000?l0P`3oP00Hooooo`00oooooooooooo
oh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003o
ooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003ooooooooooooo
P_ooool002;oooooL_l0000Wooooog7o00009ooooomao`0002OoooooL_l0000Sooooo`008_ooool0
0ol0003oooooooooo`03oooooaKoo`001OoooolFo`3o00Cooooo5ol00?l4oooooaOoogl01?ooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5_oo0005oooooaKo0?l01OoooolF
o`00o`Cooooo5oooO`04ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolG
ool000Cooooo5_l0o`05oooooaKo003o1OoooolFoomo00Cooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`02oooooaOoo`001?oooolGo`3o00Cooooo5_l00?l5oooooaKoogl01Oooool0
0ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`001OoooolFo`3o
00Cooooo5ol00?l4oooooaOoogl01?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
00;ooooo5_oo0005oooooaKo0?l01OoooolFo`00o`Cooooo5oooO`04ooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool00_oooolGool000Cooooo5_l0o`05oooooaKo003o1OoooolFoomo
00Cooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02oooooaOoo`001?oooolGo`3o
00Cooooo5_l00?l5oooooaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003o
ooooooooo`03oooooaKoo`001OoooolFo`3o00Cooooo5ol00?l4oooooaOoogl01?ooool00ol0003o
ooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5_oo0005oooooaKo0?l01OoooolFo`00o`Co
oooo5oooO`04ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolGool000Co
oooo5_l0o`05oooooaKo003o1OoooolFoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`02oooooaOoo`001?oooolGo`3o00Cooooo5_l00?l5oooooaKoogl01Oooool00ol0003o
ooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`001OoooolFo`3o01oooooo
5oooO`04ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolFool000Gooooo
5_l0o`05oooooaKo003o1?oooolGoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`02oooooaOoo`001?oooolFo`3o00Gooooo5_l00?l5oooooaKoogl01?ooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo0004oooooaOo0?l01?oooolFo`00o`Gooooo
5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo
0005oooooaKo0?l07ooooolGoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`02oooooaKoo`001OoooolFo`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool00_oooolGool000Cooooo5_l0o`05oooooaKo003o1OoooolFoomo00Cooooo00?o
0000ooooooooool09Oooool00ol0003oooooooooo`02oooooaOoo`007ooooolFo`00o`Gooooo5_oo
O`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo0005
oooooaKo0?l07ooooolGoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02
oooooaKoo`001OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02
oooooaOoo`001?oooolFo`3o00Gooooo5_l00?l5oooooaKoogl01?ooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo00;ooooo5ooo000OoooooaKo003o1OoooolFoomo00Gooooo00?o0000
ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool000Gooooo5_l0o`0Ooooo
oaOoogl01?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5_oo0005oooo
oaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo0004oooo
oaKo0?l01OoooolFo`00o`Gooooo5_ooO`04ooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool00_oooolGool001oooooo5_l00?l5oooooaKoogl01Oooool00ol0003oooooooooo`0Qoooo
o`008_ooool00ol0003oooooooooo`03oooooaKoo`00>_oooolGoomo00Cooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`02oooooaKoo`001OoooolFo`3o03[ooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`02oooooaOoo`001?oooolFo`3o00Gooooo5_l00?l5oooooaKo
ogl01?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo000OoooooaKo
003o1OoooolFoomo00Gooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool0
0ooooolFool003[ooooo5oooO`04ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
0_oooolFool000Gooooo5_l0o`0jooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
0_oooolGool000Cooooo5_l0o`05oooooaKo003o1OoooolFoomo00Cooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`02oooooaOoo`007ooooolFo`00o`Gooooo5_ooO`05ooooo`03o`00
0?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oo
oooooooo02Gooooo00?o0000ooooooooool00_oooolFool000Gooooo5_l0o`0jooooo`03o`000?oo
oooooooo02Gooooo00?o0000ooooooooool00_oooolGool000Cooooo5_l0o`05oooooaKo003o1Ooo
oolFoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02oooooaOoo`007ooo
oolFo`00o`Gooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooo
oooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolF
ool000Gooooo5_l0o`0jooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolG
ool001oooooo5_l00?l5oooooaKoogl01?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo00;ooooo5ooo000joooooaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0
003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
00;ooooo5_oo0005oooooaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
00;ooooo5ooo000OoooooaKo003o1OoooolFoomo00Cooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`02oooooaOoo`00>_oooolFoomo00Gooooo00?o0000ooooooooool08Oooool002;o
oooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`02oooooaKoo`001OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`02oooooaOoo`007ooooolFo`00o`Gooooo5_ooO`04ooooo`03o`000?oooooooooo02Go
oooo00?o0000ooooooooool00_oooolGool003[ooooo5_ooO`05ooooo`03o`000?oooooooooo027o
oooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool00_oooolFool000Gooooo5_l0o`0jooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool00_oooolGool001oooooo5_l00?l5oooooaKoogl01?ooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo000joooooaKoogl01Oooool00ol0003ooooo
ooooo`0Qooooo`008_ooool00ol0003o001oo`00O`03o`00OaKoo`00EOl007l00ol0003ooooooooo
o`0Uooooo`03o`000?l007oo001o00;o001o5_oo0005o`00OaKo0?l0>_l007l00ol0003ooooooooo
o`0Uooooo`03o`000?l007oo001o00;o001o5ooo000Oo`00OaKo003o1Ol007lFoomo00Co001o00?o
0000ooooooooool09Oooool00ol0003o001oo`00O`02o`00OaOoo`00>_l007lFoomo00Go001o00?o
0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`02oooooaKoo`001OoooolFo`3o03[ooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`02oooooaOoo`00>_oooolFoomo00Cooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`02oooooaOoo`00>_oooolFoomo00Gooooo00?o0000
ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`02oooooaKoo`001OoooolFo`3o03[ooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`02oooooaOoo`00>_oooolFoomo00Cooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`1CoooooaKoogl01Oooool00ol0003oooooooooo`0Qoooo
o`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo00;ooooo5_oo0005oooooaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo00;ooooo5ooo001Dooooo`03o`000?oooooooooo02Gooooo00?o0000oooooooo
ool0DooooolFoomo00Gooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool0
0ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02oooooaKoo`00
1OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02oooooaOoo`00
E?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`00
0?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oo
oooooooo02Gooooo00?o0000ooooooooool00_oooolFool000Gooooo5_l0o`0jooooo`03o`000?oo
oooooooo02Gooooo00?o0000ooooooooool00_oooolGool005Cooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`1CoooooaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool0
0ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo00;ooooo5_oo0005oooooaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo00;ooooo5ooo001Dooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolF
oomo00Gooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool0
05Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo001Dooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0DooooolFoomo00Gooooo00?o0000ooooooooool08Oooool002;o
oooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`0MoooooaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;o
oooo5ooo001Dooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Go
oooo00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo001Dooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool0DooooolFoomo00Gooooo00?o0000ooooooooool08Oooool002;ooooo00?o
0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`0MoooooaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo
001Dooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Gooooo00?o
0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo00;ooooo5ooo001Dooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool0DooooolFoomo00Gooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000oooo
ooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0Moooo
oaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo001Doooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Gooooo00?o0000oooo
ooooool08Oooool002;ooooo00?o0000o`00Ool007l00ol007lFool005Go001o00?o0000oooooooo
ool09Oooool00ol0003o001oo`00O`0Mo`00OaKo0?l0>_l007l00ol0003oooooooooo`0Uooooo`03
o`000?l007oo001o00;o001o5ooo001Do`00O`03o`000?oooooooooo02Gooooo00?o0000o`00Ool0
07l0Dol007lFoomo00Go001o00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool0
0ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0
>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo001Dooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Gooooo00?o0000ooooooooool0
8Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo00;ooooo5ooo001Dooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0Dooo
oolFoomo00Gooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolF
ool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`1CoooooaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0
003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
01gooooo5_l0o`0jooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo
027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Go
oooo00?o0000ooooooooool07OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Go
oooo00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`1CoooooaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003ooooooooo
o`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0
o`0jooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000R
ooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool07OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]
ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Gooooo00?o0000
ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1Coooo
oaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKo
o`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`0jooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`00
0?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
7OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Gooooo00?o0000ooooooooool0
8Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1CoooooaKoogl01Ooo
ool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003o001oo`00O`03o`00OaKoo`00EOl007l0
0ol0003oooooooooo`0Uooooo`03o`000?l007oo001o01go001o5_l0o`0jo`00O`03o`000?oooooo
oooo02Gooooo00?o0000o`00Ool007l0KOl007l00ol0003oooooooooo`0Uooooo`03o`000?l007oo
001o05?o001o5_ooO`05o`00O`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo
00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`1CoooooaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003o
ooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06go
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Go
oooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003ooooo
ooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003ooooooooo
o`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o
0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03
oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooo
oaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKo
o`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000oooooooo
ool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00
EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOoo
ool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_oo
ool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool0
0ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0
003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo02Go
oooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06go
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo027o
oooo000Rooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool08Oooool002;ooooo00?o
0000o`00Ool007l0K_l007l00ol0003oooooooooo`0Uooooo`03o`000?l007oo001o06go001o00?o
0000ooooooooool09Oooool00ol0003o001oo`00O`1]o`00O`03o`000?oooooooooo02Gooooo00?o
0000o`00Ool007l0K_l007l00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`1^
ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1^
ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo06kooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06kooooo00?o0000oooo
ooooool08Oooool002;ooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`00
8_ooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oo
oooooooo06kooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oo
oooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo06kooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool0K_ooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool0
0ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo
027ooooo000Rooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003o
ooooooooo`0Uooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool08Oooool002;ooooo
00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003ooooooooo
o`1^ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`1^ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo06kooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06kooooo00?o0000
ooooooooool08Oooool002;ooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]oooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qoooo
o`008_ooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo02Gooooo00?o0000oooooooo
ool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo027ooooo000Rooooog;o0000
9ooooomao`0002OoooooLOl0000Wooooog;o00008oooool00?oooooooooooon2ooooo`00oooooooo
oooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo
003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003ooooooooo
ooooP_ooool001Kooooo0P3o00020000o`03003o003o0000o`0000H00?l02@3o00000`3oP000o`00
0?l0000@0?l000@00?l0303o00020000o`80oh00103o00000`3oP000o`000?l0000;0?l00080oh00
1@000?l>0?l00080oh002@3o0002003o00T0o`000P3oP0030?l00080oh00103o00000`3oP000o`00
0?l000020000o`kooooo0P3oP0020000o`X00?l02@3o00020?n000D00?l01P3o00070000oa@00?l0
0`3o00020000o`T00?l01@3o0004003o0080o`005`00o`040?l000L00?l02@3o00050?n00080003o
3_ooool5003o00P0o`0000<0oh000?l0003o00001`3o00020?n000@0o`0000<0oh000?l0003o0000
0P3o00050000o`80oh00103o00020000o`<00?l00P3o0007003o00@0o`000P3oP0070?l000<00?l0
1P3o00000`3oP000o`000?l000060?l000030?n0003o0000o`0000H0003o00<0oh000000o`000?l0
0P00o`050?l000L0003o0P3o00020?n00080003o0P3oP0050000o`80oh000P3o00030000o`80oh00
3_ooool20?n000<0o`000P3oP0040?l000<0oh000P3o000=0?n000030?l000000?l0003o0080o`00
0P00o`020?n000030?l0003oP000oh0000@0o`000P3oP0000`3o0000oh000?n000020?n00080o`00
303oP0020?l00080oh000P3o00000`3oP000o`000?l000090?n000@0o`000P00o`000`000?l0o`00
0?l000020?l000@0oh001@3o00020?n00080o`0000<0oh000?l0003o00000P3o00020?n000030?l0
003oP000oh000080003o2@3oP0020?l000<0oh000P3o00020?n001Sooooo000Fooooo`80o`000P00
0?l00`00o`00o`000?l00006003o00T0o`0000<0oh000?l0003o0000403o0004003o00`0o`000P00
0?l20?n000@0o`0000<0oh000?l0003o00002`3o00020?n000D0003o3P3o00020?n000T0o`000P00
o`090?l00080oh000`3o00020?n000@0o`0000<0oh000?l0003o00000P000?l>ooooo`80oh000P00
0?l:003o00T0o`000P3oP005003o00H0o`001`000?lD003o00<0o`000P000?l9003o00D0o`001000
o`020?l001L00?l0103o0007003o00T0o`001@3oP0020000o`kooooo1@00o`080?l000030?n0003o
0000o`0000L0o`000P3oP0040?l000030?n0003o0000o`000080o`001@000?l20?n000@0o`000P00
0?l3003o0080o`001`00o`040?l00080oh001`3o0003003o00H0o`0000<0oh000?l0003o00001P3o
00000`3oP000o`000?l000060000o`030?n000000?l0003o00800?l01@3o00070000o`80o`000P3o
P0020000o`80oh001@000?l20?n00080o`000`000?l20?n000kooooo0P3oP0030?l00080oh00103o
00030?n00080o`003@3oP0000`3o0000003o0000o`020?l000800?l00P3oP0000`3o0000oh000?n0
00040?l00080oh0000<0o`000?n0003oP0000P3oP0020?l000`0oh000P3o00020?n00080o`0000<0
oh000?l0003o00002@3oP0040?l000800?l000<0003o0?l0003o00000P3o00040?n000D0o`000P3o
P0020?l000030?n0003o0000o`000080o`000P3oP0000`3o0000oh000?n000020000o`T0oh000P3o
00030?n00080o`000P3oP00Hooooo`005_ooool20?l00080003o0`3o0004003o0080oh002@3o0000
0`3oP000o`000?l000020?n000L0o`00103oP0030?l000800?l00P3o0002003o00X0o`000P000?l2
0?n000`0o`000P3oP0040?l000D0003o0P3oP00D0?l000D00?l01@3o00020?n000@0o`000P3oP003
0?l00080oh000P3o00020000o`D0oh003_ooool20?n00080003o2@00o`0<0?l000L00?l00P3o0002
003o00L0003o5000o`030?l00080003o2@00o`050?l002H00?l02`3o00050?n00080o`003_ooool5
003o00P0o`0000<0003o0?l0003o00000P3o0002003o00X0o`000P00o`020?n000D0003o0P3oP007
0?l000800?l0103o0007003o0080oh000P3o00000`3oP000o`000?l000040?l000<00?l0103oP002
0?l000<0oh001P3o00090000o`030?n0003o0000o`0000@0o`000P3oP0000`3o0000003o0000o`0@
0?n00080o`000P3oP0050000o`80oh003_ooool20?n000<0o`000P00o`020?l000T0oh000P3o0003
0?n00080003o0P3oP0030?l000@0003o0P00o`020?l000<0oh000P3o00020?n000<0o`000P000?l4
0?l000D0oh00103o0002003o00<0oh000P3o000>0?n00080003o103o00000`3oP000o`000?l00002
0?l000D0oh000P3o00040?n00080o`000`3oP0020?l00080oh0000<0o`000?n0003oP000503oP00H
ooooo`005_ooool20?l00080003o0`3o0004003o0080oh002@3o00000`3oP000o`000?l000020?n0
00L0o`00103oP0030?l000800?l00P3o0002003o00X0o`000P000?l20?n000`0o`000P3oP0040?l0
00D0003o0P3oP00D0?l000D00?l01@3o00020?n000@0o`000P3oP0030?l00080oh000P3o00020000
o`D0oh003_ooool20?n00080003o2@00o`0<0?l000L00?l00P3o0002003o00L0003o5000o`030?l0
0080003o2@00o`050?l002H00?l02`3o00050?n00080o`003_ooool5003o00P0o`0000<0003o0?l0
003o00000P3o0002003o00X0o`000P00o`020?n000D0003o0P3oP0070?l000800?l0103o0007003o
0080oh000P3o00000`3oP000o`000?l000040?l000<00?l0103oP0020?l000<0oh001P3o00090000
o`030?n0003o0000o`0000@0o`000P3oP0000`3o0000003o0000o`0@0?n00080o`000P3oP0050000
o`80oh003_ooool20?n000<0o`000P00o`020?l000T0oh000P3o00030?n00080003o0P3oP0030?l0
00@0003o0P00o`020?l000<0oh000P3o00020?n000<0o`000P000?l40?l000D0oh00103o0002003o
00<0oh000P3o000>0?n00080003o103o00000`3oP000o`000?l000020?l000D0oh000P3o00040?n0
0080o`000`3oP0020?l00080oh0000<0o`000?n0003oP000503oP00Hooooo`005_ooool20?l00080
003o1@00o`020?l00080oh000`3o00020?n00080o`000P3oP0030?l000@0oh000`3o00020000o`80
oh005@3o00040?n000`0o`000P3oP0020?l000@0oh0000<0003o0?l0003o00000P3oP0020?l00080
oh002P3o00020?n000@0o`001`00o`050?l000H0oh000`3o00020?n000@0o`001@3oP00>ooooo`80
oh000P000?l9003o00X0o`003@00o`050000o`80oh005000o`030?l00080003o2@00o`050?l002H0
0?l01@3o00020?n000@0o`001`3oP00>ooooo`L00?l01P3o00000`000?l0o`000?l000040?l00080
0?l0203o0002003o0080o`0010000?l30?n000H0o`0000<0oh000?l0003o00000P3o00020?n000D0
o`000P00o`020000o`80o`0000<0oh000?l0003o00001@3o0002003o00@0003o2@3oP0020?l000D0
003o1@3oP0060?l000D0003o1@3oP0040?l000D0oh00103o00050000o`80oh000P000?l>ooooo`80
oh000`3o00040?n00080o`003P3oP0020000o`<0o`0010000?l2003o0080oh0000<0o`000?n0003o
P0001P3oP0000`3o0000oh000?n00002003o0080oh000P3o00030?n00080o`000P3oP0020?l000D0
oh000P3o00000`3oP000o`000?l000020000o`L0oh001P3o00000`3oP000o`000?l000070?n00080
o`00103oP0020?l000<0oh000P3o00030?n00080o`00103oP0020000o`030?n000000?l0003o00/0
oh006?ooool001Kooooo1@00o`0;0?l00080oh001`3o00020?n00080003o1@3o00020?n00080o`00
00<0oh000?l0003o00000P3o0002003o00`0o`00103oP00;0?l000030?n000000?l0003o0080003o
103oP0000`000?l0o`000?l000040?l00080oh002P3o00020?n000@0o`000P000?l5003o00<0oh00
0P3o00020?n00080003o0P3oP0030?l00080oh000P3o00020?n000D0o`003_ooool20?n000`00?l0
0P3o0002003o00D0o`002@00o`040?l000D0003o0P3oP00;003o00<0o`001P00o`030?l00080003o
2@00o`050?l000@00?l0103o0007003o0080o`004`00o`020?l000L0003o103o00050?n00080o`00
3_ooool20?n00080o`001@00o`0D0?l000030000o`00o`000?l00080o`000P3oP0020000o`<0oh00
1P000?l00`3oP000o`000?l0000=0?l000<0003o0P3oP0070?l000H0003o1`3oP0020?l000030?n0
00000?l0003o0080003o1@3oP0060?l000L0003o0P3o00000`00o`00oh000?n000060?l00003003o
003o0000o`000080o`002@3oP00>ooooo`@0oh0000<0003o0?n0003oP0000P3o00090?n00080o`00
0`3oP0020?l00080003o1@3o00020000o`800?l00P3oP0000`3o0000oh000?n000090?n000@0o`00
0P00o`000`3o0000oh000?n0000>0?n000800?l00P000?l40?n000030?l0003oP000oh000080oh00
0P00o`0>0?n00080o`001`3oP002003o0080o`0000<0003o0?n0003oP000103oP0020000o`h0oh00
6?ooool001Kooooo1@00o`0;0?l00080oh001`3o00020?n00080003o1@3o00020?n00080o`0000<0
oh000?l0003o00000P3o0002003o00`0o`00103oP00;0?l000030?n000000?l0003o0080003o103o
P0000`000?l0o`000?l000040?l00080oh002P3o00020?n000@0o`000P000?l5003o00<0oh000P3o
00020?n00080003o0P3oP0030?l00080oh000P3o00020?n000D0o`003_ooool20?n000`00?l00P3o
0002003o00D0o`002@00o`040?l000D0003o0P3oP00;003o00<0o`001P00o`030?l00080003o2@00
o`050?l000@00?l0103o0007003o0080o`004`00o`020?l000L0003o103o00050?n00080o`003_oo
ool20?n00080o`001@00o`0D0?l000030000o`00o`000?l00080o`000P3oP0020000o`<0oh001P00
0?l00`3oP000o`000?l0000=0?l000<0003o0P3oP0070?l000H0003o1`3oP0020?l000030?n00000
0?l0003o0080003o1@3oP0060?l000L0003o0P3o00000`00o`00oh000?n000060?l00003003o003o
0000o`000080o`002@3oP00>ooooo`@0oh0000<0003o0?n0003oP0000P3o00090?n00080o`000`3o
P0020?l00080003o1@3o00020000o`800?l00P3oP0000`3o0000oh000?n000090?n000@0o`000P00
o`000`3o0000oh000?n0000>0?n000800?l00P000?l40?n000030?l0003oP000oh000080oh000P00
o`0>0?n00080o`001`3oP002003o0080o`0000<0003o0?n0003oP000103oP0020000o`h0oh006?oo
ool001Kooooo1`00o`0B0?l00080003o0P00o`050?l00080oh001`3o00020?n000`0o`00103oP007
0?l00080oh000P3o00000`3oP000003o0000o`020000o`@0oh001`3o00020?n000X0o`000P3oP007
0?l000@00?l00P3o00030?n00080003o0P3oP0090?l000<0oh00103o000>ooooo`80o`001@00o`04
0?l000<00?l02@3o0009003o0080o`000P00o`050000o`80oh002`00o`020?l000D00?l01@3o0004
003o00D0o`000P00o`050?l000800?l01P3o0003003o00@0o`000`00o`0;0?l000T00?l01@000?l2
0?n000@0o`000P3oP0000`3o0000oh000?n000020?l000kooooo103oP0000`00o`00o`000?l00002
003o00D0o`000P3oP0040?l000<0oh001`3o0002003o00T0oh001P000?l00`3oP000o`000?l00002
003o00D0o`000P00o`060?l000D0oh000`3o00020?n000L0003o2`3oP0020?l00080oh000P3o0000
0`3oP000o`000?l000060?l000<0oh002`3o00020?n000030?l0003oP000oh0000@0o`001`3oP00>
ooooo`80oh000P3o000>0?n00080o`000P3oP0000`3o0000oh000?n000020?n000D0o`000P000?l7
003o00L0o`00103oP0020?l000<0oh001P000?l20?n000<0o`000P3oP0050?l000T0003o0P3oP002
0?l00080oh0000<0o`000?n0003oP0004P3oP0040?l001L0oh006?ooool001Kooooo1`00o`0B0?l0
0080003o0P00o`050?l00080oh001`3o00020?n000`0o`00103oP0070?l00080oh000P3o00000`3o
P000003o0000o`020000o`@0oh001`3o00020?n000X0o`000P3oP0070?l000@00?l00P3o00030?n0
0080003o0P3oP0090?l000<0oh00103o000>ooooo`80o`001@00o`040?l000<00?l02@3o0009003o
0080o`000P00o`050000o`80oh002`00o`020?l000D00?l01@3o0004003o00D0o`000P00o`050?l0
00800?l01P3o0003003o00@0o`000`00o`0;0?l000T00?l01@000?l20?n000@0o`000P3oP0000`3o
0000oh000?n000020?l000kooooo103oP0000`00o`00o`000?l00002003o00D0o`000P3oP0040?l0
00<0oh001`3o0002003o00T0oh001P000?l00`3oP000o`000?l00002003o00D0o`000P00o`060?l0
00D0oh000`3o00020?n000L0003o2`3oP0020?l00080oh000P3o00000`3oP000o`000?l000060?l0
00<0oh002`3o00020?n000030?l0003oP000oh0000@0o`001`3oP00>ooooo`80oh000P3o000>0?n0
0080o`000P3oP0000`3o0000oh000?n000020?n000D0o`000P000?l7003o00L0o`00103oP0020?l0
00<0oh001P000?l20?n000<0o`000P3oP0050?l000T0003o0P3oP0020?l00080oh0000<0o`000?n0
003oP0004P3oP0040?l001L0oh006?ooool001Kooooo1`00o`0B0?l00080003o1`3o00020?n00100
o`000P00o`030?l000@0oh001`3o00020?n00080o`000`3oP0040?l00080oh001`3o00020?n00003
0?l000000?l0003o00L0o`000P3oP00=0?l000L0oh002@3o00030?n000@0o`003_ooool7003o00H0
o`001@00o`070?l000L00?l00P3o0002003o00D0003o0P3oP009003o0080o`001`00o`050?l000@0
0?l01@3o0002003o00D0o`000P00o`060?l000030000o`00o`000?l000@0o`001@00o`020?l00080
0?l01@3o0009003o00D0003o0P3oP0020?l00080003o0P3oP0000`000?l0oh000?n000020?l000ko
oooo0P3oP0020000o`D00?l0103o00030000o`80o`000P3oP00<0?l000T0oh001P000?l00`3oP000
0?l0003o0002003o0080o`000P00o`000`3o00000?l0003o0002003o00@0o`0000<0oh000000o`00
0?l01@000?l20?n000/0003o1`3oP0020?l00080oh000P3o00050?n000@0o`0000<0oh000000o`00
0?l0103o00000`000?l0o`000?l000040000o`80oh0000<0o`000?n0003oP0001P3o00050000o`ko
oooo0P3oP0020?l000L0oh000P000?l:0?l000@0oh001@3o0007003o0080oh000P3o00040?n00003
0?l0003oP000oh000080oh000P3o00030?n000H0003o0`3oP0020?l000L0oh000P00o`020000o`80
oh0000<0003o0?n0003oP0000P3oP002003o00<0oh000P3o00070?n000@0003o1`3oP0020?l000`0
oh000P3o00020?n000800?l01@3oP0020000oaSooooo000Fooooo`L00?l04P3o00020000o`L0o`00
0P3oP00@0?l000800?l00`3o00040?n000L0o`000P3oP0020?l000<0oh00103o00020?n000L0o`00
0P3oP0000`3o0000003o0000o`070?l00080oh003@3o00070?n000T0o`000`3oP0040?l000kooooo
1`00o`060?l000D00?l01`3o0007003o0080o`000P00o`050000o`80oh002@00o`020?l000L00?l0
1@3o0004003o00D0o`000P00o`050?l000800?l01P3o00000`000?l00?l0003o00040?l000D00?l0
0P3o0002003o00D0o`002@00o`050000o`80oh000P3o00020000o`80oh0000<0003o0?n0003oP000
0P3o000>ooooo`80oh000P000?l5003o00@0o`000`000?l20?l00080oh00303o00090?n000H0003o
00<0oh00003o0000o`000P00o`020?l000800?l000<0o`00003o0000o`000P00o`040?l000030?n0
00000?l0003o00D0003o0P3oP00;0000o`L0oh000P3o00020?n00080o`001@3oP0040?l000030?n0
00000?l0003o00@0o`0000<0003o0?l0003o000010000?l20?n000030?l0003oP000oh0000H0o`00
1@000?l>ooooo`80oh000P3o00070?n00080003o2P3o00040?n000D0o`001`00o`020?n00080o`00
103oP0000`3o0000oh000?n000020?n00080o`000`3oP0060000o`<0oh000P3o00070?n000800?l0
0P000?l20?n000030000o`3oP000oh000080oh000P00o`030?n00080o`001`3oP0040000o`L0oh00
0P3o000<0?n00080o`000P3oP002003o00D0oh000P000?lHooooo`005_ooool9003o01T0o`000P3o
P00@0?l000800?l00`3o00040?n000L0o`000P3oP0030?l00080oh003@3o00030?n000@0o`00103o
P0000`3o0000oh000?n000070?l00080oh00103o00030?n000L0o`000P3oP0040?l00080003o00<0
o`000?n0003oP0000P3oP00>ooooo`T00?l0103o0005003o00D0o`002@00o`020?l000800?l01@00
0?l20?n000/00?l00P3o00000`00o`00o`000?l00004003o00<0o`001000o`0<0?l000800?l00P3o
00020?n00080o`0000<0003o003o0000o`000P00o`020?l000<00?l00P3o0006003o00<0o`002@00
o`020?n00080o`000`000?l20?l00080003o0P3oP0000`000?l0oh000?n000020?l000kooooo103o
P005003o00@0o`0000<0oh000000o`000?l00P00o`040?n000<0o`000P3oP0050?l00100oh000P00
o`040?l000800?l000<0o`00003o0000o`001000o`020?l000D0003o0P3oP0000`000?l0oh000?n0
00070000o`80oh000P000?l40?n000030000o`3oP000oh000080o`001@000?l40?n00080o`000P00
o`000`3o0000003o0000o`050000o`80o`0010000?l20?n000<0003o0P3oP0040?l000D0003o3_oo
ool20?n00080o`0000<0oh000?l0003o0000103o00020?n000D0o`000P3oP0020?l000030000o`3o
P000oh000080oh001`3o00020?n000<0o`000P3oP0090?l00100oh000P3o00050?n00080o`000P3o
P0020?l000<0003o103oP0020000o`80oh0000<0o`000?n0003oP0001P3oP0000`3o0000oh000?n0
00020000o`@0oh0000<0o`000?n0003oP0002@3oP0020?l00080oh0000<00?l00?n0003oP0000P3o
P0020?l000D0oh000P3o000Hooooo`005_ooool7003o02@0o`000P00o`050?l000800?l01`3o0002
0?n000L0o`000P3oP0030?l00080oh00403o00020?n00080o`000P3oP0020000o`03003o003o0000
o`0000h0o`000P3oP00=0?l00080oh000`3o00020?n000kooooo5000o`030?l000L00?l0103o0002
003o00D0003o0P3oP00;003o0080o`001`00o`030?l000@00?l00P3oP00:0?l000800?l00P3o0002
0?n00080o`0000<0003o003o0000o`003`00o`000`3o00000?l0003o0009003o00D0o`000P000?l2
0?l00080003o0P3oP0000`000?l0oh000?n000020?l000kooooo0P3oP0020000o`D00?l01@3o0002
0?n000h0o`003@3oP0040?l000030?n0003o0000o`000080o`001000o`000`3o00000?l0003o0004
003o0080o`001@000?l50?n000L0003o0P3oP0020000o`80oh000`000?l20?n00080o`002@000?l4
003o00<0o`000P000?l20?n000030000o`3o0000o`000080003o103oP0050000o`@0o`001@000?l>
ooooo`@0o`0000<0oh000?l0003o00000P3o00020?n00080003o0`3o00040?n00080o`001`3oP000
0`3o0000oh000?n000020?l00080oh000`3o00020?n000@0o`000`00o`020?l000L0oh000P3o0007
0?n00080003o1`3oP0020?l00080oh000`000?l=0?n00080o`002@3oP0020000oa00oh0000<0o`00
0?n0003oP0001`3oP0020000o`80o`006?ooool001Kooooo1`00o`0T0?l000800?l01@3o0002003o
00L0o`000P3oP0070?l00080oh000`3o00020?n00100o`000P3oP0020?l00080oh000P000?l00`00
o`00o`000?l0000>0?l00080oh003@3o00020?n000<0o`000P3oP00>oooooa@00?l00`3o0007003o
00@0o`000P00o`050000o`80oh002`00o`020?l000L00?l00`3o0004003o0080oh002P3o0002003o
0080o`000P3oP0020?l000030000o`00o`000?l000l00?l000<0o`00003o0000o`002@00o`050?l0
0080003o0P3o00020000o`80oh0000<0003o0?n0003oP0000P3o000>ooooo`80oh000P000?l5003o
00D0o`000P3oP00>0?l000d0oh00103o00000`3oP000o`000?l000020?l000@00?l000<0o`00003o
0000o`001000o`020?l000D0003o1@3oP0070000o`80oh000P000?l20?n000<0003o0P3oP0020?l0
00T0003o1000o`030?l00080003o0P3oP0000`000?l0o`000?l000020000o`@0oh001@000?l40?l0
00D0003o3_ooool40?l000030?n0003o0000o`000080o`000P3oP0020000o`<0o`00103oP0020?l0
00L0oh0000<0o`000?n0003oP0000P3o00020?n000<0o`000P3oP0040?l000<00?l00P3o00070?n0
0080o`001`3oP0020000o`L0oh000P3o00020?n000<0003o3@3oP0020?l000T0oh000P000?l@0?n0
00030?l0003oP000oh0000L0oh000P000?l20?l001Sooooo000Fooooo`L00?l02@3o00020?n000/0
o`0000<0oh000?l0003o00000P3o00020?n000<0o`001000o`020?n000L0o`000P3oP0050?l00080
oh001`3o00020?n000<0o`000P3oP0060?l000030000o`3oP000oh0000L0o`000P3oP0040?l00080
003o0`00o`020?l00080oh00403o00020?n000<0o`000P3oP0020?l000<0oh000P3o00020?n000ko
oooo7P00o`040?l000800?l01`000?lD003o00<0o`001000o`0<0?l000800?l01P3o000N003o00/0
o`001@3oP0020?l000kooooo0P3o000;003o00L0o`000`00o`020?l00080oh000`3o00020?n00080
o`001`3oP0020?l00080oh000P000?l:0?l000@00?l0103o00070000o`<0o`000P3oP0040000o`L0
oh000P000?l00`00o`00oh000?n000020?l00080oh001@000?l20?l000@00?l00`3o00040?n000<0
0?l00P3o00070?n00080003o0P00o`020?l000030?n000000?l0003o0080003o3_ooool4003o0003
0?n0003o0000o`000080o`000P3oP002003o00030?l00000o`000?l00100oh000P3o000=0?n00003
0?l0003oP000oh000080o`00103oP0030?l00080oh000P000?l<0?n00080003o0P3oP0020000o`T0
oh0000<0o`000?n0003oP000503oP0020?l000T0oh0000<0o`000?n0003oP0001P3oP0000`000?l0
oh000?n000020?n001Sooooo000Fooooo`L00?l02@3o00020?n000/0o`0000<0oh000?l0003o0000
0P3o00020?n000<0o`001000o`020?n000L0o`000P3oP0050?l00080oh001`3o00020?n000<0o`00
0P3oP0060?l000030000o`3oP000oh0000L0o`000P3oP0040?l00080003o0`00o`020?l00080oh00
403o00020?n000<0o`000P3oP0020?l000<0oh000P3o00020?n000kooooo7P00o`040?l000800?l0
1`000?lD003o00<0o`001000o`0<0?l000800?l01P3o000N003o00/0o`001@3oP0020?l000kooooo
0P3o000;003o00L0o`000`00o`020?l00080oh000`3o00020?n00080o`001`3oP0020?l00080oh00
0P000?l:0?l000@00?l0103o00070000o`<0o`000P3oP0040000o`L0oh000P000?l00`00o`00oh00
0?n000020?l00080oh001@000?l20?l000@00?l00`3o00040?n000<00?l00P3o00070?n00080003o
0P00o`020?l000030?n000000?l0003o0080003o3_ooool4003o00030?n0003o0000o`000080o`00
0P3oP002003o00030?l00000o`000?l00100oh000P3o000=0?n000030?l0003oP000oh000080o`00
103oP0030?l00080oh000P000?l<0?n00080003o0P3oP0020000o`T0oh0000<0o`000?n0003oP000
503oP0020?l000T0oh0000<0o`000?n0003oP0001P3oP0000`000?l0oh000?n000020?n001Sooooo
000Fooooo`800?l00P3oP00<0?l00080oh002`3o00000`3oP000o`000?l000070?l000800?l00P3o
00020?n000`0o`000P3oP0040?l00080oh000`3o00020?n000T0o`000`000?l20?n000L0o`000P3o
P0040?l00080003o00<00?l00?l0003o00001`3o00020000o`T0o`000P3oP0030?l00080003o103o
00030?n00080o`000P3oP00>ooooo`L00?l00P3o0002003o0080o`005`00o`070000o`d00?l000<0
o`00003o0000o`000P3o0002003o00<0o`001P00o`000`3o00000?l0003o00070?l000@00?l01@3o
000A003o00030?l00000o`000?l000h00?l01P3o00020?n000030?l0003oP000oh000080o`003_oo
ool=003o00L0o`000`00o`020?l00080oh000`3o00040?n000D0o`00203oP0000`3o00000?l0003o
00070?l000@00?l01@3o00040?n00080003o0`00o`020?l000@0003o0`3oP0040?l00080003o00<0
0?l00?n0003oP0000P3o00020?n00080o`0000<0oh000000o`000?l02`3o00020000o`<00?l0103o
00050?n00080003o1000o`000`3oP000o`000?l000020000o`kooooo0P00o`020?l000<0oh000P3o
00020?n00080o`0000<0003o003o0000o`000P3o00090?n000<0o`001P3oP0020000o`030?n0003o
0000o`0000@0oh000`3o00020?n00080o`001@3oP0020?l00080oh000P000?l30?n00080003o1`3o
P0020000obT0oh000P000?l50?n00080o`00103oP0020?l000030?n000000?l0003o0080oh006?oo
ool001Kooooo0P00o`020?n000`0o`000P3oP00;0?l000030?n0003o0000o`0000L0o`000P00o`02
0?l00080oh00303o00020?n000@0o`000P3oP0030?l00080oh002@3o00030000o`80oh001`3o0002
0?n000@0o`000P000?l00`00o`00o`000?l000070?l00080003o2@3o00020?n000<0o`000P000?l4
0?l000<0oh000P3o00020?n000kooooo1`00o`020?l000800?l00P3o000G003o00L0003o3@00o`00
0`3o00000?l0003o00020?l000800?l00`3o0006003o00030?l00000o`000?l000L0o`001000o`05
0?l001400?l000<0o`00003o0000o`003P00o`060?l00080oh0000<0o`000?n0003oP0000P3o000>
ooooo`d00?l01`3o0003003o0080o`000P3oP0030?l000@0oh001@3o00080?n000030?l00000o`00
0?l000L0o`001000o`050?l000@0oh000P000?l3003o0080o`0010000?l30?n000@0o`000P000?l0
0`00o`00oh000?n000020?l00080oh000P3o00000`3oP000003o0000o`0;0?l00080003o0`00o`04
0?l000D0oh000P000?l4003o00030?n0003o0000o`000080003o3_ooool2003o0080o`000`3oP002
0?l00080oh000P3o00000`000?l00?l0003o00020?l000T0oh000`3o00060?n00080003o00<0oh00
0?l0003o0000103oP0030?l00080oh000P3o00050?n00080o`000P3oP0020000o`<0oh000P000?l7
0?n00080003o:@3oP0020000o`D0oh000P3o00040?n00080o`0000<0oh000000o`000?l00P3oP00H
ooooo`005_ooool20?l00080oh00303o00020?n000/0o`0000<0oh000?l0003o00000P3o00020?n0
0100o`000P3oP0030?l000@0oh000P3o00020?n000<0o`000P3oP0090?l000<0003o0P3oP0040?l0
0080oh0000<0o`000?n0003oP000103o00020000o`03003o003oP000oh0000L0o`000P00o`050?l0
0080oh000P3o00020?n000<0o`00103oP0020?l000<0oh00103o000>ooooo`T00?l0103o000G003o
00L0003o4000o`020?l000800?l00`3o0004003o00D0o`000P3oP0050?l000@00?l01@3o000D003o
0080o`000P00o`000`3o00000?l0003o0007003o00H0o`000P000?l00`3oP000o`000?l000020?l0
00kooooo0P3o000;003o01<0o`000P3oP0020?l00080oh0000<0o`000?n0003oP0000P3o00040?n0
00030000o`00o`000?l000@00?l01`3o00020?n00080o`0000<0003o0?n0003oP0000P3o00020?n0
0003003o003o0000o`000080o`0010000?l30?n00180o`00103oP0030?l00080oh000P3o0005003o
00H0003o00<0oh000?l0003o00000P3oP0090?l000kooooo0P3o00020?n000030?l0003oP000oh00
0080o`000P3oP0020000o`030?n0003o0000o`0000800?l0103o00030000o`800?l00`3o00090?n0
0080o`000P000?l20?n00080o`000`3oP0020?l000/0oh000P3o00000`3oP000003o0000o`0;0?n0
0080o`009`3oP0020000o`<0oh000P3o00040?n00080o`0000<0oh000000o`000?l00P3oP00Hoooo
o`005_ooool20?l00080oh00303o00020?n000/0o`0000<0oh000?l0003o00000P3o00020?n00100
o`000P3oP0030?l000@0oh000P3o00020?n000<0o`000P3oP0090?l000<0003o0P3oP0040?l00080
oh0000<0o`000?n0003oP000103o00020000o`03003o003oP000oh0000L0o`000P00o`050?l00080
oh000P3o00020?n000<0o`00103oP0020?l000<0oh00103o000>ooooo`T00?l0103o000G003o00L0
003o4000o`020?l000800?l00`3o0004003o00D0o`000P3oP0050?l000@00?l01@3o000D003o0080
o`000P00o`000`3o00000?l0003o0007003o00H0o`000P000?l00`3oP000o`000?l000020?l000ko
oooo0P3o000;003o01<0o`000P3oP0020?l00080oh0000<0o`000?n0003oP0000P3o00040?n00003
0000o`00o`000?l000@00?l01`3o00020?n00080o`0000<0003o0?n0003oP0000P3o00020?n00003
003o003o0000o`000080o`0010000?l30?n00180o`00103oP0030?l00080oh000P3o0005003o00H0
003o00<0oh000?l0003o00000P3oP0090?l000kooooo0P3o00020?n000030?l0003oP000oh000080
o`000P3oP0020000o`030?n0003o0000o`0000800?l0103o00030000o`800?l00`3o00090?n00080
o`000P000?l20?n00080o`000`3oP0020?l000/0oh000P3o00000`3oP000003o0000o`0;0?n00080
o`009`3oP0020000o`<0oh000P3o00040?n00080o`0000<0oh000000o`000?l00P3oP00Hooooo`00
5_ooool@0?l00080oh002`3o00000`3oP000o`000?l000020?l00080oh000P3o00000`00o`00o`00
0?l000070?l00080oh001`3o00020?n000@0o`000P3oP0030?l00080oh002@3o00020000o`030?l0
003oP000oh0000/0o`0010000?l00`3o00000?l0003o00070?l00080003o1@3o00020?n00080o`00
103oP0000`3o0000oh000?n000040?l00080oh0000<0003o0?l0003o00000P3o000>ooooo`T00?l0
103o000A003o0080o`001000o`020?l000D0003o2@00o`020?l000L00?l01@3o0004003o00D0o`00
0P3oP0040?l000030000o`00o`000?l000800?l01@3o000D003o00L0o`001000o`030?l00080003o
103o00020000o`030?n0003o0000o`000080o`003_ooool70?l000H00?l03P3o0003003o0080o`00
0P3oP00=0?l000030000o`3o0000o`0000/0o`000P3oP0020?l000030?n0003o0000o`0000T0o`00
0P000?l50?n000800?l01@3o0004003o00D0o`000P3oP0020?l00080003o0P3o00000`3oP000003o
0000o`020?n000<0o`000P3oP0020?l00080003o0P3oP0000`000?l0o`000?l000020?l000800?l0
1@3o00020?n000kooooo0P3o00090?n00080003o00<0oh000?l0003o00000P3o00020?n00080o`00
00<00?l00000o`000?l00P00o`020?l000030?n0003o0000o`000080oh001`3o000?0?n000030?l0
003oP000oh0000@0003o0`3oP0040000o`T0oh000P3o00000`3oP000003o0000o`020?n00080o`00
0P000?l00`3oP000003o0000o`0=0?n00080o`007P3oP00Hooooo`005_oooolM0?l000030?n0003o
0000o`000100o`000P3oP00=0?l00080oh000`3o00020?n001/0o`000P000?l3003o00L0o`000P00
o`050?l00080oh000P3o00040?n000030?l00000o`000?l00080o`000P000?l20?n000030000o`3o
0000o`000080o`003_ooool4003o00<0o`000P00o`040?l000L00?l00P3o0007003o00<0o`001000
o`050?l00080oh004P00o`070?l000800?l01@3o00070000o`@00?l01@3o000D003o00D0o`001P00
o`030?l00080003o1P3o00000`3oP000o`000?l000020?l000kooooo3P3o0006003o0080oh001@3o
0002003o00<0o`000P3oP0020?l00080oh002@3o00000`3oP000o`000?l000020?l000D00?l0103o
00020000o`D0o`000P3oP0050?l00080oh000P3o00030?n000P0o`0000<0003o0?n0003oP0000P3o
00040000o`030?l0003oP000oh000080o`000P3oP0020?l000030?n000000?l0003o0080003o0P3o
00030?n00080o`000P000?l30?n000@0o`000P00o`020?n000<0o`000P3oP00>ooooo`80o`000P3o
P0000`3o0000oh000?n000040?n00080003o1@3o00040?n000030?l0003oP000oh0000800?l01`3o
00020?n00080o`000`3oP004003o0080oh000`3o00020?n000@0003o00<0o`000?n0003oP0000P00
0?lB0?n00080o`0000<0oh00003o0000o`000P3o00020?n00080003o2@3oP0000`000?l0oh000?n0
000@0?n00080003o4P3oP00Hooooo`005_oooolM0?l000030?n0003o0000o`000100o`000P3oP00=
0?l00080oh000`3o00020?n001/0o`000P000?l3003o00L0o`000P00o`050?l00080oh000P3o0004
0?n000030?l00000o`000?l00080o`000P000?l20?n000030000o`3o0000o`000080o`003_ooool4
003o00<0o`000P00o`040?l000L00?l00P3o0007003o00<0o`001000o`050?l00080oh004P00o`07
0?l000800?l01@3o00070000o`@00?l01@3o000D003o00D0o`001P00o`030?l00080003o1P3o0000
0`3oP000o`000?l000020?l000kooooo3P3o0006003o0080oh001@3o0002003o00<0o`000P3oP002
0?l00080oh002@3o00000`3oP000o`000?l000020?l000D00?l0103o00020000o`D0o`000P3oP005
0?l00080oh000P3o00030?n000P0o`0000<0003o0?n0003oP0000P3o00040000o`030?l0003oP000
oh000080o`000P3oP0020?l000030?n000000?l0003o0080003o0P3o00030?n00080o`000P000?l3
0?n000@0o`000P00o`020?n000<0o`000P3oP00>ooooo`80o`000P3oP0000`3o0000oh000?n00004
0?n00080003o1@3o00040?n000030?l0003oP000oh0000800?l01`3o00020?n00080o`000`3oP004
003o0080oh000`3o00020?n000@0003o00<0o`000?n0003oP0000P000?lB0?n00080o`0000<0oh00
003o0000o`000P3o00020?n00080003o2@3oP0000`000?l0oh000?n0000@0?n00080003o4P3oP00H
ooooo`005_ooool90?l000800?l04P3o00000`3oP000o`000?l000070?l00080003o5P3o00020?n0
00X0o`000P00o`040?l000030?n0003o0000o`0000d0o`000P3oP0000`000?l00?l0003o0002003o
00D0o`000P00o`050?l00080oh001P000?l30?l00080oh0010000?l00`3oP000o`000?l000020?l0
00kooooo1000o`000`3o00000?l0003o000=003o00<0o`001P00o`030?l000@00?l01@3o00020?n0
00T00?l0103o0005003o0080o`000P00o`000`3o00000?l0003o0002003o00<0o`002@000?l4003o
00D0o`005000o`040?l000D00?l01@3o00020000o`800?l00P000?l70?l000kooooo0P3oP00<0?l0
00H00?l00P3oP0050?l000800?l01`3o00020?n000030000o`3o0000o`0000L0o`000P3oP0060?l0
0003003o003o0000o`000080o`000P000?l30?l000@0oh002@3o00030?n000@0o`000P3oP0020?l0
00030000o`3o0000o`000080o`001@000?l20?n00080o`000P3oP0020?l000<0oh001@000?l20?n0
00@0003o0`3oP0020000o`@0o`000P3oP0030?l00080oh003_ooool;0?n00080003o1@3o00020?n0
0080003o0`3oP002003o0080o`0000<0oh000?l0003o00000P3o00020?n000<0o`000P3oP002003o
0080o`001`3oP0020000o`80o`001@000?l@0?n000@0o`0000<0oh000?l0003o00000P3oP0050000
o`d0oh000P3o00020000o`X0oh000P000?l50?n00080003o0P3oP0040000o`D0oh006?ooool001Ko
oooo2@3o0002003o0180o`0000<0oh000?l0003o00001`3o00020000oaH0o`000P3oP00:0?l00080
0?l0103o00000`3oP000o`000?l0000=0?l00080oh0000<0003o003o0000o`000P00o`050?l00080
0?l01@3o00020?n000H0003o0`3o00020?n000@0003o00<0oh000?l0003o00000P3o000>ooooo`@0
0?l000<0o`00003o0000o`003@00o`030?l000H00?l00`3o0004003o00D0o`000P3oP009003o00@0
o`001@00o`020?l000800?l000<0o`00003o0000o`000P00o`030?l000T0003o1000o`050?l001@0
0?l0103o0005003o00D0o`000P000?l2003o0080003o1`3o000>ooooo`80oh00303o0006003o0080
oh001@3o0002003o00L0o`000P3oP0000`000?l0o`000?l000070?l00080oh001P3o00000`00o`00
o`000?l000020?l00080003o0`3o00040?n000T0o`000`3oP0040?l00080oh000P3o00000`000?l0
o`000?l000020?l000D0003o0P3oP0020?l00080oh000P3o00030?n000D0003o0P3oP0040000o`<0
oh000P000?l40?l00080oh000`3o00020?n000kooooo2`3oP0020000o`D0o`000P3oP0020000o`<0
oh000P00o`020?l000030?n0003o0000o`000080o`000P3oP0030?l00080oh000P00o`020?l000L0
oh000P000?l20?l000D0003o403oP0040?l000030?n0003o0000o`000080oh001@000?l=0?n00080
o`000P000?l:0?n00080003o1@3oP0020000o`80oh0010000?l50?n001Sooooo000Fooooo`80o`00
0P3oP0030?l00080oh00503o00000`3oP000o`000?l000020?l00080oh002`3o00000`00o`00o`00
0?l000070?l00080oh00103o00020?n000P0o`001000o`050?l00080oh00403o0002003o0080o`00
0P00o`050?l00080oh000`3o00020?n00080o`000P3oP0070?l000D0003o103o000>oooooa800?l0
1@3o000O003o0080o`000`00o`060?l00003003o003o0000o`0000800?l01@3o00070000o`P00?l0
00<0o`00003o0000o`003`00o`000`3o00000?l0003o00040?l000D00?l01@3o00020000o`80o`00
0P000?l3003o00@0o`003_ooool;0?l00080oh001`3o00020?n000<0o`000P000?l20?n000030?l0
003oP000oh0000@0o`000P3oP0030000o`80o`001@000?l20?n000@0o`001`00o`020?l00080oh00
00<0o`000?n0003oP0002`3o00020?n000030?l0003oP000oh000080oh001@3o00090000o`80oh00
1P3o00000`3oP000o`000?l000050000o`80oh0010000?l30?n00080003o103o00020000o`<0o`00
0P3oP00>ooooo`80o`002@3oP0020000o`<0o`002@3oP0040?l000030?n0003o0000o`0000@0oh00
0P3o00000`3oP000003o0000o`020?l000/0oh000P3o00050000o`D0oh000P3o00020?n00080o`00
1@3oP002003o0080oh000`3o00020?n00080003o0P3oP0000`000?l0oh000?n000060000oa00oh00
00<0003o0?l0003o00001`3oP0020000o`@0oh000P000?l50?n001Sooooo000Fooooo`80o`000P3o
P0030?l00080oh00503o00000`3oP000o`000?l000020?l00080oh002`3o00000`00o`00o`000?l0
00070?l00080oh00103o00020?n000P0o`001000o`050?l00080oh00403o0002003o0080o`000P00
o`050?l00080oh000`3o00020?n00080o`000P3oP0070?l000D0003o103o000>oooooa800?l01@3o
000O003o0080o`000`00o`060?l00003003o003o0000o`0000800?l01@3o00070000o`P00?l000<0
o`00003o0000o`003`00o`000`3o00000?l0003o00040?l000D00?l01@3o00020000o`80o`000P00
0?l3003o00@0o`003_ooool;0?l00080oh001`3o00020?n000<0o`000P000?l20?n000030?l0003o
P000oh0000@0o`000P3oP0030000o`80o`001@000?l20?n000@0o`001`00o`020?l00080oh0000<0
o`000?n0003oP0002`3o00020?n000030?l0003oP000oh000080oh001@3o00090000o`80oh001P3o
00000`3oP000o`000?l000050000o`80oh0010000?l30?n00080003o103o00020000o`<0o`000P3o
P00>ooooo`80o`002@3oP0020000o`<0o`002@3oP0040?l000030?n0003o0000o`0000@0oh000P3o
00000`3oP000003o0000o`020?l000/0oh000P3o00050000o`D0oh000P3o00020?n00080o`001@3o
P002003o0080oh000`3o00020?n00080003o0P3oP0000`000?l0oh000?n000060000oa00oh0000<0
003o0?l0003o00001`3oP0020000o`@0oh000P000?l50?n001Sooooo000Fooooo`L0o`000P3oP00D
0?l000030?n0003o0000o`000080o`000P3oP0030?l00080003o403o00020?n000@0o`000P3oP003
0?l00080oh001@3o0002003o00@0o`0000<0oh000?l0003o0000503o0002003o00D0o`000P3oP007
0?l00080oh001`3o00050?n000@0o`003_ooool20?l000800?l000<0o`00003o0000o`001`00o`09
0?l000T00?l00P3o000P003o0080o`000P00o`050?l000H0003o00<0oh00003o0000o`004P00o`04
0?l000800?l01@3o00020000o`D00?l02@3o00020000o`D00?l00P3o000>ooooo`800?l02@3o0002
0?n000030?l0003oP000oh000080oh000P3o00030?n00080o`00103oP0070?l000<0oh000P000?l2
0?l000D0003o0P3oP0060?l00003003o003o0000o`0000800?l01@3o00020?n00080003o2@3o0003
0000o`@0oh000P3o0003003o00@0003o1`3oP002003o00@0o`0000<0003o0?l0003o00000P000?l>
0?n000L0o`000P3oP0020000o`kooooo0P3o00020?n000030000o`3o0000o`000080o`000P00o`03
0?l000/0oh001000o`000`3oP000o`000?l000040?n000<0o`001000o`020?n00080o`001`3oP002
0?l000<0oh000P000?l20?n00080003o00<0oh000?l0003o00000P3oP0020000o`D0oh000P00o`02
0?n000030?l0003oP000oh000080oh0010000?l30?n000H0003o3P3oP003003o00d0oh0010000?l5
0?n001Sooooo000Fooooo`L0o`000P3oP00D0?l000030?n0003o0000o`000080o`000P3oP0030?l0
0080003o403o00020?n000@0o`000P3oP0030?l00080oh001@3o0002003o00@0o`0000<0oh000?l0
003o0000503o0002003o00D0o`000P3oP0070?l00080oh001`3o00050?n000@0o`003_ooool20?l0
00800?l000<0o`00003o0000o`001`00o`090?l000T00?l00P3o000P003o0080o`000P00o`050?l0
00H0003o00<0oh00003o0000o`004P00o`040?l000800?l01@3o00020000o`D00?l02@3o00020000
o`D00?l00P3o000>ooooo`800?l02@3o00020?n000030?l0003oP000oh000080oh000P3o00030?n0
0080o`00103oP0070?l000<0oh000P000?l20?l000D0003o0P3oP0060?l00003003o003o0000o`00
00800?l01@3o00020?n00080003o2@3o00030000o`@0oh000P3o0003003o00@0003o1`3oP002003o
00@0o`0000<0003o0?l0003o00000P000?l>0?n000L0o`000P3oP0020000o`kooooo0P3o00020?n0
00030000o`3o0000o`000080o`000P00o`030?l000/0oh001000o`000`3oP000o`000?l000040?n0
00<0o`001000o`020?n00080o`001`3oP0020?l000<0oh000P000?l20?n00080003o00<0oh000?l0
003o00000P3oP0020000o`D0oh000P00o`020?n000030?l0003oP000oh000080oh0010000?l30?n0
00H0003o3P3oP003003o00d0oh0010000?l50?n001Sooooo000Fooooo`L0o`000P3oP0020?l000<0
oh00503o00020?n000<0o`000P000?l4003o00`0o`000P3oP0040?l00080oh000`3o00020?n000L0
o`000P3oP00G0?l00080003o1000o`000`3o0000oh000?n000090?l00080oh002@3o00030?n000@0
o`003_ooool=003o00X0o`001`00o`040?l000h00?l00P3o000=003o0080oh0000<0o`00003o0000
o`001000o`030?l000H0003o00<0oh00003o0000o`001P00o`030?l00080003o2@00o`090?l00080
003o2@00o`050?l00080003o1`00o`0>ooooo`@00?l000<0o`00003o0000o`00103o00020?n00003
0?l0003oP000oh000080oh000P3o00020?n000030?l000000?l0003o00@0oh0000<0o`000?n0003o
P0002@3oP0070000o`80oh002`3o0002003o0080o`001@3oP0070?l000L0003o0P3oP0020000o`<0
o`00103oP0020?l000D0oh001P3o00030000o`T0oh000P3o00050?n000@0o`000P000?l00`3oP000
o`000?l000020000o`kooooo103oP0000`000?l0o`000?l000020000o`@0o`002P3oP006003o0003
0?n000000?l0003o0080003o0P3oP005003o0080003o0P3oP0020?l000L0oh000P3o00030?n000H0
003o00<0oh00003o0000o`000P3oP0030000o`@0oh000P000?l70?n00080003o0`3oP0020?l000@0
003o0`3oP0040?l00080oh000P3o00070?n000<0o`000P3oP0020?l00080oh0000<0o`000000o`00
0?l00P000?l70?n00080o`006?ooool001Kooooo0P00o`090?l00080oh00403o00000`3oP000o`00
0?l000020?l00080oh001`3o0002003o0180o`000P3oP0030?l00080oh001`3o00020?n00100o`00
0P00o`050?l00080003o0P00o`0<0?l000@0003o2@3o00020000o`D0o`003_ooool4003o00030?l0
0000o`000?l000H00?l02@3o0007003o00D0o`000P000?lE003o00H0o`000P3oP0030?l00080003o
0`00o`020?l000H0003o00<0oh00003o0000o`001P00o`030?l00080003o2@00o`090?l00080003o
00<00?l00?l0003o00000P3o0004003o00D0o`000P000?l5003o0080o`003_ooool40?l000D00?l0
0P3o00020?n000030?l0003oP000oh000080oh000P00o`020?l000<0003o0P3oP0020000o`D0o`00
0P3oP0020?l000030?n000000?l0003o00@0003o1@3oP0020?l000L00?l01`3o00060?n000D0o`00
1@000?l40?n00080003o0P3oP0000`3o0000oh000?n000020?n000@0o`000`3oP0060?l000L0003o
00<0o`00003o0000o`00103o0002003o00030?l000000?l0003o00@0003o0P3oP0000`3o0000oh00
0?n000020000o`kooooo1@3oP0020000o`800?l0103o000:0?n000H00?l000<0oh000000o`000?l0
0P000?l20?n000800?l000<0003o0?l0003o00000P000?l20?n00080o`0000<0003o0?n0003oP000
0P000?l40?n000030000o`3oP000oh0000@0003o0`3oP0020?l00080oh000`000?l40?n00080o`00
2@3oP0020000o`030?n0003o0000o`0000T0oh00103o00070?n00080o`000`3oP0020?l00080oh00
0`3o00020000o`800?l00P3oP0020?l000<0oh000P3o000Hooooo`005_ooool2003o00T0o`000P3o
P00@0?l000030?n0003o0000o`000080o`000P3oP0070?l000800?l04P3o00020?n000<0o`000P3o
P0070?l00080oh00403o0002003o00D0o`000P000?l2003o00`0o`0010000?l90?l00080003o1@3o
000>ooooo`@00?l000<0o`00003o0000o`001P00o`090?l000L00?l01@3o00020000oaD00?l01P3o
00020?n000<0o`000P000?l3003o0080o`001P000?l00`3oP0000?l0003o0006003o00<0o`000P00
0?l9003o00T0o`000P000?l00`00o`00o`000?l000020?l000@00?l01@3o00020000o`D00?l00P3o
000>ooooo`@0o`001@00o`020?l00080oh0000<0o`000?n0003oP0000P3oP002003o0080o`000`00
0?l20?n00080003o1@3o00020?n00080o`0000<0oh000000o`000?l010000?l50?n00080o`001`00
o`070?l000H0oh001@3o00050000o`@0oh000P000?l20?n000030?l0003oP000oh000080oh00103o
00030?n000H0o`001`000?l00`3o00000?l0003o00040?l000800?l000<0o`000000o`000?l01000
0?l20?n000030?l0003oP000oh000080003o3_ooool50?n00080003o0P00o`040?l000X0oh001P00
o`000`3oP000003o0000o`020000o`80oh000P00o`000`000?l0o`000?l000020000o`80oh000P3o
00000`000?l0oh000?n000020000o`@0oh0000<0003o0?n0003oP00010000?l30?n00080o`000P3o
P0030000o`@0oh000P3o00090?n00080003o00<0oh000?l0003o00002@3oP0040?l000L0oh000P3o
00030?n00080o`000P3oP0030?l00080003o0P00o`020?n00080o`000`3oP0020?l001Sooooo000F
ooooo`800?l02@3o00020?n00100o`0000<0oh000?l0003o00000P3o00020?n000L0o`000P00o`0;
0?l000030000o`3o0000o`0000d0o`0000<0oh000?l0003o00000P3o00020?n00100o`000P00o`0<
0?l000800?l01`3o00040000o`T0o`000P000?l00`3oP000o`000?l000020?l000kooooo0P00o`03
0?l000P00?l01`3o0009003o00D0o`000P000?lE003o0080o`000P3oP0020?l000<0oh000P3o0002
0000o`<00?l00P3o00060000o`030?n00000o`000?l00080o`001`00o`020000o`T00?l00P3o0002
003o00D0o`001@00o`040?l000800?l01@3o00020000o`L00?l03_ooool40?l000L00?l00P3oP000
0`3o0000oh000?n000020?n000800?l01@3o00020?n000L0o`000P000?l20?n000030?l000000?l0
003o00@0003o1@3oP0020?l000L00?l00P3o0002003o00<0o`0010000?l20?n000030000o`3o0000
o`0000800?l01@000?l40?n00080o`000P3oP0000`3o0000oh000?n000020?n000@0o`0000<0oh00
0?l0003o00000P3oP00;0000o`<0o`001P00o`000`3o0000oh000?n000020?l000@0oh001@000?l>
oooooa@0oh000P3o00030000o`80o`000P00o`050000o`80oh000P00o`000`3oP000o`000?l00002
0?l00080oh000P3o00000`000?l0oh000?n000060?n000030?l0003oP000oh000080003o1@3oP004
0?l000D0003o1P3oP0000`00o`00o`000?l000060000o`030?n0003o0000o`0000T0oh000P3o0009
0?n00080o`000`3oP0020000o`80oh000`3o00020000o`800?l00P000?l20?n000030?l0003oP000
oh000080oh006?ooool001Kooooo0P00o`090?l00080oh00403o00000`3oP000o`000?l000020?l0
0080oh001`3o0002003o00/0o`0000<0003o0?l0003o00003@3o00000`3oP000o`000?l000020?l0
0080oh00403o0002003o00`0o`000P00o`070?l000@0003o2@3o00020000o`030?n0003o0000o`00
0080o`003_ooool2003o00<0o`002000o`070?l000T00?l01@3o00020000oaD00?l00P3o00020?n0
0080o`000`3oP0020?l00080003o0`00o`020?l000H0003o00<0oh00003o0000o`000P3o0007003o
0080003o2@00o`020?l000800?l01@3o0005003o00@0o`000P00o`050?l00080003o1`00o`0>oooo
o`@0o`001`00o`020?n000030?l0003oP000oh000080oh000P00o`050?l00080oh001`3o00020000
o`80oh0000<0o`000000o`000?l010000?l50?n00080o`001`00o`020?l000800?l00`3o00040000
o`80oh0000<0003o0?l0003o00000P00o`050000o`@0oh000P3o00020?n000030?l0003oP000oh00
0080oh00103o00000`3oP000o`000?l000020?n000/0003o0`3o0006003o00030?l0003oP000oh00
0080o`00103oP0050000o`kooooo503oP0020?l000<0003o0P3o0002003o00D0003o0P3oP002003o
00030?n0003o0000o`000080o`000P3oP0020?l000030000o`3oP000oh0000H0oh0000<0o`000?n0
003oP0000P000?l50?n000@0o`001@000?l60?n00003003o003o0000o`0000H0003o00<0oh000?l0
003o00002@3oP0020?l000T0oh000P3o00030?n00080003o0P3oP0030?l00080003o0P00o`020000
o`80oh0000<0o`000?n0003oP0000P3oP00Hooooo`005_ooool4003o01D0o`001000o`050?l000D0
0?l09P3o00020?n00100o`000P3oP00<0?l00080oh001`3o00020?n00080003o0`3o00020?n000/0
o`003_ooool2003o00<0o`001P00o`090?l000T00?l01@3o00020000oa000?l01`3o00020?n00080
o`000`3oP0020?l00080003o0`00o`020?l000H0003o00<0oh00003o0000o`001P00o`030?l00080
003o5`00o`040?l000800?l01@3o0009003o00kooooo0P3o00020?n000`0o`000P3oP0040?l00003
0?n0003o0000o`000080oh002@3o00020?n000030?l000000?l0003o00@0003o1@3oP00;003o00D0
o`0010000?l20?n000030000o`00o`000?l000800?l01@000?l40?n00080o`001`3oP0040?l00003
0?n0003o0000o`000080oh001@000?l40?n00080003o00<0oh000?l0003o0000103o0002003o0003
0?l0003oP000oh0000H0oh001@000?l>oooooa@0oh000`3o00040?n00080o`001@000?l50?n00080
o`000P000?l20?n00080o`0000<0003o0?n0003oP0000P000?l@0?n00080o`000P000?l:0?n00080
o`002@3oP0020?l000T0oh000P3o00020000o`<0o`001`3oP0020?l000D0oh000P000?l4003o00L0
oh006?ooool001Kooooo1000o`0E0?l000@00?l01@3o0005003o02H0o`000P3oP00@0?l00080oh00
303o00020?n000L0o`000P3oP0020000o`<0o`000P3oP00;0?l000kooooo0P00o`030?l000H00?l0
2@3o0009003o00D0o`000P000?l@003o00L0o`000P3oP0020?l000<0oh000P3o00020000o`<00?l0
0P3o00060000o`030?n00000o`000?l000H00?l00`3o00020000oaL00?l0103o0002003o00D0o`00
2@00o`0>ooooo`80o`000P3oP00<0?l00080oh00103o00000`3oP000o`000?l000020?n000T0o`00
0P3oP0000`3o0000003o0000o`040000o`D0oh002`00o`050?l000@0003o0P3oP0000`000?l00?l0
003o0002003o00D0003o103oP0020?l000L0oh00103o00000`3oP000o`000?l000020?n000D0003o
103oP0020000o`030?n0003o0000o`0000@0o`000P00o`000`3o0000oh000?n000060?n000D0003o
3_oooolD0?n000<0o`00103oP0020?l000D0003o1@3oP0020?l00080003o0P3oP0020?l000030000
o`3oP000oh000080003o403oP0020?l00080003o2P3oP0020?l000T0oh000P3o00090?n00080o`00
0P000?l30?l000L0oh000P3o00050?n00080003o1000o`070?n001Sooooo000Fooooo`@00?l05@3o
0004003o00D0o`001@00o`060?l000800?l0?P3o00020?n000L0o`000P3oP0020000oa00o`003_oo
ool2003o00<0o`001P00o`030?l000@00?l00P3o000<003o00@0o`003000o`020?n000T0o`000P3o
P0020?l000<0oh000P3o0005003o0080o`001P000?l00`3oP0000?l0003o0006003o00<0o`000P00
0?lG003o00@0o`000P00o`030?l000/00?l03_ooool70?l00080oh001`3o00020?n000@0o`0000<0
oh000?l0003o00000P3oP0050?l00080003o0P00o`030?l000H0003o0`3oP0020?l000800?l0103o
0007003o0080003o303o00070000o`80oh000P3o00020?n000030?l0003oP000oh000080oh00103o
00000`3oP000o`000?l000020?n000D0003o103oP0050?l00080oh002@3o00040?n000<0003o0P3o
P00>ooooo`@0003o0`3oP0020?l000D0oh000P3o00060?n000030?l000000?l0003o0080003o0P3o
00030?n00080003o0P3oP002003o00030?n000000?l0003o0080003o0P3oP0020?l000030?n00000
0?l0003o00@0oh000P3o00090?n000030?l000000?l0003o0080o`000P000?l00`3oP000003o0000
o`060?n000<0o`001`3oP0060?l00080003o0`3o00020?n00080o`000`3oP0020?l000D0oh001000
0?l40?n000<0003o1000o`070?n001Sooooo000Fooooo`D00?l02@3o00020000o`T0o`000P000?l2
003o00<0o`000P00o`020?l000800?l0303o00020?n00300o`001000o`0;0?l000030?n0003o0000
o`000080oh000P000?l90?l00080oh001@3o000>ooooo`D0o`001000o`050?l001@00?l00P3o000<
003o00H0o`000P000?l00`00o`00o`000?l000020?n00080o`000`3oP0020?l000D00?l00P3o0006
0000o`030?n00000o`000?l000H00?l00`3o000K003o0080o`000P00o`000`3o00000?l0003o0002
0?l000T00?l03_ooool4003o00<0o`000P3oP0070?l00080oh000P3o00020?n000030000o`00o`00
0?l000D0o`000P3oP0020000o`800?l00`3o00060000o`80oh000`3o0004003o00<0o`001P00o`05
0000o`80oh000P3o0003003o0080o`001@000?l40?n00080o`000P3oP0000`3o0000oh000?n00009
0?l000H0oh0000<0003o0?n0003oP0000P3oP0050?l000@0oh000P3o00000`3oP000o`000?l00006
0?l000<0003o0P3oP00>ooooo`800?l00P000?l00`3o0000oh000?n000070?n000@0o`000P000?l2
0?n000030?l0003oP000oh000080003o0P3o00070?n00080003o00<0oh000000o`000?l0103oP002
0?l000030?n000000?l0003o0080003o0P3oP0020?l000T0oh0000<0o`000?n0003oP0002@3oP004
0000o`030?n0003o0000o`000080oh00103o00030?n000@0o`000P000?l3003o0080oh000P3o0003
0?n00080o`000P3oP0020?l000<0003o0P3o0002003o00D0003o1000o`070?n001Sooooo000Foooo
o`D00?l02@3o00020000o`T0o`000P000?l2003o00<0o`000P00o`020?l000800?l0303o00020?n0
0300o`001000o`0;0?l000030?n0003o0000o`000080oh000P000?l90?l00080oh001@3o000>oooo
o`D0o`001000o`050?l001@00?l00P3o000<003o00H0o`000P000?l00`00o`00o`000?l000020?n0
0080o`000`3oP0020?l000D00?l00P3o00060000o`030?n00000o`000?l000H00?l00`3o000K003o
0080o`000P00o`000`3o00000?l0003o00020?l000T00?l03_ooool4003o00<0o`000P3oP0070?l0
0080oh000P3o00020?n000030000o`00o`000?l000D0o`000P3oP0020000o`800?l00`3o00060000
o`80oh000`3o0004003o00<0o`001P00o`050000o`80oh000P3o0003003o0080o`001@000?l40?n0
0080o`000P3oP0000`3o0000oh000?n000090?l000H0oh0000<0003o0?n0003oP0000P3oP0050?l0
00@0oh000P3o00000`3oP000o`000?l000060?l000<0003o0P3oP00>ooooo`800?l00P000?l00`3o
0000oh000?n000070?n000@0o`000P000?l20?n000030?l0003oP000oh000080003o0P3o00070?n0
0080003o00<0oh000000o`000?l0103oP0020?l000030?n000000?l0003o0080003o0P3oP0020?l0
00T0oh0000<0o`000?n0003oP0002@3oP0040000o`030?n0003o0000o`000080oh00103o00030?n0
00@0o`000P000?l3003o0080oh000P3o00030?n00080o`000P3oP0020?l000<0003o0P3o0002003o
00D0003o1000o`070?n001Sooooo000Fooooo`D00?l02@3o0002003o00T0o`001`00o`0R0?l00080
oh001@3o0002003o01@0o`001000o`000`3o00000?l0003o0002003o00@0o`0000<0oh000?l0003o
0000103o00000`3oP000o`000?l000040?n000h0o`000P000?l>ooooo`@00?l000<0o`00003o0000
o`000P00o`050?l000P00?l000<0o`00003o0000o`001`00o`040?l00080003o2P00o`020?n00080
o`000P3oP0020000o`L00?l00P3o00000`3oP0000?l0003o0007003o00H0003o00<0oh00003o0000
o`000P00o`070?l001T00?l01P3o00000`000?l00?l0003o00020?l000L00?l00P3o000>ooooo`D0
0?l02`3o00020?n00080o`000P3oP003003o00L0o`000P00o`050?l000@0003o103oP0000`3o0000
oh000?n00006003o00<0o`001000o`070000o`L0o`000P000?l20?n000030000o`3oP000oh000080
oh000P3o00020?n000030?l0003oP000oh0000/0o`00103oP0000`000?l0oh000?n000020?n000<0
o`001P3oP0020?l000030?n0003o0000o`000080o`000P3oP0020?l000030?n000000?l0003o0080
oh003_ooool20?l00080oh0000<00?l00?n0003oP0001P3oP0000`000?l0o`000?l000020?l000L0
oh000P000?l50?l000d0oh000P3o00050000oa80oh000P3o00070?n00080o`0000<0oh000?l0003o
0000103oP0020?l000<0oh00103o00020000o`030?n0003o0000o`0000L0oh0010000?l20?l000<0
003o103o00020?n000<0003o0P3o00090?n001Sooooo000Fooooo`D00?l02@3o0002003o00T0o`00
1`00o`0R0?l00080oh001@3o0002003o01@0o`001000o`000`3o00000?l0003o0002003o00@0o`00
00<0oh000?l0003o0000103o00000`3oP000o`000?l000040?n000h0o`000P000?l>ooooo`@00?l0
00<0o`00003o0000o`000P00o`050?l000P00?l000<0o`00003o0000o`001`00o`040?l00080003o
2P00o`020?n00080o`000P3oP0020000o`L00?l00P3o00000`3oP0000?l0003o0007003o00H0003o
00<0oh00003o0000o`000P00o`070?l001T00?l01P3o00000`000?l00?l0003o00020?l000L00?l0
0P3o000>ooooo`D00?l02`3o00020?n00080o`000P3oP003003o00L0o`000P00o`050?l000@0003o
103oP0000`3o0000oh000?n00006003o00<0o`001000o`070000o`L0o`000P000?l20?n000030000
o`3oP000oh000080oh000P3o00020?n000030?l0003oP000oh0000/0o`00103oP0000`000?l0oh00
0?n000020?n000<0o`001P3oP0020?l000030?n0003o0000o`000080o`000P3oP0020?l000030?n0
00000?l0003o0080oh003_ooool20?l00080oh0000<00?l00?n0003oP0001P3oP0000`000?l0o`00
0?l000020?l000L0oh000P000?l50?l000d0oh000P3o00050000oa80oh000P3o00070?n00080o`00
00<0oh000?l0003o0000103oP0020?l000<0oh00103o00020000o`030?n0003o0000o`0000L0oh00
10000?l20?l000<0003o103o00020?n000<0003o0P3o00090?n001Sooooo000Fooooo`80oh001`3o
0005003o01P0o`000`00o`090?l000800?l0<P3o0002003o00@0o`0000<0oh000?l0003o00000P3o
00030?n00080o`00103oP00<0?l00080oh000P00o`0>ooooo`@00?l000<0o`00003o0000o`005`00
o`020?l00080oh000P3o00020000o`X00?l00P3oP0040?l00080003o2@00o`050?l000D00?l01`00
0?l4003o00D0o`000P00o`020?l001L00?l01P3o00000`000?l00?l0003o000;003o00kooooo2`00
o`050?l00080oh00103o00000`00o`00oh000?n000070?l000800?l01`3o00020000o`@0oh0000<0
o`000000o`000?l03@00o`070000o`L0o`002@3oP0020000o`80oh000`000?l40?l00080oh002P3o
00040?n000<0o`000P000?l2003o00@0o`0000<0oh000?l0003o00000P3o00020?n00080o`0000<0
oh000000o`000?l00P000?l>ooooo`@0o`002P3oP0020?l000d0oh0000<0o`000?n0003oP0003@3o
P0020?l000T0oh0000<0o`000?n0003oP0005`3oP0020?l000@0oh000`3o00090?n00080o`000P00
0?l70?n00080003o0P00o`000`000?l0o`000?l000020?l000@00?l000<0o`000?n0003oP0000P3o
00070?n00080o`006?ooool001Kooooo0P3oP0070?l000D00?l0603o0003003o00T0o`000P00o`0b
0?l000800?l0103o00000`3oP000o`000?l000020?l000<0oh000P3o00040?n000`0o`000P3oP002
003o00kooooo1000o`000`3o00000?l0003o000G003o0080o`000P3oP0020?l00080003o2P00o`02
0?n000@0o`000P000?l9003o00D0o`001@00o`070000o`@00?l01@3o0002003o0080o`005`00o`06
0?l000030000o`00o`000?l000/00?l03_ooool;003o00D0o`000P3oP0040?l00003003o003oP000
oh0000L0o`000P00o`070?l00080003o103oP0000`3o0000003o0000o`0=003o00L0003o1`3o0009
0?n00080003o0P3oP0030000o`@0o`000P3oP00:0?l000@0oh000`3o00020000o`800?l0103o0000
0`3oP000o`000?l000020?l00080oh000P3o00000`3oP000003o0000o`020000o`kooooo103o000:
0?n00080o`003@3oP0000`3o0000oh000?n0000=0?n00080o`002@3oP0000`3o0000oh000?n0000G
0?n00080o`00103oP0030?l000T0oh000P3o00020000o`L0oh000P000?l2003o00030000o`3o0000
o`000080o`001000o`000`3o0000oh000?n000020?l000L0oh000P3o000Hooooo`005_ooool20?l0
00800?l02@3o00000`00o`00o`000?l0000G0?l000800?l0903o00020?n000D0o`000P3oP00@0?l0
00800?l02`3o00000`3oP000o`000?l000020?n000@0o`0000<0oh000?l0003o00000P3o00020?n0
00<0o`000P3oP0020?l000kooooo3@00o`000`3o00000?l0003o0006003o00030?l00000o`000?l0
00800?l01@3o00020?n00080o`000P000?l:003o00H0o`000P000?l7003o00<0o`000P00o`020?l0
00D00?l00P3o00040000o`030?n00000o`000?l000800?l01@3o0009003o0080o`005000o`020?l0
00h00?l03_ooool=003o00L0o`000P000?l70?l000<00?l02@3o00060000o`030?l000000?l0003o
00L00?l00P3o0002003o0080o`001`000?l30?l000@0oh001@3o00060000o`D0oh0010000?l20?n0
00<0o`001000o`050?l00080oh000`3o00020000o`800?l00P3o00020?n000<0003o103oP0020?l0
00030?n000000?l0003o0080003o3_ooool50?l000T0oh00103o00020?n00080003o1@3oP0030?l0
00H0oh000P000?l50?n00080o`001`3oP0020000o`/0oh000`3o00020?n00080o`002P3oP0020?l0
00@0oh000P3o00000`00o`00oh000?n000040?n00080003o2P3oP0070000o`@0o`000P00o`030?l0
0080oh000P000?l40?n000<0o`000P3oP00Hooooo`005_ooool20?l000800?l02@3o00000`00o`00
o`000?l0000G0?l000800?l0903o00020?n000D0o`000P3oP00@0?l000800?l02`3o00000`3oP000
o`000?l000020?n000@0o`0000<0oh000?l0003o00000P3o00020?n000<0o`000P3oP0020?l000ko
oooo3@00o`000`3o00000?l0003o0006003o00030?l00000o`000?l000800?l01@3o00020?n00080
o`000P000?l:003o00H0o`000P000?l7003o00<0o`000P00o`020?l000D00?l00P3o00040000o`03
0?n00000o`000?l000800?l01@3o0009003o0080o`005000o`020?l000h00?l03_ooool=003o00L0
o`000P000?l70?l000<00?l02@3o00060000o`030?l000000?l0003o00L00?l00P3o0002003o0080
o`001`000?l30?l000@0oh001@3o00060000o`D0oh0010000?l20?n000<0o`001000o`050?l00080
oh000`3o00020000o`800?l00P3o00020?n000<0003o103oP0020?l000030?n000000?l0003o0080
003o3_ooool50?l000T0oh00103o00020?n00080003o1@3oP0030?l000H0oh000P000?l50?n00080
o`001`3oP0020000o`/0oh000`3o00020?n00080o`002P3oP0020?l000@0oh000P3o00000`00o`00
oh000?n000040?n00080003o2P3oP0070000o`@0o`000P00o`030?l00080oh000P000?l40?n000<0
o`000P3oP00Hooooo`005_ooool20?l000800?l02@3o00000`00o`00o`000?l000020?l000800?l0
6`3o00000`00o`00o`000?l00002003o01/0o`000P3oP0030?l00080oh000P3o00030?n000T0o`00
0P000?l;0?l00080oh000`3o00020?n000@0o`0000<0oh000?l0003o00000P3o00020?n00080o`00
00<0oh000000o`000?l00P3o000>ooooo`d00?l000<0o`00003o0000o`002`00o`050?l00080oh00
0P3o00020000o`X00?l0203o0009003o00030?l00000o`000?l000L00?l00P3oP0020?l000<0oh00
4P00o`020?l001800?l01@3o0004003o0080o`001`00o`0>ooooo`80003o1@00o`020?l000@00?l0
1`3o00020000o`<0o`0010000?l5003o0080o`000P3oP0000`3o0000003o0000o`070000o`80oh00
1000o`090?l000L0003o0P3o00030?n000@0o`000P3oP0000`3o0000003o0000o`040000o`D0oh00
10000?l20?n000030?l00000o`000?l000@0o`000P3oP0080?l000@00?l00P3o00050000o`@0oh00
1`000?l>ooooo`80oh000`3o00090?n00080003o0P3o00070?n000D0o`00103oP0020?l000D0003o
0P3oP0020?l000<0oh000P000?l20?l00080003o1`3oP0020000o`80oh000`3o000;0?n00080o`00
0`3oP0020000o`80oh001@3o00040?n00080003o2P3oP0020000o`L0oh000P3o0002003o0080o`00
3P3oP00Hooooo`005_ooool4003o00h0o`000P00o`0E0?l000@00?l05@3o0002003o0080oh001`3o
00040?n000<0o`000P3oP00>0?l00080003o403o00020?n000@0o`0000<0oh000?l0003o00001`3o
0002003o0080o`003_oooolM003o00L0o`003000o`060?l000L00?l00P3o0009003o00T0o`0000<0
oh00003o0000o`000P00o`020?l000`00?l00P3o0004003o00<0o`002`00o`070?l00080oh00103o
0005003o00kooooo0P000?l20?l000<00?l01`3o0002003o0080o`001@00o`0;0?l000800?l00`3o
00090000o`80oh001P00o`070?l00080oh001@000?l90?l000<0003o0P3oP0040000o`D0oh000P3o
00040?n000030?l00000o`000?l000800?l0103oP0080?l000H00?l00P3o00030000o`@0oh001`00
0?l>ooooo`80oh002@3o00030?n00080003o2`3oP0020?l000L0oh000`000?l90?n000@00?l00P00
0?l2003o00D0o`00103oP0000`3o0000oh000?n000040?n000030?l0003oP000oh000080oh000P3o
00020000o`030?n0003o0000o`0000@0oh000`3o00040?n00080o`001@000?l70?n00080o`001`3o
P0020?l000800?l00P3o00070?n00080o`0000<0oh000000o`000?l00P000?lHooooo`005_ooool4
003o00h0o`000P00o`0E0?l000@00?l05@3o0002003o0080oh001`3o00040?n000<0o`000P3oP00>
0?l00080003o403o00020?n000@0o`0000<0oh000?l0003o00001`3o0002003o0080o`003_oooolM
003o00L0o`003000o`060?l000L00?l00P3o0009003o00T0o`0000<0oh00003o0000o`000P00o`02
0?l000`00?l00P3o0004003o00<0o`002`00o`070?l00080oh00103o0005003o00kooooo0P000?l2
0?l000<00?l01`3o0002003o0080o`001@00o`0;0?l000800?l00`3o00090000o`80oh001P00o`07
0?l00080oh001@000?l90?l000<0003o0P3oP0040000o`D0oh000P3o00040?n000030?l00000o`00
0?l000800?l0103oP0080?l000H00?l00P3o00030000o`@0oh001`000?l>ooooo`80oh002@3o0003
0?n00080003o2`3oP0020?l000L0oh000`000?l90?n000@00?l00P000?l2003o00D0o`00103oP000
0`3o0000oh000?n000040?n000030?l0003oP000oh000080oh000P3o00020000o`030?n0003o0000
o`0000@0oh000`3o00040?n00080o`001@000?l70?n00080o`001`3oP0020?l000800?l00P3o0007
0?n00080o`0000<0oh000000o`000?l00P000?lHooooo`005_ooool9003o00T0o`000P00o`0P0?l0
00800?l0303o0002003o00@0o`0000<0oh000?l0003o00000P3o00040?n000<0o`000P3oP00P0?l0
0080oh00103o00000`3oP000o`000?l000060?l000030?n0003o0000o`000080oh003_oooolI003o
0080o`000P000?l90?l000X00?l01P3o000B003o00L0o`000P00o`050?l00080003o2`00o`030?l0
00@00?l00P3o000<003o00d0o`001@00o`0>ooooo`80003o0P3o0003003o00T0o`002@00o`090?l0
00800?l00P3o00000`3oP000o`000?l000020000o`80o`000P3oP0030000o`H00?l02P3o00020?n0
0080003o0P3o00000`00o`00o`000?l000020?l000800?l01@000?l20?n00080003o1@3oP0020?l0
0080oh000P000?l3003o00L0o`00103oP0020?l000<00?l0103o00020?n000D0003o1`3oP0020000
o`kooooo10000?l30?l00080oh000P3o00020000o`030?l0003oP000oh000080oh000P3o00020000
o`030?n0003o0000o`000080003o0P00o`000`3oP000o`000?l000020?n000@0003o303oP0020?l0
00@00?l000<0oh000?l0003o00003@3oP0000`3o00000?l0003o00020?n00080o`002`3oP0000`3o
0000oh000?n00002003o0080o`000P3oP0000`000?l0oh000?n000040?n00080o`002P3oP0040?l0
0080003o1@3oP0040?l000D0003o6?ooool001Kooooo2@00o`090?l000800?l0803o0002003o00`0
o`000P00o`040?l000030?n0003o0000o`000080o`00103oP0030?l00080oh00803o00020?n000@0
o`0000<0oh000?l0003o00001P3o00000`3oP000o`000?l000020?n000kooooo6@00o`020?l00080
003o2@3o000:003o00H0o`004P00o`070?l000800?l01@3o00020000o`/00?l00`3o0004003o0080
o`003000o`0=0?l000D00?l03_ooool20000o`80o`000`00o`090?l000T00?l02@3o0002003o0080
o`0000<0oh000?l0003o00000P000?l20?l00080oh000`000?l6003o00X0o`000P3oP0020000o`80
o`0000<00?l00?l0003o00000P3o0002003o00D0003o0P3oP0020000o`D0oh000P3o00020?n00080
003o0`00o`070?l000@0oh000P3o0003003o00@0o`000P3oP0050000o`L0oh000P000?l>ooooo`@0
003o0`3o00020?n00080o`000P000?l00`3o0000oh000?n000020?n00080o`000P000?l00`3oP000
o`000?l000020000o`800?l000<0oh000?l0003o00000P3oP0040000o``0oh000P3o0004003o0003
0?n0003o0000o`0000d0oh0000<0o`00003o0000o`000P3oP0020?l000/0oh0000<0o`000?n0003o
P0000P00o`020?l00080oh0000<0003o0?n0003oP000103oP0020?l000X0oh00103o00020000o`D0
oh00103o00050000oaSooooo000Fooooo`@0o`001@00o`0@0?l000800?l01`3o00020?n00100o`00
0P00o`0B0?l000030?n0003o0000o`000080o`000P000?l20?n000<0o`000P3oP00=0?l000<0oh00
3P3o00040?n000@0o`0000<0oh000?l0003o00002`3o000>ooooo`@00?l01@3o000@003o0080o`00
0P000?l90?l000X00?l01P3o0007003o0080oh001@3o0004003o00D0o`000P3oP002003o00D0o`00
0P000?l9003o0080o`005`00o`0;0?l000030000o`3o0000o`000080o`003_ooool20000o`80o`00
00<00?l00?l0003o0000103o00020000o`<0o`001P00o`000`3o00000?l0003o0002003o0080003o
0`3o00020000o`80o`00303oP0020000o`80o`001000o`0:0?l00080oh000P000?l3003o00@0o`00
0P00o`050000o`80oh000P3o00030?n000@0o`000P000?l2003o00L0o`001@000?l40?n000<00?l0
0P3oP0040?l000D0003o0P3oP0020000o`030?n000000?l0003o0080003o3_ooool2003o00D0o`00
0P3oP0020?l000`0oh001P00o`030?l00080003o0P3o00090?n00080o`000`3oP0020?l000800?l0
0P3o00000`00o`00o`000?l000020000oaL0oh000P000?l20?n00080003o00<0o`000?n0003oP000
1000o`050?n00080003o0P3oP0020?l000030000o`3oP000oh0000/0oh000P000?l00`3oP000003o
0000o`020?n000@0o`001@000?lHooooo`005_ooool40?l000D00?l0403o0002003o00L0o`000P3o
P00@0?l000800?l04P3o00000`3oP000o`000?l000020?l00080003o0P3oP0030?l00080oh003@3o
00030?n000h0o`00103oP0040?l000030?n0003o0000o`0000/0o`003_ooool4003o00D0o`004000
o`020?l00080003o2@3o000:003o00H0o`001`00o`020?n000D0o`001000o`050?l00080oh000P00
o`050?l00080003o2@00o`020?l001L00?l02`3o00000`000?l0o`000?l000020?l000kooooo0P00
0?l20?l00003003o003o0000o`0000@0o`000P000?l30?l000H00?l000<0o`00003o0000o`000P00
o`020000o`<0o`000P000?l20?l000`0oh000P000?l20?l000@00?l02P3o00020?n00080003o0`00
o`040?l000800?l01@000?l20?n00080o`000`3oP0040?l00080003o0P00o`070?l000D0003o103o
P003003o0080oh00103o00050000o`80oh000P000?l00`3oP000003o0000o`020000o`kooooo0P00
o`050?l00080oh000P3o000<0?n000H00?l00`3o00020000o`80o`002@3oP0020?l000<0oh000P3o
0002003o0080o`0000<00?l00?l0003o00000P000?lG0?n00080003o0P3oP0020000o`030?l0003o
P000oh0000@00?l01@3oP0020000o`80oh000P3o00000`000?l0oh000?n0000;0?n00080003o00<0
oh000000o`000?l00P3oP0040?l000D0003o6?ooool001Kooooo1@3o0006003o00D0o`000P3oP005
0?l000800?l0103o00000`3oP000o`000?l000020?l00080oh00903o00000`3oP000o`000?l00004
0?n000<0o`000P3oP00V0?l000<0oh00203o00000`3oP000o`000?l000020?l000kooooo1000o`03
0?l001@00?l00P000?l90?l000`00?l0103o0007003o0080oh000`3o0002003o00T0o`000P3oP002
003o00D0o`000P000?l5003o00H0o`0000<00?l00?l0003o00005000o`030?l000800?l02`3o000>
ooooo`80003o0P3o00000`00o`00o`000?l000040?l00080oh001@3o0004003o00030?l00000o`00
0?l000800?l01`000?l40?l000030000o`00o`000?l00080003o0P3o00050000o`80o`001000o`0<
0?l000@0003o0`3o00040?n000<0003o103oP0020?l000<0oh00103o00020000o`D0o`000P3oP002
0?l000D0003o103oP0070?l000L0003o0P3oP0030?l000@0003o3_ooool20?l000<0oh000P3o0002
0?n00080o`000P3oP0000`3o0000003o0000o`060000o`030?n0003o0000o`000080003o0P00o`00
0`3oP000o`000?l000020?l000L0oh00103o00020?n000030?l0003oP000oh000080o`000P3oP002
0?l000030?n0003o0000o`0000d0oh0000<0003o0?n0003oP0001P3oP0030000o`@0oh000P000?l0
0`3o0000oh000?n00004003o0080o`0000<0003o0?n0003oP0001`000?l?0?n000<0003o1`3oP002
0000o`80oh006?ooool001Kooooo1@3o0006003o00D0o`000P3oP0050?l000800?l0103o00000`3o
P000o`000?l000020?l00080oh00903o00000`3oP000o`000?l000040?n000<0o`000P3oP00V0?l0
00<0oh00203o00000`3oP000o`000?l000020?l000kooooo1000o`030?l001@00?l00P000?l90?l0
00`00?l0103o0007003o0080oh000`3o0002003o00T0o`000P3oP002003o00D0o`000P000?l5003o
00H0o`0000<00?l00?l0003o00005000o`030?l000800?l02`3o000>ooooo`80003o0P3o00000`00
o`00o`000?l000040?l00080oh001@3o0004003o00030?l00000o`000?l000800?l01`000?l40?l0
00030000o`00o`000?l00080003o0P3o00050000o`80o`001000o`0<0?l000@0003o0`3o00040?n0
00<0003o103oP0020?l000<0oh00103o00020000o`D0o`000P3oP0020?l000D0003o103oP0070?l0
00L0003o0P3oP0030?l000@0003o3_ooool20?l000<0oh000P3o00020?n00080o`000P3oP0000`3o
0000003o0000o`060000o`030?n0003o0000o`000080003o0P00o`000`3oP000o`000?l000020?l0
00L0oh00103o00020?n000030?l0003oP000oh000080o`000P3oP0020?l000030?n0003o0000o`00
00d0oh0000<0003o0?n0003oP0001P3oP0030000o`@0oh000P000?l00`3o0000oh000?n00004003o
0080o`0000<0003o0?n0003oP0001`000?l?0?n000<0003o1`3oP0020000o`80oh006?ooool001Ko
oooo2@3o0002003o00D0o`000P3oP0040?l000030000o`00o`000?l000@0o`0000<0oh000?l0003o
00000P3o00020?n000<0o`000P3oP00O0?l000030?n0003o0000o`000080oh001@3o00020?n000H0
o`0000<00?l00?l0003o00000P3o0004003o00<0o`000P3oP0050?l000@00?l02@3o00020?n000T0
o`0000<0oh000?l0003o00000P3o000>ooooo`80o`000P00o`030?l001H00?l02@3o000>003o0080
o`001`00o`020?n000<0o`000P000?l50?l00080oh000P3o00020?n000<00?l00P3o0009003o00@0
o`000P3oP0030?l00080003o4P00o`030?l000800?l02@3o00020?n000kooooo0P3oP0020?l00003
003o003o0000o`0000@0o`000P3oP0030?l000@00?l00`3o0004003o00L0003o103o00000`000?l0
0?l0003o00040?l000D0003o103o00020?n000<0o`000P00o`050?l00080oh000P3o0002003o00<0
o`00103oP0030000o`@0oh000P3o0002003o00030?n0003o0000o`000080o`000P00o`050?l00080
oh000P3o00050000o`80oh000P000?l30?l000H0003o00<0oh000?l0003o00000P000?l20?n00080
o`0000<0oh000000o`000?l00P000?l>ooooo`80o`001`3oP0020?l00080oh0000<0o`000000o`00
0?l01P000?l30?n00080003o1@3o00020000o`80o`000P3oP0000`3o0000oh000?n000020?l000@0
oh0000<0o`000?n0003oP0000P3o00020?n000<0003o0P3o00040?n000L0o`000P3oP0000`3o0000
oh000?n0000?0?n000<0o`000P3oP002003o0080o`00403oP0000`3o0000oh000?n0000@0?n00080
o`000P3oP00Hooooo`005_ooool70?l000800?l03@3o00000`000?l0o`000?l000070?l000@0oh00
903o00000`3oP000o`000?l000020?l00080oh000`3o00020?n000@0o`000`00o`020?l000800?l0
0P3o0002003o00030?l00000o`000?l000L0o`001000o`0<0?l00080003o2`3o000>ooooo`L0o`00
5P00o`050?l001800?l00P3o0007003o0080oh000`3o00020000o`D0o`000P3oP0040?l000h00?l0
103o00020?n000<0o`000P000?l@003o00@0o`001@00o`070?l00080oh003_ooool20?n000T0o`00
0P3oP0050?l000L00?l00P3o00070000o`@0o`0000<0003o0?l0003o00000P3o00020?n00080003o
00<0oh000000o`000?l00P000?l2003o00D0o`000P00o`040?l000<0oh000P000?l50?l00080oh00
0P000?l50?n000@0o`000P00o`050?l000800?l01@3o00020?n00080o`001`000?l20?n000D0o`00
10000?l00`3oP000o`000?l000020000o`80o`000P000?l00`3oP000003o0000o`020000o`kooooo
0P3o000>0?n000H0003o00<0oh000000o`000?l00P00o`030?n00080003o1P3oP0000`3o0000oh00
0?n0000I0?n000@0o`000P3oP0000`000?l0oh000?n0000@0?n00080o`000P3oP0040?l000`0oh00
0P3o000<0?n000H0003o1`3oP00Hooooo`005_ooool70?l000800?l03@3o00000`000?l0o`000?l0
00070?l000@0oh00903o00000`3oP000o`000?l000020?l00080oh000`3o00020?n000@0o`000`00
o`020?l000800?l00P3o0002003o00030?l00000o`000?l000L0o`001000o`0<0?l00080003o2`3o
000>ooooo`L0o`005P00o`050?l001800?l00P3o0007003o0080oh000`3o00020000o`D0o`000P3o
P0040?l000h00?l0103o00020?n000<0o`000P000?l@003o00@0o`001@00o`070?l00080oh003_oo
ool20?n000T0o`000P3oP0050?l000L00?l00P3o00070000o`@0o`0000<0003o0?l0003o00000P3o
00020?n00080003o00<0oh000000o`000?l00P000?l2003o00D0o`000P00o`040?l000<0oh000P00
0?l50?l00080oh000P000?l50?n000@0o`000P00o`050?l000800?l01@3o00020?n00080o`001`00
0?l20?n000D0o`0010000?l00`3oP000o`000?l000020000o`80o`000P000?l00`3oP000003o0000
o`020000o`kooooo0P3o000>0?n000H0003o00<0oh000000o`000?l00P00o`030?n00080003o1P3o
P0000`3o0000oh000?n0000I0?n000@0o`000P3oP0000`000?l0oh000?n0000@0?n00080o`000P3o
P0040?l000`0oh000P3o000<0?n000H0003o1`3oP00Hooooo`005_ooool2003o00D0o`000P00o`0=
0?l00003003o003o0000o`0000L0o`00103oP00;0?l000030?n0003o0000o`0000H0o`0000<0oh00
0?l0003o00001P3o00000`3oP000o`000?l000040?l000030?n0003o0000o`000080003o1@3o0002
0?n000@0o`000P00o`0B0?l000<00?l01@3o00020?n000T0o`000P3oP0020?l00080oh0000<00?l0
0?l0003o00000P3o000>ooooo`L0o`00=P00o`020?n000<0o`000P3oP0050000o`80oh00103o000>
003o00@0o`000P3oP0030?l00080oh004000o`040?l000T00?l01@3o000>ooooo`80003o2@3o0002
0?n000D0o`001000o`050?l00080oh0000<0o`000000o`000?l0103oP0020?l000030?n0003o0000
o`000080o`000P3oP0050000o`80oh000P00o`050?l000@00?l00`3o00020?n000L0o`000P3oP004
0000o`<0oh001P000?l00`3o00000?l0003o00020?l000@00?l00`3o00020?n00080o`001`000?l2
0?n000D0o`0010000?l3003o00H0003o00<0oh000000o`000?l00P000?l>ooooo`80003o3P3oP006
0000o`030?n00000o`000?l000800?l00P3o00030000o`H0oh0000<0003o0?n0003oP000603oP000
0`3o0000oh000?n000020?n000<0003o103oP0050000o`80oh00103o00030?n00080o`000P3oP004
0?l000D0oh000P000?l50?n000800?l00`3o00040?n00080o`001@3oP0040000o`80oh000`000?l2
0?n001Sooooo000Fooooo`800?l01@3o0002003o00d0o`0000<00?l00?l0003o00001`3o00040?n0
00/0o`0000<0oh000?l0003o00001P3o00000`3oP000o`000?l000060?l000030?n0003o0000o`00
00@0o`0000<0oh000?l0003o00000P000?l50?l00080oh00103o0002003o0180o`000`00o`050?l0
0080oh002@3o00020?n00080o`000P3oP0000`00o`00o`000?l000020?l000kooooo1`3o000f003o
0080oh000`3o00020?n000D0003o0P3oP0040?l000h00?l0103o00020?n000<0o`000P3oP00@003o
00@0o`002@00o`050?l000kooooo0P000?l90?l00080oh001@3o0004003o00D0o`000P3oP0000`3o
0000003o0000o`040?n00080o`0000<0oh000?l0003o00000P3o00020?n000D0003o0P3oP002003o
00D0o`001000o`030?l00080oh001`3o00020?n000@0003o0`3oP0060000o`030?l00000o`000?l0
0080o`001000o`030?l00080oh000P3o00070000o`80oh001@3o00040000o`<00?l01P000?l00`3o
P000003o0000o`020000o`kooooo0P000?l>0?n000H0003o00<0oh00003o0000o`000P00o`020?l0
00<0003o1P3oP0000`000?l0oh000?n0000H0?n000030?l0003oP000oh000080oh000`000?l40?n0
00D0003o0P3oP0040?l000<0oh000P3o00020?n000@0o`001@3oP0020000o`D0oh000P00o`030?l0
00@0oh000P3o00050?n000@0003o0P3oP0030000o`80oh006?ooool001Kooooo1000o`030?l00080
0?l03P3o00020?n000L0o`000P3oP0040?l000030?n0003o0000o`0001d0o`000P000?l00`3oP000
o`000?l000040?n000D0o`000P00o`0;0?l000800?l02@3o0003003o00D0o`000P3oP00=0?l00080
003o1@3o000>ooooo`L0o`000P000?lY003o0080o`002@00o`050?l00080oh0010000?l00`3o0000
oh000?n000040?l000h00?l0103o0002003o00D0o`004000o`040?l000T00?l01@3o000>ooooo`80
0?l02@3o00020000oa00o`0000<0oh000000o`000?l00P3oP0040?l000030?n0003o0000o`000080
o`001@3oP0020000o`80o`000P00o`020?l000<00?l01`3o00020?n000@0o`0000<0oh000?l0003o
00000P3oP0040000o`<0oh001P000?l30?l000L00?l01P3o00030000o`H0oh001`3o00020000o`<0
0?l01P000?l00`3oP000003o0000o`020000o`kooooo103oP0000`00o`00o`000?l000070?n00080
o`000P3oP0040000o`030?n000000?l0003o00@00?l01@000?l20?n000D0003o2`3oP0020?l00080
oh0000<0o`000?n0003oP0000P3oP0020?l000L0oh000`000?l40?n00080003o1@3oP0020?l001L0
oh000P3o0002003o0080oh0000<0o`000?n0003oP0000P3o00040?n000030?l0003oP000oh0000/0
oh006?ooool001Kooooo1000o`030?l000800?l03P3o00020?n000L0o`000P3oP0040?l000030?n0
003o0000o`0001d0o`000P000?l00`3oP000o`000?l000040?n000D0o`000P00o`0;0?l000800?l0
2@3o0003003o00D0o`000P3oP00=0?l00080003o1@3o000>ooooo`L0o`000P000?lY003o0080o`00
2@00o`050?l00080oh0010000?l00`3o0000oh000?n000040?l000h00?l0103o0002003o00D0o`00
4000o`040?l000T00?l01@3o000>ooooo`800?l02@3o00020000oa00o`0000<0oh000000o`000?l0
0P3oP0040?l000030?n0003o0000o`000080o`001@3oP0020000o`80o`000P00o`020?l000<00?l0
1`3o00020?n000@0o`0000<0oh000?l0003o00000P3oP0040000o`<0oh001P000?l30?l000L00?l0
1P3o00030000o`H0oh001`3o00020000o`<00?l01P000?l00`3oP000003o0000o`020000o`kooooo
103oP0000`00o`00o`000?l000070?n00080o`000P3oP0040000o`030?n000000?l0003o00@00?l0
1@000?l20?n000D0003o2`3oP0020?l00080oh0000<0o`000?n0003oP0000P3oP0020?l000L0oh00
0`000?l40?n00080003o1@3oP0020?l001L0oh000P3o0002003o0080oh0000<0o`000?n0003oP000
0P3o00040?n000030?l0003oP000oh0000/0oh006?ooool001Kooooo1000o`000`3o00000?l0003o
0002003o00D0o`000P000?l2003o02d0o`000P00o`050?l00080003o0`3o00040?n002h0o`000P00
0?l40?l000D00?l00P3o000>ooooo`L0o`000P000?l5003o0080o`00;@00o`050?l00080oh001000
0?l30?n000@0o`002@00o`090?l000<00?l0103o000@003o00@0o`002@00o`030?l00080oh003_oo
ool20000o`T0o`000P000?l50?l00080oh00303o00020?n00080003o0P3o00000`000?l0oh000?n0
00020?l00080oh000`3o0002003o00H0o`000`00o`0=0?l000030?n0003o0000o`0000T0oh001000
0?l20?n000<0o`000P000?l5003o00H0o`000`000?l60?n000L0o`000P3oP0000`3o00000?l0003o
00060000o`030?n000000?l0003o0080003o3_ooool50?n00080o`002`3oP0040000o`030?n00000
0?l0003o00800?l00`3oP0040000o`80oh000`3o000=0?n000800?l00P3o00000`00o`00o`000?l0
00020?l000/0oh0000<0003o0?n0003oP0002@3oP002003o0100oh000P000?l70?n000D0o`000P3o
P004003o0080o`003P3oP00Hooooo`005_ooool2003o00<0o`001000o`090?l000D00?l02@3o0002
0?n000800?l08P3o00020000o`D0o`000P3oP0030?l000800?l03`3o00000`3oP000o`000?l00006
0?l000030?n0003o0000o`000080o`000P3oP00@0?l000800?l000<0o`00003o0000o`000P3o000>
ooooo`80003o1@3o0007003o00@0o`001`00o`020?l000T00?l00P3o000G003o00L0o`000P000?l3
0?l00080oh00103o0005003o00H0o`0000<0003o0?l0003o00000P00o`020?l000<00?l0103o000M
003o00<0o`000P3oP00>ooooo`80003o0P00o`070?l00080003o2@3o00000`3oP000o`000?l00002
0000o`80oh000`3o00070000o`80oh000P3o00040?n000030?l00000o`000?l000H0o`0000<00?l0
0?l0003o00003@3o00000`3oP000o`000?l000090?n000H0003o0`3o00020000o`L00?l0103o0003
0000o`H0oh0000<0o`00003o0000o`000P3oP0050?l000800?l00P000?l50?l000@0003o3_ooool4
0?n000<00?l00P3oP0040?l000030?n0003o0000o`000080003o0P3oP0020000o`<0oh001000o`09
0?n000030?l0003oP000oh0000@0oh000P000?l30?l00080oh000P3o0002003o00030000o`3o0000
o`000080o`000P3oP0020?l000D0oh000P3o00090?n000030?l0003oP000oh0000800?l05`3oP002
0000o`80o`000P00o`000`3o0000oh000?n00002003o00D0o`00103oP0020?l000L0oh006?ooool0
01Kooooo0P00o`030?l000@00?l02@3o0005003o00T0o`000P3oP002003o0280o`000P000?l50?l0
0080oh000`3o0002003o00l0o`0000<0oh000?l0003o00001P3o00000`3oP000o`000?l000020?l0
0080oh00403o0002003o00030?l00000o`000?l00080o`003_ooool20000o`D0o`001`00o`040?l0
00L00?l00P3o0009003o0080o`005`00o`070?l00080003o0`3o00020?n000@0o`001@00o`060?l0
00030000o`3o0000o`0000800?l00P3o0003003o00@0o`007@00o`030?l00080oh003_ooool20000
o`800?l01`3o00020000o`T0o`0000<0oh000?l0003o00000P000?l20?n000<0o`001`000?l20?n0
0080o`00103oP0000`3o00000?l0003o00060?l00003003o003o0000o`0000d0o`0000<0oh000?l0
003o00002@3oP0060000o`<0o`000P000?l7003o00@0o`000`000?l60?n000030?l00000o`000?l0
0080oh001@3o0002003o0080003o1@3o00040000o`kooooo103oP003003o0080oh00103o00000`3o
P000o`000?l000020000o`80oh000P000?l30?n000@00?l02@3oP0000`3o0000oh000?n000040?n0
0080003o0`3o00020?n00080o`000P00o`000`000?l0o`000?l000020?l00080oh000P3o00050?n0
0080o`002@3oP0000`3o0000oh000?n00002003o01L0oh000P000?l20?l000800?l000<0o`000?n0
003oP0000P00o`050?l000@0oh000P3o00070?n001Sooooo000Fooooo`800?l00P3o0005003o02l0
o`0000<0oh000?l0003o00002@3o00040000o`<0o`000P3oP0050?l000@00?l04P3o00020?n000L0
o`000P3oP00@0?l000<00?l0103o000>ooooo`@0o`002P00o`040?l00080003o0`00o`060?l000D0
0?l0103o000:003o00@0o`002@00o`040?l00003003o003o0000o`0000@0o`000`3oP0040?l00080
003o00<0o`00003o0000o`001P3o00000`3oP000o`000?l000040?l000<00?l0103o000M003o00D0
o`003_ooool20000o`800?l01`3o00020000o``0o`000P000?l30?n00080o`001`000?l40?n00080
003o0P3oP0000`000?l00?l0003o000=0?l000<00?l01P3o00000`3oP000o`000?l000060?n00003
0?l000000?l0003o00@0003o0P3oP0030?l00080003o0P00o`090?l000D0003o103oP0030?l00080
oh001@3o0002003o00L0o`0010000?l>ooooo`80o`001@00o`020000o`800?l01@000?l40?n000<0
o`00103oP0020?l000L0oh001`3o00090?n000@0o`0000<0oh000?l0003o00000P3oP0040?l000D0
oh000P3o00090?n000030?l00000o`000?l000d0o`000P000?l00`3oP000o`000?l000040?n00080
o`0000<0003o003o0000o`000P00o`050?l00080oh000P3o00020?n000030000o`3o0000o`000080
o`000P3oP0020?l000D0oh006?ooool001Kooooo0P00o`020?l000D00?l0;`3o00000`3oP000o`00
0?l000090?l000@0003o0`3o00020?n000D0o`001000o`0B0?l00080oh001`3o00020?n00100o`00
0`00o`040?l000kooooo103o000:003o00@0o`000P000?l3003o00H0o`001@00o`040?l000X00?l0
103o0009003o00@0o`0000<00?l00?l0003o0000103o00030?n000@0o`000P000?l00`3o00000?l0
003o00060?l000030?n0003o0000o`0000@0o`000`00o`040?l001d00?l01@3o000>ooooo`80003o
0P00o`070?l00080003o303o00020000o`<0oh000P3o00070000o`@0oh000P000?l20?n000030000
o`00o`000?l000d0o`000`00o`060?l000030?n0003o0000o`0000H0oh0000<0o`000000o`000?l0
10000?l20?n000<0o`000P000?l2003o00T0o`001@000?l40?n000<0o`000P3oP0050?l000800?l0
1`3o00040000o`kooooo0P3o0005003o0080003o0P00o`050000o`@0oh000`3o00040?n00080o`00
1`3oP0070?l000T0oh00103o00000`3oP000o`000?l000020?n000@0o`001@3oP0020?l000T0oh00
00<0o`00003o0000o`003@3o00020000o`030?n0003o0000o`0000@0oh000P3o00000`000?l00?l0
003o0002003o00D0o`000P3oP0020?l00080oh0000<0003o0?l0003o00000P3o00020?n00080o`00
1@3oP00Hooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2oooo
o`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooo
oooooh;ooooo003oooooooooooooP_ooool002;oooooL_l0000Wooooog7o00009ooooomao`0002Oo
ooooL_l0000Sooooo`008_ooool00ol0003oooooooooo`0iooooo`03o`00oooooooooooo01;ooooo
00?o003oooooooooool07Oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003ooooooooo
o`03oooooaKoo`001OoooolFo`3o00Cooooo5ol00?l4oooooaOoogl01?ooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo00;ooooo5_oo0005oooooaKo0?l01OoooolFo`00o`Cooooo5ooo
O`04ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolGool000Cooooo5_l0
o`05oooooaKo003o1OoooolFoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`02oooooaOoo`001?oooolGo`3o00Cooooo5_l00?l5oooooaKoogl01Oooool00ol0003ooooooooo
o`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00>_oooolGoomo00Cooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`02oooooaKoo`001OoooolFo`3o00Gooooo5_l00?l4
oooooaOoogl01?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05
oooooaKo003o1OoooolFoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02
oooooaOoo`001?oooolGo`3o00Cooooo5_l00?l5oooooaKoogl01Oooool00ol0003oooooooooo`0Q
ooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00>_oooolGoomo00Cooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00o`Cooooo5oooO`04oooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?l5oooo
oaKoogl01?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo0004oooo
oaOo0?l01?oooolFo`00o`Gooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03
o`000?oooooooooo00?ooooo5_oo000joooooaOoogl01?ooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o1?oooolGoomo00Cooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00o`Gooooo5_ooO`04ooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolGool000Cooooo5ol0o`04oooooaKo
003o1OoooolFoomo00Gooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool0
0ooooolFool003[ooooo5oooO`04ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
7OoooolFo`3o00Gooooo5_l00?l4oooooaOoogl01?ooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo01gooooo5_l0o`05oooooaKo003o1OoooolFoomo00Cooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`02oooooaOoo`001?oooolGo`3o01oooooo5_ooO`05ooooo`03o`00
0?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oo
oooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?l4oooooaOoogl01?oo
ool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o1Ooo
oolFoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02oooooaOoo`001?oo
oolGo`3o01oooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooo
oooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolF
o`3o00Gooooo5_l00?l4oooooaOoogl01?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo00;ooooo5ooo0004oooooaOo0?l07ooooolFoomo00Gooooo00?o0000ooooooooool08Oooool0
02;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`0MoooooaKo0?l01OoooolFo`00o`Cooooo5oooO`04ooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool00_oooolGool000Cooooo5ol0o`0OoooooaKoogl01Oooool00ol0
003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003o
ooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o1?oooolGoomo00Co
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoo
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02oooooaOoo`001?oooolGo`3o01oo
oooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo
5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo
5_l00?l4oooooaOoogl01?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo
5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo
5ooo0004oooooaOo0?l07ooooolFoomo00Gooooo00?o0000ooooooooool08Oooool002;ooooo00?o
0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`0MoooooaKo0?l07ooooolGoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`02oooooaOoo`001?oooolGo`3o01oooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000R
ooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool07OoooolFo`3o01oooooo5oooO`04ooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool00_oooolGool000Cooooo5ol0o`0OoooooaKoogl01Oooool00ol0003oooooooooo`0Q
ooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo01gooooo5_l0o`0OoooooaOoogl01?ooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo01gooooo5ol0o`0OoooooaKoogl01Oooool00ol0003oooooooooo`0Qoooo
o`008_ooool00ol0003o001oo`00O`03o`00OaKoo`00EOl007l00ol0003oooooooooo`0Uooooo`03
o`000?l007oo001o01go001o5_l0o`0Oo`00OaOoogl01?l007l00ol0003oooooooooo`0Uooooo`03
o`000?l007oo001o01go001o5_l0o`05o`00OaKo003o7ol007l00ol0003oooooooooo`0Uooooo`03
o`000?l007oo001o01go001o5ol0o`0Oo`00OaKoogl01Ol007l00ol0003oooooooooo`0Qooooo`00
8_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo01gooooo5_l0o`0jooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
7OoooolFo`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
7OoooolGo`3o01oooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oo
oooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07Ooo
oolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01Ooo
oolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaOo0?l07ooo
oolFoomo00Gooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolF
ool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooo
oooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool07OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`1CoooooaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003o
ooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01go
oooo5_l0o`0jooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Go
oooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Go
oooo00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo
000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool07OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`1CoooooaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03
oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`0j
ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lO
ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Gooooo00?o0000
ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03
o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000oooooooo
ool07OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo
0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1CoooooaKo
ogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00
EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`0jooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Gooooo00?o0000ooooooooool0
8Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?l007oo
001o00?o001o5_oo001Eo`00O`03o`000?oooooooooo02Gooooo00?o0000o`00Ool007l07Ol007lF
o`3o03[o001o00?o0000ooooooooool09Oooool00ol0003o001oo`00O`0Mo`00OaKo0?l01Ol007lF
o`00oaoo001o00?o0000ooooooooool09Oooool00ol0003o001oo`00O`1Co`00OaKoogl01Ol007l0
0ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`0jooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0DooooolFoomo00Gooooo00?o0000ooooooooool08Oooool002;o
oooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`0MoooooaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01go
oooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?o
oooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo
5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o03[ooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`1CoooooaKoogl01Oooool00ol0003ooooo
ooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`0jooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool0DooooolFoomo00Gooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000
ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0M
oooooaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`0j
ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Gooooo00?o0000
ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1Coooo
oaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKo
o`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`0jooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`00
0?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
7OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Gooooo00?o0000ooooooooool0
8Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1CoooooaKoogl01Ooo
ool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`0jooooo`03o`000?oooooo
oooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo
00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o
03[ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003o
ooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01go
oooo5_l0o`0jooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003o
ooooooooo`0Uooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool08Oooool002;ooooo
00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`0MoooooaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo027ooooo
000Rooooo`03o`000?l007oo001o00?o001o5_oo001Eo`00O`03o`000?oooooooooo02Gooooo00?o
0000o`00Ool007l07Ol007lFo`3o03[o001o00?o0000ooooooooool09Oooool00ol0003o001oo`00
O`1]o`00O`03o`000?oooooooooo02Gooooo00?o0000o`00Ool007l0K_l007l00ol0003ooooooooo
o`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo01gooooo5_l0o`0jooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06kooooo00?o0000
ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1^oooo
o`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o03[ooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000oooooooo
ool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00
EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOoo
ool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_oo
ool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool0
0ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0
003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003o
ooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003o
ooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003ooooo
ooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003ooooooooo
o`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]
ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Q
ooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]oooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qoooo
o`008_ooool00ol0003o001oo`00O`03o`00OaKoo`00EOl007l00ol0003oooooooooo`0Uooooo`03
o`000?l007oo001o06go001o00?o0000ooooooooool09Oooool00ol0003o001oo`00O`1]o`00O`03
o`000?oooooooooo02Gooooo00?o0000o`00Ool007l0K_l007l00ol0003oooooooooo`0Qooooo`00
8_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_oo
ool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oo
oooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool0
0ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooo
oooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0
003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003o
ooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06go
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Go
oooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003ooooo
ooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003ooooooooo
o`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o
0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03
oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooo
oaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`1^ooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1^ooooo`03
o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool0
8Oooool002;ooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oo
oooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool0
0ol0003oooooooooo`1^ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`1^ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo
06kooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
06kooooo00?o0000ooooooooool08Oooool002;oooooL_l0000Wooooog7o00009ooooomao`0002Oo
ooooL_l0000Sooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2
ooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooo
oooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`005_oooolb0?l00080
oh00>@3o00020?n001/0o`003_ooool2003o0080oh001@00o`020?n000T00?l01@000?l5003o00H0
oh000P3o00030?n000d00?l00`3oP002003o00T0oh001000o`000`3oP000o`000?l000060?l000D0
0?l00P3oP009003o00<0oh000P3o0006003o00030000o`3oP000oh000080003o0`3oP004003o00/0
oh000P000?l5003o00kooooo5`000?l4003o00T0003o2@00o`020000o`T00?l04`000?l4003o00D0
003o1000o`0Y0000o`800?l00P3oP0000`000?l0oh000?n000020000o`kooooo1`00o`040?n000<0
o`000P000?l2003o00D0oh001P00o`000`3oP0000?l0003o00040?n000D00?l01`3oP0020?l00080
oh000P00o`030?n00080o`00103oP002003o00<0o`001000o`000`3oP0000?l0003o0006003o0003
0?n000000?l0003o00H0oh0000<0o`000?n0003oP000103oP0020000o`<0oh001000o`020?n00003
003o003oP000oh000080o`000P00o`090?n000<0o`000P00o`020?n00080o`001@3oP00Hooooo`00
5_oooolb0?l00080oh00>@3o00020?n001/0o`003_ooool2003o0080oh001@00o`020?n000T00?l0
1@000?l5003o00H0oh000P3o00030?n000d00?l00`3oP002003o00T0oh001000o`000`3oP000o`00
0?l000060?l000D00?l00P3oP009003o00<0oh000P3o0006003o00030000o`3oP000oh000080003o
0`3oP004003o00/0oh000P000?l5003o00kooooo5`000?l4003o00T0003o2@00o`020000o`T00?l0
4`000?l4003o00D0003o1000o`0Y0000o`800?l00P3oP0000`000?l0oh000?n000020000o`kooooo
1`00o`040?n000<0o`000P000?l2003o00D0oh001P00o`000`3oP0000?l0003o00040?n000D00?l0
1`3oP0020?l00080oh000P00o`030?n00080o`00103oP002003o00<0o`001000o`000`3oP0000?l0
003o0006003o00030?n000000?l0003o00H0oh0000<0o`000?n0003oP000103oP0020000o`<0oh00
1000o`020?n00003003o003oP000oh000080o`000P00o`090?n000<0o`000P00o`020?n00080o`00
1@3oP00Hooooo`005_ooool`0?l00080oh000P3o00020000o`L0o`000P3oP00^0?l00080oh006`3o
000>ooooo`800?l00P000?lI003o00030?n00000o`000?l000800?l00P3o00030?n001800?l00P3o
P002003o00D0oh000P00o`050?n00080o`000P3oP0020?l000<0003o2`00o`040?n00080o`002@3o
P007003o00T0oh0000<0o`000?n0003oP0000P00o`020?n000L00?l03_ooool40000o``00?l0103o
P0070000o`800?l030000?l2003o00D0003o0P00o`040000o`<00?l00P000?l6003o00X0003o1000
o`050000o`80oh000P000?l2003o00030?n000000?l0003o00@0oh000`000?l20?n000L00?l06`00
0?l2003o00kooooo0P3o0003003o00/0oh001000o`050?n00080o`000P00o`030?n000800?l00P00
0?l9003o0080oh0000<00?l00?n0003oP000103oP0020000o`030?n0003o0000o`000080oh000P3o
00030?n000@0o`000`00o`040?n000@00?l000<0oh00003o0000o`001000o`020?n000L0o`000`3o
P0020?l000H0oh002P3o00020?n00080o`000`3oP0020?l00080oh000P3o00070?n001Sooooo000F
oooooc00o`000P3oP0020?l00080003o1`3o00020?n002h0o`000P3oP00K0?l000kooooo0P00o`02
0000oaT00?l000<0oh00003o0000o`000P00o`020?l000<0oh004P00o`020?n000800?l01@3oP002
003o00D0oh000P3o00020?n00080o`000`000?l;003o00@0oh000P3o00090?n000L00?l02@3oP000
0`3o0000oh000?n00002003o0080oh001`00o`0>ooooo`@0003o3000o`040?n000L0003o0P00o`0<
0000o`800?l01@000?l2003o00@0003o0`00o`020000o`H00?l02P000?l4003o00D0003o0P3oP002
0000o`800?l000<0oh000000o`000?l0103oP0030000o`80oh001`00o`0K0000o`800?l03_ooool2
0?l000<00?l02`3oP004003o00D0oh000P3o0002003o00<0oh000P00o`020000o`T00?l00P3oP000
0`00o`00oh000?n000040?n00080003o00<0oh000?l0003o00000P3oP0020?l000<0oh00103o0003
003o00@0oh001000o`000`3oP0000?l0003o0004003o0080oh001`3o00030?n00080o`001P3oP00:
0?l00080oh000P3o00030?n00080o`000P3oP0020?l000L0oh006?ooool001Kooooo<03o00020?n0
01@0o`000P3oP0120?l000kooooo0P3oP00N003o00H0oh005`00o`070?n00080o`000`3oP0020?l0
00@0oh002`00o`050?n000800?l00P3o00000`3oP000o`000?l000040?n000L00?l01`3oP004003o
00X0oh001000o`0>ooooo`/0003o0P00o`070000o`D00?l03P000?l4003o0280003o0P3oP002003o
01D0003o103oP003003o0080o`0010000?l7003o0100003o3_ooool20?n000800?l000<0oh00003o
0000o`002`3oP002003o00<0oh001P3o00030?n000@0003o2@3oP003003o0080o`000P3oP002003o
01<0oh002`000?l4003o0080oh001@3o00040?n000030?l0003oP000oh0000@0oh000`3o00020?n0
0080o`000P3oP0050?l000L0oh002@3o00020?n001Sooooo000Fooooo`/0o`000P3oP01d0?l00080
oh001@3o00020?n000kooooo0P3oP0020000oaD00?l00P3oP00I003o00@0003o1@00o`040?n00003
0?l0003oP000oh000080o`000`3oP0020?l00080oh002@00o`040?n00003003o003oP000oh000080
oh00103o00050?n000T00?l00P3oP0000`00o`00oh000?n000020?n00080003o2@00o`020?l000<0
oh000P000?l>ooooo`800?l00`000?l8003o00D0003o1`00o`090000o`80oh003P000?l2003o0180
003o0P3oP0000`000?l0oh000?n000020?n00080003o0P3oP0000`000?l0oh000?n000020?n00080
0?l00P3oP003003o00/0003o0P3oP0020000o`D0oh000`000?l;003o0080oh002@000?l>ooooo`D0
0?l00P000?l60?n000<0o`00103oP002003o00D0o`000P3oP007003o00/0oh0000<0o`000?n0003o
P0004P3oP007003o00/0oh000P00o`070?n000800?l00`3oP0020?l000@0oh00303o00020?n00080
o`000P3oP0090?l000D0oh006?ooool001Kooooo2`3o00020?n007@0o`000P3oP0050?l00080oh00
3_ooool20?n00080003o5@00o`020?n001T00?l010000?l5003o00@0oh0000<0o`000?n0003oP000
0P3o00030?n00080o`000P3oP009003o00@0oh0000<00?l00?n0003oP0000P3oP0040?l000D0oh00
2@00o`020?n00003003o003oP000oh000080oh000P000?l9003o0080o`000`3oP0020000o`kooooo
0P00o`030000o`P00?l01@000?l7003o00T0003o0P3oP00>0000o`800?l04P000?l20?n000030000
o`3oP000oh000080oh000P000?l20?n000030000o`3oP000oh000080oh000P00o`020?n000<00?l0
2`000?l20?n00080003o1@3oP0030000o`/00?l00P3oP0090000o`kooooo1@00o`020000o`H0oh00
0`3o00040?n000800?l01@3o00020?n000L00?l02`3oP0000`3o0000oh000?n0000B0?n000L00?l0
2`3oP002003o00L0oh000P00o`030?n00080o`00103oP00<0?l00080oh000P3o00020?n000T0o`00
1@3oP00Hooooo`005_oooolW0?l00080oh001`3o00020?n005P0o`003_ooool40000o`03003o0000
0?l0003o01800?l00P3oP007003o00@0oh003P00o`050000o`800?l010000?l2003o00030?n00000
0?l0003o00@0003o1@00o`020?n000L00?l01@3oP0060000o`<0oh0020000?l5003o00D0003o0P3o
P004003o00D0003o0P3oP007003o00kooooo4000o`060000o`030?n00000o`000?l000d0003o00<0
oh00003o0000o`001P000?l3003o0180003o0P3oP0030000o`@0oh001000o`050000o`L0oh001P00
0?l3003o0080003o1`3oP0050000o`H00?l00`000?l2003o00H0003o1@00o`0>ooooo`80oh000P3o
00000`3oP0000?l0003o00040?n000<0o`000P3oP0040?l000<0oh000P3o00020?n00080o`0000<0
0?l00000o`000?l01000o`020?n00003003o003oP000oh0000@00?l01@3oP0020?l000800?l02@3o
P002003o00D0oh001@00o`020?n00080003o1`3oP002003o0080oh0000<0o`000?n0003oP0000P3o
00040?n000030?l0003oP000oh0000@0oh000`3o000?0?n000D0o`001`3oP0020000oaSooooo000F
ooooobL0o`000P3oP0070?l00080oh00F03o000>ooooo`@0003o00<00?l00000o`000?l04P00o`02
0?n000L00?l0103oP00>003o00D0003o0P00o`040000o`800?l000<0oh000000o`000?l010000?l5
003o0080oh001`00o`050?n000H0003o0`3oP0080000o`D00?l01@000?l20?n000@00?l01@000?l2
0?n000L00?l03_ooool@003o00H0003o00<0oh00003o0000o`003@000?l00`3oP0000?l0003o0006
0000o`<00?l04P000?l20?n000<0003o103oP004003o00D0003o1`3oP0060000o`<00?l00P000?l7
0?n000D0003o1P00o`030000o`800?l01P000?l5003o00kooooo0P3oP0020?l000030?n00000o`00
0?l000@0oh000`3o00020?n000@0o`000`3oP0020?l00080oh000P3o00000`00o`00003o0000o`04
003o0080oh0000<00?l00?n0003oP0001000o`050?n00080o`000P00o`090?n000800?l01@3oP005
003o0080oh000P000?l70?n000800?l00P3oP0000`3o0000oh000?n000020?l000@0oh0000<0o`00
0?n0003oP000103oP0030?l000l0oh001@3o00070?n00080003o6?ooool001Kooooo0P3o00020?n0
02`0o`000P3oP0070?l00080oh00:`3o00020?n00280o`003_oooolR003o0080oh005`00o`020000
o`800?l00`000?l40?n000<00?l00P000?l;003o00T0003o0P00o`020?n000<0003o5`00o`020?n0
00H00?l000<0oh00003o0000o`000P000?l>ooooo`800?l00`000?l9003o0180003o0P3oP0020000
o`<0oh003`000?l20?n000h0003o1@00o`020?n000800?l00`3oP0020000o`L0oh0020000?l30?n0
0080003o0P3oP0070000o`800?l00`000?l9003o0080oh0010000?l00`00o`00oh000?n000020000
o`kooooo0P00o`020?n000<00?l00P3o00020?n000<0o`00103oP007003o0080oh001@00o`040000
oaT0oh000P3o00050?n000800?l00P3oP0000`00o`00oh000?n000020?n000800?l00`3o00080?n0
00030000o`3oP000oh000080oh002@00o`040?l000<0oh000P00o`070?n000L0o`001P3oP0000`3o
0000oh000?n00002003o01Sooooo000Fooooo`80o`000P3oP00/0?l00080oh001`3o00020?n002/0
o`000P3oP00R0?l000kooooo8P00o`020?n001L00?l00P000?l2003o00<0003o103oP003003o0080
003o2`00o`090000o`800?l00P3oP0030000oaL00?l00P3oP006003o00030?n00000o`000?l00080
003o3_ooool2003o00<0003o2@00o`0B0000o`80oh000P000?l30?n000l0003o0P3oP00>0000o`D0
0?l00P3oP002003o00<0oh000P000?l70?n000P0003o0`3oP0020000o`80oh001`000?l2003o00<0
003o2@00o`020?n000@0003o00<00?l00?n0003oP0000P000?l>ooooo`800?l00P3oP003003o0080
o`000P3oP0030?l000@0oh001`00o`020?n000D00?l010000?lI0?n00080o`001@3oP002003o0080
oh0000<00?l00?n0003oP0000P3oP002003o00<0o`00203oP0000`000?l0oh000?n000020?n000T0
0?l0103o00030?n000800?l01`3oP0070?l000H0oh0000<0o`000?n0003oP0000P00o`0Hooooo`00
5_ooool`0?l00080oh00=03o00020?n000L0o`000P3oP00>0?l00080oh000P3o00020?n000030?l0
003oP000oh000080o`003_ooool2003o0080003o00<00?l00000o`000?l05`00o`020?n000800?l0
0P3o00020?n000D00?l01@000?l;003o00L0003o1P00o`050000o`80oh000`000?l20?n000@0003o
3P00o`020000o`/00?l00`000?l9003o0080oh000P3o0005003o0080003o3_ooool70000o`/00?l0
2@000?l20?n000<0003o2@3oP00K0000o`800?l01@000?l20?n000@00?l000<0oh000000o`000?l0
103oP0020000o`030?n000000?l0003o0080003o0P3oP0020000o`<0oh002@00o`0@0000o`800?l0
10000?l00`00o`00oh000?n000020000o`kooooo0P3oP0030?l000l0oh000P00o`000`3oP000o`00
0?l000040?n000<00?l00P3oP0020000o`<0oh000P00o`020?l00080oh000P00o`000`3oP0000?l0
003o00020?n000800?l01@3oP0070?l000@0oh000P3o00000`3oP0000?l0003o00020?n000@0o`00
1@3oP002003o0080oh0000<0003o0?n0003oP0001`00o`040?n00080o`00303oP0040?l00080oh00
00<0o`000?n0003oP0000P3o0002003o0080oh0000<00?l00?n0003oP0000P3oP00Hooooo`005_oo
oom=0?l00080oh002@3o00020?n002L0o`000P3oP0070?l000kooooo1000o`000`3oP0000?l0003o
000B003o0080oh000`00o`020?n000800?l0103oP00S003o00P0003o00<0oh00003o0000o`005P00
o`020000oa400?l00P3oP002003o0080oh000P000?l00`00o`00oh000?n00002003o00kooooo1000
o`050000oa000?l040000?lB003o0080003o1000o`050?n00080003o00<0oh000000o`000?l01`3o
P0040000o`80oh001@000?l40?n000D0003o0P00o`020?n000h00?l03P000?l20?n000800?l03_oo
ool2003o0080oh000`3o00020?n000T0003o2`3oP003003o0080003o0P00o`070?n00080o`000P00
o`000`3oP0000?l0003o000;0?n00080o`000`3oP0020?l000L0oh000P3o00020?n000L0o`00103o
P0000`00o`00oh000?n00002003o00@0oh0000<00?l00?n0003oP000103o00050?n000@0o`0000<0
oh00003o0000o`001000o`020000o`L00?l01`3oP00Hooooo`005_oooom=0?l00080oh002@3o0002
0?n002L0o`000P3oP0070?l000kooooo1000o`000`3oP0000?l0003o000B003o0080oh000`00o`02
0?n000800?l0103oP00S003o00P0003o00<0oh00003o0000o`005P00o`020000oa400?l00P3oP002
003o0080oh000P000?l00`00o`00oh000?n00002003o00kooooo1000o`050000oa000?l040000?lB
003o0080003o1000o`050?n00080003o00<0oh000000o`000?l01`3oP0040000o`80oh001@000?l4
0?n000D0003o0P00o`020?n000h00?l03P000?l20?n000800?l03_ooool2003o0080oh000`3o0002
0?n000T0003o2`3oP003003o0080003o0P00o`070?n00080o`000P00o`000`3oP0000?l0003o000;
0?n00080o`000`3oP0020?l000L0oh000P3o00020?n000L0o`00103oP0000`00o`00oh000?n00002
003o00@0oh0000<00?l00?n0003oP000103o00050?n000@0o`0000<0oh00003o0000o`001000o`02
0000o`L00?l01`3oP00Hooooo`005_oooolG0?l00080oh00;@3o00020?n00100o`000P3oP00U0?l0
00L0oh00103o000>ooooo`T00?l00P000?l7003o00L0003o1`00o`040?l000L00?l00P000?l<003o
0080003o1P00o`000`000?l00?l0003o0009003o0080oh000P00o`070000oaL00?l00P000?l;003o
00030?n00000o`000?l000/00?l03_ooool>003o00H0003o0P3oP003003o0180003o2@00o`0;0000
o`<00?l01`000?l2003o00@0oh006@000?l5003o00H0003o3000o`060000o`D00?l03_ooool2003o
00L0o`001@00o`040000o`80oh000P3o00000`3oP000o`000?l0000;0?n00080o`000`3oP0020?l0
0080oh001`00o`020000o`800?l01@3oP0070?l000D0oh000P00o`020?n00080003o0P3oP0050?l0
0080oh000P00o`000`000?l0oh000?n0000G0?n000800?l00P000?l2003o00<0003o1000o`040?n0
0003003o003o0000o`0000800?l06?ooool001Kooooo5`3o00020?n002d0o`000P3oP00@0?l00080
oh009@3o00070?n000@0o`003_ooool9003o0080003o1`00o`070000o`L00?l0103o0007003o0080
003o3000o`020000o`H00?l000<0003o003o0000o`002@00o`020?n000800?l01`000?lG003o0080
003o2`00o`000`3oP0000?l0003o000;003o00kooooo3P00o`060000o`80oh000`00o`0B0000o`T0
0?l02`000?l3003o00L0003o0P00o`040?n001T0003o1@00o`060000o``00?l01P000?l5003o00ko
oooo0P00o`070?l000D00?l010000?l20?n00080o`0000<0oh000?l0003o00002`3oP0020?l000<0
oh000P3o00020?n000L00?l00P000?l2003o00D0oh001`3o00050?n000800?l00P3oP0020000o`80
oh001@3o00020?n000800?l000<0003o0?n0003oP0005`3oP002003o0080003o0P00o`030000o`@0
0?l0103oP0000`00o`00o`000?l00002003o01Sooooo000FooooobL0o`000P3oP00M0?l00080oh00
1@3o00020?n00380o`000P3oP0070?l000kooooo1@00o`090000o`P00?l000<0003o003o0000o`00
1@00o`020?n00080003o103oP0000`000?l00?l0003o000M003o0080oh001@00o`050?n00080003o
1000o`020000o`X00?l00P000?l2003o0080003o1@00o`020000o`800?l000<0oh00003o0000o`00
0P000?l20?n00080003o1`3oP0020000o`<0oh000P00o`0>ooooo`80oh001@000?l9003o00@0003o
1@00o`0B0000o`80oh000P000?l30?n000L0003o0P00o`070000o`80oh002@000?l20?n000`0003o
0P3oP0060000o`030?n000000?l0003o0080003o1000o`000`000?l00?l0003o000B003o0080003o
0`00o`040000o`kooooo0P00o`020?n000030?l0003oP000oh0000@0oh000P000?l5003o0080oh00
0P000?l50?n000<00?l02@3oP0020?l000T0oh000P00o`020?n000<00?l00P3oP002003o0080oh00
0P3o00050?n00080o`000`00o`020000o`800?l00P3oP0030?l00080oh000P3o0002003o00030000
o`00o`000?l000/0oh001000o`070?n000030000o`3oP000oh0000@0003o1@00o`090?n000800?l0
6?ooool001Kooooo9`3o00020?n001d0o`000P3oP0050?l00080oh00<P3o00020?n000L0o`003_oo
ool5003o00T0003o2000o`000`000?l00?l0003o0005003o0080oh000P000?l40?n000030000o`00
o`000?l001d00?l00P3oP005003o00D0oh000P000?l4003o0080003o2P00o`020000o`800?l00P00
0?l5003o0080003o0P00o`000`3oP0000?l0003o00020000o`80oh000P000?l70?n00080003o0`3o
P002003o00kooooo0P3oP0050000o`T00?l010000?l5003o0180003o0P3oP0020000o`<0oh001`00
0?l2003o00L0003o0P3oP0090000o`80oh0030000?l20?n000H0003o00<0oh000000o`000?l00P00
0?l4003o00030000o`00o`000?l001800?l00P000?l3003o00@0003o3_ooool2003o0080oh0000<0
o`000?n0003oP000103oP0020000o`D00?l00P3oP0020000o`D0oh000`00o`090?n00080o`002@3o
P002003o0080oh000`00o`020?n000800?l00P3oP0020?l000D0oh000P3o0003003o0080003o0P00
o`020?n000<0o`000P3oP0020?l000800?l000<0003o003o0000o`002`3oP004003o00L0oh0000<0
003o0?n0003oP00010000?l5003o00T0oh000P00o`0Hooooo`005_oooolM0?l000030?n0003o0000
o`000100o`000P3oP00D0?l00080oh008P3o00020?n000X0o`000P3oP00B0?l000kooooo1000o`03
0000oaH00?l00`3oP002003o00@0oh004P00o`030000o`/00?l00`3oP002003o00@0oh000P3o0000
0`3oP0000?l0003o000?003o00030?n000000?l0003o0080003o0P3oP005003o0080oh000P00o`03
0?n000800?l00P3oP002003o00<0oh00103o00020?n000<0003o0P00o`0>ooooo`L0003o0P00o`02
0000o`800?l000<0003o003o0000o`002`00o`0<0000o`800?l00P000?l20?n000<0003o0P3oP006
0000o`X00?l01`000?l20?n000T0003o0P00o`0N0000o`l00?l01`000?l>ooooo`800?l00P3o0000
0`3oP000o`000?l000020?l000D0oh002000o`000`3oP0000?l0003o0005003o00H0oh000P00o`05
0?n00080o`000`3oP002003o0080oh00103o00030?n00080o`000P00o`050?n00080o`000P3oP000
0`00o`00oh000?n000020000o`D0oh000P3o00020?n000D00?l0103oP0020000o`<0oh000P00o`02
0?n000@0o`003000o`020000o`L00?l00P3oP003003o00@0oh006?ooool001Kooooo7@3o00000`3o
P000o`000?l0000@0?l00080oh00503o00020?n00280o`000P3oP00:0?l00080oh004P3o000>oooo
o`@00?l00`000?lF003o00<0oh000P00o`040?n001800?l00`000?l;003o00<0oh000P00o`040?n0
0080o`0000<0oh00003o0000o`003`00o`000`3oP000003o0000o`020000o`80oh001@00o`020?n0
00800?l00`3oP002003o0080oh000P00o`030?n000@0o`000P3oP0030000o`800?l03_ooool70000
o`800?l00P000?l2003o00030000o`00o`000?l000/00?l030000?l2003o0080003o0P3oP0030000
o`80oh001P000?l:003o00L0003o0P3oP0090000o`800?l07P000?l?003o00L0003o3_ooool2003o
0080o`0000<0oh000?l0003o00000P3o00050?n000P00?l000<0oh00003o0000o`001@00o`060?n0
00800?l01@3oP0020?l000<0oh000P00o`020?n000@0o`000`3oP0020?l000800?l01@3oP0020?l0
0080oh0000<00?l00?n0003oP0000P000?l50?n00080o`000P3oP005003o00@0oh000P000?l30?n0
00800?l00P3oP0040?l000`00?l00P000?l7003o0080oh000`00o`040?n001Sooooo000Fooooocd0
o`000P3oP0020?l000030?n0003o0000o`000080o`000P3oP00/0?l00080oh002`3o00020?n000L0
o`003_ooool20000o`80oh008000o`020?n00003003o003oP000oh000080oh004P00o`020000o`L0
0?l00P3oP0000`3o0000oh000?n000020?l000D0oh002@00o`020?n00080003o103oP00C003o00@0
003o0P3oP0000`00o`00oh000?n000020?n00080o`000P3oP005003o00kooooo1`000?l2003o00T0
003o1000o`000`000?l00?l0003o0005003o00/0003o0P00o`020?n000<0003o0P3oP002003o00D0
003o2000o`080000o`P00?l01@000?l20?n000/0003o00<0oh00003o0000o`003P000?l;003o00L0
003o1000o`0>ooooo`80003o1@3oP0020?l00080oh001`00o`070?n000@00?l04`3oP0020?l00080
0?l01@3oP0020?l000/0oh000P3o00000`3oP000o`000?l00006003o00<0oh000P000?l20?n000`0
0?l01P3oP0020?l000<0oh000P3o00070?n000D00?l00P3oP002003o00@0oh0000<0003o0?n0003o
P0000P3oP00Hooooo`005_ooool50?l00080oh00>`3o00020?n004H0o`003_ooool9003o0080003o
4000o`090000o`80oh004000o`020000o`03003o003oP000oh0000@00?l00`3oP002003o00d0oh00
1@00o`020000o`<0oh003@00o`020?n000D00?l00P3oP0020000o`<0oh001P000?l90?n000D00?l0
3_ooool50000o`H00?l00`000?l;003o0080003o0`00o`0A0000o`030?n000000?l0003o00800?l0
1@000?l2003o0080003o0P00o`020?n000D0003o0P3oP0070000o`<00?l06P000?l5003o0080003o
2@00o`000`000?l00?l0003o0002003o0080003o1`00o`0>ooooo`@00?l000<0oh000?l0003o0000
1P3o0005003o0080oh000`00o`090?n000@00?l01@3oP0020000o`80oh000P3o00000`00o`00oh00
0?n000060?n000030?l0003oP000oh0000@0oh000P000?l70?n000030?l0003oP000oh0000/0oh00
0P3o0005003o0080003o103oP003003o00H0o`0000<0oh000?l0003o00002@3oP004003o00030?n0
0000o`000?l000L00?l0103oP00Hooooo`005_ooool50?l00080oh00>`3o00020?n004H0o`003_oo
ool9003o0080003o4000o`090000o`80oh004000o`020000o`03003o003oP000oh0000@00?l00`3o
P002003o00d0oh001@00o`020000o`<0oh003@00o`020?n000D00?l00P3oP0020000o`<0oh001P00
0?l90?n000D00?l03_ooool50000o`H00?l00`000?l;003o0080003o0`00o`0A0000o`030?n00000
0?l0003o00800?l01@000?l2003o0080003o0P00o`020?n000D0003o0P3oP0070000o`<00?l06P00
0?l5003o0080003o2@00o`000`000?l00?l0003o0002003o0080003o1`00o`0>ooooo`@00?l000<0
oh000?l0003o00001P3o0005003o0080oh000`00o`090?n000@00?l01@3oP0020000o`80oh000P3o
00000`00o`00oh000?n000060?n000030?l0003oP000oh0000@0oh000P000?l70?n000030?l0003o
P000oh0000/0oh000P3o0005003o0080003o103oP003003o00H0o`0000<0oh000?l0003o00002@3o
P004003o00030?n00000o`000?l000L00?l0103oP00Hooooo`005_oooolM0?l000030?n0003o0000
o`0000L0o`000P3oP00D0?l00080oh009`3o00020?n00280o`003_ooool20000oah00?l010000?l2
0?n000<0003o5P00o`030?n000@00?l00`3o00060?n000800?l000<0oh00003o0000o`000P00o`02
0?n00080o`000`3oP0040000o`L00?l00P3oP002003o00030?n00000o`000?l000800?l00P3oP00<
003o0080003o0P3oP0020000o`03003o003oP000oh0000800?l03_ooool50000oa800?l00P000?l5
003o00@0003o1000o`030000o`800?l00P000?l2003o00X0003o0P3oP0040000o`<0oh001`000?l2
0?n000@0003o0`00o`060000o`800?l02P000?l20?n000/0003o6@00o`0>ooooo`D0oh00103o0007
0?n000@00?l00`3oP0020?l000@0oh000`00o`070?n000@00?l0103o00000`000?l0oh000?n00002
0?n000800?l01`3oP002003o00<0003o103oP002003o00030?l0003oP000oh0000800?l00P3oP002
003o00D0oh000P3o00020?n000T00?l01@3oP0020000o`D0oh00103o00090?n000D00?l010000?l5
0?n001Sooooo000Foooooad0o`0000<0oh000?l0003o00001`3o00020?n001@0o`000P3oP00W0?l0
0080oh008P3o000>ooooo`80003o7P00o`040000o`80oh000`000?lF003o00<0oh001000o`030?l0
00H0oh000P00o`000`3oP0000?l0003o0002003o0080oh000P3o00030?n000@0003o1`00o`020?n0
00800?l000<0oh00003o0000o`000P00o`020?n000`00?l00P000?l20?n00080003o00<00?l00?n0
003oP0000P00o`0>ooooo`D0003o4P00o`020000o`D00?l010000?l4003o00<0003o0P00o`020000
o`800?l02P000?l20?n000@0003o0`3oP0070000o`80oh0010000?l3003o00H0003o0P00o`0:0000
o`80oh002`000?lI003o00kooooo1@3oP0040?l000L0oh001000o`030?n00080o`00103oP003003o
00L0oh001000o`040?l000030000o`3oP000oh000080oh000P00o`070?n000800?l00`000?l40?n0
00800?l000<0o`000?n0003oP0000P00o`020?n000800?l01@3oP0020?l00080oh002@00o`050?n0
0080003o1@3oP0040?l000T0oh001@00o`040000o`D0oh006?ooool001Kooooo6`3o00050?n000L0
o`000P3oP0070?l00080oh007@3o00020?n002h0o`000P3oP0090?l000kooooo9000o`050000oa00
0?l0103oP0020000o`80oh0000<0003o0?n0003oP0000P3oP0020?l000<0oh000P3o00050?n00080
0?l0103oP002003o00<0oh001P00o`000`3oP000003o0000o`020?n000T00?l00P3oP005003o00@0
003o00<0oh000000o`000?l00P000?l20?n000800?l000<0oh00003o0000o`000P00o`0>ooooo`@0
0?l040000?l:003o00H0003o3P00o`060000o`03003o00000?l0003o00H0003o1@00o`050000o`80
oh000P000?l5003o01H0003o1@00o`050000o`80oh002@000?l;003o00kooooo1@3oP0040?l000D0
oh000P00o`040?n000<0o`001P3oP003003o0080oh001000o`000`3oP0000?l0003o0002003o0080
oh000P3o00000`3oP000o`000?l000060?n000030?l0003oP000oh0000800?l00P3oP005003o00D0
oh000P3o00040?n000<00?l0103oP0040?l000030?n00000o`000?l000800?l02@3oP0020?l000L0
oh000P000?l50?n00080o`000`3oP002003o0080oh000P00o`020?n000030?l00000o`000?l00080
oh006?ooool001Kooooo6`3o00050?n000L0o`000P3oP0070?l00080oh007@3o00020?n002h0o`00
0P3oP0090?l000kooooo9000o`050000oa000?l0103oP0020000o`80oh0000<0003o0?n0003oP000
0P3oP0020?l000<0oh000P3o00050?n000800?l0103oP002003o00<0oh001P00o`000`3oP000003o
0000o`020?n000T00?l00P3oP005003o00@0003o00<0oh000000o`000?l00P000?l20?n000800?l0
00<0oh00003o0000o`000P00o`0>ooooo`@00?l040000?l:003o00H0003o3P00o`060000o`03003o
00000?l0003o00H0003o1@00o`050000o`80oh000P000?l5003o01H0003o1@00o`050000o`80oh00
2@000?l;003o00kooooo1@3oP0040?l000D0oh000P00o`040?n000<0o`001P3oP003003o0080oh00
1000o`000`3oP0000?l0003o0002003o0080oh000P3o00000`3oP000o`000?l000060?n000030?l0
003oP000oh0000800?l00P3oP005003o00D0oh000P3o00040?n000<00?l0103oP0040?l000030?n0
0000o`000?l000800?l02@3oP0020?l000L0oh000P000?l50?n00080o`000`3oP002003o0080oh00
0P00o`020?n000030?l00000o`000?l00080oh006?ooool001Kooooo7@3o00000`3oP000o`000?l0
00070?l00080oh00>03o00020?n001h0o`000P3oP0070?l000kooooo0P00o`030000oad00?l00P00
0?l3003o0080003o4P00o`040000o`L00?l00P3oP0000`000?l0oh000?n000040?n00080o`0000<0
oh00003o0000o`00103oP002003o00030?n00000o`000?l000@00?l00P000?l3003o00/0003o2`00
o`030?n000800?l00P3oP003003o0080oh000P00o`0>ooooo`800?l00P3oP00<003o00@0003o0`00
o`020000o`800?l06@000?l5003o0180003o0P3oP0040000o`030?n000000?l0003o00d0003o0P3o
P0070000oaL00?l0103oP0030000o`800?l03_ooool20?n000<0o`000P3oP0020?l000D0oh001000
o`0;0?n000030?l0003oP000oh000080oh001000o`000`3oP000003o0000o`060?n000030?l0003o
P000oh0000@0oh000P3o00000`3oP000o`000?l000040?n000D00?l0103oP0000`3o0000oh000?n0
0002003o0080oh000P3o00070?n000800?l000<0oh000000o`000?l02@3oP002003o00@0oh000`00
o`020?n000D0003o3@3oP0030?l000800?l00P3oP00Hooooo`005_oooolM0?l000030?n0003o0000
o`0000L0o`000P3oP00h0?l00080oh007P3o00020?n000L0o`003_ooool2003o00<0003o7@00o`02
0000o`<00?l00P000?lB003o00@0003o1`00o`020?n000030000o`3oP000oh0000@0oh000P3o0000
0`3oP0000?l0003o00040?n000800?l000<0oh00003o0000o`001000o`020000o`<00?l02`000?l;
003o00<0oh000P00o`020?n000<00?l00P3oP002003o00kooooo0P00o`020?n000`00?l010000?l3
003o0080003o0P00o`0I0000o`D00?l04P000?l20?n000@0003o00<0oh000000o`000?l03@000?l2
0?n000L0003o5`00o`040?n000<0003o0P00o`0>ooooo`80oh000`3o00020?n00080o`001@3oP004
003o00/0oh0000<0o`000?n0003oP0000P3oP004003o00030?n000000?l0003o00H0oh0000<0o`00
0?n0003oP000103oP0020?l000030?n0003o0000o`0000@0oh001@00o`040?n000030?l0003oP000
oh0000800?l00P3oP0020?l000L0oh000P00o`000`3oP000003o0000o`090?n000800?l0103oP003
003o0080oh001@000?l=0?n000<0o`000P00o`020?n001Sooooo000FoooooaH0o`0000<0oh000?l0
003o00003P3o0002003o0080oh001@3o00020?n000T0o`000P3oP01=0?l000kooooo3@00o`000`00
0?l00?l0003o0009003o0080003o0P00o`000`3oP0000?l0003o0002003o0080003o5000o`090000
o`<0oh000P00o`020?n00003003o003oP000oh000080oh00103o00000`3oP000003o0000o`090?n0
0080003o4P00o`050000o`L00?l0103oP002003o0080oh0000<00?l00?n0003oP0000P00o`0>oooo
o`D00?l00P000?l9003o00H0003o00<00?l00000o`000?l00P00o`0@0000o`80oh001`000?l7003o
00L0003o103oP005003o00T0003o0P00o`040000o`<00?l01`000?l20?n00100003o0P00o`020000
o`800?l000<0003o0?n0003oP0000P00o`020?n000800?l00`000?l2003o00kooooo0P00o`0;0?n0
0003003o003oP000oh000080o`001@3oP002003o00@0003o00<00?l00?n0003oP0000P3oP0020?l0
00<0oh001000o`020?n00080o`001@3oP0020?l000D0oh000P00o`040?n00003003o00000?l0003o
0080o`000`3oP0020?l000800?l01`3oP0020?l00080oh000P000?l00`3oP000003o0000o`040?n0
0080o`001@3oP0020?l000800?l01@3oP003003o0080oh000P00o`040?n000<0003o1`3oP0020000
o`80oh006?ooool001Kooooo5`3o00020?n000L0o`000P3oP0050?l00080oh006`3o00020?n001h0
o`000P3oP00B0?l00080oh001`3o00020?n000L0o`003_oooolN003o0080003o0P00o`020?n00100
0?l00P3oP0050000o`H00?l0403oP003003o0080003o2@00o`020000o`800?l000<0oh000000o`00
0?l04P00o`020?n000800?l000<0003o0?n0003oP0001`3oP004003o00kooooo2`00o`050000o`H0
0?l01@000?l20?n000`0003o0P00o`020?n000`0003o0P3oP0040000o`<0oh000P000?l40?n00003
003o00000?l0003o00@0oh000`000?l40?n000L0003o2@00o`040000o`80oh0000<0003o003o0000
o`001@000?l4003o00D0003o203oP005003o00kooooo2@3oP0020000o`80oh0000<00?l00?n0003o
P0001P3oP0000`00o`00oh000?n00002003o00<0003o203oP0030?l000@0oh000`3o00060?n000<0
o`000P00o`020?l000@00?l00`000?l40?n000030000o`3o0000o`0000@0oh000P3o000:0?n000@0
0?l00P3oP002003o00030?l0003oP000oh0000L0oh000P000?l50?n000@0003o103oP003003o00L0
oh000P00o`020?n001Sooooo000FoooooaL0o`000P3oP0070?l00080oh001@3o00020?n001/0o`00
0P3oP00N0?l00080oh004P3o00020?n000L0o`000P3oP0070?l000kooooo7P00o`020000o`800?l0
0P3oP00@003o0080oh001@000?l6003o0100oh000`00o`020000o`T00?l00P000?l2003o00030?n0
00000?l0003o01800?l00P3oP002003o00030000o`3oP000oh0000L0oh001000o`0>ooooo`/00?l0
1@000?l6003o00D0003o0P3oP00<0000o`800?l00P3oP00<0000o`80oh0010000?l30?n00080003o
103oP0000`00o`00003o0000o`040?n000<0003o103oP0070000o`T00?l010000?l20?n000030000
o`00o`000?l000D0003o1000o`050000o`P0oh001@00o`0>ooooo`T0oh000P000?l20?n00003003o
003oP000oh0000H0oh0000<00?l00?n0003oP0000P00o`030000o`P0oh000`3o00040?n000<0o`00
1P3oP0030?l000800?l00P3o0004003o00<0003o103oP0000`000?l0o`000?l000040?n00080o`00
2P3oP004003o0080oh000P00o`000`3o0000oh000?n000070?n00080003o1@3oP0040000o`@0oh00
0`00o`070?n000800?l00P3oP00Hooooo`005_ooool;0?l00080oh00:`3o00000`3oP000o`000?l0
000a0?l000030?n0003o0000o`000180o`000P3oP0070?l000kooooo=000o`070000o`@00?l04P3o
P0000`000?l0oh000?n00002003o00D0oh000P3o0006003o00030?n00000o`000?l001L00?l00P3o
P002003o00D0oh000P000?l2003o00kooooo2`00o`050000o`d00?l02@000?l00`00o`00003o0000
o`040000o`800?l00`000?l;003o00`0003o1P3oP0030000o`@0oh000P00o`020?n000030000o`3o
P000oh000080oh001@000?l60?n00080003o2000o`040000o`800?l01`000?l40?n000D0003o3_oo
ool2003o00T0oh000P3o00000`3oP000o`000?l000020?n000800?l00P000?l<003o00@0oh0000<0
o`000?n0003oP000103oP002003o00030?l0003oP000oh0000L0o`000P00o`060?n00003003o003o
P000oh0000D0oh000P3o00020?n00080003o2@3oP007003o0080oh001@3o000;0?n000D00?l01000
0?l20?n000<00?l00P3oP002003o0080003o1@3oP00Hooooo`005_ooool;0?l00080oh00:`3o0000
0`3oP000o`000?l0000a0?l000030?n0003o0000o`000180o`000P3oP0070?l000kooooo=000o`07
0000o`@00?l04P3oP0000`000?l0oh000?n00002003o00D0oh000P3o0006003o00030?n00000o`00
0?l001L00?l00P3oP002003o00D0oh000P000?l2003o00kooooo2`00o`050000o`d00?l02@000?l0
0`00o`00003o0000o`040000o`800?l00`000?l;003o00`0003o1P3oP0030000o`@0oh000P00o`02
0?n000030000o`3oP000oh000080oh001@000?l60?n00080003o2000o`040000o`800?l01`000?l4
0?n000D0003o3_ooool2003o00T0oh000P3o00000`3oP000o`000?l000020?n000800?l00P000?l<
003o00@0oh0000<0o`000?n0003oP000103oP002003o00030?l0003oP000oh0000L0o`000P00o`06
0?n00003003o003oP000oh0000D0oh000P3o00020?n00080003o2@3oP007003o0080oh001@3o000;
0?n000D00?l010000?l20?n000<00?l00P3oP002003o0080003o1@3oP00Hooooo`005_ooool20?l0
0080oh004P3o00000`3oP000o`000?l000050?l00080oh000P3o00020?n004l0o`0000<0oh000?l0
003o00000P3o00020?n00100o`003_oooolN003o00@0003o0P3oP00>003o0080003o103oP0000`00
o`00oh000?n000040?n00080o`000`3oP0020000o`80oh0000<0o`000?n0003oP0001P3oP0050000
o`800?l00P3oP0030?l00080oh000P00o`020000o`030?n00000o`000?l00080oh0010000?l00`3o
P0000?l0003o00020000o`L00?l01`000?l20?n00080003o0P3oP0000`000?l00?l0003o0002003o
00kooooo2`00o`070000o`T00?l00P000?l00`00o`00003o0000o`0V0000o`80oh0000<0003o0?n0
003oP0000P3oP007003o0080003o0P00o`020?n000<0003o0P3oP004003o00L0003o1`00o`030000
o`80oh000P00o`020?n000<00?l0103oP0020000o`80oh000`000?l20?n000kooooo0P00o`030000
o`/0oh000P3o00090?n000D00?l0103oP002003o00030?l0003oP000oh000080oh000P00o`020?l0
0003003o003oP000oh000080oh001@3o0002003o00H0oh0000<00?l00?n0003oP0000P3oP0030?l0
00H0oh000P000?l30?n000H00?l01`3oP0020000o`<0oh00103o00050?n000800?l00P3o0003003o
00@0003o0P3oP003003o0080oh000P000?l70?n001Sooooo000Fooooo`80o`000P3oP00B0?l00003
0?n0003o0000o`0000D0o`000P3oP0020?l00080oh00C`3o00000`3oP000o`000?l000020?l00080
oh00403o000>oooooah00?l010000?l20?n000h00?l00P000?l40?n00003003o003oP000oh0000@0
oh000P3o00030?n00080003o0P3oP0000`3o0000oh000?n000060?n000D0003o0P00o`020?n000<0
o`000P3oP002003o0080003o00<0oh00003o0000o`000P3oP0040000o`030?n00000o`000?l00080
003o1`00o`070000o`80oh000P000?l20?n000030000o`00o`000?l000800?l03_ooool;003o00L0
003o2@00o`020000o`03003o00000?l0003o02H0003o0P3oP0000`000?l0oh000?n000020?n000L0
0?l00P000?l2003o0080oh000`000?l20?n000@00?l01`000?l7003o00<0003o0P3oP002003o0080
oh000`00o`040?n00080003o0P3oP0030000o`80oh003_ooool2003o00<0003o2`3oP0020?l000T0
oh001@00o`040?n000800?l000<0o`000?n0003oP0000P3oP002003o0080o`0000<00?l00?n0003o
P0000P3oP0050?l000800?l01P3oP0000`00o`00oh000?n000020?n000<0o`001P3oP0020000o`<0
oh001P00o`070?n00080003o0`3oP0040?l000D0oh000P00o`020?l000<00?l010000?l20?n000<0
0?l00P3oP0020000o`L0oh006?ooool001Kooooo7@3o00000`3oP000o`000?l0000H0?l000030?n0
003o0000o`0003P0o`0000<0oh000?l0003o00000P3oP0070?l00080oh002@3o000>oooooaL00?l0
0P000?l7003o0080003o5`00o`0?0?n000030000o`00o`000?l000H00?l000<0003o003o0000o`00
0P000?l20?n00080003o0`3oP002003o0080003o1@00o`040000o`<00?l010000?lB003o0080oh00
1@00o`0>ooooo`80003o5000o`000`000?l00?l0003o0002003o0080003o00<00?l00000o`000?l0
10000?l2003o00<0003o1000o`050000o`@00?l00P000?l3003o00T0003o0P3oP005003o0080oh00
1@00o`040?n000D0003o0P00o`040?n000<0003o103oP00;0000o`<0oh001`000?l40?n000T0003o
3_ooool4003o00D0003o0P00o`020?n00003003o003oP000oh0000H0oh0000<00?l00?n0003oP000
1@00o`020?n000T00?l00P3oP002003o0080oh0000<00?l00?n0003oP000103o00070?n00080o`00
1@3oP0040?l000030?n0003o0000o`0000T00?l02@3oP002003o0080oh001@00o`020000o`L0oh00
0P000?l30?n000@00?l01`3oP004003o0080003o1@3oP00Hooooo`005_oooon:0?l000kooooo0P3o
P00<003o0080003o4P00o`020000oa000?l0103oP0000`000?l0oh000?n0000;0000o`80oh0000<0
003o0?n0003oP0000P3oP002003o0080oh0000<00?l00?n0003oP0001P3oP0000`3o0000oh000?n0
000M003o0080003o00<0oh000000o`000?l00P3oP004003o00030?n000000?l0003o00800?l03_oo
ool40000o`030?n00000o`000?l000@0003o3P00o`020?n000<0003o0P3oP004003o00<0003o0P3o
P004003o00<0oh0010000?l9003o00L0003o0P3oP003003o0080oh000P000?l70?n00080003o0P3o
P0050000o`800?l01`000?l20?n00080003o0P3oP0050000o`80oh0000<00?l00000o`000?l00P00
o`040000o`<0oh001`000?l4003o00kooooo1@00o`0=0?n000@0o`0000<00?l00?n0003oP0001@00
o`080?n000030?l00000o`000?l00080oh000P00o`070?n000@0003o00<0oh00003o0000o`000P3o
P002003o00D0oh000P3o000<0?n000800?l00P3oP002003o00<0003o2@3oP004003o00/0oh0000<0
0?l00?n0003oP0000P3oP00;003o00L0oh006?ooool001KoooooRP3o000>ooooo`80oh003000o`02
0000oa800?l00P000?l@003o00@0oh0000<0003o0?n0003oP0002`000?l20?n000030000o`3oP000
oh000080oh000P00o`020?n00003003o003oP000oh0000H0oh0000<0o`000?n0003oP0007@00o`02
0000o`030?n000000?l0003o0080oh001000o`000`3oP000003o0000o`02003o00kooooo10000?l0
0`3oP0000?l0003o00040000o`h00?l00P3oP0030000o`80oh001000o`030000o`80oh001000o`03
0?n000@0003o2@00o`070000o`80oh000`00o`020?n00080003o1`3oP0020000o`80oh001@000?l2
003o00L0003o0P3oP0020000o`80oh001@000?l20?n00003003o00000?l0003o00800?l010000?l3
0?n000L0003o1000o`0>ooooo`D00?l03@3oP0040?l00003003o003oP000oh0000D00?l0203oP000
0`3o00000?l0003o00020?n000800?l01`3oP0040000o`030?n00000o`000?l00080oh000P00o`05
0?n00080o`00303oP002003o0080oh000P00o`030000o`T0oh001000o`0;0?n00003003o003oP000
oh000080oh002`00o`070?n001Sooooo000Foooooad0o`0000<0oh000?l0003o0000?@3o00020?n0
00@0o`0000<0oh000?l0003o00008P3o00020?n000kooooo2`00o`020000oa400?l01P000?l>003o
0080003o1000o`000`3oP0000?l0003o00040?n000<0003o1P00o`000`000?l0oh000?n000090?n0
00@0003o0P3oP0000`00o`00oh000?n0000B003o00@0003o1`00o`020000o`030?n000000?l0003o
00H0oh001@00o`0>ooooo`@0003o00<0oh00003o0000o`000P000?l2003o0080oh002@000?l5003o
00<0003o2000o`000`000?l0oh000?n000040000o`800?l01@000?l7003o00T0003o103oP0000`00
o`00oh000?n000020000o`800?l00P3oP00>003o00L0003o3@00o`030000o`L00?l00P000?l6003o
00D0003o3_ooool20?n000<00?l0203oP0000`00o`00oh000?n000020?n000@0o`001`3oP0000`00
o`00oh000?n000070?n000800?l01P3oP0000`3o0000oh000?n00006003o00030?n00000o`000?l0
00@0oh000P00o`000`000?l0oh000?n000070?n000800?l0103o00030?n000/00?l00P000?l20?n0
00D00?l00P3oP0020?l000030?n0003o0000o`0000@0o`0000<0oh00003o0000o`000P00o`050000
o`@0oh000P00o`020?n000030?l0003oP000oh000080o`006?ooool001Kooooo7@3o00000`3oP000
o`000?l0000m0?l00080oh00103o00000`3oP000o`000?l0000R0?l00080oh003_ooool;003o0080
003o4@00o`060000o`h00?l00P000?l4003o00030?n00000o`000?l000@0oh000`000?l6003o0003
0000o`3oP000oh0000T0oh0010000?l20?n00003003o003oP000oh0001800?l010000?l7003o0080
003o00<0oh000000o`000?l01P3oP005003o00kooooo10000?l00`3oP0000?l0003o00020000o`80
0?l00P3oP0090000o`D00?l00`000?l8003o00030000o`3oP000oh0000@0003o0P00o`050000o`L0
0?l02@000?l40?n00003003o003oP000oh000080003o0P00o`020?n000h00?l01`000?l=003o00<0
003o1`00o`020000o`H00?l01@000?l>ooooo`80oh000`00o`080?n00003003o003oP000oh000080
oh00103o00070?n00003003o003oP000oh0000L0oh000P00o`060?n000030?l0003oP000oh0000H0
0?l000<0oh00003o0000o`00103oP002003o00030000o`3oP000oh0000L0oh000P00o`040?l000<0
oh002`00o`020000o`80oh001@00o`020?n00080o`0000<0oh000?l0003o0000103o00000`3oP000
0?l0003o0002003o00D0003o103oP002003o0080oh0000<0o`000?n0003oP0000P3o000Hooooo`00
5_oooolK0?l000<0oh008`3o00000`3oP000o`000?l0000R0?l00080oh007P3o00020?n00080o`00
3_oooolf003o0080003o2@3oP0030000o`D00?l0203oP0000`00o`00oh000?n00004003o0080oh00
00<00?l00000o`000?l010000?l@003o0080003o00<00?l00000o`000?l00P000?l7003o00@0oh00
0`000?l4003o00kooooo0P000?l<003o00P0003o4@00o`080000o`h00?l01@000?l40?n000<0003o
103oP0020000o`D0oh000P00o`020000o`800?l030000?l=003o00030000o`00o`000?l00080oh00
0P00o`020000o`<00?l00P000?l2003o0080003o1@00o`0>ooooo`80oh000P3o00090?n00003003o
00000?l0003o0080oh00103o00000`3oP0000?l0003o0002003o00`0oh000P00o`020?n000800?l0
1@3oP006003o00<0oh000P3o00040?n00003003o00000?l0003o0080003o1`3oP002003o0080oh00
00<0o`00003o0000o`000P3oP0020000o`<00?l00P3oP004003o00<0oh001000o`020?n000<0o`00
103oP0020?l000<0oh000P00o`020000o`D00?l0103oP0050?l00080oh006?ooool001Kooooo6`3o
00030?n002<0o`0000<0oh000?l0003o00008P3o00020?n001h0o`000P3oP0020?l000kooooo=P00
o`020000o`T0oh000`000?l5003o00P0oh0000<00?l00?n0003oP0001000o`020?n00003003o0000
0?l0003o00@0003o4000o`020000o`03003o00000?l0003o0080003o1`00o`040?n000<0003o1000
o`0>ooooo`80003o3000o`080000oa400?l020000?l>003o00D0003o103oP0030000o`@0oh000P00
0?l50?n000800?l00P000?l2003o00`0003o3@00o`000`000?l00?l0003o00020?n000800?l00P00
0?l3003o0080003o0P00o`020000o`D00?l03_ooool20?n00080o`002@3oP0000`00o`00003o0000
o`020?n000@0o`0000<0oh00003o0000o`000P00o`0<0?n000800?l00P3oP002003o00D0oh001P00
o`030?n00080o`00103oP0000`00o`00003o0000o`020000o`L0oh000P00o`020?n000030?l00000
o`000?l00080oh000P000?l3003o0080oh001000o`030?n000@00?l00P3oP0030?l000@0oh000P3o
00030?n000800?l00P000?l5003o00@0oh001@3o00020?n001Sooooo000Foooooa/0o`000P3oP00C
0?l00080oh00=03o00020?n000/0o`0000<0oh000?l0003o0000403o00020?n00080o`003_ooool5
0000o`T00?l02@000?l7003o00@0003o0P3oP0020000oa800?l02@3oP005003o0100oh000P00o`03
0?n000@0003o0P00o`020?n000L0003o1@00o`020000o`800?l000<0oh00003o0000o`000P000?lB
003o00kooooo1@000?l8003o00D0003o0P00o`020000o`X00?l00P000?l>003o00P0003o1`00o`03
0000o`800?l00P3oP00;0000o`<0oh000P000?l;003o00<0003o0P00o`020000o`T00?l00`3oP002
0000o`H00?l02@000?l5003o00kooooo0P3oP0020?l000T0oh000`00o`020?n000D0o`000P000?l4
0?n000<00?l03@3oP0020000o`D0oh001000o`000`3oP000o`000?l000020?l00080oh000P3o0003
0?n00080o`00303oP002003o0080oh001@00o`040?n00080o`000P3oP003003o00@0oh000P3o0003
0?n00080o`001`3oP005003o0080oh00103o00020?n000030?l0003oP000oh000080oh006?ooool0
01Kooooo6`3o00020?n001<0o`000P3oP00d0?l00080oh002`3o00000`3oP000o`000?l0000@0?l0
0080oh000P3o000>ooooo`D0003o2@00o`090000o`L00?l010000?l20?n00080003o4P00o`090?n0
00D00?l0403oP002003o00<0oh0010000?l2003o0080oh001`000?l5003o0080003o0P00o`000`3o
P0000?l0003o00020000oa800?l03_ooool50000o`P00?l01@000?l2003o0080003o2P00o`020000
o`h00?l020000?l7003o00<0003o0P00o`020?n000/0003o0`3oP0020000o`/00?l00`000?l2003o
0080003o2@00o`030?n00080003o1P00o`090000o`D00?l03_ooool20?n00080o`002@3oP003003o
0080oh001@3o00020000o`@0oh000`00o`0=0?n00080003o1@3oP004003o00030?n0003o0000o`00
0080o`000P3oP0020?l000<0oh000P3o000<0?n000800?l00P3oP005003o00@0oh000P3o00020?n0
00<00?l0103oP0020?l000<0oh000P3o00070?n000D00?l00P3oP0040?l00080oh0000<0o`000?n0
003oP0000P3oP00Hooooo`005_ooool20?n001/0o`000`3oP00Q0?l000030?n0003o0000o`0000D0
o`000P3oP0020?l00080oh008P3o00020?n001<0o`000P3oP0020?l000kooooo6@00o`020000o`80
0?l000<0oh00003o0000o`000P000?lD003o0080003o5@3oP0040000o`<0oh001P00o`000`3oP000
0?l0003o000B003o0080003o3P00o`020000o`800?l00P3oP0030000o`80oh003_ooool7003o0080
003o0P00o`070000oa000?l00P000?lG003o00H0003o1`00o`030000o`800?l00P000?l7003o00d0
003o3000o`070000o`T00?l01P000?l5003o00kooooo1`3oP0020?l000D0oh000P00o`0@0?n00080
0?l00P3oP002003o00030?l0003oP000oh0000800?l00P000?l2003o00<0oh001000o`020?n00003
003o003oP000oh0000H0o`000`3oP002003o00T0oh000P00o`070?n000<0o`000P3oP0040?l000T0
oh00303o0006003o00030?n0003o0000o`000080oh000P3o00020?n000030?l0003oP000oh000080
oh006?ooool001KoooooI03o00020?n001@0o`000P3oP00:0?l000@0oh003_ooool2003o00<0003o
0P00o`060000obD00?l00P000?l5003o00@0oh0010000?l30?n00080o`000P3oP0000`3o00000?l0
003o00020?n00080o`001@3oP0060000o`<0oh002@00o`020?n000L00?l00P000?l;003o00030?n0
0000o`000?l000800?l00P3oP003003o00@0003o3_ooool40000o`T00?l000<0003o003o0000o`00
1P00o`030000o`800?l00P000?l3003o00H0003o6@00o`030000o`H00?l00`000?l4003o0080oh00
0`000?l9003o0080003o5000o`050000o`L00?l01P000?l00`3oP000003o0000o`02003o00kooooo
103o00070?n000<0o`002000o`000`3oP0000?l0003o0005003o0080oh001000o`030?n00080o`00
0P3oP002003o0080oh0000<00?l00?n0003oP0000P3oP0020000o`<0oh000P000?l4003o01T0oh00
0P3o00030?n00080o`000P3oP002003o00D0oh000P3o00070?n00080o`001`3oP0020000o`030?l0
003oP000oh0000@0oh000P3o00030?n000800?l06?ooool001KoooooI03o00020?n001@0o`000P3o
P00:0?l000@0oh003_ooool2003o00<0003o0P00o`060000obD00?l00P000?l5003o00@0oh001000
0?l30?n00080o`000P3oP0000`3o00000?l0003o00020?n00080o`001@3oP0060000o`<0oh002@00
o`020?n000L00?l00P000?l;003o00030?n00000o`000?l000800?l00P3oP003003o00@0003o3_oo
ool40000o`T00?l000<0003o003o0000o`001P00o`030000o`800?l00P000?l3003o00H0003o6@00
o`030000o`H00?l00`000?l4003o0080oh000`000?l9003o0080003o5000o`050000o`L00?l01P00
0?l00`3oP000003o0000o`02003o00kooooo103o00070?n000<0o`002000o`000`3oP0000?l0003o
0005003o0080oh001000o`030?n00080o`000P3oP002003o0080oh0000<00?l00?n0003oP0000P3o
P0020000o`<0oh000P000?l4003o01T0oh000P3o00030?n00080o`000P3oP002003o00D0oh000P3o
00070?n00080o`001`3oP0020000o`030?l0003oP000oh0000@0oh000P3o00030?n000800?l06?oo
ool001Kooooo403o00020?n003L0o`000P3oP00@0?l00080oh00503o00020?n00100o`000P3oP000
0`3o0000oh000?n000020?n000kooooo5`00o`020?n000@0003o00<0oh000000o`000?l00P000?l9
003o00@0003o2P00o`040?n000D0003o3`3oP0030000o`800?l00P3oP003003o0080oh000P00o`02
0?n00080003o00<0oh00003o0000o`00103oP007003o0080003o0`00o`020000o`80oh000`000?l2
0?n000@0003o0P3oP003003o0080003o3_ooool20000o`<00?l010000?l>003o00H0003o4000o`05
0000o`80oh004P00o`030000oad00?l01`000?l6003o00030000o`00o`000?l000L00?l010000?l9
003o00kooooo0P3oP0020?l00003003o003o0000o`0000@0oh000P3o00030?n000800?l00P000?l3
0?n00080003o0P00o`0E0?n000H0003o1`00o`020?l000800?l01`3oP0000`000?l0oh000?n0000;
0?n00080o`000P3oP0000`3o00000?l0003o00020?n00080o`000P00o`050?n00080o`000P3oP000
0`3o0000oh000?n000050?n000@0o`001000o`030?n00080o`001`00o`020?n001Sooooo000Foooo
oa00o`000P3oP00g0?l00080oh00403o00020?n001@0o`000P3oP00@0?l00080oh0000<0o`000?n0
003oP0000P3oP00>oooooaL00?l00P3oP0040000o`030?n000000?l0003o0080003o2@00o`040000
o`X00?l0103oP0050000o`l0oh000`000?l2003o0080oh000`00o`020?n000800?l00P3oP0020000
o`030?n00000o`000?l000@0oh001`00o`020000o`<00?l00P000?l20?n000<0003o0P3oP0040000
o`80oh000`00o`020000o`kooooo0P000?l3003o00@0003o3P00o`060000oa000?l01@000?l20?n0
01800?l00`000?lM003o00L0003o1P00o`000`000?l00?l0003o0007003o00@0003o2@00o`0>oooo
o`80oh000P3o00000`00o`00o`000?l000040?n00080o`000`3oP002003o0080003o0`3oP0020000
o`800?l05@3oP0060000o`L00?l00P3o0002003o00L0oh0000<0003o0?n0003oP0002`3oP0020?l0
0080oh0000<0o`00003o0000o`000P3oP0020?l000800?l01@3oP0020?l00080oh0000<0o`000?n0
003oP0001@3oP0040?l000@00?l00`3oP0020?l000L00?l00P3oP00Hooooo`005_oooom20?l00080
oh00;@3o00030?n000H0o`000P3oP0070?l000L0oh003_ooool9003o0080003o8000o`020000o`T0
0?l00P3oP00:0000o`L00?l01P3oP009003o0080003o00<0oh000000o`000?l00P3oP0040000o`03
0?n0003o0000o`0000@0oh003P00o`040000o`030?n00000o`000?l00080003o2@00o`0>ooooo`L0
003o8000o`040000o`800?l01@000?lM003o00L0003o1000o`030000o`80oh001`000?lO003o0003
0000o`00o`000?l000800?l03_ooool40?n000030?l0003oP000oh000080oh000P3o00020?n00003
0?l0003oP000oh000080oh000P000?l20?n000030000o`00o`000?l000L0oh000P00o`020?n00080
0?l000<0003o0?n0003oP0000P3oP0020?l000<0oh001000o`020000o`800?l00`000?l60?n00003
0?l0003oP000oh0000800?l02@3oP002003o00030?n00000o`000?l00080oh000P000?l70?n00080
o`001@3oP0020?l000@0oh0000<0o`000?n0003oP0000P3oP0020?l000<0oh000P3o00020?n00080
0?l000<0oh000?l0003o00000P3o00050?n000800?l00P3oP00Hooooo`005_oooom20?l00080oh00
;@3o00030?n000H0o`000P3oP0070?l000L0oh003_ooool9003o0080003o8000o`020000o`T00?l0
0P3oP00:0000o`L00?l01P3oP009003o0080003o00<0oh000000o`000?l00P3oP0040000o`030?n0
003o0000o`0000@0oh003P00o`040000o`030?n00000o`000?l00080003o2@00o`0>ooooo`L0003o
8000o`040000o`800?l01@000?lM003o00L0003o1000o`030000o`80oh001`000?lO003o00030000
o`00o`000?l000800?l03_ooool40?n000030?l0003oP000oh000080oh000P3o00020?n000030?l0
003oP000oh000080oh000P000?l20?n000030000o`00o`000?l000L0oh000P00o`020?n000800?l0
00<0003o0?n0003oP0000P3oP0020?l000<0oh001000o`020000o`800?l00`000?l60?n000030?l0
003oP000oh0000800?l02@3oP002003o00030?n00000o`000?l00080oh000P000?l70?n00080o`00
1@3oP0020?l000@0oh0000<0o`000?n0003oP0000P3oP0020?l000<0oh000P3o00020?n000800?l0
00<0oh000?l0003o00000P3o00050?n000800?l00P3oP00Hooooo`005_ooool20?n001`0o`000P3o
P00O0?l00080oh00203o00020?n002H0o`000P3oP0070?l00080oh0000<0o`000?n0003oP0001`3o
00020?n00080o`003_oooolm003o00@0oh000`000?l=0?n000030000o`00o`000?l00080003o0P00
o`030000o`H0oh000P000?l00`3oP000003o0000o`040?n000h00?l00P000?l@003o00kooooo1000
0?l:003o0080003o6`00o`050000o`T00?l00P000?l;003o00D0003o1000o`0>0000o`80oh001@00
0?l8003o00030000o`00o`000?l000L0003o2@00o`0;0000o`kooooo0P00o`020?n000030?l0003o
P000oh0000@0oh000P3o00000`3oP000o`000?l00004003o0080oh0000<0003o003o0000o`000P3o
P002003o00030?n00000o`000?l00080oh001000o`000`000?l0oh000?n000070?n000P00?l0303o
P002003o00<0oh00103o00020?n000800?l000<0003o0?n0003oP0000P00o`020?n00080o`001@3o
P0020?l000L0oh00103o00000`3oP000o`000?l000050?n00080o`000P3oP002003o0080oh000`3o
00060?n000D00?l06?ooool001Kooooo0P3oP00L0?l00080oh007`3o00020?n000P0o`000P3oP00V
0?l00080oh001`3o00020?n000030?l0003oP000oh0000L0o`000P3oP0020?l000kooooo?@00o`04
0?n000<0003o3@3oP0000`000?l00?l0003o00020000o`800?l00`000?l60?n00080003o00<0oh00
0000o`000?l0103oP00>003o0080003o4000o`0>ooooo`@0003o2P00o`020000oa/00?l01@000?l9
003o0080003o2`00o`050000o`@00?l03P000?l20?n000D0003o2000o`000`000?l00?l0003o0007
0000o`T00?l02`000?l>ooooo`800?l00P3oP0000`3o0000oh000?n000040?n00080o`0000<0oh00
0?l0003o00001000o`020?n000030000o`00o`000?l00080oh000P00o`000`3oP0000?l0003o0002
0?n000@00?l000<0003o0?n0003oP0001`3oP008003o00`0oh000P00o`030?n000@0o`000P3oP002
003o00030000o`3oP000oh0000800?l00P3oP0020?l000D0oh000P3o00070?n000@0o`0000<0oh00
0?l0003o00001@3oP0020?l00080oh000P00o`020?n000<0o`001P3oP005003o01Sooooo000Foooo
oa00o`000P3oP00@0?l00080oh009@3o00020?n002@0o`000P3oP0050?l00080oh000P3o00070?n0
00T0o`003_oooolN003o00@0oh005000o`020?n000030000o`3oP000oh0000@0oh002@00o`000`00
0?l0oh000?n000040000o`80oh004`00o`040000o`T00?l00P000?l@003o00<0oh001000o`0>oooo
obH00?l01@000?l5003o0080003o7@00o`090000o`D00?l01`000?l2003o00T0003o4000o`060000
o`030?n00000o`000?l000800?l03_ooool40?n000030?l0003oP000oh0000H0oh000`00o`020?n0
00800?l00P3oP003003o0080oh000P000?l30?n000T00?l00P3oP0020?l00080oh002P00o`020000
o`800?l0103oP0000`3o0000oh000?n000090?n000@00?l01`3oP0020?l000T0oh000`3o00020?n0
0080o`000P3oP0000`3o0000oh000?n0000>0?n00080o`00103oP002003o00030?n000000?l0003o
0080oh006?ooool001Kooooo403o00020?n003H0o`0000<0oh000?l0003o00002@3o00020?n000@0
o`0000<0oh000?l0003o00004P3o00020?n000L0o`000P3oP0020?l00003003o003oP000oh000080
0?l02@3o000>ooooo`800?l00P000?l:003o0080oh002`00o`020?n00003003o003oP000oh000080
oh007@00o`020?n000<00?l00P3oP003003o0080oh001000o`020?n00003003o00000?l0003o0080
003o1000o`000`3oP000003o0000o`020000o`80oh001`000?l;003o00D0003o1000o`030000o`L0
0?l00P3oP002003o00kooooo40000?l6003o00<0003o1@00o`020000o`H00?l000<0003o003o0000
o`001P00o`030000o`l00?l000<0003o003o0000o`004000o`0;0000ob/00?l03_ooool20?n00080
o`001@3oP0020000o`80oh000`000?l20?n000@0003o2P00o`020?n000800?l00`000?l20?n00080
o`000P3oP0030?l000T00?l0203oP0000`3o0000oh000?n000070?n00080o`000P00o`070?n000/0
o`000P3oP0030?l000@0oh000P3o00050?n000800?l000<0o`000?n0003oP000103o00030?n000/0
0?l00P3oP00Hooooo`005_ooool@0?l00080oh00=P3o00000`3oP000o`000?l000090?l00080oh00
103o00000`3oP000o`000?l0000B0?l00080oh001`3o00020?n00080o`0000<00?l00?n0003oP000
0P00o`090?l000kooooo0P00o`020000o`X00?l00P3oP00;003o0080oh0000<00?l00?n0003oP000
0P3oP00M003o0080oh000`00o`020?n000<00?l00P3oP004003o0080oh0000<00?l00000o`000?l0
0P000?l4003o00030?n000000?l0003o0080003o0P3oP0070000o`/00?l01@000?l4003o00<0003o
1`00o`020?n000800?l03_ooool@0000o`H00?l00`000?l5003o0080003o1P00o`000`000?l00?l0
003o0006003o00<0003o3`00o`000`000?l00?l0003o000@003o00/0003o:`00o`0>ooooo`80oh00
0P3o00050?n00080003o0P3oP0030000o`80oh0010000?l:003o0080oh000P00o`030000o`80oh00
0P3o00020?n000<0o`002@00o`080?n000030?l0003oP000oh0000L0oh000P3o0002003o00L0oh00
2`3o00020?n000<0o`00103oP0020?l000D0oh000P00o`000`3o0000oh000?n000040?l000<0oh00
2`00o`020?n001Sooooo000Fooooo`80oh00303o00020?n000L0o`000P3oP00I0?l00080oh002`3o
00020?n000X0o`000P3oP00[0?l00080oh002`3o00000`3oP000o`000?l000020?l000kooooo5`00
o`040000o`T00?l00P000?l5003o00@0003o1@00o`040000o`03003o003oP000oh000080o`000P3o
P002003o00<0oh001000o`000`3oP0000?l0003o00020?l000T00?l00P3oP00;003o00030000o`00
o`000?l001800?l010000?l5003o00D0oh001000o`0>ooooo`D00?l02`000?lO003o00030000o`00
o`000?l001/00?l00P3oP0030000oa@00?l00P3oP0020?l00080oh000`00o`040000o`h00?l02@00
0?l>ooooo`L0oh000P000?l40?n000<00?l01P3oP00A003o0080oh000P3o00020?n000D0o`000P00
0?l2003o0080003o1`3oP00>0?l00080oh000P3o00070?n000D0o`001P3oP0000`3o0000oh000?n0
00060?l000030?n00000o`000?l00080003o0P3oP0000`3o0000oh000?n000060?l000030?n00000
o`000?l000H00?l01@3oP00Hooooo`005_ooool20?n000`0o`000P3oP0070?l00080oh006@3o0002
0?n000/0o`000P3oP00:0?l00080oh00:`3o00020?n000/0o`0000<0oh000?l0003o00000P3o000>
oooooaL00?l010000?l9003o0080003o1@00o`040000o`D00?l010000?l00`00o`00oh000?n00002
0?l00080oh000P00o`030?n000@00?l000<0oh00003o0000o`000P3o0009003o0080oh002`00o`00
0`000?l00?l0003o000B003o00@0003o1@00o`050?n000@00?l03_ooool5003o00/0003o7`00o`00
0`000?l00?l0003o000K003o0080oh000`000?lD003o0080oh000P3o00020?n000<00?l010000?l>
003o00T0003o3_ooool70?n00080003o103oP003003o00H0oh004@00o`020?n00080o`000P3oP005
0?l00080003o0P00o`020000o`L0oh003P3o00020?n00080o`001`3oP0050?l000H0oh0000<0o`00
0?n0003oP0001P3o00000`3oP0000?l0003o00020000o`80oh0000<0o`000?n0003oP0001P3o0000
0`3oP0000?l0003o0006003o00D0oh006?ooool001Kooooo0P3oP00E0?l00080oh006@3o00020?n0
01@0o`0000<0oh000?l0003o0000903o00020?n001D0o`000P3oP0020?l000kooooo2`00o`020000
oa000?l01`000?lE003o0080oh000P00o`040?n00003003o003oP000oh000080oh000P000?l90?n0
00L00?l00P000?l5003o00@0oh006@00o`030000o`800?l00P000?l7003o00kooooo1@00o`020?n0
00L00?l010000?l2003o0080003o1`00o`0:0000o`80oh001P00o`000`000?l00?l0003o0004003o
00<0003o0P3oP00@003o00@0oh004000o`020?n000<00?l01P3oP00I003o0080003o00<00?l00000
o`000?l00P00o`0>ooooo`80oh000P3o00030?n00080003o0P00o`090?n001<00?l00P3oP0090?l0
00800?l00P3oP005003o00H0oh0000<0o`000?n0003oP0000P3oP0020?l000<0oh001P3o00050?n0
00D0o`00203oP0050?l000D0oh000P00o`020?n00080003o00<0oh000?l0003o00001P3o000:0?n0
00@0o`006?ooool001Kooooo0P3oP00E0?l00080oh006@3o00020?n001@0o`0000<0oh000?l0003o
0000903o00020?n001D0o`000P3oP0020?l000kooooo2`00o`020000oa000?l01`000?lE003o0080
oh000P00o`040?n00003003o003oP000oh000080oh000P000?l90?n000L00?l00P000?l5003o00@0
oh006@00o`030000o`800?l00P000?l7003o00kooooo1@00o`020?n000L00?l010000?l2003o0080
003o1`00o`0:0000o`80oh001P00o`000`000?l00?l0003o0004003o00<0003o0P3oP00@003o00@0
oh004000o`020?n000<00?l01P3oP00I003o0080003o00<00?l00000o`000?l00P00o`0>ooooo`80
oh000P3o00030?n00080003o0P00o`090?n001<00?l00P3oP0090?l000800?l00P3oP005003o00H0
oh0000<0o`000?n0003oP0000P3oP0020?l000<0oh001P3o00050?n000D0o`00203oP0050?l000D0
oh000P00o`020?n00080003o00<0oh000?l0003o00001P3o000:0?n000@0o`006?ooool001Kooooo
0P00o`020?n000030?l0003oP000oh0000L0o`000P3oP00;0?l00080oh00303o00020?n00180o`00
0P3oP0090?l000030?n0003o0000o`0001/0o`000P3oP00E0?l00080oh000P3o0002003o00L0o`00
3_ooool2003o0080oh00;P00o`060000oa400?l00P3oP002003o0080oh002`00o`000`3oP0000?l0
003o0002003o0080003o0`00o`020000o`80oh001000o`000`3oP0000?l0003o000G003o0080003o
0P00o`0>ooooob800?l01@000?l20?n00080003o0P00o`020?n001<00?l01`000?l;003o00H0003o
2P00o`020?n000L0003o1000o`020?n000030000o`00o`000?l000l00?l000<0003o003o0000o`00
0P00o`0>ooooo`80o`000P3oP003003o0080oh000P00o`030?n000P00?l000<0003o003o0000o`00
3P00o`060?n00080o`000`3oP0020?l00080oh000P00o`090?n000<0o`001`3oP0020?l00080oh00
0P3o000B0?n000T0o`0000<00?l00?n0003oP00010000?l70?n000<0o`000P3oP0020?l000800?l0
0`3oP0040?l001Sooooo000Fooooo`80o`000P3oP00:0?l00080oh001`3o00020?n00100o`000P3o
P00K0?l00080oh007P3o00020?n000/0o`0000<0oh000?l0003o00000P3o00020?n000<0o`000P3o
P00;0?l000kooooo3P00o`020?n002P00?l000<0003o0?n0003oP0000P000?l40?n000<00?l00P3o
P0020000o`T0oh001@000?l2003o00<0003o1000o`020?n000<0003o0P00o`020?n000D00?l01000
0?l>003o0080oh000P00o`030?n000@00?l03_ooool5003o0080oh000P000?l=003o00D0003o2@00
o`020000o`L00?l00P000?l00`00o`00oh000?n000060000o`030?n00000o`000?l000H00?l000<0
oh00003o0000o`005`00o`0F0000oa@00?l000<0003o003o0000o`000P00o`0>ooooo`800?l00`3o
P004003o0080003o0P00o`000`3oP0000?l0003o00020?n000D00?l00P3oP007003o00L0oh000P3o
00020?n000800?l00`3o00020?n000800?l00P3oP0020?l000T0oh0000<0o`000?n0003oP0002@3o
P0020?l000D0oh000P00o`040?n000D00?l01@3oP0020?l00080oh00103o00000`3oP0000?l0003o
00020?n000800?l00`3oP002003o00@0oh0000<0o`000?n0003oP0000P3o00020?n00080o`001@3o
P00Hooooo`005_ooool20?l00080oh002P3o00020?n000L0o`000P3oP00@0?l00080oh006`3o0002
0?n001h0o`000P3oP00;0?l000030?n0003o0000o`000080o`000P3oP0030?l00080oh002`3o000>
ooooo`h00?l00P3oP00X003o00030000o`3oP000oh000080003o103oP003003o0080oh000P000?l9
0?n000D0003o0P00o`030000o`@00?l00P3oP0030000o`800?l00P3oP005003o00@0003o3P00o`02
0?n000800?l00`3oP004003o00kooooo1@00o`020?n00080003o3@00o`050000o`T00?l00P000?l7
003o0080003o00<00?l00?n0003oP0001P000?l00`3oP0000?l0003o0006003o00030?n00000o`00
0?l001L00?l05P000?lD003o00030000o`00o`000?l000800?l03_ooool2003o00<0oh001000o`02
0000o`800?l000<0oh00003o0000o`000P3oP005003o0080oh001`00o`070?n00080o`000P3oP002
003o00<0o`000P3oP002003o0080oh000P3o00090?n000030?l0003oP000oh0000T0oh000P3o0005
0?n000800?l0103oP005003o00D0oh000P3o00020?n000@0o`0000<0oh00003o0000o`000P3oP002
003o00<0oh000P00o`040?n000030?l0003oP000oh000080o`000P3oP0020?l000D0oh006?ooool0
01Kooooo9`3o00020?n004@0o`000P3oP0040?l000030?n0003o0000o`0000L0o`000P3oP0070?l0
0080oh000P3o000>ooooo`80003o>@00o`020000o`800?l01@3oP004003o00<0oh000P3o00040?n0
00D00?l00P3oP003003o00H0003o1`00o`020?n00080003o0`3oP00@003o0080oh001000o`000`3o
P0000?l0003o0002003o00kooooo0P00o`030000oa@00?l00P000?lK003o0080003o5@00o`020000
o`80oh009@00o`020000o`T00?l010000?l5003o00kooooo0P000?l20?n000D00?l00P000?l70?n0
00D00?l00P3o00020?n000800?l00`3oP002003o00@0oh0000<00?l00?n0003oP0000P3o00090?n0
0080o`001@3oP002003o00@0oh0000<0o`000?n0003oP000103oP0000`00o`00003o0000o`020000
o`80oh000P00o`000`3oP000o`000?l000020?n00080003o2`3oP0090?l000`0oh001000o`030?n0
0080o`00103oP0000`00o`00oh000?n000020?n001Sooooo000FooooobL0o`000P3oP0140?l00080
oh00103o00000`3oP000o`000?l000070?l00080oh001`3o00020?n00080o`003_ooool20000ocT0
0?l00P000?l2003o00D0oh001000o`030?n00080o`00103oP005003o0080oh000`00o`060000o`L0
0?l00P3oP0020000o`<0oh004000o`020?n000@00?l000<0oh00003o0000o`000P00o`0>ooooo`80
0?l00`000?lD003o0080003o6`00o`020000oaD00?l00P000?l20?n002D00?l00P000?l9003o00@0
003o1@00o`0>ooooo`80003o0P3oP005003o0080003o1`3oP005003o0080o`000P3oP002003o00<0
oh000P00o`040?n00003003o003oP000oh000080o`002@3oP0020?l000D0oh000P00o`040?n00003
0?l0003oP000oh0000@0oh0000<00?l00000o`000?l00P000?l20?n000800?l000<0oh000?l0003o
00000P3oP0020000o`/0oh002@3o000<0?n000@00?l00`3oP0020?l000@0oh0000<00?l00?n0003o
P0000P3oP00Hooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2
ooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooo
oooooooooh;ooooo003oooooooooooooP_ooool002;oooooL_l0000Wooooog7o00009ooooomao`00
02OoooooLOl0000Tooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`001OoooolFo`3o00Co
oooo5ol00?l4oooooaOoogl01?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;o
oooo5_oo0005oooooaKo0?l01OoooolFo`00o`Cooooo5oooO`04ooooo`03o`000?oooooooooo02Go
oooo00?o0000ooooooooool00_oooolGool000Cooooo5_l0o`05oooooaKo003o1OoooolFoomo00Co
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02oooooaOoo`001?oooolFo`3o00Go
oooo5_l00?l5oooooaKoogl01?ooool00ol0003oooooooooo`0Rooooo`008_ooool00ol0003ooooo
ooooo`03oooooaKoo`00>_oooolGoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`02oooooaKoo`001OoooolFo`3o00Gooooo5_l00?l4oooooaOoogl01?ooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o1OoooolFoomo00Cooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`02oooooaOoo`001?oooolFo`3o00Gooooo
5_l00?l5oooooaKoogl01?ooool00ol0003oooooooooo`0Rooooo`008_ooool00ol0003ooooooooo
o`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5_oo
0005oooooaKo0?l01OoooolFo`00o`Cooooo5oooO`04ooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool07OoooolFo`3o00Gooooo5_l00?l5oooooaKoogl01?ooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo00;ooooo5ooo0004oooooaKo0?l01OoooolFo`00o`Gooooo5_oo
O`04ooooo`03o`000?oooooooooo02;ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001E
ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?l4
oooooaOoogl01?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05
oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo0004
oooooaKo0?l01OoooolFo`00o`Gooooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo000Roooo
o`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool00_oooolGool000Cooooo5_l0o`05oooooaKo003o1OoooolFoomo00Cooooo00?o0000oooo
ooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`02oooooaOoo`00>_oooolFoomo00Cooooo00?o0000oooooooo
ool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`02oooooaOoo`00>_oooolFoomo00Cooooo00?o0000ooooooooool0
8_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`02oooooaOoo`00>_oooolFoomo00Cooooo00?o0000ooooooooool08_oo
ool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`02oooooaOoo`00>_oooolFoomo00Cooooo00?o0000ooooooooool08_ooool0
02;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`02oooooaOoo`00>_oooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;o
oooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`02oooooaOoo`00>_oooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo
00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`02oooooaOoo`00>_oooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o
0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`02oooooaOoo`00>_oooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000
ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0M
oooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0M
oooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02
oooooaOoo`00>_oooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000o`00
Ool007l00ol007lFool005Go001o00?o0000ooooooooool09Oooool00ol0003o001oo`00O`0Mo`00
OaKo0?l01Ol007lFo`00oaoo001o00?o0000ooooooooool09Oooool00ol0003o001oo`00O`0Mo`00
OaKo0?l01Ol007lFo`00oaoo001o00?o0000ooooooooool09Oooool00ol0003o001oo`00O`1Co`00
OaKoogl01?l007l00ol0003oooooooooo`0Rooooo`008_ooool00ol0003oooooooooo`03oooooaKo
o`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo
003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo
003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`04ooooo`03
o`000?oooooooooo02;ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool0
8_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`1CoooooaKoogl01?ooool00ol0003oooooooooo`0Rooooo`008_ooool0
0ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo05?ooooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo000Rooooo`03o`000?oooooooooo
00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o
00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o
00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo
00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Go
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoo
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoo
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1CoooooaKoogl01?ooool00ol0003o
ooooooooo`0Rooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo
000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000
ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0M
oooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0M
oooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1C
oooooaKoogl01?ooool00ol0003oooooooooo`0Rooooo`008_ooool00ol0003oooooooooo`03oooo
oaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooo
oaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooo
oaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`04oooo
o`03o`000?oooooooooo02;ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000oooooooo
ool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`1CoooooaKoogl01?ooool00ol0003oooooooooo`0Rooooo`008_oo
ool00ol0003o001oo`00O`03o`00OaKoo`00EOl007l00ol0003oooooooooo`0Uooooo`03o`000?l0
07oo001o01go001o5_l0o`05o`00OaKo003o7ol007l00ol0003oooooooooo`0Uooooo`03o`000?l0
07oo001o01go001o5_l0o`05o`00OaKo003o7ol007l00ol0003oooooooooo`0Uooooo`03o`000?l0
07oo001o05?o001o5_ooO`04o`00O`03o`000?oooooooooo02;ooooo000Rooooo`03o`000?oooooo
oooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolF
o`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolF
o`3o00Gooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolF
oomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool0
05Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;o
oooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`0MoooooaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo03So
oooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Co
oooo00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o
0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`0MoooooaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l0
0?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Cooooo00?o
0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo03Sooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000oooo
ooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0Moooo
oaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOoooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000oooo
ooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0>_ooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo03Sooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000oooooooo
ool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool0
0ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0
>_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool0
8_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0>?oo
oolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1CoooooaKoogl01?oo
ool00ol0003oooooooooo`0Rooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolF
oomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool0
05Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
05?ooooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo000Rooooo`03o`000?l007oo001o00?o
001o5_oo001Eo`00O`03o`000?oooooooooo02Gooooo00?o0000o`00Ool007l0KOl007l00ol0003o
ooooooooo`0Uooooo`03o`000?l007oo001o06go001o00?o0000ooooooooool09Oooool00ol0003o
001oo`00O`1Co`00OaKoogl01?l007l00ol0003oooooooooo`0Rooooo`008_ooool00ol0003ooooo
ooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o
0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`04ooooo`03o`000?oooooooooo02;ooooo000R
ooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`1CoooooaKoogl01?ooool00ol0003oooooooooo`0R
ooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]oooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Cooooo00?o0000oooo
ooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000oooooooo
ool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`04ooooo`03
o`000?oooooooooo02;ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1CoooooaKoogl0
1?ooool00ol0003oooooooooo`0Rooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOoo
ool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0Dooo
oolFoomo00Cooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolF
ool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooo
oooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo06gooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool0
05Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
06gooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Go
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Go
oooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06go
oooo00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo
00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o
0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o
0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000
ooooooooool08_ooool002;ooooo00?o0000o`00Ool007l00ol007lFool005Go001o00?o0000oooo
ooooool09Oooool00ol0003o001oo`00O`1]o`00O`03o`000?oooooooooo02Gooooo00?o0000o`00
Ool007l0KOl007l00ol0003oooooooooo`0Uooooo`03o`000?l007oo001o06go001o00?o0000oooo
ooooool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000oooooooo
ool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000oooooooo
ool08_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool0
8_ooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOoo
ool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool08_oo
ool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool08_ooool0
02;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool08_ooool002;o
oooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003o
ooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool08_ooool002;ooooo
00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool08_ooool002;ooooo00?o
0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000
ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]
ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000oooo
ooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]oooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo06gooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000oooooooo
ool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo06gooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool0
0ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo06gooooo00?o0000ooooooooool08_ooool002;ooooo00?o0000ooooooooool0K_oo
ool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOoo
ool00ol0003oooooooooo`0Rooooo`008_oooombo`0002OoooooLOl0000Wooooog7o00009oooooma
o`0002Cooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo
003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003ooooooooo
ooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo000Fooooog40o`000P3oP00G
0?l000kooooo0P3o0003003o0080003o0P00o`020000o`800?l05@000?l40?n000030?l00000o`00
0?l00080003o0P00o`030000o`800?l01`3oP008003o00<0o`001000o`050000o`D00?l010000?l2
003o0080003o0`00o`020000oa800?l00P000?l7003o0080o`0000<00?l00000o`000?l00P00o`02
0000o`80oh001@00o`0>ooooo`/00?l00`000?l2003o00d0003o00<00?l00000o`000?l02`000?l7
003o00@0003o2@00o`030000o`@00?l00`000?l2003o00@0003o0P3oP00N0000o`@00?l000<0003o
003o0000o`0050000?l>ooooo`80o`000P3oP0000`3o0000oh000?n000020?n00080o`002`3oP000
0`000?l0oh000?n000070?n000@0003o1@3oP0020?l00080oh001@3o00060000o`D0oh000P00o`03
0?n00080o`000P3oP002003o00L0oh000P3o000:0?n000@0o`001@3oP0060?l000X0oh000P00o`02
0?n00080003o0`3o0004003o0080oh000`00o`040?n001Sooooo000Fooooog40o`000P3oP00G0?l0
00kooooo0P3o0003003o0080003o0P00o`020000o`800?l05@000?l40?n000030?l00000o`000?l0
0080003o0P00o`030000o`800?l01`3oP008003o00<0o`001000o`050000o`D00?l010000?l2003o
0080003o0`00o`020000oa800?l00P000?l7003o0080o`0000<00?l00000o`000?l00P00o`020000
o`80oh001@00o`0>ooooo`/00?l00`000?l2003o00d0003o00<00?l00000o`000?l02`000?l7003o
00@0003o2@00o`030000o`@00?l00`000?l2003o00@0003o0P3oP00N0000o`@00?l000<0003o003o
0000o`0050000?l>ooooo`80o`000P3oP0000`3o0000oh000?n000020?n00080o`002`3oP0000`00
0?l0oh000?n000070?n000@0003o1@3oP0020?l00080oh001@3o00060000o`D0oh000P00o`030?n0
0080o`000P3oP002003o00L0oh000P3o000:0?n000@0o`001@3oP0060?l000X0oh000P00o`020?n0
0080003o0`3o0004003o0080oh000`00o`040?n001Sooooo000Fooooo`80oh00R03o000>ooooo`@0
o`002P00o`020000o`800?l00P000?l3003o00@0003o1@00o`020000o`@0o`000`00o`020000o`80
0?l00`000?l4003o0080003o2`00o`050?l000800?l01@000?l5003o0080003o0P00o`040000o`<0
0?l010000?l3003o00@0o`001000o`000`000?l00?l0003o0009003o00@0o`0000<00?l00?l0003o
00002`00o`0>ooooo`800?l00`000?l8003o00030000o`00o`000?l000800?l07@000?l>003o01T0
003o0P00o`090000o`800?l01@000?l2003o0080003o00<00?l00000o`000?l01000o`030000o`@0
0?l00`000?l2003o00H0003o0`00o`020000o`kooooo5`3oP0020000o`80o`002@3oP0030?l00080
oh000P3o000;0?n00080003o403oP0090?l000030000o`00o`000?l000H0003o00<00?l00?l0003o
00000P3o000V0?n000<0o`000P3oP00Hooooo`005_ooool20?n008P0o`003_ooool40?l000X00?l0
0P000?l2003o0080003o0`00o`040000o`D00?l00P000?l40?l000<00?l00P000?l2003o00<0003o
1000o`020000o`/00?l01@3o0002003o00D0003o1@00o`020000o`800?l010000?l3003o00@0003o
0`00o`040?l000@00?l000<0003o003o0000o`002@00o`040?l00003003o003o0000o`0000/00?l0
3_ooool2003o00<0003o2000o`000`000?l00?l0003o0002003o01d0003o3P00o`0I0000o`800?l0
2@000?l2003o00D0003o0P00o`020000o`03003o00000?l0003o00@00?l00`000?l4003o00<0003o
0P00o`060000o`<00?l00P000?l>oooooaL0oh000P000?l20?l000T0oh000`3o00020?n00080o`00
2`3oP0020000oa00oh002@3o00000`000?l00?l0003o00060000o`03003o003o0000o`000080o`00
9P3oP0030?l00080oh006?ooool001KoooooC`3o00020?n003T0o`003_ooool20?l000D00?l00P00
0?l4003o00X0003o0P00o`020000o`800?l01`000?l30?n000@0003o1000o`000`000?l00?l0003o
0002003o00L0003o1000o`050?l001T00?l00P000?l4003o00<0oh000P3o0004003o00D0003o2@00
o`020000o`03003o00000?l0003o0080003o1@00o`020000o`800?l03_ooool50000o`/00?l02`00
0?l3003o0080003o0P00o`0R0000o`@00?l000<0003o003o0000o`001`00o`020000o`@00?l00P00
0?l00`00o`00003o0000o`02003o0080003o0P00o`050000o`@00?l000<0003o0?n0003oP0003P00
0?l8003o00030000o`00o`000?l00080003o3_ooool90?n000T0o`000P00o`0@0?n000<0o`002`3o
P002003o00@0003o00<00?l00?l0003o00000P3o00020?n00080o`0000<0003o0?n0003oP0000P00
o`020?n000D0o`000P000?l50?n00080o`000P00o`020000o`<00?l01P3oP0030?l000L0oh001`3o
0002003o0080oh000P00o`020000o`L0o`001@3oP0020?l001Sooooo000FooooohD0o`0000<0oh00
0?l0003o00000P3o000>ooooo`80003o0`00o`040000o`T00?l00P000?l2003o00L0003o2P00o`04
0000o`800?l00`000?l8003o00T0003o00<0o`000?n0003oP0000P3oP00D003o00D0003o0P00o`02
0000o`03003o00000?l0003o00800?l00P000?l2003o00P0003o0P00o`020000o`800?l00P000?l0
0`00o`00oh000?n000060?n00003003o00000?l0003o0080003o3_ooool4003o00<0003o1000o`03
0000o`@00?l00P000?l2003o00X0003o1000o`090000o`800?l02@000?l00`00o`00003o0000o`02
0000o`800?l02P000?l2003o01@0003o0P00o`0;0000o`03003o00000?l0003o0080003o0P00o`00
0`000?l00?l0003o0002003o00@0003o00<00?l00000o`000?l010000?l2003o00D0003o3_ooool=
0?n000030?l0003oP000oh0000h0oh000P3o00020?n000D0o`002`3oP002003o00@0oh000`3o000;
0?n000<0003o0P3o000;0?n000800?l00P000?l30?n000@0o`000P3oP0000`3o0000oh000?n00007
0?n00080003o1@3oP0040?l000@0003o2@3o00030?n000800?l06?ooool001KoooooQ@3o00000`3o
P000o`000?l000020?l000kooooo0P000?l3003o00@0003o2@00o`020000o`800?l01`000?l:003o
00@0003o0P00o`030000o`P00?l02@000?l00`3o0000oh000?n000020?n001@00?l01@000?l2003o
0080003o00<00?l00000o`000?l00P00o`020000o`800?l020000?l2003o0080003o0P00o`020000
o`03003o003oP000oh0000H0oh0000<00?l00000o`000?l00P000?l>ooooo`@00?l00`000?l4003o
00<0003o1000o`020000o`800?l02P000?l4003o00T0003o0P00o`090000o`03003o00000?l0003o
0080003o0P00o`0:0000o`800?l050000?l2003o00/0003o00<00?l00000o`000?l00P000?l2003o
00030000o`00o`000?l000800?l010000?l00`00o`00003o0000o`040000o`800?l01@000?l>oooo
o`d0oh0000<0o`000?n0003oP0003P3oP0020?l00080oh001@3o000;0?n000800?l0103oP0030?l0
00/0oh000`000?l20?l000/0oh000P00o`020000o`<0oh00103o00020?n000030?l0003oP000oh00
00L0oh000P000?l50?n000@0o`0010000?l90?l000<0oh000P00o`0Hooooo`005_ooool20?n008P0
o`003_ooool20000o`80oh001@000?l7003o00T0003o1000o`0C0000o`P00?l02P000?l2003o0080
003o0`00o`020000o`800?l00P000?l2003o00030000o`00o`000?l000@0003o0P00o`000`3oP000
003o0000o`020000o`80oh000`00o`020000o`800?l00P000?l3003o0080003o1@00o`020000o`80
0?l010000?l30?l000@00?l00P000?l00`00o`00003o0000o`02003o00kooooo1`000?l@003o00l0
003o00<00?l00000o`000?l010000?l2003o00030000o`00o`000?l000l0003o1@00o`020?n00003
0000o`3oP000oh000080003o1000o`000`3oP0000?l0003o0002003o0080003o0`00o`060000o`80
oh0000<0003o0?n0003oP0000P000?l;003o00030000o`00o`000?l000@0003o1@3oP0020000o`80
0?l01`000?l>ooooo`800?l0503oP0000`000?l00?l0003o0002003o0080003o00<0oh000000o`00
0?l00P3oP0050?l00140oh0000<0o`000?n0003oP0003@3oP0030?l000l0oh0000<0o`000?n0003o
P0002@3oP006003o00<0003o0P3o00030?n00080o`003@3oP0020?l000D0oh006?ooool001Kooooo
0P3oP0280?l000kooooo0P000?l20?n000D0003o1`00o`090000o`@00?l04`000?l8003o00X0003o
0P00o`020000o`<00?l00P000?l2003o0080003o0P00o`000`000?l00?l0003o00040000o`800?l0
00<0oh000000o`000?l00P000?l20?n000<00?l00P000?l2003o0080003o0`00o`020000o`D00?l0
0P000?l2003o00@0003o0`3o0004003o0080003o00<00?l00000o`000?l00P00o`0>ooooo`L0003o
4000o`0?0000o`03003o00000?l0003o00@0003o0P00o`000`000?l00?l0003o000?0000o`D00?l0
0P3oP0000`000?l0oh000?n000020000o`@00?l000<0oh00003o0000o`000P00o`020000o`<00?l0
1P000?l20?n000030000o`3oP000oh000080003o2`00o`000`000?l00?l0003o00040000o`D0oh00
0P000?l2003o00L0003o3_ooool2003o01@0oh0000<0003o003o0000o`000P00o`020000o`030?n0
00000?l0003o0080oh001@3o000A0?n000030?l0003oP000oh0000d0oh000`3o000?0?n000030?l0
003oP000oh0000T0oh001P00o`030000o`80o`000`3oP0020?l000d0oh000P3o00050?n001Sooooo
000Fooooog@0o`000P3oP00D0?l000kooooo0P000?l2003o00D0003o2`00o`020000o`P00?l01000
0?l7003o00L0003o3@00o`0<0000o`800?l010000?l2003o00D0003o0P3o00030?n00080003o103o
P0020000o`030?n000000?l0003o0080003o0P00o`050000o`800?l00`000?l2003o00@0003o0P00
o`030000o`H00?l01@000?l>ooooo`L0003o1000o`020000o`L00?l00P000?l00`00o`00003o0000
o`090000o`800?l00`000?l2003o00@0003o0P00o`090000o`T00?l000<0003o003o0000o`000P00
0?l3003o0080003o2`3oP00;0000o`80oh001`000?l2003o00030000o`3oP000oh0000@00?l000<0
003o003o0000o`003`000?l3003o0080003o3_ooool2003o00L0oh000P3o00070?n000D00?l01P3o
P0000`3o0000003o0000o`02003o00D0o`002`3oP002003o01@0oh0000<00?l00000o`000?l00P00
o`040000o`D0oh000P3o00020000o`03003o00000?l0003o00@00?l01@000?l2003o0080o`000P00
o`0N0?n001Sooooo000Fooooog@0o`000P3oP00D0?l000kooooo0P000?l2003o00D0003o2`00o`02
0000o`P00?l010000?l7003o00L0003o3@00o`0<0000o`800?l010000?l2003o00D0003o0P3o0003
0?n00080003o103oP0020000o`030?n000000?l0003o0080003o0P00o`050000o`800?l00`000?l2
003o00@0003o0P00o`030000o`H00?l01@000?l>ooooo`L0003o1000o`020000o`L00?l00P000?l0
0`00o`00003o0000o`090000o`800?l00`000?l2003o00@0003o0P00o`090000o`T00?l000<0003o
003o0000o`000P000?l3003o0080003o2`3oP00;0000o`80oh001`000?l2003o00030000o`3oP000
oh0000@00?l000<0003o003o0000o`003`000?l3003o0080003o3_ooool2003o00L0oh000P3o0007
0?n000D00?l01P3oP0000`3o0000003o0000o`02003o00D0o`002`3oP002003o01@0oh0000<00?l0
0000o`000?l00P00o`040000o`D0oh000P3o00020000o`03003o00000?l0003o00@00?l01@000?l2
003o0080o`000P00o`0N0?n001Sooooo000Fooooo`80oh00K`3o00070?n00180o`003_ooool5003o
00@0003o5000o`000`000?l00?l0003o0002003o0080oh000P3o00030?n000L0003o4@00o`000`00
0?l00?l0003o00090000o`T00?l02@000?l2003o00D0003o2`00o`050000o`T00?l0103o0003003o
00@0003o3_ooool2003o0080003o1`00o`030000o`H00?l00P000?l00`00o`00003o0000o`070000
o`800?l010000?l00`3oP0000?l0003o00040?n00200003o0P3o00020?n000030?l00000o`000?l0
00800?l010000?l5003o00@0003o0`00o`040000o`h00?l00P3oP0030000o`H00?l01`000?l>oooo
o`T0oh000P3o00070?n00080o`002@3oP0000`00o`00003o0000o`02003o00L0oh00103o00020000
o`030?n0003o0000o`0000@0oh001@3o00060?n000030?l0003oP000oh0000D0oh000P00o`0H0?n0
00<00?l02@3oP002003o00<0o`005P3oP00Hooooo`005_oooon:0?l000kooooo0P00o`020000o`03
003o00000?l0003o0080003o1@00o`040000o`800?l00P000?l5003o00D0003o0P00o`040?n000<0
0?l00P000?l4003o00030000o`00o`000?l000H00?l000<0003o003o0000o`0010000?l3003o0080
003o0P00o`030000o`800?l010000?l2003o00030000o`00o`000?l00080003o1000o`000`000?l0
0?l0003o00020000o`800?l00`000?l4003o0080003o0P00o`0:0000o`800?l01@000?l2003o0080
oh001000o`050000o`kooooo0P00o`020000o`T00?l000<0003o003o0000o`001000o`070000o`80
0?l000<0003o003o0000o`000P00o`090000o`800?l040000?l2003o0080003o00<00?l00000o`00
0?l01000o`000`000?l0oh000?n000020?n00080003o0`3oP004003o0080003o1`00o`050000o`H0
0?l00P000?l7003o00030000o`00o`000?l00080003o0P3oP0020000o`030?n000000?l0003o00/0
003o3_ooool20?n000`0o`000P000?l20?l000800?l00P3oP0030?l000D0oh000P00o`0H0?n00003
0?l00000o`000?l00080oh001000o`080?n000800?l00P000?l40?n000D0o`00803oP0020000o`d0
oh0000<0o`000?n0003oP0000P00o`0Hooooo`005_oooon:0?l000kooooo0P00o`020000o`03003o
00000?l0003o0080003o1@00o`040000o`800?l00P000?l5003o00D0003o0P00o`040?n000<00?l0
0P000?l4003o00030000o`00o`000?l000H00?l000<0003o003o0000o`0010000?l3003o0080003o
0P00o`030000o`800?l010000?l2003o00030000o`00o`000?l00080003o1000o`000`000?l00?l0
003o00020000o`800?l00`000?l4003o0080003o0P00o`0:0000o`800?l01@000?l2003o0080oh00
1000o`050000o`kooooo0P00o`020000o`T00?l000<0003o003o0000o`001000o`070000o`800?l0
00<0003o003o0000o`000P00o`090000o`800?l040000?l2003o0080003o00<00?l00000o`000?l0
1000o`000`000?l0oh000?n000020?n00080003o0`3oP004003o0080003o1`00o`050000o`H00?l0
0P000?l7003o00030000o`00o`000?l00080003o0P3oP0020000o`030?n000000?l0003o00/0003o
3_ooool20?n000`0o`000P000?l20?l000800?l00P3oP0030?l000D0oh000P00o`0H0?n000030?l0
0000o`000?l00080oh001000o`080?n000800?l00P000?l40?n000D0o`00803oP0020000o`d0oh00
00<0o`000?n0003oP0000P00o`0Hooooo`005_oooon:0?l000kooooo0P00o`020000o`03003o0000
0?l0003o0080003o1000o`0:0000o`H00?l000<0003o003o0000o`000P000?l2003o00L0003o4000
o`020000o`T00?l01@000?l2003o0080003o1@00o`040000o`800?l04`000?l2003o00@0003o2@00
o`000`000?l00?l0003o00020000o`D00?l010000?l>ooooo`80003o0P3oP0000`000?l00?l0003o
0002003o0080003o0P00o`050000o`800?l00P000?l00`00o`00003o0000o`050000o`@00?l01000
0?l7003o0080003o00<00?l00000o`000?l01000o`030000o`800?l01P000?l00`00o`00003o0000
o`020000o`L00?l00P000?l7003o0080003o1`00o`020000o`800?l00`000?l2003o00/0003o00<0
0?l00000o`000?l01`00o`060000o`800?l000<0003o003o0000o`000P000?l>ooooo`@0o`0000<0
oh000?l0003o00000P3o00090?n000@0o`0000<0oh000?l0003o00001`3o0004003o00<0o`00103o
P0040?l000D00?l00P3o00030?n00080o`000P3oP0020?l000800?l00`3oP0020?l001T0oh001@3o
000?0?n00003003o00000?l0003o00d0oh001@3o00020?n001Sooooo000FooooohX0o`003_ooool2
003o0080003o00<00?l00000o`000?l00P000?l4003o00X0003o1P00o`000`000?l00?l0003o0002
0000o`800?l01`000?l@003o0080003o2@00o`050000o`800?l00P000?l5003o00@0003o0P00o`0C
0000o`800?l010000?l9003o00030000o`00o`000?l00080003o1@00o`040000o`kooooo0P000?l2
0?n000030000o`00o`000?l000800?l00P000?l2003o00D0003o0P00o`020000o`03003o00000?l0
003o00D0003o1000o`040000o`L00?l00P000?l00`00o`00003o0000o`04003o00<0003o0P00o`06
0000o`03003o00000?l0003o0080003o1`00o`020000o`L00?l00P000?l7003o0080003o0P00o`03
0000o`800?l02`000?l00`00o`00003o0000o`07003o00H0003o0P00o`000`000?l00?l0003o0002
0000o`kooooo103o00000`3oP000o`000?l000020?l000T0oh00103o00000`3oP000o`000?l00007
0?l000@00?l00`3o00040?n000@0o`001@00o`020?l000<0oh000P3o00020?n00080o`000P00o`03
0?n00080o`006@3oP0050?l000l0oh0000<00?l00000o`000?l03@3oP0050?l00080oh006?ooool0
01KoooooRP3o000>ooooo`/0003o0`00o`0=0000o`800?l01@000?l5003o00@0003o1000o`030000
o`800?l00P000?l<003o0080o`001000o`050000o`D0oh000P00o`020000o`L00?l00P000?l4003o
00030000o`00o`000?l000@0003o1@00o`020000o`800?l00`000?l4003o00L0oh00103o0003003o
0080003o3_ooool20000o`L00?l00P000?l2003o00030000o`00o`000?l000800?l010000?l3003o
00L0003o1000o`0;0000o`D00?l01@000?l2003o00T0003o0P00o`020000o`<00?l01P000?l3003o
00@0003o1`00o`020000o`<00?l00P000?l2003o0080003o0P00o`000`000?l00?l0003o000=0000
o`030?n000000?l0003o00800?l02@000?l>ooooo`@0oh000`3o00020?n000800?l00P000?l00`3o
P000003o0000o`02003o0280oh000P3o00020?n000030?l0003oP000oh0000L0o`00103oP0030?l0
0080oh000P000?l20?n00080003o8P3oP0000`00o`00003o0000o`040000o`80o`001@3oP0020?l0
00L0oh006?ooool001KoooooRP3o000>ooooo`/0003o0`00o`0=0000o`800?l01@000?l5003o00@0
003o1000o`030000o`800?l00P000?l<003o0080o`001000o`050000o`D0oh000P00o`020000o`L0
0?l00P000?l4003o00030000o`00o`000?l000@0003o1@00o`020000o`800?l00`000?l4003o00L0
oh00103o0003003o0080003o3_ooool20000o`L00?l00P000?l2003o00030000o`00o`000?l00080
0?l010000?l3003o00L0003o1000o`0;0000o`D00?l01@000?l2003o00T0003o0P00o`020000o`<0
0?l01P000?l3003o00@0003o1`00o`020000o`<00?l00P000?l2003o0080003o0P00o`000`000?l0
0?l0003o000=0000o`030?n000000?l0003o00800?l02@000?l>ooooo`@0oh000`3o00020?n00080
0?l00P000?l00`3oP000003o0000o`02003o0280oh000P3o00020?n000030?l0003oP000oh0000L0
o`00103oP0030?l00080oh000P000?l20?n00080003o8P3oP0000`00o`00003o0000o`040000o`80
o`001@3oP0020?l000L0oh006?ooool001KoooooRP3o000>ooooo`800?l00P000?l00`00o`00003o
0000o`020000o`800?l00`000?l4003o0080003o0P00o`050000o`800?l000<0003o003o0000o`00
2@000?l7003o00P0003o1@3oP002003o0080o`0000<0003o0?l0003o00001000o`030000o`800?l0
103oP003003o00@0o`002@00o`020000o`03003o00000?l0003o00L0003o1`00o`020000o`800?l0
0P000?l7003o00D0o`000P00o`020000o`kooooo10000?l>003o00D0003o3`00o`030000o`@00?l0
0P000?l00`00o`00003o0000o`040000o`800?l000<0003o003o0000o`001000o`020000o`030?n0
0000o`000?l00080oh000P000?l7003o00<0003o1000o`040000o`03003o00000?l0003o0080003o
0P3oP00T0000o`030?n00000o`000?l00080003o3_ooool=0?n000<00?l0403oP0020?l000T0oh00
1@3o0002003o00l0oh0000<0o`000?n0003oP0000P3oP0020?l000030?n0003o0000o`0001T0oh00
0P3o00020?n000T0o`000`3oP0020?l000@0oh000P3o000>0?n001Sooooo000FooooohX0o`003_oo
ool2003o0080003o00<00?l00000o`000?l00P000?l2003o00<0003o1000o`020000o`800?l01@00
0?l2003o00030000o`00o`000?l000T0003o1`00o`080000o`D0oh000P00o`020?l000030000o`3o
0000o`0000@00?l00`000?l2003o00@0oh000`00o`040?l000T00?l00P000?l00`00o`00003o0000
o`070000o`L00?l00P000?l2003o0080003o1`00o`050?l000800?l00P000?l>ooooo`@0003o3P00
o`050000o`l00?l00`000?l4003o0080003o00<00?l00000o`000?l010000?l2003o00030000o`00
o`000?l000@00?l00P000?l00`3oP0000?l0003o00020?n00080003o1`00o`030000o`@00?l01000
0?l00`00o`00003o0000o`020000o`80oh0090000?l00`3oP0000?l0003o00020000o`kooooo3@3o
P003003o0100oh000P3o00090?n000D0o`000P00o`0?0?n000030?l0003oP000oh000080oh000P3o
00000`3oP000o`000?l0000I0?n00080o`000P3oP0090?l000<0oh000P3o00040?n00080o`003P3o
P00Hooooo`005_oooon:0?l000kooooo1@00o`020000o`/00?l01`000?l4003o00030000o`00o`00
0?l000800?l00P000?l3003o0080003o1000o`0;0000o`D0oh000P00o`020?n000030?l0003oP000
oh000080oh000P00o`050000o`80oh000P3o00000`3oP0000?l0003o0002003o0080o`000`00o`02
0000o`L00?l010000?l5003o00@0003o0P00o`030000o`800?l00P000?l5003o00/0003o3_ooool4
0000o`X00?l00P000?l4003o0080003o00<00?l00000o`000?l03P00o`020000o`80oh000P000?l3
0?n000@0003o0P3oP0030000o`P00?l020000?l2003o0080003o0P00o`030000o`H00?l00P000?l0
0`00o`00003o0000o`0?0000o`03003o00000?l0003o0080003o0`00o`020000o`800?l00P000?l2
003o00X0003o0P00o`020000o`kooooo2`3oP0020?l000<0oh000P000?l40?n000<00?l00P000?l3
0?l00180oh001P3o00030?n00080o`000P3oP0020?l000800?l00`000?l50?n000@00?l00P3oP002
003o00D0oh000P3o00050?n000800?l00P3oP0020?l000T0oh001@3o000I0?n001Sooooo000Foooo
ohX0o`003_ooool5003o0080003o1000o`030000o`@00?l01@000?l20?n000@0003o0`00o`0;0000
o`D00?l00P000?l2003o00@0003o00<00?l00000o`000?l00P000?l70?l00080003o0P00o`000`00
0?l00?l0003o0002003o00@0o`0000<0oh000000o`000?l0103o00020000o`030?n000000?l0003o
00@0oh000`000?l2003o0080003o1@00o`040000o`80oh002@000?l5003o0080003o0`00o`020000
o`800?l03_ooool20000oa800?l00P000?l7003o00030000o`00o`000?l000H00?l000<0003o003o
0000o`000P00o`090000o`T00?l00P000?l2003o00030000o`00o`000?l000@0003o00<00?l00000
o`000?l010000?l5003o00@0003o1@00o`0@0000o`800?l010000?l00`00o`00003o0000o`04003o
00<0003o0P00o`070000o`800?l00P000?l>ooooo`D0o`00103oP0070?l000d0oh000`3o0002003o
0080003o2`3oP0000`3o0000oh000?n000020?l00080oh000P00o`000`3oP000o`000?l000070?n0
0080003o0P3oP0020?l000030?n0003o0000o`0000@0oh000P00o`000`3oP000o`000?l000040?l0
00<0oh000P3o000G0?n000T0o`002@3oP0020?l00080oh006?ooool001KoooooRP3o000>ooooo`D0
0?l00P000?l4003o00<0003o1000o`050000o`80oh0010000?l3003o00/0003o1@00o`020000o`80
0?l010000?l00`00o`00003o0000o`020000o`L0o`000P000?l2003o00030000o`00o`000?l00080
0?l0103o00000`3oP000003o0000o`040?l00080003o00<0oh000000o`000?l0103oP0030000o`80
0?l00P000?l5003o00@0003o0P3oP0090000o`D00?l00P000?l3003o0080003o0P00o`0>ooooo`80
003o4P00o`020000o`L00?l000<0003o003o0000o`001P00o`000`000?l00?l0003o0002003o00T0
003o2@00o`020000o`800?l000<0003o003o0000o`0010000?l00`00o`00003o0000o`040000o`D0
0?l010000?l5003o0100003o0P00o`040000o`03003o00000?l0003o00@00?l00`000?l2003o00L0
003o0P00o`020000o`kooooo1@3o00040?n000L0o`003@3oP0030?l000800?l00P000?l;0?n00003
0?l0003oP000oh000080o`000P3oP002003o00030?n0003o0000o`0000L0oh000P000?l20?n00080
o`0000<0oh000?l0003o0000103oP002003o00030?n0003o0000o`0000@0o`000`3oP0020?l001L0
oh002@3o00090?n00080o`000P3oP00Hooooo`005_oooon:0?l000kooooo0P000?l2003o00<0003o
0P00o`020000o`T00?l02P000?l4003o0080003o0`3oP0020000o`L00?l00P000?l4003o00<0003o
1000o`020?l000<00?l00P000?l4003o00<0003o0P3oP0020?l000800?l000<0oh000000o`000?l0
103oP002003o00030?n00000o`000?l000@0oh001@00o`040000o`X00?l00P000?l4003o00<0003o
2@00o`040000o`kooooo0P000?lI003o0080003o2@00o`050000o`800?l00P000?l>003o0180003o
0`00o`080000o`03003o00000?l0003o0080003o0P00o`020000o`03003o00000?l0003o0080003o
1`00o`020000o`L00?l02@000?l2003o0080003o00<00?l00000o`000?l00P00o`0>ooooo`L0o`00
6`3oP002003o0080o`001@3oP0050?l000@0003o103oP0030?l000h0oh00103o0002003o0080o`00
5@3oP0020?l00080oh000`3o00040?n00080o`001@3oP0020?l000/0oh0000<0003o0?n0003oP000
0P3oP00Hooooo`005_oooon:0?l000kooooo0P000?l2003o00<0003o0P00o`020000o`T00?l02P00
0?l4003o0080003o0`3oP0020000o`L00?l00P000?l4003o00<0003o1000o`020?l000<00?l00P00
0?l4003o00<0003o0P3oP0020?l000800?l000<0oh000000o`000?l0103oP002003o00030?n00000
o`000?l000@0oh001@00o`040000o`X00?l00P000?l4003o00<0003o2@00o`040000o`kooooo0P00
0?lI003o0080003o2@00o`050000o`800?l00P000?l>003o0180003o0`00o`080000o`03003o0000
0?l0003o0080003o0P00o`020000o`03003o00000?l0003o0080003o1`00o`020000o`L00?l02@00
0?l2003o0080003o00<00?l00000o`000?l00P00o`0>ooooo`L0o`006`3oP002003o0080o`001@3o
P0050?l000@0003o103oP0030?l000h0oh00103o0002003o0080o`005@3oP0020?l00080oh000`3o
00040?n00080o`001@3oP0020?l000/0oh0000<0003o0?n0003oP0000P3oP00Hooooo`005_oooon:
0?l000kooooo0P00o`050000o`800?l010000?l7003o00L0003o0`00o`020000oa000?l00P000?l2
003o00@0003o00<00?l00000o`000?l01`3oP0020000o`@00?l000<0003o003o0000o`00103o0002
003o00030000o`00o`000?l000@00?l00`000?l6003o0080003o0`00o`040000o`800?l000<0003o
003o0000o`000P000?l3003o00l0003o0P00o`030000o`800?l03_ooool40000o`h00?l00P000?l2
003o00P0003o1P00o`020000o`03003o00000?l0003o00L0003o1000o`070000o`@00?l00`000?l2
003o0080003o0`00o`060000o`800?l03P000?l2003o00D0003o1000o`090000o`03003o00000?l0
003o00800?l03@000?l5003o00kooooo0P3oP0030?l00080oh001P3o00090?n000X0o`000P3oP002
0000o`80oh0000<0003o0?n0003oP0001P3oP0050?l000D0oh001P3o0002003o00D0oh000P3o0000
0`3oP000o`000?l000070?n000H0o`002@3oP0030?l002H0oh006?ooool001KoooooRP3o000>oooo
o`800?l01@000?l2003o00@0003o1`00o`070000o`<00?l00P000?l@003o0080003o0P00o`040000
o`03003o00000?l0003o00L0oh000P000?l4003o00030000o`00o`000?l000@0o`000P00o`000`00
0?l00?l0003o0004003o00<0003o1P00o`020000o`<00?l010000?l2003o00030000o`00o`000?l0
0080003o0`00o`0?0000o`800?l00`000?l2003o00kooooo10000?l>003o0080003o0P00o`080000
o`H00?l00P000?l00`00o`00003o0000o`070000o`@00?l01`000?l4003o00<0003o0P00o`020000
o`<00?l01P000?l2003o00h0003o0P00o`050000o`@00?l02@000?l00`00o`00003o0000o`02003o
00d0003o1@00o`0>ooooo`80oh000`3o00020?n000H0o`002@3oP00:0?l00080oh000P000?l20?n0
00030000o`3oP000oh0000H0oh001@3o00050?n000H0o`000P00o`050?n00080o`0000<0oh000?l0
003o00001`3oP0060?l000T0oh000`3o000V0?n001Sooooo000FooooohX0o`003_ooool2003o00T0
003o0P00o`000`000?l00?l0003o0006003o00D0003o0P00o`000`000?l00?l0003o00040000o`80
0?l00`000?l;003o00D0003o2`00o`020000o`<00?l02@000?l2003o0080003o0P00o`050000o`H0
0?l000<0003o003o0000o`000P00o`020000o`800?l000<0003o003o0000o`001000o`030000o`@0
0?l01@000?l7003o0080003o0P3oP00>ooooo`@0003o3P00o`020000o`800?l000<0003o003o0000
o`001@000?l=003o00L0003o0P00o`040000o`03003o00000?l0003o00T0003o1`00o`020000o`80
0?l00P000?l00`00o`00003o0000o`070000o`80oh000P000?l4003o00030000o`00o`000?l000T0
0?l010000?l@003o00L0003o3_ooool=0?n000030?l0003oP000oh0000@0o`001@3oP0020?l000<0
oh00103o000;0?n00080o`00303oP002003o0080oh000P00o`0S0?n000800?l0403oP0040?l000T0
oh000P00o`000`000?l0oh000?n000020?n001Sooooo000FooooohX0o`003_ooool2003o00T0003o
0P00o`000`000?l00?l0003o0006003o00D0003o0P00o`000`000?l00?l0003o00040000o`800?l0
0`000?l;003o00D0003o2`00o`020000o`<00?l02@000?l2003o0080003o0P00o`050000o`H00?l0
00<0003o003o0000o`000P00o`020000o`800?l000<0003o003o0000o`001000o`030000o`@00?l0
1@000?l7003o0080003o0P3oP00>ooooo`@0003o3P00o`020000o`800?l000<0003o003o0000o`00
1@000?l=003o00L0003o0P00o`040000o`03003o00000?l0003o00T0003o1`00o`020000o`800?l0
0P000?l00`00o`00003o0000o`070000o`80oh000P000?l4003o00030000o`00o`000?l000T00?l0
10000?l@003o00L0003o3_ooool=0?n000030?l0003oP000oh0000@0o`001@3oP0020?l000<0oh00
103o000;0?n00080o`00303oP002003o0080oh000P00o`0S0?n000800?l0403oP0040?l000T0oh00
0P00o`000`000?l0oh000?n000020?n001Sooooo000Fooooo`80oh00R03o000>ooooo`T00?l00P00
0?l7003o0080003o0P00o`000`000?l00?l0003o0005003o00H0003o0P00o`000`000?l00?l0003o
000;003o00@0003o00<00?l00000o`000?l010000?l2003o00030000o`3oP000oh000080oh000P00
o`030000o`800?l01@000?l8003o00D0003o0P00o`050000o`800?l00P000?l2003o00D0003o0P3o
P0050000o`@00?l000<0003o003o0000o`000P00o`020?l000L00?l03_ooool4003o00L0003o0`00
o`020000o`800?l02`000?l:003o00H0oh002`000?l00`00o`00003o0000o`07003o00@0003o0P00
o`000`000?l00?l0003o0007003o02d0003o0P00o`070000o`800?l03_ooool70?n000H0o`004@3o
P0020?l000@0oh000P00o`000`000?l00?l0003o0002003o0080o`00403oP0020?l00080003o00<0
o`000000o`000?l00P000?l2003o00030?n000000?l0003o00/0oh000P00o`020000o`T0oh0000<0
o`000?n0003oP000903oP00Hooooo`005_oooom10?l000030?n0003o0000o`0004H0o`003_ooool=
003o00<0003o3P00o`080000o`h00?l01@000?l2003o0080003o1@00o`020000o`@00?l000<0003o
003o0000o`001000o`030000o`@00?l00P000?l9003o0080oh000`000?l9003o0080003o0P00o`05
0000o`800?l01`000?l5003o0080003o0P00o`0>ooooo`80003o0P00o`070000o`<00?l00P000?l2
003o00D0003o0P00o`040000o`03003o00000?l0003o00800?l01`000?l4003o01L0003o0P00o`02
0000o`030?n00000o`000?l000800?l00P000?l2003o00030?n000000?l0003o00T00?l010000?l2
003o00D0003o0P3oP0020000o`<00?l00P000?l3003o00@0003o2@00o`020000o`<00?l010000?l>
oooooa80oh000P3o00020?n000030000o`3oP000oh0000L0o`00103oP002003o00030?n00000o`00
0?l00080003o4P3oP0020?l00180oh000`3o0006003o0300oh006?ooool001Kooooo@@3o00000`3o
P000o`000?l000160?l000kooooo3@00o`030000o`h00?l020000?l>003o00D0003o0P00o`020000
o`D00?l00P000?l4003o00030000o`00o`000?l000@00?l00`000?l4003o0080003o2@00o`020?n0
00<0003o2@00o`020000o`800?l01@000?l2003o00L0003o1@00o`020000o`800?l03_ooool20000
o`800?l01`000?l3003o0080003o0P00o`050000o`800?l010000?l00`00o`00003o0000o`02003o
00L0003o1000o`0G0000o`800?l00P000?l00`3oP0000?l0003o0002003o0080003o0P00o`000`3o
P000003o0000o`09003o00@0003o0P00o`050000o`80oh000P000?l3003o0080003o0`00o`040000
o`T00?l00P000?l3003o00@0003o3_oooolB0?n00080o`000P3oP0000`000?l0oh000?n000070?l0
00@0oh000P00o`000`3oP0000?l0003o00020000oa80oh000P3o000B0?n000<0o`001P00o`0`0?n0
01Sooooo000FooooohX0o`003_ooool5003o0080003o1`00o`040000o`h00?l01P000?l:003o0080
003o0P00o`0;0000o`<00?l00P000?l2003o00<0003o0P00o`020000o`D00?l00P000?l4003o0080
003o1`00o`090000o`D00?l00P000?l2003o00T0003o00<00?l00000o`000?l02@00o`020000o`ko
oooo1@00o`040000o`D00?l020000?l5003o00D0003o0P00o`020000o`<00?l00P000?l40?n00080
0?l00`000?l4003o00L0003o1@00o`090000o`80o`000P000?l30?n000H00?l00`000?l2003o0080
003o0P00o`050000o`800?l010000?l30?n000@0003o00<00?l00000o`000?l00P000?l4003o00D0
oh001`000?l2003o00kooooo2`3oP0030?l000T0oh000P000?l;0?n000<0003o0P00o`0B0?n00080
o`006@3oP0020?l000<0oh000P3o00020?n000@0o`00803oP0020?l000D0oh006?ooool001Kooooo
RP3o000>ooooo`D00?l00P000?l7003o00@0003o3P00o`060000o`X00?l00P000?l2003o00/0003o
0`00o`020000o`800?l00`000?l2003o0080003o1@00o`020000o`@00?l00P000?l7003o00T0003o
1@00o`020000o`800?l02@000?l00`00o`00003o0000o`09003o0080003o3_ooool5003o00@0003o
1@00o`080000o`D00?l01@000?l2003o0080003o0`00o`020000o`@0oh000P00o`030000o`@00?l0
1`000?l5003o00T0003o0P3o00020000o`<0oh001P00o`030000o`800?l00P000?l2003o00D0003o
0P00o`040000o`<0oh0010000?l00`00o`00003o0000o`020000o`@00?l01@3oP0070000o`800?l0
3_ooool;0?n000<0o`002@3oP0020000o`/0oh000`000?l2003o0180oh000P3o000I0?n00080o`00
0`3oP0020?l00080oh00103o000P0?n00080o`001@3oP00Hooooo`005_oooon:0?l000kooooo0P00
o`050000o`H00?l000<0003o003o0000o`004000o`020000o`800?l00P000?l:003o00T0003o2000
o`030000o`@00?l00`000?l6003o00X0003o0P00o`020?l000@00?l01@000?l2003o00L0003o0P00
o`000`000?l00?l0003o00070000o`800?l00P000?l50?n000@0003o3_ooool70000o`L00?l02@00
0?l2003o00L0003o0P00o`070000o`800?l00P3oP002003o00D0003o0P00o`030000o`P00?l00`00
0?l2003o0080003o00<0oh00003o0000o`000P3oP0020000o`800?l000<0003o003o0000o`000P00
0?l2003o00L0003o0P00o`050000o`800?l03P000?l2003o0080003o1`00o`020000o`800?l000<0
oh000000o`000?l00P00o`0>ooooo`D0oh00103o00020?n000<0o`00603oP0000`000?l0oh000?n0
00060?n00003003o003oP000oh000080oh000P00o`050?l00080oh000P000?l2003o00P0oh001000
o`040000o`T0oh0000<0003o0?n0003oP000103oP0030?l00080oh000P3o00020000oah0oh000P3o
000Hooooo`005_oooon:0?l000kooooo0P00o`050000o`H00?l000<0003o003o0000o`004000o`02
0000o`800?l00P000?l:003o00T0003o2000o`030000o`@00?l00`000?l6003o00X0003o0P00o`02
0?l000@00?l01@000?l2003o00L0003o0P00o`000`000?l00?l0003o00070000o`800?l00P000?l5
0?n000@0003o3_ooool70000o`L00?l02@000?l2003o00L0003o0P00o`070000o`800?l00P3oP002
003o00D0003o0P00o`030000o`P00?l00`000?l2003o0080003o00<0oh00003o0000o`000P3oP002
0000o`800?l000<0003o003o0000o`000P000?l2003o00L0003o0P00o`050000o`800?l03P000?l2
003o0080003o1`00o`020000o`800?l000<0oh000000o`000?l00P00o`0>ooooo`D0oh00103o0002
0?n000<0o`00603oP0000`000?l0oh000?n000060?n00003003o003oP000oh000080oh000P00o`05
0?l00080oh000P000?l2003o00P0oh001000o`040000o`T0oh0000<0003o0?n0003oP000103oP003
0?l00080oh000P3o00020000oah0oh000P3o000Hooooo`005_oooon:0?l000kooooo1000o`000`00
0?l00?l0003o0002003o00T0003o3P00o`060000o``00?l00P000?l2003o00L0003o0P00o`020?n0
00D00?l00`000?l2003o0080003o1000o`050000o`800?l00`000?l4003o00@0003o0`00o`040000
o`800?l01@000?l5003o0080003o0P00o`070000o`D00?l00P000?l2003o00kooooo10000?l00`00
o`00003o0000o`04003o0080003o1@00o`0B0000o`800?l000<0003o003o0000o`000P3oP0020000
o`800?l000<0oh000000o`000?l00P000?l7003o00/0003o1@00o`040000o`800?l01@3o0004003o
00030000o`00o`000?l000T00?l00P3oP0020000o`80oh0000<0003o003o0000o`000P00o`020000
o`03003o00000?l0003o0080003o0P00o`030000o`800?l00P3oP0020000o`<00?l010000?l>oooo
o`@00?l000<0o`000?n0003oP000103oP0050?l00100oh000P00o`050?n00080o`000P3oP0040?l0
00D0oh000P000?lN0?n000800?l00P3oP002003o00D0oh000P3o000I0?n00080o`0000<0003o0?l0
003o000010000?l20?l000D0oh006?ooool001Kooooo0P3oP0280?l000kooooo10000?l00`00o`00
003o0000o`020000o`@00?l00`000?l>003o0080003o0P00o`020000oa000?l00P000?l5003o0080
003o1@00o`020000o`/00?l00P3oP0000`00o`00003o0000o`0;0000oaL00?l00P000?l2003o00<0
003o1000o`020000o`80oh0000<00?l00000o`000?l00P00o`0>ooooo`800?l01`000?l2003o00T0
003o0`00o`040000o`<0oh0010000?l2003o00L0003o1000o`000`000?l00?l0003o00090000o`H0
0?l00`000?l2003o00<0003o0P00o`020000o`@00?l000<0oh00003o0000o`001000o`0>0000o`80
0?l050000?l30?n000800?l01@000?l4003o00kooooo1`3oP00?0?l000<00?l0403oP0070?l000T0
oh000P3o00090?n00080o`000`00o`090?n00080003o4P3oP0020?l00280oh006?ooool001Kooooo
0P3oP0280?l000kooooo10000?l00`00o`00003o0000o`020000o`@00?l00`000?l>003o0080003o
0P00o`020000oa000?l00P000?l5003o0080003o1@00o`020000o`/00?l00P3oP0000`00o`00003o
0000o`0;0000oaL00?l00P000?l2003o00<0003o1000o`020000o`80oh0000<00?l00000o`000?l0
0P00o`0>ooooo`800?l01`000?l2003o00T0003o0`00o`040000o`<0oh0010000?l2003o00L0003o
1000o`000`000?l00?l0003o00090000o`H00?l00`000?l2003o00<0003o0P00o`020000o`@00?l0
00<0oh00003o0000o`001000o`0>0000o`800?l050000?l30?n000800?l01@000?l4003o00kooooo
1`3oP00?0?l000<00?l0403oP0070?l000T0oh000P3o00090?n00080o`000`00o`090?n00080003o
4P3oP0020?l00280oh006?ooool001Kooooo0P3oP0280?l000kooooo6@00o`020000o`L00?l01000
0?l:003o0080003o1000o`020000o`03003o00000?l0003o00800?l00P000?l3003o0080003o0P00
o`090000o`800?l000<0003o003o0000o`001P000?l00`00o`00003o0000o`070000oa800?l00P00
0?l5003o0080003o0P00o`050000o`@00?l03_oooolN0000o`800?l00P000?l4003o00030000o`00
o`000?l000800?l00P000?l2003o00030?n00000o`000?l000T0003o1P00o`050000o`<00?l00P00
0?l4003o00D0003o1`00o`040000o`800?l01@000?l2003o00@0003o3P00o`030000o`800?l00P00
0?l20?n000<0003o1000o`0>oooooa@0oh000P3o0003003o0080o`00903oP002003o00h0oh001@3o
00020?n00080003o8P3oP0030000o`800?l02`3oP00Hooooo`005_ooool20?n008P0o`003_oooolI
003o0080003o1`00o`040000o`X00?l00P000?l4003o0080003o00<00?l00000o`000?l00P00o`02
0000o`<00?l00P000?l2003o00T0003o0P00o`000`000?l00?l0003o00060000o`03003o00000?l0
003o00L0003o4P00o`020000o`D00?l00P000?l2003o00D0003o1000o`0>oooooah0003o0P00o`02
0000o`@00?l000<0003o003o0000o`000P00o`020000o`800?l000<0oh00003o0000o`002@000?l6
003o00D0003o0`00o`020000o`@00?l01@000?l7003o00@0003o0P00o`050000o`800?l010000?l>
003o00<0003o0P00o`020000o`80oh000`000?l4003o00kooooo503oP0020?l000<00?l00P3o000T
0?n000800?l03P3oP0050?l00080oh000P000?lR0?n000<0003o0P00o`0;0?n001Sooooo000Foooo
ohX0o`003_ooool4003o00D0003o1`00o`020000o`L00?l010000?l00`00o`00003o0000o`060000
o`X00?l01P000?l2003o00<0003o0P00o`020000oaD00?l03@000?l3003o0080003o4000o`0=0000
o`L00?l03_ooool70000o`@00?l04`000?l9003o00@0003o2@00o`0;0000o`800?l04`000?l20?n0
00@0003o00<00?l00000o`000?l02`000?l2003o0080003o00<00?l00000o`000?l00P00o`020?n0
00<00?l03`000?l00`00o`00003o0000o`020000o`kooooo0P000?l50?l00080oh00103o00090?n0
00030?l00000o`000?l00080003o0`3oP0020000o`T0oh001P3o000@0?n00080o`0000<00?l00000
o`000?l0103oP0000`00o`00o`000?l000060?l000<0oh001P000?l00`3o0000oh000?n000060?n0
00X0o`003`3oP0000`00o`00003o0000o`090?n00080o`006?ooool001KoooooRP3o000>ooooo`@0
0?l01@000?l7003o0080003o1`00o`040000o`03003o00000?l0003o00H0003o2P00o`060000o`80
0?l00`000?l2003o0080003o5@00o`0=0000o`<00?l00P000?l@003o00d0003o1`00o`0>ooooo`L0
003o1000o`0C0000o`T00?l010000?l9003o00/0003o0P00o`0C0000o`80oh0010000?l00`00o`00
003o0000o`0;0000o`800?l00P000?l00`00o`00003o0000o`02003o0080oh000`00o`0?0000o`03
003o00000?l0003o0080003o3_ooool20000o`D0o`000P3oP0040?l000T0oh0000<0o`00003o0000
o`000P000?l30?n00080003o2@3oP0060?l00100oh000P3o00000`00o`00003o0000o`040?n00003
003o003o0000o`0000H0o`000`3oP0060000o`030?l0003oP000oh0000H0oh002P3o000?0?n00003
003o00000?l0003o00T0oh000P3o000Hooooo`005_oooon:0?l000kooooo4P000?l2003o0080003o
00<00?l00000o`000?l010000?l5003o0080003o2`00o`000`000?l00?l0003o0002003o0080003o
1`00o`020000o`800?l000<0oh000000o`000?l02@000?l50?n000@00?l010000?l00`00o`00003o
0000o`02003o00T0003o2@00o`020000o`03003o00000?l0003o00H0003o00<0oh000000o`000?l0
1`000?l4003o00kooooo0P000?l2003o00<0003o1P00o`050000o`800?l01`000?l2003o00030000
o`00o`000?l000L00?l010000?l5003o00/0003o1`00o`020000o`@00?l00`000?l2003o00T0003o
1@00o`020000o`800?l05@000?l6003o0080003o00<00?l00000o`000?l01000o`070000o`kooooo
2`3oP0020?l00003003o003o0000o`0000L0oh000P00o`020?n000D0o`004P3oP0040?l000D0oh00
103o00050?n000@0o`00303oP0040?l00080oh002@3o00000`3oP000o`000?l000040?l000<0oh00
103o00090?n000800?l03P3oP00Hooooo`005_oooon:0?l000kooooo4P000?l2003o0080003o00<0
0?l00000o`000?l010000?l5003o0080003o2`00o`000`000?l00?l0003o0002003o0080003o1`00
o`020000o`800?l000<0oh000000o`000?l02@000?l50?n000@00?l010000?l00`00o`00003o0000
o`02003o00T0003o2@00o`020000o`03003o00000?l0003o00H0003o00<0oh000000o`000?l01`00
0?l4003o00kooooo0P000?l2003o00<0003o1P00o`050000o`800?l01`000?l2003o00030000o`00
o`000?l000L00?l010000?l5003o00/0003o1`00o`020000o`@00?l00`000?l2003o00T0003o1@00
o`020000o`800?l05@000?l6003o0080003o00<00?l00000o`000?l01000o`070000o`kooooo2`3o
P0020?l00003003o003o0000o`0000L0oh000P00o`020?n000D0o`004P3oP0040?l000D0oh00103o
00050?n000@0o`00303oP0040?l00080oh002@3o00000`3oP000o`000?l000040?l000<0oh00103o
00090?n000800?l03P3oP00Hooooo`005_oooolK0?l00080oh00K@3o000>ooooo`D00?l02`000?l7
003o00T0003o1P00o`000`000?l00?l0003o0006003o00L0003o0P00o`000`000?l00?l0003o0002
0000o`80oh000P00o`000`000?l00?l0003o0002003o0080003o00<00?l00000o`000?l01P000?lC
003o00@0003o1`00o`020000o`T00?l00P000?l00`00o`00003o0000o`060?n000<0003o0P00o`0>
oooooa000?l00P000?l2003o0080003o4`00o`060000o`03003o00000?l0003o00L0003o2000o`00
0`000?l00?l0003o0002003o00L0003o0P00o`050000o`800?l010000?l7003o00<0003o0P00o`0I
0000o`800?l010000?l00`00o`00003o0000o`020000o`kooooo1@3oP0020000o`@0oh000P3o0003
0?n000H0003o00<0oh00003o0000o`00103oP003003o0080003o2@3oP0040?l001L0oh000P00o`00
0`000?l0oh000?n0000;0?n00080o`001`3oP0040?l000L0oh000`3o00070?n000@0o`001`00o`04
0?l000L0oh006?ooool001KoooooRP3o000>ooooo`80003o0P00o`000`000?l00?l0003o0004003o
00D0003o3P00o`040000o`800?l00P000?l<003o00H0003o1`00o`020000o`<00?l010000?l:003o
0080003o1P00o`000`000?l00?l0003o0009003o00@0003o1@00o`020000o`<00?l00P000?l2003o
0080003o1@00o`020000o`800?l01`000?l>ooooo`D0003o3`00o`030000oa800?l02@000?l6003o
00030000o`00o`000?l000/00?l00P000?l00`00o`00003o0000o`020000o`T00?l00P000?l>003o
00H0003o0`00o`070000o`@00?l01@000?l2003o00T0003o3_ooool40000o`T0oh0000<00?l00000
o`000?l00P3oP0040000o`030?l000000?l0003o00@0o`000`3oP0020?l00080oh002@3o00050?n0
0080o`00103oP0000`00o`00oh000?n0000M0?n00080003o1@3o00020?n000D0o`003P3oP0020?l0
0080oh000P3o00030?n000@0003o1`3oP0020?l001Sooooo000FooooohX0o`003_ooool20000o`80
0?l000<0003o003o0000o`001000o`050000o`h00?l010000?l2003o0080003o3000o`060000o`L0
0?l00P000?l3003o00@0003o2P00o`020000o`H00?l000<0003o003o0000o`002@00o`040000o`D0
0?l00P000?l3003o0080003o0P00o`020000o`D00?l00P000?l2003o00L0003o3_ooool50000o`l0
0?l00`000?lB003o00T0003o1P00o`000`000?l00?l0003o000;003o0080003o00<00?l00000o`00
0?l00P000?l9003o0080003o3P00o`060000o`<00?l01`000?l4003o00D0003o0P00o`090000o`ko
oooo10000?l90?n00003003o00000?l0003o0080oh0010000?l00`3o0000003o0000o`040?l000<0
oh000P3o00020?n000T0o`001@3oP0020?l000@0oh0000<00?l00?n0003oP0007@3oP0020000o`D0
o`000P3oP0050?l000h0oh000P3o00020?n00080o`000`3oP0040000o`L0oh000P3o000Hooooo`00
5_oooon:0?l000kooooo0P00o`070000o`800?l01@000?l@003o00@0003o3000o`080000o`03003o
00000?l0003o0080003o1000o`000`3oP0000?l0003o00020000o`T00?l01`000?l2003o00T0003o
1@00o`020000o``00?l01P000?l7003o00@0003o00<00?l00000o`000?l00P000?l>ooooo`800?l0
0`000?l?003o0080003o0`00o`020000oa000?l02@000?lB003o0080003o1`00o`050000o`800?l0
10000?l:003o0080003o0P00o`050000o`800?l010000?l00`00o`00003o0000o`07003o00/0003o
0P00o`0>oooooaT0oh001@3o000J0?n000<00?l05`3oP0020000o`@0oh001@3o00020?n000D0o`00
103oP0040?l001L0oh001@3o00020?n001Sooooo000FooooohX0o`003_ooool2003o00L0003o0P00
o`050000oa000?l010000?l<003o00P0003o00<00?l00000o`000?l00P000?l4003o00030?n00000
o`000?l00080003o2@00o`070000o`800?l02@000?l5003o0080003o3000o`060000o`L00?l01000
0?l00`00o`00003o0000o`020000o`kooooo0P00o`030000o`l00?l00P000?l3003o0080003o4000
o`090000oa800?l00P000?l7003o00D0003o0P00o`040000o`X00?l00P000?l2003o00D0003o0P00
o`040000o`03003o00000?l0003o00L00?l02`000?l2003o00kooooo6@3oP0050?l001X0oh000`00
o`0G0?n00080003o103oP0050?l00080oh001@3o00040?n000@0o`005`3oP0050?l00080oh006?oo
ool001KoooooO@3o00020?n000T0o`000P000?l>ooooo`@00?l01@000?l5003o0080003o3@00o`00
0`000?l00?l0003o00040000oa000?l01@000?l;003o00T0003o0P00o`0<0000o`800?l02`000?l7
003o0080003o0`00o`020000o`@00?l00`000?l2003o00L0003o0P00o`020000o`kooooo0P00o`02
0000oaD00?l01`000?l7003o00d0003o4P00o`020000o`T00?l02P000?l4003o0080003o0P00o`00
0`000?l00?l0003o0002003o0080003o3P00o`040000o`03003o00000?l0003o00800?l00P000?l2
003o00<0003o0P00o`0>ooooo`D0oh00103o00040?n000030000o`3oP000oh0000d0oh0000<0o`00
0?n0003oP0002`3oP0020?l000D0oh000P00o`020?n000800?l000<0003o003o0000o`000P00o`09
0?n000800?l0303oP0040?l00080oh002P3o00080?n000030?l0003oP000oh0001L0oh000P3o0002
0?n001Sooooo000Fooooogd0o`000P3oP0090?l00080003o3_ooool4003o00D0003o1@00o`020000
o`d00?l000<0003o003o0000o`0010000?l@003o00D0003o2`00o`090000o`800?l030000?l2003o
00/0003o1`00o`020000o`<00?l00P000?l4003o00<0003o0P00o`070000o`800?l00P000?l>oooo
o`800?l00P000?lE003o00L0003o1`00o`0=0000oa800?l00P000?l9003o00X0003o1000o`020000
o`800?l000<0003o003o0000o`000P00o`020000o`h00?l010000?l00`00o`00003o0000o`02003o
0080003o0P00o`030000o`800?l03_ooool50?n000@0o`00103oP0000`000?l0oh000?n0000=0?n0
00030?l0003oP000oh0000/0oh000P3o00050?n000800?l00P3oP002003o00030000o`00o`000?l0
00800?l02@3oP002003o00`0oh00103o00020?n000X0o`00203oP0000`3o0000oh000?n0000G0?n0
0080o`000P3oP00Hooooo`005_oooon:0?l000kooooo1@00o`040000o`D00?l01P000?l5003o0080
003o1@00o`020000o`800?l00P000?lC003o0080003o1000o`020?l000<00?l02`000?l2003o0003
0000o`00o`000?l000H0003o00<00?l00000o`000?l010000?l20?n000L0003o2P00o`060000o`80
oh0000<0003o0?n0003oP0001`000?l2003o0080003o3_oooolI003o00@0003o00<00?l00000o`00
0?l01`00o`0=0000o`L00?l00P000?l9003o0080003o1@00o`0P0000o`800?l00P000?l2003o00<0
003o0P00o`040000o`D00?l00P000?l3003o0080003o0P00o`0>ooooo`/0oh001@3o000@0?n00080
o`000P3oP0030?l000@0oh00103o00050000o`80oh000P000?lJ0?n000H0o`002`3oP0050?l000L0
oh000P3o00050?n000@0o`000P3oP0000`3o0000oh000?n000020?n00080o`000P3oP0000`3o0000
oh000?n000020?n001Sooooo000FooooohX0o`003_ooool5003o00@0003o1@00o`060000o`D00?l0
0P000?l5003o0080003o0P00o`020000oa<00?l00P000?l4003o0080o`000`00o`0;0000o`800?l0
00<0003o003o0000o`001P000?l00`00o`00003o0000o`040000o`80oh001`000?l:003o00H0003o
0P3oP0000`000?l0oh000?n000070000o`800?l00P000?l>oooooaT00?l010000?l00`00o`00003o
0000o`07003o00d0003o1`00o`020000o`T00?l00P000?l5003o0200003o0P00o`020000o`800?l0
0`000?l2003o00@0003o1@00o`020000o`<00?l00P000?l2003o00kooooo2`3oP0050?l00100oh00
0P3o00020?n000<0o`00103oP0040?l000D0003o0P3oP0020000oaX0oh001P3o000;0?n000D0o`00
1`3oP0020?l000D0oh00103o00020?n000030?l0003oP000oh000080oh000P3o00020?n000030?l0
003oP000oh000080oh006?ooool001KoooooMP3o00020?n000L0o`000P3oP0090?l000kooooo0P00
o`070000o`L00?l02`000?l2003o00<0003o0P00o`040000o`h00?l02`000?l7003o00/0003o0`00
o`070000o`H00?l00P000?l00`00o`00003o0000o`020000o`L00?l00P000?l7003o0080003o0P00
o`000`000?l00?l0003o0004003o00<0003o0P00o`020000o`kooooo0P00o`070000oa800?l00`00
0?l=003o00D0003o1P00o`020000o`<00?l00P000?l9003o0080003o00<00?l00000o`000?l01000
o`0<0000o`800?l03`000?l00`00o`00003o0000o`02003o00h0003o1000o`050000o`800?l03_oo
ool2003o0080oh000`3o000D0?n000D0o`00503oP0020000o`80oh0000<00?l00?n0003oP0001P3o
P003003o0080oh002@3o00020000o`<0o`002@3oP0020?l000@0oh0000<0o`000?n0003oP0001`3o
P0040?l000<0oh001P3o000>0?n000800?l06?ooool001KoooooRP3o000>ooooo`80003o0`00o`02
0000o`800?l00P000?l<003o00@0003o0P00o`030000o`@00?l00`000?l;003o0080003o1000o`00
0`000?l00?l0003o00020000o`D00?l00P3oP0070000o`800?l01@000?l2003o0080003o0P00o`02
0000o`T00?l01@000?l7003o00@0003o00<00?l00000o`000?l00P000?l4003o00030000o`00o`00
0?l000/00?l03_ooool2003o00/0003o1`00o`050000o`800?l00`000?l9003o0080003o1000o`03
0000o`H00?l00P000?l3003o00@0003o0P00o`030000o`800?l030000?l2003o00d0003o2@00o`03
0000o`800?l00P000?l2003o00030000o`00o`000?l000@0003o0P00o`050000o`@00?l00`000?l2
003o00kooooo2@3oP0020?l001L0oh000P000?lU0?n00080o`003`3oP0030?l00080oh001@3o0002
0?n000@0o`001@3oP0020?l001L0oh000P000?lHooooo`005_oooon:0?l000kooooo0P000?l3003o
0080003o0P00o`020000o``00?l010000?l2003o00<0003o1000o`030000o`/00?l00P000?l4003o
00030000o`00o`000?l00080003o1@00o`020?n000L0003o0P00o`050000o`800?l00P000?l2003o
0080003o2@00o`050000o`L00?l010000?l00`00o`00003o0000o`020000o`@00?l000<0003o003o
0000o`002`00o`0>ooooo`800?l02`000?l7003o00D0003o0P00o`030000o`T00?l00P000?l4003o
00<0003o1P00o`020000o`<00?l010000?l2003o00<0003o0P00o`0<0000o`800?l03@000?l9003o
00<0003o0P00o`020000o`800?l000<0003o003o0000o`0010000?l2003o00D0003o1000o`030000
o`800?l03_ooool90?n00080o`005`3oP0020000obD0oh000P3o000?0?n000<0o`000P3oP0050?l0
0080oh00103o00050?n00080o`005`3oP0020000oaSooooo000Fooooog`0o`0000<0003o0?l0003o
00002`3o000>ooooo`@0003o00<00?l00000o`000?l00P00o`050000o`P00?l01`000?l00`00o`00
003o0000o`070000o`d00?l01`000?l2003o0080003o0P3oP003003o00@0003o0`00o`020?l000/0
0?l01`000?l2003o0080003o00<00?l00000o`000?l00P000?l<003o00/0003o1`00o`020000o`80
0?l03_ooool5003o00P0003o1@00o`020000o`800?l020000?l4003o00@0003o00<00?l00000o`00
0?l01P00o`030000o`T00?l02@000?l2003o00L0003o0P00o`020000o`<00?l01P000?l00`00o`00
003o0000o`02003o00L0003o1P00o`080000o`800?l00P000?l2003o00/0003o00<00?l00000o`00
0?l00P00o`0>ooooo`800?l00P3o00050?n000@0o`0000<0oh000?l0003o0000103oP0090?l00003
0?n00000o`000?l000800?l01@000?l;0?n000@0o`00303oP0040?l000<0oh000P00o`090?n000/0
o`00103oP00C0?l000L0oh000P3o000;0?n001Sooooo000Fooooog`0o`0000<0003o0?l0003o0000
2`3o000>ooooo`@0003o00<00?l00000o`000?l00P00o`050000o`P00?l01`000?l00`00o`00003o
0000o`070000o`d00?l01`000?l2003o0080003o0P3oP003003o00@0003o0`00o`020?l000/00?l0
1`000?l2003o0080003o00<00?l00000o`000?l00P000?l<003o00/0003o1`00o`020000o`800?l0
3_ooool5003o00P0003o1@00o`020000o`800?l020000?l4003o00@0003o00<00?l00000o`000?l0
1P00o`030000o`T00?l02@000?l2003o00L0003o0P00o`020000o`<00?l01P000?l00`00o`00003o
0000o`02003o00L0003o1P00o`080000o`800?l00P000?l2003o00/0003o00<00?l00000o`000?l0
0P00o`0>ooooo`800?l00P3o00050?n000@0o`0000<0oh000?l0003o0000103oP0090?l000030?n0
0000o`000?l000800?l01@000?l;0?n000@0o`00303oP0040?l000<0oh000P00o`090?n000/0o`00
103oP00C0?l000L0oh000P3o000;0?n001Sooooo000Fooooo`h0o`000P3oP00P0?l000800?l09P3o
0002003o02D0o`000P00o`090?l000kooooo0P00o`020?l000030000o`00o`000?l000800?l01`00
0?l6003o00<0003o0P00o`090000oa@00?l01`000?l2003o00030000o`00o`000?l00080003o0P00
o`000`000?l0o`000?l000040?l001@00?l01@000?l7003o0080003o2@00o`020000o`03003o003o
P000oh000080oh000P3o0005003o0080003o3_ooool20000o`<00?l020000?l7003o00X0003o0P00
o`020000o`800?l00P000?l00`00o`00003o0000o`060000o`D00?l01`000?l2003o00L0003o0P00
o`020000o`03003o00000?l0003o01T0003o0P00o`020000o`D00?l010000?l2003o00<0003o1000
o`020000o`<00?l00P000?l4003o00030000o`00o`000?l00080003o3_ooool>0?n00080o`00403o
P0040?l000800?l01@000?l70?n00080o`000P3oP002003o0140oh000P3o00020?n000D0o`00103o
P0040?l000030?n0003o0000o`000080o`000P3oP0020?l000<0oh00103o00020?n000<0o`000P3o
P0020000o`030?l0003oP000oh0000L0oh000P3o0002003o00D0003o103oP00Hooooo`005_ooool>
0?l00080oh00803o0002003o02H0o`000P00o`0U0?l000800?l02@3o000>ooooo`800?l00P3o0000
0`000?l00?l0003o0002003o00L0003o1P00o`030000o`800?l02@000?lD003o00L0003o0P00o`00
0`000?l00?l0003o00020000o`800?l000<0003o0?l0003o0000103o000D003o00D0003o1`00o`02
0000o`T00?l00P000?l00`00o`00oh000?n000020?n00080o`001@00o`020000o`kooooo0P000?l3
003o00P0003o1`00o`0:0000o`800?l00P000?l2003o0080003o00<00?l00000o`000?l01P000?l5
003o00L0003o0P00o`070000o`800?l00P000?l00`00o`00003o0000o`0I0000o`800?l00P000?l5
003o00@0003o0P00o`030000o`@00?l00P000?l3003o0080003o1000o`000`000?l00?l0003o0002
0000o`kooooo3P3oP0020?l00100oh00103o0002003o00D0003o1`3oP0020?l00080oh000P00o`0A
0?n00080o`000P3oP0050?l000@0oh00103o00000`3oP000o`000?l000020?l00080oh000P3o0003
0?n000@0o`000P3oP0030?l00080oh000P000?l00`3o0000oh000?n000070?n00080o`000P00o`05
0000o`@0oh006?ooool001KoooooO`3o0002003o00T0o`003_ooool2003o0080oh000`000?l2003o
00T0003o2`00o`070000oa@00?l00`000?l2003o0080003o2@00o`030?l000H00?l000<0003o003o
0000o`002`00o`090000o`D00?l010000?l5003o00T0003o103oP002003o00<0003o0P00o`0>oooo
o`D00?l02`000?l2003o00L0003o0P00o`030000o`H00?l00P000?l00`00o`00003o0000o`060000
o`03003o00000?l0003o00@00?l00P000?l00`00o`00003o0000o`04003o0080003o1@00o`050000
o`@00?l00P000?l00`00o`00003o0000o`0=0000o`80oh0000<0003o0?n0003oP0001P3oP0000`00
0?l00?l0003o0004003o00030?n00000o`000?l00080003o0P00o`020?n000030000o`3oP000oh00
00H0003o00<00?l00000o`000?l00P00o`0>ooooobL0oh000P000?l60?l000X0oh00103o000@0?n0
0080o`005@3oP0020?l002@0oh006?ooool001KoooooB03o00000`3oP000o`000?l0000I0?l00080
oh00903o000>ooooo`@0003o00<00?l00000o`000?l01P000?lA003o00H0003o5000o`070000o`80
0?l000<0003o003o0000o`000P000?l2003o00<0oh001`00o`040000o`h00?l010000?l2003o0080
003o00<00?l00000o`000?l00P000?l5003o0080003o1`00o`020?l000T00?l03_ooool7003o00T0
003o1000o`020000o`030?n000000?l0003o0080003o0`00o`090000o`800?l040000?l20?n000H0
003o1@00o`020000o`03003o00000?l0003o00L0003o1P00o`030000o`@00?l010000?l00`00o`00
003o0000o`04003o00X0003o0P00o`020000o`800?l00P000?l7003o0080003o0`00o`020000o`ko
oooo1@3oP002003o00/0oh000P3o000<0?n00080o`003P3oP0020?l000@0oh000P000?l00`3oP000
o`000?l000020?n00080o`00403oP0020?l000<0oh000P000?l20?n00080003o8P3oP0000`3o0000
oh000?n000060?n000D0o`006?ooool001KoooooB03o00000`3oP000o`000?l0000I0?l00080oh00
903o000>ooooo`@0003o00<00?l00000o`000?l01P000?lA003o00H0003o5000o`070000o`800?l0
00<0003o003o0000o`000P000?l2003o00<0oh001`00o`040000o`h00?l010000?l2003o0080003o
00<00?l00000o`000?l00P000?l5003o0080003o1`00o`020?l000T00?l03_ooool7003o00T0003o
1000o`020000o`030?n000000?l0003o0080003o0`00o`090000o`800?l040000?l20?n000H0003o
1@00o`020000o`03003o00000?l0003o00L0003o1P00o`030000o`@00?l010000?l00`00o`00003o
0000o`04003o00X0003o0P00o`020000o`800?l00P000?l7003o0080003o0`00o`020000o`kooooo
1@3oP002003o00/0oh000P3o000<0?n00080o`003P3oP0020?l000@0oh000P000?l00`3oP000o`00
0?l000020?n00080o`00403oP0020?l000<0oh000P000?l20?n00080003o8P3oP0000`3o0000oh00
0?n000060?n000D0o`006?ooool001Kooooo0P3oP0100?l00080oh00AP3o000>ooooo`@00?l000<0
003o003o0000o`001000o`050000o`H00?l000<0003o003o0000o`002@00o`020000oa@00?l01@00
0?l9003o00<0003o0P3o00020000o`@00?l00`000?l4003o0080003o0`3oP004003o0080003o2P00
o`020000o`800?l00P000?l7003o0080003o00<0oh000000o`000?l010000?l7003o00kooooo0P00
o`020000o`03003o00000?l0003o00L0003o2@00o`040000o`800?l02@000?l7003o00<0003o1000
o`040000o`03003o00000?l0003o00800?l010000?l00`00o`00003o0000o`02003o0080003o00<0
0?l00000o`000?l060000?l>003o0080003o00<00?l00000o`000?l00P000?l7003o0080003o0P00
o`020?l000<00?l00P3oP00>ooooo`@0oh001@000?l70?n000@0o`001`3oP0020?l000`0oh000P3o
00020?n000800?l02@3oP003003o0080o`002@3oP0020?l000030?n0003o0000o`0000@0o`00503o
P00:0?l001d0oh006?ooool001Kooooo0P3oP0100?l00080oh00AP3o000>ooooo`@00?l000<0003o
003o0000o`001000o`050000o`H00?l000<0003o003o0000o`002@00o`020000oa@00?l01@000?l9
003o00<0003o0P3o00020000o`@00?l00`000?l4003o0080003o0`3oP004003o0080003o2P00o`02
0000o`800?l00P000?l7003o0080003o00<0oh000000o`000?l010000?l7003o00kooooo0P00o`02
0000o`03003o00000?l0003o00L0003o2@00o`040000o`800?l02@000?l7003o00<0003o1000o`04
0000o`03003o00000?l0003o00800?l010000?l00`00o`00003o0000o`02003o0080003o00<00?l0
0000o`000?l060000?l>003o0080003o00<00?l00000o`000?l00P000?l7003o0080003o0P00o`02
0?l000<00?l00P3oP00>ooooo`@0oh001@000?l70?n000@0o`001`3oP0020?l000`0oh000P3o0002
0?n000800?l02@3oP003003o0080o`002@3oP0020?l000030?n0003o0000o`0000@0o`00503oP00:
0?l001d0oh006?ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003ooooooooooooo
P_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oo
oooooooooon2ooooo`00oooooooooooooh;ooooo000Rooooog;o00009ooooomao`0002OoooooLOl0
000Wooooog;o00008oooool002;ooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]
ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Q
ooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`001OoooolFo`3o00Cooooo5ol00?l4oooo
oaOoogl01?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5_oo0005oooo
oaKo0?l01OoooolFo`00o`Cooooo5oooO`04ooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool00_oooolGool000Cooooo5_l0o`05oooooaKo003o1OoooolFoomo00Cooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`02oooooaOoo`001?oooolGo`3o00Cooooo5_l00?l5oooo
oaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKo
o`001OoooolFo`3o00Cooooo5ol00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000oooooooo
ool00_oooolFool000Gooooo5_l0o`05oooooaKo003o1?oooolGoomo00Cooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`02oooooaOoo`001?oooolFo`3o00Gooooo5_l00?l5oooooaKo
ogl01?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo0004oooooaOo
0?l01?oooolFo`00o`Gooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`00
0?oooooooooo00?ooooo5_oo000OoooooaOo003o7oooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo00;ooooo5_oo0005oooooaKo0?l01OoooolFo`00o`Cooooo5oooO`04ooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool00_oooolGool000Cooooo5_l0o`05oooooaKo003o
1OoooolFoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`02oooooaOoo`00
1?oooolGo`3o01oooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oo
oooooooo00?ooooo5_oo000OoooooaOo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo01gooooo5_l0o`05oooooaKo003o1?oooolGoomo00Cooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`02oooooaOoo`001?oooolFo`3o00Gooooo5_l00?l5oooooaKoogl01?oo
ool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5ol0o`0OoooooaKoogl01Ooo
ool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`007ooooolG
o`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolF
o`00o`Cooooo5oooO`04ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolG
ool000Cooooo5_l0o`05oooooaKo003o1OoooolFoomo00Cooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`0MoooooaOo0?l07ooooolFoomo00Gooooo00?o0000ooooooooool08Oooool0
02;ooooo00?o0000ooooooooool00ooooolFool001oooooo5ol00?lOooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?l4oooooaOoogl01?ooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo00;ooooo5ooo0004oooooaKo0?l01OoooolFo`00
o`Gooooo5_ooO`04ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolGo`3o
01oooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?o
oooo5_oo000OoooooaOo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01go
oooo5_l0o`05oooooaKo003o1?oooolGoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`02oooooaOoo`001?oooolFo`3o00Gooooo5_l00?l5oooooaKoogl01?ooool00ol0003o
ooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5ol0o`0OoooooaKoogl01Oooool00ol0003o
ooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`007ooooolGo`00oaoooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00o`Cooooo
5oooO`04ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oooolGool000Cooooo
5_l0o`05oooooaKo003o1OoooolFoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`0MoooooaOo0?l07ooooolFoomo00Gooooo00?o0000ooooooooool08Oooool002;ooooo00?o
0000ooooooooool00ooooolFool001oooooo5ol00?lOooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool07OoooolFo`3o00Gooooo5_l00?l4oooooaOoogl01?ooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo00;ooooo5ooo0004oooooaKo0?l01OoooolFo`00o`Gooooo5_oo
O`04ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolGo`3o01oooooo5_oo
O`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo000O
oooooaOo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05
oooooaKo003o1?oooolGoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0M
oooooaKo0?l01OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0M
oooooaOo0?l07ooooolFoomo00Gooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000oooo
ooooool00ooooolFool001oooooo5ol00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool07OoooolFo`3o00Gooooo5_l00?l4oooooaOoogl01?ooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o7oooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo01gooooo5ol0o`0OoooooaKoogl01Oooool00ol0003oooooooooo`0Qoooo
o`008_ooool00ol0003oooooooooo`03oooooaKoo`007ooooolGo`00oaoooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01OoooolFo`00o`Cooooo5oooO`04ooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Gooooo5_l00?lOooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolGo`3o01oooooo5_ooO`05ooooo`03
o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo000OoooooaOo003o
7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`05oooooaKo003o
1?oooolGoomo00Cooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l0
1OoooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaOo0?l0
7ooooolFoomo00Gooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000o`00Ool007l00ol0
07lFool001oo001o5ol00?lOo`00O`03o`000?oooooooooo02Gooooo00?o0000o`00Ool007l07Ol0
07lFo`3o00Go001o5_l00?l4o`00OaOoogl01?l007l00ol0003oooooooooo`0Uooooo`03o`000?l0
07oo001o01go001o5_l0o`05o`00OaKo003o7ol007l00ol0003oooooooooo`0Uooooo`03o`000?l0
07oo001o01go001o5ol0o`0Oo`00OaKoogl01Ol007l00ol0003oooooooooo`0Qooooo`008_ooool0
0ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo01gooooo5_l0o`05oooooaKo003o1?oooolGoomo00Cooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo01gooooo5ol0o`0OoooooaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0
003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
01gooooo5_l0o`05oooooaKo003o1?oooolGoomo00Cooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
01gooooo5ol0o`0OoooooaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003o
ooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01go
oooo5_l0o`0OoooooaOoogl01?ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo03So
oooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolGo`3o01oo
oooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo
5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o01oooooo
5oooO`04ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0>?oooolFo`00oaoooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaOo0?l07ooooolFoomo00Gooooo
00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o
0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l07ooooolGoomo00Cooooo00?o
0000ooooooooool09Oooool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000R
ooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool07OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0h
oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`05
ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eoooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o03[ooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03
o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000oooooooo
ool07OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0hoooooaKo
003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03
o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o03[ooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oo
oooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07Ooo
oolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0hoooooaKo003o7ooo
ool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oo
oooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooo
oooo02Gooooo00?o0000ooooooooool07OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo
00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o
03[ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo
027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Go
oooo00?o0000ooooooooool07OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?o
oooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?l007oo001o00?o001o
5_oo001Eo`00O`03o`000?oooooooooo02Gooooo00?o0000o`00Ool007l07Ol007lFo`3o03[o001o
00?o0000ooooooooool09Oooool00ol0003o001oo`00O`0ho`00OaKo003o7ol007l00ol0003ooooo
ooooo`0Uooooo`03o`000?l007oo001o05?o001o5_ooO`05o`00O`03o`000?oooooooooo027ooooo
000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool07OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_oo
O`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001E
ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o03[ooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Roooo
o`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool07OoooolFo`3o03[ooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0hoooo
oaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`05oooo
o`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o03[ooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`00
0?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Gooooo00?o0000ooooooooool0
8Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0>?oo
oolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1CoooooaKoogl01Ooo
ool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo05?ooooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo
00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo03Sooooo5_l00?lOooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0DooooolFoomo00Gooooo00?o0000ooooooooool08Oooool002;o
oooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0>?oooolFo`00oaoo
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1CoooooaKoogl01Oooool00ol0003o
ooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`0hoooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo
5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo
001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`1CoooooaKoogl01Oooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03
oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000
ooooooooool0DooooolFoomo00Gooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000o`00
Ool007l00ol007lFool005Go001o00?o0000ooooooooool09Oooool00ol0003o001oo`00O`1]o`00
O`03o`000?oooooooooo02Gooooo00?o0000o`00Ool007l0KOl007l00ol0003oooooooooo`0Uoooo
o`03o`000?l007oo001o05?o001o5_ooO`05o`00O`03o`000?oooooooooo027ooooo000Rooooo`03
o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000oooooooo
ool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`1CoooooaKoogl01Oooool00ol0003oooooooooo`0Qooooo`00
8_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo00Gooooo00?o0000ooooooooool0
8Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Gooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOoo
ool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?ooooo5_ooO`05ooooo`03o`000?oo
oooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`000?oooooo
oooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1CoooooaKoogl01Oooool0
0ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`03oooooaKoo`00EOooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0DooooolFoomo
00Gooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool00ooooolFool005Go
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Go
oooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo05?o
oooo5_ooO`05ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo
5_oo001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`1^ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo
001Eooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003ooooooooo
o`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`1^ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001E
ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1^
ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eoooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uoooo
o`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1^oooo
o`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1^ooooo`03
o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo00?ooooo5_oo001Eooooo`03o`00
0?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1^ooooo`03o`00
0?oooooooooo027ooooo000Rooooo`03o`000?l007oo001o06ko001o00?o0000ooooooooool09Ooo
ool00ol0003o001oo`00O`1]o`00O`03o`000?oooooooooo02Gooooo00?o0000o`00Ool007l0KOl0
07l00ol0003oooooooooo`0Uooooo`03o`000?l007oo001o06ko001o00?o0000ooooooooool08Ooo
ool002;ooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooo
oooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooo
oooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0
003oooooooooo`1^ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0
003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`1^ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo06ko
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Go
oooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06ko
oooo00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool0K_ooool00ol0003ooooo
ooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003ooooo
ooooo`0Qooooo`008_ooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo02Gooooo00?o
0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o
0000ooooooooool09Oooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo027ooooo000R
ooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]
ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0U
ooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000oooo
ooooool0K_ooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`1^ooooo`03
o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1^ooooo`03
o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0
KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06kooooo00?o0000ooooooooool0
8Oooool002;ooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Uooooo`03o`000?oo
oooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oo
oooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003oooooooooo`0Qooooo`008_ooool0
0ol0003oooooooooo`1^ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`1^ooooo`03o`000?oooooooooo027ooooo000Rooooo`03o`000?oooooooooo
06kooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo
06kooooo00?o0000ooooooooool08Oooool002;ooooo00?o0000ooooooooool0K_ooool00ol0003o
ooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0K_ooool00ol0003o
ooooooooo`0Qooooo`008_ooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`1^ooooo`03o`000?oooooooooo027ooooo
000Rooooog;o00009ooooomao`0002OoooooLOl0000Wooooog;o00008oooool00?oooooooooooon2
ooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooo
oooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;o
oooo003oooooooooooooP_ooool001Kooooo0P3o0002003o01<0o`000P000?l=0?l000030000o`3o
0000o`000080003o9P3o00000`000?l0o`000?l000090?l00180003o403o00040000o`<0o`001000
0?l>ooooo`@0oh001@00o`020?n000D00?l00P3oP002003o00T0oh0000<00?l00000o`000?l01000
0?l5003o0080oh003@00o`000`3oP0000?l0003o00020?n000800?l00`000?l20?n000L00?l00P00
0?l4003o00D0003o1@00o`020000o`@00?l00P000?l00`3o0000003o0000o`020000o`80oh001`00
o`020000o`03003o003oP000oh0000H0oh0000<00?l00?n0003oP0001000o`030000o`80oh000P00
o`0>ooooo`80003o0`3o00020?n000800?l0103oP0000`00o`00o`000?l00002003o00D0oh000P3o
00020000o`800?l00`000?l40?l000800?l00`3oP002003o00L0003o0P00o`020?n000<0o`000P00
o`020?l000D0003o0P3o00020000o`<0o`003`000?l20?l000X0003o1000o`040?l000030000o`00
o`000?l00080003o1@3oP006003o00030000o`3o0000o`0000H0003o0`3o00020000o`kooooo0P3o
P002003o00030?n00000o`000?l000@00?l00`3oP002003o00T0oh000P00o`070?n000@00?l00`3o
P002003o00D0oh002@00o`020?n000800?l00P3oP002003o00030?n00000o`000?l00080oh000P00
o`0<0?n000@00?l00P3oP0030?l000@00?l00`3oP002003o00@0003o0P3o00030?n000800?l00`3o
P002003o0080oh000P00o`030?l00080oh000P00o`020000o`<0oh001000o`0Hooooo`005_ooool2
0?l000800?l04`3o00020000o`d0o`0000<0003o0?l0003o00000P000?lV0?l000030000o`3o0000
o`0000T0o`004P000?l@0?l000@0003o0`3o00040000o`kooooo103oP005003o0080oh001@00o`02
0?n000800?l02@3oP0000`00o`00003o0000o`040000o`D00?l00P3oP00=003o00030?n00000o`00
0?l00080oh000P00o`030000o`80oh001`00o`020000o`@00?l01@000?l5003o0080003o1000o`02
0000o`030?l000000?l0003o0080003o0P3oP007003o0080003o00<00?l00?n0003oP0001P3oP000
0`00o`00oh000?n00004003o00<0003o0P3oP002003o00kooooo0P000?l30?l00080oh000P00o`04
0?n00003003o003o0000o`0000800?l01@3oP0020?l00080003o0P00o`030000o`@0o`000P00o`03
0?n000800?l01`000?l2003o0080oh000`3o0002003o0080o`001@000?l20?l00080003o0`3o000?
0000o`80o`002P000?l4003o00@0o`0000<0003o003o0000o`000P000?l50?n000H00?l000<0003o
0?l0003o00001P000?l30?l00080003o3_ooool20?n000800?l000<0oh00003o0000o`001000o`03
0?n000800?l02@3oP002003o00L0oh001000o`030?n000800?l01@3oP009003o0080oh000P00o`02
0?n000800?l000<0oh00003o0000o`000P3oP002003o00`0oh001000o`020?n000<0o`001000o`03
0?n000800?l010000?l20?l000<0oh000P00o`030?n000800?l00P3oP002003o00<0o`000P3oP002
003o0080003o0`3oP004003o01Sooooo000Fooooo`80o`000P00o`0C0?l00080003o3@3o00000`00
0?l0o`000?l000020000obH0o`0000<0003o0?l0003o00002@3o000B0000oaL0o`000P000?l20?l0
00kooooo103oP005003o0080oh002@00o`090?n000D00?l00P000?l5003o0080oh003@00o`000`3o
P0000?l0003o00020?n000800?l00`000?l40?n000D00?l00P3oP005003o00@0003o2`00o`020000
o`030?l000000?l0003o0080003o0P3oP005003o00@0003o2P3oP006003o0080003o00<00?l00?n0
003oP0000P00o`0>ooooo`80003o0P3oP0000`3o0000oh000?n00004003o0080oh000`3o00040000
o`80oh0000<00?l00?l0003o00000P000?l2003o00<0003o1P3o00030?n000800?l01@000?l60?n0
00<0o`000P00o`020?l000@0003o00<0o`000000o`000?l03@3o000C0000o`@00?l00P3o00030000
o`800?l010000?l30?n000@00?l01@000?l20?l000@0003o00<00?l00000o`000?l00P000?l>oooo
o`80oh000P00o`000`3oP0000?l0003o0004003o00<0oh000P00o`0B0?n000@00?l00`3oP002003o
00L0oh001`00o`060?n000800?l000<0oh00003o0000o`000P3oP0020000o``0oh001000o`020?n0
00<0o`001`3oP002003o0080003o103o00000`000?l0oh000?n000090?n000800?l00P3o00030?n0
00800?l01@3oP0020?l000800?l06?ooool001Kooooo0P3o0002003o01<0o`000P000?l=0?l00003
0000o`3o0000o`000080003o9P3o00000`000?l0o`000?l000090?l00180003o5`3o00020000o`80
o`003_ooool40?n000D00?l00P3oP009003o00T0oh001@00o`020000o`D00?l00P3oP00=003o0003
0?n00000o`000?l00080oh000P00o`030000o`@0oh001@00o`020?n000D00?l010000?l;003o0080
003o00<0o`000000o`000?l00P000?l20?n000D00?l010000?l:0?n000H00?l00P000?l00`00o`00
oh000?n00002003o00kooooo0P000?l20?n000030?l0003oP000oh0000@00?l00P3oP0030?l000@0
003o0P3oP0000`00o`00o`000?l000020000o`800?l00`000?l60?l000<0oh000P00o`050000o`H0
oh000`3o0002003o0080o`0010000?l00`3o0000003o0000o`0=0?l001<0003o1000o`020?l000<0
003o0P00o`040000o`<0oh001000o`050000o`80o`0010000?l00`00o`00003o0000o`020000o`ko
oooo0P3oP002003o00030?n00000o`000?l000@00?l00`3oP002003o0180oh001000o`030?n00080
0?l01`3oP007003o00H0oh000P00o`000`3oP0000?l0003o00020?n00080003o303oP004003o0080
oh000`3o00070?n000800?l00P000?l40?l000030000o`3oP000oh0000T0oh000P00o`020?l000<0
oh000P00o`050?n00080o`000P00o`0Hooooo`005_ooool20000o`800?l02@3o00000`000?l0o`00
0?l000020000o`@0o`000`000?l70?l00080003o103o00000`000?l0o`000?l0000B0?l00080003o
503o00030000o`@0o`000P000?l30?l00180003o2@3o00020000oa00o`003_ooool20000o`80oh00
1@00o`020?n000T00?l02@3oP005003o0080003o0P3oP0000`00o`00oh000?n0000?003o00030?n0
0000o`000?l00080oh000P00o`030000o`@0oh001@00o`020?l000D00?l010000?l;003o00L0003o
0P3oP005003o00@0003o3P3oP005003o0080oh000P00o`0>ooooo`D0o`000P3oP004003o0080003o
0`3o00060000o`03003o003o0000o`000080o`000P00o`0:0000o`80oh000P00o`040000o`L0oh00
0P3o00030?n000@0003o0P3o00000`00o`00003o0000o`050?l00080003o0P00o`020000o`80o`00
4`000?l20?n000800?l010000?l00`3o00000?l0003o00050000o`80oh000P00o`070000o`80o`00
10000?l00`3oP000003o0000o`02003o00kooooo0P3oP002003o00030?n00000o`000?l000@00?l0
0P3oP005003o00D0oh000P00o`090?n000@00?l02@3oP0000`00o`00oh000?n00007003o00T0oh00
0P00o`0@0?n000@00?l03P3oP0040000o`80o`0000<0003o0?n0003oP0000P00o`090?n000<0o`00
3@3oP00Hooooo`005_ooool20000o`800?l02@3o00000`000?l0o`000?l000060?l000030000o`3o
0000o`0000L0o`000P000?lI0?l00080003o303o00020000o`L0o`000P000?l40?l00080003o0`3o
000D0000o`<0oh00103o00040000o`T0o`0000<0003o0?l0003o00000P3o000>ooooo`D0oh000P00
o`060?n000L00?l01`3oP007003o0080003o0P3oP0000`00o`00oh000?n0000=003o00<0oh000P00
o`020000o`800?l01@000?l50?n00080o`000P000?l4003o00D0003o2@00o`040000o`03003o0000
0?l0003o0080003o0P3oP005003o00@0003o1`3oP002003o00030?n000000?l0003o0080oh001@00
o`020?n000800?l03_ooool20000o`<0o`00103oP002003o0080003o1`3o00020?n00003003o003o
0000o`0000@00?l02P000?l20?n000@00?l00P000?l30?n000H00?l000<0oh00003o0000o`001P00
0?l00`00o`00003o0000o`040?l000<0003o0P00o`020?n00080o`004`000?l20?n000800?l01000
0?l00`3o00000?l0003o00050000o`@00?l040000?l20?n000kooooo0P3oP002003o00030?n00000
o`000?l000800?l01@3oP002003o0080oh001000o`000`3oP0000?l0003o00050?n000800?l00P3o
P002003o00T0oh000`00o`060?n000800?l00`3oP004003o00T0oh000`00o`090?n000@00?l0403o
P0020000o`80o`000`3oP002003o00L0oh000P000?l50?l000/0oh006?ooool001Kooooo0P000?l2
003o00T0o`0000<0003o0?l0003o00001P3o00000`000?l0o`000?l000070?l00080003o6@3o0002
0000o``0o`000P000?l70?l00080003o103o00020000o`<0o`0050000?l30?n000@0o`0010000?l9
0?l000030000o`3o0000o`000080o`003_ooool50?n000800?l01P3oP007003o00L0oh001`00o`02
0000o`80oh0000<00?l00?n0003oP0003@00o`030?n000800?l00P000?l2003o00D0003o1@3oP002
0?l00080003o1000o`050000o`T00?l010000?l00`00o`00003o0000o`020000o`80oh001@00o`04
0000o`L0oh000P00o`000`3oP000003o0000o`020?n000D00?l00P3oP002003o00kooooo0P000?l3
0?l000@0oh000P00o`020000o`L0o`000P3oP0000`00o`00o`000?l00004003o00X0003o0P3oP004
003o0080003o0`3oP006003o00030?n00000o`000?l000H0003o00<00?l00000o`000?l0103o0003
0000o`800?l00P3oP0020?l001<0003o0P3oP002003o00@0003o00<0o`00003o0000o`001@000?l4
003o0100003o0P3oP00>ooooo`80oh000P00o`000`3oP0000?l0003o0002003o00D0oh000P00o`02
0?n000@00?l000<0oh00003o0000o`001@3oP002003o0080oh000P00o`090?n000<00?l01P3oP002
003o00<0oh001000o`090?n000<00?l02@3oP004003o0100oh000P000?l20?l000<0oh000P00o`07
0?n00080003o1@3o000;0?n001Sooooo000Fooooo`@0003o2P3o00020000o`H0o`0000<0003o0?l0
003o00001`3o00020000o`D0o`000P000?lB0?l00080003o303o00060000o`<0o`000P000?l90?l0
00l0003o00<0o`000000o`000?l00P000?lB0?l00080oh001@3o000>ooooo`80003o0P00o`000`3o
P0000?l0003o00020?n000800?l00P3oP007003o00L0oh001`00o`020000o`80oh0000<00?l00?n0
003oP0003`00o`000`3oP000003o0000o`090000o`80oh000P00o`0<0000o`80oh001000o`000`3o
0000003o0000o`02003o00@0003o00<00?l00?l0003o00001P3oP003003o00@0003o1`3oP002003o
00<0003o0P3oP002003o0080003o00<00?l00?n0003oP0000P00o`0>ooooo`800?l00P3o00000`00
o`00oh000?n000060?n000030000o`3o0000o`000080o`001@000?l6003o00`0003o0P00o`040000
o`<0oh001000o`020?l000030000o`00o`000?l000@0003o0P00o`030000o`D0o`000P000?l2003o
0080oh005`000?l2003o0080003o0P00o`000`3o00000?l0003o00040000o`030?n00000o`000?l0
00800?l00P000?l2003o00D0003o0P00o`030000o`80o`000P3oP00>ooooo`80oh000P00o`0>0?n0
00@00?l0203oP009003o01P0oh000P00o`080?n000@00?l0103oP0000`00o`00oh000?n00002003o
01@0oh000P3o00050?n000800?l000<0oh00003o0000o`000P3oP0050?l000d0oh006?ooool001Ko
oooo10000?l:0?l00080003o1P3o00000`000?l0o`000?l000070?l00080003o1@3o00020000oa80
o`000P000?l<0?l000H0003o0`3o00020000o`T0o`003`000?l00`3o0000003o0000o`020000oa80
o`000P3oP0050?l000kooooo0P000?l2003o00030?n00000o`000?l00080oh000P00o`020?n000L0
0?l01`3oP007003o0080003o0P3oP0000`00o`00oh000?n0000?003o00030?n000000?l0003o00T0
003o0P3oP002003o00`0003o0P3oP004003o00030?l000000?l0003o00800?l010000?l00`00o`00
o`000?l000060?n000<00?l010000?l70?n000800?l00`000?l20?n000800?l00P000?l00`00o`00
oh000?n00002003o00kooooo0P00o`020?l00003003o003oP000oh0000H0oh0000<0003o0?l0003o
00000P3o00050000o`H00?l030000?l2003o00@0003o0`3oP004003o0080o`0000<0003o003o0000
o`0010000?l2003o00<0003o1@3o00020000o`800?l00P3oP00G0000o`800?l00P000?l2003o0003
0?l00000o`000?l000@0003o00<0oh00003o0000o`000P00o`020000o`800?l01@000?l2003o00<0
003o0P3o00020?n000kooooo0P3oP002003o00h0oh001000o`080?n000T00?l0603oP002003o00P0
oh001000o`040?n00003003o003oP000oh0000800?l0503oP0020?l000D0oh000P00o`000`3oP000
0?l0003o00020?n000D0o`003@3oP00Hooooo`005_ooool20?l00080003o2P3o00020000oa00o`00
10000?l30?l00080003o9P3o00020000o``0o`000P000?l20?l000/0003o00<0o`000000o`000?l0
0P000?l50?l000@0003o0`3o00060000o`800?l000<0o`000000o`000?l00P3o000>ooooo`80003o
0P00o`000`3oP0000?l0003o00020?n000800?l00P3oP007003o0080oh0000<00?l00?l0003o0000
103oP0000`00o`00oh000?n00002003o0080003o0P3oP0000`00o`00oh000?n0000?003o00030?n0
00000?l0003o00T0003o0P3oP002003o00`0003o0P3oP0040000o`030?l000000?l0003o00800?l0
10000?l00`00o`00003o0000o`060?n000<00?l010000?l70?n000800?l00`000?l20?n000D00?l0
0P3oP002003o00kooooo0P3oP0030?l000P0oh000`00o`020?l00080003o0P00o`000`3oP0000?l0
003o0002003o0080003o00<0o`000000o`000?l02@000?l2003o0080003o0P3o00030?n000@00?l0
0`3o0002003o00H0003o1@3o00020000o`030?l00000o`000?l000T0003o103o00070000o`800?l0
1@000?l2003o0080003o0`3o00040000o`80o`000`3oP006003o00030?n00000o`000?l00080003o
0P00o`030000o`80o`000P000?l>ooooo`@0oh0000<00?l00?n0003oP0000P00o`040?n000<00?l0
0P3oP007003o00L0oh001`00o`0J0?n00003003o003oP000oh0000D0oh001000o`040?n00003003o
003oP000oh0001d0oh000P00o`000`3oP0000?l0003o00040?l00080003o3P3oP00Hooooo`005_oo
ool20?l00080003o2P3o00020000oa00o`0010000?l30?l00080003o9P3o00020000o``0o`000P00
0?l20?l000/0003o00<0o`000000o`000?l00P000?l50?l000@0003o0`3o00060000o`800?l000<0
o`000000o`000?l00P3o000>ooooo`80003o0P00o`000`3oP0000?l0003o00020?n000800?l00P3o
P007003o0080oh0000<00?l00?l0003o0000103oP0000`00o`00oh000?n00002003o0080003o0P3o
P0000`00o`00oh000?n0000?003o00030?n000000?l0003o00T0003o0P3oP002003o00`0003o0P3o
P0040000o`030?l000000?l0003o00800?l010000?l00`00o`00003o0000o`060?n000<00?l01000
0?l70?n000800?l00`000?l20?n000D00?l00P3oP002003o00kooooo0P3oP0030?l000P0oh000`00
o`020?l00080003o0P00o`000`3oP0000?l0003o0002003o0080003o00<0o`000000o`000?l02@00
0?l2003o0080003o0P3o00030?n000@00?l00`3o0002003o00H0003o1@3o00020000o`030?l00000
o`000?l000T0003o103o00070000o`800?l01@000?l2003o0080003o0`3o00040000o`80o`000`3o
P006003o00030?n00000o`000?l00080003o0P00o`030000o`80o`000P000?l>ooooo`@0oh0000<0
0?l00?n0003oP0000P00o`040?n000<00?l00P3oP007003o00L0oh001`00o`0J0?n00003003o003o
P000oh0000D0oh001000o`040?n00003003o003oP000oh0001d0oh000P00o`000`3oP0000?l0003o
00040?l00080003o3P3oP00Hooooo`005_ooool40000o`X0o`000P000?lG0?l00080003o103o0002
0000oaT0o`0000<0003o0?l0003o0000103o00030000o`l0o`001`000?l20?l00080003o00<0o`00
0000o`000?l00P000?l70?l00080003o0`3o00040000o`D0o`000P3oP0020?l000kooooo0P000?l2
003o00030?n00000o`000?l00080oh000P00o`020?n000D00?l0103oP0000`000?l0o`000?l00004
0?n00003003o003oP000oh0000800?l00P000?l20?n00003003o003oP000oh0000l00?l00`3oP00;
0000o`800?l000<0003o0?l0003o00002@000?l20?n000@0003o00<0o`000000o`000?l00P00o`04
0000o`03003o00000?l0003o0080003o103oP003003o00@0003o303oP007003o00@0oh003_ooool2
0?n000<00?l01P3oP005003o00H0003o00<0oh00003o0000o`000P00o`0>0000o`@00?l00P000?l3
0?n000H00?l000<0o`000000o`000?l00P000?l20?l000T0003o00<0oh00003o0000o`001P000?l0
0`00o`00003o0000o`040?l000L0003o0P00o`090000o`D0o`000P000?l20?l000D0oh001@00o`02
0?n000@00?l00P3o00000`000?l0o`000?l000020000o`kooooo0P00o`020?n000D00?l0103oP003
003o00/0oh000P00o`050?n000D00?l00P3oP002003o0080oh000P00o`070?n000800?l000<0oh00
003o0000o`000P3oP002003o0080oh0000<00?l00?n0003oP0000P3oP002003o00030?n00000o`00
0?l002d0oh000P00o`070?n000800?l00`3oP002003o0080oh006?ooool001Kooooo10000?l90?l0
00<0003o403o00020000o`D0o`0010000?lM0?l000030000o`3o0000o`0000@0o`000P000?lC0?l0
00@0003o0P3o00070000o`L0o`000P000?l50?l00080003o1@3o00020?n000800?l03_ooool40000
o`030?n00000o`000?l00080oh000P00o`020?n000D00?l00P3oP002003o00030000o`3o0000o`00
0080oh000`00o`020?l000@0003o0P3oP0000`00o`00oh000?n00006003o00030?n00000o`000?l0
00@00?l01@3oP002003o00T0003o0P00o`000`000?l0o`000?l000090000o`80oh001`000?l2003o
0080o`000P000?l00`00o`00003o0000o`020000o`@0oh000`00o`040000o`D0oh000P00o`050?n0
00L00?l0103oP00>ooooo`@0oh0000<00?l00?n0003oP0000P3oP002003o0080oh002@00o`000`3o
P0000?l0003o0002003o00`0003o0P3oP004003o00<0003o0P3oP004003o00<0o`003@000?l20?l0
00030000o`00o`000?l00080oh0010000?l00`00o`00003o0000o`0D0000o`80o`000P000?l30?l0
00@0003o1@3oP004003o00<0003o1000o`070000o`kooooo0P00o`020?n00003003o003o0000o`00
00@0oh001@00o`0F0?n00003003o003oP000oh0000800?l0103oP003003o00H0oh000`00o`0;0?n0
00800?l000<0oh00003o0000o`00;@3oP002003o0080oh0000<00?l00?n0003oP0002`3oP00Hoooo
o`005_ooool40000o`T0o`000`000?l@0?l00080003o1@3o00040000oad0o`0000<0003o0?l0003o
0000103o00020000oa<0o`0010000?l20?l000L0003o1`3o00020000o`D0o`000P000?l50?l00080
oh000P00o`0>ooooo`@0003o00<0oh00003o0000o`000P3oP002003o0080oh001@00o`020?n00080
0?l000<0003o0?l0003o00000P3oP003003o0080o`0010000?l20?n00003003o003oP000oh0000H0
0?l000<0oh00003o0000o`001000o`050?n000800?l02@000?l2003o00030000o`3o0000o`0000T0
003o0P3oP0070000o`800?l00P3o00020000o`03003o00000?l0003o0080003o103oP003003o00@0
003o1@3oP002003o00D0oh001`00o`040?n000kooooo103oP0000`00o`00oh000?n000020?n00080
0?l00P3oP009003o00030?n00000o`000?l000800?l030000?l20?n000@00?l00`000?l20?n000@0
0?l00`3o000=0000o`80o`0000<0003o003o0000o`000P3oP0040000o`03003o00000?l0003o01@0
003o0P3o00020000o`<0o`0010000?l50?n000@00?l00`000?l4003o00L0003o3_ooool2003o0080
oh0000<00?l00?l0003o0000103oP005003o01H0oh0000<00?l00?n0003oP0000P00o`040?n000<0
0?l01P3oP003003o00/0oh000P00o`000`3oP0000?l0003o000]0?n000800?l00P3oP0000`00o`00
oh000?n0000;0?n001Sooooo000Fooooo`@0003o2P3o00040000o`/0o`0000<0003o0?l0003o0000
1`3o00040000oad0o`0000<0003o0?l0003o00000P3o00020000o`/0o`001`000?l30?l000@0003o
0P3o00070000o`L0o`000P000?l50?l00080003o2@3o000>ooooo`D0oh000P00o`060?n000D00?l0
103oP0000`000?l0o`000?l000020?n000<00?l00P3o0002003o00@0oh0000<00?l00?n0003oP000
1P00o`000`3oP0000?l0003o0004003o00<0oh001000o`050000o`@0oh000P00o`0E0000o`800?l0
0P3o00020000o`03003o00000?l0003o0080003o103oP003003o00@0003o1@3oP002003o0080oh00
0`3o0006003o00030?l0003oP000oh0000800?l03_ooool20?n00080003o0`00o`020?n000D00?l0
0P3oP006003o00030?n00000o`000?l000800?l03P000?l4003o00<0003o0P3oP006003o00<0o`00
2`000?l20?l000<0003o1P3oP0000`00o`00003o0000o`020?l00180003o0P3o00030000o`80o`00
0P000?l2003o00<0oh000P00o`020000o`800?l03P000?l>ooooo`@0oh000`00o`040?n000D00?l0
503oP002003o00X0oh002000o`030?n000800?l01@3oP002003o0080oh000P00o`0B0?n00003003o
003oP000oh0001l0oh0000<00?l00?n0003oP0002`3oP00Hooooo`005_ooool40000o`X0o`001000
0?l;0?l000030000o`3o0000o`0000L0o`0010000?lM0?l000030000o`3o0000o`000080o`000P00
0?l;0?l000L0003o0`3o00040000o`80o`001`000?l70?l00080003o1@3o00020000o`T0o`003_oo
ool50?n000800?l01P3oP005003o00@0oh0000<0003o0?l0003o00000P3oP003003o0080o`000P00
o`040?n00003003o003oP000oh0000H00?l000<0oh00003o0000o`001000o`030?n000@00?l01@00
0?l40?n000800?l05@000?l2003o0080o`000P000?l00`00o`00003o0000o`020000o`@0oh000`00
o`040000o`D0oh000P00o`020?n000<0o`001P00o`000`3o0000oh000?n00002003o00kooooo0P3o
P0020000o`<00?l00P3oP005003o0080oh001P00o`000`3oP0000?l0003o0002003o00h0003o1000
o`030000o`80oh001P00o`030?l000/0003o0P3o00030000o`H0oh0000<00?l00000o`000?l00P3o
000B0000o`80o`000`000?l20?l00080003o0P00o`030?n000800?l00P000?l2003o00h0003o3_oo
ool40?n000<00?l0103oP005003o01@0oh000P00o`0:0?n000P00?l00`3oP002003o00D0oh000P00
o`020?n000800?l04P3oP0000`00o`00oh000?n0000O0?n00003003o003oP000oh0000/0oh006?oo
ool001Kooooo10000?l00`3o0000003o0000o`070?l000@0003o9`3o00040000o`D0o`000P000?l7
0?l00080003o103o00000`000?l0o`000?l000060?l000L0003o0`3o00040000o`80o`001@000?l5
0?l00080003o3@3o00020?n000030000o`3o0000o`000080o`003_ooool20?n000800?l000<0oh00
003o0000o`001`3oP006003o00L0oh000`00o`020?n000800?l01@3oP008003o00<0oh001000o`03
0?n000H00?l00`000?l40?n000800?l03P000?l2003o0080003o00<00?l00?l0003o00001000o`02
0000o`<00?l00P000?l40?n000D00?l00P3oP0000`00o`00oh000?n00006003o00T0003o00<0o`00
0000o`000?l00P00o`0>ooooo`80o`000P000?l00`00o`00003o0000o`020?n000@00?l000<0003o
0?n0003oP0001`3oP006003o00030000o`3o0000o`0000T0003o0P00o`040000o`<0oh001000o`02
0000o`030?n00000o`000?l000@00?l01`000?l20?l000030000o`3o0000o`000080o`000P00o`02
0?n00003003o00000?l0003o00H0003o00<0o`000000o`000?l050000?l2003o00D0oh000P3o0002
003o00<0003o0P3o0002003o00L0003o3_ooool2003o0080oh000`00o`040?n000D00?l02@3oP002
003o00T0oh000P00o`0<0?n000L00?l00P3oP004003o00D0oh000P00o`0G0?n00080o`007@3oP000
0`00o`00oh000?n000040?n000800?l000<0oh00003o0000o`000P3oP00Hooooo`005_ooool40000
o`030?l000000?l0003o00L0o`0010000?lW0?l000@0003o1@3o00020000o`L0o`000P000?l40?l0
00030000o`3o0000o`0000H0o`001`000?l30?l000@0003o0P3o00050000o`D0o`000P000?l=0?l0
0080oh0000<0003o0?l0003o00000P3o000>ooooo`80oh000P00o`000`3oP0000?l0003o00070?n0
00H00?l01`3oP003003o0080oh000P00o`050?n000P00?l00`3oP004003o00<0oh001P00o`030000
o`@0oh000P00o`0>0000o`800?l00P000?l00`00o`00o`000?l00004003o0080003o0`00o`020000
o`@0oh001@00o`020?n00003003o003oP000oh0000H00?l02@000?l00`3o0000003o0000o`02003o
00kooooo0P3o00020000o`03003o00000?l0003o0080oh001000o`000`000?l0oh000?n000070?n0
00H00?l000<0003o0?l0003o00002@000?l2003o00@0003o0`3oP004003o0080003o00<0oh00003o
0000o`001000o`070000o`80o`0000<0003o0?l0003o00000P3o0002003o0080oh0000<00?l00000
o`000?l01P000?l00`3o0000003o0000o`0D0000o`800?l01@3oP0020?l000800?l00`000?l20?l0
00800?l01`000?l>ooooo`800?l00P3oP003003o00@0oh001@00o`090?n000800?l02@3oP002003o
00`0oh001`00o`020?n000@00?l01@3oP002003o01L0oh000P3o000M0?n00003003o003oP000oh00
00@0oh000P00o`000`3oP0000?l0003o00020?n001Sooooo000Fooooo`@0003o2P3o00040000o`T0
o`000P000?l50?l00080003o5@3o00040000o`L0o`000P000?l30?l000@0003o0P3o000B0000o`80
o`000`000?l60?l000030000o`3o0000o`0000D0o`000P000?l40?l00080003o0`3o00020000o`80
o`000P3oP0000`000?l0o`000?l000020?l000kooooo0P3oP002003o00030?n00000o`000?l000L0
oh001P00o`050?n000T00?l01@3oP002003o0080oh002@00o`040?n000L00?l00`000?l40?n00080
0?l04P000?l00`00o`00oh000?n000020?n000D00?l0203oP005003o00<0oh001P00o`0<0000o`80
oh000P00o`0>ooooo`80003o0P3o00000`00o`00003o0000o`020?n000@00?l00`3o0004003o0080
oh001@00o`070000o`80o`002`000?l30?n000@00?l01@000?l4003o00<0003o0P3o00020000o`80
o`0000<0003o0?l0003o00000P000?l2003o0080003o00<00?l00000o`000?l04P000?l20?n000@0
003o00<0o`000000o`000?l01@00o`060?n000800?l000<0003o0?l0003o00000P3o0002003o00L0
003o3_ooool2003o00<0oh000P00o`070?n000800?l02@3oP002003o00L0oh001000o`0>0?n000D0
0?l0103oP002003o00T0oh0000<00?l00?n0003oP0000P00o`020?n000800?l03P3oP002003o0080
oh0000<00?l00?n0003oP0001P3oP0000`000?l0oh000?n0000;0?n000800?l000<0oh00003o0000
o`000P00o`020?n000800?l000<0o`00003o0000o`000P3oP00Hooooo`005_ooool40000o`X0o`00
10000?l90?l00080003o1@3o00020000oaD0o`0010000?l70?l00080003o0`3o00040000o`80o`00
4P000?l20?l000<0003o1P3o00000`000?l0o`000?l000050?l00080003o103o00020000o`<0o`00
0P000?l20?l00080oh0000<0003o0?l0003o00000P3o000>ooooo`80oh000P00o`000`3oP0000?l0
003o00070?n000H00?l01@3oP009003o00D0oh000P00o`020?n000T00?l0103oP007003o00<0003o
103oP002003o0180003o00<00?l00?n0003oP0000P3oP005003o00P0oh001@00o`030?n000H00?l0
30000?l20?n000800?l03_ooool20000o`80o`0000<00?l00000o`000?l00P3oP004003o00<0o`00
1000o`020?n000D00?l01`000?l20?l000/0003o0`3oP004003o00D0003o1000o`030000o`80o`00
0P000?l20?l000030000o`3o0000o`000080003o0P00o`020000o`03003o00000?l0003o0180003o
0P3oP0040000o`030?l000000?l0003o00D00?l01P3oP002003o00030000o`3o0000o`000080o`00
0P00o`070000o`kooooo0P00o`030?n000800?l01`3oP002003o00T0oh000P00o`070?n000@00?l0
3P3oP005003o00@0oh000P00o`090?n00003003o003oP000oh0000800?l00P3oP002003o00h0oh00
0P00o`020?n00003003o003oP000oh0000H0oh0000<0003o0?n0003oP0002`3oP002003o00030?n0
0000o`000?l000800?l00P3oP002003o00030?l00000o`000?l00080oh006?ooool001Kooooo1000
0?l:0?l000@0003o1@3o00020000o`L0o`0010000?l20?l000030000o`3o0000o`000180o`000P00
0?l70?l000@0003o00<0o`000000o`000?l00P000?l20?l00080003o1@3o00040000o`030?l00000
0?l0003o00@0003o0P3o00030000o`80o`000P000?l:0?l000@0003o0P3o00030000o`80o`001000
0?l20?n000030000o`3o0000o`000080003o3_ooool20?n000D00?l00P3oP002003o0080oh001`00
o`050?n000T00?l01@3oP00F003o0080003o0P3o00000`000?l0oh000?n000020?n000800?l02P00
0?l20?l000H0003o00<0oh000000o`000?l00P3oP007003o00@0oh001`00o`030?n000H00?l00P00
0?l00`3o0000003o0000o`020000o`800?l00`000?l20?n000800?l03_ooool40000o`03003o0000
0?l0003o0080003o2`00o`030?n000@00?l00P3o00000`000?l0o`000?l000040?l000`0003o0P3o
P004003o0080003o00<0o`000000o`000?l01000o`030000o`L0o`002`000?l20?l00100003o0P00
o`050?l00080oh000P00o`020000o`03003o003oP000oh0000@0oh001@00o`0;0000o`kooooo0P00
o`0<0?n000@00?l03P3oP002003o01@0oh000`00o`040?n000800?l00`3oP002003o00@0oh0000<0
0?l00?n0003oP0000P00o`0>0?n000800?l00P3oP002003o0080oh0000<00?l00?n0003oP0000P3o
P002003o0080oh000`000?l40?n00003003o003oP000oh0000800?l00P3oP002003o00030?n00000
o`000?l000800?l00P3oP002003o00030?l0003oP000oh000080oh006?ooool001Kooooo10000?l3
0?l00080003o1`3o00040000o`80o`000`000?l70?l000@0003o5`3o00020000o`L0o`0010000?l9
0?l000030000o`3o0000o`0000T0o`000P000?l40?l000<0003o0P3o00020000oa00o`000P000?l5
0?l00080003o0P00o`050000o`kooooo0P3oP005003o0080oh000P00o`020?n000T00?l00`3oP005
003o0080oh001000o`030?n000800?l00P3oP00;003o0080oh001@00o`020000o`80o`000`000?l2
0?n000800?l030000?l20?l000@0003o00<0oh000000o`000?l00P3oP0020000o`80oh000`00o`02
0?n000T00?l00`3oP002003o0080oh001`00o`070000o`80oh000P00o`0>ooooo`@0oh0000<00?l0
0000o`000?l00P000?l40?n000L00?l01@3oP0040?l000L0003o0P00o`000`3o0000003o0000o`02
0?l000L0003o1000o`020000o`<0o`000P000?l20?l001L0003o0P3o00020000o`03003o00000?l0
003o00800?l02@000?l20?l00080oh0000<0o`000000o`000?l00P00o`030000o`P0oh0000<00?l0
0000o`000?l02`000?l>ooooo`800?l00`3oP002003o0080o`001@3oP004003o00L0oh000P00o`02
0?n00003003o003oP000oh0000800?l0103oP0000`00o`00oh000?n000020?n000800?l04P3oP002
003o00030?n00000o`000?l000@0oh0000<00?l00?n0003oP0000P00o`070?n000800?l00P000?l3
0?n000800?l00P3o0002003o0080o`0000<00?l00?n0003oP0001P3oP0000`000?l0oh000?n00002
0?n000800?l0303oP002003o0080oh000P00o`050?n001Sooooo000Fooooo`@0003o0`3o00020000
o`L0o`0010000?l20?l000<0003o1`3o00040000oaL0o`000P000?l70?l000@0003o2@3o00000`00
0?l0o`000?l000090?l00080003o103o00030000o`80o`000P000?l@0?l00080003o1@3o00020000
o`800?l01@000?l>ooooo`80oh001@00o`020?n000800?l00P3oP009003o00<0oh001@00o`020?n0
00@00?l00`3oP002003o0080oh002`00o`020?n000D00?l00P000?l20?l000<0003o0P3oP002003o
00`0003o0P3o00040000o`030?n000000?l0003o0080oh000P000?l20?n000<00?l00P3oP009003o
00<0oh000P00o`020?n000L00?l01`000?l20?n000800?l03_ooool40?n00003003o00000?l0003o
0080003o103oP007003o00D0oh00103o00070000o`800?l000<0o`000000o`000?l00P3o00070000
o`@00?l00P000?l30?l00080003o0P3o000G0000o`80o`000P000?l00`00o`00003o0000o`02003o
00T0003o0P3o00020?n000030?l000000?l0003o00800?l00`000?l80?n00003003o00000?l0003o
00/0003o3_ooool2003o00<0oh000P00o`020?l000D0oh001000o`070?n000800?l00P3oP0000`00
o`00oh000?n00002003o00@0oh0000<00?l00?n0003oP0000P3oP002003o0180oh000P00o`000`3o
P0000?l0003o00040?n00003003o003oP000oh0000800?l01`3oP002003o0080003o0`3oP002003o
0080o`000P00o`020?l00003003o003oP000oh0000H0oh0000<0003o0?n0003oP0000P3oP002003o
00`0oh000P00o`020?n000800?l01@3oP00Hooooo`005_ooool20000o`800?l00`3o00020000o`L0
o`0010000?l20?l000030000o`3o0000o`0000L0o`0010000?l<0?l000800?l02@3o00020000oa@0
o`0000<0003o0?l0003o00001P3o00000`000?l0o`000?l000020000o`@0o`000`000?l20?l00080
003o2P3o00080000o`D0o`000P000?l3003o0080003o0P3o000>ooooo`80oh002@00o`020?n000X0
0?l00P3oP00=003o00030?n00000o`000?l000d00?l00P3oP0000`00o`00oh000?n000090000o`80
oh001@00o`060000o`030?l000000?l0003o0080003o0P3o00050000o`L0oh000P00o`020?n000@0
0?l000<0oh00003o0000o`000P3o00020?n00003003o003oP000oh0000T00?l01`000?l40?n000ko
oooo0P3o00030?n000@00?l02@3oP002003o0080oh0000<00?l00000o`000?l00P3o0002003o00D0
003o0P3o0002003o00<0003o1P3o00030000o`800?l02@000?l20?l000T0003o00<0o`000000o`00
0?l04P000?l2003o00D0003o0P3o00020000o`D0o`000P3oP004003o00030000o`3oP000oh000080
oh000P00o`020?n00003003o00000?l0003o00H0003o0`3o00020000o`kooooo1000o`0:0?n000@0
0?l01`3oP002003o0080oh0000<00?l00?n0003oP0001`00o`020?n000@00?l02`3oP0000`00o`00
oh000?n000040?n000800?l000<0oh00003o0000o`00103oP0000`00o`00oh000?n00002003o00h0
oh001000o`020?n00080003o00<0o`000?n0003oP0001P3oP0000`000?l0o`000?l000020?n000L0
0?l01`3oP006003o00D0oh006?ooool001Kooooo0P000?l2003o00<0o`000P000?l70?l000@0003o
0P3o00000`000?l0o`000?l000070?l000@0003o303o0002003o00T0o`000P000?lD0?l000030000
o`3o0000o`0000H0o`0000<0003o0?l0003o00000P000?l40?l000<0003o0P3o00020000o`X0o`00
20000?l50?l00080003o0`00o`020000o`80o`003_ooool20?n000T00?l00P3oP00:003o0080oh00
3@00o`000`3oP0000?l0003o000=003o0080oh0000<00?l00?n0003oP0002@000?l20?n000D00?l0
1P000?l00`3o0000003o0000o`020000o`80o`001@000?l70?n000800?l00P3oP004003o00030?n0
0000o`000?l00080o`000P3oP0000`00o`00oh000?n00009003o00L0003o103oP00>ooooo`80o`00
0`3oP004003o00T0oh000P00o`020?n00003003o00000?l0003o0080o`000P00o`050000o`80o`00
0P00o`030000o`H0o`000`000?l2003o00T0003o0P3o00090000o`030?l000000?l0003o0180003o
0P00o`050000o`80o`000P000?l50?l00080oh001000o`000`000?l0oh000?n000020?n000800?l0
0P3oP0000`00o`00003o0000o`060000o`<0o`000P000?l>ooooo`@00?l02P3oP004003o00L0oh00
0P00o`020?n00003003o003oP000oh0000L00?l00P3oP004003o00/0oh0000<00?l00?n0003oP000
103oP002003o00030?n00000o`000?l000@0oh0000<00?l00?n0003oP0000P00o`0>0?n000@00?l0
0P3oP0020000o`030?l0003oP000oh0000H0oh0000<0003o0?l0003o00000P3oP007003o00L0oh00
1P00o`050?n001Sooooo000Fooooo`80003o0P00o`030?l00080003o1`3o00020000o`h0o`001000
0?l30?l00080003o1`3o0002003o02P0o`0040000?l:0?l000T0003o0P3o00070000o`@0o`003_oo
ool;003o0080oh002P00o`040?n000800?l00`3oP00K003o00/0003o1@00o`0;0000o`80o`000P00
0?l00`3o0000003o0000o`0;0?n000@00?l000<0oh00003o0000o`0010000?l00`3oP0000?l0003o
0006003o00030?n00000o`000?l000@0003o0P3o00000`000?l0oh000?n000020?n000kooooo0P00
o`030?n000800?l00P3oP004003o00<0003o1P3oP0000`00o`00003o0000o`05003o00@0003o0P3o
0002003o00L0003o0P3o00000`000?l0o`000?l000020?l000T0003o0P3o00090000o`030?l00000
0?l0003o00d0003o0`3o000=0000o`D0o`000P3oP004003o00D0oh000P00o`020?n00003003o0000
0?l0003o00H0003o00<0o`000000o`000?l00P000?l>ooooo`800?l02`3oP005003o00@0oh0000<0
0?l00?n0003oP0000P00o`090?n000<00?l00P3oP004003o00/0oh0000<00?l00?n0003oP0001`3o
P002003o00L0oh000P00o`070?n000800?l00P3o0002003o00<0oh000P3o00040?n00003003o003o
P000oh000080oh001000o`000`000?l00?l0003o00020?n000800?l03P3oP004003o00D0oh006?oo
ool001Kooooo0P000?l2003o00<0o`000P000?l70?l00080003o3P3o00040000o`<0o`000P000?l7
0?l000800?l0:03o000@0000o`X0o`002@000?l20?l000L0003o103o000>ooooo`/00?l00P3oP00:
003o00@0oh000P00o`030?n001/00?l02`000?l5003o00/0003o0P3o00020000o`030?l000000?l0
003o00/0oh001000o`000`3oP0000?l0003o00040000o`030?n00000o`000?l000H00?l000<0oh00
003o0000o`0010000?l20?l000030000o`3oP000oh000080oh003_ooool2003o00<0oh000P00o`02
0?n000@00?l00`000?l60?n00003003o00000?l0003o00D00?l010000?l20?l000800?l01`000?l2
0?l000030000o`3o0000o`000080o`002@000?l20?l000T0003o00<0o`000000o`000?l03@000?l3
0?l000d0003o1@3o00020?n000@00?l01@3oP002003o0080oh0000<00?l00000o`000?l01P000?l0
0`3o0000003o0000o`020000o`kooooo0P00o`0;0?n000D00?l0103oP0000`00o`00oh000?n00002
003o00T0oh000`00o`020?n000@00?l02`3oP0000`00o`00oh000?n000070?n000800?l01`3oP002
003o00L0oh000P00o`020?l000800?l00`3oP0020?l000@0oh0000<00?l00?n0003oP0000P3oP004
003o00030000o`00o`000?l00080oh000P00o`0>0?n000@00?l01@3oP00Hooooo`005_ooool20000
o`800?l01`3o00020?n000<0o`000P000?l>0?l000@0003o0`3o00020000o`L0o`000P000?lX0?l0
0180003o1`3o00090000o`<0o`000P000?l40?l000030000o`3o0000o`000080o`003_ooool;003o
00D0oh002@00o`020?n000800?l00`3oP006003o00030?n00000o`000?l000T00?l00P3oP007003o
00/0003o0P00o`0>0000o`80o`000P000?l30?l000@00?l00`3oP0020000o`@0oh000P00o`000`3o
P0000?l0003o00040000o`030?n00000o`000?l000H00?l000<0oh00003o0000o`0010000?l20?l0
0003003o003oP000oh000080oh003_ooool40000o`030?l00000o`000?l00080oh001000o`030000
o`80o`000P00o`020?n00003003o00000?l0003o00@0o`0000<00?l00000o`000?l00P000?l20?n0
00<0003o0P00o`060000o`03003o003o0000o`000080oh0010000?l30?n000h0003o103o000O0000
o`030?l0003oP000oh0000800?l00`000?l20?n000H00?l00`000?l20?l000@0003o00<00?l00000
o`000?l00P3o000>ooooo`800?l02`3oP0000`00o`00oh000?n00004003o0080oh0000<00?l00?n0
003oP0000P00o`090?n000L00?l0503oP002003o0100oh0000<00?l00?n0003oP0000P00o`020?n0
00800?l00`3oP002003o00/0oh001@00o`0D0?n000800?l000<0oh00003o0000o`000P3oP00Hoooo
o`005_ooool20000o`800?l01`3o00020?n000<0o`000P000?l>0?l000@0003o0`3o00020000o`L0
o`000P000?lX0?l00180003o1`3o00090000o`<0o`000P000?l40?l000030000o`3o0000o`000080
o`003_ooool;003o00D0oh002@00o`020?n000800?l00`3oP006003o00030?n00000o`000?l000T0
0?l00P3oP007003o00/0003o0P00o`0>0000o`80o`000P000?l30?l000@00?l00`3oP0020000o`@0
oh000P00o`000`3oP0000?l0003o00040000o`030?n00000o`000?l000H00?l000<0oh00003o0000
o`0010000?l20?l00003003o003oP000oh000080oh003_ooool40000o`030?l00000o`000?l00080
oh001000o`030000o`80o`000P00o`020?n00003003o00000?l0003o00@0o`0000<00?l00000o`00
0?l00P000?l20?n000<0003o0P00o`060000o`03003o003o0000o`000080oh0010000?l30?n000h0
003o103o000O0000o`030?l0003oP000oh0000800?l00`000?l20?n000H00?l00`000?l20?l000@0
003o00<00?l00000o`000?l00P3o000>ooooo`800?l02`3oP0000`00o`00oh000?n00004003o0080
oh0000<00?l00?n0003oP0000P00o`090?n000L00?l0503oP002003o0100oh0000<00?l00?n0003o
P0000P00o`020?n000800?l00`3oP002003o00/0oh001@00o`0D0?n000800?l000<0oh00003o0000
o`000P3oP00Hooooo`005_ooool20000o`800?l00`3o00020?n00080o`000P3oP0030?l00080003o
1@3o00020000o`T0o`000P000?l30?l000800?l0:@3o00020000o`L0o`0010000?l20?l000L0003o
0P3o00050000o`D0o`0020000?l30?l00080003o2@3o000>ooooo`/00?l01@3oP009003o0080oh00
0P00o`030?n000H00?l000<0oh00003o0000o`003`00o`000`3oP0000?l0003o00090000o`@00?l0
5@000?l4003o0080oh001@000?l50?n000800?l010000?l00`3oP0000?l0003o0002003o00T0oh00
0P00o`020?n00003003o003oP000oh000080oh003_ooool40000o`030?l00000o`000?l00080003o
1000o`000`3o0000003o0000o`020?l000800?l00P3oP0030000o`@0o`001@000?l20?n000h0003o
0P3oP005003o0080oh0010000?l30?l000L0003o103o00020000o`<0o`006P000?l30?n000800?l0
0`000?l20?n000H00?l00`000?l60?l00003003o00000?l0003o0080003o3_ooool=0?n000<00?l0
0P3oP002003o0080oh0000<00?l00?n0003oP0000P00o`050?n000800?l01@3oP002003o0100oh00
1000o`020?n000800?l04`3oP002003o0080oh000P00o`000`3oP0000?l0003o0002003o0080oh00
0P00o`090?n000<00?l00P3oP005003o0080oh001@000?l60?n000800?l00`3oP002003o01Sooooo
000Fooooo`80003o0P00o`0C0?l00080003o?`3o00020000o`030?l000000?l0003o0080003o0P3o
00070000o`D0o`000P000?l90?l000@0003o0`3o00020000o`T0o`003_ooool;003o00<0oh003`00
o`000`3oP0000?l0003o0002003o0080oh000P00o`000`3oP0000?l0003o000=003o00<0oh000P00
o`070000o`80oh001000o`0<0000o`80o`001`000?l4003o0080oh001@000?l50?n000800?l01000
0?l00`3oP0000?l0003o0004003o00<0oh000P00o`020?n000800?l01`3oP00>ooooo`@0oh000`00
o`020000o`80oh000P00o`030000o`80o`000P00o`020?n000<0003o0P3o000@0000o`80o`001@00
0?l20?n000@00?l000<0oh00003o0000o`000P000?l90?l000<00?l010000?l20?l000<0oh000P00
o`0F0000o`<0o`001000o`030000o`80oh001000o`030000o`80o`000P000?l40?l00003003o0000
0?l0003o0080003o3_ooool2003o00`0oh000P00o`020?n000800?l00P3oP0000`00o`00oh000?n0
0002003o0080oh0000<00?l00?n0003oP0003`3oP0000`00o`00oh000?n000070?n000800?l0103o
P002003o00<0oh000P00o`0B0?n000800?l000<0oh00003o0000o`000P00o`050?n000H00?l00P3o
P003003o00@0oh0000<00?l00?n0003oP0000P3oP0040000o`L0oh000`00o`040?n001Sooooo000F
ooooo`80003o0P00o`0C0?l00080003o?`3o00020000o`030?l000000?l0003o0080003o0P3o0007
0000o`D0o`000P000?l90?l000@0003o0`3o00020000o`T0o`003_ooool;003o00<0oh003`00o`00
0`3oP0000?l0003o0002003o0080oh000P00o`000`3oP0000?l0003o000=003o00<0oh000P00o`07
0000o`80oh001000o`0<0000o`80o`001`000?l4003o0080oh001@000?l50?n000800?l010000?l0
0`3oP0000?l0003o0004003o00<0oh000P00o`020?n000800?l01`3oP00>ooooo`@0oh000`00o`02
0000o`80oh000P00o`030000o`80o`000P00o`020?n000<0003o0P3o000@0000o`80o`001@000?l2
0?n000@00?l000<0oh00003o0000o`000P000?l90?l000<00?l010000?l20?l000<0oh000P00o`0F
0000o`<0o`001000o`030000o`80oh001000o`030000o`80o`000P000?l40?l00003003o00000?l0
003o0080003o3_ooool2003o00`0oh000P00o`020?n000800?l00P3oP0000`00o`00oh000?n00002
003o0080oh0000<00?l00?n0003oP0003`3oP0000`00o`00oh000?n000070?n000800?l0103oP002
003o00<0oh000P00o`0B0?n000800?l000<0oh00003o0000o`000P00o`050?n000H00?l00P3oP003
003o00@0oh0000<00?l00?n0003oP0000P3oP0040000o`L0oh000`00o`040?n001Sooooo000Foooo
o`80003o1`3o00020000o``0o`000P000?lB0?l000800?l0303o0002003o0200o`001P000?l30?l0
00@0003o1@3o00020000o`T0o`0010000?l30?l00080003o2@3o000>ooooo`/00?l00P3oP00@003o
00<0oh000P00o`020?n000800?l000<0oh00003o0000o`003@00o`030?n000@00?l01@000?l20?n0
00@00?l00`000?l60?l00003003o00000?l0003o00T0003o0P00o`020000o`D00?l00P3oP0020000
o`<0oh000P00o`040000o`030?n00000o`000?l000/00?l010000?l50?n000kooooo0P000?l;003o
00030000o`00o`000?l00080003o103oP0030000o`80o`006`000?l30?n000@0003o0P3o00090000
o`03003o00000?l0003o00/0003o0P00o`020000o`030?l000000?l0003o00d0003o0`3o00090000
o`@00?l00P000?l00`3o0000003o0000o`020?l000@0003o00<00?l00000o`000?l00P000?l>oooo
o`800?l01`3oP002003o0080oh0000<00?l00?n0003oP0000P3oP002003o0080oh000`00o`040?n0
0003003o003oP000oh0000800?l02@3oP005003o00T0oh000P00o`070?n000@00?l00P3oP0000`00
o`00oh000?n000060?n00003003o003oP000oh0000L0oh001000o`020?n000800?l00`3oP004003o
0080oh0000<00?l00?n0003oP0000P3oP003003o00@0oh001@000?l20?n000800?l00P3oP003003o
00@0oh006?ooool001Kooooo0P000?l70?l00080003o303o00020000oa80o`000P00o`0<0?l00080
0?l0803o00060000o`<0o`0010000?l50?l00080003o2@3o00040000o`<0o`000P000?l90?l000ko
oooo2`00o`020?n001000?l00`3oP002003o0080oh000P00o`000`3oP0000?l0003o000=003o00<0
oh001000o`050000o`80oh001000o`030000o`H0o`0000<00?l00000o`000?l02@000?l2003o0080
003o1@00o`020?n00080003o0`3oP002003o00@0003o00<0oh00003o0000o`002`00o`040000o`D0
oh003_ooool20000o`/00?l000<0003o003o0000o`000P000?l40?n000<0003o0P3o000K0000o`<0
oh0010000?l20?l000T0003o00<00?l00000o`000?l02`000?l2003o0080003o00<0o`000000o`00
0?l03@000?l30?l000T0003o1000o`020000o`030?l000000?l0003o0080o`0010000?l00`00o`00
003o0000o`020000o`kooooo0P00o`070?n000800?l00P3oP0000`00o`00oh000?n000020?n00080
0?l00P3oP003003o00@0oh0000<00?l00?n0003oP0000P00o`090?n000D00?l02@3oP002003o00L0
oh001000o`020?n00003003o003oP000oh0000H0oh0000<00?l00?n0003oP0001`3oP004003o0080
oh000P00o`030?n000@00?l00P3oP0000`00o`00oh000?n000020?n000<00?l0103oP0050000o`80
oh000P00o`020?n000<00?l0103oP00Hooooo`005_ooooli0?l000800?l0803o00080000o`030?l0
00000?l0003o0080003o0P3o00030000o`/0o`000P000?l50?l000@0003o1`3o000>oooooad00?l0
00<0oh000000o`000?l00P000?l20?n001800?l00P3oP005003o00D0003o0P3oP004003o00030000
o`3o0000o`000080o`000P000?l20?l00003003o00000?l0003o00T0003o0P3oP0020000o`<00?l0
103oP005003o00H0003o00<0oh00003o0000o`002`00o`040000o`<0oh000P00o`0>ooooo`80003o
1@3oP002003o0080oh000P00o`050000o`@0oh0000<00?l00000o`000?l00P00o`0M0000o`030?n0
00000?l0003o00d0003o00<00?l00000o`000?l0103o00070000o`@00?l000<0o`000000o`000?l0
0P00o`070000o`@0oh000P000?l3003o00L0003o1000o`0@0000o`kooooo1000o`0>0?n000800?l0
0P3oP005003o0080oh0000<00?l00?n0003oP0001`3oP002003o0080oh000P00o`020?n00003003o
003oP000oh0000@0oh000P00o`0<0?n000800?l02`3oP0000`00o`00oh000?n000070?n000800?l0
103oP002003o00D0oh001000o`080?n000800?l01P3oP0000`00o`00oh000?n00002003o00D0oh00
0P3o00020?n001Sooooo000FooooocT0o`000P00o`0P0?l000P0003o00<0o`000000o`000?l00P00
0?l20?l000<0003o2`3o00020000o`D0o`0010000?l70?l000kooooo7@00o`000`3oP000003o0000
o`020000o`80oh004P00o`020?n000D00?l01@000?l20?n000@00?l000<0003o0?l0003o00000P3o
00020000o`80o`0000<00?l00000o`000?l02@000?l20?n00080003o0`00o`040?n000D00?l01P00
0?l00`3oP0000?l0003o000;003o00@0003o0`3oP002003o00kooooo0P000?l50?n000800?l00P3o
P002003o00D0003o103oP0000`00o`00003o0000o`02003o01d0003o00<0oh000000o`000?l03@00
0?l00`00o`00003o0000o`040?l000L0003o1000o`000`3o0000003o0000o`02003o00L0003o103o
P0020000o`<00?l01`000?l4003o0100003o3_ooool4003o00h0oh000P00o`020?n000D00?l00P3o
P0000`00o`00oh000?n000070?n000800?l00P3oP002003o0080oh0000<00?l00?n0003oP000103o
P002003o00`0oh000P00o`0;0?n00003003o003oP000oh0000L0oh000P00o`040?n000800?l01@3o
P004003o00P0oh000P00o`060?n00003003o003oP000oh0000800?l01@3oP0020?l00080oh006?oo
ool001KoooooG@3o00040000o`80o`002P000?lB0?l00080003o1`3o00020000o`kooooo0P3oP00L
003o00@0003o0P3oP00B003o0080oh000`00o`070000o`80oh000P00o`020?n000<0003o0P3o0007
0000o`80oh001`000?l40?n000<00?l0103oP005003o00H0003o1@3oP002003o0080oh001@00o`04
0000o`<0oh000P00o`0>ooooo`D0oh000P00o`040?n000800?l000<0003o003o0000o`000P3oP005
003o0080oh001000o`000`000?l0o`000?l000020?l000D0003o0P3o00040000o`800?l00`000?l4
0?n000800?l000<0oh00003o0000o`003@000?l00`00o`00o`000?l000040?l000D0003o0P00o`02
0?n000800?l030000?l4003o0080oh000`00o`070000o`80oh000P00o`0@0000o`kooooo1000o`0B
0?n000<00?l00P3oP003003o00@0oh000P00o`020?n000<00?l01P3oP0000`00o`00oh000?n00004
0?n000800?l000<0oh00003o0000o`003P3oP002003o00H0oh0000<00?l00?n0003oP0001`3oP008
003o00030?n00000o`000?l00080oh001000o`0@0?n00003003o003oP000oh0000800?l0103oP000
0`000?l0o`000?l000020?n001Sooooo000FoooooaL0o`000P000?lF0?l00003003o003o0000o`00
02H0o`000P000?l50?l000/0003o0P3o00000`000?l0o`000?l000090?l00080003o3P3o00020000
o`kooooo0P3oP00E003o0080oh001@00o`040000o`80oh004P00o`020?n000<00?l02@000?l2003o
0080o`0030000?l20?n000L0003o103oP003003o00@0oh001@00o`060000o`D0oh000P00o`030?n0
00800?l010000?l20?l000D0oh003_ooool20000o`<0oh000P00o`040?n000800?l000<0003o003o
0000o`000P3oP004003o00D0oh000P00o`000`000?l0o`000?l000020?l00080003o0P00o`030?l0
00@0oh000P00o`030000o`80o`000P3oP005003o00L0003o1P3o00000`000?l0o`000?l000070000
o`80o`0010000?l2003o00h0003o0P00o`020?l000D00?l01@000?l20?n000@00?l03P000?l>oooo
o`@0oh0000<00?l00?n0003oP0000P00o`0=0?n00003003o003oP000oh0000@0oh0000<00?l00?n0
003oP0000P00o`020?n000D00?l01`3oP002003o00L0oh000P00o`0D0?n000<00?l03@3oP004003o
00030?n00000o`000?l000L0oh000P3o00020?n000800?l000<0o`000?n0003oP0001P3oP0000`00
o`00oh000?n00002003o00D0oh000P3o00020?n001Sooooo000FoooooaL0o`000P000?lF0?l00003
003o003o0000o`0002H0o`000P000?l50?l000/0003o0P3o00000`000?l0o`000?l000090?l00080
003o3P3o00020000o`kooooo0P3oP00E003o0080oh001@00o`040000o`80oh004P00o`020?n000<0
0?l02@000?l2003o0080o`0030000?l20?n000L0003o103oP003003o00@0oh001@00o`060000o`D0
oh000P00o`030?n000800?l010000?l20?l000D0oh003_ooool20000o`<0oh000P00o`040?n00080
0?l000<0003o003o0000o`000P3oP004003o00D0oh000P00o`000`000?l0o`000?l000020?l00080
003o0P00o`030?l000@0oh000P00o`030000o`80o`000P3oP005003o00L0003o1P3o00000`000?l0
o`000?l000070000o`80o`0010000?l2003o00h0003o0P00o`020?l000D00?l01@000?l20?n000@0
0?l03P000?l>ooooo`@0oh0000<00?l00?n0003oP0000P00o`0=0?n00003003o003oP000oh0000@0
oh0000<00?l00?n0003oP0000P00o`020?n000D00?l01`3oP002003o00L0oh000P00o`0D0?n000<0
0?l03@3oP004003o00030?n00000o`000?l000L0oh000P3o00020?n000800?l000<0o`000?n0003o
P0001P3oP0000`00o`00oh000?n00002003o00D0oh000P3o00020?n001Sooooo000Fooooo`@0o`00
00<0oh00003o0000o`008P3o00020000o`@0o`0000<0003o0?l0003o00006`3o00020000o`/0o`00
00<0003o0?l0003o00000P3o00040000o`030?l000000?l0003o0080003o103o00000`000?l0o`00
0?l000050?l00080003o0P3o00020000o``0o`0010000?l>ooooo`80oh005000o`030?n000D00?l0
10000?l20?n000800?l000<0oh00003o0000o`001000o`020?n00003003o003oP000oh000080oh00
0P00o`020?n000<00?l02`000?l2003o00X0003o0P3o00020?n00080003o0P3o00030000o`@0oh00
1@00o`020?n000800?l00P3oP005003o00L0oh000P00o`030?n00080o`001P000?l50?n000kooooo
0P000?l30?n000H00?l00P3oP003003o0080oh001000o`030000o`80oh000P00o`000`000?l00?l0
003o00040000o`800?l00`3o00040?n000800?l000<0oh000000o`000?l0103o0005003o00@0003o
303o000=0000o`800?l04P000?l00`3oP0000?l0003o0004003o00<0003o1P3oP0070000o`80o`00
1@000?l>ooooo`@0oh0000<00?l00?n0003oP0000P00o`050?n000800?l0103oP003003o00H0oh00
2P00o`020?n000800?l00P3oP005003o00L0oh000P00o`0T0?n000@00?l000<0oh00003o0000o`00
0P3oP0020?l00080oh0000<0003o0?l0003o0000103oP0030?l000@0oh000`00o`020?n000800?l0
0P3oP002003o00030000o`3o0000o`000080oh006?ooool001Kooooo103o00000`3oP0000?l0003o
000R0?l00080003o103o00000`000?l0o`000?l0000K0?l00080003o2`3o00000`000?l0o`000?l0
00020?l000@0003o00<0o`000000o`000?l00P000?l40?l000030000o`3o0000o`0000D0o`000P00
0?l20?l00080003o303o00040000o`kooooo0P3oP00D003o00<0oh001@00o`040000o`80oh000P00
o`000`3oP0000?l0003o0004003o0080oh0000<00?l00?n0003oP0000P3oP002003o0080oh000`00
o`0;0000o`800?l02P000?l20?l00080oh000P000?l20?l000<0003o103oP005003o0080oh000P00
o`020?n000D00?l01`3oP002003o00<0oh000P3o00060000o`D0oh003_ooool20000o`<0oh001P00
o`020?n000<00?l00P3oP004003o00<0003o0P3oP002003o00030000o`00o`000?l000@0003o0P00
o`030?l000@0oh000P00o`000`3oP000003o0000o`040?l000D00?l010000?l<0?l000d0003o0P00
o`0B0000o`030?n00000o`000?l000@00?l00`000?l60?n000L0003o0P3o00050000o`kooooo103o
P0000`00o`00oh000?n00002003o00D0oh000P00o`040?n000<00?l01P3oP00:003o0080oh000P00
o`020?n000D00?l01`3oP002003o02@0oh001000o`000`3oP0000?l0003o00020?n00080o`000P3o
P0000`000?l0o`000?l000040?n000<0o`00103oP003003o0080oh000P00o`020?n000800?l000<0
003o0?l0003o00000P3oP00Hooooo`005_ooool20000o`80o`0000<0oh00003o0000o`00<@3o0000
0`000?l0o`000?l0000B0?l00080003o403o00020000o`<0o`0010000?l40?l00003003o003o0000
o`0000D0o`000P000?l=0?l00080003o00<0o`000000o`000?l00P000?l>ooooo`80oh005000o`00
0`3oP0000?l0003o0004003o00<0oh000P000?l50?n000800?l02`3oP002003o0080oh001@00o`09
0000o`<00?l02`000?l4003o00<0003o1P3oP005003o0080oh001`00o`020000o`L0oh000P00o`02
0?n000030?l000000?l0003o00L0003o103oP00>ooooo`80003o0`3oP006003o0080oh0000<00?l0
0?n0003oP0000P3o0004003o00030000o`3oP000oh0000@00?l01`000?l2003o00030?l000000?l0
003o0080oh001@00o`040000o`@00?l000<0oh00003o0000o`001`000?l2003o0080003o1@3o000=
0000o`80o`004P000?l00`3oP0000?l0003o00020?l000800?l00`000?l20?l000800?l00P3oP000
0`00o`00003o0000o`0;0000o`kooooo2@3oP002003o00<0oh000P00o`0=0?n00003003o003oP000
oh0000@00?l00`3oP008003o01h0oh001@000?l20?n000800?l02`3oP0020000o`030?n00000o`00
0?l00080oh001@3o00060?n000<0o`00103oP002003o00<0oh000P00o`020?n000800?l000<0003o
0?l0003o00000P3oP00Hooooo`005_ooool20000o`80o`0000<0oh00003o0000o`00<@3o00000`00
0?l0o`000?l0000B0?l00080003o403o00020000o`<0o`0010000?l40?l00003003o003o0000o`00
00D0o`000P000?l=0?l00080003o00<0o`000000o`000?l00P000?l>ooooo`80oh005000o`000`3o
P0000?l0003o0004003o00<0oh000P000?l50?n000800?l02`3oP002003o0080oh001@00o`090000
o`<00?l02`000?l4003o00<0003o1P3oP005003o0080oh001`00o`020000o`L0oh000P00o`020?n0
00030?l000000?l0003o00L0003o103oP00>ooooo`80003o0`3oP006003o0080oh0000<00?l00?n0
003oP0000P3o0004003o00030000o`3oP000oh0000@00?l01`000?l2003o00030?l000000?l0003o
0080oh001@00o`040000o`@00?l000<0oh00003o0000o`001`000?l2003o0080003o1@3o000=0000
o`80o`004P000?l00`3oP0000?l0003o00020?l000800?l00`000?l20?l000800?l00P3oP0000`00
o`00003o0000o`0;0000o`kooooo2@3oP002003o00<0oh000P00o`0=0?n00003003o003oP000oh00
00@00?l00`3oP008003o01h0oh001@000?l20?n000800?l02`3oP0020000o`030?n00000o`000?l0
0080oh001@3o00060?n000<0o`00103oP002003o00<0oh000P00o`020?n000800?l000<0003o0?l0
003o00000P3oP00Hooooo`005_ooool20?n00080o`0000<00?l00?l0003o00004P3o00020000oaL0
o`000P3oP002003o02h0o`002@000?lF0?l00080003o00<0o`000000o`000?l00P000?l>ooooo`80
oh005`00o`020?n000800?l02P3oP002003o00/0oh002@00o`020?l00080003o1@3oP003003o00/0
003o1000o`070000o`80oh001@00o`040?n000800?l00`3oP0020000o`L0oh000P00o`020?n00003
003o00000?l0003o00H0003o1@3oP00>ooooo`80003o0P3o00000`3oP0000?l0003o00040000o`80
oh0000<0003o0?l0003o00000P3o0004003o00<0oh001000o`070000o`800?l01@000?l2003o0080
oh000`00o`020?l000800?l00`000?l6003o00T0003o00<00?l00000o`000?l03@000?l20?l00180
003o00<0oh000000o`000?l00P3o00030000o`80o`000P000?l2003o0080003o00<0o`000000o`00
0?l01P000?l00`00o`00003o0000o`020000o`kooooo103oP0000`00o`00oh000?n000070?n00080
0?l06@3oP004003o0100oh001000o`0>0?n000<0003o0P3oP002003o00@0oh000`00o`020?l00080
oh000P000?l00`3oP0000?l0003o00060?l000L0oh0000<00?l00?l0003o0000103oP002003o00L0
oh000`00o`020?l00080oh006?ooool001Kooooo0P3oP0020?l00003003o003o0000o`000180o`00
0P000?lG0?l00080oh000P00o`0^0?l000T0003o5P3o00020000o`030?l000000?l0003o0080003o
3_ooool20?n001L00?l00P3oP002003o00X0oh000P00o`0;0?n000T00?l00P3o00020000o`D0oh00
0`00o`0;0000o`@00?l01`000?l20?n000D00?l0103oP002003o00<0oh000P000?l70?n000800?l0
0P3oP0000`00o`00003o0000o`060000o`D0oh003_ooool20000o`80o`0000<0oh00003o0000o`00
10000?l20?n000030000o`3o0000o`000080o`001000o`030?n000@00?l01`000?l2003o00D0003o
0P00o`020?n000<00?l00P3o0002003o00<0003o1P00o`090000o`03003o00000?l0003o00d0003o
0P3o000B0000o`030?n000000?l0003o0080o`000`000?l20?l00080003o0P00o`020000o`030?l0
00000?l0003o00H0003o00<00?l00000o`000?l00P000?l>ooooo`@0oh0000<00?l00?n0003oP000
1`3oP002003o01T0oh001000o`0@0?n000@00?l03P3oP0030000o`80oh000P00o`040?n000<00?l0
0P3o00020?n00080003o00<0oh00003o0000o`001P3o00070?n00003003o003o0000o`0000@0oh00
0P00o`070?n000<00?l00P3o00020?n001Sooooo000Fooooo`80oh000P3o00000`3oP000o`000?l0
000B0?l00080003o4P3o00030000o`80o`000P3oP00;0?l00080003o6P3o00020000o`80o`000P00
0?l20?l000T0003o4`3o0002003o00D0003o0P3o00020000o`kooooo103oP00E003o0080oh000P00
o`0:0?n000800?l01`3oP002003o0080oh001`00o`070000o`80oh001@00o`020000o`800?l01@00
0?l20?l000L00?l00P000?l2003o0080oh001@00o`040?n000800?l01`3oP0000`00o`00oh000?n0
00020?n000@00?l000<0o`000000o`000?l010000?l70?n000kooooo0P3o00020000o`030?n00000
o`000?l000@0003o0`3oP0040?l000@00?l00`3oP007003o0080o`0010000?l00`00o`00003o0000
o`020000o`800?l00P3oP003003o0080o`000P00o`030000o`P00?l01@000?l20?l00003003o0000
0?l0003o00d0003o0`3o00060000o`800?l00`000?l20?l00080003o0P3oP0000`00o`00003o0000
o`020?l00080003o0`3o00020000o`800?l02`000?l00`00o`00003o0000o`020000o`kooooo0P3o
P003003o00/0oh000P00o`0;0?n000<00?l01P3oP0000`00o`00oh000?n00004003o0080oh0000<0
0?l00?n0003oP000403oP002003o00@0oh0000<00?l00?n0003oP000103oP0050000o`800?l0103o
P003003o00L0oh000P00o`060?l000P0oh000P3o0002003o00T0oh001000o`050?l001Sooooo000F
ooooo`800?l00P3o00000`000?l0o`000?l0000B0?l00080003o5@3o00020?n000@0o`000P000?l:
0?l00080003o2`3o00020000o`h0o`000P000?l20?l000030000o`3o0000o`0000H0003o703o0002
0000o`kooooo0P00o`030?n001@00?l03P3oP004003o0080oh000P000?l00`3oP0000?l0003o0002
0?n000D00?l02@000?l20?n000@00?l02P000?l4003o0080oh000`00o`060000o`80o`0000<0003o
003o0000o`00103oP003003o0080oh000P00o`070?n000@00?l01`3o00070?n000kooooo0P000?l2
0?l000030?n00000o`000?l000@0003o0P3oP003003o0080oh001000o`030?n000L00?l00P000?l2
0?l00080003o00<00?l00000o`000?l01P000?l30?l00080oh000P00o`030000o`80oh001P00o`05
0000o`80oh0000<00?l00000o`000?l03`000?l00`00o`00003o0000o`020000o`@00?l00`000?l2
0?l000L0oh000P00o`020000o`030?l000000?l0003o0080003o0P00o`020?l000030000o`3o0000
o`000080003o0P3o00070000o`kooooo0P3oP005003o00T0oh000P00o`0<0?n000800?l01P3oP000
0`00o`00oh000?n000020?n000800?l00P3oP0000`00o`00oh000?n0000?0?n000<00?l0103oP000
0`00o`00oh000?n000020?n000800?l00P000?l00`00o`00003o0000o`02003o00T0oh000P00o`02
0?n000030?l0003oP000oh0000T0oh000P3o00020?n00003003o003o0000o`0000800?l02@3oP005
003o0080o`000P3oP00Hooooo`005_ooool2003o0080o`0000<0003o0?l0003o00004P3o00020000
oaD0o`000P3oP0040?l00080003o2P3o00020000o`/0o`000P000?l>0?l00080003o0P3o00000`00
0?l0o`000?l000060000oa`0o`000P000?l>ooooo`800?l00`3oP00D003o00h0oh001000o`020?n0
0080003o00<0oh00003o0000o`000P3oP005003o00T0003o0P3oP004003o00X0003o1000o`020?n0
00<00?l01P000?l20?l000030000o`00o`000?l000@0oh000`00o`020?n000800?l01`3oP004003o
00L0o`001`3oP00>ooooo`80003o0P3o00000`3oP0000?l0003o00040000o`80oh000`00o`020?n0
00@00?l00`3oP007003o0080003o0P3o00020000o`03003o00000?l0003o00H0003o0`3o00020?n0
00800?l00`000?l20?n000H00?l01@000?l20?n00003003o00000?l0003o00l0003o00<00?l00000
o`000?l00P000?l4003o00<0003o0P3o00070?n000800?l00P000?l00`3o0000003o0000o`020000
o`800?l00P3o00000`000?l0o`000?l000020000o`80o`001`000?l>ooooo`80oh001@00o`090?n0
00800?l0303oP002003o00H0oh0000<00?l00?n0003oP0000P3oP002003o0080oh0000<00?l00?n0
003oP0003`3oP003003o00@0oh0000<00?l00?n0003oP0000P3oP002003o0080003o00<00?l00000
o`000?l00P00o`090?n000800?l00P3oP0000`3o0000oh000?n000090?n00080o`000P3oP0000`00
o`00o`000?l00002003o00T0oh001@00o`020?l00080oh006?ooool001Kooooo0P00o`030?l00080
003o1P3o00000`00o`00o`000?l0000V0?l00080003o5@3o00040000oa80o`001@000?l20?l000<0
003o7@3o000>ooooo`D00?l00P3oP00@003o00/0oh001000o`0:0?n000800?l00P3oP005003o00T0
003o0P3oP002003o00/0003o1`3oP003003o00H0003o0P00o`030000o`@0oh001`00o`030000o`@0
oh001@00o`080?l000D0oh003_ooool20000o`80oh000`00o`040000o`80oh002P00o`020?n000@0
0?l000<0003o003o0000o`000P000?l2003o00/0003o00<0o`000000o`000?l00P000?l20?l000<0
003o0P3oP004003o00<0003o0P3o00020000o`80oh0000<00?l00000o`000?l050000?l2003o00T0
003o0P00o`000`3oP0000?l0003o0002003o00`0003o0P3o00020000o`80o`001`000?l>ooooo`80
oh001@00o`090?n000800?l0303oP002003o0080oh000P00o`070?n000800?l00P3oP0000`00o`00
oh000?n0000?0?n000<0003o3@3oP0000`00o`00o`000?l00002003o00T0oh000P00o`0;0?n00003
003o003oP000oh0000L0oh001000o`070?n000800?l00P3oP0000`00o`00oh000?n00002003o01So
oooo000Fooooo`800?l00`3o00020000o`H0o`0000<00?l00?l0003o00009P3o00020000oaD0o`00
10000?lB0?l000D0003o0P3o00030000oad0o`003_ooool5003o0080oh004000o`0;0?n000@00?l0
2P3oP002003o0080oh001@00o`090000o`80oh000P00o`0;0000o`L0oh000`00o`060000o`800?l0
0`000?l40?n000L00?l00`000?l40?n000D00?l0203o00050?n000kooooo0P000?l20?n000<00?l0
10000?l20?n000X00?l00P3oP004003o00030000o`00o`000?l00080003o0P00o`0;0000o`030?l0
00000?l0003o0080003o0P3o00030000o`80oh001000o`030000o`80o`000P000?l20?n00003003o
00000?l0003o01@0003o0P00o`090000o`800?l000<0oh00003o0000o`000P00o`0<0000o`80o`00
0P000?l20?l000L0003o3_ooool20?n000D00?l02@3oP002003o00`0oh000P00o`020?n000800?l0
1`3oP002003o0080oh0000<00?l00?n0003oP0003`3oP0030000o`d0oh0000<00?l00?l0003o0000
0P00o`090?n000800?l02`3oP0000`00o`00oh000?n000070?n000@00?l01`3oP002003o0080oh00
00<00?l00?n0003oP0000P00o`0Hooooo`005_ooool2003o0080003oB@3o00020000oa@0o`001@00
0?l40?l000030000o`3o0000o`0001/0o`003_oooolF003o00P0oh000P00o`020?n000L00?l0103o
P002003o00030?n00000o`000?l00080oh000P00o`020?n000<00?l01`000?l20?n000800?l02`00
0?l50?n000800?l00P3oP0000`00o`00003o0000o`040000o`800?l00`000?l40?n000L00?l00`00
0?l20?n00080003o1000o`030000o`800?l00P000?l70?n000kooooo0P000?l30?n000H00?l00P3o
P0000`00o`00oh000?n000020?n000/00?l000<0003o003o0000o`000P000?l2003o0080003o00<0
o`000000o`000?l00P000?l20?l000800?l030000?l4003o0200003o0P00o`090000o`80oh001@00
o`020000o`030?l000000?l0003o00L0003o0P3o00020000o`80o`001`000?l>ooooo`D00?l04P3o
P002003o00D0oh000P00o`020?n000@00?l0303oP002003o0080oh001@00o`060?n000<0003o3@3o
P0000`00o`00o`000?l00002003o00T0oh000P00o`020?n000030000o`3oP000oh000080003o103o
P005003o0100oh002@00o`0Hooooo`005_ooool2003o0080003oB@3o00020000oa@0o`001@000?l4
0?l000030000o`3o0000o`0001/0o`003_oooolF003o00P0oh000P00o`020?n000L00?l0103oP002
003o00030?n00000o`000?l00080oh000P00o`020?n000<00?l01`000?l20?n000800?l02`000?l5
0?n000800?l00P3oP0000`00o`00003o0000o`040000o`800?l00`000?l40?n000L00?l00`000?l2
0?n00080003o1000o`030000o`800?l00P000?l70?n000kooooo0P000?l30?n000H00?l00P3oP000
0`00o`00oh000?n000020?n000/00?l000<0003o003o0000o`000P000?l2003o0080003o00<0o`00
0000o`000?l00P000?l20?l000800?l030000?l4003o0200003o0P00o`090000o`80oh001@00o`02
0000o`030?l000000?l0003o00L0003o0P3o00020000o`80o`001`000?l>ooooo`D00?l04P3oP002
003o00D0oh000P00o`020?n000@00?l0303oP002003o0080oh001@00o`060?n000<0003o3@3oP000
0`00o`00o`000?l00002003o00T0oh000P00o`020?n000030000o`3oP000oh000080003o103oP005
003o0100oh002@00o`0Hooooo`005_ooool2003o0080003o4P3o00000`000?l0o`000?l0001;0?l0
00@0003o103o00000`000?l0o`000?l000090?l00080003o1@3o00040000o`L0o`003_oooolF003o
00030?n00000o`000?l00080oh001@00o`020?n000T00?l00P3oP005003o0080oh000P000?l20?n0
00L00?l00`000?l20?n000800?l02`000?l00`3o0000oh000?n000020?n000800?l00P3oP0000`00
o`00003o0000o`040000o`800?l00`000?l40?n000L00?l00`000?l40?n000800?l00P3oP0030000
o`H00?l000<0003o003o0000o`000P00o`0>ooooo`D0oh001P00o`020?n000<00?l00P3oP0020000
o`<00?l00P3o00020000o`80o`000`000?l20?l000800?l05`000?l20?n000800?l080000?l50?l0
00H0003o0P3oP005003o0080003o00<0o`000000o`000?l02`000?l40?l000D0003o3_ooool90?n0
00800?l00P3oP0000`00o`00oh000?n000070?n000800?l0103oP003003o0080oh001000o`090?n0
0003003o003oP000oh0000/00?l0103oP0000`3o0000003o0000o`090?n000800?l00`3oP004003o
00@0oh0000<00?l00000o`000?l00P00o`0=0?n000D00?l05@3oP004003o01Sooooo000Fooooo`80
0?l00P000?lB0?l000030000o`3o0000o`0004/0o`0010000?l40?l000030000o`3o0000o`0000T0
o`000P000?l50?l000@0003o1`3o000>oooooaH00?l000<0oh00003o0000o`000P3oP005003o0080
oh002@00o`020?n000D00?l00P3oP0020000o`80oh001`00o`030000o`80oh000P00o`0;0000o`03
0?l0003oP000oh000080oh000P00o`020?n00003003o00000?l0003o00@0003o0P00o`030000o`@0
oh001`00o`030000o`@0oh000P00o`020?n000<0003o1P00o`000`000?l00?l0003o0002003o00ko
oooo1@3oP006003o0080oh000`00o`020?n00080003o0`00o`020?l00080003o0P3o00030000o`80
o`000P00o`0G0000o`80oh000P00o`0P0000o`D0o`001P000?l20?n000D00?l00P000?l00`3o0000
003o0000o`0;0000o`@0o`001@000?l>ooooo`T0oh000P00o`020?n00003003o003oP000oh0000L0
oh000P00o`040?n000<00?l00P3oP004003o00T0oh0000<00?l00?n0003oP0002`00o`040?n00003
0?l000000?l0003o00T0oh000P00o`030?n000@00?l0103oP0000`00o`00003o0000o`02003o00d0
oh001@00o`0E0?n000@00?l06?ooool001Kooooo0P00o`020000oeL0o`000P000?l20?l000@0003o
00<0o`000000o`000?l00P00o`0G0?l000@0003o1`3o000>ooooob000?l00P3oP002003o0080oh00
1@00o`020?n000D00?l00P3oP0020000o`80oh0000<00?l00?n0003oP0001000o`030000o`80oh00
1@00o`060000o`<0o`000P000?l4003o00<0003o103oP0020000o`<0oh000P00o`040?n000L00?l0
0`000?l90?n000800?l00P000?l2003o0080oh001@00o`0>ooooo`80oh001@3o0004003o0080oh00
0`00o`060000o`03003o003o0000o`0000D0o`000P000?l40?l001D0003o103oP002003o0180003o
00<00?l00000o`000?l01P000?l00`00o`00003o0000o`040?l000T0003o0`3oP004003o0080003o
00<0o`000000o`000?l03@000?l20?l000D0003o3_ooool20?n000800?l01@3oP002003o0080oh00
00<00?l00000o`000?l00P000?l;0?n000<00?l0103oP002003o00<0oh000P000?l40?n00003003o
003oP000oh0000800?l01`3oP002003o00@0o`00203oP002003o0080o`000P00o`050?n000800?l0
103oP0000`00o`00003o0000o`040?n000800?l0303oP002003o00D0oh000P00o`090?n000800?l0
0P3oP005003o01Sooooo000Fooooo`@00?l0<03o00020000obD0o`000P000?l70?l000@0003o5`3o
00020000o`@0o`000`000?l20?l000kooooo8000o`020?n000800?l00P3oP005003o0080oh001@00
o`020?n00080003o0P3oP0000`00o`00oh000?n00004003o00<0003o103oP003003o00@0oh000P00
o`030?l00080003o0P00o`050000o`T0oh000P00o`040?n000<00?l00P3oP002003o00`0oh001P00
o`020?n000D00?l03_ooool40000o`030?l000000?l0003o00@00?l00`3oP002003o00L0003o2`3o
000D0000o`800?l000<0003o0?n0003oP0000P3oP002003o00X0003o0P3o00040000o`80o`0000<0
0?l00000o`000?l010000?l20?l00003003o00000?l0003o00L0003o0P3o00040000o`<0oh001000
o`020000o`030?l000000?l0003o01@0003o3_ooool20?n000800?l02@3oP0000`00o`00003o0000
o`020000o`D0oh000P00o`040?n000<00?l0103oP002003o00D0oh000P00o`020?n00003003o003o
P000oh0000@00?l01@3oP0020000o`80o`002P3oP002003o0080oh000P3o00020?n00003003o003o
P000oh0000H0oh0000<00?l00000o`000?l0103oP002003o00<0oh000P00o`020?l000D0oh000P00
o`050?n000800?l0103oP0000`00o`00oh000?n000020?n000800?l00P3oP0000`00o`00oh000?n0
00020?n001Sooooo000Fooooo`@00?l0<03o00020000obD0o`000P000?l70?l000@0003o5`3o0002
0000o`@0o`000`000?l20?l000kooooo8000o`020?n000800?l00P3oP005003o0080oh001@00o`02
0?n00080003o0P3oP0000`00o`00oh000?n00004003o00<0003o103oP003003o00@0oh000P00o`03
0?l00080003o0P00o`050000o`T0oh000P00o`040?n000<00?l00P3oP002003o00`0oh001P00o`02
0?n000D00?l03_ooool40000o`030?l000000?l0003o00@00?l00`3oP002003o00L0003o2`3o000D
0000o`800?l000<0003o0?n0003oP0000P3oP002003o00X0003o0P3o00040000o`80o`0000<00?l0
0000o`000?l010000?l20?l00003003o00000?l0003o00L0003o0P3o00040000o`<0oh001000o`02
0000o`030?l000000?l0003o01@0003o3_ooool20?n000800?l02@3oP0000`00o`00003o0000o`02
0000o`D0oh000P00o`040?n000<00?l0103oP002003o00D0oh000P00o`020?n00003003o003oP000
oh0000@00?l01@3oP0020000o`80o`002P3oP002003o0080oh000P3o00020?n00003003o003oP000
oh0000H0oh0000<00?l00000o`000?l0103oP002003o00<0oh000P00o`020?l000D0oh000P00o`05
0?n000800?l0103oP0000`00o`00oh000?n000020?n000800?l00P3oP0000`00o`00oh000?n00002
0?n001Sooooo000Fooooo`@00?l08P3o00000`000?l0o`000?l0000T0?l00080003o503o00050000
oaL0o`000P000?l90?l000kooooo2`00o`020?n001000?l000<0oh00003o0000o`001P3oP00<003o
0080oh000P000?l20?n00003003o00000?l0003o00@00?l00`000?l20?n000800?l00`000?l20?n0
00@00?l00P000?l00`3o0000003o0000o`02003o0080003o0`00o`060?n000800?l000<0oh00003o
0000o`00403oP004003o00<0oh001P00o`020?n00003003o003oP000oh0000800?l03_ooool40000
o`030?n000000?l0003o00L0oh000P00o`090000o`@0o`006@000?l70?n000800?l030000?l2003o
0080003o0P3o00070000o`80o`0000<00?l00000o`000?l02@000?l20?l00080003o0`3oP006003o
00030?l000000?l0003o01@0003o3_ooool2003o00L0oh000P00o`020?n00003003o00000?l0003o
0080003o1@3oP004003o00T0oh000P00o`000`3oP0000?l0003o00060?n00003003o003oP000oh00
0080oh001000o`000`3oP0000?l0003o00020000o`80o`001@3oP0020?l00080oh0000<00?l00?n0
003oP0001P00o`030?n000800?l0103oP0000`3o0000003o0000o`040?n000800?l00`3oP002003o
0080o`000`3oP004003o00D0oh000P00o`040?n000L00?l00P3oP0000`00o`00oh000?n000020?n0
01Sooooo000Fooooo`@00?l08P3o00000`000?l0o`000?l0000T0?l00080003o503o00050000oaL0
o`000P000?l90?l000kooooo2`00o`020?n001000?l000<0oh00003o0000o`001P3oP00<003o0080
oh000P000?l20?n00003003o00000?l0003o00@00?l00`000?l20?n000800?l00`000?l20?n000@0
0?l00P000?l00`3o0000003o0000o`02003o0080003o0`00o`060?n000800?l000<0oh00003o0000
o`00403oP004003o00<0oh001P00o`020?n00003003o003oP000oh0000800?l03_ooool40000o`03
0?n000000?l0003o00L0oh000P00o`090000o`@0o`006@000?l70?n000800?l030000?l2003o0080
003o0P3o00070000o`80o`0000<00?l00000o`000?l02@000?l20?l00080003o0`3oP006003o0003
0?l000000?l0003o01@0003o3_ooool2003o00L0oh000P00o`020?n00003003o00000?l0003o0080
003o1@3oP004003o00T0oh000P00o`000`3oP0000?l0003o00060?n00003003o003oP000oh000080
oh001000o`000`3oP0000?l0003o00020000o`80o`001@3oP0020?l00080oh0000<00?l00?n0003o
P0001P00o`030?n000800?l0103oP0000`3o0000003o0000o`040?n000800?l00`3oP002003o0080
o`000`3oP004003o00D0oh000P00o`040?n000L00?l00P3oP0000`00o`00oh000?n000020?n001So
oooo000Fooooo`@00?l0<03o0002003o02d0o`002@000?lE0?l00080oh001`3o000>oooooad00?l0
00<0oh00003o0000o`001P3oP00<003o0080oh000P000?l20?n00003003o00000?l0003o0080003o
0P00o`030000o`80oh000P00o`030000o`80oh001000o`020000o`03003o00000?l0003o00H00?l0
00<0o`000000o`000?l01000o`0E0?n000@00?l01@3oP004003o00L0oh003_ooool40000o`030?n0
00000?l0003o0080oh000P000?l30?n000800?l02@000?l40?l001T0003o0P00o`030?n000@00?l0
30000?l2003o0080003o0P3o00070000o`80o`0030000?l20?l00080003o1@3oP005003o01H0003o
3_ooool2003o00<0oh000P00o`0@0?n000H00?l04P3oP0000`00o`00oh000?n000090?n000800?l0
0P3o000<0?n000@00?l00`3o00020?n000800?l0103oP0000`00o`00oh000?n0000;0?n000800?l0
0`3oP004003o00/0oh000`00o`020?n000800?l00P3oP003003o0080oh006?ooool001Kooooo1000
o`0`0?l000800?l0;@3o00090000oaD0o`000P3oP0070?l000kooooo7@00o`000`3oP0000?l0003o
00060?n000`00?l00P3oP0020000o`80oh0000<00?l00000o`000?l00P000?l2003o00<0003o0P3o
P002003o00<0003o0P3oP004003o0080003o00<00?l00000o`000?l01P00o`000`3o0000003o0000
o`04003o01D0oh001000o`050?n000@00?l01`3oP00>ooooo`@0003o00<0oh000000o`000?l00P3o
P0020000o`<0oh000P00o`090000o`@0o`006@000?l2003o00<0oh001000o`0<0000o`800?l00P00
0?l20?l000L0003o0P3o000<0000o`80o`000P000?l50?n000D00?l05P000?l>ooooo`800?l00`3o
P002003o0100oh001P00o`0B0?n00003003o003oP000oh0000T0oh000P00o`020?l000`0oh001000
o`030?l00080oh000P00o`040?n00003003o003oP000oh0000/0oh000P00o`030?n000@00?l02`3o
P003003o0080oh000P00o`020?n000<00?l00P3oP00Hooooo`005_ooool4003o00`0o`000P000?mA
0?l000T0003o4`3o00020000o`T0o`003_oooolM003o00030?n00000o`000?l000H0oh003000o`02
0?n000800?l00P3oP0000`00o`00003o0000o`04003o00<0003o0P3oP002003o00<0003o0P3oP004
003o00D0003o1P00o`030000o`800?l00P3oP002003o00<0003o1P3oP003003o0080oh000P00o`03
0?n000@00?l0403oP00>ooooo`@0003o00<0oh000000o`000?l010000?l30?n000T0003o0P00o`02
0?n000800?l000<0003o003o0000o`005P000?l30?l00080oh001000o`0<0000o`800?l05`000?l4
0?l00080003o1@3oP005003o01H0003o3_ooool2003o00<0oh000P00o`020?n000800?l0303oP006
003o0180oh0000<00?l00?n0003oP0003@3oP002003o00L0oh0000<00?l00?n0003oP0002@3oP002
003o01L0oh001P00o`0>0?n000800?l00P3oP0000`00o`00oh000?n000020?n001Sooooo000Foooo
o`800?l0H@3o00090000oaT0o`0000<00?l00?n0003oP0000P3o000>oooooaT00?l0103oP003003o
00H0oh004`00o`020000o`800?l030000?l20?l000800?l00P3o00050000o`@0o`000P00o`000`3o
0000003o0000o`02003o0080oh000P00o`030000o`@0oh000`00o`040000o`D0oh001000o`020?n0
0003003o003oP000oh0000/0oh003_ooool40000o`030?n000000?l0003o00@0003o0P3oP0000`00
0?l00?l0003o00070000o`@0oh000P00o`000`000?l00?l0003o000H0000o`<0o`001000o`0<0000
o`800?l05`000?l2003o00@0003o1@3oP005003o00P0003o00<00?l00000o`000?l02`000?l>oooo
o`@00?l000<0oh00003o0000o`000P3oP002003o00T0oh000P00o`000`3oP0000?l0003o0004003o
00D0oh000P00o`090?n000<00?l0103oP004003o00030?n00000o`000?l000@0oh000P00o`030?n0
00800?l00P3oP005003o00L0oh000P00o`090?n000800?l0303oP002003o00L0oh001000o`0;0?n0
0003003o003oP000oh000080oh006?ooool001Kooooo0P00o`1Q0?l000T0003o6@3o00000`00o`00
oh000?n000020?l000kooooo6@00o`040?n000<00?l01P3oP00C003o0080003o0P00o`0<0000o`80
o`000P00o`020?l000D0003o103o0002003o00030?l000000?l0003o00800?l00P3oP002003o00<0
003o103oP003003o00@0003o1@3oP004003o0080oh0000<00?l00?n0003oP0002`3oP00>ooooo`@0
003o00<0oh000000o`000?l010000?l20?n000030000o`00o`000?l000L0003o103oP002003o0003
0000o`00o`000?l001P0003o0`3o0004003o00`0003o0P00o`0G0000o`800?l010000?l50?n000D0
0?l020000?l00`00o`00003o0000o`0;0000o`kooooo1000o`000`3oP0000?l0003o00020?n00080
0?l02@3oP002003o00030?n00000o`000?l000@00?l01@3oP002003o00T0oh000`00o`040?n000@0
0?l000<0oh00003o0000o`00103oP002003o00<0oh000P00o`020?n000D00?l01`3oP002003o00T0
oh000P00o`0<0?n000800?l01`3oP004003o00/0oh0000<00?l00?n0003oP0000P3oP00Hooooo`00
5_ooool4003o00D0o`000P000?lC0?l00080003o@`3o000:0000oa@0o`000P000?l20?l000030?n0
003o0000o`000080o`003_oooolI003o00@0oh000`00o`060?n000h00?l01@3oP0020000o`800?l0
30000?l20?l000800?l02@000?l20?l000800?l000<0o`000000o`000?l010000?l2003o00<0003o
103oP003003o00@0003o0`3oP009003o00H0oh000P00o`000`3oP0000?l0003o0002003o00kooooo
10000?l00`3oP000003o0000o`040000o`80oh000`00o`070000o`@0oh000P00o`000`3o00000?l0
003o000D0000o`800?l00`000?l20?l000@00?l030000?l2003o01L0003o0P00o`040000o`D0oh00
1000o`000`3o0000003o0000o`0D0000o`kooooo1000o`000`3oP0000?l0003o00020?n000800?l0
2@3oP002003o01L0oh000`00o`060?n000800?l000<0oh00003o0000o`003@3oP005003o00L0oh00
1000o`020?n00003003o00000?l0003o0080003o503oP0000`00o`00oh000?n000070?n00080003o
0P3oP002003o0080oh0000<00?l00?n0003oP0000P3oP00Hooooo`005_ooool4003o00D0o`000P00
0?lC0?l00080003o@`3o000:0000oa@0o`000P000?l20?l000030?n0003o0000o`000080o`003_oo
oolI003o00@0oh000`00o`060?n000h00?l01@3oP0020000o`800?l030000?l20?l000800?l02@00
0?l20?l000800?l000<0o`000000o`000?l010000?l2003o00<0003o103oP003003o00@0003o0`3o
P009003o00H0oh000P00o`000`3oP0000?l0003o0002003o00kooooo10000?l00`3oP000003o0000
o`040000o`80oh000`00o`070000o`@0oh000P00o`000`3o00000?l0003o000D0000o`800?l00`00
0?l20?l000@00?l030000?l2003o01L0003o0P00o`040000o`D0oh001000o`000`3o0000003o0000
o`0D0000o`kooooo1000o`000`3oP0000?l0003o00020?n000800?l02@3oP002003o01L0oh000`00
o`060?n000800?l000<0oh00003o0000o`003@3oP005003o00L0oh001000o`020?n00003003o0000
0?l0003o0080003o503oP0000`00o`00oh000?n000070?n00080003o0P3oP002003o0080oh0000<0
0?l00?n0003oP0000P3oP00Hooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oo
oooooooooon2ooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2
ooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooo
oooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;o
oooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003ooooo
ooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003oooooooooooooP_oo
ool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooo
oooooon2ooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2oooo
o`00oooooooooooooh;ooooo0000\
\>"],
  ImageRangeCache->{{{0, 639}, {967.688, 0}} -> {-0.100021, -0.152945, \
0.00657284, 0.00657284}, {{15.1875, 167.312}, {944.375, 790.875}} -> \
{-0.129866, -0.18615, 0.00690264, 0.00690264}, {{18.8125, 163.688}, {940.688, \
871.062}} -> {-0.901974, -42.331, 0.0372819, 1.49372}, {{18.8125, 163.688}, \
{864.125, 794.5}} -> {-12.4241, -58.7232, 0.558072, 0.558072}, {{167.312, \
319.5}, {944.375, 790.875}} -> {-1.18015, -0.18615, 0.00690264, 0.00690264}, \
{{170.938, 315.812}, {940.688, 871.062}} -> {-6.57349, -42.331, 0.0372819, \
1.49372}, {{170.938, 315.812}, {864.125, 794.5}} -> {-97.3209, -58.7232, \
0.558072, 0.558072}, {{319.5, 471.625}, {944.375, 790.875}} -> {-2.23043, \
-0.18615, 0.00690264, 0.00690264}, {{323.125, 468}, {940.688, 871.062}} -> \
{-12.2473, -42.331, 0.0372819, 1.49372}, {{323.125, 468}, {864.125, 794.5}} -> \
{-182.252, -58.7232, 0.558072, 0.558072}, {{471.625, 623.75}, {944.375, \
790.875}} -> {-3.28049, -0.18615, 0.00690264, 0.00690264}, {{475.25, \
620.125}, {940.688, 871.062}} -> {-17.9188, -42.331, 0.0372819, 1.49372}, \
{{475.25, 620.125}, {864.125, 794.5}} -> {-267.149, -58.7232, 0.558072, \
0.558072}, {{15.1875, 167.312}, {790.875, 637.312}} -> {-0.129834, -1.24583, \
0.00690229, 0.00690229}, {{18.75, 163.688}, {787.188, 717.562}} -> \
{-0.900809, -271.617, 0.0372819, 1.49372}, {{18.75, 163.688}, {710.562, \
640.938}} -> {-12.3996, -144.4, 0.557995, 0.557995}, {{167.312, 319.5}, \
{790.875, 637.312}} -> {-1.17946, -1.24521, 0.00689983, 0.00689983}, \
{{170.938, 315.812}, {787.188, 717.562}} -> {-6.57349, -271.617, 0.0372819, \
1.49372}, {{170.938, 315.812}, {710.562, 640.938}} -> {-97.3209, -144.422, \
0.558072, 0.558072}, {{319.5, 471.625}, {790.875, 637.312}} -> {-2.23029, \
-1.24583, 0.00690229, 0.00690229}, {{323.062, 468}, {787.188, 717.562}} -> \
{-12.2462, -271.617, 0.0372819, 1.49372}, {{323.062, 468}, {710.562, \
640.938}} -> {-182.204, -144.4, 0.557995, 0.557995}, {{471.625, 623.75}, \
{790.875, 637.312}} -> {-3.2803, -1.24583, 0.00690229, 0.00690229}, \
{{475.188, 620.125}, {787.188, 717.562}} -> {-17.9177, -271.617, 0.0372819, \
1.49372}, {{475.188, 620.125}, {710.562, 640.938}} -> {-267.089, -144.4, \
0.557995, 0.557995}, {{15.1875, 167.312}, {637.312, 483.812}} -> {-0.129866, \
-2.30569, 0.00690264, 0.00690264}, {{18.8125, 163.688}, {633.625, 564}} -> \
{-0.901974, -500.997, 0.0372819, 1.49372}, {{18.8125, 163.688}, {557.062, \
487.438}} -> {-12.4241, -230.086, 0.558072, 0.558072}, {{167.312, 319.5}, \
{637.312, 483.812}} -> {-1.18015, -2.30569, 0.00690264, 0.00690264}, \
{{170.938, 315.812}, {633.625, 564}} -> {-6.57349, -500.997, 0.0372819, \
1.49372}, {{170.938, 315.812}, {557.062, 487.438}} -> {-97.3209, -230.086, \
0.558072, 0.558072}, {{319.5, 471.625}, {637.312, 483.812}} -> {-2.23043, \
-2.30569, 0.00690264, 0.00690264}, {{323.125, 468}, {633.625, 564}} -> \
{-12.2473, -500.997, 0.0372819, 1.49372}, {{323.125, 468}, {557.062, \
487.438}} -> {-182.252, -230.086, 0.558072, 0.558072}, {{471.625, 623.75}, \
{637.312, 483.812}} -> {-3.28049, -2.30569, 0.00690264, 0.00690264}, \
{{475.25, 620.125}, {633.625, 564}} -> {-17.9188, -500.997, 0.0372819, \
1.49372}, {{475.25, 620.125}, {557.062, 487.438}} -> {-267.149, -230.086, \
0.558072, 0.558072}, {{15.1875, 167.312}, {483.812, 330.312}} -> {-0.129866, \
-3.36525, 0.00690264, 0.00690264}, {{18.8125, 163.688}, {480.125, 410.5}} -> \
{-0.901974, -730.284, 0.0372819, 1.49372}, {{18.8125, 163.688}, {403.562, \
333.938}} -> {-12.4241, -315.75, 0.558072, 0.558072}, {{167.312, 319.5}, \
{483.812, 330.312}} -> {-1.18015, -3.36525, 0.00690264, 0.00690264}, \
{{170.938, 315.812}, {480.125, 410.5}} -> {-6.57349, -730.284, 0.0372819, \
1.49372}, {{170.938, 315.812}, {403.562, 333.938}} -> {-97.3209, -315.75, \
0.558072, 0.558072}, {{319.5, 471.625}, {483.812, 330.312}} -> {-2.23043, \
-3.36525, 0.00690264, 0.00690264}, {{323.125, 468}, {480.125, 410.5}} -> \
{-12.2473, -730.284, 0.0372819, 1.49372}, {{323.125, 468}, {403.562, \
333.938}} -> {-182.252, -315.75, 0.558072, 0.558072}, {{471.625, 623.75}, \
{483.812, 330.312}} -> {-3.28049, -3.36525, 0.00690264, 0.00690264}, \
{{475.25, 620.125}, {480.125, 410.5}} -> {-17.9188, -730.284, 0.0372819, \
1.49372}, {{475.25, 620.125}, {403.562, 333.938}} -> {-267.149, -315.75, \
0.558072, 0.558072}, {{15.1875, 167.312}, {330.312, 176.75}} -> {-0.129834, \
-4.42477, 0.00690229, 0.00690229}, {{18.75, 163.688}, {326.625, 257}} -> \
{-0.900809, -959.57, 0.0372819, 1.49372}, {{18.75, 163.688}, {250, 180.375}} -> \
{-12.3996, -401.391, 0.557995, 0.557995}, {{167.312, 319.5}, {330.312, \
176.75}} -> {-1.17946, -4.42301, 0.00689983, 0.00689983}, {{170.938, \
315.812}, {326.625, 257}} -> {-6.57349, -959.57, 0.0372819, 1.49372}, \
{{170.938, 315.812}, {250, 180.375}} -> {-97.3209, -401.449, 0.558072, \
0.558072}, {{319.5, 471.625}, {330.312, 176.75}} -> {-2.23029, -4.42477, \
0.00690229, 0.00690229}, {{323.062, 468}, {326.625, 257}} -> {-12.2462, \
-959.57, 0.0372819, 1.49372}, {{323.062, 468}, {250, 180.375}} -> {-182.204, \
-401.391, 0.557995, 0.557995}, {{471.625, 623.75}, {330.312, 176.75}} -> \
{-3.2803, -4.42477, 0.00690229, 0.00690229}, {{475.188, 620.125}, {326.625, \
257}} -> {-17.9177, -959.57, 0.0372819, 1.49372}, {{475.188, 620.125}, {250, \
180.375}} -> {-267.089, -401.391, 0.557995, 0.557995}, {{15.1875, 167.312}, \
{176.75, 23.25}} -> {-0.129866, -5.48479, 0.00690264, 0.00690264}, {{18.8125, \
163.688}, {173.062, 103.438}} -> {-0.901974, -1188.95, 0.0372819, 1.49372}, \
{{18.8125, 163.688}, {96.5, 26.875}} -> {-12.4241, -487.113, 0.558072, \
0.558072}, {{167.312, 319.5}, {176.75, 23.25}} -> {-1.18015, -5.48479, \
0.00690264, 0.00690264}, {{170.938, 315.812}, {173.062, 103.438}} -> \
{-6.57349, -1188.95, 0.0372819, 1.49372}, {{170.938, 315.812}, {96.5, \
26.875}} -> {-97.3209, -487.113, 0.558072, 0.558072}, {{319.5, 471.625}, \
{176.75, 23.25}} -> {-2.23043, -5.48479, 0.00690264, 0.00690264}, {{323.125, \
468}, {173.062, 103.438}} -> {-12.2473, -1188.95, 0.0372819, 1.49372}, \
{{323.125, 468}, {96.5, 26.875}} -> {-182.252, -487.113, 0.558072, 0.558072}, \
{{471.625, 623.75}, {176.75, 23.25}} -> {-3.28049, -5.48479, 0.00690264, \
0.00690264}, {{475.25, 620.125}, {173.062, 103.438}} -> {-17.9188, -1188.95, \
0.0372819, 1.49372}, {{475.25, 620.125}, {96.5, 26.875}} -> {-267.149, \
-487.113, 0.558072, 0.558072}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Visualize overall classifications of test images", "Subsection"],

Cell[BoxData[
    \(\(resultsCombined = 
        Map[VisualizeResults[Times\  @@ #, \ colors, \ 
              DisplayFunction \[Rule] Identity] &, \ probs];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(gresultsCombined = 
        Map[Last, resultsCombined];\)\), "\[IndentingNewLine]", 
    \(\(gimages = \(ShowImage[#, 1, 255, 0, 
              DisplayFunction \[Rule] Identity] &\)\  /@ \ 
          testImages;\)\), "\[IndentingNewLine]", 
    \(\(Show[GraphicsArray[gimages], ImageSize \[Rule] 640, 
        DisplayFunction \[Rule] $DisplayFunction];\)\), \
"\[IndentingNewLine]", 
    \(\(Show[GraphicsArray[gresultsCombined], ImageSize \[Rule] 640, 
        DisplayFunction \[Rule] $DisplayFunction, 
        GraphicsSpacing \[Rule] 0];\)\)}], "Input"],

Cell[CellGroupData[{

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .11628 
%%ImageSize: 640 74.4186 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.238095 0.00276855 0.238095 [
[ 0 0 0 0 ]
[ 1 .11628 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .11628 L
0 .11628 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.00276855 0.245293 0.113511 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 0.003125 0 0.003125 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 .5 scale
320 string
320 160 8 [320 0 0 160 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
AB5E5E7B85BA6E395C7E86B74630526966B16E2B4E835AA6CC4E2A66AE8DAB85335961AF
5A803D356783556C7F19495D6C416373245069613B7E455F6E983347592E5A6443326A35
526684383C5243348DA6333F7B422E75BF432F484D518E862F435856669E642B56577469
993F35516664AF5527536D556CA43F21458654AAB735414E6756A8C339435B6A57AB7F36
4E6A7564CD564561A67A87D5643A797C68A0C72F506B9C97A5912D5E71C184C93242778E
9B5C91324A6695775CB5545075A0517D8D445861613177484F5474463653364F72784977
5A4A8AB34945633F7EC36C43714D6ABB933B786788B2BC545A8651A2722A576B5462B64C
317C8A54B2B5494A957176C96C4250787471A84941526172AA8E313B4D6D9AC369314572
8096D66A3A3F7A7461C2473E6A6F66A6CC4F2A6DA57799D3591C63995251B663
7B82765F4D848049763E45617065762A2590A04453491E3C95682F513759698A5137804B
52B36527426F327AB92D237093558FB22B2F756262D07D203D7F5170892B3E606057B12F
3462684F7A81214A40857C80B8243E4D76847FA430365F7B5171A21742445950BA832254
76725EA1273A798F4B545A283F71AB5F73403651987D67743D3656A27A5E87403E8BA654
5B704B45A9D04D676B423E8FAE3A8B543943957549543151629A61535A3078A07D3B588D
4260CB8F31647E6274DC673A6C786CC6E53835746D56CAA830666E6F8ACE43476C908CCF
9E27509E69B4B731368DBDA6B75B334DBB84BE5E356CA58A9D68445D7F4BAFA140478192
6B752B42669888C3BC5A4D96BF80B86F3E4F7D7F88AE3254589B6C81C13B47B8B4649AA3
4D519FAB506B36424BADA34677415155928137514F314D98AC5049693D6B705D
7672A0793D5A7F8B81442C4D6578A43F1D2E737C74501B28537598A64F3E6A7C7E9FB639
3B64A152A55F2D3C7279569769214D75756C87352158955668441C2F6F6347601B265381
484A5F225281482C514E4556943A2D6B52355392592B56566C87A25745703A5793843044
5E4F70A55A2B7D7B5582A633355E776294752949428479BF473B4778647DB85A1E5A9E6F
7697362B6FB67B888C30595A936DA86F356D84699DB12D3A676B6B90A12B5266A98388A4
3F5276C378C086464D9DDD739D415F76C2C86691436E5E956A7D833057A48774803050AB
C5708E98384E9C6D5E7F36628D964E7A3F64CD9F4E925E579EC15775784B6EBF4F3D6B47
87C7713893623471C4986C70665984B964617C417BD267456A6E5F8CD05A3B83797494D2
5E39779480CD69294971A167C572405172AB6DA569213B5B8F96917B1B2D4953
A9618DAD5DA0793CA0B37E702155657B427C46436F9C4082842D2FA7AD38626D4A77BA91
3E5B56347C85292E554E43958A23345E4B58B76031415266A49B2439484183A7352F4950
60958C2345595A52A0641D52707A51C1B7232B3F737EB6601F4B5C6D80993D1D4B714779
A2324666905EC755434D76926A76306F5B7B5A6B782E54689F4E728C335B697A59787647
4F80744A7C49546E805C69782C5E8082444B883B489C9C50667B2E60887E4C8E5F5678C9
7F408C59399CB4484065515CCA883D556D7892DC6B3C715D61B4CD372A725E86C24D2C4E
997AC5CE397BA86E8BE15C2A88777DB5484470AB6BC9534654C6779D7C4C5888AB86B436
47839D4EB3783741B28299A7365781A48C8FB13B43A58A7BBB8549778D9F669E50476798
9672A256418A8C3D49714D3B5DA75765642F5F737445616D3C3B91B43D2D6044
9C755D8968AC8B2E4C58899433315B5350C2781B5180468EAE421C62BE89B7913D84A497
6FC84B274A9B6D8485293B499765899328374C946F96374A4F7E6D844F314165525C8233
3D468369558E1D35679145407B182F6A9286669218258DA34559622232778B3C694E2955
AB59525B3036698D345448337287504986612E87C34F4176753E909C4149695D518D9D39
535A617490633B61725187D1553A7A7A5EA2CD523D656C6DBBA03A40799A75BD7E4373A6
D58EDA4145698EB2A7C83C51AC9C5DA39F3144ABA065B9503346B9A0836D394F64AB7771
7C3060BE9B70A9604689BF649C46549C86527652609B825579635B839E387160415BCC79
395F4E44999C3F427C5A72A0834166994C83C37C42705F5287A34249954D76CE8C31485C
6F8DC5783061994D7BBE403F4A7E59A8CE472D6DB2449CAE3F1F33965E6EB947
6D9D9C6546557354902E3468684B782F235B904A6E58192B90945F5D225363704A345B3B
6F7151353C4E2D7FB24D2E43463F86C94B3E6F526A9667204B5E338FB534355D3F5D7B4A
235E6250579B5F154D827382B8202A40686D9FDE21245566509DC125404C7462CD861E4A
7B536E6A28244B8C629C78262C788B6997614F515D7668A066397DA65262A24F57569D67
93A33B547D775C93953742B5B75E95614A55A6935C68414C619A606160214576A0648494
2D5DBA824D7E7B3A6FD06A3E72664497D2674C7449409EAB465F535597915362806657A8
9F39427B9A94A361506F888ECD5A427F9D6CB5543E94A696A86C3956A477BEA4293B8E93
6E92253462AB8FAA9C28495D7D89BE7B3F5B88A494B02F385A689190CF3A344FA7558677
3E51BAA2518E5F3849D2A7647A474256BF9D61814E294493AE536759283D7C7E
896673974E695B4E9B582F494E5D9D41252D604986902941518B86CB4D27656671ACAB29
34786D5B96722E31556D63B6513B4B778E82B733335E81427F7319419553636B1639514E
44696A143F497E428292283FAB5F408D772E3CBFA34794343640974D396A343C769F6052
5A3C487A5E5A76363E667B4A4E6C40487C6E374C62313E81C2597F8A495D7953445E505D
6CAF563D6C5148A6A53D417C686BB66C2D4652647F823436408E7991BE2B3A538769BEDE
4749608F84D998387DB09174AA4B3086AF7D97B937417C8881BA8D2742A9877FA64E404C
CF8B92922C6193917294414A85A060A2405B8E9371A14946A5B669A1793175A9528DA741
6FAB6B6485534560D27375754E498BCD5B8B6C4A6D9F6F3564535B7DC87B44726E72C3B7
4C5462895DB5B93B31805A80D56F3A43907B65A3B83240615E7EA58E22293C62
C45C718970A94C4950785C7826484F7D627840374E55445172315194BD61755B3E57AEAF
4067483052984B4162342868713271382F41978A355D5E497B94583E58467191332A4A49
4B7DBA6124494E30828B36254E543FA192382A458169CE663941547767B53D343F6F4E89
99214650697AB1313560624D7CBD4D45848B469BAE352D587D5E91BC3C476D8D8BA76442
675F8158BE5A2F6EBE8D808F464A60B2678D861C5B89875A7D6B374FA6A450955D334E97
A8649D2C43ABB07777BA4455C8A7596A82404FCAAC4E5C614A60BE58365C4C619E88435E
5C5EBAD1405C70567BBE5C4B796675CC8A3B867E87D6673460976EC0962E595B4E97D574
664F4784BC513C60717EA6B6425668AA65D5963E5671A17DBC67436292B999C8B23479D3
8E7BB4605253B39463CF593A5AA26F86942A4C69615F70C25A1D73BB4F4A5E50
92877E7E537386386245496F4F2B554651857720365A54677F371C659977AB8C42375D9F
84AC3D2A2C64A281C73732384E4FCEA01542545F83CA57204A7251B247204A5D566EB02E
2756739E6EC52E2130797684B2242B3B909C7FAA22324F796977852750546D53612E3874
885F5673194CA7B34F6664235C937E6A715C3C59A6603C683D51739C5148644688B08C3A
5980546DD2AD4041694C71C954416A5F6F97DA6536616B6688A03F325D6559AE8139444E
6A8BCB533E469B5FAEE34D4447AB6BACB33F5F629288B447476C8C808E742E355DA99ACC
893E568C7FCDB24A66897F88974B4796E790A869408FCB6B6C783E4D925BA558344E8E8D
4E8A2F49958B5F70514F9DB5965C9A484580AD53608B394F647B35605752445F9B554B55
757E9A7E37404F6B58CA6839514172ACDE762F56593763CD852A3C6D5661733F
778488A4504C8597B03F3E5D6D6FA335405A7251854E2E4277875B742F405C9671795928
6A877A466D41325C7E8948773E2B5888575E7F38367AA0438731244F844537483538624D
28425B433C6280356F4542477F60254F3B62607F4529443955938D2A31534A95C7571D3A
72668589223244766996682F4556798AB0572F4B5E7B79E077225DA173A2C5492B88B85D
A6A6503782D187B49C4D3C8BA47EBC40406F8092A6A7474257877270A2513F73CA919880
4054807A5F876B415DACA675A13E517E867C5285284DB09E4B48753277A14F44693B5597
B54E9C5A445FBB82758A417AB55539756F64CA8A455D6F75A58048478141BEA4374E7E79
79C3374580838D9B9037468A6D83AF35334D8A7996BE293F81884CA5B62B4362767CB2BE
2444736F63AD841A56B2AE649E6F2348CFD75A773F373C56A3657D6E2D534D69
A3516981949A354C7971485A3D2D4F6626403921426720233E40538F492C32484D6A8C2F
2A507148829124464A7C5CAF7D2C484D4F84D94C283F6750CC8728575E5576952226535B
4696B92139736674E1943437597F61B6883A4663A982B7721B5CB77B5C9A2D3F75A1497E
671C678368648B442B5D883F5A7D2336838F45795F2C3A7CA65D7B8E434A96A84B717237
62BE854357473562B96D565F514FA28B444C74664F835E334870577D893E3F74695290AE
40466F6A75AF9D395D8E92BFD68D2F6A9A8A98D1502B55708CA0A02B4D8C668EB83D296C
BB61DD983259A2A2B5CC5E61A3A778AD563DA2A664A7724174B092A07E3F7CA15A878138
417A7C557F2D3F8672616F7A4252A6BD639A89568BC9A44878554870DCB445A44E326FC4
9A588B60567FD5613A7A8875A98A262C5F8467B371223F69626470782D306E75
BD9559697AD973304468719B2E313B5A47967414485645779634387A9A70A454394C7E8E
5E6A2A383F765FA3A41D4150A15D684B2B3B7A9B5B601F3352964E4C3F2B357161403F2E
2C4AB0A13E8229317BA964467433337AAB3F2E6A303AA49333375E695E963D3545644E77
AC36406A5A5AA98828425940A1B92E2A4C7650C2C7343D5E715696CD383E538E5EA6843B
43639472BF70424F83C5B5B32F5E73AA93B49E39676788847A95285480815ADA4C4B6B6D
7665BA283E81B17BA2BA3E6792C598A7A1447A8A8B6C8E354687AA7A478831429D8E5857
5B428BD15B79743853A3884395664581A040918975AFA9374F5C4BA9B83B47785386B73F
494A418DCE52265B5B5C8FC0543B7E8575A5A74A4C96B084B25C3D4D7DD3AEE378454264
8777AC52474F839491BE3F3763AA5478B83F386E8B528BB44D3C5A726068603C
70A58C7F4885C25B872D3473963F64362654A8685B4D1E3A8452405B395281902D415F54
50927E20465D5982BE43323154528FB123404D5B64B37025557658749F21416548528129
123952837ED92E2139628768C02F173BA69A7EA429242A977B83811E4E60486168442753
866B6D6E3342558C577A7C33409255548B433564AD8F72AB33315F816458A1434D5E8D6E
5F7E2E475F983D6B5D43939257448C3E3773856F3F7E3C6786964836773F52818030526C
5472CC6E305F70557CE0554663726F78D037458E5793BF6728447670969C353D5A6182D5
922E71A662C2AE4141828A80C96262A8C061C2713F93BD657B683E57A56D7A71345DAB89
9062333C6EBB627536485986979AC63F556991956A9B3A576772547C683068679350716F
3F68829D5F6D562956D2A24374493848A890476746384F8593375365326B7857
637F978D494D95629C552937874F96522C435E609D9F1746596C6DB4392963997854981B
444B85585F4D243E3D506C7D472D3F6E6176B2342751A249877C2B3052685F831E2B817B
2D577925318E85407E42335D8E873F794D416A96632F5F325776AB452666506D864B334B
474B869534446562579D9140475E5A63AB4A3046535C7BB7512360865771D2842B917B63
8DE260375D6852A2BE404A6D7F7BDE5D395C7C92869D354078A5AA689838556EBA98B288
31495BA9749A87424A828062AF4856597D6F51AA3E3789B27AB8642A56C583687E2D2563
C55E87664061C89A75753B4688A550B9814ECFB85F9D864979AC4B4C626D798B33435B70
96A43946517D6B8A5A285162626CCC933D6B6B7A77C73A356F625390B42742A2906491AB
3337A8B66EB17D262E78B17BC47735585F808AC399285555595888B21F2F6F68
B958599B8BC6554A6DA44F5837345F69578B67344C6A3D315B3773BD7E3355496E83BD47
2B4948439E7E3A46553B4BA5A62D44483380EC85233A6A4694B8334046596ABB4F143B6F
485D7D2C2147744AAA8C1C295BB76EA58B37587AAA54A9354673B28A48802F5F8990509F
3F414E8B5C6D9E36334782837C7E295BAAAA4F8171295978BB56A791314E809860D27C3D
4E7D9D71C8384053AB98787E374C69AB509D583A4784943263683E4B76536D843A496D9D
5742833D6A99B8465C926060D47337606B6082A95231576164B3852D4E8198A8D95D2C45
9185C9B73C4179AD9FDC573559A488C05A38557C72B3843F4B885F7F7D2241725DA38837
615766A6A53F397BA2579B814D63A5D7699D82435A9DB465837149608A7A548A53426891
554B583B45528F47435947556C8F364A613A50609C382F69462B4FAF5C3D603B
9081516D58837B374C5B8E7C341F424536A48B2143885E7588354077C177856A3070A0AC
5D8C373F59A159A7A21D41458F728551303E529868A3383561897965501A5C9CC5587319
234E6757498F242D81875179A12C565864386C95264776642D713E4469873A4064343866
7A314F5A475094A24666683997BC3C305F72599EBB384A6A5D53B2D536385E554AD0BD31
454C7185D98132647179B4BA404A588369C8AD2C555A83688EA0255E76698AC2533D529F
849CC2372487CD82CADC51499FEB8EC09E364FA4B55BB03A425695689E8D457259C254C9
4D3F8CB16C7D71295799C355AC3A4FC7B9649F424C82E2739679587FBA5655494670AD53
573D496BA49E3E4D5365629970C4D5B9878FAC3F4C7765549992364D6C5969D6832E426E
6485C95E276462526ACB553E688A4F93BB3A4162617A67B14A334460627EB04E
8EBC8566507ACA6B9D35567067476B3D322A5A517445255658403D6F3F3F8660475E7C28
4D6351266A44475A9A322F4D4C4392AE283C424A57B19327655F457CA22E324E6E589535
1C32718C5FBB642439835B7ED81D3583875397D33B39539959CD79203A8E5DB79F2E3E45
7B6E7E8127516093679A8D2863BB90698E45396FA5815F702F4D86A5706CA04D4592A585
8F803B5497A85A9B574E5BAB8C7F71354877A9727D6F30494F9A606E513148649949898D
2F63D695397E68396BBA824B614F5797D8634A71494FAC963C604F4791C4413D626D7EC3
BA32579386C7A9323578BC7AD66938599372DD3B42798587C2843D93C17F7F4F3E64AB8B
B877234D6ABBA9974A2E68603A588E874A7D7C8E5DAA263B43A06A7F8532484F84547180
3D458EB6758A634C586D944B6848324D806F5CA9683D4160893D8C50364D667B
7F64919D5759595DB68731525745A1812556493296B023336E585CB361213A704987E955
236E894EBD6C3B528163779A40286E956A8B8D253A458D54BE753053606A648C1E334C53
32837C2536446C4A555A1F254D3749764635515E51347549306F902A674C3662A0852C84
4E3C4D714F405C3D3257A64F335F4E77A8592F47595DAFB661374D8382849C463A709D6F
97A8433E7EB262D95F38459C609FB0402C6BA461A6C34A317A9F6DBA882F3F5D905CA594
363F85A38DBC7F2757B49DC7DF6E3E5DADB0ACDB485478C290C04D3C718CA279B62D4D8A
CC84B77C3490D79582814D4075B46DBC2F42D3AA4DA4643C72CE789A8842968E42708461
8EA052556F4E516BB046417A2C3A599D8491AC8385C6883663604385C84B2C707071A3AF
3F38558C76AE652E48777B63BC532D38667450CC83303F53964CA899274C4C83
AB6C6EB981AF425689C65EAD2B4965A571D76C3D466D5E698E2C5395905382672358A7AE
61A41B404C7045626C2750835D3B594B4A7BAA6C3364354D8DA34336565F5A9A6B20453D
44768C2D253E5243779D36344E4561BB67233E7176728A422D4065469E833635518E51C8
84393262658BA749324E786564932B387EB574A78D4669A2886ACC8E3A5687835393644A
5D8E947773393D77CC7E8A8C4D4092B656A23D555497A15EA0655A669E6B6C963129767F
6F759D3136B7B85E89C4423EA2BF566559434CAB6B425755627C902C4E7E5673C38C2F5D
948BCEA6354F6E6879B03C4A6C6E73E5562B869475D6673F4D9074C0A72C7A7B658D9539
3C836F8F8C37404F905871C6F9ECDEEFE2EC9F3F6D70D674B95A465A678A62A34E365785
6476A2304062976296954151A7BE426363534182AF654A82403E4D7E4042593E
78A46C6C68BC6F3E9A4C567088356736286E8A32384D4D7E96402A5A775AAC9F29266396
7DE2532661714FA193282A65626DBB543049867E70A941306C93696E5D2853537A65A338
244C5E5B60AE39215C957C93A8223F729166B0892645A08050B27620459791627D273D50
935E5F6D2F4296A34B8D4E37408357384F41546FB0503D6E644F9FB140496D3F65D0C03C
718B3F6DD06E3B7B645E89C55B464E4E8DCD80304961565CA17C2F72737479BB331F6373
7497EB482779C0A8A6F44D5354A5A1AF84345B6B758CA9534994847AB7B5224B84C877CB
623786AF7796AE333FA48068973A3BC3C590A64A3AB1C760804D3A6EB57D785A3F759664
605E1F59817B387C464945454872705B8EBA974E56774A569A7B37576B5576A6952A655D
4C7CCA622860524279CF7C2E5A8868767935344F7E9E95B33E39515C63759E42
5EA6B7804A64A27CBD444267A36CC5314F4E6C5A8559133997637C904C3359BD776E7F41
49A0B45F723C4173B25C576F2F4794B3446B8C4447957E527864448CBF603B6124337B4B
224E445269A94A243363638BD6323860616396872B2C52665E8B89222D4A5E93A7462132
5D6879A6393A49827DB775295160585C9D4A235860696C83393181AF648CB947346EC15F
9BB7576390DA75C74942798480808B392F83B3587F66514982A94E6F604461A2BB568772
35535E86568C4D3A6392DA97B6723E90D46289823680BCB34B80633191C9744B584B3CBE
A434546C688BC26541BD5A7CC9643D688A8CB65030797B7BCA8B364C8163B1A01B366C89
AE8B2C3D67A5599F8C3C31393D2B4F857CACBA6DC2C73E4193834C8C8A21357B9A4BA959
3D3C8BB354913F294D80A86C984C465A66823E698A23609C7B4E7C7133577572
A75775B98787474E8691477E41426E843A4C49426E6F232F5B4E68B038223D5F6E8A8030
394F76719C53213D76427EAD2E3D45806A84A3275A858C72B86B1D3A7C91A79625366073
5F8C58343F7D8961AD742451B28B61AF462082BA665DB12D4065A7546984264368C85476
402A6B6E9255972F3394B35B507A2B55AF973E606B4D7DA15835575B4D6568654E665D56
85AD574763486A99994F6E835EAD9D604161573E94A83F46827D45AC8E30506A8E529CBB
2D4C7D5E5EC68C2D53609D98CB9E49789975CDB7353078A07ACC653D54A4AC9DC5463F50
AE8CA67B24518CA37CBC593A799A929D2D548F859E9F4D275CCD63B1514767974D5E7A22
56BE7E64803E464F66427481A8ACB183A6CB763B50896652803A3178B85844505642887B
3241623B5DB96327306E5F66B384294E55503396C82C1C65705A9582232D4B72
9E7D57679BAC6A4858507BBD433F3F6363BD763453647A697E193096C779AD8F254FA384
98B31D57727D698B7725568B87567F401E4C554E44792F4A7BA44F554433516678345F27
2D7A5C2D43724440B185353858335BB55F2749576668CF5D374E5A4A5E953F2E47578EB7
621E36815170C967183B87618AA8313E5D8366AA81344874947FAD703E7A8D6D7FA77C2E
59B98F99A9304370B782B994405690AA70B55F3D5979AA7BC6453F81B57A7D8224415398
696B8A2553599A4684892967609067857B2385E4A76587493780C1607580354E70A15286
424BBBA63C588B4965CF7D44906566BA6E439C5D87B95A3451825ABE8B2567644184BE33
2B7E587FB93558B1944580BB9C97B89C9CD98B3A61684D5EBB6023499F50616C2F3A62AE
6A8C803D3F5C806F82864B41779B4E81822D33338F8775A1383F3F6A5A687E2F
71A9A48755798F8EB83A44856E5D7F253463A35F693D4A8B93583D4B3E2E75A240353F46
A3A544355C3E3098C53837685060D8AB1D35565674CB571B529A78945631425D785BC146
11608E8695BD451B65A0718BA43F2F84A46CAB773743626B81BE6B30407C854D9A482D6B
845B569F432B6FAD6186744649A5714D9D442984A64D5B72376790A746556C475AB0B98E
6F753C62CE9E42845E429BA25E476B426D94853B4C625984C0A243426D678DC44A2B674A
3D9AB53A317378619BCF473A877362A9B7263FA5A593B644394C9966A58A2246557C78D2
6E2C76AB617AAC2C459FC170BD4B5A69B27CCC3744CBCB6CC67F514F9C54A45D3E7CA979
62712B37898664C84B3D283A596F57628B87893B68872E748777437C9B4761B465375849
4F80C3603C5A40476DA98D43514A49557F2F23484F5150A1682E5F5756629B4E
6E5F77A54D5A4B53AF4A3D57629BD5532C4C736FAA372A429E835B7631473FABA0B28516
4665766AA0383734735F6E76203548905D655D1E4A74786C853A30558A565A743C3969A1
476C5F284E4F7D3F685C2457A159455D45336DA54150652C3B69AE454359535D70923A42
744452996740536E477785353B5E4160A9622B507D4A9A933A3E4C916682A13C41598AA9
B2B43B38789D7FCA6F3C58967D92BB303A8FAE6CB9A1495389D4659A784374B9A355D36D
2860C58B68AE6D4286C6C069CE4D577DD17F7E824293C7BA6379504B94CD7752AD4C4CBC
AE587E78508FC8855B70434EA27C48795276C88A3B755066B9934578694FC6B62B53A96B
879623315E4F64AE6A1D1F233533393E316E9B6888BE3C4598B454A4C5574E7D79578C39
4B50878365BA472E5C4D7A5CB04629556F6154906F233F506754785E274F9C79
CA6763B7849F4B50748A98AE36336EAC8D8D4E4144763561644B317A68396D323E5B984C
3451414F928B335E4548629654345E4B5B88BC3D2C7C6B63B479313A505E79752C394178
597D7D283433889BAB9139394C6282963F254A5C5881CF3032406260A68F244D49847394
87265072574B863631638C63827E4B4277AF63A37731417D74557E7F435263725D824742
5989816BA14841A8CD64709B4B55C1C659755C495C9463577F5059677E42616D4B4AC1A1
3A40664372A566234A555959BB6F4053687CA7B73F317C8BA3CB7B2939896A8CE5822D4C
9C64D89C2A6E7C7299DE58297ABA869D3F3D8DA679A741516DC1859C6B2C61B670989530
42AD8F5F902B3D64874D6799EDD2AF5965A1593E5F6A4E54A863445D705F52B53F366A6D
3C60945C4E6962334D81485295BC3A54583F4887AE543C873A4E6B6638487A56
819557554FA278474F484D8E4A2D4738508C763247446A89AB38453F6474C1612B376576
A3A51F56606863B18D205A685D6CB62A444A6A7E89802633428481972A1F4D547475C517
286A826D78AD1D4D6D89827E992A61A98B5869772D43B99A636A393259B572326E3B3E59
87453A61405967612C43483A698E48304A435D6E8F383D504855AEAB314D4C6064C49B2D
4B626F86CD54424D897F99B8413D71A69ACA8F315C886774D77D3090AA4C798D362B7FCC
829685323093E16B9D863E5083AF96AD4E3777CC96678A39418CD9717B6B4C588EB66B86
5047BACA69A26D3764D0E075802F5D9A7A42785A72C95E42864A70AD98414A59406EB94C
3C625491A62D3B5E6151A1ACAFE8E9B99AAB513A6A875086BB764D688E6580E16B2A5B8F
4485823F42736A5180C64B3386B467A9792E2E565B5E81DD54368085436C7A50
69A1AE6B4A5575918D2442505A81A31D3F478F838B3C3C65A7984B5F163E5BAC62475132
546A5D51602845689936374D4249A6A43A575C36569B5D284A4E4E90952A30443260AF4C
184B757C78C5431C5B7C7089C12E1A4A8D6CA284323D547097B2463D576D8063C491265A
545984BC2E3C869756A1852E48809984CB4C3861906C7A81454352AF559D8A3D5B76B36F
938546507AAF7795404D499E905F71484D69B97D8DA1494055CE8A71803950A37D567877
506AC15A49545C4F95D86149617E74B0A02D54637472BC6F314D7B7690982939876F75C8
872462B55BB7AB2A2E628476A9293F737E78A53E46A28051CE53427DBA5681632F599994
7F8742506E62719A2844434F3F60644F7095AF56617A4353727C99A68F556A8B9F4C6348
47588E816B6B30384A82873D69482F686D4639456F2F3CA3984157683246A365
A653A2AC63804957AD6B415E2C36715A344A304B5F672342634379C135304C6D6C896D2E
48524B95C72B2F75A74D79933F4355617BAF921C435479869C35354B728C777915437991
5194592546899D57CF433B539C8C576F263B71875B4A432B58826B32585B264656823C55
712E72D5A94F5D4D2948784B58463A4990622F403F466992472D4F565D86B75639615A56
91B54F475F717B9D9E395F5F72BBB1553B574F3CA1CA4A4C5177A2BAB44851606A62C777
3B4974777AC88848458D92AFB5455D65A673B189478093B9819E6F416DC0C584CB654563
B95B958245478A5A707B3B4BAC7755BF4A44A3976C823B4B7C9D57714752AD9C3659773B
689D36616D495BA833233B49203B523557547D7B9FA92A4B6665518EB02A445A8A68A468
2B50668981B455224E66B970A97B3D47676975754D352F51955F82563B46577A
B17A5D9470C5463B6E826D863626547479782C2E5CA77FB0A353535B7A69973E424E8D47
687E26596E872A667F363A663F316E351D5DAA553F703B2B7193473F3B32468445285654
3F948E2B2D55403883732541525460A6361C417A3D669D21347B814D96811E52999B609E
7536315F8A549C4E27535B6983A82E3483B460B285305A6D7A7B99684551979573A43D3F
5D8A8C81962068689769A3812B58848570B55434718B97639A33378BB68663AB312C6DBD
6867722D3E799A455F602D71B46A4A71513B71B55A3F796D597AA333415A5F90C9714197
6E50BBBC335465588FBA3E356D715ED9703E747E8DCE4A49669088B4613663A360AF9840
4D8C7BAB913B637E7F567B634455564854C057385A7B6588BD383D5B836773A6455D6BA7
6A6B723A4860A44F5D81535960933F696B2E4C62624054632F38586B20435738
7AA276774F7B846C6B34505B55586F295A97553047464173AD3F29623D67B69C303D583A
7DB1223F544E31A0B7293C475C62AD662D466F7571822B4853706B694E175D7F7C488D2C
215CBAA96688214361976D8CA0244A4A7B448D654D4B5D5B5E7D2D4F51915F766E304A5F
8C4D62552C4593943B3D493750704F3E473046536F3E4462292F90963B45494A4DA3962F
4452635DCF6635635B6D88BE50396F575FB6B3333F7B7176CF8527509E6B72D03934468C
599BBC224661856AC7A6213F64795FB4673A61836A87A1514972877D8F832268A3A27AAD
822B73DF719F4F345BD597747129417FC46CAA386B879072B04F54AD9550625E417AC664
5D414B618C5344743B668CA0546B825E48B29A35636E4D65A37236694C5985A859344E53
4991C047303C644C6BA9313758735383892235517B527FAE2F2D4C7B4F807B3A
8D79858D4855797C9C2949575E76B33D2173A268A25A4536868E8EBE3A2B289285A0702F
385D576FA42F192E856652721E3240694C6F7E1D50A0682F724E26508C482E4E39495E38
283D563E65734A2F755E2F89DE683064645D95AB2F3C635469A54D2D5252589ACB32366E
705499B0321E519089795E27455A63769C33425C554D74AB2F3A5B6C4F8881334B528680
966F3B48B09A6BA44D3C6377B98699264A698F9671812C5870AF6F64774C5A7293507E41
3E519B5B56733E4067A45D678C3F497CB4413B6D4555A499374F593987CB6765A16068B4
98315C9C94A6AA3746707F9F983448756874DB5D4095656DD46C3D57785096633C608B76
AA5F2C59806491A9233D6A94588C68334173A450AA5D385385A0499C672D5090A4558A62
45458384545F273C498374556C3642658F6140924041657F70567F3A256A9160
C14E73957C903F58686441492B536D83543D23466599585161334584A83F5B36368EAC34
413B475DA4691C423A5E69C33829524F3297C0331D7E79498D461B33743D8778172A584A
54879A3B41556143B1873B435B877EC3333A539C6375A1313C63A065B92C414889A548A9
67374C938450764B2C70A743565D364A73673F73742B66C1A54E8E622F7AC17D588F5E4E
8FC2544E91663E92B038635E4E77A963336485378ECC5A386F745079C354387F567DE492
275781597CD2592B758D7199CF544E6884849FA238615BB5939C5337507476A1CD235A91
BE52B88F3695C279717A307EACA07AA5373C7C8D698B343955A85B784F315C7A4A8A7834
6C8A344D562F54C2764763683587A3524866436D96933741644B59A5B938406756569BA8
48458E617495672F5D6B6858916F234A746257A695203B906847AE5A28395957
8978627D6C9642397391656C2F2E425584A83F1E434C78B8C7274D4C6B7CC95F3678814D
8D8D11424C6647B176214D6D69547E3232526E9650782A457A925B5F3E1E60B783576F21
297576AC6DAF2D53516C446B6C3A5D636D337A3E3B449C5C465A317DC05633556E4C50C1
9E3330483976816831436167778831404B7266B2973F4C647856CE8B3949678B84BB534C
5F89845CA835575E777E9C72336C6A7A759B8A5B5494A37FC24E356275A57BC9483B66A3
567871403879AA60856C3B5F709150C1493774A07B6F89485C70B463618F4265CEA1619B
87458DD354555660A2A13C5D6E87ABC853406D4D7CD1703B67745FAE972B53645FB19235
5BA863859C2D398A85676F5D37598E8D659F4F3560B8565AB23A296CB96170D44E617291
6F92B3405B919347ACA85B4B608947613D2E464B8154636A2F4555822A3F6D44
829DB1574F6B74699C2F456747487132205E8D52773D303A7A653E6424515C8A3B525A4B
578D403471324381A33B32635F45A46127506E3875C85821596854AE58142057676BB52D
202B497972D42B3B6A7A71A4BE243C97744EC36A20557F718CAF3D376EBC7D969B4B386D
BD8B71721B4283705F86791B5C92565B6C2D4077A558485E273D4E8F495F743450718548
82812C738B733D804632A0BE404770314CAA933B52565772C6722E536051608A5C3D575D
6B9D99383650745EAAA82738459175D2753343586F9ECA733C49A091A0A6445A5D9F9DC6
6B446A886FD48A29408A89AA92224B7F6C8CC54E2B8BBA77C6523D4CB98D93693583C477
8F5D4D53816A7D823D57A7A73D6C7C5F5CA49D4D6D4B585EC0753B61583C6C967E345A54
4151C7923154604D6D9F593B4D6B597F8D2E3B554C5493B2392D445F2F6C9651
926F81985065695EA140415148559B301E30484093932D4453658CB32829418C5BC07C20
47514C79C13D2D65656483BA3934687F457E87262E798B5D87311D69A84C5A4E282B3C64
42AF45162E55824C5D4234778A5A4E6E3B29558C345F74282C7B6F3D5E282D435E502F57
4D3D8CB04834604B348C892F2D515D65B646283E765F78B6433865685092C04E427B7159
9BD6354E9D8576C7692560B4847D943F315ACF7A9F952266A3BE73CAA73C666B9B53C465
317A6E6368B82D3E75AD837E8F38508BBD567462326594C263B7444A68B0757C802B6E9C
AA53AD684790D26452564071BC6D436D3E6ED1813753585785873E494E71C19C2A596A6D
AFB13B578C617BC5722654A758839A383D5B9277B15A3247627C72BE9140355D9B7DA66E
3D3FB5C994BD3F304BA7B1759758213985827AB35931365B7C487F3C302F415D
B65F6E849BBC4665AA788F99264E867778954A2D669F638EA22E436478547F3A2C5C9843
396F252B8CA6324D3E20488F8A3D5E3C2C658F3342503958A7A021396F5B707831373C31
4BC3851D2D464D5BA3771E3D554472D25431494F5496DC3B3C4A5C85C57E2D544C5D6B9D
912243706458945E2E41635B93712A3E658C4F8B7134433F5E53906033425A7B54BB5B33
419183557E2B487285467C5931607B7036665B466A864142954058859B4D427A4D69907D
2E51705171AF6A24406F7273B36527539681A2912F4C667375AF8C405C9B6587AF265595
7E78CE5B326EA6A0AA6F2C6A916FA9A921418391859C484B8CCE6FAF5A2593CC5AA05B32
5DAA977F7B325789CD929D7142597E855DA43731839047529A59345DAD4C4F5C493F71A2
4A3E7D5D52917638534F50548472314759306ABDA34D5A4F463899A7253B5D68
997D90525D995B6E6E3F608D4D3A56353990552C425F4277BF583E69698DC66E22378657
9AC5253C517B69C1902147535766BD53315651769F821A3F4A675BA14C2037455C647B2B
1F53A45E767B252E4E895A766A214D959A4A71432A48968639661D578B825B587A352F69
A266404D34465491436C42257B9B43354E49408CB3564163423F82A12B4D765C72B19F45
565D84AFD2493C556D48B9A83345567B5ABC853859687387D54141719EA7A2C5333E70AD
5A917E3A6891B07593573D6D5FB893993D70949D585D7A4A4C7FB883748C449C8B7780C6
6137A7C44B7174678EAD525E72475DC87645564753B06F4A6A6F759C9D32668D508D982C
387E7689963E56687560B9A5404B587A5EAF73354A7D515DB0403038686082BD4D47527C
707BAC403D7E8741897A274D7B827D75562C3C52815D7B782B26488445479746
6A73C25C53736385C731469A8E8B9C2A364A7C7C98344041446E84A4332C5599736A4734
49524E406B31209198363D703026709A3A6B4D35469B4D334F2C407165293A3D346C8E31
1A3B383A95D13422554746A8C721195983656E5B293D6A7881AB312D3F705CA6AF4E2842
7A6BA28220444F9865BC7323345B684E6E2E4E46728E9CC2374E709C629BCA404B6BA977
9A813F6FBCB569C86D4943928E7A6F294864A15D5F8F384E5C8B53C07E3C63C266436859
3A6CA32D377251499BB22E3D6D55529D8F335F816DB79C323D487766B5B6464B786F7CB5
5F2C516563BA96316DA25BB4A3374A776E92832A49809C70C052375FB753AC6949A1C079
71573D5C82548E9C304C4C9A6495AD314B6F8F57805A2E397DA04A786C25487D8E3D6B40
37558EAA455C4B516B924B34693B50707E2E2A5C5F5E819B4221436E544C8B91
9E585BA16B754245889C7555295C87653E68503B85B03B3E434A579E612F4F6068CD9928
30503776CB67143F824479CE3E2B56A447B88F2E3C6C608F843622686136967213355F66
549E4425356C7D4A78491F4F97614C95322B6E6E3F60A43B569082568250394469644B95
453F3B70433F6F3E35507335424D3073A74F28676B244F9A453A574B4458A6573D654D50
A6BB48445A69679F9F3C58444197AB2E38567554A4CC485070936CAE9C334E5F9191A94A
45516A5E6FAE3B495D7778889C3F57699865B2A34353988683834D3C6098795E7B1A47A7
9053BE393856A06A8E56406ACA8C51822E64BD9948793B539C8C375F48578F773966964A
A19E4759614B66B1584979A26F91CF474865545B98982F4156615A8C8B2A477189A0C560
3A615A5F88BF3950849E7578C8373D53835479B754385B7853588D761D2D3C71
B8555C7A7BBE4549536CC4A9383D437687A63E3E539F82CF8734515BB167AA3D3E5D9E7A
68AD203078B24E6374252E7D8D3C68371C438D6B648A2156AA9E4E753E3C5B944542483D
3669852E57434C3F985E375D594947B983314A473571AC2E3265555D90A62D3D4E5B66B4
9034474E4C69B347355B634977681D2B747A50B29B28284B5461B97A326085947BB06B44
5D8B7762932C437D71649F743C51B3A35BA454415390935C87375F6E9C7F878C2C3BADAB
486864304FA28131633C4462AE863D844D3A6C983C4C4C338DBC43296D7D4293CC482F7B
875DC787236287739E7D2C3E636C99CF2F325D949AC9412F588B6FBB864166795099B932
337C72A17A305567A863A4703E72778171CE484693BB5561B3404860A88271C33C3C93A3
4C8EA93F3D667230597E48649F893A6378473D90A43C586B424C76C34A4F8654
998BAB4E607C6849802A62AE825A822C3094B25580504A67A95239743A2F7AAB2B3F7661
5B9F3B1F3A7667957F312B4F694E9971222D595963B0291D2E72678E39254C663875A120
3245684C9D9E1E4C586D4CA0752C3A457A50A95D3127748D4E7E33395993928397313A47
8B6E615D24484D7E5783652F6491456467323D67AA4B2A111A355F963B67594E5C90A648
648B587ED99F356C673C849A394056626E95642B5D76806D995A3BA29766AEC0333E619A
58A08D3356968C5DA93B474280A169BE3D3365997B7F723F4F848D52B966304BA2916988
474086BE6F685237669F656481364AB991534A28598389609952307693416E453368B355
5C4B3296C055495A4D57AEAA3F3F674461AC5D2D4F707179BE5E3955516966D25926576C
4AA3D64334586570A090302C63836A8D9C463C4486787BA1294094926883AD55
8285AA8645897CA7943630647587CA3143577E62C3851C61A07D71C5354D58AA896A6B35
5C567E7C91552D7773495F84302F636252825C2341A28C6566193D58544E656B1D487F73
6179483654AB52355A39286CB84240452D346C99303A4762728A492E42496A7F9645324D
47649D8B2A4B594F60BA8A2C42624071B93D314F71713A1C0F2B516166A1A0346A7A7576
A3BA355393BD6AB86A384EB591867D234C6CA1768DA43F605E8E66B496404289856BB47B
3F49965842985047749B49585E3B48937934585A4198793F537E4D4E8BA04A58616B6AA6
512C5064789D6B344C5B6B8ABA4C2B5D5B778D423E597B8BC75E3F4F8469C67B19599E75
977F4F46938A7F9D3F3091A89999A333548F7F4C80532C47919788B89C2941B4BB629658
3F4C729746743C274C78544B5938233478504D67383B7B8C6D31474C2240BA7B
D05A6AAA739D4B46AA906F65413665B53E8E4739459D575B682F3F756D47923955646D3B
5A841F408E9633445E365A9C4E2645704098AD2C224E5D979334256655529D771D1F5255
5D96552D2E6A6673A2483942566D7B9E274C4A9E777E8C3243588253A84B244778A3589C
3B1C53935A7684413870BF6C724A2B38578041BF562D2E475F4462733277A95B377A5D57
7DB06C3E5D33538E7D384757567598443861356A8AA8693A525B67A3AC324C3A5E67BA85
3D56537E5DCD463C565575C3BE3C51615865BCA3284B954F97C348465FAC8EB3992E5373
9E61A64D407EBC6BB7474168A0C7689D2E5FBA806E663878A8AB6BBC3D49A6C34D857254
74AD6E5F8D515CB0854E7CA3525FC47D4F7B523C758336415D47407A982A316A764E7F9A
32586A818AA35E3853615380A33637375253557D502035698E568F42372E6D9B
B2606A6060B05A3B4C6B959C3B31436D48B25741565B5B96922C4853616ECA5C2E3D6947
A4C82A4152605AABAD1B3D6E946C9071313E7682848F312E8593637F47244D8B594F6022
284B73487972133C6D562C5E66308191613A5635515F925D2E4D39487BAD4A48AF7864B6
8A3138574E7CB54635466D59808A2A3D46504BC26E264152655FB675404C686F59CB8D34
6D886885D13C3C90AF659A7B476BA07570B379287C85AC829D2444A0B788AA613E489EB3
71885D1F4EAD8358B1613953C3AA5386433F59AB91785B2B4B696F53883C4AA0BD644C86
3946B9AE3B58425E897E4B4E734E91EC84375D6D727F32397A6068E572236BA85090682B
446D6D9EBA493A58966DC9CA4945636196CD49465E75579AA2403A4E586A71B53B3D5A9B
6B9B982E59717678A4A14441526B4361633343537F383947375A749934446B4E
8D89B3655091B3818D38598FB179B4344073AD65B13E42806C637D902F567C9543554B37
617D6137563D3C67845B38462140BEAB313A43336DCB6037483337766F2233503D89BD25
2D4556429BB71F2966653D8A942628858F66B9732D3487B1869E304A5686808AC73A5652
9D5C7C7D2156858F669B484357635465702E4D4B7A5C807C2F3E528B43736E314B7CBD8C
86914362ABB15985453A78B14A6275324D5C974950853891AF7E4864614FB7942E379059
4A978B2147717556BE7B2144614F5CBF3D285D6857B5B725487B5D6DCE77195EAF796CC2
373685B765854E334C6F71917D384F9F95639A3B519B7F577A5B4A63B080B26F46828851
644C3E5FA1A169822F404E92847B9D3541A9B1576E5330449E823869724648A0832C6849
377DBB583985555696CF59406060457DBA413861674D89C7384253735671CF52
805865854C5A514B85383744627792402D3F5381905A1B48543AADC52E20627D719C8C18
56756684BE7219719D8492B1372356CD537F6B1F4095C76180152B4470555C6B2F346E84
3D80442D467B75447C3624356F626A803C464F6F4B746C2F75A2933A684A405FA5804555
414A65B078474746728C7B2D4C4F5477BE532F5753458D914B265A77698BB2442A6A7563
96C73D367D8A6DCA4A27369A6F998729434B7A6F7EC43D507E839BCF9D1D71BC7B80EF6B
2F74AC53759235487BD6866F9B3C527B964D987A214FAD615D6F2D4774A2454F66396BA2
7B3E855944A9AB4C84792E79C9644D6D636ECB5A3B475C89CF8E4E77354FCBB02D688B70
79762942735670DA4A354C616599AA3449799265C789384663995F95A23E607E7F5FBC83
304CA08964A7435C8F91887676275B5D58744D69413B6573663D703729487B95
B15456997CAB4D596272918A2749778075B52E3A788E667E8921458D775077294A6D6136
5167171E60802D4B6B2E3A5C6C2755434C4F7B323B4231708E472145402C7A942C2F3C3A
33906C214049554BAD79234242557DB0762051654C82C13B2656925BA3742B3F83A2A3A9
38345A908299AF244E9CB160A68B224BAE9B5BA53D3A5E9CC76486BB436466805CD04E44
5B87837469285CA9A166874D38669E695067434557A4875C7E2C5BA2D4654C5A4957D3A7
4558754570C73C345F623B9ADC522F6B673995CA3C2A805581D13032446B58A6E32A2E85
9364DF5750519060B07E3F5D7A758DBF3459A7798F7D3643958E62A1373F59B75A85633A
6B946C4E734C3F578D4E8F8C27438C5250784041697F4D559744455B8E5A58923634A0B8
516D81564C8F62386B6D5B739F3A3A505C676D9E9026366D5C61A04F2428568E
A96E7863527D7767773E49794F3E854C4981873E6E533A82AF413D574681A0381F316740
8CAC211D3C6D837A9B1F41496859B57C19587144698E3723606F5D8631383F8561849D3D
2763AA5D85822B3A71A85C5956234378A7487C422D4A7BA751943A3F598D6E518D333F77
6430605238507B4F2858493A66A333334C4F548466273D45455C6A90293B534561B28525
636B6586AE563761616EA1662D406C827EB7423D5A90747FC03A3D89B47EB983354D99C2
6BA85C475D839E5EB33F3C44BD9D6287353268D8BF827F4D58707A6EA94A394FA19D6066
3C44A4CB607B38477DCA66766F42507E784C6D576D7C43344E5474B86227557566B5922C
34407583A66D3C577A589FBD4E3E6D4157AC5F2037665964B6441B45747379DB573855A1
868CA1414254825D9883345E7B6B3C4B442742708F5C49602543619A4C5A7143
8071A94E5F6463829D3E435E5D7F93385970885BCA6232327C5F797C40297B8650715741
3E858F547B57215F854D4961292C63A14E564D363A7562515628257C552B4E511E377549
21364A294D972C223F4E4A9DAD2832427250826D2945377386A15D35443D5F73B63A355F
6659A39526488389609D46214E9C826790362B649A6786AE2B2F5985657F893139A4AD47
9B83236589776D973729659B7E5D78264C67A2528956414E7C69459D85416BA156499456
44659033639E3B5BA467305DA24E6DAC5329545B4963A75142595A5EAD9E2746867275B4
5F1B588192BC38314D7F60B38B22588E7D87C64756836480AE4357AAB14D7642495FB890
8A4D2D729BAF779C2C3380CA8A727E35346EB9646F4B3B58757C3F7E583C51967F3A542F
495AA15E3F613C6B778D532C516163868027263C4A787A9E2B2746827D7D9C3D
B1558D9267874157B5726659363D73533E853C4F80AC495C71304F96614859574EB36829
2A4342466A551D30785E6B912E2A41466CB29C273B505355B1452B2C82568B91232D3C89
559B672A2A607261B46B1E3F5A58717E373B518170859F233F55AB5890372950917A4588
373967895C607A3A577E80415841465A7F5B385F344A62B98D347B6B33567A5241876949
6AA34F4978396AA2692B5A553A6491403864654EC9B62F4B68706ACDB13A5953837EBA5E
408BA56E80D0512970686495E33E35866F86D86328419BA3648D3250569E66669124515A
7C597C4C438E8B3D55682E71D0A165663556B97A4C5635499D8D3A534B5B7C6635454853
6481363477467ED142234D7E5985C13B3340676FA0AF2F4D8D7B54A696304966888CBB57
374D797B9BAB32526EADA07B983F45699E795E7E413C58AA863952203167BA8E
BC655E86798F424B5C56977A39388F6B7FCD4434629B7281BD3B338FC285B831444EA069
8C91175983654A7DB02B51A28D50A5662844AD6A4D612B2D6194486F2C2F3F6D605F8430
3F88A44E5E6A305685A74B45612F4A93712B3B37455196752B5E4546A29D2A426C624B8E
5D2E49776478BB302F48744A8F742F3D5B7254AF732A2F5F825BD59E3834818D69C97231
5FA677899C2F3C65654C7D6542549D66606D5E477DD39F7D8B394980CD73805E3A5B9C8D
476F572C61899753A9592D52B08A4A6B34345F8B3F445D35768E3C2C48513D77CE413572
5A69D4A32D517D67A1672C3A8098B8AD2A2E607AB5A328414F9396B044295A7F6A985828
606E5B62843F486BA753C46F465B7967679441475F976357A2313B4C7D485D7F412B5C85
35526E324E6F682C5967484F69582D465241627B5A2332575048804D2372574B
AFAABA515C878F864D4C6782767F63544A9478427C413E5B8B4F4F53314F65452A464D29
91CD652C4E3A1B68B74C5674613A85821F3860528FA63227406A54A8621E67634075C43D
14316B4E959C262374B7698CA722306F985B937B213D519559A16D4541915BA2992F5557
8A77CE7521506F695FAC503063935B69903E3977A85D89783E3E68AF6E70733F3B76B754
664F3E4D9B9A5695443C8A7D2F41715C5DA2523C45704F61BD463F4F8373808F303B7583
62B49A1A38656450C9A32841545C5CB8542B555D5A96AB273B55565274622B6DA18576AB
39368FD96F7A2F33529773706C2F76A45A655A41578D5E81803D517B844A755B4293AE47
56604D73A752306E54399FCF5F44765D4C995C2F3E5C6462A56825403F5B6DAC742F514F
6390C83D4A796864B8C3434C656F5092A02C3C55805281B63D546F8F488F7236
8A658870516A5F9D6B2548563C91A82741544355B7852B657C5471A3402F525275A57A18
3F5E78558838273586875CCE4A3A50694A6B9F3B407C78697E2F2D387C50505E1B2C4F8B
445951283E785F2A504B364A975A3F533A556B9C3637544F4790A32F4255426DAC404274
494FA1CE5444584856AF74353D6C4991C3543557654B91BE4D2542705591C64D48629E73
BA90355A695F78D0783E5BBA878090444296BB5B937E3B44A1C161A1863D8381485F9635
4481B96E7A82493E60D3749694465197CD647866325FC99D657E313D91B2515481494DA2
6D375C547FC3642D5B644D87A9344367729CAC464C8E59BFB84D3B466579C47B2B6F8878
AB971E49B19A60B8444E51A47A8E8E2B5B897847AC701D468E934D93772D588083569837
2D69928C607D453747897D4F853D3D378667366C422E2A60803461292C6E905B
C4567AAB847A4155A15E76723A2F795C79A33D4366AA588E782C447F715680362363893F
3E3B49458481282842494B725B1F54443167BA42285F4461BC701D3F424A869728295073
497D88192556785390662B5569646CC3392F598D6273B9334064AF75723A27488FAA97C9
3F1F5B9D73AA722D375C97688252253E89925185402B39679650895C3548838753743D54
6D9C5A5B633E3A747C41763E3A4796573F5239427F7F433A854185A09F3850784E95C15C
2556995F9CC83F38477192AADD4D3F93B879B17C2848AE7D9BC12A315FBE88A8AE295B8C
8770BC5A3D929E5E93683746A89C7A7F397FB07B89663B61AEA45C642B47C5CB65848145
50C56D828F454E637C35726F4168B35D376466509EB13326687F4D76A93B2B69684DA162
1E395D5376BC502E47545B5FA46A3D6B605F65A4883043537B76AB3B1A2B8F81
976B7F506E924C554F3F799B3639535666A33A203F5E8BB1962A2C558057B1322B417A54
91792C39537F5E76BE332C7C983FA46535419F855CAA332242B7688A343343799C57863D
3142684665912745626C37558E2631748933576D344370663377373F4767356672535183
53264954358EBA2D2A3C523F8A871D37535753AF7B253D455F68C88723475C6665CD6525
5C9D6F94982C407B7B71B6802441656B689F3C365C9CCD66C92E4991947E8787224EA08F
538F4C294CAEA1537A305259B29D5B6F305159B6666B643D3F92723B4453386BB4543279
5649CFB83746403E5F8F34374D57A5AD30448474C89F2A346D6D9BB43E31639989A4833C
5D8072BBBE30407C8957A8963F57619B6BB35D36608AB37AA2623C4E7C8168B44D4B4E71
4E7C6C3F5174BA558C853B4178A149777C2D35609C545581322B91AC30602C3F
7181C553589077A79E315E9B817982394C75818A92597662A2696276223565A73F806523
417E71407F501E4E69613777483984892A534845578656203B3943945C1D444E5C3D8F6C
1D5B5C3764B94616527C4E72C32F2C3B7555809B304B586B4BC861305A52519A9D212E65
7152707136505E715B7D38515E6F464A7A2E33628E46605F2B3261884861733B4780C854
6348354D8074486926467A825A70812D567B772C57524458B255255C72546BAF2E2F6D58
3D94931E33588866AA692F4F528A70BD35434F7B699FA12F575C8F60B47E377F96B487C6
513491B889C33E3157BC8D9A73274F889082614075A388769A3258A9794E675A3385AD7F
85543150AFA1516C41606FA36A5B7631499AB745696D334D7F9F3950424169949D456443
438DB9454C5F7A6C9A9C462C4473487EB53C27357F596AB13C344D604FB5A73D
A65F73D8927D556E835555652E74B12E3A583C56654B2F455357B5B4353255524CB27C23
2B394F57A044203C537071D077294C8D58C1951C2C567D458A2F193E90469A6E2D1C4585
447854152E608260734D25446C795256322C4162534F622A2B4D7D445B571F4F83564266
362E79862D465E37478667204B645D66934D31515B5A688F352745616480A44B3D5B8777
A26E2D575A525DB6651F5488609BC23F2E66A949948E295296C15CB2734F537469607122
4256534A697B363B85AA567F72457494904F61563868AE7D547C40365EA15B6C7A314D6F
7E588B3B45A3922E433D4B7BB14B4A615BB98B3B56733F87D44D3F788871C39525407B73
B57A24517B7078C747365C917D88CA3D1C59AC5594872A2951AB66AA8A25505E8372A956
29529A686F863845798E795875482C415D6D557760333D677A4E983628699462
C7557BB69BAC37596171849A294A87698B67332B4B856592991959B963549E371B618F4B
6661233169BE497684242B718A455839354F7057458141569F8F38534E39838B492B4648
35858C272A415B3C8869242D516162AE6C2532566372B144303B5F4E758F2A32577F6F8B
53363D60558A9B24333D5859826C272F4F6F64A46D31495A5E60A45A2F52578166B63340
7B994D6A623B407C9B557A4C2958AB893E584A476D895950723D438AA02F4845387CC052
265E5B4692AD351B3E63489A802C44838C729E773035697B81C74B3149556BD4BD2B558D
6155E3552770A25D8B3F4367B5775C651F61BA86876D484D8F8F7B6E3E4C5CB7709B703D
6CB3686678444B74BB4E5649304D7F7D4A65383D9AB65D3361513A78C35A446F474E8CAD
5748534D6E823C3049653D80B6681F395C434DB5C3312F474560A2872139575B
9B768B4C65A374605C345897546156305D96443741443C74A34538646462955A213B6550
6A78222E366442ACB52531434A5F9E6B2B5C745880AC24455D6963866621357A905FCD40
3B44735C8C7D39473C79528EA3204049925785972B48627E4A812C305084625D5A224956
6A46583E3A537254355B3B339AAF542E412E3694AC3A3E5E3C4799AB2E435D4D6CBBBA3D
58645498983A3B6E7E55B7682E37889064AB612D84887C79CE38406FA3929D594138AAA6
5A7F5B3A31A79B657970354E89766571354F67A3696A5B2352708E4D774E285CA55F4F77
394384A555694A29618635446C5392AB4B3A5E4692AA2F37644F7FBD5D2B5D6773A5C04A
4C5D57B8A63335478563BAB63442868C60CF882955A39876A57A3E4D8B9A80CD4D4B6CAE
CDAA70244E70774C809B2C387D9D50646035314F9079698B2D3561994D8A6A3B
73829C554F74857C8C3143938C8D8C3A4F697D77631C4677A55A838A274A82A0509F5134
4B765C45793322578F7145A1452470A63E4959303FAD9232493A245274342B3D2B39A485
213739215DA4592C474A33519E4A233F3B3D5794333D405072A2492548555797A12F4653
5F5FA96B315B745E509D3E2C535E7D8D8737485B6E597BA547354A817D8CA23C5FA395A5
C39741448E90778C4440549F737379474A5DAA3F646F3053899D42826A4A7C7F3B41603B
59846F20295C646F896F2D4D5170A69E4C3F557F81958E2F2859696A906B2D315C6778B8
3A265B8579B4712B39765B9F92324588838F9E503F849790B7434555886DA4313E518266
A35E2A3A899A609D4D4157AE965971393F658444725836326D883F554F2C4A806F3A6457
4589D669395C4B548DAC4C2148594A6A89293042625076CC4921375D4788893C
A251628D8380445A9D78524B3B5772384C8B3E509D7C31535D27498D492E4E4A3E977E3B
364A2F3A854523363B3959C56A23318746A3BD28226A9B65AE311B309B70747C191F43AE
61976D24202B6F648153364435835C7935284B4B4956883A338B8B375E55334C95583E74
323053805145652749765B3255672159AF752C634D2D3E96503865423D92A94D3F61424A
A4C44D424F4B52B37B2D4B465A74AB332C415861848D253E5A6360B48F2463AC5E838233
2C6BBC656F963F3B75A555A06A4060ADD573C24F5C74B39069823E568C8056838F1B63D2
B052713B4E97C65D653A3F68B9714E4D2E5CC17F70573F53A58B4D693D74B5913F9E695C
A99F3E47666A63C2913A475E76A5CA3531625C45C3CA271F6E794FABA72F2B828965B55C
344089797669252E559F7964754C304D928E4A94561E387B7645682F2F4E876F
C254586C8AAE4047538397992A4761637E983450597B61AF8C2642656E609E272E386C39
667112354F58499BB019394F875553501D3187654D6A2C1967A13D42413943765F35482C
454D722D3A3A313547582B3E55335F6C4018485B2E5CA548204366476E652A355570679D
2F2C3A7B7087AF23255987586A81222A6F9245AC5A25377B9365A64E2654A1947379304A
628C847B8021547E7A669D4B275D637B5F86252D7569434C85322E89B75754683946A45B
2B545628439E4E286344316FAB383849564E747D39545B3A94BE432969894F93C4312C81
8970B425327FB75E953D1E4498857A531F4989819777294BA69187902F4A7CA869C14A47
889B7D89793F4070C391AB963C5DAA7F4F76452D4EA56E318A673C458851477B45466D80
4571543C6A9D51356C63573E898A272E365656869F2127617D658F761F3C4661
7C8AB54B74A1857C6C2D737E5C706F3364A977404B372F6AC6773D5A3155856537514747
5E7C332D753D349A85483B58496C7F69272B3E4F68A84B1B2F5666805A1F52807557A63E
225F694583B823235C53447CC03827505E499FAD2839618C599034324794597A7C216880
81616E48485072593B5F353E4D5B5460713A3E58803A4D5D3A3C7F8A265E673E8BB46A2A
607B4779C6643D75644C88A9363D655C61B1992648845B6BC063296DA86F7888322D8891
50915634507EA1629A3F5184A8717586373785CE6A8B96383B7BAE6078674E6B786E627A
3946B4A82B543E3F55753150603B93A73F49604269A349576948688448356B4052BE862E
46715485B93829476674A490384B733F54C46D2E51516D448D7A394363765DB3663E606D
53A57E404C739A508968434266844E626617404B66726E9D364B4E774A654E2C
825C735A4F5E6684893B537A70789132377389A29434455B6B8BB1992F435C8C5EA05136
607E6453C9601E3E715472995F2369A65E9B982E3F7AB0578049284F8F4D4453263C6182
505A4D2C418762335A412F4571743D5E323C739E4B305C3F40636D2139494C6995362E53
424D877328435F6165B360215E594974C5432A3F785F8AC93E3E5678459DCA423A7CC050
9C92484882806AB5514F64896D7B7A2A4368D38FA4793D527EB878966C3A49797742763F
507179355084304C8A842E527C6079CE8B514E874598B5533F434E578A8F374F847E8ABD
4A325F8B7FB963325877569EA5304AA55D9B5A22397D6593BC31456D636FAC46398CAC75
A14A304AA4BA629F312779A05674B847366CA54AA1572F474C6948775F1836817A397859
3D6EC866373C48327B7B333853574080895B3876403F57904A31544253A0B33D
AE505C759E943B515E5CAF7C344865487B6D254F6D6B33633B2A576F4F3F6D483E618237
3D50272379862C415F1B2B8182333D4239737A38347450409C7123265E526C862740496B
4F91603738637850A6911F46766058A73E3558898066CC3F404D7E576F57364777946886
3342537C54637237405E895465522743746C386239294C757B4B7A41244B88634D7B422F
4F7C545D563C4E6A884760583185CC6A3268592C719A33364E4741A0B331475B607F8959
2B665F57A5E23C237AAA4BA7943243ADB79EB3A047419F9199D44B41699A7F97BD2167C8
B9999E313FA9C94D83573650C67F6978456C95547763304B9157405A3157786F48605037
879831384C5F67A1542A505C7795BD4C294C7043B3861E2D74675EA1B93020788865B14B
404E92747A8E2F47498D5C6491791D42646B68BF8520316D70658A5A36406564
8E65775060A758534E447791344948516A99472C535A57828F2541564E6E8B3A3A3A5A54
7170241740625391C5192C354F5AAD732339545F5EBA441D4E7A5C62631748725134631B
2F495E35526622364B7E4C4E713746635B2241592E385B4A2D4C28424C8339454B3A4D65
4133504354817E2C3A4B4F728A6D2739636A718F9B1D393F5D73C0802D4B5C5D7DE55D3F
827E7AAFB527436A785AB8A53A4594976EC0552756AC8C77C24B376A999F7B7F233D779D
5EAC642133B895408E4F353F9A5A506F4E37598B595E683E3F81963164503B429C3F2E5F
4D9BCF552C57925593852B496259A3A6244D7265A297354B7A72758E32496D956EAB723B
559F7E9FA82848A77F4BC18C295ABA73536E383B74C76249C04D2F569D7B4DA46C4775B4
4567863F427873438DA7412A75B14F4D652F2D9584314E792F4BA3A93142604D
816FC65359887888703C569E68A088204D8B7279882A465696777A99253D548442564F24
526050376930194B91734C612C1956A03B4D62293A9B6B3536402F4E762E37523A5AB056
194568478CAD1F233755586CAC372F487451798F272E5A756B943A28457F69888A1B3A50
7259845421546267547B352D5F5C54518E1A355A6D4A596B2743558A4675872F56748332
8666456A958F475F2E3B497F434457303F9699393D493C50616A32534763A9A12A2A4D3F
4FB99C2A3C624958BE6F2D47555997C95940576766A1D5444353917DBB8A307D927284B6
3431A48C629C41385CC35CA4562E44AE57754A334B7F67819634607C514F7F494E789954
8969497D98604D9A46669A953F4E9A6A707E472F44546862A85A2A3B4F5E4DBD6F30574D
69A1C155415F493F94BE3E384E8A618FA93F2A3E806E9DC357263F95547E7138
92527964606F485E8F525F454E688D3A385C5852A55C29503C3C909B452534697993711D
25456665CD5219215E6075C240253153488CA71E223D685779581C3A4850629440254B58
3C73642A3B6C724383601C4856693563323344733F506B345B63682B54343C5F754B376E
2D4C696F35466643648D7028515D5170D75B214A563F86BE50324C67539DC23640686C48
B4B032627B698DAF332A5A7F50788A2C2D75AD77AA9337476C8C5691843D428D845E8244
3C6AB7698C7C363A78C3508A5B3E3C8ABE5B985F3340899F509635334D8B4191623B5F89
746D7E4347BB8D30626C4CB19F303C8E89C88D33505758ADDA473E8C647BB3443E51835E
BB83395C927C8AE03F4163995569BA3562A27F4EA66D3D5A957F58B194213C92985BB37C
2F57947A6A943F214275844F832737418C7853642F333C7A5F3C5D3D3C648A41
B15574849297384C9666954E3F5DA66F6B8A4A47559E75AE642F4478A45E97453F5D7C50
67442E3663983B4F6823326D843F6042404971584B4E354A8BB7485E502C4E7D98365C3C
466991533C574550788C2C276B725DA644342D684974B3422B479351A26824376B60659E
2F244B7B5B729526415B83618166294F7B8C55AE522A4B7C8D61843D3461AC8A60B74B3A
4C9261647E1D5F8E8846526B2D5083675572403992A34E4D4D52579D964F5D656568A23C
1F494F4B8F962D3451595786A73E3F536D5999C042474B786ED3843A707049ACA3194A6D
668FBE3E4D7BB464A1532A6DC47480962253AC7B7C563655836F7C84283288886D8D3630
65BE6B708B3D4F7D8542645F3861B95A41944F3489832838702E367AC25924745F549B6C
2B4E796069C08925405F686CBB6E2137535F559148284C5D5E5CB162264A7348
8663944C4E84635F4D316B7A45634F383C6751425E34595B6C313E5F343E6A532F313E37
4E75231E495C506797361F5459547B96264F744E76B7371B5B985BB36E271F65B273734D
2746717D86A71F2E458170649234304665537359243B405D4D4F25305C83504D4B264864
843B51353D6F7648344D404173892934443632817C272F3D5844A77E303853766CA2582F
49485A8EB33A32626B6D80BB3645706D95C76C244C6E5E9BAD3A5F669981A0A03C449E73
5E995A2542AF9465B95D3752797C659234314A837B6677513D4D81569A6A368AAC5E5178
3C3CA5803A647E417474354B5C5F7EAA2D3660588F9D2A45735774AF371D626578D06727
38786D8D9330496D8974D39A21569C5866CF651F4A984D73C24E384C8A7746A8484F738D
40836E364E88BF4A627F2E47789846675A29435398385A513140858F46664445
727BC95556625A6F7D365F80568B8F32545E587FB14C3540756A73C1392457A55C845629
3554685F9C403849666950722C30445F6150551D41656925393C2A506A493E46214C797C
31484142609B451B3D43425AB43D2F4F43627E8921405051668E4A19355B54968E252C4A
72589A7731387D725A9C352E36695086AC2929536D64A48734415787509683385088B160
B96A3A64A7A14B672B3F4B946D705C1F427C8754A58A2E438A5036723C3E596F675F6F38
2775AA38515E3C5D7E85385642707DA65427474A77A9A03D39534057BD9032456B88C4BF
3B4A6D6879D179415A834CA7662976B2766D6E494BAA7D968335508C7D6C663950ABBC69
76472F6B8F665582434A55B9764D8B534B8D9E565C483D4A736A2948444548935B2D537C
52A6A54A2B5D76619F8332355060479DB12D394B6052A1AC292D5A81539F543A
8D55849EA0A24856767C966B4651716986902C3D56762D4A4825436C6F22546047566623
3B612440955F1F3865385CB3591C41474F7C8F2B1D576D40615F1D2F7E5878942F224696
52AB791F2F7F994A8C6B1B398774606220276F9F4E606D2D405877486B29333D6D5B5765
24315DAD5358682A2E736E324E471F40884C2E613F3065B25D394C3B4D78AE4147696445
79934058706091AC3C2B5272437D832B2D4D7F5AAF933A365E724FB6731B556E6B7CBA4B
38478F6D9F832967817B669C4E236283675E7D4F1B6FA0796690413178CF8EAB772154BC
874C73362E68A2618C2D3E5E8944736D33789D3E4A4E4C81B23B475A5C559742334D628A
C29A3D538A4C88CD6E3962758074BC4754736D79C39F19354C515598921C446B5F468C5D
494D9A825086452F66B18660AC612B5CB27369863F38569591716C2E4068884C
A95A5D5562A650473E3F79832D4C5A476EDF58245E8D769072353E5FA05C98793036735A
8DBE3A295AAF515781331E68A7417E7F2861AA8C5D7C24505368526942203385A157943E
4253A0686479373B59922F405D495162362A3E4F4863913723324664A466252D3F466CB9
4E22394F428CC732325574599F8B233153626AB5303B3D4F6F93A63B3938838374983A41
586155A9A63C3B6E7C50945B413F6D82718626395794656E79254759A8595D6330509992
4774454B69A04E4E7A457796C3563B7A7D63AC842B3177527BAD662E3B577AAEB9283F53
5A7EB7323E5C8182DF4823677845A8AA2157BE7A74652F5E9491AAAB3852889658923D2A
83B188B8BF4843A1AA607A5B3B5486624E96403487BF484D733A2C6DAF4D355D45599765
2D4C624961935D263E623977CB7D27305556678C5C2D3C7F729BC73D33426454
6D95CA574851809361394773769D662C4989987864333E5B814668762A215898444A3124
44495B41482F404B78473448412D6A852E3D4B3974C44E194D5F53967525184C5D69BE46
2B325D70A1B21E30566E699FBF264B6D4F567955204C926A6178373187BB5D7062213167
C75173382C3A69844E6F2D37497C4A53732938BA962F403E3953B1773753584C6CBC6131
6A593A88C96E365E3F5377A23F59602F62CA5C2F4F583A7BCF40276675728FBA21246666
719F7F2A5A627E62B0343585A79192B2364795D06B878D352B7B8D637335416596646E9A
383B7C966595526982BC5C644B3F73B4665B5D48969A445257396DA34A3457406A985A20
585C6682C538335E7A8AA7772A4D727E79AB3F3150768D8DAE3F3B4B7A9C72B73034619C
597E8A304281B94C728E3E2F548A5379472140517B486B782B33647F38813041
805586977C783D61983F615A3B84AF2B4052373C9B4D2F2A53557E9E3C2B3B7B70B36912
30475676AE67193D5C4267AC5521445F4C679E2B1F528B4D682B2D4A706F4F6C201E578E
424E3C184FA86D34823E28458B4D354B282D61863F294A3572A04B1A30704D8C94271E37
545BAD5B19364456629B4F19535B4E82733C2A5A8D5693402F4757825692543E5D6E7D76
AE662E6AB9BF8CA5383B659C5D879134375E934F9D7B2A4B948E72773F3959A36F5A8633
4A6E914C6577435B9A974062623750AEAD3949534E51855E2A554C6DB9703169634C8DCB
3F40656A6BB37532636B51B491215D907F9B631E46935988922D486C717A95294379994E
92542A74BE8879823C5283A75248933768B697427B57346F69743D6442445997683E6F40
5D688B463F573C3F4F894D39473D479E7E2D4547536B8F97434158594CAC4F31
B1526672B3B33B4F6E50B0723335835488A5392A5A8D7E7F641D4070814CB5762D458942
405F3A2F87BE5A676C26317D7D3B6F4C2B62BA6136484A36777122373C5E679A47264166
5775731C30467650CA752B546351657B201B466D5A758A2F2C6D9F57672A2042978C52A3
2F2D57A43A506D213575724E543C2443854E39532D395B7D564B623754607A3838726352
87A54245532B56BA6D2A46403497A13B31514645B59E272D555F89A9423155676C84AF32
306A66549FCA363F89A07FB173373E798B9DCA493C4076A069CE5249766D559D573965C8
8A8F79374366BB5C942E4A61804E67562A8CE5734F6C506693424E633B51A8AA466B4559
93833556803D89CC4D415A636981BB3D326D796AA3B42837506164A5912236676545A84E
2B4C8F7F7DBE492F54906A727D3E3B669C536D632C2955555A667238334BAE49
92749F4B588A7159444457673152564E61864B23474445758F272D4D7C429772312C5A5C
5996342848905F85C72816537F44A782173586817496253C444B576E651745578870C82F
2E455850678021334F79527F8F3B425E3D404830364B87493A49374D7969242D3E755781
4A3163546B6A753322486358814B27464C617CAC2C28576F628F7D2C437F8778A3A4454E
598E80B0A42E354D7362AB501D4AA1A570793E435B9C57698C1C2E8F7E4891722B315D89
4998433040B18C5C66373792BA639B80383382AF5642483A3C7E953A594849C87A213E50
4169CB5D31466E92AB5B1E58795CAC8222367A7E82641D58A55D887B2D5062BB5681253E
97967889A7454A938E6DAC3E415871625FAA3E4462735F7A78315B688C4545713E557A79
436768303F989D3B3F7C3B568F4A355845465675452E454C5B67A24A3755575E
8267CA514B646BB0663D547D61A48829576A835C9B2A3A3B74769AA31D446C854C754416
2F4D5E3F4B302137518954894021477A3B4C422E496C6B32492F3F4E6C2833443D487769
25464C3C7FA9431B46424488C1283864586F8D69173A6F6F69783A345B916F837E1D4A45
6E63C048284937586C9D252A6572327A3F4238817B3C483D354FB0715279512F5F765841
613C43567A5540713A50696C34474D48768C4035556165739A39275D4D3590A33C2D4E44
57A86A344D5287B2C7502D5F845C8BCA442C5695766E652E55548E679846286D9B4F8267
254A586C5E712D308BC35E7A422A699A33603B2D7FB2505466314A924F3F6A50588A6728
4A4E4895B15C4E515A52C8962E56604769C95A2A5E4452A59B482567734E91CF522A7E95
5181892C2745A37F728D3D5A6A8B607E442351809281834938449580688C3347
8D4F7F8B7D7C3743814E674C304B853958562846626332563B40588E6631674E5E9A7929
2A576E607A8C243B69665FAA661D415A5186B22426566A5197381A3B77617789222D697B
6E8B70232E89974E9C5D1F4B845C695A1F3F657334435030445343234726415A75453956
21387D702748621B4CAD6D1F493F4A7350242F585E5D7E3D333F587357A778284970696E
BD8C2B597C8573C7553A726B4B9288223453725EC560397C7C916DB35E30596D8171A928
467362627A523C5D649D406C6F3535696A5D874E3C565E3E63863B557166376173529F99
4651813775C671374A4E4DCB9E28438C426E6B25516B688D8D32455E6778BE3D4E808F59
8E5E1D56A7898A8B3D4378B04D736A434B6A7149956F1E657B6E53887B2B406D8B516B33
5366B06B4E6B304E518962403C353E949336473E404584642D333C5B69914F31
A44E5A83868A44355E62996B3B4E645E98BB2B497A746B8B4831316FAA70883E37477678
6F7F243E455D73618D2D3B4E535A91712A5F94755C7A36267E9A3C463D2A40894E425429
364D7B2E46444346686B213770455690321A46613D5392303D5C6C56A445244A806978D2
36295A72487A9B292C495C42A22C2A58655C60AD2F4D5D794D573E334A82B7615E764E43
84C14E4D5A31447A92487B382A638F6D52782F448D9142557E3050849C575C55235F9756
2F60453B99AA3349687453A5CF464952556EB8B0393B5B5695C7522D43654C9D872D548B
8299C1383165626CA0462F6DAE52796C39439663AF5B27739B576A6B25317FAB5B8E4346
6E9A55607C3A63B64B3C634C428EA73042794C52AAB74E39745861B9A3333A4E425FC47B
3157735C72C0742D4F704F8E96213D58B36B905E284C4F9C6C747B2A45669551
8E99B6494C737B9439354C7953953A3A6B8C805B5747585F6F3837363D6578642139362C
64AD3D29443C266C86592D4E413F90AD273E735A6EB142194B8667784D263F4B6565A519
35505F4681A71C45645D455892203F6D5560955929467660516633314C913D5B6D3C3C54
65556B39353F4F47488C3E368967265147324D92552B554464788A3D3C5461748988372B
5D8E6E98823C266A8D59A54C2540725E71C1452854A38AA4D22F355E806AD0951F508B6E
6F9F5A3C57A86B72B6553D66B7778E942F395AB2926D53344E69A6626D2B3F8DA655684F
307FB09D4B8237579C592B645B5097AB3F434145AE861E35615A8FB22D28577D71A06F38
45716793A742396B7580A9503C407A616EA8444450B36799673E4266BC53957D393A72BC
495F92403D5F9D5478993845948A546B33375471343F66434751783E3D633A63
6862B95A4F5954926E2C494A65CB9E1A3B826596872A4B6988649DA32E45866964AA7F16
4C5C7E61A2411C24337186A2332B497C7E7B721C3B77813D5E262B60955758521E51586A
2B474F28479C5431623E54609C5C504B3F3B64651F38503E838E341F255F5187901E4252
3F70C13D26465D528DAE513B697C5393571E23846F849635335AAC9691B850426ECB9670
984F346F807967994C325D7A3761793542708243714C325E72683482362C698D30458A39
3890812A4C5C5344A66B38627560719C432F683F56A68D204E5F7979B5541B56877DA462
1F3F8BA85CA6322A6A91559F58284E6C5E8E40215E98546F933044998F70B330375C6B54
7B5A1D65C38F7C8D4352949156817539539C683C56464F6F8F5239494047609243485C4B
5A6A9F4A3449675D7DB144385A6D5C8F4B1B4570736DA26123408E627FB53E31
9D577A7B8F6E3F5E8C62A7502876C46797662638785344773928427F5D63812C43546B45
40861940A996244F5523509D731C4145556DAC39245075727340254C594D88AD23395C8A
5DB0851A37805B77A24820478B5A93731F36486152B45D353E5E72667820364B7C605C59
1D506676326B422A4E6D4C4D712B2A82994B3B6A293F827C2D3D343C48735A26385C4F66
8A4E2B435C3F6B9D6B313E615B7B96362A5C7A66964935405D7496DE4B246BA4558AC539
3475B16292702858918C6AC88C35654F765CB95A4146975F678246395C9F518D614766A5
6E697B5147916E4B4E4F47C0913B624B4A8D6F335E8F3C7E96362D4B3149B37932563C5C
B5AD345080648FCA3F3A5F8C75AB90273A7E7179B0772246836270B0882156BB8E6DD34D
2E509F8E69AB3F346AB0655B5B284B71A7637C3A264A5C7047625C1F378A7E3C
9B697F4B5B864A553A49744F303C465375782B2C4F6758A36B29255D618CCA3527366F45
7CBE251750874F8E921630617B618258203E6F7B6E8B432E6687456F4127406E7559642F
245E8E484054212B84722D443B2353AB5E3D4B2C2A7AA5273856425A713D274A413B80A7
2A276C734691711D2F6F566FBF2D28466D7865A12C2F3F715A73431F4960795DB0A33957
6A8386A67D324E7BB17DB330416B74785AA22C4F9A7A488477344EC7A369A06E335BB363
457A483D72C462377A524693D057655F3B72CB60357C55477DBA413951625F89892B6667
6897C63B6C91817F98231B678B6EA8772247A97FA3762239796B617C273A51AE72B05835
469369AB77384375626A75473445864D61A65A3067833B5664373A66A74952484C6490A9
415C614B4A8799255565638E9141345377719B77193F7769677F93312A5DB17E
8498D84C476472C6492D69AF81C340344470669D91343045746D7A803E5465A3509F3E32
5F6E4F496A3030558C4D2D483C3784922F3335243592671E363B34728323263B44629749
27515F6F84A52218448460B96E1B1B3E62A1AD34242E66546D9D2A2D4A6A79B0652E3772
5D6770433353784167613B3D947C4E7246244E596D4976363C8A9C574578283D598D4640
5C2459718951486855719C55235A525A5FBE5927656A4CAC8935325D5165C8AC27396E73
6AC84E364EA7BE88BF494C6EA57ECF9A27458E9E5FD471473C797F6A77214178AB549E70
333CC39C7B7C3357816250713A3E82A852603047A9B24347714059A35C5677376CAE7041
554352BBAE414F5B689EC37F3E4C5A5695B763304F774AB1C851384A5E4A80B0313C737D
5086B83D4380B56E978E306A5F5A497B353E6E644645883F436B815C53713B4F
6B668275734945668B39573D377E983152432D4B944D235570365A7B372A426E858E4415
3D638582BA611B4C4E5A5688402047607471852131507F5A97312D586450696D1E35597E
49C4822341755D42522331649F3C423827497B772C3E432E4280683139335A85621F3439
3250974821315D3A84963A274E604B935E2B2D697A6DB3403147635E9CA930454B808199
CE3D466E8B8BA9B72F63979F65A98F294E979B6CA5342B4EAC738668364163BC6F818525
488CAB44715F3067AF6E43754C3569AE5E3E683844909436A07B647292452E554E56B68C
2F4562738EBB383C6F567AC548215E8277CC621D4FB07F8F9C3C4A5F65789D343A8D8E84
AD514454A6717E6D213F77937BA6862149B98A5A88512B36898C485D5141437E6E2D6E65
4D73AB633860745B86A34B434B3F6A7D5A274C6E4A60AB6520326C656EAF3931
A155786CA556444F8C56AB4840398F5F945A3A5553A075BD7E3B3A58587FCB17364B5841
82512744706A2E6A721C3E5678325250366760452B832E2D866220365133619B392C3339
3D8A6F22324172749330225073657654233A797844896E13395B5E638E1C37618363995E
283A6A7E696E53194B7C683E5F1D335DA046555221346EA24C6847225098845781622D6A
6567395E373855A27438562C4A5D7A3534504F699C661F3A676877AD30415C8B739CD560
2A6E8776B4852D4F828D6CCA7232436E92A2DC343D3E7B5DC385214881855D74373D4780
5283532A618CA17C902331B3AD5766634074A6606D42445B87394D572468B7583E633041
7B482C53556380943A32756F7397AC24325E6A7DAE9B2D38596083A9A028475E6F6CBC72
2B4A579B9BB64B3D4D70558D96292D718A548963312C5AA1616B47474384B160
9D778B48655D43603D3D655C27583331589A3C4149342E62B13B4A463A54735C253A3266
A86219213B69548AAE22163F76578E9A1D4F645367C4481C5B7D6294561F2C4C68859D24
2D3A64465C6425535C4756873C2A34493456632C3334764B315534295C40284139285168
202F3A48449696223662686388281B2F7359937F2325516C609F641F44476064AA8E1C54
736475CA74324682987CA625258D916191BF3E3A8A9D75A17C4B5B7B8D7E9F593D5875A8
67BD55445DA15B76963F416FB058867C296687754E9F3B457D9D33425E3079BB542A6048
40B1BF4330605381B5292D536976A25F2B5F6574C2721858875491882823608779C2333A
6D8970B0743067909C80A2474A76BA749785354858726C67B131408180526888462E75C4
515B5726528788365C4D4162A0503772675F8C812E324C4E6881531F243F4D58
6977B35664695B8740325B876FAF44363D6959757A1E3F397F6CAA8A20435F7C64A62923
5D744A6AAA2E274D5559546925335A6F4A44431F6CC8982D5027205988473134354A6246
224E3C4A65902A1F544259A08623443953739B452E3046637693252536696BA24E21474E
367752212052A68A7C92243E7F6D5664212C58A96B675635314F9640404E1D46718C514D
64364D76AE434057463877853A6362344399493A5A3A48719930403A4358B39D32465854
7ACD5A3762504F83DD7A27659B5C8FBB3D2E7EA968C5543951A15693942342917A92C46D
2674CDB0A968275F7F555D803647819C4473232F8AB8557C802B41A365575830417B7540
A12C4AA37B374A564A5F91563E704941A0AF2F2A4C513579BF571F528C537CB83D335F7E
5D8893313176A8468063254391815CBC59314A96825F94221D51A2524D603D3A
9363946D92514D666F558F4234726E3C5633343F706027693F4B5D712242614D628A4E18
365B3383B54C163B6A5B599F66174169658385242658A953AF4D274360667760184E6055
4EB76E2344715A699E2D395B875A7330254674925B6D3E19427C7931711D27638531403D
234B7868516C5D1F5CA13D333E3E5D7B4A1D2F4D4D6D81322B3B685D9494242350675489
A42F30438A63898E27494B665AB86F324B62847DB9413872996073593B4476B069909033
437FB14B86423A4B92DC6093443646779A7591314E536D3A7762406DB7874E7B434BA3B7
3F3F636588AC3336694E6FC68B30476A7AA45E2B544958BA8D2B315E6D97AF2B4A715F80
CC6E2170AE7BA4894656808E738B502E37AB986CA479322D838F616B5E2A3F9CA948763C
2848874A4569423B6F9B4B5C4D36314F85434B6F2E69BC7C42413C544D9F8742
9A4B57529F58384A5F63BE40303856699B3B243D555D6CC5472A756E44ACE132417F6B4E
AF55183562696DA691192F3D725566551F40583C3C682D384E61385D602D50975F5B5E4E
3B788E373E443F3D8B4622453C437C67202C4A7A7A73581A27486C43A9271B316A567963
193148A084B245294D63647687203C658E4E6F45253B82A9565537335A5F4736733F224F
7C815B955B41486B533E4D22355B804B5465255D5969387B594FA07547546744707F8D3A
355D73829F63303E509087AB7D31505460A7D83C427E8B5F9A861D509C88ACA93F325A9F
8EC994465F827A9D991938647D859B3B538A86589843237DC46D6E65294A9E836169223E
73B9697241316172415285364FA1873047375D4595A654483A454E85A7372D606A54B15E
243A796677BD40313A7358A39E32394391607A8F1E42826446A867343370BC64
757669504E815B8D324F6D704F62362E62A94A3D3C2F3F6E844A5436152B7E7C415A2D23
5C8129344E2E3B53913F26533C5CA36E182F4535589B5A2A4A3E3895902A3D44567DAD27
3349565895A51F28546B6DCC64245AD16D80962046435D4E806A1846666C437930354F57
294C591A3E6A693B595A3A477245334433495364262A3A3C607B6B22414D5F4F9B832331
5D4F5C7F9B39656B50988C2A2775575EB3972F4A556457DFAB215AAE83A9D7582F688887
80C860438CA4679D8032569EB3636B683355A8AF58742F4B6B99476E601F4E97694A6D3B
2A67995D5B4142819752314E4B54AD6C1E5B666FB37D2057648DB99129256368B7AB2C2A
54857ECD73315B6E797BB2543C5A834AA6902726477963B284202B459181787528595C65
557873333B687F516C362F44685B4F61303F417B663A7F3E44496D4F214D4861
7E74B8575F4E53B552366E5855813A2F4B697C6C631A374F4754C3801C32396799BD3032
5C665483A11D2447597179A334214B993D5F5B214F5A7A479430204A9F706B48203E6486
396B3C1E3386543F591E304F873F5D323240685B1D41213F9CAD3C2C4D3B50A44C203940
3169A32B21585845AEBE2B3E5C506BAF5D215B725175BB47235E9B4E795C273F60BD8574
762E3C588E9495793060946756A44D3C80C0646E60333A9CD666593947549A9B5C67474A
619385334A594D7AC354314F78657A8B363A59888ECF3E1F52725682732528456D629E53
2F52515FBC821847815C617B2D3763A45E9A1F3B81A649A4492A68A07B728230468A6658
64284952725D8A5A2E5974815E765B346EA977375847467F973E344A464B81BF3F376758
5585B42F3753555EA05A2653717A88D0382D594F56589746284F62A355A9823C
9C5A686B9C3F47518081863531495F50A944173C624D91BC383143694F6B643249797C30
5F452363AC662F3E3D23439E6425553B5E637D2B365D4655874318395644908A19263B72
6AB9751B49536383C12F2957646B9E4D2043646B648A481A49A38982A91C34527D3C5D2B
253D55524052383443908356551531628E404C4D283A5D974E3E4229696B2B2243393464
724B2758443898BA31366E495FC0741D4C7D80BC7C2D2769A686C86E316A838984D69729
4CACAC86BF682C647B96A09E253E7181608DC92451BC8D3F674C6B8CBE8654783D48688E
4A6E4A3589CB7B4F622E83D05E405A3E549A582E3F3F4A998C2E54616F8FC66E3A636DA0
AB333C624B6CD3BF24426B6B91B7733854567C8DD05045497E6977AA892E2F8B8C88932B
3442907C7163264A566F4F7349345D675E3D695A253E7C7F3E6D54242655A44F
976A825D707645613B46A14C3E49434A789431393343708D5A30595356917E1C25576C69
BA681533619064AF6F27353E546DA232253D4F4771B73434747A518C4B343D7B5D89732C
3D66823474471E5C976E55593E356A733138522F2F77812831432E4F9D2D26403556A96C
132E4F3D57A84A25434B496D9325164D88539A821F42496B60967B173352535AA24E1B37
649B6CB1523742AF8F7961222856A8576043204A835144663C2F47903342572D3F6DA557
4067486782A45970784E5BA36531658679A1D3362F67896AA53B34706F6C8AC5384B8F74
8AC07A2333807CADBB2428788E9195314C5B6183AE373473927C9E78355085789A872F54
C28C8385463987D5977E84466E97C757894932548BA94F5788393580AD494B673357A255
3663494B70A74A3370647C905A263F63564CA594372954765F98B13E2759A670
8583A0485C7172B949345777849B27302B7E5C8E663148697463A04B1C519B75635F213F
507C454D50225E5F83404863224EAD7C2D54373F4D9F9134483066B248284A5A2F759525
1B3B5A44AA9423214466748B3B27396866797C1D23416853797D1B254B5E58AE3A19567F
44585F2E2B385D4C7A7139327D6549582E2E3989594262253355A5524C54263A597E3135
4A323864933B684D3A84B3312E4C5E5BB08B253B5A5975D04C2A36535A9AB23B2C427368
9EB22F426A655CC6BC3753857C54AD5C416FAA77688F3A3488A0428235394570B2515E1F
2F77A6654740375198533B5E2D719D74476B2A50A78D4664514091B0443B5D4660833235
506DA09D332949725FB0B43A376CA279D0A5312850837594B33B3E4D6D64AA9239595165
62AE4A31579974666A1F2A83A2578A642E304985635570293F43677D4785474B
6D526F5E7A3D465A5E5F67374478704A5C2F3A4F5024414F46649353223D486879912626
443B6F7482192343565071742A384A5D77A62D2C305976828D332778805387643F457856
52612E2D557D6F5B5715314F5B4C762E244A94532C67272B498F3D3A46313D68501C372D
2D6D8932303D3A489596343A5645639C452A4E5A5B8B9A263456735393801C2E59996190
A82F3E54737CDEC04C3E757D84B63656768E598A6D3259B2B07B80513C739173536C552D
5CB7795490422F68BE9B65993B60AB8749729B4C64A14B2F555D5A8B68242D5F7A6BAB54
274B748AAE6F27669A62899B303483729C9A1E365D6462833A214CA98EAD8B324673715C
6A274B897A43824D2F3B8B605459403E40965E4D80382F5073582F56332D657D3D4E4B2C
4971A0433B433B85B02D1B36565C90A3222A414F3385CF502D3B416E56AC613B
AB636B61BA5A3F4C5568AB3442576A77983B2B626A649BAA194A647F5F86852641797555
A2431B52805335822E2944835E39572B2553783F5A5E2C3270443B5D3A395D642D517830
6D953C2342443783B12C2945554C9F421B21485C5E823632274D629E621646544C59955C
1020445762751B242F735D6A4D1A37435D3D5F66294C93743E704D394291623774433763
C09760923B3B85A55E51504152876526644E4E69B25336524791C84A2954767682CE582C
3B7F9685D04B2E5B978B9DBF543368B778B1AA3E79AD9679BC443A6EB278A0A1463E5880
64996735688A577C9A41338361516A2B44905D415D245EAE72436D714374A9557056358D
BA35506D5958AD942F495C6675B2643F54767E74B55D2447737558B85A29628C47958122
42729773A16733406265485F3C48555B7A4472742F4676704B7045263683C15C
B3A69E525F66568334498375597327506C863E755B325A736640633D49668A333241504E
815A1428655254B6491D31605B5C94273B3A5686989B232639517EB42919486E5492953D
3B495D5E9D44274C7A75768F32284F7B577D6E183E658B545D5E1B62913C466326256188
3A5A3F293D814F233E374A677D3C2D5C3D499F771F294F53569D5F293854615494482F68
777F81C439385C6A71949827357B8E6CA74626415761819544337D92558871504F73A04B
6A7A4270A3B365A6664953A16D39714357729C50366B3B75A77330573E3C80C0692F4F41
41AEAE35444F3B7CA92E3D685E76AB35506A536F711D1B62846397553A5D5E468C752249
A7457A9149447DC36297984B5B7FC165AD61366D93B95F8E5A304479A647666645547F48
4166383A809433617337478E5D234964454D94A4312F77644597A0351F4F7C6C
5E78B156655D6CC0472D6B49699C381F3D564C90792C2C51627DBB571B346A72A07B2157
7A9B566F63253A6C79556D4E27474E45404E15365456566A71293F8B9936712420476D2A
2751223E8C992738422D59A5531D353562B297291C3F436FAE7E192E384C7BA5281D3A52
407D971F33717D4C8547194B8473716836404C8445558D36326198556D6D1E3876AA4164
6427538D874478403F8590413E562D4DB8BE2D495F2F6EBC512F4D4B3F7E8731555F8364
958C2E55628D74BB92384698B36BCB593F57758876A42E2A87A85AAB4F1B2370745F9136
1F4DC36C8D441F43B76A62722D40574F565C214B7B47515D393288954678604371783D5A
69406CAB5B4B736252627D423C575553A2B53F324B634FC39F2F3E556A6AAF9E3E5F685C
76B3502C73906F7994292B5F73537384372E3FA7B761CE543A3B6F7A428B3D3C
6C6EA282AC465B597C856B3749727F5C79183756734F6B70282B718B43643E41648F5339
4D2F3E66742C204B483674A72F2C664B579F4F224A653FB7801F404C5F62BE36264C746F
81AD522E3D64488B51212F54826E8F2E174F957F508B3B2F507843473527527C6E30533A
2D50925C3D4D2E497B75323E2E3E5074451F5669513D7A2D2234504A768F27274A713E89
912B335B9654C05B23517150827B2945519A78889832374E7C77972D354B62599E85253E
7674546A7E343983B8616D88404F6D6F519089285A8B814D7D50397E82414B584652B274
3E5D7755AF751E3B4D607AAB3445584E7BB53A3166588ABB562476766BD24E2764AD6AA2
5D3E4D8D897ECD64345FAE7492973B3981A28474A2343C86AD5E53934C326EA76659551F
5590904A744A3C55814E46463656557E3926404E5F5D917C2F43476862A0703D
88656F679778415C4A74872D2A49406F7537214C614B8EA21B35475B5EBE752052885D89
AF2A1F548F8A6D882C515C7064616619579E884556471C5A9B4B407C2B276A691D615420
4B60713857303F5356356071344578461C2E38417A8A31283C6859928E14317455588B40
2255685674942E3B557771973C1D3671B863C64F1B36718C5E7C3633546974488D3F313D
856A526A494768A7598775385CA9505277503A77B9544F4B277D9273375748518AA35B31
4E6B658FC44D24497CA1B4C63758918170BEC52A38719DA2AD453F4D68939D872C4E58A4
7398602E95AD51993A3B577F59585B2B55896C66A04B42BACE4CA15E436C854C634F347E
A03A556547338DA43B638C4C53936E386D945A77BD94384D686B6EAD73423E768092A336
256291698E51405A6852719240416C7F884E93552543707549833B223D5F905C
7F8E9F4C6D8FA0943C4BA97997824044589D61A8354152636753812D4E6F9E566940453B
7B59316A3C294564511E5636526A872D1B395D58783215226C64789825213D4D3E9A7219
24547072A9292A5652479EBD47364F8F57957D1936717C687F4B264E8D4F7A6E2B286280
3778573040917E3069323076913C2C4B2B5F8F7A2F4F5F3E57724C1C565C3D478E56213F
414974B84831435E499AAE20338A6D83C55133418877BA8F20395D8391B8543B55849674
B8854371B6BE86AA5B4B6DA8907AA5343490B561644F2C4A909B3A53243A789452355A37
518FB5495251568984234B805A71A5312D624B658734334B8F62B85B2F576C56A1812935
6661859C3A43549678AEA53C69788F4FC08731417C987BAD6C31425E7F577F4B535A775E
706535366F682A626B3035646F3048674035725819445D3D617A722637526764
597476648E5450A956514136609C4836406F3C817229444F59639334295E685180762841
4F4B61DA501C23537A73BA4119457C58715A1F3D6F923E5F362F4A6D64559B302B54763A
4F7439527F893D61482B52627529782F35506E39433D1F3C7A611E40413C738B2C1D303A
488F7E2B2E5C8B6C973F263F748971992F3657885E85942334659B5E8182224180795BAD
782C5260946DA64145729C7E937A334884BB587D902D5CB1625D68524873A247526B493C
74A137334E3D467F843C435A6B7CBE5A2947747E838C292B507B8B9D57235C6B547AB32B
4591A879B152435D8D5C87844E5392738D8B2E506B6B776C21417BAC495A212B647E4B75
433B5D8E6D55753A4F62BD885C693C3C7EA941455E484F7C9434445C4E70A6742F4A5160
71B859304A515F8FAF1F48524F4D82AD214062573E81B81D44675D3858AE2A4E
83627E6BB6613F6966916E366274658C7C312B557C51836D215D485442A75C31716F6165
98481E69B0536C5C333986792D39492852793E2F4D4F3A615C1E344B47459C3E1F4B4C42
61A935214C6B519044283C5C7D52B6331E4663598CAD2924507A576C421E3D7979637B21
2F3D5C786B6122285C835087391D374C624776271F4A63513558313562703737883D449E
9D4A405A353E9A5D236A855767962F2D577558989F303955576F9D3045616D7F9FB23C55
63A085899C1D417097858F8A28536E8457875E4084726230A2522B85AC5196544164977E
4ABA443891A9446F382D56B54A5D573A50854B3D61499D9F3E3155617590361F49624F96
622C52687B779F5040476A7E83B5364F66A7886DA0355D819F666E9142378AB958815326
585E875380422C84AA593868482B498A523464452E6EAC5F273F513959BB7F3C
928F744F636A6C6A3A347560516938345D31253A303F6C8C21334F4D5B9DC52730535D84
7829193F524E8DB531245C6B5197692732695D76973B2A5351346A672431627B53753717
3F83655C6215405B68373C561D4B6A4F345B4023646264426036425A592C3F532050994A
1D29293B7C913A27435961AE89293A463D7CC962234C57626DBD311936717A6FC53A3353
A18B87A93C50587352AC573079887B809534419AC97B817B23588D8A598E4C3E56716444
A0644364876E346E515A70AE413C5F4C769DA333426A614EC38C233D6B4FA3741E4D8774
6DD43F19598F79B94F2B497E6CA08D24404C71849623447D636CAF373A967C3D75542460
9B4471532F4D7C8A5A7B5F475D7C5541603A42628E5D2A4F4A6B7EA5612D39647F887A27
357B7C59914C22355A5A5B923D2D3D776F6CBB602E45748852A3572247549B60
6A81A04D6F5288A234385E4185BB3144554A78B2373C5B78586F91242A87A36E84451541
7A6A549C7726337D8C56974813469B5B505A262F68934C4B3E305B5C2D4360364D994D1B
39634A67A3441F49517183812721425850A7721E2954636F9A3327597E477E5F1730675C
4850191C2873696463212D4C8060605E313462B4636C67253F5366335F49374A62813472
525B74A44F34583B4B7F963357754091BB473B536C58AAB12D45506A6EB7873155548169
C7942D56668E77B2932466999D8C8B414C85B46295862E437DA94B632E3172AD595E7B1D
41AFAA4E7A42334F89346D5A3E4D955A43603951A07C577744589B824054623F8D72274B
724B8F9C2C31446566B38B335A8E7B73AB4E343F807D75BB70235992BB7D8F43389BB574
58A8333E7E874064653049667B2F625B434B54693A5C803546486F2E2A553934
5F757086B8454E74775C61373A716C6E602B4F694F397A3C43586A3C34603C4F829B2837
62413B8670294840434EB0601B24453D9690252C405D51A0891A376F6180B73023738A53
737A376978887085313757848960781E31417F55795A3137636130452F33638F36364A34
48663F22413E5A757D2C1E26574D858D242D31544E827D243C706846879133528CA95EC3
713756A36E7A963027669460AAA8433E729E71AB622B4C8F5B5962344D6A845291A33443
6593515969264873A04F62753481854A3A6E3E5475642B2A5B6E74854F223F5F6673AF2D
2A4E937DDE7C40426063BE952E49596BA0B12658696BAEB530477F7383BE4F2C65B6688E
64454FA3725B702F4D53A05B74633B6B8AA14B5D5E3B5C6B7F382E413785948D323D4546
64B25632465B5F83851B3146595162751D4154604C6BAA2B41686F6B5AB5313E
835D5876AE7D454F5B8D5729393A337B98182D4B4757B3762054824D77C14F245C986B7D
B52B414971598C9327283F8A53713F2F29655452702C3F355134533F1D485E4243854D1D
65B040375D3A3C696C215E532F66B23522433B3F88A4213536415DBB45273F545F8E9B1B
326F7A4B9A381F457F747DA4223F599A945B813A275F715A4C6F21234B805B547F254369
674564662F4DA85D2A3C3D3C626E40434E3369919F40445845ABA8253A515C4EAAB42C48
6D7B59B9BB2E37739B7CB4A62A41826F86CF663F7A8771669A345C75824DA43337719A66
62611E47A6B3555A234C5CA96B4952287BB66370772658CE9D64593F407B70374E3D609B
83294B59566AA752274E6051A992363C5C8075A79B2C57809668897F22548CA281735643
629BAD7C9C3C2D447A4E466F2C28687767548C4D284C5C733977482F3B687F41
A795654588A38E96314E7049917A243A715B464C242869A45C7549324F849736503D274B
6F7643643A48595F4D3D633747997B252D423646934026724F37787D13376C3F4DBA771E
4A6760B6A61F49647263A9B41D43988C568649224C87737482292769853447272547815A
32302B3155553636452B698855272B36434C8F451B3C563D5ECD3C2049503C6BC13C327A
8256ABCA2A3E57816DA53A244280737A7F2E3D5CA17EA3822B57987071A2533C4E687A52
B949434E7254429829415C8D4A52733D468594385F443F5D8771356942336E72294D6A64
8F7728255D6B7BAA383551616D9880175675539AAC3123719886892947477B719846266F
AA68A5752D4C6DA16AA75B2D53AC7A639652357B9B7252762C4B5C968F3F662B608B802F
5D603E468A5F2F53585759A0542E4444515D9E8B203F56524DA29623384D8C65
62676C5666485C70535D424F558C4C3B36336587261D403F607162293349818664281D26
6161799B35253D5C776DAF351752874D7B5E2B365E96439B2F2B45654341651A2C607E34
76411F34705C315E28355E77492643353F85621E353E447DB4361441575E9244182A6E65
5F6C26393F5960828B283D5E7A7E7B5928457F9E628276364580AA40A85D3E3F787A579C
5C42699D845E91354A6683326B5B444F93514F6E423A698545416B41447871345E644D6D
AC7234625D4885DD4B2B69876695833E3D5F5863BD8D253A5D686ABB5E3F4967509E8922
63924F5B86444086823F843A3B5D794C7A552F4EA26E5A652B7CAE815F82352E8D812B54
352D557B41334E316EA9AB3E3E754877C279343D677D9EA4472F585F807FAA45235F9867
A5A9334F74936F785D3F4F64924A78561F4D699C5760833248729A6741872742
5C897E9A9D3C5E7765BC3E3D535D52887A2C286A92569A4D2D4063484C793A366B854348
54312E68732F3E231F3877953B4538276389321E3A595889731E374A568E9C261E465846
B183212B506B6CC25D214978626C7A2122346F509D6619335081529A30275A6B3848492D
2B574E31691E314E604A517D192D76683C31583453556325304D4F4787721C3438495297
5E29578C6E85DC3D457B905C82863750798289CB3B355C7A627DB21F4A796A48AAB51D41
89866CACBE394485A14A876D3146704C4D89313759954751662C467995434E2F3F769B4D
454D3E57BF7930454D578B9F2F5068549278243F5C6B5EBB362D7A7B77B9273E746648A7
4C1E3E956E659C2B2A50AF847D90354D4A8661916E3069665E436E661B49C1AC4E554921
5CAA5136582E2C618D3833503D366685373050525C6AAA4220325A4646AA2C3B
93774A59905343473E707C3D444B26649B4043253451504929433C3173A8502340514482
78273D4880628A7A1949456A58AB442C4244685D952E3E4E5139746113317C54567B2724
468D544D4B234072B16181432459B35A3D5643357EA738346B35588F6F1F65543B8E7223
2F433758B633182E575A69BD28233E556477B72C396277696A792B2F6B865492AA1D358D
8D6D9C773A49C98D52B36130576B5A61573C5F6983397C66364B877A3B6E452D65B65332
5F53517AAA454C5145399ABA3C3F5E476B8E442A447C596971282A64924F8C21254F6E42
95861E3871A17172234B5CB16F9357335B8F4B73362E5B7B73554E3A446C7333672E3570
84344F563A597E7333834A5E8EA1553B6E4A4BAEAE3447685A47A9B6372E597E6EB97731
42759781C24542546D6B86972B3C515D6F61B1521D54826E506D6E1D414C763B
72C3555974779F5E3E396D5FB6632D295947696F1E33525A40846213236485588B29394D
827D5A6629466C7A525E6623305365303F472C466B7C34563830798216274833689F2B27
2F396B76741D2A337B6EAA5929355575778E3D384F7C7B7A873833748E63A05120439459
878A1F30606044774E3C596E913D6532234B675C3071383662643D4F904841599A933C6C
434092A4302F6A4C45996B2D39765F7E8B29547F8E61A38A2E5B5F9861CD444546709067
8D562D75923F8AAB30394F9B9B714F1E60727A4B8D4C34695E4F68863B3C61743D7F542D
4779374C5A4B475753416B45308A912A6465538F7C2745536A7C932F374A6057B83D3146
525864A01F4B6B9160AA6A2659817563943E3373A56B7986363363B6AB8AA2323253A351
687B1D48769138515C214F8F682B61432E4C6459353F513E566B7E3030625653
58778B66734A538548652F294A6D334D4A2E4F6D433C4F324E688B3B294C515389601B45
473E68A436194367426B935D1A447562737E1A2A406760B2322A385C519450213D707565
923E185A94674466234E718B404348173A4A4C2E5A44214E6152357A2D3466411B39422D
549351263F414F60452D3F514F71B9631B334C3F4FBC5D1E455D435EB2603338687F64CB
4E434F8C7DA395375867B3647D653951625A4D8E2A335175507A8E1E47727E45795A3D5E
74783E728450679A6B337A743CA3BF453D726752A38F323D5A3C86CB54283E5E55C97D31
396C69A6D75B2C49546DC7793039665FB387266FBE8882723E75AE746C7528478B7F519D
53354C81555B7F2E5D908C393F6B3F6279583A57695B5A7D4D2E5C623E60BB5234416368
7D8D21314375767B8B23345A7D4586731D4B726A4C93A4322762947865B73334
5A56738D843D3F726A974828404F3BA0971D3060507B941B47546A70754C263E66875E8B
453342799D5D76471E487070516A451D46533C33461D2175673457351D497E2A35472E47
94781B1B4D5E65A02626316054798015394B55338D6C1E3D495E4CC12C2E5060526F3F1E
315C8557771E2C4960477EBB262B4C834761831F2A5B76304A5722215E7343575631335D
693A5C46455BA4543B484D39716D2B495D4380AF40275F54579FB1264E6B6F52B6882B5A
6F6E56D4A92D49546E52B29D52458C64938B363B6A8B585F572B42A0885470242C8F9E52
67423649BAB8714F254177AE558B374076B84E613C497C692C54793C7EAD3D315654479B
412B3B6E667A93322F5F885B7A94393783626ECF99425B597167D68E4B4D6086809D4530
54666D578B3331425A45498535394B7548496134324E5E60324A4438457D4F2B
A29B5063B1645F40537FC15570582C729339484A1F3E82421D61413C89853323432C60B0
61292C4C705CB14E1F1B4042417261181F493563871F182E504D874E2A375C4379AA1C3F
4F714D5C681A39406846784338475D4A47974F4C70A1536589204A8092475A24374C7443
384F344F753D1B3E3D426798482151486D72C63C20587C68976F251C49604E759A1B404B
6258957B2B45656B64983544507D57567A1B2D748749AC5F1D5D9D4B498F3421689E5051
5F38395DB4564954292878AA554D393E63602E2F63415F874C202B5D57959120285A684D
984C1F326F8AC29D27364E866ED25A2C5A7A71976B1A56AA6E808930327E5F5A7D2A2D65
8F65754642666A5E809431304EAF735784393674A15A675B3129528D4E6433345380853A
5B4F3C8AD16032584A4379A7513148555480CC5F274249656F9276284570826B
58BA705B5E5BA14C324A4F68C5642338554070662629406351B04E172484A363B833286F
B48F7A7B311E5095657D9B2A273763465C5225318B95447B35264A4D3647472A43594731
48273E538A5C2F5D365A818F2B3E394D6BBA681B3A35386D902330434CA5B43A25375A6F
84A31D2D527D527C41204E6B9459832E3574B29B4C5C294375AB4F465A29275160422D73
2939557C3A5B4D254B624723532B42567A313F67525199583843584C6BC06B1C506E4374
CE5B2D4F7197A6A938335DA27F9772224F626361B5411D45674A557534365B7440AD6429
3F713D5F7E4536866D458045303F5940664839978C40675C3578B4423B53445E9B352944
5D74896E2B57623D92C1472D48777184C13E3E4E7475979B264655624BC59B1A4A7E7F4C
7968434C9EA567AB3E40416F79547C59283F5D784759621F406865444D832A2A
59697DAE8949566A659236343D614370392E394B4A525A2626346344574F2E40683E3154
2E287279423B5C321B4C854B214740596A721735457587C041204A41599B8E122D5E4179
C43B14396E7691D0332D538463B98F2F366486538D362F4C5E4F5A81233E765F35513D19
59A1772C5323456A742E2D52395A734419314772767C272A3B556774AE31213B5D4F7886
241B3E7C54A980282060825EA6321C51675376914133689253A77B2F4763A75B796B3B42
5A783F82552F6C756259774C475F9555696A3D60829D39526534619A393B58393AB8732F
574B3A8AA55232474777B17639544A7B8F66164D764EBF66313BAD90958325437D64505D
3B3B689D5C9987433B8A8257657E3F48A8523F823D529FA23D4B9B4250A1A33233514E5F
7C61233744686687691E3F616164A0762A486B5B497A702A1F49866763B8313D
9090546996425A44477F7232443856897C234042496FAE3E2D564C4480702535655F539C
871346695D579B4E1F3E395C40781A2A464F4B537015438C6F3E66362C419F5C4C5E2232
545D22384624497D433A4B383A4B7D383D79505C7E4729583C3885B02C20465E4B897D1D
285C865E841E19306960819B1B344B7E609BBB274F6A64488E672D476D5A39753A2349A9
75324A393C639054404A25438E8F2B354E303E7A4D2C4B4943758B2A3747526B8072233F
5B615F9EA71D3A58626AD04F4A5B7D7E9F81434382AB5F844E204C956A90A0442B70824F
9C43262C8CAA6B772E2A66BD6CAE3F2EA2B74A733F285A752D6248364D783F537542859A
503952585298942B4662575CAA883848626C89B0473150935E91D8823F569B6175B03A32
70955D8E7A31465C94485495362D74924039804C246778584868331F5E836E3E
9EAD50578E68964D406D9F6F83472982C2687A442D458D794F5F322158946D4B7A24335A
7D525D572E4158752B384A36568E611F4A464466AD451D4C526E952E31317146748E153F
59673B6B551B314E4D69AD451D435E5983C25B1556843B91652345696C49641B28597256
59461F40666C30672B17315B57285B252230823C2B4736454A66223F84474AA86E252961
7068792C2445838181A033276691496E88303B6C8C78BE373B5D78667CB33F2C588D4A58
A441314E704A694E25506A783B6F2D315F8158397C3B46738B3E4E472A5D7F473D574357
726F1F375D585B9130273F7362C56125458D6EA04D2D5267449F8B194F9E4D5E951E244F
80619859285992506893382D4D7D5C98A1214A8563405F4536486F8D4B705E303F5B5641
4B402E718B432E5D514362A27A2A4E6E615D904829384F7B77BC55182E695D6C
52B27B6E524975863E733E4D7357294F75566D5A1F223A454C733013205F5B6C901B193D
6A51B19729293677439979192C3D714FAD541E2D69834896342977933F5E422A3E765939
5F1B21496B473E4D1F43756926412F2660AD7E20502E436C802C2F4F4055622319433740
738414254655489A5E1C35477D5587292250566C8E892D334E7043587C463D56A95D757B
28608D88465D3542556B67426E1F233678594A782D3980813764533850A66335503F4D5B
6B7E2B4C4245AFA929357A8C56B7771A3573865BC14B1F3A82559E6C1F2A486C8ABA3B2C
4C9A57737D3047718662952B4A4C604B86563E84914E6B3A3788A2424A5D4559A1563258
354979873A50564BA1A43D2F53653C8BAE41327177569D69203C55646295911F42636063
85551E4D97675D794922515577599E8720446368388A682B4E5F763D6982383E
557D7C9F653D57AA5FA32E306B93429D4C2E3C6D527686252958AE5B6B3532537268447E
32264A8863375D261C5167383A621B3A71772A364128629348293F3A5A624B233559665F
8B20134977587C881625748A50A261262D658949C74320335F557989123B535A36AC3D29
25595B55751A275C85454652204A6F783D6C4A1B54AC472057413B45803A2B3B38434D89
283151496E87561E3D63626CAF501C475D6E78C52F45566A607F7A24599480648781173B
49605E9A84204B77556069353D4A9D5C614C255179903F5C2E3A4F644666591F5D8B5137
502F3D5F8337636839639053365A5577A97C234D4A62CA5633458651909F3140706581B2
463C54686CC794204C836B67AC7F3C4467556E973C4C567F465A673E466DA64F524A3A48
977C263F423E4B7D722D26523F2D92B930296D543886A733295051589B9F4F2B
76734F6772465E41466671424235306F9336563527376C471C47265671652D3A433561A2
2929315D6D69B1201243796F778129185A84538D7E28296366638C212E446132526B1B4E
667C4776473838785A54843B454878354D782A2D76772D522E355375432749284CA4A02B
2E483176B53C1B335A36816925424B5748B368232657775F9D42255E6E5762A35C15429A
6C50883130578B5F5D6E165895A34E5E4F395C8062457D36298783453C6B313988A04934
6A3148848F333D5B396D9B6B2D614B627A7D1F5998786C894D2171646499A01C1D7A6461
934125558C659A922B5B8680505B1E2A84B14D863A277DAB415A3634498F5353662E4683
764A92413468BE7740593839759653575B5C659A482E4F5C5F747B3B37426B607C652B2C
6A9256726E1B2E7A83637584312B5197785786502C317F5C5F70381E405F6360
7A98575A6B5FB361406C5880BE603040747B9B961F456E80417729255DB048766C234E49
53355E4A275D796D375F51265E9D59305A3C2C7AB63624511A35936D2760483E7BA53140
3E3149B97D254D806D6FB14E1F575052A6E1491A6B6C45BE7C1A3A8B65708139225BB98E
8A882F346E805D8B3D2D3968425581244052853443582B3E6B881D35563C2C81A52C315A
4841775C1847553679B5511D3B5743A37D1D53615469D1611A5687448DB8472A508C5E7C
BB4623528869A5BB373B5C9463B85C4165635353AF303C7E7A21572F1C56AA6A555D2E5F
7E812B4358509E933741535264944E25566367A554344C64518B711E4E66567CA53B3669
7A72AC3D30398E96878B27465B7D647B7129546F6F588F4D34577B6F556D5A2567855943
6B51417AA0543B73693A5B83855A3D4343487152283E50515379521E46504679
5F71729F624665A26FA02E4054713D6D2F486092623542334866381C3C32406D7D2F2D50
6E5BB09D1B26566D54716F1A2356695B83501B3187937C7F2544516F5B9D442E5B704AB8
9A1D21447C487EB43431477F39863921477E7B374E2128496433433B2927583C243E2C37
526E2B202F37307F6A193035495E95372441577084832431538F5C6591322D418F71807B
2654568B46836F304E638F658B3E2D38694C6F7E3C44629E48817C28368670385854403E
7B8933603B1D4595593A5444355F7A2E3F604A5F967D293E75419E601929605677A72A3A
6A6769799A29367D5C878F323F6E875AA5314D886C57714352558F3C71583A3794905564
334E55532E5259336E6A42385B4F4CA48D3044866C59AC7F3150574F5E715D2934626666
9B701F3879714B887B333C58795A94582438556E53796D1E454F68406C6F1C52
676967978E3C476C67A93C2E4A64578E49212E666F9D8923264E6651C3491B4A713D617D
21296289416478112751545E758714344358355D6716607C3B495D1F2E75B1402F3B2F52
8E573149301B579E3C27464A699F6B182D435C77AC5A244E4139839D161B425E56B73E10
2455455E8917223E774A7568182E5E8437723625546166426D21394A5F6935592E303383
4C2F57382F5A9B514F5C3C3F87753548433162892C314C4B5D959D3C3852656E98772942
43765EC2A32F43546C80A641346480616D5B2853687C61898B323A82676A72293A514D3E
7B2A2D587D5550511F4C8F6C5570323293915B613B6DC8572F59614088A1212A4F4F5EA4
2E436E6B4F968B2E4173536687602E59924E7C7C3E5F62A14C77803B49618552746C312D
7E9A546471373F597930417442236FA133394E3B224D993330733C305FA23C33
8099506B8C56A04647605A5F6E3A3E4E5A4151511E3A837146531C1C4F8B442C432D429A
3B14364E3159B94F1B3938487EB52629523F3885B01D206D46656F203D5A88735537252C
496573A42D27294F5D4683202B788433607629546662557A422D438A73435B32345A8F41
4259183A844C2B4C383463A536304347597F7E192E52347EC06D1A43414A61BA75214152
6381AC3432486A6D87A019296074678E492C4487866B871F3660904F6E8529367879314C
8925284B784D656F1B4A5967365E53395B972E3A73384A6392434331346D79382F5D4479
AD5B194D5F4C9388222E48566DB037374E66819C212E778B596F37255A9C7F4A793D6280
6D5785383F486C486A641B5184743A5F662D5190573F794D3361B66A3F6A56345CA44D30
5261425B96471F395F50609A4424376B59689C3F2B38695D63A833384D4B5564
59785257514A7B6542544C8196432C514563A39D2B12395267AD352735506590881E2E53
7D4EAF693A2C5A7468703224605A6D4DC3361E369476574F2432451B38692D3B96892948
361E499F561A4B33556EB8531B33585675A9253051425BAD4D1B48484F61802B24454341
6FA7262F585B5C9A51283F615F787619304E5846557F2B2B7D893552362C2963963F5558
2553858735803F3073C47E4D85342D77872C43543685D25B28445B677FB44035505F5D89
9944314C525C89B32339597848AA3C3070AB62698E361D4FA47174411E537E785A9C362B
94AA2D695F455EAE434160323A997F5058354B9A7235554A4C8B64214D41476E82384955
4D7DAA4536566D5590A9302B58675EAE963340516072C7683538597164A864456169716E
9A6736648D97519C6D2B4B7A7032634E1E3770723E797438336493326D3C2451
588482B850434E8997982142645E6898433053936E616329374B824B703B345D67392B5D
2E3A4F4F23335E2F5A7F3A20394F376E8B46152A28488C962C425F3A68BC3C303A5D3683
781A2C5F714FD58B234F7798618C1E38576F69325324356B90376237143C826D41753316
50A5873141282F767B343C2C2554A2471D45594F6D791E1B41733B7A601C274184525F7D
1D21597F4FAD6319427A9063D13A3F5061497382152E64584F653321517F6D65AE4E2353
77693B813A2F67946F5270274864633D567B1F56A6662F51522B8587233C4244439C671E
41523D7E5A243966446A8C26325A5C88B62F455D788390364E738251A9711F58A7688E8A
2E4376986873512E58A28A577F36386BA04A51532F4866954A50613434A2B53A3B593E3B
91AD34315E71438C9A3029515E4C92861B3542563D98A8263F52755CB3841A37
667745968543613B52913A2F48443BA36423586A475F942C342A374CB76116274D4A74C0
2A21487C4A77A91D2A627C5E9A752C2C66583B8B58133C81466F41254C8D5D486C6F2252
853B3C612D2B558325294C346B81492332394966902A1B364956837221203F5E43A3361C
2D7185477813294674589B5A1A4073604585392253A0774A63213A4AAD56407A44254273
515772303D5F6E3F6C46234DAB5F2B4C5130A5CA4C3F5F405EBD6B233B5E4863BF702D3D
725459B477333E7869618E392A88897A90BB322C727E5681531F3876477D771B2066924A
9A2B2F436A445D452A52764E45763D47747E65822551A776426C4C3C71A44861534A4D7E
4638522C65BD9A2B38545266B36E1F39756281A8312E446E6A949D31264587505A872A35
6E9F6E4F7D2F2E3B553D53A23A324D6C3C4F6023203D714A2B531D2B8476354A
7DAE4C4B8260A34E3E4B617B862A3044814FB259322E6987485E1B314085525551292F58
672D523B212D4A382C6E291F45672131572D5BBB6A2334463E95691526535458B563152F
644A72B334223F798F70A62834444F498883123662464F7226204C8188779B3927457437
625B31537339236A383A6FA63A3450364C885B19335A6253923A243E55635B985C2B375D
5C5B9C6325536749A293293079676A97693760C9647D8A2D3D689D3D5D58372E53BE5B52
893E2E5D8D3E594E273E5B6F556D282A4B755B515D272772864348452854B04027524250
996B316D495EB99C2B31724F75AC405A5A5586B1363A7780699E3B2148793392983A5B78
6884972F306BA95E6173305260A358645C3E5478744776393046775731513C48659E541B
3A46495C823A304D46566194443F49624E7A8C2B1951684F6C89281A5B873971
52A368734347718759672A49937933713A2F8C962B3A49425694411E444242757A214356
95619E7A1541695D4FA52A0D3F78506B8E291A509E479134383A5237564C132D5D784042
441E486F552D6132315C785831442A59749E372B423B5394471530323C4BA25F222B4441
8691183F7D5A48AB461B426F3974851138748144A18523437E7D4971632346738248724D
1D4656924B65441F427D41475F242F5A69274D4C52637C3C3784604E979D3C3647745570
AD3A2C42604982872A3D554A63BA581F316B65998B2D213A7F5E7E4E295B5E6B426C272E
83A345964E25336A3E436224385545524A1F495E615B70273DB8824063482653A3574A60
3B8DB5722C555C557CA94843537280818831414D7B63B64A303D536760846A214A837754
71601D46645F5486321F46837267742B34568188409B33203276702B622C1D34
4F8469A744385983727E37374D8C66B9302649776A77592B2E4B734683282C4F5C465173
1D3D5F85506566203B7B8631454A2A3777472C412F35778A2939424975A7382052736297
7B1E28575A58AD8B22316A8284AF1C3667816765550F498E864382422E3357805BA22129
485D494142192F7A662A4B332E3C79441C473D335B7B26254C37306D8D2F2E4C56387793
283155785D9A751F384C5373AA2438696C53A7662D4880736A84582162B1637490513A65
746B6593394A679B54508A1F3D65B477816B234FB8AE386660205E782344553C6B9E4829
484440949A2A454F5D6DA23D39544899A53138585A7C9C30217A7A72A45F1A407050797A
3032669E657E5A3D589259737D44516686476447315074894A7C852A394C664363642632
6E5B2246423360B75B1F2F543F5AAD6D223D47352AAF77173A463C49A3671F44
6E715578735E5F445A8245404F3A46AB5A27473532897D3B2941315C856B203B54686CAB
411245733C8F9F151644754576802A576F5043743921425B3F552F1E4A6D644385542E46
8A40446323304874395B562C4A5842185038349AA73F23411F2B735516312F514F8E5F1D
43564D968D152142465BA454252F40725E9A1F243984726A8632443C5D6A777E1F45514D
5940742F365B864D5548183B4F6E3281462E6D7127543C3B46875731433C42407C6A1C4A
524154A76C234B7D6A61B4441B4A6F56A97C26204B7C6E926E154364539EA3392F93864B
89332A3D825F87771F557E5575593133865D6D431D456B50658341377CA5425D563B5790
4C565D4856969741495D5C78A2682B6155557E9E3B3069575077B7322F446B517688292A
44652F6248193A5E592945533F275B9236526225204A6D315D521B3876A24965
787E514F576C854138625C6571302A657F75A5A421457E646A9730345D914D8762213978
9A53945E2E36763C2B5F2415295F252A460F273D642A343855974F1722413160B6382934
453A6CAE213244604D97B41B304757509B7F23328A685875244B547E4A6A7F1B38616037
685019418350556A253D4A7538475C1242B1833152482347A16B213F492653A66C1B4D47
5253BB751B384C71885A1F3043464B9E413475643D84951F4D687AAAAB5C1C4D656E4B9F
76254C517D5B79492E4D88935C5A26325B9971405E1D2C5F8C40422D274A6534305B2D52
A4602C4D38384C702F2B4345918B343C534DB5A82A2C443669CA6329599C529F9B323474
6D9EB133384797789E9737446BAD6AC0792D58765B528B42354D5E545D8A483F57844D30
88453D667D473A7D674D607337243E4B4068832720384B3A9177211E456F4D65
556A5F873F4E636C597E3240555D4D75323F737956536B2E38995B2D5C342F5B8B2B326C
40288454293A513B4F924A1C4353445F9D21364D4F49743A164247356C721B334F4B5085
761A3C6C64448259214C874954681E4257794245522D598E652F4E3B344E93421A2F2A24
887415344A4065A44A15415D43627A211F5475658E6B182866A24D6D591C42777E3E5B3D
2E3B7ECA5C8A33205B7237585E25395E74367535315978503D722946B3B5542A53522C73
B65C42683E58957F274A53646C8C38395151436C8B26275F684F9B49174468716AA83323
78A76CA03A29437F7E6B7017469D75655E3A3797785C62264E88A267813A4354A3725953
31397D73384F45396CB8483A464044B09C2744583B49BD9A2E42654855A961234D685558
C56023445A596ACB59343A58638C8A424758796E476C30264C79484B78302D59
72656C9C624C5659829C2F2849487DAD2F213250409392281446745FB33D2931866988B6
2A3F39425B847919322751409156262A453F537B48366F6637513F225FA24D425E2B3277
71262A4A2C3BA07E1D334F5571922622385D4E81771E31567161983820476D5F658B1817
2F70495C65162B5C58324E2E214F6C464264253544894934593B384D6F2E2D514860575D
1F3255455AB4671E375149759C1A405E6F53A65F1B3F936362D24A225A8A6769944C444D
75755D9339395B8F565D8020366E8F4CA25E4142755B506E2A37555E335C3A3151826E3F
5435567C8A41575957727A313B364D606B2D354154618B2D2B635869C5703A4371658880
384A488C5FAF6B4147575D70AC3A464E8D4D6C6227394CA55E6170383D4E783B7B79302E
75963456322139798A3842572037984F213A4E2F50874820393841458042253C
927F4B616E7278345768555A5F2C3A6F7E3D813C283B856046342C395D802352362E3E93
523787292583B1282249303596A1252A473F54B54D2A39564B904F254F544F83BB2D2446
70567CA2252E47763A8A5F21485D605789422A3D7C363C4D192F557E2F566C18407F562D
4B403663912F2C434D3D8948171E44376BA5361C22524C53AE3A2745626369A33B35497A
735A903333409057965A2E4176505D912E3067884951451F369AA747666035496D5D2A43
3D466D834D37453D3D53B74E2A374978A4661E3F5D634C9330194954348C621D2C5F635E
A034356C896D8B851D56834565652656795064672D607153527D3E3683B4415B463B509B
62595C2544597E475C6A407598733E765260657647336258506B8F36378060357B97372E
554E4E9D5A223B455979818E382373914A65AD2528585342876B1C49556B3C84
6D954D5648507756384F4C55663229545950A24F23304C5864622330468D539E4629467E
4F47CE30292F674F5676253C3E744E5A4C1F3A4C7E4B66112D4552314C65374C91621E44
3B3D326B4D21564A5482A33F3F4A3760C96B22276546849C1F144E6334979A2E254F5846
863B1D3A6B64798826315AA1497747233D69855580531D36546A315358264850582E4934
37786D4626402A387C88334748394E98672E453F4AAF9F252F597961B1BF2F24738D5597
811C3188A16CB040252F919F45551F2262A774575A1A3D7A79416B371F437C434F5B3229
6C85374F3326457F3D40583E55873E415041839C4736593F6BA73825446552935728445A
72939A1B41666D5C85A42538636675CA7B2A545D5E70B35C4957677F5AB44F372C715243
933C2A4B5F354A68283656835A486830384E6C5C38563039508A432A54374D66
566F678D46515A7564892355895E47A3554274AB46796D2047604B335A1D3A4A652D715C
334885732F414F29649C3B1D52336253731E2A56565DB956205056516E7623275C6D4F76
32274E464640A82A3E6065487D621B2167A1505D38273B8D9C4680202F5467473F652647
54491E35342D5D8D4B19453B5C85651C355C31368596272D656135819A25356351566A6D
1748785C6685232D5263487A65234E5E6F61B25A26409D7F43A8472152A36E3D9937475A
744554562C4C72923C415F3C869A512C5F42537686253F443E4B7F622C364C5675973A31
4B4B6FAE4D2B62613C98A02A4B794C728021397D507D8D2A2B7C7371754D4351965F7356
435F755761B2604F7E9B575B9A48308CC6485158273F7EB04D4770332F7F7B343C39366E
854F234B434273863E1E3F4355689F5127394B618EA2382C3255646A83332746
6560575F5A5B5A4055684E4E6D3D50826A2D3A4C345B8820283B445DA63A1E605E49A6B2
183E706F647D411C234A6C4C94342B3B827963813E2857B75B5F3A1B558834446921277C
81273E5530517A571B3742425D8D2C223C515482671632346B55AB2E1247504463971F25
6F723E6A5C1525587B3B692418475B3A486B2D17448F59435A332A31895D4F6E22335A73
4145563D41645A25494D346F832533593C468C841E35515C689771284F5B6953AA692650
4F3880AF2F2D519A5F78C7443B78A474AF8B21538B415A691B26606864856A314CBB8851
8E515974A0477A3B1F5A966353642F4E8C4A46503F69993E3F593B7DC45D3A51494BA391
4049724760BB953762905563A56E295D835985852B394B9D6C7E9A2B42577961A7802349
8F6D3C784C18388F81445B6629346084425D43194567623664312D41916F1736
96894F90728DB6364D846A84813F395D9B5F90402C3270764A49182E4784347B2D21506E
3C35642A425F774440413241786A1F395C2F43923627304C48845117504B52668E262E4F
6C4C7E7A1A42576E668A2C304E5B665D91193862753347630E255F7A2D5D51273D443B33
532C34506C303A3A2341575E0D273933616C52232C474D427C2B1F484D4B7F94341E3C56
74AA93274D808658954329528D416BAE2F2D6EA1687580314B6EA662B286374393B45093
512B4DA7AA607E602C508473495024386874455F452161A156334A3B3B7D781B32575056
94553358785AA46C1940564A8C8E25357367887D334A6A3E53631F3A816756873E2A5795
59726C363A6DAD80A4793758507771B26332598C475F68293B5384574B703328579C482E
564A3E7080272C5258447988293B3F3D72A87B232B483F51AC701B3661665594
769B6B693F67878A594D2E575D502D56343D556D3B43483B377A362E524A416C3D214055
4673A43F1E50504C82A727355C77609C76323D5C7D718F26334E6F57A2672D4C837C3C94
552F3C6956346B3A30766E30496F335271391F4E5C3B5D992F203C4046967B1E3C57454E
7A3315346B2F5B7917304B8B3355301026508A3E6446223A6DA8469941273B8973325A3E
2F6CA37D2F5929355479473F5C1B5B915729552E2E5F752D1962313F5E7B2C3573413D69
7E24264B434BA97A2A2D6992739D272A3F875199861C416C614186312658843D3E3E1C36
80A557723A3A8E943F6F482542865F6D2E2E4E855149863D57914236634664924B243D39
467A6A2532435F6C8C8B2A4364495CB98228377B5664A83B354A936A6DCC43354876526C
6E25416A623F456A27235E80716488381F5FB4595F5123373860392F661E2D60
5565899743386C837D7628386B9A66A63335586E626D7D2949466459B51B274E3A356E66
16227B795069221229715750782E1934864C5F6429424F5B344F2D2790A92F313D2D2D80
8229404F3677DF6713456A58939833405F4F5C8F5F10506A6165A546126BAB686455284E
685F26504B37517D552D5F394855882A1F4C29253477282E4D444C76472024424556663A
1D2966965B932223316654646B233E5160418C3A22407E7046BA443F6A56544EC7393870
77463E6E273968A65758633747666E285E2F3B756D483C472F416C714A49443F8B963D2B
533968AA832B3B5C4495782E384A5DA3451B4E5A5775893554665A5F773423597A6CBD52
3E49744A7394503E78874D5E8E3B445F8F6A7F5A33445568396F373149525C396A5A2D4E
A86C3355682F5BA35715364C3777A54F20363F446D912535545B414FAB351E3F
945C52785E675948578147455540508C792E6A4B4B5B6938214427347A4B174C41297185
1B213E3A4DA4411B253D4963B6362124465095AD1B27354A444B332E37664053871E3052
57697FA12A3D779A2E55152B446D373F6E29256E782E3A4A384B784C21354D38698C1730
49584282822947515242AA32323F5B5773822229366855798C1F385B844C868328435896
4B6C582B4E705449765030577E3C598D344D6364264F58313083A633576C29488B88394A
523D5B9B34213B575A62752C404D82628840183981405F592947546756785232567A6736
7D3E3F547E525C353141764B794F295F82466C6D2D45B24E52662F3E806541572E3D736B
324D4944478E57354E515962AA434557616B868F31454E5960A87935405D64519E523539
7160606F6F3C2B6C893C614234376592427A3E2B4B635E3A69402F4060553056
768A4D6B508C923A4759417987334456674592362C4869585C6E2D3E66975AA346273F6F
49668B25347B8145605A1E415F5F38483E1936844D365C213C855D173D63354B83461B55
3A2D70C4362569765A8B271727634963731B1A4894577C871F2C77A067AA3D36415C624A
7B242B42703D475B1E283653325843152457681B474C1A35876A315A322F47885328684A
40618C32425B3445916E1B28473B909D4622545B37889F3744508E59A198373E4D6A66AC
6B3C54547046855C2759764E3F513550A17D306030315B78211C41353A8C6C2632553750
7F6B314A41638960314042557F5F2D3C465E6D7836395D52669D2925606655762D3C6D64
436957265453614D933D2A57726A6B914A44518F577B883325667D49567F3A27A39B3639
5E36355C783E506C3152695431363343498F642736523954655C2335624B60AA
68848E824B45715F878B304E4E50497920304A77386841383E5F425796361F504B234738
333860342252435991602E28484E5787671E1F4B528898201D40686C7A7520354D6C4FB0
391C3A78754DA23620495C3F572E3D5F625D327E27283D582F2A411E416D681D35343D53
913515383E3F99A02729435447818B211D405F46925020474860619A3729415A7155A333
346486627A772630518B475A5624388087517F49284680462A5F2C455497596675273792
AF413F653F3EA9C141417259646328315C5D56B04F18336C445C8F2E21519D4B69762330
7C7A65A440306B9C506A363A686C4E733D247FA8555F703F4FB77E4C5C405B88752F634B
44959F454A8E4959BE9D3D3F633861BB4F404B4F477CAD512948574A6483382054896B64
5E2E3E47714C4B7022376E623E5F5018315E9B3D3C442A494B613D4478262B7C
7B575094604F5B3A7D963F556051799240364E4858BB901C3E342C5DB54236334667B997
1B2A516B79BC4F2B3D8A798E771F325F724751361624434E304C1C2D606427294A264EAF
70183D38255BD9641E3937459873203D4F4E3EB95A1D47674A5385233C4174476E692031
807B3A574C1D458F754763212C578D5A38622833549C475F502A285F722F4D5B33276E7A
253E493E97771D213A4456785F19334757649F6116496C3C52BF4D194A7D5A93BE442453
A370798C302B48747B929C24587C666DC2492370953D575B3637585839524A3741834B5C
713A416660313C3247636A234054498A672B487C62809A294C585E69AE393D876354B359
2663884C7E99502B76A26A79914D3F718943726B434051634D90402F3C53593C5C353D53
6249415E4A323B6D331949343B6A85331B385B535A7F1D2756703F5FA82D193A
A360555E5D7980355C884F8F913649A6875480341D689A54583D191F6593457A2E28718D
323C5A1B377A8123343A2D63A0571537435499862332523D61AD35193166496580272D69
A3478E611E3182C84F68152F53724B6A6712314B605555641A3462512F71413C65762B33
513A30756A2E393B424C72372C465E5B576E233C386E51777A1E393A5B548D7C2A3E576D
488B82194796564D9D3E225F8D384A92412F8A953D6C621E29739F3464481D37745F283D
39384B944F2749383180B3582C4C4F6AC25E2D3A3F3676A02A3552475293762F58695A80
B63F5B8864518E502B5677446962444480528E4B3C469682787B43477F754F623E2F637E
546844435897623C5E3E5C6A8E534D65614478753E41534B68A58D2940496485A14D2253
5B4672B1682E2E646754902C343672766C72382C3F74723B4F2928498C7C2A60
64B56E6E4B629F5A9C573C54713B3E4833898642172E4B43956F1B2E4A5374C12A1F4D73
3F7C9F212847664D815F1E386497508C831F4A8A5D4C6B2526525A405739203A996C447C
2F446666233F4D435B866520493F46988C263352495D8BA52E26577853C23F1142894672
8C2721428E37725A29464D63417A35174A665F3A7D231F4C614146663B2547956153922B
2D51753135372F465D462351412833854E2145483A5E841E20505E4CA7732334616B5A99
4022307C964A97351F38828A77481D37476B4FA3532D375C6649552929498F3B484A2E40
7C723F53353D5D733E4F3E4C737A3F4F3E43C69A32485B3E96C0394F685A649E4A2F5553
56B1473954A06B73BC612E57536A6E8C36434D64669F8B213A599377A4802D2D7A7C4D48
4E3B376DA34F4B5F31406A682F473A2153706E333D4338758F401B36625467BE
5E7F9B8945426A70DA823239716BA38B1B34617757834E23364D2C496926375C4C2F4434
214FAD7A30342C2C377838313C344C5E854C3B483486A52E1D393C479A6717193B4655C9
63224A5B4396CA2A3C747955B54C18487D43689A4A2D6B91414E6427387E853A51362F42
5D4339533537587B2B2D353D579450193541545C9D371B444046718C2F253543625DDF46
347D7E4382971D31735A459848154457573EAC451F5C774C3D6F3120477A3A4F5E373366
9550305B24415F6A2E34443C585E2B334321358A953A3D5E487D963C2347473D80792C4D
575D6EBC2C36667B5B96462B6B8F677336256EB56F6A683143765F6C6C263E50684C801F
31998531506A3B4299AE423F673851BD7E354C51376EC2613F483D3577CA4D314F4F6EAB
561B3850675D659D2531746949664A22285B82597B771F2767A64F70973F2759
A74766885E685B376AA5405138444988504A553B396A6321423A3B4AAA4B1A4E5A6A8F58
15235B8D668C2E222D685886992B314F8855573C2F6E7F4E494C1B335D7131625E1C3A71
8C3D6B4B2A5F979342612D437F65344B542261AC4E1539682D5689171C3C5A4AA9741537
654F64BC50163E7154766F232F5E924A8F8D2A3D5D833C6C5125637E7F25325225344972
374C453C556F53306C303BA1843740633630ABB12D405D5550C07E24464C485698313859
5B7676B42F3F4B726B7676283F554B628E312D2F726461631B2B776E46895C4044724444
6020495C4D2C593C435C762B4359388C6A295640326FA334615343698A2C3957626FB53D
286876688A9E364C548B848071384568A96A924D484E649482A73A3640688A546A3A2B4B
864F597A4A3A577D5B41693D30657738324A343245782B26372A2757A4352C3F
AE944B8573867E315993639D61455C52607B76212B68742F8738234B8A714D5B1F295082
45515432365B62263C382A40785C1E3C3859705E23436B32597E2217506B3C8E681C1B64
745EAB40202F8AA759AD1E244665436673242F4B92478C5C242E83514F6726295BA13339
5921328B992D583E24335D402B402D203A774733402F41516721394C4F54865C1E434858
57A94A213E503C529037273D61437A9736414085406C3E253F88BB5F7D60304584784D7A
3E305C86743E75243C4D9B3E3F4336418672365541385AA0423346423C895B244D4F4D7F
B343466764639755275483488293353D623F9759334896608C843C4E6370556B324B6A6A
4483463E82A662579039507CA3603F6146688A66324C36406AA28B3441454440895B2F42
445753904F3159655E6D9B423A426F9C5E873D31475B5859852F202E8D87597D
89AE8C6E47698F497A48467159413D4E2773886F344F552A8555212D5D565F98251A2D57
4C7660181B3761617D4E263B3E6971C935233654388D73204C62492D863A22415A4D3C5D
1C3166773650342D3F5E4333612042657135393E4168757F2538395B6C9D48172F6B4E76
87251738814372841F3949536B8D221A38385979A32B335D553B478E4B1C61A1434B6732
2D709431464A3D4D855D33551D254B8D57326C442E6D9B363D714C398D88292D564E3793
6C1F4E5E335CA45B2E4D52645F882A46595B6B944D30505B7B6C931E3B5F744C8B7A2F56
984D79A7373A508959966F365FA75A703837686E3D7146416396524852307AB64A395646
54AE71264173558AC8464360766C7DA5384B647E85A9482F4D83AD6A817E254C687F497E
411F484D864B75432C5F4F523A4E4D213D78742E5352223F6F4C225C413C516F
707C87893F4C5E6DA84F375B7B6A858E1B33545D5B9F7D3D33594490962E416F6A678921
353FB66B756F1A37426C3E465C227091903857431D63953F38503341828928454352509D
5D2852464DB298253658395DC852175DA34266B2224B4F9945877E1E5354724074641D39
926E437A431D2B573F315A1947786C323D4F1B57875429424C2C53A647253E69456AAD3F
1D4C76556F4E2D38685A6B934B2148606E59B8543D4F808771B53E374C7B8B57603C475D
74447A6B273F486A468C3F294E52523E701A3354773D5587224EAC702A6A5A3066B05538
4F4E63B05835413A49BA8028406971A8732D566A4CAC8A353D7471AA942B3C7E6D58752F
5070AF6C727141638A74406D4339598C91597C3D387D8F606786364C5D9761356D4F5477
953D354E4A488374233F564C4D798723354D75549B821B2C5B934C8F7D1D465C
C14B6D8467BA673BAAB7489249435AC98B4E732D374E8E39533B213D873C335F384A7850
1D4D3E3E54A5412545525C808F2D3B7D92758D3F233A615F6096272B46804194681B3372
8A5E904C1F50B15D767B233D636B4459492F4E6B503172454159942F2E5C453972AB471D
4D442D8288253C3D476A9A1B335F8F4D7C6A272863794D7D4D383D637A5074551A2F4A6A
4369393148472352541F2F5E4D32726D1E435A742E49322655727D2974402B809C31304D
412E70BA58324A4A5FAAAB3343625F57B350294865809EB03632588351A14C294D8F6E67
4C26556FA5877D40205B8266933A36847E4371403663CA76586F49837C4C3A6650558C49
2F514C5494B2373C7162539F883142717266A6472B40515B7BB2393337696470983D4468
6254677E254E5181736B661B3C51566067872E2A5693545462313A5868403C69
89764B6D4E86873A5058707F6035424E668F732D343D655786401A4A57505E53343A5C74
449C3E334248773562371B405541436E162A6A8B334E491A49524C3A5719354C65382D36
354D885515303D5E6FA13A39564E558D7F1E2D796D51B45B1F366C6571CC392D508C9A61
B83E42506D4F5558293A6A623A5029326B803F375333318794333A66393596B83A375230
40A79C233D552D61BB56235E6E3E9496292A74A54A8940235479AF48BA5632456B605D6F
2F285C965F575A262966A67E4E602F435671618B3D33646D39526127396D823F6934477B
7B3A414A3D59A2621B3B3759B69739515E5FCA64273F688CAD8D34596E4D73B05B415A83
62A84F4360817C8595405C7988628A60384A856D4B8159393DAEB9575B472B3769763C4E
5D3557976635512B4A95824A2A4B492445B16F203E4E346BA03028427E6E63A0
9884776F4076955FA0443565A15E837A1B4C6F6F589B253660584061752E4457313F5220
4E65521F384B4354738B2332513E67B147264E5B4C8493133F62555BBE60164747747C9A
1D2E5D5D53753D1055897485A618264C746784A2343C55753C5F6F203E64411E4D363044
76632355363D9394253F4A3551983620598D52609D291D4C9D6265AA2A1F59B16D71A721
3B519A57604B27586A66506C39376E7345558726436569355979383D77A34C6C582F73BD
7326405F4068934A373A4E64739334385A5258BE95255797656B7C233F4C4B49B1582655
7F445E671D2F648B5A69382F42694B733819486945865123558D4A4D4A2B3671763C5327
53847D3573534660B67C30525F397FA83C2C5F685499552D464F8E78918120556F636D87
6D27648B6D689C352D4D8C643C5739272D59B25A59642126616D3656481C2E3A
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.267442 0.00276855 0.488926 0.113511 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 0.003125 0 0.003125 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 .5 scale
320 string
320 160 8 [320 0 0 160 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage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P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.511074 0.00276855 0.732558 0.113511 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 0.003125 0 0.003125 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 .5 scale
320 string
320 160 8 [320 0 0 160 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
B793CD6F504F4A31642F664A84521F564C28401F175859354E22154250686D6A192C553D
371D503856562725693F3C371A3B59582538755E2247404A403B4F1F5B7A53333E440B2F
712948272C782F3C3F553A452C4E2B7773885B454E523B6F3737286D4C5E997F673D703D
675B674E2B4A60314E8457465C395D493856554350534F43804989515A6D3C673653867C
4CAC6325486E655A836F3C776324698457446F5747A1764B7A2F3686826762704540668C
695F264F78383E9E616AC19A5A94865555635F377B65678E5B465F414FA2676995415FCD
683C5D3B2D7A796F441F8F5B3A5597A176374D6A42706F3B23986959B48C4F2B54683C38
3C6A497C9B6D42456F3B66524267685A3E9BA64C3B33715B597EA49A5A4776794B8F985C
572E8F51387B5D273F6444666A762A5BAC4D2E55649B854F796263376B593C96
B187B5643F504B236238766455682D5B3D232916195F563E4A31234A5B5C60963F284138
35454636446141246D40354A4B5459682D3563662C5F635B404A5F334C846B37487D2336
812F41492D883E3844694C49405F2D708D865B4B494C3F794B3C2C5D3E43A0AD6F4D6B30
795E51651C3E6A3E5084563776477772355D583C677B774277388888534232745F465D44
45BB8641497C2B336877409472476278603B62753C6465617A523F846E6567575D755598
79724086A0733BA34B3CBAA25B8D98384261694A71303E69624F483935725C89B6253CB9
7F3D724721417A644D309A6152385C7559377A9E5A6A7D402C7D70478D67422456707C59
49703C81BA9257437B275D67525E66563E6A7F58452C6A6A5D5B668963446F5A32847E74
552D78843A605B2836703D6B66853B66A171352D2F807856A99E8639715A309A
A27B8D74455E6E2C813D7D68346B35593B1B291E205E4F42473B2B495866568D412E3227
2A503E2D3981522160473E525460565125346D5D23725832384D5633438E322D4F70282D
5C274458319332395C9055434F382150716F4E59542A2D51453C33616534758F66528B41
5940417F36455D204A8486365D2963914364412544547B4366315F845164384433404D54
35897E474981714E66752A898C53708248588AA1487C323A677D4E8B5C43708286685CB3
85652D5D737B40B2955AC9A63B7D955F4A5A7F4C8A76506664525E512D71507B9B4D46BF
7F4189812D338B5B4F2B7F62613E716B794A488A6263714D51888C4B9974352F737D9054
447053589C855E3C8749566062595957456383634E2A6E5D6C755980754068894F6B9887
4B2D6C85416871333C743D65646F2E79A4873F2D37842E3C6D8082376C2F1C7E
BD8A6E7C4963762D7C416F63205F3D553514282A275F4D30505432475056578E3E383324
3350422029826820541C2C47405558512727583A1969846334475B4147916E3B3F773329
5F2F51592D89212842734256583D25738E7439667E593A563A291D3B724C91A256408D4F
614035733545512B315982386036557D4551553D484D6E4A82565C785093454B313C6B80
529C6F30578989527A7B3E8EA23E83A461375E99496D705B668662A99F8C73619A5957BF
AD9C3B6E5C462F7E8C61C1916A748E634A544C54A17F5A5E4D3F51552B683972A86D459C
8432637639278C5A503A7C554E3A675C774B4785666A6A452A6A9152886E40234C5E7D57
40686163916C5A3195545A55444D5D4A487872413C26764C6D81588780396987427C8E7A
3744848F4E616B21307547657B642B60984D31384A8977396A7D562A7959216B
934559674770391F63404A1711553849380926322647481D3141284E281F317743374730
283E453520715819606039403843687341275A3D19618C80454D451E4086653A415C2C36
834B5D4C28745F405563273A6360347B8D662350614D355243311E326035899C60478E58
5E4F3B703154624D37527F4C794F4D6630626D445257502B68644B783F763E673D314D6E
4E9A8B4741745D3D61735278B15B796D7E406C8745848F71707450918E8E6D6D71453FAF
88904A7F5E8D3A7B7551C0866A75A1634B4C8539A18A525C82563D5E2869526DAD4D3F7E
8E3360583B1F8F543540604345407068684E4279726C7A5636678F61786746294B52825F
42817A599C776436924459492B4D7753477269532937907E89765D873636808F537F974A
1B325B72386E7F3B4B99385758714884A1753B222F757350868D8F5757472873
A27469723C4D5A2E4826544F1646394A5C1B393120315A4D453A1743553F4D9A33336762
2A13305522804915545E353E2D29607B4E29475A2849817D5057545F346B6B37554E211D
62434E47236D4D386568774055713893AB885B585E3E3F647253262D5B2376A0784E9228
5965396A233A56553F5F67306B5252823D435F2A49657B496943467D4B5A3062513F564F
3485A56D4864775F6869554FBF7D867A9539648A5F809F7573918796948A6E889C8A49CC
6D7F3D7C659449748A46A1796758A6664F588C2F848F4A6D7D534166227D7766A6644775
9A3D6168342E8A9C6F5468A847213F3B6F494F77345E7469375C7E4C839A5C37484F803A
51778D44B689212B7755616C2F2D40563465867C3D205964485A8D9B4C254A733B819B76
3E3D548750546F2A3B8A4C66566B4A999E7D46282D652C5E80708C5E74441966
7A5CAF9E4F50703C61483F5F1C321E30551C3B270F1A4B413E34152D4D405FA641224163
30392F4022838728515F4357663B496D5D3055804D4B8A3F324D586B326A592A6C7A401F
682D3256396E3322607879407236195F9C804660834327383E432539753469897044A35C
444D2C735F44513D334B8E4D6B573D7B65406A47435282616C714588546A5455533C5580
5684A374385E9568706C5E47CE90737A9F4165AA757A886774896C86A48174B490865DC8
7A4F4582908E4D707429828A6047B0495C7E882B67463873875040542B70895C9E5E1E5E
8B4869712C1A73763033595C473F5646915E40726F5960464C7866264260633D5F66914E
375F6838958A5B37725045503F41584F2C635B5247296D767150868A6E418EA971819568
2A51809D6A70993122695A4F9C7D316B756448282C675C34603D7B66686A3C5D
779FD0B85A604E2B6E6C565E224E50304F093846231B572A3D5D3135432B5495662E453B
2A60412F105B702857312746693C5955151C445F3837645341464A47287D73295C5A5320
624B30654357733F4B786D417C62337DB583353B565F364143371921634A79A272379E68
385F41777D41497B444E986B5F79457F6B456A54494B7165607643886166584867455C85
6B72A07F3764A8696B6F4E43AF715B87874F8FC77C657E4D7B7D626E903A42727E7B48AE
8A74465C63794A70934197894A5DA84B323E643A70864C5B84475862235C6B4CA7723646
772F5470561D7A883B515C7252556B4B9C6F45677C6E656D3760906466665A2B3D4E9175
375A894498957D45707646544E466B73555A5F62303E85978D4F9D9B562B4A945683BC93
4C31417E404A7E522E674132997D4A8C9A513A47516F5B6A8D3E7A5D3F6C4556
5872D3B9585758294E4D413214414129581F31400E136D3E263819385A2A4A846628546F
34472D502574621C475833393D296C795F233755442C7872574F4368226B833663452C1E
59653A674060854E507A4B2C8166396CA885654B5959433F45482B2C6B5D6F92A3469968
34604B748B43488C46518F6F6887467A605675605048614367873C7C2C665A3A55294E7A
5D5CAD5D38637F625A873F388F5D51977936537F745A9056636F5963B88F67B5A37146A1
AB7F4E9076754459864CA2A07C63B56B45416035619055618C4B3B662F4F5D318682413D
7D5C5484711F61B05E546D7548557F6692513E6A6274986A326FA97C7A756234405C9B83
3D6C98408D9D663E8F7846614D2344556B598F804D2F5D764A3D8496893750AC706AA088
76433F937444876A38566343B1A35E8A95655425305548509B4983803D59364B
5442A1BC814D6B52433B366E28414E2153282F4F19154C34202F1D264D32648C4B183969
303B33432665733F436E362D5237546F84362C504524745957514680325F9D444951691B
4972386335378F4158766F476A593965B3846C47483C3D507062342D6A465493A3537D49
33664A7D4F34587A44556A31626F437540315F57444C634848773B79535858375233496C
5E47AB804153804D4B8C634E986845767E596A8563528B6A75755248AA9163B88F6862A2
9D78558C7F9B625D9E5B96BD6753C36A584775445E96565D9F664460373D7D5185844B36
83634980683071C96858817770283B3E71434880523D5E83326194446AA67C613D47834C
335887438AAD5F29534F3A665F4748564A3D7067613A62765E578C8E8F3D46B48E709D93
85483C7B6A498A7246496E469B67616C9269552B335C623C967A838337513451
534454C7AF586B5F3D8740813444722A4E362661261550411D2C2C2E3C3469735A203364
373C3F412A5A7E4339783C3161495D4C7B3026552B1959464B56507C364E8C424090831C
4079456C413596295D7D7B556B60283E706F5441483A1D3D455235336135456587447B63
3147347665313C5C3F65856764693F7D83497741374683774C793A5A64544C3C6C425D5C
45398C7E455A9A6D4D87494C9A7F5783683377919D5A965F7D90754CB16F51A8A99060B2
9A5F2E70869A5850852950907B47CC74344862425C7A397F8E60544423539E627DA1394E
85674CA2251940A659417F75805653589075416D685A765441869B5A41555D2A395A8476
4246765B72A3833A64834457553F45535D44735D3F3B536D626491828F5A39BC8C68997E
74465D8884547A7641677E4D8764515CA98E6332366B41235C8A94642E4E1B3C
5B4548C0B95D656935853E773C577B314C4E21602F15433A1D302929403C61787D33365C
45514243274EAB57316D38566E446058512E476444186D3122313F623744851F1F56671C
38451F494F3077173A59593F4E58232D877C4E3E586E3B42473C1B3B636E608CB4414F6E
3F54295182474442274B75806C763F5E8E4569583F45798E5488496D727270466D48597D
7440976E336B9879487364377B835781735B456395578F5A4E616C50BF6B4C85928659A0
976A4B6864803C619C526BA57C34AE6A392E515566914E4464435C602C31783B92943620
6733365A512A37C432337B3D4254636C7E643E5C585C916E294F8E7E4D6A581D21426588
4B4671707DB47A2D776F375E4332587654456A5A3E416EA367678E8C812B42A47F4D8B70
2B224064654385371C48693880A0403D7A5F512E3359292D3E746C713983473E
554E3EA1A9635848336A3040152F56354C47195C40173C3319312A2B4219335B713B3032
1242373C0D286244225D252C463E585D39231D45541D54685E3A3B552D449E541E49601F
34854154652898614B64795F4B6D473799872F2D3C3C43475249303347524781AF6E5162
2F66534C8D50547E4F55747D7B9E595E75487F6843465C3C498A49604F4B69474D37395F
7241A87D3546A98447797049999252827C70425570508D8A5A9C5D3E8598788C904D3D85
A68051677082513F8A5B81A3643D986D38324B4B5A8D6849774051593E32A06D7E893124
5F493059433744C75A458D466330507D82743A5553377F7C39717F2F35828856224B615F
315F836D72BC601886753C5A5B1F2D43573B5E6246293B5064587AB2753022886D57868A
6F3C3F71552C916E3D54691C558E684B7F713A3A52664E4044949B512E703D39
51433D806B474F4E2B6E2E59432D532C484613352F133436192729245633435F7A3D3551
4534274A3D37973219723F473F294563784628503B14365B5545454738327B662349671C
206A374F58298B70545B772D3799612F8EA37742453B2C4F6F7755343F4C3A7296844A6B
2163634054283F6645536E3D4D79555B71326456455B74724B8A4F628761674771503A64
5C3CAEA54A458F65468F744467964884717835508A688B8E4F827C4D8EA4838586556077
9F6F53636EA16E408F826DB18642878F4D61737147716A5AA0735A57414CB88B7BAD4148
867528602C1E219169438A476E433F526C6B3A4D705763743E758F6339667B492643537C
4A357C7553C47B415B743B5E5B5E4B5059305D967C4F3B68462B449A8353268B6F2D6E8F
74363E634E2B785D473D444463826E457E905E292A553E5735A1A08031553A39
54664D8D7C3D587237491E4E573C5D1F2B532A453407264122332A173C394A7689212639
533729322A35B7691D732E5571523244704E4A676024323B3842575755337C461B42692A
1E6D3154832F88665F6890723C972C1F708E533D715D32263E4649373F5B43608561487F
455340435F4B48755857769654804B5A8C4C7C4638435885487F464988564B34674E3D64
5E398570493B7F57315F6F434DB768898483375EAC70918A508F6B4B8D89677D9A77757F
A76B43558B936E54AE5234758D3C82982A41585F45643A3372724F6E343289816AB34034
967B2B7C563C209C6634927C7A7355717E6F335C88688276437696885085833723415D93
67357247389BA14C72731B455B5D434F56284E63574F3C6459673D99864D257A92604E8F
8C3D3F5B723E83755B4D4B6470836E49929C73333159575F3DBCA87B365D5240
5191607C7831496338683D594735764036542039551E274F0D263E1E411420527F462C35
1D373E2F1018665D1E67242C435334514219263E5C272D39141B333F4B2B8C6D26325E2F
21702E3166307E493633675D3F9C5C2EA2BE573050503D345545272C3C73547597743983
5B51554F4943484E3546678B5278704E8F616A49464C5573518F6F5C895C91555953448F
8D4E90754855A68C43787A2A56815D879F4D506C93697F793075804B8C9C3B48746F6D81
BC7E503B5F9027398AA84DACA63A5F965D44454056987E42826548876D40A86C5BBC6643
816A27686F441E968B1B48527D6D4F7676792D387B658C58307FA28758898255173F6495
683D527838899D5A6B7B3F52554B465656506165615F32637A7A45987F652D79A9745BA3
85455980753B7D6D354481737CA8691D6B7D79373B60652D2B70695B31673923
527B667C8E434A2E1C433A3D1E144135374B0F1934151F40131E2B1B4140395084553140
3B313F3D402077521C6E3D4F40292E616544233E5B2D284D49304049231D6A7B303B461C
1C7047325F2D8188534A763228A4623397B2674C5D4B313A5E472B3132724B6A9982447C
595969575448555A5541697349877552722D595A414C583F2E6162406A3B6B4B513D2348
654190862E255C57317792404FA93742768A3A4C73377AA45584733D76A09D5B7D7E3E5D
B2A47F497D96723A8DB85081C861558F5E4638673C5C7F3C7561466B6B399244429B6040
9C5627534B3F1D907A276569673E4C9D88763F3B81538E7A525E86654496A851272C4A7E
6D4667914E68AB59808D345957413D7C64545A6B74564F89897943918F2D297E986864A5
6D2E3F48582D8C9E5E406E2B669F7A5489825A2B547E6E5A2E797F561B70553A
546138438148536D363D26436D33591B2053272B4A0D0E422521311E414C3D57822F273F
4B3025273D1F825A296340434B62314A6B53243E47141C584D2F364B60296E8B3B47503E
1F72532B6A3F5D81554C615C459C763886BF6B515940333D8E8240333273316A929B4D7B
234D76505E1C2F4E523E5D5C3B5C6E54815B4D5438587B8151773E3D7C5B6D37504C4158
5F1F6D96652E5F6D485C7D506BC07E54679C43517260528B66868871629C965482726458
BA9E8D447C8C874078C0557ACC5C4C8B734B42854C4D853B699A546A653E917954B47E4C
A49037525F3D27909B4D7B6A703A2A564872433C7345515F5D5699714A5B55773F2F4969
32334B7D5989A43D506B4066704532495355509D8648294A5B4235B2AC5C2C5A82357097
9357404450194A64423F5845659F704DB8A25A29283A4454417F7355445C3C2F
50986E4D8C3743624342604B9036505F345F4A314C2A1243291F322F2E65436585342131
4B2A492C351E7F69345B4D3C4C78365576672B4B6D311750322B3A6E742D6564234C6450
1D683D2D7352516B45647C6B4C8F7917337F692C5F6B3D1F45454B2F4B8D35497E88327B
5D4A5330775F4353443C758E61723A458B6D4B452C3B4D563C6A57387E5D56475A704A57
76657C885F3661786A699F6B67C59A585FA948538872468F6E82968071A1A5607F68705B
CA8E6644ABB4783E6DA14A7AC35D4496514C6B774C41704D999F54596247AB864CA26F2D
838F364A4F301A80913396A18E704367558829548558695F6C5AAE9258706D89484D7599
8E3152643C92A57C637D234A736F51645E33336676692D5D7247276E814C284A9F6C4E6C
745B546A7B3B5270504D7E695AAA5C194D7C823A3B466B2825598F7246584131
5193654E964F3A54403F7345672D5376357147385D4E154729224B373156417FA556323D
462F5D2A3219677B376333335F76376E7B2322446941185F292221425C286C803B2F3D2B
1E6D562547314A82302F514D3B809E5057BF862D3B413E3C664D2B222D745969ACA94172
684F52476862434840304F7D5F63704488774058604C515B465F7E43916B6B5C568D5B71
8576798C74377A935F568E4659B0885666A25D8CBC7D3C8561AEB38165967B5CC183705B
C9A04A40789F863D6DA6656FCA673EA17C3D3F5B513B623B5F8B4E5C76438F8051B86748
73A33F5E775B1982B71D4955626E558499974D25656A7B652F428E997C76958D202A4172
7A426B874B58A45E74A6424E523F2E7A7B6B4268642C3770836A316689292D7299714F7E
6629343E5F4D477531256E5251BB8557829D80374D637156384E5A414049391B
516534305F48394619234A3A2D13264326751D0E39321546100D24212D330F34695A2A4E
3F1D46283A13453E194C4827312C2241654B21305B1B195961552E4B4B21518C5537434B
194F642C4E1E38833E3F594F235E9E5F40C78B555945423F6C6436293585585FA6BB506D
6C4A695A72744357783D5C85606F8E4C8171559179524E5E44759143824284624670345E
7865628D3B2E5E7766449142449C784B619245516E6C36745B627E69699B70537E91684F
CDC89D45A1B6713688BB7F7BD1702E94A6504D4E2F639761748B3481884D7B652D6E7C49
6DA9272D593F3E83BB4F676C7947444EA2A95E286F3C666C6E337E87273967765A2D4785
3B243F825861AF493D5B625159513A444B70456A6C5E234066773B6881543E3683685795
A26D45375C3344A6563F904F268E97615C9E7D2F262F575E444E5F562543512D
4F7F5D49744D375D463132285538232A124429193A170B2621172111162C29658B461929
512E3A1940265854494A5930336936545B7338395E621E4262593143774C4D9364304A66
1A5776284A3A2F9B3B374F5E60419F7540C99B6854435A43648F622E3779514C9AB85E5C
6E4479516543324B703B5F69384D744B805D3B84604F4E614C5E8A41895D755545954D54
685A60B56A36526C4143A16448A76F4473966E5A89663899837B79314E929D5FBCA03331
8CA65D4A95BD943057893355BC9C235B6F334163372558468AB1373E633887A060897C2C
5F935C375E4C174FAD4E6C80665E51434397683462705E6283388FAA934C8A87512F4C83
7B4947904E79BF8C5F574B4B806F645A5D613D729F76274266532B79A87C4539831E4373
8D7B4839565039725C3A9085569B745D2D847F3B2634767B634D545060454533
5E5F69527B59304860236D3E814B2346254E331C371F1331291A241F2542356385441C2F
4D2A52292E1F46674C445228315F3D56546D3B48625F204145432F51754F49804B3A6E6E
194F6C316462259453385483733DA35331969A70564F50264C7363302F6C5C4B8BB15644
8653564D535B3C4158386C81765D6136809045782A3E6D8F6E623D186E76692B2D957954
5315369E94435F7814255A603A97874540693A7AA7892464416B9E60477545347C888554
97C0663C778D9A4C65AA7C6ACEB22D4CA64E3D774725595154915F3E674650A16E999A5E
509E683D97781C4DCC3D678F897B5D5D4DA43C385D766669494489A1914E93A62333697F
8F4B447F344F9888586150394E585D585A623344736D2B3C66422562705C4A2E8F7C5757
8D7639354B603B86753B929E699468592C50854B292F788668455C4F603B4439
79738465958D3C4C5F227940794D2B643553462048471C2A3C172F3F2E4F2F618851262F
38205A2C3C16359956404E2450653A4957492B56735B213743251A27594A508459233E61
1C4B532144411D751E202D5F59339F5232A3A6604F91793D4A8A501A2563824FA6D44839
9D5E562B4085424741185480704E32195D8A427621263D625A565F2B5C69686B2F864755
7B663F8C4A255487623542612463846E47775B447B853552694E6B705471996EB18A9C5E
90C7834681608D577CC47A64AEB7457AB6563D70502E786F5F9256597D48419348938D58
4D912F275740183EC84D456D7B69566B518B7736435E6163712C50A99459978B52243A59
8C5A406F7758B9AB4C588A636F51675A557A535E887832376760387C74654F44A7905B5B
7766467788703E7F7554A8A96C9F5D525384885533347B8970487173703E4B31
967C7442656336582C154B2E561B2055295B4C0E3349151E47101A302C55293556632F32
3F26542C4D2528674235604F99882B3E7B6E39334B59223271682E31571632768C345448
11558D44533A186D4A2B356A2C269C804E9CC22228587C5845965E1B1B3A744598D0662B
86577843366D3A676C3A46663E4F714254652F766935444E213B8E505C56477D3D77563D
6A7D4BB74F3E4976635A5D863D6E7D3A61947F46868940729B636F6160809C6FB3845544
88B97F387D4E8B464495685667B4335187543B533B3D85695E983643603E346D3F756A31
407E513251321A33C93D53877D514A65557F5F3845635067833A55B69A64928242293F5E
7B693F697B4DB59B4E8586586D49457D948356657D6E28548F7C48746A712D236B785661
875C223342593488682A588141987F4E314B79503C40844E344C838F5E387730
C59384689158213D522F2F1741441D3A0B365D2D3D5A030A553425230E4D5C6D7B800C23
594C3C1360452F7027203962BFAC141A417351355331091A4D63323A522D295878355B67
16295F3D60461951673E458430108178486DBD693D535E373F8D955C3836522453A08933
78305A583B522343775A4C6648426B53558C4185713E657E61437D495A714F7231776340
5E4549A56746546B472D3D6F325CA471414A7A4188762F5173737A6B40508269A3707A6A
82CE8443624E8D6E43A67C5F82CE515FA9693F48612D7C795B8E6648654926787A946F32
4185653454432831CD514B8E7E6338557B887F3D3E614E94943C50A58055AB9E46323F55
87475D887B48B285328372507B5F2040495641788C7029205555215A7B5C5025804E395D
7D752E304B512386754665942F7262715776793127437D644A2C519336325F30
A5D6A8609BA542386B3D5942586D284C443E4E3031442315401F29351E4369596E7A381D
31455035573A26875C1F31394183542E40665944836938194E372B4D636736615D124582
283852173957255D2E272F506731803C1F3FA46D3F5B58323B6F525037497D4D9CBC7432
8D6261523E75424C704E5F7D6F4D491C528C46823C284054543C7D5257704B76336A7949
547848BD6648556E734B618C2B57B39A5956993E8985475D66526C8658558D678C73789F
CED8946367588279409F8166A9D67158A474464A6C36837A48907150595E248690947753
55A6893C6343283FA77B4D8F82884133536D553E517B3F7D533058A5702F5655523E5773
9845295E3E2B9C9C3C5F4F27595954525E6A294B63833D225E7249514F6F6D2779684447
706E2937455326674B3654A2638059646680743F2B3D5E59643D6C95743E5125
B9C98E5493A7493B5631574F4A5E2E644C3D4928204B2B16501F2F4D2C515F5392905526
3633513F63291B866D183320475B6D2A50511A284460481A4836272B455C35798A45337C
293B80543C66335751262B5B623F707F4B54CA803B618E5C4A7E59262F6288579EBC901F
75635B3627705158584D3B4B7D4D4E4F417D4D5F4E393D504D308771839B578958748E4A
538651C286544E636C4C74A33649AF9B654E99398F925254763A77945D4E93708C9A9768
A5C4964363B6B18C3E9C552F86C17E53AC434984712F73647DB67256654F46969592A13D
386874356F4A23189D653F6F727C673E6E637B3D3674499389473A83956D7E6B46293345
916C3A667F3E8EAD5670826B5D5761585C634E596B934D226377515F50685E318E885952
786333676E783D6D5C497EA863836F5657847D4B2944524F2E2468897239552A
AFC27D374F753B3E3F1F353D3F391546403C480D0F442D164F11122F2B4E361C3D625024
3F3432316130155D3E0E350A326D861A435A5D324666221957633E3E432B17567E502967
1725674F3C48164B744037663B1D588B663ECA76283E51564371503B283F7F568FB68D38
7466535B4B78644F6B8048568F59566C477D63656C4E474F6936837872AA608D675B7145
54764BA98B404362524564944254B8A064537A4BC0B66A437B3D91A1624A863B52908A60
89C99E3E6A99924B52A778575DCC3E368F7036333D3998653E6E5455945523715278A85D
466E7E3F5B4D3519987B4D6471856C43A0718F47347350808C5D41939A727C7359383449
997C536D80488BC1647E9E7D645D59648E88634B5D8A513F97854F695177321F6B745657
6F5825444E6E3F70622641862A75A277496D7319204A5B716C2E4CA04A338E3B
C8AA7052858929304F3F381E275E2D471E223C2016330B0F3E26202D153F5D506C772F18
3E533217574C1E7B43104B241741AE454157673B4466411139473A343B64245E85632387
3B2F8269364C2E3471433E63502F3A9C673DB8A7554D4D582F6454633A458B6886AEA14B
6E6B47615275584F6B894C5C86577572447B567494575254764489844C8040A56D517C3A
6F83529C9D2F5285804F5892232A666B4F555D3488A3703FA02A536E51629F6D5E9A3B1B
6DDBBFA5AEC79F794666908681D28E3A698D413A611F6287457F814A5569256C9189A27B
4D72A453625D4439A2A9586879AB532F6B6584434066328383623B8B8537879A84583543
852F567C834882AF3778896C577A453F515B475B8D9B3F1F50674961658F7C33848C5F60
98713339405F285A7A5E4AAB3D677A8268867E3718303F76712C57B238297E5A
82A57A4D79AD6032575032472B6F33453320411D191B17134630243328414C4B6D683A19
305929325C4A1B6F43214C77302C95525651633B42665A173639342F3D6A2D5679682880
3E2E7F72344C442D763746776E4C389F39369AB85C4C4B4D34849B6B394A774F76ACB862
60464A7150793B5F7A7B4B795D255D634C8F31426D3A5F7A571F603F4E743055374C8232
47492C80944939586A3B82AB753B9D5C3D4F7F4C7A96432B7F52727B493C738448A97C5C
4DA4BEB3C9D8D0A344669D8989D8A23B4F8C5C667D264E8145967B544E6134767A699676
378D9E575B4E191C758A2D56608962356E4E7B283A78546D591A399CA05C636F6B3C465F
956A3A5283377CC2706972504A68684E6B5C3B3E727D71347E814A484A807347959A3537
5B6C544D57704C5F705F59BA707D7A6E47716E581B2B2E7B322E7AA17C3A6B50
9F82734D69B77234585E28513578344F492648251F21281A424E1B373743504A63714122
2D4F253B50541B694F233F76503F82525852563D4059511A343D203867773551842F1B6E
45207745396D572E71253C62633736A6362170932F4769551C3D4C54325F843E57709837
53653839236B44344D3B2F5D6F5655463776703A622641516B345B6E3A634655754EA958
4E695B8978534B6E6B45589D713BB08C5D4F884AC2CB7F45734D88B071437A4434737664
59696A5C857994844162803A48B08940476D2F4156213D5D539581584C5E2F6787528A77
2E5C9330637A4B1A77A44359507944377C6C7E3E3559367E7960346C9A699BC471232C47
7680496981364F9B6575A2734A5E41467E6B6542777E3C274672695350703F2B587E543B
635F1E3E516C4B60652430745E756D7B4E659B7C2E28276571314C9573367360
C48071507BBC7135504E385531733F556D2A5F472D3D411A3F4318403F43643B5E96572F
2E3C2B4D3747136D853235671A3288485959453F697D5F243553231E354A22589E721E66
3727765E25332B2773293359661A2799503BAAAB23334F472C5651371F41695D7FA9A74A
436C424B58644B3F5786586083715D7A487C7F3B7640454D644764803B7E5B7D8E4BAB65
60706682634F52ACA15B59BC4420918F604A8D47618E663F6D466A8D6F416C8E4C9B7773
5446557AB9D2CA8E4971BEA261D4883B73A04E44742C3A893F7F9A5E5A7A36657335809C
54679C394D652F2F76A35F5D538C3E245F456D2E2B64546054663A6C832A49695C3D2542
7B353D6B815E5AB440375A6F45644945635B714777686D35476F706454557B3F778D533E
6B5A22313E4625694E3F3C8359737087587EA1742F1F2862833460AC8E437078
B44F6F3C3E603B30541D1D2B1F4F1C3B58256E381043411A3D430C1E2A3E711529535825
3E471827244C1840341720572C273F1F4B5E4C272C4E39162B67552D3E56313B8D872F72
611D6C8A35452D1C6E2F41535F4334B371459CAD3B3E4844315D5D49314D7D72609AB968
3C874D64625F7144768F635E816D788C427452377151444858324A84346B34546438772B
3D695A607C3E296B944F4F9A693A988B45548D716B9F4A3B7587709A5E4F8D933F986848
3076D2ABD0E4DC934752AB8F77D083354E82526B672452924C9495436D80568765366F92
2E518F6550614C1958B25C5F5C7B4E3C7C57743828757D7B5E7F476CA35F7E9476532741
6E72413E68415EAF874F6455465F505B885B7C4A75816332457A8B6A5C6E8A46739B4241
92884035424E415D6264468E5758A9A45E809266311D2373823461A8873E968A
C34E55624F9C562641572D231C4F463143103C52285A301525451B1C2E225C5C60778B39
253B233A1B51374B4A412066693A714F54546461385C60271E62643245625A4285722D71
6A1D60914464582275423956846B39AA7C4980C46C403A473752A18C564D7F5F4F95C685
35693165564B5A2C5177574F6F41497242537932837B35416D685988396559655439735D
475C39539F61377D73224598662F8D903B406C735F95673D5C8C67886D3F6E874CA27776
3E3ABFE1D4E4E1AF575D883546AA984F5B7C34345B34345A41749C4A45522C4A7C4F6A9E
5747B07842624D1D4DC3585DA4A1644B8A58774D326884586E6E4D509C886895A2702E48
5E75454F773E5DC194506165507A716D8254634C84AA5B313A725F5877AD954562AF442A
5B64602F49542E5BA8874B90523BA39E5F91A1652B436494823C5B9F5318707B
CC57556E53A78E344D772F4A3B615F2C5730463D1E4A301E1F36191E3335536668688D4C
2D372F3B214535466056274363387A5C524B4D5E377263201C564725617C5838766F2F52
701D49632047592671364969917B389A512248A7623637433127414E40366E5C4273B476
2A675B5B6E4A5F3C58886B4268845378525F923F783C335483625E6A3C6B58667A4B7F55
4958613D814A327F775645AA6C2A8FA876577C6F63B47E4073855C92854C788F5F938688
3A3F83E0DFE5E3AD5D559D867EBE7D525F8F5B417247316B3B5F9D485056365783566099
4243A35B407F591E3FCC5F2C5A76714D7B5B954C4478714F73424268A5925D969A38447F
9A915159772F51AE91485B5D3155665D7F5F5D29648C8944357C4A42497290605A995833
72595A2D5142433B7F874E8574488E795F99AD6D461B276F7742549E7E467274
B55653594E927E354846334D315C4A3F68303F3D1642331B183B132D4A3D5F464F837F4A
2C43264726311D326C621D34312F67514D5448393D8165371D492D18385F663878935F4F
831A3E99485568286138273C68402281755B61CD432B4C615C3D6F4841438C8E5CA8C988
2F796355364A6D385A6740334C6B485C3A467E397166323C605C5E79546B4965974C8555
60797C4193624197AB6F42B54E377D916A5E733A538C924D58513A958547824733799B71
494F78E4E6EAD9AB5950AC9588AE7A4B7EB96B42744F335F4D6696534B5A3D606E436798
393E9A45356E3E1933C95244685B4C31826990432D5D60497D372E3E725A47877E1F2246
5479475B602C3FBB8B4759844C56404578946B4A618A6434549476675F75722F63926437
6A61424E837F574E7F885389935291795791AF83581D2D6E4B2F6BAC8639703F
734F5158427978343D47283A2D493E25493A4154152B3B21172F171D38394F422F455C37
2C4430382B3225234B4019304C31644B3C5B65582853593B234F54304745392955795B3E
6F17439253545A29705E3E4B8840267C88626DCC7344433E4537845F312A4F8452ABBA8E
305647666640552D658D7430474F3984654A6220627B3D40522549744F723D3681416840
37596D3B9460255080633DA75C2B5E7E8A69698941779945478148768E49625E5C7F7B4B
3B2B6BDED8CE748E6070BEA395CB97387CB96E4A6851458E504EAD6A608B49578B2C60A7
6D449667366041202DD37A4D75536C4D9797A768285A4B4B92634447775C4996784D2F4A
588B528E784A51C0673E9380435A4731354D614C67764120225F68635D6278555293542F
634F321F47414F3F77463667853992703A5D8C612E304E8C69322D786B31907D
56585179557C8532456B4A4138496B325D2C36602E364E1B19492B243A193B6D62677B2F
1F3B58391B333B325F421A1C5B426729325B6B783662651E1A35452E5148763C5785593C
8D2F32823B5B5C2B5F48375C7E4E297470373585774B424328244D675D2B4B862E738297
3C57244B664548273E70712E4A623169654B6D3755763F3C566A576F537D7657833E538B
5E5A813D9955425B8B5C3FB38A32816E5C498A90446E864071924F6E964C6972795D6964
63373EB39BA25D752E4684A365BBA14F519973515F50488E5243A257457356519C5254B9
6D418C99456A4E3349CD9C4C66506E2D3F4D686925704034757447467F393E828064324B
5C5F2A4D67694ABC71395A7646523F583D6D8C6267725D49246080776C87A769529C4524
848E4E245A45483DA58B546678309C9A6E89AB7B2319295474352E895315616C
724E53765B78832F33553C2E3742712D5F43316F352D583516452B213A273A796262916D
2A38663F39324531736A3E224A2C78672F5174633377784D203021183D445D305B742322
6930317F2C255535574F19305B47287DA96E4BA388385E624C365E4746203E8E567D98AC
376F6E5F58517A65466F5D346F934E704C539357555D354D907A5F742F5D885863264A8A
5E56543DCC8F50586E3326846A27796E7C565088415E8B3B58974F676F47989988507E96
984A8BA894878AB172528C6E419FAC6C579D6160896032745C40A75C3C6C67519F7E5CB2
6951AFAA3E7E451F20BD904B7F639974675796823F7D89456E715662A288436F725B4278
A0A056567D6956BF9461738A5D6474733A3A6D557EAB83592452572A6F9AB4693EC24E19
4255662F63456644717F5B6C8D3898717490BD74531D384471423E9F5E23685D
7B4F4E594D819544385237475F47642F6C4D28562325492C19401A21472F3D5C3E739B7C
2A342F325031291340773E2529124B4928547131223C435B2D3C563E4138311B6C9A7727
803D2B7E702E723953732E3F75381657857B53A8862F41665E41714F4524519C67A69FC5
315A7157523E615E41795632536D4E5D43357050467C32345075535A66517A5475614F83
555A511F928555566A3F3096652C86AC895D609B5FA0BB585184596E8E5565907D46A9A3
685D7D92343961885F5278804A96936A6CA7742A565B296143659F6B42723B5D927752B1
602D638D3AA38C3B20C8AC32666E8F6E826E93401F4C894A8F471E3661823F829C321F46
67926E5A7532378791676E68283152694130544D4D5B61572151686770549A644F9E7247
5458763A827C8B5389865B90A349AA623667A98C6B4567676C3E339373417550
5A475863374E603840401D294A4132124051326A20154E2E1A541E10312944551F497365
2D4D513E3E39562451613943743C6E65306C917C504C4C481C31423B4141643649718027
802F2D997B2D6331608F49405F5C355E84674FBA986234474C4A7247522443826A7FADBC
5C60664F6352824D416A8449607361597A4D796546854B404C6B60776C4F6F4193654C78
507C8A46C2644E6D94643E875C1B5B676E5465953B4F805C4A632970875E769C3E3A8F99
696087A16B4377A048608BAC4EA09A4144889A45702D3878603994414D9B843B92343B91
BC4C728B175A924324C5AF4D5E4F654B6891986B29516B3C897B5A448B7E499BA3884140
5B91497F837A569F76538D9A5F4A4343495B6F64506A604A397B94786E548C553AA9864F
54563C20605E7D4B9D793F6B9D46BF897C70A56C351E2C3C5C4124887A327687
663F446D466D6F1D2C41382B293D5422393C2A6E361D3D1F1745462235172654447E874F
17295B39342243234F5C21193D375A1E23486B7250394E41202B3B303136643C4D7A8A2D
6C4C18556F2C522A4B8A433E61663D4F7B4944ACAF6042433F405150502A4679515C99BD
6058525B784270396181734D626B405C6A5073384F7B4B4B4C3627476639782E4E523F71
324D6745BA91393F585C47909355839E6F58ADBD5E68A44353AF69715F3B85A0883E7D94
1F2646A38B52769A46355287558E925A47898E43614D1D436D3DA162305D733B90453975
763F619D406C673648BBA25D6568663239586C562F4A5B42705B4A45875F39798D6C4343
5B974F555F735AAA76425B74594C513F363A62644D7E7D5B1C3E707B66529B994ABF8555
777C69364F697F41918E5C7A892978576981BE8176203A3D856329816A3E6875
786D47744A677D40314C431D493C733339361D70521B512615404F243543295C576A794C
203269374326491F4C5E2A1D5049503523415D7151335E4A2D3038323E4A6A42467B7338
88731D54802769574084394271532C3A6D3140BE9E5A49493923488069394A723F334D72
54545A3C583F734F3C4956427C964955624A83633E81334F8A7C485F4942936A592B3E94
6063402386A07655733419548F5571C44D374E835E7EA142346D676C5C44538388516584
787090C1894A80987848638A4C87B372568A9A3E6665224B783A9683456C79338F745688
82337EB0426873282BAC84567A94795A3D6C6C5F3A5E6C507C6D575C9B8C4B7682584A4A
63A66E536D644FAEA0836A685B5D667B4848573C4B958D7B214260637894C58E48C26E2D
504E693C37715D2F657D6272A152A0546E8AC7987C2A3840867A3D797659696A
937E4B87506490663E534D275B458941405A2E746420534323314924493B3455546A803A
1C39653A532B4C19468C482B412B51472F4B6D4D2B35676A4531441D2C3B58393A734B23
565D1D43581A4552376D192F4A4D403A99543CA594434E82993F364747356D9D888AB1C2
3F4E8B57591D4C89535B401F4671725A3C21557D48842F2A4B684B5F64397064645C3799
605660599297745C82552C56893760CB8E5A78A45FA7C28A545F796778815F85835F528C
8F84B1BC7F3F8FAD7C4B82A03C849B6A508D9D4D7F6F28427C3FA783466F613B937D528D
8122559828767E3329A7714B4D5152433880645B2E3D57447553365586734071833D3059
79875E4E5C593B6B62695F693F3D435A445258593C5C706F1E3D61474149797B49A3683C
5A537F4E2F67503186705967A9599A594F7FAF9C82373838673B4C927F5D5B5F
5A545278313B574A3B551C1A3B3B57162E4E2E6635114944262F3C11202E2D59211E504B
214B3C17422A4C192B4736315A414236245F816657214B522A4978732E334A27478E923F
704A1C4B7C324D413A8C623D635C12274F7254C3C751254253403C54422533547E54B5C9
6F37725C684E5D7351558E7062737F4B8C485590499776474B704D6F8D4C795F7083469B
566583649081565BAA8D4868913542C37C3F80A05461856963A75E476E64719E644E71A4
8A8BC6C3882957974F32647F527373584999873B6B574375573C86683E74713482473969
8A4B488D2948652D2BB4716869625F474D835D632E425F4C74504D4B8072336B743A3F3D
518E604C605E43868A8049816D57484D4F5655604A657984233F657E5451838E5994794C
625C604026675835775546619C529E5760509093563033377D7137546A344650
6A413D714E435D232B4B5329232C6935243117584A1B45271730542C2614125747408041
093159313D0F4635446725173C5D521E0B2141636627633F131E41442C423D271D4A743E
646B2228693A4F591F705542656B4E215558328FB081485C4A3C405C59564C5E9154BEC5
7D426A51765E67544E6E916C69816D53904F506D3D91694D5A67384E7A4992545D653E98
514467528CB25F3F707D2658896540C8893F617F68737E4F3B826F4D612E3F76664B36A7
4A457E9C75446B968346658B5C77716F409E9C4055502538633383974E5C933771785671
9C564787544E773D27B3698C6B6D5B6559924F7C3F36726773665E55897E40666B5E5E3C
5383725067683D8F8A6742A07A644C505379736242647769395F9C7A544C758353918F59
6158633E3B88833D765226478255876D684A667450374A4C8060385E75333F65
6971487F574B7757243A5C29272453502B41112F5B2B5D311012312228371B4E4B40893B
13272A1E3524322E456E42263F633F523B3E52736F2E6F3422284C472F4A4F5C30678A48
3F91422A6F415D6E2A7F5D3A6869532054695285B7804A5A593A457896584E507830A0A7
7841714A54536D6C4551665366A270417B495C8A3F884442637C505A5C31747E57392E85
6D4957335397794E807C2D5F95543BB3956F688283826F6F497665546E86618472613994
9177BCBF87425B81864C68896B607579469AA3586371334F7A397CA15256832F6154506B
8C4F45946848703F31BCA993798267793F927D9E5438705778805F54A28F5E8A817C5E3B
4A835E466D754E8B7F63449D79585046365260543F70896C30345C72544E72603257864E
61654F231C5B63286A4A345677507E92654F6B6B291D2C427560395284364284
70914D995B4B7D7A35487634464680622B522D3F77345245362A392D2F3E25675E3F8F5C
293F432D37413445328962293A6440755D425A6D5B318362462F574A2C4E535E306D7B29
288C4D235E32566D2D824931657433276986705CCA7E3A759B5639656151577EA24DC0A8
7D3177664E2B4AA259505B313C66844E5122438B41855B3D48686459775276975175476D
704C66794E945A4A876F2D6598423295916F6C7B8C8E9D8D4A877B496B8D5D758577439C
8A6DAFC2834B59A08F5063825A81AF9A468A9D56707A3A4A743A79954D596C3C796E516B
893347855E406F2A1F6B537874A572522753427A3E325B3A46564F61A6742B525E4B4135
4E796B405878458D967E45825D4F54635658566B415B6575383C63514167857548387E2D
55655B43205F6D1D5D6A464F6E2F7A776754827A4B272C3A7C7833447F2F3376
6E8751B9694F817537594F2F474276512B5420336D194944302F321D2B422B6D572B8974
373F4A2B3A472F351D71682445482F58394B6F70523B735447376E5E414A45372A737945
289042317F623C692473733454843A204171754DB36A3D4755514B725C3136499157C4AD
9C4A726B4F5F5F88554C71814B6E7E476C594C8644647E454A5C68545C44728B4C6F4278
703C677C449451487A5F355B8B3E2D90935D71855572979147687A5A918454787C6350A0
9071A7CC7C3E63948A4A628B534E7E7F37709750616A353A6C4E8E88455B712E52825567
7E4B356A653A7545236F518E4D877B6045654A8A353F6E6556664E6AB87D3A6F5D473746
5A847949596E26567D6B3A787750494A565E58784453738745397C7D6443666642277861
606D6C342B7B925254643346805C838556609B94772A2B39623230688949306C
5A55487B543C5F402034432124206B441D4D1D2760213A34152639251C2F163F5129666C
112B3B272A21203B2171531921452D5A403A42686436844D3D1F465942534E5827577B47
296D461A6365357532677E3E557D642A477463408F7051514E544379615247559C5DC5B6
9D51705D5A6D5A6251596B79477463456C67506B444D774A4C505B607149727C46833D73
674974823C986447827C4E508E5830798E7F5F6A8977938A3D4F703D616C4E7E5D48569E
855991C5844855818D4C68937C5D907A437ABA6E54563532633D668F404E72424F8B5E68
9A78418770398A682D7E66A85D7F67883A545A94652E7868708F4C458F8A4B85867C613F
3A66754F91944957906942919052463B2F626D71474C774F2F3D817E57445B643B1E686D
82A36F2B19456A45485C3B456C567EA4705592944F424D477A731E32A35A2F8E
5D854C8E6F466F89333C6C392A3255572753291F5533374A2B2128271F4335474937616D
1F2B2A1B1F391B37225B712A324B2F67754F557B7D48396250254F4642574B643457734A
3449572070762F873D5D7D3757805B28466A5133737A5C524B543E82A0564B5A8B50AABD
A44F7051596754714A52646D40686D4572604C693F6B844B524F4E445C46538C48885058
853E5A7E3F99733B526A4D37797D3E729A837D7F8763A5893752814B718A5889848E4DAA
785B82C98E565B777538649F845EAB813356A06F4C6D3948865767873B678640497B355B
7F6F489E944261442E59699158766F7F2D2D346E612C675241524F42977335495361574B
3D6A7134456E4A53AC6A2F6C70597659434755674564A18253235F734B647D99543B6B5A
4E5A664B23467B304D7A4C497939577E71617F8F4031447C8B6A1E36BA492D87
56954C9F704972963C3D693A2B47626E32523C245A34343C31312E2A2035304B5039536F
292B2F2B22461F4D295479322B4D325465455172724124774722454D40515765325B794C
37385F1E7081307B44438346669D6A3246614D31548859524840335F7557455E7738839B
9D4A65665A6B4B77564D698536607C4C6A7043695D438E4B474C82505F3B58984D6F3859
9544558144957650558C593D8A773F6D883E5358755BC9A4405C8A4E7D62468987884EBC
5D3C55B98E536C918A556BA768618F865999CB7165823E2870516890494860315A8A595B
8C45266075396D5020564A9A45848370473A507B4E4D76664E5A54439F814C6567603E4C
4978894958743B5F977E4F80643D4A5F634E58632C3A708363265E7B60415082594B6480
53686C4832466B573A7362456F666E95796C7D87813C27346761233BC0843778
61A24B9754466D7C3A394F323E4F57723947411C5F472C482F2D3F2B2027254657395487
24232F26295021522C57AA48244E2E5867465B6B533C3C7367343E5C2B3F666A32618836
39445E1E615F34814E377730428A6E343E5C845943836153455445676758455E776497B0
A14C567B6066526F534C5D8C3A588C586770406D654190514959764A5742588B4A724F3C
6B42615F3683734847932B2D6373446F9C806B655B316F7F606A893D727153706D6957C4
806270CAA03346596A265086884F8B6C33517D6F4791333D863F72905B4C6032388B5E5C
955E325F78439369226152AA4762616B4337628154315262476C32306877455264651E39
6170864B4C77563BAA9E4E78915A5A52644D5C5C394A6F876F255B867E4D5095615A6D87
5674623C363E81453E73593A7156759C726BBDAD94412E466D5B2F68C7753077
5685479D6C47748835394D2D243C3E53213F46206335305B2F27502E173C354951203063
3C1F302D204A1A42162C5A35204433566140576C6D6B315267303455443E496637697C41
302F4F1C5E90337B5D2E88534A78763E4B6F81513C88625748594C63616E4C6D8A6D91C0
B442578053642F7B48526F5A3A5A785681593F805B4584453B4751466853578334946730
60376985448760354C7F4B32655B2B4092506D817F4FA08D368BAD3F4B5F4196653F3FAF
4F15329F99615A608F4B4789884B947C437DA9844F8F2F206A594B874B75985643853A5A
927A415E892E5462256355A75C6C6E8257476D8181465D7A4F8E6324739943486761584A
3F5C8E5351816B519F6C41787F546E45554B785B3F476D826E2C6882765852845B66878A
4E6D742E345BA3543A672B2A746A6B9A7A468395693D5985873214388C7B2A83
6393466E685AA79F222D5B4E2F413C73473C492161451C5B2F20524F265322365D3A376E
2C19453A2B38124B313364291D4450583D2D576C7C803C59601D415F4C49444523657746
3A2C2F155183306C4F3091564A657434345774494085515D596044798D6A2E4B74506BAE
A154475732604F86423A56613D4F6E2C696F418C31327D594C5C4B354B494E9B5B755B46
8F504D78458C97522B6C3B267C9B5D3CA72B4A5275459B7835508F4D59702752607E4ACB
6B4262B49B7058698E56448B8E53A19D64788A90538A51185F5747823A38583635604948
76612C529A41644E2A6283AA63838893322947407C414D3F316C692145A42F4179786247
495F8834618D7044A7732F5C7F5084671D30545F39477268282F576D55534A85273A6963
4A82672B31337F3C3E805C2A7E405EA09974AFA155312343736A1F3EB45D1893
5C884D754536505C322643363C20286B443F3C0C5B3C1564372A5C462054324155404075
281826322D461F423F37784B213D4B3F4931456082784962763B34544B4B666C36557134
3C534E173E781E3F542474394F84684236755F2F3F745963514B26434E55394A6D595495
974E5080565D41917642626E3E5E97655B5F3CA67B2D672C3F5E8347533D3A7A60694E2B
5F53525D34486D5A3082441B4D8B433479726C52994F968F5A5DA65D58856D83768C4CBD
A7906DC3A0535685945D477756407E99696F76634DB2631D636747915B4D5732387E604B
78632159903F6D48183B4E9D354F6C81534E9B75AC537E554B775E2E4997522E4A5A2B32
6580885A4C718541BA6A50457E619562603D5757205695765C284C472E617CB96970803A
355F71644C3C902139845D2D7D504D849564BDAA894C2F4E6F7D2F43AE722A6B
547E4B8B7F499689413349291B322B5B16203C185A2F16481D29693D1C3C2B355F251F43
331B31371542243E221E455E5C593A2655364B6E5A362736573B34572420344D31648E46
223B401A50862C28392A80572F54721F2E6D834C498B294672713E5550462147827C47A8
BB4B317A53502D6A693C5B502131806E553E216E7A2F6F5636406D444C54469E72707C42
6369557B665E624D3F8258315C97423F9B896652884C9EBC8749A24C62716B7A6F473593
9A7969C2A23932627239458A8962789D5D648A803D68511F524F5C9369495A26206B7242
787134469A2D627C355166A65B69666A3752BA8BBA414A8456A558234F715F3962614445
444E72386678553193AB5E44672F486D763F655C182E5C6D5D274C4F213E429363728571
3556543C3A4AA0302C5651398C5D6892895799945D3E6BABA1683467AD833377
525D325B7646A08E18204D4127252A6E4F2A39144E43214529175C5235511E27655C3151
3819534A38441D504421413F2A4967324D23486B646D4339751527605831403C0F2B625B
2345370E2F6737353A155155315C6E3423476F4A59914147525643606A47282648753A90
BC6D41755389688F593C6760564272635670489A8F408175424157385A64499F427B8E44
592F4278605468411A646F455689222179785B71743467796E3B8E445067676E5C64438D
AA905EBBA4794E808D384378937078B4395892954E8C632E666C38906F626F5E2B72563F
898F5949A6355471294E70A2635976845971D673B0494C8A518771334B7F69385D6A5454
6B8DA0434B596B30858648536B46526047347C5E27485F5E422E565427514FA4514C7C65
42676B45343472393E7C3D2576453C888B68B4B7654023425658283D955F1C6E
566B4E577F3A8A982C213D46261A1B715C263A083461282A4911364B3C5D35254B442A3F
390C2138373811345023523016215B29491A34456E7653315429203B50294C502F3A6352
294F631A307730374226675D2D706E532F5A6436579A6D5967472D5F8B694439485D2C7B
8F79425B176C6F8C581954745D4474354D854681552763683E46442E395D398B4C5B6E3B
756045696751A356336E562B46A06823836C4670885C73944D4596725F714B6B91835780
A94552C2AF805673936C396C94749ABE7F4C7C723D805D185A6D3B9D413C64662F78482F
6F74274FAB4D516B2A415395466F98A46363C56E9D5A769F5F835F3A6CA9883A6F6B272D
424763584A4D723FB7C64C4B725660655E355A572F5C9F811E2D595D305C82AD66577F53
314E52423C2F88262B60573193533D69764DAFAD77593B537478363F997A3179
58786E58993D85B8703651612B473B7B5A22412D4575291B241350483F49253C62552730
3129373938342A333E296F683B475D2F5A4C4654795D482C6D502A3C301D516A43415D35
215278252B70161E512E63431B617E5435767B3754A85B457A5E354C613B3B3345663DB1
A0783C835E7561848B424F7061426B5C415C495E93426D71464F7260564E248887665131
79744E662340A8793D787715388C6B327E8748587464629E5F378D7462724E4171885F7A
B1483DAFBA7E52A2AB823D648B5571AA906099805456842A535C4CAC8A6D7F4323755C4A
A38A413CB14164753D3C4DA34A3E4D424D4BB275983E4B684B8D5625274D6C396E5C3441
43487F6654516B2851865F537E3F596169415757332247574727405B172F3B8668508178
3F50646054336C3A2D6265358770537B8655BBC89067405A7379363E91863B76
4E5C5A506F285985522F4729234434712B26513F31571723351B3E4B1B4346416A3F1C2D
675937412B3D532F2E173E5D3242340E4C414062733F41175049283E49293E4845427262
212C5D23287C311F3F2760772C5D78212056735364C3272F697E445A933B2A216B8B487B
AE83206568695F56834A5372304A888357745869A95271453C55705D51462B638A5A6A4A
84684D758149A253417F835137997530446A6699A26562B6813C9A49526D795183574350
90542D7C916E418A94705093B07567A5733B5F865F5666357278378F4D526F792E6E682F
5D7C4E3AAC4B3B64333A4AA36C4C61535766B583994458815C81654A28517C467E61364A
41478C685154794870AA4B47885F434859465D4C403B565F3121375B28494B93794C877F
4A50684E48346E513A5F553599824D7E823666A38F5F415A74482E66AD933A64
795C45517D494792322D574E283E306A35163737234D22102A173E5436353F24634C1F22
6844494B3532292E3244645A3147613E532B4C817F7B681C653421584F3654451D2C6E6D
2F325B11247A4F2569153773325273522C476A5E68CC774A5B4B514BA1402C18366D3971
B0A0416E6D7081708448585E362C5C74566A2C569B51745B3745583C555446728C54804A
93765A85614FB87D3572822B2A807638406244526A5B43984A4AA1645A51574C8F784F4C
B06862B9B87E4C8FA861294A98665CA39B4A6B987553682F3155308F965C6473364F6A48
71925D3A9F5F4C74483746B574556F667E4EA0799E58529E7985685D2E5786498D67414F
3C40736E5A5B70457AB9534E7D6D4D525C3D5C523437585B35243773284552877748A287
48516545385BAF65316033206F66477E5F5F8CA6815C41596B6E32347A7A3962
6E5D6A3E6F5D4CB3391D2E46342513625C22402016543C103A0F1D54512F5F133B5D3931
7A3028404A352B314D5077631627553D4F252A4D66684821663619414C314E4A39334E5E
2D2A6523196449263C263C78335A7C6F3A618E5363C389637E565A52BD5D492B56854676
9380377B5D5B7260A04348645451946A415D435D9D3F6453447484624A4C2151894B6031
5C76536233357F5E44708E34256C7F403B70736F7B7B47A962407D5B57567C4E8A7F5D4A
979E81C4BF955388929A3D56AE7055979F4D619D7B4F6D393B5B3081A0666079374C6C4C
659257339E62456736354CB86754705E70579EA9B76B499A777B846C345486598B745759
4036667C565F6D3E74C3745499715D53664791945338576831304E8C554A6471512F687B
445C793B1E2E70402C8D45367E593C9A857089A76C414D959B82373F897C3671
6C5B8E46868246B78940455737403A61612B4D2A18643D17412C205741264525576A4438
624C33425639372047368F8E3A3750294E4D446A7B422D236D4F2B3F3E2752605C416662
232C814125744B3035344B7B304C6C45203F4E584997503D84814840A244341D2E63528B
9C91284D785E7842825B44573B345374495561448258645A3B3D4D5E4A555165944B724D
647B5F795646A055476191362C63823E39797C7A768C49A67C4985755D698B5E9F827446
979276BAC48C508EAA9B4154AF696DC0AF4E5A8D7758863932673C7FA4676D703452634A
5897513FB17D3A7835275BAD6A509C9E83344858855963A93C5F876E395B8038817F555C
403D5F40417C763B6EC94B487C6B45545F3750515834777B211D2B6D4B407C9B8B488B7A
294E705D4D3A742F1D6E6740775B2B5D5E70919EA14A25475A7035427B7C395E
645185457F4C38A4843C48452C45546449235245264E2F152F241A4C351A3D2753693A3E
735B3558462D492D5D1C536A3A3E5841543D46978D5638236C402A5B5444494D493B7365
3A1E722D164762303D2730673B445E6D35465C7C4AC88B536A456B4998615E3133776480
9DA64E5A8E667852A76D3F58733E5C8B4A59804998676588575054544A47486188436D50
6C87555D4E4C9A634B3E9A423172924352827E6A717855C6A84C967756A9A2679F74424A
8A8D6AB4D15528678D9A3C49A44A3770A04F5A86383A67382A424086AD67806B28557959
50AF422D8B6F399E6C273EBB482E4C4C684A4967943C408E7F51832D4580814D5673402B
43688A694658592277BB76535F55326C7551604F41204A503A292D693621436E874A8685
4B4F6D55443EAF492D6650396B68436E5E5D6192A3682E57677A445082965664
72508D47928B3F7F7B365C6E443C436D582F4645273A49202F221A4C3A1E4C1949655043
9236254D54352D1741255A742D294751554B3D59605A45277746162E4344504F4F2E516A
4F2062541A515C31342E326C3E4366743C45557C49C0A4557C426C3A878F69353269616A
A09C5E4B8C598454B170445373415D89465A814495595E86524E4745516247589241915D
5F854172754D8E7E32459F794B68965136657962626F46889641775E34727E578D75653B
858C71AFCA9E668E9E934375C389212B8B578BB87F4F5D4E497F3C6BAB5F9F9D66518146
5AA571397C74257061333CBB8F6362657D6B466E90603E9F9C5F94643A4E825861725354
2E244B583D5D5A4C5AB86556597A394058426750503349602E1D25634B417970803D7C96
58597E5F303495633A673D31727F43645347224C79673A5D5E481C5188805761
684A793A707C44874121324E3A27303C592C39491F605527361D19624F244F2D3F595245
793C284B5D461C1E461D4F7C453F4F664266537B6D6D3522724524405150464C63406374
541D605F1D4C6F373E2E317A3C45758144424E644BBEA25682475E3787CA72423A634A39
91A9714478438F46875853736E415C6E34777F4081325A7E4A453F473753634384417A50
3B79474E58548782432FB274516D99714A788857749B5B8678376C7B4DA58466A29B803F
84602270B5925E8E8C7A29409B822745664B47A27D6059384D7A3B649938597357496D2B
45906A4068943B6A754035AE86646D5B9F8B517D9872409898599B7D42658E6667705853
37244A6B434F544069C25C4C53894C465C3F617A653C506E2C2545905F4189775A224A78
446288641C2F49875E633C313F4C3C6A5A3841737E602E575C6F44385A986258
7549944279945A7B8E4C3D584D2E595276403469373957233A371F6556304F4F5662423F
8941274D6247323F5130489D623D54593F70436C6355292B7C60353C6252478982497575
4C217070244B572B594F2F6F2F4A72864A3E4D392A808D5384484A2943A4704C4C69522F
7D946E44856080467B834C607A49788D495771488E78617F51517C73555A3D499C5D8637
398150583F2E7781583CB43B25588F714174473B50894E8067358484548B64367D7C8341
828E7C8DCA87638CB79E67429F8E405DA26857AA835263592E6E3851B56F7C6F48476949
579A64386B9133696F3D54B581545F567D2A5D91AD93528A885AAC8A4366843551676868
38204169407B67426EC06055878F4B476351534862324D721F131F536249847791425F82
434279401D2F83536F8F454764684A71594D2A495A462A5F7476453959A95C5C
6E499144807956989055374535446547672E3D8354655A0C3753387C3B0D3D51495F1920
82612A4661281E3D4C10236067314C232A5E3471A130281C457C3D57783122405D3C6388
4317546A283F5A2B4758407154395E8E525D87994CB9A03C8D8C8A433DD16C243B547546
B1BB65357958872F527A5C5B392E4E6F4D4D39295B684E6A332E425B4C5E3F4582607C60
478E4A525657717F42318E5C4F59765D38537A6A66A0676F8C48698C5B7C8F82877D583E
6D876F84BD89424870854E3C8C573A78A06C56A34E4F725E275D4B6DB76F8669435F7558
53944C2C798D36685B1E366E7D665C557B3E334C678C3D7C53528B873E667B5752498177
351D3B5F29445B535DBB764354744C4B5A6D6B56572E4B6C35191E57735F858DA84B558B
3D6694602929633D53A1876177893560856A425F94171741696C403650A81954
6C425D4395632E766A383A412D2E5044632D25512A5B5A18292D25706C36353F3A5D5C27
AE6926576C58343F544B406A4727545E3E651A3E6E66623B5A8D172B5F5D3B4B4F253E79
7B2753622B628041425C3C746D4B597D2A324E7B45AE912E4B44654835CC81503B406D49
9DBF83477E69845A7D9B5E4F7B4B547A57485D406370497F5C3D4953466F4D3D8E5D8571
4F9049686E5E638E312E896B586A795C4C718F716E865C9CB550707D33788776867A5A3E
6284806DC0998452A0A86F337197585AA76E55A0733140562251373DA2707A6B542F554E
57A465443D9237608668245A88725C5F92644C746F8D356D874A7C873E5D77664E4A797A
3B1B358650515E6E319167576B804B436C88675B674157753C1F21546A386895A645528F
1D304A5859296525213F7177789833475F57417C8E55243D4E713C3F99BA423C
6A4664314D725DBD60172C3A2E393A2E53402E4B10234527332E18313E322D2E162B4820
B38910203847252135463F7D401B2E542F672E455E6149304E84312F5A4C39484B31256F
5C243C682B337B3F4347327D78537A7B583A5456399FA45E6A4B5A4940B88F6C47416E39
86B49350733C81597D83578689545167435B623C5D4C4F734C3A47482E4B6341823F577A
437F553D566255AD522860695B4E89502D386F59766338707C385A82385873777A998F50
67844D4FA69A8442B5934B3B79A7645F9B564EB99844516820586542B88172776B345A48
45A8575B3F993A59755925557A5B5B86A673477B629A3B6574437D8B4264866545505256
40212D7B4851474C3597644F7B7743697A855C64641B3B60553D26727034514E7C52527E
63427F634B2E9658524746425E7E5256A48261649D1D13426E773A2744873940
82467F525F8B76CF7551283F5A2143347B5B2C4F44397231343538645744254945586E27
90912D334C501A2C3A583E9260353C6137714C646E65422E5288342D5853446A714C297B
57264B8B3F35902342683B824D3A55715D3855482358A65F634E4C252D939361453B773C
6A889549745373616B8348426D4C50644D3C503D5761417A573344524B4A644E877F6889
547F6D54565D48AE4D3C49764A3F916B37408C6287836A83883052955E5D73506AA98D51
5A87294C7C917D42C0A5421B40915F58A26E3976704353383F776637AB52548C6C3A653F
379E6F5F3A8426708C561E65861C3B5D85643F8E63A4297D77437C794985A87A3B636044
48545A7F53474C312A747F6A79711236495044706E38364F451523576E587B546F244373
5554A990341E573A2D705F5757812321687B6178AA38131931654A3357B71822
654382575B7A43A595602E4248285E3C7A23225D5037610E284C4369590B17614F62420B
679C5039572F204F332D1D7175353945175B43687E4D2418337E482B5A4A3A4947512A6E
632E3479412F9046344527788A4661766E3765724489C458768A817F52A78D7C4A429C65
A7B3A8479169745B64AF624F704963896F4D654C6C914F875B495A70554F593F98905468
508B6E504C33509D70495F921F2B6B84443F9F2240565080AF3156985E607436406B8A52
6B8D8C8EA4A0413995AC8127447B396AA592467B7153746E1435533A9B5130446A327444
2A677957619C2D3A6936287F8150455A6335256F779B2C4796408A76282E6A6037827143
3222305E396488511F776137667D375264383752412045764D241C435B2268A69E5B4973
1729597075337A2D24445D6569734D3A6E723C7D96572A24426E613258B76941
6D43685C537532645A51273A1F19382E571B1241452F5C1A1D2C306B5D3A1A4A44716329
45A43B3F6151284A2F60316854284760385824508170633E38B6231E5E5C528A42271B66
77413184211E87603D461B547240668947285E634078CF353946646540BDBD413A357159
9FBBB13353696C4A467657535138375762444B2B3D633D722F202E40465246246D71486B
406D6C4F6564397561496296702F6F6732498C5C5E4E2F699A543A933A5A7E81408B4A2D
47736D64B1B27D42AEBB733358945D40626E40957927345A1D45314C8A844D5B532F7478
43A38E382B614165963D185E8E2E4782A074316F568C1C8CB0616B501F48A86F21318457
424C5E912F384A462B95945C585C275B6B6C435A54151B547A5E1F46540E3457845D5669
4431734D2C30935251636A776A65644B93723784B0452724416B462F89BF7042
6A4770425A9B6276816C2A4E643B382458492F4B3625633E3833215372571C3F133C5835
3DAD19214659252817553567561D1D433D533F475561523D3DA12E1E3D51558E504F1C79
743E3472411F7968394B2B4874466771622E56663F69D374605E5B6A4294B5764D418163
7FBDB1566C60775A5F8254767E6D486E5C595B5B5362317864393C4F2B5B62426B423785
53835C3C5E6D469488273B8E6C3782804433884D6978506265583998544956573FA08A55
599B1E1F6AA99650C2CC201B53866742856E185384433A4838616E388A5B2DA19030703E
1177A2742A6211387C5D1869961D2F505D7D4B7657833D42945A705728275E7740449B47
1B1F3F675F3D503B1D64806E4B721C464464405F4221305A521F204955335D5C745D677C
79456851211F55515E6F6F5A497F62489E623571A15A341D386E432CA3B0793D
5B486C4B3862516572291A244C3A301A385A313E37175A38362B113244541C4142535F4B
369233353F5925492261446560332845445A5B606E605A4E4A9B442945675B684D612F8E
7E51328F57217B6633592F4E80497A9B793D64712341A97392868A34253E84AF826D9F4C
508AA04C7C4655525D8E3144634E598D443A4D485C7E295C4E31547950445B29748D506D
318A964E5C32368F90504CAA33296F7946327B41415450675D341C6E6D54783E1E556F51
49977B8AAAB0955FBECA8739387F7A4A8780426D8C6236571933623978682D526922573E
296F8B5F2C704540633416678D3F5271557B4377747C4B3E8F4D7178603B8A7434469077
4F273C7F663A4D5E31748176486C40684A613068513A35686629266B8452586C733A3A63
7042593B2329805459556E2618677645A371546AA66039214370611C4F91632E
554366644389516D8D7B333B553D4C3E445A323F3F2169293D3523415B44205E5C625D4B
36A046353E603464335C375B98492F3E245058527B50353132855B2B4E502F6D6D6A47AE
9633287D602B97622C583557632A417B553B7DA5594ED2483F495B6D5A3581D5977D6E70
84D0CE404C7065452F8663655D1B315B7249511932785375461E244A5F515B1F5B8A526F
3F5676475F3F2368774B538C3C2046803F2B5B8B626980755A653A877D56697D3C888264
448B7EADBAB2915AB9D6913F3A7F5154918D4D6C77622E63234554336D8E576565245D4F
4B8AAB6D3E925B497A301A6A994C506E446B34445F7841317C347E6D4F397A6929397E4C
36283F814B2E4B88487F937B50AF65764E5625767956284D6E26265A7E6949847A40475E
5A3D704E3435666848508C642B544A2C7880687EC57452161D52612990C45823
584C544A4086393641472950412137393C2012203A1D5F1A1C2F295DA7421B4D45636D25
1A575438615C22502E4E1F304C302A5447564065A87667493B98501C5B60434F45322C95
914F1F783822626C2C53192E6540587B5C2548545563D36B56585F6A55276DBBE0E27C93
77CBCB6F568F5A63569668596B763E768C4D515947773D5B6D4A3C54594D4A2E56945B87
63606A406B7E4C7F7C3A53736343549E472D527D68667E8265854A85844D695341817C61
4C918D84C1CA572F85C69740356F393975874D627259355C233F623D7E925A73702D6D73
5D8EAD4246A067477A4127A1C17E516E426D313E7B934F30742E72714E36797636618F46
3F25347630356E79477587503A81607D716C2E4C5338224E725221486632438F8D665786
872F415462425D7852376F763D656B41867B23296F7A52191F54291A7DA47B3B
534C5B4335554B555A3416314F41361C31472C2A320F45382A310A1A4259244A1D294A4C
1A644516374D3E46183D3C526921142A56443F2D545D5943329A571E344B3D583F532D80
7A5C21705D1E678232442E3C8A304756743F617E615FD5887170517F502653BCD9E38294
55C4CF6F4F96535856A5665569763E70884E52534970345A6A3E41525B57553F5A944D7B
694A8754838C4C8496485B858954549A493E7F856B664E86B9893F778749996144786729
2D71836DB8C47C4DA5D191532C6F8B486B9052768C33254E21355246859D577875255061
5689A56E36825F4C9645196EA757496F6B7F4A27577E3F40854E6B5D523D7C6F276D9D74
513247845035425E266D8A693D733C5243653F765F3821375C3F2A719855506D65162357
8846517A37299DB1774276492B5D9C4B9F7F51485E876916234265234286714D
515151783A725C557D6B3138474329402B62372A4929755122251C204761222B4A3F5A5F
184753233A563E3B4046555060473236603C6F5287736F3F3298602D463B3C5B3D7345C0
9078274C6E245C8D38454239791E447979455A754B44D48E6971534E2C22369DD0E3BA91
38A2CD7D4492517D53A969647D7C42698E4F6C6147783E62794B444D58416D694D9F4583
65398B3965814A83B2412A69804061985023734E557C5344617D446F7B296A6C44779671
3D9471448FBC9357ADD46B364582804D7B9B2B52755B4358363E5E32499F5D86853D4D6E
4777956A459D48418657269ACE69505D51744D437AA04924734690AA3E2D596E3475AF79
382127653A3C848D3F64805747AC7B6546631B5372592744692022588F7C4D7B76393D57
74364A7E1815507A6A39852610377F40988B7F4E5374501F325B7C3151AC8165
515C4E823B776D4C7D8B4747554A254A356A3E2C4A2E5B4513211E1E556427344B435956
1A366D2F4C57452C45495B49544C3C3A6B376C5C7F7E7243338F503245573249767C3DC9
99582A3F732A487134484E37781B4AA589485C66382CB97972744A8E6337256EB0DCC68A
386688643C843B764C994B3A6C733D588A426965427B4F457E5945516840475D41A76B82
583A8861585A3E6F9F554B506E2F427A5C35A17C4755666F648B2269974D726A27537367
52A7984B8AC4612981C9A35B2D56525A899268425E261026192D4D5798B0384B553E7D82
5548A15A4AB7843F7F1E1575A286655F5872190F1C827A2870163F665835606F2249966C
59262561451C2D61405A8C37297C76605759152F4A602F455A21173763584C897B90555D
6C356A944936709F574EA076414A6F309CA8785242903F1B2E46652B83BD892A
52515B9041736B457D944B485B51314F336735284F295D3917391F1D47572449573B4B51
183D7A314C575C354B474E4275693237493C755B74894421257066313E5727436A7537BA
B32C1939612D437C25515936732B2E6A69305E767D3ECD6E43775F8F534A3D78B9D28097
53A1C17E39905B77518E7C4656793A5997596C59407E66417335445C7B50553C3B86618C
54325E4881805A5CBC514F6E865B538D3B33898E546D6648B7B84B46803A778A4B6D6524
1D5589699AD0662B6AC3A466395B2B324E676B41586F2A3E3222482146986C51482A3481
6E49C55D1F888B39833C1872676062535A6059292B6F672D6B6F72886D3655846A62B68A
772A2F5D865A4A7E374D9066466769495C653A635250304A79432E437C4A53BDAE93526D
6E24378F3624818A492B6C64425167336C83784E46937A1C193356264B7D8357
50403F7D4379382A6265393E532035572E4C09124628543D1340371E44561F52582F4A46
0F46803D4C67462F38432C1C4E4C1F36432C4034698252372285421738575553554E208B
934F2B3A3F1C36843C493B2D593D3969744B42568242CC94476D3C8A61443567B2803067
4398C6802E88607E326C7956774B26467D5083362A54464C8D553D4F5238797547A22171
823863183D806756BE492D56736557A04123A07758917E403E7C663A85213F6853567977
406C926A94D29C6A96DEA3746C9F8B5D6A85627A867E404D3724643A56AA7E564C564A9D
754880624091763EA55D2195697964609C8D5B40428A7446998D848C6636598C6D5BB182
7B2E3D658758507E4A60B1905C6F7554957D586A684A21417764333F8E3F347173805386
905A448C3D378984432C79392D678A4C85B3743361A782334E6D8A4644878A55
4C55486A376C4B29584A19345A3327211B572D1B2E11404F1741151D404E2F5034113252
1E20641A2F4A45231E2F503E5D462123534D6A3A4C645729207C4B2A24575B65A5802F67
B659346D56332F7439414F30572E3A81885E40575D41C1A84C7B352A1A212D6F713E3460
3377B7A53779348B4E52492B546B30456F20455D3E5D4D2E675A47594D324C5A3DA96376
734370566A686262CB6C3A518436599E732B81544C616E575E885651936A5C76515CA287
417AA14159BA843BA9DF9A3F35597267999D57404F5743453B4A8D4053A6747D8950427C
2C2D64623B8C4C3E9B511F9E5D7A2F3956744B4F5A8B6127616491AF4B2B6189644CBF65
2B315B7D7855547D2342897C529B9F3D615C2555786B37385E3E2A65C3896B9EA350355F
874D7EA480648E4F5A3CBD83504370297A9576405CA45D111D3F76373B736825
4E494F8A4465613E87852F304541586428522B2C563B5166154D47213F4417586440324A
29429D373748422B3D2B41437D80272B312D633F504F5B2F1B4357382643192B617A403F
C7311C4C724A2A702A327E38631818456E58395A43288C9657842F2A235481C4753F305A
255F948C35776179514780454F753D518359574E407376407D3E4C627F53644A2C8B7A84
57386D5F7C6C36389E6A3B578B514485742FA27269778C688EA249338C65597E443B6C78
3B7BB48192D27A4E81C68D6D315D6D4C759B7E4C5243355E3924544665A35C445032698B
3E35495C3D956B43974535B7AD976B4C876E323142684E2C694155845A2C4F8E6A4DAB7A
5E2E3F50632659793D44BA6739588653A18C5D4F4E7731509B60332A664A388697875E87
8639458D8A5EBD37402B7478484A77526E8C7F457782821F23277752336F8868
4D523E79426C4D25354A38303F2623582F502A0E383D43570B344E254148352A48463C4B
15155C37384A3F1B342040354355262D505076474A4D715A35425439213D5F3C6879472C
B05F3146604E286A5E3D5E335F5C2E5D6C6651879847B9A4609E45371F3057AA8936496B
5D8DBB9834837187454B844A565C293E6A63594C33466C3C79683A4969515A7146A18788
93468B5E7C748D41B473325BA37E57925D3DA37C5179895471ACA4399F6F6E8061477B3C
28597B7594D17E618CD8806A5FA2B08477AE6E72947A48374E305B355296905D6E4A3684
6150686740A28049BA4A2F874C7D3948938C5B3E48784E308B716371453680BA43539C63
433E6C7D824B4A771947A8A0576371385D6D6A4B4C76332D4D5C3C2B706F5A8F77927A9E
81733FA78C68A2375831808B5448876C8B9D9251867D86282527795E2D77A388
526C4487468685484563592F5B633F71346B5D1A3E46416F2548562A4450533959493454
361F81352D4F553B3E225E5C6758283864556F42527D7E673E3D682A3B575C3B5E552A46
9B6E3B4E674C338C743D593A78433A5A582B2C4B7347BEA748747B80672761916C324C64
5F7ACAA437725D9A6040804D5F7D573E71646459515764368B814645522D52643EA04972
9B3C9046629D8B43A66B358CBA835AA6662E775E529FBA7554828E44AC41506E7047A678
48619E3B3FC19E5489DF725125478D6F6D9F5C3F4C5D493433527F4F3E98466E8E613396
2323606A347F5426795323A54C6F50416D485F3F537F662C4F72747556203C6A603D9A6A
3A1E383E694B4A733732A987565947388A78794E486A453E596E48256B7572B9789177B3
A4783CAC894E753B633D7D894E4C9173878B7841ACA28D29242A6423409A9D8A
4B5B4376364D6045474C212C695145501F5C4119221035541F3F311D4C5D4C2F3E142C61
361E9B3A1A3F563A3A2F5A5365531D305A4B653136585D49404C74461C2D52386E5F361F
5466383259371B4C6A3459324F3C1F6767663950562A8CA258684B6D5A428E956B324157
3B49AFA34268446774424F326A805840733D4A684F546B34716F4359785D615139A68F72
8F4C974E4E79745AC09E3A4C9F3F519C7B31765E3452605351755C41A97C5D742F237687
3D626E6F8FD3855586D58F88354F906E60A68A5C5366512C54267F584196896956553276
323F52553676623E894523A54C674C4084475233547B5D2E4759656A582F507960349372
4E274F47754B4569673CBA8566564B48A8736E3F4B624F375947332265706DA5745B2769
A3742FB7642866A2883D707B4E5B9D7281834D178D8F7D24272F5E3E1E58766F
536544A24A6D7F4262765326414E4C6639593D1C302E325621402F1E374E233266423457
22289E6D2E4061365E2D5A4C7EA33C346246876941705536262562503160351E57706529
4E81462B757224577A327F4B5F462262766B3F789B479BA65286A28D452B6A9D6D4E8C98
6479C59E336D7C626347834D577A3C5D8E71464A245585496B39283D6D5C534D22838F5B
7035736451763530787F384F9B6B477F87357966626B8C7544638B63A76164565B3B9569
3D596C58AAD475569ADE697B374D93766EB892615567532B5B31876849A29D708173337C
51594E733C7F7A44875C33A993A6774E864953336183552D3C4D527248335B7153247962
2B2A4F48612736625E45B27B5254694BAC6C5E35748763374C4A2C419CA0729E7872577F
7F412BA76117376A6E3675761E2F7D677FA67B4B9CA26832454E6E541D5BA639
534E416D40767A2735484A2F4B282B5432661D0B2E3D357A143B42324C7F3D1E49364370
36156C6829585622573251273E5D2D377D3B67464E8C756649335425182E4C354D553532
678D3122474729707D345F35625D29556F3D2340694185953A4B42696A33539C4B2C434F
5D7FC7AB4F63614C7145824A50755A416C723F4945486A4A556E3C3D5E58475149A19654
8159935B4C8473429F903449977F5372784FA7A46A62826A3B698F759B52595C67438D57
4065A682A2D3871970B0B19F3A4D692E7AC1A26252685252592F7F6F68B4A06D947C3A8D
5C5C557831969D42725F397D82AC8A53914B4D141B51933D346147B07537697E54255B5F
29294F425626417E5D42AB5B3D91B34E9E7D331848765A3D62642125616448869E9B7C85
89333697784935504F248294504A7019486F8F50A0CC8C201D24603A2C93B958
4B5943773346663B43361B1C334D35391B4A4B191E181A5C2739320F36554E1B4020235E
4B177054142D52303B235A4B51702D22533E5F4636616B644A343E3A1F2C31343C4C4620
2C69381C36501C487C3D553F486B2F4A576C3C487B437192517453737B44789B88505564
6A73C7A1566977436C4AA15B51746D61776F444942507247576C423C5152535C418A8C4C
8B5D8D6F52817F4992963A5BB3A44E7E613371914F5F85465BB2A8649164738D874C8657
2953AB9182D4AD567CD065834456815553939E61525A17253E1C6D4564ABB054A23E3C99
6C5A5471208A94406D6C2C7E3C7A5F52BD88763A4662833F355C35707A336A7D2E298E83
663F514353122E52873EB37E29668C4B859D6D2F4761191C3B59511F564F35534F77477B
B46F1B716A40469C71335B66385DA4694970661774B282272C25562715559C6B
576742AB515792424348712F4F65303D425E4E1F202129561E351D17324B461D3B2F355E
4A18615D23384C2A3F28674C4477413957326C5E52776D6B50343D4223302F3B32544D2A
2D6E2D29566221427431544546562F3B53643C47704280AA708A586E312A509477415B5C
5459AFBA6B5888527946875E596765596565526C4F5066357696424A4F47446951697A39
985E6C733392A5508A8B1B3790885085935278893A80A66C26547D48834556978E4B8E7C
4E47B5433EC7AB5782D1561E31749F7262A3836298804C2E5E1C345B3B849756BE53319C
7458748B43738C3893933A9450895329585F673E5B8487214D8256915523637555315087
462C59646F55404A33276E7B5D66782859696127657C622B4B51373187AE72794A634845
846E2073991F2FA193456270183277694E78564337B4621F342D577A1942AF7B
5D6043AF5A4E82438EA07F384D5B3954465A4C1F322B325B1E3D31234152482A633C3441
3D2163753236532D3C33654C4DA4623F5938877657807E645853626A282F4C453B8E6F36
267D2020417126396C2F78603E6C2950898342465E1C598E5E7D8C85774C58A56C4D8D87
6D5076935E4A7641854573317D8C6F4B57564177554A6C4C605F45576B63544721608B4A
6529667B3A653F2E7496533A92372A5675709DA12D4C53632C61674292926078321E467F
604AB03A65CF9F4687CE74752B3F75646EB08A56494B3F434527576F41A7473EB07C3BA4
31143B6F4076852752673395A1A7845C8A4D4D234A6992412B7934907B34466D2317447B
5A2C302C4C3661897040886B3880B3626E7457123466682D494E2A1843696B884E83854E
AC633485923B1D66633E7B603647716C5CA56A53319A7D214F5C79721C3DBF61
5853387A504A74322E576D2A48321A57414F13093B442D7D14345E3C3C5D3A184759403B
242673903E3158204635441D2A665731572F66644F80681A2E49627128485725204B5729
37874924374D264A7E2347533B6C2A36575F2F59885559AE24324567703D59BC5625323C
5F7CCDC836397A514C26654C475C39446D6750761D346860655B1D28405253592E427437
813F4E7730755C3E779A504F9E8645577058668F656D735E378298524E7975996D4C4884
655EBE769FDE942570C08D9038558E2E5CA5A388836D38546A1A234769BC794997573BAB
58576C7C1F789D4D67601D5D4F676653764A5D273B4A6E2D296424565A3C5D6E46252360
563C34344E0F265A7F4E907E24517664615C733A4F6466324B5B6825465E6D7A7A9A7042
B3481F45774A297F5C288291594A6B3A4472685D2E8D711E20254F661C3CB865
4D6B3D8B425999454A407C304B6E3F4C334B35191F1D165B2A2D400D306B512A39292D55
421A395E20406B3535205B403373251F514A71322C73715F3F334A1A14344D4C2750420E
1E4D442945291026663E4B361A5242324E4F24285F5A54B244545D5E48326EB68C53503F
5476BCAF4A367751672C56455A6F4730405245873D3F6F45586E4942463741643B4C6937
8C5A4C8F356D8350849A213B77A34A62864E5670487E7F571D48755E90685C9F9C584478
2347A88677E1A22D5E9274805F89B9756B87997FC0832D376F2525474F8391598860309E
7967888E21568E3C5D7A34775A6E4E4D906965355A5B7C28315A39623A37597365352765
62483834656D4A48783E97786564726C577F8C5E595C512C587C7A213F603B4B458E8D47
B6742B4997552D7C624A7C793F4B697A469761552A976D272D2F62832838B088
5C6C408447354E395447311F36583F39132243281F1B0F3241324B1019314A262F1E1930
51214C750E12393D2D123F5340803017353A60472A4E5E5B474F5B44282D4D5632503B1E
1A5D3823555D252C693452572866402C5366473E623B45AB73555B69311B4F9B834E5742
693E8DB1683F7D47A34F83224A7A7A40534E1B89755079314184554C4943355E56467C4E
8470469B505173457E9F47245B6E5C99A0656B6F4594B77F356071707E79455E89534F8B
613F963B35CFAB4C7BAC59363A69976F6C919A3DB1793A426535386C30818F666A6D2D92
64508B8A474B8E2E7A7F3E78626B61325A65704865619E1F49834C6F4121527C663D2670
3E2539597270524A531A94A965597E3A5C9B8F5B50546030486A79283D77886D4A838B4C
C3802E4CAB582A9972557F80594B796E4D9F4D2F409179362B3564673535A691
6E89459A6B4071395F7B5B2B2D4A3448553E72342549294A40335E4027495E31354A5037
4B14476E3126482D37314E3F3E90733439355B76436B7A201D4D746734325B462E6C713A
1F613A1C6482342D612B4B681D6E2E2B6D7B4635592A2B895F54667F816055A773527B6F
664954866B45823B7D48833D33556B4A7474447B6A55896A407F546E84754E6544579960
7A4D4D985E625A387DA1722B594D35567775656B496D6062314C63495F975E72552F368A
7B4AA98368D2B65E91A89796384FA38563848F55B27F41494F2F566A308F5232657D3B9A
603C6690604DA4486561376B706874595A58925B7766A4492D82536745354B7769442662
5243252C5C71514862275C825C557737436E795F5C548747505E66315A88775B538B4637
8F6E3D53AB592CA2925070834E42896F44A65644287C783959696E3B212F9382
606437615F395A211F435F3133330D3D5E384B1F375B33452D34876B28495E0C3679513B
4F185C97482D45152A4243121D596A36393746713A75752D1D2A3F4D2C3F7D2E17466748
257C4F15365B373F8B3E3D6227673E17396B41457E743A9D49384C5D7B5B66B95B363B3F
6B82A3C46A3A88536730625C4A684030495F508C483171694567363E4451475842477E5B
6E563E7E566361475C9F7527477A49698A6E657371948D913A4F5C7241A45D5D9E6D5995
8B4F9F9877ADAA6594B1A2A1415D9F805B849F5791513744703531652B723B4D427F3A8E
9070A7B16C53B85B747B4064857381636C55A064735BA14C2A7B6462473D527D66381F69
614A2B3566865747693061B688507E64507B7E556B75834555653924386A604A6F98763C
A457325890521C71563E838E5A3A885343B58F632C8179331D2B54672F167B5E
5379416C67438752436E7B3A2E59382645314D1D144927412C1E654C213A693825533F43
5B2A2D62332C5C3C3832514A2F5B411C3E524E592154585A41425C31182951593B464C18
144E491B314317236246404B214F59284A6A2A254C743B9F654D5A526A7082AD87674C46
8697CBC788418A5B68608B674D5D8E4955766891844980794C5D63534E66484F4A4D7569
66523F93455F6E5765B08123407E4C608A7183766091A399454A758452AE69619D70648E
5F54A7966F95AC4B5C86998A3F5694405C9597599D5E4340783A37632F75515C3D663689
634AA08A546BBB527478316996A18058795D943A6A72A25F25845C746F49537A6E48256E
72632D35678464696D2E66A8725AB052479C852E464B5C33637B7A3437824C4155897446
AE7C2E3876602B9C711B5F81613E9048317D714B4EB57F48212B6868434C8B82
636D44634F2F5947535B5A29376D4C2C2F235432182B0B2E4E255D1D112F544425391123
5631316A1E153C54361A3A5D3C66462A274E436C3D625D714C43605D3B2D57725D444734
1B5D43192D4027306F40435C285C511F4063543054713A9962555F6257344B8E76565347
808DD2D18D47845C716291794862955452645398874A7876554F6051515A425938527E63
7E5C3E9B5981856468B07138587F4A587E5894856A8BCC8D547EA79049855B8AA4725D8E
3D407E715A80A56E6A766058485D926A51886B529C6D2A3C7A39385C40864D4C364E3798
644693A4263F93406A6A2746725852436F628D405B558F432572353D5036557347314876
735334406B40384966476BB7533983563C8884616E867A3344593E305695894657844C2D
8172273B725E359D9B5D687A6D4C8D5F39826E41297263461F2C5E5236247C80
7B924F507335655C6A537F4636595B2542255B3618301E3240264D342D28564B26403934
513434715F2A416E3B3B407144564F3A2D4D3C80457D758350435D68452A55736747504F
1F595C27304F302F6E3D4B55335252273F625530564C2E74815666668B7760B776575A4A
56427AA18247814D7658704C5F8F88495259297D834A80503F53624B524F35535F53A83A
655C479032486B5A61C473233E753B6B814F767A4584CD83353658475AB652508B6D7EA6
81519A4F31619E814C716C4342688A6B60A475488649333050343E4E32546C4A40672764
5937A28F574EA9486963314486534139868278467452AB23458D53495820547B56452851
77202F496278575268194C788050A27040795B5663517B3D4B5D5C323170855279627D4E
9A83345A9380367B5C586C6D5F367E6944959E673585723E3B4D6B5637268881
8081555480376A647771A1674A606922472A583F193B39414E205F483D304C4D295C4E30
4736348E73283D6D343E39704961953E3559449A447F7F653F506F6E522952503B60925A
1F5458234867322C6437526234464C2644675A2E6B7C348E803A7592876F65CF6A466571
765CCDDB8C4996576A40556A465763477289648B75478B844C474947696E534E483E9066
65593C8375715A324BA398293C75484E8A407274506ACF86573D6B7A578E59717F455DA3
6B4E9E83884BAD8048808789495399867CC3A843786F453B563569963C707762556B3D78
744AB1AE7953C1536D7E4F4AAB6D744D665F86468761AE462155475A73394B594D253D79
6D422733556752848A2646744C3B90814C79728579538B425C8B743B2F603722455D734C
895219306A733896574F86AC91499A7C3A7E8B6736855D2F16205872352D8E65
687A3F4760356439394A714E3F5C393355264A281F4C3744591F6F4736456C361B473C32
481C1F5A562A45461F3835392038553340532C6131849150393342543B3969675F3B5345
23716E282B4E373A794D4659363D5C2F3E6E442A5474398AA1445A5C6C6A6DD585633F39
809BBFD59B38756875654979515D79485165578B883F78854F4C6E4447705E5063458F6A
5D844484856B585E46BD8A32486737467E449F854D42C77F5D65889659804B7C8C7759AE
8563B699753EB96D4E708F884A649E444D947B4A8F814D566934325938736D3064614181
6E2877723B499F504E703A6088878D65706A8D427B79A26B26624B415A4A4C59715C2D46
865D2837545C444C6C3D41757455994A308C777884576D204465512F2C55695760427258
795A332E6A7C3C8E563D587E7240935A2C6769463B8449491926688343299089
68655D476D2E5761585D8F5945855E232D25593614260D2B6220572F1926475020351925
564D1C5A4222476A3B243968463C622B284637774461647052566E693A2445706E3F633D
173F5A242E4D20275D5B534F203E5B343D5359256782468AB66B6B6A643355AA845B4A3C
8C93C6CE90437F7484734B8250638B5F5C6D80879F4A797B4F566B464F6C61707440925A
7568428139586F5A46BE5C2B3E6D364B803872814A37B1844E3C75844C8164AA9C7853A3
5748846D4540A2945487A96C417E9E8273C375338A5C312D50373E62388472405C3E2A4F
51279C9B593D9141566C3040864571615B6D85637B5C9E6B3255574A695F414F716B3843
9E662B354C896360836B4AB19D6B965434837D70745A895B415C4C33265D88727953845A
8E883C346B673594565E6C90753D84473B67786458B2664616246A4D314E9E90
8A72714E983940565093A0372D4D632C2F1C5549172C1D2D611D58281D2A495A24332E2B
465719476B2A286A33433679553D834B33574478516F65503F3646604F23444F39509056
23415728557534215D4955734E316939485F7B2D69572F6F95536D74826D63D16A386E80
984794BA893A723F7E734A66508689515869576F924A765C45515D456C743B4D5D47AA5A
5D42398A6F56524862C28B343470315A8A43837E3F48A48F6E37567549895665815B5CAF
8C539A62334A949165898F7A49628F7F73C571378C57303B503A7695345E764577533D3F
501FAFA96236844273763A3D8A497A4E6E8399647F5C9149468947486749597F67633445
893433576C7B53515D3F37A6875A9082647578846D638B59477C724331768D698B8D5338
8E8A3D3C72743BA4B066638E6C37877D4B68AC67417E524C272D5D5136267581
7968473F70373E5239466C4F2E51454265284D411D412821541A60352F2D41442756362B
3E371B617F311B662D5237604A50AA552B3F216250656D6A63404C734A1E416B7147656B
264268362C5B4B205F50395B5631662E40756C2C7C6B387E9D335D6E655F69D86C4E3038
7850BDC38C429B6A5B665A904B586A46657C5F6977387A864E544230526351534E357E67
5D4434776B65584F3A9E8732357B3D3D6F3F7E6B5F4463656339583D44836A7875313699
7E509D876A59AE574686997F4A71925866B5843E886E333E4F3730663765782B5A614750
69297F995F4198484466454F9E8F9D594F5E77687E6A91432A45453B5C3432436F552C49
721A1E3040673147593C37B9864785967579774B508285524674562F2D4F66498999845C
99773A46736B357A915265A75C277A8D4A447E8693BB6B3D1D2D5F593E2A8472
696F7D3F793D3B7C8279A6762C4D45223F22431B1A413E2D5F144B40224E6A5E233C3B25
485916336B2C164E253D2A44322F5B3B31533A643F75A55D5D484E52301D67797B47704E
1E558146314A2B2D756B4B5A403C7C343A614D288580386AAD3E5F544D6565D76F654842
776FCEC96F235B5C4D4C3C5C475D73414A564D4C94436C8D4F5466464D634B556646A67A
676752858C6A5F6B45AA6836347037467F406E58613E58766E36526444736775735740A2
684B9A8F6D609E8F838C9D855766969163C49B3E748F3F55683D3474416A7D3F55583339
683EBCBE583B745748773D2D67377E6C5E7E95607799A86A3642473C72463651823C2F5F
6426284249591D61814D299BAD376E82476D735C545A7C374F827C4628623B3A73A3492A
7F5D414E786C1C66826B4681461F71915C51B48162936155383E706335349587
70556E3A5B2D294F6B6D9459387480474B34656725350E1E662543402225315528492229
60621B3060231D63382D366F553E6B1F1C3933622E36526052424F661B17415A54387345
1D3B5F272645261A454B4159421D64283850562363893552C3676E595D566BCF8C6C4D47
7A61C1B3483579625A70597748598F6563726F46A04F6E8D4F56824F53755F587B4F9379
80624E818C67525A41AB5A3639753E456B388794724C387F703B585D498873858C763E9B
704E8A8D6677B38F848EAA8F5974A99060C687447D9C3A2E503A2D6E3C5C6E42525E433A
6E4593B4834788584E995430753B7548598196666B6C7E4D355B452E56363669936E454D
5C1728545B622F3F5D492485B241617046616D4F56838F533F60562B3150514A6FAA8554
8C5F33375D71367282765498542C5881312A84997DC7763F192A5E5728235E6E
995D923A735F2F703D5B7C351E384F372E154B5C233134256E2B384845383D6A2B3B3339
53641B28592015492F2421554E396F4E2F403A5F68526776683F596D592F3F776B405D3F
264162252745271D47533A5767277F3338596C326482383FC06784736D5D58C663615748
8149ABCA713C7A574C735C7945738A675C77714C994C667948616A4F69834A5B64379D50
624C418270556F5652BB81343D7030507842697B5842419152456A524B975F687A58469C
8B55917F4D5F8C7A617DB66B365F867859BB7331627F33324D3B49924A6476365A504435
583597A768408C543E6F5D2F763C7B6B4470977877809275453D5D4866622B4E8D815155
6D42252E43682B4F6C623184C6494D89686D6F6149557B5A446B8A551D433F2257A88151
8D6846385B89405B894F3573542B5D7A5234AA916FCB6E5D1D215B5130277C74
8F5493387675378E426B966F33476538433451562629222566253343343D405B2E3B4552
52561929523D17503C3C375F5A48906B3C4848577450645D3D345F705229455D3D3C6B5F
3737763E356257204462406E7A28742B35737B3467843631A44A6E8066668AD459466475
915FB5B981347C604655507B3448614561865E376F3E688A4E4C4E355B67495361356874
7A494B6D8154564738885F35306C2F406E5175797B49226D733453564380777881603689
7F538A75667FB971515BB4674958977B51BD8D4B7D9C432F4B403070475A7B2F405D4733
59469EA06242835C406F523E9567875A3766725D648E8C83473554285C6E33518A4E3B4A
676F2E354868243C4F673F589D423E704867746F5C6A835D5D8F9655223E392B41BD8260
7C6D3E4B7B98434C8C683F90713857715442988668CA6F5E2127533E312E7187
7D546E3A614E3D9433488359354F4D27593147392338201C63203041294348531B443E3C
634E163E653F1E42263A2E4B2E32554A3450443A5F42805B50365362261F506651354353
2E4A79311C3A4D275C74324B542D763C2F59632962814C25895B6768556692DD89624947
7059CEC67531707C455753773B546B60487071368F4D5785544D5C484C6C4B4D64436E7A
7B6D72758B5D555E3C8C73362D80404771646E7B7B52244B5E355764408B7A758D793888
7D558F7D7784C28A684ACA7F6259958B52DBA84F7CA75A35714A2A7A556B71313D553941
504C9DB04E3B8C753F803D246B2D5C593F7974626287776836586F3F6C543550868A6457
63553A3D417F4C4452824457B3683D8F677A916A656E733A37558D5A213B373636C09268
6E89563D5A7B373D8E7C42915B33528B8B546F7C5BCDAA6E272C5D5D5028767E
984C7F406A7B4BA447547C5C2F3B472A442544401B2B231C7034253B1936445D2F402B21
47491E26502D223C3C221B4F4C415E34283C4F3956406550593652472C2537614C295051
312C602E1F3A331C355728504F1A5F3231547634588453217D6F6D53564190D68F64544C
807ACFC36C3C737D525B5885455676664E767B36865A5C76554E59474F806850774E6B71
8279737977586A543E946C32438942526E4F79AC75482F4353305C5B3D91665A7F6F3691
5A559A777184C67E4B48D79868609B6141736C4195B04D1F3E3C376B41696F31446B2536
4B3A8B8F2D2D8156428D4527722D5A4152928A766F95817A3E376F477059395F7E79545E
586340586586513D42631B5FA86B3B865563707964736E633A5A8D541F3E5D483BB89562
5C915941628A55338E7B4C87653B629C9D5BB5785BC2B173282A6E593A398A82
B74E973C65A46EA5475D8D87403C4F323E384260292C261E65301D3F32343666313E3B3E
434F2220413E2539492823595F495C4B38445E43694A725D68335D6848363C6E3E2F6C53
282A6D363250291C355A255057216625315C772D518E5F2A75815E72715A88D581516C86
9A5BC6C98B447364617F657F46617A5D4D7E5E3D8E595F89485364424D764E5179448E4E
5F655A7A7053513548A97F3A2B7028656E62556B7148385A513D656C429A80677E643E93
7F578B5966A9C97B3B45B8A66282C26B4297833C7989432043434F824856822250773929
45217C884C3A7A49578B4D21772F5529427A666B5F9C877D3E36664B6C66414E727F5E52
595F2B31406F3C2A3B5E28528B54406A416B79786A717E6B394E88531F3E593837AC7D56
758D553E5E7F4A37B190507F5B2A78B38C66C4633AB0915E2A2F633311377B7C
9B4C77375E8688B43C6F9C8A3B3F56354647416431272F1F66321A463B3E3C60133F4F3E
354E1F2D564B2D35472F2E4F4D5571673E434742873E84704C25465B3F27456967374B51
21307B511B374E2A31592461662066382958722E498F4922566A505F547185D989523E4F
7F54ADBD9840726D4A5F5D955751593757998C457D40678D594F57336A895A4A60319F5D
5958457B8C585342348E773D2D843A3E6D6359737A5B55696E2A564A388178676B713081
83517C686271AB774D2B9FA24560A07A479E8F649696642441502D7252508A2E45813F2E
4B4C7D8C423D74693E6840236E325640496A4F5A5CAD748030364F346968564A8290734E
4B65403642775729427A345E947C516B776D7D855E828C7437478C52214E5C3B34947717
4E996C3D596E332679794A76511455928174C58045AF9D5E2733544237296F88
974B713F575769A943515D4D3564632045443D4023304B2484471035233F617039273425
4F4E241A56473B4A372E2D594A47514C4766653D6230727167364054342B376765324B64
38207457203046274C693E50461D6A5D2D516B2854886D2661745F674C5B5AD68D613E50
6658C7D4AA38786B4B4F498266565549436B7B3E7442507B554D5338496556405F3E6C5A
546A58677E5A5C513E947C38317F4553727956665C414C565E2A54543A80715C6D6E3D8C
875582727278B86F543EACB56562A48D56BEC16FA29A7925415A2B726450803C4A74342E
4F5A698C6D4B7B7D3F675337878495795284586452CEAC9A41354F40797F604D6E8F7044
46654C34426D54314B882D677F7C527059608A6C639C967131416C2827666C342F727A57
67A869396F836530AC7B4A68697777AE947CD5A24896AB5750536C75582D7C99
B94C89435B7771BD495F626B2F4167312F2949582D1F24166D49153D1D1E2F5F3B232B18
2D59331C4F3B273841221D4C534D5A353747653F553865676E3C3C4F4E42385E572A4451
291C6D45202D48232F4F294F4D205958275560323C76692A647A6F6C4F5547CC9D6A3C59
7E64A0D197415D714C5B56806C534F4B4C7F8D42795B567C62424F3D527C653F5E4D7560
4F7465667C63582F3079672C2F7A3A457B7562966A49475B45569D7F3B8181547B843567
625797906476C46741377FA66466A06E25668B529F8C5C326664255C48958F374E612C59
6750437C5D418C672F79453B77627F634E90754D1943659D6B3E4F186189554E636D434F
7F8D51374465303D70822D777B4F47855D5690634FABA070253F672B1E3E46213478805E
6AAE352A578F6C33B05D3C7C79958CB45D88D099409BAD4527406B7751398380
A74B8241568B71C64F574E854C3B533428313446251F261A5E411229232B2F643E202F2E
2A4C381D4059362950253440614C6044344B773F4A446F6C743F304E54473462693C4D4F
231C7751272C3F23395B3A515420525625535E343A7B6C3C769B7176654D3CC2C5893C60
884B8ADCBA5F58645397598E616A5D364269904EA26D6092534A7956568544437B466E45
61865C576B5180433688763F53994F4C854B3855734E4C5551213E673A8A63358F85327C
2528547C7466C286793955C3746AA18B45A0A86192AC741F3C3E1F50584C723B4E71352B
4D3C3B8A754A7066348D5027723A60353E888E78486C5E87395777314B5C46538963342B
51784149587F31213C66438F9B4C2B698259AF69386270452F527C4A1D425632265E9378
6CB981415688602BBD6F376065858AB6696DD56850929F4D29356546294DA087
B04E6A3B63826AC8576E5B8A57374B352C463A59302444296F531E31363838623E234534
304136224C6037285B2A3F3D655068632F4B7B3C6A4B676A7F4E2C50554B2F66795E6163
371C80702B34533C3F5A2D6C7021445D2F707D3A3B77262A85995F8266281E7C7A6A3655
6D32519D9A4F544534684E87263E5E4839606A488F605A8B342D5B4B5973362C593E633B
385953547651644F328885342E66464F98745D8B785E5363733048553D8A834A59742C6B
61406564776BC18E7B3C46BA757FBEA449A0AC507F8D6F294D5B2B66685981365586532B
593D337B714D7A6125524E2A643D5A5E4A946F7451A171715130584681684F3C59826935
5048273344705C2A4B702C757E6F36696B45897D5E8C7F68273C7C5A21494F2C28599448
419F60404F813C289E7E4381707395B76B60D06C4679A549526974472A338186
A852592B435B539B2D556B734B3138203B573951213F69396D561C415F3F416717255A43
2E38242E5F622B40313136372E275D7326405A2C6A3D5069543146745C49275F34225F7A
3D1B743E3D5E64473E5821666F253C452047683547785028577B5EA06B61408887642E52
63528ECCA63455664E5A458D4B4852412D5C9554967653835438574D6F94613C603C7558
4E66446A84626B423884804739904E42786454A06143657E7D374C4D428C89545F86316D
78435D5C5B6AB2886C3A3EB6395698803C99B07981998B2A4D692265644C8E314A6C562B
5E5C42897647647C376D5E2565445A5E4F816F8856AF8085632E564C656E683651957B3A
515D452B4277432960793F6C8D7A3E76785E965D5E967E7639436B5033686529377B9D6D
51A72A356BA67032AC692C8691427DB44387C78C437D9742253B595F37379A72
9651703F446552A65F4E58714B353A191B40314C1916421E65651833452D315A3517382E
2248471C3B3D1B4B49222929413550561C37683E6A473D736933283A4A3A35675C355E4D
241A61422E2A462F43622D4C55273E5C336560243E6E60436D803D81655D3E8DBD734168
705A96B5CB3F3F6F4D533F6559446A392F778E4E844C519964496338486B714264477E5C
4C7F595482687E4E2A795D2934945942877B5DB466554B5A5B33414931908A5264613179
6F425D75565BBC61414883CC796097833DA1B9787A9C9B334F741D5A704C893F536C5737
756E44767A336F803270692A69717B70485D5D773D726A785A3154317285633850814A4B
7285532E477E64304A81376E9451418056518E574F9F886F34607B451B453A0D235E9F65
5ABD4325396F4A57D2A43B44666388A1313B974D409F975C26365D4D417BB677
AE567942405C3E66565A697E464364442C54335428233919615B1A3538273D5C491C3321
2F5E4A23403E154A4E262731613E5F491B457042632F388C784A2C495E4F194A60457752
1B19695436324124466F32454B19325A2F6C5D37246677546BA263995B4E2E66956D2440
5F618CB4CB5642734B794F82654674402E588E57866343855E5E724E5C7B653F5E43784E
5594574F85467F553A747E2C469D5C428A5A43A37462455F5334445239708B4E6D6F356B
4832414B6268C5895331499F77608C9952C1D0746898A0417A701F4468629F3B516A542E
5C60586D7C4C67862E9671356863776A47A9806F4A9065923D4C795F505E2D456F8F6636
5C5C163D6A8D6439436622749778376C4C4E8F8D6B9487331B37685C1C4B4D27265A8E49
46B373353E7466349F834F6F706EA1B3574295462F718E502633544C21386D63
A34C6646487A437379506B783F3057402539254A232336156A681F403F1C2B47371B301D
204A372142551E4D5D253F265E39694D1F2359435C553465653C3B6E6D621B4E60538A65
3615696339366555387030444827315F2663724D206267537CA25EA0617A5659A35F314B
6E4661A8C65B486B427B4B9E403C7657376A803B7F503E834C46594A6798514875426E4B
4A835B4D9A727952418D7E40338D5737806F46C169614273564155633C6F834783853C74
795246684872C39B5F414C9F6276BF9E3393AB5D78CAAA365161375C51457430628B7330
773A26599353658B21758E3C5E737052264D7967417356763D296F725A665D3047836A48
726D35213B725F3748652A4F84682E7946367D7F5E9E8778323F756D1F556B362B678D61
58C375373A69732F8BAA576E6B4B7E8A4A4087633B8B91432337605E3D40BA83
B25D66383F70446A72508E9A613768512D6231612E26502758711C51503030602E284A2E
294F431F43572056652142256532848D2E3462365A64336772512D514B4E21555A3E5368
5617737A3E536D584692363B5135365329668258367E58317A90569895854C3BA83C3B77
856C65A0D13751A153581D6C75447434356F87598B272D9178536221518D8F4E6A24757A
4F746A599F677C341862765934863F42A08B3FC2765A718B5E393D4642838F4A615F3693
8B6061863A3E8C734A264C8D4D3B638A45A7C45A3F58873F605127586A41931E335F6A2E
645C5F639A5A5887315F5D39759894594292587A43A06473502E6363598467384576313B
5A7A4F294077554D8481366A99894373615B78715EA78D6E33476E5A237475392A599974
5CC09037366D76465ABB636D799489A2523A7A683F95914127386671363BC093
8A45744A404A39685F36455236328A463C44317D31344E2F99851F4E432C6681451A3120
4B591F255C532161681E322F541C6058255E7C4B52453483785731535C1C306675586A4E
34226E772E32481B40705E443D142E6D395C681F27386439608C405F45745531A959252A
404B6A94CE472F554251365951396051254973517A622F6F6557625741867D4C5D3E656B
4D635C5A976E7F5038735D3640A952357A6F57CB7B4030485737414B2346824867663188
793955616255BE8050373B878A48648A3390C17F5471814086652839613B7D30536B6F2A
51497272985954A64966642D496D595C429554732E60528654395A3645524F31417A5D37
5E954E2B46754D285776336A7D625490523F6B705CBEB07E3A4B633D1A4C604D29657C50
54BE783E4A8F86438B905F437C9879A14B62957A3A949A3C274A715D3D3E8398
935A65583F843C487A45494B301B364427361C5535243E11575D216E3F1926443E1B2C11
1D53481D4046133756211E1B54326D511C22443F40411F3E5855314D665017396E56704B
480F4D5F373672502F592B334032265827515547164E7B37528D6F9656765A39C188373E
56446B8ECA543863465C37653E35695328548561726931786577985C4F8F6C465E3C6E63
51735766955980563B82762D23765A388153347A755D3A5E434A5F7142437E3584854B84
74677C414759C1863A244B9B8E475F913E97C1895569884442502338734D96414B905F34
704D786C81314495405A6931578455432B887D85459E5B9B57365F8456665A3446876D3E
5D743E2640777142628B295772552F614D3B665A36927C7A344070601D4E563D3381754F
4BBE71312A697B3AAD74483582876484243B6D6C3C9B86462433535B3335B882
A86E6554489441498E46456B6B2B465125612B6238274F3354591D565E3F464F37182C21
2B564D1E3D5936375F2338205B446562403949433D76385363622C49606E213B7759753C
4E11577F4955735F365A253146302D5B284456591953733A5B886EA16067391D8A693139
623D5091CA62364E3368376339286A5F2C547637725F358449415B57458E46477438603F
557B5E629F576B42388E7043298C4B3B806938B47B624864422F3B55463588437B77516C
656BA33A4E54B299583C2C867D4D7FA74F95C76A578C8253594B26304D41724A359F4A21
5F67735172503D8B416788385688493E2370547A3C9A5C9E4442757F626941416C937344
5B79423556886A4A5F783253838E406F624C525A58B38D813B5671491C495128226B8970
45C291403D787938AB635B427F8B587F503C704431977A512330624D203EC698
8C645D3C3E723B4A7256828C6A29405229682B5E413257334C5221535B3A4A4731284B20
2D544F224A693E3363244328574872923A304A2C4F7F3C5D4D343960675C273A5C415864
62164363233F675C31572F3D633D2A5B31627C5D194E6040738F5E996E796221896B3E54
6440686ABA4E345443573A6660325D5E31608A55754F408967516D53489768555D305556
50624B3B83656C352C7E724D2D8F59347F8240AA814D528258463F5644367F428676516D
757CBB5D4667B4945F3F34846F3E4D884A8AC48E5E686F605C4A2D386C3F703029854F25
5F516358B76D39993C4C683F538E455832805A83459B599745275D6A626F46285680563B
4E7A3B23396569455F79256295843F6C4E2B686852C7B17C253D616A29546C462352813C
4AAB9441426771227F544F46724F70AE67407B4C38948E6E2E4274672C2FA28B
90755D60417D3D545B3343595522432E2A612856233761424E4B144354365659291E3C1E
3C5E1A256F6D3E395F274620371C4E663D2C4632346D2E64674F2B314C493446684B5434
4A1537703D367E632A49292A48392E59253357511F665A2C577D558D6F71671F84602B37
5B476D86BA57305A424F2B4F622D4E642F3F7F5E63684B875F4F6F59356C66656755613F
526D5E43956F724C40896E352D855A37806F3DB385654058493C3E544D3D904B856D517D
766ABE6D49579E9155373681904B52935881C998626A717062463258843F73332E8C6734
67453440995A3A7D4D57663F4E9F4E542F765B7B4E9A5198572763766674633060813B44
6D995A273B6A322E7F88324F6F6344854725534638906768294157352055753F2C636C3F
379065354E967E337766484D8470588E4076A5663A99AD36283765733521B47E
9372605441763F47833E595541336D5A25592E5E502B56355D702F4160253C514A192B15
1E53461D41582B4477323924624D586225324B42386E2950616431345A2F234664546937
3A1A447D3C2D6B372F4950414516355F384A5649163F6B3E64834D915F71632DAB87383E
57334FA3B6523067475B416972394F6F3F46855F6B7063AA5E426D67476E71646A5C4F54
59756F4C9B6592433288592C2C8052337B7536A5856E4F7042374058494189478971425F
5456B587585A7CA4413D74A5A6525490597DD4AC69989A6E91733435854477632E895531
7B7B625BA35056945E5353396EA48468326C6C6449B59A94622E564860915F306380513B
458B5A3F557B724150893B698D642E6268425C5759A968643A5F78581B3C4A152380875E
357F6F3231606C2C5E5F3F25777461771E397442369F6B58222C5D5F241FA598
8D56525B428C3E4B8F444E5E4C22465427362252532656443D6A313359393D4937192C17
2B585421406F4D2772343F19595E546032223E55368A3E605D763E316276293A734F572E
4C1F2E7A4E1D75572C3B2F2E3C2D345E33505E5A16537345779E708F756A461C695A3545
634149B5C0733C723383516969357A6B4046884F886A5CC85958826C44726B6B8C5E4551
519B773BA541595238868C22339B7A3A917740A5976C60915A3A5B8462358B3E95834B5A
4F39A6AA825C97C85E35377F785782AA5B48B675467260434653364A554C8B79318F4B25
5D5B5C65A57227705078903F3D9846612E727E83406C4F8E5A557C443F5D39439192713E
477A2136446B71404C8A195897893B7370307F8561AC91452A4262611D3C463622528250
306D7346446378363E534D2D8D8C588137458B4C38A5836227315C602A2BA097
A782523D4A813E3D5D49738373243F4F4968455352306453425C2B4C62443A4624294C22
2B5851253D6C552D712C361B5D6D505F53454969309A445F516B453365772F3E77515744
71322F8B631D8781384E4644623935612E536F52155B523270926B9173746E247D65384B
613D2E928C552B4E2352357074293F5C435A953C717965D3513658615F96644C5D42535E
4D755349937369452F978447267B5D3685822C84723D5D875A3C495256508C2B7A985186
7D4F9AB7444599B1854D3574895058817A38C496747E7475453D2F836A3B5F4235A06630
655A2448856724723A465F4943AA4770384F62723F7F51A54A28635F576A652F6B987341
416C392E3F6A773F507B3D4E776E295F5F30684A46AB83662F41566622375230264D724A
337C604B495964433A796756A8875D9D5549854544A19668262F5E4512306F8E
755F56553B60455D66402D5467294A391D4D376222236768435B171B453D354924133130
2B601B1D446C533190254C223C2D547D5837583C2D87466267503443766B3C656A313837
702E3293351D767C40704E4643333F7023437D6020687A3B6A8E54A6AA99702094663370
7F3F43C6A96A2E6C3C502D40713B4752314C705B704262BB674657394D596D51593B395C
59735A47836A603224816E28267970337F83398C9C6B537263474A4E46317E4E83743358
6F599999615587A29364397D99545B856C49C7AA71937C73463E2E477B4261433185552C
655946688863276E4C5E6B403EBD5C683966718945866F8D6B37646F556578346D9A8445
4082623343797A404E8346628A7131526C467A4C51BA8E603A445869244B6E3B2542692B
3B72834F4D595E1F5EBA7345AC43539954478A3A3BAC9774282D5B6D371C598F
725F494F3957273A6B4C405C5A3887682C4133604C204E4252723933391E2F4F38183725
33554B202C4C17265D1D23204E49454E25407458236125537161352C9B372544684A4E30
50224787621680332C626C49401F3B7F44437754195977496E8943524467661D69592E42
4C4044B9B15D2F5C2E4F3C54773744644E4F865D5B8385CC69444C67515B7D4E5B39474E
54655F5795715B402B84773E25828648A67D4086A9695172533D4B57593391517B725154
4F4A9292504F7F7C8E4C60929A565F864D43B6A16380734C526D3440804D8168437C6B3C
7C625F659B5B3C855E65773160989A8A436A70763483929C6635594B476D663C69917942
488F5E304177724972995D598E5C42858A498563379491652E43563028466A61233E6D4C
2B68924652897528338555269B79449323689D5E43AEAD332A47727D3A23608D
668B5264397F48416D4344542D21565C2D2026705A275A42356D31203A29233A2D15373E
1F6163292D565827612422174B5C5159312D3549207449584D604C2F9C61314077624832
452D2686711E605D3C5230423C43337F3B405E77286A7843728C777E545D412B80733656
57312D9FBF57366528513B5282404F7B56578B68768A77BC694B6974645B74597042566F
65897866A573826B3C836F393D95943D90724388A364486A413B7E9F5C2B794AAB875752
5238809F804E866B845837679D5C60868348C2986B6E78752B401E275533616C316E6522
677E796293562D734E4E7B4737994662496F778353684C7B632B543E418D5F3563843549
73894F2F456B41314A70465F924B225C72357F5D469D7A543E5F7D671F3B5B342A717D54
3B9584262C5D7D493B82471F5F774C8B413994413FC2985F1F224C743A256496
658C546E3B874F42793B536A7E37525E31445F95602F5C643D71331D4555363E1D143E63
2C6372353567652B5E252E1D516B42583C484B5C2A88625D4F584F3EA8512F3E78614442
573729987D215F6D3C594538462F3C74313A65561B74724B80A172604F585E2563482453
5D261C6D967A446D245B57447833646946538635747C617C4E415D62514F6C445351595F
47766E458D3D4E593C827A24226E7A3771703B6C7D4C375A52293C554A235E2D5A714B52
745774727466908A9A764268935A80A99B4FC0BC509FBE753944171B58445D472745562B
5E496B66AF6D35794C6AA55639C8507A5684ACAD6A856F965825767651653D3088946340
4D79224C6F977C494E7D2A5EB275486B753B937250BDAA561D39565523445D2719396412
1D548B563E66885440B17129526841A7633C9D3A2D949F6625275F63172E8E94
848E53503C6E42404F316F83733B4B5A3E5A4F6B4D2B5D5A3A6939376757403E331E5E5E
28595A364F7E692A62374B265766458857594B4D2161494D64473022704E2F3262343455
582F2B8C45256B702C5A42436C3A3C7320497759255D57256694775A586A72298055306F
76514983AD7333753C4D4C387A3F4B4F525094664C825D7E63585B60505D7C70784F646D
63815B3E887256362872763E2F7A703F9A8A47739C44538050424C6E52317C5483726175
836C7B92683A86A68C5F456586353F6B4C46C7CD5F55725E454A2E635B405F5A3C766F35
7260253A916C348E3C3575634AC1A5B0493F5E7B3B8CBAB65E1F4A6F5B962B1C4569374B
7C8B35223860554C808D2D305A574979862571613CB6C4833B3D52472054806B23487C43
2C5D865447676D1332B77D264C36288E4D67C261329B9C4C3E436869282D6986
615954503257314F87464A4B5E2C492B2D5934642C1F746637782E2B5052394B55183A49
2952291836635C236E1B3D1B4E332955464E476C3C84535B6F6860377B603E3D7F5F4642
3B1D237B51215052306D433F43383C7A354271561E619A4C749D3F3A374C37299E801E33
54544492BC7326703C4C2F2E7946515327476B634F403D73595A643A455C696580265968
5B7F54327A476C552D67811A288E7A2E67783578B354274B57306DAD5232505E8D813155
655568905A5C7CB57B515D9AB3603F6D473A8B9F706085503665252247427B822A46521F
716B5A6FA549329263446E193B994C6F5E4A85812E343C7A8D375243399468314E812F2C
496F4B25456B2C1D78905D4C9A551F738043836B277E9C7739516938143154251550A690
366853163F9F8F3D2B8A622B6768548F1755B1703A9F9C1318468E833C22557A
695E4A593C793B2D67415E673D3069603B3A256642214E3F3C7E47446C2B2E535D24412D
274748233E6D342D682B3B1D69643B6A363F6C662D7022435B5043416C37254F72563D4B
4B19397E5D296529226157454F1A38873C3C7E5B154A734D8BB991542F4A171D73763C50
6F3335A7C08D466F296A593784346E735941695572795C6D454882725157603D95617B6D
40908052AB20496C47719018257B893D7B7C1A55A96D3E7171193A686D264F42C4A26952
624058BB886E88B97217417CA366469A963E979D6676AD72255434304F3282994F49541F
616E716FC85625816D5894403CB557804276928142906B8E5F3753493B6A3D2C527E6543
507032264965674547765B53BC622B5C7C3C7C725BBD99642365AB7E1C3B5E1F18346C55
3063703F2A465E4F2DA339227F6A73953231A53F3298823A1820446E3E2A6A89
7694523E3B6D42405A3594B06F2E3856344F2B553E265B5630592E4E895325435319595B
2A43552748714020501D27123545367D472F3957297659564F555048A54F382B6F623A47
5B3721786F32675C33614B4B65273B8D2D34886B3357761B5999817F8F7163286E44274F
6E29265A79924778124C623B6E1343775E427715579B727B4530466E4C5F572B5B6E9384
33596A5CB2704A514A8C91211A5876458F9B4969B33046697B44445A4A4F78287D8F7051
8C1835858E8290D095503A608B4871A1AA369BA7594159612D5C3C57652B4C7D4C7B8235
6D673C6BB88833854354786B41B960815A41545C489367C33E3B9674597E5832537E613E
5570394589A6824D4A8B245A96805D666D1B74A664BB965B1C425B651D3B643E253D712F
26817A5441646C2734B8531E7A5368BD513BA94C287B8B4B23294D6737296383
648252563C6E3F4763263E56673340321F5D3B6B221458733C721A265F74353F4A124054
3C481B114B8374296918471F483A3E95753B48490D7A6A546A491A1F68743B30612A396F
6F3F278565338478366E573C7B4F4794142B6B7B464B962D57A269484D5E5A20694A2E8B
A04255B3B16A3C90454A463A99543C563660936053505BAC705A5C5B74B984566232578E
5F5E42387770563D1C7C9C4E2C6F712184963B66904E2C4A48444B62361D554B8E855C7B
9A714D7A8F3D73C8996D375D8654306F9633859A755970803551254B81404E6B21567A31
6958414D8168337E524D6A6238A77179835B717467A665AB67295578558243398DA57945
5573291F43696F4A4B7F57478A8864706924545B3FA281483F56625A1F51775E26478163
2A698A5139566841259C621F794065BB4238AA4E277392381E26476F48266790
6167566E4066263F7D47566C71435A68284A3B594E294166376C41276B773A4B64214056
4846491D2E6767297B0F47224D2F2654542F4768316F555072686756575C44457132222E
624F298649116B652E7B61394D2F64A14D316E7025488E5463A583944C4D71286951193A
605B5DADB3721E794F3F2C1D69453D5835316272485F3543544F59432B4B664E5E46566B
5959705767625E341B48713F2B6F802F6F96404A9E6C34566C4C476B5A2C6556A4732950
89754C8D914B4EBC956E36639F64397C89348EB36F5B72863B52352957374F7548465326
5C4D4C7EAE5B347F4C5185493D977F7F85677264499AA69A753C5856477D562E56876D49
568A57294B63534F7095726270827581894A58795BB7B47A3F637D5A1D3B5F6B2D5B9D7B
316F8A4C357E874C309F5527886472CF2E58BA7C2E54A62F2235657138225180
6445475E3E783E3C88435458282877793A4B3F7771364A5A3D974927574233616B253820
2E665E2E3D7A2A1E86373D23696F357342367D8740662F395B85634C693C175079603F47
6E2F3D767233831B0A56714D5125297250376F6E223198535BA183937547482D7C61194B
68383BBAB93823652E4A3A2F854C40613D418174448A6367665451754E69775052425185
52606E618E6F65493A5991402F7892447CA54455A97B406F893F579E74386D5D80687456
58644E98977763D1A159518799576FB698377AAA536CA08B3F5E3838553654875F51632C
6D715A8AC55C33885352AA57398975816176A87E33775E8A5A447D48439C7C4B82A1533E
5F6E2744737F67595D766F5A726D3C8C965B628E5E9F9C691E4F6D5F1E3B552021556865
33664E1821596E57309E3919666D47CD4B319D8325478A5A2B1F386533284E88
6166474F3F8747437A325E8358273C51232C19424D26482E165C321C3F492330511C301B
1F3B552F35664E21562130135361337D1C13385C3A5F2C4F50735B4886673C2458483B4B
67221F72793677562D7C6A3E4E48399A4D2E686E3F3DA64D62B58B85AB5F5A266F402284
9A3C3EA0C43B3081376147379155576A5049977C4E986773714D6F8650707D4E61575CA3
51785C51A67953553E76B4562D7B743F8CA333509C3E2D4B5843476F4A42845882837379
88644AAB9E3D66CAA57336589B4E356EA04178915F404F682C4C3263782F54783C62743A
666A162C987A368E5334735B39857D806A4B68734B9476C248225D784F85301E4D7A6739
4F5E232643475B505D6F2A36796B4F8B841A47773C967D59384A6064203D5E5E2E567B59
2A634B3B32738B722FB5752A75814ED08B4795861E5790693029416A4F29528D
617C4E563F793E519547425271444B6D2A4847564B2A44543156311D3970414D521E4537
2A45552D466A562471243F2E7172389B5E27456C39564970827144396359433C6A392545
4F314E9F7029775E308967396F504CAE4B3B8778424DAF3740AE695984635C21804F1135
69554DA7B34D22674F49381E725047542D488E6D4B5C3F796556574F30607D5354363494
6259462D6464633F1F3C7A33266B803759993C5B7F6F3A58A0614C6C462D65618C6F3E4E
755746A2866A73C9A46C3D6AB16740728B4098CB6C586D902E4732304E3750873847472D
68573F6AB2612A81535691543D838871877C7D814FA682A16E3E61514C87663F609D753F
4A60412F4E4766575762794C7A88418885324C45259D753F3C4C5C622052786737586456
2785764F3D697D2B27AB742B784B2CAA805BBC8B2B388D493F647A6525317E8D
5F4F494C3C72373B844457582A377E6F2549335D52245949337A3E213E362D6061204122
2165511E2F62301A74212E2969312D9B392C605F30511E4E6E7656465D57202F57512C37
441222595F257030174F4D334A272681513B796C19307C4240AC7A63BA585E288F902F4C
895340C0C45134734C454C34866E485C504394875C81585F595B5880446568514D4C529D
605965457C5571453A47883121608A4A719C3C53AB873E509B5D497A563C827B91687B67
5E424EAE81838ABAA3565B8BAB703F7995406A936154659334453D3A6938529560535C34
6366549AA95A316A42558F433576848B8D746A6E46B7C79576466048438F5D3B5C906D4A
69874E36554A5B5B887B8C585B8858828F425B5238A3BB614749593D19687F69355A5D37
215B5B393A816C5826B0702F686933823A3992773745915C1F1A415535184883
5E864A583E915547955955506D4240623136212E402744391E62421C334F2946431B3536
2348643138665E1E5D173D33564C40B95132354F463C20766A6B524B5A704A24655F333A
4D3842948428734A2C5B39344D3F2B96473B70523435AD533AAD8C4858A1401D7748274F
9654177BAB5634793E64542C795C506552439E85598C5C575F54558A516D7255534B749E
627E754AA14D8A744644A0301F5D8F4A7D9A345EA6803E5A845D52904F33847B8A6C786C
545B5CBFAC8672B89C523861AD7641849B5381AE6C4F8CAC43523B37613653905D4A5329
63717599B97143A66B4D8050335E787069816D6237786A7878446935479662375D7D2C2C
4E693637544545305B78825B64843475A7507180327F826A485966601E3657553B928F75
286F4219308B875F2CA55E24848D536076619C893A61AE612D203A64331D5189
64954D44407A484A8045909F8F5048783C3B464A3623495C376D2D16316C3C524418394B
344C5E373968642B5B174240564C49BE5E46475951371B83716850498C883F266255333C
5D4C4EA183268B69277554395151299D573A6B6C5733AC4745B5934A6F846430873E1D3F
784C1B4EB05A4C99334B5A30782B71856748B15A4B925E6464588790516F824E675F5F8D
4E676A538E57665D584BB72E3086964C67932571A37D3268763772916134824C87978264
60473D99857258A9A55E396BA54F5C9CA85D8ABF5E2755782D313C417044709656435A33
73835C84BE5B33896C4585503E728C74557C9F8F509F92997B4566383F7F60385985693F
53613D48774C7D49526C8B4C6E80375A8C346E4C2A958447356D8A6F2038502B1E405E53
2F94634F2159624E3CBD671663395180816EC0A0334294633A27405A25267891
748750533D8B4D49904A45506D484373403E4B4E3C294D6034682F17315D404C4B1D5453
2E4C401C52775525581F4E51586444B16A414462565134777D692B264F7445467128254D
644854A46A209B792B88673E786A2CB3404B8D875A3DBF5035A77747B76A54288B401644
784C1D30983C44614B4E4F2B8A635055305198704B4B297278364F64408E94534F4975AD
79753D3A814E6C55285FA05A29587440858F2D72A276387D7167497C4135867B836C7960
5C5061B0A18158ABB58E467DA149365C794895CA773E5888334B37809A3A55854C6E7239
50752145C481429D4951B75442778D893767658E51BD91A96F489163568B623F80A87F42
5367253C7D918B5855718C46A38B4F61981C75795BA688553A4550532C6C895E24495A27
1F73826A1F6E6136297D62325156437F5E50D5A66A58AD393858705D411C427A
65544C54406B3D59A257566783433C402C6C474957357379376A44263F52334B54253C51
27626436335D52236C1B4C3B80543DB97F474A532A5237698E69473E4F85282E5C53332D
583D54AB6C1B875029776338443A2A9255366E852F2A845339A98D4ABA50462B916E1D41
7A843C3FA7739D9B445F682B6352667450368C7A508656576B436C7C3E5C75587944458F
589A72479E2E5472443DAA322B7169219CAA4277A140246A81655B90432D727DA47F4F69
9A6E68A6926B5BC4C1776598B7723759824070A06F404A9446685435774B6E96366C863A
55654250A36943AB39305E5F44728F78487B5D7649B691865F355045489B5B304F8B704B
706B3A2A3F515E4D96B13E21938F527399305A553C93C36545484936173E5C5826507F69
34908A6E1E7D6E733A93531F5662458B3640ADA45F67A95C2F2053753E1D4C98
69714750419047437C475A583C42808D3E46375D6C2A485123897926342E092257394037
0A2864353877351351274244812D228B5D41735F4C57224C6440343E4D803D296A5F3239
522D347C6C1E9453274D36404A5321945235688A58339E603DA79F4EAE7232238F872E47
83501E4B7196A1C02F385D2F6820497564388337277E595650346988517270294D62528A
38786E53834E57574B40C0323281873966893483B672216E642E639C822E65409EB7533D
838564A9BB8444AE9F404568AC593C55924D86C184403D8533413C2E633281984B5C6724
6D85687B984D61C15146744D3267A48F56845B783D9FB3AA773861404EA0803A4C833B39
5269472F4E6D4B2D57756F3A9C632E5774505F7F3C749968494E617126375043316B6F58
33737830257D714D328B63324D6A54B05A458D98357DB25E2C22427643306595
7C9A4C424277484A6B307C9874302E54385B212F5F3350591151532E4D5E28203F194A64
203556273F80641F5E1F42484D4B379469373F3E43653966654D363C5F813628655C1321
653E338271217B7133525445605F23985D45959E692B83373D848A49914D3621804F1D42
7C752356519ECD9A46485E316059455E60386D7559975D5C4F2E4771516E6A4C5A5A5C9C
6570724B8B6A70563E48BA2A236A7134557C1E69896A1C506D54466864376F406576815F
7D533273879358ADA15C4D65B06049719C5289AA824B7F8F4255542855375C844A556B26
44796975B24E3AA77B44754C22359EA35498AB913974638486487C8072977C4655967646
546E4D3A7084864D737B3746B48A4A6A885D7577568FA65E3563846F253B484520477067
2F6C87661B6D57433DAA8238693F20655D349096435E9E4F2F2940651E155483
608B4C3B376B48456B1E2E496A3D32300F50413B451475843C622F14345C331F410C265A
283448041B55632677114547461F147985494E3F0F49515B5E5817155C7E412361472338
683137804B1A706B365660286D64228035305C825322965F2F958A5A8A69462D9848224F
8274354641B6885F3A505A29615C4E605F3D899163A5615166496888495574647269509A
7A7F6F337F597151384BC022266C7B3157903C8A94651F647C6B476C5A38696DB4787D5A
83746BA1976557AC976B475F9B3B558D904782B861204D7F4B6855285C486D93584F723A
486C2F59AF663CAB4F527F68414FB99531596F764FBA86A75E246A887499563372A47344
5865171F3576804D6D775D4494683E60721E50715087BC903150606E2B566F6229488566
37A89F75206C571A1E6063396D6A2F8A633FB5A07A86BD77352B42784C2C4C90
619A4D5B4191494DA151525284433554253C413A4810426E39604119294C351E392B3264
3E2B4C2A225B5F257819462D554D3A7A77484E4A445553434B584C213075432A6A452A35
4F2E35776D34595F38584135566224825A498786352196643498943284854D2C8F412146
716D345536B43D654F4E4E21775D4A623B3E858C5B8E5079744A6883486A8E65785C60A0
6C795231907F665A2061BD5B39677C368CA140879C4C3370776248664B44657395826D57
A779609B817A48B5A263476DB4483A66863689CB743D5F8C4D5631438947668C34468D42
447A244FB55631964F4D7E5B484FC087506D667347B797A06E3A5C706E936D304988683F
48654A2E477D7B4671745C61A28A576E7D4F5D6E3D82D79D525566421F61825C264E6A32
1C73876F276C4F74347468459D8E388B4A2981A765A8C56939324B6F522B519C
686F4E5441662545744A5A4E674B557D3D434B395A304B813553402735552E355636365A
2B3B513D247168237C2F48275463417B6E456E3E335E55436A77722B377F1E317064424A
5E2C4D90724B5160397D53465B432E74613B899D42297D502994A8366E68552F87601725
556E523B5F852F5840484E21604838634C295769426A1C4D6647675F3A65824C6B504A8B
617C653D7D53625D453FAA4E2F616C246B8C3E808F4A40846D5F4A6A4E254B536D995052
9F88779F927254B59A3D4182B97D41659143609F8459848E3C6E3A355A41679B44448730
517C5B67BB4E34A155415E4C5683D598678F54672C879093803C6B5C66B8874E52A05849
6194733248903A3189905F5DB6412D5E7A624E832767B37A447BB17A232E56622348676F
2F82645664AE947737774C2EAD803B98642971A456B7C75A3553717C4B314F94
6160454F40813F3E7A48594D3916355F36392C526B3A4F7131796232344C0E123D3C3141
0A24523A266D3A0E62382E144F7238724B1E434A3F5A3422345F6A2D357C371B564F4854
5114467365573F592F4D4A3D6F4322734539A4AF551FA8363FB19B5C6169281B798F4535
5865254EA96D45602648682F6C3B406D6B36617A5199515B534180813D456C2665765D93
3C938C4A9F394F81624FCB503E8D95375F854084BC692D6B623E62B282375F4A7BA6634E
A271478EA08F5FC9AA484063AA5581A0A0486FBA6733909940433F215C447D9459549120
476D607AC04D2385756AA35B3A9FD6AF53915889449564916F3E695B405C804F5BA54536
3E786E2F498864385D745A50A9743F62665B5E845787B69542445B692739572A2E5B6D6E
338E5F3A214650613987745DB7533DAA844775B82F93BA8E382E4670384587A4
659D4C474380474477356A717A3C2B6238401F23613651631957442E4B6A44173D2D3C6F
3F35563737725527712B42354B60317B6944492E1E80523E4958362A54975B25551D336C
7B322D884D35497F4B655C2262662589201A608F5E2DC5431D778A5B53641D1470462B31
486D1A2376694B6D2A3B794A94382E5D794275511F8B5E683D1C568D44506B183B7C5CA5
38545E469A614E5B3C7ECF42205B80355C852839955F4D928B3A3A585D49884F487A8951
9E714786796C42AF93494A64A7433D5C823F83BE814C629945563F4F844B55833D599041
477E25409059297E3E3C61603A3EC89566B8AE7143AE7FB05B45A471647A825C95B38E4D
5577553F63948846646A3369AF8440605D2036562C91C0795F526D7A293E6559234A7230
24938F58297C8B2E29A18B57BE3E2167362E65B85E89CB6137698E741F1B4B93
5A8753423452404F5817374E804C343C1767553E69317C854672351C3E6F601F392A1F57
413A3E0E1455722975124B594A13107C7D4B6B4A0E58634275773B17296A4152842A233C
5E2F40853D26266B4A6B54296B6E30944F2A5D8A2223945A2495B65F7A9335258A3E2953
69874B2774593B78523549246D68325F3D335A6D4D893D81754A6479507C84585B534395
646A5937914F5A592F45A049346C7741979A3F48B54A2D738A6C475C5D304B4B826C6D4A
AD956596887756B7C04C4760A84F4A6F844689D0A067729F5B6D3D2A684A558338528B47
4C714A6FBC58328B474D614C4A50C972406059613FB18C924828625B57723F3166996B47
536A3321527F8D6466714059A873325A573D3D592F8DC997615A634A285F7F5B3486A243
1B507F313A94A64B1D633C2D7C77368D6A3F666B4581CF64362A3E67392C5995
5A5842453848365D76363A43664C6752133F423F3A0862804080431837665A47553A2E54
37615942264F571F762C4B4762461A63382D4944295037284B4F5C382F71211F57514657
5B33769D695931603FA5773A76402996663060726121BC602788944645734D228A3C192F
416C612C80432A6B3F3E4A307C602F54542D5B886189515859445E7C3E466E65554B56A4
6972694B966C6368545FC74931698333668F4052A76B3277926E47645D3D6F41606C764E
8288669D908771AEAE58546DB96E5B81732D64AA9E5174932F6D57347F5D5D895169A659
8B886D8CD0622476755471483561CB8C5C846D5F40BABC9B5E3F67495C77534D68A16642
6C785A2F788E3943A09A5465C47638748658448E3B6FD4AC5B6E865819314E2021567E7A
2F5A864F20606B6C2D7C5A2B4C6734887B498958347AAC4E423E58833D266495
62974D563C72434583424E503928436530373C72743056492F6E5024533F201A372F2E4E
1E2B4A35325E3B12553D39264E531B6C4E375035224E44407E747D42327E45256A5D505C
6829365D575741482757663D7F6E339B6A3060756622BE67298A725B3F8B361F864E4939
39845F317C5843732C3E47327C5D355355366F9C6A85686C6645588C505B6F705A6057A7
6D727B4D854A548A645DC6403A8C88397E854770B4753060862F5DAE8B40734B5F5F995B
81865E9F838E749BAE535C8EB87C8AB3A85C7FC583578CAD3B4E4C33754B899D5274862A
5F9A7D7FBC833982624B7F794F40A67D5A9E615F297D7D8C8F456B3B2E687B5081B2372C
4F734D3E878D5A364F713D71D05D134B675B5DA3396BAF5632708878253B5D4A2787A36C
3D5C8C50287392843F7E703B4F7A339A794897674985976F48476A8B3E2C6699
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.754707 0.00276855 0.97619 0.113511 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 0.003125 0 0.003125 [
[ 0 0 0 0 ]
[ 1 .5 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .5 L
0 .5 L
closepath
clip
newpath
% Start of gray image
p
0 0 translate
1 .5 scale
320 string
320 160 8 [320 0 0 160 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage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P
% End of image
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{640, 74.375},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000:00000BR000`400?l00000o`00003ooooooooo
oooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`004oooool0C@22PX80Jf][
08^;R`1EEED0L71`05=CD`1GEeL0M7Ad05]KF`1ADE40TI6A04a<C01cLg<0RHV905ADE01ZJVX0MWIf
0492@P1UIFD0H61P04m?C`1/K6`0GUiN05QHF01cLg<0DE5A075aL@1?Cdl0CTi>05YJFP1ZJVX0B4Q8
05eMG@1SHf<0B4Q807MgM`1mOGd0=CDe07EeM@1WIfL0@T9206QXJ01/K6`0ATI605YJFP1kNg/0EEEE
06ATI01PH600LW9b05IFEP1BDU80LG5a05]KF`1GEeL0Ti>C05MGE`1CDe<0VIVI05IFEP1SHf<0OWin
04e=C@1WIfL0S8b<05MGE`1dM7@0RXZ:05iNGP1MGEd0T92@05EEE@1^KVh0SHf=05MGE`1lO7`0QhN7
0080IfMW00/0R8R8059BDP1eMGD0U9BD05]KF`24Q8@0O7al05UIF@1_Kfl0G5aL06MWI`0205iNGP08
07YjNP27QhL0HF5Q0861P@25QHD0JFUY08F5Q@1dM7@207inOP0U065QH@2PX:00MWIf06IVIP1JFUX0
LG5a07UiN@1OGel0H61P09ZJVP1hN7P0R8R808f=S@1cLg<0IfMW079bLP1cLg<0EeMG07MgM`1cLg<0
Hf=S0:NWY`1]KFd0KVi^0:>SX`1ADE40FEUI09NGU`1IFET0KFe]08b<S018B4P0Jf][09fMW@16ATH0
HV9R07YjNP0>ooooo`0605EEE@2l_;`0MgMg079bLP1[Jf/0H61P0P1^KVh0:P26QXH0IFEU04];B`18
B4P0DU9B04Y:BP1ADE40GUiN04M7A`1<C4`0F5QH04U9B@0g=cL0FEUI059BDP1@D500EUIF04A4A01:
BTX0HF5Q06QXJ019BDT0EUIF04a<C01@D500Kfm_07emO@1ZJVX0Ng]k06MWI`1nOWh0M7Ad09:BTP1j
NWX0G5aL06m_K`1EEED0IVIV05MGE`1/K6`0B4Q806ATI080IfMW03L0Kfm_069RHP1iNGT0Jf][08B4
Q021PH40UIFE07moO`1`L700KFe]05mOG`1iNGT0RHV908f=S@1JFUX0LG5a06IVIP1BDU80FEUI04m?
C`1JFUX0ADE507MgM`1;Bd/0EUIF04i>CP1HF5P0Bd];06=SH`1BDU80F5QH06EUI@1FEUH0EEEE05aL
G01SHf<0EUIF05=CD`1PH600GUiN05IFEP1IFET0HF5Q07YjNP1fMWH0CDe=06EUI@1RHV80OWin07Ui
N@1IFET0HV9R06UYJ@18B4P0H61P0080IfMW01l0EEEE06IVIP1]KFd0^[Zj08F5Q@2@T900Vi^K08j>
SP1^KVh0VIVI06m_K`29RHT0JVYZ06QXJ022PX80NGUi07MgM`1jNWX0Rh^;08R8R023Ph<0GemO07al
O01nOWh0F5QH06m_K`23Ph<0MgMg05QHF025QHD0YJFU00kooooo0440XJ6Q07alO01>CTh0AdM7055A
D@1PH600@41003Hf=P0S8b<0FUYJ04=3@`0S8b<0EEEE07AdM00[:b/0?clo03hn?P12@T80B4Q804E5
A@0m?Cd0?Shn04];B`1BDU80ADE505=CD`17AdL0@41006YZJP18B4P0?3`l05IFEP0i>CT0C4a<03Ti
>@1ADE40@4100492@P1jNWX0LG5a04a<C01IFET0?Shn04M7A`1gMgL0PH6105YJFP1GEeL0Fe]K03@d
=01?Cdl0KVi^05ADE01EEED0D51@04a<C01RHV80HF5Q06EUI@1RHV80B4Q805MGE`1YJFT0Ogmo051@
D00205mOG`0U05UIF@1RHV80LW9b04i>CP1KFe/0NGUi06QXJ00n?Sh0MWIf06EUI@1FEUH0MWIf04i>
CP1lO7`0BTY:06i^KP2^[Zh0MWIf065QH@1SHf<0G5aL04m?C`1RHV80BDU906IVIP20P800@D5109bL
W01CDe<0@T92079bLP0i>CT0J6QX065QH@1jNWX0CTi>06YZJP0205YJFP0?07EeM@1RHV80?clo06AT
I01CDe<0HF5Q09JFUP1GEeL0BTY:05ADE01UIFD0GUiN07=cL`0h>3P0I6AT0080NGUi0140GEeM06IV
IP1jNWX0@T9206=SH`1UIFD0<S8b05IFEP1`L700BTY:08>3P`0o?cl0QhN7079bLP1kNg/0EEEE06MW
I`0>ooooo`0N06UYJ@0/;2`07QhN020P801mOGd092@T024Q8@0[:b/0>c/k069RHP2;Rh/0;Bd]04U9
B@0m?Cd0D51@03Ti>@0f=SH0FEUI09:BTP1BDU809BDU02/[:`0X:2P0<S8b028R8P16ATH0QXJ607Mg
M`0];Bd0;2`/0P0F5QH03P0G5aL0NGUi08^;R`2RXZ80LW9b0492@P1>CTh0GUiN04e=C@0g=cL0QXJ6
069RHP0m?Cd0IVIV0P0R8R80900X:2P0DE5A09^KV`2LW9`0NWYj04i>CP24Q8@0FUYJ06i^KP2XZ:P0
J6QX02DU9@0_;bl0?Shn05QHF01ADE40MWIf09>CT`2][Jd0MWIf07YjNP1_Kfl0a/K60;Rh^02JVYX0
J6QX04i>CP0^;Rh0<S8b04I6AP1EEED0Y:BT08V9R@2RXZ80H61P02l_;`80OGem03H0IVIV08R8R02[
Zj/0K6a/06ATI02/[:`0VIVI0;2`/01jNWX0_[jn08^;R`2k^k/0Lg=c08F5Q@31`L40XZ:R0=3@d01V
IVH0ATI607alO01]KFd0DU9B07UiN@2h^;P0c/k>08Z:RP1OGel0Fe]K06a/K01PH600EEEE069RHP1Q
HF40EeMG08J6QP2:RXX0U9BD05eMG@23Ph<0BTY:02TY:@17AdL0FUYJ09VIV@1iNGT0<30`05YJFP2T
Y:@0RXZ:08R8R02OWil0GemO03Xj>P0i>CTDooooo`004oooool0DP21PH40N7Qh08N7Q`1VIVH0KFe]
04Y:BP1YJFT0FUYJ05eMG@1HF5P0MGEe06=SH`1dM7@0P82008J6QP1RHV80MgMg05YJFP1KFe/0H61P
069RHP1LG5`0HV9R06a/K01HF5P0Fe]K07AdM015ADD0DU9B05QHF01=CDd0EeMG05YJFP1TI6@0D51@
059BDP1WIfL0?clo07=cL`1QHF40BDU906][J`1]KFd0D51@05]KF`1]KFd0B4Q806EUI@1[Jf/0J6QX
05aLG01TI6@0J6QX05QHF01RHV80M7Ad05QHF01[Jf/0OGem05ADE01eMGD0KFe]05MGE`1ZJVX0Jf][
06IVIP1UIFD0LW9b075aL@1QHF40RHV906a/K01MGEd0Ph>306MWI`1TI6@0RHV908B4Q01cLg<0RXZ:
07EeM@1iNGT208V9R@0Q06a/K02;Rh/0SXj>06m_K`1PH600UYJF05QHF01_Kfl0N7Qh06EUI@24Q8@0
R8R808J6QP1kNg/0H61P0861P@1aLG40LW9b07AdM027QhL0NWYj07EeM@1VIVH0MGEe08:2PP1EEED0
K6a/08n?S`15ADD0TI6A07YjNP1UIFD0R8R80080O7al01@0IVIV07MgM`2;Rh/0IFEU08b<S01aLG40
Kfm_08^;R`1hN7P0Bd];07QhN01_Kfl0Hf=S07IfMP1[Jf/0HV9R06][J`1QHF40GEeM061PH0kooooo
01d0EUIF0:ZZZP23Ph<0KVi^061PH01XJ6P0KFe]06YZJP20P800HV9R0451@@16ATH0CDe=03De=@17
AdL0EEEE05mOG`1DE5@0GemO04a<C00e=CD0FUYJ055AD@1IFET0IFEU04Q8B01IFET0IfMW06m_K`02
05YJFP0`051@D01;Bd/0MGEe07EeM@1ZJVX0OWin05QHF022PX80Ng]k08B4Q01YJFT0D51@05IFEP1;
Bd/0EeMG05=CD`1OGel0@T9204m?C`1JFUX0E5AD05YJFP1MGEd0Lg=c05IFEP1RHV80J6QX071`L01]
KFd0F5QH05EEE@1QHF40Lg=c07IfMP29RHT0DE5A06e]K@1NGUh0GEeM069RHP1eMGD0GUiN05YJFP2F
UYH0HF5Q05UIF@1?Cdl0Hf=S0P1bLW8205iNGP0C075aL@1HF5P0ATI606QXJ01hN7P0IFEU059BDP1@
D500Fe]K05iNGP1FEUH0NWYj07IfMP1aLG40HV9R06YZJP1jNWX0MWIf08j>SP0206EUI@0C079bLP1D
E5@0JVYZ079bLP1fMWH0GemO06][J`1bLW80UIFE07emO@1lO7`0UIFE079bLP1RHV80NGUi07alO01g
MgL0G5aL06QXJ00207inOP0?07YjNP2KVi/0Ph>3096AT@1_Kfl0I6AT08N7Q`1dM7@0KFe]06i^KP1c
Lg<0MGEe05QHF028R8P0WYjN00kooooo01h0RXZ:06=SH`1SHf<0Cdm?05=CD`14A4@0ATI602HV9P0W
9bL0DU9B03Xj>P0i>CT0B4Q806QXJ00k>c/0;bl_04I6AP0[:b/0K6a/03hn?P0o?cl0C4a<04e=C@16
ATH0<S8b075aL@1;Bd/0@T9206QXJ016ATH204e=C@1/04A4A01GEeL0>CTi06ATI01:BTX0>3Ph07Mg
M`1DE5@0EeMG04A4A00j>SX0ATI6065QH@1mOGd0Jf][051@D01IFET0?clo04=3@`1dM7@0De=C04U9
B@1MGEd0AdM704m?C`1LG5`0De=C06ATI01LG5`0@D51065QH@1^KVh0F5QH07UiN@1KFe/0FUYJ07al
O020P800E5AD06m_K`1`L700HF5Q06UYJ@25QHD0L71`06m_K`29RHT0HF5Q06UYJ@1DE5@0NWYj0:b/
[01YJFT0K6a/069RHP1aLG40JVYZ05aLG01CDe<0Bd];07IfMP1>CTh0TY:B05=CD`15ADD0Lg=c03lo
?`1MGEd0EEEE06a/K019BDT0LG5a061PH01GEeL0NGUi05IFEP11@D40MGEe05YJFP1QHF40R8R8065Q
H@1BDU80CTi>05]KF`1HF5P0I6AT03De=@1aLG40JVYZ07EeM@1EEED0Kfm_0861P@0e=CD0IVIV06i^
KP0g=cL0H61P06MWI`1EEED0QhN703Hf=P1_Kfl0Fe]K07inOP1>CTh0Bd];3_ooool02P1iNGT0BDU9
04=3@`1PH600XJ6Q06UYJ@0M7Ad06ATI02DU9@1FEUH209RHV00B09jNWP1YJFT0?clo02<S8`0Q8B40
:b/[04=3@`24Q8@0AdM702/[:`0];Bd0ATI60410@01LG5`0YZJV08j>SP1WIfL0A4A40P0[:b/0>P0k
>c/0B4Q808b<S02CTi<0NWYj04m?C`0c<c<08R8R01hN7P0`<300IFEU06m_K`1nOWh0SHf=065QH@17
AdL0D51@06YZJP2YZJT0[Zj^07]kN`1IFET0QHF507QhN01lO7`0ZZZZ09NGU`0`<30071`L01hN7P0^
;Rh0?Shn075aL@2LW9`0MgMg06e]K@1CDe<0RHV90:f][@2KVi/0Hf=S06e]K@12@T80?Cdm03Xj>P0n
?Sh0WYjN07IfMP2YZJT0O7al07EeM@2RXZ80T92@055AD@0e=CD0>c/k069RHP21PH4206i^KP0`06EU
I@1eMGD0OGem0;fm_@2WYjL0M7Ad03Ti>@1<C4`0/;2`0=gMg@2^[Zh0QhN705UIF@1GEeL0Fe]K08V9
R@2NWYh0Y:BT0:RXZ01^KVh0CTi>04=3@`1LG5`0>c/k03De=@1@D500Lg=c079bLP1jNWX0];Bd09VI
V@1OGel0NGUi08>3P`16ATH0MGEe0<30`03EeMD0][Jf08V9R@2IVIT0SXj>07EeM@21PH40CTi>07=c
L`10@400>3Ph5?ooool001?ooooo01l0Ogmo071`L01SHf<0M7Ad04E5A@1HF5P0HV9R03@d=01YJFT0
Bd];05ADE028R8P0>c/k06QXJ01`L700@T9206YZJP1IFET0CDe=075aL@1SHf<0DE5A08N7Q`1jNWX0
FEUI05=CD`1WIfL0AdM706MWI`18B4P0FEUI0080C4a<02h0GemO05IFEP1/K6`0RXZ:03/k>`1RHV80
WinO04a<C01GEeL0SXj>03/k>`1EEED0Vi^K04M7A`1JFUX0Fe]K051@D01IFET0IVIV05QHF01PH600
IfMW05QHF01DE5@0NWYj08^;R`1GEeL0L71`061PH01aLG40NGUi065QH@1hN7P0K6a/07UiN@26QXH0
DU9B06EUI@1eMGD0?Shn07=cL`1lO7`0<c<c07emO@2FUYH0AdM70P2;Rh/01`1XJ6P0WinO07AdM01Q
HF40VYZJ08F5Q@1JFUX00P1oOgl04@1FEUH0OWin07MgM`1SHf<0PH6107EeM@2;Rh/0M7Ad06UYJ@2C
Ti<0L71`08>3P`1VIVH0RXZ:08f=S@1WIfL0LW9b0080P82001h0KFe]06IVIP1jNWX0N7Qh08>3P`1U
IFD0QhN707UiN@1gMgL0UIFE04=3@`1VIVH0ZZZZ04a<C01fMWH0Rh^;04a<C02BTY80PX:203`l?02E
UID0O7al04I6AP2JVYX0Ng]k055AD@20P800N7Qh04A4A01VIVH>ooooo`1Z05QHF02MWId0N7Qh06a/
K01OGel0K6a/065QH@1aLG40Jf][06EUI@19BDT0CDe=04Q8B013@d<0Bd];059BDP1JFUX0HF5Q06a/
K01>CTh0<S8b065QH@1IFET0C4a<071`L018B4P0H61P06YZJP1UIFD0FUYJ06MWI`1<C4`0B4Q8069R
HP28R8P0MGEe09:BTP1[Jf/0HV9R07YjNP2CTi<0S8b<05aLG01]KFd0EEEE06ATI01DE5@0HF5Q03Ph
>01CDe<0HF5Q05UIF@1VIVH0EeMG05aLG01SHf<0IVIV05IFEP1CDe<0D51@05EEE@1?Cdl0H61P06m_
K`1lO7`0[Zj^03lo?`1ADE40BDU903dm?@1MGEd0DU9B05UIF@1@D500KVi^05YJFP1HF5P0CDe=03De
=@1PH600KFe]05]KF`1]KFd0Hf=S05ADE01HF5P0GEeM06m_K`1PH600HV9R06UYJ@1VIVH0Kfm_05=C
D`23Ph<0OWin09:BTP1RHV80T92@08:2PP25QHD0Lg=c051@D01SHf<0OWin069RHP80KFe]0200IfMW
05YJFP1NGUh0HV9R08J6QP1ZJVX0NGUi09jNWP1dM7@0MgMg07moO`1jNWX0Q8B406MWI`1fMWH0XJ6Q
07MgM`2:RXX0S8b<06][J`25QHD0TY:B08F5Q@2MWId0K6a/05ADE01YJFT0JVYZ05mOG`1LG5`0RHV9
08B4Q0kooooo05X0Ng]k07=cL`1JFUX0ADE50451@@0i>CT0=CDe03/k>`0Z:RX0?Cdm04E5A@0W9bL0
@d=307]kN`0c<c<0C4a<02/[:`0g=cL0MGEe04I6AP14A4@0?Shn061PH00n?Sh0E5AD06QXJ01@D500
GUiN04m?C`1ADE40Bd];0492@P0l?3`0CTi>03Lg=`1ZJVX0GemO03lo?`2?Shl0Jf][05YJFP11@D40
DU9B0410@01>CTh0QHF5079bLP1>CTh0DE5A0410@017AdL0J6QX05EEE@1?Cdl0Fe]K04m?C`1ADE40
JVYZ051@D01ZJVX0DU9B04m?C`1OGel0IFEU079bLP1dM7@0J6QX06=SH`28R8P0O7al06a/K01hN7P0
RHV9075aL@24Q8@0T92@08R8R02?Shl0QXJ605ADE01mOGd0KVi^06=SH`2:RXX0FUYJ07=cL`1kNg/0
DU9B05IFEP1/K6`204m?C`0;0861P@1:BTX0Ogmo05ADE010@400Q8B404Y:BP1SHf<0?Shn065QH@1>
CTh00P1QHF405P1FEUH0FUYJ06m_K`17AdL0LW9b055AD@1MGEd0V9RH04a<C01CDe<0Bd];0492@P1=
CDd0K6a/03@d=01TI6@0L71`07QhN01@D500MgMg08B4Q00n?Sh207=cL`0;02l_;`1WIfL0KFe]061P
H01nOWh0=3@d05EEE@1MGEd0KVi^069RHP17AdL03_ooool0S036a/H0NGUi06UYJ@1UIFD0>c/k06IV
IP1bLW80@D5103Hf=P0e=CD0S8b<05IFEP2PX:00@D5102l_;`0W9bL0820P024Q8@0O7al0=SHf08n?
S`2<S8`0K6a/071`L01HF5P0?Shn07emO@21PH40a<C40:>SX`1PH600D51@03dm?@0Q8B40De=C07Ui
N@1cLg<0>c/k04e=C@1CDe<0Cdm?08B4Q00o?cl0D51@07QhN02l_;`0C4a<04E5A@1XJ6P0C4a<079b
LP1mOGd0PX:205]KF`1>CTh0GemO04A4A01^KVh0J6QX071`L00Y:BT07QhN01lO7`16ATH0WYjN0:>S
X`2k^k/0XJ6Q08n?S`2FUYH0LW9b05aLG015ADD0KFe]07emO@1GEeL0H61P09NGU`1ZJVX0OWin08n?
S`1^KVh0=SHf08V9R@22PX80:BTY01lO7`0b<S80SHf=0::RXP2h^;P0Ph>306m_K`2=SHd0ZJVY0;Zj
^P2i^KT0Kfm_04Q8B01OGel0Z:RX0;^k^`1_Kfl0Jf][01hN7P0P820092@T08V9R@1BDU80O7al08R8
R00g=cL0HF5Q06UYJ@1KFe/0?3`l03dm?@1;Bd/0EeMG07UiN@2ZZZX0YJFU07UiN@2`/;00K6a/09VI
V@3De=@0alO70<71`@3GemL0T92@08b<S02[Zj/0N7Qh08V9R@2HV9P0CTi>08F5Q@2@T900WIfM5?oo
ool001?ooooo0540Shn?07EeM@1lO7`0CDe=069RHP0k>c/0FUYJ06e]K@13@d<0DU9B06][J`1=CDd0
S8b<08:2PP1JFUX0QHF506MWI`0f=SH0LW9b06IVIP0o?cl0Jf][04M7A`1LG5`0MgMg04Q8B01cLg<0
IfMW04];B`1NGUh0IFEU04Y:BP2<S8`0J6QX0410@01/K6`0Jf][03Ti>@1hN7P0IfMW051@D01`L700
IFEU05IFEP1OGel0D51@065QH@1PH600DU9B08R8R01JFUX0EEEE08Z:RP1OGel0IFEU09:BTP10@400
G5aL09NGU`18B4P0JVYZ08n?S`1?Cdl0KVi^092@T018B4P0PH6107]kN`1FEUH0NWYj0:6QX@1EEED0
KFe]08F5Q@1EEED0KFe]08V9R@1ADE40PH6107YjNP1MGEd00P2<S8`0901[Jf/0QhN706i^KP1gMgL0
QXJ604m?C`26QXH0IfMW051@D02<S8`0PX:206i^KP28R8P0Ti>C07MgM`1/K6`0LG5a09RHV01oOgl0
K6a/0:JVYP1ZJVX0Kfm_06e]K@1NGUh0Ti>C06i^KP1JFUX0S8b<05]KF`1TI6@0UiNG06e]K@1WIfL0
QhN706e]K@80JVYZ00X0Jf][07]kN`2NWYh0JFUY0820P02;Rh/0Fe]K06m_K`2GUiL0EUIF0P25QHD0
1P1?Cdl0Fe]K09bLW00n?Sh0JVYZ05UIF@kooooo00l0J6QX0:RXZ01kNg/0G5aL05]KF`1RHV80I6AT
06IVIP1^KVh0DE5A05aLG01?Cdl0E5AD0492@P1DE5@00P1;Bd/01@1ZJVX0KFe]051@D00n?Sh0HV9R
0080C4a<00X0DU9B04m?C`1CDe<0H61P05YJFP1IFET0CTi>04m?C`16ATH0GemO0P1YJFT09P20P800
X:2P08N7Q`1PH600MgMg06YZJP1?Cdl0GEeM05EEE@1MGEd0CTi>06=SH`10@400Bd];05ADE019BDT0
Cdm?059BDP1iNGT0LW9b06EUI@1CDe<0MWIf0820P01WIfL0H61P05YJFP1[Jf/0QhN709BDU01CDe<0
MgMg06MWI`1]KFd0GEeM051@D017AdL0B4Q80P1ADE40>@1;Bd/0Cdm?0451@@1?Cdl0Fe]K04A4A01K
Fe/0FUYJ04i>CP1;Bd/0GemO05]KF`1NGUh0FEUI06IVIP1WIfL0Fe]K05EEE@1/K6`0QhN706MWI`1E
EED0IVIV075aL@1eMGD0MgMg03hn?P1EEED0FEUI05]KF`1OGel0Hf=S05aLG01GEeL0IVIV079bLP2<
S8`0Jf][07alO02=SHd0JFUY06IVIP2BTY80LG5a07inOP1TI6@0GemO07]kN`1dM7@0HV9R06YZJP1d
M7@0MgMg05MGE`1NGUh0PX:205aLG00205QHF005079bLP1fMWH0Fe]K08n?S`2`/;003_ooool03`1i
NGT0XJ6Q05QHF014A4@0EEEE03Ti>@0k>c/0=3@d030`<00m?Cd0<c<c03Ph>00k>c/0KVi^04U9B@02
04I6AP0X02h^;P1YJFT0?Cdm03Lg=`13@d<0EeMG02l_;`1<C4`0D51@05=CD`1@D500CDe=05UIF@18
B4P0?clo04M7A`1GEeL0HV9R06ATI01KFe/0CDe=09:BTP1YJFT0DU9B03lo?`12@T80>c/k06ATI02C
Ti<0KVi^05EEE@1NGUh0Hf=S05]KF`1bLW80JFUY06=SH`1VIVH0IFEU04a<C080H61P01/0IFEU061P
H017AdL0Jf][06IVIP1_Kfl0O7al06EUI@1QHF40K6a/07=cL`1RHV80Q8B407AdM01QHF40JVYZ07]k
N`1eMGD0Q8B408>3P`1RHV80LW9b05mOG`1XJ6P0VIVI069RHP1]KFd00P1ADE40=01^KVh0IFEU05MG
E`10@400JFUY05]KF`1NGUh0CDe=05ADE025QHD0D51@069RHP1UIFD0N7Qh055AD@1^KVh0MWIf04m?
C`1oOgl0IVIV03Ph>01fMWH0GUiN06YZJP2FUYH0IfMW05mOG`1EEED0AdM705mOG`1dM7@0>c/k07Mg
M`1/K6`0QHF503lo?`1nOWh0WYjN04i>CP1QHF40IFEU04U9B@1IFET0Ng]k071`L01jNWX0?clo05YJ
FP1iNGT0HV9R05]KF`18B4P>ooooo`170<W9b@25QHD0E5AD059BDP1MGEd08b<S0:j^[P29RHT0Ogmo
08R8R02QXJ40Z:RX09FEU@2FUYH0@T9202PX:00N7Qh07AdM01lO7`0Q8B40Q8B408V9R@1_Kfl0W9bL
0::RXP2>SXh0Ti>C09nOW`2:RXX0a<C40:b/[01EEED0:RXZ0451@@1;Bd/0?Cdm04M7A`1IFET0>3Ph
02/[:`0d=3@0F5QH07alO01HF5P0EUIF08F5Q@0Y:BT0:2PX03/k>`2WYjL0BDU9051@D01:BTX0RHV9
02XZ:P1NGUh0>c/k05EEE@0_;bl0M7Ad04I6AP0V9RH07alO0451@@1ZJVX0LW9b09BDU01XJ6P0De=C
07QhN025QHD00P1@D5007`1TI6@0=CDe02<S8`0e=CD0LW9b06=SH`1TI6@0]kNg0:ZZZP2;Rh/0^[Zj
05QHF010@4009BDU03De=@1QHF40P8200:6QX@1TI6@0U9BD0<[:bP2RXZ80alO70::RXP1bLW80?Shn
01lO7`1UIFD0UYJF09NGU`1CDe<00P0M7Ad01@0U9BD0Rh^;07=cL`23Ph<0Rh^;00808B4Q01/0RHV9
08B4Q01UIFD0=3@d05EEE@0g=cL0DU9B07EeM@1gMgL0clo?09jNWP2^[Zh0]kNg0;fm_@2l_;`0WYjN
0;2`/02IVIT0BTY:05=CD`1TI6@0BTY:0:ZZZP1`L700=SHf0492@P2<S8`05?ooool001?ooooo00@0
O7al07moO`1XJ6P0Q8B40P1BDU80J`1TI6@0@410055AD@11@D40AdM706YZJP0i>CT0Fe]K06a/K00k
>c/0LG5a061PH01<C4`0M7Ad07=cL`19BDT0GemO08n?S`15ADD0IfMW08>3P`0m?Cd0G5aL04i>CP15
ADD0DU9B05iNGP1CDe<0Ng]k05ADE01FEUH0FUYJ06IVIP1RHV80IfMW05UIF@2=SHd0CDe=05mOG`2=
SHd0B4Q8065QH@1VIVH0FUYJ05ADE01SHf<0GEeM05MGE`1]KFd0HF5Q05MGE`1oOgl0PX:206UYJ@2D
U9@0Kfm_06MWI`2?Shl0IfMW07EeM@24Q8@0J6QX08^;R`1DE5@0K6a/08B4Q017AdL0IVIV07]kN`14
A4@0QhN708Z:RP17AdL0MGEe08>3P`1OGel0WIfM0:^[Z`1NGUh0RXZ:092@T01DE5@0NWYj0820P019
BDT0M7Ad06MWI`1QHF40VYZJ05YJFP26QXH0Vi^K05mOG`2EUID0Kfm_08B4Q01gMgL0NWYj0820P01a
LG40H61P08>3P`1fMWH0Fe]K06e]K@1/K6`0O7al0080IfMW01P0Rh^;069RHP1gMgL0Zj^[05IFEP1W
IfL0ZZZZ05YJFP1cLg<0SXj>05MGE`1dM7@0Rh^;04=3@`2IVIT0QHF5030`<02SXj<0Ng]k0451@@1b
LW80KVi^04];B`1WIfL>ooooo`0M05YJFP3GemL0TI6A07YjNP1hN7P0GemO06IVIP1/K6`0SHf=06UY
J@1BDU80Cdm?05UIF@1>CTh0GemO05MGE`1KFe/0Lg=c07MgM`1CDe<0?Cdm06UYJ@1JFUX0ATI606=S
H`1RHV80Cdm?04A4A01=CDd00P1JFUX0C01>CTh0B4Q8051@D01_Kfl0JVYZ07EeM@1=CDd0ADE503dm
?@1mOGd0NGUi065QH@1NGUh0Cdm?05UIF@1@D500JVYZ04U9B@1:BTX0Cdm?0451@@16ATH0GUiN06m_
K`1XJ6P0JVYZ05MGE`1PH600EUIF0492@P1BDU80Cdm?06a/K01bLW80Ng]k04E5A@1LG5`0D51@04m?
C`1;Bd/0D51@04Q8B01CDe<0Lg=c05UIF@1UIFD0EEEE069RHP1TI6@0KFe]065QH@1]KFd0IFEU04E5
A@0j>SX0Cdm?06e]K@1QHF40CTi>05mOG`1IFET0G5aL05mOG`1KFe/0Ng]k059BDP1VIVH0JFUY07in
OP20P800SXj>05QHF01WIfL0TY:B05QHF01UIFD206e]K@0O059BDP1MGEd0NGUi09^KV`1oOgl0RXZ:
0820P01XJ6P0MGEe07moO`1]KFd0RHV904a<C01=CDd0PX:207AdM01VIVH0JVYZ07emO@2;Rh/0OGem
07UiN@2@T900DE5A04I6AP1/K6`0LW9b06IVIP1WIfL0P82008n?S`0>ooooo`0404a<C02LW9`0GemO
04e=C@80CTi>08H0@d=30410@00f=SH0<S8b02d];@0Z:RX0=SHf07AdM00h>3P0ATI603/k>`0e=CD0
KFe]055AD@0d=3@0CDe=06QXJ00^;Rh0ATI604=3@`1BDU80IFEU04I6AP1OGel0M7Ad034a<@1KFe/0
B4Q806ATI01[Jf/0IFEU04M7A`1bLW80KFe]0492@P0j>SX0Fe]K04A4A016ATH0RHV906ATI01:BTX0
GEeM04M7A`1BDU80Jf][05QHF01GEeL0GEeM05aLG014A4@0GemO05YJFP1SHf<0E5AD04M7A`1LG5`0
BDU907EeM@1bLW80EEEE071`L01cLg<0IVIV05eMG@1oOgl0M7Ad06m_K`1dM7@0OGem08F5Q@2<S8`0
SHf=05MGE`21PH40O7al06ATI02=SHd0GemO07moO`1JFUX0D51@06QXJ01hN7P0Cdm?03/k>`1hN7P0
Jf][06=SH`1HF5P0E5AD08V9R@1GEeL0NWYj04=3@`1VIVH0DE5A061PH01[Jf/0EeMG06YZJP1fMWH0
@D5106][J`1>CTh0IfMW09>CT`1EEED0E5AD05eMG@15ADD0CTi>06YZJP16ATH0I6AT07MgM`2<S8`0
ATI6092@T02BTY80HV9R06e]K@1YJFT0G5aL06MWI`1[Jf/0GemO08f=S@10@400Fe]K05EEE@27QhL0
DU9B03Ph>0kooooo04h0ZZZZ08N7Q`19BDT0Cdm?06ATI01<C4`0IFEU05mOG`1:BTX0K6a/07IfMP2I
VIT0WIfM07alO01>CTh0B4Q804M7A`0_;bl0:b/[04I6AP1jNWX0ZZZZ07MgM`2IVIT0R8R804i>CP0Z
:RX09RHV061PH01lO7`0];Bd0:^[Z`1JFUX09bLW01/K6`0Z:RX0=cLg02LW9`1?Cdl0B4Q802h^;P1E
EED0U9BD0861P@1iNGT0VIVI04e=C@0W9bL0>CTi06m_K`1SHf<0?Cdm04A4A01LG5`0;Rh^04Y:BP23
Ph<0Ti>C06i^KP14A4@0Kfm_03Lg=`0M7Ad0CDe=0:JVYP1bLW80I6AT038b<P0[:b/0:2PX06a/K01R
HV80RHV90:VYZ@1UIFD0>CTi04Y:BP1`L70205IFEP0F07]kN`1<C4`0CDe=065QH@1dM7@0IFEU04A4
A010@400;bl_05iNGP1ZJVX0WYjN06m_K`3AdM40a<C40:BTY01[Jf/0>CTi04e=C@1?Cdl0Lg=c0861
P@80LG5a0180:RXZ01dM7@0j>SX0MGEe07alO01[Jf/0W9bL05YJFP1[Jf/0Ti>C03`l?01FEUH0TI6A
06][J`0Y:BT0?Shn06UYJ@1fMWH20<30`00=0<;2`P2c/k<0^[Zj0<30`03Jf]X0GEeM02/[:`1hN7P0
Fe]K06=SH`2:RXX0QHF509bLW00208j>SP030:FUYOoooooooooo01;ooooo000Cooooo`0b08b<S01a
LG40U9BD06ATI01EEED0EeMG05UIF@1<C4`0IVIV04Y:BP1WIfL0H61P05ADE021PH40Fe]K05=CD`1Y
JFT0A4A4061PH01SHf<0D51@05iNGP1JFUX0IfMW069RHP1>CTh0M7Ad04];B`1=CDd0Fe]K05EEE@0l
?3`0MWIf07UiN@1;Bd/0IfMW07QhN00n?Sh0Kfm_07EeM@0g=cL0IFEU07AdM015ADD0Fe]K05aLG01:
BTX0IVIV05eMG@1QHF42069RHP1G07UiN@1;Bd/0G5aL08V9R@1RHV80H61P08N7Q`1EEED0QHF5075a
L@1OGel0O7al06i^KP1MGEd0O7al07YjNP1oOgl0HF5Q08J6QP1_Kfl0G5aL08b<S01WIfL0EEEE08f=
S@1_Kfl0FUYJ08F5Q@1UIFD0MGEe08R8R01cLg<0Ng]k07YjNP1iNGT0MgMg05iNGP21PH40HF5Q06YZ
JP1^KVh0KFe]079bLP1jNWX0UYJF06YZJP1TI6@0SXj>08J6QP23Ph<0MgMg0::RXP1VIVH0Fe]K07Ui
N@1QHF40Ng]k075aL@1GEeL0Q8B4069RHP19BDT0MgMg08j>SP1UIFD0Ogmo08J6QP1NGUh0IfMW07in
OP1[Jf/0UiNG06YZJP1HF5P0UIFE06e]K@1DE5@0VYZJ075aL@1?Cdl0Ph>3071`L01NGUh0MgMg05IF
EP1LG5`0HF5Q00kooooo08@0D51@09VIV@28R8P0IVIV05]KF`1IFET0J6QX07UiN@2>SXh0HV9R04A4
A01>CTh0DE5A0451@@1ADE40EUIF06=SH`1WIfL0Kfm_06=SH`11@D40K6a/05YJFP11@D40J6QX04a<
C01BDU80EUIF065QH@1JFUX0EeMG05QHF01:BTX0Fe]K06i^KP1XJ6P0QhN706e]K@1GEeL0EUIF08b<
S01jNWX0F5QH06=SH`1JFUX0IFEU05]KF`1UIFD0DU9B05]KF`1LG5`0E5AD059BDP1PH600MgMg06IV
IP22PX80JVYZ06QXJ01MGEd0J6QX069RHP1EEED0OGem08J6QP2>SXh0GUiN05YJFP1OGel0Fe]K05YJ
FP1EEED0AdM705eMG@1^KVh0GEeM05YJFP1EEED0D51@08Z:RP1fMWH0EeMG06YZJP1JFUX0A4A403dm
?@1XJ6P0JFUY05mOG`1WIfL0IVIV05QHF01VIVH0GEeM06a/K01^KVh0HF5Q06IVIP1UIFD0KVi^07in
OP1hN7P0CDe=05eMG@1WIfL0DU9B06a/K01[Jf/0GemO055AD@1GEeL0JFUY08f=S@27QhL0OWin08^;
R`1YJFT0Jf][07QhN01KFe/0KVi^059BDP1EEED0K6a/06m_K`1eMGD0Jf][07EeM@24Q8@0KFe]07If
MP1gMgL205QHF006069RHP1_Kfl0GemO061PH01hN7P0TY:B3_ooool0@01?Cdl0OWin05UIF@13@d<0
E5AD04=3@`19BDT0BTY:03De=@0Y:BT0:2PX02/[:`0d=3@0EeMG05UIF@0k>c/0?3`l03De=@1SHf<0
ATI603`l?015ADD0F5QH034a<@1=CDd0=CDe04I6AP1;Bd/0>CTi055AD@1IFET09RHV04e=C@16ATH0
Jf][05=CD`1BDU80GEeM061PH01_Kfl0ADE50451@@1FEUH0?Shn059BDP2DU9@0FUYJ05=CD`1:BTX0
Fe]K04e=C@1RHV80J6QX06YZJP1NGUh0IVIV04Q8B01QHF40JVYZ06UYJ@1UIFD0D51@05QHF01EEED2
07MgM`1:05mOG`1fMWH0Kfm_06UYJ@1aLG40GemO07AdM01WIfL0MWIf07]kN`1jNWX0N7Qh07inOP1R
HV80JFUY071`L01[Jf/0SXj>05YJFP1`L700CDe=05YJFP1cLg<0GemO05YJFP0j>SX0PX:206UYJ@1?
Cdl0@4100492@P1dM7@0B4Q8069RHP1:BTX0LG5a05ADE01FEUH0NGUi04i>CP1QHF40Lg=c034a<@1B
DU80EUIF07IfMP2DU9@0Fe]K05MGE`1HF5P0@41004i>CP1ZJVX0BTY:05iNGP1OGel0SXj>03@d=01_
Kfl0PX:204I6AP1EEED0G5aL04I6AP1CDe<0I6AT055AD@1iNGT0?Cdm04a<C00o?cl0SXj>05UIF@0m
?Cd>ooooo`0605IFEP1MGEd0Jf][06m_K`1ZJVX0I6AT0P0n?Sh04P11@D40De=C06i^KP2f][H0WIfM
05QHF01KFe/0JVYZ06ATI01]KFd0MgMg07emO@2GUiL0X:2P08V9R@2:RXX0:2PX01LG5`<061PH00@0
I6AT0:f][@2][Jd071`L0P0G5aL0FP0H61P08b<S034a<@0T92@0<c<c05EEE@1QHF40RHV907]kN`1B
DU80LW9b051@D01NGUh0VYZJ05ADE014A4@0NWYj055AD@0n?Sh0C4a<05IFEP1VIVH0KVi^04m?C`1T
I6@0Ng]k092@T02ATI40/k>c0;Bd]01jNWX0DU9B05iNGP1?Cdl0=CDe02XZ:P27QhL0_Kfm09:BTP1V
IVH0KFe]08>3P`22PX80IfMW06QXJ00Z:RX0820P03/k>`1:BTX0PX:206UYJ@1@D500AdM704i>CP21
PH40LG5a06a/K02ATI40X:2P06YZJP29RHT0GEeM03Ti>@0g=cL0J6QX0<C4a02IVIT0Xj>S0:2PX01N
GUh0>c/k07EeM@1LG5`0SXj>0;Zj^P2@T900F5QH055AD@1IFET0MGEe08n?S`1RHV80OWin079bLP2D
U9@0WinO08b<S010@400];Bd055AD@80KFe]00<0QXJ608f=S@2=SHd00P0U9BD0202/[:`0OWin06YZ
JP1>CTh0[Jf]05EEE@1?Cdl0M7Ad5?ooool001?ooooo00X0RHV906i^KP27QhL0Cdm?07=cL`0`<300
E5AD071`L01?Cdl0I6AT0P1SHf<0@P1kNg/0I6AT07emO@1OGel0EUIF05mOG`1`L700E5AD05iNGP1`
L700@41007moO`1/K6`0B4Q808j>SP1EEED0CDe=09^KV`17AdL0ADE505YJFP1@D500GemO065QH@1X
J6P0JVYZ06EUI@1eMGD0Fe]K05eMG@1eMGD0H61P05QHF01]KFd0I6AT04a<C01SHf<0O7al05EEE@1S
Hf<0U9BD03dm?@1hN7P0Ti>C0492@P1aLG40/;2`03`l?01cLg<0UYJF0451@@1fMWH0SXj>04];B`2<
S8`0NWYj05IFEP1jNWX0Vi^K051@D01]KFd0Ogmo05ADE01]KFd0MGEe06][J`80Kfm_01P0K6a/09>C
T`28R8P0R8R808^;R`1XJ6P0R8R806m_K`1ADE40RHV9065QH@21PH40UIFE06ATI01oOgl0Q8B406YZ
JP2;Rh/0JFUY07UiN@2CTi<0I6AT05mOG`2WYjL206i^KP0Q06a/K01DE5@0Ti>C055AD@1TI6@0RXZ:
051@D01]KFd0NGUi0:VYZ@2PX:00NWYj06m_K`1eMGD0L71`06UYJ@1UIFD0SXj>05YJFP1SHf<0Kfm_
05UIF@1BDU80L71`06][J`1IFET0I6AT05EEE@1QHF40Jf][03dm?@1/K6`0IfMW00kooooo00H0DE5A
07UiN@1RHV80F5QH04U9B@1DE5@205eMG@1007alO01MGEd0@41003lo?`11@D40@T9203hn?P1=CDd0
GemO06MWI`1dM7@0JFUY04];B`1TI6@0GUiN05UIF@1]KFd0G5aL05ADE01FEUH0GUiN069RHP1ADE40
EEEE04m?C`1]KFd0QHF506EUI@2MWId0Ogmo079bLP1?Cdl0Rh^;07moO`1HF5P0KVi^05aLG01dM7@0
Hf=S07QhN01<C4`0EEEE061PH01>CTh0E5AD05UIF@1RHV80Hf=S05aLG01BDU80G5aL05ADE01kNg/0
EEEE05UIF@1_Kfl0PX:208>3P`1KFe/0OWin07AdM016ATH0Kfm_055AD@80CDe=0480JVYZ05IFEP1T
I6@0GEeM05QHF01OGel0I6AT05]KF`1SHf<0De=C051@D016ATH0G5aL06QXJ01KFe/0GUiN05aLG01J
FUX0F5QH04I6AP1hN7P0Ogmo04i>CP12@T80GUiN05YJFP1iNGT0Ph>304];B`1EEED0PX:205eMG@1Y
JFT0Jf][051@D01CDe<0JFUY05ADE02=SHd0PH6108J6QP1[Jf/0JVYZ07QhN01]KFd0HV9R0861P@1Y
JFT0N7Qh08^;R`1HF5P0IVIV06UYJ@26QXH0I6AT04a<C01OGel0SXj>04];B`1IFET0IfMW08B4Q01Z
JVX0IVIV0861P@2FUYH>ooooo`1P06QXJ01bLW80A4A4059BDP14A4@0BDU904e=C@13@d<0>3Ph01dM
7@0`<300<C4a02/[:`1=CDd0G5aL03<c<`0f=SH09bLW05]KF`1?Cdl0=CDe05MGE`18B4P0=SHf05UI
F@0[:b/0<30`04Q8B010@400EEEE04m?C`0W9bL0D51@04e=C@1^KVh0CDe=065QH@1_Kfl0F5QH07in
OP1@D500<c<c04Q8B00h>3P0FUYJ07emO@1FEUH0HF5Q04Y:BP1>CTh0D51@05mOG`1bLW80KFe]06QX
J01NGUh0A4A4061PH01aLG40KVi^06ATI01BDU80GUiN05eMG@1]KFd0KVi^05IFEP1aLG40HV9R07If
MP1mOGd0KVi^07]kN`1QHF40O7al07UiN@1JFUX0OGem08b<S01PH600GemO06m_K`1eMGD0JFUY06][
J`1_Kfl0BDU904m?C`1]KFd0H61P06ATI015ADD0O7al08J6QP1TI6@0CTi>0P1LG5`0401ADE40L71`
065QH@1gMgL0DU9B06IVIP1gMgL0Fe]K07emO@25QHD0<S8b055AD@1WIfL0HF5Q06EUI@1fMWH2059B
DP0:04e=C@1<C4`0GUiN05YJFP1GEeL0FEUI08j>SP11@D40PH6107MgM`80IFEU00`0GemO05eMG@1H
F5P0OGem051@D026QXH0Bd];05iNGP0m?Cd0T92@05IFEP0l?3`>ooooo`0T04U9B@10@400De=C0:NW
Y`2UYJD0I6AT03hn?P1oOgl0M7Ad0492@P0W9bL0CDe=0:f][@1lO7`0A4A4096AT@2LW9`0FEUI06a/
K01oOgl0LW9b075aL@0c<c<0=CDe05ADE00^;Rh05aLG01PH600d=3@0HF5Q09NGU`2h^;P0G5aL030`
<00];Bd0<c<c0P0[:b/02P1IFET0BTY:06MWI`1/K6`0GUiN07emO@1BDU80[jn_05iNGP0`<3020861
P@1J06i^KP2CTi<0VYZJ09ZJVP2FUYH0G5aL02d];@0d=3@0HF5Q06=SH`2CTi<0VYZJ0:b/[02h^;P0
^k^k09JFUP1FEUH0<c<c03Lg=`1DE5@0N7Qh0;Rh^02HV9P0JVYZ08f=S@1VIVH0HV9R08F5Q@2QXJ40
ATI601dM7@0O7al0=CDe05MGE`2OWil0JFUY02h^;P0P82007AdM03Xj>P2JVYX0R8R804a<C018B4P0
KFe]05mOG`21PH40BTY:06i^KP2`/;00Q8B407alO02DU9@0Ogmo071`L03=cLd0YZJV0:ZZZP2UYJD0
M7Ad0820P015ADD0F5QH05]KF`1jNWX0/[:b0;Fe]@1KFe/0TY:B0;^k^`37alL0>SXj09:BTP2f][H0
>CTi03`l?01KFe/0Q8B409NGU`1fMWH0Rh^;03Xj>P1jNWX0@d=309VIV@2[Zj/0[jn_05mOG`1dM7@0
KVi^5?ooool001?ooooo08/0S8b<07IfMP1nOWh0QhN705IFEP1CDe<0Lg=c03Hf=P1NGUh0D51@05AD
E01SHf<0@41006ATI01jNWX0?3`l06][J`1WIfL0D51@06e]K@1TI6@0E5AD07IfMP1VIVH0E5AD06MW
I`1nOWh0DU9B05eMG@1JFUX0G5aL0492@P1XJ6P0IVIV04e=C@1FEUH0Lg=c03De=@1SHf<0QhN703lo
?`1NGUh0TY:B03/k>`1/K6`0OGem04e=C@1PH600FUYJ05ADE01KFe/0L71`06a/K01`L700GemO06e]
K@1kNg/0GemO07moO`1iNGT0M7Ad07IfMP27QhL0JVYZ0820P02<S8`0Cdm?07UiN@2@T900>c/k08b<
S027QhL0?Cdm0861P@1hN7P0>SXj07]kN`1nOWh0@T920:FUY@20P800EeMG0:BTY02EUID0HF5Q0:NW
Y`2ATI40I6AT08V9R@1_Kfl0HV9R07alO01]KFd0QhN708b<S01cLg<0][Jf06YZJP1aLG40Vi^K06YZ
JP2IVIT0GUiN08:2PP2<S8`0NGUi08>3P`1YJFT0PH6108f=S@1MGEd0NGUi07moO`1LG5`0DU9B05mO
G`1mOGd0S8b<09FEU@18B4P0Hf=S09FEU@0m?Cd0KFe]08:2PP0o?cl0RXZ:07emO@17AdL0OGem06QX
J00j>SX0Lg=c08V9R@1?Cdl0MGEe071`L01@D500K6a/00kooooo0140D51@08:2PP2IVIT0K6a/055A
D@1UIFD0HF5Q061PH01OGel0@D5103dm?@0n?Sh0B4Q80492@P1?Cdl0De=C05QHF002071`L00W04U9
B@0j>SX0Hf=S04U9B@0m?Cd0@T9204a<C013@d<0E5AD05IFEP1EEED0CDe=04Q8B01:BTX0C4a<061P
H01JFUX0Jf][05ADE01/K6`0@T9208b<S01NGUh0AdM704i>CP16ATH0@D5104A4A01EEED0@41004A4
A015ADD0?3`l04A4A016ATH0FUYJ04];B`1TI6@0DE5A0080F5QH01@0K6a/065QH@1EEED0JVYZ08N7
Q`1HF5P0J6QX06][J`20P800SHf=0820P01bLW80H61P055AD@1nOWh0H61P05ADE01XJ6P0CTi>08^;
R`80GemO02l0MGEe06][J`1DE5@0EEEE05mOG`1gMgL0HV9R055AD@1?Cdl0HF5Q06EUI@1[Jf/0S8b<
06e]K@1SHf<0EeMG06YZJP1XJ6P0Ti>C08f=S@18B4P0J6QX06e]K@1eMGD0GEeM05mOG`1BDU80CTi>
05eMG@1JFUX0Q8B406a/K01aLG40P82005eMG@1/K6`0QHF506QXJ01`L700Bd];061PH01_Kfl0H61P
06MWI`1nOWh0Ti>C09:BTP0207QhN00908^;R`1@D500CDe=05aLG01eMGD0IVIV06e]K@27QhL0[Jf]
00kooooo08`0HF5Q06QXJ01;Bd/0?3`l03Ph>00i>CT0>c/k03hn?P0Y:BT051@D02l_;`0W9bL0=cLg
04Q8B01SHf<0>SXj038b<P0i>CT0Cdm?04];B`14A4@0ATI604m?C`0h>3P0B4Q8028R8P15ADD0@d=3
03<c<`1RHV80ATI602HV9P1OGel0@d=307YjNP1@D500CTi>08J6QP1SHf<0TI6A051@D00g=cL0KVi^
03<c<`1IFET0P82006a/K01JFUX0HV9R04I6AP1>CTh0DU9B05EEE@1bLW80Fe]K05EEE@18B4P0FEUI
05iNGP1IFET0G5aL04Q8B011@D40BDU9079bLP14A4@0D51@07UiN@1cLg<0De=C0861P@1FEUH0FEUI
07]kN`22PX80MWIf07UiN@1lO7`0MWIf09BDU01RHV80QhN707moO`1YJFT0UIFE06e]K@13@d<0GEeM
06a/K01SHf<0G5aL059BDP1YJFT0S8b<07=cL`1ADE40CDe=05UIF@1SHf<0KFe]03`l?01aLG40Fe]K
05IFEP1_Kfl0FEUI06][J`1lO7`0C4a<03`l?01FEUH0L71`06MWI`1dM7@0Fe]K069RHP0n?Sh0G5aL
06e]K@1VIVH0GEeM04e=C@2PX:00>3Ph06e]K@24Q8@0HV9R04U9B@17AdL0JFUY059BDP1ZJVX0Hf=S
09ZJVP11@D40Fe]K04Q8B01kNg/0Cdm?03lo?`kooooo00T0M7Ad030`<00d=3@0JVYZ065QH@0/;2`0
7alO02d];@0W9bL00P0R8R80601<C4`0A4A405eMG@2SXj<0U9BD0:FUY@21PH40Q8B4079bLP1nOWh0
V9RH04a<C00Z:RX0;Rh^04a<C01DE5@0=3@d02LW9`0^;Rh0Shn?0;Zj^P2UYJD0NWYj04A4A0809bLW
06L0<C4a05UIF@0`<300IVIV06][J`1FEUH0RXZ:07AdM01cLg<0HV9R05aLG02BTY80ADE502d];@1;
Bd/0WIfM0<G5a@2a/K40MgMg03Ph>00_;bl09bLW02h^;P2^[Zh0VIVI0;jn_P2RXZ80TY:B09BDU01L
G5`0EeMG05]KF`18B4P0SHf=09VIV@2QXJ40D51@055AD@1?Cdl0MGEe0;Fe]@2XZ:P0Fe]K02h^;P1F
EUH0IfMW04i>CP2;Rh/0A4A403hn?P0N7Qh06a/K030`<02m_Kd0/[:b02`/;00_;bl0UYJF07]kN`2R
XZ80WinO08Z:RP26QXH0=cLg02XZ:P16ATH0E5AD07EeM@23Ph<0TI6A0=3@d02=SHd0G5aL05MGE`1Y
JFT0Cdm?05iNGP2e]KD0[jn_0;6a/@1aLG40I6AT0:ZZZP2o_kl0W9bL07QhN01UIFD0GUiN07IfMP1l
O7`0>c/k07moO`21PH40OGem0:^[Z`1BDU80CDe=0:^[Z`2=SHd0WIfM03@d=01HF5P0EUIF01Cooooo
000Cooooo`1`08V9R@1mOGd0QXJ607inOP1/K6`0H61P071`L01ZJVX0FUYJ04i>CP1jNWX0GEeM05iN
GP2<S8`0G5aL079bLP2<S8`0DE5A065QH@1UIFD0G5aL06ATI01HF5P0OGem06YZJP1;Bd/0PX:205]K
F`1LG5`0I6AT05YJFP13@d<0EeMG06][J`14A4@0LW9b08F5Q@0n?Sh0KFe]0861P@17AdL0I6AT07]k
N`14A4@0JFUY06QXJ01DE5@0Hf=S061PH01]KFd0IfMW06IVIP1XJ6P0AdM706IVIP22PX80F5QH06MW
I`2UYJD0EeMG07AdM024Q8@0HV9R07moO`23Ph<0LG5a06m_K`21PH40J6QX06=SH`2ATI40GemO069R
HP2=SHd0IFEU06MWI`23Ph<0H61P06UYJ@1lO7`0J6QX08>3P`2NWYh0NGUi08F5Q@1/K6`0MgMg07Ee
M@1SHf<0PH6106QXJ01`L700RXZ:07MgM`1hN7P0NGUi08f=S@1cLg<0HF5Q07UiN@23Ph<0QhN705QH
F02MWId0MGEe06EUI@1lO7`0KFe]07QhN01XJ6P0GUiN07MgM`80H61P00d0J6QX08N7Q`2OWil0UIFE
08V9R@1WIfL0Lg=c06m_K`1HF5P0Rh^;05iNGP1JFUX0RXZ:0080G5aL00X0S8b<07AdM01KFe/0HV9R
05=CD`1eMGD0MgMg051@D01JFUX0IFEU3_ooool01@1EEED0RHV9075aL@1bLW80BDU90080FEUI0180
DE5A06a/K01KFe/0@d=30451@@1;Bd/0?Cdm051@D01HF5P0AdM705iNGP1RHV80DU9B0410@01KFe/0
EeMG03lo?`1VIVH204e=C@0O04a<C01?Cdl0CDe=059BDP1;Bd/0B4Q8059BDP1RHV80De=C07AdM01L
G5`0I6AT05IFEP29RHT0K6a/05MGE`1ADE40A4A404Q8B014A4@0E5AD04M7A`12@T80Bd];03lo?`12
@T80AdM705aLG01>CTh0K6a/059BDP02051@D01@05EEE@1:BTX0<S8b069RHP23Ph<0D51@06UYJ@1R
HV80KVi^051@D00k>c/0HV9R05aLG01TI6@0Lg=c06UYJ@1MGEd0HF5Q051@D01_Kfl0GEeM05IFEP1Y
JFT0L71`06YZJP1EEED0LW9b07alO01bLW80LG5a06][J`1aLG40KVi^06YZJP1iNGT0LW9b06MWI`1U
IFD0Kfm_06e]K@2GUiL0O7al05MGE`1QHF40LW9b05IFEP1VIVH0IfMW05iNGP1LG5`0JVYZ05aLG020
P800Hf=S05YJFP1OGel0K6a/06m_K`1kNg/0H61P07emO@1YJFT0QhN707QhN01IFET0NGUi069RHP1m
OGd0PX:208f=S@1dM7@0S8b<05=CD`1QHF40K6a/06m_K`1QHF40IfMW07]kN`2MWId>ooooo`0k06][
J`1OGel0@D5104A4A01;Bd/0A4A4051@D01>CTh0>SXj02LW9`0c<c<0:2PX03Ti>@17AdL0JFUY03Xj
>P0k>c/0<30`03lo?`1=CDd0>c/k05ADE01CDe<0=SHf05=CD`0T92@0A4A40451@@13@d<0GEeM04A4
A00_;bl0FUYJ03dm?@1QHF40@D51055AD@26QXH0@d=309FEU@19BDT0>CTi05QHF00d=3@0J6QX07]k
N`1cLg<0HV9R051@D01/K6`0CDe=04a<C01cLg<0JVYZ06][J`1HF5P0D51@04i>CP1aLG400P1SHf<0
C`1QHF40EEEE07AdM01bLW80CTi>07=cL`1fMWH0Lg=c06i^KP21PH40K6a/05UIF@1cLg<0Shn?07mo
O`1`L700RXZ:08F5Q@2;Rh/0KFe]09BDU01dM7@0KVi^09VIV@1`L700CDe=059BDP1QHF40Lg=c05MG
E`1OGel0Kfm_08F5Q@1KFe/0JFUY04i>CP1DE5@0LG5a07AdM01FEUH0MgMg071`L01EEED0JVYZ04m?
C`1mOGd0K6a/06=SH`16ATH0HF5Q06MWI`1LG5`0MgMg065QH@1GEeL0BTY:05mOG`1CDe<0GemO05AD
E01:BTX0MgMg03Xj>P1`L700JFUY06QXJ01BDU80A4A406IVIP1MGEd0K6a/069RHP1kNg/0D51@05MG
E`10@400H61P04Y:BP0f=SH03_ooool0I01oOgl0J6QX07AdM02GUiL0@41002<S8`0J6QX05ADE01dM
7@0N7Qh08R8R061PH00^;Rh0>3Ph08Z:RP2=SHd0NGUi04=3@`1=CDd0E5AD08^;R`1/K6`0DU9B04e=
C@0m?Cd0DE5A07=cL`0e=CD0<S8b0451@@2<S8`0YJFU09^KV`2PX:00UiNG08j>SP1[Jf/0FEUI05AD
E01aLG40MGEe05]KF`20P800Shn?08f=S@1OGel0;bl_059BDP1iNGT0Rh^;07=cL`1NGUh0LG5a08j>
SP25QHD0R8R805MGE`0o?cl0BDU906UYJ@2TY:@0J6QX07=cL`1^KVh0?Shn03Lg=`1HF5P0LW9b04a<
C01`L700TY:B07=cL`1>CTh0820P02`/;01TI6@0IfMW06a/K028R8P0NWYj0;Zj^P2n_[h0MWIf08>3
P`2JVYX0IFEU06e]K@1HF5P0DE5A08R8R02>SXh0QXJ6071`L023Ph<0a<C408B4Q02QXJ40NGUi06][
J`1^KVh202PX:00V0492@P2JVYX0SXj>09JFUP30`<00YJFU0:>SX`1bLW80C4a<04E5A@1cLg<0Kfm_
0;2`/02b/[80NGUi0:JVYP2@T900Hf=S0861P@2VYZH0MGEe06][J`2OWil0ATI609RHV011@D40AdM7
09:BTP2^[Zh0_[jn0820P02FUYH0Y:BT0861P@2VYZH0:b/[02XZ:P0n?ShDooooo`004oooool02@27
QhL0PX:207YjNP1MGEd0Ng]k0492@P1VIVH0JVYZ05ADE00205eMG@0^065QH@1WIfL0MWIf06i^KP1R
HV80IVIV07IfMP1KFe/0EUIF08>3P`1HF5P0DU9B079bLP1HF5P0GUiN096AT@0o?cl0JVYZ06MWI`1;
Bd/0GEeM04I6AP1MGEd0N7Qh05EEE@1UIFD0GemO06=SH`1hN7P0IFEU06][J`1^KVh0FUYJ05=CD`1d
M7@0Q8B40492@P1eMGD0RXZ:03Ti>@1fMWH0Ogmo03`l?01gMgL0OWin059BDP80M7Ad0500Fe]K079b
LP1dM7@0FEUI09>CT`1VIVH0KVi^08V9R@1jNWX0SHf=06m_K`1cLg<0KFe]06][J`23Ph<0OWin05iN
GP1XJ6P0NGUi05ADE01SHf<0MGEe06QXJ01/K6`0Rh^;07YjNP1[Jf/0_;bl06QXJ01WIfL0WIfM05YJ
FP1nOWh0R8R805YJFP27QhL0HF5Q075aL@2?Shl0EEEE08>3P`1mOGd0LG5a08:2PP20P800I6AT08b<
S01BDU80I6AT08N7Q`0f=SH0OWin06a/K01XJ6P0HF5Q09>CT`2/[:`0VIVI08F5Q@1YJFT0FEUI05aL
G027QhL0E5AD055AD@1]KFd0Fe]K05MGE`1^KVh0DU9B06QXJ01nOWh0DU9B05IFEP1<C4`0L71`06a/
K01LG5`0EEEE05]KF`kooooo04d0Bd];05iNGP1^KVh0LW9b03Xj>P1FEUH0I6AT04i>CP1/K6`0HF5Q
04i>CP1:BTX0ATI603Xj>P1BDU80EEEE05=CD`1IFET0FUYJ04M7A`0k>c/0HV9R05MGE`1;Bd/0IfMW
06][J`1:BTX0GUiN05mOG`1KFe/0FUYJ051@D01DE5@0H61P07QhN01RHV80LW9b06e]K@1eMGD0J6QX
09BDU021PH40G5aL061PH01ADE40G5aL04];B`1OGel0DE5A04U9B@1@D500ATI604m?C`18B4P0HF5Q
05ADE01jNWX0FUYJ04U9B@1?Cdl0HV9R04a<C011@D40JFUY08^;R`1UIFD0Kfm_08R8R01iNGT0=CDe
061PH00m?Cd0ATI604Q8B01_Kfl0CTi>04U9B@0204Q8B00905EEE@1>CTh0C4a<05MGE`1;Bd/0ATI6
04A4A01?Cdl0JVYZ0080GUiN0380Fe]K061PH01GEeL0CDe=05ADE01dM7@0GEeM06e]K@1TI6@0G5aL
06ATI01YJFT0DU9B04e=C@1WIfL0ADE5061PH01QHF40BDU905ADE01RHV80Bd];06i^KP1FEUH0P820
075aL@1OGel0KVi^08B4Q01GEeL0O7al06=SH`28R8P0NGUi05UIF@1eMGD0LG5a07AdM02WYjL0U9BD
06YZJP2MWId0CDe=05UIF@1^KVh0JFUY05eMG@1WIfL0NWYj08F5Q@kooooo0P1SHf<0L014A4@0De=C
03dm?@1;Bd/0;Rh^03<c<`0[:b/05ADE02TY:@0N7Qh092@T04Y:BP1WIfL0<S8b03Ti>@0/;2`0>CTi
051@D011@D40Bd];04e=C@1EEED0DE5A0410@01<C4`0?Cdm069RHP1TI6@0BDU90492@P1RHV80ATI6
065QH@10@400I6AT071`L01?Cdl0YZJV05UIF@15ADD0KFe]0492@P1UIFD0L71`08R8R01MGEd0G5aL
05IFEP1;Bd/0D51@06IVIP1PH600J6QX05mOG`16ATH0IVIV069RHP1EEED0KFe]05YJFP1LG5`0Bd];
07alO01;Bd/0Cdm?05UIF@1[Jf/0H61P06e]K@1bLW80De=C075aL@20P800GEeM05iNGP2HV9P0IVIV
08^;R`1UIFD0WYjN06MWI`1LG5`0][Jf05QHF010@400EUIF03lo?`1dM7@0ADE5059BDP1lO7`0SXj>
04m?C`1nOWh0GEeM03@d=022PX80NGUi069RHP1<C4`0N7Qh05QHF01TI6@0DU9B09NGU`1^KVh0JFUY
04I6AP1`L700IfMW05=CD`26QXH205EEE@0806=SH`1MGEd0BTY:07inOP15ADD0>SXj07emO@1JFUX2
05UIF@0>08>3P`13@d<0BDU907=cL`1VIVH0Ph>306EUI@1<C4`0Cdm?05EEE@1dM7@0DE5A051@D00n
?Sh>ooooo`2<0820P02DU9@0^k^k0;Jf]P0_;bl07QhN01HF5P0E5AD0ADE5038b<P1:BTX0EeMG034a
<@1[Jf/0PH6107IfMP1jNWX0=cLg03Hf=P0];Bd0DE5A04Q8B017AdL0EUIF07moO`1CDe<0HV9R0861
P@0_;bl0BDU906i^KP1ZJVX0HV9R08N7Q`1bLW80G5aL092@T026QXH0V9RH0<;2`P2HV9P0IFEU05AD
E019BDT0EUIF08R8R01^KVh0TI6A06MWI`2BTY80VIVI09fMW@22PX80>c/k05MGE`1NGUh0De=C04E5
A@0];Bd0=CDe0;Jf]P2OWil0Q8B408f=S@1=CDd09RHV02XZ:P0f=SH0ATI604e=C@1OGel0EeMG059B
DP0i>CT0<C4a02`/;00Z:RX0<C4a069RHP2WYjL0e=CD0;fm_@2RXZ80DU9B055AD@1aLG40De=C03Lg
=`1kNg/0];Bd0;2`/02IVIT0Z:RX0<O7a`33`l<0]kNg0<O7a`1eMGD0LG5a07]kN`1XJ6P0EeMG03dm
?@1]KFd0WinO0:FUY@21PH40QHF5069RHP0o?cl0BTY:06m_K`0V9RH0EeMG05mOG`1]KFd0<30`07If
MP11@D409RHV05EEE@1fMWH0clo?0:FUY@1gMgL0BTY:06e]K@26QXH0Rh^;06ATI02g]kL0W9bL0:n_
[`2a/K40WYjN09^KV`35aLD0MgMg0492@P16ATHDooooo`004oooool0R`1jNWX0O7al06a/K02CTi<0
D51@079bLP1cLg<0A4A407moO`0j>SX0P82005iNGP0j>SX0PH6106e]K@1BDU80LG5a06MWI`17AdL0
M7Ad065QH@1LG5`0KVi^05ADE01XJ6P0EUIF06][J`1UIFD0EUIF069RHP1`L700EeMG06MWI`1lO7`0
DU9B06EUI@2:RXX0@d=306MWI`2=SHd0@d=305YJFP2ATI40?clo06ATI01iNGT0ATI6061PH01iNGT0
C4a<075aL@1YJFT0HV9R05QHF01PH600OWin06ATI01JFUX0QHF5065QH@1UIFD0O7al09NGU`1YJFT0
QHF508B4Q01CDe<0PH610820P017AdL0UYJF07=cL`1OGel0/;2`06MWI`1[Jf/0Vi^K06QXJ01HF5P0
T92@06m_K`1bLW80Z:RX07moO`1`L700S8b<07YjNP2GUiL0IfMW06a/K01kNg/0K6a/07=cL`2EUID0
KVi^07IfMP2PX:00IVIV075aL@2:RXX0K6a/0::RXP1ADE40UiNG096AT@1HF5P0Jf][07alO029RHT0
M7Ad04Y:BP1`L700T92@04A4A00c<c<0CDe=05MGE`1eMGD0SXj>04U9B@1kNg/0X:2P05ADE024Q8@0
De=C05]KF`2<S8`0H61P051@D020P800GUiN04M7A`1PH600E5AD04];B`1aLG40EeMG055AD@20P800
3_ooool0D`16ATH0IVIV06][J`1[Jf/0>3Ph05YJFP1HF5P0HV9R06YZJP1RHV80EUIF051@D01GEeL0
ADE504Q8B01BDU80EeMG07QhN01[Jf/0@d=303Lg=`1NGUh0FEUI04i>CP1_Kfl0DU9B04];B`1LG5`0
F5QH04a<C01GEeL0BTY:05MGE`1QHF40MWIf065QH@2@T900KFe]06IVIP1WIfL0ZZZZ07]kN`1[Jf/0
GemO059BDP1LG5`0ATI605]KF`13@d<0BTY:05EEE@15ADD0?Shn04U9B@1IFET0EEEE06=SH`1QHF40
Fe]K05=CD`1NGUh0De=C03dm?@1TI6@0LW9b069RHP1mOGd0Kfm_08F5Q@1aLG40I6AT05EEE@1?Cdl0
EEEE08J6QP1XJ6P0GUiN06m_K`1YJFT0MGEe065QH@19BDT0LW9b0080EUIF03L0CTi>06ATI025QHD0
Hf=S061PH01CDe<0Jf][05QHF014A4@0Hf=S05]KF`1YJFT0CTi>06YZJP1ADE40Q8B406UYJ@1;Bd/0
De=C06ATI01RHV80IVIV06UYJ@1UIFD0JVYZ07EeM@1OGel0NGUi06ATI01iNGT0PX:207AdM01]KFd0
MWIf06m_K`25QHD0Ogmo0::RXP22PX80Fe]K06=SH`1dM7@0Ph>309bLW02QXJ40P8200:ZZZP1IFET0
GUiN06YZJP1hN7P0Hf=S06IVIP2MWId0YZJV00kooooo07l0P82006m_K`1IFET0C4a<03lo?`16ATH0
@T9204=3@`0e=CD0<C4a03<c<`0M7Ad0?Cdm03`l?01UIFD0<c<c03dm?@0i>CT09bLW05QHF019BDT0
MWIf04];B`17AdL0DU9B03hn?P17AdL0:2PX04A4A01VIVH0>c/k03Hf=P1PH600?clo04I6AP0];Bd0
B4Q806IVIP1EEED0ZZZZ055AD@1SHf<0K6a/034a<@1UIFD0MGEe09FEU@1PH600Cdm?05YJFP1?Cdl0
>c/k06EUI@1ADE40EEEE05QHF00j>SX0B4Q8061PH01=CDd0G5aL06=SH`1<C4`0Lg=c06MWI`11@D40
LG5a04a<C01JFUX0Jf][06e]K@1VIVH0HF5Q079bLP1SHf<0JFUY08F5Q@2MWId0LW9b0:6QX@1OGel0
LW9b07AdM01WIfL0XJ6Q06][J`19BDT0E5AD05]KF`1eMGD0BTY:05UIF@1EEED0P82004I6AP1TI6@0
?Shn02TY:@27QhL0Jf][05MGE`1NGUh0N7Qh04U9B@1OGel0EeMG08V9R@1jNWX0JFUY038b<P1[Jf/0
EUIF06IVIP2=SHd0O7al06IVIP1BDU80MGEe05aLG01kNg/0Cdm?05MGE`1dM7@0EEEE06][J`1LG5`0
L71`0080Cdm?00/0NWYj061PH023Ph<0GemO05EEE@1IFET0GEeM05mOG`1RHV80HF5Q04U9B@0>oooo
o`0H08B4Q029RHT0Fe]K0:FUY@2:RXX08R8R01TI6@0K6a/0<C4a05QHF02ZZZX0XZ:R07]kN`29RHT0
Shn?04I6AP0l?3`0K6a/03Ph>00/;2`0DU9B05UIF@0W9bL0=3@d0P1iNGT03`0g=cL0GUiN04];B`0d
=3@09RHV024Q8@0Y:BT0<C4a01DE5@0F5QH0<c<c03hn?P1EEED0`l?309ZJVP0203dm?@1405UIF@0n
?Sh0?Cdm07moO`2>SXh0T92@06=SH`1XJ6P0VYZJ08J6QP1>CTh0B4Q804M7A`0k>c/0@D5106UYJ@1o
Ogl0[jn_09nOW`1`L700MWIf059BDP1:BTX09RHV02LW9`0d=3@0JFUY04Q8B01?Cdl0EUIF02d];@0Z
:RX092@T02/[:`11@D40KVi^09nOW`2RXZ80]KFe0;no_`2SXj<0Ogmo04m?C`1;Bd/0CTi>06e]K@20
P800YjNW0:ZZZP2m_Kd0/k>c08f=S@2PX:00WIfM04Q8B013@d<0EeMG09fMW@2HV9P0Rh^;09BDU02m
_Kd0cLg=09jNWP24Q8@0?Cdm03`l?00a<C4203Lg=`0K04e=C@1CDe<0D51@034a<@1mOGd0=cLg02@T
901CDe<0ADE506e]K@2WYjL0M7Ad08R8R01VIVH0QXJ60:b/[030`<00/;2`09ZJVP28R8P0/;2`0:f]
[@2HV9P0NWYj04=3@`15ADD0P82001Cooooo000Cooooo`1O09:BTP1_Kfl0PX:205MGE`1nOWh0>3Ph
06MWI`1nOWh0Bd];05MGE`1oOgl0BDU905mOG`1aLG40E5AD06MWI`1FEUH0IVIV06m_K`1?Cdl0Kfm_
06IVIP16ATH0OWin06QXJ01:BTX0OGem065QH@1:BTX0Fe]K06e]K@1DE5@0IVIV06=SH`1aLG40MWIf
06QXJ01LG5`0K6a/069RHP1CDe<0O7al059BDP1UIFD0I6AT05iNGP1XJ6P0EUIF05]KF`1UIFD0?Shn
05UIF@1nOWh0@T9206UYJ@26QXH0>SXj05iNGP2FUYH0ADE5065QH@1lO7`0ADE507MgM`1oOgl0GemO
08n?S`1lO7`0LG5a09RHV02;Rh/0HV9R06IVIP29RHT0LW9b05YJFP1OGel0QXJ606EUI@1IFET0MgMg
07EeM@1MGEd0NWYj07inOP1KFe/0ZJVY06UYJ@1ZJVX0Zj^[05ADE01]KFd0Zj^[05]KF`27QhL00P1o
Ogl0:P2>SXh0MWIf07UiN@1iNGT0HV9R07]kN`24Q8@0KVi^092@T01MGEd0IfMW09FEU@0o?cl0Ng]k
06ATI01?Cdl0HV9R0:NWY`3EeMD0QHF506QXJ01_Kfl0GUiN07QhN01aLG40IVIV092@T01KFe/0FUYJ
06e]K@1OGel0EEEE07MgM`23Ph<0J6QX051@D01EEED0JVYZ07emO@1LG5`0FUYJ06MWI`kooooo03@0
Cdm?06IVIP1VIVH0MWIf05ADE01KFe/0I6AT05]KF`1NGUh0J6QX04U9B@1>CTh0EeMG04=3@`1BDU80
FEUI06UYJ@1eMGD0Hf=S03lo?`0b<S80GEeM051@D00o?cl0J6QX05iNGP1@D500EEEE05aLG01;Bd/0
EeMG05EEE@1RHV80EEEE06e]K@1OGel0Q8B407=cL`1lO7`0HV9R08j>SP1aLG40De=C05MGE`13@d<0
Bd];03lo?`1CDe<0@d=304M7A`1BDU80?Cdm0P15ADD0=P1GEeL0Bd];06ATI01?Cdl0Lg=c05YJFP1=
CDd0EUIF03`l?01SHf<0LW9b06UYJ@1ZJVX0Hf=S06e]K@1OGel0H61P06ATI019BDT0EeMG07]kN`1U
IFD0EUIF04m?C`1JFUX0IFEU05YJFP1EEED0Lg=c05UIF@1OGel0A4A4065QH@1gMgL0EEEE05ADE01S
Hf<0J6QX05aLG016ATH0FEUI05ADE01bLW80HV9R05]KF`1IFET0N7Qh05iNGP1ADE40GEeM079bLP1J
FUX0Lg=c07alO080K6a/01h0PX:203dm?@1/K6`0IFEU07YjNP1aLG40LW9b075aL@2;Rh/0Q8B407Ui
N@1bLW80YjNW08j>SP1aLG40Q8B406][J`2>SXh0[jn_09RHV024Q8@0];Bd06UYJ@1LG5`0HV9R06][
J`1NGUh0G5aL08:2PP2_[jl>ooooo`0L0;Fe]@25QHD0FEUI04e=C@0m?Cd0ATI603<c<`0a<C40?clo
034a<@0^;Rh0:BTY02h^;P1SHf<0NGUi02d];@18B4P0@41003Hf=P1:BTX0@T9208b<S00c<c<0HF5Q
05ADE00g=cL0?3`l03Xj>P80De=C06h0>SXj04Y:BP15ADD0@T9203/k>`0o?cl0D51@055AD@17AdL0
Q8B405EEE@18B4P0GUiN05IFEP1DE5@0FEUI09NGU`1EEED0FUYJ055AD@1@D500EEEE06MWI`1ADE40
GEeM06ATI015ADD0GEeM05mOG`1DE5@0GemO05eMG@1JFUX0G5aL07IfMP1>CTh0IFEU04I6AP1HF5P0
VIVI04m?C`1aLG40GEeM069RHP1[Jf/0GEeM07QhN025QHD0SHf=0;2`/01LG5`0KVi^07=cL`1aLG40
];Bd07MgM`1IFET0F5QH05UIF@1dM7@0G5aL05YJFP1SHf<0Q8B404M7A`27QhL0B4Q8030`<02@T900
M7Ad05YJFP1FEUH0Lg=c055AD@1XJ6P0EEEE07]kN`1RHV80Hf=S0492@P1YJFT0Fe]K04a<C01jNWX0
JFUY05=CD`1;Bd/0E5AD04a<C01iNGT0>SXj04e=C@1NGUh0Hf=S05EEE@19BDT0MGEe03Ph>00l?3`0
K6a/04i>CP1dM7@0IFEU06e]K@18B4P0EeMG05]KF`1:BTX0H61P0492@Pkooooo0280>c/k06EUI@1d
M7@0[Jf]0:n_[`2MWId0IfMW04E5A@17AdL0E5AD0:ZZZP23Ph<0L71`05YJFP0g=cL0EEEE03@d=00/
;2`0PX:204U9B@18B4P0EUIF02l_;`0S8b<0KVi^0:n_[`2DU9@0Xj>S06a/K018B4P0=CDe01TI6@0G
5aL0=SHf0P0F5QH01P0Q8B40IfMW06UYJ@1mOGd0HF5Q038b<P80<C4a0600De=C06UYJ@1mOGd0G5aL
05ADE00k>c/0HV9R03`l?01QHF40MgMg06YZJP2JVYX0SHf=04Q8B01kNg/0EEEE08:2PP2MWId0?Cdm
03Lg=`12@T80@410055AD@0[:b/09RHV06=SH`1hN7P0RXZ:06e]K@0];Bd0:RXZ02TY:@0l?3`0GUiN
0861P@1oOgl0T92@04];B`2`/;00UiNG02XZ:P0k>c/07alO03/k>`27QhL0Q8B409>CT`2^[Zh0D51@
07alO01`L700M7Ad07alO013@d<0Cdm?05IFEP2g]kL0SXj>075aL@2n_[h0`<3007YjNP1JFUX0QHF5
02d];@0a<C40=3@d03Lg=`1GEeL0MgMg05eMG@1OGel0T92@0:f][@24Q8@0BTY:06a/K012@T80HF5Q
08R8R02?Shl0KVi^0492@P10@400O7al09RHV02VYZH0ATI6034a<@1RHV80UIFE05iNGP1QHF40CDe=
03Lg=`1@D50Dooooo`004oooool06028R8P0Ogmo071`L01gMgL0Fe]K04E5A@1SHf<0E5AD05YJFP1L
G5`0PH6106][J`15ADD0KVi^06=SH`18B4P0EeMG07UiN@1>CTh0JVYZ05iNGP1BDU80F5QH07moO`80
DU9B0140J6QX055AD@1HF5P0CDe=069RHP1NGUh0G5aL06e]K@1SHf<0EUIF07QhN01@D500EeMG07Yj
NP19BDT0F5QH07]kN`0205IFEP0@092@T01HF5P0FEUI08j>SP1UIFD0Cdm?069RHP1^KVh0EUIF079b
LP1aLG40Fe]K069RHP1bLW80IfMW0820P080K6a/04`0O7al06][J`1nOWh0IVIV071`L024Q8@0FUYJ
07QhN020P800FUYJ07AdM02>SXh0F5QH07IfMP1hN7P0F5QH075aL@1_Kfl0IFEU07alO029RHT0K6a/
06UYJ@24Q8@0Lg=c071`L01nOWh0G5aL07YjNP2@T900MGEe07MgM`1fMWH0OGem06MWI`1CDe<0NGUi
07QhN01dM7@0KVi^07moO`1dM7@0MWIf061PH01aLG40PX:2065QH@1bLW80Hf=S07emO@14A4@0BTY:
05EEE@1ADE40N7Qh07moO`1DE5@0KFe]08j>SP1GEeL0Ogmo06e]K@1CDe<0LW9b06i^KP15ADD0Ogmo
06a/K01;Bd/0FUYJ05UIF@17AdL0Lg=c05MGE`18B4P0IfMW3_ooool06P1NGUh0JFUY06YZJP1iNGT0
C4a<05YJFP1XJ6P0Kfm_06EUI@1TI6@0Bd];04Q8B01ADE40@41005UIF@1OGel0IFEU06e]K@1OGel0
A4A402d];@1GEeL0De=C0451@@1^KVh0Cdm?0P1ADE40K01DE5@0CDe=05eMG@1FEUH0GemO06IVIP1k
Ng/0G5aL07EeM@1nOWh0Jf][06e]K@2UYJD0PH6105YJFP1PH600DE5A05QHF019BDT0GEeM04e=C@1>
CTh0E5AD04M7A`15ADD0@d=305mOG`1@D500IfMW04e=C@1MGEd0EEEE051@D01CDe<0?Shn06=SH`1_
Kfl0Lg=c075aL@1SHf<0Kfm_04];B`1FEUH0DE5A04E5A@1<C4`0Fe]K055AD@1FEUH0I6AT06MWI`1j
NWX0IVIV05aLG01cLg<0G5aL06QXJ01=CDd0GEeM06UYJ@1VIVH0GUiN06ATI01QHF40EeMG04Q8B01H
F5P0G5aL05mOG`1DE5@0Hf=S05EEE@1jNWX0Hf=S03lo?`1<C4`0CTi>05aLG01aLG40Y:BT06UYJ@1Q
HF40MWIf05ADE01[Jf/0JFUY07EeM@1nOWh0K6a/07EeM@2>SXh0IFEU07YjNP1aLG40UYJF09nOW`21
PH40N7Qh07=cL`2NWYh0_kno0:f][@2KVi/0[Jf]05UIF@1JFUX0K6a/07MgM`80GemO00<0Ph>30:6Q
XOooool03Oooool0<@2l_;`0Jf][06=SH`12@T80>CTi0492@P0k>c/0=cLg02PX:00a<C40;bl_01lO
7`0g=cL0BDU9075aL@0g=cL0?Cdm04I6AP0Y:BT0AdM702/[:`14A4@0EEEE051@D00l?3`0Cdm?03Ti
>@0e=CD0B4Q805YJFP0o?cl0DE5A05ADE01=CDd0>3Ph051@D01<C4`0@T9207EeM@25QHD0GEeM05mO
G`1HF5P0@T9206YZJP1gMgL0YJFU05YJFP1BDU800P1KFe/0>01LG5`0KFe]05UIF@1QHF40H61P05eM
G@1DE5@0H61P07=cL`1eMGD0Lg=c065QH@1_Kfl0Q8B404M7A`1/K6`0F5QH05YJFP2DU9@0F5QH079b
LP24Q8@0IfMW05EEE@1YJFT0N7Qh0820P01^KVh0^[Zj071`L02ZZZX0K6a/06MWI`2QXJ40JFUY05IF
EP1EEED0E5AD071`L01PH600IFEU06=SH`2EUID0D51@07YjNP1BDU80;2`/092@T01SHf<0NGUi05eM
G@1gMgL0CTi>06IVIP1`L700Lg=c0P1_Kfl07`0i>CT0IVIV05iNGP1SHf<0Q8B408>3P`1WIfL0GEeM
06=SH`1DE5@0Ph>304a<C01RHV80GEeM06YZJP1UIFD0FEUI079bLP15ADD0DU9B06QXJ01>CTh0LW9b
07YjNP1[Jf/0ADE504M7A`1PH600@T92061PH01KFe/03_ooool07P11@D40R8R80:ZZZP2XZ:P0YJFU
0:n_[`2>SXh0D51@05=CD`2YZJT0PX:20861P@2ATI40FEUI02LW9`0a<C40;bl_01XJ6P0_;bl0WinO
06YZJP1FEUH0H61P06i^KP2?Shl0_Kfm06UYJ@10@400GUiN05QHF080@D5100X0@410055AD@19BDT0
AdM704U9B@0n?Sh0GUiN0;^k^`24Q8@0?Shn0P0M7Ad0D@0k>c/0M7Ad06a/K014A4@0UIFE06m_K`21
PH40EUIF05mOG`1dM7@0T92@05iNGP14A4@0=SHf09JFUP19BDT0?clo08R8R01:BTX0:BTY03/k>`0j
>SX0<c<c05eMG@1kNg/0Q8B40:VYZ@1cLg<0M7Ad075aL@1iNGT0Cdm?0861P@1KFe/0E5AD08V9R@2g
]kL0W9bL0;Zj^P0^;Rh07QhN0410@00M7Ad0Bd];04Q8B01nOWh0K6a/0:RXZ01`L700NGUi08^;R`2d
];@0I6AT03@d=01?Cdl0JVYZ07IfMP1`L700YjNW0;Vi^@1FEUH0<S8b05YJFP1KFe/0MGEe02l_;`14
A4@0Fe]K08Z:RP2g]kL0/K6a0:2PX01GEeL0I6AT05QHF025QHD0RXZ:09:BTP2UYJD0Y:BT09fMW@02
04Q8B00=06][J`2IVIT0OWin03lo?`0f=SH0:BTY061PH01>CTh0ATI604];B`1=CDd0IVIV03`l?00D
ooooo`004oooool01P29RHT0Kfm_07AdM01lO7`0AdM7065QH@80HV9R00l0JVYZ04Y:BP1hN7P0LG5a
04=3@`23Ph<0IFEU04Q8B01RHV80JVYZ04A4A01cLg<0DU9B05EEE@1CDe<00P1YJFT0CP13@d<0HF5Q
05=CD`1<C4`0HF5Q061PH00g=cL0OWin079bLP19BDT0IFEU0:FUY@12@T80GEeM08f=S@11@D40GemO
06MWI`18B4P0LW9b06MWI`1DE5@0Kfm_05UIF@1BDU80NWYj04Y:BP1HF5P0FUYJ04e=C@1MGEd0DE5A
04e=C@1lO7`0EUIF059BDP1bLW80Hf=S05QHF026QXH0JVYZ05EEE@27QhL0LW9b04Y:BP2IVIT0JFUY
05]KF`2MWId0IFEU065QH@24Q8@0FEUI065QH@1fMWH0G5aL06m_K`1bLW80IVIV06e]K@1VIVH0Hf=S
07QhN01KFe/0JFUY07=cL`1aLG40TY:B096AT@1QHF40XJ6Q092@T01SHf<0Rh^;05YJFP1bLW80VYZJ
055AD@80QHF500T0IVIV06a/K01OGel0Rh^;07UiN@1>CTh0LG5a07emO@11@D400P1iNGT02017AdL0
O7al07MgM`1@D500M7Ad06m_K`1GEeL0SXj>0P1IFET03@24Q8@0AdM704m?C`1eMGD0E5AD061PH01W
IfL0EeMG05iNGP1dM7@0E5AD05UIF@1ZJVX03_ooool05@1DE5@0K6a/06YZJP1nOWh0@d=305aLG01`
L700PH6106a/K01YJFT0ATI603dm?@1:BTX0@d=305iNGP1EEED0Hf=S071`L01SHf<0@D5103/k>`02
05EEE@0T0410@01eMGD0CDe=051@D01JFUX0F5QH059BDP1PH600EeMG05mOG`1XJ6P0P82007MgM`1V
IVH0P82006ATI01ZJVX0[jn_08R8R01SHf<0J6QX05YJFP1RHV80DU9B06=SH`1KFe/0HF5Q05QHF01E
EED0BTY:04U9B@1YJFT0F5QH06EUI@1bLW80GUiN0P1FEUH0C`1LG5`0C4a<06=SH`21PH40Rh^;08B4
Q029RHT0K6a/055AD@1BDU80C4a<04Y:BP1DE5@0LG5a05=CD`1ADE40GUiN05MGE`24Q8@0GUiN05=C
D`1TI6@0FEUI07EeM@1=CDd0DE5A07EeM@2?Shl0DU9B071`L01FEUH0EEEE04i>CP1/K6`0FEUI05YJ
FP1BDU80Hf=S05iNGP1RHV80JVYZ03/k>`16ATH0DU9B04I6AP1YJFT0KVi^05IFEP1DE5@0IfMW06EU
I@22PX80IVIV06a/K01_Kfl0HV9R071`L02CTi<0P82007alO01dM7@0N7Qh09VIV@2EUID0K6a/0861
P@2_[jl0/K6a0::RXP2l_;`0WYjN05]KF`1@D500IfMW06i^KP1GEeL0Fe]K08b<S02JVYX03_ooool0
N`2CTi<0IfMW08j>SP16ATH0@T92051@D00o?cl0<C4a02/[:`0O7al0;Rh^030`<00h>3P0CDe=06][
J`0^;Rh0?Shn04M7A`0e=CD0B4Q805mOG`0k>c/0IfMW05QHF00o?cl0Fe]K02h^;P0_;bl0HV9R05MG
E`0g=cL0GEeM055AD@1@D500?Shn055AD@1MGEd0?Cdm06i^KP1IFET0IfMW05IFEP15ADD0FEUI06YZ
JP1GEeL0Vi^K05ADE01;Bd/0EUIF04m?C`1<C4`0JFUY04a<C01NGUh0B4Q804a<C01MGEd0A4A404i>
CP1DE5@0De=C06i^KP1BDU80L71`04Q8B01WIfL0EeMG075aL@2>SXh0CTi>07YjNP29RHT0FEUI06MW
I`1VIVH0KVi^06IVIP1TI6@0SXj>09RHV02^[Zh0JFUY07UiN@2GUiL0H61P069RHP1OGel0=CDe071`
L01ZJVX0EeMG065QH@1gMgL0EEEE08N7Q`1HF5P09bLW08N7Q`1=CDd0JVYZ05IFEP1]KFd0AdM705iN
GP1CDe<0Lg=c075aL@24Q8@0=CDe06m_K`1@D500GEeM0820P01mOGd0EUIF05aLG01QHF40B4Q807Yj
NP1=CDd0JVYZ04i>CP0206UYJ@0?03lo?`1SHf<0ATI605eMG@1VIVH0A4A407emO@1eMGD0Fe]K05aL
G00[:b/0HF5Q04Y:BP1^KVh0I6AT00kooooo00T0DU9B061PH02FUYH0Ti>C071`L02HV9P0W9bL04A4
A00g=cL00P2[Zj/0P@2`/;00IfMW06IVIP1EEED0ADE508b<S017AdL0UIFE07QhN01dM7@0Y:BT09BD
U00d=3@0=cLg08Z:RP1XJ6P0EUIF03Xj>P0h>3P0Hf=S06i^KP1@D500HF5Q04U9B@17AdL0A4A404Y:
BP1XJ6P0RHV905IFEP15ADD0OWin04];B`0e=CD0?Shn0:2PX02=SHd0ATI606IVIP2JVYX0HV9R05EE
E@0k>c/0Xj>S09:BTP0g=cL0EeMG08F5Q@20P800G5aL0:6QX@1UIFD0F5QH06UYJ@2<S8`0KVi^092@
T02KVi/0@T9208j>SP2OWil0TI6A09VIV@2YZJT0[jn_0=CDe021PH40Q8B409:BTP1hN7P0Y:BT0<c<
c01dM7@0CDe=09JFUP1WIfL0TY:B07emO@1bLW80DE5A0:f][@22PX80Ph>308B4Q02j^[X0NGUi04I6
AP1jNWX0<c<c01dM7@0[:b/0?Cdm0861P@23Ph<0LG5a06ATI01QHF40YJFU0820P02PX:00`/;20<K6
aP1jNWX0NGUi03Lg=`0l?3`0>CTi0492@P0c<c<0QHF50<G5a@2o_kl0/k>c09fMW@1WIfL0Jf][0:>S
X`1]KFd0FUYJ02/[:`0e=CD0;Rh^05QHF00g=cL0CDe=06MWI`1mOGd0Zj^[08N7Q`0Dooooo`004ooo
ool0R`24Q8@0N7Qh06YZJP1LG5`0Hf=S03Lg=`1RHV80L71`03dm?@1^KVh0SHf=04Y:BP1dM7@0Kfm_
071`L01DE5@0B4Q806ATI01LG5`0C4a<07YjNP1CDe<0>SXj08J6QP1MGEd0CDe=07QhN012@T80E5AD
07=cL`1CDe<0DE5A07inOP1]KFd0G5aL05eMG@1NGUh0J6QX05iNGP1ZJVX0EUIF069RHP1TI6@0UiNG
04=3@`1kNg/0PH61059BDP1WIfL0L71`04E5A@1VIVH0KFe]03dm?@1PH600SHf=04M7A`1lO7`0Ti>C
04];B`2<S8`0NGUi069RHP29RHT0Jf][069RHP1hN7P0Hf=S07inOP1RHV80Q8B407UiN@1^KVh0OGem
06m_K`1/K6`0NWYj08V9R@1DE5@0Hf=S0820P01/K6`0JVYZ08j>SP1bLW80Hf=S09JFUP1WIfL0O7al
08F5Q@1CDe<0MWIf09RHV01jNWX0QXJ607]kN`21PH40PX:2059BDP2ATI40QHF505YJFP1`L700P820
069RHP1hN7P0DE5A05mOG`2BTY80CDe=06m_K`1DE5@0MgMg06][J`1FEUH0QHF506ATI01OGel0TI6A
04];B`1JFUX0XJ6Q05=CD`1hN7P0P82006ATI01iNGT0OGem061PH01fMWH0KFe]04I6AP1ADE40I6AT
05aLG01UIFD0I6AT04I6AP1IFET03_ooool0>P1DE5@0HF5Q05UIF@1NGUh0AdM704m?C`15ADD0J6QX
06e]K@1NGUh0AdM70410@014A4@0@410061PH01FEUH0HF5Q06e]K@1VIVH0@d=3038b<P1NGUh0EEEE
04A4A024Q8@0HV9R05mOG`1PH600GUiN05IFEP1UIFD0DU9B05ADE01YJFT0Ng]k07MgM`1kNg/0Lg=c
08V9R@1MGEd0WIfM08f=S@1QHF40Kfm_05YJFP1LG5`0Cdm?06EUI@1BDU80EEEE04m?C`1<C4`0CDe=
04U9B@1JFUX0F5QH05mOG`1JFUX205IFEP80G5aL01/0I6AT06][J`26QXH0ZJVY09>CT`20P800O7al
04U9B@1FEUH0E5AD04E5A@1HF5P0NGUi06MWI`1HF5P0DU9B06EUI@1lO7`0L71`05UIF@1^KVh0Hf=S
07QhN018B4P0FUYJ05eMG@21PH400P1FEUH02`1DE5@0EeMG0410@01WIfL0FUYJ05mOG`1QHF40IfMW
05IFEP1hN7P0GUiN0080BDU902@0CTi>05=CD`1SHf<0HV9R05=CD`1NGUh0J6QX06UYJ@20P800J6QX
06UYJ@1VIVH0EEEE06IVIP25QHD0M7Ad05QHF01^KVh0Fe]K09FEU@2<S8`0M7Ad075aL@2IVIT0]kNg
09fMW@2d];@0[:b/06e]K@1<C4`0I6AT065QH@1FEUH0H61P071`L025QHD>ooooo`0P09:BTP1LG5`0
LW9b04=3@`0a<C40@d=303/k>`16ATH0@T920451@@0/;2`0:b/[03Lg=`1:BTX0GEeM03dm?@0b<S80
=3@d030`<018B4P0ADE502PX:01>CTh0De=C03lo?`1IFET0>CTi034a<@15ADD0FEUI059BDP1LG5`2
04Y:BP0g02LW9`1?Cdl0DE5A02l_;`24Q8@0LW9b05aLG019BDT0ADE50410@01MGEd0K6a/0:VYZ@1L
G5`0D51@061PH01HF5P0MGEe075aL@1]KFd0Jf][055AD@1OGel0Cdm?04Y:BP1MGEd0GemO065QH@1]
KFd0GUiN06][J`13@d<0U9BD05EEE@1UIFD0TI6A055AD@1`L700N7Qh05MGE`1ZJVX0K6a/08N7Q`1`
L700Fe]K07UiN@2/[:`0emOG06UYJ@2WYjL0WinO061PH01aLG40Hf=S03Lg=`0207EeM@0J075aL@1S
Hf<0EeMG06a/K01iNGT0EUIF034a<@29RHT0G5aL06EUI@1?Cdl0H61P04=3@`1/K6`0IVIV06][J`1E
EED0M7Ad03Lg=`1IFET0BTY:06=SH`1kNg/0DE5A05ADE01HF5P2061PH00E07IfMP1;Bd/0NWYj061P
H01cLg<0Kfm_04I6AP1hN7P0=cLg03lo?`1NGUh0BTY:06e]K@2BTY80KFe]06QXJ00R8R80MgMg04];
B`1mOGd0PX:200kooooo02h0BTY:079bLP1hN7P0DU9B038b<P1LG5`0_[jn03Xj>P0Y:BT0ZJVY0=KF
eP21PH40@d=3051@D01<C4`0?clo06QXJ02=SHd0EEEE04e=C@1IFET0Kfm_08J6QP0e=CD0>SXj07=c
L`1:BTX0P82004Q8B00];Bd0@T9207emO@1`L700M7Ad02`/;00N7Qh08B4Q02`/;015ADD0Q8B4051@
D00b<S80Hf=S03@d=00Q8B40AdM70P1YJFT0B`1QHF40OWin05QHF01;Bd/0De=C07emO@2VYZH0Lg=c
08N7Q`18B4P0:BTY071`L02<S8`0EeMG06i^KP1oOgl0JVYZ0<?3``2j^[X0QhN708R8R02ATI40f=SH
03De=@0];Bd0L71`06m_K`1GEeL0V9RH0;Ng]`2_[jl0BDU906YZJP2a/K40][Jf0:ZZZP2g]kL0ADE5
02XZ:P1RHV80TI6A07moO`1VIVH0J6QX05ADE01cLg<0UYJF0:VYZ@1bLW809bLW04e=C@0Q8B409BDU
024Q8@0X:2P0ZZZZ0=7Ad@2VYZH0CTi>05eMG@2BTY80IfMW0<G5a@2k^k/0UiNG0492@P1PH600A4A4
03lo?`1kNg/0Xj>S0:f][@2n_[h0QhN706MWI`0205eMG@0?07UiN@2][Jd0_;bl05]KF`0l?3`0@410
034a<@0c<c<0TI6A05IFEP1?Cdl0a/K60;Vi^@2PX:00[Zj^01Cooooo000Cooooo`0h08V9R@1kNg/0
Lg=c08>3P`1NGUh0ADE506MWI`1?Cdl0GUiN05eMG@1ZJVX0I6AT04=3@`1XJ6P0IfMW0451@@1EEED0
Kfm_05=CD`1GEeL0LW9b05eMG@19BDT0M7Ad05IFEP1CDe<0MGEe055AD@1GEeL0D51@05ADE01GEeL0
Bd];05mOG`1MGEd0E5AD07moO`1;Bd/0I6AT075aL@1CDe<0IVIV06YZJP1]KFd0HF5Q061PH01NGUh0
Jf][06ATI01QHF40Cdm?061PH01XJ6P0D51@05aLG01hN7P2059BDP0W06m_K`1EEED0KFe]06i^KP1N
GUh0N7Qh06i^KP1dM7@0CTi>07UiN@1aLG40A4A408N7Q`1/K6`0CTi>09>CT`1hN7P0DE5A07=cL`1^
KVh0GUiN06IVIP1fMWH0FUYJ07]kN`1cLg<0K6a/07UiN@1aLG40Kfm_07EeM@26QXH0Jf][07inOP1Z
JVX0Shn?0861P@1NGUh0Xj>S0080JVYZ02P0Ng]k06IVIP2>SXh0HF5Q07UiN@23Ph<0GEeM06a/K01n
OWh0PH6107=cL`1bLW80K6a/08J6QP1]KFd0MgMg07YjNP1IFET0LW9b07UiN@1<C4`0Kfm_08>3P`1:
BTX0Lg=c06][J`14A4@0WYjN06a/K01;Bd/0OGem06MWI`17AdL0HV9R07alO01LG5`0I6AT05MGE`1:
BTX0HV9R3_ooool02P1>CTh0IVIV065QH@1dM7@0A4A405EEE@1IFET0Hf=S06EUI@1RHV8204=3@`1C
04Y:BP11@D40GEeM05mOG`1YJFT0Lg=c06YZJP1;Bd/0@T9206QXJ01GEeL0=CDe06e]K@1=CDd0CTi>
04e=C@1@D500EUIF05eMG@1DE5@0FUYJ06e]K@1WIfL0L71`075aL@1YJFT0Fe]K061PH02:RXX0O7al
05QHF01PH600EUIF05YJFP1BDU80CDe=059BDP19BDT0CDe=04m?C`1DE5@0Bd];069RHP1LG5`0GEeM
05IFEP1LG5`0GemO05YJFP1PH600De=C04Y:BP1WIfL0V9RH08Z:RP1lO7`0K6a/059BDP1RHV80EUIF
0492@P15ADD0KVi^069RHP1<C4`0FEUI05QHF022PX80J6QX065QH@1gMgL0IVIV07QhN01DE5@0Fe]K
06a/K01PH600GemO05QHF01EEED0FEUI04I6AP1lO7`00P1QHF40:`1=CDd0J6QX04Y:BP1JFUX0Ng]k
04i>CP1<C4`0BTY:05IFEP1UIFD0E5AD05EEE@1KFe/0L71`06e]K@1kNg/0J6QX05mOG`1IFET0De=C
06YZJP1oOgl0OGem06ATI01VIVH0GEeM08b<S029RHT0P82007EeM@2LW9`0][Jf09NGU`2PX:00Rh^;
07AdM01MGEd0IfMW07IfMP1NGUh0HF5Q06m_K`2<S8`03_ooool0<@2;Rh/0F5QH08V9R@0l?3`0ATI6
0451@@12@T80?Shn03Hf=P0o?cl07AdM02/[:`0_;bl0FEUI06YZJP1BDU80<S8b030`<00h>3P0D51@
03Hf=P15ADD0GEeM059BDP16ATH0JVYZ03@d=00g=cL0FUYJ05aLG01MGEd0J6QX03De=@1KFe/0@T92
05ADE01IFET0BTY:0861P@1CDe<0Kfm_04M7A`0o?cl0GUiN061PH01OGel0ZJVY05mOG`1FEUH00P1?
Cdl0201WIfL0DE5A05UIF@1GEeL0GUiN06a/K01>CTh0GemO0P1NGUh04@1PH600Jf][05YJFP1WIfL0
BTY:08N7Q`19BDT0J6QX092@T01HF5P0IFEU08f=S@1CDe<0IFEU075aL@1fMWH0KFe]0080Jf][01P0
h>3P0>?Sh`1dM7@0QXJ6096AT@1YJFT0MGEe05ADE00k>c/0FEUI07=cL`1ADE40EeMG05YJFP1`L700
M7Ad05UIF@0d=3@0Q8B405aLG01^KVh0Hf=S07AdM01>CTh3065QH@0Q06][J`2BTY80>SXj06i^KP1@
D500DE5A08R8R01CDe<0FEUI061PH01/K6`0DU9B08:2PP1=CDd0LG5a05]KF`28R8P0K6a/04=3@`1T
I6@0Cdm?051@D01]KFd0Jf][06a/K02>SXh0N7Qh07EeM@0c<c<0M7Ad04a<C01]KFd0Jf][00kooooo
08`0GEeM04e=C@1TI6@0IfMW07EeM@1aLG40IfMW08:2PP2a/K40^k^k079bLP1kNg/0IFEU03Xj>P15
ADD0Hf=S051@D011@D40Cdm?07AdM01OGel0Bd];07IfMP0S8b<0=SHf06a/K00n?Sh0GEeM09^KV`1X
J6P0OGem08Z:RP1<C4`0=CDe092@T01VIVH0GUiN06YZJP2BTY80a<C407YjNP19BDT0@d=3069RHP0R
8R80FEUI03dm?@24Q8@0SHf=06a/K012@T80@41002l_;`0`<300HV9R04a<C01ZJVX0OGem0:VYZ@24
Q8@0KFe]059BDP19BDT0XZ:R07AdM022PX80Z:RX0;6a/@29RHT0UiNG06YZJP1SHf<0=cLg04=3@`1O
Gel09bLW03<c<`1MGEd0Z:RX075aL@13@d<0<C4a08j>SP2MWId0VYZJ06][J`2DU9@0IVIV07YjNP1_
Kfl0Rh^;05ADE012@T80FEUI08:2PP20P800WinO08R8R00c<c<07QhN01dM7@0[:b/0OGem0:j^[P2;
Rh/0VYZJ0;2`/02NWYh0aLG50;Fe]@2m_Kd0Z:RX08^;R`19BDT0A4A4096AT@1^KVh0IVIV08n?S`2m
_Kd0WIfM02/[:`0S8b<092@T03Lg=`1DE5@0NGUi07IfMP1_Kfl0?Shn07]kN`2DU9@0Shn?07alO02C
Ti<0^[Zj0:b/[036a/H0YJFU08J6QQCooooo000Cooooo`1707emO@1cLg<0Lg=c08^;R`1PH600JFUY
05EEE@1CDe<0Hf=S04I6AP1_Kfl0KVi^05IFEP29RHT0EUIF05aLG01SHf<0CTi>06a/K01PH600DU9B
061PH01GEeL0F5QH07=cL`0g=cL0C4a<06a/K00m?Cd0ADE507MgM`0a<C40GEeM09JFUP0h>3P0G5aL
09BDU00Z:RX0MGEe061PH00g=cL0M7Ad05QHF01DE5@0LG5a06a/K011@D40N7Qh04U9B@1<C4`0MGEe
05aLG01ZJVX0C4a<04];B`1nOWh0LG5a05eMG@2<S8`0GUiN07MgM`26QXH0Jf][06UYJ@2LW9`0LG5a
05]KF`1oOgl0HV9R059BDP2BTY800P1QHF401020P800Ng]k07]kN`29RHT206EUI@0306=SH`1YJFT0
OGem0080GUiN03L0QhN705aLG025QHD0OGem0492@P1nOWh0QHF507=cL`1kNg/0WYjN05YJFP1fMWH0
SXj>07]kN`1`L700IFEU07MgM`1]KFd0A4A408n?S`1aLG40JVYZ07MgM`1`L700XZ:R05ADE01XJ6P0
Jf][07YjNP1EEED0O7al07YjNP1FEUH0LW9b06UYJ@16ATH0K6a/065QH@1DE5@0QXJ605EEE@1NGUh0
RHV905eMG@1RHV80H61P05]KF`1]KFd0C4a<04E5A@1_Kfl0HF5Q03De=@1KFe/0Ogmo00kooooo01D0
G5aL07AdM01YJFT0KFe]05=CD`1KFe/0HF5Q06a/K01fMWH0GEeM04I6AP1<C4`0Cdm?0492@P1UIFD0
FEUI05mOG`1WIfL0J6QX03Ph>00k>c/00P1IFET0B00j>SX0H61P04Y:BP1;Bd/0Cdm?055AD@18B4P0
GEeM04m?C`1<C4`0I6AT06e]K@1KFe/0MgMg05iNGP1CDe<0HF5Q07moO`1eMGD0De=C05aLG017AdL0
DE5A04=3@`18B4P0F5QH05ADE01JFUX0Cdm?051@D01:BTX0FUYJ05iNGP1LG5`0De=C05aLG01KFe/0
Kfm_05aLG01:BTX0@T9206ATI02:RXX0P82007inOP25QHD0DU9B06IVIP1GEeL0CDe=03dm?@1/K6`0
Hf=S04M7A`1CDe<0EEEE096AT@1PH600HV9R05mOG`1MGEd0G5aL04E5A@1FEUH0HF5Q04];B`1BDU80
Kfm_05YJFP1GEeL0CDe=075aL@80I6AT01/0G5aL05mOG`14A4@0I6AT07UiN@1EEED0AdM703/k>`1=
CDd0GUiN05EEE@1=CDd0F5QH06e]K@1EEED0IFEU069RHP1KFe/0FEUI04i>CP1RHV80PX:206i^KP1Z
JVX0EEEE05iNGP25QHD00P1YJFT03P1LG5`0QHF509^KV`2;Rh/0NWYj09RHV01LG5`0ATI605UIF@1R
HV80DU9B061PH01XJ6P0QXJ63_ooool03`1MGEd0EeMG07emO@0j>SX0BDU903hn?P11@D40@d=303dm
?@0k>c/06a/K02l_;`0/;2`0Cdm?051@D0020492@P1k02h^;P0b<S80@410020P8017AdL0B4Q8069R
HP1<C4`0GEeM02l_;`0n?Sh0BTY:04];B`1]KFd0GUiN02l_;`1SHf<0@d=305iNGP1KFe/0>3Ph07Yj
NP1?Cdl0MgMg0492@P0c<c<0G5aL03`l?01SHf<0UiNG061PH018B4P0EEEE04A4A01iNGT0?3`l04M7
A`1RHV80BTY:06][J`15ADD0C4a<069RHP1YJFT0F5QH069RHP1CDe<0KVi^04Q8B01^KVh0Cdm?071`
L01cLg<0IVIV06i^KP22PX80D51@06][J`1`L700IfMW06e]K@1NGUh0@d=30<;2`P2ATI40HF5Q0::R
XP2PX:00KFe]08f=S@1IFET0C4a<06=SH`25QHD0JFUY06QXJ01=CDd0Shn?06YZJP1QHF40>CTi08J6
QP1]KFd0H61P05ADE01mOGd0DU9B04=3@`1hN7P0F5QH04Q8B024Q8@0ADE505iNGP1FEUH0IFEU07em
O@1JFUX0H61P04e=C@19BDT0IfMW06i^KP0h>3P0JVYZ06UYJ@23Ph<0KVi^03`l?01bLW80?3`l04Q8
B01VIVH0EeMG05iNGP2>SXh0I6AT06IVIP0a<C40L71`034a<@1FEUH0MgMg00kooooo00X0FEUI05MG
E`1HF5P0GUiN08N7Q`1[Jf/0HF5Q09NGU`3@d=00Kfm_0P0W9bL0K@0];Bd0H61P04A4A01=CDd071`L
01dM7@0_;bl0KFe]05ADE00l?3`0E5AD0;>c/`2LW9`0WYjN079bLP1BDU809bLW01/K6`0L71`0FUYJ
051@D00e=CD0De=C09NGU`0Z:RX0:BTY02d];@14A4@0XJ6Q08N7Q`1ZJVX0YjNW08B4Q02YZJT0HV9R
06YZJP1oOgl0F5QH04M7A`0i>CT0:2PX02TY:@16ATH0C4a<02DU9@0c<c<0Jf][030`<01dM7@0ZJVY
09NGU`2IVIT0NWYj02d];@26QXH0NWYj04=3@`1ADE40L71`030`<01/K6`0EeMG07EeM@0c<c<0@T92
04e=C@1lO7`0=CDe02LW9`0T92@0RHV90;fm_@2[Zj/0ZZZZ0<K6aP20P800EEEE08B4Q00m?Cd0BDU9
07AdM02WYjL0CTi>079bLP21PH40YZJV08^;R`0V9RH0@d=309bLW02BTY80QhN70:>SX`19BDT0<c<c
06IVIP3Jf]X0];Bd06a/K019BDT0MgMg06UYJ@13@d<0IfMW028R8P0^;Rh0F5QH0<O7a`1nOWh00P0N
7Qh04@0O7al09RHV03Ph>01@D500Ogmo0<C4a02o_kl0W9bL09fMW@2EUID0UYJF096AT@2k^k/0`l?3
0:VYZ@2f][H0OGem01Cooooo000Cooooo`2;0861P@1dM7@0Kfm_05]KF`2;Rh/0?clo06YZJP1eMGD0
D51@08B4Q01fMWH0Fe]K06YZJP1VIVH0PH6104m?C`1FEUH0Ng]k05YJFP1HF5P0OGem05=CD`1@D500
QXJ605eMG@1JFUX0PH6105ADE01FEUH0F5QH04Q8B01NGUh0B4Q804m?C`1VIVH0@41006MWI`1@D500
IFEU05aLG01BDU80H61P07AdM01bLW80D51@05iNGP1oOgl0CDe=05ADE01cLg<0?3`l05EEE@1LG5`0
>c/k06EUI@28R8P0<C4a05YJFP1fMWH0C4a<08J6QP1/K6`0IVIV08:2PP1LG5`0LW9b06a/K01MGEd0
TY:B05YJFP1aLG40O7al069RHP1aLG40QHF5069RHP1[Jf/0Q8B4059BDP1GEeL0Q8B405ADE01LG5`0
SHf=065QH@1RHV80UiNG05IFEP1cLg<0M7Ad04e=C@1fMWH0Jf][06IVIP2=SHd0AdM707QhN01jNWX0
ADE508Z:RP1_Kfl0K6a/08B4Q01`L700KFe]071`L01JFUX0J6QX09BDU01DE5@0N7Qh05IFEP1eMGD0
Jf][07YjNP2CTi<0HF5Q06QXJ02PX:00BTY:069RHP28R8P0FEUI07moO`29RHT0GUiN06UYJ@1_Kfl0
F5QH07MgM`21PH40C4a<06IVIP20P800Fe]K05mOG`1ZJVX0FUYJ069RHP0>ooooo`0A04M7A`1ZJVX0
IVIV079bLP1DE5@0H61P05eMG@1QHF40JFUY05aLG00l?3`0ATI60492@P0i>CT0Fe]K061PH01RHV80
0P1ZJVX01`1;Bd/0@410061PH01CDe<0ADE506QXJ016ATH00P1@D500801DE5@0C4a<05aLG01?Cdl0
FEUI06UYJ@1eMGD0IVIV07emO@1JFUX0BDU905aLG021PH40LG5a05]KF`1PH600D51@06EUI@1CDe<0
EUIF04e=C@1>CTh0De=C04a<C01JFUX0BDU905YJFP1TI6@0K6a/06IVIP1JFUX0DU9B0P1]KFd02P1S
Hf<0BDU906MWI`1KFe/0Ph>308:2PP1lO7`0Cdm?05mOG`1MGEd203`l?00906QXJ01^KVh0HV9R05QH
F01kNg/0];Bd07emO@1KFe/0Lg=c0080IFEU01@0Cdm?05QHF01aLG40EeMG055AD@1ZJVX0DU9B05AD
E01LG5`0H61P07MgM`1SHf<0H61P06UYJ@1>CTh0L71`07moO`1MGEd0DE5A04a<C080FUYJ0240D51@
05=CD`1GEeL0KFe]06a/K01SHf<0HF5Q06MWI`1aLG40FUYJ061PH01oOgl0OWin04E5A@1FEUH0HV9R
079bLP1mOGd0QHF506MWI`28R8P0Vi^K06i^KP1jNWX0S8b<06UYJ@1=CDd0IFEU05mOG`1CDe<0Hf=S
06i^KP28R8P03_ooool0S01SHf<0FUYJ08F5Q@0h>3P0AdM704A4A01=CDd0EeMG03lo?`0m?Cd09BDU
03<c<`0_;bl0E5AD05iNGP1WIfL0@D5103lo?`0e=CD0Cdm?034a<@0];Bd0CDe=06=SH`0k>c/0G5aL
03/k>`0d=3@0?clo04I6AP1ZJVX0CDe=03<c<`1BDU80CTi>05]KF`11@D40<30`0861P@1RHV80MWIf
05YJFP19BDT0De=C03De=@1jNWX0UiNG075aL@1VIVH0Cdm?05aLG01WIfL0BTY:06MWI`1ADE40Jf][
05iNGP1<C4`0JFUY05UIF@1/K6`0EeMG06YZJP1IFET0IFEU06m_K`1RHV80<S8b061PH027QhL0Kfm_
06UYJ@2ATI40D51@075aL@1XJ6P0RXZ:06MWI`2@T900K6a/0861P@1[Jf/0Lg=c07emO@1aLG40NWYj
08N7Q`1KFe/0ATI605YJFP24Q8@0EeMG06][J`1YJFT0S8b<06ATI01hN7P0Bd];079bLP1`L700Ogmo
07=cL`1lO7`0DU9B06UYJ@1^KVh0F5QH06=SH`20P800?3`l07moO`1MGEd0I6AT07moO`1[Jf/0GUiN
05MGE`1;Bd/0FEUI07EeM@1?Cdl0FEUI05eMG@2?Shl0MGEe04Q8B01ADE40GEeM06e]K@1UIFD0Kfm_
079bLP26QXH0JFUY08J6QP11@D40Hf=S03dm?@1WIfL0Hf=S3_ooool04@1?Cdl0<C4a01lO7`0J6QX0
7AdM07moO`2IVIT0NWYj096AT@2PX:00IFEU030`<00b<S80<C4a02TY:@0R8R806a/K00807AdM0140
=CDe03lo?`0g=cL0S8b<08R8R013@d<0C4a<07IfMP1FEUH0;2`/02DU9@0M7Ad0IfMW04E5A@1XJ6P0
=SHf06IVIP0201`L700X01dM7@0O7al0BTY:08Z:RP3Hf=P0M7Ad038b<P13@d<0Ng]k0:b/[02d];@0
?Cdm02<S8`0X:2P0ATI604E5A@2EUID0YjNW034a<@0V9RH0UiNG08R8R01OGel0Kfm_05EEE@1QHF40
LW9b05mOG`1ADE40GemO02`/;00b<S80O7al04I6AP1>CTh0TI6A09RHV01UIFD0Ng]k05EEE@80DE5A
01L0;Rh^03/k>`2[Zj/0QhN708>3P`2_[jl0^k^k0;Bd]020P800WYjN02h^;P0Z:RX0N7Qh0:FUY@32
`/80HV9R02<S8`2e]KD0];Bd092@T02f][H0XZ:R08>3P`0203Xj>P0P05QHF019BDT0NGUi0=CDe02H
V9P0CTi>04m?C`1EEED0OGem07UiN@1[Jf/0:b/[03<c<`2VYZH0/k>c07]kN`0i>CT0;Bd]02@T900Q
8B408R8R03Xj>P2ZZZX0aLG50<k>cP2[Zj/0UYJF07]kN`1SHf<0W9bL0:j^[P2IVIT20;Vi^@030<30
`?oooooooooo01;ooooo000Cooooo`1R08^;R`1hN7P0Ng]k0861P@1FEUH0MgMg06][J`1HF5P0QhN7
04m?C`25QHD0Lg=c03/k>`2:RXX0H61P05ADE01/K6`0FUYJ06MWI`1KFe/0F5QH05UIF@1BDU80FUYJ
06QXJ01>CTh0B4Q806=SH`19BDT0BTY:07QhN01ADE40CDe=08n?S`16ATH0AdM708n?S`0j>SX0BDU9
08>3P`0a<C40IfMW065QH@11@D40NWYj06UYJ@10@400L71`04i>CP1?Cdl0HV9R06a/K01WIfL0EeMG
05EEE@1eMGD0B4Q801lO7`1ZJVX0Jf][05mOG`1mOGd0LW9b07EeM@2JVYX0LG5a059BDP1mOGd0IfMW
051@D022PX80LG5a06ATI01hN7P0QXJ606IVIP21PH40NGUi065QH@1/K6`0LW9b071`L01JFUX0GUiN
07]kN`18B4P0KFe]07EeM@18B4P0MgMg07UiN@1KFe/0P8200861P@1?Cdl0Ph>3075aL@1IFET2071`
L080JFUY01H0@D5107QhN01oOgl0CTi>075aL@1PH600QHF5071`L01LG5`0SHf=071`L015ADD0TY:B
07UiN@1<C4`0KVi^06EUI@1HF5P0PH6107moO`1?Cdl0YjNW0P1OGel03@29RHT0DE5A065QH@1SHf<0
De=C05ADE01[Jf/0CTi>06MWI`1aLG40E5AD05eMG@2>SXh03_ooool0:P1GEeL0MWIf06IVIP1XJ6P0
EEEE06ATI01=CDd0IfMW05UIF@1TI6@0BTY:05aLG01CDe<0<c<c05mOG`1LG5`0H61P06i^KP1UIFD0
DU9B03/k>`1IFET0FUYJ055AD@1fMWH0EEEE05YJFP1OGel0G5aL04a<C01MGEd0DE5A05YJFP1LG5`0
KFe]05UIF@1bLW80I6AT05EEE@1DE5@0Rh^;079bLP80GUiN0600Bd];06QXJ017AdL0Fe]K04e=C@1C
De<0CDe=04U9B@1;Bd/0FUYJ06=SH`1UIFD0N7Qh06][J`1IFET0H61P05=CD`1fMWH0PX:207MgM`1c
Lg<0Ogmo08>3P`25QHD0MGEe05MGE`0i>CT0Cdm?03`l?00k>c/0IfMW06=SH`1>CTh0FEUI06IVIP2X
Z:P0K6a/05YJFP1RHV80DE5A06a/K01>CTh0D51@06IVIP1?Cdl0@T9206ATI01DE5@0Fe]K069RHP1D
E5@0IfMW05eMG@1PH600GUiN05=CD`1`L700MWIf065QH@1DE5@0>3Ph05UIF@1QHF40CDe=059BDP1E
EED0JVYZ06][J`1>CTh0H61P06a/K01ZJVX0Hf=S05eMG@1hN7P0K6a/04m?C`1EEED0CDe=05aLG01]
KFd0Ogmo06ATI02MWId0R8R807]kN`2JVYX0WIfM06e]K@1ADE40JVYZ061PH01BDU80G5aL06IVIP1f
MWH>ooooo`80DU9B00D0Hf=S034a<@0f=SH0=SHf02l_;`020492@P0B030`<00P8200@41002@T9015
ADD0Bd];05=CD`19BDT0>c/k03hn?P16ATH0;Rh^04Y:BP15ADD0K6a/06ATI018B4P0<30`0P0k>c/0
K`1=CDd0OWin051@D00a<C40GUiN04E5A@1bLW80C4a<04e=C@1/K6`0DE5A09>CT`10@400ATI605EE
E@0f=SH0KVi^08V9R@1mOGd0FEUI05aLG01JFUX0NGUi05IFEP1PH600GemO06=SH`1VIVH0BTY:04];
B`1TI6@0G5aL05MGE`1NGUh0De=C07moO`1PH600IVIV059BDP1YJFT0NGUi06=SH`20P800MgMg051@
D01YJFT0IVIV08j>SP1@D500MWIf05EEE@2?Shl0HF5Q06EUI@25QHD0MGEe06=SH`2?Shl0EeMG03@d
=01EEED0MgMg05iNGP1]KFd0?3`l08j>SP1HF5P0HF5Q05eMG@1kNg/0MGEe065QH@18B4P0O7al04U9
B@1ADE40M7Ad061PH01HF5P0TI6A05iNGP1VIVH0Fe]K07AdM01lO7`0H61P079bLP1;Bd/0A4A406MW
I`1RHV80Cdm?071`L01bLW80M7Ad07YjNP1/K6`0J6QX04M7A`1aLG40Kfm_06UYJ@1ZJVX0QXJ607If
MP20P800:BTY05MGE`0m?Cd0IFEU07MgM`0>ooooo`1o04U9B@0j>SX0>3Ph04U9B@1:BTX0E5AD09jN
WP2<S8`0P82004m?C`1TI6@0L71`04m?C`0W9bL07alO01DE5@0D51@06QXJ01/K6`0P8200DE5A02h^
;P1HF5P0EUIF03`l?010@400MWIf055AD@1VIVH0Hf=S051@D02SXj<0L71`06a/K01QHF40QXJ601dM
7@0L71`07AdM01hN7P1KFe/0WIfM0;Rh^01iNGT0EUIF06UYJ@1HF5P0RXZ:092@T01TI6@0?Cdm01lO
7`0V9RH0HV9R051@D024Q8@0K6a/07EeM@1gMgL0C4a<04U9B@1TI6@0=SHf03lo?`1MGEd0@41003dm
?@1TI6@09bLW02<S8`1DE5@0;bl_06IVIP2ATI40O7al0451@@0d=3@0GEeM0861P@2^[Zh0NWYj0:ZZ
ZP1SHf<0HV9R0:NWY`1oOgl0M7Ad03Ti>@0l?3`0Hf=S07]kN`1@D500K6a/05aLG023Ph<08R8R08:2
PP2WYjL0`<300<O7a`2j^[X0Hf=S06ATI00k>c/071`L02`/;02GUiL0P8200<g=c@2a/K40C4a<04Q8
B01NGUh0];Bd09jNWP2;Rh/0P82006MWI`2h^;P0_;bl08>3P`17AdL0@T9204=3@`1QHF40Ng]k07Qh
N0020<K6aP0;08F5Q@0e=CD0?3`l07AdM03IfMT0SXj>0451@@0j>SX0OWin0;fm_@25QHD05?ooool0
01?ooooo04L0SXj>07AdM029RHT0K6a/07EeM@1HF5P0Lg=c06YZJP1RHV80G5aL07UiN@1HF5P0EeMG
0820P01ADE40FUYJ06UYJ@1BDU80GUiN05mOG`1KFe/0HF5Q05eMG@1YJFT0IfMW059BDP1TI6@0FUYJ
04i>CP1GEeL0IVIV04I6AP1>CTh0NGUi059BDP16ATH0JVYZ055AD@1ZJVX0L71`05MGE`1^KVh0GEeM
06EUI@1TI6@0S8b<07AdM01GEeL0GUiN0861P@1BDU80K6a/06][J`11@D40DU9B08:2PP15ADD0De=C
07]kN`1BDU80JVYZ07UiN@1/K6`0Ng]k07EeM@1]KFd0HV9R06][J`1mOGd0D51@07AdM00306m_K`0?
07alO01dM7@0E5AD07emO@1XJ6P0E5AD07]kN`1`L700EUIF08Z:RP1^KVh0ATI608N7Q`1gMgL0C4a<
0080MGEe02@0LW9b08>3P`1gMgL0K6a/06i^KP1bLW80JVYZ065QH@1jNWX0QXJ6079bLP1UIFD0IfMW
07=cL`26QXH0HV9R0820P01oOgl0DU9B06i^KP26QXH0K6a/065QH@25QHD0O7al05mOG`1dM7@0KFe]
051@D01OGel0KVi^04a<C01nOWh0HV9R065QH@1fMWH206i^KP0:07IfMP1OGel0CTi>05iNGP1FEUH0
D51@06UYJ@1RHV80E5AD07]kN`kooooo08`0F5QH06a/K01WIfL0K6a/04];B`1TI6@0FUYJ07AdM01W
IfL0F5QH03/k>`19BDT0DE5A034a<@1SHf<0DE5A051@D01MGEd0JFUY04I6AP10@400DE5A05YJFP14
A4@0IFEU04U9B@1BDU80EUIF05aLG01>CTh0E5AD05eMG@1?Cdl0HF5Q07AdM01QHF40LW9b05IFEP1D
E5@0EeMG08V9R@1]KFd0EUIF06EUI@1EEED0JFUY05EEE@1QHF40DE5A05QHF01KFe/0EEEE059BDP1S
Hf<0JVYZ075aL@22PX80P82006a/K01^KVh0NGUi07emO@21PH40QHF507MgM`1eMGD0Ogmo07QhN01e
MGD0EUIF05ADE01XJ6P0Cdm?04A4A01iNGT0LW9b05YJFP1QHF40L71`0:6QX@1PH600Hf=S06][J`1U
IFD0KVi^04E5A@1XJ6P0HV9R05=CD`1<C4`0Jf][05IFEP1HF5P0GemO05QHF01aLG40E5AD06ATI01P
H600EUIF05QHF01LG5`0E5AD04];B`0/;2`0CDe=05]KF`1LG5`0FEUI055AD@1OGel0H61P05ADE01P
H600FUYJ05aLG01BDU80KFe]07inOP1dM7@0GEeM05]KF`1RHV80GemO07moO`24Q8@0NWYj096AT@2J
VYX0Xj>S09RHV023Ph<0Jf][05EEE@1RHV80IFEU05EEE@1dM7@0HV9R06m_K`kooooo00<0LG5a05UI
F@1WIfL00P15ADD0Q`0h>3P0D51@0451@@1<C4`0=3@d02d];@0n?Sh0;bl_04A4A01;Bd/0ATI604];
B`0k>c/0>SXj0410@00`<300ADE503Xj>P1NGUh0E5AD04E5A@11@D40BDU903Ph>016ATH0N7Qh055A
D@0n?Sh0Bd];055AD@1MGEd0AdM703lo?`1CDe<0C4a<08n?S`1=CDd0C4a<04m?C`10@400LG5a07If
MP1]KFd0HF5Q04a<C01MGEd0FUYJ05ADE01dM7@0DE5A071`L01WIfL0CDe=05iNGP1WIfL0HF5Q061P
H01[Jf/0GUiN06QXJ01oOgl0LW9b04I6AP1SHf<0TI6A05aLG01eMGD0Ti>C05]KF`1cLg<0I6AT07Mg
M`1XJ6P0OWin09NGU`2SXj<0FEUI06m_K`1nOWh0I6AT06m_K`2CTi<0FEUI04Y:BP1EEED0R8R805YJ
FP1UIFD0FUYJ08:2PP1:BTX0IFEU051@D01/K6`0IfMW06QXJ017AdL0Kfm_04U9B@1FEUH0JFUY06=S
H`1MGEd0N7Qh0492@P1/K6`0FUYJ065QH@1aLG40JVYZ06EUI@1@D500De=C05EEE@1LG5`0GEeM055A
D@1OGel0OWin07moO`1FEUH0EeMG04Y:BP1QHF40E5AD065QH@1eMGD0NWYj06UYJ@2BTY80<c<c05YJ
FP1ADE40J6QX05aLG00>ooooo`80Cdm?04T0C4a<07=cL`1FEUH0Bd];02/[:`1?Cdl0ZZZZ0:NWY`2H
V9P0YZJV08n?S`0c<c<06QXJ01hN7P0F5QH05aLG01/K6`0N7Qh0DE5A08n?S`2;Rh/0DE5A03<c<`0h
>3P08R8R01lO7`0^;Rh0D51@03hn?P1WIfL0K6a/08F5Q@24Q8@0/K6a06][J`0n?Sh0BTY:05MGE`1h
N7P0^KVi0<K6aP2o_kl0a<C40:JVYP1VIVH0C4a<07IfMP2SXj<0NWYj06m_K`1gMgL0RHV906QXJ02h
^;P0P82009nOW`2ZZZX0RHV907=cL`1hN7P0;bl_03De=@1hN7P0EeMG08f=S@29RHT0@D5103<c<`1c
Lg<0Kfm_06=SH`19BDT0@4100080>c/k03l0=cLg06MWI`2;Rh/0Xj>S0:VYZ@1`L700N7Qh05=CD`1U
IFD0JFUY020P800T92@0=CDe04a<C02CTi<0Ph>308:2PP2/[:`0[Zj^06e]K@1YJFT0MWIf08>3P`2P
X:00J6QX061PH01mOGd0HF5Q05mOG`1eMGD0M7Ad0;Rh^01;Bd/0J6QX079bLP1gMgL0QHF509RHV026
QXH0VYZJ09BDU01hN7P0Ph>307emO@26QXH0Shn?09fMW@2CTi<0NWYj092@T02f][H0WIfM092@T01A
DE4092@T02l_;`28R8P0JFUY02/[:`0f=SH0OWin09FEU@1MGEd05?ooool001?ooooo01`0N7Qh071`
L01YJFT0Hf=S061PH01CDe<0Lg=c06IVIP1QHF40Fe]K06UYJ@1`L700ATI6061PH01cLg<0?Shn05MG
E`1`L700De=C06m_K`1gMgL0A4A406a/K01]KFd0ATI607=cL`1KFe/0=SHf0P1GEeL0K@14A4@0H61P
0410@01CDe<0Jf][03hn?P1;Bd/0J6QX065QH@1MGEd0I6AT05iNGP28R8P0JFUY04a<C02CTi<0NWYj
04U9B@2?Shl0MWIf04];B`1kNg/0Lg=c04i>CP1kNg/0Hf=S05]KF`1gMgL0QHF506MWI`1gMgL0Jf][
06MWI`24Q8@0H61P092@T01OGel0HV9R08Z:RP1:BTX0G5aL07inOP1LG5`0Jf][0:BTY01@D500MgMg
09fMW@1@D500Kfm_092@T01<C4`0Ogmo0:6QX@1DE5@0HF5Q09fMW@1FEUH0IFEU06QXJ01=CDd0L71`
06a/K01aLG40Ph>3055AD@2>SXh0NGUi055AD@28R8P0L71`08n?S`1XJ6P0EUIF08f=S@1bLW80?clo
08>3P`1gMgL0I6AT05eMG@1RHV80NGUi05MGE`1TI6@0QHF504Q8B01WIfL0SXj>04U9B@1ZJVX0R8R8
05ADE01bLW80QhN704m?C`21PH40KFe]06ATI01iNGT0IVIV055AD@1TI6@0LG5a04e=C@1YJFT0K6a/
03Hf=P1nOWh03_ooool01`1FEUH0LG5a05eMG@1JFUX0Cdm?059BDP1CDe<00P1UIFD0601EEED0AdM7
055AD@1ADE40A4A405eMG@1>CTh0BTY:051@D01OGel0@41002LW9`13@d<0Cdm?04U9B@1VIVH0AdM7
04e=C@19BDT0CTi>05ADE01?Cdl0FUYJ05=CD`80HV9R03X0IfMW06m_K`1DE5@0E5AD05QHF01dM7@0
HF5Q04e=C@1HF5P0Bd];055AD@1FEUH0FUYJ04E5A@1HF5P0H61P051@D01CDe<0IFEU07MgM`1TI6@0
Kfm_07AdM01JFUX0IVIV0820P01gMgL0QHF50861P@1_Kfl0RHV908R8R02EUID0MgMg05iNGP1DE5@0
G5aL04];B`0d=3@0JFUY05]KF`1HF5P0C4a<075aL@2OWil0LG5a06EUI@1WIfL0DE5A05QHF01FEUH0
Jf][06MWI`18B4P0DE5A07EeM@1ADE40Cdm?0P1NGUh0601/K6`0J6QX061PH01RHV80FUYJ05iNGP1L
G5`0Bd];05MGE`0j>SX0HV9R06EUI@1NGUh0EEEE05YJFP1]KFd0F5QH04U9B@1YJFT0H61P06EUI@1J
FUX0JVYZ07]kN`80H61P01<0DE5A06YZJP1?Cdl0K6a/08^;R`1PH600Rh^;0861P@2ATI40]KFe09fM
W@1dM7@0DU9B06][J`1/K6`0F5QH06a/K01`L700LW9b00kooooo01`0HV9R05YJFP1IFET0<c<c04i>
CP0W9bL0DE5A034a<@0i>CT0>SXj020P800b<S809bLW03De=@15ADD0ATI603Lg=`0a<C40:b/[04];
B`0Z:RX0Cdm?03Lg=`0m?Cd0Jf][04Y:BP0X:2P0AdM70P0j>SX07@1/K6`0Bd];05QHF016ATH0GEeM
04m?C`1EEED0GUiN03/k>`1BDU80UIFE059BDP17AdL0HV9R059BDP1^KVh0NGUi07alO01NGUh0GemO
05aLG01^KVh0I6AT0861P@1OGel0JFUY06IVIP1EEED0Fe]K0080H61P0340Fe]K06=SH`1CDe<0Fe]K
08V9R@1PH600E5AD06e]K@1nOWh0Lg=c07AdM01iNGT0DE5A07=cL`1LG5`0IfMW065QH@1/K6`0OWin
09JFUP1CDe<0Lg=c07UiN@1XJ6P0Hf=S09jNWP1DE5@0AdM704i>CP2>SXh0EEEE069RHP1MGEd0PH61
04e=C@1XJ6P0FEUI079bLP24Q8@0KFe]05iNGP1lO7`0FUYJ06QXJ01eMGD0KVi^06][J`1gMgL0IVIV
0080FUYJ01`0KFe]07=cL`1DE5@0SHf=05IFEP1BDU80Hf=S05IFEP1PH600Hf=S06EUI@1QHF40Hf=S
071`L01PH600?Cdm07=cL`1BDU80?Shn061PH022PX80FUYJ05eMG@0c<c<0HF5Q04e=C@1IFET0Fe]K
3_ooool06`1MGEd0CTi>05eMG@1ZJVX0EeMG06e]K@0U9BD0A4A4065QH@17AdL0E5AD07=cL`2PX:00
TI6A069RHP15ADD0<30`024Q8@0_;bl0G5aL06UYJ@2:RXX0UIFE08R8R02XZ:P07AdM01@D500201HF
5P0>05=CD`27QhL0TY:B05iNGP1OGel0M7Ad04E5A@0e=CD0EEEE05UIF@23Ph<0WinO09FEU@39bLT2
079bLP0_0:>SX`0l?3`0=SHf01lO7`1eMGD0?Cdm06e]K@2LW9`0LW9b08f=S@2DU9@0Fe]K09FEU@2j
^[X0Xj>S04M7A`1NGUh0HF5Q034a<@1oOgl0Kfm_07emO@2a/K40L71`07AdM02TY:@0VIVI08>3P`0`
<300D51@02`/;00c<c<0De=C02PX:012@T80Shn?03hn?P0a<C40Shn?05IFEP1XJ6P0DU9B03Xj>P0M
7Ad0;2`/05UIF@1jNWX00P2b/[80;P2e]KD0XZ:R0492@P1BDU80PX:208>3P`2:RXX0OGem05mOG`1n
OWh0SXj>0=CDe034a<@0bl_;09VIV@0o?cl0=3@d07=cL`1GEeL0O7al06MWI`1YJFT0TY:B07=cL`1C
De<0B4Q808:2PP2PX:00CDe=03@d=00c<c<0>SXj06a/K02GUiL0CDe=05=CD`11@D40C4a<08B4Q01f
MWH0GEeM02`/;014A4@0WinO06QXJ02LW9`Dooooo`004oooool05`23Ph<0K6a/06m_K`20P800D51@
05]KF`1WIfL0IFEU07]kN`19BDT0N7Qh05aLG01<C4`0MgMg04e=C@1CDe<0NGUi04];B`1PH600Hf=S
04e=C@1MGEd0HV9R0080Fe]K06/0A4A405ADE01QHF40?clo05IFEP1dM7@0?3`l05mOG`1kNg/0>c/k
06MWI`1oOgl0<C4a06MWI`1/K6`0>SXj075aL@1dM7@0ADE506EUI@1]KFd0De=C06ATI01BDU80IVIV
05QHF01@D500OGem05ADE01@D500NWYj05EEE@1CDe<0MgMg059BDP1EEED0QXJ605]KF`1NGUh0OWin
06MWI`1<C4`0Lg=c065QH@1MGEd0PH6106ATI01QHF40N7Qh07]kN`1iNGT0K6a/07YjNP1`L700FUYJ
07UiN@1mOGd0ADE50861P@1mOGd0>CTi07alO023Ph<0CTi>06QXJ029RHT0D51@08j>SP1lO7`0E5AD
092@T01ZJVX0KFe]07alO01HF5P0LW9b08:2PP1IFET0JVYZ065QH@2<S8`0HV9R05iNGP2HV9P0GemO
05aLG02HV9P0JVYZ04e=C@2EUID0IfMW0451@@1dM7@0EUIF05ADE01QHF40I6AT05mOG`2BTY80DU9B
06i^KP1iNGT0E5AD07IfMP1VIVH0Fe]K05eMG@0204E5A@0506QXJ01DE5@0ADE507EeM@1TI6@03_oo
ool0C@1GEeL0EUIF05EEE@1NGUh0B4Q805UIF@1WIfL0R8R806EUI@1ZJVX0?3`l05MGE`1CDe<0ATI6
06UYJ@1MGEd0GUiN055AD@1@D500ATI604=3@`1;Bd/0DU9B04U9B@1fMWH0@41004m?C`1=CDd0Bd];
04A4A00m?Cd0F5QH04i>CP1HF5P0EeMG05=CD`1`L700ADE50492@P1EEED0Lg=c04U9B@0k>c/0@D51
03Lg=`0o?cl0ADE504Q8B00m?Cd0De=C04i>CP12@T80?Shn04Y:BP1HF5P0D51@05iNGP1[Jf/0EeMG
06ATI01gMgL0Jf][07AdM01nOWh0NWYj06m_K`1bLW80RHV907emO@1TI6@0@D5106MWI`1VIVH0DE5A
08>3P`1aLG40GemO0080LG5a03d0YZJV05QHF01IFET0KFe]06i^KP1UIFD0C4a<05mOG`1[Jf/0GemO
05UIF@1[Jf/0GEeM05MGE`1]KFd0GUiN05YJFP1WIfL0C4a<06=SH`1NGUh0LG5a05YJFP1OGel0Cdm?
03Ph>01EEED0FUYJ05]KF`1@D500OGem071`L01nOWh0G5aL079bLP1QHF40GemO05MGE`1VIVH0J6QX
05aLG01UIFD0HV9R04];B`19BDT0Kfm_07UiN@1fMWH0Rh^;06IVIP1[Jf/0X:2P07emO@1[Jf/0Fe]K
04a<C01[Jf/0B4Q807UiN@1ZJVX0OWin00kooooo0700OWin07=cL`1VIVH0EeMG05YJFP0o?cl0Jf][
03lo?`16ATH0>c/k03`l?00b<S80<C4a04U9B@18B4P0HF5Q04=3@`0c<c<0>c/k04];B`16ATH0Bd];
04i>CP1FEUH0Hf=S05UIF@17AdL0G5aL0492@P11@D40MGEe030`<01[Jf/0C4a<05YJFP1BDU80Cdm?
05YJFP0m?Cd0N7Qh08>3P`1=CDd0IVIV05YJFP14A4@0Ogmo08Z:RP1mOGd0Kfm_04Y:BP1[Jf/0FUYJ
04m?C`1fMWH0CDe=06UYJ@1LG5`0DU9B06ATI01QHF40H61P075aL@1LG5`0FUYJ06ATI01eMGD0IFEU
04a<C01VIVH0HV9R07alO01eMGD0SHf=06a/K01jNWX0Lg=c06QXJ01gMgL0O7al08f=S@2TY:@0DU9B
07alO01eMGD0H61P07IfMP2<S8`0GUiN05EEE@1BDU80RXZ:059BDP1IFET0IVIV07MgM`0o?cl0GEeM
05EEE@17AdL0IfMW08N7Q`18B4P0DE5A059BDP1KFe/0EUIF07MgM`1FEUH0H61P04=3@`1KFe/0G5aL
0P1VIVH06P1KFe/0M7Ad051@D01HF5P0IFEU04e=C@1SHf<0EUIF05eMG@1VIVH0C4a<065QH@1RHV80
@T9207MgM`19BDT0DU9B05QHF01oOgl0GEeM07emO@0[:b/0EEEE04A4A01MGEd0DU9B3_ooool0@P18
B4P0DE5A07inOP1JFUX0>SXj03De=@1TI6@0PX:205YJFP0T92@0=CDe02XZ:P2?Shl0NWYj06QXJ01/
K6`0MWIf07UiN@1`L700IVIV05mOG`25QHD0XJ6Q06][J`1iNGT07AdM01DE5@0K6a/07alO02/[:`13
@d<0IfMW0:BTY02GUiL0=CDe01dM7@0O7al09bLW06e]K@2SXj<0WinO07MgM`19BDT0EUIF051@D01[
Jf/0JFUY034a<@0P8200BDU904];B`1mOGd0FUYJ065QH@1^KVh0;Rh^02/[:`0o?cl0PH6109:BTP1l
O7`0I6AT09bLW02HV9P0^k^k0;>c/`80WinO04P0Lg=c08R8R01HF5P0Rh^;08Z:RP2<S8`0Vi^K051@
D01FEUH0:RXZ01dM7@13@d<0N7Qh030`<00U9BD0;Bd]06i^KP1ZJVX0ADE508J6QP1FEUH0Cdm?0410
@01FEUH0aLG50<O7a`2@T900XZ:R05mOG`28R8P0Xj>S0451@@1WIfL0JFUY04U9B@2ATI40bl_;0<K6
aP2FUYH0^KVi0<o?c`1lO7`0Bd];07QhN01/K6`0H61P08b<S01eMGD0Ng]k04];B`1EEED0ADE5092@
T02IVIT0>SXj03Hf=P1>CTh0M7Ad0:6QX@1BDU80=CDe0492@P18B4P0BDU904];B`1VIVH0CDe=02TY
:@25QHD0Xj>S05eMG@13@d<Dooooo`004oooool00`2FUYH0NWYj07=cL`0206a/K00>04m?C`1YJFT0
OGem05UIF@1[Jf/0OWin05IFEP1XJ6P0HF5Q05mOG`1FEUH0IFEU05MGE`18B4P207AdM00I05aLG01A
DE40NWYj05iNGP0j>SX0Kfm_05MGE`1BDU80G5aL06a/K01DE5@0H61P06][J`1NGUh0Kfm_06UYJ@13
@d<0I6AT06=SH`1BDU80HV9R07=cL`1TI6@0GEeM071`L00206=SH`1K06ATI01hN7P0BDU9069RHP1^
KVh0CDe=05aLG01iNGT0DU9B06QXJ02CTi<0CTi>06e]K@1hN7P0EEEE071`L01]KFd0Kfm_04a<C01M
GEd0Kfm_05ADE01/K6`0Ng]k065QH@1`L700PX:206YZJP1MGEd0NWYj06m_K`1RHV80NWYj075aL@1K
Fe/0Ph>306QXJ01?Cdl0NGUi06=SH`1IFET0GUiN051@D01QHF40OGem06UYJ@1_Kfl0LG5a06i^KP1T
I6@0DE5A08J6QP1bLW80IFEU06e]K@1PH600KFe]071`L01ADE40HF5Q07EeM@1@D500Fe]K06YZJP1`
L700DU9B07inOP1iNGT0DE5A05mOG`1kNg/0EEEE06MWI`1/K6`0DE5A07QhN01SHf<0EEEE06m_K`1[
Jf/0IVIV0861P@1TI6@0EUIF06UYJ@1RHV80DE5A07MgM`1IFET0EEEE06i^KP0>ooooo`07061PH01X
J6P0Fe]K06ATI01@D500DU9B05mOG`0206][J`0_05iNGP0m?Cd0De=C04e=C@14A4@0H61P05EEE@1F
EUH0F5QH06IVIP19BDT0A4A4059BDP1EEED0ATI606EUI@1=CDd0EUIF04];B`1JFUX0Fe]K05=CD`1U
IFD0Cdm?06=SH`1[Jf/0J6QX08V9R@17AdL0Bd];07IfMP2KVi/0Hf=S04U9B@1=CDd0A4A404Q8B01<
C4`0CDe=04Y:BP1;Bd/0Cdm?04=3@`14A4@0F5QH05EEE@1>CTh00P1FEUH0=@1>CTh0AdM705MGE`1X
J6P0G5aL07UiN@1bLW80IfMW075aL@1OGel0LG5a06][J`1`L700HF5Q05IFEP11@D40OGem06][J`1M
GEd0F5QH06a/K02g]kL0M7Ad06MWI`1YJFT0KVi^06MWI`1KFe/0N7Qh07EeM@1SHf<0EUIF07UiN@1H
F5P0EeMG06QXJ01UIFD0L71`075aL@1LG5`0Jf][075aL@1cLg<0K6a/05MGE`1=CDd0?3`l055AD@1M
GEd0Fe]K05QHF01TI6@0OWin0080HF5Q01/0K6a/04Y:BP10@400De=C06IVIP1GEeL0C4a<06ATI01@
D500IVIV03Hf=P1CDe<0G5aL05MGE`1fMWH0K6a/08>3P`2=SHd0S8b<092@T01PH600GUiN079bLP1R
HV80N7Qh07YjNP25QHD03_ooool206YZJP0705QHF01CDe<0C4a<02l_;`1MGEd0?3`l03Ph>002038b
<P2102`/;00Z:RX0>3Ph0451@@1>CTh0;Rh^02@T900Z:RX0@D5104U9B@0j>SX0CTi>04U9B@1bLW80
ATI604A4A01<C4`0Bd];04Y:BP1XJ6P0?Shn059BDP1DE5@0EeMG04Y:BP1MGEd0MWIf03Ti>@1PH600
HF5Q05EEE@1<C4`0I6AT04m?C`1bLW80NGUi08V9R@1TI6@0HV9R05]KF`1NGUh0G5aL06a/K01OGel0
FUYJ051@D01NGUh0FEUI05]KF`1ADE40LG5a059BDP1RHV80H61P08>3P`1EEED0IVIV06MWI`1GEeL0
OWin075aL@29RHT0IfMW06UYJ@1SHf<0JVYZ07MgM`1jNWX0I6AT0:NWY`1HF5P0L71`08>3P`1/K6`0
MGEe09RHV01SHf<0De=C059BDP1jNWX0Bd];05EEE@1ZJVX0MWIf04m?C`1YJFT0I6AT04M7A`1_Kfl0
NGUi05aLG014A4@0G5aL06UYJ@1OGel0Hf=S069RHP1YJFT0GUiN055AD@1OGel0LG5a06QXJ01HF5P0
Ng]k059BDP1>CTh0I6AT04I6AP1[Jf/0C4a<06QXJ01IFET0EeMG06][J`1/K6`0ADE505]KF`15ADD0
GUiN05QHF01oOgl0JVYZ07YjNP0l?3`0IVIV04E5A@1_Kfl0Fe]K00kooooo01/0C4a<03Ti>@1OGel0
CTi>02h^;P0d=3@0AdM70:2PX020P800?Shn030`<01NGUh0IFEU09:BTP1?Cdl0CDe=07MgM`2=SHd0
MgMg065QH@18B4P0LG5a0:JVYP29RHT0N7Qh05]KF`0V9RH00P0L71`0<00K6a/0HV9R08b<S01nOWh0
Ti>C038b<P0P820071`L020P801ZJVX0IVIV09>CT`0d=3@07QhN02/[:`1=CDd0HV9R06][J`1OGel0
IVIV06YZJP1mOGd0Q8B406=SH`1eMGD0MgMg03<c<`0O7al0;bl_08R8R02;Rh/0][Jf09^KV`2j^[X0
`L7108J6QP2OWil0^[Zj0:j^[P2JVYX0KFe]03Lg=`1YJFT0K6a/07YjNP1=CDd0OWin055AD@8071`L
03d0C4a<06UYJ@1KFe/0;bl_02h^;P1VIVH0QhN707YjNP1oOgl0C4a<02<S8`0O7al08R8R08f=S@1d
M7@0GUiN0:>SX`0l?3`0>c/k09BDU017AdL0CTi>04m?C`12@T80Hf=S09^KV`22PX80FUYJ0:ZZZP2W
YjL0?3`l04i>CP29RHT0>SXj04A4A01YJFT0YJFU05mOG`0o?cl0YZJV08N7Q`2RXZ80^k^k07inOP1j
NWX0PH610:f][@2SXj<0>3Ph024Q8@0[:b/0GUiN0820P01]KFd0UIFE05iNGP2@T900^;Rh05QHF012
@T80Ng]k01Cooooo000Cooooo`2408>3P`1nOWh0HF5Q0820P01GEeL0CTi>06YZJP1]KFd0GUiN06e]
K@1jNWX0I6AT04i>CP1GEeL0LG5a03Hf=P1GEeL0JVYZ03`l?01VIVH0NWYj04I6AP1;Bd/0Rh^;04];
B`1RHV80MgMg034a<@1MGEd0Cdm?03hn?P1VIVH0AdM704e=C@1NGUh0ADE506QXJ01KFe/0F5QH07al
O01ADE40FEUI09:BTP1FEUH0B4Q808Z:RP26QXH0De=C08f=S@1^KVh0@D5107AdM01PH600D51@07Ad
M01QHF40D51@06][J`1VIVH0FEUI08R8R01RHV80O7al06m_K`1KFe/0OWin05eMG@1]KFd0O7al04Q8
B01gMgL0LW9b03lo?`2:RXX0SHf=05=CD`1TI6@0NWYj055AD@28R8P0V9RH04I6AP2CTi<0[Jf]069R
HP2IVIT0Ph>306YZJP27QhL0IfMW05QHF02<S8`0JFUY07IfMP1cLg<0Fe]K09JFUP1QHF40EEEE08V9
R@1SHf<0WYjN06IVIP1RHV80U9BD055AD@1DE5@0T92@0820P01_Kfl0Lg=c08R8R01cLg<0DE5A06i^
KP2>SXh0AdM706IVIP2KVi/0;bl_071`L02;Rh/0>c/k07IfMP1YJFT0?3`l07AdM01aLG40?Cdm06i^
KP1[Jf/0AdM70P1[Jf/01@14A4@0K6a/06i^KP0f=SH0Ogmo00kooooo01T0KVi^06UYJ@1VIVH0IfMW
04Q8B01>CTh0IVIV075aL@1NGUh0G5aL0410@01>CTh0EEEE04];B`1WIfL0FUYJ05]KF`1XJ6P0JVYZ
04];B`1?Cdl0E5AD06IVIP1EEED0M7Ad0080FEUI03D0EeMG06=SH`1@D500IFEU061PH019BDT0Fe]K
06MWI`1YJFT0M7Ad04=3@`1GEeL0H61P0:>SX`1JFUX0ADE504M7A`14A4@0C4a<04];B`1BDU80?clo
04Q8B01EEED0AdM703`l?01BDU80G5aL05iNGP1HF5P0HF5Q04U9B@1RHV80MGEe079bLP1LG5`0KVi^
07MgM`1UIFD0Ogmo079bLP1iNGT0JVYZ06e]K@1@D500EUIF04M7A`1/K6`0EUIF04e=C@1IFET0Fe]K
0;6a/@02059BDP0G05iNGP1HF5P0E5AD04e=C@1WIfL0IVIV04=3@`1GEeL0Q8B4051@D017AdL0EEEE
055AD@1DE5@0JFUY069RHP1ADE40G5aL06EUI@1EEED0FUYJ059BDP0i>CT00P1@D5008@1DE5@0EUIF
04Y:BP1aLG40KFe]04=3@`1JFUX0Fe]K04U9B@1FEUH0G5aL05eMG@1OGel0Jf][04=3@`1^KVh0A4A4
06QXJ01RHV80IFEU08>3P`1SHf<0QXJ60;2`/01hN7P0M7Ad04U9B@1XJ6P0IfMW04a<C025QHD0MGEe
08:2PP0>ooooo`1W07QhN01/K6`0G5aL05eMG@14A4@0>3Ph05MGE`0h>3P0@D5103Hf=P11@D40>CTi
02TY:@0k>c/0@D5104e=C@0Y:BT0:2PX03Hf=P0j>SX0HF5Q03Hf=P1?Cdl0CTi>06IVIP16ATH0E5AD
04U9B@0l?3`0DU9B07AdM00i>CT0BDU9059BDP1IFET0ADE505QHF01jNWX0?clo07EeM@1NGUh0G5aL
04m?C`1KFe/0EUIF07AdM01oOgl0SXj>06][J`1PH600EeMG05]KF`1GEeL0KVi^06EUI@1LG5`0DU9B
05eMG@1JFUX0E5AD059BDP1^KVh0BDU9059BDP1LG5`0N7Qh04E5A@1RHV80E5AD04Q8B020P800Kfm_
06ATI01YJFT0Rh^;05MGE`1WIfL0GemO07YjNP17AdL0ZJVY04e=C@1LG5`0JVYZ06][J`1ZJVX0OGem
06=SH`1QHF40DE5A07IfMP1JFUX0EEEE06YZJP1hN7P0De=C05iNGP1]KFd0@d=3071`L01XJ6P0HV9R
05ADE00206=SH`0S06ATI01<C4`0IVIV05YJFP1?Cdl0De=C06e]K@1VIVH0Kfm_06MWI`20P800GUiN
059BDP1SHf<0@d=307IfMP16ATH0Ogmo059BDP1^KVh0PX:2069RHP17AdL0L71`04E5A@1HF5P0EeMG
08J6QP1XJ6P0UIFE0492@P1`L700=CDe07]kN`1EEED03_ooool0=@1>CTh0:2PX02@T901FEUH0E5AD
05eMG@27QhL0W9bL05eMG@1GEeL0K6a/071`L01KFe/07AdM02PX:01BDU80PX:20861P@1TI6@0;bl_
03<c<`1aLG40H61P0861P@1hN7P0UYJF06=SH`1BDU80@41005IFEP2HV9P0O7al05]KF`1QHF40E5AD
06ATI01GEeL0K6a/06IVIP2OWil0SHf=02HV9P0M7Ad0;bl_07QhN02VYZH0U9BD07inOP1ZJVX0M7Ad
04Q8B01ZJVX0F5QH0080RXZ:00@0GemO06i^KP1oOgl0Bd];0P2JVYX05P1/K6`0/k>c09:BTP1ADE40
JVYZ08n?S`1_Kfl0?3`l08f=S@0R8R80?clo06ATI018B4P0?Shn02h^;P2LW9`0EEEE04i>CP2@T900
UiNG051@D00e=CD205aLG00g07AdM02>SXh0[:b/069RHP0N7Qh07AdM03Lg=`2EUID0;Bd]030`<01l
O7`0;Bd]024Q8@1UIFD0YZJV04e=C@2CTi<0Rh^;06=SH`1kNg/0QHF502/[:`2<S8`0Xj>S071`L00f
=SH0GUiN04];B`1BDU80SXj>03/k>`1RHV80UYJF0;Jf]P2k^k/0/;2`0;6a/@2RXZ80RXZ:06QXJ02D
U9@0Y:BT05IFEP0d=3@0IFEU07]kN`2ATI40Vi^K09>CT`2WYjL0cLg=0:n_[`1YJFT0X:2P079bLP0D
ooooo`004oooool0IP1mOGd0LW9b07YjNP1kNg/0JVYZ05]KF`1JFUX0Lg=c05mOG`1_Kfl0Ti>C059B
DP1<C4`0O7al05MGE`1CDe<0IVIV05=CD`1HF5P0Kfm_05IFEP1[Jf/0E5AD05mOG`1bLW80<c<c05aL
G01cLg<0?3`l071`L01[Jf/0A4A4055AD@25QHD0@41005aLG027QhL0;Bd]05iNGP1hN7P0?Cdm05UI
F@1fMWH0ADE5061PH01QHF40FUYJ05ADE01=CDd0OWin04U9B@1>CTh0N7Qh03Xj>P1?Cdl0QhN704=3
@`1<C4`0Shn?04E5A@1MGEd0QhN704m?C`1SHf<0Kfm_05]KF`1GEeL0Lg=c06IVIP1FEUH0M7Ad05IF
EP1dM7@0P82006IVIP1aLG40HV9R06UYJ@1]KFd0BTY:069RHP21PH40AdM706][J`25QHD0?clo07Mg
M`1kNg/0@D5107alO01UIFD0GEeM09FEU@1oOgl0K6a/08b<S01RHV80J6QX06EUI@1@D500O7al06][
J`80OGem02<0Hf=S09FEU@1>CTh0JVYZ09NGU`1XJ6P0Jf][092@T01OGel0HF5Q07moO`1SHf<0H61P
07YjNP1aLG40K6a/06m_K`1GEeL0IFEU08f=S@1DE5@0IfMW06][J`1aLG40NWYj06UYJ@1PH600HV9R
06MWI`0m?Cd0K6a/06QXJ01@D500L71`05ADE00>ooooo`0805IFEP1SHf<0DE5A05UIF@1?Cdl0C4a<
04e=C@1UIFD205MGE`1F03lo?`1?Cdl0D51@04U9B@1;Bd/0A4A404=3@`1BDU80Hf=S04A4A019BDT0
IVIV061PH01BDU80Ph>305MGE`1SHf<0D51@055AD@1EEED0Lg=c06a/K01ADE40Hf=S06UYJ@1SHf<0
MWIf05UIF@1KFe/0Cdm?079bLP1QHF40Bd];055AD@1>CTh0Cdm?04Y:BP1<C4`0AdM7055AD@1]KFd0
CTi>04M7A`1GEeL0IFEU06=SH`1]KFd0LG5a04A4A01`L700P82006MWI`1`L700YjNW08F5Q@1fMWH0
R8R806a/K02=SHd0HV9R07YjNP1>CTh0EUIF04M7A`1`L700CDe=04i>CP1LG5`0LG5a0;Bd]01QHF40
GUiN06=SH`1BDU80IFEU059BDP1WIfL0KVi^04a<C01SHf<0IfMW055AD@1:BTX0ADE503Xj>P1TI6@2
05ADE00S06ATI01YJFT0GemO05mOG`14A4@0BDU903Lg=`16ATH0A4A4051@D01:BTX0De=C05QHF01C
De<0?3`l051@D018B4P0BTY:06MWI`1lO7`0N7Qh05YJFP1eMGD0FUYJ05aLG01ADE40KVi^06ATI01V
IVH0SXj>065QH@1YJFT0Q8B409^KV`2ATI400P1cLg<01@1nOWh0HF5Q08R8R01hN7P0P82000kooooo
01/0MgMg05aLG01RHV80E5AD03dm?@0h>3P0DE5A0492@P0m?Cd0;Bd]04Y:BP0n?Sh0@D5103<c<`1;
Bd/0B4Q802PX:00a<C40;2`/03`l?01DE5@0<30`04Y:BP0o?cl0Lg=c061PH01BDU800P1;Bd/03@19
BDT0JFUY03`l?010@400BDU903lo?`0n?Sh0GEeM079bLP0h>3P0J6QX05=CD`1KFe/00P1FEUH0?01:
BTX0GUiN05]KF`25QHD0G5aL059BDP1UIFD0CDe=05=CD`1]KFd0Fe]K06a/K013@d<0E5AD069RHP17
AdL0A4A407=cL`17AdL0IFEU05=CD`1fMWH0@T9205aLG01NGUh0A4A4071`L01WIfL0LW9b06IVIP1i
NGT0Hf=S05]KF`1hN7P0R8R806UYJ@2][Jd0GEeM07IfMP1TI6@0GUiN08J6QP1jNWX0IFEU07alO019
BDT0JFUY04M7A`11@D40HV9R065QH@15ADD0I6AT05YJFP0h>3P0Lg=c079bLP1WIfL0EeMG06QXJ080
F5QH0280ATI6071`L01=CDd0F5QH05MGE`1XJ6P0Jf][07YjNP1HF5P0J6QX07=cL`19BDT0FEUI03hn
?P1WIfL0?clo04i>CP1OGel0JFUY069RHP1IFET0DE5A061PH00f=SH0L71`055AD@1dM7@0PX:209VI
V@0d=3@0GEeM04i>CP1cLg<0D51@3_ooool0S015ADD0FUYJ04A4A01PH600G5aL04Q8B017AdL0P820
07inOP1/K6`0PH6108n?S`1eMGD0:RXZ03Ti>@1BDU80;bl_03dm?@1HF5P08b<S028R8P0f=SH0H61P
0:FUY@28R8P0AdM703lo?`1<C4`0F5QH08J6QP2i^KT0XJ6Q09^KV`1@D5008R8R02l_;`1?Cdl0UIFE
0;fm_@1JFUX0Bd];05mOG`1=CDd0A4A402d];@0Y:BT0?clo09^KV`2f][H0Xj>S03/k>`0^;Rh0IVIV
08n?S`1DE5@0Jf][075aL@2][Jd0LW9b06YZJP1=CDd0@D5109nOW`2ZZZX0ATI6071`L021PH40Bd];
028R8P1;Bd/0<30`05EEE@24Q8@0?Shn03De=@1/K6`0Ogmo05ADE02VYZH0/K6a0:2PX02k^k/0MGEe
05UIF@1hN7P0?Cdm06EUI@1mOGd0];Bd07MgM`1ZJVX0UIFE0;^k^`0/;2`0;bl_0861P@30`<00VIVI
08f=S@2OWil0WYjN09fMW@1[Jf/0N7Qh0::RXP2l_;`0V9RH0<[:bP1YJFT0ATI609JFUP2MWId0QHF5
09fMW@2GUiL0Q8B408f=S@2VYZH0Kfm_09^KV`29RHT0a<C408B4Q01RHV80?Shn05]KF`21PH40Shn?
0:JVYP1kNg/0M7Ad09FEU@2WYjL0c/k>0:FUY@2LW9`0IFEU06IVIP2QXJ40HV9R5?ooool001?ooooo
02X0RHV909bLW01^KVh0Jf][071`L016ATH0M7Ad069RHP16ATH0FEUI075aL@1mOGd0B4Q8061PH01/
K6`0@41004=3@`1UIFD0FUYJ05]KF`1hN7P0FEUI05EEE@26QXH0BTY:05MGE`1jNWX0>CTi06YZJP1L
G5`0=CDe065QH@17AdL0?clo06UYJ@10@400C4a<06ATI01CDe<0Bd];05iNGP1@D50205UIF@0F04Q8
B01/K6`0JFUY04I6AP1JFUX0Lg=c03Hf=P1EEED0J6QX04M7A`1NGUh0L71`051@D01FEUH0JVYZ05YJ
FP1bLW80J6QX07IfMP1QHF40EeMG07AdM080GEeM01@0VIVI04e=C@1]KFd0LW9b04U9B@22PX80OGem
04M7A`1GEeL0Jf][04M7A`1@D500RHV90451@@1[Jf/0Ph>3061PH01hN7P0M7Ad05YJFP80M7Ad0340
A4A407moO`1UIFD0J6QX07moO`0o?cl0TY:B04i>CP1LG5`0RXZ:055AD@2HV9P0J6QX05YJFP23Ph<0
J6QX065QH@20P800LW9b07EeM@1XJ6P0JVYZ0820P01RHV80GemO08R8R018B4P0GUiN09^KV`0i>CT0
HV9R09bLW015ADD0IVIV07moO`17AdL0JFUY06ATI01>CTh0Jf][07=cL`0b<S80Fe]K07AdM01CDe<0
G5aL07YjNP17AdL0Jf][00kooooo0P1HF5P06P1JFUX0Hf=S04];B`1;Bd/0FUYJ061PH01LG5`0DE5A
03Hf=P19BDT0AdM703/k>`1;Bd/0ATI604i>CP1HF5P0D51@04M7A`1<C4`0DU9B05QHF01BDU80Jf][
04e=C@1QHF40?clo0P1@D500KP1OGel0F5QH04e=C@1RHV80IVIV06=SH`1gMgL0CTi>05=CD`1IFET0
Kfm_04U9B@16ATH0AdM704U9B@1<C4`0BDU9055AD@0o?cl0CTi>051@D01<C4`0Bd];05IFEP1NGUh0
GemO06EUI@1YJFT0BTY:05YJFP1cLg<0NGUi05ADE02OWil0PX:207alO01_Kfl0Ogmo06YZJP1aLG40
P82005iNGP1HF5P0G5aL08f=S@1YJFT0D51@05aLG01kNg/0_kno04m?C`1OGel0JFUY055AD@1YJFT0
F5QH06a/K01ZJVX0B4Q804m?C`1_Kfl0EeMG04];B`1FEUH0AdM706IVIP1SHf<0AdM7055AD@1aLG40
JVYZ05YJFP1FEUH0@D5102h^;P12@T80B4Q8051@D01;Bd/0EEEE04e=C@1[Jf/0<S8b04a<C01ADE40
ADE505YJFP1SHf<0M7Ad05YJFP1XJ6P0DU9B06=SH`1FEUH0HF5Q05ADE01GEeL0UIFE07QhN028R8P0
YZJV0:b/[02ATI40DE5A06YZJP1_Kfl0Bd];07inOP1ZJVX0R8R83_ooool0=P1QHF40I6AT079bLP1M
GEd0>c/k02/[:`18B4P0;2`/03Lg=`0[:b/0=CDe0492@P0m?Cd0:RXZ059BDP0h>3P0;Rh^03@d=00b
<S80<c<c04U9B@1:BTX0ADE503`l?01VIVH0A4A40492@P15ADD0=3@d03De=@1PH600>SXj03hn?P14
A4@0<C4a02d];@1@D500I6AT02XZ:P1ZJVX0Hf=S03hn?P1SHf<0De=C03Hf=P1>CTh0GEeM096AT@1I
FET0H61P06=SH`1?Cdl0BTY:05YJFP80GEeM01X0GemO05iNGP1BDU80BDU9059BDP1jNWX0I6AT05EE
E@1SHf<0GemO03Xj>P1/K6`0HF5Q034a<@26QXH0Jf][069RHP2;Rh/0KFe]05YJFP1_Kfl0GUiN07Yj
NP1eMGD0/K6a04e=C@80Hf=S03P0O7al07UiN@1fMWH0JFUY06YZJP1;Bd/0KVi^065QH@14A4@0JVYZ
065QH@1TI6@0GUiN06UYJ@10@400NWYj06QXJ01PH600UIFE07inOP1YJFT0MGEe04Y:BP1^KVh0CDe=
05UIF@1LG5`0I6AT065QH@1OGel0MWIf05]KF`14A4@0EEEE06MWI`0^;Rh0G5aL03hn?P0k>c/0BDU9
079bLP1fMWH0CTi>04Q8B01TI6@0=CDe05QHF01FEUH0L71`07EeM@2?Shl0@d=307YjNP17AdL0MgMg
04i>CPkooooo07d0BDU906EUI@1gMgL0ATI6051@D00T92@05ADE02PX:01[Jf/0_kno06EUI@2CTi<0
MgMg08J6QP1;Bd/09bLW020P800S8b<0EUIF03Ti>@0S8b<0:2PX09>CT`2=SHd0NGUi03De=@0Y:BT0
;2`/034a<@1[Jf/0WinO0;Vi^@2DU9@0>CTi01hN7P0O7al09BDU0410@02YZJT0IVIV04m?C`13@d<0
:BTY01`L700S8b<092@T03lo?`25QHD0alO70<30`02HV9P0U9BD055AD@0O7al07QhN030`<01MGEd0
/[:b0<_;b`24Q8@0?Cdm02<S8`2KVi/0YjNW08f=S@2CTi<0NGUi051@D00_;bl0@41006e]K@0m?Cd0
7AdM04A4A00i>CT0Lg=c04i>CP1DE5@0c/k>0::RXP23Ph<0Xj>S0861P@20P800Ogmo065QH@1HF5P0
ADE50;Vi^@2>SXh0=SHf08^;R`2ATI40D51@02l_;`2;Rh/0IfMW0:6QX@27QhL0WYjN0<G5a@2b/[80
?3`l051@D02/[:`0UIFE09RHV02RXZ80TI6A07QhN01cLg<0Ogmo0:RXZ02KVi/0GemO06e]K@1iNGT0
?Shn010@400X:2P0XJ6Q06=SH`0Z:RX0?3`l07AdM00205aLG00=07]kN`2PX:00U9BD0;>c/`1`L700
`L7107moO`15ADD0EUIF07=cL`27QhL0WinO04a<C00Dooooo`004oooool0401oOgl0L71`07UiN@1e
MGD0G5aL06EUI@1DE5@0Jf][05aLG013@d<0Ng]k05eMG@18B4P0LW9b05mOG`17AdL2055AD@0:04Y:
BP1SHf<0JVYZ055AD@1QHF40N7Qh05mOG`1MGEd0G5aL04M7A`80EeMG06d0IfMW05eMG@1:BTX0I6AT
059BDP1>CTh0Kfm_04Q8B01>CTh0L71`0451@@1IFET0LW9b0451@@1PH600GemO059BDP1IFET0DE5A
061PH017AdL0D51@07AdM01GEeL0EEEE07EeM@19BDT0FEUI07AdM01RHV80JVYZ07alO01mOGd0L71`
06QXJ01cLg<0EUIF06MWI`1gMgL0DE5A079bLP1ZJVX0D51@07alO01eMGD0JVYZ06EUI@1WIfL0HV9R
065QH@1VIVH0Jf][05mOG`1[Jf/0NWYj06ATI01mOGd0Ng]k04a<C01[Jf/0L71`05ADE01XJ6P0QhN7
05YJFP1[Jf/0RHV9059BDP1JFUX0LW9b06MWI`1kNg/0I6AT06e]K@1aLG40KFe]05eMG@1/K6`0Rh^;
075aL@1NGUh0N7Qh071`L01PH600O7al0861P@1<C4`0JFUY08f=S@1:BTX0Jf][07QhN01BDU80Ph>3
06e]K@1eMGD0NGUi05QHF01NGUh0NGUi05aLG01ADE40IfMW04i>CP1MGEd0MGEe04E5A@1MGEd0IVIV
00kooooo00P0EUIF04a<C012@T80CTi>0451@@14A4@0C4a<05QHF080A4A400D0=cLg04Y:BP1<C4`0
>SXj04Y:BP0204Q8B00K055AD@1@D500@T9204=3@`1=CDd0BDU904m?C`1BDU80ADE5059BDP17AdL0
B4Q804e=C@1KFe/0CTi>04Q8B01IFET0Jf][069RHP1OGel0CTi>05MGE`1@D500FEUI03dm?@13@d<0
@T920080?clo00D0CTi>059BDP0i>CT0CTi>05UIF@0204Y:BP1G05eMG@1HF5P0GEeM05QHF01MGEd0
BTY:05EEE@1QHF40HV9R04];B`25QHD0MGEe06e]K@1eMGD0NWYj06QXJ01SHf<0EEEE05iNGP1EEED0
Hf=S08:2PP1eMGD0FUYJ07MgM`24Q8@0_[jn06ATI01LG5`0JVYZ06i^KP1WIfL0DU9B06UYJ@1TI6@0
FEUI05QHF01kNg/0JFUY05MGE`1]KFd0CDe=05aLG01EEED0G5aL065QH@1/K6`0GEeM05IFEP1OGel0
Cdm?03Xj>P13@d<0C4a<05]KF`1BDU80E5AD05=CD`1hN7P0B4Q804i>CP1QHF40Cdm?06YZJP1VIVH0
J6QX05eMG@1RHV80C4a<061PH016ATH0HF5Q05mOG`1EEED0Shn?069RHP1gMgL0O7al08N7Q`2GUiL0
IfMW06][J`1hN7P0De=C07]kN`1iNGT0Q8B400kooooo02`0GemO06MWI`1GEeL0MWIf03Xj>P0f=SH0
DE5A03Ph>00g=cL0A4A402d];@0[:b/0@d=303Hf=P1CDe<0=CDe030`<00g=cL0=3@d034a<@1DE5@0
<c<c0410@00k>c/0I6AT059BDP11@D40<C4a03@d=019BDT0De=C03Ti>@1FEUH0@D5102PX:00h>3P0
CDe=06i^KP0h>3P0JVYZ06e]K@1:BTX0J6QX05YJFP80@D5101P0EEEE08f=S@1ADE40G5aL075aL@15
ADD0HF5Q05mOG`1NGUh0GEeM069RHP1IFET0F5QH04e=C@0o?cl0L71`051@D01bLW80Fe]K07MgM`1>
CTh0M7Ad05IFEP1=CDd2071`L00L06EUI@24Q8@0JFUY06IVIP1HF5P0O7al07MgM`15ADD0YjNW06AT
I029RHT0G5aL08:2PP2>SXh0IFEU06MWI`1YJFT0DE5A06i^KP1IFET0E5AD06MWI`1IFET0GUiN06QX
J01NGUh0>3Ph06][J`80H61P00h0Rh^;07=cL`1aLG40Kfm_04M7A`1dM7@0FEUI04Y:BP12@T80KVi^
055AD@1SHf<0OGem06EUI@80De=C01H0EeMG03Xj>P1FEUH0=SHf0451@@1@D500LW9b07QhN017AdL0
DU9B05ADE00f=SH0GUiN06IVIP1PH600I6AT09bLW01@D500IVIV04i>CP1jNWX0E5AD3_ooool05P29
RHT0T92@071`L01UIFD0Hf=S04a<C00d=3@0?Cdm09ZJVP2UYJD0LW9b05EEE@1eMGD0KVi^06EUI@1?
Cdl0AdM703Xj>P1<C4`0R8R806EUI@1IFET20;Fe]@1d0:j^[P25QHD0F5QH04M7A`12@T80?Shn04A4
A01JFUX0MGEe04e=C@0m?Cd0>SXj04I6AP1>CTh0SHf=096AT@1^KVh0MGEe06YZJP0e=CD07alO03@d
=016ATH0GemO0;:b/P2`/;00CTi>06YZJP0V9RH071`L01TI6@0K6a/0:BTY08j>SP2@T900Ng]k03Ti
>@10@400GEeM06i^KP1FEUH0Cdm?065QH@1SHf<0Jf][05=CD`1@D500:RXZ01HF5P0H61P0;Bd]0861
P@2XZ:P0TY:B051@D01HF5P0>SXj04];B`1PH600GemO06m_K`2=SHd0L71`08n?S`1aLG40Ph>30;jn
_P2EUID0T92@0<C4a021PH40Hf=S01XJ6P0L71`0820P09FEU@37alL0Ng]k09>CT`1>CTh0MgMg08B4
Q02][Jd0VIVI0;6a/@1oOgl08R8R020P802@T900XZ:R07QhN02ATI40OWin08F5Q@1:BTX0Jf][09:B
TP1RHV808b<S01lO7`0W9bL08R8R02@T901RHV80^k^k0;bl_02OWil0bl_;071`L00N7Qh071`L02HV
9P2JVYX0/K6a0;Jf]P1VIVHDooooo`004oooool0?`27QhL0MWIf08J6QP1QHF40N7Qh051@D01JFUX0
RHV9051@D01ADE40Vi^K051@D01FEUH0K6a/04I6AP1ADE40CDe=04Y:BP1RHV80IVIV04M7A`1VIVH0
@41004U9B@1ZJVX0:b/[06ATI01JFUX0@T9207=cL`1OGel0@41005ADE01/K6`0>3Ph04M7A`1dM7@0
?Cdm04Y:BP1lO7`0>c/k06=SH`1LG5`0C4a<05eMG@1IFET0O7al055AD@1OGel0Ogmo03Lg=`1GEeL0
IFEU0410@01MGEd0N7Qh03Ti>@1QHF40P8200410@01gMgL0MWIf04i>CP0207EeM@1:069RHP1TI6@0
JFUY05UIF@1VIVH0OWin05EEE@1FEUH0MgMg079bLP1UIFD0EeMG06e]K@1IFET0@T9206a/K01YJFT0
@T9209:BTP1LG5`0ADE509:BTP1YJFT0DE5A08^;R`1KFe/0KFe]07MgM`1SHf<0Ng]k04];B`1cLg<0
GemO03Ti>@1^KVh0IVIV05mOG`1aLG40Hf=S06e]K@21PH40?Shn06e]K@2DU9@0DE5A07emO@1[Jf/0
BDU908b<S01lO7`0LW9b05UIF@1QHF40IVIV05EEE@1XJ6P0GEeM05]KF`1gMgL0H61P06a/K01DE5@0
Lg=c05mOG`1JFUX0JVYZ0492@P13@d<0FUYJ05eMG@1]KFd0I6AT05ADE019BDT>ooooo`1[05IFEP1E
EED0DE5A051@D019BDT0@d=3059BDP1KFe/0F5QH04M7A`0e=CD0ADE504a<C017AdL0H61P05ADE01E
EED0G5aL051@D014A4@0Bd];05QHF01NGUh0Jf][08>3P`1LG5`0F5QH04];B`1KFe/0CTi>065QH@1A
DE40CTi>05MGE`1:BTX0Hf=S061PH01CDe<0EeMG055AD@0j>SX0?clo03Hf=P14A4@0@41004E5A@1:
BTX0ATI604M7A`10@400EUIF04a<C010@400GEeM04U9B@1ZJVX0?clo06EUI@16ATH0FUYJ06m_K`1V
IVH0Fe]K08N7Q`21PH40IFEU08^;R`1_Kfl0IfMW06QXJ01LG5`0DE5A04U9B@1IFET0JFUY05aLG01V
IVH0L71`05UIF@2c/k<0Cdm?06QXJ01dM7@0FUYJ05mOG`1ADE40O7al07YjNP1RHV80DU9B09:BTP1S
Hf<0G5aL06EUI@1HF5P0GUiN04i>CP1EEED0AdM706a/K01RHV80EUIF06ATI01=CDd0?clo04=3@`1<
C4`00P1EEED03P1KFe/0H61P07AdM01?Cdl0G5aL05eMG@13@d<0FUYJ05IFEP1PH600E5AD07alO01R
HV80Ogmo0P1QHF403`1IFET0GEeM08Z:RP1FEUH0QXJ60:FUY@2>SXh0R8R806m_K`1KFe/0GEeM05AD
E029RHT0MWIf0861P@0>ooooo`0:06UYJ@1MGEd0Cdm?06e]K@0d=3@0>CTi04A4A00c<c<0;2`/0410
@0808B4Q0100@d=303Hf=P1BDU80@T9203Xj>P13@d<0;bl_03hn?P1WIfL0;Rh^04i>CP0b<S80KFe]
06EUI@0n?Sh0=CDe0P11@D401017AdL0BTY:04M7A`0h>3P203Ti>@1X04a<C01WIfL0BTY:06QXJ022
PX80K6a/05mOG`1aLG40ADE503@d=01MGEd0SHf=05UIF@1VIVH0O7al04];B`1CDe<0G5aL061PH019
BDT0LW9b05IFEP1VIVH0D51@0492@P1ZJVX0EeMG07MgM`1LG5`0Kfm_05IFEP1mOGd0ADE505aLG01D
E5@0IFEU05QHF01lO7`0JVYZ05YJFP1LG5`0QHF506a/K01kNg/0_[jn06e]K@2=SHd0@T9208N7Q`26
QXH0FEUI08Z:RP1NGUh0@D5105aLG01nOWh0FUYJ05eMG@1KFe/0MWIf06QXJ01LG5`0>c/k07YjNP1S
Hf<0KFe]07UiN@2:RXX0MGEe07alO01IFET0KVi^06m_K`1MGEd0A4A4075aL@1KFe/0Hf=S096AT@1^
KVh0HV9R059BDP1eMGD0?Shn04m?C`0g=cL0H61P059BDP1OGel0Ph>304m?C`19BDT0Jf][0451@@1J
FUX0EUIF06MWI`1]KFd0SXj>04];B`1mOGd0EeMG06QXJ01ADE4>ooooo`0X09jNWP1EEED0H61P03`l
?00V9RH0B4Q8071`L01QHF40J6QX0;no_`38b<P0`L7108f=S@23Ph<0Xj>S09BDU01hN7P0UIFE0<30
`01NGUh0L71`05UIF@22PX80M7Ad03/k>`28R8P0];Bd092@T011@D407QhN02d];@0T92@0MGEe08V9
R@1bLW80GemO04e=C@0a<C40MWIf0820P080GUiN03H0LG5a06MWI`2=SHd0EeMG03`l?01`L700aLG5
0<K6aP1]KFd0?3`l04E5A@0L71`06ATI028R8P1SHf<0UYJF0:2PX02n_[h0WinO02LW9`0/;2`0LW9b
0410@00];Bd0BDU907]kN`1]KFd0KVi^03hn?P1>CTh061PH02XZ:P1FEUH0PX:209JFUP0`<3006ATI
01hN7P0h>3P0<S8b03dm?@0_;bl0RHV908N7Q`1[Jf/0C4a<06ATI02RXZ80=SHf01lO7`10@400f=SH
0=cLg01ZJVX201PH600Z02DU9@2SXj<0LW9b02@T901EEED0=SHf055AD@18B4P0NGUi07alO01DE5@0
[jn_038b<P0d=3@0Ng]k0:VYZ@0a<C40:BTY02DU9@0f=SH0B4Q80;bl_01gMgL0YJFU05YJFP1=CDd0
=SHf04A4A01XJ6P0ZZZZ0<?3``38b<P0^KVi0820P00Y:BT06a/K02@T901bLW80Bd];06QXJ02_[jl0
IfMW5?ooool001?ooooo00d0MgMg06e]K@1UIFD0L71`06e]K@1IFET0G5aL06EUI@1`L700ATI605UI
F@1fMWH0@4100080J6QX00h0@d=305MGE`1^KVh0B4Q8051@D01^KVh0D51@061PH028R8P0C4a<06IV
IP1fMWH0<C4a061PH080BTY:0500JVYZ04E5A@1IFET0IFEU04Q8B01MGEd0Fe]K055AD@1mOGd0FEUI
05aLG01WIfL0B4Q804i>CP1fMWH0G5aL051@D01KFe/0GUiN051@D01XJ6P0G5aL04Y:BP1RHV80EEEE
05QHF01[Jf/0Hf=S05EEE@1nOWh0Jf][05]KF`1[Jf/0GemO07IfMP1LG5`0HF5Q09:BTP13@d<0Ogmo
07alO00n?Sh0Lg=c08>3P`0o?cl0Jf][06ATI01EEED0FEUI09NGU`1@D500JVYZ08>3P`1gMgL0SHf=
08^;R`1dM7@0Kfm_06e]K@1SHf<0MWIf07UiN@2NWYh0N7Qh05mOG`2TY:@0G5aL05mOG`27QhL0JFUY
07inOP1:BTX0GUiN07=cL`1DE5@0EeMG06EUI@1jNWX0MGEe04a<C080Ph>301X0=SHf06i^KP2PX:00
?Cdm06EUI@2=SHd0<C4a05iNGP2<S8`0:2PX07moO`1WIfL0Cdm?08f=S@1CDe<0D51@06IVIP1SHf<0
B4Q806e]K@1fMWH0B4Q806i^KP1MGEd0EUIF06m_K`kooooo03`0FEUI059BDP1@D500FUYJ04i>CP15
ADD0EUIF05iNGP1MGEd0DE5A03dm?@1ADE40BTY:04A4A01HF5P0EeMG05YJFP1SHf<0Cdm?0451@@1>
CTh0C4a<059BDP1DE5@0KVi^04A4A01HF5P0B4Q806EUI@1EEED0Hf=S04e=C@1;Bd/0CDe=05EEE@1X
J6P0HV9R04M7A`1BDU80D51@03`l?012@T80<c<c04M7A`1<C4`0De=C05mOG`1?Cdl0E5AD0410@01I
FET0@d=303/k>`1EEED0DU9B05UIF@18B4P0MgMg05iNGP1HF5P2065QH@0<04Y:BP1iNGT0Ti>C06UY
J@2>SXh0JVYZ05UIF@1MGEd0L71`04m?C`1ADE40C4a<0P1LG5`03`1GEeL0K6a/07alO02o_kl0Ng]k
06a/K01[Jf/0IVIV05iNGP14A4@0Kfm_05MGE`1DE5@0D51@09RHV002069RHP0_05]KF`1TI6@0J6QX
04U9B@1NGUh0EeMG065QH@1jNWX0DU9B05MGE`1>CTh0>3Ph04Q8B01:BTX0HV9R04];B`1:BTX0EeMG
07]kN`1<C4`0BTY:05IFEP13@d<0HV9R04i>CP1EEED0E5AD05iNGP1GEeL0J6QX04U9B@1YJFT0EUIF
06][J`2FUYH0EEEE06][J`1hN7P0Ph>307QhN01IFET0HF5Q04m?C`1CDe<0L71`07=cL`1kNg/03_oo
ool0J@1OGel0LG5a05=CD`2KVi/0@d=30451@@1EEED0?Shn03Ph>018B4P09bLW02<S8`0j>SX0=3@d
061PH01<C4`0A4A404i>CP0j>SX0=SHf07MgM`0k>c/0BDU904E5A@23Ph<0@41004Q8B00l?3`0@410
05=CD`1FEUH0BTY:051@D00l?3`0A4A403@d=01<C4`0EeMG0492@P1<C4`0MgMg05]KF`1]KFd0JFUY
04i>CP0/;2`0I6AT09FEU@18B4P0Lg=c079bLP1CDe<0F5QH04U9B@1PH600G5aL07MgM`1PH600C4a<
059BDP1>CTh0J6QX061PH01eMGD0GEeM07=cL`1CDe<0Lg=c04M7A`1VIVH0HV9R07IfMP1YJFT0Y:BT
06e]K@1cLg<0OGem08f=S@1WIfL0P8200<71`@1GEeL0TI6A04M7A`22PX80QXJ605EEE@1aLG40H61P
03lo?`1QHF40R8R805aLG01QHF40De=C07MgM`1^KVh0Hf=S03`l?01cLg<0FUYJ07IfMP14A4@0J6QX
07MgM`0205]KF`0Q07QhN01HF5P0HV9R04Q8B01UIFD0EeMG05UIF@2BTY80GUiN04i>CP1JFUX0E5AD
03`l?01BDU80:2PX05IFEP19BDT0LW9b08F5Q@15ADD0Fe]K06MWI`0a<C40Hf=S05]KF`1?Cdl0Hf=S
096AT@12@T80F5QH05UIF@1`L700EEEE00kooooo04h0G5aL06m_K`1WIfL0DE5A06a/K01SHf<0IfMW
07UiN@0Y:BT0Z:RX0;:b/P1[Jf/0;2`/038b<P1GEeL0M7Ad04=3@`0i>CT0P82006ATI01WIfL0SXj>
02TY:@2?Shl0CTi>05iNGP1QHF40@d=304Y:BP1TI6@0JFUY06i^KP1bLW80`l?309jNWP1DE5@0>CTi
07]kN`1=CDd09bLW020P800b<S80?clo06ATI02XZ:P0Zj^[0;Bd]01bLW80Jf][07=cL`2j^[X0A4A4
03hn?P16ATH0<S8b06][J`1TI6@0=3@d059BDP2e]KD0[Jf]03lo?`1?Cdl0J6QX051@D00c<c<0=SHf
07emO@1hN7P0>c/k06=SH`23Ph<0F5QH03<c<`0Q8B40P8200<;2`P0k>c/201LG5`0l01lO7`13@d<0
@T9202TY:@1[Jf/0>c/k02@T901OGel0W9bL0;Jf]P1QHF40GemO09BDU02e]KD0^;Rh0;no_`0n?Sh0
>CTi071`L03:b/X0HV9R02<S8`1BDU80ATI6034a<@0c<c<0L71`0492@P14A4@0]kNg07MgM`1MGEd0
EeMG07AdM00T92@06QXJ01TI6@0G5aL0Ph>30;Fe]@21PH40O7al04A4A01:BTX0Q8B406IVIP21PH40
NWYj06YZJP3=cLd0b/[:04E5A@17AdL07QhN065QH@19BDT0ATI609nOW`2NWYh0NGUi5?ooool001?o
oooo08/0MgMg06e]K@1dM7@0K6a/069RHP1OGel0EeMG06QXJ01TI6@0AdM706e]K@1XJ6P0@D5105mO
G`1KFe/0@41005=CD`1MGEd0A4A4071`L01QHF40B4Q805MGE`1XJ6P0E5AD04];B`1HF5P0@d=304Y:
BP1LG5`0FEUI0492@P1=CDd0Kfm_0492@P1CDe<0JFUY0451@@1=CDd0IFEU03Lg=`1FEUH0F5QH03lo
?`1LG5`0DU9B051@D01DE5@0B4Q806IVIP1<C4`0EeMG06IVIP1<C4`0HF5Q06i^KP1:BTX0FEUI07Ui
N@1KFe/0IfMW07IfMP1YJFT0K6a/07alO01]KFd0A4A406a/K01YJFT0CDe=096AT@1WIfL0F5QH07Qh
N01RHV80LG5a06][J`1JFUX0M7Ad06MWI`1^KVh0J6QX05iNGP1UIFD0KFe]05mOG`1_Kfl0LG5a05=C
D`1gMgL0HV9R061PH01ZJVX0Lg=c07EeM@1DE5@0PX:206][J`1XJ6P0F5QH07=cL`1iNGT0EeMG06IV
IP1kNg/0LW9b05EEE@1ZJVX0PH6106a/K01bLW80NGUi08^;R`1DE5@0N7Qh08F5Q@1OGel0GEeM07al
O01HF5P0KFe]07IfMP1=CDd0TI6A06a/K01PH600O7al05mOG`1FEUH0NWYj05mOG`1FEUH0Jf][055A
D@1JFUX0Kfm_059BDP1PH600Fe]K00kooooo0680I6AT05aLG01EEED0F5QH051@D014A4@0EeMG05mO
G`1ADE40C4a<03Lg=`19BDT0De=C04a<C01WIfL0GEeM05ADE01LG5`0FUYJ04I6AP1=CDd0F5QH04];
B`1?Cdl0N7Qh05ADE01NGUh0De=C065QH@1KFe/0JVYZ05UIF@1UIFD0IfMW04I6AP1TI6@0GemO04i>
CP1FEUH0BTY:04I6AP15ADD0=cLg04e=C@15ADD0BTY:05QHF01FEUH0AdM704a<C01IFET0BDU904i>
CP1EEED0F5QH069RHP1<C4`0IfMW061PH01LG5`0K6a/065QH@1PH600U9BD09VIV@1aLG40Ng]k07in
OP1TI6@0Hf=S075aL@1IFET0CTi>05=CD`1QHF40IFEU05IFEP1/K6`0J6QX0:NWY`1ADE40GEeM06][
J`1IFET0F5QH04e=C@1GEeL0H61P04a<C01EEED0Ph>3079bLP1ADE40G5aL03hn?P1<C4`0F5QH05IF
EP80FEUI02H0Hf=S05UIF@1cLg<0Cdm?03dm?@15ADD0AdM705ADE015ADD0C4a<04A4A01fMWH0@d=3
04U9B@1@D500?clo065QH@1GEeL0Cdm?05IFEP1IFET0G5aL05mOG`1GEeL0IFEU04];B`1OGel0SHf=
04e=C@1WIfL0Kfm_06IVIP1ZJVX0EUIF05=CD`1@D500EeMG07QhN080MWIf3_ooool0NP1NGUh0KFe]
06YZJP1gMgL0?Cdm04E5A@1<C4`0Bd];03dm?@1<C4`0;2`/024Q8@14A4@0@41005EEE@1LG5`0<S8b
05ADE00e=CD0<S8b06YZJP0o?cl0D51@05=CD`1XJ6P0Bd];04m?C`0_;bl0D51@05]KF`1@D500HF5Q
04=3@`14A4@0BDU903lo?`19BDT0Fe]K065QH@19BDT0KFe]07UiN@1TI6@0EEEE08b<S00o?cl0GUiN
07alO01HF5P0K6a/07IfMP1MGEd0Jf][05EEE@1RHV80H61P075aL@1UIFD0DE5A051@D01DE5@0HV9R
06UYJ@1SHf<0EEEE06EUI@1RHV80M7Ad05QHF01nOWh0F5QH061PH01[Jf/0PH6105UIF@1]KFd0I6AT
08Z:RP1YJFT0K6a/0:VYZ@1jNWX0U9BD04a<C02IVIT0DU9B05iNGP2FUYH0FUYJ05IFEP1=CDd0P820
075aL@1OGel0B4Q808:2PP1VIVH0Cdm?05ADE01oOgl0HV9R07=cL`1IFET0QXJ606i^KP1hN7P0LW9b
07=cL`1HF5P0IFEU03/k>`1FEUH0DE5A05QHF02?Shl0G5aL06IVIP1=CDd0GEeM04Y:BP1;Bd/09BDU
0P1SHf<0401QHF40M7Ad04m?C`1CDe<0E5AD065QH@1IFET0De=C05MGE`1>CTh0DU9B04I6AP1QHF40
CTi>06][J`1LG5`>ooooo`0e030`<00U9BD09bLW0451@@2HV9P0?Cdm01hN7P0a<C40Ng]k03lo?`0W
9bL0G5aL05eMG@12@T80=cLg04M7A`1:BTX08R8R03Xj>P1mOGd0D51@06e]K@1MGEd0IFEU0410@00j
>SX0<C4a038b<P0i>CT0=3@d02l_;`1ZJVX0>SXj06m_K`1nOWh09RHV02LW9`20P800HF5Q03lo?`0g
=cL0;bl_03dm?@1@D500OWin09^KV`2HV9P0ADE506QXJ02=SHd0_Kfm09>CT`2KVi/00P2e]KD0E@24
Q8@0>CTi05eMG@1fMWH0N7Qh0:b/[025QHD0TY:B05aLG01/K6`0Rh^;09ZJVP1gMgL0Xj>S07alO02?
Shl0/k>c05aLG00N7Qh0:RXZ07emO@2SXj<0H61P038b<P0J6QX05ADE02HV9P18B4P0<C4a04i>CP0O
7al07QhN03Ph>025QHD0V9RH04M7A`0l?3`0CTi>09BDU01DE5@0MgMg0;^k^`26QXH0AdM709NGU`1a
LG40Ogmo08N7Q`0f=SH0=3@d059BDP1IFET0=cLg069RHP2/[:`0PX:209BDU02KVi/0WYjN03Ti>@0P
82006ATI03Xj>P23Ph<0J6QX0:NWY`2e]KD0N7Qh059BDP1gMgL0N7Qh05UIF@1aLG40Xj>S0<C4a01Z
JVX0DU9B07UiN@1TI6@0JFUY06EUI@0m?Cd0H61P02/[:`1NGUh05?ooool001?ooooo08/0O7al07Ad
M01^KVh0I6AT06EUI@1ADE40J6QX06MWI`1?Cdl0J6QX07MgM`1:BTX0GEeM06MWI`19BDT0D51@04=3
@`10@400GUiN05MGE`15ADD0HV9R051@D01PH600J6QX03De=@2>SXh0KVi^0410@02;Rh/0GemO04M7
A`1eMGD0L71`04E5A@1[Jf/0Hf=S04e=C@1JFUX0FEUI04Q8B01GEeL0E5AD061PH01ADE40Cdm?06a/
K018B4P0DU9B07alO00i>CT0F5QH07UiN@0f=SH0EeMG0820P00c<c<0HF5Q07YjNP10@400P82007Mg
M`10@400IVIV06YZJP1QHF40JVYZ06EUI@1SHf<0L71`07YjNP1RHV80IFEU069RHP1lO7`0M7Ad06YZ
JP1_Kfl0F5QH04M7A`26QXH0IVIV055AD@1nOWh0HF5Q04];B`2:RXX0GEeM05iNGP2:RXX0IfMW06e]
K@1bLW80HF5Q07QhN01BDU80QHF506][J`16ATH0OGem06e]K@1VIVH0N7Qh05EEE@1TI6@0RHV902`/
;01eMGD0Ogmo03hn?P22PX80Hf=S065QH@1aLG40KFe]08B4Q01GEeL0IfMW07alO019BDT0FUYJ08F5
Q@1?Cdl0LG5a079bLP1@D500H61P08b<S015ADD0GEeM07emO@12@T80CTi>05mOG`1ADE40D51@07Ee
M@1HF5P0DU9B00kooooo01X0GUiN05MGE`1LG5`0EeMG04e=C@13@d<0FEUI06IVIP1JFUX0Cdm?0410
@018B4P0Bd];04U9B@1JFUX0EEEE069RHP1SHf<0GemO04e=C@1@D500HV9R05]KF`1HF5P0KFe]04i>
CP80EUIF05X0BTY:05MGE`1RHV80DU9B05MGE`1nOWh0>3Ph05mOG`1UIFD0HV9R06MWI`1?Cdl0Fe]K
04e=C@17AdL0D51@03lo?`0e=CD0?Shn055AD@0j>SX0Cdm?059BDP15ADD0De=C04];B`1PH600KVi^
06ATI01KFe/0FEUI05mOG`1[Jf/0KVi^06ATI02<S8`0O7al07IfMP1eMGD0QXJ605QHF01IFET0L71`
05]KF`1;Bd/0G5aL05MGE`1NGUh0@T9205MGE`1[Jf/0_[jn05QHF01bLW80PH6106QXJ01GEeL0A4A4
059BDP1_Kfl0Cdm?05YJFP2;Rh/0PX:2055AD@1MGEd0Cdm?051@D01EEED0@4100451@@1MGEd0Ng]k
061PH01OGel0@D5103`l?00n?Sh0@41004i>CP10@400DE5A03lo?`25QHD0?Cdm05QHF01XJ6P0Hf=S
04e=C@1JFUX205eMG@0D05IFEP1RHV80J6QX04i>CP1GEeL0De=C05UIF@2@T900De=C079bLP1dM7@0
IVIV07IfMP1@D500Cdm?06a/K01BDU80RHV9079bLP1XJ6P>ooooo`0E05YJFP1]KFd0F5QH08:2PP10
@400>SXj055AD@1:BTX0BDU905eMG@0Q8B40>3Ph04m?C`11@D40D51@059BDP17AdL0Fe]K03De=@0o
?cl0C4a<0080@T9202`0ADE5071`L018B4P0EUIF03Hf=P1JFUX0A4A40451@@1`L700>SXj04];B`1B
DU80D51@05]KF`1?Cdl0HV9R05YJFP22PX80IfMW06YZJP1BDU80Xj>S03/k>`1NGUh0Shn?05UIF@1_
Kfl0IVIV07AdM01HF5P0AdM705]KF`10@400IVIV05QHF00n?Sh0D51@05MGE`1SHf<0MGEe06e]K@1H
F5P0Hf=S065QH@80GUiN02D0Jf][059BDP1dM7@0Ng]k09BDU01MGEd0KFe]08B4Q022PX80EEEE07mo
O`2LW9`0NWYj07UiN@1<C4`0Ng]k079bLP1RHV80PX:204e=C@17AdL0Bd];08n?S`1WIfL0EUIF05AD
E01kNg/0HF5Q04i>CP1JFUX0G5aL06ATI01aLG40BDU907=cL`1GEeL0GUiN0080KVi^0200F5QH06AT
I014A4@0EUIF04A4A01GEeL0Rh^;05MGE`1SHf<0HV9R065QH@1<C4`0E5AD01hN7P1SHf<0HV9R0820
P01aLG40?3`l05UIF@17AdL0=SHf07IfMP1lO7`0A4A406YZJP1[Jf/0:RXZ04i>CP1GEeL0MWIf03/k
>`kooooo0440<S8b02<S8`0L71`06ATI03Hf=P1LG5`0:BTY05aLG02IVIT0Cdm?03Ti>@10@400EEEE
02l_;`0j>SX0;Bd]0:JVYP26QXH0T92@0;Bd]02k^k/0_[jn09fMW@2:RXX07alO01PH600Q8B40DE5A
03/k>`1TI6@0R8R80<?3``1=CDd0HV9R05=CD`0o?cl0@41005UIF@0W9bL0820P01hN7P0M7Ad071`L
04Y:BP2=SHd0LW9b08V9R@2^[Zh0[:b/06UYJ@0l?3`0JFUY04Y:BP1`L700[:b/09bLW02KVi/0FEUI
03<c<`1VIVH0Q8B405=CD`2UYJD0`L7108:2PP0209FEU@0K05=CD`1`L700D51@0:NWY`1KFe/0IFEU
0<?3``25QHD0FEUI08b<S01KFe/0?Cdm02TY:@0M7Ad061PH08N7Q`0c<c<0?clo02@T900R8R80KVi^
07MgM`1>CTh07QhN01lO7`0N7Qh0KFe]0080G5aL02`0b/[:07EeM@1<C4`0JFUY0:j^[P2i^KT0Xj>S
07=cL`0e=CD0EUIF05aLG00f=SH0FEUI0:BTY01ZJVX0820P01lO7`1MGEd0Bd];02`/;01HF5P0KFe]
07IfMP2PX:00MGEe09fMW@1;Bd/0ATI607EeM@1dM7@0FEUI03dm?@0n?Sh0KFe]06][J`2?Shl0MGEe
06=SH`1[Jf/0<c<c01lO7`0R8R80:b/[05aLG1Cooooo000Cooooo`0k07AdM029RHT0JVYZ06m_K`1^
KVh0GUiN06MWI`1bLW80HF5Q04U9B@1BDU80JFUY03`l?01]KFd0GemO02l_;`1ADE40MGEe03lo?`1L
G5`0MGEe03Ti>@1FEUH0K6a/03/k>`1?Cdl0CTi>038b<P1PH600Bd];04a<C01/K6`0AdM706][J`1H
F5P0@d=307AdM01ADE40ADE508J6QP14A4@0D51@06YZJP11@D40D51@05mOG`1=CDd0FEUI06YZJP1N
GUh0BDU905UIF@1=CDd0CTi>05IFEP1HF5P0J6QX065QH@1JFUX00P1[Jf/03`1`L700MWIf05iNGP1i
NGT0O7al03De=@1SHf<0M7Ad02l_;`1dM7@0QHF503Ti>@1[Jf/0IFEU0492@P0206a/K00m04M7A`1h
N7P0MGEe05aLG01XJ6P0HF5Q071`L01ADE40F5QH07alO01FEUH0FUYJ071`L023Ph<0Hf=S0:FUY@1O
Gel0DU9B09JFUP1OGel0I6AT05QHF01cLg<0OGem04Q8B01aLG40MGEe061PH01WIfL0EEEE08J6QP2;
Rh/0EeMG07=cL`2<S8`0CTi>07YjNP23Ph<0Cdm?07YjNP22PX80=SHf05ADE01^KVh0?3`l06i^KP1M
GEd0I6AT08>3P`1JFUX0IVIV07]kN`1SHf<0Fe]K0861P@1MGEd0Fe]K07moO`1IFET0HV9R05UIF@0>
ooooo`2<06YZJP1PH600EUIF05UIF@18B4P0@T9205ADE01RHV80FUYJ05UIF@0i>CT0CTi>0451@@1;
Bd/0KVi^065QH@1UIFD0J6QX06MWI`11@D40ATI605mOG`1?Cdl0C4a<06ATI016ATH0F5QH04A4A01D
E5@0CTi>05aLG01EEED0Cdm?06EUI@13@d<0Ph>307IfMP1EEED0FEUI04M7A`15ADD0?3`l03lo?`1:
BTX0@d=30410@01;Bd/0B4Q803/k>`1?Cdl0D51@04];B`1=CDd0HF5Q07AdM01OGel0IFEU06MWI`1F
EUH0JVYZ06ATI01YJFT0IVIV06YZJP29RHT0JFUY071`L023Ph<0Fe]K05aLG01EEED0E5AD05QHF01U
IFD0G5aL05iNGP1CDe<0KFe]05]KF`2HV9P0GemO06i^KP1UIFD0IfMW05UIF@1BDU80EeMG079bLP1M
GEd0IfMW08^;R`28R8P0GEeM05YJFP0i>CT0De=C05eMG@1SHf<0DE5A07AdM01TI6@0Cdm?05iNGP1G
EeL0=cLg03dm?@13@d<0EeMG0451@@1?Cdl0@D51079bLP12@T80Bd];05aLG01WIfL0HV9R05mOG`1J
FUX0Bd];04e=C@1LG5`0J6QX059BDP1NGUh0@d=306EUI@2=SHd0ATI607emO@1/K6`0G5aL05ADE01I
FET0DE5A071`L01@D500MGEe07MgM`29RHT>ooooo`0A05iNGP1IFET0MGEe09VIV@0_;bl0>SXj03Ph
>01KFe/0=cLg04E5A@0`<300;Bd]04I6AP0c<c<0>3Ph061PH01FEUH00P0f=SH0I00j>SX0I6AT03`l
?011@D40EEEE05iNGP1EEED0EeMG034a<@1EEED0DU9B03<c<`1XJ6P0=CDe04a<C01LG5`0CTi>05iN
GP1HF5P0JFUY04M7A`1CDe<0PX:205YJFP10@400W9bL05MGE`1IFET0L71`075aL@1NGUh0KVi^06YZ
JP1`L700DE5A05=CD`1;Bd/0FEUI05]KF`0o?cl0ADE505QHF01TI6@0LG5a06IVIP1EEED0F5QH07mo
O`14A4@0FEUI07IfMP0g=cL0M7Ad069RHP1nOWh0ADE506IVIP1/K6`0RXZ:06][J`1QHF40L71`08b<
S028R8P0>CTi07emO@1dM7@0De=C08j>SP1;Bd/0CDe=059BDP2>SXh0L71`04a<C011@D40Ph>306=S
H`1;Bd/0KFe]05EEE@14A4@0Q8B405QHF022PX80DU9B05eMG@1/K6`0QXJ605QHF01JFUX0A4A4065Q
H@1=CDd0?Shn07alO01/K6`0D51@05mOG`1MGEd204Y:BP0C02LW9`1ZJVX0EUIF05IFEP1RHV80E5AD
06m_K`1?Cdl0ADE505QHF01SHf<0?3`l08>3P`1lO7`07alO03lo?`1HF5P0GUiN02d];@0>ooooo`1B
03Xj>P0T92@071`L01LG5`0O7al0M7Ad0:BTY029RHT0U9BD04M7A`11@D40=cLg038b<P17AdL0DE5A
03hn?P1QHF40;Rh^04Y:BP2o_kl0Z:RX09ZJVP1>CTh0=cLg07EeM@1CDe<0EeMG04A4A01LG5`0SHf=
04e=C@0o?cl0TI6A0;Ng]`25QHD0SHf=079bLP0k>c/07QhN01`L700J6QX07AdM01XJ6P15ADD0J6QX
07moO`2KVi/0];Bd09nOW`2l_;`0IfMW03Xj>P2>SXh0UIFE08>3P`2VYZH0]kNg071`L00m?Cd0>SXj
04M7A`1EEED0_;bl0<W9b@1WIfL0BTY:05ADE01CDe<0JFUY06IVIP1DE5@0FUYJ061PH02NWYh0IfMW
020P800K6a/0J6QX05ADE00o?cl0F5QH05aLG080W9bL00H0N7Qh06IVIP1nOWh0^;Rh0;^k^`0i>CT2
01dM7@0`01lO7`10@400:2PX03Lg=`2FUYH0=cLg03De=@1OGel0EEEE09bLW02TY:@0QhN702<S8`0_
;bl0HV9R0820P02b/[80`<3005iNGP14A4@0>c/k071`L020P800M7Ad0:n_[`1SHf<0OWin09>CT`25
QHD0I6AT0;Zj^P1@D500?Cdm03<c<`0V9RH08b<S01lO7`12@T80FEUI08B4Q01/K6`0>3Ph03De=@0a
<C4071`L01lO7`0[:b/0Shn?5?ooool001?ooooo08/0PX:206YZJP1bLW80GemO05eMG@1TI6@0CDe=
08R8R01RHV80GEeM07QhN01DE5@0DU9B0861P@14A4@0E5AD06QXJ013@d<0F5QH05eMG@11@D40H61P
05MGE`1UIFD0NGUi03Lg=`1JFUX0JVYZ03Lg=`1dM7@0QHF503lo?`1fMWH0SHf=03hn?P1VIVH0Ogmo
04=3@`1>CTh0EeMG04I6AP1RHV80BTY:075aL@1PH600@d=3071`L01NGUh0?Shn07inOP1>CTh0E5AD
07alO00m?Cd0De=C08j>SP0j>SX0FUYJ08>3P`0o?cl0Jf][07QhN01=CDd0De=C08f=S@1@D500F5QH
06a/K01JFUX0HV9R07EeM@1DE5@0Hf=S06a/K01RHV80KVi^05aLG01WIfL0Lg=c04m?C`1bLW80LG5a
04Y:BP2RXZ80Ng]k05QHF02VYZH0Hf=S04a<C027QhL0D51@06EUI@1gMgL0IfMW06][J`1KFe/0LW9b
08j>SP1CDe<0DU9B0820P01dM7@0Lg=c05eMG@1VIVH0VYZJ0492@P1YJFT0LG5a04Y:BP29RHT0PH61
05iNGP24Q8@0I6AT06MWI`1ZJVX0G5aL08B4Q01/K6`0EUIF07UiN@1UIFD0Hf=S06a/K01DE5@0FUYJ
07QhN019BDT0FUYJ07QhN019BDT0ADE5069RHP1DE5@0IVIV07moO`1BDU80Bd];00kooooo00H0JFUY
05=CD`1HF5P0De=C04E5A@1:BTX2069RHP0=05MGE`1LG5`0>c/k04I6AP17AdL0CTi>05mOG`1QHF40
IVIV05aLG01MGEd0@D5104];B`0204m?C`1:04Q8B01]KFd0@d=305ADE017AdL0EUIF04];B`1LG5`0
Cdm?05aLG01SHf<0D51@079bLP1WIfL0G5aL06EUI@1DE5@0Bd];04a<C01;Bd/0DE5A04m?C`1:BTX0
Fe]K0492@P1BDU80Cdm?05mOG`1<C4`0Fe]K06QXJ01YJFT0EUIF071`L01JFUX0EUIF079bLP1/K6`0
HF5Q05YJFP1jNWX0QXJ606m_K`1KFe/0Ogmo05MGE`1MGEd0FEUI05EEE@1UIFD0Jf][06QXJ01VIVH0
DE5A06EUI@1cLg<0][Jf05ADE01iNGT0GemO06MWI`19BDT0D51@05iNGP1@D500K6a/06=SH`2OWil0
TY:B06i^KP1CDe<0CDe=05UIF@1PH60205ADE00Y07MgM`1oOgl0D51@065QH@18B4P0>c/k04A4A01B
DU80EeMG04U9B@1=CDd0BTY:08n?S`1LG5`0EUIF04m?C`1:BTX0CDe=05]KF`1JFUX0EeMG04a<C01^
KVh0PX:204m?C`1KFe/0@d=305iNGP24Q8@0DE5A08^;R`1nOWh0Ng]k06EUI@1EEED0BDU906ATI01S
Hf<0[:b/08:2PP1aLG403_ooool0501FEUH0HV9R05YJFP1oOgl0@T92034a<@10@400A4A403`l?019
BDT071`L03/k>`1=CDd0>3Ph05YJFP11@D40Lg=c04a<C00b<S80@T920P1:BTX0C00n?Sh0AdM706i^
KP1BDU80E5AD02h^;P1HF5P0DU9B030`<01/K6`0=SHf04Q8B01`L700@41005MGE`1DE5@0KVi^04U9
B@1UIFD0RHV9061PH01UIFD0[:b/055AD@1RHV80Ti>C07EeM@1^KVh0GEeM07QhN01IFET0AdM7065Q
H@1:BTX0J6QX05QHF00m?Cd0De=C055AD@1WIfL0G5aL069RHP1MGEd0BDU907UiN@1EEED0F5QH079b
LP10@400HF5Q04U9B@2=SHd0ATI606YZJP1[Jf/0GUiN05IFEP1oOgl0TY:B08B4Q027QhL0E5AD06][
J`1QHF40HV9R07moO`19BDT0?Shn04=3@`1oOgl0Cdm?05=CD`1;Bd/0QHF505YJFP14A4@205eMG@0K
04=3@`1hN7P0@41007UiN@17AdL0NWYj05IFEP1RHV80BTY:06][J`0o?cl0H61P0451@@19BDT0PH61
05EEE@1>CTh0Fe]K04Y:BP0c<c<0De=C030`<01?Cdl0<c<c07UiN@1OGel0;Rh^0080GEeM00/0?Shn
05aLG01/K6`0CTi>07YjNP1eMGD0?3`l03<c<`1QHF40M7Ad05QHF00>ooooo`0A05iNGP0a<C408R8R
02HV9P0/;2`0WYjN0;2`/02SXj<0N7Qh03<c<`0`<300=SHf02LW9`0h>3P0IVIV092@T01eMGD00P0n
?Sh0201PH600LW9b09>CT`1JFUX071`L07YjNP0a<C408b<S0P0F5QH0;P0k>c/0MGEe04E5A@2;Rh/0
[Zj^09ZJVP2EUID0/k>c05=CD`0d=3@0AdM702d];@0I6AT092@T06ATI00b<S80>SXj07emO@1ZJVX0
;bl_06a/K01OGel0/;2`08Z:RP1NGUh0Jf][0:ZZZP2NWYh0JFUY01TI6@0F5QH07QhN04m?C`2FUYH0
alO709:BTP1OGel0ADE502d];@1>CTh0:BTY034a<@1RHV80OWin07=cL`1:BTX201DE5@0o01dM7@1P
H600GemO096AT@1QHF40Cdm?09RHV02SXj<0Cdm?055AD@2d];@0a<C406ATI00X:2P071`L02/[:`0a
<C40?3`l06IVIP2SXj<0=cLg03@d=01aLG40Jf][03lo?`1ADE40IfMW08Z:RP1kNg/0`<300;no_`0d
=3@0Jf][08:2PP16ATH0@d=306UYJ@2[Zj/0Q8B40:RXZ025QHD0?Cdm04i>CP2BTY80@d=308>3P`0T
92@08b<S030`<00U9BD08R8R020P800a<C40Jf][07moO`1_Kfl0@41002XZ:P0T92@07AdM02@T901S
Hf<0_Kfm01Cooooo000Cooooo`0D07YjNP1mOGd0L71`06=SH`1QHF40AdM706ATI01aLG40AdM705QH
F01VIVH0DU9B05=CD`1lO7`0G5aL03lo?`1EEED0HV9R06UYJ@1KFe/205eMG@0Z04Y:BP1jNWX0EUIF
04Y:BP1GEeL0A4A405ADE01RHV80FEUI05ADE012@T80I6AT05mOG`1>CTh0KVi^04U9B@0n?Sh0FEUI
04Y:BP1BDU80JFUY04e=C@19BDT0KVi^05]KF`1CDe<0F5QH05UIF@0l?3`0G5aL05MGE`16ATH0EeMG
061PH01EEED0Fe]K05YJFP1UIFD0HF5Q06i^KP1aLG40K6a/0P1fMWH0901@D500Fe]K08Z:RP1BDU80
H61P07QhN012@T80JVYZ06e]K@1CDe<0Jf][06EUI@1NGUh0EeMG08V9R@1TI6@0IVIV08V9R@1WIfL0
JVYZ07]kN`1DE5@0EeMG07UiN@1ZJVX0Jf][04m?C`23Ph<0L71`051@D020P800JFUY05aLG01TI6@0
J6QX08F5Q@80G5aL01`0KVi^06MWI`1CDe<0Lg=c06MWI`1`L700IVIV07inOP28R8P0FEUI06a/K01k
Ng/0CDe=07=cL`20P800Cdm?05]KF`1hN7P0@41006YZJP1RHV80DE5A06YZJP1/K6`0@T9206a/K01H
F5P0FEUI0P1LG5`01@1<C4`0GUiN06YZJP1BDU80GEeM00kooooo0P1EEED0101ADE40F5QH055AD@16
ATH205aLG00705UIF@1BDU80?Cdm04=3@`12@T80B4Q806UYJ@0205iNGP1Q061PH01SHf<0@41004A4
A01>CTh0B4Q804A4A01DE5@0@41004Q8B010@400ADE504U9B@1BDU80BDU904Y:BP1PH600?clo06EU
I@1>CTh0DE5A04m?C`18B4P0?3`l03Xj>P1@D500?Shn03Lg=`0k>c/0Bd];03Ph>010@400@T9205MG
E`0n?Sh0GEeM06a/K01IFET0I6AT05QHF01NGUh0H61P06YZJP1OGel0IVIV06][J`1aLG40TY:B07Qh
N01jNWX0QhN704e=C@1DE5@0Hf=S05]KF`18B4P0IVIV05ADE01IFET0Cdm?06a/K01cLg<0XJ6Q05eM
G@1[Jf/0IVIV06ATI01CDe<0AdM7065QH@19BDT0HV9R05QHF02SXj<0Ogmo04m?C`1ADE40C4a<05mO
G`1LG5`0CTi>05YJFP1YJFT0O7al04i>CP1UIFD0B4Q803Ti>@2=SHd0EeMG05QHF015ADD0AdM704E5
A@1kNg/0C4a<05ADE00204e=C@0H04i>CP1ZJVX0Fe]K059BDP19BDT0Hf=S06YZJP1BDU80FEUI059B
DP1VIVH0T92@06=SH`29RHT0UIFE0820P01hN7P0H61P06QXJ01ZJVX0Fe]K08^;R`1]KFd0OWin3_oo
ool0901BDU80EUIF06e]K@1gMgL0@T9204A4A00c<c<0CTi>03Xj>P11@D40=cLg02LW9`1?Cdl0=3@d
04I6AP18B4P0JFUY03Ph>013@d<0@T9204m?C`1ADE40=CDe05=CD`1OGel0De=C05aLG00e=CD0Cdm?
05iNGP18B4P0Rh^;03Ph>01BDU80MGEe04A4A080D51@0240M7Ad051@D01KFe/0UIFE06EUI@1GEeL0
MWIf092@T01eMGD0PH610820P01RHV80EeMG06YZJP1UIFD0AdM705eMG@19BDT0GemO05ADE00e=CD0
DU9B05MGE`1IFET0GemO05aLG01OGel0@41008F5Q@15ADD0FEUI07=cL`0j>SX00P1TI6@0701UIFD0
@41006YZJP1RHV80FEUI065QH@1iNGT0QXJ60:6QX@2RXZ80CDe=069RHP1YJFT0Fe]K07=cL`17AdL0
@T9204a<C01eMGD0HF5Q04A4A00h>3P0S8b<05aLG00n?Sh0FUYJ05aLG00o?cl205aLG00U06e]K@0o
?cl0JVYZ069RHP1EEED0De=C06QXJ00l?3`0Fe]K04U9B@1:BTX0OGem065QH@1RHV80F5QH04i>CP1:
BTX0CTi>02DU9@1VIVH0DU9B06ATI01LG5`0EeMG04Q8B01:BTX0F5QH06YZJP1<C4`0D51@08:2PP1k
Ng/0De=C02PX:01NGUh0LG5a04];B`0>ooooo`0<069RHP1]KFd0MWIf079bLP1lO7`0]kNg0;6a/@2X
Z:P0T92@05ADE01;Bd/0F5QH0P0g=cL0301;Bd/0PX:204];B`1=CDd0De=C038b<P14A4@0?Shn06QX
J01lO7`0IfMW01TI6@@05ADE02d0EUIF08Z:RP1WIfL0V9RH0:JVYP1mOGd0L71`08Z:RP1NGUh0@410
02`/;00a<C40QXJ608V9R@0Z:RX0820P04M7A`1[Jf/061PH020P8014A4@0KVi^04=3@`0O7al0<c<c
05=CD`1MGEd0Bd];02HV9P0M7Ad07QhN028R8P1fMWH0Z:RX069RHP0O7al0:RXZ03`l?00a<C407QhN
01/K6`0a<C40Jf][06EUI@1KFe/00P0E5AD0;P0F5QH0FEUI0:2PX02>SXh0T92@08R8R01QHF40IfMW
0:^[Z`1nOWh0/[:b0;Bd]01dM7@0HF5Q06m_K`1jNWX0GemO04I6AP0n?Sh0LG5a05iNGP25QHD0EeMG
05mOG`2>SXh0FUYJ04Y:BP2HV9P0EUIF06ATI01bLW80OWin0:j^[P2][Jd0PX:207]kN`2MWId0_;bl
0;no_`36a/H0WYjN06a/K01eMGD0MgMg06EUI@1oOgl2020P800601`L700R8R807alO020P801>CTh0
W9bL0P2k^k/01`1_Kfl0JVYZ06i^KP2DU9@0Rh^;06QXJ02EUID05?ooool001?ooooo00H0LW9b07em
O@1QHF40MgMg05QHF01QHF4205aLG00Y069RHP0o?cl0DE5A07YjNP0o?cl0JVYZ06UYJ@0W9bL0G5aL
05YJFP0n?Sh0Jf][07YjNP0m?Cd0F5QH079bLP11@D40EeMG059BDP0i>CT0FEUI04a<C01FEUH0HV9R
04M7A`1hN7P0G5aL0451@@27QhL0BTY:055AD@1dM7@0<c<c05QHF01CDe<0?clo065QH@1FEUH0B4Q8
06MWI`12@T800P1;Bd/0F01LG5`0F5QH05]KF`0j>SX0KFe]05QHF00m?Cd0Ng]k07=cL`19BDT0Hf=S
07AdM01>CTh0Jf][07]kN`17AdL0GUiN06a/K015ADD0IFEU06QXJ01TI6@0N7Qh06=SH`1DE5@0M7Ad
05IFEP1_Kfl0FEUI05]KF`20P800EeMG071`L01/K6`0CTi>075aL@1OGel0BDU906UYJ@1bLW80Cdm?
06a/K029RHT0D51@05iNGP1fMWH0Hf=S07emO@1=CDd0HF5Q05eMG@19BDT0MWIf05mOG`1PH600G5aL
06EUI@20P800EeMG05EEE@2@T900KFe]04a<C020P800IFEU051@D01YJFT0JVYZ05mOG`1QHF40N7Qh
05=CD`27QhL0GEeM06=SH`1eMGD0E5AD06EUI@1^KVh0EEEE06a/K01=CDd0EUIF071`L015ADD0De=C
07MgM`0o?cl>ooooo`0A05iNGP1LG5`0FEUI05]KF`1>CTh0BTY:05aLG01YJFT0DU9B04m?C`0g=cL0
?3`l03hn?P1CDe<0L71`05]KF`1YJFT00P1`L70204Q8B00U04e=C@15ADD0ATI604U9B@16ATH0?Cdm
04A4A00g=cL0Bd];04m?C`1@D500BTY:05iNGP0j>SX0IfMW04];B`1<C4`0@d=303hn?P11@D40=3@d
0451@@0h>3P0:RXZ03dm?@13@d<0>CTi03lo?`1=CDd0FEUI04U9B@1KFe/0KFe]05eMG@1ZJVX0LW9b
06=SH`0205eMG@1@065QH@1GEeL0Ogmo07MgM`2:RXX0IVIV07emO@28R8P0A4A405EEE@1QHF40Hf=S
06ATI02;Rh/0GemO05MGE`1QHF40LW9b07emO@2_[jl0CDe=07=cL`1TI6@0J6QX04Y:BP0n?Sh0De=C
059BDP1LG5`0E5AD09FEU@1_Kfl0E5AD07]kN`1]KFd0IVIV05MGE`1@D500E5AD06][J`1aLG40A4A4
06YZJP1CDe<0ADE50492@P1BDU80Fe]K051@D01CDe<0AdM706EUI@19BDT0GemO05EEE@1:BTX0CDe=
06MWI`1EEED0Fe]K04E5A@1PH600J6QX06][J`1EEED0@d=305eMG@26QXH0GEeM07IfMP1mOGd0JVYZ
05]KF`1FEUH0EeMG04E5A@18B4P0N7Qh06MWI`1^KVh>ooooo`10055AD@19BDT0F5QH04Y:BP0_;bl0
?Shn034a<@0f=SH0:BTY03@d=00V9RH09BDU05EEE@0f=SH0?3`l03Xj>P1FEUH0>SXj04=3@`0l?3`0
>3Ph0492@P0j>SX0AdM706ATI01UIFD0GEeM03Ti>@1?Cdl0AdM703Ph>029RHT0>c/k04Y:BP1^KVh0
?Shn04I6AP14A4@0JFUY04A4A01RHV80YjNW06ATI01PH600A4A40<g=c@2a/K40UYJF09jNWP1cLg<0
FEUI08>3P`1QHF40FUYJ06i^KP1DE5@0GUiN05EEE@12@T80EeMG055AD@1QHF40Jf][05iNGP80JFUY
0380QXJ604a<C01ZJVX0N7Qh03lo?`1aLG40IVIV08Z:RP1VIVH0L71`06i^KP1PH600DE5A08F5Q@2K
Vi/0VIVI08Z:RP1_Kfl0D51@05aLG01^KVh0MWIf03Xj>P0k>c/0CTi>08f=S@1[Jf/0A4A4069RHP2K
Vi/0HV9R05MGE`1VIVH0M7Ad05aLG01SHf<0FUYJ069RHP10@400GUiN069RHP1KFe/0DE5A07moO`16
ATH0GUiN03/k>`1DE5@0OGem04e=C@80I6AT01H0C4a<04U9B@14A4@0>3Ph06i^KP17AdL0IfMW05IF
EP1RHV80E5AD05YJFP1mOGd0F5QH059BDP1UIFD0R8R804U9B@1`L7007AdM04U9B@1LG5`0GEeM3_oo
ool06P1CDe<0/[:b079bLP10@400=SHf071`L01;Bd/0K6a/0:BTY01bLW80GUiN05EEE@1fMWH0;Bd]
03dm?@2ZZZX0G5aL03Ti>@0U9BD0;2`/01/K6`0G5aL0:b/[079bLP2<S8`0BDU90P0E5AD0L00D51@0
6QXJ071`L00[:b/0;bl_05UIF@1GEeL08b<S01dM7@1JFUX0GUiN05mOG`1EEED0R8R808B4Q01MGEd0
JFUY06][J`2?Shl0RHV9024Q8@0F5QH09RHV05IFEP0a<C404Q8B02HV9P1lO7`0MgMg06=SH`1ZJVX0
A4A401hN7P0K6a/0Rh^;0=cLg01UIFD06a/K02XZ:P0g=cL08R8R01hN7P0H61P092@T05YJFP25QHD0
];Bd03/k>`0H61P0A4A40861P@1dM7@0T92@05iNGP1>CTh08b<S04m?C`1mOGd0:b/[055AD@1/K6`0
FEUI05mOG`1bLW80EUIF030`<010@400Cdm?05aLG00X:2P0][Jf0=7Ad@37alL0YJFU071`L01_Kfl0
MGEe03<c<`11@D40a/K60:^[Z`1SHf<0ADE503lo?`13@d<0Ng]k0:RXZ02BTY80FUYJ05mOG`0/;2`0
;bl_05UIF@1PH600UiNG02`/;00];Bd0@T9203<c<`0g=cL0NWYj07moO`2=SHd0ZJVY0:6QX@2LW9`0
[:b/0=?Cd`1lO7`0Cdm?06YZJP1LG5`Dooooo`004oooool0R`27QhL0LW9b071`L01IFET0IVIV05eM
G@1>CTh0Ti>C05]KF`1PH600JFUY03lo?`1`L700JFUY0451@@1NGUh0I6AT03dm?@1FEUH0IfMW0451
@@1SHf<0G5aL06YZJP1fMWH0;Rh^07]kN`1IFET0>SXj069RHP1YJFT0=cLg05ADE01]KFd0BDU904a<
C01UIFD0?clo05EEE@1VIVH0@41005aLG01IFET0GUiN051@D01UIFD0MgMg051@D01;Bd/0JFUY03Ti
>@1ADE40L71`03lo?`1ADE40LG5a04];B`1PH600CTi>05MGE`1lO7`0IfMW05ADE023Ph<0GEeM05AD
E01aLG40De=C06YZJP1TI6@0K6a/06][J`1bLW80K6a/06][J`1lO7`0Bd];06i^KP1XJ6P0ATI6096A
T@1YJFT0EeMG0:FUY@1ZJVX0Bd];09fMW@1KFe/0G5aL08Z:RP1@D500HF5Q06a/K01VIVH0RXZ:05EE
E@1QHF40L71`04];B`1[Jf/0L71`05QHF022PX80BTY:06ATI026QXH0:b/[07MgM`1hN7P0CDe=07Ee
M@1XJ6P0L71`05mOG`1LG5`0O7al055AD@1QHF40VIVI069RHP1IFET0X:2P05IFEP1BDU80PH6104a<
C01YJFT0Ng]k05=CD`1UIFD0LW9b05IFEP1RHV80KFe]04E5A@1UIFD0Kfm_04U9B@1QHF403_ooool0
O01CDe<0EEEE05IFEP1>CTh0IVIV051@D01SHf<0JVYZ05aLG01<C4`0>3Ph03Xj>P19BDT0C4a<06m_
K`1NGUh0IfMW069RHP1MGEd0BDU90492@P1=CDd0AdM704Y:BP1HF5P0CDe=04Y:BP11@D40CDe=05=C
D`1@D500C4a<04i>CP1GEeL0E5AD07MgM`1TI6@0C4a<051@D01IFET0?3`l0451@@17AdL0@D5103lo
?`16ATH0CTi>03hn?P1;Bd/0A4A4059BDP1>CTh0EeMG07YjNP1IFET0EeMG07alO010@400?clo05QH
F01`L700Cdm?08f=S@1mOGd0P82005UIF@1VIVH0RHV905IFEP0j>SX0BDU9059BDP1MGEd0GUiN05EE
E@16ATH0JFUY071`L02;Rh/0V9RH06][J`1SHf<0NWYj05eMG@1:BTX0Cdm?05IFEP1@D500H61P069R
HP2TY:@0UYJF05IFEP1_Kfl0FUYJ059BDP1SHf<0De=C04I6AP1XJ6P0K6a/04E5A@1JFUX0DU9B055A
D@17AdL0@T9205ADE012@T80CDe=04M7A`28R8P0EeMG04Q8B01NGUh0ATI604e=C@1>CTh0ATI605eM
G@1@D500E5AD05IFEP1BDU82051@D00405ADE026QXH0NWYj07MgM`80L71`00P0BTY:05QHF01IFET0
AdM703hn?P1mOGd0LW9b06m_K`kooooo03H0E5AD05QHF01]KFd0IfMW05aLG017AdL0=SHf04];B`0k
>c/0BTY:03<c<`0P8200ADE502/[:`16ATH0ADE5051@D00f=SH0CDe=0410@01CDe<0DU9B04M7A`1D
E5@0KFe]07EeM@1HF5P0A4A404Q8B01<C4`0GEeM0:f][@19BDT0BTY:06UYJ@0o?cl0De=C038b<P28
R8P0DE5A04e=C@2VYZH0LG5a05MGE`0^;Rh0Xj>S0=3@d01^KVh0SXj>06IVIP1KFe/0OWin069RHP1L
G5`206MWI`1D061PH01MGEd0C4a<05QHF01>CTh0Kfm_06i^KP1=CDd0IfMW05UIF@2ATI40@d=305UI
F@1^KVh0@D5106YZJP1SHf<0GemO05aLG01UIFD0KFe]04i>CP1XJ6P0TY:B06][J`2MWId0SHf=06QX
J01CDe<0JVYZ075aL@1JFUX0=SHf03@d=01>CTh0SHf=061PH01BDU80Hf=S07inOP1aLG40De=C05mO
G`1eMGD0JVYZ05YJFP1CDe<0EeMG04A4A01@D500JFUY04M7A`1=CDd0PH6104];B`15ADD0=SHf05]K
F`1cLg<0@41008N7Q`1KFe/0;Bd]05iNGP16ATH07alO065QH@1LG5`0MGEe059BDP1CDe<0G5aL04=3
@`1gMgL0K6a/03dm?@1HF5P0VYZJ05eMG@1UIFD09bLW065QH@1_Kfl0IfMW3_ooool04@1>CTh0BTY:
04I6AP0/;2`0820P024Q8@11@D40De=C0:VYZ@2?Shl0QHF509bLW01LG5`0;bl_02d];@2IVIT0GEeM
00808B4Q03P07alO01LG5`0E5AD05ADE04I6AP2?Shl0ZJVY07IfMP1KFe/0Hf=S08>3P`36a/H0IFEU
02@T901LG5`0C4a<01TI6@0H61P071`L05=CD`13@d<0KVi^08J6QP1MGEd0@410020P800T92@0A4A4
07AdM01`L700CDe=02XZ:P1XJ6P06ATI018B4P1OGel0OWin04I6AP0N7Qh0G5aL09BDU02SXj<0LW9b
0<?3``3De=@0`/;205ADE00j>SX06a/K01XJ6P0Y:BT0:2PX028R8P0f=SH0d];B0;Vi^@80YjNW03l0
WYjN06=SH`0f=SH0HF5Q08B4Q01YJFT0;bl_02h^;P0f=SH061PH01dM7@1DE5@0Q8B409JFUP1/K6`0
?Shn02LW9`19BDT0S8b<08V9R@11@D40_[jn07MgM`1jNWX0Rh^;05mOG`2LW9`0Jf][05]KF`23Ph<0
ZJVY0;Jf]P20P800:b/[04=3@`0^;Rh0QHF509^KV`1VIVH0>CTi03Ph>00b<S80;bl_03dm?@2<S8`0
YjNW0820P01GEeL0@41006UYJ@1oOgl0LW9b08Z:RP2BTY80K6a/09JFUP2ZZZX0^;Rh0:f][@27QhL0
/[:b0861P@1PH6005?ooool001?ooooo08H0MWIf07=cL`1MGEd0JVYZ05YJFP1OGel0JVYZ075aL@1E
EED0CDe=07=cL`1LG5`0Fe]K06m_K`1PH600AdM706EUI@1SHf<0E5AD04a<C01aLG40D51@05QHF01c
Lg<0De=C05UIF@1LG5`0B4Q805QHF01LG5`0EEEE05UIF@1BDU80GEeM05YJFP15ADD0HV9R05QHF01@
D500DU9B05IFEP1BDU80G5aL059BDP18B4P0EeMG06QXJ01EEED0C4a<06=SH`16ATH0HV9R06QXJ01F
EUH0EEEE06=SH`1RHV80De=C05UIF@1cLg<0DU9B07UiN@1dM7@0EUIF06MWI`1hN7P0HV9R05YJFP1g
MgL0AdM706m_K`1XJ6P0CTi>06ATI01mOGd0GEeM06ATI01bLW80GUiN065QH@1kNg/0H61P05iNGP2H
V9P0HV9R05=CD`1lO7`0JFUY05ADE01_Kfl0HV9R06EUI@1VIVH0Ng]k079bLP1QHF40O7al05eMG@1Z
JVX0HF5Q05iNGP1eMGD0GUiN06=SH`1VIVH0GUiN04m?C`1_Kfl0IVIV06EUI@1YJFT0O7al08B4Q01?
Cdl0K6a/0820P015ADD0KFe]07=cL`19BDT0GEeM07QhN01:BTX0Lg=c07EeM@1@D500MWIf06MWI`1C
De<0N7Qh05aLG01MGEd0I6AT06YZJP80F5QH00<0L71`05YJFP1TI6@03_ooool02P1GEeL0J6QX05]K
F`1MGEd0FEUI04e=C@1PH600GemO04i>CP1<C4`203hn?P0n04=3@`1BDU80K6a/05]KF`1PH600I6AT
061PH017AdL0BDU904Y:BP13@d<0A4A4051@D018B4P0>3Ph03`l?00d=3@0BTY:055AD@1>CTh0BTY:
05EEE@0l?3`0JFUY05MGE`1EEED0D51@05MGE`0h>3P0?Shn04M7A`0m?Cd0@T9203/k>`1@D500>c/k
03lo?`14A4@0Fe]K04m?C`1HF5P0LG5a05MGE`1XJ6P0JVYZ05QHF019BDT0IFEU065QH@1HF5P0YJFU
08f=S@1YJFT0JVYZ065QH@1eMGD0EEEE04E5A@1EEED0CTi>055AD@1VIVH204E5A@1006MWI`1MGEd0
Lg=c096AT@1UIFD0HF5Q06][J`1TI6@0CDe=04];B`1=CDd0B4Q805aLG01KFe/0UYJF07]kN`1NGUh0
M7Ad04m?C`1IFET0FUYJ04U9B@11@D40H61P05eMG@17AdL0ADE504I6AP0n?Sh0B4Q804U9B@1IFET0
@41004I6AP1>CTh0N7Qh059BDP14A4@0Hf=S0451@@1EEED0DU9B05ADE01UIFD0BTY:04m?C`1EEED0
Bd];0492@P1=CDd0EEEE06][J`1QHF40K6a/065QH@1ZJVX0Fe]K05MGE`1NGUh0CTi>04a<C01]KFd0
L71`07IfMPkooooo03H0CTi>05MGE`1QHF40CTi>051@D014A4@0>CTi0451@@0[:b/0>CTi02`/;00a
<C40?3`l03Xj>P0k>c/0>3Ph05IFEP0h>3P0De=C03Hf=P0l?3`0FUYJ03De=@1>CTh0De=C06EUI@13
@d<0?3`l04E5A@1LG5`0CDe=09bLW00h>3P0@T9205YJFP0n?Sh0BTY:03@d=01dM7@0BTY:05aLG02/
[:`0GUiN059BDP0h>3P0QhN706UYJ@1]KFd0YZJV05iNGP1MGEd0KFe]055AD@1;Bd/205]KF`1D04m?
C`1HF5P0B4Q805]KF`1>CTh0IfMW06ATI01DE5@0EeMG06e]K@2?Shl0BTY:06ATI01hN7P0?Shn075a
L@1cLg<0GUiN07IfMP1KFe/0JVYZ05QHF01MGEd0Ogmo06m_K`2VYZH0SHf=07moO`1MGEd0HV9R079b
LP1LG5`0De=C03Ti>@1>CTh0NWYj06MWI`13@d<0LW9b06MWI`1PH600EUIF079bLP1JFUX0HV9R05mO
G`1VIVH0EEEE04i>CP1fMWH0OGem04I6AP1iNGT0R8R805QHF01FEUH0KFe]05MGE`1eMGD0IVIV07em
O@1QHF40Cdm?05mOG`1:BTX0?Cdm07AdM01FEUH0SHf=05eMG@1SHf<0J6QX06ATI01QHF40GUiN055A
D@1GEeL0RHV905UIF@29RHT09RHV05iNGP1BDU80HF5Q3_ooool0M`22PX80<c<c02h^;P0H61P05QHF
018B4P0U9BD0L71`07=cL`2?Shl0c<c<06ATI01/K6`0FEUI06ATI02HV9P0PX:2071`L016ATH0@410
03Ph>013@d<0:b/[03/k>`1fMWH0PX:205UIF@0h>3P0<C4a02h^;P0];Bd0LW9b08R8R025QHD0AdM7
01PH600F5QH05aLG01XJ6P1FEUH0GEeM08Z:RP0I6AT0:b/[01LG5`0J6QX07AdM05ADE01cLg<0>c/k
07EeM@1lO7`0?Shn04Q8B01/K6`0Q8B4020P800B4Q809RHV06UYJ@20P800_;bl05UIF@1KFe/0[:b/
09JFUP1lO7`0:2PX01HF5P0g=cL0:RXZ04Y:BP1mOGd0YZJV08Z:RP2TY:@0/;2`0<G5a@1WIfL0<C4a
02PX:00d=3@0>3Ph04a<C01;Bd/0@d=301/K6`0N7Qh0GEeM0:ZZZP2TY:@0/[:b09nOW`1SHf<0@T92
0451@@25QHD0TY:B09>CT`1MGEd0Kfm_09bLW02VYZH0[Jf]08^;R`1lO7`0N7Qh075aL@0i>CT0Lg=c
04];B`1HF5P0HV9R09RHV02g]kL0FEUI03Ti>@0T92@0:b/[0080>SXj00D0VYZJ0<O7a`3@d=00UiNG
0820P00206UYJ@0708R8R02][Jd0]KFe09BDU03>c/h0eMGE0;jn_P020:JVYP030:>SX`2l_;`0PX:2
01Cooooo000Cooooo`0N07alO01iNGT0FUYJ0820P01HF5P0Ogmo051@D01CDe<0L71`03Hf=P1PH600
JFUY03dm?@1_Kfl0K6a/03Ti>@1ZJVX0O7al0410@01QHF40EEEE04E5A@1VIVH0F5QH0492@P1HF5P0
Cdm?04];B`1BDU80@d=30P1RHV803@1MGEd0N7Qh03dm?@1?Cdl0Rh^;02h^;P1BDU80MGEe02l_;`1?
Cdl0K6a/03Ti>@1aLG400P16ATH0G01SHf<0>3Ph05MGE`1JFUX0CTi>071`L01XJ6P0?Cdm06][J`1h
N7P0BDU906i^KP21PH40=cLg06YZJP21PH40@D5107QhN025QHD0EUIF07=cL`1jNWX0CDe=09>CT`1O
Gel0GEeM07QhN01FEUH0MGEe0820P01QHF40PX:205eMG@1bLW80SHf=059BDP1fMWH0RHV904=3@`1c
Lg<0PX:205MGE`1fMWH0NWYj04I6AP1YJFT0U9BD05MGE`27QhL0JFUY05mOG`20P800>CTi07UiN@1`
L700EeMG08V9R@1UIFD0JVYZ06ATI01LG5`0Shn?079bLP1OGel0PX:206IVIP1JFUX0PH6107EeM@1F
EUH0O7al06QXJ01HF5P0Lg=c05IFEP1DE5@0M7Ad05eMG@1VIVH0PH6106a/K01jNWX0K6a/05=CD`1[
Jf/0?Shn06=SH`1WIfL0@T9206=SH`1aLG40?Cdm3_ooool00`1KFe/0EEEE04i>CP0204m?C`1b04=3
@`1VIVH0KVi^05aLG01:BTX0=SHf03Ph>010@400>c/k05]KF`15ADD0De=C04e=C@1LG5`0AdM704Q8
B013@d<0=cLg03lo?`18B4P0@d=303@d=012@T80=cLg04a<C019BDT0Bd];05QHF01QHF40ATI607]k
N`1LG5`0DE5A05EEE@1=CDd0=cLg03`l?018B4P0?Shn04=3@`0m?Cd0D51@04Y:BP16ATH0Cdm?061P
H01OGel0JVYZ08b<S01RHV80KVi^07EeM@1PH600Cdm?06EUI@1FEUH0De=C0:^[Z`2XZ:P0IfMW05]K
F`1EEED0Ng]k05IFEP1EEED0GUiN05IFEP1<C4`0Ti>C04M7A`15ADD0HV9R06IVIP1fMWH0T92@04m?
C`1DE5@0G5aL069RHP1NGUh0CDe=04a<C015ADD0A4A405aLG01mOGd0PH6103Hf=P29RHT0De=C05AD
E01FEUH0Bd];04Q8B01JFUX0E5AD04U9B@1<C4`0@d=303/k>`12@T80@D5105]KF`0e=CD0A4A40410
@01]KFd0@d=303Ti>@1;Bd/0<S8b04a<C01JFUX0Hf=S0P1ADE403019BDT0@d=305UIF@10@400AdM7
05EEE@1hN7P0JFUY071`L01/K6`0JVYZ04e=C@80De=C00D0Bd];04A4A01[Jf/0JVYZ07YjNP0>oooo
o`0d04i>CP1IFET0H61P04Q8B01>CTh0=cLg04i>CP0o?cl0<30`04=3@`0h>3P0ATI603Ph>00d=3@0
CDe=03Hf=P1EEED0>3Ph04E5A@0[:b/0?clo06IVIP0a<C40EUIF04Q8B01JFUX0>c/k04Q8B00b<S80
AdM705mOG`1iNGT0>3Ph05EEE@1;Bd/0?Shn05MGE`0_;bl0GemO059BDP1MGEd0X:2P07MgM`0_;bl0
EeMG09^KV`0l?3`0GUiN09fMW@1IFET0IfMW06IVIP80CTi>04<0GemO05=CD`1<C4`0Hf=S04];B`1U
IFD0FEUI06ATI024Q8@0FUYJ079bLP1YJFT0Lg=c05=CD`1cLg<0KFe]051@D01mOGd0PX:206m_K`2?
Shl0IfMW079bLP1:BTX0EEEE0861P@27QhL0YjNW08n?S`1jNWX0I6AT07YjNP2>SXh0JFUY04m?C`0n
?Sh0Bd];07emO@1RHV80AdM706a/K01>CTh0JVYZ05iNGP1iNGT0Kfm_06i^KP1/K6`0HV9R04m?C`0n
?Sh0JVYZ069RHP1=CDd0OWin07YjNP1BDU80GemO05IFEP1BDU80O7al06IVIP1]KFd0Jf][05UIF@1R
HV80C4a<0080Bd];0140De=C08Z:RP20P800KFe]07QhN02:RXX0@T9205ADE01NGUh0Kfm_08V9R@1a
LG40PX:202@T901@D500E5AD08N7Q`0>ooooo`0304a<C016ATH0<30`00805aLG0P0f=SH07P0J6QX0
;2`/06i^KP32`/80^k^k06e]K@1NGUh0:2PX04=3@`1BDU80B4Q808^;R`2_[jl0J6QX04Q8B00b<S80
GemO08n?S`2ATI40<c<c01/K6`0E5AD0=CDe02LW9`0g=cL0I6AT09:BTP0[:b/0>3Ph01XJ6P805aLG
02P061PH05eMG@1aLG40<S8b051@D00/;2`06ATI01XJ6P0b<S80dm?C071`L025QHD0bLW90;Rh^020
P800AdM702PX:012@T80AdM702`/;02XZ:P0QHF50:ZZZP1`L700KFe]08>3P`1CDe<0OWin08j>SP1^
KVh0<c<c04m?C`25QHD0IVIV07QhN01/K6`0RXZ:03<c<`1/K6`0FEUI0P0F5QH0>`0[:b/06ATI02`/
;026QXH0[:b/08F5Q@22PX80Kfm_07AdM02i^KT0SXj>09FEU@1HF5P0?3`l02TY:@2<S8`0O7al06IV
IP1jNWX0I6AT075aL@1/K6`0/[:b09:BTP1KFe/0KVi^03@d=00K6a/0>SXj04E5A@1kNg/0NGUi06UY
J@3?cll0YZJV04m?C`0L71`07AdM02TY:@1oOgl0[jn_0;6a/@2h^;P0FUYJ03Ti>@1:BTX0CTi>03`l
?00X:2P0C4a<055AD@26QXH0JVYZ0;:b/P36a/H0QXJ604U9B@20P800FUYJ01Cooooo000Cooooo`0m
08>3P`1fMWH0Cdm?05mOG`1QHF40?clo05EEE@1hN7P0AdM705iNGP2DU9@0A4A405YJFP1]KFd0?Cdm
07EeM@0m?Cd0ADE5069RHP1HF5P0>c/k06ATI01@D500GemO06QXJ00`<300LG5a05=CD`11@D40Hf=S
065QH@12@T80Cdm?04e=C@1SHf<0?3`l069RHP1?Cdl0@D51061PH01>CTh0AdM7055AD@1NGUh0>SXj
059BDP1/K6`0>CTi05eMG@1lO7`0<30`06=SH`1<C4`0=cLg06m_K`16ATH0AdM707IfMP0l?3`0GEeM
06QXJ00206IVIP1<05mOG`1PH600L71`071`L01:BTX0OWin05MGE`1XJ6P0N7Qh05aLG01VIVH0M7Ad
06m_K`1UIFD0S8b<071`L01VIVH0SXj>06IVIP1OGel0UiNG061PH01aLG40Q8B405]KF`1iNGT0GUiN
05]KF`1eMGD0?Cdm05aLG01iNGT0@41007QhN01]KFd0FEUI07UiN@1NGUh0N7Qh0861P@14A4@0Hf=S
071`L00f=SH0Lg=c06=SH`1IFET0J6QX05QHF023Ph<0HV9R06e]K@2MWId0FEUI069RHP25QHD0Cdm?
06EUI@2CTi<0Bd];06MWI`20P800BDU905aLG020P800B4Q805EEE@1`L700DU9B06YZJP1oOgl0>c/k
06m_K`1RHV80@d=306i^KPkooooo0380GemO06IVIP1^KVh0F5QH05eMG@1>CTh0LG5a07AdM01KFe/0
FEUI03Xj>P10@400C4a<04M7A`1WIfL0EeMG05UIF@1RHV80FEUI04U9B@1?Cdl0CTi>03/k>`0m?Cd0
CTi>04a<C00m?Cd0?3`l04=3@`1BDU80Bd];05=CD`1ZJVX0MGEe04a<C01/K6`0EUIF05aLG01ZJVX0
De=C0451@@14A4@0Bd];03dm?@11@D40?clo04Q8B00o?cl0?Cdm04U9B@80Cdm?00d0HF5Q08F5Q@1J
FUX0KFe]06m_K`1OGel0C4a<06ATI01HF5P0Cdm?08b<S02[Zj/0G5aL0080IVIV04T0MWIf055AD@12
@T80H61P04I6AP1MGEd0OWin05IFEP1;Bd/0TI6A05=CD`1hN7P0TI6A06=SH`1JFUX0HF5Q05eMG@1B
DU80B4Q805iNGP1RHV80DE5A06ATI01oOgl0Ph>304=3@`1hN7P0@41004Y:BP1TI6@0CDe=04U9B@1K
Fe/0A4A404a<C015ADD0@d=303@d=00o?cl0AdM705mOG`0e=CD0BDU904E5A@1SHf<0BDU903/k>`1<
C4`0?Shn03Xj>P1ADE40F5QH05EEE@12@T80BDU9051@D01GEeL0=cLg03dm?@1HF5P0Jf][06e]K@1X
J6P0EUIF051@D01;Bd/0De=C04Y:BP1=CDd0BTY:06YZJP1SHf<0MWIf00kooooo00P0FUYJ05IFEP1_
Kfl0AdM704];B`1>CTh0DU9B04M7A`80<c<c05X0@T9204A4A011@D40@T9203hn?P10@400EeMG02h^
;P1?Cdl0=CDe05YJFP1NGUh0?Cdm05eMG@0o?cl0KFe]04U9B@1CDe<0=cLg059BDP15ADD0EUIF059B
DP1;Bd/0BTY:05ADE01ADE40=cLg065QH@0n?Sh0Cdm?0:FUY@1OGel0JFUY05MGE`20P800?Cdm05YJ
FP2a/K40EEEE06UYJ@1EEED0De=C06a/K01FEUH0G5aL05=CD`1NGUh0GemO05YJFP1@D500JFUY06m_
K`1]KFd0IfMW08J6QP21PH40GemO07inOP1lO7`0DE5A05eMG@23Ph<0G5aL07UiN@1_Kfl0I6AT04=3
@`1`L700M7Ad05iNGP2^[Zh0TY:B07MgM`0l?3`0MgMg08^;R`1?Cdl0F5QH03/k>`1ZJVX0K6a/06e]
K@17AdL0FEUI04E5A@1KFe/0AdM706MWI`1LG5`205eMG@0V055AD@1IFET0B4Q805UIF@1WIfL0=cLg
06UYJ@1XJ6P0AdM704A4A01MGEd0EeMG07EeM@1bLW80DE5A071`L01NGUh0F5QH04Y:BP1=CDd0ATI6
079bLP29RHT0Kfm_08f=S@1eMGD0EEEE07=cL`1JFUX0HF5Q08F5Q@26QXH0I6AT08n?S`0W9bL0AdM7
04Q8B024Q8@>ooooo`0V075aL@1UIFD0OGem04Q8B00W9bL09BDU01PH600E5AD051@D04M7A`2VYZH0
TI6A05IFEP0^;Rh0NWYj05]KF`1MGEd0EEEE055AD@1lO7`0:RXZ02PX:017AdL0S8b<0:b/[01[Jf/0
?Cdm01`L700J6QX071`L01hN7P11@D40:RXZ08J6QP0k>c/061PH02XZ:P0J6QX201HF5P0G03Xj>P2F
UYH0QXJ603Lg=`0j>SX0DU9B06e]K@1`L700I6AT0<G5a@3Jf]X0b/[:09jNWP0o?cl06ATI01`L700H
61P0?Shn0;>c/`2JVYX0Zj^[07]kN`0];Bd00P0T92@01`1UIFD0FEUI075aL@2b/[80W9bL0;bl_02P
X:000P1:BTX09P0X:2P0J6QX02/[:`0h>3P0KVi^01/K6`0E5AD06ATI01LG5`1ADE40Ng]k09^KV`2f
][H0`<300:>SX`1jNWX0BTY:05YJFP1bLW80@41004M7A`0e=CD0JVYZ0<;2`P1XJ6P08B4Q01/K6`0_
;bl0LG5a07AdM01MGEd0Ph>309:BTP0L71`06QXJ01dM7@1PH600:RXZ0P1gMgL06P22PX80GEeM05AD
E00J6QX071`L03Ti>@1=CDd0Fe]K08B4Q01BDU80IfMW03`l?00Z:RX07AdM01`L700K6a/0;2`/03Ti
>@2ATI40Ogmo0:>SX`37alL0Ogmo03Xj>P1eMGD0GUiN5?ooool001?ooooo08/0MWIf0820P01IFET0
J6QX05mOG`1bLW80BDU904a<C01MGEd0>CTi05QHF01eMGD0<S8b06=SH`1XJ6P0=3@d05ADE02DU9@0
=CDe05eMG@1^KVh0?Shn069RHP1TI6@0E5AD075aL@1MGEd0>CTi061PH00i>CT0EeMG05QHF016ATH0
Jf][05EEE@11@D40Ph>304i>CP15ADD0NWYj04Y:BP19BDT0K6a/0410@01JFUX0BTY:04E5A@1ADE40
?clo075aL@1QHF40EeMG05]KF`1UIFD0=CDe06a/K01[Jf/0>3Ph06MWI`1^KVh0:b/[06EUI@1hN7P0
>CTi06m_K`1/K6`0?3`l061PH01lO7`0?Cdm07=cL`1QHF40FEUI075aL@1<C4`0LW9b07]kN`1QHF40
Ph>306ATI01OGel0MgMg07QhN01fMWH0Kfm_05mOG`1kNg/0KFe]04m?C`1WIfL0Ti>C065QH@1WIfL0
Xj>S069RHP2BTY80G5aL05]KF`25QHD0Cdm?06e]K@1WIfL0CTi>06a/K01_Kfl0GEeM071`L01CDe<0
L71`08b<S01JFUX0MgMg07AdM01BDU80LG5a06IVIP1BDU80M7Ad06m_K`14A4@0LG5a06YZJP1?Cdl0
Shn?05MGE`1FEUH0IVIV061PH01MGEd0G5aL05iNGP1WIfL0EEEE06MWI`1gMgL0@41005aLG01PH600
D51@00kooooo02l0IfMW061PH01PH600EUIF06=SH`1=CDd0MgMg08b<S01TI6@0G5aL03De=@0m?Cd0
A4A403@d=01^KVh0G5aL05YJFP1CDe<0GEeM04i>CP1:BTX0Bd];03Ti>@0m?Cd0CTi>03Ph>00k>c/0
@T9204Y:BP16ATH0EUIF055AD@1MGEd0Lg=c04m?C`1ZJVX0H61P04Y:BP1EEED0De=C03Ph>00m?Cd0
C4a<03`l?01:BTX0@D5104i>CP0204A4A00e04e=C@1;Bd/0A4A4061PH02CTi<0De=C06a/K023Ph<0
Kfm_05ADE01;Bd/0K6a/06m_K`2i^KT0b<S808^;R`1_Kfl0IfMW06i^KP1;Bd/0@d=304i>CP13@d<0
EUIF0820P01JFUX0ADE508B4Q01BDU80Lg=c08b<S01QHF40J6QX06MWI`1ZJVX0D51@05UIF@1HF5P0
JVYZ055AD@1TI6@0OWin08F5Q@1FEUH0MgMg06EUI@1KFe/0GUiN04m?C`15ADD0GEeM04i>CP1<C4`0
0P1;Bd/01@0k>c/0D51@04m?C`1SHf<0>c/k0080ATI601d0CDe=051@D00c<c<0EEEE034a<@1;Bd/0
BTY:04M7A`1EEED0Bd];04M7A`1ADE40DU9B04U9B@0n?Sh0@41006a/K01KFe/0OGem06UYJ@1XJ6P0
De=C06YZJP1PH600EEEE05IFEP1VIVH0HF5Q07IfMP0>ooooo`0b05MGE`1KFe/0N7Qh0451@@1GEeL0
>CTi04e=C@1;Bd/09BDU03De=@12@T80?3`l04I6AP0/;2`0BTY:04A4A01BDU80C4a<05ADE00m?Cd0
B4Q806m_K`19BDT0G5aL051@D01bLW80>c/k04];B`0f=SH0?clo05QHF015ADD0HV9R04];B`16ATH0
F5QH05]KF`0l?3`0J6QX04=3@`1QHF40UiNG05QHF01/K6`0AdM707emO@1CDe<0L71`0;Jf]P1EEED2
06EUI@0/051@D01RHV80JVYZ04M7A`12@T80HF5Q04=3@`1BDU80E5AD05eMG@1fMWH0H61P06IVIP1Y
JFT0Hf=S05mOG`1cLg<0Hf=S065QH@1nOWh0MgMg05QHF01jNWX0N7Qh05aLG01@D500IFEU07AdM02:
RXX0ZJVY0861P@2=SHd0@d=306a/K02/[:`0Fe]K05iNGP13@d<0N7Qh08:2PP28R8P0F5QH06i^KP1G
EeL206QXJ00Z06i^KP1aLG40UIFE06m_K`1=CDd0A4A405EEE@18B4P0JVYZ04a<C01VIVH0B4Q803`l
?01:BTX0@T9205UIF@1jNWX0FUYJ0861P@1hN7P0>3Ph06m_K`1@D500?clo05UIF@1`L700S8b<07If
MP1OGel0JVYZ03<c<`1NGUh0EEEE05=CD`1LG5`0N7Qh07]kN`2IVIT0;Bd]051@D01<C4`0OGem3_oo
ool0H`1=CDd0Fe]K08>3P`1JFUX0ATI602@T900E5AD04a<C018B4P0b<S80MGEe05]KF`1;Bd/04A4A
01<C4`0M7Ad06ATI02PX:00o?cl0G5aL05YJFP0J6QX05ADE05YJFP2BTY80S8b<06ATI00c<c<0;Bd]
02`/;00];Bd0XJ6Q08:2PP2UYJD0Kfm_028R8P0J6QX0:b/[02HV9P0J6QX0:b/[0:FUY@32`/80Ogmo
07emO@1QHF40DU9B02DU9@0J6QX0Jf][0;Vi^@2LW9`0CDe=05mOG`1GEeL0?Shn03lo?`16ATH0OGem
079bLP0k>c/0J6QX01XJ6P0C4a<051@D034a<@2VYZH0ZZZZ0:RXZ02;Rh/0MgMg07inOP1MGEd08B4Q
020P801GEeL0?clo04M7A`1NGUh0C4a<01`L700N7Qh0ATI604M7A`0U9BD0R8R80;bl_02c/k<0]KFe
071`L00R8R808B4Q03`l?00J6QX092@T05EEE@1kNg/0MGEe04a<C00202<S8`0W05YJFP1_Kfl0E5AD
06=SH`1jNWX0[:b/028R8P0K6a/06ATI02PX:01YJFT0Lg=c0410@00W9bL0?Cdm0410@01JFUX0P820
0:NWY`1ADE40O7al08f=S@27QhL0De=C01LG5`0I6AT061PH01/K6`0L71`0:b/[03/k>`2QXJ40];Bd
096AT@3AdM40L71`08N7Q`1JFUX0<30`01Cooooo000Cooooo`0D07IfMP1ZJVX0F5QH06QXJ01TI6@0
De=C0492@P1^KVh0Cdm?0451@@1^KVh0Hf=S0410@01bLW80Hf=S05MGE`1>CTh0H61P04i>CP1QHF42
05IFEP0B04i>CP1<C4`0D51@04M7A`1EEED0CDe=05iNGP1DE5@0K6a/051@D01=CDd0JFUY04U9B@13
@d<0K6a/04m?C`1HF5P0H61P0P1>CTh0BP1KFe/0F5QH04m?C`13@d<0EEEE04Y:BP1<C4`0JVYZ05EE
E@1LG5`0EeMG05iNGP1DE5@0BDU906=SH`1KFe/0D51@05YJFP13@d<0JVYZ065QH@1CDe<0LG5a06i^
KP1GEeL0D51@06m_K`1WIfL0JFUY05UIF@1eMGD0Lg=c06][J`27QhL0GUiN06UYJ@1hN7P0Hf=S07Ui
N@1/K6`0G5aL08^;R`1RHV80EEEE08f=S@1_Kfl0Fe]K069RHP1cLg<0HV9R05=CD`1^KVh0Kfm_06YZ
JP1JFUX0KVi^05ADE01GEeL0Jf][06e]K@1hN7P0De=C07=cL`26QXH0B4Q8075aL@1SHf<0DU9B07If
MP1ADE40Jf][05QHF01WIfL0NGUi0P1GEeL05@1_Kfl0JFUY04];B`1^KVh0G5aL06MWI`1RHV80EeMG
06MWI`1RHV80G5aL04Q8B01ZJVX0K6a/04a<C01XJ6P0GEeM04A4A01YJFT0Fe]K06a/K00>ooooo`1h
06MWI`1NGUh0FUYJ05mOG`1PH600FEUI06m_K`1dM7@0FEUI055AD@0h>3P0@D51059BDP1WIfL0KFe]
05IFEP1SHf<0GEeM069RHP1=CDd0Cdm?051@D00k>c/0=cLg04U9B@11@D40>SXj0451@@12@T80@d=3
05aLG016ATH0FUYJ06IVIP1=CDd0FUYJ04m?C`0o?cl0CTi>04Q8B00e=CD0?clo059BDP0d=3@0>SXj
034a<@15ADD0?Cdm03hn?P1;Bd/0B4Q804=3@`1>CTh0RHV904m?C`1JFUX0Ng]k06YZJP1GEeL0De=C
05EEE@0o?cl0S8b<0:VYZ@1hN7P0O7al05iNGP0o?cl0D51@04Y:BP1@D500Bd];05eMG@24Q8@0EUIF
04U9B@24Q8@0B4Q8071`L01jNWX0ATI6051@D01JFUX0Lg=c05YJFP1HF5P0DU9B06=SH`1>CTh0IfMW
07inOP1lO7`0ADE506=SH`16ATH0?clo051@D018B4P0>CTi05EEE@14A4@0?Cdm04=3@`18B4P0=SHf
0410@016ATH0De=C03De=@18B4P0?clo04];B`19BDT0=SHf055AD@0e=CD0Bd];051@D01<C4`0Fe]K
0P1=CDd04P1VIVH0De=C03hn?P11@D40CDe=05iNGP1hN7P0FUYJ05EEE@19BDT0ATI6051@D01GEeL0
B4Q8055AD@1XJ6P0HV9R07IfMPkooooo08`0Fe]K06=SH`1VIVH0CTi>05aLG014A4@0?3`l04I6AP0_
;bl092@T03hn?P0b<S80=SHf03De=@0l?3`0@d=30451@@13@d<0@T92034a<@1GEeL0J6QX04E5A@1E
EED0E5AD07=cL`1BDU80C4a<02TY:@0j>SX0GUiN02/[:`1?Cdl0@T92034a<@1DE5@0D51@04U9B@1G
EeL0F5QH04i>CP24Q8@0Kfm_06][J`1?Cdl0Rh^;05YJFP1UIFD0YJFU05eMG@1YJFT0J6QX05QHF01b
LW80H61P05IFEP1>CTh0G5aL05eMG@1BDU80DE5A05MGE`1VIVH0J6QX06=SH`1jNWX0I6AT06EUI@1b
LW80Fe]K06EUI@1aLG40LW9b04];B`1nOWh0K6a/07IfMP1EEED0I6AT07inOP1NGUh0^k^k08b<S01Q
HF40AdM706m_K`2HV9P0HV9R04a<C00d=3@0EeMG07AdM021PH40BDU907emO@1<C4`0I6AT065QH@1d
M7@0HF5Q07]kN`1YJFT0IFEU055AD@1PH600EUIF06MWI`15ADD0EUIF04A4A01IFET0B4Q804M7A`1I
FET0Hf=S05]KF`21PH40IfMW04=3@`1LG5`0?Cdm04M7A`12@T80JFUY06IVIP1KFe/0QXJ6065QH@0b
<S80PX:205=CD`19BDT0MgMg06][J`1<C4`0TI6A02h^;P1>CTh0?clo08j>SPkooooo01X0AdM706MW
I`1iNGT0B4Q803<c<`0f=SH07alO02`/;00f=SH0QhN709fMW@22PX80J6QX01/K6`0?3`l03@d=010@
400V9RH0JFUY071`L02PX:00Ogmo06IVIP2m_Kd0ZJVY06=SH`80JVYZ0580F5QH05eMG@1mOGd0CTi>
055AD@1bLW80TI6A07EeM@1?Cdl0@d=305QHF01;Bd/0IVIV08>3P`2TY:@0N7Qh02d];@0O7al0;Rh^
04I6AP1jNWX0SXj>0<71`@1=CDd061PH024Q8@18B4P0E5AD020P8011@D40Bd];04Q8B00X:2P0E5AD
02DU9@0C4a<04Q8B01@D501<C4`0SHf=05QHF014A4@0Fe]K038b<P1@D500<30`024Q8@11@D40C4a<
02DU9@1<C4`0EeMG08V9R@1EEED0?Shn01lO7`0f=SH0LG5a059BDP1PH600IVIV06YZJP0U9BD0<S8b
055AD@0a<C40:RXZ04Q8B01RHV80C4a<04Y:BP0a<C40@d=302l_;`0T92@0<S8b03/k>`26QXH0H61P
04a<C00X:2P07alO0P1ZJVX0200^;Rh06a/K01XJ6P0X:2P0RHV905iNGP2f][H0[jn_0P2j^[X0102Q
XJ40NGUi03lo?`0V9RH201LG5`0<024Q8@0O7al0?Cdm06=SH`1TI6@0C4a<06EUI@1^KVh0TY:B07mo
O`0d=3@0<c<c5?ooool001?ooooo0140OWin07EeM@1ADE40G5aL06QXJ011@D40Cdm?07moO`0i>CT0
IFEU07MgM`1:BTX0JFUY04a<C01LG5`0KFe]03Xj>P0206UYJ@1h03hn?P1EEED0H61P03dm?@1KFe/0
GEeM0451@@1_Kfl0C4a<055AD@1OGel0F5QH06IVIP1BDU80EUIF07AdM017AdL0EEEE06YZJP12@T80
E5AD06IVIP11@D40J6QX08J6QP0d=3@0DE5A06QXJ00g=cL0BTY:05mOG`0h>3P0IfMW04U9B@0n?Sh0
KFe]04I6AP19BDT0Jf][0410@01GEeL0?clo04Y:BP1MGEd0GemO05QHF02ATI40G5aL04];B`2=SHd0
B4Q805QHF01[Jf/0Cdm?06IVIP1XJ6P0HV9R05mOG`20P800J6QX07UiN@2DU9@0HV9R07QhN01cLg<0
HV9R08:2PP1nOWh0M7Ad06i^KP1:BTX0Rh^;08:2PP13@d<0Ogmo07IfMP15ADD0TI6A06MWI`1mOGd0
L71`05EEE@1/K6`0J6QX03dm?@1iNGT0Lg=c065QH@21PH40HV9R09VIV@1WIfL0CTi>0;bl_01OGel0
L71`0::RXP1LG5`0GEeM09JFUP1=CDd0Hf=S09RHV00m?Cd0KFe]06i^KP14A4@0JFUY06EUI@1ADE40
J6QX05IFEP1JFUX0D51@07moO`13@d<0IFEU08>3P`0j>SX0Ph>33_ooool0;P1UIFD0GUiN05EEE@1Q
HF40GUiN05UIF@1`L700Ng]k05mOG`1JFUX0>3Ph04E5A@1>CTh0F5QH06m_K`1GEeL0GEeM05MGE`1K
Fe/0Bd];04U9B@1=CDd0@d=3038b<P1;Bd/0?Shn03dm?@19BDT0B4Q80451@@1JFUX0CDe=06YZJP1]
KFd0EeMG069RHP1ADE40>c/k05YJFP1ADE40=CDe04E5A@1<C4`0?3`l0451@@0l?3`204E5A@090410
@01KFe/0Cdm?03Xj>P1DE5@0YZJV05aLG015ADD0GEeM0080A4A40540L71`0410@00m?Cd0G5aL09ZJ
VP1WIfL0N7Qh0820P018B4P0Bd];0451@@1FEUH0BDU906ATI02@T900B4Q8055AD@1kNg/0CDe=08>3
P`26QXH0DE5A04m?C`1aLG40NGUi05IFEP1CDe<0EEEE06=SH`11@D40FEUI061PH01[Jf/0B4Q806UY
J@15ADD0@T92055AD@16ATH0BTY:05IFEP1GEeL0C4a<04I6AP0j>SX0:BTY0492@P1:BTX0G5aL03Lg
=`1CDe<0BDU905IFEP1HF5P0?3`l05EEE@0f=SH0@D5104m?C`1:BTX0DE5A04i>CP18B4P0DU9B05EE
E@16ATH0C4a<05]KF`1/K6`0Ogmo06i^KP23Ph<0G5aL05iNGP1FEUH0H61P04e=C@1TI6@0JFUY05mO
G`1ZJVX03_ooool02`1IFET0F5QH06e]K@0l?3`0HF5Q0492@P10@400A4A4024Q8@0Z:RX0ATI60080
@41003T0=3@d0451@@0k>c/0>c/k05=CD`1>CTh0<C4a0410@01>CTh0?Shn05EEE@15ADD0M7Ad03/k
>`1>CTh0;Rh^03Xj>P1>CTh092@T05UIF@0e=CD0:RXZ059BDP0n?Sh0Bd];04A4A011@D40FUYJ08B4
Q00l?3`0Fe]K04i>CP2EUID0?clo05]KF`31`L40?3`l065QH@14A4@0CDe=05=CD`1?Cdl0IFEU03@d
=01OGel0C4a<03`l?018B4P0BDU905=CD`1ZJVX0EUIF075aL@1SHf<0JVYZ07MgM`1CDe<00P1WIfL0
;@1hN7P0@d=307IfMP1XJ6P0Ph>306a/K01HF5P0Ph>308F5Q@2n_[h0J6QX08F5Q@1NGUh0Lg=c09ZJ
VP2CTi<0EUIF04I6AP0f=SH0OWin07MgM`19BDT0QhN706e]K@1IFET0KVi^06QXJ01=CDd0Hf=S06YZ
JP1NGUh0A4A4051@D016ATH0AdM704];B`1TI6@0:BTY05IFEP0e=CD0C4a<05MGE`1]KFd0H61P06QX
J00206IVIP0E05YJFP1:BTX0Kfm_03<c<`1JFUX0LW9b069RHP2:RXX0DE5A03hn?P1_Kfl0G5aL05mO
G`1RHV80IFEU04i>CP21PH409BDU0492@P16ATH0UYJF00kooooo01l0<S8b04e=C@1MGEd0XJ6Q06e]
K@1`L700MgMg05iNGP1CDe<0G5aL0;Rh^02RXZ80NGUi01hN7P0Z:RX0;bl_03<c<`0Z:RX08b<S01lO
7`1<C4`0QXJ60=7Ad@2[Zj/0X:2P07inOP1kNg/0N7Qh04a<C00e=CD0GUiN0080?Shn01h07AdM04];
B`2k^k/0U9BD07=cL`15ADD0>CTi05QHF02BTY80Y:BT06][J`1;Bd/0@D5102h^;P0k>c/0X:2P08>3
P`2BTY80NGUi02PX:00K6a/08b<S02h^;P0i>CT0C4a<02h^;P15ADD0WIfM09NGU`0i>CT201<C4`0=
01lO7`22PX80P82002l_;`0X:2P0<C4a02HV9P0Z:RX0@d=304e=C@0a<C40O7al0;^k^`020861P@0P
096AT@0m?Cd0<S8b03Hf=P1nOWh0Lg=c03Ti>@0_;bl0FEUI0:FUY@1iNGT0OWin059BDP19BDT0?Shn
04a<C02ATI40XJ6Q08N7Q`2MWId0LW9b04e=C@1DE5@0HV9R08f=S@3>c/h0QHF50820P01jNWX0/;2`
06m_K`0M7Ad301PH600I028R8P2?Shl0[Zj^0;^k^`2MWId0IfMW051@D01PH600M7Ad05mOG`11@D40
7alO01/K6`0Q8B40>SXj09VIV@1mOGd0Fe]K02DU9@1NGUh0BTY:06UYJ@1SHf<0KFe]04U9B@0Doooo
o`004oooool0H@1bLW80KFe]05YJFP1_Kfl0EEEE07MgM`15ADD0CTi>05aLG00h>3P0GEeM06m_K`0c
<c<0LG5a05UIF@0o?cl0Fe]K06EUI@0k>c/0Fe]K06EUI@1<C4`0H61P05QHF01@D500L71`04m?C`1^
KVh0FEUI04];B`1bLW80@D5103lo?`25QHD0AdM704U9B@1LG5`0BDU9055AD@1PH600?Shn04e=C@1I
FET0?Cdm05mOG`0`<300Cdm?05]KF`0_;bl0A4A406UYJ@1@D500H61P06QXJ00i>CT0IVIV07=cL`0k
>c/0L71`06e]K@0[:b/0IFEU06i^KP0f=SH0JFUY09FEU@1BDU80K6a/06IVIP1fMWH0IVIV05YJFP2D
U9@0JFUY05=CD`2EUID0C4a<05MGE`2IVIT0D51@06UYJ@29RHT0Hf=S07IfMP1mOGd0Fe]K08>3P`1[
Jf/0DE5A06MWI`1[Jf/0ADE507YjNP1PH600GUiN07YjNP14A4@00P1`L700:011@D40O7al05ADE01S
Hf<0IfMW04i>CP2>SXh0Kfm_05IFEP1WIfL0De=C075aL@17AdL0Hf=S05mOG`1VIVH0JVYZ079bLP1P
H600Hf=S069RHP1LG5`0DE5A06UYJ@1RHV80CTi>07=cL`1HF5P0FUYJ08:2PP0U9BD0IVIV08V9R@0[
:b/0F5QH07=cL`0j>SX0BTY:07EeM@1<C4`>ooooo`0=07]kN`1SHf<0G5aL065QH@1WIfL0EUIF06i^
KP24Q8@0GEeM05IFEP0a<C40?Cdm04i>CP0206MWI`0?069RHP1KFe/0Fe]K05]KF`1@D500EUIF05=C
D`1;Bd/0A4A4065QH@1IFET0@D5104I6AP1;Bd/0D51@0080EEEE05l0E5AD06EUI@1FEUH0EeMG05aL
G015ADD0GemO051@D00g=cL0C4a<04I6AP0j>SX0>c/k0410@015ADD0@d=303dm?@1>CTh0De=C04Y:
BP1UIFD0TY:B04i>CP13@d<0ADE504e=C@1CDe<0GEeM05QHF01QHF40QhN70:NWY`29RHT0QhN708b<
S01>CTh0DU9B05=CD`1DE5@0CTi>061PH01gMgL0DU9B04e=C@1ZJVX0FEUI07=cL`1bLW80Cdm?061P
H01^KVh0GemO069RHP1JFUX0OGem06i^KP1?Cdl0Hf=S06a/K01PH600EEEE04a<C01KFe/0ADE5059B
DP1<C4`0A4A405YJFP1XJ6P0D51@04U9B@1GEeL0?3`l059BDP1=CDd0De=C03<c<`1@D500DE5A04Y:
BP1DE5@0>3Ph05YJFP0a<C40@D5104E5A@17AdL0DE5A04E5A@1;Bd/0CTi>04I6AP1BDU80D51@04=3
@`0205MGE`0;08F5Q@1CDe<0GUiN059BDP1FEUH0E5AD04Q8B01ZJVX0Hf=S05ADE01TI6@03_ooool0
5`1`L700I6AT04i>CP1:BTX0E5AD03Ph>00n?Sh0<S8b03Ti>@0W9bL0@41004A4A00/;2`0@41003<c
<`0m?Cd0<c<c04U9B@0_;bl0;Rh^04Q8B01TI6@0>SXj0080Bd];01d0IFEU03lo?`1OGel0;bl_04I6
AP1JFUX09BDU04a<C01?Cdl0;Bd]04];B`16ATH0DE5A04I6AP1LG5`0A4A406UYJ@1OGel0IFEU04Q8
B02>SXh0GEeM05MGE`27QhL0EUIF071`L01XJ6P0>3Ph06i^KP0205UIF@1D069RHP1KFe/0K6a/069R
HP1?Cdl0EEEE075aL@1_Kfl0JFUY065QH@1OGel0Ogmo04];B`1QHF40OGem05mOG`27QhL0DU9B061P
H01aLG40GEeM05QHF01YJFT0Lg=c05MGE`31`L40PX:206m_K`1;Bd/0P820092@T01nOWh0GEeM04I6
AP1IFET0G5aL069RHP1EEED0O7al061PH01TI6@0JVYZ071`L01MGEd0Jf][051@D01`L700HF5Q06][
J`1OGel0EEEE03dm?@1aLG40=3@d04a<C00i>CT0K6a/059BDP1=CDd0O7al06UYJ@1JFUX0LG5a05EE
E@1DE5@0JVYZ03dm?@20P800JFUY05IFEP2@T900IVIV0492@P2@T900IVIV05mOG`1hN7P0MWIf0410
@020P800?Cdm05aLG00j>SX0U9BD3_ooool03`0a<C40=SHf0::RXP2EUID0MgMg092@T037alL0U9BD
08b<S02YZJT0bl_;09>CT`2WYjL0WinO03Ph>00201TI6@1N01/K6`0J6QX06ATI04E5A@28R8P0/;2`
055AD@29RHT0b/[:08F5Q@2OWil0VIVI03Xj>P14A4@08b<S020P800J6QX0K6a/0:2PX02QXJ40UYJF
06MWI`1KFe/0GEeM07]kN`30`<00@41001DE5@0M7Ad0>3Ph08Z:RP2FUYH0W9bL055AD@2ATI40QXJ6
038b<P0L71`07QhN02@T901PH600De=C08f=S@1QHF40ATI606MWI`16ATH0=cLg059BDP28R8P0WYjN
05aLG01iNGT09BDU01/K6`0S8b<0;bl_059BDP1UIFD0/K6a092@T01cLg<0];Bd06a/K01UIFD0K6a/
0:b/[01IFET0Fe]K02h^;P0l?3`0RHV9071`L01WIfL0GEeM09FEU@1>CTh0:2PX0451@@2OWil0[Jf]
0:VYZ@2QXJ40NGUi06m_K`1DE5@0JVYZ09bLW02NWYh0[Zj^08>3P`2k^k/0e]KF06YZJP805aLG01/0
5ADE01TI6@1HF5P0De=C07inOP24Q8@0E5AD03Xj>P0a<C40<30`04Y:BP1QHF40Jf][092@T01NGUh0
GEeM08B4Q02[Zj/0X:2P03/k>`0a<C40KFe]06ATI01IFET0?clo03`l?00N7Qh05?ooool001?ooooo
05@0RXZ:07UiN@1KFe/0IFEU065QH@1WIfL0B4Q806ATI01OGel0DE5A06=SH`1WIfL0DE5A065QH@1K
Fe/0Kfm_04Y:BP1NGUh0Fe]K0492@P1IFET0E5AD03@d=01;Bd/0IVIV03`l?01?Cdl0KVi^04Y:BP14
A4@0LG5a04U9B@0o?cl0KFe]05EEE@1<C4`0I6AT04U9B@1HF5P0DU9B051@D01JFUX0Bd];06EUI@1F
EUH0=3@d06IVIP1:BTX0?Shn06=SH`11@D40D51@05UIF@1@D500DU9B05=CD`1OGel0Fe]K05EEE@1O
Gel0ATI605mOG`1gMgL0JFUY06IVIP1bLW80JVYZ059BDP1ZJVX0L71`05ADE01LG5`0MgMg04e=C@1R
HV80MgMg05YJFP1aLG40MWIf06=SH`2@T900Ng]k061PH02BTY8206a/K00e07UiN@1^KVh0P82005]K
F`1cLg<0Kfm_05=CD`1MGEd0MgMg05QHF01=CDd0Lg=c05eMG@1DE5@0EUIF05aLG01QHF40>3Ph069R
HP1lO7`0F5QH06UYJ@1?Cdl0Ogmo06IVIP1ADE40Shn?05aLG01RHV80PH6106m_K`1VIVH0PX:206MW
I`1HF5P0OWin06=SH`1TI6@0HV9R06a/K01DE5@0LG5a06][J`10@400F5QH06ATI019BDT0PH6106QX
J01CDe<0KVi^04m?C`1[Jf/03_ooool07P20P800GEeM05EEE@1IFET0J6QX05]KF`1aLG40LW9b06EU
I@1QHF40=SHf0410@014A4@0EUIF05YJFP1NGUh0FEUI05eMG@1PH600G5aL04a<C01=CDd0A4A403Hf
=P1GEeL0DU9B04M7A`1=CDd0EeMG0451@@80DU9B00T0DE5A06i^KP1JFUX0Lg=c05IFEP0l?3`0D51@
04Q8B00j>SX00P13@d<00`0a<C40>SXj0492@P0203hn?P0V04i>CP1GEeL0Bd];05EEE@1TI6@0QhN7
06ATI01BDU80De=C04Y:BP17AdL0FUYJ06YZJP1[Jf/0Lg=c08R8R01jNWX0P8200:VYZ@1MGEd0EUIF
04E5A@1:BTX0=CDe05EEE@1[Jf/0ATI60451@@1`L700E5AD07IfMP1/K6`0@T9206e]K@1IFET0GEeM
05IFEP1EEED2065QH@0d055AD@1YJFT0I6AT05QHF01:BTX0Bd];04a<C00h>3P0B4Q804a<C019BDT0
FUYJ06i^KP13@d<0A4A403Ph>00b<S80>c/k04A4A019BDT0<C4a0492@P18B4P0?Shn04M7A`0d=3@0
Bd];02l_;`10@400B4Q804A4A01@D500Bd];051@D01MGEd0FUYJ04=3@`1;Bd/0?3`l04];B`1]KFd0
TY:B08f=S@1GEeL0ADE504m?C`1TI6@0CDe=065QH@1XJ6P0HV9R06QXJ0kooooo02/0IVIV05EEE@1A
DE40?Cdm061PH00m?Cd0<30`03/k>`0b<S80=3@d03Xj>P1;Bd/0=CDe04I6AP11@D40@410038b<P1E
EED0>3Ph03<c<`12@T80ATI603Xj>P18B4P0CDe=06ATI012@T80Cdm?02h^;P1@D500C4a<02LW9`1F
EUH0<C4a02`/;01KFe/0?clo05ADE00`<300E5AD05MGE`1[Jf/0DU9B0080GUiN04T0SHf=04e=C@1_
Kfl0aLG5065QH@1^KVh0J6QX05YJFP1/K6`0E5AD07IfMP1IFET0J6QX05QHF01<C4`0DE5A04a<C01_
Kfl0I6AT05mOG`1_Kfl0GEeM096AT@1ADE40EEEE07MgM`1fMWH0WIfM06a/K01]KFd0N7Qh06MWI`28
R8P0L71`079bLP1kNg/0X:2P07IfMP25QHD0Cdm?075aL@2;Rh/0N7Qh04];B`1?Cdl0Bd];05MGE`17
AdL0Cdm?0861P@1jNWX0Jf][07inOP1cLg<0G5aL07EeM@1SHf<0NGUi05aLG01eMGD0DE5A05MGE`19
BDT0K6a/03De=@1VIVH0=3@d06][J`1JFUX0Cdm?096AT@1^KVh0Hf=S0080IfMW01@0EEEE05aLG00h
>3P0M7Ad06EUI@1ADE40Q8B404m?C`12@T80PX:205IFEP1RHV80L71`06IVIP1EEED0PH61030`<015
ADD0@T9207]kN`kooooo03T0FEUI04];B`1QHF40WIfM09FEU@2ATI40QXJ6061PH01FEUH0PH6104e=
C@1FEUH0?Shn06ATI01XJ6P0C4a<04A4A010@400>c/k04M7A`2HV9P0YjNW08F5Q@0o?cl0B4Q805MG
E`0L71`08B4Q03Ph>018B4P0FEUI069RHP1TI6@0JVYZ06IVIP1oOgl0NGUi06IVIP2YZJT0KFe]03`l
?00Z:RX0?clo061PH00^;Rh08B4Q03Hf=P0S8b<0?clo05=CD`19BDT0JFUY0<S8b02^[Zh0NGUi06a/
K01bLW800P28R8P03@1ZJVX0JFUY04];B`1ADE40RHV908b<S01mOGd0Shn?05]KF`15ADD0ATI6038b
<P0^;Rh00P1DE5@0;`1JFUX0ZJVY08J6QP1MGEd0CTi>07MgM`1jNWX0:BTY055AD@2k^k/0Jf][030`
<01>CTh0GUiN04=3@`0e=CD0IfMW05MGE`1:BTX0OGem09NGU`29RHT0VYZJ09nOW`1mOGd0JVYZ05aL
G01VIVH0PH61092@T00b<S80MgMg09FEU@2k^k/0]kNg07=cL`29RHT0GemO05]KF`1MGEd0CTi>07em
O@0e=CD0KFe]06UYJ@0l?3`0;Bd]0080:BTY01409RHV06QXJ02RXZ80[:b/09JFUP2OWil0Shn?08F5
Q@1PH600PH6108V9R@27QhL0=cLg04A4A012@T80ADE502PX:00Dooooo`004oooool0J@1bLW80Rh^;
05MGE`1OGel0Hf=S04a<C01SHf<0D51@04e=C@1UIFD0EUIF05EEE@1aLG40ATI606YZJP1`L700;Rh^
071`L01IFET0?clo07YjNP1BDU80ADE508J6QP19BDT0G5aL069RHP1BDU80GemO03Ti>@1NGUh0IfMW
04];B`1:BTX0F5QH03Xj>P1OGel0Kfm_034a<@1TI6@0N7Qh03Ph>01OGel0J6QX04Q8B01CDe<0EUIF
05=CD`1IFET0D51@05EEE@1VIVH0<S8b05]KF`1_Kfl0BDU905QHF01BDU80@D51075aL@13@d<0E5AD
07MgM`17AdL0H61P08^;R`1EEED0HV9R06QXJ01HF5P0IfMW07UiN@18B4P0IVIV05iNGP1CDe<0MGEe
05UIF@1YJFT0KFe]075aL@1UIFD0RHV9079bLP1^KVh0Q8B406QXJ01bLW80QhN704A4A01^KVh0Kfm_
04E5A@29RHT0Jf][079bLP1UIFD0E5AD07moO`1;Bd/0EEEE07UiN@1XJ6P0G5aL08:2PP0207]kN`0P
05QHF01WIfL0R8R805MGE`1QHF40VIVI04e=C@1kNg/0MGEe05QHF020P800RXZ:059BDP1`L700P820
04Q8B021PH40GemO05]KF`1fMWH0FUYJ06MWI`1FEUH0DE5A071`L01>CTh0GUiN05aLG01KFe/0DU9B
05aLG01WIfL>ooooo`0<07moO`1LG5`0Cdm?05ADE01PH600FEUI07AdM01fMWH0GUiN06=SH`12@T80
BTY:0P19BDT08@1`L700Fe]K06MWI`1HF5P0IFEU05]KF`17AdL0EEEE04M7A`0l?3`0C4a<059BDP10
@400>CTi06EUI@1;Bd/0D51@04e=C@1CDe<0K6a/05mOG`1^KVh0H61P03@d=01BDU80B4Q803Ph>00i
>CT0@D5103Xj>P18B4P0DE5A03`l?00203Ti>@0_04I6AP0f=SH0AdM705aLG01aLG40De=C059BDP1C
De<0CDe=055AD@1JFUX0H61P06IVIP1KFe/0JVYZ07=cL`1[Jf/0XZ:R055AD@1IFET0CDe=04Y:BP16
ATH0GEeM06i^KP16ATH0Cdm?05MGE`1BDU80K6a/06=SH`0l?3`0GemO05QHF01>CTh0F5QH051@D01L
G5`0J6QX05YJFP1`L700IVIV05QHF01CDe<0EEEE04E5A@0c<c<00P1:BTX02`0c<c<0@d=3075aL@0g
=cL0;Rh^034a<@0k>c/0Bd];04a<C01DE5@0@4100080Cdm?01X0Bd];04=3@`10@400A4A403Hf=P1@
D500BTY:0410@01RHV80?Cdm04Q8B01SHf<0De=C04Y:BP1?Cdl0Fe]K075aL@1WIfL0L71`08f=S@19
BDT0E5AD04a<C01YJFT0DU9B06QXJ080HV9R00<0H61Pooooooooool03?ooool05P24Q8@0GemO04m?
C`1YJFT0NGUi055AD@13@d<0=cLg03Xj>P0a<C40@D5103dm?@0e=CD0Cdm?03/k>`11@D40=CDe07Ee
M@0d=3@0C4a<05MGE`1BDU8204Y:BP0W06UYJ@1lO7`0EeMG06UYJ@0k>c/0F5QH06=SH`0j>SX0FEUI
03lo?`0`<300DU9B04Y:BP1>CTh0=CDe05mOG`1ADE40FEUI06EUI@1lO7`0Jf][09ZJVP1HF5P0G5aL
0;6a/@1XJ6P0Kfm_05QHF01GEeL0MGEe05iNGP1VIVH0Hf=S06MWI`1BDU80E5AD059BDP1?Cdl0MWIf
00<0H61P00@0CTi>09fMW@18B4P0D51@0P1WIfL0A02SXj<0IVIV04i>CP1dM7@0G5aL06m_K`23Ph<0
MWIf06=SH`1nOWh0K6a/071`L01BDU80OWin0::RXP1SHf<0C4a<03lo?`1SHf<0CDe=05aLG01>CTh0
Jf][07IfMP1cLg<0PX:206i^KP1PH600GUiN05eMG@1hN7P0GemO075aL@1BDU80EeMG03lo?`1NGUh0
=3@d05QHF00e=CD0IVIV06QXJ00e=CD0KVi^071`L01XJ6P0Jf][061PH01MGEd0KVi^03@d=01SHf<0
De=C06=SH`1oOgl0CTi>05UIF@1bLW80IfMW06a/K020P800IFEU06IVIP1gMgL0<30`04e=C@11@D40
Ogmo3_ooool0900k>c/092@T01/K6`0j>SX0UiNG07emO@0a<C4051@D014A4@0S8b<0KVi^01LG5`0U
9BD0WYjN0:RXZ01jNWX0@d=303hn?P0j>SX0O7al0:j^[P2c/k<0Ng]k05iNGP0`<300=cLg01LG5`0C
4a<06ATI03De=@1jNWX0Ogmo075aL@0f=SH07QhN01TI6@80>c/k03h0JFUY04A4A019BDT07alO028R
8P17AdL0K6a/04E5A@0O7al08B4Q02XZ:P1GEeL0CTi>01lO7`28R8P0YJFU08Z:RP1DE5@0De=C08b<
S01FEUH0:BTY02PX:01@D500?clo06EUI@1GEeL0?clo03De=@0];Bd05QHF02/[:`0/;2`0Bd];05MG
E`1BDU80RHV90;Fe]@1CDe<0D51@034a<@0/;2`0IfMW06][J`1VIVH0Ti>C09NGU`1;Bd/0Q8B404E5
A@0O7al0820P02`/;01LG5`0OGem075aL@1OGel0T92@0<;2`P1oOgl0PX:204U9B@<071`L0180BTY:
06][J`2PX:00>CTi06a/K02CTi<0=3@d08:2PP0T92@0;Bd]07=cL`0X:2P0GemO05EEE@1]KFd0SHf=
02DU9@0Q8B4201hN7P0A03Ti>@2=SHd0SXj>07moO`25QHD0V9RH0=SHf02k^k/0?clo069RHP0a<C40
EUIF038b<P1<C4`0VIVI09fMW@1kNg/05?ooool001?ooooo00X0MgMg07IfMP1eMGD0M7Ad05iNGP26
QXH0@D5106UYJ@1XJ6P0@4100P1MGEd0O`1BDU80K6a/05QHF01BDU80CTi>07AdM00l?3`0F5QH069R
HP1KFe/0ADE504A4A01PH600=SHf04i>CP1bLW80Cdm?03Xj>P1fMWH0@d=305aLG01gMgL0BTY:05UI
F@1gMgL0BTY:05eMG@1@D500BTY:05mOG`0l?3`0EeMG06UYJ@0X:2P0De=C06EUI@0c<c<0OWin06MW
I`10@400Lg=c065QH@1>CTh0I6AT079bLP1<C4`0FEUI06i^KP0j>SX0I6AT06YZJP1?Cdl0KVi^06=S
H`1WIfL0LW9b06UYJ@26QXH0IVIV06YZJP2FUYH0ATI606UYJ@29RHT0Bd];06][J`2EUID0B4Q80820
P01iNGT0CTi>0820P01lO7`0CTi>08>3P`1JFUX0EUIF07MgM`1KFe/0GUiN05IFEP1LG5`0Ph>3079b
LP1@D500JFUY07IfMP1KFe/0KVi^05YJFP1ZJVX0De=C05MGE`2?Shl0Bd];05UIF@1MGEd0EEEE075a
L@1@D500IfMW07IfMP1mOGd0K6a/07=cL`1LG5`0Kfm_07emO@1BDU80HV9R07alO01ADE40G5aL0820
P00m?Cd0EEEE08>3P`0d=3@0De=C07QhN00f=SH0HF5Q05iNGP10@400KFe]059BDP1ADE403_ooool0
;P2>SXh0JVYZ059BDP1JFUX0H61P05YJFP1RHV80KFe]06=SH`1QHF40>c/k04Q8B01;Bd/0KVi^06YZ
JP1XJ6P0HV9R06IVIP1LG5`0Fe]K05eMG@1=CDd0ATI603Xj>P1EEED0BDU904m?C`1NGUh0Hf=S055A
D@1EEED0B4Q805=CD`1RHV80JFUY08^;R`1HF5P0B4Q805IFEP17AdL0@T9204A4A016ATH0=cLg04M7
A`1UIFD20410@00<04A4A01?Cdl0@D5104Y:BP1SHf<0JFUY05mOG`1iNGT0CTi>05=CD`1ADE40F5QH
0P1WIfL0=`1YJFT0JVYZ07MgM`1UIFD0VYZJ06EUI@1VIVH0D51@04I6AP0o?cl0G5aL079bLP1CDe<0
FEUI07EeM@1[Jf/0OGem06MWI`19BDT0Fe]K06a/K01IFET0GEeM05]KF`1WIfL0@T9206YZJP1NGUh0
DE5A04E5A@1;Bd/0@d=304i>CP12@T80BTY:04Q8B00i>CT0>SXj04U9B@0_;bl0;Bd]04A4A00j>SX0
@T920451@@1>CTh0>3Ph0410@01:BTX0CDe=05EEE@17AdL0;Bd]04=3@`15ADD00P17AdL05@1>CTh0
>c/k04i>CP1GEeL0C4a<055AD@1CDe<0BDU9061PH01@D500Ng]k07alO01KFe/0De=C065QH@1`L700
CTi>05QHF01CDe<0FUYJ05UIF@0>ooooo`0706a/K01DE5@0B4Q804];B`1ZJVX0F5QH03`l?00203@d
=00=02h^;P1;Bd/0@T9203<c<`1?Cdl0;Rh^03Lg=`0Z:RX0De=C03Hf=P0l?3`0A4A403Lg=`020410
@00P05ADE01gMgL0CTi>05aLG00e=CD0Hf=S04a<C00_;bl0GUiN03Hf=P0_;bl0H61P04A4A01=CDd0
>3Ph05QHF01FEUH0IFEU08:2PP1UIFD0F5QH0:>SX`1BDU80Lg=c0<_;b`1ZJVX0MgMg05]KF`1XJ6P0
LG5a05aLG01kNg/206e]K@0A05mOG`18B4P0IfMW05YJFP1jNWX0L71`069RHP1SHf<0DE5A09VIV@19
BDT0@41005mOG`1cLg<0OGem06a/K01XJ6P00P1fMWH0?`26QXH0NWYj07YjNP1`L700MgMg06m_K`2;
Rh/0G5aL07IfMP2BTY80IVIV05MGE`17AdL0Bd];05YJFP1HF5P0@d=306ATI01IFET0G5aL075aL@1P
H600FEUI07=cL`1UIFD0Ng]k06QXJ026QXH0A4A405=CD`1>CTh0Hf=S04a<C01XJ6P0<30`069RHP1Q
HF40CDe=08j>SP1lO7`0F5QH07MgM`1[Jf/0E5AD05QHF00];Bd0JFUY05mOG`1ZJVX0Ng]k04M7A`1F
EUH0M7Ad05QHF01YJFT0Kfm_04i>CP1KFe/0NWYj030`<017AdL0BTY:09:BTP0>ooooo`0K03De=@0]
;Bd0>CTi02PX:01cLg<0T92@01XJ6P0@41004A4A010@400k>c/0GEeM01LG5`1oOgl0PH6103Ph>00B
4Q805ADE01HF5P0G5aL0=CDe06=SH`2NWYh0YJFU0861P@1XJ6P05QHF008051@D00D0=cLg08f=S@2=
SHd0C4a<024Q8@0201DE5@1101lO7`0h>3P0I6AT03dm?@1PH600De=C04A4A01oOgl0/;2`07IfMP14
A4@0Cdm?06][J`1mOGd0Bd];0451@@0e=CD0@41007alO018B4P0G5aL06UYJ@1gMgL06ATI034a<@2?
Shl0N7Qh07emO@0^;Rh09RHV01dM7@17AdL071`L01PH600o?cl0M7Ad0:6QX@2CTi<0RXZ:092@T00R
8R8071`L01dM7@0X:2P0CDe=020P800X:2P0SXj>0;^k^`1iNGT0PX:208N7Q`1CDe<0=SHf01`L700o
?cl0Hf=S02d];@15ADD0?clo0::RXP1<C4`0Bd];065QH@0L71`00P0O7al01`0g=cL0S8b<096AT@1G
EeL0MWIf04A4A01WIfL00P18B4P0700`<300ATI60;Fe]@2^[Zh0NGUi07MgM`2;Rh/0LW9b04m?C`0]
;Bd0BDU9092@T02>SXh0B4Q803dm?@1GEeL0WIfM08F5Q@1VIVH0UIFE08^;R`2>SXh0OWin08Z:RP1e
MGD0TY:B03@d=01>CThDooooo`004oooool0BP22PX80MWIf05iNGP1MGEd0H61P06EUI@1HF5P0Cdm?
0492@P1PH600Cdm?04a<C01iNGT0GemO05]KF`1oOgl0:b/[06][J`1bLW80<C4a079bLP1EEED0=CDe
061PH01IFET0ATI60410@01DE5@0D51@04];B`1ADE40G5aL04I6AP1<C4`0G5aL03Hf=P1CDe<0L71`
03hn?P18B4P0P82004I6AP1CDe<0FEUI04=3@`1DE5@0H61P03@d=01WIfL0E5AD03`l?01fMWH0@T92
0451@@1aLG40<c<c05ADE01WIfL0;2`/05mOG`1FEUH0H61P08b<S01XJ6P0EUIF075aL@1SHf<0HF5Q
079bLP1WIfL0DE5A08b<S01SHf<0DE5A0P1[Jf/0?`1]KFd0IFEU08:2PP1^KVh0K6a/05mOG`1hN7P0
JFUY05eMG@1mOGd0GUiN06IVIP1iNGT0Bd];07MgM`1`L700ADE508B4Q01gMgL0Cdm?07inOP1GEeL0
MGEe06e]K@1>CTh0I6AT06m_K`1DE5@0IFEU06MWI`1fMWH0CDe=04];B`1mOGd0De=C05aLG01nOWh0
EeMG06QXJ01`L700EUIF06YZJP1`L700EUIF075aL@1gMgL0BTY:07inOP1LG5`0FUYJ06a/K01BDU80
L71`059BDP1?Cdl0HV9R05UIF@1bLW80GEeM05UIF@1SHf<0EeMG06i^KP0>ooooo`0E07]kN`1QHF40
CDe=04e=C@1OGel0DU9B06YZJP1iNGT0KVi^06UYJ@0i>CT0B4Q804A4A01>CTh0F5QH04a<C01JFUX0
GEeM05EEE@1ADE40FEUI0080ATI60100@D5104Q8B015ADD0@41004i>CP1[Jf/0E5AD05iNGP1JFUX0
HV9R06YZJP1LG5`0K6a/05mOG`15ADD0HV9R0P1BDU80=P0i>CT0?Cdm030`<010@400JVYZ04M7A`13
@d<0BDU905IFEP19BDT0?Shn05QHF01mOGd0JFUY06MWI`1@D500JFUY05ADE01bLW80Kfm_06e]K@2<
S8`0P82006IVIP1@D500Jf][05YJFP1LG5`0C4a<05QHF01<C4`0KVi^06e]K@1EEED0Cdm?06a/K01P
H600OWin06][J`18B4P0EEEE06YZJP1OGel0H61P05EEE@1JFUX0JVYZ065QH@1UIFD0Bd];055AD@1C
De<0@41005QHF080>c/k02/0Bd];03De=@0i>CT0GEeM03Xj>P0b<S80=SHf01lO7`17AdL0ADE5059B
DP11@D40B4Q805EEE@16ATH0AdM704E5A@0R8R80@d=304];B`0m?Cd0Bd];05YJFP0o?cl0E5AD05mO
G`1JFUX0G5aL04m?C`1HF5P0K6a/04E5A@1OGel0IfMW04U9B@12@T80De=C05QHF014A4@0FUYJ05=C
D`1HF5P0Kfm_00kooooo02P0N7Qh069RHP0l?3`0CDe=07YjNP10@400ATI603Hf=P0k>c/0;Bd]0451
@@0l?3`0:b/[04Y:BP0f=SH0=CDe034a<@1UIFD0:2PX04a<C01BDU80C4a<05eMG@0o?cl0GemO06YZ
JP1>CTh0FUYJ03Lg=`1BDU80I6AT04=3@`1ADE40B4Q8034a<@1FEUH0EEEE04e=C@0l?3`0Kfm_0P1F
EUH206i^KP0d06IVIP2PX:00BDU906EUI@2d];@0IFEU06i^KP1PH600IFEU07EeM@1OGel0I6AT06=S
H`1VIVH0DU9B04e=C@1IFET0C4a<07]kN`1@D500HV9R05YJFP1=CDd0WIfM04Y:BP12@T80GemO06a/
K01UIFD0LW9b04Q8B01RHV80J6QX065QH@1kNg/0MgMg069RHP1jNWX0LG5a08V9R@1JFUX0NGUi09bL
W01SHf<0Bd];0492@P1YJFT0C4a<05eMG@1>CTh0De=C05iNGP80JVYZ02X0IFEU05iNGP1lO7`0FUYJ
07emO@1]KFd0K6a/051@D01>CTh0@d=306IVIP16ATH0Cdm?03Lg=`1TI6@0CTi>03/k>`2LW9`0KVi^
07]kN`1`L700JFUY069RHP1^KVh0=3@d06m_K`1mOGd0Fe]K08^;R`1@D500E5AD09RHV01PH600IFEU
08>3P`1ADE40OGem07inOP0d=3@0ATI603dm?@1lO7`>ooooo`0R05MGE`1=CDd0:BTY03/k>`0i>CT0
Kfm_04i>CP0G5aL04Q8B00l?3`0B4Q80De=C05ADE02IVIT0DE5A05IFEP0T92@0:2PX02d];@0e=CD0
ADE506][J`28R8P0emOG0<G5a@2DU9@0FEUI03/k>`0g=cL0LG5a0::RXP2d];@0EUIF01DE5@804a<C
02H05ADE01lO7`2:RXX0N7Qh09^KV`1CDe<0?clo065QH@2KVi/0W9bL06=SH`0a<C40=3@d04m?C`2U
YJD0NWYj05mOG`1bLW80De=C059BDP0c<c<0;Rh^06MWI`1VIVH0@D5107]kN`2g]kL0Cdm?04Q8B00o
?cl0=CDe03<c<`1/K6`0F5QH07=cL`1dM7@0`<3005mOG`80=3@d0180DE5A01`L700K6a/0:b/[061P
H00F5QH061PH05UIF@1DE5@0@d=306i^KP1GEeL0:b/[02@T900P8200EUIF05mOG`1;Bd/205EEE@0@
0;fm_@13@d<0<C4a06QXJ01SHf<0CDe=05MGE`1RHV80TY:B04I6AP1?Cdl0S8b<08J6QP0Y:BT0BDU9
0410@080C4a<01P0aLG50;>c/`1FEUH0Lg=c06YZJP0`<300BDU909fMW@2XZ:P0Rh^;07AdM00U9BD0
8b<S04a<C02d];@0MgMg06MWI`1?Cdl0SXj>08n?S`0c<c<0KFe]09bLW02UYJD2061PH1Cooooo000C
ooooo`2;071`L021PH40GEeM06MWI`1ADE40IFEU05EEE@1?Cdl0FEUI06ATI01ADE40GEeM06YZJP1H
F5P0I6AT06a/K016ATH0MWIf05EEE@1;Bd/0M7Ad04M7A`1>CTh0De=C059BDP1EEED0A4A405=CD`1J
FUX0>SXj07AdM01JFUX0C4a<06YZJP1DE5@0EUIF05mOG`1SHf<0ATI6051@D01ZJVX0CTi>0492@P1R
HV80EUIF04E5A@1DE5@0CTi>0492@P1=CDd0JVYZ04];B`1ADE40MWIf04i>CP1JFUX0LG5a04Y:BP12
@T80KFe]04U9B@1VIVH0MgMg05MGE`1VIVH0IFEU04Y:BP1VIVH0Hf=S06e]K@1MGEd0Jf][07UiN@1I
FET0EeMG07YjNP1VIVH0Hf=S08Z:RP1PH600IfMW07alO01XJ6P0Lg=c06e]K@1hN7P0Hf=S06YZJP1Q
HF40IVIV06EUI@1KFe/0GemO06UYJ@1TI6@0Ph>305mOG`1ADE40QHF504U9B@1UIFD0KFe]06m_K`1W
IfL0Lg=c079bLP1]KFd0EUIF06ATI01hN7P0Hf=S059BDP1QHF40GEeM06i^KP1XJ6P0LW9b06QXJ01U
IFD0N7Qh05mOG`1WIfL0PH6106EUI@1VIVH0Ph>305]KF`1MGEd0KVi^03Xj>P1>CTh0JFUY04M7A`1R
HV80GEeM04=3@`1RHV80EEEE04E5A@0>ooooo`1D07moO`1JFUX0EUIF05QHF01SHf<0EEEE079bLP1W
IfL0GemO05QHF00m?Cd0>SXj04Q8B01;Bd/0HV9R051@D01RHV80EeMG05aLG01=CDd0AdM70451@@0n
?Sh0>c/k04E5A@0o?cl0@41003Xj>P1EEED0DU9B05YJFP1=CDd0HF5Q06=SH`1VIVH0Jf][05IFEP12
@T80G5aL05IFEP1@D500@41004Q8B011@D40C4a<06e]K@12@T80Bd];04I6AP1IFET0C4a<04M7A`1E
EED0IfMW06EUI@21PH40IVIV06YZJP1;Bd/0Cdm?05eMG@1gMgL0Ti>C07alO01nOWh0IfMW07MgM`1X
J6P0E5AD04U9B@1FEUH0Bd];06m_K`1hN7P0G5aL05=CD`1IFET0CTi>075aL@1KFe/0FUYJ05QHF01T
I6@0De=C0P1PH6004P1NGUh0GemO061PH01bLW80LG5a04];B`1EEED0B4Q805EEE@0l?3`0AdM70492
@P19BDT0A4A406i^KP14A4@0<c<c03Ti>@80B4Q80280AdM705QHF010@400@T9204I6AP10@400@d=3
0410@00T92@0?3`l051@D01>CTh0ATI606MWI`16ATH0Bd];06QXJ01VIVH0EeMG04I6AP1SHf<0IfMW
04Y:BP1GEeL0I6AT05eMG@1HF5P0EUIF04m?C`18B4P0F5QH059BDP1MGEd0I6AT3_ooool0<`1^KVh0
Hf=S03Xj>P1VIVH0Ogmo04M7A`19BDT0<C4a03lo?`0b<S80ATI6038b<P0i>CT0E5AD03@d=00k>c/0
?Cdm04Q8B00O7al0>c/k0492@P0o?cl0@d=303hn?P1EEED0IFEU05MGE`1JFUX0;2`/065QH@1FEUH0
<c<c05UIF@0f=SH09BDU05MGE`1>CTh0D51@03De=@1JFUX0GemO04I6AP2:RXX0I6AT061PH02RXZ80
Fe]K061PH030`<00BTY:05mOG`0205]KF`0L07AdM01OGel0EeMG079bLP1ZJVX0IFEU059BDP1SHf<0
F5QH08F5Q@1UIFD0L71`065QH@1CDe<0SXj>04];B`10@400FUYJ071`L015ADD0M7Ad04e=C@1SHf<0
Kfm_071`L01bLW80LG5a07EeM@80Ph>30140UIFE05]KF`1oOgl0YJFU05MGE`1TI6@0ADE505]KF`1C
De<0F5QH04];B`1?Cdl0L71`07EeM@1]KFd0I6AT051@D00205YJFP0A08R8R01eMGD0O7al0492@P1?
Cdl0?Shn079bLP1CDe<0C4a<03@d=01GEeL0CDe=03lo?`2RXZ80D51@06QXJ01UIFD00P1LG5`04`1b
LW80=CDe04];B`1]KFd0Fe]K07EeM@1;Bd/0E5AD07EeM@1EEED0De=C07=cL`12@T80RXZ:09:BTP0k
>c/0BDU903/k>`1iNGT03_ooool0<`26QXH0Cdm?04i>CP0a<C40@T9204A4A00g=cL0ADE501dM7@0B
4Q80410@01TI6@2QXJ40RHV903Lg=`1>CTh09bLW01/K6`0H61P0820P01hN7P0m?Cd0Lg=c08F5Q@1b
LW80IVIV06m_K`0o?cl0BDU907QhN02MWId0XZ:R061PH00V9RH08B4Q02<S8`0P8200CTi>08f=S@1=
CDd0[Zj^08:2PP1;Bd/0@41005MGE`1WIfL0H61P08j>SP22PX80WYjN0;bl_00208>3P`1G04M7A`1@
D500>CTi02PX:00L71`0DE5A07=cL`1SHf<0NGUi08F5Q@1UIFD0?clo04I6AP0f=SH0=cLg07MgM`1Q
HF40NGUi07AdM019BDT0?3`l038b<P0I6AT0=3@d04a<C00W9bL0E5AD0;:b/P10@400=3@d02/[:`0m
?Cd0F5QH0:2PX01FEUH0D51@04a<C01XJ6P0WinO06MWI`0^;Rh0Lg=c06e]K@1bLW80];Bd06MWI`0j
>SX0S8b<02HV9P0h>3P0O7al08J6QP15ADD0C4a<04=3@`0_;bl05QHF01`L701=CDd0Bd];08n?S`35
aLD0][Jf09JFUP0m?Cd08b<S03`l?01YJFT0QHF509bLW00m?Cd0;Bd]02h^;P0/;2`0Jf][04M7A`0b
<S80EEEE07UiN@1=CDd0A4A407UiN@1/K6`0JVYZ07]kN`1UIFD0DU9B01Cooooo000Cooooo`0_08:2
PP1UIFD0MWIf06=SH`1DE5@0Ng]k03Xj>P1>CTh0C4a<03`l?01XJ6P0DU9B05ADE01eMGD0>SXj055A
D@1UIFD0G5aL04];B`1LG5`0H61P069RHP17AdL0@D51065QH@1NGUh0>3Ph06QXJ015ADD0=CDe08B4
Q01:BTX0Fe]K06EUI@1>CTh0J6QX06EUI@1BDU80IVIV03/k>`1BDU80Ng]k02LW9`1RHV80Cdm?02`/
;01/K6`00P10@4009`1ADE40HV9R051@D01NGUh0CTi>05MGE`1IFET0Hf=S06IVIP1BDU80J6QX04U9
B@1ZJVX0OGem06][J`1cLg<0J6QX075aL@1;Bd/0IFEU07MgM`13@d<0IVIV08F5Q@12@T80IfMW07Qh
N017AdL0Hf=S08V9R@1:BTX0Fe]K07=cL`1=CDd0LG5a06ATI01JFUX0HV9R06=SH`0205mOG`0a075a
L@1FEUH0FUYJ05mOG`1YJFT0FEUI05]KF`1UIFD0GEeM06QXJ01MGEd0De=C08j>SP18B4P0Kfm_07Ui
N@1LG5`0JFUY04M7A`1YJFT0Jf][03`l?01TI6@0RXZ:05IFEP1iNGT0SHf=059BDP1mOGd0Ph>304I6
AP22PX80M7Ad03dm?@1_Kfl0Ph>30492@P1]KFd0HF5Q051@D01VIVH0EUIF04M7A`2<S8`0De=C04M7
A`2ATI40Cdm?05MGE`0>ooooo`0R06i^KP1?Cdl0HF5Q05=CD`1HF5P0Cdm?06i^KP1PH600HF5Q04i>
CP14A4@0<c<c04M7A`0k>c/0E5AD04U9B@1RHV80E5AD05QHF017AdL0>c/k0410@00e=CD0;bl_03Hf
=P0k>c/0@D5104M7A`1BDU80CDe=05ADE01CDe<0IVIV04a<C080F5QH04P0C4a<03dm?@1FEUH0DE5A
04m?C`11@D40B4Q803dm?@1=CDd0KVi^04Q8B01:BTX0B4Q805QHF01<C4`0A4A406=SH`1bLW80IFEU
08>3P`1nOWh0GEeM05=CD`1ADE40FUYJ06UYJ@2CTi<0QHF50820P01VIVH0QHF504I6AP14A4@0BDU9
04E5A@1GEeL0E5AD05iNGP1CDe<0EEEE065QH@1^KVh0O7al04a<C01=CDd0EEEE05]KF`1:BTX0Jf][
069RHP1LG5`0B4Q806e]K@1_Kfl0N7Qh03/k>`1EEED0BTY:04e=C@1OGel0AdM705YJFP1?Cdl0BDU9
05mOG`0l?3`0;bl_03`l?013@d<0>c/k04M7A`1HF5P204I6AP0N04i>CP18B4P0AdM704a<C00n?Sh0
B4Q805ADE01GEeL0C4a<04m?C`10@400De=C05UIF@1NGUh0FUYJ05MGE`1dM7@0MGEe055AD@1TI6@0
DU9B04=3@`14A4@0CDe=059BDP1<C4`0De=C055AD@1JFUX0JVYZ3_ooool0S01]KFd0HV9R04i>CP1V
IVH0M7Ad04a<C015ADD0@D5103hn?P0Y:BT0@d=302/[:`0k>c/0CTi>03<c<`1<C4`0=CDe04Q8B00]
;Bd0?3`l04A4A011@D40EeMG04U9B@1JFUX0GUiN04Q8B01QHF40<30`055AD@18B4P0ATI605eMG@0j
>SX0>c/k05iNGP1DE5@0D51@03<c<`1[Jf/0GEeM03Hf=P1eMGD0JVYZ07YjNP2UYJD0EUIF06UYJ@31
`L40EeMG06=SH`1DE5@0FEUI05YJFP1QHF40D51@06=SH`1XJ6P0DE5A04Y:BP1JFUX0D51@075aL@1[
Jf/0LW9b05QHF017AdL0NWYj04U9B@0n?Sh0IVIV07QhN00i>CT0Hf=S04E5A@1RHV80N7Qh07YjNP1Z
JVX0LG5a07AdM02PX:00IFEU09VIV@1HF5P0NGUi0;Bd]01UIFD0NGUi04M7A`1ADE40FEUI04a<C01=
CDd0B4Q807AdM01ZJVX0NGUi05aLG01CDe<0GUiN04];B`1bLW80Kfm_07alO013@d<0De=C04];B`1]
KFd0GUiN05eMG@0g=cL0GEeM04=3@`1LG5`0P82004Y:BP1^KVh0N7Qh06MWI`1KFe/0PX:203dm?@0k
>c/0I6AT07MgM`1hN7P0De=C06ATI01YJFT0FUYJ06ATI01ZJVX0GUiN07QhN02][Jd0ADE50451@@0n
?Sh0O7al3_ooool04`2<S8`0L71`06a/K00i>CT071`L01hN7P0^;Rh0<C4a04I6AP0P8200=SHf05iN
GP1RHV80X:2P08R8R00`<3004a<C010@400E5AD00P0F5QH03P0H61P0Fe]K03Hf=P0d=3@0FEUI069R
HP0j>SX0B4Q8071`L02QXJ40OWin079bLP0E5AD051@D0`0E5AD0IP0F5QH0D51@08n?S`0Y:BT05ADE
01@D500W9bL0>CTi03@d=01fMWH0AdM706][J`1jNWX0FUYJ0410@00o?cl0=3@d02XZ:P0Q8B40:RXZ
08B4Q02=SHd0Vi^K059BDP0g=cL0@T92051@D01;Bd/0TY:B092@T028R8P0Fe]K01lO7`0_;bl04a<C
01DE5@0g=cL0DE5A01dM7@1EEED0YZJV0:n_[`1BDU80LG5a08Z:RP2>SXh0WinO09ZJVP2g]kL0H61P
03/k>`0/;2`0;bl_05ADE01kNg/0OWin05aLG00H61P0ATI606=SH`2WYjL0Z:RX0:NWY`19BDT0Cdm?
09BDU02l_;`0QHF5059BDP20P800<c<c01/K6`0E5AD051@D05mOG`2SXj<0^;Rh0:^[Z`1[Jf/0@D51
03<c<`1FEUH0D51@05eMG@1PH6009RHV01TI6@0i>CT0NGUi06][J`1OGel0HV9R04i>CP0o?cl0Ti>C
04Q8B020P800FEUI02@T9017AdL05aLG04Y:BQCooooo000Cooooo`80O7al0480CDe=05UIF@1UIFD0
D51@05UIF@1QHF40Bd];06EUI@0k>c/0ADE507UiN@0i>CT0EeMG06IVIP0a<C40JFUY05ADE00_;bl0
Kfm_05EEE@0o?cl0FUYJ04i>CP1LG5`0AdM705IFEP1PH600?3`l055AD@1XJ6P0?3`l04];B`1mOGd0
De=C055AD@1mOGd0?3`l04i>CP2;Rh/0?Shn05QHF01cLg<0C4a<05=CD`13@d<0?clo06][J`0W9bL0
CDe=06m_K`0U9BD0GEeM06ATI00k>c/0IFEU059BDP0l?3`0Ng]k04i>CP1=CDd0Q8B406=SH`1ZJVX0
W9bL079bLP1TI6@2065QH@0N06YZJP1UIFD0KVi^05QHF01TI6@0IVIV069RHP1SHf<0LG5a07UiN@1j
NWX0HV9R06YZJP1VIVH0DE5A07AdM00f=SH0Hf=S06EUI@0a<C40PX:204Y:BP1;Bd/0IfMW05MGE`1`
L700Kfm_04a<C02>SXh0HF5Q0P1NGUh0401ADE40EEEE06IVIP1PH600NGUi055AD@15ADD0NWYj04e=
C@1JFUX0KVi^04e=C@1lO7`0Kfm_051@D01eMGD2061PH00C06ATI01LG5`0IFEU07YjNP1KFe/0L71`
06QXJ01EEED0K6a/05QHF01;Bd/0IfMW04i>CP1ZJVX0IVIV04Q8B01JFUX0D51@04I6AP0>ooooo`0M
06IVIP1GEeL0HF5Q061PH01OGel0GUiN07=cL`1iNGT0LW9b065QH@18B4P0BDU905UIF@1LG5`0J6QX
05aLG01UIFD0G5aL06=SH`15ADD0D51@04];B`18B4P0=3@d04E5A@0a<C40AdM705QHF01IFET00P1O
Gel0201GEeL0MGEe04];B`1FEUH0LG5a05UIF@1;Bd/0FUYJ0P1;Bd/01`0f=SH0?Cdm03lo?`13@d<0
EeMG04I6AP11@D400P16ATH0FP1<C4`0?3`l051@D01MGEd0DE5A05=CD`1QHF40GUiN05MGE`1<C4`0
EeMG07AdM02ATI40IfMW08:2PP1OGel0KVi^04U9B@0n?Sh0Bd];04m?C`1RHV80O7al06a/K01DE5@0
De=C06a/K01GEeL0NWYj04U9B@17AdL0E5AD05mOG`1IFET0J6QX04];B`1LG5`0DU9B069RHP1ZJVX0
W9bL05aLG01EEED0CDe=05mOG`1XJ6P0Bd];065QH@1LG5`0Fe]K079bLP1_Kfl0CTi>04A4A01<C4`0
CDe=0451@@1FEUH0BTY:04I6AP1LG5`0GEeM04=3@`1GEeL0>CTi04E5A@1:BTX0CDe=05ADE01bLW80
DE5A061PH011@D40GEeM05aLG01ZJVX0O7al07AdM013@d<0EEEE07YjNP1QHF40ADE504i>CP15ADD0
?Shn05UIF@17AdL0F5QH06IVIPkooooo00h0Ng]k06IVIP1gMgL0MWIf06m_K`1=CDd0?Cdm03Ph>00l
?3`0:BTY04A4A00Z:RX0<c<c051@D080>3Ph0P0f=SH0KP0];Bd0=SHf03Xj>P1>CTh0A4A404a<C01?
Cdl0FEUI04m?C`1HF5P0=3@d04e=C@15ADD0@41004U9B@0e=CD09BDU04M7A`13@d<0@D5102l_;`1X
J6P0FUYJ0410@01iNGT0I6AT06e]K@2_[jl0GUiN07IfMP39bLT0GUiN06YZJP1VIVH0I6AT06YZJP1S
Hf<0E5AD079bLP1TI6@0F5QH04Y:BP1[Jf/0Fe]K06m_K`1`L700LW9b05YJFP14A4@0Rh^;04i>CP19
BDT0I6AT07MgM`0n?Sh0DE5A04a<C01YJFT0JVYZ07EeM@1XJ6P0MGEe06YZJP2`/;00F5QH0;Bd]01[
Jf/0LW9b07moO`1SHf<0Lg=c03Hf=P1MGEd0De=C04m?C`1@D500?3`l05UIF@1DE5@0IfMW06e]K@1;
Bd/0@d=30410@01oOgl0Ng]k0820P00k>c/0HF5Q04m?C`1XJ6P0Jf][05]KF`0n?Sh0IfMW03hn?P12
@T80NWYj04m?C`1SHf<0MgMg06e]K@1MGEd0L71`03Xj>P1=CDd0I6AT07=cL`1lO7`0DU9B0P1[Jf/0
2P1ZJVX0GEeM08^;R`1kNg/0OGem0:f][@19BDT0Bd];03Lg=`21PH4>ooooo`0507emO@16ATH0S8b<
03Lg=`0F5QH00P0C4a<0Q@1<C4`0O7al06m_K`0];Bd0:2PX051@D02c/k<0LG5a061PH00N7Qh05ADE
01HF5P0E5AD051@D01DE5@1:BTX09BDU01DE5@0];Bd0Ogmo04I6AP0`<300=3@d03Ph>01fMWH0`/;2
03dm?@0E5AD051@D01TI6@0T92@0Cdm?08R8R02i^KT0A4A401LG5`0E5AD0E5AD01lO7`0Z:RX0HF5Q
04E5A@0g=cL0MgMg096AT@11@D40CDe=01HF5P0O7al0;2`/03Hf=P2:RXX0Lg=c05ADE00^;Rh051@D
01/K6`0_;bl0B4Q80:2PX033`l<0CDe=01PH600b<S808b<S01HF5P0D51@05ADE04M7A`1:BTX0OWin
0=;BdP1PH600EEEE06YZJP2:RXX0YjNW0;Zj^P1aLG40:2PX05mOG`20P800@D510410@02>SXh0[Zj^
07]kN`0Q8B40B4Q803De=@0S8b<0SXj>09BDU027QhL0W9bL096AT@2GUiL0FUYJ05EEE@20P800[:b/
071`L019BDT071`L02/[:`1KFe/0g]kN0=?Cd`17AdL09BDU04M7A`24Q8@0:BTY02<S8`1DE5@0Hf=S
02PX:00T92@0KVi^0:j^[P1lO7`0N7Qh05ADE00a<C40=SHf0:ZZZP20P800PX:207AdM01^KVh09bLW
01HF5P0h>3P05?ooool001?ooooo0P1cLg<0A`1NGUh0HF5Q04m?C`1VIVH0BDU904e=C@1@D500Fe]K
0410@01OGel0BTY:055AD@1LG5`0GEeM05=CD`1[Jf/0EeMG04];B`1TI6@0CTi>04M7A`14A4@0BDU9
05iNGP0k>c/0FUYJ04];B`0j>SX0G5aL05QHF012@T80GemO051@D01MGEd0EUIF05QHF01<C4`0?clo
06IVIP1DE5@0ADE506m_K`1GEeL0?clo04Y:BP1SHf<0C4a<0410@01IFET0GEeM051@D01[Jf/0BTY:
03/k>`1^KVh0B4Q803hn?P1eMGD0ADE5069RHP1]KFd0EEEE06IVIP1ZJVX0Fe]K05mOG`1GEeL0IVIV
055AD@1RHV80LG5a0080HV9R00@0NWYj06=SH`1QHF40LW9b0P1ZJVX0>P1^KVh0GemO07UiN@1ZJVX0
L71`05=CD`1ZJVX0FEUI055AD@22PX80CTi>06i^KP1KFe/0DU9B08N7Q`1QHF40C4a<08B4Q01FEUH0
I6AT06MWI`1iNGT0H61P05eMG@1_Kfl0KFe]05YJFP1?Cdl0IVIV05]KF`1EEED0J6QX06a/K01MGEd0
J6QX06e]K@1QHF40Lg=c06e]K@1NGUh0NWYj07inOP1CDe<0HF5Q075aL@0o?cl0GemO06=SH`0i>CT0
Fe]K06EUI@0n?Sh0GEeM05YJFP10@400J6QX05iNGP12@T8>ooooo`0`06UYJ@1QHF40KFe]05mOG`1/
K6`0GEeM075aL@1^KVh0J6QX05IFEP14A4@0CDe=065QH@1HF5P0GemO05MGE`1PH600HV9R05QHF01<
C4`0Bd];051@D016ATH0@d=30492@P0o?cl0ADE505YJFP1<C4`0GEeM05]KF`1GEeL0NWYj05IFEP1M
GEd0IFEU05EEE@10@400IfMW05IFEP1=CDd0?3`l04a<C013@d<0ATI605mOG`14A4@0@d=30P17AdL0
<01>CTh0Bd];06QXJ01PH600FUYJ051@D01>CTh0GEeM04a<C01>CTh0EeMG06EUI@20P800B4Q805iN
GP17AdL0NGUi04e=C@0k>c/0@D5104e=C@1GEeL0Lg=c04Y:BP1?Cdl0EeMG06EUI@1TI6@0RXZ:069R
HP1DE5@0I6AT05aLG01>CTh0C4a<05EEE@1GEeL0Bd];07=cL`1lO7`0SHf=05=CD`1>CTh0Fe]K07Yj
NP26QXH0ADE504U9B@80EEEE02P0QXJ607AdM017AdL0B4Q805ADE01<C4`0?clo05=CD`18B4P0ADE5
059BDP1CDe<0?3`l05ADE00d=3@0<30`04A4A019BDT0AdM706a/K00o?cl0HV9R04Y:BP1LG5`0GemO
06=SH`1mOGd0O7al055AD@1hN7P0NGUi04i>CP10@400DE5A05ADE019BDT0FUYJ04e=C@1GEeL0I6AT
3_ooool05@1hN7P0G5aL07AdM01kNg/0JVYZ04m?C`17AdL0?Cdm0410@00^;Rh0B4Q802/[:`0c<c<0
EEEE02h^;P0e=CD0?Cdm03/k>`0j>SX0>3Ph03dm?@0204i>CP1e05IFEP1EEED0EUIF05EEE@1;Bd/0
>CTi05IFEP0m?Cd0@D5104a<C00e=CD0>3Ph04Y:BP17AdL0@D5103dm?@1PH600De=C04=3@`1YJFT0
G5aL065QH@2c/k<0CTi>065QH@2n_[h0KVi^069RHP1FEUH0MWIf05ADE01KFe/0IFEU069RHP1WIfL0
Cdm?04Q8B01[Jf/0Bd];06m_K`1NGUh0K6a/05YJFP0l?3`0OWin04a<C013@d<0LW9b06][J`1=CDd0
GEeM05=CD`1NGUh0Jf][07AdM01TI6@0KVi^075aL@2GUiL0D51@0::RXP1OGel0MGEe09ZJVP20P800
OWin04E5A@1;Bd/0GUiN05QHF01DE5@0BDU905]KF`1QHF40MgMg05=CD`1ADE40Jf][05MGE`1_Kfl0
IFEU08Z:RP11@D40CTi>06YZJP1OGel0Lg=c05eMG@10@400FEUI04E5A@1HF5P0MgMg05iNGP1YJFT0
OGem07YjNP1XJ6P0G5aL02d];@1FEUH0Bd];061PH023Ph<0Cdm?06a/K01UIFD0Fe]K06QXJ02=SHd0
O7al08f=S@2TY:@0A4A405=CD`1;Bd/0Ph>300kooooo01/0P82009BDU01[Jf/0>c/k02h^;P0X:2P0
7AdM02/[:`0k>c/0T92@0:FUY@2CTi<0QXJ60:f][@2GUiL0HV9R059BDP0h>3P0<30`02PX:00G5aL0
820P04Q8B00O7al05aLG04i>CP0c<c<00P0Q8B404P0L71`06a/K024Q8@21PH40OGem06YZJP0b<S80
;Bd]02/[:`0Q8B40=cLg08N7Q`1gMgL0NGUi08R8R01SHf<0?3`l03hn?P809BDU00`0DE5A09>CT`27
QhL0Ti>C08V9R@1FEUH0BTY:06ATI027QhL0XJ6Q0:^[Z`1SHf<201@D5009014A4@0I6AT0>SXj06][
J`1`L700MWIf04m?C`1eMGD07AdM00805QHF048051@D01lO7`2EUID0KFe]075aL@2/[:`0VIVI06QX
J01KFe/0HF5Q0:JVYP1[Jf/08b<S03De=@1:BTX0SHf=05iNGP2FUYH0Lg=c08V9R@2YZJT0WYjN09RH
V01nOWh0RHV9051@D00/;2`0?3`l09ZJVP0a<C40;Rh^04U9B@1HF5P0AdM7061PH01mOGd0UiNG09>C
T`2?Shl0WinO0:j^[P26QXH0<30`03Xj>P20P800?clo01`L700O7al0JFUY02l_;`0e=CD0Xj>S08V9
R@1ZJVX07alO02@T900h>3P0De=C07]kN`0/;2`0CDe=08j>SP2e]KD0L71`02XZ:P0P820Dooooo`00
4oooool0R`2DU9@0KFe]071`L015ADD0Kfm_07emO@0o?cl0FEUI04Q8B01;Bd/0ADE505ADE01^KVh0
EeMG055AD@2>SXh0F5QH05]KF`1UIFD0AdM7059BDP1>CTh0=cLg03hn?P1TI6@0@T9203lo?`1LG5`0
@41004];B`1JFUX0ADE504m?C`1JFUX0ATI606MWI`1>CTh0AdM706m_K`18B4P0H61P06UYJ@0`<300
JFUY06ATI00h>3P0H61P05EEE@1ADE40BTY:04i>CP1JFUX0EUIF05=CD`20P800DE5A05MGE`1hN7P0
=SHf04a<C01?Cdl0C4a<05=CD`1OGel0EEEE055AD@1[Jf/0@D51055AD@1XJ6P0;Rh^06=SH`20P800
?Shn061PH01lO7`0?Shn06IVIP1aLG40ADE508F5Q@1SHf<0CDe=08R8R01CDe<0GEeM04U9B@1IFET0
H61P03/k>`1HF5P0HV9R05aLG01ADE40De=C061PH01bLW80GUiN06MWI`1/K6`0D51@05]KF`2:RXX0
ATI6071`L01/K6`0D51@06UYJ@0^;Rh0IVIV06e]K@1;Bd/0FUYJ09ZJVP0m?Cd0Ph>308V9R@18B4P0
NWYj07emO@0m?Cd0Hf=S07inOP0k>c/0K6a/06=SH`1FEUH0Ph>306=SH`1@D500IVIV05eMG@1?Cdl0
PH6104U9B@1=CDd0MWIf05=CD`1ADE403_ooool0D`1TI6@0IfMW06IVIP1QHF40FUYJ05aLG01jNWX0
QhN706QXJ01ADE40A4A404Q8B01LG5`0DU9B04];B`1QHF40GEeM06i^KP1OGel0FEUI05aLG017AdL0
ADE504A4A01=CDd0=SHf0451@@1CDe<0BDU904m?C`1CDe<0GEeM07]kN`1KFe/0GEeM065QH@1EEED0
@T9206UYJ@1IFET0BDU90410@017AdL0?Cdm0492@P1LG5`0>SXj03dm?@0o?cl0BDU904Y:BP1<C4`0
Jf][05mOG`1CDe<0BTY:04M7A`1DE5@0AdM705aLG01TI6@0PX:209JFUP1aLG40IfMW04e=C@1nOWh0
IFEU04=3@`1ADE40Bd];06=SH`1oOgl0F5QH06][J`1BDU80JFUY06QXJ02:RXX0Cdm?04I6AP17AdL0
DU9B0080Bd];01d0JFUY05MGE`1KFe/0F5QH0861P@1oOgl0@d=305=CD`1FEUH0Kfm_06QXJ01:BTX0
B4Q8061PH010@400MgMg04Q8B00Z:RX0>3Ph04m?C`12@T80?Cdm04U9B@13@d<0C4a<04Q8B01HF5P0
;Rh^04m?C`020492@P0H04U9B@1:BTX0DE5A07AdM015ADD0HV9R04M7A`1FEUH0HF5Q055AD@1WIfL0
Kfm_04I6AP1^KVh0G5aL04m?C`1@D500E5AD04];B`16ATH0NGUi04U9B@1KFe/0J6QX3_ooool0101m
OGd0Fe]K07]kN`2=SHd205aLG0260492@P0c<c<0>c/k02`/;015ADD0<C4a02l_;`1<C4`0<30`03<c
<`0o?cl0<c<c04Y:BP0h>3P0?Cdm05UIF@1>CTh0HV9R04a<C01JFUX0I6AT03lo?`1?Cdl0F5QH059B
DP13@d<0C4a<03hn?P0m?Cd0C4a<059BDP0f=SH0@T9206QXJ01>CTh0?Shn08b<S01^KVh0=3@d0:2P
X01JFUX0G5aL09JFUP21PH40D51@06YZJP1XJ6P0FEUI04];B`1TI6@0P82006EUI@1CDe<0EEEE05eM
G@1<C4`0EEEE05mOG`1PH600HV9R03lo?`23Ph<0E5AD04e=C@1hN7P0IVIV05=CD`1OGel0=SHf065Q
H@1JFUX0NGUi04e=C@19BDT0Lg=c09JFUP1ZJVX0QhN707EeM@25QHD0YJFU075aL@28R8P0?Cdm0410
@01JFUX0EeMG061PH00o?cl0>SXj06][J`1]KFd0Cdm?04a<C01=CDd0AdM708N7Q`1^KVh0LW9b04E5
A@1IFET0EeMG05eMG@1QHF40Bd];03Ti>@1SHf<0>c/k059BDP28R8P0B4Q806EUI@28R8P0HF5Q05=C
D`1RHV80>CTi04e=C@0k>c/0NWYj08:2PP1GEeL0JVYZ06a/K01JFUX0Lg=c0:BTY01YJFT0Ph>309BD
U015ADD0GUiN03/k>`2<S8`>ooooo`0M08B4Q01@D500=3@d02XZ:P0c<c<0Fe]K05=CD`0L71`0@d=3
04i>CP0h>3P0?Shn07moO`27QhL0S8b<0::RXP1ADE40;2`/03Ti>@15ADD0DU9B08Z:RP18B4P0?clo
05YJFP0i>CT0Bd];02TY:@0D51@00P0C4a<0K@1<C4`0NGUi08R8R02IVIT0CDe=04M7A`1IFET0SHf=
0::RXP0G5aL0;Bd]079bLP2>SXh0LW9b09VIV@1/K6`0>CTi04U9B@15ADD0FEUI0;Vi^@2GUiL0@T92
055AD@11@D40CDe=06YZJP2HV9P0UiNG05YJFP0D51@04Q8B010@400I6AT0L71`0;6a/@2k^k/0LG5a
06IVIP2`/;00>3Ph028R8P0G5aL05ADE028R8P1/K6`0DE5A07emO@1kNg/0BTY:01/K6`0H61P0?3`l
05MGE`2CTi<0BDU902PX:012@T80>SXj08N7Q`0n?Sh0K6a/0:f][@3Ogml0g=cL0=oOg`2SXj<0MGEe
03Ti>@0N7Qh0@41006QXJ00Y:BT0=CDe030`<00j>SX0V9RH05eMG@1LG5`0WYjN09>CT`16ATH0BDU9
04a<C02ATI40Q8B403hn?P0g=cL0HV9R05IFEP1XJ6P0X:2P071`L02@T900O7al06=SH`1=CDd07AdM
02TY:@1:BTX0GemO08b<S01=CDd0Ti>C0;^k^`1TI6@0De=C05aLG01EEED05?ooool001?ooooo03T0
NGUi08:2PP1?Cdl0EeMG05]KF`17AdL0IFEU0492@P15ADD0Lg=c02h^;P1;Bd/0IVIV03<c<`1CDe<0
JVYZ03dm?@1QHF40EeMG034a<@1JFUX0?clo04e=C@19BDT0C4a<08J6QP14A4@0BTY:06e]K@0j>SX0
B4Q806EUI@0Z:RX0E5AD071`L012@T80EeMG08B4Q01:BTX0HV9R06=SH`14A4@0FUYJ065QH@1?Cdl0
EeMG03`l?01ZJVX0FEUI02PX:01NGUh0G5aL03Ti>@1PH600HF5Q05QHF01MGEd00P1DE5@01@1ZJVX0
@d=304e=C@1cLg<0=SHf0080Hf=S04T0@d=306YZJP13@d<0Cdm?04i>CP1DE5@0I6AT06e]K@1@D500
JFUY071`L01:BTX0I6AT071`L01BDU80GemO08>3P`1<C4`0L71`07emO@19BDT0Ng]k06=SH`11@D40
MGEe061PH01TI6@0GEeM051@D01jNWX0IfMW04i>CP2CTi<0JVYZ06IVIP1PH600GUiN06ATI01?Cdl0
Kfm_07QhN01BDU80GemO065QH@1KFe/0N7Qh06=SH`1LG5`0L71`061PH01WIfL0M7Ad0492@P2:RXX0
OGem05EEE@2>SXh0Cdm?05YJFP1iNGT0DU9B05IFEP1iNGT0?clo05]KF`27QhL0>c/k06EUI@1OGel0
>c/k05]KF`1[Jf/0AdM700kooooo0100HF5Q04e=C@1MGEd0C4a<05aLG01BDU80GemO079bLP1YJFT0
ADE504Y:BP19BDT0FUYJ04];B`1JFUX0CDe=0P1MGEd01`1JFUX0BTY:04U9B@14A4@0?Cdm0492@P11
@D400P0o?cl0=P1OGel0C4a<04e=C@1EEED0HV9R0861P@1RHV80JFUY06MWI`1<C4`0AdM705aLG01O
Gel0DE5A04Q8B01CDe<0@41005EEE@1[Jf/0BTY:04M7A`1@D500BDU904i>CP1HF5P0Ogmo05mOG`1D
E5@0GEeM04I6AP1EEED0DE5A04I6AP1JFUX0IVIV08>3P`1JFUX0I6AT06IVIP2>SXh0IVIV051@D01>
CTh0AdM705EEE@24Q8@0I6AT05UIF@1lO7`0OWin07moO`1nOWh0JVYZ051@D080E5AD00/0D51@059B
DP1hN7P0GUiN05MGE`1CDe<0IfMW08V9R@1TI6@0D51@04Y:BP0207=cL`0L04E5A@18B4P0HV9R04m?
C`1lO7`0GemO03@d=010@400CTi>04Y:BP16ATH0Bd];04e=C@1FEUH0CTi>05mOG`12@T80J6QX0451
@@0m?Cd0BDU904e=C@1<C4`0IFEU04Q8B01SHf<0>c/k069RHP80Bd];00h0G5aL05]KF`1=CDd0KFe]
05=CD`0n?Sh0@d=304M7A`0l?3`0CTi>06i^KP17AdL0EUIF07=cL`kooooo0180NGUi04Y:BP1JFUX0
M7Ad05YJFP1@D500:b/[03`l?00f=SH0@D5104];B`0l?3`0@D5104e=C@0R8R80>CTi03/k>`0i>CT2
03Xj>P0b030`<00c<c<0BTY:04m?C`1@D500ADE505EEE@17AdL0Bd];04Q8B016ATH0Bd];04=3@`0n
?Sh0B4Q804m?C`1;Bd/0<S8b03hn?P1MGEd0C4a<04M7A`1`L700KFe]04m?C`2GUiL0DU9B05mOG`2Z
ZZX0NWYj05IFEP1=CDd0IVIV059BDP17AdL0LW9b07MgM`1YJFT0Cdm?051@D01eMGD0C4a<06YZJP1P
H600IVIV06e]K@0m?Cd0MgMg05IFEP1;Bd/207QhN00e059BDP1/K6`0?Shn05aLG01_Kfl0Jf][05eM
G@1@D500HF5Q08f=S@1JFUX0TI6A05YJFP1bLW80ZJVY07alO02GUiL0Cdm?0410@01KFe/0G5aL069R
HP1>CTh0E5AD06MWI`1dM7@0DU9B055AD@1SHf<0FUYJ06][J`1dM7@0NWYj04Q8B01EEED0L71`04A4
A01gMgL0GEeM03dm?@1PH600AdM705aLG01oOgl0De=C06UYJ@1fMWH0NGUi06ATI01SHf<0>CTi05AD
E00l?3`00P24Q8@03@0m?Cd0M7Ad08:2PP1NGUh0IfMW09NGU`1>CTh0MGEe09>CT`0k>c/0BTY:04i>
CP2?Shl03_ooool0B@2FUYH0JVYZ03Hf=P0_;bl08b<S04Y:BP25QHD0OGem03Xj>P0a<C409BDU02/[
:`0X:2P05QHF05]KF`2JVYX0UiNG05YJFP0[:b/0;2`/051@D02>SXh0MGEe0:>SX`2BTY80R8R805AD
E00Z:RX05aLG01TI6@1BDU80<S8b02XZ:P2MWId0XJ6Q07YjNP1^KVh0OGem0;no_`2YZJT0FUYJ01`L
701?Cdl0clo?0:>SX`1iNGT0EeMG06IVIP0H61P05ADE03@d=01UIFD0IfMW0492@P0`<300<C4a02h^
;P1EEED0Ogmo05UIF@1/K6`0=cLg01@D500K6a/0>3Ph05QHF01lO7`0b<S80;6a/@2d];@0E5AD03`l
?00f=SH00P0/;2`0@@1?Cdl0Ph>308F5Q@2:RXX0GemO01TI6@0C4a<05ADE02PX:01CDe<0O7al07If
MP1NGUh092@T01HF5P0Y:BT0F5QH06UYJ@2WYjL0[jn_08>3P`1]KFd0TY:B0;:b/P1mOGd0;bl_05UI
F@11@D40:BTY05=CD`0N7Qh0<C4a05UIF@0[:b/09bLW05YJFP0l?3`0@d=302DU9@0X:2P0L71`0861
P@1mOGd0L71`0:FUY@29RHT0ATI603Ti>@2ATI40d];B04I6AP1XJ6P0Bd];0410@017AdL0FEUI06m_
K`2<S8`0Ogmo0:j^[P2l_;`0<c<c02TY:@1UIFD0QHF501Cooooo000Cooooo`0L09BDU01XJ6P0L71`
05EEE@1ZJVX0IFEU04U9B@1nOWh0CTi>04a<C01GEeL0?3`l05]KF`1PH600A4A4061PH01]KFd0DU9B
055AD@1@D500ADE505]KF`1JFUX0?clo06QXJ01BDU80B4Q807AdM080BDU906d0GEeM04Q8B014A4@0
BDU903lo?`1dM7@0?Cdm04E5A@1iNGT0A4A405IFEP1VIVH0<C4a05UIF@16ATH0>3Ph051@D01<C4`0
ADE504m?C`1SHf<0=CDe05EEE@1<C4`0ADE505UIF@1=CDd0DE5A0410@01TI6@0HV9R05mOG`1MGEd0
M7Ad04];B`1OGel0NGUi04m?C`17AdL0JFUY04A4A01SHf<0OWin04e=C@1NGUh0Ogmo04=3@`1IFET0
M7Ad03hn?P1ADE40IFEU05]KF`1VIVH0FEUI069RHP1MGEd0G5aL06m_K`1PH600@41005IFEP1gMgL0
@D51061PH01QHF40Bd];061PH01MGEd0L71`05EEE@1TI6@0PH6104m?C`1QHF40Hf=S06IVIP1[Jf/0
<c<c05QHF01_Kfl0@d=3065QH@2ATI40?Shn05mOG`2MWId0CTi>05aLG01[Jf/0ADE506QXJ01nOWh0
=CDe05UIF@1JFUX0@d=306ATI01LG5`0AdM706QXJ01iNGT0BDU906UYJ@1?Cdl0Fe]K07YjNP1?Cdl0
D51@00kooooo00<0KVi^05=CD`1QHF400P1MGEd0Q`1DE5@0IVIV07YjNP1aLG40ATI604e=C@17AdL0
EeMG04a<C01MGEd0E5AD06EUI@1RHV80E5AD059BDP1FEUH0Cdm?059BDP1=CDd0Cdm?03Ph>00k>c/0
CDe=03Ti>@16ATH0EUIF061PH028R8P0CDe=06ATI01ZJVX0Fe]K04U9B@1GEeL0EEEE05MGE`18B4P0
DE5A03lo?`16ATH0EEEE04=3@`1FEUH0C4a<04I6AP1IFET0EUIF07inOP1RHV80GemO05YJFP1[Jf/0
Cdm?04e=C@1>CTh0GUiN06i^KP2HV9P0Jf][06IVIP1BDU80L71`05QHF00m?Cd0BDU905ADE01IFET0
NWYj04M7A`1@D500DE5A05QHF01VIVH0Ogmo05UIF@18B4P0CTi>05UIF@18B4P0@41006][J`1ADE40
BTY:051@D01BDU80RHV905MGE`1OGel0CTi>06UYJ@1nOWh0ADE504M7A`1OGel0E5AD08:2PP12@T80
;bl_04Y:BP1>CTh0ATI603/k>`18B4P0ATI604m?C`1ADE40FEUI0410@01=CDd0<c<c03Hf=P10@400
?Cdm04e=C@1BDU80@d=305UIF@0_;bl0EUIF0410@01?Cdl0G5aL05eMG@1DE5@0EUIF04];B`15ADD0
AdM704];B`12@T80?Shn06m_K`1=CDd0H61P06=SH`0>ooooo`0A07emO@1CDe<0EUIF06][J`1PH600
De=C051@D00h>3P0=SHf02h^;P0o?cl0@41003<c<`13@d<0;Rh^02PX:01;Bd/00P0b<S80N@13@d<0
<30`04i>CP14A4@0CDe=059BDP0l?3`0I6AT04Q8B01PH600@T9206MWI`13@d<0@41004=3@`18B4P0
F5QH0410@00Y:BT0A4A406QXJ013@d<0HV9R08^;R`1dM7@0De=C07inOP19BDT0EUIF09>CT`2ATI40
E5AD06ATI01bLW80Fe]K04];B`1XJ6P0Kfm_05iNGP1LG5`0EeMG07=cL`1@D500H61P06i^KP1XJ6P0
K6a/04U9B@20P800F5QH04a<C01eMGD0MgMg05=CD`1OGel0A4A405MGE`1YJFT0NGUi05UIF@13@d<0
F5QH09VIV@1QHF40MGEe06a/K022PX80];Bd06e]K@2[Zj/0FEUI03hn?P1IFET0GUiN071`L01=CDd0
ATI606UYJ@1hN7P0EeMG05mOG`1`L700Bd];07alO01VIVH0LG5a03`l?01VIVH0E5AD04];B`1iNGT0
FEUI02/[:`1ZJVX0Bd];04I6AP1hN7P0DU9B05EEE@26QXH0Ogmo05MGE`1EEED0@T9205IFEP0n?Sh0
LW9b08F5Q@1=CDd0KVi^06ATI01eMGD0JFUY09RHV019BDT0GEeM08N7Q`0h>3P0B4Q80410@024Q8@0
3_ooool0S02ZZZX0U9BD059BDP1:BTX0@41004m?C`2CTi<0I6AT03dm?@0h>3P0;Bd]04=3@`0G5aL0
5ADE02@T901?Cdl0/;2`0<c<c02KVi/0WYjN0=KFeP3Bd]80Vi^K0861P@2>SXh0>3Ph03dm?@1`L700
Fe]K075aL@0O7al05ADE02TY:@1gMgL0O7al065QH@0c<c<0;Bd]06a/K01hN7P0EeMG05IFEP1cLg<0
e=CD0:BTY01NGUh0LW9b06i^KP13@d<0?Cdm06i^KP2<S8`0Jf][01PH600E5AD06a/K05eMG@0l?3`0
71`L02PX:01@D500QXJ6061PH00k>c/0Bd];065QH@1MGEd0WYjN09bLW02GUiL09bLW01/K6`0_;bl0
A4A403Ph>01mOGd0`<3009>CT`2PX:00>SXj01@D500E5AD04Q8B01TI6@1[Jf/0Vi^K07EeM@1bLW80
DU9B04];B`1ZJVX0Y:BT06][J`1NGUh0U9BD04Q8B026QXH0Y:BT07QhN01@D500WinO0:j^[P0`<300
B4Q805YJFP0l?3`0>3Ph05mOG`1<C4`0Fe]K07emO@0^;Rh0:2PX028R8P0V9RH0?Cdm06][J`2OWil0
/k>c0<W9b@1nOWh0EUIF065QH@1dM7@0c/k>09VIV@2ATI40T92@0:>SX`2LW9`0MGEe0410@00_;bl0
U9BD09bLW02FUYH0I6AT05mOG`2FUYH0Z:RX5?ooool001?ooooo05@0N7Qh06][J`1RHV80GUiN061P
H01^KVh0De=C05iNGP1EEED0Hf=S03hn?P1CDe<0Jf][04M7A`1@D500FEUI03Xj>P1LG5`0Fe]K03Xj
>P1EEED0FUYJ04M7A`1@D500G5aL05ADE012@T80G5aL04];B`14A4@0FUYJ05=CD`1JFUX0De=C061P
H01^KVh0C4a<06MWI`1UIFD0@T9207QhN01BDU80AdM7065QH@1<C4`0Hf=S06UYJ@1>CTh0Kfm_03hn
?P1OGel0Cdm?03`l?01MGEd0HV9R03<c<`1QHF40FUYJ03Ti>@1HF5P0FUYJ04M7A`1TI6@0FEUI04e=
C@1BDU80Kfm_05YJFP1DE5@0J6QX059BDP1SHf<0LG5a05EEE@1OGel0Lg=c05aLG01OGel0HF5Q05mO
G`1TI6@0Hf=S06MWI`1FEUH206QXJ00@04=3@`1^KVh0I6AT04=3@`1:BTX0IfMW05iNGP1?Cdl0DU9B
06EUI@1/K6`0FUYJ06e]K@1NGUh0EUIF06MWI`80Hf=S0240GemO04e=C@1eMGD0H61P04e=C@1iNGT0
HV9R05mOG`22PX80JVYZ05YJFP1/K6`0HV9R06MWI`1<C4`0IFEU05mOG`1:BTX0K6a/05MGE`1<C4`0
MgMg059BDP1=CDd0MgMg04];B`1EEED0Lg=c04U9B@1UIFD0EEEE04Y:BP1IFET00P1EEED>ooooo`0/
06MWI`1HF5P0De=C04a<C01:BTX0G5aL06i^KP2=SHd0KVi^04a<C011@D40Bd];05QHF01LG5`0Fe]K
05eMG@1]KFd0IVIV05aLG01BDU80De=C04U9B@10@400Bd];04I6AP0h>3P0?clo055AD@11@D40C4a<
05=CD`1MGEd0T92@059BDP1OGel0Hf=S055AD@1DE5@0AdM70492@P1:BTX0BDU904a<C00i>CT20451
@@80@T9200X0B4Q804I6AP1@D500F5QH075aL@1OGel0EeMG06UYJ@1<C4`0CTi>0P1?Cdl06P1OGel0
Ng]k0:n_[`1UIFD0GemO05QHF01kNg/0G5aL059BDP1KFe/0GemO07=cL`2EUID0J6QX06MWI`1YJFT0
PX:2075aL@1hN7P0GUiN04e=C@15ADD0BTY:05]KF`1;Bd/0FEUI0P1<C4`0=01>CTh0Fe]K07inOP1F
EUH0Fe]K06=SH`1_Kfl0Rh^;04];B`0k>c/0E5AD04e=C@1UIFD0CDe=02h^;P15ADD0De=C04E5A@0m
?Cd0>SXj03hn?P1LG5`0DE5A05mOG`12@T80EeMG03De=@0h>3P0?Shn04];B`1?Cdl0FEUI0410@01M
GEd0@d=305EEE@1;Bd/0DU9B05]KF`1JFUX0HV9R05EEE@1RHV80Bd];03dm?@19BDT0Bd];04e=C@1e
MGD0DU9B05UIF@1`L70>ooooo`0:07MgM`1ADE40E5AD06IVIP1BDU80AdM705IFEP0m?Cd0@41003Xj
>P80B4Q803@0@T92055AD@0];Bd0;Bd]04M7A`0e=CD0=SHf04E5A@0[:b/0@d=305ADE01?Cdl0B4Q8
03hn?P1QHF40@41004a<C01:BTX0FUYJ04i>CP0o?cl0C4a<059BDP1HF5P0@41002`/;016ATH0Hf=S
03lo?`1?Cdl0NGUi06][J`1WIfL0M7Ad04U9B@1DE5@0P82008Z:RP1CDe<0@d=305eMG@1CDe<0A4A4
06e]K@1WIfL0GEeM06YZJP19BDT0Q8B404];B`1]KFd0GemO0P1bLW80BP0l?3`0L71`05ADE014A4@0
O7al07QhN01>CTh0GEeM04A4A01=CDd0I6AT06m_K`1[Jf/0GUiN05QHF01oOgl0D51@06][J`1MGEd0
I6AT0:j^[P1RHV80MWIf05ADE00e=CD0EeMG05IFEP1^KVh0BDU9061PH01]KFd0Lg=c04m?C`1@D500
MgMg04M7A`1jNWX0Jf][075aL@14A4@0FUYJ069RHP0l?3`0JFUY05eMG@0i>CT0G5aL05IFEP1IFET0
M7Ad05MGE`1LG5`0K6a/07alO01[Jf/0FEUI03Ph>01OGel0B4Q8071`L02:RXX0Cdm?07YjNP1TI6@0
MgMg07=cL`28R8P0A4A406=SH`2DU9@0=3@d051@D017AdL0W9bL3_ooool09`1oOgl0IFEU06EUI@1O
Gel0PX:209nOW`1mOGd0S8b<065QH@1?Cdl0FEUI069RHP0g=cL0>c/k02l_;`19BDT0NWYj09jNWP1N
GUh08R8R03Lg=`2IVIT0E5AD05aLG02QXJ40C4a<04I6AP0l?3`0@41003hn?P1@D500I6AT09NGU`16
ATH0>CTi020P800/;2`0:RXZ02@T900205mOG`1S04e=C@1aLG40UIFE0451@@2ATI40Lg=c07IfMP0o
?cl0=3@d03`l?01mOGd0MGEe06=SH`0l?3`0@D5105EEE@0Z:RX05ADE01hN7P0Y:BT0HF5Q08j>SP2Y
ZJT0]kNg06QXJ01kNg/071`L05UIF@19BDT0ATI604M7A`0e=CD07alO04U9B@2OWil0S8b<0;Ng]`0k
>c/0AdM703Xj>P0b<S80<C4a0451@@1jNWX0=cLg03Ph>01OGel0ADE5038b<P1<C4`0Q8B40:NWY`2E
UID0LW9b07=cL`1mOGd0RHV907EeM@1gMgL0F5QH08Z:RP1fMWH06a/K02<S8`1;Bd/0QXJ609nOW`1^
KVh0E5AD08R8R024Q8@0B4Q802@T900];Bd08R8R03Ti>@1[Jf/0SHf=0;Ng]`0o?cl071`L02`/;01j
NWX0Xj>S07alO01hN7P0WYjN0;jn_P22PX80E5AD06UYJ@1ZJVX0Ph>30:f][@22PX80=SHf04a<C01H
F5P0=CDe01Cooooo000Cooooo`0g06QXJ020P800C4a<07UiN@1KFe/0GEeM06ATI00i>CT0I6AT06QX
J00a<C40Bd];05mOG`15ADD0DE5A05QHF01VIVH0HF5Q04i>CP1@D500Cdm?05=CD`1FEUH0?Shn05aL
G01fMWH0?3`l04m?C`1^KVh0?Shn04Y:BP1aLG40>3Ph04A4A01_Kfl0>SXj05]KF`1cLg<0?Cdm069R
HP1NGUh0@T9204e=C@16ATH0?Shn05aLG00];Bd0AdM704i>CP0X:2P0J6QX04M7A`0l?3`0K6a/04E5
A@0205aLG01B055AD@1BDU80Hf=S03lo?`1IFET0G5aL04e=C@1IFET0Hf=S05QHF01[Jf/0ATI604];
B`1DE5@0DE5A06IVIP1_Kfl0DU9B06a/K01VIVH0BDU9071`L01]KFd0Bd];05mOG`1]KFd0?3`l069R
HP1JFUX0BDU906][J`1bLW80De=C05UIF@1ADE40Jf][03`l?01HF5P0Ng]k0451@@1JFUX0Kfm_05eM
G@1ZJVX0Fe]K06EUI@20P800C4a<05]KF`1mOGd0ATI606EUI@1KFe/0FEUI05YJFP1BDU80I6AT05UI
F@1;Bd/0R8R8061PH01IFET0S8b<05YJFP1=CDd0KFe]04U9B@1DE5@0RHV904A4A01JFUX0MWIf03Ti
>@1FEUH0P82003<c<`1DE5@0H61P038b<P1JFUX0HV9R03Ph>0kooooo03/0M7Ad05eMG@1QHF40EUIF
05=CD`1TI6@0NWYj0::RXP1gMgL0De=C04A4A01=CDd0FEUI04m?C`1DE5@0IFEU07MgM`1bLW80EEEE
055AD@1PH600CDe=04Y:BP1;Bd/0B4Q804M7A`0k>c/0K6a/04e=C@1FEUH0CTi>05eMG@2<S8`0H61P
065QH@1XJ6P0E5AD04];B`1@D500?Cdm04e=C@16ATH0CDe=0492@P15ADD0CDe=0451@@17AdL0BTY:
05=CD`1KFe/0FUYJ07]kN`1YJFT0HF5Q061PH01DE5@0EEEE04e=C@0204a<C00g06QXJ025QHD0GEeM
06YZJP10@400IVIV05IFEP10@400ATI60410@01UIFD0OWin05aLG01EEED0LG5a06a/K01GEeL0PH61
05EEE@1:BTX0BDU905MGE`19BDT0>CTi05YJFP1IFET0EEEE06QXJ01VIVH0Lg=c04U9B@1BDU80FUYJ
07EeM@2;Rh/0EUIF04A4A01LG5`0D51@05UIF@1>CTh0<S8b04M7A`1DE5@0BDU904M7A`19BDT0B4Q8
061PH01ADE40GEeM0451@@1LG5`0?Shn04M7A`0204Q8B00F04e=C@1<C4`0@41005IFEP0j>SX0AdM7
055AD@17AdL0JFUY05QHF01@D500CDe=05]KF`17AdL0@T9204];B`1LG5`0F5QH06m_K`1<C4`0Fe]K
06UYJ@kooooo08`0PX:204m?C`1QHF40IFEU05UIF@1KFe/0Bd];04I6AP13@d<0@4100492@P0k>c/0
D51@04Y:BP0Z:RX0;Rh^059BDP0b<S80CTi>03hn?P0a<C40D51@065QH@10@400?clo05]KF`1DE5@0
A4A406EUI@13@d<0FUYJ05ADE00a<C40D51@06IVIP17AdL0?Cdm038b<P14A4@0GUiN0492@P1DE5@0
NWYj07UiN@1OGel0EEEE055AD@1>CTh0KFe]08f=S@10@400CDe=05UIF@19BDT0CDe=06=SH`1VIVH0
GUiN05MGE`1>CTh0MGEe04i>CP1CDe<0HF5Q06MWI`1UIFD0=cLg07alO01FEUH0ADE507]kN`1hN7P0
De=C061PH00l?3`0ATI606ATI01lO7`0I6AT09:BTP1;Bd/0RHV906IVIP1KFe/0GUiN06i^KP2[Zj/0
K6a/07YjNP1EEED0;bl_04m?C`1:BTX0L71`0451@@1TI6@0KVi^06MWI`1=CDd0F5QH06UYJ@0k>c/0
J6QX07AdM01kNg/0?Cdm06][J`1HF5P0Bd];07QhN01JFUX0=SHf061PH01KFe/0De=C07AdM01UIFD0
EUIF05mOG`29RHT0Kfm_05YJFP0o?cl0EeMG03hn?P1^KVh0P82005aLG01cLg<0HF5Q05UIF@1SHf<0
OWin04e=C@1ADE40SHf=04E5A@1BDU80?Cdm0:>SX`kooooo02@0A4A40451@@10@400QHF508b<S01W
IfL0@D5102DU9@1PH600ADE507alO01BDU80820P01dM7@0C4a<0DE5A05]KF`1LG5`09RHV02<S8`0>
3Ph092@T0451@@0J6QX0Fe]K020P800G5aL09BDU028R8P0E5AD06ATI01dM7@1eMGD0?Cdm01<C4`0B
4Q8201@D500;02TY:@0o?cl0Bd];02h^;P1>CTh0HV9R04m?C`1^KVh0:2PX034a<@0G5aL00P0E5AD0
F@0N7Qh0De=C08b<S01PH600PH6103Ph>00i>CT0;2`/01TI6@0P8200<C4a02PX:010@400NWYj03lo
?`12@T80DU9B04Q8B00l?3`06QXJ01`L7016ATH0Rh^;09BDU00c<c<0MGEe0;Bd]00];Bd08R8R02/[
:`0K6a/07AdM05IFEP2n_[h0?Cdm02PX:01;Bd/09bLW04=3@`1dM7@0S8b<05eMG@22PX80;2`/030`
<01SHf<0MgMg07inOP2UYJD0S8b<06UYJ@1XJ6P05QHF030`<01kNg/0/;2`0<_;b`20P800BDU9020P
801oOgl0EUIF03Xj>P0J6QX071`L04m?C`0T92@0CTi>04m?C`2JVYX0@41006=SH`2FUYH0PX:20492
@P0J6QX0@D5108N7Q`1JFUX092@T01dM7@0/;2`0Shn?05ADE016ATH0<S8b01lO7`0^;Rh0EEEE01Co
oooo000Cooooo`1007]kN`1PH600Kfm_04i>CP1OGel0HF5Q04=3@`1ADE40GEeM0492@P1MGEd0A4A4
05QHF01JFUX0@D5106YZJP1BDU80BDU906EUI@1<C4`0=SHf06][J`1SHf<0<c<c06i^KP1=CDd0@D51
03`l?01CDe<0IfMW04A4A00o?cl0P820038b<P17AdL0O7al02l_;`1RHV80Jf][03De=@1GEeL0IfMW
03De=@1UIFD0D51@03hn?P1EEED0De=C059BDP1<C4`0G5aL05IFEP1@D500AdM705eMG@0i>CT0CDe=
07YjNP0j>SX0?3`l06YZJP1FEUH0CDe=08:2PP80DU9B04T0V9RH05=CD`1?Cdl0MWIf03Ph>01/K6`0
MGEe0451@@1]KFd0JFUY04e=C@1OGel0Hf=S03dm?@1MGEd0HV9R04i>CP1HF5P0EUIF08B4Q01=CDd0
Cdm?07YjNP1KFe/0<C4a06MWI`1QHF40B4Q806e]K@1MGEd0HF5Q05MGE`1<C4`0OGem055AD@1JFUX0
Lg=c04a<C01YJFT0>CTi069RHP1ZJVX0BDU906ATI01[Jf/0ADE505iNGP22PX80ATI605eMG@23Ph<0
CTi>05]KF`1nOWh0F5QH061PH01cLg<0CTi>06MWI`1]KFd0CTi>06QXJ01YJFT0BTY:05YJFP1EEED0
ATI6071`L01DE5@0Bd];07EeM@17AdL0C4a<00kooooo0540MWIf05UIF@1ZJVX0D51@065QH@1CDe<0
N7Qh06EUI@1RHV80BDU904i>CP0h>3P0DU9B03hn?P1fMWH0DU9B06EUI@1ZJVX0J6QX04U9B@1FEUH0
H61P05=CD`1RHV80DE5A04=3@`1MGEd0L71`059BDP1:BTX0DU9B06YZJP2;Rh/0HF5Q079bLP1UIFD0
DU9B06EUI@1LG5`0BTY:04];B`1>CTh0DU9B04E5A@1>CTh0EEEE04m?C`1:BTX0CDe=05ADE01HF5P0
Hf=S07moO`1RHV80JFUY07UiN@1YJFT0EEEE06=SH`15ADD0F5QH069RHP2`/;00ATI607QhN01^KVh0
Ng]k05ADE00];Bd0=SHf04I6AP1?Cdl0SXj>05UIF@1JFUX0?Shn04Y:BP1SHf<0LW9b05eMG@1BDU80
0P1ADE40>@1=CDd0>SXj04I6AP0n?Sh0EeMG05]KF`1SHf<0M7Ad05aLG01FEUH0Fe]K05UIF@1bLW80
DE5A03lo?`14A4@0DU9B04Y:BP17AdL0;Rh^03hn?P1>CTh0ADE504I6AP17AdL0@d=306e]K@1CDe<0
DE5A038b<P1=CDd0@41004E5A@0l?3`0BDU904Y:BP1QHF40<c<c04Q8B017AdL0FUYJ05ADE01:BTX0
@d=306IVIP1LG5`0EUIF05QHF017AdL0>SXj0451@@1CDe<0B4Q808>3P`1NGUh0FUYJ06YZJP0>oooo
o`0408>3P`1CDe<0G5aL05mOG`80A4A401H0CTi>04=3@`10@400AdM704U9B@10@400@d=304m?C`0[
:b/0:BTY04E5A@0m?Cd0C4a<04Q8B00a<C40@D5104e=C@0l?3`0>3Ph04e=C@1MGEd0<C4a0P18B4P0
>@1HF5P0?3`l02/[:`1=CDd0HF5Q03/k>`0n?Sh0;Bd]04I6AP1;Bd/0?Shn04a<C01_Kfl0KFe]06][
J`1OGel0De=C04I6AP1jNWX0QhN704M7A`16ATH0Hf=S0451@@1<C4`0LG5a06YZJP1`L700G5aL04m?
C`1]KFd0HV9R055AD@1TI6@0KFe]07IfMP11@D40MgMg04];B`16ATH0N7Qh07UiN@1IFET0EEEE03dm
?@1;Bd/0Jf][07]kN`1OGel0SHf=06IVIP1cLg<0H61P07AdM01eMGD0JVYZ0:NWY`0208B4Q00D06e]
K@0m?Cd0I6AT04m?C`1eMGD0DU9B05IFEP1[Jf/0JFUY05IFEP1BDU80QXJ604Q8B01[Jf/0MgMg08J6
QP14A4@0H61P079bLP19BDT206ATI00M04M7A`1NGUh0DE5A061PH01]KFd0Cdm?05EEE@1DE5@0Lg=c
05EEE@1LG5`0<c<c05EEE@11@D40IFEU05mOG`1:BTX0N7Qh04i>CP1DE5@0Hf=S071`L014A4@0GUiN
09BDU00f=SH0BTY:03dm?@2LW9`03_ooool08P0f=SH0:RXZ01XJ6P0l?3`0ATI605eMG@1UIFD08b<S
07MgM`2b/[80a/K606EUI@0c<c<07AdM01/K6`0l?3`0820P05YJFP0X:2P04a<C00l?3`0F5QH0K6a/
01XJ6P1lO7`0=cLg014A4@0C4a<051@D01LG5`0S8b<071`L06a/K00o?cl2018B4P0=01@D500E5AD0
4A4A010@400n?Sh07AdM03Lg=`0k>c/0[jn_02l_;`0N7Qh0;Bd]01HF5P0301DE5@0/03lo?`1MGEd0
Bd];06i^KP0^;Rh09BDU04];B`1SHf<0HV9R02<S8`0T92@08b<S069RHP0K6a/09bLW05YJFP0`<300
H61P02d];@0M7Ad07QhN07EeM@1;Bd/09BDU06i^KP1fMWH0A4A406MWI`1kNg/0N7Qh02TY:@1QHF40
h>3P06MWI`1KFe/0K6a/05aLG00^;Rh0J6QX065QH@1GEeL0WinO04U9B@1=CDd201lO7`0802@T901K
Fe/0OGem06UYJ@31`L40IfMW051@D01MGEd209JFUP0J06a/K00d=3@0FUYJ08R8R01RHV80C4a<02HV
9P0W9bL0ATI602@T901<C4`0;2`/07]kN`0L71`0;Bd]06m_K`1EEED0GemO03dm?@1OGel0@d=3055A
D@0N7Qh0?Cdm07UiN@2d];@203Ph>00404e=C@0V9RH0820P04I6AQCooooo000Cooooo`2;07alO01b
LW80GemO05mOG`1IFET0EEEE06=SH`1=CDd0IVIV069RHP13@d<0ADE506a/K00n?Sh0AdM705iNGP0n
?Sh0@d=304a<C01FEUH0?clo04m?C`1IFET0?clo055AD@1TI6@0E5AD0410@01PH600GEeM04E5A@1F
EUH0Hf=S03dm?@1[Jf/0Hf=S05=CD`1GEeL0DU9B04=3@`1?Cdl0CDe=051@D01ADE40Fe]K05MGE`0g
=cL0IFEU05ADE00X:2P0HV9R05aLG00f=SH0HF5Q05=CD`10@400IfMW04i>CP15ADD0GemO05UIF@1:
BTX0FEUI05mOG`1IFET0Cdm?075aL@1YJFT0CTi>069RHP1LG5`0Jf][05UIF@1YJFT0Bd];05]KF`1d
M7@0EUIF06QXJ01TI6@0D51@05mOG`1RHV80A4A405iNGP1jNWX0@41005YJFP1XJ6P0@d=304i>CP1H
F5P0D51@05=CD`1DE5@0H61P05UIF@1@D500GemO06MWI`1TI6@0GUiN06IVIP1eMGD0IfMW04U9B@1n
OWh0HV9R05]KF`1UIFD0GUiN06m_K`1RHV80I6AT06QXJ01DE5@0JVYZ06ATI013@d<0IFEU06ATI00g
=cL0Lg=c04e=C@1?Cdl0MgMg04a<C01ADE40HV9R04I6AP1IFET0JVYZ04I6AP1>CTh0F5QH04A4A01B
DU80JFUY03lo?`0>ooooo`03079bLP1JFUX0IFEU0080FEUI06@0HF5Q07UiN@2;Rh/0Kfm_04a<C013
@d<0C4a<065QH@1;Bd/0Hf=S05EEE@1[Jf/0HF5Q05=CD`19BDT0EUIF05=CD`1@D500EeMG051@D013
@d<0?Shn04i>CP0c<c<0@4100492@P1HF5P0OWin06IVIP1SHf<0GUiN05EEE@1<C4`0EUIF03lo?`11
@D40ATI604Q8B00h>3P0?Shn03Xj>P0o?cl0?Shn03Xj>P1:BTX0ADE5051@D01TI6@0F5QH05YJFP1V
IVH0EEEE06=SH`1TI6@0CDe=059BDP1WIfL0YZJV061PH01]KFd0HV9R08J6QP1NGUh0DU9B05ADE01Q
HF40MgMg09FEU@1VIVH0JVYZ061PH01IFET0F5QH07]kN`1KFe/0CTi>04];B`1?Cdl0DU9B04E5A@1U
IFD0DU9B04m?C`1MGEd0K6a/07UiN@15ADD0Cdm?05iNGP2HV9P0MWIf06=SH`13@d<0GUiN069RHP1C
De<0DU9B04I6AP1KFe/0EEEE0P1:BTX08@11@D40@T9205UIF@1KFe/0GemO059BDP1EEED0?Shn04M7
A`12@T80AdM704];B`1?Cdl0B4Q805ADE00g=cL0Hf=S05eMG@1NGUh0HV9R05]KF`1`L700JVYZ05]K
F`1EEED0DE5A04e=C@1NGUh0Fe]K07MgM`1NGUh0Hf=S07emO@0>ooooo`1V07]kN`1<C4`0GEeM05mO
G`1:BTX0Cdm?04M7A`0o?cl0ADE504A4A01;Bd/0@d=30492@P11@D408b<S02d];@1>CTh0<C4a061P
H017AdL0;bl_059BDP1GEeL0@4100492@P1YJFT0HF5Q03/k>`1_Kfl0Cdm?04Y:BP1>CTh0;Bd]055A
D@1fMWH0ADE504=3@`0f=SH0BDU905iNGP15ADD0De=C08>3P`21PH40Kfm_04m?C`1<C4`0EEEE07Mg
M`27QhL0@T9204m?C`1WIfL0A4A404i>CP1YJFT0KFe]0861P@1GEeL0E5AD071`L01IFET0D51@06YZ
JP1XJ6P0H61P03Ti>@25QHD0BTY:059BDP23Ph<0LW9b05aLG01`L700AdM7055AD@1RHV80QXJ605eM
G@1eMGD0QXJ6079bLP1oOgl0FEUI06m_K`1fMWH0PX:207]kN`1iNGT0D51@04M7A`1FEUH0DU9B07Ad
M017AdL0CTi>079bLP1;Bd/0De=C059BDP1lO7`0Cdm?0P1YJFT06P1hN7P0Bd];05aLG01QHF40F5QH
08R8R01DE5@0>CTi05UIF@1GEeL0G5aL071`L01EEED0EUIF06YZJP22PX80G5aL04m?C`13@d<0Bd];
03Lg=`1KFe/0EEEE05MGE`1SHf<0?Cdm0P1bLW80201eMGD0BDU905UIF@2KVi/0BDU904I6AP0h>3P0
QHF53_ooool08P1?Cdl0:2PX01/K6`0b<S80GUiN0;:b/P2DU9@0Lg=c0;Zj^P38b<P0LW9b04U9B@0e
=CD0ADE5055AD@1HF5P05QHF065QH@0i>CT06a/K02DU9@0N7Qh0X:2P09nOW`2/[:`0VIVI07]kN`1N
GUh0E5AD04i>CP23Ph<0JVYZ09fMW@0f=SH2018B4P0<01@D500B4Q80410@010@4011@D408b<S0451
@@1ZJVX0[Jf]03Lg=`0Z:RX0;Rh^0`0E5AD0C00T92@0Ph>305YJFP0S8b<0BTY:04U9B@12@T80EUIF
08Z:RP2^[Zh0DU9B05iNGP1IFET0P82004i>CP0V9RH08R8R03lo?`1=CDd0HF5Q06a/K01KFe/0NGUi
079bLP1EEED0GemO04U9B@0R8R80>CTi0820P01nOWh0Ogmo07EeM@2KVi/0Z:RX0:FUY@1kNg/0DE5A
059BDP2GUiL0S8b<03De=@2KVi/0DU9B038b<P0J6QX0:b/[04=3@`1_Kfl0[Zj^08B4Q02e]KD0`l?3
0:VYZ@1`L700K6a/05=CD`1XJ6P0CTi>0451@@1MGEd0UIFE08Z:RP27QhL0UYJF08F5Q@1IFET0A4A4
071`L01CDe<09bLW04A4A01VIVH08B4Q0492@P1UIFD204];B`0;0;^k^`2HV9P0S8b<07inOP1^KVh0
ADE503Ph>01fMWH0>c/k04E5A@1/K6`05?ooool001?ooooo00`0Lg=c06UYJ@1CDe<0MWIf04a<C01j
NWX0LG5a030`<01ZJVX0EEEE03dm?@1DE5@204e=C@0A04i>CP1OGel0IfMW05YJFP1CDe<0Jf][03`l
?01;Bd/0JVYZ02h^;P1?Cdl0LW9b03Hf=P15ADD0EeMG0410@01TI6@00`1>CTh09P1KFe/0G5aL06AT
I01?Cdl0KFe]04a<C00i>CT0KFe]04=3@`0h>3P0I6AT05=CD`0d=3@0FUYJ051@D00k>c/0MGEe03hn
?P16ATH0EEEE04e=C@1MGEd0De=C055AD@1FEUH0DE5A04e=C@1PH600BTY:04U9B@1cLg<0?Shn055A
D@1TI6@0;2`/06MWI`1LG5`0D51@0P1[Jf/07`1KFe/0O7al065QH@1MGEd0LW9b05YJFP1<C4`0K6a/
05aLG013@d<0MWIf069RHP1LG5`0JVYZ061PH01GEeL0ADE505EEE@1^KVh0<S8b05mOG`25QHD0<S8b
05IFEP1JFUX0AdM706MWI`0e=CD0GEeM07UiN@1ADE400P1/K6`0801ADE40Fe]K059BDP1mOGd0FUYJ
059BDP24Q8@0G5aL069RHP1nOWh0B4Q805QHF020P800EeMG04];B`1XJ6P0ADE505MGE`1WIfL0<c<c
059BDP1cLg<0;Bd]069RHP1_Kfl0>c/k069RHP1UIFD0:BTY05YJFP1HF5P0;Bd]3_ooool0;01`L700
FEUI06YZJP1QHF40M7Ad061PH020P800T92@071`L01<C4`0B4Q804Y:BP1YJFT0GemO07EeM@1FEUH0
LW9b061PH01LG5`0ATI604m?C`17AdL0Cdm?04e=C@1>CTh0A4A40451@@1GEeL0>SXj05UIF@19BDT0
GemO07IfMP1OGel0IfMW05eMG@17AdL0CDe=04Y:BP14A4@0?clo04=3@`0o?cl0?3`l0P0h>3P03P0b
<S80<c<c038b<P1<C4`0ATI604U9B@1bLW80DE5A05=CD`1ZJVX0GemO05ADE01MGEd0AdM70P1FEUH0
>027QhL0EeMG065QH@1HF5P0Q8B405iNGP10@400F5QH06MWI`1bLW80VYZJ07MgM`1ZJVX0LW9b06MW
I`1[Jf/0EeMG05eMG@1BDU80FUYJ059BDP1NGUh0Bd];05MGE`1@D500Fe]K05YJFP1ZJVX0LW9b04e=
C@1CDe<0RHV909>CT`1fMWH0DU9B04=3@`1EEED0HF5Q079bLP1;Bd/0?3`l055AD@19BDT0D51@05AD
E01:BTX0B4Q805YJFP1DE5@0I6AT04m?C`1=CDd0@41004];B`18B4P0BTY:0P18B4P0500o?cl0H61P
0451@@1/K6`0Cdm?04U9B@1`L700HF5Q08J6QP1kNg/0FUYJ06IVIP1IFET0EEEE04i>CP1cLg<0Ti>C
06YZJP1IFET0F5QH3_ooool0:`1aLG40Bd];059BDP1EEED0ADE504=3@`1YJFT0;bl_04e=C@0l?3`0
@d=305=CD`0];Bd0=cLg02DU9@0a<C40FEUI02TY:@15ADD0>3Ph020P801@D500@d=304I6AP0j>SX0
DE5A05eMG@0h>3P0M7Ad04Y:BP1ADE40?Shn030`<01IFET0G5aL04M7A`16ATH0=3@d061PH01GEeL0
CTi>05mOG`1nOWh00P1KFe/0?01=CDd0C4a<04Q8B01cLg<0RHV90410@016ATH0J6QX04=3@`1LG5`0
MWIf07QhN02>SXh0DU9B06EUI@1ZJVX0De=C05MGE`1ZJVX0N7Qh075aL@15ADD0O7al05=CD`1XJ6P0
RXZ:07YjNP1JFUX0G5aL055AD@1GEeL0JVYZ08R8R01CDe<0IFEU0861P@1YJFT0L71`06EUI@1kNg/0
Kfm_0861P@22PX80MgMg059BDP0_;bl0EUIF05aLG01_Kfl0DE5A06i^KP1aLG40FEUI05QHF01FEUH0
QHF505iNGP1aLG40J6QX07inOP80Bd];00d0L71`051@D01aLG40IFEU04A4A01JFUX0DU9B06a/K01f
MWH0@d=306EUI@1aLG40K6a/0080GUiN0180>3Ph059BDP13@d<0Hf=S05UIF@1EEED0MWIf03Ph>01Y
JFT0GEeM06a/K012@T80Hf=S0:j^[P0g=cL0BTY:04Q8B01jNWX>ooooo`0[07AdM011@D4092@T02d]
;@1@D500BTY:05]KF`1RHV80C4a<02l_;`22PX80P82006YZJP1MGEd0:BTY06e]K@1TI6@0[Jf]08j>
SP0V9RH0>SXj09RHV026QXH0/;2`04E5A@0b<S80:BTY02@T900W9bL0>SXj061PH01VIVH0/k>c08f=
S@0k>c/09BDU03`l?00e=CD04A4A018B4P1;Bd/0PH6109nOW`020<30`01O0820P017AdL0MgMg04m?
C`1;Bd/0F5QH07UiN@2HV9P0UYJF055AD@1<C4`071`L01LG5`0N7Qh0K6a/09>CT`2f][H0RXZ:09^K
V`0P8200?Cdm05YJFP1:BTX0FEUI09^KV`27QhL0G5aL09bLW01bLW80Xj>S08J6QP1TI6@0C4a<01LG
5`0P8200CTi>034a<@1cLg<0ATI608f=S@2IVIT0SXj>055AD@1JFUX0E5AD0<71`@1_Kfl0HF5Q08J6
QP14A4@08B4Q01TI6@0a<C40FEUI08R8R024Q8@0O7al0;Bd]03Cdm<0`l?307MgM`1VIVH0>c/k01lO
7`0G5aL07AdM079bLP26QXH0RXZ:04U9B@25QHD0]KFe06MWI`10@400V9RH04a<C00o?cl0GUiN05mO
G`0J6QX06a/K05EEE@1jNWX0Z:RX06EUI@0Z:RX07alO024Q8@1?Cdl0Kfm_08n?S`2GUiL0EEEE02d]
;@0V9RH05?ooool001?ooooo04`0MWIf05mOG`1/K6`0ATI605eMG@1;Bd/0H61P05iNGP1JFUX0Lg=c
04e=C@1<C4`0N7Qh04a<C01ADE40P820051@D01?Cdl0GEeM03Xj>P0m?Cd0CDe=04i>CP0c<c<0ADE5
04=3@`1PH600;Bd]04Q8B01hN7P0>CTi04];B`1aLG40:RXZ055AD@1oOgl0<C4a04m?C`1GEeL0BDU9
07MgM`0n?Sh0D51@04e=C@19BDT0GUiN04E5A@19BDT0NWYj03Lg=`16ATH0Hf=S03De=@18B4P0KFe]
03Ti>@1IFET0JFUY03Ti>@1>CTh0O7al0492@P1@D500LG5a0492@P1TI6@0H61P03@d=01=CDd0JVYZ
055AD@1HF5P0Bd];065QH@1_Kfl0?Shn0P1LG5`0?@1<C4`0IVIV05YJFP1KFe/0Ng]k05ADE01;Bd/0
QhN703hn?P1@D500Ng]k03Lg=`13@d<0IFEU05ADE01EEED0FEUI055AD@1KFe/0BDU905MGE`1kNg/0
B4Q807=cL`1=CDd0BDU9071`L011@D40QXJ606][J`0o?cl0L71`06YZJP1KFe/0NWYj07MgM`1UIFD0
Jf][07alO01WIfL0ATI606ATI01UIFD0ADE5065QH@1HF5P0Cdm?05iNGP1=CDd0A4A406MWI`17AdL0
@D5105iNGP1<C4`0CDe=03lo?`1GEeL0@T9205]KF`1HF5P03_ooool0501/K6`0BTY:06UYJ@1LG5`0
OWin05mOG`2=SHd0/k>c07QhN01GEeL0ATI604e=C@1WIfL0I6AT06a/K01]KFd0MgMg06i^KP1GEeL0
BTY:0P1=CDd05@1?Cdl0BTY:0410@015ADD0<C4a05]KF`0e=CD0B4Q803lo?`1IFET0Jf][05EEE@1Q
HF40GEeM04Q8B00n?Sh0De=C03/k>`0o?cl0CTi>03De=@0203hn?P0Y04I6AP12@T80AdM703`l?01F
EUH0CTi>04a<C020P800EEEE05]KF`1JFUX0Jf][05YJFP1QHF40C4a<04Y:BP1XJ6P0U9BD06ATI01A
DE40BDU907YjNP1=CDd0Cdm?06e]K@1KFe/0HV9R08Z:RP1<C4`0D51@05EEE@1GEeL0JFUY07QhN022
PX80F5QH06MWI`1FEUH0Bd];05EEE@1KFe/00P1BDU80101JFUX0MGEe07emO@1CDe<206EUI@0905=C
D`1IFET0De=C04=3@`11@D40CTi>06ATI01=CDd0>CTi0080AdM700L0ADE504=3@`0n?Sh0@d=306EU
I@1JFUX0LG5a0080Cdm?01X0>c/k04I6AP0m?Cd0ATI604Y:BP1EEED0>c/k05EEE@1=CDd0GUiN05YJ
FP1:BTX0OWin05eMG@20P800I6AT04a<C01VIVH0@D5103Hf=P10@400H61P06UYJ@1NGUh0GEeM06MW
I`kooooo08`0PH61051@D01RHV80De=C04a<C01SHf<0EUIF04A4A01UIFD0ADE504i>CP1BDU80AdM7
03dm?@0Q8B40B4Q8069RHP0l?3`0KFe]03hn?P0`<300GemO05EEE@1>CTh0?Shn06IVIP1EEED0>3Ph
06i^KP17AdL0A4A4059BDP0_;bl0GEeM06YZJP17AdL0BDU903Ti>@1?Cdl0F5QH04];B`1?Cdl0QhN7
069RHP1UIFD0CDe=03dm?@1HF5P0EUIF08f=S@1:BTX0De=C05aLG019BDT0De=C06i^KP1`L700IfMW
05EEE@1IFET0IFEU05]KF`1SHf<0JFUY06IVIP1EEED0?Cdm07UiN@15ADD0F5QH08:2PP1ZJVX0AdM7
05mOG`0m?Cd0BDU905MGE`1aLG40IfMW06e]K@1hN7P0KVi^08N7Q`1FEUH0Jf][07EeM@28R8P0Shn?
08V9R@1@D500<C4a04i>CP1;Bd/0HF5Q04a<C01?Cdl0M7Ad05eMG@1:BTX0I6AT0:2PX01cLg<0M7Ad
07]kN`2NWYh0?clo06=SH`1IFET0AdM706IVIP1TI6@0<S8b06QXJ01NGUh0FEUI06MWI`1HF5P0H61P
07IfMP20P800L71`04Q8B00h>3P0GUiN03Ph>01>CTh0?clo061PH01aLG40=cLg09JFUP12@T80J6QX
05EEE@1UIFD0Vi^K04I6AP1=CDd0?3`l08F5Q@kooooo00H0MgMg06a/K01VIVH0M7Ad04Q8B00S8b<2
01`L700A059BDP0X:2P0:2PX092@T03[jn/0i>CT0:BTY01oOgl0`/;20492@P0b<S80RHV90=SHf02i
^KT0C4a<069RHP18B4P00P0N7Qh0:@0J6QX05ADE01PH600j>SX0MGEe0:ZZZP2`/;00OWin075aL@2a
/K40Kfm_04=3@`14A4@0NWYj0:b/[03?cll0cLg=04Y:BP1TI6@0OWin09jNWP1RHV80Bd];04E5A@1e
MGD0Ph>305YJFP1RHV80TI6A028R8P0I6AT0;bl_065QH@1cLg<0BDU90:f][@1IFET0F5QH01hN7P0e
=CD0RXZ:0080][Jf04H0NGUi059BDP1NGUh0]KFe05aLG00_;bl0KFe]09>CT`1OGel0IFEU02<S8`0N
7Qh0IVIV04e=C@1@D500?Cdm04m?C`1aLG40EUIF0<_;b`2BTY80NWYj07inOP3:b/X0UYJF06a/K01j
NWX0Lg=c07QhN01IFET07alO02@T901SHf<0Xj>S07inOP2c/k<0RXZ:03lo?`0M7Ad06ATI04Y:BP1F
EUH0@D5103Hf=P1_Kfl0JVYZ0:VYZ@2j^[X0bLW90:ZZZP13@d<0MWIf06e]K@1/K6`0?Cdm01XJ6P0Z
:RX0Ogmo07YjNP0R8R807alO02PX:00N7Qh09RHV06ATI02@T900Ng]k01dM7@0Q8B40De=C5?ooool0
01?ooooo00@0IVIV06i^KP1EEED0JFUY0P1ADE40301kNg/0<C4a05IFEP1iNGT0:BTY05mOG`1QHF40
>CTi06EUI@1SHf<0BTY:04M7A`80Cdm?07L0?Shn055AD@1KFe/0>c/k051@D01QHF40>CTi03Ph>01Q
HF40BDU903hn?P1KFe/0AdM704A4A01OGel0EUIF05UIF@1@D500C4a<05IFEP16ATH0HF5Q05iNGP0b
<S80GemO04m?C`0P8200E5AD051@D00j>SX0JFUY03/k>`16ATH0E5AD04U9B@1KFe/0EUIF05ADE01@
D500ADE504i>CP1IFET0>SXj055AD@21PH40=SHf05IFEP1QHF40;bl_06QXJ01ZJVX0?clo071`L01i
NGT0DU9B07moO`1OGel0CDe=0820P01WIfL0@d=306YZJP1OGel0BTY:065QH@1YJFT0CDe=05ADE01H
F5P0GUiN03dm?@1ADE40NWYj0451@@1WIfL0N7Qh0451@@1]KFd0CDe=05IFEP1SHf<0=CDe06UYJ@1[
Jf/0BTY:07=cL`1lO7`0G5aL06m_K`1@D500HV9R06IVIP1;Bd/0LG5a061PH011@D40RXZ:051@D00i
>CT0Ti>C05IFEP15ADD0MGEe04Y:BP1BDU80S8b<03dm?@17AdL0RXZ:03Xj>P1DE5@0KFe]03lo?`1J
FUX0EUIF03<c<`1?Cdl0E5AD03Xj>P0>ooooo`04075aL@1GEeL0Hf=S05IFEP80HV9R00H0PX:206MW
I`1QHF40ADE503`l?01<C4`2065QH@0m07AdM01HF5P0I6AT05MGE`1?Cdl0?clo04e=C@1;Bd/0Cdm?
055AD@1:BTX0@41003Xj>P1KFe/0@41004a<C01@D500GemO07alO01CDe<0GemO05QHF010@400@T92
04];B`19BDT0?Cdm04I6AP0l?3`0?Cdm0410@015ADD0?clo0492@P1?Cdl0F5QH04M7A`1EEED0Lg=c
069RHP1NGUh0KVi^061PH01QHF40LW9b04A4A01ADE40FEUI09jNWP1GEeL0CTi>055AD@1kNg/0GEeM
05ADE01iNGT0Jf][06m_K`2>SXh0CTi>05ADE00204=3@`0O04a<C01VIVH0DU9B04M7A`1JFUX0AdM7
04m?C`16ATH0Cdm?04M7A`15ADD0EUIF06IVIP1UIFD0DU9B04M7A`1YJFT0CTi>05MGE`1;Bd/0@410
0492@P1GEeL0JVYZ061PH019BDT0Bd];04Q8B01?Cdl0FEUI04E5A@0204U9B@0N05eMG@1HF5P0EEEE
05ADE015ADD0FEUI04I6AP1<C4`0ATI605=CD`0h>3P0C4a<03De=@1@D500Fe]K05YJFP1VIVH0Hf=S
08J6QP1ZJVX0F5QH06=SH`1:BTX0ATI604=3@`19BDT0IVIV05YJFP1KFe/0Jf][3_ooool06@1QHF40
BDU904m?C`1KFe/0C4a<04M7A`1@D500?clo04=3@`11@D40AdM705]KF`17AdL0@T9203`l?00f=SH0
@d=302`/;01HF5P0?Cdm02`/;01DE5@0ADE5055AD@1?Cdl00P1KFe/02017AdL0H61P051@D01?Cdl0
ADE502DU9@1CDe<0CDe=0P18B4P05`0b<S80GEeM05aLG013@d<0JVYZ09JFUP1;Bd/0>CTi05IFEP1D
E5@0CDe=06e]K@2OWil0B4Q8051@D01IFET0Cdm?04e=C@1SHf<0GemO05UIF@1RHV80DE5A0080H61P
0P1[Jf/05@1ZJVX0B4Q804i>CP1kNg/0@T9205aLG01fMWH0JVYZ04Y:BP1QHF40?Cdm05eMG@1@D500
WYjN05MGE`1FEUH0S8b<07AdM01fMWH0KFe]08N7Q`0206MWI`0:08F5Q@22PX80Cdm?02/[:`13@d<0
KVi^04i>CP14A4@0JVYZ07emO@80Fe]K02T0CTi>07emO@1TI6@0MWIf05iNGP1/K6`0EeMG04I6AP1Y
JFT0@41006EUI@1BDU80<c<c05QHF019BDT0L71`07inOP11@D40HV9R07QhN01`L700J6QX07=cL`0j
>SX0B4Q802TY:@1nOWh0CDe=0451@@1eMGD0>CTi07=cL`1FEUH0Ng]k03@d=01XJ6P0UYJF020P801>
CTh0BDU9075aL@0>ooooo`0508j>SP2GUiL0ZJVY0:FUY@26QXH00P0i>CT05P15ADD0>3Ph0492@P18
B4P0R8R80=OGe`29RHT0;bl_02TY:@1BDU80@D5101DE5@1KFe/0c/k>0:j^[P1EEED0:b/[0820P01I
FET0E5AD04E5A@0/;2`202DU9@0603Xj>P2m_Kd0^KVi09FEU@1MGEd0CTi>0P1^KVh03P0e=CD0@T92
07emO@2c/k<0ZZZZ065QH@1eMGD0PX:2075aL@0O7al07AdM020P801/K6`0_Kfm0P2BTY80BP2_[jl0
IFEU061PH01WIfL0LW9b05eMG@0T92@0LG5a069RHP28R8P0Ogmo06IVIP1bLW80[Jf]0;Zj^P22PX80
I6AT04A4A021PH40SXj>04i>CP2?Shl0TI6A061PH01dM7@0?Shn03Hf=P1RHV80;2`/01LG5`0F5QH0
9RHV06UYJ@1hN7P0Bd];02/[:`1lO7`0UYJF06MWI`14A4@0;Rh^06EUI@2i^KT0Fe]K02PX:00U9BD0
ATI60861P@1dM7@0VYZJ0:f][@1[Jf/0=CDe04i>CP1/K6`0De=C03Ph>00K6a/05QHF038b<P1]KFd0
T92@09>CT`1dM7@0OWin0:n_[`1?Cdl08R8R02LW9`1]KFd0HF5Q06EUI@2VYZH0JVYZ0P0N7Qh02@0[
:b/07QhN02@T902DU9@0Jf][03Ti>@0n?Sh0<30`06a/K00Dooooo`004oooool0;P1jNWX0G5aL06EU
I@1CDe<0EUIF05MGE`1HF5P0EEEE061PH01KFe/0C4a<03hn?P1LG5`0DU9B04=3@`1QHF40L71`03Lg
=`1EEED0GemO030`<01MGEd0D51@03<c<`1MGEd0FEUI04];B`15ADD0DU9B071`L012@T80BTY:06=S
H`0g=cL0EEEE07UiN@18B4P0BTY:06i^KP0j>SX0BTY:05eMG@18B4P0A4A405aLG01?Cdl204=3@`07
05IFEP1;Bd/0FEUI051@D01?Cdl0?3`l05]KF`0204e=C@0?05UIF@17AdL0EUIF061PH01:BTX0Cdm?
06e]K@1:BTX0ADE5079bLP1@D500ATI606e]K@1KFe/0F5QH0080HF5Q0440IVIV06i^KP1LG5`0F5QH
04m?C`1TI6@0EeMG05QHF01WIfL0Fe]K04];B`1dM7@0De=C04a<C01OGel0Cdm?03dm?@1BDU80H61P
0451@@1KFe/0J6QX05=CD`1JFUX0G5aL06MWI`1FEUH0CTi>065QH@1WIfL0GEeM04U9B@1`L700GEeM
04e=C@1VIVH0GUiN06ATI01PH600PX:205mOG`1FEUH0T92@05IFEP18B4P0Ogmo05mOG`17AdL0NWYj
0451@@1BDU80HF5Q04e=C@19BDT0De=C055AD@1GEeL0KFe]04A4A01QHF40E5AD04Q8B01HF5P0DU9B
051@D00>ooooo`040861P@1BDU80IVIV05IFEP80J6QX01L0MgMg096AT@1`L700Bd];04m?C`1BDU80
F5QH069RHP1YJFT0I6AT05mOG`1NGUh0EUIF03Ti>@1@D500BTY:04m?C`16ATH0A4A403Lg=`0d=3@0
EUIF034a<@0204E5A@0Z051@D01jNWX0DE5A051@D01BDU80@d=304M7A`1<C4`0>c/k03lo?`12@T80
=CDe03Hf=P0h>3P0?3`l03Ph>00m?Cd0?clo04Y:BP10@400@T9206EUI@1<C4`0CDe=05=CD`1FEUH0
ATI6061PH016ATH0@T9205QHF02<S8`0JFUY04m?C`14A4@0IFEU04];B`10@400JVYZ051@D01SHf<0
SXj>0P1:BTX05@1TI6@0CTi>05aLG01dM7@0GEeM061PH01UIFD0@d=304e=C@19BDT0BTY:04a<C019
BDT0Fe]K075aL@1]KFd0CDe=04i>CP1CDe<0FEUI05QHF00203dm?@0B0492@P1CDe<0B4Q804m?C`12
@T80ADE50492@P1?Cdl0De=C03Ti>@15ADD0E5AD05EEE@1LG5`0B4Q8055AD@0o?cl0<c<c0P13@d<0
5P19BDT0EEEE04A4A015ADD0@D5104Y:BP1>CTh0Bd];065QH@1NGUh0IfMW06ATI01EEED0G5aL059B
DP19BDT0CDe=051@D01TI6@0E5AD061PH01TI6@>ooooo`2<07AdM01=CDd0DU9B05IFEP1=CDd0H61P
04];B`0n?Sh0ADE503Ti>@1>CTh0Bd];05YJFP0k>c/0>3Ph04Q8B012@T80;Rh^06UYJ@0g=cL0;2`/
03lo?`1GEeL0>c/k03dm?@1YJFT0FUYJ04=3@`1/K6`0BTY:03Ti>@1HF5P0=3@d05QHF01]KFd0D51@
05=CD`12@T80Hf=S05=CD`1BDU80GemO07inOP1`L700HV9R04Q8B00h>3P0GEeM06i^KP2=SHd0Cdm?
04M7A`1FEUH0?3`l04i>CP1SHf<0H61P069RHP1>CTh0E5AD075aL@1@D500MGEe05EEE@1VIVH0H61P
03Hf=P1mOGd0@T9205EEE@2=SHd0JVYZ04=3@`1OGel0AdM704Y:BP1EEED0RHV906][J`1=CDd0RXZ:
06EUI@2FUYH0C4a<07=cL`1gMgL0HV9R08V9R@1NGUh0EUIF03/k>`1@D500EEEE06MWI`1;Bd/0CTi>
07emO@1JFUX0CTi>05]KF`2<S8`0NWYj05mOG`1bLW80U9BD04Y:BP1SHf<0J6QX04e=C@1nOWh0G5aL
03Lg=`1/K6`0Fe]K06e]K@1^KVh0MGEe05aLG01_Kfl0Ph>306ATI01OGel0?Shn05IFEP13@d<0L71`
04m?C`1JFUX0KVi^03De=@20P800D51@09JFUP12@T80IfMW0861P@0^;Rh0@41004E5A@1bLW8>oooo
o`0H059BDP24Q8@0V9RH0:2PX02g]kL0KVi^03@d=00R8R809RHV02DU9@0c<c<0UIFE0<71`@23Ph<0
GUiN04=3@`0a<C40JFUY03De=@1[Jf/0YjNW0<?3``1:BTX0@D51100B4Q82010@4004018B4P0U9BD0
Cdm?0820P0806a/K01H09bLW02@T901HF5P0TY:B08b<S01>CTh071`L03Hf=P20P800?Cdm02XZ:P1O
Gel0Kfm_03Lg=`1iNGT0U9BD051@D01BDU80IFEU08b<S02VYZH0/[:b0P1KFe/05`1`L700H61P069R
HP2ZZZX0Shn?065QH@0`<300:RXZ09BDU02=SHd0LG5a06][J`16ATH0@d=3061PH01@D5009BDU05eM
G@1[Jf/0A4A408J6QP2`/;00BDU900@05ADE01T07QhN08>3P`2^[Zh0ZJVY0;jn_P2d];@0De=C04Q8
B013@d<0UiNG04A4A00m?Cd0:BTY065QH@15ADD05aLG02DU9@2f][H0a<C407IfMP1@D500HF5Q09>C
T`1;Bd/05QHF00805aLG01X0@41007MgM`1oOgl0MGEe07=cL`2VYZH0YJFU05]KF`1>CTh0ADE503Ph
>01FEUH0Vi^K0=3@d02SXj<0DE5A020P800h>3P0De=C08B4Q02d];@0P82006a/K02HV9P0S8b<055A
DACooooo000Cooooo`0507YjNP1VIVH0OWin04I6AP1YJFT00P1KFe/0J`1UIFD0BDU9069RHP12@T80
?Cdm06][J`1<C4`0DU9B05eMG@1BDU80D51@05=CD`0l?3`0Bd];05=CD`1EEED0E5AD04m?C`1SHf<0
HV9R030`<01EEED0JFUY02PX:01MGEd0Hf=S03Ph>01JFUX0C4a<04Q8B01OGel0BDU905aLG01:BTX0
@T92065QH@0j>SX0B4Q805mOG`0];Bd0CTi>065QH@0Z:RX0C4a<04Y:BP0U9BD0@D5105IFEP0[:b/0
GemO04Y:BP0a<C40H61P07AdM00e=CD0H61P07MgM`1>CTh0F5QH05aLG01MGEd0@D5105eMG@1LG5`0
I6AT05QHF01_Kfl0LG5a065QH@1eMGD0JFUY0451@@29RHT0FUYJ06ATI025QHD0Cdm?05mOG`26QXH0
ATI6059BDP1OGel0@D51055AD@1MGEd0I6AT061PH01SHf<0NGUi071`L00o?cl0IfMW07EeM@1?Cdl0
H61P051@D01^KVh0C4a<04E5A@23Ph<0De=C05aLG01`L700FEUI075aL@1oOgl0Q8B40080H61P01L0
OWin06YZJP13@d<0GemO06m_K`14A4@0GEeM069RHP1EEED0J6QX05UIF@0m?Cd0Kfm_05MGE`18B4P0
FEUI06m_K`0o?cl0Bd];06IVIP0k>c/0H61P04i>CP0>ooooo`1408F5Q@1NGUh0MWIf05ADE01=CDd0
CTi>05YJFP1lO7`0GEeM03lo?`17AdL0E5AD05=CD`15ADD0De=C05aLG01KFe/0IVIV04m?C`0m?Cd0
DE5A04Q8B01?Cdl0ATI604M7A`18B4P0=cLg05mOG`0Z:RX0ATI603dm?@13@d<0GEeM05UIF@1;Bd/0
CTi>0410@01=CDd0>3Ph03De=@13@d<0ADE502l_;`0k>c/0>SXj03dm?@12@T80?clo04i>CP1=CDd0
AdM704=3@`1TI6@0C4a<04A4A01?Cdl0B4Q804a<C01TI6@0EeMG0451@@1?Cdl0U9BD05aLG01UIFD0
GemO06MWI`1RHV8204E5A@0A03lo?`1YJFT0Ph>304Q8B019BDT0Lg=c0492@P1<C4`0GEeM05aLG01A
DE40KVi^0492@P1=CDd0EeMG05=CD`1BDU800P1:BTX0<`1dM7@0FEUI05UIF@1;Bd/0N7Qh07MgM`1M
GEd0C4a<0492@P11@D40GUiN05]KF`1FEUH0G5aL04=3@`10@400CTi>05UIF@0o?cl0Bd];065QH@1G
EeL0HF5Q04e=C@1>CTh0A4A404e=C@1<C4`0DE5A051@D01<C4`0AdM705ADE016ATH0=cLg04a<C01>
CTh0HF5Q06a/K01YJFT0L71`05]KF`1RHV80FEUI04M7A`1PH600L71`07inOP1GEeL0HV9R06][J`0>
ooooo`2<05eMG@1:BTX0HV9R065QH@1:BTX0AdM7061PH00f=SH0CDe=03`l?00g=cL0G5aL03`l?00j
>SX0@41002PX:01FEUH0=3@d05IFEP0n?Sh0:BTY06=SH`19BDT0@41005QHF019BDT0GEeM05EEE@1U
IFD0A4A404U9B@1JFUX0;2`/061PH01KFe/0@d=305MGE`0d=3@0Jf][051@D017AdL0N7Qh08F5Q@2<
S8`0DU9B051@D00h>3P0HF5Q07EeM@1mOGd0ADE504a<C01OGel0Cdm?055AD@22PX80KFe]06YZJP1I
FET0IfMW07AdM01BDU80MWIf05mOG`1cLg<0J6QX04I6AP26QXH0DE5A06=SH`2DU9@0IVIV04Y:BP1W
IfL0B4Q8065QH@1ZJVX0O7al06m_K`1HF5P0WinO061PH02MWId0FUYJ07MgM`20P800Fe]K07alO01W
IfL0EeMG0492@P1=CDd0HF5Q06=SH`1?Cdl0De=C07alO01PH600Bd];061PH020P800M7Ad07EeM@1Z
JVX0R8R80492@P1MGEd0M7Ad04m?C`1fMWH0De=C02d];@1OGel0G5aL061PH01OGel0FEUI075aL@1/
K6`0M7Ad05iNGP1JFUX0@D5104];B`0o?cl0JFUY04Y:BP0f=SH0LW9b04];B`21PH40F5QH08f=S@1D
E5@0Jf][06i^KP18B4P0<C4a04I6AP1^KVh>ooooo`0>034a<@1PH600NWYj03Ti>@1IFET0A4A400l?
3`0E5AD0410@014A4@0K6a/0H61P05QHF011@D4201XJ6P0903dm?@0k>c/0<30`055AD@2ATI40a<C4
05]KF`0O7al092@T00D03`l?00D03Ph>01@D500d=3@0LG5a01<C4`02010@400400h>3P0?3`l0De=C
05eMG@<04A4A03T0>SXj01XJ6P0g=cL0Jf][0:j^[P30`<00EEEE03hn?P0S8b<06QXJ03Ti>@1YJFT0
/;2`0;>c/`1?Cdl0>CTi05ADE01=CDd0Kfm_09JFUP2TY:@0NGUi03dm?@0[:b/0FEUI08V9R@1:BTX0
B4Q80492@P0F5QH07alO065QH@1KFe/0JFUY09RHV01EEED0Vi^K0;Ng]`16ATH07QhN01DE5@0D51@0
6QXJ03/k>`2?Shl0bLW906QXJ03=cLd0YjNW04=3@`0`<300;bl_061PH00j>SX0=CDe07IfMP0U9BD0
0P0E5AD02@0X:2P0YZJV0;Jf]P1`L700MGEe03Hf=P0X:2P0HV9R01lO7`0201DE5@0J03<c<`1VIVH0
N7Qh04=3@`0`<300MGEe0;:b/P2YZJT0GEeM07UiN@1ADE40De=C0:2PX02=SHd0Y:BT0<?3``2h^;P0
[Zj^07alO036a/H0[Zj^03Hf=P1KFe/0ZZZZ0<_;b`2@T90Dooooo`004oooool0R`1cLg<0JFUY06EU
I@1iNGT0AdM706e]K@1TI6@0=cLg05aLG01NGUh0>CTi051@D01LG5`0BDU903lo?`16ATH0KVi^04Y:
BP12@T80LG5a0451@@1CDe<0Lg=c04M7A`1JFUX0F5QH0492@P17AdL0L71`051@D015ADD0HF5Q04e=
C@1>CTh0GEeM06EUI@1IFET0G5aL059BDP1GEeL0@T92061PH01JFUX0>3Ph079bLP1CDe<0@d=304Q8
B01?Cdl0ADE505UIF@1=CDd0BTY:03dm?@14A4@0?clo0492@P1HF5P0JFUY03@d=01;Bd/0EEEE034a
<@1XJ6P0K6a/02TY:@1JFUX0EeMG03Hf=P1WIfL0De=C03Ti>@1UIFD0GemO04Y:BP1EEED0L71`04a<
C01HF5P0B4Q803`l?01cLg<0EEEE065QH@1BDU80EEEE06MWI`1>CTh0DE5A05eMG@15ADD0DE5A06QX
J00_;bl0J6QX07MgM`1<C4`0JFUY04E5A@1CDe<0H61P03Xj>P1RHV80M7Ad05UIF@1=CDd0EEEE05mO
G`1SHf<0A4A405QHF01PH600EeMG08J6QP1GEeL0BDU908^;R`1JFUX0CDe=06UYJ@1CDe<0EEEE071`
L018B4P0EeMG06YZJP1=CDd0Cdm?06=SH`0c<c<0Hf=S06i^KP0f=SH0IFEU05ADE00n?Sh0CTi>055A
D@0k>c/03_ooool03@22PX80DU9B06IVIP1@D500Hf=S04Q8B01VIVH0MGEe05UIF@0l?3`0ATI603dm
?@1:BTX00P1>CTh0?`1BDU80EeMG05EEE@1NGUh0?3`l04U9B@1=CDd0BTY:04M7A`18B4P0ADE503`l
?01QHF40<S8b04m?C`1BDU80EEEE06MWI`1KFe/0F5QH069RHP1<C4`0ATI604E5A@0b<S80AdM7051@
D00n?Sh0>c/k03hn?P15ADD0>SXj03lo?`18B4P0DU9B0410@017AdL0LG5a05QHF018B4P0D51@03hn
?P13@d<0CDe=04E5A@12@T80Hf=S0861P@1KFe/0Lg=c061PH01`L700C4a<055AD@1TI6@0EEEE061P
H01TI6@0DE5A05=CD`1KFe/0D51@04m?C`0205aLG00S05ADE01IFET0C4a<04M7A`1OGel0FEUI03dm
?@0h>3P0AdM705QHF01RHV80A4A405YJFP2BTY80F5QH03`l?01@D500EUIF061PH01]KFd0CDe=05AD
E017AdL0Cdm?05=CD`1KFe/0IFEU055AD@1HF5P0NGUi06ATI01fMWH0E5AD059BDP0k>c/00P16ATH0
1`1BDU80AdM704i>CP17AdL0ADE505ADE01EEED00P1>CTh03P1ADE40JVYZ06ATI01[Jf/0ATI605]K
F`16ATH0AdM70492@P1IFET0KFe]05UIF@1BDU80Lg=c3_ooool0S01SHf<0BDU905UIF@1ZJVX0BTY:
04];B`1ZJVX0>CTi04e=C@0k>c/0@D51051@D00j>SX0AdM703dm?@0b<S80E5AD034a<@1EEED0?3`l
038b<P1PH600GUiN03lo?`1>CTh0A4A406i^KP17AdL0G5aL04=3@`0g=cL0A4A402d];@1IFET0GUiN
04Q8B01?Cdl0>c/k07=cL`1NGUh0BTY:06YZJP1gMgL0NWYj05iNGP1GEeL0B4Q805YJFP1nOWh0QXJ6
04e=C@15ADD0De=C05ADE01:BTX0QXJ6069RHP1MGEd0FEUI055AD@1[Jf/0C4a<06i^KP1MGEd0EEEE
06ATI00h>3P0HV9R03lo?`1SHf<0P82006=SH`1LG5`0NGUi05EEE@1BDU80LG5a07inOP1QHF40C4a<
09JFUP1kNg/0V9RH06ATI02=SHd0KFe]065QH@2;Rh/0H61P05QHF00g=cL0BTY:06ATI01@D500C4a<
05EEE@25QHD0DU9B055AD@1LG5`0PX:208B4Q01fMWH0Ng]k0:2PX01JFUX0DU9B07MgM`1>CTh0Q8B4
05aLG00n?Sh0CTi>06=SH`1iNGT0GemO06][J`1^KVh0D51@06QXJ01SHf<0De=C03Lg=`1]KFd0DU9B
05ADE01:BTX0CTi>07EeM@0e=CD0T92@059BDP1SHf<0E5AD07=cL`1WIfL0@D5104m?C`0j>SX0MWIf
3_ooool01P0f=SH0I6AT09^KV`23Ph<0IFEU03Ph>0803@d=02h03Ph>014A4@0L71`0GEeM0820P00o
?cl051@D018B4P0S8b<051@D014A4@0L71`0Zj^[0:ZZZP1fMWH0G5aL03`l?00R8R803Ph>018B4P0I
6AT05ADE00l?3`0@4100D51@09bLW01<C4`071`L00l?3`0>3Ph03`l?03hn?P0o?cl04A4A00l?3`0@
41007alO030`<00B4Q80R8R80>;RhP2e]KD08R8R01LG5`0O7al08b<S0P0/;2`0C`1:BTX0M7Ad06QX
J01EEED0KVi^06=SH`1CDe<0PX:208j>SP1VIVH0=cLg01LG5`16ATH0F5QH08:2PP1<C4`0H61P02TY
:@0P82008R8R09>CT`2][Jd0IfMW06m_K`2YZJT0ADE506MWI`16ATH06QXJ02h^;P0d=3@0GUiN0820
P01nOWh0Bd];0;no_`2<S8`0?Cdm04a<C01EEED0WIfM07MgM`1:BTX0Ph>301HF5P0G5aL09BDU06QX
J02SXj<0SXj>02l_;`1CDe<0:b/[02@T9016ATH0De=C03Ph>00V9RH0BDU904Y:BP1@D500H61P051@
D02:RXX0SHf=0;no_`1mOGd0:b/[03/k>`1hN7P0<30`051@D02:RXX0`L710=;BdP3GemL0`l?30<g=
c@2ATI400P2?Shl00`2QXJ40cLg=07IfMP0Dooooo`004oooool0:P1jNWX0IFEU06i^KP1GEeL0FUYJ
059BDP1OGel0ATI605=CD`1KFe/0@D5104E5A@1]KFd0CDe=03lo?`1;Bd/0DU9B03@d=01KFe/0F5QH
03/k>`1DE5@0D51@03hn?P1HF5P0BDU904Q8B017AdL0CTi>05aLG00l?3`0D51@0861P@11@D40HV9R
04];B`0f=SH0BTY:05=CD`17AdL0D51@04];B`80CTi>00/0Cdm?05EEE@11@D40B4Q805eMG@1=CDd0
DE5A04U9B@1;Bd/0BDU904e=C@0204A4A01B05aLG01JFUX0@d=306UYJ@1GEeL0CTi>071`L01GEeL0
?Cdm075aL@18B4P0BTY:079bLP1DE5@0C4a<05aLG01>CTh0Hf=S06ATI01OGel0Hf=S05aLG01PH600
@T9206MWI`1TI6@0BDU904m?C`1`L700DU9B04I6AP1=CDd0DE5A03lo?`1OGel0GUiN03lo?`1RHV80
H61P05iNGP1@D500C4a<06YZJP1=CDd0Fe]K065QH@1UIFD0IfMW0451@@1QHF40F5QH059BDP1>CTh0
Fe]K06=SH`1@D500MgMg05aLG01DE5@0L71`06ATI01GEeL0IVIV05iNGP1;Bd/0GUiN05eMG@1IFET0
De=C055AD@1BDU80Fe]K05EEE@14A4@0CTi>05aLG01=CDd0ATI605=CD`12@T80CTi>05mOG`kooooo
01D0Rh^;05EEE@1]KFd0EEEE05aLG01GEeL0L71`09fMW@1YJFT0A4A404U9B@1<C4`0Cdm?061PH01K
Fe/0FEUI05]KF`1PH600EeMG03hn?P1?Cdl00P18B4P0<P1?Cdl0BTY:04=3@`0i>CT0EUIF02/[:`1<
C4`0D51@05ADE01/K6`0DU9B05IFEP1DE5@0ADE504I6AP1:BTX0>CTi04I6AP1?Cdl0?Cdm03/k>`0j
>SX0A4A403/k>`0h>3P0EEEE055AD@1=CDd0D51@06IVIP1DE5@0Cdm?05IFEP1KFe/0C4a<05ADE014
A4@0D51@06a/K02IVIT0C4a<07=cL`1RHV80Hf=S05]KF`1;Bd/0C4a<03Ti>@1EEED0Kfm_0P1>CTh0
@@1IFET0?clo05UIF@1^KVh0J6QX04i>CP1QHF40A4A405YJFP1BDU80G5aL04e=C@1:BTX0E5AD06EU
I@1QHF40E5AD055AD@1gMgL0HV9R04E5A@18B4P0AdM705EEE@1ZJVX0EeMG05UIF@1QHF40BTY:04U9
B@1BDU80EEEE0451@@1<C4`0JFUY05UIF@1cLg<0DE5A04e=C@0n?Sh0ADE50492@P1FEUH0@d=304M7
A`1;Bd/0De=C05UIF@15ADD0BTY:051@D01BDU80KVi^06e]K@21PH40De=C05EEE@14A4@0@d=304U9
B@1MGEd0KVi^065QH@1GEeL0K6a/00kooooo0380NGUi04U9B@1[Jf/0Jf][05QHF01TI6@0FUYJ03Xj
>P0m?Cd0>3Ph0410@01?Cdl0>3Ph0492@P0a<C40?3`l055AD@0g=cL0IFEU03<c<`13@d<0EUIF07=c
L`0o?cl0E5AD03<c<`1bLW80@D51071`L01:BTX0?3`l04i>CP18B4P0JVYZ07IfMP1?Cdl0CDe=03lo
?`1fMWH0G5aL051@D021PH40MGEe06m_K`1/K6`0EEEE030`<01MGEd0=cLg07]kN`80E5AD05P0EUIF
05EEE@1FEUH0RHV906ATI01NGUh0F5QH06EUI@1nOWh0E5AD08F5Q@1[Jf/0J6QX06=SH`1BDU80O7al
04E5A@1XJ6P0QXJ606UYJ@1JFUX0LW9b05MGE`1JFUX0N7Qh0861P@1YJFT0DU9B0:BTY01`L700UIFE
05IFEP20P800Lg=c06m_K`2DU9@0DU9B05UIF@17AdL0Fe]K06][J`1JFUX0BTY:06EUI@2<S8`0Jf][
05IFEP1PH600NWYj06IVIP1iNGT0MgMg08b<S01NGUh0EUIF07IfMP1ADE40MGEe051@D00i>CT0IfMW
05UIF@1mOGd0IVIV069RHP1VIVH0Kfm_06QXJ01NGUh0HV9R0492@P1GEeL0ATI605mOG`1BDU80A4A4
071`L00n?Sh0PH6104i>CP1QHF40JFUY08Z:RP1mOGd0ADE50492@P0k>c/0L71`3_ooool0?P0h>3P0
9RHV03hn?P2=SHd0UiNG05MGE`0C4a<06a/K02h^;P10@400D51@08B4Q01QHF40Fe]K04=3@`1MGEd0
=SHf014A4@0B4Q80=CDe028R8P1^KVh0@41005YJFP1KFe/0C4a<03hn?P0d=3@08b<S02`/;01HF5P0
GUiN05UIF@1bLW80EeMG03dm?@0a<C40:2PX02DU9@12@T80;Bd]01HF5P0@41003`l?02`/;01`L700
BTY:09>CT`37alL0RHV905=CD`0V9RH06a/K03@d=00T92@0=CDe07QhN01EEED0J6QX03Hf=P1MGEd0
Hf=S0P1iNGT0C00^;Rh0ATI606MWI`1TI6@0Kfm_07EeM@1PH600Jf][09bLW01mOGd0IFEU06=SH`2Q
XJ40^KVi06a/K00k>c/0FEUI07emO@1MGEd0IFEU0861P@1?Cdl0MWIf06EUI@1@D500BTY:0:j^[P10
@400C4a<01DE5@0I6AT092@T06=SH`1XJ6P0Ogmo038b<P1UIFD0L71`06EUI@2SXj<0T92@07moO`1F
EUH0IVIV07EeM@1PH600GemO07=cL`1jNWX0WIfM07AdM01aLG40PH6102h^;P0O7al0F5QH08n?S`2O
Wil0PH61079bLP27QhL0:2PX02HV9P0o?cl0TY:B0:JVYP2f][H0YJFU06e]K@16ATH0Jf][0861P@1Z
JVX0/[:b07EeM@14A4@Dooooo`004oooool03P1fMWH0I6AT06UYJ@1CDe<0KFe]05mOG`1oOgl0EEEE
04];B`2DU9@0@T9203<c<`1lO7`0Bd];0P1MGEd0N`15ADD0CTi>06=SH`1>CTh0IVIV04i>CP1CDe<0
HV9R03Lg=`19BDT0Hf=S02HV9P1=CDd0EUIF03<c<`1]KFd0FUYJ03@d=01nOWh0?3`l05QHF01EEED0
BTY:05eMG@1;Bd/0CDe=05aLG00h>3P0DU9B06e]K@0/;2`0De=C065QH@0c<c<0HF5Q04];B`0n?Sh0
JFUY04M7A`0o?cl0JFUY055AD@0o?cl0EEEE06e]K@12@T80EUIF06IVIP1QHF40>CTi05MGE`1OGel0
=3@d05eMG@1`L700EEEE04Y:BP1/K6`0DE5A051@D01oOgl0@410065QH@1iNGT0=CDe059BDP2IVIT0
@41006ATI02;Rh/0?clo05UIF@1GEeL0ATI604];B`1GEeL0HV9R065QH@1PH600KFe]05=CD`18B4P0
De=C05QHF01QHF40EEEE05YJFP1nOWh0HF5Q05EEE@1gMgL0FEUI06MWI`1EEED0G5aL08:2PP17AdL0
KFe]07MgM`1;Bd/0L71`06=SH`11@D40EEEE07AdM00e=CD0EeMG071`L011@D40E5AD075aL@0o?cl0
Fe]K059BDP0m?Cd0JFUY04e=C@15ADD0FUYJ0492@P11@D40K6a/03Ph>00>ooooo`0909BDU01LG5`0
KVi^059BDP1FEUH0GEeM0861P@1RHV80HF5Q0080?3`l06P0BTY:04=3@`1^KVh0Hf=S05iNGP1XJ6P0
JVYZ05eMG@11@D40Cdm?05ADE01;Bd/0DE5A04i>CP11@D40>3Ph05iNGP0Q8B40C4a<0451@@17AdL0
D51@04e=C@18B4P0BTY:04I6AP10@400CDe=03dm?@15ADD0D51@03/k>`0e=CD0<C4a03Ti>@0e=CD0
=SHf04Q8B00o?cl0?Shn0410@01MGEd0B4Q804I6AP1:BTX0J6QX055AD@1DE5@0BDU9055AD@1MGEd0
WIfM04Q8B01kNg/0ADE505eMG@1HF5P0CTi>07UiN@1PH600OGem096AT@1ADE40B4Q8061PH01;Bd/0
GUiN07alO01OGel0?clo06][J`1:BTX0E5AD05QHF01YJFT0B4Q805EEE@1GEeL0Jf][05aLG01HF5P0
De=C08F5Q@1MGEd0>CTi05EEE@19BDT0GEeM06IVIP1>CTh0De=C04m?C`14A4@0BTY:04U9B@17AdL0
<c<c03lo?`1FEUH0EeMG05=CD`1=CDd0?Cdm034a<@80AdM700L0EUIF04Q8B01OGel0E5AD04I6AP1=
CDd0BTY:0080BDU900h0F5QH05EEE@1eMGD0JFUY05ADE01=CDd0@D5104M7A`1:BTX0H61P06e]K@19
BDT0D51@06EUI@kooooo02<0IFEU04Y:BP1PH600Kfm_05ADE01FEUH0Hf=S04M7A`1ADE40AdM704U9
B@1]KFd0=CDe02/[:`13@d<0=3@d05EEE@0f=SH0EeMG03@d=014A4@0HF5Q06][J`1BDU80BTY:0451
@@1SHf<0?Shn06UYJ@11@D40ATI604A4A00m?Cd0GemO071`L00204E5A@1@03Lg=`1dM7@0DE5A05AD
E01fMWH0LW9b08N7Q`1?Cdl0G5aL055AD@1TI6@0>CTi08V9R@22PX80F5QH04Y:BP19BDT0EeMG075a
L@1OGel0FUYJ05ADE01UIFD0KFe]059BDP1]KFd0JFUY06m_K`1:BTX0EeMG07YjNP19BDT0De=C08j>
SP1gMgL0?Shn071`L01EEED0HV9R06IVIP2NWYh0IVIV071`L02XZ:P0I6AT09NGU`1[Jf/0S8b<05iN
GP1QHF40UYJF0492@P1PH600?Cdm05IFEP1`L700KVi^04Q8B01UIFD0MGEe08B4Q010@400De=C08B4
Q01ZJVX0KVi^07MgM`2MWId0DE5A04e=C@26QXH0@T9206m_K`1JFUX0=cLg05=CD`1HF5P0MGEe06=S
H`1MGEd206ATI00E061PH01dM7@0D51@03Xj>P19BDT0B4Q806a/K01KFe/0DE5A07MgM`1;Bd/0MGEe
04];B`1fMWH0ATI608:2PP2@T900ATI603Hf=P1:BTX0N7Qh00kooooo08`0<S8b0492@P1:BTX0Cdm?
08R8R01hN7P0QhN705eMG@0R8R807alO01XJ6P0U9BD0E5AD06YZJP0d=3@0?clo051@D01GEeL0BTY:
02h^;P0F5QH0>SXj014A4@0@4100:RXZ05iNGP1XJ6P0LW9b09NGU`2QXJ40KFe]04e=C@1PH600LG5a
0451@@1GEeL0B4Q804I6AP0i>CT0IfMW02PX:00d=3@05aLG02@T9017AdL0NGUi07emO@1UIFD0QhN7
04E5A@1CDe<0Kfm_08b<S02?Shl0FUYJ079bLP1JFUX0GUiN030`<00b<S80H61P05IFEP0f=SH0MgMg
03/k>`0L71`0Kfm_08b<S00b<S80D51@0;Ng]`1^KVh0HF5Q055AD@0X:2P0BDU905MGE`1CDe<0EEEE
03@d=00/;2`0B4Q809NGU`2?Shl0Hf=S09VIV@1OGel0<30`01TI6@0F5QH0DU9B05QHF016ATH0:2PX
02/[:`0Q8B40;bl_04I6AP13@d<0@T9208>3P`2=SHd0?clo06][J`2e]KD0Ng]k01dM7@0L71`0820P
030`<01HF5P0L71`08R8R02`/;00[:b/08N7Q`1ADE40Kfm_08F5Q@2c/k<0`/;207MgM`1eMGD0GUiN
075aL@1XJ6P0IFEU03<c<`28R8P0YZJV0;jn_P1cLg<0P820061PH01ADE40Q8B40;^k^`38b<P0B4Q8
03<c<aCooooo000Cooooo`1Y07inOP1^KVh0K6a/092@T01>CTh0HV9R06i^KP13@d<0F5QH05=CD`1=
CDd0?clo04a<C01cLg<0>CTi0492@P26QXH0?Cdm0451@@1TI6@0<30`05MGE`1mOGd0?Cdm05iNGP1I
FET0=3@d04Q8B01<C4`0BTY:05=CD`1PH600GEeM04m?C`1JFUX0MWIf05eMG@0k>c/0Hf=S069RHP18
B4P0Kfm_04a<C01;Bd/0Hf=S05EEE@1;Bd/0B4Q805MGE`1>CTh0Bd];04A4A01ZJVX0;Rh^04m?C`1/
K6`09BDU04Y:BP1[Jf/0;Bd]05EEE@1lO7`0=CDe075aL@1DE5@0<C4a05aLG01?Cdl0;Rh^065QH@1E
EED0ATI605YJFP18B4P0DU9B05=CD`1ADE40GEeM059BDP1>CTh0DU9B055AD@1=CDd0IVIV03Ti>@16
ATH0KFe]04];B`1JFUX0KFe]02d];@1@D500Hf=S03Xj>P1SHf<0GEeM04i>CP1XJ6P0?clo07YjNP1?
Cdl0IfMW07emO@1NGUh0Jf][0080EEEE01H0H61P05EEE@0o?cl0Jf][06a/K01GEeL0VYZJ04];B`1I
FET0P82004a<C01[Jf/0M7Ad059BDP1bLW80LG5a04Y:BP1ZJVX0?clo04a<C01VIVH0EEEE0P1DE5@0
200b<S80DU9B05mOG`1<C4`0IFEU05ADE01NGUh0JVYZ3_ooool0:P1mOGd0Bd];06YZJP1TI6@0DU9B
06][J`23Ph<0NWYj065QH@11@D40?Cdm04Y:BP1>CTh0J6QX05=CD`1;Bd/0GemO06IVIP1IFET0@T92
04a<C012@T80BTY:059BDP14A4@0B4Q803Ti>@1NGUh0:BTY04];B`1@D500Cdm?06MWI`19BDT0CDe=
05ADE011@D40>c/k05QHF01<C4`0@T9204U9B@80>3Ph05T0=3@d03/k>`0h>3P0>CTi04Y:BP15ADD0
=SHf0451@@1CDe<0CTi>04Y:BP1>CTh0H61P04I6AP1;Bd/0B4Q803`l?01FEUH0IfMW04M7A`1CDe<0
DU9B04E5A@1=CDd0ATI605]KF`1DE5@0F5QH06e]K@1>CTh0>c/k05MGE`0n?Sh0De=C05mOG`1cLg<0
@d=306i^KP17AdL0FEUI05YJFP1RHV80E5AD059BDP14A4@0IVIV05YJFP1MGEd0Cdm?07QhN016ATH0
?3`l04Y:BP1=CDd0E5AD069RHP1FEUH0K6a/05EEE@16ATH0Cdm?06][J`1GEeL0ADE505QHF01aLG40
KVi^06][J`1PH600EUIF05YJFP18B4P0EUIF05]KF`1:BTX0EeMG04m?C`1=CDd0K6a/04e=C@17AdL0
FUYJ06i^KP1CDe<0Lg=c06UYJ@1KFe/0Cdm?04e=C@0204i>CP0506ATI01WIfL0EUIF069RHP1lO7`0
3_ooool01`21PH40@d=305eMG@1JFUX0?Shn05eMG@0i>CT00P0m?Cd0;014A4@0Bd];04i>CP10@400
9bLW02XZ:P0n?Sh0@T92028R8P1XJ6P0=cLg04I6AP10@400HF5Q04A4A00d=3@0CTi>05eMG@0X:2P0
Kfm_0492@P0g=cL0BTY:03Lg=`1DE5@0LW9b04E5A@1CDe<0@d=306][J`1JFUX0G5aL06a/K01RHV80
NGUi04e=C@1JFUX0=SHf06YZJP0m?Cd0NWYj07QhN01DE5@0AdM70P1CDe<0E@1oOgl0OWin05]KF`1:
BTX0IFEU06UYJ@1PH600NGUi07AdM01WIfL0JVYZ04Q8B024Q8@0?Cdm05EEE@1^KVh0IfMW04=3@`1X
J6P0E5AD05IFEP1RHV80PX:207AdM01JFUX0SHf=06YZJP2?Shl0EUIF07YjNP1gMgL0JVYZ09:BTP1>
CTh0I6AT03lo?`1=CDd0Jf][065QH@0k>c/0H61P07]kN`1bLW80F5QH05MGE`1gMgL0L71`08>3P`1e
MGD0QXJ605ADE01iNGT0PH61055AD@29RHT0EeMG030`<01XJ6P0Jf][069RHP1eMGD0GemO061PH01/
K6`0Kfm_07AdM01VIVH0C4a<05=CD`10@400LG5a05mOG`1PH600HV9R02l_;`1lO7`0De=C051@D01=
CDd0RXZ:092@T01;Bd/0=SHf0492@P1]KFd03_ooool0E`1SHf<0@D5102`/;00L71`0820P05]KF`2P
X:00MgMg02LW9`0c<c<0:BTY03hn?P1GEeL0@D5104E5A@1GEeL0=SHf08J6QP2;Rh/05aLG01@D500f
=SH06a/K00l?3`0@4100;2`/08f=S@1LG5`0Ti>C07MgM`0I6AT061PH04M7A`0U9BD09RHV02DU9@14
A4@0?clo02XZ:P0n?Sh0V9RH09nOW`1JFUX0A4A405=CD`2@T900IFEU04A4A01?Cdl0820P038b<P1N
GUh0J6QX0;6a/@2QXJ40B4Q805QHF00l?3`0:2PX01lO7`0j>SX0NGUi04m?C`0b<S80>CTi065QH@2J
VYX0R8R806QXJ018B4P0S8b<06e]K@1FEUH0<c<c02TY:@0g=cL0D51@05=CD`2e]KD0NWYj06QXJ01H
F5P0E5AD06m_K`2UYJD0MWIf01HF5P0301DE5@0b03/k>`1BDU80JFUY03De=@0];Bd09RHV01XJ6P0T
92@0>c/k07emO@2>SXh0[:b/05QHF01aLG40/K6a061PH015ADD092@T04];B`0n?Sh09bLW02<S8`1I
FET0YjNW0<K6aP2e]KD0Jf][05aLG01aLG40GEeM07]kN`2IVIT0UYJF09nOW`2ZZZX0/k>c08V9R@2@
T900I6AT06m_K`1iNGT0P8200;Ng]`1RHV80CDe=06e]K@3Bd]80cLg=09JFUP1eMGDDooooo`004ooo
ool0502=SHd0KVi^079bLP1;Bd/0MgMg04U9B@1PH600GUiN0492@P1[Jf/0AdM703`l?01lO7`0ADE5
059BDP1=CDd0BDU905MGE`1;Bd/0@d=30P1BDU800`1CDe<0GUiN04E5A@0205]KF`0=028R8P1DE5@0
KVi^02@T901XJ6P0JFUY03Hf=P1oOgl0EeMG04a<C01ADE40Bd];069RHP0204m?C`0j06ATI00e=CD0
A4A408N7Q`0^;Rh0Bd];08f=S@0g=cL0BTY:04m?C`0n?Sh0Bd];061PH014A4@0C4a<04Y:BP1JFUX0
DU9B05YJFP0h>3P0EUIF069RHP18B4P0ADE505UIF@1LG5`0CDe=05YJFP1HF5P0Hf=S04I6AP1fMWH0
Jf][04U9B@1^KVh0EUIF051@D023Ph<0BDU905EEE@1fMWH0>CTi04Y:BP1^KVh0>SXj05MGE`1IFET0
AdM704e=C@1MGEd0EEEE05IFEP1GEeL0GUiN061PH017AdL0G5aL06e]K@80EUIF02D0@41007emO@1T
I6@0CTi>06=SH`1ADE40MgMg04E5A@1RHV80U9BD04m?C`1ZJVX0Ng]k04M7A`1_Kfl0L71`04E5A@1L
G5`0Ogmo03/k>`1OGel0HV9R0410@01LG5`0KVi^04U9B@1WIfL0KFe]04=3@`1_Kfl0FEUI03Hf=P1A
DE40Cdm?02TY:@1YJFT0DE5A00kooooo01P0QHF504a<C01NGUh0C4a<05IFEP1WIfL0OGem08:2PP1P
H600@T9204I6AP19BDT0@41006ATI01ZJVX0EeMG05YJFP1VIVH0EEEE04=3@`16ATH0A4A404U9B@1>
CTh204E5A@1I03Ti>@1FEUH0<30`04e=C@1MGEd0De=C06IVIP1;Bd/0BTY:04e=C@16ATH0?Shn055A
D@1;Bd/0@T9204Y:BP12@T80>3Ph03hn?P10@400@T9203Xj>P1DE5@0DE5A0410@019BDT0G5aL05]K
F`1:BTX0HF5Q051@D01JFUX0FEUI04E5A@18B4P0C4a<08N7Q`13@d<0M7Ad06MWI`1=CDd0GemO059B
DP16ATH0@T92059BDP21PH40@D5104I6AP1HF5P0ADE504Y:BP1YJFT0B4Q804i>CP1@D500BDU904Q8
B01ADE40HF5Q04Q8B01IFET0De=C06e]K@1KFe/0Bd];05eMG@2<S8`0EeMG03Hf=P14A4@0B4Q804I6
AP1DE5@0D51@06ATI01ADE40?clo04Q8B01ADE40CDe=0410@01ADE40GEeM065QH@1?Cdl0DE5A04M7
A`10@4000P19BDT02@1HF5P0HF5Q05=CD`1;Bd/0BDU906UYJ@13@d<0D51@05ADE00204Y:BP0<061P
H01WIfL0ATI604m?C`0n?Sh0>CTi03lo?`1MGEd0J6QX05MGE`1EEED0Hf=S3_ooool06@1eMGD0CDe=
05YJFP1ZJVX0DU9B05YJFP1GEeL0>CTi04I6AP0d=3@0F5QH04i>CP0e=CD0;Rh^03dm?@13@d<0@D51
02`/;01UIFD0@D5103Ti>@1GEeL0I6AT05=CD`10@4000P1GEeL07`13@d<0EeMG04A4A016ATH0C4a<
03Lg=`1WIfL0Fe]K059BDP1;Bd/0?Shn06e]K@1UIFD0DE5A079bLP1SHf<0KVi^06ATI01MGEd0=cLg
05QHF016ATH0M7Ad04Y:BP1=CDd0AdM704Y:BP19BDT0MGEe06IVIP1GEeL00P1IFET0801hN7P0H61P
07IfMP1aLG40GUiN06MWI`18B4P0PX:204m?C`1ADE40RXZ:07=cL`13@d<0MWIf05EEE@1>CTh0GUiN
08R8R01bLW80MgMg09BDU01RHV80S8b<05iNGP2=SHd0IFEU05UIF@2IVIT0GemO06MWI`0i>CT0Fe]K
0P1XJ6P0:`12@T80GUiN07MgM`1XJ6P0CDe=05]KF`2JVYX0MWIf06i^KP1]KFd0UiNG05UIF@1WIfL0
T92@04M7A`1jNWX0GEeM04Q8B01XJ6P0G5aL071`L026QXH0E5AD06UYJ@1XJ6P0E5AD08V9R@1jNWX0
@41005YJFP15ADD0GEeM055AD@1PH600P82005IFEP1XJ6P0MgMg06IVIP10@400Ogmo0;^k^`1=CDd0
0P1ADE400`1dM7Coooooooooo`0<ooooo`0;065QH@0/;2`071`L01HF5P0D51@0ADE509JFUP1XJ6P0
;2`/01dM7@0E5AD00P0L71`0C`0[:b/0;Bd]03lo?`1/K6`0_Kfm0<C4a01LG5`0Bd];05ADE019BDT0
>CTi034a<@1XJ6P07QhN04];B`1nOWh0L71`075aL@1oOgl05ADE00d=3@1ADE4071`L014A4@0S8b<0
JVYZ0:^[Z`2FUYH0N7Qh07MgM`0b<S80HF5Q04a<C01ZJVX0=CDe05IFEP0[:b/092@T02h^;P27QhL0
/k>c09bLW01>CTh07QhN02<S8`0h>3P0<S8b04=3@`1VIVH0BTY:01`L700V9RH0Ng]k0:^[Z`1lO7`0
X:2P09VIV@2CTi<0LG5a06EUI@1:BTX0BDU907inOP2;Rh/0Hf=S07moO`19BDT0<C4a02l_;`0^;Rh0
@41006ATI01_Kfl07AdM01@D500E5AD07QhN051@D01LG5`00P1XJ6P0;P1PH600I6AT05EEE@0d=3@0
Cdm?09FEU@2c/k<0h^;R079bLP1TI6@0GUiN06IVIP1QHF40XZ:R051@D00M7Ad09BDU03Lg=`1<C4`0
P82009VIV@2CTi<0HV9R04U9B@1?Cdl0<S8b03`l?01/K6`0CDe=04U9B@1UIFD0M7Ad075aL@1eMGD0
Jf][0861P@1@D500H61P07QhN01iNGT0]kNg0:VYZ@2SXj<0U9BD0;Fe]@1[Jf/Dooooo`004oooool0
3028R8P0Kfm_07IfMP1eMGD0HV9R05MGE`1fMWH0Bd];05MGE`1_Kfl0AdM704E5A@80GemO02@0CTi>
03`l?01TI6@0ATI604m?C`1HF5P0Bd];05EEE@28R8P0DU9B04I6AP1LG5`0GEeM03De=@1HF5P0Fe]K
03lo?`1IFET0Hf=S05ADE01HF5P0CTi>06ATI00o?cl0F5QH069RHP14A4@0G5aL06IVIP14A4@0DE5A
06e]K@10@400Cdm?06][J`0j>SX204Y:BP0a069RHP19BDT0AdM706a/K016ATH0C4a<06QXJ015ADD0
JFUY059BDP0l?3`0IfMW04U9B@0l?3`0FEUI04m?C`10@400H61P069RHP1<C4`0GUiN06m_K`1CDe<0
Fe]K06ATI01;Bd/0B4Q807EeM@10@400F5QH07EeM@18B4P0E5AD065QH@1DE5@0GemO06IVIP1/K6`0
C4a<05YJFP1oOgl0DU9B05UIF@1oOgl0EUIF069RHP1NGUh0JVYZ06][J`0204m?C`0T08F5Q@1PH600
Jf][05MGE`1VIVH0H61P04=3@`1[Jf/0NWYj05ADE01kNg/0I6AT059BDP1jNWX0IVIV069RHP1IFET0
Lg=c05EEE@1WIfL0IFEU05eMG@1BDU80EEEE05UIF@1EEED0FEUI04U9B@1=CDd0G5aL04];B`1LG5`0
DU9B04Y:BP1FEUH0EeMG3_ooool02`1XJ6P0E5AD05MGE`1:BTX0D51@05IFEP1dM7@0FUYJ065QH@0f
=SH0=cLg0080Bd];06/0GemO06MWI`1DE5@0De=C06ATI01[Jf/0>SXj04E5A@17AdL0B4Q804M7A`0l
?3`0ADE5038b<P1@D500;Rh^0492@P1;Bd/0De=C06IVIP1@D500CDe=05eMG@1=CDd0@41004a<C018
B4P0@T9204E5A@0m?Cd0=CDe03Hf=P0h>3P0@D5103<c<`18B4P0BTY:03Xj>P0n?Sh0Fe]K055AD@16
ATH0GUiN04E5A@1?Cdl0CDe=04I6AP18B4P0IFEU08Z:RP1SHf<0K6a/069RHP1<C4`0DE5A04a<C01K
Fe/0ATI607=cL`2DU9@0GemO04E5A@1[Jf/0ADE504=3@`1NGUh0E5AD03hn?P1PH600?Cdm04=3@`1:
BTX0EUIF03<c<`1IFET0Bd];075aL@1GEeL0ATI6055AD@2DU9@0CDe=02h^;P0l?3`0Bd];04M7A`1Q
HF40HV9R06][J`1GEeL0BDU904m?C`1FEUH0D51@03`l?019BDT0LG5a061PH01MGEd0Bd];04i>CP0n
?Sh0@D5104M7A`1=CDd0BTY:03hn?P0204Q8B00B05aLG01?Cdl0E5AD04i>CP1PH600Cdm?06EUI@1O
Gel0FEUI05=CD`11@D40@T9204e=C@1UIFD0K6a/05MGE`1FEUH0Fe]K3_ooool0=`1aLG40ATI605YJ
FP1JFUX0@T92055AD@13@d<0?3`l04Q8B01;Bd/0DU9B05YJFP13@d<0:RXZ03Lg=`0l?3`0?clo02PX
:01OGel0?3`l03Xj>P1<C4`0EeMG04I6AP0c<c<0FUYJ04m?C`0g=cL0GEeM04U9B@0d=3@0G5aL038b
<P1IFET0De=C051@D01>CTh0@d=3065QH@1@D500EeMG07UiN@1PH600MGEe04m?C`1<C4`0B4Q805QH
F00a<C40MWIf04e=C@1@D500EeMG03lo?`1DE5@00P1WIfL0D`1KFe/0DE5A06ATI01XJ6P0DE5A07Mg
M`1LG5`0KFe]051@D01FEUH0SHf=04i>CP1OGel0Ogmo06UYJ@1:BTX0O7al04a<C01NGUh0FEUI07Qh
N01gMgL0IfMW08f=S@1WIfL0Shn?05ADE01kNg/0MGEe065QH@2SXj<0HV9R06YZJP0i>CT0Fe]K06EU
I@1bLW80@D5105eMG@1hN7P0F5QH059BDP1JFUX0VIVI07=cL`22PX80M7Ad08^;R`1:BTX0IFEU06a/
K01NGUh0R8R805IFEP11@D40Kfm_06ATI01JFUX0Ph>305YJFP1CDe<0EUIF06ATI02;Rh/0FUYJ04I6
AP1HF5P0BDU906IVIP1ADE40CDe=07UiN@0j>SX0MgMg06i^KP1NGUh0Cdm?071`L02YZJT0E5AD04i>
CP14A4@0Ogmo00kooooo02h0MgMg03/k>`0X:2P07QhN01LG5`0l?3`0PH6106=SH`11@D40410@014A
4@0E5AD051@D01`L700^;Rh0E5AD09jNWP2LW9`0Xj>S055AD@1?Cdl0FUYJ05UIF@1cLg<0Zj^[07in
OP0Y:BT0EUIF08n?S`0h>3P0Shn?07QhN01QHF40>CTi03Xj>P2f][H0GUiN05mOG`24Q8@0/k>c09VI
V@0O7al0<c<c01XJ6P1XJ6P0DU9B0P2;Rh/0900f=SH0;2`/04E5A@1KFe/0K6a/0861P@1lO7`0;2`/
03Ti>@0e=CD0;bl_06UYJ@1jNWX0E5AD09bLW01RHV80<S8b06YZJP2h^;P0YJFU08^;R`1:BTX0C4a<
06=SH`2_[jl0KVi^04e=C@1FEUH0UYJF09NGU`1bLW80ADE503Xj>P0N7Qh07alO02`/;080UiNG0180
EUIF024Q8@0S8b<08B4Q01dM7@1VIVH0/;2`028R8P0E5AD05QHF01hN7P17AdL0U9BD0:6QX@1=CDd0
O7al09BDU017AdL2069RHP0<055AD@16ATH0@d=303De=@0K6a/0@41007AdM027QhL0=CDe05eMG@0c
<c<08b<S0P0a<C40501CDe<0TY:B05UIF@1lO7`0Ph>306EUI@1OGel0JVYZ08:2PP18B4P0=cLg065Q
H@1RHV80ADE507moO`21PH40@D5103@d=01QHF40F5QH5?ooool001?ooooo00`0QhN705eMG@26QXH0
Hf=S05aLG01VIVH0I6AT05QHF01GEeL0FEUI05eMG@15ADD205=CD`0f0492@P1:BTX0IFEU03<c<`1?
Cdl0IVIV02`/;01=CDd0H61P0492@P1OGel0@d=304m?C`1>CTh0Cdm?051@D01FEUH0ADE506IVIP14
A4@0AdM707IfMP1_Kfl0>CTi06=SH`1dM7@0C4a<05iNGP1MGEd0CTi>05ADE01HF5P0G5aL05iNGP1C
De<0GEeM04m?C`1<C4`0GEeM02d];@1VIVH0HV9R02/[:`1_Kfl0K6a/02h^;P2GUiL0HV9R03@d=02<
S8`0BDU904A4A01iNGT0DE5A0P1NGUh01@1OGel0HF5Q0451@@1XJ6P0L71`0080GUiN0280KFe]03`l
?01UIFD0F5QH04Y:BP1UIFD0JFUY04i>CP1MGEd0PH61055AD@1YJFT0IfMW034a<@1HF5P0JVYZ051@
D01IFET0@d=306ATI01GEeL0?Shn07inOP1@D500Hf=S04m?C`1PH600MgMg04U9B@1<C4`0L71`04];
B`1HF5P0J6QX0P1ZJVX06P22PX80H61P06YZJP1OGel0GEeM06m_K`15ADD0MgMg06m_K`15ADD0FEUI
06ATI01:BTX0N7Qh06EUI@0f=SH0O7al04U9B@0d=3@0GemO06MWI`15ADD0IFEU03lo?`1ADE40Fe]K
3_ooool0S01dM7@0>c/k05eMG@1IFET0FUYJ071`L01jNWX0JFUY06MWI`1=CDd0?Cdm04i>CP1<C4`0
Ng]k06YZJP1VIVH0Fe]K071`L01MGEd0=CDe04=3@`0o?cl0A4A404U9B@14A4@0@D5102d];@1TI6@0
;2`/03Xj>P0n?Sh0C4a<05aLG01;Bd/0CDe=04M7A`1=CDd0CTi>05IFEP1=CDd0@d=304i>CP10@400
<S8b02`/;00l?3`0=SHf03`l?018B4P0@d=30410@00c<c<0EUIF055AD@14A4@0ADE505=CD`10@400
EUIF0451@@1FEUH0JFUY092@T01KFe/0NWYj07]kN`1ADE40EeMG0410@01DE5@0=SHf06YZJP1mOGd0
EeMG04I6AP1JFUX0A4A402`/;01`L700AdM704=3@`1@D500@d=305]KF`1[Jf/0F5QH04U9B@1ADE40
Fe]K06YZJP1DE5@0DU9B05ADE02IVIT0E5AD03<c<`12@T80CDe=04A4A01@D500GemO05iNGP19BDT0
;Rh^04M7A`11@D40B4Q80492@P14A4@0EeMG0410@018B4P0ATI603lo?`0g=cL0ADE50492@P1@D500
Fe]K04Y:BP13@d<0ATI604e=C@1BDU80B4Q805aLG01WIfL0H61P05QHF026QXH0GEeM059BDP14A4@0
ADE504I6AP1TI6@0GUiN061PH018B4P0CDe=3_ooool06`1[Jf/0ADE504e=C@1WIfL0@41004M7A`1I
FET0;bl_051@D00l?3`0Cdm?065QH@0m?Cd0=cLg03hn?P0i>CT0C4a<03@d=01@D500?3`l03dm?@1C
De<0@T920410@00b<S80AdM705ADE00205EEE@0H03dm?@1ADE40GUiN0492@P1WIfL0AdM705UIF@1R
HV80@d=30861P@18B4P0De=C096AT@1IFET0GEeM065QH@1EEED0>c/k04i>CP18B4P0IVIV04U9B@14
A4@0EeMG0P19BDT0201lO7`0MgMg061PH01@D500IVIV061PH01NGUh0Ogmo0P1_Kfl03@1LG5`0Jf][
09>CT`1BDU80GUiN07inOP1aLG40DE5A07alO01ADE40JFUY05aLG01SHf<00P1eMGD04P2DU9@0Ogmo
08j>SP1[Jf/0UiNG08V9R@1LG5`0YJFU06YZJP1VIVH0@d=306MWI`1cLg<0Ph>305aLG023Ph<0UIFE
05IFEP80GUiN02@0QHF5071`L01lO7`0IFEU09VIV@1MGEd0D51@06a/K01RHV80OGem05UIF@16ATH0
QhN705EEE@1XJ6P0Vi^K04Q8B01VIVH0MGEe05ADE023Ph<0Ng]k04A4A016ATH0@d=308:2PP19BDT0
E5AD07EeM@1GEeL0LG5a055AD@1SHf<0HV9R06EUI@2ATI42059BDP0304m?C`1nOWkooooo00gooooo
08`0Ng]k05YJFP0];Bd0>3Ph05YJFP0[:b/051@D03Lg=`1LG5`0>CTi010@400D51@06QXJ02TY:@0e
=CD0VIVI04E5A@1MGEd0LW9b05]KF`1BDU809bLW03Lg=`19BDT0Jf][0<W9b@1fMWH0GEeM07YjNP2L
W9`0KVi^04=3@`0l?3`0@D5105YJFP1HF5P0>SXj0410@015ADD0UIFE051@D02>SXh0Shn?04A4A00_
;bl0:BTY055AD@1hN7P0Bd];04Y:BP1XJ6P0LG5a09RHV01iNGT0/K6a065QH@0n?Sh0BDU903@d=00U
9BD0Hf=S09FEU@2FUYH0GemO02HV9P1XJ6P0Z:RX0:j^[P17AdL0>c/k01`L701MGEd0HF5Q07IfMP19
BDT0ADE50451@@2?Shl0^KVi03lo?`0H61P0AdM703`l?01dM7@0UiNG0;6a/@1DE5@0EEEE059BDP0g
=cL06a/K04e=C@2LW9`0IFEU02XZ:P0H61P071`L02h^;P2?Shl0P82002d];@14A4@0=CDe02l_;`1j
NWX0C4a<03De=@0Z:RX0GemO05YJFP1lO7`0OGem0820P02?Shl0:b/[05ADE00S8b<0<C4a04=3@`1]
KFd0Vi^K0<_;b`23Ph<0UIFE08J6QP0Z:RX0=cLg0:6QX@1kNg/0N7Qh069RHP1UIFD0L71`05aLG01/
K6`0DE5A04M7A`12@T80Fe]K05eMGACooooo003oooooooooooooP_ooool00001\
\>"],
  ImageRangeCache->{{{0, 639}, {73.375, 0}} -> {-0.126578, -0.0116304, \
0.00665596, 0.00665596}, {{19, 158.75}, {71.625, 1.6875}} -> {-43.5083, \
-4.07955, 2.28983, 2.28983}, {{172.75, 312.5}, {71.625, 1.6875}} -> \
{-395.569, -4.07955, 2.28983, 2.28983}, {{326.438, 466.188}, {71.625, \
1.6875}} -> {-747.487, -4.07955, 2.28983, 2.28983}, {{480.188, 619.938}, \
{71.625, 1.6875}} -> {-1099.55, -4.07955, 2.28983, 2.28983}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .25227 
%%ImageSize: 640 161.455 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.238095 0.00600649 0.238095 [
[ 0 0 0 0 ]
[ 1 .25227 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .25227 L
0 .25227 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.00600649 0.261905 0.246266 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .59322 L
.2381 .59322 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .59322 L
.2381 .59322 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .01355 L
.47619 .01355 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .01355 L
.47619 .01355 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .01209 L
.71429 .01209 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .01209 L
.71429 .01209 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .02294 L
.95238 .02294 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .02294 L
.95238 .02294 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF8000FF8000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF0000
FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF8000FF8000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF800000FF00FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000
FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF8000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF8000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF8000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00000000FF
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
00FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF8000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000
FF0000FF0000FF0000FF0000FF0000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.261905 0.00600649 0.5 0.246266 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .01585 L
.2381 .01585 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .01585 L
.2381 .01585 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .47578 L
.47619 .47578 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .47578 L
.47619 .47578 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .13307 L
.71429 .13307 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .13307 L
.71429 .13307 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .0171 L
.95238 .0171 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .0171 L
.95238 .0171 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF00FF00FF8000FF80000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF00
00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00
00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF00
0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF8000FF80000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF0000FF0000FF00
FF00000000FF00FF000000FF00FF00
FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF80000000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
FF000000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
00FF000000FF00FF000000FF0000FF
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF000000FF
FF0000FF8000FF800000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF00FF00
FF8000FF80000000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF000000FF0000FF0000FF
0000FF00FF0000FF000000FF0000FF0000FF0000FF00FF000000FF0000FF00FF0000FF00
00FF0000FF0000FF000000FF00FF000000FF00FF0000FF000000FF00FF000000FF00FF00
00FF000000FF00FF000000FF0000FF0000FF00FF000000FF0000FFFF80000000FF0000FF
0000FF0000FF00FF000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF00FF000000FF00FF000000FF00FF0000FF000000FF0000FF0000FF00FF00
00FF000000FF0000FF0000FF00FF000000FF00FF000000FF0000FF00FF0000FF0000FF00
00FF000000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF00
0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00FF800000FF0000FF000000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF
0000FF0000FF00FF0000FF0000FF0000FF000000FF00FF000000FF0000FF00FF0000FF00
0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF000000FF00FF00
00FF000000FF00FF000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF00FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00
0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF00
0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF00FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF00
0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF00
0000FF00FF0000FF0000FF0000FF000000FFFF8000FF8000FF00000000FF00FF0000FF00
00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF00FF80000000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF00
0000FF00FF0000FF0000FF0000FF000000FF0000FFFF8000FF80000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF000000FF0000FF00FF00
00FF00FF000000FF000000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
00FF0000FF0000FF0000FF00FF80000000FFFF80000000FFFF000000FF0000FF0000FF00
00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF00
FF00000000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF000000FF
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF00
00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF00FF0000FF0000FF000000FF00FF0000FF000000FF
0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF00FF0000FF00FF00000000FF0000FF00FF000000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF00FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF00
00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF
0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
00FF000000FF00FF0000FF000000FF0000FF0000FF0000FFFF80000000FF00FF0000FF00
00FF000000FF00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF00FF0000FF000000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF00FF0000FF0000FF000000FF00FF000000FF00FF00FF80000000FF0000FF00FF00
0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF000000FF0000FF00FF00
0000FF00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00
00FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF00FF000000FF
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF00
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF00
0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF00
0000FF00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF000000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00
0000FF0000FF00FF000000FF00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF00FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF000000FF0000FF0000FF00FF0000FF000000FF00FF000000FF
0000FF0000FF00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF000000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF0000FF00
0000FF0000FF00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF0000FF
0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF00FF000000FF00FF0000FF00
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF000000FF0000FF0000FF
0000FF00FF000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF00
00FF000000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF000000FF00FF00
0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF00FF0000FF0000FF000000FF0000FF0000FFFF00000000FF0000FF00FF0000FF00
0000FF00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF00
00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF000000FF0000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF0000FF000000FFFF0000
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF0000FF
00FF0000FF000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF00
00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
00FF0000FF0000FF0000FF0000FF00
00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF00FF00
0000FF0000FFFF80000000FF0000FFFF800000FF0000FF000000FF00FF0000FF0000FF00
00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF00
0000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FFFF80000000FFFF000000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FFFF8000FF000000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF00FF000000FF00FF0000FF000000FF00FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
00FF000000FF0000FF0000FF0000FFFF000000FF000000FF0000FF0000FF00FF000000FF
0000FF00FF000000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF0000FF0000FF00
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.00600649 0.738095 0.246266 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .0146 L
.2381 .0146 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .0146 L
.2381 .0146 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .09991 L
.47619 .09991 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .09991 L
.47619 .09991 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .51124 L
.71429 .51124 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .51124 L
.71429 .51124 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .01606 L
.95238 .01606 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .01606 L
.95238 .01606 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
0000FF0000FF0000FF00FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
00FF0000FF0000FF0000FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FFFF00000000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FFFF00000000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FFFF80000000FF0000FF0000FF0000FF0000FFFF00000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FFFF000000FF000000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF8000FF8000FF00000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF8000FF80000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF00FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF8000FF8000FF80000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF00
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000FF8000FF80000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF000000FF00FF000000FF0000FF0000FF0000FF0000FF
00FF0000FF000000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF000000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF00FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF00FF000000FF00FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF00FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF000000FF
0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF000000FF00FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF
0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF00FF000000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FFFF00000000FF0000FFFF80000000FF0000FF
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF00FF00000000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF000000FF0000FF00FF000000FF0000FF0000FF
0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF8000
FF80000000FFFF00000000FF00FF0000FF000000FF00FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF0000FF0000
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FFFF00000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF00FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FFFF80000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF000000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF000000FF00FF000000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FF00FF000000FF0000FF0000FF00FF000000FF00FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF00FF0000FF00
00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF000000FF00FF0000FF000000FF00FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF00FF0000FF000000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF
00FF000000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF000000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF000000FF00FF000000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF
00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00
00FF0000FF0000FF000000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF00FF0000FF0000FF0000FF0000FF000000FF0000FF00FF0000FF000000FF00FF00
00FF000000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF00FF0000FF000000FF
00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
00FF000000FF00FF000000FF00FF0000FF0000FF0000FF0000FF0000FF0000FF000000FF
00FF000000FF00FF000000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF0000FF0000FF000000FF
0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF00FF0000FF000000FF00FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF000000FF00FF000000FF0000FF0000FF00FF0000FF0000FF0000FF00
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF00FF0000FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF0000FF000000FF00FF000000FF
0000FF00FF0000FF000000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF00FF0000FF0000FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF000000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF00FF00
0000FF00FF0000FF000000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF00FF0000FF0000FF000000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF00FF000000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF
0000FF0000FF0000FF0000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF
0000FF0000FF00FF000000FF0000FF0000FF0000FF0000FF00FF000000FF0000FF0000FF
0000FF00FF000000FF0000FF0000FF
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.738095 0.00600649 0.97619 0.246266 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.024026 0.952381 [
[ 0 0 0 0 ]
[ 1 1.00909 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.00909 L
0 1.00909 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.024026 0.97619 0.481664 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0118853 0.00594263 [
[ 0 0 -0.125 0 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13074 m
1 .13074 L
s
0 .24959 m
1 .24959 L
s
0 .36844 m
1 .36844 L
s
0 .4873 m
1 .4873 L
s
0 g
0 0 m
1 0 L
s
0 0 m
0 .61803 L
s
0 .61803 m
1 .61803 L
s
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01189 m
.04762 .01397 L
.2381 .01397 L
.2381 .01189 L
F
.5 Mabswid
.04762 .01189 m
.04762 .01397 L
.2381 .01397 L
.2381 .01189 L
.04762 .01189 L
s
0 1 0 r
.28571 .01189 m
.28571 .0146 L
.47619 .0146 L
.47619 .01189 L
F
.28571 .01189 m
.28571 .0146 L
.47619 .0146 L
.47619 .01189 L
.28571 .01189 L
s
0 0 1 r
.52381 .01189 m
.52381 .0123 L
.71429 .0123 L
.71429 .01189 L
F
.52381 .01189 m
.52381 .0123 L
.71429 .0123 L
.71429 .01189 L
.52381 .01189 L
s
1 .5 0 r
.7619 .01189 m
.7619 .60093 L
.95238 .60093 L
.95238 .01189 L
F
.7619 .01189 m
.7619 .60093 L
.95238 .60093 L
.95238 .01189 L
.7619 .01189 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.527427 0.97619 0.985065 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0123686 0.0114409 0.0123686 [
[ 0 0 0 0 ]
[ 1 .48052 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .48052 L
0 .48052 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01144 translate
.95238 .45764 scale
231 string
77 37 8 [77 0 0 37 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FF
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF800000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF0000FF00
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
00FF00FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF0000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF0000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF80000000FFFF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF800000FF00
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
FF8000FF800000FF0000FF00FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000FF8000
FF8000FF8000FF8000FF8000FF8000
pop
P
% End of image
MathSubEnd
P
% End of sub-graphic
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{640, 161.438},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000:00000XB000`400?l00000o`00003ooooooooo
oooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo
003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003ooooooooo
ooooP_ooool002CoooooLOl0000Vooooog7o00009ooooomao`0002KoooooLOl0000Uooooo`009?oo
ool00ol0003oooooooooo`02oooooaKoo`001OoooolFo`3o00Cooooo5ol00?l4oooooaOoogl01?oo
ool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo00?ooooo5_oo0004oooooaOo0?l01?oo
oolFo`00o`Gooooo5_ooO`04ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool00_oo
oolFool000Gooooo5_l0o`04oooooaOo003o1?oooolFoomo00Gooooo00?o0000ooooooooool09?oo
ool00ol0003oooooooooo`02oooooaOoo`001?oooolFo`3o00Gooooo5_l00?l4oooooaOoogl01?oo
ool00ol0003oooooooooo`0Sooooo`009?ooool00ol0003oooooooooo`02oooooaKoo`00>_oooolG
oomo00Cooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`0MoooooaOo0?l01?oooolF
o`00o`Gooooo5_ooO`04ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolF
o`3o00Cooooo5ol00?lOooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0D_oooolG
oomo00Cooooo00?o0000ooooooooool08oooool002Cooooo00?o0000ooooooooool00_oooolFool0
05Gooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`0MoooooaOo0?l01?oooolFo`00
oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01?oooolGo`00
oaoooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`1BoooooaOoogl01?ooool00ol0
003oooooooooo`0Sooooo`009?ooool00ol0003oooooooooo`02oooooaKoo`00EOooool00ol0003o
ooooooooo`0Tooooo`03o`000?oooooooooo01gooooo5ol0o`04oooooaKo003o7oooool00ol0003o
ooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`04oooooaOo003o7oooool00ol0003o
ooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?o
oooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo
00?o0000ooooooooool07OoooolGo`3o00Cooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool07OoooolFo`3o00Cooooo5ol00?lOooooo`03o`000?oooooooooo02Cooooo
00?o0000ooooooooool0D_oooolGoomo00Cooooo00?o0000ooooooooool08oooool002Cooooo00?o
0000ooooooooool00_oooolFool005Gooooo00?o0000ooooooooool09?ooool00ol0003ooooooooo
o`0MoooooaOo0?l01?oooolFo`00oaoooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`0MoooooaKo0?l01?oooolGo`00oaoooooo00?o0000ooooooooool09?ooool00ol0003ooooooooo
o`1BoooooaOoogl01?ooool00ol0003oooooooooo`0Sooooo`009?ooool00ol0003oooooooooo`02
oooooaKoo`00EOooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo01gooooo5ol0o`04
oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo01gooooo5_l0o`04
oooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04
ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eoooo
o`03o`000?oooooooooo02Cooooo00?o0000ooooooooool07OoooolGo`3o00Cooooo5_l00?lOoooo
o`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07OoooolFo`3o00Cooooo5ol00?lOoooo
o`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0D_oooolGoomo00Cooooo00?o0000oooo
ooooool08oooool002Cooooo00?o0000ooooooooool00_oooolFool005Gooooo00?o0000oooooooo
ool09?ooool00ol0003oooooooooo`0MoooooaOo0?l01?oooolFo`00oaoooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`0MoooooaKo0?l01?oooolGo`00oaoooooo00?o0000oooooooo
ool09?ooool00ol0003oooooooooo`1BoooooaOoogl01?ooool00ol0003oooooooooo`0Sooooo`00
9?ooool00ol0003oooooooooo`02oooooaKoo`00EOooool00ol0003oooooooooo`0Tooooo`03o`00
0?oooooooooo01gooooo5ol0o`04oooooaKo003o7oooool00ol0003oooooooooo`0Uooooo`03o`00
0?oooooooooo01gooooo5_l0o`04oooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`00
0?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oo
oooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool07Ooo
oolGo`3o00Cooooo5_l00?lOooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool07Ooo
oolFo`3o00Cooooo5ol00?lOooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0D_oo
oolGoomo00Cooooo00?o0000ooooooooool08oooool002Cooooo00?o0000ooooooooool00_oooolF
ool005Gooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`0MoooooaOo0?l01?oooolF
o`00oaoooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o7oooool0
0ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooo
oooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo
02Cooooo00?o0000ooooooooool07OoooolGo`3o00Cooooo5_l00?lOooooo`03o`000?oooooooooo
02Gooooo00?o0000ooooooooool0=ooooolGo`00oaoooooo00?o0000ooooooooool09?ooool00ol0
003oooooooooo`1BoooooaOoogl01?ooool00ol0003oooooooooo`0Sooooo`009?ooool00ol0003o
001oo`00O`02o`00OaKoo`00EOl007l00ol0003oooooooooo`0Tooooo`03o`000?l007oo001o01go
001o5ol0o`04o`00OaKo003o7ol007l00ol0003oooooooooo`0Uooooo`03o`000?l007oo001o03Oo
001o5ol00?lOo`00O`03o`000?oooooooooo02Cooooo00?o0000o`00Ool007l0D_l007lGoomo00Co
001o00?o0000ooooooooool08oooool002Cooooo00?o0000ooooooooool00_oooolFool005Gooooo
00?o0000ooooooooool09?ooool00ol0003oooooooooo`0MoooooaOo0?l01?oooolFo`00oaoooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o7oooool00ol0003ooooo
ooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo
000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o
0000ooooooooool07OoooolGo`3o03Wooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`0goooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5ooo
O`04ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001E
ooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool07OoooolGo`3o03Wooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o7oooool00ol0003oooooooooo`0T
ooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000Toooo
o`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000oooo
ooooool07OoooolGo`3o03Wooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0goooo
oaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04oooo
o`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03
o`000?oooooooooo02Cooooo00?o0000ooooooooool07OoooolGo`3o03Wooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`0goooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03
o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`00
0?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0
7OoooolGo`3o03Wooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o
7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`00
0?oooooooooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oo
oooooooo02Cooooo00?o0000ooooooooool07OoooolGo`3o03Wooooo00?o0000ooooooooool09Ooo
ool00ol0003oooooooooo`0goooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`000?oo
oooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooo
oooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool07OoooolG
o`3o03Wooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o7oooool0
0ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooo
oooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo
02Cooooo00?o0000ooooooooool07OoooolGo`3o03Wooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`0goooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo
05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooooooo00;o
oooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool07OoooolGo`3o03Wo
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o7oooool00ol0003o
ooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?o
oooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo
00?o0000ooooooooool07OoooolGo`3o03Wooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`0goooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo
5oooO`04ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo
001Eooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool07OoooolGo`3o03Wooooo00?o
0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o7oooool00ol0003ooooooooo
o`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000T
ooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000
ooooooooool07OoooolGo`3o03Wooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0g
oooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04
ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?l007oo001o00;o001o5_oo001Eo`00
O`03o`000?oooooooooo02Cooooo00?o0000o`00Ool007l07Ol007lGo`3o03Wo001o00?o0000oooo
ooooool09Oooool00ol0003o001oo`00O`0go`00OaOo003o7ol007l00ol0003oooooooooo`0Toooo
o`03o`000?l007oo001o05;o001o5oooO`04o`00O`03o`000?oooooooooo02?ooooo000Tooooo`03
o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000oooooooo
ool07OoooolGo`3o03Wooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo
003o7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03
o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`00
0?oooooooooo02Cooooo00?o0000ooooooooool07OoooolGo`3o03Wooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`0goooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`00
0?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oo
oooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool07Ooo
oolGo`3o03Wooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o7ooo
ool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oo
oooooooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooo
oooo02Cooooo00?o0000ooooooooool07OoooolGo`3o03Wooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`0goooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooo
oooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooooooo
00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool07OoooolGo`3o
03Wooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o7oooool00ol0
003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo
02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Co
oooo00?o0000ooooooooool07OoooolGo`3o03Wooooo00?o0000ooooooooool09Oooool00ol0003o
ooooooooo`0goooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;o
oooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo
5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool07OoooolGo`3o03Wooooo
00?o0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o7oooool00ol0003ooooo
ooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo
000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o
0000ooooooooool07OoooolGo`3o03Wooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`0goooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5ooo
O`04ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001E
ooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool07OoooolGo`3o03Wooooo00?o0000
ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o7oooool00ol0003oooooooooo`0T
ooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000Toooo
o`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000oooo
ooooool07OoooolGo`3o03Wooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0goooo
oaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04oooo
o`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03
o`000?oooooooooo02Cooooo00?o0000ooooooooool07OoooolGo`3o03Wooooo00?o0000oooooooo
ool09Oooool00ol0003oooooooooo`0goooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03
o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`00
0?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0
7OoooolGo`3o03Wooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o
7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`00
0?oooooooooo02?ooooo000Tooooo`03o`000?l007oo001o00;o001o5_oo001Eo`00O`03o`000?oo
oooooooo02Cooooo00?o0000o`00Ool007l07Ol007lGo`3o03Wo001o00?o0000ooooooooool09Ooo
ool00ol0003o001oo`00O`0go`00OaOo003o7ol007l00ol0003oooooooooo`0Tooooo`03o`000?l0
07oo001o05;o001o5oooO`04o`00O`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooo
oooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool07OoooolG
o`3o03Wooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o7oooool0
0ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooo
oooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo
02Cooooo00?o0000ooooooooool07OoooolGo`3o03Wooooo00?o0000ooooooooool09Oooool00ol0
003oooooooooo`0goooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo
05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooooooo00;o
oooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool07OoooolGo`3o03Wo
oooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o7oooool00ol0003o
ooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?o
oooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo
00?o0000ooooooooool07OoooolGo`3o03Wooooo00?o0000ooooooooool09Oooool00ol0003ooooo
ooooo`0goooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo
5oooO`04ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo
001Eooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool07OoooolGo`3o03Wooooo00?o
0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o7oooool00ol0003ooooooooo
o`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000T
ooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000
ooooooooool07OoooolGo`3o03Wooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0g
oooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04
ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eoooo
o`03o`000?oooooooooo02Cooooo00?o0000ooooooooool07OoooolGo`3o03Wooooo00?o0000oooo
ooooool09Oooool00ol0003oooooooooo`0goooooaOo003o7oooool00ol0003oooooooooo`0Toooo
o`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03
o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000oooooooo
ool07OoooolGo`3o03Wooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo
003o7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03
o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`00
0?oooooooooo02Cooooo00?o0000ooooooooool07OoooolGo`3o03Wooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`0goooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`00
0?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oo
oooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool07Ooo
oolGo`3o03Wooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o7ooo
ool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oo
oooooooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooo
oooo02Cooooo00?o0000ooooooooool07OoooolGo`3o03Wooooo00?o0000ooooooooool09Oooool0
0ol0003oooooooooo`0goooooaOo003o7oooool00ol0003oooooooooo`0Tooooo`03o`000?oooooo
oooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooooooo
00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool07OoooolGo`3o
03Wooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o7oooool00ol0
003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo
02?ooooo000Tooooo`03o`000?l007oo001o00;o001o5_oo001Eo`00O`03o`000?oooooooooo02Co
oooo00?o0000o`00Ool007l0KOl007l00ol0003oooooooooo`0Uooooo`03o`000?l007oo001o03Oo
001o5ol00?lOo`00O`03o`000?oooooooooo02Cooooo00?o0000o`00Ool007l0D_l007lGoomo00Co
001o00?o0000ooooooooool08oooool002Cooooo00?o0000ooooooooool00_oooolFool005Gooooo
00?o0000ooooooooool09?ooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo
00?o0000ooooooooool0=ooooolGo`00oaoooooo00?o0000ooooooooool09?ooool00ol0003ooooo
ooooo`1BoooooaOoogl01?ooool00ol0003oooooooooo`0Sooooo`009?ooool00ol0003ooooooooo
o`02oooooaKoo`00EOooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo06gooooo00?o
0000ooooooooool09Oooool00ol0003oooooooooo`0goooooaOo003o7oooool00ol0003ooooooooo
o`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000T
ooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000
ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo03Oooooo5ol00?lO
ooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0D_oooolGoomo00Cooooo00?o0000
ooooooooool08oooool002Cooooo00?o0000ooooooooool00_oooolFool005Gooooo00?o0000oooo
ooooool09?ooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000oooo
ooooool0KOooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04oooo
o`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03
o`000?oooooooooo02Cooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03
o`000?oooooooooo06gooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`1BoooooaOo
ogl01?ooool00ol0003oooooooooo`0Sooooo`009?ooool00ol0003oooooooooo`02oooooaKoo`00
EOooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool0
9Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0
D_oooolGoomo00Cooooo00?o0000ooooooooool08oooool002Cooooo00?o0000ooooooooool00_oo
oolFool005Gooooo00?o0000ooooooooool09?ooool00ol0003oooooooooo`1]ooooo`03o`000?oo
oooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Tooooo`03o`000?oo
oooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?ooooo000Tooooo`03o`000?oooooo
oooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0KOooool0
0ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09?ooool0
0ol0003oooooooooo`1BoooooaOoogl01?ooool00ol0003oooooooooo`0Sooooo`009?ooool00ol0
003oooooooooo`02oooooaKoo`00EOooool00ol0003oooooooooo`0Tooooo`03o`000?oooooooooo
06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]ooooo`03o`000?oooooooooo
02Cooooo00?o0000ooooooooool0D_oooolGoomo00Cooooo00?o0000ooooooooool08oooool002Co
oooo00?o0000ooooooooool00_oooolFool005Gooooo00?o0000ooooooooool09?ooool00ol0003o
ooooooooo`1]ooooo`03o`000?oooooooooo02Gooooo00?o0000ooooooooool0KOooool00ol0003o
ooooooooo`0Tooooo`03o`000?oooooooooo05;ooooo5oooO`04ooooo`03o`000?oooooooooo02?o
oooo000Tooooo`03o`000?oooooooooo00;ooooo5_oo001Eooooo`03o`000?oooooooooo02Cooooo
00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Uooooo`03o`000?oooooooooo06gooooo
00?o0000ooooooooool09?ooool00ol0003oooooooooo`1BoooooaOoogl01?ooool00ol0003ooooo
ooooo`0Sooooo`009?ooool00ol0003oooooooooo`02oooooaKoo`00EOooool00ol0003ooooooooo
o`0Tooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003ooooooooo
o`1]ooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0D_oooolGoomo00Cooooo00?o
0000ooooooooool08oooool002Cooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0T
ooooo`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]
ooooo`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0D_oooolGoomo00Cooooo00?o0000
ooooooooool08oooool002Cooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Toooo
o`03o`000?oooooooooo06gooooo00?o0000ooooooooool09Oooool00ol0003oooooooooo`1]oooo
o`03o`000?oooooooooo02Cooooo00?o0000ooooooooool0KOooool00ol0003oooooooooo`0Soooo
o`009?oooomao`0002KoooooLOl0000Wooooog7o00009_oooomao`0002Gooooo003ooooooooooooo
P_ooool00?oooooooooooon2ooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oo
oooooooooon2ooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2
ooooo`005ooooomA0?l00003003o003oP000oh0000d0o`000P3oP00>0?l00080oh001@3o00040?n0
00030?l0003oP000oh0000T0o`000P3oP00>ooooo`80o`007000o`040000o`800?l00`3oP0020000
obL00?l00P000?l9003o0080003o1`00o`000`3o00000?l0003o0007003o00@0003o2`00o`000`00
0?l00?l0003o00070000o`800?l03_ooool50000o`@00?l00P000?l00`00o`00003o0000o`0[0000
o`800?l0>@000?l00`00o`00003o0000o`0B0000o`kooooo=03oP002003o04H0oh0000<0o`000?n0
003oP0002`3oP00Iooooo`005ooooomA0?l00003003o003oP000oh0000d0o`000P3oP00>0?l00080
oh001@3o00040?n000030?l0003oP000oh0000T0o`000P3oP00>ooooo`80o`007000o`040000o`80
0?l00`3oP0020000obL00?l00P000?l9003o0080003o1`00o`000`3o00000?l0003o0007003o00@0
003o2`00o`000`000?l00?l0003o00070000o`800?l03_ooool50000o`@00?l00P000?l00`00o`00
003o0000o`0[0000o`800?l0>@000?l00`00o`00003o0000o`0B0000o`kooooo=03oP002003o04H0
oh0000<0o`000?n0003oP0002`3oP00Iooooo`005ooooold0?l000D0oh00@@3o00020?n000030?l0
003oP000oh0000H0o`0000<0oh000?l0003o00000P3o000>ooooob800?l00P3o00030?n00080003o
9@00o`040000oa000?l00P000?lE003o0080oh001@00o`020?l00080003o00<00?l00000o`000?l0
0P00o`0>ooooo`T00?l00P000?l00`00o`00003o0000o`0@0000o`800?l040000?l2003o00D0003o
1000o`1>0000o`koooooE03oP0020000oa@0oh000P3o000N0?n001Wooooo000Goooooc@0o`001@3o
P0110?l00080oh0000<0o`000?n0003oP0001P3o00000`3oP000o`000?l000020?l000kooooo8P00
o`020?l000<0oh000P000?lU003o00@0003o4000o`020000oaD00?l00P3oP005003o0080o`000P00
0?l00`00o`00003o0000o`02003o00kooooo2@00o`020000o`03003o00000?l0003o0100003o0P00
o`0@0000o`800?l01@000?l4003o04h0003o3_oooomD0?n00080003o503oP0020?l001h0oh006Ooo
ool001OoooooDP3o00020?n002H0o`000P3oP0090?l000D0oh003_ooool20?l001<00?l00P000?l=
003o00030?n000000?l0003o0080003o5`00o`020?l000T00?l00P000?l/003o00L0003o0P00o`02
0000o`03003o00000?l0003o0080003o3_ooool50000o`800?l050000?l00`00o`00003o0000o`1?
0000o`030?l000000?l0003o00@0003o00<00?l00000o`000?l04P000?l>ooooo`T0oh000P3o000i
0?n00080o`00;@3oP0000`3o0000oh000?n0000D0?n001Wooooo000GooooocP0o`0000<0oh000?l0
003o00005P3o00030?n003H0o`003_ooool20000ob000?l01`000?l<003o0080003o1`00o`020000
o`80o`000`3oP006003o0080003o8`00o`020000o`D00?l01`000?l2003o00@0oh001@000?l>oooo
o`D0003o1P00o`0/0000o`800?l0?P000?l20?l00080003o0P3o000<0000o`koooooO@3oP0020?l0
00/0oh006Oooool001Oooooo>03o00000`3oP000o`000?l0000F0?l000<0oh00=P3o000>ooooo`80
003o8000o`070000o``00?l00P000?l7003o0080003o0P3o00030?n000H00?l00P000?lS003o0080
003o1@00o`070000o`800?l0103oP0050000o`kooooo1@000?l6003o02`0003o0P00o`0n0000o`80
o`000P000?l20?l000`0003o3_oooomm0?n00080o`002`3oP00Iooooo`005ooooolR0?l000@0oh00
;03o00040?n00280o`000P3oP00>0?l00080oh003_ooool40000ob000?l000<0003o003o0000o`00
0P000?l<003o0080003o1000o`070000o`<00?l020000?l00`00o`00003o0000o`020000o`T00?l0
0P000?l00`00o`00003o0000o`04003o00030000o`00o`000?l00080003o1000o`000`000?l00?l0
003o00050000o`800?l010000?l20?n000L0003o00<00?l00000o`000?l00P00o`0>ooooo`@0003o
2000o`0;0000o`800?l0;0000?l20?n000T0003o0P3o000R0000o`<00?l010000?l20?l00003003o
00000?l0003o00T0003o3_oooon:0?n001Wooooo000Gooooob80o`00103oP00/0?l000@0oh008P3o
00020?n000h0o`000P3oP00>ooooo`@0003o8000o`000`000?l00?l0003o00020000o``00?l00P00
0?l4003o00L0003o0`00o`080000o`03003o00000?l0003o0080003o2@00o`020000o`03003o0000
0?l0003o00@00?l000<0003o003o0000o`000P000?l4003o00030000o`00o`000?l000D0003o0P00
o`040000o`80oh001`000?l00`00o`00003o0000o`02003o00kooooo10000?l8003o00/0003o0P00
o`0/0000o`80oh002@000?l20?l00280003o0`00o`040000o`80o`0000<00?l00000o`000?l02@00
0?l>ooooohX0oh006Oooool001OoooooL`3o00030?n000800?l04P3o000>oooooaL00?l00P000?l@
003o0080003o4@00o`080000o`03003o00000?l0003o00800?l00P000?l3003o00H0003o0`00o`06
0000o`03003o00000?l0003o00800?l00`000?l8003o00030000o`00o`000?l000L00?l01`000?l2
003o00L0003o0P00o`0>ooooo`D0003o1`00o`0o0000o`D0oh000P3o000R0000o`03003o00000?l0
003o00@0003o0`00o`0;0000o`koooooEP3oP0020?l00380oh006Oooool001OoooooL`3o00030?n0
00800?l04P3o000>oooooaL00?l00P000?l@003o0080003o4@00o`080000o`03003o00000?l0003o
00800?l00P000?l3003o00H0003o0`00o`060000o`03003o00000?l0003o00800?l00`000?l8003o
00030000o`00o`000?l000L00?l01`000?l2003o00L0003o0P00o`0>ooooo`D0003o1`00o`0o0000
o`D0oh000P3o000R0000o`03003o00000?l0003o00@0003o0`00o`0;0000o`koooooEP3oP0020?l0
0380oh006Oooool001Oooooo0P3oP01b0?l00080oh00503o000>oooooaD00?l00P000?l=003o0003
0?n00000o`000?l000800?l00P000?lA003o01H0003o0P00o`050000o`L00?l00P000?l2003o00<0
003o1000o`020000o`X00?l00P000?l4003o00030000o`00o`000?l000800?l00P000?l00`00o`00
003o0000o`02003o00kooooo10000?l5003o0080003o00<00?l00000o`000?l0?`000?l50?n002<0
003o0P00o`0B0000o`koooooRP3oP00Iooooo`005ooooon:0?l000kooooo:@00o`020000oa400?l0
1P000?l3003o0080003o0P00o`090000o`<00?l01P000?l>003o00D0003o2`00o`050000o`800?l0
1`000?l2003o00kooooo0P000?l<003o0080003o2@00o`0B0000o`03003o00000?l0003o00@0003o
0`00o`0F0000o`L0oh0080000?l2003o01D0003o3_oooon:0?n001Wooooo000GooooohX0o`003_oo
oolY003o0080003o4@00o`060000o`<00?l00P000?l2003o00T0003o0`00o`060000o`h00?l01@00
0?l;003o00D0003o0P00o`070000o`800?l03_ooool20000o``00?l00P000?l9003o0180003o00<0
0?l00000o`000?l010000?l3003o01H0003o1`3oP00P0000o`800?l05@000?l>ooooohX0oh006Ooo
ool001OoooooRP3o000>oooooaD00?l010000?l;003o00<0003o5@00o`060000o`<00?l00P000?l2
003o0100003o8P00o`030000o`800?l01`000?l2003o00kooooo0P00o`020000o`X00?l01P000?l5
003o01/0003o0`00o`0k0000o`800?l05@000?l>ooooo`h0oh000P00o`1j0?n001Wooooo000Goooo
ohX0o`003_oooolE003o00@0003o2`00o`030000oaD00?l01P000?l3003o0080003o0P00o`0@0000
ob800?l00`000?l2003o00L0003o0P00o`0>ooooo`800?l00P000?l:003o00H0003o1@00o`0K0000
o`<00?l0>`000?l2003o01D0003o3_ooool>0?n000800?l0NP3oP00Iooooo`005ooooomA0?l00003
003o003o0000o`0002l0o`000P3oP0050?l000kooooo9000o`000`000?l00?l0003o000D003o0003
0000o`00o`000?l000800?l00P000?l7003o0080003o0`3oP0020?l00080003o1@00o`020000obT0
0?l00P3oP0030000o`800?l03_ooool70000o`@00?l00`000?l;003o01L0003o0P00o`070000o`80
0?l04P000?l50?n00180003o00<0oh000000o`000?l02@000?l4003o00030000o`00o`000?l000T0
003o1000o`050000o`koooooRP3oP00Iooooo`005ooooomA0?l00003003o003o0000o`0002l0o`00
0P3oP0050?l000kooooo9000o`000`000?l00?l0003o000D003o00030000o`00o`000?l000800?l0
0P000?l7003o0080003o0`3oP0020?l00080003o1@00o`020000obT00?l00P3oP0030000o`800?l0
3_ooool70000o`@00?l00`000?l;003o01L0003o0P00o`070000o`800?l04P000?l50?n00180003o
00<0oh000000o`000?l02@000?l4003o00030000o`00o`000?l000T0003o1000o`050000o`kooooo
RP3oP00Iooooo`005ooooomA0?l00003003o003o0000o`0000L0o`000P00o`0]0?l000kooooo9@00
o`020000oaD00?l00P000?l2003o0080003o1`00o`040000o`<0oh000P000?lP003o0080003o1`00
o`030000o`@00?l00P3o00000`00o`00003o0000o`02003o00kooooo10000?l8003o00@0003o1000
o`030000o`800?l06@000?l2003o00L0003o00<00?l00000o`000?l0=P000?l00`00o`00003o0000
o`090000o`800?l01`000?l>oooooaT0oh000P00o`1_0?n001Wooooo000Goooooe40o`0000<00?l0
0?l0003o00001`3o0002003o02d0o`003_oooolU003o0080003o5@00o`020000o`800?l00P000?l7
003o00@0003o0`3oP0020000ob000?l00P000?l7003o00<0003o1000o`020?l00003003o00000?l0
003o00800?l03_ooool40000o`P00?l010000?l4003o00<0003o0P00o`0I0000o`800?l01`000?l0
0`00o`00003o0000o`0f0000o`03003o00000?l0003o00T0003o0P00o`070000o`kooooo6@3oP002
003o06l0oh006Oooool001Oooooo=P3o00020?n00580o`003_oooolU003o0080003o5@00o`040000
o`L00?l00P3oP0020000o`80oh0000<0003o0?l0003o00002@00o`020000oah00?l01@000?l2003o
0080o`001@000?l2003o00kooooo1`000?l7003o00@0003o0P00o`0L0000o`@00?l01`000?l00`00
o`00003o0000o`02003o0380003o1`00o`020000o`<00?l02`000?l>oooooaT0oh000P00o`1_0?n0
01Wooooo000GooooodH0o`000P00o`120?l000kooooo9000o`000`000?l00?l0003o0002003o0080
003o4`00o`020?l000/00?l01@000?l7003o0080003o5`00o`040000o`D00?l00P000?l3003o00T0
003o0P00o`0>ooooo``0003o2000o`030000o`800?l00P000?l00`00o`00003o0000o`0K0000o`L0
0?l0=@000?l2003o0180003o3_oooon:0?n001Wooooo000GooooodH0o`000P00o`120?l000kooooo
9000o`000`000?l00?l0003o0002003o0080003o4`00o`020?l000/00?l01@000?l7003o0080003o
5`00o`040000o`D00?l00P000?l3003o00T0003o0P00o`0>ooooo``0003o2000o`030000o`800?l0
0P000?l00`00o`00003o0000o`0K0000o`L00?l0=@000?l2003o0180003o3_oooon:0?n001Wooooo
000GooooohX0o`003_oooolT003o00<0003o5000o`090000o`<00?l00P3o00040000o`03003o0000
0?l0003o0080003o7000o`020000o`T00?l00P000?l2003o00X0003o0P00o`0>ooooo`@0003o1@00
o`020000o`T00?l01`000?l00`00o`00003o0000o`0M0000o`03003o00000?l0003o04/0003o3_oo
oon:0?n001Wooooo000GooooohX0o`003_oooolT003o00<0003o5000o`090000o`<00?l00P3o0004
0000o`03003o00000?l0003o0080003o7000o`020000o`T00?l00P000?l2003o00X0003o0P00o`0>
ooooo`@0003o1@00o`020000o`T00?l01`000?l00`00o`00003o0000o`0M0000o`03003o00000?l0
003o04/0003o3_oooon:0?n001Wooooo000GooooohD0o`0000<0oh000?l0003o00000P3o000>oooo
oc/00?l02@000?l:003o00@0003o1@00o`040000oah00?l00P000?l2003o00P0003o1000o`0>oooo
oa80003o0P00o`050000o`800?l070000?l2003o0500003o3_oooon:0?n001Wooooo000GooooohD0
o`0000<0oh000?l0003o00000P3o000>oooooc/00?l02@000?l:003o00@0003o1@00o`040000oah0
0?l00P000?l2003o00P0003o1000o`0>oooooa80003o0P00o`050000o`800?l070000?l2003o0500
003o3_oooon:0?n001Wooooo000Gooooofd0o`00103oP0070?l000800?l02`3o00000`3oP000o`00
0?l000020?l000kooooo>`00o`050000o`800?l00P000?l3003o00L0003o0P3oP0020000o`D00?l0
0P000?l2003o0080003o3P00o`020000o`<00?l00P000?l7003o00T0003o1`00o`0>ooooo`D0003o
0P00o`0[0000o`800?l000<0003o003o0000o`000P00o`0I0000o`80o`006@000?l00`00o`00003o
0000o`0I0000o`koooooRP3oP00Iooooo`005ooooom]0?l000@0oh001`3o0002003o00/0o`0000<0
oh000?l0003o00000P3o000>oooooc/00?l01@000?l2003o0080003o0`00o`070000o`80oh000P00
0?l5003o0080003o0P00o`020000o`h00?l00P000?l3003o0080003o1`00o`090000o`L00?l03_oo
ool50000o`800?l0:`000?l2003o00030000o`00o`000?l000800?l06@000?l20?l001T0003o00<0
0?l00000o`000?l06@000?l>ooooohX0oh006Oooool001Oooooo1`3o00020?n00080o`000P3oP01m
0?l000kooooo>`00o`070000o`D00?l00P000?l2003o0080003o00<00?l00?n0003oP00010000?l0
0`00o`00003o0000o`070000o`@00?l00P000?l00`00o`00003o0000o`020000o`800?l00P000?l0
0`00o`00003o0000o`020000oaL00?l03_oooolB0000o`800?l08@000?l4003o0500003o3_oooon:
0?n001Wooooo000Goooood40o`0000<0oh000?l0003o0000AP3o000>oooooah00?l00P000?lI003o
00T0003o1`00o`090000o`D00?l01P000?l8003o0080003o0P00o`040000oa000?l01@000?l7003o
00kooooo0P000?l5003o00/0003o0`00o`0T0000o`800?l0<P000?l00`00o`00003o0000o`0@0000
o`800?l01`000?l>ooooohX0oh006Oooool001Oooooo@@3o00000`3oP000o`000?l000160?l000ko
oooo7P00o`020000oaT00?l02@000?l7003o00T0003o1@00o`060000o`P00?l00P000?l2003o00@0
003o4000o`050000o`L00?l03_ooool20000o`D00?l02`000?l3003o02@0003o0P00o`0b0000o`03
003o00000?l0003o0100003o0P00o`070000o`koooooRP3oP00Iooooo`005ooooon:0?l000kooooo
>@00o`090000o`T00?l01`000?lE003o0080003o0P00o`020000oa<00?l00P000?l7003o00kooooo
0P000?l9003o00L0003o0P00o`030000o`800?l06`000?l5003o0180003o0P3o00030000o`80oh00
0`000?l2003o0200003o0P00o`0@0000o`koooooRP3oP00Iooooo`005ooooon:0?l000kooooo>@00
o`090000o`T00?l01`000?lE003o0080003o0P00o`020000oa<00?l00P000?l7003o00kooooo0P00
0?l9003o00L0003o0P00o`030000o`800?l06`000?l5003o0180003o0P3o00030000o`80oh000`00
0?l2003o0200003o0P00o`0@0000o`koooooRP3oP00Iooooo`005ooooomA0?l000030?n0003o0000
o`0003H0o`003_oooolN003o00@0003o6@00o`030000o`800?l010000?l5003o00T0003o4`00o`02
0000o`@00?l00P000?l00`00o`00003o0000o`0;003o00D0003o2@00o`0>ooooo`@0003o1@00o`10
0000o`800?l00P3o00080000o`800?l07P000?l2003o0080003o0P00o`0>0000o`koooooO`3oP002
003o00T0oh006Oooool001OoooooD@3o00000`3oP000o`000?l0000f0?l000kooooo7P00o`040000
oaT00?l00`000?l2003o00@0003o1@00o`090000oa<00?l00P000?l4003o0080003o00<00?l00000
o`000?l02`00o`050000o`T00?l03_ooool40000o`D00?l0@0000?l2003o0080o`0020000?l2003o
01h0003o0P00o`020000o`800?l03P000?l>ooooogl0oh000P00o`090?n001Wooooo000Gooooocd0
o`000P3oP01;0?l000kooooo8000o`020000oa<00?l00P000?l4003o00L0003o1`00o`090000o`<0
0?l010000?l2003o0080003o00<00?l00000o`000?l03P00o`020000oaT00?l03_ooool40000o`L0
0?l00`000?l2003o00L0003o0P00o`0@0000o`<0oh000P000?l20?l00080003o0`00o`020000o`80
0?l06@000?l30?l00100003o0P3oP00R0000o`kooooo2`3oP0020?l00700oh001000o`090?n001Wo
oooo000Goooooh<0o`000P3oP0050?l000kooooo7P00o`020000o`800?l00P000?lE003o00/0003o
0`00o`060000o`<00?l00P000?l2003o00D0003o0P00o`020000ob`00?l03_ooool20000o`h00?l0
1`000?l4003o01T0003o2000o`090000o`80o`00;P000?l2003o0180003o3_oooomo0?n000@00?l0
1`3oP00Iooooo`005ooooon30?l00080oh001@3o000>oooooah00?l00P000?l2003o0080003o5@00
o`0;0000o`<00?l01P000?l3003o0080003o0P00o`050000o`800?l00P000?l/003o00kooooo0P00
0?l>003o00L0003o1000o`0I0000o`P00?l02@000?l20?l002h0003o0P00o`0B0000o`koooooO`3o
P004003o00L0oh006Oooool001OoooooRP3o000>oooooah00?l01P000?lA003o0080003o1000o`07
0000o`T00?l01`000?l3003o00@0003o0P00o`020000oaX00?l010000?l2003o00D0003o1`00o`0>
ooooo`@0003o3000o`050000o`800?l07P000?l6003o00030000o`00o`000?l00100003o0P3oP00P
0000o`800?l05`000?l>ooooohX0oh006Oooool001OoooooRP3o000>oooooah00?l01P000?lA003o
0080003o1000o`070000o`T00?l01`000?l3003o00@0003o0P00o`020000oaX00?l010000?l2003o
00D0003o1`00o`0>ooooo`@0003o3000o`050000o`800?l07P000?l6003o00030000o`00o`000?l0
0100003o0P3oP00P0000o`800?l05`000?l>ooooohX0oh006Oooool001OoooooRP3o000>ooooob00
0?l00P000?l>003o0080003o0`00o`020000o`T00?l010000?l7003o00<0003o0P00o`020000o`<0
0?l020000?lH003o00@0003o0P00o`070000o`800?l000<0003o003o0000o`000P00o`0>ooooo`80
003o2P00o`0=0000o`<00?l010000?l2003o0100003o2P00o`0_0000o`800?l00`000?l2003o01L0
003o3_oooon:0?n001Wooooo000GooooohX0o`003_oooolP003o0080003o3P00o`020000o`<00?l0
0P000?l9003o00@0003o1`00o`030000o`800?l00P000?l3003o00P0003o6000o`040000o`800?l0
1`000?l2003o00030000o`00o`000?l000800?l03_ooool20000o`X00?l03@000?l3003o00@0003o
0P00o`0@0000o`X00?l0;`000?l2003o00<0003o0P00o`0G0000o`koooooRP3oP00Iooooo`005ooo
oolb0?l00080oh002@3o00020?n004/0o`003_oooolP003o0080003o9`00o`0;0000o`<00?l01P00
0?l/003o00kooooo10000?l8003o00/0003o1@00o`020000o`800?l010000?l00`00o`00003o0000
o`070000o`800?l01@000?l2003o0080003o1@00o`0d0000o`800?l000<0003o003o0000o`004P00
0?l>oooooc/0oh000P3o001=0?n001Wooooo000Goooooc80o`000P3oP0090?l00080oh00B`3o000>
ooooob000?l00P000?lW003o00/0003o0`00o`060000ob`00?l03_ooool40000o`P00?l02`000?l5
003o0080003o0P00o`040000o`03003o00000?l0003o00L0003o0P00o`050000o`800?l00P000?l5
003o03@0003o0P00o`000`000?l00?l0003o000B0000o`kooooo>`3oP0020?l004d0oh006Oooool0
01Oooooo<P3o00020?n005H0o`003_oooolP003o0080003o4`00o`020000o`D00?l00P000?l2003o
0080003o1`00o`090000obD00?l00P000?l2003o00L0003o1`00o`0>ooooo`80003o2P00o`060000
oa800?l05@000?l5003o02d0003o0P00o`0L0000o`kooooo7@3oP0000`3o0000oh000?n0001Z0?n0
01Wooooo000GooooohX0o`003_oooolk003o00L0003o1@00o`060000o`030?l000000?l0003o0080
003o0`00o`020000o`800?l010000?lQ003o0080003o2@00o`0>ooooo`@0003o0`00o`020000o`<0
0?l00P000?l=003o00030000o`00o`000?l000800?l010000?l00`00o`00003o0000o`0;0000o``0
0?l090000?l3003o00H0003o0P00o`0:0000o`800?l040000?l>oooooa80oh000P000?mf0?n001Wo
oooo000GooooohX0o`003_oooolk003o00L0003o1@00o`060000o`030?l000000?l0003o0080003o
0`00o`020000o`800?l010000?lQ003o0080003o2@00o`0>ooooo`@0003o0`00o`020000o`<00?l0
0P000?l=003o00030000o`00o`000?l000800?l010000?l00`00o`00003o0000o`0;0000o``00?l0
90000?l3003o00H0003o0P00o`0:0000o`800?l040000?l>oooooa80oh000P000?mf0?n001Wooooo
000Gooooo`h0o`000P3oP01j0?l000kooooo6`00o`000`000?l00?l0003o0002003o0080003o6P00
o`020000o`T00?l01`000?l2003o0080003o9@00o`020000o`800?l02@000?l5003o00kooooo0P00
0?l3003o00@0003o4000o`020000o`03003o00000?l0003o00800?l050000?l:003o02@0003o0P00
o`070000o`<00?l06`000?l>ooooohX0oh006Oooool001Oooooo3P3o00020?n007X0o`003_oooolK
003o00030000o`00o`000?l000800?l00P000?lJ003o0080003o2@00o`070000o`800?l00P000?lU
003o0080003o0P00o`090000o`D00?l03_ooool20000o`<00?l010000?l@003o0080003o00<00?l0
0000o`000?l00P00o`0D0000o`X00?l090000?l2003o00L0003o0`00o`0K0000o`koooooRP3oP00I
ooooo`005ooooomm0?l00080oh002`3o000>oooooc`00?l01P000?l5003o0080003o4P00o`020000
o`03003o00000?l0003o0080003o6@00o`0:0000o`@00?l03_ooool40000oaL00?l00`000?l2003o
01D0003o0P00o`020000o`D00?l0B`000?l>ooooohX0oh006Oooool001OoooooO@3o00020?n000/0
o`003_ooooll003o00H0003o1@00o`020000oa800?l00P000?l00`00o`00003o0000o`020000oaT0
0?l02P000?l4003o00kooooo10000?lG003o00<0003o0P00o`0E0000o`800?l00P000?l5003o04/0
003o3_oooon:0?n001Wooooo000GooooohX0o`003_oooold003o00030000o`00o`000?l000D00?l0
0P000?l20?l000h0003o0P00o`020000o`L00?l02@000?l2003o00030000o`00o`000?l001000?l0
0P000?l2003o00D0003o1000o`000`000?l00?l0003o0002003o00kooooo0P000?l9003o00<0003o
1000o`020000o`<00?l02`000?l5003o00T0003o0P00o`020000o`03003o00000?l0003o00@0003o
0`00o`020000o`800?l0A`000?l>ooooohX0oh006Oooool001OoooooRP3o000>oooooc@00?l000<0
003o003o0000o`001@00o`020000o`80o`003P000?l2003o0080003o1`00o`090000o`800?l000<0
003o003o0000o`004000o`020000o`800?l01@000?l4003o00030000o`00o`000?l000800?l03_oo
ool20000o`T00?l00`000?l4003o0080003o0`00o`0;0000o`D00?l02@000?l2003o0080003o00<0
0?l00000o`000?l010000?l3003o0080003o0P00o`170000o`koooooRP3oP00Iooooo`005ooooomf
0?l00080oh001`3o00020?n000T0o`003_ooooll003o00H0003o0P00o`0<0000o`T00?l00P000?lS
003o0080003o2@00o`0>ooooo`800?l00P000?l00`00o`00003o0000o`0=003o00030000o`00o`00
0?l00080003o0P00o`070000o`800?l03P000?l2003o00D0003o1@00o`070000o`800?l0@P000?l>
ooooohX0oh006Oooool001Oooooo;`3o00000`3oP000o`000?l0000K0?l00080oh00<03o00020000
o`T0o`003_ooool2003o0080003o=`00o`030000o`800?l010000?l00`3oP000003o0000o`020000
o`80oh000`00o`020000o`T00?l010000?lN003o0080003o0`00o`020000o`L00?l03_ooool>0000
o`L00?l02`000?l2003o0080003o00<00?l00000o`000?l03@000?l3003o0080003o0`00o`020000
o`800?l0B@000?l>ooooohX0oh006Oooool001Oooooo;`3o00000`3oP000o`000?l0000K0?l00080
oh00<03o00020000o`T0o`003_ooool2003o0080003o=`00o`030000o`800?l010000?l00`3oP000
003o0000o`020000o`80oh000`00o`020000o`T00?l010000?lN003o0080003o0`00o`020000o`L0
0?l03_ooool>0000o`L00?l02`000?l2003o0080003o00<00?l00000o`000?l03@000?l3003o0080
003o0`00o`020000o`800?l0B@000?l>ooooohX0oh006Oooool001OoooooRP3o000>oooooc`00?l0
2`000?l20?n00080003o0P3o000^003o0080003o3000o`0>ooooo`@0003o00<00?l00000o`000?l0
0P00o`050000o`L00?l080000?l2003o00@0003o0`00o`0K0000o`@00?l02P000?l2003o0200003o
3_oooon:0?n001Wooooo000GooooohX0o`003_ooooll003o00/0003o0P3oP0020000o`80o`00;P00
o`020000o``00?l03_ooool40000o`03003o00000?l0003o00800?l01@000?l7003o0200003o0P00
o`040000o`<00?l06`000?l4003o00X0003o0P00o`0P0000o`koooooRP3oP00Iooooo`005ooooon:
0?l000kooooo0P00o`020?l003P00?l03@000?l20?n00080o`003000o`040000o`03003o00000?l0
003o02T00?l03_ooool70000o`800?l00P000?l5003o0080003o0P00o`030000o`@00?l06@000?l0
0`00o`00003o0000o`05003o04d0003o3_oooon:0?n001Wooooo000GooooohX0o`003_ooool2003o
0080o`00>000o`0=0000o`80oh000P3o000<003o00@0003o00<00?l00000o`000?l0:@00o`0>oooo
o`L0003o0P00o`020000o`D00?l00P000?l2003o00<0003o1000o`0I0000o`03003o00000?l0003o
00D00?l0C@000?l>ooooohX0oh006Oooool001Oooooo0P00o`280?l000kooooo?000o`0;0000oa80
0?l010000?l00`00o`00003o0000o`04003o00030000o`00o`000?l002800?l03_ooool40000o`03
003o00000?l0003o00@0003o1@00o`040000o`03003o00000?l0003o00@00?l00`000?l2003o01L0
003o1000o`1>0000o`koooooE03oP002003o03@0oh006Oooool001OoooooB03o00000`3oP000o`00
0?l0000o0?l000kooooo>@00o`0E0000o`800?l03@000?l3003o0080003o9`00o`0>ooooo`@0003o
00<00?l00000o`000?l01`000?l2003o02/0003o00<00?l00000o`000?l0=0000?l2003o00L0003o
0P00o`0<0000o`koooooE03oP002003o03@0oh006Oooool001OoooooB03o00000`3oP000o`000?l0
000o0?l000kooooo>@00o`0E0000o`800?l03@000?l3003o0080003o9`00o`0>ooooo`@0003o00<0
0?l00000o`000?l01`000?l2003o02/0003o00<00?l00000o`000?l0=0000?l2003o00L0003o0P00
o`0<0000o`koooooE03oP002003o03@0oh006Oooool001OoooooRP3o000>oooooc/00?l01@000?l2
003o00L0003o0P3o0002003o00D0003o0P00o`030000o`800?l01`000?lY003o00kooooo2`000?l5
003o02@0003o00<00?l00000o`000?l010000?l00`00o`00003o0000o`0V0000o`03003o00000?l0
003o00L0003o0P00o`090000o`800?l01`000?l2003o00D0003o3_ooool40?n000<00?l0P`3oP00I
ooooo`005ooooon:0?l000kooooo>`00o`050000o`800?l01`000?l20?l000800?l01@000?l2003o
00<0003o0P00o`070000obT00?l03_ooool;0000o`D00?l090000?l00`00o`00003o0000o`040000
o`03003o00000?l0003o02H0003o00<00?l00000o`000?l01`000?l2003o00T0003o0P00o`070000
o`800?l01@000?l>ooooo`@0oh000`00o`230?n001Wooooo003oooooooooooooP_ooool00?oooooo
oooooon2ooooo`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2oooo
o`00oooooooooooooh;ooooo003oooooooooooooP_ooool00?oooooooooooon2ooooo`00oooooooo
oooooh;ooooo0000\
\>"],
  ImageRangeCache->{{{0, 639}, {160.438, 0}} -> {-0.110009, -0.0252326, \
0.0066041, 0.0066041}, {{16.625, 168.062}, {156.562, 3.8125}} -> {-0.140545, \
-0.0521117, 0.00693653, 0.00693653}, {{20.25, 164.375}, {152.875, 83.625}} -> \
{-0.960223, -13.358, 0.0374838, 1.50181}, {{20.25, 164.375}, {76.6875, \
7.4375}} -> {-13.2889, -47.9103, 0.561016, 0.561016}, {{168.062, 319.5}, \
{156.562, 3.8125}} -> {-1.191, -0.0521117, 0.00693653, 0.00693653}, \
{{171.688, 315.812}, {152.875, 83.625}} -> {-6.63668, -13.358, 0.0374838, \
1.50181}, {{171.688, 315.812}, {76.6875, 7.4375}} -> {-98.2478, -47.9103, \
0.561016, 0.561016}, {{319.5, 470.875}, {156.562, 3.8125}} -> {-2.24123, \
-0.0521117, 0.00693653, 0.00693653}, {{323.062, 467.25}, {152.875, 83.625}} -> \
{-12.312, -13.358, 0.0374838, 1.50181}, {{323.062, 467.25}, {76.6875, \
7.4375}} -> {-183.189, -47.9103, 0.561016, 0.561016}, {{470.875, 622.312}, \
{156.562, 3.8125}} -> {-3.29146, -0.0521117, 0.00693653, 0.00693653}, \
{{474.5, 618.625}, {152.875, 83.625}} -> {-17.9872, -13.358, 0.0374838, \
1.50181}, {{474.5, 618.625}, {76.6875, 7.4375}} -> {-268.131, -47.9103, \
0.561016, 0.561016}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Summary of percent correct classification", "Subsection"],

Cell[CellGroupData[{

Cell["\<\
By band (1 - HL1, 2 - LH1, 3 - HH1, 4 - HL2, 5 - LH2, 6 - \
HH3)\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(correctbyband = 
        Table[\(Map[First, results, {2}]\)[\([j, i, j]\)], {j, 1, 
            nImages}, {i, 1, nBands}];\)\), "\n", 
    \(\(BarChart[Sequence\  @@ \ correctbyband, BarStyle \[Rule] colors, 
        BarEdgeStyle \[Rule] colors, \ Frame \[Rule] True, \ 
        GridLines \[Rule] {None, Automatic}, 
        ImageSize \[Rule] 400];\)\)}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
%%ImageSize: 400 247.214 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.112903 0.16129 0.0147151 0.00594868 [
[.09677 -0.0125 -3 -9 ]
[.09677 -0.0125 3 0 ]
[.25806 -0.0125 -3 -9 ]
[.25806 -0.0125 3 0 ]
[.41935 -0.0125 -3 -9 ]
[.41935 -0.0125 3 0 ]
[.58065 -0.0125 -3 -9 ]
[.58065 -0.0125 3 0 ]
[.74194 -0.0125 -3 -9 ]
[.74194 -0.0125 3 0 ]
[.90323 -0.0125 -3 -9 ]
[.90323 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .01472 -6 -4.5 ]
[-0.0125 .01472 0 4.5 ]
[-0.0125 .13369 -12 -4.5 ]
[-0.0125 .13369 0 4.5 ]
[-0.0125 .25266 -12 -4.5 ]
[-0.0125 .25266 0 4.5 ]
[-0.0125 .37164 -12 -4.5 ]
[-0.0125 .37164 0 4.5 ]
[-0.0125 .49061 -12 -4.5 ]
[-0.0125 .49061 0 4.5 ]
[-0.0125 .60958 -18 -4.5 ]
[-0.0125 .60958 0 4.5 ]
[ 0 0 -0.125 0 ]
[.09677 .63053 -3 0 ]
[.09677 .63053 3 9 ]
[.25806 .63053 -3 0 ]
[.25806 .63053 3 9 ]
[.41935 .63053 -3 0 ]
[.41935 .63053 3 9 ]
[.58065 .63053 -3 0 ]
[.58065 .63053 3 9 ]
[.74194 .63053 -3 0 ]
[.74194 .63053 3 9 ]
[.90323 .63053 -3 0 ]
[.90323 .63053 3 9 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
0 .13369 m
1 .13369 L
s
0 .25266 m
1 .25266 L
s
0 .37164 m
1 .37164 L
s
0 .49061 m
1 .49061 L
s
0 g
.09677 0 m
.09677 .00625 L
s
[(1)] .09677 -0.0125 0 1 Mshowa
.25806 0 m
.25806 .00625 L
s
[(2)] .25806 -0.0125 0 1 Mshowa
.41935 0 m
.41935 .00625 L
s
[(3)] .41935 -0.0125 0 1 Mshowa
.58065 0 m
.58065 .00625 L
s
[(4)] .58065 -0.0125 0 1 Mshowa
.74194 0 m
.74194 .00625 L
s
[(5)] .74194 -0.0125 0 1 Mshowa
.90323 0 m
.90323 .00625 L
s
[(6)] .90323 -0.0125 0 1 Mshowa
0 0 m
1 0 L
s
0 .01472 m
.00625 .01472 L
s
[(0)] -0.0125 .01472 1 0 Mshowa
0 .13369 m
.00625 .13369 L
s
[(20)] -0.0125 .13369 1 0 Mshowa
0 .25266 m
.00625 .25266 L
s
[(40)] -0.0125 .25266 1 0 Mshowa
0 .37164 m
.00625 .37164 L
s
[(60)] -0.0125 .37164 1 0 Mshowa
0 .49061 m
.00625 .49061 L
s
[(80)] -0.0125 .49061 1 0 Mshowa
0 .60958 m
.00625 .60958 L
s
[(100)] -0.0125 .60958 1 0 Mshowa
.125 Mabswid
0 .04446 m
.00375 .04446 L
s
0 .0742 m
.00375 .0742 L
s
0 .10395 m
.00375 .10395 L
s
0 .16343 m
.00375 .16343 L
s
0 .19318 m
.00375 .19318 L
s
0 .22292 m
.00375 .22292 L
s
0 .28241 m
.00375 .28241 L
s
0 .31215 m
.00375 .31215 L
s
0 .34189 m
.00375 .34189 L
s
0 .40138 m
.00375 .40138 L
s
0 .43112 m
.00375 .43112 L
s
0 .46087 m
.00375 .46087 L
s
0 .52035 m
.00375 .52035 L
s
0 .5501 m
.00375 .5501 L
s
0 .57984 m
.00375 .57984 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.09677 .61178 m
.09677 .61803 L
s
[(1)] .09677 .63053 0 -1 Mshowa
.25806 .61178 m
.25806 .61803 L
s
[(2)] .25806 .63053 0 -1 Mshowa
.41935 .61178 m
.41935 .61803 L
s
[(3)] .41935 .63053 0 -1 Mshowa
.58065 .61178 m
.58065 .61803 L
s
[(4)] .58065 .63053 0 -1 Mshowa
.74194 .61178 m
.74194 .61803 L
s
[(5)] .74194 .63053 0 -1 Mshowa
.90323 .61178 m
.90323 .61803 L
s
[(6)] .90323 .63053 0 -1 Mshowa
0 .61803 m
1 .61803 L
s
.99375 .01472 m
1 .01472 L
s
.99375 .13369 m
1 .13369 L
s
.99375 .25266 m
1 .25266 L
s
.99375 .37164 m
1 .37164 L
s
.99375 .49061 m
1 .49061 L
s
.99375 .60958 m
1 .60958 L
s
.125 Mabswid
.99625 .04446 m
1 .04446 L
s
.99625 .0742 m
1 .0742 L
s
.99625 .10395 m
1 .10395 L
s
.99625 .16343 m
1 .16343 L
s
.99625 .19318 m
1 .19318 L
s
.99625 .22292 m
1 .22292 L
s
.99625 .28241 m
1 .28241 L
s
.99625 .31215 m
1 .31215 L
s
.99625 .34189 m
1 .34189 L
s
.99625 .40138 m
1 .40138 L
s
.99625 .43112 m
1 .43112 L
s
.99625 .46087 m
1 .46087 L
s
.99625 .52035 m
1 .52035 L
s
.99625 .5501 m
1 .5501 L
s
.99625 .57984 m
1 .57984 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .01472 m
1 .01472 L
s
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.03226 .01472 m
.03226 .48117 L
.06452 .48117 L
.06452 .01472 L
F
.5 Mabswid
.03226 .01472 m
.03226 .48117 L
.06452 .48117 L
.06452 .01472 L
.03226 .01472 L
s
.19355 .01472 m
.19355 .60332 L
.22581 .60332 L
.22581 .01472 L
F
.19355 .01472 m
.19355 .60332 L
.22581 .60332 L
.22581 .01472 L
.19355 .01472 L
s
.35484 .01472 m
.35484 .56845 L
.3871 .56845 L
.3871 .01472 L
F
.35484 .01472 m
.35484 .56845 L
.3871 .56845 L
.3871 .01472 L
.35484 .01472 L
s
.51613 .01472 m
.51613 .46969 L
.54839 .46969 L
.54839 .01472 L
F
.51613 .01472 m
.51613 .46969 L
.54839 .46969 L
.54839 .01472 L
.51613 .01472 L
s
.67742 .01472 m
.67742 .45716 L
.70968 .45716 L
.70968 .01472 L
F
.67742 .01472 m
.67742 .45716 L
.70968 .45716 L
.70968 .01472 L
.67742 .01472 L
s
.83871 .01472 m
.83871 .50769 L
.87097 .50769 L
.87097 .01472 L
F
.83871 .01472 m
.83871 .50769 L
.87097 .50769 L
.87097 .01472 L
.83871 .01472 L
s
0 1 0 r
.06452 .01472 m
.06452 .29158 L
.09677 .29158 L
.09677 .01472 L
F
.06452 .01472 m
.06452 .29158 L
.09677 .29158 L
.09677 .01472 L
.06452 .01472 L
s
.22581 .01472 m
.22581 .33042 L
.25806 .33042 L
.25806 .01472 L
F
.22581 .01472 m
.22581 .33042 L
.25806 .33042 L
.25806 .01472 L
.22581 .01472 L
s
.3871 .01472 m
.3871 .39828 L
.41935 .39828 L
.41935 .01472 L
F
.3871 .01472 m
.3871 .39828 L
.41935 .39828 L
.41935 .01472 L
.3871 .01472 L
s
.54839 .01472 m
.54839 .3703 L
.58065 .3703 L
.58065 .01472 L
F
.54839 .01472 m
.54839 .3703 L
.58065 .3703 L
.58065 .01472 L
.54839 .01472 L
s
.70968 .01472 m
.70968 .27321 L
.74194 .27321 L
.74194 .01472 L
F
.70968 .01472 m
.70968 .27321 L
.74194 .27321 L
.74194 .01472 L
.70968 .01472 L
s
.87097 .01472 m
.87097 .3536 L
.90323 .3536 L
.90323 .01472 L
F
.87097 .01472 m
.87097 .3536 L
.90323 .3536 L
.90323 .01472 L
.87097 .01472 L
s
0 0 1 r
.09677 .01472 m
.09677 .349 L
.12903 .349 L
.12903 .01472 L
F
.09677 .01472 m
.09677 .349 L
.12903 .349 L
.12903 .01472 L
.09677 .01472 L
s
.25806 .01472 m
.25806 .34336 L
.29032 .34336 L
.29032 .01472 L
F
.25806 .01472 m
.25806 .34336 L
.29032 .34336 L
.29032 .01472 L
.25806 .01472 L
s
.41935 .01472 m
.41935 .2826 L
.45161 .2826 L
.45161 .01472 L
F
.41935 .01472 m
.41935 .2826 L
.45161 .2826 L
.45161 .01472 L
.41935 .01472 L
s
.58065 .01472 m
.58065 .1377 L
.6129 .1377 L
.6129 .01472 L
F
.58065 .01472 m
.58065 .1377 L
.6129 .1377 L
.6129 .01472 L
.58065 .01472 L
s
.74194 .01472 m
.74194 .35193 L
.77419 .35193 L
.77419 .01472 L
F
.74194 .01472 m
.74194 .35193 L
.77419 .35193 L
.77419 .01472 L
.74194 .01472 L
s
.90323 .01472 m
.90323 .49015 L
.93548 .49015 L
.93548 .01472 L
F
.90323 .01472 m
.90323 .49015 L
.93548 .49015 L
.93548 .01472 L
.90323 .01472 L
s
1 .5 0 r
.12903 .01472 m
.12903 .42292 L
.16129 .42292 L
.16129 .01472 L
F
.12903 .01472 m
.12903 .42292 L
.16129 .42292 L
.16129 .01472 L
.12903 .01472 L
s
.29032 .01472 m
.29032 .43252 L
.32258 .43252 L
.32258 .01472 L
F
.29032 .01472 m
.29032 .43252 L
.32258 .43252 L
.32258 .01472 L
.29032 .01472 L
s
.45161 .01472 m
.45161 .33522 L
.48387 .33522 L
.48387 .01472 L
F
.45161 .01472 m
.45161 .33522 L
.48387 .33522 L
.48387 .01472 L
.45161 .01472 L
s
.6129 .01472 m
.6129 .37886 L
.64516 .37886 L
.64516 .01472 L
F
.6129 .01472 m
.6129 .37886 L
.64516 .37886 L
.64516 .01472 L
.6129 .01472 L
s
.77419 .01472 m
.77419 .5553 L
.80645 .5553 L
.80645 .01472 L
F
.77419 .01472 m
.77419 .5553 L
.80645 .5553 L
.80645 .01472 L
.77419 .01472 L
s
.93548 .01472 m
.93548 .33334 L
.96774 .33334 L
.96774 .01472 L
F
.93548 .01472 m
.93548 .33334 L
.96774 .33334 L
.96774 .01472 L
.93548 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{400, 247.188},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006@0000mb000`400?l00000o`00003ooooooonA
ooooo`00@Oooool3o`0003Gooooo1Ol0000fooooo`?o0000=oooool3o`0003Gooooo0ol0000foooo
o`?o0000;_ooool004;ooooo00?o0000ooooooooool0=Oooool01?l0003oooooooooool0000eoooo
o`05o`000?ooooooooooooooool00000=oooool00ol0003oooooooooo`0cooooo`05o`000?oooooo
ooooooooool00000=?ooool01Ol0003ooooooooooooooooo000002gooooo0012ooooo`03o`000?oo
oooooooo03Kooooo00?o0000ooooooooool0>Oooool00ol0003oooooooooo`0booooo`Go0000>?oo
ool00ol0003oooooooooo`0booooo`05o`000?ooooooooooooooool00000;Oooool004;ooooo00?o
0000ooooooooool0=oooool00ol0003oooooooooo`0fooooo`;o0000=Oooool01?l0003ooooooooo
ool0000iooooo`03o`000?oooooooooo03;ooooo1?l0000^ooooo`00@_ooool00ol0003ooooooooo
o`0dooooo`05o`000?ooooooooooooooool00000>Oooool00ol0003oooooooooo`0cooooo`03o`00
0?oooooo000003Gooooo1?l0000eooooo`03o`000?oooooooooo02oooooo0011ooooo`;o0000=_oo
ool01Ol0003ooooooooooooooooo000003Gooooo00Go0000ooooooooooooooooo`00000fooooo`;o
0000=Oooool00ol0003oooooooooo`0gooooo`03o`000?oooooooooo02kooooo0012ooooo`03o`00
0?oooooooooo03Gooooo0ol0000gooooo`?o0000>?ooool00ol0003oooooooooo`0cooooo`Go0000
=_ooool2o`0002kooooo003ooooooi7ooooo003ooooooi7ooooo003ooooooi7ooooo003ooooooi7o
oooo003ooooooi7ooooo003ooooooi7ooooo000Qoooooooo0000IOl0000;ooooo`008Oooool00ol0
003oooooooooo`0Oooooo`03o`000?oooooooooo03Kooooo00?o0000ooooooooool0=oooool00ol0
003oooooooooo`0fooooo`03o`000?oooooooooo03Kooooo00?o0000ooooooooool0=_ooool00ol0
003oooooooooo`0Pooooo`03o`000?oooooooooo00Wooooo000Hooooo`;o00001oooool00ol0003o
ooooooooo`0Oooooo`03o`000?oooooooooo03Kooooo00?o0000ooooooooool0=oooool00ol0003o
ooooooooo`0fooooo`03o`000?oooooooooo03Kooooo00?o0000ooooooooool0=_ooool00ol0003o
ooooooooo`0Pooooo`03o`000?oooooooooo00Wooooo000Gooooo`04o`000?ooooooooooo`0000Ko
oooo00?o0000ooooooooool0oooooomQooooo`03o`000?oooooooooo00Wooooo000Gooooo`04o`00
0?ooooooooooo`0000Kooooo00?o0000ooooooooool0oooooomQooooo`03o`000?oooooooooo00Wo
oooo000Gooooo`04o`000?ooooooooooo`0000Kooooo2ol0000<ool000_o0?l03?l00?l<oomo00[o
00003?oo000;o`3o00co003o3?ooO`0;o`0000_oo`003?l0o`0;o`00o`coogl02ol0000;ool000co
0?l02ol00?l=oomo00[o00003?oo000;o`3o00_o003o3OooO`0:o`0000coo`002ol0o`0<o`00o`co
ogl03?l0000;ooooo`005oooool01?l0003oooooooooool00006ooooo`03o`000?oooooooooo00So
oooo3?oo000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool;ool000co
0?l02ol00?l<oomo00_ooooo2ooo000<o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0;o`00o`go
ogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000ooooooooool02Oooool001Oooooo
00Co0000oooooooooooo00001_ooool00ol0003oooooooooo`08ooooo`coo`002ol0o`0<o`00o`co
ogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o00_o003o3?ooO`0;ooooo`_o
o`003?l0o`0;o`00o`googl02_ooool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00co
003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Hooooo`;o00001oooool00ol0003ooooo
ooooo`08ooooo`coo`002ol0o`0<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo
2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`googl02_ooool<ool000_o0?l0
2ol00?l=oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo
000Qooooo`03o`000?oooooooooo00Sooooo3?oo000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0
o`0<o`00o`coogl02oooool;ool000co0?l02ol00?l<oomo00_ooooo2ooo000<o`3o00_o003o3Ooo
O`0:ooooo`coo`002ol0o`0;o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o
0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool02?ooool<ool000_o0?l03?l00?l<
oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;
ool000co0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<
o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08oooo
o`coo`002ol0o`0<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o
00_o003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`googl02_ooool<ool000_o0?l02ol00?l=oomo
00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03
o`000?oooooooooo00Sooooo3?oo000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`co
ogl02oooool;ool000co0?l02ol00?l<oomo00_ooooo2ooo000<o`3o00_o003o3OooO`0:ooooo`co
o`002ol0o`0;o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000oooooooo
ool02Oooool0027ooooo00?o0000ooooooooool02?ooool<ool000_o0?l03?l00?l<oomo00[ooooo
3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;ool000co0?l0
2ol00?l=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl0
2oooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08ooooo`coo`002ol0
o`0<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o00_o003o3?oo
O`0;ooooo`_oo`003?l0o`0;o`00o`googl02_ooool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo
000;o`3o00co003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooo
oooo00Sooooo3?oo000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool;
ool000co0?l02ol00?l<oomo00_ooooo2ooo000<o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0;
o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000ooooooooool02Oooool0
027ooooo0_l00009ooooo`coo`002ol0o`0<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo
00_ooooo2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`googl02_ooool<ool0
00_o0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00co003o3?ooO`09ooooo`?o00002oooool0027o
oooo00?o0000ooooooooool02?ooool<ool000_o0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co
003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;ool000co0?l02ol00?l=oomo00[o
oooo3?oo000;o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool00ol0003o
ooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08ooooo`coo`002ol0o`0<o`00o`coogl0
2_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`00
3?l0o`0;o`00o`googl02_ooool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00co003o
3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3?oo
000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool;ool000co0?l02ol0
0?l<oomo00_ooooo2ooo000<o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0;o`00o`googl02_oo
ool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000
ooooooooool02?ooool<ool000_o0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;
ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;ool000co0?l02ol00?l=oomo00[ooooo3?oo000;
o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09
ooooo`008Oooool00ol0003oooooooooo`08ooooo`coo`002ol0o`0<o`00o`coogl02_ooool<ool0
00_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00
o`googl02_ooool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;oooo
o`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3?oo000;o`3o00co
003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool;ool000co0?l02ol00?l<oomo00_o
oooo2ooo000<o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0;o`00o`googl02_ooool<ool000_o
0?l03?l00?l<oomo00_ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool0
2?ooool<ool000_o0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`00
3?l0o`0;o`00o`coogl02oooool;ool000co0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00_o003o
3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`008Ooo
ool00ol0003oooooooooo`08ooooo`coo`002ol0o`0<o`00o`coogl02_ooool<ool000_o0?l03?l0
0?l<oomo00_ooooo2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`googl02_oo
ool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`03o`000?oo
oooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3?oo000;o`3o00co003o3?ooO`0:
ooooo`coo`002ol0o`0<o`00o`coogl02oooool;ool000co0?l02ol00?l<oomo00_ooooo2ooo000<
o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0;o`00o`googl02_ooool<ool000_o0?l03?l00?l<
oomo00_ooooo00?o0000ooooooooool02Oooool0027ooooo0_l00009ooooo`coo`002ol0o`0<o`00
o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o00_o003o3?ooO`0;oooo
o`_oo`003?l0o`0;o`00o`googl02_ooool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo000;o`3o
00co003o3?ooO`09ooooo`?o00002oooool0027ooooo00?o0000ooooooooool02?ooool<ool000_o
0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`co
ogl02oooool;ool000co0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`co
o`002ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003ooooo
ooooo`08ooooo`coo`002ol0o`0<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo
2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`googl02_ooool<ool000_o0?l0
2ol00?l=oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo
000Qooooo`03o`000?oooooooooo00Sooooo3?oo000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0
o`0<o`00o`coogl02oooool;ool000co0?l02ol00?l<oomo00_ooooo2ooo000<o`3o00_o003o3Ooo
O`0:ooooo`coo`002ol0o`0;o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o
0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool02?ooool<ool000_o0?l03?l00?l<
oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;
ool000co0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<
o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08oooo
o`coo`002ol0o`0<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o
00_o003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`googl02_ooool<ool000_o0?l02ol00?l=oomo
00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03
o`000?oooooooooo00Sooooo3?oo000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`co
ogl02oooool;ool000co0?l02ol00?l<oomo00_ooooo2ooo000<o`3o00_o003o3OooO`0:ooooo`co
o`002ol0o`0;o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000oooooooo
ool02Oooool0027ooooo00?o0000ooooooooool02?ooool<ool000_o0?l03?l00?l<oomo00[ooooo
3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;ool000co0?l0
2ol00?l=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl0
2oooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08ooooo`coo`002ol0
o`0<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o00_o003o3?oo
O`0;ooooo`_oo`003?l0o`0;o`00o`googl02_ooool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo
000;o`3o00co003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooo
oooo00Sooooo3?oo000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool;
ool000co0?l02ol00?l<oomo00_ooooo2ooo000<o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0;
o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000ooooooooool02Oooool0
027ooooo00?o0000ooooooooool02?ooool<ool000_o0?l03?l00?l<oomo00[ooooo3?oo000;o`3o
00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;ool000co0?l02ol00?l=oomo
00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool00ol0
003oooooooooo`09ooooo`008Oooool2o`0000Wooooo3?oo000;o`3o00co003o3?ooO`0:ooooo`co
o`002ol0o`0<o`00o`coogl02oooool;ool000co0?l02ol00?l<oomo00_ooooo2ooo000<o`3o00_o
003o3OooO`0:ooooo`coo`002ol0o`0;o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo00Wo
oooo0ol0000;ooooo`008Oooool00ol0003oooooooooo`08ooooo`coo`002ol0o`0<o`00o`coogl0
2_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`00
3?l0o`0;o`00o`googl02_ooool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00co003o
3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3?oo
000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool;ool000co0?l02ol0
0?l<oomo00_ooooo2ooo000<o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0;o`00o`googl02_oo
ool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000
ooooooooool02?ooool<ool000_o0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;
ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;ool000co0?l02ol00?l=oomo00[ooooo3?oo000;
o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09
ooooo`008Oooool00ol0003oooooooooo`08ooooo`coo`002ol0o`0<o`00o`coogl02_ooool<ool0
00_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00
o`googl02_ooool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;oooo
o`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3?oo000;o`3o00co
003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool;ool000co0?l02ol00?l<oomo00_o
oooo2ooo000<o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0;o`00o`googl02_ooool<ool000_o
0?l03?l00?l<oomo00_ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool0
2?ooool<ool000_o0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`00
3?l0o`0;o`00o`coogl02oooool;ool000co0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00_o003o
3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`008Ooo
ool00ol0003oooooooooo`08ooooo`coo`002ol0o`0<o`00o`coogl02_ooool<ool000_o0?l03?l0
0?l<oomo00_ooooo2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`googl02_oo
ool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`03o`000?oo
oooooooo00Wooooo000@ooooo`Go00000oooool2o`0000Oooooo00?o0000ooooooooool02?ooool<
ool000_o0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;
o`00o`coogl02oooool;ool000co0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:
ooooo`coo`002ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`004Oooool01?l0
003oooooooooool00002ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool02?oo
ool<ool000_o0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0
o`0;o`00o`coogl02oooool;ool000co0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00_o003o3Ooo
O`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`004_ooool0
0ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000Kooooo0ol00008o`00O`coo`002ol0
o`0<o`00o`coogl02_l007l<ool000_o0?l03?l00?l<oomo00_o001o2ooo000<o`3o00_o003o3?oo
O`0;o`00O`_oo`003?l0o`0;o`00o`googl02_l007l<ool000_o0?l02ol00?l=oomo00[o001o3?oo
000;o`3o00co003o3?ooO`08o`00O`Co00002oooool001?ooooo00Go0000ooooooooooooooooo`00
0002ooooo`03o`000?oooooooooo00Cooooo00?o0000ooooooooool02?ooool<ool000_o0?l03?l0
0?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`coogl02ooo
ool;ool000co0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`coo`002ol0
o`0<o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`004?ooool01Ol0003ooooooooooooo
oooo000000;ooooo00Co0000oooooooooooo00001_ooool00ol0003oooooooooo`08ooooo`coo`00
2ol0o`0<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o00_o003o
3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`googl02_ooool<ool000_o0?l02ol00?l=oomo00[ooooo
3?oo000;o`3o00co003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000@ooooo`05o`000?oo
ooooooooooooool000000_ooool01?l0003oooooooooool00006ooooo`03o`000?oooooooooo00So
oooo3?oo000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool;ool000co
0?l02ol00?l<oomo00_ooooo2ooo000=o`3o00[ooooo3OooO`0:ooooo`coo`002ol0o`0;o`00o`go
ogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000ooooooooool02Oooool0017ooooo
0ol00004ooooo`;o00001oooool00ol0003oooooooooo`08ooooo`coo`002ol0o`0<o`00o`coogl0
2_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`00
3Ol0o`0:ooooo`googl02_ooool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00co003o
3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3?oo
000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool;ool000co0?l02ol0
0?l<oomo00_ooooo2ooo000=o`3o00[ooooo3OooO`0:ooooo`coo`002ol0o`0;o`00o`googl02_oo
ool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000
ooooooooool02?ooool<ool000_o0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;
ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;ool000go0?l02_ooool=oomo00[ooooo3?oo000;
o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09
ooooo`008Oooool00ol0003oooooooooo`08ooooo`coo`002ol0o`0<o`00o`coogl02_ooool<ool0
00_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`003Ol0o`0:oooo
o`googl02_ooool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;oooo
o`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3?oo000;o`3o00co
003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool;ool000co0?l02ol00?l<oomo00_o
oooo2ooo000=o`3o00[ooooo3OooO`0:ooooo`coo`002ol0o`0;o`00o`googl02_ooool<ool000_o
0?l03?l00?l<oomo00_ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool0
2?ooool<ool000_o0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`00
3?l0o`0;o`00o`coogl02oooool;ool000go0?l02_ooool=oomo00[ooooo3?oo000;o`3o00_o003o
3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`008Ooo
ool00ol0003oooooooooo`08ooooo`coo`002ol0o`0<o`00o`coogl02_ooool<ool000_o0?l03?l0
0?l<oomo00_ooooo2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`003Ol0o`0:ooooo`googl02_oo
ool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`03o`000?oo
oooooooo00Wooooo000Qooooo`;o00002Oooool<ool000_o0?l03?l00?l<oomo00[ooooo3?oo000;
o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;ool000go0?l02_ooool=
oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02Oooool3
o`0000_ooooo000Qooooo`03o`000?oooooooooo00Sooooo3?oo000;o`3o00co003o3?ooO`0:oooo
o`coo`002ol0o`0<o`00o`coogl02oooool;ool000co0?l02ol00?l<oomo00_ooooo2ooo000=o`3o
00[ooooo3OooO`0:ooooo`coo`002ol0o`0;o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo
00_ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool02?ooool<ool000_o
0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`co
ogl02oooool;ool000go0?l02_ooool=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`co
o`002ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003ooooo
ooooo`08ooooo`coo`002ol0o`0<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo
2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`003Ol0o`0:ooooo`googl02_ooool<ool000_o0?l0
2ol00?l=oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo
000Qooooo`03o`000?oooooooooo00Sooooo3?oo000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0
o`0<o`00o`coogl02oooool;ool000co0?l02ol00?l<oomo00_ooooo2ooo000=o`3o00[ooooo3Ooo
O`0:ooooo`coo`002ol0o`0;o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o
0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool02?ooool<ool000_o0?l03?l00?l<
oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;
ool000go0?l02_ooool=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<
o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08oooo
o`coo`002ol0o`0<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o
00_o003o3?ooO`0;ooooo`_oo`003Ol0o`0:ooooo`googl02_ooool<ool000_o0?l02ol00?l=oomo
00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03
o`000?oooooooooo00Sooooo3?oo000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`co
ogl02oooool;ool000co0?l02ol00?l<oomo00_ooooo2ooo000=o`3o00[ooooo3OooO`0:ooooo`co
o`002ol0o`0;o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000oooooooo
ool02Oooool0027ooooo00?o0000ooooooooool02?ooool<ool000_o0?l03?l00?l<oomo00[ooooo
3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;ool000go0?l0
2_ooool=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl0
2oooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08ooooo`coo`002ol0
o`0<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o00_o003o3?oo
O`0;ooooo`_oo`003Ol0o`0:ooooo`googl02_ooool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo
000;o`3o00co003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Qooooo`;o00002Oooool<
ool000_o0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;
o`00o`coogl02oooool;ool000go0?l02_ooool=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:
ooooo`coo`002ol0o`0<o`00o`coogl02Oooool3o`0000_ooooo000Qooooo`03o`000?oooooooooo
00Sooooo3?oo000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool;ool0
00co0?l02ol00?l<oomo00_ooooo2ooo000=o`3o00[ooooo3OooO`0:ooooo`coo`002ol0o`0;o`00
o`googl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000ooooooooool02Oooool0027o
oooo00?o0000ooooooooool02?ooool<ool000_o0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co
003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;ool000go0?l02_ooool=oomo00[o
oooo3?oo000;o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool00ol0003o
ooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08ooooo`coo`002ol0o`0<o`00o`coogl0
2_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`00
3Ol0o`0:ooooo`googl02_ooool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00co003o
3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3?oo
000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool;ool000co0?l02ol0
0?l<oomo00_ooooo2ooo000=o`3o00[ooooo3OooO`0:ooooo`coo`002ol0o`0;o`00o`googl02_oo
ool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000
ooooooooool02?ooool<ool000_o0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;
ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;ool000go0?l02_ooool=oomo00[ooooo3?oo000;
o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09
ooooo`008Oooool00ol0003oooooooooo`08ooooo`coo`002ol0o`0<o`00o`coogl02_ooool<ool0
00_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`003Ol0o`0:oooo
o`googl02_ooool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;oooo
o`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3?oo000;o`3o00co
003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool;ool000co0?l02ol00?l<oomo00_o
oooo2ooo000=o`3o00[ooooo3OooO`0:ooooo`coo`002ol0o`0;o`00o`googl02_ooool<ool000_o
0?l03?l00?l<oomo00_ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool0
2?ooool<ool000_o0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`00
3?l0o`0;o`00o`coogl02oooool;ool000go0?l02_ooool=oomo00[ooooo3?oo000;o`3o00_o003o
3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`008Ooo
ool00ol0003oooooooooo`08ooooo`coo`002ol0o`0<o`00o`coogl02_ooool<ool000_o0?l03?l0
0?l<oomo00_ooooo2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`003Ol0o`0:ooooo`googl02_oo
ool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`03o`000?oo
oooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3?oo000;o`3o00co003o3?ooO`0:
ooooo`coo`002ol0o`0<o`00o`coogl02oooool;ool000co0?l02ol00?l<oomo00_ooooo2ooo000=
o`3o00[ooooo3OooO`0:ooooo`coo`002ol0o`0;o`00o`googl02_ooool<ool000_o0?l03?l00?l<
oomo00_ooooo00?o0000ooooooooool02Oooool0027ooooo0_l00009ooooo`coo`002ol0o`0<o`00
o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o00_o003o3?ooO`0;oooo
o`_oo`003Ol0o`0:ooooo`googl02_ooool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo000;o`3o
00co003o3?ooO`09ooooo`?o00002oooool0027ooooo00?o0000ooooooooool02?ooool<ool000_o
0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`co
ogl02oooool;ool000go0?l02_ooool=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`co
o`002ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003ooooo
ooooo`08ooooo`coo`002ol0o`0<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo
2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`003Ol0o`0:ooooo`googl02_ooool<ool000_o0?l0
2ol00?l=oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo
000Qooooo`03o`000?oooooooooo00Sooooo3?oo000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0
o`0<o`00o`coogl02oooool;ool000co0?l02ol00?l<oomo00_ooooo2ooo000=o`3o00[ooooo3Ooo
O`0:ooooo`coo`002ol0o`0;o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o
0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool02?ooool<ool000_o0?l03?l00?l<
oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;
ool000go0?l02_ooool=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<
o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08oooo
o`coo`002ol0o`0<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o
00_o003o3?ooO`0;ooooo`_oo`003Ol0o`0:ooooo`googl02_ooool<ool000_o0?l02ol00?l=oomo
00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03
o`000?oooooooooo00Sooooo3?oo000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`co
ogl02oooool;ool000co0?l02ol00?l<oomo00_ooooo2ooo000=o`3o00[ooooo3OooO`0:ooooo`co
o`002ol0o`0;o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000oooooooo
ool02Oooool0027ooooo00?o0000ooooooooool02?ooool<ool000_o0?l03?l00?l<oomo00[ooooo
3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;ool000go0?l0
2_ooool=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl0
2oooool00ol0003oooooooooo`09ooooo`004_ooool3o`0000?ooooo0_l00007ooooo`03o`000?oo
oooooooo00Sooooo3?oo000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02ooo
ool;ool000co0?l02ol00?l<oomo00_ooooo2ooo000=o`3o00[ooooo3OooO`0:ooooo`coo`002ol0
o`0;o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000ooooooooool02Ooo
ool001?ooooo00Go0000ooooooooooooooooo`000002ooooo`03o`000?oooooooooo00Cooooo00?o
0000ooooooooool02?ooool<ool000_o0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?oo
O`0;ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;ool000go0?l02_ooool=oomo00[ooooo3?oo
000;o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool00ol0003ooooooooo
o`09ooooo`004?ooool5o`0000;ooooo00Co0000oooooooooooo00001_ooool3o`0000So001o3?oo
000;o`3o00co003o3?ooO`0:o`00O`coo`002ol0o`0<o`00o`coogl02ol007l;ool000co0?l02ol0
0?l<oomo00_o001o2ooo000=o`3o00[o001o3OooO`0:o`00O`coo`002ol0o`0;o`00o`googl02_l0
07l<ool000_o0?l03?l00?l<oomo00So001o1?l0000;ooooo`004?ooool01?l0003oooooooooool0
0003ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool02?ooool<ool000_o0?l0
3?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`coogl0
2oooool;ool000go0?l02_ooool=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`coo`00
2ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`004Oooool00ol0003oooooo`00
0003ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool02?ooool<ool000_o0?l0
3?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`coogl0
2oooool;ool000go0?l02_ooool=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`coo`00
2ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`004_ooool2o`0000?ooooo00Co
0000oooooooooooo00001_ooool00ol0003oooooooooo`08ooooo`coo`002ol0o`0<o`00o`coogl0
2_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`00
3Ol0o`0:ooooo`googl02_ooool<ool000_o0?l02ol00?l=oomo00[ooooo3?oo000;o`3o00co003o
3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Cooooo`03o`000?oooooooooo00;ooooo0_l0
0007ooooo`03o`000?oooooooooo00Sooooo3?oo000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0
o`0<o`00o`coogl02oooool;ool000co0?l02ol00?l<oomo00_ooooo2ooo000=o`3o00[ooooo3Ooo
O`0:ooooo`coo`002ol0o`0;o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o
0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool02?ooool<ool000_o0?l03?l00?l<
oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;
ool000go0?l02_ooool=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<
o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08oooo
o`coo`002ol0o`0<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o
00_o003o3?ooO`0;ooooo`_oo`003Ol0o`0:ooooo`googl02_ooool<ool000_o0?l02ol00?l=oomo
00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03
o`000?oooooooooo00Sooooo3?oo000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`co
ogl02oooool;ool000co0?l02ol00?l<oomo00_ooooo2ooo000=o`3o00[ooooo3OooO`0:ooooo`co
o`002ol0o`0;o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000oooooooo
ool02Oooool0027ooooo00?o0000ooooooooool02?ooool<ool000_o0?l03?l00?l<oomo00[ooooo
3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003?l0o`0;o`00o`coogl02oooool;ool000go0?l0
2_ooool=oomo00[ooooo3?oo000;o`3o00_o003o3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl0
2oooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08ooooo`coo`002ol0
o`0<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000<o`3o00_o003o3?oo
O`0;ooooo`_oo`003Ol0o`0:ooooo`googl02_ooool=ool000[ooooo2ol00?l=oomo00[ooooo3?oo
000;o`3o00co003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooo
oooo00Sooooo3?oo000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool;
ool000co0?l02ol00?l<oomo00_ooooo2ooo000=o`3o00[ooooo3OooO`0:ooooo`goo`002_ooool;
o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000ooooooooool02Oooool0
027ooooo0_l00009ooooo`coo`002ol0o`0<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo
00_ooooo2ooo000<o`3o00_o003o3?ooO`0;ooooo`_oo`003Ol0o`0:ooooo`googl02_ooool=ool0
00[ooooo2ol00?l=oomo00[ooooo3?oo000;o`3o00co003o3?ooO`09ooooo`?o00002oooool0027o
oooo00?o0000ooooooooool02?ooool<ool000_o0?l03?l00?l<oomo00[ooooo3?oo000;o`3o00co
003o3?ooO`0;ooooo`_oo`003Ol0o`0:ooooo`coogl02oooool;ool000go0?l02_ooool=oomo00[o
oooo3Ooo000:ooooo`_o003o3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool00ol0003o
ooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08ooooo`coo`002ol0o`0<o`00o`coogl0
2_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000=o`3o00[ooooo3?ooO`0;ooooo`_oo`00
3Ol0o`0:ooooo`googl02_ooool=ool000[ooooo2ol00?l=oomo00[ooooo3?oo000;o`3o00co003o
3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3?oo
000;o`3o00co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool;ool000go0?l02_oo
ool<oomo00_ooooo2ooo000=o`3o00[ooooo3OooO`0:ooooo`goo`002_ooool;o`00o`googl02_oo
ool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000
ooooooooool02?ooool=ool000[ooooo3?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;
ooooo`_oo`003Ol0o`0:ooooo`coogl02oooool;ool000go0?l02_ooool=oomo00[ooooo3Ooo000:
ooooo`_o003o3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09
ooooo`008Oooool00ol0003oooooooooo`08ooooo`goo`002_ooool<o`00o`coogl02_ooool<ool0
00_o0?l03?l00?l<oomo00_ooooo2ooo000=o`3o00[ooooo3?ooO`0;ooooo`_oo`003Ol0o`0:oooo
o`googl02_ooool=ool000[ooooo2ol00?l=oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;oooo
o`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3Ooo000:ooooo`co
003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool;ool000go0?l02_ooool<oomo00_o
oooo2ooo000=o`3o00[ooooo3OooO`0:ooooo`goo`002_ooool;o`00o`googl02_ooool<ool000_o
0?l03?l00?l<oomo00_ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool0
2?ooool=ool000[ooooo3?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`00
3Ol0o`0:ooooo`coogl02oooool;ool000go0?l02_ooool=oomo00[ooooo3Ooo000:ooooo`_o003o
3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`008Ooo
ool00ol0003oooooooooo`08ooooo`goo`002_ooool<o`00o`coogl02_ooool<ool000_o0?l03?l0
0?l<oomo00_ooooo2ooo000=o`3o00[ooooo3?ooO`0;ooooo`_oo`003Ol0o`0:ooooo`googl02_oo
ool=ool000[ooooo2ol00?l=oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`03o`000?oo
oooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3Ooo000:ooooo`co003o3?ooO`0:
ooooo`coo`002ol0o`0<o`00o`coogl02oooool;ool000go0?l02_ooool<oomo00_ooooo2ooo000=
o`3o00[ooooo3OooO`0:ooooo`goo`002_ooool;o`00o`googl02_ooool<ool000_o0?l03?l00?l<
oomo00_ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool02?ooool=ool0
00[ooooo3?l00?l<oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003Ol0o`0:oooo
o`coogl02oooool;ool000go0?l02_ooool=oomo00[ooooo3Ooo000:ooooo`_o003o3OooO`0:oooo
o`coo`002ol0o`0<o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`008Oooool2o`0000Wo
oooo3Ooo000:ooooo`co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`coogl02oooool;ool000go
0?l02_ooool<oomo00_ooooo2ooo000=o`3o00[ooooo3OooO`0:ooooo`goo`002_ooool;o`00o`go
ogl02_ooool<ool000_o0?l03?l00?l<oomo00Wooooo0ol0000;ooooo`008Oooool00ol0003ooooo
ooooo`08ooooo`goo`002_ooool<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo
2ooo000=o`3o00[ooooo3?ooO`0;ooooo`_oo`003Ol0o`0:ooooo`googl02_ooool=ool000[ooooo
2ol00?l=oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo
000Qooooo`03o`000?oooooooooo00Sooooo3Ooo000:ooooo`co003o3?ooO`0:ooooo`coo`002ol0
o`0<o`00o`coogl02oooool;ool000go0?l02_ooool<oomo00_ooooo2ooo000=o`3o00[ooooo3Ooo
O`0:ooooo`goo`002_ooool;o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o
0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool02?ooool=ool000[ooooo3?l00?l<
oomo00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003Ol0o`0:ooooo`coogl02oooool;
ool000go0?l02_ooool=oomo00[ooooo3Ooo000:ooooo`_o003o3OooO`0:ooooo`coo`002ol0o`0<
o`00o`coogl02oooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08oooo
o`goo`002_ooool<o`00o`coogl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo2ooo000=o`3o
00[ooooo3?ooO`0;ooooo`_oo`003Ol0o`0:ooooo`googl02_ooool=ool000[ooooo2ol00?l=oomo
00[ooooo3?oo000;o`3o00co003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03
o`000?oooooooooo00Sooooo3Ooo000:ooooo`co003o3?ooO`0:ooooo`coo`002ol0o`0<o`00o`co
ogl02oooool;ool000go0?l02_ooool<oomo00_ooooo2ooo000=o`3o00[ooooo3OooO`0:ooooo`go
o`002_ooool;o`00o`googl02_ooool<ool000_o0?l03?l00?l<oomo00_ooooo00?o0000oooooooo
ool02Oooool0027ooooo00?o0000ooooooooool02?ooool=ool000[ooooo3?l00?l<oomo00[ooooo
3?oo000;o`3o00co003o3?ooO`0;ooooo`_oo`003Ol0o`0:ooooo`coogl02oooool;ool000go0?l0
2_ooool=oomo00[ooooo3Ooo000:ooooo`_o003o3OooO`0:ooooo`coo`002ol0o`0<o`00o`coogl0
2oooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08ooooo`goo`002_oo
ool<o`00o`coogl02_ooool=ool000[ooooo3?l00?l<oomo00_ooooo2ooo000=o`3o00[ooooo3?oo
O`0;ooooo`_oo`003Ol0o`0:ooooo`googl02_ooool=ool000[ooooo2ol00?l=oomo00[ooooo3?oo
000;o`3o00co003o3?ooO`0;ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooo
oooo00Sooooo3Ooo000:ooooo`co003o3?ooO`0:ooooo`goo`002_ooool<o`00o`coogl02oooool;
ool000go0?l02_ooool<oomo00_ooooo2ooo000=o`3o00[ooooo3OooO`0:ooooo`goo`002_ooool;
o`00o`googl02_ooool<ool000_o0?l03Ol00?lFooooo`03o`000?oooooooooo00Wooooo000Qoooo
o`03o`000?oooooooooo00Sooooo3Ooo000:ooooo`co003o3?ooO`0:ooooo`goo`002_ooool<o`00
o`coogl02oooool;ool000go0?l08Oooool;ool000go0?l02_ooool=oomo00[ooooo3Ooo000:oooo
o`_o003o3OooO`0:ooooo`coo`002ol0o`0=o`00oaKooooo00?o0000ooooooooool02Oooool0027o
oooo0_l00009ooooo`goo`002_ooool<o`00o`coogl02_ooool=ool000[ooooo3?l00?l<oomo00_o
oooo2ooo000=o`3o027ooooo2ooo000=o`3o00[ooooo3OooO`0:ooooo`goo`002_ooool;o`00o`go
ogl02_ooool<ool000_o0?l03Ol00?lDooooo`?o00002oooool0027ooooo00?o0000ooooooooool0
2?ooool=ool000[ooooo3?l00?l<oomo00[ooooo3Ooo000:ooooo`co003o3?ooO`0;ooooo`_oo`00
3Ol0o`0Qooooo`_oo`003Ol0o`0:ooooo`googl02_ooool=ool000[ooooo2ol00?l=oomo00[ooooo
3?oo000;o`3o00go003o5_ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003ooooooooo
o`08ooooo`goo`002_ooool<o`00o`coogl02_ooool=ool001Kooooo3?ooO`0;ooooo`_oo`003Ol0
o`0Qooooo`_oo`003Ol0o`0:ooooo`googl02_ooool=ool000[ooooo2ol00?l=oomo00[ooooo3?oo
000;o`3o00go003o5_ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08
ooooo`goo`002_ooool<o`00o`coogl02_ooool=ool001Kooooo3?ooO`0;ooooo`_oo`003Ol0o`0Q
ooooo`_oo`003Ol0o`0:ooooo`googl02_ooool=ool000[ooooo2ol00?l=oomo00[ooooo3?oo000;
o`3o00go003o5_ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08oooo
o`goo`005_ooool<oomo00[ooooo3Ooo000Fooooo`coogl02oooool;ool000go0?l08Oooool;ool0
00go0?l02_ooool=oomo00[ooooo3Ooo000:ooooo`_o003o3OooO`0:ooooo`coo`002ol0o`0=o`00
oaKooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool02?ooool=ool001Ko
oooo3?ooO`0:ooooo`goo`005_ooool<oomo00_ooooo2ooo000=o`3o027ooooo2ooo000=o`3o00[o
oooo3OooO`0:ooooo`goo`005Oooool=oomo00[ooooo3Ooo000:ooooo`go003o5_ooool00ol0003o
ooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08ooooo`goo`005_ooool<oomo00[ooooo
3Ooo000Fooooo`coogl02oooool;ool000go0?l08Oooool;ool000go0?l02_ooool=oomo00[ooooo
3Ooo000Eooooo`googl02_ooool=ool000[ooooo3Ol00?lFooooo`03o`000?oooooooooo00Wooooo
000Qooooo`03o`000?oooooooooo00Sooooo3Ooo000Fooooo`coogl02_ooool=ool001Kooooo3?oo
O`0;ooooo`_oo`003Ol0o`0Qooooo`_oo`003Ol0o`0:ooooo`googl02_ooool=ool001Gooooo3Ooo
O`0:ooooo`goo`002_ooool=o`00oaKooooo00?o0000ooooooooool02Oooool0017ooooo0ol00004
ooooo`;o00001oooool00ol0003oooooooooo`08ooooo`goo`005_ooool<oomo00[ooooo3Ooo000F
ooooo`coogl02oooool;ool000go0?l08Oooool;ool000go0?l02_ooool=oomo00[ooooo3Ooo000E
ooooo`googl02_ooool=ool000[ooooo3Ol00?lFooooo`03o`000?oooooooooo00Wooooo000@oooo
o`05o`000?ooooooooooooooool000000_ooool01?l0003oooooooooool00006ooooo`03o`000?oo
oooooooo00Sooooo3Ooo000Fooooo`coogl02_ooool=ool001Kooooo3?ooO`0;ooooo`_oo`003Ol0
o`0Qooooo`_oo`003Ol0o`0:ooooo`googl02_ooool=ool001Gooooo3OooO`0:ooooo`goo`002_oo
ool=o`00oaKooooo00?o0000ooooooooool02Oooool0013ooooo00Go0000ooooooooooooooooo`00
0002ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool02?ooool=ool001Kooooo
3?ooO`0:ooooo`goo`005_ooool<oomo00_ooooo2ooo000=o`3o027ooooo2ooo000=o`3o00[ooooo
3OooO`0:ooooo`goo`005Oooool=oomo00[ooooo3Ooo000:ooooo`go003o5_ooool00ol0003ooooo
ooooo`09ooooo`004?ooool4o`0000?ooooo00Co0000oooooooooooo00001_ooool3o`0000So001o
3Ooo000Fo`00O`coogl02_l007l=ool001Ko001o3?ooO`0;o`00O`_oo`003Ol0o`0Qo`00O`coo`00
5_l007l=oomo00[o001o3Ooo000Eo`00O`googl02_l007l=ool000[o001o3Ol00?lCo`00O`Co0000
2oooool0013ooooo00?o0000ooooooooool01?ooool01?l0003oooooooooool00006ooooo`03o`00
0?oooooooooo00Sooooo3Ooo000Fooooo`coogl02_ooool=ool001Kooooo3?ooO`0;ooooo`_oo`00
3Ol0o`0Qooooo`coo`005_ooool=oomo00[ooooo3Ooo000Eooooo`googl02_ooool=ool000[ooooo
3Ol00?lFooooo`03o`000?oooooooooo00Wooooo000Aooooo`03o`000?oooooooooo00?ooooo00Co
0000oooooooooooo00001_ooool00ol0003oooooooooo`08ooooo`goo`005_ooool<oomo00[ooooo
3Ooo000Fooooo`coogl02oooool;ool000go0?l08Oooool<ool001Kooooo3OooO`0:ooooo`goo`00
5Oooool=oomo00[ooooo3Ooo000:ooooo`go003o5_ooool00ol0003oooooooooo`09ooooo`004_oo
ool2o`0000Cooooo0_l00007ooooo`03o`000?oooooooooo00Sooooo3Ooo000Fooooo`coogl02_oo
ool=ool001Kooooo3?ooO`0;ooooo`_oo`003Ol0o`0Qooooo`coo`00;Oooool=ool001Gooooo3Ooo
O`0:ooooo`goo`002_ooool=o`00oaKooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000
ooooooooool02?ooool=ool001Kooooo3?ooO`0:ooooo`goo`005_ooool<oomo00_ooooo2ooo000=
o`3o027ooooo3?oo000]ooooo`goo`005Oooool=oomo00[ooooo3Ooo000:ooooo`go003o5_ooool0
0ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08ooooo`goo`005_ooool<oomo
00[ooooo3Ooo000Fooooo`coogl02oooool;ool000go0?l08Oooool<ool002gooooo3Ooo000Eoooo
o`googl02_ooool=ool000[ooooo3Ol00?lFooooo`03o`000?oooooooooo00Wooooo000Qooooo`03
o`000?oooooooooo00Sooooo3Ooo000Fooooo`coogl02_ooool=ool001Kooooo3?ooO`0;ooooo`_o
o`003Ol0o`0Qooooo`coo`00;Oooool=ool001Gooooo3OooO`0:ooooo`goo`002_ooool=o`00oaKo
oooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool02?ooool=ool001Kooooo
3?ooO`0:ooooo`goo`005_ooool<oomo00_ooooo2ooo000=o`3o027ooooo3?oo000]ooooo`goo`00
5Oooool=oomo00[ooooo3Ooo000:ooooo`go003o5_ooool00ol0003oooooooooo`09ooooo`008Ooo
ool00ol0003oooooooooo`08ooooo`goo`005_ooool<oomo00[ooooo3Ooo000Fooooo`coogl02ooo
ool;ool000go0?l08Oooool<ool002gooooo3Ooo000Eooooo`googl02_ooool=ool000[ooooo3Ol0
0?lFooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3Ooo000F
ooooo`coogl02_ooool=ool001Kooooo3?ooO`0;ooooo`_oo`003Ol0o`0Qooooo`coo`00;Oooool=
ool001Gooooo3OooO`0:ooooo`goo`002_ooool=o`00oaKooooo00?o0000ooooooooool02Oooool0
027ooooo0_l00009ooooo`goo`005_ooool<oomo00[ooooo3Ooo000Fooooo`coogl02oooool<ool0
02gooooo3?oo000]ooooo`goo`005Oooool=oomo00[ooooo3Ooo000:ooooo`go003o5?ooool3o`00
00_ooooo000Qooooo`03o`000?oooooooooo00Sooooo3Ooo000Fooooo`coogl02_ooool=ool001Ko
oooo3?ooO`0;ooooo`coo`00;Oooool<ool002gooooo3Ooo000Eooooo`googl02_ooool=ool000[o
oooo3Ol00?lFooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo
3Ooo000Fooooo`coogl02_ooool=ool001Kooooo3?ooO`0;ooooo`coo`00;Oooool<ool002gooooo
3Ooo000Eooooo`googl02_ooool=ool000[ooooo3Ol00?lFooooo`03o`000?oooooooooo00Wooooo
000Qooooo`03o`000?oooooooooo00Sooooo3Ooo000Fooooo`coogl02_ooool=ool001Kooooo3?oo
O`0;ooooo`coo`00;Oooool<ool002gooooo3Ooo000Eooooo`googl02_ooool=ool000[ooooo3Ol0
0?lFooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3Ooo000F
ooooo`coogl02_ooool=ool001Kooooo3?ooO`0;ooooo`coo`00;Oooool<ool002gooooo3Ooo000E
ooooo`googl02_ooool=ool000[ooooo3Ol00?lFooooo`03o`000?oooooooooo00Wooooo000Qoooo
o`03o`000?oooooooooo00Sooooo3Ooo000Fooooo`coogl02_ooool=ool001Kooooo3?ooO`0;oooo
o`coo`00;Oooool<ool002gooooo3Ooo000Eooooo`googl02_ooool=ool000[ooooo3Ol00?lFoooo
o`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3Ooo000Fooooo`co
ogl02_ooool=ool001Kooooo3?ooO`0;ooooo`coo`00;Oooool<ool002gooooo3Ooo000Eooooo`go
ogl02_ooool=ool000[ooooo3Ol00?lFooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`00
0?oooooooooo00Sooooo3Ooo000Fooooo`coogl02_ooool=ool001Kooooo3?ooO`0;ooooo`coo`00
;Oooool<ool002gooooo3Ooo000Eooooo`googl02_ooool=ool000[ooooo3Ol00?lFooooo`03o`00
0?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3Ooo000Fooooo`coogl02_oo
ool=ool001Kooooo3?ooO`0;ooooo`coo`00;Oooool<ool002gooooo3Ooo000Eooooo`googl02_oo
ool=ool000[ooooo3Ol00?lFooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooo
oooo00Sooooo3Ooo000/ooooo`goo`005_ooool<oomo00_ooooo3?oo000]ooooo`coo`00;Oooool=
ool001Gooooo3OooO`0:ooooo`goo`002_ooool=o`00oaKooooo00?o0000ooooooooool02Oooool0
027ooooo00?o0000ooooooooool02?ooool=ool002cooooo3Ooo000Fooooo`coogl02oooool<ool0
02gooooo3?oo000]ooooo`goo`005Oooool=oomo00[ooooo3Ooo000:ooooo`go003o5_ooool00ol0
003oooooooooo`09ooooo`008Oooool2o`0000Wooooo3Ooo000/ooooo`goo`005_ooool<oomo00_o
oooo3?oo000]ooooo`coo`00;Oooool=ool001Gooooo3OooO`0:ooooo`goo`002_ooool=o`00oaCo
oooo0ol0000;ooooo`008Oooool00ol0003oooooooooo`08ooooo`goo`00;?ooool=ool002gooooo
3?oo000]ooooo`coo`00;Oooool=ool001Gooooo3OooO`0:ooooo`goo`002_ooool=o`00oaKooooo
00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool02?ooool=ool002cooooo3Ooo
000]ooooo`coo`00;Oooool<ool002gooooo3Ooo000Eooooo`googl02_ooool=ool000[ooooo3Ol0
0?lFooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3Ooo000/
ooooo`goo`00;Oooool<ool002gooooo3?oo000]ooooo`goo`005Oooool=oomo00[ooooo3Ooo000:
ooooo`go003o5_ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08oooo
o`goo`00;?ooool=ool002gooooo3?oo000]ooooo`coo`00;Oooool=ool001Gooooo3OooO`0:oooo
o`goo`002_ooool=o`00oaKooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000oooooooo
ool02?ooool=ool002cooooo3Ooo000]ooooo`coo`00;Oooool<ool002gooooo3Ooo000Eooooo`go
ogl02_ooool=ool000[ooooo3Ol00?lFooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`00
0?oooooooooo00Sooooo3Ooo000/ooooo`goo`00;Oooool<ool002gooooo3?oo000]ooooo`goo`00
5Oooool=oomo00[ooooo3Ooo000:ooooo`go003o5_ooool00ol0003oooooooooo`09ooooo`008Ooo
ool00ol0003oooooooooo`08ooooo`goo`00;?ooool=ool002gooooo3?oo000]ooooo`coo`00;Ooo
ool=ool001Gooooo3OooO`0:ooooo`goo`002_ooool=o`00oaKooooo00?o0000ooooooooool02Ooo
ool0027ooooo00?o0000ooooooooool02?ooool=ool002cooooo3Ooo000]ooooo`coo`00;Oooool<
ool002gooooo3Ooo000Eooooo`googl02_ooool=ool000[ooooo3Ol00?lFooooo`03o`000?oooooo
oooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3Ooo000/ooooo`goo`00;Oooool<ool0
02gooooo3?oo000]ooooo`goo`005Oooool=oomo00[ooooo3Ooo000:ooooo`go003o5_ooool00ol0
003oooooooooo`09ooooo`008Oooool2o`0000Wooooo3Ooo000/ooooo`goo`00;Oooool<ool002go
oooo3?oo001?ooooo`googl02_ooool=ool000[ooooo3Ol00?lDooooo`?o00002oooool0027ooooo
00?o0000ooooooooool02?ooool=ool002cooooo3Ooo000]ooooo`coo`00;Oooool<ool004oooooo
3OooO`0:ooooo`goo`002_ooool=o`00oaKooooo00?o0000ooooooooool02Oooool0027ooooo00?o
0000ooooooooool02?ooool=ool002cooooo3Ooo000]ooooo`coo`00;Oooool<ool004oooooo3Ooo
O`0:ooooo`goo`002_ooool=o`00oaKooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000
ooooooooool02?ooool=ool002cooooo3Ooo000]ooooo`coo`00;Oooool<ool004oooooo3OooO`0:
ooooo`goo`002_ooool=o`00oaKooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000oooo
ooooool02?ooool=ool002cooooo3Ooo000]ooooo`coo`00R?ooool=oomo00[ooooo3Ooo000:oooo
o`go003o5_ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`08ooooo`go
o`00;?ooool=ool002gooooo3?oo0028ooooo`googl02_ooool=ool000[ooooo3Ol00?lFooooo`03
o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00Sooooo3Ooo000/ooooo`goo`00
;Oooool<ool008Sooooo3OooO`0:ooooo`goo`002_ooool=o`00oaKooooo00?o0000ooooooooool0
2Oooool0027ooooo00?o0000ooooooooool02?ooool=ool002cooooo3Ooo000]ooooo`coo`00R?oo
ool=oomo00[ooooo3Ooo000:ooooo`go003o5_ooool00ol0003oooooooooo`09ooooo`004Oooool3
o`0000Cooooo0_l00007ooooo`03o`000?oooooooooo047ooooo3Ooo000]ooooo`coo`00R?ooool=
oomo00[ooooo3Ooo000:ooooo`go003o5_ooool00ol0003oooooooooo`09ooooo`004?ooool01Ol0
003ooooooooooooooooo000000;ooooo00Co0000oooooooooooo00001_ooool00ol0003ooooooooo
o`11ooooo`goo`00;Oooool<ool008Sooooo3OooO`0:ooooo`goo`002_ooool=o`00oaKooooo00?o
0000ooooooooool02Oooool0013ooooo00Go0000ooooooooooooooooo`000002ooooo`04o`000?oo
ooooooooo`0000Kooooo00?o0000ooooooooool0@Oooool=ool002gooooo3?oo0028ooooo`googl0
2_ooool=ool000[ooooo3Ol00?lFooooo`03o`000?oooooooooo00Wooooo000Aooooo`?o00000ooo
ool01?l0003oooooooooool00006ooooo`?o0000@Ol007l=ool002go001o3?oo0028o`00O`googl0
2_l007l=ool000[o001o3Ol00?lCo`00O`Co00002oooool0013ooooo00Go0000oooooooooooooooo
o`000002ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool0@Oooool=ool002go
oooo3?oo0028ooooo`googl02_ooool=ool002gooooo00?o0000ooooooooool02Oooool0013ooooo
00Go0000ooooooooooooooooo`000002ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000oooo
ooooool0@Oooool=ool002gooooo3?oo0028ooooo`googl02_ooool=ool002gooooo00?o0000oooo
ooooool02Oooool0017ooooo0ol00004ooooo`;o00001oooool00ol0003oooooooooo`11ooooo`go
o`00;Oooool<ool008Sooooo3OooO`0:ooooo`goo`00;Oooool00ol0003oooooooooo`09ooooo`00
8Oooool00ol0003oooooooooo`11ooooo`goo`00;Oooool<ool008Sooooo3OooO`0:ooooo`goo`00
;Oooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`11ooooo`goo`00;Ooo
ool<ool008Sooooo3OooO`0:ooooo`goo`00;Oooool00ol0003oooooooooo`09ooooo`008Oooool0
0ol0003oooooooooo`11ooooo`goo`00;Oooool<ool008Sooooo3OooO`0:ooooo`goo`00;Oooool0
0ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`11ooooo`goo`00;Oooool<ool0
08Sooooo3OooO`14ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo047o
oooo3Ooo000]ooooo`coo`00R?ooool=oomo04Cooooo00?o0000ooooooooool02Oooool0027ooooo
00?o0000ooooooooool0@Oooool=ool002gooooo3?oo0028ooooo`googl0A?ooool00ol0003ooooo
ooooo`09ooooo`008Oooool2o`0004;ooooo3Ooo000]ooooo`coo`00R?ooool=oomo04;ooooo0ol0
000;ooooo`008Oooool00ol0003oooooooooo`11ooooo`goo`00;Oooool<ool008Sooooo3OooO`14
ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo047ooooo3Ooo000]oooo
o`coo`00R?ooool=oomo04Cooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000oooooooo
ool0@Oooool=ool002gooooo3?oo0028ooooo`googl0A?ooool00ol0003oooooooooo`09ooooo`00
8Oooool00ol0003oooooooooo`11ooooo`goo`00;Oooool<ool008Sooooo3OooO`14ooooo`03o`00
0?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo047ooooo3Ooo000]ooooo`coo`00R?oo
ool=oomo04Cooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool0@Oooool=
ool002gooooo3?oo0028ooooo`googl0A?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0
003oooooooooo`11ooooo`goo`00;Oooool<ool008Sooooo3OooO`14ooooo`03o`000?oooooooooo
00Wooooo000Qooooo`03o`000?oooooooooo047ooooo3Ooo000]ooooo`coo`00R?ooool=oomo04Co
oooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool0@Oooool=ool002gooooo
3?oo0028ooooo`googl0A?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003ooooooooo
o`11ooooo`goo`00;Oooool<ool008Sooooo3OooO`14ooooo`03o`000?oooooooooo00Wooooo000Q
ooooo`;o0000@_ooool=ool002gooooo3?oo0028ooooo`googl0@_ooool3o`0000_ooooo000Qoooo
o`03o`000?oooooooooo047ooooo3Ooo000]ooooo`coo`00R?ooool=oomo04Cooooo00?o0000oooo
ooooool02Oooool0027ooooo00?o0000ooooooooool0@Oooool=ool002gooooo3?oo0028ooooo`go
ogl0A?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`11ooooo`goo`00
;Oooool<ool00=Wooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool0@Ooo
ool=ool002gooooo3?oo003Iooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooo
oooo047ooooo3Ooo000]ooooo`coo`00fOooool00ol0003oooooooooo`09ooooo`008Oooool00ol0
003oooooooooo`11ooooo`goo`00;Oooool<ool00=Wooooo00?o0000ooooooooool02Oooool0027o
oooo00?o0000ooooooooool0@Oooool=ool00?oooooo4oooool00ol0003oooooooooo`09ooooo`00
8Oooool00ol0003oooooooooo`11ooooo`goo`00oooooolCooooo`03o`000?oooooooooo00Wooooo
000Qooooo`03o`000?oooooooooo047ooooo3Ooo003ooooooa?ooooo00?o0000ooooooooool02Ooo
ool0027ooooo00?o0000ooooooooool0@Oooool=ool00?oooooo4oooool00ol0003oooooooooo`09
ooooo`008Oooool2o`0004;ooooo3Ooo003ooooooa7ooooo0ol0000;ooooo`008Oooool00ol0003o
ooooooooo`11ooooo`goo`00oooooolCooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`00
0?oooooooooo047ooooo3Ooo003ooooooa?ooooo00?o0000ooooooooool02Oooool0027ooooo00?o
0000ooooooooool0@Oooool=ool00?oooooo4oooool00ol0003oooooooooo`09ooooo`008Oooool0
0ol0003oooooooooo`11ooooo`goo`00oooooolCooooo`03o`000?oooooooooo00Wooooo000Qoooo
o`03o`000?oooooooooo047ooooo3Ooo003ooooooa?ooooo00?o0000ooooooooool02Oooool0027o
oooo00?o0000ooooooooool0@Oooool=ool00?oooooo4oooool00ol0003oooooooooo`09ooooo`00
8Oooool00ol0003oooooooooo`11ooooo`goo`00oooooolCooooo`03o`000?oooooooooo00Wooooo
000;ooooo`?o00001?ooool2o`0000Cooooo0_l00007ooooo`03o`000?oooooooooo047ooooo3Ooo
003ooooooa?ooooo00?o0000ooooooooool02Oooool000cooooo00?o0000ooooooooool00_ooool0
1?l0003oooooooooool00002ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool0
oooooomQooooo`03o`000?oooooooooo00Wooooo000<ooooo`03o`000?oooooooooo00;ooooo00Co
0000oooooooooooo00000_ooool01?l0003oooooooooool00006ooooo`?o0000oooooomNooooo`Co
00002oooool000cooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`04
o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool07oooool00ol0003oooooooooo`0foooo
o`03o`000?oooooooooo03Oooooo00?o0000ooooooooool0=_ooool00ol0003oooooooooo`0foooo
o`03o`000?oooooooooo03Kooooo00?o0000ooooooooool08?ooool00ol0003oooooooooo`09oooo
o`003?ooool00ol0003oooooooooo`02ooooo`04o`000?ooooooooooo`0000;ooooo00Co0000oooo
oooooooo00001_ooool00ol0003oooooooooo`0Oooooo`03o`000?oooooooooo03Kooooo00?o0000
ooooooooool0=oooool00ol0003oooooooooo`0fooooo`03o`000?oooooooooo03Kooooo00?o0000
ooooooooool0=_ooool00ol0003oooooooooo`0Pooooo`03o`000?oooooooooo00Wooooo000;oooo
o`;o00001?ooool01?l0003oooooooooool00002ooooo`04o`000?ooooooooooo`0000Koooooool0
001Uo`0000_ooooo000<ooooo`03o`000?oooooooooo00?ooooo0_l00004ooooo`;o0000oooooomg
ooooo`00oooooonAooooo`00oooooonAooooo`00oooooonAooooo`00oooooonAooooo`00oooooonA
ooooo`00@Oooool3o`0003Gooooo1Ol0000fooooo`?o0000=oooool3o`0003Gooooo0ol0000foooo
o`?o0000;_ooool004;ooooo00?o0000ooooooooool0=Oooool01?l0003oooooooooool0000eoooo
o`05o`000?ooooooooooooooool00000=oooool00ol0003oooooooooo`0cooooo`05o`000?oooooo
ooooooooool00000=?ooool01Ol0003ooooooooooooooooo000002gooooo0012ooooo`03o`000?oo
oooooooo03Kooooo00?o0000ooooooooool0>Oooool00ol0003oooooooooo`0booooo`Go0000>?oo
ool00ol0003oooooooooo`0booooo`05o`000?ooooooooooooooool00000;Oooool004;ooooo00?o
0000ooooooooool0=oooool00ol0003oooooooooo`0fooooo`;o0000=Oooool01?l0003ooooooooo
ool0000iooooo`03o`000?oooooooooo03;ooooo1?l0000^ooooo`00@_ooool00ol0003ooooooooo
o`0dooooo`05o`000?ooooooooooooooool00000>Oooool00ol0003oooooooooo`0cooooo`03o`00
0?oooooo000003Gooooo1?l0000eooooo`03o`000?oooooooooo02oooooo0011ooooo`;o0000=_oo
ool01Ol0003ooooooooooooooooo000003Gooooo00Go0000ooooooooooooooooo`00000fooooo`;o
0000=Oooool00ol0003oooooooooo`0gooooo`03o`000?oooooooooo02kooooo0012ooooo`03o`00
0?oooooooooo03Gooooo0ol0000gooooo`?o0000>?ooool00ol0003oooooooooo`0cooooo`Go0000
=_ooool2o`0002kooooo0000\
\>"],
  ImageRangeCache->{{{0, 399}, {246.188, 0}} -> {0.119489, -8.83902, \
0.0174717, 0.473722}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Overall bands and overall correct classification percentage", \
"Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(correctoverall = 
        Table[\(Map[First, resultsCombined]\)[\([i, i]\)], {i, 1, 
            nImages}];\)\), "\[IndentingNewLine]", 
    \(\(BarChart[correctoverall, \ BarStyle \[Rule] colors, 
        BarEdgeStyle \[Rule] colors, Frame \[Rule] True, \ 
        ImageSize \[Rule] 400];\)\), "\[IndentingNewLine]", 
    \(Mean[correctoverall]\)}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
%%ImageSize: 400 247.214 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.238095 0.0147151 0.00593815 [
[.14286 -0.0125 -3 -9 ]
[.14286 -0.0125 3 0 ]
[.38095 -0.0125 -3 -9 ]
[.38095 -0.0125 3 0 ]
[.61905 -0.0125 -3 -9 ]
[.61905 -0.0125 3 0 ]
[.85714 -0.0125 -3 -9 ]
[.85714 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .01472 -6 -4.5 ]
[-0.0125 .01472 0 4.5 ]
[-0.0125 .13348 -12 -4.5 ]
[-0.0125 .13348 0 4.5 ]
[-0.0125 .25224 -12 -4.5 ]
[-0.0125 .25224 0 4.5 ]
[-0.0125 .371 -12 -4.5 ]
[-0.0125 .371 0 4.5 ]
[-0.0125 .48977 -12 -4.5 ]
[-0.0125 .48977 0 4.5 ]
[-0.0125 .60853 -18 -4.5 ]
[-0.0125 .60853 0 4.5 ]
[ 0 0 -0.125 0 ]
[.14286 .63053 -3 0 ]
[.14286 .63053 3 9 ]
[.38095 .63053 -3 0 ]
[.38095 .63053 3 9 ]
[.61905 .63053 -3 0 ]
[.61905 .63053 3 9 ]
[.85714 .63053 -3 0 ]
[.85714 .63053 3 9 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.14286 0 m
.14286 .00625 L
s
[(1)] .14286 -0.0125 0 1 Mshowa
.38095 0 m
.38095 .00625 L
s
[(2)] .38095 -0.0125 0 1 Mshowa
.61905 0 m
.61905 .00625 L
s
[(3)] .61905 -0.0125 0 1 Mshowa
.85714 0 m
.85714 .00625 L
s
[(4)] .85714 -0.0125 0 1 Mshowa
0 0 m
1 0 L
s
0 .01472 m
.00625 .01472 L
s
[(0)] -0.0125 .01472 1 0 Mshowa
0 .13348 m
.00625 .13348 L
s
[(20)] -0.0125 .13348 1 0 Mshowa
0 .25224 m
.00625 .25224 L
s
[(40)] -0.0125 .25224 1 0 Mshowa
0 .371 m
.00625 .371 L
s
[(60)] -0.0125 .371 1 0 Mshowa
0 .48977 m
.00625 .48977 L
s
[(80)] -0.0125 .48977 1 0 Mshowa
0 .60853 m
.00625 .60853 L
s
[(100)] -0.0125 .60853 1 0 Mshowa
.125 Mabswid
0 .04441 m
.00375 .04441 L
s
0 .0741 m
.00375 .0741 L
s
0 .10379 m
.00375 .10379 L
s
0 .16317 m
.00375 .16317 L
s
0 .19286 m
.00375 .19286 L
s
0 .22255 m
.00375 .22255 L
s
0 .28193 m
.00375 .28193 L
s
0 .31162 m
.00375 .31162 L
s
0 .34131 m
.00375 .34131 L
s
0 .40069 m
.00375 .40069 L
s
0 .43039 m
.00375 .43039 L
s
0 .46008 m
.00375 .46008 L
s
0 .51946 m
.00375 .51946 L
s
0 .54915 m
.00375 .54915 L
s
0 .57884 m
.00375 .57884 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.14286 .61178 m
.14286 .61803 L
s
[(1)] .14286 .63053 0 -1 Mshowa
.38095 .61178 m
.38095 .61803 L
s
[(2)] .38095 .63053 0 -1 Mshowa
.61905 .61178 m
.61905 .61803 L
s
[(3)] .61905 .63053 0 -1 Mshowa
.85714 .61178 m
.85714 .61803 L
s
[(4)] .85714 .63053 0 -1 Mshowa
0 .61803 m
1 .61803 L
s
.99375 .01472 m
1 .01472 L
s
.99375 .13348 m
1 .13348 L
s
.99375 .25224 m
1 .25224 L
s
.99375 .371 m
1 .371 L
s
.99375 .48977 m
1 .48977 L
s
.99375 .60853 m
1 .60853 L
s
.125 Mabswid
.99625 .04441 m
1 .04441 L
s
.99625 .0741 m
1 .0741 L
s
.99625 .10379 m
1 .10379 L
s
.99625 .16317 m
1 .16317 L
s
.99625 .19286 m
1 .19286 L
s
.99625 .22255 m
1 .22255 L
s
.99625 .28193 m
1 .28193 L
s
.99625 .31162 m
1 .31162 L
s
.99625 .34131 m
1 .34131 L
s
.99625 .40069 m
1 .40069 L
s
.99625 .43039 m
1 .43039 L
s
.99625 .46008 m
1 .46008 L
s
.99625 .51946 m
1 .51946 L
s
.99625 .54915 m
1 .54915 L
s
.99625 .57884 m
1 .57884 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .01472 m
1 .01472 L
s
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.04762 .01472 m
.04762 .59561 L
.2381 .59561 L
.2381 .01472 L
F
.5 Mabswid
.04762 .01472 m
.04762 .59561 L
.2381 .59561 L
.2381 .01472 L
.04762 .01472 L
s
0 1 0 r
.28571 .01472 m
.28571 .47826 L
.47619 .47826 L
.47619 .01472 L
F
.28571 .01472 m
.28571 .47826 L
.47619 .47826 L
.47619 .01472 L
.28571 .01472 L
s
0 0 1 r
.52381 .01472 m
.52381 .51369 L
.71429 .51369 L
.71429 .01472 L
F
.52381 .01472 m
.52381 .51369 L
.71429 .51369 L
.71429 .01472 L
.52381 .01472 L
s
1 .5 0 r
.7619 .01472 m
.7619 .60332 L
.95238 .60332 L
.95238 .01472 L
F
.7619 .01472 m
.7619 .60332 L
.95238 .60332 L
.95238 .01472 L
.7619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{400, 247.188},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006@0000mb000`400?l00000o`00003ooooooonA
ooooo`00DOooool3o`00057ooooo1Ol0001@ooooo`?o0000Doooool3o`0003gooooo001Booooo`03
o`000?oooooooooo057ooooo00Co0000oooooooooooo0000Coooool01Ol0003ooooooooooooooooo
000005?ooooo00?o0000ooooooooool0??ooool005;ooooo00?o0000ooooooooool0D_ooool00ol0
003oooooooooo`1Cooooo`03o`000?oooooooooo04kooooo1Ol0000mooooo`00D_ooool00ol0003o
ooooooooo`1Cooooo`03o`000?oooooooooo053ooooo0_l0001Aooooo`04o`000?ooooooooooo`00
03kooooo001Booooo`03o`000?oooooooooo053ooooo00Go0000ooooooooooooooooo`00001Coooo
o`03o`000?oooooooooo04oooooo00?o0000ooooool00000?_ooool0057ooooo0_l0001Booooo`05
o`000?ooooooooooooooool00000Coooool01Ol0003ooooooooooooooooo000005;ooooo0_l0000n
ooooo`00D_ooool00ol0003oooooooooo`1Aooooo`?o0000DOooool3o`0005Cooooo00?o0000oooo
ooooool0??ooool00?ooooooTOooool00?ooooooTOooool00?ooooooTOooool00?ooooooTOooool0
0?ooooooTOooool00?ooooooTOooool0027oooooool0001Uo`0000_ooooo000Qooooo`03o`000?oo
oooooooo02oooooo00?o0000ooooooooool0D_ooool00ol0003oooooooooo`1Aooooo`03o`000?oo
oooooooo05;ooooo00?o0000ooooooooool0<?ooool00ol0003oooooooooo`09ooooo`006?ooool2
o`0000Oooooo00?o0000ooooooooool0;oooool00ol0003oooooooooo`1Booooo`03o`000?oooooo
oooo057ooooo00?o0000ooooooooool0D_ooool00ol0003oooooooooo`0`ooooo`03o`000?oooooo
oooo00Wooooo000Gooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool0oooooomQ
ooooo`03o`000?oooooooooo00Wooooo000Gooooo`04o`000?ooooooooooo`0000Kooooo00?o0000
ooooooooool0oooooomQooooo`03o`000?oooooooooo00Wooooo000Gooooo`04o`000?oooooooooo
o`0000Kooooo4Ol00014ool0013o0000AOl0o`0@o`0004Co003o4?l00015oomo017o00002oooool0
01Oooooo00Co0000oooooooooooo00001_ooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5
o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`005oooool01?l0
003oooooooooool00006ooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4
o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Hooooo`;o00001oooool00ol0
003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0
003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013o
ooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003ooooo
ooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003ooooo
ooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l0
0?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>
ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09
ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@oooo
odGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCo
o`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`00
8Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl0
4?ooool00ol0003oooooooooo`09ooooo`008Oooool2o`0000ooooooA?oo000@ooooodGo0?l04?oo
oom4o`00oa3oooooAOooO`0>ooooo`?o00002oooool0027ooooo00?o0000ooooooooool03_oooom4
ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0
027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo
013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013o
ooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo
00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo
00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0
o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000
ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000
ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@oooo
odCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000oooooooo
ool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000oooooooo
ool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o
4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo0_l0000?ooooodCoo`004?oo
oom5o`3o013oooooA?l00?l@ooooodGoogl03_ooool3o`0000_ooooo000Qooooo`03o`000?oooooo
oooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooo
oooo00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00
oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00ko
ooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wo
oooo000Qooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3ooooo
AOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo
000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Q
ooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@
ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo000@oooo
odGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03
o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03
o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l0
4?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oo
oooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oo
oooooooo00Wooooo000Qooooo`;o00003ooooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5
oomo00kooooo0ol0000;ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o
013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003o
ooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003o
ooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013ooooo
A?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003ooooooooo
o`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003ooooooooo
o`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@
ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>oooo
odCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09oooo
o`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGo
ogl04?ooool00ol0003oooooooooo`09ooooo`004?ooool5o`0000?ooooo0_l00007ooooo`03o`00
0?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`00
0?oooooooooo00Wooooo000Aooooo`04o`000?ooooooooooo`0000;ooooo00Co0000oooooooooooo
00001_ooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGo
ogl04?ooool00ol0003oooooooooo`09ooooo`004_ooool00ol0003oooooooooo`02ooooo`04o`00
0?ooooooooooo`0000Kooooo0ol0000>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGo
ogl03Oooool4o`0000_ooooo000Cooooo`05o`000?ooooooooooooooool000000_ooool00ol0003o
ooooooooo`04ooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3o
ooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000@ooooo`05o`000?ooooooooooooooool0
00000_ooool01?l0003oooooooooool00006ooooo`03o`000?oooooooooo00koooooA?oo000@oooo
odGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000@ooooo`05
o`000?ooooooooooooooool000000_ooool01?l0003oooooooooool00006ooooo`03o`000?oooooo
oooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooo
oooo00Wooooo000Aooooo`?o00001?ooool2o`0000Oooooo00?o0000ooooooooool03_oooom4ool0
013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027o
oooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013o
oooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013ooooo
AOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o
0000ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o
0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@
ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000oooo
ooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000oooo
ooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo
003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo0_l0000?ooooodCoo`00
4?oooom5o`3o013oooooA?l00?l@ooooodGoogl03_ooool3o`0000_ooooo000Qooooo`03o`000?oo
oooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oo
oooooooo00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4
o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo
00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo
00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3o
ooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00kooooo
A?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo
000Qooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOoo
O`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo000@
ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qoooo
o`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@oooo
o`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo
0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`;o0000
3ooooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo00kooooo0ol0000;ooooo`008Ooo
ool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?oo
ool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5
o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0
003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0
003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013o
ooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003ooooo
ooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003ooooo
ooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l0
0?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>
ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09
ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@oooo
odGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCo
o`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`00
8Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl0
4?ooool00ol0003oooooooooo`09ooooo`008Oooool2o`0000ooooooA?oo000@ooooodGo0?l04?oo
oom4o`00oa3oooooAOooO`0>ooooo`?o00002oooool0027ooooo00?o0000ooooooooool03_oooom4
ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0
027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo
013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013o
ooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo
00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo
00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0
o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000
ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000
ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@oooo
odCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool001;ooooo0ol00003ooooo`;o
00001oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGo
ogl04?ooool00ol0003oooooooooo`09ooooo`004oooool01Ol0003ooooooooooooooooo000000;o
oooo00?o0000ooooooooool01?ooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013o
ooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`004?ooool5o`0000;ooooo
00Co0000oooooooooooo00001_ooool3o`0000koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3o
ooooAOooO`0=ooooo`Co00002oooool0013ooooo00Co0000oooooooooooo00000oooool01?l0003o
ooooooooool00006ooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00
oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Aooooo`03o`000?oooooo000000?o
oooo00Co0000oooooooooooo00001_ooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o
013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`004_ooool2o`0000?o
oooo00Co0000oooooooooooo00001_ooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o
013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`004oooool00ol0003o
ooooooooo`02ooooo`;o00001oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013o
ooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003ooooo
ooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003ooooo
ooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l0
0?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>
ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09
ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@oooo
odGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCo
o`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`00
8Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl0
4?ooool00ol0003oooooooooo`09ooooo`008Oooool2o`0000ooooooA?oo000@ooooodGo0?l04?oo
oom4o`00oa3oooooAOooO`0>ooooo`?o00002oooool0027ooooo00?o0000ooooooooool03_oooom4
ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0
027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo
013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013o
ooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo
00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo
00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0
o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000
ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000
ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@oooo
odCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000oooooooo
ool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000oooooooo
ool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o
4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo0_l0000?ooooodCoo`004?oo
oom5o`3o013oooooA?l00?l@ooooodGoogl03_ooool3o`0000_ooooo000Qooooo`03o`000?oooooo
oooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooo
oooo00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00
oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00ko
ooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wo
oooo000Qooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3ooooo
AOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo
000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Q
ooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@
ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo000@oooo
odGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03
o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03
o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l0
4?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oo
oooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oo
oooooooo00Wooooo000Qooooo`;o00003ooooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5
oomo00kooooo0ol0000;ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o
013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003o
ooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003o
ooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013ooooo
A?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003ooooooooo
o`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003ooooooooo
o`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@
ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>oooo
odCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09oooo
o`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGo
ogl04?ooool00ol0003oooooooooo`09ooooo`004Oooool3o`0000Cooooo0_l00007ooooo`03o`00
0?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`00
0?oooooooooo00Wooooo000@ooooo`05o`000?ooooooooooooooool000000_ooool01?l0003ooooo
ooooool00006ooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3o
ooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000@ooooo`05o`000?ooooooooooooooool0
00000_ooool01?l0003oooooooooool00006ooooo`?o00003_oooom4ool0013oooooAOl0o`0@oooo
odCo003o4?oooom5oomo00gooooo1?l0000;ooooo`004?ooool4o`0000?ooooo00Co0000oooooooo
oooo00001_ooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@oooo
odGoogl04?ooool00ol0003oooooooooo`09ooooo`004?ooool00ol0003oooooooooo`04ooooo`04
o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@oooo
odCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0017ooooo00?o0000oooooooo
ool00oooool01?l0003oooooooooool00006ooooo`03o`000?oooooooooo00koooooA?oo000@oooo
odGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Booooo`;o
00001?ooool2o`0000Oooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo
003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool0
3_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool0
2Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oo
oom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4
ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0
027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo
013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013o
ooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo
00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo
00?o0000ooooooooool02Oooool0027ooooo0_l0000?ooooodCoo`004?oooom5o`3o013oooooA?l0
0?l@ooooodGoogl03_ooool3o`0000_ooooo000Qooooo`03o`000?oooooooooo00koooooA?oo000@
ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qoooo
o`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@oooo
o`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo
0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`00
0?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`00
0?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oo
oom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooo
oooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooo
oooo00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00
oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00ko
ooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wo
oooo000Qooooo`03o`000?oooooooooo00koooooA?oo000@ooooodGo0?l04?oooom4o`00oa3ooooo
AOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`;o00003ooooom4ool0013oooooAOl0
o`0@ooooodCo003o4?oooom5oomo00kooooo0ol0000;ooooo`008Oooool00ol0003oooooooooo`0>
ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09
ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@oooo
odGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCo
o`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`00
8Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl0
4?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oo
oom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool0
0ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool0
0ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o
013oooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003o
ooooooooo`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003o
ooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`004?oooom5o`3o013ooooo
A?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003ooooooooo
o`0>ooooodCoo`004?oooom5o`3o013oooooA?l00?l@ooooodGoogl04?ooool00ol0003ooooooooo
o`09ooooo`008Oooool2o`0000ooooooA?oo000@ooooodGo0?l04?oooom4o`00oa3oooooAOooO`0>
ooooo`?o00002oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@oooo
odCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000oooooooo
ool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000oooooooo
ool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o
4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oo
oom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Ooo
ool0027ooooo00?o0000ooooooooool03_oooom4ool0013oooooAOl0o`0@ooooodCo003o4?oooom5
oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0
013oooooAOl0o`0@ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0027o
oooo00?o0000ooooooooool03_oooom4ool006GoooooA?l00?l@ooooodGoogl04?ooool00ol0003o
ooooooooo`09ooooo`004Oooool3o`0000Cooooo0_l00007ooooo`03o`000?oooooooooo00kooooo
A?oo001UooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0013ooooo00Go
0000ooooooooooooooooo`000002ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000oooooooo
ool03_oooom4ool006GoooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`00
4?ooool01Ol0003ooooooooooooooooo000000;ooooo00Co0000oooooooooooo00001_ooool00ol0
003oooooooooo`0>ooooodCoo`00IOoooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo
00Wooooo000Aooooo`?o00000oooool01?l0003oooooooooool00006ooooo`?o00003_oooom4ool0
06GoooooA?l00?l@ooooodGoogl03Oooool4o`0000_ooooo000@ooooo`05o`000?oooooooooooooo
ool000000_ooool01?l0003oooooooooool00006ooooo`03o`000?oooooooooo00koooooA?oo001U
ooooodCo003o4?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0013ooooo00Go0000oooo
ooooooooooooo`000002ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000ooooooooool03_oo
oom4ool006GoooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`004Oooool3
o`0000Cooooo0_l00007ooooo`03o`000?oooooooooo00koooooA?oo001UooooodCo003o4?oooom5
oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool0
06GoooooA?l00?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003o
ooooooooo`0>ooooodCoo`00IOoooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wo
oooo000Qooooo`03o`000?oooooooooo00koooooA?oo001UooooodCo003o4?oooom5oomo013ooooo
00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool006GoooooA?l0
0?l@ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>
ooooodCoo`00IOoooom4o`00oa3oooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qoooo
o`03o`000?oooooooooo00koooooA?oo002iooooodGoogl04?ooool00ol0003oooooooooo`09oooo
o`008Oooool2o`0000ooooooA?oo002iooooodGoogl03_ooool3o`0000_ooooo000Qooooo`03o`00
0?oooooooooo00koooooA?oo002iooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Ooo
ool00ol0003oooooooooo`0>ooooodCoo`00^Ooooom5oomo013ooooo00?o0000ooooooooool02Ooo
ool0027ooooo00?o0000ooooooooool03_oooom4ool00;WoooooAOooO`0@ooooo`03o`000?oooooo
oooo00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo002iooooodGoogl04?ooool00ol0
003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`00^Ooooom5oomo013o
oooo00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool00;Wooooo
AOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo
002iooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>
ooooodCoo`00^Ooooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000oooo
ooooool03_oooom4ool00;WoooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03
o`000?oooooooooo00koooooA?oo002iooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`00
8Oooool2o`0000ooooooA?oo002iooooodGoogl03_ooool3o`0000_ooooo000Qooooo`03o`000?oo
oooooooo00koooooA?oo002iooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool0
0ol0003oooooooooo`0>ooooodCoo`00^Ooooom5oomo013ooooo00?o0000ooooooooool02Oooool0
027ooooo00?o0000ooooooooool03_oooom4ool00;WoooooAOooO`0@ooooo`03o`000?oooooooooo
00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo002iooooodGoogl04?ooool00ol0003o
ooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`00^Ooooom5oomo013ooooo
00?o0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool03_oooom4ool00;WoooooAOoo
O`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooooooo00koooooA?oo002i
ooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0003oooooooooo`0>oooo
odCoo`00^Ooooom5oomo013ooooo00?o0000ooooooooool02Oooool0027ooooo00?o0000oooooooo
ool03_oooom4ool00;WoooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`;o0000
3ooooom4ool00;WoooooAOooO`0>ooooo`?o00002oooool0027ooooo00?o0000ooooooooool03_oo
oom4ool00;WoooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000Qooooo`03o`000?oooooo
oooo00koooooA?oo002iooooodGoogl04?ooool00ol0003oooooooooo`09ooooo`008Oooool00ol0
003oooooooooo`0>ooooodCoo`00^Ooooom5oomo013ooooo00?o0000ooooooooool02Oooool0027o
oooo00?o0000ooooooooool03_oooom4ool00;WoooooAOooO`0@ooooo`03o`000?oooooooooo00Wo
oooo000Qooooo`03o`000?oooooooooo00koooooA?oo002iooooodGoogl04?ooool00ol0003ooooo
ooooo`09ooooo`008Oooool00ol0003oooooooooo`0>ooooodCoo`00^Ooooom5oomo013ooooo00?o
0000ooooooooool02Oooool0027ooooo00?o0000ooooooooool0ooooool<ooooodGoogl04?ooool0
0ol0003oooooooooo`09ooooo`002oooool3o`0000Cooooo0_l00004ooooo`;o00001oooool00ol0
003oooooooooo`3oooooo`coooooAOooO`0@ooooo`03o`000?oooooooooo00Wooooo000<ooooo`03
o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool01?l0003oooooooooool00006
ooooo`03o`000?oooooooooo0?oooooo3?oooom5oomo013ooooo00?o0000ooooooooool02Oooool0
00cooooo00?o0000ooooooooool00_ooool01?l0003oooooooooool00002ooooo`04o`000?oooooo
ooooo`0000Kooooo00?o0000ooooooooool0oooooomQooooo`03o`000?oooooooooo00Wooooo000<
ooooo`03o`000?oooooooooo00;ooooo00Co0000oooooooooooo00000_ooool01?l0003ooooooooo
ool00006ooooo`?o0000oooooomNooooo`Co00002oooool000cooooo00?o0000ooooooooool00_oo
ool01?l0003oooooooooool00002ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000oooooooo
ool0;oooool00ol0003oooooooooo`1Booooo`03o`000?oooooooooo057ooooo00?o0000oooooooo
ool0D_ooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo00Wooooo000;ooooo`;o0000
1?ooool01?l0003oooooooooool00002ooooo`04o`000?ooooooooooo`0000Kooooo00?o0000oooo
ooooool0;oooool00ol0003oooooooooo`1Booooo`03o`000?oooooooooo057ooooo00?o0000oooo
ooooool0D_ooool00ol0003oooooooooo`0`ooooo`03o`000?oooooooooo00Wooooo000<ooooo`03
o`000?oooooooooo00?ooooo0_l00004ooooo`;o00001oooooooo`0006Go00002oooool00?oooooo
TOooool00?ooooooTOooool00?ooooooTOooool00?ooooooTOooool00?ooooooTOooool00?oooooo
TOooool0057ooooo0ol0001Aooooo`Go0000D?ooool3o`0005?ooooo0ol0000mooooo`00D_ooool0
0ol0003oooooooooo`1Aooooo`04o`000?ooooooooooo`0004oooooo00Go0000oooooooooooooooo
o`00001Cooooo`03o`000?oooooooooo03cooooo001Booooo`03o`000?oooooooooo05;ooooo00?o
0000ooooooooool0Doooool00ol0003oooooooooo`1>ooooo`Go0000?Oooool005;ooooo00?o0000
ooooooooool0Doooool00ol0003oooooooooo`1@ooooo`;o0000DOooool01?l0003oooooooooool0
000nooooo`00D_ooool00ol0003oooooooooo`1@ooooo`05o`000?ooooooooooooooool00000Dooo
ool00ol0003oooooooooo`1?ooooo`03o`000?oooooo000003kooooo001Aooooo`;o0000D_ooool0
1Ol0003ooooooooooooooooo000004oooooo00Go0000ooooooooooooooooo`00001Booooo`;o0000
?_ooool005;ooooo00?o0000ooooooooool0DOooool3o`00057ooooo0ol0001Dooooo`03o`000?oo
oooooooo03cooooo0000\
\>"],
  ImageRangeCache->{{{0, 399}, {246.188, 0}} -> {0.00675159, -8.85469, \
0.0118357, 0.474562}}],

Cell[BoxData[
    \(89.75956475956475`\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Closed]]
},
FrontEndVersion->"5.0 for Macintosh",
ScreenRectangle->{{0, 1280}, {0, 828}},
WindowSize->{749, 833},
WindowMargins->{{Automatic, -1151}, {Automatic, 82}},
ShowSelection->True,
StyleDefinitions -> "Classroom.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "dir"->{
    Cell[6660, 186, 86, 1, 39, "Section",
      CellTags->"dir"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"dir", 5998303, 101452}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 59, 0, 62, "Title"],
Cell[1816, 53, 818, 23, 114, "Text"],

Cell[CellGroupData[{
Cell[2659, 80, 36, 0, 59, "Section"],

Cell[CellGroupData[{
Cell[2720, 84, 60, 0, 49, "Subsection"],
Cell[2783, 86, 162, 3, 63, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2982, 94, 41, 0, 49, "Subsection"],
Cell[3026, 96, 666, 12, 175, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3729, 113, 43, 0, 49, "Subsection"],
Cell[3775, 115, 230, 5, 63, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4042, 125, 46, 0, 49, "Subsection"],
Cell[4091, 127, 202, 3, 63, "Input"],
Cell[4296, 132, 239, 4, 79, "Input"],
Cell[4538, 138, 223, 4, 79, "Input"],
Cell[4764, 144, 155, 2, 63, "Input"],
Cell[4922, 148, 249, 4, 79, "Input"],
Cell[5174, 154, 307, 5, 111, "Input"],
Cell[5484, 161, 1127, 19, 239, "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[6660, 186, 86, 1, 39, "Section",
  CellTags->"dir"],
Cell[6749, 189, 208, 4, 63, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[6994, 198, 66, 0, 39, "Section"],

Cell[CellGroupData[{
Cell[7085, 202, 41, 0, 49, "Subsection"],
Cell[7129, 204, 86, 1, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7252, 210, 56, 0, 49, "Subsection"],
Cell[7311, 212, 917, 20, 143, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8265, 237, 64, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[8354, 241, 54, 0, 43, "Subsubsection"],
Cell[8411, 243, 245, 4, 63, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8693, 252, 52, 0, 43, "Subsubsection"],
Cell[8748, 254, 301, 5, 79, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9086, 264, 50, 0, 43, "Subsubsection"],

Cell[CellGroupData[{
Cell[9161, 268, 565, 11, 127, "Input"],
Cell[9729, 281, 802, 13, 188, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[10604, 302, 69, 0, 39, "Section"],

Cell[CellGroupData[{
Cell[10698, 306, 47, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[10770, 310, 368, 7, 79, "Input"],
Cell[11141, 319, 1259495, 16346, 308, 418292, 5957, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1270685, 16671, 71, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[1270781, 16675, 67, 0, 43, "Subsubsection"],
Cell[1270851, 16677, 204, 4, 63, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1271092, 16686, 42, 0, 43, "Subsubsection"],
Cell[1271137, 16688, 116, 2, 47, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1271302, 16696, 49, 0, 49, "Subsection"],
Cell[1271354, 16698, 126, 3, 28, "Text"],

Cell[CellGroupData[{
Cell[1271505, 16705, 114, 2, 47, "Input"],

Cell[CellGroupData[{
Cell[1271644, 16711, 76290, 2240, 68, 22694, 1573, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1347937, 18953, 76225, 2239, 68, 22694, 1573, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1424165, 21194, 76027, 2237, 68, 22694, 1573, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1500195, 23433, 50296, 1832, 68, 19430, 1445, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1550494, 25267, 50202, 1830, 68, 19430, 1445, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1600699, 27099, 50296, 1832, 68, 19430, 1445, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1651056, 28938, 42, 0, 49, "Subsection"],
Cell[1651101, 28940, 148, 3, 28, "Text"],
Cell[1651252, 28945, 73, 1, 47, "Input"],

Cell[CellGroupData[{
Cell[1651350, 28950, 206, 3, 63, "Input"],

Cell[CellGroupData[{
Cell[1651581, 28957, 126093, 3243, 317, 28663, 2037, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1777677, 32202, 125176, 3225, 317, 28503, 2028, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1902856, 35429, 122742, 3208, 317, 28665, 2043, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2025601, 38639, 108489, 3094, 317, 29487, 2115, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2134093, 41735, 97180, 2898, 317, 28728, 2049, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2231276, 44635, 95187, 2906, 317, 29178, 2088, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2326524, 47548, 43, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[2326592, 47552, 366, 7, 79, "Input"],
Cell[2326961, 47561, 1259644, 16347, 308, 418292, 5957, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[3586654, 63914, 60, 0, 49, "Subsection"],
Cell[3586717, 63916, 135, 2, 47, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3586889, 63923, 93, 3, 49, "Subsection"],
Cell[3586985, 63928, 150, 3, 28, "Text"],
Cell[3587138, 63933, 123, 2, 47, "Input"],
Cell[3587264, 63937, 171, 4, 63, "Input"],

Cell[CellGroupData[{
Cell[3587460, 63945, 545, 10, 127, "Input"],

Cell[CellGroupData[{
Cell[3588030, 63959, 633215, 8613, 83, 418321, 5957, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4221248, 72574, 930398, 16039, 977, 474791, 10405, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5151707, 88620, 70, 0, 49, "Subsection"],
Cell[5151780, 88622, 172, 3, 63, "Input"],

Cell[CellGroupData[{
Cell[5151977, 88629, 579, 11, 127, "Input"],

Cell[CellGroupData[{
Cell[5152581, 88644, 633215, 8613, 83, 418321, 5957, "GraphicsData", \
"PostScript", "Graphics"],
Cell[5785799, 97259, 120347, 2272, 170, 79655, 1765, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5906207, 99538, 63, 0, 49, "Subsection"],

Cell[CellGroupData[{
Cell[5906295, 99542, 97, 3, 43, "Subsubsection"],

Cell[CellGroupData[{
Cell[5906417, 99549, 387, 7, 95, "Input"],
Cell[5906807, 99558, 57974, 1190, 256, 7557, 564, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5964830, 100754, 86, 1, 43, "Subsubsection"],

Cell[CellGroupData[{
Cell[5964941, 100759, 382, 7, 95, "Input"],
Cell[5965326, 100768, 32229, 658, 256, 4089, 307, "GraphicsData", \
"PostScript", "Graphics"],
Cell[5997558, 101428, 52, 1, 60, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Closed]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

